#include <setjmp.h>
/* This is a C header file to be used by the output of the Cyclone to
   C translator.  The corresponding definitions are in file lib/runtime_*.c */
#ifndef _CYC_INCLUDE_H_
#define _CYC_INCLUDE_H_

/* Need one of these per thread (see runtime_stack.c). The runtime maintains 
   a stack that contains either _handler_cons structs or _RegionHandle structs.
   The tag is 0 for a handler_cons and 1 for a region handle.  */
struct _RuntimeStack {
  int tag; 
  struct _RuntimeStack *next;
  void (*cleanup)(struct _RuntimeStack *frame);
};

#ifndef offsetof
/* should be size_t, but int is fine. */
#define offsetof(t,n) ((int)(&(((t *)0)->n)))
#endif

/* Fat pointers */
struct _fat_ptr {
  unsigned char *curr; 
  unsigned char *base; 
  unsigned char *last_plus_one; 
};  

/* Discriminated Unions */
struct _xtunion_struct { char *tag; };

/* Regions */
struct _RegionPage
#ifdef CYC_REGION_PROFILE
{ unsigned total_bytes;
  unsigned free_bytes;
  /* MWH: wish we didn't have to include the stuff below ... */
  struct _RegionPage *next;
  char data[1];
}
#endif
; // abstract -- defined in runtime_memory.c
struct _RegionHandle {
  struct _RuntimeStack s;
  struct _RegionPage *curr;
  char               *offset;
  char               *last_plus_one;
  struct _DynRegionHandle *sub_regions;
#ifdef CYC_REGION_PROFILE
  const char         *name;
#else
  unsigned used_bytes;
  unsigned wasted_bytes;
#endif
};
struct _DynRegionFrame {
  struct _RuntimeStack s;
  struct _DynRegionHandle *x;
};
// A dynamic region is just a region handle.  The wrapper struct is for type
// abstraction.
struct Cyc_Core_DynamicRegion {
  struct _RegionHandle h;
};

struct _RegionHandle _new_region(const char*);
void* _region_malloc(struct _RegionHandle*, unsigned);
void* _region_calloc(struct _RegionHandle*, unsigned t, unsigned n);
void   _free_region(struct _RegionHandle*);
struct _RegionHandle*_open_dynregion(struct _DynRegionFrame*,struct _DynRegionHandle*);
void   _pop_dynregion();

/* Exceptions */
struct _handler_cons {
  struct _RuntimeStack s;
  jmp_buf handler;
};
void _push_handler(struct _handler_cons *);
void _push_region(struct _RegionHandle *);
void _npop_handler(int);
void _pop_handler();
void _pop_region();

#ifndef _throw
void* _throw_null_fn(const char*,unsigned);
void* _throw_arraybounds_fn(const char*,unsigned);
void* _throw_badalloc_fn(const char*,unsigned);
void* _throw_match_fn(const char*,unsigned);
void* _throw_fn(void*,const char*,unsigned);
void* _rethrow(void*);
#define _throw_null() (_throw_null_fn(__FILE__,__LINE__))
#define _throw_arraybounds() (_throw_arraybounds_fn(__FILE__,__LINE__))
#define _throw_badalloc() (_throw_badalloc_fn(__FILE__,__LINE__))
#define _throw_match() (_throw_match_fn(__FILE__,__LINE__))
#define _throw(e) (_throw_fn((e),__FILE__,__LINE__))
#endif

struct _xtunion_struct* Cyc_Core_get_exn_thrown();
/* Built-in Exceptions */
struct Cyc_Null_Exception_exn_struct { char *tag; };
struct Cyc_Array_bounds_exn_struct { char *tag; };
struct Cyc_Match_Exception_exn_struct { char *tag; };
struct Cyc_Bad_alloc_exn_struct { char *tag; };
extern char Cyc_Null_Exception[];
extern char Cyc_Array_bounds[];
extern char Cyc_Match_Exception[];
extern char Cyc_Bad_alloc[];

/* Built-in Run-time Checks and company */
#ifdef CYC_ANSI_OUTPUT
#define _INLINE  
#else
#define _INLINE inline
#endif

#ifdef NO_CYC_NULL_CHECKS
#define _check_null(ptr) (ptr)
#else
#define _check_null(ptr) \
  ({ void*_cks_null = (void*)(ptr); \
     if (!_cks_null) _throw_null(); \
     _cks_null; })
#endif

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index)\
   (((char*)ptr) + (elt_sz)*(index))
#ifdef NO_CYC_NULL_CHECKS
#define _check_known_subscript_null _check_known_subscript_notnull
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr);\
  int _index = (index);\
  if (!_cks_ptr) _throw_null(); \
  _cks_ptr + (elt_sz)*_index; })
#endif
#define _zero_arr_plus_fn(orig_x,orig_sz,orig_i,f,l) ((orig_x)+(orig_i))
#define _zero_arr_plus_char_fn _zero_arr_plus_fn
#define _zero_arr_plus_short_fn _zero_arr_plus_fn
#define _zero_arr_plus_int_fn _zero_arr_plus_fn
#define _zero_arr_plus_float_fn _zero_arr_plus_fn
#define _zero_arr_plus_double_fn _zero_arr_plus_fn
#define _zero_arr_plus_longdouble_fn _zero_arr_plus_fn
#define _zero_arr_plus_voidstar_fn _zero_arr_plus_fn
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (!_cks_ptr) _throw_null(); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })

/* _zero_arr_plus_*_fn(x,sz,i,filename,lineno) adds i to zero-terminated ptr
   x that has at least sz elements */
char* _zero_arr_plus_char_fn(char*,unsigned,int,const char*,unsigned);
short* _zero_arr_plus_short_fn(short*,unsigned,int,const char*,unsigned);
int* _zero_arr_plus_int_fn(int*,unsigned,int,const char*,unsigned);
float* _zero_arr_plus_float_fn(float*,unsigned,int,const char*,unsigned);
double* _zero_arr_plus_double_fn(double*,unsigned,int,const char*,unsigned);
long double* _zero_arr_plus_longdouble_fn(long double*,unsigned,int,const char*, unsigned);
void** _zero_arr_plus_voidstar_fn(void**,unsigned,int,const char*,unsigned);
#endif

/* _get_zero_arr_size_*(x,sz) returns the number of elements in a
   zero-terminated array that is NULL or has at least sz elements */
int _get_zero_arr_size_char(const char*,unsigned);
int _get_zero_arr_size_short(const short*,unsigned);
int _get_zero_arr_size_int(const int*,unsigned);
int _get_zero_arr_size_float(const float*,unsigned);
int _get_zero_arr_size_double(const double*,unsigned);
int _get_zero_arr_size_longdouble(const long double*,unsigned);
int _get_zero_arr_size_voidstar(const void**,unsigned);

/* _zero_arr_inplace_plus_*_fn(x,i,filename,lineno) sets
   zero-terminated pointer *x to *x + i */
char* _zero_arr_inplace_plus_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_short_fn(short**,int,const char*,unsigned);
int* _zero_arr_inplace_plus_int(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_longdouble_fn(long double**,int,const char*,unsigned);
void** _zero_arr_inplace_plus_voidstar_fn(void***,int,const char*,unsigned);
/* like the previous functions, but does post-addition (as in e++) */
char* _zero_arr_inplace_plus_post_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_post_short_fn(short**x,int,const char*,unsigned);
int* _zero_arr_inplace_plus_post_int_fn(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_post_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_post_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_post_longdouble_fn(long double**,int,const char *,unsigned);
void** _zero_arr_inplace_plus_post_voidstar_fn(void***,int,const char*,unsigned);
#define _zero_arr_plus_char(x,s,i) \
  (_zero_arr_plus_char_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_short(x,s,i) \
  (_zero_arr_plus_short_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_int(x,s,i) \
  (_zero_arr_plus_int_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_float(x,s,i) \
  (_zero_arr_plus_float_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_double(x,s,i) \
  (_zero_arr_plus_double_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_longdouble(x,s,i) \
  (_zero_arr_plus_longdouble_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_voidstar(x,s,i) \
  (_zero_arr_plus_voidstar_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_inplace_plus_char(x,i) \
  _zero_arr_inplace_plus_char_fn((char **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_short(x,i) \
  _zero_arr_inplace_plus_short_fn((short **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_int(x,i) \
  _zero_arr_inplace_plus_int_fn((int **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_float(x,i) \
  _zero_arr_inplace_plus_float_fn((float **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_double(x,i) \
  _zero_arr_inplace_plus_double_fn((double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_longdouble(x,i) \
  _zero_arr_inplace_plus_longdouble_fn((long double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_voidstar(x,i) \
  _zero_arr_inplace_plus_voidstar_fn((void ***)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_char(x,i) \
  _zero_arr_inplace_plus_post_char_fn((char **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_short(x,i) \
  _zero_arr_inplace_plus_post_short_fn((short **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_int(x,i) \
  _zero_arr_inplace_plus_post_int_fn((int **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_float(x,i) \
  _zero_arr_inplace_plus_post_float_fn((float **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_double(x,i) \
  _zero_arr_inplace_plus_post_double_fn((double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_longdouble(x,i) \
  _zero_arr_inplace_plus_post_longdouble_fn((long double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_voidstar(x,i) \
  _zero_arr_inplace_plus_post_voidstar_fn((void***)(x),(i),__FILE__,__LINE__)

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_fat_subscript(arr,elt_sz,index) ((arr).curr + (elt_sz) * (index))
#define _untag_fat_ptr(arr,elt_sz,num_elts) ((arr).curr)
#else
#define _check_fat_subscript(arr,elt_sz,index) ({ \
  struct _fat_ptr _cus_arr = (arr); \
  unsigned char *_cus_ans = _cus_arr.curr + (elt_sz) * (index); \
  /* JGM: not needed! if (!_cus_arr.base) _throw_null();*/ \
  if (_cus_ans < _cus_arr.base || _cus_ans >= _cus_arr.last_plus_one) \
    _throw_arraybounds(); \
  _cus_ans; })
#define _untag_fat_ptr(arr,elt_sz,num_elts) ({ \
  struct _fat_ptr _arr = (arr); \
  unsigned char *_curr = _arr.curr; \
  if ((_curr < _arr.base || _curr + (elt_sz) * (num_elts) > _arr.last_plus_one) &&\
      _curr != (unsigned char *)0) \
    _throw_arraybounds(); \
  _curr; })
#endif

#define _tag_fat(tcurr,elt_sz,num_elts) ({ \
  struct _fat_ptr _ans; \
  unsigned _num_elts = (num_elts);\
  _ans.base = _ans.curr = (void*)(tcurr); \
  /* JGM: if we're tagging NULL, ignore num_elts */ \
  _ans.last_plus_one = _ans.base ? (_ans.base + (elt_sz) * _num_elts) : 0; \
  _ans; })

#define _get_fat_size(arr,elt_sz) \
  ({struct _fat_ptr _arr = (arr); \
    unsigned char *_arr_curr=_arr.curr; \
    unsigned char *_arr_last=_arr.last_plus_one; \
    (_arr_curr < _arr.base || _arr_curr >= _arr_last) ? 0 : \
    ((_arr_last - _arr_curr) / (elt_sz));})

#define _fat_ptr_plus(arr,elt_sz,change) ({ \
  struct _fat_ptr _ans = (arr); \
  int _change = (change);\
  _ans.curr += (elt_sz) * _change;\
  _ans; })
#define _fat_ptr_inplace_plus(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  _arr_ptr->curr += (elt_sz) * (change);\
  *_arr_ptr; })
#define _fat_ptr_inplace_plus_post(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  struct _fat_ptr _ans = *_arr_ptr; \
  _arr_ptr->curr += (elt_sz) * (change);\
  _ans; })

//Not a macro since initialization order matters. Defined in runtime_zeroterm.c.
struct _fat_ptr _fat_ptr_decrease_size(struct _fat_ptr,unsigned sz,unsigned numelts);

/* Allocation */
void* GC_malloc(int);
void* GC_malloc_atomic(int);
void* GC_calloc(unsigned,unsigned);
void* GC_calloc_atomic(unsigned,unsigned);
// bound the allocation size to be < MAX_ALLOC_SIZE. See macros below for usage.
#define MAX_MALLOC_SIZE (1 << 28)
void* _bounded_GC_malloc(int,const char*,int);
void* _bounded_GC_malloc_atomic(int,const char*,int);
void* _bounded_GC_calloc(unsigned,unsigned,const char*,int);
void* _bounded_GC_calloc_atomic(unsigned,unsigned,const char*,int);
/* these macros are overridden below ifdef CYC_REGION_PROFILE */
#ifndef CYC_REGION_PROFILE
#define _cycalloc(n) _bounded_GC_malloc(n,__FILE__,__LINE__)
#define _cycalloc_atomic(n) _bounded_GC_malloc_atomic(n,__FILE__,__LINE__)
#define _cyccalloc(n,s) _bounded_GC_calloc(n,s,__FILE__,__LINE__)
#define _cyccalloc_atomic(n,s) _bounded_GC_calloc_atomic(n,s,__FILE__,__LINE__)
#endif

static _INLINE unsigned int _check_times(unsigned x, unsigned y) {
  unsigned long long whole_ans = 
    ((unsigned long long) x)*((unsigned long long)y);
  unsigned word_ans = (unsigned)whole_ans;
  if(word_ans < whole_ans || word_ans > MAX_MALLOC_SIZE)
    _throw_badalloc();
  return word_ans;
}

#define _CYC_MAX_REGION_CONST 2
#define _CYC_MIN_ALIGNMENT (sizeof(double))

#ifdef CYC_REGION_PROFILE
extern int rgn_total_bytes;
#endif

static _INLINE void *_fast_region_malloc(struct _RegionHandle *r, unsigned orig_s) {  
  if (r > (struct _RegionHandle *)_CYC_MAX_REGION_CONST && r->curr != 0) { 
#ifdef CYC_NOALIGN
    unsigned s =  orig_s;
#else
    unsigned s =  (orig_s + _CYC_MIN_ALIGNMENT - 1) & (~(_CYC_MIN_ALIGNMENT -1)); 
#endif
    char *result; 
    result = r->offset; 
    if (s <= (r->last_plus_one - result)) {
      r->offset = result + s; 
#ifdef CYC_REGION_PROFILE
    r->curr->free_bytes = r->curr->free_bytes - s;
    rgn_total_bytes += s;
#endif
      return result;
    }
  } 
  return _region_malloc(r,orig_s); 
}

#ifdef CYC_REGION_PROFILE
/* see macros below for usage. defined in runtime_memory.c */
void* _profile_GC_malloc(int,const char*,const char*,int);
void* _profile_GC_malloc_atomic(int,const char*,const char*,int);
void* _profile_GC_calloc(unsigned,unsigned,const char*,const char*,int);
void* _profile_GC_calloc_atomic(unsigned,unsigned,const char*,const char*,int);
void* _profile_region_malloc(struct _RegionHandle*,unsigned,const char*,const char*,int);
void* _profile_region_calloc(struct _RegionHandle*,unsigned,unsigned,const char *,const char*,int);
struct _RegionHandle _profile_new_region(const char*,const char*,const char*,int);
void _profile_free_region(struct _RegionHandle*,const char*,const char*,int);
#ifndef RUNTIME_CYC
#define _new_region(n) _profile_new_region(n,__FILE__,__FUNCTION__,__LINE__)
#define _free_region(r) _profile_free_region(r,__FILE__,__FUNCTION__,__LINE__)
#define _region_malloc(rh,n) _profile_region_malloc(rh,n,__FILE__,__FUNCTION__,__LINE__)
#define _region_calloc(rh,n,t) _profile_region_calloc(rh,n,t,__FILE__,__FUNCTION__,__LINE__)
#  endif
#define _cycalloc(n) _profile_GC_malloc(n,__FILE__,__FUNCTION__,__LINE__)
#define _cycalloc_atomic(n) _profile_GC_malloc_atomic(n,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc(n,s) _profile_GC_calloc(n,s,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc_atomic(n,s) _profile_GC_calloc_atomic(n,s,__FILE__,__FUNCTION__,__LINE__)
#endif
#endif
 struct Cyc___cycFILE;struct Cyc_String_pa_PrintArg_struct{int tag;struct _fat_ptr f1;};struct Cyc_Int_pa_PrintArg_struct{int tag;unsigned long f1;};struct Cyc_Double_pa_PrintArg_struct{int tag;double f1;};struct Cyc_LongDouble_pa_PrintArg_struct{int tag;long double f1;};struct Cyc_ShortPtr_pa_PrintArg_struct{int tag;short*f1;};struct Cyc_IntPtr_pa_PrintArg_struct{int tag;unsigned long*f1;};struct Cyc_ShortPtr_sa_ScanfArg_struct{int tag;short*f1;};struct Cyc_UShortPtr_sa_ScanfArg_struct{int tag;unsigned short*f1;};struct Cyc_IntPtr_sa_ScanfArg_struct{int tag;int*f1;};struct Cyc_UIntPtr_sa_ScanfArg_struct{int tag;unsigned*f1;};struct Cyc_StringPtr_sa_ScanfArg_struct{int tag;struct _fat_ptr f1;};struct Cyc_DoublePtr_sa_ScanfArg_struct{int tag;double*f1;};struct Cyc_FloatPtr_sa_ScanfArg_struct{int tag;float*f1;};struct Cyc_CharPtr_sa_ScanfArg_struct{int tag;struct _fat_ptr f1;};extern char Cyc_FileCloseError[15U];struct Cyc_FileCloseError_exn_struct{char*tag;};extern char Cyc_FileOpenError[14U];struct Cyc_FileOpenError_exn_struct{char*tag;struct _fat_ptr f1;};
# 300 "cycboot.h"
int isspace(int);
# 331
extern const long Cyc_long_max;extern const long Cyc_long_min;
# 22 "ctype.h"
int isspace(int);struct Cyc_Core_Opt{void*v;};extern char Cyc_Core_Invalid_argument[17U];struct Cyc_Core_Invalid_argument_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Failure[8U];struct Cyc_Core_Failure_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Impossible[11U];struct Cyc_Core_Impossible_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Not_found[10U];struct Cyc_Core_Not_found_exn_struct{char*tag;};extern char Cyc_Core_Unreachable[12U];struct Cyc_Core_Unreachable_exn_struct{char*tag;struct _fat_ptr f1;};
# 168 "core.h"
extern struct _RegionHandle*Cyc_Core_unique_region;struct Cyc_Core_DynamicRegion;struct Cyc_Core_NewDynamicRegion{struct Cyc_Core_DynamicRegion*key;};struct Cyc_Core_ThinRes{void*arr;unsigned nelts;};struct Cyc_List_List{void*hd;struct Cyc_List_List*tl;};extern char Cyc_List_List_mismatch[14U];struct Cyc_List_List_mismatch_exn_struct{char*tag;};extern char Cyc_List_Nth[4U];struct Cyc_List_Nth_exn_struct{char*tag;};
# 81 "string.h"
struct _fat_ptr Cyc__memcpy(struct _fat_ptr d,struct _fat_ptr s,unsigned long,unsigned);
# 29 "assert.h"
void*Cyc___assert_fail(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line);
# 7 "ap.h"
extern struct Cyc_AP_T*Cyc_AP_zero;
extern struct Cyc_AP_T*Cyc_AP_one;
struct Cyc_AP_T*Cyc_AP_new(long n);
struct Cyc_AP_T*Cyc_AP_fromint(long x);
# 16
struct Cyc_AP_T*Cyc_AP_add(struct Cyc_AP_T*x,struct Cyc_AP_T*y);
struct Cyc_AP_T*Cyc_AP_sub(struct Cyc_AP_T*x,struct Cyc_AP_T*y);
struct Cyc_AP_T*Cyc_AP_mul(struct Cyc_AP_T*x,struct Cyc_AP_T*y);
struct Cyc_AP_T*Cyc_AP_div(struct Cyc_AP_T*x,struct Cyc_AP_T*y);
struct Cyc_AP_T*Cyc_AP_mod(struct Cyc_AP_T*x,struct Cyc_AP_T*y);
struct Cyc_AP_T*Cyc_AP_pow(struct Cyc_AP_T*x,struct Cyc_AP_T*y,struct Cyc_AP_T*p);
struct Cyc_AP_T*Cyc_AP_addi(struct Cyc_AP_T*x,long y);
# 28
struct Cyc_AP_T*Cyc_AP_rshift(struct Cyc_AP_T*x,int s);
struct Cyc_AP_T*Cyc_AP_and(struct Cyc_AP_T*x,struct Cyc_AP_T*y);
struct Cyc_AP_T*Cyc_AP_or(struct Cyc_AP_T*x,struct Cyc_AP_T*y);
struct Cyc_AP_T*Cyc_AP_xor(struct Cyc_AP_T*x,struct Cyc_AP_T*y);
int Cyc_AP_cmp(struct Cyc_AP_T*x,struct Cyc_AP_T*y);
# 34
struct Cyc_AP_T*Cyc_AP_gcd(struct Cyc_AP_T*x,struct Cyc_AP_T*y);
# 8 "xp.h"
int Cyc_XP_add(int n,struct _fat_ptr z,struct _fat_ptr x,struct _fat_ptr y,int carry);
int Cyc_XP_sub(int n,struct _fat_ptr z,struct _fat_ptr x,struct _fat_ptr y,int borrow);
int Cyc_XP_mul(struct _fat_ptr z,int n,struct _fat_ptr x,int m,struct _fat_ptr y);
int Cyc_XP_div(int n,struct _fat_ptr q,struct _fat_ptr x,int m,struct _fat_ptr y,struct _fat_ptr r,struct _fat_ptr tmp);
int Cyc_XP_sum(int n,struct _fat_ptr z,struct _fat_ptr x,int y);
int Cyc_XP_diff(int n,struct _fat_ptr z,struct _fat_ptr x,int y);
# 17
int Cyc_XP_cmp(int n,struct _fat_ptr x,struct _fat_ptr y);
void Cyc_XP_lshift(int n,struct _fat_ptr z,int m,struct _fat_ptr x,int s,int fill);
# 20
void Cyc_XP_rshift(int n,struct _fat_ptr z,int m,struct _fat_ptr x,int s,int fill);
# 22
void Cyc_XP_and(int n,struct _fat_ptr z,struct _fat_ptr x,struct _fat_ptr y);
void Cyc_XP_or(int n,struct _fat_ptr z,struct _fat_ptr x,struct _fat_ptr y);
void Cyc_XP_xor(int n,struct _fat_ptr z,struct _fat_ptr x,struct _fat_ptr y);
int Cyc_XP_length(int n,struct _fat_ptr x);
unsigned long Cyc_XP_fromint(int n,struct _fat_ptr z,unsigned long u);
# 28
unsigned long Cyc_XP_toint(int n,struct _fat_ptr x);
int Cyc_XP_fromstr(int n,struct _fat_ptr z,const char*str,int base);
# 31
struct _fat_ptr Cyc_XP_tostr(struct _fat_ptr str,int size,int base,int n,struct _fat_ptr x);struct Cyc_AP_T{int sign;int ndigits;int size;struct _fat_ptr digits;};
# 21 "ap.cyc"
struct Cyc_AP_T*Cyc_AP_zero;
struct Cyc_AP_T*Cyc_AP_one;
int Cyc_init=0;
# 25
static struct Cyc_AP_T*Cyc_normalize(struct Cyc_AP_T*z,int n);
static int Cyc_cmp(struct Cyc_AP_T*x,struct Cyc_AP_T*y);
static void Cyc_AP_init(){
Cyc_init=1;
Cyc_AP_zero=Cyc_AP_fromint(0);
Cyc_AP_one=Cyc_AP_fromint(1);}
# 27
static struct Cyc_AP_T*Cyc_mk(int size){
# 33
struct Cyc_AP_T*z;
if(!Cyc_init)Cyc_AP_init();z=
_cycalloc(sizeof(struct Cyc_AP_T));
({struct _fat_ptr _tmp70=({unsigned _tmp0=size;_tag_fat(_cyccalloc_atomic(sizeof(unsigned char),_tmp0),sizeof(unsigned char),_tmp0);});z->digits=_tmp70;});
size > 0?0:({int(*_tmp72)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=({int(*_tmp1)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=(int(*)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line))Cyc___assert_fail;_tmp1;});struct _fat_ptr _tmp71=({const char*_tmp2="size > 0";_tag_fat(_tmp2,sizeof(char),9U);});_tmp72(_tmp71,({const char*_tmp3="ap.cyc";_tag_fat(_tmp3,sizeof(char),7U);}),37U);});
z->sign=1;
z->size=size;
z->ndigits=1;
return z;}
# 27
static struct Cyc_AP_T*Cyc_set(struct Cyc_AP_T*z,long n){
# 44
if(n == Cyc_long_min)
Cyc_XP_fromint(((struct Cyc_AP_T*)_check_null(z))->size,z->digits,(unsigned)Cyc_long_max + 1U);else{
if(n < 0)
Cyc_XP_fromint(((struct Cyc_AP_T*)_check_null(z))->size,z->digits,(unsigned long)(- n));else{
# 49
Cyc_XP_fromint(((struct Cyc_AP_T*)_check_null(z))->size,z->digits,(unsigned long)n);}}
z->sign=n < 0?- 1: 1;
return Cyc_normalize(z,z->size);}
# 27
static struct Cyc_AP_T*Cyc_normalize(struct Cyc_AP_T*z,int n){
# 54
({int _tmp73=Cyc_XP_length(n,((struct Cyc_AP_T*)_check_null(z))->digits);z->ndigits=_tmp73;});
return z;}
# 27
static struct Cyc_AP_T*Cyc_add(struct Cyc_AP_T*z,struct Cyc_AP_T*x,struct Cyc_AP_T*y){
# 58
int n=((struct Cyc_AP_T*)_check_null(y))->ndigits;
if(((struct Cyc_AP_T*)_check_null(x))->ndigits < n)
return Cyc_add(z,y,x);else{
if(x->ndigits > n){
int carry=Cyc_XP_add(n,((struct Cyc_AP_T*)_check_null(z))->digits,x->digits,y->digits,0);
# 64
({unsigned char _tmp77=(unsigned char)({int _tmp76=x->ndigits - n;struct _fat_ptr _tmp75=
_fat_ptr_plus(z->digits,sizeof(unsigned char),n);
# 64
struct _fat_ptr _tmp74=
_fat_ptr_plus(x->digits,sizeof(unsigned char),n);
# 64
Cyc_XP_sum(_tmp76,_tmp75,_tmp74,carry);});*((unsigned char*)_check_fat_subscript(z->digits,sizeof(unsigned char),z->size - 1))=_tmp77;});}else{
# 67
({unsigned char _tmp78=(unsigned char)Cyc_XP_add(n,((struct Cyc_AP_T*)_check_null(z))->digits,x->digits,y->digits,0);*((unsigned char*)_check_fat_subscript(z->digits,sizeof(unsigned char),n))=_tmp78;});}}
# 69
return Cyc_normalize(z,z->size);}
# 27
static struct Cyc_AP_T*Cyc_sub(struct Cyc_AP_T*z,struct Cyc_AP_T*x,struct Cyc_AP_T*y){
# 72
int borrow;int n=((struct Cyc_AP_T*)_check_null(y))->ndigits;
borrow=({int _tmp7B=n;struct _fat_ptr _tmp7A=((struct Cyc_AP_T*)_check_null(z))->digits;struct _fat_ptr _tmp79=((struct Cyc_AP_T*)_check_null(x))->digits;Cyc_XP_sub(_tmp7B,_tmp7A,_tmp79,y->digits,0);});
# 75
if(x->ndigits > n)
borrow=({int _tmp7E=x->ndigits - n;struct _fat_ptr _tmp7D=_fat_ptr_plus(z->digits,sizeof(unsigned char),n);struct _fat_ptr _tmp7C=
_fat_ptr_plus(x->digits,sizeof(unsigned char),n);
# 76
Cyc_XP_diff(_tmp7E,_tmp7D,_tmp7C,borrow);});
# 75

# 78
borrow == 0?0:({int(*_tmp80)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=({int(*_tmp4)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=(int(*)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line))Cyc___assert_fail;_tmp4;});struct _fat_ptr _tmp7F=({const char*_tmp5="borrow == 0";_tag_fat(_tmp5,sizeof(char),12U);});_tmp80(_tmp7F,({const char*_tmp6="ap.cyc";_tag_fat(_tmp6,sizeof(char),7U);}),78U);});
return Cyc_normalize(z,z->size);}
# 27
static struct Cyc_AP_T*Cyc_mulmod(struct Cyc_AP_T*x,struct Cyc_AP_T*y,struct Cyc_AP_T*p){
# 82
struct Cyc_AP_T*z;struct Cyc_AP_T*xy=Cyc_AP_mul(x,y);
z=Cyc_AP_mod(xy,p);
return z;}
# 27
static int Cyc_cmp(struct Cyc_AP_T*x,struct Cyc_AP_T*y){
# 87
if(({int _tmp81=((struct Cyc_AP_T*)_check_null(x))->ndigits;_tmp81 != ((struct Cyc_AP_T*)_check_null(y))->ndigits;}))
return x->ndigits - y->ndigits;else{
# 90
return Cyc_XP_cmp(x->ndigits,x->digits,y->digits);}}
# 27
struct Cyc_AP_T*Cyc_AP_new(long n){
# 93
return({struct Cyc_AP_T*_tmp82=Cyc_mk((int)sizeof(long));Cyc_set(_tmp82,n);});}
# 27
struct Cyc_AP_T*Cyc_AP_neg(struct Cyc_AP_T*x){
# 96
struct Cyc_AP_T*z;
(unsigned)x?0:({int(*_tmp84)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=({int(*_tmp7)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=(int(*)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line))Cyc___assert_fail;_tmp7;});struct _fat_ptr _tmp83=({const char*_tmp8="x";_tag_fat(_tmp8,sizeof(char),2U);});_tmp84(_tmp83,({const char*_tmp9="ap.cyc";_tag_fat(_tmp9,sizeof(char),7U);}),97U);});
z=Cyc_mk(x->ndigits);
Cyc__memcpy(((struct Cyc_AP_T*)_check_null(z))->digits,(struct _fat_ptr)x->digits,(unsigned)x->ndigits / sizeof(*((unsigned char*)(x->digits).curr))+ (unsigned)((unsigned)x->ndigits % sizeof(*((unsigned char*)(x->digits).curr))== (unsigned)0?0: 1),sizeof(*((unsigned char*)(x->digits).curr)));
z->ndigits=x->ndigits;
z->sign=z->ndigits == 1 &&(int)*((unsigned char*)_check_fat_subscript(z->digits,sizeof(unsigned char),0))== 0?1: - x->sign;
return z;}
# 27
struct Cyc_AP_T*Cyc_AP_abs(struct Cyc_AP_T*x){
# 105
struct Cyc_AP_T*z;
(unsigned)x?0:({int(*_tmp86)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=({int(*_tmpA)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=(int(*)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line))Cyc___assert_fail;_tmpA;});struct _fat_ptr _tmp85=({const char*_tmpB="x";_tag_fat(_tmpB,sizeof(char),2U);});_tmp86(_tmp85,({const char*_tmpC="ap.cyc";_tag_fat(_tmpC,sizeof(char),7U);}),106U);});
z=Cyc_mk(x->ndigits);
Cyc__memcpy(((struct Cyc_AP_T*)_check_null(z))->digits,(struct _fat_ptr)x->digits,(unsigned)x->ndigits / sizeof(*((unsigned char*)(x->digits).curr))+ (unsigned)((unsigned)x->ndigits % sizeof(*((unsigned char*)(x->digits).curr))== (unsigned)0?0: 1),sizeof(*((unsigned char*)(x->digits).curr)));
z->ndigits=x->ndigits;
z->sign=1;
return z;}
# 27
struct Cyc_AP_T*Cyc_AP_mul(struct Cyc_AP_T*x,struct Cyc_AP_T*y){
# 114
struct Cyc_AP_T*z;
(unsigned)x?0:({int(*_tmp88)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=({int(*_tmpD)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=(int(*)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line))Cyc___assert_fail;_tmpD;});struct _fat_ptr _tmp87=({const char*_tmpE="x";_tag_fat(_tmpE,sizeof(char),2U);});_tmp88(_tmp87,({const char*_tmpF="ap.cyc";_tag_fat(_tmpF,sizeof(char),7U);}),115U);});
(unsigned)y?0:({int(*_tmp8A)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=({int(*_tmp10)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=(int(*)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line))Cyc___assert_fail;_tmp10;});struct _fat_ptr _tmp89=({const char*_tmp11="y";_tag_fat(_tmp11,sizeof(char),2U);});_tmp8A(_tmp89,({const char*_tmp12="ap.cyc";_tag_fat(_tmp12,sizeof(char),7U);}),116U);});
z=Cyc_mk(x->ndigits + y->ndigits);
Cyc_XP_mul(((struct Cyc_AP_T*)_check_null(z))->digits,x->ndigits,x->digits,y->ndigits,y->digits);
# 120
Cyc_normalize(z,z->size);
z->sign=(z->ndigits == 1 &&(int)*((unsigned char*)_check_fat_subscript(z->digits,sizeof(unsigned char),0))== 0 ||(x->sign ^ y->sign)== 0)?1: - 1;
# 123
return z;}
# 27
struct Cyc_AP_T*Cyc_AP_add(struct Cyc_AP_T*x,struct Cyc_AP_T*y){
# 126
struct Cyc_AP_T*z;
(unsigned)x?0:({int(*_tmp8C)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=({int(*_tmp13)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=(int(*)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line))Cyc___assert_fail;_tmp13;});struct _fat_ptr _tmp8B=({const char*_tmp14="x";_tag_fat(_tmp14,sizeof(char),2U);});_tmp8C(_tmp8B,({const char*_tmp15="ap.cyc";_tag_fat(_tmp15,sizeof(char),7U);}),127U);});
(unsigned)y?0:({int(*_tmp8E)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=({int(*_tmp16)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=(int(*)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line))Cyc___assert_fail;_tmp16;});struct _fat_ptr _tmp8D=({const char*_tmp17="y";_tag_fat(_tmp17,sizeof(char),2U);});_tmp8E(_tmp8D,({const char*_tmp18="ap.cyc";_tag_fat(_tmp18,sizeof(char),7U);}),128U);});
if((x->sign ^ y->sign)== 0){
z=({struct Cyc_AP_T*_tmp90=Cyc_mk((x->ndigits > y->ndigits?x->ndigits: y->ndigits)+ 1);struct Cyc_AP_T*_tmp8F=x;Cyc_add(_tmp90,_tmp8F,y);});
({int _tmp91=((struct Cyc_AP_T*)_check_null(z))->ndigits == 1 &&(int)*((unsigned char*)_check_fat_subscript(z->digits,sizeof(unsigned char),0))== 0?1: x->sign;z->sign=_tmp91;});}else{
# 133
if(Cyc_cmp(x,y)> 0){
z=({struct Cyc_AP_T*_tmp93=Cyc_mk(x->ndigits);struct Cyc_AP_T*_tmp92=x;Cyc_sub(_tmp93,_tmp92,y);});
({int _tmp94=((struct Cyc_AP_T*)_check_null(z))->ndigits == 1 &&(int)*((unsigned char*)_check_fat_subscript(z->digits,sizeof(unsigned char),0))== 0?1: x->sign;z->sign=_tmp94;});}else{
# 138
z=({struct Cyc_AP_T*_tmp96=Cyc_mk(y->ndigits);struct Cyc_AP_T*_tmp95=y;Cyc_sub(_tmp96,_tmp95,x);});
({int _tmp97=((struct Cyc_AP_T*)_check_null(z))->ndigits == 1 &&(int)*((unsigned char*)_check_fat_subscript(z->digits,sizeof(unsigned char),0))== 0?1: - x->sign;z->sign=_tmp97;});}}
# 141
return z;}
# 27
struct Cyc_AP_T*Cyc_AP_sub(struct Cyc_AP_T*x,struct Cyc_AP_T*y){
# 144
struct Cyc_AP_T*z;
(unsigned)x?0:({int(*_tmp99)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=({int(*_tmp19)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=(int(*)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line))Cyc___assert_fail;_tmp19;});struct _fat_ptr _tmp98=({const char*_tmp1A="x";_tag_fat(_tmp1A,sizeof(char),2U);});_tmp99(_tmp98,({const char*_tmp1B="ap.cyc";_tag_fat(_tmp1B,sizeof(char),7U);}),145U);});
(unsigned)y?0:({int(*_tmp9B)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=({int(*_tmp1C)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=(int(*)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line))Cyc___assert_fail;_tmp1C;});struct _fat_ptr _tmp9A=({const char*_tmp1D="y";_tag_fat(_tmp1D,sizeof(char),2U);});_tmp9B(_tmp9A,({const char*_tmp1E="ap.cyc";_tag_fat(_tmp1E,sizeof(char),7U);}),146U);});
if(!((x->sign ^ y->sign)== 0)){
z=({struct Cyc_AP_T*_tmp9D=Cyc_mk((x->ndigits > y->ndigits?x->ndigits: y->ndigits)+ 1);struct Cyc_AP_T*_tmp9C=x;Cyc_add(_tmp9D,_tmp9C,y);});
({int _tmp9E=((struct Cyc_AP_T*)_check_null(z))->ndigits == 1 &&(int)*((unsigned char*)_check_fat_subscript(z->digits,sizeof(unsigned char),0))== 0?1: x->sign;z->sign=_tmp9E;});}else{
# 151
if(Cyc_cmp(x,y)> 0){
z=({struct Cyc_AP_T*_tmpA0=Cyc_mk(x->ndigits);struct Cyc_AP_T*_tmp9F=x;Cyc_sub(_tmpA0,_tmp9F,y);});
({int _tmpA1=((struct Cyc_AP_T*)_check_null(z))->ndigits == 1 &&(int)*((unsigned char*)_check_fat_subscript(z->digits,sizeof(unsigned char),0))== 0?1: x->sign;z->sign=_tmpA1;});}else{
# 155
z=({struct Cyc_AP_T*_tmpA3=Cyc_mk(y->ndigits);struct Cyc_AP_T*_tmpA2=y;Cyc_sub(_tmpA3,_tmpA2,x);});
({int _tmpA4=((struct Cyc_AP_T*)_check_null(z))->ndigits == 1 &&(int)*((unsigned char*)_check_fat_subscript(z->digits,sizeof(unsigned char),0))== 0?1: - x->sign;z->sign=_tmpA4;});}}
# 158
return z;}
# 27
struct Cyc_AP_T*Cyc_AP_div(struct Cyc_AP_T*x,struct Cyc_AP_T*y){
# 161
struct Cyc_AP_T*q;struct Cyc_AP_T*r;
(unsigned)x?0:({int(*_tmpA6)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=({int(*_tmp1F)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=(int(*)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line))Cyc___assert_fail;_tmp1F;});struct _fat_ptr _tmpA5=({const char*_tmp20="x";_tag_fat(_tmp20,sizeof(char),2U);});_tmpA6(_tmpA5,({const char*_tmp21="ap.cyc";_tag_fat(_tmp21,sizeof(char),7U);}),162U);});
(unsigned)y?0:({int(*_tmpA8)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=({int(*_tmp22)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=(int(*)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line))Cyc___assert_fail;_tmp22;});struct _fat_ptr _tmpA7=({const char*_tmp23="y";_tag_fat(_tmp23,sizeof(char),2U);});_tmpA8(_tmpA7,({const char*_tmp24="ap.cyc";_tag_fat(_tmp24,sizeof(char),7U);}),163U);});
!(y->ndigits == 1 &&(int)*((unsigned char*)_check_fat_subscript(y->digits,sizeof(unsigned char),0))== 0)?0:({int(*_tmpAA)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=({int(*_tmp25)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=(int(*)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line))Cyc___assert_fail;_tmp25;});struct _fat_ptr _tmpA9=({const char*_tmp26="!((y)->ndigits==1 && (y)->digits[0]==0)";_tag_fat(_tmp26,sizeof(char),40U);});_tmpAA(_tmpA9,({const char*_tmp27="ap.cyc";_tag_fat(_tmp27,sizeof(char),7U);}),164U);});
q=Cyc_mk(x->ndigits);
r=Cyc_mk(y->ndigits);
{
struct _fat_ptr tmp=({unsigned _tmp28=(unsigned)((x->ndigits + y->ndigits)+ 2)* sizeof(unsigned char);_tag_fat(_cycalloc_atomic(_tmp28),1U,_tmp28);});
({int _tmpB0=x->ndigits;struct _fat_ptr _tmpAF=((struct Cyc_AP_T*)_check_null(q))->digits;struct _fat_ptr _tmpAE=x->digits;int _tmpAD=y->ndigits;struct _fat_ptr _tmpAC=y->digits;struct _fat_ptr _tmpAB=((struct Cyc_AP_T*)_check_null(r))->digits;Cyc_XP_div(_tmpB0,_tmpAF,_tmpAE,_tmpAD,_tmpAC,_tmpAB,tmp);});}
# 172
Cyc_normalize(q,q->size);
Cyc_normalize(r,r->size);
q->sign=(q->ndigits == 1 &&(int)*((unsigned char*)_check_fat_subscript(q->digits,sizeof(unsigned char),0))== 0 ||(x->sign ^ y->sign)== 0)?1: - 1;
# 176
return q;}
# 27
struct Cyc_AP_T*Cyc_AP_mod(struct Cyc_AP_T*x,struct Cyc_AP_T*y){
# 179
struct Cyc_AP_T*q;struct Cyc_AP_T*r;
(unsigned)x?0:({int(*_tmpB2)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=({int(*_tmp29)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=(int(*)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line))Cyc___assert_fail;_tmp29;});struct _fat_ptr _tmpB1=({const char*_tmp2A="x";_tag_fat(_tmp2A,sizeof(char),2U);});_tmpB2(_tmpB1,({const char*_tmp2B="ap.cyc";_tag_fat(_tmp2B,sizeof(char),7U);}),180U);});
(unsigned)y?0:({int(*_tmpB4)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=({int(*_tmp2C)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=(int(*)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line))Cyc___assert_fail;_tmp2C;});struct _fat_ptr _tmpB3=({const char*_tmp2D="y";_tag_fat(_tmp2D,sizeof(char),2U);});_tmpB4(_tmpB3,({const char*_tmp2E="ap.cyc";_tag_fat(_tmp2E,sizeof(char),7U);}),181U);});
!(y->ndigits == 1 &&(int)*((unsigned char*)_check_fat_subscript(y->digits,sizeof(unsigned char),0))== 0)?0:({int(*_tmpB6)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=({int(*_tmp2F)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=(int(*)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line))Cyc___assert_fail;_tmp2F;});struct _fat_ptr _tmpB5=({const char*_tmp30="!((y)->ndigits==1 && (y)->digits[0]==0)";_tag_fat(_tmp30,sizeof(char),40U);});_tmpB6(_tmpB5,({const char*_tmp31="ap.cyc";_tag_fat(_tmp31,sizeof(char),7U);}),182U);});
q=Cyc_mk(x->ndigits);
r=Cyc_mk(y->ndigits);
{
struct _fat_ptr tmp=({unsigned _tmp32=(unsigned)((x->ndigits + y->ndigits)+ 2)* sizeof(unsigned char);_tag_fat(_cycalloc_atomic(_tmp32),1U,_tmp32);});
({int _tmpBC=x->ndigits;struct _fat_ptr _tmpBB=((struct Cyc_AP_T*)_check_null(q))->digits;struct _fat_ptr _tmpBA=x->digits;int _tmpB9=y->ndigits;struct _fat_ptr _tmpB8=y->digits;struct _fat_ptr _tmpB7=((struct Cyc_AP_T*)_check_null(r))->digits;Cyc_XP_div(_tmpBC,_tmpBB,_tmpBA,_tmpB9,_tmpB8,_tmpB7,tmp);});}
# 190
Cyc_normalize(q,q->size);
Cyc_normalize(r,r->size);
r->sign=r->ndigits == 1 &&(int)*((unsigned char*)_check_fat_subscript(r->digits,sizeof(unsigned char),0))== 0?1: x->sign;
return r;}
# 27
struct Cyc_AP_T*Cyc_AP_pow(struct Cyc_AP_T*x,struct Cyc_AP_T*y,struct Cyc_AP_T*p){
# 196
struct Cyc_AP_T*z;
(unsigned)x?0:({int(*_tmpBE)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=({int(*_tmp33)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=(int(*)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line))Cyc___assert_fail;_tmp33;});struct _fat_ptr _tmpBD=({const char*_tmp34="x";_tag_fat(_tmp34,sizeof(char),2U);});_tmpBE(_tmpBD,({const char*_tmp35="ap.cyc";_tag_fat(_tmp35,sizeof(char),7U);}),197U);});
(unsigned)y?0:({int(*_tmpC0)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=({int(*_tmp36)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=(int(*)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line))Cyc___assert_fail;_tmp36;});struct _fat_ptr _tmpBF=({const char*_tmp37="y";_tag_fat(_tmp37,sizeof(char),2U);});_tmpC0(_tmpBF,({const char*_tmp38="ap.cyc";_tag_fat(_tmp38,sizeof(char),7U);}),198U);});
y->sign == 1?0:({int(*_tmpC2)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=({int(*_tmp39)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=(int(*)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line))Cyc___assert_fail;_tmp39;});struct _fat_ptr _tmpC1=({const char*_tmp3A="y->sign == 1";_tag_fat(_tmp3A,sizeof(char),13U);});_tmpC2(_tmpC1,({const char*_tmp3B="ap.cyc";_tag_fat(_tmp3B,sizeof(char),7U);}),199U);});
(!((unsigned)p)||(p->sign == 1 && !(p->ndigits == 1 &&(int)*((unsigned char*)_check_fat_subscript(p->digits,sizeof(unsigned char),0))== 0))&& !(p->ndigits == 1 &&(int)*((unsigned char*)_check_fat_subscript(p->digits,sizeof(unsigned char),0))== 1))?0:({int(*_tmpC4)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=({int(*_tmp3C)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=(int(*)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line))Cyc___assert_fail;_tmp3C;});struct _fat_ptr _tmpC3=({const char*_tmp3D="!p || p->sign==1 && !((p)->ndigits==1 && (p)->digits[0]==0) && !((p)->ndigits==1 && (p)->digits[0]==1)";_tag_fat(_tmp3D,sizeof(char),103U);});_tmpC4(_tmpC3,({const char*_tmp3E="ap.cyc";_tag_fat(_tmp3E,sizeof(char),7U);}),200U);});
if(x->ndigits == 1 &&(int)*((unsigned char*)_check_fat_subscript(x->digits,sizeof(unsigned char),0))== 0)
return Cyc_AP_new(0);
# 201
if(
# 203
y->ndigits == 1 &&(int)*((unsigned char*)_check_fat_subscript(y->digits,sizeof(unsigned char),0))== 0)
return Cyc_AP_new(1);
# 201
if(
# 205
x->ndigits == 1 &&(int)*((unsigned char*)_check_fat_subscript(x->digits,sizeof(unsigned char),0))== 1)
return Cyc_AP_new(((int)*((unsigned char*)_check_fat_subscript(y->digits,sizeof(unsigned char),0))& 1)== 0?1: x->sign);
# 201
if((unsigned)p){
# 208
if(y->ndigits == 1 &&(int)*((unsigned char*)_check_fat_subscript(y->digits,sizeof(unsigned char),0))== 1)
z=Cyc_AP_mod(x,p);else{
# 211
struct Cyc_AP_T*y2=Cyc_AP_rshift(y,1);struct Cyc_AP_T*t=Cyc_AP_pow(x,y2,p);
z=Cyc_mulmod(t,t,p);
if(!(((int)*((unsigned char*)_check_fat_subscript(y->digits,sizeof(unsigned char),0))& 1)== 0))
z=({struct Cyc_AP_T*_tmpC6=y2=Cyc_AP_mod(x,p);struct Cyc_AP_T*_tmpC5=t=z;Cyc_mulmod(_tmpC6,_tmpC5,p);});}}else{
# 218
if(y->ndigits == 1 &&(int)*((unsigned char*)_check_fat_subscript(y->digits,sizeof(unsigned char),0))== 1)
z=Cyc_AP_addi(x,0);else{
# 221
struct Cyc_AP_T*y2=Cyc_AP_rshift(y,1);struct Cyc_AP_T*t=Cyc_AP_pow(x,y2,0);
z=Cyc_AP_mul(t,t);
if(!(((int)*((unsigned char*)_check_fat_subscript(y->digits,sizeof(unsigned char),0))& 1)== 0))
z=({struct Cyc_AP_T*_tmpC7=x;Cyc_AP_mul(_tmpC7,(t=z));});}}
# 227
return z;}
# 27
int Cyc_AP_cmp(struct Cyc_AP_T*x,struct Cyc_AP_T*y){
# 230
(unsigned)x?0:({int(*_tmpC9)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=({int(*_tmp3F)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=(int(*)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line))Cyc___assert_fail;_tmp3F;});struct _fat_ptr _tmpC8=({const char*_tmp40="x";_tag_fat(_tmp40,sizeof(char),2U);});_tmpC9(_tmpC8,({const char*_tmp41="ap.cyc";_tag_fat(_tmp41,sizeof(char),7U);}),230U);});
(unsigned)y?0:({int(*_tmpCB)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=({int(*_tmp42)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=(int(*)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line))Cyc___assert_fail;_tmp42;});struct _fat_ptr _tmpCA=({const char*_tmp43="y";_tag_fat(_tmp43,sizeof(char),2U);});_tmpCB(_tmpCA,({const char*_tmp44="ap.cyc";_tag_fat(_tmp44,sizeof(char),7U);}),231U);});
if(!((x->sign ^ y->sign)== 0))
return x->sign;else{
if(x->sign == 1)
return Cyc_cmp(x,y);else{
# 237
return Cyc_cmp(y,x);}}}
# 27
struct Cyc_AP_T*Cyc_AP_addi(struct Cyc_AP_T*x,long y){
# 240
struct Cyc_AP_T*t=Cyc_mk((int)(sizeof(unsigned long)/ sizeof(unsigned char)));
return({struct Cyc_AP_T*_tmpCC=x;Cyc_AP_add(_tmpCC,Cyc_set(t,y));});}
# 27
struct Cyc_AP_T*Cyc_AP_subi(struct Cyc_AP_T*x,long y){
# 244
struct Cyc_AP_T*t=Cyc_mk((int)(sizeof(unsigned long)/ sizeof(unsigned char)));
return({struct Cyc_AP_T*_tmpCD=x;Cyc_AP_sub(_tmpCD,Cyc_set(t,y));});}
# 27
struct Cyc_AP_T*Cyc_AP_muli(struct Cyc_AP_T*x,long y){
# 248
struct Cyc_AP_T*t=Cyc_mk((int)(sizeof(unsigned long)/ sizeof(unsigned char)));
return({struct Cyc_AP_T*_tmpCE=x;Cyc_AP_mul(_tmpCE,Cyc_set(t,y));});}
# 27
struct Cyc_AP_T*Cyc_AP_divi(struct Cyc_AP_T*x,long y){
# 252
struct Cyc_AP_T*t=Cyc_mk((int)(sizeof(unsigned long)/ sizeof(unsigned char)));
return({struct Cyc_AP_T*_tmpCF=x;Cyc_AP_div(_tmpCF,Cyc_set(t,y));});}
# 27
int Cyc_AP_cmpi(struct Cyc_AP_T*x,long y){
# 256
struct Cyc_AP_T*t=Cyc_mk((int)(sizeof(unsigned long)/ sizeof(unsigned char)));
return({struct Cyc_AP_T*_tmpD0=x;Cyc_AP_cmp(_tmpD0,Cyc_set(t,y));});}
# 27
long Cyc_AP_modi(struct Cyc_AP_T*x,long y){
# 260
long rem;
struct Cyc_AP_T*r;
struct Cyc_AP_T*t=Cyc_mk((int)(sizeof(unsigned long)/ sizeof(unsigned char)));
r=({struct Cyc_AP_T*_tmpD1=x;Cyc_AP_mod(_tmpD1,Cyc_set(t,y));});
rem=(long)Cyc_XP_toint(((struct Cyc_AP_T*)_check_null(r))->ndigits,r->digits);
return rem;}
# 27
struct Cyc_AP_T*Cyc_AP_lshift(struct Cyc_AP_T*x,int s){
# 268
struct Cyc_AP_T*z;
(unsigned)x?0:({int(*_tmpD3)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=({int(*_tmp45)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=(int(*)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line))Cyc___assert_fail;_tmp45;});struct _fat_ptr _tmpD2=({const char*_tmp46="x";_tag_fat(_tmp46,sizeof(char),2U);});_tmpD3(_tmpD2,({const char*_tmp47="ap.cyc";_tag_fat(_tmp47,sizeof(char),7U);}),269U);});
s >= 0?0:({int(*_tmpD5)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=({int(*_tmp48)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=(int(*)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line))Cyc___assert_fail;_tmp48;});struct _fat_ptr _tmpD4=({const char*_tmp49="s >= 0";_tag_fat(_tmp49,sizeof(char),7U);});_tmpD5(_tmpD4,({const char*_tmp4A="ap.cyc";_tag_fat(_tmp4A,sizeof(char),7U);}),270U);});
z=Cyc_mk(x->ndigits + (s + 7 & ~ 7)/ 8);
Cyc_XP_lshift(((struct Cyc_AP_T*)_check_null(z))->size,z->digits,x->ndigits,x->digits,s,0);
# 274
z->sign=x->sign;
return Cyc_normalize(z,z->size);}
# 27
struct Cyc_AP_T*Cyc_AP_rshift(struct Cyc_AP_T*x,int s){
# 278
(unsigned)x?0:({int(*_tmpD7)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=({int(*_tmp4B)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=(int(*)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line))Cyc___assert_fail;_tmp4B;});struct _fat_ptr _tmpD6=({const char*_tmp4C="x";_tag_fat(_tmp4C,sizeof(char),2U);});_tmpD7(_tmpD6,({const char*_tmp4D="ap.cyc";_tag_fat(_tmp4D,sizeof(char),7U);}),278U);});
s >= 0?0:({int(*_tmpD9)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=({int(*_tmp4E)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=(int(*)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line))Cyc___assert_fail;_tmp4E;});struct _fat_ptr _tmpD8=({const char*_tmp4F="s >= 0";_tag_fat(_tmp4F,sizeof(char),7U);});_tmpD9(_tmpD8,({const char*_tmp50="ap.cyc";_tag_fat(_tmp50,sizeof(char),7U);}),279U);});
if(s >= 8 * x->ndigits)
return Cyc_AP_new(0);else{
# 283
struct Cyc_AP_T*z=Cyc_mk(x->ndigits - s / 8);
Cyc_XP_rshift(((struct Cyc_AP_T*)_check_null(z))->size,z->digits,x->ndigits,x->digits,s,0);
# 286
Cyc_normalize(z,z->size);
z->sign=z->ndigits == 1 &&(int)*((unsigned char*)_check_fat_subscript(z->digits,sizeof(unsigned char),0))== 0?1: x->sign;
return z;}}
# 27
struct Cyc_AP_T*Cyc_AP_and(struct Cyc_AP_T*x,struct Cyc_AP_T*y){
# 292
int i;
struct Cyc_AP_T*z;
if(({int _tmpDA=((struct Cyc_AP_T*)_check_null(x))->ndigits;_tmpDA < ((struct Cyc_AP_T*)_check_null(y))->ndigits;}))
return Cyc_AP_and(y,x);
# 294
z=
# 296
Cyc_mk(y->ndigits);
Cyc_XP_and(y->ndigits,((struct Cyc_AP_T*)_check_null(z))->digits,x->digits,y->digits);
Cyc_normalize(z,z->size);
z->sign=1;
return z;}
# 27
struct Cyc_AP_T*Cyc_AP_or(struct Cyc_AP_T*x,struct Cyc_AP_T*y){
# 303
int i;
struct Cyc_AP_T*z;
if(({int _tmpDB=((struct Cyc_AP_T*)_check_null(x))->ndigits;_tmpDB < ((struct Cyc_AP_T*)_check_null(y))->ndigits;}))
return Cyc_AP_or(y,x);
# 305
z=
# 307
Cyc_mk(x->ndigits);
Cyc_XP_or(y->ndigits,((struct Cyc_AP_T*)_check_null(z))->digits,x->digits,y->digits);
for(i=y->ndigits;i < x->ndigits;++ i){
Cyc_normalize(z,z->size);}
z->sign=1;
return z;}
# 27
struct Cyc_AP_T*Cyc_AP_xor(struct Cyc_AP_T*x,struct Cyc_AP_T*y){
# 315
int i;
struct Cyc_AP_T*z;
if(({int _tmpDC=((struct Cyc_AP_T*)_check_null(x))->ndigits;_tmpDC < ((struct Cyc_AP_T*)_check_null(y))->ndigits;}))
return Cyc_AP_xor(y,x);
# 317
z=
# 319
Cyc_mk(x->ndigits);
Cyc_XP_xor(y->ndigits,((struct Cyc_AP_T*)_check_null(z))->digits,x->digits,y->digits);
for(i=y->ndigits;i < x->ndigits;++ i){
Cyc_normalize(z,z->size);}
z->sign=1;
return z;}
# 27
struct Cyc_AP_T*Cyc_AP_fromint(long x){
# 327
struct Cyc_AP_T*t=Cyc_mk((int)(sizeof(unsigned long)/ sizeof(unsigned char)));
return Cyc_set(t,x);}
# 27
long Cyc_AP_toint(struct Cyc_AP_T*x){
# 331
unsigned long u;
(unsigned)x?0:({int(*_tmpDE)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=({int(*_tmp51)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=(int(*)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line))Cyc___assert_fail;_tmp51;});struct _fat_ptr _tmpDD=({const char*_tmp52="x";_tag_fat(_tmp52,sizeof(char),2U);});_tmpDE(_tmpDD,({const char*_tmp53="ap.cyc";_tag_fat(_tmp53,sizeof(char),7U);}),332U);});
u=({unsigned long _tmpDF=Cyc_XP_toint(x->ndigits,x->digits);_tmpDF % ((unsigned)Cyc_long_max + 1U);});
if(x->sign == - 1)
return -(long)u;else{
# 337
return(long)u;}}
# 27
struct Cyc_AP_T*Cyc_AP_fromstr(const char*str,int base){
# 340
struct Cyc_AP_T*z;
struct _fat_ptr p=({const char*_tmp5E=str;_tag_fat(_tmp5E,sizeof(char),_get_zero_arr_size_char((void*)_tmp5E,1U));});
char sign='\000';
int carry;
(unsigned)p.curr?0:({int(*_tmpE1)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=({int(*_tmp54)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=(int(*)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line))Cyc___assert_fail;_tmp54;});struct _fat_ptr _tmpE0=({const char*_tmp55="p";_tag_fat(_tmp55,sizeof(char),2U);});_tmpE1(_tmpE0,({const char*_tmp56="ap.cyc";_tag_fat(_tmp56,sizeof(char),7U);}),344U);});
base >= 2 && base <= 36?0:({int(*_tmpE3)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=({int(*_tmp57)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=(int(*)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line))Cyc___assert_fail;_tmp57;});struct _fat_ptr _tmpE2=({const char*_tmp58="base >= 2 && base <= 36";_tag_fat(_tmp58,sizeof(char),24U);});_tmpE3(_tmpE2,({const char*_tmp59="ap.cyc";_tag_fat(_tmp59,sizeof(char),7U);}),345U);});
while((int)*((const char*)_check_fat_subscript(p,sizeof(char),0U))&& isspace((int)*((const char*)_check_fat_subscript(p,sizeof(char),0U)))){
_fat_ptr_inplace_plus(& p,sizeof(char),1);}
if((int)*((const char*)_check_fat_subscript(p,sizeof(char),0U))== (int)'-' ||(int)*((const char*)_check_fat_subscript(p,sizeof(char),0U))== (int)'+')
sign=*((const char*)_check_fat_subscript(_fat_ptr_inplace_plus_post(& p,sizeof(char),1),sizeof(char),0U));
# 348
{
# 351
const char*start;
int k;int n=0;
for(0;(int)*((const char*)_check_fat_subscript(p,sizeof(char),0U))== (int)'0' &&(int)*((const char*)_check_fat_subscript(p,sizeof(char),1))== (int)'0';_fat_ptr_inplace_plus(& p,sizeof(char),1)){
;}
start=(const char*)_untag_fat_ptr(p,sizeof(char),1U);
for(0;(((int)'0' <= (int)*((const char*)_check_fat_subscript(p,sizeof(char),0U))&&(int)*((const char*)_check_fat_subscript(p,sizeof(char),0U))<= (int)'9')&&(int)*((const char*)_check_fat_subscript(p,sizeof(char),0U))< (int)'0' + base ||
((int)'a' <= (int)*((const char*)_check_fat_subscript(p,sizeof(char),0U))&&(int)*((const char*)_check_fat_subscript(p,sizeof(char),0U))<= (int)'z')&&(int)*((const char*)_check_fat_subscript(p,sizeof(char),0U))< ((int)'a' + base)- 10)||
((int)'A' <= (int)*((const char*)_check_fat_subscript(p,sizeof(char),0U))&&(int)*((const char*)_check_fat_subscript(p,sizeof(char),0U))<= (int)'Z')&&(int)*((const char*)_check_fat_subscript(p,sizeof(char),0U))< ((int)'A' + base)- 10;_fat_ptr_inplace_plus(& p,sizeof(char),1)){
++ n;}
for(k=1;1 << k < base;++ k){
;}
z=Cyc_mk((k * n + 7 & ~ 7)/ 8);
p=({const char*_tmp5A=start;_tag_fat(_tmp5A,sizeof(char),_get_zero_arr_size_char((void*)_tmp5A,1U));});}
# 365
carry=({int _tmpE6=((struct Cyc_AP_T*)_check_null(z))->size;struct _fat_ptr _tmpE5=z->digits;const char*_tmpE4=(const char*)_untag_fat_ptr(p,sizeof(char),1U);Cyc_XP_fromstr(_tmpE6,_tmpE5,_tmpE4,base);});
# 367
carry == 0?0:({int(*_tmpE8)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=({int(*_tmp5B)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=(int(*)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line))Cyc___assert_fail;_tmp5B;});struct _fat_ptr _tmpE7=({const char*_tmp5C="carry == 0";_tag_fat(_tmp5C,sizeof(char),11U);});_tmpE8(_tmpE7,({const char*_tmp5D="ap.cyc";_tag_fat(_tmp5D,sizeof(char),7U);}),367U);});
Cyc_normalize(z,z->size);
z->sign=(z->ndigits == 1 &&(int)*((unsigned char*)_check_fat_subscript(z->digits,sizeof(unsigned char),0))== 0 ||(int)sign != (int)'-')?1: - 1;
return z;}
# 27
char*Cyc_AP_tostr(struct Cyc_AP_T*x,int base){
# 373
struct _fat_ptr q;
struct _fat_ptr str;
int size;int k;
(unsigned)x?0:({int(*_tmpEA)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=({int(*_tmp5F)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=(int(*)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line))Cyc___assert_fail;_tmp5F;});struct _fat_ptr _tmpE9=({const char*_tmp60="x";_tag_fat(_tmp60,sizeof(char),2U);});_tmpEA(_tmpE9,({const char*_tmp61="ap.cyc";_tag_fat(_tmp61,sizeof(char),7U);}),376U);});
base >= 2 && base <= 36?0:({int(*_tmpEC)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=({int(*_tmp62)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=(int(*)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line))Cyc___assert_fail;_tmp62;});struct _fat_ptr _tmpEB=({const char*_tmp63="base >= 2 && base <= 36";_tag_fat(_tmp63,sizeof(char),24U);});_tmpEC(_tmpEB,({const char*_tmp64="ap.cyc";_tag_fat(_tmp64,sizeof(char),7U);}),377U);});
for(k=5;1 << k > base;-- k){
;}
size=((8 * x->ndigits)/ k + 1)+ 1;
if(x->sign == - 1)
++ size;
# 381
str=({unsigned _tmp65=size;_tag_fat(_cyccalloc_atomic(sizeof(char),_tmp65),sizeof(char),_tmp65);});
# 384
q=({unsigned _tmp66=(unsigned)x->ndigits * sizeof(unsigned char);_tag_fat(_cycalloc_atomic(_tmp66),1U,_tmp66);});
Cyc__memcpy(q,(struct _fat_ptr)x->digits,(unsigned)x->ndigits / sizeof(*((unsigned char*)(x->digits).curr))+ (unsigned)((unsigned)x->ndigits % sizeof(*((unsigned char*)(x->digits).curr))== (unsigned)0?0: 1),sizeof(*((unsigned char*)(x->digits).curr)));
if(x->sign == - 1){
({struct _fat_ptr _tmp67=_fat_ptr_plus(str,sizeof(char),0);char _tmp68=*((char*)_check_fat_subscript(_tmp67,sizeof(char),0U));char _tmp69='-';if(_get_fat_size(_tmp67,sizeof(char))== 1U &&(_tmp68 == 0 && _tmp69 != 0))_throw_arraybounds();*((char*)_tmp67.curr)=_tmp69;});
({struct _fat_ptr _tmpF0=_fat_ptr_plus(str,sizeof(char),1);int _tmpEF=size - 1;int _tmpEE=base;int _tmpED=x->ndigits;Cyc_XP_tostr(_tmpF0,_tmpEF,_tmpEE,_tmpED,q);});}else{
# 390
Cyc_XP_tostr(str,size,base,x->ndigits,q);}
return(char*)_untag_fat_ptr(str,sizeof(char),1U);}
# 27
struct Cyc_AP_T*Cyc_AP_gcd(struct Cyc_AP_T*x,struct Cyc_AP_T*y){
# 394
struct Cyc_AP_T*z;struct Cyc_AP_T*tmp;
(unsigned)x?0:({int(*_tmpF2)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=({int(*_tmp6A)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=(int(*)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line))Cyc___assert_fail;_tmp6A;});struct _fat_ptr _tmpF1=({const char*_tmp6B="x";_tag_fat(_tmp6B,sizeof(char),2U);});_tmpF2(_tmpF1,({const char*_tmp6C="ap.cyc";_tag_fat(_tmp6C,sizeof(char),7U);}),395U);});
(unsigned)y?0:({int(*_tmpF4)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=({int(*_tmp6D)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=(int(*)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line))Cyc___assert_fail;_tmp6D;});struct _fat_ptr _tmpF3=({const char*_tmp6E="y";_tag_fat(_tmp6E,sizeof(char),2U);});_tmpF4(_tmpF3,({const char*_tmp6F="ap.cyc";_tag_fat(_tmp6F,sizeof(char),7U);}),396U);});
while(!(((struct Cyc_AP_T*)_check_null(y))->ndigits == 1 &&(int)*((unsigned char*)_check_fat_subscript(y->digits,sizeof(unsigned char),0))== 0)){
tmp=Cyc_AP_mod(x,y);
x=y;
y=tmp;}
# 402
z=Cyc_mk(x->ndigits);
Cyc__memcpy(((struct Cyc_AP_T*)_check_null(z))->digits,(struct _fat_ptr)x->digits,(unsigned)x->ndigits / sizeof(*((unsigned char*)(x->digits).curr))+ (unsigned)((unsigned)x->ndigits % sizeof(*((unsigned char*)(x->digits).curr))== (unsigned)0?0: 1),sizeof(*((unsigned char*)(x->digits).curr)));
z->ndigits=x->ndigits;
z->sign=x->sign;
return z;}
# 27
struct Cyc_AP_T*Cyc_AP_lcm(struct Cyc_AP_T*x,struct Cyc_AP_T*y){
# 409
return({struct Cyc_AP_T*_tmpF6=x;Cyc_AP_mul(_tmpF6,({struct Cyc_AP_T*_tmpF5=y;Cyc_AP_div(_tmpF5,Cyc_AP_gcd(x,y));}));});}
