#include <setjmp.h>
/* This is a C header file to be used by the output of the Cyclone to
   C translator.  The corresponding definitions are in file lib/runtime_*.c */
#ifndef _CYC_INCLUDE_H_
#define _CYC_INCLUDE_H_

/* Need one of these per thread (see runtime_stack.c). The runtime maintains 
   a stack that contains either _handler_cons structs or _RegionHandle structs.
   The tag is 0 for a handler_cons and 1 for a region handle.  */
struct _RuntimeStack {
  int tag; 
  struct _RuntimeStack *next;
  void (*cleanup)(struct _RuntimeStack *frame);
};

#ifndef offsetof
/* should be size_t, but int is fine. */
#define offsetof(t,n) ((int)(&(((t *)0)->n)))
#endif

/* Fat pointers */
struct _fat_ptr {
  unsigned char *curr; 
  unsigned char *base; 
  unsigned char *last_plus_one; 
};  

/* Discriminated Unions */
struct _xtunion_struct { char *tag; };

/* Regions */
struct _RegionPage
#ifdef CYC_REGION_PROFILE
{ unsigned total_bytes;
  unsigned free_bytes;
  /* MWH: wish we didn't have to include the stuff below ... */
  struct _RegionPage *next;
  char data[1];
}
#endif
; // abstract -- defined in runtime_memory.c
struct _RegionHandle {
  struct _RuntimeStack s;
  struct _RegionPage *curr;
  char               *offset;
  char               *last_plus_one;
  struct _DynRegionHandle *sub_regions;
#ifdef CYC_REGION_PROFILE
  const char         *name;
#else
  unsigned used_bytes;
  unsigned wasted_bytes;
#endif
};
struct _DynRegionFrame {
  struct _RuntimeStack s;
  struct _DynRegionHandle *x;
};
// A dynamic region is just a region handle.  The wrapper struct is for type
// abstraction.
struct Cyc_Core_DynamicRegion {
  struct _RegionHandle h;
};

struct _RegionHandle _new_region(const char*);
void* _region_malloc(struct _RegionHandle*, unsigned);
void* _region_calloc(struct _RegionHandle*, unsigned t, unsigned n);
void   _free_region(struct _RegionHandle*);
struct _RegionHandle*_open_dynregion(struct _DynRegionFrame*,struct _DynRegionHandle*);
void   _pop_dynregion();

/* Exceptions */
struct _handler_cons {
  struct _RuntimeStack s;
  jmp_buf handler;
};
void _push_handler(struct _handler_cons *);
void _push_region(struct _RegionHandle *);
void _npop_handler(int);
void _pop_handler();
void _pop_region();

#ifndef _throw
void* _throw_null_fn(const char*,unsigned);
void* _throw_arraybounds_fn(const char*,unsigned);
void* _throw_badalloc_fn(const char*,unsigned);
void* _throw_match_fn(const char*,unsigned);
void* _throw_fn(void*,const char*,unsigned);
void* _rethrow(void*);
#define _throw_null() (_throw_null_fn(__FILE__,__LINE__))
#define _throw_arraybounds() (_throw_arraybounds_fn(__FILE__,__LINE__))
#define _throw_badalloc() (_throw_badalloc_fn(__FILE__,__LINE__))
#define _throw_match() (_throw_match_fn(__FILE__,__LINE__))
#define _throw(e) (_throw_fn((e),__FILE__,__LINE__))
#endif

struct _xtunion_struct* Cyc_Core_get_exn_thrown();
/* Built-in Exceptions */
struct Cyc_Null_Exception_exn_struct { char *tag; };
struct Cyc_Array_bounds_exn_struct { char *tag; };
struct Cyc_Match_Exception_exn_struct { char *tag; };
struct Cyc_Bad_alloc_exn_struct { char *tag; };
extern char Cyc_Null_Exception[];
extern char Cyc_Array_bounds[];
extern char Cyc_Match_Exception[];
extern char Cyc_Bad_alloc[];

/* Built-in Run-time Checks and company */
#ifdef CYC_ANSI_OUTPUT
#define _INLINE  
#else
#define _INLINE inline
#endif

#ifdef NO_CYC_NULL_CHECKS
#define _check_null(ptr) (ptr)
#else
#define _check_null(ptr) \
  ({ void*_cks_null = (void*)(ptr); \
     if (!_cks_null) _throw_null(); \
     _cks_null; })
#endif

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index)\
   (((char*)ptr) + (elt_sz)*(index))
#ifdef NO_CYC_NULL_CHECKS
#define _check_known_subscript_null _check_known_subscript_notnull
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr);\
  int _index = (index);\
  if (!_cks_ptr) _throw_null(); \
  _cks_ptr + (elt_sz)*_index; })
#endif
#define _zero_arr_plus_fn(orig_x,orig_sz,orig_i,f,l) ((orig_x)+(orig_i))
#define _zero_arr_plus_char_fn _zero_arr_plus_fn
#define _zero_arr_plus_short_fn _zero_arr_plus_fn
#define _zero_arr_plus_int_fn _zero_arr_plus_fn
#define _zero_arr_plus_float_fn _zero_arr_plus_fn
#define _zero_arr_plus_double_fn _zero_arr_plus_fn
#define _zero_arr_plus_longdouble_fn _zero_arr_plus_fn
#define _zero_arr_plus_voidstar_fn _zero_arr_plus_fn
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (!_cks_ptr) _throw_null(); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })

/* _zero_arr_plus_*_fn(x,sz,i,filename,lineno) adds i to zero-terminated ptr
   x that has at least sz elements */
char* _zero_arr_plus_char_fn(char*,unsigned,int,const char*,unsigned);
short* _zero_arr_plus_short_fn(short*,unsigned,int,const char*,unsigned);
int* _zero_arr_plus_int_fn(int*,unsigned,int,const char*,unsigned);
float* _zero_arr_plus_float_fn(float*,unsigned,int,const char*,unsigned);
double* _zero_arr_plus_double_fn(double*,unsigned,int,const char*,unsigned);
long double* _zero_arr_plus_longdouble_fn(long double*,unsigned,int,const char*, unsigned);
void** _zero_arr_plus_voidstar_fn(void**,unsigned,int,const char*,unsigned);
#endif

/* _get_zero_arr_size_*(x,sz) returns the number of elements in a
   zero-terminated array that is NULL or has at least sz elements */
int _get_zero_arr_size_char(const char*,unsigned);
int _get_zero_arr_size_short(const short*,unsigned);
int _get_zero_arr_size_int(const int*,unsigned);
int _get_zero_arr_size_float(const float*,unsigned);
int _get_zero_arr_size_double(const double*,unsigned);
int _get_zero_arr_size_longdouble(const long double*,unsigned);
int _get_zero_arr_size_voidstar(const void**,unsigned);

/* _zero_arr_inplace_plus_*_fn(x,i,filename,lineno) sets
   zero-terminated pointer *x to *x + i */
char* _zero_arr_inplace_plus_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_short_fn(short**,int,const char*,unsigned);
int* _zero_arr_inplace_plus_int(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_longdouble_fn(long double**,int,const char*,unsigned);
void** _zero_arr_inplace_plus_voidstar_fn(void***,int,const char*,unsigned);
/* like the previous functions, but does post-addition (as in e++) */
char* _zero_arr_inplace_plus_post_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_post_short_fn(short**x,int,const char*,unsigned);
int* _zero_arr_inplace_plus_post_int_fn(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_post_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_post_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_post_longdouble_fn(long double**,int,const char *,unsigned);
void** _zero_arr_inplace_plus_post_voidstar_fn(void***,int,const char*,unsigned);
#define _zero_arr_plus_char(x,s,i) \
  (_zero_arr_plus_char_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_short(x,s,i) \
  (_zero_arr_plus_short_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_int(x,s,i) \
  (_zero_arr_plus_int_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_float(x,s,i) \
  (_zero_arr_plus_float_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_double(x,s,i) \
  (_zero_arr_plus_double_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_longdouble(x,s,i) \
  (_zero_arr_plus_longdouble_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_voidstar(x,s,i) \
  (_zero_arr_plus_voidstar_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_inplace_plus_char(x,i) \
  _zero_arr_inplace_plus_char_fn((char **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_short(x,i) \
  _zero_arr_inplace_plus_short_fn((short **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_int(x,i) \
  _zero_arr_inplace_plus_int_fn((int **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_float(x,i) \
  _zero_arr_inplace_plus_float_fn((float **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_double(x,i) \
  _zero_arr_inplace_plus_double_fn((double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_longdouble(x,i) \
  _zero_arr_inplace_plus_longdouble_fn((long double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_voidstar(x,i) \
  _zero_arr_inplace_plus_voidstar_fn((void ***)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_char(x,i) \
  _zero_arr_inplace_plus_post_char_fn((char **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_short(x,i) \
  _zero_arr_inplace_plus_post_short_fn((short **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_int(x,i) \
  _zero_arr_inplace_plus_post_int_fn((int **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_float(x,i) \
  _zero_arr_inplace_plus_post_float_fn((float **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_double(x,i) \
  _zero_arr_inplace_plus_post_double_fn((double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_longdouble(x,i) \
  _zero_arr_inplace_plus_post_longdouble_fn((long double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_voidstar(x,i) \
  _zero_arr_inplace_plus_post_voidstar_fn((void***)(x),(i),__FILE__,__LINE__)

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_fat_subscript(arr,elt_sz,index) ((arr).curr + (elt_sz) * (index))
#define _untag_fat_ptr(arr,elt_sz,num_elts) ((arr).curr)
#else
#define _check_fat_subscript(arr,elt_sz,index) ({ \
  struct _fat_ptr _cus_arr = (arr); \
  unsigned char *_cus_ans = _cus_arr.curr + (elt_sz) * (index); \
  /* JGM: not needed! if (!_cus_arr.base) _throw_null();*/ \
  if (_cus_ans < _cus_arr.base || _cus_ans >= _cus_arr.last_plus_one) \
    _throw_arraybounds(); \
  _cus_ans; })
#define _untag_fat_ptr(arr,elt_sz,num_elts) ({ \
  struct _fat_ptr _arr = (arr); \
  unsigned char *_curr = _arr.curr; \
  if ((_curr < _arr.base || _curr + (elt_sz) * (num_elts) > _arr.last_plus_one) &&\
      _curr != (unsigned char *)0) \
    _throw_arraybounds(); \
  _curr; })
#endif

#define _tag_fat(tcurr,elt_sz,num_elts) ({ \
  struct _fat_ptr _ans; \
  unsigned _num_elts = (num_elts);\
  _ans.base = _ans.curr = (void*)(tcurr); \
  /* JGM: if we're tagging NULL, ignore num_elts */ \
  _ans.last_plus_one = _ans.base ? (_ans.base + (elt_sz) * _num_elts) : 0; \
  _ans; })

#define _get_fat_size(arr,elt_sz) \
  ({struct _fat_ptr _arr = (arr); \
    unsigned char *_arr_curr=_arr.curr; \
    unsigned char *_arr_last=_arr.last_plus_one; \
    (_arr_curr < _arr.base || _arr_curr >= _arr_last) ? 0 : \
    ((_arr_last - _arr_curr) / (elt_sz));})

#define _fat_ptr_plus(arr,elt_sz,change) ({ \
  struct _fat_ptr _ans = (arr); \
  int _change = (change);\
  _ans.curr += (elt_sz) * _change;\
  _ans; })
#define _fat_ptr_inplace_plus(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  _arr_ptr->curr += (elt_sz) * (change);\
  *_arr_ptr; })
#define _fat_ptr_inplace_plus_post(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  struct _fat_ptr _ans = *_arr_ptr; \
  _arr_ptr->curr += (elt_sz) * (change);\
  _ans; })

//Not a macro since initialization order matters. Defined in runtime_zeroterm.c.
struct _fat_ptr _fat_ptr_decrease_size(struct _fat_ptr,unsigned sz,unsigned numelts);

/* Allocation */
void* GC_malloc(int);
void* GC_malloc_atomic(int);
void* GC_calloc(unsigned,unsigned);
void* GC_calloc_atomic(unsigned,unsigned);
// bound the allocation size to be < MAX_ALLOC_SIZE. See macros below for usage.
#define MAX_MALLOC_SIZE (1 << 28)
void* _bounded_GC_malloc(int,const char*,int);
void* _bounded_GC_malloc_atomic(int,const char*,int);
void* _bounded_GC_calloc(unsigned,unsigned,const char*,int);
void* _bounded_GC_calloc_atomic(unsigned,unsigned,const char*,int);
/* these macros are overridden below ifdef CYC_REGION_PROFILE */
#ifndef CYC_REGION_PROFILE
#define _cycalloc(n) _bounded_GC_malloc(n,__FILE__,__LINE__)
#define _cycalloc_atomic(n) _bounded_GC_malloc_atomic(n,__FILE__,__LINE__)
#define _cyccalloc(n,s) _bounded_GC_calloc(n,s,__FILE__,__LINE__)
#define _cyccalloc_atomic(n,s) _bounded_GC_calloc_atomic(n,s,__FILE__,__LINE__)
#endif

static _INLINE unsigned int _check_times(unsigned x, unsigned y) {
  unsigned long long whole_ans = 
    ((unsigned long long) x)*((unsigned long long)y);
  unsigned word_ans = (unsigned)whole_ans;
  if(word_ans < whole_ans || word_ans > MAX_MALLOC_SIZE)
    _throw_badalloc();
  return word_ans;
}

#define _CYC_MAX_REGION_CONST 2
#define _CYC_MIN_ALIGNMENT (sizeof(double))

#ifdef CYC_REGION_PROFILE
extern int rgn_total_bytes;
#endif

static _INLINE void *_fast_region_malloc(struct _RegionHandle *r, unsigned orig_s) {  
  if (r > (struct _RegionHandle *)_CYC_MAX_REGION_CONST && r->curr != 0) { 
#ifdef CYC_NOALIGN
    unsigned s =  orig_s;
#else
    unsigned s =  (orig_s + _CYC_MIN_ALIGNMENT - 1) & (~(_CYC_MIN_ALIGNMENT -1)); 
#endif
    char *result; 
    result = r->offset; 
    if (s <= (r->last_plus_one - result)) {
      r->offset = result + s; 
#ifdef CYC_REGION_PROFILE
    r->curr->free_bytes = r->curr->free_bytes - s;
    rgn_total_bytes += s;
#endif
      return result;
    }
  } 
  return _region_malloc(r,orig_s); 
}

#ifdef CYC_REGION_PROFILE
/* see macros below for usage. defined in runtime_memory.c */
void* _profile_GC_malloc(int,const char*,const char*,int);
void* _profile_GC_malloc_atomic(int,const char*,const char*,int);
void* _profile_GC_calloc(unsigned,unsigned,const char*,const char*,int);
void* _profile_GC_calloc_atomic(unsigned,unsigned,const char*,const char*,int);
void* _profile_region_malloc(struct _RegionHandle*,unsigned,const char*,const char*,int);
void* _profile_region_calloc(struct _RegionHandle*,unsigned,unsigned,const char *,const char*,int);
struct _RegionHandle _profile_new_region(const char*,const char*,const char*,int);
void _profile_free_region(struct _RegionHandle*,const char*,const char*,int);
#ifndef RUNTIME_CYC
#define _new_region(n) _profile_new_region(n,__FILE__,__FUNCTION__,__LINE__)
#define _free_region(r) _profile_free_region(r,__FILE__,__FUNCTION__,__LINE__)
#define _region_malloc(rh,n) _profile_region_malloc(rh,n,__FILE__,__FUNCTION__,__LINE__)
#define _region_calloc(rh,n,t) _profile_region_calloc(rh,n,t,__FILE__,__FUNCTION__,__LINE__)
#  endif
#define _cycalloc(n) _profile_GC_malloc(n,__FILE__,__FUNCTION__,__LINE__)
#define _cycalloc_atomic(n) _profile_GC_malloc_atomic(n,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc(n,s) _profile_GC_calloc(n,s,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc_atomic(n,s) _profile_GC_calloc_atomic(n,s,__FILE__,__FUNCTION__,__LINE__)
#endif
#endif
 struct Cyc___cycFILE;
# 53 "cycboot.h"
extern struct Cyc___cycFILE*Cyc_stderr;struct Cyc_String_pa_PrintArg_struct{int tag;struct _fat_ptr f1;};struct Cyc_Int_pa_PrintArg_struct{int tag;unsigned long f1;};struct Cyc_Double_pa_PrintArg_struct{int tag;double f1;};struct Cyc_LongDouble_pa_PrintArg_struct{int tag;long double f1;};struct Cyc_ShortPtr_pa_PrintArg_struct{int tag;short*f1;};struct Cyc_IntPtr_pa_PrintArg_struct{int tag;unsigned long*f1;};
# 100
int Cyc_fprintf(struct Cyc___cycFILE*,struct _fat_ptr,struct _fat_ptr);struct Cyc_ShortPtr_sa_ScanfArg_struct{int tag;short*f1;};struct Cyc_UShortPtr_sa_ScanfArg_struct{int tag;unsigned short*f1;};struct Cyc_IntPtr_sa_ScanfArg_struct{int tag;int*f1;};struct Cyc_UIntPtr_sa_ScanfArg_struct{int tag;unsigned*f1;};struct Cyc_StringPtr_sa_ScanfArg_struct{int tag;struct _fat_ptr f1;};struct Cyc_DoublePtr_sa_ScanfArg_struct{int tag;double*f1;};struct Cyc_FloatPtr_sa_ScanfArg_struct{int tag;float*f1;};struct Cyc_CharPtr_sa_ScanfArg_struct{int tag;struct _fat_ptr f1;};
# 197 "cycboot.h"
int Cyc_sscanf(struct _fat_ptr,struct _fat_ptr,struct _fat_ptr);extern char Cyc_FileCloseError[15U];struct Cyc_FileCloseError_exn_struct{char*tag;};extern char Cyc_FileOpenError[14U];struct Cyc_FileOpenError_exn_struct{char*tag;struct _fat_ptr f1;};
# 300 "cycboot.h"
int isspace(int);struct Cyc_Core_Opt{void*v;};extern char Cyc_Core_Invalid_argument[17U];struct Cyc_Core_Invalid_argument_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Failure[8U];struct Cyc_Core_Failure_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Impossible[11U];struct Cyc_Core_Impossible_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Not_found[10U];struct Cyc_Core_Not_found_exn_struct{char*tag;};extern char Cyc_Core_Unreachable[12U];struct Cyc_Core_Unreachable_exn_struct{char*tag;struct _fat_ptr f1;};
# 168 "core.h"
extern struct _RegionHandle*Cyc_Core_unique_region;struct Cyc_Core_DynamicRegion;struct Cyc_Core_NewDynamicRegion{struct Cyc_Core_DynamicRegion*key;};struct Cyc_Core_ThinRes{void*arr;unsigned nelts;};struct Cyc_List_List{void*hd;struct Cyc_List_List*tl;};extern char Cyc_List_List_mismatch[14U];struct Cyc_List_List_mismatch_exn_struct{char*tag;};extern char Cyc_List_Nth[4U];struct Cyc_List_Nth_exn_struct{char*tag;};
# 38 "string.h"
unsigned long Cyc_strlen(struct _fat_ptr s);
# 49 "string.h"
int Cyc_strcmp(struct _fat_ptr s1,struct _fat_ptr s2);
# 51
int Cyc_strncmp(struct _fat_ptr s1,struct _fat_ptr s2,unsigned long len);
# 62
struct _fat_ptr Cyc_strconcat(struct _fat_ptr,struct _fat_ptr);extern char Cyc_Arg_Bad[4U];struct Cyc_Arg_Bad_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Arg_Error[6U];struct Cyc_Arg_Error_exn_struct{char*tag;};struct Cyc_Arg_Unit_spec_Arg_Spec_struct{int tag;void(*f1)();};struct Cyc_Arg_Flag_spec_Arg_Spec_struct{int tag;void(*f1)(struct _fat_ptr);};struct Cyc_Arg_FlagString_spec_Arg_Spec_struct{int tag;void(*f1)(struct _fat_ptr,struct _fat_ptr);};struct Cyc_Arg_Set_spec_Arg_Spec_struct{int tag;int*f1;};struct Cyc_Arg_Clear_spec_Arg_Spec_struct{int tag;int*f1;};struct Cyc_Arg_String_spec_Arg_Spec_struct{int tag;void(*f1)(struct _fat_ptr);};struct Cyc_Arg_Int_spec_Arg_Spec_struct{int tag;void(*f1)(int);};struct Cyc_Arg_Rest_spec_Arg_Spec_struct{int tag;void(*f1)(struct _fat_ptr);};
# 66 "arg.h"
void Cyc_Arg_usage(struct Cyc_List_List*,struct _fat_ptr);
# 69
extern int Cyc_Arg_current;struct Cyc_Buffer_t;
# 50 "buffer.h"
struct Cyc_Buffer_t*Cyc_Buffer_create(unsigned n);
# 58
struct _fat_ptr Cyc_Buffer_contents(struct Cyc_Buffer_t*);
# 83
void Cyc_Buffer_add_substring(struct Cyc_Buffer_t*,struct _fat_ptr,int offset,int len);
# 92 "buffer.h"
void Cyc_Buffer_add_string(struct Cyc_Buffer_t*,struct _fat_ptr);
# 29 "assert.h"
void*Cyc___assert_fail(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line);char Cyc_Arg_Bad[4U]="Bad";char Cyc_Arg_Error[6U]="Error";struct Cyc_Arg_Prefix_Arg_Flag_struct{int tag;struct _fat_ptr f1;};struct Cyc_Arg_Exact_Arg_Flag_struct{int tag;struct _fat_ptr f1;};struct Cyc_Arg_Unknown_Arg_error_struct{int tag;struct _fat_ptr f1;};struct Cyc_Arg_Missing_Arg_error_struct{int tag;struct _fat_ptr f1;};struct Cyc_Arg_Message_Arg_error_struct{int tag;struct _fat_ptr f1;};struct Cyc_Arg_Wrong_Arg_error_struct{int tag;struct _fat_ptr f1;struct _fat_ptr f2;struct _fat_ptr f3;};struct _tuple0{struct _fat_ptr f1;int f2;struct _fat_ptr f3;void*f4;struct _fat_ptr f5;};
# 68 "arg.cyc"
static void*Cyc_Arg_lookup(struct Cyc_List_List*l,struct _fat_ptr x){
while(l != 0){
struct _fat_ptr flag=(*((struct _tuple0*)l->hd)).f1;
unsigned long len=Cyc_strlen((struct _fat_ptr)flag);
if(len > (unsigned long)0 &&(*((struct _tuple0*)l->hd)).f2){
if(Cyc_strncmp((struct _fat_ptr)x,(struct _fat_ptr)(*((struct _tuple0*)l->hd)).f1,len)== 0)
return(*((struct _tuple0*)l->hd)).f4;}else{
# 76
if(Cyc_strcmp((struct _fat_ptr)x,(struct _fat_ptr)(*((struct _tuple0*)l->hd)).f1)== 0)
return(*((struct _tuple0*)l->hd)).f4;}
# 72
l=l->tl;}
# 80
(int)_throw((void*)({struct Cyc_Core_Not_found_exn_struct*_tmp0=_cycalloc(sizeof(*_tmp0));_tmp0->tag=Cyc_Core_Not_found;_tmp0;}));}
# 89
static struct _fat_ptr Cyc_Arg_Justify_break_line(struct Cyc_Buffer_t*b,int howmuch,struct _fat_ptr s){
if(({char*_tmp63=(char*)s.curr;_tmp63 == (char*)(_tag_fat(0,0,0)).curr;}))return _tag_fat(0,0,0);if(howmuch < 0)
howmuch=0;{
# 90
unsigned long len=
# 92
Cyc_strlen((struct _fat_ptr)s);
len <= _get_fat_size(s,sizeof(char))?0:({int(*_tmp65)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=({int(*_tmp1)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line)=(int(*)(struct _fat_ptr assertion,struct _fat_ptr file,unsigned line))Cyc___assert_fail;_tmp1;});struct _fat_ptr _tmp64=({const char*_tmp2="len <= numelts(s)";_tag_fat(_tmp2,sizeof(char),18U);});_tmp65(_tmp64,({const char*_tmp3="arg.cyc";_tag_fat(_tmp3,sizeof(char),8U);}),93U);});
if((unsigned long)howmuch > len){
Cyc_Buffer_add_string(b,s);
return _tag_fat(0,0,0);}{
# 94
int i;
# 101
for(i=howmuch - 1;i >= 0 && !isspace((int)*((const char*)_check_fat_subscript(s,sizeof(char),i)));-- i){
;}
# 105
if(i < 0)
for(i=howmuch?howmuch - 1: 0;((unsigned long)i < len &&(int)((const char*)s.curr)[i])&& !isspace((int)((const char*)s.curr)[i]);++ i){
;}
# 105
Cyc_Buffer_add_substring(b,s,0,i);{
# 114
struct _fat_ptr whatsleft=_tag_fat(0,0,0);
# 116
for(0;((unsigned long)i < len &&(int)((const char*)s.curr)[i])&& isspace((int)((const char*)s.curr)[i]);++ i){
;}
if((unsigned long)i < len &&(int)((const char*)s.curr)[i])whatsleft=_fat_ptr_plus(s,sizeof(char),i);return whatsleft;}}}}
# 89
void Cyc_Arg_Justify_justify_b(struct Cyc_Buffer_t*b,int indent,int margin,struct _fat_ptr item,struct _fat_ptr desc){
# 129
if(({char*_tmp66=(char*)item.curr;_tmp66 != (char*)(_tag_fat(0,0,0)).curr;}))Cyc_Buffer_add_string(b,item);if(({
char*_tmp67=(char*)desc.curr;_tmp67 == (char*)(_tag_fat(0,0,0)).curr;}))return;
# 129
if(indent < 0)
# 131
indent=0;
# 129
if(margin < 0)
# 132
margin=0;{
# 129
struct _fat_ptr indentstr=({unsigned _tmpE=(unsigned)(indent + 2)+ 1U;char*_tmpD=_cycalloc_atomic(_check_times(_tmpE,sizeof(char)));({{unsigned _tmp55=(unsigned)(indent + 2);unsigned i;for(i=0;i < _tmp55;++ i){_tmpD[i]='\000';}_tmpD[_tmp55]=0;}0;});_tag_fat(_tmpD,sizeof(char),_tmpE);});
# 136
{unsigned i=0U;for(0;i < (unsigned)(indent + 1);++ i){
({struct _fat_ptr _tmp4=_fat_ptr_plus(indentstr,sizeof(char),(int)i);char _tmp5=*((char*)_check_fat_subscript(_tmp4,sizeof(char),0U));char _tmp6=i == (unsigned)0?'\n':' ';if(_get_fat_size(_tmp4,sizeof(char))== 1U &&(_tmp5 == 0 && _tmp6 != 0))_throw_arraybounds();*((char*)_tmp4.curr)=_tmp6;});}}{
unsigned long itemlen=Cyc_strlen((struct _fat_ptr)item);
struct _fat_ptr itemsep;
if(Cyc_strlen((struct _fat_ptr)desc)> (unsigned long)0){
if(itemlen + (unsigned long)1 > (unsigned long)indent)
itemsep=indentstr;else{
# 144
struct _fat_ptr temp=({unsigned _tmpB=(((unsigned long)indent - itemlen)+ (unsigned long)1)+ 1U;char*_tmpA=_cycalloc_atomic(_check_times(_tmpB,sizeof(char)));({{unsigned _tmp54=((unsigned long)indent - itemlen)+ (unsigned long)1;unsigned i;for(i=0;i < _tmp54;++ i){_tmpA[i]='\000';}_tmpA[_tmp54]=0;}0;});_tag_fat(_tmpA,sizeof(char),_tmpB);});
{unsigned i=0U;for(0;i < (unsigned long)indent - itemlen;++ i){({struct _fat_ptr _tmp7=_fat_ptr_plus(temp,sizeof(char),(int)i);char _tmp8=*((char*)_check_fat_subscript(_tmp7,sizeof(char),0U));char _tmp9=' ';if(_get_fat_size(_tmp7,sizeof(char))== 1U &&(_tmp8 == 0 && _tmp9 != 0))_throw_arraybounds();*((char*)_tmp7.curr)=_tmp9;});}}
itemsep=temp;}}else{
# 149
return;}
# 151
Cyc_Buffer_add_string(b,(struct _fat_ptr)itemsep);
# 153
while(({char*_tmp68=(char*)desc.curr;_tmp68 != (char*)(_tag_fat(0,0,0)).curr;})){
desc=Cyc_Arg_Justify_break_line(b,margin - indent,desc);
if(({char*_tmp69=(char*)desc.curr;_tmp69 != (char*)(_tag_fat(0,0,0)).curr;}))
Cyc_Buffer_add_string(b,(struct _fat_ptr)indentstr);else{
# 158
({struct Cyc_Buffer_t*_tmp6A=b;Cyc_Buffer_add_string(_tmp6A,({const char*_tmpC="\n";_tag_fat(_tmpC,sizeof(char),2U);}));});}}
# 160
return;}}}
# 164
void Cyc_Arg_usage(struct Cyc_List_List*speclist,struct _fat_ptr errmsg){
# 166
({struct Cyc_String_pa_PrintArg_struct _tmp11=({struct Cyc_String_pa_PrintArg_struct _tmp56;_tmp56.tag=0U,_tmp56.f1=(struct _fat_ptr)((struct _fat_ptr)errmsg);_tmp56;});void*_tmpF[1U];_tmpF[0]=& _tmp11;({struct Cyc___cycFILE*_tmp6C=Cyc_stderr;struct _fat_ptr _tmp6B=({const char*_tmp10="%s\n";_tag_fat(_tmp10,sizeof(char),4U);});Cyc_fprintf(_tmp6C,_tmp6B,_tag_fat(_tmpF,sizeof(void*),1U));});});{
struct Cyc_Buffer_t*b=Cyc_Buffer_create(1024U);
while(speclist != 0){
({struct Cyc_Buffer_t*_tmp6E=b;struct _fat_ptr _tmp6D=(struct _fat_ptr)
Cyc_strconcat((struct _fat_ptr)(*((struct _tuple0*)speclist->hd)).f1,(struct _fat_ptr)(*((struct _tuple0*)speclist->hd)).f3);
# 169
Cyc_Arg_Justify_justify_b(_tmp6E,12,72,_tmp6D,(*((struct _tuple0*)speclist->hd)).f5);});
# 172
speclist=speclist->tl;}
# 174
({struct Cyc_String_pa_PrintArg_struct _tmp14=({struct Cyc_String_pa_PrintArg_struct _tmp57;_tmp57.tag=0U,({struct _fat_ptr _tmp6F=(struct _fat_ptr)((struct _fat_ptr)Cyc_Buffer_contents(b));_tmp57.f1=_tmp6F;});_tmp57;});void*_tmp12[1U];_tmp12[0]=& _tmp14;({struct Cyc___cycFILE*_tmp71=Cyc_stderr;struct _fat_ptr _tmp70=({const char*_tmp13="%s";_tag_fat(_tmp13,sizeof(char),3U);});Cyc_fprintf(_tmp71,_tmp70,_tag_fat(_tmp12,sizeof(void*),1U));});});}}
# 164
int Cyc_Arg_current=0;
# 178
static struct _fat_ptr Cyc_Arg_args={(void*)0,(void*)0,(void*)(0 + 0)};
# 180
static void Cyc_Arg_stop(int prog_pos,void*e,struct Cyc_List_List*speclist,struct _fat_ptr errmsg){
# 183
struct _fat_ptr progname=(unsigned)prog_pos < _get_fat_size(Cyc_Arg_args,sizeof(struct _fat_ptr))?*((struct _fat_ptr*)_check_fat_subscript(Cyc_Arg_args,sizeof(struct _fat_ptr),prog_pos)):({const char*_tmp2F="(?)";_tag_fat(_tmp2F,sizeof(char),4U);});
{void*_tmp15=e;struct _fat_ptr _tmp16;struct _fat_ptr _tmp19;struct _fat_ptr _tmp18;struct _fat_ptr _tmp17;struct _fat_ptr _tmp1A;struct _fat_ptr _tmp1B;switch(*((int*)_tmp15)){case 0U: _LL1: _tmp1B=((struct Cyc_Arg_Unknown_Arg_error_struct*)_tmp15)->f1;_LL2: {struct _fat_ptr s=_tmp1B;
# 186
if(({struct _fat_ptr _tmp72=(struct _fat_ptr)s;Cyc_strcmp(_tmp72,({const char*_tmp1C="-help";_tag_fat(_tmp1C,sizeof(char),6U);}));})!= 0)
({struct Cyc_String_pa_PrintArg_struct _tmp1F=({struct Cyc_String_pa_PrintArg_struct _tmp59;_tmp59.tag=0U,_tmp59.f1=(struct _fat_ptr)((struct _fat_ptr)progname);_tmp59;});struct Cyc_String_pa_PrintArg_struct _tmp20=({struct Cyc_String_pa_PrintArg_struct _tmp58;_tmp58.tag=0U,_tmp58.f1=(struct _fat_ptr)((struct _fat_ptr)s);_tmp58;});void*_tmp1D[2U];_tmp1D[0]=& _tmp1F,_tmp1D[1]=& _tmp20;({struct Cyc___cycFILE*_tmp74=Cyc_stderr;struct _fat_ptr _tmp73=({const char*_tmp1E="%s: unknown option `%s'.\n";_tag_fat(_tmp1E,sizeof(char),26U);});Cyc_fprintf(_tmp74,_tmp73,_tag_fat(_tmp1D,sizeof(void*),2U));});});
# 186
goto _LL0;}case 1U: _LL3: _tmp1A=((struct Cyc_Arg_Missing_Arg_error_struct*)_tmp15)->f1;_LL4: {struct _fat_ptr s=_tmp1A;
# 190
({struct Cyc_String_pa_PrintArg_struct _tmp23=({struct Cyc_String_pa_PrintArg_struct _tmp5B;_tmp5B.tag=0U,_tmp5B.f1=(struct _fat_ptr)((struct _fat_ptr)progname);_tmp5B;});struct Cyc_String_pa_PrintArg_struct _tmp24=({struct Cyc_String_pa_PrintArg_struct _tmp5A;_tmp5A.tag=0U,_tmp5A.f1=(struct _fat_ptr)((struct _fat_ptr)s);_tmp5A;});void*_tmp21[2U];_tmp21[0]=& _tmp23,_tmp21[1]=& _tmp24;({struct Cyc___cycFILE*_tmp76=Cyc_stderr;struct _fat_ptr _tmp75=({const char*_tmp22="%s: option `%s' needs an argument.\n";_tag_fat(_tmp22,sizeof(char),36U);});Cyc_fprintf(_tmp76,_tmp75,_tag_fat(_tmp21,sizeof(void*),2U));});});
goto _LL0;}case 3U: _LL5: _tmp17=((struct Cyc_Arg_Wrong_Arg_error_struct*)_tmp15)->f1;_tmp18=((struct Cyc_Arg_Wrong_Arg_error_struct*)_tmp15)->f2;_tmp19=((struct Cyc_Arg_Wrong_Arg_error_struct*)_tmp15)->f3;_LL6: {struct _fat_ptr flag=_tmp17;struct _fat_ptr cmd=_tmp18;struct _fat_ptr t=_tmp19;
# 193
({struct Cyc_String_pa_PrintArg_struct _tmp27=({struct Cyc_String_pa_PrintArg_struct _tmp5F;_tmp5F.tag=0U,_tmp5F.f1=(struct _fat_ptr)((struct _fat_ptr)progname);_tmp5F;});struct Cyc_String_pa_PrintArg_struct _tmp28=({struct Cyc_String_pa_PrintArg_struct _tmp5E;_tmp5E.tag=0U,_tmp5E.f1=(struct _fat_ptr)((struct _fat_ptr)cmd);_tmp5E;});struct Cyc_String_pa_PrintArg_struct _tmp29=({struct Cyc_String_pa_PrintArg_struct _tmp5D;_tmp5D.tag=0U,_tmp5D.f1=(struct _fat_ptr)((struct _fat_ptr)flag);_tmp5D;});struct Cyc_String_pa_PrintArg_struct _tmp2A=({struct Cyc_String_pa_PrintArg_struct _tmp5C;_tmp5C.tag=0U,_tmp5C.f1=(struct _fat_ptr)((struct _fat_ptr)t);_tmp5C;});void*_tmp25[4U];_tmp25[0]=& _tmp27,_tmp25[1]=& _tmp28,_tmp25[2]=& _tmp29,_tmp25[3]=& _tmp2A;({struct Cyc___cycFILE*_tmp78=Cyc_stderr;struct _fat_ptr _tmp77=({const char*_tmp26="%s: wrong argument `%s'; option `%s' expects %s.\n";_tag_fat(_tmp26,sizeof(char),50U);});Cyc_fprintf(_tmp78,_tmp77,_tag_fat(_tmp25,sizeof(void*),4U));});});
# 195
goto _LL0;}default: _LL7: _tmp16=((struct Cyc_Arg_Message_Arg_error_struct*)_tmp15)->f1;_LL8: {struct _fat_ptr s=_tmp16;
# 197
({struct Cyc_String_pa_PrintArg_struct _tmp2D=({struct Cyc_String_pa_PrintArg_struct _tmp61;_tmp61.tag=0U,_tmp61.f1=(struct _fat_ptr)((struct _fat_ptr)progname);_tmp61;});struct Cyc_String_pa_PrintArg_struct _tmp2E=({struct Cyc_String_pa_PrintArg_struct _tmp60;_tmp60.tag=0U,_tmp60.f1=(struct _fat_ptr)((struct _fat_ptr)s);_tmp60;});void*_tmp2B[2U];_tmp2B[0]=& _tmp2D,_tmp2B[1]=& _tmp2E;({struct Cyc___cycFILE*_tmp7A=Cyc_stderr;struct _fat_ptr _tmp79=({const char*_tmp2C="%s: %s.\n";_tag_fat(_tmp2C,sizeof(char),9U);});Cyc_fprintf(_tmp7A,_tmp79,_tag_fat(_tmp2B,sizeof(void*),2U));});});
goto _LL0;}}_LL0:;}
# 200
Cyc_Arg_usage(speclist,errmsg);
Cyc_Arg_current=(int)_get_fat_size(Cyc_Arg_args,sizeof(struct _fat_ptr));}
# 180
void Cyc_Arg_parse(struct Cyc_List_List*speclist,void(*anonfun)(struct _fat_ptr),int(*anonflagfun)(struct _fat_ptr),struct _fat_ptr errmsg,struct _fat_ptr orig_args){
# 209
Cyc_Arg_args=orig_args;{
# 211
int initpos=Cyc_Arg_current;
unsigned l=_get_fat_size(Cyc_Arg_args,sizeof(struct _fat_ptr));
# 214
if(({char*_tmp7B=(char*)((struct _fat_ptr*)_check_fat_subscript(Cyc_Arg_args,sizeof(struct _fat_ptr),(int)(l - (unsigned)1)))->curr;_tmp7B == (char*)(_tag_fat(0,0,0)).curr;}))l=l - (unsigned)1;++ Cyc_Arg_current;
# 216
while((unsigned)Cyc_Arg_current < l){
struct _fat_ptr s=*((struct _fat_ptr*)_check_fat_subscript(Cyc_Arg_args,sizeof(struct _fat_ptr),Cyc_Arg_current));
if((({char*_tmp7C=(char*)s.curr;_tmp7C != (char*)(_tag_fat(0,0,0)).curr;})&& _get_fat_size(s,sizeof(char))>= (unsigned)1)&&(int)((const char*)s.curr)[0]== (int)'-'){
void*action;
{struct _handler_cons _tmp30;_push_handler(& _tmp30);{int _tmp32=0;if(setjmp(_tmp30.handler))_tmp32=1;if(!_tmp32){action=Cyc_Arg_lookup(speclist,s);;_pop_handler();}else{void*_tmp31=(void*)Cyc_Core_get_exn_thrown();void*_tmp33=_tmp31;void*_tmp34;if(((struct Cyc_Core_Not_found_exn_struct*)_tmp33)->tag == Cyc_Core_Not_found){_LL1: _LL2:
# 224
 if(!anonflagfun(s)){
({int _tmp7F=initpos;void*_tmp7E=(void*)({struct Cyc_Arg_Unknown_Arg_error_struct*_tmp35=_cycalloc(sizeof(*_tmp35));_tmp35->tag=0U,_tmp35->f1=s;_tmp35;});struct Cyc_List_List*_tmp7D=speclist;Cyc_Arg_stop(_tmp7F,_tmp7E,_tmp7D,errmsg);});
return;}else{
# 229
++ Cyc_Arg_current;
continue;}}else{_LL3: _tmp34=_tmp33;_LL4: {void*exn=_tmp34;(int)_rethrow(exn);}}_LL0:;}}}
# 233
{struct _handler_cons _tmp36;_push_handler(& _tmp36);{int _tmp38=0;if(setjmp(_tmp36.handler))_tmp38=1;if(!_tmp38){
{void*_tmp39=action;void(*_tmp3A)(struct _fat_ptr);void(*_tmp3B)(int);void(*_tmp3C)(struct _fat_ptr);void(*_tmp3D)(struct _fat_ptr,struct _fat_ptr);int*_tmp3E;int*_tmp3F;void(*_tmp40)(struct _fat_ptr);void(*_tmp41)();switch(*((int*)_tmp39)){case 0U: _LL6: _tmp41=((struct Cyc_Arg_Unit_spec_Arg_Spec_struct*)_tmp39)->f1;_LL7: {void(*f)()=_tmp41;
f();goto _LL5;}case 1U: _LL8: _tmp40=((struct Cyc_Arg_Flag_spec_Arg_Spec_struct*)_tmp39)->f1;_LL9: {void(*f)(struct _fat_ptr)=_tmp40;
f(s);goto _LL5;}case 3U: _LLA: _tmp3F=((struct Cyc_Arg_Set_spec_Arg_Spec_struct*)_tmp39)->f1;_LLB: {int*r=_tmp3F;
*r=1;goto _LL5;}case 4U: _LLC: _tmp3E=((struct Cyc_Arg_Clear_spec_Arg_Spec_struct*)_tmp39)->f1;_LLD: {int*r=_tmp3E;
*r=0;goto _LL5;}case 2U: _LLE: _tmp3D=((struct Cyc_Arg_FlagString_spec_Arg_Spec_struct*)_tmp39)->f1;_LLF: {void(*f)(struct _fat_ptr,struct _fat_ptr)=_tmp3D;
# 240
if((unsigned)(Cyc_Arg_current + 1)< l){
f(s,*((struct _fat_ptr*)_check_fat_subscript(Cyc_Arg_args,sizeof(struct _fat_ptr),Cyc_Arg_current + 1)));
++ Cyc_Arg_current;}else{
# 244
({int _tmp82=initpos;void*_tmp81=(void*)({struct Cyc_Arg_Missing_Arg_error_struct*_tmp42=_cycalloc(sizeof(*_tmp42));_tmp42->tag=1U,_tmp42->f1=s;_tmp42;});struct Cyc_List_List*_tmp80=speclist;Cyc_Arg_stop(_tmp82,_tmp81,_tmp80,errmsg);});}
goto _LL5;}case 5U: _LL10: _tmp3C=((struct Cyc_Arg_String_spec_Arg_Spec_struct*)_tmp39)->f1;_LL11: {void(*f)(struct _fat_ptr)=_tmp3C;
# 247
if((unsigned)(Cyc_Arg_current + 1)< l){
f(*((struct _fat_ptr*)_check_fat_subscript(Cyc_Arg_args,sizeof(struct _fat_ptr),Cyc_Arg_current + 1)));
++ Cyc_Arg_current;}else{
# 251
({int _tmp85=initpos;void*_tmp84=(void*)({struct Cyc_Arg_Missing_Arg_error_struct*_tmp43=_cycalloc(sizeof(*_tmp43));_tmp43->tag=1U,_tmp43->f1=s;_tmp43;});struct Cyc_List_List*_tmp83=speclist;Cyc_Arg_stop(_tmp85,_tmp84,_tmp83,errmsg);});}
goto _LL5;}case 6U: _LL12: _tmp3B=((struct Cyc_Arg_Int_spec_Arg_Spec_struct*)_tmp39)->f1;_LL13: {void(*f)(int)=_tmp3B;
# 254
struct _fat_ptr arg=*((struct _fat_ptr*)_check_fat_subscript(Cyc_Arg_args,sizeof(struct _fat_ptr),Cyc_Arg_current + 1));
int n=0;
if(({struct Cyc_IntPtr_sa_ScanfArg_struct _tmp46=({struct Cyc_IntPtr_sa_ScanfArg_struct _tmp62;_tmp62.tag=2U,_tmp62.f1=& n;_tmp62;});void*_tmp44[1U];_tmp44[0]=& _tmp46;({struct _fat_ptr _tmp87=arg;struct _fat_ptr _tmp86=({const char*_tmp45="%d";_tag_fat(_tmp45,sizeof(char),3U);});Cyc_sscanf(_tmp87,_tmp86,_tag_fat(_tmp44,sizeof(void*),1U));});})!= 1){
({int _tmp8B=initpos;void*_tmp8A=(void*)({struct Cyc_Arg_Wrong_Arg_error_struct*_tmp48=_cycalloc(sizeof(*_tmp48));_tmp48->tag=3U,_tmp48->f1=s,_tmp48->f2=arg,({struct _fat_ptr _tmp88=({const char*_tmp47="an integer";_tag_fat(_tmp47,sizeof(char),11U);});_tmp48->f3=_tmp88;});_tmp48;});struct Cyc_List_List*_tmp89=speclist;Cyc_Arg_stop(_tmp8B,_tmp8A,_tmp89,errmsg);});
_npop_handler(0U);return;}
# 256
f(n);
# 261
++ Cyc_Arg_current;
goto _LL5;}default: _LL14: _tmp3A=((struct Cyc_Arg_Rest_spec_Arg_Spec_struct*)_tmp39)->f1;_LL15: {void(*f)(struct _fat_ptr)=_tmp3A;
# 264
while((unsigned)Cyc_Arg_current < l - (unsigned)1){
f(*((struct _fat_ptr*)_check_fat_subscript(Cyc_Arg_args,sizeof(struct _fat_ptr),Cyc_Arg_current + 1)));
++ Cyc_Arg_current;}
# 268
goto _LL5;}}_LL5:;}
# 234
;_pop_handler();}else{void*_tmp37=(void*)Cyc_Core_get_exn_thrown();void*_tmp49=_tmp37;void*_tmp4A;struct _fat_ptr _tmp4B;if(((struct Cyc_Arg_Bad_exn_struct*)_tmp49)->tag == Cyc_Arg_Bad){_LL17: _tmp4B=((struct Cyc_Arg_Bad_exn_struct*)_tmp49)->f1;_LL18: {struct _fat_ptr s2=_tmp4B;
# 271
({int _tmp8E=initpos;void*_tmp8D=(void*)({struct Cyc_Arg_Message_Arg_error_struct*_tmp4C=_cycalloc(sizeof(*_tmp4C));_tmp4C->tag=2U,_tmp4C->f1=s2;_tmp4C;});struct Cyc_List_List*_tmp8C=speclist;Cyc_Arg_stop(_tmp8E,_tmp8D,_tmp8C,errmsg);});goto _LL16;}}else{_LL19: _tmp4A=_tmp49;_LL1A: {void*exn=_tmp4A;(int)_rethrow(exn);}}_LL16:;}}}
# 273
++ Cyc_Arg_current;}else{
# 276
{struct _handler_cons _tmp4D;_push_handler(& _tmp4D);{int _tmp4F=0;if(setjmp(_tmp4D.handler))_tmp4F=1;if(!_tmp4F){anonfun(s);;_pop_handler();}else{void*_tmp4E=(void*)Cyc_Core_get_exn_thrown();void*_tmp50=_tmp4E;void*_tmp51;struct _fat_ptr _tmp52;if(((struct Cyc_Arg_Bad_exn_struct*)_tmp50)->tag == Cyc_Arg_Bad){_LL1C: _tmp52=((struct Cyc_Arg_Bad_exn_struct*)_tmp50)->f1;_LL1D: {struct _fat_ptr s2=_tmp52;
# 278
({int _tmp91=initpos;void*_tmp90=(void*)({struct Cyc_Arg_Message_Arg_error_struct*_tmp53=_cycalloc(sizeof(*_tmp53));_tmp53->tag=2U,_tmp53->f1=s2;_tmp53;});struct Cyc_List_List*_tmp8F=speclist;Cyc_Arg_stop(_tmp91,_tmp90,_tmp8F,errmsg);});goto _LL1B;}}else{_LL1E: _tmp51=_tmp50;_LL1F: {void*exn=_tmp51;(int)_rethrow(exn);}}_LL1B:;}}}
# 280
++ Cyc_Arg_current;}}}}
