#include <setjmp.h>
/* This is a C header file to be used by the output of the Cyclone to
   C translator.  The corresponding definitions are in file lib/runtime_*.c */
#ifndef _CYC_INCLUDE_H_
#define _CYC_INCLUDE_H_

/* Need one of these per thread (see runtime_stack.c). The runtime maintains 
   a stack that contains either _handler_cons structs or _RegionHandle structs.
   The tag is 0 for a handler_cons and 1 for a region handle.  */
struct _RuntimeStack {
  int tag; 
  struct _RuntimeStack *next;
  void (*cleanup)(struct _RuntimeStack *frame);
};

#ifndef offsetof
/* should be size_t, but int is fine. */
#define offsetof(t,n) ((int)(&(((t *)0)->n)))
#endif

/* Fat pointers */
struct _fat_ptr {
  unsigned char *curr; 
  unsigned char *base; 
  unsigned char *last_plus_one; 
};  

/* Discriminated Unions */
struct _xtunion_struct { char *tag; };

/* Regions */
struct _RegionPage
#ifdef CYC_REGION_PROFILE
{ unsigned total_bytes;
  unsigned free_bytes;
  /* MWH: wish we didn't have to include the stuff below ... */
  struct _RegionPage *next;
  char data[1];
}
#endif
; // abstract -- defined in runtime_memory.c
struct _RegionHandle {
  struct _RuntimeStack s;
  struct _RegionPage *curr;
  char               *offset;
  char               *last_plus_one;
  struct _DynRegionHandle *sub_regions;
#ifdef CYC_REGION_PROFILE
  const char         *name;
#else
  unsigned used_bytes;
  unsigned wasted_bytes;
#endif
};
struct _DynRegionFrame {
  struct _RuntimeStack s;
  struct _DynRegionHandle *x;
};
// A dynamic region is just a region handle.  The wrapper struct is for type
// abstraction.
struct Cyc_Core_DynamicRegion {
  struct _RegionHandle h;
};

struct _RegionHandle _new_region(const char*);
void* _region_malloc(struct _RegionHandle*, unsigned);
void* _region_calloc(struct _RegionHandle*, unsigned t, unsigned n);
void   _free_region(struct _RegionHandle*);
struct _RegionHandle*_open_dynregion(struct _DynRegionFrame*,struct _DynRegionHandle*);
void   _pop_dynregion();

/* Exceptions */
struct _handler_cons {
  struct _RuntimeStack s;
  jmp_buf handler;
};
void _push_handler(struct _handler_cons *);
void _push_region(struct _RegionHandle *);
void _npop_handler(int);
void _pop_handler();
void _pop_region();

#ifndef _throw
void* _throw_null_fn(const char*,unsigned);
void* _throw_arraybounds_fn(const char*,unsigned);
void* _throw_badalloc_fn(const char*,unsigned);
void* _throw_match_fn(const char*,unsigned);
void* _throw_fn(void*,const char*,unsigned);
void* _rethrow(void*);
#define _throw_null() (_throw_null_fn(__FILE__,__LINE__))
#define _throw_arraybounds() (_throw_arraybounds_fn(__FILE__,__LINE__))
#define _throw_badalloc() (_throw_badalloc_fn(__FILE__,__LINE__))
#define _throw_match() (_throw_match_fn(__FILE__,__LINE__))
#define _throw(e) (_throw_fn((e),__FILE__,__LINE__))
#endif

struct _xtunion_struct* Cyc_Core_get_exn_thrown();
/* Built-in Exceptions */
struct Cyc_Null_Exception_exn_struct { char *tag; };
struct Cyc_Array_bounds_exn_struct { char *tag; };
struct Cyc_Match_Exception_exn_struct { char *tag; };
struct Cyc_Bad_alloc_exn_struct { char *tag; };
extern char Cyc_Null_Exception[];
extern char Cyc_Array_bounds[];
extern char Cyc_Match_Exception[];
extern char Cyc_Bad_alloc[];

/* Built-in Run-time Checks and company */
#ifdef CYC_ANSI_OUTPUT
#define _INLINE  
#else
#define _INLINE inline
#endif

#ifdef NO_CYC_NULL_CHECKS
#define _check_null(ptr) (ptr)
#else
#define _check_null(ptr) \
  ({ void*_cks_null = (void*)(ptr); \
     if (!_cks_null) _throw_null(); \
     _cks_null; })
#endif

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index)\
   (((char*)ptr) + (elt_sz)*(index))
#ifdef NO_CYC_NULL_CHECKS
#define _check_known_subscript_null _check_known_subscript_notnull
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr);\
  int _index = (index);\
  if (!_cks_ptr) _throw_null(); \
  _cks_ptr + (elt_sz)*_index; })
#endif
#define _zero_arr_plus_fn(orig_x,orig_sz,orig_i,f,l) ((orig_x)+(orig_i))
#define _zero_arr_plus_char_fn _zero_arr_plus_fn
#define _zero_arr_plus_short_fn _zero_arr_plus_fn
#define _zero_arr_plus_int_fn _zero_arr_plus_fn
#define _zero_arr_plus_float_fn _zero_arr_plus_fn
#define _zero_arr_plus_double_fn _zero_arr_plus_fn
#define _zero_arr_plus_longdouble_fn _zero_arr_plus_fn
#define _zero_arr_plus_voidstar_fn _zero_arr_plus_fn
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (!_cks_ptr) _throw_null(); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })

/* _zero_arr_plus_*_fn(x,sz,i,filename,lineno) adds i to zero-terminated ptr
   x that has at least sz elements */
char* _zero_arr_plus_char_fn(char*,unsigned,int,const char*,unsigned);
short* _zero_arr_plus_short_fn(short*,unsigned,int,const char*,unsigned);
int* _zero_arr_plus_int_fn(int*,unsigned,int,const char*,unsigned);
float* _zero_arr_plus_float_fn(float*,unsigned,int,const char*,unsigned);
double* _zero_arr_plus_double_fn(double*,unsigned,int,const char*,unsigned);
long double* _zero_arr_plus_longdouble_fn(long double*,unsigned,int,const char*, unsigned);
void** _zero_arr_plus_voidstar_fn(void**,unsigned,int,const char*,unsigned);
#endif

/* _get_zero_arr_size_*(x,sz) returns the number of elements in a
   zero-terminated array that is NULL or has at least sz elements */
int _get_zero_arr_size_char(const char*,unsigned);
int _get_zero_arr_size_short(const short*,unsigned);
int _get_zero_arr_size_int(const int*,unsigned);
int _get_zero_arr_size_float(const float*,unsigned);
int _get_zero_arr_size_double(const double*,unsigned);
int _get_zero_arr_size_longdouble(const long double*,unsigned);
int _get_zero_arr_size_voidstar(const void**,unsigned);

/* _zero_arr_inplace_plus_*_fn(x,i,filename,lineno) sets
   zero-terminated pointer *x to *x + i */
char* _zero_arr_inplace_plus_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_short_fn(short**,int,const char*,unsigned);
int* _zero_arr_inplace_plus_int(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_longdouble_fn(long double**,int,const char*,unsigned);
void** _zero_arr_inplace_plus_voidstar_fn(void***,int,const char*,unsigned);
/* like the previous functions, but does post-addition (as in e++) */
char* _zero_arr_inplace_plus_post_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_post_short_fn(short**x,int,const char*,unsigned);
int* _zero_arr_inplace_plus_post_int_fn(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_post_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_post_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_post_longdouble_fn(long double**,int,const char *,unsigned);
void** _zero_arr_inplace_plus_post_voidstar_fn(void***,int,const char*,unsigned);
#define _zero_arr_plus_char(x,s,i) \
  (_zero_arr_plus_char_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_short(x,s,i) \
  (_zero_arr_plus_short_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_int(x,s,i) \
  (_zero_arr_plus_int_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_float(x,s,i) \
  (_zero_arr_plus_float_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_double(x,s,i) \
  (_zero_arr_plus_double_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_longdouble(x,s,i) \
  (_zero_arr_plus_longdouble_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_voidstar(x,s,i) \
  (_zero_arr_plus_voidstar_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_inplace_plus_char(x,i) \
  _zero_arr_inplace_plus_char_fn((char **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_short(x,i) \
  _zero_arr_inplace_plus_short_fn((short **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_int(x,i) \
  _zero_arr_inplace_plus_int_fn((int **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_float(x,i) \
  _zero_arr_inplace_plus_float_fn((float **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_double(x,i) \
  _zero_arr_inplace_plus_double_fn((double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_longdouble(x,i) \
  _zero_arr_inplace_plus_longdouble_fn((long double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_voidstar(x,i) \
  _zero_arr_inplace_plus_voidstar_fn((void ***)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_char(x,i) \
  _zero_arr_inplace_plus_post_char_fn((char **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_short(x,i) \
  _zero_arr_inplace_plus_post_short_fn((short **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_int(x,i) \
  _zero_arr_inplace_plus_post_int_fn((int **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_float(x,i) \
  _zero_arr_inplace_plus_post_float_fn((float **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_double(x,i) \
  _zero_arr_inplace_plus_post_double_fn((double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_longdouble(x,i) \
  _zero_arr_inplace_plus_post_longdouble_fn((long double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_voidstar(x,i) \
  _zero_arr_inplace_plus_post_voidstar_fn((void***)(x),(i),__FILE__,__LINE__)

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_fat_subscript(arr,elt_sz,index) ((arr).curr + (elt_sz) * (index))
#define _untag_fat_ptr(arr,elt_sz,num_elts) ((arr).curr)
#else
#define _check_fat_subscript(arr,elt_sz,index) ({ \
  struct _fat_ptr _cus_arr = (arr); \
  unsigned char *_cus_ans = _cus_arr.curr + (elt_sz) * (index); \
  /* JGM: not needed! if (!_cus_arr.base) _throw_null();*/ \
  if (_cus_ans < _cus_arr.base || _cus_ans >= _cus_arr.last_plus_one) \
    _throw_arraybounds(); \
  _cus_ans; })
#define _untag_fat_ptr(arr,elt_sz,num_elts) ({ \
  struct _fat_ptr _arr = (arr); \
  unsigned char *_curr = _arr.curr; \
  if ((_curr < _arr.base || _curr + (elt_sz) * (num_elts) > _arr.last_plus_one) &&\
      _curr != (unsigned char *)0) \
    _throw_arraybounds(); \
  _curr; })
#endif

#define _tag_fat(tcurr,elt_sz,num_elts) ({ \
  struct _fat_ptr _ans; \
  unsigned _num_elts = (num_elts);\
  _ans.base = _ans.curr = (void*)(tcurr); \
  /* JGM: if we're tagging NULL, ignore num_elts */ \
  _ans.last_plus_one = _ans.base ? (_ans.base + (elt_sz) * _num_elts) : 0; \
  _ans; })

#define _get_fat_size(arr,elt_sz) \
  ({struct _fat_ptr _arr = (arr); \
    unsigned char *_arr_curr=_arr.curr; \
    unsigned char *_arr_last=_arr.last_plus_one; \
    (_arr_curr < _arr.base || _arr_curr >= _arr_last) ? 0 : \
    ((_arr_last - _arr_curr) / (elt_sz));})

#define _fat_ptr_plus(arr,elt_sz,change) ({ \
  struct _fat_ptr _ans = (arr); \
  int _change = (change);\
  _ans.curr += (elt_sz) * _change;\
  _ans; })
#define _fat_ptr_inplace_plus(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  _arr_ptr->curr += (elt_sz) * (change);\
  *_arr_ptr; })
#define _fat_ptr_inplace_plus_post(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  struct _fat_ptr _ans = *_arr_ptr; \
  _arr_ptr->curr += (elt_sz) * (change);\
  _ans; })

//Not a macro since initialization order matters. Defined in runtime_zeroterm.c.
struct _fat_ptr _fat_ptr_decrease_size(struct _fat_ptr,unsigned sz,unsigned numelts);

/* Allocation */
void* GC_malloc(int);
void* GC_malloc_atomic(int);
void* GC_calloc(unsigned,unsigned);
void* GC_calloc_atomic(unsigned,unsigned);
// bound the allocation size to be < MAX_ALLOC_SIZE. See macros below for usage.
#define MAX_MALLOC_SIZE (1 << 28)
void* _bounded_GC_malloc(int,const char*,int);
void* _bounded_GC_malloc_atomic(int,const char*,int);
void* _bounded_GC_calloc(unsigned,unsigned,const char*,int);
void* _bounded_GC_calloc_atomic(unsigned,unsigned,const char*,int);
/* these macros are overridden below ifdef CYC_REGION_PROFILE */
#ifndef CYC_REGION_PROFILE
#define _cycalloc(n) _bounded_GC_malloc(n,__FILE__,__LINE__)
#define _cycalloc_atomic(n) _bounded_GC_malloc_atomic(n,__FILE__,__LINE__)
#define _cyccalloc(n,s) _bounded_GC_calloc(n,s,__FILE__,__LINE__)
#define _cyccalloc_atomic(n,s) _bounded_GC_calloc_atomic(n,s,__FILE__,__LINE__)
#endif

static _INLINE unsigned int _check_times(unsigned x, unsigned y) {
  unsigned long long whole_ans = 
    ((unsigned long long) x)*((unsigned long long)y);
  unsigned word_ans = (unsigned)whole_ans;
  if(word_ans < whole_ans || word_ans > MAX_MALLOC_SIZE)
    _throw_badalloc();
  return word_ans;
}

#define _CYC_MAX_REGION_CONST 2
#define _CYC_MIN_ALIGNMENT (sizeof(double))

#ifdef CYC_REGION_PROFILE
extern int rgn_total_bytes;
#endif

static _INLINE void *_fast_region_malloc(struct _RegionHandle *r, unsigned orig_s) {  
  if (r > (struct _RegionHandle *)_CYC_MAX_REGION_CONST && r->curr != 0) { 
#ifdef CYC_NOALIGN
    unsigned s =  orig_s;
#else
    unsigned s =  (orig_s + _CYC_MIN_ALIGNMENT - 1) & (~(_CYC_MIN_ALIGNMENT -1)); 
#endif
    char *result; 
    result = r->offset; 
    if (s <= (r->last_plus_one - result)) {
      r->offset = result + s; 
#ifdef CYC_REGION_PROFILE
    r->curr->free_bytes = r->curr->free_bytes - s;
    rgn_total_bytes += s;
#endif
      return result;
    }
  } 
  return _region_malloc(r,orig_s); 
}

#ifdef CYC_REGION_PROFILE
/* see macros below for usage. defined in runtime_memory.c */
void* _profile_GC_malloc(int,const char*,const char*,int);
void* _profile_GC_malloc_atomic(int,const char*,const char*,int);
void* _profile_GC_calloc(unsigned,unsigned,const char*,const char*,int);
void* _profile_GC_calloc_atomic(unsigned,unsigned,const char*,const char*,int);
void* _profile_region_malloc(struct _RegionHandle*,unsigned,const char*,const char*,int);
void* _profile_region_calloc(struct _RegionHandle*,unsigned,unsigned,const char *,const char*,int);
struct _RegionHandle _profile_new_region(const char*,const char*,const char*,int);
void _profile_free_region(struct _RegionHandle*,const char*,const char*,int);
#ifndef RUNTIME_CYC
#define _new_region(n) _profile_new_region(n,__FILE__,__FUNCTION__,__LINE__)
#define _free_region(r) _profile_free_region(r,__FILE__,__FUNCTION__,__LINE__)
#define _region_malloc(rh,n) _profile_region_malloc(rh,n,__FILE__,__FUNCTION__,__LINE__)
#define _region_calloc(rh,n,t) _profile_region_calloc(rh,n,t,__FILE__,__FUNCTION__,__LINE__)
#  endif
#define _cycalloc(n) _profile_GC_malloc(n,__FILE__,__FUNCTION__,__LINE__)
#define _cycalloc_atomic(n) _profile_GC_malloc_atomic(n,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc(n,s) _profile_GC_calloc(n,s,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc_atomic(n,s) _profile_GC_calloc_atomic(n,s,__FILE__,__FUNCTION__,__LINE__)
#endif
#endif
 struct Cyc_Core_Opt{void*v;};extern char Cyc_Core_Invalid_argument[17U];struct Cyc_Core_Invalid_argument_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Failure[8U];struct Cyc_Core_Failure_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Impossible[11U];struct Cyc_Core_Impossible_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Not_found[10U];struct Cyc_Core_Not_found_exn_struct{char*tag;};extern char Cyc_Core_Unreachable[12U];struct Cyc_Core_Unreachable_exn_struct{char*tag;struct _fat_ptr f1;};
# 165 "core.h"
extern struct _RegionHandle*Cyc_Core_heap_region;
# 168
extern struct _RegionHandle*Cyc_Core_unique_region;struct Cyc_Core_DynamicRegion;struct Cyc_Core_NewDynamicRegion{struct Cyc_Core_DynamicRegion*key;};struct Cyc_Core_ThinRes{void*arr;unsigned nelts;};struct Cyc_List_List{void*hd;struct Cyc_List_List*tl;};
# 61 "list.h"
int Cyc_List_length(struct Cyc_List_List*x);extern char Cyc_List_List_mismatch[14U];struct Cyc_List_List_mismatch_exn_struct{char*tag;};extern char Cyc_List_Nth[4U];struct Cyc_List_Nth_exn_struct{char*tag;};struct Cyc___cycFILE;struct Cyc_String_pa_PrintArg_struct{int tag;struct _fat_ptr f1;};struct Cyc_Int_pa_PrintArg_struct{int tag;unsigned long f1;};struct Cyc_Double_pa_PrintArg_struct{int tag;double f1;};struct Cyc_LongDouble_pa_PrintArg_struct{int tag;long double f1;};struct Cyc_ShortPtr_pa_PrintArg_struct{int tag;short*f1;};struct Cyc_IntPtr_pa_PrintArg_struct{int tag;unsigned long*f1;};struct Cyc_ShortPtr_sa_ScanfArg_struct{int tag;short*f1;};struct Cyc_UShortPtr_sa_ScanfArg_struct{int tag;unsigned short*f1;};struct Cyc_IntPtr_sa_ScanfArg_struct{int tag;int*f1;};struct Cyc_UIntPtr_sa_ScanfArg_struct{int tag;unsigned*f1;};struct Cyc_StringPtr_sa_ScanfArg_struct{int tag;struct _fat_ptr f1;};struct Cyc_DoublePtr_sa_ScanfArg_struct{int tag;double*f1;};struct Cyc_FloatPtr_sa_ScanfArg_struct{int tag;float*f1;};struct Cyc_CharPtr_sa_ScanfArg_struct{int tag;struct _fat_ptr f1;};
# 252 "cycboot.h"
int Cyc_getw(struct Cyc___cycFILE*);extern char Cyc_FileCloseError[15U];struct Cyc_FileCloseError_exn_struct{char*tag;};extern char Cyc_FileOpenError[14U];struct Cyc_FileOpenError_exn_struct{char*tag;struct _fat_ptr f1;};struct Cyc_Iter_Iter{void*env;int(*next)(void*env,void*dest);};struct Cyc_Dict_T;struct Cyc_Dict_Dict{int(*rel)(void*,void*);struct _RegionHandle*r;const struct Cyc_Dict_T*t;};extern char Cyc_Dict_Present[8U];struct Cyc_Dict_Present_exn_struct{char*tag;};extern char Cyc_Dict_Absent[7U];struct Cyc_Dict_Absent_exn_struct{char*tag;};
# 62 "dict.h"
struct Cyc_Dict_Dict Cyc_Dict_empty(int(*cmp)(void*,void*));
# 68
struct Cyc_Dict_Dict Cyc_Dict_rempty(struct _RegionHandle*,int(*cmp)(void*,void*));
# 83
int Cyc_Dict_member(struct Cyc_Dict_Dict d,void*k);
# 87
struct Cyc_Dict_Dict Cyc_Dict_insert(struct Cyc_Dict_Dict d,void*k,void*v);
# 104
struct Cyc_Dict_Dict Cyc_Dict_rsingleton(struct _RegionHandle*,int(*cmp)(void*,void*),void*k,void*v);
# 110
void*Cyc_Dict_lookup(struct Cyc_Dict_Dict d,void*k);
# 135 "dict.h"
void*Cyc_Dict_fold_c(void*(*f)(void*,void*,void*,void*),void*env,struct Cyc_Dict_Dict d,void*accum);
# 149
void Cyc_Dict_iter_c(void(*f)(void*,void*,void*),void*env,struct Cyc_Dict_Dict d);
# 170
struct Cyc_Dict_Dict Cyc_Dict_rcopy(struct _RegionHandle*,struct Cyc_Dict_Dict);
# 181
struct Cyc_Dict_Dict Cyc_Dict_rmap(struct _RegionHandle*,void*(*f)(void*),struct Cyc_Dict_Dict d);
# 187
struct Cyc_Dict_Dict Cyc_Dict_rmap_c(struct _RegionHandle*,void*(*f)(void*,void*),void*env,struct Cyc_Dict_Dict d);
# 212
struct Cyc_Dict_Dict Cyc_Dict_intersect_c(void*(*f)(void*,void*,void*,void*),void*env,struct Cyc_Dict_Dict d1,struct Cyc_Dict_Dict d2);
# 218
int Cyc_Dict_forall_c(int(*f)(void*,void*,void*),void*env,struct Cyc_Dict_Dict d);
# 236
struct Cyc_List_List*Cyc_Dict_rto_list(struct _RegionHandle*,struct Cyc_Dict_Dict d);
# 244
struct Cyc_Dict_Dict Cyc_Dict_rfilter(struct _RegionHandle*,int(*f)(void*,void*),struct Cyc_Dict_Dict d);
# 250
struct Cyc_Dict_Dict Cyc_Dict_rfilter_c(struct _RegionHandle*,int(*f)(void*,void*,void*),void*env,struct Cyc_Dict_Dict d);
# 263
struct Cyc_Dict_Dict Cyc_Dict_rdifference(struct _RegionHandle*,struct Cyc_Dict_Dict d1,struct Cyc_Dict_Dict d2);
# 274
struct Cyc_Dict_Dict Cyc_Dict_rdelete(struct _RegionHandle*,struct Cyc_Dict_Dict,void*);char Cyc_Dict_Absent[7U]="Absent";char Cyc_Dict_Present[8U]="Present";
# 27 "dict.cyc"
struct Cyc_Dict_Absent_exn_struct Cyc_Dict_Absent_val={Cyc_Dict_Absent};
struct Cyc_Dict_Present_exn_struct Cyc_Dict_Present_val={Cyc_Dict_Present};
# 30
enum Cyc_Dict_Color{Cyc_Dict_R =0U,Cyc_Dict_B =1U};struct _tuple0{void*f1;void*f2;};struct Cyc_Dict_T{enum Cyc_Dict_Color color;const struct Cyc_Dict_T*left;const struct Cyc_Dict_T*right;struct _tuple0 key_val;};
# 40
struct Cyc_Dict_Dict Cyc_Dict_rempty(struct _RegionHandle*r,int(*comp)(void*,void*)){
return({struct Cyc_Dict_Dict _tmp99;_tmp99.rel=comp,_tmp99.r=r,_tmp99.t=0;_tmp99;});}
# 40
struct Cyc_Dict_Dict Cyc_Dict_empty(int(*comp)(void*,void*)){
# 44
return Cyc_Dict_rempty(Cyc_Core_heap_region,comp);}
# 40
struct Cyc_Dict_Dict Cyc_Dict_rsingleton(struct _RegionHandle*r,int(*comp)(void*,void*),void*key,void*data){
# 48
return({struct Cyc_Dict_Dict _tmp9A;_tmp9A.rel=comp,_tmp9A.r=r,({const struct Cyc_Dict_T*_tmpB4=(const struct Cyc_Dict_T*)({struct Cyc_Dict_T*_tmp0=_region_malloc(r,sizeof(*_tmp0));_tmp0->color=Cyc_Dict_B,_tmp0->left=0,_tmp0->right=0,(_tmp0->key_val).f1=key,(_tmp0->key_val).f2=data;_tmp0;});_tmp9A.t=_tmpB4;});_tmp9A;});}
# 40
struct Cyc_Dict_Dict Cyc_Dict_singleton(int(*comp)(void*,void*),void*key,void*data){
# 51
return Cyc_Dict_rsingleton(Cyc_Core_heap_region,comp,key,data);}
# 40
struct Cyc_Dict_Dict Cyc_Dict_rshare(struct _RegionHandle*r,struct Cyc_Dict_Dict d){
# 55
const struct Cyc_Dict_T*t=d.t;
const struct Cyc_Dict_T*t2=t;
return({struct Cyc_Dict_Dict _tmp9B;_tmp9B.rel=d.rel,_tmp9B.r=r,_tmp9B.t=t2;_tmp9B;});}
# 40
int Cyc_Dict_is_empty(struct Cyc_Dict_Dict d){
# 61
return d.t == (const struct Cyc_Dict_T*)0;}
# 40
int Cyc_Dict_member(struct Cyc_Dict_Dict d,void*key){
# 65
int(*rel)(void*,void*)=d.rel;
const struct Cyc_Dict_T*t=d.t;
while(t != (const struct Cyc_Dict_T*)0){
int i=rel(key,(t->key_val).f1);
if(i < 0)t=t->left;else{
if(i > 0)t=t->right;else{
return 1;}}}
# 73
return 0;}
# 40
void*Cyc_Dict_lookup(struct Cyc_Dict_Dict d,void*key){
# 77
int(*rel)(void*,void*)=d.rel;
const struct Cyc_Dict_T*t=d.t;
while(t != (const struct Cyc_Dict_T*)0){
int i=rel(key,(t->key_val).f1);
if(i < 0)t=t->left;else{
if(i > 0)t=t->right;else{
return(t->key_val).f2;}}}
# 85
(int)_throw((void*)& Cyc_Dict_Absent_val);}
# 40
void*Cyc_Dict_lookup_other(struct Cyc_Dict_Dict d,int(*cmp)(void*,void*),void*key){
# 89
const struct Cyc_Dict_T*t=d.t;
while(t != (const struct Cyc_Dict_T*)0){
int i=cmp(key,(t->key_val).f1);
if(i < 0)t=t->left;else{
if(i > 0)t=t->right;else{
return(t->key_val).f2;}}}
# 96
(int)_throw((void*)& Cyc_Dict_Absent_val);}
# 40
void**Cyc_Dict_lookup_opt(struct Cyc_Dict_Dict d,void*key){
# 100
int(*rel)(void*,void*)=d.rel;
const struct Cyc_Dict_T*t=d.t;
while(t != (const struct Cyc_Dict_T*)0){
int i=rel(key,(t->key_val).f1);
if(i < 0)t=t->left;else{
if(i > 0)t=t->right;else{
return&(t->key_val).f2;}}}
# 108
return 0;}
# 40
int Cyc_Dict_lookup_bool(struct Cyc_Dict_Dict d,void*key,void**ans_place){
# 112
int(*rel)(void*,void*)=d.rel;
const struct Cyc_Dict_T*t=d.t;
while(t != (const struct Cyc_Dict_T*)0){
int i=rel(key,(t->key_val).f1);
if(i < 0)t=t->left;else{
if(i > 0)t=t->right;else{
# 119
*ans_place=(t->key_val).f2;
return 1;}}}
# 123
return 0;}struct _tuple1{enum Cyc_Dict_Color f1;const struct Cyc_Dict_T*f2;const struct Cyc_Dict_T*f3;struct _tuple0 f4;};
# 40
static struct Cyc_Dict_T*Cyc_Dict_balance(struct _RegionHandle*r,struct _tuple1 quad){
# 129
struct _tuple1 _tmp1=quad;struct _tuple0 _tmp5;const struct Cyc_Dict_T*_tmp4;const struct Cyc_Dict_T*_tmp3;enum Cyc_Dict_Color _tmp2;struct _tuple0 _tmpC;struct _tuple0 _tmpB;struct _tuple0 _tmpA;const struct Cyc_Dict_T*_tmp9;const struct Cyc_Dict_T*_tmp8;const struct Cyc_Dict_T*_tmp7;const struct Cyc_Dict_T*_tmp6;struct _tuple0 _tmp13;struct _tuple0 _tmp12;const struct Cyc_Dict_T*_tmp11;struct _tuple0 _tmp10;const struct Cyc_Dict_T*_tmpF;const struct Cyc_Dict_T*_tmpE;const struct Cyc_Dict_T*_tmpD;struct _tuple0 _tmp1A;const struct Cyc_Dict_T*_tmp19;struct _tuple0 _tmp18;struct _tuple0 _tmp17;const struct Cyc_Dict_T*_tmp16;const struct Cyc_Dict_T*_tmp15;const struct Cyc_Dict_T*_tmp14;struct _tuple0 _tmp21;const struct Cyc_Dict_T*_tmp20;struct _tuple0 _tmp1F;const struct Cyc_Dict_T*_tmp1E;struct _tuple0 _tmp1D;const struct Cyc_Dict_T*_tmp1C;const struct Cyc_Dict_T*_tmp1B;if(_tmp1.f1 == Cyc_Dict_B){if(_tmp1.f2 != 0){if(((const struct Cyc_Dict_T*)_tmp1.f2)->color == Cyc_Dict_R){if(((const struct Cyc_Dict_T*)_tmp1.f2)->left != 0){if(((const struct Cyc_Dict_T*)((const struct Cyc_Dict_T*)_tmp1.f2)->left)->color == Cyc_Dict_R){_LL1: _tmp1B=((_tmp1.f2)->left)->left;_tmp1C=((_tmp1.f2)->left)->right;_tmp1D=((_tmp1.f2)->left)->key_val;_tmp1E=(_tmp1.f2)->right;_tmp1F=(_tmp1.f2)->key_val;_tmp20=_tmp1.f3;_tmp21=_tmp1.f4;_LL2: {const struct Cyc_Dict_T*a=_tmp1B;const struct Cyc_Dict_T*b=_tmp1C;struct _tuple0 x=_tmp1D;const struct Cyc_Dict_T*c=_tmp1E;struct _tuple0 y=_tmp1F;const struct Cyc_Dict_T*d=_tmp20;struct _tuple0 z=_tmp21;
# 131
return({struct Cyc_Dict_T*_tmp24=_region_malloc(r,sizeof(*_tmp24));_tmp24->color=Cyc_Dict_R,({const struct Cyc_Dict_T*_tmpB6=(const struct Cyc_Dict_T*)({struct Cyc_Dict_T*_tmp22=_region_malloc(r,sizeof(*_tmp22));_tmp22->color=Cyc_Dict_B,_tmp22->left=a,_tmp22->right=b,_tmp22->key_val=x;_tmp22;});_tmp24->left=_tmpB6;}),({const struct Cyc_Dict_T*_tmpB5=(const struct Cyc_Dict_T*)({struct Cyc_Dict_T*_tmp23=_region_malloc(r,sizeof(*_tmp23));_tmp23->color=Cyc_Dict_B,_tmp23->left=c,_tmp23->right=d,_tmp23->key_val=z;_tmp23;});_tmp24->right=_tmpB5;}),_tmp24->key_val=y;_tmp24;});}}else{if(((const struct Cyc_Dict_T*)_tmp1.f2)->right != 0){if(((const struct Cyc_Dict_T*)((const struct Cyc_Dict_T*)_tmp1.f2)->right)->color == Cyc_Dict_R)goto _LL3;else{if(_tmp1.f3 != 0){if(((const struct Cyc_Dict_T*)_tmp1.f3)->color == Cyc_Dict_R){if(((const struct Cyc_Dict_T*)_tmp1.f3)->left != 0){if(((const struct Cyc_Dict_T*)((const struct Cyc_Dict_T*)_tmp1.f3)->left)->color == Cyc_Dict_R)goto _LL5;else{if(((const struct Cyc_Dict_T*)_tmp1.f3)->right != 0){if(((const struct Cyc_Dict_T*)((const struct Cyc_Dict_T*)_tmp1.f3)->right)->color == Cyc_Dict_R)goto _LL7;else{goto _LL9;}}else{goto _LL9;}}}else{if(((const struct Cyc_Dict_T*)_tmp1.f3)->right != 0){if(((const struct Cyc_Dict_T*)((const struct Cyc_Dict_T*)_tmp1.f3)->right)->color == Cyc_Dict_R)goto _LL7;else{goto _LL9;}}else{goto _LL9;}}}else{goto _LL9;}}else{goto _LL9;}}}else{if(_tmp1.f3 != 0){if(((const struct Cyc_Dict_T*)_tmp1.f3)->color == Cyc_Dict_R){if(((const struct Cyc_Dict_T*)_tmp1.f3)->left != 0){if(((const struct Cyc_Dict_T*)((const struct Cyc_Dict_T*)_tmp1.f3)->left)->color == Cyc_Dict_R)goto _LL5;else{if(((const struct Cyc_Dict_T*)_tmp1.f3)->right != 0){if(((const struct Cyc_Dict_T*)((const struct Cyc_Dict_T*)_tmp1.f3)->right)->color == Cyc_Dict_R)goto _LL7;else{goto _LL9;}}else{goto _LL9;}}}else{if(((const struct Cyc_Dict_T*)_tmp1.f3)->right != 0){if(((const struct Cyc_Dict_T*)((const struct Cyc_Dict_T*)_tmp1.f3)->right)->color == Cyc_Dict_R)goto _LL7;else{goto _LL9;}}else{goto _LL9;}}}else{goto _LL9;}}else{goto _LL9;}}}}else{if(((const struct Cyc_Dict_T*)_tmp1.f2)->right != 0){if(((const struct Cyc_Dict_T*)((const struct Cyc_Dict_T*)_tmp1.f2)->right)->color == Cyc_Dict_R){_LL3: _tmp14=(_tmp1.f2)->left;_tmp15=((_tmp1.f2)->right)->left;_tmp16=((_tmp1.f2)->right)->right;_tmp17=((_tmp1.f2)->right)->key_val;_tmp18=(_tmp1.f2)->key_val;_tmp19=_tmp1.f3;_tmp1A=_tmp1.f4;_LL4: {const struct Cyc_Dict_T*a=_tmp14;const struct Cyc_Dict_T*b=_tmp15;const struct Cyc_Dict_T*c=_tmp16;struct _tuple0 y=_tmp17;struct _tuple0 x=_tmp18;const struct Cyc_Dict_T*d=_tmp19;struct _tuple0 z=_tmp1A;
# 133
return({struct Cyc_Dict_T*_tmp27=_region_malloc(r,sizeof(*_tmp27));_tmp27->color=Cyc_Dict_R,({const struct Cyc_Dict_T*_tmpB8=(const struct Cyc_Dict_T*)({struct Cyc_Dict_T*_tmp25=_region_malloc(r,sizeof(*_tmp25));_tmp25->color=Cyc_Dict_B,_tmp25->left=a,_tmp25->right=b,_tmp25->key_val=x;_tmp25;});_tmp27->left=_tmpB8;}),({const struct Cyc_Dict_T*_tmpB7=(const struct Cyc_Dict_T*)({struct Cyc_Dict_T*_tmp26=_region_malloc(r,sizeof(*_tmp26));_tmp26->color=Cyc_Dict_B,_tmp26->left=c,_tmp26->right=d,_tmp26->key_val=z;_tmp26;});_tmp27->right=_tmpB7;}),_tmp27->key_val=y;_tmp27;});}}else{if(_tmp1.f3 != 0){if(((const struct Cyc_Dict_T*)_tmp1.f3)->color == Cyc_Dict_R){if(((const struct Cyc_Dict_T*)_tmp1.f3)->left != 0){if(((const struct Cyc_Dict_T*)((const struct Cyc_Dict_T*)_tmp1.f3)->left)->color == Cyc_Dict_R)goto _LL5;else{if(((const struct Cyc_Dict_T*)_tmp1.f3)->right != 0){if(((const struct Cyc_Dict_T*)((const struct Cyc_Dict_T*)_tmp1.f3)->right)->color == Cyc_Dict_R)goto _LL7;else{goto _LL9;}}else{goto _LL9;}}}else{if(((const struct Cyc_Dict_T*)_tmp1.f3)->right != 0){if(((const struct Cyc_Dict_T*)((const struct Cyc_Dict_T*)_tmp1.f3)->right)->color == Cyc_Dict_R)goto _LL7;else{goto _LL9;}}else{goto _LL9;}}}else{goto _LL9;}}else{goto _LL9;}}}else{if(_tmp1.f3 != 0){if(((const struct Cyc_Dict_T*)_tmp1.f3)->color == Cyc_Dict_R){if(((const struct Cyc_Dict_T*)_tmp1.f3)->left != 0){if(((const struct Cyc_Dict_T*)((const struct Cyc_Dict_T*)_tmp1.f3)->left)->color == Cyc_Dict_R)goto _LL5;else{if(((const struct Cyc_Dict_T*)_tmp1.f3)->right != 0){if(((const struct Cyc_Dict_T*)((const struct Cyc_Dict_T*)_tmp1.f3)->right)->color == Cyc_Dict_R)goto _LL7;else{goto _LL9;}}else{goto _LL9;}}}else{if(((const struct Cyc_Dict_T*)_tmp1.f3)->right != 0){if(((const struct Cyc_Dict_T*)((const struct Cyc_Dict_T*)_tmp1.f3)->right)->color == Cyc_Dict_R)goto _LL7;else{goto _LL9;}}else{goto _LL9;}}}else{goto _LL9;}}else{goto _LL9;}}}}else{if(_tmp1.f3 != 0){if(((const struct Cyc_Dict_T*)_tmp1.f3)->color == Cyc_Dict_R){if(((const struct Cyc_Dict_T*)_tmp1.f3)->left != 0){if(((const struct Cyc_Dict_T*)((const struct Cyc_Dict_T*)_tmp1.f3)->left)->color == Cyc_Dict_R)goto _LL5;else{if(((const struct Cyc_Dict_T*)_tmp1.f3)->right != 0){if(((const struct Cyc_Dict_T*)((const struct Cyc_Dict_T*)_tmp1.f3)->right)->color == Cyc_Dict_R)goto _LL7;else{goto _LL9;}}else{goto _LL9;}}}else{if(((const struct Cyc_Dict_T*)_tmp1.f3)->right != 0){if(((const struct Cyc_Dict_T*)((const struct Cyc_Dict_T*)_tmp1.f3)->right)->color == Cyc_Dict_R)goto _LL7;else{goto _LL9;}}else{goto _LL9;}}}else{goto _LL9;}}else{goto _LL9;}}}else{if(_tmp1.f3 != 0){if(((const struct Cyc_Dict_T*)_tmp1.f3)->color == Cyc_Dict_R){if(((const struct Cyc_Dict_T*)_tmp1.f3)->left != 0){if(((const struct Cyc_Dict_T*)((const struct Cyc_Dict_T*)_tmp1.f3)->left)->color == Cyc_Dict_R){_LL5: _tmpD=_tmp1.f2;_tmpE=((_tmp1.f3)->left)->left;_tmpF=((_tmp1.f3)->left)->right;_tmp10=((_tmp1.f3)->left)->key_val;_tmp11=(_tmp1.f3)->right;_tmp12=(_tmp1.f3)->key_val;_tmp13=_tmp1.f4;_LL6: {const struct Cyc_Dict_T*a=_tmpD;const struct Cyc_Dict_T*b=_tmpE;const struct Cyc_Dict_T*c=_tmpF;struct _tuple0 y=_tmp10;const struct Cyc_Dict_T*d=_tmp11;struct _tuple0 z=_tmp12;struct _tuple0 x=_tmp13;
# 135
return({struct Cyc_Dict_T*_tmp2A=_region_malloc(r,sizeof(*_tmp2A));_tmp2A->color=Cyc_Dict_R,({const struct Cyc_Dict_T*_tmpBA=(const struct Cyc_Dict_T*)({struct Cyc_Dict_T*_tmp28=_region_malloc(r,sizeof(*_tmp28));_tmp28->color=Cyc_Dict_B,_tmp28->left=a,_tmp28->right=b,_tmp28->key_val=x;_tmp28;});_tmp2A->left=_tmpBA;}),({const struct Cyc_Dict_T*_tmpB9=(const struct Cyc_Dict_T*)({struct Cyc_Dict_T*_tmp29=_region_malloc(r,sizeof(*_tmp29));_tmp29->color=Cyc_Dict_B,_tmp29->left=c,_tmp29->right=d,_tmp29->key_val=z;_tmp29;});_tmp2A->right=_tmpB9;}),_tmp2A->key_val=y;_tmp2A;});}}else{if(((const struct Cyc_Dict_T*)_tmp1.f3)->right != 0){if(((const struct Cyc_Dict_T*)((const struct Cyc_Dict_T*)_tmp1.f3)->right)->color == Cyc_Dict_R)goto _LL7;else{goto _LL9;}}else{goto _LL9;}}}else{if(((const struct Cyc_Dict_T*)_tmp1.f3)->right != 0){if(((const struct Cyc_Dict_T*)((const struct Cyc_Dict_T*)_tmp1.f3)->right)->color == Cyc_Dict_R){_LL7: _tmp6=_tmp1.f2;_tmp7=(_tmp1.f3)->left;_tmp8=((_tmp1.f3)->right)->left;_tmp9=((_tmp1.f3)->right)->right;_tmpA=((_tmp1.f3)->right)->key_val;_tmpB=(_tmp1.f3)->key_val;_tmpC=_tmp1.f4;_LL8: {const struct Cyc_Dict_T*a=_tmp6;const struct Cyc_Dict_T*b=_tmp7;const struct Cyc_Dict_T*c=_tmp8;const struct Cyc_Dict_T*d=_tmp9;struct _tuple0 z=_tmpA;struct _tuple0 y=_tmpB;struct _tuple0 x=_tmpC;
# 137
return({struct Cyc_Dict_T*_tmp2D=_region_malloc(r,sizeof(*_tmp2D));_tmp2D->color=Cyc_Dict_R,({const struct Cyc_Dict_T*_tmpBC=(const struct Cyc_Dict_T*)({struct Cyc_Dict_T*_tmp2B=_region_malloc(r,sizeof(*_tmp2B));_tmp2B->color=Cyc_Dict_B,_tmp2B->left=a,_tmp2B->right=b,_tmp2B->key_val=x;_tmp2B;});_tmp2D->left=_tmpBC;}),({const struct Cyc_Dict_T*_tmpBB=(const struct Cyc_Dict_T*)({struct Cyc_Dict_T*_tmp2C=_region_malloc(r,sizeof(*_tmp2C));_tmp2C->color=Cyc_Dict_B,_tmp2C->left=c,_tmp2C->right=d,_tmp2C->key_val=z;_tmp2C;});_tmp2D->right=_tmpBB;}),_tmp2D->key_val=y;_tmp2D;});}}else{goto _LL9;}}else{goto _LL9;}}}else{goto _LL9;}}else{goto _LL9;}}}else{_LL9: _tmp2=_tmp1.f1;_tmp3=_tmp1.f2;_tmp4=_tmp1.f3;_tmp5=_tmp1.f4;_LLA: {enum Cyc_Dict_Color a=_tmp2;const struct Cyc_Dict_T*b=_tmp3;const struct Cyc_Dict_T*c=_tmp4;struct _tuple0 d=_tmp5;
# 139
return({struct Cyc_Dict_T*_tmp2E=_region_malloc(r,sizeof(*_tmp2E));_tmp2E->color=a,_tmp2E->left=b,_tmp2E->right=c,_tmp2E->key_val=d;_tmp2E;});}}_LL0:;}
# 40
static struct Cyc_Dict_T*Cyc_Dict_ins(struct _RegionHandle*r,int(*rel)(void*,void*),struct _tuple0 key_val,const struct Cyc_Dict_T*t){
# 145
const struct Cyc_Dict_T*_tmp2F=t;struct _tuple0 _tmp33;const struct Cyc_Dict_T*_tmp32;const struct Cyc_Dict_T*_tmp31;enum Cyc_Dict_Color _tmp30;if(_tmp2F == 0){_LL1: _LL2:
 return({struct Cyc_Dict_T*_tmp34=_region_malloc(r,sizeof(*_tmp34));_tmp34->color=Cyc_Dict_R,_tmp34->left=0,_tmp34->right=0,_tmp34->key_val=key_val;_tmp34;});}else{_LL3: _tmp30=_tmp2F->color;_tmp31=_tmp2F->left;_tmp32=_tmp2F->right;_tmp33=_tmp2F->key_val;_LL4: {enum Cyc_Dict_Color color=_tmp30;const struct Cyc_Dict_T*a=_tmp31;const struct Cyc_Dict_T*b=_tmp32;struct _tuple0 y=_tmp33;
# 148
int i=rel(key_val.f1,y.f1);
if(i < 0)return({struct _RegionHandle*_tmpBE=r;Cyc_Dict_balance(_tmpBE,({struct _tuple1 _tmp9C;_tmp9C.f1=color,({const struct Cyc_Dict_T*_tmpBD=(const struct Cyc_Dict_T*)Cyc_Dict_ins(r,rel,key_val,a);_tmp9C.f2=_tmpBD;}),_tmp9C.f3=b,_tmp9C.f4=y;_tmp9C;}));});else{
if(i > 0)return({struct _RegionHandle*_tmpC0=r;Cyc_Dict_balance(_tmpC0,({struct _tuple1 _tmp9D;_tmp9D.f1=color,_tmp9D.f2=a,({const struct Cyc_Dict_T*_tmpBF=(const struct Cyc_Dict_T*)Cyc_Dict_ins(r,rel,key_val,b);_tmp9D.f3=_tmpBF;}),_tmp9D.f4=y;_tmp9D;}));});else{
return({struct Cyc_Dict_T*_tmp35=_region_malloc(r,sizeof(*_tmp35));_tmp35->color=color,_tmp35->left=a,_tmp35->right=b,_tmp35->key_val=key_val;_tmp35;});}}}}_LL0:;}
# 40
struct Cyc_Dict_Dict Cyc_Dict_insert(struct Cyc_Dict_Dict d,void*key,void*data){
# 156
struct Cyc_Dict_T*ans=({struct _RegionHandle*_tmpC3=d.r;int(*_tmpC2)(void*,void*)=d.rel;struct _tuple0 _tmpC1=({struct _tuple0 _tmp9F;_tmp9F.f1=key,_tmp9F.f2=data;_tmp9F;});Cyc_Dict_ins(_tmpC3,_tmpC2,_tmpC1,d.t);});
((struct Cyc_Dict_T*)_check_null(ans))->color=Cyc_Dict_B;
return({struct Cyc_Dict_Dict _tmp9E;_tmp9E.rel=d.rel,_tmp9E.r=d.r,_tmp9E.t=(const struct Cyc_Dict_T*)ans;_tmp9E;});}
# 40
struct Cyc_Dict_Dict Cyc_Dict_insert_new(struct Cyc_Dict_Dict d,void*key,void*data){
# 163
if(Cyc_Dict_member(d,key))
(int)_throw((void*)& Cyc_Dict_Absent_val);
# 163
return
# 165
 Cyc_Dict_insert(d,key,data);}
# 40
struct Cyc_Dict_Dict Cyc_Dict_inserts(struct Cyc_Dict_Dict d,struct Cyc_List_List*kds){
# 170
for(0;kds != 0;kds=kds->tl){
d=Cyc_Dict_insert(d,(((struct _tuple0*)kds->hd)[0]).f1,(((struct _tuple0*)kds->hd)[0]).f2);}
return d;}
# 40
static void*Cyc_Dict_fold_tree(void*(*f)(void*,void*,void*),const struct Cyc_Dict_T*t,void*accum){
# 176
void*_tmp39;void*_tmp38;const struct Cyc_Dict_T*_tmp37;const struct Cyc_Dict_T*_tmp36;_LL1: _tmp36=t->left;_tmp37=t->right;_tmp38=(t->key_val).f1;_tmp39=(t->key_val).f2;_LL2: {const struct Cyc_Dict_T*left=_tmp36;const struct Cyc_Dict_T*right=_tmp37;void*key=_tmp38;void*val=_tmp39;
if(left != (const struct Cyc_Dict_T*)0)accum=Cyc_Dict_fold_tree(f,left,accum);accum=
f(key,val,accum);
if(right != (const struct Cyc_Dict_T*)0)accum=Cyc_Dict_fold_tree(f,right,accum);return accum;}}
# 40
void*Cyc_Dict_fold(void*(*f)(void*,void*,void*),struct Cyc_Dict_Dict d,void*accum){
# 184
const struct Cyc_Dict_T*t=d.t;
if(t == (const struct Cyc_Dict_T*)0)
return accum;
# 185
return
# 187
 Cyc_Dict_fold_tree(f,t,accum);}
# 40
static void*Cyc_Dict_fold_tree_c(void*(*f)(void*,void*,void*,void*),void*env,const struct Cyc_Dict_T*t,void*accum){
# 192
void*_tmp3D;void*_tmp3C;const struct Cyc_Dict_T*_tmp3B;const struct Cyc_Dict_T*_tmp3A;_LL1: _tmp3A=t->left;_tmp3B=t->right;_tmp3C=(t->key_val).f1;_tmp3D=(t->key_val).f2;_LL2: {const struct Cyc_Dict_T*left=_tmp3A;const struct Cyc_Dict_T*right=_tmp3B;void*key=_tmp3C;void*val=_tmp3D;
if(left != (const struct Cyc_Dict_T*)0)
accum=Cyc_Dict_fold_tree_c(f,env,left,accum);
# 193
accum=
# 195
f(env,key,val,accum);
if(right != (const struct Cyc_Dict_T*)0)
accum=Cyc_Dict_fold_tree_c(f,env,right,accum);
# 196
return accum;}}
# 40
void*Cyc_Dict_fold_c(void*(*f)(void*,void*,void*,void*),void*env,struct Cyc_Dict_Dict d,void*accum){
# 202
const struct Cyc_Dict_T*t=d.t;
if(t == (const struct Cyc_Dict_T*)0)
return accum;
# 203
return
# 205
 Cyc_Dict_fold_tree_c(f,env,t,accum);}
# 40
static void Cyc_Dict_app_tree(void*(*f)(void*,void*),const struct Cyc_Dict_T*t){
# 209
void*_tmp41;void*_tmp40;const struct Cyc_Dict_T*_tmp3F;const struct Cyc_Dict_T*_tmp3E;_LL1: _tmp3E=t->left;_tmp3F=t->right;_tmp40=(t->key_val).f1;_tmp41=(t->key_val).f2;_LL2: {const struct Cyc_Dict_T*left=_tmp3E;const struct Cyc_Dict_T*right=_tmp3F;void*key=_tmp40;void*val=_tmp41;
if(left != (const struct Cyc_Dict_T*)0)Cyc_Dict_app_tree(f,left);f(key,val);
# 212
if(right != (const struct Cyc_Dict_T*)0)Cyc_Dict_app_tree(f,right);}}
# 40
void Cyc_Dict_app(void*(*f)(void*,void*),struct Cyc_Dict_Dict d){
# 216
const struct Cyc_Dict_T*t=d.t;
if(t != (const struct Cyc_Dict_T*)0)
Cyc_Dict_app_tree(f,t);}
# 40
static void Cyc_Dict_app_tree_c(void*(*f)(void*,void*,void*),void*env,const struct Cyc_Dict_T*t){
# 222
void*_tmp45;void*_tmp44;const struct Cyc_Dict_T*_tmp43;const struct Cyc_Dict_T*_tmp42;_LL1: _tmp42=t->left;_tmp43=t->right;_tmp44=(t->key_val).f1;_tmp45=(t->key_val).f2;_LL2: {const struct Cyc_Dict_T*left=_tmp42;const struct Cyc_Dict_T*right=_tmp43;void*key=_tmp44;void*val=_tmp45;
if(left != (const struct Cyc_Dict_T*)0)Cyc_Dict_app_tree_c(f,env,left);f(env,key,val);
# 225
if(right != (const struct Cyc_Dict_T*)0)Cyc_Dict_app_tree_c(f,env,right);}}
# 40
void Cyc_Dict_app_c(void*(*f)(void*,void*,void*),void*env,struct Cyc_Dict_Dict d){
# 229
const struct Cyc_Dict_T*t=d.t;
if(t != (const struct Cyc_Dict_T*)0)
Cyc_Dict_app_tree_c(f,env,t);}
# 40
static void Cyc_Dict_iter_tree(void(*f)(void*,void*),const struct Cyc_Dict_T*t){
# 235
void*_tmp49;void*_tmp48;const struct Cyc_Dict_T*_tmp47;const struct Cyc_Dict_T*_tmp46;_LL1: _tmp46=t->left;_tmp47=t->right;_tmp48=(t->key_val).f1;_tmp49=(t->key_val).f2;_LL2: {const struct Cyc_Dict_T*left=_tmp46;const struct Cyc_Dict_T*right=_tmp47;void*key=_tmp48;void*val=_tmp49;
if(left != (const struct Cyc_Dict_T*)0)Cyc_Dict_iter_tree(f,left);f(key,val);
# 238
if(right != (const struct Cyc_Dict_T*)0)Cyc_Dict_iter_tree(f,right);}}
# 40
void Cyc_Dict_iter(void(*f)(void*,void*),struct Cyc_Dict_Dict d){
# 242
const struct Cyc_Dict_T*t=d.t;
if(t != (const struct Cyc_Dict_T*)0)
Cyc_Dict_iter_tree(f,t);}
# 40
static void Cyc_Dict_iter_tree_c(void(*f)(void*,void*,void*),void*env,const struct Cyc_Dict_T*t){
# 248
void*_tmp4D;void*_tmp4C;const struct Cyc_Dict_T*_tmp4B;const struct Cyc_Dict_T*_tmp4A;_LL1: _tmp4A=t->left;_tmp4B=t->right;_tmp4C=(t->key_val).f1;_tmp4D=(t->key_val).f2;_LL2: {const struct Cyc_Dict_T*left=_tmp4A;const struct Cyc_Dict_T*right=_tmp4B;void*key=_tmp4C;void*val=_tmp4D;
if(left != (const struct Cyc_Dict_T*)0)Cyc_Dict_iter_tree_c(f,env,left);f(env,key,val);
# 251
if(right != (const struct Cyc_Dict_T*)0)Cyc_Dict_iter_tree_c(f,env,right);}}
# 40
void Cyc_Dict_iter_c(void(*f)(void*,void*,void*),void*env,struct Cyc_Dict_Dict d){
# 255
const struct Cyc_Dict_T*t=d.t;
if(t != (const struct Cyc_Dict_T*)0)
Cyc_Dict_iter_tree_c(f,env,t);}
# 40
static void Cyc_Dict_count_elem(int*cnt,void*a,void*b){
# 261
*cnt=*cnt + 1;}
# 40
int Cyc_Dict_cardinality(struct Cyc_Dict_Dict d){
# 264
int num=0;
({void(*_tmpC4)(void(*f)(int*,void*,void*),int*env,struct Cyc_Dict_Dict d)=({void(*_tmp4E)(void(*f)(int*,void*,void*),int*env,struct Cyc_Dict_Dict d)=(void(*)(void(*f)(int*,void*,void*),int*env,struct Cyc_Dict_Dict d))Cyc_Dict_iter_c;_tmp4E;});_tmpC4(Cyc_Dict_count_elem,& num,d);});
return num;}struct _tuple2{void(*f1)(void*,void*);struct Cyc_Dict_Dict f2;};
# 40
static void Cyc_Dict_iter2_f(struct _tuple2*env,void*a,void*b1){
# 271
struct Cyc_Dict_Dict _tmp50;void(*_tmp4F)(void*,void*);_LL1: _tmp4F=env->f1;_tmp50=env->f2;_LL2: {void(*f)(void*,void*)=_tmp4F;struct Cyc_Dict_Dict d2=_tmp50;
({void(*_tmpC6)(void*,void*)=f;void*_tmpC5=b1;_tmpC6(_tmpC5,Cyc_Dict_lookup(d2,a));});}}
# 40
void Cyc_Dict_iter2(void(*f)(void*,void*),struct Cyc_Dict_Dict d1,struct Cyc_Dict_Dict d2){
# 278
struct _tuple2 env=({struct _tuple2 _tmpA0;_tmpA0.f1=f,_tmpA0.f2=d2;_tmpA0;});
({void(*_tmpC7)(void(*f)(struct _tuple2*,void*,void*),struct _tuple2*env,struct Cyc_Dict_Dict d)=({void(*_tmp51)(void(*f)(struct _tuple2*,void*,void*),struct _tuple2*env,struct Cyc_Dict_Dict d)=(void(*)(void(*f)(struct _tuple2*,void*,void*),struct _tuple2*env,struct Cyc_Dict_Dict d))Cyc_Dict_iter_c;_tmp51;});_tmpC7(Cyc_Dict_iter2_f,& env,d1);});}struct _tuple3{void(*f1)(void*,void*,void*);struct Cyc_Dict_Dict f2;void*f3;};
# 40
static void Cyc_Dict_iter2_c_f(struct _tuple3*env,void*a,void*b1){
# 284
void*_tmp54;struct Cyc_Dict_Dict _tmp53;void(*_tmp52)(void*,void*,void*);_LL1: _tmp52=env->f1;_tmp53=env->f2;_tmp54=env->f3;_LL2: {void(*f)(void*,void*,void*)=_tmp52;struct Cyc_Dict_Dict d2=_tmp53;void*inner_env=_tmp54;
({void(*_tmpCA)(void*,void*,void*)=f;void*_tmpC9=inner_env;void*_tmpC8=b1;_tmpCA(_tmpC9,_tmpC8,Cyc_Dict_lookup(d2,a));});}}
# 40
void Cyc_Dict_iter2_c(void(*f)(void*,void*,void*),void*inner_env,struct Cyc_Dict_Dict d1,struct Cyc_Dict_Dict d2){
# 291
struct _tuple3 env=({struct _tuple3 _tmpA1;_tmpA1.f1=f,_tmpA1.f2=d2,_tmpA1.f3=inner_env;_tmpA1;});
({void(*_tmpCB)(void(*f)(struct _tuple3*,void*,void*),struct _tuple3*env,struct Cyc_Dict_Dict d)=({void(*_tmp55)(void(*f)(struct _tuple3*,void*,void*),struct _tuple3*env,struct Cyc_Dict_Dict d)=(void(*)(void(*f)(struct _tuple3*,void*,void*),struct _tuple3*env,struct Cyc_Dict_Dict d))Cyc_Dict_iter_c;_tmp55;});_tmpCB(Cyc_Dict_iter2_c_f,& env,d1);});}struct _tuple4{void*(*f1)(void*,void*,void*,void*,void*);struct Cyc_Dict_Dict f2;void*f3;};
# 40
static void*Cyc_Dict_fold2_c_f(struct _tuple4*env,void*a,void*b1,void*accum){
# 299
void*_tmp58;struct Cyc_Dict_Dict _tmp57;void*(*_tmp56)(void*,void*,void*,void*,void*);_LL1: _tmp56=env->f1;_tmp57=env->f2;_tmp58=env->f3;_LL2: {void*(*f)(void*,void*,void*,void*,void*)=_tmp56;struct Cyc_Dict_Dict d2=_tmp57;void*inner_env=_tmp58;
return({void*(*_tmpD0)(void*,void*,void*,void*,void*)=f;void*_tmpCF=inner_env;void*_tmpCE=a;void*_tmpCD=b1;void*_tmpCC=Cyc_Dict_lookup(d2,a);_tmpD0(_tmpCF,_tmpCE,_tmpCD,_tmpCC,accum);});}}
# 40
void*Cyc_Dict_fold2_c(void*(*f)(void*,void*,void*,void*,void*),void*inner_env,struct Cyc_Dict_Dict d1,struct Cyc_Dict_Dict d2,void*accum){
# 307
struct _tuple4 env=({struct _tuple4 _tmpA2;_tmpA2.f1=f,_tmpA2.f2=d2,_tmpA2.f3=inner_env;_tmpA2;});
return({void*(*_tmpD2)(void*(*f)(struct _tuple4*,void*,void*,void*),struct _tuple4*env,struct Cyc_Dict_Dict d,void*accum)=({void*(*_tmp59)(void*(*f)(struct _tuple4*,void*,void*,void*),struct _tuple4*env,struct Cyc_Dict_Dict d,void*accum)=(void*(*)(void*(*f)(struct _tuple4*,void*,void*,void*),struct _tuple4*env,struct Cyc_Dict_Dict d,void*accum))Cyc_Dict_fold_c;_tmp59;});struct Cyc_Dict_Dict _tmpD1=d1;_tmpD2(Cyc_Dict_fold2_c_f,& env,_tmpD1,accum);});}
# 40
static const struct Cyc_Dict_T*Cyc_Dict_copy_tree(struct _RegionHandle*r2,const struct Cyc_Dict_T*t){
# 312
if(t == (const struct Cyc_Dict_T*)0)return 0;else{
# 314
struct Cyc_Dict_T _stmttmp0=*t;struct _tuple0 _tmp5D;const struct Cyc_Dict_T*_tmp5C;const struct Cyc_Dict_T*_tmp5B;enum Cyc_Dict_Color _tmp5A;_LL1: _tmp5A=_stmttmp0.color;_tmp5B=_stmttmp0.left;_tmp5C=_stmttmp0.right;_tmp5D=_stmttmp0.key_val;_LL2: {enum Cyc_Dict_Color c=_tmp5A;const struct Cyc_Dict_T*left=_tmp5B;const struct Cyc_Dict_T*right=_tmp5C;struct _tuple0 pr=_tmp5D;
const struct Cyc_Dict_T*new_left=Cyc_Dict_copy_tree(r2,left);
const struct Cyc_Dict_T*new_right=Cyc_Dict_copy_tree(r2,right);
return(const struct Cyc_Dict_T*)({struct Cyc_Dict_T*_tmp5E=_region_malloc(r2,sizeof(*_tmp5E));_tmp5E->color=c,_tmp5E->left=new_left,_tmp5E->right=new_right,_tmp5E->key_val=pr;_tmp5E;});}}}
# 40
struct Cyc_Dict_Dict Cyc_Dict_rcopy(struct _RegionHandle*r2,struct Cyc_Dict_Dict d){
# 322
return({struct Cyc_Dict_Dict _tmpA3;_tmpA3.rel=d.rel,_tmpA3.r=r2,({const struct Cyc_Dict_T*_tmpD3=Cyc_Dict_copy_tree(r2,d.t);_tmpA3.t=_tmpD3;});_tmpA3;});}
# 40
struct Cyc_Dict_Dict Cyc_Dict_copy(struct Cyc_Dict_Dict d){
# 326
return Cyc_Dict_rcopy(Cyc_Core_heap_region,d);}
# 40
static const struct Cyc_Dict_T*Cyc_Dict_map_tree(struct _RegionHandle*r,void*(*f)(void*),const struct Cyc_Dict_T*t){
# 331
void*_tmp63;void*_tmp62;const struct Cyc_Dict_T*_tmp61;const struct Cyc_Dict_T*_tmp60;enum Cyc_Dict_Color _tmp5F;_LL1: _tmp5F=t->color;_tmp60=t->left;_tmp61=t->right;_tmp62=(t->key_val).f1;_tmp63=(t->key_val).f2;_LL2: {enum Cyc_Dict_Color c=_tmp5F;const struct Cyc_Dict_T*left=_tmp60;const struct Cyc_Dict_T*right=_tmp61;void*key=_tmp62;void*val=_tmp63;
const struct Cyc_Dict_T*new_left=left == (const struct Cyc_Dict_T*)0?0: Cyc_Dict_map_tree(r,f,left);
void*new_val=f(val);
const struct Cyc_Dict_T*new_right=right == (const struct Cyc_Dict_T*)0?0: Cyc_Dict_map_tree(r,f,right);
return(const struct Cyc_Dict_T*)({struct Cyc_Dict_T*_tmp64=_region_malloc(r,sizeof(*_tmp64));_tmp64->color=c,_tmp64->left=new_left,_tmp64->right=new_right,(_tmp64->key_val).f1=key,(_tmp64->key_val).f2=new_val;_tmp64;});}}
# 40
struct Cyc_Dict_Dict Cyc_Dict_rmap(struct _RegionHandle*r,void*(*f)(void*),struct Cyc_Dict_Dict d){
# 339
const struct Cyc_Dict_T*t=d.t;
if(t == (const struct Cyc_Dict_T*)0)
return({struct Cyc_Dict_Dict _tmpA4;_tmpA4.rel=d.rel,_tmpA4.r=r,_tmpA4.t=0;_tmpA4;});
# 340
return({struct Cyc_Dict_Dict _tmpA5;_tmpA5.rel=d.rel,_tmpA5.r=r,({
# 342
const struct Cyc_Dict_T*_tmpD4=Cyc_Dict_map_tree(r,f,t);_tmpA5.t=_tmpD4;});_tmpA5;});}
# 40
struct Cyc_Dict_Dict Cyc_Dict_map(void*(*f)(void*),struct Cyc_Dict_Dict d){
# 346
return Cyc_Dict_rmap(Cyc_Core_heap_region,f,d);}
# 40
static const struct Cyc_Dict_T*Cyc_Dict_map_tree_c(struct _RegionHandle*r,void*(*f)(void*,void*),void*env,const struct Cyc_Dict_T*t){
# 351
void*_tmp69;void*_tmp68;const struct Cyc_Dict_T*_tmp67;const struct Cyc_Dict_T*_tmp66;enum Cyc_Dict_Color _tmp65;_LL1: _tmp65=t->color;_tmp66=t->left;_tmp67=t->right;_tmp68=(t->key_val).f1;_tmp69=(t->key_val).f2;_LL2: {enum Cyc_Dict_Color c=_tmp65;const struct Cyc_Dict_T*left=_tmp66;const struct Cyc_Dict_T*right=_tmp67;void*key=_tmp68;void*val=_tmp69;
const struct Cyc_Dict_T*new_left=
left == (const struct Cyc_Dict_T*)0?0: Cyc_Dict_map_tree_c(r,f,env,left);
void*new_val=f(env,val);
const struct Cyc_Dict_T*new_right=
right == (const struct Cyc_Dict_T*)0?0: Cyc_Dict_map_tree_c(r,f,env,right);
return(const struct Cyc_Dict_T*)({struct Cyc_Dict_T*_tmp6A=_region_malloc(r,sizeof(*_tmp6A));_tmp6A->color=c,_tmp6A->left=new_left,_tmp6A->right=new_right,(_tmp6A->key_val).f1=key,(_tmp6A->key_val).f2=new_val;_tmp6A;});}}
# 40
struct Cyc_Dict_Dict Cyc_Dict_rmap_c(struct _RegionHandle*r,void*(*f)(void*,void*),void*env,struct Cyc_Dict_Dict d){
# 362
const struct Cyc_Dict_T*t=d.t;
if(t == (const struct Cyc_Dict_T*)0)
return({struct Cyc_Dict_Dict _tmpA6;_tmpA6.rel=d.rel,_tmpA6.r=r,_tmpA6.t=0;_tmpA6;});
# 363
return({struct Cyc_Dict_Dict _tmpA7;_tmpA7.rel=d.rel,_tmpA7.r=r,({
# 365
const struct Cyc_Dict_T*_tmpD5=Cyc_Dict_map_tree_c(r,f,env,t);_tmpA7.t=_tmpD5;});_tmpA7;});}
# 40
struct Cyc_Dict_Dict Cyc_Dict_map_c(void*(*f)(void*,void*),void*env,struct Cyc_Dict_Dict d){
# 369
return Cyc_Dict_rmap_c(Cyc_Core_heap_region,f,env,d);}
# 40
struct _tuple0*Cyc_Dict_rchoose(struct _RegionHandle*r,struct Cyc_Dict_Dict d){
# 373
if(d.t == (const struct Cyc_Dict_T*)0)
(int)_throw((void*)& Cyc_Dict_Absent_val);
# 373
return({struct _tuple0*_tmp6B=_region_malloc(r,sizeof(*_tmp6B));
# 375
_tmp6B->f1=((d.t)->key_val).f1,_tmp6B->f2=((d.t)->key_val).f2;_tmp6B;});}
# 40
static int Cyc_Dict_forall_tree_c(int(*f)(void*,void*,void*),void*env,const struct Cyc_Dict_T*t){
# 379
void*_tmp6F;void*_tmp6E;const struct Cyc_Dict_T*_tmp6D;const struct Cyc_Dict_T*_tmp6C;_LL1: _tmp6C=t->left;_tmp6D=t->right;_tmp6E=(t->key_val).f1;_tmp6F=(t->key_val).f2;_LL2: {const struct Cyc_Dict_T*left=_tmp6C;const struct Cyc_Dict_T*right=_tmp6D;void*key=_tmp6E;void*val=_tmp6F;
return
((left == (const struct Cyc_Dict_T*)0 || Cyc_Dict_forall_tree_c(f,env,left))&&
 f(env,key,val))&&(
right == (const struct Cyc_Dict_T*)0 || Cyc_Dict_forall_tree_c(f,env,right));}}
# 40
int Cyc_Dict_forall_c(int(*f)(void*,void*,void*),void*env,struct Cyc_Dict_Dict d){
# 387
const struct Cyc_Dict_T*t=d.t;
if(t == (const struct Cyc_Dict_T*)0)
return 1;
# 388
return
# 390
 Cyc_Dict_forall_tree_c(f,env,t);}struct _tuple5{int(*f1)(void*,void*,void*);struct Cyc_Dict_Dict f2;};
# 40
static int Cyc_Dict_forall_intersect_f(struct _tuple5*env,void*a,void*b){
# 395
struct Cyc_Dict_Dict _tmp71;int(*_tmp70)(void*,void*,void*);_LL1: _tmp70=env->f1;_tmp71=env->f2;_LL2: {int(*f)(void*,void*,void*)=_tmp70;struct Cyc_Dict_Dict d2=_tmp71;
if(Cyc_Dict_member(d2,a))
return({int(*_tmpD8)(void*,void*,void*)=f;void*_tmpD7=a;void*_tmpD6=b;_tmpD8(_tmpD7,_tmpD6,Cyc_Dict_lookup(d2,a));});
# 396
return 1;}}
# 40
int Cyc_Dict_forall_intersect(int(*f)(void*,void*,void*),struct Cyc_Dict_Dict d1,struct Cyc_Dict_Dict d2){
# 403
struct _tuple5 env=({struct _tuple5 _tmpA8;_tmpA8.f1=f,_tmpA8.f2=d2;_tmpA8;});
return({int(*_tmpD9)(int(*f)(struct _tuple5*,void*,void*),struct _tuple5*env,struct Cyc_Dict_Dict d)=({int(*_tmp72)(int(*f)(struct _tuple5*,void*,void*),struct _tuple5*env,struct Cyc_Dict_Dict d)=(int(*)(int(*f)(struct _tuple5*,void*,void*),struct _tuple5*env,struct Cyc_Dict_Dict d))Cyc_Dict_forall_c;_tmp72;});_tmpD9(Cyc_Dict_forall_intersect_f,& env,d1);});}struct _tuple6{void*(*f1)(void*,void*,void*,void*);void*f2;};
# 40
static struct Cyc_Dict_Dict*Cyc_Dict_union_f(struct _tuple6*env,void*a,void*b,struct Cyc_Dict_Dict*d1){
# 412
if(Cyc_Dict_member(*d1,a)){
# 414
void*old_val=Cyc_Dict_lookup(*d1,a);
void*new_val=((*env).f1)((*env).f2,a,old_val,b);
if(new_val != old_val)
({struct Cyc_Dict_Dict _tmpDA=Cyc_Dict_insert(*d1,a,new_val);*d1=_tmpDA;});
# 416
return d1;}
# 412
({struct Cyc_Dict_Dict _tmpDB=
# 420
Cyc_Dict_insert(*d1,a,b);
# 412
*d1=_tmpDB;});
# 421
return d1;}
# 40
struct Cyc_Dict_Dict Cyc_Dict_union_two_c(void*(*f)(void*,void*,void*,void*),void*env,struct Cyc_Dict_Dict d1,struct Cyc_Dict_Dict d2){
# 427
if((int)d1.t == (int)d2.t)return d1;if(d1.t == (const struct Cyc_Dict_T*)0)
return d2;
# 427
if(d2.t == (const struct Cyc_Dict_T*)0)
# 429
return d1;{
# 427
struct _tuple6 fenvpair=({struct _tuple6 _tmpA9;_tmpA9.f1=f,_tmpA9.f2=env;_tmpA9;});
# 431
({struct Cyc_Dict_Dict*(*_tmpDC)(struct Cyc_Dict_Dict*(*f)(struct _tuple6*,void*,void*,struct Cyc_Dict_Dict*),struct _tuple6*env,struct Cyc_Dict_Dict d,struct Cyc_Dict_Dict*accum)=({struct Cyc_Dict_Dict*(*_tmp73)(struct Cyc_Dict_Dict*(*f)(struct _tuple6*,void*,void*,struct Cyc_Dict_Dict*),struct _tuple6*env,struct Cyc_Dict_Dict d,struct Cyc_Dict_Dict*accum)=(struct Cyc_Dict_Dict*(*)(struct Cyc_Dict_Dict*(*f)(struct _tuple6*,void*,void*,struct Cyc_Dict_Dict*),struct _tuple6*env,struct Cyc_Dict_Dict d,struct Cyc_Dict_Dict*accum))Cyc_Dict_fold_c;_tmp73;});_tmpDC(Cyc_Dict_union_f,& fenvpair,d2,& d1);});
return d1;}}
# 40
struct Cyc_Dict_Dict Cyc_Dict_intersect_c(void*(*f)(void*,void*,void*,void*),void*env,struct Cyc_Dict_Dict d1,struct Cyc_Dict_Dict d2){
# 438
const struct Cyc_Dict_T*t2=d2.t;
if(t2 == (const struct Cyc_Dict_T*)0)return d2;if((int)d1.t == (int)t2)
return d2;{
# 439
const struct Cyc_Dict_T*ans_tree=0;
# 442
struct _RegionHandle _tmp74=_new_region("temp");struct _RegionHandle*temp=& _tmp74;_push_region(temp);{
# 444
struct _fat_ptr queue=_tag_fat(({unsigned _tmp7C=16U;const struct Cyc_Dict_T**_tmp7B=({struct _RegionHandle*_tmpDD=temp;_region_malloc(_tmpDD,_check_times(_tmp7C,sizeof(const struct Cyc_Dict_T*)));});({{unsigned _tmpAC=16U;unsigned i;for(i=0;i < _tmpAC;++ i){_tmp7B[i]=t2;}}0;});_tmp7B;}),sizeof(const struct Cyc_Dict_T*),16U);
int ind=0;
while(ind != -1){
const struct Cyc_Dict_T*_stmttmp1=*((const struct Cyc_Dict_T**)_check_fat_subscript(queue,sizeof(const struct Cyc_Dict_T*),ind --));void*_tmp78;void*_tmp77;const struct Cyc_Dict_T*_tmp76;const struct Cyc_Dict_T*_tmp75;_LL1: _tmp75=_stmttmp1->left;_tmp76=_stmttmp1->right;_tmp77=(_stmttmp1->key_val).f1;_tmp78=(_stmttmp1->key_val).f2;_LL2: {const struct Cyc_Dict_T*l=_tmp75;const struct Cyc_Dict_T*r=_tmp76;void*k=_tmp77;void*v=_tmp78;
if((unsigned)(ind + 2)>= _get_fat_size(queue,sizeof(const struct Cyc_Dict_T*)))
queue=({unsigned _tmp7A=_get_fat_size(queue,sizeof(const struct Cyc_Dict_T*))* (unsigned)2;const struct Cyc_Dict_T**_tmp79=({struct _RegionHandle*_tmpDE=temp;_region_malloc(_tmpDE,_check_times(_tmp7A,sizeof(const struct Cyc_Dict_T*)));});({{unsigned _tmpAA=_get_fat_size(queue,sizeof(const struct Cyc_Dict_T*))* (unsigned)2;unsigned i;for(i=0;i < _tmpAA;++ i){
i < _get_fat_size(queue,sizeof(const struct Cyc_Dict_T*))?_tmp79[i]=((const struct Cyc_Dict_T**)queue.curr)[(int)i]:(_tmp79[i]=t2);}}0;});_tag_fat(_tmp79,sizeof(const struct Cyc_Dict_T*),_tmp7A);});
# 448
if(l != (const struct Cyc_Dict_T*)0)
# 451
*((const struct Cyc_Dict_T**)_check_fat_subscript(queue,sizeof(const struct Cyc_Dict_T*),++ ind))=l;
# 448
if(r != (const struct Cyc_Dict_T*)0)
# 452
*((const struct Cyc_Dict_T**)_check_fat_subscript(queue,sizeof(const struct Cyc_Dict_T*),++ ind))=r;
# 448
if(
# 453
Cyc_Dict_member(d1,k))
ans_tree=(const struct Cyc_Dict_T*)({struct _RegionHandle*_tmpE6=d2.r;int(*_tmpE5)(void*,void*)=d2.rel;struct _tuple0 _tmpE4=({struct _tuple0 _tmpAB;_tmpAB.f1=k,({void*_tmpE3=({void*(*_tmpE2)(void*,void*,void*,void*)=f;void*_tmpE1=env;void*_tmpE0=k;void*_tmpDF=Cyc_Dict_lookup(d1,k);_tmpE2(_tmpE1,_tmpE0,_tmpDF,v);});_tmpAB.f2=_tmpE3;});_tmpAB;});Cyc_Dict_ins(_tmpE6,_tmpE5,_tmpE4,ans_tree);});}}}{
# 457
struct Cyc_Dict_Dict _tmp7D=({struct Cyc_Dict_Dict _tmpAD;_tmpAD.rel=d2.rel,_tmpAD.r=d2.r,_tmpAD.t=ans_tree;_tmpAD;});_npop_handler(0U);return _tmp7D;}
# 442
;_pop_region(temp);}}
# 40
static void*Cyc_Dict_intersect_f(void*(*f)(void*,void*,void*),void*a,void*b1,void*b2){
# 461
return f(a,b1,b2);}
# 40
struct Cyc_Dict_Dict Cyc_Dict_intersect(void*(*f)(void*,void*,void*),struct Cyc_Dict_Dict d1,struct Cyc_Dict_Dict d2){
# 467
return({struct Cyc_Dict_Dict(*_tmpE9)(void*(*f)(void*(*)(void*,void*,void*),void*,void*,void*),void*(*env)(void*,void*,void*),struct Cyc_Dict_Dict d1,struct Cyc_Dict_Dict d2)=({struct Cyc_Dict_Dict(*_tmp7E)(void*(*f)(void*(*)(void*,void*,void*),void*,void*,void*),void*(*env)(void*,void*,void*),struct Cyc_Dict_Dict d1,struct Cyc_Dict_Dict d2)=(struct Cyc_Dict_Dict(*)(void*(*f)(void*(*)(void*,void*,void*),void*,void*,void*),void*(*env)(void*,void*,void*),struct Cyc_Dict_Dict d1,struct Cyc_Dict_Dict d2))Cyc_Dict_intersect_c;_tmp7E;});void*(*_tmpE8)(void*,void*,void*)=f;struct Cyc_Dict_Dict _tmpE7=d1;_tmpE9(Cyc_Dict_intersect_f,_tmpE8,_tmpE7,d2);});}
# 40
static struct Cyc_List_List*Cyc_Dict_to_list_f(struct _RegionHandle*r,void*k,void*v,struct Cyc_List_List*accum){
# 472
return({struct Cyc_List_List*_tmp80=_region_malloc(r,sizeof(*_tmp80));({struct _tuple0*_tmpEA=({struct _tuple0*_tmp7F=_region_malloc(r,sizeof(*_tmp7F));_tmp7F->f1=k,_tmp7F->f2=v;_tmp7F;});_tmp80->hd=_tmpEA;}),_tmp80->tl=accum;_tmp80;});}
# 40
struct Cyc_List_List*Cyc_Dict_rto_list(struct _RegionHandle*r,struct Cyc_Dict_Dict d){
# 476
return({struct Cyc_List_List*(*_tmpEC)(struct Cyc_List_List*(*f)(struct _RegionHandle*,void*,void*,struct Cyc_List_List*),struct _RegionHandle*env,struct Cyc_Dict_Dict d,struct Cyc_List_List*accum)=({struct Cyc_List_List*(*_tmp81)(struct Cyc_List_List*(*f)(struct _RegionHandle*,void*,void*,struct Cyc_List_List*),struct _RegionHandle*env,struct Cyc_Dict_Dict d,struct Cyc_List_List*accum)=(struct Cyc_List_List*(*)(struct Cyc_List_List*(*f)(struct _RegionHandle*,void*,void*,struct Cyc_List_List*),struct _RegionHandle*env,struct Cyc_Dict_Dict d,struct Cyc_List_List*accum))Cyc_Dict_fold_c;_tmp81;});struct _RegionHandle*_tmpEB=r;_tmpEC(Cyc_Dict_to_list_f,_tmpEB,d,0);});}
# 40
struct Cyc_List_List*Cyc_Dict_to_list(struct Cyc_Dict_Dict d){
# 479
return Cyc_Dict_rto_list(Cyc_Core_heap_region,d);}struct _tuple7{int(*f1)(void*,void*);struct _RegionHandle*f2;};
# 40
static struct Cyc_Dict_Dict*Cyc_Dict_filter_f(struct _tuple7*env,void*x,void*y,struct Cyc_Dict_Dict*acc){
# 484
struct _RegionHandle*_tmp83;int(*_tmp82)(void*,void*);_LL1: _tmp82=env->f1;_tmp83=env->f2;_LL2: {int(*f)(void*,void*)=_tmp82;struct _RegionHandle*r=_tmp83;
if(f(x,y))
({struct Cyc_Dict_Dict _tmpED=Cyc_Dict_insert(*acc,x,y);*acc=_tmpED;});
# 485
return acc;}}
# 40
struct Cyc_Dict_Dict Cyc_Dict_rfilter(struct _RegionHandle*r2,int(*f)(void*,void*),struct Cyc_Dict_Dict d){
# 492
struct _tuple7 env=({struct _tuple7 _tmpAE;_tmpAE.f1=f,_tmpAE.f2=r2;_tmpAE;});
struct Cyc_Dict_Dict res=Cyc_Dict_rempty(r2,d.rel);
return*({struct Cyc_Dict_Dict*(*_tmpEE)(struct Cyc_Dict_Dict*(*f)(struct _tuple7*,void*,void*,struct Cyc_Dict_Dict*),struct _tuple7*env,struct Cyc_Dict_Dict d,struct Cyc_Dict_Dict*accum)=({struct Cyc_Dict_Dict*(*_tmp84)(struct Cyc_Dict_Dict*(*f)(struct _tuple7*,void*,void*,struct Cyc_Dict_Dict*),struct _tuple7*env,struct Cyc_Dict_Dict d,struct Cyc_Dict_Dict*accum)=(struct Cyc_Dict_Dict*(*)(struct Cyc_Dict_Dict*(*f)(struct _tuple7*,void*,void*,struct Cyc_Dict_Dict*),struct _tuple7*env,struct Cyc_Dict_Dict d,struct Cyc_Dict_Dict*accum))Cyc_Dict_fold_c;_tmp84;});_tmpEE(Cyc_Dict_filter_f,& env,d,& res);});}
# 40
struct Cyc_Dict_Dict Cyc_Dict_filter(int(*f)(void*,void*),struct Cyc_Dict_Dict d){
# 503
return Cyc_Dict_rfilter(Cyc_Core_heap_region,f,d);}struct _tuple8{int(*f1)(void*,void*,void*);void*f2;struct _RegionHandle*f3;};
# 40
static struct Cyc_Dict_Dict*Cyc_Dict_filter_c_f(struct _tuple8*env,void*x,void*y,struct Cyc_Dict_Dict*acc){
# 510
struct _RegionHandle*_tmp87;void*_tmp86;int(*_tmp85)(void*,void*,void*);_LL1: _tmp85=env->f1;_tmp86=env->f2;_tmp87=env->f3;_LL2: {int(*f)(void*,void*,void*)=_tmp85;void*f_env=_tmp86;struct _RegionHandle*r2=_tmp87;
if(f(f_env,x,y))
({struct Cyc_Dict_Dict _tmpEF=Cyc_Dict_insert(*acc,x,y);*acc=_tmpEF;});
# 511
return acc;}}
# 40
struct Cyc_Dict_Dict Cyc_Dict_rfilter_c(struct _RegionHandle*r2,int(*f)(void*,void*,void*),void*f_env,struct Cyc_Dict_Dict d){
# 518
struct _tuple8 env=({struct _tuple8 _tmpAF;_tmpAF.f1=f,_tmpAF.f2=f_env,_tmpAF.f3=r2;_tmpAF;});
struct Cyc_Dict_Dict res=Cyc_Dict_rempty(r2,d.rel);
return*({struct Cyc_Dict_Dict*(*_tmpF0)(struct Cyc_Dict_Dict*(*f)(struct _tuple8*,void*,void*,struct Cyc_Dict_Dict*),struct _tuple8*env,struct Cyc_Dict_Dict d,struct Cyc_Dict_Dict*accum)=({struct Cyc_Dict_Dict*(*_tmp88)(struct Cyc_Dict_Dict*(*f)(struct _tuple8*,void*,void*,struct Cyc_Dict_Dict*),struct _tuple8*env,struct Cyc_Dict_Dict d,struct Cyc_Dict_Dict*accum)=(struct Cyc_Dict_Dict*(*)(struct Cyc_Dict_Dict*(*f)(struct _tuple8*,void*,void*,struct Cyc_Dict_Dict*),struct _tuple8*env,struct Cyc_Dict_Dict d,struct Cyc_Dict_Dict*accum))Cyc_Dict_fold_c;_tmp88;});_tmpF0(Cyc_Dict_filter_c_f,& env,d,& res);});}
# 40
struct Cyc_Dict_Dict Cyc_Dict_filter_c(int(*f)(void*,void*,void*),void*f_env,struct Cyc_Dict_Dict d){
# 524
return Cyc_Dict_rfilter_c(Cyc_Core_heap_region,f,f_env,d);}
# 40
static int Cyc_Dict_difference_f(struct Cyc_Dict_Dict*d,void*x,void*y){
# 529
return !Cyc_Dict_member(*d,x);}
# 40
struct Cyc_Dict_Dict Cyc_Dict_rdifference(struct _RegionHandle*r2,struct Cyc_Dict_Dict d1,struct Cyc_Dict_Dict d2){
# 534
return({struct Cyc_Dict_Dict(*_tmpF2)(struct _RegionHandle*r2,int(*f)(struct Cyc_Dict_Dict*,void*,void*),struct Cyc_Dict_Dict*f_env,struct Cyc_Dict_Dict d)=({struct Cyc_Dict_Dict(*_tmp89)(struct _RegionHandle*r2,int(*f)(struct Cyc_Dict_Dict*,void*,void*),struct Cyc_Dict_Dict*f_env,struct Cyc_Dict_Dict d)=(struct Cyc_Dict_Dict(*)(struct _RegionHandle*r2,int(*f)(struct Cyc_Dict_Dict*,void*,void*),struct Cyc_Dict_Dict*f_env,struct Cyc_Dict_Dict d))Cyc_Dict_rfilter_c;_tmp89;});struct _RegionHandle*_tmpF1=r2;_tmpF2(_tmpF1,Cyc_Dict_difference_f,& d2,d1);});}
# 40
struct Cyc_Dict_Dict Cyc_Dict_difference(struct Cyc_Dict_Dict d1,struct Cyc_Dict_Dict d2){
# 537
return Cyc_Dict_rdifference(Cyc_Core_heap_region,d1,d2);}struct _tuple9{int(*f1)(void*,void*);void*f2;};
# 40
static int Cyc_Dict_delete_f(struct _tuple9*env,void*x,void*y){
# 541
void*_tmp8B;int(*_tmp8A)(void*,void*);_LL1: _tmp8A=env->f1;_tmp8B=env->f2;_LL2: {int(*rel)(void*,void*)=_tmp8A;void*x0=_tmp8B;
return rel(x0,x)!= 0;}}
# 40
struct Cyc_Dict_Dict Cyc_Dict_rdelete(struct _RegionHandle*r2,struct Cyc_Dict_Dict d,void*x){
# 546
if(!Cyc_Dict_member(d,x))return Cyc_Dict_rcopy(r2,d);{struct _tuple9 env=({struct _tuple9 _tmpB0;_tmpB0.f1=d.rel,_tmpB0.f2=x;_tmpB0;});
# 548
return({struct Cyc_Dict_Dict(*_tmpF4)(struct _RegionHandle*r2,int(*f)(struct _tuple9*,void*,void*),struct _tuple9*f_env,struct Cyc_Dict_Dict d)=({struct Cyc_Dict_Dict(*_tmp8C)(struct _RegionHandle*r2,int(*f)(struct _tuple9*,void*,void*),struct _tuple9*f_env,struct Cyc_Dict_Dict d)=(struct Cyc_Dict_Dict(*)(struct _RegionHandle*r2,int(*f)(struct _tuple9*,void*,void*),struct _tuple9*f_env,struct Cyc_Dict_Dict d))Cyc_Dict_rfilter_c;_tmp8C;});struct _RegionHandle*_tmpF3=r2;_tmpF4(_tmpF3,Cyc_Dict_delete_f,& env,d);});}}
# 40
struct Cyc_Dict_Dict Cyc_Dict_rdelete_same(struct Cyc_Dict_Dict d,void*x){
# 552
if(!Cyc_Dict_member(d,x))return d;{struct _tuple9 env=({struct _tuple9 _tmpB1;_tmpB1.f1=d.rel,_tmpB1.f2=x;_tmpB1;});
# 554
return({struct Cyc_Dict_Dict(*_tmpF6)(struct _RegionHandle*r2,int(*f)(struct _tuple9*,void*,void*),struct _tuple9*f_env,struct Cyc_Dict_Dict d)=({struct Cyc_Dict_Dict(*_tmp8D)(struct _RegionHandle*r2,int(*f)(struct _tuple9*,void*,void*),struct _tuple9*f_env,struct Cyc_Dict_Dict d)=(struct Cyc_Dict_Dict(*)(struct _RegionHandle*r2,int(*f)(struct _tuple9*,void*,void*),struct _tuple9*f_env,struct Cyc_Dict_Dict d))Cyc_Dict_rfilter_c;_tmp8D;});struct _RegionHandle*_tmpF5=d.r;_tmpF6(_tmpF5,Cyc_Dict_delete_f,& env,d);});}}
# 40
struct Cyc_Dict_Dict Cyc_Dict_delete(struct Cyc_Dict_Dict d,void*x){
# 558
return Cyc_Dict_rdelete(Cyc_Core_heap_region,d,x);}struct _tuple10{struct _fat_ptr f1;int f2;};
# 40
int Cyc_Dict_iter_f(struct _tuple10*stk,struct _tuple0*dest){
# 564
int*_tmp8F;struct _fat_ptr _tmp8E;_LL1: _tmp8E=stk->f1;_tmp8F=(int*)& stk->f2;_LL2: {struct _fat_ptr stack=_tmp8E;int*indp=_tmp8F;
int ind=*indp;
if(ind == -1)
return 0;{
# 566
const struct Cyc_Dict_T*t=*((const struct Cyc_Dict_T**)_check_fat_subscript(stack,sizeof(const struct Cyc_Dict_T*),ind));
# 569
*dest=((const struct Cyc_Dict_T*)_check_null(t))->key_val;
-- ind;
if((unsigned)t->left)
*((const struct Cyc_Dict_T**)_check_fat_subscript(stack,sizeof(const struct Cyc_Dict_T*),++ ind))=t->left;
# 571
if((unsigned)t->right)
# 574
*((const struct Cyc_Dict_T**)_check_fat_subscript(stack,sizeof(const struct Cyc_Dict_T*),++ ind))=t->right;
# 571
*indp=ind;
# 576
return 1;}}}
# 40
struct Cyc_Iter_Iter Cyc_Dict_make_iter(struct _RegionHandle*rgn,struct Cyc_Dict_Dict d){
# 584
int half_max_size=1;
const struct Cyc_Dict_T*t=d.t;
while(t != (const struct Cyc_Dict_T*)0){
t=t->left;
++ half_max_size;}
# 590
t=d.t;{
struct _fat_ptr stack=({unsigned _tmp92=(unsigned)(2 * half_max_size);const struct Cyc_Dict_T**_tmp91=({struct _RegionHandle*_tmpF7=rgn;_region_malloc(_tmpF7,_check_times(_tmp92,sizeof(const struct Cyc_Dict_T*)));});({{unsigned _tmpB3=(unsigned)(2 * half_max_size);unsigned i;for(i=0;i < _tmpB3;++ i){_tmp91[i]=t;}}0;});_tag_fat(_tmp91,sizeof(const struct Cyc_Dict_T*),_tmp92);});
return({struct Cyc_Iter_Iter _tmpB2;({struct _tuple10*_tmpF8=({struct _tuple10*_tmp90=_region_malloc(rgn,sizeof(*_tmp90));_tmp90->f1=stack,(unsigned)t?_tmp90->f2=0:(_tmp90->f2=- 1);_tmp90;});_tmpB2.env=_tmpF8;}),_tmpB2.next=Cyc_Dict_iter_f;_tmpB2;});}}
# 40
void*Cyc_Dict_marshal(struct _RegionHandle*rgn,void*env,void*(*write_key)(void*,struct Cyc___cycFILE*,void*),void*(*write_val)(void*,struct Cyc___cycFILE*,void*),struct Cyc___cycFILE*fp,struct Cyc_Dict_Dict dict){
# 607
struct Cyc_List_List*dict_list=Cyc_Dict_rto_list(rgn,dict);
int len=Cyc_List_length(dict_list);
# 611
(int)_throw((void*)({struct Cyc_Core_Failure_exn_struct*_tmp94=_cycalloc(sizeof(*_tmp94));_tmp94->tag=Cyc_Core_Failure,({struct _fat_ptr _tmpF9=({const char*_tmp93="Dict::marshal: Write failure";_tag_fat(_tmp93,sizeof(char),29U);});_tmp94->f1=_tmpF9;});_tmp94;}));
# 613
while(dict_list != 0){
env=({void*(*_tmpFC)(void*,struct Cyc___cycFILE*,struct _tuple0*)=({void*(*_tmp95)(void*,struct Cyc___cycFILE*,struct _tuple0*)=(void*(*)(void*,struct Cyc___cycFILE*,struct _tuple0*))write_key;_tmp95;});void*_tmpFB=env;struct Cyc___cycFILE*_tmpFA=fp;_tmpFC(_tmpFB,_tmpFA,(struct _tuple0*)dict_list->hd);});
env=({void*(*_tmpFF)(void*,struct Cyc___cycFILE*,struct _tuple0*)=({void*(*_tmp96)(void*,struct Cyc___cycFILE*,struct _tuple0*)=(void*(*)(void*,struct Cyc___cycFILE*,struct _tuple0*))write_val;_tmp96;});void*_tmpFE=env;struct Cyc___cycFILE*_tmpFD=fp;_tmpFF(_tmpFE,_tmpFD,(struct _tuple0*)dict_list->hd);});
dict_list=dict_list->tl;}
# 618
return env;}
# 40
struct Cyc_Dict_Dict Cyc_Dict_unmarshal(struct _RegionHandle*rgn,void*env,int(*cmp)(void*,void*),void*(*read_key)(void*,struct Cyc___cycFILE*),void*(*read_val)(void*,struct Cyc___cycFILE*),struct Cyc___cycFILE*fp){
# 628
struct Cyc_Dict_Dict dict=Cyc_Dict_empty(cmp);
int len=Cyc_getw(fp);
if(len == -1)
(int)_throw((void*)({struct Cyc_Core_Failure_exn_struct*_tmp98=_cycalloc(sizeof(*_tmp98));_tmp98->tag=Cyc_Core_Failure,({struct _fat_ptr _tmp100=({const char*_tmp97="Dict::unmarshal: list length is -1";_tag_fat(_tmp97,sizeof(char),35U);});_tmp98->f1=_tmp100;});_tmp98;}));
# 630
{
# 632
int i=0;
# 630
for(0;i < len;++ i){
# 633
void*key=read_key(env,fp);
void*val=read_val(env,fp);
dict=Cyc_Dict_insert(dict,key,val);}}
# 637
return dict;}
