#include <setjmp.h>
/* This is a C header file to be used by the output of the Cyclone to
   C translator.  The corresponding definitions are in file lib/runtime_*.c */
#ifndef _CYC_INCLUDE_H_
#define _CYC_INCLUDE_H_

/* Need one of these per thread (see runtime_stack.c). The runtime maintains 
   a stack that contains either _handler_cons structs or _RegionHandle structs.
   The tag is 0 for a handler_cons and 1 for a region handle.  */
struct _RuntimeStack {
  int tag; 
  struct _RuntimeStack *next;
  void (*cleanup)(struct _RuntimeStack *frame);
};

#ifndef offsetof
/* should be size_t, but int is fine. */
#define offsetof(t,n) ((int)(&(((t *)0)->n)))
#endif

/* Fat pointers */
struct _fat_ptr {
  unsigned char *curr; 
  unsigned char *base; 
  unsigned char *last_plus_one; 
};  

/* Discriminated Unions */
struct _xtunion_struct { char *tag; };

/* Regions */
struct _RegionPage
#ifdef CYC_REGION_PROFILE
{ unsigned total_bytes;
  unsigned free_bytes;
  /* MWH: wish we didn't have to include the stuff below ... */
  struct _RegionPage *next;
  char data[1];
}
#endif
; // abstract -- defined in runtime_memory.c
struct _RegionHandle {
  struct _RuntimeStack s;
  struct _RegionPage *curr;
  char               *offset;
  char               *last_plus_one;
  struct _DynRegionHandle *sub_regions;
#ifdef CYC_REGION_PROFILE
  const char         *name;
#else
  unsigned used_bytes;
  unsigned wasted_bytes;
#endif
};
struct _DynRegionFrame {
  struct _RuntimeStack s;
  struct _DynRegionHandle *x;
};
// A dynamic region is just a region handle.  The wrapper struct is for type
// abstraction.
struct Cyc_Core_DynamicRegion {
  struct _RegionHandle h;
};

struct _RegionHandle _new_region(const char*);
void* _region_malloc(struct _RegionHandle*, unsigned);
void* _region_calloc(struct _RegionHandle*, unsigned t, unsigned n);
void   _free_region(struct _RegionHandle*);
struct _RegionHandle*_open_dynregion(struct _DynRegionFrame*,struct _DynRegionHandle*);
void   _pop_dynregion();

/* Exceptions */
struct _handler_cons {
  struct _RuntimeStack s;
  jmp_buf handler;
};
void _push_handler(struct _handler_cons *);
void _push_region(struct _RegionHandle *);
void _npop_handler(int);
void _pop_handler();
void _pop_region();

#ifndef _throw
void* _throw_null_fn(const char*,unsigned);
void* _throw_arraybounds_fn(const char*,unsigned);
void* _throw_badalloc_fn(const char*,unsigned);
void* _throw_match_fn(const char*,unsigned);
void* _throw_fn(void*,const char*,unsigned);
void* _rethrow(void*);
#define _throw_null() (_throw_null_fn(__FILE__,__LINE__))
#define _throw_arraybounds() (_throw_arraybounds_fn(__FILE__,__LINE__))
#define _throw_badalloc() (_throw_badalloc_fn(__FILE__,__LINE__))
#define _throw_match() (_throw_match_fn(__FILE__,__LINE__))
#define _throw(e) (_throw_fn((e),__FILE__,__LINE__))
#endif

struct _xtunion_struct* Cyc_Core_get_exn_thrown();
/* Built-in Exceptions */
struct Cyc_Null_Exception_exn_struct { char *tag; };
struct Cyc_Array_bounds_exn_struct { char *tag; };
struct Cyc_Match_Exception_exn_struct { char *tag; };
struct Cyc_Bad_alloc_exn_struct { char *tag; };
extern char Cyc_Null_Exception[];
extern char Cyc_Array_bounds[];
extern char Cyc_Match_Exception[];
extern char Cyc_Bad_alloc[];

/* Built-in Run-time Checks and company */
#ifdef CYC_ANSI_OUTPUT
#define _INLINE  
#else
#define _INLINE inline
#endif

#ifdef NO_CYC_NULL_CHECKS
#define _check_null(ptr) (ptr)
#else
#define _check_null(ptr) \
  ({ void*_cks_null = (void*)(ptr); \
     if (!_cks_null) _throw_null(); \
     _cks_null; })
#endif

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index)\
   (((char*)ptr) + (elt_sz)*(index))
#ifdef NO_CYC_NULL_CHECKS
#define _check_known_subscript_null _check_known_subscript_notnull
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr);\
  int _index = (index);\
  if (!_cks_ptr) _throw_null(); \
  _cks_ptr + (elt_sz)*_index; })
#endif
#define _zero_arr_plus_fn(orig_x,orig_sz,orig_i,f,l) ((orig_x)+(orig_i))
#define _zero_arr_plus_char_fn _zero_arr_plus_fn
#define _zero_arr_plus_short_fn _zero_arr_plus_fn
#define _zero_arr_plus_int_fn _zero_arr_plus_fn
#define _zero_arr_plus_float_fn _zero_arr_plus_fn
#define _zero_arr_plus_double_fn _zero_arr_plus_fn
#define _zero_arr_plus_longdouble_fn _zero_arr_plus_fn
#define _zero_arr_plus_voidstar_fn _zero_arr_plus_fn
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (!_cks_ptr) _throw_null(); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })

/* _zero_arr_plus_*_fn(x,sz,i,filename,lineno) adds i to zero-terminated ptr
   x that has at least sz elements */
char* _zero_arr_plus_char_fn(char*,unsigned,int,const char*,unsigned);
short* _zero_arr_plus_short_fn(short*,unsigned,int,const char*,unsigned);
int* _zero_arr_plus_int_fn(int*,unsigned,int,const char*,unsigned);
float* _zero_arr_plus_float_fn(float*,unsigned,int,const char*,unsigned);
double* _zero_arr_plus_double_fn(double*,unsigned,int,const char*,unsigned);
long double* _zero_arr_plus_longdouble_fn(long double*,unsigned,int,const char*, unsigned);
void** _zero_arr_plus_voidstar_fn(void**,unsigned,int,const char*,unsigned);
#endif

/* _get_zero_arr_size_*(x,sz) returns the number of elements in a
   zero-terminated array that is NULL or has at least sz elements */
int _get_zero_arr_size_char(const char*,unsigned);
int _get_zero_arr_size_short(const short*,unsigned);
int _get_zero_arr_size_int(const int*,unsigned);
int _get_zero_arr_size_float(const float*,unsigned);
int _get_zero_arr_size_double(const double*,unsigned);
int _get_zero_arr_size_longdouble(const long double*,unsigned);
int _get_zero_arr_size_voidstar(const void**,unsigned);

/* _zero_arr_inplace_plus_*_fn(x,i,filename,lineno) sets
   zero-terminated pointer *x to *x + i */
char* _zero_arr_inplace_plus_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_short_fn(short**,int,const char*,unsigned);
int* _zero_arr_inplace_plus_int(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_longdouble_fn(long double**,int,const char*,unsigned);
void** _zero_arr_inplace_plus_voidstar_fn(void***,int,const char*,unsigned);
/* like the previous functions, but does post-addition (as in e++) */
char* _zero_arr_inplace_plus_post_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_post_short_fn(short**x,int,const char*,unsigned);
int* _zero_arr_inplace_plus_post_int_fn(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_post_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_post_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_post_longdouble_fn(long double**,int,const char *,unsigned);
void** _zero_arr_inplace_plus_post_voidstar_fn(void***,int,const char*,unsigned);
#define _zero_arr_plus_char(x,s,i) \
  (_zero_arr_plus_char_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_short(x,s,i) \
  (_zero_arr_plus_short_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_int(x,s,i) \
  (_zero_arr_plus_int_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_float(x,s,i) \
  (_zero_arr_plus_float_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_double(x,s,i) \
  (_zero_arr_plus_double_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_longdouble(x,s,i) \
  (_zero_arr_plus_longdouble_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_voidstar(x,s,i) \
  (_zero_arr_plus_voidstar_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_inplace_plus_char(x,i) \
  _zero_arr_inplace_plus_char_fn((char **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_short(x,i) \
  _zero_arr_inplace_plus_short_fn((short **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_int(x,i) \
  _zero_arr_inplace_plus_int_fn((int **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_float(x,i) \
  _zero_arr_inplace_plus_float_fn((float **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_double(x,i) \
  _zero_arr_inplace_plus_double_fn((double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_longdouble(x,i) \
  _zero_arr_inplace_plus_longdouble_fn((long double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_voidstar(x,i) \
  _zero_arr_inplace_plus_voidstar_fn((void ***)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_char(x,i) \
  _zero_arr_inplace_plus_post_char_fn((char **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_short(x,i) \
  _zero_arr_inplace_plus_post_short_fn((short **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_int(x,i) \
  _zero_arr_inplace_plus_post_int_fn((int **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_float(x,i) \
  _zero_arr_inplace_plus_post_float_fn((float **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_double(x,i) \
  _zero_arr_inplace_plus_post_double_fn((double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_longdouble(x,i) \
  _zero_arr_inplace_plus_post_longdouble_fn((long double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_voidstar(x,i) \
  _zero_arr_inplace_plus_post_voidstar_fn((void***)(x),(i),__FILE__,__LINE__)

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_fat_subscript(arr,elt_sz,index) ((arr).curr + (elt_sz) * (index))
#define _untag_fat_ptr(arr,elt_sz,num_elts) ((arr).curr)
#else
#define _check_fat_subscript(arr,elt_sz,index) ({ \
  struct _fat_ptr _cus_arr = (arr); \
  unsigned char *_cus_ans = _cus_arr.curr + (elt_sz) * (index); \
  /* JGM: not needed! if (!_cus_arr.base) _throw_null();*/ \
  if (_cus_ans < _cus_arr.base || _cus_ans >= _cus_arr.last_plus_one) \
    _throw_arraybounds(); \
  _cus_ans; })
#define _untag_fat_ptr(arr,elt_sz,num_elts) ({ \
  struct _fat_ptr _arr = (arr); \
  unsigned char *_curr = _arr.curr; \
  if ((_curr < _arr.base || _curr + (elt_sz) * (num_elts) > _arr.last_plus_one) &&\
      _curr != (unsigned char *)0) \
    _throw_arraybounds(); \
  _curr; })
#endif

#define _tag_fat(tcurr,elt_sz,num_elts) ({ \
  struct _fat_ptr _ans; \
  unsigned _num_elts = (num_elts);\
  _ans.base = _ans.curr = (void*)(tcurr); \
  /* JGM: if we're tagging NULL, ignore num_elts */ \
  _ans.last_plus_one = _ans.base ? (_ans.base + (elt_sz) * _num_elts) : 0; \
  _ans; })

#define _get_fat_size(arr,elt_sz) \
  ({struct _fat_ptr _arr = (arr); \
    unsigned char *_arr_curr=_arr.curr; \
    unsigned char *_arr_last=_arr.last_plus_one; \
    (_arr_curr < _arr.base || _arr_curr >= _arr_last) ? 0 : \
    ((_arr_last - _arr_curr) / (elt_sz));})

#define _fat_ptr_plus(arr,elt_sz,change) ({ \
  struct _fat_ptr _ans = (arr); \
  int _change = (change);\
  _ans.curr += (elt_sz) * _change;\
  _ans; })
#define _fat_ptr_inplace_plus(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  _arr_ptr->curr += (elt_sz) * (change);\
  *_arr_ptr; })
#define _fat_ptr_inplace_plus_post(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  struct _fat_ptr _ans = *_arr_ptr; \
  _arr_ptr->curr += (elt_sz) * (change);\
  _ans; })

//Not a macro since initialization order matters. Defined in runtime_zeroterm.c.
struct _fat_ptr _fat_ptr_decrease_size(struct _fat_ptr,unsigned sz,unsigned numelts);

/* Allocation */
void* GC_malloc(int);
void* GC_malloc_atomic(int);
void* GC_calloc(unsigned,unsigned);
void* GC_calloc_atomic(unsigned,unsigned);
// bound the allocation size to be < MAX_ALLOC_SIZE. See macros below for usage.
#define MAX_MALLOC_SIZE (1 << 28)
void* _bounded_GC_malloc(int,const char*,int);
void* _bounded_GC_malloc_atomic(int,const char*,int);
void* _bounded_GC_calloc(unsigned,unsigned,const char*,int);
void* _bounded_GC_calloc_atomic(unsigned,unsigned,const char*,int);
/* these macros are overridden below ifdef CYC_REGION_PROFILE */
#ifndef CYC_REGION_PROFILE
#define _cycalloc(n) _bounded_GC_malloc(n,__FILE__,__LINE__)
#define _cycalloc_atomic(n) _bounded_GC_malloc_atomic(n,__FILE__,__LINE__)
#define _cyccalloc(n,s) _bounded_GC_calloc(n,s,__FILE__,__LINE__)
#define _cyccalloc_atomic(n,s) _bounded_GC_calloc_atomic(n,s,__FILE__,__LINE__)
#endif

static _INLINE unsigned int _check_times(unsigned x, unsigned y) {
  unsigned long long whole_ans = 
    ((unsigned long long) x)*((unsigned long long)y);
  unsigned word_ans = (unsigned)whole_ans;
  if(word_ans < whole_ans || word_ans > MAX_MALLOC_SIZE)
    _throw_badalloc();
  return word_ans;
}

#define _CYC_MAX_REGION_CONST 2
#define _CYC_MIN_ALIGNMENT (sizeof(double))

#ifdef CYC_REGION_PROFILE
extern int rgn_total_bytes;
#endif

static _INLINE void *_fast_region_malloc(struct _RegionHandle *r, unsigned orig_s) {  
  if (r > (struct _RegionHandle *)_CYC_MAX_REGION_CONST && r->curr != 0) { 
#ifdef CYC_NOALIGN
    unsigned s =  orig_s;
#else
    unsigned s =  (orig_s + _CYC_MIN_ALIGNMENT - 1) & (~(_CYC_MIN_ALIGNMENT -1)); 
#endif
    char *result; 
    result = r->offset; 
    if (s <= (r->last_plus_one - result)) {
      r->offset = result + s; 
#ifdef CYC_REGION_PROFILE
    r->curr->free_bytes = r->curr->free_bytes - s;
    rgn_total_bytes += s;
#endif
      return result;
    }
  } 
  return _region_malloc(r,orig_s); 
}

#ifdef CYC_REGION_PROFILE
/* see macros below for usage. defined in runtime_memory.c */
void* _profile_GC_malloc(int,const char*,const char*,int);
void* _profile_GC_malloc_atomic(int,const char*,const char*,int);
void* _profile_GC_calloc(unsigned,unsigned,const char*,const char*,int);
void* _profile_GC_calloc_atomic(unsigned,unsigned,const char*,const char*,int);
void* _profile_region_malloc(struct _RegionHandle*,unsigned,const char*,const char*,int);
void* _profile_region_calloc(struct _RegionHandle*,unsigned,unsigned,const char *,const char*,int);
struct _RegionHandle _profile_new_region(const char*,const char*,const char*,int);
void _profile_free_region(struct _RegionHandle*,const char*,const char*,int);
#ifndef RUNTIME_CYC
#define _new_region(n) _profile_new_region(n,__FILE__,__FUNCTION__,__LINE__)
#define _free_region(r) _profile_free_region(r,__FILE__,__FUNCTION__,__LINE__)
#define _region_malloc(rh,n) _profile_region_malloc(rh,n,__FILE__,__FUNCTION__,__LINE__)
#define _region_calloc(rh,n,t) _profile_region_calloc(rh,n,t,__FILE__,__FUNCTION__,__LINE__)
#  endif
#define _cycalloc(n) _profile_GC_malloc(n,__FILE__,__FUNCTION__,__LINE__)
#define _cycalloc_atomic(n) _profile_GC_malloc_atomic(n,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc(n,s) _profile_GC_calloc(n,s,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc_atomic(n,s) _profile_GC_calloc_atomic(n,s,__FILE__,__FUNCTION__,__LINE__)
#endif
#endif
 struct Cyc_Core_Opt{void*v;};extern char Cyc_Core_Invalid_argument[17U];struct Cyc_Core_Invalid_argument_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Failure[8U];struct Cyc_Core_Failure_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Impossible[11U];struct Cyc_Core_Impossible_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Not_found[10U];struct Cyc_Core_Not_found_exn_struct{char*tag;};extern char Cyc_Core_Unreachable[12U];struct Cyc_Core_Unreachable_exn_struct{char*tag;struct _fat_ptr f1;};
# 168 "core.h"
extern struct _RegionHandle*Cyc_Core_unique_region;struct Cyc_Core_DynamicRegion;struct Cyc_Core_NewDynamicRegion{struct Cyc_Core_DynamicRegion*key;};struct Cyc_Core_ThinRes{void*arr;unsigned nelts;};struct Cyc_List_List{void*hd;struct Cyc_List_List*tl;};
# 61 "list.h"
int Cyc_List_length(struct Cyc_List_List*x);extern char Cyc_List_List_mismatch[14U];struct Cyc_List_List_mismatch_exn_struct{char*tag;};extern char Cyc_List_Nth[4U];struct Cyc_List_Nth_exn_struct{char*tag;};
# 276
struct Cyc_List_List*Cyc_List_rzip(struct _RegionHandle*r1,struct _RegionHandle*r2,struct Cyc_List_List*x,struct Cyc_List_List*y);struct Cyc___cycFILE;struct Cyc_String_pa_PrintArg_struct{int tag;struct _fat_ptr f1;};struct Cyc_Int_pa_PrintArg_struct{int tag;unsigned long f1;};struct Cyc_Double_pa_PrintArg_struct{int tag;double f1;};struct Cyc_LongDouble_pa_PrintArg_struct{int tag;long double f1;};struct Cyc_ShortPtr_pa_PrintArg_struct{int tag;short*f1;};struct Cyc_IntPtr_pa_PrintArg_struct{int tag;unsigned long*f1;};
# 73 "cycboot.h"
struct _fat_ptr Cyc_aprintf(struct _fat_ptr,struct _fat_ptr);struct Cyc_ShortPtr_sa_ScanfArg_struct{int tag;short*f1;};struct Cyc_UShortPtr_sa_ScanfArg_struct{int tag;unsigned short*f1;};struct Cyc_IntPtr_sa_ScanfArg_struct{int tag;int*f1;};struct Cyc_UIntPtr_sa_ScanfArg_struct{int tag;unsigned*f1;};struct Cyc_StringPtr_sa_ScanfArg_struct{int tag;struct _fat_ptr f1;};struct Cyc_DoublePtr_sa_ScanfArg_struct{int tag;double*f1;};struct Cyc_FloatPtr_sa_ScanfArg_struct{int tag;float*f1;};struct Cyc_CharPtr_sa_ScanfArg_struct{int tag;struct _fat_ptr f1;};extern char Cyc_FileCloseError[15U];struct Cyc_FileCloseError_exn_struct{char*tag;};extern char Cyc_FileOpenError[14U];struct Cyc_FileOpenError_exn_struct{char*tag;struct _fat_ptr f1;};
# 50 "string.h"
int Cyc_strptrcmp(struct _fat_ptr*s1,struct _fat_ptr*s2);struct Cyc_PP_Ppstate;struct Cyc_PP_Out;struct Cyc_PP_Doc;struct Cyc_Position_Error;struct Cyc_Relations_Reln;struct _union_Nmspace_Rel_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_Abs_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_C_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_Loc_n{int tag;int val;};union Cyc_Absyn_Nmspace{struct _union_Nmspace_Rel_n Rel_n;struct _union_Nmspace_Abs_n Abs_n;struct _union_Nmspace_C_n C_n;struct _union_Nmspace_Loc_n Loc_n;};struct _tuple0{union Cyc_Absyn_Nmspace f1;struct _fat_ptr*f2;};
# 161 "absyn.h"
enum Cyc_Absyn_Scope{Cyc_Absyn_Static =0U,Cyc_Absyn_Abstract =1U,Cyc_Absyn_Public =2U,Cyc_Absyn_Extern =3U,Cyc_Absyn_ExternC =4U,Cyc_Absyn_Register =5U};struct Cyc_Absyn_Tqual{int print_const: 1;int q_volatile: 1;int q_restrict: 1;int real_const: 1;unsigned loc;};
# 182
enum Cyc_Absyn_Size_of{Cyc_Absyn_Char_sz =0U,Cyc_Absyn_Short_sz =1U,Cyc_Absyn_Int_sz =2U,Cyc_Absyn_Long_sz =3U,Cyc_Absyn_LongLong_sz =4U};
enum Cyc_Absyn_Sign{Cyc_Absyn_Signed =0U,Cyc_Absyn_Unsigned =1U,Cyc_Absyn_None =2U};
enum Cyc_Absyn_AggrKind{Cyc_Absyn_StructA =0U,Cyc_Absyn_UnionA =1U};
# 187
enum Cyc_Absyn_AliasQual{Cyc_Absyn_Aliasable =0U,Cyc_Absyn_Unique =1U,Cyc_Absyn_Top =2U};
# 192
enum Cyc_Absyn_KindQual{Cyc_Absyn_AnyKind =0U,Cyc_Absyn_MemKind =1U,Cyc_Absyn_BoxKind =2U,Cyc_Absyn_RgnKind =3U,Cyc_Absyn_EffKind =4U,Cyc_Absyn_IntKind =5U,Cyc_Absyn_BoolKind =6U,Cyc_Absyn_PtrBndKind =7U};struct Cyc_Absyn_Kind{enum Cyc_Absyn_KindQual kind;enum Cyc_Absyn_AliasQual aliasqual;};struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct{int tag;struct Cyc_Absyn_Kind*f1;};struct Cyc_Absyn_Unknown_kb_Absyn_KindBound_struct{int tag;struct Cyc_Core_Opt*f1;};struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct{int tag;struct Cyc_Core_Opt*f1;struct Cyc_Absyn_Kind*f2;};struct Cyc_Absyn_Tvar{struct _fat_ptr*name;int identity;void*kind;};struct Cyc_Absyn_PtrLoc{unsigned ptr_loc;unsigned rgn_loc;unsigned zt_loc;};struct Cyc_Absyn_PtrAtts{void*rgn;void*nullable;void*bounds;void*zero_term;struct Cyc_Absyn_PtrLoc*ptrloc;};struct Cyc_Absyn_PtrInfo{void*elt_type;struct Cyc_Absyn_Tqual elt_tq;struct Cyc_Absyn_PtrAtts ptr_atts;};struct Cyc_Absyn_VarargInfo{struct _fat_ptr*name;struct Cyc_Absyn_Tqual tq;void*type;int inject;};struct Cyc_Absyn_Star_Absyn_Cap_struct{int tag;};struct Cyc_Absyn_Bot_Absyn_Cap_struct{int tag;};struct Cyc_Absyn_Nat_Absyn_Cap_struct{int tag;struct Cyc_Absyn_Exp*f1;int f2;};struct Cyc_Absyn_RgnEffect{void*name;void*rgn;void*lock;void*unadopt;void*parent;};struct Cyc_Absyn_FnInfo{struct Cyc_List_List*tvars;void*effect;struct Cyc_Absyn_Tqual ret_tqual;void*ret_type;struct Cyc_List_List*args;int c_varargs;struct Cyc_Absyn_VarargInfo*cyc_varargs;struct Cyc_List_List*rgn_po;struct Cyc_List_List*attributes;struct Cyc_Absyn_Exp*requires_clause;struct Cyc_List_List*requires_relns;struct Cyc_Absyn_Exp*ensures_clause;struct Cyc_List_List*ensures_relns;struct Cyc_List_List*ieffect;struct Cyc_List_List*oeffect;};struct Cyc_Absyn_UnknownDatatypeInfo{struct _tuple0*name;int is_extensible;};struct _union_DatatypeInfo_UnknownDatatype{int tag;struct Cyc_Absyn_UnknownDatatypeInfo val;};struct _union_DatatypeInfo_KnownDatatype{int tag;struct Cyc_Absyn_Datatypedecl**val;};union Cyc_Absyn_DatatypeInfo{struct _union_DatatypeInfo_UnknownDatatype UnknownDatatype;struct _union_DatatypeInfo_KnownDatatype KnownDatatype;};struct Cyc_Absyn_UnknownDatatypeFieldInfo{struct _tuple0*datatype_name;struct _tuple0*field_name;int is_extensible;};struct _union_DatatypeFieldInfo_UnknownDatatypefield{int tag;struct Cyc_Absyn_UnknownDatatypeFieldInfo val;};struct _tuple1{struct Cyc_Absyn_Datatypedecl*f1;struct Cyc_Absyn_Datatypefield*f2;};struct _union_DatatypeFieldInfo_KnownDatatypefield{int tag;struct _tuple1 val;};union Cyc_Absyn_DatatypeFieldInfo{struct _union_DatatypeFieldInfo_UnknownDatatypefield UnknownDatatypefield;struct _union_DatatypeFieldInfo_KnownDatatypefield KnownDatatypefield;};struct _tuple2{enum Cyc_Absyn_AggrKind f1;struct _tuple0*f2;struct Cyc_Core_Opt*f3;};struct _union_AggrInfo_UnknownAggr{int tag;struct _tuple2 val;};struct _union_AggrInfo_KnownAggr{int tag;struct Cyc_Absyn_Aggrdecl**val;};union Cyc_Absyn_AggrInfo{struct _union_AggrInfo_UnknownAggr UnknownAggr;struct _union_AggrInfo_KnownAggr KnownAggr;};struct Cyc_Absyn_ArrayInfo{void*elt_type;struct Cyc_Absyn_Tqual tq;struct Cyc_Absyn_Exp*num_elts;void*zero_term;unsigned zt_loc;};struct Cyc_Absyn_Aggr_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Aggrdecl*f1;};struct Cyc_Absyn_Enum_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;};struct Cyc_Absyn_Datatype_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;};struct Cyc_Absyn_TypeDecl{void*r;unsigned loc;};struct Cyc_Absyn_VoidCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_IntCon_Absyn_TyCon_struct{int tag;enum Cyc_Absyn_Sign f1;enum Cyc_Absyn_Size_of f2;};struct Cyc_Absyn_FloatCon_Absyn_TyCon_struct{int tag;int f1;};struct Cyc_Absyn_RgnHandleCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_TagCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_HeapCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_UniqueCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_RefCntCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_AccessCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_JoinCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_RgnsCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_TrueCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_FalseCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_ThinCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_FatCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_EnumCon_Absyn_TyCon_struct{int tag;struct _tuple0*f1;struct Cyc_Absyn_Enumdecl*f2;};struct Cyc_Absyn_AnonEnumCon_Absyn_TyCon_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_BuiltinCon_Absyn_TyCon_struct{int tag;struct _fat_ptr f1;struct Cyc_Absyn_Kind*f2;};struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_DatatypeInfo f1;};struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_DatatypeFieldInfo f1;};struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_AggrInfo f1;};struct Cyc_Absyn_AppType_Absyn_Type_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Evar_Absyn_Type_struct{int tag;struct Cyc_Core_Opt*f1;void*f2;int f3;struct Cyc_Core_Opt*f4;};struct Cyc_Absyn_VarType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Tvar*f1;};struct Cyc_Absyn_PointerType_Absyn_Type_struct{int tag;struct Cyc_Absyn_PtrInfo f1;};struct Cyc_Absyn_ArrayType_Absyn_Type_struct{int tag;struct Cyc_Absyn_ArrayInfo f1;};struct Cyc_Absyn_FnType_Absyn_Type_struct{int tag;struct Cyc_Absyn_FnInfo f1;};struct Cyc_Absyn_TupleType_Absyn_Type_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct{int tag;enum Cyc_Absyn_AggrKind f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_TypedefType_Absyn_Type_struct{int tag;struct _tuple0*f1;struct Cyc_List_List*f2;struct Cyc_Absyn_Typedefdecl*f3;void*f4;};struct Cyc_Absyn_ValueofType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct{int tag;struct Cyc_Absyn_TypeDecl*f1;void**f2;};struct Cyc_Absyn_TypeofType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_NoTypes_Absyn_Funcparams_struct{int tag;struct Cyc_List_List*f1;unsigned f2;};struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct{int tag;struct Cyc_List_List*f1;int f2;struct Cyc_Absyn_VarargInfo*f3;void*f4;struct Cyc_List_List*f5;struct Cyc_Absyn_Exp*f6;struct Cyc_Absyn_Exp*f7;struct Cyc_List_List*f8;struct Cyc_List_List*f9;};
# 437 "absyn.h"
enum Cyc_Absyn_Format_Type{Cyc_Absyn_Printf_ft =0U,Cyc_Absyn_Scanf_ft =1U};struct Cyc_Absyn_Regparm_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Stdcall_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Cdecl_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Fastcall_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Noreturn_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Const_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Aligned_att_Absyn_Attribute_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Packed_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Section_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Nocommon_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Shared_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Unused_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Weak_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Dllimport_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Dllexport_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_instrument_function_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Constructor_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Destructor_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_check_memory_usage_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Format_att_Absyn_Attribute_struct{int tag;enum Cyc_Absyn_Format_Type f1;int f2;int f3;};struct Cyc_Absyn_Initializes_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Noliveunique_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Consume_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Pure_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Mode_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Alias_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Always_inline_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_throw_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Non_null_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Deprecated_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Carray_mod_Absyn_Type_modifier_struct{int tag;void*f1;unsigned f2;};struct Cyc_Absyn_ConstArray_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_Absyn_Exp*f1;void*f2;unsigned f3;};struct Cyc_Absyn_Pointer_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_Absyn_PtrAtts f1;struct Cyc_Absyn_Tqual f2;};struct Cyc_Absyn_Function_mod_Absyn_Type_modifier_struct{int tag;void*f1;};struct Cyc_Absyn_TypeParams_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_List_List*f1;unsigned f2;int f3;};struct Cyc_Absyn_Attributes_mod_Absyn_Type_modifier_struct{int tag;unsigned f1;struct Cyc_List_List*f2;};struct _union_Cnst_Null_c{int tag;int val;};struct _tuple3{enum Cyc_Absyn_Sign f1;char f2;};struct _union_Cnst_Char_c{int tag;struct _tuple3 val;};struct _union_Cnst_Wchar_c{int tag;struct _fat_ptr val;};struct _tuple4{enum Cyc_Absyn_Sign f1;short f2;};struct _union_Cnst_Short_c{int tag;struct _tuple4 val;};struct _tuple5{enum Cyc_Absyn_Sign f1;int f2;};struct _union_Cnst_Int_c{int tag;struct _tuple5 val;};struct _tuple6{enum Cyc_Absyn_Sign f1;long long f2;};struct _union_Cnst_LongLong_c{int tag;struct _tuple6 val;};struct _tuple7{struct _fat_ptr f1;int f2;};struct _union_Cnst_Float_c{int tag;struct _tuple7 val;};struct _union_Cnst_String_c{int tag;struct _fat_ptr val;};struct _union_Cnst_Wstring_c{int tag;struct _fat_ptr val;};union Cyc_Absyn_Cnst{struct _union_Cnst_Null_c Null_c;struct _union_Cnst_Char_c Char_c;struct _union_Cnst_Wchar_c Wchar_c;struct _union_Cnst_Short_c Short_c;struct _union_Cnst_Int_c Int_c;struct _union_Cnst_LongLong_c LongLong_c;struct _union_Cnst_Float_c Float_c;struct _union_Cnst_String_c String_c;struct _union_Cnst_Wstring_c Wstring_c;};
# 522
union Cyc_Absyn_Cnst Cyc_Absyn_Char_c(enum Cyc_Absyn_Sign,char);
# 524
union Cyc_Absyn_Cnst Cyc_Absyn_Short_c(enum Cyc_Absyn_Sign,short);
union Cyc_Absyn_Cnst Cyc_Absyn_Int_c(enum Cyc_Absyn_Sign,int);
union Cyc_Absyn_Cnst Cyc_Absyn_LongLong_c(enum Cyc_Absyn_Sign,long long);
# 532
enum Cyc_Absyn_Primop{Cyc_Absyn_Plus =0U,Cyc_Absyn_Times =1U,Cyc_Absyn_Minus =2U,Cyc_Absyn_Div =3U,Cyc_Absyn_Mod =4U,Cyc_Absyn_Eq =5U,Cyc_Absyn_Neq =6U,Cyc_Absyn_Gt =7U,Cyc_Absyn_Lt =8U,Cyc_Absyn_Gte =9U,Cyc_Absyn_Lte =10U,Cyc_Absyn_Not =11U,Cyc_Absyn_Bitnot =12U,Cyc_Absyn_Bitand =13U,Cyc_Absyn_Bitor =14U,Cyc_Absyn_Bitxor =15U,Cyc_Absyn_Bitlshift =16U,Cyc_Absyn_Bitlrshift =17U,Cyc_Absyn_Numelts =18U};
# 539
enum Cyc_Absyn_Incrementor{Cyc_Absyn_PreInc =0U,Cyc_Absyn_PostInc =1U,Cyc_Absyn_PreDec =2U,Cyc_Absyn_PostDec =3U};struct Cyc_Absyn_VarargCallInfo{int num_varargs;struct Cyc_List_List*injectors;struct Cyc_Absyn_VarargInfo*vai;};struct Cyc_Absyn_StructField_Absyn_OffsetofField_struct{int tag;struct _fat_ptr*f1;};struct Cyc_Absyn_TupleIndex_Absyn_OffsetofField_struct{int tag;unsigned f1;};
# 557
enum Cyc_Absyn_Coercion{Cyc_Absyn_Unknown_coercion =0U,Cyc_Absyn_No_coercion =1U,Cyc_Absyn_Null_to_NonNull =2U,Cyc_Absyn_Other_coercion =3U};struct Cyc_Absyn_MallocInfo{int is_calloc;struct Cyc_Absyn_Exp*rgn;void**elt_type;struct Cyc_Absyn_Exp*num_elts;int fat_result;int inline_call;};struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct{int tag;union Cyc_Absyn_Cnst f1;};struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Pragma_e_Absyn_Raw_exp_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct{int tag;enum Cyc_Absyn_Primop f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_AssignOp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Core_Opt*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_Increment_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;enum Cyc_Absyn_Incrementor f2;};struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;struct Cyc_Absyn_VarargCallInfo*f3;int f4;};struct Cyc_Absyn_Throw_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;int f2;};struct Cyc_Absyn_NoInstantiate_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Instantiate_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_Absyn_Exp*f2;int f3;enum Cyc_Absyn_Coercion f4;};struct Cyc_Absyn_Address_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_New_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Sizeoftype_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Sizeofexp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Offsetof_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Deref_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;int f3;int f4;};struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;int f3;int f4;};struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Tuple_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;};struct _tuple8{struct _fat_ptr*f1;struct Cyc_Absyn_Tqual f2;void*f3;};struct Cyc_Absyn_CompoundLit_e_Absyn_Raw_exp_struct{int tag;struct _tuple8*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Array_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Exp*f2;struct Cyc_Absyn_Exp*f3;int f4;};struct Cyc_Absyn_ComprehensionNoinit_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;void*f2;int f3;};struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct{int tag;struct _tuple0*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;struct Cyc_Absyn_Aggrdecl*f4;};struct Cyc_Absyn_AnonStruct_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Datatype_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;struct Cyc_Absyn_Datatypedecl*f2;struct Cyc_Absyn_Datatypefield*f3;};struct Cyc_Absyn_Enum_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_AnonEnum_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_Spawn_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_MallocInfo f1;};struct Cyc_Absyn_Swap_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_UnresolvedMem_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Core_Opt*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_StmtExp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Stmt*f1;};struct Cyc_Absyn_Tagcheck_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;};struct Cyc_Absyn_Valueof_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Asm_e_Absyn_Raw_exp_struct{int tag;int f1;struct _fat_ptr f2;struct Cyc_List_List*f3;struct Cyc_List_List*f4;struct Cyc_List_List*f5;};struct Cyc_Absyn_Extension_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Exp{void*topt;void*r;unsigned loc;void*annot;};struct Cyc_Absyn_Skip_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Exp_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Seq_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Return_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_IfThenElse_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Stmt*f2;struct Cyc_Absyn_Stmt*f3;};struct _tuple9{struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_While_s_Absyn_Raw_stmt_struct{int tag;struct _tuple9 f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Break_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Continue_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Goto_s_Absyn_Raw_stmt_struct{int tag;struct _fat_ptr*f1;};struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _tuple9 f2;struct _tuple9 f3;struct Cyc_Absyn_Stmt*f4;};struct Cyc_Absyn_Switch_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;void*f3;};struct Cyc_Absyn_Fallthru_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_List_List*f1;struct Cyc_Absyn_Switch_clause**f2;};struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Decl*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Label_s_Absyn_Raw_stmt_struct{int tag;struct _fat_ptr*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Do_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct _tuple9 f2;};struct Cyc_Absyn_TryCatch_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct Cyc_List_List*f2;void*f3;};struct Cyc_Absyn_Stmt{void*r;unsigned loc;void*annot;};struct Cyc_Absyn_Wild_p_Absyn_Raw_pat_struct{int tag;};struct Cyc_Absyn_Var_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Pat*f2;};struct Cyc_Absyn_AliasVar_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;};struct Cyc_Absyn_Reference_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Pat*f2;};struct Cyc_Absyn_TagInt_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;};struct Cyc_Absyn_Tuple_p_Absyn_Raw_pat_struct{int tag;struct Cyc_List_List*f1;int f2;};struct Cyc_Absyn_Pointer_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Pat*f1;};struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct{int tag;union Cyc_Absyn_AggrInfo*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;int f4;};struct Cyc_Absyn_Datatype_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;struct Cyc_Absyn_Datatypefield*f2;struct Cyc_List_List*f3;int f4;};struct Cyc_Absyn_Null_p_Absyn_Raw_pat_struct{int tag;};struct Cyc_Absyn_Int_p_Absyn_Raw_pat_struct{int tag;enum Cyc_Absyn_Sign f1;int f2;};struct Cyc_Absyn_Char_p_Absyn_Raw_pat_struct{int tag;char f1;};struct Cyc_Absyn_Float_p_Absyn_Raw_pat_struct{int tag;struct _fat_ptr f1;int f2;};struct Cyc_Absyn_Enum_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_AnonEnum_p_Absyn_Raw_pat_struct{int tag;void*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_UnknownId_p_Absyn_Raw_pat_struct{int tag;struct _tuple0*f1;};struct Cyc_Absyn_UnknownCall_p_Absyn_Raw_pat_struct{int tag;struct _tuple0*f1;struct Cyc_List_List*f2;int f3;};struct Cyc_Absyn_Exp_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Pat{void*r;void*topt;unsigned loc;};struct Cyc_Absyn_Switch_clause{struct Cyc_Absyn_Pat*pattern;struct Cyc_Core_Opt*pat_vars;struct Cyc_Absyn_Exp*where_clause;struct Cyc_Absyn_Stmt*body;unsigned loc;};struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct{int tag;struct _tuple0*f1;};struct Cyc_Absyn_Global_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Funname_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Fndecl*f1;};struct Cyc_Absyn_Param_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Local_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Pat_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Vardecl{enum Cyc_Absyn_Scope sc;struct _tuple0*name;unsigned varloc;struct Cyc_Absyn_Tqual tq;void*type;struct Cyc_Absyn_Exp*initializer;void*rgn;struct Cyc_List_List*attributes;int escapes;};struct Cyc_Absyn_Fndecl{enum Cyc_Absyn_Scope sc;int is_inline;struct _tuple0*name;struct Cyc_Absyn_Stmt*body;struct Cyc_Absyn_FnInfo i;void*cached_type;struct Cyc_Core_Opt*param_vardecls;struct Cyc_Absyn_Vardecl*fn_vardecl;};struct Cyc_Absyn_Aggrfield{struct _fat_ptr*name;struct Cyc_Absyn_Tqual tq;void*type;struct Cyc_Absyn_Exp*width;struct Cyc_List_List*attributes;struct Cyc_Absyn_Exp*requires_clause;};struct Cyc_Absyn_AggrdeclImpl{struct Cyc_List_List*exist_vars;struct Cyc_List_List*rgn_po;struct Cyc_List_List*fields;int tagged;};struct Cyc_Absyn_Aggrdecl{enum Cyc_Absyn_AggrKind kind;enum Cyc_Absyn_Scope sc;struct _tuple0*name;struct Cyc_List_List*tvs;struct Cyc_Absyn_AggrdeclImpl*impl;struct Cyc_List_List*attributes;int expected_mem_kind;};struct Cyc_Absyn_Datatypefield{struct _tuple0*name;struct Cyc_List_List*typs;unsigned loc;enum Cyc_Absyn_Scope sc;};struct Cyc_Absyn_Datatypedecl{enum Cyc_Absyn_Scope sc;struct _tuple0*name;struct Cyc_List_List*tvs;struct Cyc_Core_Opt*fields;int is_extensible;};struct Cyc_Absyn_Enumfield{struct _tuple0*name;struct Cyc_Absyn_Exp*tag;unsigned loc;};struct Cyc_Absyn_Enumdecl{enum Cyc_Absyn_Scope sc;struct _tuple0*name;struct Cyc_Core_Opt*fields;};struct Cyc_Absyn_Typedefdecl{struct _tuple0*name;struct Cyc_Absyn_Tqual tq;struct Cyc_List_List*tvs;struct Cyc_Core_Opt*kind;void*defn;struct Cyc_List_List*atts;int extern_c;};struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Fn_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Fndecl*f1;};struct Cyc_Absyn_Let_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Pat*f1;struct Cyc_Core_Opt*f2;struct Cyc_Absyn_Exp*f3;void*f4;};struct Cyc_Absyn_Letv_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_Region_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_Aggr_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Aggrdecl*f1;};struct Cyc_Absyn_Datatype_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;};struct Cyc_Absyn_Enum_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;};struct Cyc_Absyn_Typedef_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Typedefdecl*f1;};struct Cyc_Absyn_Namespace_d_Absyn_Raw_decl_struct{int tag;struct _fat_ptr*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Using_d_Absyn_Raw_decl_struct{int tag;struct _tuple0*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_ExternC_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;};struct _tuple10{unsigned f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_ExternCinclude_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;struct _tuple10*f4;};struct Cyc_Absyn_Porton_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Portoff_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Tempeston_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Tempestoff_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Decl{void*r;unsigned loc;};struct Cyc_Absyn_ArrayElement_Absyn_Designator_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_FieldName_Absyn_Designator_struct{int tag;struct _fat_ptr*f1;};extern char Cyc_Absyn_EmptyAnnot[11U];struct Cyc_Absyn_EmptyAnnot_Absyn_AbsynAnnot_struct{char*tag;};
# 938 "absyn.h"
extern void*Cyc_Absyn_uint_type;
# 1015
struct Cyc_Absyn_Exp*Cyc_Absyn_const_exp(union Cyc_Absyn_Cnst,unsigned);struct Cyc_Absynpp_Params{int expand_typedefs;int qvar_to_Cids;int add_cyc_prefix;int to_VC;int decls_first;int rewrite_temp_tvars;int print_all_tvars;int print_all_kinds;int print_all_effects;int print_using_stmts;int print_externC_stmts;int print_full_evars;int print_zeroterm;int generate_line_directives;int use_curr_namespace;struct Cyc_List_List*curr_namespace;};
# 62 "absynpp.h"
struct _fat_ptr Cyc_Absynpp_typ2string(void*);
# 67
struct _fat_ptr Cyc_Absynpp_exp2string(struct Cyc_Absyn_Exp*);
# 29 "unify.h"
int Cyc_Unify_unify(void*,void*);struct _union_RelnOp_RConst{int tag;unsigned val;};struct _union_RelnOp_RVar{int tag;struct Cyc_Absyn_Vardecl*val;};struct _union_RelnOp_RNumelts{int tag;struct Cyc_Absyn_Vardecl*val;};struct _union_RelnOp_RType{int tag;void*val;};struct _union_RelnOp_RParam{int tag;unsigned val;};struct _union_RelnOp_RParamNumelts{int tag;unsigned val;};struct _union_RelnOp_RReturn{int tag;unsigned val;};union Cyc_Relations_RelnOp{struct _union_RelnOp_RConst RConst;struct _union_RelnOp_RVar RVar;struct _union_RelnOp_RNumelts RNumelts;struct _union_RelnOp_RType RType;struct _union_RelnOp_RParam RParam;struct _union_RelnOp_RParamNumelts RParamNumelts;struct _union_RelnOp_RReturn RReturn;};
# 50 "relations-ap.h"
enum Cyc_Relations_Relation{Cyc_Relations_Req =0U,Cyc_Relations_Rneq =1U,Cyc_Relations_Rlte =2U,Cyc_Relations_Rlt =3U};struct Cyc_Relations_Reln{union Cyc_Relations_RelnOp rop1;enum Cyc_Relations_Relation relation;union Cyc_Relations_RelnOp rop2;};struct Cyc_Iter_Iter{void*env;int(*next)(void*env,void*dest);};struct Cyc_Dict_T;struct Cyc_Dict_Dict{int(*rel)(void*,void*);struct _RegionHandle*r;const struct Cyc_Dict_T*t;};extern char Cyc_Dict_Present[8U];struct Cyc_Dict_Present_exn_struct{char*tag;};extern char Cyc_Dict_Absent[7U];struct Cyc_Dict_Absent_exn_struct{char*tag;};struct Cyc_RgnOrder_RgnPO;extern char Cyc_Tcenv_Env_error[10U];struct Cyc_Tcenv_Env_error_exn_struct{char*tag;};struct Cyc_Tcenv_Genv{struct Cyc_Dict_Dict aggrdecls;struct Cyc_Dict_Dict datatypedecls;struct Cyc_Dict_Dict enumdecls;struct Cyc_Dict_Dict typedefs;struct Cyc_Dict_Dict ordinaries;};struct Cyc_Tcenv_Fenv;struct Cyc_Tcenv_Tenv{struct Cyc_List_List*ns;struct Cyc_Tcenv_Genv*ae;struct Cyc_Tcenv_Fenv*le;int allow_valueof: 1;int in_extern_c_include: 1;int in_tempest: 1;int tempest_generalize: 1;};
# 88 "tcenv.h"
enum Cyc_Tcenv_NewStatus{Cyc_Tcenv_NoneNew =0U,Cyc_Tcenv_InNew =1U,Cyc_Tcenv_InNewAggr =2U};
# 30 "tcutil.h"
void*Cyc_Tcutil_impos(struct _fat_ptr,struct _fat_ptr);
void Cyc_Tcutil_terr(unsigned,struct _fat_ptr,struct _fat_ptr);
# 105
struct Cyc_Absyn_Kind*Cyc_Tcutil_tvar_kind(struct Cyc_Absyn_Tvar*,struct Cyc_Absyn_Kind*def);
# 107
void*Cyc_Tcutil_compress(void*);
# 137
extern struct Cyc_Absyn_Kind Cyc_Tcutil_bk;
# 182
int Cyc_Tcutil_typecmp(void*,void*);
# 187
void*Cyc_Tcutil_rsubstitute(struct _RegionHandle*,struct Cyc_List_List*,void*);struct _tuple11{unsigned f1;int f2;};
# 28 "evexp.h"
struct _tuple11 Cyc_Evexp_eval_const_uint_exp(struct Cyc_Absyn_Exp*e);
# 32
int Cyc_Evexp_c_can_eval(struct Cyc_Absyn_Exp*e);
# 41 "evexp.h"
int Cyc_Evexp_same_const_exp(struct Cyc_Absyn_Exp*e1,struct Cyc_Absyn_Exp*e2);
# 45
int Cyc_Evexp_const_exp_cmp(struct Cyc_Absyn_Exp*e1,struct Cyc_Absyn_Exp*e2);
# 48
int Cyc_Evexp_okay_szofarg(void*t);struct _tuple12{union Cyc_Absyn_Cnst f1;int f2;};
# 35 "evexp.cyc"
static struct _tuple12 Cyc_Evexp_eval_const_exp(struct Cyc_Absyn_Exp*);
# 37
static union Cyc_Absyn_Cnst Cyc_Evexp_promote_const(union Cyc_Absyn_Cnst cn){
union Cyc_Absyn_Cnst _tmp0=cn;short _tmp2;enum Cyc_Absyn_Sign _tmp1;char _tmp4;enum Cyc_Absyn_Sign _tmp3;switch((_tmp0.Short_c).tag){case 2U: _LL1: _tmp3=((_tmp0.Char_c).val).f1;_tmp4=((_tmp0.Char_c).val).f2;_LL2: {enum Cyc_Absyn_Sign sn=_tmp3;char c=_tmp4;
return Cyc_Absyn_Int_c(sn,(int)c);}case 4U: _LL3: _tmp1=((_tmp0.Short_c).val).f1;_tmp2=((_tmp0.Short_c).val).f2;_LL4: {enum Cyc_Absyn_Sign sn=_tmp1;short s=_tmp2;
# 41
return Cyc_Absyn_Int_c(sn,(int)s);}default: _LL5: _LL6:
 return cn;}_LL0:;}
# 37
struct _tuple11 Cyc_Evexp_eval_const_uint_exp(struct Cyc_Absyn_Exp*e){
# 47
struct _tuple12 _stmttmp0=Cyc_Evexp_eval_const_exp(e);int _tmp6;union Cyc_Absyn_Cnst _tmp5;_LL1: _tmp5=_stmttmp0.f1;_tmp6=_stmttmp0.f2;_LL2: {union Cyc_Absyn_Cnst cn=_tmp5;int known=_tmp6;
if(!known)
return({struct _tuple11 _tmpDF;_tmpDF.f1=0U,_tmpDF.f2=0;_tmpDF;});{
# 48
union Cyc_Absyn_Cnst _stmttmp1=
# 50
Cyc_Evexp_promote_const(cn);
# 48
union Cyc_Absyn_Cnst _tmp7=_stmttmp1;long long _tmp8;int _tmp9;switch((_tmp7.Null_c).tag){case 5U: _LL4: _tmp9=((_tmp7.Int_c).val).f2;_LL5: {int i=_tmp9;
# 51
return({struct _tuple11 _tmpE0;_tmpE0.f1=(unsigned)i,_tmpE0.f2=1;_tmpE0;});}case 6U: _LL6: _tmp8=((_tmp7.LongLong_c).val).f2;_LL7: {long long x=_tmp8;
# 54
unsigned long long y=(unsigned long long)x >> (unsigned long long)32;
if(y != (unsigned long long)-1 && y != (unsigned long long)0)
return({struct _tuple11 _tmpE1;_tmpE1.f1=0U,_tmpE1.f2=0;_tmpE1;});else{
return({struct _tuple11 _tmpE2;_tmpE2.f1=(unsigned)x,_tmpE2.f2=1;_tmpE2;});}}case 7U: _LL8: _LL9:
 return({struct _tuple11 _tmpE3;_tmpE3.f1=0U,_tmpE3.f2=0;_tmpE3;});case 1U: _LLA: _LLB:
 return({struct _tuple11 _tmpE4;_tmpE4.f1=0U,_tmpE4.f2=1;_tmpE4;});default: _LLC: _LLD:
# 61
({struct Cyc_String_pa_PrintArg_struct _tmpC=({struct Cyc_String_pa_PrintArg_struct _tmpE5;_tmpE5.tag=0U,({struct _fat_ptr _tmp10C=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_exp2string(e));_tmpE5.f1=_tmp10C;});_tmpE5;});void*_tmpA[1U];_tmpA[0]=& _tmpC;({unsigned _tmp10E=e->loc;struct _fat_ptr _tmp10D=({const char*_tmpB="expecting unsigned int found %s";_tag_fat(_tmpB,sizeof(char),32U);});Cyc_Tcutil_terr(_tmp10E,_tmp10D,_tag_fat(_tmpA,sizeof(void*),1U));});});return({struct _tuple11 _tmpE6;_tmpE6.f1=0U,_tmpE6.f2=1;_tmpE6;});}_LL3:;}}}struct _tuple13{int f1;int f2;};
# 37
static struct _tuple13 Cyc_Evexp_eval_const_bool_exp(struct Cyc_Absyn_Exp*e){
# 66
struct _tuple12 _stmttmp2=Cyc_Evexp_eval_const_exp(e);int _tmpE;union Cyc_Absyn_Cnst _tmpD;_LL1: _tmpD=_stmttmp2.f1;_tmpE=_stmttmp2.f2;_LL2: {union Cyc_Absyn_Cnst cn=_tmpD;int known=_tmpE;
if(!known)
return({struct _tuple13 _tmpE7;_tmpE7.f1=0,_tmpE7.f2=0;_tmpE7;});{
# 67
union Cyc_Absyn_Cnst _stmttmp3=
# 69
Cyc_Evexp_promote_const(cn);
# 67
union Cyc_Absyn_Cnst _tmpF=_stmttmp3;long long _tmp10;int _tmp11;switch((_tmpF.Float_c).tag){case 5U: _LL4: _tmp11=((_tmpF.Int_c).val).f2;_LL5: {int b=_tmp11;
# 70
return({struct _tuple13 _tmpE8;_tmpE8.f1=b != 0,_tmpE8.f2=1;_tmpE8;});}case 6U: _LL6: _tmp10=((_tmpF.LongLong_c).val).f2;_LL7: {long long b=_tmp10;
return({struct _tuple13 _tmpE9;_tmpE9.f1=b != (long long)0,_tmpE9.f2=1;_tmpE9;});}case 1U: _LL8: _LL9:
 return({struct _tuple13 _tmpEA;_tmpEA.f1=0,_tmpEA.f2=0;_tmpEA;});case 7U: _LLA: _LLB:
 return({struct _tuple13 _tmpEB;_tmpEB.f1=0,_tmpEB.f2=1;_tmpEB;});default: _LLC: _LLD:
({void*_tmp12=0U;({unsigned _tmp110=e->loc;struct _fat_ptr _tmp10F=({const char*_tmp13="expecting bool";_tag_fat(_tmp13,sizeof(char),15U);});Cyc_Tcutil_terr(_tmp110,_tmp10F,_tag_fat(_tmp12,sizeof(void*),0U));});});return({struct _tuple13 _tmpEC;_tmpEC.f1=0,_tmpEC.f2=0;_tmpEC;});}_LL3:;}}}struct _tuple14{enum Cyc_Absyn_Primop f1;union Cyc_Absyn_Cnst f2;};
# 37
static struct _tuple12 Cyc_Evexp_eval_const_unprimop(enum Cyc_Absyn_Primop p,struct Cyc_Absyn_Exp*e){
# 79
struct _tuple12 _stmttmp4=Cyc_Evexp_eval_const_exp(e);int _tmp15;union Cyc_Absyn_Cnst _tmp14;_LL1: _tmp14=_stmttmp4.f1;_tmp15=_stmttmp4.f2;_LL2: {union Cyc_Absyn_Cnst cn=_tmp14;int known=_tmp15;
if(!known)
return({struct _tuple12 _tmpED;_tmpED.f1=cn,_tmpED.f2=0;_tmpED;});
# 80
{struct _tuple14 _stmttmp5=({struct _tuple14 _tmpEF;_tmpEF.f1=p,_tmpEF.f2=cn;_tmpEF;});struct _tuple14 _tmp16=_stmttmp5;long long _tmp17;char _tmp18;short _tmp19;int _tmp1A;long long _tmp1C;enum Cyc_Absyn_Sign _tmp1B;char _tmp1E;enum Cyc_Absyn_Sign _tmp1D;short _tmp20;enum Cyc_Absyn_Sign _tmp1F;int _tmp22;enum Cyc_Absyn_Sign _tmp21;long long _tmp24;enum Cyc_Absyn_Sign _tmp23;char _tmp26;enum Cyc_Absyn_Sign _tmp25;short _tmp28;enum Cyc_Absyn_Sign _tmp27;int _tmp2A;enum Cyc_Absyn_Sign _tmp29;switch(_tmp16.f1){case Cyc_Absyn_Plus: _LL4: _LL5:
# 83
 goto _LL3;case Cyc_Absyn_Minus: switch(((_tmp16.f2).LongLong_c).tag){case 5U: _LL6: _tmp29=(((_tmp16.f2).Int_c).val).f1;_tmp2A=(((_tmp16.f2).Int_c).val).f2;_LL7: {enum Cyc_Absyn_Sign s=_tmp29;int i=_tmp2A;
cn=Cyc_Absyn_Int_c(Cyc_Absyn_Signed,- i);goto _LL3;}case 4U: _LL8: _tmp27=(((_tmp16.f2).Short_c).val).f1;_tmp28=(((_tmp16.f2).Short_c).val).f2;_LL9: {enum Cyc_Absyn_Sign s=_tmp27;short i=_tmp28;
cn=Cyc_Absyn_Short_c(Cyc_Absyn_Signed,- i);goto _LL3;}case 2U: _LLA: _tmp25=(((_tmp16.f2).Char_c).val).f1;_tmp26=(((_tmp16.f2).Char_c).val).f2;_LLB: {enum Cyc_Absyn_Sign s=_tmp25;char i=_tmp26;
cn=Cyc_Absyn_Char_c(Cyc_Absyn_Signed,- i);goto _LL3;}case 6U: _LLC: _tmp23=(((_tmp16.f2).LongLong_c).val).f1;_tmp24=(((_tmp16.f2).LongLong_c).val).f2;_LLD: {enum Cyc_Absyn_Sign s=_tmp23;long long i=_tmp24;
cn=Cyc_Absyn_LongLong_c(Cyc_Absyn_Signed,- i);goto _LL3;}default: goto _LL20;}case Cyc_Absyn_Bitnot: switch(((_tmp16.f2).LongLong_c).tag){case 5U: _LLE: _tmp21=(((_tmp16.f2).Int_c).val).f1;_tmp22=(((_tmp16.f2).Int_c).val).f2;_LLF: {enum Cyc_Absyn_Sign s=_tmp21;int i=_tmp22;
cn=Cyc_Absyn_Int_c(Cyc_Absyn_Unsigned,~ i);goto _LL3;}case 4U: _LL10: _tmp1F=(((_tmp16.f2).Short_c).val).f1;_tmp20=(((_tmp16.f2).Short_c).val).f2;_LL11: {enum Cyc_Absyn_Sign s=_tmp1F;short i=_tmp20;
cn=Cyc_Absyn_Short_c(Cyc_Absyn_Unsigned,~ i);goto _LL3;}case 2U: _LL12: _tmp1D=(((_tmp16.f2).Char_c).val).f1;_tmp1E=(((_tmp16.f2).Char_c).val).f2;_LL13: {enum Cyc_Absyn_Sign s=_tmp1D;char i=_tmp1E;
cn=Cyc_Absyn_Char_c(Cyc_Absyn_Unsigned,~ i);goto _LL3;}case 6U: _LL14: _tmp1B=(((_tmp16.f2).LongLong_c).val).f1;_tmp1C=(((_tmp16.f2).LongLong_c).val).f2;_LL15: {enum Cyc_Absyn_Sign s=_tmp1B;long long i=_tmp1C;
cn=Cyc_Absyn_LongLong_c(Cyc_Absyn_Unsigned,~ i);goto _LL3;}default: goto _LL20;}case Cyc_Absyn_Not: switch(((_tmp16.f2).Null_c).tag){case 5U: _LL16: _tmp1A=(((_tmp16.f2).Int_c).val).f2;_LL17: {int i=_tmp1A;
cn=Cyc_Absyn_Int_c(Cyc_Absyn_Signed,i == 0?1: 0);goto _LL3;}case 4U: _LL18: _tmp19=(((_tmp16.f2).Short_c).val).f2;_LL19: {short i=_tmp19;
cn=Cyc_Absyn_Short_c(Cyc_Absyn_Signed,(int)i == 0?1: 0);goto _LL3;}case 2U: _LL1A: _tmp18=(((_tmp16.f2).Char_c).val).f2;_LL1B: {char i=_tmp18;
cn=Cyc_Absyn_Char_c(Cyc_Absyn_Signed,(int)i == 0?'\001':'\000');goto _LL3;}case 6U: _LL1C: _tmp17=(((_tmp16.f2).LongLong_c).val).f2;_LL1D: {long long i=_tmp17;
cn=Cyc_Absyn_LongLong_c(Cyc_Absyn_Signed,(long long)(i == (long long)0?1: 0));goto _LL3;}case 1U: _LL1E: _LL1F:
 cn=Cyc_Absyn_Int_c(Cyc_Absyn_Signed,1);goto _LL3;default: goto _LL20;}default: _LL20: _LL21:
 return({struct _tuple12 _tmpEE;_tmpEE.f1=cn,_tmpEE.f2=0;_tmpEE;});}_LL3:;}
# 99
return({struct _tuple12 _tmpF0;_tmpF0.f1=cn,_tmpF0.f2=1;_tmpF0;});}}struct _tuple15{enum Cyc_Absyn_Primop f1;int f2;};
# 37
static struct _tuple12 Cyc_Evexp_eval_const_binprimop(enum Cyc_Absyn_Primop p,struct Cyc_Absyn_Exp*e1,struct Cyc_Absyn_Exp*e2){
# 104
struct _tuple12 _stmttmp6=Cyc_Evexp_eval_const_exp(e1);int _tmp2C;union Cyc_Absyn_Cnst _tmp2B;_LL1: _tmp2B=_stmttmp6.f1;_tmp2C=_stmttmp6.f2;_LL2: {union Cyc_Absyn_Cnst cn1=_tmp2B;int known1=_tmp2C;
struct _tuple12 _stmttmp7=Cyc_Evexp_eval_const_exp(e2);int _tmp2E;union Cyc_Absyn_Cnst _tmp2D;_LL4: _tmp2D=_stmttmp7.f1;_tmp2E=_stmttmp7.f2;_LL5: {union Cyc_Absyn_Cnst cn2=_tmp2D;int known2=_tmp2E;
if(!known1 || !known2)
return({struct _tuple12 _tmpF1;_tmpF1.f1=cn1,_tmpF1.f2=0;_tmpF1;});
# 106
cn1=
# 108
Cyc_Evexp_promote_const(cn1);
cn2=Cyc_Evexp_promote_const(cn2);{
enum Cyc_Absyn_Sign s1;enum Cyc_Absyn_Sign s2;
int i1;int i2;
{union Cyc_Absyn_Cnst _tmp2F=cn1;int _tmp31;enum Cyc_Absyn_Sign _tmp30;if((_tmp2F.Int_c).tag == 5){_LL7: _tmp30=((_tmp2F.Int_c).val).f1;_tmp31=((_tmp2F.Int_c).val).f2;_LL8: {enum Cyc_Absyn_Sign x=_tmp30;int y=_tmp31;
s1=x;i1=y;goto _LL6;}}else{_LL9: _LLA:
 return({struct _tuple12 _tmpF2;_tmpF2.f1=cn1,_tmpF2.f2=0;_tmpF2;});}_LL6:;}
# 116
{union Cyc_Absyn_Cnst _tmp32=cn2;int _tmp34;enum Cyc_Absyn_Sign _tmp33;if((_tmp32.Int_c).tag == 5){_LLC: _tmp33=((_tmp32.Int_c).val).f1;_tmp34=((_tmp32.Int_c).val).f2;_LLD: {enum Cyc_Absyn_Sign x=_tmp33;int y=_tmp34;
s2=x;i2=y;goto _LLB;}}else{_LLE: _LLF:
 return({struct _tuple12 _tmpF3;_tmpF3.f1=cn1,_tmpF3.f2=0;_tmpF3;});}_LLB:;}
# 120
{enum Cyc_Absyn_Primop _tmp35=p;switch(_tmp35){case Cyc_Absyn_Div: _LL11: _LL12:
 goto _LL14;case Cyc_Absyn_Mod: _LL13: _LL14:
# 123
 if(i2 == 0){
({void*_tmp36=0U;({unsigned _tmp112=e2->loc;struct _fat_ptr _tmp111=({const char*_tmp37="division by zero in constant expression";_tag_fat(_tmp37,sizeof(char),40U);});Cyc_Tcutil_terr(_tmp112,_tmp111,_tag_fat(_tmp36,sizeof(void*),0U));});});
return({struct _tuple12 _tmpF4;_tmpF4.f1=cn1,_tmpF4.f2=1;_tmpF4;});}
# 123
goto _LL10;default: _LL15: _LL16:
# 128
 goto _LL10;}_LL10:;}{
# 130
int has_u_arg=(int)s1 == (int)1U ||(int)s2 == (int)Cyc_Absyn_Unsigned;
unsigned u1=(unsigned)i1;
unsigned u2=(unsigned)i2;
int i3=0;
unsigned u3=0U;
int b3=1;
int use_i3=0;
int use_u3=0;
int use_b3=0;
{struct _tuple15 _stmttmp8=({struct _tuple15 _tmpF6;_tmpF6.f1=p,_tmpF6.f2=has_u_arg;_tmpF6;});struct _tuple15 _tmp38=_stmttmp8;switch(_tmp38.f1){case Cyc_Absyn_Plus: switch(_tmp38.f2){case 0U: _LL18: _LL19:
 i3=i1 + i2;use_i3=1;goto _LL17;case 1U: _LL22: _LL23:
# 145
 u3=u1 + u2;use_u3=1;goto _LL17;default: goto _LL4A;}case Cyc_Absyn_Times: switch(_tmp38.f2){case 0U: _LL1A: _LL1B:
# 141
 i3=i1 * i2;use_i3=1;goto _LL17;case 1U: _LL24: _LL25:
# 146
 u3=u1 * u2;use_u3=1;goto _LL17;default: goto _LL4A;}case Cyc_Absyn_Minus: switch(_tmp38.f2){case 0U: _LL1C: _LL1D:
# 142
 i3=i1 - i2;use_i3=1;goto _LL17;case 1U: _LL26: _LL27:
# 147
 u3=u1 - u2;use_u3=1;goto _LL17;default: goto _LL4A;}case Cyc_Absyn_Div: switch(_tmp38.f2){case 0U: _LL1E: _LL1F:
# 143
 i3=i1 / i2;use_i3=1;goto _LL17;case 1U: _LL28: _LL29:
# 148
 u3=u1 / u2;use_u3=1;goto _LL17;default: goto _LL4A;}case Cyc_Absyn_Mod: switch(_tmp38.f2){case 0U: _LL20: _LL21:
# 144
 i3=i1 % i2;use_i3=1;goto _LL17;case 1U: _LL2A: _LL2B:
# 149
 u3=u1 % u2;use_u3=1;goto _LL17;default: goto _LL4A;}case Cyc_Absyn_Eq: _LL2C: _LL2D:
 b3=i1 == i2;use_b3=1;goto _LL17;case Cyc_Absyn_Neq: _LL2E: _LL2F:
 b3=i1 != i2;use_b3=1;goto _LL17;case Cyc_Absyn_Gt: switch(_tmp38.f2){case 0U: _LL30: _LL31:
 b3=i1 > i2;use_b3=1;goto _LL17;case 1U: _LL38: _LL39:
# 156
 b3=u1 > u2;use_b3=1;goto _LL17;default: goto _LL4A;}case Cyc_Absyn_Lt: switch(_tmp38.f2){case 0U: _LL32: _LL33:
# 153
 b3=i1 < i2;use_b3=1;goto _LL17;case 1U: _LL3A: _LL3B:
# 157
 b3=u1 < u2;use_b3=1;goto _LL17;default: goto _LL4A;}case Cyc_Absyn_Gte: switch(_tmp38.f2){case 0U: _LL34: _LL35:
# 154
 b3=i1 >= i2;use_b3=1;goto _LL17;case 1U: _LL3C: _LL3D:
# 158
 b3=u1 >= u2;use_b3=1;goto _LL17;default: goto _LL4A;}case Cyc_Absyn_Lte: switch(_tmp38.f2){case 0U: _LL36: _LL37:
# 155
 b3=i1 <= i2;use_b3=1;goto _LL17;case 1U: _LL3E: _LL3F:
# 159
 b3=u1 <= u2;use_b3=1;goto _LL17;default: goto _LL4A;}case Cyc_Absyn_Bitand: _LL40: _LL41:
 u3=u1 & u2;use_u3=1;goto _LL17;case Cyc_Absyn_Bitor: _LL42: _LL43:
 u3=u1 | u2;use_u3=1;goto _LL17;case Cyc_Absyn_Bitxor: _LL44: _LL45:
 u3=u1 ^ u2;use_u3=1;goto _LL17;case Cyc_Absyn_Bitlshift: _LL46: _LL47:
 u3=u1 << u2;use_u3=1;goto _LL17;case Cyc_Absyn_Bitlrshift: _LL48: _LL49:
 u3=u1 >> u2;use_u3=1;goto _LL17;default: _LL4A: _LL4B:
({void*_tmp39=0U;({unsigned _tmp114=e1->loc;struct _fat_ptr _tmp113=({const char*_tmp3A="bad constant expression";_tag_fat(_tmp3A,sizeof(char),24U);});Cyc_Tcutil_terr(_tmp114,_tmp113,_tag_fat(_tmp39,sizeof(void*),0U));});});return({struct _tuple12 _tmpF5;_tmpF5.f1=cn1,_tmpF5.f2=1;_tmpF5;});}_LL17:;}
# 167
if(use_i3)return({struct _tuple12 _tmpF7;({union Cyc_Absyn_Cnst _tmp115=Cyc_Absyn_Int_c(Cyc_Absyn_Signed,i3);_tmpF7.f1=_tmp115;}),_tmpF7.f2=1;_tmpF7;});if(use_u3)
return({struct _tuple12 _tmpF8;({union Cyc_Absyn_Cnst _tmp116=Cyc_Absyn_Int_c(Cyc_Absyn_Unsigned,(int)u3);_tmpF8.f1=_tmp116;}),_tmpF8.f2=1;_tmpF8;});
# 167
if(use_b3)
# 169
return({struct _tuple12 _tmpF9;({union Cyc_Absyn_Cnst _tmp117=Cyc_Absyn_Int_c(Cyc_Absyn_Signed,b3?1: 0);_tmpF9.f1=_tmp117;}),_tmpF9.f2=1;_tmpF9;});
# 167
({void*_tmp3B=0U;({int(*_tmp119)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp3D)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp3D;});struct _fat_ptr _tmp118=({const char*_tmp3C="Evexp::eval_const_binop";_tag_fat(_tmp3C,sizeof(char),24U);});_tmp119(_tmp118,_tag_fat(_tmp3B,sizeof(void*),0U));});});}}}}}struct _tuple16{void*f1;union Cyc_Absyn_Cnst f2;};
# 37
static struct _tuple12 Cyc_Evexp_eval_const_exp(struct Cyc_Absyn_Exp*e){
# 177
struct _tuple12 ans;
{void*_stmttmp9=e->r;void*_tmp3E=_stmttmp9;struct Cyc_Absyn_Enumfield*_tmp3F;struct Cyc_Absyn_Enumfield*_tmp40;struct Cyc_Absyn_Exp*_tmp42;void*_tmp41;void*_tmp43;struct Cyc_List_List*_tmp45;enum Cyc_Absyn_Primop _tmp44;struct Cyc_Absyn_Exp*_tmp47;struct Cyc_Absyn_Exp*_tmp46;struct Cyc_Absyn_Exp*_tmp49;struct Cyc_Absyn_Exp*_tmp48;struct Cyc_Absyn_Exp*_tmp4C;struct Cyc_Absyn_Exp*_tmp4B;struct Cyc_Absyn_Exp*_tmp4A;union Cyc_Absyn_Cnst _tmp4D;switch(*((int*)_tmp3E)){case 1U: _LL1: _LL2:
# 180
 return({struct _tuple12 _tmpFA;({union Cyc_Absyn_Cnst _tmp11A=Cyc_Absyn_Int_c(Cyc_Absyn_Signed,0);_tmpFA.f1=_tmp11A;}),_tmpFA.f2=0;_tmpFA;});case 0U: _LL3: _tmp4D=((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmp3E)->f1;_LL4: {union Cyc_Absyn_Cnst c=_tmp4D;
return({struct _tuple12 _tmpFB;_tmpFB.f1=c,_tmpFB.f2=1;_tmpFB;});}case 2U: _LL5: _LL6:
 return({struct _tuple12 _tmpFC;({union Cyc_Absyn_Cnst _tmp11B=Cyc_Absyn_Int_c(Cyc_Absyn_Signed,0);_tmpFC.f1=_tmp11B;}),_tmpFC.f2=1;_tmpFC;});case 6U: _LL7: _tmp4A=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmp3E)->f1;_tmp4B=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmp3E)->f2;_tmp4C=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmp3E)->f3;_LL8: {struct Cyc_Absyn_Exp*e1=_tmp4A;struct Cyc_Absyn_Exp*e2=_tmp4B;struct Cyc_Absyn_Exp*e3=_tmp4C;
# 184
struct _tuple13 _stmttmpA=Cyc_Evexp_eval_const_bool_exp(e1);int _tmp4F;int _tmp4E;_LL20: _tmp4E=_stmttmpA.f1;_tmp4F=_stmttmpA.f2;_LL21: {int bool1=_tmp4E;int known1=_tmp4F;
if(!known1){
Cyc_Evexp_eval_const_exp(e2);
Cyc_Evexp_eval_const_exp(e3);
return({struct _tuple12 _tmpFD;({union Cyc_Absyn_Cnst _tmp11C=Cyc_Absyn_Int_c(Cyc_Absyn_Signed,0);_tmpFD.f1=_tmp11C;}),_tmpFD.f2=0;_tmpFD;});}
# 185
ans=
# 190
bool1?Cyc_Evexp_eval_const_exp(e2): Cyc_Evexp_eval_const_exp(e3);
goto _LL0;}}case 7U: _LL9: _tmp48=((struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct*)_tmp3E)->f1;_tmp49=((struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct*)_tmp3E)->f2;_LLA: {struct Cyc_Absyn_Exp*e1=_tmp48;struct Cyc_Absyn_Exp*e2=_tmp49;
# 193
struct _tuple13 _stmttmpB=Cyc_Evexp_eval_const_bool_exp(e1);int _tmp51;int _tmp50;_LL23: _tmp50=_stmttmpB.f1;_tmp51=_stmttmpB.f2;_LL24: {int bool1=_tmp50;int known1=_tmp51;
if(!known1){
Cyc_Evexp_eval_const_exp(e2);
return({struct _tuple12 _tmpFE;({union Cyc_Absyn_Cnst _tmp11D=Cyc_Absyn_Int_c(Cyc_Absyn_Signed,0);_tmpFE.f1=_tmp11D;}),_tmpFE.f2=0;_tmpFE;});}
# 194
if(bool1)
# 198
ans=Cyc_Evexp_eval_const_exp(e2);else{
ans=({struct _tuple12 _tmpFF;({union Cyc_Absyn_Cnst _tmp11E=Cyc_Absyn_Int_c(Cyc_Absyn_Signed,0);_tmpFF.f1=_tmp11E;}),_tmpFF.f2=1;_tmpFF;});}
goto _LL0;}}case 8U: _LLB: _tmp46=((struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct*)_tmp3E)->f1;_tmp47=((struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct*)_tmp3E)->f2;_LLC: {struct Cyc_Absyn_Exp*e1=_tmp46;struct Cyc_Absyn_Exp*e2=_tmp47;
# 202
struct _tuple13 _stmttmpC=Cyc_Evexp_eval_const_bool_exp(e1);int _tmp53;int _tmp52;_LL26: _tmp52=_stmttmpC.f1;_tmp53=_stmttmpC.f2;_LL27: {int bool1=_tmp52;int known1=_tmp53;
if(!known1){
Cyc_Evexp_eval_const_exp(e2);
return({struct _tuple12 _tmp100;({union Cyc_Absyn_Cnst _tmp11F=Cyc_Absyn_Int_c(Cyc_Absyn_Signed,0);_tmp100.f1=_tmp11F;}),_tmp100.f2=0;_tmp100;});}
# 203
if(!bool1)
# 207
ans=Cyc_Evexp_eval_const_exp(e2);else{
ans=({struct _tuple12 _tmp101;({union Cyc_Absyn_Cnst _tmp120=Cyc_Absyn_Int_c(Cyc_Absyn_Signed,1);_tmp101.f1=_tmp120;}),_tmp101.f2=1;_tmp101;});}
goto _LL0;}}case 3U: _LLD: _tmp44=((struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct*)_tmp3E)->f1;_tmp45=((struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct*)_tmp3E)->f2;_LLE: {enum Cyc_Absyn_Primop p=_tmp44;struct Cyc_List_List*es=_tmp45;
# 211
if(es == 0){
({void*_tmp54=0U;({unsigned _tmp122=e->loc;struct _fat_ptr _tmp121=({const char*_tmp55="bad static expression (no args to primop)";_tag_fat(_tmp55,sizeof(char),42U);});Cyc_Tcutil_terr(_tmp122,_tmp121,_tag_fat(_tmp54,sizeof(void*),0U));});});
return({struct _tuple12 _tmp102;({union Cyc_Absyn_Cnst _tmp123=Cyc_Absyn_Int_c(Cyc_Absyn_Signed,0);_tmp102.f1=_tmp123;}),_tmp102.f2=1;_tmp102;});}
# 211
if(es->tl == 0){
# 216
ans=Cyc_Evexp_eval_const_unprimop(p,(struct Cyc_Absyn_Exp*)es->hd);
goto _LL0;}
# 211
if(((struct Cyc_List_List*)_check_null(es->tl))->tl == 0){
# 220
ans=Cyc_Evexp_eval_const_binprimop(p,(struct Cyc_Absyn_Exp*)es->hd,(struct Cyc_Absyn_Exp*)((struct Cyc_List_List*)_check_null(es->tl))->hd);
goto _LL0;}
# 211
({void*_tmp56=0U;({unsigned _tmp125=e->loc;struct _fat_ptr _tmp124=({const char*_tmp57="bad static expression (too many args to primop)";_tag_fat(_tmp57,sizeof(char),48U);});Cyc_Tcutil_terr(_tmp125,_tmp124,_tag_fat(_tmp56,sizeof(void*),0U));});});
# 224
return({struct _tuple12 _tmp103;({union Cyc_Absyn_Cnst _tmp126=Cyc_Absyn_Int_c(Cyc_Absyn_Signed,0);_tmp103.f1=_tmp126;}),_tmp103.f2=1;_tmp103;});}case 40U: _LLF: _tmp43=(void*)((struct Cyc_Absyn_Valueof_e_Absyn_Raw_exp_struct*)_tmp3E)->f1;_LL10: {void*t=_tmp43;
# 227
{void*_stmttmpD=Cyc_Tcutil_compress(t);void*_tmp58=_stmttmpD;struct Cyc_Absyn_Exp*_tmp59;if(((struct Cyc_Absyn_ValueofType_Absyn_Type_struct*)_tmp58)->tag == 9U){_LL29: _tmp59=((struct Cyc_Absyn_ValueofType_Absyn_Type_struct*)_tmp58)->f1;_LL2A: {struct Cyc_Absyn_Exp*e2=_tmp59;
# 229
e->r=e2->r;
return Cyc_Evexp_eval_const_exp(e2);}}else{_LL2B: _LL2C:
 goto _LL28;}_LL28:;}
# 233
goto _LL12;}case 17U: _LL11: _LL12:
 goto _LL14;case 18U: _LL13: _LL14:
 goto _LL16;case 19U: _LL15: _LL16:
 ans=({struct _tuple12 _tmp104;({union Cyc_Absyn_Cnst _tmp127=Cyc_Absyn_Int_c(Cyc_Absyn_Signed,0);_tmp104.f1=_tmp127;}),_tmp104.f2=0;_tmp104;});goto _LL0;case 14U: _LL17: _tmp41=(void*)((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp3E)->f1;_tmp42=((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp3E)->f2;_LL18: {void*t=_tmp41;struct Cyc_Absyn_Exp*e2=_tmp42;
# 239
ans=Cyc_Evexp_eval_const_exp(e2);
if(ans.f2){
struct _tuple16 _stmttmpE=({struct _tuple16 _tmp106;({void*_tmp128=Cyc_Tcutil_compress(t);_tmp106.f1=_tmp128;}),_tmp106.f2=ans.f1;_tmp106;});struct _tuple16 _tmp5A=_stmttmpE;int _tmp5D;enum Cyc_Absyn_Sign _tmp5C;void*_tmp5B;short _tmp60;enum Cyc_Absyn_Sign _tmp5F;void*_tmp5E;char _tmp63;enum Cyc_Absyn_Sign _tmp62;void*_tmp61;int _tmp67;enum Cyc_Absyn_Sign _tmp66;enum Cyc_Absyn_Size_of _tmp65;enum Cyc_Absyn_Sign _tmp64;short _tmp6B;enum Cyc_Absyn_Sign _tmp6A;enum Cyc_Absyn_Size_of _tmp69;enum Cyc_Absyn_Sign _tmp68;char _tmp6F;enum Cyc_Absyn_Sign _tmp6E;enum Cyc_Absyn_Size_of _tmp6D;enum Cyc_Absyn_Sign _tmp6C;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp5A.f1)->tag == 0U)switch(*((int*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp5A.f1)->f1)){case 1U: switch(((_tmp5A.f2).Int_c).tag){case 2U: _LL2E: _tmp6C=((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp5A.f1)->f1)->f1;_tmp6D=((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp5A.f1)->f1)->f2;_tmp6E=(((_tmp5A.f2).Char_c).val).f1;_tmp6F=(((_tmp5A.f2).Char_c).val).f2;_LL2F: {enum Cyc_Absyn_Sign sn=_tmp6C;enum Cyc_Absyn_Size_of sz=_tmp6D;enum Cyc_Absyn_Sign sn2=_tmp6E;char x=_tmp6F;
# 244
_tmp68=sn;_tmp69=sz;_tmp6A=sn2;_tmp6B=(short)x;goto _LL31;}case 4U: _LL30: _tmp68=((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp5A.f1)->f1)->f1;_tmp69=((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp5A.f1)->f1)->f2;_tmp6A=(((_tmp5A.f2).Short_c).val).f1;_tmp6B=(((_tmp5A.f2).Short_c).val).f2;_LL31: {enum Cyc_Absyn_Sign sn=_tmp68;enum Cyc_Absyn_Size_of sz=_tmp69;enum Cyc_Absyn_Sign sn2=_tmp6A;short x=_tmp6B;
# 246
_tmp64=sn;_tmp65=sz;_tmp66=sn2;_tmp67=(int)x;goto _LL33;}case 5U: _LL32: _tmp64=((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp5A.f1)->f1)->f1;_tmp65=((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp5A.f1)->f1)->f2;_tmp66=(((_tmp5A.f2).Int_c).val).f1;_tmp67=(((_tmp5A.f2).Int_c).val).f2;_LL33: {enum Cyc_Absyn_Sign sn=_tmp64;enum Cyc_Absyn_Size_of sz=_tmp65;enum Cyc_Absyn_Sign sn2=_tmp66;int x=_tmp67;
# 248
if((int)sn != (int)sn2)
({union Cyc_Absyn_Cnst _tmp129=Cyc_Absyn_Int_c(sn,x);ans.f1=_tmp129;});
# 248
goto _LL2D;}default: goto _LL3A;}case 4U: if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp5A.f1)->f2 != 0)switch(((_tmp5A.f2).Int_c).tag){case 2U: _LL34: _tmp61=(void*)(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp5A.f1)->f2)->hd;_tmp62=(((_tmp5A.f2).Char_c).val).f1;_tmp63=(((_tmp5A.f2).Char_c).val).f2;_LL35: {void*it=_tmp61;enum Cyc_Absyn_Sign sn2=_tmp62;char x=_tmp63;
# 253
_tmp5E=it;_tmp5F=sn2;_tmp60=(short)x;goto _LL37;}case 4U: _LL36: _tmp5E=(void*)(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp5A.f1)->f2)->hd;_tmp5F=(((_tmp5A.f2).Short_c).val).f1;_tmp60=(((_tmp5A.f2).Short_c).val).f2;_LL37: {void*it=_tmp5E;enum Cyc_Absyn_Sign sn2=_tmp5F;short x=_tmp60;
# 255
_tmp5B=it;_tmp5C=sn2;_tmp5D=(int)x;goto _LL39;}case 5U: _LL38: _tmp5B=(void*)(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp5A.f1)->f2)->hd;_tmp5C=(((_tmp5A.f2).Int_c).val).f1;_tmp5D=(((_tmp5A.f2).Int_c).val).f2;_LL39: {void*it=_tmp5B;enum Cyc_Absyn_Sign sn2=_tmp5C;int x=_tmp5D;
# 257
if(x < 0)
({void*_tmp70=0U;({unsigned _tmp12B=e->loc;struct _fat_ptr _tmp12A=({const char*_tmp71="cannot cast negative number to a tag type";_tag_fat(_tmp71,sizeof(char),42U);});Cyc_Tcutil_terr(_tmp12B,_tmp12A,_tag_fat(_tmp70,sizeof(void*),0U));});});
# 257
({void*_tmp12D=it;Cyc_Unify_unify(_tmp12D,(void*)({struct Cyc_Absyn_ValueofType_Absyn_Type_struct*_tmp72=_cycalloc(sizeof(*_tmp72));
# 259
_tmp72->tag=9U,({struct Cyc_Absyn_Exp*_tmp12C=Cyc_Absyn_const_exp(ans.f1,0U);_tmp72->f1=_tmp12C;});_tmp72;}));});
({union Cyc_Absyn_Cnst _tmp12E=Cyc_Absyn_Int_c(Cyc_Absyn_Unsigned,x);ans.f1=_tmp12E;});
goto _LL2D;}default: goto _LL3A;}else{goto _LL3A;}default: goto _LL3A;}else{_LL3A: _LL3B:
({struct Cyc_String_pa_PrintArg_struct _tmp75=({struct Cyc_String_pa_PrintArg_struct _tmp105;_tmp105.tag=0U,({struct _fat_ptr _tmp12F=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t));_tmp105.f1=_tmp12F;});_tmp105;});void*_tmp73[1U];_tmp73[0]=& _tmp75;({unsigned _tmp131=e->loc;struct _fat_ptr _tmp130=({const char*_tmp74="cannot cast to %s";_tag_fat(_tmp74,sizeof(char),18U);});Cyc_Tcutil_terr(_tmp131,_tmp130,_tag_fat(_tmp73,sizeof(void*),1U));});});goto _LL2D;}_LL2D:;}
# 240
goto _LL0;}case 33U: _LL19: _tmp40=((struct Cyc_Absyn_AnonEnum_e_Absyn_Raw_exp_struct*)_tmp3E)->f2;_LL1A: {struct Cyc_Absyn_Enumfield*ef=_tmp40;
# 265
_tmp3F=ef;goto _LL1C;}case 32U: _LL1B: _tmp3F=((struct Cyc_Absyn_Enum_e_Absyn_Raw_exp_struct*)_tmp3E)->f2;_LL1C: {struct Cyc_Absyn_Enumfield*ef=_tmp3F;
ans=Cyc_Evexp_eval_const_exp((struct Cyc_Absyn_Exp*)_check_null(ef->tag));goto _LL0;}default: _LL1D: _LL1E:
# 269
 return({struct _tuple12 _tmp107;({union Cyc_Absyn_Cnst _tmp132=Cyc_Absyn_Int_c(Cyc_Absyn_Signed,0);_tmp107.f1=_tmp132;}),_tmp107.f2=0;_tmp107;});}_LL0:;}
# 271
if(ans.f2){
void*c;
{union Cyc_Absyn_Cnst c=ans.f1;
({void*_tmp133=(void*)({struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*_tmp76=_cycalloc(sizeof(*_tmp76));_tmp76->tag=0U,_tmp76->f1=c;_tmp76;});e->r=_tmp133;});}}
# 271
return ans;}
# 37
int Cyc_Evexp_c_can_eval(struct Cyc_Absyn_Exp*e){
# 280
void*_stmttmpF=e->r;void*_tmp77=_stmttmpF;struct Cyc_Absyn_Exp*_tmp78;struct Cyc_List_List*_tmp7A;enum Cyc_Absyn_Primop _tmp79;struct Cyc_Absyn_Exp*_tmp7C;struct Cyc_Absyn_Exp*_tmp7B;struct Cyc_Absyn_Exp*_tmp7E;struct Cyc_Absyn_Exp*_tmp7D;struct Cyc_Absyn_Exp*_tmp81;struct Cyc_Absyn_Exp*_tmp80;struct Cyc_Absyn_Exp*_tmp7F;switch(*((int*)_tmp77)){case 33U: _LL1: _LL2:
 goto _LL4;case 32U: _LL3: _LL4:
 goto _LL6;case 17U: _LL5: _LL6:
 goto _LL8;case 18U: _LL7: _LL8:
 goto _LLA;case 19U: _LL9: _LLA:
 goto _LLC;case 0U: _LLB: _LLC:
 return 1;case 6U: _LLD: _tmp7F=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmp77)->f1;_tmp80=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmp77)->f2;_tmp81=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmp77)->f3;_LLE: {struct Cyc_Absyn_Exp*e1=_tmp7F;struct Cyc_Absyn_Exp*e2=_tmp80;struct Cyc_Absyn_Exp*e3=_tmp81;
# 288
return(Cyc_Evexp_c_can_eval(e1)&& Cyc_Evexp_c_can_eval(e2))&& Cyc_Evexp_c_can_eval(e3);}case 7U: _LLF: _tmp7D=((struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct*)_tmp77)->f1;_tmp7E=((struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct*)_tmp77)->f2;_LL10: {struct Cyc_Absyn_Exp*e1=_tmp7D;struct Cyc_Absyn_Exp*e2=_tmp7E;
_tmp7B=e1;_tmp7C=e2;goto _LL12;}case 8U: _LL11: _tmp7B=((struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct*)_tmp77)->f1;_tmp7C=((struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct*)_tmp77)->f2;_LL12: {struct Cyc_Absyn_Exp*e1=_tmp7B;struct Cyc_Absyn_Exp*e2=_tmp7C;
# 291
return Cyc_Evexp_c_can_eval(e1)&& Cyc_Evexp_c_can_eval(e2);}case 3U: _LL13: _tmp79=((struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct*)_tmp77)->f1;_tmp7A=((struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct*)_tmp77)->f2;_LL14: {enum Cyc_Absyn_Primop p=_tmp79;struct Cyc_List_List*es=_tmp7A;
# 293
for(0;es != 0;es=es->tl){
if(!Cyc_Evexp_c_can_eval((struct Cyc_Absyn_Exp*)es->hd))return 0;}
# 293
return 1;}case 40U: _LL15: _LL16:
# 296
 return 0;case 14U: _LL17: _tmp78=((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp77)->f2;_LL18: {struct Cyc_Absyn_Exp*e=_tmp78;
return Cyc_Evexp_c_can_eval(e);}default: _LL19: _LL1A:
 return 0;}_LL0:;}
# 37
static int Cyc_Evexp_const_exp_case_number(struct Cyc_Absyn_Exp*e){
# 303
void*_stmttmp10=e->r;void*_tmp82=_stmttmp10;switch(*((int*)_tmp82)){case 0U: _LL1: _LL2:
 return 1;case 6U: _LL3: _LL4:
 return 2;case 3U: _LL5: _LL6:
 return 3;case 17U: _LL7: _LL8:
 goto _LLA;case 18U: _LL9: _LLA:
 return 4;case 19U: _LLB: _LLC:
 return 5;case 14U: _LLD: _LLE:
 return 6;case 7U: _LLF: _LL10:
 return 7;case 8U: _LL11: _LL12:
 return 8;case 40U: _LL13: _LL14:
 return 9;default: _LL15: _LL16:
# 315
({struct Cyc_String_pa_PrintArg_struct _tmp85=({struct Cyc_String_pa_PrintArg_struct _tmp108;_tmp108.tag=0U,({struct _fat_ptr _tmp134=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_exp2string(e));_tmp108.f1=_tmp134;});_tmp108;});void*_tmp83[1U];_tmp83[0]=& _tmp85;({unsigned _tmp136=e->loc;struct _fat_ptr _tmp135=({const char*_tmp84="bad static expression %s";_tag_fat(_tmp84,sizeof(char),25U);});Cyc_Tcutil_terr(_tmp136,_tmp135,_tag_fat(_tmp83,sizeof(void*),1U));});});return 0;}_LL0:;}
# 37
static struct Cyc_Absyn_Exp*Cyc_Evexp_strip_cast(struct Cyc_Absyn_Exp*e){
# 321
{void*_stmttmp11=e->r;void*_tmp86=_stmttmp11;struct Cyc_Absyn_Exp*_tmp88;void*_tmp87;if(((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp86)->tag == 14U){_LL1: _tmp87=(void*)((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp86)->f1;_tmp88=((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp86)->f2;_LL2: {void*t=_tmp87;struct Cyc_Absyn_Exp*e2=_tmp88;
# 323
if(Cyc_Unify_unify(t,Cyc_Absyn_uint_type)){
void*_stmttmp12=e2->r;void*_tmp89=_stmttmp12;if(((struct Cyc_Absyn_Valueof_e_Absyn_Raw_exp_struct*)_tmp89)->tag == 40U){_LL6: _LL7:
 return e2;}else{_LL8: _LL9:
 goto _LL5;}_LL5:;}
# 323
goto _LL0;}}else{_LL3: _LL4:
# 330
 goto _LL0;}_LL0:;}
# 332
return e;}struct _tuple17{void*f1;void*f2;};
# 37
int Cyc_Evexp_const_exp_cmp(struct Cyc_Absyn_Exp*e1,struct Cyc_Absyn_Exp*e2){
# 337
e1=Cyc_Evexp_strip_cast(e1);
e2=Cyc_Evexp_strip_cast(e2);{
struct _tuple11 _stmttmp13=Cyc_Evexp_eval_const_uint_exp(e1);int _tmp8B;unsigned _tmp8A;_LL1: _tmp8A=_stmttmp13.f1;_tmp8B=_stmttmp13.f2;_LL2: {unsigned i1=_tmp8A;int known1=_tmp8B;
struct _tuple11 _stmttmp14=Cyc_Evexp_eval_const_uint_exp(e2);int _tmp8D;unsigned _tmp8C;_LL4: _tmp8C=_stmttmp14.f1;_tmp8D=_stmttmp14.f2;_LL5: {unsigned i2=_tmp8C;int known2=_tmp8D;
if(known1 && known2)
return(int)(i1 - i2);{
# 341
int e1case=
# 343
Cyc_Evexp_const_exp_case_number(e1);
int e2case=Cyc_Evexp_const_exp_case_number(e2);
if(e1case != e2case)
return e1case - e2case;{
# 345
struct _tuple17 _stmttmp15=({struct _tuple17 _tmp10A;_tmp10A.f1=e1->r,_tmp10A.f2=e2->r;_tmp10A;});struct _tuple17 _tmp8E=_stmttmp15;void*_tmp90;void*_tmp8F;struct Cyc_Absyn_Exp*_tmp94;void*_tmp93;struct Cyc_Absyn_Exp*_tmp92;void*_tmp91;struct Cyc_List_List*_tmp98;void*_tmp97;struct Cyc_List_List*_tmp96;void*_tmp95;struct Cyc_Absyn_Exp*_tmp9A;struct Cyc_Absyn_Exp*_tmp99;void*_tmp9C;struct Cyc_Absyn_Exp*_tmp9B;struct Cyc_Absyn_Exp*_tmp9E;void*_tmp9D;void*_tmpA0;void*_tmp9F;struct Cyc_List_List*_tmpA4;enum Cyc_Absyn_Primop _tmpA3;struct Cyc_List_List*_tmpA2;enum Cyc_Absyn_Primop _tmpA1;struct Cyc_Absyn_Exp*_tmpA8;struct Cyc_Absyn_Exp*_tmpA7;struct Cyc_Absyn_Exp*_tmpA6;struct Cyc_Absyn_Exp*_tmpA5;struct Cyc_Absyn_Exp*_tmpAC;struct Cyc_Absyn_Exp*_tmpAB;struct Cyc_Absyn_Exp*_tmpAA;struct Cyc_Absyn_Exp*_tmpA9;struct Cyc_Absyn_Exp*_tmpB2;struct Cyc_Absyn_Exp*_tmpB1;struct Cyc_Absyn_Exp*_tmpB0;struct Cyc_Absyn_Exp*_tmpAF;struct Cyc_Absyn_Exp*_tmpAE;struct Cyc_Absyn_Exp*_tmpAD;switch(*((int*)_tmp8E.f1)){case 6U: if(((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmp8E.f2)->tag == 6U){_LL7: _tmpAD=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmp8E.f1)->f1;_tmpAE=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmp8E.f1)->f2;_tmpAF=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmp8E.f1)->f3;_tmpB0=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmp8E.f2)->f1;_tmpB1=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmp8E.f2)->f2;_tmpB2=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmp8E.f2)->f3;_LL8: {struct Cyc_Absyn_Exp*e11=_tmpAD;struct Cyc_Absyn_Exp*e12=_tmpAE;struct Cyc_Absyn_Exp*e13=_tmpAF;struct Cyc_Absyn_Exp*e21=_tmpB0;struct Cyc_Absyn_Exp*e22=_tmpB1;struct Cyc_Absyn_Exp*e23=_tmpB2;
# 352
int c1=Cyc_Evexp_const_exp_cmp(e13,e23);
if(c1 != 0)return c1;_tmpA9=e11;_tmpAA=e12;_tmpAB=e21;_tmpAC=e22;goto _LLA;}}else{goto _LL1D;}case 7U: if(((struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct*)_tmp8E.f2)->tag == 7U){_LL9: _tmpA9=((struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct*)_tmp8E.f1)->f1;_tmpAA=((struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct*)_tmp8E.f1)->f2;_tmpAB=((struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct*)_tmp8E.f2)->f1;_tmpAC=((struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct*)_tmp8E.f2)->f2;_LLA: {struct Cyc_Absyn_Exp*e11=_tmpA9;struct Cyc_Absyn_Exp*e12=_tmpAA;struct Cyc_Absyn_Exp*e21=_tmpAB;struct Cyc_Absyn_Exp*e22=_tmpAC;
# 355
_tmpA5=e11;_tmpA6=e12;_tmpA7=e21;_tmpA8=e22;goto _LLC;}}else{goto _LL1D;}case 8U: if(((struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct*)_tmp8E.f2)->tag == 8U){_LLB: _tmpA5=((struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct*)_tmp8E.f1)->f1;_tmpA6=((struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct*)_tmp8E.f1)->f2;_tmpA7=((struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct*)_tmp8E.f2)->f1;_tmpA8=((struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct*)_tmp8E.f2)->f2;_LLC: {struct Cyc_Absyn_Exp*e11=_tmpA5;struct Cyc_Absyn_Exp*e12=_tmpA6;struct Cyc_Absyn_Exp*e21=_tmpA7;struct Cyc_Absyn_Exp*e22=_tmpA8;
# 357
int c1=Cyc_Evexp_const_exp_cmp(e11,e21);
if(c1 != 0)return c1;return
 Cyc_Evexp_const_exp_cmp(e12,e22);}}else{goto _LL1D;}case 3U: if(((struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct*)_tmp8E.f2)->tag == 3U){_LLD: _tmpA1=((struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct*)_tmp8E.f1)->f1;_tmpA2=((struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct*)_tmp8E.f1)->f2;_tmpA3=((struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct*)_tmp8E.f2)->f1;_tmpA4=((struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct*)_tmp8E.f2)->f2;_LLE: {enum Cyc_Absyn_Primop p1=_tmpA1;struct Cyc_List_List*es1=_tmpA2;enum Cyc_Absyn_Primop p2=_tmpA3;struct Cyc_List_List*es2=_tmpA4;
# 361
int c1=(int)p1 - (int)p2;
if(c1 != 0)return c1;for(0;
es1 != 0 && es2 != 0;(es1=es1->tl,es2=es2->tl)){
int c2=Cyc_Evexp_const_exp_cmp((struct Cyc_Absyn_Exp*)es1->hd,(struct Cyc_Absyn_Exp*)es2->hd);
if(c2 != 0)
return c2;}
# 368
return 0;}}else{goto _LL1D;}case 17U: switch(*((int*)_tmp8E.f2)){case 17U: _LLF: _tmp9F=(void*)((struct Cyc_Absyn_Sizeoftype_e_Absyn_Raw_exp_struct*)_tmp8E.f1)->f1;_tmpA0=(void*)((struct Cyc_Absyn_Sizeoftype_e_Absyn_Raw_exp_struct*)_tmp8E.f2)->f1;_LL10: {void*t1=_tmp9F;void*t2=_tmpA0;
# 370
return Cyc_Tcutil_typecmp(t1,t2);}case 18U: _LL11: _tmp9D=(void*)((struct Cyc_Absyn_Sizeoftype_e_Absyn_Raw_exp_struct*)_tmp8E.f1)->f1;_tmp9E=((struct Cyc_Absyn_Sizeofexp_e_Absyn_Raw_exp_struct*)_tmp8E.f2)->f1;_LL12: {void*t1=_tmp9D;struct Cyc_Absyn_Exp*e2a=_tmp9E;
# 372
void*e2atopt=e2a->topt;
if(e2atopt == 0)
({void*_tmpB3=0U;({unsigned _tmp138=e2->loc;struct _fat_ptr _tmp137=({const char*_tmpB4="cannot handle sizeof(exp) here -- use sizeof(type)";_tag_fat(_tmpB4,sizeof(char),51U);});Cyc_Tcutil_terr(_tmp138,_tmp137,_tag_fat(_tmpB3,sizeof(void*),0U));});});
# 373
return
# 375
 Cyc_Tcutil_typecmp(t1,(void*)_check_null(e2atopt));}default: goto _LL1D;}case 18U: switch(*((int*)_tmp8E.f2)){case 17U: _LL13: _tmp9B=((struct Cyc_Absyn_Sizeofexp_e_Absyn_Raw_exp_struct*)_tmp8E.f1)->f1;_tmp9C=(void*)((struct Cyc_Absyn_Sizeoftype_e_Absyn_Raw_exp_struct*)_tmp8E.f2)->f1;_LL14: {struct Cyc_Absyn_Exp*e1a=_tmp9B;void*t2=_tmp9C;
# 377
void*e1atopt=e1a->topt;
if(e1atopt == 0)
({void*_tmpB5=0U;({unsigned _tmp13A=e1->loc;struct _fat_ptr _tmp139=({const char*_tmpB6="cannot handle sizeof(exp) here -- use sizeof(type)";_tag_fat(_tmpB6,sizeof(char),51U);});Cyc_Tcutil_terr(_tmp13A,_tmp139,_tag_fat(_tmpB5,sizeof(void*),0U));});});
# 378
return
# 380
 Cyc_Tcutil_typecmp((void*)_check_null(e1atopt),t2);}case 18U: _LL15: _tmp99=((struct Cyc_Absyn_Sizeofexp_e_Absyn_Raw_exp_struct*)_tmp8E.f1)->f1;_tmp9A=((struct Cyc_Absyn_Sizeofexp_e_Absyn_Raw_exp_struct*)_tmp8E.f2)->f1;_LL16: {struct Cyc_Absyn_Exp*e1a=_tmp99;struct Cyc_Absyn_Exp*e2a=_tmp9A;
# 382
void*e1atopt=e1a->topt;
void*e2atopt=e2a->topt;
if(e1atopt == 0)
({void*_tmpB7=0U;({unsigned _tmp13C=e1->loc;struct _fat_ptr _tmp13B=({const char*_tmpB8="cannot handle sizeof(exp) here -- use sizeof(type)";_tag_fat(_tmpB8,sizeof(char),51U);});Cyc_Tcutil_terr(_tmp13C,_tmp13B,_tag_fat(_tmpB7,sizeof(void*),0U));});});
# 384
if(e2atopt == 0)
# 387
({void*_tmpB9=0U;({unsigned _tmp13E=e2->loc;struct _fat_ptr _tmp13D=({const char*_tmpBA="cannot handle sizeof(exp) here -- use sizeof(type)";_tag_fat(_tmpBA,sizeof(char),51U);});Cyc_Tcutil_terr(_tmp13E,_tmp13D,_tag_fat(_tmpB9,sizeof(void*),0U));});});
# 384
return({
# 388
void*_tmp13F=(void*)_check_null(e1atopt);Cyc_Tcutil_typecmp(_tmp13F,(void*)_check_null(e2atopt));});}default: goto _LL1D;}case 19U: if(((struct Cyc_Absyn_Offsetof_e_Absyn_Raw_exp_struct*)_tmp8E.f2)->tag == 19U){_LL17: _tmp95=(void*)((struct Cyc_Absyn_Offsetof_e_Absyn_Raw_exp_struct*)_tmp8E.f1)->f1;_tmp96=((struct Cyc_Absyn_Offsetof_e_Absyn_Raw_exp_struct*)_tmp8E.f1)->f2;_tmp97=(void*)((struct Cyc_Absyn_Offsetof_e_Absyn_Raw_exp_struct*)_tmp8E.f2)->f1;_tmp98=((struct Cyc_Absyn_Offsetof_e_Absyn_Raw_exp_struct*)_tmp8E.f2)->f2;_LL18: {void*t1=_tmp95;struct Cyc_List_List*f1=_tmp96;void*t2=_tmp97;struct Cyc_List_List*f2=_tmp98;
# 390
int c1=Cyc_Tcutil_typecmp(t1,t2);
if(c1 != 0)return c1;{int l1=
Cyc_List_length(f1);
int l2=Cyc_List_length(f2);
if(l1 < l2)return - 1;if(l2 < l1)
return 1;
# 394
for(0;
# 396
f1 != 0 && f2 != 0;(f1=f1->tl,f2=f2->tl)){
struct _tuple17 _stmttmp16=({struct _tuple17 _tmp109;_tmp109.f1=(void*)f1->hd,_tmp109.f2=(void*)f2->hd;_tmp109;});struct _tuple17 _tmpBB=_stmttmp16;unsigned _tmpBD;unsigned _tmpBC;struct _fat_ptr*_tmpBF;struct _fat_ptr*_tmpBE;if(((struct Cyc_Absyn_StructField_Absyn_OffsetofField_struct*)_tmpBB.f1)->tag == 0U){if(((struct Cyc_Absyn_TupleIndex_Absyn_OffsetofField_struct*)_tmpBB.f2)->tag == 1U){_LL20: _LL21:
 return - 1;}else{_LL24: _tmpBE=((struct Cyc_Absyn_StructField_Absyn_OffsetofField_struct*)_tmpBB.f1)->f1;_tmpBF=((struct Cyc_Absyn_StructField_Absyn_OffsetofField_struct*)_tmpBB.f2)->f1;_LL25: {struct _fat_ptr*fn1=_tmpBE;struct _fat_ptr*fn2=_tmpBF;
# 401
int c=Cyc_strptrcmp(fn1,fn2);
if(c != 0)return c;goto _LL1F;}}}else{if(((struct Cyc_Absyn_StructField_Absyn_OffsetofField_struct*)_tmpBB.f2)->tag == 0U){_LL22: _LL23:
# 399
 return 1;}else{_LL26: _tmpBC=((struct Cyc_Absyn_TupleIndex_Absyn_OffsetofField_struct*)_tmpBB.f1)->f1;_tmpBD=((struct Cyc_Absyn_TupleIndex_Absyn_OffsetofField_struct*)_tmpBB.f2)->f1;_LL27: {unsigned i1=_tmpBC;unsigned i2=_tmpBD;
# 405
int c=(int)(i1 - i2);
if(c != 0)return c;goto _LL1F;}}}_LL1F:;}
# 409
return 0;}}}else{goto _LL1D;}case 14U: if(((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp8E.f2)->tag == 14U){_LL19: _tmp91=(void*)((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp8E.f1)->f1;_tmp92=((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp8E.f1)->f2;_tmp93=(void*)((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp8E.f2)->f1;_tmp94=((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp8E.f2)->f2;_LL1A: {void*t1=_tmp91;struct Cyc_Absyn_Exp*e1a=_tmp92;void*t2=_tmp93;struct Cyc_Absyn_Exp*e2a=_tmp94;
# 411
int c1=Cyc_Tcutil_typecmp(t1,t2);
if(c1 != 0)return c1;return
 Cyc_Evexp_const_exp_cmp(e1a,e2a);}}else{goto _LL1D;}case 40U: if(((struct Cyc_Absyn_Valueof_e_Absyn_Raw_exp_struct*)_tmp8E.f2)->tag == 40U){_LL1B: _tmp8F=(void*)((struct Cyc_Absyn_Valueof_e_Absyn_Raw_exp_struct*)_tmp8E.f1)->f1;_tmp90=(void*)((struct Cyc_Absyn_Valueof_e_Absyn_Raw_exp_struct*)_tmp8E.f2)->f1;_LL1C: {void*t1=_tmp8F;void*t2=_tmp90;
# 415
if(Cyc_Unify_unify(t1,t2))return 0;return
 Cyc_Tcutil_typecmp(t1,t2);}}else{goto _LL1D;}default: _LL1D: _LL1E:
({void*_tmpC0=0U;({int(*_tmp141)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmpC2)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmpC2;});struct _fat_ptr _tmp140=({const char*_tmpC1="Evexp::const_exp_cmp, unexpected case";_tag_fat(_tmpC1,sizeof(char),38U);});_tmp141(_tmp140,_tag_fat(_tmpC0,sizeof(void*),0U));});});}_LL6:;}}}}}}
# 37
int Cyc_Evexp_same_const_exp(struct Cyc_Absyn_Exp*e1,struct Cyc_Absyn_Exp*e2){
# 421
return Cyc_Evexp_const_exp_cmp(e1,e2)== 0;}
# 37
int Cyc_Evexp_lte_const_exp(struct Cyc_Absyn_Exp*e1,struct Cyc_Absyn_Exp*e2){
# 424
struct _tuple11 _stmttmp17=Cyc_Evexp_eval_const_uint_exp(e1);int _tmpC4;unsigned _tmpC3;_LL1: _tmpC3=_stmttmp17.f1;_tmpC4=_stmttmp17.f2;_LL2: {unsigned i1=_tmpC3;int known1=_tmpC4;
struct _tuple11 _stmttmp18=Cyc_Evexp_eval_const_uint_exp(e2);int _tmpC6;unsigned _tmpC5;_LL4: _tmpC5=_stmttmp18.f1;_tmpC6=_stmttmp18.f2;_LL5: {unsigned i2=_tmpC5;int known2=_tmpC6;
if(known1 && known2)
return i1 <= i2;
# 426
return
# 428
 Cyc_Evexp_same_const_exp(e1,e2);}}}struct _tuple18{struct Cyc_Absyn_Tqual f1;void*f2;};
# 37
int Cyc_Evexp_okay_szofarg(void*t){
# 434
void*_stmttmp19=Cyc_Tcutil_compress(t);void*_tmpC7=_stmttmp19;struct Cyc_Absyn_Typedefdecl*_tmpC8;struct Cyc_List_List*_tmpC9;struct Cyc_Absyn_Exp*_tmpCB;void*_tmpCA;struct Cyc_List_List*_tmpCC;struct Cyc_Absyn_Tvar*_tmpCD;struct Cyc_List_List*_tmpCF;void*_tmpCE;switch(*((int*)_tmpC7)){case 0U: _LL1: _tmpCE=(void*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmpC7)->f1;_tmpCF=((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmpC7)->f2;_LL2: {void*c=_tmpCE;struct Cyc_List_List*ts=_tmpCF;
# 436
void*_tmpD0=c;struct Cyc_Absyn_Aggrdecl*_tmpD1;struct Cyc_Absyn_Datatypefield*_tmpD2;switch(*((int*)_tmpD0)){case 19U: if(((((struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct*)_tmpD0)->f1).KnownDatatypefield).tag == 2){_LL1C: _tmpD2=(((((struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct*)_tmpD0)->f1).KnownDatatypefield).val).f2;_LL1D: {struct Cyc_Absyn_Datatypefield*tuf=_tmpD2;
# 438
{struct Cyc_List_List*tqs=tuf->typs;for(0;tqs != 0;tqs=tqs->tl){
if(!Cyc_Evexp_okay_szofarg((*((struct _tuple18*)tqs->hd)).f2))
return 0;}}
# 438
return 1;}}else{_LL1E: _LL1F:
# 442
 return 0;}case 4U: _LL20: _LL21:
 goto _LL23;case 1U: _LL22: _LL23:
 goto _LL25;case 2U: _LL24: _LL25:
 goto _LL27;case 17U: _LL26: _LL27:
 goto _LL29;case 15U: _LL28: _LL29:
 goto _LL2B;case 3U: _LL2A: _LL2B:
 goto _LL2D;case 18U: _LL2C: _LL2D:
 goto _LL2F;case 16U: _LL2E: _LL2F:
 return 1;case 0U: _LL30: _LL31:
# 452
 goto _LL33;case 5U: _LL32: _LL33:
 goto _LL35;case 7U: _LL34: _LL35:
 goto _LL37;case 6U: _LL36: _LL37:
 goto _LL39;case 9U: _LL38: _LL39:
 goto _LL3B;case 8U: _LL3A: _LL3B:
 goto _LL3D;case 11U: _LL3C: _LL3D:
 goto _LL3F;case 12U: _LL3E: _LL3F:
 goto _LL41;case 13U: _LL40: _LL41:
 goto _LL43;case 14U: _LL42: _LL43:
 goto _LL45;case 10U: _LL44: _LL45:
 return 0;default: if(((((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)_tmpD0)->f1).UnknownAggr).tag == 1){_LL46: _LL47:
# 464
({void*_tmpD3=0U;({int(*_tmp143)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmpD5)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmpD5;});struct _fat_ptr _tmp142=({const char*_tmpD4="szof on unchecked StructType";_tag_fat(_tmpD4,sizeof(char),29U);});_tmp143(_tmp142,_tag_fat(_tmpD3,sizeof(void*),0U));});});}else{_LL48: _tmpD1=*((((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)_tmpD0)->f1).KnownAggr).val;_LL49: {struct Cyc_Absyn_Aggrdecl*ad=_tmpD1;
# 466
if(ad->impl == 0)return 0;else{
# 468
struct _RegionHandle _tmpD6=_new_region("temp");struct _RegionHandle*temp=& _tmpD6;_push_region(temp);
{struct Cyc_List_List*inst=Cyc_List_rzip(temp,temp,ad->tvs,ts);
{struct Cyc_List_List*fs=((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->fields;for(0;fs != 0;fs=fs->tl){
if(!Cyc_Evexp_okay_szofarg(Cyc_Tcutil_rsubstitute(temp,inst,((struct Cyc_Absyn_Aggrfield*)fs->hd)->type))){int _tmpD7=0;_npop_handler(0U);return _tmpD7;}}}{
# 470
int _tmpD8=1;_npop_handler(0U);return _tmpD8;}}
# 469
;_pop_region(temp);}}}}_LL1B:;}case 2U: _LL3: _tmpCD=((struct Cyc_Absyn_VarType_Absyn_Type_struct*)_tmpC7)->f1;_LL4: {struct Cyc_Absyn_Tvar*tv=_tmpCD;
# 476
enum Cyc_Absyn_KindQual _stmttmp1A=(Cyc_Tcutil_tvar_kind(tv,& Cyc_Tcutil_bk))->kind;enum Cyc_Absyn_KindQual _tmpD9=_stmttmp1A;if(_tmpD9 == Cyc_Absyn_BoxKind){_LL4B: _LL4C:
 return 1;}else{_LL4D: _LL4E:
 return 0;}_LL4A:;}case 1U: if(((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmpC7)->f1 != 0){if(((struct Cyc_Absyn_Kind*)((struct Cyc_Core_Opt*)((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmpC7)->f1)->v)->kind == Cyc_Absyn_BoxKind){_LL5: _LL6:
# 480
 return 1;}else{goto _LL7;}}else{_LL7: _LL8:
 return 0;}case 6U: _LL9: _tmpCC=((struct Cyc_Absyn_TupleType_Absyn_Type_struct*)_tmpC7)->f1;_LLA: {struct Cyc_List_List*tqs=_tmpCC;
# 483
for(0;tqs != 0;tqs=tqs->tl){
if(!Cyc_Evexp_okay_szofarg((*((struct _tuple18*)tqs->hd)).f2))
return 0;}
# 483
return 1;}case 3U: _LLB: _LLC:
# 489
 return 1;case 4U: _LLD: _tmpCA=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmpC7)->f1).elt_type;_tmpCB=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmpC7)->f1).num_elts;_LLE: {void*t2=_tmpCA;struct Cyc_Absyn_Exp*e=_tmpCB;
# 492
return e != 0;}case 5U: _LLF: _LL10:
 return 0;case 7U: _LL11: _tmpC9=((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmpC7)->f2;_LL12: {struct Cyc_List_List*fs=_tmpC9;
# 495
for(0;fs != 0;fs=fs->tl){
if(!Cyc_Evexp_okay_szofarg(((struct Cyc_Absyn_Aggrfield*)fs->hd)->type))
return 0;}
# 495
return 1;}case 10U: _LL13: _LL14:
# 499
 goto _LL16;case 9U: _LL15: _LL16:
 goto _LL18;case 11U: _LL17: _LL18:
 return 0;default: _LL19: _tmpC8=((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmpC7)->f3;_LL1A: {struct Cyc_Absyn_Typedefdecl*td=_tmpC8;
# 504
if(td == 0 || td->kind == 0)
({void*_tmpDA=0U;({int(*_tmp147)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmpDE)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmpDE;});struct _fat_ptr _tmp146=(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmpDD=({struct Cyc_String_pa_PrintArg_struct _tmp10B;_tmp10B.tag=0U,({struct _fat_ptr _tmp144=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t));_tmp10B.f1=_tmp144;});_tmp10B;});void*_tmpDB[1U];_tmpDB[0]=& _tmpDD;({struct _fat_ptr _tmp145=({const char*_tmpDC="szof typedeftype %s";_tag_fat(_tmpDC,sizeof(char),20U);});Cyc_aprintf(_tmp145,_tag_fat(_tmpDB,sizeof(void*),1U));});});_tmp147(_tmp146,_tag_fat(_tmpDA,sizeof(void*),0U));});});
# 504
return(int)((struct Cyc_Absyn_Kind*)((struct Cyc_Core_Opt*)_check_null(td->kind))->v)->kind == (int)Cyc_Absyn_BoxKind;}}_LL0:;}
