#include <setjmp.h>
/* This is a C header file to be used by the output of the Cyclone to
   C translator.  The corresponding definitions are in file lib/runtime_*.c */
#ifndef _CYC_INCLUDE_H_
#define _CYC_INCLUDE_H_

/* Need one of these per thread (see runtime_stack.c). The runtime maintains 
   a stack that contains either _handler_cons structs or _RegionHandle structs.
   The tag is 0 for a handler_cons and 1 for a region handle.  */
struct _RuntimeStack {
  int tag; 
  struct _RuntimeStack *next;
  void (*cleanup)(struct _RuntimeStack *frame);
};

#ifndef offsetof
/* should be size_t, but int is fine. */
#define offsetof(t,n) ((int)(&(((t *)0)->n)))
#endif

/* Fat pointers */
struct _fat_ptr {
  unsigned char *curr; 
  unsigned char *base; 
  unsigned char *last_plus_one; 
};  

/* Discriminated Unions */
struct _xtunion_struct { char *tag; };

/* Regions */
struct _RegionPage
#ifdef CYC_REGION_PROFILE
{ unsigned total_bytes;
  unsigned free_bytes;
  /* MWH: wish we didn't have to include the stuff below ... */
  struct _RegionPage *next;
  char data[1];
}
#endif
; // abstract -- defined in runtime_memory.c
struct _RegionHandle {
  struct _RuntimeStack s;
  struct _RegionPage *curr;
  char               *offset;
  char               *last_plus_one;
  struct _DynRegionHandle *sub_regions;
#ifdef CYC_REGION_PROFILE
  const char         *name;
#else
  unsigned used_bytes;
  unsigned wasted_bytes;
#endif
};
struct _DynRegionFrame {
  struct _RuntimeStack s;
  struct _DynRegionHandle *x;
};
// A dynamic region is just a region handle.  The wrapper struct is for type
// abstraction.
struct Cyc_Core_DynamicRegion {
  struct _RegionHandle h;
};

struct _RegionHandle _new_region(const char*);
void* _region_malloc(struct _RegionHandle*, unsigned);
void* _region_calloc(struct _RegionHandle*, unsigned t, unsigned n);
void   _free_region(struct _RegionHandle*);
struct _RegionHandle*_open_dynregion(struct _DynRegionFrame*,struct _DynRegionHandle*);
void   _pop_dynregion();

/* Exceptions */
struct _handler_cons {
  struct _RuntimeStack s;
  jmp_buf handler;
};
void _push_handler(struct _handler_cons *);
void _push_region(struct _RegionHandle *);
void _npop_handler(int);
void _pop_handler();
void _pop_region();

#ifndef _throw
void* _throw_null_fn(const char*,unsigned);
void* _throw_arraybounds_fn(const char*,unsigned);
void* _throw_badalloc_fn(const char*,unsigned);
void* _throw_match_fn(const char*,unsigned);
void* _throw_fn(void*,const char*,unsigned);
void* _rethrow(void*);
#define _throw_null() (_throw_null_fn(__FILE__,__LINE__))
#define _throw_arraybounds() (_throw_arraybounds_fn(__FILE__,__LINE__))
#define _throw_badalloc() (_throw_badalloc_fn(__FILE__,__LINE__))
#define _throw_match() (_throw_match_fn(__FILE__,__LINE__))
#define _throw(e) (_throw_fn((e),__FILE__,__LINE__))
#endif

struct _xtunion_struct* Cyc_Core_get_exn_thrown();
/* Built-in Exceptions */
struct Cyc_Null_Exception_exn_struct { char *tag; };
struct Cyc_Array_bounds_exn_struct { char *tag; };
struct Cyc_Match_Exception_exn_struct { char *tag; };
struct Cyc_Bad_alloc_exn_struct { char *tag; };
extern char Cyc_Null_Exception[];
extern char Cyc_Array_bounds[];
extern char Cyc_Match_Exception[];
extern char Cyc_Bad_alloc[];

/* Built-in Run-time Checks and company */
#ifdef CYC_ANSI_OUTPUT
#define _INLINE  
#else
#define _INLINE inline
#endif

#ifdef NO_CYC_NULL_CHECKS
#define _check_null(ptr) (ptr)
#else
#define _check_null(ptr) \
  ({ void*_cks_null = (void*)(ptr); \
     if (!_cks_null) _throw_null(); \
     _cks_null; })
#endif

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index)\
   (((char*)ptr) + (elt_sz)*(index))
#ifdef NO_CYC_NULL_CHECKS
#define _check_known_subscript_null _check_known_subscript_notnull
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr);\
  int _index = (index);\
  if (!_cks_ptr) _throw_null(); \
  _cks_ptr + (elt_sz)*_index; })
#endif
#define _zero_arr_plus_fn(orig_x,orig_sz,orig_i,f,l) ((orig_x)+(orig_i))
#define _zero_arr_plus_char_fn _zero_arr_plus_fn
#define _zero_arr_plus_short_fn _zero_arr_plus_fn
#define _zero_arr_plus_int_fn _zero_arr_plus_fn
#define _zero_arr_plus_float_fn _zero_arr_plus_fn
#define _zero_arr_plus_double_fn _zero_arr_plus_fn
#define _zero_arr_plus_longdouble_fn _zero_arr_plus_fn
#define _zero_arr_plus_voidstar_fn _zero_arr_plus_fn
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (!_cks_ptr) _throw_null(); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })

/* _zero_arr_plus_*_fn(x,sz,i,filename,lineno) adds i to zero-terminated ptr
   x that has at least sz elements */
char* _zero_arr_plus_char_fn(char*,unsigned,int,const char*,unsigned);
short* _zero_arr_plus_short_fn(short*,unsigned,int,const char*,unsigned);
int* _zero_arr_plus_int_fn(int*,unsigned,int,const char*,unsigned);
float* _zero_arr_plus_float_fn(float*,unsigned,int,const char*,unsigned);
double* _zero_arr_plus_double_fn(double*,unsigned,int,const char*,unsigned);
long double* _zero_arr_plus_longdouble_fn(long double*,unsigned,int,const char*, unsigned);
void** _zero_arr_plus_voidstar_fn(void**,unsigned,int,const char*,unsigned);
#endif

/* _get_zero_arr_size_*(x,sz) returns the number of elements in a
   zero-terminated array that is NULL or has at least sz elements */
int _get_zero_arr_size_char(const char*,unsigned);
int _get_zero_arr_size_short(const short*,unsigned);
int _get_zero_arr_size_int(const int*,unsigned);
int _get_zero_arr_size_float(const float*,unsigned);
int _get_zero_arr_size_double(const double*,unsigned);
int _get_zero_arr_size_longdouble(const long double*,unsigned);
int _get_zero_arr_size_voidstar(const void**,unsigned);

/* _zero_arr_inplace_plus_*_fn(x,i,filename,lineno) sets
   zero-terminated pointer *x to *x + i */
char* _zero_arr_inplace_plus_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_short_fn(short**,int,const char*,unsigned);
int* _zero_arr_inplace_plus_int(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_longdouble_fn(long double**,int,const char*,unsigned);
void** _zero_arr_inplace_plus_voidstar_fn(void***,int,const char*,unsigned);
/* like the previous functions, but does post-addition (as in e++) */
char* _zero_arr_inplace_plus_post_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_post_short_fn(short**x,int,const char*,unsigned);
int* _zero_arr_inplace_plus_post_int_fn(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_post_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_post_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_post_longdouble_fn(long double**,int,const char *,unsigned);
void** _zero_arr_inplace_plus_post_voidstar_fn(void***,int,const char*,unsigned);
#define _zero_arr_plus_char(x,s,i) \
  (_zero_arr_plus_char_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_short(x,s,i) \
  (_zero_arr_plus_short_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_int(x,s,i) \
  (_zero_arr_plus_int_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_float(x,s,i) \
  (_zero_arr_plus_float_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_double(x,s,i) \
  (_zero_arr_plus_double_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_longdouble(x,s,i) \
  (_zero_arr_plus_longdouble_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_voidstar(x,s,i) \
  (_zero_arr_plus_voidstar_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_inplace_plus_char(x,i) \
  _zero_arr_inplace_plus_char_fn((char **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_short(x,i) \
  _zero_arr_inplace_plus_short_fn((short **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_int(x,i) \
  _zero_arr_inplace_plus_int_fn((int **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_float(x,i) \
  _zero_arr_inplace_plus_float_fn((float **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_double(x,i) \
  _zero_arr_inplace_plus_double_fn((double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_longdouble(x,i) \
  _zero_arr_inplace_plus_longdouble_fn((long double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_voidstar(x,i) \
  _zero_arr_inplace_plus_voidstar_fn((void ***)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_char(x,i) \
  _zero_arr_inplace_plus_post_char_fn((char **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_short(x,i) \
  _zero_arr_inplace_plus_post_short_fn((short **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_int(x,i) \
  _zero_arr_inplace_plus_post_int_fn((int **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_float(x,i) \
  _zero_arr_inplace_plus_post_float_fn((float **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_double(x,i) \
  _zero_arr_inplace_plus_post_double_fn((double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_longdouble(x,i) \
  _zero_arr_inplace_plus_post_longdouble_fn((long double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_voidstar(x,i) \
  _zero_arr_inplace_plus_post_voidstar_fn((void***)(x),(i),__FILE__,__LINE__)

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_fat_subscript(arr,elt_sz,index) ((arr).curr + (elt_sz) * (index))
#define _untag_fat_ptr(arr,elt_sz,num_elts) ((arr).curr)
#else
#define _check_fat_subscript(arr,elt_sz,index) ({ \
  struct _fat_ptr _cus_arr = (arr); \
  unsigned char *_cus_ans = _cus_arr.curr + (elt_sz) * (index); \
  /* JGM: not needed! if (!_cus_arr.base) _throw_null();*/ \
  if (_cus_ans < _cus_arr.base || _cus_ans >= _cus_arr.last_plus_one) \
    _throw_arraybounds(); \
  _cus_ans; })
#define _untag_fat_ptr(arr,elt_sz,num_elts) ({ \
  struct _fat_ptr _arr = (arr); \
  unsigned char *_curr = _arr.curr; \
  if ((_curr < _arr.base || _curr + (elt_sz) * (num_elts) > _arr.last_plus_one) &&\
      _curr != (unsigned char *)0) \
    _throw_arraybounds(); \
  _curr; })
#endif

#define _tag_fat(tcurr,elt_sz,num_elts) ({ \
  struct _fat_ptr _ans; \
  unsigned _num_elts = (num_elts);\
  _ans.base = _ans.curr = (void*)(tcurr); \
  /* JGM: if we're tagging NULL, ignore num_elts */ \
  _ans.last_plus_one = _ans.base ? (_ans.base + (elt_sz) * _num_elts) : 0; \
  _ans; })

#define _get_fat_size(arr,elt_sz) \
  ({struct _fat_ptr _arr = (arr); \
    unsigned char *_arr_curr=_arr.curr; \
    unsigned char *_arr_last=_arr.last_plus_one; \
    (_arr_curr < _arr.base || _arr_curr >= _arr_last) ? 0 : \
    ((_arr_last - _arr_curr) / (elt_sz));})

#define _fat_ptr_plus(arr,elt_sz,change) ({ \
  struct _fat_ptr _ans = (arr); \
  int _change = (change);\
  _ans.curr += (elt_sz) * _change;\
  _ans; })
#define _fat_ptr_inplace_plus(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  _arr_ptr->curr += (elt_sz) * (change);\
  *_arr_ptr; })
#define _fat_ptr_inplace_plus_post(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  struct _fat_ptr _ans = *_arr_ptr; \
  _arr_ptr->curr += (elt_sz) * (change);\
  _ans; })

//Not a macro since initialization order matters. Defined in runtime_zeroterm.c.
struct _fat_ptr _fat_ptr_decrease_size(struct _fat_ptr,unsigned sz,unsigned numelts);

/* Allocation */
void* GC_malloc(int);
void* GC_malloc_atomic(int);
void* GC_calloc(unsigned,unsigned);
void* GC_calloc_atomic(unsigned,unsigned);
// bound the allocation size to be < MAX_ALLOC_SIZE. See macros below for usage.
#define MAX_MALLOC_SIZE (1 << 28)
void* _bounded_GC_malloc(int,const char*,int);
void* _bounded_GC_malloc_atomic(int,const char*,int);
void* _bounded_GC_calloc(unsigned,unsigned,const char*,int);
void* _bounded_GC_calloc_atomic(unsigned,unsigned,const char*,int);
/* these macros are overridden below ifdef CYC_REGION_PROFILE */
#ifndef CYC_REGION_PROFILE
#define _cycalloc(n) _bounded_GC_malloc(n,__FILE__,__LINE__)
#define _cycalloc_atomic(n) _bounded_GC_malloc_atomic(n,__FILE__,__LINE__)
#define _cyccalloc(n,s) _bounded_GC_calloc(n,s,__FILE__,__LINE__)
#define _cyccalloc_atomic(n,s) _bounded_GC_calloc_atomic(n,s,__FILE__,__LINE__)
#endif

static _INLINE unsigned int _check_times(unsigned x, unsigned y) {
  unsigned long long whole_ans = 
    ((unsigned long long) x)*((unsigned long long)y);
  unsigned word_ans = (unsigned)whole_ans;
  if(word_ans < whole_ans || word_ans > MAX_MALLOC_SIZE)
    _throw_badalloc();
  return word_ans;
}

#define _CYC_MAX_REGION_CONST 2
#define _CYC_MIN_ALIGNMENT (sizeof(double))

#ifdef CYC_REGION_PROFILE
extern int rgn_total_bytes;
#endif

static _INLINE void *_fast_region_malloc(struct _RegionHandle *r, unsigned orig_s) {  
  if (r > (struct _RegionHandle *)_CYC_MAX_REGION_CONST && r->curr != 0) { 
#ifdef CYC_NOALIGN
    unsigned s =  orig_s;
#else
    unsigned s =  (orig_s + _CYC_MIN_ALIGNMENT - 1) & (~(_CYC_MIN_ALIGNMENT -1)); 
#endif
    char *result; 
    result = r->offset; 
    if (s <= (r->last_plus_one - result)) {
      r->offset = result + s; 
#ifdef CYC_REGION_PROFILE
    r->curr->free_bytes = r->curr->free_bytes - s;
    rgn_total_bytes += s;
#endif
      return result;
    }
  } 
  return _region_malloc(r,orig_s); 
}

#ifdef CYC_REGION_PROFILE
/* see macros below for usage. defined in runtime_memory.c */
void* _profile_GC_malloc(int,const char*,const char*,int);
void* _profile_GC_malloc_atomic(int,const char*,const char*,int);
void* _profile_GC_calloc(unsigned,unsigned,const char*,const char*,int);
void* _profile_GC_calloc_atomic(unsigned,unsigned,const char*,const char*,int);
void* _profile_region_malloc(struct _RegionHandle*,unsigned,const char*,const char*,int);
void* _profile_region_calloc(struct _RegionHandle*,unsigned,unsigned,const char *,const char*,int);
struct _RegionHandle _profile_new_region(const char*,const char*,const char*,int);
void _profile_free_region(struct _RegionHandle*,const char*,const char*,int);
#ifndef RUNTIME_CYC
#define _new_region(n) _profile_new_region(n,__FILE__,__FUNCTION__,__LINE__)
#define _free_region(r) _profile_free_region(r,__FILE__,__FUNCTION__,__LINE__)
#define _region_malloc(rh,n) _profile_region_malloc(rh,n,__FILE__,__FUNCTION__,__LINE__)
#define _region_calloc(rh,n,t) _profile_region_calloc(rh,n,t,__FILE__,__FUNCTION__,__LINE__)
#  endif
#define _cycalloc(n) _profile_GC_malloc(n,__FILE__,__FUNCTION__,__LINE__)
#define _cycalloc_atomic(n) _profile_GC_malloc_atomic(n,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc(n,s) _profile_GC_calloc(n,s,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc_atomic(n,s) _profile_GC_calloc_atomic(n,s,__FILE__,__FUNCTION__,__LINE__)
#endif
#endif
 struct Cyc_Core_Opt{void*v;};extern char Cyc_Core_Invalid_argument[17U];struct Cyc_Core_Invalid_argument_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Failure[8U];struct Cyc_Core_Failure_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Impossible[11U];struct Cyc_Core_Impossible_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Not_found[10U];struct Cyc_Core_Not_found_exn_struct{char*tag;};extern char Cyc_Core_Unreachable[12U];struct Cyc_Core_Unreachable_exn_struct{char*tag;struct _fat_ptr f1;};
# 168 "core.h"
extern struct _RegionHandle*Cyc_Core_unique_region;struct Cyc_Core_DynamicRegion;struct Cyc_Core_NewDynamicRegion{struct Cyc_Core_DynamicRegion*key;};struct Cyc_Core_ThinRes{void*arr;unsigned nelts;};struct Cyc_List_List{void*hd;struct Cyc_List_List*tl;};extern char Cyc_List_List_mismatch[14U];struct Cyc_List_List_mismatch_exn_struct{char*tag;};
# 178 "list.h"
struct Cyc_List_List*Cyc_List_imp_rev(struct Cyc_List_List*x);extern char Cyc_List_Nth[4U];struct Cyc_List_Nth_exn_struct{char*tag;};struct Cyc___cycFILE;struct Cyc_String_pa_PrintArg_struct{int tag;struct _fat_ptr f1;};struct Cyc_Int_pa_PrintArg_struct{int tag;unsigned long f1;};struct Cyc_Double_pa_PrintArg_struct{int tag;double f1;};struct Cyc_LongDouble_pa_PrintArg_struct{int tag;long double f1;};struct Cyc_ShortPtr_pa_PrintArg_struct{int tag;short*f1;};struct Cyc_IntPtr_pa_PrintArg_struct{int tag;unsigned long*f1;};struct Cyc_ShortPtr_sa_ScanfArg_struct{int tag;short*f1;};struct Cyc_UShortPtr_sa_ScanfArg_struct{int tag;unsigned short*f1;};struct Cyc_IntPtr_sa_ScanfArg_struct{int tag;int*f1;};struct Cyc_UIntPtr_sa_ScanfArg_struct{int tag;unsigned*f1;};struct Cyc_StringPtr_sa_ScanfArg_struct{int tag;struct _fat_ptr f1;};struct Cyc_DoublePtr_sa_ScanfArg_struct{int tag;double*f1;};struct Cyc_FloatPtr_sa_ScanfArg_struct{int tag;float*f1;};struct Cyc_CharPtr_sa_ScanfArg_struct{int tag;struct _fat_ptr f1;};extern char Cyc_FileCloseError[15U];struct Cyc_FileCloseError_exn_struct{char*tag;};extern char Cyc_FileOpenError[14U];struct Cyc_FileOpenError_exn_struct{char*tag;struct _fat_ptr f1;};
# 290 "cycboot.h"
int isdigit(int);
# 38 "string.h"
unsigned long Cyc_strlen(struct _fat_ptr s);
# 136 "string.h"
struct _fat_ptr Cyc_implode(struct Cyc_List_List*c);struct Cyc_Position_Error;struct Cyc_Relations_Reln;struct _union_Nmspace_Rel_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_Abs_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_C_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_Loc_n{int tag;int val;};union Cyc_Absyn_Nmspace{struct _union_Nmspace_Rel_n Rel_n;struct _union_Nmspace_Abs_n Abs_n;struct _union_Nmspace_C_n C_n;struct _union_Nmspace_Loc_n Loc_n;};struct _tuple0{union Cyc_Absyn_Nmspace f1;struct _fat_ptr*f2;};
# 161 "absyn.h"
enum Cyc_Absyn_Scope{Cyc_Absyn_Static =0U,Cyc_Absyn_Abstract =1U,Cyc_Absyn_Public =2U,Cyc_Absyn_Extern =3U,Cyc_Absyn_ExternC =4U,Cyc_Absyn_Register =5U};struct Cyc_Absyn_Tqual{int print_const: 1;int q_volatile: 1;int q_restrict: 1;int real_const: 1;unsigned loc;};
# 182
enum Cyc_Absyn_Size_of{Cyc_Absyn_Char_sz =0U,Cyc_Absyn_Short_sz =1U,Cyc_Absyn_Int_sz =2U,Cyc_Absyn_Long_sz =3U,Cyc_Absyn_LongLong_sz =4U};
enum Cyc_Absyn_Sign{Cyc_Absyn_Signed =0U,Cyc_Absyn_Unsigned =1U,Cyc_Absyn_None =2U};
enum Cyc_Absyn_AggrKind{Cyc_Absyn_StructA =0U,Cyc_Absyn_UnionA =1U};
# 187
enum Cyc_Absyn_AliasQual{Cyc_Absyn_Aliasable =0U,Cyc_Absyn_Unique =1U,Cyc_Absyn_Top =2U};
# 192
enum Cyc_Absyn_KindQual{Cyc_Absyn_AnyKind =0U,Cyc_Absyn_MemKind =1U,Cyc_Absyn_BoxKind =2U,Cyc_Absyn_RgnKind =3U,Cyc_Absyn_EffKind =4U,Cyc_Absyn_IntKind =5U,Cyc_Absyn_BoolKind =6U,Cyc_Absyn_PtrBndKind =7U};struct Cyc_Absyn_Kind{enum Cyc_Absyn_KindQual kind;enum Cyc_Absyn_AliasQual aliasqual;};struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct{int tag;struct Cyc_Absyn_Kind*f1;};struct Cyc_Absyn_Unknown_kb_Absyn_KindBound_struct{int tag;struct Cyc_Core_Opt*f1;};struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct{int tag;struct Cyc_Core_Opt*f1;struct Cyc_Absyn_Kind*f2;};struct Cyc_Absyn_Tvar{struct _fat_ptr*name;int identity;void*kind;};struct Cyc_Absyn_PtrLoc{unsigned ptr_loc;unsigned rgn_loc;unsigned zt_loc;};struct Cyc_Absyn_PtrAtts{void*rgn;void*nullable;void*bounds;void*zero_term;struct Cyc_Absyn_PtrLoc*ptrloc;};struct Cyc_Absyn_PtrInfo{void*elt_type;struct Cyc_Absyn_Tqual elt_tq;struct Cyc_Absyn_PtrAtts ptr_atts;};struct Cyc_Absyn_VarargInfo{struct _fat_ptr*name;struct Cyc_Absyn_Tqual tq;void*type;int inject;};struct Cyc_Absyn_Star_Absyn_Cap_struct{int tag;};struct Cyc_Absyn_Bot_Absyn_Cap_struct{int tag;};struct Cyc_Absyn_Nat_Absyn_Cap_struct{int tag;struct Cyc_Absyn_Exp*f1;int f2;};struct Cyc_Absyn_RgnEffect{void*name;void*rgn;void*lock;void*unadopt;void*parent;};struct Cyc_Absyn_FnInfo{struct Cyc_List_List*tvars;void*effect;struct Cyc_Absyn_Tqual ret_tqual;void*ret_type;struct Cyc_List_List*args;int c_varargs;struct Cyc_Absyn_VarargInfo*cyc_varargs;struct Cyc_List_List*rgn_po;struct Cyc_List_List*attributes;struct Cyc_Absyn_Exp*requires_clause;struct Cyc_List_List*requires_relns;struct Cyc_Absyn_Exp*ensures_clause;struct Cyc_List_List*ensures_relns;struct Cyc_List_List*ieffect;struct Cyc_List_List*oeffect;};struct Cyc_Absyn_UnknownDatatypeInfo{struct _tuple0*name;int is_extensible;};struct _union_DatatypeInfo_UnknownDatatype{int tag;struct Cyc_Absyn_UnknownDatatypeInfo val;};struct _union_DatatypeInfo_KnownDatatype{int tag;struct Cyc_Absyn_Datatypedecl**val;};union Cyc_Absyn_DatatypeInfo{struct _union_DatatypeInfo_UnknownDatatype UnknownDatatype;struct _union_DatatypeInfo_KnownDatatype KnownDatatype;};struct Cyc_Absyn_UnknownDatatypeFieldInfo{struct _tuple0*datatype_name;struct _tuple0*field_name;int is_extensible;};struct _union_DatatypeFieldInfo_UnknownDatatypefield{int tag;struct Cyc_Absyn_UnknownDatatypeFieldInfo val;};struct _tuple1{struct Cyc_Absyn_Datatypedecl*f1;struct Cyc_Absyn_Datatypefield*f2;};struct _union_DatatypeFieldInfo_KnownDatatypefield{int tag;struct _tuple1 val;};union Cyc_Absyn_DatatypeFieldInfo{struct _union_DatatypeFieldInfo_UnknownDatatypefield UnknownDatatypefield;struct _union_DatatypeFieldInfo_KnownDatatypefield KnownDatatypefield;};struct _tuple2{enum Cyc_Absyn_AggrKind f1;struct _tuple0*f2;struct Cyc_Core_Opt*f3;};struct _union_AggrInfo_UnknownAggr{int tag;struct _tuple2 val;};struct _union_AggrInfo_KnownAggr{int tag;struct Cyc_Absyn_Aggrdecl**val;};union Cyc_Absyn_AggrInfo{struct _union_AggrInfo_UnknownAggr UnknownAggr;struct _union_AggrInfo_KnownAggr KnownAggr;};struct Cyc_Absyn_ArrayInfo{void*elt_type;struct Cyc_Absyn_Tqual tq;struct Cyc_Absyn_Exp*num_elts;void*zero_term;unsigned zt_loc;};struct Cyc_Absyn_Aggr_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Aggrdecl*f1;};struct Cyc_Absyn_Enum_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;};struct Cyc_Absyn_Datatype_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;};struct Cyc_Absyn_TypeDecl{void*r;unsigned loc;};struct Cyc_Absyn_VoidCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_IntCon_Absyn_TyCon_struct{int tag;enum Cyc_Absyn_Sign f1;enum Cyc_Absyn_Size_of f2;};struct Cyc_Absyn_FloatCon_Absyn_TyCon_struct{int tag;int f1;};struct Cyc_Absyn_RgnHandleCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_TagCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_HeapCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_UniqueCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_RefCntCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_AccessCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_JoinCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_RgnsCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_TrueCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_FalseCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_ThinCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_FatCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_EnumCon_Absyn_TyCon_struct{int tag;struct _tuple0*f1;struct Cyc_Absyn_Enumdecl*f2;};struct Cyc_Absyn_AnonEnumCon_Absyn_TyCon_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_BuiltinCon_Absyn_TyCon_struct{int tag;struct _fat_ptr f1;struct Cyc_Absyn_Kind*f2;};struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_DatatypeInfo f1;};struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_DatatypeFieldInfo f1;};struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_AggrInfo f1;};struct Cyc_Absyn_AppType_Absyn_Type_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Evar_Absyn_Type_struct{int tag;struct Cyc_Core_Opt*f1;void*f2;int f3;struct Cyc_Core_Opt*f4;};struct Cyc_Absyn_VarType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Tvar*f1;};struct Cyc_Absyn_PointerType_Absyn_Type_struct{int tag;struct Cyc_Absyn_PtrInfo f1;};struct Cyc_Absyn_ArrayType_Absyn_Type_struct{int tag;struct Cyc_Absyn_ArrayInfo f1;};struct Cyc_Absyn_FnType_Absyn_Type_struct{int tag;struct Cyc_Absyn_FnInfo f1;};struct Cyc_Absyn_TupleType_Absyn_Type_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct{int tag;enum Cyc_Absyn_AggrKind f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_TypedefType_Absyn_Type_struct{int tag;struct _tuple0*f1;struct Cyc_List_List*f2;struct Cyc_Absyn_Typedefdecl*f3;void*f4;};struct Cyc_Absyn_ValueofType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct{int tag;struct Cyc_Absyn_TypeDecl*f1;void**f2;};struct Cyc_Absyn_TypeofType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_NoTypes_Absyn_Funcparams_struct{int tag;struct Cyc_List_List*f1;unsigned f2;};struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct{int tag;struct Cyc_List_List*f1;int f2;struct Cyc_Absyn_VarargInfo*f3;void*f4;struct Cyc_List_List*f5;struct Cyc_Absyn_Exp*f6;struct Cyc_Absyn_Exp*f7;struct Cyc_List_List*f8;struct Cyc_List_List*f9;};
# 437 "absyn.h"
enum Cyc_Absyn_Format_Type{Cyc_Absyn_Printf_ft =0U,Cyc_Absyn_Scanf_ft =1U};struct Cyc_Absyn_Regparm_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Stdcall_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Cdecl_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Fastcall_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Noreturn_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Const_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Aligned_att_Absyn_Attribute_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Packed_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Section_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Nocommon_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Shared_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Unused_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Weak_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Dllimport_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Dllexport_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_instrument_function_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Constructor_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Destructor_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_check_memory_usage_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Format_att_Absyn_Attribute_struct{int tag;enum Cyc_Absyn_Format_Type f1;int f2;int f3;};struct Cyc_Absyn_Initializes_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Noliveunique_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Consume_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Pure_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Mode_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Alias_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Always_inline_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_throw_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Non_null_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Deprecated_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Carray_mod_Absyn_Type_modifier_struct{int tag;void*f1;unsigned f2;};struct Cyc_Absyn_ConstArray_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_Absyn_Exp*f1;void*f2;unsigned f3;};struct Cyc_Absyn_Pointer_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_Absyn_PtrAtts f1;struct Cyc_Absyn_Tqual f2;};struct Cyc_Absyn_Function_mod_Absyn_Type_modifier_struct{int tag;void*f1;};struct Cyc_Absyn_TypeParams_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_List_List*f1;unsigned f2;int f3;};struct Cyc_Absyn_Attributes_mod_Absyn_Type_modifier_struct{int tag;unsigned f1;struct Cyc_List_List*f2;};struct _union_Cnst_Null_c{int tag;int val;};struct _tuple3{enum Cyc_Absyn_Sign f1;char f2;};struct _union_Cnst_Char_c{int tag;struct _tuple3 val;};struct _union_Cnst_Wchar_c{int tag;struct _fat_ptr val;};struct _tuple4{enum Cyc_Absyn_Sign f1;short f2;};struct _union_Cnst_Short_c{int tag;struct _tuple4 val;};struct _tuple5{enum Cyc_Absyn_Sign f1;int f2;};struct _union_Cnst_Int_c{int tag;struct _tuple5 val;};struct _tuple6{enum Cyc_Absyn_Sign f1;long long f2;};struct _union_Cnst_LongLong_c{int tag;struct _tuple6 val;};struct _tuple7{struct _fat_ptr f1;int f2;};struct _union_Cnst_Float_c{int tag;struct _tuple7 val;};struct _union_Cnst_String_c{int tag;struct _fat_ptr val;};struct _union_Cnst_Wstring_c{int tag;struct _fat_ptr val;};union Cyc_Absyn_Cnst{struct _union_Cnst_Null_c Null_c;struct _union_Cnst_Char_c Char_c;struct _union_Cnst_Wchar_c Wchar_c;struct _union_Cnst_Short_c Short_c;struct _union_Cnst_Int_c Int_c;struct _union_Cnst_LongLong_c LongLong_c;struct _union_Cnst_Float_c Float_c;struct _union_Cnst_String_c String_c;struct _union_Cnst_Wstring_c Wstring_c;};
# 532
enum Cyc_Absyn_Primop{Cyc_Absyn_Plus =0U,Cyc_Absyn_Times =1U,Cyc_Absyn_Minus =2U,Cyc_Absyn_Div =3U,Cyc_Absyn_Mod =4U,Cyc_Absyn_Eq =5U,Cyc_Absyn_Neq =6U,Cyc_Absyn_Gt =7U,Cyc_Absyn_Lt =8U,Cyc_Absyn_Gte =9U,Cyc_Absyn_Lte =10U,Cyc_Absyn_Not =11U,Cyc_Absyn_Bitnot =12U,Cyc_Absyn_Bitand =13U,Cyc_Absyn_Bitor =14U,Cyc_Absyn_Bitxor =15U,Cyc_Absyn_Bitlshift =16U,Cyc_Absyn_Bitlrshift =17U,Cyc_Absyn_Numelts =18U};
# 539
enum Cyc_Absyn_Incrementor{Cyc_Absyn_PreInc =0U,Cyc_Absyn_PostInc =1U,Cyc_Absyn_PreDec =2U,Cyc_Absyn_PostDec =3U};struct Cyc_Absyn_VarargCallInfo{int num_varargs;struct Cyc_List_List*injectors;struct Cyc_Absyn_VarargInfo*vai;};struct Cyc_Absyn_StructField_Absyn_OffsetofField_struct{int tag;struct _fat_ptr*f1;};struct Cyc_Absyn_TupleIndex_Absyn_OffsetofField_struct{int tag;unsigned f1;};
# 557
enum Cyc_Absyn_Coercion{Cyc_Absyn_Unknown_coercion =0U,Cyc_Absyn_No_coercion =1U,Cyc_Absyn_Null_to_NonNull =2U,Cyc_Absyn_Other_coercion =3U};struct Cyc_Absyn_MallocInfo{int is_calloc;struct Cyc_Absyn_Exp*rgn;void**elt_type;struct Cyc_Absyn_Exp*num_elts;int fat_result;int inline_call;};struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct{int tag;union Cyc_Absyn_Cnst f1;};struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Pragma_e_Absyn_Raw_exp_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct{int tag;enum Cyc_Absyn_Primop f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_AssignOp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Core_Opt*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_Increment_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;enum Cyc_Absyn_Incrementor f2;};struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;struct Cyc_Absyn_VarargCallInfo*f3;int f4;};struct Cyc_Absyn_Throw_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;int f2;};struct Cyc_Absyn_NoInstantiate_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Instantiate_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_Absyn_Exp*f2;int f3;enum Cyc_Absyn_Coercion f4;};struct Cyc_Absyn_Address_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_New_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Sizeoftype_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Sizeofexp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Offsetof_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Deref_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;int f3;int f4;};struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;int f3;int f4;};struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Tuple_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;};struct _tuple8{struct _fat_ptr*f1;struct Cyc_Absyn_Tqual f2;void*f3;};struct Cyc_Absyn_CompoundLit_e_Absyn_Raw_exp_struct{int tag;struct _tuple8*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Array_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Exp*f2;struct Cyc_Absyn_Exp*f3;int f4;};struct Cyc_Absyn_ComprehensionNoinit_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;void*f2;int f3;};struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct{int tag;struct _tuple0*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;struct Cyc_Absyn_Aggrdecl*f4;};struct Cyc_Absyn_AnonStruct_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Datatype_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;struct Cyc_Absyn_Datatypedecl*f2;struct Cyc_Absyn_Datatypefield*f3;};struct Cyc_Absyn_Enum_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_AnonEnum_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_Spawn_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_MallocInfo f1;};struct Cyc_Absyn_Swap_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_UnresolvedMem_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Core_Opt*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_StmtExp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Stmt*f1;};struct Cyc_Absyn_Tagcheck_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;};struct Cyc_Absyn_Valueof_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Asm_e_Absyn_Raw_exp_struct{int tag;int f1;struct _fat_ptr f2;struct Cyc_List_List*f3;struct Cyc_List_List*f4;struct Cyc_List_List*f5;};struct Cyc_Absyn_Extension_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Exp{void*topt;void*r;unsigned loc;void*annot;};struct Cyc_Absyn_Skip_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Exp_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Seq_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Return_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_IfThenElse_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Stmt*f2;struct Cyc_Absyn_Stmt*f3;};struct _tuple9{struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_While_s_Absyn_Raw_stmt_struct{int tag;struct _tuple9 f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Break_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Continue_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Goto_s_Absyn_Raw_stmt_struct{int tag;struct _fat_ptr*f1;};struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _tuple9 f2;struct _tuple9 f3;struct Cyc_Absyn_Stmt*f4;};struct Cyc_Absyn_Switch_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;void*f3;};struct Cyc_Absyn_Fallthru_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_List_List*f1;struct Cyc_Absyn_Switch_clause**f2;};struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Decl*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Label_s_Absyn_Raw_stmt_struct{int tag;struct _fat_ptr*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Do_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct _tuple9 f2;};struct Cyc_Absyn_TryCatch_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct Cyc_List_List*f2;void*f3;};struct Cyc_Absyn_Stmt{void*r;unsigned loc;void*annot;};struct Cyc_Absyn_Wild_p_Absyn_Raw_pat_struct{int tag;};struct Cyc_Absyn_Var_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Pat*f2;};struct Cyc_Absyn_AliasVar_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;};struct Cyc_Absyn_Reference_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Pat*f2;};struct Cyc_Absyn_TagInt_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;};struct Cyc_Absyn_Tuple_p_Absyn_Raw_pat_struct{int tag;struct Cyc_List_List*f1;int f2;};struct Cyc_Absyn_Pointer_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Pat*f1;};struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct{int tag;union Cyc_Absyn_AggrInfo*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;int f4;};struct Cyc_Absyn_Datatype_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;struct Cyc_Absyn_Datatypefield*f2;struct Cyc_List_List*f3;int f4;};struct Cyc_Absyn_Null_p_Absyn_Raw_pat_struct{int tag;};struct Cyc_Absyn_Int_p_Absyn_Raw_pat_struct{int tag;enum Cyc_Absyn_Sign f1;int f2;};struct Cyc_Absyn_Char_p_Absyn_Raw_pat_struct{int tag;char f1;};struct Cyc_Absyn_Float_p_Absyn_Raw_pat_struct{int tag;struct _fat_ptr f1;int f2;};struct Cyc_Absyn_Enum_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_AnonEnum_p_Absyn_Raw_pat_struct{int tag;void*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_UnknownId_p_Absyn_Raw_pat_struct{int tag;struct _tuple0*f1;};struct Cyc_Absyn_UnknownCall_p_Absyn_Raw_pat_struct{int tag;struct _tuple0*f1;struct Cyc_List_List*f2;int f3;};struct Cyc_Absyn_Exp_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Pat{void*r;void*topt;unsigned loc;};struct Cyc_Absyn_Switch_clause{struct Cyc_Absyn_Pat*pattern;struct Cyc_Core_Opt*pat_vars;struct Cyc_Absyn_Exp*where_clause;struct Cyc_Absyn_Stmt*body;unsigned loc;};struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct{int tag;struct _tuple0*f1;};struct Cyc_Absyn_Global_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Funname_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Fndecl*f1;};struct Cyc_Absyn_Param_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Local_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Pat_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Vardecl{enum Cyc_Absyn_Scope sc;struct _tuple0*name;unsigned varloc;struct Cyc_Absyn_Tqual tq;void*type;struct Cyc_Absyn_Exp*initializer;void*rgn;struct Cyc_List_List*attributes;int escapes;};struct Cyc_Absyn_Fndecl{enum Cyc_Absyn_Scope sc;int is_inline;struct _tuple0*name;struct Cyc_Absyn_Stmt*body;struct Cyc_Absyn_FnInfo i;void*cached_type;struct Cyc_Core_Opt*param_vardecls;struct Cyc_Absyn_Vardecl*fn_vardecl;};struct Cyc_Absyn_Aggrfield{struct _fat_ptr*name;struct Cyc_Absyn_Tqual tq;void*type;struct Cyc_Absyn_Exp*width;struct Cyc_List_List*attributes;struct Cyc_Absyn_Exp*requires_clause;};struct Cyc_Absyn_AggrdeclImpl{struct Cyc_List_List*exist_vars;struct Cyc_List_List*rgn_po;struct Cyc_List_List*fields;int tagged;};struct Cyc_Absyn_Aggrdecl{enum Cyc_Absyn_AggrKind kind;enum Cyc_Absyn_Scope sc;struct _tuple0*name;struct Cyc_List_List*tvs;struct Cyc_Absyn_AggrdeclImpl*impl;struct Cyc_List_List*attributes;int expected_mem_kind;};struct Cyc_Absyn_Datatypefield{struct _tuple0*name;struct Cyc_List_List*typs;unsigned loc;enum Cyc_Absyn_Scope sc;};struct Cyc_Absyn_Datatypedecl{enum Cyc_Absyn_Scope sc;struct _tuple0*name;struct Cyc_List_List*tvs;struct Cyc_Core_Opt*fields;int is_extensible;};struct Cyc_Absyn_Enumfield{struct _tuple0*name;struct Cyc_Absyn_Exp*tag;unsigned loc;};struct Cyc_Absyn_Enumdecl{enum Cyc_Absyn_Scope sc;struct _tuple0*name;struct Cyc_Core_Opt*fields;};struct Cyc_Absyn_Typedefdecl{struct _tuple0*name;struct Cyc_Absyn_Tqual tq;struct Cyc_List_List*tvs;struct Cyc_Core_Opt*kind;void*defn;struct Cyc_List_List*atts;int extern_c;};struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Fn_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Fndecl*f1;};struct Cyc_Absyn_Let_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Pat*f1;struct Cyc_Core_Opt*f2;struct Cyc_Absyn_Exp*f3;void*f4;};struct Cyc_Absyn_Letv_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_Region_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_Aggr_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Aggrdecl*f1;};struct Cyc_Absyn_Datatype_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;};struct Cyc_Absyn_Enum_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;};struct Cyc_Absyn_Typedef_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Typedefdecl*f1;};struct Cyc_Absyn_Namespace_d_Absyn_Raw_decl_struct{int tag;struct _fat_ptr*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Using_d_Absyn_Raw_decl_struct{int tag;struct _tuple0*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_ExternC_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;};struct _tuple10{unsigned f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_ExternCinclude_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;struct _tuple10*f4;};struct Cyc_Absyn_Porton_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Portoff_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Tempeston_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Tempestoff_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Decl{void*r;unsigned loc;};struct Cyc_Absyn_ArrayElement_Absyn_Designator_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_FieldName_Absyn_Designator_struct{int tag;struct _fat_ptr*f1;};extern char Cyc_Absyn_EmptyAnnot[11U];struct Cyc_Absyn_EmptyAnnot_Absyn_AbsynAnnot_struct{char*tag;};
# 916 "absyn.h"
struct Cyc_Absyn_Tqual Cyc_Absyn_const_tqual(unsigned);
struct Cyc_Absyn_Tqual Cyc_Absyn_empty_tqual(unsigned);
# 933
void*Cyc_Absyn_new_evar(struct Cyc_Core_Opt*k,struct Cyc_Core_Opt*tenv);
# 938
extern void*Cyc_Absyn_char_type;extern void*Cyc_Absyn_uchar_type;extern void*Cyc_Absyn_ushort_type;extern void*Cyc_Absyn_uint_type;extern void*Cyc_Absyn_ulong_type;
# 940
extern void*Cyc_Absyn_schar_type;extern void*Cyc_Absyn_sshort_type;extern void*Cyc_Absyn_sint_type;extern void*Cyc_Absyn_slong_type;
# 942
extern void*Cyc_Absyn_float_type;extern void*Cyc_Absyn_double_type;extern void*Cyc_Absyn_long_double_type;
# 949
extern void*Cyc_Absyn_true_type;extern void*Cyc_Absyn_false_type;
# 992
void*Cyc_Absyn_at_type(void*t,void*rgn,struct Cyc_Absyn_Tqual,void*zero_term);
# 996
void*Cyc_Absyn_fatptr_type(void*t,void*rgn,struct Cyc_Absyn_Tqual,void*zeroterm);struct _union_RelnOp_RConst{int tag;unsigned val;};struct _union_RelnOp_RVar{int tag;struct Cyc_Absyn_Vardecl*val;};struct _union_RelnOp_RNumelts{int tag;struct Cyc_Absyn_Vardecl*val;};struct _union_RelnOp_RType{int tag;void*val;};struct _union_RelnOp_RParam{int tag;unsigned val;};struct _union_RelnOp_RParamNumelts{int tag;unsigned val;};struct _union_RelnOp_RReturn{int tag;unsigned val;};union Cyc_Relations_RelnOp{struct _union_RelnOp_RConst RConst;struct _union_RelnOp_RVar RVar;struct _union_RelnOp_RNumelts RNumelts;struct _union_RelnOp_RType RType;struct _union_RelnOp_RParam RParam;struct _union_RelnOp_RParamNumelts RParamNumelts;struct _union_RelnOp_RReturn RReturn;};
# 50 "relations-ap.h"
enum Cyc_Relations_Relation{Cyc_Relations_Req =0U,Cyc_Relations_Rneq =1U,Cyc_Relations_Rlte =2U,Cyc_Relations_Rlt =3U};struct Cyc_Relations_Reln{union Cyc_Relations_RelnOp rop1;enum Cyc_Relations_Relation relation;union Cyc_Relations_RelnOp rop2;};struct Cyc_Iter_Iter{void*env;int(*next)(void*env,void*dest);};struct Cyc_Dict_T;struct Cyc_Dict_Dict{int(*rel)(void*,void*);struct _RegionHandle*r;const struct Cyc_Dict_T*t;};extern char Cyc_Dict_Present[8U];struct Cyc_Dict_Present_exn_struct{char*tag;};extern char Cyc_Dict_Absent[7U];struct Cyc_Dict_Absent_exn_struct{char*tag;};struct Cyc_RgnOrder_RgnPO;extern char Cyc_Tcenv_Env_error[10U];struct Cyc_Tcenv_Env_error_exn_struct{char*tag;};struct Cyc_Tcenv_Genv{struct Cyc_Dict_Dict aggrdecls;struct Cyc_Dict_Dict datatypedecls;struct Cyc_Dict_Dict enumdecls;struct Cyc_Dict_Dict typedefs;struct Cyc_Dict_Dict ordinaries;};struct Cyc_Tcenv_Fenv;struct Cyc_Tcenv_Tenv{struct Cyc_List_List*ns;struct Cyc_Tcenv_Genv*ae;struct Cyc_Tcenv_Fenv*le;int allow_valueof: 1;int in_extern_c_include: 1;int in_tempest: 1;int tempest_generalize: 1;};
# 88 "tcenv.h"
enum Cyc_Tcenv_NewStatus{Cyc_Tcenv_NoneNew =0U,Cyc_Tcenv_InNew =1U,Cyc_Tcenv_InNewAggr =2U};
# 98
struct Cyc_List_List*Cyc_Tcenv_lookup_type_vars(struct Cyc_Tcenv_Tenv*);
# 31 "tcutil.h"
void Cyc_Tcutil_terr(unsigned,struct _fat_ptr,struct _fat_ptr);
# 154
extern struct Cyc_Core_Opt Cyc_Tcutil_rko;struct _tuple11{struct Cyc_List_List*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;struct Cyc_List_List*f4;char f5;int f6;};
# 36 "formatstr.cyc"
struct Cyc_Core_Opt*Cyc_Formatstr_parse_conversionspecification(struct _RegionHandle*r,struct _fat_ptr s,int i){
# 40
unsigned long len=Cyc_strlen((struct _fat_ptr)s);
if(i < 0 ||(unsigned long)i >= len)return 0;{struct Cyc_List_List*flags=0;
# 45
char c=' ';
for(0;(unsigned long)i < len;++ i){
c=*((const char*)_check_fat_subscript(s,sizeof(char),i));
{char _tmp0=c;switch(_tmp0){case 43U: _LL1: _LL2:
 goto _LL4;case 45U: _LL3: _LL4:
 goto _LL6;case 32U: _LL5: _LL6:
 goto _LL8;case 35U: _LL7: _LL8:
 goto _LLA;case 48U: _LL9: _LLA:
 flags=({struct Cyc_List_List*_tmp1=_region_malloc(r,sizeof(*_tmp1));_tmp1->hd=(void*)((int)c),_tmp1->tl=flags;_tmp1;});continue;default: _LLB: _LLC:
 goto _LL0;}_LL0:;}
# 56
break;}
# 58
if((unsigned long)i >= len)return 0;flags=
Cyc_List_imp_rev(flags);{
# 62
struct Cyc_List_List*width=0;
c=*((const char*)_check_fat_subscript(s,sizeof(char),i));
if((int)c == (int)'*'){
width=({struct Cyc_List_List*_tmp2=_region_malloc(r,sizeof(*_tmp2));_tmp2->hd=(void*)((int)c),_tmp2->tl=width;_tmp2;});
++ i;}else{
# 68
for(0;(unsigned long)i < len;++ i){
c=*((const char*)_check_fat_subscript(s,sizeof(char),i));
if(isdigit((int)c))width=({struct Cyc_List_List*_tmp3=_region_malloc(r,sizeof(*_tmp3));_tmp3->hd=(void*)((int)c),_tmp3->tl=width;_tmp3;});else{
break;}}}
# 74
if((unsigned long)i >= len)return 0;width=
Cyc_List_imp_rev(width);{
# 78
struct Cyc_List_List*precision=0;
c=*((const char*)_check_fat_subscript(s,sizeof(char),i));
if((int)c == (int)'.'){
precision=({struct Cyc_List_List*_tmp4=_region_malloc(r,sizeof(*_tmp4));_tmp4->hd=(void*)((int)c),_tmp4->tl=precision;_tmp4;});
if((unsigned long)++ i >= len)return 0;c=*((const char*)_check_fat_subscript(s,sizeof(char),i));
# 84
if((int)c == (int)'*'){
precision=({struct Cyc_List_List*_tmp5=_region_malloc(r,sizeof(*_tmp5));_tmp5->hd=(void*)((int)c),_tmp5->tl=precision;_tmp5;});
++ i;}else{
# 88
for(0;(unsigned long)i < len;++ i){
c=*((const char*)_check_fat_subscript(s,sizeof(char),i));
if(isdigit((int)c))precision=({struct Cyc_List_List*_tmp6=_region_malloc(r,sizeof(*_tmp6));_tmp6->hd=(void*)((int)c),_tmp6->tl=precision;_tmp6;});else{
break;}}}}
# 80
if((unsigned long)i >= len)
# 94
return 0;
# 80
precision=
# 95
Cyc_List_imp_rev(precision);{
# 99
struct Cyc_List_List*lenmod=0;
c=*((const char*)_check_fat_subscript(s,sizeof(char),i));
{char _tmp7=c;switch(_tmp7){case 104U: _LLE: _LLF:
# 103
 lenmod=({struct Cyc_List_List*_tmp8=_region_malloc(r,sizeof(*_tmp8));_tmp8->hd=(void*)((int)c),_tmp8->tl=lenmod;_tmp8;});
if((unsigned long)++ i >= len)return 0;c=*((const char*)_check_fat_subscript(s,sizeof(char),i));
# 106
if((int)c == (int)'h'){lenmod=({struct Cyc_List_List*_tmp9=_region_malloc(r,sizeof(*_tmp9));_tmp9->hd=(void*)((int)c),_tmp9->tl=lenmod;_tmp9;});++ i;}goto _LLD;case 108U: _LL10: _LL11:
# 109
 lenmod=({struct Cyc_List_List*_tmpA=_region_malloc(r,sizeof(*_tmpA));_tmpA->hd=(void*)((int)c),_tmpA->tl=lenmod;_tmpA;});
if((unsigned long)++ i >= len)return 0;c=*((const char*)_check_fat_subscript(s,sizeof(char),i));
# 112
if((int)c == (int)'l'){lenmod=({struct Cyc_List_List*_tmpB=_region_malloc(r,sizeof(*_tmpB));_tmpB->hd=(void*)((int)c),_tmpB->tl=lenmod;_tmpB;});++ i;}goto _LLD;case 106U: _LL12: _LL13:
# 114
 goto _LL15;case 122U: _LL14: _LL15:
 goto _LL17;case 116U: _LL16: _LL17:
 goto _LL19;case 76U: _LL18: _LL19:
# 118
 lenmod=({struct Cyc_List_List*_tmpC=_region_malloc(r,sizeof(*_tmpC));_tmpC->hd=(void*)((int)c),_tmpC->tl=lenmod;_tmpC;});
++ i;
goto _LLD;default: _LL1A: _LL1B:
 goto _LLD;}_LLD:;}
# 123
if((unsigned long)i >= len)return 0;lenmod=
Cyc_List_imp_rev(lenmod);
# 127
c=*((const char*)_check_fat_subscript(s,sizeof(char),i));
{char _tmpD=c;switch(_tmpD){case 100U: _LL1D: _LL1E:
 goto _LL20;case 105U: _LL1F: _LL20:
 goto _LL22;case 111U: _LL21: _LL22:
 goto _LL24;case 117U: _LL23: _LL24:
 goto _LL26;case 120U: _LL25: _LL26:
 goto _LL28;case 88U: _LL27: _LL28:
 goto _LL2A;case 102U: _LL29: _LL2A:
 goto _LL2C;case 70U: _LL2B: _LL2C:
 goto _LL2E;case 101U: _LL2D: _LL2E:
 goto _LL30;case 69U: _LL2F: _LL30:
 goto _LL32;case 103U: _LL31: _LL32:
 goto _LL34;case 71U: _LL33: _LL34:
 goto _LL36;case 97U: _LL35: _LL36:
 goto _LL38;case 65U: _LL37: _LL38:
 goto _LL3A;case 99U: _LL39: _LL3A:
 goto _LL3C;case 115U: _LL3B: _LL3C:
 goto _LL3E;case 112U: _LL3D: _LL3E:
 goto _LL40;case 110U: _LL3F: _LL40:
 goto _LL42;case 37U: _LL41: _LL42:
 goto _LL1C;default: _LL43: _LL44:
 return 0;}_LL1C:;}
# 159 "formatstr.cyc"
return({struct Cyc_Core_Opt*_tmpF=_region_malloc(r,sizeof(*_tmpF));({struct _tuple11*_tmp105=({struct _tuple11*_tmpE=_region_malloc(r,sizeof(*_tmpE));_tmpE->f1=flags,_tmpE->f2=width,_tmpE->f3=precision,_tmpE->f4=lenmod,_tmpE->f5=c,_tmpE->f6=i + 1;_tmpE;});_tmpF->v=_tmp105;});_tmpF;});}}}}}
# 36 "formatstr.cyc"
struct Cyc_List_List*Cyc_Formatstr_get_format_types(struct Cyc_Tcenv_Tenv*te,struct _fat_ptr s,int isCproto,unsigned loc){
# 165 "formatstr.cyc"
unsigned long len=Cyc_strlen((struct _fat_ptr)s);
struct Cyc_List_List*typs=0;
int i;
struct _RegionHandle _tmp10=_new_region("temp");struct _RegionHandle*temp=& _tmp10;_push_region(temp);
for(i=0;(unsigned long)i < len;++ i){
if((int)*((const char*)_check_fat_subscript(s,sizeof(char),i))!= (int)'%')continue;{struct Cyc_Core_Opt*cs=
Cyc_Formatstr_parse_conversionspecification(temp,s,i + 1);
if(cs == 0){
({void*_tmp11=0U;({unsigned _tmp107=loc;struct _fat_ptr _tmp106=({const char*_tmp12="bad format string";_tag_fat(_tmp12,sizeof(char),18U);});Cyc_Tcutil_terr(_tmp107,_tmp106,_tag_fat(_tmp11,sizeof(void*),0U));});});{
struct Cyc_List_List*_tmp13=0;_npop_handler(0U);return _tmp13;}}{
# 172
struct _tuple11*_stmttmp0=(struct _tuple11*)cs->v;int _tmp19;char _tmp18;struct Cyc_List_List*_tmp17;struct Cyc_List_List*_tmp16;struct Cyc_List_List*_tmp15;struct Cyc_List_List*_tmp14;_LL1: _tmp14=_stmttmp0->f1;_tmp15=_stmttmp0->f2;_tmp16=_stmttmp0->f3;_tmp17=_stmttmp0->f4;_tmp18=_stmttmp0->f5;_tmp19=_stmttmp0->f6;_LL2: {struct Cyc_List_List*flags=_tmp14;struct Cyc_List_List*width=_tmp15;struct Cyc_List_List*precision=_tmp16;struct Cyc_List_List*lenmod=_tmp17;char c=_tmp18;int j=_tmp19;
# 177
i=j - 1;
{struct Cyc_List_List*_tmp1A=lenmod;int _tmp1B;if(_tmp1A != 0){if(((struct Cyc_List_List*)_tmp1A)->tl == 0){_LL4: _tmp1B=(int)_tmp1A->hd;if(
(_tmp1B == (int)'j' || _tmp1B == (int)'z')|| _tmp1B == (int)'t'){_LL5: {int x=_tmp1B;
# 182
({struct Cyc_Int_pa_PrintArg_struct _tmp1E=({struct Cyc_Int_pa_PrintArg_struct _tmpE6;_tmpE6.tag=1U,_tmpE6.f1=(unsigned long)x;_tmpE6;});void*_tmp1C[1U];_tmp1C[0]=& _tmp1E;({unsigned _tmp109=loc;struct _fat_ptr _tmp108=({const char*_tmp1D="length modifier '%c' is not supported";_tag_fat(_tmp1D,sizeof(char),38U);});Cyc_Tcutil_terr(_tmp109,_tmp108,_tag_fat(_tmp1C,sizeof(void*),1U));});});{
struct Cyc_List_List*_tmp1F=0;_npop_handler(0U);return _tmp1F;}}}else{goto _LL6;}}else{goto _LL6;}}else{_LL6: _LL7:
 goto _LL3;}_LL3:;}
# 186
{struct Cyc_List_List*_tmp20=width;int _tmp21;if(_tmp20 != 0){if(((struct Cyc_List_List*)_tmp20)->tl == 0){_LL9: _tmp21=(int)_tmp20->hd;if(_tmp21 == (int)'*'){_LLA: {int x=_tmp21;
typs=({struct Cyc_List_List*_tmp22=_cycalloc(sizeof(*_tmp22));_tmp22->hd=Cyc_Absyn_sint_type,_tmp22->tl=typs;_tmp22;});goto _LL8;}}else{goto _LLB;}}else{goto _LLB;}}else{_LLB: _LLC:
 goto _LL8;}_LL8:;}
# 190
{struct Cyc_List_List*_tmp23=precision;int _tmp25;int _tmp24;if(_tmp23 != 0){if(((struct Cyc_List_List*)_tmp23)->tl != 0){if(((struct Cyc_List_List*)((struct Cyc_List_List*)_tmp23)->tl)->tl == 0){_LLE: _tmp24=(int)_tmp23->hd;_tmp25=(int)(_tmp23->tl)->hd;if(
_tmp24 == (int)'.' && _tmp25 == (int)'*'){_LLF: {int x=_tmp24;int y=_tmp25;
typs=({struct Cyc_List_List*_tmp26=_cycalloc(sizeof(*_tmp26));_tmp26->hd=Cyc_Absyn_sint_type,_tmp26->tl=typs;_tmp26;});goto _LLD;}}else{goto _LL10;}}else{goto _LL10;}}else{goto _LL10;}}else{_LL10: _LL11:
 goto _LLD;}_LLD:;}{
# 195
void*t;
char _tmp27=c;switch(_tmp27){case 100U: _LL13: _LL14:
 goto _LL16;case 105U: _LL15: _LL16:
# 199
{struct Cyc_List_List*f=flags;for(0;f != 0;f=f->tl){
if((int)f->hd == (int)'#'){
({struct Cyc_Int_pa_PrintArg_struct _tmp2A=({struct Cyc_Int_pa_PrintArg_struct _tmpE7;_tmpE7.tag=1U,_tmpE7.f1=(unsigned long)((int)c);_tmpE7;});void*_tmp28[1U];_tmp28[0]=& _tmp2A;({unsigned _tmp10B=loc;struct _fat_ptr _tmp10A=({const char*_tmp29="flag '#' is not valid with %%%c";_tag_fat(_tmp29,sizeof(char),32U);});Cyc_Tcutil_terr(_tmp10B,_tmp10A,_tag_fat(_tmp28,sizeof(void*),1U));});});{
struct Cyc_List_List*_tmp2B=0;_npop_handler(0U);return _tmp2B;}}}}
# 199
{struct Cyc_List_List*_tmp2C=lenmod;int _tmp2E;int _tmp2D;int _tmp2F;int _tmp30;if(_tmp2C == 0){_LL3C: _LL3D:
# 205
 t=Cyc_Absyn_sint_type;goto _LL3B;}else{if(((struct Cyc_List_List*)_tmp2C)->tl == 0){_LL3E: _tmp30=(int)_tmp2C->hd;if(_tmp30 == (int)'l'){_LL3F: {int x=_tmp30;
t=Cyc_Absyn_slong_type;goto _LL3B;}}else{_LL40: _tmp2F=(int)_tmp2C->hd;if(_tmp2F == (int)'h'){_LL41: {int x=_tmp2F;
t=Cyc_Absyn_sshort_type;goto _LL3B;}}else{goto _LL44;}}}else{if(((struct Cyc_List_List*)((struct Cyc_List_List*)_tmp2C)->tl)->tl == 0){_LL42: _tmp2D=(int)_tmp2C->hd;_tmp2E=(int)(_tmp2C->tl)->hd;if(
_tmp2D == (int)'h' && _tmp2E == (int)'h'){_LL43: {int x=_tmp2D;int y=_tmp2E;
t=Cyc_Absyn_schar_type;goto _LL3B;}}else{goto _LL44;}}else{_LL44: _LL45:
# 211
({struct Cyc_String_pa_PrintArg_struct _tmp33=({struct Cyc_String_pa_PrintArg_struct _tmpE9;_tmpE9.tag=0U,({
struct _fat_ptr _tmp10C=(struct _fat_ptr)((struct _fat_ptr)Cyc_implode(lenmod));_tmpE9.f1=_tmp10C;});_tmpE9;});struct Cyc_Int_pa_PrintArg_struct _tmp34=({struct Cyc_Int_pa_PrintArg_struct _tmpE8;_tmpE8.tag=1U,_tmpE8.f1=(unsigned long)((int)c);_tmpE8;});void*_tmp31[2U];_tmp31[0]=& _tmp33,_tmp31[1]=& _tmp34;({unsigned _tmp10E=loc;struct _fat_ptr _tmp10D=({const char*_tmp32="length modifier '%s' is not allowed with %%%c";_tag_fat(_tmp32,sizeof(char),46U);});Cyc_Tcutil_terr(_tmp10E,_tmp10D,_tag_fat(_tmp31,sizeof(void*),2U));});});{
struct Cyc_List_List*_tmp35=0;_npop_handler(0U);return _tmp35;}}}}_LL3B:;}
# 215
typs=({struct Cyc_List_List*_tmp36=_cycalloc(sizeof(*_tmp36));_tmp36->hd=t,_tmp36->tl=typs;_tmp36;});
goto _LL12;case 117U: _LL17: _LL18:
# 218
{struct Cyc_List_List*f=flags;for(0;f != 0;f=f->tl){
if((int)f->hd == (int)'#'){
({void*_tmp37=0U;({unsigned _tmp110=loc;struct _fat_ptr _tmp10F=({const char*_tmp38="Flag '#' not valid with %%u";_tag_fat(_tmp38,sizeof(char),28U);});Cyc_Tcutil_terr(_tmp110,_tmp10F,_tag_fat(_tmp37,sizeof(void*),0U));});});{
struct Cyc_List_List*_tmp39=0;_npop_handler(0U);return _tmp39;}}}}
# 218
goto _LL1A;case 111U: _LL19: _LL1A:
# 224
 goto _LL1C;case 120U: _LL1B: _LL1C:
 goto _LL1E;case 88U: _LL1D: _LL1E:
# 227
{struct Cyc_List_List*_tmp3A=lenmod;int _tmp3C;int _tmp3B;int _tmp3D;int _tmp3E;if(_tmp3A == 0){_LL47: _LL48:
 t=Cyc_Absyn_uint_type;goto _LL46;}else{if(((struct Cyc_List_List*)_tmp3A)->tl == 0){_LL49: _tmp3E=(int)_tmp3A->hd;if(_tmp3E == (int)'l'){_LL4A: {int x=_tmp3E;
t=Cyc_Absyn_ulong_type;goto _LL46;}}else{_LL4B: _tmp3D=(int)_tmp3A->hd;if(_tmp3D == (int)'h'){_LL4C: {int x=_tmp3D;
t=Cyc_Absyn_ushort_type;goto _LL46;}}else{goto _LL4F;}}}else{if(((struct Cyc_List_List*)((struct Cyc_List_List*)_tmp3A)->tl)->tl == 0){_LL4D: _tmp3B=(int)_tmp3A->hd;_tmp3C=(int)(_tmp3A->tl)->hd;if(
_tmp3B == (int)'h' && _tmp3C == (int)'h'){_LL4E: {int x=_tmp3B;int y=_tmp3C;
t=Cyc_Absyn_uchar_type;goto _LL46;}}else{goto _LL4F;}}else{_LL4F: _LL50:
# 235
({struct Cyc_String_pa_PrintArg_struct _tmp41=({struct Cyc_String_pa_PrintArg_struct _tmpEB;_tmpEB.tag=0U,({
struct _fat_ptr _tmp111=(struct _fat_ptr)((struct _fat_ptr)Cyc_implode(lenmod));_tmpEB.f1=_tmp111;});_tmpEB;});struct Cyc_Int_pa_PrintArg_struct _tmp42=({struct Cyc_Int_pa_PrintArg_struct _tmpEA;_tmpEA.tag=1U,_tmpEA.f1=(unsigned long)((int)c);_tmpEA;});void*_tmp3F[2U];_tmp3F[0]=& _tmp41,_tmp3F[1]=& _tmp42;({unsigned _tmp113=loc;struct _fat_ptr _tmp112=({const char*_tmp40="length modifier '%s' is not allowed with %%%c";_tag_fat(_tmp40,sizeof(char),46U);});Cyc_Tcutil_terr(_tmp113,_tmp112,_tag_fat(_tmp3F,sizeof(void*),2U));});});{
struct Cyc_List_List*_tmp43=0;_npop_handler(0U);return _tmp43;}}}}_LL46:;}
# 239
typs=({struct Cyc_List_List*_tmp44=_cycalloc(sizeof(*_tmp44));_tmp44->hd=t,_tmp44->tl=typs;_tmp44;});
goto _LL12;case 102U: _LL1F: _LL20:
 goto _LL22;case 70U: _LL21: _LL22:
 goto _LL24;case 101U: _LL23: _LL24:
 goto _LL26;case 69U: _LL25: _LL26:
 goto _LL28;case 103U: _LL27: _LL28:
 goto _LL2A;case 71U: _LL29: _LL2A:
 goto _LL2C;case 97U: _LL2B: _LL2C:
 goto _LL2E;case 65U: _LL2D: _LL2E:
# 255
{struct Cyc_List_List*_tmp45=lenmod;int _tmp46;if(_tmp45 == 0){_LL52: _LL53:
# 257
 typs=({struct Cyc_List_List*_tmp47=_cycalloc(sizeof(*_tmp47));_tmp47->hd=Cyc_Absyn_double_type,_tmp47->tl=typs;_tmp47;});
goto _LL51;}else{if(((struct Cyc_List_List*)_tmp45)->tl == 0){_LL54: _tmp46=(int)_tmp45->hd;if(_tmp46 == (int)'l'){_LL55: {int x=_tmp46;
# 260
typs=({struct Cyc_List_List*_tmp48=_cycalloc(sizeof(*_tmp48));_tmp48->hd=Cyc_Absyn_long_double_type,_tmp48->tl=typs;_tmp48;});goto _LL51;}}else{goto _LL56;}}else{_LL56: _LL57:
# 262
({struct Cyc_String_pa_PrintArg_struct _tmp4B=({struct Cyc_String_pa_PrintArg_struct _tmpED;_tmpED.tag=0U,({
struct _fat_ptr _tmp114=(struct _fat_ptr)((struct _fat_ptr)Cyc_implode(lenmod));_tmpED.f1=_tmp114;});_tmpED;});struct Cyc_Int_pa_PrintArg_struct _tmp4C=({struct Cyc_Int_pa_PrintArg_struct _tmpEC;_tmpEC.tag=1U,_tmpEC.f1=(unsigned long)((int)c);_tmpEC;});void*_tmp49[2U];_tmp49[0]=& _tmp4B,_tmp49[1]=& _tmp4C;({unsigned _tmp116=loc;struct _fat_ptr _tmp115=({const char*_tmp4A="length modifier '%s' is not allowed with %%%c";_tag_fat(_tmp4A,sizeof(char),46U);});Cyc_Tcutil_terr(_tmp116,_tmp115,_tag_fat(_tmp49,sizeof(void*),2U));});});{
struct Cyc_List_List*_tmp4D=0;_npop_handler(0U);return _tmp4D;}}}_LL51:;}
# 266
goto _LL12;case 99U: _LL2F: _LL30:
# 268
{struct Cyc_List_List*f=flags;for(0;f != 0;f=f->tl){
if((int)f->hd == (int)'#' ||(int)f->hd == (int)'0'){
({struct Cyc_Int_pa_PrintArg_struct _tmp50=({struct Cyc_Int_pa_PrintArg_struct _tmpEE;_tmpEE.tag=1U,_tmpEE.f1=(unsigned long)((int)f->hd);_tmpEE;});void*_tmp4E[1U];_tmp4E[0]=& _tmp50;({unsigned _tmp118=loc;struct _fat_ptr _tmp117=({const char*_tmp4F="flag '%c' not allowed with %%c";_tag_fat(_tmp4F,sizeof(char),31U);});Cyc_Tcutil_terr(_tmp118,_tmp117,_tag_fat(_tmp4E,sizeof(void*),1U));});});{
struct Cyc_List_List*_tmp51=0;_npop_handler(0U);return _tmp51;}}}}
# 268
if(lenmod != 0){
# 276
({struct Cyc_String_pa_PrintArg_struct _tmp54=({struct Cyc_String_pa_PrintArg_struct _tmpEF;_tmpEF.tag=0U,({
struct _fat_ptr _tmp119=(struct _fat_ptr)((struct _fat_ptr)Cyc_implode(lenmod));_tmpEF.f1=_tmp119;});_tmpEF;});void*_tmp52[1U];_tmp52[0]=& _tmp54;({unsigned _tmp11B=loc;struct _fat_ptr _tmp11A=({const char*_tmp53="length modifier '%s' not allowed with %%c";_tag_fat(_tmp53,sizeof(char),42U);});Cyc_Tcutil_terr(_tmp11B,_tmp11A,_tag_fat(_tmp52,sizeof(void*),1U));});});{
struct Cyc_List_List*_tmp55=0;_npop_handler(0U);return _tmp55;}}
# 268
if(precision != 0){
# 281
({struct Cyc_String_pa_PrintArg_struct _tmp58=({struct Cyc_String_pa_PrintArg_struct _tmpF0;_tmpF0.tag=0U,({
struct _fat_ptr _tmp11C=(struct _fat_ptr)((struct _fat_ptr)Cyc_implode(precision));_tmpF0.f1=_tmp11C;});_tmpF0;});void*_tmp56[1U];_tmp56[0]=& _tmp58;({unsigned _tmp11E=loc;struct _fat_ptr _tmp11D=({const char*_tmp57="precision '%s' not allowed with %%c";_tag_fat(_tmp57,sizeof(char),36U);});Cyc_Tcutil_terr(_tmp11E,_tmp11D,_tag_fat(_tmp56,sizeof(void*),1U));});});{
struct Cyc_List_List*_tmp59=0;_npop_handler(0U);return _tmp59;}}
# 268
typs=({struct Cyc_List_List*_tmp5A=_cycalloc(sizeof(*_tmp5A));
# 285
_tmp5A->hd=Cyc_Absyn_sint_type,_tmp5A->tl=typs;_tmp5A;});
goto _LL12;case 115U: _LL31: _LL32:
# 289
{struct Cyc_List_List*f=flags;for(0;f != 0;f=f->tl){
if((int)f->hd != (int)'-'){
({void*_tmp5B=0U;({unsigned _tmp120=loc;struct _fat_ptr _tmp11F=({const char*_tmp5C="a flag not allowed with %%s";_tag_fat(_tmp5C,sizeof(char),28U);});Cyc_Tcutil_terr(_tmp120,_tmp11F,_tag_fat(_tmp5B,sizeof(void*),0U));});});{
struct Cyc_List_List*_tmp5D=0;_npop_handler(0U);return _tmp5D;}}}}
# 289
if(lenmod != 0){
# 297
({void*_tmp5E=0U;({unsigned _tmp122=loc;struct _fat_ptr _tmp121=({const char*_tmp5F="length modifiers not allowed with %%s";_tag_fat(_tmp5F,sizeof(char),38U);});Cyc_Tcutil_terr(_tmp122,_tmp121,_tag_fat(_tmp5E,sizeof(void*),0U));});});{
struct Cyc_List_List*_tmp60=0;_npop_handler(0U);return _tmp60;}}{
# 289
void*ptr;
# 303
if(!isCproto)
ptr=({void*_tmp126=Cyc_Absyn_char_type;void*_tmp125=
Cyc_Absyn_new_evar(& Cyc_Tcutil_rko,({struct Cyc_Core_Opt*_tmp61=_cycalloc(sizeof(*_tmp61));({
struct Cyc_List_List*_tmp123=Cyc_Tcenv_lookup_type_vars(te);_tmp61->v=_tmp123;});_tmp61;}));
# 304
struct Cyc_Absyn_Tqual _tmp124=
# 307
Cyc_Absyn_const_tqual(0U);
# 304
Cyc_Absyn_fatptr_type(_tmp126,_tmp125,_tmp124,Cyc_Absyn_false_type);});else{
# 309
ptr=({void*_tmp12A=Cyc_Absyn_char_type;void*_tmp129=
Cyc_Absyn_new_evar(& Cyc_Tcutil_rko,({struct Cyc_Core_Opt*_tmp62=_cycalloc(sizeof(*_tmp62));({
struct Cyc_List_List*_tmp127=Cyc_Tcenv_lookup_type_vars(te);_tmp62->v=_tmp127;});_tmp62;}));
# 309
struct Cyc_Absyn_Tqual _tmp128=
# 312
Cyc_Absyn_const_tqual(0U);
# 309
Cyc_Absyn_at_type(_tmp12A,_tmp129,_tmp128,Cyc_Absyn_true_type);});}
# 313
typs=({struct Cyc_List_List*_tmp63=_cycalloc(sizeof(*_tmp63));_tmp63->hd=ptr,_tmp63->tl=typs;_tmp63;});
goto _LL12;}case 112U: _LL33: _LL34:
# 317
 typs=({struct Cyc_List_List*_tmp64=_cycalloc(sizeof(*_tmp64));_tmp64->hd=Cyc_Absyn_uint_type,_tmp64->tl=typs;_tmp64;});
goto _LL12;case 110U: _LL35: _LL36:
# 320
{struct Cyc_List_List*f=flags;for(0;f != 0;f=f->tl){
if((int)f->hd == (int)'#' ||(int)f->hd == (int)'0'){
({struct Cyc_Int_pa_PrintArg_struct _tmp67=({struct Cyc_Int_pa_PrintArg_struct _tmpF1;_tmpF1.tag=1U,_tmpF1.f1=(unsigned long)((int)f->hd);_tmpF1;});void*_tmp65[1U];_tmp65[0]=& _tmp67;({unsigned _tmp12C=loc;struct _fat_ptr _tmp12B=({const char*_tmp66="flag '%c' not allowed with %%n";_tag_fat(_tmp66,sizeof(char),31U);});Cyc_Tcutil_terr(_tmp12C,_tmp12B,_tag_fat(_tmp65,sizeof(void*),1U));});});{
struct Cyc_List_List*_tmp68=0;_npop_handler(0U);return _tmp68;}}}}
# 320
if(precision != 0){
# 326
({struct Cyc_String_pa_PrintArg_struct _tmp6B=({struct Cyc_String_pa_PrintArg_struct _tmpF2;_tmpF2.tag=0U,({
struct _fat_ptr _tmp12D=(struct _fat_ptr)((struct _fat_ptr)Cyc_implode(precision));_tmpF2.f1=_tmp12D;});_tmpF2;});void*_tmp69[1U];_tmp69[0]=& _tmp6B;({unsigned _tmp12F=loc;struct _fat_ptr _tmp12E=({const char*_tmp6A="precision '%s' not allowed with %%n";_tag_fat(_tmp6A,sizeof(char),36U);});Cyc_Tcutil_terr(_tmp12F,_tmp12E,_tag_fat(_tmp69,sizeof(void*),1U));});});{
struct Cyc_List_List*_tmp6C=0;_npop_handler(0U);return _tmp6C;}}{
# 320
struct Cyc_Core_Opt*tvs=({struct Cyc_Core_Opt*_tmp78=_cycalloc(sizeof(*_tmp78));({
# 330
struct Cyc_List_List*_tmp130=Cyc_Tcenv_lookup_type_vars(te);_tmp78->v=_tmp130;});_tmp78;});
{struct Cyc_List_List*_tmp6D=lenmod;int _tmp6F;int _tmp6E;int _tmp70;int _tmp71;if(_tmp6D == 0){_LL59: _LL5A:
 t=Cyc_Absyn_sint_type;goto _LL58;}else{if(((struct Cyc_List_List*)_tmp6D)->tl == 0){_LL5B: _tmp71=(int)_tmp6D->hd;if(_tmp71 == (int)'l'){_LL5C: {int x=_tmp71;
# 334
t=Cyc_Absyn_ulong_type;goto _LL58;}}else{_LL5D: _tmp70=(int)_tmp6D->hd;if(_tmp70 == (int)'h'){_LL5E: {int x=_tmp70;
t=Cyc_Absyn_sshort_type;goto _LL58;}}else{goto _LL61;}}}else{if(((struct Cyc_List_List*)((struct Cyc_List_List*)_tmp6D)->tl)->tl == 0){_LL5F: _tmp6E=(int)_tmp6D->hd;_tmp6F=(int)(_tmp6D->tl)->hd;if(
_tmp6E == (int)'h' && _tmp6F == (int)'h'){_LL60: {int x=_tmp6E;int y=_tmp6F;
t=Cyc_Absyn_schar_type;goto _LL58;}}else{goto _LL61;}}else{_LL61: _LL62:
# 339
({struct Cyc_String_pa_PrintArg_struct _tmp74=({struct Cyc_String_pa_PrintArg_struct _tmpF4;_tmpF4.tag=0U,({
struct _fat_ptr _tmp131=(struct _fat_ptr)((struct _fat_ptr)Cyc_implode(lenmod));_tmpF4.f1=_tmp131;});_tmpF4;});struct Cyc_Int_pa_PrintArg_struct _tmp75=({struct Cyc_Int_pa_PrintArg_struct _tmpF3;_tmpF3.tag=1U,_tmpF3.f1=(unsigned long)((int)c);_tmpF3;});void*_tmp72[2U];_tmp72[0]=& _tmp74,_tmp72[1]=& _tmp75;({unsigned _tmp133=loc;struct _fat_ptr _tmp132=({const char*_tmp73="length modifier '%s' is not allowed with %%%c";_tag_fat(_tmp73,sizeof(char),46U);});Cyc_Tcutil_terr(_tmp133,_tmp132,_tag_fat(_tmp72,sizeof(void*),2U));});});{
struct Cyc_List_List*_tmp76=0;_npop_handler(0U);return _tmp76;}}}}_LL58:;}
# 343
t=({void*_tmp136=t;void*_tmp135=Cyc_Absyn_new_evar(& Cyc_Tcutil_rko,tvs);struct Cyc_Absyn_Tqual _tmp134=Cyc_Absyn_empty_tqual(0U);Cyc_Absyn_at_type(_tmp136,_tmp135,_tmp134,Cyc_Absyn_false_type);});
typs=({struct Cyc_List_List*_tmp77=_cycalloc(sizeof(*_tmp77));_tmp77->hd=t,_tmp77->tl=typs;_tmp77;});
goto _LL12;}case 37U: _LL37: _LL38:
# 347
 if(flags != 0){
({struct Cyc_String_pa_PrintArg_struct _tmp7B=({struct Cyc_String_pa_PrintArg_struct _tmpF5;_tmpF5.tag=0U,({struct _fat_ptr _tmp137=(struct _fat_ptr)((struct _fat_ptr)Cyc_implode(flags));_tmpF5.f1=_tmp137;});_tmpF5;});void*_tmp79[1U];_tmp79[0]=& _tmp7B;({unsigned _tmp139=loc;struct _fat_ptr _tmp138=({const char*_tmp7A="flags '%s' not allowed with %%%%";_tag_fat(_tmp7A,sizeof(char),33U);});Cyc_Tcutil_terr(_tmp139,_tmp138,_tag_fat(_tmp79,sizeof(void*),1U));});});{
struct Cyc_List_List*_tmp7C=0;_npop_handler(0U);return _tmp7C;}}
# 347
if(width != 0){
# 352
({struct Cyc_String_pa_PrintArg_struct _tmp7F=({struct Cyc_String_pa_PrintArg_struct _tmpF6;_tmpF6.tag=0U,({struct _fat_ptr _tmp13A=(struct _fat_ptr)((struct _fat_ptr)Cyc_implode(width));_tmpF6.f1=_tmp13A;});_tmpF6;});void*_tmp7D[1U];_tmp7D[0]=& _tmp7F;({unsigned _tmp13C=loc;struct _fat_ptr _tmp13B=({const char*_tmp7E="width '%s' not allowed with %%%%";_tag_fat(_tmp7E,sizeof(char),33U);});Cyc_Tcutil_terr(_tmp13C,_tmp13B,_tag_fat(_tmp7D,sizeof(void*),1U));});});{
struct Cyc_List_List*_tmp80=0;_npop_handler(0U);return _tmp80;}}
# 347
if(precision != 0){
# 356
({struct Cyc_String_pa_PrintArg_struct _tmp83=({struct Cyc_String_pa_PrintArg_struct _tmpF7;_tmpF7.tag=0U,({struct _fat_ptr _tmp13D=(struct _fat_ptr)((struct _fat_ptr)Cyc_implode(precision));_tmpF7.f1=_tmp13D;});_tmpF7;});void*_tmp81[1U];_tmp81[0]=& _tmp83;({unsigned _tmp13F=loc;struct _fat_ptr _tmp13E=({const char*_tmp82="precision '%s' not allowed with %%%%";_tag_fat(_tmp82,sizeof(char),37U);});Cyc_Tcutil_terr(_tmp13F,_tmp13E,_tag_fat(_tmp81,sizeof(void*),1U));});});{
struct Cyc_List_List*_tmp84=0;_npop_handler(0U);return _tmp84;}}
# 347
if(lenmod != 0){
# 360
({struct Cyc_String_pa_PrintArg_struct _tmp87=({struct Cyc_String_pa_PrintArg_struct _tmpF8;_tmpF8.tag=0U,({struct _fat_ptr _tmp140=(struct _fat_ptr)((struct _fat_ptr)Cyc_implode(lenmod));_tmpF8.f1=_tmp140;});_tmpF8;});void*_tmp85[1U];_tmp85[0]=& _tmp87;({unsigned _tmp142=loc;struct _fat_ptr _tmp141=({const char*_tmp86="length modifier '%s' not allowed with %%%%";_tag_fat(_tmp86,sizeof(char),43U);});Cyc_Tcutil_terr(_tmp142,_tmp141,_tag_fat(_tmp85,sizeof(void*),1U));});});{
struct Cyc_List_List*_tmp88=0;_npop_handler(0U);return _tmp88;}}
# 347
goto _LL12;default: _LL39: _LL3A: {
# 365
struct Cyc_List_List*_tmp89=0;_npop_handler(0U);return _tmp89;}}_LL12:;}}}}}{
# 368
struct Cyc_List_List*_tmp8A=Cyc_List_imp_rev(typs);_npop_handler(0U);return _tmp8A;}
# 169
;_pop_region(temp);}struct _tuple12{int f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;char f4;int f5;};
# 36 "formatstr.cyc"
struct Cyc_Core_Opt*Cyc_Formatstr_parse_inputformat(struct _RegionHandle*r,struct _fat_ptr s,int i){
# 381 "formatstr.cyc"
unsigned long len=Cyc_strlen((struct _fat_ptr)s);
if(i < 0 ||(unsigned long)i >= len)return 0;{int suppress=0;
# 385
char c=*((const char*)_check_fat_subscript(s,sizeof(char),i));
if((int)c == (int)'*'){
suppress=1;
++ i;
if((unsigned long)i >= len)return 0;}{
# 386
struct Cyc_List_List*width=0;
# 393
for(0;(unsigned long)i < len;++ i){
c=*((const char*)_check_fat_subscript(s,sizeof(char),i));
if(isdigit((int)c))width=({struct Cyc_List_List*_tmp8B=_region_malloc(r,sizeof(*_tmp8B));_tmp8B->hd=(void*)((int)c),_tmp8B->tl=width;_tmp8B;});else{
break;}}
# 398
if((unsigned long)i >= len)return 0;width=
Cyc_List_imp_rev(width);{
# 403
struct Cyc_List_List*lenmod=0;
c=*((const char*)_check_fat_subscript(s,sizeof(char),i));
{char _tmp8C=c;switch(_tmp8C){case 104U: _LL1: _LL2:
# 407
 lenmod=({struct Cyc_List_List*_tmp8D=_region_malloc(r,sizeof(*_tmp8D));_tmp8D->hd=(void*)((int)c),_tmp8D->tl=lenmod;_tmp8D;});
++ i;
if((unsigned long)i >= len)return 0;c=*((const char*)_check_fat_subscript(s,sizeof(char),i));
# 411
if((int)c == (int)'h'){lenmod=({struct Cyc_List_List*_tmp8E=_region_malloc(r,sizeof(*_tmp8E));_tmp8E->hd=(void*)((int)c),_tmp8E->tl=lenmod;_tmp8E;});++ i;}goto _LL0;case 108U: _LL3: _LL4:
# 414
 lenmod=({struct Cyc_List_List*_tmp8F=_region_malloc(r,sizeof(*_tmp8F));_tmp8F->hd=(void*)((int)c),_tmp8F->tl=lenmod;_tmp8F;});
++ i;
if((unsigned long)i >= len)return 0;c=*((const char*)_check_fat_subscript(s,sizeof(char),i));
# 418
if((int)c == (int)'l'){lenmod=({struct Cyc_List_List*_tmp90=_region_malloc(r,sizeof(*_tmp90));_tmp90->hd=(void*)((int)c),_tmp90->tl=lenmod;_tmp90;});++ i;}goto _LL0;case 106U: _LL5: _LL6:
# 420
 goto _LL8;case 122U: _LL7: _LL8:
 goto _LLA;case 116U: _LL9: _LLA:
 goto _LLC;case 76U: _LLB: _LLC:
# 424
 lenmod=({struct Cyc_List_List*_tmp91=_region_malloc(r,sizeof(*_tmp91));_tmp91->hd=(void*)((int)c),_tmp91->tl=lenmod;_tmp91;});
++ i;
goto _LL0;default: _LLD: _LLE:
 goto _LL0;}_LL0:;}
# 429
if((unsigned long)i >= len)return 0;lenmod=
Cyc_List_imp_rev(lenmod);
# 433
c=*((const char*)_check_fat_subscript(s,sizeof(char),i));
{char _tmp92=c;switch(_tmp92){case 100U: _LL10: _LL11:
 goto _LL13;case 105U: _LL12: _LL13:
 goto _LL15;case 111U: _LL14: _LL15:
 goto _LL17;case 117U: _LL16: _LL17:
 goto _LL19;case 120U: _LL18: _LL19:
 goto _LL1B;case 88U: _LL1A: _LL1B:
 goto _LL1D;case 102U: _LL1C: _LL1D:
 goto _LL1F;case 70U: _LL1E: _LL1F:
 goto _LL21;case 101U: _LL20: _LL21:
 goto _LL23;case 69U: _LL22: _LL23:
 goto _LL25;case 103U: _LL24: _LL25:
 goto _LL27;case 71U: _LL26: _LL27:
 goto _LL29;case 97U: _LL28: _LL29:
 goto _LL2B;case 65U: _LL2A: _LL2B:
 goto _LL2D;case 99U: _LL2C: _LL2D:
 goto _LL2F;case 115U: _LL2E: _LL2F:
 goto _LL31;case 112U: _LL30: _LL31:
 goto _LL33;case 110U: _LL32: _LL33:
 goto _LL35;case 37U: _LL34: _LL35:
 goto _LLF;default: _LL36: _LL37:
 return 0;}_LLF:;}
# 456
return({struct Cyc_Core_Opt*_tmp94=_region_malloc(r,sizeof(*_tmp94));({struct _tuple12*_tmp143=({struct _tuple12*_tmp93=_region_malloc(r,sizeof(*_tmp93));_tmp93->f1=suppress,_tmp93->f2=width,_tmp93->f3=lenmod,_tmp93->f4=c,_tmp93->f5=i + 1;_tmp93;});_tmp94->v=_tmp143;});_tmp94;});}}}}
# 36 "formatstr.cyc"
struct Cyc_List_List*Cyc_Formatstr_get_scanf_types(struct Cyc_Tcenv_Tenv*te,struct _fat_ptr s,int isCproto,unsigned loc){
# 461 "formatstr.cyc"
unsigned long len=Cyc_strlen((struct _fat_ptr)s);
struct Cyc_List_List*typs=0;
int i;
{struct _RegionHandle _tmp95=_new_region("temp");struct _RegionHandle*temp=& _tmp95;_push_region(temp);
for(i=0;(unsigned long)i < len;++ i){
if((int)*((const char*)_check_fat_subscript(s,sizeof(char),i))!= (int)'%')continue;{struct Cyc_Core_Opt*x=
Cyc_Formatstr_parse_inputformat(temp,s,i + 1);
if(x == 0){
({void*_tmp96=0U;({unsigned _tmp145=loc;struct _fat_ptr _tmp144=({const char*_tmp97="bad format string";_tag_fat(_tmp97,sizeof(char),18U);});Cyc_Tcutil_terr(_tmp145,_tmp144,_tag_fat(_tmp96,sizeof(void*),0U));});});{
struct Cyc_List_List*_tmp98=0;_npop_handler(0U);return _tmp98;}}{
# 468
struct _tuple12*_stmttmp1=(struct _tuple12*)x->v;int _tmp9D;char _tmp9C;struct Cyc_List_List*_tmp9B;struct Cyc_List_List*_tmp9A;int _tmp99;_LL1: _tmp99=_stmttmp1->f1;_tmp9A=_stmttmp1->f2;_tmp9B=_stmttmp1->f3;_tmp9C=_stmttmp1->f4;_tmp9D=_stmttmp1->f5;_LL2: {int suppress=_tmp99;struct Cyc_List_List*width=_tmp9A;struct Cyc_List_List*lenmod=_tmp9B;char c=_tmp9C;int j=_tmp9D;
# 473
i=j - 1;
{struct Cyc_List_List*_tmp9E=lenmod;int _tmp9F;if(_tmp9E != 0){if(((struct Cyc_List_List*)_tmp9E)->tl == 0){_LL4: _tmp9F=(int)_tmp9E->hd;if(
(_tmp9F == (int)'j' || _tmp9F == (int)'z')|| _tmp9F == (int)'t'){_LL5: {int x=_tmp9F;
# 477
({struct Cyc_Int_pa_PrintArg_struct _tmpA2=({struct Cyc_Int_pa_PrintArg_struct _tmpF9;_tmpF9.tag=1U,_tmpF9.f1=(unsigned long)x;_tmpF9;});void*_tmpA0[1U];_tmpA0[0]=& _tmpA2;({unsigned _tmp147=loc;struct _fat_ptr _tmp146=({const char*_tmpA1="length modifier '%c' is not supported";_tag_fat(_tmpA1,sizeof(char),38U);});Cyc_Tcutil_terr(_tmp147,_tmp146,_tag_fat(_tmpA0,sizeof(void*),1U));});});{
struct Cyc_List_List*_tmpA3=0;_npop_handler(0U);return _tmpA3;}}}else{goto _LL6;}}else{goto _LL6;}}else{_LL6: _LL7:
 goto _LL3;}_LL3:;}
# 481
if(suppress)continue;{void*t;
# 483
char _tmpA4=c;switch(_tmpA4){case 100U: _LL9: _LLA:
 goto _LLC;case 105U: _LLB: _LLC: {
# 486
struct Cyc_Core_Opt*tvs=({struct Cyc_Core_Opt*_tmpB0=_cycalloc(sizeof(*_tmpB0));({struct Cyc_List_List*_tmp148=Cyc_Tcenv_lookup_type_vars(te);_tmpB0->v=_tmp148;});_tmpB0;});
{struct Cyc_List_List*_tmpA5=lenmod;int _tmpA7;int _tmpA6;int _tmpA8;int _tmpA9;if(_tmpA5 == 0){_LL34: _LL35:
 t=Cyc_Absyn_sint_type;goto _LL33;}else{if(((struct Cyc_List_List*)_tmpA5)->tl == 0){_LL36: _tmpA9=(int)_tmpA5->hd;if(_tmpA9 == (int)'l'){_LL37: {int x=_tmpA9;
t=Cyc_Absyn_slong_type;goto _LL33;}}else{_LL38: _tmpA8=(int)_tmpA5->hd;if(_tmpA8 == (int)'h'){_LL39: {int x=_tmpA8;
t=Cyc_Absyn_sshort_type;goto _LL33;}}else{goto _LL3C;}}}else{if(((struct Cyc_List_List*)((struct Cyc_List_List*)_tmpA5)->tl)->tl == 0){_LL3A: _tmpA6=(int)_tmpA5->hd;_tmpA7=(int)(_tmpA5->tl)->hd;if(
_tmpA6 == (int)'h' && _tmpA7 == (int)'h'){_LL3B: {int x=_tmpA6;int y=_tmpA7;t=Cyc_Absyn_schar_type;goto _LL33;}}else{goto _LL3C;}}else{_LL3C: _LL3D:
# 493
({struct Cyc_String_pa_PrintArg_struct _tmpAC=({struct Cyc_String_pa_PrintArg_struct _tmpFB;_tmpFB.tag=0U,({
struct _fat_ptr _tmp149=(struct _fat_ptr)((struct _fat_ptr)Cyc_implode(lenmod));_tmpFB.f1=_tmp149;});_tmpFB;});struct Cyc_Int_pa_PrintArg_struct _tmpAD=({struct Cyc_Int_pa_PrintArg_struct _tmpFA;_tmpFA.tag=1U,_tmpFA.f1=(unsigned long)((int)c);_tmpFA;});void*_tmpAA[2U];_tmpAA[0]=& _tmpAC,_tmpAA[1]=& _tmpAD;({unsigned _tmp14B=loc;struct _fat_ptr _tmp14A=({const char*_tmpAB="length modifier '%s' is not allowed with %%%c";_tag_fat(_tmpAB,sizeof(char),46U);});Cyc_Tcutil_terr(_tmp14B,_tmp14A,_tag_fat(_tmpAA,sizeof(void*),2U));});});{
struct Cyc_List_List*_tmpAE=0;_npop_handler(0U);return _tmpAE;}}}}_LL33:;}
# 497
t=({void*_tmp14E=t;void*_tmp14D=Cyc_Absyn_new_evar(& Cyc_Tcutil_rko,tvs);struct Cyc_Absyn_Tqual _tmp14C=Cyc_Absyn_empty_tqual(0U);Cyc_Absyn_at_type(_tmp14E,_tmp14D,_tmp14C,Cyc_Absyn_false_type);});
typs=({struct Cyc_List_List*_tmpAF=_cycalloc(sizeof(*_tmpAF));_tmpAF->hd=t,_tmpAF->tl=typs;_tmpAF;});
goto _LL8;}case 117U: _LLD: _LLE:
 goto _LL10;case 111U: _LLF: _LL10:
 goto _LL12;case 120U: _LL11: _LL12:
 goto _LL14;case 88U: _LL13: _LL14: {
# 504
struct Cyc_Core_Opt*tvs=({struct Cyc_Core_Opt*_tmpBC=_cycalloc(sizeof(*_tmpBC));({struct Cyc_List_List*_tmp14F=Cyc_Tcenv_lookup_type_vars(te);_tmpBC->v=_tmp14F;});_tmpBC;});
{struct Cyc_List_List*_tmpB1=lenmod;int _tmpB3;int _tmpB2;int _tmpB4;int _tmpB5;if(_tmpB1 == 0){_LL3F: _LL40:
 t=Cyc_Absyn_uint_type;goto _LL3E;}else{if(((struct Cyc_List_List*)_tmpB1)->tl == 0){_LL41: _tmpB5=(int)_tmpB1->hd;if(_tmpB5 == (int)'l'){_LL42: {int x=_tmpB5;
t=Cyc_Absyn_ulong_type;goto _LL3E;}}else{_LL43: _tmpB4=(int)_tmpB1->hd;if(_tmpB4 == (int)'h'){_LL44: {int x=_tmpB4;
t=Cyc_Absyn_ushort_type;goto _LL3E;}}else{goto _LL47;}}}else{if(((struct Cyc_List_List*)((struct Cyc_List_List*)_tmpB1)->tl)->tl == 0){_LL45: _tmpB2=(int)_tmpB1->hd;_tmpB3=(int)(_tmpB1->tl)->hd;if(
_tmpB2 == (int)'h' && _tmpB3 == (int)'h'){_LL46: {int x=_tmpB2;int y=_tmpB3;t=Cyc_Absyn_uchar_type;goto _LL3E;}}else{goto _LL47;}}else{_LL47: _LL48:
# 511
({struct Cyc_String_pa_PrintArg_struct _tmpB8=({struct Cyc_String_pa_PrintArg_struct _tmpFD;_tmpFD.tag=0U,({
struct _fat_ptr _tmp150=(struct _fat_ptr)((struct _fat_ptr)Cyc_implode(lenmod));_tmpFD.f1=_tmp150;});_tmpFD;});struct Cyc_Int_pa_PrintArg_struct _tmpB9=({struct Cyc_Int_pa_PrintArg_struct _tmpFC;_tmpFC.tag=1U,_tmpFC.f1=(unsigned long)((int)c);_tmpFC;});void*_tmpB6[2U];_tmpB6[0]=& _tmpB8,_tmpB6[1]=& _tmpB9;({unsigned _tmp152=loc;struct _fat_ptr _tmp151=({const char*_tmpB7="length modifier '%s' is not allowed with %%%c";_tag_fat(_tmpB7,sizeof(char),46U);});Cyc_Tcutil_terr(_tmp152,_tmp151,_tag_fat(_tmpB6,sizeof(void*),2U));});});{
struct Cyc_List_List*_tmpBA=0;_npop_handler(0U);return _tmpBA;}}}}_LL3E:;}
# 515
t=({void*_tmp155=t;void*_tmp154=Cyc_Absyn_new_evar(& Cyc_Tcutil_rko,tvs);struct Cyc_Absyn_Tqual _tmp153=Cyc_Absyn_empty_tqual(0U);Cyc_Absyn_at_type(_tmp155,_tmp154,_tmp153,Cyc_Absyn_false_type);});
typs=({struct Cyc_List_List*_tmpBB=_cycalloc(sizeof(*_tmpBB));_tmpBB->hd=t,_tmpBB->tl=typs;_tmpBB;});
goto _LL8;}case 102U: _LL15: _LL16:
 goto _LL18;case 70U: _LL17: _LL18:
 goto _LL1A;case 101U: _LL19: _LL1A:
 goto _LL1C;case 69U: _LL1B: _LL1C:
 goto _LL1E;case 103U: _LL1D: _LL1E:
 goto _LL20;case 71U: _LL1F: _LL20:
 goto _LL22;case 97U: _LL21: _LL22:
 goto _LL24;case 65U: _LL23: _LL24: {
# 526
struct Cyc_Core_Opt*tvs=({struct Cyc_Core_Opt*_tmpC5=_cycalloc(sizeof(*_tmpC5));({struct Cyc_List_List*_tmp156=Cyc_Tcenv_lookup_type_vars(te);_tmpC5->v=_tmp156;});_tmpC5;});
{struct Cyc_List_List*_tmpBD=lenmod;int _tmpBE;if(_tmpBD == 0){_LL4A: _LL4B:
 t=Cyc_Absyn_float_type;goto _LL49;}else{if(((struct Cyc_List_List*)_tmpBD)->tl == 0){_LL4C: _tmpBE=(int)_tmpBD->hd;if(_tmpBE == (int)'l'){_LL4D: {int x=_tmpBE;
# 530
t=Cyc_Absyn_double_type;goto _LL49;}}else{goto _LL4E;}}else{_LL4E: _LL4F:
# 532
({struct Cyc_String_pa_PrintArg_struct _tmpC1=({struct Cyc_String_pa_PrintArg_struct _tmpFF;_tmpFF.tag=0U,({
struct _fat_ptr _tmp157=(struct _fat_ptr)((struct _fat_ptr)Cyc_implode(lenmod));_tmpFF.f1=_tmp157;});_tmpFF;});struct Cyc_Int_pa_PrintArg_struct _tmpC2=({struct Cyc_Int_pa_PrintArg_struct _tmpFE;_tmpFE.tag=1U,_tmpFE.f1=(unsigned long)((int)c);_tmpFE;});void*_tmpBF[2U];_tmpBF[0]=& _tmpC1,_tmpBF[1]=& _tmpC2;({unsigned _tmp159=loc;struct _fat_ptr _tmp158=({const char*_tmpC0="length modifier '%s' is not allowed with %%%c";_tag_fat(_tmpC0,sizeof(char),46U);});Cyc_Tcutil_terr(_tmp159,_tmp158,_tag_fat(_tmpBF,sizeof(void*),2U));});});{
struct Cyc_List_List*_tmpC3=0;_npop_handler(0U);return _tmpC3;}}}_LL49:;}
# 536
t=({void*_tmp15C=t;void*_tmp15B=Cyc_Absyn_new_evar(& Cyc_Tcutil_rko,tvs);struct Cyc_Absyn_Tqual _tmp15A=Cyc_Absyn_empty_tqual(0U);Cyc_Absyn_at_type(_tmp15C,_tmp15B,_tmp15A,Cyc_Absyn_false_type);});
typs=({struct Cyc_List_List*_tmpC4=_cycalloc(sizeof(*_tmpC4));_tmpC4->hd=t,_tmpC4->tl=typs;_tmpC4;});
goto _LL8;}case 99U: _LL25: _LL26: {
# 541
struct Cyc_Core_Opt*tvs=({struct Cyc_Core_Opt*_tmpC7=_cycalloc(sizeof(*_tmpC7));({struct Cyc_List_List*_tmp15D=Cyc_Tcenv_lookup_type_vars(te);_tmpC7->v=_tmp15D;});_tmpC7;});
void*ptr;
if(!isCproto)
ptr=({void*_tmp160=Cyc_Absyn_char_type;void*_tmp15F=Cyc_Absyn_new_evar(& Cyc_Tcutil_rko,tvs);struct Cyc_Absyn_Tqual _tmp15E=
Cyc_Absyn_empty_tqual(0U);
# 544
Cyc_Absyn_fatptr_type(_tmp160,_tmp15F,_tmp15E,Cyc_Absyn_false_type);});else{
# 548
ptr=({void*_tmp163=Cyc_Absyn_char_type;void*_tmp162=
Cyc_Absyn_new_evar(& Cyc_Tcutil_rko,tvs);
# 548
struct Cyc_Absyn_Tqual _tmp161=
# 550
Cyc_Absyn_empty_tqual(0U);
# 548
Cyc_Absyn_at_type(_tmp163,_tmp162,_tmp161,Cyc_Absyn_false_type);});}
# 551
typs=({struct Cyc_List_List*_tmpC6=_cycalloc(sizeof(*_tmpC6));_tmpC6->hd=ptr,_tmpC6->tl=typs;_tmpC6;});
goto _LL8;}case 115U: _LL27: _LL28: {
# 554
struct Cyc_Core_Opt*tvs=({struct Cyc_Core_Opt*_tmpC9=_cycalloc(sizeof(*_tmpC9));({struct Cyc_List_List*_tmp164=Cyc_Tcenv_lookup_type_vars(te);_tmpC9->v=_tmp164;});_tmpC9;});
# 556
void*ptr;
if(!isCproto)
ptr=({void*_tmp167=Cyc_Absyn_char_type;void*_tmp166=Cyc_Absyn_new_evar(& Cyc_Tcutil_rko,tvs);struct Cyc_Absyn_Tqual _tmp165=
Cyc_Absyn_empty_tqual(0U);
# 558
Cyc_Absyn_fatptr_type(_tmp167,_tmp166,_tmp165,Cyc_Absyn_false_type);});else{
# 561
ptr=({void*_tmp16A=Cyc_Absyn_char_type;void*_tmp169=Cyc_Absyn_new_evar(& Cyc_Tcutil_rko,tvs);struct Cyc_Absyn_Tqual _tmp168=
Cyc_Absyn_empty_tqual(0U);
# 561
Cyc_Absyn_at_type(_tmp16A,_tmp169,_tmp168,Cyc_Absyn_true_type);});}
# 563
typs=({struct Cyc_List_List*_tmpC8=_cycalloc(sizeof(*_tmpC8));_tmpC8->hd=ptr,_tmpC8->tl=typs;_tmpC8;});
goto _LL8;}case 91U: _LL29: _LL2A:
 goto _LL2C;case 112U: _LL2B: _LL2C:
# 567
({struct Cyc_Int_pa_PrintArg_struct _tmpCC=({struct Cyc_Int_pa_PrintArg_struct _tmp100;_tmp100.tag=1U,_tmp100.f1=(unsigned long)((int)c);_tmp100;});void*_tmpCA[1U];_tmpCA[0]=& _tmpCC;({unsigned _tmp16C=loc;struct _fat_ptr _tmp16B=({const char*_tmpCB="%%%c is not supported";_tag_fat(_tmpCB,sizeof(char),22U);});Cyc_Tcutil_terr(_tmp16C,_tmp16B,_tag_fat(_tmpCA,sizeof(void*),1U));});});{
struct Cyc_List_List*_tmpCD=0;_npop_handler(0U);return _tmpCD;}case 110U: _LL2D: _LL2E: {
# 570
struct Cyc_Core_Opt*tvs=({struct Cyc_Core_Opt*_tmpD9=_cycalloc(sizeof(*_tmpD9));({struct Cyc_List_List*_tmp16D=Cyc_Tcenv_lookup_type_vars(te);_tmpD9->v=_tmp16D;});_tmpD9;});
{struct Cyc_List_List*_tmpCE=lenmod;int _tmpD0;int _tmpCF;int _tmpD1;int _tmpD2;if(_tmpCE == 0){_LL51: _LL52:
 t=Cyc_Absyn_sint_type;goto _LL50;}else{if(((struct Cyc_List_List*)_tmpCE)->tl == 0){_LL53: _tmpD2=(int)_tmpCE->hd;if(_tmpD2 == (int)'l'){_LL54: {int x=_tmpD2;
t=Cyc_Absyn_ulong_type;goto _LL50;}}else{_LL55: _tmpD1=(int)_tmpCE->hd;if(_tmpD1 == (int)'h'){_LL56: {int x=_tmpD1;
t=Cyc_Absyn_sshort_type;goto _LL50;}}else{goto _LL59;}}}else{if(((struct Cyc_List_List*)((struct Cyc_List_List*)_tmpCE)->tl)->tl == 0){_LL57: _tmpCF=(int)_tmpCE->hd;_tmpD0=(int)(_tmpCE->tl)->hd;if(
_tmpCF == (int)'h' && _tmpD0 == (int)'h'){_LL58: {int x=_tmpCF;int y=_tmpD0;t=Cyc_Absyn_schar_type;goto _LL50;}}else{goto _LL59;}}else{_LL59: _LL5A:
# 577
({struct Cyc_String_pa_PrintArg_struct _tmpD5=({struct Cyc_String_pa_PrintArg_struct _tmp102;_tmp102.tag=0U,({
struct _fat_ptr _tmp16E=(struct _fat_ptr)((struct _fat_ptr)Cyc_implode(lenmod));_tmp102.f1=_tmp16E;});_tmp102;});struct Cyc_Int_pa_PrintArg_struct _tmpD6=({struct Cyc_Int_pa_PrintArg_struct _tmp101;_tmp101.tag=1U,_tmp101.f1=(unsigned long)((int)c);_tmp101;});void*_tmpD3[2U];_tmpD3[0]=& _tmpD5,_tmpD3[1]=& _tmpD6;({unsigned _tmp170=loc;struct _fat_ptr _tmp16F=({const char*_tmpD4="length modifier '%s' is not allowed with %%%c";_tag_fat(_tmpD4,sizeof(char),46U);});Cyc_Tcutil_terr(_tmp170,_tmp16F,_tag_fat(_tmpD3,sizeof(void*),2U));});});{
struct Cyc_List_List*_tmpD7=0;_npop_handler(0U);return _tmpD7;}}}}_LL50:;}
# 581
t=({void*_tmp173=t;void*_tmp172=Cyc_Absyn_new_evar(& Cyc_Tcutil_rko,tvs);struct Cyc_Absyn_Tqual _tmp171=Cyc_Absyn_empty_tqual(0U);Cyc_Absyn_at_type(_tmp173,_tmp172,_tmp171,Cyc_Absyn_false_type);});
typs=({struct Cyc_List_List*_tmpD8=_cycalloc(sizeof(*_tmpD8));_tmpD8->hd=t,_tmpD8->tl=typs;_tmpD8;});
goto _LL8;}case 37U: _LL2F: _LL30:
# 585
 if(suppress){
({void*_tmpDA=0U;({unsigned _tmp175=loc;struct _fat_ptr _tmp174=({const char*_tmpDB="Assignment suppression (*) is not allowed with %%%%";_tag_fat(_tmpDB,sizeof(char),52U);});Cyc_Tcutil_terr(_tmp175,_tmp174,_tag_fat(_tmpDA,sizeof(void*),0U));});});{
struct Cyc_List_List*_tmpDC=0;_npop_handler(0U);return _tmpDC;}}
# 585
if(width != 0){
# 590
({struct Cyc_String_pa_PrintArg_struct _tmpDF=({struct Cyc_String_pa_PrintArg_struct _tmp103;_tmp103.tag=0U,({struct _fat_ptr _tmp176=(struct _fat_ptr)((struct _fat_ptr)Cyc_implode(width));_tmp103.f1=_tmp176;});_tmp103;});void*_tmpDD[1U];_tmpDD[0]=& _tmpDF;({unsigned _tmp178=loc;struct _fat_ptr _tmp177=({const char*_tmpDE="width '%s' not allowed with %%%%";_tag_fat(_tmpDE,sizeof(char),33U);});Cyc_Tcutil_terr(_tmp178,_tmp177,_tag_fat(_tmpDD,sizeof(void*),1U));});});{
struct Cyc_List_List*_tmpE0=0;_npop_handler(0U);return _tmpE0;}}
# 585
if(lenmod != 0){
# 594
({struct Cyc_String_pa_PrintArg_struct _tmpE3=({struct Cyc_String_pa_PrintArg_struct _tmp104;_tmp104.tag=0U,({struct _fat_ptr _tmp179=(struct _fat_ptr)((struct _fat_ptr)Cyc_implode(lenmod));_tmp104.f1=_tmp179;});_tmp104;});void*_tmpE1[1U];_tmpE1[0]=& _tmpE3;({unsigned _tmp17B=loc;struct _fat_ptr _tmp17A=({const char*_tmpE2="length modifier '%s' not allowed with %%%%";_tag_fat(_tmpE2,sizeof(char),43U);});Cyc_Tcutil_terr(_tmp17B,_tmp17A,_tag_fat(_tmpE1,sizeof(void*),1U));});});{
struct Cyc_List_List*_tmpE4=0;_npop_handler(0U);return _tmpE4;}}
# 585
goto _LL8;default: _LL31: _LL32: {
# 599
struct Cyc_List_List*_tmpE5=0;_npop_handler(0U);return _tmpE5;}}_LL8:;}}}}}
# 465
;_pop_region(temp);}
# 603
return Cyc_List_imp_rev(typs);}
