#include <setjmp.h>
/* This is a C header file to be used by the output of the Cyclone to
   C translator.  The corresponding definitions are in file lib/runtime_*.c */
#ifndef _CYC_INCLUDE_H_
#define _CYC_INCLUDE_H_

/* Need one of these per thread (see runtime_stack.c). The runtime maintains 
   a stack that contains either _handler_cons structs or _RegionHandle structs.
   The tag is 0 for a handler_cons and 1 for a region handle.  */
struct _RuntimeStack {
  int tag; 
  struct _RuntimeStack *next;
  void (*cleanup)(struct _RuntimeStack *frame);
};

#ifndef offsetof
/* should be size_t, but int is fine. */
#define offsetof(t,n) ((int)(&(((t *)0)->n)))
#endif

/* Fat pointers */
struct _fat_ptr {
  unsigned char *curr; 
  unsigned char *base; 
  unsigned char *last_plus_one; 
};  

/* Discriminated Unions */
struct _xtunion_struct { char *tag; };

/* Regions */
struct _RegionPage
#ifdef CYC_REGION_PROFILE
{ unsigned total_bytes;
  unsigned free_bytes;
  /* MWH: wish we didn't have to include the stuff below ... */
  struct _RegionPage *next;
  char data[1];
}
#endif
; // abstract -- defined in runtime_memory.c
struct _RegionHandle {
  struct _RuntimeStack s;
  struct _RegionPage *curr;
  char               *offset;
  char               *last_plus_one;
  struct _DynRegionHandle *sub_regions;
#ifdef CYC_REGION_PROFILE
  const char         *name;
#else
  unsigned used_bytes;
  unsigned wasted_bytes;
#endif
};
struct _DynRegionFrame {
  struct _RuntimeStack s;
  struct _DynRegionHandle *x;
};
// A dynamic region is just a region handle.  The wrapper struct is for type
// abstraction.
struct Cyc_Core_DynamicRegion {
  struct _RegionHandle h;
};

struct _RegionHandle _new_region(const char*);
void* _region_malloc(struct _RegionHandle*, unsigned);
void* _region_calloc(struct _RegionHandle*, unsigned t, unsigned n);
void   _free_region(struct _RegionHandle*);
struct _RegionHandle*_open_dynregion(struct _DynRegionFrame*,struct _DynRegionHandle*);
void   _pop_dynregion();

/* Exceptions */
struct _handler_cons {
  struct _RuntimeStack s;
  jmp_buf handler;
};
void _push_handler(struct _handler_cons *);
void _push_region(struct _RegionHandle *);
void _npop_handler(int);
void _pop_handler();
void _pop_region();

#ifndef _throw
void* _throw_null_fn(const char*,unsigned);
void* _throw_arraybounds_fn(const char*,unsigned);
void* _throw_badalloc_fn(const char*,unsigned);
void* _throw_match_fn(const char*,unsigned);
void* _throw_fn(void*,const char*,unsigned);
void* _rethrow(void*);
#define _throw_null() (_throw_null_fn(__FILE__,__LINE__))
#define _throw_arraybounds() (_throw_arraybounds_fn(__FILE__,__LINE__))
#define _throw_badalloc() (_throw_badalloc_fn(__FILE__,__LINE__))
#define _throw_match() (_throw_match_fn(__FILE__,__LINE__))
#define _throw(e) (_throw_fn((e),__FILE__,__LINE__))
#endif

struct _xtunion_struct* Cyc_Core_get_exn_thrown();
/* Built-in Exceptions */
struct Cyc_Null_Exception_exn_struct { char *tag; };
struct Cyc_Array_bounds_exn_struct { char *tag; };
struct Cyc_Match_Exception_exn_struct { char *tag; };
struct Cyc_Bad_alloc_exn_struct { char *tag; };
extern char Cyc_Null_Exception[];
extern char Cyc_Array_bounds[];
extern char Cyc_Match_Exception[];
extern char Cyc_Bad_alloc[];

/* Built-in Run-time Checks and company */
#ifdef CYC_ANSI_OUTPUT
#define _INLINE  
#else
#define _INLINE inline
#endif

#ifdef NO_CYC_NULL_CHECKS
#define _check_null(ptr) (ptr)
#else
#define _check_null(ptr) \
  ({ void*_cks_null = (void*)(ptr); \
     if (!_cks_null) _throw_null(); \
     _cks_null; })
#endif

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index)\
   (((char*)ptr) + (elt_sz)*(index))
#ifdef NO_CYC_NULL_CHECKS
#define _check_known_subscript_null _check_known_subscript_notnull
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr);\
  int _index = (index);\
  if (!_cks_ptr) _throw_null(); \
  _cks_ptr + (elt_sz)*_index; })
#endif
#define _zero_arr_plus_fn(orig_x,orig_sz,orig_i,f,l) ((orig_x)+(orig_i))
#define _zero_arr_plus_char_fn _zero_arr_plus_fn
#define _zero_arr_plus_short_fn _zero_arr_plus_fn
#define _zero_arr_plus_int_fn _zero_arr_plus_fn
#define _zero_arr_plus_float_fn _zero_arr_plus_fn
#define _zero_arr_plus_double_fn _zero_arr_plus_fn
#define _zero_arr_plus_longdouble_fn _zero_arr_plus_fn
#define _zero_arr_plus_voidstar_fn _zero_arr_plus_fn
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (!_cks_ptr) _throw_null(); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })

/* _zero_arr_plus_*_fn(x,sz,i,filename,lineno) adds i to zero-terminated ptr
   x that has at least sz elements */
char* _zero_arr_plus_char_fn(char*,unsigned,int,const char*,unsigned);
short* _zero_arr_plus_short_fn(short*,unsigned,int,const char*,unsigned);
int* _zero_arr_plus_int_fn(int*,unsigned,int,const char*,unsigned);
float* _zero_arr_plus_float_fn(float*,unsigned,int,const char*,unsigned);
double* _zero_arr_plus_double_fn(double*,unsigned,int,const char*,unsigned);
long double* _zero_arr_plus_longdouble_fn(long double*,unsigned,int,const char*, unsigned);
void** _zero_arr_plus_voidstar_fn(void**,unsigned,int,const char*,unsigned);
#endif

/* _get_zero_arr_size_*(x,sz) returns the number of elements in a
   zero-terminated array that is NULL or has at least sz elements */
int _get_zero_arr_size_char(const char*,unsigned);
int _get_zero_arr_size_short(const short*,unsigned);
int _get_zero_arr_size_int(const int*,unsigned);
int _get_zero_arr_size_float(const float*,unsigned);
int _get_zero_arr_size_double(const double*,unsigned);
int _get_zero_arr_size_longdouble(const long double*,unsigned);
int _get_zero_arr_size_voidstar(const void**,unsigned);

/* _zero_arr_inplace_plus_*_fn(x,i,filename,lineno) sets
   zero-terminated pointer *x to *x + i */
char* _zero_arr_inplace_plus_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_short_fn(short**,int,const char*,unsigned);
int* _zero_arr_inplace_plus_int(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_longdouble_fn(long double**,int,const char*,unsigned);
void** _zero_arr_inplace_plus_voidstar_fn(void***,int,const char*,unsigned);
/* like the previous functions, but does post-addition (as in e++) */
char* _zero_arr_inplace_plus_post_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_post_short_fn(short**x,int,const char*,unsigned);
int* _zero_arr_inplace_plus_post_int_fn(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_post_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_post_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_post_longdouble_fn(long double**,int,const char *,unsigned);
void** _zero_arr_inplace_plus_post_voidstar_fn(void***,int,const char*,unsigned);
#define _zero_arr_plus_char(x,s,i) \
  (_zero_arr_plus_char_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_short(x,s,i) \
  (_zero_arr_plus_short_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_int(x,s,i) \
  (_zero_arr_plus_int_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_float(x,s,i) \
  (_zero_arr_plus_float_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_double(x,s,i) \
  (_zero_arr_plus_double_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_longdouble(x,s,i) \
  (_zero_arr_plus_longdouble_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_voidstar(x,s,i) \
  (_zero_arr_plus_voidstar_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_inplace_plus_char(x,i) \
  _zero_arr_inplace_plus_char_fn((char **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_short(x,i) \
  _zero_arr_inplace_plus_short_fn((short **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_int(x,i) \
  _zero_arr_inplace_plus_int_fn((int **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_float(x,i) \
  _zero_arr_inplace_plus_float_fn((float **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_double(x,i) \
  _zero_arr_inplace_plus_double_fn((double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_longdouble(x,i) \
  _zero_arr_inplace_plus_longdouble_fn((long double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_voidstar(x,i) \
  _zero_arr_inplace_plus_voidstar_fn((void ***)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_char(x,i) \
  _zero_arr_inplace_plus_post_char_fn((char **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_short(x,i) \
  _zero_arr_inplace_plus_post_short_fn((short **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_int(x,i) \
  _zero_arr_inplace_plus_post_int_fn((int **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_float(x,i) \
  _zero_arr_inplace_plus_post_float_fn((float **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_double(x,i) \
  _zero_arr_inplace_plus_post_double_fn((double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_longdouble(x,i) \
  _zero_arr_inplace_plus_post_longdouble_fn((long double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_voidstar(x,i) \
  _zero_arr_inplace_plus_post_voidstar_fn((void***)(x),(i),__FILE__,__LINE__)

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_fat_subscript(arr,elt_sz,index) ((arr).curr + (elt_sz) * (index))
#define _untag_fat_ptr(arr,elt_sz,num_elts) ((arr).curr)
#else
#define _check_fat_subscript(arr,elt_sz,index) ({ \
  struct _fat_ptr _cus_arr = (arr); \
  unsigned char *_cus_ans = _cus_arr.curr + (elt_sz) * (index); \
  /* JGM: not needed! if (!_cus_arr.base) _throw_null();*/ \
  if (_cus_ans < _cus_arr.base || _cus_ans >= _cus_arr.last_plus_one) \
    _throw_arraybounds(); \
  _cus_ans; })
#define _untag_fat_ptr(arr,elt_sz,num_elts) ({ \
  struct _fat_ptr _arr = (arr); \
  unsigned char *_curr = _arr.curr; \
  if ((_curr < _arr.base || _curr + (elt_sz) * (num_elts) > _arr.last_plus_one) &&\
      _curr != (unsigned char *)0) \
    _throw_arraybounds(); \
  _curr; })
#endif

#define _tag_fat(tcurr,elt_sz,num_elts) ({ \
  struct _fat_ptr _ans; \
  unsigned _num_elts = (num_elts);\
  _ans.base = _ans.curr = (void*)(tcurr); \
  /* JGM: if we're tagging NULL, ignore num_elts */ \
  _ans.last_plus_one = _ans.base ? (_ans.base + (elt_sz) * _num_elts) : 0; \
  _ans; })

#define _get_fat_size(arr,elt_sz) \
  ({struct _fat_ptr _arr = (arr); \
    unsigned char *_arr_curr=_arr.curr; \
    unsigned char *_arr_last=_arr.last_plus_one; \
    (_arr_curr < _arr.base || _arr_curr >= _arr_last) ? 0 : \
    ((_arr_last - _arr_curr) / (elt_sz));})

#define _fat_ptr_plus(arr,elt_sz,change) ({ \
  struct _fat_ptr _ans = (arr); \
  int _change = (change);\
  _ans.curr += (elt_sz) * _change;\
  _ans; })
#define _fat_ptr_inplace_plus(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  _arr_ptr->curr += (elt_sz) * (change);\
  *_arr_ptr; })
#define _fat_ptr_inplace_plus_post(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  struct _fat_ptr _ans = *_arr_ptr; \
  _arr_ptr->curr += (elt_sz) * (change);\
  _ans; })

//Not a macro since initialization order matters. Defined in runtime_zeroterm.c.
struct _fat_ptr _fat_ptr_decrease_size(struct _fat_ptr,unsigned sz,unsigned numelts);

/* Allocation */
void* GC_malloc(int);
void* GC_malloc_atomic(int);
void* GC_calloc(unsigned,unsigned);
void* GC_calloc_atomic(unsigned,unsigned);
// bound the allocation size to be < MAX_ALLOC_SIZE. See macros below for usage.
#define MAX_MALLOC_SIZE (1 << 28)
void* _bounded_GC_malloc(int,const char*,int);
void* _bounded_GC_malloc_atomic(int,const char*,int);
void* _bounded_GC_calloc(unsigned,unsigned,const char*,int);
void* _bounded_GC_calloc_atomic(unsigned,unsigned,const char*,int);
/* these macros are overridden below ifdef CYC_REGION_PROFILE */
#ifndef CYC_REGION_PROFILE
#define _cycalloc(n) _bounded_GC_malloc(n,__FILE__,__LINE__)
#define _cycalloc_atomic(n) _bounded_GC_malloc_atomic(n,__FILE__,__LINE__)
#define _cyccalloc(n,s) _bounded_GC_calloc(n,s,__FILE__,__LINE__)
#define _cyccalloc_atomic(n,s) _bounded_GC_calloc_atomic(n,s,__FILE__,__LINE__)
#endif

static _INLINE unsigned int _check_times(unsigned x, unsigned y) {
  unsigned long long whole_ans = 
    ((unsigned long long) x)*((unsigned long long)y);
  unsigned word_ans = (unsigned)whole_ans;
  if(word_ans < whole_ans || word_ans > MAX_MALLOC_SIZE)
    _throw_badalloc();
  return word_ans;
}

#define _CYC_MAX_REGION_CONST 2
#define _CYC_MIN_ALIGNMENT (sizeof(double))

#ifdef CYC_REGION_PROFILE
extern int rgn_total_bytes;
#endif

static _INLINE void *_fast_region_malloc(struct _RegionHandle *r, unsigned orig_s) {  
  if (r > (struct _RegionHandle *)_CYC_MAX_REGION_CONST && r->curr != 0) { 
#ifdef CYC_NOALIGN
    unsigned s =  orig_s;
#else
    unsigned s =  (orig_s + _CYC_MIN_ALIGNMENT - 1) & (~(_CYC_MIN_ALIGNMENT -1)); 
#endif
    char *result; 
    result = r->offset; 
    if (s <= (r->last_plus_one - result)) {
      r->offset = result + s; 
#ifdef CYC_REGION_PROFILE
    r->curr->free_bytes = r->curr->free_bytes - s;
    rgn_total_bytes += s;
#endif
      return result;
    }
  } 
  return _region_malloc(r,orig_s); 
}

#ifdef CYC_REGION_PROFILE
/* see macros below for usage. defined in runtime_memory.c */
void* _profile_GC_malloc(int,const char*,const char*,int);
void* _profile_GC_malloc_atomic(int,const char*,const char*,int);
void* _profile_GC_calloc(unsigned,unsigned,const char*,const char*,int);
void* _profile_GC_calloc_atomic(unsigned,unsigned,const char*,const char*,int);
void* _profile_region_malloc(struct _RegionHandle*,unsigned,const char*,const char*,int);
void* _profile_region_calloc(struct _RegionHandle*,unsigned,unsigned,const char *,const char*,int);
struct _RegionHandle _profile_new_region(const char*,const char*,const char*,int);
void _profile_free_region(struct _RegionHandle*,const char*,const char*,int);
#ifndef RUNTIME_CYC
#define _new_region(n) _profile_new_region(n,__FILE__,__FUNCTION__,__LINE__)
#define _free_region(r) _profile_free_region(r,__FILE__,__FUNCTION__,__LINE__)
#define _region_malloc(rh,n) _profile_region_malloc(rh,n,__FILE__,__FUNCTION__,__LINE__)
#define _region_calloc(rh,n,t) _profile_region_calloc(rh,n,t,__FILE__,__FUNCTION__,__LINE__)
#  endif
#define _cycalloc(n) _profile_GC_malloc(n,__FILE__,__FUNCTION__,__LINE__)
#define _cycalloc_atomic(n) _profile_GC_malloc_atomic(n,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc(n,s) _profile_GC_calloc(n,s,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc_atomic(n,s) _profile_GC_calloc_atomic(n,s,__FILE__,__FUNCTION__,__LINE__)
#endif
#endif
 struct Cyc_Core_Opt{void*v;};
# 119 "core.h"
int Cyc_Core_intcmp(int,int);extern char Cyc_Core_Invalid_argument[17U];struct Cyc_Core_Invalid_argument_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Failure[8U];struct Cyc_Core_Failure_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Impossible[11U];struct Cyc_Core_Impossible_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Not_found[10U];struct Cyc_Core_Not_found_exn_struct{char*tag;};extern char Cyc_Core_Unreachable[12U];struct Cyc_Core_Unreachable_exn_struct{char*tag;struct _fat_ptr f1;};
# 168
extern struct _RegionHandle*Cyc_Core_unique_region;struct Cyc_Core_DynamicRegion;struct Cyc_Core_NewDynamicRegion{struct Cyc_Core_DynamicRegion*key;};struct Cyc_Core_ThinRes{void*arr;unsigned nelts;};struct Cyc_List_List{void*hd;struct Cyc_List_List*tl;};
# 61 "list.h"
int Cyc_List_length(struct Cyc_List_List*x);extern char Cyc_List_List_mismatch[14U];struct Cyc_List_List_mismatch_exn_struct{char*tag;};extern char Cyc_List_Nth[4U];struct Cyc_List_Nth_exn_struct{char*tag;};struct Cyc___cycFILE;struct Cyc_String_pa_PrintArg_struct{int tag;struct _fat_ptr f1;};struct Cyc_Int_pa_PrintArg_struct{int tag;unsigned long f1;};struct Cyc_Double_pa_PrintArg_struct{int tag;double f1;};struct Cyc_LongDouble_pa_PrintArg_struct{int tag;long double f1;};struct Cyc_ShortPtr_pa_PrintArg_struct{int tag;short*f1;};struct Cyc_IntPtr_pa_PrintArg_struct{int tag;unsigned long*f1;};
# 100 "cycboot.h"
int Cyc_fprintf(struct Cyc___cycFILE*,struct _fat_ptr,struct _fat_ptr);struct Cyc_ShortPtr_sa_ScanfArg_struct{int tag;short*f1;};struct Cyc_UShortPtr_sa_ScanfArg_struct{int tag;unsigned short*f1;};struct Cyc_IntPtr_sa_ScanfArg_struct{int tag;int*f1;};struct Cyc_UIntPtr_sa_ScanfArg_struct{int tag;unsigned*f1;};struct Cyc_StringPtr_sa_ScanfArg_struct{int tag;struct _fat_ptr f1;};struct Cyc_DoublePtr_sa_ScanfArg_struct{int tag;double*f1;};struct Cyc_FloatPtr_sa_ScanfArg_struct{int tag;float*f1;};struct Cyc_CharPtr_sa_ScanfArg_struct{int tag;struct _fat_ptr f1;};extern char Cyc_FileCloseError[15U];struct Cyc_FileCloseError_exn_struct{char*tag;};extern char Cyc_FileOpenError[14U];struct Cyc_FileOpenError_exn_struct{char*tag;struct _fat_ptr f1;};struct Cyc_Iter_Iter{void*env;int(*next)(void*env,void*dest);};struct Cyc_Set_Set;
# 51 "set.h"
struct Cyc_Set_Set*Cyc_Set_empty(int(*cmp)(void*,void*));
# 63
struct Cyc_Set_Set*Cyc_Set_insert(struct Cyc_Set_Set*s,void*elt);
# 75
struct Cyc_Set_Set*Cyc_Set_union_two(struct Cyc_Set_Set*s1,struct Cyc_Set_Set*s2);
# 82
struct Cyc_Set_Set*Cyc_Set_diff(struct Cyc_Set_Set*s1,struct Cyc_Set_Set*s2);
# 85
struct Cyc_Set_Set*Cyc_Set_delete(struct Cyc_Set_Set*s,void*elt);
# 97
int Cyc_Set_is_empty(struct Cyc_Set_Set*s);
# 100
int Cyc_Set_member(struct Cyc_Set_Set*s,void*elt);extern char Cyc_Set_Absent[7U];struct Cyc_Set_Absent_exn_struct{char*tag;};
# 137
void*Cyc_Set_choose(struct Cyc_Set_Set*s);struct Cyc_Dict_T;struct Cyc_Dict_Dict{int(*rel)(void*,void*);struct _RegionHandle*r;const struct Cyc_Dict_T*t;};extern char Cyc_Dict_Present[8U];struct Cyc_Dict_Present_exn_struct{char*tag;};extern char Cyc_Dict_Absent[7U];struct Cyc_Dict_Absent_exn_struct{char*tag;};
# 62 "dict.h"
struct Cyc_Dict_Dict Cyc_Dict_empty(int(*cmp)(void*,void*));
# 83
int Cyc_Dict_member(struct Cyc_Dict_Dict d,void*k);
# 87
struct Cyc_Dict_Dict Cyc_Dict_insert(struct Cyc_Dict_Dict d,void*k,void*v);
# 110
void*Cyc_Dict_lookup(struct Cyc_Dict_Dict d,void*k);
# 122 "dict.h"
void**Cyc_Dict_lookup_opt(struct Cyc_Dict_Dict d,void*k);
# 233
struct Cyc_List_List*Cyc_Dict_to_list(struct Cyc_Dict_Dict d);
# 270
struct Cyc_Dict_Dict Cyc_Dict_delete(struct Cyc_Dict_Dict,void*);
# 50 "graph.h"
struct Cyc_Dict_Dict Cyc_Graph_empty(int(*cmp)(void*,void*));
# 53
struct Cyc_Dict_Dict Cyc_Graph_add_node(struct Cyc_Dict_Dict g,void*s);
# 56
struct Cyc_Dict_Dict Cyc_Graph_add_edge(struct Cyc_Dict_Dict g,void*s,void*t);
# 59
struct Cyc_Dict_Dict Cyc_Graph_add_edges(struct Cyc_Dict_Dict g,void*s,struct Cyc_Set_Set*T);
# 69
int Cyc_Graph_is_edge(struct Cyc_Dict_Dict g,void*s,void*t);
# 72
struct Cyc_Set_Set*Cyc_Graph_get_targets(struct Cyc_Dict_Dict,void*s);
# 27 "graph.cyc"
struct Cyc_Dict_Dict Cyc_Graph_empty(int(*cmp)(void*,void*)){
return Cyc_Dict_empty(cmp);}
# 27
struct Cyc_Set_Set*Cyc_Graph_get_targets(struct Cyc_Dict_Dict g,void*source){
# 35
struct Cyc_Set_Set**_stmttmp0=({struct Cyc_Set_Set**(*_tmp5D)(struct Cyc_Dict_Dict d,void*k)=({struct Cyc_Set_Set**(*_tmp2)(struct Cyc_Dict_Dict d,void*k)=(struct Cyc_Set_Set**(*)(struct Cyc_Dict_Dict d,void*k))Cyc_Dict_lookup_opt;_tmp2;});struct Cyc_Dict_Dict _tmp5C=g;_tmp5D(_tmp5C,source);});struct Cyc_Set_Set**_tmp0=_stmttmp0;struct Cyc_Set_Set*_tmp1;if(_tmp0 == 0){_LL1: _LL2:
 return Cyc_Set_empty(g.rel);}else{_LL3: _tmp1=*_tmp0;_LL4: {struct Cyc_Set_Set*targets=_tmp1;
return targets;}}_LL0:;}
# 27
struct Cyc_Dict_Dict Cyc_Graph_add_node(struct Cyc_Dict_Dict g,void*source){
# 46
if(Cyc_Dict_member(g,source))return g;else{
return({struct Cyc_Dict_Dict(*_tmp60)(struct Cyc_Dict_Dict d,void*k,struct Cyc_Set_Set*v)=({struct Cyc_Dict_Dict(*_tmp3)(struct Cyc_Dict_Dict d,void*k,struct Cyc_Set_Set*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,void*k,struct Cyc_Set_Set*v))Cyc_Dict_insert;_tmp3;});struct Cyc_Dict_Dict _tmp5F=g;void*_tmp5E=source;_tmp60(_tmp5F,_tmp5E,Cyc_Set_empty(g.rel));});}}
# 27
struct Cyc_Dict_Dict Cyc_Graph_add_edge(struct Cyc_Dict_Dict g,void*source,void*target){
# 54
struct Cyc_Set_Set*sourceTargets=Cyc_Graph_get_targets(g,source);
return({struct Cyc_Dict_Dict(*_tmp63)(struct Cyc_Dict_Dict d,void*k,struct Cyc_Set_Set*v)=({struct Cyc_Dict_Dict(*_tmp4)(struct Cyc_Dict_Dict d,void*k,struct Cyc_Set_Set*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,void*k,struct Cyc_Set_Set*v))Cyc_Dict_insert;_tmp4;});struct Cyc_Dict_Dict _tmp62=g;void*_tmp61=source;_tmp63(_tmp62,_tmp61,Cyc_Set_insert(sourceTargets,target));});}
# 27
struct Cyc_Dict_Dict Cyc_Graph_add_edges(struct Cyc_Dict_Dict g,void*source,struct Cyc_Set_Set*targets){
# 62
struct Cyc_Set_Set*sourceTargets=Cyc_Graph_get_targets(g,source);
return({struct Cyc_Dict_Dict(*_tmp66)(struct Cyc_Dict_Dict d,void*k,struct Cyc_Set_Set*v)=({struct Cyc_Dict_Dict(*_tmp5)(struct Cyc_Dict_Dict d,void*k,struct Cyc_Set_Set*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,void*k,struct Cyc_Set_Set*v))Cyc_Dict_insert;_tmp5;});struct Cyc_Dict_Dict _tmp65=g;void*_tmp64=source;_tmp66(_tmp65,_tmp64,Cyc_Set_union_two(sourceTargets,targets));});}
# 27
struct Cyc_Dict_Dict Cyc_Graph_remove_edge(struct Cyc_Dict_Dict g,void*source,void*target){
# 70
struct Cyc_Set_Set*sourceTargets=Cyc_Graph_get_targets(g,source);
return({struct Cyc_Dict_Dict(*_tmp69)(struct Cyc_Dict_Dict d,void*k,struct Cyc_Set_Set*v)=({struct Cyc_Dict_Dict(*_tmp6)(struct Cyc_Dict_Dict d,void*k,struct Cyc_Set_Set*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,void*k,struct Cyc_Set_Set*v))Cyc_Dict_insert;_tmp6;});struct Cyc_Dict_Dict _tmp68=g;void*_tmp67=source;_tmp69(_tmp68,_tmp67,Cyc_Set_delete(sourceTargets,target));});}
# 27
struct Cyc_Dict_Dict Cyc_Graph_remove_edges(struct Cyc_Dict_Dict g,void*source,struct Cyc_Set_Set*targets){
# 78
struct Cyc_Set_Set*sourceTargets=Cyc_Graph_get_targets(g,source);
return({struct Cyc_Dict_Dict(*_tmp6C)(struct Cyc_Dict_Dict d,void*k,struct Cyc_Set_Set*v)=({struct Cyc_Dict_Dict(*_tmp7)(struct Cyc_Dict_Dict d,void*k,struct Cyc_Set_Set*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,void*k,struct Cyc_Set_Set*v))Cyc_Dict_insert;_tmp7;});struct Cyc_Dict_Dict _tmp6B=g;void*_tmp6A=source;_tmp6C(_tmp6B,_tmp6A,Cyc_Set_diff(sourceTargets,targets));});}
# 27
static struct Cyc_List_List*Cyc_Graph_to_list(struct Cyc_Set_Set*ts){
# 88
struct Cyc_List_List*result=0;
while(!Cyc_Set_is_empty(ts)){
void*z=Cyc_Set_choose(ts);
ts=Cyc_Set_delete(ts,z);
result=({struct Cyc_List_List*_tmp8=_cycalloc(sizeof(*_tmp8));_tmp8->hd=z,_tmp8->tl=result;_tmp8;});}
# 94
return result;}
# 27
int Cyc_Graph_is_edge(struct Cyc_Dict_Dict g,void*source,void*target){
# 101
struct Cyc_Set_Set**_stmttmp1=({struct Cyc_Set_Set**(*_tmp6E)(struct Cyc_Dict_Dict d,void*k)=({struct Cyc_Set_Set**(*_tmpB)(struct Cyc_Dict_Dict d,void*k)=(struct Cyc_Set_Set**(*)(struct Cyc_Dict_Dict d,void*k))Cyc_Dict_lookup_opt;_tmpB;});struct Cyc_Dict_Dict _tmp6D=g;_tmp6E(_tmp6D,source);});struct Cyc_Set_Set**_tmp9=_stmttmp1;struct Cyc_Set_Set*_tmpA;if(_tmp9 == 0){_LL1: _LL2:
# 103
 return 0;}else{_LL3: _tmpA=*_tmp9;_LL4: {struct Cyc_Set_Set*sourceTargets=_tmpA;
# 105
return Cyc_Set_member(sourceTargets,target);}}_LL0:;}struct _tuple0{void*f1;struct Cyc_Set_Set*f2;};
# 27
void Cyc_Graph_print(struct Cyc___cycFILE*f,struct Cyc_Dict_Dict g,void(*nodeprint)(struct Cyc___cycFILE*,void*)){
# 113
({void*_tmpC=0U;({struct Cyc___cycFILE*_tmp70=f;struct _fat_ptr _tmp6F=({const char*_tmpD="digraph {\n";_tag_fat(_tmpD,sizeof(char),11U);});Cyc_fprintf(_tmp70,_tmp6F,_tag_fat(_tmpC,sizeof(void*),0U));});});{
struct Cyc_List_List*edges=Cyc_Dict_to_list(g);
for(0;(unsigned)edges;edges=edges->tl){
struct _tuple0*_stmttmp2=(struct _tuple0*)edges->hd;struct Cyc_Set_Set*_tmpF;void*_tmpE;_LL1: _tmpE=_stmttmp2->f1;_tmpF=_stmttmp2->f2;_LL2: {void*s=_tmpE;struct Cyc_Set_Set*ts=_tmpF;
struct Cyc_List_List*tslist=Cyc_Graph_to_list(ts);
if((unsigned)tslist)
for(0;(unsigned)tslist;tslist=tslist->tl){
nodeprint(f,s);
({void*_tmp10=0U;({struct Cyc___cycFILE*_tmp72=f;struct _fat_ptr _tmp71=({const char*_tmp11=" -> ";_tag_fat(_tmp11,sizeof(char),5U);});Cyc_fprintf(_tmp72,_tmp71,_tag_fat(_tmp10,sizeof(void*),0U));});});
nodeprint(f,tslist->hd);
({void*_tmp12=0U;({struct Cyc___cycFILE*_tmp74=f;struct _fat_ptr _tmp73=({const char*_tmp13=";\n";_tag_fat(_tmp13,sizeof(char),3U);});Cyc_fprintf(_tmp74,_tmp73,_tag_fat(_tmp12,sizeof(void*),0U));});});}else{
# 126
({void*_tmp14=0U;({struct Cyc___cycFILE*_tmp76=f;struct _fat_ptr _tmp75=({const char*_tmp15="node ";_tag_fat(_tmp15,sizeof(char),6U);});Cyc_fprintf(_tmp76,_tmp75,_tag_fat(_tmp14,sizeof(void*),0U));});});
nodeprint(f,s);
({void*_tmp16=0U;({struct Cyc___cycFILE*_tmp78=f;struct _fat_ptr _tmp77=({const char*_tmp17="; // no targets\n";_tag_fat(_tmp17,sizeof(char),17U);});Cyc_fprintf(_tmp78,_tmp77,_tag_fat(_tmp16,sizeof(void*),0U));});});}}}
# 131
({void*_tmp18=0U;({struct Cyc___cycFILE*_tmp7A=f;struct _fat_ptr _tmp79=({const char*_tmp19="}\n";_tag_fat(_tmp19,sizeof(char),3U);});Cyc_fprintf(_tmp7A,_tmp79,_tag_fat(_tmp18,sizeof(void*),0U));});});}}
# 27
static struct Cyc_List_List*Cyc_Graph_sourcesOf(struct Cyc_Dict_Dict g,void*node){
# 142 "graph.cyc"
struct Cyc_List_List*result=0;
# 144
{struct Cyc_List_List*edges=Cyc_Dict_to_list(g);for(0;(unsigned)edges;edges=edges->tl){
struct _tuple0*_stmttmp3=(struct _tuple0*)edges->hd;struct Cyc_Set_Set*_tmp1B;void*_tmp1A;_LL1: _tmp1A=_stmttmp3->f1;_tmp1B=_stmttmp3->f2;_LL2: {void*s=_tmp1A;struct Cyc_Set_Set*ts=_tmp1B;
if(Cyc_Set_member(ts,node))result=({struct Cyc_List_List*_tmp1C=_cycalloc(sizeof(*_tmp1C));_tmp1C->hd=s,_tmp1C->tl=result;_tmp1C;});}}}
# 148
return result;}struct _tuple1{struct Cyc_Set_Set*f1;struct Cyc_Dict_Dict f2;};
# 27 "graph.cyc"
static struct _tuple1 Cyc_Graph_divideGraph(struct Cyc_Dict_Dict g,void*source){
# 160 "graph.cyc"
struct Cyc_Set_Set**_stmttmp4=({struct Cyc_Set_Set**(*_tmp7C)(struct Cyc_Dict_Dict d,void*k)=({struct Cyc_Set_Set**(*_tmp1F)(struct Cyc_Dict_Dict d,void*k)=(struct Cyc_Set_Set**(*)(struct Cyc_Dict_Dict d,void*k))Cyc_Dict_lookup_opt;_tmp1F;});struct Cyc_Dict_Dict _tmp7B=g;_tmp7C(_tmp7B,source);});struct Cyc_Set_Set**_tmp1D=_stmttmp4;struct Cyc_Set_Set*_tmp1E;if(_tmp1D == 0){_LL1: _LL2:
 return({struct _tuple1 _tmp5A;({struct Cyc_Set_Set*_tmp7D=Cyc_Set_empty(g.rel);_tmp5A.f1=_tmp7D;}),_tmp5A.f2=g;_tmp5A;});}else{_LL3: _tmp1E=*_tmp1D;_LL4: {struct Cyc_Set_Set*sourceTargets=_tmp1E;
return({struct _tuple1 _tmp5B;_tmp5B.f1=sourceTargets,({struct Cyc_Dict_Dict _tmp7E=Cyc_Dict_delete(g,source);_tmp5B.f2=_tmp7E;});_tmp5B;});}}_LL0:;}
# 27 "graph.cyc"
static struct Cyc_Dict_Dict Cyc_Graph_add_edgeTc(struct Cyc_Dict_Dict g,void*source,void*target){
# 170 "graph.cyc"
struct Cyc_Set_Set*targetTargets=Cyc_Graph_get_targets(g,target);
struct _tuple1 _stmttmp5=Cyc_Graph_divideGraph(g,source);struct Cyc_Dict_Dict _tmp21;struct Cyc_Set_Set*_tmp20;_LL1: _tmp20=_stmttmp5.f1;_tmp21=_stmttmp5.f2;_LL2: {struct Cyc_Set_Set*sourceTargets=_tmp20;struct Cyc_Dict_Dict graphWithoutSource=_tmp21;
struct Cyc_List_List*sourceSources=Cyc_Graph_sourcesOf(graphWithoutSource,source);
struct Cyc_Set_Set*newSourceTargets=({struct Cyc_Set_Set*_tmp7F=Cyc_Set_union_two(sourceTargets,targetTargets);Cyc_Set_insert(_tmp7F,target);});
struct Cyc_Dict_Dict result=({struct Cyc_Dict_Dict(*_tmp82)(struct Cyc_Dict_Dict d,void*k,struct Cyc_Set_Set*v)=({struct Cyc_Dict_Dict(*_tmp22)(struct Cyc_Dict_Dict d,void*k,struct Cyc_Set_Set*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,void*k,struct Cyc_Set_Set*v))Cyc_Dict_insert;_tmp22;});struct Cyc_Dict_Dict _tmp81=graphWithoutSource;void*_tmp80=source;_tmp82(_tmp81,_tmp80,newSourceTargets);});
{struct Cyc_List_List*s=sourceSources;for(0;(unsigned)s;s=s->tl){
result=Cyc_Graph_add_edges(result,s->hd,newSourceTargets);}}
# 178
return result;}}
# 27 "graph.cyc"
struct Cyc_Dict_Dict Cyc_Graph_tc_slow(struct Cyc_Dict_Dict g){
# 181 "graph.cyc"
struct Cyc_Dict_Dict result=Cyc_Graph_empty(g.rel);
struct Cyc_List_List*edges=Cyc_Dict_to_list(g);
for(0;(unsigned)edges;edges=edges->tl){
struct _tuple0*_stmttmp6=(struct _tuple0*)edges->hd;struct Cyc_Set_Set*_tmp24;void*_tmp23;_LL1: _tmp23=_stmttmp6->f1;_tmp24=_stmttmp6->f2;_LL2: {void*source=_tmp23;struct Cyc_Set_Set*targets=_tmp24;
struct Cyc_List_List*tslist=Cyc_Graph_to_list(targets);
if((unsigned)tslist){
struct Cyc_List_List*t=tslist;for(0;(unsigned)t;t=t->tl){
result=Cyc_Graph_add_edgeTc(result,source,t->hd);}}else{
result=Cyc_Graph_add_node(result,source);}}}
# 191
return result;}
# 27 "graph.cyc"
struct Cyc_Dict_Dict Cyc_Graph_tkernel(struct Cyc_Dict_Dict g){
# 199 "graph.cyc"
int(*cmp)(void*,void*)=g.rel;
struct Cyc_Dict_Dict closure=Cyc_Graph_empty(cmp);
struct Cyc_Dict_Dict result=Cyc_Graph_empty(cmp);
{struct Cyc_List_List*edges=Cyc_Dict_to_list(g);for(0;(unsigned)edges;edges=edges->tl){
struct _tuple0*_stmttmp7=(struct _tuple0*)edges->hd;struct Cyc_Set_Set*_tmp26;void*_tmp25;_LL1: _tmp25=_stmttmp7->f1;_tmp26=_stmttmp7->f2;_LL2: {void*source=_tmp25;struct Cyc_Set_Set*targets=_tmp26;
while(!Cyc_Set_is_empty(targets)){
void*target=Cyc_Set_choose(targets);
targets=Cyc_Set_delete(targets,target);
if(Cyc_Graph_is_edge(closure,source,target))continue;closure=
Cyc_Graph_add_edgeTc(closure,source,target);
result=Cyc_Graph_add_edge(result,source,target);}}}}
# 212
return result;}struct Cyc_Graph_nodestate{void*root;int C;int visitindex;};struct Cyc_Graph_componentstate{struct Cyc_Set_Set*Succ;struct Cyc_Set_Set*nodes;};struct Cyc_Graph_tcstate{struct Cyc_Set_Set*visited;int visitindex;struct Cyc_Dict_Dict ns;struct Cyc_Dict_Dict cs;int Cindex;struct Cyc_List_List*nstack;struct Cyc_List_List*cstack;};
# 238
static void Cyc_Graph_comp_tc(struct Cyc_Graph_tcstate*ts,struct Cyc_Dict_Dict g,void*v){
# 240
int(*cmp)(void*,void*)=g.rel;
({struct Cyc_Set_Set*_tmp83=Cyc_Set_insert(ts->visited,v);ts->visited=_tmp83;});{
struct Cyc_Graph_nodestate*nsv=({struct Cyc_Graph_nodestate*_tmp38=_cycalloc(sizeof(*_tmp38));_tmp38->root=v,_tmp38->C=0,_tmp38->visitindex=ts->visitindex ++;_tmp38;});
({struct Cyc_Dict_Dict _tmp87=({struct Cyc_Dict_Dict(*_tmp86)(struct Cyc_Dict_Dict d,void*k,struct Cyc_Graph_nodestate*v)=({struct Cyc_Dict_Dict(*_tmp27)(struct Cyc_Dict_Dict d,void*k,struct Cyc_Graph_nodestate*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,void*k,struct Cyc_Graph_nodestate*v))Cyc_Dict_insert;_tmp27;});struct Cyc_Dict_Dict _tmp85=ts->ns;void*_tmp84=v;_tmp86(_tmp85,_tmp84,nsv);});ts->ns=_tmp87;});
({struct Cyc_List_List*_tmp88=({struct Cyc_List_List*_tmp28=_cycalloc(sizeof(*_tmp28));_tmp28->hd=v,_tmp28->tl=ts->nstack;_tmp28;});ts->nstack=_tmp88;});{
int hsaved=Cyc_List_length(ts->cstack);
struct Cyc_Set_Set*targets=Cyc_Graph_get_targets(g,v);
while(!Cyc_Set_is_empty(targets)){
void*w=Cyc_Set_choose(targets);
targets=Cyc_Set_delete(targets,w);
if(cmp(v,w)== 0)continue;{int is_forward_edge;
# 252
struct Cyc_Graph_nodestate*nsw;
if(Cyc_Set_member(ts->visited,w)){
nsw=({struct Cyc_Graph_nodestate*(*_tmp8A)(struct Cyc_Dict_Dict d,void*k)=({struct Cyc_Graph_nodestate*(*_tmp29)(struct Cyc_Dict_Dict d,void*k)=(struct Cyc_Graph_nodestate*(*)(struct Cyc_Dict_Dict d,void*k))Cyc_Dict_lookup;_tmp29;});struct Cyc_Dict_Dict _tmp89=ts->ns;_tmp8A(_tmp89,w);});
is_forward_edge=nsw->visitindex < nsv->visitindex?0: 1;}else{
# 258
is_forward_edge=0;
Cyc_Graph_comp_tc(ts,g,w);
nsw=({struct Cyc_Graph_nodestate*(*_tmp8C)(struct Cyc_Dict_Dict d,void*k)=({struct Cyc_Graph_nodestate*(*_tmp2A)(struct Cyc_Dict_Dict d,void*k)=(struct Cyc_Graph_nodestate*(*)(struct Cyc_Dict_Dict d,void*k))Cyc_Dict_lookup;_tmp2A;});struct Cyc_Dict_Dict _tmp8B=ts->ns;_tmp8C(_tmp8B,w);});}{
# 262
int Cw=nsw->C;
if(Cw == 0){
struct Cyc_Graph_nodestate*nsrootv=({struct Cyc_Graph_nodestate*(*_tmp8E)(struct Cyc_Dict_Dict d,void*k)=({struct Cyc_Graph_nodestate*(*_tmp2C)(struct Cyc_Dict_Dict d,void*k)=(struct Cyc_Graph_nodestate*(*)(struct Cyc_Dict_Dict d,void*k))Cyc_Dict_lookup;_tmp2C;});struct Cyc_Dict_Dict _tmp8D=ts->ns;_tmp8E(_tmp8D,nsv->root);});
struct Cyc_Graph_nodestate*nsrootw=({struct Cyc_Graph_nodestate*(*_tmp90)(struct Cyc_Dict_Dict d,void*k)=({struct Cyc_Graph_nodestate*(*_tmp2B)(struct Cyc_Dict_Dict d,void*k)=(struct Cyc_Graph_nodestate*(*)(struct Cyc_Dict_Dict d,void*k))Cyc_Dict_lookup;_tmp2B;});struct Cyc_Dict_Dict _tmp8F=ts->ns;_tmp90(_tmp8F,nsw->root);});
# 267
if(nsrootv->visitindex > nsrootw->visitindex)
nsv->root=nsw->root;}else{
# 271
if(!is_forward_edge)
({struct Cyc_List_List*_tmp91=({struct Cyc_List_List*_tmp2D=_cycalloc(sizeof(*_tmp2D));_tmp2D->hd=(void*)Cw,_tmp2D->tl=ts->cstack;_tmp2D;});ts->cstack=_tmp91;});}}}}
# 274
if(cmp(nsv->root,v)!= 0)
return;{
# 274
int Cnew=ts->Cindex ++;
# 278
struct Cyc_Graph_componentstate*csCnew=({struct Cyc_Graph_componentstate*_tmp37=_cycalloc(sizeof(*_tmp37));({struct Cyc_Set_Set*_tmp93=({struct Cyc_Set_Set*(*_tmp36)(int(*cmp)(int,int))=(struct Cyc_Set_Set*(*)(int(*cmp)(int,int)))Cyc_Set_empty;_tmp36;})(Cyc_Core_intcmp);_tmp37->Succ=_tmp93;}),({
struct Cyc_Set_Set*_tmp92=Cyc_Set_empty(cmp);_tmp37->nodes=_tmp92;});_tmp37;});
({struct Cyc_Dict_Dict _tmp97=({struct Cyc_Dict_Dict(*_tmp96)(struct Cyc_Dict_Dict d,int k,struct Cyc_Graph_componentstate*v)=({struct Cyc_Dict_Dict(*_tmp2E)(struct Cyc_Dict_Dict d,int k,struct Cyc_Graph_componentstate*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,int k,struct Cyc_Graph_componentstate*v))Cyc_Dict_insert;_tmp2E;});struct Cyc_Dict_Dict _tmp95=ts->cs;int _tmp94=Cnew;_tmp96(_tmp95,_tmp94,csCnew);});ts->cs=_tmp97;});
{struct Cyc_List_List*_stmttmp8=ts->nstack;struct Cyc_List_List*_tmp2F=_stmttmp8;void*_tmp30;if(_tmp2F == 0){_LL1: _LL2:
 goto _LL0;}else{_LL3: _tmp30=(void*)_tmp2F->hd;_LL4: {void*top=_tmp30;
# 284
if(cmp(top,v)!= 0 ||
 Cyc_Graph_is_edge(g,v,v))
({struct Cyc_Set_Set*_tmp9A=({struct Cyc_Set_Set*(*_tmp99)(struct Cyc_Set_Set*s,int elt)=({struct Cyc_Set_Set*(*_tmp31)(struct Cyc_Set_Set*s,int elt)=(struct Cyc_Set_Set*(*)(struct Cyc_Set_Set*s,int elt))Cyc_Set_insert;_tmp31;});struct Cyc_Set_Set*_tmp98=csCnew->Succ;_tmp99(_tmp98,Cnew);});csCnew->Succ=_tmp9A;});
# 284
goto _LL0;}}_LL0:;}
# 290
{int hnow=Cyc_List_length(ts->cstack);for(0;hnow > hsaved;-- hnow){
int X=(int)((struct Cyc_List_List*)_check_null(ts->cstack))->hd;
ts->cstack=((struct Cyc_List_List*)_check_null(ts->cstack))->tl;
if(!({int(*_tmp9C)(struct Cyc_Set_Set*s,int elt)=({int(*_tmp32)(struct Cyc_Set_Set*s,int elt)=(int(*)(struct Cyc_Set_Set*s,int elt))Cyc_Set_member;_tmp32;});struct Cyc_Set_Set*_tmp9B=csCnew->Succ;_tmp9C(_tmp9B,X);})){
struct Cyc_Set_Set*s=({struct Cyc_Set_Set*(*_tmp9E)(struct Cyc_Set_Set*s,int elt)=({struct Cyc_Set_Set*(*_tmp34)(struct Cyc_Set_Set*s,int elt)=(struct Cyc_Set_Set*(*)(struct Cyc_Set_Set*s,int elt))Cyc_Set_insert;_tmp34;});struct Cyc_Set_Set*_tmp9D=csCnew->Succ;_tmp9E(_tmp9D,X);});
s=({struct Cyc_Set_Set*_tmpA1=s;Cyc_Set_union_two(_tmpA1,({struct Cyc_Graph_componentstate*(*_tmpA0)(struct Cyc_Dict_Dict d,int k)=({struct Cyc_Graph_componentstate*(*_tmp33)(struct Cyc_Dict_Dict d,int k)=(struct Cyc_Graph_componentstate*(*)(struct Cyc_Dict_Dict d,int k))Cyc_Dict_lookup;_tmp33;});struct Cyc_Dict_Dict _tmp9F=ts->cs;_tmpA0(_tmp9F,X);})->Succ);});
csCnew->Succ=s;}}}
# 299
while((unsigned)ts->nstack){
void*w=((struct Cyc_List_List*)_check_null(ts->nstack))->hd;
ts->nstack=((struct Cyc_List_List*)_check_null(ts->nstack))->tl;{
struct Cyc_Graph_nodestate*nsw=({struct Cyc_Graph_nodestate*(*_tmpA3)(struct Cyc_Dict_Dict d,void*k)=({struct Cyc_Graph_nodestate*(*_tmp35)(struct Cyc_Dict_Dict d,void*k)=(struct Cyc_Graph_nodestate*(*)(struct Cyc_Dict_Dict d,void*k))Cyc_Dict_lookup;_tmp35;});struct Cyc_Dict_Dict _tmpA2=ts->ns;_tmpA3(_tmpA2,w);});
nsw->C=Cnew;
({struct Cyc_Set_Set*_tmpA4=Cyc_Set_insert(csCnew->nodes,w);csCnew->nodes=_tmpA4;});
if(cmp(w,v)== 0)break;}}}}}}
# 238
struct Cyc_Dict_Dict Cyc_Graph_tc(struct Cyc_Dict_Dict g){
# 309
int(*cmp)(void*,void*)=g.rel;
struct Cyc_Graph_tcstate*ts=({struct Cyc_Graph_tcstate*_tmp40=_cycalloc(sizeof(*_tmp40));
({
struct Cyc_Set_Set*_tmpA7=Cyc_Set_empty(cmp);_tmp40->visited=_tmpA7;}),_tmp40->visitindex=1,({
# 314
struct Cyc_Dict_Dict _tmpA6=Cyc_Dict_empty(cmp);_tmp40->ns=_tmpA6;}),({
struct Cyc_Dict_Dict _tmpA5=({struct Cyc_Dict_Dict(*_tmp3F)(int(*cmp)(int,int))=(struct Cyc_Dict_Dict(*)(int(*cmp)(int,int)))Cyc_Dict_empty;_tmp3F;})(Cyc_Core_intcmp);_tmp40->cs=_tmpA5;}),_tmp40->Cindex=1,_tmp40->nstack=0,_tmp40->cstack=0;_tmp40;});
# 320
{struct Cyc_List_List*edges=Cyc_Dict_to_list(g);for(0;(unsigned)edges;edges=edges->tl){
struct _tuple0*_stmttmp9=(struct _tuple0*)edges->hd;struct Cyc_Set_Set*_tmp3A;void*_tmp39;_LL1: _tmp39=_stmttmp9->f1;_tmp3A=_stmttmp9->f2;_LL2: {void*source=_tmp39;struct Cyc_Set_Set*targets=_tmp3A;
if(!Cyc_Set_member(ts->visited,source))Cyc_Graph_comp_tc(ts,g,source);}}}{
# 325
struct Cyc_Dict_Dict result=Cyc_Graph_empty(cmp);
{int C=1;for(0;C < ts->Cindex;++ C){
struct Cyc_Graph_componentstate*cs=({struct Cyc_Graph_componentstate*(*_tmpA9)(struct Cyc_Dict_Dict d,int k)=({struct Cyc_Graph_componentstate*(*_tmp3E)(struct Cyc_Dict_Dict d,int k)=(struct Cyc_Graph_componentstate*(*)(struct Cyc_Dict_Dict d,int k))Cyc_Dict_lookup;_tmp3E;});struct Cyc_Dict_Dict _tmpA8=ts->cs;_tmpA9(_tmpA8,C);});
struct Cyc_Set_Set*targets=Cyc_Set_empty(cmp);
struct Cyc_Set_Set*Succ=cs->Succ;
while(!Cyc_Set_is_empty(Succ)){
int C2=({int(*_tmpAA)(struct Cyc_Set_Set*s)=({int(*_tmp3D)(struct Cyc_Set_Set*s)=(int(*)(struct Cyc_Set_Set*s))Cyc_Set_choose;_tmp3D;});_tmpAA(Succ);});
Succ=({struct Cyc_Set_Set*(*_tmpAC)(struct Cyc_Set_Set*s,int elt)=({struct Cyc_Set_Set*(*_tmp3B)(struct Cyc_Set_Set*s,int elt)=(struct Cyc_Set_Set*(*)(struct Cyc_Set_Set*s,int elt))Cyc_Set_delete;_tmp3B;});struct Cyc_Set_Set*_tmpAB=Succ;_tmpAC(_tmpAB,C2);});
targets=({struct Cyc_Set_Set*_tmpAF=targets;Cyc_Set_union_two(_tmpAF,({
struct Cyc_Graph_componentstate*(*_tmpAE)(struct Cyc_Dict_Dict d,int k)=({struct Cyc_Graph_componentstate*(*_tmp3C)(struct Cyc_Dict_Dict d,int k)=(struct Cyc_Graph_componentstate*(*)(struct Cyc_Dict_Dict d,int k))Cyc_Dict_lookup;_tmp3C;});struct Cyc_Dict_Dict _tmpAD=ts->cs;_tmpAE(_tmpAD,C2);})->nodes);});}{
# 336
struct Cyc_Set_Set*nodes=cs->nodes;
while(!Cyc_Set_is_empty(nodes)){
void*v=Cyc_Set_choose(nodes);
nodes=Cyc_Set_delete(nodes,v);
result=Cyc_Graph_add_edges(result,v,targets);}}}}
# 343
return result;}}
# 238
static void Cyc_Graph_traverse(struct Cyc_Dict_Dict input,struct Cyc_Dict_Dict*output,struct Cyc_Dict_Dict*N,struct Cyc_List_List**S,void*x,unsigned k){
# 360 "graph.cyc"
({struct Cyc_List_List*_tmpB0=({struct Cyc_List_List*_tmp41=_cycalloc(sizeof(*_tmp41));_tmp41->hd=x,_tmp41->tl=*S;_tmp41;});*S=_tmpB0;});
({struct Cyc_Dict_Dict _tmpB4=({struct Cyc_Dict_Dict(*_tmpB3)(struct Cyc_Dict_Dict d,void*k,unsigned v)=({struct Cyc_Dict_Dict(*_tmp42)(struct Cyc_Dict_Dict d,void*k,unsigned v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,void*k,unsigned v))Cyc_Dict_insert;_tmp42;});struct Cyc_Dict_Dict _tmpB2=*N;void*_tmpB1=x;_tmpB3(_tmpB2,_tmpB1,k);});*N=_tmpB4;});
{struct Cyc_Set_Set*targets=Cyc_Graph_get_targets(input,x);for(0;!Cyc_Set_is_empty(targets);0){
void*y=Cyc_Set_choose(targets);
targets=Cyc_Set_delete(targets,y);
if(({unsigned(*_tmpB6)(struct Cyc_Dict_Dict d,void*k)=({unsigned(*_tmp43)(struct Cyc_Dict_Dict d,void*k)=(unsigned(*)(struct Cyc_Dict_Dict d,void*k))Cyc_Dict_lookup;_tmp43;});struct Cyc_Dict_Dict _tmpB5=*N;_tmpB6(_tmpB5,y);})== (unsigned)0)
Cyc_Graph_traverse(input,output,N,S,y,k + (unsigned)1);{
# 365
unsigned Ny=({
# 367
unsigned(*_tmpB8)(struct Cyc_Dict_Dict d,void*k)=({unsigned(*_tmp46)(struct Cyc_Dict_Dict d,void*k)=(unsigned(*)(struct Cyc_Dict_Dict d,void*k))Cyc_Dict_lookup;_tmp46;});struct Cyc_Dict_Dict _tmpB7=*N;_tmpB8(_tmpB7,y);});
if(({unsigned _tmpBB=Ny;_tmpBB < ({unsigned(*_tmpBA)(struct Cyc_Dict_Dict d,void*k)=({unsigned(*_tmp44)(struct Cyc_Dict_Dict d,void*k)=(unsigned(*)(struct Cyc_Dict_Dict d,void*k))Cyc_Dict_lookup;_tmp44;});struct Cyc_Dict_Dict _tmpB9=*N;_tmpBA(_tmpB9,x);});}))
({struct Cyc_Dict_Dict _tmpBF=({struct Cyc_Dict_Dict(*_tmpBE)(struct Cyc_Dict_Dict d,void*k,unsigned v)=({struct Cyc_Dict_Dict(*_tmp45)(struct Cyc_Dict_Dict d,void*k,unsigned v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,void*k,unsigned v))Cyc_Dict_insert;_tmp45;});struct Cyc_Dict_Dict _tmpBD=*N;void*_tmpBC=x;_tmpBE(_tmpBD,_tmpBC,Ny);});*N=_tmpBF;});}}}
# 371
if(({unsigned _tmpC2=({unsigned(*_tmpC1)(struct Cyc_Dict_Dict d,void*k)=({unsigned(*_tmp47)(struct Cyc_Dict_Dict d,void*k)=(unsigned(*)(struct Cyc_Dict_Dict d,void*k))Cyc_Dict_lookup;_tmp47;});struct Cyc_Dict_Dict _tmpC0=*N;_tmpC1(_tmpC0,x);});_tmpC2 == k;})){
int(*cmp)(void*,void*)=(*output).rel;
struct Cyc_Set_Set*component=Cyc_Set_empty(cmp);
{struct Cyc_List_List*s=*S;for(0;(unsigned)s;s=s->tl){
void*top=s->hd;
component=Cyc_Set_insert(component,top);
if(cmp(top,x)== 0)break;}}
# 379
for(0;(unsigned)*S;*S=((struct Cyc_List_List*)_check_null(*S))->tl){
void*top=((struct Cyc_List_List*)_check_null(*S))->hd;
({struct Cyc_Dict_Dict _tmpC5=({struct Cyc_Dict_Dict(*_tmpC4)(struct Cyc_Dict_Dict d,void*k,unsigned v)=({struct Cyc_Dict_Dict(*_tmp48)(struct Cyc_Dict_Dict d,void*k,unsigned v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,void*k,unsigned v))Cyc_Dict_insert;_tmp48;});struct Cyc_Dict_Dict _tmpC3=*N;_tmpC4(_tmpC3,top,- 1U);});*N=_tmpC5;});
({struct Cyc_Dict_Dict _tmpC6=Cyc_Graph_add_node(*output,top);*output=_tmpC6;});
({struct Cyc_Dict_Dict _tmpC7=Cyc_Graph_add_edges(*output,top,component);*output=_tmpC7;});
if(cmp(top,x)== 0){
*S=((struct Cyc_List_List*)_check_null(*S))->tl;
break;}}}}
# 238 "graph.cyc"
struct Cyc_Dict_Dict Cyc_Graph_scc(struct Cyc_Dict_Dict input){
# 392 "graph.cyc"
int(*cmp)(void*,void*)=input.rel;
struct Cyc_List_List*edges=Cyc_Dict_to_list(input);
struct Cyc_Dict_Dict output=Cyc_Graph_empty(cmp);
struct Cyc_List_List*S=0;
struct Cyc_Dict_Dict N=Cyc_Dict_empty(cmp);
{struct Cyc_List_List*e=edges;for(0;(unsigned)e;e=e->tl){
struct _tuple0*_stmttmpA=(struct _tuple0*)e->hd;void*_tmp49;_LL1: _tmp49=_stmttmpA->f1;_LL2: {void*x=_tmp49;
N=({struct Cyc_Dict_Dict(*_tmpC9)(struct Cyc_Dict_Dict d,void*k,unsigned v)=({struct Cyc_Dict_Dict(*_tmp4A)(struct Cyc_Dict_Dict d,void*k,unsigned v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,void*k,unsigned v))Cyc_Dict_insert;_tmp4A;});struct Cyc_Dict_Dict _tmpC8=N;_tmpC9(_tmpC8,x,0U);});}}}
# 401
{struct Cyc_List_List*e=edges;for(0;(unsigned)e;e=e->tl){
struct _tuple0*_stmttmpB=(struct _tuple0*)e->hd;void*_tmp4B;_LL4: _tmp4B=_stmttmpB->f1;_LL5: {void*x=_tmp4B;
if(({unsigned(*_tmpCB)(struct Cyc_Dict_Dict d,void*k)=({unsigned(*_tmp4C)(struct Cyc_Dict_Dict d,void*k)=(unsigned(*)(struct Cyc_Dict_Dict d,void*k))Cyc_Dict_lookup;_tmp4C;});struct Cyc_Dict_Dict _tmpCA=N;_tmpCB(_tmpCA,x);})== (unsigned)0)
Cyc_Graph_traverse(input,& output,& N,& S,x,1U);}}}
# 406
return output;}
# 238 "graph.cyc"
static void Cyc_Graph_tsort0(struct Cyc_Dict_Dict input,struct Cyc_List_List**output,struct Cyc_Dict_Dict*N,struct Cyc_List_List**S,void*x,unsigned k){
# 414 "graph.cyc"
({struct Cyc_List_List*_tmpCC=({struct Cyc_List_List*_tmp4D=_cycalloc(sizeof(*_tmp4D));_tmp4D->hd=x,_tmp4D->tl=*S;_tmp4D;});*S=_tmpCC;});
({struct Cyc_Dict_Dict _tmpD0=({struct Cyc_Dict_Dict(*_tmpCF)(struct Cyc_Dict_Dict d,void*k,unsigned v)=({struct Cyc_Dict_Dict(*_tmp4E)(struct Cyc_Dict_Dict d,void*k,unsigned v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,void*k,unsigned v))Cyc_Dict_insert;_tmp4E;});struct Cyc_Dict_Dict _tmpCE=*N;void*_tmpCD=x;_tmpCF(_tmpCE,_tmpCD,k);});*N=_tmpD0;});
{struct Cyc_Set_Set*targets=Cyc_Graph_get_targets(input,x);for(0;!Cyc_Set_is_empty(targets);0){
void*y=Cyc_Set_choose(targets);
targets=Cyc_Set_delete(targets,y);
if(({unsigned(*_tmpD2)(struct Cyc_Dict_Dict d,void*k)=({unsigned(*_tmp4F)(struct Cyc_Dict_Dict d,void*k)=(unsigned(*)(struct Cyc_Dict_Dict d,void*k))Cyc_Dict_lookup;_tmp4F;});struct Cyc_Dict_Dict _tmpD1=*N;_tmpD2(_tmpD1,y);})== (unsigned)0)
Cyc_Graph_tsort0(input,output,N,S,y,k + (unsigned)1);{
# 419
unsigned Ny=({
# 421
unsigned(*_tmpD4)(struct Cyc_Dict_Dict d,void*k)=({unsigned(*_tmp52)(struct Cyc_Dict_Dict d,void*k)=(unsigned(*)(struct Cyc_Dict_Dict d,void*k))Cyc_Dict_lookup;_tmp52;});struct Cyc_Dict_Dict _tmpD3=*N;_tmpD4(_tmpD3,y);});
if(({unsigned _tmpD7=Ny;_tmpD7 < ({unsigned(*_tmpD6)(struct Cyc_Dict_Dict d,void*k)=({unsigned(*_tmp50)(struct Cyc_Dict_Dict d,void*k)=(unsigned(*)(struct Cyc_Dict_Dict d,void*k))Cyc_Dict_lookup;_tmp50;});struct Cyc_Dict_Dict _tmpD5=*N;_tmpD6(_tmpD5,x);});}))
({struct Cyc_Dict_Dict _tmpDB=({struct Cyc_Dict_Dict(*_tmpDA)(struct Cyc_Dict_Dict d,void*k,unsigned v)=({struct Cyc_Dict_Dict(*_tmp51)(struct Cyc_Dict_Dict d,void*k,unsigned v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,void*k,unsigned v))Cyc_Dict_insert;_tmp51;});struct Cyc_Dict_Dict _tmpD9=*N;void*_tmpD8=x;_tmpDA(_tmpD9,_tmpD8,Ny);});*N=_tmpDB;});}}}
# 425
if(({unsigned _tmpDE=({unsigned(*_tmpDD)(struct Cyc_Dict_Dict d,void*k)=({unsigned(*_tmp53)(struct Cyc_Dict_Dict d,void*k)=(unsigned(*)(struct Cyc_Dict_Dict d,void*k))Cyc_Dict_lookup;_tmp53;});struct Cyc_Dict_Dict _tmpDC=*N;_tmpDD(_tmpDC,x);});_tmpDE == k;})){
int(*cmp)(void*,void*)=input.rel;
struct Cyc_Set_Set*component=Cyc_Set_empty(cmp);
{struct Cyc_List_List*s=*S;for(0;(unsigned)s;s=s->tl){
void*top=s->hd;
component=Cyc_Set_insert(component,top);
if(cmp(top,x)== 0)break;}}
# 433
for(0;(unsigned)*S;*S=((struct Cyc_List_List*)_check_null(*S))->tl){
void*top=((struct Cyc_List_List*)_check_null(*S))->hd;
({struct Cyc_Dict_Dict _tmpE1=({struct Cyc_Dict_Dict(*_tmpE0)(struct Cyc_Dict_Dict d,void*k,unsigned v)=({struct Cyc_Dict_Dict(*_tmp54)(struct Cyc_Dict_Dict d,void*k,unsigned v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,void*k,unsigned v))Cyc_Dict_insert;_tmp54;});struct Cyc_Dict_Dict _tmpDF=*N;_tmpE0(_tmpDF,top,- 1U);});*N=_tmpE1;});
({struct Cyc_List_List*_tmpE2=({struct Cyc_List_List*_tmp55=_cycalloc(sizeof(*_tmp55));_tmp55->hd=top,_tmp55->tl=*output;_tmp55;});*output=_tmpE2;});
if(cmp(top,x)== 0){
*S=((struct Cyc_List_List*)_check_null(*S))->tl;
break;}}}}
# 238 "graph.cyc"
struct Cyc_List_List*Cyc_Graph_tsort(struct Cyc_Dict_Dict input){
# 445 "graph.cyc"
int(*cmp)(void*,void*)=input.rel;
struct Cyc_List_List*edges=Cyc_Dict_to_list(input);
struct Cyc_List_List*output=0;
struct Cyc_List_List*S=0;
struct Cyc_Dict_Dict N=Cyc_Dict_empty(cmp);
{struct Cyc_List_List*e=edges;for(0;(unsigned)e;e=e->tl){
struct _tuple0*_stmttmpC=(struct _tuple0*)e->hd;void*_tmp56;_LL1: _tmp56=_stmttmpC->f1;_LL2: {void*x=_tmp56;
N=({struct Cyc_Dict_Dict(*_tmpE4)(struct Cyc_Dict_Dict d,void*k,unsigned v)=({struct Cyc_Dict_Dict(*_tmp57)(struct Cyc_Dict_Dict d,void*k,unsigned v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,void*k,unsigned v))Cyc_Dict_insert;_tmp57;});struct Cyc_Dict_Dict _tmpE3=N;_tmpE4(_tmpE3,x,0U);});}}}
# 454
{struct Cyc_List_List*e=edges;for(0;(unsigned)e;e=e->tl){
struct _tuple0*_stmttmpD=(struct _tuple0*)e->hd;void*_tmp58;_LL4: _tmp58=_stmttmpD->f1;_LL5: {void*x=_tmp58;
if(({unsigned(*_tmpE6)(struct Cyc_Dict_Dict d,void*k)=({unsigned(*_tmp59)(struct Cyc_Dict_Dict d,void*k)=(unsigned(*)(struct Cyc_Dict_Dict d,void*k))Cyc_Dict_lookup;_tmp59;});struct Cyc_Dict_Dict _tmpE5=N;_tmpE6(_tmpE5,x);})== (unsigned)0)
Cyc_Graph_tsort0(input,& output,& N,& S,x,1U);}}}
# 459
return output;}
