#include <setjmp.h>
/* This is a C header file to be used by the output of the Cyclone to
   C translator.  The corresponding definitions are in file lib/runtime_*.c */
#ifndef _CYC_INCLUDE_H_
#define _CYC_INCLUDE_H_

/* Need one of these per thread (see runtime_stack.c). The runtime maintains 
   a stack that contains either _handler_cons structs or _RegionHandle structs.
   The tag is 0 for a handler_cons and 1 for a region handle.  */
struct _RuntimeStack {
  int tag; 
  struct _RuntimeStack *next;
  void (*cleanup)(struct _RuntimeStack *frame);
};

#ifndef offsetof
/* should be size_t, but int is fine. */
#define offsetof(t,n) ((int)(&(((t *)0)->n)))
#endif

/* Fat pointers */
struct _fat_ptr {
  unsigned char *curr; 
  unsigned char *base; 
  unsigned char *last_plus_one; 
};  

/* Discriminated Unions */
struct _xtunion_struct { char *tag; };

/* Regions */
struct _RegionPage
#ifdef CYC_REGION_PROFILE
{ unsigned total_bytes;
  unsigned free_bytes;
  /* MWH: wish we didn't have to include the stuff below ... */
  struct _RegionPage *next;
  char data[1];
}
#endif
; // abstract -- defined in runtime_memory.c
struct _RegionHandle {
  struct _RuntimeStack s;
  struct _RegionPage *curr;
  char               *offset;
  char               *last_plus_one;
  struct _DynRegionHandle *sub_regions;
#ifdef CYC_REGION_PROFILE
  const char         *name;
#else
  unsigned used_bytes;
  unsigned wasted_bytes;
#endif
};
struct _DynRegionFrame {
  struct _RuntimeStack s;
  struct _DynRegionHandle *x;
};
// A dynamic region is just a region handle.  The wrapper struct is for type
// abstraction.
struct Cyc_Core_DynamicRegion {
  struct _RegionHandle h;
};

struct _RegionHandle _new_region(const char*);
void* _region_malloc(struct _RegionHandle*, unsigned);
void* _region_calloc(struct _RegionHandle*, unsigned t, unsigned n);
void   _free_region(struct _RegionHandle*);
struct _RegionHandle*_open_dynregion(struct _DynRegionFrame*,struct _DynRegionHandle*);
void   _pop_dynregion();

/* Exceptions */
struct _handler_cons {
  struct _RuntimeStack s;
  jmp_buf handler;
};
void _push_handler(struct _handler_cons *);
void _push_region(struct _RegionHandle *);
void _npop_handler(int);
void _pop_handler();
void _pop_region();

#ifndef _throw
void* _throw_null_fn(const char*,unsigned);
void* _throw_arraybounds_fn(const char*,unsigned);
void* _throw_badalloc_fn(const char*,unsigned);
void* _throw_match_fn(const char*,unsigned);
void* _throw_fn(void*,const char*,unsigned);
void* _rethrow(void*);
#define _throw_null() (_throw_null_fn(__FILE__,__LINE__))
#define _throw_arraybounds() (_throw_arraybounds_fn(__FILE__,__LINE__))
#define _throw_badalloc() (_throw_badalloc_fn(__FILE__,__LINE__))
#define _throw_match() (_throw_match_fn(__FILE__,__LINE__))
#define _throw(e) (_throw_fn((e),__FILE__,__LINE__))
#endif

struct _xtunion_struct* Cyc_Core_get_exn_thrown();
/* Built-in Exceptions */
struct Cyc_Null_Exception_exn_struct { char *tag; };
struct Cyc_Array_bounds_exn_struct { char *tag; };
struct Cyc_Match_Exception_exn_struct { char *tag; };
struct Cyc_Bad_alloc_exn_struct { char *tag; };
extern char Cyc_Null_Exception[];
extern char Cyc_Array_bounds[];
extern char Cyc_Match_Exception[];
extern char Cyc_Bad_alloc[];

/* Built-in Run-time Checks and company */
#ifdef CYC_ANSI_OUTPUT
#define _INLINE  
#else
#define _INLINE inline
#endif

#ifdef NO_CYC_NULL_CHECKS
#define _check_null(ptr) (ptr)
#else
#define _check_null(ptr) \
  ({ void*_cks_null = (void*)(ptr); \
     if (!_cks_null) _throw_null(); \
     _cks_null; })
#endif

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index)\
   (((char*)ptr) + (elt_sz)*(index))
#ifdef NO_CYC_NULL_CHECKS
#define _check_known_subscript_null _check_known_subscript_notnull
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr);\
  int _index = (index);\
  if (!_cks_ptr) _throw_null(); \
  _cks_ptr + (elt_sz)*_index; })
#endif
#define _zero_arr_plus_fn(orig_x,orig_sz,orig_i,f,l) ((orig_x)+(orig_i))
#define _zero_arr_plus_char_fn _zero_arr_plus_fn
#define _zero_arr_plus_short_fn _zero_arr_plus_fn
#define _zero_arr_plus_int_fn _zero_arr_plus_fn
#define _zero_arr_plus_float_fn _zero_arr_plus_fn
#define _zero_arr_plus_double_fn _zero_arr_plus_fn
#define _zero_arr_plus_longdouble_fn _zero_arr_plus_fn
#define _zero_arr_plus_voidstar_fn _zero_arr_plus_fn
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (!_cks_ptr) _throw_null(); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })

/* _zero_arr_plus_*_fn(x,sz,i,filename,lineno) adds i to zero-terminated ptr
   x that has at least sz elements */
char* _zero_arr_plus_char_fn(char*,unsigned,int,const char*,unsigned);
short* _zero_arr_plus_short_fn(short*,unsigned,int,const char*,unsigned);
int* _zero_arr_plus_int_fn(int*,unsigned,int,const char*,unsigned);
float* _zero_arr_plus_float_fn(float*,unsigned,int,const char*,unsigned);
double* _zero_arr_plus_double_fn(double*,unsigned,int,const char*,unsigned);
long double* _zero_arr_plus_longdouble_fn(long double*,unsigned,int,const char*, unsigned);
void** _zero_arr_plus_voidstar_fn(void**,unsigned,int,const char*,unsigned);
#endif

/* _get_zero_arr_size_*(x,sz) returns the number of elements in a
   zero-terminated array that is NULL or has at least sz elements */
int _get_zero_arr_size_char(const char*,unsigned);
int _get_zero_arr_size_short(const short*,unsigned);
int _get_zero_arr_size_int(const int*,unsigned);
int _get_zero_arr_size_float(const float*,unsigned);
int _get_zero_arr_size_double(const double*,unsigned);
int _get_zero_arr_size_longdouble(const long double*,unsigned);
int _get_zero_arr_size_voidstar(const void**,unsigned);

/* _zero_arr_inplace_plus_*_fn(x,i,filename,lineno) sets
   zero-terminated pointer *x to *x + i */
char* _zero_arr_inplace_plus_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_short_fn(short**,int,const char*,unsigned);
int* _zero_arr_inplace_plus_int(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_longdouble_fn(long double**,int,const char*,unsigned);
void** _zero_arr_inplace_plus_voidstar_fn(void***,int,const char*,unsigned);
/* like the previous functions, but does post-addition (as in e++) */
char* _zero_arr_inplace_plus_post_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_post_short_fn(short**x,int,const char*,unsigned);
int* _zero_arr_inplace_plus_post_int_fn(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_post_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_post_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_post_longdouble_fn(long double**,int,const char *,unsigned);
void** _zero_arr_inplace_plus_post_voidstar_fn(void***,int,const char*,unsigned);
#define _zero_arr_plus_char(x,s,i) \
  (_zero_arr_plus_char_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_short(x,s,i) \
  (_zero_arr_plus_short_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_int(x,s,i) \
  (_zero_arr_plus_int_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_float(x,s,i) \
  (_zero_arr_plus_float_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_double(x,s,i) \
  (_zero_arr_plus_double_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_longdouble(x,s,i) \
  (_zero_arr_plus_longdouble_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_voidstar(x,s,i) \
  (_zero_arr_plus_voidstar_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_inplace_plus_char(x,i) \
  _zero_arr_inplace_plus_char_fn((char **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_short(x,i) \
  _zero_arr_inplace_plus_short_fn((short **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_int(x,i) \
  _zero_arr_inplace_plus_int_fn((int **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_float(x,i) \
  _zero_arr_inplace_plus_float_fn((float **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_double(x,i) \
  _zero_arr_inplace_plus_double_fn((double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_longdouble(x,i) \
  _zero_arr_inplace_plus_longdouble_fn((long double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_voidstar(x,i) \
  _zero_arr_inplace_plus_voidstar_fn((void ***)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_char(x,i) \
  _zero_arr_inplace_plus_post_char_fn((char **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_short(x,i) \
  _zero_arr_inplace_plus_post_short_fn((short **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_int(x,i) \
  _zero_arr_inplace_plus_post_int_fn((int **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_float(x,i) \
  _zero_arr_inplace_plus_post_float_fn((float **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_double(x,i) \
  _zero_arr_inplace_plus_post_double_fn((double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_longdouble(x,i) \
  _zero_arr_inplace_plus_post_longdouble_fn((long double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_voidstar(x,i) \
  _zero_arr_inplace_plus_post_voidstar_fn((void***)(x),(i),__FILE__,__LINE__)

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_fat_subscript(arr,elt_sz,index) ((arr).curr + (elt_sz) * (index))
#define _untag_fat_ptr(arr,elt_sz,num_elts) ((arr).curr)
#else
#define _check_fat_subscript(arr,elt_sz,index) ({ \
  struct _fat_ptr _cus_arr = (arr); \
  unsigned char *_cus_ans = _cus_arr.curr + (elt_sz) * (index); \
  /* JGM: not needed! if (!_cus_arr.base) _throw_null();*/ \
  if (_cus_ans < _cus_arr.base || _cus_ans >= _cus_arr.last_plus_one) \
    _throw_arraybounds(); \
  _cus_ans; })
#define _untag_fat_ptr(arr,elt_sz,num_elts) ({ \
  struct _fat_ptr _arr = (arr); \
  unsigned char *_curr = _arr.curr; \
  if ((_curr < _arr.base || _curr + (elt_sz) * (num_elts) > _arr.last_plus_one) &&\
      _curr != (unsigned char *)0) \
    _throw_arraybounds(); \
  _curr; })
#endif

#define _tag_fat(tcurr,elt_sz,num_elts) ({ \
  struct _fat_ptr _ans; \
  unsigned _num_elts = (num_elts);\
  _ans.base = _ans.curr = (void*)(tcurr); \
  /* JGM: if we're tagging NULL, ignore num_elts */ \
  _ans.last_plus_one = _ans.base ? (_ans.base + (elt_sz) * _num_elts) : 0; \
  _ans; })

#define _get_fat_size(arr,elt_sz) \
  ({struct _fat_ptr _arr = (arr); \
    unsigned char *_arr_curr=_arr.curr; \
    unsigned char *_arr_last=_arr.last_plus_one; \
    (_arr_curr < _arr.base || _arr_curr >= _arr_last) ? 0 : \
    ((_arr_last - _arr_curr) / (elt_sz));})

#define _fat_ptr_plus(arr,elt_sz,change) ({ \
  struct _fat_ptr _ans = (arr); \
  int _change = (change);\
  _ans.curr += (elt_sz) * _change;\
  _ans; })
#define _fat_ptr_inplace_plus(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  _arr_ptr->curr += (elt_sz) * (change);\
  *_arr_ptr; })
#define _fat_ptr_inplace_plus_post(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  struct _fat_ptr _ans = *_arr_ptr; \
  _arr_ptr->curr += (elt_sz) * (change);\
  _ans; })

//Not a macro since initialization order matters. Defined in runtime_zeroterm.c.
struct _fat_ptr _fat_ptr_decrease_size(struct _fat_ptr,unsigned sz,unsigned numelts);

/* Allocation */
void* GC_malloc(int);
void* GC_malloc_atomic(int);
void* GC_calloc(unsigned,unsigned);
void* GC_calloc_atomic(unsigned,unsigned);
// bound the allocation size to be < MAX_ALLOC_SIZE. See macros below for usage.
#define MAX_MALLOC_SIZE (1 << 28)
void* _bounded_GC_malloc(int,const char*,int);
void* _bounded_GC_malloc_atomic(int,const char*,int);
void* _bounded_GC_calloc(unsigned,unsigned,const char*,int);
void* _bounded_GC_calloc_atomic(unsigned,unsigned,const char*,int);
/* these macros are overridden below ifdef CYC_REGION_PROFILE */
#ifndef CYC_REGION_PROFILE
#define _cycalloc(n) _bounded_GC_malloc(n,__FILE__,__LINE__)
#define _cycalloc_atomic(n) _bounded_GC_malloc_atomic(n,__FILE__,__LINE__)
#define _cyccalloc(n,s) _bounded_GC_calloc(n,s,__FILE__,__LINE__)
#define _cyccalloc_atomic(n,s) _bounded_GC_calloc_atomic(n,s,__FILE__,__LINE__)
#endif

static _INLINE unsigned int _check_times(unsigned x, unsigned y) {
  unsigned long long whole_ans = 
    ((unsigned long long) x)*((unsigned long long)y);
  unsigned word_ans = (unsigned)whole_ans;
  if(word_ans < whole_ans || word_ans > MAX_MALLOC_SIZE)
    _throw_badalloc();
  return word_ans;
}

#define _CYC_MAX_REGION_CONST 2
#define _CYC_MIN_ALIGNMENT (sizeof(double))

#ifdef CYC_REGION_PROFILE
extern int rgn_total_bytes;
#endif

static _INLINE void *_fast_region_malloc(struct _RegionHandle *r, unsigned orig_s) {  
  if (r > (struct _RegionHandle *)_CYC_MAX_REGION_CONST && r->curr != 0) { 
#ifdef CYC_NOALIGN
    unsigned s =  orig_s;
#else
    unsigned s =  (orig_s + _CYC_MIN_ALIGNMENT - 1) & (~(_CYC_MIN_ALIGNMENT -1)); 
#endif
    char *result; 
    result = r->offset; 
    if (s <= (r->last_plus_one - result)) {
      r->offset = result + s; 
#ifdef CYC_REGION_PROFILE
    r->curr->free_bytes = r->curr->free_bytes - s;
    rgn_total_bytes += s;
#endif
      return result;
    }
  } 
  return _region_malloc(r,orig_s); 
}

#ifdef CYC_REGION_PROFILE
/* see macros below for usage. defined in runtime_memory.c */
void* _profile_GC_malloc(int,const char*,const char*,int);
void* _profile_GC_malloc_atomic(int,const char*,const char*,int);
void* _profile_GC_calloc(unsigned,unsigned,const char*,const char*,int);
void* _profile_GC_calloc_atomic(unsigned,unsigned,const char*,const char*,int);
void* _profile_region_malloc(struct _RegionHandle*,unsigned,const char*,const char*,int);
void* _profile_region_calloc(struct _RegionHandle*,unsigned,unsigned,const char *,const char*,int);
struct _RegionHandle _profile_new_region(const char*,const char*,const char*,int);
void _profile_free_region(struct _RegionHandle*,const char*,const char*,int);
#ifndef RUNTIME_CYC
#define _new_region(n) _profile_new_region(n,__FILE__,__FUNCTION__,__LINE__)
#define _free_region(r) _profile_free_region(r,__FILE__,__FUNCTION__,__LINE__)
#define _region_malloc(rh,n) _profile_region_malloc(rh,n,__FILE__,__FUNCTION__,__LINE__)
#define _region_calloc(rh,n,t) _profile_region_calloc(rh,n,t,__FILE__,__FUNCTION__,__LINE__)
#  endif
#define _cycalloc(n) _profile_GC_malloc(n,__FILE__,__FUNCTION__,__LINE__)
#define _cycalloc_atomic(n) _profile_GC_malloc_atomic(n,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc(n,s) _profile_GC_calloc(n,s,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc_atomic(n,s) _profile_GC_calloc_atomic(n,s,__FILE__,__FUNCTION__,__LINE__)
#endif
#endif
 struct Cyc_Core_Opt{void*v;};
# 95 "core.h"
struct _fat_ptr Cyc_Core_new_string(unsigned);
# 117
void*Cyc_Core_identity(void*);
# 119
int Cyc_Core_intcmp(int,int);extern char Cyc_Core_Invalid_argument[17U];struct Cyc_Core_Invalid_argument_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Failure[8U];struct Cyc_Core_Failure_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Impossible[11U];struct Cyc_Core_Impossible_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Not_found[10U];struct Cyc_Core_Not_found_exn_struct{char*tag;};extern char Cyc_Core_Unreachable[12U];struct Cyc_Core_Unreachable_exn_struct{char*tag;struct _fat_ptr f1;};
# 168
extern struct _RegionHandle*Cyc_Core_unique_region;struct Cyc_Core_DynamicRegion;struct Cyc_Core_NewDynamicRegion{struct Cyc_Core_DynamicRegion*key;};struct Cyc_Core_ThinRes{void*arr;unsigned nelts;};struct Cyc___cycFILE;struct Cyc_String_pa_PrintArg_struct{int tag;struct _fat_ptr f1;};struct Cyc_Int_pa_PrintArg_struct{int tag;unsigned long f1;};struct Cyc_Double_pa_PrintArg_struct{int tag;double f1;};struct Cyc_LongDouble_pa_PrintArg_struct{int tag;long double f1;};struct Cyc_ShortPtr_pa_PrintArg_struct{int tag;short*f1;};struct Cyc_IntPtr_pa_PrintArg_struct{int tag;unsigned long*f1;};
# 100 "cycboot.h"
int Cyc_fprintf(struct Cyc___cycFILE*,struct _fat_ptr,struct _fat_ptr);struct Cyc_ShortPtr_sa_ScanfArg_struct{int tag;short*f1;};struct Cyc_UShortPtr_sa_ScanfArg_struct{int tag;unsigned short*f1;};struct Cyc_IntPtr_sa_ScanfArg_struct{int tag;int*f1;};struct Cyc_UIntPtr_sa_ScanfArg_struct{int tag;unsigned*f1;};struct Cyc_StringPtr_sa_ScanfArg_struct{int tag;struct _fat_ptr f1;};struct Cyc_DoublePtr_sa_ScanfArg_struct{int tag;double*f1;};struct Cyc_FloatPtr_sa_ScanfArg_struct{int tag;float*f1;};struct Cyc_CharPtr_sa_ScanfArg_struct{int tag;struct _fat_ptr f1;};extern char Cyc_FileCloseError[15U];struct Cyc_FileCloseError_exn_struct{char*tag;};extern char Cyc_FileOpenError[14U];struct Cyc_FileOpenError_exn_struct{char*tag;struct _fat_ptr f1;};struct Cyc_List_List{void*hd;struct Cyc_List_List*tl;};
# 61 "list.h"
int Cyc_List_length(struct Cyc_List_List*x);extern char Cyc_List_List_mismatch[14U];struct Cyc_List_List_mismatch_exn_struct{char*tag;};extern char Cyc_List_Nth[4U];struct Cyc_List_Nth_exn_struct{char*tag;};
# 38 "string.h"
unsigned long Cyc_strlen(struct _fat_ptr s);
# 54 "string.h"
int Cyc_zstrptrcmp(struct _fat_ptr*,struct _fat_ptr*);
# 66
struct _fat_ptr Cyc_str_sepstr(struct Cyc_List_List*,struct _fat_ptr);struct Cyc_Hashtable_Table;
# 39 "hashtable.h"
struct Cyc_Hashtable_Table*Cyc_Hashtable_create(int sz,int(*cmp)(void*,void*),int(*hash)(void*));
# 50
void Cyc_Hashtable_insert(struct Cyc_Hashtable_Table*t,void*key,void*val);
# 52
void*Cyc_Hashtable_lookup(struct Cyc_Hashtable_Table*t,void*key);
# 78
int Cyc_Hashtable_hash_stringptr(struct _fat_ptr*p);struct Cyc_Fn_Function{void*(*f)(void*,void*);void*env;};
# 48 "fn.h"
struct Cyc_Fn_Function*Cyc_Fn_make_fn(void*(*f)(void*,void*),void*x);
# 51
struct Cyc_Fn_Function*Cyc_Fn_fp2fn(void*(*f)(void*));
# 54
void*Cyc_Fn_apply(struct Cyc_Fn_Function*f,void*x);
# 39 "pp.h"
extern int Cyc_PP_tex_output;struct Cyc_PP_Ppstate;struct Cyc_PP_Out;struct Cyc_PP_Doc;
# 67 "pp.h"
struct Cyc_PP_Doc*Cyc_PP_nil_doc();
# 72
struct Cyc_PP_Doc*Cyc_PP_line_doc();
# 74
struct Cyc_PP_Doc*Cyc_PP_oline_doc();
# 78
struct Cyc_PP_Doc*Cyc_PP_text(struct _fat_ptr s);
# 94
struct Cyc_PP_Doc*Cyc_PP_cat(struct _fat_ptr);
# 97
struct Cyc_PP_Doc*Cyc_PP_cats(struct Cyc_List_List*doclist);
# 103
struct Cyc_PP_Doc*Cyc_PP_doc_union(struct Cyc_PP_Doc*d1,struct Cyc_PP_Doc*d2);
# 105
struct Cyc_PP_Doc*Cyc_PP_tab(struct Cyc_PP_Doc*d);
# 108
struct Cyc_PP_Doc*Cyc_PP_seq(struct _fat_ptr sep,struct Cyc_List_List*l);
# 117
struct Cyc_PP_Doc*Cyc_PP_seql(struct _fat_ptr sep,struct Cyc_List_List*l0);struct Cyc_Xarray_Xarray{struct _fat_ptr elmts;int num_elmts;};
# 40 "xarray.h"
int Cyc_Xarray_length(struct Cyc_Xarray_Xarray*);
# 42
void*Cyc_Xarray_get(struct Cyc_Xarray_Xarray*,int);
# 45
void Cyc_Xarray_set(struct Cyc_Xarray_Xarray*,int,void*);
# 48
struct Cyc_Xarray_Xarray*Cyc_Xarray_create(int,void*);
# 66
void Cyc_Xarray_add(struct Cyc_Xarray_Xarray*,void*);
# 121
void Cyc_Xarray_reuse(struct Cyc_Xarray_Xarray*xarr);struct Cyc_PP_Empty_PP_Alist_struct{int tag;int f1;};struct Cyc_PP_Single_PP_Alist_struct{int tag;void*f1;};struct Cyc_PP_Append_PP_Alist_struct{int tag;void*f1;void*f2;};
# 51 "pp.cyc"
struct Cyc_PP_Empty_PP_Alist_struct Cyc_PP_Empty_stringptr={0U,0};
struct Cyc_PP_Empty_PP_Alist_struct Cyc_PP_Empty_link={0U,0};struct _tuple0{void*f1;void*f2;};
# 54
void*Cyc_PP_append(void*a1,void*a2){
struct _tuple0 _stmttmp0=({struct _tuple0 _tmpAD;_tmpAD.f1=a1,_tmpAD.f2=a2;_tmpAD;});struct _tuple0 _tmp0=_stmttmp0;if(((struct Cyc_PP_Empty_PP_Alist_struct*)_tmp0.f1)->tag == 0U){_LL1: _LL2:
 return a2;}else{if(((struct Cyc_PP_Empty_PP_Alist_struct*)_tmp0.f2)->tag == 0U){_LL3: _LL4:
 return a1;}else{_LL5: _LL6:
 return(void*)({struct Cyc_PP_Append_PP_Alist_struct*_tmp1=_cycalloc(sizeof(*_tmp1));_tmp1->tag=2U,_tmp1->f1=a1,_tmp1->f2=a2;_tmp1;});}}_LL0:;}
# 54
struct Cyc_List_List*Cyc_PP_list_of_alist_f(void*y,struct Cyc_List_List*l){
# 63
void*_tmp2=y;void*_tmp4;void*_tmp3;void*_tmp5;switch(*((int*)_tmp2)){case 0U: _LL1: _LL2:
 return l;case 1U: _LL3: _tmp5=(void*)((struct Cyc_PP_Single_PP_Alist_struct*)_tmp2)->f1;_LL4: {void*z=_tmp5;
return({struct Cyc_List_List*_tmp6=_cycalloc(sizeof(*_tmp6));_tmp6->hd=z,_tmp6->tl=l;_tmp6;});}default: _LL5: _tmp3=(void*)((struct Cyc_PP_Append_PP_Alist_struct*)_tmp2)->f1;_tmp4=(void*)((struct Cyc_PP_Append_PP_Alist_struct*)_tmp2)->f2;_LL6: {void*a1=_tmp3;void*a2=_tmp4;
return({void*_tmpAF=a1;Cyc_PP_list_of_alist_f(_tmpAF,Cyc_PP_list_of_alist_f(a2,l));});}}_LL0:;}
# 54
struct Cyc_List_List*Cyc_PP_list_of_alist(void*x){
# 70
return Cyc_PP_list_of_alist_f(x,0);}struct Cyc_PP_Ppstate{int ci;int cc;int cl;int pw;int epw;};struct Cyc_PP_Out{int newcc;int newcl;void*ppout;void*links;};struct Cyc_PP_Doc{int mwo;int mw;struct Cyc_Fn_Function*f;};
# 98
static void Cyc_PP_dump_out(struct Cyc___cycFILE*f,void*al){
struct Cyc_Xarray_Xarray*xarr=Cyc_Xarray_create(16,al);
Cyc_Xarray_add(xarr,al);{
int last=0;
while(last >= 0){
void*_stmttmp1=Cyc_Xarray_get(xarr,last);void*_tmp7=_stmttmp1;void*_tmp9;void*_tmp8;struct _fat_ptr*_tmpA;switch(*((int*)_tmp7)){case 0U: _LL1: _LL2:
 -- last;goto _LL0;case 1U: _LL3: _tmpA=(struct _fat_ptr*)((struct Cyc_PP_Single_PP_Alist_struct*)_tmp7)->f1;_LL4: {struct _fat_ptr*s=_tmpA;
-- last;({struct Cyc_String_pa_PrintArg_struct _tmpD=({struct Cyc_String_pa_PrintArg_struct _tmpAE;_tmpAE.tag=0U,_tmpAE.f1=(struct _fat_ptr)((struct _fat_ptr)*s);_tmpAE;});void*_tmpB[1U];_tmpB[0]=& _tmpD;({struct Cyc___cycFILE*_tmpB1=f;struct _fat_ptr _tmpB0=({const char*_tmpC="%s";_tag_fat(_tmpC,sizeof(char),3U);});Cyc_fprintf(_tmpB1,_tmpB0,_tag_fat(_tmpB,sizeof(void*),1U));});});goto _LL0;}default: _LL5: _tmp8=(void*)((struct Cyc_PP_Append_PP_Alist_struct*)_tmp7)->f1;_tmp9=(void*)((struct Cyc_PP_Append_PP_Alist_struct*)_tmp7)->f2;_LL6: {void*a1=_tmp8;void*a2=_tmp9;
# 107
Cyc_Xarray_set(xarr,last,a2);
if(({int _tmpB2=last;_tmpB2 == Cyc_Xarray_length(xarr)- 1;}))
Cyc_Xarray_add(xarr,a1);else{
# 111
Cyc_Xarray_set(xarr,last + 1,a1);}
++ last;
goto _LL0;}}_LL0:;}
# 116
Cyc_Xarray_reuse(xarr);}}
# 98
void Cyc_PP_file_of_doc(struct Cyc_PP_Doc*d,int w,struct Cyc___cycFILE*f){
# 121
struct Cyc_PP_Out*o=({struct Cyc_PP_Out*(*_tmpB4)(struct Cyc_Fn_Function*f,struct Cyc_PP_Ppstate*x)=({struct Cyc_PP_Out*(*_tmpE)(struct Cyc_Fn_Function*f,struct Cyc_PP_Ppstate*x)=(struct Cyc_PP_Out*(*)(struct Cyc_Fn_Function*f,struct Cyc_PP_Ppstate*x))Cyc_Fn_apply;_tmpE;});struct Cyc_Fn_Function*_tmpB3=d->f;_tmpB4(_tmpB3,({struct Cyc_PP_Ppstate*_tmpF=_cycalloc(sizeof(*_tmpF));_tmpF->ci=0,_tmpF->cc=0,_tmpF->cl=1,_tmpF->pw=w,_tmpF->epw=w;_tmpF;}));});
Cyc_PP_dump_out(f,o->ppout);}
# 98
struct _fat_ptr Cyc_PP_string_of_doc(struct Cyc_PP_Doc*d,int w){
# 127
struct Cyc_PP_Out*o=({struct Cyc_PP_Out*(*_tmpB6)(struct Cyc_Fn_Function*f,struct Cyc_PP_Ppstate*x)=({struct Cyc_PP_Out*(*_tmp11)(struct Cyc_Fn_Function*f,struct Cyc_PP_Ppstate*x)=(struct Cyc_PP_Out*(*)(struct Cyc_Fn_Function*f,struct Cyc_PP_Ppstate*x))Cyc_Fn_apply;_tmp11;});struct Cyc_Fn_Function*_tmpB5=d->f;_tmpB6(_tmpB5,({struct Cyc_PP_Ppstate*_tmp12=_cycalloc(sizeof(*_tmp12));_tmp12->ci=0,_tmp12->cc=0,_tmp12->cl=1,_tmp12->pw=w,_tmp12->epw=w;_tmp12;}));});
return(struct _fat_ptr)({struct Cyc_List_List*_tmpB7=Cyc_PP_list_of_alist(o->ppout);Cyc_str_sepstr(_tmpB7,({const char*_tmp10="";_tag_fat(_tmp10,sizeof(char),1U);}));});}struct _tuple1{struct _fat_ptr f1;struct Cyc_List_List*f2;};
# 98
struct _tuple1*Cyc_PP_string_and_links(struct Cyc_PP_Doc*d,int w){
# 134
struct Cyc_PP_Out*o=({struct Cyc_PP_Out*(*_tmpB9)(struct Cyc_Fn_Function*f,struct Cyc_PP_Ppstate*x)=({struct Cyc_PP_Out*(*_tmp15)(struct Cyc_Fn_Function*f,struct Cyc_PP_Ppstate*x)=(struct Cyc_PP_Out*(*)(struct Cyc_Fn_Function*f,struct Cyc_PP_Ppstate*x))Cyc_Fn_apply;_tmp15;});struct Cyc_Fn_Function*_tmpB8=d->f;_tmpB9(_tmpB8,({struct Cyc_PP_Ppstate*_tmp16=_cycalloc(sizeof(*_tmp16));_tmp16->ci=0,_tmp16->cc=0,_tmp16->cl=1,_tmp16->pw=w,_tmp16->epw=w;_tmp16;}));});
return({struct _tuple1*_tmp14=_cycalloc(sizeof(*_tmp14));({struct _fat_ptr _tmpBC=(struct _fat_ptr)({
struct Cyc_List_List*_tmpBB=Cyc_PP_list_of_alist(o->ppout);Cyc_str_sepstr(_tmpBB,({const char*_tmp13="";_tag_fat(_tmp13,sizeof(char),1U);}));});
# 135
_tmp14->f1=_tmpBC;}),({
# 137
struct Cyc_List_List*_tmpBA=Cyc_PP_list_of_alist(o->links);_tmp14->f2=_tmpBA;});_tmp14;});}
# 98
static struct Cyc_Core_Opt*Cyc_PP_bhashtbl=0;
# 142
int Cyc_PP_tex_output=0;
# 144
struct _fat_ptr Cyc_PP_nlblanks(int i){
if(Cyc_PP_bhashtbl == 0)
Cyc_PP_bhashtbl=({struct Cyc_Core_Opt*_tmp19=_cycalloc(sizeof(*_tmp19));({struct Cyc_Hashtable_Table*_tmpBE=({struct Cyc_Hashtable_Table*(*_tmpBD)(int sz,int(*cmp)(int,int),int(*hash)(int))=({struct Cyc_Hashtable_Table*(*_tmp17)(int sz,int(*cmp)(int,int),int(*hash)(int))=(struct Cyc_Hashtable_Table*(*)(int sz,int(*cmp)(int,int),int(*hash)(int)))Cyc_Hashtable_create;_tmp17;});_tmpBD(61,Cyc_Core_intcmp,({int(*_tmp18)(int)=(int(*)(int))Cyc_Core_identity;_tmp18;}));});_tmp19->v=_tmpBE;});_tmp19;});
# 145
if(i < 0)
# 147
(int)_throw((void*)({struct Cyc_Core_Failure_exn_struct*_tmp1B=_cycalloc(sizeof(*_tmp1B));_tmp1B->tag=Cyc_Core_Failure,({struct _fat_ptr _tmpBF=({const char*_tmp1A="nlblanks";_tag_fat(_tmp1A,sizeof(char),9U);});_tmp1B->f1=_tmpBF;});_tmp1B;}));{
# 145
struct _handler_cons _tmp1C;_push_handler(& _tmp1C);{int _tmp1E=0;if(setjmp(_tmp1C.handler))_tmp1E=1;if(!_tmp1E){
# 149
{struct _fat_ptr _tmp20=*({struct _fat_ptr*(*_tmpC1)(struct Cyc_Hashtable_Table*t,int key)=({struct _fat_ptr*(*_tmp1F)(struct Cyc_Hashtable_Table*t,int key)=(struct _fat_ptr*(*)(struct Cyc_Hashtable_Table*t,int key))Cyc_Hashtable_lookup;_tmp1F;});struct Cyc_Hashtable_Table*_tmpC0=(struct Cyc_Hashtable_Table*)((struct Cyc_Core_Opt*)_check_null(Cyc_PP_bhashtbl))->v;_tmpC1(_tmpC0,i);});_npop_handler(0U);return _tmp20;};_pop_handler();}else{void*_tmp1D=(void*)Cyc_Core_get_exn_thrown();void*_tmp21=_tmp1D;void*_tmp22;if(((struct Cyc_Core_Not_found_exn_struct*)_tmp21)->tag == Cyc_Core_Not_found){_LL1: _LL2:
# 152
 if(!Cyc_PP_tex_output){
int num_tabs=i / 8;
int num_spaces=i % 8;
int total=(2 + num_tabs)+ num_spaces;
struct _fat_ptr nlb=Cyc_Core_new_string((unsigned)total);
({struct _fat_ptr _tmp23=_fat_ptr_plus(nlb,sizeof(char),0);char _tmp24=*((char*)_check_fat_subscript(_tmp23,sizeof(char),0U));char _tmp25='\n';if(_get_fat_size(_tmp23,sizeof(char))== 1U &&(_tmp24 == 0 && _tmp25 != 0))_throw_arraybounds();*((char*)_tmp23.curr)=_tmp25;});
{int j=0;for(0;j < num_tabs;++ j){
({struct _fat_ptr _tmp26=_fat_ptr_plus(nlb,sizeof(char),j + 1);char _tmp27=*((char*)_check_fat_subscript(_tmp26,sizeof(char),0U));char _tmp28='\t';if(_get_fat_size(_tmp26,sizeof(char))== 1U &&(_tmp27 == 0 && _tmp28 != 0))_throw_arraybounds();*((char*)_tmp26.curr)=_tmp28;});}}
{int j=0;for(0;j < num_spaces;++ j){
({struct _fat_ptr _tmp29=_fat_ptr_plus(nlb,sizeof(char),(j + 1)+ num_tabs);char _tmp2A=*((char*)_check_fat_subscript(_tmp29,sizeof(char),0U));char _tmp2B=' ';if(_get_fat_size(_tmp29,sizeof(char))== 1U &&(_tmp2A == 0 && _tmp2B != 0))_throw_arraybounds();*((char*)_tmp29.curr)=_tmp2B;});}}
({void(*_tmpC4)(struct Cyc_Hashtable_Table*t,int key,struct _fat_ptr*val)=({void(*_tmp2C)(struct Cyc_Hashtable_Table*t,int key,struct _fat_ptr*val)=(void(*)(struct Cyc_Hashtable_Table*t,int key,struct _fat_ptr*val))Cyc_Hashtable_insert;_tmp2C;});struct Cyc_Hashtable_Table*_tmpC3=(struct Cyc_Hashtable_Table*)((struct Cyc_Core_Opt*)_check_null(Cyc_PP_bhashtbl))->v;int _tmpC2=i;_tmpC4(_tmpC3,_tmpC2,({unsigned _tmp2E=1;struct _fat_ptr*_tmp2D=_cycalloc(_check_times(_tmp2E,sizeof(struct _fat_ptr)));_tmp2D[0]=(struct _fat_ptr)nlb;_tmp2D;}));});
return(struct _fat_ptr)nlb;}else{
# 168
int total=3 + i;
struct _fat_ptr nlb=Cyc_Core_new_string((unsigned)(total + 1));
({struct _fat_ptr _tmp2F=_fat_ptr_plus(nlb,sizeof(char),0);char _tmp30=*((char*)_check_fat_subscript(_tmp2F,sizeof(char),0U));char _tmp31='\\';if(_get_fat_size(_tmp2F,sizeof(char))== 1U &&(_tmp30 == 0 && _tmp31 != 0))_throw_arraybounds();*((char*)_tmp2F.curr)=_tmp31;});
({struct _fat_ptr _tmp32=_fat_ptr_plus(nlb,sizeof(char),1);char _tmp33=*((char*)_check_fat_subscript(_tmp32,sizeof(char),0U));char _tmp34='\\';if(_get_fat_size(_tmp32,sizeof(char))== 1U &&(_tmp33 == 0 && _tmp34 != 0))_throw_arraybounds();*((char*)_tmp32.curr)=_tmp34;});
({struct _fat_ptr _tmp35=_fat_ptr_plus(nlb,sizeof(char),2);char _tmp36=*((char*)_check_fat_subscript(_tmp35,sizeof(char),0U));char _tmp37='\n';if(_get_fat_size(_tmp35,sizeof(char))== 1U &&(_tmp36 == 0 && _tmp37 != 0))_throw_arraybounds();*((char*)_tmp35.curr)=_tmp37;});
{int j=3;for(0;j < total;++ j){
({struct _fat_ptr _tmp38=_fat_ptr_plus(nlb,sizeof(char),j);char _tmp39=*((char*)_check_fat_subscript(_tmp38,sizeof(char),0U));char _tmp3A='~';if(_get_fat_size(_tmp38,sizeof(char))== 1U &&(_tmp39 == 0 && _tmp3A != 0))_throw_arraybounds();*((char*)_tmp38.curr)=_tmp3A;});}}
({void(*_tmpC7)(struct Cyc_Hashtable_Table*t,int key,struct _fat_ptr*val)=({void(*_tmp3B)(struct Cyc_Hashtable_Table*t,int key,struct _fat_ptr*val)=(void(*)(struct Cyc_Hashtable_Table*t,int key,struct _fat_ptr*val))Cyc_Hashtable_insert;_tmp3B;});struct Cyc_Hashtable_Table*_tmpC6=(struct Cyc_Hashtable_Table*)((struct Cyc_Core_Opt*)_check_null(Cyc_PP_bhashtbl))->v;int _tmpC5=i;_tmpC7(_tmpC6,_tmpC5,({unsigned _tmp3D=1;struct _fat_ptr*_tmp3C=_cycalloc(_check_times(_tmp3D,sizeof(struct _fat_ptr)));_tmp3C[0]=(struct _fat_ptr)nlb;_tmp3C;}));});
return(struct _fat_ptr)nlb;}}else{_LL3: _tmp22=_tmp21;_LL4: {void*exn=_tmp22;(int)_rethrow(exn);}}_LL0:;}}}}
# 144
static struct Cyc_Core_Opt*Cyc_PP_str_hashtbl=0;
# 184
int Cyc_PP_infinity=9999999;struct _tuple2{int f1;struct _fat_ptr f2;};
# 186
static struct Cyc_PP_Out*Cyc_PP_text_doc_f(struct _tuple2*clo,struct Cyc_PP_Ppstate*st){
struct _fat_ptr _tmp3F;int _tmp3E;_LL1: _tmp3E=clo->f1;_tmp3F=clo->f2;_LL2: {int slen=_tmp3E;struct _fat_ptr s=_tmp3F;
return({struct Cyc_PP_Out*_tmp43=_cycalloc(sizeof(*_tmp43));_tmp43->newcc=st->cc + slen,_tmp43->newcl=st->cl,({
# 190
void*_tmpC9=(void*)({struct Cyc_PP_Single_PP_Alist_struct*_tmp42=_cycalloc(sizeof(*_tmp42));_tmp42->tag=1U,({struct _fat_ptr*_tmpC8=({unsigned _tmp41=1;struct _fat_ptr*_tmp40=_cycalloc(_check_times(_tmp41,sizeof(struct _fat_ptr)));_tmp40[0]=s;_tmp40;});_tmp42->f1=_tmpC8;});_tmp42;});_tmp43->ppout=_tmpC9;}),_tmp43->links=(void*)& Cyc_PP_Empty_link;_tmp43;});}}
# 186
static struct Cyc_PP_Doc*Cyc_PP_text_doc(struct _fat_ptr s){
# 194
int slen=(int)Cyc_strlen((struct _fat_ptr)s);
return({struct Cyc_PP_Doc*_tmp46=_cycalloc(sizeof(*_tmp46));
_tmp46->mwo=slen,_tmp46->mw=Cyc_PP_infinity,({
# 198
struct Cyc_Fn_Function*_tmpCB=({struct Cyc_Fn_Function*(*_tmpCA)(struct Cyc_PP_Out*(*f)(struct _tuple2*,struct Cyc_PP_Ppstate*),struct _tuple2*x)=({struct Cyc_Fn_Function*(*_tmp44)(struct Cyc_PP_Out*(*f)(struct _tuple2*,struct Cyc_PP_Ppstate*),struct _tuple2*x)=(struct Cyc_Fn_Function*(*)(struct Cyc_PP_Out*(*f)(struct _tuple2*,struct Cyc_PP_Ppstate*),struct _tuple2*x))Cyc_Fn_make_fn;_tmp44;});_tmpCA(Cyc_PP_text_doc_f,({struct _tuple2*_tmp45=_cycalloc(sizeof(*_tmp45));
# 200
_tmp45->f1=slen,_tmp45->f2=s;_tmp45;}));});
# 198
_tmp46->f=_tmpCB;});_tmp46;});}
# 186
struct Cyc_PP_Doc*Cyc_PP_text(struct _fat_ptr s){
# 203
struct Cyc_Hashtable_Table*t;
if(Cyc_PP_str_hashtbl == 0){
t=({struct Cyc_Hashtable_Table*(*_tmp47)(int sz,int(*cmp)(struct _fat_ptr*,struct _fat_ptr*),int(*hash)(struct _fat_ptr*))=(struct Cyc_Hashtable_Table*(*)(int sz,int(*cmp)(struct _fat_ptr*,struct _fat_ptr*),int(*hash)(struct _fat_ptr*)))Cyc_Hashtable_create;_tmp47;})(101,Cyc_zstrptrcmp,Cyc_Hashtable_hash_stringptr);
Cyc_PP_str_hashtbl=({struct Cyc_Core_Opt*_tmp48=_cycalloc(sizeof(*_tmp48));_tmp48->v=t;_tmp48;});}else{
# 208
t=(struct Cyc_Hashtable_Table*)((struct Cyc_Core_Opt*)_check_null(Cyc_PP_str_hashtbl))->v;}{
# 210
struct _handler_cons _tmp49;_push_handler(& _tmp49);{int _tmp4B=0;if(setjmp(_tmp49.handler))_tmp4B=1;if(!_tmp4B){
{struct Cyc_PP_Doc*_tmp4E=({struct Cyc_PP_Doc*(*_tmpCD)(struct Cyc_Hashtable_Table*t,struct _fat_ptr*key)=({struct Cyc_PP_Doc*(*_tmp4C)(struct Cyc_Hashtable_Table*t,struct _fat_ptr*key)=(struct Cyc_PP_Doc*(*)(struct Cyc_Hashtable_Table*t,struct _fat_ptr*key))Cyc_Hashtable_lookup;_tmp4C;});struct Cyc_Hashtable_Table*_tmpCC=t;_tmpCD(_tmpCC,({struct _fat_ptr*_tmp4D=_cycalloc(sizeof(*_tmp4D));*_tmp4D=s;_tmp4D;}));});_npop_handler(0U);return _tmp4E;};_pop_handler();}else{void*_tmp4A=(void*)Cyc_Core_get_exn_thrown();void*_tmp4F=_tmp4A;void*_tmp50;if(((struct Cyc_Core_Not_found_exn_struct*)_tmp4F)->tag == Cyc_Core_Not_found){_LL1: _LL2: {
# 214
struct Cyc_PP_Doc*d=Cyc_PP_text_doc(s);
({void(*_tmpD0)(struct Cyc_Hashtable_Table*t,struct _fat_ptr*key,struct Cyc_PP_Doc*val)=({void(*_tmp51)(struct Cyc_Hashtable_Table*t,struct _fat_ptr*key,struct Cyc_PP_Doc*val)=(void(*)(struct Cyc_Hashtable_Table*t,struct _fat_ptr*key,struct Cyc_PP_Doc*val))Cyc_Hashtable_insert;_tmp51;});struct Cyc_Hashtable_Table*_tmpCF=t;struct _fat_ptr*_tmpCE=({struct _fat_ptr*_tmp52=_cycalloc(sizeof(*_tmp52));*_tmp52=s;_tmp52;});_tmpD0(_tmpCF,_tmpCE,d);});
return d;}}else{_LL3: _tmp50=_tmp4F;_LL4: {void*exn=_tmp50;(int)_rethrow(exn);}}_LL0:;}}}}
# 186
struct Cyc_PP_Doc*Cyc_PP_textptr(struct _fat_ptr*s){
# 219
return Cyc_PP_text(*s);}
# 186
static struct Cyc_PP_Doc*Cyc_PP_text_width_doc(struct _fat_ptr s,int slen){
# 225
return({struct Cyc_PP_Doc*_tmp55=_cycalloc(sizeof(*_tmp55));
_tmp55->mwo=slen,_tmp55->mw=Cyc_PP_infinity,({
# 228
struct Cyc_Fn_Function*_tmpD2=({struct Cyc_Fn_Function*(*_tmpD1)(struct Cyc_PP_Out*(*f)(struct _tuple2*,struct Cyc_PP_Ppstate*),struct _tuple2*x)=({struct Cyc_Fn_Function*(*_tmp53)(struct Cyc_PP_Out*(*f)(struct _tuple2*,struct Cyc_PP_Ppstate*),struct _tuple2*x)=(struct Cyc_Fn_Function*(*)(struct Cyc_PP_Out*(*f)(struct _tuple2*,struct Cyc_PP_Ppstate*),struct _tuple2*x))Cyc_Fn_make_fn;_tmp53;});_tmpD1(Cyc_PP_text_doc_f,({struct _tuple2*_tmp54=_cycalloc(sizeof(*_tmp54));
# 230
_tmp54->f1=slen,_tmp54->f2=s;_tmp54;}));});
# 228
_tmp55->f=_tmpD2;});_tmp55;});}
# 186
static struct Cyc_Core_Opt*Cyc_PP_str_hashtbl2=0;
# 233
struct Cyc_PP_Doc*Cyc_PP_text_width(struct _fat_ptr s,int slen){
struct Cyc_Hashtable_Table*t;
if(Cyc_PP_str_hashtbl2 == 0){
t=({struct Cyc_Hashtable_Table*(*_tmp56)(int sz,int(*cmp)(struct _fat_ptr*,struct _fat_ptr*),int(*hash)(struct _fat_ptr*))=(struct Cyc_Hashtable_Table*(*)(int sz,int(*cmp)(struct _fat_ptr*,struct _fat_ptr*),int(*hash)(struct _fat_ptr*)))Cyc_Hashtable_create;_tmp56;})(101,Cyc_zstrptrcmp,Cyc_Hashtable_hash_stringptr);
Cyc_PP_str_hashtbl2=({struct Cyc_Core_Opt*_tmp57=_cycalloc(sizeof(*_tmp57));_tmp57->v=t;_tmp57;});}else{
# 239
t=(struct Cyc_Hashtable_Table*)((struct Cyc_Core_Opt*)_check_null(Cyc_PP_str_hashtbl2))->v;}{
# 241
struct _handler_cons _tmp58;_push_handler(& _tmp58);{int _tmp5A=0;if(setjmp(_tmp58.handler))_tmp5A=1;if(!_tmp5A){
{struct Cyc_PP_Doc*_tmp5D=({struct Cyc_PP_Doc*(*_tmpD4)(struct Cyc_Hashtable_Table*t,struct _fat_ptr*key)=({struct Cyc_PP_Doc*(*_tmp5B)(struct Cyc_Hashtable_Table*t,struct _fat_ptr*key)=(struct Cyc_PP_Doc*(*)(struct Cyc_Hashtable_Table*t,struct _fat_ptr*key))Cyc_Hashtable_lookup;_tmp5B;});struct Cyc_Hashtable_Table*_tmpD3=t;_tmpD4(_tmpD3,({struct _fat_ptr*_tmp5C=_cycalloc(sizeof(*_tmp5C));*_tmp5C=s;_tmp5C;}));});_npop_handler(0U);return _tmp5D;};_pop_handler();}else{void*_tmp59=(void*)Cyc_Core_get_exn_thrown();void*_tmp5E=_tmp59;void*_tmp5F;if(((struct Cyc_Core_Not_found_exn_struct*)_tmp5E)->tag == Cyc_Core_Not_found){_LL1: _LL2: {
# 245
struct Cyc_PP_Doc*d=Cyc_PP_text_width_doc(s,slen);
({void(*_tmpD7)(struct Cyc_Hashtable_Table*t,struct _fat_ptr*key,struct Cyc_PP_Doc*val)=({void(*_tmp60)(struct Cyc_Hashtable_Table*t,struct _fat_ptr*key,struct Cyc_PP_Doc*val)=(void(*)(struct Cyc_Hashtable_Table*t,struct _fat_ptr*key,struct Cyc_PP_Doc*val))Cyc_Hashtable_insert;_tmp60;});struct Cyc_Hashtable_Table*_tmpD6=t;struct _fat_ptr*_tmpD5=({struct _fat_ptr*_tmp61=_cycalloc(sizeof(*_tmp61));*_tmp61=s;_tmp61;});_tmpD7(_tmpD6,_tmpD5,d);});
return d;}}else{_LL3: _tmp5F=_tmp5E;_LL4: {void*exn=_tmp5F;(int)_rethrow(exn);}}_LL0:;}}}}
# 233
struct Cyc_Core_Opt*Cyc_PP_nil_doc_opt=0;
# 252
struct Cyc_Core_Opt*Cyc_PP_blank_doc_opt=0;
struct Cyc_Core_Opt*Cyc_PP_line_doc_opt=0;
# 257
struct Cyc_PP_Doc*Cyc_PP_nil_doc(){
if(Cyc_PP_nil_doc_opt == 0)
Cyc_PP_nil_doc_opt=({struct Cyc_Core_Opt*_tmp63=_cycalloc(sizeof(*_tmp63));({struct Cyc_PP_Doc*_tmpD8=Cyc_PP_text(({const char*_tmp62="";_tag_fat(_tmp62,sizeof(char),1U);}));_tmp63->v=_tmpD8;});_tmp63;});
# 258
return(struct Cyc_PP_Doc*)((struct Cyc_Core_Opt*)_check_null(Cyc_PP_nil_doc_opt))->v;}
# 257
struct Cyc_PP_Doc*Cyc_PP_blank_doc(){
# 265
if(Cyc_PP_blank_doc_opt == 0)
Cyc_PP_blank_doc_opt=({struct Cyc_Core_Opt*_tmp65=_cycalloc(sizeof(*_tmp65));({struct Cyc_PP_Doc*_tmpD9=Cyc_PP_text(({const char*_tmp64="";_tag_fat(_tmp64,sizeof(char),1U);}));_tmp65->v=_tmpD9;});_tmp65;});
# 265
return(struct Cyc_PP_Doc*)((struct Cyc_Core_Opt*)_check_null(Cyc_PP_blank_doc_opt))->v;}struct _tuple3{int f1;struct _fat_ptr f2;struct _fat_ptr f3;};struct _tuple4{int f1;int f2;int f3;struct _fat_ptr f4;};
# 257
static struct Cyc_PP_Out*Cyc_PP_hyperlink_f(struct _tuple3*clo,struct Cyc_PP_Ppstate*st){
# 271
struct _fat_ptr _tmp68;struct _fat_ptr _tmp67;int _tmp66;_LL1: _tmp66=clo->f1;_tmp67=clo->f2;_tmp68=clo->f3;_LL2: {int slen=_tmp66;struct _fat_ptr shrt=_tmp67;struct _fat_ptr full=_tmp68;
return({struct Cyc_PP_Out*_tmp6E=_cycalloc(sizeof(*_tmp6E));_tmp6E->newcc=st->cc + slen,_tmp6E->newcl=st->cl,({
# 274
void*_tmpDD=(void*)({struct Cyc_PP_Single_PP_Alist_struct*_tmp6B=_cycalloc(sizeof(*_tmp6B));_tmp6B->tag=1U,({struct _fat_ptr*_tmpDC=({unsigned _tmp6A=1;struct _fat_ptr*_tmp69=_cycalloc(_check_times(_tmp6A,sizeof(struct _fat_ptr)));_tmp69[0]=shrt;_tmp69;});_tmp6B->f1=_tmpDC;});_tmp6B;});_tmp6E->ppout=_tmpDD;}),({
void*_tmpDB=(void*)({struct Cyc_PP_Single_PP_Alist_struct*_tmp6D=_cycalloc(sizeof(*_tmp6D));_tmp6D->tag=1U,({struct _tuple4*_tmpDA=({struct _tuple4*_tmp6C=_cycalloc(sizeof(*_tmp6C));_tmp6C->f1=st->cl,_tmp6C->f2=st->cc,_tmp6C->f3=slen,_tmp6C->f4=full;_tmp6C;});_tmp6D->f1=_tmpDA;});_tmp6D;});_tmp6E->links=_tmpDB;});_tmp6E;});}}
# 257
struct Cyc_PP_Doc*Cyc_PP_hyperlink(struct _fat_ptr shrt,struct _fat_ptr full){
# 278
int slen=(int)Cyc_strlen((struct _fat_ptr)shrt);
return({struct Cyc_PP_Doc*_tmp71=_cycalloc(sizeof(*_tmp71));_tmp71->mwo=slen,_tmp71->mw=Cyc_PP_infinity,({
# 281
struct Cyc_Fn_Function*_tmpDF=({struct Cyc_Fn_Function*(*_tmpDE)(struct Cyc_PP_Out*(*f)(struct _tuple3*,struct Cyc_PP_Ppstate*),struct _tuple3*x)=({struct Cyc_Fn_Function*(*_tmp6F)(struct Cyc_PP_Out*(*f)(struct _tuple3*,struct Cyc_PP_Ppstate*),struct _tuple3*x)=(struct Cyc_Fn_Function*(*)(struct Cyc_PP_Out*(*f)(struct _tuple3*,struct Cyc_PP_Ppstate*),struct _tuple3*x))Cyc_Fn_make_fn;_tmp6F;});_tmpDE(Cyc_PP_hyperlink_f,({struct _tuple3*_tmp70=_cycalloc(sizeof(*_tmp70));_tmp70->f1=slen,_tmp70->f2=shrt,_tmp70->f3=full;_tmp70;}));});_tmp71->f=_tmpDF;});_tmp71;});}
# 257
static struct Cyc_PP_Out*Cyc_PP_line_f(struct Cyc_PP_Ppstate*st){
# 286
return({struct Cyc_PP_Out*_tmp75=_cycalloc(sizeof(*_tmp75));_tmp75->newcc=st->ci,_tmp75->newcl=st->cl + 1,({
# 288
void*_tmpE2=(void*)({struct Cyc_PP_Single_PP_Alist_struct*_tmp74=_cycalloc(sizeof(*_tmp74));_tmp74->tag=1U,({struct _fat_ptr*_tmpE1=({unsigned _tmp73=1;struct _fat_ptr*_tmp72=_cycalloc(_check_times(_tmp73,sizeof(struct _fat_ptr)));({struct _fat_ptr _tmpE0=Cyc_PP_nlblanks(st->ci);_tmp72[0]=_tmpE0;});_tmp72;});_tmp74->f1=_tmpE1;});_tmp74;});_tmp75->ppout=_tmpE2;}),_tmp75->links=(void*)& Cyc_PP_Empty_link;_tmp75;});}
# 257
struct Cyc_PP_Doc*Cyc_PP_line_doc(){
# 292
if(Cyc_PP_line_doc_opt == 0)
Cyc_PP_line_doc_opt=({struct Cyc_Core_Opt*_tmp78=_cycalloc(sizeof(*_tmp78));({struct Cyc_PP_Doc*_tmpE4=({struct Cyc_PP_Doc*_tmp77=_cycalloc(sizeof(*_tmp77));_tmp77->mwo=0,_tmp77->mw=0,({struct Cyc_Fn_Function*_tmpE3=({struct Cyc_Fn_Function*(*_tmp76)(struct Cyc_PP_Out*(*f)(struct Cyc_PP_Ppstate*))=(struct Cyc_Fn_Function*(*)(struct Cyc_PP_Out*(*f)(struct Cyc_PP_Ppstate*)))Cyc_Fn_fp2fn;_tmp76;})(Cyc_PP_line_f);_tmp77->f=_tmpE3;});_tmp77;});_tmp78->v=_tmpE4;});_tmp78;});
# 292
return(struct Cyc_PP_Doc*)((struct Cyc_Core_Opt*)_check_null(Cyc_PP_line_doc_opt))->v;}struct _tuple5{int f1;struct Cyc_PP_Doc*f2;};
# 257
static struct Cyc_PP_Out*Cyc_PP_nest_f(struct _tuple5*clo,struct Cyc_PP_Ppstate*st){
# 298
struct Cyc_PP_Doc*_tmp7A;int _tmp79;_LL1: _tmp79=clo->f1;_tmp7A=clo->f2;_LL2: {int k=_tmp79;struct Cyc_PP_Doc*d=_tmp7A;
struct Cyc_PP_Ppstate*st2=({struct Cyc_PP_Ppstate*_tmp7C=_cycalloc(sizeof(*_tmp7C));
_tmp7C->ci=st->ci + k,_tmp7C->cc=st->cc,_tmp7C->cl=st->cl,_tmp7C->pw=st->pw,_tmp7C->epw=st->epw;_tmp7C;});
# 305
return({struct Cyc_PP_Out*(*_tmpE6)(struct Cyc_Fn_Function*f,struct Cyc_PP_Ppstate*x)=({struct Cyc_PP_Out*(*_tmp7B)(struct Cyc_Fn_Function*f,struct Cyc_PP_Ppstate*x)=(struct Cyc_PP_Out*(*)(struct Cyc_Fn_Function*f,struct Cyc_PP_Ppstate*x))Cyc_Fn_apply;_tmp7B;});struct Cyc_Fn_Function*_tmpE5=d->f;_tmpE6(_tmpE5,st2);});}}
# 257
struct Cyc_PP_Doc*Cyc_PP_nest(int k,struct Cyc_PP_Doc*d){
# 308
return({struct Cyc_PP_Doc*_tmp7F=_cycalloc(sizeof(*_tmp7F));_tmp7F->mwo=d->mwo,_tmp7F->mw=d->mw,({
# 310
struct Cyc_Fn_Function*_tmpE8=({struct Cyc_Fn_Function*(*_tmpE7)(struct Cyc_PP_Out*(*f)(struct _tuple5*,struct Cyc_PP_Ppstate*),struct _tuple5*x)=({struct Cyc_Fn_Function*(*_tmp7D)(struct Cyc_PP_Out*(*f)(struct _tuple5*,struct Cyc_PP_Ppstate*),struct _tuple5*x)=(struct Cyc_Fn_Function*(*)(struct Cyc_PP_Out*(*f)(struct _tuple5*,struct Cyc_PP_Ppstate*),struct _tuple5*x))Cyc_Fn_make_fn;_tmp7D;});_tmpE7(Cyc_PP_nest_f,({struct _tuple5*_tmp7E=_cycalloc(sizeof(*_tmp7E));
# 312
_tmp7E->f1=k,_tmp7E->f2=d;_tmp7E;}));});
# 310
_tmp7F->f=_tmpE8;});_tmp7F;});}
# 257
int Cyc_PP_min(int x,int y){
# 316
if(x <= y)return x;else{
return y;}}
# 257
int Cyc_PP_max(int x,int y){
# 321
if(x >= y)return x;else{
return y;}}struct _tuple6{struct Cyc_PP_Doc*f1;struct Cyc_PP_Doc*f2;};
# 257
static struct Cyc_PP_Out*Cyc_PP_concat_f(struct _tuple6*clo,struct Cyc_PP_Ppstate*st){
# 326
struct Cyc_PP_Doc*_tmp81;struct Cyc_PP_Doc*_tmp80;_LL1: _tmp80=clo->f1;_tmp81=clo->f2;_LL2: {struct Cyc_PP_Doc*d1=_tmp80;struct Cyc_PP_Doc*d2=_tmp81;
int epw2=Cyc_PP_max(st->pw - d2->mw,st->epw - d1->mwo);
struct Cyc_PP_Ppstate*st2=({struct Cyc_PP_Ppstate*_tmp86=_cycalloc(sizeof(*_tmp86));_tmp86->ci=st->ci,_tmp86->cc=st->cc,_tmp86->cl=st->cl,_tmp86->pw=st->pw,_tmp86->epw=epw2;_tmp86;});
struct Cyc_PP_Out*o1=({struct Cyc_PP_Out*(*_tmpEA)(struct Cyc_Fn_Function*f,struct Cyc_PP_Ppstate*x)=({struct Cyc_PP_Out*(*_tmp85)(struct Cyc_Fn_Function*f,struct Cyc_PP_Ppstate*x)=(struct Cyc_PP_Out*(*)(struct Cyc_Fn_Function*f,struct Cyc_PP_Ppstate*x))Cyc_Fn_apply;_tmp85;});struct Cyc_Fn_Function*_tmpE9=d1->f;_tmpEA(_tmpE9,st2);});
struct Cyc_PP_Ppstate*st3=({struct Cyc_PP_Ppstate*_tmp84=_cycalloc(sizeof(*_tmp84));_tmp84->ci=st->ci,_tmp84->cc=o1->newcc,_tmp84->cl=o1->newcl,_tmp84->pw=st->pw,_tmp84->epw=epw2;_tmp84;});
struct Cyc_PP_Out*o2=({struct Cyc_PP_Out*(*_tmpEC)(struct Cyc_Fn_Function*f,struct Cyc_PP_Ppstate*x)=({struct Cyc_PP_Out*(*_tmp83)(struct Cyc_Fn_Function*f,struct Cyc_PP_Ppstate*x)=(struct Cyc_PP_Out*(*)(struct Cyc_Fn_Function*f,struct Cyc_PP_Ppstate*x))Cyc_Fn_apply;_tmp83;});struct Cyc_Fn_Function*_tmpEB=d2->f;_tmpEC(_tmpEB,st3);});
struct Cyc_PP_Out*o3=({struct Cyc_PP_Out*_tmp82=_cycalloc(sizeof(*_tmp82));_tmp82->newcc=o2->newcc,_tmp82->newcl=o2->newcl,({
# 334
void*_tmpEE=Cyc_PP_append(o1->ppout,o2->ppout);_tmp82->ppout=_tmpEE;}),({
void*_tmpED=Cyc_PP_append(o1->links,o2->links);_tmp82->links=_tmpED;});_tmp82;});
return o3;}}
# 257
static struct Cyc_PP_Doc*Cyc_PP_concat(struct Cyc_PP_Doc*d1,struct Cyc_PP_Doc*d2){
# 339
return({struct Cyc_PP_Doc*_tmp89=_cycalloc(sizeof(*_tmp89));({int _tmpF2=Cyc_PP_min(d1->mw,d1->mwo + d2->mwo);_tmp89->mwo=_tmpF2;}),({
int _tmpF1=Cyc_PP_min(d1->mw,d1->mwo + d2->mw);_tmp89->mw=_tmpF1;}),({
struct Cyc_Fn_Function*_tmpF0=({struct Cyc_Fn_Function*(*_tmpEF)(struct Cyc_PP_Out*(*f)(struct _tuple6*,struct Cyc_PP_Ppstate*),struct _tuple6*x)=({struct Cyc_Fn_Function*(*_tmp87)(struct Cyc_PP_Out*(*f)(struct _tuple6*,struct Cyc_PP_Ppstate*),struct _tuple6*x)=(struct Cyc_Fn_Function*(*)(struct Cyc_PP_Out*(*f)(struct _tuple6*,struct Cyc_PP_Ppstate*),struct _tuple6*x))Cyc_Fn_make_fn;_tmp87;});_tmpEF(Cyc_PP_concat_f,({struct _tuple6*_tmp88=_cycalloc(sizeof(*_tmp88));_tmp88->f1=d1,_tmp88->f2=d2;_tmp88;}));});_tmp89->f=_tmpF0;});_tmp89;});}
# 257
struct Cyc_PP_Doc*Cyc_PP_cat(struct _fat_ptr l){
# 344
struct Cyc_PP_Doc*d=Cyc_PP_nil_doc();
{int i=(int)(_get_fat_size(l,sizeof(struct Cyc_PP_Doc*))- (unsigned)1);for(0;i >= 0;-- i){
d=Cyc_PP_concat(*((struct Cyc_PP_Doc**)_check_fat_subscript(l,sizeof(struct Cyc_PP_Doc*),i)),d);}}
# 348
return d;}
# 257
static struct Cyc_PP_Out*Cyc_PP_long_cats_f(struct Cyc_List_List*ds0,struct Cyc_PP_Ppstate*st){
# 354
struct Cyc_List_List*os=0;
{struct Cyc_List_List*ds=ds0;for(0;ds != 0;ds=ds->tl){
struct Cyc_PP_Doc*d=(struct Cyc_PP_Doc*)ds->hd;
struct Cyc_PP_Out*o=({struct Cyc_PP_Out*(*_tmpF4)(struct Cyc_Fn_Function*f,struct Cyc_PP_Ppstate*x)=({struct Cyc_PP_Out*(*_tmp8C)(struct Cyc_Fn_Function*f,struct Cyc_PP_Ppstate*x)=(struct Cyc_PP_Out*(*)(struct Cyc_Fn_Function*f,struct Cyc_PP_Ppstate*x))Cyc_Fn_apply;_tmp8C;});struct Cyc_Fn_Function*_tmpF3=d->f;_tmpF4(_tmpF3,st);});
st=({struct Cyc_PP_Ppstate*_tmp8A=_cycalloc(sizeof(*_tmp8A));_tmp8A->ci=st->ci,_tmp8A->cc=o->newcc,_tmp8A->cl=o->newcl,_tmp8A->pw=st->pw,_tmp8A->epw=st->epw - d->mwo;_tmp8A;});
os=({struct Cyc_List_List*_tmp8B=_cycalloc(sizeof(*_tmp8B));_tmp8B->hd=o,_tmp8B->tl=os;_tmp8B;});}}{
# 361
int newcc=((struct Cyc_PP_Out*)((struct Cyc_List_List*)_check_null(os))->hd)->newcc;
int newcl=((struct Cyc_PP_Out*)os->hd)->newcl;
void*s=(void*)& Cyc_PP_Empty_stringptr;
void*links=(void*)& Cyc_PP_Empty_link;
for(0;os != 0;os=os->tl){
s=Cyc_PP_append(((struct Cyc_PP_Out*)os->hd)->ppout,s);
links=Cyc_PP_append(((struct Cyc_PP_Out*)os->hd)->links,links);}
# 369
return({struct Cyc_PP_Out*_tmp8D=_cycalloc(sizeof(*_tmp8D));_tmp8D->newcc=newcc,_tmp8D->newcl=newcl,_tmp8D->ppout=s,_tmp8D->links=links;_tmp8D;});}}
# 257
static struct Cyc_PP_Doc*Cyc_PP_long_cats(struct Cyc_List_List*doclist){
# 375
struct Cyc_List_List*orig=doclist;
struct Cyc_PP_Doc*d=(struct Cyc_PP_Doc*)((struct Cyc_List_List*)_check_null(doclist))->hd;
doclist=doclist->tl;{
int mw=d->mw;
int mwo=d->mw;
# 381
{struct Cyc_List_List*ds=doclist;for(0;ds != 0;ds=ds->tl){
int mw2=Cyc_PP_min(mw,mwo + ((struct Cyc_PP_Doc*)ds->hd)->mwo);
int mwo2=Cyc_PP_min(mw,mwo + ((struct Cyc_PP_Doc*)ds->hd)->mw);
mw=mw2;
mwo=mwo2;}}
# 387
return({struct Cyc_PP_Doc*_tmp8F=_cycalloc(sizeof(*_tmp8F));_tmp8F->mwo=mw,_tmp8F->mw=mwo,({struct Cyc_Fn_Function*_tmpF6=({struct Cyc_Fn_Function*(*_tmpF5)(struct Cyc_PP_Out*(*f)(struct Cyc_List_List*,struct Cyc_PP_Ppstate*),struct Cyc_List_List*x)=({struct Cyc_Fn_Function*(*_tmp8E)(struct Cyc_PP_Out*(*f)(struct Cyc_List_List*,struct Cyc_PP_Ppstate*),struct Cyc_List_List*x)=(struct Cyc_Fn_Function*(*)(struct Cyc_PP_Out*(*f)(struct Cyc_List_List*,struct Cyc_PP_Ppstate*),struct Cyc_List_List*x))Cyc_Fn_make_fn;_tmp8E;});_tmpF5(Cyc_PP_long_cats_f,orig);});_tmp8F->f=_tmpF6;});_tmp8F;});}}
# 257
struct Cyc_PP_Doc*Cyc_PP_cats(struct Cyc_List_List*doclist){
# 391
if(doclist == 0)return Cyc_PP_nil_doc();else{
if(doclist->tl == 0)return(struct Cyc_PP_Doc*)doclist->hd;else{
# 394
if(Cyc_List_length(doclist)> 30)return Cyc_PP_long_cats(doclist);else{
return({struct Cyc_PP_Doc*_tmpF7=(struct Cyc_PP_Doc*)doclist->hd;Cyc_PP_concat(_tmpF7,Cyc_PP_cats(doclist->tl));});}}}}
# 257
static struct Cyc_PP_Out*Cyc_PP_cats_arr_f(struct _fat_ptr*docs_ptr,struct Cyc_PP_Ppstate*st){
# 399
struct Cyc_List_List*os=0;
struct _fat_ptr docs=*docs_ptr;
int sz=(int)_get_fat_size(docs,sizeof(struct Cyc_PP_Doc*));
{int i=0;for(0;i < sz;++ i){
struct Cyc_PP_Doc*d=*((struct Cyc_PP_Doc**)_check_fat_subscript(docs,sizeof(struct Cyc_PP_Doc*),i));
struct Cyc_PP_Out*o=({struct Cyc_PP_Out*(*_tmpF9)(struct Cyc_Fn_Function*f,struct Cyc_PP_Ppstate*x)=({struct Cyc_PP_Out*(*_tmp92)(struct Cyc_Fn_Function*f,struct Cyc_PP_Ppstate*x)=(struct Cyc_PP_Out*(*)(struct Cyc_Fn_Function*f,struct Cyc_PP_Ppstate*x))Cyc_Fn_apply;_tmp92;});struct Cyc_Fn_Function*_tmpF8=d->f;_tmpF9(_tmpF8,st);});
st=({struct Cyc_PP_Ppstate*_tmp90=_cycalloc(sizeof(*_tmp90));_tmp90->ci=st->ci,_tmp90->cc=o->newcc,_tmp90->cl=o->newcl,_tmp90->pw=st->pw,_tmp90->epw=st->epw - d->mwo;_tmp90;});
os=({struct Cyc_List_List*_tmp91=_cycalloc(sizeof(*_tmp91));_tmp91->hd=o,_tmp91->tl=os;_tmp91;});}}{
# 408
int newcc=((struct Cyc_PP_Out*)((struct Cyc_List_List*)_check_null(os))->hd)->newcc;
int newcl=((struct Cyc_PP_Out*)os->hd)->newcl;
void*s=(void*)& Cyc_PP_Empty_stringptr;
void*links=(void*)& Cyc_PP_Empty_link;
for(0;os != 0;os=os->tl){
s=Cyc_PP_append(((struct Cyc_PP_Out*)os->hd)->ppout,s);
links=Cyc_PP_append(((struct Cyc_PP_Out*)os->hd)->links,links);}
# 416
return({struct Cyc_PP_Out*_tmp93=_cycalloc(sizeof(*_tmp93));_tmp93->newcc=newcc,_tmp93->newcl=newcl,_tmp93->ppout=s,_tmp93->links=links;_tmp93;});}}
# 257
struct Cyc_PP_Doc*Cyc_PP_cats_arr(struct _fat_ptr docs){
# 420
int sz=(int)_get_fat_size(docs,sizeof(struct Cyc_PP_Doc*));
if(sz == 0)
(int)_throw((void*)({struct Cyc_Core_Failure_exn_struct*_tmp95=_cycalloc(sizeof(*_tmp95));_tmp95->tag=Cyc_Core_Failure,({struct _fat_ptr _tmpFA=({const char*_tmp94="cats_arr -- size zero array";_tag_fat(_tmp94,sizeof(char),28U);});_tmp95->f1=_tmpFA;});_tmp95;}));{
# 421
struct Cyc_PP_Doc*d=*((struct Cyc_PP_Doc**)_check_fat_subscript(docs,sizeof(struct Cyc_PP_Doc*),0));
# 424
int mw=d->mw;
int mwo=d->mw;
{int i=1;for(0;i < sz;++ i){
int mw2=Cyc_PP_min(mw,mwo + (*((struct Cyc_PP_Doc**)_check_fat_subscript(docs,sizeof(struct Cyc_PP_Doc*),i)))->mwo);
int mwo2=Cyc_PP_min(mw,mwo + (((struct Cyc_PP_Doc**)docs.curr)[i])->mw);
mw=mw2;
mwo=mwo2;}}
# 432
return({struct Cyc_PP_Doc*_tmp99=_cycalloc(sizeof(*_tmp99));_tmp99->mwo=mw,_tmp99->mw=mwo,({struct Cyc_Fn_Function*_tmpFC=({struct Cyc_Fn_Function*(*_tmpFB)(struct Cyc_PP_Out*(*f)(struct _fat_ptr*,struct Cyc_PP_Ppstate*),struct _fat_ptr*x)=({struct Cyc_Fn_Function*(*_tmp96)(struct Cyc_PP_Out*(*f)(struct _fat_ptr*,struct Cyc_PP_Ppstate*),struct _fat_ptr*x)=(struct Cyc_Fn_Function*(*)(struct Cyc_PP_Out*(*f)(struct _fat_ptr*,struct Cyc_PP_Ppstate*),struct _fat_ptr*x))Cyc_Fn_make_fn;_tmp96;});_tmpFB(Cyc_PP_cats_arr_f,({unsigned _tmp98=1;struct _fat_ptr*_tmp97=_cycalloc(_check_times(_tmp98,sizeof(struct _fat_ptr)));_tmp97[0]=docs;_tmp97;}));});_tmp99->f=_tmpFC;});_tmp99;});}}
# 257
static struct Cyc_PP_Out*Cyc_PP_doc_union_f(struct _tuple6*clo,struct Cyc_PP_Ppstate*st){
# 436
struct Cyc_PP_Doc*_tmp9B;struct Cyc_PP_Doc*_tmp9A;_LL1: _tmp9A=clo->f1;_tmp9B=clo->f2;_LL2: {struct Cyc_PP_Doc*d=_tmp9A;struct Cyc_PP_Doc*d2=_tmp9B;
int dfits=st->cc + d->mwo <= st->epw || st->cc + d->mw <= st->pw;
# 439
if(dfits)return({struct Cyc_PP_Out*(*_tmpFE)(struct Cyc_Fn_Function*f,struct Cyc_PP_Ppstate*x)=({struct Cyc_PP_Out*(*_tmp9C)(struct Cyc_Fn_Function*f,struct Cyc_PP_Ppstate*x)=(struct Cyc_PP_Out*(*)(struct Cyc_Fn_Function*f,struct Cyc_PP_Ppstate*x))Cyc_Fn_apply;_tmp9C;});struct Cyc_Fn_Function*_tmpFD=d->f;_tmpFE(_tmpFD,st);});else{
return({struct Cyc_PP_Out*(*_tmp100)(struct Cyc_Fn_Function*f,struct Cyc_PP_Ppstate*x)=({struct Cyc_PP_Out*(*_tmp9D)(struct Cyc_Fn_Function*f,struct Cyc_PP_Ppstate*x)=(struct Cyc_PP_Out*(*)(struct Cyc_Fn_Function*f,struct Cyc_PP_Ppstate*x))Cyc_Fn_apply;_tmp9D;});struct Cyc_Fn_Function*_tmpFF=d2->f;_tmp100(_tmpFF,st);});}}}
# 257
struct Cyc_PP_Doc*Cyc_PP_doc_union(struct Cyc_PP_Doc*d,struct Cyc_PP_Doc*d2){
# 443
return({struct Cyc_PP_Doc*_tmpA0=_cycalloc(sizeof(*_tmpA0));({int _tmp104=Cyc_PP_min(d->mwo,d2->mwo);_tmpA0->mwo=_tmp104;}),({
int _tmp103=Cyc_PP_min(d->mw,d2->mw);_tmpA0->mw=_tmp103;}),({
struct Cyc_Fn_Function*_tmp102=({struct Cyc_Fn_Function*(*_tmp101)(struct Cyc_PP_Out*(*f)(struct _tuple6*,struct Cyc_PP_Ppstate*),struct _tuple6*x)=({struct Cyc_Fn_Function*(*_tmp9E)(struct Cyc_PP_Out*(*f)(struct _tuple6*,struct Cyc_PP_Ppstate*),struct _tuple6*x)=(struct Cyc_Fn_Function*(*)(struct Cyc_PP_Out*(*f)(struct _tuple6*,struct Cyc_PP_Ppstate*),struct _tuple6*x))Cyc_Fn_make_fn;_tmp9E;});_tmp101(Cyc_PP_doc_union_f,({struct _tuple6*_tmp9F=_cycalloc(sizeof(*_tmp9F));_tmp9F->f1=d,_tmp9F->f2=d2;_tmp9F;}));});_tmpA0->f=_tmp102;});_tmpA0;});}
# 257
struct Cyc_PP_Doc*Cyc_PP_oline_doc(){
# 450
return({struct Cyc_PP_Doc*_tmp105=Cyc_PP_nil_doc();Cyc_PP_doc_union(_tmp105,Cyc_PP_line_doc());});}
# 257
static struct Cyc_PP_Out*Cyc_PP_tab_f(struct Cyc_PP_Doc*d,struct Cyc_PP_Ppstate*st){
# 454
struct Cyc_PP_Ppstate*st2=({struct Cyc_PP_Ppstate*_tmpA2=_cycalloc(sizeof(*_tmpA2));_tmpA2->ci=st->cc,_tmpA2->cc=st->cc,_tmpA2->cl=st->cl,_tmpA2->pw=st->pw,_tmpA2->epw=st->epw;_tmpA2;});
return({struct Cyc_PP_Out*(*_tmp107)(struct Cyc_Fn_Function*f,struct Cyc_PP_Ppstate*x)=({struct Cyc_PP_Out*(*_tmpA1)(struct Cyc_Fn_Function*f,struct Cyc_PP_Ppstate*x)=(struct Cyc_PP_Out*(*)(struct Cyc_Fn_Function*f,struct Cyc_PP_Ppstate*x))Cyc_Fn_apply;_tmpA1;});struct Cyc_Fn_Function*_tmp106=d->f;_tmp107(_tmp106,st2);});}
# 257
struct Cyc_PP_Doc*Cyc_PP_tab(struct Cyc_PP_Doc*d){
# 458
struct Cyc_PP_Doc*d2=({struct Cyc_PP_Doc*_tmpA4=_cycalloc(sizeof(*_tmpA4));
_tmpA4->mwo=d->mwo,_tmpA4->mw=d->mw,({
# 461
struct Cyc_Fn_Function*_tmp109=({struct Cyc_Fn_Function*(*_tmp108)(struct Cyc_PP_Out*(*f)(struct Cyc_PP_Doc*,struct Cyc_PP_Ppstate*),struct Cyc_PP_Doc*x)=({struct Cyc_Fn_Function*(*_tmpA3)(struct Cyc_PP_Out*(*f)(struct Cyc_PP_Doc*,struct Cyc_PP_Ppstate*),struct Cyc_PP_Doc*x)=(struct Cyc_Fn_Function*(*)(struct Cyc_PP_Out*(*f)(struct Cyc_PP_Doc*,struct Cyc_PP_Ppstate*),struct Cyc_PP_Doc*x))Cyc_Fn_make_fn;_tmpA3;});_tmp108(Cyc_PP_tab_f,d);});_tmpA4->f=_tmp109;});_tmpA4;});
return d2;}
# 257
static struct Cyc_PP_Doc*Cyc_PP_ppseq_f(struct Cyc_PP_Doc*(*pp)(void*),struct _fat_ptr sep,struct Cyc_List_List*l){
# 468
if(l == 0)return Cyc_PP_nil_doc();else{
if(l->tl == 0)return pp(l->hd);else{
return({struct Cyc_PP_Doc*_tmpA5[4U];({struct Cyc_PP_Doc*_tmp10D=pp(l->hd);_tmpA5[0]=_tmp10D;}),({struct Cyc_PP_Doc*_tmp10C=Cyc_PP_text(sep);_tmpA5[1]=_tmp10C;}),({struct Cyc_PP_Doc*_tmp10B=Cyc_PP_oline_doc();_tmpA5[2]=_tmp10B;}),({struct Cyc_PP_Doc*_tmp10A=Cyc_PP_ppseq_f(pp,sep,l->tl);_tmpA5[3]=_tmp10A;});Cyc_PP_cat(_tag_fat(_tmpA5,sizeof(struct Cyc_PP_Doc*),4U));});}}}
# 257
struct Cyc_PP_Doc*Cyc_PP_ppseq(struct Cyc_PP_Doc*(*pp)(void*),struct _fat_ptr sep,struct Cyc_List_List*l){
# 474
return Cyc_PP_tab(Cyc_PP_ppseq_f(pp,sep,l));}
# 257
struct Cyc_PP_Doc*Cyc_PP_seq_f(struct _fat_ptr sep,struct Cyc_List_List*l){
# 478
if(l == 0)return Cyc_PP_nil_doc();else{
if(l->tl == 0)return(struct Cyc_PP_Doc*)l->hd;else{
# 481
struct Cyc_PP_Doc*sep2=Cyc_PP_text(sep);
struct Cyc_PP_Doc*oline=Cyc_PP_oline_doc();
struct Cyc_List_List*x=l;
while(((struct Cyc_List_List*)_check_null(x))->tl != 0){
struct Cyc_List_List*temp=x->tl;
({struct Cyc_List_List*_tmp10F=({struct Cyc_List_List*_tmpA7=_cycalloc(sizeof(*_tmpA7));_tmpA7->hd=sep2,({struct Cyc_List_List*_tmp10E=({struct Cyc_List_List*_tmpA6=_cycalloc(sizeof(*_tmpA6));_tmpA6->hd=oline,_tmpA6->tl=temp;_tmpA6;});_tmpA7->tl=_tmp10E;});_tmpA7;});x->tl=_tmp10F;});
x=temp;}
# 489
return Cyc_PP_cats(l);}}}
# 257
struct Cyc_PP_Doc*Cyc_PP_seq(struct _fat_ptr sep,struct Cyc_List_List*l){
# 498
return Cyc_PP_tab(Cyc_PP_seq_f(sep,l));}
# 257
struct Cyc_PP_Doc*Cyc_PP_ppseql_f(struct Cyc_PP_Doc*(*pp)(void*),struct _fat_ptr sep,struct Cyc_List_List*l){
# 503
if(l == 0)return Cyc_PP_nil_doc();else{
if(l->tl == 0)return pp(l->hd);else{
return({struct Cyc_PP_Doc*_tmpA8[4U];({struct Cyc_PP_Doc*_tmp113=pp(l->hd);_tmpA8[0]=_tmp113;}),({struct Cyc_PP_Doc*_tmp112=Cyc_PP_text(sep);_tmpA8[1]=_tmp112;}),({struct Cyc_PP_Doc*_tmp111=Cyc_PP_line_doc();_tmpA8[2]=_tmp111;}),({struct Cyc_PP_Doc*_tmp110=Cyc_PP_ppseql_f(pp,sep,l->tl);_tmpA8[3]=_tmp110;});Cyc_PP_cat(_tag_fat(_tmpA8,sizeof(struct Cyc_PP_Doc*),4U));});}}}
# 257
struct Cyc_PP_Doc*Cyc_PP_ppseql(struct Cyc_PP_Doc*(*pp)(void*),struct _fat_ptr sep,struct Cyc_List_List*l){
# 508
return Cyc_PP_tab(Cyc_PP_ppseql_f(pp,sep,l));}
# 257
static struct Cyc_PP_Doc*Cyc_PP_seql_f(struct _fat_ptr sep,struct Cyc_List_List*l){
# 512
if(l == 0)return Cyc_PP_nil_doc();else{
if(l->tl == 0)return(struct Cyc_PP_Doc*)l->hd;else{
return({struct Cyc_PP_Doc*_tmpA9[4U];_tmpA9[0]=(struct Cyc_PP_Doc*)l->hd,({struct Cyc_PP_Doc*_tmp116=Cyc_PP_text(sep);_tmpA9[1]=_tmp116;}),({struct Cyc_PP_Doc*_tmp115=Cyc_PP_line_doc();_tmpA9[2]=_tmp115;}),({struct Cyc_PP_Doc*_tmp114=Cyc_PP_seql_f(sep,l->tl);_tmpA9[3]=_tmp114;});Cyc_PP_cat(_tag_fat(_tmpA9,sizeof(struct Cyc_PP_Doc*),4U));});}}}
# 257
struct Cyc_PP_Doc*Cyc_PP_seql(struct _fat_ptr sep,struct Cyc_List_List*l0){
# 517
return Cyc_PP_tab(Cyc_PP_seql_f(sep,l0));}
# 257
struct Cyc_PP_Doc*Cyc_PP_group(struct _fat_ptr start,struct _fat_ptr stop,struct _fat_ptr sep,struct Cyc_List_List*ss){
# 522
return({struct Cyc_PP_Doc*_tmpAA[3U];({struct Cyc_PP_Doc*_tmp119=Cyc_PP_text(start);_tmpAA[0]=_tmp119;}),({
struct Cyc_PP_Doc*_tmp118=Cyc_PP_seq(sep,ss);_tmpAA[1]=_tmp118;}),({
struct Cyc_PP_Doc*_tmp117=Cyc_PP_text(stop);_tmpAA[2]=_tmp117;});Cyc_PP_cat(_tag_fat(_tmpAA,sizeof(struct Cyc_PP_Doc*),3U));});}
# 257
struct Cyc_PP_Doc*Cyc_PP_egroup(struct _fat_ptr start,struct _fat_ptr stop,struct _fat_ptr sep,struct Cyc_List_List*ss){
# 529
if(ss == 0)return Cyc_PP_nil_doc();else{
return({struct Cyc_PP_Doc*_tmpAB[3U];({struct Cyc_PP_Doc*_tmp11C=Cyc_PP_text(start);_tmpAB[0]=_tmp11C;}),({
struct Cyc_PP_Doc*_tmp11B=Cyc_PP_seq(sep,ss);_tmpAB[1]=_tmp11B;}),({
struct Cyc_PP_Doc*_tmp11A=Cyc_PP_text(stop);_tmpAB[2]=_tmp11A;});Cyc_PP_cat(_tag_fat(_tmpAB,sizeof(struct Cyc_PP_Doc*),3U));});}}
# 257
struct Cyc_PP_Doc*Cyc_PP_groupl(struct _fat_ptr start,struct _fat_ptr stop,struct _fat_ptr sep,struct Cyc_List_List*ss){
# 536
return({struct Cyc_PP_Doc*_tmpAC[3U];({struct Cyc_PP_Doc*_tmp11F=Cyc_PP_text(start);_tmpAC[0]=_tmp11F;}),({
struct Cyc_PP_Doc*_tmp11E=Cyc_PP_seql(sep,ss);_tmpAC[1]=_tmp11E;}),({
struct Cyc_PP_Doc*_tmp11D=Cyc_PP_text(stop);_tmpAC[2]=_tmp11D;});Cyc_PP_cat(_tag_fat(_tmpAC,sizeof(struct Cyc_PP_Doc*),3U));});}
