#include <setjmp.h>
/* This is a C header file to be used by the output of the Cyclone to
   C translator.  The corresponding definitions are in file lib/runtime_*.c */
#ifndef _CYC_INCLUDE_H_
#define _CYC_INCLUDE_H_

/* Need one of these per thread (see runtime_stack.c). The runtime maintains 
   a stack that contains either _handler_cons structs or _RegionHandle structs.
   The tag is 0 for a handler_cons and 1 for a region handle.  */
struct _RuntimeStack {
  int tag; 
  struct _RuntimeStack *next;
  void (*cleanup)(struct _RuntimeStack *frame);
};

#ifndef offsetof
/* should be size_t, but int is fine. */
#define offsetof(t,n) ((int)(&(((t *)0)->n)))
#endif

/* Fat pointers */
struct _fat_ptr {
  unsigned char *curr; 
  unsigned char *base; 
  unsigned char *last_plus_one; 
};  

/* Discriminated Unions */
struct _xtunion_struct { char *tag; };

/* Regions */
struct _RegionPage
#ifdef CYC_REGION_PROFILE
{ unsigned total_bytes;
  unsigned free_bytes;
  /* MWH: wish we didn't have to include the stuff below ... */
  struct _RegionPage *next;
  char data[1];
}
#endif
; // abstract -- defined in runtime_memory.c
struct _RegionHandle {
  struct _RuntimeStack s;
  struct _RegionPage *curr;
  char               *offset;
  char               *last_plus_one;
  struct _DynRegionHandle *sub_regions;
#ifdef CYC_REGION_PROFILE
  const char         *name;
#else
  unsigned used_bytes;
  unsigned wasted_bytes;
#endif
};
struct _DynRegionFrame {
  struct _RuntimeStack s;
  struct _DynRegionHandle *x;
};
// A dynamic region is just a region handle.  The wrapper struct is for type
// abstraction.
struct Cyc_Core_DynamicRegion {
  struct _RegionHandle h;
};

struct _RegionHandle _new_region(const char*);
void* _region_malloc(struct _RegionHandle*, unsigned);
void* _region_calloc(struct _RegionHandle*, unsigned t, unsigned n);
void   _free_region(struct _RegionHandle*);
struct _RegionHandle*_open_dynregion(struct _DynRegionFrame*,struct _DynRegionHandle*);
void   _pop_dynregion();

/* Exceptions */
struct _handler_cons {
  struct _RuntimeStack s;
  jmp_buf handler;
};
void _push_handler(struct _handler_cons *);
void _push_region(struct _RegionHandle *);
void _npop_handler(int);
void _pop_handler();
void _pop_region();

#ifndef _throw
void* _throw_null_fn(const char*,unsigned);
void* _throw_arraybounds_fn(const char*,unsigned);
void* _throw_badalloc_fn(const char*,unsigned);
void* _throw_match_fn(const char*,unsigned);
void* _throw_fn(void*,const char*,unsigned);
void* _rethrow(void*);
#define _throw_null() (_throw_null_fn(__FILE__,__LINE__))
#define _throw_arraybounds() (_throw_arraybounds_fn(__FILE__,__LINE__))
#define _throw_badalloc() (_throw_badalloc_fn(__FILE__,__LINE__))
#define _throw_match() (_throw_match_fn(__FILE__,__LINE__))
#define _throw(e) (_throw_fn((e),__FILE__,__LINE__))
#endif

struct _xtunion_struct* Cyc_Core_get_exn_thrown();
/* Built-in Exceptions */
struct Cyc_Null_Exception_exn_struct { char *tag; };
struct Cyc_Array_bounds_exn_struct { char *tag; };
struct Cyc_Match_Exception_exn_struct { char *tag; };
struct Cyc_Bad_alloc_exn_struct { char *tag; };
extern char Cyc_Null_Exception[];
extern char Cyc_Array_bounds[];
extern char Cyc_Match_Exception[];
extern char Cyc_Bad_alloc[];

/* Built-in Run-time Checks and company */
#ifdef CYC_ANSI_OUTPUT
#define _INLINE  
#else
#define _INLINE inline
#endif

#ifdef NO_CYC_NULL_CHECKS
#define _check_null(ptr) (ptr)
#else
#define _check_null(ptr) \
  ({ void*_cks_null = (void*)(ptr); \
     if (!_cks_null) _throw_null(); \
     _cks_null; })
#endif

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index)\
   (((char*)ptr) + (elt_sz)*(index))
#ifdef NO_CYC_NULL_CHECKS
#define _check_known_subscript_null _check_known_subscript_notnull
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr);\
  int _index = (index);\
  if (!_cks_ptr) _throw_null(); \
  _cks_ptr + (elt_sz)*_index; })
#endif
#define _zero_arr_plus_fn(orig_x,orig_sz,orig_i,f,l) ((orig_x)+(orig_i))
#define _zero_arr_plus_char_fn _zero_arr_plus_fn
#define _zero_arr_plus_short_fn _zero_arr_plus_fn
#define _zero_arr_plus_int_fn _zero_arr_plus_fn
#define _zero_arr_plus_float_fn _zero_arr_plus_fn
#define _zero_arr_plus_double_fn _zero_arr_plus_fn
#define _zero_arr_plus_longdouble_fn _zero_arr_plus_fn
#define _zero_arr_plus_voidstar_fn _zero_arr_plus_fn
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (!_cks_ptr) _throw_null(); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })

/* _zero_arr_plus_*_fn(x,sz,i,filename,lineno) adds i to zero-terminated ptr
   x that has at least sz elements */
char* _zero_arr_plus_char_fn(char*,unsigned,int,const char*,unsigned);
short* _zero_arr_plus_short_fn(short*,unsigned,int,const char*,unsigned);
int* _zero_arr_plus_int_fn(int*,unsigned,int,const char*,unsigned);
float* _zero_arr_plus_float_fn(float*,unsigned,int,const char*,unsigned);
double* _zero_arr_plus_double_fn(double*,unsigned,int,const char*,unsigned);
long double* _zero_arr_plus_longdouble_fn(long double*,unsigned,int,const char*, unsigned);
void** _zero_arr_plus_voidstar_fn(void**,unsigned,int,const char*,unsigned);
#endif

/* _get_zero_arr_size_*(x,sz) returns the number of elements in a
   zero-terminated array that is NULL or has at least sz elements */
int _get_zero_arr_size_char(const char*,unsigned);
int _get_zero_arr_size_short(const short*,unsigned);
int _get_zero_arr_size_int(const int*,unsigned);
int _get_zero_arr_size_float(const float*,unsigned);
int _get_zero_arr_size_double(const double*,unsigned);
int _get_zero_arr_size_longdouble(const long double*,unsigned);
int _get_zero_arr_size_voidstar(const void**,unsigned);

/* _zero_arr_inplace_plus_*_fn(x,i,filename,lineno) sets
   zero-terminated pointer *x to *x + i */
char* _zero_arr_inplace_plus_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_short_fn(short**,int,const char*,unsigned);
int* _zero_arr_inplace_plus_int(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_longdouble_fn(long double**,int,const char*,unsigned);
void** _zero_arr_inplace_plus_voidstar_fn(void***,int,const char*,unsigned);
/* like the previous functions, but does post-addition (as in e++) */
char* _zero_arr_inplace_plus_post_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_post_short_fn(short**x,int,const char*,unsigned);
int* _zero_arr_inplace_plus_post_int_fn(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_post_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_post_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_post_longdouble_fn(long double**,int,const char *,unsigned);
void** _zero_arr_inplace_plus_post_voidstar_fn(void***,int,const char*,unsigned);
#define _zero_arr_plus_char(x,s,i) \
  (_zero_arr_plus_char_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_short(x,s,i) \
  (_zero_arr_plus_short_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_int(x,s,i) \
  (_zero_arr_plus_int_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_float(x,s,i) \
  (_zero_arr_plus_float_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_double(x,s,i) \
  (_zero_arr_plus_double_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_longdouble(x,s,i) \
  (_zero_arr_plus_longdouble_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_voidstar(x,s,i) \
  (_zero_arr_plus_voidstar_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_inplace_plus_char(x,i) \
  _zero_arr_inplace_plus_char_fn((char **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_short(x,i) \
  _zero_arr_inplace_plus_short_fn((short **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_int(x,i) \
  _zero_arr_inplace_plus_int_fn((int **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_float(x,i) \
  _zero_arr_inplace_plus_float_fn((float **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_double(x,i) \
  _zero_arr_inplace_plus_double_fn((double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_longdouble(x,i) \
  _zero_arr_inplace_plus_longdouble_fn((long double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_voidstar(x,i) \
  _zero_arr_inplace_plus_voidstar_fn((void ***)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_char(x,i) \
  _zero_arr_inplace_plus_post_char_fn((char **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_short(x,i) \
  _zero_arr_inplace_plus_post_short_fn((short **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_int(x,i) \
  _zero_arr_inplace_plus_post_int_fn((int **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_float(x,i) \
  _zero_arr_inplace_plus_post_float_fn((float **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_double(x,i) \
  _zero_arr_inplace_plus_post_double_fn((double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_longdouble(x,i) \
  _zero_arr_inplace_plus_post_longdouble_fn((long double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_voidstar(x,i) \
  _zero_arr_inplace_plus_post_voidstar_fn((void***)(x),(i),__FILE__,__LINE__)

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_fat_subscript(arr,elt_sz,index) ((arr).curr + (elt_sz) * (index))
#define _untag_fat_ptr(arr,elt_sz,num_elts) ((arr).curr)
#else
#define _check_fat_subscript(arr,elt_sz,index) ({ \
  struct _fat_ptr _cus_arr = (arr); \
  unsigned char *_cus_ans = _cus_arr.curr + (elt_sz) * (index); \
  /* JGM: not needed! if (!_cus_arr.base) _throw_null();*/ \
  if (_cus_ans < _cus_arr.base || _cus_ans >= _cus_arr.last_plus_one) \
    _throw_arraybounds(); \
  _cus_ans; })
#define _untag_fat_ptr(arr,elt_sz,num_elts) ({ \
  struct _fat_ptr _arr = (arr); \
  unsigned char *_curr = _arr.curr; \
  if ((_curr < _arr.base || _curr + (elt_sz) * (num_elts) > _arr.last_plus_one) &&\
      _curr != (unsigned char *)0) \
    _throw_arraybounds(); \
  _curr; })
#endif

#define _tag_fat(tcurr,elt_sz,num_elts) ({ \
  struct _fat_ptr _ans; \
  unsigned _num_elts = (num_elts);\
  _ans.base = _ans.curr = (void*)(tcurr); \
  /* JGM: if we're tagging NULL, ignore num_elts */ \
  _ans.last_plus_one = _ans.base ? (_ans.base + (elt_sz) * _num_elts) : 0; \
  _ans; })

#define _get_fat_size(arr,elt_sz) \
  ({struct _fat_ptr _arr = (arr); \
    unsigned char *_arr_curr=_arr.curr; \
    unsigned char *_arr_last=_arr.last_plus_one; \
    (_arr_curr < _arr.base || _arr_curr >= _arr_last) ? 0 : \
    ((_arr_last - _arr_curr) / (elt_sz));})

#define _fat_ptr_plus(arr,elt_sz,change) ({ \
  struct _fat_ptr _ans = (arr); \
  int _change = (change);\
  _ans.curr += (elt_sz) * _change;\
  _ans; })
#define _fat_ptr_inplace_plus(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  _arr_ptr->curr += (elt_sz) * (change);\
  *_arr_ptr; })
#define _fat_ptr_inplace_plus_post(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  struct _fat_ptr _ans = *_arr_ptr; \
  _arr_ptr->curr += (elt_sz) * (change);\
  _ans; })

//Not a macro since initialization order matters. Defined in runtime_zeroterm.c.
struct _fat_ptr _fat_ptr_decrease_size(struct _fat_ptr,unsigned sz,unsigned numelts);

/* Allocation */
void* GC_malloc(int);
void* GC_malloc_atomic(int);
void* GC_calloc(unsigned,unsigned);
void* GC_calloc_atomic(unsigned,unsigned);
// bound the allocation size to be < MAX_ALLOC_SIZE. See macros below for usage.
#define MAX_MALLOC_SIZE (1 << 28)
void* _bounded_GC_malloc(int,const char*,int);
void* _bounded_GC_malloc_atomic(int,const char*,int);
void* _bounded_GC_calloc(unsigned,unsigned,const char*,int);
void* _bounded_GC_calloc_atomic(unsigned,unsigned,const char*,int);
/* these macros are overridden below ifdef CYC_REGION_PROFILE */
#ifndef CYC_REGION_PROFILE
#define _cycalloc(n) _bounded_GC_malloc(n,__FILE__,__LINE__)
#define _cycalloc_atomic(n) _bounded_GC_malloc_atomic(n,__FILE__,__LINE__)
#define _cyccalloc(n,s) _bounded_GC_calloc(n,s,__FILE__,__LINE__)
#define _cyccalloc_atomic(n,s) _bounded_GC_calloc_atomic(n,s,__FILE__,__LINE__)
#endif

static _INLINE unsigned int _check_times(unsigned x, unsigned y) {
  unsigned long long whole_ans = 
    ((unsigned long long) x)*((unsigned long long)y);
  unsigned word_ans = (unsigned)whole_ans;
  if(word_ans < whole_ans || word_ans > MAX_MALLOC_SIZE)
    _throw_badalloc();
  return word_ans;
}

#define _CYC_MAX_REGION_CONST 2
#define _CYC_MIN_ALIGNMENT (sizeof(double))

#ifdef CYC_REGION_PROFILE
extern int rgn_total_bytes;
#endif

static _INLINE void *_fast_region_malloc(struct _RegionHandle *r, unsigned orig_s) {  
  if (r > (struct _RegionHandle *)_CYC_MAX_REGION_CONST && r->curr != 0) { 
#ifdef CYC_NOALIGN
    unsigned s =  orig_s;
#else
    unsigned s =  (orig_s + _CYC_MIN_ALIGNMENT - 1) & (~(_CYC_MIN_ALIGNMENT -1)); 
#endif
    char *result; 
    result = r->offset; 
    if (s <= (r->last_plus_one - result)) {
      r->offset = result + s; 
#ifdef CYC_REGION_PROFILE
    r->curr->free_bytes = r->curr->free_bytes - s;
    rgn_total_bytes += s;
#endif
      return result;
    }
  } 
  return _region_malloc(r,orig_s); 
}

#ifdef CYC_REGION_PROFILE
/* see macros below for usage. defined in runtime_memory.c */
void* _profile_GC_malloc(int,const char*,const char*,int);
void* _profile_GC_malloc_atomic(int,const char*,const char*,int);
void* _profile_GC_calloc(unsigned,unsigned,const char*,const char*,int);
void* _profile_GC_calloc_atomic(unsigned,unsigned,const char*,const char*,int);
void* _profile_region_malloc(struct _RegionHandle*,unsigned,const char*,const char*,int);
void* _profile_region_calloc(struct _RegionHandle*,unsigned,unsigned,const char *,const char*,int);
struct _RegionHandle _profile_new_region(const char*,const char*,const char*,int);
void _profile_free_region(struct _RegionHandle*,const char*,const char*,int);
#ifndef RUNTIME_CYC
#define _new_region(n) _profile_new_region(n,__FILE__,__FUNCTION__,__LINE__)
#define _free_region(r) _profile_free_region(r,__FILE__,__FUNCTION__,__LINE__)
#define _region_malloc(rh,n) _profile_region_malloc(rh,n,__FILE__,__FUNCTION__,__LINE__)
#define _region_calloc(rh,n,t) _profile_region_calloc(rh,n,t,__FILE__,__FUNCTION__,__LINE__)
#  endif
#define _cycalloc(n) _profile_GC_malloc(n,__FILE__,__FUNCTION__,__LINE__)
#define _cycalloc_atomic(n) _profile_GC_malloc_atomic(n,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc(n,s) _profile_GC_calloc(n,s,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc_atomic(n,s) _profile_GC_calloc_atomic(n,s,__FILE__,__FUNCTION__,__LINE__)
#endif
#endif
 struct Cyc_Core_Opt{void*v;};extern char Cyc_Core_Invalid_argument[17U];struct Cyc_Core_Invalid_argument_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Failure[8U];struct Cyc_Core_Failure_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Impossible[11U];struct Cyc_Core_Impossible_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Not_found[10U];struct Cyc_Core_Not_found_exn_struct{char*tag;};extern char Cyc_Core_Unreachable[12U];struct Cyc_Core_Unreachable_exn_struct{char*tag;struct _fat_ptr f1;};
# 165 "core.h"
extern struct _RegionHandle*Cyc_Core_heap_region;
# 168
extern struct _RegionHandle*Cyc_Core_unique_region;struct Cyc_Core_DynamicRegion;struct Cyc_Core_NewDynamicRegion{struct Cyc_Core_DynamicRegion*key;};struct Cyc_Core_ThinRes{void*arr;unsigned nelts;};
# 41 "cycboot.h"
double modf(double,double*);struct Cyc___cycFILE;
# 51
extern struct Cyc___cycFILE*Cyc_stdout;struct Cyc_String_pa_PrintArg_struct{int tag;struct _fat_ptr f1;};struct Cyc_Int_pa_PrintArg_struct{int tag;unsigned long f1;};struct Cyc_Double_pa_PrintArg_struct{int tag;double f1;};struct Cyc_LongDouble_pa_PrintArg_struct{int tag;long double f1;};struct Cyc_ShortPtr_pa_PrintArg_struct{int tag;short*f1;};struct Cyc_IntPtr_pa_PrintArg_struct{int tag;unsigned long*f1;};
# 73
struct _fat_ptr Cyc_aprintf(struct _fat_ptr,struct _fat_ptr);struct Cyc_ShortPtr_sa_ScanfArg_struct{int tag;short*f1;};struct Cyc_UShortPtr_sa_ScanfArg_struct{int tag;unsigned short*f1;};struct Cyc_IntPtr_sa_ScanfArg_struct{int tag;int*f1;};struct Cyc_UIntPtr_sa_ScanfArg_struct{int tag;unsigned*f1;};struct Cyc_StringPtr_sa_ScanfArg_struct{int tag;struct _fat_ptr f1;};struct Cyc_DoublePtr_sa_ScanfArg_struct{int tag;double*f1;};struct Cyc_FloatPtr_sa_ScanfArg_struct{int tag;float*f1;};struct Cyc_CharPtr_sa_ScanfArg_struct{int tag;struct _fat_ptr f1;};
# 161 "cycboot.h"
int Cyc_putc(int,struct Cyc___cycFILE*);
# 224 "cycboot.h"
int Cyc_vfprintf(struct Cyc___cycFILE*,struct _fat_ptr,struct _fat_ptr);
# 228
int Cyc_vprintf(struct _fat_ptr,struct _fat_ptr);
# 232
struct _fat_ptr Cyc_vrprintf(struct _RegionHandle*,struct _fat_ptr,struct _fat_ptr);
# 239
int Cyc_vsnprintf(struct _fat_ptr,unsigned long,struct _fat_ptr,struct _fat_ptr);
# 243
int Cyc_vsprintf(struct _fat_ptr,struct _fat_ptr,struct _fat_ptr);extern char Cyc_FileCloseError[15U];struct Cyc_FileCloseError_exn_struct{char*tag;};extern char Cyc_FileOpenError[14U];struct Cyc_FileOpenError_exn_struct{char*tag;struct _fat_ptr f1;};struct Cyc_List_List{void*hd;struct Cyc_List_List*tl;};extern char Cyc_List_List_mismatch[14U];struct Cyc_List_List_mismatch_exn_struct{char*tag;};extern char Cyc_List_Nth[4U];struct Cyc_List_Nth_exn_struct{char*tag;};
# 104 "string.h"
struct _fat_ptr Cyc_strdup(struct _fat_ptr src);
# 87 "printf.cyc"
static struct _fat_ptr Cyc_parg2string(void*x){
void*_tmp0=x;switch(*((int*)_tmp0)){case 0U: _LL1: _LL2:
 return({const char*_tmp1="string";_tag_fat(_tmp1,sizeof(char),7U);});case 1U: _LL3: _LL4:
 return({const char*_tmp2="int";_tag_fat(_tmp2,sizeof(char),4U);});case 2U: _LL5: _LL6:
# 92
 return({const char*_tmp3="double";_tag_fat(_tmp3,sizeof(char),7U);});case 3U: _LL7: _LL8:
 return({const char*_tmp4="long double";_tag_fat(_tmp4,sizeof(char),12U);});case 4U: _LL9: _LLA:
 return({const char*_tmp5="short *";_tag_fat(_tmp5,sizeof(char),8U);});default: _LLB: _LLC:
 return({const char*_tmp6="unsigned long *";_tag_fat(_tmp6,sizeof(char),16U);});}_LL0:;}
# 87
static void*Cyc_badarg(struct _fat_ptr s){
# 100
return(void*)_throw((void*)({struct Cyc_Core_Invalid_argument_exn_struct*_tmp7=_cycalloc(sizeof(*_tmp7));_tmp7->tag=Cyc_Core_Invalid_argument,_tmp7->f1=s;_tmp7;}));}
# 87
static int Cyc_va_arg_int(struct _fat_ptr ap){
# 105
void*_stmttmp0=*((void**)_check_fat_subscript(ap,sizeof(void*),0U));void*_tmp8=_stmttmp0;unsigned long _tmp9;if(((struct Cyc_Int_pa_PrintArg_struct*)_tmp8)->tag == 1U){_LL1: _tmp9=((struct Cyc_Int_pa_PrintArg_struct*)_tmp8)->f1;_LL2: {unsigned long i=_tmp9;
return(int)i;}}else{_LL3: _LL4:
 return({int(*_tmpC1)(struct _fat_ptr s)=({int(*_tmpA)(struct _fat_ptr s)=(int(*)(struct _fat_ptr s))Cyc_badarg;_tmpA;});_tmpC1(({const char*_tmpB="printf expected int";_tag_fat(_tmpB,sizeof(char),20U);}));});}_LL0:;}
# 87
static long Cyc_va_arg_long(struct _fat_ptr ap){
# 112
void*_stmttmp1=*((void**)_check_fat_subscript(ap,sizeof(void*),0U));void*_tmpC=_stmttmp1;unsigned long _tmpD;if(((struct Cyc_Int_pa_PrintArg_struct*)_tmpC)->tag == 1U){_LL1: _tmpD=((struct Cyc_Int_pa_PrintArg_struct*)_tmpC)->f1;_LL2: {unsigned long i=_tmpD;
return(long)i;}}else{_LL3: _LL4:
 return({long(*_tmpC2)(struct _fat_ptr s)=({long(*_tmpE)(struct _fat_ptr s)=(long(*)(struct _fat_ptr s))Cyc_badarg;_tmpE;});_tmpC2(({const char*_tmpF="printf expected int";_tag_fat(_tmpF,sizeof(char),20U);}));});}_LL0:;}
# 87
static unsigned long Cyc_va_arg_ulong(struct _fat_ptr ap){
# 119
void*_stmttmp2=*((void**)_check_fat_subscript(ap,sizeof(void*),0U));void*_tmp10=_stmttmp2;unsigned long _tmp11;if(((struct Cyc_Int_pa_PrintArg_struct*)_tmp10)->tag == 1U){_LL1: _tmp11=((struct Cyc_Int_pa_PrintArg_struct*)_tmp10)->f1;_LL2: {unsigned long i=_tmp11;
return i;}}else{_LL3: _LL4:
 return({unsigned long(*_tmpC3)(struct _fat_ptr s)=({unsigned long(*_tmp12)(struct _fat_ptr s)=(unsigned long(*)(struct _fat_ptr s))Cyc_badarg;_tmp12;});_tmpC3(({const char*_tmp13="printf expected int";_tag_fat(_tmp13,sizeof(char),20U);}));});}_LL0:;}
# 87
static unsigned long Cyc_va_arg_uint(struct _fat_ptr ap){
# 126
void*_stmttmp3=*((void**)_check_fat_subscript(ap,sizeof(void*),0U));void*_tmp14=_stmttmp3;unsigned long _tmp15;if(((struct Cyc_Int_pa_PrintArg_struct*)_tmp14)->tag == 1U){_LL1: _tmp15=((struct Cyc_Int_pa_PrintArg_struct*)_tmp14)->f1;_LL2: {unsigned long i=_tmp15;
return i;}}else{_LL3: _LL4:
 return({unsigned long(*_tmpC4)(struct _fat_ptr s)=({unsigned long(*_tmp16)(struct _fat_ptr s)=(unsigned long(*)(struct _fat_ptr s))Cyc_badarg;_tmp16;});_tmpC4(({const char*_tmp17="printf expected int";_tag_fat(_tmp17,sizeof(char),20U);}));});}_LL0:;}
# 87
static double Cyc_va_arg_double(struct _fat_ptr ap){
# 134
void*_stmttmp4=*((void**)_check_fat_subscript(ap,sizeof(void*),0U));void*_tmp18=_stmttmp4;long double _tmp19;double _tmp1A;switch(*((int*)_tmp18)){case 2U: _LL1: _tmp1A=((struct Cyc_Double_pa_PrintArg_struct*)_tmp18)->f1;_LL2: {double d=_tmp1A;
return d;}case 3U: _LL3: _tmp19=((struct Cyc_LongDouble_pa_PrintArg_struct*)_tmp18)->f1;_LL4: {long double ld=_tmp19;
return(double)ld;}default: _LL5: _LL6:
# 138
(int)_throw(Cyc_badarg((struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp1D=({struct Cyc_String_pa_PrintArg_struct _tmpBD;_tmpBD.tag=0U,({struct _fat_ptr _tmpC5=(struct _fat_ptr)((struct _fat_ptr)Cyc_parg2string(*((void**)_check_fat_subscript(ap,sizeof(void*),0U))));_tmpBD.f1=_tmpC5;});_tmpBD;});void*_tmp1B[1U];_tmp1B[0]=& _tmp1D;({struct _fat_ptr _tmpC6=({const char*_tmp1C="printf expected double but found %s";_tag_fat(_tmp1C,sizeof(char),36U);});Cyc_aprintf(_tmpC6,_tag_fat(_tmp1B,sizeof(void*),1U));});})));}_LL0:;}
# 87
static short*Cyc_va_arg_short_ptr(struct _fat_ptr ap){
# 145
void*_stmttmp5=*((void**)_check_fat_subscript(ap,sizeof(void*),0U));void*_tmp1E=_stmttmp5;short*_tmp1F;if(((struct Cyc_ShortPtr_pa_PrintArg_struct*)_tmp1E)->tag == 4U){_LL1: _tmp1F=((struct Cyc_ShortPtr_pa_PrintArg_struct*)_tmp1E)->f1;_LL2: {short*p=_tmp1F;
return p;}}else{_LL3: _LL4:
(int)_throw(Cyc_badarg(({const char*_tmp20="printf expected short pointer";_tag_fat(_tmp20,sizeof(char),30U);})));}_LL0:;}
# 87
static unsigned long*Cyc_va_arg_int_ptr(struct _fat_ptr ap){
# 153
void*_stmttmp6=*((void**)_check_fat_subscript(ap,sizeof(void*),0U));void*_tmp21=_stmttmp6;unsigned long*_tmp22;if(((struct Cyc_IntPtr_pa_PrintArg_struct*)_tmp21)->tag == 5U){_LL1: _tmp22=((struct Cyc_IntPtr_pa_PrintArg_struct*)_tmp21)->f1;_LL2: {unsigned long*p=_tmp22;
return p;}}else{_LL3: _LL4:
(int)_throw(Cyc_badarg(({const char*_tmp23="printf expected int pointer";_tag_fat(_tmp23,sizeof(char),28U);})));}_LL0:;}
# 87
static const struct _fat_ptr Cyc_va_arg_string(struct _fat_ptr ap){
# 161
void*_stmttmp7=*((void**)_check_fat_subscript(ap,sizeof(void*),0U));void*_tmp24=_stmttmp7;struct _fat_ptr _tmp25;if(((struct Cyc_String_pa_PrintArg_struct*)_tmp24)->tag == 0U){_LL1: _tmp25=((struct Cyc_String_pa_PrintArg_struct*)_tmp24)->f1;_LL2: {struct _fat_ptr s=_tmp25;
return s;}}else{_LL3: _LL4:
(int)_throw(Cyc_badarg(({const char*_tmp26="printf expected string";_tag_fat(_tmp26,sizeof(char),23U);})));}_LL0:;}
# 87
int Cyc___cvt_double(double number,int prec,int flags,int*signp,int fmtch,struct _fat_ptr startp,struct _fat_ptr endp);
# 206 "printf.cyc"
enum Cyc_BASE{Cyc_OCT =0U,Cyc_DEC =1U,Cyc_HEX =2U};
# 212
inline static int Cyc__IO_sputn(int(*ioputc)(int,void*),void*ioputc_env,struct _fat_ptr s,int howmany){
# 214
int i=0;
while(i < howmany){
if(ioputc((int)*((const char*)_check_fat_subscript(s,sizeof(char),0U)),ioputc_env)== - 1)return i;_fat_ptr_inplace_plus(& s,sizeof(char),1);
# 218
++ i;}
# 220
return i;}
# 212
static int Cyc__IO_nzsputn(int(*ioputc)(int,void*),void*ioputc_env,struct _fat_ptr s,int howmany){
# 225
int i=0;
while(i < howmany){
char c;
if((int)(c=*((const char*)_check_fat_subscript(s,sizeof(char),0U)))== 0 || ioputc((int)c,ioputc_env)== - 1)return i;_fat_ptr_inplace_plus(& s,sizeof(char),1);
# 230
++ i;}
# 232
return i;}
# 212
static int Cyc__IO_padn(int(*ioputc)(int,void*),void*ioputc_env,char c,int howmany){
# 240
int i=0;
while(i < howmany){
if(ioputc((int)c,ioputc_env)== - 1)return i;++ i;}
# 245
return i;}
# 212
static struct _fat_ptr Cyc_my_memchr(struct _fat_ptr s,char c,int n){
# 250
int sz=(int)_get_fat_size(s,sizeof(char));
n=n < sz?n: sz;
for(0;n != 0;(n --,_fat_ptr_inplace_plus_post(& s,sizeof(char),1))){
if((int)*((const char*)_check_fat_subscript(s,sizeof(char),0U))== (int)c)return s;}
# 255
return _tag_fat(0,0,0);}
# 212
static struct _fat_ptr Cyc_my_nzmemchr(struct _fat_ptr s,char c,int n){
# 259
int sz=(int)_get_fat_size(s,sizeof(char));
n=n < sz?n: sz;
for(0;n != 0;(n --,_fat_ptr_inplace_plus_post(& s,sizeof(char),1))){
if((int)*((const char*)_check_fat_subscript(s,sizeof(char),0U))== (int)c)return s;}
# 264
return _tag_fat(0,0,0);}
# 212
inline static const unsigned long Cyc_my_strlen(struct _fat_ptr s){
# 268
unsigned sz=_get_fat_size(s,sizeof(char));
unsigned i=0U;
while(i < sz &&(int)((const char*)s.curr)[(int)i]!= 0){++ i;}
return i;}
# 212
int Cyc__IO_vfprintf(int(*ioputc)(int,void*),void*ioputc_env,struct _fat_ptr fmt0,struct _fat_ptr ap){
# 282
struct _fat_ptr fmt;
register int ch;
register int n;
# 286
struct _fat_ptr cp=_tag_fat(0,0,0);
# 289
struct _fat_ptr cp2=_tag_fat(0,0,0);
# 292
struct _fat_ptr cp3=_tag_fat(0,0,0);
# 295
int which_cp;
struct _fat_ptr fmark;
register int flags;
int ret;
int width;
int prec;
char sign;
# 303
char sign_string[2U];sign_string[0]='\000',sign_string[1]='\000';{
int softsign=0;
double _double;
int fpprec;
unsigned long _ulong;
int dprec;
int dpad;
int fieldsz;
# 314
int size=0;
# 316
char buf[349U];({{unsigned _tmpBE=348U;unsigned i;for(i=0;i < _tmpBE;++ i){buf[i]='\000';}buf[_tmpBE]=0;}0;});{
char ox[2U];ox[0]='\000',ox[1]='\000';{
enum Cyc_BASE base;
# 342 "printf.cyc"
fmt=fmt0;
ret=0;
# 348
for(0;1;0){
# 351
fmark=fmt;{
unsigned fmt_sz=_get_fat_size(fmt,sizeof(char));
for(n=0;((unsigned)n < fmt_sz &&(ch=(int)((const char*)fmt.curr)[n])!= (int)'\000')&& ch != (int)'%';++ n){
;}
fmt=_fat_ptr_plus(fmt,sizeof(char),n);
# 357
if((n=(fmt.curr - fmark.curr)/ sizeof(char))!= 0){
do{if(({int _tmpC7=Cyc__IO_sputn(ioputc,ioputc_env,(struct _fat_ptr)fmark,n);_tmpC7 != n;}))goto error;}while(0);
ret +=n;}
# 357
if(ch == (int)'\000')
# 362
goto done;
# 357
_fat_ptr_inplace_plus(& fmt,sizeof(char),1);
# 365
flags=0;
dprec=0;
fpprec=0;
width=0;
prec=-1;
sign='\000';
# 372
rflag: ch=(int)*((const char*)_check_fat_subscript(_fat_ptr_inplace_plus_post(& fmt,sizeof(char),1),sizeof(char),0U));
reswitch: which_cp=0;
{int _tmp27=ch;switch(_tmp27){case 32U: _LL1: _LL2:
# 381
 if(!((int)sign))
sign=' ';
# 381
goto rflag;case 35U: _LL3: _LL4:
# 385
 flags |=8;
goto rflag;case 42U: _LL5: _LL6:
# 394
 width=Cyc_va_arg_int(ap);_fat_ptr_inplace_plus(& ap,sizeof(void*),1);
if(width >= 0)
goto rflag;
# 395
width=- width;
# 398
goto _LL8;case 45U: _LL7: _LL8:
# 400
 flags |=16;
flags &=~ 32;
goto rflag;case 43U: _LL9: _LLA:
# 404
 sign='+';
goto rflag;case 46U: _LLB: _LLC:
# 407
 if((ch=(int)*((const char*)_check_fat_subscript(_fat_ptr_inplace_plus_post(& fmt,sizeof(char),1),sizeof(char),0U)))== (int)'*'){
n=Cyc_va_arg_int(ap);_fat_ptr_inplace_plus(& ap,sizeof(void*),1);
prec=n < 0?- 1: n;
goto rflag;}
# 407
n=0;
# 413
while((unsigned)(ch - (int)'0')<= (unsigned)9){
n=10 * n + (ch - (int)'0');
ch=(int)*((const char*)_check_fat_subscript(_fat_ptr_inplace_plus_post(& fmt,sizeof(char),1),sizeof(char),0U));}
# 417
prec=n < 0?- 1: n;
goto reswitch;case 48U: _LLD: _LLE:
# 425
 if(!(flags & 16))
flags |=32;
# 425
goto rflag;case 49U: _LLF: _LL10:
# 428
 goto _LL12;case 50U: _LL11: _LL12: goto _LL14;case 51U: _LL13: _LL14: goto _LL16;case 52U: _LL15: _LL16: goto _LL18;case 53U: _LL17: _LL18:
 goto _LL1A;case 54U: _LL19: _LL1A: goto _LL1C;case 55U: _LL1B: _LL1C: goto _LL1E;case 56U: _LL1D: _LL1E: goto _LL20;case 57U: _LL1F: _LL20:
 n=0;
do{
n=10 * n + (ch - (int)'0');
ch=(int)*((const char*)_check_fat_subscript(_fat_ptr_inplace_plus_post(& fmt,sizeof(char),1),sizeof(char),0U));}while((unsigned)(ch - (int)'0')<= (unsigned)9);
# 435
width=n;
goto reswitch;case 76U: _LL21: _LL22:
# 438
 flags |=2;
goto rflag;case 104U: _LL23: _LL24:
# 441
 flags |=4;
goto rflag;case 108U: _LL25: _LL26:
# 444
 flags |=1;
goto rflag;case 99U: _LL27: _LL28:
# 447
 cp=({char*_tmp28=buf;_tag_fat(_tmp28,sizeof(char),349U);});
({struct _fat_ptr _tmp29=cp;char _tmp2A=*((char*)_check_fat_subscript(_tmp29,sizeof(char),0U));char _tmp2B=(char)Cyc_va_arg_int(ap);if(_get_fat_size(_tmp29,sizeof(char))== 1U &&(_tmp2A == 0 && _tmp2B != 0))_throw_arraybounds();*((char*)_tmp29.curr)=_tmp2B;});_fat_ptr_inplace_plus(& ap,sizeof(void*),1);
size=1;
sign='\000';
goto _LL0;case 68U: _LL29: _LL2A:
# 453
 flags |=1;
goto _LL2C;case 100U: _LL2B: _LL2C:
 goto _LL2E;case 105U: _LL2D: _LL2E:
 _ulong=(unsigned long)(flags & 1?Cyc_va_arg_long(ap):(flags & 4?(long)((short)Cyc_va_arg_int(ap)): Cyc_va_arg_int(ap)));_fat_ptr_inplace_plus(& ap,sizeof(void*),1);
if((long)_ulong < 0){
_ulong=- _ulong;
sign='-';}
# 457
base=1U;
# 462
goto number;case 101U: _LL2F: _LL30:
 goto _LL32;case 69U: _LL31: _LL32: goto _LL34;case 102U: _LL33: _LL34: goto _LL36;case 70U: _LL35: _LL36: goto _LL38;case 103U: _LL37: _LL38:
 goto _LL3A;case 71U: _LL39: _LL3A:
 _double=Cyc_va_arg_double(ap);_fat_ptr_inplace_plus(& ap,sizeof(void*),1);
# 470
if(prec > 39){
if(ch != (int)'g' && ch != (int)'G' || flags & 8)
fpprec=prec - 39;
# 471
prec=39;}else{
# 474
if(prec == -1)
prec=6;}
# 470
cp=({char*_tmp2C=buf;_tag_fat(_tmp2C,sizeof(char),349U);});
# 483
({struct _fat_ptr _tmp2D=cp;char _tmp2E=*((char*)_check_fat_subscript(_tmp2D,sizeof(char),0U));char _tmp2F='\000';if(_get_fat_size(_tmp2D,sizeof(char))== 1U &&(_tmp2E == 0 && _tmp2F != 0))_throw_arraybounds();*((char*)_tmp2D.curr)=_tmp2F;});
size=({double _tmpCC=_double;int _tmpCB=prec;int _tmpCA=flags;int _tmpC9=ch;struct _fat_ptr _tmpC8=cp;Cyc___cvt_double(_tmpCC,_tmpCB,_tmpCA,& softsign,_tmpC9,_tmpC8,
# 486
_fat_ptr_plus(({char*_tmp30=buf;_tag_fat(_tmp30,sizeof(char),349U);}),sizeof(char),(int)(sizeof(buf)- (unsigned)1)));});
if(softsign)
sign='-';
# 487
if((int)*((char*)_check_fat_subscript(cp,sizeof(char),0U))== (int)'\000')
# 490
_fat_ptr_inplace_plus(& cp,sizeof(char),1);
# 487
goto _LL0;case 110U: _LL3B: _LL3C:
# 493
 if(flags & 1)
({unsigned long _tmpCD=(unsigned long)ret;*Cyc_va_arg_int_ptr(ap)=_tmpCD;});else{
if(flags & 4)
({short _tmpCE=(short)ret;*Cyc_va_arg_short_ptr(ap)=_tmpCE;});else{
# 498
({unsigned long _tmpCF=(unsigned long)ret;*Cyc_va_arg_int_ptr(ap)=_tmpCF;});}}
_fat_ptr_inplace_plus(& ap,sizeof(void*),1);
continue;case 79U: _LL3D: _LL3E:
# 502
 flags |=1;
goto _LL40;case 111U: _LL3F: _LL40:
# 505
 _ulong=flags & 1?Cyc_va_arg_ulong(ap):(flags & 4?(unsigned long)((unsigned short)Cyc_va_arg_int(ap)): Cyc_va_arg_uint(ap));_fat_ptr_inplace_plus(& ap,sizeof(void*),1);
base=0U;
goto nosign;case 112U: _LL41: _LL42:
# 517 "printf.cyc"
 _ulong=(unsigned long)Cyc_va_arg_long(ap);_fat_ptr_inplace_plus(& ap,sizeof(void*),1);
base=2U;
flags |=64;
ch=(int)'x';
goto nosign;case 115U: _LL43: _LL44: {
# 523
struct _fat_ptr b=Cyc_va_arg_string(ap);_fat_ptr_inplace_plus(& ap,sizeof(void*),1);
which_cp=3;cp3=b;
if(prec >= 0){
struct _fat_ptr p=Cyc_my_nzmemchr(cp3,'\000',prec);
if(({char*_tmpD0=(char*)p.curr;_tmpD0 != (char*)(_tag_fat(0,0,0)).curr;})){
size=(p.curr - cp3.curr)/ sizeof(char);
if(size > prec)
size=prec;}else{
# 532
size=prec;}}else{
# 534
size=(int)Cyc_my_strlen(cp3);}
sign='\000';
goto _LL0;}case 85U: _LL45: _LL46:
# 538
 flags |=1;
goto _LL48;case 117U: _LL47: _LL48:
# 541
 _ulong=flags & 1?Cyc_va_arg_ulong(ap):(flags & 4?(unsigned long)((unsigned short)Cyc_va_arg_int(ap)): Cyc_va_arg_uint(ap));_fat_ptr_inplace_plus(& ap,sizeof(void*),1);
base=1U;
goto nosign;case 88U: _LL49: _LL4A:
 goto _LL4C;case 120U: _LL4B: _LL4C:
 _ulong=flags & 1?Cyc_va_arg_ulong(ap):(flags & 4?(unsigned long)((unsigned short)Cyc_va_arg_int(ap)): Cyc_va_arg_uint(ap));_fat_ptr_inplace_plus(& ap,sizeof(void*),1);
base=2U;
# 548
if(flags & 8 && _ulong != (unsigned long)0)
flags |=64;
# 548
nosign:
# 552
 sign='\000';
# 558
number: if((dprec=prec)>= 0)
flags &=~ 32;
# 558
cp=
# 566
_fat_ptr_plus(({char*_tmp31=buf;_tag_fat(_tmp31,sizeof(char),349U);}),sizeof(char),(308 + 39)+ 1);
if(_ulong != (unsigned long)0 || prec != 0){
struct _fat_ptr xdigs;
# 574
enum Cyc_BASE _tmp32=base;switch(_tmp32){case Cyc_OCT: _LL50: _LL51:
# 576
 do{
({struct _fat_ptr _tmp33=_fat_ptr_inplace_plus(& cp,sizeof(char),-1);char _tmp34=*((char*)_check_fat_subscript(_tmp33,sizeof(char),0U));char _tmp35=(char)((_ulong & (unsigned long)7)+ (unsigned long)'0');if(_get_fat_size(_tmp33,sizeof(char))== 1U &&(_tmp34 == 0 && _tmp35 != 0))_throw_arraybounds();*((char*)_tmp33.curr)=_tmp35;});
_ulong >>=3U;}while((int)_ulong);
# 581
if(flags & 8 &&(int)*((char*)_check_fat_subscript(cp,sizeof(char),0U))!= (int)'0')
({struct _fat_ptr _tmp36=_fat_ptr_inplace_plus(& cp,sizeof(char),-1);char _tmp37=*((char*)_check_fat_subscript(_tmp36,sizeof(char),0U));char _tmp38='0';if(_get_fat_size(_tmp36,sizeof(char))== 1U &&(_tmp37 == 0 && _tmp38 != 0))_throw_arraybounds();*((char*)_tmp36.curr)=_tmp38;});
# 581
goto _LL4F;case Cyc_DEC: _LL52: _LL53:
# 587
 while(_ulong >= (unsigned long)10){
({struct _fat_ptr _tmp39=_fat_ptr_inplace_plus(& cp,sizeof(char),-1);char _tmp3A=*((char*)_check_fat_subscript(_tmp39,sizeof(char),0U));char _tmp3B=(char)(_ulong % (unsigned long)10 + (unsigned long)'0');if(_get_fat_size(_tmp39,sizeof(char))== 1U &&(_tmp3A == 0 && _tmp3B != 0))_throw_arraybounds();*((char*)_tmp39.curr)=_tmp3B;});
_ulong /=10U;}
# 591
({struct _fat_ptr _tmp3C=_fat_ptr_inplace_plus(& cp,sizeof(char),-1);char _tmp3D=*((char*)_check_fat_subscript(_tmp3C,sizeof(char),0U));char _tmp3E=(char)(_ulong + (unsigned long)'0');if(_get_fat_size(_tmp3C,sizeof(char))== 1U &&(_tmp3D == 0 && _tmp3E != 0))_throw_arraybounds();*((char*)_tmp3C.curr)=_tmp3E;});
goto _LL4F;case Cyc_HEX: _LL54: _LL55:
# 595
 if(ch == (int)'X')
xdigs=({const char*_tmp3F="0123456789ABCDEF";_tag_fat(_tmp3F,sizeof(char),17U);});else{
# 598
xdigs=({const char*_tmp40="0123456789abcdef";_tag_fat(_tmp40,sizeof(char),17U);});}
do{
({struct _fat_ptr _tmp41=_fat_ptr_inplace_plus(& cp,sizeof(char),-1);char _tmp42=*((char*)_check_fat_subscript(_tmp41,sizeof(char),0U));char _tmp43=*((const char*)_check_fat_subscript(xdigs,sizeof(char),(int)(_ulong & (unsigned long)15)));if(_get_fat_size(_tmp41,sizeof(char))== 1U &&(_tmp42 == 0 && _tmp43 != 0))_throw_arraybounds();*((char*)_tmp41.curr)=_tmp43;});
_ulong >>=4U;}while((int)_ulong);
# 603
goto _LL4F;default: _LL56: _LL57:
# 605
 cp=(struct _fat_ptr)Cyc_strdup(({const char*_tmp44="bug in vform: bad base";_tag_fat(_tmp44,sizeof(char),23U);}));
goto skipsize;}_LL4F:;}
# 567
size=({
# 609
unsigned char*_tmpD1=(_fat_ptr_plus(({char*_tmp45=buf;_tag_fat(_tmp45,sizeof(char),349U);}),sizeof(char),(308 + 39)+ 1)).curr;_tmpD1 - cp.curr;})/ sizeof(char);
skipsize:
 goto _LL0;default: _LL4D: _LL4E:
# 613
 if(ch == (int)'\000')
goto done;
# 613
cp=({char*_tmp46=buf;_tag_fat(_tmp46,sizeof(char),349U);});
# 617
({struct _fat_ptr _tmp47=cp;char _tmp48=*((char*)_check_fat_subscript(_tmp47,sizeof(char),0U));char _tmp49=(char)ch;if(_get_fat_size(_tmp47,sizeof(char))== 1U &&(_tmp48 == 0 && _tmp49 != 0))_throw_arraybounds();*((char*)_tmp47.curr)=_tmp49;});
size=1;
sign='\000';
goto _LL0;}_LL0:;}
# 645 "printf.cyc"
fieldsz=size + fpprec;
dpad=dprec - size;
if(dpad < 0)
dpad=0;
# 647
if((int)sign)
# 651
++ fieldsz;else{
if(flags & 64)
fieldsz +=2;}
# 647
fieldsz +=dpad;
# 657
if((flags & (16 | 32))== 0){
if(({int _tmpD2=Cyc__IO_padn(ioputc,ioputc_env,' ',width - fieldsz);_tmpD2 < width - fieldsz;}))goto error;}
# 661
if((int)sign){
({struct _fat_ptr _tmp4B=_fat_ptr_plus(({char*_tmp4A=sign_string;_tag_fat(_tmp4A,sizeof(char),2U);}),sizeof(char),0);char _tmp4C=*((char*)_check_fat_subscript(_tmp4B,sizeof(char),0U));char _tmp4D=sign;if(_get_fat_size(_tmp4B,sizeof(char))== 1U &&(_tmp4C == 0 && _tmp4D != 0))_throw_arraybounds();*((char*)_tmp4B.curr)=_tmp4D;});
do{if(({int(*_tmpD4)(int,void*)=ioputc;void*_tmpD3=ioputc_env;Cyc__IO_sputn(_tmpD4,_tmpD3,(struct _fat_ptr)({char*_tmp4E=sign_string;_tag_fat(_tmp4E,sizeof(char),2U);}),1);})!= 1)goto error;}while(0);}else{
if(flags & 64){
ox[0]='0';
ox[1]=(char)ch;
do{if(({int(*_tmpD6)(int,void*)=ioputc;void*_tmpD5=ioputc_env;Cyc__IO_nzsputn(_tmpD6,_tmpD5,_tag_fat(ox,sizeof(char),2U),2);})!= 2)goto error;}while(0);}}
# 661
if((flags & (16 | 32))== 32){
# 672
if(({int _tmpD7=Cyc__IO_padn(ioputc,ioputc_env,'0',width - fieldsz);_tmpD7 < width - fieldsz;}))goto error;}
# 675
if(({int _tmpD8=Cyc__IO_padn(ioputc,ioputc_env,'0',dpad);_tmpD8 < dpad;}))goto error;
# 678
if(which_cp == 0)
do{if(({int _tmpD9=Cyc__IO_sputn(ioputc,ioputc_env,(struct _fat_ptr)cp,size);_tmpD9 != size;}))goto error;}while(0);else{
if(which_cp == 2)
do{if(({int _tmpDA=Cyc__IO_sputn(ioputc,ioputc_env,(struct _fat_ptr)cp2,size);_tmpDA != size;}))goto error;}while(0);else{
if(which_cp == 3)
do{if(({int _tmpDB=Cyc__IO_nzsputn(ioputc,ioputc_env,(struct _fat_ptr)cp3,size);_tmpDB != size;}))goto error;}while(0);}}
# 678
if(({
# 686
int _tmpDC=Cyc__IO_padn(ioputc,ioputc_env,'0',fpprec);_tmpDC < fpprec;}))goto error;
# 689
if(flags & 16){
if(({int _tmpDD=Cyc__IO_padn(ioputc,ioputc_env,' ',width - fieldsz);_tmpDD < width - fieldsz;}))goto error;}
# 693
ret +=width > fieldsz?width: fieldsz;}}
# 696
done:
 return ret;
error:
 return - 1;}}}}
# 212 "printf.cyc"
static struct _fat_ptr Cyc_exponent(struct _fat_ptr p,int exp,int fmtch){
# 705 "printf.cyc"
struct _fat_ptr t;
char expbuffer[309U];({{unsigned _tmpBF=308U;unsigned i;for(i=0;i < _tmpBF;++ i){expbuffer[i]='0';}expbuffer[_tmpBF]=0;}0;});{
struct _fat_ptr expbuf=({char*_tmp67=(char*)expbuffer;_tag_fat(_tmp67,sizeof(char),309U);});
({struct _fat_ptr _tmp4F=_fat_ptr_inplace_plus_post(& p,sizeof(char),1);char _tmp50=*((char*)_check_fat_subscript(_tmp4F,sizeof(char),0U));char _tmp51=(char)fmtch;if(_get_fat_size(_tmp4F,sizeof(char))== 1U &&(_tmp50 == 0 && _tmp51 != 0))_throw_arraybounds();*((char*)_tmp4F.curr)=_tmp51;});
if(exp < 0){
exp=- exp;
({struct _fat_ptr _tmp52=_fat_ptr_inplace_plus_post(& p,sizeof(char),1);char _tmp53=*((char*)_check_fat_subscript(_tmp52,sizeof(char),0U));char _tmp54='-';if(_get_fat_size(_tmp52,sizeof(char))== 1U &&(_tmp53 == 0 && _tmp54 != 0))_throw_arraybounds();*((char*)_tmp52.curr)=_tmp54;});}else{
# 714
({struct _fat_ptr _tmp55=_fat_ptr_inplace_plus_post(& p,sizeof(char),1);char _tmp56=*((char*)_check_fat_subscript(_tmp55,sizeof(char),0U));char _tmp57='+';if(_get_fat_size(_tmp55,sizeof(char))== 1U &&(_tmp56 == 0 && _tmp57 != 0))_throw_arraybounds();*((char*)_tmp55.curr)=_tmp57;});}
t=_fat_ptr_plus(expbuf,sizeof(char),308);
if(exp > 9){
do{
({struct _fat_ptr _tmp58=_fat_ptr_inplace_plus(& t,sizeof(char),-1);char _tmp59=*((char*)_check_fat_subscript(_tmp58,sizeof(char),0U));char _tmp5A=(char)(exp % 10 + (int)'0');if(_get_fat_size(_tmp58,sizeof(char))== 1U &&(_tmp59 == 0 && _tmp5A != 0))_throw_arraybounds();*((char*)_tmp58.curr)=_tmp5A;});}while((exp /=10)> 9);
# 720
({struct _fat_ptr _tmp5B=_fat_ptr_inplace_plus(& t,sizeof(char),-1);char _tmp5C=*((char*)_check_fat_subscript(_tmp5B,sizeof(char),0U));char _tmp5D=(char)(exp + (int)'0');if(_get_fat_size(_tmp5B,sizeof(char))== 1U &&(_tmp5C == 0 && _tmp5D != 0))_throw_arraybounds();*((char*)_tmp5B.curr)=_tmp5D;});
for(0;({char*_tmpDE=(char*)t.curr;_tmpDE < (char*)(_fat_ptr_plus(expbuf,sizeof(char),308)).curr;});({struct _fat_ptr _tmp5E=_fat_ptr_inplace_plus_post(& p,sizeof(char),1);char _tmp5F=*((char*)_check_fat_subscript(_tmp5E,sizeof(char),0U));char _tmp60=*((char*)_check_fat_subscript(_fat_ptr_inplace_plus_post(& t,sizeof(char),1),sizeof(char),0U));if(_get_fat_size(_tmp5E,sizeof(char))== 1U &&(_tmp5F == 0 && _tmp60 != 0))_throw_arraybounds();*((char*)_tmp5E.curr)=_tmp60;})){;}}else{
# 724
({struct _fat_ptr _tmp61=_fat_ptr_inplace_plus_post(& p,sizeof(char),1);char _tmp62=*((char*)_check_fat_subscript(_tmp61,sizeof(char),0U));char _tmp63='0';if(_get_fat_size(_tmp61,sizeof(char))== 1U &&(_tmp62 == 0 && _tmp63 != 0))_throw_arraybounds();*((char*)_tmp61.curr)=_tmp63;});
({struct _fat_ptr _tmp64=_fat_ptr_inplace_plus_post(& p,sizeof(char),1);char _tmp65=*((char*)_check_fat_subscript(_tmp64,sizeof(char),0U));char _tmp66=(char)(exp + (int)'0');if(_get_fat_size(_tmp64,sizeof(char))== 1U &&(_tmp65 == 0 && _tmp66 != 0))_throw_arraybounds();*((char*)_tmp64.curr)=_tmp66;});}
# 727
return p;}}
# 212 "printf.cyc"
static struct _fat_ptr Cyc_sround(double fract,int*exp,struct _fat_ptr start,struct _fat_ptr end,char ch,int*signp){
# 734 "printf.cyc"
double tmp=0.0;
# 736
if(fract != 0.0)
modf(fract * (double)10,& tmp);else{
# 739
tmp=(double)((int)ch - (int)'0');}
if(tmp > (double)4)
for(0;1;_fat_ptr_inplace_plus(& end,sizeof(char),-1)){
if((int)*((char*)_check_fat_subscript(end,sizeof(char),0U))== (int)'.')
_fat_ptr_inplace_plus(& end,sizeof(char),-1);
# 742
if((int)({struct _fat_ptr _tmp68=end;char _tmp69=*((char*)_check_fat_subscript(_tmp68,sizeof(char),0U));char _tmp6A=_tmp69 + (int)'\001';if(_get_fat_size(_tmp68,sizeof(char))== 1U &&(_tmp69 == 0 && _tmp6A != 0))_throw_arraybounds();*((char*)_tmp68.curr)=_tmp6A;})<= (int)'9')
# 745
break;
# 742
({struct _fat_ptr _tmp6B=end;char _tmp6C=*((char*)_check_fat_subscript(_tmp6B,sizeof(char),0U));char _tmp6D='0';if(_get_fat_size(_tmp6B,sizeof(char))== 1U &&(_tmp6C == 0 && _tmp6D != 0))_throw_arraybounds();*((char*)_tmp6B.curr)=_tmp6D;});
# 747
if((char*)end.curr == (char*)start.curr){
if((unsigned)exp){
({struct _fat_ptr _tmp6E=end;char _tmp6F=*((char*)_check_fat_subscript(_tmp6E,sizeof(char),0U));char _tmp70='1';if(_get_fat_size(_tmp6E,sizeof(char))== 1U &&(_tmp6F == 0 && _tmp70 != 0))_throw_arraybounds();*((char*)_tmp6E.curr)=_tmp70;});
++(*exp);}else{
# 753
({struct _fat_ptr _tmp71=_fat_ptr_inplace_plus(& end,sizeof(char),-1);char _tmp72=*((char*)_check_fat_subscript(_tmp71,sizeof(char),0U));char _tmp73='1';if(_get_fat_size(_tmp71,sizeof(char))== 1U &&(_tmp72 == 0 && _tmp73 != 0))_throw_arraybounds();*((char*)_tmp71.curr)=_tmp73;});
_fat_ptr_inplace_plus(& start,sizeof(char),-1);}
# 756
break;}}else{
# 760
if(*signp == (int)'-')
for(0;1;_fat_ptr_inplace_plus(& end,sizeof(char),-1)){
if((int)*((char*)_check_fat_subscript(end,sizeof(char),0U))== (int)'.')
_fat_ptr_inplace_plus(& end,sizeof(char),-1);
# 762
if((int)*((char*)_check_fat_subscript(end,sizeof(char),0U))!= (int)'0')
# 765
break;
# 762
if((char*)end.curr == (char*)start.curr)
# 767
*signp=0;}}
# 740
return start;}
# 212 "printf.cyc"
int Cyc___cvt_double(double number,int prec,int flags,int*signp,int fmtch,struct _fat_ptr startp,struct _fat_ptr endp){
# 775 "printf.cyc"
struct _fat_ptr p;struct _fat_ptr t;
register double fract;
int dotrim=0;int expcnt;int gformat=0;
double integer=0.0;double tmp=0.0;
# 780
expcnt=0;
if(number < (double)0){
number=- number;
*signp=(int)'-';}else{
# 785
*signp=0;}
# 787
fract=modf(number,& integer);
# 790
t=_fat_ptr_inplace_plus(& startp,sizeof(char),1);
# 796
for(p=_fat_ptr_plus(endp,sizeof(char),- 1);(char*)p.curr >= (char*)startp.curr && integer != 0.0;++ expcnt){
tmp=modf(integer / (double)10,& integer);
({struct _fat_ptr _tmp74=_fat_ptr_inplace_plus_post(& p,sizeof(char),-1);char _tmp75=*((char*)_check_fat_subscript(_tmp74,sizeof(char),0U));char _tmp76=(char)((int)((tmp + .01)* (double)10)+ (int)'0');if(_get_fat_size(_tmp74,sizeof(char))== 1U &&(_tmp75 == 0 && _tmp76 != 0))_throw_arraybounds();*((char*)_tmp74.curr)=_tmp76;});}
# 800
{int _tmp77=fmtch;switch(_tmp77){case 102U: _LL1: _LL2:
 goto _LL4;case 70U: _LL3: _LL4:
# 803
 if(expcnt)
for(0;({char*_tmpDF=(char*)(_fat_ptr_inplace_plus(& p,sizeof(char),1)).curr;_tmpDF < (char*)endp.curr;});({struct _fat_ptr _tmp78=_fat_ptr_inplace_plus_post(& t,sizeof(char),1);char _tmp79=*((char*)_check_fat_subscript(_tmp78,sizeof(char),0U));char _tmp7A=*((char*)_check_fat_subscript(p,sizeof(char),0U));if(_get_fat_size(_tmp78,sizeof(char))== 1U &&(_tmp79 == 0 && _tmp7A != 0))_throw_arraybounds();*((char*)_tmp78.curr)=_tmp7A;})){;}else{
# 806
({struct _fat_ptr _tmp7B=_fat_ptr_inplace_plus_post(& t,sizeof(char),1);char _tmp7C=*((char*)_check_fat_subscript(_tmp7B,sizeof(char),0U));char _tmp7D='0';if(_get_fat_size(_tmp7B,sizeof(char))== 1U &&(_tmp7C == 0 && _tmp7D != 0))_throw_arraybounds();*((char*)_tmp7B.curr)=_tmp7D;});}
# 811
if(prec || flags & 8)
({struct _fat_ptr _tmp7E=_fat_ptr_inplace_plus_post(& t,sizeof(char),1);char _tmp7F=*((char*)_check_fat_subscript(_tmp7E,sizeof(char),0U));char _tmp80='.';if(_get_fat_size(_tmp7E,sizeof(char))== 1U &&(_tmp7F == 0 && _tmp80 != 0))_throw_arraybounds();*((char*)_tmp7E.curr)=_tmp80;});
# 811
if(fract != 0.0){
# 815
if(prec)
do{
fract=modf(fract * (double)10,& tmp);
({struct _fat_ptr _tmp81=_fat_ptr_inplace_plus_post(& t,sizeof(char),1);char _tmp82=*((char*)_check_fat_subscript(_tmp81,sizeof(char),0U));char _tmp83=(char)((int)tmp + (int)'0');if(_get_fat_size(_tmp81,sizeof(char))== 1U &&(_tmp82 == 0 && _tmp83 != 0))_throw_arraybounds();*((char*)_tmp81.curr)=_tmp83;});}while(
-- prec && fract != 0.0);
# 815
if(fract != 0.0)
# 821
startp=({double _tmpE2=fract;struct _fat_ptr _tmpE1=startp;struct _fat_ptr _tmpE0=
_fat_ptr_plus(t,sizeof(char),- 1);
# 821
Cyc_sround(_tmpE2,0,_tmpE1,_tmpE0,'\000',signp);});}
# 811
for(0;prec --;({struct _fat_ptr _tmp84=
# 824
_fat_ptr_inplace_plus_post(& t,sizeof(char),1);char _tmp85=*((char*)_check_fat_subscript(_tmp84,sizeof(char),0U));char _tmp86='0';if(_get_fat_size(_tmp84,sizeof(char))== 1U &&(_tmp85 == 0 && _tmp86 != 0))_throw_arraybounds();*((char*)_tmp84.curr)=_tmp86;})){;}
goto _LL0;case 101U: _LL5: _LL6:
 goto _LL8;case 69U: _LL7: _LL8:
 eformat: if(expcnt){
({struct _fat_ptr _tmp87=_fat_ptr_inplace_plus_post(& t,sizeof(char),1);char _tmp88=*((char*)_check_fat_subscript(_tmp87,sizeof(char),0U));char _tmp89=*((char*)_check_fat_subscript(_fat_ptr_inplace_plus(& p,sizeof(char),1),sizeof(char),0U));if(_get_fat_size(_tmp87,sizeof(char))== 1U &&(_tmp88 == 0 && _tmp89 != 0))_throw_arraybounds();*((char*)_tmp87.curr)=_tmp89;});
if(prec || flags & 8)
({struct _fat_ptr _tmp8A=_fat_ptr_inplace_plus_post(& t,sizeof(char),1);char _tmp8B=*((char*)_check_fat_subscript(_tmp8A,sizeof(char),0U));char _tmp8C='.';if(_get_fat_size(_tmp8A,sizeof(char))== 1U &&(_tmp8B == 0 && _tmp8C != 0))_throw_arraybounds();*((char*)_tmp8A.curr)=_tmp8C;});
# 829
for(0;
# 832
prec &&({char*_tmpE3=(char*)(_fat_ptr_inplace_plus(& p,sizeof(char),1)).curr;_tmpE3 < (char*)endp.curr;});-- prec){
({struct _fat_ptr _tmp8D=_fat_ptr_inplace_plus_post(& t,sizeof(char),1);char _tmp8E=*((char*)_check_fat_subscript(_tmp8D,sizeof(char),0U));char _tmp8F=*((char*)_check_fat_subscript(p,sizeof(char),0U));if(_get_fat_size(_tmp8D,sizeof(char))== 1U &&(_tmp8E == 0 && _tmp8F != 0))_throw_arraybounds();*((char*)_tmp8D.curr)=_tmp8F;});}
# 839
if(!prec &&({char*_tmpE4=(char*)(_fat_ptr_inplace_plus(& p,sizeof(char),1)).curr;_tmpE4 < (char*)endp.curr;})){
fract=(double)0;
startp=({struct _fat_ptr _tmpE7=startp;struct _fat_ptr _tmpE6=
_fat_ptr_plus(t,sizeof(char),- 1);
# 841
char _tmpE5=*((char*)_check_fat_subscript(p,sizeof(char),0U));Cyc_sround((double)0,& expcnt,_tmpE7,_tmpE6,_tmpE5,signp);});}
# 839
-- expcnt;}else{
# 848
if(fract != 0.0){
# 850
for(expcnt=- 1;1;-- expcnt){
fract=modf(fract * (double)10,& tmp);
if(tmp != 0.0)
break;}
# 855
({struct _fat_ptr _tmp90=_fat_ptr_inplace_plus_post(& t,sizeof(char),1);char _tmp91=*((char*)_check_fat_subscript(_tmp90,sizeof(char),0U));char _tmp92=(char)((int)tmp + (int)'0');if(_get_fat_size(_tmp90,sizeof(char))== 1U &&(_tmp91 == 0 && _tmp92 != 0))_throw_arraybounds();*((char*)_tmp90.curr)=_tmp92;});
if(prec || flags & 8)
({struct _fat_ptr _tmp93=_fat_ptr_inplace_plus_post(& t,sizeof(char),1);char _tmp94=*((char*)_check_fat_subscript(_tmp93,sizeof(char),0U));char _tmp95='.';if(_get_fat_size(_tmp93,sizeof(char))== 1U &&(_tmp94 == 0 && _tmp95 != 0))_throw_arraybounds();*((char*)_tmp93.curr)=_tmp95;});}else{
# 860
({struct _fat_ptr _tmp96=_fat_ptr_inplace_plus_post(& t,sizeof(char),1);char _tmp97=*((char*)_check_fat_subscript(_tmp96,sizeof(char),0U));char _tmp98='0';if(_get_fat_size(_tmp96,sizeof(char))== 1U &&(_tmp97 == 0 && _tmp98 != 0))_throw_arraybounds();*((char*)_tmp96.curr)=_tmp98;});
if(prec || flags & 8)
({struct _fat_ptr _tmp99=_fat_ptr_inplace_plus_post(& t,sizeof(char),1);char _tmp9A=*((char*)_check_fat_subscript(_tmp99,sizeof(char),0U));char _tmp9B='.';if(_get_fat_size(_tmp99,sizeof(char))== 1U &&(_tmp9A == 0 && _tmp9B != 0))_throw_arraybounds();*((char*)_tmp99.curr)=_tmp9B;});}}
# 865
if(fract != 0.0){
if(prec)
do{
fract=modf(fract * (double)10,& tmp);
({struct _fat_ptr _tmp9C=_fat_ptr_inplace_plus_post(& t,sizeof(char),1);char _tmp9D=*((char*)_check_fat_subscript(_tmp9C,sizeof(char),0U));char _tmp9E=(char)((int)tmp + (int)'0');if(_get_fat_size(_tmp9C,sizeof(char))== 1U &&(_tmp9D == 0 && _tmp9E != 0))_throw_arraybounds();*((char*)_tmp9C.curr)=_tmp9E;});}while(
-- prec && fract != 0.0);
# 866
if(fract != 0.0)
# 872
startp=({double _tmpEA=fract;struct _fat_ptr _tmpE9=startp;struct _fat_ptr _tmpE8=
_fat_ptr_plus(t,sizeof(char),- 1);
# 872
Cyc_sround(_tmpEA,& expcnt,_tmpE9,_tmpE8,'\000',signp);});}
# 865
for(0;prec --;({struct _fat_ptr _tmp9F=
# 876
_fat_ptr_inplace_plus_post(& t,sizeof(char),1);char _tmpA0=*((char*)_check_fat_subscript(_tmp9F,sizeof(char),0U));char _tmpA1='0';if(_get_fat_size(_tmp9F,sizeof(char))== 1U &&(_tmpA0 == 0 && _tmpA1 != 0))_throw_arraybounds();*((char*)_tmp9F.curr)=_tmpA1;})){;}
# 879
if(gformat && !(flags & 8)){
while((char*)t.curr > (char*)startp.curr &&(int)*((char*)_check_fat_subscript(_fat_ptr_inplace_plus(& t,sizeof(char),-1),sizeof(char),0U))== (int)'0'){;}
if((int)*((char*)_check_fat_subscript(t,sizeof(char),0U))== (int)'.')
_fat_ptr_inplace_plus(& t,sizeof(char),-1);
# 881
_fat_ptr_inplace_plus(& t,sizeof(char),1);}
# 879
t=
# 885
Cyc_exponent(t,expcnt,fmtch);
goto _LL0;case 103U: _LL9: _LLA:
 goto _LLC;case 71U: _LLB: _LLC:
# 889
 if(!prec)
++ prec;
# 889
if(
# 897
expcnt > prec ||(!expcnt && fract != 0.0)&& fract < .0001){
# 905
-- prec;
fmtch -=2;
gformat=1;
goto eformat;}
# 889
if(expcnt)
# 915
for(0;({char*_tmpEB=(char*)(_fat_ptr_inplace_plus(& p,sizeof(char),1)).curr;_tmpEB < (char*)endp.curr;});(({struct _fat_ptr _tmpA2=_fat_ptr_inplace_plus_post(& t,sizeof(char),1);char _tmpA3=*((char*)_check_fat_subscript(_tmpA2,sizeof(char),0U));char _tmpA4=*((char*)_check_fat_subscript(p,sizeof(char),0U));if(_get_fat_size(_tmpA2,sizeof(char))== 1U &&(_tmpA3 == 0 && _tmpA4 != 0))_throw_arraybounds();*((char*)_tmpA2.curr)=_tmpA4;}),-- prec)){;}else{
# 917
({struct _fat_ptr _tmpA5=_fat_ptr_inplace_plus_post(& t,sizeof(char),1);char _tmpA6=*((char*)_check_fat_subscript(_tmpA5,sizeof(char),0U));char _tmpA7='0';if(_get_fat_size(_tmpA5,sizeof(char))== 1U &&(_tmpA6 == 0 && _tmpA7 != 0))_throw_arraybounds();*((char*)_tmpA5.curr)=_tmpA7;});}
# 922
if(prec || flags & 8){
dotrim=1;
({struct _fat_ptr _tmpA8=_fat_ptr_inplace_plus_post(& t,sizeof(char),1);char _tmpA9=*((char*)_check_fat_subscript(_tmpA8,sizeof(char),0U));char _tmpAA='.';if(_get_fat_size(_tmpA8,sizeof(char))== 1U &&(_tmpA9 == 0 && _tmpAA != 0))_throw_arraybounds();*((char*)_tmpA8.curr)=_tmpAA;});}else{
# 927
dotrim=0;}
# 929
if(fract != 0.0){
if(prec){
# 933
do{
fract=modf(fract * (double)10,& tmp);
({struct _fat_ptr _tmpAB=_fat_ptr_inplace_plus_post(& t,sizeof(char),1);char _tmpAC=*((char*)_check_fat_subscript(_tmpAB,sizeof(char),0U));char _tmpAD=(char)((int)tmp + (int)'0');if(_get_fat_size(_tmpAB,sizeof(char))== 1U &&(_tmpAC == 0 && _tmpAD != 0))_throw_arraybounds();*((char*)_tmpAB.curr)=_tmpAD;});}while(
tmp == 0.0 && !expcnt);
while(-- prec && fract != 0.0){
fract=modf(fract * (double)10,& tmp);
({struct _fat_ptr _tmpAE=_fat_ptr_inplace_plus_post(& t,sizeof(char),1);char _tmpAF=*((char*)_check_fat_subscript(_tmpAE,sizeof(char),0U));char _tmpB0=(char)((int)tmp + (int)'0');if(_get_fat_size(_tmpAE,sizeof(char))== 1U &&(_tmpAF == 0 && _tmpB0 != 0))_throw_arraybounds();*((char*)_tmpAE.curr)=_tmpB0;});}}
# 930
if(fract != 0.0)
# 943
startp=({double _tmpEE=fract;struct _fat_ptr _tmpED=startp;struct _fat_ptr _tmpEC=
_fat_ptr_plus(t,sizeof(char),- 1);
# 943
Cyc_sround(_tmpEE,0,_tmpED,_tmpEC,'\000',signp);});}
# 929
if(flags & 8)
# 948
for(0;prec --;({struct _fat_ptr _tmpB1=_fat_ptr_inplace_plus_post(& t,sizeof(char),1);char _tmpB2=*((char*)_check_fat_subscript(_tmpB1,sizeof(char),0U));char _tmpB3='0';if(_get_fat_size(_tmpB1,sizeof(char))== 1U &&(_tmpB2 == 0 && _tmpB3 != 0))_throw_arraybounds();*((char*)_tmpB1.curr)=_tmpB3;})){;}else{
if(dotrim){
while((char*)t.curr > (char*)startp.curr &&(int)*((char*)_check_fat_subscript(_fat_ptr_inplace_plus(& t,sizeof(char),-1),sizeof(char),0U))== (int)'0'){;}
if((int)*((char*)_check_fat_subscript(t,sizeof(char),0U))!= (int)'.')
_fat_ptr_inplace_plus(& t,sizeof(char),1);}}
# 929
goto _LL0;default: _LLD: _LLE:
# 955
(int)_throw((void*)({struct Cyc_Core_Impossible_exn_struct*_tmpB5=_cycalloc(sizeof(*_tmpB5));_tmpB5->tag=Cyc_Core_Impossible,({struct _fat_ptr _tmpEF=({const char*_tmpB4="__cvt_double";_tag_fat(_tmpB4,sizeof(char),13U);});_tmpB5->f1=_tmpEF;});_tmpB5;}));}_LL0:;}
# 957
return(t.curr - startp.curr)/ sizeof(char);}
# 212 "printf.cyc"
int Cyc_vfprintf(struct Cyc___cycFILE*f,struct _fat_ptr fmt,struct _fat_ptr ap){
# 964 "printf.cyc"
int ans;
ans=({int(*_tmpF2)(int(*ioputc)(int,struct Cyc___cycFILE*),struct Cyc___cycFILE*ioputc_env,struct _fat_ptr fmt0,struct _fat_ptr ap)=({int(*_tmpB6)(int(*ioputc)(int,struct Cyc___cycFILE*),struct Cyc___cycFILE*ioputc_env,struct _fat_ptr fmt0,struct _fat_ptr ap)=(int(*)(int(*ioputc)(int,struct Cyc___cycFILE*),struct Cyc___cycFILE*ioputc_env,struct _fat_ptr fmt0,struct _fat_ptr ap))Cyc__IO_vfprintf;_tmpB6;});struct Cyc___cycFILE*_tmpF1=f;struct _fat_ptr _tmpF0=fmt;_tmpF2(Cyc_putc,_tmpF1,_tmpF0,ap);});
return ans;}
# 212 "printf.cyc"
int Cyc_fprintf(struct Cyc___cycFILE*f,struct _fat_ptr fmt,struct _fat_ptr ap){
# 973 "printf.cyc"
return Cyc_vfprintf(f,fmt,ap);}
# 212 "printf.cyc"
int Cyc_vprintf(struct _fat_ptr fmt,struct _fat_ptr ap){
# 979 "printf.cyc"
int ans;
ans=({int(*_tmpF5)(int(*ioputc)(int,struct Cyc___cycFILE*),struct Cyc___cycFILE*ioputc_env,struct _fat_ptr fmt0,struct _fat_ptr ap)=({int(*_tmpB7)(int(*ioputc)(int,struct Cyc___cycFILE*),struct Cyc___cycFILE*ioputc_env,struct _fat_ptr fmt0,struct _fat_ptr ap)=(int(*)(int(*ioputc)(int,struct Cyc___cycFILE*),struct Cyc___cycFILE*ioputc_env,struct _fat_ptr fmt0,struct _fat_ptr ap))Cyc__IO_vfprintf;_tmpB7;});struct Cyc___cycFILE*_tmpF4=Cyc_stdout;struct _fat_ptr _tmpF3=fmt;_tmpF5(Cyc_putc,_tmpF4,_tmpF3,ap);});
return ans;}
# 212 "printf.cyc"
int Cyc_printf(struct _fat_ptr fmt,struct _fat_ptr ap){
# 987 "printf.cyc"
int ans;
ans=Cyc_vprintf(fmt,ap);
return ans;}struct _tuple0{struct _fat_ptr*f1;unsigned long*f2;};
# 212 "printf.cyc"
static int Cyc_putc_string(int c,struct _tuple0*sptr_n){
# 993 "printf.cyc"
unsigned long*_tmpB9;struct _fat_ptr*_tmpB8;_LL1: _tmpB8=sptr_n->f1;_tmpB9=sptr_n->f2;_LL2: {struct _fat_ptr*sptr=_tmpB8;unsigned long*nptr=_tmpB9;
struct _fat_ptr s=*sptr;
unsigned long n=*nptr;
if(n == (unsigned long)0)return - 1;*((char*)_check_fat_subscript(s,sizeof(char),0U))=(char)c;
# 998
_fat_ptr_inplace_plus(sptr,sizeof(char),1);
*nptr=n - (unsigned long)1;
return 1;}}
# 212 "printf.cyc"
int Cyc_vsnprintf(struct _fat_ptr s,unsigned long n,struct _fat_ptr fmt,struct _fat_ptr ap){
# 1006 "printf.cyc"
int ans;
struct _fat_ptr sptr=s;
unsigned long nptr=n;
struct _tuple0 sptr_n=({struct _tuple0 _tmpC0;_tmpC0.f1=& sptr,_tmpC0.f2=& nptr;_tmpC0;});
ans=({int(*_tmpF7)(int(*ioputc)(int,struct _tuple0*),struct _tuple0*ioputc_env,struct _fat_ptr fmt0,struct _fat_ptr ap)=({int(*_tmpBA)(int(*ioputc)(int,struct _tuple0*),struct _tuple0*ioputc_env,struct _fat_ptr fmt0,struct _fat_ptr ap)=(int(*)(int(*ioputc)(int,struct _tuple0*),struct _tuple0*ioputc_env,struct _fat_ptr fmt0,struct _fat_ptr ap))Cyc__IO_vfprintf;_tmpBA;});struct _fat_ptr _tmpF6=fmt;_tmpF7(Cyc_putc_string,& sptr_n,_tmpF6,ap);});
if(0 <= ans)
*((char*)_check_fat_subscript(s,sizeof(char),ans))='\000';
# 1011
return ans;}
# 212 "printf.cyc"
int Cyc_snprintf(struct _fat_ptr s,unsigned long n,struct _fat_ptr fmt,struct _fat_ptr ap){
# 1019 "printf.cyc"
return Cyc_vsnprintf(s,n,fmt,ap);}
# 212 "printf.cyc"
int Cyc_vsprintf(struct _fat_ptr s,struct _fat_ptr fmt,struct _fat_ptr ap){
# 1025 "printf.cyc"
return Cyc_vsnprintf(s,_get_fat_size(s,sizeof(char)),fmt,ap);}
# 212 "printf.cyc"
int Cyc_sprintf(struct _fat_ptr s,struct _fat_ptr fmt,struct _fat_ptr ap){
# 1031 "printf.cyc"
return Cyc_vsnprintf(s,_get_fat_size(s,sizeof(char)),fmt,ap);}
# 212 "printf.cyc"
static int Cyc_putc_void(int c,int dummy){
# 1035 "printf.cyc"
return 1;}
# 212 "printf.cyc"
struct _fat_ptr Cyc_vrprintf(struct _RegionHandle*r1,struct _fat_ptr fmt,struct _fat_ptr ap){
# 1042 "printf.cyc"
int size=({int(*_tmpF9)(int(*ioputc)(int,int),int ioputc_env,struct _fat_ptr fmt0,struct _fat_ptr ap)=({int(*_tmpBC)(int(*ioputc)(int,int),int ioputc_env,struct _fat_ptr fmt0,struct _fat_ptr ap)=(int(*)(int(*ioputc)(int,int),int ioputc_env,struct _fat_ptr fmt0,struct _fat_ptr ap))Cyc__IO_vfprintf;_tmpBC;});struct _fat_ptr _tmpF8=fmt;_tmpF9(Cyc_putc_void,0,_tmpF8,ap);})+ 1;
struct _fat_ptr s=(struct _fat_ptr)({unsigned _tmpBB=size + 1;_tag_fat(_cyccalloc_atomic(sizeof(char),_tmpBB),sizeof(char),_tmpBB);});
({struct _fat_ptr _tmpFB=_fat_ptr_decrease_size(s,sizeof(char),1U);struct _fat_ptr _tmpFA=fmt;Cyc_vsprintf(_tmpFB,_tmpFA,ap);});
return s;}
# 212 "printf.cyc"
struct _fat_ptr Cyc_rprintf(struct _RegionHandle*r1,struct _fat_ptr fmt,struct _fat_ptr ap){
# 1051 "printf.cyc"
return Cyc_vrprintf(r1,fmt,ap);}
# 212 "printf.cyc"
struct _fat_ptr Cyc_aprintf(struct _fat_ptr fmt,struct _fat_ptr ap){
# 1057 "printf.cyc"
return Cyc_vrprintf(Cyc_Core_heap_region,fmt,ap);}
