#include <setjmp.h>
/* This is a C header file to be used by the output of the Cyclone to
   C translator.  The corresponding definitions are in file lib/runtime_*.c */
#ifndef _CYC_INCLUDE_H_
#define _CYC_INCLUDE_H_

/* Need one of these per thread (see runtime_stack.c). The runtime maintains 
   a stack that contains either _handler_cons structs or _RegionHandle structs.
   The tag is 0 for a handler_cons and 1 for a region handle.  */
struct _RuntimeStack {
  int tag; 
  struct _RuntimeStack *next;
  void (*cleanup)(struct _RuntimeStack *frame);
};

#ifndef offsetof
/* should be size_t, but int is fine. */
#define offsetof(t,n) ((int)(&(((t *)0)->n)))
#endif

/* Fat pointers */
struct _fat_ptr {
  unsigned char *curr; 
  unsigned char *base; 
  unsigned char *last_plus_one; 
};  

/* Discriminated Unions */
struct _xtunion_struct { char *tag; };

/* Regions */
struct _RegionPage
#ifdef CYC_REGION_PROFILE
{ unsigned total_bytes;
  unsigned free_bytes;
  /* MWH: wish we didn't have to include the stuff below ... */
  struct _RegionPage *next;
  char data[1];
}
#endif
; // abstract -- defined in runtime_memory.c
struct _RegionHandle {
  struct _RuntimeStack s;
  struct _RegionPage *curr;
  char               *offset;
  char               *last_plus_one;
  struct _DynRegionHandle *sub_regions;
#ifdef CYC_REGION_PROFILE
  const char         *name;
#else
  unsigned used_bytes;
  unsigned wasted_bytes;
#endif
};
struct _DynRegionFrame {
  struct _RuntimeStack s;
  struct _DynRegionHandle *x;
};
// A dynamic region is just a region handle.  The wrapper struct is for type
// abstraction.
struct Cyc_Core_DynamicRegion {
  struct _RegionHandle h;
};

struct _RegionHandle _new_region(const char*);
void* _region_malloc(struct _RegionHandle*, unsigned);
void* _region_calloc(struct _RegionHandle*, unsigned t, unsigned n);
void   _free_region(struct _RegionHandle*);
struct _RegionHandle*_open_dynregion(struct _DynRegionFrame*,struct _DynRegionHandle*);
void   _pop_dynregion();

/* Exceptions */
struct _handler_cons {
  struct _RuntimeStack s;
  jmp_buf handler;
};
void _push_handler(struct _handler_cons *);
void _push_region(struct _RegionHandle *);
void _npop_handler(int);
void _pop_handler();
void _pop_region();

#ifndef _throw
void* _throw_null_fn(const char*,unsigned);
void* _throw_arraybounds_fn(const char*,unsigned);
void* _throw_badalloc_fn(const char*,unsigned);
void* _throw_match_fn(const char*,unsigned);
void* _throw_fn(void*,const char*,unsigned);
void* _rethrow(void*);
#define _throw_null() (_throw_null_fn(__FILE__,__LINE__))
#define _throw_arraybounds() (_throw_arraybounds_fn(__FILE__,__LINE__))
#define _throw_badalloc() (_throw_badalloc_fn(__FILE__,__LINE__))
#define _throw_match() (_throw_match_fn(__FILE__,__LINE__))
#define _throw(e) (_throw_fn((e),__FILE__,__LINE__))
#endif

struct _xtunion_struct* Cyc_Core_get_exn_thrown();
/* Built-in Exceptions */
struct Cyc_Null_Exception_exn_struct { char *tag; };
struct Cyc_Array_bounds_exn_struct { char *tag; };
struct Cyc_Match_Exception_exn_struct { char *tag; };
struct Cyc_Bad_alloc_exn_struct { char *tag; };
extern char Cyc_Null_Exception[];
extern char Cyc_Array_bounds[];
extern char Cyc_Match_Exception[];
extern char Cyc_Bad_alloc[];

/* Built-in Run-time Checks and company */
#ifdef CYC_ANSI_OUTPUT
#define _INLINE  
#else
#define _INLINE inline
#endif

#ifdef NO_CYC_NULL_CHECKS
#define _check_null(ptr) (ptr)
#else
#define _check_null(ptr) \
  ({ void*_cks_null = (void*)(ptr); \
     if (!_cks_null) _throw_null(); \
     _cks_null; })
#endif

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index)\
   (((char*)ptr) + (elt_sz)*(index))
#ifdef NO_CYC_NULL_CHECKS
#define _check_known_subscript_null _check_known_subscript_notnull
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr);\
  int _index = (index);\
  if (!_cks_ptr) _throw_null(); \
  _cks_ptr + (elt_sz)*_index; })
#endif
#define _zero_arr_plus_fn(orig_x,orig_sz,orig_i,f,l) ((orig_x)+(orig_i))
#define _zero_arr_plus_char_fn _zero_arr_plus_fn
#define _zero_arr_plus_short_fn _zero_arr_plus_fn
#define _zero_arr_plus_int_fn _zero_arr_plus_fn
#define _zero_arr_plus_float_fn _zero_arr_plus_fn
#define _zero_arr_plus_double_fn _zero_arr_plus_fn
#define _zero_arr_plus_longdouble_fn _zero_arr_plus_fn
#define _zero_arr_plus_voidstar_fn _zero_arr_plus_fn
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (!_cks_ptr) _throw_null(); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })

/* _zero_arr_plus_*_fn(x,sz,i,filename,lineno) adds i to zero-terminated ptr
   x that has at least sz elements */
char* _zero_arr_plus_char_fn(char*,unsigned,int,const char*,unsigned);
short* _zero_arr_plus_short_fn(short*,unsigned,int,const char*,unsigned);
int* _zero_arr_plus_int_fn(int*,unsigned,int,const char*,unsigned);
float* _zero_arr_plus_float_fn(float*,unsigned,int,const char*,unsigned);
double* _zero_arr_plus_double_fn(double*,unsigned,int,const char*,unsigned);
long double* _zero_arr_plus_longdouble_fn(long double*,unsigned,int,const char*, unsigned);
void** _zero_arr_plus_voidstar_fn(void**,unsigned,int,const char*,unsigned);
#endif

/* _get_zero_arr_size_*(x,sz) returns the number of elements in a
   zero-terminated array that is NULL or has at least sz elements */
int _get_zero_arr_size_char(const char*,unsigned);
int _get_zero_arr_size_short(const short*,unsigned);
int _get_zero_arr_size_int(const int*,unsigned);
int _get_zero_arr_size_float(const float*,unsigned);
int _get_zero_arr_size_double(const double*,unsigned);
int _get_zero_arr_size_longdouble(const long double*,unsigned);
int _get_zero_arr_size_voidstar(const void**,unsigned);

/* _zero_arr_inplace_plus_*_fn(x,i,filename,lineno) sets
   zero-terminated pointer *x to *x + i */
char* _zero_arr_inplace_plus_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_short_fn(short**,int,const char*,unsigned);
int* _zero_arr_inplace_plus_int(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_longdouble_fn(long double**,int,const char*,unsigned);
void** _zero_arr_inplace_plus_voidstar_fn(void***,int,const char*,unsigned);
/* like the previous functions, but does post-addition (as in e++) */
char* _zero_arr_inplace_plus_post_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_post_short_fn(short**x,int,const char*,unsigned);
int* _zero_arr_inplace_plus_post_int_fn(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_post_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_post_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_post_longdouble_fn(long double**,int,const char *,unsigned);
void** _zero_arr_inplace_plus_post_voidstar_fn(void***,int,const char*,unsigned);
#define _zero_arr_plus_char(x,s,i) \
  (_zero_arr_plus_char_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_short(x,s,i) \
  (_zero_arr_plus_short_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_int(x,s,i) \
  (_zero_arr_plus_int_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_float(x,s,i) \
  (_zero_arr_plus_float_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_double(x,s,i) \
  (_zero_arr_plus_double_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_longdouble(x,s,i) \
  (_zero_arr_plus_longdouble_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_voidstar(x,s,i) \
  (_zero_arr_plus_voidstar_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_inplace_plus_char(x,i) \
  _zero_arr_inplace_plus_char_fn((char **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_short(x,i) \
  _zero_arr_inplace_plus_short_fn((short **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_int(x,i) \
  _zero_arr_inplace_plus_int_fn((int **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_float(x,i) \
  _zero_arr_inplace_plus_float_fn((float **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_double(x,i) \
  _zero_arr_inplace_plus_double_fn((double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_longdouble(x,i) \
  _zero_arr_inplace_plus_longdouble_fn((long double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_voidstar(x,i) \
  _zero_arr_inplace_plus_voidstar_fn((void ***)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_char(x,i) \
  _zero_arr_inplace_plus_post_char_fn((char **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_short(x,i) \
  _zero_arr_inplace_plus_post_short_fn((short **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_int(x,i) \
  _zero_arr_inplace_plus_post_int_fn((int **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_float(x,i) \
  _zero_arr_inplace_plus_post_float_fn((float **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_double(x,i) \
  _zero_arr_inplace_plus_post_double_fn((double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_longdouble(x,i) \
  _zero_arr_inplace_plus_post_longdouble_fn((long double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_voidstar(x,i) \
  _zero_arr_inplace_plus_post_voidstar_fn((void***)(x),(i),__FILE__,__LINE__)

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_fat_subscript(arr,elt_sz,index) ((arr).curr + (elt_sz) * (index))
#define _untag_fat_ptr(arr,elt_sz,num_elts) ((arr).curr)
#else
#define _check_fat_subscript(arr,elt_sz,index) ({ \
  struct _fat_ptr _cus_arr = (arr); \
  unsigned char *_cus_ans = _cus_arr.curr + (elt_sz) * (index); \
  /* JGM: not needed! if (!_cus_arr.base) _throw_null();*/ \
  if (_cus_ans < _cus_arr.base || _cus_ans >= _cus_arr.last_plus_one) \
    _throw_arraybounds(); \
  _cus_ans; })
#define _untag_fat_ptr(arr,elt_sz,num_elts) ({ \
  struct _fat_ptr _arr = (arr); \
  unsigned char *_curr = _arr.curr; \
  if ((_curr < _arr.base || _curr + (elt_sz) * (num_elts) > _arr.last_plus_one) &&\
      _curr != (unsigned char *)0) \
    _throw_arraybounds(); \
  _curr; })
#endif

#define _tag_fat(tcurr,elt_sz,num_elts) ({ \
  struct _fat_ptr _ans; \
  unsigned _num_elts = (num_elts);\
  _ans.base = _ans.curr = (void*)(tcurr); \
  /* JGM: if we're tagging NULL, ignore num_elts */ \
  _ans.last_plus_one = _ans.base ? (_ans.base + (elt_sz) * _num_elts) : 0; \
  _ans; })

#define _get_fat_size(arr,elt_sz) \
  ({struct _fat_ptr _arr = (arr); \
    unsigned char *_arr_curr=_arr.curr; \
    unsigned char *_arr_last=_arr.last_plus_one; \
    (_arr_curr < _arr.base || _arr_curr >= _arr_last) ? 0 : \
    ((_arr_last - _arr_curr) / (elt_sz));})

#define _fat_ptr_plus(arr,elt_sz,change) ({ \
  struct _fat_ptr _ans = (arr); \
  int _change = (change);\
  _ans.curr += (elt_sz) * _change;\
  _ans; })
#define _fat_ptr_inplace_plus(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  _arr_ptr->curr += (elt_sz) * (change);\
  *_arr_ptr; })
#define _fat_ptr_inplace_plus_post(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  struct _fat_ptr _ans = *_arr_ptr; \
  _arr_ptr->curr += (elt_sz) * (change);\
  _ans; })

//Not a macro since initialization order matters. Defined in runtime_zeroterm.c.
struct _fat_ptr _fat_ptr_decrease_size(struct _fat_ptr,unsigned sz,unsigned numelts);

/* Allocation */
void* GC_malloc(int);
void* GC_malloc_atomic(int);
void* GC_calloc(unsigned,unsigned);
void* GC_calloc_atomic(unsigned,unsigned);
// bound the allocation size to be < MAX_ALLOC_SIZE. See macros below for usage.
#define MAX_MALLOC_SIZE (1 << 28)
void* _bounded_GC_malloc(int,const char*,int);
void* _bounded_GC_malloc_atomic(int,const char*,int);
void* _bounded_GC_calloc(unsigned,unsigned,const char*,int);
void* _bounded_GC_calloc_atomic(unsigned,unsigned,const char*,int);
/* these macros are overridden below ifdef CYC_REGION_PROFILE */
#ifndef CYC_REGION_PROFILE
#define _cycalloc(n) _bounded_GC_malloc(n,__FILE__,__LINE__)
#define _cycalloc_atomic(n) _bounded_GC_malloc_atomic(n,__FILE__,__LINE__)
#define _cyccalloc(n,s) _bounded_GC_calloc(n,s,__FILE__,__LINE__)
#define _cyccalloc_atomic(n,s) _bounded_GC_calloc_atomic(n,s,__FILE__,__LINE__)
#endif

static _INLINE unsigned int _check_times(unsigned x, unsigned y) {
  unsigned long long whole_ans = 
    ((unsigned long long) x)*((unsigned long long)y);
  unsigned word_ans = (unsigned)whole_ans;
  if(word_ans < whole_ans || word_ans > MAX_MALLOC_SIZE)
    _throw_badalloc();
  return word_ans;
}

#define _CYC_MAX_REGION_CONST 2
#define _CYC_MIN_ALIGNMENT (sizeof(double))

#ifdef CYC_REGION_PROFILE
extern int rgn_total_bytes;
#endif

static _INLINE void *_fast_region_malloc(struct _RegionHandle *r, unsigned orig_s) {  
  if (r > (struct _RegionHandle *)_CYC_MAX_REGION_CONST && r->curr != 0) { 
#ifdef CYC_NOALIGN
    unsigned s =  orig_s;
#else
    unsigned s =  (orig_s + _CYC_MIN_ALIGNMENT - 1) & (~(_CYC_MIN_ALIGNMENT -1)); 
#endif
    char *result; 
    result = r->offset; 
    if (s <= (r->last_plus_one - result)) {
      r->offset = result + s; 
#ifdef CYC_REGION_PROFILE
    r->curr->free_bytes = r->curr->free_bytes - s;
    rgn_total_bytes += s;
#endif
      return result;
    }
  } 
  return _region_malloc(r,orig_s); 
}

#ifdef CYC_REGION_PROFILE
/* see macros below for usage. defined in runtime_memory.c */
void* _profile_GC_malloc(int,const char*,const char*,int);
void* _profile_GC_malloc_atomic(int,const char*,const char*,int);
void* _profile_GC_calloc(unsigned,unsigned,const char*,const char*,int);
void* _profile_GC_calloc_atomic(unsigned,unsigned,const char*,const char*,int);
void* _profile_region_malloc(struct _RegionHandle*,unsigned,const char*,const char*,int);
void* _profile_region_calloc(struct _RegionHandle*,unsigned,unsigned,const char *,const char*,int);
struct _RegionHandle _profile_new_region(const char*,const char*,const char*,int);
void _profile_free_region(struct _RegionHandle*,const char*,const char*,int);
#ifndef RUNTIME_CYC
#define _new_region(n) _profile_new_region(n,__FILE__,__FUNCTION__,__LINE__)
#define _free_region(r) _profile_free_region(r,__FILE__,__FUNCTION__,__LINE__)
#define _region_malloc(rh,n) _profile_region_malloc(rh,n,__FILE__,__FUNCTION__,__LINE__)
#define _region_calloc(rh,n,t) _profile_region_calloc(rh,n,t,__FILE__,__FUNCTION__,__LINE__)
#  endif
#define _cycalloc(n) _profile_GC_malloc(n,__FILE__,__FUNCTION__,__LINE__)
#define _cycalloc_atomic(n) _profile_GC_malloc_atomic(n,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc(n,s) _profile_GC_calloc(n,s,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc_atomic(n,s) _profile_GC_calloc_atomic(n,s,__FILE__,__FUNCTION__,__LINE__)
#endif
#endif
 struct Cyc___cycFILE;
# 53 "cycboot.h"
extern struct Cyc___cycFILE*Cyc_stderr;struct Cyc_String_pa_PrintArg_struct{int tag;struct _fat_ptr f1;};struct Cyc_Int_pa_PrintArg_struct{int tag;unsigned long f1;};struct Cyc_Double_pa_PrintArg_struct{int tag;double f1;};struct Cyc_LongDouble_pa_PrintArg_struct{int tag;long double f1;};struct Cyc_ShortPtr_pa_PrintArg_struct{int tag;short*f1;};struct Cyc_IntPtr_pa_PrintArg_struct{int tag;unsigned long*f1;};
# 79
int Cyc_fclose(struct Cyc___cycFILE*);
# 88
int Cyc_fflush(struct Cyc___cycFILE*);
# 90
int Cyc_fgetc(struct Cyc___cycFILE*);
# 98
struct Cyc___cycFILE*Cyc_fopen(const char*,const char*);
# 100
int Cyc_fprintf(struct Cyc___cycFILE*,struct _fat_ptr,struct _fat_ptr);struct Cyc_ShortPtr_sa_ScanfArg_struct{int tag;short*f1;};struct Cyc_UShortPtr_sa_ScanfArg_struct{int tag;unsigned short*f1;};struct Cyc_IntPtr_sa_ScanfArg_struct{int tag;int*f1;};struct Cyc_UIntPtr_sa_ScanfArg_struct{int tag;unsigned*f1;};struct Cyc_StringPtr_sa_ScanfArg_struct{int tag;struct _fat_ptr f1;};struct Cyc_DoublePtr_sa_ScanfArg_struct{int tag;double*f1;};struct Cyc_FloatPtr_sa_ScanfArg_struct{int tag;float*f1;};struct Cyc_CharPtr_sa_ScanfArg_struct{int tag;struct _fat_ptr f1;};extern char Cyc_FileCloseError[15U];struct Cyc_FileCloseError_exn_struct{char*tag;};extern char Cyc_FileOpenError[14U];struct Cyc_FileOpenError_exn_struct{char*tag;struct _fat_ptr f1;};
# 271 "cycboot.h"
struct Cyc___cycFILE*Cyc_file_open(struct _fat_ptr,struct _fat_ptr);
# 300
int isspace(int);
# 313
char*getenv(const char*);struct Cyc_Core_Opt{void*v;};extern char Cyc_Core_Invalid_argument[17U];struct Cyc_Core_Invalid_argument_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Failure[8U];struct Cyc_Core_Failure_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Impossible[11U];struct Cyc_Core_Impossible_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Not_found[10U];struct Cyc_Core_Not_found_exn_struct{char*tag;};extern char Cyc_Core_Unreachable[12U];struct Cyc_Core_Unreachable_exn_struct{char*tag;struct _fat_ptr f1;};
# 168 "core.h"
extern struct _RegionHandle*Cyc_Core_unique_region;struct Cyc_Core_DynamicRegion;struct Cyc_Core_NewDynamicRegion{struct Cyc_Core_DynamicRegion*key;};struct Cyc_Core_ThinRes{void*arr;unsigned nelts;};struct Cyc_List_List{void*hd;struct Cyc_List_List*tl;};extern char Cyc_List_List_mismatch[14U];struct Cyc_List_List_mismatch_exn_struct{char*tag;};
# 178 "list.h"
struct Cyc_List_List*Cyc_List_imp_rev(struct Cyc_List_List*x);extern char Cyc_List_Nth[4U];struct Cyc_List_Nth_exn_struct{char*tag;};
# 336
void*Cyc_List_assoc_cmp(int(*cmp)(void*,void*),struct Cyc_List_List*l,void*x);
# 364
struct _fat_ptr Cyc_List_to_array(struct Cyc_List_List*x);
# 38 "string.h"
unsigned long Cyc_strlen(struct _fat_ptr s);
# 49 "string.h"
int Cyc_strcmp(struct _fat_ptr s1,struct _fat_ptr s2);
int Cyc_strptrcmp(struct _fat_ptr*s1,struct _fat_ptr*s2);
int Cyc_strncmp(struct _fat_ptr s1,struct _fat_ptr s2,unsigned long len);
# 62
struct _fat_ptr Cyc_strconcat(struct _fat_ptr,struct _fat_ptr);
# 104 "string.h"
struct _fat_ptr Cyc_strdup(struct _fat_ptr src);
# 109
struct _fat_ptr Cyc_substring(struct _fat_ptr,int ofs,unsigned long n);
# 30 "filename.h"
struct _fat_ptr Cyc_Filename_concat(struct _fat_ptr,struct _fat_ptr);extern char Cyc_Arg_Bad[4U];struct Cyc_Arg_Bad_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Arg_Error[6U];struct Cyc_Arg_Error_exn_struct{char*tag;};struct Cyc_Arg_Unit_spec_Arg_Spec_struct{int tag;void(*f1)();};struct Cyc_Arg_Flag_spec_Arg_Spec_struct{int tag;void(*f1)(struct _fat_ptr);};struct Cyc_Arg_FlagString_spec_Arg_Spec_struct{int tag;void(*f1)(struct _fat_ptr,struct _fat_ptr);};struct Cyc_Arg_Set_spec_Arg_Spec_struct{int tag;int*f1;};struct Cyc_Arg_Clear_spec_Arg_Spec_struct{int tag;int*f1;};struct Cyc_Arg_String_spec_Arg_Spec_struct{int tag;void(*f1)(struct _fat_ptr);};struct Cyc_Arg_Int_spec_Arg_Spec_struct{int tag;void(*f1)(int);};struct Cyc_Arg_Rest_spec_Arg_Spec_struct{int tag;void(*f1)(struct _fat_ptr);};
# 69 "arg.h"
extern int Cyc_Arg_current;
# 71
void Cyc_Arg_parse(struct Cyc_List_List*specs,void(*anonfun)(struct _fat_ptr),int(*anonflagfun)(struct _fat_ptr),struct _fat_ptr errmsg,struct _fat_ptr args);
# 29 "specsfile.h"
struct _fat_ptr Cyc_Specsfile_target_arch;
# 31
struct Cyc_List_List*Cyc_Specsfile_cyclone_exec_path;
# 37
struct Cyc_List_List*Cyc_Specsfile_cyclone_arch_path;
struct _fat_ptr Cyc_Specsfile_def_lib_path;
# 31 "specsfile.cyc"
extern char*Cdef_lib_path;
extern char*Carch;
# 38
struct _fat_ptr Cyc_Specsfile_target_arch={(void*)0,(void*)0,(void*)(0 + 0)};
void Cyc_Specsfile_set_target_arch(struct _fat_ptr s){
Cyc_Specsfile_target_arch=s;}
# 39
struct Cyc_List_List*Cyc_Specsfile_cyclone_exec_path=0;
# 44
void Cyc_Specsfile_add_cyclone_exec_path(struct _fat_ptr s){
unsigned long len=Cyc_strlen((struct _fat_ptr)s);
if(len <= (unsigned long)2)return;{struct _fat_ptr dir=(struct _fat_ptr)
Cyc_substring((struct _fat_ptr)s,2,len - (unsigned long)2);
Cyc_Specsfile_cyclone_exec_path=({struct Cyc_List_List*_tmp1=_cycalloc(sizeof(*_tmp1));({struct _fat_ptr*_tmp56=({struct _fat_ptr*_tmp0=_cycalloc(sizeof(*_tmp0));*_tmp0=dir;_tmp0;});_tmp1->hd=_tmp56;}),_tmp1->tl=Cyc_Specsfile_cyclone_exec_path;_tmp1;});}}struct _tuple0{struct _fat_ptr*f1;struct _fat_ptr*f2;};
# 44
struct Cyc_List_List*Cyc_Specsfile_read_specs(struct _fat_ptr file){
# 58
struct Cyc_List_List*result=0;
int c;
int i;
char strname[256U];
char strvalue[4096U];
struct Cyc___cycFILE*spec_file=Cyc_fopen((const char*)_check_null(_untag_fat_ptr(file,sizeof(char),1U)),"r");
# 65
if(spec_file == 0){
({struct Cyc_String_pa_PrintArg_struct _tmp4=({struct Cyc_String_pa_PrintArg_struct _tmp49;_tmp49.tag=0U,_tmp49.f1=(struct _fat_ptr)((struct _fat_ptr)file);_tmp49;});void*_tmp2[1U];_tmp2[0]=& _tmp4;({struct Cyc___cycFILE*_tmp58=Cyc_stderr;struct _fat_ptr _tmp57=({const char*_tmp3="Error opening spec file %s\n";_tag_fat(_tmp3,sizeof(char),28U);});Cyc_fprintf(_tmp58,_tmp57,_tag_fat(_tmp2,sizeof(void*),1U));});});
Cyc_fflush(Cyc_stderr);
return 0;}
# 65
while(1){
# 72
while(1){
c=Cyc_fgetc(spec_file);
if(isspace(c))continue;if(c == (int)'*')
break;
# 74
if(c != -1){
# 77
({struct Cyc_String_pa_PrintArg_struct _tmp7=({struct Cyc_String_pa_PrintArg_struct _tmp4B;_tmp4B.tag=0U,_tmp4B.f1=(struct _fat_ptr)((struct _fat_ptr)file);_tmp4B;});struct Cyc_Int_pa_PrintArg_struct _tmp8=({struct Cyc_Int_pa_PrintArg_struct _tmp4A;_tmp4A.tag=1U,_tmp4A.f1=(unsigned long)c;_tmp4A;});void*_tmp5[2U];_tmp5[0]=& _tmp7,_tmp5[1]=& _tmp8;({struct Cyc___cycFILE*_tmp5A=Cyc_stderr;struct _fat_ptr _tmp59=({const char*_tmp6="Error reading spec file %s: unexpected character '%c'\n";_tag_fat(_tmp6,sizeof(char),55U);});Cyc_fprintf(_tmp5A,_tmp59,_tag_fat(_tmp5,sizeof(void*),2U));});});
# 80
Cyc_fflush(Cyc_stderr);}
# 74
goto CLEANUP_AND_RETURN;}
# 84
JUST_AFTER_STAR:
 i=0;
while(1){
c=Cyc_fgetc(spec_file);
if(c == -1){
({struct Cyc_String_pa_PrintArg_struct _tmpB=({struct Cyc_String_pa_PrintArg_struct _tmp4C;_tmp4C.tag=0U,_tmp4C.f1=(struct _fat_ptr)((struct _fat_ptr)file);_tmp4C;});void*_tmp9[1U];_tmp9[0]=& _tmpB;({struct Cyc___cycFILE*_tmp5C=Cyc_stderr;struct _fat_ptr _tmp5B=({const char*_tmpA="Error reading spec file %s: unexpected EOF\n";_tag_fat(_tmpA,sizeof(char),44U);});Cyc_fprintf(_tmp5C,_tmp5B,_tag_fat(_tmp9,sizeof(void*),1U));});});
# 92
Cyc_fflush(Cyc_stderr);
goto CLEANUP_AND_RETURN;}
# 88
if(c == (int)':'){
# 96
*((char*)_check_known_subscript_notnull(strname,256U,sizeof(char),i))='\000';
break;}
# 88
*((char*)_check_known_subscript_notnull(strname,256U,sizeof(char),i))=(char)c;
# 100
++ i;
if(i >= 256){
({struct Cyc_String_pa_PrintArg_struct _tmpE=({struct Cyc_String_pa_PrintArg_struct _tmp4D;_tmp4D.tag=0U,_tmp4D.f1=(struct _fat_ptr)((struct _fat_ptr)file);_tmp4D;});void*_tmpC[1U];_tmpC[0]=& _tmpE;({struct Cyc___cycFILE*_tmp5E=Cyc_stderr;struct _fat_ptr _tmp5D=({const char*_tmpD="Error reading spec file %s: string name too long\n";_tag_fat(_tmpD,sizeof(char),50U);});Cyc_fprintf(_tmp5E,_tmp5D,_tag_fat(_tmpC,sizeof(void*),1U));});});
# 105
Cyc_fflush(Cyc_stderr);
goto CLEANUP_AND_RETURN;}}
# 109
while(1){
c=Cyc_fgetc(spec_file);
if(isspace(c))continue;break;}
# 114
if(c == (int)'*'){
# 116
result=({struct Cyc_List_List*_tmp13=_cycalloc(sizeof(*_tmp13));({struct _tuple0*_tmp63=({struct _tuple0*_tmp12=_cycalloc(sizeof(*_tmp12));({struct _fat_ptr*_tmp62=({struct _fat_ptr*_tmpF=_cycalloc(sizeof(*_tmpF));({struct _fat_ptr _tmp61=(struct _fat_ptr)Cyc_strdup(_tag_fat(strname,sizeof(char),256U));*_tmpF=_tmp61;});_tmpF;});_tmp12->f1=_tmp62;}),({
struct _fat_ptr*_tmp60=({struct _fat_ptr*_tmp11=_cycalloc(sizeof(*_tmp11));({struct _fat_ptr _tmp5F=(struct _fat_ptr)Cyc_strdup(({const char*_tmp10="";_tag_fat(_tmp10,sizeof(char),1U);}));*_tmp11=_tmp5F;});_tmp11;});_tmp12->f2=_tmp60;});_tmp12;});
# 116
_tmp13->hd=_tmp63;}),_tmp13->tl=result;_tmp13;});
# 118
goto JUST_AFTER_STAR;}
# 114
strvalue[0]=(char)c;
# 122
i=1;
while(1){
c=Cyc_fgetc(spec_file);
if((c == -1 || c == (int)'\n')|| c == (int)'\r'){*((char*)_check_known_subscript_notnull(strvalue,4096U,sizeof(char),i))='\000';break;}*((char*)_check_known_subscript_notnull(strvalue,4096U,sizeof(char),i))=(char)c;
# 127
++ i;
if(i >= 4096){
({struct Cyc_String_pa_PrintArg_struct _tmp16=({struct Cyc_String_pa_PrintArg_struct _tmp4F;_tmp4F.tag=0U,_tmp4F.f1=(struct _fat_ptr)((struct _fat_ptr)file);_tmp4F;});struct Cyc_String_pa_PrintArg_struct _tmp17=({struct Cyc_String_pa_PrintArg_struct _tmp4E;_tmp4E.tag=0U,({
# 131
struct _fat_ptr _tmp64=(struct _fat_ptr)_tag_fat(strname,sizeof(char),256U);_tmp4E.f1=_tmp64;});_tmp4E;});void*_tmp14[2U];_tmp14[0]=& _tmp16,_tmp14[1]=& _tmp17;({struct Cyc___cycFILE*_tmp66=Cyc_stderr;struct _fat_ptr _tmp65=({const char*_tmp15="Error reading spec file %s: value of %s too long\n";_tag_fat(_tmp15,sizeof(char),50U);});Cyc_fprintf(_tmp66,_tmp65,_tag_fat(_tmp14,sizeof(void*),2U));});});
Cyc_fflush(Cyc_stderr);
goto CLEANUP_AND_RETURN;}}
# 137
result=({struct Cyc_List_List*_tmp1B=_cycalloc(sizeof(*_tmp1B));({struct _tuple0*_tmp6B=({struct _tuple0*_tmp1A=_cycalloc(sizeof(*_tmp1A));({struct _fat_ptr*_tmp6A=({struct _fat_ptr*_tmp18=_cycalloc(sizeof(*_tmp18));({struct _fat_ptr _tmp69=(struct _fat_ptr)Cyc_strdup(_tag_fat(strname,sizeof(char),256U));*_tmp18=_tmp69;});_tmp18;});_tmp1A->f1=_tmp6A;}),({
struct _fat_ptr*_tmp68=({struct _fat_ptr*_tmp19=_cycalloc(sizeof(*_tmp19));({struct _fat_ptr _tmp67=(struct _fat_ptr)Cyc_strdup(_tag_fat(strvalue,sizeof(char),4096U));*_tmp19=_tmp67;});_tmp19;});_tmp1A->f2=_tmp68;});_tmp1A;});
# 137
_tmp1B->hd=_tmp6B;}),_tmp1B->tl=result;_tmp1B;});
# 139
if(c == -1)goto CLEANUP_AND_RETURN;}
# 142
CLEANUP_AND_RETURN:
 Cyc_fclose(spec_file);
return result;}
# 44
struct _fat_ptr Cyc_Specsfile_split_specs(struct _fat_ptr cmdline){
# 149
if(({char*_tmp6C=(char*)cmdline.curr;_tmp6C == (char*)(_tag_fat(0,0,0)).curr;}))return _tag_fat(0,0,0);{unsigned long n=
Cyc_strlen((struct _fat_ptr)cmdline);
struct Cyc_List_List*l=0;
char buf[4096U];
int i=0;
int j=0;
if(n > (unsigned long)4096)goto DONE;while(1){
# 158
while(1){
if((unsigned long)i >= n)goto DONE;if((int)*((const char*)_check_fat_subscript(cmdline,sizeof(char),i))== 0)
goto DONE;
# 159
if(!
# 161
isspace((int)((const char*)cmdline.curr)[i]))break;
# 159
++ i;}
# 164
j=0;
# 169
while(1){
if((unsigned long)i >= n)break;if((int)*((const char*)_check_fat_subscript(cmdline,sizeof(char),i))== 0)
break;
# 170
if(
# 172
isspace((int)((const char*)cmdline.curr)[i]))break;
# 170
if((int)((const char*)cmdline.curr)[i]== (int)'\\'){
# 175
++ i;
if((unsigned long)i >= n)break;if((int)*((const char*)_check_fat_subscript(cmdline,sizeof(char),i))== 0)
break;
# 176
*((char*)_check_known_subscript_notnull(buf,4096U,sizeof(char),j))=((const char*)cmdline.curr)[i];
# 179
++ j;}else{
# 182
*((char*)_check_known_subscript_notnull(buf,4096U,sizeof(char),j))=((const char*)cmdline.curr)[i];
++ j;}
# 185
++ i;}
# 187
if(j < 4096)
*((char*)_check_known_subscript_notnull(buf,4096U,sizeof(char),j))='\000';
# 187
l=({struct Cyc_List_List*_tmp1D=_cycalloc(sizeof(*_tmp1D));
# 190
({struct _fat_ptr*_tmp6E=({struct _fat_ptr*_tmp1C=_cycalloc(sizeof(*_tmp1C));({struct _fat_ptr _tmp6D=(struct _fat_ptr)Cyc_strdup(_tag_fat(buf,sizeof(char),4096U));*_tmp1C=_tmp6D;});_tmp1C;});_tmp1D->hd=_tmp6E;}),_tmp1D->tl=l;_tmp1D;});}
# 192
DONE:
 l=Cyc_List_imp_rev(l);
l=({struct Cyc_List_List*_tmp20=_cycalloc(sizeof(*_tmp20));({struct _fat_ptr*_tmp70=({struct _fat_ptr*_tmp1F=_cycalloc(sizeof(*_tmp1F));({struct _fat_ptr _tmp6F=({const char*_tmp1E="";_tag_fat(_tmp1E,sizeof(char),1U);});*_tmp1F=_tmp6F;});_tmp1F;});_tmp20->hd=_tmp70;}),_tmp20->tl=l;_tmp20;});{
struct _fat_ptr ptrarray=Cyc_List_to_array(l);
struct _fat_ptr result=({unsigned _tmp22=_get_fat_size(ptrarray,sizeof(struct _fat_ptr*));struct _fat_ptr*_tmp21=_cycalloc(_check_times(_tmp22,sizeof(struct _fat_ptr)));({{unsigned _tmp50=_get_fat_size(ptrarray,sizeof(struct _fat_ptr*));unsigned k;for(k=0;k < _tmp50;++ k){_tmp21[k]=*((struct _fat_ptr**)ptrarray.curr)[(int)k];}}0;});_tag_fat(_tmp21,sizeof(struct _fat_ptr),_tmp22);});
return result;}}}
# 44
struct _fat_ptr Cyc_Specsfile_get_spec(struct Cyc_List_List*specs,struct _fat_ptr spec_name){
# 201
struct _handler_cons _tmp23;_push_handler(& _tmp23);{int _tmp25=0;if(setjmp(_tmp23.handler))_tmp25=1;if(!_tmp25){
{struct _fat_ptr _tmp27=*({struct _fat_ptr*(*_tmp71)(int(*cmp)(struct _fat_ptr*,struct _fat_ptr*),struct Cyc_List_List*l,struct _fat_ptr*x)=({struct _fat_ptr*(*_tmp26)(int(*cmp)(struct _fat_ptr*,struct _fat_ptr*),struct Cyc_List_List*l,struct _fat_ptr*x)=(struct _fat_ptr*(*)(int(*cmp)(struct _fat_ptr*,struct _fat_ptr*),struct Cyc_List_List*l,struct _fat_ptr*x))Cyc_List_assoc_cmp;_tmp26;});_tmp71(Cyc_strptrcmp,specs,& spec_name);});_npop_handler(0U);return _tmp27;};_pop_handler();}else{void*_tmp24=(void*)Cyc_Core_get_exn_thrown();void*_tmp28=_tmp24;void*_tmp29;if(((struct Cyc_Core_Not_found_exn_struct*)_tmp28)->tag == Cyc_Core_Not_found){_LL1: _LL2:
# 205
 return _tag_fat(0,0,0);}else{_LL3: _tmp29=_tmp28;_LL4: {void*exn=_tmp29;(int)_rethrow(exn);}}_LL0:;}}}
# 44
struct Cyc_List_List*Cyc_Specsfile_cyclone_arch_path=0;
# 210
struct _fat_ptr Cyc_Specsfile_def_lib_path={(void*)0,(void*)0,(void*)(0 + 0)};
# 214
static struct Cyc_List_List*Cyc_Specsfile_also_subdir(struct Cyc_List_List*dirs,struct _fat_ptr subdir){
# 216
struct Cyc_List_List*l=0;
for(0;dirs != 0;dirs=dirs->tl){
l=({struct Cyc_List_List*_tmp2A=_cycalloc(sizeof(*_tmp2A));_tmp2A->hd=(struct _fat_ptr*)dirs->hd,_tmp2A->tl=l;_tmp2A;});
l=({struct Cyc_List_List*_tmp2C=_cycalloc(sizeof(*_tmp2C));({struct _fat_ptr*_tmp73=({struct _fat_ptr*_tmp2B=_cycalloc(sizeof(*_tmp2B));({struct _fat_ptr _tmp72=(struct _fat_ptr)Cyc_Filename_concat(*((struct _fat_ptr*)dirs->hd),subdir);*_tmp2B=_tmp72;});_tmp2B;});_tmp2C->hd=_tmp73;}),_tmp2C->tl=l;_tmp2C;});}
# 221
l=Cyc_List_imp_rev(l);
return l;}
# 214
struct _fat_ptr Cyc_Specsfile_parse_b(struct Cyc_List_List*specs,void(*anonfun)(struct _fat_ptr),int(*anonflagfun)(struct _fat_ptr),struct _fat_ptr errmsg,struct _fat_ptr argv){
# 244 "specsfile.cyc"
int argc=(int)_get_fat_size(argv,sizeof(struct _fat_ptr));
struct _fat_ptr bindices=({unsigned _tmp3C=(unsigned)argc;int*_tmp3B=_cycalloc_atomic(_check_times(_tmp3C,sizeof(int)));({{unsigned _tmp53=(unsigned)argc;unsigned i;for(i=0;i < _tmp53;++ i){_tmp3B[i]=0;}}0;});_tag_fat(_tmp3B,sizeof(int),_tmp3C);});
int numbindices=0;
int i;int j;int k;
for(i=1;i < argc;++ i){
if(({struct _fat_ptr _tmp74=({const char*_tmp2D="-B";_tag_fat(_tmp2D,sizeof(char),3U);});Cyc_strncmp(_tmp74,(struct _fat_ptr)*((struct _fat_ptr*)_check_fat_subscript(argv,sizeof(struct _fat_ptr),i)),2U);})== 0){
*((int*)_check_fat_subscript(bindices,sizeof(int),i))=1;
++ numbindices;}else{
# 253
if(({struct _fat_ptr _tmp75=({const char*_tmp2E="-b";_tag_fat(_tmp2E,sizeof(char),3U);});Cyc_strcmp(_tmp75,(struct _fat_ptr)((struct _fat_ptr*)argv.curr)[i]);})== 0){
*((int*)_check_fat_subscript(bindices,sizeof(int),i))=1;
++ numbindices;
++ i;if(i >= argc)break;*((int*)_check_fat_subscript(bindices,sizeof(int),i))=1;
# 258
++ numbindices;}}}{
# 265
struct _fat_ptr bargs=({unsigned _tmp3A=(unsigned)(numbindices + 1);struct _fat_ptr*_tmp39=_cycalloc(_check_times(_tmp3A,sizeof(struct _fat_ptr)));({{unsigned _tmp52=(unsigned)(numbindices + 1);unsigned n;for(n=0;n < _tmp52;++ n){({struct _fat_ptr _tmp76=(struct _fat_ptr)_tag_fat(0,0,0);_tmp39[n]=_tmp76;});}}0;});_tag_fat(_tmp39,sizeof(struct _fat_ptr),_tmp3A);});
struct _fat_ptr otherargs=({unsigned _tmp38=(unsigned)(argc - numbindices);struct _fat_ptr*_tmp37=_cycalloc(_check_times(_tmp38,sizeof(struct _fat_ptr)));({{unsigned _tmp51=(unsigned)(argc - numbindices);unsigned n;for(n=0;n < _tmp51;++ n){({struct _fat_ptr _tmp77=(struct _fat_ptr)_tag_fat(0,0,0);_tmp37[n]=_tmp77;});}}0;});_tag_fat(_tmp37,sizeof(struct _fat_ptr),_tmp38);});
({struct _fat_ptr _tmp79=({struct _fat_ptr _tmp78=*((struct _fat_ptr*)_check_fat_subscript(argv,sizeof(struct _fat_ptr),0));*((struct _fat_ptr*)_check_fat_subscript(otherargs,sizeof(struct _fat_ptr),0))=_tmp78;});*((struct _fat_ptr*)_check_fat_subscript(bargs,sizeof(struct _fat_ptr),0))=_tmp79;});
for(i=(j=(k=1));i < argc;++ i){
if(*((int*)_check_fat_subscript(bindices,sizeof(int),i)))({struct _fat_ptr _tmp7A=*((struct _fat_ptr*)_check_fat_subscript(argv,sizeof(struct _fat_ptr),i));*((struct _fat_ptr*)_check_fat_subscript(bargs,sizeof(struct _fat_ptr),j ++))=_tmp7A;});else{
({struct _fat_ptr _tmp7B=*((struct _fat_ptr*)_check_fat_subscript(argv,sizeof(struct _fat_ptr),i));*((struct _fat_ptr*)_check_fat_subscript(otherargs,sizeof(struct _fat_ptr),k ++))=_tmp7B;});}}
# 273
Cyc_Arg_current=0;
Cyc_Arg_parse(specs,anonfun,anonflagfun,errmsg,bargs);
# 278
if(({char*_tmp7C=(char*)Cyc_Specsfile_target_arch.curr;_tmp7C == (char*)(_tag_fat(0,0,0)).curr;}))Cyc_Specsfile_target_arch=({char*_tmp2F=Carch;_tag_fat(_tmp2F,sizeof(char),_get_zero_arr_size_char((void*)_tmp2F,1U));});{struct _fat_ptr cyclone_exec_prefix=({char*_tmp36=
# 287 "specsfile.cyc"
getenv("CYCLONE_EXEC_PREFIX");_tag_fat(_tmp36,sizeof(char),_get_zero_arr_size_char((void*)_tmp36,1U));});
if(({char*_tmp7D=(char*)cyclone_exec_prefix.curr;_tmp7D != (char*)(_tag_fat(0,0,0)).curr;}))
Cyc_Specsfile_cyclone_exec_path=({struct Cyc_List_List*_tmp31=_cycalloc(sizeof(*_tmp31));({struct _fat_ptr*_tmp7E=({struct _fat_ptr*_tmp30=_cycalloc(sizeof(*_tmp30));*_tmp30=cyclone_exec_prefix;_tmp30;});_tmp31->hd=_tmp7E;}),_tmp31->tl=Cyc_Specsfile_cyclone_exec_path;_tmp31;});
# 288
Cyc_Specsfile_def_lib_path=({char*_tmp32=Cdef_lib_path;_tag_fat(_tmp32,sizeof(char),_get_zero_arr_size_char((void*)_tmp32,1U));});
# 291
if(Cyc_strlen((struct _fat_ptr)Cyc_Specsfile_def_lib_path)> (unsigned long)0)
Cyc_Specsfile_cyclone_exec_path=({struct Cyc_List_List*_tmp35=_cycalloc(sizeof(*_tmp35));
({struct _fat_ptr*_tmp81=({struct _fat_ptr*_tmp34=_cycalloc(sizeof(*_tmp34));({struct _fat_ptr _tmp80=(struct _fat_ptr)({struct _fat_ptr _tmp7F=Cyc_Specsfile_def_lib_path;Cyc_Filename_concat(_tmp7F,({const char*_tmp33="cyc-lib";_tag_fat(_tmp33,sizeof(char),8U);}));});*_tmp34=_tmp80;});_tmp34;});_tmp35->hd=_tmp81;}),_tmp35->tl=Cyc_Specsfile_cyclone_exec_path;_tmp35;});
# 291
Cyc_Specsfile_cyclone_exec_path=
# 295
Cyc_List_imp_rev(Cyc_Specsfile_cyclone_exec_path);
Cyc_Specsfile_cyclone_arch_path=Cyc_Specsfile_also_subdir(Cyc_Specsfile_cyclone_exec_path,Cyc_Specsfile_target_arch);
return otherargs;}}}
# 214 "specsfile.cyc"
static int Cyc_Specsfile_file_exists(struct _fat_ptr file){
# 302 "specsfile.cyc"
struct Cyc___cycFILE*f=0;
{struct _handler_cons _tmp3D;_push_handler(& _tmp3D);{int _tmp3F=0;if(setjmp(_tmp3D.handler))_tmp3F=1;if(!_tmp3F){f=({struct _fat_ptr _tmp82=file;Cyc_file_open(_tmp82,({const char*_tmp40="r";_tag_fat(_tmp40,sizeof(char),2U);}));});;_pop_handler();}else{void*_tmp3E=(void*)Cyc_Core_get_exn_thrown();void*_tmp41=_tmp3E;_LL1: _LL2: goto _LL0;_LL0:;}}}
if(f == 0)return 0;else{
Cyc_fclose(f);return 1;}}
# 214 "specsfile.cyc"
static struct _fat_ptr*Cyc_Specsfile_find(struct Cyc_List_List*dirs,struct _fat_ptr file){
# 311 "specsfile.cyc"
if(({char*_tmp83=(char*)file.curr;_tmp83 == (char*)(_tag_fat(0,0,0)).curr;}))return 0;for(0;dirs != 0;dirs=dirs->tl){
# 313
struct _fat_ptr dir=*((struct _fat_ptr*)dirs->hd);
if(({char*_tmp84=(char*)dir.curr;_tmp84 == (char*)(_tag_fat(0,0,0)).curr;})|| Cyc_strlen((struct _fat_ptr)dir)== (unsigned long)0)continue;{struct _fat_ptr s=(struct _fat_ptr)
Cyc_Filename_concat(dir,file);
if(Cyc_Specsfile_file_exists(s))return({struct _fat_ptr*_tmp42=_cycalloc(sizeof(*_tmp42));*_tmp42=s;_tmp42;});}}
# 318
return 0;}
# 214 "specsfile.cyc"
static struct _fat_ptr Cyc_Specsfile_sprint_list(struct Cyc_List_List*dirs){
# 323 "specsfile.cyc"
struct _fat_ptr tmp=({const char*_tmp44="";_tag_fat(_tmp44,sizeof(char),1U);});
for(0;dirs != 0;dirs=dirs->tl){
struct _fat_ptr dir=*((struct _fat_ptr*)dirs->hd);
if(({char*_tmp85=(char*)dir.curr;_tmp85 == (char*)(_tag_fat(0,0,0)).curr;})|| Cyc_strlen((struct _fat_ptr)dir)== (unsigned long)0)continue;dir=(struct _fat_ptr)({
struct _fat_ptr _tmp86=(struct _fat_ptr)dir;Cyc_strconcat(_tmp86,({const char*_tmp43=":";_tag_fat(_tmp43,sizeof(char),2U);}));});
tmp=(struct _fat_ptr)Cyc_strconcat((struct _fat_ptr)dir,(struct _fat_ptr)tmp);}
# 330
return tmp;}
# 214 "specsfile.cyc"
static struct _fat_ptr Cyc_Specsfile_do_find(struct Cyc_List_List*dirs,struct _fat_ptr file){
# 334 "specsfile.cyc"
struct _fat_ptr*f=Cyc_Specsfile_find(dirs,file);
if(f == 0){
({struct Cyc_String_pa_PrintArg_struct _tmp47=({struct Cyc_String_pa_PrintArg_struct _tmp55;_tmp55.tag=0U,_tmp55.f1=(struct _fat_ptr)((struct _fat_ptr)file);_tmp55;});struct Cyc_String_pa_PrintArg_struct _tmp48=({struct Cyc_String_pa_PrintArg_struct _tmp54;_tmp54.tag=0U,({
struct _fat_ptr _tmp87=(struct _fat_ptr)((struct _fat_ptr)Cyc_Specsfile_sprint_list(dirs));_tmp54.f1=_tmp87;});_tmp54;});void*_tmp45[2U];_tmp45[0]=& _tmp47,_tmp45[1]=& _tmp48;({struct Cyc___cycFILE*_tmp89=Cyc_stderr;struct _fat_ptr _tmp88=({const char*_tmp46="Error: can't find internal compiler file %s in path %s\n";_tag_fat(_tmp46,sizeof(char),56U);});Cyc_fprintf(_tmp89,_tmp88,_tag_fat(_tmp45,sizeof(void*),2U));});});
Cyc_fflush(Cyc_stderr);
return _tag_fat(0,0,0);}
# 335
return*f;}
# 214 "specsfile.cyc"
struct _fat_ptr Cyc_Specsfile_find_in_arch_path(struct _fat_ptr s){
# 345 "specsfile.cyc"
return Cyc_Specsfile_do_find(Cyc_Specsfile_cyclone_arch_path,s);}
# 214 "specsfile.cyc"
struct _fat_ptr Cyc_Specsfile_find_in_exec_path(struct _fat_ptr s){
# 349 "specsfile.cyc"
return Cyc_Specsfile_do_find(Cyc_Specsfile_cyclone_exec_path,s);}
