#include <setjmp.h>
/* This is a C header file to be used by the output of the Cyclone to
   C translator.  The corresponding definitions are in file lib/runtime_*.c */
#ifndef _CYC_INCLUDE_H_
#define _CYC_INCLUDE_H_

/* Need one of these per thread (see runtime_stack.c). The runtime maintains 
   a stack that contains either _handler_cons structs or _RegionHandle structs.
   The tag is 0 for a handler_cons and 1 for a region handle.  */
struct _RuntimeStack {
  int tag; 
  struct _RuntimeStack *next;
  void (*cleanup)(struct _RuntimeStack *frame);
};

#ifndef offsetof
/* should be size_t, but int is fine. */
#define offsetof(t,n) ((int)(&(((t *)0)->n)))
#endif

/* Fat pointers */
struct _fat_ptr {
  unsigned char *curr; 
  unsigned char *base; 
  unsigned char *last_plus_one; 
};  

/* Discriminated Unions */
struct _xtunion_struct { char *tag; };

/* Regions */
struct _RegionPage
#ifdef CYC_REGION_PROFILE
{ unsigned total_bytes;
  unsigned free_bytes;
  /* MWH: wish we didn't have to include the stuff below ... */
  struct _RegionPage *next;
  char data[1];
}
#endif
; // abstract -- defined in runtime_memory.c
struct _RegionHandle {
  struct _RuntimeStack s;
  struct _RegionPage *curr;
  char               *offset;
  char               *last_plus_one;
  struct _DynRegionHandle *sub_regions;
#ifdef CYC_REGION_PROFILE
  const char         *name;
#else
  unsigned used_bytes;
  unsigned wasted_bytes;
#endif
};
struct _DynRegionFrame {
  struct _RuntimeStack s;
  struct _DynRegionHandle *x;
};
// A dynamic region is just a region handle.  The wrapper struct is for type
// abstraction.
struct Cyc_Core_DynamicRegion {
  struct _RegionHandle h;
};

struct _RegionHandle _new_region(const char*);
void* _region_malloc(struct _RegionHandle*, unsigned);
void* _region_calloc(struct _RegionHandle*, unsigned t, unsigned n);
void   _free_region(struct _RegionHandle*);
struct _RegionHandle*_open_dynregion(struct _DynRegionFrame*,struct _DynRegionHandle*);
void   _pop_dynregion();

/* Exceptions */
struct _handler_cons {
  struct _RuntimeStack s;
  jmp_buf handler;
};
void _push_handler(struct _handler_cons *);
void _push_region(struct _RegionHandle *);
void _npop_handler(int);
void _pop_handler();
void _pop_region();

#ifndef _throw
void* _throw_null_fn(const char*,unsigned);
void* _throw_arraybounds_fn(const char*,unsigned);
void* _throw_badalloc_fn(const char*,unsigned);
void* _throw_match_fn(const char*,unsigned);
void* _throw_fn(void*,const char*,unsigned);
void* _rethrow(void*);
#define _throw_null() (_throw_null_fn(__FILE__,__LINE__))
#define _throw_arraybounds() (_throw_arraybounds_fn(__FILE__,__LINE__))
#define _throw_badalloc() (_throw_badalloc_fn(__FILE__,__LINE__))
#define _throw_match() (_throw_match_fn(__FILE__,__LINE__))
#define _throw(e) (_throw_fn((e),__FILE__,__LINE__))
#endif

struct _xtunion_struct* Cyc_Core_get_exn_thrown();
/* Built-in Exceptions */
struct Cyc_Null_Exception_exn_struct { char *tag; };
struct Cyc_Array_bounds_exn_struct { char *tag; };
struct Cyc_Match_Exception_exn_struct { char *tag; };
struct Cyc_Bad_alloc_exn_struct { char *tag; };
extern char Cyc_Null_Exception[];
extern char Cyc_Array_bounds[];
extern char Cyc_Match_Exception[];
extern char Cyc_Bad_alloc[];

/* Built-in Run-time Checks and company */
#ifdef CYC_ANSI_OUTPUT
#define _INLINE  
#else
#define _INLINE inline
#endif

#ifdef NO_CYC_NULL_CHECKS
#define _check_null(ptr) (ptr)
#else
#define _check_null(ptr) \
  ({ void*_cks_null = (void*)(ptr); \
     if (!_cks_null) _throw_null(); \
     _cks_null; })
#endif

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index)\
   (((char*)ptr) + (elt_sz)*(index))
#ifdef NO_CYC_NULL_CHECKS
#define _check_known_subscript_null _check_known_subscript_notnull
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr);\
  int _index = (index);\
  if (!_cks_ptr) _throw_null(); \
  _cks_ptr + (elt_sz)*_index; })
#endif
#define _zero_arr_plus_fn(orig_x,orig_sz,orig_i,f,l) ((orig_x)+(orig_i))
#define _zero_arr_plus_char_fn _zero_arr_plus_fn
#define _zero_arr_plus_short_fn _zero_arr_plus_fn
#define _zero_arr_plus_int_fn _zero_arr_plus_fn
#define _zero_arr_plus_float_fn _zero_arr_plus_fn
#define _zero_arr_plus_double_fn _zero_arr_plus_fn
#define _zero_arr_plus_longdouble_fn _zero_arr_plus_fn
#define _zero_arr_plus_voidstar_fn _zero_arr_plus_fn
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (!_cks_ptr) _throw_null(); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })

/* _zero_arr_plus_*_fn(x,sz,i,filename,lineno) adds i to zero-terminated ptr
   x that has at least sz elements */
char* _zero_arr_plus_char_fn(char*,unsigned,int,const char*,unsigned);
short* _zero_arr_plus_short_fn(short*,unsigned,int,const char*,unsigned);
int* _zero_arr_plus_int_fn(int*,unsigned,int,const char*,unsigned);
float* _zero_arr_plus_float_fn(float*,unsigned,int,const char*,unsigned);
double* _zero_arr_plus_double_fn(double*,unsigned,int,const char*,unsigned);
long double* _zero_arr_plus_longdouble_fn(long double*,unsigned,int,const char*, unsigned);
void** _zero_arr_plus_voidstar_fn(void**,unsigned,int,const char*,unsigned);
#endif

/* _get_zero_arr_size_*(x,sz) returns the number of elements in a
   zero-terminated array that is NULL or has at least sz elements */
int _get_zero_arr_size_char(const char*,unsigned);
int _get_zero_arr_size_short(const short*,unsigned);
int _get_zero_arr_size_int(const int*,unsigned);
int _get_zero_arr_size_float(const float*,unsigned);
int _get_zero_arr_size_double(const double*,unsigned);
int _get_zero_arr_size_longdouble(const long double*,unsigned);
int _get_zero_arr_size_voidstar(const void**,unsigned);

/* _zero_arr_inplace_plus_*_fn(x,i,filename,lineno) sets
   zero-terminated pointer *x to *x + i */
char* _zero_arr_inplace_plus_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_short_fn(short**,int,const char*,unsigned);
int* _zero_arr_inplace_plus_int(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_longdouble_fn(long double**,int,const char*,unsigned);
void** _zero_arr_inplace_plus_voidstar_fn(void***,int,const char*,unsigned);
/* like the previous functions, but does post-addition (as in e++) */
char* _zero_arr_inplace_plus_post_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_post_short_fn(short**x,int,const char*,unsigned);
int* _zero_arr_inplace_plus_post_int_fn(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_post_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_post_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_post_longdouble_fn(long double**,int,const char *,unsigned);
void** _zero_arr_inplace_plus_post_voidstar_fn(void***,int,const char*,unsigned);
#define _zero_arr_plus_char(x,s,i) \
  (_zero_arr_plus_char_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_short(x,s,i) \
  (_zero_arr_plus_short_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_int(x,s,i) \
  (_zero_arr_plus_int_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_float(x,s,i) \
  (_zero_arr_plus_float_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_double(x,s,i) \
  (_zero_arr_plus_double_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_longdouble(x,s,i) \
  (_zero_arr_plus_longdouble_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_voidstar(x,s,i) \
  (_zero_arr_plus_voidstar_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_inplace_plus_char(x,i) \
  _zero_arr_inplace_plus_char_fn((char **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_short(x,i) \
  _zero_arr_inplace_plus_short_fn((short **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_int(x,i) \
  _zero_arr_inplace_plus_int_fn((int **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_float(x,i) \
  _zero_arr_inplace_plus_float_fn((float **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_double(x,i) \
  _zero_arr_inplace_plus_double_fn((double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_longdouble(x,i) \
  _zero_arr_inplace_plus_longdouble_fn((long double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_voidstar(x,i) \
  _zero_arr_inplace_plus_voidstar_fn((void ***)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_char(x,i) \
  _zero_arr_inplace_plus_post_char_fn((char **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_short(x,i) \
  _zero_arr_inplace_plus_post_short_fn((short **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_int(x,i) \
  _zero_arr_inplace_plus_post_int_fn((int **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_float(x,i) \
  _zero_arr_inplace_plus_post_float_fn((float **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_double(x,i) \
  _zero_arr_inplace_plus_post_double_fn((double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_longdouble(x,i) \
  _zero_arr_inplace_plus_post_longdouble_fn((long double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_voidstar(x,i) \
  _zero_arr_inplace_plus_post_voidstar_fn((void***)(x),(i),__FILE__,__LINE__)

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_fat_subscript(arr,elt_sz,index) ((arr).curr + (elt_sz) * (index))
#define _untag_fat_ptr(arr,elt_sz,num_elts) ((arr).curr)
#else
#define _check_fat_subscript(arr,elt_sz,index) ({ \
  struct _fat_ptr _cus_arr = (arr); \
  unsigned char *_cus_ans = _cus_arr.curr + (elt_sz) * (index); \
  /* JGM: not needed! if (!_cus_arr.base) _throw_null();*/ \
  if (_cus_ans < _cus_arr.base || _cus_ans >= _cus_arr.last_plus_one) \
    _throw_arraybounds(); \
  _cus_ans; })
#define _untag_fat_ptr(arr,elt_sz,num_elts) ({ \
  struct _fat_ptr _arr = (arr); \
  unsigned char *_curr = _arr.curr; \
  if ((_curr < _arr.base || _curr + (elt_sz) * (num_elts) > _arr.last_plus_one) &&\
      _curr != (unsigned char *)0) \
    _throw_arraybounds(); \
  _curr; })
#endif

#define _tag_fat(tcurr,elt_sz,num_elts) ({ \
  struct _fat_ptr _ans; \
  unsigned _num_elts = (num_elts);\
  _ans.base = _ans.curr = (void*)(tcurr); \
  /* JGM: if we're tagging NULL, ignore num_elts */ \
  _ans.last_plus_one = _ans.base ? (_ans.base + (elt_sz) * _num_elts) : 0; \
  _ans; })

#define _get_fat_size(arr,elt_sz) \
  ({struct _fat_ptr _arr = (arr); \
    unsigned char *_arr_curr=_arr.curr; \
    unsigned char *_arr_last=_arr.last_plus_one; \
    (_arr_curr < _arr.base || _arr_curr >= _arr_last) ? 0 : \
    ((_arr_last - _arr_curr) / (elt_sz));})

#define _fat_ptr_plus(arr,elt_sz,change) ({ \
  struct _fat_ptr _ans = (arr); \
  int _change = (change);\
  _ans.curr += (elt_sz) * _change;\
  _ans; })
#define _fat_ptr_inplace_plus(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  _arr_ptr->curr += (elt_sz) * (change);\
  *_arr_ptr; })
#define _fat_ptr_inplace_plus_post(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  struct _fat_ptr _ans = *_arr_ptr; \
  _arr_ptr->curr += (elt_sz) * (change);\
  _ans; })

//Not a macro since initialization order matters. Defined in runtime_zeroterm.c.
struct _fat_ptr _fat_ptr_decrease_size(struct _fat_ptr,unsigned sz,unsigned numelts);

/* Allocation */
void* GC_malloc(int);
void* GC_malloc_atomic(int);
void* GC_calloc(unsigned,unsigned);
void* GC_calloc_atomic(unsigned,unsigned);
// bound the allocation size to be < MAX_ALLOC_SIZE. See macros below for usage.
#define MAX_MALLOC_SIZE (1 << 28)
void* _bounded_GC_malloc(int,const char*,int);
void* _bounded_GC_malloc_atomic(int,const char*,int);
void* _bounded_GC_calloc(unsigned,unsigned,const char*,int);
void* _bounded_GC_calloc_atomic(unsigned,unsigned,const char*,int);
/* these macros are overridden below ifdef CYC_REGION_PROFILE */
#ifndef CYC_REGION_PROFILE
#define _cycalloc(n) _bounded_GC_malloc(n,__FILE__,__LINE__)
#define _cycalloc_atomic(n) _bounded_GC_malloc_atomic(n,__FILE__,__LINE__)
#define _cyccalloc(n,s) _bounded_GC_calloc(n,s,__FILE__,__LINE__)
#define _cyccalloc_atomic(n,s) _bounded_GC_calloc_atomic(n,s,__FILE__,__LINE__)
#endif

static _INLINE unsigned int _check_times(unsigned x, unsigned y) {
  unsigned long long whole_ans = 
    ((unsigned long long) x)*((unsigned long long)y);
  unsigned word_ans = (unsigned)whole_ans;
  if(word_ans < whole_ans || word_ans > MAX_MALLOC_SIZE)
    _throw_badalloc();
  return word_ans;
}

#define _CYC_MAX_REGION_CONST 2
#define _CYC_MIN_ALIGNMENT (sizeof(double))

#ifdef CYC_REGION_PROFILE
extern int rgn_total_bytes;
#endif

static _INLINE void *_fast_region_malloc(struct _RegionHandle *r, unsigned orig_s) {  
  if (r > (struct _RegionHandle *)_CYC_MAX_REGION_CONST && r->curr != 0) { 
#ifdef CYC_NOALIGN
    unsigned s =  orig_s;
#else
    unsigned s =  (orig_s + _CYC_MIN_ALIGNMENT - 1) & (~(_CYC_MIN_ALIGNMENT -1)); 
#endif
    char *result; 
    result = r->offset; 
    if (s <= (r->last_plus_one - result)) {
      r->offset = result + s; 
#ifdef CYC_REGION_PROFILE
    r->curr->free_bytes = r->curr->free_bytes - s;
    rgn_total_bytes += s;
#endif
      return result;
    }
  } 
  return _region_malloc(r,orig_s); 
}

#ifdef CYC_REGION_PROFILE
/* see macros below for usage. defined in runtime_memory.c */
void* _profile_GC_malloc(int,const char*,const char*,int);
void* _profile_GC_malloc_atomic(int,const char*,const char*,int);
void* _profile_GC_calloc(unsigned,unsigned,const char*,const char*,int);
void* _profile_GC_calloc_atomic(unsigned,unsigned,const char*,const char*,int);
void* _profile_region_malloc(struct _RegionHandle*,unsigned,const char*,const char*,int);
void* _profile_region_calloc(struct _RegionHandle*,unsigned,unsigned,const char *,const char*,int);
struct _RegionHandle _profile_new_region(const char*,const char*,const char*,int);
void _profile_free_region(struct _RegionHandle*,const char*,const char*,int);
#ifndef RUNTIME_CYC
#define _new_region(n) _profile_new_region(n,__FILE__,__FUNCTION__,__LINE__)
#define _free_region(r) _profile_free_region(r,__FILE__,__FUNCTION__,__LINE__)
#define _region_malloc(rh,n) _profile_region_malloc(rh,n,__FILE__,__FUNCTION__,__LINE__)
#define _region_calloc(rh,n,t) _profile_region_calloc(rh,n,t,__FILE__,__FUNCTION__,__LINE__)
#  endif
#define _cycalloc(n) _profile_GC_malloc(n,__FILE__,__FUNCTION__,__LINE__)
#define _cycalloc_atomic(n) _profile_GC_malloc_atomic(n,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc(n,s) _profile_GC_calloc(n,s,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc_atomic(n,s) _profile_GC_calloc_atomic(n,s,__FILE__,__FUNCTION__,__LINE__)
#endif
#endif
 struct Cyc___cycFILE;
# 53 "cycboot.h"
extern struct Cyc___cycFILE*Cyc_stderr;struct Cyc_String_pa_PrintArg_struct{int tag;struct _fat_ptr f1;};struct Cyc_Int_pa_PrintArg_struct{int tag;unsigned long f1;};struct Cyc_Double_pa_PrintArg_struct{int tag;double f1;};struct Cyc_LongDouble_pa_PrintArg_struct{int tag;long double f1;};struct Cyc_ShortPtr_pa_PrintArg_struct{int tag;short*f1;};struct Cyc_IntPtr_pa_PrintArg_struct{int tag;unsigned long*f1;};
# 73
struct _fat_ptr Cyc_aprintf(struct _fat_ptr,struct _fat_ptr);
# 100
int Cyc_fprintf(struct Cyc___cycFILE*,struct _fat_ptr,struct _fat_ptr);struct Cyc_ShortPtr_sa_ScanfArg_struct{int tag;short*f1;};struct Cyc_UShortPtr_sa_ScanfArg_struct{int tag;unsigned short*f1;};struct Cyc_IntPtr_sa_ScanfArg_struct{int tag;int*f1;};struct Cyc_UIntPtr_sa_ScanfArg_struct{int tag;unsigned*f1;};struct Cyc_StringPtr_sa_ScanfArg_struct{int tag;struct _fat_ptr f1;};struct Cyc_DoublePtr_sa_ScanfArg_struct{int tag;double*f1;};struct Cyc_FloatPtr_sa_ScanfArg_struct{int tag;float*f1;};struct Cyc_CharPtr_sa_ScanfArg_struct{int tag;struct _fat_ptr f1;};
# 232 "cycboot.h"
struct _fat_ptr Cyc_vrprintf(struct _RegionHandle*,struct _fat_ptr,struct _fat_ptr);extern char Cyc_FileCloseError[15U];struct Cyc_FileCloseError_exn_struct{char*tag;};extern char Cyc_FileOpenError[14U];struct Cyc_FileOpenError_exn_struct{char*tag;struct _fat_ptr f1;};struct Cyc_Core_Opt{void*v;};extern char Cyc_Core_Invalid_argument[17U];struct Cyc_Core_Invalid_argument_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Failure[8U];struct Cyc_Core_Failure_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Impossible[11U];struct Cyc_Core_Impossible_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Not_found[10U];struct Cyc_Core_Not_found_exn_struct{char*tag;};extern char Cyc_Core_Unreachable[12U];struct Cyc_Core_Unreachable_exn_struct{char*tag;struct _fat_ptr f1;};
# 165 "core.h"
extern struct _RegionHandle*Cyc_Core_heap_region;
# 168
extern struct _RegionHandle*Cyc_Core_unique_region;struct Cyc_Core_DynamicRegion;struct Cyc_Core_NewDynamicRegion{struct Cyc_Core_DynamicRegion*key;};struct Cyc_Core_ThinRes{void*arr;unsigned nelts;};struct Cyc_List_List{void*hd;struct Cyc_List_List*tl;};
# 61 "list.h"
int Cyc_List_length(struct Cyc_List_List*x);
# 83
struct Cyc_List_List*Cyc_List_map_c(void*(*f)(void*,void*),void*env,struct Cyc_List_List*x);extern char Cyc_List_List_mismatch[14U];struct Cyc_List_List_mismatch_exn_struct{char*tag;};
# 184
struct Cyc_List_List*Cyc_List_append(struct Cyc_List_List*x,struct Cyc_List_List*y);extern char Cyc_List_Nth[4U];struct Cyc_List_Nth_exn_struct{char*tag;};struct Cyc_Iter_Iter{void*env;int(*next)(void*env,void*dest);};struct Cyc_Dict_T;struct Cyc_Dict_Dict{int(*rel)(void*,void*);struct _RegionHandle*r;const struct Cyc_Dict_T*t;};extern char Cyc_Dict_Present[8U];struct Cyc_Dict_Present_exn_struct{char*tag;};extern char Cyc_Dict_Absent[7U];struct Cyc_Dict_Absent_exn_struct{char*tag;};
# 38 "string.h"
unsigned long Cyc_strlen(struct _fat_ptr s);
# 50 "string.h"
int Cyc_strptrcmp(struct _fat_ptr*s1,struct _fat_ptr*s2);
# 62
struct _fat_ptr Cyc_strconcat(struct _fat_ptr,struct _fat_ptr);struct Cyc_Position_Error;
# 43 "position.h"
struct Cyc_Position_Error*Cyc_Position_mk_err(unsigned,struct _fat_ptr);
# 49
void Cyc_Position_post_error(struct Cyc_Position_Error*);struct Cyc_Relations_Reln;struct _union_Nmspace_Rel_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_Abs_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_C_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_Loc_n{int tag;int val;};union Cyc_Absyn_Nmspace{struct _union_Nmspace_Rel_n Rel_n;struct _union_Nmspace_Abs_n Abs_n;struct _union_Nmspace_C_n C_n;struct _union_Nmspace_Loc_n Loc_n;};struct _tuple0{union Cyc_Absyn_Nmspace f1;struct _fat_ptr*f2;};
# 161 "absyn.h"
enum Cyc_Absyn_Scope{Cyc_Absyn_Static =0U,Cyc_Absyn_Abstract =1U,Cyc_Absyn_Public =2U,Cyc_Absyn_Extern =3U,Cyc_Absyn_ExternC =4U,Cyc_Absyn_Register =5U};struct Cyc_Absyn_Tqual{int print_const: 1;int q_volatile: 1;int q_restrict: 1;int real_const: 1;unsigned loc;};
# 182
enum Cyc_Absyn_Size_of{Cyc_Absyn_Char_sz =0U,Cyc_Absyn_Short_sz =1U,Cyc_Absyn_Int_sz =2U,Cyc_Absyn_Long_sz =3U,Cyc_Absyn_LongLong_sz =4U};
enum Cyc_Absyn_Sign{Cyc_Absyn_Signed =0U,Cyc_Absyn_Unsigned =1U,Cyc_Absyn_None =2U};
enum Cyc_Absyn_AggrKind{Cyc_Absyn_StructA =0U,Cyc_Absyn_UnionA =1U};
# 187
enum Cyc_Absyn_AliasQual{Cyc_Absyn_Aliasable =0U,Cyc_Absyn_Unique =1U,Cyc_Absyn_Top =2U};
# 192
enum Cyc_Absyn_KindQual{Cyc_Absyn_AnyKind =0U,Cyc_Absyn_MemKind =1U,Cyc_Absyn_BoxKind =2U,Cyc_Absyn_RgnKind =3U,Cyc_Absyn_EffKind =4U,Cyc_Absyn_IntKind =5U,Cyc_Absyn_BoolKind =6U,Cyc_Absyn_PtrBndKind =7U};struct Cyc_Absyn_Kind{enum Cyc_Absyn_KindQual kind;enum Cyc_Absyn_AliasQual aliasqual;};struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct{int tag;struct Cyc_Absyn_Kind*f1;};struct Cyc_Absyn_Unknown_kb_Absyn_KindBound_struct{int tag;struct Cyc_Core_Opt*f1;};struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct{int tag;struct Cyc_Core_Opt*f1;struct Cyc_Absyn_Kind*f2;};struct Cyc_Absyn_Tvar{struct _fat_ptr*name;int identity;void*kind;};struct Cyc_Absyn_PtrLoc{unsigned ptr_loc;unsigned rgn_loc;unsigned zt_loc;};struct Cyc_Absyn_PtrAtts{void*rgn;void*nullable;void*bounds;void*zero_term;struct Cyc_Absyn_PtrLoc*ptrloc;};struct Cyc_Absyn_PtrInfo{void*elt_type;struct Cyc_Absyn_Tqual elt_tq;struct Cyc_Absyn_PtrAtts ptr_atts;};struct Cyc_Absyn_VarargInfo{struct _fat_ptr*name;struct Cyc_Absyn_Tqual tq;void*type;int inject;};struct Cyc_Absyn_Star_Absyn_Cap_struct{int tag;};struct Cyc_Absyn_Bot_Absyn_Cap_struct{int tag;};struct Cyc_Absyn_Nat_Absyn_Cap_struct{int tag;struct Cyc_Absyn_Exp*f1;int f2;};struct Cyc_Absyn_RgnEffect{void*name;void*rgn;void*lock;void*unadopt;void*parent;};struct Cyc_Absyn_FnInfo{struct Cyc_List_List*tvars;void*effect;struct Cyc_Absyn_Tqual ret_tqual;void*ret_type;struct Cyc_List_List*args;int c_varargs;struct Cyc_Absyn_VarargInfo*cyc_varargs;struct Cyc_List_List*rgn_po;struct Cyc_List_List*attributes;struct Cyc_Absyn_Exp*requires_clause;struct Cyc_List_List*requires_relns;struct Cyc_Absyn_Exp*ensures_clause;struct Cyc_List_List*ensures_relns;struct Cyc_List_List*ieffect;struct Cyc_List_List*oeffect;};struct Cyc_Absyn_UnknownDatatypeInfo{struct _tuple0*name;int is_extensible;};struct _union_DatatypeInfo_UnknownDatatype{int tag;struct Cyc_Absyn_UnknownDatatypeInfo val;};struct _union_DatatypeInfo_KnownDatatype{int tag;struct Cyc_Absyn_Datatypedecl**val;};union Cyc_Absyn_DatatypeInfo{struct _union_DatatypeInfo_UnknownDatatype UnknownDatatype;struct _union_DatatypeInfo_KnownDatatype KnownDatatype;};struct Cyc_Absyn_UnknownDatatypeFieldInfo{struct _tuple0*datatype_name;struct _tuple0*field_name;int is_extensible;};struct _union_DatatypeFieldInfo_UnknownDatatypefield{int tag;struct Cyc_Absyn_UnknownDatatypeFieldInfo val;};struct _tuple1{struct Cyc_Absyn_Datatypedecl*f1;struct Cyc_Absyn_Datatypefield*f2;};struct _union_DatatypeFieldInfo_KnownDatatypefield{int tag;struct _tuple1 val;};union Cyc_Absyn_DatatypeFieldInfo{struct _union_DatatypeFieldInfo_UnknownDatatypefield UnknownDatatypefield;struct _union_DatatypeFieldInfo_KnownDatatypefield KnownDatatypefield;};struct _tuple2{enum Cyc_Absyn_AggrKind f1;struct _tuple0*f2;struct Cyc_Core_Opt*f3;};struct _union_AggrInfo_UnknownAggr{int tag;struct _tuple2 val;};struct _union_AggrInfo_KnownAggr{int tag;struct Cyc_Absyn_Aggrdecl**val;};union Cyc_Absyn_AggrInfo{struct _union_AggrInfo_UnknownAggr UnknownAggr;struct _union_AggrInfo_KnownAggr KnownAggr;};struct Cyc_Absyn_ArrayInfo{void*elt_type;struct Cyc_Absyn_Tqual tq;struct Cyc_Absyn_Exp*num_elts;void*zero_term;unsigned zt_loc;};struct Cyc_Absyn_Aggr_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Aggrdecl*f1;};struct Cyc_Absyn_Enum_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;};struct Cyc_Absyn_Datatype_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;};struct Cyc_Absyn_TypeDecl{void*r;unsigned loc;};struct Cyc_Absyn_VoidCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_IntCon_Absyn_TyCon_struct{int tag;enum Cyc_Absyn_Sign f1;enum Cyc_Absyn_Size_of f2;};struct Cyc_Absyn_FloatCon_Absyn_TyCon_struct{int tag;int f1;};struct Cyc_Absyn_RgnHandleCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_TagCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_HeapCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_UniqueCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_RefCntCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_AccessCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_JoinCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_RgnsCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_TrueCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_FalseCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_ThinCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_FatCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_EnumCon_Absyn_TyCon_struct{int tag;struct _tuple0*f1;struct Cyc_Absyn_Enumdecl*f2;};struct Cyc_Absyn_AnonEnumCon_Absyn_TyCon_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_BuiltinCon_Absyn_TyCon_struct{int tag;struct _fat_ptr f1;struct Cyc_Absyn_Kind*f2;};struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_DatatypeInfo f1;};struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_DatatypeFieldInfo f1;};struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_AggrInfo f1;};struct Cyc_Absyn_AppType_Absyn_Type_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Evar_Absyn_Type_struct{int tag;struct Cyc_Core_Opt*f1;void*f2;int f3;struct Cyc_Core_Opt*f4;};struct Cyc_Absyn_VarType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Tvar*f1;};struct Cyc_Absyn_PointerType_Absyn_Type_struct{int tag;struct Cyc_Absyn_PtrInfo f1;};struct Cyc_Absyn_ArrayType_Absyn_Type_struct{int tag;struct Cyc_Absyn_ArrayInfo f1;};struct Cyc_Absyn_FnType_Absyn_Type_struct{int tag;struct Cyc_Absyn_FnInfo f1;};struct Cyc_Absyn_TupleType_Absyn_Type_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct{int tag;enum Cyc_Absyn_AggrKind f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_TypedefType_Absyn_Type_struct{int tag;struct _tuple0*f1;struct Cyc_List_List*f2;struct Cyc_Absyn_Typedefdecl*f3;void*f4;};struct Cyc_Absyn_ValueofType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct{int tag;struct Cyc_Absyn_TypeDecl*f1;void**f2;};struct Cyc_Absyn_TypeofType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_NoTypes_Absyn_Funcparams_struct{int tag;struct Cyc_List_List*f1;unsigned f2;};struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct{int tag;struct Cyc_List_List*f1;int f2;struct Cyc_Absyn_VarargInfo*f3;void*f4;struct Cyc_List_List*f5;struct Cyc_Absyn_Exp*f6;struct Cyc_Absyn_Exp*f7;struct Cyc_List_List*f8;struct Cyc_List_List*f9;};
# 437 "absyn.h"
enum Cyc_Absyn_Format_Type{Cyc_Absyn_Printf_ft =0U,Cyc_Absyn_Scanf_ft =1U};struct Cyc_Absyn_Regparm_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Stdcall_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Cdecl_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Fastcall_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Noreturn_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Const_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Aligned_att_Absyn_Attribute_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Packed_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Section_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Nocommon_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Shared_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Unused_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Weak_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Dllimport_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Dllexport_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_instrument_function_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Constructor_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Destructor_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_check_memory_usage_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Format_att_Absyn_Attribute_struct{int tag;enum Cyc_Absyn_Format_Type f1;int f2;int f3;};struct Cyc_Absyn_Initializes_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Noliveunique_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Consume_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Pure_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Mode_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Alias_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Always_inline_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_throw_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Non_null_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Deprecated_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Carray_mod_Absyn_Type_modifier_struct{int tag;void*f1;unsigned f2;};struct Cyc_Absyn_ConstArray_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_Absyn_Exp*f1;void*f2;unsigned f3;};struct Cyc_Absyn_Pointer_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_Absyn_PtrAtts f1;struct Cyc_Absyn_Tqual f2;};struct Cyc_Absyn_Function_mod_Absyn_Type_modifier_struct{int tag;void*f1;};struct Cyc_Absyn_TypeParams_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_List_List*f1;unsigned f2;int f3;};struct Cyc_Absyn_Attributes_mod_Absyn_Type_modifier_struct{int tag;unsigned f1;struct Cyc_List_List*f2;};struct _union_Cnst_Null_c{int tag;int val;};struct _tuple3{enum Cyc_Absyn_Sign f1;char f2;};struct _union_Cnst_Char_c{int tag;struct _tuple3 val;};struct _union_Cnst_Wchar_c{int tag;struct _fat_ptr val;};struct _tuple4{enum Cyc_Absyn_Sign f1;short f2;};struct _union_Cnst_Short_c{int tag;struct _tuple4 val;};struct _tuple5{enum Cyc_Absyn_Sign f1;int f2;};struct _union_Cnst_Int_c{int tag;struct _tuple5 val;};struct _tuple6{enum Cyc_Absyn_Sign f1;long long f2;};struct _union_Cnst_LongLong_c{int tag;struct _tuple6 val;};struct _tuple7{struct _fat_ptr f1;int f2;};struct _union_Cnst_Float_c{int tag;struct _tuple7 val;};struct _union_Cnst_String_c{int tag;struct _fat_ptr val;};struct _union_Cnst_Wstring_c{int tag;struct _fat_ptr val;};union Cyc_Absyn_Cnst{struct _union_Cnst_Null_c Null_c;struct _union_Cnst_Char_c Char_c;struct _union_Cnst_Wchar_c Wchar_c;struct _union_Cnst_Short_c Short_c;struct _union_Cnst_Int_c Int_c;struct _union_Cnst_LongLong_c LongLong_c;struct _union_Cnst_Float_c Float_c;struct _union_Cnst_String_c String_c;struct _union_Cnst_Wstring_c Wstring_c;};
# 532
enum Cyc_Absyn_Primop{Cyc_Absyn_Plus =0U,Cyc_Absyn_Times =1U,Cyc_Absyn_Minus =2U,Cyc_Absyn_Div =3U,Cyc_Absyn_Mod =4U,Cyc_Absyn_Eq =5U,Cyc_Absyn_Neq =6U,Cyc_Absyn_Gt =7U,Cyc_Absyn_Lt =8U,Cyc_Absyn_Gte =9U,Cyc_Absyn_Lte =10U,Cyc_Absyn_Not =11U,Cyc_Absyn_Bitnot =12U,Cyc_Absyn_Bitand =13U,Cyc_Absyn_Bitor =14U,Cyc_Absyn_Bitxor =15U,Cyc_Absyn_Bitlshift =16U,Cyc_Absyn_Bitlrshift =17U,Cyc_Absyn_Numelts =18U};
# 539
enum Cyc_Absyn_Incrementor{Cyc_Absyn_PreInc =0U,Cyc_Absyn_PostInc =1U,Cyc_Absyn_PreDec =2U,Cyc_Absyn_PostDec =3U};struct Cyc_Absyn_VarargCallInfo{int num_varargs;struct Cyc_List_List*injectors;struct Cyc_Absyn_VarargInfo*vai;};struct Cyc_Absyn_StructField_Absyn_OffsetofField_struct{int tag;struct _fat_ptr*f1;};struct Cyc_Absyn_TupleIndex_Absyn_OffsetofField_struct{int tag;unsigned f1;};
# 557
enum Cyc_Absyn_Coercion{Cyc_Absyn_Unknown_coercion =0U,Cyc_Absyn_No_coercion =1U,Cyc_Absyn_Null_to_NonNull =2U,Cyc_Absyn_Other_coercion =3U};struct Cyc_Absyn_MallocInfo{int is_calloc;struct Cyc_Absyn_Exp*rgn;void**elt_type;struct Cyc_Absyn_Exp*num_elts;int fat_result;int inline_call;};struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct{int tag;union Cyc_Absyn_Cnst f1;};struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Pragma_e_Absyn_Raw_exp_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct{int tag;enum Cyc_Absyn_Primop f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_AssignOp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Core_Opt*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_Increment_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;enum Cyc_Absyn_Incrementor f2;};struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;struct Cyc_Absyn_VarargCallInfo*f3;int f4;};struct Cyc_Absyn_Throw_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;int f2;};struct Cyc_Absyn_NoInstantiate_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Instantiate_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_Absyn_Exp*f2;int f3;enum Cyc_Absyn_Coercion f4;};struct Cyc_Absyn_Address_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_New_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Sizeoftype_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Sizeofexp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Offsetof_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Deref_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;int f3;int f4;};struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;int f3;int f4;};struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Tuple_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;};struct _tuple8{struct _fat_ptr*f1;struct Cyc_Absyn_Tqual f2;void*f3;};struct Cyc_Absyn_CompoundLit_e_Absyn_Raw_exp_struct{int tag;struct _tuple8*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Array_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Exp*f2;struct Cyc_Absyn_Exp*f3;int f4;};struct Cyc_Absyn_ComprehensionNoinit_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;void*f2;int f3;};struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct{int tag;struct _tuple0*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;struct Cyc_Absyn_Aggrdecl*f4;};struct Cyc_Absyn_AnonStruct_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Datatype_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;struct Cyc_Absyn_Datatypedecl*f2;struct Cyc_Absyn_Datatypefield*f3;};struct Cyc_Absyn_Enum_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_AnonEnum_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_Spawn_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_MallocInfo f1;};struct Cyc_Absyn_Swap_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_UnresolvedMem_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Core_Opt*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_StmtExp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Stmt*f1;};struct Cyc_Absyn_Tagcheck_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;};struct Cyc_Absyn_Valueof_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Asm_e_Absyn_Raw_exp_struct{int tag;int f1;struct _fat_ptr f2;struct Cyc_List_List*f3;struct Cyc_List_List*f4;struct Cyc_List_List*f5;};struct Cyc_Absyn_Extension_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Exp{void*topt;void*r;unsigned loc;void*annot;};struct Cyc_Absyn_Skip_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Exp_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Seq_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Return_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_IfThenElse_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Stmt*f2;struct Cyc_Absyn_Stmt*f3;};struct _tuple9{struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_While_s_Absyn_Raw_stmt_struct{int tag;struct _tuple9 f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Break_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Continue_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Goto_s_Absyn_Raw_stmt_struct{int tag;struct _fat_ptr*f1;};struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _tuple9 f2;struct _tuple9 f3;struct Cyc_Absyn_Stmt*f4;};struct Cyc_Absyn_Switch_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;void*f3;};struct Cyc_Absyn_Fallthru_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_List_List*f1;struct Cyc_Absyn_Switch_clause**f2;};struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Decl*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Label_s_Absyn_Raw_stmt_struct{int tag;struct _fat_ptr*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Do_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct _tuple9 f2;};struct Cyc_Absyn_TryCatch_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct Cyc_List_List*f2;void*f3;};struct Cyc_Absyn_Stmt{void*r;unsigned loc;void*annot;};struct Cyc_Absyn_Wild_p_Absyn_Raw_pat_struct{int tag;};struct Cyc_Absyn_Var_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Pat*f2;};struct Cyc_Absyn_AliasVar_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;};struct Cyc_Absyn_Reference_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Pat*f2;};struct Cyc_Absyn_TagInt_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;};struct Cyc_Absyn_Tuple_p_Absyn_Raw_pat_struct{int tag;struct Cyc_List_List*f1;int f2;};struct Cyc_Absyn_Pointer_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Pat*f1;};struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct{int tag;union Cyc_Absyn_AggrInfo*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;int f4;};struct Cyc_Absyn_Datatype_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;struct Cyc_Absyn_Datatypefield*f2;struct Cyc_List_List*f3;int f4;};struct Cyc_Absyn_Null_p_Absyn_Raw_pat_struct{int tag;};struct Cyc_Absyn_Int_p_Absyn_Raw_pat_struct{int tag;enum Cyc_Absyn_Sign f1;int f2;};struct Cyc_Absyn_Char_p_Absyn_Raw_pat_struct{int tag;char f1;};struct Cyc_Absyn_Float_p_Absyn_Raw_pat_struct{int tag;struct _fat_ptr f1;int f2;};struct Cyc_Absyn_Enum_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_AnonEnum_p_Absyn_Raw_pat_struct{int tag;void*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_UnknownId_p_Absyn_Raw_pat_struct{int tag;struct _tuple0*f1;};struct Cyc_Absyn_UnknownCall_p_Absyn_Raw_pat_struct{int tag;struct _tuple0*f1;struct Cyc_List_List*f2;int f3;};struct Cyc_Absyn_Exp_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Pat{void*r;void*topt;unsigned loc;};struct Cyc_Absyn_Switch_clause{struct Cyc_Absyn_Pat*pattern;struct Cyc_Core_Opt*pat_vars;struct Cyc_Absyn_Exp*where_clause;struct Cyc_Absyn_Stmt*body;unsigned loc;};struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct{int tag;struct _tuple0*f1;};struct Cyc_Absyn_Global_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Funname_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Fndecl*f1;};struct Cyc_Absyn_Param_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Local_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Pat_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Vardecl{enum Cyc_Absyn_Scope sc;struct _tuple0*name;unsigned varloc;struct Cyc_Absyn_Tqual tq;void*type;struct Cyc_Absyn_Exp*initializer;void*rgn;struct Cyc_List_List*attributes;int escapes;};struct Cyc_Absyn_Fndecl{enum Cyc_Absyn_Scope sc;int is_inline;struct _tuple0*name;struct Cyc_Absyn_Stmt*body;struct Cyc_Absyn_FnInfo i;void*cached_type;struct Cyc_Core_Opt*param_vardecls;struct Cyc_Absyn_Vardecl*fn_vardecl;};struct Cyc_Absyn_Aggrfield{struct _fat_ptr*name;struct Cyc_Absyn_Tqual tq;void*type;struct Cyc_Absyn_Exp*width;struct Cyc_List_List*attributes;struct Cyc_Absyn_Exp*requires_clause;};struct Cyc_Absyn_AggrdeclImpl{struct Cyc_List_List*exist_vars;struct Cyc_List_List*rgn_po;struct Cyc_List_List*fields;int tagged;};struct Cyc_Absyn_Aggrdecl{enum Cyc_Absyn_AggrKind kind;enum Cyc_Absyn_Scope sc;struct _tuple0*name;struct Cyc_List_List*tvs;struct Cyc_Absyn_AggrdeclImpl*impl;struct Cyc_List_List*attributes;int expected_mem_kind;};struct Cyc_Absyn_Datatypefield{struct _tuple0*name;struct Cyc_List_List*typs;unsigned loc;enum Cyc_Absyn_Scope sc;};struct Cyc_Absyn_Datatypedecl{enum Cyc_Absyn_Scope sc;struct _tuple0*name;struct Cyc_List_List*tvs;struct Cyc_Core_Opt*fields;int is_extensible;};struct Cyc_Absyn_Enumfield{struct _tuple0*name;struct Cyc_Absyn_Exp*tag;unsigned loc;};struct Cyc_Absyn_Enumdecl{enum Cyc_Absyn_Scope sc;struct _tuple0*name;struct Cyc_Core_Opt*fields;};struct Cyc_Absyn_Typedefdecl{struct _tuple0*name;struct Cyc_Absyn_Tqual tq;struct Cyc_List_List*tvs;struct Cyc_Core_Opt*kind;void*defn;struct Cyc_List_List*atts;int extern_c;};struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Fn_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Fndecl*f1;};struct Cyc_Absyn_Let_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Pat*f1;struct Cyc_Core_Opt*f2;struct Cyc_Absyn_Exp*f3;void*f4;};struct Cyc_Absyn_Letv_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_Region_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_Aggr_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Aggrdecl*f1;};struct Cyc_Absyn_Datatype_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;};struct Cyc_Absyn_Enum_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;};struct Cyc_Absyn_Typedef_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Typedefdecl*f1;};struct Cyc_Absyn_Namespace_d_Absyn_Raw_decl_struct{int tag;struct _fat_ptr*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Using_d_Absyn_Raw_decl_struct{int tag;struct _tuple0*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_ExternC_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;};struct _tuple10{unsigned f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_ExternCinclude_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;struct _tuple10*f4;};struct Cyc_Absyn_Porton_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Portoff_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Tempeston_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Tempestoff_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Decl{void*r;unsigned loc;};struct Cyc_Absyn_ArrayElement_Absyn_Designator_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_FieldName_Absyn_Designator_struct{int tag;struct _fat_ptr*f1;};extern char Cyc_Absyn_EmptyAnnot[11U];struct Cyc_Absyn_EmptyAnnot_Absyn_AbsynAnnot_struct{char*tag;};
# 908 "absyn.h"
int Cyc_Absyn_qvar_cmp(struct _tuple0*,struct _tuple0*);
# 917
struct Cyc_Absyn_Tqual Cyc_Absyn_empty_tqual(unsigned);
# 919
int Cyc_Absyn_equal_tqual(struct Cyc_Absyn_Tqual,struct Cyc_Absyn_Tqual);
# 923
struct Cyc_Absyn_Kind*Cyc_Absyn_force_kb(void*);
# 1164
struct _fat_ptr Cyc_Absyn_attribute2string(void*);
# 27 "unify.h"
void Cyc_Unify_explain_failure();
int Cyc_Unify_unify_kindbound(void*,void*);
int Cyc_Unify_unify(void*,void*);struct _union_RelnOp_RConst{int tag;unsigned val;};struct _union_RelnOp_RVar{int tag;struct Cyc_Absyn_Vardecl*val;};struct _union_RelnOp_RNumelts{int tag;struct Cyc_Absyn_Vardecl*val;};struct _union_RelnOp_RType{int tag;void*val;};struct _union_RelnOp_RParam{int tag;unsigned val;};struct _union_RelnOp_RParamNumelts{int tag;unsigned val;};struct _union_RelnOp_RReturn{int tag;unsigned val;};union Cyc_Relations_RelnOp{struct _union_RelnOp_RConst RConst;struct _union_RelnOp_RVar RVar;struct _union_RelnOp_RNumelts RNumelts;struct _union_RelnOp_RType RType;struct _union_RelnOp_RParam RParam;struct _union_RelnOp_RParamNumelts RParamNumelts;struct _union_RelnOp_RReturn RReturn;};
# 50 "relations-ap.h"
enum Cyc_Relations_Relation{Cyc_Relations_Req =0U,Cyc_Relations_Rneq =1U,Cyc_Relations_Rlte =2U,Cyc_Relations_Rlt =3U};struct Cyc_Relations_Reln{union Cyc_Relations_RelnOp rop1;enum Cyc_Relations_Relation relation;union Cyc_Relations_RelnOp rop2;};struct Cyc_RgnOrder_RgnPO;extern char Cyc_Tcenv_Env_error[10U];struct Cyc_Tcenv_Env_error_exn_struct{char*tag;};struct Cyc_Tcenv_Genv{struct Cyc_Dict_Dict aggrdecls;struct Cyc_Dict_Dict datatypedecls;struct Cyc_Dict_Dict enumdecls;struct Cyc_Dict_Dict typedefs;struct Cyc_Dict_Dict ordinaries;};struct Cyc_Tcenv_Fenv;struct Cyc_Tcenv_Tenv{struct Cyc_List_List*ns;struct Cyc_Tcenv_Genv*ae;struct Cyc_Tcenv_Fenv*le;int allow_valueof: 1;int in_extern_c_include: 1;int in_tempest: 1;int tempest_generalize: 1;};
# 88 "tcenv.h"
enum Cyc_Tcenv_NewStatus{Cyc_Tcenv_NoneNew =0U,Cyc_Tcenv_InNew =1U,Cyc_Tcenv_InNewAggr =2U};
# 185 "tcutil.h"
void*Cyc_Tcutil_substitute(struct Cyc_List_List*,void*);
# 266
int Cyc_Tcutil_same_atts(struct Cyc_List_List*,struct Cyc_List_List*);struct Cyc_PP_Ppstate;struct Cyc_PP_Out;struct Cyc_PP_Doc;struct Cyc_Absynpp_Params{int expand_typedefs;int qvar_to_Cids;int add_cyc_prefix;int to_VC;int decls_first;int rewrite_temp_tvars;int print_all_tvars;int print_all_kinds;int print_all_effects;int print_using_stmts;int print_externC_stmts;int print_full_evars;int print_zeroterm;int generate_line_directives;int use_curr_namespace;struct Cyc_List_List*curr_namespace;};
# 62 "absynpp.h"
struct _fat_ptr Cyc_Absynpp_typ2string(void*);
# 64
struct _fat_ptr Cyc_Absynpp_kind2string(struct Cyc_Absyn_Kind*);
# 67
struct _fat_ptr Cyc_Absynpp_exp2string(struct Cyc_Absyn_Exp*);
# 69
struct _fat_ptr Cyc_Absynpp_qvar2string(struct _tuple0*);
# 73
struct _fat_ptr Cyc_Absynpp_scope2string(enum Cyc_Absyn_Scope sc);extern char Cyc_Tcdecl_Incompatible[13U];struct Cyc_Tcdecl_Incompatible_exn_struct{char*tag;};struct Cyc_Tcdecl_Xdatatypefielddecl{struct Cyc_Absyn_Datatypedecl*base;struct Cyc_Absyn_Datatypefield*field;};
# 38 "tcdecl.h"
void Cyc_Tcdecl_merr(unsigned loc,struct _fat_ptr*msg1,struct _fat_ptr fmt,struct _fat_ptr ap);
# 61 "tcdecl.h"
struct Cyc_Absyn_Vardecl*Cyc_Tcdecl_merge_vardecl(struct Cyc_Absyn_Vardecl*,struct Cyc_Absyn_Vardecl*,unsigned,struct _fat_ptr*msg);
# 74
struct Cyc_List_List*Cyc_Tcdecl_sort_xdatatype_fields(struct Cyc_List_List*f,int*res,struct _fat_ptr*v,unsigned,struct _fat_ptr*msg);char Cyc_Tcdecl_Incompatible[13U]="Incompatible";
# 36 "tcdecl.cyc"
struct Cyc_Tcdecl_Incompatible_exn_struct Cyc_Tcdecl_Incompatible_val={Cyc_Tcdecl_Incompatible};
# 38
void Cyc_Tcdecl_merr(unsigned loc,struct _fat_ptr*msg1,struct _fat_ptr fmt,struct _fat_ptr ap){
# 42
if(msg1 == 0)
(int)_throw((void*)& Cyc_Tcdecl_Incompatible_val);{
# 42
struct _fat_ptr fmt2;
# 45
if(Cyc_strlen((struct _fat_ptr)*msg1)== (unsigned long)0)
fmt2=(struct _fat_ptr)({struct _fat_ptr _tmp188=({const char*_tmp0="%s";_tag_fat(_tmp0,sizeof(char),3U);});Cyc_strconcat(_tmp188,(struct _fat_ptr)fmt);});else{
# 48
fmt2=(struct _fat_ptr)({struct _fat_ptr _tmp189=({const char*_tmp1="%s ";_tag_fat(_tmp1,sizeof(char),4U);});Cyc_strconcat(_tmp189,(struct _fat_ptr)fmt);});}{
struct _fat_ptr ap2=({unsigned _tmp4=_get_fat_size(ap,sizeof(void*))+ (unsigned)1;void**_tmp3=_cycalloc(_check_times(_tmp4,sizeof(void*)));({{unsigned _tmp11F=_get_fat_size(ap,sizeof(void*))+ (unsigned)1;unsigned i;for(i=0;i < _tmp11F;++ i){
i == (unsigned)0?({void*_tmp18A=(void*)({struct Cyc_String_pa_PrintArg_struct*_tmp2=_cycalloc(sizeof(*_tmp2));_tmp2->tag=0U,_tmp2->f1=(struct _fat_ptr)((struct _fat_ptr)*msg1);_tmp2;});_tmp3[i]=_tmp18A;}):(_tmp3[i]=*((void**)_check_fat_subscript(ap,sizeof(void*),(int)(i - (unsigned)1))));}}0;});_tag_fat(_tmp3,sizeof(void*),_tmp4);});
Cyc_Position_post_error(({unsigned _tmp18B=loc;Cyc_Position_mk_err(_tmp18B,(struct _fat_ptr)Cyc_vrprintf(Cyc_Core_heap_region,fmt2,ap2));}));}}}
# 38
static void Cyc_Tcdecl_merge_scope_err(enum Cyc_Absyn_Scope s0,enum Cyc_Absyn_Scope s1,struct _fat_ptr t,struct _fat_ptr v,unsigned loc,struct _fat_ptr*msg){
# 58
struct _fat_ptr s0_str=(int)s0 == (int)2U?({const char*_tmpC="public ";_tag_fat(_tmpC,sizeof(char),8U);}): Cyc_Absynpp_scope2string(s0);
struct _fat_ptr s1_str=(int)s1 == (int)2U?({const char*_tmpB="public ";_tag_fat(_tmpB,sizeof(char),8U);}): Cyc_Absynpp_scope2string(s1);
({struct Cyc_String_pa_PrintArg_struct _tmp7=({struct Cyc_String_pa_PrintArg_struct _tmp123;_tmp123.tag=0U,_tmp123.f1=(struct _fat_ptr)((struct _fat_ptr)t);_tmp123;});struct Cyc_String_pa_PrintArg_struct _tmp8=({struct Cyc_String_pa_PrintArg_struct _tmp122;_tmp122.tag=0U,_tmp122.f1=(struct _fat_ptr)((struct _fat_ptr)v);_tmp122;});struct Cyc_String_pa_PrintArg_struct _tmp9=({struct Cyc_String_pa_PrintArg_struct _tmp121;_tmp121.tag=0U,_tmp121.f1=(struct _fat_ptr)((struct _fat_ptr)s1_str);_tmp121;});struct Cyc_String_pa_PrintArg_struct _tmpA=({struct Cyc_String_pa_PrintArg_struct _tmp120;_tmp120.tag=0U,_tmp120.f1=(struct _fat_ptr)((struct _fat_ptr)s0_str);_tmp120;});void*_tmp5[4U];_tmp5[0]=& _tmp7,_tmp5[1]=& _tmp8,_tmp5[2]=& _tmp9,_tmp5[3]=& _tmpA;({unsigned _tmp18E=loc;struct _fat_ptr*_tmp18D=msg;struct _fat_ptr _tmp18C=({const char*_tmp6="%s %s is %sbut expected scope is %s";_tag_fat(_tmp6,sizeof(char),36U);});Cyc_Tcdecl_merr(_tmp18E,_tmp18D,_tmp18C,_tag_fat(_tmp5,sizeof(void*),4U));});});}struct _tuple11{enum Cyc_Absyn_Scope f1;int f2;};struct _tuple12{enum Cyc_Absyn_Scope f1;enum Cyc_Absyn_Scope f2;};
# 38
struct _tuple11 Cyc_Tcdecl_merge_scope(enum Cyc_Absyn_Scope s0,enum Cyc_Absyn_Scope s1,struct _fat_ptr t,struct _fat_ptr v,unsigned loc,struct _fat_ptr*msg,int externCmerge){
# 66
{struct _tuple12 _stmttmp0=({struct _tuple12 _tmp126;_tmp126.f1=s0,_tmp126.f2=s1;_tmp126;});struct _tuple12 _tmpD=_stmttmp0;switch(_tmpD.f1){case Cyc_Absyn_ExternC: switch(_tmpD.f2){case Cyc_Absyn_ExternC: _LL1: _LL2:
# 68
 goto _LL0;case Cyc_Absyn_Public: _LL3: _LL4:
# 70
 goto _LL6;case Cyc_Absyn_Extern: _LL5: _LL6:
 goto _LL8;default: _LLB: _LLC:
# 74
 goto _LLE;}case Cyc_Absyn_Public: switch(_tmpD.f2){case Cyc_Absyn_ExternC: _LL7: _LL8:
# 72
 goto _LLA;case Cyc_Absyn_Extern: goto _LLF;case Cyc_Absyn_Public: _LL15: _LL16:
# 81
 goto _LL18;default: goto _LL1B;}case Cyc_Absyn_Extern: switch(_tmpD.f2){case Cyc_Absyn_ExternC: _LL9: _LLA:
# 73
 if(externCmerge)goto _LL0;else{goto _LLC;}case Cyc_Absyn_Extern: goto _LLF;default: _LL11: _LL12:
# 78
 goto _LL0;}default: switch(_tmpD.f2){case Cyc_Absyn_ExternC: _LLD: _LLE:
# 75
 Cyc_Tcdecl_merge_scope_err(s0,s1,t,v,loc,msg);return({struct _tuple11 _tmp124;_tmp124.f1=s1,_tmp124.f2=0;_tmp124;});case Cyc_Absyn_Extern: _LLF: _LL10:
# 77
 s1=s0;goto _LL0;default: switch(_tmpD.f1){case Cyc_Absyn_Static: if(_tmpD.f2 == Cyc_Absyn_Static){_LL13: _LL14:
# 80
 goto _LL16;}else{goto _LL1B;}case Cyc_Absyn_Abstract: if(_tmpD.f2 == Cyc_Absyn_Abstract){_LL17: _LL18:
# 82
 goto _LL1A;}else{goto _LL1B;}case Cyc_Absyn_Register: if(_tmpD.f2 == Cyc_Absyn_Register){_LL19: _LL1A:
 goto _LL0;}else{goto _LL1B;}default: _LL1B: _LL1C:
 Cyc_Tcdecl_merge_scope_err(s0,s1,t,v,loc,msg);return({struct _tuple11 _tmp125;_tmp125.f1=s1,_tmp125.f2=0;_tmp125;});}}}_LL0:;}
# 86
return({struct _tuple11 _tmp127;_tmp127.f1=s1,_tmp127.f2=1;_tmp127;});}
# 38
static int Cyc_Tcdecl_check_type(void*t0,void*t1){
# 91
return Cyc_Unify_unify(t0,t1);}
# 38
static unsigned Cyc_Tcdecl_get_uint_const_value(struct Cyc_Absyn_Exp*e){
# 96
void*_stmttmp1=e->r;void*_tmpE=_stmttmp1;int _tmpF;if(((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmpE)->tag == 0U){if(((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmpE)->f1).Int_c).tag == 5){_LL1: _tmpF=(((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmpE)->f1).Int_c).val).f2;_LL2: {int x=_tmpF;
return(unsigned)x;}}else{goto _LL3;}}else{_LL3: _LL4:
(int)_throw((void*)({struct Cyc_Core_Invalid_argument_exn_struct*_tmp11=_cycalloc(sizeof(*_tmp11));_tmp11->tag=Cyc_Core_Invalid_argument,({struct _fat_ptr _tmp18F=({const char*_tmp10="Tcdecl::get_uint_const_value";_tag_fat(_tmp10,sizeof(char),29U);});_tmp11->f1=_tmp18F;});_tmp11;}));}_LL0:;}struct _tuple13{struct Cyc_Absyn_Tvar*f1;void*f2;};
# 38
static struct Cyc_List_List*Cyc_Tcdecl_build_tvs_map(struct Cyc_List_List*tvs0,struct Cyc_List_List*tvs1){
# 104
struct Cyc_List_List*inst=0;
for(0;tvs0 != 0;(tvs0=tvs0->tl,tvs1=tvs1->tl)){
inst=({struct Cyc_List_List*_tmp14=_cycalloc(sizeof(*_tmp14));({struct _tuple13*_tmp191=({struct _tuple13*_tmp13=_cycalloc(sizeof(*_tmp13));_tmp13->f1=(struct Cyc_Absyn_Tvar*)((struct Cyc_List_List*)_check_null(tvs1))->hd,({void*_tmp190=(void*)({struct Cyc_Absyn_VarType_Absyn_Type_struct*_tmp12=_cycalloc(sizeof(*_tmp12));_tmp12->tag=2U,_tmp12->f1=(struct Cyc_Absyn_Tvar*)tvs0->hd;_tmp12;});_tmp13->f2=_tmp190;});_tmp13;});_tmp14->hd=_tmp191;}),_tmp14->tl=inst;_tmp14;});}
return inst;}
# 38
static int Cyc_Tcdecl_check_tvs(struct Cyc_List_List*tvs0,struct Cyc_List_List*tvs1,struct _fat_ptr t,struct _fat_ptr v,unsigned loc,struct _fat_ptr*msg){
# 113
if(({int _tmp192=Cyc_List_length(tvs0);_tmp192 != Cyc_List_length(tvs1);})){
({struct Cyc_String_pa_PrintArg_struct _tmp17=({struct Cyc_String_pa_PrintArg_struct _tmp129;_tmp129.tag=0U,_tmp129.f1=(struct _fat_ptr)((struct _fat_ptr)t);_tmp129;});struct Cyc_String_pa_PrintArg_struct _tmp18=({struct Cyc_String_pa_PrintArg_struct _tmp128;_tmp128.tag=0U,_tmp128.f1=(struct _fat_ptr)((struct _fat_ptr)v);_tmp128;});void*_tmp15[2U];_tmp15[0]=& _tmp17,_tmp15[1]=& _tmp18;({unsigned _tmp195=loc;struct _fat_ptr*_tmp194=msg;struct _fat_ptr _tmp193=({const char*_tmp16="%s %s has a different number of type parameters";_tag_fat(_tmp16,sizeof(char),48U);});Cyc_Tcdecl_merr(_tmp195,_tmp194,_tmp193,_tag_fat(_tmp15,sizeof(void*),2U));});});
return 0;}{
# 113
struct Cyc_List_List*x0=tvs0;
# 118
struct Cyc_List_List*x1=tvs1;
for(0;x0 != 0;(x0=x0->tl,x1=x1->tl)){
Cyc_Unify_unify_kindbound(((struct Cyc_Absyn_Tvar*)x0->hd)->kind,((struct Cyc_Absyn_Tvar*)((struct Cyc_List_List*)_check_null(x1))->hd)->kind);{
struct Cyc_Absyn_Kind*c0=Cyc_Absyn_force_kb(((struct Cyc_Absyn_Tvar*)x0->hd)->kind);
struct Cyc_Absyn_Kind*c1=Cyc_Absyn_force_kb(((struct Cyc_Absyn_Tvar*)x1->hd)->kind);
if(c0 == c1)continue;({struct Cyc_String_pa_PrintArg_struct _tmp1B=({struct Cyc_String_pa_PrintArg_struct _tmp12E;_tmp12E.tag=0U,_tmp12E.f1=(struct _fat_ptr)((struct _fat_ptr)t);_tmp12E;});struct Cyc_String_pa_PrintArg_struct _tmp1C=({struct Cyc_String_pa_PrintArg_struct _tmp12D;_tmp12D.tag=0U,_tmp12D.f1=(struct _fat_ptr)((struct _fat_ptr)v);_tmp12D;});struct Cyc_String_pa_PrintArg_struct _tmp1D=({struct Cyc_String_pa_PrintArg_struct _tmp12C;_tmp12C.tag=0U,({
# 125
struct _fat_ptr _tmp196=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_kind2string(c0));_tmp12C.f1=_tmp196;});_tmp12C;});struct Cyc_String_pa_PrintArg_struct _tmp1E=({struct Cyc_String_pa_PrintArg_struct _tmp12B;_tmp12B.tag=0U,_tmp12B.f1=(struct _fat_ptr)((struct _fat_ptr)*((struct Cyc_Absyn_Tvar*)x0->hd)->name);_tmp12B;});struct Cyc_String_pa_PrintArg_struct _tmp1F=({struct Cyc_String_pa_PrintArg_struct _tmp12A;_tmp12A.tag=0U,({struct _fat_ptr _tmp197=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_kind2string(c1));_tmp12A.f1=_tmp197;});_tmp12A;});void*_tmp19[5U];_tmp19[0]=& _tmp1B,_tmp19[1]=& _tmp1C,_tmp19[2]=& _tmp1D,_tmp19[3]=& _tmp1E,_tmp19[4]=& _tmp1F;({unsigned _tmp19A=loc;struct _fat_ptr*_tmp199=msg;struct _fat_ptr _tmp198=({const char*_tmp1A="%s %s has a different kind (%s) for type parameter %s (%s)";_tag_fat(_tmp1A,sizeof(char),59U);});Cyc_Tcdecl_merr(_tmp19A,_tmp199,_tmp198,_tag_fat(_tmp19,sizeof(void*),5U));});});
return 0;}}
# 128
return 1;}}
# 38
static int Cyc_Tcdecl_check_atts(struct Cyc_List_List*atts0,struct Cyc_List_List*atts1,struct _fat_ptr t,struct _fat_ptr v,unsigned loc,struct _fat_ptr*msg){
# 133
if(!Cyc_Tcutil_same_atts(atts0,atts1)){
({struct Cyc_String_pa_PrintArg_struct _tmp22=({struct Cyc_String_pa_PrintArg_struct _tmp130;_tmp130.tag=0U,_tmp130.f1=(struct _fat_ptr)((struct _fat_ptr)t);_tmp130;});struct Cyc_String_pa_PrintArg_struct _tmp23=({struct Cyc_String_pa_PrintArg_struct _tmp12F;_tmp12F.tag=0U,_tmp12F.f1=(struct _fat_ptr)((struct _fat_ptr)v);_tmp12F;});void*_tmp20[2U];_tmp20[0]=& _tmp22,_tmp20[1]=& _tmp23;({unsigned _tmp19D=loc;struct _fat_ptr*_tmp19C=msg;struct _fat_ptr _tmp19B=({const char*_tmp21="%s %s has different attributes";_tag_fat(_tmp21,sizeof(char),31U);});Cyc_Tcdecl_merr(_tmp19D,_tmp19C,_tmp19B,_tag_fat(_tmp20,sizeof(void*),2U));});});
return 0;}
# 133
return 1;}struct _tuple14{struct Cyc_Absyn_AggrdeclImpl*f1;struct Cyc_Absyn_AggrdeclImpl*f2;};struct _tuple15{void*f1;void*f2;};
# 38
struct Cyc_Absyn_Aggrdecl*Cyc_Tcdecl_merge_aggrdecl(struct Cyc_Absyn_Aggrdecl*d0,struct Cyc_Absyn_Aggrdecl*d1,unsigned loc,struct _fat_ptr*msg){
# 143
struct _fat_ptr v=Cyc_Absynpp_qvar2string(d0->name);
int res=1;
# 149
if(!((int)d0->kind == (int)d1->kind))return 0;if(!({
# 152
struct Cyc_List_List*_tmp1A2=d0->tvs;struct Cyc_List_List*_tmp1A1=d1->tvs;struct _fat_ptr _tmp1A0=({const char*_tmp24="type";_tag_fat(_tmp24,sizeof(char),5U);});struct _fat_ptr _tmp19F=v;unsigned _tmp19E=loc;Cyc_Tcdecl_check_tvs(_tmp1A2,_tmp1A1,_tmp1A0,_tmp19F,_tmp19E,msg);}))return 0;{
# 149
struct _tuple11 _stmttmp2=({
# 156
enum Cyc_Absyn_Scope _tmp1A7=d0->sc;enum Cyc_Absyn_Scope _tmp1A6=d1->sc;struct _fat_ptr _tmp1A5=({const char*_tmp73="type";_tag_fat(_tmp73,sizeof(char),5U);});struct _fat_ptr _tmp1A4=v;unsigned _tmp1A3=loc;Cyc_Tcdecl_merge_scope(_tmp1A7,_tmp1A6,_tmp1A5,_tmp1A4,_tmp1A3,msg,1);});
# 149
int _tmp26;enum Cyc_Absyn_Scope _tmp25;_LL1: _tmp25=_stmttmp2.f1;_tmp26=_stmttmp2.f2;_LL2: {enum Cyc_Absyn_Scope new_scope=_tmp25;int res_scope=_tmp26;
# 157
if(!res_scope)res=0;if(!({
# 160
struct Cyc_List_List*_tmp1AC=d0->attributes;struct Cyc_List_List*_tmp1AB=d1->attributes;struct _fat_ptr _tmp1AA=({const char*_tmp27="type";_tag_fat(_tmp27,sizeof(char),5U);});struct _fat_ptr _tmp1A9=v;unsigned _tmp1A8=loc;Cyc_Tcdecl_check_atts(_tmp1AC,_tmp1AB,_tmp1AA,_tmp1A9,_tmp1A8,msg);}))res=0;{
# 157
struct Cyc_Absyn_Aggrdecl*d2;
# 164
{struct _tuple14 _stmttmp3=({struct _tuple14 _tmp14B;_tmp14B.f1=d0->impl,_tmp14B.f2=d1->impl;_tmp14B;});struct _tuple14 _tmp28=_stmttmp3;int _tmp30;struct Cyc_List_List*_tmp2F;struct Cyc_List_List*_tmp2E;struct Cyc_List_List*_tmp2D;int _tmp2C;struct Cyc_List_List*_tmp2B;struct Cyc_List_List*_tmp2A;struct Cyc_List_List*_tmp29;if(_tmp28.f2 == 0){_LL4: _LL5:
 d2=d0;goto _LL3;}else{if(_tmp28.f1 == 0){_LL6: _LL7:
 d2=d1;goto _LL3;}else{_LL8: _tmp29=(_tmp28.f1)->exist_vars;_tmp2A=(_tmp28.f1)->rgn_po;_tmp2B=(_tmp28.f1)->fields;_tmp2C=(_tmp28.f1)->tagged;_tmp2D=(_tmp28.f2)->exist_vars;_tmp2E=(_tmp28.f2)->rgn_po;_tmp2F=(_tmp28.f2)->fields;_tmp30=(_tmp28.f2)->tagged;_LL9: {struct Cyc_List_List*exist_vars0=_tmp29;struct Cyc_List_List*rgn_po0=_tmp2A;struct Cyc_List_List*f0s=_tmp2B;int tagged0=_tmp2C;struct Cyc_List_List*exist_vars1=_tmp2D;struct Cyc_List_List*rgn_po1=_tmp2E;struct Cyc_List_List*f1s=_tmp2F;int tagged1=_tmp30;
# 171
if(!({struct Cyc_List_List*_tmp1B1=exist_vars0;struct Cyc_List_List*_tmp1B0=exist_vars1;struct _fat_ptr _tmp1AF=({const char*_tmp31="type";_tag_fat(_tmp31,sizeof(char),5U);});struct _fat_ptr _tmp1AE=v;unsigned _tmp1AD=loc;Cyc_Tcdecl_check_tvs(_tmp1B1,_tmp1B0,_tmp1AF,_tmp1AE,_tmp1AD,msg);}))
return 0;{
# 171
struct Cyc_List_List*inst=({
# 175
struct Cyc_List_List*_tmp1B2=Cyc_List_append(d0->tvs,exist_vars0);Cyc_Tcdecl_build_tvs_map(_tmp1B2,
Cyc_List_append(d1->tvs,exist_vars1));});
# 180
for(0;rgn_po0 != 0 && rgn_po1 != 0;(
rgn_po0=rgn_po0->tl,rgn_po1=rgn_po1->tl)){
Cyc_Tcdecl_check_type((*((struct _tuple15*)rgn_po0->hd)).f1,(*((struct _tuple15*)rgn_po1->hd)).f1);
Cyc_Tcdecl_check_type((*((struct _tuple15*)rgn_po0->hd)).f2,(*((struct _tuple15*)rgn_po1->hd)).f2);}
# 187
for(0;f0s != 0 && f1s != 0;(f0s=f0s->tl,f1s=f1s->tl)){
struct Cyc_Absyn_Aggrfield*_stmttmp4=(struct Cyc_Absyn_Aggrfield*)f0s->hd;struct Cyc_Absyn_Exp*_tmp37;struct Cyc_List_List*_tmp36;struct Cyc_Absyn_Exp*_tmp35;void*_tmp34;struct Cyc_Absyn_Tqual _tmp33;struct _fat_ptr*_tmp32;_LLB: _tmp32=_stmttmp4->name;_tmp33=_stmttmp4->tq;_tmp34=_stmttmp4->type;_tmp35=_stmttmp4->width;_tmp36=_stmttmp4->attributes;_tmp37=_stmttmp4->requires_clause;_LLC: {struct _fat_ptr*fn0=_tmp32;struct Cyc_Absyn_Tqual tq0=_tmp33;void*t0=_tmp34;struct Cyc_Absyn_Exp*width0=_tmp35;struct Cyc_List_List*atts0=_tmp36;struct Cyc_Absyn_Exp*req0=_tmp37;
struct Cyc_Absyn_Aggrfield*_stmttmp5=(struct Cyc_Absyn_Aggrfield*)f1s->hd;struct Cyc_Absyn_Exp*_tmp3D;struct Cyc_List_List*_tmp3C;struct Cyc_Absyn_Exp*_tmp3B;void*_tmp3A;struct Cyc_Absyn_Tqual _tmp39;struct _fat_ptr*_tmp38;_LLE: _tmp38=_stmttmp5->name;_tmp39=_stmttmp5->tq;_tmp3A=_stmttmp5->type;_tmp3B=_stmttmp5->width;_tmp3C=_stmttmp5->attributes;_tmp3D=_stmttmp5->requires_clause;_LLF: {struct _fat_ptr*fn1=_tmp38;struct Cyc_Absyn_Tqual tq1=_tmp39;void*t1=_tmp3A;struct Cyc_Absyn_Exp*width1=_tmp3B;struct Cyc_List_List*atts1=_tmp3C;struct Cyc_Absyn_Exp*req1=_tmp3D;
# 191
if(Cyc_strptrcmp(fn0,fn1)!= 0){
({struct Cyc_String_pa_PrintArg_struct _tmp40=({struct Cyc_String_pa_PrintArg_struct _tmp134;_tmp134.tag=0U,({struct _fat_ptr _tmp1B3=(struct _fat_ptr)({const char*_tmp44="type";_tag_fat(_tmp44,sizeof(char),5U);});_tmp134.f1=_tmp1B3;});_tmp134;});struct Cyc_String_pa_PrintArg_struct _tmp41=({struct Cyc_String_pa_PrintArg_struct _tmp133;_tmp133.tag=0U,_tmp133.f1=(struct _fat_ptr)((struct _fat_ptr)v);_tmp133;});struct Cyc_String_pa_PrintArg_struct _tmp42=({struct Cyc_String_pa_PrintArg_struct _tmp132;_tmp132.tag=0U,_tmp132.f1=(struct _fat_ptr)((struct _fat_ptr)*fn0);_tmp132;});struct Cyc_String_pa_PrintArg_struct _tmp43=({struct Cyc_String_pa_PrintArg_struct _tmp131;_tmp131.tag=0U,_tmp131.f1=(struct _fat_ptr)((struct _fat_ptr)*fn1);_tmp131;});void*_tmp3E[4U];_tmp3E[0]=& _tmp40,_tmp3E[1]=& _tmp41,_tmp3E[2]=& _tmp42,_tmp3E[3]=& _tmp43;({unsigned _tmp1B6=loc;struct _fat_ptr*_tmp1B5=msg;struct _fat_ptr _tmp1B4=({const char*_tmp3F="%s %s : field name mismatch %s != %s";_tag_fat(_tmp3F,sizeof(char),37U);});Cyc_Tcdecl_merr(_tmp1B6,_tmp1B5,_tmp1B4,_tag_fat(_tmp3E,sizeof(void*),4U));});});
# 194
return 0;}
# 191
if(!
# 196
Cyc_Tcutil_same_atts(atts0,atts1)){
({struct Cyc_String_pa_PrintArg_struct _tmp47=({struct Cyc_String_pa_PrintArg_struct _tmp137;_tmp137.tag=0U,({struct _fat_ptr _tmp1B7=(struct _fat_ptr)({const char*_tmp4A="type";_tag_fat(_tmp4A,sizeof(char),5U);});_tmp137.f1=_tmp1B7;});_tmp137;});struct Cyc_String_pa_PrintArg_struct _tmp48=({struct Cyc_String_pa_PrintArg_struct _tmp136;_tmp136.tag=0U,_tmp136.f1=(struct _fat_ptr)((struct _fat_ptr)v);_tmp136;});struct Cyc_String_pa_PrintArg_struct _tmp49=({struct Cyc_String_pa_PrintArg_struct _tmp135;_tmp135.tag=0U,_tmp135.f1=(struct _fat_ptr)((struct _fat_ptr)*fn0);_tmp135;});void*_tmp45[3U];_tmp45[0]=& _tmp47,_tmp45[1]=& _tmp48,_tmp45[2]=& _tmp49;({unsigned _tmp1BA=loc;struct _fat_ptr*_tmp1B9=msg;struct _fat_ptr _tmp1B8=({const char*_tmp46="%s %s : attribute mismatch on field %s";_tag_fat(_tmp46,sizeof(char),39U);});Cyc_Tcdecl_merr(_tmp1BA,_tmp1B9,_tmp1B8,_tag_fat(_tmp45,sizeof(void*),3U));});});
res=0;}
# 191
if(!
# 200
Cyc_Absyn_equal_tqual(tq0,tq1)){
({struct Cyc_String_pa_PrintArg_struct _tmp4D=({struct Cyc_String_pa_PrintArg_struct _tmp13A;_tmp13A.tag=0U,({struct _fat_ptr _tmp1BB=(struct _fat_ptr)({const char*_tmp50="type";_tag_fat(_tmp50,sizeof(char),5U);});_tmp13A.f1=_tmp1BB;});_tmp13A;});struct Cyc_String_pa_PrintArg_struct _tmp4E=({struct Cyc_String_pa_PrintArg_struct _tmp139;_tmp139.tag=0U,_tmp139.f1=(struct _fat_ptr)((struct _fat_ptr)v);_tmp139;});struct Cyc_String_pa_PrintArg_struct _tmp4F=({struct Cyc_String_pa_PrintArg_struct _tmp138;_tmp138.tag=0U,_tmp138.f1=(struct _fat_ptr)((struct _fat_ptr)*fn0);_tmp138;});void*_tmp4B[3U];_tmp4B[0]=& _tmp4D,_tmp4B[1]=& _tmp4E,_tmp4B[2]=& _tmp4F;({unsigned _tmp1BE=loc;struct _fat_ptr*_tmp1BD=msg;struct _fat_ptr _tmp1BC=({const char*_tmp4C="%s %s : qualifier mismatch on field %s";_tag_fat(_tmp4C,sizeof(char),39U);});Cyc_Tcdecl_merr(_tmp1BE,_tmp1BD,_tmp1BC,_tag_fat(_tmp4B,sizeof(void*),3U));});});
res=0;}
# 191
if(
# 204
((width0 != 0 && width1 != 0)&&({
unsigned _tmp1BF=Cyc_Tcdecl_get_uint_const_value(width0);_tmp1BF != 
Cyc_Tcdecl_get_uint_const_value(width1);})||
 width0 == 0 && width1 != 0)||
 width0 != 0 && width1 == 0){
({struct Cyc_String_pa_PrintArg_struct _tmp53=({struct Cyc_String_pa_PrintArg_struct _tmp13D;_tmp13D.tag=0U,({struct _fat_ptr _tmp1C0=(struct _fat_ptr)({const char*_tmp56="type";_tag_fat(_tmp56,sizeof(char),5U);});_tmp13D.f1=_tmp1C0;});_tmp13D;});struct Cyc_String_pa_PrintArg_struct _tmp54=({struct Cyc_String_pa_PrintArg_struct _tmp13C;_tmp13C.tag=0U,_tmp13C.f1=(struct _fat_ptr)((struct _fat_ptr)v);_tmp13C;});struct Cyc_String_pa_PrintArg_struct _tmp55=({struct Cyc_String_pa_PrintArg_struct _tmp13B;_tmp13B.tag=0U,_tmp13B.f1=(struct _fat_ptr)((struct _fat_ptr)*fn0);_tmp13B;});void*_tmp51[3U];_tmp51[0]=& _tmp53,_tmp51[1]=& _tmp54,_tmp51[2]=& _tmp55;({unsigned _tmp1C3=loc;struct _fat_ptr*_tmp1C2=msg;struct _fat_ptr _tmp1C1=({const char*_tmp52="%s %s : bitfield mismatch on field %s";_tag_fat(_tmp52,sizeof(char),38U);});Cyc_Tcdecl_merr(_tmp1C3,_tmp1C2,_tmp1C1,_tag_fat(_tmp51,sizeof(void*),3U));});});
res=0;}
# 191
if(
# 212
req0 != 0 && req1 != 0){
void*subst_w1=({struct Cyc_List_List*_tmp1C4=inst;Cyc_Tcutil_substitute(_tmp1C4,(void*)({struct Cyc_Absyn_ValueofType_Absyn_Type_struct*_tmp5D=_cycalloc(sizeof(*_tmp5D));_tmp5D->tag=9U,_tmp5D->f1=req1;_tmp5D;}));});
if(!({void*_tmp1C5=(void*)({struct Cyc_Absyn_ValueofType_Absyn_Type_struct*_tmp57=_cycalloc(sizeof(*_tmp57));_tmp57->tag=9U,_tmp57->f1=req0;_tmp57;});Cyc_Tcdecl_check_type(_tmp1C5,subst_w1);})){
({struct Cyc_String_pa_PrintArg_struct _tmp5A=({struct Cyc_String_pa_PrintArg_struct _tmp140;_tmp140.tag=0U,_tmp140.f1=(struct _fat_ptr)((struct _fat_ptr)v);_tmp140;});struct Cyc_String_pa_PrintArg_struct _tmp5B=({struct Cyc_String_pa_PrintArg_struct _tmp13F;_tmp13F.tag=0U,({
struct _fat_ptr _tmp1C6=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_exp2string(req0));_tmp13F.f1=_tmp1C6;});_tmp13F;});struct Cyc_String_pa_PrintArg_struct _tmp5C=({struct Cyc_String_pa_PrintArg_struct _tmp13E;_tmp13E.tag=0U,({struct _fat_ptr _tmp1C7=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_exp2string(req1));_tmp13E.f1=_tmp1C7;});_tmp13E;});void*_tmp58[3U];_tmp58[0]=& _tmp5A,_tmp58[1]=& _tmp5B,_tmp58[2]=& _tmp5C;({unsigned _tmp1CA=loc;struct _fat_ptr*_tmp1C9=msg;struct _fat_ptr _tmp1C8=({const char*_tmp59="type %s : mismatch on @requires clauses: %s != %s";_tag_fat(_tmp59,sizeof(char),50U);});Cyc_Tcdecl_merr(_tmp1CA,_tmp1C9,_tmp1C8,_tag_fat(_tmp58,sizeof(void*),3U));});});
res=0;}}else{
# 219
if(req0 == 0 && req1 != 0 ||
 req0 != 0 && req1 == 0){
({struct Cyc_String_pa_PrintArg_struct _tmp60=({struct Cyc_String_pa_PrintArg_struct _tmp141;_tmp141.tag=0U,_tmp141.f1=(struct _fat_ptr)((struct _fat_ptr)v);_tmp141;});void*_tmp5E[1U];_tmp5E[0]=& _tmp60;({unsigned _tmp1CD=loc;struct _fat_ptr*_tmp1CC=msg;struct _fat_ptr _tmp1CB=({const char*_tmp5F="type %s : mismatch on @requires clauses";_tag_fat(_tmp5F,sizeof(char),40U);});Cyc_Tcdecl_merr(_tmp1CD,_tmp1CC,_tmp1CB,_tag_fat(_tmp5E,sizeof(void*),1U));});});
res=0;}}{
# 191
void*subst_t1=
# 225
Cyc_Tcutil_substitute(inst,t1);
if(!Cyc_Tcdecl_check_type(t0,subst_t1)){
({struct Cyc_String_pa_PrintArg_struct _tmp63=({struct Cyc_String_pa_PrintArg_struct _tmp145;_tmp145.tag=0U,_tmp145.f1=(struct _fat_ptr)((struct _fat_ptr)v);_tmp145;});struct Cyc_String_pa_PrintArg_struct _tmp64=({struct Cyc_String_pa_PrintArg_struct _tmp144;_tmp144.tag=0U,_tmp144.f1=(struct _fat_ptr)((struct _fat_ptr)*fn0);_tmp144;});struct Cyc_String_pa_PrintArg_struct _tmp65=({struct Cyc_String_pa_PrintArg_struct _tmp143;_tmp143.tag=0U,({
struct _fat_ptr _tmp1CE=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t0));_tmp143.f1=_tmp1CE;});_tmp143;});struct Cyc_String_pa_PrintArg_struct _tmp66=({struct Cyc_String_pa_PrintArg_struct _tmp142;_tmp142.tag=0U,({struct _fat_ptr _tmp1CF=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(subst_t1));_tmp142.f1=_tmp1CF;});_tmp142;});void*_tmp61[4U];_tmp61[0]=& _tmp63,_tmp61[1]=& _tmp64,_tmp61[2]=& _tmp65,_tmp61[3]=& _tmp66;({unsigned _tmp1D2=loc;struct _fat_ptr*_tmp1D1=msg;struct _fat_ptr _tmp1D0=({const char*_tmp62="type %s : type mismatch on field %s: %s != %s";_tag_fat(_tmp62,sizeof(char),46U);});Cyc_Tcdecl_merr(_tmp1D2,_tmp1D1,_tmp1D0,_tag_fat(_tmp61,sizeof(void*),4U));});});
Cyc_Unify_explain_failure();
res=0;}}}}}
# 234
if(f0s != 0){
({struct Cyc_String_pa_PrintArg_struct _tmp69=({struct Cyc_String_pa_PrintArg_struct _tmp147;_tmp147.tag=0U,_tmp147.f1=(struct _fat_ptr)((struct _fat_ptr)v);_tmp147;});struct Cyc_String_pa_PrintArg_struct _tmp6A=({struct Cyc_String_pa_PrintArg_struct _tmp146;_tmp146.tag=0U,_tmp146.f1=(struct _fat_ptr)((struct _fat_ptr)*((struct Cyc_Absyn_Aggrfield*)f0s->hd)->name);_tmp146;});void*_tmp67[2U];_tmp67[0]=& _tmp69,_tmp67[1]=& _tmp6A;({unsigned _tmp1D5=loc;struct _fat_ptr*_tmp1D4=msg;struct _fat_ptr _tmp1D3=({const char*_tmp68="type %s is missing field %s";_tag_fat(_tmp68,sizeof(char),28U);});Cyc_Tcdecl_merr(_tmp1D5,_tmp1D4,_tmp1D3,_tag_fat(_tmp67,sizeof(void*),2U));});});
res=0;}
# 234
if(f1s != 0){
# 239
({struct Cyc_String_pa_PrintArg_struct _tmp6D=({struct Cyc_String_pa_PrintArg_struct _tmp149;_tmp149.tag=0U,_tmp149.f1=(struct _fat_ptr)((struct _fat_ptr)v);_tmp149;});struct Cyc_String_pa_PrintArg_struct _tmp6E=({struct Cyc_String_pa_PrintArg_struct _tmp148;_tmp148.tag=0U,_tmp148.f1=(struct _fat_ptr)((struct _fat_ptr)*((struct Cyc_Absyn_Aggrfield*)f1s->hd)->name);_tmp148;});void*_tmp6B[2U];_tmp6B[0]=& _tmp6D,_tmp6B[1]=& _tmp6E;({unsigned _tmp1D8=loc;struct _fat_ptr*_tmp1D7=msg;struct _fat_ptr _tmp1D6=({const char*_tmp6C="type %s has extra field %s";_tag_fat(_tmp6C,sizeof(char),27U);});Cyc_Tcdecl_merr(_tmp1D8,_tmp1D7,_tmp1D6,_tag_fat(_tmp6B,sizeof(void*),2U));});});
res=0;}
# 234
if(tagged0 != tagged1){
# 244
({struct Cyc_String_pa_PrintArg_struct _tmp71=({struct Cyc_String_pa_PrintArg_struct _tmp14A;_tmp14A.tag=0U,_tmp14A.f1=(struct _fat_ptr)((struct _fat_ptr)v);_tmp14A;});void*_tmp6F[1U];_tmp6F[0]=& _tmp71;({unsigned _tmp1DB=loc;struct _fat_ptr*_tmp1DA=msg;struct _fat_ptr _tmp1D9=({const char*_tmp70="%s : previous declaration disagrees with @tagged qualifier";_tag_fat(_tmp70,sizeof(char),59U);});Cyc_Tcdecl_merr(_tmp1DB,_tmp1DA,_tmp1D9,_tag_fat(_tmp6F,sizeof(void*),1U));});});
res=0;}
# 234
d2=d0;
# 249
goto _LL3;}}}}_LL3:;}
# 252
if(!res)return 0;if((int)new_scope == (int)d2->sc)
# 255
return d2;else{
# 257
d2=({struct Cyc_Absyn_Aggrdecl*_tmp72=_cycalloc(sizeof(*_tmp72));*_tmp72=*d2;_tmp72;});
d2->sc=new_scope;
return d2;}}}}}
# 38
inline static struct _fat_ptr Cyc_Tcdecl_is_x2string(int is_x){
# 264
return is_x?({const char*_tmp74="@extensible datatype";_tag_fat(_tmp74,sizeof(char),21U);}):({const char*_tmp75="datatype";_tag_fat(_tmp75,sizeof(char),9U);});}struct _tuple16{struct Cyc_Absyn_Tqual f1;void*f2;};
# 38
static struct Cyc_Absyn_Datatypefield*Cyc_Tcdecl_merge_datatypefield(struct Cyc_Absyn_Datatypefield*f0,struct Cyc_Absyn_Datatypefield*f1,struct Cyc_List_List*inst,struct _fat_ptr t,struct _fat_ptr v,struct _fat_ptr*msg){
# 273
unsigned loc=f1->loc;
# 276
if(Cyc_strptrcmp((*f0->name).f2,(*f1->name).f2)!= 0){
({struct Cyc_String_pa_PrintArg_struct _tmp78=({struct Cyc_String_pa_PrintArg_struct _tmp14F;_tmp14F.tag=0U,_tmp14F.f1=(struct _fat_ptr)((struct _fat_ptr)t);_tmp14F;});struct Cyc_String_pa_PrintArg_struct _tmp79=({struct Cyc_String_pa_PrintArg_struct _tmp14E;_tmp14E.tag=0U,_tmp14E.f1=(struct _fat_ptr)((struct _fat_ptr)v);_tmp14E;});struct Cyc_String_pa_PrintArg_struct _tmp7A=({struct Cyc_String_pa_PrintArg_struct _tmp14D;_tmp14D.tag=0U,_tmp14D.f1=(struct _fat_ptr)((struct _fat_ptr)*(*f1->name).f2);_tmp14D;});struct Cyc_String_pa_PrintArg_struct _tmp7B=({struct Cyc_String_pa_PrintArg_struct _tmp14C;_tmp14C.tag=0U,_tmp14C.f1=(struct _fat_ptr)((struct _fat_ptr)*(*f0->name).f2);_tmp14C;});void*_tmp76[4U];_tmp76[0]=& _tmp78,_tmp76[1]=& _tmp79,_tmp76[2]=& _tmp7A,_tmp76[3]=& _tmp7B;({unsigned _tmp1DE=loc;struct _fat_ptr*_tmp1DD=msg;struct _fat_ptr _tmp1DC=({const char*_tmp77="%s %s: field name mismatch %s != %s";_tag_fat(_tmp77,sizeof(char),36U);});Cyc_Tcdecl_merr(_tmp1DE,_tmp1DD,_tmp1DC,_tag_fat(_tmp76,sizeof(void*),4U));});});
# 279
return 0;}{
# 276
struct _fat_ptr name=*(*f0->name).f2;
# 285
struct _tuple11 _stmttmp6=({
enum Cyc_Absyn_Scope _tmp1E4=f0->sc;enum Cyc_Absyn_Scope _tmp1E3=f1->sc;struct _fat_ptr _tmp1E2=(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp92=({struct Cyc_String_pa_PrintArg_struct _tmp15C;_tmp15C.tag=0U,_tmp15C.f1=(struct _fat_ptr)((struct _fat_ptr)t);_tmp15C;});struct Cyc_String_pa_PrintArg_struct _tmp93=({struct Cyc_String_pa_PrintArg_struct _tmp15B;_tmp15B.tag=0U,_tmp15B.f1=(struct _fat_ptr)((struct _fat_ptr)v);_tmp15B;});void*_tmp90[2U];_tmp90[0]=& _tmp92,_tmp90[1]=& _tmp93;({struct _fat_ptr _tmp1DF=({const char*_tmp91="in %s %s, field";_tag_fat(_tmp91,sizeof(char),16U);});Cyc_aprintf(_tmp1DF,_tag_fat(_tmp90,sizeof(void*),2U));});});struct _fat_ptr _tmp1E1=name;unsigned _tmp1E0=loc;Cyc_Tcdecl_merge_scope(_tmp1E4,_tmp1E3,_tmp1E2,_tmp1E1,_tmp1E0,msg,0);});
# 285
int _tmp7D;enum Cyc_Absyn_Scope _tmp7C;_LL1: _tmp7C=_stmttmp6.f1;_tmp7D=_stmttmp6.f2;_LL2: {enum Cyc_Absyn_Scope new_scope=_tmp7C;int res=_tmp7D;
# 289
struct Cyc_List_List*typs0=f0->typs;
struct Cyc_List_List*typs1=f1->typs;
if(({int _tmp1E5=Cyc_List_length(typs0);_tmp1E5 != Cyc_List_length(typs1);})){
({struct Cyc_String_pa_PrintArg_struct _tmp80=({struct Cyc_String_pa_PrintArg_struct _tmp152;_tmp152.tag=0U,_tmp152.f1=(struct _fat_ptr)((struct _fat_ptr)t);_tmp152;});struct Cyc_String_pa_PrintArg_struct _tmp81=({struct Cyc_String_pa_PrintArg_struct _tmp151;_tmp151.tag=0U,_tmp151.f1=(struct _fat_ptr)((struct _fat_ptr)v);_tmp151;});struct Cyc_String_pa_PrintArg_struct _tmp82=({struct Cyc_String_pa_PrintArg_struct _tmp150;_tmp150.tag=0U,_tmp150.f1=(struct _fat_ptr)((struct _fat_ptr)name);_tmp150;});void*_tmp7E[3U];_tmp7E[0]=& _tmp80,_tmp7E[1]=& _tmp81,_tmp7E[2]=& _tmp82;({unsigned _tmp1E8=loc;struct _fat_ptr*_tmp1E7=msg;struct _fat_ptr _tmp1E6=({const char*_tmp7F="%s %s, field %s: parameter number mismatch";_tag_fat(_tmp7F,sizeof(char),43U);});Cyc_Tcdecl_merr(_tmp1E8,_tmp1E7,_tmp1E6,_tag_fat(_tmp7E,sizeof(void*),3U));});});
res=0;}
# 291
for(0;
# 296
typs0 != 0 && typs1 != 0;(typs0=typs0->tl,typs1=typs1->tl)){
# 298
if(!Cyc_Absyn_equal_tqual((*((struct _tuple16*)typs0->hd)).f1,(*((struct _tuple16*)typs1->hd)).f1)){
({struct Cyc_String_pa_PrintArg_struct _tmp85=({struct Cyc_String_pa_PrintArg_struct _tmp155;_tmp155.tag=0U,_tmp155.f1=(struct _fat_ptr)((struct _fat_ptr)t);_tmp155;});struct Cyc_String_pa_PrintArg_struct _tmp86=({struct Cyc_String_pa_PrintArg_struct _tmp154;_tmp154.tag=0U,_tmp154.f1=(struct _fat_ptr)((struct _fat_ptr)v);_tmp154;});struct Cyc_String_pa_PrintArg_struct _tmp87=({struct Cyc_String_pa_PrintArg_struct _tmp153;_tmp153.tag=0U,_tmp153.f1=(struct _fat_ptr)((struct _fat_ptr)name);_tmp153;});void*_tmp83[3U];_tmp83[0]=& _tmp85,_tmp83[1]=& _tmp86,_tmp83[2]=& _tmp87;({unsigned _tmp1EB=loc;struct _fat_ptr*_tmp1EA=msg;struct _fat_ptr _tmp1E9=({const char*_tmp84="%s %s, field %s: parameter qualifier";_tag_fat(_tmp84,sizeof(char),37U);});Cyc_Tcdecl_merr(_tmp1EB,_tmp1EA,_tmp1E9,_tag_fat(_tmp83,sizeof(void*),3U));});});
res=0;}{
# 298
void*subst_t1=
# 302
Cyc_Tcutil_substitute(inst,(*((struct _tuple16*)typs1->hd)).f2);
# 304
if(!Cyc_Tcdecl_check_type((*((struct _tuple16*)typs0->hd)).f2,subst_t1)){
({struct Cyc_String_pa_PrintArg_struct _tmp8A=({struct Cyc_String_pa_PrintArg_struct _tmp15A;_tmp15A.tag=0U,_tmp15A.f1=(struct _fat_ptr)((struct _fat_ptr)t);_tmp15A;});struct Cyc_String_pa_PrintArg_struct _tmp8B=({struct Cyc_String_pa_PrintArg_struct _tmp159;_tmp159.tag=0U,_tmp159.f1=(struct _fat_ptr)((struct _fat_ptr)v);_tmp159;});struct Cyc_String_pa_PrintArg_struct _tmp8C=({struct Cyc_String_pa_PrintArg_struct _tmp158;_tmp158.tag=0U,_tmp158.f1=(struct _fat_ptr)((struct _fat_ptr)name);_tmp158;});struct Cyc_String_pa_PrintArg_struct _tmp8D=({struct Cyc_String_pa_PrintArg_struct _tmp157;_tmp157.tag=0U,({
struct _fat_ptr _tmp1EC=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string((*((struct _tuple16*)typs0->hd)).f2));_tmp157.f1=_tmp1EC;});_tmp157;});struct Cyc_String_pa_PrintArg_struct _tmp8E=({struct Cyc_String_pa_PrintArg_struct _tmp156;_tmp156.tag=0U,({struct _fat_ptr _tmp1ED=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(subst_t1));_tmp156.f1=_tmp1ED;});_tmp156;});void*_tmp88[5U];_tmp88[0]=& _tmp8A,_tmp88[1]=& _tmp8B,_tmp88[2]=& _tmp8C,_tmp88[3]=& _tmp8D,_tmp88[4]=& _tmp8E;({unsigned _tmp1F0=loc;struct _fat_ptr*_tmp1EF=msg;struct _fat_ptr _tmp1EE=({const char*_tmp89="%s %s, field %s: parameter type mismatch %s != %s";_tag_fat(_tmp89,sizeof(char),50U);});Cyc_Tcdecl_merr(_tmp1F0,_tmp1EF,_tmp1EE,_tag_fat(_tmp88,sizeof(void*),5U));});});
Cyc_Unify_explain_failure();
res=0;}}}
# 312
if(!res)return 0;if((int)f0->sc != (int)new_scope){
# 314
struct Cyc_Absyn_Datatypefield*f2=({struct Cyc_Absyn_Datatypefield*_tmp8F=_cycalloc(sizeof(*_tmp8F));*_tmp8F=*f0;_tmp8F;});
f2->sc=new_scope;
return f2;}else{
return f0;}}}}
# 38
static struct _tuple16*Cyc_Tcdecl_substitute_datatypefield_f2(struct Cyc_List_List*inst,struct _tuple16*x){
# 323
void*_tmp95;struct Cyc_Absyn_Tqual _tmp94;_LL1: _tmp94=x->f1;_tmp95=x->f2;_LL2: {struct Cyc_Absyn_Tqual tq=_tmp94;void*t=_tmp95;
return({struct _tuple16*_tmp96=_cycalloc(sizeof(*_tmp96));_tmp96->f1=tq,({void*_tmp1F1=Cyc_Tcutil_substitute(inst,t);_tmp96->f2=_tmp1F1;});_tmp96;});}}
# 38
static struct Cyc_Absyn_Datatypefield*Cyc_Tcdecl_substitute_datatypefield(struct Cyc_List_List*inst1,struct Cyc_Absyn_Datatypefield*f1){
# 331
struct Cyc_Absyn_Datatypefield*f0=({struct Cyc_Absyn_Datatypefield*_tmp98=_cycalloc(sizeof(*_tmp98));*_tmp98=*f1;_tmp98;});
({struct Cyc_List_List*_tmp1F4=({struct Cyc_List_List*(*_tmp1F3)(struct _tuple16*(*f)(struct Cyc_List_List*,struct _tuple16*),struct Cyc_List_List*env,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp97)(struct _tuple16*(*f)(struct Cyc_List_List*,struct _tuple16*),struct Cyc_List_List*env,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct _tuple16*(*f)(struct Cyc_List_List*,struct _tuple16*),struct Cyc_List_List*env,struct Cyc_List_List*x))Cyc_List_map_c;_tmp97;});struct Cyc_List_List*_tmp1F2=inst1;_tmp1F3(Cyc_Tcdecl_substitute_datatypefield_f2,_tmp1F2,f1->typs);});f0->typs=_tmp1F4;});
# 334
return f0;}
# 38
static struct Cyc_List_List*Cyc_Tcdecl_merge_xdatatype_fields(struct Cyc_List_List*f0s,struct Cyc_List_List*f1s,struct Cyc_List_List*inst,struct Cyc_List_List*tvs0,struct Cyc_List_List*tvs1,int*res,int*incl,struct _fat_ptr t,struct _fat_ptr v,unsigned loc,struct _fat_ptr*msg){
# 350
struct Cyc_List_List**f2sp=({struct Cyc_List_List**_tmp9C=_cycalloc(sizeof(*_tmp9C));*_tmp9C=0;_tmp9C;});
struct Cyc_List_List**next=f2sp;
int cmp=-1;
# 354
for(0;f0s != 0 && f1s != 0;f1s=f1s->tl){
while(f0s != 0 &&(cmp=Cyc_Absyn_qvar_cmp(((struct Cyc_Absyn_Datatypefield*)f0s->hd)->name,((struct Cyc_Absyn_Datatypefield*)f1s->hd)->name))< 0){
struct Cyc_List_List*node=({struct Cyc_List_List*_tmp99=_cycalloc(sizeof(*_tmp99));_tmp99->hd=(struct Cyc_Absyn_Datatypefield*)f0s->hd,_tmp99->tl=0;_tmp99;});
*next=node;
next=& node->tl;
f0s=f0s->tl;}
# 361
if(f0s == 0 || cmp > 0){
# 363
*incl=0;{
struct Cyc_List_List*node=({struct Cyc_List_List*_tmp9A=_cycalloc(sizeof(*_tmp9A));({struct Cyc_Absyn_Datatypefield*_tmp1F5=Cyc_Tcdecl_substitute_datatypefield(inst,(struct Cyc_Absyn_Datatypefield*)f1s->hd);_tmp9A->hd=_tmp1F5;}),_tmp9A->tl=0;_tmp9A;});
*next=node;
next=& node->tl;}}else{
# 369
struct Cyc_Absyn_Datatypefield*f2=Cyc_Tcdecl_merge_datatypefield((struct Cyc_Absyn_Datatypefield*)f0s->hd,(struct Cyc_Absyn_Datatypefield*)f1s->hd,inst,t,v,msg);
if(f2 != 0){
if(f2 != (struct Cyc_Absyn_Datatypefield*)f0s->hd)*incl=0;{struct Cyc_List_List*node=({struct Cyc_List_List*_tmp9B=_cycalloc(sizeof(*_tmp9B));
_tmp9B->hd=f2,_tmp9B->tl=0;_tmp9B;});
*next=node;
next=& node->tl;}}else{
# 376
*res=0;}
# 378
f0s=f0s->tl;}}
# 383
if(f1s != 0){
*incl=0;
*next=f1s;}else{
# 387
*next=f0s;}
# 390
return*f2sp;}struct _tuple17{struct Cyc_List_List*f1;struct Cyc_List_List*f2;};
# 38
static struct _tuple17 Cyc_Tcdecl_split(struct Cyc_List_List*f){
# 395
if(f == 0)return({struct _tuple17 _tmp15D;_tmp15D.f1=0,_tmp15D.f2=0;_tmp15D;});if(f->tl == 0)
return({struct _tuple17 _tmp15E;_tmp15E.f1=f,_tmp15E.f2=0;_tmp15E;});{
# 395
struct _tuple17 _stmttmp7=
# 397
Cyc_Tcdecl_split(((struct Cyc_List_List*)_check_null(f->tl))->tl);
# 395
struct Cyc_List_List*_tmp9E;struct Cyc_List_List*_tmp9D;_LL1: _tmp9D=_stmttmp7.f1;_tmp9E=_stmttmp7.f2;_LL2: {struct Cyc_List_List*f1=_tmp9D;struct Cyc_List_List*f2=_tmp9E;
# 398
return({struct _tuple17 _tmp15F;({struct Cyc_List_List*_tmp1F7=({struct Cyc_List_List*_tmp9F=_cycalloc(sizeof(*_tmp9F));_tmp9F->hd=f->hd,_tmp9F->tl=f1;_tmp9F;});_tmp15F.f1=_tmp1F7;}),({struct Cyc_List_List*_tmp1F6=({struct Cyc_List_List*_tmpA0=_cycalloc(sizeof(*_tmpA0));_tmpA0->hd=((struct Cyc_List_List*)_check_null(f->tl))->hd,_tmpA0->tl=f2;_tmpA0;});_tmp15F.f2=_tmp1F6;});_tmp15F;});}}}
# 38
struct Cyc_List_List*Cyc_Tcdecl_sort_xdatatype_fields(struct Cyc_List_List*f,int*res,struct _fat_ptr*v,unsigned loc,struct _fat_ptr*msg){
# 406
struct _tuple17 _stmttmp8=Cyc_Tcdecl_split(f);struct Cyc_List_List*_tmpA2;struct Cyc_List_List*_tmpA1;_LL1: _tmpA1=_stmttmp8.f1;_tmpA2=_stmttmp8.f2;_LL2: {struct Cyc_List_List*f1=_tmpA1;struct Cyc_List_List*f2=_tmpA2;
if(f1 != 0 && f1->tl != 0)
f1=Cyc_Tcdecl_sort_xdatatype_fields(f1,res,v,loc,msg);
# 407
if(
# 409
f2 != 0 && f2->tl != 0)
f2=Cyc_Tcdecl_sort_xdatatype_fields(f2,res,v,loc,msg);
# 407
return({
# 411
struct Cyc_List_List*_tmp1FE=f1;struct Cyc_List_List*_tmp1FD=f2;int*_tmp1FC=res;int*_tmp1FB=({int*_tmpA3=_cycalloc_atomic(sizeof(*_tmpA3));*_tmpA3=1;_tmpA3;});struct _fat_ptr _tmp1FA=({const char*_tmpA4="@extensible datatype";_tag_fat(_tmpA4,sizeof(char),21U);});struct _fat_ptr _tmp1F9=*v;unsigned _tmp1F8=loc;Cyc_Tcdecl_merge_xdatatype_fields(_tmp1FE,_tmp1FD,0,0,0,_tmp1FC,_tmp1FB,_tmp1FA,_tmp1F9,_tmp1F8,msg);});}}struct _tuple18{struct Cyc_Core_Opt*f1;struct Cyc_Core_Opt*f2;};
# 38
struct Cyc_Absyn_Datatypedecl*Cyc_Tcdecl_merge_datatypedecl(struct Cyc_Absyn_Datatypedecl*d0,struct Cyc_Absyn_Datatypedecl*d1,unsigned loc,struct _fat_ptr*msg){
# 417
struct _fat_ptr v=Cyc_Absynpp_qvar2string(d0->name);
struct _fat_ptr t=({const char*_tmpBC="datatype";_tag_fat(_tmpBC,sizeof(char),9U);});
int res=1;
# 424
if(d0->is_extensible != d1->is_extensible){
({struct Cyc_String_pa_PrintArg_struct _tmpA7=({struct Cyc_String_pa_PrintArg_struct _tmp162;_tmp162.tag=0U,_tmp162.f1=(struct _fat_ptr)((struct _fat_ptr)v);_tmp162;});struct Cyc_String_pa_PrintArg_struct _tmpA8=({struct Cyc_String_pa_PrintArg_struct _tmp161;_tmp161.tag=0U,({
struct _fat_ptr _tmp1FF=(struct _fat_ptr)((struct _fat_ptr)Cyc_Tcdecl_is_x2string(d0->is_extensible));_tmp161.f1=_tmp1FF;});_tmp161;});struct Cyc_String_pa_PrintArg_struct _tmpA9=({struct Cyc_String_pa_PrintArg_struct _tmp160;_tmp160.tag=0U,({struct _fat_ptr _tmp200=(struct _fat_ptr)((struct _fat_ptr)Cyc_Tcdecl_is_x2string(d1->is_extensible));_tmp160.f1=_tmp200;});_tmp160;});void*_tmpA5[3U];_tmpA5[0]=& _tmpA7,_tmpA5[1]=& _tmpA8,_tmpA5[2]=& _tmpA9;({unsigned _tmp203=loc;struct _fat_ptr*_tmp202=msg;struct _fat_ptr _tmp201=({const char*_tmpA6="expected %s to be a %s instead of a %s";_tag_fat(_tmpA6,sizeof(char),39U);});Cyc_Tcdecl_merr(_tmp203,_tmp202,_tmp201,_tag_fat(_tmpA5,sizeof(void*),3U));});});
res=0;}else{
# 429
t=Cyc_Tcdecl_is_x2string(d0->is_extensible);}
# 432
if(!Cyc_Tcdecl_check_tvs(d0->tvs,d1->tvs,t,v,loc,msg))return 0;{struct _tuple11 _stmttmp9=
# 435
Cyc_Tcdecl_merge_scope(d0->sc,d1->sc,t,v,loc,msg,0);
# 432
int _tmpAB;enum Cyc_Absyn_Scope _tmpAA;_LL1: _tmpAA=_stmttmp9.f1;_tmpAB=_stmttmp9.f2;_LL2: {enum Cyc_Absyn_Scope new_scope=_tmpAA;int res_scope=_tmpAB;
# 436
if(!res_scope)res=0;{struct Cyc_Absyn_Datatypedecl*d2;
# 441
{struct _tuple18 _stmttmpA=({struct _tuple18 _tmp169;_tmp169.f1=d0->fields,_tmp169.f2=d1->fields;_tmp169;});struct _tuple18 _tmpAC=_stmttmpA;struct Cyc_List_List*_tmpAE;struct Cyc_List_List*_tmpAD;if(_tmpAC.f2 == 0){_LL4: _LL5:
 d2=d0;goto _LL3;}else{if(_tmpAC.f1 == 0){_LL6: _LL7:
 d2=d1;goto _LL3;}else{_LL8: _tmpAD=(struct Cyc_List_List*)(_tmpAC.f1)->v;_tmpAE=(struct Cyc_List_List*)(_tmpAC.f2)->v;_LL9: {struct Cyc_List_List*f0s=_tmpAD;struct Cyc_List_List*f1s=_tmpAE;
# 455 "tcdecl.cyc"
struct Cyc_List_List*inst=Cyc_Tcdecl_build_tvs_map(d0->tvs,d1->tvs);
# 458
if(d0->is_extensible){
# 462
int incl=1;
struct Cyc_List_List*f2s=Cyc_Tcdecl_merge_xdatatype_fields(f0s,f1s,inst,d0->tvs,d1->tvs,& res,& incl,t,v,loc,msg);
# 466
if(incl)
d2=d0;else{
# 469
d2=({struct Cyc_Absyn_Datatypedecl*_tmpAF=_cycalloc(sizeof(*_tmpAF));*_tmpAF=*d0;_tmpAF;});
d2->sc=new_scope;
({struct Cyc_Core_Opt*_tmp204=({struct Cyc_Core_Opt*_tmpB0=_cycalloc(sizeof(*_tmpB0));_tmpB0->v=f2s;_tmpB0;});d2->fields=_tmp204;});}}else{
# 475
for(0;f0s != 0 && f1s != 0;(f0s=f0s->tl,f1s=f1s->tl)){
Cyc_Tcdecl_merge_datatypefield((struct Cyc_Absyn_Datatypefield*)f0s->hd,(struct Cyc_Absyn_Datatypefield*)f1s->hd,inst,t,v,msg);}
# 478
if(f0s != 0){
({struct Cyc_String_pa_PrintArg_struct _tmpB3=({struct Cyc_String_pa_PrintArg_struct _tmp165;_tmp165.tag=0U,_tmp165.f1=(struct _fat_ptr)((struct _fat_ptr)t);_tmp165;});struct Cyc_String_pa_PrintArg_struct _tmpB4=({struct Cyc_String_pa_PrintArg_struct _tmp164;_tmp164.tag=0U,_tmp164.f1=(struct _fat_ptr)((struct _fat_ptr)v);_tmp164;});struct Cyc_String_pa_PrintArg_struct _tmpB5=({struct Cyc_String_pa_PrintArg_struct _tmp163;_tmp163.tag=0U,_tmp163.f1=(struct _fat_ptr)((struct _fat_ptr)*(*((struct Cyc_Absyn_Datatypefield*)f0s->hd)->name).f2);_tmp163;});void*_tmpB1[3U];_tmpB1[0]=& _tmpB3,_tmpB1[1]=& _tmpB4,_tmpB1[2]=& _tmpB5;({unsigned _tmp207=loc;struct _fat_ptr*_tmp206=msg;struct _fat_ptr _tmp205=({const char*_tmpB2="%s %s is missing field %s";_tag_fat(_tmpB2,sizeof(char),26U);});Cyc_Tcdecl_merr(_tmp207,_tmp206,_tmp205,_tag_fat(_tmpB1,sizeof(void*),3U));});});
res=0;}
# 478
if(f1s != 0){
# 483
({struct Cyc_String_pa_PrintArg_struct _tmpB8=({struct Cyc_String_pa_PrintArg_struct _tmp168;_tmp168.tag=0U,_tmp168.f1=(struct _fat_ptr)((struct _fat_ptr)t);_tmp168;});struct Cyc_String_pa_PrintArg_struct _tmpB9=({struct Cyc_String_pa_PrintArg_struct _tmp167;_tmp167.tag=0U,_tmp167.f1=(struct _fat_ptr)((struct _fat_ptr)v);_tmp167;});struct Cyc_String_pa_PrintArg_struct _tmpBA=({struct Cyc_String_pa_PrintArg_struct _tmp166;_tmp166.tag=0U,_tmp166.f1=(struct _fat_ptr)((struct _fat_ptr)*(*((struct Cyc_Absyn_Datatypefield*)f1s->hd)->name).f2);_tmp166;});void*_tmpB6[3U];_tmpB6[0]=& _tmpB8,_tmpB6[1]=& _tmpB9,_tmpB6[2]=& _tmpBA;({unsigned _tmp20A=loc;struct _fat_ptr*_tmp209=msg;struct _fat_ptr _tmp208=({const char*_tmpB7="%s %s has extra field %s";_tag_fat(_tmpB7,sizeof(char),25U);});Cyc_Tcdecl_merr(_tmp20A,_tmp209,_tmp208,_tag_fat(_tmpB6,sizeof(void*),3U));});});
res=0;}
# 478
d2=d0;}
# 488
goto _LL3;}}}_LL3:;}
# 491
if(!res)return 0;if((int)new_scope == (int)d2->sc)
# 494
return d2;else{
# 496
d2=({struct Cyc_Absyn_Datatypedecl*_tmpBB=_cycalloc(sizeof(*_tmpBB));*_tmpBB=*d2;_tmpBB;});
d2->sc=new_scope;
return d2;}}}}}
# 38 "tcdecl.cyc"
struct Cyc_Absyn_Enumdecl*Cyc_Tcdecl_merge_enumdecl(struct Cyc_Absyn_Enumdecl*d0,struct Cyc_Absyn_Enumdecl*d1,unsigned loc,struct _fat_ptr*msg){
# 504 "tcdecl.cyc"
struct _fat_ptr v=Cyc_Absynpp_qvar2string(d0->name);
int res=1;
# 508
struct _tuple11 _stmttmpB=({enum Cyc_Absyn_Scope _tmp20F=d0->sc;enum Cyc_Absyn_Scope _tmp20E=d1->sc;struct _fat_ptr _tmp20D=({const char*_tmpD5="enum";_tag_fat(_tmpD5,sizeof(char),5U);});struct _fat_ptr _tmp20C=v;unsigned _tmp20B=loc;Cyc_Tcdecl_merge_scope(_tmp20F,_tmp20E,_tmp20D,_tmp20C,_tmp20B,msg,1);});int _tmpBE;enum Cyc_Absyn_Scope _tmpBD;_LL1: _tmpBD=_stmttmpB.f1;_tmpBE=_stmttmpB.f2;_LL2: {enum Cyc_Absyn_Scope new_scope=_tmpBD;int res_scope=_tmpBE;
if(!res_scope)res=0;{struct Cyc_Absyn_Enumdecl*d2;
# 514
{struct _tuple18 _stmttmpC=({struct _tuple18 _tmp170;_tmp170.f1=d0->fields,_tmp170.f2=d1->fields;_tmp170;});struct _tuple18 _tmpBF=_stmttmpC;struct Cyc_List_List*_tmpC1;struct Cyc_List_List*_tmpC0;if(_tmpBF.f2 == 0){_LL4: _LL5:
 d2=d0;goto _LL3;}else{if(_tmpBF.f1 == 0){_LL6: _LL7:
 d2=d1;goto _LL3;}else{_LL8: _tmpC0=(struct Cyc_List_List*)(_tmpBF.f1)->v;_tmpC1=(struct Cyc_List_List*)(_tmpBF.f2)->v;_LL9: {struct Cyc_List_List*f0s=_tmpC0;struct Cyc_List_List*f1s=_tmpC1;
# 519
for(0;f0s != 0 && f1s != 0;(f0s=f0s->tl,f1s=f1s->tl)){
struct Cyc_Absyn_Enumfield*_stmttmpD=(struct Cyc_Absyn_Enumfield*)f0s->hd;unsigned _tmpC4;struct Cyc_Absyn_Exp*_tmpC3;struct _fat_ptr*_tmpC2;_LLB: _tmpC2=(_stmttmpD->name)->f2;_tmpC3=_stmttmpD->tag;_tmpC4=_stmttmpD->loc;_LLC: {struct _fat_ptr*fname0=_tmpC2;struct Cyc_Absyn_Exp*tag0=_tmpC3;unsigned loc0=_tmpC4;
struct Cyc_Absyn_Enumfield*_stmttmpE=(struct Cyc_Absyn_Enumfield*)f1s->hd;unsigned _tmpC7;struct Cyc_Absyn_Exp*_tmpC6;struct _fat_ptr*_tmpC5;_LLE: _tmpC5=(_stmttmpE->name)->f2;_tmpC6=_stmttmpE->tag;_tmpC7=_stmttmpE->loc;_LLF: {struct _fat_ptr*fname1=_tmpC5;struct Cyc_Absyn_Exp*tag1=_tmpC6;unsigned loc1=_tmpC7;
# 523
if(Cyc_strptrcmp(fname0,fname1)!= 0){
({struct Cyc_String_pa_PrintArg_struct _tmpCA=({struct Cyc_String_pa_PrintArg_struct _tmp16C;_tmp16C.tag=0U,_tmp16C.f1=(struct _fat_ptr)((struct _fat_ptr)v);_tmp16C;});struct Cyc_String_pa_PrintArg_struct _tmpCB=({struct Cyc_String_pa_PrintArg_struct _tmp16B;_tmp16B.tag=0U,_tmp16B.f1=(struct _fat_ptr)((struct _fat_ptr)*fname0);_tmp16B;});struct Cyc_String_pa_PrintArg_struct _tmpCC=({struct Cyc_String_pa_PrintArg_struct _tmp16A;_tmp16A.tag=0U,_tmp16A.f1=(struct _fat_ptr)((struct _fat_ptr)*fname1);_tmp16A;});void*_tmpC8[3U];_tmpC8[0]=& _tmpCA,_tmpC8[1]=& _tmpCB,_tmpC8[2]=& _tmpCC;({unsigned _tmp212=loc;struct _fat_ptr*_tmp211=msg;struct _fat_ptr _tmp210=({const char*_tmpC9="enum %s: field name mismatch %s != %s";_tag_fat(_tmpC9,sizeof(char),38U);});Cyc_Tcdecl_merr(_tmp212,_tmp211,_tmp210,_tag_fat(_tmpC8,sizeof(void*),3U));});});
res=0;}
# 523
if(({
# 527
unsigned _tmp213=Cyc_Tcdecl_get_uint_const_value((struct Cyc_Absyn_Exp*)_check_null(tag0));_tmp213 != Cyc_Tcdecl_get_uint_const_value((struct Cyc_Absyn_Exp*)_check_null(tag1));})){
({struct Cyc_String_pa_PrintArg_struct _tmpCF=({struct Cyc_String_pa_PrintArg_struct _tmp16E;_tmp16E.tag=0U,_tmp16E.f1=(struct _fat_ptr)((struct _fat_ptr)v);_tmp16E;});struct Cyc_String_pa_PrintArg_struct _tmpD0=({struct Cyc_String_pa_PrintArg_struct _tmp16D;_tmp16D.tag=0U,_tmp16D.f1=(struct _fat_ptr)((struct _fat_ptr)*fname1);_tmp16D;});void*_tmpCD[2U];_tmpCD[0]=& _tmpCF,_tmpCD[1]=& _tmpD0;({unsigned _tmp216=loc;struct _fat_ptr*_tmp215=msg;struct _fat_ptr _tmp214=({const char*_tmpCE="enum %s, field %s, value mismatch";_tag_fat(_tmpCE,sizeof(char),34U);});Cyc_Tcdecl_merr(_tmp216,_tmp215,_tmp214,_tag_fat(_tmpCD,sizeof(void*),2U));});});
res=0;}}}}
# 532
if(f0s != 0 || f1s != 0){
({struct Cyc_String_pa_PrintArg_struct _tmpD3=({struct Cyc_String_pa_PrintArg_struct _tmp16F;_tmp16F.tag=0U,_tmp16F.f1=(struct _fat_ptr)((struct _fat_ptr)v);_tmp16F;});void*_tmpD1[1U];_tmpD1[0]=& _tmpD3;({unsigned _tmp219=loc;struct _fat_ptr*_tmp218=msg;struct _fat_ptr _tmp217=({const char*_tmpD2="enum %s declarations: number of fields differs";_tag_fat(_tmpD2,sizeof(char),47U);});Cyc_Tcdecl_merr(_tmp219,_tmp218,_tmp217,_tag_fat(_tmpD1,sizeof(void*),1U));});});
res=0;}
# 532
d2=d0;
# 537
goto _LL3;}}}_LL3:;}
# 540
if(!res)return 0;if((int)d2->sc == (int)new_scope)
# 543
return d2;else{
# 545
d2=({struct Cyc_Absyn_Enumdecl*_tmpD4=_cycalloc(sizeof(*_tmpD4));*_tmpD4=*d2;_tmpD4;});
d2->sc=new_scope;
return d2;}}}}
# 38 "tcdecl.cyc"
static struct _tuple11 Cyc_Tcdecl_check_var_or_fn_decl(enum Cyc_Absyn_Scope sc0,void*t0,struct Cyc_Absyn_Tqual tq0,struct Cyc_List_List*atts0,enum Cyc_Absyn_Scope sc1,void*t1,struct Cyc_Absyn_Tqual tq1,struct Cyc_List_List*atts1,struct _fat_ptr t,struct _fat_ptr v,unsigned loc,struct _fat_ptr*msg){
# 555 "tcdecl.cyc"
int res=1;
# 558
struct _tuple11 _stmttmpF=Cyc_Tcdecl_merge_scope(sc0,sc1,t,v,loc,msg,0);int _tmpD7;enum Cyc_Absyn_Scope _tmpD6;_LL1: _tmpD6=_stmttmpF.f1;_tmpD7=_stmttmpF.f2;_LL2: {enum Cyc_Absyn_Scope new_scope=_tmpD6;int res_scope=_tmpD7;
if(!res_scope)res=0;if(!
# 562
Cyc_Tcdecl_check_type(t0,t1)){
struct _fat_ptr s1=Cyc_Absynpp_typ2string(t0);
struct _fat_ptr s2=Cyc_Absynpp_typ2string(t1);
if(({unsigned long _tmp21D=({unsigned long _tmp21C=({unsigned long _tmp21B=({unsigned long _tmp21A=Cyc_strlen((struct _fat_ptr)s1);_tmp21A + Cyc_strlen((struct _fat_ptr)s2);});_tmp21B + Cyc_strlen((struct _fat_ptr)t);});_tmp21C + Cyc_strlen((struct _fat_ptr)v);});_tmp21D + 
Cyc_strlen(({const char*_tmpD8="  was declared  instead of ";_tag_fat(_tmpD8,sizeof(char),28U);}));})> (unsigned long)70)
({struct Cyc_String_pa_PrintArg_struct _tmpDB=({struct Cyc_String_pa_PrintArg_struct _tmp174;_tmp174.tag=0U,_tmp174.f1=(struct _fat_ptr)((struct _fat_ptr)t);_tmp174;});struct Cyc_String_pa_PrintArg_struct _tmpDC=({struct Cyc_String_pa_PrintArg_struct _tmp173;_tmp173.tag=0U,_tmp173.f1=(struct _fat_ptr)((struct _fat_ptr)v);_tmp173;});struct Cyc_String_pa_PrintArg_struct _tmpDD=({struct Cyc_String_pa_PrintArg_struct _tmp172;_tmp172.tag=0U,_tmp172.f1=(struct _fat_ptr)((struct _fat_ptr)s2);_tmp172;});struct Cyc_String_pa_PrintArg_struct _tmpDE=({struct Cyc_String_pa_PrintArg_struct _tmp171;_tmp171.tag=0U,_tmp171.f1=(struct _fat_ptr)((struct _fat_ptr)s1);_tmp171;});void*_tmpD9[4U];_tmpD9[0]=& _tmpDB,_tmpD9[1]=& _tmpDC,_tmpD9[2]=& _tmpDD,_tmpD9[3]=& _tmpDE;({unsigned _tmp220=loc;struct _fat_ptr*_tmp21F=msg;struct _fat_ptr _tmp21E=({const char*_tmpDA="%s %s was declared \n\t%s\n instead of \n\t%s";_tag_fat(_tmpDA,sizeof(char),41U);});Cyc_Tcdecl_merr(_tmp220,_tmp21F,_tmp21E,_tag_fat(_tmpD9,sizeof(void*),4U));});});else{
# 570
({struct Cyc_String_pa_PrintArg_struct _tmpE1=({struct Cyc_String_pa_PrintArg_struct _tmp178;_tmp178.tag=0U,_tmp178.f1=(struct _fat_ptr)((struct _fat_ptr)t);_tmp178;});struct Cyc_String_pa_PrintArg_struct _tmpE2=({struct Cyc_String_pa_PrintArg_struct _tmp177;_tmp177.tag=0U,_tmp177.f1=(struct _fat_ptr)((struct _fat_ptr)v);_tmp177;});struct Cyc_String_pa_PrintArg_struct _tmpE3=({struct Cyc_String_pa_PrintArg_struct _tmp176;_tmp176.tag=0U,_tmp176.f1=(struct _fat_ptr)((struct _fat_ptr)s2);_tmp176;});struct Cyc_String_pa_PrintArg_struct _tmpE4=({struct Cyc_String_pa_PrintArg_struct _tmp175;_tmp175.tag=0U,_tmp175.f1=(struct _fat_ptr)((struct _fat_ptr)s1);_tmp175;});void*_tmpDF[4U];_tmpDF[0]=& _tmpE1,_tmpDF[1]=& _tmpE2,_tmpDF[2]=& _tmpE3,_tmpDF[3]=& _tmpE4;({unsigned _tmp223=loc;struct _fat_ptr*_tmp222=msg;struct _fat_ptr _tmp221=({const char*_tmpE0="%s %s was declared %s instead of %s";_tag_fat(_tmpE0,sizeof(char),36U);});Cyc_Tcdecl_merr(_tmp223,_tmp222,_tmp221,_tag_fat(_tmpDF,sizeof(void*),4U));});});}
# 572
Cyc_Unify_explain_failure();
res=0;}
# 559
if(!
# 577
Cyc_Absyn_equal_tqual(tq0,tq1)){
({struct Cyc_String_pa_PrintArg_struct _tmpE7=({struct Cyc_String_pa_PrintArg_struct _tmp17A;_tmp17A.tag=0U,_tmp17A.f1=(struct _fat_ptr)((struct _fat_ptr)t);_tmp17A;});struct Cyc_String_pa_PrintArg_struct _tmpE8=({struct Cyc_String_pa_PrintArg_struct _tmp179;_tmp179.tag=0U,_tmp179.f1=(struct _fat_ptr)((struct _fat_ptr)v);_tmp179;});void*_tmpE5[2U];_tmpE5[0]=& _tmpE7,_tmpE5[1]=& _tmpE8;({unsigned _tmp226=loc;struct _fat_ptr*_tmp225=msg;struct _fat_ptr _tmp224=({const char*_tmpE6="%s %s has different type qualifiers";_tag_fat(_tmpE6,sizeof(char),36U);});Cyc_Tcdecl_merr(_tmp226,_tmp225,_tmp224,_tag_fat(_tmpE5,sizeof(void*),2U));});});
res=0;}
# 559
if(!
# 583
Cyc_Tcutil_same_atts(atts0,atts1)){
({struct Cyc_String_pa_PrintArg_struct _tmpEB=({struct Cyc_String_pa_PrintArg_struct _tmp17C;_tmp17C.tag=0U,_tmp17C.f1=(struct _fat_ptr)((struct _fat_ptr)t);_tmp17C;});struct Cyc_String_pa_PrintArg_struct _tmpEC=({struct Cyc_String_pa_PrintArg_struct _tmp17B;_tmp17B.tag=0U,_tmp17B.f1=(struct _fat_ptr)((struct _fat_ptr)v);_tmp17B;});void*_tmpE9[2U];_tmpE9[0]=& _tmpEB,_tmpE9[1]=& _tmpEC;({unsigned _tmp229=loc;struct _fat_ptr*_tmp228=msg;struct _fat_ptr _tmp227=({const char*_tmpEA="%s %s has different attributes";_tag_fat(_tmpEA,sizeof(char),31U);});Cyc_Tcdecl_merr(_tmp229,_tmp228,_tmp227,_tag_fat(_tmpE9,sizeof(void*),2U));});});
({void*_tmpED=0U;({struct Cyc___cycFILE*_tmp22B=Cyc_stderr;struct _fat_ptr _tmp22A=({const char*_tmpEE="\tprevious attributes: ";_tag_fat(_tmpEE,sizeof(char),23U);});Cyc_fprintf(_tmp22B,_tmp22A,_tag_fat(_tmpED,sizeof(void*),0U));});});
for(0;atts0 != 0;atts0=atts0->tl){
({struct Cyc_String_pa_PrintArg_struct _tmpF1=({struct Cyc_String_pa_PrintArg_struct _tmp17D;_tmp17D.tag=0U,({struct _fat_ptr _tmp22C=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absyn_attribute2string((void*)atts0->hd));_tmp17D.f1=_tmp22C;});_tmp17D;});void*_tmpEF[1U];_tmpEF[0]=& _tmpF1;({struct Cyc___cycFILE*_tmp22E=Cyc_stderr;struct _fat_ptr _tmp22D=({const char*_tmpF0="%s ";_tag_fat(_tmpF0,sizeof(char),4U);});Cyc_fprintf(_tmp22E,_tmp22D,_tag_fat(_tmpEF,sizeof(void*),1U));});});}
({void*_tmpF2=0U;({struct Cyc___cycFILE*_tmp230=Cyc_stderr;struct _fat_ptr _tmp22F=({const char*_tmpF3="\n\tcurrent attributes: ";_tag_fat(_tmpF3,sizeof(char),23U);});Cyc_fprintf(_tmp230,_tmp22F,_tag_fat(_tmpF2,sizeof(void*),0U));});});
for(0;atts1 != 0;atts1=atts1->tl){
({struct Cyc_String_pa_PrintArg_struct _tmpF6=({struct Cyc_String_pa_PrintArg_struct _tmp17E;_tmp17E.tag=0U,({struct _fat_ptr _tmp231=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absyn_attribute2string((void*)atts1->hd));_tmp17E.f1=_tmp231;});_tmp17E;});void*_tmpF4[1U];_tmpF4[0]=& _tmpF6;({struct Cyc___cycFILE*_tmp233=Cyc_stderr;struct _fat_ptr _tmp232=({const char*_tmpF5="%s ";_tag_fat(_tmpF5,sizeof(char),4U);});Cyc_fprintf(_tmp233,_tmp232,_tag_fat(_tmpF4,sizeof(void*),1U));});});}
({void*_tmpF7=0U;({struct Cyc___cycFILE*_tmp235=Cyc_stderr;struct _fat_ptr _tmp234=({const char*_tmpF8="\n";_tag_fat(_tmpF8,sizeof(char),2U);});Cyc_fprintf(_tmp235,_tmp234,_tag_fat(_tmpF7,sizeof(void*),0U));});});
res=0;}
# 559
return({struct _tuple11 _tmp17F;_tmp17F.f1=new_scope,_tmp17F.f2=res;_tmp17F;});}}
# 38 "tcdecl.cyc"
struct Cyc_Absyn_Vardecl*Cyc_Tcdecl_merge_vardecl(struct Cyc_Absyn_Vardecl*d0,struct Cyc_Absyn_Vardecl*d1,unsigned loc,struct _fat_ptr*msg){
# 600 "tcdecl.cyc"
struct _fat_ptr v=Cyc_Absynpp_qvar2string(d0->name);
# 603
struct _tuple11 _stmttmp10=({
enum Cyc_Absyn_Scope _tmp240=d0->sc;void*_tmp23F=d0->type;struct Cyc_Absyn_Tqual _tmp23E=d0->tq;struct Cyc_List_List*_tmp23D=d0->attributes;enum Cyc_Absyn_Scope _tmp23C=d1->sc;void*_tmp23B=d1->type;struct Cyc_Absyn_Tqual _tmp23A=d1->tq;struct Cyc_List_List*_tmp239=d1->attributes;struct _fat_ptr _tmp238=({const char*_tmpFC="variable";_tag_fat(_tmpFC,sizeof(char),9U);});struct _fat_ptr _tmp237=v;unsigned _tmp236=loc;Cyc_Tcdecl_check_var_or_fn_decl(_tmp240,_tmp23F,_tmp23E,_tmp23D,_tmp23C,_tmp23B,_tmp23A,_tmp239,_tmp238,_tmp237,_tmp236,msg);});
# 603
int _tmpFA;enum Cyc_Absyn_Scope _tmpF9;_LL1: _tmpF9=_stmttmp10.f1;_tmpFA=_stmttmp10.f2;_LL2: {enum Cyc_Absyn_Scope new_scope=_tmpF9;int res=_tmpFA;
# 607
if(!res)return 0;if((int)d0->sc == (int)new_scope)
# 610
return d0;else{
# 612
struct Cyc_Absyn_Vardecl*d2=({struct Cyc_Absyn_Vardecl*_tmpFB=_cycalloc(sizeof(*_tmpFB));*_tmpFB=*d0;_tmpFB;});
d2->sc=new_scope;
return d2;}}}
# 38 "tcdecl.cyc"
struct Cyc_Absyn_Typedefdecl*Cyc_Tcdecl_merge_typedefdecl(struct Cyc_Absyn_Typedefdecl*d0,struct Cyc_Absyn_Typedefdecl*d1,unsigned loc,struct _fat_ptr*msg){
# 620 "tcdecl.cyc"
struct _fat_ptr v=Cyc_Absynpp_qvar2string(d0->name);
# 623
if(!({struct Cyc_List_List*_tmp245=d0->tvs;struct Cyc_List_List*_tmp244=d1->tvs;struct _fat_ptr _tmp243=({const char*_tmpFD="typedef";_tag_fat(_tmpFD,sizeof(char),8U);});struct _fat_ptr _tmp242=v;unsigned _tmp241=loc;Cyc_Tcdecl_check_tvs(_tmp245,_tmp244,_tmp243,_tmp242,_tmp241,msg);}))return 0;{struct Cyc_List_List*inst=
# 626
Cyc_Tcdecl_build_tvs_map(d0->tvs,d1->tvs);
# 629
if(d0->defn != 0 && d1->defn != 0){
void*subst_defn1=Cyc_Tcutil_substitute(inst,(void*)_check_null(d1->defn));
if(!Cyc_Tcdecl_check_type((void*)_check_null(d0->defn),subst_defn1)){
({struct Cyc_String_pa_PrintArg_struct _tmp100=({struct Cyc_String_pa_PrintArg_struct _tmp182;_tmp182.tag=0U,_tmp182.f1=(struct _fat_ptr)((struct _fat_ptr)v);_tmp182;});struct Cyc_String_pa_PrintArg_struct _tmp101=({struct Cyc_String_pa_PrintArg_struct _tmp181;_tmp181.tag=0U,({
struct _fat_ptr _tmp246=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(subst_defn1));_tmp181.f1=_tmp246;});_tmp181;});struct Cyc_String_pa_PrintArg_struct _tmp102=({struct Cyc_String_pa_PrintArg_struct _tmp180;_tmp180.tag=0U,({struct _fat_ptr _tmp247=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string((void*)_check_null(d0->defn)));_tmp180.f1=_tmp247;});_tmp180;});void*_tmpFE[3U];_tmpFE[0]=& _tmp100,_tmpFE[1]=& _tmp101,_tmpFE[2]=& _tmp102;({unsigned _tmp24A=loc;struct _fat_ptr*_tmp249=msg;struct _fat_ptr _tmp248=({const char*_tmpFF="typedef %s does not refer to the same type: %s != %s";_tag_fat(_tmpFF,sizeof(char),53U);});Cyc_Tcdecl_merr(_tmp24A,_tmp249,_tmp248,_tag_fat(_tmpFE,sizeof(void*),3U));});});
return 0;}}
# 629
return d0;}}
# 38 "tcdecl.cyc"
void*Cyc_Tcdecl_merge_binding(void*b0,void*b1,unsigned loc,struct _fat_ptr*msg){
# 642 "tcdecl.cyc"
struct _tuple15 _stmttmp11=({struct _tuple15 _tmp184;_tmp184.f1=b0,_tmp184.f2=b1;_tmp184;});struct _tuple15 _tmp103=_stmttmp11;struct Cyc_Absyn_Vardecl*_tmp105;struct Cyc_Absyn_Fndecl*_tmp104;struct Cyc_Absyn_Fndecl*_tmp106;struct Cyc_Absyn_Fndecl*_tmp108;struct Cyc_Absyn_Vardecl*_tmp107;struct Cyc_Absyn_Vardecl*_tmp10A;struct Cyc_Absyn_Vardecl*_tmp109;switch(*((int*)_tmp103.f1)){case 1U: switch(*((int*)_tmp103.f2)){case 1U: _LL1: _tmp109=((struct Cyc_Absyn_Global_b_Absyn_Binding_struct*)_tmp103.f1)->f1;_tmp10A=((struct Cyc_Absyn_Global_b_Absyn_Binding_struct*)_tmp103.f2)->f1;_LL2: {struct Cyc_Absyn_Vardecl*d0=_tmp109;struct Cyc_Absyn_Vardecl*d1=_tmp10A;
# 644
struct Cyc_Absyn_Vardecl*d2=Cyc_Tcdecl_merge_vardecl(d0,d1,loc,msg);
if(d2 == 0)return 0;if(d2 == d0)
return b0;
# 645
if(d2 == d1)
# 647
return b1;
# 645
return(void*)({struct Cyc_Absyn_Global_b_Absyn_Binding_struct*_tmp10B=_cycalloc(sizeof(*_tmp10B));
# 648
_tmp10B->tag=1U,_tmp10B->f1=d2;_tmp10B;});}case 2U: _LL3: _tmp107=((struct Cyc_Absyn_Global_b_Absyn_Binding_struct*)_tmp103.f1)->f1;_tmp108=((struct Cyc_Absyn_Funname_b_Absyn_Binding_struct*)_tmp103.f2)->f1;_LL4: {struct Cyc_Absyn_Vardecl*d0=_tmp107;struct Cyc_Absyn_Fndecl*d1=_tmp108;
# 650
struct _tuple11 _stmttmp12=({enum Cyc_Absyn_Scope _tmp255=d0->sc;void*_tmp254=d0->type;struct Cyc_Absyn_Tqual _tmp253=d0->tq;struct Cyc_List_List*_tmp252=d0->attributes;enum Cyc_Absyn_Scope _tmp251=d1->sc;void*_tmp250=(void*)_check_null(d1->cached_type);struct Cyc_Absyn_Tqual _tmp24F=
Cyc_Absyn_empty_tqual(0U);
# 650
struct Cyc_List_List*_tmp24E=(d1->i).attributes;struct _fat_ptr _tmp24D=({const char*_tmp10D="function";_tag_fat(_tmp10D,sizeof(char),9U);});struct _fat_ptr _tmp24C=
# 653
Cyc_Absynpp_qvar2string(d0->name);
# 650
unsigned _tmp24B=loc;Cyc_Tcdecl_check_var_or_fn_decl(_tmp255,_tmp254,_tmp253,_tmp252,_tmp251,_tmp250,_tmp24F,_tmp24E,_tmp24D,_tmp24C,_tmp24B,msg);});int _tmp10C;_LLC: _tmp10C=_stmttmp12.f2;_LLD: {int res=_tmp10C;
# 654
if(res)return b1;return 0;}}default: goto _LL9;}case 2U: switch(*((int*)_tmp103.f2)){case 2U: _LL5: _tmp106=((struct Cyc_Absyn_Funname_b_Absyn_Binding_struct*)_tmp103.f2)->f1;_LL6: {struct Cyc_Absyn_Fndecl*d1=_tmp106;
# 657
({struct Cyc_String_pa_PrintArg_struct _tmp110=({struct Cyc_String_pa_PrintArg_struct _tmp183;_tmp183.tag=0U,({struct _fat_ptr _tmp256=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(d1->name));_tmp183.f1=_tmp256;});_tmp183;});void*_tmp10E[1U];_tmp10E[0]=& _tmp110;({unsigned _tmp259=loc;struct _fat_ptr*_tmp258=msg;struct _fat_ptr _tmp257=({const char*_tmp10F="redefinition of function %s";_tag_fat(_tmp10F,sizeof(char),28U);});Cyc_Tcdecl_merr(_tmp259,_tmp258,_tmp257,_tag_fat(_tmp10E,sizeof(void*),1U));});});
return 0;}case 1U: _LL7: _tmp104=((struct Cyc_Absyn_Funname_b_Absyn_Binding_struct*)_tmp103.f1)->f1;_tmp105=((struct Cyc_Absyn_Global_b_Absyn_Binding_struct*)_tmp103.f2)->f1;_LL8: {struct Cyc_Absyn_Fndecl*d0=_tmp104;struct Cyc_Absyn_Vardecl*d1=_tmp105;
# 660
struct _tuple11 _stmttmp13=({enum Cyc_Absyn_Scope _tmp264=d0->sc;void*_tmp263=(void*)_check_null(d0->cached_type);struct Cyc_Absyn_Tqual _tmp262=Cyc_Absyn_empty_tqual(0U);struct Cyc_List_List*_tmp261=(d0->i).attributes;enum Cyc_Absyn_Scope _tmp260=d1->sc;void*_tmp25F=d1->type;struct Cyc_Absyn_Tqual _tmp25E=d1->tq;struct Cyc_List_List*_tmp25D=d1->attributes;struct _fat_ptr _tmp25C=({const char*_tmp112="variable";_tag_fat(_tmp112,sizeof(char),9U);});struct _fat_ptr _tmp25B=
# 663
Cyc_Absynpp_qvar2string(d0->name);
# 660
unsigned _tmp25A=loc;Cyc_Tcdecl_check_var_or_fn_decl(_tmp264,_tmp263,_tmp262,_tmp261,_tmp260,_tmp25F,_tmp25E,_tmp25D,_tmp25C,_tmp25B,_tmp25A,msg);});int _tmp111;_LLF: _tmp111=_stmttmp13.f2;_LL10: {int res=_tmp111;
# 664
if(res)return b0;return 0;}}default: goto _LL9;}default: _LL9: _LLA:
# 666
 return 0;}_LL0:;}
# 38 "tcdecl.cyc"
struct Cyc_Tcdecl_Xdatatypefielddecl*Cyc_Tcdecl_merge_xdatatypefielddecl(struct Cyc_Tcdecl_Xdatatypefielddecl*d0,struct Cyc_Tcdecl_Xdatatypefielddecl*d1,unsigned loc,struct _fat_ptr*msg){
# 673 "tcdecl.cyc"
struct Cyc_Absyn_Datatypefield*_tmp114;struct Cyc_Absyn_Datatypedecl*_tmp113;_LL1: _tmp113=d0->base;_tmp114=d0->field;_LL2: {struct Cyc_Absyn_Datatypedecl*b0=_tmp113;struct Cyc_Absyn_Datatypefield*f0=_tmp114;
struct Cyc_Absyn_Datatypefield*_tmp116;struct Cyc_Absyn_Datatypedecl*_tmp115;_LL4: _tmp115=d1->base;_tmp116=d1->field;_LL5: {struct Cyc_Absyn_Datatypedecl*b1=_tmp115;struct Cyc_Absyn_Datatypefield*f1=_tmp116;
struct _fat_ptr v=Cyc_Absynpp_qvar2string(f0->name);
# 678
if(Cyc_Absyn_qvar_cmp(b0->name,b1->name)!= 0){
({struct Cyc_String_pa_PrintArg_struct _tmp119=({struct Cyc_String_pa_PrintArg_struct _tmp187;_tmp187.tag=0U,({
struct _fat_ptr _tmp265=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(b0->name));_tmp187.f1=_tmp265;});_tmp187;});struct Cyc_String_pa_PrintArg_struct _tmp11A=({struct Cyc_String_pa_PrintArg_struct _tmp186;_tmp186.tag=0U,({struct _fat_ptr _tmp266=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(b1->name));_tmp186.f1=_tmp266;});_tmp186;});struct Cyc_String_pa_PrintArg_struct _tmp11B=({struct Cyc_String_pa_PrintArg_struct _tmp185;_tmp185.tag=0U,_tmp185.f1=(struct _fat_ptr)((struct _fat_ptr)v);_tmp185;});void*_tmp117[3U];_tmp117[0]=& _tmp119,_tmp117[1]=& _tmp11A,_tmp117[2]=& _tmp11B;({unsigned _tmp269=loc;struct _fat_ptr*_tmp268=msg;struct _fat_ptr _tmp267=({const char*_tmp118="@extensible datatype %s and %s have both a field named %s";_tag_fat(_tmp118,sizeof(char),58U);});Cyc_Tcdecl_merr(_tmp269,_tmp268,_tmp267,_tag_fat(_tmp117,sizeof(void*),3U));});});
return 0;}
# 678
if(!({
# 685
struct Cyc_List_List*_tmp26E=b0->tvs;struct Cyc_List_List*_tmp26D=b1->tvs;struct _fat_ptr _tmp26C=({const char*_tmp11C="@extensible datatype";_tag_fat(_tmp11C,sizeof(char),21U);});struct _fat_ptr _tmp26B=Cyc_Absynpp_qvar2string(b0->name);unsigned _tmp26A=loc;Cyc_Tcdecl_check_tvs(_tmp26E,_tmp26D,_tmp26C,_tmp26B,_tmp26A,msg);}))
return 0;{
# 678
struct Cyc_List_List*inst1=
# 688
Cyc_Tcdecl_build_tvs_map(b0->tvs,b1->tvs);
# 691
struct Cyc_Absyn_Datatypefield*f2=({struct Cyc_Absyn_Datatypefield*_tmp273=f0;struct Cyc_Absyn_Datatypefield*_tmp272=f1;struct Cyc_List_List*_tmp271=inst1;struct _fat_ptr _tmp270=({const char*_tmp11E="@extensible datatype field";_tag_fat(_tmp11E,sizeof(char),27U);});struct _fat_ptr _tmp26F=v;Cyc_Tcdecl_merge_datatypefield(_tmp273,_tmp272,_tmp271,_tmp270,_tmp26F,msg);});
if(f2 == 0)return 0;if(f2 == f0)
return d0;
# 692
return({struct Cyc_Tcdecl_Xdatatypefielddecl*_tmp11D=_cycalloc(sizeof(*_tmp11D));
# 694
_tmp11D->base=b0,_tmp11D->field=f2;_tmp11D;});}}}}
