#include <setjmp.h>
/* This is a C header file to be used by the output of the Cyclone to
   C translator.  The corresponding definitions are in file lib/runtime_*.c */
#ifndef _CYC_INCLUDE_H_
#define _CYC_INCLUDE_H_

/* Need one of these per thread (see runtime_stack.c). The runtime maintains 
   a stack that contains either _handler_cons structs or _RegionHandle structs.
   The tag is 0 for a handler_cons and 1 for a region handle.  */
struct _RuntimeStack {
  int tag; 
  struct _RuntimeStack *next;
  void (*cleanup)(struct _RuntimeStack *frame);
};

#ifndef offsetof
/* should be size_t, but int is fine. */
#define offsetof(t,n) ((int)(&(((t *)0)->n)))
#endif

/* Fat pointers */
struct _fat_ptr {
  unsigned char *curr; 
  unsigned char *base; 
  unsigned char *last_plus_one; 
};  

/* Discriminated Unions */
struct _xtunion_struct { char *tag; };

/* Regions */
struct _RegionPage
#ifdef CYC_REGION_PROFILE
{ unsigned total_bytes;
  unsigned free_bytes;
  /* MWH: wish we didn't have to include the stuff below ... */
  struct _RegionPage *next;
  char data[1];
}
#endif
; // abstract -- defined in runtime_memory.c
struct _RegionHandle {
  struct _RuntimeStack s;
  struct _RegionPage *curr;
  char               *offset;
  char               *last_plus_one;
  struct _DynRegionHandle *sub_regions;
#ifdef CYC_REGION_PROFILE
  const char         *name;
#else
  unsigned used_bytes;
  unsigned wasted_bytes;
#endif
};
struct _DynRegionFrame {
  struct _RuntimeStack s;
  struct _DynRegionHandle *x;
};
// A dynamic region is just a region handle.  The wrapper struct is for type
// abstraction.
struct Cyc_Core_DynamicRegion {
  struct _RegionHandle h;
};

struct _RegionHandle _new_region(const char*);
void* _region_malloc(struct _RegionHandle*, unsigned);
void* _region_calloc(struct _RegionHandle*, unsigned t, unsigned n);
void   _free_region(struct _RegionHandle*);
struct _RegionHandle*_open_dynregion(struct _DynRegionFrame*,struct _DynRegionHandle*);
void   _pop_dynregion();

/* Exceptions */
struct _handler_cons {
  struct _RuntimeStack s;
  jmp_buf handler;
};
void _push_handler(struct _handler_cons *);
void _push_region(struct _RegionHandle *);
void _npop_handler(int);
void _pop_handler();
void _pop_region();

#ifndef _throw
void* _throw_null_fn(const char*,unsigned);
void* _throw_arraybounds_fn(const char*,unsigned);
void* _throw_badalloc_fn(const char*,unsigned);
void* _throw_match_fn(const char*,unsigned);
void* _throw_fn(void*,const char*,unsigned);
void* _rethrow(void*);
#define _throw_null() (_throw_null_fn(__FILE__,__LINE__))
#define _throw_arraybounds() (_throw_arraybounds_fn(__FILE__,__LINE__))
#define _throw_badalloc() (_throw_badalloc_fn(__FILE__,__LINE__))
#define _throw_match() (_throw_match_fn(__FILE__,__LINE__))
#define _throw(e) (_throw_fn((e),__FILE__,__LINE__))
#endif

struct _xtunion_struct* Cyc_Core_get_exn_thrown();
/* Built-in Exceptions */
struct Cyc_Null_Exception_exn_struct { char *tag; };
struct Cyc_Array_bounds_exn_struct { char *tag; };
struct Cyc_Match_Exception_exn_struct { char *tag; };
struct Cyc_Bad_alloc_exn_struct { char *tag; };
extern char Cyc_Null_Exception[];
extern char Cyc_Array_bounds[];
extern char Cyc_Match_Exception[];
extern char Cyc_Bad_alloc[];

/* Built-in Run-time Checks and company */
#ifdef CYC_ANSI_OUTPUT
#define _INLINE  
#else
#define _INLINE inline
#endif

#ifdef NO_CYC_NULL_CHECKS
#define _check_null(ptr) (ptr)
#else
#define _check_null(ptr) \
  ({ void*_cks_null = (void*)(ptr); \
     if (!_cks_null) _throw_null(); \
     _cks_null; })
#endif

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index)\
   (((char*)ptr) + (elt_sz)*(index))
#ifdef NO_CYC_NULL_CHECKS
#define _check_known_subscript_null _check_known_subscript_notnull
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr);\
  int _index = (index);\
  if (!_cks_ptr) _throw_null(); \
  _cks_ptr + (elt_sz)*_index; })
#endif
#define _zero_arr_plus_fn(orig_x,orig_sz,orig_i,f,l) ((orig_x)+(orig_i))
#define _zero_arr_plus_char_fn _zero_arr_plus_fn
#define _zero_arr_plus_short_fn _zero_arr_plus_fn
#define _zero_arr_plus_int_fn _zero_arr_plus_fn
#define _zero_arr_plus_float_fn _zero_arr_plus_fn
#define _zero_arr_plus_double_fn _zero_arr_plus_fn
#define _zero_arr_plus_longdouble_fn _zero_arr_plus_fn
#define _zero_arr_plus_voidstar_fn _zero_arr_plus_fn
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (!_cks_ptr) _throw_null(); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })

/* _zero_arr_plus_*_fn(x,sz,i,filename,lineno) adds i to zero-terminated ptr
   x that has at least sz elements */
char* _zero_arr_plus_char_fn(char*,unsigned,int,const char*,unsigned);
short* _zero_arr_plus_short_fn(short*,unsigned,int,const char*,unsigned);
int* _zero_arr_plus_int_fn(int*,unsigned,int,const char*,unsigned);
float* _zero_arr_plus_float_fn(float*,unsigned,int,const char*,unsigned);
double* _zero_arr_plus_double_fn(double*,unsigned,int,const char*,unsigned);
long double* _zero_arr_plus_longdouble_fn(long double*,unsigned,int,const char*, unsigned);
void** _zero_arr_plus_voidstar_fn(void**,unsigned,int,const char*,unsigned);
#endif

/* _get_zero_arr_size_*(x,sz) returns the number of elements in a
   zero-terminated array that is NULL or has at least sz elements */
int _get_zero_arr_size_char(const char*,unsigned);
int _get_zero_arr_size_short(const short*,unsigned);
int _get_zero_arr_size_int(const int*,unsigned);
int _get_zero_arr_size_float(const float*,unsigned);
int _get_zero_arr_size_double(const double*,unsigned);
int _get_zero_arr_size_longdouble(const long double*,unsigned);
int _get_zero_arr_size_voidstar(const void**,unsigned);

/* _zero_arr_inplace_plus_*_fn(x,i,filename,lineno) sets
   zero-terminated pointer *x to *x + i */
char* _zero_arr_inplace_plus_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_short_fn(short**,int,const char*,unsigned);
int* _zero_arr_inplace_plus_int(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_longdouble_fn(long double**,int,const char*,unsigned);
void** _zero_arr_inplace_plus_voidstar_fn(void***,int,const char*,unsigned);
/* like the previous functions, but does post-addition (as in e++) */
char* _zero_arr_inplace_plus_post_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_post_short_fn(short**x,int,const char*,unsigned);
int* _zero_arr_inplace_plus_post_int_fn(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_post_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_post_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_post_longdouble_fn(long double**,int,const char *,unsigned);
void** _zero_arr_inplace_plus_post_voidstar_fn(void***,int,const char*,unsigned);
#define _zero_arr_plus_char(x,s,i) \
  (_zero_arr_plus_char_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_short(x,s,i) \
  (_zero_arr_plus_short_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_int(x,s,i) \
  (_zero_arr_plus_int_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_float(x,s,i) \
  (_zero_arr_plus_float_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_double(x,s,i) \
  (_zero_arr_plus_double_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_longdouble(x,s,i) \
  (_zero_arr_plus_longdouble_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_voidstar(x,s,i) \
  (_zero_arr_plus_voidstar_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_inplace_plus_char(x,i) \
  _zero_arr_inplace_plus_char_fn((char **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_short(x,i) \
  _zero_arr_inplace_plus_short_fn((short **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_int(x,i) \
  _zero_arr_inplace_plus_int_fn((int **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_float(x,i) \
  _zero_arr_inplace_plus_float_fn((float **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_double(x,i) \
  _zero_arr_inplace_plus_double_fn((double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_longdouble(x,i) \
  _zero_arr_inplace_plus_longdouble_fn((long double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_voidstar(x,i) \
  _zero_arr_inplace_plus_voidstar_fn((void ***)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_char(x,i) \
  _zero_arr_inplace_plus_post_char_fn((char **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_short(x,i) \
  _zero_arr_inplace_plus_post_short_fn((short **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_int(x,i) \
  _zero_arr_inplace_plus_post_int_fn((int **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_float(x,i) \
  _zero_arr_inplace_plus_post_float_fn((float **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_double(x,i) \
  _zero_arr_inplace_plus_post_double_fn((double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_longdouble(x,i) \
  _zero_arr_inplace_plus_post_longdouble_fn((long double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_voidstar(x,i) \
  _zero_arr_inplace_plus_post_voidstar_fn((void***)(x),(i),__FILE__,__LINE__)

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_fat_subscript(arr,elt_sz,index) ((arr).curr + (elt_sz) * (index))
#define _untag_fat_ptr(arr,elt_sz,num_elts) ((arr).curr)
#else
#define _check_fat_subscript(arr,elt_sz,index) ({ \
  struct _fat_ptr _cus_arr = (arr); \
  unsigned char *_cus_ans = _cus_arr.curr + (elt_sz) * (index); \
  /* JGM: not needed! if (!_cus_arr.base) _throw_null();*/ \
  if (_cus_ans < _cus_arr.base || _cus_ans >= _cus_arr.last_plus_one) \
    _throw_arraybounds(); \
  _cus_ans; })
#define _untag_fat_ptr(arr,elt_sz,num_elts) ({ \
  struct _fat_ptr _arr = (arr); \
  unsigned char *_curr = _arr.curr; \
  if ((_curr < _arr.base || _curr + (elt_sz) * (num_elts) > _arr.last_plus_one) &&\
      _curr != (unsigned char *)0) \
    _throw_arraybounds(); \
  _curr; })
#endif

#define _tag_fat(tcurr,elt_sz,num_elts) ({ \
  struct _fat_ptr _ans; \
  unsigned _num_elts = (num_elts);\
  _ans.base = _ans.curr = (void*)(tcurr); \
  /* JGM: if we're tagging NULL, ignore num_elts */ \
  _ans.last_plus_one = _ans.base ? (_ans.base + (elt_sz) * _num_elts) : 0; \
  _ans; })

#define _get_fat_size(arr,elt_sz) \
  ({struct _fat_ptr _arr = (arr); \
    unsigned char *_arr_curr=_arr.curr; \
    unsigned char *_arr_last=_arr.last_plus_one; \
    (_arr_curr < _arr.base || _arr_curr >= _arr_last) ? 0 : \
    ((_arr_last - _arr_curr) / (elt_sz));})

#define _fat_ptr_plus(arr,elt_sz,change) ({ \
  struct _fat_ptr _ans = (arr); \
  int _change = (change);\
  _ans.curr += (elt_sz) * _change;\
  _ans; })
#define _fat_ptr_inplace_plus(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  _arr_ptr->curr += (elt_sz) * (change);\
  *_arr_ptr; })
#define _fat_ptr_inplace_plus_post(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  struct _fat_ptr _ans = *_arr_ptr; \
  _arr_ptr->curr += (elt_sz) * (change);\
  _ans; })

//Not a macro since initialization order matters. Defined in runtime_zeroterm.c.
struct _fat_ptr _fat_ptr_decrease_size(struct _fat_ptr,unsigned sz,unsigned numelts);

/* Allocation */
void* GC_malloc(int);
void* GC_malloc_atomic(int);
void* GC_calloc(unsigned,unsigned);
void* GC_calloc_atomic(unsigned,unsigned);
// bound the allocation size to be < MAX_ALLOC_SIZE. See macros below for usage.
#define MAX_MALLOC_SIZE (1 << 28)
void* _bounded_GC_malloc(int,const char*,int);
void* _bounded_GC_malloc_atomic(int,const char*,int);
void* _bounded_GC_calloc(unsigned,unsigned,const char*,int);
void* _bounded_GC_calloc_atomic(unsigned,unsigned,const char*,int);
/* these macros are overridden below ifdef CYC_REGION_PROFILE */
#ifndef CYC_REGION_PROFILE
#define _cycalloc(n) _bounded_GC_malloc(n,__FILE__,__LINE__)
#define _cycalloc_atomic(n) _bounded_GC_malloc_atomic(n,__FILE__,__LINE__)
#define _cyccalloc(n,s) _bounded_GC_calloc(n,s,__FILE__,__LINE__)
#define _cyccalloc_atomic(n,s) _bounded_GC_calloc_atomic(n,s,__FILE__,__LINE__)
#endif

static _INLINE unsigned int _check_times(unsigned x, unsigned y) {
  unsigned long long whole_ans = 
    ((unsigned long long) x)*((unsigned long long)y);
  unsigned word_ans = (unsigned)whole_ans;
  if(word_ans < whole_ans || word_ans > MAX_MALLOC_SIZE)
    _throw_badalloc();
  return word_ans;
}

#define _CYC_MAX_REGION_CONST 2
#define _CYC_MIN_ALIGNMENT (sizeof(double))

#ifdef CYC_REGION_PROFILE
extern int rgn_total_bytes;
#endif

static _INLINE void *_fast_region_malloc(struct _RegionHandle *r, unsigned orig_s) {  
  if (r > (struct _RegionHandle *)_CYC_MAX_REGION_CONST && r->curr != 0) { 
#ifdef CYC_NOALIGN
    unsigned s =  orig_s;
#else
    unsigned s =  (orig_s + _CYC_MIN_ALIGNMENT - 1) & (~(_CYC_MIN_ALIGNMENT -1)); 
#endif
    char *result; 
    result = r->offset; 
    if (s <= (r->last_plus_one - result)) {
      r->offset = result + s; 
#ifdef CYC_REGION_PROFILE
    r->curr->free_bytes = r->curr->free_bytes - s;
    rgn_total_bytes += s;
#endif
      return result;
    }
  } 
  return _region_malloc(r,orig_s); 
}

#ifdef CYC_REGION_PROFILE
/* see macros below for usage. defined in runtime_memory.c */
void* _profile_GC_malloc(int,const char*,const char*,int);
void* _profile_GC_malloc_atomic(int,const char*,const char*,int);
void* _profile_GC_calloc(unsigned,unsigned,const char*,const char*,int);
void* _profile_GC_calloc_atomic(unsigned,unsigned,const char*,const char*,int);
void* _profile_region_malloc(struct _RegionHandle*,unsigned,const char*,const char*,int);
void* _profile_region_calloc(struct _RegionHandle*,unsigned,unsigned,const char *,const char*,int);
struct _RegionHandle _profile_new_region(const char*,const char*,const char*,int);
void _profile_free_region(struct _RegionHandle*,const char*,const char*,int);
#ifndef RUNTIME_CYC
#define _new_region(n) _profile_new_region(n,__FILE__,__FUNCTION__,__LINE__)
#define _free_region(r) _profile_free_region(r,__FILE__,__FUNCTION__,__LINE__)
#define _region_malloc(rh,n) _profile_region_malloc(rh,n,__FILE__,__FUNCTION__,__LINE__)
#define _region_calloc(rh,n,t) _profile_region_calloc(rh,n,t,__FILE__,__FUNCTION__,__LINE__)
#  endif
#define _cycalloc(n) _profile_GC_malloc(n,__FILE__,__FUNCTION__,__LINE__)
#define _cycalloc_atomic(n) _profile_GC_malloc_atomic(n,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc(n,s) _profile_GC_calloc(n,s,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc_atomic(n,s) _profile_GC_calloc_atomic(n,s,__FILE__,__FUNCTION__,__LINE__)
#endif
#endif
 struct Cyc_Core_Opt{void*v;};extern char Cyc_Core_Invalid_argument[17U];struct Cyc_Core_Invalid_argument_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Failure[8U];struct Cyc_Core_Failure_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Impossible[11U];struct Cyc_Core_Impossible_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Not_found[10U];struct Cyc_Core_Not_found_exn_struct{char*tag;};extern char Cyc_Core_Unreachable[12U];struct Cyc_Core_Unreachable_exn_struct{char*tag;struct _fat_ptr f1;};
# 168 "core.h"
extern struct _RegionHandle*Cyc_Core_unique_region;struct Cyc_Core_DynamicRegion;struct Cyc_Core_NewDynamicRegion{struct Cyc_Core_DynamicRegion*key;};struct Cyc_Core_ThinRes{void*arr;unsigned nelts;};struct Cyc_List_List{void*hd;struct Cyc_List_List*tl;};extern char Cyc_List_List_mismatch[14U];struct Cyc_List_List_mismatch_exn_struct{char*tag;};
# 178 "list.h"
struct Cyc_List_List*Cyc_List_imp_rev(struct Cyc_List_List*x);
# 184
struct Cyc_List_List*Cyc_List_append(struct Cyc_List_List*x,struct Cyc_List_List*y);extern char Cyc_List_Nth[4U];struct Cyc_List_Nth_exn_struct{char*tag;};struct Cyc___cycFILE;struct Cyc_String_pa_PrintArg_struct{int tag;struct _fat_ptr f1;};struct Cyc_Int_pa_PrintArg_struct{int tag;unsigned long f1;};struct Cyc_Double_pa_PrintArg_struct{int tag;double f1;};struct Cyc_LongDouble_pa_PrintArg_struct{int tag;long double f1;};struct Cyc_ShortPtr_pa_PrintArg_struct{int tag;short*f1;};struct Cyc_IntPtr_pa_PrintArg_struct{int tag;unsigned long*f1;};
# 73 "cycboot.h"
struct _fat_ptr Cyc_aprintf(struct _fat_ptr,struct _fat_ptr);struct Cyc_ShortPtr_sa_ScanfArg_struct{int tag;short*f1;};struct Cyc_UShortPtr_sa_ScanfArg_struct{int tag;unsigned short*f1;};struct Cyc_IntPtr_sa_ScanfArg_struct{int tag;int*f1;};struct Cyc_UIntPtr_sa_ScanfArg_struct{int tag;unsigned*f1;};struct Cyc_StringPtr_sa_ScanfArg_struct{int tag;struct _fat_ptr f1;};struct Cyc_DoublePtr_sa_ScanfArg_struct{int tag;double*f1;};struct Cyc_FloatPtr_sa_ScanfArg_struct{int tag;float*f1;};struct Cyc_CharPtr_sa_ScanfArg_struct{int tag;struct _fat_ptr f1;};extern char Cyc_FileCloseError[15U];struct Cyc_FileCloseError_exn_struct{char*tag;};extern char Cyc_FileOpenError[14U];struct Cyc_FileOpenError_exn_struct{char*tag;struct _fat_ptr f1;};
# 50 "string.h"
int Cyc_strptrcmp(struct _fat_ptr*s1,struct _fat_ptr*s2);struct Cyc_PP_Ppstate;struct Cyc_PP_Out;struct Cyc_PP_Doc;struct Cyc_Position_Error;struct Cyc_Relations_Reln;struct _union_Nmspace_Rel_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_Abs_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_C_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_Loc_n{int tag;int val;};union Cyc_Absyn_Nmspace{struct _union_Nmspace_Rel_n Rel_n;struct _union_Nmspace_Abs_n Abs_n;struct _union_Nmspace_C_n C_n;struct _union_Nmspace_Loc_n Loc_n;};struct _tuple0{union Cyc_Absyn_Nmspace f1;struct _fat_ptr*f2;};
# 161 "absyn.h"
enum Cyc_Absyn_Scope{Cyc_Absyn_Static =0U,Cyc_Absyn_Abstract =1U,Cyc_Absyn_Public =2U,Cyc_Absyn_Extern =3U,Cyc_Absyn_ExternC =4U,Cyc_Absyn_Register =5U};struct Cyc_Absyn_Tqual{int print_const: 1;int q_volatile: 1;int q_restrict: 1;int real_const: 1;unsigned loc;};
# 182
enum Cyc_Absyn_Size_of{Cyc_Absyn_Char_sz =0U,Cyc_Absyn_Short_sz =1U,Cyc_Absyn_Int_sz =2U,Cyc_Absyn_Long_sz =3U,Cyc_Absyn_LongLong_sz =4U};
enum Cyc_Absyn_Sign{Cyc_Absyn_Signed =0U,Cyc_Absyn_Unsigned =1U,Cyc_Absyn_None =2U};
enum Cyc_Absyn_AggrKind{Cyc_Absyn_StructA =0U,Cyc_Absyn_UnionA =1U};
# 187
enum Cyc_Absyn_AliasQual{Cyc_Absyn_Aliasable =0U,Cyc_Absyn_Unique =1U,Cyc_Absyn_Top =2U};
# 192
enum Cyc_Absyn_KindQual{Cyc_Absyn_AnyKind =0U,Cyc_Absyn_MemKind =1U,Cyc_Absyn_BoxKind =2U,Cyc_Absyn_RgnKind =3U,Cyc_Absyn_EffKind =4U,Cyc_Absyn_IntKind =5U,Cyc_Absyn_BoolKind =6U,Cyc_Absyn_PtrBndKind =7U};struct Cyc_Absyn_Kind{enum Cyc_Absyn_KindQual kind;enum Cyc_Absyn_AliasQual aliasqual;};struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct{int tag;struct Cyc_Absyn_Kind*f1;};struct Cyc_Absyn_Unknown_kb_Absyn_KindBound_struct{int tag;struct Cyc_Core_Opt*f1;};struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct{int tag;struct Cyc_Core_Opt*f1;struct Cyc_Absyn_Kind*f2;};struct Cyc_Absyn_Tvar{struct _fat_ptr*name;int identity;void*kind;};struct Cyc_Absyn_PtrLoc{unsigned ptr_loc;unsigned rgn_loc;unsigned zt_loc;};struct Cyc_Absyn_PtrAtts{void*rgn;void*nullable;void*bounds;void*zero_term;struct Cyc_Absyn_PtrLoc*ptrloc;};struct Cyc_Absyn_PtrInfo{void*elt_type;struct Cyc_Absyn_Tqual elt_tq;struct Cyc_Absyn_PtrAtts ptr_atts;};struct Cyc_Absyn_VarargInfo{struct _fat_ptr*name;struct Cyc_Absyn_Tqual tq;void*type;int inject;};struct Cyc_Absyn_Star_Absyn_Cap_struct{int tag;};struct Cyc_Absyn_Bot_Absyn_Cap_struct{int tag;};struct Cyc_Absyn_Nat_Absyn_Cap_struct{int tag;struct Cyc_Absyn_Exp*f1;int f2;};struct Cyc_Absyn_RgnEffect{void*name;void*rgn;void*lock;void*unadopt;void*parent;};struct Cyc_Absyn_FnInfo{struct Cyc_List_List*tvars;void*effect;struct Cyc_Absyn_Tqual ret_tqual;void*ret_type;struct Cyc_List_List*args;int c_varargs;struct Cyc_Absyn_VarargInfo*cyc_varargs;struct Cyc_List_List*rgn_po;struct Cyc_List_List*attributes;struct Cyc_Absyn_Exp*requires_clause;struct Cyc_List_List*requires_relns;struct Cyc_Absyn_Exp*ensures_clause;struct Cyc_List_List*ensures_relns;struct Cyc_List_List*ieffect;struct Cyc_List_List*oeffect;};struct Cyc_Absyn_UnknownDatatypeInfo{struct _tuple0*name;int is_extensible;};struct _union_DatatypeInfo_UnknownDatatype{int tag;struct Cyc_Absyn_UnknownDatatypeInfo val;};struct _union_DatatypeInfo_KnownDatatype{int tag;struct Cyc_Absyn_Datatypedecl**val;};union Cyc_Absyn_DatatypeInfo{struct _union_DatatypeInfo_UnknownDatatype UnknownDatatype;struct _union_DatatypeInfo_KnownDatatype KnownDatatype;};struct Cyc_Absyn_UnknownDatatypeFieldInfo{struct _tuple0*datatype_name;struct _tuple0*field_name;int is_extensible;};struct _union_DatatypeFieldInfo_UnknownDatatypefield{int tag;struct Cyc_Absyn_UnknownDatatypeFieldInfo val;};struct _tuple1{struct Cyc_Absyn_Datatypedecl*f1;struct Cyc_Absyn_Datatypefield*f2;};struct _union_DatatypeFieldInfo_KnownDatatypefield{int tag;struct _tuple1 val;};union Cyc_Absyn_DatatypeFieldInfo{struct _union_DatatypeFieldInfo_UnknownDatatypefield UnknownDatatypefield;struct _union_DatatypeFieldInfo_KnownDatatypefield KnownDatatypefield;};struct _tuple2{enum Cyc_Absyn_AggrKind f1;struct _tuple0*f2;struct Cyc_Core_Opt*f3;};struct _union_AggrInfo_UnknownAggr{int tag;struct _tuple2 val;};struct _union_AggrInfo_KnownAggr{int tag;struct Cyc_Absyn_Aggrdecl**val;};union Cyc_Absyn_AggrInfo{struct _union_AggrInfo_UnknownAggr UnknownAggr;struct _union_AggrInfo_KnownAggr KnownAggr;};struct Cyc_Absyn_ArrayInfo{void*elt_type;struct Cyc_Absyn_Tqual tq;struct Cyc_Absyn_Exp*num_elts;void*zero_term;unsigned zt_loc;};struct Cyc_Absyn_Aggr_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Aggrdecl*f1;};struct Cyc_Absyn_Enum_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;};struct Cyc_Absyn_Datatype_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;};struct Cyc_Absyn_TypeDecl{void*r;unsigned loc;};struct Cyc_Absyn_VoidCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_IntCon_Absyn_TyCon_struct{int tag;enum Cyc_Absyn_Sign f1;enum Cyc_Absyn_Size_of f2;};struct Cyc_Absyn_FloatCon_Absyn_TyCon_struct{int tag;int f1;};struct Cyc_Absyn_RgnHandleCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_TagCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_HeapCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_UniqueCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_RefCntCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_AccessCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_JoinCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_RgnsCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_TrueCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_FalseCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_ThinCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_FatCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_EnumCon_Absyn_TyCon_struct{int tag;struct _tuple0*f1;struct Cyc_Absyn_Enumdecl*f2;};struct Cyc_Absyn_AnonEnumCon_Absyn_TyCon_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_BuiltinCon_Absyn_TyCon_struct{int tag;struct _fat_ptr f1;struct Cyc_Absyn_Kind*f2;};struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_DatatypeInfo f1;};struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_DatatypeFieldInfo f1;};struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_AggrInfo f1;};struct Cyc_Absyn_AppType_Absyn_Type_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Evar_Absyn_Type_struct{int tag;struct Cyc_Core_Opt*f1;void*f2;int f3;struct Cyc_Core_Opt*f4;};struct Cyc_Absyn_VarType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Tvar*f1;};struct Cyc_Absyn_PointerType_Absyn_Type_struct{int tag;struct Cyc_Absyn_PtrInfo f1;};struct Cyc_Absyn_ArrayType_Absyn_Type_struct{int tag;struct Cyc_Absyn_ArrayInfo f1;};struct Cyc_Absyn_FnType_Absyn_Type_struct{int tag;struct Cyc_Absyn_FnInfo f1;};struct Cyc_Absyn_TupleType_Absyn_Type_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct{int tag;enum Cyc_Absyn_AggrKind f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_TypedefType_Absyn_Type_struct{int tag;struct _tuple0*f1;struct Cyc_List_List*f2;struct Cyc_Absyn_Typedefdecl*f3;void*f4;};struct Cyc_Absyn_ValueofType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct{int tag;struct Cyc_Absyn_TypeDecl*f1;void**f2;};struct Cyc_Absyn_TypeofType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_NoTypes_Absyn_Funcparams_struct{int tag;struct Cyc_List_List*f1;unsigned f2;};struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct{int tag;struct Cyc_List_List*f1;int f2;struct Cyc_Absyn_VarargInfo*f3;void*f4;struct Cyc_List_List*f5;struct Cyc_Absyn_Exp*f6;struct Cyc_Absyn_Exp*f7;struct Cyc_List_List*f8;struct Cyc_List_List*f9;};
# 437 "absyn.h"
enum Cyc_Absyn_Format_Type{Cyc_Absyn_Printf_ft =0U,Cyc_Absyn_Scanf_ft =1U};struct Cyc_Absyn_Regparm_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Stdcall_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Cdecl_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Fastcall_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Noreturn_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Const_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Aligned_att_Absyn_Attribute_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Packed_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Section_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Nocommon_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Shared_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Unused_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Weak_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Dllimport_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Dllexport_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_instrument_function_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Constructor_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Destructor_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_check_memory_usage_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Format_att_Absyn_Attribute_struct{int tag;enum Cyc_Absyn_Format_Type f1;int f2;int f3;};struct Cyc_Absyn_Initializes_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Noliveunique_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Consume_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Pure_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Mode_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Alias_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Always_inline_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_throw_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Non_null_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Deprecated_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Carray_mod_Absyn_Type_modifier_struct{int tag;void*f1;unsigned f2;};struct Cyc_Absyn_ConstArray_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_Absyn_Exp*f1;void*f2;unsigned f3;};struct Cyc_Absyn_Pointer_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_Absyn_PtrAtts f1;struct Cyc_Absyn_Tqual f2;};struct Cyc_Absyn_Function_mod_Absyn_Type_modifier_struct{int tag;void*f1;};struct Cyc_Absyn_TypeParams_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_List_List*f1;unsigned f2;int f3;};struct Cyc_Absyn_Attributes_mod_Absyn_Type_modifier_struct{int tag;unsigned f1;struct Cyc_List_List*f2;};struct _union_Cnst_Null_c{int tag;int val;};struct _tuple3{enum Cyc_Absyn_Sign f1;char f2;};struct _union_Cnst_Char_c{int tag;struct _tuple3 val;};struct _union_Cnst_Wchar_c{int tag;struct _fat_ptr val;};struct _tuple4{enum Cyc_Absyn_Sign f1;short f2;};struct _union_Cnst_Short_c{int tag;struct _tuple4 val;};struct _tuple5{enum Cyc_Absyn_Sign f1;int f2;};struct _union_Cnst_Int_c{int tag;struct _tuple5 val;};struct _tuple6{enum Cyc_Absyn_Sign f1;long long f2;};struct _union_Cnst_LongLong_c{int tag;struct _tuple6 val;};struct _tuple7{struct _fat_ptr f1;int f2;};struct _union_Cnst_Float_c{int tag;struct _tuple7 val;};struct _union_Cnst_String_c{int tag;struct _fat_ptr val;};struct _union_Cnst_Wstring_c{int tag;struct _fat_ptr val;};union Cyc_Absyn_Cnst{struct _union_Cnst_Null_c Null_c;struct _union_Cnst_Char_c Char_c;struct _union_Cnst_Wchar_c Wchar_c;struct _union_Cnst_Short_c Short_c;struct _union_Cnst_Int_c Int_c;struct _union_Cnst_LongLong_c LongLong_c;struct _union_Cnst_Float_c Float_c;struct _union_Cnst_String_c String_c;struct _union_Cnst_Wstring_c Wstring_c;};
# 532
enum Cyc_Absyn_Primop{Cyc_Absyn_Plus =0U,Cyc_Absyn_Times =1U,Cyc_Absyn_Minus =2U,Cyc_Absyn_Div =3U,Cyc_Absyn_Mod =4U,Cyc_Absyn_Eq =5U,Cyc_Absyn_Neq =6U,Cyc_Absyn_Gt =7U,Cyc_Absyn_Lt =8U,Cyc_Absyn_Gte =9U,Cyc_Absyn_Lte =10U,Cyc_Absyn_Not =11U,Cyc_Absyn_Bitnot =12U,Cyc_Absyn_Bitand =13U,Cyc_Absyn_Bitor =14U,Cyc_Absyn_Bitxor =15U,Cyc_Absyn_Bitlshift =16U,Cyc_Absyn_Bitlrshift =17U,Cyc_Absyn_Numelts =18U};
# 539
enum Cyc_Absyn_Incrementor{Cyc_Absyn_PreInc =0U,Cyc_Absyn_PostInc =1U,Cyc_Absyn_PreDec =2U,Cyc_Absyn_PostDec =3U};struct Cyc_Absyn_VarargCallInfo{int num_varargs;struct Cyc_List_List*injectors;struct Cyc_Absyn_VarargInfo*vai;};struct Cyc_Absyn_StructField_Absyn_OffsetofField_struct{int tag;struct _fat_ptr*f1;};struct Cyc_Absyn_TupleIndex_Absyn_OffsetofField_struct{int tag;unsigned f1;};
# 557
enum Cyc_Absyn_Coercion{Cyc_Absyn_Unknown_coercion =0U,Cyc_Absyn_No_coercion =1U,Cyc_Absyn_Null_to_NonNull =2U,Cyc_Absyn_Other_coercion =3U};struct Cyc_Absyn_MallocInfo{int is_calloc;struct Cyc_Absyn_Exp*rgn;void**elt_type;struct Cyc_Absyn_Exp*num_elts;int fat_result;int inline_call;};struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct{int tag;union Cyc_Absyn_Cnst f1;};struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Pragma_e_Absyn_Raw_exp_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct{int tag;enum Cyc_Absyn_Primop f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_AssignOp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Core_Opt*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_Increment_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;enum Cyc_Absyn_Incrementor f2;};struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;struct Cyc_Absyn_VarargCallInfo*f3;int f4;};struct Cyc_Absyn_Throw_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;int f2;};struct Cyc_Absyn_NoInstantiate_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Instantiate_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_Absyn_Exp*f2;int f3;enum Cyc_Absyn_Coercion f4;};struct Cyc_Absyn_Address_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_New_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Sizeoftype_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Sizeofexp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Offsetof_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Deref_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;int f3;int f4;};struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;int f3;int f4;};struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Tuple_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;};struct _tuple8{struct _fat_ptr*f1;struct Cyc_Absyn_Tqual f2;void*f3;};struct Cyc_Absyn_CompoundLit_e_Absyn_Raw_exp_struct{int tag;struct _tuple8*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Array_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Exp*f2;struct Cyc_Absyn_Exp*f3;int f4;};struct Cyc_Absyn_ComprehensionNoinit_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;void*f2;int f3;};struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct{int tag;struct _tuple0*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;struct Cyc_Absyn_Aggrdecl*f4;};struct Cyc_Absyn_AnonStruct_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Datatype_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;struct Cyc_Absyn_Datatypedecl*f2;struct Cyc_Absyn_Datatypefield*f3;};struct Cyc_Absyn_Enum_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_AnonEnum_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_Spawn_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_MallocInfo f1;};struct Cyc_Absyn_Swap_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_UnresolvedMem_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Core_Opt*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_StmtExp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Stmt*f1;};struct Cyc_Absyn_Tagcheck_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;};struct Cyc_Absyn_Valueof_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Asm_e_Absyn_Raw_exp_struct{int tag;int f1;struct _fat_ptr f2;struct Cyc_List_List*f3;struct Cyc_List_List*f4;struct Cyc_List_List*f5;};struct Cyc_Absyn_Extension_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Exp{void*topt;void*r;unsigned loc;void*annot;};struct Cyc_Absyn_Skip_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Exp_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Seq_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Return_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_IfThenElse_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Stmt*f2;struct Cyc_Absyn_Stmt*f3;};struct _tuple9{struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_While_s_Absyn_Raw_stmt_struct{int tag;struct _tuple9 f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Break_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Continue_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Goto_s_Absyn_Raw_stmt_struct{int tag;struct _fat_ptr*f1;};struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _tuple9 f2;struct _tuple9 f3;struct Cyc_Absyn_Stmt*f4;};struct Cyc_Absyn_Switch_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;void*f3;};struct Cyc_Absyn_Fallthru_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_List_List*f1;struct Cyc_Absyn_Switch_clause**f2;};struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Decl*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Label_s_Absyn_Raw_stmt_struct{int tag;struct _fat_ptr*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Do_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct _tuple9 f2;};struct Cyc_Absyn_TryCatch_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct Cyc_List_List*f2;void*f3;};struct Cyc_Absyn_Stmt{void*r;unsigned loc;void*annot;};struct Cyc_Absyn_Wild_p_Absyn_Raw_pat_struct{int tag;};struct Cyc_Absyn_Var_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Pat*f2;};struct Cyc_Absyn_AliasVar_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;};struct Cyc_Absyn_Reference_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Pat*f2;};struct Cyc_Absyn_TagInt_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;};struct Cyc_Absyn_Tuple_p_Absyn_Raw_pat_struct{int tag;struct Cyc_List_List*f1;int f2;};struct Cyc_Absyn_Pointer_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Pat*f1;};struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct{int tag;union Cyc_Absyn_AggrInfo*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;int f4;};struct Cyc_Absyn_Datatype_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;struct Cyc_Absyn_Datatypefield*f2;struct Cyc_List_List*f3;int f4;};struct Cyc_Absyn_Null_p_Absyn_Raw_pat_struct{int tag;};struct Cyc_Absyn_Int_p_Absyn_Raw_pat_struct{int tag;enum Cyc_Absyn_Sign f1;int f2;};struct Cyc_Absyn_Char_p_Absyn_Raw_pat_struct{int tag;char f1;};struct Cyc_Absyn_Float_p_Absyn_Raw_pat_struct{int tag;struct _fat_ptr f1;int f2;};struct Cyc_Absyn_Enum_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_AnonEnum_p_Absyn_Raw_pat_struct{int tag;void*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_UnknownId_p_Absyn_Raw_pat_struct{int tag;struct _tuple0*f1;};struct Cyc_Absyn_UnknownCall_p_Absyn_Raw_pat_struct{int tag;struct _tuple0*f1;struct Cyc_List_List*f2;int f3;};struct Cyc_Absyn_Exp_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Pat{void*r;void*topt;unsigned loc;};struct Cyc_Absyn_Switch_clause{struct Cyc_Absyn_Pat*pattern;struct Cyc_Core_Opt*pat_vars;struct Cyc_Absyn_Exp*where_clause;struct Cyc_Absyn_Stmt*body;unsigned loc;};struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct{int tag;struct _tuple0*f1;};struct Cyc_Absyn_Global_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Funname_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Fndecl*f1;};struct Cyc_Absyn_Param_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Local_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Pat_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Vardecl{enum Cyc_Absyn_Scope sc;struct _tuple0*name;unsigned varloc;struct Cyc_Absyn_Tqual tq;void*type;struct Cyc_Absyn_Exp*initializer;void*rgn;struct Cyc_List_List*attributes;int escapes;};struct Cyc_Absyn_Fndecl{enum Cyc_Absyn_Scope sc;int is_inline;struct _tuple0*name;struct Cyc_Absyn_Stmt*body;struct Cyc_Absyn_FnInfo i;void*cached_type;struct Cyc_Core_Opt*param_vardecls;struct Cyc_Absyn_Vardecl*fn_vardecl;};struct Cyc_Absyn_Aggrfield{struct _fat_ptr*name;struct Cyc_Absyn_Tqual tq;void*type;struct Cyc_Absyn_Exp*width;struct Cyc_List_List*attributes;struct Cyc_Absyn_Exp*requires_clause;};struct Cyc_Absyn_AggrdeclImpl{struct Cyc_List_List*exist_vars;struct Cyc_List_List*rgn_po;struct Cyc_List_List*fields;int tagged;};struct Cyc_Absyn_Aggrdecl{enum Cyc_Absyn_AggrKind kind;enum Cyc_Absyn_Scope sc;struct _tuple0*name;struct Cyc_List_List*tvs;struct Cyc_Absyn_AggrdeclImpl*impl;struct Cyc_List_List*attributes;int expected_mem_kind;};struct Cyc_Absyn_Datatypefield{struct _tuple0*name;struct Cyc_List_List*typs;unsigned loc;enum Cyc_Absyn_Scope sc;};struct Cyc_Absyn_Datatypedecl{enum Cyc_Absyn_Scope sc;struct _tuple0*name;struct Cyc_List_List*tvs;struct Cyc_Core_Opt*fields;int is_extensible;};struct Cyc_Absyn_Enumfield{struct _tuple0*name;struct Cyc_Absyn_Exp*tag;unsigned loc;};struct Cyc_Absyn_Enumdecl{enum Cyc_Absyn_Scope sc;struct _tuple0*name;struct Cyc_Core_Opt*fields;};struct Cyc_Absyn_Typedefdecl{struct _tuple0*name;struct Cyc_Absyn_Tqual tq;struct Cyc_List_List*tvs;struct Cyc_Core_Opt*kind;void*defn;struct Cyc_List_List*atts;int extern_c;};struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Fn_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Fndecl*f1;};struct Cyc_Absyn_Let_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Pat*f1;struct Cyc_Core_Opt*f2;struct Cyc_Absyn_Exp*f3;void*f4;};struct Cyc_Absyn_Letv_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_Region_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_Aggr_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Aggrdecl*f1;};struct Cyc_Absyn_Datatype_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;};struct Cyc_Absyn_Enum_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;};struct Cyc_Absyn_Typedef_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Typedefdecl*f1;};struct Cyc_Absyn_Namespace_d_Absyn_Raw_decl_struct{int tag;struct _fat_ptr*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Using_d_Absyn_Raw_decl_struct{int tag;struct _tuple0*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_ExternC_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;};struct _tuple10{unsigned f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_ExternCinclude_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;struct _tuple10*f4;};struct Cyc_Absyn_Porton_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Portoff_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Tempeston_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Tempestoff_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Decl{void*r;unsigned loc;};struct Cyc_Absyn_ArrayElement_Absyn_Designator_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_FieldName_Absyn_Designator_struct{int tag;struct _fat_ptr*f1;};extern char Cyc_Absyn_EmptyAnnot[11U];struct Cyc_Absyn_EmptyAnnot_Absyn_AbsynAnnot_struct{char*tag;};
# 908 "absyn.h"
int Cyc_Absyn_qvar_cmp(struct _tuple0*,struct _tuple0*);
# 945
extern void*Cyc_Absyn_heap_rgn_type;extern void*Cyc_Absyn_unique_rgn_type;extern void*Cyc_Absyn_refcnt_rgn_type;
# 947
extern void*Cyc_Absyn_empty_effect;
# 949
extern void*Cyc_Absyn_false_type;
# 951
void*Cyc_Absyn_access_eff(void*);void*Cyc_Absyn_join_eff(struct Cyc_List_List*);
# 966
extern struct _tuple0*Cyc_Absyn_exn_name;
struct Cyc_Absyn_Datatypedecl*Cyc_Absyn_exn_tud();
# 996
void*Cyc_Absyn_fatptr_type(void*t,void*rgn,struct Cyc_Absyn_Tqual,void*zeroterm);struct Cyc_Absynpp_Params{int expand_typedefs;int qvar_to_Cids;int add_cyc_prefix;int to_VC;int decls_first;int rewrite_temp_tvars;int print_all_tvars;int print_all_kinds;int print_all_effects;int print_using_stmts;int print_externC_stmts;int print_full_evars;int print_zeroterm;int generate_line_directives;int use_curr_namespace;struct Cyc_List_List*curr_namespace;};
# 62 "absynpp.h"
struct _fat_ptr Cyc_Absynpp_typ2string(void*);
# 64
struct _fat_ptr Cyc_Absynpp_kind2string(struct Cyc_Absyn_Kind*);struct Cyc_RgnOrder_RgnPO;
# 33 "rgnorder.h"
struct Cyc_RgnOrder_RgnPO*Cyc_RgnOrder_initial_fn_po(struct Cyc_List_List*tvs,struct Cyc_List_List*po,void*effect,struct Cyc_Absyn_Tvar*fst_rgn,unsigned);
# 37
struct Cyc_RgnOrder_RgnPO*Cyc_RgnOrder_add_outlives_constraint(struct Cyc_RgnOrder_RgnPO*,void*eff,void*rgn,unsigned);
struct Cyc_RgnOrder_RgnPO*Cyc_RgnOrder_add_youngest(struct Cyc_RgnOrder_RgnPO*,struct Cyc_Absyn_Tvar*rgn,int opened);
struct Cyc_RgnOrder_RgnPO*Cyc_RgnOrder_add_unordered(struct Cyc_RgnOrder_RgnPO*,struct Cyc_Absyn_Tvar*rgn);
# 41
int Cyc_RgnOrder_rgn_outlives_rgn(struct Cyc_RgnOrder_RgnPO*,void*rgn1,void*rgn2);
int Cyc_RgnOrder_eff_outlives_eff(struct Cyc_RgnOrder_RgnPO*,void*eff1,void*eff2);
int Cyc_RgnOrder_satisfies_constraints(struct Cyc_RgnOrder_RgnPO*,struct Cyc_List_List*constraints,void*default_bound,int do_pin);struct _union_RelnOp_RConst{int tag;unsigned val;};struct _union_RelnOp_RVar{int tag;struct Cyc_Absyn_Vardecl*val;};struct _union_RelnOp_RNumelts{int tag;struct Cyc_Absyn_Vardecl*val;};struct _union_RelnOp_RType{int tag;void*val;};struct _union_RelnOp_RParam{int tag;unsigned val;};struct _union_RelnOp_RParamNumelts{int tag;unsigned val;};struct _union_RelnOp_RReturn{int tag;unsigned val;};union Cyc_Relations_RelnOp{struct _union_RelnOp_RConst RConst;struct _union_RelnOp_RVar RVar;struct _union_RelnOp_RNumelts RNumelts;struct _union_RelnOp_RType RType;struct _union_RelnOp_RParam RParam;struct _union_RelnOp_RParamNumelts RParamNumelts;struct _union_RelnOp_RReturn RReturn;};
# 50 "relations-ap.h"
enum Cyc_Relations_Relation{Cyc_Relations_Req =0U,Cyc_Relations_Rneq =1U,Cyc_Relations_Rlte =2U,Cyc_Relations_Rlt =3U};struct Cyc_Relations_Reln{union Cyc_Relations_RelnOp rop1;enum Cyc_Relations_Relation relation;union Cyc_Relations_RelnOp rop2;};struct Cyc_Iter_Iter{void*env;int(*next)(void*env,void*dest);};struct Cyc_Dict_T;struct Cyc_Dict_Dict{int(*rel)(void*,void*);struct _RegionHandle*r;const struct Cyc_Dict_T*t;};extern char Cyc_Dict_Present[8U];struct Cyc_Dict_Present_exn_struct{char*tag;};extern char Cyc_Dict_Absent[7U];struct Cyc_Dict_Absent_exn_struct{char*tag;};
# 62 "dict.h"
struct Cyc_Dict_Dict Cyc_Dict_empty(int(*cmp)(void*,void*));
# 87
struct Cyc_Dict_Dict Cyc_Dict_insert(struct Cyc_Dict_Dict d,void*k,void*v);
# 110
void*Cyc_Dict_lookup(struct Cyc_Dict_Dict d,void*k);extern char Cyc_Tcenv_Env_error[10U];struct Cyc_Tcenv_Env_error_exn_struct{char*tag;};struct Cyc_Tcenv_Genv{struct Cyc_Dict_Dict aggrdecls;struct Cyc_Dict_Dict datatypedecls;struct Cyc_Dict_Dict enumdecls;struct Cyc_Dict_Dict typedefs;struct Cyc_Dict_Dict ordinaries;};struct Cyc_Tcenv_Fenv;struct Cyc_Tcenv_Tenv{struct Cyc_List_List*ns;struct Cyc_Tcenv_Genv*ae;struct Cyc_Tcenv_Fenv*le;int allow_valueof: 1;int in_extern_c_include: 1;int in_tempest: 1;int tempest_generalize: 1;};
# 66 "tcenv.h"
void*Cyc_Tcenv_env_err(struct _fat_ptr msg);
# 88
enum Cyc_Tcenv_NewStatus{Cyc_Tcenv_NoneNew =0U,Cyc_Tcenv_InNew =1U,Cyc_Tcenv_InNewAggr =2U};
# 98
struct Cyc_List_List*Cyc_Tcenv_lookup_type_vars(struct Cyc_Tcenv_Tenv*);
# 131
struct Cyc_Tcenv_Tenv*Cyc_Tcenv_new_named_block(unsigned,struct Cyc_Tcenv_Tenv*,struct Cyc_Absyn_Tvar*);
struct Cyc_Tcenv_Tenv*Cyc_Tcenv_new_outlives_constraints(struct Cyc_Tcenv_Tenv*,struct Cyc_List_List*,unsigned);
# 30 "tcutil.h"
void*Cyc_Tcutil_impos(struct _fat_ptr,struct _fat_ptr);
void Cyc_Tcutil_terr(unsigned,struct _fat_ptr,struct _fat_ptr);
# 100
int Cyc_Tcutil_kind_leq(struct Cyc_Absyn_Kind*,struct Cyc_Absyn_Kind*);
# 105
struct Cyc_Absyn_Kind*Cyc_Tcutil_tvar_kind(struct Cyc_Absyn_Tvar*,struct Cyc_Absyn_Kind*def);
struct Cyc_Absyn_Kind*Cyc_Tcutil_type_kind(void*);
void*Cyc_Tcutil_compress(void*);
# 135
extern struct Cyc_Absyn_Kind Cyc_Tcutil_rk;
# 137
extern struct Cyc_Absyn_Kind Cyc_Tcutil_bk;
# 174
void*Cyc_Tcutil_kind_to_bound(struct Cyc_Absyn_Kind*k);struct _tuple11{struct Cyc_Absyn_Tvar*f1;void*f2;};
# 176
struct _tuple11 Cyc_Tcutil_swap_kind(void*,void*);
# 196
int Cyc_Tcutil_subset_effect(int may_constrain_evars,void*e1,void*e2);
# 200
int Cyc_Tcutil_region_in_effect(int constrain,void*r,void*e);
# 215
void Cyc_Tcutil_check_unique_tvars(unsigned,struct Cyc_List_List*);
# 256
struct Cyc_Absyn_Tvar*Cyc_Tcutil_new_tvar(void*);
# 258
int Cyc_Tcutil_new_tvar_id();
# 260
void Cyc_Tcutil_add_tvar_identity(struct Cyc_Absyn_Tvar*);
void Cyc_Tcutil_add_tvar_identities(struct Cyc_List_List*);char Cyc_Tcenv_Env_error[10U]="Env_error";
# 50 "tcenv.cyc"
struct Cyc_Tcenv_Env_error_exn_struct Cyc_Tcenv_Env_error_val={Cyc_Tcenv_Env_error};
# 52
void*Cyc_Tcenv_env_err(struct _fat_ptr msg){
(int)_throw((void*)& Cyc_Tcenv_Env_error_val);}struct _tuple12{struct Cyc_Absyn_Switch_clause*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;};struct Cyc_Tcenv_SharedFenv{void*return_typ;struct Cyc_List_List*delayed_effect_checks;struct Cyc_List_List*delayed_constraint_checks;};struct Cyc_Tcenv_FenvFlags{enum Cyc_Tcenv_NewStatus in_new;int in_notreadctxt: 1;int in_lhs: 1;int abstract_ok: 1;int in_stmt_exp: 1;};struct Cyc_Tcenv_Fenv{struct Cyc_Tcenv_SharedFenv*shared;struct Cyc_List_List*type_vars;struct Cyc_RgnOrder_RgnPO*region_order;const struct _tuple12*ctrl_env;void*capability;void*curr_rgn;struct Cyc_Tcenv_FenvFlags flags;};
# 112
struct Cyc_Tcenv_Tenv*Cyc_Tcenv_tc_init(){
# 114
struct Cyc_Tcenv_Genv*ae=({struct Cyc_Tcenv_Genv*_tmp8=_cycalloc(sizeof(*_tmp8));({struct Cyc_Dict_Dict _tmpA3=({struct Cyc_Dict_Dict(*_tmp3)(int(*cmp)(struct _tuple0*,struct _tuple0*))=(struct Cyc_Dict_Dict(*)(int(*cmp)(struct _tuple0*,struct _tuple0*)))Cyc_Dict_empty;_tmp3;})(Cyc_Absyn_qvar_cmp);_tmp8->aggrdecls=_tmpA3;}),({
struct Cyc_Dict_Dict _tmpA2=({struct Cyc_Dict_Dict(*_tmp4)(int(*cmp)(struct _tuple0*,struct _tuple0*))=(struct Cyc_Dict_Dict(*)(int(*cmp)(struct _tuple0*,struct _tuple0*)))Cyc_Dict_empty;_tmp4;})(Cyc_Absyn_qvar_cmp);_tmp8->datatypedecls=_tmpA2;}),({
struct Cyc_Dict_Dict _tmpA1=({struct Cyc_Dict_Dict(*_tmp5)(int(*cmp)(struct _tuple0*,struct _tuple0*))=(struct Cyc_Dict_Dict(*)(int(*cmp)(struct _tuple0*,struct _tuple0*)))Cyc_Dict_empty;_tmp5;})(Cyc_Absyn_qvar_cmp);_tmp8->enumdecls=_tmpA1;}),({
struct Cyc_Dict_Dict _tmpA0=({struct Cyc_Dict_Dict(*_tmp6)(int(*cmp)(struct _tuple0*,struct _tuple0*))=(struct Cyc_Dict_Dict(*)(int(*cmp)(struct _tuple0*,struct _tuple0*)))Cyc_Dict_empty;_tmp6;})(Cyc_Absyn_qvar_cmp);_tmp8->typedefs=_tmpA0;}),({
struct Cyc_Dict_Dict _tmp9F=({struct Cyc_Dict_Dict(*_tmp7)(int(*cmp)(struct _tuple0*,struct _tuple0*))=(struct Cyc_Dict_Dict(*)(int(*cmp)(struct _tuple0*,struct _tuple0*)))Cyc_Dict_empty;_tmp7;})(Cyc_Absyn_qvar_cmp);_tmp8->ordinaries=_tmp9F;});_tmp8;});
({struct Cyc_Dict_Dict _tmpA8=({struct Cyc_Dict_Dict(*_tmpA7)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Datatypedecl**v)=({struct Cyc_Dict_Dict(*_tmp0)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Datatypedecl**v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Datatypedecl**v))Cyc_Dict_insert;_tmp0;});struct Cyc_Dict_Dict _tmpA6=ae->datatypedecls;struct _tuple0*_tmpA5=Cyc_Absyn_exn_name;_tmpA7(_tmpA6,_tmpA5,({struct Cyc_Absyn_Datatypedecl**_tmp1=_cycalloc(sizeof(*_tmp1));({struct Cyc_Absyn_Datatypedecl*_tmpA4=Cyc_Absyn_exn_tud();*_tmp1=_tmpA4;});_tmp1;}));});ae->datatypedecls=_tmpA8;});
return({struct Cyc_Tcenv_Tenv*_tmp2=_cycalloc(sizeof(*_tmp2));_tmp2->ns=0,_tmp2->ae=ae,_tmp2->le=0,_tmp2->allow_valueof=0,_tmp2->in_extern_c_include=0,_tmp2->in_tempest=0,_tmp2->tempest_generalize=0;_tmp2;});}struct _tuple13{void*f1;int f2;};
# 112
void*Cyc_Tcenv_lookup_ordinary_global(struct Cyc_Tcenv_Tenv*te,unsigned loc,struct _tuple0*q,int is_use){
# 123
struct _tuple13*ans=({struct _tuple13*(*_tmpAA)(struct Cyc_Dict_Dict d,struct _tuple0*k)=({struct _tuple13*(*_tmp9)(struct Cyc_Dict_Dict d,struct _tuple0*k)=(struct _tuple13*(*)(struct Cyc_Dict_Dict d,struct _tuple0*k))Cyc_Dict_lookup;_tmp9;});struct Cyc_Dict_Dict _tmpA9=(te->ae)->ordinaries;_tmpAA(_tmpA9,q);});
if(is_use)
(*ans).f2=1;
# 124
return(*ans).f1;}
# 112
struct Cyc_Absyn_Aggrdecl**Cyc_Tcenv_lookup_aggrdecl(struct Cyc_Tcenv_Tenv*te,unsigned loc,struct _tuple0*q){
# 129
return({struct Cyc_Absyn_Aggrdecl**(*_tmpAC)(struct Cyc_Dict_Dict d,struct _tuple0*k)=({struct Cyc_Absyn_Aggrdecl**(*_tmpA)(struct Cyc_Dict_Dict d,struct _tuple0*k)=(struct Cyc_Absyn_Aggrdecl**(*)(struct Cyc_Dict_Dict d,struct _tuple0*k))Cyc_Dict_lookup;_tmpA;});struct Cyc_Dict_Dict _tmpAB=(te->ae)->aggrdecls;_tmpAC(_tmpAB,q);});}
# 112
struct Cyc_Absyn_Datatypedecl**Cyc_Tcenv_lookup_datatypedecl(struct Cyc_Tcenv_Tenv*te,unsigned loc,struct _tuple0*q){
# 132
return({struct Cyc_Absyn_Datatypedecl**(*_tmpAE)(struct Cyc_Dict_Dict d,struct _tuple0*k)=({struct Cyc_Absyn_Datatypedecl**(*_tmpB)(struct Cyc_Dict_Dict d,struct _tuple0*k)=(struct Cyc_Absyn_Datatypedecl**(*)(struct Cyc_Dict_Dict d,struct _tuple0*k))Cyc_Dict_lookup;_tmpB;});struct Cyc_Dict_Dict _tmpAD=(te->ae)->datatypedecls;_tmpAE(_tmpAD,q);});}
# 112
struct Cyc_Absyn_Datatypedecl***Cyc_Tcenv_lookup_xdatatypedecl(struct _RegionHandle*r,struct Cyc_Tcenv_Tenv*te,unsigned loc,struct _tuple0*q){
# 135
return({struct Cyc_Absyn_Datatypedecl***_tmpD=_region_malloc(r,sizeof(*_tmpD));({struct Cyc_Absyn_Datatypedecl**_tmpB1=({struct Cyc_Absyn_Datatypedecl**(*_tmpB0)(struct Cyc_Dict_Dict d,struct _tuple0*k)=({struct Cyc_Absyn_Datatypedecl**(*_tmpC)(struct Cyc_Dict_Dict d,struct _tuple0*k)=(struct Cyc_Absyn_Datatypedecl**(*)(struct Cyc_Dict_Dict d,struct _tuple0*k))Cyc_Dict_lookup;_tmpC;});struct Cyc_Dict_Dict _tmpAF=(te->ae)->datatypedecls;_tmpB0(_tmpAF,q);});*_tmpD=_tmpB1;});_tmpD;});}
# 112
struct Cyc_Absyn_Enumdecl**Cyc_Tcenv_lookup_enumdecl(struct Cyc_Tcenv_Tenv*te,unsigned loc,struct _tuple0*q){
# 138
return({struct Cyc_Absyn_Enumdecl**(*_tmpB3)(struct Cyc_Dict_Dict d,struct _tuple0*k)=({struct Cyc_Absyn_Enumdecl**(*_tmpE)(struct Cyc_Dict_Dict d,struct _tuple0*k)=(struct Cyc_Absyn_Enumdecl**(*)(struct Cyc_Dict_Dict d,struct _tuple0*k))Cyc_Dict_lookup;_tmpE;});struct Cyc_Dict_Dict _tmpB2=(te->ae)->enumdecls;_tmpB3(_tmpB2,q);});}
# 112
struct Cyc_Absyn_Typedefdecl*Cyc_Tcenv_lookup_typedefdecl(struct Cyc_Tcenv_Tenv*te,unsigned loc,struct _tuple0*q){
# 141
return({struct Cyc_Absyn_Typedefdecl*(*_tmpB5)(struct Cyc_Dict_Dict d,struct _tuple0*k)=({struct Cyc_Absyn_Typedefdecl*(*_tmpF)(struct Cyc_Dict_Dict d,struct _tuple0*k)=(struct Cyc_Absyn_Typedefdecl*(*)(struct Cyc_Dict_Dict d,struct _tuple0*k))Cyc_Dict_lookup;_tmpF;});struct Cyc_Dict_Dict _tmpB4=(te->ae)->typedefs;_tmpB5(_tmpB4,q);});}
# 112
static struct Cyc_Tcenv_Fenv*Cyc_Tcenv_get_fenv(struct Cyc_Tcenv_Tenv*te,struct _fat_ptr err_msg){
# 147
struct Cyc_Tcenv_Fenv*le=te->le;
if(le == 0)({int(*_tmpB6)(struct _fat_ptr msg)=({int(*_tmp10)(struct _fat_ptr msg)=(int(*)(struct _fat_ptr msg))Cyc_Tcenv_env_err;_tmp10;});_tmpB6(err_msg);});return le;}
# 112
static struct Cyc_Tcenv_Tenv*Cyc_Tcenv_put_fenv(struct Cyc_Tcenv_Tenv*te,struct Cyc_Tcenv_Fenv*fe){
# 152
if(te->le == 0)({int(*_tmpB7)(struct _fat_ptr msg)=({int(*_tmp11)(struct _fat_ptr msg)=(int(*)(struct _fat_ptr msg))Cyc_Tcenv_env_err;_tmp11;});_tmpB7(({const char*_tmp12="put_fenv";_tag_fat(_tmp12,sizeof(char),9U);}));});{struct Cyc_Tcenv_Tenv*ans=({struct Cyc_Tcenv_Tenv*_tmp13=_cycalloc(sizeof(*_tmp13));*_tmp13=*te;_tmp13;});
# 154
ans->le=fe;
return ans;}}
# 112
static struct Cyc_Tcenv_Tenv*Cyc_Tcenv_put_emptyfenv(struct Cyc_Tcenv_Tenv*te){
# 158
struct Cyc_Tcenv_Tenv*ans=({struct Cyc_Tcenv_Tenv*_tmp14=_cycalloc(sizeof(*_tmp14));*_tmp14=*te;_tmp14;});
ans->le=0;
return ans;}
# 112
void*Cyc_Tcenv_return_typ(struct Cyc_Tcenv_Tenv*te){
# 164
return(({struct Cyc_Tcenv_Tenv*_tmpB8=te;Cyc_Tcenv_get_fenv(_tmpB8,({const char*_tmp15="return_typ";_tag_fat(_tmp15,sizeof(char),11U);}));})->shared)->return_typ;}
# 112
struct Cyc_List_List*Cyc_Tcenv_lookup_type_vars(struct Cyc_Tcenv_Tenv*te){
# 168
struct Cyc_Tcenv_Fenv*le=te->le;
if(te->le == 0)return 0;return((struct Cyc_Tcenv_Fenv*)_check_null(le))->type_vars;}
# 112
struct Cyc_Core_Opt*Cyc_Tcenv_lookup_opt_type_vars(struct Cyc_Tcenv_Tenv*te){
# 174
return({struct Cyc_Core_Opt*_tmp16=_cycalloc(sizeof(*_tmp16));({struct Cyc_List_List*_tmpB9=Cyc_Tcenv_lookup_type_vars(te);_tmp16->v=_tmpB9;});_tmp16;});}
# 112
struct Cyc_Tcenv_Tenv*Cyc_Tcenv_add_type_vars(unsigned loc,struct Cyc_Tcenv_Tenv*te,struct Cyc_List_List*tvs){
# 178
struct Cyc_Tcenv_Fenv*fe=({struct Cyc_Tcenv_Fenv*_tmp18=_cycalloc(sizeof(*_tmp18));({struct Cyc_Tcenv_Fenv _tmpBB=*({struct Cyc_Tcenv_Tenv*_tmpBA=te;Cyc_Tcenv_get_fenv(_tmpBA,({const char*_tmp17="add_type_vars";_tag_fat(_tmp17,sizeof(char),14U);}));});*_tmp18=_tmpBB;});_tmp18;});
Cyc_Tcutil_add_tvar_identities(tvs);{
struct Cyc_List_List*new_tvs=Cyc_List_append(tvs,fe->type_vars);
Cyc_Tcutil_check_unique_tvars(loc,new_tvs);
fe->type_vars=new_tvs;
return Cyc_Tcenv_put_fenv(te,fe);}}
# 112
struct Cyc_Tcenv_Tenv*Cyc_Tcenv_set_new_status(enum Cyc_Tcenv_NewStatus status,struct Cyc_Tcenv_Tenv*te){
# 187
struct Cyc_Tcenv_Fenv*le=te->le;
if(le == 0)return Cyc_Tcenv_put_emptyfenv(te);{struct Cyc_Tcenv_Fenv*ans=({struct Cyc_Tcenv_Fenv*_tmp19=_cycalloc(sizeof(*_tmp19));*_tmp19=*le;_tmp19;});
# 190
(ans->flags).in_new=status;
return Cyc_Tcenv_put_fenv(te,ans);}}
# 112
enum Cyc_Tcenv_NewStatus Cyc_Tcenv_new_status(struct Cyc_Tcenv_Tenv*te){
# 194
struct Cyc_Tcenv_Fenv*le=te->le;
if(le == 0)return Cyc_Tcenv_NoneNew;return(le->flags).in_new;}
# 112
int Cyc_Tcenv_abstract_val_ok(struct Cyc_Tcenv_Tenv*te){
# 199
struct Cyc_Tcenv_Fenv*le=te->le;
if(le == 0)return 0;return(le->flags).abstract_ok;}
# 112
struct Cyc_Tcenv_Tenv*Cyc_Tcenv_enter_abstract_val_ok(struct Cyc_Tcenv_Tenv*te){
# 204
struct Cyc_Tcenv_Fenv*le=te->le;
if(le == 0)return Cyc_Tcenv_put_emptyfenv(te);{struct Cyc_Tcenv_Fenv*ans=({struct Cyc_Tcenv_Fenv*_tmp1A=_cycalloc(sizeof(*_tmp1A));*_tmp1A=*le;_tmp1A;});
# 207
(ans->flags).abstract_ok=1;
return Cyc_Tcenv_put_fenv(te,ans);}}
# 112
struct Cyc_Tcenv_Tenv*Cyc_Tcenv_clear_abstract_val_ok(struct Cyc_Tcenv_Tenv*te){
# 211
struct Cyc_Tcenv_Fenv*le=te->le;
if(le == 0)return Cyc_Tcenv_put_emptyfenv(te);{struct Cyc_Tcenv_Fenv*ans=({struct Cyc_Tcenv_Fenv*_tmp1B=_cycalloc(sizeof(*_tmp1B));*_tmp1B=*le;_tmp1B;});
# 214
(ans->flags).abstract_ok=0;
return Cyc_Tcenv_put_fenv(te,ans);}}
# 112
struct Cyc_Tcenv_Tenv*Cyc_Tcenv_enter_notreadctxt(struct Cyc_Tcenv_Tenv*te){
# 218
struct Cyc_Tcenv_Fenv*le=te->le;
if(le == 0)return Cyc_Tcenv_put_emptyfenv(te);{struct Cyc_Tcenv_Fenv*ans=({struct Cyc_Tcenv_Fenv*_tmp1C=_cycalloc(sizeof(*_tmp1C));*_tmp1C=*le;_tmp1C;});
# 221
(ans->flags).in_notreadctxt=1;
return Cyc_Tcenv_put_fenv(te,ans);}}
# 112
struct Cyc_Tcenv_Tenv*Cyc_Tcenv_clear_notreadctxt(struct Cyc_Tcenv_Tenv*te){
# 225
struct Cyc_Tcenv_Fenv*le=te->le;
if(le == 0)return Cyc_Tcenv_put_emptyfenv(te);{struct Cyc_Tcenv_Fenv*ans=({struct Cyc_Tcenv_Fenv*_tmp1D=_cycalloc(sizeof(*_tmp1D));*_tmp1D=*le;_tmp1D;});
# 228
(ans->flags).in_notreadctxt=0;
return Cyc_Tcenv_put_fenv(te,ans);}}
# 112
int Cyc_Tcenv_in_notreadctxt(struct Cyc_Tcenv_Tenv*te){
# 232
struct Cyc_Tcenv_Fenv*le=te->le;
if(le == 0)return 0;return(le->flags).in_notreadctxt;}
# 112
struct Cyc_Tcenv_Tenv*Cyc_Tcenv_enter_lhs(struct Cyc_Tcenv_Tenv*te){
# 237
struct Cyc_Tcenv_Fenv*le=te->le;
if(le == 0)return Cyc_Tcenv_put_emptyfenv(te);{struct Cyc_Tcenv_Fenv*ans=({struct Cyc_Tcenv_Fenv*_tmp1E=_cycalloc(sizeof(*_tmp1E));*_tmp1E=*le;_tmp1E;});
# 240
(ans->flags).in_lhs=1;
return Cyc_Tcenv_put_fenv(te,ans);}}
# 112
struct Cyc_Tcenv_Tenv*Cyc_Tcenv_clear_lhs(struct Cyc_Tcenv_Tenv*te){
# 244
struct Cyc_Tcenv_Fenv*le=te->le;
if(le == 0)return Cyc_Tcenv_put_emptyfenv(te);{struct Cyc_Tcenv_Fenv*ans=({struct Cyc_Tcenv_Fenv*_tmp1F=_cycalloc(sizeof(*_tmp1F));*_tmp1F=*le;_tmp1F;});
# 247
(ans->flags).in_lhs=0;
return Cyc_Tcenv_put_fenv(te,ans);}}
# 112
int Cyc_Tcenv_in_lhs(struct Cyc_Tcenv_Tenv*te){
# 251
struct Cyc_Tcenv_Fenv*le=te->le;
if(le == 0)return 0;return(le->flags).in_lhs;}
# 112
struct Cyc_Tcenv_Tenv*Cyc_Tcenv_enter_stmt_exp(struct Cyc_Tcenv_Tenv*te){
# 256
struct Cyc_Tcenv_Fenv*le=te->le;
if(le == 0)return Cyc_Tcenv_put_emptyfenv(te);{struct Cyc_Tcenv_Fenv*ans=({struct Cyc_Tcenv_Fenv*_tmp20=_cycalloc(sizeof(*_tmp20));*_tmp20=*le;_tmp20;});
# 259
(ans->flags).in_stmt_exp=1;
return Cyc_Tcenv_put_fenv(te,ans);}}
# 112
int Cyc_Tcenv_in_stmt_exp(struct Cyc_Tcenv_Tenv*te){
# 263
struct Cyc_Tcenv_Fenv*le=te->le;
if(le == 0)return 0;return(le->flags).in_stmt_exp;}
# 112
const struct _tuple12*const Cyc_Tcenv_process_fallthru(struct Cyc_Tcenv_Tenv*te,struct Cyc_Absyn_Stmt*s,struct Cyc_Absyn_Switch_clause***clauseopt){
# 271
const struct _tuple12*ans=({struct Cyc_Tcenv_Tenv*_tmpBC=te;Cyc_Tcenv_get_fenv(_tmpBC,({const char*_tmp22="process_fallthru";_tag_fat(_tmp22,sizeof(char),17U);}));})->ctrl_env;
if(ans != (const struct _tuple12*)0)
({struct Cyc_Absyn_Switch_clause**_tmpBD=({struct Cyc_Absyn_Switch_clause**_tmp21=_cycalloc(sizeof(*_tmp21));*_tmp21=(*ans).f1;_tmp21;});*clauseopt=_tmpBD;});
# 272
return ans;}
# 112
struct Cyc_Tcenv_Tenv*Cyc_Tcenv_set_fallthru(struct Cyc_Tcenv_Tenv*te,struct Cyc_List_List*new_tvs,struct Cyc_List_List*vds,struct Cyc_Absyn_Switch_clause*clause){
# 279
struct Cyc_List_List*ft_typ=0;
for(0;vds != 0;vds=vds->tl){
ft_typ=({struct Cyc_List_List*_tmp23=_cycalloc(sizeof(*_tmp23));_tmp23->hd=((struct Cyc_Absyn_Vardecl*)vds->hd)->type,_tmp23->tl=ft_typ;_tmp23;});}{
struct _tuple12*new_ctrl_env=({struct _tuple12*_tmp26=_cycalloc(sizeof(*_tmp26));_tmp26->f1=clause,_tmp26->f2=new_tvs,({struct Cyc_List_List*_tmpBE=Cyc_List_imp_rev(ft_typ);_tmp26->f3=_tmpBE;});_tmp26;});
struct Cyc_Tcenv_Fenv*new_fe=({struct Cyc_Tcenv_Fenv*_tmp25=_cycalloc(sizeof(*_tmp25));({struct Cyc_Tcenv_Fenv _tmpC0=*({struct Cyc_Tcenv_Tenv*_tmpBF=te;Cyc_Tcenv_get_fenv(_tmpBF,({const char*_tmp24="set_fallthru";_tag_fat(_tmp24,sizeof(char),13U);}));});*_tmp25=_tmpC0;});_tmp25;});
new_fe->ctrl_env=(const struct _tuple12*)new_ctrl_env;
return Cyc_Tcenv_put_fenv(te,new_fe);}}
# 112
struct Cyc_Tcenv_Tenv*Cyc_Tcenv_clear_fallthru(struct Cyc_Tcenv_Tenv*te){
# 289
struct Cyc_Tcenv_Fenv*fe=({struct Cyc_Tcenv_Fenv*_tmp28=_cycalloc(sizeof(*_tmp28));({struct Cyc_Tcenv_Fenv _tmpC2=*({struct Cyc_Tcenv_Tenv*_tmpC1=te;Cyc_Tcenv_get_fenv(_tmpC1,({const char*_tmp27="clear_fallthru";_tag_fat(_tmp27,sizeof(char),15U);}));});*_tmp28=_tmpC2;});_tmp28;});
fe->ctrl_env=0;
return Cyc_Tcenv_put_fenv(te,fe);}
# 112
struct Cyc_Tcenv_Tenv*Cyc_Tcenv_allow_valueof(struct Cyc_Tcenv_Tenv*te){
# 295
struct Cyc_Tcenv_Fenv*fe=te->le == 0?0:({struct Cyc_Tcenv_Fenv*_tmp2A=_cycalloc(sizeof(*_tmp2A));*_tmp2A=*((struct Cyc_Tcenv_Fenv*)_check_null(te->le));_tmp2A;});
struct Cyc_Tcenv_Tenv*ans=({struct Cyc_Tcenv_Tenv*_tmp29=_cycalloc(sizeof(*_tmp29));*_tmp29=*te;_tmp29;});
ans->allow_valueof=1;
return ans;}
# 112
struct Cyc_Tcenv_Tenv*Cyc_Tcenv_enter_extern_c_include(struct Cyc_Tcenv_Tenv*te){
# 301
struct Cyc_Tcenv_Fenv*fe=te->le == 0?0:({struct Cyc_Tcenv_Fenv*_tmp2C=_cycalloc(sizeof(*_tmp2C));*_tmp2C=*((struct Cyc_Tcenv_Fenv*)_check_null(te->le));_tmp2C;});
struct Cyc_Tcenv_Tenv*ans=({struct Cyc_Tcenv_Tenv*_tmp2B=_cycalloc(sizeof(*_tmp2B));*_tmp2B=*te;_tmp2B;});
ans->in_extern_c_include=1;
return ans;}
# 112
struct Cyc_Tcenv_Tenv*Cyc_Tcenv_enter_tempest(struct Cyc_Tcenv_Tenv*te){
# 307
struct Cyc_Tcenv_Fenv*fe=te->le == 0?0:({struct Cyc_Tcenv_Fenv*_tmp2E=_cycalloc(sizeof(*_tmp2E));*_tmp2E=*((struct Cyc_Tcenv_Fenv*)_check_null(te->le));_tmp2E;});
struct Cyc_Tcenv_Tenv*ans=({struct Cyc_Tcenv_Tenv*_tmp2D=_cycalloc(sizeof(*_tmp2D));*_tmp2D=*te;_tmp2D;});
ans->in_tempest=1;
return ans;}
# 112
struct Cyc_Tcenv_Tenv*Cyc_Tcenv_clear_tempest(struct Cyc_Tcenv_Tenv*te){
# 313
struct Cyc_Tcenv_Fenv*fe=te->le == 0?0:({struct Cyc_Tcenv_Fenv*_tmp30=_cycalloc(sizeof(*_tmp30));*_tmp30=*((struct Cyc_Tcenv_Fenv*)_check_null(te->le));_tmp30;});
struct Cyc_Tcenv_Tenv*ans=({struct Cyc_Tcenv_Tenv*_tmp2F=_cycalloc(sizeof(*_tmp2F));*_tmp2F=*te;_tmp2F;});
ans->in_tempest=0;
return ans;}
# 112
int Cyc_Tcenv_in_tempest(struct Cyc_Tcenv_Tenv*te){
# 319
return te->in_tempest;}
# 112
struct Cyc_Tcenv_Tenv*Cyc_Tcenv_add_region(struct Cyc_Tcenv_Tenv*te,void*rgn,int opened){
# 322
struct Cyc_Tcenv_Fenv*ans=({struct Cyc_Tcenv_Fenv*_tmp36=_cycalloc(sizeof(*_tmp36));({struct Cyc_Tcenv_Fenv _tmpC4=*({struct Cyc_Tcenv_Tenv*_tmpC3=te;Cyc_Tcenv_get_fenv(_tmpC3,({const char*_tmp35="add_region";_tag_fat(_tmp35,sizeof(char),11U);}));});*_tmp36=_tmpC4;});_tmp36;});
struct Cyc_RgnOrder_RgnPO*region_order=ans->region_order;
{void*_stmttmp0=Cyc_Tcutil_compress(rgn);void*_tmp31=_stmttmp0;struct Cyc_Absyn_Tvar*_tmp32;if(((struct Cyc_Absyn_VarType_Absyn_Type_struct*)_tmp31)->tag == 2U){_LL1: _tmp32=((struct Cyc_Absyn_VarType_Absyn_Type_struct*)_tmp31)->f1;_LL2: {struct Cyc_Absyn_Tvar*x=_tmp32;
# 326
region_order=Cyc_RgnOrder_add_youngest(region_order,x,opened);
goto _LL0;}}else{_LL3: _LL4:
 goto _LL0;}_LL0:;}
# 330
ans->region_order=region_order;
({void*_tmpC7=Cyc_Absyn_join_eff(({struct Cyc_List_List*_tmp34=_cycalloc(sizeof(*_tmp34));({void*_tmpC6=Cyc_Absyn_access_eff(rgn);_tmp34->hd=_tmpC6;}),({
struct Cyc_List_List*_tmpC5=({struct Cyc_List_List*_tmp33=_cycalloc(sizeof(*_tmp33));_tmp33->hd=ans->capability,_tmp33->tl=0;_tmp33;});_tmp34->tl=_tmpC5;});_tmp34;}));
# 331
ans->capability=_tmpC7;});
# 333
return Cyc_Tcenv_put_fenv(te,ans);}
# 112
struct Cyc_Tcenv_Tenv*Cyc_Tcenv_new_named_block(unsigned loc,struct Cyc_Tcenv_Tenv*te,struct Cyc_Absyn_Tvar*block_rgn){
# 337
struct Cyc_Tcenv_Fenv*ans=({struct Cyc_Tcenv_Fenv*_tmp3C=_cycalloc(sizeof(*_tmp3C));({struct Cyc_Tcenv_Fenv _tmpC9=*({struct Cyc_Tcenv_Tenv*_tmpC8=te;Cyc_Tcenv_get_fenv(_tmpC8,({const char*_tmp3B="new_named_block";_tag_fat(_tmp3B,sizeof(char),16U);}));});*_tmp3C=_tmpC9;});_tmp3C;});
struct Cyc_Absyn_VarType_Absyn_Type_struct*block_typ=({struct Cyc_Absyn_VarType_Absyn_Type_struct*_tmp3A=_cycalloc(sizeof(*_tmp3A));_tmp3A->tag=2U,_tmp3A->f1=block_rgn;_tmp3A;});
({struct Cyc_List_List*_tmpCA=({struct Cyc_List_List*_tmp37=_cycalloc(sizeof(*_tmp37));_tmp37->hd=block_rgn,_tmp37->tl=ans->type_vars;_tmp37;});ans->type_vars=_tmpCA;});
Cyc_Tcutil_check_unique_tvars(loc,ans->type_vars);
({struct Cyc_RgnOrder_RgnPO*_tmpCB=Cyc_RgnOrder_add_youngest(ans->region_order,block_rgn,0);ans->region_order=_tmpCB;});
({void*_tmpCE=Cyc_Absyn_join_eff(({struct Cyc_List_List*_tmp39=_cycalloc(sizeof(*_tmp39));({void*_tmpCD=Cyc_Absyn_access_eff((void*)block_typ);_tmp39->hd=_tmpCD;}),({
struct Cyc_List_List*_tmpCC=({struct Cyc_List_List*_tmp38=_cycalloc(sizeof(*_tmp38));_tmp38->hd=ans->capability,_tmp38->tl=0;_tmp38;});_tmp39->tl=_tmpCC;});_tmp39;}));
# 342
ans->capability=_tmpCE;});
# 344
ans->curr_rgn=(void*)block_typ;
return Cyc_Tcenv_put_fenv(te,ans);}
# 112
static struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct Cyc_Tcenv_rgn_kb={0U,& Cyc_Tcutil_rk};
# 350
struct Cyc_Tcenv_Tenv*Cyc_Tcenv_new_block(unsigned loc,struct Cyc_Tcenv_Tenv*te){
struct Cyc_Absyn_Tvar*t=Cyc_Tcutil_new_tvar((void*)& Cyc_Tcenv_rgn_kb);
Cyc_Tcutil_add_tvar_identity(t);
return Cyc_Tcenv_new_named_block(loc,te,t);}struct _tuple14{void*f1;void*f2;};
# 350
struct Cyc_Tcenv_Tenv*Cyc_Tcenv_new_outlives_constraints(struct Cyc_Tcenv_Tenv*te,struct Cyc_List_List*cs,unsigned loc){
# 363
struct Cyc_Tcenv_Fenv*ans=({struct Cyc_Tcenv_Fenv*_tmp3E=_cycalloc(sizeof(*_tmp3E));({struct Cyc_Tcenv_Fenv _tmpD0=*({struct Cyc_Tcenv_Tenv*_tmpCF=te;Cyc_Tcenv_get_fenv(_tmpCF,({const char*_tmp3D="new_outlives_constraints";_tag_fat(_tmp3D,sizeof(char),25U);}));});*_tmp3E=_tmpD0;});_tmp3E;});
for(0;cs != 0;cs=cs->tl){
({struct Cyc_RgnOrder_RgnPO*_tmpD1=
Cyc_RgnOrder_add_outlives_constraint(ans->region_order,(*((struct _tuple14*)cs->hd)).f1,(*((struct _tuple14*)cs->hd)).f2,loc);
# 365
ans->region_order=_tmpD1;});}
# 367
return Cyc_Tcenv_put_fenv(te,ans);}
# 350
struct Cyc_Tcenv_Tenv*Cyc_Tcenv_add_region_equality(struct Cyc_Tcenv_Tenv*te,void*r10,void*r20,struct _tuple11**oldtv,unsigned loc){
# 374
void*r1=Cyc_Tcutil_compress(r10);
void*r2=Cyc_Tcutil_compress(r20);
struct Cyc_Absyn_Kind*r1k=Cyc_Tcutil_type_kind(r1);
struct Cyc_Absyn_Kind*r2k=Cyc_Tcutil_type_kind(r2);
# 380
int r1_le_r2=Cyc_Tcutil_kind_leq(r1k,r2k);
int r2_le_r1=Cyc_Tcutil_kind_leq(r2k,r1k);
if(!r1_le_r2 && !r2_le_r1){
({struct Cyc_String_pa_PrintArg_struct _tmp41=({struct Cyc_String_pa_PrintArg_struct _tmp99;_tmp99.tag=0U,({
struct _fat_ptr _tmpD2=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(r1));_tmp99.f1=_tmpD2;});_tmp99;});struct Cyc_String_pa_PrintArg_struct _tmp42=({struct Cyc_String_pa_PrintArg_struct _tmp98;_tmp98.tag=0U,({struct _fat_ptr _tmpD3=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(r2));_tmp98.f1=_tmpD3;});_tmp98;});struct Cyc_String_pa_PrintArg_struct _tmp43=({struct Cyc_String_pa_PrintArg_struct _tmp97;_tmp97.tag=0U,({struct _fat_ptr _tmpD4=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_kind2string(r1k));_tmp97.f1=_tmpD4;});_tmp97;});struct Cyc_String_pa_PrintArg_struct _tmp44=({struct Cyc_String_pa_PrintArg_struct _tmp96;_tmp96.tag=0U,({struct _fat_ptr _tmpD5=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_kind2string(r2k));_tmp96.f1=_tmpD5;});_tmp96;});void*_tmp3F[4U];_tmp3F[0]=& _tmp41,_tmp3F[1]=& _tmp42,_tmp3F[2]=& _tmp43,_tmp3F[3]=& _tmp44;({unsigned _tmpD7=loc;struct _fat_ptr _tmpD6=({const char*_tmp40="Cannot compare region handles for %s and %s\n  kinds %s and %s are not compatible\n";_tag_fat(_tmp40,sizeof(char),82U);});Cyc_Tcutil_terr(_tmpD7,_tmpD6,_tag_fat(_tmp3F,sizeof(void*),4U));});});
return Cyc_Tcenv_new_outlives_constraints(te,0,loc);}else{
# 387
if(r1_le_r2 && !r2_le_r1)
({struct _tuple11*_tmpDA=({struct _tuple11*_tmp45=_cycalloc(sizeof(*_tmp45));({struct _tuple11 _tmpD9=({void*_tmpD8=r2;Cyc_Tcutil_swap_kind(_tmpD8,Cyc_Tcutil_kind_to_bound(r1k));});*_tmp45=_tmpD9;});_tmp45;});*oldtv=_tmpDA;});else{
if(!r1_le_r2 && r2_le_r1)
({struct _tuple11*_tmpDD=({struct _tuple11*_tmp46=_cycalloc(sizeof(*_tmp46));({struct _tuple11 _tmpDC=({void*_tmpDB=r1;Cyc_Tcutil_swap_kind(_tmpDB,Cyc_Tcutil_kind_to_bound(r2k));});*_tmp46=_tmpDC;});_tmp46;});*oldtv=_tmpDD;});}}{
# 382
struct Cyc_List_List*bds=0;
# 394
if((r1 != Cyc_Absyn_heap_rgn_type && r1 != Cyc_Absyn_unique_rgn_type)&& r1 != Cyc_Absyn_refcnt_rgn_type)
bds=({struct Cyc_List_List*_tmp48=_cycalloc(sizeof(*_tmp48));({struct _tuple14*_tmpDF=({struct _tuple14*_tmp47=_cycalloc(sizeof(*_tmp47));({void*_tmpDE=Cyc_Absyn_access_eff(r1);_tmp47->f1=_tmpDE;}),_tmp47->f2=r2;_tmp47;});_tmp48->hd=_tmpDF;}),_tmp48->tl=bds;_tmp48;});
# 394
if(
# 396
(r2 != Cyc_Absyn_heap_rgn_type && r2 != Cyc_Absyn_unique_rgn_type)&& r2 != Cyc_Absyn_refcnt_rgn_type)
bds=({struct Cyc_List_List*_tmp4A=_cycalloc(sizeof(*_tmp4A));({struct _tuple14*_tmpE1=({struct _tuple14*_tmp49=_cycalloc(sizeof(*_tmp49));({void*_tmpE0=Cyc_Absyn_access_eff(r2);_tmp49->f1=_tmpE0;}),_tmp49->f2=r1;_tmp49;});_tmp4A->hd=_tmpE1;}),_tmp4A->tl=bds;_tmp4A;});
# 394
return
# 398
 Cyc_Tcenv_new_outlives_constraints(te,bds,loc);}}
# 350
void*Cyc_Tcenv_curr_rgn(struct Cyc_Tcenv_Tenv*te){
# 404
struct Cyc_Tcenv_Fenv*le=te->le;
if(le == 0)return Cyc_Absyn_heap_rgn_type;return le->curr_rgn;}
# 350
static struct Cyc_RgnOrder_RgnPO*Cyc_Tcenv_curr_rgnpo(struct Cyc_Tcenv_Tenv*te){
# 412
struct Cyc_Tcenv_Fenv*le=te->le;
if(le == 0)return 0;return le->region_order;}
# 350
void Cyc_Tcenv_check_rgn_accessible(struct Cyc_Tcenv_Tenv*te,unsigned loc,void*rgn){
# 423
struct Cyc_Tcenv_Fenv*fe=({struct Cyc_Tcenv_Tenv*_tmpE2=te;Cyc_Tcenv_get_fenv(_tmpE2,({const char*_tmp4E="check_rgn_accessible";_tag_fat(_tmp4E,sizeof(char),21U);}));});
if(Cyc_Tcutil_region_in_effect(0,rgn,fe->capability)||
 Cyc_Tcutil_region_in_effect(1,rgn,fe->capability))
return;
# 424
if(({
# 427
struct Cyc_RgnOrder_RgnPO*_tmpE4=fe->region_order;void*_tmpE3=Cyc_Absyn_access_eff(rgn);Cyc_RgnOrder_eff_outlives_eff(_tmpE4,_tmpE3,fe->capability);}))
return;
# 424
({struct Cyc_String_pa_PrintArg_struct _tmp4D=({struct Cyc_String_pa_PrintArg_struct _tmp9A;_tmp9A.tag=0U,({
# 429
struct _fat_ptr _tmpE5=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(rgn));_tmp9A.f1=_tmpE5;});_tmp9A;});void*_tmp4B[1U];_tmp4B[0]=& _tmp4D;({unsigned _tmpE7=loc;struct _fat_ptr _tmpE6=({const char*_tmp4C="Expression accesses unavailable region %s";_tag_fat(_tmp4C,sizeof(char),42U);});Cyc_Tcutil_terr(_tmpE7,_tmpE6,_tag_fat(_tmp4B,sizeof(void*),1U));});});}struct _tuple15{void*f1;void*f2;struct Cyc_RgnOrder_RgnPO*f3;unsigned f4;};
# 350
void Cyc_Tcenv_check_effect_accessible(struct Cyc_Tcenv_Tenv*te,unsigned loc,void*eff){
# 436
struct Cyc_Tcenv_Fenv*_stmttmp1=({
struct Cyc_Tcenv_Tenv*_tmpE8=te;Cyc_Tcenv_get_fenv(_tmpE8,({const char*_tmp54="check_effect_accessible";_tag_fat(_tmp54,sizeof(char),24U);}));});
# 436
struct Cyc_Tcenv_SharedFenv*_tmp51;struct Cyc_RgnOrder_RgnPO*_tmp50;void*_tmp4F;_LL1: _tmp4F=_stmttmp1->capability;_tmp50=_stmttmp1->region_order;_tmp51=_stmttmp1->shared;_LL2: {void*capability=_tmp4F;struct Cyc_RgnOrder_RgnPO*ro=_tmp50;struct Cyc_Tcenv_SharedFenv*shared=_tmp51;
# 438
if(Cyc_Tcutil_subset_effect(0,eff,capability))
return;
# 438
if(
# 440
Cyc_RgnOrder_eff_outlives_eff(ro,eff,capability))
return;
# 438
({struct Cyc_List_List*_tmpEA=({struct Cyc_List_List*_tmp53=_cycalloc(sizeof(*_tmp53));
# 443
({struct _tuple15*_tmpE9=({struct _tuple15*_tmp52=_cycalloc(sizeof(*_tmp52));_tmp52->f1=capability,_tmp52->f2=eff,_tmp52->f3=ro,_tmp52->f4=loc;_tmp52;});_tmp53->hd=_tmpE9;}),_tmp53->tl=shared->delayed_effect_checks;_tmp53;});
# 438
shared->delayed_effect_checks=_tmpEA;});}}
# 350
void Cyc_Tcenv_check_delayed_effects(struct Cyc_Tcenv_Tenv*te){
# 448
struct Cyc_List_List*checks=(({struct Cyc_Tcenv_Tenv*_tmpEB=te;Cyc_Tcenv_get_fenv(_tmpEB,({const char*_tmp5D="check_delayed_constraints";_tag_fat(_tmp5D,sizeof(char),26U);}));})->shared)->delayed_effect_checks;
for(0;checks != 0;checks=checks->tl){
# 451
struct _tuple15*_stmttmp2=(struct _tuple15*)checks->hd;unsigned _tmp58;struct Cyc_RgnOrder_RgnPO*_tmp57;void*_tmp56;void*_tmp55;_LL1: _tmp55=_stmttmp2->f1;_tmp56=_stmttmp2->f2;_tmp57=_stmttmp2->f3;_tmp58=_stmttmp2->f4;_LL2: {void*capability=_tmp55;void*eff=_tmp56;struct Cyc_RgnOrder_RgnPO*rgn_order=_tmp57;unsigned loc=_tmp58;
if(Cyc_Tcutil_subset_effect(1,eff,capability))continue;if(
Cyc_RgnOrder_eff_outlives_eff(rgn_order,eff,capability))continue;
# 452
({struct Cyc_String_pa_PrintArg_struct _tmp5B=({struct Cyc_String_pa_PrintArg_struct _tmp9C;_tmp9C.tag=0U,({
# 454
struct _fat_ptr _tmpEC=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(capability));_tmp9C.f1=_tmpEC;});_tmp9C;});struct Cyc_String_pa_PrintArg_struct _tmp5C=({struct Cyc_String_pa_PrintArg_struct _tmp9B;_tmp9B.tag=0U,({struct _fat_ptr _tmpED=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(eff));_tmp9B.f1=_tmpED;});_tmp9B;});void*_tmp59[2U];_tmp59[0]=& _tmp5B,_tmp59[1]=& _tmp5C;({unsigned _tmpEF=loc;struct _fat_ptr _tmpEE=({const char*_tmp5A="Capability \n%s\ndoes not cover function's effect\n%s";_tag_fat(_tmp5A,sizeof(char),51U);});Cyc_Tcutil_terr(_tmpEF,_tmpEE,_tag_fat(_tmp59,sizeof(void*),2U));});});}}}struct _tuple16{struct Cyc_RgnOrder_RgnPO*f1;struct Cyc_List_List*f2;unsigned f3;};
# 350
void Cyc_Tcenv_check_rgn_partial_order(struct Cyc_Tcenv_Tenv*te,unsigned loc,struct Cyc_List_List*po){
# 467
struct Cyc_Tcenv_Fenv*le=te->le;
if(le == 0){
# 471
for(0;po != 0;po=po->tl){
if(!Cyc_Tcutil_subset_effect(1,(*((struct _tuple14*)po->hd)).f1,Cyc_Absyn_empty_effect)|| !({
void*_tmpF0=Cyc_Absyn_access_eff((*((struct _tuple14*)po->hd)).f2);Cyc_Tcutil_subset_effect(1,_tmpF0,Cyc_Absyn_empty_effect);}))
({void*_tmp5E=0U;({unsigned _tmpF2=loc;struct _fat_ptr _tmpF1=({const char*_tmp5F="the required region ordering is not satisfied here";_tag_fat(_tmp5F,sizeof(char),51U);});Cyc_Tcutil_terr(_tmpF2,_tmpF1,_tag_fat(_tmp5E,sizeof(void*),0U));});});}
# 471
return;}{
# 468
struct Cyc_Tcenv_Fenv*_stmttmp3=le;struct Cyc_Tcenv_SharedFenv*_tmp61;struct Cyc_RgnOrder_RgnPO*_tmp60;_LL1: _tmp60=_stmttmp3->region_order;_tmp61=_stmttmp3->shared;_LL2: {struct Cyc_RgnOrder_RgnPO*ro=_tmp60;struct Cyc_Tcenv_SharedFenv*shared=_tmp61;
# 478
if(!Cyc_RgnOrder_satisfies_constraints(ro,po,Cyc_Absyn_heap_rgn_type,0))
({struct Cyc_List_List*_tmpF4=({struct Cyc_List_List*_tmp63=_cycalloc(sizeof(*_tmp63));
({struct _tuple16*_tmpF3=({struct _tuple16*_tmp62=_cycalloc(sizeof(*_tmp62));_tmp62->f1=ro,_tmp62->f2=po,_tmp62->f3=loc;_tmp62;});_tmp63->hd=_tmpF3;}),_tmp63->tl=shared->delayed_constraint_checks;_tmp63;});
# 479
shared->delayed_constraint_checks=_tmpF4;});}}}
# 350
void Cyc_Tcenv_check_delayed_constraints(struct Cyc_Tcenv_Tenv*te){
# 484
struct Cyc_List_List*checks=(({
struct Cyc_Tcenv_Tenv*_tmpF5=te;Cyc_Tcenv_get_fenv(_tmpF5,({const char*_tmp69="check_delayed_constraints";_tag_fat(_tmp69,sizeof(char),26U);}));})->shared)->delayed_constraint_checks;
for(0;checks != 0;checks=checks->tl){
struct _tuple16*_stmttmp4=(struct _tuple16*)checks->hd;unsigned _tmp66;struct Cyc_List_List*_tmp65;struct Cyc_RgnOrder_RgnPO*_tmp64;_LL1: _tmp64=_stmttmp4->f1;_tmp65=_stmttmp4->f2;_tmp66=_stmttmp4->f3;_LL2: {struct Cyc_RgnOrder_RgnPO*rgn_order=_tmp64;struct Cyc_List_List*po=_tmp65;unsigned loc=_tmp66;
if(!Cyc_RgnOrder_satisfies_constraints(rgn_order,po,Cyc_Absyn_heap_rgn_type,1))
({void*_tmp67=0U;({unsigned _tmpF7=loc;struct _fat_ptr _tmpF6=({const char*_tmp68="the required region ordering is not satisfied here";_tag_fat(_tmp68,sizeof(char),51U);});Cyc_Tcutil_terr(_tmpF7,_tmpF6,_tag_fat(_tmp67,sizeof(void*),0U));});});}}}
# 350
static struct Cyc_Tcenv_SharedFenv*Cyc_Tcenv_new_shared_fenv(void*ret){
# 494
return({struct Cyc_Tcenv_SharedFenv*_tmp6A=_cycalloc(sizeof(*_tmp6A));_tmp6A->return_typ=ret,_tmp6A->delayed_effect_checks=0,_tmp6A->delayed_constraint_checks=0;_tmp6A;});}
# 350
struct Cyc_Tcenv_Fenv*Cyc_Tcenv_new_fenv(unsigned loc,struct Cyc_Absyn_Fndecl*fd){
# 500
struct Cyc_Absyn_Tvar*rgn0=({struct Cyc_Absyn_Tvar*_tmp78=_cycalloc(sizeof(*_tmp78));
({struct _fat_ptr*_tmpFB=({struct _fat_ptr*_tmp77=_cycalloc(sizeof(*_tmp77));({struct _fat_ptr _tmpFA=(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp76=({struct Cyc_String_pa_PrintArg_struct _tmp9D;_tmp9D.tag=0U,_tmp9D.f1=(struct _fat_ptr)((struct _fat_ptr)*(*fd->name).f2);_tmp9D;});void*_tmp74[1U];_tmp74[0]=& _tmp76;({struct _fat_ptr _tmpF9=({const char*_tmp75="`%s";_tag_fat(_tmp75,sizeof(char),4U);});Cyc_aprintf(_tmpF9,_tag_fat(_tmp74,sizeof(void*),1U));});});*_tmp77=_tmpFA;});_tmp77;});_tmp78->name=_tmpFB;}),({
int _tmpF8=Cyc_Tcutil_new_tvar_id();_tmp78->identity=_tmpF8;}),_tmp78->kind=(void*)& Cyc_Tcenv_rgn_kb;_tmp78;});
struct Cyc_List_List*tvs=({struct Cyc_List_List*_tmp73=_cycalloc(sizeof(*_tmp73));_tmp73->hd=rgn0,_tmp73->tl=(fd->i).tvars;_tmp73;});
Cyc_Tcutil_check_unique_tvars(loc,tvs);{
# 506
struct Cyc_RgnOrder_RgnPO*rgn_po=Cyc_RgnOrder_initial_fn_po((fd->i).tvars,(fd->i).rgn_po,(void*)_check_null((fd->i).effect),rgn0,loc);
void*param_rgn=(void*)({struct Cyc_Absyn_VarType_Absyn_Type_struct*_tmp72=_cycalloc(sizeof(*_tmp72));_tmp72->tag=2U,_tmp72->f1=rgn0;_tmp72;});
{struct Cyc_List_List*vds=(struct Cyc_List_List*)((struct Cyc_Core_Opt*)_check_null(fd->param_vardecls))->v;for(0;vds != 0;vds=vds->tl){
((struct Cyc_Absyn_Vardecl*)vds->hd)->rgn=param_rgn;}}
if((fd->i).cyc_varargs != 0){
struct Cyc_Absyn_VarargInfo _stmttmp5=*((struct Cyc_Absyn_VarargInfo*)_check_null((fd->i).cyc_varargs));int _tmp6E;void*_tmp6D;struct Cyc_Absyn_Tqual _tmp6C;struct _fat_ptr*_tmp6B;_LL1: _tmp6B=_stmttmp5.name;_tmp6C=_stmttmp5.tq;_tmp6D=_stmttmp5.type;_tmp6E=_stmttmp5.inject;_LL2: {struct _fat_ptr*n=_tmp6B;struct Cyc_Absyn_Tqual tq=_tmp6C;void*t=_tmp6D;int i=_tmp6E;
# 513
struct Cyc_List_List*vds=(struct Cyc_List_List*)((struct Cyc_Core_Opt*)_check_null(fd->param_vardecls))->v;for(0;vds != 0;vds=vds->tl){
struct Cyc_Absyn_Vardecl*vd=(struct Cyc_Absyn_Vardecl*)vds->hd;
if(Cyc_strptrcmp((*vd->name).f2,(struct _fat_ptr*)_check_null(n))== 0){
({void*_tmpFC=Cyc_Absyn_fatptr_type(t,param_rgn,tq,Cyc_Absyn_false_type);vd->type=_tmpFC;});
break;}}}}
# 510
return({struct Cyc_Tcenv_Fenv*_tmp71=_cycalloc(sizeof(*_tmp71));
# 522
({struct Cyc_Tcenv_SharedFenv*_tmp100=Cyc_Tcenv_new_shared_fenv((fd->i).ret_type);_tmp71->shared=_tmp100;}),_tmp71->type_vars=tvs,_tmp71->region_order=rgn_po,_tmp71->ctrl_env=0,({
# 526
void*_tmpFF=Cyc_Absyn_join_eff(({struct Cyc_List_List*_tmp70=_cycalloc(sizeof(*_tmp70));({void*_tmpFE=Cyc_Absyn_access_eff(param_rgn);_tmp70->hd=_tmpFE;}),({
struct Cyc_List_List*_tmpFD=({struct Cyc_List_List*_tmp6F=_cycalloc(sizeof(*_tmp6F));_tmp6F->hd=(void*)_check_null((fd->i).effect),_tmp6F->tl=0;_tmp6F;});_tmp70->tl=_tmpFD;});_tmp70;}));
# 526
_tmp71->capability=_tmpFF;}),_tmp71->curr_rgn=param_rgn,
# 529
(_tmp71->flags).in_new=Cyc_Tcenv_NoneNew,(_tmp71->flags).in_notreadctxt=0,(_tmp71->flags).in_lhs=0,(_tmp71->flags).abstract_ok=0,(_tmp71->flags).in_stmt_exp=0;_tmp71;});}}
# 350
struct Cyc_Tcenv_Fenv*Cyc_Tcenv_nested_fenv(unsigned loc,struct Cyc_Tcenv_Fenv*old_fenv,struct Cyc_Absyn_Fndecl*fd){
# 535
struct Cyc_Tcenv_SharedFenv*_tmp7B;struct Cyc_List_List*_tmp7A;struct Cyc_RgnOrder_RgnPO*_tmp79;_LL1: _tmp79=old_fenv->region_order;_tmp7A=old_fenv->type_vars;_tmp7B=old_fenv->shared;_LL2: {struct Cyc_RgnOrder_RgnPO*rgn_po=_tmp79;struct Cyc_List_List*type_vars=_tmp7A;struct Cyc_Tcenv_SharedFenv*shared=_tmp7B;
# 537
struct Cyc_Absyn_Tvar*rgn0=({struct Cyc_Absyn_Tvar*_tmp8E=_cycalloc(sizeof(*_tmp8E));
({struct _fat_ptr*_tmp104=({struct _fat_ptr*_tmp8D=_cycalloc(sizeof(*_tmp8D));({struct _fat_ptr _tmp103=(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp8C=({struct Cyc_String_pa_PrintArg_struct _tmp9E;_tmp9E.tag=0U,_tmp9E.f1=(struct _fat_ptr)((struct _fat_ptr)*(*fd->name).f2);_tmp9E;});void*_tmp8A[1U];_tmp8A[0]=& _tmp8C;({struct _fat_ptr _tmp102=({const char*_tmp8B="`%s";_tag_fat(_tmp8B,sizeof(char),4U);});Cyc_aprintf(_tmp102,_tag_fat(_tmp8A,sizeof(void*),1U));});});*_tmp8D=_tmp103;});_tmp8D;});_tmp8E->name=_tmp104;}),({
int _tmp101=Cyc_Tcutil_new_tvar_id();_tmp8E->identity=_tmp101;}),_tmp8E->kind=(void*)& Cyc_Tcenv_rgn_kb;_tmp8E;});
{struct Cyc_List_List*tvars=(fd->i).tvars;for(0;tvars != 0;tvars=tvars->tl){
struct Cyc_Absyn_Kind*_stmttmp6=Cyc_Tcutil_tvar_kind((struct Cyc_Absyn_Tvar*)tvars->hd,& Cyc_Tcutil_bk);enum Cyc_Absyn_AliasQual _tmp7D;enum Cyc_Absyn_KindQual _tmp7C;_LL4: _tmp7C=_stmttmp6->kind;_tmp7D=_stmttmp6->aliasqual;_LL5: {enum Cyc_Absyn_KindQual k=_tmp7C;enum Cyc_Absyn_AliasQual a=_tmp7D;
if((int)k == (int)3U){
if((int)a == (int)0U)
rgn_po=Cyc_RgnOrder_add_unordered(rgn_po,(struct Cyc_Absyn_Tvar*)tvars->hd);else{
# 546
({void*_tmp7E=0U;({int(*_tmp106)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp80)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp80;});struct _fat_ptr _tmp105=({const char*_tmp7F="non-intuitionistic tvar in nested_fenv";_tag_fat(_tmp7F,sizeof(char),39U);});_tmp106(_tmp105,_tag_fat(_tmp7E,sizeof(void*),0U));});});}}}}}
# 548
rgn_po=Cyc_RgnOrder_add_youngest(rgn_po,rgn0,0);
{struct Cyc_List_List*po2=(fd->i).rgn_po;for(0;po2 != 0;po2=po2->tl){
rgn_po=Cyc_RgnOrder_add_outlives_constraint(rgn_po,(*((struct _tuple14*)po2->hd)).f1,(*((struct _tuple14*)po2->hd)).f2,loc);}}{
struct Cyc_List_List*tvs=({struct Cyc_List_List*_tmp89=_cycalloc(sizeof(*_tmp89));_tmp89->hd=rgn0,({struct Cyc_List_List*_tmp107=Cyc_List_append((fd->i).tvars,type_vars);_tmp89->tl=_tmp107;});_tmp89;});
Cyc_Tcutil_check_unique_tvars(loc,tvs);{
void*param_rgn=(void*)({struct Cyc_Absyn_VarType_Absyn_Type_struct*_tmp88=_cycalloc(sizeof(*_tmp88));_tmp88->tag=2U,_tmp88->f1=rgn0;_tmp88;});
{struct Cyc_List_List*vds=(struct Cyc_List_List*)((struct Cyc_Core_Opt*)_check_null(fd->param_vardecls))->v;for(0;vds != 0;vds=vds->tl){
((struct Cyc_Absyn_Vardecl*)vds->hd)->rgn=param_rgn;}}
if((fd->i).cyc_varargs != 0){
struct Cyc_Absyn_VarargInfo _stmttmp7=*((struct Cyc_Absyn_VarargInfo*)_check_null((fd->i).cyc_varargs));int _tmp84;void*_tmp83;struct Cyc_Absyn_Tqual _tmp82;struct _fat_ptr*_tmp81;_LL7: _tmp81=_stmttmp7.name;_tmp82=_stmttmp7.tq;_tmp83=_stmttmp7.type;_tmp84=_stmttmp7.inject;_LL8: {struct _fat_ptr*n=_tmp81;struct Cyc_Absyn_Tqual tq=_tmp82;void*t=_tmp83;int i=_tmp84;
# 559
struct Cyc_List_List*vds=(struct Cyc_List_List*)((struct Cyc_Core_Opt*)_check_null(fd->param_vardecls))->v;for(0;vds != 0;vds=vds->tl){
struct Cyc_Absyn_Vardecl*vd=(struct Cyc_Absyn_Vardecl*)vds->hd;
if(Cyc_strptrcmp((*vd->name).f2,(struct _fat_ptr*)_check_null(n))== 0){
({void*_tmp108=Cyc_Absyn_fatptr_type(t,param_rgn,tq,Cyc_Absyn_false_type);vd->type=_tmp108;});
break;}}}}
# 556
return({struct Cyc_Tcenv_Fenv*_tmp87=_cycalloc(sizeof(*_tmp87));
# 568
({struct Cyc_Tcenv_SharedFenv*_tmp10C=Cyc_Tcenv_new_shared_fenv((fd->i).ret_type);_tmp87->shared=_tmp10C;}),_tmp87->type_vars=tvs,_tmp87->region_order=rgn_po,_tmp87->ctrl_env=0,({
# 572
void*_tmp10B=Cyc_Absyn_join_eff(({struct Cyc_List_List*_tmp86=_cycalloc(sizeof(*_tmp86));({void*_tmp10A=Cyc_Absyn_access_eff(param_rgn);_tmp86->hd=_tmp10A;}),({
struct Cyc_List_List*_tmp109=({struct Cyc_List_List*_tmp85=_cycalloc(sizeof(*_tmp85));_tmp85->hd=(void*)_check_null((fd->i).effect),_tmp85->tl=0;_tmp85;});_tmp86->tl=_tmp109;});_tmp86;}));
# 572
_tmp87->capability=_tmp10B;}),_tmp87->curr_rgn=param_rgn,
# 575
(_tmp87->flags).in_new=Cyc_Tcenv_NoneNew,(_tmp87->flags).in_notreadctxt=0,(_tmp87->flags).in_lhs=0,(_tmp87->flags).abstract_ok=0,(_tmp87->flags).in_stmt_exp=0;_tmp87;});}}}}
# 350
struct Cyc_Tcenv_Fenv*Cyc_Tcenv_bogus_fenv(void*ret_type,struct Cyc_List_List*args){
# 581
struct Cyc_Absyn_Tvar*rgn0=({struct Cyc_Absyn_Tvar*_tmp95=_cycalloc(sizeof(*_tmp95));({struct _fat_ptr*_tmp10F=({struct _fat_ptr*_tmp94=_cycalloc(sizeof(*_tmp94));({struct _fat_ptr _tmp10E=({const char*_tmp93="bogus";_tag_fat(_tmp93,sizeof(char),6U);});*_tmp94=_tmp10E;});_tmp94;});_tmp95->name=_tmp10F;}),({int _tmp10D=Cyc_Tcutil_new_tvar_id();_tmp95->identity=_tmp10D;}),_tmp95->kind=(void*)& Cyc_Tcenv_rgn_kb;_tmp95;});
struct Cyc_List_List*tvs=({struct Cyc_List_List*_tmp92=_cycalloc(sizeof(*_tmp92));_tmp92->hd=rgn0,_tmp92->tl=0;_tmp92;});
struct Cyc_RgnOrder_RgnPO*rgn_po=({void*_tmp110=Cyc_Absyn_join_eff(0);Cyc_RgnOrder_initial_fn_po(0,0,_tmp110,rgn0,0U);});
void*param_rgn=(void*)({struct Cyc_Absyn_VarType_Absyn_Type_struct*_tmp91=_cycalloc(sizeof(*_tmp91));_tmp91->tag=2U,_tmp91->f1=rgn0;_tmp91;});
return({struct Cyc_Tcenv_Fenv*_tmp90=_cycalloc(sizeof(*_tmp90));
({struct Cyc_Tcenv_SharedFenv*_tmp113=Cyc_Tcenv_new_shared_fenv(ret_type);_tmp90->shared=_tmp113;}),_tmp90->type_vars=tvs,_tmp90->region_order=rgn_po,_tmp90->ctrl_env=0,({
# 590
void*_tmp112=Cyc_Absyn_join_eff(({struct Cyc_List_List*_tmp8F=_cycalloc(sizeof(*_tmp8F));({void*_tmp111=Cyc_Absyn_access_eff(param_rgn);_tmp8F->hd=_tmp111;}),_tmp8F->tl=0;_tmp8F;}));_tmp90->capability=_tmp112;}),_tmp90->curr_rgn=param_rgn,
# 592
(_tmp90->flags).in_new=Cyc_Tcenv_NoneNew,(_tmp90->flags).in_notreadctxt=0,(_tmp90->flags).in_lhs=0,(_tmp90->flags).abstract_ok=0,(_tmp90->flags).in_stmt_exp=0;_tmp90;});}
# 350
int Cyc_Tcenv_rgn_outlives_rgn(struct Cyc_Tcenv_Tenv*te,void*rgn1,void*rgn2){
# 599
return({struct Cyc_RgnOrder_RgnPO*_tmp115=Cyc_Tcenv_curr_rgnpo(te);void*_tmp114=rgn1;Cyc_RgnOrder_rgn_outlives_rgn(_tmp115,_tmp114,rgn2);});}
