#include <setjmp.h>
/* This is a C header file to be used by the output of the Cyclone to
   C translator.  The corresponding definitions are in file lib/runtime_*.c */
#ifndef _CYC_INCLUDE_H_
#define _CYC_INCLUDE_H_

/* Need one of these per thread (see runtime_stack.c). The runtime maintains 
   a stack that contains either _handler_cons structs or _RegionHandle structs.
   The tag is 0 for a handler_cons and 1 for a region handle.  */
struct _RuntimeStack {
  int tag; 
  struct _RuntimeStack *next;
  void (*cleanup)(struct _RuntimeStack *frame);
};

#ifndef offsetof
/* should be size_t, but int is fine. */
#define offsetof(t,n) ((int)(&(((t *)0)->n)))
#endif

/* Fat pointers */
struct _fat_ptr {
  unsigned char *curr; 
  unsigned char *base; 
  unsigned char *last_plus_one; 
};  

/* Discriminated Unions */
struct _xtunion_struct { char *tag; };

/* Regions */
struct _RegionPage
#ifdef CYC_REGION_PROFILE
{ unsigned total_bytes;
  unsigned free_bytes;
  /* MWH: wish we didn't have to include the stuff below ... */
  struct _RegionPage *next;
  char data[1];
}
#endif
; // abstract -- defined in runtime_memory.c
struct _RegionHandle {
  struct _RuntimeStack s;
  struct _RegionPage *curr;
  char               *offset;
  char               *last_plus_one;
  struct _DynRegionHandle *sub_regions;
#ifdef CYC_REGION_PROFILE
  const char         *name;
#else
  unsigned used_bytes;
  unsigned wasted_bytes;
#endif
};
struct _DynRegionFrame {
  struct _RuntimeStack s;
  struct _DynRegionHandle *x;
};
// A dynamic region is just a region handle.  The wrapper struct is for type
// abstraction.
struct Cyc_Core_DynamicRegion {
  struct _RegionHandle h;
};

struct _RegionHandle _new_region(const char*);
void* _region_malloc(struct _RegionHandle*, unsigned);
void* _region_calloc(struct _RegionHandle*, unsigned t, unsigned n);
void   _free_region(struct _RegionHandle*);
struct _RegionHandle*_open_dynregion(struct _DynRegionFrame*,struct _DynRegionHandle*);
void   _pop_dynregion();

/* Exceptions */
struct _handler_cons {
  struct _RuntimeStack s;
  jmp_buf handler;
};
void _push_handler(struct _handler_cons *);
void _push_region(struct _RegionHandle *);
void _npop_handler(int);
void _pop_handler();
void _pop_region();

#ifndef _throw
void* _throw_null_fn(const char*,unsigned);
void* _throw_arraybounds_fn(const char*,unsigned);
void* _throw_badalloc_fn(const char*,unsigned);
void* _throw_match_fn(const char*,unsigned);
void* _throw_fn(void*,const char*,unsigned);
void* _rethrow(void*);
#define _throw_null() (_throw_null_fn(__FILE__,__LINE__))
#define _throw_arraybounds() (_throw_arraybounds_fn(__FILE__,__LINE__))
#define _throw_badalloc() (_throw_badalloc_fn(__FILE__,__LINE__))
#define _throw_match() (_throw_match_fn(__FILE__,__LINE__))
#define _throw(e) (_throw_fn((e),__FILE__,__LINE__))
#endif

struct _xtunion_struct* Cyc_Core_get_exn_thrown();
/* Built-in Exceptions */
struct Cyc_Null_Exception_exn_struct { char *tag; };
struct Cyc_Array_bounds_exn_struct { char *tag; };
struct Cyc_Match_Exception_exn_struct { char *tag; };
struct Cyc_Bad_alloc_exn_struct { char *tag; };
extern char Cyc_Null_Exception[];
extern char Cyc_Array_bounds[];
extern char Cyc_Match_Exception[];
extern char Cyc_Bad_alloc[];

/* Built-in Run-time Checks and company */
#ifdef CYC_ANSI_OUTPUT
#define _INLINE  
#else
#define _INLINE inline
#endif

#ifdef NO_CYC_NULL_CHECKS
#define _check_null(ptr) (ptr)
#else
#define _check_null(ptr) \
  ({ void*_cks_null = (void*)(ptr); \
     if (!_cks_null) _throw_null(); \
     _cks_null; })
#endif

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index)\
   (((char*)ptr) + (elt_sz)*(index))
#ifdef NO_CYC_NULL_CHECKS
#define _check_known_subscript_null _check_known_subscript_notnull
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr);\
  int _index = (index);\
  if (!_cks_ptr) _throw_null(); \
  _cks_ptr + (elt_sz)*_index; })
#endif
#define _zero_arr_plus_fn(orig_x,orig_sz,orig_i,f,l) ((orig_x)+(orig_i))
#define _zero_arr_plus_char_fn _zero_arr_plus_fn
#define _zero_arr_plus_short_fn _zero_arr_plus_fn
#define _zero_arr_plus_int_fn _zero_arr_plus_fn
#define _zero_arr_plus_float_fn _zero_arr_plus_fn
#define _zero_arr_plus_double_fn _zero_arr_plus_fn
#define _zero_arr_plus_longdouble_fn _zero_arr_plus_fn
#define _zero_arr_plus_voidstar_fn _zero_arr_plus_fn
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (!_cks_ptr) _throw_null(); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })

/* _zero_arr_plus_*_fn(x,sz,i,filename,lineno) adds i to zero-terminated ptr
   x that has at least sz elements */
char* _zero_arr_plus_char_fn(char*,unsigned,int,const char*,unsigned);
short* _zero_arr_plus_short_fn(short*,unsigned,int,const char*,unsigned);
int* _zero_arr_plus_int_fn(int*,unsigned,int,const char*,unsigned);
float* _zero_arr_plus_float_fn(float*,unsigned,int,const char*,unsigned);
double* _zero_arr_plus_double_fn(double*,unsigned,int,const char*,unsigned);
long double* _zero_arr_plus_longdouble_fn(long double*,unsigned,int,const char*, unsigned);
void** _zero_arr_plus_voidstar_fn(void**,unsigned,int,const char*,unsigned);
#endif

/* _get_zero_arr_size_*(x,sz) returns the number of elements in a
   zero-terminated array that is NULL or has at least sz elements */
int _get_zero_arr_size_char(const char*,unsigned);
int _get_zero_arr_size_short(const short*,unsigned);
int _get_zero_arr_size_int(const int*,unsigned);
int _get_zero_arr_size_float(const float*,unsigned);
int _get_zero_arr_size_double(const double*,unsigned);
int _get_zero_arr_size_longdouble(const long double*,unsigned);
int _get_zero_arr_size_voidstar(const void**,unsigned);

/* _zero_arr_inplace_plus_*_fn(x,i,filename,lineno) sets
   zero-terminated pointer *x to *x + i */
char* _zero_arr_inplace_plus_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_short_fn(short**,int,const char*,unsigned);
int* _zero_arr_inplace_plus_int(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_longdouble_fn(long double**,int,const char*,unsigned);
void** _zero_arr_inplace_plus_voidstar_fn(void***,int,const char*,unsigned);
/* like the previous functions, but does post-addition (as in e++) */
char* _zero_arr_inplace_plus_post_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_post_short_fn(short**x,int,const char*,unsigned);
int* _zero_arr_inplace_plus_post_int_fn(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_post_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_post_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_post_longdouble_fn(long double**,int,const char *,unsigned);
void** _zero_arr_inplace_plus_post_voidstar_fn(void***,int,const char*,unsigned);
#define _zero_arr_plus_char(x,s,i) \
  (_zero_arr_plus_char_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_short(x,s,i) \
  (_zero_arr_plus_short_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_int(x,s,i) \
  (_zero_arr_plus_int_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_float(x,s,i) \
  (_zero_arr_plus_float_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_double(x,s,i) \
  (_zero_arr_plus_double_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_longdouble(x,s,i) \
  (_zero_arr_plus_longdouble_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_voidstar(x,s,i) \
  (_zero_arr_plus_voidstar_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_inplace_plus_char(x,i) \
  _zero_arr_inplace_plus_char_fn((char **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_short(x,i) \
  _zero_arr_inplace_plus_short_fn((short **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_int(x,i) \
  _zero_arr_inplace_plus_int_fn((int **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_float(x,i) \
  _zero_arr_inplace_plus_float_fn((float **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_double(x,i) \
  _zero_arr_inplace_plus_double_fn((double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_longdouble(x,i) \
  _zero_arr_inplace_plus_longdouble_fn((long double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_voidstar(x,i) \
  _zero_arr_inplace_plus_voidstar_fn((void ***)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_char(x,i) \
  _zero_arr_inplace_plus_post_char_fn((char **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_short(x,i) \
  _zero_arr_inplace_plus_post_short_fn((short **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_int(x,i) \
  _zero_arr_inplace_plus_post_int_fn((int **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_float(x,i) \
  _zero_arr_inplace_plus_post_float_fn((float **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_double(x,i) \
  _zero_arr_inplace_plus_post_double_fn((double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_longdouble(x,i) \
  _zero_arr_inplace_plus_post_longdouble_fn((long double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_voidstar(x,i) \
  _zero_arr_inplace_plus_post_voidstar_fn((void***)(x),(i),__FILE__,__LINE__)

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_fat_subscript(arr,elt_sz,index) ((arr).curr + (elt_sz) * (index))
#define _untag_fat_ptr(arr,elt_sz,num_elts) ((arr).curr)
#else
#define _check_fat_subscript(arr,elt_sz,index) ({ \
  struct _fat_ptr _cus_arr = (arr); \
  unsigned char *_cus_ans = _cus_arr.curr + (elt_sz) * (index); \
  /* JGM: not needed! if (!_cus_arr.base) _throw_null();*/ \
  if (_cus_ans < _cus_arr.base || _cus_ans >= _cus_arr.last_plus_one) \
    _throw_arraybounds(); \
  _cus_ans; })
#define _untag_fat_ptr(arr,elt_sz,num_elts) ({ \
  struct _fat_ptr _arr = (arr); \
  unsigned char *_curr = _arr.curr; \
  if ((_curr < _arr.base || _curr + (elt_sz) * (num_elts) > _arr.last_plus_one) &&\
      _curr != (unsigned char *)0) \
    _throw_arraybounds(); \
  _curr; })
#endif

#define _tag_fat(tcurr,elt_sz,num_elts) ({ \
  struct _fat_ptr _ans; \
  unsigned _num_elts = (num_elts);\
  _ans.base = _ans.curr = (void*)(tcurr); \
  /* JGM: if we're tagging NULL, ignore num_elts */ \
  _ans.last_plus_one = _ans.base ? (_ans.base + (elt_sz) * _num_elts) : 0; \
  _ans; })

#define _get_fat_size(arr,elt_sz) \
  ({struct _fat_ptr _arr = (arr); \
    unsigned char *_arr_curr=_arr.curr; \
    unsigned char *_arr_last=_arr.last_plus_one; \
    (_arr_curr < _arr.base || _arr_curr >= _arr_last) ? 0 : \
    ((_arr_last - _arr_curr) / (elt_sz));})

#define _fat_ptr_plus(arr,elt_sz,change) ({ \
  struct _fat_ptr _ans = (arr); \
  int _change = (change);\
  _ans.curr += (elt_sz) * _change;\
  _ans; })
#define _fat_ptr_inplace_plus(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  _arr_ptr->curr += (elt_sz) * (change);\
  *_arr_ptr; })
#define _fat_ptr_inplace_plus_post(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  struct _fat_ptr _ans = *_arr_ptr; \
  _arr_ptr->curr += (elt_sz) * (change);\
  _ans; })

//Not a macro since initialization order matters. Defined in runtime_zeroterm.c.
struct _fat_ptr _fat_ptr_decrease_size(struct _fat_ptr,unsigned sz,unsigned numelts);

/* Allocation */
void* GC_malloc(int);
void* GC_malloc_atomic(int);
void* GC_calloc(unsigned,unsigned);
void* GC_calloc_atomic(unsigned,unsigned);
// bound the allocation size to be < MAX_ALLOC_SIZE. See macros below for usage.
#define MAX_MALLOC_SIZE (1 << 28)
void* _bounded_GC_malloc(int,const char*,int);
void* _bounded_GC_malloc_atomic(int,const char*,int);
void* _bounded_GC_calloc(unsigned,unsigned,const char*,int);
void* _bounded_GC_calloc_atomic(unsigned,unsigned,const char*,int);
/* these macros are overridden below ifdef CYC_REGION_PROFILE */
#ifndef CYC_REGION_PROFILE
#define _cycalloc(n) _bounded_GC_malloc(n,__FILE__,__LINE__)
#define _cycalloc_atomic(n) _bounded_GC_malloc_atomic(n,__FILE__,__LINE__)
#define _cyccalloc(n,s) _bounded_GC_calloc(n,s,__FILE__,__LINE__)
#define _cyccalloc_atomic(n,s) _bounded_GC_calloc_atomic(n,s,__FILE__,__LINE__)
#endif

static _INLINE unsigned int _check_times(unsigned x, unsigned y) {
  unsigned long long whole_ans = 
    ((unsigned long long) x)*((unsigned long long)y);
  unsigned word_ans = (unsigned)whole_ans;
  if(word_ans < whole_ans || word_ans > MAX_MALLOC_SIZE)
    _throw_badalloc();
  return word_ans;
}

#define _CYC_MAX_REGION_CONST 2
#define _CYC_MIN_ALIGNMENT (sizeof(double))

#ifdef CYC_REGION_PROFILE
extern int rgn_total_bytes;
#endif

static _INLINE void *_fast_region_malloc(struct _RegionHandle *r, unsigned orig_s) {  
  if (r > (struct _RegionHandle *)_CYC_MAX_REGION_CONST && r->curr != 0) { 
#ifdef CYC_NOALIGN
    unsigned s =  orig_s;
#else
    unsigned s =  (orig_s + _CYC_MIN_ALIGNMENT - 1) & (~(_CYC_MIN_ALIGNMENT -1)); 
#endif
    char *result; 
    result = r->offset; 
    if (s <= (r->last_plus_one - result)) {
      r->offset = result + s; 
#ifdef CYC_REGION_PROFILE
    r->curr->free_bytes = r->curr->free_bytes - s;
    rgn_total_bytes += s;
#endif
      return result;
    }
  } 
  return _region_malloc(r,orig_s); 
}

#ifdef CYC_REGION_PROFILE
/* see macros below for usage. defined in runtime_memory.c */
void* _profile_GC_malloc(int,const char*,const char*,int);
void* _profile_GC_malloc_atomic(int,const char*,const char*,int);
void* _profile_GC_calloc(unsigned,unsigned,const char*,const char*,int);
void* _profile_GC_calloc_atomic(unsigned,unsigned,const char*,const char*,int);
void* _profile_region_malloc(struct _RegionHandle*,unsigned,const char*,const char*,int);
void* _profile_region_calloc(struct _RegionHandle*,unsigned,unsigned,const char *,const char*,int);
struct _RegionHandle _profile_new_region(const char*,const char*,const char*,int);
void _profile_free_region(struct _RegionHandle*,const char*,const char*,int);
#ifndef RUNTIME_CYC
#define _new_region(n) _profile_new_region(n,__FILE__,__FUNCTION__,__LINE__)
#define _free_region(r) _profile_free_region(r,__FILE__,__FUNCTION__,__LINE__)
#define _region_malloc(rh,n) _profile_region_malloc(rh,n,__FILE__,__FUNCTION__,__LINE__)
#define _region_calloc(rh,n,t) _profile_region_calloc(rh,n,t,__FILE__,__FUNCTION__,__LINE__)
#  endif
#define _cycalloc(n) _profile_GC_malloc(n,__FILE__,__FUNCTION__,__LINE__)
#define _cycalloc_atomic(n) _profile_GC_malloc_atomic(n,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc(n,s) _profile_GC_calloc(n,s,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc_atomic(n,s) _profile_GC_calloc_atomic(n,s,__FILE__,__FUNCTION__,__LINE__)
#endif
#endif
 struct Cyc_Core_Opt{void*v;};struct _tuple0{void*f1;void*f2;};
# 108 "core.h"
void*Cyc_Core_fst(struct _tuple0*);
# 111
void*Cyc_Core_snd(struct _tuple0*);extern char Cyc_Core_Invalid_argument[17U];struct Cyc_Core_Invalid_argument_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Failure[8U];struct Cyc_Core_Failure_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Impossible[11U];struct Cyc_Core_Impossible_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Not_found[10U];struct Cyc_Core_Not_found_exn_struct{char*tag;};extern char Cyc_Core_Unreachable[12U];struct Cyc_Core_Unreachable_exn_struct{char*tag;struct _fat_ptr f1;};
# 168
extern struct _RegionHandle*Cyc_Core_unique_region;struct Cyc_Core_DynamicRegion;struct Cyc_Core_NewDynamicRegion{struct Cyc_Core_DynamicRegion*key;};struct Cyc_Core_ThinRes{void*arr;unsigned nelts;};struct Cyc___cycFILE;struct Cyc_String_pa_PrintArg_struct{int tag;struct _fat_ptr f1;};struct Cyc_Int_pa_PrintArg_struct{int tag;unsigned long f1;};struct Cyc_Double_pa_PrintArg_struct{int tag;double f1;};struct Cyc_LongDouble_pa_PrintArg_struct{int tag;long double f1;};struct Cyc_ShortPtr_pa_PrintArg_struct{int tag;short*f1;};struct Cyc_IntPtr_pa_PrintArg_struct{int tag;unsigned long*f1;};
# 73 "cycboot.h"
struct _fat_ptr Cyc_aprintf(struct _fat_ptr,struct _fat_ptr);struct Cyc_ShortPtr_sa_ScanfArg_struct{int tag;short*f1;};struct Cyc_UShortPtr_sa_ScanfArg_struct{int tag;unsigned short*f1;};struct Cyc_IntPtr_sa_ScanfArg_struct{int tag;int*f1;};struct Cyc_UIntPtr_sa_ScanfArg_struct{int tag;unsigned*f1;};struct Cyc_StringPtr_sa_ScanfArg_struct{int tag;struct _fat_ptr f1;};struct Cyc_DoublePtr_sa_ScanfArg_struct{int tag;double*f1;};struct Cyc_FloatPtr_sa_ScanfArg_struct{int tag;float*f1;};struct Cyc_CharPtr_sa_ScanfArg_struct{int tag;struct _fat_ptr f1;};extern char Cyc_FileCloseError[15U];struct Cyc_FileCloseError_exn_struct{char*tag;};extern char Cyc_FileOpenError[14U];struct Cyc_FileOpenError_exn_struct{char*tag;struct _fat_ptr f1;};struct Cyc_List_List{void*hd;struct Cyc_List_List*tl;};
# 54 "list.h"
struct Cyc_List_List*Cyc_List_list(struct _fat_ptr);
# 61
int Cyc_List_length(struct Cyc_List_List*x);
# 76
struct Cyc_List_List*Cyc_List_map(void*(*f)(void*),struct Cyc_List_List*x);
# 83
struct Cyc_List_List*Cyc_List_map_c(void*(*f)(void*,void*),void*env,struct Cyc_List_List*x);extern char Cyc_List_List_mismatch[14U];struct Cyc_List_List_mismatch_exn_struct{char*tag;};
# 178
struct Cyc_List_List*Cyc_List_imp_rev(struct Cyc_List_List*x);
# 184
struct Cyc_List_List*Cyc_List_append(struct Cyc_List_List*x,struct Cyc_List_List*y);extern char Cyc_List_Nth[4U];struct Cyc_List_Nth_exn_struct{char*tag;};
# 251
int Cyc_List_forall(int(*pred)(void*),struct Cyc_List_List*x);struct _tuple1{struct Cyc_List_List*f1;struct Cyc_List_List*f2;};
# 294
struct _tuple1 Cyc_List_split(struct Cyc_List_List*x);
# 391
struct Cyc_List_List*Cyc_List_filter(int(*f)(void*),struct Cyc_List_List*x);struct Cyc_PP_Ppstate;struct Cyc_PP_Out;struct Cyc_PP_Doc;struct Cyc_Position_Error;struct Cyc_Relations_Reln;struct _union_Nmspace_Rel_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_Abs_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_C_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_Loc_n{int tag;int val;};union Cyc_Absyn_Nmspace{struct _union_Nmspace_Rel_n Rel_n;struct _union_Nmspace_Abs_n Abs_n;struct _union_Nmspace_C_n C_n;struct _union_Nmspace_Loc_n Loc_n;};
# 95 "absyn.h"
extern union Cyc_Absyn_Nmspace Cyc_Absyn_Loc_n;struct _tuple2{union Cyc_Absyn_Nmspace f1;struct _fat_ptr*f2;};
# 161
enum Cyc_Absyn_Scope{Cyc_Absyn_Static =0U,Cyc_Absyn_Abstract =1U,Cyc_Absyn_Public =2U,Cyc_Absyn_Extern =3U,Cyc_Absyn_ExternC =4U,Cyc_Absyn_Register =5U};struct Cyc_Absyn_Tqual{int print_const: 1;int q_volatile: 1;int q_restrict: 1;int real_const: 1;unsigned loc;};
# 182
enum Cyc_Absyn_Size_of{Cyc_Absyn_Char_sz =0U,Cyc_Absyn_Short_sz =1U,Cyc_Absyn_Int_sz =2U,Cyc_Absyn_Long_sz =3U,Cyc_Absyn_LongLong_sz =4U};
enum Cyc_Absyn_Sign{Cyc_Absyn_Signed =0U,Cyc_Absyn_Unsigned =1U,Cyc_Absyn_None =2U};
enum Cyc_Absyn_AggrKind{Cyc_Absyn_StructA =0U,Cyc_Absyn_UnionA =1U};
# 187
enum Cyc_Absyn_AliasQual{Cyc_Absyn_Aliasable =0U,Cyc_Absyn_Unique =1U,Cyc_Absyn_Top =2U};
# 192
enum Cyc_Absyn_KindQual{Cyc_Absyn_AnyKind =0U,Cyc_Absyn_MemKind =1U,Cyc_Absyn_BoxKind =2U,Cyc_Absyn_RgnKind =3U,Cyc_Absyn_EffKind =4U,Cyc_Absyn_IntKind =5U,Cyc_Absyn_BoolKind =6U,Cyc_Absyn_PtrBndKind =7U};struct Cyc_Absyn_Kind{enum Cyc_Absyn_KindQual kind;enum Cyc_Absyn_AliasQual aliasqual;};struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct{int tag;struct Cyc_Absyn_Kind*f1;};struct Cyc_Absyn_Unknown_kb_Absyn_KindBound_struct{int tag;struct Cyc_Core_Opt*f1;};struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct{int tag;struct Cyc_Core_Opt*f1;struct Cyc_Absyn_Kind*f2;};struct Cyc_Absyn_Tvar{struct _fat_ptr*name;int identity;void*kind;};struct Cyc_Absyn_PtrLoc{unsigned ptr_loc;unsigned rgn_loc;unsigned zt_loc;};struct Cyc_Absyn_PtrAtts{void*rgn;void*nullable;void*bounds;void*zero_term;struct Cyc_Absyn_PtrLoc*ptrloc;};struct Cyc_Absyn_PtrInfo{void*elt_type;struct Cyc_Absyn_Tqual elt_tq;struct Cyc_Absyn_PtrAtts ptr_atts;};struct Cyc_Absyn_VarargInfo{struct _fat_ptr*name;struct Cyc_Absyn_Tqual tq;void*type;int inject;};struct Cyc_Absyn_Star_Absyn_Cap_struct{int tag;};struct Cyc_Absyn_Bot_Absyn_Cap_struct{int tag;};struct Cyc_Absyn_Nat_Absyn_Cap_struct{int tag;struct Cyc_Absyn_Exp*f1;int f2;};struct Cyc_Absyn_RgnEffect{void*name;void*rgn;void*lock;void*unadopt;void*parent;};struct Cyc_Absyn_FnInfo{struct Cyc_List_List*tvars;void*effect;struct Cyc_Absyn_Tqual ret_tqual;void*ret_type;struct Cyc_List_List*args;int c_varargs;struct Cyc_Absyn_VarargInfo*cyc_varargs;struct Cyc_List_List*rgn_po;struct Cyc_List_List*attributes;struct Cyc_Absyn_Exp*requires_clause;struct Cyc_List_List*requires_relns;struct Cyc_Absyn_Exp*ensures_clause;struct Cyc_List_List*ensures_relns;struct Cyc_List_List*ieffect;struct Cyc_List_List*oeffect;};struct Cyc_Absyn_UnknownDatatypeInfo{struct _tuple2*name;int is_extensible;};struct _union_DatatypeInfo_UnknownDatatype{int tag;struct Cyc_Absyn_UnknownDatatypeInfo val;};struct _union_DatatypeInfo_KnownDatatype{int tag;struct Cyc_Absyn_Datatypedecl**val;};union Cyc_Absyn_DatatypeInfo{struct _union_DatatypeInfo_UnknownDatatype UnknownDatatype;struct _union_DatatypeInfo_KnownDatatype KnownDatatype;};struct Cyc_Absyn_UnknownDatatypeFieldInfo{struct _tuple2*datatype_name;struct _tuple2*field_name;int is_extensible;};struct _union_DatatypeFieldInfo_UnknownDatatypefield{int tag;struct Cyc_Absyn_UnknownDatatypeFieldInfo val;};struct _tuple3{struct Cyc_Absyn_Datatypedecl*f1;struct Cyc_Absyn_Datatypefield*f2;};struct _union_DatatypeFieldInfo_KnownDatatypefield{int tag;struct _tuple3 val;};union Cyc_Absyn_DatatypeFieldInfo{struct _union_DatatypeFieldInfo_UnknownDatatypefield UnknownDatatypefield;struct _union_DatatypeFieldInfo_KnownDatatypefield KnownDatatypefield;};struct _tuple4{enum Cyc_Absyn_AggrKind f1;struct _tuple2*f2;struct Cyc_Core_Opt*f3;};struct _union_AggrInfo_UnknownAggr{int tag;struct _tuple4 val;};struct _union_AggrInfo_KnownAggr{int tag;struct Cyc_Absyn_Aggrdecl**val;};union Cyc_Absyn_AggrInfo{struct _union_AggrInfo_UnknownAggr UnknownAggr;struct _union_AggrInfo_KnownAggr KnownAggr;};
# 331
union Cyc_Absyn_AggrInfo Cyc_Absyn_UnknownAggr(enum Cyc_Absyn_AggrKind,struct _tuple2*,struct Cyc_Core_Opt*);struct Cyc_Absyn_ArrayInfo{void*elt_type;struct Cyc_Absyn_Tqual tq;struct Cyc_Absyn_Exp*num_elts;void*zero_term;unsigned zt_loc;};struct Cyc_Absyn_Aggr_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Aggrdecl*f1;};struct Cyc_Absyn_Enum_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;};struct Cyc_Absyn_Datatype_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;};struct Cyc_Absyn_TypeDecl{void*r;unsigned loc;};struct Cyc_Absyn_VoidCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_IntCon_Absyn_TyCon_struct{int tag;enum Cyc_Absyn_Sign f1;enum Cyc_Absyn_Size_of f2;};struct Cyc_Absyn_FloatCon_Absyn_TyCon_struct{int tag;int f1;};struct Cyc_Absyn_RgnHandleCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_TagCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_HeapCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_UniqueCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_RefCntCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_AccessCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_JoinCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_RgnsCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_TrueCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_FalseCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_ThinCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_FatCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_EnumCon_Absyn_TyCon_struct{int tag;struct _tuple2*f1;struct Cyc_Absyn_Enumdecl*f2;};struct Cyc_Absyn_AnonEnumCon_Absyn_TyCon_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_BuiltinCon_Absyn_TyCon_struct{int tag;struct _fat_ptr f1;struct Cyc_Absyn_Kind*f2;};struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_DatatypeInfo f1;};struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_DatatypeFieldInfo f1;};struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_AggrInfo f1;};struct Cyc_Absyn_AppType_Absyn_Type_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Evar_Absyn_Type_struct{int tag;struct Cyc_Core_Opt*f1;void*f2;int f3;struct Cyc_Core_Opt*f4;};struct Cyc_Absyn_VarType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Tvar*f1;};struct Cyc_Absyn_PointerType_Absyn_Type_struct{int tag;struct Cyc_Absyn_PtrInfo f1;};struct Cyc_Absyn_ArrayType_Absyn_Type_struct{int tag;struct Cyc_Absyn_ArrayInfo f1;};struct Cyc_Absyn_FnType_Absyn_Type_struct{int tag;struct Cyc_Absyn_FnInfo f1;};struct Cyc_Absyn_TupleType_Absyn_Type_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct{int tag;enum Cyc_Absyn_AggrKind f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_TypedefType_Absyn_Type_struct{int tag;struct _tuple2*f1;struct Cyc_List_List*f2;struct Cyc_Absyn_Typedefdecl*f3;void*f4;};struct Cyc_Absyn_ValueofType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct{int tag;struct Cyc_Absyn_TypeDecl*f1;void**f2;};struct Cyc_Absyn_TypeofType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_NoTypes_Absyn_Funcparams_struct{int tag;struct Cyc_List_List*f1;unsigned f2;};struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct{int tag;struct Cyc_List_List*f1;int f2;struct Cyc_Absyn_VarargInfo*f3;void*f4;struct Cyc_List_List*f5;struct Cyc_Absyn_Exp*f6;struct Cyc_Absyn_Exp*f7;struct Cyc_List_List*f8;struct Cyc_List_List*f9;};
# 437 "absyn.h"
enum Cyc_Absyn_Format_Type{Cyc_Absyn_Printf_ft =0U,Cyc_Absyn_Scanf_ft =1U};struct Cyc_Absyn_Regparm_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Stdcall_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Cdecl_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Fastcall_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Noreturn_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Const_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Aligned_att_Absyn_Attribute_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Packed_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Section_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Nocommon_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Shared_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Unused_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Weak_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Dllimport_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Dllexport_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_instrument_function_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Constructor_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Destructor_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_check_memory_usage_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Format_att_Absyn_Attribute_struct{int tag;enum Cyc_Absyn_Format_Type f1;int f2;int f3;};struct Cyc_Absyn_Initializes_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Noliveunique_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Consume_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Pure_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Mode_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Alias_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Always_inline_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_throw_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Non_null_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Deprecated_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Carray_mod_Absyn_Type_modifier_struct{int tag;void*f1;unsigned f2;};struct Cyc_Absyn_ConstArray_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_Absyn_Exp*f1;void*f2;unsigned f3;};struct Cyc_Absyn_Pointer_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_Absyn_PtrAtts f1;struct Cyc_Absyn_Tqual f2;};struct Cyc_Absyn_Function_mod_Absyn_Type_modifier_struct{int tag;void*f1;};struct Cyc_Absyn_TypeParams_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_List_List*f1;unsigned f2;int f3;};struct Cyc_Absyn_Attributes_mod_Absyn_Type_modifier_struct{int tag;unsigned f1;struct Cyc_List_List*f2;};struct _union_Cnst_Null_c{int tag;int val;};struct _tuple5{enum Cyc_Absyn_Sign f1;char f2;};struct _union_Cnst_Char_c{int tag;struct _tuple5 val;};struct _union_Cnst_Wchar_c{int tag;struct _fat_ptr val;};struct _tuple6{enum Cyc_Absyn_Sign f1;short f2;};struct _union_Cnst_Short_c{int tag;struct _tuple6 val;};struct _tuple7{enum Cyc_Absyn_Sign f1;int f2;};struct _union_Cnst_Int_c{int tag;struct _tuple7 val;};struct _tuple8{enum Cyc_Absyn_Sign f1;long long f2;};struct _union_Cnst_LongLong_c{int tag;struct _tuple8 val;};struct _tuple9{struct _fat_ptr f1;int f2;};struct _union_Cnst_Float_c{int tag;struct _tuple9 val;};struct _union_Cnst_String_c{int tag;struct _fat_ptr val;};struct _union_Cnst_Wstring_c{int tag;struct _fat_ptr val;};union Cyc_Absyn_Cnst{struct _union_Cnst_Null_c Null_c;struct _union_Cnst_Char_c Char_c;struct _union_Cnst_Wchar_c Wchar_c;struct _union_Cnst_Short_c Short_c;struct _union_Cnst_Int_c Int_c;struct _union_Cnst_LongLong_c LongLong_c;struct _union_Cnst_Float_c Float_c;struct _union_Cnst_String_c String_c;struct _union_Cnst_Wstring_c Wstring_c;};
# 532
enum Cyc_Absyn_Primop{Cyc_Absyn_Plus =0U,Cyc_Absyn_Times =1U,Cyc_Absyn_Minus =2U,Cyc_Absyn_Div =3U,Cyc_Absyn_Mod =4U,Cyc_Absyn_Eq =5U,Cyc_Absyn_Neq =6U,Cyc_Absyn_Gt =7U,Cyc_Absyn_Lt =8U,Cyc_Absyn_Gte =9U,Cyc_Absyn_Lte =10U,Cyc_Absyn_Not =11U,Cyc_Absyn_Bitnot =12U,Cyc_Absyn_Bitand =13U,Cyc_Absyn_Bitor =14U,Cyc_Absyn_Bitxor =15U,Cyc_Absyn_Bitlshift =16U,Cyc_Absyn_Bitlrshift =17U,Cyc_Absyn_Numelts =18U};
# 539
enum Cyc_Absyn_Incrementor{Cyc_Absyn_PreInc =0U,Cyc_Absyn_PostInc =1U,Cyc_Absyn_PreDec =2U,Cyc_Absyn_PostDec =3U};struct Cyc_Absyn_VarargCallInfo{int num_varargs;struct Cyc_List_List*injectors;struct Cyc_Absyn_VarargInfo*vai;};struct Cyc_Absyn_StructField_Absyn_OffsetofField_struct{int tag;struct _fat_ptr*f1;};struct Cyc_Absyn_TupleIndex_Absyn_OffsetofField_struct{int tag;unsigned f1;};
# 557
enum Cyc_Absyn_Coercion{Cyc_Absyn_Unknown_coercion =0U,Cyc_Absyn_No_coercion =1U,Cyc_Absyn_Null_to_NonNull =2U,Cyc_Absyn_Other_coercion =3U};struct Cyc_Absyn_MallocInfo{int is_calloc;struct Cyc_Absyn_Exp*rgn;void**elt_type;struct Cyc_Absyn_Exp*num_elts;int fat_result;int inline_call;};struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct{int tag;union Cyc_Absyn_Cnst f1;};struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Pragma_e_Absyn_Raw_exp_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct{int tag;enum Cyc_Absyn_Primop f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_AssignOp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Core_Opt*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_Increment_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;enum Cyc_Absyn_Incrementor f2;};struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;struct Cyc_Absyn_VarargCallInfo*f3;int f4;};struct Cyc_Absyn_Throw_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;int f2;};struct Cyc_Absyn_NoInstantiate_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Instantiate_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_Absyn_Exp*f2;int f3;enum Cyc_Absyn_Coercion f4;};struct Cyc_Absyn_Address_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_New_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Sizeoftype_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Sizeofexp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Offsetof_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Deref_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;int f3;int f4;};struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;int f3;int f4;};struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Tuple_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;};struct _tuple10{struct _fat_ptr*f1;struct Cyc_Absyn_Tqual f2;void*f3;};struct Cyc_Absyn_CompoundLit_e_Absyn_Raw_exp_struct{int tag;struct _tuple10*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Array_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Exp*f2;struct Cyc_Absyn_Exp*f3;int f4;};struct Cyc_Absyn_ComprehensionNoinit_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;void*f2;int f3;};struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct{int tag;struct _tuple2*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;struct Cyc_Absyn_Aggrdecl*f4;};struct Cyc_Absyn_AnonStruct_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Datatype_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;struct Cyc_Absyn_Datatypedecl*f2;struct Cyc_Absyn_Datatypefield*f3;};struct Cyc_Absyn_Enum_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_AnonEnum_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_Spawn_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_MallocInfo f1;};struct Cyc_Absyn_Swap_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_UnresolvedMem_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Core_Opt*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_StmtExp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Stmt*f1;};struct Cyc_Absyn_Tagcheck_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;};struct Cyc_Absyn_Valueof_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Asm_e_Absyn_Raw_exp_struct{int tag;int f1;struct _fat_ptr f2;struct Cyc_List_List*f3;struct Cyc_List_List*f4;struct Cyc_List_List*f5;};struct Cyc_Absyn_Extension_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Exp{void*topt;void*r;unsigned loc;void*annot;};struct Cyc_Absyn_Skip_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Exp_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Seq_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Return_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_IfThenElse_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Stmt*f2;struct Cyc_Absyn_Stmt*f3;};struct _tuple11{struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_While_s_Absyn_Raw_stmt_struct{int tag;struct _tuple11 f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Break_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Continue_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Goto_s_Absyn_Raw_stmt_struct{int tag;struct _fat_ptr*f1;};struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _tuple11 f2;struct _tuple11 f3;struct Cyc_Absyn_Stmt*f4;};struct Cyc_Absyn_Switch_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;void*f3;};struct Cyc_Absyn_Fallthru_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_List_List*f1;struct Cyc_Absyn_Switch_clause**f2;};struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Decl*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Label_s_Absyn_Raw_stmt_struct{int tag;struct _fat_ptr*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Do_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct _tuple11 f2;};struct Cyc_Absyn_TryCatch_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct Cyc_List_List*f2;void*f3;};struct Cyc_Absyn_Stmt{void*r;unsigned loc;void*annot;};struct Cyc_Absyn_Wild_p_Absyn_Raw_pat_struct{int tag;};struct Cyc_Absyn_Var_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Pat*f2;};struct Cyc_Absyn_AliasVar_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;};struct Cyc_Absyn_Reference_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Pat*f2;};struct Cyc_Absyn_TagInt_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;};struct Cyc_Absyn_Tuple_p_Absyn_Raw_pat_struct{int tag;struct Cyc_List_List*f1;int f2;};struct Cyc_Absyn_Pointer_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Pat*f1;};struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct{int tag;union Cyc_Absyn_AggrInfo*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;int f4;};struct Cyc_Absyn_Datatype_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;struct Cyc_Absyn_Datatypefield*f2;struct Cyc_List_List*f3;int f4;};struct Cyc_Absyn_Null_p_Absyn_Raw_pat_struct{int tag;};struct Cyc_Absyn_Int_p_Absyn_Raw_pat_struct{int tag;enum Cyc_Absyn_Sign f1;int f2;};struct Cyc_Absyn_Char_p_Absyn_Raw_pat_struct{int tag;char f1;};struct Cyc_Absyn_Float_p_Absyn_Raw_pat_struct{int tag;struct _fat_ptr f1;int f2;};struct Cyc_Absyn_Enum_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_AnonEnum_p_Absyn_Raw_pat_struct{int tag;void*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_UnknownId_p_Absyn_Raw_pat_struct{int tag;struct _tuple2*f1;};struct Cyc_Absyn_UnknownCall_p_Absyn_Raw_pat_struct{int tag;struct _tuple2*f1;struct Cyc_List_List*f2;int f3;};struct Cyc_Absyn_Exp_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Exp*f1;};
# 725 "absyn.h"
extern struct Cyc_Absyn_Wild_p_Absyn_Raw_pat_struct Cyc_Absyn_Wild_p_val;struct Cyc_Absyn_Pat{void*r;void*topt;unsigned loc;};struct Cyc_Absyn_Switch_clause{struct Cyc_Absyn_Pat*pattern;struct Cyc_Core_Opt*pat_vars;struct Cyc_Absyn_Exp*where_clause;struct Cyc_Absyn_Stmt*body;unsigned loc;};struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct{int tag;struct _tuple2*f1;};struct Cyc_Absyn_Global_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Funname_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Fndecl*f1;};struct Cyc_Absyn_Param_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Local_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Pat_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Vardecl{enum Cyc_Absyn_Scope sc;struct _tuple2*name;unsigned varloc;struct Cyc_Absyn_Tqual tq;void*type;struct Cyc_Absyn_Exp*initializer;void*rgn;struct Cyc_List_List*attributes;int escapes;};struct Cyc_Absyn_Fndecl{enum Cyc_Absyn_Scope sc;int is_inline;struct _tuple2*name;struct Cyc_Absyn_Stmt*body;struct Cyc_Absyn_FnInfo i;void*cached_type;struct Cyc_Core_Opt*param_vardecls;struct Cyc_Absyn_Vardecl*fn_vardecl;};struct Cyc_Absyn_Aggrfield{struct _fat_ptr*name;struct Cyc_Absyn_Tqual tq;void*type;struct Cyc_Absyn_Exp*width;struct Cyc_List_List*attributes;struct Cyc_Absyn_Exp*requires_clause;};struct Cyc_Absyn_AggrdeclImpl{struct Cyc_List_List*exist_vars;struct Cyc_List_List*rgn_po;struct Cyc_List_List*fields;int tagged;};struct Cyc_Absyn_Aggrdecl{enum Cyc_Absyn_AggrKind kind;enum Cyc_Absyn_Scope sc;struct _tuple2*name;struct Cyc_List_List*tvs;struct Cyc_Absyn_AggrdeclImpl*impl;struct Cyc_List_List*attributes;int expected_mem_kind;};struct Cyc_Absyn_Datatypefield{struct _tuple2*name;struct Cyc_List_List*typs;unsigned loc;enum Cyc_Absyn_Scope sc;};struct Cyc_Absyn_Datatypedecl{enum Cyc_Absyn_Scope sc;struct _tuple2*name;struct Cyc_List_List*tvs;struct Cyc_Core_Opt*fields;int is_extensible;};struct Cyc_Absyn_Enumfield{struct _tuple2*name;struct Cyc_Absyn_Exp*tag;unsigned loc;};struct Cyc_Absyn_Enumdecl{enum Cyc_Absyn_Scope sc;struct _tuple2*name;struct Cyc_Core_Opt*fields;};struct Cyc_Absyn_Typedefdecl{struct _tuple2*name;struct Cyc_Absyn_Tqual tq;struct Cyc_List_List*tvs;struct Cyc_Core_Opt*kind;void*defn;struct Cyc_List_List*atts;int extern_c;};struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Fn_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Fndecl*f1;};struct Cyc_Absyn_Let_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Pat*f1;struct Cyc_Core_Opt*f2;struct Cyc_Absyn_Exp*f3;void*f4;};struct Cyc_Absyn_Letv_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_Region_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_Aggr_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Aggrdecl*f1;};struct Cyc_Absyn_Datatype_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;};struct Cyc_Absyn_Enum_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;};struct Cyc_Absyn_Typedef_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Typedefdecl*f1;};struct Cyc_Absyn_Namespace_d_Absyn_Raw_decl_struct{int tag;struct _fat_ptr*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Using_d_Absyn_Raw_decl_struct{int tag;struct _tuple2*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_ExternC_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;};struct _tuple12{unsigned f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_ExternCinclude_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;struct _tuple12*f4;};struct Cyc_Absyn_Porton_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Portoff_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Tempeston_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Tempestoff_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Decl{void*r;unsigned loc;};struct Cyc_Absyn_ArrayElement_Absyn_Designator_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_FieldName_Absyn_Designator_struct{int tag;struct _fat_ptr*f1;};extern char Cyc_Absyn_EmptyAnnot[11U];struct Cyc_Absyn_EmptyAnnot_Absyn_AbsynAnnot_struct{char*tag;};
# 916
struct Cyc_Absyn_Tqual Cyc_Absyn_const_tqual(unsigned);
struct Cyc_Absyn_Tqual Cyc_Absyn_empty_tqual(unsigned);
# 933
void*Cyc_Absyn_new_evar(struct Cyc_Core_Opt*k,struct Cyc_Core_Opt*tenv);
# 945
extern void*Cyc_Absyn_heap_rgn_type;extern void*Cyc_Absyn_unique_rgn_type;extern void*Cyc_Absyn_refcnt_rgn_type;
# 949
extern void*Cyc_Absyn_false_type;
# 951
extern void*Cyc_Absyn_void_type;void*Cyc_Absyn_rgn_handle_type(void*);
# 968
void*Cyc_Absyn_exn_type();
# 992
void*Cyc_Absyn_at_type(void*t,void*rgn,struct Cyc_Absyn_Tqual,void*zero_term);
# 1004
void*Cyc_Absyn_array_type(void*elt_type,struct Cyc_Absyn_Tqual tq,struct Cyc_Absyn_Exp*num_elts,void*zero_term,unsigned ztloc);
# 1009
void*Cyc_Absyn_aggr_type(union Cyc_Absyn_AggrInfo,struct Cyc_List_List*args);
# 1012
struct Cyc_Absyn_Exp*Cyc_Absyn_new_exp(void*,unsigned);
# 1022
struct Cyc_Absyn_Exp*Cyc_Absyn_uint_exp(unsigned,unsigned);
# 1029
struct Cyc_Absyn_Exp*Cyc_Absyn_varb_exp(void*,unsigned);
# 1081
struct Cyc_Absyn_Stmt*Cyc_Absyn_new_stmt(void*,unsigned);
struct Cyc_Absyn_Stmt*Cyc_Absyn_skip_stmt(unsigned);
struct Cyc_Absyn_Stmt*Cyc_Absyn_exp_stmt(struct Cyc_Absyn_Exp*,unsigned);
# 1094
struct Cyc_Absyn_Stmt*Cyc_Absyn_decl_stmt(struct Cyc_Absyn_Decl*,struct Cyc_Absyn_Stmt*,unsigned);
# 1103
struct Cyc_Absyn_Pat*Cyc_Absyn_new_pat(void*,unsigned);
# 1108
struct Cyc_Absyn_Decl*Cyc_Absyn_let_decl(struct Cyc_Absyn_Pat*,struct Cyc_Absyn_Exp*,unsigned);
# 1112
struct Cyc_Absyn_Vardecl*Cyc_Absyn_new_vardecl(unsigned varloc,struct _tuple2*,void*,struct Cyc_Absyn_Exp*init);
# 1164
struct _fat_ptr Cyc_Absyn_attribute2string(void*);struct Cyc_Absynpp_Params{int expand_typedefs;int qvar_to_Cids;int add_cyc_prefix;int to_VC;int decls_first;int rewrite_temp_tvars;int print_all_tvars;int print_all_kinds;int print_all_effects;int print_using_stmts;int print_externC_stmts;int print_full_evars;int print_zeroterm;int generate_line_directives;int use_curr_namespace;struct Cyc_List_List*curr_namespace;};
# 62 "absynpp.h"
struct _fat_ptr Cyc_Absynpp_typ2string(void*);
# 27 "unify.h"
void Cyc_Unify_explain_failure();
# 29
int Cyc_Unify_unify(void*,void*);struct _union_RelnOp_RConst{int tag;unsigned val;};struct _union_RelnOp_RVar{int tag;struct Cyc_Absyn_Vardecl*val;};struct _union_RelnOp_RNumelts{int tag;struct Cyc_Absyn_Vardecl*val;};struct _union_RelnOp_RType{int tag;void*val;};struct _union_RelnOp_RParam{int tag;unsigned val;};struct _union_RelnOp_RParamNumelts{int tag;unsigned val;};struct _union_RelnOp_RReturn{int tag;unsigned val;};union Cyc_Relations_RelnOp{struct _union_RelnOp_RConst RConst;struct _union_RelnOp_RVar RVar;struct _union_RelnOp_RNumelts RNumelts;struct _union_RelnOp_RType RType;struct _union_RelnOp_RParam RParam;struct _union_RelnOp_RParamNumelts RParamNumelts;struct _union_RelnOp_RReturn RReturn;};
# 50 "relations-ap.h"
enum Cyc_Relations_Relation{Cyc_Relations_Req =0U,Cyc_Relations_Rneq =1U,Cyc_Relations_Rlte =2U,Cyc_Relations_Rlt =3U};struct Cyc_Relations_Reln{union Cyc_Relations_RelnOp rop1;enum Cyc_Relations_Relation relation;union Cyc_Relations_RelnOp rop2;};struct Cyc_Iter_Iter{void*env;int(*next)(void*env,void*dest);};struct Cyc_Dict_T;struct Cyc_Dict_Dict{int(*rel)(void*,void*);struct _RegionHandle*r;const struct Cyc_Dict_T*t;};extern char Cyc_Dict_Present[8U];struct Cyc_Dict_Present_exn_struct{char*tag;};extern char Cyc_Dict_Absent[7U];struct Cyc_Dict_Absent_exn_struct{char*tag;};struct Cyc_RgnOrder_RgnPO;extern char Cyc_Tcenv_Env_error[10U];struct Cyc_Tcenv_Env_error_exn_struct{char*tag;};struct Cyc_Tcenv_Genv{struct Cyc_Dict_Dict aggrdecls;struct Cyc_Dict_Dict datatypedecls;struct Cyc_Dict_Dict enumdecls;struct Cyc_Dict_Dict typedefs;struct Cyc_Dict_Dict ordinaries;};struct Cyc_Tcenv_Fenv;struct Cyc_Tcenv_Tenv{struct Cyc_List_List*ns;struct Cyc_Tcenv_Genv*ae;struct Cyc_Tcenv_Fenv*le;int allow_valueof: 1;int in_extern_c_include: 1;int in_tempest: 1;int tempest_generalize: 1;};
# 70 "tcenv.h"
struct Cyc_Tcenv_Fenv*Cyc_Tcenv_nested_fenv(unsigned,struct Cyc_Tcenv_Fenv*old_fenv,struct Cyc_Absyn_Fndecl*new_fn);
# 88
enum Cyc_Tcenv_NewStatus{Cyc_Tcenv_NoneNew =0U,Cyc_Tcenv_InNew =1U,Cyc_Tcenv_InNewAggr =2U};
# 96
void*Cyc_Tcenv_return_typ(struct Cyc_Tcenv_Tenv*);
# 98
struct Cyc_List_List*Cyc_Tcenv_lookup_type_vars(struct Cyc_Tcenv_Tenv*);
# 100
struct Cyc_Tcenv_Tenv*Cyc_Tcenv_add_type_vars(unsigned,struct Cyc_Tcenv_Tenv*,struct Cyc_List_List*);
# 102
struct Cyc_Tcenv_Tenv*Cyc_Tcenv_set_fallthru(struct Cyc_Tcenv_Tenv*,struct Cyc_List_List*new_tvs,struct Cyc_List_List*vds,struct Cyc_Absyn_Switch_clause*clause);
# 106
struct Cyc_Tcenv_Tenv*Cyc_Tcenv_clear_fallthru(struct Cyc_Tcenv_Tenv*);
# 124
int Cyc_Tcenv_in_stmt_exp(struct Cyc_Tcenv_Tenv*);struct _tuple13{struct Cyc_Absyn_Switch_clause*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;};
# 127
const struct _tuple13*const Cyc_Tcenv_process_fallthru(struct Cyc_Tcenv_Tenv*,struct Cyc_Absyn_Stmt*,struct Cyc_Absyn_Switch_clause***);
# 130
struct Cyc_Tcenv_Tenv*Cyc_Tcenv_new_block(unsigned,struct Cyc_Tcenv_Tenv*);
struct Cyc_Tcenv_Tenv*Cyc_Tcenv_new_named_block(unsigned,struct Cyc_Tcenv_Tenv*,struct Cyc_Absyn_Tvar*);
struct Cyc_Tcenv_Tenv*Cyc_Tcenv_new_outlives_constraints(struct Cyc_Tcenv_Tenv*,struct Cyc_List_List*,unsigned);
# 137
void*Cyc_Tcenv_curr_rgn(struct Cyc_Tcenv_Tenv*);
# 139
struct Cyc_Tcenv_Tenv*Cyc_Tcenv_add_region(struct Cyc_Tcenv_Tenv*,void*,int opened);
# 148
void Cyc_Tcenv_check_delayed_effects(struct Cyc_Tcenv_Tenv*);
void Cyc_Tcenv_check_delayed_constraints(struct Cyc_Tcenv_Tenv*);
# 30 "tcutil.h"
void*Cyc_Tcutil_impos(struct _fat_ptr,struct _fat_ptr);
void Cyc_Tcutil_terr(unsigned,struct _fat_ptr,struct _fat_ptr);
void Cyc_Tcutil_warn(unsigned,struct _fat_ptr,struct _fat_ptr);
# 36
int Cyc_Tcutil_is_void_type(void*);
# 38
int Cyc_Tcutil_is_any_int_type(void*);
int Cyc_Tcutil_is_any_float_type(void*);
# 107
void*Cyc_Tcutil_compress(void*);
# 113
int Cyc_Tcutil_coerce_arg(struct Cyc_Tcenv_Tenv*te,struct Cyc_Absyn_Exp*,void*,int*alias_coercion);
int Cyc_Tcutil_coerce_assign(struct Cyc_Tcenv_Tenv*te,struct Cyc_Absyn_Exp*,void*);
# 135
extern struct Cyc_Absyn_Kind Cyc_Tcutil_rk;
# 137
extern struct Cyc_Absyn_Kind Cyc_Tcutil_bk;
# 144
extern struct Cyc_Absyn_Kind Cyc_Tcutil_trk;
# 147
extern struct Cyc_Absyn_Kind Cyc_Tcutil_tmk;
# 174
void*Cyc_Tcutil_kind_to_bound(struct Cyc_Absyn_Kind*k);
# 185
void*Cyc_Tcutil_substitute(struct Cyc_List_List*,void*);
# 202
void*Cyc_Tcutil_fndecl2type(struct Cyc_Absyn_Fndecl*);struct _tuple14{struct Cyc_Absyn_Tvar*f1;void*f2;};
# 206
struct _tuple14*Cyc_Tcutil_make_inst_var(struct Cyc_List_List*,struct Cyc_Absyn_Tvar*);
# 241
int Cyc_Tcutil_is_noalias_path(struct Cyc_Absyn_Exp*);
# 246
int Cyc_Tcutil_is_noalias_pointer_or_aggr(void*);
# 258
int Cyc_Tcutil_new_tvar_id();
# 273
int Cyc_Tcutil_is_const_exp(struct Cyc_Absyn_Exp*);
# 280
int Cyc_Tcutil_extract_const_from_typedef(unsigned,int declared_const,void*);
# 284
struct Cyc_List_List*Cyc_Tcutil_transfer_fn_type_atts(void*,struct Cyc_List_List*);
# 294
struct Cyc_List_List*Cyc_Tcutil_filter_nulls(struct Cyc_List_List*);
# 35 "tctyp.h"
void Cyc_Tctyp_check_fndecl_valid_type(unsigned,struct Cyc_Tcenv_Tenv*,struct Cyc_Absyn_Fndecl*);
# 44 "tctyp.h"
void Cyc_Tctyp_check_type(unsigned,struct Cyc_Tcenv_Tenv*,struct Cyc_List_List*bound_tvars,struct Cyc_Absyn_Kind*k,int allow_evars,int allow_abs_aggr,void*);
# 28 "tcexp.h"
void*Cyc_Tcexp_tcExp(struct Cyc_Tcenv_Tenv*,void**,struct Cyc_Absyn_Exp*);
void*Cyc_Tcexp_tcExpInitializer(struct Cyc_Tcenv_Tenv*,void**,struct Cyc_Absyn_Exp*);
void Cyc_Tcexp_tcTest(struct Cyc_Tcenv_Tenv*te,struct Cyc_Absyn_Exp*e,struct _fat_ptr msg_part);struct Cyc_Tcpat_TcPatResult{struct _tuple1*tvars_and_bounds_opt;struct Cyc_List_List*patvars;};
# 53 "tcpat.h"
struct Cyc_Tcpat_TcPatResult Cyc_Tcpat_tcPat(struct Cyc_Tcenv_Tenv*te,struct Cyc_Absyn_Pat*p,void**topt,struct Cyc_Absyn_Exp*pat_var_exp);
# 55
void Cyc_Tcpat_check_pat_regions(struct Cyc_Tcenv_Tenv*te,struct Cyc_Absyn_Pat*p,struct Cyc_List_List*patvars);struct Cyc_Tcpat_WhereTest_Tcpat_PatTest_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Tcpat_EqNull_Tcpat_PatTest_struct{int tag;};struct Cyc_Tcpat_NeqNull_Tcpat_PatTest_struct{int tag;};struct Cyc_Tcpat_EqEnum_Tcpat_PatTest_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Tcpat_EqAnonEnum_Tcpat_PatTest_struct{int tag;void*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Tcpat_EqFloat_Tcpat_PatTest_struct{int tag;struct _fat_ptr f1;int f2;};struct Cyc_Tcpat_EqConst_Tcpat_PatTest_struct{int tag;unsigned f1;};struct Cyc_Tcpat_EqDatatypeTag_Tcpat_PatTest_struct{int tag;int f1;struct Cyc_Absyn_Datatypedecl*f2;struct Cyc_Absyn_Datatypefield*f3;};struct Cyc_Tcpat_EqTaggedUnion_Tcpat_PatTest_struct{int tag;struct _fat_ptr*f1;int f2;};struct Cyc_Tcpat_EqExtensibleDatatype_Tcpat_PatTest_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;struct Cyc_Absyn_Datatypefield*f2;};struct Cyc_Tcpat_Dummy_Tcpat_Access_struct{int tag;};struct Cyc_Tcpat_Deref_Tcpat_Access_struct{int tag;};struct Cyc_Tcpat_TupleField_Tcpat_Access_struct{int tag;unsigned f1;};struct Cyc_Tcpat_DatatypeField_Tcpat_Access_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;struct Cyc_Absyn_Datatypefield*f2;unsigned f3;};struct Cyc_Tcpat_AggrField_Tcpat_Access_struct{int tag;int f1;struct _fat_ptr*f2;};struct _union_PatOrWhere_pattern{int tag;struct Cyc_Absyn_Pat*val;};struct _union_PatOrWhere_where_clause{int tag;struct Cyc_Absyn_Exp*val;};union Cyc_Tcpat_PatOrWhere{struct _union_PatOrWhere_pattern pattern;struct _union_PatOrWhere_where_clause where_clause;};struct Cyc_Tcpat_PathNode{union Cyc_Tcpat_PatOrWhere orig_pat;void*access;};struct Cyc_Tcpat_Rhs{int used;unsigned pat_loc;struct Cyc_Absyn_Stmt*rhs;};struct Cyc_Tcpat_Failure_Tcpat_Decision_struct{int tag;void*f1;};struct Cyc_Tcpat_Success_Tcpat_Decision_struct{int tag;struct Cyc_Tcpat_Rhs*f1;};struct Cyc_Tcpat_SwitchDec_Tcpat_Decision_struct{int tag;struct Cyc_List_List*f1;struct Cyc_List_List*f2;void*f3;};
# 107
void Cyc_Tcpat_check_switch_exhaustive(unsigned,struct Cyc_Tcenv_Tenv*,struct Cyc_List_List*,void**);
# 109
int Cyc_Tcpat_check_let_pat_exhaustive(unsigned,struct Cyc_Tcenv_Tenv*,struct Cyc_Absyn_Pat*p,void**);
# 111
void Cyc_Tcpat_check_catch_overlap(unsigned,struct Cyc_Tcenv_Tenv*,struct Cyc_List_List*,void**);struct Cyc_Set_Set;extern char Cyc_Set_Absent[7U];struct Cyc_Set_Absent_exn_struct{char*tag;};struct Cyc_Hashtable_Table;struct Cyc_JumpAnalysis_Jump_Anal_Result{struct Cyc_Hashtable_Table*pop_tables;struct Cyc_Hashtable_Table*succ_tables;struct Cyc_Hashtable_Table*pat_pop_tables;};
# 41 "cf_flowinfo.h"
int Cyc_CfFlowInfo_anal_error;struct Cyc_CfFlowInfo_VarRoot_CfFlowInfo_Root_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_CfFlowInfo_MallocPt_CfFlowInfo_Root_struct{int tag;struct Cyc_Absyn_Exp*f1;void*f2;};struct Cyc_CfFlowInfo_InitParam_CfFlowInfo_Root_struct{int tag;int f1;void*f2;};struct Cyc_CfFlowInfo_Dot_CfFlowInfo_PathCon_struct{int tag;int f1;};struct Cyc_CfFlowInfo_Star_CfFlowInfo_PathCon_struct{int tag;};struct Cyc_CfFlowInfo_Place{void*root;struct Cyc_List_List*path;};
# 74
enum Cyc_CfFlowInfo_InitLevel{Cyc_CfFlowInfo_NoneIL =0U,Cyc_CfFlowInfo_AllIL =1U};extern char Cyc_CfFlowInfo_IsZero[7U];struct Cyc_CfFlowInfo_IsZero_Absyn_AbsynAnnot_struct{char*tag;};extern char Cyc_CfFlowInfo_NotZero[8U];struct Cyc_CfFlowInfo_NotZero_Absyn_AbsynAnnot_struct{char*tag;struct Cyc_List_List*f1;};extern char Cyc_CfFlowInfo_UnknownZ[9U];struct Cyc_CfFlowInfo_UnknownZ_Absyn_AbsynAnnot_struct{char*tag;struct Cyc_List_List*f1;};struct _union_AbsLVal_PlaceL{int tag;struct Cyc_CfFlowInfo_Place*val;};struct _union_AbsLVal_UnknownL{int tag;int val;};union Cyc_CfFlowInfo_AbsLVal{struct _union_AbsLVal_PlaceL PlaceL;struct _union_AbsLVal_UnknownL UnknownL;};struct Cyc_CfFlowInfo_UnionRInfo{int is_union;int fieldnum;};struct Cyc_CfFlowInfo_Zero_CfFlowInfo_AbsRVal_struct{int tag;};struct Cyc_CfFlowInfo_NotZeroAll_CfFlowInfo_AbsRVal_struct{int tag;};struct Cyc_CfFlowInfo_UnknownR_CfFlowInfo_AbsRVal_struct{int tag;enum Cyc_CfFlowInfo_InitLevel f1;};struct Cyc_CfFlowInfo_Esc_CfFlowInfo_AbsRVal_struct{int tag;enum Cyc_CfFlowInfo_InitLevel f1;};struct Cyc_CfFlowInfo_AddressOf_CfFlowInfo_AbsRVal_struct{int tag;struct Cyc_CfFlowInfo_Place*f1;};struct Cyc_CfFlowInfo_UniquePtr_CfFlowInfo_AbsRVal_struct{int tag;void*f1;};struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct{int tag;struct Cyc_CfFlowInfo_UnionRInfo f1;struct _fat_ptr f2;};struct Cyc_CfFlowInfo_Consumed_CfFlowInfo_AbsRVal_struct{int tag;struct Cyc_Absyn_Exp*f1;int f2;void*f3;};struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct{int tag;struct Cyc_Absyn_Vardecl*f1;void*f2;};struct _union_FlowInfo_BottomFL{int tag;int val;};struct _tuple15{struct Cyc_Dict_Dict f1;struct Cyc_List_List*f2;};struct _union_FlowInfo_ReachableFL{int tag;struct _tuple15 val;};union Cyc_CfFlowInfo_FlowInfo{struct _union_FlowInfo_BottomFL BottomFL;struct _union_FlowInfo_ReachableFL ReachableFL;};struct Cyc_CfFlowInfo_FlowEnv{void*zero;void*notzeroall;void*unknown_none;void*unknown_all;void*esc_none;void*esc_all;struct Cyc_Dict_Dict mt_flowdict;struct Cyc_CfFlowInfo_Place*dummy_place;};
# 26 "tcstmt.h"
void Cyc_Tcstmt_tcStmt(struct Cyc_Tcenv_Tenv*,struct Cyc_Absyn_Stmt*,int new_block);
# 51 "tcstmt.cyc"
static void Cyc_Tcstmt_simplify_unused_result_exp(struct Cyc_Absyn_Exp*e){
void*_stmttmp0=e->r;void*_tmp0=_stmttmp0;struct Cyc_Absyn_Exp*_tmp1;struct Cyc_Absyn_Exp*_tmp2;if(((struct Cyc_Absyn_Increment_e_Absyn_Raw_exp_struct*)_tmp0)->tag == 5U)switch(((struct Cyc_Absyn_Increment_e_Absyn_Raw_exp_struct*)_tmp0)->f2){case Cyc_Absyn_PostInc: _LL1: _tmp2=((struct Cyc_Absyn_Increment_e_Absyn_Raw_exp_struct*)_tmp0)->f1;_LL2: {struct Cyc_Absyn_Exp*e1=_tmp2;
({void*_tmp132=(void*)({struct Cyc_Absyn_Increment_e_Absyn_Raw_exp_struct*_tmp3=_cycalloc(sizeof(*_tmp3));_tmp3->tag=5U,_tmp3->f1=e1,_tmp3->f2=Cyc_Absyn_PreInc;_tmp3;});e->r=_tmp132;});goto _LL0;}case Cyc_Absyn_PostDec: _LL3: _tmp1=((struct Cyc_Absyn_Increment_e_Absyn_Raw_exp_struct*)_tmp0)->f1;_LL4: {struct Cyc_Absyn_Exp*e1=_tmp1;
({void*_tmp133=(void*)({struct Cyc_Absyn_Increment_e_Absyn_Raw_exp_struct*_tmp4=_cycalloc(sizeof(*_tmp4));_tmp4->tag=5U,_tmp4->f1=e1,_tmp4->f2=Cyc_Absyn_PreDec;_tmp4;});e->r=_tmp133;});goto _LL0;}default: goto _LL5;}else{_LL5: _LL6:
 goto _LL0;}_LL0:;}
# 51
static int Cyc_Tcstmt_is_var_exp(struct Cyc_Absyn_Exp*e){
# 61
while(1){
void*_stmttmp1=e->r;void*_tmp5=_stmttmp1;struct Cyc_Absyn_Exp*_tmp6;struct Cyc_Absyn_Exp*_tmp7;switch(*((int*)_tmp5)){case 1U: _LL1: _LL2:
 return 1;case 12U: _LL3: _tmp7=((struct Cyc_Absyn_NoInstantiate_e_Absyn_Raw_exp_struct*)_tmp5)->f1;_LL4: {struct Cyc_Absyn_Exp*e2=_tmp7;
_tmp6=e2;goto _LL6;}case 13U: _LL5: _tmp6=((struct Cyc_Absyn_Instantiate_e_Absyn_Raw_exp_struct*)_tmp5)->f1;_LL6: {struct Cyc_Absyn_Exp*e2=_tmp6;
e=e2;continue;}default: _LL7: _LL8:
# 67
 return 0;}_LL0:;}}struct _tuple16{struct Cyc_List_List*f1;struct Cyc_Absyn_Exp*f2;};
# 51
static int Cyc_Tcstmt_noassign_exp(struct Cyc_Absyn_Exp*e){
# 78
void*_stmttmp2=e->r;void*_tmp8=_stmttmp2;struct Cyc_List_List*_tmp9;struct Cyc_List_List*_tmpA;struct Cyc_List_List*_tmpB;struct Cyc_List_List*_tmpC;struct Cyc_List_List*_tmpD;struct Cyc_List_List*_tmpE;struct Cyc_List_List*_tmpF;struct Cyc_List_List*_tmp10;struct Cyc_Absyn_Exp*_tmp12;struct Cyc_Absyn_Exp*_tmp11;struct Cyc_Absyn_Exp*_tmp14;struct Cyc_Absyn_Exp*_tmp13;struct Cyc_Absyn_Exp*_tmp15;struct Cyc_Absyn_Exp*_tmp16;struct Cyc_Absyn_Exp*_tmp17;struct Cyc_Absyn_Exp*_tmp18;struct Cyc_Absyn_Exp*_tmp19;struct Cyc_Absyn_Exp*_tmp1A;struct Cyc_Absyn_Exp*_tmp1B;struct Cyc_Absyn_Exp*_tmp1C;struct Cyc_Absyn_Exp*_tmp1D;struct Cyc_Absyn_Exp*_tmp1E;struct Cyc_Absyn_Exp*_tmp1F;struct Cyc_Absyn_Exp*_tmp20;struct Cyc_Absyn_Exp*_tmp22;struct Cyc_Absyn_Exp*_tmp21;struct Cyc_Absyn_Exp*_tmp24;struct Cyc_Absyn_Exp*_tmp23;struct Cyc_Absyn_Exp*_tmp26;struct Cyc_Absyn_Exp*_tmp25;struct Cyc_Absyn_Exp*_tmp28;struct Cyc_Absyn_Exp*_tmp27;struct Cyc_Absyn_Exp*_tmp2A;struct Cyc_Absyn_Exp*_tmp29;struct Cyc_Absyn_Exp*_tmp2C;struct Cyc_Absyn_Exp*_tmp2B;struct Cyc_Absyn_Exp*_tmp2F;struct Cyc_Absyn_Exp*_tmp2E;struct Cyc_Absyn_Exp*_tmp2D;switch(*((int*)_tmp8)){case 10U: _LL1: _LL2:
 goto _LL4;case 4U: _LL3: _LL4:
 goto _LL6;case 36U: _LL5: _LL6:
 goto _LL8;case 41U: _LL7: _LL8:
 goto _LLA;case 38U: _LL9: _LLA:
 goto _LLC;case 5U: _LLB: _LLC:
 return 0;case 40U: _LLD: _LLE:
# 86
 goto _LL10;case 0U: _LLF: _LL10:
 goto _LL12;case 1U: _LL11: _LL12:
 goto _LL14;case 17U: _LL13: _LL14:
 goto _LL16;case 19U: _LL15: _LL16:
 goto _LL18;case 32U: _LL17: _LL18:
 goto _LL1A;case 33U: _LL19: _LL1A:
 goto _LL1C;case 2U: _LL1B: _LL1C:
 return 1;case 6U: _LL1D: _tmp2D=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmp8)->f1;_tmp2E=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmp8)->f2;_tmp2F=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmp8)->f3;_LL1E: {struct Cyc_Absyn_Exp*e1=_tmp2D;struct Cyc_Absyn_Exp*e2=_tmp2E;struct Cyc_Absyn_Exp*e3=_tmp2F;
# 96
if(!Cyc_Tcstmt_noassign_exp(e1))return 0;_tmp2B=e2;_tmp2C=e3;goto _LL20;}case 27U: _LL1F: _tmp2B=((struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct*)_tmp8)->f2;_tmp2C=((struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct*)_tmp8)->f3;_LL20: {struct Cyc_Absyn_Exp*e1=_tmp2B;struct Cyc_Absyn_Exp*e2=_tmp2C;
# 98
_tmp29=e1;_tmp2A=e2;goto _LL22;}case 7U: _LL21: _tmp29=((struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct*)_tmp8)->f1;_tmp2A=((struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct*)_tmp8)->f2;_LL22: {struct Cyc_Absyn_Exp*e1=_tmp29;struct Cyc_Absyn_Exp*e2=_tmp2A;
_tmp27=e1;_tmp28=e2;goto _LL24;}case 8U: _LL23: _tmp27=((struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct*)_tmp8)->f1;_tmp28=((struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct*)_tmp8)->f2;_LL24: {struct Cyc_Absyn_Exp*e1=_tmp27;struct Cyc_Absyn_Exp*e2=_tmp28;
_tmp25=e1;_tmp26=e2;goto _LL26;}case 23U: _LL25: _tmp25=((struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct*)_tmp8)->f1;_tmp26=((struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct*)_tmp8)->f2;_LL26: {struct Cyc_Absyn_Exp*e1=_tmp25;struct Cyc_Absyn_Exp*e2=_tmp26;
_tmp23=e1;_tmp24=e2;goto _LL28;}case 34U: _LL27: _tmp23=((struct Cyc_Absyn_Spawn_e_Absyn_Raw_exp_struct*)_tmp8)->f1;_tmp24=((struct Cyc_Absyn_Spawn_e_Absyn_Raw_exp_struct*)_tmp8)->f2;_LL28: {struct Cyc_Absyn_Exp*e1=_tmp23;struct Cyc_Absyn_Exp*e2=_tmp24;
_tmp21=e1;_tmp22=e2;goto _LL2A;}case 9U: _LL29: _tmp21=((struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct*)_tmp8)->f1;_tmp22=((struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct*)_tmp8)->f2;_LL2A: {struct Cyc_Absyn_Exp*e1=_tmp21;struct Cyc_Absyn_Exp*e2=_tmp22;
return Cyc_Tcstmt_noassign_exp(e1)&& Cyc_Tcstmt_noassign_exp(e2);}case 42U: _LL2B: _tmp20=((struct Cyc_Absyn_Extension_e_Absyn_Raw_exp_struct*)_tmp8)->f1;_LL2C: {struct Cyc_Absyn_Exp*e=_tmp20;
# 105
_tmp1F=e;goto _LL2E;}case 39U: _LL2D: _tmp1F=((struct Cyc_Absyn_Tagcheck_e_Absyn_Raw_exp_struct*)_tmp8)->f1;_LL2E: {struct Cyc_Absyn_Exp*e=_tmp1F;
_tmp1E=e;goto _LL30;}case 11U: _LL2F: _tmp1E=((struct Cyc_Absyn_Throw_e_Absyn_Raw_exp_struct*)_tmp8)->f1;_LL30: {struct Cyc_Absyn_Exp*e=_tmp1E;
_tmp1D=e;goto _LL32;}case 12U: _LL31: _tmp1D=((struct Cyc_Absyn_NoInstantiate_e_Absyn_Raw_exp_struct*)_tmp8)->f1;_LL32: {struct Cyc_Absyn_Exp*e=_tmp1D;
_tmp1C=e;goto _LL34;}case 13U: _LL33: _tmp1C=((struct Cyc_Absyn_Instantiate_e_Absyn_Raw_exp_struct*)_tmp8)->f1;_LL34: {struct Cyc_Absyn_Exp*e=_tmp1C;
_tmp1B=e;goto _LL36;}case 14U: _LL35: _tmp1B=((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp8)->f2;_LL36: {struct Cyc_Absyn_Exp*e=_tmp1B;
_tmp1A=e;goto _LL38;}case 18U: _LL37: _tmp1A=((struct Cyc_Absyn_Sizeofexp_e_Absyn_Raw_exp_struct*)_tmp8)->f1;_LL38: {struct Cyc_Absyn_Exp*e=_tmp1A;
_tmp19=e;goto _LL3A;}case 20U: _LL39: _tmp19=((struct Cyc_Absyn_Deref_e_Absyn_Raw_exp_struct*)_tmp8)->f1;_LL3A: {struct Cyc_Absyn_Exp*e=_tmp19;
_tmp18=e;goto _LL3C;}case 21U: _LL3B: _tmp18=((struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct*)_tmp8)->f1;_LL3C: {struct Cyc_Absyn_Exp*e=_tmp18;
_tmp17=e;goto _LL3E;}case 22U: _LL3D: _tmp17=((struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct*)_tmp8)->f1;_LL3E: {struct Cyc_Absyn_Exp*e=_tmp17;
_tmp16=e;goto _LL40;}case 28U: _LL3F: _tmp16=((struct Cyc_Absyn_ComprehensionNoinit_e_Absyn_Raw_exp_struct*)_tmp8)->f1;_LL40: {struct Cyc_Absyn_Exp*e=_tmp16;
_tmp15=e;goto _LL42;}case 15U: _LL41: _tmp15=((struct Cyc_Absyn_Address_e_Absyn_Raw_exp_struct*)_tmp8)->f1;_LL42: {struct Cyc_Absyn_Exp*e=_tmp15;
return Cyc_Tcstmt_noassign_exp(e);}case 35U: _LL43: _tmp13=(((struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct*)_tmp8)->f1).rgn;_tmp14=(((struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct*)_tmp8)->f1).num_elts;_LL44: {struct Cyc_Absyn_Exp*eopt=_tmp13;struct Cyc_Absyn_Exp*e=_tmp14;
# 119
_tmp11=eopt;_tmp12=e;goto _LL46;}case 16U: _LL45: _tmp11=((struct Cyc_Absyn_New_e_Absyn_Raw_exp_struct*)_tmp8)->f1;_tmp12=((struct Cyc_Absyn_New_e_Absyn_Raw_exp_struct*)_tmp8)->f2;_LL46: {struct Cyc_Absyn_Exp*eopt=_tmp11;struct Cyc_Absyn_Exp*e=_tmp12;
# 121
if(!Cyc_Tcstmt_noassign_exp(e))return 0;if(eopt != 0)
return Cyc_Tcstmt_noassign_exp(eopt);else{
return 1;}}case 3U: _LL47: _tmp10=((struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct*)_tmp8)->f2;_LL48: {struct Cyc_List_List*es=_tmp10;
# 125
_tmpF=es;goto _LL4A;}case 31U: _LL49: _tmpF=((struct Cyc_Absyn_Datatype_e_Absyn_Raw_exp_struct*)_tmp8)->f1;_LL4A: {struct Cyc_List_List*es=_tmpF;
_tmpE=es;goto _LL4C;}case 24U: _LL4B: _tmpE=((struct Cyc_Absyn_Tuple_e_Absyn_Raw_exp_struct*)_tmp8)->f1;_LL4C: {struct Cyc_List_List*es=_tmpE;
return({int(*_tmp134)(int(*pred)(struct Cyc_Absyn_Exp*),struct Cyc_List_List*x)=({int(*_tmp30)(int(*pred)(struct Cyc_Absyn_Exp*),struct Cyc_List_List*x)=(int(*)(int(*pred)(struct Cyc_Absyn_Exp*),struct Cyc_List_List*x))Cyc_List_forall;_tmp30;});_tmp134(Cyc_Tcstmt_noassign_exp,es);});}case 37U: _LL4D: _tmpD=((struct Cyc_Absyn_UnresolvedMem_e_Absyn_Raw_exp_struct*)_tmp8)->f2;_LL4E: {struct Cyc_List_List*dles=_tmpD;
# 129
_tmpC=dles;goto _LL50;}case 29U: _LL4F: _tmpC=((struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct*)_tmp8)->f3;_LL50: {struct Cyc_List_List*dles=_tmpC;
_tmpB=dles;goto _LL52;}case 30U: _LL51: _tmpB=((struct Cyc_Absyn_AnonStruct_e_Absyn_Raw_exp_struct*)_tmp8)->f2;_LL52: {struct Cyc_List_List*dles=_tmpB;
_tmpA=dles;goto _LL54;}case 26U: _LL53: _tmpA=((struct Cyc_Absyn_Array_e_Absyn_Raw_exp_struct*)_tmp8)->f1;_LL54: {struct Cyc_List_List*dles=_tmpA;
_tmp9=dles;goto _LL56;}default: _LL55: _tmp9=((struct Cyc_Absyn_CompoundLit_e_Absyn_Raw_exp_struct*)_tmp8)->f2;_LL56: {struct Cyc_List_List*dles=_tmp9;
# 134
for(0;dles != 0;dles=dles->tl){
struct _tuple16*_stmttmp3=(struct _tuple16*)dles->hd;struct Cyc_Absyn_Exp*_tmp31;_LL58: _tmp31=_stmttmp3->f2;_LL59: {struct Cyc_Absyn_Exp*e=_tmp31;
if(!Cyc_Tcstmt_noassign_exp(e))return 0;}}
# 138
return 1;}}_LL0:;}struct _tuple17{struct Cyc_Absyn_Tvar*f1;int f2;};struct _tuple18{struct Cyc_Absyn_Vardecl**f1;struct Cyc_Absyn_Exp*f2;};
# 51
static void Cyc_Tcstmt_pattern_synth(unsigned loc,struct Cyc_Tcenv_Tenv*te,struct Cyc_Tcpat_TcPatResult pat_res,struct Cyc_Absyn_Stmt*s,struct Cyc_Absyn_Exp*where_opt,int new_block){
# 145
struct Cyc_List_List*_tmp33;struct _tuple1*_tmp32;_LL1: _tmp32=pat_res.tvars_and_bounds_opt;_tmp33=pat_res.patvars;_LL2: {struct _tuple1*p=_tmp32;struct Cyc_List_List*vse=_tmp33;
struct Cyc_List_List*vs=({struct Cyc_List_List*(*_tmp136)(struct Cyc_Absyn_Vardecl**(*f)(struct _tuple18*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp3E)(struct Cyc_Absyn_Vardecl**(*f)(struct _tuple18*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_Absyn_Vardecl**(*f)(struct _tuple18*),struct Cyc_List_List*x))Cyc_List_map;_tmp3E;});struct Cyc_Absyn_Vardecl**(*_tmp135)(struct _tuple18*)=({struct Cyc_Absyn_Vardecl**(*_tmp3F)(struct _tuple18*)=(struct Cyc_Absyn_Vardecl**(*)(struct _tuple18*))Cyc_Core_fst;_tmp3F;});_tmp136(_tmp135,vse);});
struct Cyc_List_List*tvs=p == 0?0:({struct Cyc_List_List*(*_tmp138)(struct Cyc_Absyn_Tvar*(*f)(struct _tuple17*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp3C)(struct Cyc_Absyn_Tvar*(*f)(struct _tuple17*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_Absyn_Tvar*(*f)(struct _tuple17*),struct Cyc_List_List*x))Cyc_List_map;_tmp3C;});struct Cyc_Absyn_Tvar*(*_tmp137)(struct _tuple17*)=({struct Cyc_Absyn_Tvar*(*_tmp3D)(struct _tuple17*)=(struct Cyc_Absyn_Tvar*(*)(struct _tuple17*))Cyc_Core_fst;_tmp3D;});_tmp138(_tmp137,(*p).f1);});
struct Cyc_List_List*bds=p == 0?0:(*p).f2;
struct Cyc_List_List*rgns=p == 0?0:({struct Cyc_List_List*(*_tmp13C)(struct Cyc_Absyn_Tvar*(*f)(struct _tuple17*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp38)(struct Cyc_Absyn_Tvar*(*f)(struct _tuple17*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_Absyn_Tvar*(*f)(struct _tuple17*),struct Cyc_List_List*x))Cyc_List_map;_tmp38;});struct Cyc_Absyn_Tvar*(*_tmp13B)(struct _tuple17*)=({struct Cyc_Absyn_Tvar*(*_tmp39)(struct _tuple17*)=(struct Cyc_Absyn_Tvar*(*)(struct _tuple17*))Cyc_Core_fst;_tmp39;});_tmp13C(_tmp13B,({struct Cyc_List_List*(*_tmp13A)(int(*f)(struct _tuple17*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp3A)(int(*f)(struct _tuple17*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(int(*f)(struct _tuple17*),struct Cyc_List_List*x))Cyc_List_filter;_tmp3A;});int(*_tmp139)(struct _tuple17*)=({int(*_tmp3B)(struct _tuple17*)=(int(*)(struct _tuple17*))Cyc_Core_snd;_tmp3B;});_tmp13A(_tmp139,(*p).f1);}));});
struct Cyc_Tcenv_Tenv*te2=Cyc_Tcenv_add_type_vars(loc,te,tvs);
for(0;rgns != 0;rgns=rgns->tl){
te2=({struct Cyc_Tcenv_Tenv*_tmp13D=te2;Cyc_Tcenv_add_region(_tmp13D,(void*)({struct Cyc_Absyn_VarType_Absyn_Type_struct*_tmp34=_cycalloc(sizeof(*_tmp34));_tmp34->tag=2U,_tmp34->f1=(struct Cyc_Absyn_Tvar*)rgns->hd;_tmp34;}),0);});}
te2=Cyc_Tcenv_new_outlives_constraints(te2,bds,loc);
if(new_block)
te2=Cyc_Tcenv_new_block(loc,te2);{
# 154
void*ropt=
# 156
Cyc_Tcenv_curr_rgn(te2);
{struct Cyc_List_List*vs2=vs;for(0;vs2 != 0;vs2=vs2->tl){
if((struct Cyc_Absyn_Vardecl**)vs2->hd != 0)
(*((struct Cyc_Absyn_Vardecl**)_check_null((struct Cyc_Absyn_Vardecl**)vs2->hd)))->rgn=ropt;}}
# 157
if(where_opt != 0){
# 162
({struct Cyc_Tcenv_Tenv*_tmp13F=te2;struct Cyc_Absyn_Exp*_tmp13E=where_opt;Cyc_Tcexp_tcTest(_tmp13F,_tmp13E,({const char*_tmp35="switch clause guard";_tag_fat(_tmp35,sizeof(char),20U);}));});
if(!Cyc_Tcstmt_noassign_exp(where_opt))
({void*_tmp36=0U;({unsigned _tmp141=where_opt->loc;struct _fat_ptr _tmp140=({const char*_tmp37="cannot show &&-clause in pattern has no effects";_tag_fat(_tmp37,sizeof(char),48U);});Cyc_Tcutil_terr(_tmp141,_tmp140,_tag_fat(_tmp36,sizeof(void*),0U));});});}
# 157
Cyc_Tcstmt_tcStmt(te2,s,0);}}}
# 51
static int Cyc_Tcstmt_stmt_temp_var_counter=0;
# 170
static struct _tuple2*Cyc_Tcstmt_stmt_temp_var(){
int i=Cyc_Tcstmt_stmt_temp_var_counter ++;
struct _tuple2*res=({struct _tuple2*_tmp44=_cycalloc(sizeof(*_tmp44));_tmp44->f1=Cyc_Absyn_Loc_n,({struct _fat_ptr*_tmp144=({struct _fat_ptr*_tmp43=_cycalloc(sizeof(*_tmp43));({struct _fat_ptr _tmp143=(struct _fat_ptr)({struct Cyc_Int_pa_PrintArg_struct _tmp42=({struct Cyc_Int_pa_PrintArg_struct _tmp117;_tmp117.tag=1U,_tmp117.f1=(unsigned)i;_tmp117;});void*_tmp40[1U];_tmp40[0]=& _tmp42;({struct _fat_ptr _tmp142=({const char*_tmp41="_stmttmp%X";_tag_fat(_tmp41,sizeof(char),11U);});Cyc_aprintf(_tmp142,_tag_fat(_tmp40,sizeof(void*),1U));});});*_tmp43=_tmp143;});_tmp43;});_tmp44->f2=_tmp144;});_tmp44;});
return res;}
# 170
void Cyc_Tcstmt_tcStmt(struct Cyc_Tcenv_Tenv*te,struct Cyc_Absyn_Stmt*s0,int new_block){
# 185 "tcstmt.cyc"
void*_stmttmp4=s0->r;void*_tmp45=_stmttmp4;struct Cyc_Absyn_Stmt*_tmp47;struct Cyc_Absyn_Decl*_tmp46;void**_tmp4A;struct Cyc_List_List**_tmp49;struct Cyc_Absyn_Stmt*_tmp48;void**_tmp4D;struct Cyc_List_List*_tmp4C;struct Cyc_Absyn_Exp**_tmp4B;struct Cyc_Absyn_Stmt*_tmp4F;struct _fat_ptr*_tmp4E;struct Cyc_Absyn_Switch_clause***_tmp51;struct Cyc_List_List*_tmp50;struct Cyc_Absyn_Stmt*_tmp54;struct Cyc_Absyn_Exp*_tmp53;struct Cyc_Absyn_Stmt*_tmp52;struct Cyc_Absyn_Stmt*_tmp5A;struct Cyc_Absyn_Stmt*_tmp59;struct Cyc_Absyn_Exp*_tmp58;struct Cyc_Absyn_Stmt*_tmp57;struct Cyc_Absyn_Exp*_tmp56;struct Cyc_Absyn_Exp*_tmp55;struct Cyc_Absyn_Stmt*_tmp5D;struct Cyc_Absyn_Stmt*_tmp5C;struct Cyc_Absyn_Exp*_tmp5B;struct Cyc_Absyn_Stmt*_tmp60;struct Cyc_Absyn_Stmt*_tmp5F;struct Cyc_Absyn_Exp*_tmp5E;struct Cyc_Absyn_Exp*_tmp61;struct Cyc_Absyn_Stmt*_tmp63;struct Cyc_Absyn_Stmt*_tmp62;struct Cyc_Absyn_Exp*_tmp64;switch(*((int*)_tmp45)){case 0U: _LL1: _LL2:
# 187
 return;case 1U: _LL3: _tmp64=((struct Cyc_Absyn_Exp_s_Absyn_Raw_stmt_struct*)_tmp45)->f1;_LL4: {struct Cyc_Absyn_Exp*e=_tmp64;
# 190
Cyc_Tcexp_tcExp(te,0,e);
if(!Cyc_Tcenv_in_stmt_exp(te))
Cyc_Tcstmt_simplify_unused_result_exp(e);
# 191
return;}case 2U: _LL5: _tmp62=((struct Cyc_Absyn_Seq_s_Absyn_Raw_stmt_struct*)_tmp45)->f1;_tmp63=((struct Cyc_Absyn_Seq_s_Absyn_Raw_stmt_struct*)_tmp45)->f2;_LL6: {struct Cyc_Absyn_Stmt*s1=_tmp62;struct Cyc_Absyn_Stmt*s2=_tmp63;
# 196
Cyc_Tcstmt_tcStmt(te,s1,1);
Cyc_Tcstmt_tcStmt(te,s2,1);
return;}case 3U: _LL7: _tmp61=((struct Cyc_Absyn_Return_s_Absyn_Raw_stmt_struct*)_tmp45)->f1;_LL8: {struct Cyc_Absyn_Exp*eopt=_tmp61;
# 201
void*t=Cyc_Tcenv_return_typ(te);
if(eopt == 0){
if(!Cyc_Tcutil_is_void_type(t)){
if(Cyc_Tcutil_is_any_float_type(t)|| Cyc_Tcutil_is_any_int_type(t))
({struct Cyc_String_pa_PrintArg_struct _tmp67=({struct Cyc_String_pa_PrintArg_struct _tmp118;_tmp118.tag=0U,({struct _fat_ptr _tmp145=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t));_tmp118.f1=_tmp145;});_tmp118;});void*_tmp65[1U];_tmp65[0]=& _tmp67;({unsigned _tmp147=s0->loc;struct _fat_ptr _tmp146=({const char*_tmp66="should return a value of type %s";_tag_fat(_tmp66,sizeof(char),33U);});Cyc_Tcutil_warn(_tmp147,_tmp146,_tag_fat(_tmp65,sizeof(void*),1U));});});else{
# 207
({struct Cyc_String_pa_PrintArg_struct _tmp6A=({struct Cyc_String_pa_PrintArg_struct _tmp119;_tmp119.tag=0U,({struct _fat_ptr _tmp148=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t));_tmp119.f1=_tmp148;});_tmp119;});void*_tmp68[1U];_tmp68[0]=& _tmp6A;({unsigned _tmp14A=s0->loc;struct _fat_ptr _tmp149=({const char*_tmp69="must return a value of type %s";_tag_fat(_tmp69,sizeof(char),31U);});Cyc_Tcutil_terr(_tmp14A,_tmp149,_tag_fat(_tmp68,sizeof(void*),1U));});});}}}else{
# 211
int bogus=0;
struct Cyc_Absyn_Exp*e=eopt;
Cyc_Tcexp_tcExp(te,& t,e);
if(!Cyc_Tcutil_coerce_arg(te,e,t,& bogus)){
({struct Cyc_String_pa_PrintArg_struct _tmp6D=({struct Cyc_String_pa_PrintArg_struct _tmp11B;_tmp11B.tag=0U,({
struct _fat_ptr _tmp14B=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string((void*)_check_null(e->topt)));_tmp11B.f1=_tmp14B;});_tmp11B;});struct Cyc_String_pa_PrintArg_struct _tmp6E=({struct Cyc_String_pa_PrintArg_struct _tmp11A;_tmp11A.tag=0U,({struct _fat_ptr _tmp14C=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t));_tmp11A.f1=_tmp14C;});_tmp11A;});void*_tmp6B[2U];_tmp6B[0]=& _tmp6D,_tmp6B[1]=& _tmp6E;({unsigned _tmp14E=s0->loc;struct _fat_ptr _tmp14D=({const char*_tmp6C="returns value of type %s but requires %s";_tag_fat(_tmp6C,sizeof(char),41U);});Cyc_Tcutil_terr(_tmp14E,_tmp14D,_tag_fat(_tmp6B,sizeof(void*),2U));});});
Cyc_Unify_explain_failure();}
# 214
if(
# 219
Cyc_Tcutil_is_noalias_pointer_or_aggr(t)&& !Cyc_Tcutil_is_noalias_path(e))
({void*_tmp6F=0U;({unsigned _tmp150=e->loc;struct _fat_ptr _tmp14F=({const char*_tmp70="Cannot consume non-unique paths; do swap instead";_tag_fat(_tmp70,sizeof(char),49U);});Cyc_Tcutil_terr(_tmp150,_tmp14F,_tag_fat(_tmp6F,sizeof(void*),0U));});});}
# 222
return;}case 4U: _LL9: _tmp5E=((struct Cyc_Absyn_IfThenElse_s_Absyn_Raw_stmt_struct*)_tmp45)->f1;_tmp5F=((struct Cyc_Absyn_IfThenElse_s_Absyn_Raw_stmt_struct*)_tmp45)->f2;_tmp60=((struct Cyc_Absyn_IfThenElse_s_Absyn_Raw_stmt_struct*)_tmp45)->f3;_LLA: {struct Cyc_Absyn_Exp*e=_tmp5E;struct Cyc_Absyn_Stmt*s1=_tmp5F;struct Cyc_Absyn_Stmt*s2=_tmp60;
# 225
({struct Cyc_Tcenv_Tenv*_tmp152=te;struct Cyc_Absyn_Exp*_tmp151=e;Cyc_Tcexp_tcTest(_tmp152,_tmp151,({const char*_tmp71="if statement";_tag_fat(_tmp71,sizeof(char),13U);}));});
Cyc_Tcstmt_tcStmt(te,s1,1);
Cyc_Tcstmt_tcStmt(te,s2,1);
return;}case 5U: _LLB: _tmp5B=(((struct Cyc_Absyn_While_s_Absyn_Raw_stmt_struct*)_tmp45)->f1).f1;_tmp5C=(((struct Cyc_Absyn_While_s_Absyn_Raw_stmt_struct*)_tmp45)->f1).f2;_tmp5D=((struct Cyc_Absyn_While_s_Absyn_Raw_stmt_struct*)_tmp45)->f2;_LLC: {struct Cyc_Absyn_Exp*e=_tmp5B;struct Cyc_Absyn_Stmt*cont_s=_tmp5C;struct Cyc_Absyn_Stmt*s=_tmp5D;
# 231
({struct Cyc_Tcenv_Tenv*_tmp154=te;struct Cyc_Absyn_Exp*_tmp153=e;Cyc_Tcexp_tcTest(_tmp154,_tmp153,({const char*_tmp72="while loop";_tag_fat(_tmp72,sizeof(char),11U);}));});
Cyc_Tcstmt_tcStmt(te,s,1);
return;}case 9U: _LLD: _tmp55=((struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct*)_tmp45)->f1;_tmp56=(((struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct*)_tmp45)->f2).f1;_tmp57=(((struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct*)_tmp45)->f2).f2;_tmp58=(((struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct*)_tmp45)->f3).f1;_tmp59=(((struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct*)_tmp45)->f3).f2;_tmp5A=((struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct*)_tmp45)->f4;_LLE: {struct Cyc_Absyn_Exp*e1=_tmp55;struct Cyc_Absyn_Exp*e2=_tmp56;struct Cyc_Absyn_Stmt*guard_s=_tmp57;struct Cyc_Absyn_Exp*e3=_tmp58;struct Cyc_Absyn_Stmt*cont_s=_tmp59;struct Cyc_Absyn_Stmt*s=_tmp5A;
# 236
Cyc_Tcexp_tcExp(te,0,e1);
({struct Cyc_Tcenv_Tenv*_tmp156=te;struct Cyc_Absyn_Exp*_tmp155=e2;Cyc_Tcexp_tcTest(_tmp156,_tmp155,({const char*_tmp73="for loop";_tag_fat(_tmp73,sizeof(char),9U);}));});
Cyc_Tcstmt_tcStmt(te,s,1);
Cyc_Tcexp_tcExp(te,0,e3);
Cyc_Tcstmt_simplify_unused_result_exp(e3);
return;}case 14U: _LLF: _tmp52=((struct Cyc_Absyn_Do_s_Absyn_Raw_stmt_struct*)_tmp45)->f1;_tmp53=(((struct Cyc_Absyn_Do_s_Absyn_Raw_stmt_struct*)_tmp45)->f2).f1;_tmp54=(((struct Cyc_Absyn_Do_s_Absyn_Raw_stmt_struct*)_tmp45)->f2).f2;_LL10: {struct Cyc_Absyn_Stmt*s=_tmp52;struct Cyc_Absyn_Exp*e=_tmp53;struct Cyc_Absyn_Stmt*cont_s=_tmp54;
# 244
Cyc_Tcstmt_tcStmt(te,s,1);
({struct Cyc_Tcenv_Tenv*_tmp158=te;struct Cyc_Absyn_Exp*_tmp157=e;Cyc_Tcexp_tcTest(_tmp158,_tmp157,({const char*_tmp74="do loop";_tag_fat(_tmp74,sizeof(char),8U);}));});
return;}case 6U: _LL11: _LL12:
# 248
 goto _LL14;case 7U: _LL13: _LL14:
 goto _LL16;case 8U: _LL15: _LL16:
 return;case 11U: _LL17: _tmp50=((struct Cyc_Absyn_Fallthru_s_Absyn_Raw_stmt_struct*)_tmp45)->f1;_tmp51=(struct Cyc_Absyn_Switch_clause***)&((struct Cyc_Absyn_Fallthru_s_Absyn_Raw_stmt_struct*)_tmp45)->f2;_LL18: {struct Cyc_List_List*es=_tmp50;struct Cyc_Absyn_Switch_clause***clauseopt=_tmp51;
# 253
const struct _tuple13*trip_opt=Cyc_Tcenv_process_fallthru(te,s0,clauseopt);
if(trip_opt == (const struct _tuple13*)0){
({void*_tmp75=0U;({unsigned _tmp15A=s0->loc;struct _fat_ptr _tmp159=({const char*_tmp76="fallthru not in a non-last case";_tag_fat(_tmp76,sizeof(char),32U);});Cyc_Tcutil_terr(_tmp15A,_tmp159,_tag_fat(_tmp75,sizeof(void*),0U));});});
return;}{
# 254
struct Cyc_List_List*tvs=(*trip_opt).f2;
# 259
struct Cyc_List_List*ts_orig=(*trip_opt).f3;
struct Cyc_List_List*instantiation=({
struct Cyc_List_List*(*_tmp15C)(struct _tuple14*(*f)(struct Cyc_List_List*,struct Cyc_Absyn_Tvar*),struct Cyc_List_List*env,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp82)(struct _tuple14*(*f)(struct Cyc_List_List*,struct Cyc_Absyn_Tvar*),struct Cyc_List_List*env,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct _tuple14*(*f)(struct Cyc_List_List*,struct Cyc_Absyn_Tvar*),struct Cyc_List_List*env,struct Cyc_List_List*x))Cyc_List_map_c;_tmp82;});struct Cyc_List_List*_tmp15B=Cyc_Tcenv_lookup_type_vars(te);_tmp15C(Cyc_Tcutil_make_inst_var,_tmp15B,tvs);});
struct Cyc_List_List*ts=({struct Cyc_List_List*(*_tmp15E)(void*(*f)(struct Cyc_List_List*,void*),struct Cyc_List_List*env,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp81)(void*(*f)(struct Cyc_List_List*,void*),struct Cyc_List_List*env,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(void*(*f)(struct Cyc_List_List*,void*),struct Cyc_List_List*env,struct Cyc_List_List*x))Cyc_List_map_c;_tmp81;});struct Cyc_List_List*_tmp15D=instantiation;_tmp15E(Cyc_Tcutil_substitute,_tmp15D,ts_orig);});
for(0;ts != 0 && es != 0;(ts=ts->tl,es=es->tl)){
# 265
int bogus=0;
Cyc_Tcexp_tcExp(te,0,(struct Cyc_Absyn_Exp*)es->hd);
if(!Cyc_Tcutil_coerce_arg(te,(struct Cyc_Absyn_Exp*)es->hd,(void*)ts->hd,& bogus)){
({struct Cyc_String_pa_PrintArg_struct _tmp79=({struct Cyc_String_pa_PrintArg_struct _tmp11D;_tmp11D.tag=0U,({
# 270
struct _fat_ptr _tmp15F=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string((void*)_check_null(((struct Cyc_Absyn_Exp*)es->hd)->topt)));_tmp11D.f1=_tmp15F;});_tmp11D;});struct Cyc_String_pa_PrintArg_struct _tmp7A=({struct Cyc_String_pa_PrintArg_struct _tmp11C;_tmp11C.tag=0U,({struct _fat_ptr _tmp160=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string((void*)ts->hd));_tmp11C.f1=_tmp160;});_tmp11C;});void*_tmp77[2U];_tmp77[0]=& _tmp79,_tmp77[1]=& _tmp7A;({unsigned _tmp162=s0->loc;struct _fat_ptr _tmp161=({const char*_tmp78="fallthru argument has type %s but pattern variable has type %s";_tag_fat(_tmp78,sizeof(char),63U);});Cyc_Tcutil_terr(_tmp162,_tmp161,_tag_fat(_tmp77,sizeof(void*),2U));});});
Cyc_Unify_explain_failure();}
# 267
if(
# 274
Cyc_Tcutil_is_noalias_pointer_or_aggr((void*)ts->hd)&& !Cyc_Tcutil_is_noalias_path((struct Cyc_Absyn_Exp*)es->hd))
({void*_tmp7B=0U;({unsigned _tmp164=((struct Cyc_Absyn_Exp*)es->hd)->loc;struct _fat_ptr _tmp163=({const char*_tmp7C="Cannot consume non-unique paths; do swap instead";_tag_fat(_tmp7C,sizeof(char),49U);});Cyc_Tcutil_terr(_tmp164,_tmp163,_tag_fat(_tmp7B,sizeof(void*),0U));});});}
# 277
if(es != 0)
({void*_tmp7D=0U;({unsigned _tmp166=s0->loc;struct _fat_ptr _tmp165=({const char*_tmp7E="too many arguments in explicit fallthru";_tag_fat(_tmp7E,sizeof(char),40U);});Cyc_Tcutil_terr(_tmp166,_tmp165,_tag_fat(_tmp7D,sizeof(void*),0U));});});
# 277
if(ts != 0)
# 280
({void*_tmp7F=0U;({unsigned _tmp168=s0->loc;struct _fat_ptr _tmp167=({const char*_tmp80="too few arguments in explicit fallthru";_tag_fat(_tmp80,sizeof(char),39U);});Cyc_Tcutil_terr(_tmp168,_tmp167,_tag_fat(_tmp7F,sizeof(void*),0U));});});
# 277
return;}}case 13U: _LL19: _tmp4E=((struct Cyc_Absyn_Label_s_Absyn_Raw_stmt_struct*)_tmp45)->f1;_tmp4F=((struct Cyc_Absyn_Label_s_Absyn_Raw_stmt_struct*)_tmp45)->f2;_LL1A: {struct _fat_ptr*l=_tmp4E;struct Cyc_Absyn_Stmt*s=_tmp4F;
# 287
({struct Cyc_Tcenv_Tenv*_tmp170=({unsigned _tmp16F=s0->loc;struct Cyc_Tcenv_Tenv*_tmp16E=te;Cyc_Tcenv_new_named_block(_tmp16F,_tmp16E,({struct Cyc_Absyn_Tvar*_tmp87=_cycalloc(sizeof(*_tmp87));
({struct _fat_ptr*_tmp16D=({struct _fat_ptr*_tmp86=_cycalloc(sizeof(*_tmp86));({struct _fat_ptr _tmp16C=(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp85=({struct Cyc_String_pa_PrintArg_struct _tmp11E;_tmp11E.tag=0U,_tmp11E.f1=(struct _fat_ptr)((struct _fat_ptr)*l);_tmp11E;});void*_tmp83[1U];_tmp83[0]=& _tmp85;({struct _fat_ptr _tmp16B=({const char*_tmp84="`%s";_tag_fat(_tmp84,sizeof(char),4U);});Cyc_aprintf(_tmp16B,_tag_fat(_tmp83,sizeof(void*),1U));});});*_tmp86=_tmp16C;});_tmp86;});_tmp87->name=_tmp16D;}),({
int _tmp16A=Cyc_Tcutil_new_tvar_id();_tmp87->identity=_tmp16A;}),({
void*_tmp169=Cyc_Tcutil_kind_to_bound(& Cyc_Tcutil_rk);_tmp87->kind=_tmp169;});_tmp87;}));});
# 287
Cyc_Tcstmt_tcStmt(_tmp170,s,0);});
# 291
return;}case 10U: _LL1B: _tmp4B=(struct Cyc_Absyn_Exp**)&((struct Cyc_Absyn_Switch_s_Absyn_Raw_stmt_struct*)_tmp45)->f1;_tmp4C=((struct Cyc_Absyn_Switch_s_Absyn_Raw_stmt_struct*)_tmp45)->f2;_tmp4D=(void**)&((struct Cyc_Absyn_Switch_s_Absyn_Raw_stmt_struct*)_tmp45)->f3;_LL1C: {struct Cyc_Absyn_Exp**exp=_tmp4B;struct Cyc_List_List*scs0=_tmp4C;void**dtp=_tmp4D;
# 296
struct Cyc_Absyn_Exp*e=*exp;
if(!Cyc_Tcstmt_is_var_exp(e)){
struct Cyc_Absyn_Vardecl*vd=({struct _tuple2*_tmp171=Cyc_Tcstmt_stmt_temp_var();Cyc_Absyn_new_vardecl(0U,_tmp171,Cyc_Absyn_new_evar(0,0),0);});
struct Cyc_Absyn_Stmt*s1=Cyc_Absyn_new_stmt(s0->r,s0->loc);
struct Cyc_Absyn_Decl*d=({struct Cyc_Absyn_Pat*_tmp175=({void*_tmp173=(void*)({struct Cyc_Absyn_Var_p_Absyn_Raw_pat_struct*_tmp89=_cycalloc(sizeof(*_tmp89));_tmp89->tag=1U,_tmp89->f1=vd,({struct Cyc_Absyn_Pat*_tmp172=Cyc_Absyn_new_pat((void*)& Cyc_Absyn_Wild_p_val,e->loc);_tmp89->f2=_tmp172;});_tmp89;});Cyc_Absyn_new_pat(_tmp173,e->loc);});struct Cyc_Absyn_Exp*_tmp174=e;Cyc_Absyn_let_decl(_tmp175,_tmp174,s0->loc);});
# 303
struct Cyc_Absyn_Stmt*s2=Cyc_Absyn_decl_stmt(d,s1,s0->loc);
s0->r=s2->r;
({struct Cyc_Absyn_Exp*_tmp177=({void*_tmp176=(void*)({struct Cyc_Absyn_Local_b_Absyn_Binding_struct*_tmp88=_cycalloc(sizeof(*_tmp88));_tmp88->tag=4U,_tmp88->f1=vd;_tmp88;});Cyc_Absyn_varb_exp(_tmp176,e->loc);});*exp=_tmp177;});
Cyc_Tcstmt_tcStmt(te,s0,new_block);
return;}{
# 297
void*t=
# 310
Cyc_Tcexp_tcExp(te,0,e);
# 312
if(Cyc_Tcutil_is_noalias_pointer_or_aggr(t)&& !Cyc_Tcutil_is_noalias_path(e))
({void*_tmp8A=0U;({unsigned _tmp179=e->loc;struct _fat_ptr _tmp178=({const char*_tmp8B="Cannot consume non-unique paths; do swap instead";_tag_fat(_tmp8B,sizeof(char),49U);});Cyc_Tcutil_terr(_tmp179,_tmp178,_tag_fat(_tmp8A,sizeof(void*),0U));});});{
# 312
struct Cyc_Tcpat_TcPatResult this_pattern;
# 318
struct Cyc_Tcpat_TcPatResult*next_pattern=0;
int first_case=1;
{struct Cyc_List_List*scs=scs0;for(0;scs != 0;scs=scs->tl){
if(first_case){
first_case=0;
this_pattern=Cyc_Tcpat_tcPat(te,((struct Cyc_Absyn_Switch_clause*)scs->hd)->pattern,& t,e);}else{
# 325
this_pattern=*((struct Cyc_Tcpat_TcPatResult*)_check_null(next_pattern));}
# 327
if(scs->tl != 0){
next_pattern=({struct Cyc_Tcpat_TcPatResult*_tmp8C=_cycalloc(sizeof(*_tmp8C));({struct Cyc_Tcpat_TcPatResult _tmp17A=Cyc_Tcpat_tcPat(te,((struct Cyc_Absyn_Switch_clause*)((struct Cyc_List_List*)_check_null(scs->tl))->hd)->pattern,& t,e);*_tmp8C=_tmp17A;});_tmp8C;});
if(next_pattern->tvars_and_bounds_opt != 0 &&(*next_pattern->tvars_and_bounds_opt).f2 != 0)
# 331
te=Cyc_Tcenv_clear_fallthru(te);else{
# 333
struct Cyc_List_List*vs=Cyc_List_imp_rev(Cyc_Tcutil_filter_nulls((Cyc_List_split(next_pattern->patvars)).f1));
struct Cyc_List_List*new_tvs=next_pattern->tvars_and_bounds_opt == 0?0:({
struct Cyc_List_List*(*_tmp17C)(struct Cyc_Absyn_Tvar*(*f)(struct _tuple17*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp8D)(struct Cyc_Absyn_Tvar*(*f)(struct _tuple17*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_Absyn_Tvar*(*f)(struct _tuple17*),struct Cyc_List_List*x))Cyc_List_map;_tmp8D;});struct Cyc_Absyn_Tvar*(*_tmp17B)(struct _tuple17*)=({struct Cyc_Absyn_Tvar*(*_tmp8E)(struct _tuple17*)=(struct Cyc_Absyn_Tvar*(*)(struct _tuple17*))Cyc_Core_fst;_tmp8E;});_tmp17C(_tmp17B,(*next_pattern->tvars_and_bounds_opt).f1);});
te=Cyc_Tcenv_set_fallthru(te,new_tvs,vs,(struct Cyc_Absyn_Switch_clause*)((struct Cyc_List_List*)_check_null(scs->tl))->hd);}}else{
# 339
te=Cyc_Tcenv_clear_fallthru(te);}{
# 341
struct Cyc_Absyn_Pat*p=((struct Cyc_Absyn_Switch_clause*)scs->hd)->pattern;
if(!Cyc_Unify_unify((void*)_check_null(p->topt),t)){
({struct Cyc_String_pa_PrintArg_struct _tmp91=({struct Cyc_String_pa_PrintArg_struct _tmp120;_tmp120.tag=0U,({
struct _fat_ptr _tmp17D=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t));_tmp120.f1=_tmp17D;});_tmp120;});struct Cyc_String_pa_PrintArg_struct _tmp92=({struct Cyc_String_pa_PrintArg_struct _tmp11F;_tmp11F.tag=0U,({struct _fat_ptr _tmp17E=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string((void*)_check_null(p->topt)));_tmp11F.f1=_tmp17E;});_tmp11F;});void*_tmp8F[2U];_tmp8F[0]=& _tmp91,_tmp8F[1]=& _tmp92;({unsigned _tmp180=((struct Cyc_Absyn_Switch_clause*)scs->hd)->loc;struct _fat_ptr _tmp17F=({const char*_tmp90="switch on type %s, but case expects type %s";_tag_fat(_tmp90,sizeof(char),44U);});Cyc_Tcutil_terr(_tmp180,_tmp17F,_tag_fat(_tmp8F,sizeof(void*),2U));});});
Cyc_Unify_explain_failure();}else{
# 348
Cyc_Tcpat_check_pat_regions(te,p,this_pattern.patvars);}
# 350
({struct Cyc_Core_Opt*_tmp181=({struct Cyc_Core_Opt*_tmp93=_cycalloc(sizeof(*_tmp93));_tmp93->v=this_pattern.patvars;_tmp93;});((struct Cyc_Absyn_Switch_clause*)scs->hd)->pat_vars=_tmp181;});
Cyc_Tcstmt_pattern_synth(((struct Cyc_Absyn_Switch_clause*)scs->hd)->loc,te,this_pattern,((struct Cyc_Absyn_Switch_clause*)scs->hd)->body,((struct Cyc_Absyn_Switch_clause*)scs->hd)->where_clause,1);}}}
# 354
Cyc_Tcpat_check_switch_exhaustive(s0->loc,te,scs0,dtp);
return;}}}case 15U: _LL1D: _tmp48=((struct Cyc_Absyn_TryCatch_s_Absyn_Raw_stmt_struct*)_tmp45)->f1;_tmp49=(struct Cyc_List_List**)&((struct Cyc_Absyn_TryCatch_s_Absyn_Raw_stmt_struct*)_tmp45)->f2;_tmp4A=(void**)&((struct Cyc_Absyn_TryCatch_s_Absyn_Raw_stmt_struct*)_tmp45)->f3;_LL1E: {struct Cyc_Absyn_Stmt*s=_tmp48;struct Cyc_List_List**scs0=_tmp49;void**dtp=_tmp4A;
# 362
({void*_tmp185=({void*_tmp184=(void*)({struct Cyc_Absyn_Seq_s_Absyn_Raw_stmt_struct*_tmp94=_cycalloc(sizeof(*_tmp94));_tmp94->tag=2U,({struct Cyc_Absyn_Stmt*_tmp183=Cyc_Absyn_new_stmt(s->r,s->loc);_tmp94->f1=_tmp183;}),({struct Cyc_Absyn_Stmt*_tmp182=Cyc_Absyn_skip_stmt(s->loc);_tmp94->f2=_tmp182;});_tmp94;});Cyc_Absyn_new_stmt(_tmp184,s->loc);})->r;s->r=_tmp185;});
# 364
Cyc_Tcstmt_tcStmt(te,s,1);{
# 367
struct _tuple2*def_v=({struct _tuple2*_tmpA4=_cycalloc(sizeof(*_tmpA4));_tmpA4->f1=Cyc_Absyn_Loc_n,({struct _fat_ptr*_tmp187=({struct _fat_ptr*_tmpA3=_cycalloc(sizeof(*_tmpA3));({struct _fat_ptr _tmp186=({const char*_tmpA2="exn";_tag_fat(_tmpA2,sizeof(char),4U);});*_tmpA3=_tmp186;});_tmpA3;});_tmpA4->f2=_tmp187;});_tmpA4;});
struct Cyc_Absyn_Vardecl*vd=Cyc_Absyn_new_vardecl(0U,def_v,Cyc_Absyn_void_type,0);
struct Cyc_Absyn_Pat*def_pat=Cyc_Absyn_new_pat((void*)({struct Cyc_Absyn_Var_p_Absyn_Raw_pat_struct*_tmpA1=_cycalloc(sizeof(*_tmpA1));_tmpA1->tag=1U,_tmpA1->f1=vd,({struct Cyc_Absyn_Pat*_tmp188=Cyc_Absyn_new_pat((void*)& Cyc_Absyn_Wild_p_val,0U);_tmpA1->f2=_tmp188;});_tmpA1;}),0U);
struct Cyc_Absyn_Stmt*def_stmt=
Cyc_Absyn_exp_stmt(Cyc_Absyn_new_exp((void*)({struct Cyc_Absyn_Throw_e_Absyn_Raw_exp_struct*_tmpA0=_cycalloc(sizeof(*_tmpA0));_tmpA0->tag=11U,({struct Cyc_Absyn_Exp*_tmp189=Cyc_Absyn_varb_exp((void*)({struct Cyc_Absyn_Local_b_Absyn_Binding_struct*_tmp9F=_cycalloc(sizeof(*_tmp9F));_tmp9F->tag=4U,_tmp9F->f1=vd;_tmp9F;}),0U);_tmpA0->f1=_tmp189;}),_tmpA0->f2=1;_tmpA0;}),0U),0U);
struct Cyc_Absyn_Switch_clause*def_clause=({struct Cyc_Absyn_Switch_clause*_tmp9E=_cycalloc(sizeof(*_tmp9E));_tmp9E->pattern=def_pat,_tmp9E->pat_vars=0,_tmp9E->where_clause=0,_tmp9E->body=def_stmt,_tmp9E->loc=0U;_tmp9E;});
({struct Cyc_List_List*_tmp18B=({struct Cyc_List_List*_tmp18A=*scs0;Cyc_List_append(_tmp18A,({struct Cyc_Absyn_Switch_clause*_tmp95[1U];_tmp95[0]=def_clause;Cyc_List_list(_tag_fat(_tmp95,sizeof(struct Cyc_Absyn_Switch_clause*),1U));}));});*scs0=_tmp18B;});{
# 379
void*exception_type=Cyc_Absyn_exn_type();
struct Cyc_Tcpat_TcPatResult this_pattern;
struct Cyc_Tcpat_TcPatResult*next_pattern=0;
int first_case=1;
{struct Cyc_List_List*scs=*scs0;for(0;scs != 0;scs=scs->tl){
if(first_case){
first_case=0;
this_pattern=Cyc_Tcpat_tcPat(te,((struct Cyc_Absyn_Switch_clause*)scs->hd)->pattern,& exception_type,0);}else{
# 388
this_pattern=*((struct Cyc_Tcpat_TcPatResult*)_check_null(next_pattern));}
# 390
if(scs->tl != 0){
next_pattern=({struct Cyc_Tcpat_TcPatResult*_tmp96=_cycalloc(sizeof(*_tmp96));({struct Cyc_Tcpat_TcPatResult _tmp18C=Cyc_Tcpat_tcPat(te,((struct Cyc_Absyn_Switch_clause*)((struct Cyc_List_List*)_check_null(scs->tl))->hd)->pattern,& exception_type,0);*_tmp96=_tmp18C;});_tmp96;});
if(next_pattern->tvars_and_bounds_opt != 0 &&(*next_pattern->tvars_and_bounds_opt).f2 != 0)
# 394
te=Cyc_Tcenv_clear_fallthru(te);else{
# 396
struct Cyc_List_List*vs=Cyc_List_imp_rev(Cyc_Tcutil_filter_nulls((Cyc_List_split(next_pattern->patvars)).f1));
struct Cyc_List_List*new_tvs=next_pattern->tvars_and_bounds_opt == 0?0:({
struct Cyc_List_List*(*_tmp18E)(struct Cyc_Absyn_Tvar*(*f)(struct _tuple17*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp97)(struct Cyc_Absyn_Tvar*(*f)(struct _tuple17*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_Absyn_Tvar*(*f)(struct _tuple17*),struct Cyc_List_List*x))Cyc_List_map;_tmp97;});struct Cyc_Absyn_Tvar*(*_tmp18D)(struct _tuple17*)=({struct Cyc_Absyn_Tvar*(*_tmp98)(struct _tuple17*)=(struct Cyc_Absyn_Tvar*(*)(struct _tuple17*))Cyc_Core_fst;_tmp98;});_tmp18E(_tmp18D,(*next_pattern->tvars_and_bounds_opt).f1);});
te=Cyc_Tcenv_set_fallthru(te,new_tvs,vs,(struct Cyc_Absyn_Switch_clause*)((struct Cyc_List_List*)_check_null(scs->tl))->hd);}}else{
# 402
te=Cyc_Tcenv_clear_fallthru(te);}{
# 404
struct Cyc_Absyn_Pat*p=((struct Cyc_Absyn_Switch_clause*)scs->hd)->pattern;
if(!Cyc_Unify_unify((void*)_check_null(p->topt),exception_type)){
({struct Cyc_String_pa_PrintArg_struct _tmp9B=({struct Cyc_String_pa_PrintArg_struct _tmp122;_tmp122.tag=0U,({
struct _fat_ptr _tmp18F=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(exception_type));_tmp122.f1=_tmp18F;});_tmp122;});struct Cyc_String_pa_PrintArg_struct _tmp9C=({struct Cyc_String_pa_PrintArg_struct _tmp121;_tmp121.tag=0U,({struct _fat_ptr _tmp190=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string((void*)_check_null(p->topt)));_tmp121.f1=_tmp190;});_tmp121;});void*_tmp99[2U];_tmp99[0]=& _tmp9B,_tmp99[1]=& _tmp9C;({unsigned _tmp192=((struct Cyc_Absyn_Switch_clause*)scs->hd)->loc;struct _fat_ptr _tmp191=({const char*_tmp9A="switch on type %s, but case expects type %s";_tag_fat(_tmp9A,sizeof(char),44U);});Cyc_Tcutil_terr(_tmp192,_tmp191,_tag_fat(_tmp99,sizeof(void*),2U));});});
Cyc_Unify_explain_failure();}else{
# 411
Cyc_Tcpat_check_pat_regions(te,p,this_pattern.patvars);}
# 413
({struct Cyc_Core_Opt*_tmp193=({struct Cyc_Core_Opt*_tmp9D=_cycalloc(sizeof(*_tmp9D));_tmp9D->v=this_pattern.patvars;_tmp9D;});((struct Cyc_Absyn_Switch_clause*)scs->hd)->pat_vars=_tmp193;});
Cyc_Tcstmt_pattern_synth(((struct Cyc_Absyn_Switch_clause*)scs->hd)->loc,te,this_pattern,((struct Cyc_Absyn_Switch_clause*)scs->hd)->body,((struct Cyc_Absyn_Switch_clause*)scs->hd)->where_clause,1);}}}
# 417
Cyc_Tcpat_check_catch_overlap(s0->loc,te,*scs0,dtp);
return;}}}default: _LL1F: _tmp46=((struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct*)_tmp45)->f1;_tmp47=((struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct*)_tmp45)->f2;_LL20: {struct Cyc_Absyn_Decl*d=_tmp46;struct Cyc_Absyn_Stmt*s=_tmp47;
# 421
struct _fat_ptr unimp_msg_part;
struct Cyc_Tcenv_Tenv*_tmpA5=new_block?Cyc_Tcenv_new_block(s0->loc,te): te;struct Cyc_Tcenv_Tenv*te=_tmpA5;
{void*_stmttmp5=d->r;void*_tmpA6=_stmttmp5;struct Cyc_List_List*_tmpA8;struct _tuple2*_tmpA7;struct Cyc_List_List*_tmpAA;struct _fat_ptr*_tmpA9;struct Cyc_Absyn_Fndecl*_tmpAB;struct Cyc_Absyn_Exp*_tmpAE;struct Cyc_Absyn_Vardecl*_tmpAD;struct Cyc_Absyn_Tvar*_tmpAC;struct Cyc_List_List*_tmpAF;void**_tmpB3;struct Cyc_Absyn_Exp**_tmpB2;struct Cyc_Core_Opt**_tmpB1;struct Cyc_Absyn_Pat*_tmpB0;struct Cyc_Absyn_Vardecl*_tmpB4;switch(*((int*)_tmpA6)){case 0U: _LL22: _tmpB4=((struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct*)_tmpA6)->f1;_LL23: {struct Cyc_Absyn_Vardecl*vd=_tmpB4;
# 425
struct Cyc_List_List*_tmpBC;void**_tmpBB;struct Cyc_Absyn_Exp*_tmpBA;void*_tmpB9;struct Cyc_Absyn_Tqual _tmpB8;struct _fat_ptr*_tmpB7;union Cyc_Absyn_Nmspace _tmpB6;enum Cyc_Absyn_Scope _tmpB5;_LL45: _tmpB5=vd->sc;_tmpB6=(vd->name)->f1;_tmpB7=(vd->name)->f2;_tmpB8=vd->tq;_tmpB9=vd->type;_tmpBA=vd->initializer;_tmpBB=(void**)& vd->rgn;_tmpBC=vd->attributes;_LL46: {enum Cyc_Absyn_Scope sc=_tmpB5;union Cyc_Absyn_Nmspace nsl=_tmpB6;struct _fat_ptr*x=_tmpB7;struct Cyc_Absyn_Tqual tq=_tmpB8;void*t=_tmpB9;struct Cyc_Absyn_Exp*initializer=_tmpBA;void**rgn_ptr=_tmpBB;struct Cyc_List_List*atts=_tmpBC;
void*curr_bl=Cyc_Tcenv_curr_rgn(te);
int is_local;
{enum Cyc_Absyn_Scope _tmpBD=sc;switch(_tmpBD){case Cyc_Absyn_Static: _LL48: _LL49:
 goto _LL4B;case Cyc_Absyn_Extern: _LL4A: _LL4B:
 goto _LL4D;case Cyc_Absyn_ExternC: _LL4C: _LL4D:
# 432
 vd->escapes=1;
is_local=0;goto _LL47;case Cyc_Absyn_Abstract: _LL4E: _LL4F:
# 435
({void*_tmpBE=0U;({unsigned _tmp195=d->loc;struct _fat_ptr _tmp194=({const char*_tmpBF="bad abstract scope for local variable";_tag_fat(_tmpBF,sizeof(char),38U);});Cyc_Tcutil_terr(_tmp195,_tmp194,_tag_fat(_tmpBE,sizeof(void*),0U));});});
goto _LL51;case Cyc_Absyn_Register: _LL50: _LL51:
 goto _LL53;case Cyc_Absyn_Public: _LL52: _LL53:
 goto _LL55;default: _LL54: _LL55:
 is_local=1;goto _LL47;}_LL47:;}
# 442
*rgn_ptr=is_local?curr_bl: Cyc_Absyn_heap_rgn_type;
# 444
{void*_stmttmp6=Cyc_Tcutil_compress(t);void*_tmpC0=_stmttmp6;unsigned _tmpC4;void*_tmpC3;struct Cyc_Absyn_Tqual _tmpC2;void*_tmpC1;switch(*((int*)_tmpC0)){case 5U: _LL57: if(is_local){_LL58:
# 447
 vd->escapes=1;
sc=3U;
is_local=0;
goto _LL56;}else{goto _LL5B;}case 4U: if((((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmpC0)->f1).num_elts == 0){_LL59: _tmpC1=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmpC0)->f1).elt_type;_tmpC2=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmpC0)->f1).tq;_tmpC3=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmpC0)->f1).zero_term;_tmpC4=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmpC0)->f1).zt_loc;if(vd->initializer != 0){_LL5A: {void*telt=_tmpC1;struct Cyc_Absyn_Tqual tq=_tmpC2;void*zt=_tmpC3;unsigned ztl=_tmpC4;
# 452
{void*_stmttmp7=((struct Cyc_Absyn_Exp*)_check_null(vd->initializer))->r;void*_tmpC5=_stmttmp7;struct Cyc_List_List*_tmpC6;struct Cyc_List_List*_tmpC7;struct Cyc_Absyn_Exp*_tmpC8;struct Cyc_Absyn_Exp*_tmpC9;struct _fat_ptr _tmpCA;struct _fat_ptr _tmpCB;switch(*((int*)_tmpC5)){case 0U: switch(((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmpC5)->f1).Wstring_c).tag){case 8U: _LL5E: _tmpCB=((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmpC5)->f1).String_c).val;_LL5F: {struct _fat_ptr s=_tmpCB;
# 454
t=({void*_tmp19A=({void*_tmp199=telt;struct Cyc_Absyn_Tqual _tmp198=tq;struct Cyc_Absyn_Exp*_tmp197=Cyc_Absyn_uint_exp(_get_fat_size(s,sizeof(char)),0U);void*_tmp196=zt;Cyc_Absyn_array_type(_tmp199,_tmp198,_tmp197,_tmp196,ztl);});vd->type=_tmp19A;});
goto _LL5D;}case 9U: _LL60: _tmpCA=((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmpC5)->f1).Wstring_c).val;_LL61: {struct _fat_ptr s=_tmpCA;
# 457
t=({void*_tmp19F=({void*_tmp19E=telt;struct Cyc_Absyn_Tqual _tmp19D=tq;struct Cyc_Absyn_Exp*_tmp19C=Cyc_Absyn_uint_exp(1U,0U);void*_tmp19B=zt;Cyc_Absyn_array_type(_tmp19E,_tmp19D,_tmp19C,_tmp19B,ztl);});vd->type=_tmp19F;});
goto _LL5D;}default: goto _LL6A;}case 27U: _LL62: _tmpC9=((struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct*)_tmpC5)->f2;_LL63: {struct Cyc_Absyn_Exp*e=_tmpC9;
_tmpC8=e;goto _LL65;}case 28U: _LL64: _tmpC8=((struct Cyc_Absyn_ComprehensionNoinit_e_Absyn_Raw_exp_struct*)_tmpC5)->f1;_LL65: {struct Cyc_Absyn_Exp*e=_tmpC8;
# 462
t=({void*_tmp1A0=Cyc_Absyn_array_type(telt,tq,e,zt,ztl);vd->type=_tmp1A0;});
goto _LL5D;}case 37U: _LL66: _tmpC7=((struct Cyc_Absyn_UnresolvedMem_e_Absyn_Raw_exp_struct*)_tmpC5)->f2;_LL67: {struct Cyc_List_List*es=_tmpC7;
_tmpC6=es;goto _LL69;}case 26U: _LL68: _tmpC6=((struct Cyc_Absyn_Array_e_Absyn_Raw_exp_struct*)_tmpC5)->f1;_LL69: {struct Cyc_List_List*es=_tmpC6;
# 466
t=({void*_tmp1A5=({void*_tmp1A4=telt;struct Cyc_Absyn_Tqual _tmp1A3=tq;struct Cyc_Absyn_Exp*_tmp1A2=Cyc_Absyn_uint_exp((unsigned)Cyc_List_length(es),0U);void*_tmp1A1=zt;Cyc_Absyn_array_type(_tmp1A4,_tmp1A3,_tmp1A2,_tmp1A1,ztl);});vd->type=_tmp1A5;});
goto _LL5D;}default: _LL6A: _LL6B:
 goto _LL5D;}_LL5D:;}
# 470
goto _LL56;}}else{goto _LL5B;}}else{goto _LL5B;}default: _LL5B: _LL5C:
 goto _LL56;}_LL56:;}{
# 477
struct Cyc_List_List*bound_vars=!is_local?0: Cyc_Tcenv_lookup_type_vars(te);
int allow_evars=!is_local?0: 1;
Cyc_Tctyp_check_type(s0->loc,te,bound_vars,& Cyc_Tcutil_tmk,allow_evars,1,t);
({int _tmp1A6=Cyc_Tcutil_extract_const_from_typedef(s0->loc,(vd->tq).print_const,t);(vd->tq).real_const=_tmp1A6;});{
# 482
struct Cyc_Tcenv_Tenv*te2=te;
if((int)sc == (int)3U ||(int)sc == (int)4U)
({void*_tmpCC=0U;({unsigned _tmp1A8=d->loc;struct _fat_ptr _tmp1A7=({const char*_tmpCD="extern declarations are not yet supported within functions";_tag_fat(_tmpCD,sizeof(char),59U);});Cyc_Tcutil_terr(_tmp1A8,_tmp1A7,_tag_fat(_tmpCC,sizeof(void*),0U));});});
# 483
if(initializer != 0){
# 486
Cyc_Tcexp_tcExpInitializer(te2,& t,initializer);
# 488
if(!is_local && !Cyc_Tcutil_is_const_exp(initializer))
({void*_tmpCE=0U;({unsigned _tmp1AA=d->loc;struct _fat_ptr _tmp1A9=({const char*_tmpCF="initializer for static variable needs to be a constant expression";_tag_fat(_tmpCF,sizeof(char),66U);});Cyc_Tcutil_terr(_tmp1AA,_tmp1A9,_tag_fat(_tmpCE,sizeof(void*),0U));});});
# 488
if(!
# 490
Cyc_Tcutil_coerce_assign(te2,initializer,t)){
struct _fat_ptr varstr=*x;
struct _fat_ptr tstr=Cyc_Absynpp_typ2string(t);
struct _fat_ptr estr=Cyc_Absynpp_typ2string((void*)_check_null(initializer->topt));
if(((_get_fat_size(varstr,sizeof(char))+ _get_fat_size(tstr,sizeof(char)))+ _get_fat_size(estr,sizeof(char)))+ (unsigned)50 < (unsigned)80)
({struct Cyc_String_pa_PrintArg_struct _tmpD2=({struct Cyc_String_pa_PrintArg_struct _tmp125;_tmp125.tag=0U,_tmp125.f1=(struct _fat_ptr)((struct _fat_ptr)varstr);_tmp125;});struct Cyc_String_pa_PrintArg_struct _tmpD3=({struct Cyc_String_pa_PrintArg_struct _tmp124;_tmp124.tag=0U,_tmp124.f1=(struct _fat_ptr)((struct _fat_ptr)tstr);_tmp124;});struct Cyc_String_pa_PrintArg_struct _tmpD4=({struct Cyc_String_pa_PrintArg_struct _tmp123;_tmp123.tag=0U,_tmp123.f1=(struct _fat_ptr)((struct _fat_ptr)estr);_tmp123;});void*_tmpD0[3U];_tmpD0[0]=& _tmpD2,_tmpD0[1]=& _tmpD3,_tmpD0[2]=& _tmpD4;({unsigned _tmp1AC=d->loc;struct _fat_ptr _tmp1AB=({const char*_tmpD1="%s was declared with type %s but initialized with type %s.";_tag_fat(_tmpD1,sizeof(char),59U);});Cyc_Tcutil_terr(_tmp1AC,_tmp1AB,_tag_fat(_tmpD0,sizeof(void*),3U));});});else{
# 497
if((_get_fat_size(varstr,sizeof(char))+ _get_fat_size(tstr,sizeof(char)))+ (unsigned)25 < (unsigned)80 &&
 _get_fat_size(estr,sizeof(char))+ (unsigned)25 < (unsigned)80)
({struct Cyc_String_pa_PrintArg_struct _tmpD7=({struct Cyc_String_pa_PrintArg_struct _tmp128;_tmp128.tag=0U,_tmp128.f1=(struct _fat_ptr)((struct _fat_ptr)varstr);_tmp128;});struct Cyc_String_pa_PrintArg_struct _tmpD8=({struct Cyc_String_pa_PrintArg_struct _tmp127;_tmp127.tag=0U,_tmp127.f1=(struct _fat_ptr)((struct _fat_ptr)tstr);_tmp127;});struct Cyc_String_pa_PrintArg_struct _tmpD9=({struct Cyc_String_pa_PrintArg_struct _tmp126;_tmp126.tag=0U,_tmp126.f1=(struct _fat_ptr)((struct _fat_ptr)estr);_tmp126;});void*_tmpD5[3U];_tmpD5[0]=& _tmpD7,_tmpD5[1]=& _tmpD8,_tmpD5[2]=& _tmpD9;({unsigned _tmp1AE=d->loc;struct _fat_ptr _tmp1AD=({const char*_tmpD6="%s was declared with type %s\n but initialized with type %s.";_tag_fat(_tmpD6,sizeof(char),60U);});Cyc_Tcutil_terr(_tmp1AE,_tmp1AD,_tag_fat(_tmpD5,sizeof(void*),3U));});});else{
# 502
({struct Cyc_String_pa_PrintArg_struct _tmpDC=({struct Cyc_String_pa_PrintArg_struct _tmp12B;_tmp12B.tag=0U,_tmp12B.f1=(struct _fat_ptr)((struct _fat_ptr)varstr);_tmp12B;});struct Cyc_String_pa_PrintArg_struct _tmpDD=({struct Cyc_String_pa_PrintArg_struct _tmp12A;_tmp12A.tag=0U,_tmp12A.f1=(struct _fat_ptr)((struct _fat_ptr)tstr);_tmp12A;});struct Cyc_String_pa_PrintArg_struct _tmpDE=({struct Cyc_String_pa_PrintArg_struct _tmp129;_tmp129.tag=0U,_tmp129.f1=(struct _fat_ptr)((struct _fat_ptr)estr);_tmp129;});void*_tmpDA[3U];_tmpDA[0]=& _tmpDC,_tmpDA[1]=& _tmpDD,_tmpDA[2]=& _tmpDE;({unsigned _tmp1B0=d->loc;struct _fat_ptr _tmp1AF=({const char*_tmpDB="%s declared with type \n%s\n but initialized with type \n%s.";_tag_fat(_tmpDB,sizeof(char),58U);});Cyc_Tcutil_terr(_tmp1B0,_tmp1AF,_tag_fat(_tmpDA,sizeof(void*),3U));});});}}
# 504
Cyc_Unify_unify(t,(void*)_check_null(initializer->topt));
Cyc_Unify_explain_failure();}}
# 483
Cyc_Tcstmt_tcStmt(te2,s,0);
# 509
return;}}}}case 2U: _LL24: _tmpB0=((struct Cyc_Absyn_Let_d_Absyn_Raw_decl_struct*)_tmpA6)->f1;_tmpB1=(struct Cyc_Core_Opt**)&((struct Cyc_Absyn_Let_d_Absyn_Raw_decl_struct*)_tmpA6)->f2;_tmpB2=(struct Cyc_Absyn_Exp**)&((struct Cyc_Absyn_Let_d_Absyn_Raw_decl_struct*)_tmpA6)->f3;_tmpB3=(void**)&((struct Cyc_Absyn_Let_d_Absyn_Raw_decl_struct*)_tmpA6)->f4;_LL25: {struct Cyc_Absyn_Pat*p=_tmpB0;struct Cyc_Core_Opt**vds=_tmpB1;struct Cyc_Absyn_Exp**exp=_tmpB2;void**dtp=_tmpB3;
# 516
struct Cyc_Absyn_Exp*e=*exp;
{void*_stmttmp8=p->r;void*_tmpDF=_stmttmp8;switch(*((int*)_tmpDF)){case 1U: _LL6D: _LL6E:
 goto _LL70;case 2U: _LL6F: _LL70:
 goto _LL72;case 15U: _LL71: _LL72:
 goto _LL6C;default: _LL73: _LL74:
# 522
 if(!Cyc_Tcstmt_is_var_exp(e)){
struct Cyc_Absyn_Vardecl*vd=({struct _tuple2*_tmp1B1=Cyc_Tcstmt_stmt_temp_var();Cyc_Absyn_new_vardecl(0U,_tmp1B1,Cyc_Absyn_new_evar(0,0),0);});
struct Cyc_Absyn_Stmt*s1=Cyc_Absyn_new_stmt(s0->r,s0->loc);
struct Cyc_Absyn_Decl*d=({struct Cyc_Absyn_Pat*_tmp1B5=({void*_tmp1B3=(void*)({struct Cyc_Absyn_Var_p_Absyn_Raw_pat_struct*_tmpE1=_cycalloc(sizeof(*_tmpE1));_tmpE1->tag=1U,_tmpE1->f1=vd,({struct Cyc_Absyn_Pat*_tmp1B2=Cyc_Absyn_new_pat((void*)& Cyc_Absyn_Wild_p_val,e->loc);_tmpE1->f2=_tmp1B2;});_tmpE1;});Cyc_Absyn_new_pat(_tmp1B3,e->loc);});struct Cyc_Absyn_Exp*_tmp1B4=e;Cyc_Absyn_let_decl(_tmp1B5,_tmp1B4,s0->loc);});
# 528
struct Cyc_Absyn_Stmt*s2=Cyc_Absyn_decl_stmt(d,s1,s0->loc);
# 530
s0->r=s2->r;
({struct Cyc_Absyn_Exp*_tmp1B7=({void*_tmp1B6=(void*)({struct Cyc_Absyn_Local_b_Absyn_Binding_struct*_tmpE0=_cycalloc(sizeof(*_tmpE0));_tmpE0->tag=4U,_tmpE0->f1=vd;_tmpE0;});Cyc_Absyn_varb_exp(_tmp1B6,e->loc);});*exp=_tmp1B7;});
Cyc_Tcstmt_tcStmt(te,s0,new_block);
return;}}_LL6C:;}
# 536
Cyc_Tcexp_tcExpInitializer(te,0,e);{
# 538
void*pat_type=(void*)_check_null(e->topt);
# 540
struct Cyc_Tcpat_TcPatResult pat_res=Cyc_Tcpat_tcPat(te,p,& pat_type,e);
({struct Cyc_Core_Opt*_tmp1B8=({struct Cyc_Core_Opt*_tmpE2=_cycalloc(sizeof(*_tmpE2));_tmpE2->v=pat_res.patvars;_tmpE2;});*vds=_tmp1B8;});
if(!({void*_tmp1B9=(void*)_check_null(p->topt);Cyc_Unify_unify(_tmp1B9,(void*)_check_null(e->topt));})&& !
Cyc_Tcutil_coerce_assign(te,e,(void*)_check_null(p->topt))){
({struct Cyc_String_pa_PrintArg_struct _tmpE5=({struct Cyc_String_pa_PrintArg_struct _tmp12D;_tmp12D.tag=0U,({
struct _fat_ptr _tmp1BA=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string((void*)_check_null(p->topt)));_tmp12D.f1=_tmp1BA;});_tmp12D;});struct Cyc_String_pa_PrintArg_struct _tmpE6=({struct Cyc_String_pa_PrintArg_struct _tmp12C;_tmp12C.tag=0U,({struct _fat_ptr _tmp1BB=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string((void*)_check_null(e->topt)));_tmp12C.f1=_tmp1BB;});_tmp12C;});void*_tmpE3[2U];_tmpE3[0]=& _tmpE5,_tmpE3[1]=& _tmpE6;({unsigned _tmp1BD=d->loc;struct _fat_ptr _tmp1BC=({const char*_tmpE4="pattern type %s does not match definition type %s";_tag_fat(_tmpE4,sizeof(char),50U);});Cyc_Tcutil_terr(_tmp1BD,_tmp1BC,_tag_fat(_tmpE3,sizeof(void*),2U));});});
({void*_tmp1BE=(void*)_check_null(p->topt);Cyc_Unify_unify(_tmp1BE,(void*)_check_null(e->topt));});
Cyc_Unify_explain_failure();}else{
# 550
Cyc_Tcpat_check_pat_regions(te,p,pat_res.patvars);}
Cyc_Tcpat_check_let_pat_exhaustive(p->loc,te,p,dtp);
Cyc_Tcstmt_pattern_synth(s0->loc,te,pat_res,s,0,0);
return;}}case 3U: _LL26: _tmpAF=((struct Cyc_Absyn_Letv_d_Absyn_Raw_decl_struct*)_tmpA6)->f1;_LL27: {struct Cyc_List_List*vds=_tmpAF;
# 556
void*curr_bl=Cyc_Tcenv_curr_rgn(te);
struct Cyc_Tcenv_Tenv*te2=te;
for(0;vds != 0;vds=vds->tl){
struct Cyc_Absyn_Vardecl*vd=(struct Cyc_Absyn_Vardecl*)vds->hd;
void**_tmpE9;void*_tmpE8;union Cyc_Absyn_Nmspace _tmpE7;_LL76: _tmpE7=(vd->name)->f1;_tmpE8=vd->type;_tmpE9=(void**)& vd->rgn;_LL77: {union Cyc_Absyn_Nmspace nsl=_tmpE7;void*t=_tmpE8;void**rgn_ptr=_tmpE9;
*rgn_ptr=curr_bl;
({unsigned _tmp1C1=s0->loc;struct Cyc_Tcenv_Tenv*_tmp1C0=te2;struct Cyc_List_List*_tmp1BF=Cyc_Tcenv_lookup_type_vars(te2);Cyc_Tctyp_check_type(_tmp1C1,_tmp1C0,_tmp1BF,& Cyc_Tcutil_tmk,1,1,t);});}}
# 564
Cyc_Tcstmt_tcStmt(te2,s,0);
return;}case 4U: _LL28: _tmpAC=((struct Cyc_Absyn_Region_d_Absyn_Raw_decl_struct*)_tmpA6)->f1;_tmpAD=((struct Cyc_Absyn_Region_d_Absyn_Raw_decl_struct*)_tmpA6)->f2;_tmpAE=((struct Cyc_Absyn_Region_d_Absyn_Raw_decl_struct*)_tmpA6)->f3;_LL29: {struct Cyc_Absyn_Tvar*tv=_tmpAC;struct Cyc_Absyn_Vardecl*vd=_tmpAD;struct Cyc_Absyn_Exp*open_exp_opt=_tmpAE;
# 570
({void*_tmp1C5=({void*_tmp1C4=(void*)({struct Cyc_Absyn_Seq_s_Absyn_Raw_stmt_struct*_tmpEA=_cycalloc(sizeof(*_tmpEA));_tmpEA->tag=2U,({struct Cyc_Absyn_Stmt*_tmp1C3=Cyc_Absyn_new_stmt(s->r,s->loc);_tmpEA->f1=_tmp1C3;}),({struct Cyc_Absyn_Stmt*_tmp1C2=Cyc_Absyn_skip_stmt(s->loc);_tmpEA->f2=_tmp1C2;});_tmpEA;});Cyc_Absyn_new_stmt(_tmp1C4,s->loc);})->r;s->r=_tmp1C5;});{
# 572
struct Cyc_Tcenv_Tenv*te2=te;
void**_tmpEC;void**_tmpEB;_LL79: _tmpEB=(void**)& vd->type;_tmpEC=(void**)& vd->rgn;_LL7A: {void**t=_tmpEB;void**rgn_ptr=_tmpEC;
void*curr_bl=Cyc_Tcenv_curr_rgn(te);
*rgn_ptr=curr_bl;{
void*rgn_typ;
if((unsigned)open_exp_opt){
# 580
struct _tuple2*drname=({struct _tuple2*_tmpFB=_cycalloc(sizeof(*_tmpFB));((_tmpFB->f1).Abs_n).tag=2U,({struct Cyc_List_List*_tmp1CA=({struct _fat_ptr*_tmpF6[1U];({struct _fat_ptr*_tmp1C9=({struct _fat_ptr*_tmpF8=_cycalloc(sizeof(*_tmpF8));({struct _fat_ptr _tmp1C8=({const char*_tmpF7="Core";_tag_fat(_tmpF7,sizeof(char),5U);});*_tmpF8=_tmp1C8;});_tmpF8;});_tmpF6[0]=_tmp1C9;});Cyc_List_list(_tag_fat(_tmpF6,sizeof(struct _fat_ptr*),1U));});((_tmpFB->f1).Abs_n).val=_tmp1CA;}),({
struct _fat_ptr*_tmp1C7=({struct _fat_ptr*_tmpFA=_cycalloc(sizeof(*_tmpFA));({struct _fat_ptr _tmp1C6=({const char*_tmpF9="DynamicRegion";_tag_fat(_tmpF9,sizeof(char),14U);});*_tmpFA=_tmp1C6;});_tmpFA;});_tmpFB->f2=_tmp1C7;});_tmpFB;});
void*outer_rgn=Cyc_Absyn_new_evar(({struct Cyc_Core_Opt*_tmpF5=_cycalloc(sizeof(*_tmpF5));_tmpF5->v=& Cyc_Tcutil_trk;_tmpF5;}),0);
rgn_typ=Cyc_Absyn_new_evar(({struct Cyc_Core_Opt*_tmpED=_cycalloc(sizeof(*_tmpED));_tmpED->v=& Cyc_Tcutil_rk;_tmpED;}),0);{
void*dr_type=({
union Cyc_Absyn_AggrInfo _tmp1CB=Cyc_Absyn_UnknownAggr(Cyc_Absyn_StructA,drname,0);Cyc_Absyn_aggr_type(_tmp1CB,({void*_tmpF4[1U];_tmpF4[0]=rgn_typ;Cyc_List_list(_tag_fat(_tmpF4,sizeof(void*),1U));}));});
void*exp_type=({void*_tmp1CE=dr_type;void*_tmp1CD=outer_rgn;struct Cyc_Absyn_Tqual _tmp1CC=
Cyc_Absyn_empty_tqual(0U);
# 586
Cyc_Absyn_at_type(_tmp1CE,_tmp1CD,_tmp1CC,Cyc_Absyn_false_type);});
# 588
({unsigned _tmp1D1=s0->loc;struct Cyc_Tcenv_Tenv*_tmp1D0=te;struct Cyc_List_List*_tmp1CF=Cyc_Tcenv_lookup_type_vars(te);Cyc_Tctyp_check_type(_tmp1D1,_tmp1D0,_tmp1CF,& Cyc_Tcutil_tmk,1,0,exp_type);});{
void*key_typ=Cyc_Tcexp_tcExp(te,& exp_type,open_exp_opt);
if(!Cyc_Unify_unify(exp_type,key_typ)&& !
Cyc_Tcutil_coerce_assign(te,open_exp_opt,exp_type))
({struct Cyc_String_pa_PrintArg_struct _tmpF0=({struct Cyc_String_pa_PrintArg_struct _tmp12F;_tmp12F.tag=0U,({
struct _fat_ptr _tmp1D2=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(exp_type));_tmp12F.f1=_tmp1D2;});_tmp12F;});struct Cyc_String_pa_PrintArg_struct _tmpF1=({struct Cyc_String_pa_PrintArg_struct _tmp12E;_tmp12E.tag=0U,({struct _fat_ptr _tmp1D3=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(key_typ));_tmp12E.f1=_tmp1D3;});_tmp12E;});void*_tmpEE[2U];_tmpEE[0]=& _tmpF0,_tmpEE[1]=& _tmpF1;({unsigned _tmp1D5=s0->loc;struct _fat_ptr _tmp1D4=({const char*_tmpEF="expected %s but found %s";_tag_fat(_tmpEF,sizeof(char),25U);});Cyc_Tcutil_terr(_tmp1D5,_tmp1D4,_tag_fat(_tmpEE,sizeof(void*),2U));});});
# 590
if(
# 594
!Cyc_Unify_unify(outer_rgn,Cyc_Absyn_unique_rgn_type)&& !
Cyc_Unify_unify(outer_rgn,Cyc_Absyn_refcnt_rgn_type))
({void*_tmpF2=0U;({unsigned _tmp1D7=s0->loc;struct _fat_ptr _tmp1D6=({const char*_tmpF3="open is only allowed on unique or reference-counted keys";_tag_fat(_tmpF3,sizeof(char),57U);});Cyc_Tcutil_terr(_tmp1D7,_tmp1D6,_tag_fat(_tmpF2,sizeof(void*),0U));});});
# 590
({void*_tmp1D8=
# 597
Cyc_Absyn_rgn_handle_type(rgn_typ);
# 590
*t=_tmp1D8;});
# 598
te2=Cyc_Tcenv_add_region(te2,rgn_typ,1);}}}else{
# 600
rgn_typ=(void*)({struct Cyc_Absyn_VarType_Absyn_Type_struct*_tmpFC=_cycalloc(sizeof(*_tmpFC));_tmpFC->tag=2U,_tmpFC->f1=tv;_tmpFC;});
te2=({unsigned _tmp1DA=s0->loc;struct Cyc_Tcenv_Tenv*_tmp1D9=te2;Cyc_Tcenv_add_type_vars(_tmp1DA,_tmp1D9,({struct Cyc_List_List*_tmpFD=_cycalloc(sizeof(*_tmpFD));_tmpFD->hd=tv,_tmpFD->tl=0;_tmpFD;}));});
te2=Cyc_Tcenv_add_region(te2,rgn_typ,0);}
# 604
({unsigned _tmp1DD=s0->loc;struct Cyc_Tcenv_Tenv*_tmp1DC=te;struct Cyc_List_List*_tmp1DB=Cyc_Tcenv_lookup_type_vars(te2);Cyc_Tctyp_check_type(_tmp1DD,_tmp1DC,_tmp1DB,& Cyc_Tcutil_bk,1,0,*t);});
if(!({void*_tmp1DE=*t;Cyc_Unify_unify(_tmp1DE,Cyc_Absyn_rgn_handle_type(rgn_typ));}))
({void*_tmpFE=0U;({int(*_tmp1E0)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp100)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp100;});struct _fat_ptr _tmp1DF=({const char*_tmpFF="region stmt: type of region handle is wrong!";_tag_fat(_tmpFF,sizeof(char),45U);});_tmp1E0(_tmp1DF,_tag_fat(_tmpFE,sizeof(void*),0U));});});
# 605
Cyc_Tcstmt_tcStmt(te2,s,0);
# 609
return;}}}}case 1U: _LL2A: _tmpAB=((struct Cyc_Absyn_Fn_d_Absyn_Raw_decl_struct*)_tmpA6)->f1;_LL2B: {struct Cyc_Absyn_Fndecl*fd=_tmpAB;
# 613
void*curr_bl=Cyc_Tcenv_curr_rgn(te);
if((int)fd->sc != (int)Cyc_Absyn_Public)
({void*_tmp101=0U;({unsigned _tmp1E2=d->loc;struct _fat_ptr _tmp1E1=({const char*_tmp102="bad storage class for inner function";_tag_fat(_tmp102,sizeof(char),37U);});Cyc_Tcutil_terr(_tmp1E2,_tmp1E1,_tag_fat(_tmp101,sizeof(void*),0U));});});
# 614
{
# 617
struct Cyc_List_List*atts=(fd->i).attributes;
# 614
for(0;(unsigned)atts;atts=atts->tl){
# 618
void*_stmttmp9=(void*)atts->hd;void*_tmp103=_stmttmp9;switch(*((int*)_tmp103)){case 7U: _LL7C: _LL7D:
# 620
 goto _LL7F;case 6U: _LL7E: _LL7F:
({struct Cyc_String_pa_PrintArg_struct _tmp106=({struct Cyc_String_pa_PrintArg_struct _tmp130;_tmp130.tag=0U,({
struct _fat_ptr _tmp1E3=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absyn_attribute2string((void*)atts->hd));_tmp130.f1=_tmp1E3;});_tmp130;});void*_tmp104[1U];_tmp104[0]=& _tmp106;({unsigned _tmp1E5=d->loc;struct _fat_ptr _tmp1E4=({const char*_tmp105="bad attribute %s for function";_tag_fat(_tmp105,sizeof(char),30U);});Cyc_Tcutil_terr(_tmp1E5,_tmp1E4,_tag_fat(_tmp104,sizeof(void*),1U));});});
goto _LL7B;default: _LL80: _LL81:
 goto _LL7B;}_LL7B:;}}
# 626
Cyc_Tctyp_check_fndecl_valid_type(d->loc,te,fd);{
void*t=Cyc_Tcutil_fndecl2type(fd);
({struct Cyc_List_List*_tmp1E6=Cyc_Tcutil_transfer_fn_type_atts(t,(fd->i).attributes);(fd->i).attributes=_tmp1E6;});{
struct Cyc_Absyn_Vardecl*vd=({struct Cyc_Absyn_Vardecl*_tmp107=_cycalloc(sizeof(*_tmp107));_tmp107->sc=fd->sc,_tmp107->name=fd->name,_tmp107->varloc=0U,({struct Cyc_Absyn_Tqual _tmp1EB=Cyc_Absyn_const_tqual(0U);_tmp107->tq=_tmp1EB;}),({
void*_tmp1EA=({void*_tmp1E9=t;void*_tmp1E8=curr_bl;struct Cyc_Absyn_Tqual _tmp1E7=Cyc_Absyn_empty_tqual(0U);Cyc_Absyn_at_type(_tmp1E9,_tmp1E8,_tmp1E7,Cyc_Absyn_false_type);});_tmp107->type=_tmp1EA;}),_tmp107->initializer=0,_tmp107->rgn=curr_bl,_tmp107->attributes=0,_tmp107->escapes=0;_tmp107;});
# 633
fd->fn_vardecl=vd;{
struct Cyc_Tcenv_Fenv*old_fenv=(struct Cyc_Tcenv_Fenv*)_check_null(te->le);
({struct Cyc_Tcenv_Fenv*_tmp1EC=Cyc_Tcenv_nested_fenv(d->loc,old_fenv,fd);te->le=_tmp1EC;});
Cyc_Tcstmt_tcStmt(te,fd->body,0);
Cyc_Tcenv_check_delayed_effects(te);
Cyc_Tcenv_check_delayed_constraints(te);
te->le=old_fenv;
Cyc_Tcstmt_tcStmt(te,s,0);
return;}}}}case 9U: _LL2C: _tmpA9=((struct Cyc_Absyn_Namespace_d_Absyn_Raw_decl_struct*)_tmpA6)->f1;_tmpAA=((struct Cyc_Absyn_Namespace_d_Absyn_Raw_decl_struct*)_tmpA6)->f2;_LL2D: {struct _fat_ptr*n=_tmpA9;struct Cyc_List_List*tds=_tmpAA;
unimp_msg_part=({const char*_tmp108="namespace";_tag_fat(_tmp108,sizeof(char),10U);});goto _LL21;}case 10U: _LL2E: _tmpA7=((struct Cyc_Absyn_Using_d_Absyn_Raw_decl_struct*)_tmpA6)->f1;_tmpA8=((struct Cyc_Absyn_Using_d_Absyn_Raw_decl_struct*)_tmpA6)->f2;_LL2F: {struct _tuple2*q=_tmpA7;struct Cyc_List_List*tds=_tmpA8;
unimp_msg_part=({const char*_tmp109="using";_tag_fat(_tmp109,sizeof(char),6U);});goto _LL21;}case 5U: _LL30: _LL31:
 unimp_msg_part=({const char*_tmp10A="type";_tag_fat(_tmp10A,sizeof(char),5U);});goto _LL21;case 6U: _LL32: _LL33:
 unimp_msg_part=({const char*_tmp10B="datatype";_tag_fat(_tmp10B,sizeof(char),9U);});goto _LL21;case 7U: _LL34: _LL35:
 unimp_msg_part=({const char*_tmp10C="enum";_tag_fat(_tmp10C,sizeof(char),5U);});goto _LL21;case 8U: _LL36: _LL37:
 unimp_msg_part=({const char*_tmp10D="typedef";_tag_fat(_tmp10D,sizeof(char),8U);});goto _LL21;case 11U: _LL38: _LL39:
 unimp_msg_part=({const char*_tmp10E="extern \"C\"";_tag_fat(_tmp10E,sizeof(char),11U);});goto _LL21;case 12U: _LL3A: _LL3B:
# 650
 unimp_msg_part=({const char*_tmp10F="extern \"C include\"";_tag_fat(_tmp10F,sizeof(char),19U);});goto _LL21;case 13U: _LL3C: _LL3D:
 unimp_msg_part=({const char*_tmp110="__cyclone_port_on__";_tag_fat(_tmp110,sizeof(char),20U);});goto _LL21;case 14U: _LL3E: _LL3F:
 unimp_msg_part=({const char*_tmp111="__cyclone_port_off__";_tag_fat(_tmp111,sizeof(char),21U);});goto _LL21;case 15U: _LL40: _LL41:
 unimp_msg_part=({const char*_tmp112="__tempest_on__";_tag_fat(_tmp112,sizeof(char),15U);});goto _LL21;default: _LL42: _LL43:
 unimp_msg_part=({const char*_tmp113="__tempest_off__";_tag_fat(_tmp113,sizeof(char),16U);});goto _LL21;}_LL21:;}
# 656
(int)_throw(({struct Cyc_String_pa_PrintArg_struct _tmp116=({struct Cyc_String_pa_PrintArg_struct _tmp131;_tmp131.tag=0U,_tmp131.f1=(struct _fat_ptr)((struct _fat_ptr)unimp_msg_part);_tmp131;});void*_tmp114[1U];_tmp114[0]=& _tmp116;({struct _fat_ptr _tmp1ED=({const char*_tmp115="tcStmt: nested %s declarations unimplemented";_tag_fat(_tmp115,sizeof(char),45U);});Cyc_Tcutil_impos(_tmp1ED,_tag_fat(_tmp114,sizeof(void*),1U));});}));}}_LL0:;}
