#include <setjmp.h>
/* This is a C header file to be used by the output of the Cyclone to
   C translator.  The corresponding definitions are in file lib/runtime_*.c */
#ifndef _CYC_INCLUDE_H_
#define _CYC_INCLUDE_H_

/* Need one of these per thread (see runtime_stack.c). The runtime maintains 
   a stack that contains either _handler_cons structs or _RegionHandle structs.
   The tag is 0 for a handler_cons and 1 for a region handle.  */
struct _RuntimeStack {
  int tag; 
  struct _RuntimeStack *next;
  void (*cleanup)(struct _RuntimeStack *frame);
};

#ifndef offsetof
/* should be size_t, but int is fine. */
#define offsetof(t,n) ((int)(&(((t *)0)->n)))
#endif

/* Fat pointers */
struct _fat_ptr {
  unsigned char *curr; 
  unsigned char *base; 
  unsigned char *last_plus_one; 
};  

/* Discriminated Unions */
struct _xtunion_struct { char *tag; };

/* Regions */
struct _RegionPage
#ifdef CYC_REGION_PROFILE
{ unsigned total_bytes;
  unsigned free_bytes;
  /* MWH: wish we didn't have to include the stuff below ... */
  struct _RegionPage *next;
  char data[1];
}
#endif
; // abstract -- defined in runtime_memory.c
struct _RegionHandle {
  struct _RuntimeStack s;
  struct _RegionPage *curr;
  char               *offset;
  char               *last_plus_one;
  struct _DynRegionHandle *sub_regions;
#ifdef CYC_REGION_PROFILE
  const char         *name;
#else
  unsigned used_bytes;
  unsigned wasted_bytes;
#endif
};
struct _DynRegionFrame {
  struct _RuntimeStack s;
  struct _DynRegionHandle *x;
};
// A dynamic region is just a region handle.  The wrapper struct is for type
// abstraction.
struct Cyc_Core_DynamicRegion {
  struct _RegionHandle h;
};

struct _RegionHandle _new_region(const char*);
void* _region_malloc(struct _RegionHandle*, unsigned);
void* _region_calloc(struct _RegionHandle*, unsigned t, unsigned n);
void   _free_region(struct _RegionHandle*);
struct _RegionHandle*_open_dynregion(struct _DynRegionFrame*,struct _DynRegionHandle*);
void   _pop_dynregion();

/* Exceptions */
struct _handler_cons {
  struct _RuntimeStack s;
  jmp_buf handler;
};
void _push_handler(struct _handler_cons *);
void _push_region(struct _RegionHandle *);
void _npop_handler(int);
void _pop_handler();
void _pop_region();

#ifndef _throw
void* _throw_null_fn(const char*,unsigned);
void* _throw_arraybounds_fn(const char*,unsigned);
void* _throw_badalloc_fn(const char*,unsigned);
void* _throw_match_fn(const char*,unsigned);
void* _throw_fn(void*,const char*,unsigned);
void* _rethrow(void*);
#define _throw_null() (_throw_null_fn(__FILE__,__LINE__))
#define _throw_arraybounds() (_throw_arraybounds_fn(__FILE__,__LINE__))
#define _throw_badalloc() (_throw_badalloc_fn(__FILE__,__LINE__))
#define _throw_match() (_throw_match_fn(__FILE__,__LINE__))
#define _throw(e) (_throw_fn((e),__FILE__,__LINE__))
#endif

struct _xtunion_struct* Cyc_Core_get_exn_thrown();
/* Built-in Exceptions */
struct Cyc_Null_Exception_exn_struct { char *tag; };
struct Cyc_Array_bounds_exn_struct { char *tag; };
struct Cyc_Match_Exception_exn_struct { char *tag; };
struct Cyc_Bad_alloc_exn_struct { char *tag; };
extern char Cyc_Null_Exception[];
extern char Cyc_Array_bounds[];
extern char Cyc_Match_Exception[];
extern char Cyc_Bad_alloc[];

/* Built-in Run-time Checks and company */
#ifdef CYC_ANSI_OUTPUT
#define _INLINE  
#else
#define _INLINE inline
#endif

#ifdef NO_CYC_NULL_CHECKS
#define _check_null(ptr) (ptr)
#else
#define _check_null(ptr) \
  ({ void*_cks_null = (void*)(ptr); \
     if (!_cks_null) _throw_null(); \
     _cks_null; })
#endif

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index)\
   (((char*)ptr) + (elt_sz)*(index))
#ifdef NO_CYC_NULL_CHECKS
#define _check_known_subscript_null _check_known_subscript_notnull
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr);\
  int _index = (index);\
  if (!_cks_ptr) _throw_null(); \
  _cks_ptr + (elt_sz)*_index; })
#endif
#define _zero_arr_plus_fn(orig_x,orig_sz,orig_i,f,l) ((orig_x)+(orig_i))
#define _zero_arr_plus_char_fn _zero_arr_plus_fn
#define _zero_arr_plus_short_fn _zero_arr_plus_fn
#define _zero_arr_plus_int_fn _zero_arr_plus_fn
#define _zero_arr_plus_float_fn _zero_arr_plus_fn
#define _zero_arr_plus_double_fn _zero_arr_plus_fn
#define _zero_arr_plus_longdouble_fn _zero_arr_plus_fn
#define _zero_arr_plus_voidstar_fn _zero_arr_plus_fn
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (!_cks_ptr) _throw_null(); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })

/* _zero_arr_plus_*_fn(x,sz,i,filename,lineno) adds i to zero-terminated ptr
   x that has at least sz elements */
char* _zero_arr_plus_char_fn(char*,unsigned,int,const char*,unsigned);
short* _zero_arr_plus_short_fn(short*,unsigned,int,const char*,unsigned);
int* _zero_arr_plus_int_fn(int*,unsigned,int,const char*,unsigned);
float* _zero_arr_plus_float_fn(float*,unsigned,int,const char*,unsigned);
double* _zero_arr_plus_double_fn(double*,unsigned,int,const char*,unsigned);
long double* _zero_arr_plus_longdouble_fn(long double*,unsigned,int,const char*, unsigned);
void** _zero_arr_plus_voidstar_fn(void**,unsigned,int,const char*,unsigned);
#endif

/* _get_zero_arr_size_*(x,sz) returns the number of elements in a
   zero-terminated array that is NULL or has at least sz elements */
int _get_zero_arr_size_char(const char*,unsigned);
int _get_zero_arr_size_short(const short*,unsigned);
int _get_zero_arr_size_int(const int*,unsigned);
int _get_zero_arr_size_float(const float*,unsigned);
int _get_zero_arr_size_double(const double*,unsigned);
int _get_zero_arr_size_longdouble(const long double*,unsigned);
int _get_zero_arr_size_voidstar(const void**,unsigned);

/* _zero_arr_inplace_plus_*_fn(x,i,filename,lineno) sets
   zero-terminated pointer *x to *x + i */
char* _zero_arr_inplace_plus_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_short_fn(short**,int,const char*,unsigned);
int* _zero_arr_inplace_plus_int(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_longdouble_fn(long double**,int,const char*,unsigned);
void** _zero_arr_inplace_plus_voidstar_fn(void***,int,const char*,unsigned);
/* like the previous functions, but does post-addition (as in e++) */
char* _zero_arr_inplace_plus_post_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_post_short_fn(short**x,int,const char*,unsigned);
int* _zero_arr_inplace_plus_post_int_fn(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_post_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_post_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_post_longdouble_fn(long double**,int,const char *,unsigned);
void** _zero_arr_inplace_plus_post_voidstar_fn(void***,int,const char*,unsigned);
#define _zero_arr_plus_char(x,s,i) \
  (_zero_arr_plus_char_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_short(x,s,i) \
  (_zero_arr_plus_short_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_int(x,s,i) \
  (_zero_arr_plus_int_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_float(x,s,i) \
  (_zero_arr_plus_float_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_double(x,s,i) \
  (_zero_arr_plus_double_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_longdouble(x,s,i) \
  (_zero_arr_plus_longdouble_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_voidstar(x,s,i) \
  (_zero_arr_plus_voidstar_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_inplace_plus_char(x,i) \
  _zero_arr_inplace_plus_char_fn((char **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_short(x,i) \
  _zero_arr_inplace_plus_short_fn((short **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_int(x,i) \
  _zero_arr_inplace_plus_int_fn((int **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_float(x,i) \
  _zero_arr_inplace_plus_float_fn((float **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_double(x,i) \
  _zero_arr_inplace_plus_double_fn((double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_longdouble(x,i) \
  _zero_arr_inplace_plus_longdouble_fn((long double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_voidstar(x,i) \
  _zero_arr_inplace_plus_voidstar_fn((void ***)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_char(x,i) \
  _zero_arr_inplace_plus_post_char_fn((char **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_short(x,i) \
  _zero_arr_inplace_plus_post_short_fn((short **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_int(x,i) \
  _zero_arr_inplace_plus_post_int_fn((int **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_float(x,i) \
  _zero_arr_inplace_plus_post_float_fn((float **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_double(x,i) \
  _zero_arr_inplace_plus_post_double_fn((double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_longdouble(x,i) \
  _zero_arr_inplace_plus_post_longdouble_fn((long double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_voidstar(x,i) \
  _zero_arr_inplace_plus_post_voidstar_fn((void***)(x),(i),__FILE__,__LINE__)

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_fat_subscript(arr,elt_sz,index) ((arr).curr + (elt_sz) * (index))
#define _untag_fat_ptr(arr,elt_sz,num_elts) ((arr).curr)
#else
#define _check_fat_subscript(arr,elt_sz,index) ({ \
  struct _fat_ptr _cus_arr = (arr); \
  unsigned char *_cus_ans = _cus_arr.curr + (elt_sz) * (index); \
  /* JGM: not needed! if (!_cus_arr.base) _throw_null();*/ \
  if (_cus_ans < _cus_arr.base || _cus_ans >= _cus_arr.last_plus_one) \
    _throw_arraybounds(); \
  _cus_ans; })
#define _untag_fat_ptr(arr,elt_sz,num_elts) ({ \
  struct _fat_ptr _arr = (arr); \
  unsigned char *_curr = _arr.curr; \
  if ((_curr < _arr.base || _curr + (elt_sz) * (num_elts) > _arr.last_plus_one) &&\
      _curr != (unsigned char *)0) \
    _throw_arraybounds(); \
  _curr; })
#endif

#define _tag_fat(tcurr,elt_sz,num_elts) ({ \
  struct _fat_ptr _ans; \
  unsigned _num_elts = (num_elts);\
  _ans.base = _ans.curr = (void*)(tcurr); \
  /* JGM: if we're tagging NULL, ignore num_elts */ \
  _ans.last_plus_one = _ans.base ? (_ans.base + (elt_sz) * _num_elts) : 0; \
  _ans; })

#define _get_fat_size(arr,elt_sz) \
  ({struct _fat_ptr _arr = (arr); \
    unsigned char *_arr_curr=_arr.curr; \
    unsigned char *_arr_last=_arr.last_plus_one; \
    (_arr_curr < _arr.base || _arr_curr >= _arr_last) ? 0 : \
    ((_arr_last - _arr_curr) / (elt_sz));})

#define _fat_ptr_plus(arr,elt_sz,change) ({ \
  struct _fat_ptr _ans = (arr); \
  int _change = (change);\
  _ans.curr += (elt_sz) * _change;\
  _ans; })
#define _fat_ptr_inplace_plus(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  _arr_ptr->curr += (elt_sz) * (change);\
  *_arr_ptr; })
#define _fat_ptr_inplace_plus_post(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  struct _fat_ptr _ans = *_arr_ptr; \
  _arr_ptr->curr += (elt_sz) * (change);\
  _ans; })

//Not a macro since initialization order matters. Defined in runtime_zeroterm.c.
struct _fat_ptr _fat_ptr_decrease_size(struct _fat_ptr,unsigned sz,unsigned numelts);

/* Allocation */
void* GC_malloc(int);
void* GC_malloc_atomic(int);
void* GC_calloc(unsigned,unsigned);
void* GC_calloc_atomic(unsigned,unsigned);
// bound the allocation size to be < MAX_ALLOC_SIZE. See macros below for usage.
#define MAX_MALLOC_SIZE (1 << 28)
void* _bounded_GC_malloc(int,const char*,int);
void* _bounded_GC_malloc_atomic(int,const char*,int);
void* _bounded_GC_calloc(unsigned,unsigned,const char*,int);
void* _bounded_GC_calloc_atomic(unsigned,unsigned,const char*,int);
/* these macros are overridden below ifdef CYC_REGION_PROFILE */
#ifndef CYC_REGION_PROFILE
#define _cycalloc(n) _bounded_GC_malloc(n,__FILE__,__LINE__)
#define _cycalloc_atomic(n) _bounded_GC_malloc_atomic(n,__FILE__,__LINE__)
#define _cyccalloc(n,s) _bounded_GC_calloc(n,s,__FILE__,__LINE__)
#define _cyccalloc_atomic(n,s) _bounded_GC_calloc_atomic(n,s,__FILE__,__LINE__)
#endif

static _INLINE unsigned int _check_times(unsigned x, unsigned y) {
  unsigned long long whole_ans = 
    ((unsigned long long) x)*((unsigned long long)y);
  unsigned word_ans = (unsigned)whole_ans;
  if(word_ans < whole_ans || word_ans > MAX_MALLOC_SIZE)
    _throw_badalloc();
  return word_ans;
}

#define _CYC_MAX_REGION_CONST 2
#define _CYC_MIN_ALIGNMENT (sizeof(double))

#ifdef CYC_REGION_PROFILE
extern int rgn_total_bytes;
#endif

static _INLINE void *_fast_region_malloc(struct _RegionHandle *r, unsigned orig_s) {  
  if (r > (struct _RegionHandle *)_CYC_MAX_REGION_CONST && r->curr != 0) { 
#ifdef CYC_NOALIGN
    unsigned s =  orig_s;
#else
    unsigned s =  (orig_s + _CYC_MIN_ALIGNMENT - 1) & (~(_CYC_MIN_ALIGNMENT -1)); 
#endif
    char *result; 
    result = r->offset; 
    if (s <= (r->last_plus_one - result)) {
      r->offset = result + s; 
#ifdef CYC_REGION_PROFILE
    r->curr->free_bytes = r->curr->free_bytes - s;
    rgn_total_bytes += s;
#endif
      return result;
    }
  } 
  return _region_malloc(r,orig_s); 
}

#ifdef CYC_REGION_PROFILE
/* see macros below for usage. defined in runtime_memory.c */
void* _profile_GC_malloc(int,const char*,const char*,int);
void* _profile_GC_malloc_atomic(int,const char*,const char*,int);
void* _profile_GC_calloc(unsigned,unsigned,const char*,const char*,int);
void* _profile_GC_calloc_atomic(unsigned,unsigned,const char*,const char*,int);
void* _profile_region_malloc(struct _RegionHandle*,unsigned,const char*,const char*,int);
void* _profile_region_calloc(struct _RegionHandle*,unsigned,unsigned,const char *,const char*,int);
struct _RegionHandle _profile_new_region(const char*,const char*,const char*,int);
void _profile_free_region(struct _RegionHandle*,const char*,const char*,int);
#ifndef RUNTIME_CYC
#define _new_region(n) _profile_new_region(n,__FILE__,__FUNCTION__,__LINE__)
#define _free_region(r) _profile_free_region(r,__FILE__,__FUNCTION__,__LINE__)
#define _region_malloc(rh,n) _profile_region_malloc(rh,n,__FILE__,__FUNCTION__,__LINE__)
#define _region_calloc(rh,n,t) _profile_region_calloc(rh,n,t,__FILE__,__FUNCTION__,__LINE__)
#  endif
#define _cycalloc(n) _profile_GC_malloc(n,__FILE__,__FUNCTION__,__LINE__)
#define _cycalloc_atomic(n) _profile_GC_malloc_atomic(n,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc(n,s) _profile_GC_calloc(n,s,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc_atomic(n,s) _profile_GC_calloc_atomic(n,s,__FILE__,__FUNCTION__,__LINE__)
#endif
#endif
 struct Cyc_Core_Opt{void*v;};extern char Cyc_Core_Invalid_argument[17U];struct Cyc_Core_Invalid_argument_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Failure[8U];struct Cyc_Core_Failure_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Impossible[11U];struct Cyc_Core_Impossible_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Not_found[10U];struct Cyc_Core_Not_found_exn_struct{char*tag;};extern char Cyc_Core_Unreachable[12U];struct Cyc_Core_Unreachable_exn_struct{char*tag;struct _fat_ptr f1;};
# 165 "core.h"
extern struct _RegionHandle*Cyc_Core_heap_region;
# 168
extern struct _RegionHandle*Cyc_Core_unique_region;
# 175
void Cyc_Core_ufree(void*ptr);struct Cyc_Core_DynamicRegion;struct Cyc_Core_NewDynamicRegion{struct Cyc_Core_DynamicRegion*key;};struct Cyc_Core_ThinRes{void*arr;unsigned nelts;};struct Cyc_Xarray_Xarray{struct _fat_ptr elmts;int num_elmts;};
# 40 "xarray.h"
int Cyc_Xarray_length(struct Cyc_Xarray_Xarray*);
# 51
struct Cyc_Xarray_Xarray*Cyc_Xarray_rcreate(struct _RegionHandle*,int,void*);
# 57
struct Cyc_Xarray_Xarray*Cyc_Xarray_rcreate_empty(struct _RegionHandle*);
# 63
struct Cyc_Xarray_Xarray*Cyc_Xarray_rsingleton(struct _RegionHandle*,int,void*);
# 66
void Cyc_Xarray_add(struct Cyc_Xarray_Xarray*,void*);
# 75
struct _fat_ptr Cyc_Xarray_rto_array(struct _RegionHandle*,struct Cyc_Xarray_Xarray*);
# 81
struct Cyc_Xarray_Xarray*Cyc_Xarray_rfrom_array(struct _RegionHandle*,struct _fat_ptr arr);
# 89
struct Cyc_Xarray_Xarray*Cyc_Xarray_rappend(struct _RegionHandle*,struct Cyc_Xarray_Xarray*,struct Cyc_Xarray_Xarray*);
# 110
struct Cyc_Xarray_Xarray*Cyc_Xarray_rmap(struct _RegionHandle*,void*(*f)(void*),struct Cyc_Xarray_Xarray*);
# 117
struct Cyc_Xarray_Xarray*Cyc_Xarray_rmap_c(struct _RegionHandle*,void*(*f)(void*,void*),void*,struct Cyc_Xarray_Xarray*);
# 25 "xarray.cyc"
int Cyc_Xarray_length(struct Cyc_Xarray_Xarray*xarr){
return xarr->num_elmts;}
# 25
void*Cyc_Xarray_get(struct Cyc_Xarray_Xarray*xarr,int i){
# 30
if(i < 0 || i >= xarr->num_elmts)
(int)_throw((void*)({struct Cyc_Core_Invalid_argument_exn_struct*_tmp1=_cycalloc(sizeof(*_tmp1));_tmp1->tag=Cyc_Core_Invalid_argument,({struct _fat_ptr _tmp32=({const char*_tmp0="Xarray::get: bad index";_tag_fat(_tmp0,sizeof(char),23U);});_tmp1->f1=_tmp32;});_tmp1;}));
# 30
return*((void**)_check_fat_subscript(xarr->elmts,sizeof(void*),i));}
# 25
void Cyc_Xarray_set(struct Cyc_Xarray_Xarray*xarr,int i,void*a){
# 36
if(i < 0 || i >= xarr->num_elmts)
(int)_throw((void*)({struct Cyc_Core_Invalid_argument_exn_struct*_tmp3=_cycalloc(sizeof(*_tmp3));_tmp3->tag=Cyc_Core_Invalid_argument,({struct _fat_ptr _tmp33=({const char*_tmp2="Xarray::set: bad index";_tag_fat(_tmp2,sizeof(char),23U);});_tmp3->f1=_tmp33;});_tmp3;}));
# 36
*((void**)_check_fat_subscript(xarr->elmts,sizeof(void*),i))=a;}
# 25
struct Cyc_Xarray_Xarray*Cyc_Xarray_rcreate_empty(struct _RegionHandle*r){
# 42
struct _fat_ptr x=_tag_fat(0,0,0);
return({struct Cyc_Xarray_Xarray*_tmp4=_region_malloc(r,sizeof(*_tmp4));_tmp4->elmts=x,_tmp4->num_elmts=0;_tmp4;});}
# 25
struct Cyc_Xarray_Xarray*Cyc_Xarray_create_empty(){
# 45
return Cyc_Xarray_rcreate_empty(Cyc_Core_heap_region);}
# 25
struct Cyc_Xarray_Xarray*Cyc_Xarray_rcreate(struct _RegionHandle*r,int len,void*a){
# 49
if(len < 0)
(int)_throw((void*)({struct Cyc_Core_Invalid_argument_exn_struct*_tmp6=_cycalloc(sizeof(*_tmp6));_tmp6->tag=Cyc_Core_Invalid_argument,({struct _fat_ptr _tmp34=({const char*_tmp5="xarrays must have a non-negative size buffer";_tag_fat(_tmp5,sizeof(char),45U);});_tmp6->f1=_tmp34;});_tmp6;}));
# 49
return({struct Cyc_Xarray_Xarray*_tmp9=_region_malloc(r,sizeof(*_tmp9));
# 51
({struct _fat_ptr _tmp36=({unsigned _tmp8=(unsigned)len;void**_tmp7=({struct _RegionHandle*_tmp35=Cyc_Core_unique_region;_region_malloc(_tmp35,_check_times(_tmp8,sizeof(void*)));});({{unsigned _tmp2A=(unsigned)len;unsigned i;for(i=0;i < _tmp2A;++ i){_tmp7[i]=a;}}0;});_tag_fat(_tmp7,sizeof(void*),_tmp8);});_tmp9->elmts=_tmp36;}),_tmp9->num_elmts=0;_tmp9;});}
# 25
struct Cyc_Xarray_Xarray*Cyc_Xarray_create(int len,void*a){
# 55
return Cyc_Xarray_rcreate(Cyc_Core_heap_region,len,a);}
# 25
struct Cyc_Xarray_Xarray*Cyc_Xarray_rsingleton(struct _RegionHandle*r,int len,void*a){
# 59
if(len < 1)(int)_throw((void*)({struct Cyc_Core_Invalid_argument_exn_struct*_tmpB=_cycalloc(sizeof(*_tmpB));_tmpB->tag=Cyc_Core_Invalid_argument,({struct _fat_ptr _tmp37=({const char*_tmpA="singleton xarray must have size >=1";_tag_fat(_tmpA,sizeof(char),36U);});_tmpB->f1=_tmp37;});_tmpB;}));{struct Cyc_Xarray_Xarray*x=
Cyc_Xarray_rcreate(r,len,a);
x->num_elmts=1;
return x;}}
# 25
struct Cyc_Xarray_Xarray*Cyc_Xarray_singleton(int len,void*a){
# 66
return Cyc_Xarray_rsingleton(Cyc_Core_heap_region,len,a);}
# 25
void Cyc_Xarray_add(struct Cyc_Xarray_Xarray*xarr,void*a){
# 70
if((unsigned)xarr->num_elmts == _get_fat_size(xarr->elmts,sizeof(void*))){
if(xarr->num_elmts == 0)
({struct _fat_ptr _tmp39=_tag_fat(({unsigned _tmpD=10U;void**_tmpC=({struct _RegionHandle*_tmp38=Cyc_Core_unique_region;_region_malloc(_tmp38,_check_times(_tmpD,sizeof(void*)));});({{unsigned _tmp2B=10U;unsigned i;for(i=0;i < _tmp2B;++ i){_tmpC[i]=a;}}0;});_tmpC;}),sizeof(void*),10U);xarr->elmts=_tmp39;});else{
# 74
struct _fat_ptr newarr=({unsigned _tmp12=(unsigned)(xarr->num_elmts * 2);void**_tmp11=({struct _RegionHandle*_tmp3A=Cyc_Core_unique_region;_region_malloc(_tmp3A,_check_times(_tmp12,sizeof(void*)));});({{unsigned _tmp2C=(unsigned)(xarr->num_elmts * 2);unsigned i;for(i=0;i < _tmp2C;++ i){_tmp11[i]=*((void**)_check_fat_subscript(xarr->elmts,sizeof(void*),0));}}0;});_tag_fat(_tmp11,sizeof(void*),_tmp12);});
{int i=1;for(0;i < xarr->num_elmts;++ i){
({void*_tmp3B=*((void**)_check_fat_subscript(xarr->elmts,sizeof(void*),i));*((void**)_check_fat_subscript(newarr,sizeof(void*),i))=_tmp3B;});}}
({struct _fat_ptr _tmpE=xarr->elmts;struct _fat_ptr _tmpF=newarr;xarr->elmts=_tmpF;newarr=_tmpE;});
({void(*_tmp3C)(void**ptr)=({void(*_tmp10)(void**ptr)=(void(*)(void**ptr))Cyc_Core_ufree;_tmp10;});_tmp3C((void**)_untag_fat_ptr(newarr,sizeof(void*),1U));});}}
# 70
*((void**)_check_fat_subscript(xarr->elmts,sizeof(void*),xarr->num_elmts ++))=a;}
# 25
int Cyc_Xarray_add_ind(struct Cyc_Xarray_Xarray*xarr,void*a){
# 85
Cyc_Xarray_add(xarr,a);
return xarr->num_elmts - 1;}
# 25
struct _fat_ptr Cyc_Xarray_rto_array(struct _RegionHandle*r,struct Cyc_Xarray_Xarray*xarr){
# 90
if(xarr->num_elmts == 0)
return _tag_fat(({unsigned _tmp14=0;void**_tmp13=({struct _RegionHandle*_tmp3D=r;_region_malloc(_tmp3D,_check_times(_tmp14,sizeof(void*)));});*_tmp13=0;_tmp13;}),sizeof(void*),0U);{
# 90
struct _fat_ptr ans=({unsigned _tmp16=(unsigned)xarr->num_elmts;void**_tmp15=({struct _RegionHandle*_tmp3E=r;_region_malloc(_tmp3E,_check_times(_tmp16,sizeof(void*)));});({{unsigned _tmp2D=(unsigned)xarr->num_elmts;unsigned i;for(i=0;i < _tmp2D;++ i){_tmp15[i]=*((void**)_check_fat_subscript(xarr->elmts,sizeof(void*),(int)i));}}0;});_tag_fat(_tmp15,sizeof(void*),_tmp16);});
# 93
return ans;}}
# 25
struct _fat_ptr Cyc_Xarray_to_array(struct Cyc_Xarray_Xarray*xarr){
# 97
return Cyc_Xarray_rto_array(Cyc_Core_heap_region,xarr);}
# 25
struct Cyc_Xarray_Xarray*Cyc_Xarray_rfrom_array(struct _RegionHandle*r,struct _fat_ptr arr){
# 101
if(_get_fat_size(arr,sizeof(void*))== (unsigned)0)
return Cyc_Xarray_rcreate_empty(r);{
# 101
struct Cyc_Xarray_Xarray*ans=({struct Cyc_Xarray_Xarray*_tmp19=_region_malloc(r,sizeof(*_tmp19));
# 103
({
struct _fat_ptr _tmp40=({unsigned _tmp18=_get_fat_size(arr,sizeof(void*));void**_tmp17=({struct _RegionHandle*_tmp3F=Cyc_Core_unique_region;_region_malloc(_tmp3F,_check_times(_tmp18,sizeof(void*)));});({{unsigned _tmp2E=_get_fat_size(arr,sizeof(void*));unsigned i;for(i=0;i < _tmp2E;++ i){_tmp17[i]=((void**)arr.curr)[(int)i];}}0;});_tag_fat(_tmp17,sizeof(void*),_tmp18);});_tmp19->elmts=_tmp40;}),_tmp19->num_elmts=(int)
_get_fat_size(arr,sizeof(void*));_tmp19;});
return ans;}}
# 25
struct Cyc_Xarray_Xarray*Cyc_Xarray_from_array(struct _fat_ptr arr){
# 110
return Cyc_Xarray_rfrom_array(Cyc_Core_heap_region,arr);}
# 25
struct Cyc_Xarray_Xarray*Cyc_Xarray_rappend(struct _RegionHandle*r,struct Cyc_Xarray_Xarray*xarr1,struct Cyc_Xarray_Xarray*xarr2){
# 115
int newsz=(int)(_get_fat_size(xarr1->elmts,sizeof(void*))+ _get_fat_size(xarr2->elmts,sizeof(void*)));
if(newsz == 0)
return Cyc_Xarray_rcreate_empty(r);{
# 116
void*init=
# 118
_get_fat_size(xarr1->elmts,sizeof(void*))== (unsigned)0?*((void**)_check_fat_subscript(xarr2->elmts,sizeof(void*),0)):*((void**)_check_fat_subscript(xarr1->elmts,sizeof(void*),0));
struct Cyc_Xarray_Xarray*ans=({struct Cyc_Xarray_Xarray*_tmp1C=_region_malloc(r,sizeof(*_tmp1C));({struct _fat_ptr _tmp42=({unsigned _tmp1B=(unsigned)newsz;void**_tmp1A=({struct _RegionHandle*_tmp41=Cyc_Core_unique_region;_region_malloc(_tmp41,_check_times(_tmp1B,sizeof(void*)));});({{unsigned _tmp2F=(unsigned)newsz;unsigned i;for(i=0;i < _tmp2F;++ i){_tmp1A[i]=init;}}0;});_tag_fat(_tmp1A,sizeof(void*),_tmp1B);});_tmp1C->elmts=_tmp42;}),_tmp1C->num_elmts=0;_tmp1C;});
# 121
{int i=0;for(0;i < xarr1->num_elmts;++ i){
Cyc_Xarray_add(ans,*((void**)_check_fat_subscript(xarr1->elmts,sizeof(void*),i)));}}
{int i=0;for(0;i < xarr2->num_elmts;++ i){
Cyc_Xarray_add(ans,*((void**)_check_fat_subscript(xarr2->elmts,sizeof(void*),i)));}}
return ans;}}
# 25
struct Cyc_Xarray_Xarray*Cyc_Xarray_append(struct Cyc_Xarray_Xarray*xarr1,struct Cyc_Xarray_Xarray*xarr2){
# 129
return Cyc_Xarray_rappend(Cyc_Core_heap_region,xarr1,xarr2);}
# 25
void Cyc_Xarray_app(void*(*f)(void*),struct Cyc_Xarray_Xarray*xarr){
# 133
int i=0;for(0;i < xarr->num_elmts;++ i){
f(*((void**)_check_fat_subscript(xarr->elmts,sizeof(void*),i)));}}
# 25
void Cyc_Xarray_app_c(void*(*f)(void*,void*),void*env,struct Cyc_Xarray_Xarray*xarr){
# 138
int i=0;for(0;i < xarr->num_elmts;++ i){
f(env,*((void**)_check_fat_subscript(xarr->elmts,sizeof(void*),i)));}}
# 25
void Cyc_Xarray_iter(void(*f)(void*),struct Cyc_Xarray_Xarray*xarr){
# 143
int i=0;for(0;i < xarr->num_elmts;++ i){
f(*((void**)_check_fat_subscript(xarr->elmts,sizeof(void*),i)));}}
# 25
void Cyc_Xarray_iter_c(void(*f)(void*,void*),void*env,struct Cyc_Xarray_Xarray*xarr){
# 148
int i=0;for(0;i < xarr->num_elmts;++ i){
f(env,*((void**)_check_fat_subscript(xarr->elmts,sizeof(void*),i)));}}
# 25
struct Cyc_Xarray_Xarray*Cyc_Xarray_rmap(struct _RegionHandle*r,void*(*f)(void*),struct Cyc_Xarray_Xarray*xarr){
# 153
if(xarr->num_elmts == 0)return Cyc_Xarray_rcreate_empty(r);{struct Cyc_Xarray_Xarray*ans=({struct Cyc_Xarray_Xarray*_tmp1F=_region_malloc(r,sizeof(*_tmp1F));
# 156
({struct _fat_ptr _tmp45=({unsigned _tmp1E=_get_fat_size(xarr->elmts,sizeof(void*));void**_tmp1D=({struct _RegionHandle*_tmp43=Cyc_Core_unique_region;_region_malloc(_tmp43,_check_times(_tmp1E,sizeof(void*)));});({{unsigned _tmp30=_get_fat_size(xarr->elmts,sizeof(void*));unsigned i;for(i=0;i < _tmp30;++ i){({void*_tmp44=f(*((void**)_check_fat_subscript(xarr->elmts,sizeof(void*),0)));_tmp1D[i]=_tmp44;});}}0;});_tag_fat(_tmp1D,sizeof(void*),_tmp1E);});_tmp1F->elmts=_tmp45;}),_tmp1F->num_elmts=xarr->num_elmts;_tmp1F;});
# 158
{int i=1;for(0;i < xarr->num_elmts;++ i){
({void*_tmp46=f(*((void**)_check_fat_subscript(xarr->elmts,sizeof(void*),i)));*((void**)_check_fat_subscript(ans->elmts,sizeof(void*),i))=_tmp46;});}}
return ans;}}
# 25
struct Cyc_Xarray_Xarray*Cyc_Xarray_map(void*(*f)(void*),struct Cyc_Xarray_Xarray*xarr){
# 164
return Cyc_Xarray_rmap(Cyc_Core_heap_region,f,xarr);}
# 25
struct Cyc_Xarray_Xarray*Cyc_Xarray_rmap_c(struct _RegionHandle*r,void*(*f)(void*,void*),void*env,struct Cyc_Xarray_Xarray*xarr){
# 168
if(xarr->num_elmts == 0)return Cyc_Xarray_rcreate_empty(r);{struct Cyc_Xarray_Xarray*ans=({struct Cyc_Xarray_Xarray*_tmp22=_region_malloc(r,sizeof(*_tmp22));
# 171
({struct _fat_ptr _tmp49=({unsigned _tmp21=_get_fat_size(xarr->elmts,sizeof(void*));void**_tmp20=({struct _RegionHandle*_tmp47=Cyc_Core_unique_region;_region_malloc(_tmp47,_check_times(_tmp21,sizeof(void*)));});({{unsigned _tmp31=_get_fat_size(xarr->elmts,sizeof(void*));unsigned i;for(i=0;i < _tmp31;++ i){({void*_tmp48=f(env,*((void**)_check_fat_subscript(xarr->elmts,sizeof(void*),0)));_tmp20[i]=_tmp48;});}}0;});_tag_fat(_tmp20,sizeof(void*),_tmp21);});_tmp22->elmts=_tmp49;}),_tmp22->num_elmts=xarr->num_elmts;_tmp22;});
# 173
{int i=1;for(0;i < xarr->num_elmts;++ i){
({void*_tmp4A=f(env,*((void**)_check_fat_subscript(xarr->elmts,sizeof(void*),i)));*((void**)_check_fat_subscript(ans->elmts,sizeof(void*),i))=_tmp4A;});}}
return ans;}}
# 25
struct Cyc_Xarray_Xarray*Cyc_Xarray_map_c(void*(*f)(void*,void*),void*env,struct Cyc_Xarray_Xarray*xarr){
# 179
return Cyc_Xarray_rmap_c(Cyc_Core_heap_region,f,env,xarr);}
# 25
void Cyc_Xarray_reuse(struct Cyc_Xarray_Xarray*xarr){
# 183
struct _fat_ptr newarr=_tag_fat(0,0,0);
({struct _fat_ptr _tmp23=newarr;struct _fat_ptr _tmp24=xarr->elmts;newarr=_tmp24;xarr->elmts=_tmp23;});
xarr->num_elmts=0;
({void(*_tmp4B)(void**ptr)=({void(*_tmp25)(void**ptr)=(void(*)(void**ptr))Cyc_Core_ufree;_tmp25;});_tmp4B((void**)_untag_fat_ptr(newarr,sizeof(void*),1U));});}
# 25
void Cyc_Xarray_delete(struct Cyc_Xarray_Xarray*xarr,int num){
# 190
if(({int _tmp4C=num;_tmp4C > Cyc_Xarray_length(xarr);}))
(int)_throw((void*)({struct Cyc_Core_Invalid_argument_exn_struct*_tmp27=_cycalloc(sizeof(*_tmp27));_tmp27->tag=Cyc_Core_Invalid_argument,({struct _fat_ptr _tmp4D=({const char*_tmp26="number deleted is greater than length of xarray";_tag_fat(_tmp26,sizeof(char),48U);});_tmp27->f1=_tmp4D;});_tmp27;}));
# 190
xarr->num_elmts -=num;}
# 25
void Cyc_Xarray_remove(struct Cyc_Xarray_Xarray*xarr,int i){
# 196
if(i < 0 || i > xarr->num_elmts - 1)
(int)_throw((void*)({struct Cyc_Core_Invalid_argument_exn_struct*_tmp29=_cycalloc(sizeof(*_tmp29));_tmp29->tag=Cyc_Core_Invalid_argument,({struct _fat_ptr _tmp4E=({const char*_tmp28="xarray index out of bounds";_tag_fat(_tmp28,sizeof(char),27U);});_tmp29->f1=_tmp4E;});_tmp29;}));
# 196
{
# 198
int j=i;
# 196
for(0;j < xarr->num_elmts - 1;++ j){
# 199
({void*_tmp4F=*((void**)_check_fat_subscript(xarr->elmts,sizeof(void*),j + 1));*((void**)_check_fat_subscript(xarr->elmts,sizeof(void*),j))=_tmp4F;});}}
-- xarr->num_elmts;}
