#include <setjmp.h>
/* This is a C header file to be used by the output of the Cyclone to
   C translator.  The corresponding definitions are in file lib/runtime_*.c */
#ifndef _CYC_INCLUDE_H_
#define _CYC_INCLUDE_H_

/* Need one of these per thread (see runtime_stack.c). The runtime maintains 
   a stack that contains either _handler_cons structs or _RegionHandle structs.
   The tag is 0 for a handler_cons and 1 for a region handle.  */
struct _RuntimeStack {
  int tag; 
  struct _RuntimeStack *next;
  void (*cleanup)(struct _RuntimeStack *frame);
};

#ifndef offsetof
/* should be size_t, but int is fine. */
#define offsetof(t,n) ((int)(&(((t *)0)->n)))
#endif

/* Fat pointers */
struct _fat_ptr {
  unsigned char *curr; 
  unsigned char *base; 
  unsigned char *last_plus_one; 
};  

/* Discriminated Unions */
struct _xtunion_struct { char *tag; };

/* Regions */
struct _RegionPage
#ifdef CYC_REGION_PROFILE
{ unsigned total_bytes;
  unsigned free_bytes;
  /* MWH: wish we didn't have to include the stuff below ... */
  struct _RegionPage *next;
  char data[1];
}
#endif
; // abstract -- defined in runtime_memory.c
struct _RegionHandle {
  struct _RuntimeStack s;
  struct _RegionPage *curr;
  char               *offset;
  char               *last_plus_one;
  struct _DynRegionHandle *sub_regions;
#ifdef CYC_REGION_PROFILE
  const char         *name;
#else
  unsigned used_bytes;
  unsigned wasted_bytes;
#endif
};
struct _DynRegionFrame {
  struct _RuntimeStack s;
  struct _DynRegionHandle *x;
};
// A dynamic region is just a region handle.  The wrapper struct is for type
// abstraction.
struct Cyc_Core_DynamicRegion {
  struct _RegionHandle h;
};

struct _RegionHandle _new_region(const char*);
void* _region_malloc(struct _RegionHandle*, unsigned);
void* _region_calloc(struct _RegionHandle*, unsigned t, unsigned n);
void   _free_region(struct _RegionHandle*);
struct _RegionHandle*_open_dynregion(struct _DynRegionFrame*,struct _DynRegionHandle*);
void   _pop_dynregion();

/* Exceptions */
struct _handler_cons {
  struct _RuntimeStack s;
  jmp_buf handler;
};
void _push_handler(struct _handler_cons *);
void _push_region(struct _RegionHandle *);
void _npop_handler(int);
void _pop_handler();
void _pop_region();

#ifndef _throw
void* _throw_null_fn(const char*,unsigned);
void* _throw_arraybounds_fn(const char*,unsigned);
void* _throw_badalloc_fn(const char*,unsigned);
void* _throw_match_fn(const char*,unsigned);
void* _throw_fn(void*,const char*,unsigned);
void* _rethrow(void*);
#define _throw_null() (_throw_null_fn(__FILE__,__LINE__))
#define _throw_arraybounds() (_throw_arraybounds_fn(__FILE__,__LINE__))
#define _throw_badalloc() (_throw_badalloc_fn(__FILE__,__LINE__))
#define _throw_match() (_throw_match_fn(__FILE__,__LINE__))
#define _throw(e) (_throw_fn((e),__FILE__,__LINE__))
#endif

struct _xtunion_struct* Cyc_Core_get_exn_thrown();
/* Built-in Exceptions */
struct Cyc_Null_Exception_exn_struct { char *tag; };
struct Cyc_Array_bounds_exn_struct { char *tag; };
struct Cyc_Match_Exception_exn_struct { char *tag; };
struct Cyc_Bad_alloc_exn_struct { char *tag; };
extern char Cyc_Null_Exception[];
extern char Cyc_Array_bounds[];
extern char Cyc_Match_Exception[];
extern char Cyc_Bad_alloc[];

/* Built-in Run-time Checks and company */
#ifdef CYC_ANSI_OUTPUT
#define _INLINE  
#else
#define _INLINE inline
#endif

#ifdef NO_CYC_NULL_CHECKS
#define _check_null(ptr) (ptr)
#else
#define _check_null(ptr) \
  ({ void*_cks_null = (void*)(ptr); \
     if (!_cks_null) _throw_null(); \
     _cks_null; })
#endif

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index)\
   (((char*)ptr) + (elt_sz)*(index))
#ifdef NO_CYC_NULL_CHECKS
#define _check_known_subscript_null _check_known_subscript_notnull
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr);\
  int _index = (index);\
  if (!_cks_ptr) _throw_null(); \
  _cks_ptr + (elt_sz)*_index; })
#endif
#define _zero_arr_plus_fn(orig_x,orig_sz,orig_i,f,l) ((orig_x)+(orig_i))
#define _zero_arr_plus_char_fn _zero_arr_plus_fn
#define _zero_arr_plus_short_fn _zero_arr_plus_fn
#define _zero_arr_plus_int_fn _zero_arr_plus_fn
#define _zero_arr_plus_float_fn _zero_arr_plus_fn
#define _zero_arr_plus_double_fn _zero_arr_plus_fn
#define _zero_arr_plus_longdouble_fn _zero_arr_plus_fn
#define _zero_arr_plus_voidstar_fn _zero_arr_plus_fn
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (!_cks_ptr) _throw_null(); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })

/* _zero_arr_plus_*_fn(x,sz,i,filename,lineno) adds i to zero-terminated ptr
   x that has at least sz elements */
char* _zero_arr_plus_char_fn(char*,unsigned,int,const char*,unsigned);
short* _zero_arr_plus_short_fn(short*,unsigned,int,const char*,unsigned);
int* _zero_arr_plus_int_fn(int*,unsigned,int,const char*,unsigned);
float* _zero_arr_plus_float_fn(float*,unsigned,int,const char*,unsigned);
double* _zero_arr_plus_double_fn(double*,unsigned,int,const char*,unsigned);
long double* _zero_arr_plus_longdouble_fn(long double*,unsigned,int,const char*, unsigned);
void** _zero_arr_plus_voidstar_fn(void**,unsigned,int,const char*,unsigned);
#endif

/* _get_zero_arr_size_*(x,sz) returns the number of elements in a
   zero-terminated array that is NULL or has at least sz elements */
int _get_zero_arr_size_char(const char*,unsigned);
int _get_zero_arr_size_short(const short*,unsigned);
int _get_zero_arr_size_int(const int*,unsigned);
int _get_zero_arr_size_float(const float*,unsigned);
int _get_zero_arr_size_double(const double*,unsigned);
int _get_zero_arr_size_longdouble(const long double*,unsigned);
int _get_zero_arr_size_voidstar(const void**,unsigned);

/* _zero_arr_inplace_plus_*_fn(x,i,filename,lineno) sets
   zero-terminated pointer *x to *x + i */
char* _zero_arr_inplace_plus_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_short_fn(short**,int,const char*,unsigned);
int* _zero_arr_inplace_plus_int(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_longdouble_fn(long double**,int,const char*,unsigned);
void** _zero_arr_inplace_plus_voidstar_fn(void***,int,const char*,unsigned);
/* like the previous functions, but does post-addition (as in e++) */
char* _zero_arr_inplace_plus_post_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_post_short_fn(short**x,int,const char*,unsigned);
int* _zero_arr_inplace_plus_post_int_fn(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_post_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_post_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_post_longdouble_fn(long double**,int,const char *,unsigned);
void** _zero_arr_inplace_plus_post_voidstar_fn(void***,int,const char*,unsigned);
#define _zero_arr_plus_char(x,s,i) \
  (_zero_arr_plus_char_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_short(x,s,i) \
  (_zero_arr_plus_short_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_int(x,s,i) \
  (_zero_arr_plus_int_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_float(x,s,i) \
  (_zero_arr_plus_float_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_double(x,s,i) \
  (_zero_arr_plus_double_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_longdouble(x,s,i) \
  (_zero_arr_plus_longdouble_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_voidstar(x,s,i) \
  (_zero_arr_plus_voidstar_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_inplace_plus_char(x,i) \
  _zero_arr_inplace_plus_char_fn((char **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_short(x,i) \
  _zero_arr_inplace_plus_short_fn((short **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_int(x,i) \
  _zero_arr_inplace_plus_int_fn((int **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_float(x,i) \
  _zero_arr_inplace_plus_float_fn((float **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_double(x,i) \
  _zero_arr_inplace_plus_double_fn((double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_longdouble(x,i) \
  _zero_arr_inplace_plus_longdouble_fn((long double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_voidstar(x,i) \
  _zero_arr_inplace_plus_voidstar_fn((void ***)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_char(x,i) \
  _zero_arr_inplace_plus_post_char_fn((char **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_short(x,i) \
  _zero_arr_inplace_plus_post_short_fn((short **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_int(x,i) \
  _zero_arr_inplace_plus_post_int_fn((int **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_float(x,i) \
  _zero_arr_inplace_plus_post_float_fn((float **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_double(x,i) \
  _zero_arr_inplace_plus_post_double_fn((double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_longdouble(x,i) \
  _zero_arr_inplace_plus_post_longdouble_fn((long double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_voidstar(x,i) \
  _zero_arr_inplace_plus_post_voidstar_fn((void***)(x),(i),__FILE__,__LINE__)

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_fat_subscript(arr,elt_sz,index) ((arr).curr + (elt_sz) * (index))
#define _untag_fat_ptr(arr,elt_sz,num_elts) ((arr).curr)
#else
#define _check_fat_subscript(arr,elt_sz,index) ({ \
  struct _fat_ptr _cus_arr = (arr); \
  unsigned char *_cus_ans = _cus_arr.curr + (elt_sz) * (index); \
  /* JGM: not needed! if (!_cus_arr.base) _throw_null();*/ \
  if (_cus_ans < _cus_arr.base || _cus_ans >= _cus_arr.last_plus_one) \
    _throw_arraybounds(); \
  _cus_ans; })
#define _untag_fat_ptr(arr,elt_sz,num_elts) ({ \
  struct _fat_ptr _arr = (arr); \
  unsigned char *_curr = _arr.curr; \
  if ((_curr < _arr.base || _curr + (elt_sz) * (num_elts) > _arr.last_plus_one) &&\
      _curr != (unsigned char *)0) \
    _throw_arraybounds(); \
  _curr; })
#endif

#define _tag_fat(tcurr,elt_sz,num_elts) ({ \
  struct _fat_ptr _ans; \
  unsigned _num_elts = (num_elts);\
  _ans.base = _ans.curr = (void*)(tcurr); \
  /* JGM: if we're tagging NULL, ignore num_elts */ \
  _ans.last_plus_one = _ans.base ? (_ans.base + (elt_sz) * _num_elts) : 0; \
  _ans; })

#define _get_fat_size(arr,elt_sz) \
  ({struct _fat_ptr _arr = (arr); \
    unsigned char *_arr_curr=_arr.curr; \
    unsigned char *_arr_last=_arr.last_plus_one; \
    (_arr_curr < _arr.base || _arr_curr >= _arr_last) ? 0 : \
    ((_arr_last - _arr_curr) / (elt_sz));})

#define _fat_ptr_plus(arr,elt_sz,change) ({ \
  struct _fat_ptr _ans = (arr); \
  int _change = (change);\
  _ans.curr += (elt_sz) * _change;\
  _ans; })
#define _fat_ptr_inplace_plus(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  _arr_ptr->curr += (elt_sz) * (change);\
  *_arr_ptr; })
#define _fat_ptr_inplace_plus_post(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  struct _fat_ptr _ans = *_arr_ptr; \
  _arr_ptr->curr += (elt_sz) * (change);\
  _ans; })

//Not a macro since initialization order matters. Defined in runtime_zeroterm.c.
struct _fat_ptr _fat_ptr_decrease_size(struct _fat_ptr,unsigned sz,unsigned numelts);

/* Allocation */
void* GC_malloc(int);
void* GC_malloc_atomic(int);
void* GC_calloc(unsigned,unsigned);
void* GC_calloc_atomic(unsigned,unsigned);
// bound the allocation size to be < MAX_ALLOC_SIZE. See macros below for usage.
#define MAX_MALLOC_SIZE (1 << 28)
void* _bounded_GC_malloc(int,const char*,int);
void* _bounded_GC_malloc_atomic(int,const char*,int);
void* _bounded_GC_calloc(unsigned,unsigned,const char*,int);
void* _bounded_GC_calloc_atomic(unsigned,unsigned,const char*,int);
/* these macros are overridden below ifdef CYC_REGION_PROFILE */
#ifndef CYC_REGION_PROFILE
#define _cycalloc(n) _bounded_GC_malloc(n,__FILE__,__LINE__)
#define _cycalloc_atomic(n) _bounded_GC_malloc_atomic(n,__FILE__,__LINE__)
#define _cyccalloc(n,s) _bounded_GC_calloc(n,s,__FILE__,__LINE__)
#define _cyccalloc_atomic(n,s) _bounded_GC_calloc_atomic(n,s,__FILE__,__LINE__)
#endif

static _INLINE unsigned int _check_times(unsigned x, unsigned y) {
  unsigned long long whole_ans = 
    ((unsigned long long) x)*((unsigned long long)y);
  unsigned word_ans = (unsigned)whole_ans;
  if(word_ans < whole_ans || word_ans > MAX_MALLOC_SIZE)
    _throw_badalloc();
  return word_ans;
}

#define _CYC_MAX_REGION_CONST 2
#define _CYC_MIN_ALIGNMENT (sizeof(double))

#ifdef CYC_REGION_PROFILE
extern int rgn_total_bytes;
#endif

static _INLINE void *_fast_region_malloc(struct _RegionHandle *r, unsigned orig_s) {  
  if (r > (struct _RegionHandle *)_CYC_MAX_REGION_CONST && r->curr != 0) { 
#ifdef CYC_NOALIGN
    unsigned s =  orig_s;
#else
    unsigned s =  (orig_s + _CYC_MIN_ALIGNMENT - 1) & (~(_CYC_MIN_ALIGNMENT -1)); 
#endif
    char *result; 
    result = r->offset; 
    if (s <= (r->last_plus_one - result)) {
      r->offset = result + s; 
#ifdef CYC_REGION_PROFILE
    r->curr->free_bytes = r->curr->free_bytes - s;
    rgn_total_bytes += s;
#endif
      return result;
    }
  } 
  return _region_malloc(r,orig_s); 
}

#ifdef CYC_REGION_PROFILE
/* see macros below for usage. defined in runtime_memory.c */
void* _profile_GC_malloc(int,const char*,const char*,int);
void* _profile_GC_malloc_atomic(int,const char*,const char*,int);
void* _profile_GC_calloc(unsigned,unsigned,const char*,const char*,int);
void* _profile_GC_calloc_atomic(unsigned,unsigned,const char*,const char*,int);
void* _profile_region_malloc(struct _RegionHandle*,unsigned,const char*,const char*,int);
void* _profile_region_calloc(struct _RegionHandle*,unsigned,unsigned,const char *,const char*,int);
struct _RegionHandle _profile_new_region(const char*,const char*,const char*,int);
void _profile_free_region(struct _RegionHandle*,const char*,const char*,int);
#ifndef RUNTIME_CYC
#define _new_region(n) _profile_new_region(n,__FILE__,__FUNCTION__,__LINE__)
#define _free_region(r) _profile_free_region(r,__FILE__,__FUNCTION__,__LINE__)
#define _region_malloc(rh,n) _profile_region_malloc(rh,n,__FILE__,__FUNCTION__,__LINE__)
#define _region_calloc(rh,n,t) _profile_region_calloc(rh,n,t,__FILE__,__FUNCTION__,__LINE__)
#  endif
#define _cycalloc(n) _profile_GC_malloc(n,__FILE__,__FUNCTION__,__LINE__)
#define _cycalloc_atomic(n) _profile_GC_malloc_atomic(n,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc(n,s) _profile_GC_calloc(n,s,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc_atomic(n,s) _profile_GC_calloc_atomic(n,s,__FILE__,__FUNCTION__,__LINE__)
#endif
#endif
 struct Cyc_Core_Opt{void*v;};
# 119 "core.h"
int Cyc_Core_intcmp(int,int);extern char Cyc_Core_Invalid_argument[17U];struct Cyc_Core_Invalid_argument_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Failure[8U];struct Cyc_Core_Failure_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Impossible[11U];struct Cyc_Core_Impossible_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Not_found[10U];struct Cyc_Core_Not_found_exn_struct{char*tag;};extern char Cyc_Core_Unreachable[12U];struct Cyc_Core_Unreachable_exn_struct{char*tag;struct _fat_ptr f1;};
# 168
extern struct _RegionHandle*Cyc_Core_unique_region;struct Cyc_Core_DynamicRegion;struct Cyc_Core_NewDynamicRegion{struct Cyc_Core_DynamicRegion*key;};struct Cyc_Core_ThinRes{void*arr;unsigned nelts;};struct Cyc_List_List{void*hd;struct Cyc_List_List*tl;};extern char Cyc_List_List_mismatch[14U];struct Cyc_List_List_mismatch_exn_struct{char*tag;};
# 178 "list.h"
struct Cyc_List_List*Cyc_List_imp_rev(struct Cyc_List_List*x);extern char Cyc_List_Nth[4U];struct Cyc_List_Nth_exn_struct{char*tag;};
# 371
struct Cyc_List_List*Cyc_List_from_array(struct _fat_ptr arr);
# 34 "position.h"
unsigned Cyc_Position_segment_join(unsigned,unsigned);struct Cyc_Position_Error;struct Cyc_Relations_Reln;struct _union_Nmspace_Rel_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_Abs_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_C_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_Loc_n{int tag;int val;};union Cyc_Absyn_Nmspace{struct _union_Nmspace_Rel_n Rel_n;struct _union_Nmspace_Abs_n Abs_n;struct _union_Nmspace_C_n C_n;struct _union_Nmspace_Loc_n Loc_n;};
# 98 "absyn.h"
union Cyc_Absyn_Nmspace Cyc_Absyn_Abs_n(struct Cyc_List_List*ns,int C_scope);struct _tuple0{union Cyc_Absyn_Nmspace f1;struct _fat_ptr*f2;};
# 161
enum Cyc_Absyn_Scope{Cyc_Absyn_Static =0U,Cyc_Absyn_Abstract =1U,Cyc_Absyn_Public =2U,Cyc_Absyn_Extern =3U,Cyc_Absyn_ExternC =4U,Cyc_Absyn_Register =5U};struct Cyc_Absyn_Tqual{int print_const: 1;int q_volatile: 1;int q_restrict: 1;int real_const: 1;unsigned loc;};
# 182
enum Cyc_Absyn_Size_of{Cyc_Absyn_Char_sz =0U,Cyc_Absyn_Short_sz =1U,Cyc_Absyn_Int_sz =2U,Cyc_Absyn_Long_sz =3U,Cyc_Absyn_LongLong_sz =4U};
enum Cyc_Absyn_Sign{Cyc_Absyn_Signed =0U,Cyc_Absyn_Unsigned =1U,Cyc_Absyn_None =2U};
enum Cyc_Absyn_AggrKind{Cyc_Absyn_StructA =0U,Cyc_Absyn_UnionA =1U};
# 187
enum Cyc_Absyn_AliasQual{Cyc_Absyn_Aliasable =0U,Cyc_Absyn_Unique =1U,Cyc_Absyn_Top =2U};
# 192
enum Cyc_Absyn_KindQual{Cyc_Absyn_AnyKind =0U,Cyc_Absyn_MemKind =1U,Cyc_Absyn_BoxKind =2U,Cyc_Absyn_RgnKind =3U,Cyc_Absyn_EffKind =4U,Cyc_Absyn_IntKind =5U,Cyc_Absyn_BoolKind =6U,Cyc_Absyn_PtrBndKind =7U};struct Cyc_Absyn_Kind{enum Cyc_Absyn_KindQual kind;enum Cyc_Absyn_AliasQual aliasqual;};struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct{int tag;struct Cyc_Absyn_Kind*f1;};struct Cyc_Absyn_Unknown_kb_Absyn_KindBound_struct{int tag;struct Cyc_Core_Opt*f1;};struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct{int tag;struct Cyc_Core_Opt*f1;struct Cyc_Absyn_Kind*f2;};struct Cyc_Absyn_Tvar{struct _fat_ptr*name;int identity;void*kind;};struct Cyc_Absyn_PtrLoc{unsigned ptr_loc;unsigned rgn_loc;unsigned zt_loc;};struct Cyc_Absyn_PtrAtts{void*rgn;void*nullable;void*bounds;void*zero_term;struct Cyc_Absyn_PtrLoc*ptrloc;};struct Cyc_Absyn_PtrInfo{void*elt_type;struct Cyc_Absyn_Tqual elt_tq;struct Cyc_Absyn_PtrAtts ptr_atts;};struct Cyc_Absyn_VarargInfo{struct _fat_ptr*name;struct Cyc_Absyn_Tqual tq;void*type;int inject;};struct Cyc_Absyn_Star_Absyn_Cap_struct{int tag;};struct Cyc_Absyn_Bot_Absyn_Cap_struct{int tag;};struct Cyc_Absyn_Nat_Absyn_Cap_struct{int tag;struct Cyc_Absyn_Exp*f1;int f2;};struct Cyc_Absyn_RgnEffect{void*name;void*rgn;void*lock;void*unadopt;void*parent;};struct Cyc_Absyn_FnInfo{struct Cyc_List_List*tvars;void*effect;struct Cyc_Absyn_Tqual ret_tqual;void*ret_type;struct Cyc_List_List*args;int c_varargs;struct Cyc_Absyn_VarargInfo*cyc_varargs;struct Cyc_List_List*rgn_po;struct Cyc_List_List*attributes;struct Cyc_Absyn_Exp*requires_clause;struct Cyc_List_List*requires_relns;struct Cyc_Absyn_Exp*ensures_clause;struct Cyc_List_List*ensures_relns;struct Cyc_List_List*ieffect;struct Cyc_List_List*oeffect;};struct Cyc_Absyn_UnknownDatatypeInfo{struct _tuple0*name;int is_extensible;};struct _union_DatatypeInfo_UnknownDatatype{int tag;struct Cyc_Absyn_UnknownDatatypeInfo val;};struct _union_DatatypeInfo_KnownDatatype{int tag;struct Cyc_Absyn_Datatypedecl**val;};union Cyc_Absyn_DatatypeInfo{struct _union_DatatypeInfo_UnknownDatatype UnknownDatatype;struct _union_DatatypeInfo_KnownDatatype KnownDatatype;};struct Cyc_Absyn_UnknownDatatypeFieldInfo{struct _tuple0*datatype_name;struct _tuple0*field_name;int is_extensible;};struct _union_DatatypeFieldInfo_UnknownDatatypefield{int tag;struct Cyc_Absyn_UnknownDatatypeFieldInfo val;};struct _tuple1{struct Cyc_Absyn_Datatypedecl*f1;struct Cyc_Absyn_Datatypefield*f2;};struct _union_DatatypeFieldInfo_KnownDatatypefield{int tag;struct _tuple1 val;};union Cyc_Absyn_DatatypeFieldInfo{struct _union_DatatypeFieldInfo_UnknownDatatypefield UnknownDatatypefield;struct _union_DatatypeFieldInfo_KnownDatatypefield KnownDatatypefield;};struct _tuple2{enum Cyc_Absyn_AggrKind f1;struct _tuple0*f2;struct Cyc_Core_Opt*f3;};struct _union_AggrInfo_UnknownAggr{int tag;struct _tuple2 val;};struct _union_AggrInfo_KnownAggr{int tag;struct Cyc_Absyn_Aggrdecl**val;};union Cyc_Absyn_AggrInfo{struct _union_AggrInfo_UnknownAggr UnknownAggr;struct _union_AggrInfo_KnownAggr KnownAggr;};
# 331
union Cyc_Absyn_AggrInfo Cyc_Absyn_UnknownAggr(enum Cyc_Absyn_AggrKind,struct _tuple0*,struct Cyc_Core_Opt*);struct Cyc_Absyn_ArrayInfo{void*elt_type;struct Cyc_Absyn_Tqual tq;struct Cyc_Absyn_Exp*num_elts;void*zero_term;unsigned zt_loc;};struct Cyc_Absyn_Aggr_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Aggrdecl*f1;};struct Cyc_Absyn_Enum_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;};struct Cyc_Absyn_Datatype_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;};struct Cyc_Absyn_TypeDecl{void*r;unsigned loc;};struct Cyc_Absyn_VoidCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_IntCon_Absyn_TyCon_struct{int tag;enum Cyc_Absyn_Sign f1;enum Cyc_Absyn_Size_of f2;};struct Cyc_Absyn_FloatCon_Absyn_TyCon_struct{int tag;int f1;};struct Cyc_Absyn_RgnHandleCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_TagCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_HeapCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_UniqueCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_RefCntCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_AccessCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_JoinCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_RgnsCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_TrueCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_FalseCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_ThinCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_FatCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_EnumCon_Absyn_TyCon_struct{int tag;struct _tuple0*f1;struct Cyc_Absyn_Enumdecl*f2;};struct Cyc_Absyn_AnonEnumCon_Absyn_TyCon_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_BuiltinCon_Absyn_TyCon_struct{int tag;struct _fat_ptr f1;struct Cyc_Absyn_Kind*f2;};struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_DatatypeInfo f1;};struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_DatatypeFieldInfo f1;};struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_AggrInfo f1;};struct Cyc_Absyn_AppType_Absyn_Type_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Evar_Absyn_Type_struct{int tag;struct Cyc_Core_Opt*f1;void*f2;int f3;struct Cyc_Core_Opt*f4;};struct Cyc_Absyn_VarType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Tvar*f1;};struct Cyc_Absyn_PointerType_Absyn_Type_struct{int tag;struct Cyc_Absyn_PtrInfo f1;};struct Cyc_Absyn_ArrayType_Absyn_Type_struct{int tag;struct Cyc_Absyn_ArrayInfo f1;};struct Cyc_Absyn_FnType_Absyn_Type_struct{int tag;struct Cyc_Absyn_FnInfo f1;};struct Cyc_Absyn_TupleType_Absyn_Type_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct{int tag;enum Cyc_Absyn_AggrKind f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_TypedefType_Absyn_Type_struct{int tag;struct _tuple0*f1;struct Cyc_List_List*f2;struct Cyc_Absyn_Typedefdecl*f3;void*f4;};struct Cyc_Absyn_ValueofType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct{int tag;struct Cyc_Absyn_TypeDecl*f1;void**f2;};struct Cyc_Absyn_TypeofType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_NoTypes_Absyn_Funcparams_struct{int tag;struct Cyc_List_List*f1;unsigned f2;};struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct{int tag;struct Cyc_List_List*f1;int f2;struct Cyc_Absyn_VarargInfo*f3;void*f4;struct Cyc_List_List*f5;struct Cyc_Absyn_Exp*f6;struct Cyc_Absyn_Exp*f7;struct Cyc_List_List*f8;struct Cyc_List_List*f9;};
# 437 "absyn.h"
enum Cyc_Absyn_Format_Type{Cyc_Absyn_Printf_ft =0U,Cyc_Absyn_Scanf_ft =1U};struct Cyc_Absyn_Regparm_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Stdcall_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Cdecl_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Fastcall_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Noreturn_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Const_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Aligned_att_Absyn_Attribute_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Packed_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Section_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Nocommon_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Shared_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Unused_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Weak_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Dllimport_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Dllexport_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_instrument_function_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Constructor_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Destructor_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_check_memory_usage_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Format_att_Absyn_Attribute_struct{int tag;enum Cyc_Absyn_Format_Type f1;int f2;int f3;};struct Cyc_Absyn_Initializes_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Noliveunique_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Consume_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Pure_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Mode_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Alias_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Always_inline_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_throw_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Non_null_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Deprecated_att_Absyn_Attribute_struct{int tag;};
# 476
extern struct Cyc_Absyn_Stdcall_att_Absyn_Attribute_struct Cyc_Absyn_Stdcall_att_val;
extern struct Cyc_Absyn_Cdecl_att_Absyn_Attribute_struct Cyc_Absyn_Cdecl_att_val;
extern struct Cyc_Absyn_Fastcall_att_Absyn_Attribute_struct Cyc_Absyn_Fastcall_att_val;
extern struct Cyc_Absyn_Noreturn_att_Absyn_Attribute_struct Cyc_Absyn_Noreturn_att_val;
extern struct Cyc_Absyn_Const_att_Absyn_Attribute_struct Cyc_Absyn_Const_att_val;
extern struct Cyc_Absyn_Packed_att_Absyn_Attribute_struct Cyc_Absyn_Packed_att_val;
# 483
extern struct Cyc_Absyn_Shared_att_Absyn_Attribute_struct Cyc_Absyn_Shared_att_val;
extern struct Cyc_Absyn_Unused_att_Absyn_Attribute_struct Cyc_Absyn_Unused_att_val;
extern struct Cyc_Absyn_Weak_att_Absyn_Attribute_struct Cyc_Absyn_Weak_att_val;
extern struct Cyc_Absyn_Dllimport_att_Absyn_Attribute_struct Cyc_Absyn_Dllimport_att_val;
extern struct Cyc_Absyn_Dllexport_att_Absyn_Attribute_struct Cyc_Absyn_Dllexport_att_val;
extern struct Cyc_Absyn_No_instrument_function_att_Absyn_Attribute_struct Cyc_Absyn_No_instrument_function_att_val;
extern struct Cyc_Absyn_Constructor_att_Absyn_Attribute_struct Cyc_Absyn_Constructor_att_val;
extern struct Cyc_Absyn_Destructor_att_Absyn_Attribute_struct Cyc_Absyn_Destructor_att_val;
extern struct Cyc_Absyn_No_check_memory_usage_att_Absyn_Attribute_struct Cyc_Absyn_No_check_memory_usage_att_val;
extern struct Cyc_Absyn_Pure_att_Absyn_Attribute_struct Cyc_Absyn_Pure_att_val;
extern struct Cyc_Absyn_Always_inline_att_Absyn_Attribute_struct Cyc_Absyn_Always_inline_att_val;
extern struct Cyc_Absyn_No_throw_att_Absyn_Attribute_struct Cyc_Absyn_No_throw_att_val;
extern struct Cyc_Absyn_Non_null_att_Absyn_Attribute_struct Cyc_Absyn_Non_null_att_val;
extern struct Cyc_Absyn_Deprecated_att_Absyn_Attribute_struct Cyc_Absyn_Deprecated_att_val;struct Cyc_Absyn_Carray_mod_Absyn_Type_modifier_struct{int tag;void*f1;unsigned f2;};struct Cyc_Absyn_ConstArray_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_Absyn_Exp*f1;void*f2;unsigned f3;};struct Cyc_Absyn_Pointer_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_Absyn_PtrAtts f1;struct Cyc_Absyn_Tqual f2;};struct Cyc_Absyn_Function_mod_Absyn_Type_modifier_struct{int tag;void*f1;};struct Cyc_Absyn_TypeParams_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_List_List*f1;unsigned f2;int f3;};struct Cyc_Absyn_Attributes_mod_Absyn_Type_modifier_struct{int tag;unsigned f1;struct Cyc_List_List*f2;};struct _union_Cnst_Null_c{int tag;int val;};struct _tuple3{enum Cyc_Absyn_Sign f1;char f2;};struct _union_Cnst_Char_c{int tag;struct _tuple3 val;};struct _union_Cnst_Wchar_c{int tag;struct _fat_ptr val;};struct _tuple4{enum Cyc_Absyn_Sign f1;short f2;};struct _union_Cnst_Short_c{int tag;struct _tuple4 val;};struct _tuple5{enum Cyc_Absyn_Sign f1;int f2;};struct _union_Cnst_Int_c{int tag;struct _tuple5 val;};struct _tuple6{enum Cyc_Absyn_Sign f1;long long f2;};struct _union_Cnst_LongLong_c{int tag;struct _tuple6 val;};struct _tuple7{struct _fat_ptr f1;int f2;};struct _union_Cnst_Float_c{int tag;struct _tuple7 val;};struct _union_Cnst_String_c{int tag;struct _fat_ptr val;};struct _union_Cnst_Wstring_c{int tag;struct _fat_ptr val;};union Cyc_Absyn_Cnst{struct _union_Cnst_Null_c Null_c;struct _union_Cnst_Char_c Char_c;struct _union_Cnst_Wchar_c Wchar_c;struct _union_Cnst_Short_c Short_c;struct _union_Cnst_Int_c Int_c;struct _union_Cnst_LongLong_c LongLong_c;struct _union_Cnst_Float_c Float_c;struct _union_Cnst_String_c String_c;struct _union_Cnst_Wstring_c Wstring_c;};
# 522
union Cyc_Absyn_Cnst Cyc_Absyn_Char_c(enum Cyc_Absyn_Sign,char);
union Cyc_Absyn_Cnst Cyc_Absyn_Wchar_c(struct _fat_ptr);
# 525
union Cyc_Absyn_Cnst Cyc_Absyn_Int_c(enum Cyc_Absyn_Sign,int);
# 527
union Cyc_Absyn_Cnst Cyc_Absyn_Float_c(struct _fat_ptr,int);
union Cyc_Absyn_Cnst Cyc_Absyn_String_c(struct _fat_ptr);
union Cyc_Absyn_Cnst Cyc_Absyn_Wstring_c(struct _fat_ptr);
# 532
enum Cyc_Absyn_Primop{Cyc_Absyn_Plus =0U,Cyc_Absyn_Times =1U,Cyc_Absyn_Minus =2U,Cyc_Absyn_Div =3U,Cyc_Absyn_Mod =4U,Cyc_Absyn_Eq =5U,Cyc_Absyn_Neq =6U,Cyc_Absyn_Gt =7U,Cyc_Absyn_Lt =8U,Cyc_Absyn_Gte =9U,Cyc_Absyn_Lte =10U,Cyc_Absyn_Not =11U,Cyc_Absyn_Bitnot =12U,Cyc_Absyn_Bitand =13U,Cyc_Absyn_Bitor =14U,Cyc_Absyn_Bitxor =15U,Cyc_Absyn_Bitlshift =16U,Cyc_Absyn_Bitlrshift =17U,Cyc_Absyn_Numelts =18U};
# 539
enum Cyc_Absyn_Incrementor{Cyc_Absyn_PreInc =0U,Cyc_Absyn_PostInc =1U,Cyc_Absyn_PreDec =2U,Cyc_Absyn_PostDec =3U};struct Cyc_Absyn_VarargCallInfo{int num_varargs;struct Cyc_List_List*injectors;struct Cyc_Absyn_VarargInfo*vai;};struct Cyc_Absyn_StructField_Absyn_OffsetofField_struct{int tag;struct _fat_ptr*f1;};struct Cyc_Absyn_TupleIndex_Absyn_OffsetofField_struct{int tag;unsigned f1;};
# 557
enum Cyc_Absyn_Coercion{Cyc_Absyn_Unknown_coercion =0U,Cyc_Absyn_No_coercion =1U,Cyc_Absyn_Null_to_NonNull =2U,Cyc_Absyn_Other_coercion =3U};struct Cyc_Absyn_MallocInfo{int is_calloc;struct Cyc_Absyn_Exp*rgn;void**elt_type;struct Cyc_Absyn_Exp*num_elts;int fat_result;int inline_call;};struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct{int tag;union Cyc_Absyn_Cnst f1;};struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Pragma_e_Absyn_Raw_exp_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct{int tag;enum Cyc_Absyn_Primop f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_AssignOp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Core_Opt*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_Increment_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;enum Cyc_Absyn_Incrementor f2;};struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;struct Cyc_Absyn_VarargCallInfo*f3;int f4;};struct Cyc_Absyn_Throw_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;int f2;};struct Cyc_Absyn_NoInstantiate_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Instantiate_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_Absyn_Exp*f2;int f3;enum Cyc_Absyn_Coercion f4;};struct Cyc_Absyn_Address_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_New_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Sizeoftype_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Sizeofexp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Offsetof_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Deref_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;int f3;int f4;};struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;int f3;int f4;};struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Tuple_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;};struct _tuple8{struct _fat_ptr*f1;struct Cyc_Absyn_Tqual f2;void*f3;};struct Cyc_Absyn_CompoundLit_e_Absyn_Raw_exp_struct{int tag;struct _tuple8*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Array_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Exp*f2;struct Cyc_Absyn_Exp*f3;int f4;};struct Cyc_Absyn_ComprehensionNoinit_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;void*f2;int f3;};struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct{int tag;struct _tuple0*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;struct Cyc_Absyn_Aggrdecl*f4;};struct Cyc_Absyn_AnonStruct_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Datatype_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;struct Cyc_Absyn_Datatypedecl*f2;struct Cyc_Absyn_Datatypefield*f3;};struct Cyc_Absyn_Enum_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_AnonEnum_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_Spawn_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_MallocInfo f1;};struct Cyc_Absyn_Swap_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_UnresolvedMem_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Core_Opt*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_StmtExp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Stmt*f1;};struct Cyc_Absyn_Tagcheck_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;};struct Cyc_Absyn_Valueof_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Asm_e_Absyn_Raw_exp_struct{int tag;int f1;struct _fat_ptr f2;struct Cyc_List_List*f3;struct Cyc_List_List*f4;struct Cyc_List_List*f5;};struct Cyc_Absyn_Extension_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Exp{void*topt;void*r;unsigned loc;void*annot;};struct Cyc_Absyn_Skip_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Exp_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Seq_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Return_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_IfThenElse_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Stmt*f2;struct Cyc_Absyn_Stmt*f3;};struct _tuple9{struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_While_s_Absyn_Raw_stmt_struct{int tag;struct _tuple9 f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Break_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Continue_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Goto_s_Absyn_Raw_stmt_struct{int tag;struct _fat_ptr*f1;};struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _tuple9 f2;struct _tuple9 f3;struct Cyc_Absyn_Stmt*f4;};struct Cyc_Absyn_Switch_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;void*f3;};struct Cyc_Absyn_Fallthru_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_List_List*f1;struct Cyc_Absyn_Switch_clause**f2;};struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Decl*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Label_s_Absyn_Raw_stmt_struct{int tag;struct _fat_ptr*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Do_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct _tuple9 f2;};struct Cyc_Absyn_TryCatch_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct Cyc_List_List*f2;void*f3;};
# 693 "absyn.h"
extern struct Cyc_Absyn_Skip_s_Absyn_Raw_stmt_struct Cyc_Absyn_Skip_s_val;struct Cyc_Absyn_Stmt{void*r;unsigned loc;void*annot;};struct Cyc_Absyn_Wild_p_Absyn_Raw_pat_struct{int tag;};struct Cyc_Absyn_Var_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Pat*f2;};struct Cyc_Absyn_AliasVar_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;};struct Cyc_Absyn_Reference_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Pat*f2;};struct Cyc_Absyn_TagInt_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;};struct Cyc_Absyn_Tuple_p_Absyn_Raw_pat_struct{int tag;struct Cyc_List_List*f1;int f2;};struct Cyc_Absyn_Pointer_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Pat*f1;};struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct{int tag;union Cyc_Absyn_AggrInfo*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;int f4;};struct Cyc_Absyn_Datatype_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;struct Cyc_Absyn_Datatypefield*f2;struct Cyc_List_List*f3;int f4;};struct Cyc_Absyn_Null_p_Absyn_Raw_pat_struct{int tag;};struct Cyc_Absyn_Int_p_Absyn_Raw_pat_struct{int tag;enum Cyc_Absyn_Sign f1;int f2;};struct Cyc_Absyn_Char_p_Absyn_Raw_pat_struct{int tag;char f1;};struct Cyc_Absyn_Float_p_Absyn_Raw_pat_struct{int tag;struct _fat_ptr f1;int f2;};struct Cyc_Absyn_Enum_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_AnonEnum_p_Absyn_Raw_pat_struct{int tag;void*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_UnknownId_p_Absyn_Raw_pat_struct{int tag;struct _tuple0*f1;};struct Cyc_Absyn_UnknownCall_p_Absyn_Raw_pat_struct{int tag;struct _tuple0*f1;struct Cyc_List_List*f2;int f3;};struct Cyc_Absyn_Exp_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Pat{void*r;void*topt;unsigned loc;};struct Cyc_Absyn_Switch_clause{struct Cyc_Absyn_Pat*pattern;struct Cyc_Core_Opt*pat_vars;struct Cyc_Absyn_Exp*where_clause;struct Cyc_Absyn_Stmt*body;unsigned loc;};struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct{int tag;struct _tuple0*f1;};struct Cyc_Absyn_Global_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Funname_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Fndecl*f1;};struct Cyc_Absyn_Param_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Local_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Pat_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Vardecl{enum Cyc_Absyn_Scope sc;struct _tuple0*name;unsigned varloc;struct Cyc_Absyn_Tqual tq;void*type;struct Cyc_Absyn_Exp*initializer;void*rgn;struct Cyc_List_List*attributes;int escapes;};struct Cyc_Absyn_Fndecl{enum Cyc_Absyn_Scope sc;int is_inline;struct _tuple0*name;struct Cyc_Absyn_Stmt*body;struct Cyc_Absyn_FnInfo i;void*cached_type;struct Cyc_Core_Opt*param_vardecls;struct Cyc_Absyn_Vardecl*fn_vardecl;};struct Cyc_Absyn_Aggrfield{struct _fat_ptr*name;struct Cyc_Absyn_Tqual tq;void*type;struct Cyc_Absyn_Exp*width;struct Cyc_List_List*attributes;struct Cyc_Absyn_Exp*requires_clause;};struct Cyc_Absyn_AggrdeclImpl{struct Cyc_List_List*exist_vars;struct Cyc_List_List*rgn_po;struct Cyc_List_List*fields;int tagged;};struct Cyc_Absyn_Aggrdecl{enum Cyc_Absyn_AggrKind kind;enum Cyc_Absyn_Scope sc;struct _tuple0*name;struct Cyc_List_List*tvs;struct Cyc_Absyn_AggrdeclImpl*impl;struct Cyc_List_List*attributes;int expected_mem_kind;};struct Cyc_Absyn_Datatypefield{struct _tuple0*name;struct Cyc_List_List*typs;unsigned loc;enum Cyc_Absyn_Scope sc;};struct Cyc_Absyn_Datatypedecl{enum Cyc_Absyn_Scope sc;struct _tuple0*name;struct Cyc_List_List*tvs;struct Cyc_Core_Opt*fields;int is_extensible;};struct Cyc_Absyn_Enumfield{struct _tuple0*name;struct Cyc_Absyn_Exp*tag;unsigned loc;};struct Cyc_Absyn_Enumdecl{enum Cyc_Absyn_Scope sc;struct _tuple0*name;struct Cyc_Core_Opt*fields;};struct Cyc_Absyn_Typedefdecl{struct _tuple0*name;struct Cyc_Absyn_Tqual tq;struct Cyc_List_List*tvs;struct Cyc_Core_Opt*kind;void*defn;struct Cyc_List_List*atts;int extern_c;};struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Fn_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Fndecl*f1;};struct Cyc_Absyn_Let_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Pat*f1;struct Cyc_Core_Opt*f2;struct Cyc_Absyn_Exp*f3;void*f4;};struct Cyc_Absyn_Letv_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_Region_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_Aggr_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Aggrdecl*f1;};struct Cyc_Absyn_Datatype_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;};struct Cyc_Absyn_Enum_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;};struct Cyc_Absyn_Typedef_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Typedefdecl*f1;};struct Cyc_Absyn_Namespace_d_Absyn_Raw_decl_struct{int tag;struct _fat_ptr*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Using_d_Absyn_Raw_decl_struct{int tag;struct _tuple0*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_ExternC_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;};struct _tuple10{unsigned f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_ExternCinclude_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;struct _tuple10*f4;};struct Cyc_Absyn_Porton_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Portoff_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Tempeston_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Tempestoff_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Decl{void*r;unsigned loc;};struct Cyc_Absyn_ArrayElement_Absyn_Designator_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_FieldName_Absyn_Designator_struct{int tag;struct _fat_ptr*f1;};char Cyc_Absyn_EmptyAnnot[11U]="EmptyAnnot";struct Cyc_Absyn_EmptyAnnot_Absyn_AbsynAnnot_struct{char*tag;};
# 901
extern struct Cyc_Absyn_EmptyAnnot_Absyn_AbsynAnnot_struct Cyc_Absyn_EmptyAnnot_val;
# 908
int Cyc_Absyn_qvar_cmp(struct _tuple0*,struct _tuple0*);
# 916
struct Cyc_Absyn_Tqual Cyc_Absyn_const_tqual(unsigned);
struct Cyc_Absyn_Tqual Cyc_Absyn_empty_tqual(unsigned);
# 922
void*Cyc_Absyn_compress_kb(void*);
# 930
void*Cyc_Absyn_app_type(void*,struct _fat_ptr);
# 933
void*Cyc_Absyn_new_evar(struct Cyc_Core_Opt*k,struct Cyc_Core_Opt*tenv);
# 938
extern void*Cyc_Absyn_char_type;extern void*Cyc_Absyn_uchar_type;extern void*Cyc_Absyn_ushort_type;extern void*Cyc_Absyn_uint_type;extern void*Cyc_Absyn_ulong_type;extern void*Cyc_Absyn_ulonglong_type;
# 940
extern void*Cyc_Absyn_schar_type;extern void*Cyc_Absyn_sshort_type;extern void*Cyc_Absyn_sint_type;extern void*Cyc_Absyn_slong_type;extern void*Cyc_Absyn_slonglong_type;
# 942
extern void*Cyc_Absyn_float_type;extern void*Cyc_Absyn_double_type;extern void*Cyc_Absyn_long_double_type;
# 945
extern void*Cyc_Absyn_heap_rgn_type;extern void*Cyc_Absyn_unique_rgn_type;
# 949
extern void*Cyc_Absyn_true_type;extern void*Cyc_Absyn_false_type;
# 951
void*Cyc_Absyn_rgn_handle_type(void*);void*Cyc_Absyn_valueof_type(struct Cyc_Absyn_Exp*);
# 966
extern struct _tuple0*Cyc_Absyn_exn_name;
struct Cyc_Absyn_Datatypedecl*Cyc_Absyn_exn_tud();
# 976
extern void*Cyc_Absyn_fat_bound_type;
void*Cyc_Absyn_thin_bounds_type(void*);
void*Cyc_Absyn_thin_bounds_exp(struct Cyc_Absyn_Exp*);
void*Cyc_Absyn_thin_bounds_int(unsigned);
void*Cyc_Absyn_bounds_one();
# 982
void*Cyc_Absyn_pointer_type(struct Cyc_Absyn_PtrInfo);
# 984
void*Cyc_Absyn_starb_type(void*t,void*rgn,struct Cyc_Absyn_Tqual,void*,void*zero_term);
# 987
void*Cyc_Absyn_atb_type(void*t,void*rgn,struct Cyc_Absyn_Tqual tq,void*b,void*zero_term);
# 990
void*Cyc_Absyn_star_type(void*t,void*rgn,struct Cyc_Absyn_Tqual,void*zero_term);
# 992
void*Cyc_Absyn_at_type(void*t,void*rgn,struct Cyc_Absyn_Tqual,void*zero_term);
# 1007
void*Cyc_Absyn_datatype_type(union Cyc_Absyn_DatatypeInfo,struct Cyc_List_List*args);
# 1009
void*Cyc_Absyn_aggr_type(union Cyc_Absyn_AggrInfo,struct Cyc_List_List*args);
# 1012
struct Cyc_Absyn_Exp*Cyc_Absyn_new_exp(void*,unsigned);
# 1015
struct Cyc_Absyn_Exp*Cyc_Absyn_const_exp(union Cyc_Absyn_Cnst,unsigned);
# 1017
struct Cyc_Absyn_Exp*Cyc_Absyn_bool_exp(int,unsigned);
# 1020
struct Cyc_Absyn_Exp*Cyc_Absyn_int_exp(enum Cyc_Absyn_Sign,int,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_signed_int_exp(int,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_uint_exp(unsigned,unsigned);
# 1028
struct Cyc_Absyn_Exp*Cyc_Absyn_var_exp(struct _tuple0*,unsigned);
# 1032
struct Cyc_Absyn_Exp*Cyc_Absyn_primop_exp(enum Cyc_Absyn_Primop,struct Cyc_List_List*,unsigned);
# 1034
struct Cyc_Absyn_Exp*Cyc_Absyn_prim2_exp(enum Cyc_Absyn_Primop,struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*,unsigned);
# 1045
struct Cyc_Absyn_Exp*Cyc_Absyn_assignop_exp(struct Cyc_Absyn_Exp*,struct Cyc_Core_Opt*,struct Cyc_Absyn_Exp*,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_assign_exp(struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*,unsigned);
# 1078
struct _tuple0*Cyc_Absyn_uniquergn_qvar();
# 1081
struct Cyc_Absyn_Stmt*Cyc_Absyn_new_stmt(void*,unsigned);
struct Cyc_Absyn_Stmt*Cyc_Absyn_skip_stmt(unsigned);
struct Cyc_Absyn_Stmt*Cyc_Absyn_exp_stmt(struct Cyc_Absyn_Exp*,unsigned);
struct Cyc_Absyn_Stmt*Cyc_Absyn_seq_stmt(struct Cyc_Absyn_Stmt*,struct Cyc_Absyn_Stmt*,unsigned);
struct Cyc_Absyn_Stmt*Cyc_Absyn_seq_stmts(struct Cyc_List_List*,unsigned);
# 1103
struct Cyc_Absyn_Pat*Cyc_Absyn_new_pat(void*,unsigned);
# 1107
struct Cyc_Absyn_Decl*Cyc_Absyn_new_decl(void*,unsigned);
# 1112
struct Cyc_Absyn_Vardecl*Cyc_Absyn_new_vardecl(unsigned varloc,struct _tuple0*,void*,struct Cyc_Absyn_Exp*init);
# 1118
struct Cyc_Absyn_Decl*Cyc_Absyn_aggr_decl(enum Cyc_Absyn_AggrKind,enum Cyc_Absyn_Scope,struct _tuple0*,struct Cyc_List_List*ts,struct Cyc_Absyn_AggrdeclImpl*,struct Cyc_List_List*,unsigned);
# 1149
void*Cyc_Absyn_pointer_expand(void*,int fresh_evar);
# 1151
int Cyc_Absyn_is_lvalue(struct Cyc_Absyn_Exp*);
# 1154
struct Cyc_Absyn_Aggrfield*Cyc_Absyn_lookup_field(struct Cyc_List_List*,struct _fat_ptr*);
# 1162
struct _fat_ptr*Cyc_Absyn_decl_name(struct Cyc_Absyn_Decl*decl);
# 1170
int Cyc_Absyn_attribute_cmp(void*,void*);
# 1182
struct _tuple0*Cyc_Absyn_binding2qvar(void*);
# 1193
void Cyc_Absyn_visit_stmt(int(*)(void*,struct Cyc_Absyn_Exp*),int(*)(void*,struct Cyc_Absyn_Stmt*),void*,struct Cyc_Absyn_Stmt*);
void Cyc_Absyn_visit_exp(int(*)(void*,struct Cyc_Absyn_Exp*),int(*)(void*,struct Cyc_Absyn_Stmt*),void*,struct Cyc_Absyn_Exp*);struct Cyc___cycFILE;struct Cyc_String_pa_PrintArg_struct{int tag;struct _fat_ptr f1;};struct Cyc_Int_pa_PrintArg_struct{int tag;unsigned long f1;};struct Cyc_Double_pa_PrintArg_struct{int tag;double f1;};struct Cyc_LongDouble_pa_PrintArg_struct{int tag;long double f1;};struct Cyc_ShortPtr_pa_PrintArg_struct{int tag;short*f1;};struct Cyc_IntPtr_pa_PrintArg_struct{int tag;unsigned long*f1;};
# 73 "cycboot.h"
struct _fat_ptr Cyc_aprintf(struct _fat_ptr,struct _fat_ptr);struct Cyc_ShortPtr_sa_ScanfArg_struct{int tag;short*f1;};struct Cyc_UShortPtr_sa_ScanfArg_struct{int tag;unsigned short*f1;};struct Cyc_IntPtr_sa_ScanfArg_struct{int tag;int*f1;};struct Cyc_UIntPtr_sa_ScanfArg_struct{int tag;unsigned*f1;};struct Cyc_StringPtr_sa_ScanfArg_struct{int tag;struct _fat_ptr f1;};struct Cyc_DoublePtr_sa_ScanfArg_struct{int tag;double*f1;};struct Cyc_FloatPtr_sa_ScanfArg_struct{int tag;float*f1;};struct Cyc_CharPtr_sa_ScanfArg_struct{int tag;struct _fat_ptr f1;};extern char Cyc_FileCloseError[15U];struct Cyc_FileCloseError_exn_struct{char*tag;};extern char Cyc_FileOpenError[14U];struct Cyc_FileOpenError_exn_struct{char*tag;struct _fat_ptr f1;};
# 35 "warn.h"
void Cyc_Warn_err(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap);
# 40
void*Cyc_Warn_impos(struct _fat_ptr fmt,struct _fat_ptr ap);struct Cyc_Warn_String_Warn_Warg_struct{int tag;struct _fat_ptr f1;};struct Cyc_Warn_Exp_Warn_Warg_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Warn_Typ_Warn_Warg_struct{int tag;void*f1;};struct Cyc_Warn_Qvar_Warn_Warg_struct{int tag;struct _tuple0*f1;};
# 49 "string.h"
int Cyc_strcmp(struct _fat_ptr s1,struct _fat_ptr s2);
int Cyc_strptrcmp(struct _fat_ptr*s1,struct _fat_ptr*s2);
# 52
int Cyc_zstrcmp(struct _fat_ptr,struct _fat_ptr);
# 109 "string.h"
struct _fat_ptr Cyc_substring(struct _fat_ptr,int ofs,unsigned long n);struct Cyc_Hashtable_Table;
# 47 "hashtable.h"
struct Cyc_Hashtable_Table*Cyc_Hashtable_rcreate(struct _RegionHandle*r,int sz,int(*cmp)(void*,void*),int(*hash)(void*));
# 50
void Cyc_Hashtable_insert(struct Cyc_Hashtable_Table*t,void*key,void*val);
# 56
void**Cyc_Hashtable_lookup_opt(struct Cyc_Hashtable_Table*t,void*key);struct _union_RelnOp_RConst{int tag;unsigned val;};struct _union_RelnOp_RVar{int tag;struct Cyc_Absyn_Vardecl*val;};struct _union_RelnOp_RNumelts{int tag;struct Cyc_Absyn_Vardecl*val;};struct _union_RelnOp_RType{int tag;void*val;};struct _union_RelnOp_RParam{int tag;unsigned val;};struct _union_RelnOp_RParamNumelts{int tag;unsigned val;};struct _union_RelnOp_RReturn{int tag;unsigned val;};union Cyc_Relations_RelnOp{struct _union_RelnOp_RConst RConst;struct _union_RelnOp_RVar RVar;struct _union_RelnOp_RNumelts RNumelts;struct _union_RelnOp_RType RType;struct _union_RelnOp_RParam RParam;struct _union_RelnOp_RParamNumelts RParamNumelts;struct _union_RelnOp_RReturn RReturn;};
# 50 "relations-ap.h"
enum Cyc_Relations_Relation{Cyc_Relations_Req =0U,Cyc_Relations_Rneq =1U,Cyc_Relations_Rlte =2U,Cyc_Relations_Rlt =3U};struct Cyc_Relations_Reln{union Cyc_Relations_RelnOp rop1;enum Cyc_Relations_Relation relation;union Cyc_Relations_RelnOp rop2;};struct Cyc_Iter_Iter{void*env;int(*next)(void*env,void*dest);};struct Cyc_Dict_T;struct Cyc_Dict_Dict{int(*rel)(void*,void*);struct _RegionHandle*r;const struct Cyc_Dict_T*t;};extern char Cyc_Dict_Present[8U];struct Cyc_Dict_Present_exn_struct{char*tag;};extern char Cyc_Dict_Absent[7U];struct Cyc_Dict_Absent_exn_struct{char*tag;};struct Cyc_RgnOrder_RgnPO;extern char Cyc_Tcenv_Env_error[10U];struct Cyc_Tcenv_Env_error_exn_struct{char*tag;};struct Cyc_Tcenv_Genv{struct Cyc_Dict_Dict aggrdecls;struct Cyc_Dict_Dict datatypedecls;struct Cyc_Dict_Dict enumdecls;struct Cyc_Dict_Dict typedefs;struct Cyc_Dict_Dict ordinaries;};struct Cyc_Tcenv_Fenv;struct Cyc_Tcenv_Tenv{struct Cyc_List_List*ns;struct Cyc_Tcenv_Genv*ae;struct Cyc_Tcenv_Fenv*le;int allow_valueof: 1;int in_extern_c_include: 1;int in_tempest: 1;int tempest_generalize: 1;};
# 88 "tcenv.h"
enum Cyc_Tcenv_NewStatus{Cyc_Tcenv_NoneNew =0U,Cyc_Tcenv_InNew =1U,Cyc_Tcenv_InNewAggr =2U};
# 107 "tcutil.h"
void*Cyc_Tcutil_compress(void*);
# 135
extern struct Cyc_Absyn_Kind Cyc_Tcutil_rk;
# 137
extern struct Cyc_Absyn_Kind Cyc_Tcutil_bk;
# 166
extern struct Cyc_Core_Opt Cyc_Tcutil_tmko;
# 174
void*Cyc_Tcutil_kind_to_bound(struct Cyc_Absyn_Kind*k);
# 26 "cyclone.h"
enum Cyc_Cyclone_C_Compilers{Cyc_Cyclone_Gcc_c =0U,Cyc_Cyclone_Vc_c =1U};
# 32
extern enum Cyc_Cyclone_C_Compilers Cyc_Cyclone_c_compiler;
# 45 "evexp.h"
int Cyc_Evexp_const_exp_cmp(struct Cyc_Absyn_Exp*e1,struct Cyc_Absyn_Exp*e2);
# 31 "absyn.cyc"
struct _fat_ptr Cyc_Absynpp_exp2string(struct Cyc_Absyn_Exp*e);
# 40
int Cyc_Cyclone_tovc_r=0;
# 42
enum Cyc_Cyclone_C_Compilers Cyc_Cyclone_c_compiler=Cyc_Cyclone_Gcc_c;
# 49
static int Cyc_Absyn_strlist_cmp(struct Cyc_List_List*ss1,struct Cyc_List_List*ss2){
for(0;ss1 != 0;ss1=ss1->tl){
if(ss2 == 0)return 1;{int i=
Cyc_strptrcmp((struct _fat_ptr*)ss1->hd,(struct _fat_ptr*)ss2->hd);
if(i != 0)return i;ss2=ss2->tl;}}
# 56
if(ss2 != 0)return - 1;return 0;}
# 49
int Cyc_Absyn_varlist_cmp(struct Cyc_List_List*vs1,struct Cyc_List_List*vs2){
# 60
if((int)vs1 == (int)vs2)return 0;return
 Cyc_Absyn_strlist_cmp(vs1,vs2);}struct _tuple11{union Cyc_Absyn_Nmspace f1;union Cyc_Absyn_Nmspace f2;};
# 49
int Cyc_Absyn_qvar_cmp(struct _tuple0*q1,struct _tuple0*q2){
# 64
if(q1 == q2)return 0;{struct _fat_ptr*_tmp1;union Cyc_Absyn_Nmspace _tmp0;_LL1: _tmp0=q1->f1;_tmp1=q1->f2;_LL2: {union Cyc_Absyn_Nmspace n1=_tmp0;struct _fat_ptr*v1=_tmp1;
# 66
struct _fat_ptr*_tmp3;union Cyc_Absyn_Nmspace _tmp2;_LL4: _tmp2=q2->f1;_tmp3=q2->f2;_LL5: {union Cyc_Absyn_Nmspace n2=_tmp2;struct _fat_ptr*v2=_tmp3;
int i=Cyc_strptrcmp(v1,v2);
if(i != 0)return i;{struct _tuple11 _stmttmp0=({struct _tuple11 _tmp233;_tmp233.f1=n1,_tmp233.f2=n2;_tmp233;});struct _tuple11 _tmp4=_stmttmp0;struct Cyc_List_List*_tmp6;struct Cyc_List_List*_tmp5;struct Cyc_List_List*_tmp8;struct Cyc_List_List*_tmp7;struct Cyc_List_List*_tmpA;struct Cyc_List_List*_tmp9;switch(((_tmp4.f1).Abs_n).tag){case 4U: if(((_tmp4.f2).Loc_n).tag == 4){_LL7: _LL8:
# 70
 return 0;}else{_LLF: _LL10:
# 75
 return - 1;}case 1U: switch(((_tmp4.f2).Loc_n).tag){case 1U: _LL9: _tmp9=((_tmp4.f1).Rel_n).val;_tmpA=((_tmp4.f2).Rel_n).val;_LLA: {struct Cyc_List_List*x1=_tmp9;struct Cyc_List_List*x2=_tmpA;
# 71
return Cyc_Absyn_strlist_cmp(x1,x2);}case 4U: goto _LL11;default: _LL13: _LL14:
# 77
 return - 1;}case 2U: switch(((_tmp4.f2).Rel_n).tag){case 2U: _LLB: _tmp7=((_tmp4.f1).Abs_n).val;_tmp8=((_tmp4.f2).Abs_n).val;_LLC: {struct Cyc_List_List*x1=_tmp7;struct Cyc_List_List*x2=_tmp8;
# 72
return Cyc_Absyn_strlist_cmp(x1,x2);}case 4U: goto _LL11;case 1U: goto _LL15;default: _LL17: _LL18:
# 79
 return - 1;}default: switch(((_tmp4.f2).Rel_n).tag){case 3U: _LLD: _tmp5=((_tmp4.f1).C_n).val;_tmp6=((_tmp4.f2).C_n).val;_LLE: {struct Cyc_List_List*x1=_tmp5;struct Cyc_List_List*x2=_tmp6;
# 73
return Cyc_Absyn_strlist_cmp(x1,x2);}case 4U: _LL11: _LL12:
# 76
 return 1;case 1U: _LL15: _LL16:
# 78
 return 1;default: _LL19: _LL1A:
# 80
 return 1;}}_LL6:;}}}}}
# 49
int Cyc_Absyn_tvar_cmp(struct Cyc_Absyn_Tvar*tv1,struct Cyc_Absyn_Tvar*tv2){
# 85
int i=Cyc_strptrcmp(tv1->name,tv2->name);
if(i != 0)return i;return tv1->identity - tv2->identity;}
# 49
union Cyc_Absyn_Nmspace Cyc_Absyn_Loc_n={.Loc_n={4,0}};
# 91
union Cyc_Absyn_Nmspace Cyc_Absyn_Abs_n(struct Cyc_List_List*x,int C_scope){
return C_scope?({union Cyc_Absyn_Nmspace _tmp234;(_tmp234.C_n).tag=3U,(_tmp234.C_n).val=x;_tmp234;}):({union Cyc_Absyn_Nmspace _tmp235;(_tmp235.Abs_n).tag=2U,(_tmp235.Abs_n).val=x;_tmp235;});}
# 91
union Cyc_Absyn_Nmspace Cyc_Absyn_Rel_n(struct Cyc_List_List*x){
# 94
return({union Cyc_Absyn_Nmspace _tmp236;(_tmp236.Rel_n).tag=1U,(_tmp236.Rel_n).val=x;_tmp236;});}
# 91
union Cyc_Absyn_Nmspace Cyc_Absyn_rel_ns_null={.Rel_n={1,0}};
# 97
int Cyc_Absyn_is_qvar_qualified(struct _tuple0*qv){
union Cyc_Absyn_Nmspace _stmttmp1=(*qv).f1;union Cyc_Absyn_Nmspace _tmpB=_stmttmp1;switch((_tmpB.Loc_n).tag){case 1U: if((_tmpB.Rel_n).val == 0){_LL1: _LL2:
 goto _LL4;}else{goto _LL7;}case 2U: if((_tmpB.Abs_n).val == 0){_LL3: _LL4:
 goto _LL6;}else{goto _LL7;}case 4U: _LL5: _LL6:
 return 0;default: _LL7: _LL8:
 return 1;}_LL0:;}
# 97
void*Cyc_Absyn_new_evar(struct Cyc_Core_Opt*k,struct Cyc_Core_Opt*env){
# 107
static int new_type_counter=0;
return(void*)({struct Cyc_Absyn_Evar_Absyn_Type_struct*_tmpC=_cycalloc(sizeof(*_tmpC));_tmpC->tag=1U,_tmpC->f1=k,_tmpC->f2=0,_tmpC->f3=new_type_counter ++,_tmpC->f4=env;_tmpC;});}
# 97
void*Cyc_Absyn_wildtyp(struct Cyc_Core_Opt*tenv){
# 111
return Cyc_Absyn_new_evar(& Cyc_Tcutil_tmko,tenv);}
# 97
struct Cyc_Absyn_Tqual Cyc_Absyn_empty_tqual(unsigned loc){
# 114
return({struct Cyc_Absyn_Tqual _tmp237;_tmp237.print_const=0,_tmp237.q_volatile=0,_tmp237.q_restrict=0,_tmp237.real_const=0,_tmp237.loc=loc;_tmp237;});}
# 97
struct Cyc_Absyn_Tqual Cyc_Absyn_const_tqual(unsigned loc){
# 115
return({struct Cyc_Absyn_Tqual _tmp238;_tmp238.print_const=1,_tmp238.q_volatile=0,_tmp238.q_restrict=0,_tmp238.real_const=1,_tmp238.loc=loc;_tmp238;});}
# 97
struct Cyc_Absyn_Tqual Cyc_Absyn_combine_tqual(struct Cyc_Absyn_Tqual x,struct Cyc_Absyn_Tqual y){
# 117
return({struct Cyc_Absyn_Tqual _tmp239;_tmp239.print_const=x.print_const || y.print_const,_tmp239.q_volatile=
x.q_volatile || y.q_volatile,_tmp239.q_restrict=
x.q_restrict || y.q_restrict,_tmp239.real_const=
x.real_const || y.real_const,({
unsigned _tmp26D=Cyc_Position_segment_join(x.loc,y.loc);_tmp239.loc=_tmp26D;});_tmp239;});}
# 97
int Cyc_Absyn_equal_tqual(struct Cyc_Absyn_Tqual tq1,struct Cyc_Absyn_Tqual tq2){
# 124
return(tq1.real_const == tq2.real_const && tq1.q_volatile == tq2.q_volatile)&& tq1.q_restrict == tq2.q_restrict;}
# 97
struct Cyc_Absyn_EmptyAnnot_Absyn_AbsynAnnot_struct Cyc_Absyn_EmptyAnnot_val={Cyc_Absyn_EmptyAnnot};
# 131
union Cyc_Absyn_DatatypeInfo Cyc_Absyn_UnknownDatatype(struct Cyc_Absyn_UnknownDatatypeInfo udi){
return({union Cyc_Absyn_DatatypeInfo _tmp23A;(_tmp23A.UnknownDatatype).tag=1U,(_tmp23A.UnknownDatatype).val=udi;_tmp23A;});}
# 131
union Cyc_Absyn_DatatypeInfo Cyc_Absyn_KnownDatatype(struct Cyc_Absyn_Datatypedecl**d){
# 135
return({union Cyc_Absyn_DatatypeInfo _tmp23B;(_tmp23B.KnownDatatype).tag=2U,(_tmp23B.KnownDatatype).val=d;_tmp23B;});}
# 131
union Cyc_Absyn_DatatypeFieldInfo Cyc_Absyn_UnknownDatatypefield(struct Cyc_Absyn_UnknownDatatypeFieldInfo s){
# 138
return({union Cyc_Absyn_DatatypeFieldInfo _tmp23C;(_tmp23C.UnknownDatatypefield).tag=1U,(_tmp23C.UnknownDatatypefield).val=s;_tmp23C;});}
# 131
union Cyc_Absyn_DatatypeFieldInfo Cyc_Absyn_KnownDatatypefield(struct Cyc_Absyn_Datatypedecl*dd,struct Cyc_Absyn_Datatypefield*df){
# 141
return({union Cyc_Absyn_DatatypeFieldInfo _tmp23D;(_tmp23D.KnownDatatypefield).tag=2U,((_tmp23D.KnownDatatypefield).val).f1=dd,((_tmp23D.KnownDatatypefield).val).f2=df;_tmp23D;});}
# 131
union Cyc_Absyn_AggrInfo Cyc_Absyn_UnknownAggr(enum Cyc_Absyn_AggrKind ak,struct _tuple0*n,struct Cyc_Core_Opt*tagged){
# 144
return({union Cyc_Absyn_AggrInfo _tmp23E;(_tmp23E.UnknownAggr).tag=1U,((_tmp23E.UnknownAggr).val).f1=ak,((_tmp23E.UnknownAggr).val).f2=n,((_tmp23E.UnknownAggr).val).f3=tagged;_tmp23E;});}
# 131
union Cyc_Absyn_AggrInfo Cyc_Absyn_KnownAggr(struct Cyc_Absyn_Aggrdecl**ad){
# 147
return({union Cyc_Absyn_AggrInfo _tmp23F;(_tmp23F.KnownAggr).tag=2U,(_tmp23F.KnownAggr).val=ad;_tmp23F;});}
# 131
void*Cyc_Absyn_compress_kb(void*k){
# 150
void*_tmpD=k;void**_tmpE;void**_tmpF;switch(*((int*)_tmpD)){case 0U: _LL1: _LL2:
 goto _LL4;case 1U: if(((struct Cyc_Absyn_Unknown_kb_Absyn_KindBound_struct*)_tmpD)->f1 == 0){_LL3: _LL4:
 goto _LL6;}else{_LL7: _tmpF=(void**)&(((struct Cyc_Absyn_Unknown_kb_Absyn_KindBound_struct*)_tmpD)->f1)->v;_LL8: {void**k2=_tmpF;
# 154
_tmpE=k2;goto _LLA;}}default: if(((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmpD)->f1 == 0){_LL5: _LL6:
# 153
 return k;}else{_LL9: _tmpE=(void**)&(((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmpD)->f1)->v;_LLA: {void**k2=_tmpE;
# 156
({void*_tmp26E=Cyc_Absyn_compress_kb(*k2);*k2=_tmp26E;});
return*k2;}}}_LL0:;}
# 131
struct Cyc_Absyn_Kind*Cyc_Absyn_force_kb(void*kb){
# 161
void*_stmttmp2=Cyc_Absyn_compress_kb(kb);void*_tmp10=_stmttmp2;struct Cyc_Absyn_Kind*_tmp12;struct Cyc_Core_Opt**_tmp11;struct Cyc_Core_Opt**_tmp13;struct Cyc_Absyn_Kind*_tmp14;switch(*((int*)_tmp10)){case 0U: _LL1: _tmp14=((struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct*)_tmp10)->f1;_LL2: {struct Cyc_Absyn_Kind*k=_tmp14;
return k;}case 1U: _LL3: _tmp13=(struct Cyc_Core_Opt**)&((struct Cyc_Absyn_Unknown_kb_Absyn_KindBound_struct*)_tmp10)->f1;_LL4: {struct Cyc_Core_Opt**f=_tmp13;
_tmp11=f;_tmp12=& Cyc_Tcutil_bk;goto _LL6;}default: _LL5: _tmp11=(struct Cyc_Core_Opt**)&((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp10)->f1;_tmp12=((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp10)->f2;_LL6: {struct Cyc_Core_Opt**f=_tmp11;struct Cyc_Absyn_Kind*k=_tmp12;
# 165
({struct Cyc_Core_Opt*_tmp270=({struct Cyc_Core_Opt*_tmp15=_cycalloc(sizeof(*_tmp15));({void*_tmp26F=Cyc_Tcutil_kind_to_bound(k);_tmp15->v=_tmp26F;});_tmp15;});*f=_tmp270;});
return k;}}_LL0:;}
# 131
void*Cyc_Absyn_app_type(void*c,struct _fat_ptr args){
# 172
return(void*)({struct Cyc_Absyn_AppType_Absyn_Type_struct*_tmp16=_cycalloc(sizeof(*_tmp16));_tmp16->tag=0U,_tmp16->f1=c,({struct Cyc_List_List*_tmp271=Cyc_List_from_array(args);_tmp16->f2=_tmp271;});_tmp16;});}
# 131
void*Cyc_Absyn_valueof_type(struct Cyc_Absyn_Exp*e){
# 175
return(void*)({struct Cyc_Absyn_ValueofType_Absyn_Type_struct*_tmp17=_cycalloc(sizeof(*_tmp17));_tmp17->tag=9U,_tmp17->f1=e;_tmp17;});}
# 131
static struct Cyc_Absyn_VoidCon_Absyn_TyCon_struct Cyc_Absyn_void_type_cval={0U};
# 190 "absyn.cyc"
static struct Cyc_Absyn_AppType_Absyn_Type_struct Cyc_Absyn_void_type_tval={0U,(void*)& Cyc_Absyn_void_type_cval,0};void*Cyc_Absyn_void_type=(void*)& Cyc_Absyn_void_type_tval;
static struct Cyc_Absyn_HeapCon_Absyn_TyCon_struct Cyc_Absyn_heap_rgn_type_cval={5U};static struct Cyc_Absyn_AppType_Absyn_Type_struct Cyc_Absyn_heap_rgn_type_tval={0U,(void*)& Cyc_Absyn_heap_rgn_type_cval,0};void*Cyc_Absyn_heap_rgn_type=(void*)& Cyc_Absyn_heap_rgn_type_tval;
static struct Cyc_Absyn_UniqueCon_Absyn_TyCon_struct Cyc_Absyn_unique_rgn_type_cval={6U};static struct Cyc_Absyn_AppType_Absyn_Type_struct Cyc_Absyn_unique_rgn_type_tval={0U,(void*)& Cyc_Absyn_unique_rgn_type_cval,0};void*Cyc_Absyn_unique_rgn_type=(void*)& Cyc_Absyn_unique_rgn_type_tval;
static struct Cyc_Absyn_RefCntCon_Absyn_TyCon_struct Cyc_Absyn_refcnt_rgn_type_cval={7U};static struct Cyc_Absyn_AppType_Absyn_Type_struct Cyc_Absyn_refcnt_rgn_type_tval={0U,(void*)& Cyc_Absyn_refcnt_rgn_type_cval,0};void*Cyc_Absyn_refcnt_rgn_type=(void*)& Cyc_Absyn_refcnt_rgn_type_tval;
static struct Cyc_Absyn_TrueCon_Absyn_TyCon_struct Cyc_Absyn_true_type_cval={11U};static struct Cyc_Absyn_AppType_Absyn_Type_struct Cyc_Absyn_true_type_tval={0U,(void*)& Cyc_Absyn_true_type_cval,0};void*Cyc_Absyn_true_type=(void*)& Cyc_Absyn_true_type_tval;
static struct Cyc_Absyn_FalseCon_Absyn_TyCon_struct Cyc_Absyn_false_type_cval={12U};static struct Cyc_Absyn_AppType_Absyn_Type_struct Cyc_Absyn_false_type_tval={0U,(void*)& Cyc_Absyn_false_type_cval,0};void*Cyc_Absyn_false_type=(void*)& Cyc_Absyn_false_type_tval;
static struct Cyc_Absyn_FatCon_Absyn_TyCon_struct Cyc_Absyn_fat_bound_type_cval={14U};static struct Cyc_Absyn_AppType_Absyn_Type_struct Cyc_Absyn_fat_bound_type_tval={0U,(void*)& Cyc_Absyn_fat_bound_type_cval,0};void*Cyc_Absyn_fat_bound_type=(void*)& Cyc_Absyn_fat_bound_type_tval;
static struct Cyc_Absyn_IntCon_Absyn_TyCon_struct Cyc_Absyn_schar_type_cval={1U,Cyc_Absyn_Signed,Cyc_Absyn_Char_sz};static struct Cyc_Absyn_AppType_Absyn_Type_struct Cyc_Absyn_schar_type_tval={0U,(void*)& Cyc_Absyn_schar_type_cval,0};void*Cyc_Absyn_schar_type=(void*)& Cyc_Absyn_schar_type_tval;
static struct Cyc_Absyn_IntCon_Absyn_TyCon_struct Cyc_Absyn_uchar_type_cval={1U,Cyc_Absyn_Unsigned,Cyc_Absyn_Char_sz};static struct Cyc_Absyn_AppType_Absyn_Type_struct Cyc_Absyn_uchar_type_tval={0U,(void*)& Cyc_Absyn_uchar_type_cval,0};void*Cyc_Absyn_uchar_type=(void*)& Cyc_Absyn_uchar_type_tval;
static struct Cyc_Absyn_IntCon_Absyn_TyCon_struct Cyc_Absyn_char_type_cval={1U,Cyc_Absyn_None,Cyc_Absyn_Char_sz};static struct Cyc_Absyn_AppType_Absyn_Type_struct Cyc_Absyn_char_type_tval={0U,(void*)& Cyc_Absyn_char_type_cval,0};void*Cyc_Absyn_char_type=(void*)& Cyc_Absyn_char_type_tval;
static struct Cyc_Absyn_IntCon_Absyn_TyCon_struct Cyc_Absyn_sshort_type_cval={1U,Cyc_Absyn_Signed,Cyc_Absyn_Short_sz};static struct Cyc_Absyn_AppType_Absyn_Type_struct Cyc_Absyn_sshort_type_tval={0U,(void*)& Cyc_Absyn_sshort_type_cval,0};void*Cyc_Absyn_sshort_type=(void*)& Cyc_Absyn_sshort_type_tval;
static struct Cyc_Absyn_IntCon_Absyn_TyCon_struct Cyc_Absyn_ushort_type_cval={1U,Cyc_Absyn_Unsigned,Cyc_Absyn_Short_sz};static struct Cyc_Absyn_AppType_Absyn_Type_struct Cyc_Absyn_ushort_type_tval={0U,(void*)& Cyc_Absyn_ushort_type_cval,0};void*Cyc_Absyn_ushort_type=(void*)& Cyc_Absyn_ushort_type_tval;
static struct Cyc_Absyn_IntCon_Absyn_TyCon_struct Cyc_Absyn_nshort_type_cval={1U,Cyc_Absyn_None,Cyc_Absyn_Short_sz};static struct Cyc_Absyn_AppType_Absyn_Type_struct Cyc_Absyn_nshort_type_tval={0U,(void*)& Cyc_Absyn_nshort_type_cval,0};void*Cyc_Absyn_nshort_type=(void*)& Cyc_Absyn_nshort_type_tval;
static struct Cyc_Absyn_IntCon_Absyn_TyCon_struct Cyc_Absyn_sint_type_cval={1U,Cyc_Absyn_Signed,Cyc_Absyn_Int_sz};static struct Cyc_Absyn_AppType_Absyn_Type_struct Cyc_Absyn_sint_type_tval={0U,(void*)& Cyc_Absyn_sint_type_cval,0};void*Cyc_Absyn_sint_type=(void*)& Cyc_Absyn_sint_type_tval;
static struct Cyc_Absyn_IntCon_Absyn_TyCon_struct Cyc_Absyn_uint_type_cval={1U,Cyc_Absyn_Unsigned,Cyc_Absyn_Int_sz};static struct Cyc_Absyn_AppType_Absyn_Type_struct Cyc_Absyn_uint_type_tval={0U,(void*)& Cyc_Absyn_uint_type_cval,0};void*Cyc_Absyn_uint_type=(void*)& Cyc_Absyn_uint_type_tval;
static struct Cyc_Absyn_IntCon_Absyn_TyCon_struct Cyc_Absyn_nint_type_cval={1U,Cyc_Absyn_None,Cyc_Absyn_Int_sz};static struct Cyc_Absyn_AppType_Absyn_Type_struct Cyc_Absyn_nint_type_tval={0U,(void*)& Cyc_Absyn_nint_type_cval,0};void*Cyc_Absyn_nint_type=(void*)& Cyc_Absyn_nint_type_tval;
static struct Cyc_Absyn_IntCon_Absyn_TyCon_struct Cyc_Absyn_slong_type_cval={1U,Cyc_Absyn_Signed,Cyc_Absyn_Long_sz};static struct Cyc_Absyn_AppType_Absyn_Type_struct Cyc_Absyn_slong_type_tval={0U,(void*)& Cyc_Absyn_slong_type_cval,0};void*Cyc_Absyn_slong_type=(void*)& Cyc_Absyn_slong_type_tval;
static struct Cyc_Absyn_IntCon_Absyn_TyCon_struct Cyc_Absyn_ulong_type_cval={1U,Cyc_Absyn_Unsigned,Cyc_Absyn_Long_sz};static struct Cyc_Absyn_AppType_Absyn_Type_struct Cyc_Absyn_ulong_type_tval={0U,(void*)& Cyc_Absyn_ulong_type_cval,0};void*Cyc_Absyn_ulong_type=(void*)& Cyc_Absyn_ulong_type_tval;
static struct Cyc_Absyn_IntCon_Absyn_TyCon_struct Cyc_Absyn_nlong_type_cval={1U,Cyc_Absyn_None,Cyc_Absyn_Long_sz};static struct Cyc_Absyn_AppType_Absyn_Type_struct Cyc_Absyn_nlong_type_tval={0U,(void*)& Cyc_Absyn_nlong_type_cval,0};void*Cyc_Absyn_nlong_type=(void*)& Cyc_Absyn_nlong_type_tval;
static struct Cyc_Absyn_IntCon_Absyn_TyCon_struct Cyc_Absyn_slonglong_type_cval={1U,Cyc_Absyn_Signed,Cyc_Absyn_LongLong_sz};static struct Cyc_Absyn_AppType_Absyn_Type_struct Cyc_Absyn_slonglong_type_tval={0U,(void*)& Cyc_Absyn_slonglong_type_cval,0};void*Cyc_Absyn_slonglong_type=(void*)& Cyc_Absyn_slonglong_type_tval;
static struct Cyc_Absyn_IntCon_Absyn_TyCon_struct Cyc_Absyn_ulonglong_type_cval={1U,Cyc_Absyn_Unsigned,Cyc_Absyn_LongLong_sz};static struct Cyc_Absyn_AppType_Absyn_Type_struct Cyc_Absyn_ulonglong_type_tval={0U,(void*)& Cyc_Absyn_ulonglong_type_cval,0};void*Cyc_Absyn_ulonglong_type=(void*)& Cyc_Absyn_ulonglong_type_tval;
static struct Cyc_Absyn_IntCon_Absyn_TyCon_struct Cyc_Absyn_nlonglong_type_cval={1U,Cyc_Absyn_None,Cyc_Absyn_LongLong_sz};static struct Cyc_Absyn_AppType_Absyn_Type_struct Cyc_Absyn_nlonglong_type_tval={0U,(void*)& Cyc_Absyn_nlonglong_type_cval,0};void*Cyc_Absyn_nlonglong_type=(void*)& Cyc_Absyn_nlonglong_type_tval;
static struct Cyc_Absyn_FloatCon_Absyn_TyCon_struct Cyc_Absyn_float_type_cval={2U,0};static struct Cyc_Absyn_AppType_Absyn_Type_struct Cyc_Absyn_float_type_tval={0U,(void*)& Cyc_Absyn_float_type_cval,0};void*Cyc_Absyn_float_type=(void*)& Cyc_Absyn_float_type_tval;
static struct Cyc_Absyn_FloatCon_Absyn_TyCon_struct Cyc_Absyn_double_type_cval={2U,1};static struct Cyc_Absyn_AppType_Absyn_Type_struct Cyc_Absyn_double_type_tval={0U,(void*)& Cyc_Absyn_double_type_cval,0};void*Cyc_Absyn_double_type=(void*)& Cyc_Absyn_double_type_tval;
static struct Cyc_Absyn_FloatCon_Absyn_TyCon_struct Cyc_Absyn_long_double_type_cval={2U,2};static struct Cyc_Absyn_AppType_Absyn_Type_struct Cyc_Absyn_long_double_type_tval={0U,(void*)& Cyc_Absyn_long_double_type_cval,0};void*Cyc_Absyn_long_double_type=(void*)& Cyc_Absyn_long_double_type_tval;
# 216
static struct Cyc_Absyn_JoinCon_Absyn_TyCon_struct Cyc_Absyn_empty_effect_cval={9U};static struct Cyc_Absyn_AppType_Absyn_Type_struct Cyc_Absyn_empty_effect_tval={0U,(void*)& Cyc_Absyn_empty_effect_cval,0};void*Cyc_Absyn_empty_effect=(void*)& Cyc_Absyn_empty_effect_tval;
# 218
static struct Cyc_Absyn_RgnHandleCon_Absyn_TyCon_struct Cyc_Absyn_RgnHandleCon_val={3U};
static struct Cyc_Absyn_TagCon_Absyn_TyCon_struct Cyc_Absyn_TagCon_val={4U};
static struct Cyc_Absyn_AccessCon_Absyn_TyCon_struct Cyc_Absyn_AccessCon_val={8U};
static struct Cyc_Absyn_RgnsCon_Absyn_TyCon_struct Cyc_Absyn_RgnsCon_val={10U};
static struct Cyc_Absyn_ThinCon_Absyn_TyCon_struct Cyc_Absyn_ThinCon_val={13U};
static struct Cyc_Absyn_JoinCon_Absyn_TyCon_struct Cyc_Absyn_JoinCon_val={9U};
void*Cyc_Absyn_rgn_handle_type(void*r){return({void*_tmp18[1U];_tmp18[0]=r;Cyc_Absyn_app_type((void*)& Cyc_Absyn_RgnHandleCon_val,_tag_fat(_tmp18,sizeof(void*),1U));});}void*Cyc_Absyn_tag_type(void*t){
return({void*_tmp19[1U];_tmp19[0]=t;Cyc_Absyn_app_type((void*)& Cyc_Absyn_TagCon_val,_tag_fat(_tmp19,sizeof(void*),1U));});}
# 224
void*Cyc_Absyn_access_eff(void*r){
# 226
return({void*_tmp1A[1U];_tmp1A[0]=r;Cyc_Absyn_app_type((void*)& Cyc_Absyn_AccessCon_val,_tag_fat(_tmp1A,sizeof(void*),1U));});}
# 224
void*Cyc_Absyn_regionsof_eff(void*t){
# 227
return({void*_tmp1B[1U];_tmp1B[0]=t;Cyc_Absyn_app_type((void*)& Cyc_Absyn_RgnsCon_val,_tag_fat(_tmp1B,sizeof(void*),1U));});}
# 224
void*Cyc_Absyn_thin_bounds_type(void*t){
# 228
return({void*_tmp1C[1U];_tmp1C[0]=t;Cyc_Absyn_app_type((void*)& Cyc_Absyn_ThinCon_val,_tag_fat(_tmp1C,sizeof(void*),1U));});}
# 224
void*Cyc_Absyn_join_eff(struct Cyc_List_List*ts){
# 229
return(void*)({struct Cyc_Absyn_AppType_Absyn_Type_struct*_tmp1D=_cycalloc(sizeof(*_tmp1D));_tmp1D->tag=0U,_tmp1D->f1=(void*)& Cyc_Absyn_empty_effect_cval,_tmp1D->f2=ts;_tmp1D;});}
# 224
void*Cyc_Absyn_enum_type(struct _tuple0*n,struct Cyc_Absyn_Enumdecl*d){
# 232
return({void*_tmp1E=0U;({void*_tmp272=(void*)({struct Cyc_Absyn_EnumCon_Absyn_TyCon_struct*_tmp1F=_cycalloc(sizeof(*_tmp1F));_tmp1F->tag=15U,_tmp1F->f1=n,_tmp1F->f2=d;_tmp1F;});Cyc_Absyn_app_type(_tmp272,_tag_fat(_tmp1E,sizeof(void*),0U));});});}
# 224
void*Cyc_Absyn_anon_enum_type(struct Cyc_List_List*fs){
# 235
return({void*_tmp20=0U;({void*_tmp273=(void*)({struct Cyc_Absyn_AnonEnumCon_Absyn_TyCon_struct*_tmp21=_cycalloc(sizeof(*_tmp21));_tmp21->tag=16U,_tmp21->f1=fs;_tmp21;});Cyc_Absyn_app_type(_tmp273,_tag_fat(_tmp20,sizeof(void*),0U));});});}
# 224
void*Cyc_Absyn_builtin_type(struct _fat_ptr s,struct Cyc_Absyn_Kind*k){
# 238
return({void*_tmp22=0U;({void*_tmp274=(void*)({struct Cyc_Absyn_BuiltinCon_Absyn_TyCon_struct*_tmp23=_cycalloc(sizeof(*_tmp23));_tmp23->tag=17U,_tmp23->f1=s,_tmp23->f2=k;_tmp23;});Cyc_Absyn_app_type(_tmp274,_tag_fat(_tmp22,sizeof(void*),0U));});});}
# 224
void*Cyc_Absyn_datatype_type(union Cyc_Absyn_DatatypeInfo di,struct Cyc_List_List*args){
# 241
return(void*)({struct Cyc_Absyn_AppType_Absyn_Type_struct*_tmp25=_cycalloc(sizeof(*_tmp25));_tmp25->tag=0U,({void*_tmp275=(void*)({struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct*_tmp24=_cycalloc(sizeof(*_tmp24));_tmp24->tag=18U,_tmp24->f1=di;_tmp24;});_tmp25->f1=_tmp275;}),_tmp25->f2=args;_tmp25;});}
# 224
void*Cyc_Absyn_datatype_field_type(union Cyc_Absyn_DatatypeFieldInfo di,struct Cyc_List_List*args){
# 244
return(void*)({struct Cyc_Absyn_AppType_Absyn_Type_struct*_tmp27=_cycalloc(sizeof(*_tmp27));_tmp27->tag=0U,({void*_tmp276=(void*)({struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct*_tmp26=_cycalloc(sizeof(*_tmp26));_tmp26->tag=19U,_tmp26->f1=di;_tmp26;});_tmp27->f1=_tmp276;}),_tmp27->f2=args;_tmp27;});}
# 224
void*Cyc_Absyn_aggr_type(union Cyc_Absyn_AggrInfo ai,struct Cyc_List_List*args){
# 247
return(void*)({struct Cyc_Absyn_AppType_Absyn_Type_struct*_tmp29=_cycalloc(sizeof(*_tmp29));_tmp29->tag=0U,({void*_tmp277=(void*)({struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*_tmp28=_cycalloc(sizeof(*_tmp28));_tmp28->tag=20U,_tmp28->f1=ai;_tmp28;});_tmp29->f1=_tmp277;}),_tmp29->f2=args;_tmp29;});}
# 224
void*Cyc_Absyn_var_type(struct Cyc_Absyn_Tvar*x){
# 250
return(void*)({struct Cyc_Absyn_VarType_Absyn_Type_struct*_tmp2A=_cycalloc(sizeof(*_tmp2A));_tmp2A->tag=2U,_tmp2A->f1=x;_tmp2A;});}
# 224
void*Cyc_Absyn_gen_float_type(unsigned i){
# 252
unsigned _tmp2B=i;switch(_tmp2B){case 0U: _LL1: _LL2:
 return Cyc_Absyn_float_type;case 1U: _LL3: _LL4:
 return Cyc_Absyn_double_type;case 2U: _LL5: _LL6:
 return Cyc_Absyn_long_double_type;default: _LL7: _LL8:
({struct Cyc_Int_pa_PrintArg_struct _tmp2F=({struct Cyc_Int_pa_PrintArg_struct _tmp240;_tmp240.tag=1U,_tmp240.f1=(unsigned long)((int)i);_tmp240;});void*_tmp2C[1U];_tmp2C[0]=& _tmp2F;({int(*_tmp279)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp2E)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmp2E;});struct _fat_ptr _tmp278=({const char*_tmp2D="gen_float_type(%d)";_tag_fat(_tmp2D,sizeof(char),19U);});_tmp279(_tmp278,_tag_fat(_tmp2C,sizeof(void*),1U));});});}_LL0:;}
# 224
void*Cyc_Absyn_int_type(enum Cyc_Absyn_Sign sn,enum Cyc_Absyn_Size_of sz){
# 260
enum Cyc_Absyn_Sign _tmp30=sn;switch(_tmp30){case Cyc_Absyn_Signed: _LL1: _LL2: {
# 262
enum Cyc_Absyn_Size_of _tmp31=sz;switch(_tmp31){case Cyc_Absyn_Char_sz: _LLA: _LLB:
 return Cyc_Absyn_schar_type;case Cyc_Absyn_Short_sz: _LLC: _LLD:
 return Cyc_Absyn_sshort_type;case Cyc_Absyn_Int_sz: _LLE: _LLF:
 return Cyc_Absyn_sint_type;case Cyc_Absyn_Long_sz: _LL10: _LL11:
 return Cyc_Absyn_slong_type;case Cyc_Absyn_LongLong_sz: _LL12: _LL13:
 goto _LL15;default: _LL14: _LL15:
 return Cyc_Absyn_slonglong_type;}_LL9:;}case Cyc_Absyn_Unsigned: _LL3: _LL4: {
# 271
enum Cyc_Absyn_Size_of _tmp32=sz;switch(_tmp32){case Cyc_Absyn_Char_sz: _LL17: _LL18:
 return Cyc_Absyn_uchar_type;case Cyc_Absyn_Short_sz: _LL19: _LL1A:
 return Cyc_Absyn_ushort_type;case Cyc_Absyn_Int_sz: _LL1B: _LL1C:
 return Cyc_Absyn_uint_type;case Cyc_Absyn_Long_sz: _LL1D: _LL1E:
 return Cyc_Absyn_ulong_type;case Cyc_Absyn_LongLong_sz: _LL1F: _LL20:
 goto _LL22;default: _LL21: _LL22:
 return Cyc_Absyn_ulonglong_type;}_LL16:;}case Cyc_Absyn_None: _LL5: _LL6:
# 279
 goto _LL8;default: _LL7: _LL8: {
# 281
enum Cyc_Absyn_Size_of _tmp33=sz;switch(_tmp33){case Cyc_Absyn_Char_sz: _LL24: _LL25:
 return Cyc_Absyn_char_type;case Cyc_Absyn_Short_sz: _LL26: _LL27:
 return Cyc_Absyn_nshort_type;case Cyc_Absyn_Int_sz: _LL28: _LL29:
 return Cyc_Absyn_nint_type;case Cyc_Absyn_Long_sz: _LL2A: _LL2B:
 return Cyc_Absyn_nlong_type;case Cyc_Absyn_LongLong_sz: _LL2C: _LL2D:
 goto _LL2F;default: _LL2E: _LL2F:
 return Cyc_Absyn_nlonglong_type;}_LL23:;}}_LL0:;}
# 224
void*Cyc_Absyn_thin_bounds_exp(struct Cyc_Absyn_Exp*e){
# 293
return Cyc_Absyn_thin_bounds_type(Cyc_Absyn_valueof_type(e));}
# 224
void*Cyc_Absyn_thin_bounds_int(unsigned i){
# 296
struct Cyc_Absyn_Exp*e=Cyc_Absyn_uint_exp(i,0U);
e->topt=Cyc_Absyn_uint_type;
return Cyc_Absyn_thin_bounds_exp(e);}
# 224
void*Cyc_Absyn_bounds_one(){
# 301
static void*bone=0;
void*b=bone;
if(b == 0){
b=Cyc_Absyn_thin_bounds_int(1U);
bone=b;}
# 303
return b;}
# 313
extern int Wchar_t_unsigned;
extern int Sizeof_wchar_t;
# 316
void*Cyc_Absyn_wchar_type(){
int _tmp34=Sizeof_wchar_t;switch(_tmp34){case 1U: _LL1: _LL2:
# 327 "absyn.cyc"
 return Wchar_t_unsigned?Cyc_Absyn_uchar_type: Cyc_Absyn_schar_type;case 2U: _LL3: _LL4:
 return Wchar_t_unsigned?Cyc_Absyn_ushort_type: Cyc_Absyn_sshort_type;default: _LL5: _LL6:
# 331
 return Wchar_t_unsigned?Cyc_Absyn_uint_type: Cyc_Absyn_sint_type;}_LL0:;}static char _tmp35[4U]="exn";
# 316 "absyn.cyc"
static struct _fat_ptr Cyc_Absyn_exn_str={_tmp35,_tmp35,_tmp35 + 4U};
# 337 "absyn.cyc"
static struct _tuple0 Cyc_Absyn_exn_name_v={{.Abs_n={2,0}},& Cyc_Absyn_exn_str};
struct _tuple0*Cyc_Absyn_exn_name=& Cyc_Absyn_exn_name_v;static char _tmp3C[15U]="Null_Exception";static char _tmp3D[13U]="Array_bounds";static char _tmp3E[16U]="Match_Exception";static char _tmp3F[10U]="Bad_alloc";
# 340
struct Cyc_Absyn_Datatypedecl*Cyc_Absyn_exn_tud(){
static struct _fat_ptr builtin_exns[4U]={{_tmp3C,_tmp3C,_tmp3C + 15U},{_tmp3D,_tmp3D,_tmp3D + 13U},{_tmp3E,_tmp3E,_tmp3E + 16U},{_tmp3F,_tmp3F,_tmp3F + 10U}};
# 343
static struct Cyc_Absyn_Datatypedecl*tud_opt=0;
if(tud_opt == 0){
struct Cyc_List_List*tufs=0;
{int i=0;for(0;(unsigned)i < 4U;++ i){
tufs=({struct Cyc_List_List*_tmp39=_cycalloc(sizeof(*_tmp39));({struct Cyc_Absyn_Datatypefield*_tmp27C=({struct Cyc_Absyn_Datatypefield*_tmp38=_cycalloc(sizeof(*_tmp38));({struct _tuple0*_tmp27B=({struct _tuple0*_tmp37=_cycalloc(sizeof(*_tmp37));((_tmp37->f1).Abs_n).tag=2U,((_tmp37->f1).Abs_n).val=0,({
struct _fat_ptr*_tmp27A=({struct _fat_ptr*_tmp36=_cycalloc(sizeof(*_tmp36));*_tmp36=*((struct _fat_ptr*)_check_known_subscript_notnull(builtin_exns,4U,sizeof(struct _fat_ptr),i));_tmp36;});_tmp37->f2=_tmp27A;});_tmp37;});
# 347
_tmp38->name=_tmp27B;}),_tmp38->typs=0,_tmp38->loc=0U,_tmp38->sc=Cyc_Absyn_Extern;_tmp38;});_tmp39->hd=_tmp27C;}),_tmp39->tl=tufs;_tmp39;});}}
# 351
tud_opt=({struct Cyc_Absyn_Datatypedecl*_tmp3B=_cycalloc(sizeof(*_tmp3B));_tmp3B->sc=Cyc_Absyn_Extern,_tmp3B->name=Cyc_Absyn_exn_name,_tmp3B->tvs=0,({struct Cyc_Core_Opt*_tmp27D=({struct Cyc_Core_Opt*_tmp3A=_cycalloc(sizeof(*_tmp3A));_tmp3A->v=tufs;_tmp3A;});_tmp3B->fields=_tmp27D;}),_tmp3B->is_extensible=1;_tmp3B;});}
# 344
return(struct Cyc_Absyn_Datatypedecl*)_check_null(tud_opt);}
# 340
void*Cyc_Absyn_exn_type(){
# 357
static void*exn_typ=0;
static void*eopt=0;
if(exn_typ == 0){
eopt=Cyc_Absyn_datatype_type(({union Cyc_Absyn_DatatypeInfo _tmp241;(_tmp241.KnownDatatype).tag=2U,({struct Cyc_Absyn_Datatypedecl**_tmp27F=({struct Cyc_Absyn_Datatypedecl**_tmp40=_cycalloc(sizeof(*_tmp40));({struct Cyc_Absyn_Datatypedecl*_tmp27E=Cyc_Absyn_exn_tud();*_tmp40=_tmp27E;});_tmp40;});(_tmp241.KnownDatatype).val=_tmp27F;});_tmp241;}),0);
exn_typ=({void*_tmp282=(void*)_check_null(eopt);void*_tmp281=Cyc_Absyn_heap_rgn_type;struct Cyc_Absyn_Tqual _tmp280=Cyc_Absyn_empty_tqual(0U);Cyc_Absyn_at_type(_tmp282,_tmp281,_tmp280,Cyc_Absyn_false_type);});}
# 359
return(void*)_check_null(exn_typ);}
# 340
struct _tuple0*Cyc_Absyn_datatype_print_arg_qvar(){
# 367
static struct _tuple0*q=0;
if(q == 0)
q=({struct _tuple0*_tmp43=_cycalloc(sizeof(*_tmp43));({union Cyc_Absyn_Nmspace _tmp285=Cyc_Absyn_Abs_n(0,0);_tmp43->f1=_tmp285;}),({struct _fat_ptr*_tmp284=({struct _fat_ptr*_tmp42=_cycalloc(sizeof(*_tmp42));({struct _fat_ptr _tmp283=({const char*_tmp41="PrintArg";_tag_fat(_tmp41,sizeof(char),9U);});*_tmp42=_tmp283;});_tmp42;});_tmp43->f2=_tmp284;});_tmp43;});
# 368
return(struct _tuple0*)_check_null(q);}
# 340
struct _tuple0*Cyc_Absyn_datatype_scanf_arg_qvar(){
# 373
static struct _tuple0*q=0;
if(q == 0)
q=({struct _tuple0*_tmp46=_cycalloc(sizeof(*_tmp46));({union Cyc_Absyn_Nmspace _tmp288=Cyc_Absyn_Abs_n(0,0);_tmp46->f1=_tmp288;}),({struct _fat_ptr*_tmp287=({struct _fat_ptr*_tmp45=_cycalloc(sizeof(*_tmp45));({struct _fat_ptr _tmp286=({const char*_tmp44="ScanfArg";_tag_fat(_tmp44,sizeof(char),9U);});*_tmp45=_tmp286;});_tmp45;});_tmp46->f2=_tmp287;});_tmp46;});
# 374
return(struct _tuple0*)_check_null(q);}
# 340
struct _tuple0*Cyc_Absyn_uniquergn_qvar(){
# 385
static struct _tuple0*q=0;
if(q == 0)
q=({struct _tuple0*_tmp4C=_cycalloc(sizeof(*_tmp4C));({union Cyc_Absyn_Nmspace _tmp28D=Cyc_Absyn_Abs_n(({struct Cyc_List_List*_tmp49=_cycalloc(sizeof(*_tmp49));({struct _fat_ptr*_tmp28C=({struct _fat_ptr*_tmp48=_cycalloc(sizeof(*_tmp48));({struct _fat_ptr _tmp28B=({const char*_tmp47="Core";_tag_fat(_tmp47,sizeof(char),5U);});*_tmp48=_tmp28B;});_tmp48;});_tmp49->hd=_tmp28C;}),_tmp49->tl=0;_tmp49;}),0);_tmp4C->f1=_tmp28D;}),({struct _fat_ptr*_tmp28A=({struct _fat_ptr*_tmp4B=_cycalloc(sizeof(*_tmp4B));({struct _fat_ptr _tmp289=({const char*_tmp4A="unique_region";_tag_fat(_tmp4A,sizeof(char),14U);});*_tmp4B=_tmp289;});_tmp4B;});_tmp4C->f2=_tmp28A;});_tmp4C;});
# 386
return(struct _tuple0*)_check_null(q);}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_uniquergn_exp(){
# 391
static struct Cyc_Absyn_Exp*e=0;
if(e == 0){
void*t=Cyc_Absyn_rgn_handle_type(Cyc_Absyn_unique_rgn_type);
e=({struct Cyc_Absyn_Exp*_tmp50=_cycalloc(sizeof(*_tmp50));_tmp50->topt=t,_tmp50->loc=0U,_tmp50->annot=(void*)& Cyc_Absyn_EmptyAnnot_val,({
void*_tmp292=(void*)({struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct*_tmp4F=_cycalloc(sizeof(*_tmp4F));_tmp4F->tag=1U,({void*_tmp291=(void*)({struct Cyc_Absyn_Global_b_Absyn_Binding_struct*_tmp4E=_cycalloc(sizeof(*_tmp4E));_tmp4E->tag=1U,({struct Cyc_Absyn_Vardecl*_tmp290=({struct Cyc_Absyn_Vardecl*_tmp4D=_cycalloc(sizeof(*_tmp4D));_tmp4D->sc=Cyc_Absyn_Extern,({struct _tuple0*_tmp28F=Cyc_Absyn_uniquergn_qvar();_tmp4D->name=_tmp28F;}),_tmp4D->varloc=0U,({
struct Cyc_Absyn_Tqual _tmp28E=Cyc_Absyn_empty_tqual(0U);_tmp4D->tq=_tmp28E;}),_tmp4D->type=t,_tmp4D->initializer=0,_tmp4D->rgn=0,_tmp4D->attributes=0,_tmp4D->escapes=0;_tmp4D;});
# 395
_tmp4E->f1=_tmp290;});_tmp4E;});_tmp4F->f1=_tmp291;});_tmp4F;});_tmp50->r=_tmp292;});_tmp50;});}
# 392
return(struct Cyc_Absyn_Exp*)_check_null(e);}
# 340
void*Cyc_Absyn_pointer_type(struct Cyc_Absyn_PtrInfo s){
# 404
return(void*)({struct Cyc_Absyn_PointerType_Absyn_Type_struct*_tmp51=_cycalloc(sizeof(*_tmp51));_tmp51->tag=3U,_tmp51->f1=s;_tmp51;});}
# 340
void*Cyc_Absyn_fatptr_type(void*t,void*r,struct Cyc_Absyn_Tqual tq,void*zt){
# 408
return Cyc_Absyn_pointer_type(({struct Cyc_Absyn_PtrInfo _tmp242;_tmp242.elt_type=t,_tmp242.elt_tq=tq,(_tmp242.ptr_atts).rgn=r,(_tmp242.ptr_atts).nullable=Cyc_Absyn_true_type,(_tmp242.ptr_atts).bounds=Cyc_Absyn_fat_bound_type,(_tmp242.ptr_atts).zero_term=zt,(_tmp242.ptr_atts).ptrloc=0;_tmp242;}));}
# 340
void*Cyc_Absyn_starb_type(void*t,void*r,struct Cyc_Absyn_Tqual tq,void*b,void*zt){
# 413
return Cyc_Absyn_pointer_type(({struct Cyc_Absyn_PtrInfo _tmp243;_tmp243.elt_type=t,_tmp243.elt_tq=tq,(_tmp243.ptr_atts).rgn=r,(_tmp243.ptr_atts).nullable=Cyc_Absyn_true_type,(_tmp243.ptr_atts).bounds=b,(_tmp243.ptr_atts).zero_term=zt,(_tmp243.ptr_atts).ptrloc=0;_tmp243;}));}
# 340
void*Cyc_Absyn_atb_type(void*t,void*r,struct Cyc_Absyn_Tqual tq,void*b,void*zt){
# 418
return Cyc_Absyn_pointer_type(({struct Cyc_Absyn_PtrInfo _tmp244;_tmp244.elt_type=t,_tmp244.elt_tq=tq,(_tmp244.ptr_atts).rgn=r,(_tmp244.ptr_atts).nullable=Cyc_Absyn_false_type,(_tmp244.ptr_atts).bounds=b,(_tmp244.ptr_atts).zero_term=zt,(_tmp244.ptr_atts).ptrloc=0;_tmp244;}));}
# 340
void*Cyc_Absyn_star_type(void*t,void*r,struct Cyc_Absyn_Tqual tq,void*zeroterm){
# 423
return({void*_tmp296=t;void*_tmp295=r;struct Cyc_Absyn_Tqual _tmp294=tq;void*_tmp293=Cyc_Absyn_bounds_one();Cyc_Absyn_starb_type(_tmp296,_tmp295,_tmp294,_tmp293,zeroterm);});}
# 340
void*Cyc_Absyn_cstar_type(void*t,struct Cyc_Absyn_Tqual tq){
# 426
return Cyc_Absyn_star_type(t,Cyc_Absyn_heap_rgn_type,tq,Cyc_Absyn_false_type);}
# 340
void*Cyc_Absyn_at_type(void*t,void*r,struct Cyc_Absyn_Tqual tq,void*zeroterm){
# 429
return({void*_tmp29A=t;void*_tmp299=r;struct Cyc_Absyn_Tqual _tmp298=tq;void*_tmp297=Cyc_Absyn_bounds_one();Cyc_Absyn_atb_type(_tmp29A,_tmp299,_tmp298,_tmp297,zeroterm);});}
# 340
void*Cyc_Absyn_string_type(void*rgn){
# 432
return({void*_tmp29E=Cyc_Absyn_char_type;void*_tmp29D=rgn;struct Cyc_Absyn_Tqual _tmp29C=Cyc_Absyn_empty_tqual(0U);void*_tmp29B=Cyc_Absyn_fat_bound_type;Cyc_Absyn_starb_type(_tmp29E,_tmp29D,_tmp29C,_tmp29B,Cyc_Absyn_true_type);});}
# 340
void*Cyc_Absyn_const_string_type(void*rgn){
# 435
return({void*_tmp2A2=Cyc_Absyn_char_type;void*_tmp2A1=rgn;struct Cyc_Absyn_Tqual _tmp2A0=Cyc_Absyn_const_tqual(0U);void*_tmp29F=Cyc_Absyn_fat_bound_type;Cyc_Absyn_starb_type(_tmp2A2,_tmp2A1,_tmp2A0,_tmp29F,Cyc_Absyn_true_type);});}
# 340
void*Cyc_Absyn_array_type(void*elt_type,struct Cyc_Absyn_Tqual tq,struct Cyc_Absyn_Exp*num_elts,void*zero_term,unsigned ztloc){
# 440
return(void*)({struct Cyc_Absyn_ArrayType_Absyn_Type_struct*_tmp52=_cycalloc(sizeof(*_tmp52));_tmp52->tag=4U,(_tmp52->f1).elt_type=elt_type,(_tmp52->f1).tq=tq,(_tmp52->f1).num_elts=num_elts,(_tmp52->f1).zero_term=zero_term,(_tmp52->f1).zt_loc=ztloc;_tmp52;});}
# 340
void*Cyc_Absyn_typeof_type(struct Cyc_Absyn_Exp*e){
# 444
return(void*)({struct Cyc_Absyn_TypeofType_Absyn_Type_struct*_tmp53=_cycalloc(sizeof(*_tmp53));_tmp53->tag=11U,_tmp53->f1=e;_tmp53;});}
# 340
void*Cyc_Absyn_typedef_type(struct _tuple0*n,struct Cyc_List_List*args,struct Cyc_Absyn_Typedefdecl*d,void*defn){
# 450
return(void*)({struct Cyc_Absyn_TypedefType_Absyn_Type_struct*_tmp54=_cycalloc(sizeof(*_tmp54));_tmp54->tag=8U,_tmp54->f1=n,_tmp54->f2=args,_tmp54->f3=d,_tmp54->f4=defn;_tmp54;});}
# 340
static void*Cyc_Absyn_aggregate_type(enum Cyc_Absyn_AggrKind k,struct _fat_ptr*name){
# 457
return(void*)({struct Cyc_Absyn_AppType_Absyn_Type_struct*_tmp57=_cycalloc(sizeof(*_tmp57));_tmp57->tag=0U,({void*_tmp2A5=(void*)({struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*_tmp56=_cycalloc(sizeof(*_tmp56));_tmp56->tag=20U,({union Cyc_Absyn_AggrInfo _tmp2A4=({enum Cyc_Absyn_AggrKind _tmp2A3=k;Cyc_Absyn_UnknownAggr(_tmp2A3,({struct _tuple0*_tmp55=_cycalloc(sizeof(*_tmp55));_tmp55->f1=Cyc_Absyn_rel_ns_null,_tmp55->f2=name;_tmp55;}),0);});_tmp56->f1=_tmp2A4;});_tmp56;});_tmp57->f1=_tmp2A5;}),_tmp57->f2=0;_tmp57;});}
# 340
void*Cyc_Absyn_strct(struct _fat_ptr*name){
# 459
return Cyc_Absyn_aggregate_type(Cyc_Absyn_StructA,name);}
# 340
void*Cyc_Absyn_union_typ(struct _fat_ptr*name){
# 460
return Cyc_Absyn_aggregate_type(Cyc_Absyn_UnionA,name);}
# 340
void*Cyc_Absyn_strctq(struct _tuple0*name){
# 463
return Cyc_Absyn_aggr_type(Cyc_Absyn_UnknownAggr(Cyc_Absyn_StructA,name,0),0);}
# 340
void*Cyc_Absyn_unionq_type(struct _tuple0*name){
# 466
return Cyc_Absyn_aggr_type(Cyc_Absyn_UnknownAggr(Cyc_Absyn_UnionA,name,0),0);}
# 340
union Cyc_Absyn_Cnst Cyc_Absyn_Char_c(enum Cyc_Absyn_Sign sn,char c){
# 469
return({union Cyc_Absyn_Cnst _tmp245;(_tmp245.Char_c).tag=2U,((_tmp245.Char_c).val).f1=sn,((_tmp245.Char_c).val).f2=c;_tmp245;});}
# 340
union Cyc_Absyn_Cnst Cyc_Absyn_Wchar_c(struct _fat_ptr s){
# 470
return({union Cyc_Absyn_Cnst _tmp246;(_tmp246.Wchar_c).tag=3U,(_tmp246.Wchar_c).val=s;_tmp246;});}
# 340
union Cyc_Absyn_Cnst Cyc_Absyn_Short_c(enum Cyc_Absyn_Sign sn,short s){
# 471
return({union Cyc_Absyn_Cnst _tmp247;(_tmp247.Short_c).tag=4U,((_tmp247.Short_c).val).f1=sn,((_tmp247.Short_c).val).f2=s;_tmp247;});}
# 340
union Cyc_Absyn_Cnst Cyc_Absyn_Int_c(enum Cyc_Absyn_Sign sn,int i){
# 472
return({union Cyc_Absyn_Cnst _tmp248;(_tmp248.Int_c).tag=5U,((_tmp248.Int_c).val).f1=sn,((_tmp248.Int_c).val).f2=i;_tmp248;});}
# 340
union Cyc_Absyn_Cnst Cyc_Absyn_LongLong_c(enum Cyc_Absyn_Sign sn,long long l){
# 473
return({union Cyc_Absyn_Cnst _tmp249;(_tmp249.LongLong_c).tag=6U,((_tmp249.LongLong_c).val).f1=sn,((_tmp249.LongLong_c).val).f2=l;_tmp249;});}
# 340
union Cyc_Absyn_Cnst Cyc_Absyn_Float_c(struct _fat_ptr s,int i){
# 474
return({union Cyc_Absyn_Cnst _tmp24A;(_tmp24A.Float_c).tag=7U,((_tmp24A.Float_c).val).f1=s,((_tmp24A.Float_c).val).f2=i;_tmp24A;});}
# 340
union Cyc_Absyn_Cnst Cyc_Absyn_String_c(struct _fat_ptr s){
# 475
return({union Cyc_Absyn_Cnst _tmp24B;(_tmp24B.String_c).tag=8U,(_tmp24B.String_c).val=s;_tmp24B;});}
# 340
union Cyc_Absyn_Cnst Cyc_Absyn_Wstring_c(struct _fat_ptr s){
# 476
return({union Cyc_Absyn_Cnst _tmp24C;(_tmp24C.Wstring_c).tag=9U,(_tmp24C.Wstring_c).val=s;_tmp24C;});}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_new_exp(void*r,unsigned loc){
# 480
return({struct Cyc_Absyn_Exp*_tmp58=_cycalloc(sizeof(*_tmp58));_tmp58->topt=0,_tmp58->r=r,_tmp58->loc=loc,_tmp58->annot=(void*)& Cyc_Absyn_EmptyAnnot_val;_tmp58;});}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_New_exp(struct Cyc_Absyn_Exp*rgn_handle,struct Cyc_Absyn_Exp*e,unsigned loc){
# 483
return({void*_tmp2A6=(void*)({struct Cyc_Absyn_New_e_Absyn_Raw_exp_struct*_tmp59=_cycalloc(sizeof(*_tmp59));_tmp59->tag=16U,_tmp59->f1=rgn_handle,_tmp59->f2=e;_tmp59;});Cyc_Absyn_new_exp(_tmp2A6,loc);});}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_copy_exp(struct Cyc_Absyn_Exp*e){
# 486
return({struct Cyc_Absyn_Exp*_tmp5A=_cycalloc(sizeof(*_tmp5A));*_tmp5A=*e;_tmp5A;});}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_const_exp(union Cyc_Absyn_Cnst c,unsigned loc){
# 489
return({void*_tmp2A7=(void*)({struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*_tmp5B=_cycalloc(sizeof(*_tmp5B));_tmp5B->tag=0U,_tmp5B->f1=c;_tmp5B;});Cyc_Absyn_new_exp(_tmp2A7,loc);});}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_null_exp(unsigned loc){
# 492
static struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct null_const={0U,{.Null_c={1,0}}};
return Cyc_Absyn_new_exp((void*)& null_const,loc);}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_int_exp(enum Cyc_Absyn_Sign s,int i,unsigned seg){
# 495
return({union Cyc_Absyn_Cnst _tmp2A8=Cyc_Absyn_Int_c(s,i);Cyc_Absyn_const_exp(_tmp2A8,seg);});}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_signed_int_exp(int i,unsigned loc){
# 497
static struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct szero={0U,{.Int_c={5,{Cyc_Absyn_Signed,0}}}};
static struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct sone={0U,{.Int_c={5,{Cyc_Absyn_Signed,1}}}};
if(i == 0)return Cyc_Absyn_new_exp((void*)& szero,loc);else{
if(i == 1)return Cyc_Absyn_new_exp((void*)& sone,loc);}
# 499
return
# 501
 Cyc_Absyn_int_exp(Cyc_Absyn_Signed,i,loc);}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_uint_exp(unsigned i,unsigned loc){
# 504
static struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct uzero={0U,{.Int_c={5,{Cyc_Absyn_Unsigned,0}}}};
static struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct uone={0U,{.Int_c={5,{Cyc_Absyn_Unsigned,1}}}};
if(i == (unsigned)0)return Cyc_Absyn_new_exp((void*)& uzero,loc);else{
if(i == (unsigned)1)return Cyc_Absyn_new_exp((void*)& uone,loc);else{
return Cyc_Absyn_int_exp(Cyc_Absyn_Unsigned,(int)i,loc);}}}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_bool_exp(int b,unsigned loc){
# 510
return Cyc_Absyn_signed_int_exp(b?1: 0,loc);}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_true_exp(unsigned loc){
# 511
return Cyc_Absyn_bool_exp(1,loc);}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_false_exp(unsigned loc){
# 512
return Cyc_Absyn_bool_exp(0,loc);}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_char_exp(char c,unsigned loc){
# 513
return({union Cyc_Absyn_Cnst _tmp2A9=Cyc_Absyn_Char_c(Cyc_Absyn_None,c);Cyc_Absyn_const_exp(_tmp2A9,loc);});}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_float_exp(struct _fat_ptr f,int i,unsigned loc){
# 515
return({union Cyc_Absyn_Cnst _tmp2AA=Cyc_Absyn_Float_c(f,i);Cyc_Absyn_const_exp(_tmp2AA,loc);});}
# 340
static struct Cyc_Absyn_Exp*Cyc_Absyn_str2exp(union Cyc_Absyn_Cnst(*f)(struct _fat_ptr),struct _fat_ptr s,unsigned loc){
# 518
return({union Cyc_Absyn_Cnst _tmp2AB=f(s);Cyc_Absyn_const_exp(_tmp2AB,loc);});}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_wchar_exp(struct _fat_ptr s,unsigned loc){
# 520
return Cyc_Absyn_str2exp(Cyc_Absyn_Wchar_c,s,loc);}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_string_exp(struct _fat_ptr s,unsigned loc){
# 521
return Cyc_Absyn_str2exp(Cyc_Absyn_String_c,s,loc);}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_wstring_exp(struct _fat_ptr s,unsigned loc){
# 522
return Cyc_Absyn_str2exp(Cyc_Absyn_Wstring_c,s,loc);}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_var_exp(struct _tuple0*q,unsigned loc){
# 525
return({void*_tmp2AD=(void*)({struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct*_tmp5D=_cycalloc(sizeof(*_tmp5D));_tmp5D->tag=1U,({void*_tmp2AC=(void*)({struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct*_tmp5C=_cycalloc(sizeof(*_tmp5C));_tmp5C->tag=0U,_tmp5C->f1=q;_tmp5C;});_tmp5D->f1=_tmp2AC;});_tmp5D;});Cyc_Absyn_new_exp(_tmp2AD,loc);});}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_varb_exp(void*b,unsigned loc){
# 528
return({void*_tmp2AE=(void*)({struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct*_tmp5E=_cycalloc(sizeof(*_tmp5E));_tmp5E->tag=1U,_tmp5E->f1=b;_tmp5E;});Cyc_Absyn_new_exp(_tmp2AE,loc);});}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_unknownid_exp(struct _tuple0*q,unsigned loc){
# 532
return Cyc_Absyn_var_exp(q,loc);}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_pragma_exp(struct _fat_ptr s,unsigned loc){
# 535
return({void*_tmp2AF=(void*)({struct Cyc_Absyn_Pragma_e_Absyn_Raw_exp_struct*_tmp5F=_cycalloc(sizeof(*_tmp5F));_tmp5F->tag=2U,_tmp5F->f1=s;_tmp5F;});Cyc_Absyn_new_exp(_tmp2AF,loc);});}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_primop_exp(enum Cyc_Absyn_Primop p,struct Cyc_List_List*es,unsigned loc){
# 538
return({void*_tmp2B0=(void*)({struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct*_tmp60=_cycalloc(sizeof(*_tmp60));_tmp60->tag=3U,_tmp60->f1=p,_tmp60->f2=es;_tmp60;});Cyc_Absyn_new_exp(_tmp2B0,loc);});}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_prim1_exp(enum Cyc_Absyn_Primop p,struct Cyc_Absyn_Exp*e,unsigned loc){
# 541
return({enum Cyc_Absyn_Primop _tmp2B2=p;struct Cyc_List_List*_tmp2B1=({struct Cyc_List_List*_tmp61=_cycalloc(sizeof(*_tmp61));_tmp61->hd=e,_tmp61->tl=0;_tmp61;});Cyc_Absyn_primop_exp(_tmp2B2,_tmp2B1,loc);});}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_prim2_exp(enum Cyc_Absyn_Primop p,struct Cyc_Absyn_Exp*e1,struct Cyc_Absyn_Exp*e2,unsigned loc){
# 544
return({enum Cyc_Absyn_Primop _tmp2B5=p;struct Cyc_List_List*_tmp2B4=({struct Cyc_List_List*_tmp63=_cycalloc(sizeof(*_tmp63));_tmp63->hd=e1,({struct Cyc_List_List*_tmp2B3=({struct Cyc_List_List*_tmp62=_cycalloc(sizeof(*_tmp62));_tmp62->hd=e2,_tmp62->tl=0;_tmp62;});_tmp63->tl=_tmp2B3;});_tmp63;});Cyc_Absyn_primop_exp(_tmp2B5,_tmp2B4,loc);});}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_swap_exp(struct Cyc_Absyn_Exp*e1,struct Cyc_Absyn_Exp*e2,unsigned loc){
# 547
return({void*_tmp2B6=(void*)({struct Cyc_Absyn_Swap_e_Absyn_Raw_exp_struct*_tmp64=_cycalloc(sizeof(*_tmp64));_tmp64->tag=36U,_tmp64->f1=e1,_tmp64->f2=e2;_tmp64;});Cyc_Absyn_new_exp(_tmp2B6,loc);});}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_add_exp(struct Cyc_Absyn_Exp*e1,struct Cyc_Absyn_Exp*e2,unsigned loc){
# 549
return Cyc_Absyn_prim2_exp(Cyc_Absyn_Plus,e1,e2,loc);}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_times_exp(struct Cyc_Absyn_Exp*e1,struct Cyc_Absyn_Exp*e2,unsigned loc){
# 550
return Cyc_Absyn_prim2_exp(Cyc_Absyn_Times,e1,e2,loc);}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_divide_exp(struct Cyc_Absyn_Exp*e1,struct Cyc_Absyn_Exp*e2,unsigned loc){
# 551
return Cyc_Absyn_prim2_exp(Cyc_Absyn_Div,e1,e2,loc);}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_eq_exp(struct Cyc_Absyn_Exp*e1,struct Cyc_Absyn_Exp*e2,unsigned loc){
# 552
return Cyc_Absyn_prim2_exp(Cyc_Absyn_Eq,e1,e2,loc);}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_neq_exp(struct Cyc_Absyn_Exp*e1,struct Cyc_Absyn_Exp*e2,unsigned loc){
# 553
return Cyc_Absyn_prim2_exp(Cyc_Absyn_Neq,e1,e2,loc);}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_gt_exp(struct Cyc_Absyn_Exp*e1,struct Cyc_Absyn_Exp*e2,unsigned loc){
# 554
return Cyc_Absyn_prim2_exp(Cyc_Absyn_Gt,e1,e2,loc);}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_lt_exp(struct Cyc_Absyn_Exp*e1,struct Cyc_Absyn_Exp*e2,unsigned loc){
# 555
return Cyc_Absyn_prim2_exp(Cyc_Absyn_Lt,e1,e2,loc);}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_gte_exp(struct Cyc_Absyn_Exp*e1,struct Cyc_Absyn_Exp*e2,unsigned loc){
# 556
return Cyc_Absyn_prim2_exp(Cyc_Absyn_Gte,e1,e2,loc);}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_lte_exp(struct Cyc_Absyn_Exp*e1,struct Cyc_Absyn_Exp*e2,unsigned loc){
# 557
return Cyc_Absyn_prim2_exp(Cyc_Absyn_Lte,e1,e2,loc);}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_assignop_exp(struct Cyc_Absyn_Exp*e1,struct Cyc_Core_Opt*popt,struct Cyc_Absyn_Exp*e2,unsigned loc){
# 560
return({void*_tmp2B7=(void*)({struct Cyc_Absyn_AssignOp_e_Absyn_Raw_exp_struct*_tmp65=_cycalloc(sizeof(*_tmp65));_tmp65->tag=4U,_tmp65->f1=e1,_tmp65->f2=popt,_tmp65->f3=e2;_tmp65;});Cyc_Absyn_new_exp(_tmp2B7,loc);});}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_assign_exp(struct Cyc_Absyn_Exp*e1,struct Cyc_Absyn_Exp*e2,unsigned loc){
# 563
return Cyc_Absyn_assignop_exp(e1,0,e2,loc);}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_increment_exp(struct Cyc_Absyn_Exp*e,enum Cyc_Absyn_Incrementor i,unsigned loc){
# 566
return({void*_tmp2B8=(void*)({struct Cyc_Absyn_Increment_e_Absyn_Raw_exp_struct*_tmp66=_cycalloc(sizeof(*_tmp66));_tmp66->tag=5U,_tmp66->f1=e,_tmp66->f2=i;_tmp66;});Cyc_Absyn_new_exp(_tmp2B8,loc);});}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_conditional_exp(struct Cyc_Absyn_Exp*e1,struct Cyc_Absyn_Exp*e2,struct Cyc_Absyn_Exp*e3,unsigned loc){
# 569
return({void*_tmp2B9=(void*)({struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*_tmp67=_cycalloc(sizeof(*_tmp67));_tmp67->tag=6U,_tmp67->f1=e1,_tmp67->f2=e2,_tmp67->f3=e3;_tmp67;});Cyc_Absyn_new_exp(_tmp2B9,loc);});}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_and_exp(struct Cyc_Absyn_Exp*e1,struct Cyc_Absyn_Exp*e2,unsigned loc){
# 572
return({void*_tmp2BA=(void*)({struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct*_tmp68=_cycalloc(sizeof(*_tmp68));_tmp68->tag=7U,_tmp68->f1=e1,_tmp68->f2=e2;_tmp68;});Cyc_Absyn_new_exp(_tmp2BA,loc);});}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_or_exp(struct Cyc_Absyn_Exp*e1,struct Cyc_Absyn_Exp*e2,unsigned loc){
# 575
return({void*_tmp2BB=(void*)({struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct*_tmp69=_cycalloc(sizeof(*_tmp69));_tmp69->tag=8U,_tmp69->f1=e1,_tmp69->f2=e2;_tmp69;});Cyc_Absyn_new_exp(_tmp2BB,loc);});}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_spawn_exp(struct Cyc_Absyn_Exp*e1,struct Cyc_Absyn_Exp*e2,unsigned loc){
# 579
return({void*_tmp2BC=(void*)({struct Cyc_Absyn_Spawn_e_Absyn_Raw_exp_struct*_tmp6A=_cycalloc(sizeof(*_tmp6A));_tmp6A->tag=34U,_tmp6A->f1=e1,_tmp6A->f2=e2;_tmp6A;});Cyc_Absyn_new_exp(_tmp2BC,loc);});}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_seq_exp(struct Cyc_Absyn_Exp*e1,struct Cyc_Absyn_Exp*e2,unsigned loc){
# 583
return({void*_tmp2BD=(void*)({struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct*_tmp6B=_cycalloc(sizeof(*_tmp6B));_tmp6B->tag=9U,_tmp6B->f1=e1,_tmp6B->f2=e2;_tmp6B;});Cyc_Absyn_new_exp(_tmp2BD,loc);});}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_unknowncall_exp(struct Cyc_Absyn_Exp*e,struct Cyc_List_List*es,unsigned loc){
# 586
return({void*_tmp2BE=(void*)({struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct*_tmp6C=_cycalloc(sizeof(*_tmp6C));_tmp6C->tag=10U,_tmp6C->f1=e,_tmp6C->f2=es,_tmp6C->f3=0,_tmp6C->f4=0;_tmp6C;});Cyc_Absyn_new_exp(_tmp2BE,loc);});}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_fncall_exp(struct Cyc_Absyn_Exp*e,struct Cyc_List_List*es,unsigned loc){
# 589
return({void*_tmp2BF=(void*)({struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct*_tmp6D=_cycalloc(sizeof(*_tmp6D));_tmp6D->tag=10U,_tmp6D->f1=e,_tmp6D->f2=es,_tmp6D->f3=0,_tmp6D->f4=1;_tmp6D;});Cyc_Absyn_new_exp(_tmp2BF,loc);});}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_noinstantiate_exp(struct Cyc_Absyn_Exp*e,unsigned loc){
# 592
return({void*_tmp2C0=(void*)({struct Cyc_Absyn_NoInstantiate_e_Absyn_Raw_exp_struct*_tmp6E=_cycalloc(sizeof(*_tmp6E));_tmp6E->tag=12U,_tmp6E->f1=e;_tmp6E;});Cyc_Absyn_new_exp(_tmp2C0,loc);});}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_instantiate_exp(struct Cyc_Absyn_Exp*e,struct Cyc_List_List*ts,unsigned loc){
# 595
return({void*_tmp2C1=(void*)({struct Cyc_Absyn_Instantiate_e_Absyn_Raw_exp_struct*_tmp6F=_cycalloc(sizeof(*_tmp6F));_tmp6F->tag=13U,_tmp6F->f1=e,_tmp6F->f2=ts;_tmp6F;});Cyc_Absyn_new_exp(_tmp2C1,loc);});}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_cast_exp(void*t,struct Cyc_Absyn_Exp*e,int user_cast,enum Cyc_Absyn_Coercion c,unsigned loc){
# 598
return({void*_tmp2C2=(void*)({struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*_tmp70=_cycalloc(sizeof(*_tmp70));_tmp70->tag=14U,_tmp70->f1=t,_tmp70->f2=e,_tmp70->f3=user_cast,_tmp70->f4=c;_tmp70;});Cyc_Absyn_new_exp(_tmp2C2,loc);});}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_throw_exp(struct Cyc_Absyn_Exp*e,unsigned loc){
# 601
return({void*_tmp2C3=(void*)({struct Cyc_Absyn_Throw_e_Absyn_Raw_exp_struct*_tmp71=_cycalloc(sizeof(*_tmp71));_tmp71->tag=11U,_tmp71->f1=e,_tmp71->f2=0;_tmp71;});Cyc_Absyn_new_exp(_tmp2C3,loc);});}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_rethrow_exp(struct Cyc_Absyn_Exp*e,unsigned loc){
# 604
return({void*_tmp2C4=(void*)({struct Cyc_Absyn_Throw_e_Absyn_Raw_exp_struct*_tmp72=_cycalloc(sizeof(*_tmp72));_tmp72->tag=11U,_tmp72->f1=e,_tmp72->f2=1;_tmp72;});Cyc_Absyn_new_exp(_tmp2C4,loc);});}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_address_exp(struct Cyc_Absyn_Exp*e,unsigned loc){
# 606
return({void*_tmp2C5=(void*)({struct Cyc_Absyn_Address_e_Absyn_Raw_exp_struct*_tmp73=_cycalloc(sizeof(*_tmp73));_tmp73->tag=15U,_tmp73->f1=e;_tmp73;});Cyc_Absyn_new_exp(_tmp2C5,loc);});}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_sizeoftype_exp(void*t,unsigned loc){
# 608
return({void*_tmp2C6=(void*)({struct Cyc_Absyn_Sizeoftype_e_Absyn_Raw_exp_struct*_tmp74=_cycalloc(sizeof(*_tmp74));_tmp74->tag=17U,_tmp74->f1=t;_tmp74;});Cyc_Absyn_new_exp(_tmp2C6,loc);});}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_sizeofexp_exp(struct Cyc_Absyn_Exp*e,unsigned loc){
# 611
return({void*_tmp2C7=(void*)({struct Cyc_Absyn_Sizeofexp_e_Absyn_Raw_exp_struct*_tmp75=_cycalloc(sizeof(*_tmp75));_tmp75->tag=18U,_tmp75->f1=e;_tmp75;});Cyc_Absyn_new_exp(_tmp2C7,loc);});}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_offsetof_exp(void*t,struct Cyc_List_List*ofs,unsigned loc){
# 614
return({void*_tmp2C8=(void*)({struct Cyc_Absyn_Offsetof_e_Absyn_Raw_exp_struct*_tmp76=_cycalloc(sizeof(*_tmp76));_tmp76->tag=19U,_tmp76->f1=t,_tmp76->f2=ofs;_tmp76;});Cyc_Absyn_new_exp(_tmp2C8,loc);});}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_deref_exp(struct Cyc_Absyn_Exp*e,unsigned loc){
# 616
return({void*_tmp2C9=(void*)({struct Cyc_Absyn_Deref_e_Absyn_Raw_exp_struct*_tmp77=_cycalloc(sizeof(*_tmp77));_tmp77->tag=20U,_tmp77->f1=e;_tmp77;});Cyc_Absyn_new_exp(_tmp2C9,loc);});}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_aggrmember_exp(struct Cyc_Absyn_Exp*e,struct _fat_ptr*n,unsigned loc){
# 618
return({void*_tmp2CA=(void*)({struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct*_tmp78=_cycalloc(sizeof(*_tmp78));_tmp78->tag=21U,_tmp78->f1=e,_tmp78->f2=n,_tmp78->f3=0,_tmp78->f4=0;_tmp78;});Cyc_Absyn_new_exp(_tmp2CA,loc);});}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_aggrarrow_exp(struct Cyc_Absyn_Exp*e,struct _fat_ptr*n,unsigned loc){
# 621
return({void*_tmp2CB=(void*)({struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct*_tmp79=_cycalloc(sizeof(*_tmp79));_tmp79->tag=22U,_tmp79->f1=e,_tmp79->f2=n,_tmp79->f3=0,_tmp79->f4=0;_tmp79;});Cyc_Absyn_new_exp(_tmp2CB,loc);});}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_subscript_exp(struct Cyc_Absyn_Exp*e1,struct Cyc_Absyn_Exp*e2,unsigned loc){
# 624
return({void*_tmp2CC=(void*)({struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct*_tmp7A=_cycalloc(sizeof(*_tmp7A));_tmp7A->tag=23U,_tmp7A->f1=e1,_tmp7A->f2=e2;_tmp7A;});Cyc_Absyn_new_exp(_tmp2CC,loc);});}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_tuple_exp(struct Cyc_List_List*es,unsigned loc){
# 627
return({void*_tmp2CD=(void*)({struct Cyc_Absyn_Tuple_e_Absyn_Raw_exp_struct*_tmp7B=_cycalloc(sizeof(*_tmp7B));_tmp7B->tag=24U,_tmp7B->f1=es;_tmp7B;});Cyc_Absyn_new_exp(_tmp2CD,loc);});}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_stmt_exp(struct Cyc_Absyn_Stmt*s,unsigned loc){
# 630
return({void*_tmp2CE=(void*)({struct Cyc_Absyn_StmtExp_e_Absyn_Raw_exp_struct*_tmp7C=_cycalloc(sizeof(*_tmp7C));_tmp7C->tag=38U,_tmp7C->f1=s;_tmp7C;});Cyc_Absyn_new_exp(_tmp2CE,loc);});}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_valueof_exp(void*t,unsigned loc){
# 633
return({void*_tmp2CF=(void*)({struct Cyc_Absyn_Valueof_e_Absyn_Raw_exp_struct*_tmp7D=_cycalloc(sizeof(*_tmp7D));_tmp7D->tag=40U,_tmp7D->f1=t;_tmp7D;});Cyc_Absyn_new_exp(_tmp2CF,loc);});}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_asm_exp(int volatile_kw,struct _fat_ptr tmpl,struct Cyc_List_List*outs,struct Cyc_List_List*ins,struct Cyc_List_List*clobs,unsigned loc){
# 639
return({void*_tmp2D0=(void*)({struct Cyc_Absyn_Asm_e_Absyn_Raw_exp_struct*_tmp7E=_cycalloc(sizeof(*_tmp7E));_tmp7E->tag=41U,_tmp7E->f1=volatile_kw,_tmp7E->f2=tmpl,_tmp7E->f3=outs,_tmp7E->f4=ins,_tmp7E->f5=clobs;_tmp7E;});Cyc_Absyn_new_exp(_tmp2D0,loc);});}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_extension_exp(struct Cyc_Absyn_Exp*e,unsigned loc){
# 642
return({void*_tmp2D1=(void*)({struct Cyc_Absyn_Extension_e_Absyn_Raw_exp_struct*_tmp7F=_cycalloc(sizeof(*_tmp7F));_tmp7F->tag=42U,_tmp7F->f1=e;_tmp7F;});Cyc_Absyn_new_exp(_tmp2D1,loc);});}struct _tuple12{struct Cyc_List_List*f1;struct Cyc_Absyn_Exp*f2;};
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_array_exp(struct Cyc_List_List*es,unsigned loc){
# 646
struct Cyc_List_List*dles=0;
for(0;es != 0;es=es->tl){
dles=({struct Cyc_List_List*_tmp81=_cycalloc(sizeof(*_tmp81));({struct _tuple12*_tmp2D2=({struct _tuple12*_tmp80=_cycalloc(sizeof(*_tmp80));_tmp80->f1=0,_tmp80->f2=(struct Cyc_Absyn_Exp*)es->hd;_tmp80;});_tmp81->hd=_tmp2D2;}),_tmp81->tl=dles;_tmp81;});}
dles=Cyc_List_imp_rev(dles);
return({void*_tmp2D3=(void*)({struct Cyc_Absyn_Array_e_Absyn_Raw_exp_struct*_tmp82=_cycalloc(sizeof(*_tmp82));_tmp82->tag=26U,_tmp82->f1=dles;_tmp82;});Cyc_Absyn_new_exp(_tmp2D3,loc);});}
# 340
struct Cyc_Absyn_Exp*Cyc_Absyn_unresolvedmem_exp(struct Cyc_Core_Opt*n,struct Cyc_List_List*dles,unsigned loc){
# 655
return({void*_tmp2D4=(void*)({struct Cyc_Absyn_UnresolvedMem_e_Absyn_Raw_exp_struct*_tmp83=_cycalloc(sizeof(*_tmp83));_tmp83->tag=37U,_tmp83->f1=n,_tmp83->f2=dles;_tmp83;});Cyc_Absyn_new_exp(_tmp2D4,loc);});}
# 340
struct Cyc_Absyn_Stmt*Cyc_Absyn_new_stmt(void*s,unsigned loc){
# 659
return({struct Cyc_Absyn_Stmt*_tmp84=_cycalloc(sizeof(*_tmp84));_tmp84->r=s,_tmp84->loc=loc,_tmp84->annot=(void*)& Cyc_Absyn_EmptyAnnot_val;_tmp84;});}
# 340
struct Cyc_Absyn_Skip_s_Absyn_Raw_stmt_struct Cyc_Absyn_Skip_s_val={0U};
# 662
struct Cyc_Absyn_Break_s_Absyn_Raw_stmt_struct Cyc_Absyn_Break_s_val={6U};
struct Cyc_Absyn_Continue_s_Absyn_Raw_stmt_struct Cyc_Absyn_Continue_s_val={7U};
struct Cyc_Absyn_Stmt*Cyc_Absyn_skip_stmt(unsigned loc){return Cyc_Absyn_new_stmt((void*)& Cyc_Absyn_Skip_s_val,loc);}struct Cyc_Absyn_Stmt*Cyc_Absyn_break_stmt(unsigned loc){
return Cyc_Absyn_new_stmt((void*)& Cyc_Absyn_Break_s_val,loc);}
# 664
struct Cyc_Absyn_Stmt*Cyc_Absyn_continue_stmt(unsigned loc){
# 666
return Cyc_Absyn_new_stmt((void*)& Cyc_Absyn_Continue_s_val,loc);}
# 664
struct Cyc_Absyn_Stmt*Cyc_Absyn_exp_stmt(struct Cyc_Absyn_Exp*e,unsigned loc){
# 667
return({void*_tmp2D5=(void*)({struct Cyc_Absyn_Exp_s_Absyn_Raw_stmt_struct*_tmp85=_cycalloc(sizeof(*_tmp85));_tmp85->tag=1U,_tmp85->f1=e;_tmp85;});Cyc_Absyn_new_stmt(_tmp2D5,loc);});}
# 664
struct Cyc_Absyn_Stmt*Cyc_Absyn_return_stmt(struct Cyc_Absyn_Exp*e,unsigned loc){
# 668
return({void*_tmp2D6=(void*)({struct Cyc_Absyn_Return_s_Absyn_Raw_stmt_struct*_tmp86=_cycalloc(sizeof(*_tmp86));_tmp86->tag=3U,_tmp86->f1=e;_tmp86;});Cyc_Absyn_new_stmt(_tmp2D6,loc);});}
# 664
struct Cyc_Absyn_Stmt*Cyc_Absyn_seq_stmts(struct Cyc_List_List*ss,unsigned loc){
# 670
if(ss == 0)return Cyc_Absyn_skip_stmt(loc);else{
if(ss->tl == 0)return(struct Cyc_Absyn_Stmt*)ss->hd;else{
return({struct Cyc_Absyn_Stmt*_tmp2D8=(struct Cyc_Absyn_Stmt*)ss->hd;struct Cyc_Absyn_Stmt*_tmp2D7=Cyc_Absyn_seq_stmts(ss->tl,loc);Cyc_Absyn_seq_stmt(_tmp2D8,_tmp2D7,loc);});}}}struct _tuple13{void*f1;void*f2;};
# 664
struct Cyc_Absyn_Stmt*Cyc_Absyn_seq_stmt(struct Cyc_Absyn_Stmt*s1,struct Cyc_Absyn_Stmt*s2,unsigned loc){
# 675
struct _tuple13 _stmttmp3=({struct _tuple13 _tmp24D;_tmp24D.f1=s1->r,_tmp24D.f2=s2->r;_tmp24D;});struct _tuple13 _tmp87=_stmttmp3;if(((struct Cyc_Absyn_Skip_s_Absyn_Raw_stmt_struct*)_tmp87.f1)->tag == 0U){_LL1: _LL2:
 return s2;}else{if(((struct Cyc_Absyn_Skip_s_Absyn_Raw_stmt_struct*)_tmp87.f2)->tag == 0U){_LL3: _LL4:
 return s1;}else{_LL5: _LL6:
 return({void*_tmp2D9=(void*)({struct Cyc_Absyn_Seq_s_Absyn_Raw_stmt_struct*_tmp88=_cycalloc(sizeof(*_tmp88));_tmp88->tag=2U,_tmp88->f1=s1,_tmp88->f2=s2;_tmp88;});Cyc_Absyn_new_stmt(_tmp2D9,loc);});}}_LL0:;}
# 664
struct Cyc_Absyn_Stmt*Cyc_Absyn_ifthenelse_stmt(struct Cyc_Absyn_Exp*e,struct Cyc_Absyn_Stmt*s1,struct Cyc_Absyn_Stmt*s2,unsigned loc){
# 682
return({void*_tmp2DA=(void*)({struct Cyc_Absyn_IfThenElse_s_Absyn_Raw_stmt_struct*_tmp89=_cycalloc(sizeof(*_tmp89));_tmp89->tag=4U,_tmp89->f1=e,_tmp89->f2=s1,_tmp89->f3=s2;_tmp89;});Cyc_Absyn_new_stmt(_tmp2DA,loc);});}
# 664
struct Cyc_Absyn_Stmt*Cyc_Absyn_while_stmt(struct Cyc_Absyn_Exp*e,struct Cyc_Absyn_Stmt*s,unsigned loc){
# 685
return({void*_tmp2DC=(void*)({struct Cyc_Absyn_While_s_Absyn_Raw_stmt_struct*_tmp8A=_cycalloc(sizeof(*_tmp8A));_tmp8A->tag=5U,(_tmp8A->f1).f1=e,({struct Cyc_Absyn_Stmt*_tmp2DB=Cyc_Absyn_skip_stmt(e->loc);(_tmp8A->f1).f2=_tmp2DB;}),_tmp8A->f2=s;_tmp8A;});Cyc_Absyn_new_stmt(_tmp2DC,loc);});}
# 664
struct Cyc_Absyn_Stmt*Cyc_Absyn_for_stmt(struct Cyc_Absyn_Exp*e1,struct Cyc_Absyn_Exp*e2,struct Cyc_Absyn_Exp*e3,struct Cyc_Absyn_Stmt*s,unsigned loc){
# 688
return({void*_tmp2DF=(void*)({struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct*_tmp8B=_cycalloc(sizeof(*_tmp8B));_tmp8B->tag=9U,_tmp8B->f1=e1,(_tmp8B->f2).f1=e2,({struct Cyc_Absyn_Stmt*_tmp2DE=Cyc_Absyn_skip_stmt(e3->loc);(_tmp8B->f2).f2=_tmp2DE;}),
(_tmp8B->f3).f1=e3,({struct Cyc_Absyn_Stmt*_tmp2DD=Cyc_Absyn_skip_stmt(e3->loc);(_tmp8B->f3).f2=_tmp2DD;}),_tmp8B->f4=s;_tmp8B;});
# 688
Cyc_Absyn_new_stmt(_tmp2DF,loc);});}
# 664
struct Cyc_Absyn_Stmt*Cyc_Absyn_do_stmt(struct Cyc_Absyn_Stmt*s,struct Cyc_Absyn_Exp*e,unsigned loc){
# 693
return({void*_tmp2E1=(void*)({struct Cyc_Absyn_Do_s_Absyn_Raw_stmt_struct*_tmp8C=_cycalloc(sizeof(*_tmp8C));_tmp8C->tag=14U,_tmp8C->f1=s,(_tmp8C->f2).f1=e,({struct Cyc_Absyn_Stmt*_tmp2E0=Cyc_Absyn_skip_stmt(e->loc);(_tmp8C->f2).f2=_tmp2E0;});_tmp8C;});Cyc_Absyn_new_stmt(_tmp2E1,loc);});}
# 664
struct Cyc_Absyn_Stmt*Cyc_Absyn_switch_stmt(struct Cyc_Absyn_Exp*e,struct Cyc_List_List*scs,unsigned loc){
# 696
return({void*_tmp2E2=(void*)({struct Cyc_Absyn_Switch_s_Absyn_Raw_stmt_struct*_tmp8D=_cycalloc(sizeof(*_tmp8D));_tmp8D->tag=10U,_tmp8D->f1=e,_tmp8D->f2=scs,_tmp8D->f3=0;_tmp8D;});Cyc_Absyn_new_stmt(_tmp2E2,loc);});}
# 664
struct Cyc_Absyn_Stmt*Cyc_Absyn_trycatch_stmt(struct Cyc_Absyn_Stmt*s,struct Cyc_List_List*scs,unsigned loc){
# 699
return({void*_tmp2E3=(void*)({struct Cyc_Absyn_TryCatch_s_Absyn_Raw_stmt_struct*_tmp8E=_cycalloc(sizeof(*_tmp8E));_tmp8E->tag=15U,_tmp8E->f1=s,_tmp8E->f2=scs,_tmp8E->f3=0;_tmp8E;});Cyc_Absyn_new_stmt(_tmp2E3,loc);});}
# 664
struct Cyc_Absyn_Stmt*Cyc_Absyn_fallthru_stmt(struct Cyc_List_List*el,unsigned loc){
# 702
return({void*_tmp2E4=(void*)({struct Cyc_Absyn_Fallthru_s_Absyn_Raw_stmt_struct*_tmp8F=_cycalloc(sizeof(*_tmp8F));_tmp8F->tag=11U,_tmp8F->f1=el,_tmp8F->f2=0;_tmp8F;});Cyc_Absyn_new_stmt(_tmp2E4,loc);});}
# 664
struct Cyc_Absyn_Stmt*Cyc_Absyn_goto_stmt(struct _fat_ptr*lab,unsigned loc){
# 705
return({void*_tmp2E5=(void*)({struct Cyc_Absyn_Goto_s_Absyn_Raw_stmt_struct*_tmp90=_cycalloc(sizeof(*_tmp90));_tmp90->tag=8U,_tmp90->f1=lab;_tmp90;});Cyc_Absyn_new_stmt(_tmp2E5,loc);});}
# 664
struct Cyc_Absyn_Stmt*Cyc_Absyn_label_stmt(struct _fat_ptr*v,struct Cyc_Absyn_Stmt*s,unsigned loc){
# 708
return({void*_tmp2E6=(void*)({struct Cyc_Absyn_Label_s_Absyn_Raw_stmt_struct*_tmp91=_cycalloc(sizeof(*_tmp91));_tmp91->tag=13U,_tmp91->f1=v,_tmp91->f2=s;_tmp91;});Cyc_Absyn_new_stmt(_tmp2E6,loc);});}
# 664
struct Cyc_Absyn_Stmt*Cyc_Absyn_decl_stmt(struct Cyc_Absyn_Decl*d,struct Cyc_Absyn_Stmt*s,unsigned loc){
# 711
return({void*_tmp2E7=(void*)({struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct*_tmp92=_cycalloc(sizeof(*_tmp92));_tmp92->tag=12U,_tmp92->f1=d,_tmp92->f2=s;_tmp92;});Cyc_Absyn_new_stmt(_tmp2E7,loc);});}
# 664
struct Cyc_Absyn_Stmt*Cyc_Absyn_declare_stmt(struct _tuple0*x,void*t,struct Cyc_Absyn_Exp*init,struct Cyc_Absyn_Stmt*s,unsigned loc){
# 714
struct Cyc_Absyn_Decl*d=({void*_tmp2E9=(void*)({struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct*_tmp94=_cycalloc(sizeof(*_tmp94));_tmp94->tag=0U,({struct Cyc_Absyn_Vardecl*_tmp2E8=Cyc_Absyn_new_vardecl(0U,x,t,init);_tmp94->f1=_tmp2E8;});_tmp94;});Cyc_Absyn_new_decl(_tmp2E9,loc);});
return({void*_tmp2EA=(void*)({struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct*_tmp93=_cycalloc(sizeof(*_tmp93));_tmp93->tag=12U,_tmp93->f1=d,_tmp93->f2=s;_tmp93;});Cyc_Absyn_new_stmt(_tmp2EA,loc);});}
# 664
struct Cyc_Absyn_Stmt*Cyc_Absyn_assign_stmt(struct Cyc_Absyn_Exp*e1,struct Cyc_Absyn_Exp*e2,unsigned loc){
# 718
return({struct Cyc_Absyn_Exp*_tmp2EB=Cyc_Absyn_assign_exp(e1,e2,loc);Cyc_Absyn_exp_stmt(_tmp2EB,loc);});}
# 664
struct Cyc_Absyn_Pat*Cyc_Absyn_new_pat(void*p,unsigned s){
# 721
return({struct Cyc_Absyn_Pat*_tmp95=_cycalloc(sizeof(*_tmp95));_tmp95->r=p,_tmp95->topt=0,_tmp95->loc=s;_tmp95;});}
# 664
struct Cyc_Absyn_Pat*Cyc_Absyn_exp_pat(struct Cyc_Absyn_Exp*e){
# 722
return({void*_tmp2EC=(void*)({struct Cyc_Absyn_Exp_p_Absyn_Raw_pat_struct*_tmp96=_cycalloc(sizeof(*_tmp96));_tmp96->tag=17U,_tmp96->f1=e;_tmp96;});Cyc_Absyn_new_pat(_tmp2EC,e->loc);});}
# 664
struct Cyc_Absyn_Wild_p_Absyn_Raw_pat_struct Cyc_Absyn_Wild_p_val={0U};
# 724
struct Cyc_Absyn_Null_p_Absyn_Raw_pat_struct Cyc_Absyn_Null_p_val={9U};
# 727
struct Cyc_Absyn_Decl*Cyc_Absyn_new_decl(void*r,unsigned loc){return({struct Cyc_Absyn_Decl*_tmp97=_cycalloc(sizeof(*_tmp97));_tmp97->r=r,_tmp97->loc=loc;_tmp97;});}struct Cyc_Absyn_Decl*Cyc_Absyn_let_decl(struct Cyc_Absyn_Pat*p,struct Cyc_Absyn_Exp*e,unsigned loc){
# 729
return({void*_tmp2ED=(void*)({struct Cyc_Absyn_Let_d_Absyn_Raw_decl_struct*_tmp98=_cycalloc(sizeof(*_tmp98));_tmp98->tag=2U,_tmp98->f1=p,_tmp98->f2=0,_tmp98->f3=e,_tmp98->f4=0;_tmp98;});Cyc_Absyn_new_decl(_tmp2ED,loc);});}
# 727
struct Cyc_Absyn_Decl*Cyc_Absyn_letv_decl(struct Cyc_List_List*vds,unsigned loc){
# 732
return({void*_tmp2EE=(void*)({struct Cyc_Absyn_Letv_d_Absyn_Raw_decl_struct*_tmp99=_cycalloc(sizeof(*_tmp99));_tmp99->tag=3U,_tmp99->f1=vds;_tmp99;});Cyc_Absyn_new_decl(_tmp2EE,loc);});}
# 727
struct Cyc_Absyn_Decl*Cyc_Absyn_region_decl(struct Cyc_Absyn_Tvar*tv,struct Cyc_Absyn_Vardecl*vd,struct Cyc_Absyn_Exp*open_exp,unsigned loc){
# 735
return({void*_tmp2EF=(void*)({struct Cyc_Absyn_Region_d_Absyn_Raw_decl_struct*_tmp9A=_cycalloc(sizeof(*_tmp9A));_tmp9A->tag=4U,_tmp9A->f1=tv,_tmp9A->f2=vd,_tmp9A->f3=open_exp;_tmp9A;});Cyc_Absyn_new_decl(_tmp2EF,loc);});}
# 727
struct Cyc_Absyn_Decl*Cyc_Absyn_alias_decl(struct Cyc_Absyn_Tvar*tv,struct Cyc_Absyn_Vardecl*vd,struct Cyc_Absyn_Exp*e,unsigned loc){
# 739
return({void*_tmp2F2=(void*)({struct Cyc_Absyn_Let_d_Absyn_Raw_decl_struct*_tmp9C=_cycalloc(sizeof(*_tmp9C));_tmp9C->tag=2U,({struct Cyc_Absyn_Pat*_tmp2F1=({void*_tmp2F0=(void*)({struct Cyc_Absyn_AliasVar_p_Absyn_Raw_pat_struct*_tmp9B=_cycalloc(sizeof(*_tmp9B));_tmp9B->tag=2U,_tmp9B->f1=tv,_tmp9B->f2=vd;_tmp9B;});Cyc_Absyn_new_pat(_tmp2F0,loc);});_tmp9C->f1=_tmp2F1;}),_tmp9C->f2=0,_tmp9C->f3=e,_tmp9C->f4=0;_tmp9C;});Cyc_Absyn_new_decl(_tmp2F2,loc);});}
# 727
struct Cyc_Absyn_Vardecl*Cyc_Absyn_new_vardecl(unsigned varloc,struct _tuple0*x,void*t,struct Cyc_Absyn_Exp*init){
# 743
return({struct Cyc_Absyn_Vardecl*_tmp9D=_cycalloc(sizeof(*_tmp9D));_tmp9D->sc=Cyc_Absyn_Public,_tmp9D->name=x,_tmp9D->varloc=varloc,({
struct Cyc_Absyn_Tqual _tmp2F3=Cyc_Absyn_empty_tqual(0U);_tmp9D->tq=_tmp2F3;}),_tmp9D->type=t,_tmp9D->initializer=init,_tmp9D->rgn=0,_tmp9D->attributes=0,_tmp9D->escapes=0;_tmp9D;});}
# 727
struct Cyc_Absyn_Vardecl*Cyc_Absyn_static_vardecl(struct _tuple0*x,void*t,struct Cyc_Absyn_Exp*init){
# 749
return({struct Cyc_Absyn_Vardecl*_tmp9E=_cycalloc(sizeof(*_tmp9E));_tmp9E->sc=Cyc_Absyn_Static,_tmp9E->name=x,_tmp9E->varloc=0U,({struct Cyc_Absyn_Tqual _tmp2F4=Cyc_Absyn_empty_tqual(0U);_tmp9E->tq=_tmp2F4;}),_tmp9E->type=t,_tmp9E->initializer=init,_tmp9E->rgn=0,_tmp9E->attributes=0,_tmp9E->escapes=0;_tmp9E;});}
# 727
struct Cyc_Absyn_AggrdeclImpl*Cyc_Absyn_aggrdecl_impl(struct Cyc_List_List*exists,struct Cyc_List_List*po,struct Cyc_List_List*fs,int tagged){
# 757
return({struct Cyc_Absyn_AggrdeclImpl*_tmp9F=_cycalloc(sizeof(*_tmp9F));_tmp9F->exist_vars=exists,_tmp9F->rgn_po=po,_tmp9F->fields=fs,_tmp9F->tagged=tagged;_tmp9F;});}
# 727
struct Cyc_Absyn_Decl*Cyc_Absyn_aggr_decl(enum Cyc_Absyn_AggrKind k,enum Cyc_Absyn_Scope s,struct _tuple0*n,struct Cyc_List_List*ts,struct Cyc_Absyn_AggrdeclImpl*i,struct Cyc_List_List*atts,unsigned loc){
# 763
return({void*_tmp2F6=(void*)({struct Cyc_Absyn_Aggr_d_Absyn_Raw_decl_struct*_tmpA1=_cycalloc(sizeof(*_tmpA1));_tmpA1->tag=5U,({struct Cyc_Absyn_Aggrdecl*_tmp2F5=({struct Cyc_Absyn_Aggrdecl*_tmpA0=_cycalloc(sizeof(*_tmpA0));_tmpA0->kind=k,_tmpA0->sc=s,_tmpA0->name=n,_tmpA0->tvs=ts,_tmpA0->impl=i,_tmpA0->attributes=atts,_tmpA0->expected_mem_kind=0;_tmpA0;});_tmpA1->f1=_tmp2F5;});_tmpA1;});Cyc_Absyn_new_decl(_tmp2F6,loc);});}
# 727
struct Cyc_Absyn_TypeDecl*Cyc_Absyn_aggr_tdecl(enum Cyc_Absyn_AggrKind k,enum Cyc_Absyn_Scope s,struct _tuple0*n,struct Cyc_List_List*ts,struct Cyc_Absyn_AggrdeclImpl*i,struct Cyc_List_List*atts,unsigned loc){
# 771
return({struct Cyc_Absyn_TypeDecl*_tmpA4=_cycalloc(sizeof(*_tmpA4));({void*_tmp2F8=(void*)({struct Cyc_Absyn_Aggr_td_Absyn_Raw_typedecl_struct*_tmpA3=_cycalloc(sizeof(*_tmpA3));_tmpA3->tag=0U,({struct Cyc_Absyn_Aggrdecl*_tmp2F7=({struct Cyc_Absyn_Aggrdecl*_tmpA2=_cycalloc(sizeof(*_tmpA2));_tmpA2->kind=k,_tmpA2->sc=s,_tmpA2->name=n,_tmpA2->tvs=ts,_tmpA2->impl=i,_tmpA2->attributes=atts,_tmpA2->expected_mem_kind=0;_tmpA2;});_tmpA3->f1=_tmp2F7;});_tmpA3;});_tmpA4->r=_tmp2F8;}),_tmpA4->loc=loc;_tmpA4;});}
# 727
struct Cyc_Absyn_Decl*Cyc_Absyn_struct_decl(enum Cyc_Absyn_Scope s,struct _tuple0*n,struct Cyc_List_List*ts,struct Cyc_Absyn_AggrdeclImpl*i,struct Cyc_List_List*atts,unsigned loc){
# 780
return Cyc_Absyn_aggr_decl(Cyc_Absyn_StructA,s,n,ts,i,atts,loc);}
# 727
struct Cyc_Absyn_Decl*Cyc_Absyn_union_decl(enum Cyc_Absyn_Scope s,struct _tuple0*n,struct Cyc_List_List*ts,struct Cyc_Absyn_AggrdeclImpl*i,struct Cyc_List_List*atts,unsigned loc){
# 785
return Cyc_Absyn_aggr_decl(Cyc_Absyn_UnionA,s,n,ts,i,atts,loc);}
# 727
struct Cyc_Absyn_Decl*Cyc_Absyn_datatype_decl(enum Cyc_Absyn_Scope s,struct _tuple0*n,struct Cyc_List_List*ts,struct Cyc_Core_Opt*fs,int is_extensible,unsigned loc){
# 790
return({void*_tmp2FA=(void*)({struct Cyc_Absyn_Datatype_d_Absyn_Raw_decl_struct*_tmpA6=_cycalloc(sizeof(*_tmpA6));_tmpA6->tag=6U,({struct Cyc_Absyn_Datatypedecl*_tmp2F9=({struct Cyc_Absyn_Datatypedecl*_tmpA5=_cycalloc(sizeof(*_tmpA5));_tmpA5->sc=s,_tmpA5->name=n,_tmpA5->tvs=ts,_tmpA5->fields=fs,_tmpA5->is_extensible=is_extensible;_tmpA5;});_tmpA6->f1=_tmp2F9;});_tmpA6;});Cyc_Absyn_new_decl(_tmp2FA,loc);});}
# 727
struct Cyc_Absyn_TypeDecl*Cyc_Absyn_datatype_tdecl(enum Cyc_Absyn_Scope s,struct _tuple0*n,struct Cyc_List_List*ts,struct Cyc_Core_Opt*fs,int is_extensible,unsigned loc){
# 796
return({struct Cyc_Absyn_TypeDecl*_tmpA9=_cycalloc(sizeof(*_tmpA9));({void*_tmp2FC=(void*)({struct Cyc_Absyn_Datatype_td_Absyn_Raw_typedecl_struct*_tmpA8=_cycalloc(sizeof(*_tmpA8));_tmpA8->tag=2U,({struct Cyc_Absyn_Datatypedecl*_tmp2FB=({struct Cyc_Absyn_Datatypedecl*_tmpA7=_cycalloc(sizeof(*_tmpA7));_tmpA7->sc=s,_tmpA7->name=n,_tmpA7->tvs=ts,_tmpA7->fields=fs,_tmpA7->is_extensible=is_extensible;_tmpA7;});_tmpA8->f1=_tmp2FB;});_tmpA8;});_tmpA9->r=_tmp2FC;}),_tmpA9->loc=loc;_tmpA9;});}
# 727
void*Cyc_Absyn_function_type(struct Cyc_List_List*tvs,void*eff_type,struct Cyc_Absyn_Tqual ret_tqual,void*ret_type,struct Cyc_List_List*args,int c_varargs,struct Cyc_Absyn_VarargInfo*cyc_varargs,struct Cyc_List_List*rgn_po,struct Cyc_List_List*atts,struct Cyc_Absyn_Exp*req,struct Cyc_Absyn_Exp*ens,struct Cyc_List_List*ieff,struct Cyc_List_List*oeff){
# 818 "absyn.cyc"
{struct Cyc_List_List*args2=args;for(0;args2 != 0;args2=args2->tl){
({void*_tmp2FD=Cyc_Absyn_pointer_expand((*((struct _tuple8*)args2->hd)).f3,1);(*((struct _tuple8*)args2->hd)).f3=_tmp2FD;});}}
return(void*)({struct Cyc_Absyn_FnType_Absyn_Type_struct*_tmpAA=_cycalloc(sizeof(*_tmpAA));_tmpAA->tag=5U,(_tmpAA->f1).tvars=tvs,(_tmpAA->f1).ret_tqual=ret_tqual,({
# 822
void*_tmp2FE=Cyc_Absyn_pointer_expand(ret_type,0);(_tmpAA->f1).ret_type=_tmp2FE;}),(_tmpAA->f1).effect=eff_type,(_tmpAA->f1).args=args,(_tmpAA->f1).c_varargs=c_varargs,(_tmpAA->f1).cyc_varargs=cyc_varargs,(_tmpAA->f1).rgn_po=rgn_po,(_tmpAA->f1).attributes=atts,(_tmpAA->f1).requires_clause=req,(_tmpAA->f1).requires_relns=0,(_tmpAA->f1).ensures_clause=ens,(_tmpAA->f1).ensures_relns=0,(_tmpAA->f1).ieffect=ieff,(_tmpAA->f1).oeffect=oeff;_tmpAA;});}
# 727 "absyn.cyc"
void*Cyc_Absyn_pointer_expand(void*t,int fresh_evar){
# 839 "absyn.cyc"
void*_stmttmp4=Cyc_Tcutil_compress(t);void*_tmpAB=_stmttmp4;if(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmpAB)->tag == 5U){_LL1: _LL2:
 return({void*_tmp301=t;void*_tmp300=
fresh_evar?
Cyc_Absyn_new_evar(({struct Cyc_Core_Opt*_tmpAC=_cycalloc(sizeof(*_tmpAC));_tmpAC->v=& Cyc_Tcutil_rk;_tmpAC;}),0): Cyc_Absyn_heap_rgn_type;
# 840
struct Cyc_Absyn_Tqual _tmp2FF=
# 844
Cyc_Absyn_empty_tqual(0U);
# 840
Cyc_Absyn_at_type(_tmp301,_tmp300,_tmp2FF,Cyc_Absyn_false_type);});}else{_LL3: _LL4:
# 845
 return t;}_LL0:;}
# 727 "absyn.cyc"
int Cyc_Absyn_is_lvalue(struct Cyc_Absyn_Exp*e){
# 861 "absyn.cyc"
void*_stmttmp5=e->r;void*_tmpAD=_stmttmp5;struct Cyc_Absyn_Exp*_tmpAE;struct Cyc_Absyn_Exp*_tmpAF;struct Cyc_Absyn_Exp*_tmpB0;struct Cyc_Absyn_Vardecl*_tmpB1;struct Cyc_Absyn_Vardecl*_tmpB2;switch(*((int*)_tmpAD)){case 1U: switch(*((int*)((struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct*)_tmpAD)->f1)){case 2U: _LL1: _LL2:
 return 0;case 1U: _LL3: _tmpB2=((struct Cyc_Absyn_Global_b_Absyn_Binding_struct*)((struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct*)_tmpAD)->f1)->f1;_LL4: {struct Cyc_Absyn_Vardecl*vd=_tmpB2;
_tmpB1=vd;goto _LL6;}case 4U: _LL5: _tmpB1=((struct Cyc_Absyn_Local_b_Absyn_Binding_struct*)((struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct*)_tmpAD)->f1)->f1;_LL6: {struct Cyc_Absyn_Vardecl*vd=_tmpB1;
# 865
void*_stmttmp6=Cyc_Tcutil_compress(vd->type);void*_tmpB3=_stmttmp6;if(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmpB3)->tag == 4U){_LL18: _LL19:
 return 0;}else{_LL1A: _LL1B:
 return 1;}_LL17:;}default: _LL7: _LL8:
# 869
 goto _LLA;}case 22U: _LL9: _LLA:
 goto _LLC;case 20U: _LLB: _LLC:
 goto _LLE;case 23U: _LLD: _LLE:
 return 1;case 21U: _LLF: _tmpB0=((struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct*)_tmpAD)->f1;_LL10: {struct Cyc_Absyn_Exp*e1=_tmpB0;
return Cyc_Absyn_is_lvalue(e1);}case 13U: _LL11: _tmpAF=((struct Cyc_Absyn_Instantiate_e_Absyn_Raw_exp_struct*)_tmpAD)->f1;_LL12: {struct Cyc_Absyn_Exp*e1=_tmpAF;
return Cyc_Absyn_is_lvalue(e1);}case 12U: _LL13: _tmpAE=((struct Cyc_Absyn_NoInstantiate_e_Absyn_Raw_exp_struct*)_tmpAD)->f1;_LL14: {struct Cyc_Absyn_Exp*e1=_tmpAE;
return Cyc_Absyn_is_lvalue(e1);}default: _LL15: _LL16:
 return 0;}_LL0:;}
# 727 "absyn.cyc"
struct Cyc_Absyn_Aggrfield*Cyc_Absyn_lookup_field(struct Cyc_List_List*fields,struct _fat_ptr*v){
# 881 "absyn.cyc"
{struct Cyc_List_List*fs=fields;for(0;fs != 0;fs=fs->tl){
if(Cyc_strptrcmp(((struct Cyc_Absyn_Aggrfield*)fs->hd)->name,v)== 0)
return(struct Cyc_Absyn_Aggrfield*)fs->hd;}}
# 881
return 0;}
# 727 "absyn.cyc"
struct Cyc_Absyn_Aggrfield*Cyc_Absyn_lookup_decl_field(struct Cyc_Absyn_Aggrdecl*ad,struct _fat_ptr*v){
# 887 "absyn.cyc"
return ad->impl == 0?0: Cyc_Absyn_lookup_field(((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->fields,v);}struct _tuple14{struct Cyc_Absyn_Tqual f1;void*f2;};
# 727 "absyn.cyc"
struct _tuple14*Cyc_Absyn_lookup_tuple_field(struct Cyc_List_List*ts,int i){
# 891 "absyn.cyc"
for(0;i != 0;-- i){
if(ts == 0)
return 0;
# 892
ts=ts->tl;}
# 896
if(ts == 0)
return 0;
# 896
return(struct _tuple14*)ts->hd;}
# 727 "absyn.cyc"
struct _fat_ptr*Cyc_Absyn_decl_name(struct Cyc_Absyn_Decl*decl){
# 902 "absyn.cyc"
void*_stmttmp7=decl->r;void*_tmpB4=_stmttmp7;struct Cyc_Absyn_Fndecl*_tmpB5;struct Cyc_Absyn_Vardecl*_tmpB6;struct Cyc_Absyn_Typedefdecl*_tmpB7;struct Cyc_Absyn_Enumdecl*_tmpB8;struct Cyc_Absyn_Aggrdecl*_tmpB9;switch(*((int*)_tmpB4)){case 5U: _LL1: _tmpB9=((struct Cyc_Absyn_Aggr_d_Absyn_Raw_decl_struct*)_tmpB4)->f1;_LL2: {struct Cyc_Absyn_Aggrdecl*x=_tmpB9;
return(*x->name).f2;}case 7U: _LL3: _tmpB8=((struct Cyc_Absyn_Enum_d_Absyn_Raw_decl_struct*)_tmpB4)->f1;_LL4: {struct Cyc_Absyn_Enumdecl*x=_tmpB8;
return(*x->name).f2;}case 8U: _LL5: _tmpB7=((struct Cyc_Absyn_Typedef_d_Absyn_Raw_decl_struct*)_tmpB4)->f1;_LL6: {struct Cyc_Absyn_Typedefdecl*x=_tmpB7;
return(*x->name).f2;}case 0U: _LL7: _tmpB6=((struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct*)_tmpB4)->f1;_LL8: {struct Cyc_Absyn_Vardecl*x=_tmpB6;
return(*x->name).f2;}case 1U: _LL9: _tmpB5=((struct Cyc_Absyn_Fn_d_Absyn_Raw_decl_struct*)_tmpB4)->f1;_LLA: {struct Cyc_Absyn_Fndecl*x=_tmpB5;
return(*x->name).f2;}case 13U: _LLB: _LLC:
 goto _LLE;case 14U: _LLD: _LLE:
 goto _LL10;case 15U: _LLF: _LL10:
 goto _LL12;case 16U: _LL11: _LL12:
 goto _LL14;case 2U: _LL13: _LL14:
 goto _LL16;case 6U: _LL15: _LL16:
 goto _LL18;case 3U: _LL17: _LL18:
 goto _LL1A;case 9U: _LL19: _LL1A:
 goto _LL1C;case 10U: _LL1B: _LL1C:
 goto _LL1E;case 11U: _LL1D: _LL1E:
 goto _LL20;case 12U: _LL1F: _LL20:
 goto _LL22;default: _LL21: _LL22:
 return 0;}_LL0:;}
# 727 "absyn.cyc"
struct Cyc_Absyn_Decl*Cyc_Absyn_lookup_decl(struct Cyc_List_List*decls,struct _fat_ptr*name){
# 925 "absyn.cyc"
struct _fat_ptr*dname;
for(0;decls != 0;decls=decls->tl){
dname=Cyc_Absyn_decl_name((struct Cyc_Absyn_Decl*)decls->hd);
if((unsigned)dname && !Cyc_strptrcmp(dname,name))
return(struct Cyc_Absyn_Decl*)decls->hd;}
# 931
return 0;}
# 727 "absyn.cyc"
struct Cyc_Absyn_Stdcall_att_Absyn_Attribute_struct Cyc_Absyn_Stdcall_att_val={1U};
# 935 "absyn.cyc"
struct Cyc_Absyn_Cdecl_att_Absyn_Attribute_struct Cyc_Absyn_Cdecl_att_val={2U};
struct Cyc_Absyn_Fastcall_att_Absyn_Attribute_struct Cyc_Absyn_Fastcall_att_val={3U};
struct Cyc_Absyn_Noreturn_att_Absyn_Attribute_struct Cyc_Absyn_Noreturn_att_val={4U};
struct Cyc_Absyn_Const_att_Absyn_Attribute_struct Cyc_Absyn_Const_att_val={5U};
struct Cyc_Absyn_Packed_att_Absyn_Attribute_struct Cyc_Absyn_Packed_att_val={7U};
struct Cyc_Absyn_Nocommon_att_Absyn_Attribute_struct Cyc_Absyn_Nocommon_att_val={9U};
struct Cyc_Absyn_Shared_att_Absyn_Attribute_struct Cyc_Absyn_Shared_att_val={10U};
struct Cyc_Absyn_Unused_att_Absyn_Attribute_struct Cyc_Absyn_Unused_att_val={11U};
struct Cyc_Absyn_Weak_att_Absyn_Attribute_struct Cyc_Absyn_Weak_att_val={12U};
struct Cyc_Absyn_Dllimport_att_Absyn_Attribute_struct Cyc_Absyn_Dllimport_att_val={13U};
struct Cyc_Absyn_Dllexport_att_Absyn_Attribute_struct Cyc_Absyn_Dllexport_att_val={14U};
struct Cyc_Absyn_No_instrument_function_att_Absyn_Attribute_struct Cyc_Absyn_No_instrument_function_att_val={15U};
struct Cyc_Absyn_Constructor_att_Absyn_Attribute_struct Cyc_Absyn_Constructor_att_val={16U};
struct Cyc_Absyn_Destructor_att_Absyn_Attribute_struct Cyc_Absyn_Destructor_att_val={17U};
struct Cyc_Absyn_No_check_memory_usage_att_Absyn_Attribute_struct Cyc_Absyn_No_check_memory_usage_att_val={18U};
struct Cyc_Absyn_Pure_att_Absyn_Attribute_struct Cyc_Absyn_Pure_att_val={23U};
struct Cyc_Absyn_Always_inline_att_Absyn_Attribute_struct Cyc_Absyn_Always_inline_att_val={26U};
struct Cyc_Absyn_No_throw_att_Absyn_Attribute_struct Cyc_Absyn_No_throw_att_val={27U};
struct Cyc_Absyn_Non_null_att_Absyn_Attribute_struct Cyc_Absyn_Non_null_att_val={28U};
struct Cyc_Absyn_Deprecated_att_Absyn_Attribute_struct Cyc_Absyn_Deprecated_att_val={29U};
# 957
static void*Cyc_Absyn_bad_attribute(unsigned loc,struct _fat_ptr s){
# 959
({void*_tmpBA=0U;({unsigned _tmp304=loc;struct _fat_ptr _tmp303=(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmpBD=({struct Cyc_String_pa_PrintArg_struct _tmp24E;_tmp24E.tag=0U,_tmp24E.f1=(struct _fat_ptr)((struct _fat_ptr)s);_tmp24E;});void*_tmpBB[1U];_tmpBB[0]=& _tmpBD;({struct _fat_ptr _tmp302=({const char*_tmpBC="Unrecognized attribute : %s";_tag_fat(_tmpBC,sizeof(char),28U);});Cyc_aprintf(_tmp302,_tag_fat(_tmpBB,sizeof(void*),1U));});});Cyc_Warn_err(_tmp304,_tmp303,_tag_fat(_tmpBA,sizeof(void*),0U));});});
return(void*)& Cyc_Absyn_Cdecl_att_val;}
# 957
static struct _fat_ptr Cyc_Absyn_exp2string_(unsigned loc,struct Cyc_Absyn_Exp*e){
# 965
void*_stmttmp8=e->r;void*_tmpBE=_stmttmp8;struct _fat_ptr _tmpBF;if(((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmpBE)->tag == 0U){if(((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmpBE)->f1).String_c).tag == 8){_LL1: _tmpBF=((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmpBE)->f1).String_c).val;_LL2: {struct _fat_ptr s=_tmpBF;
return s;}}else{goto _LL3;}}else{_LL3: _LL4:
# 968
({void*_tmpC0=0U;({unsigned _tmp306=loc;struct _fat_ptr _tmp305=({const char*_tmpC1="expecting string constant";_tag_fat(_tmpC1,sizeof(char),26U);});Cyc_Warn_err(_tmp306,_tmp305,_tag_fat(_tmpC0,sizeof(void*),0U));});});
return _tag_fat(0,0,0);}_LL0:;}struct _tuple15{struct _fat_ptr f1;void*f2;};static char _tmpC2[8U]="stdcall";static char _tmpC3[6U]="cdecl";static char _tmpC4[9U]="fastcall";static char _tmpC5[9U]="noreturn";static char _tmpC6[6U]="const";static char _tmpC7[8U]="aligned";static char _tmpC8[7U]="packed";static char _tmpC9[7U]="shared";static char _tmpCA[7U]="unused";static char _tmpCB[5U]="weak";static char _tmpCC[10U]="dllimport";static char _tmpCD[10U]="dllexport";static char _tmpCE[23U]="no_instrument_function";static char _tmpCF[12U]="constructor";static char _tmpD0[11U]="destructor";static char _tmpD1[22U]="no_check_memory_usage";static char _tmpD2[5U]="pure";static char _tmpD3[14U]="always_inline";static char _tmpD4[9U]="no_throw";static char _tmpD5[8U]="nothrow";static char _tmpD6[8U]="nonnull";static char _tmpD7[11U]="deprecated";
# 957
void*Cyc_Absyn_parse_nullary_att(unsigned loc,struct _fat_ptr s){
# 975
static struct Cyc_Absyn_Aligned_att_Absyn_Attribute_struct att_aligned={6U,0};
static struct _tuple15 att_map[22U]={{{_tmpC2,_tmpC2,_tmpC2 + 8U},(void*)& Cyc_Absyn_Stdcall_att_val},{{_tmpC3,_tmpC3,_tmpC3 + 6U},(void*)& Cyc_Absyn_Cdecl_att_val},{{_tmpC4,_tmpC4,_tmpC4 + 9U},(void*)& Cyc_Absyn_Fastcall_att_val},{{_tmpC5,_tmpC5,_tmpC5 + 9U},(void*)& Cyc_Absyn_Noreturn_att_val},{{_tmpC6,_tmpC6,_tmpC6 + 6U},(void*)& Cyc_Absyn_Const_att_val},{{_tmpC7,_tmpC7,_tmpC7 + 8U},(void*)& att_aligned},{{_tmpC8,_tmpC8,_tmpC8 + 7U},(void*)& Cyc_Absyn_Packed_att_val},{{_tmpC9,_tmpC9,_tmpC9 + 7U},(void*)& Cyc_Absyn_Shared_att_val},{{_tmpCA,_tmpCA,_tmpCA + 7U},(void*)& Cyc_Absyn_Unused_att_val},{{_tmpCB,_tmpCB,_tmpCB + 5U},(void*)& Cyc_Absyn_Weak_att_val},{{_tmpCC,_tmpCC,_tmpCC + 10U},(void*)& Cyc_Absyn_Dllimport_att_val},{{_tmpCD,_tmpCD,_tmpCD + 10U},(void*)& Cyc_Absyn_Dllexport_att_val},{{_tmpCE,_tmpCE,_tmpCE + 23U},(void*)& Cyc_Absyn_No_instrument_function_att_val},{{_tmpCF,_tmpCF,_tmpCF + 12U},(void*)& Cyc_Absyn_Constructor_att_val},{{_tmpD0,_tmpD0,_tmpD0 + 11U},(void*)& Cyc_Absyn_Destructor_att_val},{{_tmpD1,_tmpD1,_tmpD1 + 22U},(void*)& Cyc_Absyn_No_check_memory_usage_att_val},{{_tmpD2,_tmpD2,_tmpD2 + 5U},(void*)& Cyc_Absyn_Pure_att_val},{{_tmpD3,_tmpD3,_tmpD3 + 14U},(void*)& Cyc_Absyn_Always_inline_att_val},{{_tmpD4,_tmpD4,_tmpD4 + 9U},(void*)& Cyc_Absyn_No_throw_att_val},{{_tmpD5,_tmpD5,_tmpD5 + 8U},(void*)& Cyc_Absyn_No_throw_att_val},{{_tmpD6,_tmpD6,_tmpD6 + 8U},(void*)& Cyc_Absyn_Non_null_att_val},{{_tmpD7,_tmpD7,_tmpD7 + 11U},(void*)& Cyc_Absyn_Deprecated_att_val}};
# 1001
if((((_get_fat_size(s,sizeof(char))> (unsigned)4 &&(int)((const char*)s.curr)[0]== (int)'_')&&(int)((const char*)s.curr)[1]== (int)'_')&&(int)*((const char*)_check_fat_subscript(s,sizeof(char),(int)(_get_fat_size(s,sizeof(char))- (unsigned)2)))== (int)'_')&&(int)*((const char*)_check_fat_subscript(s,sizeof(char),(int)(_get_fat_size(s,sizeof(char))- (unsigned)3)))== (int)'_')
# 1003
s=(struct _fat_ptr)Cyc_substring((struct _fat_ptr)s,2,_get_fat_size(s,sizeof(char))- (unsigned)5);{
# 1001
int i=0;
# 1005
for(0;(unsigned)i < 22U;++ i){
if(Cyc_strcmp((struct _fat_ptr)s,(struct _fat_ptr)(*((struct _tuple15*)_check_known_subscript_notnull(att_map,22U,sizeof(struct _tuple15),i))).f1)== 0)
return(att_map[i]).f2;}
# 1005
return
# 1008
 Cyc_Absyn_bad_attribute(loc,s);}}
# 957
static int Cyc_Absyn_exp2int(unsigned loc,struct Cyc_Absyn_Exp*e){
# 1013
void*_stmttmp9=e->r;void*_tmpD8=_stmttmp9;int _tmpD9;if(((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmpD8)->tag == 0U){if(((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmpD8)->f1).Int_c).tag == 5){_LL1: _tmpD9=(((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmpD8)->f1).Int_c).val).f2;_LL2: {int i=_tmpD9;
return i;}}else{goto _LL3;}}else{_LL3: _LL4:
# 1016
({void*_tmpDA=0U;({unsigned _tmp308=loc;struct _fat_ptr _tmp307=({const char*_tmpDB="expecting integer constant";_tag_fat(_tmpDB,sizeof(char),27U);});Cyc_Warn_err(_tmp308,_tmp307,_tag_fat(_tmpDA,sizeof(void*),0U));});});
return 0;}_LL0:;}
# 957
void*Cyc_Absyn_parse_unary_att(unsigned sloc,struct _fat_ptr s,unsigned eloc,struct Cyc_Absyn_Exp*e){
# 1024
void*a;
if(({struct _fat_ptr _tmp30A=(struct _fat_ptr)s;Cyc_zstrcmp(_tmp30A,({const char*_tmpDC="aligned";_tag_fat(_tmpDC,sizeof(char),8U);}));})== 0 ||({struct _fat_ptr _tmp309=(struct _fat_ptr)s;Cyc_zstrcmp(_tmp309,({const char*_tmpDD="__aligned__";_tag_fat(_tmpDD,sizeof(char),12U);}));})== 0)
return(void*)({struct Cyc_Absyn_Aligned_att_Absyn_Attribute_struct*_tmpDE=_cycalloc(sizeof(*_tmpDE));_tmpDE->tag=6U,_tmpDE->f1=e;_tmpDE;});
# 1025
if(
# 1027
({struct _fat_ptr _tmp30C=(struct _fat_ptr)s;Cyc_zstrcmp(_tmp30C,({const char*_tmpDF="section";_tag_fat(_tmpDF,sizeof(char),8U);}));})== 0 ||({struct _fat_ptr _tmp30B=(struct _fat_ptr)s;Cyc_zstrcmp(_tmp30B,({const char*_tmpE0="__section__";_tag_fat(_tmpE0,sizeof(char),12U);}));})== 0)
return(void*)({struct Cyc_Absyn_Section_att_Absyn_Attribute_struct*_tmpE1=_cycalloc(sizeof(*_tmpE1));_tmpE1->tag=8U,({struct _fat_ptr _tmp30D=Cyc_Absyn_exp2string_(eloc,e);_tmpE1->f1=_tmp30D;});_tmpE1;});
# 1025
if(({
# 1029
struct _fat_ptr _tmp30E=(struct _fat_ptr)s;Cyc_zstrcmp(_tmp30E,({const char*_tmpE2="__mode__";_tag_fat(_tmpE2,sizeof(char),9U);}));})== 0)
return(void*)({struct Cyc_Absyn_Mode_att_Absyn_Attribute_struct*_tmpE3=_cycalloc(sizeof(*_tmpE3));_tmpE3->tag=24U,({struct _fat_ptr _tmp30F=Cyc_Absyn_exp2string_(eloc,e);_tmpE3->f1=_tmp30F;});_tmpE3;});
# 1025
if(({
# 1031
struct _fat_ptr _tmp310=(struct _fat_ptr)s;Cyc_zstrcmp(_tmp310,({const char*_tmpE4="alias";_tag_fat(_tmpE4,sizeof(char),6U);}));})== 0)
return(void*)({struct Cyc_Absyn_Alias_att_Absyn_Attribute_struct*_tmpE5=_cycalloc(sizeof(*_tmpE5));_tmpE5->tag=25U,({struct _fat_ptr _tmp311=Cyc_Absyn_exp2string_(eloc,e);_tmpE5->f1=_tmp311;});_tmpE5;});{
# 1025
int n=
# 1033
Cyc_Absyn_exp2int(eloc,e);
if(({struct _fat_ptr _tmp313=(struct _fat_ptr)s;Cyc_zstrcmp(_tmp313,({const char*_tmpE6="regparm";_tag_fat(_tmpE6,sizeof(char),8U);}));})== 0 ||({struct _fat_ptr _tmp312=(struct _fat_ptr)s;Cyc_zstrcmp(_tmp312,({const char*_tmpE7="__regparm__";_tag_fat(_tmpE7,sizeof(char),12U);}));})== 0){
if(n < 0 || n > 3)
({void*_tmpE8=0U;({unsigned _tmp315=eloc;struct _fat_ptr _tmp314=({const char*_tmpE9="regparm requires value between 0 and 3";_tag_fat(_tmpE9,sizeof(char),39U);});Cyc_Warn_err(_tmp315,_tmp314,_tag_fat(_tmpE8,sizeof(void*),0U));});});
# 1035
return(void*)({struct Cyc_Absyn_Regparm_att_Absyn_Attribute_struct*_tmpEA=_cycalloc(sizeof(*_tmpEA));
# 1037
_tmpEA->tag=0U,_tmpEA->f1=n;_tmpEA;});}
# 1034
if(
# 1039
({struct _fat_ptr _tmp317=(struct _fat_ptr)s;Cyc_zstrcmp(_tmp317,({const char*_tmpEB="initializes";_tag_fat(_tmpEB,sizeof(char),12U);}));})== 0 ||({struct _fat_ptr _tmp316=(struct _fat_ptr)s;Cyc_zstrcmp(_tmp316,({const char*_tmpEC="__initializes__";_tag_fat(_tmpEC,sizeof(char),16U);}));})== 0)
return(void*)({struct Cyc_Absyn_Initializes_att_Absyn_Attribute_struct*_tmpED=_cycalloc(sizeof(*_tmpED));_tmpED->tag=20U,_tmpED->f1=n;_tmpED;});
# 1034
if(
# 1041
({struct _fat_ptr _tmp319=(struct _fat_ptr)s;Cyc_zstrcmp(_tmp319,({const char*_tmpEE="noliveunique";_tag_fat(_tmpEE,sizeof(char),13U);}));})== 0 ||({struct _fat_ptr _tmp318=(struct _fat_ptr)s;Cyc_zstrcmp(_tmp318,({const char*_tmpEF="__noliveunique__";_tag_fat(_tmpEF,sizeof(char),17U);}));})== 0)
return(void*)({struct Cyc_Absyn_Noliveunique_att_Absyn_Attribute_struct*_tmpF0=_cycalloc(sizeof(*_tmpF0));_tmpF0->tag=21U,_tmpF0->f1=n;_tmpF0;});
# 1034
if(
# 1043
({struct _fat_ptr _tmp31B=(struct _fat_ptr)s;Cyc_zstrcmp(_tmp31B,({const char*_tmpF1="consume";_tag_fat(_tmpF1,sizeof(char),8U);}));})== 0 ||({struct _fat_ptr _tmp31A=(struct _fat_ptr)s;Cyc_zstrcmp(_tmp31A,({const char*_tmpF2="__consume__";_tag_fat(_tmpF2,sizeof(char),12U);}));})== 0)
return(void*)({struct Cyc_Absyn_Consume_att_Absyn_Attribute_struct*_tmpF3=_cycalloc(sizeof(*_tmpF3));_tmpF3->tag=22U,_tmpF3->f1=n;_tmpF3;});
# 1034
if(
# 1045
({struct _fat_ptr _tmp31D=(struct _fat_ptr)s;Cyc_zstrcmp(_tmp31D,({const char*_tmpF4="nonnull";_tag_fat(_tmpF4,sizeof(char),8U);}));})== 0 ||({struct _fat_ptr _tmp31C=(struct _fat_ptr)s;Cyc_zstrcmp(_tmp31C,({const char*_tmpF5="__nonnull__";_tag_fat(_tmpF5,sizeof(char),12U);}));})== 0)
return(void*)({struct Cyc_Absyn_Non_null_att_Absyn_Attribute_struct*_tmpF6=_cycalloc(sizeof(*_tmpF6));_tmpF6->tag=28U;_tmpF6;});
# 1034
return
# 1047
 Cyc_Absyn_bad_attribute(sloc,s);}}
# 957
void*Cyc_Absyn_parse_format_att(unsigned loc,unsigned s2loc,struct _fat_ptr s1,struct _fat_ptr s2,unsigned u1,unsigned u2){
# 1053
if(!(({struct _fat_ptr _tmp31F=(struct _fat_ptr)s1;Cyc_zstrcmp(_tmp31F,({const char*_tmpF7="format";_tag_fat(_tmpF7,sizeof(char),7U);}));})== 0)&& !(({struct _fat_ptr _tmp31E=(struct _fat_ptr)s1;Cyc_zstrcmp(_tmp31E,({const char*_tmpF8="__format__";_tag_fat(_tmpF8,sizeof(char),11U);}));})== 0))
return Cyc_Absyn_bad_attribute(loc,s1);
# 1053
if(
# 1055
({struct _fat_ptr _tmp321=(struct _fat_ptr)s2;Cyc_zstrcmp(_tmp321,({const char*_tmpF9="printf";_tag_fat(_tmpF9,sizeof(char),7U);}));})== 0 ||({struct _fat_ptr _tmp320=(struct _fat_ptr)s2;Cyc_zstrcmp(_tmp320,({const char*_tmpFA="__printf__";_tag_fat(_tmpFA,sizeof(char),11U);}));})== 0)
return(void*)({struct Cyc_Absyn_Format_att_Absyn_Attribute_struct*_tmpFB=_cycalloc(sizeof(*_tmpFB));_tmpFB->tag=19U,_tmpFB->f1=Cyc_Absyn_Printf_ft,_tmpFB->f2=(int)u1,_tmpFB->f3=(int)u2;_tmpFB;});
# 1053
if(
# 1057
({struct _fat_ptr _tmp323=(struct _fat_ptr)s2;Cyc_zstrcmp(_tmp323,({const char*_tmpFC="scanf";_tag_fat(_tmpFC,sizeof(char),6U);}));})== 0 ||({struct _fat_ptr _tmp322=(struct _fat_ptr)s2;Cyc_zstrcmp(_tmp322,({const char*_tmpFD="__scanf__";_tag_fat(_tmpFD,sizeof(char),10U);}));})== 0)
return(void*)({struct Cyc_Absyn_Format_att_Absyn_Attribute_struct*_tmpFE=_cycalloc(sizeof(*_tmpFE));_tmpFE->tag=19U,_tmpFE->f1=Cyc_Absyn_Scanf_ft,_tmpFE->f2=(int)u1,_tmpFE->f3=(int)u2;_tmpFE;});
# 1053
({void*_tmpFF=0U;({unsigned _tmp325=loc;struct _fat_ptr _tmp324=({const char*_tmp100="unrecognized format type";_tag_fat(_tmp100,sizeof(char),25U);});Cyc_Warn_err(_tmp325,_tmp324,_tag_fat(_tmpFF,sizeof(void*),0U));});});
# 1060
return(void*)& Cyc_Absyn_Cdecl_att_val;}
# 957
struct _fat_ptr Cyc_Absyn_attribute2string(void*a){
# 1066
void*_tmp101=a;struct _fat_ptr _tmp102;struct _fat_ptr _tmp103;int _tmp104;int _tmp105;int _tmp106;int _tmp108;int _tmp107;int _tmp10A;int _tmp109;struct _fat_ptr _tmp10B;struct Cyc_Absyn_Exp*_tmp10C;int _tmp10D;switch(*((int*)_tmp101)){case 0U: _LL1: _tmp10D=((struct Cyc_Absyn_Regparm_att_Absyn_Attribute_struct*)_tmp101)->f1;_LL2: {int i=_tmp10D;
return(struct _fat_ptr)({struct Cyc_Int_pa_PrintArg_struct _tmp110=({struct Cyc_Int_pa_PrintArg_struct _tmp24F;_tmp24F.tag=1U,_tmp24F.f1=(unsigned long)i;_tmp24F;});void*_tmp10E[1U];_tmp10E[0]=& _tmp110;({struct _fat_ptr _tmp326=({const char*_tmp10F="regparm(%d)";_tag_fat(_tmp10F,sizeof(char),12U);});Cyc_aprintf(_tmp326,_tag_fat(_tmp10E,sizeof(void*),1U));});});}case 1U: _LL3: _LL4:
 return({const char*_tmp111="stdcall";_tag_fat(_tmp111,sizeof(char),8U);});case 2U: _LL5: _LL6:
 return({const char*_tmp112="cdecl";_tag_fat(_tmp112,sizeof(char),6U);});case 3U: _LL7: _LL8:
 return({const char*_tmp113="fastcall";_tag_fat(_tmp113,sizeof(char),9U);});case 4U: _LL9: _LLA:
 return({const char*_tmp114="noreturn";_tag_fat(_tmp114,sizeof(char),9U);});case 5U: _LLB: _LLC:
 return({const char*_tmp115="const";_tag_fat(_tmp115,sizeof(char),6U);});case 6U: _LLD: _tmp10C=((struct Cyc_Absyn_Aligned_att_Absyn_Attribute_struct*)_tmp101)->f1;_LLE: {struct Cyc_Absyn_Exp*e=_tmp10C;
# 1074
if(e == 0)return({const char*_tmp116="aligned";_tag_fat(_tmp116,sizeof(char),8U);});else{
# 1076
enum Cyc_Cyclone_C_Compilers _tmp117=Cyc_Cyclone_c_compiler;switch(_tmp117){case Cyc_Cyclone_Gcc_c: _LL40: _LL41:
 return(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp11A=({struct Cyc_String_pa_PrintArg_struct _tmp250;_tmp250.tag=0U,({struct _fat_ptr _tmp327=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_exp2string(e));_tmp250.f1=_tmp327;});_tmp250;});void*_tmp118[1U];_tmp118[0]=& _tmp11A;({struct _fat_ptr _tmp328=({const char*_tmp119="aligned(%s)";_tag_fat(_tmp119,sizeof(char),12U);});Cyc_aprintf(_tmp328,_tag_fat(_tmp118,sizeof(void*),1U));});});case Cyc_Cyclone_Vc_c: _LL42: _LL43:
 goto _LL45;default: _LL44: _LL45:
 return(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp11D=({struct Cyc_String_pa_PrintArg_struct _tmp251;_tmp251.tag=0U,({struct _fat_ptr _tmp329=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_exp2string(e));_tmp251.f1=_tmp329;});_tmp251;});void*_tmp11B[1U];_tmp11B[0]=& _tmp11D;({struct _fat_ptr _tmp32A=({const char*_tmp11C="align(%s)";_tag_fat(_tmp11C,sizeof(char),10U);});Cyc_aprintf(_tmp32A,_tag_fat(_tmp11B,sizeof(void*),1U));});});}_LL3F:;}}case 7U: _LLF: _LL10:
# 1081
 return({const char*_tmp11E="packed";_tag_fat(_tmp11E,sizeof(char),7U);});case 8U: _LL11: _tmp10B=((struct Cyc_Absyn_Section_att_Absyn_Attribute_struct*)_tmp101)->f1;_LL12: {struct _fat_ptr s=_tmp10B;
return(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp121=({struct Cyc_String_pa_PrintArg_struct _tmp252;_tmp252.tag=0U,_tmp252.f1=(struct _fat_ptr)((struct _fat_ptr)s);_tmp252;});void*_tmp11F[1U];_tmp11F[0]=& _tmp121;({struct _fat_ptr _tmp32B=({const char*_tmp120="section(\"%s\")";_tag_fat(_tmp120,sizeof(char),14U);});Cyc_aprintf(_tmp32B,_tag_fat(_tmp11F,sizeof(void*),1U));});});}case 9U: _LL13: _LL14:
 return({const char*_tmp122="nocommon";_tag_fat(_tmp122,sizeof(char),9U);});case 10U: _LL15: _LL16:
 return({const char*_tmp123="shared";_tag_fat(_tmp123,sizeof(char),7U);});case 11U: _LL17: _LL18:
 return({const char*_tmp124="unused";_tag_fat(_tmp124,sizeof(char),7U);});case 12U: _LL19: _LL1A:
 return({const char*_tmp125="weak";_tag_fat(_tmp125,sizeof(char),5U);});case 13U: _LL1B: _LL1C:
 return({const char*_tmp126="dllimport";_tag_fat(_tmp126,sizeof(char),10U);});case 14U: _LL1D: _LL1E:
 return({const char*_tmp127="dllexport";_tag_fat(_tmp127,sizeof(char),10U);});case 15U: _LL1F: _LL20:
 return({const char*_tmp128="no_instrument_function";_tag_fat(_tmp128,sizeof(char),23U);});case 16U: _LL21: _LL22:
 return({const char*_tmp129="constructor";_tag_fat(_tmp129,sizeof(char),12U);});case 17U: _LL23: _LL24:
 return({const char*_tmp12A="destructor";_tag_fat(_tmp12A,sizeof(char),11U);});case 18U: _LL25: _LL26:
 return({const char*_tmp12B="no_check_memory_usage";_tag_fat(_tmp12B,sizeof(char),22U);});case 19U: if(((struct Cyc_Absyn_Format_att_Absyn_Attribute_struct*)_tmp101)->f1 == Cyc_Absyn_Printf_ft){_LL27: _tmp109=((struct Cyc_Absyn_Format_att_Absyn_Attribute_struct*)_tmp101)->f2;_tmp10A=((struct Cyc_Absyn_Format_att_Absyn_Attribute_struct*)_tmp101)->f3;_LL28: {int n=_tmp109;int m=_tmp10A;
return(struct _fat_ptr)({struct Cyc_Int_pa_PrintArg_struct _tmp12E=({struct Cyc_Int_pa_PrintArg_struct _tmp254;_tmp254.tag=1U,_tmp254.f1=(unsigned)n;_tmp254;});struct Cyc_Int_pa_PrintArg_struct _tmp12F=({struct Cyc_Int_pa_PrintArg_struct _tmp253;_tmp253.tag=1U,_tmp253.f1=(unsigned)m;_tmp253;});void*_tmp12C[2U];_tmp12C[0]=& _tmp12E,_tmp12C[1]=& _tmp12F;({struct _fat_ptr _tmp32C=({const char*_tmp12D="format(printf,%u,%u)";_tag_fat(_tmp12D,sizeof(char),21U);});Cyc_aprintf(_tmp32C,_tag_fat(_tmp12C,sizeof(void*),2U));});});}}else{_LL29: _tmp107=((struct Cyc_Absyn_Format_att_Absyn_Attribute_struct*)_tmp101)->f2;_tmp108=((struct Cyc_Absyn_Format_att_Absyn_Attribute_struct*)_tmp101)->f3;_LL2A: {int n=_tmp107;int m=_tmp108;
# 1095
return(struct _fat_ptr)({struct Cyc_Int_pa_PrintArg_struct _tmp132=({struct Cyc_Int_pa_PrintArg_struct _tmp256;_tmp256.tag=1U,_tmp256.f1=(unsigned)n;_tmp256;});struct Cyc_Int_pa_PrintArg_struct _tmp133=({struct Cyc_Int_pa_PrintArg_struct _tmp255;_tmp255.tag=1U,_tmp255.f1=(unsigned)m;_tmp255;});void*_tmp130[2U];_tmp130[0]=& _tmp132,_tmp130[1]=& _tmp133;({struct _fat_ptr _tmp32D=({const char*_tmp131="format(scanf,%u,%u)";_tag_fat(_tmp131,sizeof(char),20U);});Cyc_aprintf(_tmp32D,_tag_fat(_tmp130,sizeof(void*),2U));});});}}case 20U: _LL2B: _tmp106=((struct Cyc_Absyn_Initializes_att_Absyn_Attribute_struct*)_tmp101)->f1;_LL2C: {int n=_tmp106;
return(struct _fat_ptr)({struct Cyc_Int_pa_PrintArg_struct _tmp136=({struct Cyc_Int_pa_PrintArg_struct _tmp257;_tmp257.tag=1U,_tmp257.f1=(unsigned long)n;_tmp257;});void*_tmp134[1U];_tmp134[0]=& _tmp136;({struct _fat_ptr _tmp32E=({const char*_tmp135="initializes(%d)";_tag_fat(_tmp135,sizeof(char),16U);});Cyc_aprintf(_tmp32E,_tag_fat(_tmp134,sizeof(void*),1U));});});}case 21U: _LL2D: _tmp105=((struct Cyc_Absyn_Noliveunique_att_Absyn_Attribute_struct*)_tmp101)->f1;_LL2E: {int n=_tmp105;
return(struct _fat_ptr)({struct Cyc_Int_pa_PrintArg_struct _tmp139=({struct Cyc_Int_pa_PrintArg_struct _tmp258;_tmp258.tag=1U,_tmp258.f1=(unsigned long)n;_tmp258;});void*_tmp137[1U];_tmp137[0]=& _tmp139;({struct _fat_ptr _tmp32F=({const char*_tmp138="noliveunique(%d)";_tag_fat(_tmp138,sizeof(char),17U);});Cyc_aprintf(_tmp32F,_tag_fat(_tmp137,sizeof(void*),1U));});});}case 22U: _LL2F: _tmp104=((struct Cyc_Absyn_Consume_att_Absyn_Attribute_struct*)_tmp101)->f1;_LL30: {int n=_tmp104;
return(struct _fat_ptr)({struct Cyc_Int_pa_PrintArg_struct _tmp13C=({struct Cyc_Int_pa_PrintArg_struct _tmp259;_tmp259.tag=1U,_tmp259.f1=(unsigned long)n;_tmp259;});void*_tmp13A[1U];_tmp13A[0]=& _tmp13C;({struct _fat_ptr _tmp330=({const char*_tmp13B="consume(%d)";_tag_fat(_tmp13B,sizeof(char),12U);});Cyc_aprintf(_tmp330,_tag_fat(_tmp13A,sizeof(void*),1U));});});}case 23U: _LL31: _LL32:
 return({const char*_tmp13D="pure";_tag_fat(_tmp13D,sizeof(char),5U);});case 26U: _LL33: _LL34:
 return({const char*_tmp13E="always_inline";_tag_fat(_tmp13E,sizeof(char),14U);});case 24U: _LL35: _tmp103=((struct Cyc_Absyn_Mode_att_Absyn_Attribute_struct*)_tmp101)->f1;_LL36: {struct _fat_ptr s=_tmp103;
return(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp141=({struct Cyc_String_pa_PrintArg_struct _tmp25A;_tmp25A.tag=0U,_tmp25A.f1=(struct _fat_ptr)((struct _fat_ptr)s);_tmp25A;});void*_tmp13F[1U];_tmp13F[0]=& _tmp141;({struct _fat_ptr _tmp331=({const char*_tmp140="__mode__(\"%s\")";_tag_fat(_tmp140,sizeof(char),15U);});Cyc_aprintf(_tmp331,_tag_fat(_tmp13F,sizeof(void*),1U));});});}case 25U: _LL37: _tmp102=((struct Cyc_Absyn_Alias_att_Absyn_Attribute_struct*)_tmp101)->f1;_LL38: {struct _fat_ptr s=_tmp102;
return(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp144=({struct Cyc_String_pa_PrintArg_struct _tmp25B;_tmp25B.tag=0U,_tmp25B.f1=(struct _fat_ptr)((struct _fat_ptr)s);_tmp25B;});void*_tmp142[1U];_tmp142[0]=& _tmp144;({struct _fat_ptr _tmp332=({const char*_tmp143="alias(\"%s\")";_tag_fat(_tmp143,sizeof(char),12U);});Cyc_aprintf(_tmp332,_tag_fat(_tmp142,sizeof(void*),1U));});});}case 27U: _LL39: _LL3A:
 return({const char*_tmp145="nothrow";_tag_fat(_tmp145,sizeof(char),8U);});case 28U: _LL3B: _LL3C:
 return({const char*_tmp146="nonnull";_tag_fat(_tmp146,sizeof(char),8U);});default: _LL3D: _LL3E:
 return({const char*_tmp147="deprecated";_tag_fat(_tmp147,sizeof(char),11U);});}_LL0:;}
# 957
static int Cyc_Absyn_attribute_case_number(void*att){
# 1110
void*_tmp148=att;switch(*((int*)_tmp148)){case 0U: _LL1: _LL2:
 return 0;case 1U: _LL3: _LL4:
 return 1;case 2U: _LL5: _LL6:
 return 2;case 3U: _LL7: _LL8:
 return 3;case 4U: _LL9: _LLA:
 return 4;case 5U: _LLB: _LLC:
 return 5;case 6U: _LLD: _LLE:
 return 6;case 7U: _LLF: _LL10:
 return 7;case 8U: _LL11: _LL12:
 return 8;case 9U: _LL13: _LL14:
 return 9;case 10U: _LL15: _LL16:
 return 10;case 11U: _LL17: _LL18:
 return 11;case 12U: _LL19: _LL1A:
 return 12;case 13U: _LL1B: _LL1C:
 return 13;case 14U: _LL1D: _LL1E:
 return 14;case 15U: _LL1F: _LL20:
 return 15;case 16U: _LL21: _LL22:
 return 16;case 17U: _LL23: _LL24:
 return 17;case 18U: _LL25: _LL26:
 return 18;case 19U: _LL27: _LL28:
 return 19;case 20U: _LL29: _LL2A:
 return 20;default: _LL2B: _LL2C:
 return 21;}_LL0:;}
# 957
int Cyc_Absyn_attribute_cmp(void*att1,void*att2){
# 1136
struct _tuple13 _stmttmpA=({struct _tuple13 _tmp25C;_tmp25C.f1=att1,_tmp25C.f2=att2;_tmp25C;});struct _tuple13 _tmp149=_stmttmpA;int _tmp14F;int _tmp14E;enum Cyc_Absyn_Format_Type _tmp14D;int _tmp14C;int _tmp14B;enum Cyc_Absyn_Format_Type _tmp14A;struct _fat_ptr _tmp151;struct _fat_ptr _tmp150;struct Cyc_Absyn_Exp*_tmp153;struct Cyc_Absyn_Exp*_tmp152;int _tmp155;int _tmp154;int _tmp157;int _tmp156;switch(*((int*)_tmp149.f1)){case 0U: if(((struct Cyc_Absyn_Regparm_att_Absyn_Attribute_struct*)_tmp149.f2)->tag == 0U){_LL1: _tmp156=((struct Cyc_Absyn_Regparm_att_Absyn_Attribute_struct*)_tmp149.f1)->f1;_tmp157=((struct Cyc_Absyn_Regparm_att_Absyn_Attribute_struct*)_tmp149.f2)->f1;_LL2: {int i1=_tmp156;int i2=_tmp157;
_tmp154=i1;_tmp155=i2;goto _LL4;}}else{goto _LLB;}case 20U: if(((struct Cyc_Absyn_Initializes_att_Absyn_Attribute_struct*)_tmp149.f2)->tag == 20U){_LL3: _tmp154=((struct Cyc_Absyn_Initializes_att_Absyn_Attribute_struct*)_tmp149.f1)->f1;_tmp155=((struct Cyc_Absyn_Initializes_att_Absyn_Attribute_struct*)_tmp149.f2)->f1;_LL4: {int i1=_tmp154;int i2=_tmp155;
# 1139
return Cyc_Core_intcmp(i1,i2);}}else{goto _LLB;}case 6U: if(((struct Cyc_Absyn_Aligned_att_Absyn_Attribute_struct*)_tmp149.f2)->tag == 6U){_LL5: _tmp152=((struct Cyc_Absyn_Aligned_att_Absyn_Attribute_struct*)_tmp149.f1)->f1;_tmp153=((struct Cyc_Absyn_Aligned_att_Absyn_Attribute_struct*)_tmp149.f2)->f1;_LL6: {struct Cyc_Absyn_Exp*e1=_tmp152;struct Cyc_Absyn_Exp*e2=_tmp153;
# 1141
if(e1 == e2)return 0;if(e1 == 0)
return - 1;
# 1141
if(e2 == 0)
# 1143
return 1;
# 1141
return
# 1144
 Cyc_Evexp_const_exp_cmp(e1,e2);}}else{goto _LLB;}case 8U: if(((struct Cyc_Absyn_Section_att_Absyn_Attribute_struct*)_tmp149.f2)->tag == 8U){_LL7: _tmp150=((struct Cyc_Absyn_Section_att_Absyn_Attribute_struct*)_tmp149.f1)->f1;_tmp151=((struct Cyc_Absyn_Section_att_Absyn_Attribute_struct*)_tmp149.f2)->f1;_LL8: {struct _fat_ptr s1=_tmp150;struct _fat_ptr s2=_tmp151;
return Cyc_strcmp((struct _fat_ptr)s1,(struct _fat_ptr)s2);}}else{goto _LLB;}case 19U: if(((struct Cyc_Absyn_Format_att_Absyn_Attribute_struct*)_tmp149.f2)->tag == 19U){_LL9: _tmp14A=((struct Cyc_Absyn_Format_att_Absyn_Attribute_struct*)_tmp149.f1)->f1;_tmp14B=((struct Cyc_Absyn_Format_att_Absyn_Attribute_struct*)_tmp149.f1)->f2;_tmp14C=((struct Cyc_Absyn_Format_att_Absyn_Attribute_struct*)_tmp149.f1)->f3;_tmp14D=((struct Cyc_Absyn_Format_att_Absyn_Attribute_struct*)_tmp149.f2)->f1;_tmp14E=((struct Cyc_Absyn_Format_att_Absyn_Attribute_struct*)_tmp149.f2)->f2;_tmp14F=((struct Cyc_Absyn_Format_att_Absyn_Attribute_struct*)_tmp149.f2)->f3;_LLA: {enum Cyc_Absyn_Format_Type ft1=_tmp14A;int i1=_tmp14B;int j1=_tmp14C;enum Cyc_Absyn_Format_Type ft2=_tmp14D;int i2=_tmp14E;int j2=_tmp14F;
# 1147
int ftc=Cyc_Core_intcmp((int)((unsigned)ft1),(int)((unsigned)ft2));
if(ftc != 0)return ftc;{int ic=
Cyc_Core_intcmp(i1,i2);
if(ic != 0)return ic;return
 Cyc_Core_intcmp(j1,j2);}}}else{goto _LLB;}default: _LLB: _LLC:
# 1153
 return({int _tmp333=Cyc_Absyn_attribute_case_number(att1);Cyc_Core_intcmp(_tmp333,Cyc_Absyn_attribute_case_number(att2));});}_LL0:;}
# 957
int Cyc_Absyn_equal_att(void*a1,void*a2){
# 1157
return Cyc_Absyn_attribute_cmp(a1,a2)== 0;}
# 957
int Cyc_Absyn_fntype_att(void*a){
# 1162
void*_tmp158=a;int _tmp159;switch(*((int*)_tmp158)){case 0U: _LL1: _tmp159=((struct Cyc_Absyn_Regparm_att_Absyn_Attribute_struct*)_tmp158)->f1;_LL2: {int i=_tmp159;
goto _LL4;}case 3U: _LL3: _LL4:
 goto _LL6;case 1U: _LL5: _LL6:
 goto _LL8;case 2U: _LL7: _LL8:
 goto _LLA;case 4U: _LL9: _LLA:
 goto _LLC;case 23U: _LLB: _LLC:
 goto _LLE;case 19U: _LLD: _LLE:
 goto _LL10;case 5U: _LLF: _LL10:
 goto _LL12;case 21U: _LL11: _LL12:
 goto _LL14;case 20U: _LL13: _LL14:
 goto _LL16;case 22U: _LL15: _LL16:
 return 1;default: _LL17: _LL18:
 return 0;}_LL0:;}static char _tmp160[3U]="f0";
# 957
struct _fat_ptr*Cyc_Absyn_fieldname(int i){
# 1180
static struct _fat_ptr f0={_tmp160,_tmp160,_tmp160 + 3U};
static struct _fat_ptr*field_names_v[1U]={& f0};
static struct _fat_ptr field_names={(void*)((struct _fat_ptr**)field_names_v),(void*)((struct _fat_ptr**)field_names_v),(void*)((struct _fat_ptr**)field_names_v + 1U)};
unsigned fsz=_get_fat_size(field_names,sizeof(struct _fat_ptr*));
if((unsigned)i >= fsz)
field_names=({unsigned _tmp15F=(unsigned)(i + 1);struct _fat_ptr**_tmp15E=_cycalloc(_check_times(_tmp15F,sizeof(struct _fat_ptr*)));({{unsigned _tmp25D=(unsigned)(i + 1);unsigned j;for(j=0;j < _tmp25D;++ j){
# 1187
j < fsz?_tmp15E[j]=*((struct _fat_ptr**)_check_fat_subscript(field_names,sizeof(struct _fat_ptr*),(int)j)):({struct _fat_ptr*_tmp336=({struct _fat_ptr*_tmp15D=_cycalloc(sizeof(*_tmp15D));({struct _fat_ptr _tmp335=(struct _fat_ptr)({struct Cyc_Int_pa_PrintArg_struct _tmp15C=({struct Cyc_Int_pa_PrintArg_struct _tmp25E;_tmp25E.tag=1U,_tmp25E.f1=(unsigned long)((int)j);_tmp25E;});void*_tmp15A[1U];_tmp15A[0]=& _tmp15C;({struct _fat_ptr _tmp334=({const char*_tmp15B="f%d";_tag_fat(_tmp15B,sizeof(char),4U);});Cyc_aprintf(_tmp334,_tag_fat(_tmp15A,sizeof(void*),1U));});});*_tmp15D=_tmp335;});_tmp15D;});_tmp15E[j]=_tmp336;});}}0;});_tag_fat(_tmp15E,sizeof(struct _fat_ptr*),_tmp15F);});
# 1184
return*((struct _fat_ptr**)_check_fat_subscript(field_names,sizeof(struct _fat_ptr*),i));}struct _tuple16{enum Cyc_Absyn_AggrKind f1;struct _tuple0*f2;};
# 957
struct _tuple16 Cyc_Absyn_aggr_kinded_name(union Cyc_Absyn_AggrInfo info){
# 1192
union Cyc_Absyn_AggrInfo _tmp161=info;struct _tuple0*_tmp163;enum Cyc_Absyn_AggrKind _tmp162;struct _tuple0*_tmp165;enum Cyc_Absyn_AggrKind _tmp164;if((_tmp161.UnknownAggr).tag == 1){_LL1: _tmp164=((_tmp161.UnknownAggr).val).f1;_tmp165=((_tmp161.UnknownAggr).val).f2;_LL2: {enum Cyc_Absyn_AggrKind ak=_tmp164;struct _tuple0*n=_tmp165;
return({struct _tuple16 _tmp25F;_tmp25F.f1=ak,_tmp25F.f2=n;_tmp25F;});}}else{_LL3: _tmp162=(*(_tmp161.KnownAggr).val)->kind;_tmp163=(*(_tmp161.KnownAggr).val)->name;_LL4: {enum Cyc_Absyn_AggrKind k=_tmp162;struct _tuple0*n=_tmp163;
return({struct _tuple16 _tmp260;_tmp260.f1=k,_tmp260.f2=n;_tmp260;});}}_LL0:;}
# 957
struct Cyc_Absyn_Aggrdecl*Cyc_Absyn_get_known_aggrdecl(union Cyc_Absyn_AggrInfo info){
# 1198
union Cyc_Absyn_AggrInfo _tmp166=info;struct Cyc_Absyn_Aggrdecl*_tmp167;if((_tmp166.UnknownAggr).tag == 1){_LL1: _LL2:
({void*_tmp168=0U;({int(*_tmp338)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp16A)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmp16A;});struct _fat_ptr _tmp337=({const char*_tmp169="unchecked aggrdecl";_tag_fat(_tmp169,sizeof(char),19U);});_tmp338(_tmp337,_tag_fat(_tmp168,sizeof(void*),0U));});});}else{_LL3: _tmp167=*(_tmp166.KnownAggr).val;_LL4: {struct Cyc_Absyn_Aggrdecl*ad=_tmp167;
return ad;}}_LL0:;}
# 957
int Cyc_Absyn_is_nontagged_nonrequire_union_type(void*t){
# 1204
void*_stmttmpB=Cyc_Tcutil_compress(t);void*_tmp16B=_stmttmpB;union Cyc_Absyn_AggrInfo _tmp16C;struct Cyc_List_List*_tmp16D;switch(*((int*)_tmp16B)){case 7U: if(((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmp16B)->f1 == Cyc_Absyn_UnionA){_LL1: _tmp16D=((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmp16B)->f2;_LL2: {struct Cyc_List_List*fs=_tmp16D;
# 1206
if(fs == 0)return 1;return((struct Cyc_Absyn_Aggrfield*)fs->hd)->requires_clause == 0;}}else{goto _LL5;}case 0U: if(((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp16B)->f1)->tag == 20U){_LL3: _tmp16C=((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp16B)->f1)->f1;_LL4: {union Cyc_Absyn_AggrInfo info=_tmp16C;
# 1209
union Cyc_Absyn_AggrInfo _tmp16E=info;int _tmp170;enum Cyc_Absyn_AggrKind _tmp16F;enum Cyc_Absyn_AggrKind _tmp171;struct Cyc_Absyn_Aggrdecl*_tmp172;if((_tmp16E.KnownAggr).tag == 2){_LL8: _tmp172=*(_tmp16E.KnownAggr).val;_LL9: {struct Cyc_Absyn_Aggrdecl*ad=_tmp172;
# 1211
if((int)ad->kind != (int)Cyc_Absyn_UnionA)return 0;{struct Cyc_Absyn_AggrdeclImpl*impl=ad->impl;
# 1213
if(((struct Cyc_Absyn_AggrdeclImpl*)_check_null(impl))->tagged)return 0;{struct Cyc_List_List*fields=impl->fields;
# 1215
if(fields == 0)return 1;return((struct Cyc_Absyn_Aggrfield*)fields->hd)->requires_clause == 0;}}}}else{if(((_tmp16E.UnknownAggr).val).f3 == 0){_LLA: _tmp171=((_tmp16E.UnknownAggr).val).f1;_LLB: {enum Cyc_Absyn_AggrKind k=_tmp171;
# 1217
return(int)k == (int)Cyc_Absyn_UnionA;}}else{_LLC: _tmp16F=((_tmp16E.UnknownAggr).val).f1;_tmp170=(int)(((_tmp16E.UnknownAggr).val).f3)->v;_LLD: {enum Cyc_Absyn_AggrKind k=_tmp16F;int b=_tmp170;
return(int)k == (int)1U && !b;}}}_LL7:;}}else{goto _LL5;}default: _LL5: _LL6:
# 1220
 return 0;}_LL0:;}
# 957
int Cyc_Absyn_is_require_union_type(void*t){
# 1224
void*_stmttmpC=Cyc_Tcutil_compress(t);void*_tmp173=_stmttmpC;union Cyc_Absyn_AggrInfo _tmp174;struct Cyc_List_List*_tmp175;switch(*((int*)_tmp173)){case 7U: if(((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmp173)->f1 == Cyc_Absyn_UnionA){_LL1: _tmp175=((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmp173)->f2;_LL2: {struct Cyc_List_List*fs=_tmp175;
# 1226
if(fs == 0)return 0;return((struct Cyc_Absyn_Aggrfield*)fs->hd)->requires_clause != 0;}}else{goto _LL5;}case 0U: if(((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp173)->f1)->tag == 20U){_LL3: _tmp174=((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp173)->f1)->f1;_LL4: {union Cyc_Absyn_AggrInfo info=_tmp174;
# 1229
union Cyc_Absyn_AggrInfo _tmp176=info;int _tmp178;enum Cyc_Absyn_AggrKind _tmp177;enum Cyc_Absyn_AggrKind _tmp179;struct Cyc_Absyn_Aggrdecl*_tmp17A;if((_tmp176.KnownAggr).tag == 2){_LL8: _tmp17A=*(_tmp176.KnownAggr).val;_LL9: {struct Cyc_Absyn_Aggrdecl*ad=_tmp17A;
# 1231
if((int)ad->kind != (int)Cyc_Absyn_UnionA)return 0;{struct Cyc_Absyn_AggrdeclImpl*impl=ad->impl;
# 1233
if(((struct Cyc_Absyn_AggrdeclImpl*)_check_null(impl))->tagged)return 0;{struct Cyc_List_List*fields=impl->fields;
# 1235
if(fields == 0)return 0;return((struct Cyc_Absyn_Aggrfield*)fields->hd)->requires_clause != 0;}}}}else{if(((_tmp176.UnknownAggr).val).f3 == 0){_LLA: _tmp179=((_tmp176.UnknownAggr).val).f1;_LLB: {enum Cyc_Absyn_AggrKind k=_tmp179;
# 1237
return 0;}}else{_LLC: _tmp177=((_tmp176.UnknownAggr).val).f1;_tmp178=(int)(((_tmp176.UnknownAggr).val).f3)->v;_LLD: {enum Cyc_Absyn_AggrKind k=_tmp177;int b=_tmp178;
return 0;}}}_LL7:;}}else{goto _LL5;}default: _LL5: _LL6:
# 1240
 return 0;}_LL0:;}
# 957
struct _tuple0*Cyc_Absyn_binding2qvar(void*b){
# 1245
void*_tmp17B=b;struct Cyc_Absyn_Fndecl*_tmp17C;struct Cyc_Absyn_Vardecl*_tmp17D;struct Cyc_Absyn_Vardecl*_tmp17E;struct Cyc_Absyn_Vardecl*_tmp17F;struct Cyc_Absyn_Vardecl*_tmp180;struct _tuple0*_tmp181;switch(*((int*)_tmp17B)){case 0U: _LL1: _tmp181=((struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct*)_tmp17B)->f1;_LL2: {struct _tuple0*qv=_tmp181;
return qv;}case 1U: _LL3: _tmp180=((struct Cyc_Absyn_Global_b_Absyn_Binding_struct*)_tmp17B)->f1;_LL4: {struct Cyc_Absyn_Vardecl*vd=_tmp180;
_tmp17F=vd;goto _LL6;}case 3U: _LL5: _tmp17F=((struct Cyc_Absyn_Param_b_Absyn_Binding_struct*)_tmp17B)->f1;_LL6: {struct Cyc_Absyn_Vardecl*vd=_tmp17F;
_tmp17E=vd;goto _LL8;}case 4U: _LL7: _tmp17E=((struct Cyc_Absyn_Local_b_Absyn_Binding_struct*)_tmp17B)->f1;_LL8: {struct Cyc_Absyn_Vardecl*vd=_tmp17E;
_tmp17D=vd;goto _LLA;}case 5U: _LL9: _tmp17D=((struct Cyc_Absyn_Pat_b_Absyn_Binding_struct*)_tmp17B)->f1;_LLA: {struct Cyc_Absyn_Vardecl*vd=_tmp17D;
return vd->name;}default: _LLB: _tmp17C=((struct Cyc_Absyn_Funname_b_Absyn_Binding_struct*)_tmp17B)->f1;_LLC: {struct Cyc_Absyn_Fndecl*fd=_tmp17C;
return fd->name;}}_LL0:;}
# 957
struct _fat_ptr*Cyc_Absyn_designatorlist_to_fieldname(struct Cyc_List_List*ds){
# 1256
if(ds == 0 || ds->tl != 0)
({void*_tmp182=0U;({int(*_tmp33A)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp184)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmp184;});struct _fat_ptr _tmp339=({const char*_tmp183="designator list not of length 1";_tag_fat(_tmp183,sizeof(char),32U);});_tmp33A(_tmp339,_tag_fat(_tmp182,sizeof(void*),0U));});});{
# 1256
void*_stmttmpD=(void*)ds->hd;void*_tmp185=_stmttmpD;struct _fat_ptr*_tmp186;if(((struct Cyc_Absyn_FieldName_Absyn_Designator_struct*)_tmp185)->tag == 1U){_LL1: _tmp186=((struct Cyc_Absyn_FieldName_Absyn_Designator_struct*)_tmp185)->f1;_LL2: {struct _fat_ptr*f=_tmp186;
# 1259
return f;}}else{_LL3: _LL4:
({void*_tmp187=0U;({int(*_tmp33C)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp189)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmp189;});struct _fat_ptr _tmp33B=({const char*_tmp188="array designator in struct";_tag_fat(_tmp188,sizeof(char),27U);});_tmp33C(_tmp33B,_tag_fat(_tmp187,sizeof(void*),0U));});});}_LL0:;}}
# 957
int Cyc_Absyn_type2bool(int def,void*t){
# 1265
void*_stmttmpE=Cyc_Tcutil_compress(t);void*_tmp18A=_stmttmpE;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp18A)->tag == 0U)switch(*((int*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp18A)->f1)){case 11U: _LL1: _LL2:
 return 1;case 12U: _LL3: _LL4:
 return 0;default: goto _LL5;}else{_LL5: _LL6:
 return def;}_LL0:;}
# 957
void Cyc_Absyn_visit_stmt(int(*)(void*,struct Cyc_Absyn_Exp*),int(*)(void*,struct Cyc_Absyn_Stmt*),void*,struct Cyc_Absyn_Stmt*);struct _tuple17{struct _fat_ptr f1;struct Cyc_Absyn_Exp*f2;};
# 1275
void Cyc_Absyn_visit_exp(int(*f1)(void*,struct Cyc_Absyn_Exp*),int(*f2)(void*,struct Cyc_Absyn_Stmt*),void*env,struct Cyc_Absyn_Exp*e){
if(!f1(env,e))
return;{
# 1276
void*_stmttmpF=e->r;void*_tmp18B=_stmttmpF;struct Cyc_List_List*_tmp18D;struct Cyc_List_List*_tmp18C;struct Cyc_Absyn_Stmt*_tmp18E;struct Cyc_Absyn_Exp*_tmp190;struct Cyc_Absyn_Exp*_tmp18F;struct Cyc_Absyn_Exp*_tmp192;struct Cyc_Absyn_Exp*_tmp191;struct Cyc_Absyn_Exp*_tmp194;struct Cyc_Absyn_Exp*_tmp193;struct Cyc_List_List*_tmp195;struct Cyc_List_List*_tmp196;struct Cyc_List_List*_tmp197;struct Cyc_List_List*_tmp198;struct Cyc_List_List*_tmp199;struct Cyc_List_List*_tmp19A;struct Cyc_List_List*_tmp19B;struct Cyc_List_List*_tmp19C;struct Cyc_List_List*_tmp19E;struct Cyc_Absyn_Exp*_tmp19D;struct Cyc_Absyn_Exp*_tmp1A1;struct Cyc_Absyn_Exp*_tmp1A0;struct Cyc_Absyn_Exp*_tmp19F;struct Cyc_Absyn_Exp*_tmp1A3;struct Cyc_Absyn_Exp*_tmp1A2;struct Cyc_Absyn_Exp*_tmp1A5;struct Cyc_Absyn_Exp*_tmp1A4;struct Cyc_Absyn_Exp*_tmp1A7;struct Cyc_Absyn_Exp*_tmp1A6;struct Cyc_Absyn_Exp*_tmp1A9;struct Cyc_Absyn_Exp*_tmp1A8;struct Cyc_Absyn_Exp*_tmp1AB;struct Cyc_Absyn_Exp*_tmp1AA;struct Cyc_Absyn_Exp*_tmp1AD;struct Cyc_Absyn_Exp*_tmp1AC;struct Cyc_Absyn_Exp*_tmp1AF;struct Cyc_Absyn_Exp*_tmp1AE;struct Cyc_Absyn_Exp*_tmp1B0;struct Cyc_Absyn_Exp*_tmp1B1;struct Cyc_Absyn_Exp*_tmp1B2;struct Cyc_Absyn_Exp*_tmp1B3;struct Cyc_Absyn_Exp*_tmp1B4;struct Cyc_Absyn_Exp*_tmp1B5;struct Cyc_Absyn_Exp*_tmp1B6;struct Cyc_Absyn_Exp*_tmp1B7;struct Cyc_Absyn_Exp*_tmp1B8;struct Cyc_Absyn_Exp*_tmp1B9;struct Cyc_Absyn_Exp*_tmp1BA;struct Cyc_Absyn_Exp*_tmp1BB;struct Cyc_Absyn_Exp*_tmp1BC;switch(*((int*)_tmp18B)){case 0U: _LL1: _LL2:
# 1279
 goto _LL4;case 1U: _LL3: _LL4:
 goto _LL6;case 2U: _LL5: _LL6:
 goto _LL8;case 32U: _LL7: _LL8:
 goto _LLA;case 33U: _LL9: _LLA:
 goto _LLC;case 40U: _LLB: _LLC:
 goto _LLE;case 19U: _LLD: _LLE:
 goto _LL10;case 17U: _LLF: _LL10:
 goto _LL0;case 42U: _LL11: _tmp1BC=((struct Cyc_Absyn_Extension_e_Absyn_Raw_exp_struct*)_tmp18B)->f1;_LL12: {struct Cyc_Absyn_Exp*e1=_tmp1BC;
# 1288
_tmp1BB=e1;goto _LL14;}case 5U: _LL13: _tmp1BB=((struct Cyc_Absyn_Increment_e_Absyn_Raw_exp_struct*)_tmp18B)->f1;_LL14: {struct Cyc_Absyn_Exp*e1=_tmp1BB;
# 1291
_tmp1BA=e1;goto _LL16;}case 11U: _LL15: _tmp1BA=((struct Cyc_Absyn_Throw_e_Absyn_Raw_exp_struct*)_tmp18B)->f1;_LL16: {struct Cyc_Absyn_Exp*e1=_tmp1BA;
_tmp1B9=e1;goto _LL18;}case 12U: _LL17: _tmp1B9=((struct Cyc_Absyn_NoInstantiate_e_Absyn_Raw_exp_struct*)_tmp18B)->f1;_LL18: {struct Cyc_Absyn_Exp*e1=_tmp1B9;
_tmp1B8=e1;goto _LL1A;}case 13U: _LL19: _tmp1B8=((struct Cyc_Absyn_Instantiate_e_Absyn_Raw_exp_struct*)_tmp18B)->f1;_LL1A: {struct Cyc_Absyn_Exp*e1=_tmp1B8;
_tmp1B7=e1;goto _LL1C;}case 14U: _LL1B: _tmp1B7=((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp18B)->f2;_LL1C: {struct Cyc_Absyn_Exp*e1=_tmp1B7;
_tmp1B6=e1;goto _LL1E;}case 15U: _LL1D: _tmp1B6=((struct Cyc_Absyn_Address_e_Absyn_Raw_exp_struct*)_tmp18B)->f1;_LL1E: {struct Cyc_Absyn_Exp*e1=_tmp1B6;
_tmp1B5=e1;goto _LL20;}case 20U: _LL1F: _tmp1B5=((struct Cyc_Absyn_Deref_e_Absyn_Raw_exp_struct*)_tmp18B)->f1;_LL20: {struct Cyc_Absyn_Exp*e1=_tmp1B5;
_tmp1B4=e1;goto _LL22;}case 21U: _LL21: _tmp1B4=((struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct*)_tmp18B)->f1;_LL22: {struct Cyc_Absyn_Exp*e1=_tmp1B4;
_tmp1B3=e1;goto _LL24;}case 28U: _LL23: _tmp1B3=((struct Cyc_Absyn_ComprehensionNoinit_e_Absyn_Raw_exp_struct*)_tmp18B)->f1;_LL24: {struct Cyc_Absyn_Exp*e1=_tmp1B3;
_tmp1B2=e1;goto _LL26;}case 39U: _LL25: _tmp1B2=((struct Cyc_Absyn_Tagcheck_e_Absyn_Raw_exp_struct*)_tmp18B)->f1;_LL26: {struct Cyc_Absyn_Exp*e1=_tmp1B2;
_tmp1B1=e1;goto _LL28;}case 18U: _LL27: _tmp1B1=((struct Cyc_Absyn_Sizeofexp_e_Absyn_Raw_exp_struct*)_tmp18B)->f1;_LL28: {struct Cyc_Absyn_Exp*e1=_tmp1B1;
_tmp1B0=e1;goto _LL2A;}case 22U: _LL29: _tmp1B0=((struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct*)_tmp18B)->f1;_LL2A: {struct Cyc_Absyn_Exp*e1=_tmp1B0;
Cyc_Absyn_visit_exp(f1,f2,env,e1);return;}case 4U: _LL2B: _tmp1AE=((struct Cyc_Absyn_AssignOp_e_Absyn_Raw_exp_struct*)_tmp18B)->f1;_tmp1AF=((struct Cyc_Absyn_AssignOp_e_Absyn_Raw_exp_struct*)_tmp18B)->f3;_LL2C: {struct Cyc_Absyn_Exp*e1=_tmp1AE;struct Cyc_Absyn_Exp*e2=_tmp1AF;
# 1304
_tmp1AC=e1;_tmp1AD=e2;goto _LL2E;}case 7U: _LL2D: _tmp1AC=((struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct*)_tmp18B)->f1;_tmp1AD=((struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct*)_tmp18B)->f2;_LL2E: {struct Cyc_Absyn_Exp*e1=_tmp1AC;struct Cyc_Absyn_Exp*e2=_tmp1AD;
_tmp1AA=e1;_tmp1AB=e2;goto _LL30;}case 8U: _LL2F: _tmp1AA=((struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct*)_tmp18B)->f1;_tmp1AB=((struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct*)_tmp18B)->f2;_LL30: {struct Cyc_Absyn_Exp*e1=_tmp1AA;struct Cyc_Absyn_Exp*e2=_tmp1AB;
_tmp1A8=e1;_tmp1A9=e2;goto _LL32;}case 9U: _LL31: _tmp1A8=((struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct*)_tmp18B)->f1;_tmp1A9=((struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct*)_tmp18B)->f2;_LL32: {struct Cyc_Absyn_Exp*e1=_tmp1A8;struct Cyc_Absyn_Exp*e2=_tmp1A9;
_tmp1A6=e1;_tmp1A7=e2;goto _LL34;}case 23U: _LL33: _tmp1A6=((struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct*)_tmp18B)->f1;_tmp1A7=((struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct*)_tmp18B)->f2;_LL34: {struct Cyc_Absyn_Exp*e1=_tmp1A6;struct Cyc_Absyn_Exp*e2=_tmp1A7;
_tmp1A4=e1;_tmp1A5=e2;goto _LL36;}case 36U: _LL35: _tmp1A4=((struct Cyc_Absyn_Swap_e_Absyn_Raw_exp_struct*)_tmp18B)->f1;_tmp1A5=((struct Cyc_Absyn_Swap_e_Absyn_Raw_exp_struct*)_tmp18B)->f2;_LL36: {struct Cyc_Absyn_Exp*e1=_tmp1A4;struct Cyc_Absyn_Exp*e2=_tmp1A5;
_tmp1A2=e1;_tmp1A3=e2;goto _LL38;}case 27U: _LL37: _tmp1A2=((struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct*)_tmp18B)->f2;_tmp1A3=((struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct*)_tmp18B)->f3;_LL38: {struct Cyc_Absyn_Exp*e1=_tmp1A2;struct Cyc_Absyn_Exp*e2=_tmp1A3;
# 1311
Cyc_Absyn_visit_exp(f1,f2,env,e1);Cyc_Absyn_visit_exp(f1,f2,env,e2);return;}case 6U: _LL39: _tmp19F=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmp18B)->f1;_tmp1A0=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmp18B)->f2;_tmp1A1=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmp18B)->f3;_LL3A: {struct Cyc_Absyn_Exp*e1=_tmp19F;struct Cyc_Absyn_Exp*e2=_tmp1A0;struct Cyc_Absyn_Exp*e3=_tmp1A1;
# 1314
Cyc_Absyn_visit_exp(f1,f2,env,e1);Cyc_Absyn_visit_exp(f1,f2,env,e2);Cyc_Absyn_visit_exp(f1,f2,env,e3);
return;}case 10U: _LL3B: _tmp19D=((struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct*)_tmp18B)->f1;_tmp19E=((struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct*)_tmp18B)->f2;_LL3C: {struct Cyc_Absyn_Exp*e1=_tmp19D;struct Cyc_List_List*lexp=_tmp19E;
# 1318
for(0;lexp != 0;lexp=lexp->tl){
Cyc_Absyn_visit_exp(f1,f2,env,(struct Cyc_Absyn_Exp*)lexp->hd);}
Cyc_Absyn_visit_exp(f1,f2,env,e1);
goto _LL0;}case 24U: _LL3D: _tmp19C=((struct Cyc_Absyn_Tuple_e_Absyn_Raw_exp_struct*)_tmp18B)->f1;_LL3E: {struct Cyc_List_List*lexp=_tmp19C;
# 1324
for(0;lexp != 0;lexp=lexp->tl){
Cyc_Absyn_visit_exp(f1,f2,env,(struct Cyc_Absyn_Exp*)lexp->hd);}
goto _LL0;}case 26U: _LL3F: _tmp19B=((struct Cyc_Absyn_Array_e_Absyn_Raw_exp_struct*)_tmp18B)->f1;_LL40: {struct Cyc_List_List*ldt=_tmp19B;
_tmp19A=ldt;goto _LL42;}case 25U: _LL41: _tmp19A=((struct Cyc_Absyn_CompoundLit_e_Absyn_Raw_exp_struct*)_tmp18B)->f2;_LL42: {struct Cyc_List_List*ldt=_tmp19A;
_tmp199=ldt;goto _LL44;}case 37U: _LL43: _tmp199=((struct Cyc_Absyn_UnresolvedMem_e_Absyn_Raw_exp_struct*)_tmp18B)->f2;_LL44: {struct Cyc_List_List*ldt=_tmp199;
_tmp198=ldt;goto _LL46;}case 29U: _LL45: _tmp198=((struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct*)_tmp18B)->f3;_LL46: {struct Cyc_List_List*ldt=_tmp198;
_tmp197=ldt;goto _LL48;}case 30U: _LL47: _tmp197=((struct Cyc_Absyn_AnonStruct_e_Absyn_Raw_exp_struct*)_tmp18B)->f2;_LL48: {struct Cyc_List_List*ldt=_tmp197;
# 1332
for(0;ldt != 0;ldt=ldt->tl){
Cyc_Absyn_visit_exp(f1,f2,env,(*((struct _tuple12*)ldt->hd)).f2);}
goto _LL0;}case 3U: _LL49: _tmp196=((struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct*)_tmp18B)->f2;_LL4A: {struct Cyc_List_List*lexp=_tmp196;
# 1336
_tmp195=lexp;goto _LL4C;}case 31U: _LL4B: _tmp195=((struct Cyc_Absyn_Datatype_e_Absyn_Raw_exp_struct*)_tmp18B)->f1;_LL4C: {struct Cyc_List_List*lexp=_tmp195;
# 1338
for(0;lexp != 0;lexp=lexp->tl){
Cyc_Absyn_visit_exp(f1,f2,env,(struct Cyc_Absyn_Exp*)lexp->hd);}
goto _LL0;}case 34U: _LL4D: _tmp193=((struct Cyc_Absyn_Spawn_e_Absyn_Raw_exp_struct*)_tmp18B)->f1;_tmp194=((struct Cyc_Absyn_Spawn_e_Absyn_Raw_exp_struct*)_tmp18B)->f2;_LL4E: {struct Cyc_Absyn_Exp*e1=_tmp193;struct Cyc_Absyn_Exp*e2=_tmp194;
# 1343
Cyc_Absyn_visit_exp(f1,f2,env,e1);
Cyc_Absyn_visit_exp(f1,f2,env,e2);
goto _LL0;}case 35U: _LL4F: _tmp191=(((struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct*)_tmp18B)->f1).rgn;_tmp192=(((struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct*)_tmp18B)->f1).num_elts;_LL50: {struct Cyc_Absyn_Exp*e1o=_tmp191;struct Cyc_Absyn_Exp*e2=_tmp192;
_tmp18F=e1o;_tmp190=e2;goto _LL52;}case 16U: _LL51: _tmp18F=((struct Cyc_Absyn_New_e_Absyn_Raw_exp_struct*)_tmp18B)->f1;_tmp190=((struct Cyc_Absyn_New_e_Absyn_Raw_exp_struct*)_tmp18B)->f2;_LL52: {struct Cyc_Absyn_Exp*e1=_tmp18F;struct Cyc_Absyn_Exp*e2=_tmp190;
# 1350
if(e1 != 0)Cyc_Absyn_visit_exp(f1,f2,env,e1);Cyc_Absyn_visit_exp(f1,f2,env,e2);
# 1353
goto _LL0;}case 38U: _LL53: _tmp18E=((struct Cyc_Absyn_StmtExp_e_Absyn_Raw_exp_struct*)_tmp18B)->f1;_LL54: {struct Cyc_Absyn_Stmt*s=_tmp18E;
# 1355
Cyc_Absyn_visit_stmt(f1,f2,env,s);goto _LL0;}default: _LL55: _tmp18C=((struct Cyc_Absyn_Asm_e_Absyn_Raw_exp_struct*)_tmp18B)->f3;_tmp18D=((struct Cyc_Absyn_Asm_e_Absyn_Raw_exp_struct*)_tmp18B)->f4;_LL56: {struct Cyc_List_List*sl1=_tmp18C;struct Cyc_List_List*sl2=_tmp18D;
# 1358
for(0;sl1 != 0;sl1=sl1->tl){
Cyc_Absyn_visit_exp(f1,f2,env,(*((struct _tuple17*)sl1->hd)).f2);}
for(0;sl2 != 0;sl2=sl2->tl){
Cyc_Absyn_visit_exp(f1,f2,env,(*((struct _tuple17*)sl2->hd)).f2);}
goto _LL0;}}_LL0:;}}
# 1275
static void Cyc_Absyn_visit_scs(int(*f1)(void*,struct Cyc_Absyn_Exp*),int(*f2)(void*,struct Cyc_Absyn_Stmt*),void*env,struct Cyc_List_List*scs){
# 1367
for(0;scs != 0;scs=scs->tl){
if(((struct Cyc_Absyn_Switch_clause*)scs->hd)->where_clause != 0)
Cyc_Absyn_visit_exp(f1,f2,env,(struct Cyc_Absyn_Exp*)_check_null(((struct Cyc_Absyn_Switch_clause*)scs->hd)->where_clause));
# 1368
Cyc_Absyn_visit_stmt(f1,f2,env,((struct Cyc_Absyn_Switch_clause*)scs->hd)->body);}}
# 1275
void Cyc_Absyn_visit_stmt(int(*f1)(void*,struct Cyc_Absyn_Exp*),int(*f2)(void*,struct Cyc_Absyn_Stmt*),void*env,struct Cyc_Absyn_Stmt*s){
# 1374
if(!f2(env,s))
return;{
# 1374
void*_stmttmp10=s->r;void*_tmp1BD=_stmttmp10;struct Cyc_List_List*_tmp1BF;struct Cyc_Absyn_Stmt*_tmp1BE;struct Cyc_List_List*_tmp1C1;struct Cyc_Absyn_Exp*_tmp1C0;struct Cyc_Absyn_Stmt*_tmp1C2;struct Cyc_Absyn_Stmt*_tmp1C4;struct Cyc_Absyn_Decl*_tmp1C3;struct Cyc_List_List*_tmp1C5;struct Cyc_Absyn_Stmt*_tmp1C9;struct Cyc_Absyn_Exp*_tmp1C8;struct Cyc_Absyn_Exp*_tmp1C7;struct Cyc_Absyn_Exp*_tmp1C6;struct Cyc_Absyn_Exp*_tmp1CB;struct Cyc_Absyn_Stmt*_tmp1CA;struct Cyc_Absyn_Stmt*_tmp1CD;struct Cyc_Absyn_Exp*_tmp1CC;struct Cyc_Absyn_Stmt*_tmp1CF;struct Cyc_Absyn_Stmt*_tmp1CE;struct Cyc_Absyn_Stmt*_tmp1D2;struct Cyc_Absyn_Stmt*_tmp1D1;struct Cyc_Absyn_Exp*_tmp1D0;struct Cyc_Absyn_Exp*_tmp1D3;struct Cyc_Absyn_Exp*_tmp1D4;switch(*((int*)_tmp1BD)){case 0U: _LL1: _LL2:
# 1377
 goto _LL4;case 6U: _LL3: _LL4:
 goto _LL6;case 7U: _LL5: _LL6:
 goto _LL8;case 8U: _LL7: _LL8:
 goto _LLA;case 3U: if(((struct Cyc_Absyn_Return_s_Absyn_Raw_stmt_struct*)_tmp1BD)->f1 == 0){_LL9: _LLA:
 goto _LL0;}else{_LLB: _tmp1D4=((struct Cyc_Absyn_Return_s_Absyn_Raw_stmt_struct*)_tmp1BD)->f1;_LLC: {struct Cyc_Absyn_Exp*e=_tmp1D4;
_tmp1D3=(struct Cyc_Absyn_Exp*)_check_null(e);goto _LLE;}}case 1U: _LLD: _tmp1D3=((struct Cyc_Absyn_Exp_s_Absyn_Raw_stmt_struct*)_tmp1BD)->f1;_LLE: {struct Cyc_Absyn_Exp*e=_tmp1D3;
Cyc_Absyn_visit_exp(f1,f2,env,e);goto _LL0;}case 4U: _LLF: _tmp1D0=((struct Cyc_Absyn_IfThenElse_s_Absyn_Raw_stmt_struct*)_tmp1BD)->f1;_tmp1D1=((struct Cyc_Absyn_IfThenElse_s_Absyn_Raw_stmt_struct*)_tmp1BD)->f2;_tmp1D2=((struct Cyc_Absyn_IfThenElse_s_Absyn_Raw_stmt_struct*)_tmp1BD)->f3;_LL10: {struct Cyc_Absyn_Exp*e1=_tmp1D0;struct Cyc_Absyn_Stmt*s1=_tmp1D1;struct Cyc_Absyn_Stmt*s2=_tmp1D2;
Cyc_Absyn_visit_exp(f1,f2,env,e1);_tmp1CE=s1;_tmp1CF=s2;goto _LL12;}case 2U: _LL11: _tmp1CE=((struct Cyc_Absyn_Seq_s_Absyn_Raw_stmt_struct*)_tmp1BD)->f1;_tmp1CF=((struct Cyc_Absyn_Seq_s_Absyn_Raw_stmt_struct*)_tmp1BD)->f2;_LL12: {struct Cyc_Absyn_Stmt*s1=_tmp1CE;struct Cyc_Absyn_Stmt*s2=_tmp1CF;
Cyc_Absyn_visit_stmt(f1,f2,env,s1);Cyc_Absyn_visit_stmt(f1,f2,env,s2);goto _LL0;}case 5U: _LL13: _tmp1CC=(((struct Cyc_Absyn_While_s_Absyn_Raw_stmt_struct*)_tmp1BD)->f1).f1;_tmp1CD=((struct Cyc_Absyn_While_s_Absyn_Raw_stmt_struct*)_tmp1BD)->f2;_LL14: {struct Cyc_Absyn_Exp*e=_tmp1CC;struct Cyc_Absyn_Stmt*s1=_tmp1CD;
_tmp1CA=s1;_tmp1CB=e;goto _LL16;}case 14U: _LL15: _tmp1CA=((struct Cyc_Absyn_Do_s_Absyn_Raw_stmt_struct*)_tmp1BD)->f1;_tmp1CB=(((struct Cyc_Absyn_Do_s_Absyn_Raw_stmt_struct*)_tmp1BD)->f2).f1;_LL16: {struct Cyc_Absyn_Stmt*s1=_tmp1CA;struct Cyc_Absyn_Exp*e=_tmp1CB;
Cyc_Absyn_visit_exp(f1,f2,env,e);Cyc_Absyn_visit_stmt(f1,f2,env,s1);goto _LL0;}case 9U: _LL17: _tmp1C6=((struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct*)_tmp1BD)->f1;_tmp1C7=(((struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct*)_tmp1BD)->f2).f1;_tmp1C8=(((struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct*)_tmp1BD)->f3).f1;_tmp1C9=((struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct*)_tmp1BD)->f4;_LL18: {struct Cyc_Absyn_Exp*e1=_tmp1C6;struct Cyc_Absyn_Exp*e2=_tmp1C7;struct Cyc_Absyn_Exp*e3=_tmp1C8;struct Cyc_Absyn_Stmt*s1=_tmp1C9;
# 1389
Cyc_Absyn_visit_exp(f1,f2,env,e1);Cyc_Absyn_visit_exp(f1,f2,env,e2);Cyc_Absyn_visit_exp(f1,f2,env,e3);
Cyc_Absyn_visit_stmt(f1,f2,env,s1);
goto _LL0;}case 11U: _LL19: _tmp1C5=((struct Cyc_Absyn_Fallthru_s_Absyn_Raw_stmt_struct*)_tmp1BD)->f1;_LL1A: {struct Cyc_List_List*es=_tmp1C5;
# 1393
for(0;es != 0;es=es->tl){
Cyc_Absyn_visit_exp(f1,f2,env,(struct Cyc_Absyn_Exp*)es->hd);}
goto _LL0;}case 12U: _LL1B: _tmp1C3=((struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct*)_tmp1BD)->f1;_tmp1C4=((struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct*)_tmp1BD)->f2;_LL1C: {struct Cyc_Absyn_Decl*d=_tmp1C3;struct Cyc_Absyn_Stmt*s1=_tmp1C4;
# 1397
{void*_stmttmp11=d->r;void*_tmp1D5=_stmttmp11;struct Cyc_Absyn_Exp*_tmp1D6;struct Cyc_Absyn_Exp*_tmp1D7;struct Cyc_Absyn_Fndecl*_tmp1D8;struct Cyc_Absyn_Vardecl*_tmp1D9;switch(*((int*)_tmp1D5)){case 0U: _LL24: _tmp1D9=((struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct*)_tmp1D5)->f1;_LL25: {struct Cyc_Absyn_Vardecl*vd=_tmp1D9;
# 1399
if(vd->initializer != 0)
Cyc_Absyn_visit_exp(f1,f2,env,(struct Cyc_Absyn_Exp*)_check_null(vd->initializer));
# 1399
goto _LL23;}case 1U: _LL26: _tmp1D8=((struct Cyc_Absyn_Fn_d_Absyn_Raw_decl_struct*)_tmp1D5)->f1;_LL27: {struct Cyc_Absyn_Fndecl*fd=_tmp1D8;
# 1402
Cyc_Absyn_visit_stmt(f1,f2,env,fd->body);goto _LL23;}case 2U: _LL28: _tmp1D7=((struct Cyc_Absyn_Let_d_Absyn_Raw_decl_struct*)_tmp1D5)->f3;_LL29: {struct Cyc_Absyn_Exp*e=_tmp1D7;
Cyc_Absyn_visit_exp(f1,f2,env,e);goto _LL23;}case 4U: _LL2A: _tmp1D6=((struct Cyc_Absyn_Region_d_Absyn_Raw_decl_struct*)_tmp1D5)->f3;_LL2B: {struct Cyc_Absyn_Exp*eo=_tmp1D6;
if((unsigned)eo)Cyc_Absyn_visit_exp(f1,f2,env,eo);goto _LL23;}default: _LL2C: _LL2D:
 goto _LL23;}_LL23:;}
# 1407
Cyc_Absyn_visit_stmt(f1,f2,env,s1);
goto _LL0;}case 13U: _LL1D: _tmp1C2=((struct Cyc_Absyn_Label_s_Absyn_Raw_stmt_struct*)_tmp1BD)->f2;_LL1E: {struct Cyc_Absyn_Stmt*s1=_tmp1C2;
Cyc_Absyn_visit_stmt(f1,f2,env,s1);goto _LL0;}case 10U: _LL1F: _tmp1C0=((struct Cyc_Absyn_Switch_s_Absyn_Raw_stmt_struct*)_tmp1BD)->f1;_tmp1C1=((struct Cyc_Absyn_Switch_s_Absyn_Raw_stmt_struct*)_tmp1BD)->f2;_LL20: {struct Cyc_Absyn_Exp*e=_tmp1C0;struct Cyc_List_List*scs=_tmp1C1;
# 1411
Cyc_Absyn_visit_exp(f1,f2,env,e);
Cyc_Absyn_visit_scs(f1,f2,env,scs);
goto _LL0;}default: _LL21: _tmp1BE=((struct Cyc_Absyn_TryCatch_s_Absyn_Raw_stmt_struct*)_tmp1BD)->f1;_tmp1BF=((struct Cyc_Absyn_TryCatch_s_Absyn_Raw_stmt_struct*)_tmp1BD)->f2;_LL22: {struct Cyc_Absyn_Stmt*s1=_tmp1BE;struct Cyc_List_List*scs=_tmp1BF;
# 1415
Cyc_Absyn_visit_stmt(f1,f2,env,s1);
Cyc_Absyn_visit_scs(f1,f2,env,scs);
goto _LL0;}}_LL0:;}}
# 1275
static void Cyc_Absyn_i_visit_type(int(*f)(void*,void*),void*env,void*t,struct Cyc_Hashtable_Table*);
# 1422
static void Cyc_Absyn_i_visit_tycon_types(int(*f)(void*,void*),void*env,void*t,struct Cyc_Hashtable_Table*seen){
void*_tmp1DA=t;struct Cyc_Absyn_Aggrdecl*_tmp1DB;struct Cyc_Absyn_Datatypedecl*_tmp1DC;struct Cyc_Absyn_Datatypefield*_tmp1DE;struct Cyc_Absyn_Datatypedecl*_tmp1DD;switch(*((int*)_tmp1DA)){case 19U: if(((((struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct*)_tmp1DA)->f1).KnownDatatypefield).tag == 2){_LL1: _tmp1DD=(((((struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct*)_tmp1DA)->f1).KnownDatatypefield).val).f1;_tmp1DE=(((((struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct*)_tmp1DA)->f1).KnownDatatypefield).val).f2;_LL2: {struct Cyc_Absyn_Datatypedecl*dtd=_tmp1DD;struct Cyc_Absyn_Datatypefield*dtf=_tmp1DE;
# 1425
struct Cyc_List_List*ts=dtf->typs;
for(0;(unsigned)ts;ts=ts->tl){
struct _tuple14*_stmttmp12=(struct _tuple14*)ts->hd;void*_tmp1DF;_LLA: _tmp1DF=_stmttmp12->f2;_LLB: {void*t=_tmp1DF;
Cyc_Absyn_i_visit_type(f,env,t,seen);}}
# 1430
_tmp1DC=dtd;goto _LL4;}}else{goto _LL7;}case 18U: if(((((struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct*)_tmp1DA)->f1).KnownDatatype).tag == 2){_LL3: _tmp1DC=*((((struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct*)_tmp1DA)->f1).KnownDatatype).val;_LL4: {struct Cyc_Absyn_Datatypedecl*dtd=_tmp1DC;
# 1432
if((unsigned)dtd->fields){
struct Cyc_List_List*fs=(struct Cyc_List_List*)((struct Cyc_Core_Opt*)_check_null(dtd->fields))->v;
for(0;(unsigned)fs;fs=fs->tl){
struct Cyc_List_List*ts=((struct Cyc_Absyn_Datatypefield*)fs->hd)->typs;
for(0;(unsigned)ts;ts=ts->tl){
struct _tuple14*_stmttmp13=(struct _tuple14*)ts->hd;void*_tmp1E0;_LLD: _tmp1E0=_stmttmp13->f2;_LLE: {void*t=_tmp1E0;
Cyc_Absyn_i_visit_type(f,env,t,seen);}}}}
# 1432
goto _LL0;}}else{goto _LL7;}case 20U: if(((((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)_tmp1DA)->f1).KnownAggr).tag == 2){_LL5: _tmp1DB=*((((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)_tmp1DA)->f1).KnownAggr).val;_LL6: {struct Cyc_Absyn_Aggrdecl*ad=_tmp1DB;
# 1444
if((unsigned)ad->impl){
# 1451
struct Cyc_List_List*afl=((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->fields;
for(0;(unsigned)afl;afl=afl->tl){
Cyc_Absyn_i_visit_type(f,env,((struct Cyc_Absyn_Aggrfield*)afl->hd)->type,seen);}}
# 1444
goto _LL0;}}else{goto _LL7;}default: _LL7: _LL8:
# 1458
 goto _LL0;}_LL0:;}
# 1422
static void Cyc_Absyn_i_visit_type(int(*f)(void*,void*),void*env,void*t,struct Cyc_Hashtable_Table*seen){
# 1463
if((unsigned)({int*(*_tmp33E)(struct Cyc_Hashtable_Table*t,void*key)=({int*(*_tmp1E1)(struct Cyc_Hashtable_Table*t,void*key)=(int*(*)(struct Cyc_Hashtable_Table*t,void*key))Cyc_Hashtable_lookup_opt;_tmp1E1;});struct Cyc_Hashtable_Table*_tmp33D=seen;_tmp33E(_tmp33D,t);}))
return;
# 1463
({void(*_tmp340)(struct Cyc_Hashtable_Table*t,void*key,int val)=({void(*_tmp1E2)(struct Cyc_Hashtable_Table*t,void*key,int val)=(void(*)(struct Cyc_Hashtable_Table*t,void*key,int val))Cyc_Hashtable_insert;_tmp1E2;});struct Cyc_Hashtable_Table*_tmp33F=seen;_tmp340(_tmp33F,t,1);});
# 1466
if(!f(env,t))
return;{
# 1466
void*_tmp1E3=t;void**_tmp1E4;void*_tmp1E7;struct Cyc_Absyn_Typedefdecl*_tmp1E6;struct Cyc_List_List*_tmp1E5;struct Cyc_List_List*_tmp1E8;struct Cyc_List_List*_tmp1ED;struct Cyc_Absyn_VarargInfo*_tmp1EC;struct Cyc_List_List*_tmp1EB;void*_tmp1EA;void*_tmp1E9;struct Cyc_List_List*_tmp1EE;void*_tmp1F0;void*_tmp1EF;void*_tmp1F5;void*_tmp1F4;void*_tmp1F3;void*_tmp1F2;void*_tmp1F1;void*_tmp1F6;struct Cyc_List_List*_tmp1F8;void*_tmp1F7;switch(*((int*)_tmp1E3)){case 0U: _LL1: _tmp1F7=(void*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp1E3)->f1;_tmp1F8=((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp1E3)->f2;_LL2: {void*tc=_tmp1F7;struct Cyc_List_List*ts=_tmp1F8;
# 1470
Cyc_Absyn_i_visit_tycon_types(f,env,tc,seen);
for(0;(unsigned)ts;ts=ts->tl){
Cyc_Absyn_i_visit_type(f,env,(void*)ts->hd,seen);}
goto _LL0;}case 1U: _LL3: _tmp1F6=(void*)((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp1E3)->f2;_LL4: {void*to=_tmp1F6;
# 1476
if((unsigned)to)
Cyc_Absyn_i_visit_type(f,env,to,seen);
# 1476
goto _LL0;}case 2U: _LL5: _LL6:
# 1479
 goto _LL0;case 3U: _LL7: _tmp1F1=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp1E3)->f1).elt_type;_tmp1F2=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp1E3)->f1).ptr_atts).rgn;_tmp1F3=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp1E3)->f1).ptr_atts).nullable;_tmp1F4=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp1E3)->f1).ptr_atts).bounds;_tmp1F5=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp1E3)->f1).ptr_atts).zero_term;_LL8: {void*ta=_tmp1F1;void*e=_tmp1F2;void*n=_tmp1F3;void*b=_tmp1F4;void*z=_tmp1F5;
# 1481
Cyc_Absyn_i_visit_type(f,env,ta,seen);
Cyc_Absyn_i_visit_type(f,env,e,seen);
Cyc_Absyn_i_visit_type(f,env,n,seen);
Cyc_Absyn_i_visit_type(f,env,b,seen);
Cyc_Absyn_i_visit_type(f,env,z,seen);
# 1488
goto _LL0;}case 4U: _LL9: _tmp1EF=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp1E3)->f1).elt_type;_tmp1F0=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp1E3)->f1).zero_term;_LLA: {void*ta=_tmp1EF;void*z=_tmp1F0;
# 1490
Cyc_Absyn_i_visit_type(f,env,ta,seen);
Cyc_Absyn_i_visit_type(f,env,z,seen);
goto _LL0;}case 6U: _LLB: _tmp1EE=((struct Cyc_Absyn_TupleType_Absyn_Type_struct*)_tmp1E3)->f1;_LLC: {struct Cyc_List_List*args=_tmp1EE;
# 1495
for(0;(unsigned)args;args=args->tl){
# 1497
struct _tuple14*_stmttmp14=(struct _tuple14*)args->hd;void*_tmp1F9;_LL1A: _tmp1F9=_stmttmp14->f2;_LL1B: {void*at=_tmp1F9;
Cyc_Absyn_i_visit_type(f,env,at,seen);}}
# 1500
goto _LL0;}case 5U: _LLD: _tmp1E9=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp1E3)->f1).effect;_tmp1EA=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp1E3)->f1).ret_type;_tmp1EB=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp1E3)->f1).args;_tmp1EC=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp1E3)->f1).cyc_varargs;_tmp1ED=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp1E3)->f1).rgn_po;_LLE: {void*e=_tmp1E9;void*r=_tmp1EA;struct Cyc_List_List*args=_tmp1EB;struct Cyc_Absyn_VarargInfo*va=_tmp1EC;struct Cyc_List_List*qb=_tmp1ED;
# 1502
if(e != 0)Cyc_Absyn_i_visit_type(f,env,e,seen);Cyc_Absyn_i_visit_type(f,env,r,seen);
# 1504
for(0;(unsigned)args;args=args->tl){
# 1506
struct _tuple8*_stmttmp15=(struct _tuple8*)args->hd;void*_tmp1FA;_LL1D: _tmp1FA=_stmttmp15->f3;_LL1E: {void*at=_tmp1FA;
Cyc_Absyn_i_visit_type(f,env,at,seen);}}
# 1509
if((unsigned)va)Cyc_Absyn_i_visit_type(f,env,va->type,seen);for(0;(unsigned)qb;qb=qb->tl){
# 1512
struct _tuple13*_stmttmp16=(struct _tuple13*)qb->hd;void*_tmp1FC;void*_tmp1FB;_LL20: _tmp1FB=_stmttmp16->f1;_tmp1FC=_stmttmp16->f2;_LL21: {void*t1=_tmp1FB;void*t2=_tmp1FC;
Cyc_Absyn_i_visit_type(f,env,t1,seen);
Cyc_Absyn_i_visit_type(f,env,t2,seen);}}
# 1516
goto _LL0;}case 7U: _LLF: _tmp1E8=((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmp1E3)->f2;_LL10: {struct Cyc_List_List*afl=_tmp1E8;
# 1518
for(0;(unsigned)afl;afl=afl->tl){
Cyc_Absyn_i_visit_type(f,env,((struct Cyc_Absyn_Aggrfield*)afl->hd)->type,seen);}
# 1521
goto _LL0;}case 8U: _LL11: _tmp1E5=((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmp1E3)->f2;_tmp1E6=((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmp1E3)->f3;_tmp1E7=(void*)((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmp1E3)->f4;_LL12: {struct Cyc_List_List*ts=_tmp1E5;struct Cyc_Absyn_Typedefdecl*tdef=_tmp1E6;void*to=_tmp1E7;
# 1523
for(0;(unsigned)ts;ts=ts->tl){
Cyc_Absyn_i_visit_type(f,env,(void*)ts->hd,seen);}
if(tdef != 0){
# 1527
void*x=tdef->defn;
if(x != 0)Cyc_Absyn_i_visit_type(f,env,x,seen);}
# 1525
if((unsigned)to)
# 1531
Cyc_Absyn_i_visit_type(f,env,to,seen);
# 1525
goto _LL0;}case 10U: _LL13: _tmp1E4=((struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct*)_tmp1E3)->f2;_LL14: {void**tptr=_tmp1E4;
# 1534
if((unsigned)tptr &&(unsigned)*tptr)
Cyc_Absyn_i_visit_type(f,env,*tptr,seen);
# 1534
goto _LL0;}case 9U: _LL15: _LL16:
# 1537
 goto _LL18;default: _LL17: _LL18:
 goto _LL0;}_LL0:;}}
# 1422
int Cyc_Absyn_cmp_ptr(void*a,void*b){
# 1542
return(int)((unsigned)a - (unsigned)b);}
# 1422
int Cyc_Absyn_hash_ptr(void*a){
# 1545
return(int)((unsigned)a);}
# 1422
void Cyc_Absyn_visit_type(int(*f)(void*,void*),void*env,void*t){
# 1548
struct _RegionHandle _tmp1FD=_new_region("r");struct _RegionHandle*r=& _tmp1FD;_push_region(r);
{struct Cyc_Hashtable_Table*seen=Cyc_Hashtable_rcreate(r,20,Cyc_Absyn_cmp_ptr,Cyc_Absyn_hash_ptr);
Cyc_Absyn_i_visit_type(f,env,t,seen);}
# 1549
;_pop_region(r);}
# 1422
static int Cyc_Absyn_no_side_effects_f1(int*env,struct Cyc_Absyn_Exp*e){
# 1556
void*_stmttmp17=e->r;void*_tmp1FE=_stmttmp17;switch(*((int*)_tmp1FE)){case 10U: _LL1: _LL2:
 goto _LL4;case 4U: _LL3: _LL4:
 goto _LL6;case 36U: _LL5: _LL6:
 goto _LL8;case 41U: _LL7: _LL8:
 goto _LLA;case 38U: _LL9: _LLA:
 goto _LLC;case 5U: _LLB: _LLC:
*env=0;return 0;case 18U: _LLD: _LLE:
 return 0;default: _LLF: _LL10:
 return 1;}_LL0:;}
# 1422
static int Cyc_Absyn_no_side_effects_f2(int*env,struct Cyc_Absyn_Stmt*s){
# 1568
({void*_tmp1FF=0U;({int(*_tmp342)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp201)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmp201;});struct _fat_ptr _tmp341=({const char*_tmp200="Compiler error: Absyn::no_side_effects looking at a statement";_tag_fat(_tmp200,sizeof(char),62U);});_tmp342(_tmp341,_tag_fat(_tmp1FF,sizeof(void*),0U));});});}
# 1422
int Cyc_Absyn_no_side_effects_exp(struct Cyc_Absyn_Exp*e){
# 1571
int ans=1;
({void(*_tmp343)(int(*f1)(int*,struct Cyc_Absyn_Exp*),int(*f2)(int*,struct Cyc_Absyn_Stmt*),int*env,struct Cyc_Absyn_Exp*e)=({void(*_tmp202)(int(*f1)(int*,struct Cyc_Absyn_Exp*),int(*f2)(int*,struct Cyc_Absyn_Stmt*),int*env,struct Cyc_Absyn_Exp*e)=(void(*)(int(*f1)(int*,struct Cyc_Absyn_Exp*),int(*f2)(int*,struct Cyc_Absyn_Stmt*),int*env,struct Cyc_Absyn_Exp*e))Cyc_Absyn_visit_exp;_tmp202;});_tmp343(Cyc_Absyn_no_side_effects_f1,Cyc_Absyn_no_side_effects_f2,& ans,e);});
return ans;}struct _tuple18{struct _tuple0*f1;int f2;};
# 1422
static int Cyc_Absyn_var_may_appear_f1(struct _tuple18*env,struct Cyc_Absyn_Exp*e){
# 1579
void*_stmttmp18=e->r;void*_tmp203=_stmttmp18;struct Cyc_Absyn_Vardecl*_tmp204;void*_tmp205;switch(*((int*)_tmp203)){case 1U: _LL1: _tmp205=(void*)((struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct*)_tmp203)->f1;_LL2: {void*b=_tmp205;
# 1581
if(({struct _tuple0*_tmp344=Cyc_Absyn_binding2qvar(b);Cyc_Absyn_qvar_cmp(_tmp344,(*env).f1);})== 0)
(*env).f2=1;
# 1581
return 0;}case 41U: _LL3: _LL4:
# 1584
 goto _LL6;case 38U: _LL5: _LL6:
# 1586
(*env).f2=1;
return 0;case 27U: _LL7: _tmp204=((struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct*)_tmp203)->f1;_LL8: {struct Cyc_Absyn_Vardecl*vd=_tmp204;
return Cyc_Absyn_qvar_cmp(vd->name,(*env).f1)!= 0;}default: _LL9: _LLA:
 return 1;}_LL0:;}
# 1422
static int Cyc_Absyn_var_may_appear_f2(struct _tuple18*env,struct Cyc_Absyn_Stmt*e){
# 1593
({void*_tmp206=0U;({int(*_tmp346)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp208)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmp208;});struct _fat_ptr _tmp345=({const char*_tmp207="Compiler error Absyn::no_side_effects looking at a statement";_tag_fat(_tmp207,sizeof(char),61U);});_tmp346(_tmp345,_tag_fat(_tmp206,sizeof(void*),0U));});});}
# 1422
int Cyc_Absyn_var_may_appear_exp(struct _tuple0*v,struct Cyc_Absyn_Exp*e){
# 1596
struct _tuple18 env=({struct _tuple18 _tmp261;_tmp261.f1=v,_tmp261.f2=0;_tmp261;});
({void(*_tmp347)(int(*f1)(struct _tuple18*,struct Cyc_Absyn_Exp*),int(*f2)(struct _tuple18*,struct Cyc_Absyn_Stmt*),struct _tuple18*env,struct Cyc_Absyn_Exp*e)=({void(*_tmp209)(int(*f1)(struct _tuple18*,struct Cyc_Absyn_Exp*),int(*f2)(struct _tuple18*,struct Cyc_Absyn_Stmt*),struct _tuple18*env,struct Cyc_Absyn_Exp*e)=(void(*)(int(*f1)(struct _tuple18*,struct Cyc_Absyn_Exp*),int(*f2)(struct _tuple18*,struct Cyc_Absyn_Stmt*),struct _tuple18*env,struct Cyc_Absyn_Exp*e))Cyc_Absyn_visit_exp;_tmp209;});_tmp347(Cyc_Absyn_var_may_appear_f1,Cyc_Absyn_var_may_appear_f2,& env,e);});
return env.f2;}struct Cyc_Absyn_NestedStmtEnv{void(*f)(void*,struct Cyc_Absyn_Stmt*);void*env;int szeof;};
# 1606
static int Cyc_Absyn_do_nested_stmt_f1(struct Cyc_Absyn_NestedStmtEnv*env,struct Cyc_Absyn_Exp*e){
void*_stmttmp19=e->r;void*_tmp20A=_stmttmp19;if(((struct Cyc_Absyn_Sizeofexp_e_Absyn_Raw_exp_struct*)_tmp20A)->tag == 18U){_LL1: _LL2:
 return env->szeof;}else{_LL3: _LL4:
 return 1;}_LL0:;}
# 1606
static int Cyc_Absyn_do_nested_stmt_f2(struct Cyc_Absyn_NestedStmtEnv*env,struct Cyc_Absyn_Stmt*s){
# 1614
(env->f)(env->env,s);
return 0;}
# 1606
void Cyc_Absyn_do_nested_statement(struct Cyc_Absyn_Exp*e,void*env,void(*f)(void*,struct Cyc_Absyn_Stmt*),int szeof){
# 1618
struct Cyc_Absyn_NestedStmtEnv nested_env=({struct Cyc_Absyn_NestedStmtEnv _tmp262;_tmp262.f=f,_tmp262.env=env,_tmp262.szeof=szeof;_tmp262;});
({void(*_tmp348)(int(*f1)(struct Cyc_Absyn_NestedStmtEnv*,struct Cyc_Absyn_Exp*),int(*f2)(struct Cyc_Absyn_NestedStmtEnv*,struct Cyc_Absyn_Stmt*),struct Cyc_Absyn_NestedStmtEnv*env,struct Cyc_Absyn_Exp*e)=({void(*_tmp20B)(int(*f1)(struct Cyc_Absyn_NestedStmtEnv*,struct Cyc_Absyn_Exp*),int(*f2)(struct Cyc_Absyn_NestedStmtEnv*,struct Cyc_Absyn_Stmt*),struct Cyc_Absyn_NestedStmtEnv*env,struct Cyc_Absyn_Exp*e)=(void(*)(int(*f1)(struct Cyc_Absyn_NestedStmtEnv*,struct Cyc_Absyn_Exp*),int(*f2)(struct Cyc_Absyn_NestedStmtEnv*,struct Cyc_Absyn_Stmt*),struct Cyc_Absyn_NestedStmtEnv*env,struct Cyc_Absyn_Exp*e))Cyc_Absyn_visit_exp;_tmp20B;});_tmp348(Cyc_Absyn_do_nested_stmt_f1,Cyc_Absyn_do_nested_stmt_f2,& nested_env,e);});}struct _tuple19{void*f1;int f2;struct Cyc_List_List*f3;};
# 1606
static struct _tuple19 Cyc_Absyn_iter_tvar(unsigned loc,struct Cyc_List_List*tlist,int check){
# 1624
if(check &&(tlist == 0 ||(void*)tlist->hd == 0)){
# 1626
if(check)({void*_tmp20C=0U;({unsigned _tmp34A=loc;struct _fat_ptr _tmp349=({const char*_tmp20D="Expected a type variable ...";_tag_fat(_tmp20D,sizeof(char),29U);});Cyc_Warn_err(_tmp34A,_tmp349,_tag_fat(_tmp20C,sizeof(void*),0U));});});return({struct _tuple19 _tmp263;_tmp263.f1=0,_tmp263.f2=0,_tmp263.f3=0;_tmp263;});}{
# 1624
void*_stmttmp1A=(void*)((struct Cyc_List_List*)_check_null(tlist))->hd;void*_tmp20E=_stmttmp1A;struct Cyc_Absyn_Tvar*_tmp20F;if(((struct Cyc_Absyn_VarType_Absyn_Type_struct*)_tmp20E)->tag == 2U){_LL1: _tmp20F=((struct Cyc_Absyn_VarType_Absyn_Type_struct*)_tmp20E)->f1;_LL2: {struct Cyc_Absyn_Tvar*tvar=_tmp20F;
# 1631
return({struct _tuple19 _tmp264;_tmp264.f1=(void*)tlist->hd,_tmp264.f2=1,_tmp264.f3=tlist->tl;_tmp264;});}}else{_LL3: _LL4:
# 1633
({void*_tmp210=0U;({unsigned _tmp34C=loc;struct _fat_ptr _tmp34B=({const char*_tmp211="Expected type variable";_tag_fat(_tmp211,sizeof(char),23U);});Cyc_Warn_err(_tmp34C,_tmp34B,_tag_fat(_tmp210,sizeof(void*),0U));});});
return({struct _tuple19 _tmp265;_tmp265.f1=0,_tmp265.f2=0,_tmp265.f3=0;_tmp265;});}_LL0:;}}struct _tuple20{int f1;int f2;};
# 1606
struct _tuple20 Cyc_Absyn_parse_int(struct _fat_ptr str){
# 1640
if(({char*_tmp34D=(char*)str.curr;_tmp34D == (char*)(_tag_fat(0,0,0)).curr;}))return({struct _tuple20 _tmp266;_tmp266.f1=0,_tmp266.f2=0;_tmp266;});{int ret;int cnt;
# 1642
for((ret=0,cnt=1);(int)*((const char*)_check_fat_subscript(str,sizeof(char),0U))!= (int)'\000';(_fat_ptr_inplace_plus_post(& str,sizeof(char),1),cnt *=10)){
# 1644
char x=*((const char*)_check_fat_subscript(str,sizeof(char),0U));
if((int)x > (int)'9' ||(int)x < (int)'0')return({struct _tuple20 _tmp267;_tmp267.f1=0,_tmp267.f2=0;_tmp267;});else{
ret=ret * cnt + ((int)x - (int)'0');}}
# 1648
return({struct _tuple20 _tmp268;_tmp268.f1=ret,_tmp268.f2=1;_tmp268;});}}struct _tuple21{struct Cyc_Absyn_RgnEffect*f1;int f2;};
# 1606
static struct _tuple21 Cyc_Absyn_parse_rgneffect(unsigned loc,void*t){
# 1676 "absyn.cyc"
void*_tmp212=t;struct Cyc_List_List*_tmp213;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp212)->tag == 0U){if(((struct Cyc_Absyn_JoinCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp212)->f1)->tag == 9U){_LL1: _tmp213=((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp212)->f2;_LL2: {struct Cyc_List_List*ts=_tmp213;
# 1679
if(ts == 0){
# 1681
({void*_tmp214=0U;({int(*_tmp34F)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp216)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmp216;});struct _fat_ptr _tmp34E=({const char*_tmp215="Compiler error Absyn:: parse_rgneffect no effects";_tag_fat(_tmp215,sizeof(char),50U);});_tmp34F(_tmp34E,_tag_fat(_tmp214,sizeof(void*),0U));});});
return({struct _tuple21 _tmp269;_tmp269.f1=0,_tmp269.f2=0;_tmp269;});}{
# 1679
int fail=0;
# 1686
struct _tuple19 _stmttmp1B=Cyc_Absyn_iter_tvar(loc,ts,1);struct Cyc_List_List*_tmp219;int _tmp218;void*_tmp217;_LL6: _tmp217=_stmttmp1B.f1;_tmp218=_stmttmp1B.f2;_tmp219=_stmttmp1B.f3;_LL7: {void*name=_tmp217;int fail1=_tmp218;struct Cyc_List_List*next1=_tmp219;
struct _tuple19 _stmttmp1C=Cyc_Absyn_iter_tvar(loc,next1,1);struct Cyc_List_List*_tmp21C;int _tmp21B;void*_tmp21A;_LL9: _tmp21A=_stmttmp1C.f1;_tmp21B=_stmttmp1C.f2;_tmp21C=_stmttmp1C.f3;_LLA: {void*rgn=_tmp21A;int fail2=_tmp21B;struct Cyc_List_List*next2=_tmp21C;
struct _tuple19 _stmttmp1D=Cyc_Absyn_iter_tvar(loc,next2,1);struct Cyc_List_List*_tmp21F;int _tmp21E;void*_tmp21D;_LLC: _tmp21D=_stmttmp1D.f1;_tmp21E=_stmttmp1D.f2;_tmp21F=_stmttmp1D.f3;_LLD: {void*lock=_tmp21D;int fail3=_tmp21E;struct Cyc_List_List*next3=_tmp21F;
struct _tuple19 _stmttmp1E=Cyc_Absyn_iter_tvar(loc,next3,1);struct Cyc_List_List*_tmp222;int _tmp221;void*_tmp220;_LLF: _tmp220=_stmttmp1E.f1;_tmp221=_stmttmp1E.f2;_tmp222=_stmttmp1E.f3;_LL10: {void*unadopt=_tmp220;int fail4=_tmp221;struct Cyc_List_List*next4=_tmp222;
struct _tuple19 _stmttmp1F=Cyc_Absyn_iter_tvar(loc,next4,0);void*_tmp223;_LL12: _tmp223=_stmttmp1F.f1;_LL13: {void*parent=_tmp223;
if(((!fail1 || !fail2)|| !fail3)|| !fail4)return({struct _tuple21 _tmp26A;_tmp26A.f1=0,_tmp26A.f2=0;_tmp26A;});return({struct _tuple21 _tmp26B;({
# 1695
struct Cyc_Absyn_RgnEffect*_tmp353=({struct Cyc_Absyn_RgnEffect*_tmp227=_cycalloc(sizeof(*_tmp227));_tmp227->name=(void*)_check_null(name),({void*_tmp352=(void*)({struct Cyc_Absyn_Star_Absyn_Cap_struct*_tmp224=_cycalloc(sizeof(*_tmp224));_tmp224->tag=0U;_tmp224;});_tmp227->rgn=_tmp352;}),({void*_tmp351=(void*)({struct Cyc_Absyn_Star_Absyn_Cap_struct*_tmp225=_cycalloc(sizeof(*_tmp225));_tmp225->tag=0U;_tmp225;});_tmp227->lock=_tmp351;}),({void*_tmp350=(void*)({struct Cyc_Absyn_Star_Absyn_Cap_struct*_tmp226=_cycalloc(sizeof(*_tmp226));_tmp226->tag=0U;_tmp226;});_tmp227->unadopt=_tmp350;}),_tmp227->parent=parent;_tmp227;});_tmp26B.f1=_tmp353;}),_tmp26B.f2=1;_tmp26B;});
goto _LL0;}}}}}}}}else{goto _LL3;}}else{_LL3: _LL4:
# 1698
({void*_tmp228=0U;({unsigned _tmp355=loc;struct _fat_ptr _tmp354=({const char*_tmp229="Expected tuple type";_tag_fat(_tmp229,sizeof(char),20U);});Cyc_Warn_err(_tmp355,_tmp354,_tag_fat(_tmp228,sizeof(void*),0U));});});
return({struct _tuple21 _tmp26C;_tmp26C.f1=0,_tmp26C.f2=0;_tmp26C;});}_LL0:;}
# 1606 "absyn.cyc"
struct Cyc_List_List*Cyc_Absyn_parse_rgneffects(unsigned loc,void*t){
# 1705 "absyn.cyc"
struct Cyc_List_List*ret=0;
int shouldRet=1;
if(t == 0)return 0;{void*_tmp22A=t;struct Cyc_List_List*_tmp22B;if(_tmp22A != 0){if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp22A)->tag == 0U){if(((struct Cyc_Absyn_JoinCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp22A)->f1)->tag == 9U){_LL1: _tmp22B=((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp22A)->f2;_LL2: {struct Cyc_List_List*lexp=_tmp22B;
# 1711
for(0;lexp != 0;lexp=lexp->tl){
# 1713
struct _tuple21 _stmttmp20=Cyc_Absyn_parse_rgneffect(loc,(void*)lexp->hd);int _tmp22D;struct Cyc_Absyn_RgnEffect*_tmp22C;_LL6: _tmp22C=_stmttmp20.f1;_tmp22D=_stmttmp20.f2;_LL7: {struct Cyc_Absyn_RgnEffect*eff=_tmp22C;int b=_tmp22D;
if(!b)shouldRet=0;else{
ret=({struct Cyc_List_List*_tmp22E=_cycalloc(sizeof(*_tmp22E));_tmp22E->hd=(struct Cyc_Absyn_RgnEffect*)_check_null(eff),_tmp22E->tl=ret;_tmp22E;});}}}
# 1717
goto _LL0;}}else{goto _LL3;}}else{goto _LL3;}}else{_LL3: _LL4:
# 1719
({void*_tmp22F=0U;({unsigned _tmp357=loc;struct _fat_ptr _tmp356=({const char*_tmp230="Expected application type";_tag_fat(_tmp230,sizeof(char),26U);});Cyc_Warn_err(_tmp357,_tmp356,_tag_fat(_tmp22F,sizeof(void*),0U));});});
return 0;}_LL0:;}
# 1722
return ret;}
# 1606 "absyn.cyc"
struct Cyc_List_List*Cyc_Absyn_rgneffect_rnames(struct Cyc_List_List*l){
# 1727 "absyn.cyc"
struct Cyc_List_List*ret=0;
for(0;l != 0;l=l->tl){ret=({struct Cyc_List_List*_tmp232=_cycalloc(sizeof(*_tmp232));_tmp232->hd=((struct Cyc_Absyn_RgnEffect*)l->hd)->name,((struct Cyc_Absyn_RgnEffect*)l->hd)->parent != 0?({struct Cyc_List_List*_tmp358=({struct Cyc_List_List*_tmp231=_cycalloc(sizeof(*_tmp231));_tmp231->hd=(void*)_check_null(((struct Cyc_Absyn_RgnEffect*)l->hd)->parent),_tmp231->tl=ret;_tmp231;});_tmp232->tl=_tmp358;}):(_tmp232->tl=ret);_tmp232;});}
return ret;}
# 1606 "absyn.cyc"
struct Cyc_Absyn_Porton_d_Absyn_Raw_decl_struct Cyc_Absyn_Porton_d_val={13U};
# 1733 "absyn.cyc"
struct Cyc_Absyn_Portoff_d_Absyn_Raw_decl_struct Cyc_Absyn_Portoff_d_val={14U};
struct Cyc_Absyn_Tempeston_d_Absyn_Raw_decl_struct Cyc_Absyn_Tempeston_d_val={15U};
struct Cyc_Absyn_Tempestoff_d_Absyn_Raw_decl_struct Cyc_Absyn_Tempestoff_d_val={16U};
# 1737
int Cyc_Absyn_porting_c_code=0;
int Cyc_Absyn_no_regions=0;
