#include <setjmp.h>
/* This is a C header file to be used by the output of the Cyclone to
   C translator.  The corresponding definitions are in file lib/runtime_*.c */
#ifndef _CYC_INCLUDE_H_
#define _CYC_INCLUDE_H_

/* Need one of these per thread (see runtime_stack.c). The runtime maintains 
   a stack that contains either _handler_cons structs or _RegionHandle structs.
   The tag is 0 for a handler_cons and 1 for a region handle.  */
struct _RuntimeStack {
  int tag; 
  struct _RuntimeStack *next;
  void (*cleanup)(struct _RuntimeStack *frame);
};

#ifndef offsetof
/* should be size_t, but int is fine. */
#define offsetof(t,n) ((int)(&(((t *)0)->n)))
#endif

/* Fat pointers */
struct _fat_ptr {
  unsigned char *curr; 
  unsigned char *base; 
  unsigned char *last_plus_one; 
};  

/* Discriminated Unions */
struct _xtunion_struct { char *tag; };

/* Regions */
struct _RegionPage
#ifdef CYC_REGION_PROFILE
{ unsigned total_bytes;
  unsigned free_bytes;
  /* MWH: wish we didn't have to include the stuff below ... */
  struct _RegionPage *next;
  char data[1];
}
#endif
; // abstract -- defined in runtime_memory.c
struct _RegionHandle {
  struct _RuntimeStack s;
  struct _RegionPage *curr;
  char               *offset;
  char               *last_plus_one;
  struct _DynRegionHandle *sub_regions;
#ifdef CYC_REGION_PROFILE
  const char         *name;
#else
  unsigned used_bytes;
  unsigned wasted_bytes;
#endif
};
struct _DynRegionFrame {
  struct _RuntimeStack s;
  struct _DynRegionHandle *x;
};
// A dynamic region is just a region handle.  The wrapper struct is for type
// abstraction.
struct Cyc_Core_DynamicRegion {
  struct _RegionHandle h;
};

struct _RegionHandle _new_region(const char*);
void* _region_malloc(struct _RegionHandle*, unsigned);
void* _region_calloc(struct _RegionHandle*, unsigned t, unsigned n);
void   _free_region(struct _RegionHandle*);
struct _RegionHandle*_open_dynregion(struct _DynRegionFrame*,struct _DynRegionHandle*);
void   _pop_dynregion();

/* Exceptions */
struct _handler_cons {
  struct _RuntimeStack s;
  jmp_buf handler;
};
void _push_handler(struct _handler_cons *);
void _push_region(struct _RegionHandle *);
void _npop_handler(int);
void _pop_handler();
void _pop_region();

#ifndef _throw
void* _throw_null_fn(const char*,unsigned);
void* _throw_arraybounds_fn(const char*,unsigned);
void* _throw_badalloc_fn(const char*,unsigned);
void* _throw_match_fn(const char*,unsigned);
void* _throw_fn(void*,const char*,unsigned);
void* _rethrow(void*);
#define _throw_null() (_throw_null_fn(__FILE__,__LINE__))
#define _throw_arraybounds() (_throw_arraybounds_fn(__FILE__,__LINE__))
#define _throw_badalloc() (_throw_badalloc_fn(__FILE__,__LINE__))
#define _throw_match() (_throw_match_fn(__FILE__,__LINE__))
#define _throw(e) (_throw_fn((e),__FILE__,__LINE__))
#endif

struct _xtunion_struct* Cyc_Core_get_exn_thrown();
/* Built-in Exceptions */
struct Cyc_Null_Exception_exn_struct { char *tag; };
struct Cyc_Array_bounds_exn_struct { char *tag; };
struct Cyc_Match_Exception_exn_struct { char *tag; };
struct Cyc_Bad_alloc_exn_struct { char *tag; };
extern char Cyc_Null_Exception[];
extern char Cyc_Array_bounds[];
extern char Cyc_Match_Exception[];
extern char Cyc_Bad_alloc[];

/* Built-in Run-time Checks and company */
#ifdef CYC_ANSI_OUTPUT
#define _INLINE  
#else
#define _INLINE inline
#endif

#ifdef NO_CYC_NULL_CHECKS
#define _check_null(ptr) (ptr)
#else
#define _check_null(ptr) \
  ({ void*_cks_null = (void*)(ptr); \
     if (!_cks_null) _throw_null(); \
     _cks_null; })
#endif

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index)\
   (((char*)ptr) + (elt_sz)*(index))
#ifdef NO_CYC_NULL_CHECKS
#define _check_known_subscript_null _check_known_subscript_notnull
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr);\
  int _index = (index);\
  if (!_cks_ptr) _throw_null(); \
  _cks_ptr + (elt_sz)*_index; })
#endif
#define _zero_arr_plus_fn(orig_x,orig_sz,orig_i,f,l) ((orig_x)+(orig_i))
#define _zero_arr_plus_char_fn _zero_arr_plus_fn
#define _zero_arr_plus_short_fn _zero_arr_plus_fn
#define _zero_arr_plus_int_fn _zero_arr_plus_fn
#define _zero_arr_plus_float_fn _zero_arr_plus_fn
#define _zero_arr_plus_double_fn _zero_arr_plus_fn
#define _zero_arr_plus_longdouble_fn _zero_arr_plus_fn
#define _zero_arr_plus_voidstar_fn _zero_arr_plus_fn
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (!_cks_ptr) _throw_null(); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })

/* _zero_arr_plus_*_fn(x,sz,i,filename,lineno) adds i to zero-terminated ptr
   x that has at least sz elements */
char* _zero_arr_plus_char_fn(char*,unsigned,int,const char*,unsigned);
short* _zero_arr_plus_short_fn(short*,unsigned,int,const char*,unsigned);
int* _zero_arr_plus_int_fn(int*,unsigned,int,const char*,unsigned);
float* _zero_arr_plus_float_fn(float*,unsigned,int,const char*,unsigned);
double* _zero_arr_plus_double_fn(double*,unsigned,int,const char*,unsigned);
long double* _zero_arr_plus_longdouble_fn(long double*,unsigned,int,const char*, unsigned);
void** _zero_arr_plus_voidstar_fn(void**,unsigned,int,const char*,unsigned);
#endif

/* _get_zero_arr_size_*(x,sz) returns the number of elements in a
   zero-terminated array that is NULL or has at least sz elements */
int _get_zero_arr_size_char(const char*,unsigned);
int _get_zero_arr_size_short(const short*,unsigned);
int _get_zero_arr_size_int(const int*,unsigned);
int _get_zero_arr_size_float(const float*,unsigned);
int _get_zero_arr_size_double(const double*,unsigned);
int _get_zero_arr_size_longdouble(const long double*,unsigned);
int _get_zero_arr_size_voidstar(const void**,unsigned);

/* _zero_arr_inplace_plus_*_fn(x,i,filename,lineno) sets
   zero-terminated pointer *x to *x + i */
char* _zero_arr_inplace_plus_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_short_fn(short**,int,const char*,unsigned);
int* _zero_arr_inplace_plus_int(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_longdouble_fn(long double**,int,const char*,unsigned);
void** _zero_arr_inplace_plus_voidstar_fn(void***,int,const char*,unsigned);
/* like the previous functions, but does post-addition (as in e++) */
char* _zero_arr_inplace_plus_post_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_post_short_fn(short**x,int,const char*,unsigned);
int* _zero_arr_inplace_plus_post_int_fn(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_post_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_post_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_post_longdouble_fn(long double**,int,const char *,unsigned);
void** _zero_arr_inplace_plus_post_voidstar_fn(void***,int,const char*,unsigned);
#define _zero_arr_plus_char(x,s,i) \
  (_zero_arr_plus_char_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_short(x,s,i) \
  (_zero_arr_plus_short_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_int(x,s,i) \
  (_zero_arr_plus_int_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_float(x,s,i) \
  (_zero_arr_plus_float_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_double(x,s,i) \
  (_zero_arr_plus_double_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_longdouble(x,s,i) \
  (_zero_arr_plus_longdouble_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_voidstar(x,s,i) \
  (_zero_arr_plus_voidstar_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_inplace_plus_char(x,i) \
  _zero_arr_inplace_plus_char_fn((char **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_short(x,i) \
  _zero_arr_inplace_plus_short_fn((short **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_int(x,i) \
  _zero_arr_inplace_plus_int_fn((int **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_float(x,i) \
  _zero_arr_inplace_plus_float_fn((float **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_double(x,i) \
  _zero_arr_inplace_plus_double_fn((double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_longdouble(x,i) \
  _zero_arr_inplace_plus_longdouble_fn((long double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_voidstar(x,i) \
  _zero_arr_inplace_plus_voidstar_fn((void ***)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_char(x,i) \
  _zero_arr_inplace_plus_post_char_fn((char **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_short(x,i) \
  _zero_arr_inplace_plus_post_short_fn((short **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_int(x,i) \
  _zero_arr_inplace_plus_post_int_fn((int **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_float(x,i) \
  _zero_arr_inplace_plus_post_float_fn((float **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_double(x,i) \
  _zero_arr_inplace_plus_post_double_fn((double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_longdouble(x,i) \
  _zero_arr_inplace_plus_post_longdouble_fn((long double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_voidstar(x,i) \
  _zero_arr_inplace_plus_post_voidstar_fn((void***)(x),(i),__FILE__,__LINE__)

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_fat_subscript(arr,elt_sz,index) ((arr).curr + (elt_sz) * (index))
#define _untag_fat_ptr(arr,elt_sz,num_elts) ((arr).curr)
#else
#define _check_fat_subscript(arr,elt_sz,index) ({ \
  struct _fat_ptr _cus_arr = (arr); \
  unsigned char *_cus_ans = _cus_arr.curr + (elt_sz) * (index); \
  /* JGM: not needed! if (!_cus_arr.base) _throw_null();*/ \
  if (_cus_ans < _cus_arr.base || _cus_ans >= _cus_arr.last_plus_one) \
    _throw_arraybounds(); \
  _cus_ans; })
#define _untag_fat_ptr(arr,elt_sz,num_elts) ({ \
  struct _fat_ptr _arr = (arr); \
  unsigned char *_curr = _arr.curr; \
  if ((_curr < _arr.base || _curr + (elt_sz) * (num_elts) > _arr.last_plus_one) &&\
      _curr != (unsigned char *)0) \
    _throw_arraybounds(); \
  _curr; })
#endif

#define _tag_fat(tcurr,elt_sz,num_elts) ({ \
  struct _fat_ptr _ans; \
  unsigned _num_elts = (num_elts);\
  _ans.base = _ans.curr = (void*)(tcurr); \
  /* JGM: if we're tagging NULL, ignore num_elts */ \
  _ans.last_plus_one = _ans.base ? (_ans.base + (elt_sz) * _num_elts) : 0; \
  _ans; })

#define _get_fat_size(arr,elt_sz) \
  ({struct _fat_ptr _arr = (arr); \
    unsigned char *_arr_curr=_arr.curr; \
    unsigned char *_arr_last=_arr.last_plus_one; \
    (_arr_curr < _arr.base || _arr_curr >= _arr_last) ? 0 : \
    ((_arr_last - _arr_curr) / (elt_sz));})

#define _fat_ptr_plus(arr,elt_sz,change) ({ \
  struct _fat_ptr _ans = (arr); \
  int _change = (change);\
  _ans.curr += (elt_sz) * _change;\
  _ans; })
#define _fat_ptr_inplace_plus(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  _arr_ptr->curr += (elt_sz) * (change);\
  *_arr_ptr; })
#define _fat_ptr_inplace_plus_post(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  struct _fat_ptr _ans = *_arr_ptr; \
  _arr_ptr->curr += (elt_sz) * (change);\
  _ans; })

//Not a macro since initialization order matters. Defined in runtime_zeroterm.c.
struct _fat_ptr _fat_ptr_decrease_size(struct _fat_ptr,unsigned sz,unsigned numelts);

/* Allocation */
void* GC_malloc(int);
void* GC_malloc_atomic(int);
void* GC_calloc(unsigned,unsigned);
void* GC_calloc_atomic(unsigned,unsigned);
// bound the allocation size to be < MAX_ALLOC_SIZE. See macros below for usage.
#define MAX_MALLOC_SIZE (1 << 28)
void* _bounded_GC_malloc(int,const char*,int);
void* _bounded_GC_malloc_atomic(int,const char*,int);
void* _bounded_GC_calloc(unsigned,unsigned,const char*,int);
void* _bounded_GC_calloc_atomic(unsigned,unsigned,const char*,int);
/* these macros are overridden below ifdef CYC_REGION_PROFILE */
#ifndef CYC_REGION_PROFILE
#define _cycalloc(n) _bounded_GC_malloc(n,__FILE__,__LINE__)
#define _cycalloc_atomic(n) _bounded_GC_malloc_atomic(n,__FILE__,__LINE__)
#define _cyccalloc(n,s) _bounded_GC_calloc(n,s,__FILE__,__LINE__)
#define _cyccalloc_atomic(n,s) _bounded_GC_calloc_atomic(n,s,__FILE__,__LINE__)
#endif

static _INLINE unsigned int _check_times(unsigned x, unsigned y) {
  unsigned long long whole_ans = 
    ((unsigned long long) x)*((unsigned long long)y);
  unsigned word_ans = (unsigned)whole_ans;
  if(word_ans < whole_ans || word_ans > MAX_MALLOC_SIZE)
    _throw_badalloc();
  return word_ans;
}

#define _CYC_MAX_REGION_CONST 2
#define _CYC_MIN_ALIGNMENT (sizeof(double))

#ifdef CYC_REGION_PROFILE
extern int rgn_total_bytes;
#endif

static _INLINE void *_fast_region_malloc(struct _RegionHandle *r, unsigned orig_s) {  
  if (r > (struct _RegionHandle *)_CYC_MAX_REGION_CONST && r->curr != 0) { 
#ifdef CYC_NOALIGN
    unsigned s =  orig_s;
#else
    unsigned s =  (orig_s + _CYC_MIN_ALIGNMENT - 1) & (~(_CYC_MIN_ALIGNMENT -1)); 
#endif
    char *result; 
    result = r->offset; 
    if (s <= (r->last_plus_one - result)) {
      r->offset = result + s; 
#ifdef CYC_REGION_PROFILE
    r->curr->free_bytes = r->curr->free_bytes - s;
    rgn_total_bytes += s;
#endif
      return result;
    }
  } 
  return _region_malloc(r,orig_s); 
}

#ifdef CYC_REGION_PROFILE
/* see macros below for usage. defined in runtime_memory.c */
void* _profile_GC_malloc(int,const char*,const char*,int);
void* _profile_GC_malloc_atomic(int,const char*,const char*,int);
void* _profile_GC_calloc(unsigned,unsigned,const char*,const char*,int);
void* _profile_GC_calloc_atomic(unsigned,unsigned,const char*,const char*,int);
void* _profile_region_malloc(struct _RegionHandle*,unsigned,const char*,const char*,int);
void* _profile_region_calloc(struct _RegionHandle*,unsigned,unsigned,const char *,const char*,int);
struct _RegionHandle _profile_new_region(const char*,const char*,const char*,int);
void _profile_free_region(struct _RegionHandle*,const char*,const char*,int);
#ifndef RUNTIME_CYC
#define _new_region(n) _profile_new_region(n,__FILE__,__FUNCTION__,__LINE__)
#define _free_region(r) _profile_free_region(r,__FILE__,__FUNCTION__,__LINE__)
#define _region_malloc(rh,n) _profile_region_malloc(rh,n,__FILE__,__FUNCTION__,__LINE__)
#define _region_calloc(rh,n,t) _profile_region_calloc(rh,n,t,__FILE__,__FUNCTION__,__LINE__)
#  endif
#define _cycalloc(n) _profile_GC_malloc(n,__FILE__,__FUNCTION__,__LINE__)
#define _cycalloc_atomic(n) _profile_GC_malloc_atomic(n,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc(n,s) _profile_GC_calloc(n,s,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc_atomic(n,s) _profile_GC_calloc_atomic(n,s,__FILE__,__FUNCTION__,__LINE__)
#endif
#endif
 struct Cyc_Core_Opt{void*v;};extern char Cyc_Core_Invalid_argument[17U];struct Cyc_Core_Invalid_argument_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Failure[8U];struct Cyc_Core_Failure_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Impossible[11U];struct Cyc_Core_Impossible_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Not_found[10U];struct Cyc_Core_Not_found_exn_struct{char*tag;};extern char Cyc_Core_Unreachable[12U];struct Cyc_Core_Unreachable_exn_struct{char*tag;struct _fat_ptr f1;};
# 168 "core.h"
extern struct _RegionHandle*Cyc_Core_unique_region;struct Cyc_Core_DynamicRegion;struct Cyc_Core_NewDynamicRegion{struct Cyc_Core_DynamicRegion*key;};struct Cyc_Core_ThinRes{void*arr;unsigned nelts;};struct Cyc_List_List{void*hd;struct Cyc_List_List*tl;};
# 61 "list.h"
int Cyc_List_length(struct Cyc_List_List*x);extern char Cyc_List_List_mismatch[14U];struct Cyc_List_List_mismatch_exn_struct{char*tag;};
# 178
struct Cyc_List_List*Cyc_List_imp_rev(struct Cyc_List_List*x);
# 184
struct Cyc_List_List*Cyc_List_append(struct Cyc_List_List*x,struct Cyc_List_List*y);
# 195
struct Cyc_List_List*Cyc_List_imp_append(struct Cyc_List_List*x,struct Cyc_List_List*y);extern char Cyc_List_Nth[4U];struct Cyc_List_Nth_exn_struct{char*tag;};struct Cyc___cycFILE;
# 51 "cycboot.h"
extern struct Cyc___cycFILE*Cyc_stdout;struct Cyc_String_pa_PrintArg_struct{int tag;struct _fat_ptr f1;};struct Cyc_Int_pa_PrintArg_struct{int tag;unsigned long f1;};struct Cyc_Double_pa_PrintArg_struct{int tag;double f1;};struct Cyc_LongDouble_pa_PrintArg_struct{int tag;long double f1;};struct Cyc_ShortPtr_pa_PrintArg_struct{int tag;short*f1;};struct Cyc_IntPtr_pa_PrintArg_struct{int tag;unsigned long*f1;};
# 73
struct _fat_ptr Cyc_aprintf(struct _fat_ptr,struct _fat_ptr);
# 100
int Cyc_fprintf(struct Cyc___cycFILE*,struct _fat_ptr,struct _fat_ptr);
# 104
int Cyc_fputc(int,struct Cyc___cycFILE*);struct Cyc_ShortPtr_sa_ScanfArg_struct{int tag;short*f1;};struct Cyc_UShortPtr_sa_ScanfArg_struct{int tag;unsigned short*f1;};struct Cyc_IntPtr_sa_ScanfArg_struct{int tag;int*f1;};struct Cyc_UIntPtr_sa_ScanfArg_struct{int tag;unsigned*f1;};struct Cyc_StringPtr_sa_ScanfArg_struct{int tag;struct _fat_ptr f1;};struct Cyc_DoublePtr_sa_ScanfArg_struct{int tag;double*f1;};struct Cyc_FloatPtr_sa_ScanfArg_struct{int tag;float*f1;};struct Cyc_CharPtr_sa_ScanfArg_struct{int tag;struct _fat_ptr f1;};extern char Cyc_FileCloseError[15U];struct Cyc_FileCloseError_exn_struct{char*tag;};extern char Cyc_FileOpenError[14U];struct Cyc_FileOpenError_exn_struct{char*tag;struct _fat_ptr f1;};
# 278 "cycboot.h"
int Cyc_file_string_write(struct Cyc___cycFILE*,struct _fat_ptr src,int src_offset,int max_count);
# 38 "string.h"
unsigned long Cyc_strlen(struct _fat_ptr s);struct Cyc_PP_Ppstate;struct Cyc_PP_Out;struct Cyc_PP_Doc;struct Cyc_Position_Error;struct Cyc_Relations_Reln;struct _union_Nmspace_Rel_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_Abs_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_C_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_Loc_n{int tag;int val;};union Cyc_Absyn_Nmspace{struct _union_Nmspace_Rel_n Rel_n;struct _union_Nmspace_Abs_n Abs_n;struct _union_Nmspace_C_n C_n;struct _union_Nmspace_Loc_n Loc_n;};struct _tuple0{union Cyc_Absyn_Nmspace f1;struct _fat_ptr*f2;};
# 161 "absyn.h"
enum Cyc_Absyn_Scope{Cyc_Absyn_Static =0U,Cyc_Absyn_Abstract =1U,Cyc_Absyn_Public =2U,Cyc_Absyn_Extern =3U,Cyc_Absyn_ExternC =4U,Cyc_Absyn_Register =5U};struct Cyc_Absyn_Tqual{int print_const: 1;int q_volatile: 1;int q_restrict: 1;int real_const: 1;unsigned loc;};
# 182
enum Cyc_Absyn_Size_of{Cyc_Absyn_Char_sz =0U,Cyc_Absyn_Short_sz =1U,Cyc_Absyn_Int_sz =2U,Cyc_Absyn_Long_sz =3U,Cyc_Absyn_LongLong_sz =4U};
enum Cyc_Absyn_Sign{Cyc_Absyn_Signed =0U,Cyc_Absyn_Unsigned =1U,Cyc_Absyn_None =2U};
enum Cyc_Absyn_AggrKind{Cyc_Absyn_StructA =0U,Cyc_Absyn_UnionA =1U};
# 187
enum Cyc_Absyn_AliasQual{Cyc_Absyn_Aliasable =0U,Cyc_Absyn_Unique =1U,Cyc_Absyn_Top =2U};
# 192
enum Cyc_Absyn_KindQual{Cyc_Absyn_AnyKind =0U,Cyc_Absyn_MemKind =1U,Cyc_Absyn_BoxKind =2U,Cyc_Absyn_RgnKind =3U,Cyc_Absyn_EffKind =4U,Cyc_Absyn_IntKind =5U,Cyc_Absyn_BoolKind =6U,Cyc_Absyn_PtrBndKind =7U};struct Cyc_Absyn_Kind{enum Cyc_Absyn_KindQual kind;enum Cyc_Absyn_AliasQual aliasqual;};struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct{int tag;struct Cyc_Absyn_Kind*f1;};struct Cyc_Absyn_Unknown_kb_Absyn_KindBound_struct{int tag;struct Cyc_Core_Opt*f1;};struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct{int tag;struct Cyc_Core_Opt*f1;struct Cyc_Absyn_Kind*f2;};struct Cyc_Absyn_Tvar{struct _fat_ptr*name;int identity;void*kind;};struct Cyc_Absyn_PtrLoc{unsigned ptr_loc;unsigned rgn_loc;unsigned zt_loc;};struct Cyc_Absyn_PtrAtts{void*rgn;void*nullable;void*bounds;void*zero_term;struct Cyc_Absyn_PtrLoc*ptrloc;};struct Cyc_Absyn_PtrInfo{void*elt_type;struct Cyc_Absyn_Tqual elt_tq;struct Cyc_Absyn_PtrAtts ptr_atts;};struct Cyc_Absyn_VarargInfo{struct _fat_ptr*name;struct Cyc_Absyn_Tqual tq;void*type;int inject;};struct Cyc_Absyn_Star_Absyn_Cap_struct{int tag;};struct Cyc_Absyn_Bot_Absyn_Cap_struct{int tag;};struct Cyc_Absyn_Nat_Absyn_Cap_struct{int tag;struct Cyc_Absyn_Exp*f1;int f2;};struct Cyc_Absyn_RgnEffect{void*name;void*rgn;void*lock;void*unadopt;void*parent;};struct Cyc_Absyn_FnInfo{struct Cyc_List_List*tvars;void*effect;struct Cyc_Absyn_Tqual ret_tqual;void*ret_type;struct Cyc_List_List*args;int c_varargs;struct Cyc_Absyn_VarargInfo*cyc_varargs;struct Cyc_List_List*rgn_po;struct Cyc_List_List*attributes;struct Cyc_Absyn_Exp*requires_clause;struct Cyc_List_List*requires_relns;struct Cyc_Absyn_Exp*ensures_clause;struct Cyc_List_List*ensures_relns;struct Cyc_List_List*ieffect;struct Cyc_List_List*oeffect;};struct Cyc_Absyn_UnknownDatatypeInfo{struct _tuple0*name;int is_extensible;};struct _union_DatatypeInfo_UnknownDatatype{int tag;struct Cyc_Absyn_UnknownDatatypeInfo val;};struct _union_DatatypeInfo_KnownDatatype{int tag;struct Cyc_Absyn_Datatypedecl**val;};union Cyc_Absyn_DatatypeInfo{struct _union_DatatypeInfo_UnknownDatatype UnknownDatatype;struct _union_DatatypeInfo_KnownDatatype KnownDatatype;};struct Cyc_Absyn_UnknownDatatypeFieldInfo{struct _tuple0*datatype_name;struct _tuple0*field_name;int is_extensible;};struct _union_DatatypeFieldInfo_UnknownDatatypefield{int tag;struct Cyc_Absyn_UnknownDatatypeFieldInfo val;};struct _tuple1{struct Cyc_Absyn_Datatypedecl*f1;struct Cyc_Absyn_Datatypefield*f2;};struct _union_DatatypeFieldInfo_KnownDatatypefield{int tag;struct _tuple1 val;};union Cyc_Absyn_DatatypeFieldInfo{struct _union_DatatypeFieldInfo_UnknownDatatypefield UnknownDatatypefield;struct _union_DatatypeFieldInfo_KnownDatatypefield KnownDatatypefield;};struct _tuple2{enum Cyc_Absyn_AggrKind f1;struct _tuple0*f2;struct Cyc_Core_Opt*f3;};struct _union_AggrInfo_UnknownAggr{int tag;struct _tuple2 val;};struct _union_AggrInfo_KnownAggr{int tag;struct Cyc_Absyn_Aggrdecl**val;};union Cyc_Absyn_AggrInfo{struct _union_AggrInfo_UnknownAggr UnknownAggr;struct _union_AggrInfo_KnownAggr KnownAggr;};struct Cyc_Absyn_ArrayInfo{void*elt_type;struct Cyc_Absyn_Tqual tq;struct Cyc_Absyn_Exp*num_elts;void*zero_term;unsigned zt_loc;};struct Cyc_Absyn_Aggr_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Aggrdecl*f1;};struct Cyc_Absyn_Enum_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;};struct Cyc_Absyn_Datatype_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;};struct Cyc_Absyn_TypeDecl{void*r;unsigned loc;};struct Cyc_Absyn_VoidCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_IntCon_Absyn_TyCon_struct{int tag;enum Cyc_Absyn_Sign f1;enum Cyc_Absyn_Size_of f2;};struct Cyc_Absyn_FloatCon_Absyn_TyCon_struct{int tag;int f1;};struct Cyc_Absyn_RgnHandleCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_TagCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_HeapCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_UniqueCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_RefCntCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_AccessCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_JoinCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_RgnsCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_TrueCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_FalseCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_ThinCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_FatCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_EnumCon_Absyn_TyCon_struct{int tag;struct _tuple0*f1;struct Cyc_Absyn_Enumdecl*f2;};struct Cyc_Absyn_AnonEnumCon_Absyn_TyCon_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_BuiltinCon_Absyn_TyCon_struct{int tag;struct _fat_ptr f1;struct Cyc_Absyn_Kind*f2;};struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_DatatypeInfo f1;};struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_DatatypeFieldInfo f1;};struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_AggrInfo f1;};struct Cyc_Absyn_AppType_Absyn_Type_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Evar_Absyn_Type_struct{int tag;struct Cyc_Core_Opt*f1;void*f2;int f3;struct Cyc_Core_Opt*f4;};struct Cyc_Absyn_VarType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Tvar*f1;};struct Cyc_Absyn_PointerType_Absyn_Type_struct{int tag;struct Cyc_Absyn_PtrInfo f1;};struct Cyc_Absyn_ArrayType_Absyn_Type_struct{int tag;struct Cyc_Absyn_ArrayInfo f1;};struct Cyc_Absyn_FnType_Absyn_Type_struct{int tag;struct Cyc_Absyn_FnInfo f1;};struct Cyc_Absyn_TupleType_Absyn_Type_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct{int tag;enum Cyc_Absyn_AggrKind f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_TypedefType_Absyn_Type_struct{int tag;struct _tuple0*f1;struct Cyc_List_List*f2;struct Cyc_Absyn_Typedefdecl*f3;void*f4;};struct Cyc_Absyn_ValueofType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct{int tag;struct Cyc_Absyn_TypeDecl*f1;void**f2;};struct Cyc_Absyn_TypeofType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_NoTypes_Absyn_Funcparams_struct{int tag;struct Cyc_List_List*f1;unsigned f2;};struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct{int tag;struct Cyc_List_List*f1;int f2;struct Cyc_Absyn_VarargInfo*f3;void*f4;struct Cyc_List_List*f5;struct Cyc_Absyn_Exp*f6;struct Cyc_Absyn_Exp*f7;struct Cyc_List_List*f8;struct Cyc_List_List*f9;};
# 437 "absyn.h"
enum Cyc_Absyn_Format_Type{Cyc_Absyn_Printf_ft =0U,Cyc_Absyn_Scanf_ft =1U};struct Cyc_Absyn_Regparm_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Stdcall_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Cdecl_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Fastcall_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Noreturn_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Const_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Aligned_att_Absyn_Attribute_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Packed_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Section_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Nocommon_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Shared_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Unused_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Weak_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Dllimport_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Dllexport_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_instrument_function_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Constructor_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Destructor_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_check_memory_usage_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Format_att_Absyn_Attribute_struct{int tag;enum Cyc_Absyn_Format_Type f1;int f2;int f3;};struct Cyc_Absyn_Initializes_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Noliveunique_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Consume_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Pure_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Mode_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Alias_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Always_inline_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_throw_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Non_null_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Deprecated_att_Absyn_Attribute_struct{int tag;};
# 476
extern struct Cyc_Absyn_Stdcall_att_Absyn_Attribute_struct Cyc_Absyn_Stdcall_att_val;
extern struct Cyc_Absyn_Cdecl_att_Absyn_Attribute_struct Cyc_Absyn_Cdecl_att_val;
extern struct Cyc_Absyn_Fastcall_att_Absyn_Attribute_struct Cyc_Absyn_Fastcall_att_val;
# 484
extern struct Cyc_Absyn_Unused_att_Absyn_Attribute_struct Cyc_Absyn_Unused_att_val;struct Cyc_Absyn_Carray_mod_Absyn_Type_modifier_struct{int tag;void*f1;unsigned f2;};struct Cyc_Absyn_ConstArray_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_Absyn_Exp*f1;void*f2;unsigned f3;};struct Cyc_Absyn_Pointer_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_Absyn_PtrAtts f1;struct Cyc_Absyn_Tqual f2;};struct Cyc_Absyn_Function_mod_Absyn_Type_modifier_struct{int tag;void*f1;};struct Cyc_Absyn_TypeParams_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_List_List*f1;unsigned f2;int f3;};struct Cyc_Absyn_Attributes_mod_Absyn_Type_modifier_struct{int tag;unsigned f1;struct Cyc_List_List*f2;};struct _union_Cnst_Null_c{int tag;int val;};struct _tuple3{enum Cyc_Absyn_Sign f1;char f2;};struct _union_Cnst_Char_c{int tag;struct _tuple3 val;};struct _union_Cnst_Wchar_c{int tag;struct _fat_ptr val;};struct _tuple4{enum Cyc_Absyn_Sign f1;short f2;};struct _union_Cnst_Short_c{int tag;struct _tuple4 val;};struct _tuple5{enum Cyc_Absyn_Sign f1;int f2;};struct _union_Cnst_Int_c{int tag;struct _tuple5 val;};struct _tuple6{enum Cyc_Absyn_Sign f1;long long f2;};struct _union_Cnst_LongLong_c{int tag;struct _tuple6 val;};struct _tuple7{struct _fat_ptr f1;int f2;};struct _union_Cnst_Float_c{int tag;struct _tuple7 val;};struct _union_Cnst_String_c{int tag;struct _fat_ptr val;};struct _union_Cnst_Wstring_c{int tag;struct _fat_ptr val;};union Cyc_Absyn_Cnst{struct _union_Cnst_Null_c Null_c;struct _union_Cnst_Char_c Char_c;struct _union_Cnst_Wchar_c Wchar_c;struct _union_Cnst_Short_c Short_c;struct _union_Cnst_Int_c Int_c;struct _union_Cnst_LongLong_c LongLong_c;struct _union_Cnst_Float_c Float_c;struct _union_Cnst_String_c String_c;struct _union_Cnst_Wstring_c Wstring_c;};
# 532
enum Cyc_Absyn_Primop{Cyc_Absyn_Plus =0U,Cyc_Absyn_Times =1U,Cyc_Absyn_Minus =2U,Cyc_Absyn_Div =3U,Cyc_Absyn_Mod =4U,Cyc_Absyn_Eq =5U,Cyc_Absyn_Neq =6U,Cyc_Absyn_Gt =7U,Cyc_Absyn_Lt =8U,Cyc_Absyn_Gte =9U,Cyc_Absyn_Lte =10U,Cyc_Absyn_Not =11U,Cyc_Absyn_Bitnot =12U,Cyc_Absyn_Bitand =13U,Cyc_Absyn_Bitor =14U,Cyc_Absyn_Bitxor =15U,Cyc_Absyn_Bitlshift =16U,Cyc_Absyn_Bitlrshift =17U,Cyc_Absyn_Numelts =18U};
# 539
enum Cyc_Absyn_Incrementor{Cyc_Absyn_PreInc =0U,Cyc_Absyn_PostInc =1U,Cyc_Absyn_PreDec =2U,Cyc_Absyn_PostDec =3U};struct Cyc_Absyn_VarargCallInfo{int num_varargs;struct Cyc_List_List*injectors;struct Cyc_Absyn_VarargInfo*vai;};struct Cyc_Absyn_StructField_Absyn_OffsetofField_struct{int tag;struct _fat_ptr*f1;};struct Cyc_Absyn_TupleIndex_Absyn_OffsetofField_struct{int tag;unsigned f1;};
# 557
enum Cyc_Absyn_Coercion{Cyc_Absyn_Unknown_coercion =0U,Cyc_Absyn_No_coercion =1U,Cyc_Absyn_Null_to_NonNull =2U,Cyc_Absyn_Other_coercion =3U};struct Cyc_Absyn_MallocInfo{int is_calloc;struct Cyc_Absyn_Exp*rgn;void**elt_type;struct Cyc_Absyn_Exp*num_elts;int fat_result;int inline_call;};struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct{int tag;union Cyc_Absyn_Cnst f1;};struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Pragma_e_Absyn_Raw_exp_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct{int tag;enum Cyc_Absyn_Primop f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_AssignOp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Core_Opt*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_Increment_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;enum Cyc_Absyn_Incrementor f2;};struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;struct Cyc_Absyn_VarargCallInfo*f3;int f4;};struct Cyc_Absyn_Throw_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;int f2;};struct Cyc_Absyn_NoInstantiate_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Instantiate_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_Absyn_Exp*f2;int f3;enum Cyc_Absyn_Coercion f4;};struct Cyc_Absyn_Address_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_New_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Sizeoftype_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Sizeofexp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Offsetof_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Deref_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;int f3;int f4;};struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;int f3;int f4;};struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Tuple_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;};struct _tuple8{struct _fat_ptr*f1;struct Cyc_Absyn_Tqual f2;void*f3;};struct Cyc_Absyn_CompoundLit_e_Absyn_Raw_exp_struct{int tag;struct _tuple8*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Array_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Exp*f2;struct Cyc_Absyn_Exp*f3;int f4;};struct Cyc_Absyn_ComprehensionNoinit_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;void*f2;int f3;};struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct{int tag;struct _tuple0*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;struct Cyc_Absyn_Aggrdecl*f4;};struct Cyc_Absyn_AnonStruct_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Datatype_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;struct Cyc_Absyn_Datatypedecl*f2;struct Cyc_Absyn_Datatypefield*f3;};struct Cyc_Absyn_Enum_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_AnonEnum_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_Spawn_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_MallocInfo f1;};struct Cyc_Absyn_Swap_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_UnresolvedMem_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Core_Opt*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_StmtExp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Stmt*f1;};struct Cyc_Absyn_Tagcheck_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;};struct Cyc_Absyn_Valueof_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Asm_e_Absyn_Raw_exp_struct{int tag;int f1;struct _fat_ptr f2;struct Cyc_List_List*f3;struct Cyc_List_List*f4;struct Cyc_List_List*f5;};struct Cyc_Absyn_Extension_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Exp{void*topt;void*r;unsigned loc;void*annot;};struct Cyc_Absyn_Skip_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Exp_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Seq_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Return_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_IfThenElse_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Stmt*f2;struct Cyc_Absyn_Stmt*f3;};struct _tuple9{struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_While_s_Absyn_Raw_stmt_struct{int tag;struct _tuple9 f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Break_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Continue_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Goto_s_Absyn_Raw_stmt_struct{int tag;struct _fat_ptr*f1;};struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _tuple9 f2;struct _tuple9 f3;struct Cyc_Absyn_Stmt*f4;};struct Cyc_Absyn_Switch_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;void*f3;};struct Cyc_Absyn_Fallthru_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_List_List*f1;struct Cyc_Absyn_Switch_clause**f2;};struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Decl*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Label_s_Absyn_Raw_stmt_struct{int tag;struct _fat_ptr*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Do_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct _tuple9 f2;};struct Cyc_Absyn_TryCatch_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct Cyc_List_List*f2;void*f3;};struct Cyc_Absyn_Stmt{void*r;unsigned loc;void*annot;};struct Cyc_Absyn_Wild_p_Absyn_Raw_pat_struct{int tag;};struct Cyc_Absyn_Var_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Pat*f2;};struct Cyc_Absyn_AliasVar_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;};struct Cyc_Absyn_Reference_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Pat*f2;};struct Cyc_Absyn_TagInt_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;};struct Cyc_Absyn_Tuple_p_Absyn_Raw_pat_struct{int tag;struct Cyc_List_List*f1;int f2;};struct Cyc_Absyn_Pointer_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Pat*f1;};struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct{int tag;union Cyc_Absyn_AggrInfo*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;int f4;};struct Cyc_Absyn_Datatype_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;struct Cyc_Absyn_Datatypefield*f2;struct Cyc_List_List*f3;int f4;};struct Cyc_Absyn_Null_p_Absyn_Raw_pat_struct{int tag;};struct Cyc_Absyn_Int_p_Absyn_Raw_pat_struct{int tag;enum Cyc_Absyn_Sign f1;int f2;};struct Cyc_Absyn_Char_p_Absyn_Raw_pat_struct{int tag;char f1;};struct Cyc_Absyn_Float_p_Absyn_Raw_pat_struct{int tag;struct _fat_ptr f1;int f2;};struct Cyc_Absyn_Enum_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_AnonEnum_p_Absyn_Raw_pat_struct{int tag;void*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_UnknownId_p_Absyn_Raw_pat_struct{int tag;struct _tuple0*f1;};struct Cyc_Absyn_UnknownCall_p_Absyn_Raw_pat_struct{int tag;struct _tuple0*f1;struct Cyc_List_List*f2;int f3;};struct Cyc_Absyn_Exp_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Exp*f1;};
# 725 "absyn.h"
extern struct Cyc_Absyn_Wild_p_Absyn_Raw_pat_struct Cyc_Absyn_Wild_p_val;struct Cyc_Absyn_Pat{void*r;void*topt;unsigned loc;};struct Cyc_Absyn_Switch_clause{struct Cyc_Absyn_Pat*pattern;struct Cyc_Core_Opt*pat_vars;struct Cyc_Absyn_Exp*where_clause;struct Cyc_Absyn_Stmt*body;unsigned loc;};struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct{int tag;struct _tuple0*f1;};struct Cyc_Absyn_Global_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Funname_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Fndecl*f1;};struct Cyc_Absyn_Param_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Local_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Pat_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Vardecl{enum Cyc_Absyn_Scope sc;struct _tuple0*name;unsigned varloc;struct Cyc_Absyn_Tqual tq;void*type;struct Cyc_Absyn_Exp*initializer;void*rgn;struct Cyc_List_List*attributes;int escapes;};struct Cyc_Absyn_Fndecl{enum Cyc_Absyn_Scope sc;int is_inline;struct _tuple0*name;struct Cyc_Absyn_Stmt*body;struct Cyc_Absyn_FnInfo i;void*cached_type;struct Cyc_Core_Opt*param_vardecls;struct Cyc_Absyn_Vardecl*fn_vardecl;};struct Cyc_Absyn_Aggrfield{struct _fat_ptr*name;struct Cyc_Absyn_Tqual tq;void*type;struct Cyc_Absyn_Exp*width;struct Cyc_List_List*attributes;struct Cyc_Absyn_Exp*requires_clause;};struct Cyc_Absyn_AggrdeclImpl{struct Cyc_List_List*exist_vars;struct Cyc_List_List*rgn_po;struct Cyc_List_List*fields;int tagged;};struct Cyc_Absyn_Aggrdecl{enum Cyc_Absyn_AggrKind kind;enum Cyc_Absyn_Scope sc;struct _tuple0*name;struct Cyc_List_List*tvs;struct Cyc_Absyn_AggrdeclImpl*impl;struct Cyc_List_List*attributes;int expected_mem_kind;};struct Cyc_Absyn_Datatypefield{struct _tuple0*name;struct Cyc_List_List*typs;unsigned loc;enum Cyc_Absyn_Scope sc;};struct Cyc_Absyn_Datatypedecl{enum Cyc_Absyn_Scope sc;struct _tuple0*name;struct Cyc_List_List*tvs;struct Cyc_Core_Opt*fields;int is_extensible;};struct Cyc_Absyn_Enumfield{struct _tuple0*name;struct Cyc_Absyn_Exp*tag;unsigned loc;};struct Cyc_Absyn_Enumdecl{enum Cyc_Absyn_Scope sc;struct _tuple0*name;struct Cyc_Core_Opt*fields;};struct Cyc_Absyn_Typedefdecl{struct _tuple0*name;struct Cyc_Absyn_Tqual tq;struct Cyc_List_List*tvs;struct Cyc_Core_Opt*kind;void*defn;struct Cyc_List_List*atts;int extern_c;};struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Fn_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Fndecl*f1;};struct Cyc_Absyn_Let_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Pat*f1;struct Cyc_Core_Opt*f2;struct Cyc_Absyn_Exp*f3;void*f4;};struct Cyc_Absyn_Letv_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_Region_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_Aggr_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Aggrdecl*f1;};struct Cyc_Absyn_Datatype_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;};struct Cyc_Absyn_Enum_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;};struct Cyc_Absyn_Typedef_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Typedefdecl*f1;};struct Cyc_Absyn_Namespace_d_Absyn_Raw_decl_struct{int tag;struct _fat_ptr*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Using_d_Absyn_Raw_decl_struct{int tag;struct _tuple0*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_ExternC_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;};struct _tuple10{unsigned f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_ExternCinclude_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;struct _tuple10*f4;};struct Cyc_Absyn_Porton_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Portoff_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Tempeston_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Tempestoff_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Decl{void*r;unsigned loc;};struct Cyc_Absyn_ArrayElement_Absyn_Designator_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_FieldName_Absyn_Designator_struct{int tag;struct _fat_ptr*f1;};extern char Cyc_Absyn_EmptyAnnot[11U];struct Cyc_Absyn_EmptyAnnot_Absyn_AbsynAnnot_struct{char*tag;};
# 917
struct Cyc_Absyn_Tqual Cyc_Absyn_empty_tqual(unsigned);
# 922
void*Cyc_Absyn_compress_kb(void*);
# 928
int Cyc_Absyn_type2bool(int def,void*);
# 933
void*Cyc_Absyn_new_evar(struct Cyc_Core_Opt*k,struct Cyc_Core_Opt*tenv);
# 1037
struct Cyc_Absyn_Exp*Cyc_Absyn_times_exp(struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*,unsigned);
# 1060
struct Cyc_Absyn_Exp*Cyc_Absyn_sizeoftype_exp(void*t,unsigned);
# 1164
struct _fat_ptr Cyc_Absyn_attribute2string(void*);struct _tuple11{enum Cyc_Absyn_AggrKind f1;struct _tuple0*f2;};
# 1174
struct _tuple11 Cyc_Absyn_aggr_kinded_name(union Cyc_Absyn_AggrInfo);
# 1182
struct _tuple0*Cyc_Absyn_binding2qvar(void*);struct Cyc_Absynpp_Params{int expand_typedefs;int qvar_to_Cids;int add_cyc_prefix;int to_VC;int decls_first;int rewrite_temp_tvars;int print_all_tvars;int print_all_kinds;int print_all_effects;int print_using_stmts;int print_externC_stmts;int print_full_evars;int print_zeroterm;int generate_line_directives;int use_curr_namespace;struct Cyc_List_List*curr_namespace;};
# 53 "absynpp.h"
void Cyc_Absynpp_set_params(struct Cyc_Absynpp_Params*fs);
# 61
struct _fat_ptr Cyc_Absynpp_longlong2string(unsigned long long);
# 64
struct _fat_ptr Cyc_Absynpp_kind2string(struct Cyc_Absyn_Kind*);
# 78
extern struct _fat_ptr*Cyc_Absynpp_cyc_stringptr;
int Cyc_Absynpp_exp_prec(struct Cyc_Absyn_Exp*);
struct _fat_ptr Cyc_Absynpp_char_escape(char);
struct _fat_ptr Cyc_Absynpp_string_escape(struct _fat_ptr);
struct _fat_ptr Cyc_Absynpp_prim2str(enum Cyc_Absyn_Primop p);
int Cyc_Absynpp_is_declaration(struct Cyc_Absyn_Stmt*s);struct _tuple12{struct Cyc_Absyn_Tqual f1;void*f2;struct Cyc_List_List*f3;};
struct _tuple12 Cyc_Absynpp_to_tms(struct _RegionHandle*,struct Cyc_Absyn_Tqual tq,void*t);struct _tuple13{int f1;struct Cyc_List_List*f2;};
# 93 "absynpp.h"
struct _tuple13 Cyc_Absynpp_shadows(struct Cyc_Absyn_Decl*d,struct Cyc_List_List*varsinblock);struct _tuple14{unsigned f1;int f2;};
# 28 "evexp.h"
struct _tuple14 Cyc_Evexp_eval_const_uint_exp(struct Cyc_Absyn_Exp*e);struct _union_RelnOp_RConst{int tag;unsigned val;};struct _union_RelnOp_RVar{int tag;struct Cyc_Absyn_Vardecl*val;};struct _union_RelnOp_RNumelts{int tag;struct Cyc_Absyn_Vardecl*val;};struct _union_RelnOp_RType{int tag;void*val;};struct _union_RelnOp_RParam{int tag;unsigned val;};struct _union_RelnOp_RParamNumelts{int tag;unsigned val;};struct _union_RelnOp_RReturn{int tag;unsigned val;};union Cyc_Relations_RelnOp{struct _union_RelnOp_RConst RConst;struct _union_RelnOp_RVar RVar;struct _union_RelnOp_RNumelts RNumelts;struct _union_RelnOp_RType RType;struct _union_RelnOp_RParam RParam;struct _union_RelnOp_RParamNumelts RParamNumelts;struct _union_RelnOp_RReturn RReturn;};
# 50 "relations-ap.h"
enum Cyc_Relations_Relation{Cyc_Relations_Req =0U,Cyc_Relations_Rneq =1U,Cyc_Relations_Rlte =2U,Cyc_Relations_Rlt =3U};struct Cyc_Relations_Reln{union Cyc_Relations_RelnOp rop1;enum Cyc_Relations_Relation relation;union Cyc_Relations_RelnOp rop2;};struct Cyc_Iter_Iter{void*env;int(*next)(void*env,void*dest);};struct Cyc_Dict_T;struct Cyc_Dict_Dict{int(*rel)(void*,void*);struct _RegionHandle*r;const struct Cyc_Dict_T*t;};extern char Cyc_Dict_Present[8U];struct Cyc_Dict_Present_exn_struct{char*tag;};extern char Cyc_Dict_Absent[7U];struct Cyc_Dict_Absent_exn_struct{char*tag;};struct Cyc_RgnOrder_RgnPO;extern char Cyc_Tcenv_Env_error[10U];struct Cyc_Tcenv_Env_error_exn_struct{char*tag;};struct Cyc_Tcenv_Genv{struct Cyc_Dict_Dict aggrdecls;struct Cyc_Dict_Dict datatypedecls;struct Cyc_Dict_Dict enumdecls;struct Cyc_Dict_Dict typedefs;struct Cyc_Dict_Dict ordinaries;};struct Cyc_Tcenv_Fenv;struct Cyc_Tcenv_Tenv{struct Cyc_List_List*ns;struct Cyc_Tcenv_Genv*ae;struct Cyc_Tcenv_Fenv*le;int allow_valueof: 1;int in_extern_c_include: 1;int in_tempest: 1;int tempest_generalize: 1;};
# 88 "tcenv.h"
enum Cyc_Tcenv_NewStatus{Cyc_Tcenv_NoneNew =0U,Cyc_Tcenv_InNew =1U,Cyc_Tcenv_InNewAggr =2U};
# 107 "tcutil.h"
void*Cyc_Tcutil_compress(void*);
# 40 "warn.h"
void*Cyc_Warn_impos(struct _fat_ptr fmt,struct _fat_ptr ap);struct Cyc_Warn_String_Warn_Warg_struct{int tag;struct _fat_ptr f1;};struct Cyc_Warn_Exp_Warn_Warg_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Warn_Typ_Warn_Warg_struct{int tag;void*f1;};struct Cyc_Warn_Qvar_Warn_Warg_struct{int tag;struct _tuple0*f1;};
# 26 "cyclone.h"
enum Cyc_Cyclone_C_Compilers{Cyc_Cyclone_Gcc_c =0U,Cyc_Cyclone_Vc_c =1U};
# 32
extern enum Cyc_Cyclone_C_Compilers Cyc_Cyclone_c_compiler;struct _tuple15{struct _fat_ptr f1;unsigned f2;};
# 27 "absyndump.cyc"
struct _tuple15 Cyc_Lex_xlate_pos(unsigned);
# 37
static int Cyc_Absyndump_expand_typedefs;
# 41
static int Cyc_Absyndump_qvar_to_Cids;
# 44
static int Cyc_Absyndump_add_cyc_prefix;
# 47
static int Cyc_Absyndump_generate_line_directives;
# 51
static int Cyc_Absyndump_to_VC;
# 53
void Cyc_Absyndump_set_params(struct Cyc_Absynpp_Params*fs){
Cyc_Absyndump_expand_typedefs=fs->expand_typedefs;
Cyc_Absyndump_qvar_to_Cids=fs->qvar_to_Cids;
Cyc_Absyndump_add_cyc_prefix=fs->add_cyc_prefix;
Cyc_Absyndump_to_VC=fs->to_VC;
Cyc_Absyndump_generate_line_directives=fs->generate_line_directives;
# 68 "absyndump.cyc"
Cyc_Absynpp_set_params(fs);}
# 53 "absyndump.cyc"
static void Cyc_Absyndump_dumptyp(void*);
# 72 "absyndump.cyc"
static void Cyc_Absyndump_dumpntyp(void*t);
static void Cyc_Absyndump_dumpexp(struct Cyc_Absyn_Exp*);
static void Cyc_Absyndump_dumpexp_prec(int,struct Cyc_Absyn_Exp*);
static void Cyc_Absyndump_dumppat(struct Cyc_Absyn_Pat*);
static void Cyc_Absyndump_dumpstmt(struct Cyc_Absyn_Stmt*,int expstmt,struct Cyc_List_List*);
static void Cyc_Absyndump_dumpvardecl(struct Cyc_Absyn_Vardecl*,unsigned,int do_semi);
static void Cyc_Absyndump_dump_aggrdecl(struct Cyc_Absyn_Aggrdecl*);
static void Cyc_Absyndump_dump_enumdecl(struct Cyc_Absyn_Enumdecl*);
static void Cyc_Absyndump_dump_datatypedecl(struct Cyc_Absyn_Datatypedecl*);
static void Cyc_Absyndump_dumpdecl(struct Cyc_Absyn_Decl*);
static void Cyc_Absyndump_dumptms(int is_char_ptr,struct Cyc_List_List*,void(*f)(void*),void*a);
static void Cyc_Absyndump_dumptqtd(struct Cyc_Absyn_Tqual,void*,void(*f)(void*),void*);
static void Cyc_Absyndump_dumpaggrfields(struct Cyc_List_List*fields);
static void Cyc_Absyndump_dumpenumfields(struct Cyc_List_List*fields);
# 90
static void Cyc_Absyndump_dumploc(unsigned);
# 93
struct Cyc___cycFILE**Cyc_Absyndump_dump_file=& Cyc_stdout;
static char Cyc_Absyndump_prev_char='x';
# 96
static int Cyc_Absyndump_need_space_before(){
char _tmp0=Cyc_Absyndump_prev_char;switch(_tmp0){case 123U: _LL1: _LL2:
 goto _LL4;case 125U: _LL3: _LL4:
 goto _LL6;case 40U: _LL5: _LL6:
 goto _LL8;case 41U: _LL7: _LL8:
 goto _LLA;case 91U: _LL9: _LLA:
 goto _LLC;case 93U: _LLB: _LLC:
 goto _LLE;case 59U: _LLD: _LLE:
 goto _LL10;case 44U: _LLF: _LL10:
 goto _LL12;case 61U: _LL11: _LL12:
 goto _LL14;case 63U: _LL13: _LL14:
 goto _LL16;case 33U: _LL15: _LL16:
 goto _LL18;case 32U: _LL17: _LL18:
 goto _LL1A;case 10U: _LL19: _LL1A:
 goto _LL1C;case 42U: _LL1B: _LL1C:
 return 0;default: _LL1D: _LL1E:
 return 1;}_LL0:;}
# 96
static void Cyc_Absyndump_dump(struct _fat_ptr s){
# 117
unsigned sz=Cyc_strlen((struct _fat_ptr)s);
# 123
if(Cyc_Absyndump_need_space_before())
Cyc_fputc((int)' ',*Cyc_Absyndump_dump_file);
# 123
if(sz >= (unsigned)1){
# 126
Cyc_Absyndump_prev_char=*((const char*)_check_fat_subscript(s,sizeof(char),(int)(sz - (unsigned)1)));
Cyc_file_string_write(*Cyc_Absyndump_dump_file,s,0,(int)sz);}}
# 96
static void Cyc_Absyndump_dump_nospace(struct _fat_ptr s){
# 132
int sz=(int)Cyc_strlen((struct _fat_ptr)s);
# 134
if(sz >= 1){
Cyc_file_string_write(*Cyc_Absyndump_dump_file,s,0,sz);
Cyc_Absyndump_prev_char=*((const char*)_check_fat_subscript(s,sizeof(char),sz - 1));}}
# 96
static void Cyc_Absyndump_dump_char(int c){
# 141
Cyc_fputc(c,*Cyc_Absyndump_dump_file);
Cyc_Absyndump_prev_char=(char)c;}static char _tmpC[1U]="";
# 96
static void Cyc_Absyndump_dumploc(unsigned loc){
# 146
static struct _fat_ptr last_file={_tmpC,_tmpC,_tmpC + 1U};
static unsigned last_line=0U;
if(loc == (unsigned)0)return;if(!Cyc_Absyndump_generate_line_directives)
return;{
# 148
struct _tuple15 _stmttmp0=
# 151
Cyc_Lex_xlate_pos(loc);
# 148
unsigned _tmp2;struct _fat_ptr _tmp1;_LL1: _tmp1=_stmttmp0.f1;_tmp2=_stmttmp0.f2;_LL2: {struct _fat_ptr f=_tmp1;unsigned d=_tmp2;
# 152
if(({char*_tmp2F0=(char*)f.curr;_tmp2F0 == (char*)(_tag_fat(0,0,0)).curr;})||(char*)f.curr == (char*)last_file.curr && d == last_line)return;if(
(char*)f.curr == (char*)last_file.curr && d == last_line + (unsigned)1)
({void*_tmp3=0U;({struct Cyc___cycFILE*_tmp2F2=*Cyc_Absyndump_dump_file;struct _fat_ptr _tmp2F1=({const char*_tmp4="\n";_tag_fat(_tmp4,sizeof(char),2U);});Cyc_fprintf(_tmp2F2,_tmp2F1,_tag_fat(_tmp3,sizeof(void*),0U));});});else{
if((char*)f.curr == (char*)last_file.curr)
({struct Cyc_Int_pa_PrintArg_struct _tmp7=({struct Cyc_Int_pa_PrintArg_struct _tmp2E3;_tmp2E3.tag=1U,_tmp2E3.f1=(unsigned long)((int)d);_tmp2E3;});void*_tmp5[1U];_tmp5[0]=& _tmp7;({struct Cyc___cycFILE*_tmp2F4=*Cyc_Absyndump_dump_file;struct _fat_ptr _tmp2F3=({const char*_tmp6="\n# %d\n";_tag_fat(_tmp6,sizeof(char),7U);});Cyc_fprintf(_tmp2F4,_tmp2F3,_tag_fat(_tmp5,sizeof(void*),1U));});});else{
# 158
({struct Cyc_Int_pa_PrintArg_struct _tmpA=({struct Cyc_Int_pa_PrintArg_struct _tmp2E5;_tmp2E5.tag=1U,_tmp2E5.f1=(unsigned long)((int)d);_tmp2E5;});struct Cyc_String_pa_PrintArg_struct _tmpB=({struct Cyc_String_pa_PrintArg_struct _tmp2E4;_tmp2E4.tag=0U,_tmp2E4.f1=(struct _fat_ptr)((struct _fat_ptr)f);_tmp2E4;});void*_tmp8[2U];_tmp8[0]=& _tmpA,_tmp8[1]=& _tmpB;({struct Cyc___cycFILE*_tmp2F6=*Cyc_Absyndump_dump_file;struct _fat_ptr _tmp2F5=({const char*_tmp9="\n# %d %s\n";_tag_fat(_tmp9,sizeof(char),10U);});Cyc_fprintf(_tmp2F6,_tmp2F5,_tag_fat(_tmp8,sizeof(void*),2U));});});}}
# 160
last_file=f;
last_line=d;}}}
# 96
static void Cyc_Absyndump_dump_str(struct _fat_ptr*s){
# 167
Cyc_Absyndump_dump(*s);}
# 96
static void Cyc_Absyndump_dump_str_nospace(struct _fat_ptr*s){
# 170
Cyc_Absyndump_dump_nospace(*s);}
# 96
static void Cyc_Absyndump_dump_semi(){
# 174
Cyc_Absyndump_dump_char((int)';');}
# 96
static void Cyc_Absyndump_dump_sep(void(*f)(void*),struct Cyc_List_List*l,struct _fat_ptr sep){
# 178
if(l == 0)
return;
# 178
for(0;((struct Cyc_List_List*)_check_null(l))->tl != 0;l=l->tl){
# 181
f(l->hd);
Cyc_Absyndump_dump_nospace(sep);}
# 184
f(l->hd);}
# 96
static void Cyc_Absyndump_dump_sep_c(void(*f)(void*,void*),void*env,struct Cyc_List_List*l,struct _fat_ptr sep){
# 187
if(l == 0)
return;
# 187
for(0;((struct Cyc_List_List*)_check_null(l))->tl != 0;l=l->tl){
# 190
f(env,l->hd);
Cyc_Absyndump_dump_nospace(sep);}
# 193
f(env,l->hd);}
# 96
static void Cyc_Absyndump_group(void(*f)(void*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep){
# 197
Cyc_Absyndump_dump_nospace(start);
Cyc_Absyndump_dump_sep(f,l,sep);
Cyc_Absyndump_dump_nospace(end);}
# 96
static void Cyc_Absyndump_group_c(void(*f)(void*,void*),void*env,struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep){
# 203
Cyc_Absyndump_dump_nospace(start);
Cyc_Absyndump_dump_sep_c(f,env,l,sep);
Cyc_Absyndump_dump_nospace(end);}
# 96
static void Cyc_Absyndump_egroup(void(*f)(void*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep){
# 209
if(l != 0)
Cyc_Absyndump_group(f,l,start,end,sep);}
# 96
static void Cyc_Absyndump_ignore(void*x){
# 213
;}
# 96
static void Cyc_Absyndump_dumpqvar(struct _tuple0*v){
# 216
struct Cyc_List_List*nsl=0;
struct _fat_ptr**prefix=0;
{union Cyc_Absyn_Nmspace _stmttmp1=(*v).f1;union Cyc_Absyn_Nmspace _tmpD=_stmttmp1;struct Cyc_List_List*_tmpE;struct Cyc_List_List*_tmpF;struct Cyc_List_List*_tmp10;switch((_tmpD.C_n).tag){case 4U: _LL1: _LL2:
 goto _LL0;case 1U: _LL3: _tmp10=(_tmpD.Rel_n).val;_LL4: {struct Cyc_List_List*x=_tmp10;
nsl=x;goto _LL0;}case 3U: _LL5: _tmpF=(_tmpD.C_n).val;_LL6: {struct Cyc_List_List*x=_tmpF;
goto _LL0;}default: _LL7: _tmpE=(_tmpD.Abs_n).val;_LL8: {struct Cyc_List_List*x=_tmpE;
# 223
if(Cyc_Absyndump_qvar_to_Cids || Cyc_Absyndump_add_cyc_prefix)
prefix=& Cyc_Absynpp_cyc_stringptr;
# 223
nsl=x;
# 226
goto _LL0;}}_LL0:;}
# 228
if(Cyc_Absyndump_need_space_before())
Cyc_Absyndump_dump_char((int)' ');{
# 228
struct _fat_ptr sep=
# 230
Cyc_Absyndump_qvar_to_Cids?({const char*_tmp12="_";_tag_fat(_tmp12,sizeof(char),2U);}):({const char*_tmp13="::";_tag_fat(_tmp13,sizeof(char),3U);});
if(prefix != 0){
Cyc_Absyndump_dump_nospace(*(*prefix));Cyc_Absyndump_dump_nospace(sep);}
# 231
if(nsl != 0){
# 235
({void(*_tmp2F8)(void(*f)(struct _fat_ptr*),struct Cyc_List_List*l,struct _fat_ptr sep)=({void(*_tmp11)(void(*f)(struct _fat_ptr*),struct Cyc_List_List*l,struct _fat_ptr sep)=(void(*)(void(*f)(struct _fat_ptr*),struct Cyc_List_List*l,struct _fat_ptr sep))Cyc_Absyndump_dump_sep;_tmp11;});struct Cyc_List_List*_tmp2F7=nsl;_tmp2F8(Cyc_Absyndump_dump_str_nospace,_tmp2F7,sep);});Cyc_Absyndump_dump_nospace(sep);}
# 231
Cyc_Absyndump_dump_nospace(*(*v).f2);}}
# 96
static void Cyc_Absyndump_dumptq(struct Cyc_Absyn_Tqual tq){
# 241
if(tq.q_restrict)Cyc_Absyndump_dump(({const char*_tmp14="restrict";_tag_fat(_tmp14,sizeof(char),9U);}));if(tq.q_volatile)
Cyc_Absyndump_dump(({const char*_tmp15="volatile";_tag_fat(_tmp15,sizeof(char),9U);}));
# 241
if(tq.print_const)
# 243
Cyc_Absyndump_dump(({const char*_tmp16="const";_tag_fat(_tmp16,sizeof(char),6U);}));}
# 96
static void Cyc_Absyndump_dumpscope(enum Cyc_Absyn_Scope sc){
# 246
enum Cyc_Absyn_Scope _tmp17=sc;switch(_tmp17){case Cyc_Absyn_Static: _LL1: _LL2:
 Cyc_Absyndump_dump(({const char*_tmp18="static";_tag_fat(_tmp18,sizeof(char),7U);}));return;case Cyc_Absyn_Public: _LL3: _LL4:
 return;case Cyc_Absyn_Extern: _LL5: _LL6:
 Cyc_Absyndump_dump(({const char*_tmp19="extern";_tag_fat(_tmp19,sizeof(char),7U);}));return;case Cyc_Absyn_ExternC: _LL7: _LL8:
 Cyc_Absyndump_dump(({const char*_tmp1A="extern \"C\"";_tag_fat(_tmp1A,sizeof(char),11U);}));return;case Cyc_Absyn_Abstract: _LL9: _LLA:
 Cyc_Absyndump_dump(({const char*_tmp1B="abstract";_tag_fat(_tmp1B,sizeof(char),9U);}));return;case Cyc_Absyn_Register: _LLB: _LLC:
 Cyc_Absyndump_dump(({const char*_tmp1C="register";_tag_fat(_tmp1C,sizeof(char),9U);}));return;default: _LLD: _LLE:
 return;}_LL0:;}
# 96
static void Cyc_Absyndump_dumpaggr_kind(enum Cyc_Absyn_AggrKind k){
# 257
enum Cyc_Absyn_AggrKind _tmp1D=k;if(_tmp1D == Cyc_Absyn_StructA){_LL1: _LL2:
 Cyc_Absyndump_dump(({const char*_tmp1E="struct";_tag_fat(_tmp1E,sizeof(char),7U);}));return;}else{_LL3: _LL4:
 Cyc_Absyndump_dump(({const char*_tmp1F="union";_tag_fat(_tmp1F,sizeof(char),6U);}));return;}_LL0:;}
# 96
static void Cyc_Absyndump_dumpkind(struct Cyc_Absyn_Kind*ka){
# 264
Cyc_Absyndump_dump(Cyc_Absynpp_kind2string(ka));}
# 96
static void Cyc_Absyndump_dumptps(struct Cyc_List_List*ts){
# 268
({struct Cyc_List_List*_tmp2FB=ts;struct _fat_ptr _tmp2FA=({const char*_tmp20="<";_tag_fat(_tmp20,sizeof(char),2U);});struct _fat_ptr _tmp2F9=({const char*_tmp21=">";_tag_fat(_tmp21,sizeof(char),2U);});Cyc_Absyndump_egroup(Cyc_Absyndump_dumptyp,_tmp2FB,_tmp2FA,_tmp2F9,({const char*_tmp22=",";_tag_fat(_tmp22,sizeof(char),2U);}));});}
# 96
static void Cyc_Absyndump_dumptvar(struct Cyc_Absyn_Tvar*tv){
# 271
struct _fat_ptr n=*tv->name;
if((int)*((const char*)_check_fat_subscript(n,sizeof(char),0))== (int)'#'){
Cyc_Absyndump_dump(({const char*_tmp23="`G";_tag_fat(_tmp23,sizeof(char),3U);}));
{void*_stmttmp2=Cyc_Absyn_compress_kb(tv->kind);void*_tmp24=_stmttmp2;struct Cyc_Absyn_Kind*_tmp25;struct Cyc_Absyn_Kind*_tmp26;switch(*((int*)_tmp24)){case 0U: _LL1: _tmp26=((struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct*)_tmp24)->f1;_LL2: {struct Cyc_Absyn_Kind*k=_tmp26;
_tmp25=k;goto _LL4;}case 2U: _LL3: _tmp25=((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp24)->f2;_LL4: {struct Cyc_Absyn_Kind*k=_tmp25;
Cyc_Absyndump_dumpkind(k);goto _LL0;}default: _LL5: _LL6:
 Cyc_Absyndump_dump_nospace(({const char*_tmp27="K";_tag_fat(_tmp27,sizeof(char),2U);}));goto _LL0;}_LL0:;}
# 279
Cyc_Absyndump_dump_nospace(_fat_ptr_plus(n,sizeof(char),1));}else{
# 281
Cyc_Absyndump_dump(n);}}
# 96
static void Cyc_Absyndump_dumpkindedtvar(struct Cyc_Absyn_Tvar*tv){
# 284
Cyc_Absyndump_dumptvar(tv);{
void*_stmttmp3=Cyc_Absyn_compress_kb(tv->kind);void*_tmp28=_stmttmp3;struct Cyc_Absyn_Kind*_tmp29;switch(*((int*)_tmp28)){case 1U: _LL1: _LL2:
 goto _LL4;case 2U: _LL3: _LL4:
 goto _LL6;default: if(((struct Cyc_Absyn_Kind*)((struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct*)_tmp28)->f1)->kind == Cyc_Absyn_BoxKind){if(((struct Cyc_Absyn_Kind*)((struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct*)_tmp28)->f1)->aliasqual == Cyc_Absyn_Aliasable){_LL5: _LL6:
 goto _LL0;}else{goto _LL7;}}else{_LL7: _tmp29=((struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct*)_tmp28)->f1;_LL8: {struct Cyc_Absyn_Kind*k=_tmp29;
Cyc_Absyndump_dump(({const char*_tmp2A="::";_tag_fat(_tmp2A,sizeof(char),3U);}));Cyc_Absyndump_dumpkind(k);goto _LL0;}}}_LL0:;}}
# 96
static void Cyc_Absyndump_dumptvars(struct Cyc_List_List*tvs){
# 293
({void(*_tmp2FF)(void(*f)(struct Cyc_Absyn_Tvar*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep)=({void(*_tmp2B)(void(*f)(struct Cyc_Absyn_Tvar*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep)=(void(*)(void(*f)(struct Cyc_Absyn_Tvar*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep))Cyc_Absyndump_egroup;_tmp2B;});struct Cyc_List_List*_tmp2FE=tvs;struct _fat_ptr _tmp2FD=({const char*_tmp2C="<";_tag_fat(_tmp2C,sizeof(char),2U);});struct _fat_ptr _tmp2FC=({const char*_tmp2D=">";_tag_fat(_tmp2D,sizeof(char),2U);});_tmp2FF(Cyc_Absyndump_dumptvar,_tmp2FE,_tmp2FD,_tmp2FC,({const char*_tmp2E=",";_tag_fat(_tmp2E,sizeof(char),2U);}));});}
# 96
static void Cyc_Absyndump_dumpkindedtvars(struct Cyc_List_List*tvs){
# 296
({void(*_tmp303)(void(*f)(struct Cyc_Absyn_Tvar*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep)=({void(*_tmp2F)(void(*f)(struct Cyc_Absyn_Tvar*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep)=(void(*)(void(*f)(struct Cyc_Absyn_Tvar*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep))Cyc_Absyndump_egroup;_tmp2F;});struct Cyc_List_List*_tmp302=tvs;struct _fat_ptr _tmp301=({const char*_tmp30="<";_tag_fat(_tmp30,sizeof(char),2U);});struct _fat_ptr _tmp300=({const char*_tmp31=">";_tag_fat(_tmp31,sizeof(char),2U);});_tmp303(Cyc_Absyndump_dumpkindedtvar,_tmp302,_tmp301,_tmp300,({const char*_tmp32=",";_tag_fat(_tmp32,sizeof(char),2U);}));});}struct _tuple16{struct Cyc_Absyn_Tqual f1;void*f2;};
# 96
static void Cyc_Absyndump_dumparg(struct _tuple16*pr){
# 299
({void(*_tmp306)(struct Cyc_Absyn_Tqual,void*,void(*f)(int),int)=({void(*_tmp33)(struct Cyc_Absyn_Tqual,void*,void(*f)(int),int)=(void(*)(struct Cyc_Absyn_Tqual,void*,void(*f)(int),int))Cyc_Absyndump_dumptqtd;_tmp33;});struct Cyc_Absyn_Tqual _tmp305=(*pr).f1;void*_tmp304=(*pr).f2;_tmp306(_tmp305,_tmp304,({void(*_tmp34)(int x)=(void(*)(int x))Cyc_Absyndump_ignore;_tmp34;}),0);});}
# 96
static void Cyc_Absyndump_dumpargs(struct Cyc_List_List*ts){
# 302
({void(*_tmp30A)(void(*f)(struct _tuple16*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep)=({void(*_tmp35)(void(*f)(struct _tuple16*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep)=(void(*)(void(*f)(struct _tuple16*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep))Cyc_Absyndump_group;_tmp35;});struct Cyc_List_List*_tmp309=ts;struct _fat_ptr _tmp308=({const char*_tmp36="(";_tag_fat(_tmp36,sizeof(char),2U);});struct _fat_ptr _tmp307=({const char*_tmp37=")";_tag_fat(_tmp37,sizeof(char),2U);});_tmp30A(Cyc_Absyndump_dumparg,_tmp309,_tmp308,_tmp307,({const char*_tmp38=",";_tag_fat(_tmp38,sizeof(char),2U);}));});}
# 96
static void Cyc_Absyndump_dump_callconv(struct Cyc_List_List*atts){
# 306
for(0;atts != 0;atts=atts->tl){
void*_stmttmp4=(void*)atts->hd;void*_tmp39=_stmttmp4;switch(*((int*)_tmp39)){case 1U: _LL1: _LL2:
 Cyc_Absyndump_dump(({const char*_tmp3A="_stdcall";_tag_fat(_tmp3A,sizeof(char),9U);}));return;case 2U: _LL3: _LL4:
 Cyc_Absyndump_dump(({const char*_tmp3B="_cdecl";_tag_fat(_tmp3B,sizeof(char),7U);}));return;case 3U: _LL5: _LL6:
 Cyc_Absyndump_dump(({const char*_tmp3C="_fastcall";_tag_fat(_tmp3C,sizeof(char),10U);}));return;default: _LL7: _LL8:
 goto _LL0;}_LL0:;}}
# 96
static void Cyc_Absyndump_dump_noncallconv(struct Cyc_List_List*atts){
# 317
int hasatt=0;
{struct Cyc_List_List*atts2=atts;for(0;atts2 != 0;atts2=atts2->tl){
void*_stmttmp5=(void*)((struct Cyc_List_List*)_check_null(atts))->hd;void*_tmp3D=_stmttmp5;switch(*((int*)_tmp3D)){case 1U: _LL1: _LL2:
 goto _LL4;case 2U: _LL3: _LL4:
 goto _LL6;case 3U: _LL5: _LL6:
 goto _LL0;default: _LL7: _LL8:
 hasatt=1;goto _LL0;}_LL0:;}}
# 325
if(!hasatt)
return;
# 325
Cyc_Absyndump_dump(({const char*_tmp3E="__declspec(";_tag_fat(_tmp3E,sizeof(char),12U);}));
# 328
for(0;atts != 0;atts=atts->tl){
void*_stmttmp6=(void*)atts->hd;void*_tmp3F=_stmttmp6;switch(*((int*)_tmp3F)){case 1U: _LLA: _LLB:
 goto _LLD;case 2U: _LLC: _LLD:
 goto _LLF;case 3U: _LLE: _LLF:
 goto _LL9;default: _LL10: _LL11:
 Cyc_Absyndump_dump(Cyc_Absyn_attribute2string((void*)atts->hd));goto _LL9;}_LL9:;}
# 335
Cyc_Absyndump_dump_char((int)')');}
# 96
static void Cyc_Absyndump_dumpatts(struct Cyc_List_List*atts){
# 339
if(atts == 0)return;{enum Cyc_Cyclone_C_Compilers _tmp40=Cyc_Cyclone_c_compiler;if(_tmp40 == Cyc_Cyclone_Gcc_c){_LL1: _LL2:
# 342
 Cyc_Absyndump_dump(({const char*_tmp41=" __attribute__((";_tag_fat(_tmp41,sizeof(char),17U);}));
for(0;atts != 0;atts=atts->tl){
Cyc_Absyndump_dump(Cyc_Absyn_attribute2string((void*)atts->hd));
if(atts->tl != 0)Cyc_Absyndump_dump(({const char*_tmp42=",";_tag_fat(_tmp42,sizeof(char),2U);}));}
# 347
Cyc_Absyndump_dump(({const char*_tmp43=")) ";_tag_fat(_tmp43,sizeof(char),4U);}));
return;}else{_LL3: _LL4:
# 350
 Cyc_Absyndump_dump_noncallconv(atts);
return;}_LL0:;}}
# 96
static void Cyc_Absyndump_dumprgn(void*t){
# 356
Cyc_Absyndump_dumpntyp(t);}struct _tuple17{struct Cyc_List_List*f1;struct Cyc_List_List*f2;};
# 96
static struct _tuple17 Cyc_Absyndump_effects_split(void*t){
# 360
struct Cyc_List_List*rgions=0;
struct Cyc_List_List*effects=0;
{void*_stmttmp7=Cyc_Tcutil_compress(t);void*_tmp44=_stmttmp7;struct Cyc_List_List*_tmp45;void*_tmp46;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp44)->tag == 0U)switch(*((int*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp44)->f1)){case 8U: if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp44)->f2 != 0){_LL1: _tmp46=(void*)(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp44)->f2)->hd;_LL2: {void*r=_tmp46;
rgions=({struct Cyc_List_List*_tmp47=_cycalloc(sizeof(*_tmp47));_tmp47->hd=r,_tmp47->tl=rgions;_tmp47;});goto _LL0;}}else{goto _LL5;}case 9U: _LL3: _tmp45=((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp44)->f2;_LL4: {struct Cyc_List_List*ts=_tmp45;
# 365
for(0;ts != 0;ts=ts->tl){
struct _tuple17 _stmttmp8=Cyc_Absyndump_effects_split((void*)ts->hd);struct Cyc_List_List*_tmp49;struct Cyc_List_List*_tmp48;_LL8: _tmp48=_stmttmp8.f1;_tmp49=_stmttmp8.f2;_LL9: {struct Cyc_List_List*rs=_tmp48;struct Cyc_List_List*es=_tmp49;
rgions=Cyc_List_imp_append(rs,rgions);
effects=Cyc_List_imp_append(es,effects);}}
# 370
goto _LL0;}default: goto _LL5;}else{_LL5: _LL6:
 effects=({struct Cyc_List_List*_tmp4A=_cycalloc(sizeof(*_tmp4A));_tmp4A->hd=t,_tmp4A->tl=effects;_tmp4A;});goto _LL0;}_LL0:;}
# 373
return({struct _tuple17 _tmp2E6;_tmp2E6.f1=rgions,_tmp2E6.f2=effects;_tmp2E6;});}
# 96
static void Cyc_Absyndump_dumpeff(void*t){
# 377
struct _tuple17 _stmttmp9=Cyc_Absyndump_effects_split(t);struct Cyc_List_List*_tmp4C;struct Cyc_List_List*_tmp4B;_LL1: _tmp4B=_stmttmp9.f1;_tmp4C=_stmttmp9.f2;_LL2: {struct Cyc_List_List*rgions=_tmp4B;struct Cyc_List_List*effects=_tmp4C;
rgions=Cyc_List_imp_rev(rgions);
effects=Cyc_List_imp_rev(effects);
if(effects != 0){
({struct Cyc_List_List*_tmp30D=effects;struct _fat_ptr _tmp30C=({const char*_tmp4D="";_tag_fat(_tmp4D,sizeof(char),1U);});struct _fat_ptr _tmp30B=({const char*_tmp4E="";_tag_fat(_tmp4E,sizeof(char),1U);});Cyc_Absyndump_group(Cyc_Absyndump_dumpntyp,_tmp30D,_tmp30C,_tmp30B,({const char*_tmp4F="+";_tag_fat(_tmp4F,sizeof(char),2U);}));});
if(rgions != 0)Cyc_Absyndump_dump_char((int)'+');}
# 380
if(
# 384
rgions != 0 || effects == 0)
({struct Cyc_List_List*_tmp310=rgions;struct _fat_ptr _tmp30F=({const char*_tmp50="{";_tag_fat(_tmp50,sizeof(char),2U);});struct _fat_ptr _tmp30E=({const char*_tmp51="}";_tag_fat(_tmp51,sizeof(char),2U);});Cyc_Absyndump_group(Cyc_Absyndump_dumprgn,_tmp310,_tmp30F,_tmp30E,({const char*_tmp52=",";_tag_fat(_tmp52,sizeof(char),2U);}));});}}
# 96
static void Cyc_Absyndump_dumpntyp(void*t){
# 390
void*_tmp53=t;struct Cyc_Absyn_Exp*_tmp54;struct Cyc_Absyn_Exp*_tmp55;struct Cyc_List_List*_tmp57;struct _tuple0*_tmp56;struct Cyc_Absyn_Datatypedecl*_tmp58;struct Cyc_Absyn_Enumdecl*_tmp59;struct Cyc_Absyn_Aggrdecl*_tmp5A;struct Cyc_List_List*_tmp5C;enum Cyc_Absyn_AggrKind _tmp5B;struct Cyc_List_List*_tmp5D;int _tmp60;void*_tmp5F;struct Cyc_Core_Opt*_tmp5E;int _tmp62;struct Cyc_Core_Opt*_tmp61;struct Cyc_Absyn_Tvar*_tmp63;struct Cyc_List_List*_tmp65;void*_tmp64;switch(*((int*)_tmp53)){case 4U: _LL1: _LL2:
# 392
 goto _LL4;case 5U: _LL3: _LL4:
 goto _LL6;case 3U: _LL5: _LL6:
 return;case 0U: if(((struct Cyc_Absyn_VoidCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp53)->f1)->tag == 0U){_LL7: _LL8:
 Cyc_Absyndump_dump(({const char*_tmp66="void";_tag_fat(_tmp66,sizeof(char),5U);}));return;}else{_LL1F: _tmp64=(void*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp53)->f1;_tmp65=((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp53)->f2;_LL20: {void*c=_tmp64;struct Cyc_List_List*ts=_tmp65;
# 412
void*_tmp70=c;struct _fat_ptr _tmp71;struct Cyc_List_List*_tmp72;struct _tuple0*_tmp73;union Cyc_Absyn_AggrInfo _tmp74;int _tmp75;union Cyc_Absyn_DatatypeFieldInfo _tmp76;union Cyc_Absyn_DatatypeInfo _tmp77;switch(*((int*)_tmp70)){case 18U: _LL22: _tmp77=((struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct*)_tmp70)->f1;_LL23: {union Cyc_Absyn_DatatypeInfo tu_info=_tmp77;
# 414
{union Cyc_Absyn_DatatypeInfo _tmp78=tu_info;int _tmp7A;struct _tuple0*_tmp79;int _tmp7C;struct _tuple0*_tmp7B;if((_tmp78.UnknownDatatype).tag == 1){_LL69: _tmp7B=((_tmp78.UnknownDatatype).val).name;_tmp7C=((_tmp78.UnknownDatatype).val).is_extensible;_LL6A: {struct _tuple0*n=_tmp7B;int is_x=_tmp7C;
_tmp79=n;_tmp7A=is_x;goto _LL6C;}}else{_LL6B: _tmp79=(*(_tmp78.KnownDatatype).val)->name;_tmp7A=(*(_tmp78.KnownDatatype).val)->is_extensible;_LL6C: {struct _tuple0*n=_tmp79;int is_x=_tmp7A;
# 417
if(is_x)Cyc_Absyndump_dump(({const char*_tmp7D="@extensible";_tag_fat(_tmp7D,sizeof(char),12U);}));Cyc_Absyndump_dump(({const char*_tmp7E="datatype";_tag_fat(_tmp7E,sizeof(char),9U);}));
Cyc_Absyndump_dumpqvar(n);Cyc_Absyndump_dumptps(ts);
goto _LL68;}}_LL68:;}
# 421
return;}case 19U: _LL24: _tmp76=((struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct*)_tmp70)->f1;_LL25: {union Cyc_Absyn_DatatypeFieldInfo tuf_info=_tmp76;
# 423
{union Cyc_Absyn_DatatypeFieldInfo _tmp7F=tuf_info;struct _tuple0*_tmp82;int _tmp81;struct _tuple0*_tmp80;int _tmp85;struct _tuple0*_tmp84;struct _tuple0*_tmp83;if((_tmp7F.UnknownDatatypefield).tag == 1){_LL6E: _tmp83=((_tmp7F.UnknownDatatypefield).val).datatype_name;_tmp84=((_tmp7F.UnknownDatatypefield).val).field_name;_tmp85=((_tmp7F.UnknownDatatypefield).val).is_extensible;_LL6F: {struct _tuple0*tname=_tmp83;struct _tuple0*fname=_tmp84;int is_x=_tmp85;
# 425
_tmp80=tname;_tmp81=is_x;_tmp82=fname;goto _LL71;}}else{_LL70: _tmp80=(((_tmp7F.KnownDatatypefield).val).f1)->name;_tmp81=(((_tmp7F.KnownDatatypefield).val).f1)->is_extensible;_tmp82=(((_tmp7F.KnownDatatypefield).val).f2)->name;_LL71: {struct _tuple0*tname=_tmp80;int is_x=_tmp81;struct _tuple0*fname=_tmp82;
# 428
if(is_x)Cyc_Absyndump_dump(({const char*_tmp86="@extensible ";_tag_fat(_tmp86,sizeof(char),13U);}));Cyc_Absyndump_dump(({const char*_tmp87="datatype ";_tag_fat(_tmp87,sizeof(char),10U);}));
Cyc_Absyndump_dumpqvar(tname);Cyc_Absyndump_dump(({const char*_tmp88=".";_tag_fat(_tmp88,sizeof(char),2U);}));Cyc_Absyndump_dumpqvar(fname);
Cyc_Absyndump_dumptps(ts);
goto _LL6D;}}_LL6D:;}
# 433
return;}case 1U: switch(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)_tmp70)->f1){case Cyc_Absyn_None: switch(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)_tmp70)->f2){case Cyc_Absyn_Int_sz: _LL26: _LL27:
 goto _LL29;case Cyc_Absyn_Long_sz: _LL2A: _LL2B:
# 436
 goto _LL2D;case Cyc_Absyn_Char_sz: _LL2E: _LL2F:
# 438
 Cyc_Absyndump_dump(({const char*_tmp8B="char";_tag_fat(_tmp8B,sizeof(char),5U);}));return;case Cyc_Absyn_Short_sz: _LL34: _LL35:
# 441
 goto _LL37;case Cyc_Absyn_LongLong_sz: _LL40: _LL41:
# 447
 goto _LL43;default: goto _LL66;}case Cyc_Absyn_Signed: switch(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)_tmp70)->f2){case Cyc_Absyn_Int_sz: _LL28: _LL29:
# 435
 Cyc_Absyndump_dump(({const char*_tmp89="int";_tag_fat(_tmp89,sizeof(char),4U);}));return;case Cyc_Absyn_Long_sz: _LL2C: _LL2D:
# 437
 Cyc_Absyndump_dump(({const char*_tmp8A="long";_tag_fat(_tmp8A,sizeof(char),5U);}));return;case Cyc_Absyn_Char_sz: _LL30: _LL31:
# 439
 Cyc_Absyndump_dump(({const char*_tmp8C="signed char";_tag_fat(_tmp8C,sizeof(char),12U);}));return;case Cyc_Absyn_Short_sz: _LL36: _LL37:
# 442
 Cyc_Absyndump_dump(({const char*_tmp8E="short";_tag_fat(_tmp8E,sizeof(char),6U);}));return;case Cyc_Absyn_LongLong_sz: _LL42: _LL43: {
# 449
enum Cyc_Cyclone_C_Compilers _tmp93=Cyc_Cyclone_c_compiler;if(_tmp93 == Cyc_Cyclone_Gcc_c){_LL73: _LL74:
 Cyc_Absyndump_dump(({const char*_tmp94="long long";_tag_fat(_tmp94,sizeof(char),10U);}));return;}else{_LL75: _LL76:
 Cyc_Absyndump_dump(({const char*_tmp95="__int64";_tag_fat(_tmp95,sizeof(char),8U);}));return;}_LL72:;}default: goto _LL66;}case Cyc_Absyn_Unsigned: switch(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)_tmp70)->f2){case Cyc_Absyn_Char_sz: _LL32: _LL33:
# 440
 Cyc_Absyndump_dump(({const char*_tmp8D="unsigned char";_tag_fat(_tmp8D,sizeof(char),14U);}));return;case Cyc_Absyn_Short_sz: _LL38: _LL39:
# 443
 Cyc_Absyndump_dump(({const char*_tmp8F="unsigned short";_tag_fat(_tmp8F,sizeof(char),15U);}));return;case Cyc_Absyn_Int_sz: _LL3A: _LL3B:
 Cyc_Absyndump_dump(({const char*_tmp90="unsigned";_tag_fat(_tmp90,sizeof(char),9U);}));return;case Cyc_Absyn_Long_sz: _LL3C: _LL3D:
 Cyc_Absyndump_dump(({const char*_tmp91="unsigned long";_tag_fat(_tmp91,sizeof(char),14U);}));return;case Cyc_Absyn_LongLong_sz: _LL3E: _LL3F:
 Cyc_Absyndump_dump(({const char*_tmp92="unsigned";_tag_fat(_tmp92,sizeof(char),9U);}));goto _LL41;default: goto _LL66;}default: goto _LL66;}case 2U: _LL44: _tmp75=((struct Cyc_Absyn_FloatCon_Absyn_TyCon_struct*)_tmp70)->f1;_LL45: {int i=_tmp75;
# 454
int _tmp96=i;switch(_tmp96){case 0U: _LL78: _LL79:
 Cyc_Absyndump_dump(({const char*_tmp97="float";_tag_fat(_tmp97,sizeof(char),6U);}));return;case 1U: _LL7A: _LL7B:
 Cyc_Absyndump_dump(({const char*_tmp98="double";_tag_fat(_tmp98,sizeof(char),7U);}));return;default: _LL7C: _LL7D:
 Cyc_Absyndump_dump(({const char*_tmp99="long double";_tag_fat(_tmp99,sizeof(char),12U);}));return;}_LL77:;}case 20U: _LL46: _tmp74=((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)_tmp70)->f1;_LL47: {union Cyc_Absyn_AggrInfo info=_tmp74;
# 460
struct _tuple11 _stmttmpA=Cyc_Absyn_aggr_kinded_name(info);struct _tuple0*_tmp9B;enum Cyc_Absyn_AggrKind _tmp9A;_LL7F: _tmp9A=_stmttmpA.f1;_tmp9B=_stmttmpA.f2;_LL80: {enum Cyc_Absyn_AggrKind k=_tmp9A;struct _tuple0*n=_tmp9B;
Cyc_Absyndump_dumpaggr_kind(k);Cyc_Absyndump_dumpqvar(n);Cyc_Absyndump_dumptps(ts);
return;}}case 15U: _LL48: _tmp73=((struct Cyc_Absyn_EnumCon_Absyn_TyCon_struct*)_tmp70)->f1;_LL49: {struct _tuple0*n=_tmp73;
Cyc_Absyndump_dump(({const char*_tmp9C="enum";_tag_fat(_tmp9C,sizeof(char),5U);}));Cyc_Absyndump_dumpqvar(n);return;}case 16U: _LL4A: _tmp72=((struct Cyc_Absyn_AnonEnumCon_Absyn_TyCon_struct*)_tmp70)->f1;_LL4B: {struct Cyc_List_List*fs=_tmp72;
Cyc_Absyndump_dump(({const char*_tmp9D="enum{";_tag_fat(_tmp9D,sizeof(char),6U);}));Cyc_Absyndump_dumpenumfields(fs);Cyc_Absyndump_dump(({const char*_tmp9E="}";_tag_fat(_tmp9E,sizeof(char),2U);}));return;}case 17U: _LL4C: _tmp71=((struct Cyc_Absyn_BuiltinCon_Absyn_TyCon_struct*)_tmp70)->f1;_LL4D: {struct _fat_ptr t=_tmp71;
Cyc_Absyndump_dump(t);return;}case 3U: _LL4E: _LL4F:
 Cyc_Absyndump_dump(({const char*_tmp9F="region_t<";_tag_fat(_tmp9F,sizeof(char),10U);}));Cyc_Absyndump_dumprgn((void*)((struct Cyc_List_List*)_check_null(ts))->hd);Cyc_Absyndump_dump(({const char*_tmpA0=">";_tag_fat(_tmpA0,sizeof(char),2U);}));return;case 4U: _LL50: _LL51:
 Cyc_Absyndump_dump(({const char*_tmpA1="tag_t<";_tag_fat(_tmpA1,sizeof(char),7U);}));Cyc_Absyndump_dumptyp((void*)((struct Cyc_List_List*)_check_null(ts))->hd);Cyc_Absyndump_dump(({const char*_tmpA2=">";_tag_fat(_tmpA2,sizeof(char),2U);}));return;case 6U: _LL52: _LL53:
 Cyc_Absyndump_dump(({const char*_tmpA3="`U";_tag_fat(_tmpA3,sizeof(char),3U);}));return;case 7U: _LL54: _LL55:
 Cyc_Absyndump_dump(({const char*_tmpA4="`RC";_tag_fat(_tmpA4,sizeof(char),4U);}));return;case 5U: _LL56: _LL57:
 Cyc_Absyndump_dump(({const char*_tmpA5="`H";_tag_fat(_tmpA5,sizeof(char),3U);}));return;case 8U: _LL58: _LL59:
 Cyc_Absyndump_dump(({const char*_tmpA6="{";_tag_fat(_tmpA6,sizeof(char),2U);}));Cyc_Absyndump_dumptyp((void*)((struct Cyc_List_List*)_check_null(ts))->hd);Cyc_Absyndump_dump(({const char*_tmpA7="}";_tag_fat(_tmpA7,sizeof(char),2U);}));return;case 10U: _LL5A: _LL5B:
 Cyc_Absyndump_dump(({const char*_tmpA8="regions(";_tag_fat(_tmpA8,sizeof(char),9U);}));Cyc_Absyndump_dumptyp((void*)((struct Cyc_List_List*)_check_null(ts))->hd);Cyc_Absyndump_dump(({const char*_tmpA9=")";_tag_fat(_tmpA9,sizeof(char),2U);}));return;case 9U: _LL5C: _LL5D:
({struct Cyc_List_List*_tmp313=ts;struct _fat_ptr _tmp312=({const char*_tmpAA="";_tag_fat(_tmpAA,sizeof(char),1U);});struct _fat_ptr _tmp311=({const char*_tmpAB="";_tag_fat(_tmpAB,sizeof(char),1U);});Cyc_Absyndump_group(Cyc_Absyndump_dumptyp,_tmp313,_tmp312,_tmp311,({const char*_tmpAC="+";_tag_fat(_tmpAC,sizeof(char),2U);}));});return;case 11U: _LL5E: _LL5F:
 Cyc_Absyndump_dump(({const char*_tmpAD="@true";_tag_fat(_tmpAD,sizeof(char),6U);}));return;case 12U: _LL60: _LL61:
 Cyc_Absyndump_dump(({const char*_tmpAE="@false";_tag_fat(_tmpAE,sizeof(char),7U);}));return;case 13U: _LL62: _LL63:
 Cyc_Absyndump_dump(({const char*_tmpAF="@thin";_tag_fat(_tmpAF,sizeof(char),6U);}));return;case 14U: _LL64: _LL65:
 Cyc_Absyndump_dump(({const char*_tmpB0="@fat";_tag_fat(_tmpB0,sizeof(char),5U);}));return;default: _LL66: _LL67:
({void*_tmpB1=0U;({int(*_tmp315)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmpB3)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmpB3;});struct _fat_ptr _tmp314=({const char*_tmpB2="bad type constructor";_tag_fat(_tmpB2,sizeof(char),21U);});_tmp315(_tmp314,_tag_fat(_tmpB1,sizeof(void*),0U));});});}_LL21:;}}case 2U: _LL9: _tmp63=((struct Cyc_Absyn_VarType_Absyn_Type_struct*)_tmp53)->f1;_LLA: {struct Cyc_Absyn_Tvar*tv=_tmp63;
# 396
Cyc_Absyndump_dumptvar(tv);return;}case 1U: if(((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp53)->f2 == 0){_LLB: _tmp61=((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp53)->f1;_tmp62=((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp53)->f3;_LLC: {struct Cyc_Core_Opt*k=_tmp61;int i=_tmp62;
# 398
Cyc_Absyndump_dump(({const char*_tmp67="`E";_tag_fat(_tmp67,sizeof(char),3U);}));
if(k == 0)Cyc_Absyndump_dump(({const char*_tmp68="K";_tag_fat(_tmp68,sizeof(char),2U);}));else{Cyc_Absyndump_dumpkind((struct Cyc_Absyn_Kind*)k->v);}
Cyc_Absyndump_dump((struct _fat_ptr)({struct Cyc_Int_pa_PrintArg_struct _tmp6B=({struct Cyc_Int_pa_PrintArg_struct _tmp2E7;_tmp2E7.tag=1U,_tmp2E7.f1=(unsigned long)i;_tmp2E7;});void*_tmp69[1U];_tmp69[0]=& _tmp6B;({struct _fat_ptr _tmp316=({const char*_tmp6A="%d";_tag_fat(_tmp6A,sizeof(char),3U);});Cyc_aprintf(_tmp316,_tag_fat(_tmp69,sizeof(void*),1U));});}));return;}}else{_LLD: _tmp5E=((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp53)->f1;_tmp5F=(void*)((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp53)->f2;_tmp60=((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp53)->f3;_LLE: {struct Cyc_Core_Opt*k=_tmp5E;void*t=_tmp5F;int i=_tmp60;
Cyc_Absyndump_dumpntyp((void*)_check_null(t));return;}}case 6U: _LLF: _tmp5D=((struct Cyc_Absyn_TupleType_Absyn_Type_struct*)_tmp53)->f1;_LL10: {struct Cyc_List_List*ts=_tmp5D;
Cyc_Absyndump_dump_char((int)'$');Cyc_Absyndump_dumpargs(ts);return;}case 7U: _LL11: _tmp5B=((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmp53)->f1;_tmp5C=((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmp53)->f2;_LL12: {enum Cyc_Absyn_AggrKind k=_tmp5B;struct Cyc_List_List*fs=_tmp5C;
# 404
Cyc_Absyndump_dumpaggr_kind(k);Cyc_Absyndump_dump_char((int)'{');Cyc_Absyndump_dumpaggrfields(fs);Cyc_Absyndump_dump_char((int)'}');return;}case 10U: switch(*((int*)((struct Cyc_Absyn_TypeDecl*)((struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct*)_tmp53)->f1)->r)){case 0U: _LL13: _tmp5A=((struct Cyc_Absyn_Aggr_td_Absyn_Raw_typedecl_struct*)(((struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct*)_tmp53)->f1)->r)->f1;_LL14: {struct Cyc_Absyn_Aggrdecl*d=_tmp5A;
Cyc_Absyndump_dump_aggrdecl(d);return;}case 1U: _LL15: _tmp59=((struct Cyc_Absyn_Enum_td_Absyn_Raw_typedecl_struct*)(((struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct*)_tmp53)->f1)->r)->f1;_LL16: {struct Cyc_Absyn_Enumdecl*d=_tmp59;
Cyc_Absyndump_dump_enumdecl(d);return;}default: _LL17: _tmp58=((struct Cyc_Absyn_Datatype_td_Absyn_Raw_typedecl_struct*)(((struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct*)_tmp53)->f1)->r)->f1;_LL18: {struct Cyc_Absyn_Datatypedecl*d=_tmp58;
Cyc_Absyndump_dump_datatypedecl(d);return;}}case 8U: _LL19: _tmp56=((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmp53)->f1;_tmp57=((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmp53)->f2;_LL1A: {struct _tuple0*n=_tmp56;struct Cyc_List_List*ts=_tmp57;
Cyc_Absyndump_dumpqvar(n),Cyc_Absyndump_dumptps(ts);return;}case 9U: _LL1B: _tmp55=((struct Cyc_Absyn_ValueofType_Absyn_Type_struct*)_tmp53)->f1;_LL1C: {struct Cyc_Absyn_Exp*e=_tmp55;
Cyc_Absyndump_dump(({const char*_tmp6C="valueof_t(";_tag_fat(_tmp6C,sizeof(char),11U);}));Cyc_Absyndump_dumpexp(e);Cyc_Absyndump_dump(({const char*_tmp6D=")";_tag_fat(_tmp6D,sizeof(char),2U);}));return;}default: _LL1D: _tmp54=((struct Cyc_Absyn_TypeofType_Absyn_Type_struct*)_tmp53)->f1;_LL1E: {struct Cyc_Absyn_Exp*e=_tmp54;
Cyc_Absyndump_dump(({const char*_tmp6E="typeof(";_tag_fat(_tmp6E,sizeof(char),8U);}));Cyc_Absyndump_dumpexp(e);Cyc_Absyndump_dump(({const char*_tmp6F=")";_tag_fat(_tmp6F,sizeof(char),2U);}));return;}}_LL0:;}
# 96
static void Cyc_Absyndump_dumpvaropt(struct _fat_ptr*vo){
# 484
if(vo != 0)Cyc_Absyndump_dump_str(vo);}
# 96
static void Cyc_Absyndump_dumpfunarg(struct _tuple8*t){
# 487
({void(*_tmp319)(struct Cyc_Absyn_Tqual,void*,void(*f)(struct _fat_ptr*),struct _fat_ptr*)=({void(*_tmpB4)(struct Cyc_Absyn_Tqual,void*,void(*f)(struct _fat_ptr*),struct _fat_ptr*)=(void(*)(struct Cyc_Absyn_Tqual,void*,void(*f)(struct _fat_ptr*),struct _fat_ptr*))Cyc_Absyndump_dumptqtd;_tmpB4;});struct Cyc_Absyn_Tqual _tmp318=(*t).f2;void*_tmp317=(*t).f3;_tmp319(_tmp318,_tmp317,Cyc_Absyndump_dumpvaropt,(*t).f1);});}struct _tuple18{void*f1;void*f2;};
# 96
static void Cyc_Absyndump_dump_rgncmp(struct _tuple18*cmp){
# 490
Cyc_Absyndump_dumpeff((*cmp).f1);Cyc_Absyndump_dump_char((int)'>');Cyc_Absyndump_dumprgn((*cmp).f2);}
# 96
static void Cyc_Absyndump_dump_rgnpo(struct Cyc_List_List*rgn_po){
# 493
({void(*_tmp31B)(void(*f)(struct _tuple18*),struct Cyc_List_List*l,struct _fat_ptr sep)=({void(*_tmpB5)(void(*f)(struct _tuple18*),struct Cyc_List_List*l,struct _fat_ptr sep)=(void(*)(void(*f)(struct _tuple18*),struct Cyc_List_List*l,struct _fat_ptr sep))Cyc_Absyndump_dump_sep;_tmpB5;});struct Cyc_List_List*_tmp31A=rgn_po;_tmp31B(Cyc_Absyndump_dump_rgncmp,_tmp31A,({const char*_tmpB6=",";_tag_fat(_tmpB6,sizeof(char),2U);}));});}
# 96
static void Cyc_Absyndump_dumpfunargs(struct Cyc_List_List*args,int c_varargs,struct Cyc_Absyn_VarargInfo*cyc_varargs,void*effopt,struct Cyc_List_List*rgn_po,struct Cyc_Absyn_Exp*req,struct Cyc_Absyn_Exp*ens){
# 499
({void(*_tmp31F)(void(*f)(struct _tuple8*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep)=({void(*_tmpB7)(void(*f)(struct _tuple8*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep)=(void(*)(void(*f)(struct _tuple8*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep))Cyc_Absyndump_group;_tmpB7;});struct Cyc_List_List*_tmp31E=args;struct _fat_ptr _tmp31D=({const char*_tmpB8="(";_tag_fat(_tmpB8,sizeof(char),2U);});struct _fat_ptr _tmp31C=({const char*_tmpB9="";_tag_fat(_tmpB9,sizeof(char),1U);});_tmp31F(Cyc_Absyndump_dumpfunarg,_tmp31E,_tmp31D,_tmp31C,({const char*_tmpBA=",";_tag_fat(_tmpBA,sizeof(char),2U);}));});
if(c_varargs || cyc_varargs != 0)
Cyc_Absyndump_dump(({const char*_tmpBB=",...";_tag_fat(_tmpBB,sizeof(char),5U);}));
# 500
if(cyc_varargs != 0){
# 503
if(cyc_varargs->inject)Cyc_Absyndump_dump(({const char*_tmpBC=" inject ";_tag_fat(_tmpBC,sizeof(char),9U);}));Cyc_Absyndump_dumpfunarg(({struct _tuple8*_tmpBD=_cycalloc(sizeof(*_tmpBD));
_tmpBD->f1=cyc_varargs->name,_tmpBD->f2=cyc_varargs->tq,_tmpBD->f3=cyc_varargs->type;_tmpBD;}));}
# 500
if(effopt != 0){
# 507
Cyc_Absyndump_dump_semi();Cyc_Absyndump_dumpeff(effopt);}
# 500
if(rgn_po != 0){
# 510
Cyc_Absyndump_dump_char((int)':');Cyc_Absyndump_dump_rgnpo(rgn_po);}
# 500
Cyc_Absyndump_dump_char((int)')');
# 513
if(req != 0){
Cyc_Absyndump_dump(({const char*_tmpBE=" @requires(";_tag_fat(_tmpBE,sizeof(char),12U);}));Cyc_Absyndump_dumpexp(req);Cyc_Absyndump_dump_char((int)')');}
# 513
if(ens != 0){
# 517
Cyc_Absyndump_dump(({const char*_tmpBF=" @ensures(";_tag_fat(_tmpBF,sizeof(char),11U);}));Cyc_Absyndump_dumpexp(ens);Cyc_Absyndump_dump_char((int)')');}}
# 96
static void Cyc_Absyndump_dumptyp(void*t){
# 522
({void(*_tmp322)(struct Cyc_Absyn_Tqual,void*,void(*f)(int),int)=({void(*_tmpC0)(struct Cyc_Absyn_Tqual,void*,void(*f)(int),int)=(void(*)(struct Cyc_Absyn_Tqual,void*,void(*f)(int),int))Cyc_Absyndump_dumptqtd;_tmpC0;});struct Cyc_Absyn_Tqual _tmp321=Cyc_Absyn_empty_tqual(0U);void*_tmp320=t;_tmp322(_tmp321,_tmp320,({void(*_tmpC1)(int x)=(void(*)(int x))Cyc_Absyndump_ignore;_tmpC1;}),0);});}
# 96
static void Cyc_Absyndump_dumpdesignator(void*d){
# 526
void*_tmpC2=d;struct _fat_ptr*_tmpC3;struct Cyc_Absyn_Exp*_tmpC4;if(((struct Cyc_Absyn_ArrayElement_Absyn_Designator_struct*)_tmpC2)->tag == 0U){_LL1: _tmpC4=((struct Cyc_Absyn_ArrayElement_Absyn_Designator_struct*)_tmpC2)->f1;_LL2: {struct Cyc_Absyn_Exp*e=_tmpC4;
Cyc_Absyndump_dump(({const char*_tmpC5=".[";_tag_fat(_tmpC5,sizeof(char),3U);}));Cyc_Absyndump_dumpexp(e);Cyc_Absyndump_dump_char((int)']');goto _LL0;}}else{_LL3: _tmpC3=((struct Cyc_Absyn_FieldName_Absyn_Designator_struct*)_tmpC2)->f1;_LL4: {struct _fat_ptr*v=_tmpC3;
Cyc_Absyndump_dump_char((int)'.');Cyc_Absyndump_dump_nospace(*v);goto _LL0;}}_LL0:;}struct _tuple19{struct Cyc_List_List*f1;struct Cyc_Absyn_Exp*f2;};
# 96
static void Cyc_Absyndump_dumpde(struct _tuple19*de){
# 532
({struct Cyc_List_List*_tmp325=(*de).f1;struct _fat_ptr _tmp324=({const char*_tmpC6="";_tag_fat(_tmpC6,sizeof(char),1U);});struct _fat_ptr _tmp323=({const char*_tmpC7="=";_tag_fat(_tmpC7,sizeof(char),2U);});Cyc_Absyndump_egroup(Cyc_Absyndump_dumpdesignator,_tmp325,_tmp324,_tmp323,({const char*_tmpC8="=";_tag_fat(_tmpC8,sizeof(char),2U);}));});
Cyc_Absyndump_dumpexp((*de).f2);}
# 96
static void Cyc_Absyndump_dumpoffset_field(void*f){
# 537
void*_tmpC9=f;unsigned _tmpCA;struct _fat_ptr*_tmpCB;if(((struct Cyc_Absyn_StructField_Absyn_OffsetofField_struct*)_tmpC9)->tag == 0U){_LL1: _tmpCB=((struct Cyc_Absyn_StructField_Absyn_OffsetofField_struct*)_tmpC9)->f1;_LL2: {struct _fat_ptr*n=_tmpCB;
Cyc_Absyndump_dump_nospace(*n);return;}}else{_LL3: _tmpCA=((struct Cyc_Absyn_TupleIndex_Absyn_OffsetofField_struct*)_tmpC9)->f1;_LL4: {unsigned n=_tmpCA;
Cyc_Absyndump_dump((struct _fat_ptr)({struct Cyc_Int_pa_PrintArg_struct _tmpCE=({struct Cyc_Int_pa_PrintArg_struct _tmp2E8;_tmp2E8.tag=1U,_tmp2E8.f1=(unsigned long)((int)n);_tmp2E8;});void*_tmpCC[1U];_tmpCC[0]=& _tmpCE;({struct _fat_ptr _tmp326=({const char*_tmpCD="%d";_tag_fat(_tmpCD,sizeof(char),3U);});Cyc_aprintf(_tmp326,_tag_fat(_tmpCC,sizeof(void*),1U));});}));return;}}_LL0:;}struct _tuple20{struct _fat_ptr f1;struct Cyc_Absyn_Exp*f2;};
# 96
static void Cyc_Absyndump_dump_asm_iolist(struct Cyc_List_List*iolist){
# 544
while((unsigned)iolist){
struct _tuple20*_stmttmpB=(struct _tuple20*)iolist->hd;struct Cyc_Absyn_Exp*_tmpD0;struct _fat_ptr _tmpCF;_LL1: _tmpCF=_stmttmpB->f1;_tmpD0=_stmttmpB->f2;_LL2: {struct _fat_ptr constr=_tmpCF;struct Cyc_Absyn_Exp*e1=_tmpD0;
Cyc_Absyndump_dump_char((int)'"');Cyc_Absyndump_dump_nospace(constr);Cyc_Absyndump_dump_char((int)'"');
Cyc_Absyndump_dump_char((int)'(');Cyc_Absyndump_dumpexp(e1);Cyc_Absyndump_dump_char((int)')');
iolist=iolist->tl;
if((unsigned)iolist)
Cyc_Absyndump_dump_char((int)',');}}}
# 96
static void Cyc_Absyndump_dumpexps_prec(int inprec,struct Cyc_List_List*es){
# 555
({void(*_tmp32B)(void(*f)(int,struct Cyc_Absyn_Exp*),int env,struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep)=({void(*_tmpD1)(void(*f)(int,struct Cyc_Absyn_Exp*),int env,struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep)=(void(*)(void(*f)(int,struct Cyc_Absyn_Exp*),int env,struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep))Cyc_Absyndump_group_c;_tmpD1;});int _tmp32A=inprec;struct Cyc_List_List*_tmp329=es;struct _fat_ptr _tmp328=({const char*_tmpD2="";_tag_fat(_tmpD2,sizeof(char),1U);});struct _fat_ptr _tmp327=({const char*_tmpD3="";_tag_fat(_tmpD3,sizeof(char),1U);});_tmp32B(Cyc_Absyndump_dumpexp_prec,_tmp32A,_tmp329,_tmp328,_tmp327,({const char*_tmpD4=",";_tag_fat(_tmpD4,sizeof(char),2U);}));});}
# 96
static void Cyc_Absyndump_dumpexp_prec(int inprec,struct Cyc_Absyn_Exp*e){
# 559
int myprec=Cyc_Absynpp_exp_prec(e);
if(inprec >= myprec)
Cyc_Absyndump_dump_nospace(({const char*_tmpD5="(";_tag_fat(_tmpD5,sizeof(char),2U);}));
# 560
{void*_stmttmpC=e->r;void*_tmpD6=_stmttmpC;struct Cyc_Absyn_Stmt*_tmpD7;struct Cyc_List_List*_tmpD9;struct Cyc_Core_Opt*_tmpD8;struct Cyc_Absyn_Exp*_tmpDB;struct Cyc_Absyn_Exp*_tmpDA;int _tmpE0;struct Cyc_Absyn_Exp*_tmpDF;void**_tmpDE;struct Cyc_Absyn_Exp*_tmpDD;int _tmpDC;struct Cyc_Absyn_Exp*_tmpE2;struct Cyc_Absyn_Exp*_tmpE1;struct Cyc_Absyn_Datatypefield*_tmpE4;struct Cyc_List_List*_tmpE3;struct Cyc_Absyn_Enumfield*_tmpE5;struct Cyc_Absyn_Enumfield*_tmpE6;struct Cyc_List_List*_tmpE7;struct Cyc_List_List*_tmpEA;struct Cyc_List_List*_tmpE9;struct _tuple0*_tmpE8;void*_tmpEC;struct Cyc_Absyn_Exp*_tmpEB;struct Cyc_Absyn_Exp*_tmpEF;struct Cyc_Absyn_Exp*_tmpEE;struct Cyc_Absyn_Vardecl*_tmpED;struct Cyc_List_List*_tmpF0;struct Cyc_List_List*_tmpF2;struct _tuple8*_tmpF1;struct Cyc_List_List*_tmpF3;struct Cyc_Absyn_Exp*_tmpF5;struct Cyc_Absyn_Exp*_tmpF4;struct _fat_ptr*_tmpF7;struct Cyc_Absyn_Exp*_tmpF6;struct _fat_ptr*_tmpF9;struct Cyc_Absyn_Exp*_tmpF8;struct Cyc_List_List*_tmpFB;void*_tmpFA;struct _fat_ptr*_tmpFD;struct Cyc_Absyn_Exp*_tmpFC;struct Cyc_List_List*_tmp102;struct Cyc_List_List*_tmp101;struct Cyc_List_List*_tmp100;struct _fat_ptr _tmpFF;int _tmpFE;struct Cyc_Absyn_Exp*_tmp103;void*_tmp104;struct Cyc_Absyn_Exp*_tmp105;void*_tmp106;struct Cyc_Absyn_Exp*_tmp108;struct Cyc_Absyn_Exp*_tmp107;struct Cyc_Absyn_Exp*_tmp109;struct Cyc_Absyn_Exp*_tmp10A;struct Cyc_Absyn_Exp*_tmp10C;void*_tmp10B;struct Cyc_Absyn_Exp*_tmp10D;struct Cyc_Absyn_Exp*_tmp10E;struct Cyc_Absyn_Exp*_tmp10F;struct Cyc_List_List*_tmp111;struct Cyc_Absyn_Exp*_tmp110;struct Cyc_Absyn_Exp*_tmp113;struct Cyc_Absyn_Exp*_tmp112;struct Cyc_Absyn_Exp*_tmp115;struct Cyc_Absyn_Exp*_tmp114;struct Cyc_Absyn_Exp*_tmp117;struct Cyc_Absyn_Exp*_tmp116;struct Cyc_Absyn_Exp*_tmp11A;struct Cyc_Absyn_Exp*_tmp119;struct Cyc_Absyn_Exp*_tmp118;struct Cyc_Absyn_Exp*_tmp11B;struct Cyc_Absyn_Exp*_tmp11C;struct Cyc_Absyn_Exp*_tmp11D;struct Cyc_Absyn_Exp*_tmp11E;struct Cyc_Absyn_Exp*_tmp121;struct Cyc_Core_Opt*_tmp120;struct Cyc_Absyn_Exp*_tmp11F;struct Cyc_List_List*_tmp123;enum Cyc_Absyn_Primop _tmp122;struct _fat_ptr _tmp124;void*_tmp125;struct _fat_ptr _tmp126;struct _fat_ptr _tmp127;struct _fat_ptr _tmp128;long long _tmp12A;enum Cyc_Absyn_Sign _tmp129;int _tmp12B;int _tmp12C;int _tmp12D;short _tmp12F;enum Cyc_Absyn_Sign _tmp12E;struct _fat_ptr _tmp130;char _tmp132;enum Cyc_Absyn_Sign _tmp131;switch(*((int*)_tmpD6)){case 0U: switch(((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmpD6)->f1).String_c).tag){case 2U: _LL1: _tmp131=(((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmpD6)->f1).Char_c).val).f1;_tmp132=(((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmpD6)->f1).Char_c).val).f2;_LL2: {enum Cyc_Absyn_Sign sg=_tmp131;char ch=_tmp132;
# 564
Cyc_Absyndump_dump_char((int)'\'');Cyc_Absyndump_dump_nospace(Cyc_Absynpp_char_escape(ch));Cyc_Absyndump_dump_char((int)'\'');
goto _LL0;}case 3U: _LL3: _tmp130=((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmpD6)->f1).Wchar_c).val;_LL4: {struct _fat_ptr s=_tmp130;
Cyc_Absyndump_dump((struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp135=({struct Cyc_String_pa_PrintArg_struct _tmp2E9;_tmp2E9.tag=0U,_tmp2E9.f1=(struct _fat_ptr)((struct _fat_ptr)s);_tmp2E9;});void*_tmp133[1U];_tmp133[0]=& _tmp135;({struct _fat_ptr _tmp32C=({const char*_tmp134="L'%s'";_tag_fat(_tmp134,sizeof(char),6U);});Cyc_aprintf(_tmp32C,_tag_fat(_tmp133,sizeof(void*),1U));});}));goto _LL0;}case 4U: _LL5: _tmp12E=(((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmpD6)->f1).Short_c).val).f1;_tmp12F=(((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmpD6)->f1).Short_c).val).f2;_LL6: {enum Cyc_Absyn_Sign sg=_tmp12E;short s=_tmp12F;
Cyc_Absyndump_dump((struct _fat_ptr)({struct Cyc_Int_pa_PrintArg_struct _tmp138=({struct Cyc_Int_pa_PrintArg_struct _tmp2EA;_tmp2EA.tag=1U,_tmp2EA.f1=(unsigned long)((int)s);_tmp2EA;});void*_tmp136[1U];_tmp136[0]=& _tmp138;({struct _fat_ptr _tmp32D=({const char*_tmp137="%d";_tag_fat(_tmp137,sizeof(char),3U);});Cyc_aprintf(_tmp32D,_tag_fat(_tmp136,sizeof(void*),1U));});}));goto _LL0;}case 5U: switch((((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmpD6)->f1).Int_c).val).f1){case Cyc_Absyn_None: _LL7: _tmp12D=(((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmpD6)->f1).Int_c).val).f2;_LL8: {int i=_tmp12D;
_tmp12C=i;goto _LLA;}case Cyc_Absyn_Signed: _LL9: _tmp12C=(((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmpD6)->f1).Int_c).val).f2;_LLA: {int i=_tmp12C;
Cyc_Absyndump_dump((struct _fat_ptr)({struct Cyc_Int_pa_PrintArg_struct _tmp13B=({struct Cyc_Int_pa_PrintArg_struct _tmp2EB;_tmp2EB.tag=1U,_tmp2EB.f1=(unsigned long)i;_tmp2EB;});void*_tmp139[1U];_tmp139[0]=& _tmp13B;({struct _fat_ptr _tmp32E=({const char*_tmp13A="%d";_tag_fat(_tmp13A,sizeof(char),3U);});Cyc_aprintf(_tmp32E,_tag_fat(_tmp139,sizeof(void*),1U));});}));goto _LL0;}default: _LLB: _tmp12B=(((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmpD6)->f1).Int_c).val).f2;_LLC: {int i=_tmp12B;
Cyc_Absyndump_dump((struct _fat_ptr)({struct Cyc_Int_pa_PrintArg_struct _tmp13E=({struct Cyc_Int_pa_PrintArg_struct _tmp2EC;_tmp2EC.tag=1U,_tmp2EC.f1=(unsigned)i;_tmp2EC;});void*_tmp13C[1U];_tmp13C[0]=& _tmp13E;({struct _fat_ptr _tmp32F=({const char*_tmp13D="%uU";_tag_fat(_tmp13D,sizeof(char),4U);});Cyc_aprintf(_tmp32F,_tag_fat(_tmp13C,sizeof(void*),1U));});}));goto _LL0;}}case 6U: _LLD: _tmp129=(((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmpD6)->f1).LongLong_c).val).f1;_tmp12A=(((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmpD6)->f1).LongLong_c).val).f2;_LLE: {enum Cyc_Absyn_Sign sg=_tmp129;long long i=_tmp12A;
# 573
Cyc_Absyndump_dump(Cyc_Absynpp_longlong2string((unsigned long long)i));goto _LL0;}case 7U: _LLF: _tmp128=(((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmpD6)->f1).Float_c).val).f1;_LL10: {struct _fat_ptr x=_tmp128;
Cyc_Absyndump_dump(x);goto _LL0;}case 1U: _LL11: _LL12:
 Cyc_Absyndump_dump(({const char*_tmp13F="NULL";_tag_fat(_tmp13F,sizeof(char),5U);}));goto _LL0;case 8U: _LL13: _tmp127=((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmpD6)->f1).String_c).val;_LL14: {struct _fat_ptr s=_tmp127;
# 577
Cyc_Absyndump_dump_char((int)'"');Cyc_Absyndump_dump_nospace(Cyc_Absynpp_string_escape(s));Cyc_Absyndump_dump_char((int)'"');
goto _LL0;}default: _LL15: _tmp126=((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmpD6)->f1).Wstring_c).val;_LL16: {struct _fat_ptr s=_tmp126;
# 580
Cyc_Absyndump_dump(({const char*_tmp140="L\"";_tag_fat(_tmp140,sizeof(char),3U);}));Cyc_Absyndump_dump_nospace(s);Cyc_Absyndump_dump_char((int)'"');goto _LL0;}}case 1U: _LL17: _tmp125=(void*)((struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct*)_tmpD6)->f1;_LL18: {void*b=_tmp125;
# 582
Cyc_Absyndump_dumpqvar(Cyc_Absyn_binding2qvar(b));goto _LL0;}case 2U: _LL19: _tmp124=((struct Cyc_Absyn_Pragma_e_Absyn_Raw_exp_struct*)_tmpD6)->f1;_LL1A: {struct _fat_ptr p=_tmp124;
# 585
Cyc_Absyndump_dump(({const char*_tmp141="__cyclone_pragma__(";_tag_fat(_tmp141,sizeof(char),20U);}));Cyc_Absyndump_dump_nospace(p);Cyc_Absyndump_dump_char((int)')');goto _LL0;}case 3U: _LL1B: _tmp122=((struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct*)_tmpD6)->f1;_tmp123=((struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct*)_tmpD6)->f2;_LL1C: {enum Cyc_Absyn_Primop p=_tmp122;struct Cyc_List_List*es=_tmp123;
# 588
struct _fat_ptr pstr=Cyc_Absynpp_prim2str(p);
{int _stmttmpD=Cyc_List_length(es);int _tmp142=_stmttmpD;switch(_tmp142){case 1U: _LL72: _LL73:
# 591
 if((int)p == (int)18U){
Cyc_Absyndump_dump(({const char*_tmp143="numelts(";_tag_fat(_tmp143,sizeof(char),9U);}));
Cyc_Absyndump_dumpexp((struct Cyc_Absyn_Exp*)((struct Cyc_List_List*)_check_null(es))->hd);
Cyc_Absyndump_dump(({const char*_tmp144=")";_tag_fat(_tmp144,sizeof(char),2U);}));}else{
# 596
Cyc_Absyndump_dump(pstr);
Cyc_Absyndump_dumpexp_prec(myprec,(struct Cyc_Absyn_Exp*)((struct Cyc_List_List*)_check_null(es))->hd);}
# 599
goto _LL71;case 2U: _LL74: _LL75:
# 601
 Cyc_Absyndump_dumpexp_prec(myprec,(struct Cyc_Absyn_Exp*)((struct Cyc_List_List*)_check_null(es))->hd);
Cyc_Absyndump_dump(pstr);
Cyc_Absyndump_dump_char((int)' ');
Cyc_Absyndump_dumpexp_prec(myprec,(struct Cyc_Absyn_Exp*)((struct Cyc_List_List*)_check_null(es->tl))->hd);
goto _LL71;default: _LL76: _LL77:
({void*_tmp145=0U;({int(*_tmp331)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp147)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmp147;});struct _fat_ptr _tmp330=({const char*_tmp146="Absyndump -- Bad number of arguments to primop";_tag_fat(_tmp146,sizeof(char),47U);});_tmp331(_tmp330,_tag_fat(_tmp145,sizeof(void*),0U));});});}_LL71:;}
# 608
goto _LL0;}case 4U: _LL1D: _tmp11F=((struct Cyc_Absyn_AssignOp_e_Absyn_Raw_exp_struct*)_tmpD6)->f1;_tmp120=((struct Cyc_Absyn_AssignOp_e_Absyn_Raw_exp_struct*)_tmpD6)->f2;_tmp121=((struct Cyc_Absyn_AssignOp_e_Absyn_Raw_exp_struct*)_tmpD6)->f3;_LL1E: {struct Cyc_Absyn_Exp*e1=_tmp11F;struct Cyc_Core_Opt*popt=_tmp120;struct Cyc_Absyn_Exp*e2=_tmp121;
# 611
Cyc_Absyndump_dumpexp_prec(myprec,e1);
if(popt != 0)
Cyc_Absyndump_dump(Cyc_Absynpp_prim2str((enum Cyc_Absyn_Primop)popt->v));
# 612
Cyc_Absyndump_dump_nospace(({const char*_tmp148="=";_tag_fat(_tmp148,sizeof(char),2U);}));
# 615
Cyc_Absyndump_dumpexp_prec(myprec,e2);
goto _LL0;}case 5U: switch(((struct Cyc_Absyn_Increment_e_Absyn_Raw_exp_struct*)_tmpD6)->f2){case Cyc_Absyn_PreInc: _LL1F: _tmp11E=((struct Cyc_Absyn_Increment_e_Absyn_Raw_exp_struct*)_tmpD6)->f1;_LL20: {struct Cyc_Absyn_Exp*e2=_tmp11E;
# 618
Cyc_Absyndump_dump(({const char*_tmp149="++";_tag_fat(_tmp149,sizeof(char),3U);}));Cyc_Absyndump_dumpexp_prec(myprec,e2);goto _LL0;}case Cyc_Absyn_PreDec: _LL21: _tmp11D=((struct Cyc_Absyn_Increment_e_Absyn_Raw_exp_struct*)_tmpD6)->f1;_LL22: {struct Cyc_Absyn_Exp*e2=_tmp11D;
Cyc_Absyndump_dump(({const char*_tmp14A="--";_tag_fat(_tmp14A,sizeof(char),3U);}));Cyc_Absyndump_dumpexp_prec(myprec,e2);goto _LL0;}case Cyc_Absyn_PostInc: _LL23: _tmp11C=((struct Cyc_Absyn_Increment_e_Absyn_Raw_exp_struct*)_tmpD6)->f1;_LL24: {struct Cyc_Absyn_Exp*e2=_tmp11C;
Cyc_Absyndump_dumpexp_prec(myprec,e2);Cyc_Absyndump_dump(({const char*_tmp14B="++";_tag_fat(_tmp14B,sizeof(char),3U);}));goto _LL0;}default: _LL25: _tmp11B=((struct Cyc_Absyn_Increment_e_Absyn_Raw_exp_struct*)_tmpD6)->f1;_LL26: {struct Cyc_Absyn_Exp*e2=_tmp11B;
Cyc_Absyndump_dumpexp_prec(myprec,e2);Cyc_Absyndump_dump(({const char*_tmp14C="--";_tag_fat(_tmp14C,sizeof(char),3U);}));goto _LL0;}}case 6U: _LL27: _tmp118=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmpD6)->f1;_tmp119=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmpD6)->f2;_tmp11A=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmpD6)->f3;_LL28: {struct Cyc_Absyn_Exp*e1=_tmp118;struct Cyc_Absyn_Exp*e2=_tmp119;struct Cyc_Absyn_Exp*e3=_tmp11A;
# 624
Cyc_Absyndump_dumploc(e->loc);
Cyc_Absyndump_dumpexp_prec(myprec,e1);
Cyc_Absyndump_dump_char((int)'?');Cyc_Absyndump_dumpexp_prec(0,e2);
Cyc_Absyndump_dump_char((int)':');Cyc_Absyndump_dumpexp_prec(myprec,e3);
goto _LL0;}case 7U: _LL29: _tmp116=((struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct*)_tmpD6)->f1;_tmp117=((struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct*)_tmpD6)->f2;_LL2A: {struct Cyc_Absyn_Exp*e1=_tmp116;struct Cyc_Absyn_Exp*e2=_tmp117;
# 631
Cyc_Absyndump_dumploc(e->loc);
Cyc_Absyndump_dumpexp_prec(myprec,e1);Cyc_Absyndump_dump(({const char*_tmp14D="&&";_tag_fat(_tmp14D,sizeof(char),3U);}));Cyc_Absyndump_dumpexp_prec(myprec,e2);goto _LL0;}case 8U: _LL2B: _tmp114=((struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct*)_tmpD6)->f1;_tmp115=((struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct*)_tmpD6)->f2;_LL2C: {struct Cyc_Absyn_Exp*e1=_tmp114;struct Cyc_Absyn_Exp*e2=_tmp115;
# 635
Cyc_Absyndump_dumploc(e->loc);
Cyc_Absyndump_dumpexp_prec(myprec,e1);Cyc_Absyndump_dump(({const char*_tmp14E="||";_tag_fat(_tmp14E,sizeof(char),3U);}));Cyc_Absyndump_dumpexp_prec(myprec,e2);goto _LL0;}case 9U: _LL2D: _tmp112=((struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct*)_tmpD6)->f1;_tmp113=((struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct*)_tmpD6)->f2;_LL2E: {struct Cyc_Absyn_Exp*e1=_tmp112;struct Cyc_Absyn_Exp*e2=_tmp113;
# 641
Cyc_Absyndump_dumploc(e->loc);
# 644
Cyc_Absyndump_dumpexp_prec(myprec - 1,e1);Cyc_Absyndump_dump_char((int)',');Cyc_Absyndump_dumpexp_prec(myprec - 1,e2);
goto _LL0;}case 10U: _LL2F: _tmp110=((struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct*)_tmpD6)->f1;_tmp111=((struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct*)_tmpD6)->f2;_LL30: {struct Cyc_Absyn_Exp*e1=_tmp110;struct Cyc_List_List*es=_tmp111;
# 648
Cyc_Absyndump_dumploc(e->loc);
Cyc_Absyndump_dumpexp_prec(myprec,e1);
Cyc_Absyndump_dump_nospace(({const char*_tmp14F="(";_tag_fat(_tmp14F,sizeof(char),2U);}));{
# 653
int old_generate_line_directives=Cyc_Absyndump_generate_line_directives;
Cyc_Absyndump_generate_line_directives=
(old_generate_line_directives && !(e->loc == (unsigned)0))&& !(e1->loc == (unsigned)0);
Cyc_Absyndump_dumpexps_prec(20,es);
Cyc_Absyndump_dump_nospace(({const char*_tmp150=")";_tag_fat(_tmp150,sizeof(char),2U);}));
Cyc_Absyndump_generate_line_directives=old_generate_line_directives;
goto _LL0;}}case 11U: _LL31: _tmp10F=((struct Cyc_Absyn_Throw_e_Absyn_Raw_exp_struct*)_tmpD6)->f1;_LL32: {struct Cyc_Absyn_Exp*e1=_tmp10F;
# 662
Cyc_Absyndump_dumploc(e->loc);Cyc_Absyndump_dump(({const char*_tmp151="throw";_tag_fat(_tmp151,sizeof(char),6U);}));Cyc_Absyndump_dumpexp_prec(myprec,e1);goto _LL0;}case 12U: _LL33: _tmp10E=((struct Cyc_Absyn_NoInstantiate_e_Absyn_Raw_exp_struct*)_tmpD6)->f1;_LL34: {struct Cyc_Absyn_Exp*e1=_tmp10E;
# 664
_tmp10D=e1;goto _LL36;}case 13U: _LL35: _tmp10D=((struct Cyc_Absyn_Instantiate_e_Absyn_Raw_exp_struct*)_tmpD6)->f1;_LL36: {struct Cyc_Absyn_Exp*e1=_tmp10D;
Cyc_Absyndump_dumpexp_prec(inprec,e1);goto _LL0;}case 14U: _LL37: _tmp10B=(void*)((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmpD6)->f1;_tmp10C=((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmpD6)->f2;_LL38: {void*t=_tmp10B;struct Cyc_Absyn_Exp*e1=_tmp10C;
# 668
Cyc_Absyndump_dump_char((int)'(');Cyc_Absyndump_dumptyp(t);Cyc_Absyndump_dump_char((int)')');Cyc_Absyndump_dumpexp_prec(myprec,e1);
goto _LL0;}case 15U: _LL39: _tmp10A=((struct Cyc_Absyn_Address_e_Absyn_Raw_exp_struct*)_tmpD6)->f1;_LL3A: {struct Cyc_Absyn_Exp*e1=_tmp10A;
# 671
Cyc_Absyndump_dump_char((int)'&');Cyc_Absyndump_dumpexp_prec(myprec,e1);goto _LL0;}case 20U: _LL3B: _tmp109=((struct Cyc_Absyn_Deref_e_Absyn_Raw_exp_struct*)_tmpD6)->f1;_LL3C: {struct Cyc_Absyn_Exp*e1=_tmp109;
Cyc_Absyndump_dump_char((int)'*');Cyc_Absyndump_dumpexp_prec(myprec,e1);goto _LL0;}case 16U: _LL3D: _tmp107=((struct Cyc_Absyn_New_e_Absyn_Raw_exp_struct*)_tmpD6)->f1;_tmp108=((struct Cyc_Absyn_New_e_Absyn_Raw_exp_struct*)_tmpD6)->f2;_LL3E: {struct Cyc_Absyn_Exp*ropt=_tmp107;struct Cyc_Absyn_Exp*e1=_tmp108;
Cyc_Absyndump_dump(({const char*_tmp152="new";_tag_fat(_tmp152,sizeof(char),4U);}));Cyc_Absyndump_dumpexp_prec(myprec,e1);goto _LL0;}case 17U: _LL3F: _tmp106=(void*)((struct Cyc_Absyn_Sizeoftype_e_Absyn_Raw_exp_struct*)_tmpD6)->f1;_LL40: {void*t=_tmp106;
Cyc_Absyndump_dump(({const char*_tmp153="sizeof(";_tag_fat(_tmp153,sizeof(char),8U);}));Cyc_Absyndump_dumptyp(t);Cyc_Absyndump_dump_char((int)')');goto _LL0;}case 18U: _LL41: _tmp105=((struct Cyc_Absyn_Sizeofexp_e_Absyn_Raw_exp_struct*)_tmpD6)->f1;_LL42: {struct Cyc_Absyn_Exp*e1=_tmp105;
Cyc_Absyndump_dump(({const char*_tmp154="sizeof(";_tag_fat(_tmp154,sizeof(char),8U);}));Cyc_Absyndump_dumpexp(e1);Cyc_Absyndump_dump_char((int)')');goto _LL0;}case 40U: _LL43: _tmp104=(void*)((struct Cyc_Absyn_Valueof_e_Absyn_Raw_exp_struct*)_tmpD6)->f1;_LL44: {void*t=_tmp104;
Cyc_Absyndump_dump(({const char*_tmp155="valueof(";_tag_fat(_tmp155,sizeof(char),9U);}));Cyc_Absyndump_dumptyp(t);Cyc_Absyndump_dump_char((int)')');goto _LL0;}case 42U: _LL45: _tmp103=((struct Cyc_Absyn_Extension_e_Absyn_Raw_exp_struct*)_tmpD6)->f1;_LL46: {struct Cyc_Absyn_Exp*e1=_tmp103;
# 679
Cyc_Absyndump_dump(({const char*_tmp156="__extension__(";_tag_fat(_tmp156,sizeof(char),15U);}));Cyc_Absyndump_dumpexp(e1);Cyc_Absyndump_dump_char((int)')');goto _LL0;}case 41U: _LL47: _tmpFE=((struct Cyc_Absyn_Asm_e_Absyn_Raw_exp_struct*)_tmpD6)->f1;_tmpFF=((struct Cyc_Absyn_Asm_e_Absyn_Raw_exp_struct*)_tmpD6)->f2;_tmp100=((struct Cyc_Absyn_Asm_e_Absyn_Raw_exp_struct*)_tmpD6)->f3;_tmp101=((struct Cyc_Absyn_Asm_e_Absyn_Raw_exp_struct*)_tmpD6)->f4;_tmp102=((struct Cyc_Absyn_Asm_e_Absyn_Raw_exp_struct*)_tmpD6)->f5;_LL48: {int vol=_tmpFE;struct _fat_ptr t=_tmpFF;struct Cyc_List_List*o=_tmp100;struct Cyc_List_List*i=_tmp101;struct Cyc_List_List*cl=_tmp102;
# 682
Cyc_Absyndump_dump(({const char*_tmp157="__asm__";_tag_fat(_tmp157,sizeof(char),8U);}));
if(vol)Cyc_Absyndump_dump(({const char*_tmp158=" volatile ";_tag_fat(_tmp158,sizeof(char),11U);}));Cyc_Absyndump_dump_nospace(({const char*_tmp159="(\"";_tag_fat(_tmp159,sizeof(char),3U);}));
Cyc_Absyndump_dump_nospace(t);Cyc_Absyndump_dump_char((int)'"');
if((unsigned)o){
Cyc_Absyndump_dump_char((int)':');
Cyc_Absyndump_dump_asm_iolist(o);}
# 685
if((unsigned)i){
# 690
if(!((unsigned)o))
Cyc_Absyndump_dump_char((int)':');
# 690
Cyc_Absyndump_dump_char((int)':');
# 693
Cyc_Absyndump_dump_asm_iolist(i);}
# 685
if((unsigned)cl){
# 696
int ncol=(unsigned)i?2:((unsigned)o?1: 0);
{int cols=0;for(0;cols < 3 - ncol;++ cols){
Cyc_Absyndump_dump_nospace(({const char*_tmp15A=" :";_tag_fat(_tmp15A,sizeof(char),3U);}));}}
while((unsigned)cl){
Cyc_Absyndump_dump_char((int)'"');
Cyc_Absyndump_dump_nospace(*((struct _fat_ptr*)cl->hd));
Cyc_Absyndump_dump_char((int)'"');
cl=cl->tl;
if((unsigned)cl)
Cyc_Absyndump_dump_char((int)',');}}
# 685
Cyc_Absyndump_dump_char((int)')');
# 709
goto _LL0;}case 39U: _LL49: _tmpFC=((struct Cyc_Absyn_Tagcheck_e_Absyn_Raw_exp_struct*)_tmpD6)->f1;_tmpFD=((struct Cyc_Absyn_Tagcheck_e_Absyn_Raw_exp_struct*)_tmpD6)->f2;_LL4A: {struct Cyc_Absyn_Exp*e=_tmpFC;struct _fat_ptr*f=_tmpFD;
# 712
Cyc_Absyndump_dump(({const char*_tmp15B="tagcheck(";_tag_fat(_tmp15B,sizeof(char),10U);}));Cyc_Absyndump_dumpexp(e);Cyc_Absyndump_dump_char((int)'.');Cyc_Absyndump_dump_nospace(*f);
Cyc_Absyndump_dump_char((int)')');
goto _LL0;}case 19U: _LL4B: _tmpFA=(void*)((struct Cyc_Absyn_Offsetof_e_Absyn_Raw_exp_struct*)_tmpD6)->f1;_tmpFB=((struct Cyc_Absyn_Offsetof_e_Absyn_Raw_exp_struct*)_tmpD6)->f2;_LL4C: {void*t=_tmpFA;struct Cyc_List_List*l=_tmpFB;
# 717
Cyc_Absyndump_dump(({const char*_tmp15C="offsetof(";_tag_fat(_tmp15C,sizeof(char),10U);}));Cyc_Absyndump_dumptyp(t);({struct Cyc_List_List*_tmp334=l;struct _fat_ptr _tmp333=({const char*_tmp15D=",";_tag_fat(_tmp15D,sizeof(char),2U);});struct _fat_ptr _tmp332=({const char*_tmp15E=")";_tag_fat(_tmp15E,sizeof(char),2U);});Cyc_Absyndump_group(Cyc_Absyndump_dumpoffset_field,_tmp334,_tmp333,_tmp332,({const char*_tmp15F=".";_tag_fat(_tmp15F,sizeof(char),2U);}));});goto _LL0;}case 21U: _LL4D: _tmpF8=((struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct*)_tmpD6)->f1;_tmpF9=((struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct*)_tmpD6)->f2;_LL4E: {struct Cyc_Absyn_Exp*e1=_tmpF8;struct _fat_ptr*n=_tmpF9;
# 720
Cyc_Absyndump_dumpexp_prec(myprec,e1);Cyc_Absyndump_dump_char((int)'.');Cyc_Absyndump_dump_nospace(*n);goto _LL0;}case 22U: _LL4F: _tmpF6=((struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct*)_tmpD6)->f1;_tmpF7=((struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct*)_tmpD6)->f2;_LL50: {struct Cyc_Absyn_Exp*e1=_tmpF6;struct _fat_ptr*n=_tmpF7;
# 722
Cyc_Absyndump_dumpexp_prec(myprec,e1);Cyc_Absyndump_dump_nospace(({const char*_tmp160="->";_tag_fat(_tmp160,sizeof(char),3U);}));Cyc_Absyndump_dump_nospace(*n);goto _LL0;}case 23U: _LL51: _tmpF4=((struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct*)_tmpD6)->f1;_tmpF5=((struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct*)_tmpD6)->f2;_LL52: {struct Cyc_Absyn_Exp*e1=_tmpF4;struct Cyc_Absyn_Exp*e2=_tmpF5;
# 725
Cyc_Absyndump_dumpexp_prec(myprec,e1);Cyc_Absyndump_dump_char((int)'[');Cyc_Absyndump_dumpexp(e2);Cyc_Absyndump_dump_char((int)']');goto _LL0;}case 24U: _LL53: _tmpF3=((struct Cyc_Absyn_Tuple_e_Absyn_Raw_exp_struct*)_tmpD6)->f1;_LL54: {struct Cyc_List_List*es=_tmpF3;
# 727
Cyc_Absyndump_dump(({const char*_tmp161="$(";_tag_fat(_tmp161,sizeof(char),3U);}));Cyc_Absyndump_dumpexps_prec(20,es);Cyc_Absyndump_dump_char((int)')');goto _LL0;}case 25U: _LL55: _tmpF1=((struct Cyc_Absyn_CompoundLit_e_Absyn_Raw_exp_struct*)_tmpD6)->f1;_tmpF2=((struct Cyc_Absyn_CompoundLit_e_Absyn_Raw_exp_struct*)_tmpD6)->f2;_LL56: {struct _tuple8*vat=_tmpF1;struct Cyc_List_List*des=_tmpF2;
# 730
Cyc_Absyndump_dump_char((int)'(');
Cyc_Absyndump_dumptyp((*vat).f3);
Cyc_Absyndump_dump_char((int)')');
({void(*_tmp338)(void(*f)(struct _tuple19*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep)=({void(*_tmp162)(void(*f)(struct _tuple19*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep)=(void(*)(void(*f)(struct _tuple19*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep))Cyc_Absyndump_group;_tmp162;});struct Cyc_List_List*_tmp337=des;struct _fat_ptr _tmp336=({const char*_tmp163="{";_tag_fat(_tmp163,sizeof(char),2U);});struct _fat_ptr _tmp335=({const char*_tmp164="}";_tag_fat(_tmp164,sizeof(char),2U);});_tmp338(Cyc_Absyndump_dumpde,_tmp337,_tmp336,_tmp335,({const char*_tmp165=",";_tag_fat(_tmp165,sizeof(char),2U);}));});
goto _LL0;}case 26U: _LL57: _tmpF0=((struct Cyc_Absyn_Array_e_Absyn_Raw_exp_struct*)_tmpD6)->f1;_LL58: {struct Cyc_List_List*des=_tmpF0;
# 736
({void(*_tmp33C)(void(*f)(struct _tuple19*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep)=({void(*_tmp166)(void(*f)(struct _tuple19*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep)=(void(*)(void(*f)(struct _tuple19*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep))Cyc_Absyndump_group;_tmp166;});struct Cyc_List_List*_tmp33B=des;struct _fat_ptr _tmp33A=({const char*_tmp167="{";_tag_fat(_tmp167,sizeof(char),2U);});struct _fat_ptr _tmp339=({const char*_tmp168="}";_tag_fat(_tmp168,sizeof(char),2U);});_tmp33C(Cyc_Absyndump_dumpde,_tmp33B,_tmp33A,_tmp339,({const char*_tmp169=",";_tag_fat(_tmp169,sizeof(char),2U);}));});goto _LL0;}case 27U: _LL59: _tmpED=((struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct*)_tmpD6)->f1;_tmpEE=((struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct*)_tmpD6)->f2;_tmpEF=((struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct*)_tmpD6)->f3;_LL5A: {struct Cyc_Absyn_Vardecl*vd=_tmpED;struct Cyc_Absyn_Exp*e1=_tmpEE;struct Cyc_Absyn_Exp*e2=_tmpEF;
# 739
Cyc_Absyndump_dump(({const char*_tmp16A="{for";_tag_fat(_tmp16A,sizeof(char),5U);}));Cyc_Absyndump_dump_str((*vd->name).f2);Cyc_Absyndump_dump_char((int)'<');Cyc_Absyndump_dumpexp(e1);
Cyc_Absyndump_dump_char((int)':');Cyc_Absyndump_dumpexp(e2);Cyc_Absyndump_dump_char((int)'}');
goto _LL0;}case 28U: _LL5B: _tmpEB=((struct Cyc_Absyn_ComprehensionNoinit_e_Absyn_Raw_exp_struct*)_tmpD6)->f1;_tmpEC=(void*)((struct Cyc_Absyn_ComprehensionNoinit_e_Absyn_Raw_exp_struct*)_tmpD6)->f2;_LL5C: {struct Cyc_Absyn_Exp*e=_tmpEB;void*t=_tmpEC;
# 744
Cyc_Absyndump_dump(({const char*_tmp16B="{for x ";_tag_fat(_tmp16B,sizeof(char),8U);}));Cyc_Absyndump_dump_char((int)'<');Cyc_Absyndump_dumpexp(e);
Cyc_Absyndump_dump_char((int)':');Cyc_Absyndump_dumptyp(t);Cyc_Absyndump_dump_char((int)'}');
goto _LL0;}case 29U: _LL5D: _tmpE8=((struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct*)_tmpD6)->f1;_tmpE9=((struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct*)_tmpD6)->f2;_tmpEA=((struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct*)_tmpD6)->f3;_LL5E: {struct _tuple0*n=_tmpE8;struct Cyc_List_List*ts=_tmpE9;struct Cyc_List_List*des=_tmpEA;
# 749
Cyc_Absyndump_dumpqvar(n);
Cyc_Absyndump_dump_char((int)'{');
if(ts != 0)
Cyc_Absyndump_dumptps(ts);
# 751
({void(*_tmp340)(void(*f)(struct _tuple19*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep)=({void(*_tmp16C)(void(*f)(struct _tuple19*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep)=(void(*)(void(*f)(struct _tuple19*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep))Cyc_Absyndump_group;_tmp16C;});struct Cyc_List_List*_tmp33F=des;struct _fat_ptr _tmp33E=({const char*_tmp16D="";_tag_fat(_tmp16D,sizeof(char),1U);});struct _fat_ptr _tmp33D=({const char*_tmp16E="}";_tag_fat(_tmp16E,sizeof(char),2U);});_tmp340(Cyc_Absyndump_dumpde,_tmp33F,_tmp33E,_tmp33D,({const char*_tmp16F=",";_tag_fat(_tmp16F,sizeof(char),2U);}));});
# 754
goto _LL0;}case 30U: _LL5F: _tmpE7=((struct Cyc_Absyn_AnonStruct_e_Absyn_Raw_exp_struct*)_tmpD6)->f2;_LL60: {struct Cyc_List_List*des=_tmpE7;
# 756
({void(*_tmp344)(void(*f)(struct _tuple19*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep)=({void(*_tmp170)(void(*f)(struct _tuple19*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep)=(void(*)(void(*f)(struct _tuple19*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep))Cyc_Absyndump_group;_tmp170;});struct Cyc_List_List*_tmp343=des;struct _fat_ptr _tmp342=({const char*_tmp171="{";_tag_fat(_tmp171,sizeof(char),2U);});struct _fat_ptr _tmp341=({const char*_tmp172="}";_tag_fat(_tmp172,sizeof(char),2U);});_tmp344(Cyc_Absyndump_dumpde,_tmp343,_tmp342,_tmp341,({const char*_tmp173=",";_tag_fat(_tmp173,sizeof(char),2U);}));});goto _LL0;}case 32U: _LL61: _tmpE6=((struct Cyc_Absyn_Enum_e_Absyn_Raw_exp_struct*)_tmpD6)->f2;_LL62: {struct Cyc_Absyn_Enumfield*ef=_tmpE6;
_tmpE5=ef;goto _LL64;}case 33U: _LL63: _tmpE5=((struct Cyc_Absyn_AnonEnum_e_Absyn_Raw_exp_struct*)_tmpD6)->f2;_LL64: {struct Cyc_Absyn_Enumfield*ef=_tmpE5;
Cyc_Absyndump_dumpqvar(ef->name);goto _LL0;}case 31U: _LL65: _tmpE3=((struct Cyc_Absyn_Datatype_e_Absyn_Raw_exp_struct*)_tmpD6)->f1;_tmpE4=((struct Cyc_Absyn_Datatype_e_Absyn_Raw_exp_struct*)_tmpD6)->f3;_LL66: {struct Cyc_List_List*es=_tmpE3;struct Cyc_Absyn_Datatypefield*tuf=_tmpE4;
# 761
Cyc_Absyndump_dumpqvar(tuf->name);
if(es != 0)({void(*_tmp348)(void(*f)(struct Cyc_Absyn_Exp*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep)=({void(*_tmp174)(void(*f)(struct Cyc_Absyn_Exp*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep)=(void(*)(void(*f)(struct Cyc_Absyn_Exp*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep))Cyc_Absyndump_group;_tmp174;});struct Cyc_List_List*_tmp347=es;struct _fat_ptr _tmp346=({const char*_tmp175="(";_tag_fat(_tmp175,sizeof(char),2U);});struct _fat_ptr _tmp345=({const char*_tmp176=")";_tag_fat(_tmp176,sizeof(char),2U);});_tmp348(Cyc_Absyndump_dumpexp,_tmp347,_tmp346,_tmp345,({const char*_tmp177=",";_tag_fat(_tmp177,sizeof(char),2U);}));});goto _LL0;}case 34U: _LL67: _tmpE1=((struct Cyc_Absyn_Spawn_e_Absyn_Raw_exp_struct*)_tmpD6)->f1;_tmpE2=((struct Cyc_Absyn_Spawn_e_Absyn_Raw_exp_struct*)_tmpD6)->f2;_LL68: {struct Cyc_Absyn_Exp*e1=_tmpE1;struct Cyc_Absyn_Exp*e2=_tmpE2;
# 766
Cyc_Absyndump_dumploc(e->loc);
Cyc_Absyndump_dumpexp_prec(myprec - 1,e1);Cyc_Absyndump_dump_char((int)',');Cyc_Absyndump_dumpexp_prec(myprec - 1,e2);
goto _LL0;}case 35U: _LL69: _tmpDC=(((struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct*)_tmpD6)->f1).is_calloc;_tmpDD=(((struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct*)_tmpD6)->f1).rgn;_tmpDE=(((struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct*)_tmpD6)->f1).elt_type;_tmpDF=(((struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct*)_tmpD6)->f1).num_elts;_tmpE0=(((struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct*)_tmpD6)->f1).inline_call;_LL6A: {int is_calloc=_tmpDC;struct Cyc_Absyn_Exp*ropt=_tmpDD;void**topt=_tmpDE;struct Cyc_Absyn_Exp*e=_tmpDF;int inline_call=_tmpE0;
# 771
Cyc_Absyndump_dumploc(e->loc);
if(is_calloc){
if(ropt != 0){
Cyc_Absyndump_dump(({const char*_tmp178="rcalloc(";_tag_fat(_tmp178,sizeof(char),9U);}));Cyc_Absyndump_dumpexp(ropt);Cyc_Absyndump_dump(({const char*_tmp179=",";_tag_fat(_tmp179,sizeof(char),2U);}));}else{
# 776
Cyc_Absyndump_dump(({const char*_tmp17A="calloc(";_tag_fat(_tmp17A,sizeof(char),8U);}));}
# 778
Cyc_Absyndump_dumpexp(e);Cyc_Absyndump_dump(({const char*_tmp17B=",";_tag_fat(_tmp17B,sizeof(char),2U);}));Cyc_Absyndump_dumpexp(Cyc_Absyn_sizeoftype_exp(*((void**)_check_null(topt)),0U));Cyc_Absyndump_dump(({const char*_tmp17C=")";_tag_fat(_tmp17C,sizeof(char),2U);}));}else{
# 780
if(ropt != 0){
Cyc_Absyndump_dump(inline_call?({const char*_tmp17D="rmalloc_inline(";_tag_fat(_tmp17D,sizeof(char),16U);}):({const char*_tmp17E="rmalloc(";_tag_fat(_tmp17E,sizeof(char),9U);}));
Cyc_Absyndump_dumpexp(ropt);Cyc_Absyndump_dump(({const char*_tmp17F=",";_tag_fat(_tmp17F,sizeof(char),2U);}));}else{
# 784
Cyc_Absyndump_dump(({const char*_tmp180="malloc(";_tag_fat(_tmp180,sizeof(char),8U);}));}
# 787
if(topt != 0)
Cyc_Absyndump_dumpexp(({struct Cyc_Absyn_Exp*_tmp349=Cyc_Absyn_sizeoftype_exp(*topt,0U);Cyc_Absyn_times_exp(_tmp349,e,0U);}));else{
# 790
Cyc_Absyndump_dumpexp(e);}
Cyc_Absyndump_dump(({const char*_tmp181=")";_tag_fat(_tmp181,sizeof(char),2U);}));}
# 793
goto _LL0;}case 36U: _LL6B: _tmpDA=((struct Cyc_Absyn_Swap_e_Absyn_Raw_exp_struct*)_tmpD6)->f1;_tmpDB=((struct Cyc_Absyn_Swap_e_Absyn_Raw_exp_struct*)_tmpD6)->f2;_LL6C: {struct Cyc_Absyn_Exp*e1=_tmpDA;struct Cyc_Absyn_Exp*e2=_tmpDB;
# 796
Cyc_Absyndump_dumpexp_prec(myprec,e1);Cyc_Absyndump_dump_nospace(({const char*_tmp182=":=:";_tag_fat(_tmp182,sizeof(char),4U);}));Cyc_Absyndump_dumpexp_prec(myprec,e2);goto _LL0;}case 37U: _LL6D: _tmpD8=((struct Cyc_Absyn_UnresolvedMem_e_Absyn_Raw_exp_struct*)_tmpD6)->f1;_tmpD9=((struct Cyc_Absyn_UnresolvedMem_e_Absyn_Raw_exp_struct*)_tmpD6)->f2;_LL6E: {struct Cyc_Core_Opt*n=_tmpD8;struct Cyc_List_List*des=_tmpD9;
# 799
({void(*_tmp34D)(void(*f)(struct _tuple19*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep)=({void(*_tmp183)(void(*f)(struct _tuple19*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep)=(void(*)(void(*f)(struct _tuple19*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep))Cyc_Absyndump_group;_tmp183;});struct Cyc_List_List*_tmp34C=des;struct _fat_ptr _tmp34B=({const char*_tmp184="{";_tag_fat(_tmp184,sizeof(char),2U);});struct _fat_ptr _tmp34A=({const char*_tmp185="}";_tag_fat(_tmp185,sizeof(char),2U);});_tmp34D(Cyc_Absyndump_dumpde,_tmp34C,_tmp34B,_tmp34A,({const char*_tmp186=",";_tag_fat(_tmp186,sizeof(char),2U);}));});goto _LL0;}default: _LL6F: _tmpD7=((struct Cyc_Absyn_StmtExp_e_Absyn_Raw_exp_struct*)_tmpD6)->f1;_LL70: {struct Cyc_Absyn_Stmt*s=_tmpD7;
# 802
Cyc_Absyndump_dump_nospace(({const char*_tmp187="({";_tag_fat(_tmp187,sizeof(char),3U);}));Cyc_Absyndump_dumpstmt(s,1,0);Cyc_Absyndump_dump_nospace(({const char*_tmp188="})";_tag_fat(_tmp188,sizeof(char),3U);}));goto _LL0;}}_LL0:;}
# 804
if(inprec >= myprec)
Cyc_Absyndump_dump_char((int)')');}
# 96
static void Cyc_Absyndump_dumpexp(struct Cyc_Absyn_Exp*e){
# 809
Cyc_Absyndump_dumpexp_prec(0,e);}
# 96
static void Cyc_Absyndump_dumpswitchclauses(struct Cyc_List_List*scs,struct Cyc_List_List*varsinblock){
# 814
for(0;scs != 0;scs=scs->tl){
struct Cyc_Absyn_Switch_clause*c=(struct Cyc_Absyn_Switch_clause*)scs->hd;
if(c->where_clause == 0 &&(c->pattern)->r == (void*)& Cyc_Absyn_Wild_p_val)
Cyc_Absyndump_dump(({const char*_tmp189="default:";_tag_fat(_tmp189,sizeof(char),9U);}));else{
# 819
Cyc_Absyndump_dump(({const char*_tmp18A="case";_tag_fat(_tmp18A,sizeof(char),5U);}));
Cyc_Absyndump_dumppat(c->pattern);
if(c->where_clause != 0){
Cyc_Absyndump_dump(({const char*_tmp18B="&&";_tag_fat(_tmp18B,sizeof(char),3U);}));
Cyc_Absyndump_dumpexp((struct Cyc_Absyn_Exp*)_check_null(c->where_clause));}
# 821
Cyc_Absyndump_dump_nospace(({const char*_tmp18C=":";_tag_fat(_tmp18C,sizeof(char),2U);}));}
# 827
if(Cyc_Absynpp_is_declaration(c->body)){
Cyc_Absyndump_dump(({const char*_tmp18D=" {";_tag_fat(_tmp18D,sizeof(char),3U);}));Cyc_Absyndump_dumpstmt(c->body,0,0),Cyc_Absyndump_dump_char((int)'}');}else{
# 830
Cyc_Absyndump_dumpstmt(c->body,0,varsinblock);}}}
# 96
static void Cyc_Absyndump_dumpstmt(struct Cyc_Absyn_Stmt*s,int expstmt,struct Cyc_List_List*varsinblock){
# 835
Cyc_Absyndump_dumploc(s->loc);{
void*_stmttmpE=s->r;void*_tmp18E=_stmttmpE;struct Cyc_List_List*_tmp18F;struct Cyc_Absyn_Stmt*_tmp191;struct _fat_ptr*_tmp190;struct Cyc_Absyn_Stmt*_tmp193;struct Cyc_Absyn_Decl*_tmp192;struct Cyc_List_List*_tmp195;struct Cyc_Absyn_Stmt*_tmp194;struct Cyc_List_List*_tmp197;struct Cyc_Absyn_Exp*_tmp196;struct _fat_ptr*_tmp198;struct Cyc_Absyn_Exp*_tmp19A;struct Cyc_Absyn_Stmt*_tmp199;struct Cyc_Absyn_Stmt*_tmp19E;struct Cyc_Absyn_Exp*_tmp19D;struct Cyc_Absyn_Exp*_tmp19C;struct Cyc_Absyn_Exp*_tmp19B;struct Cyc_Absyn_Stmt*_tmp1A0;struct Cyc_Absyn_Exp*_tmp19F;struct Cyc_Absyn_Stmt*_tmp1A3;struct Cyc_Absyn_Stmt*_tmp1A2;struct Cyc_Absyn_Exp*_tmp1A1;struct Cyc_Absyn_Exp*_tmp1A4;struct Cyc_Absyn_Stmt*_tmp1A6;struct Cyc_Absyn_Stmt*_tmp1A5;struct Cyc_Absyn_Exp*_tmp1A7;switch(*((int*)_tmp18E)){case 0U: _LL1: _LL2:
 Cyc_Absyndump_dump_semi();goto _LL0;case 1U: _LL3: _tmp1A7=((struct Cyc_Absyn_Exp_s_Absyn_Raw_stmt_struct*)_tmp18E)->f1;_LL4: {struct Cyc_Absyn_Exp*e=_tmp1A7;
Cyc_Absyndump_dumpexp_prec(- 100,e);Cyc_Absyndump_dump_semi();goto _LL0;}case 2U: _LL5: _tmp1A5=((struct Cyc_Absyn_Seq_s_Absyn_Raw_stmt_struct*)_tmp18E)->f1;_tmp1A6=((struct Cyc_Absyn_Seq_s_Absyn_Raw_stmt_struct*)_tmp18E)->f2;_LL6: {struct Cyc_Absyn_Stmt*s1=_tmp1A5;struct Cyc_Absyn_Stmt*s2=_tmp1A6;
# 843
if(Cyc_Absynpp_is_declaration(s1)){
Cyc_Absyndump_dump_char((int)'{');Cyc_Absyndump_dumpstmt(s1,0,0);Cyc_Absyndump_dump_char((int)'}');}else{
# 846
Cyc_Absyndump_dumpstmt(s1,0,varsinblock);}
if(Cyc_Absynpp_is_declaration(s2)){
if(expstmt)Cyc_Absyndump_dump_char((int)'(');Cyc_Absyndump_dump_char((int)'{');
Cyc_Absyndump_dumpstmt(s2,expstmt,0);Cyc_Absyndump_dump_char((int)'}');
if(expstmt){Cyc_Absyndump_dump_char((int)')');Cyc_Absyndump_dump_semi();}}else{
# 852
Cyc_Absyndump_dumpstmt(s2,expstmt,varsinblock);}
goto _LL0;}case 3U: if(((struct Cyc_Absyn_Return_s_Absyn_Raw_stmt_struct*)_tmp18E)->f1 == 0){_LL7: _LL8:
 Cyc_Absyndump_dump(({const char*_tmp1A8="return;";_tag_fat(_tmp1A8,sizeof(char),8U);}));goto _LL0;}else{_LL9: _tmp1A4=((struct Cyc_Absyn_Return_s_Absyn_Raw_stmt_struct*)_tmp18E)->f1;_LLA: {struct Cyc_Absyn_Exp*e=_tmp1A4;
Cyc_Absyndump_dump(({const char*_tmp1A9="return";_tag_fat(_tmp1A9,sizeof(char),7U);}));Cyc_Absyndump_dumpexp((struct Cyc_Absyn_Exp*)_check_null(e));Cyc_Absyndump_dump_semi();goto _LL0;}}case 4U: _LLB: _tmp1A1=((struct Cyc_Absyn_IfThenElse_s_Absyn_Raw_stmt_struct*)_tmp18E)->f1;_tmp1A2=((struct Cyc_Absyn_IfThenElse_s_Absyn_Raw_stmt_struct*)_tmp18E)->f2;_tmp1A3=((struct Cyc_Absyn_IfThenElse_s_Absyn_Raw_stmt_struct*)_tmp18E)->f3;_LLC: {struct Cyc_Absyn_Exp*e=_tmp1A1;struct Cyc_Absyn_Stmt*s1=_tmp1A2;struct Cyc_Absyn_Stmt*s2=_tmp1A3;
# 857
Cyc_Absyndump_dump(({const char*_tmp1AA="if(";_tag_fat(_tmp1AA,sizeof(char),4U);}));Cyc_Absyndump_dumpexp(e);
{void*_stmttmpF=s1->r;void*_tmp1AB=_stmttmpF;switch(*((int*)_tmp1AB)){case 2U: _LL26: _LL27:
 goto _LL29;case 12U: _LL28: _LL29:
 goto _LL2B;case 4U: _LL2A: _LL2B:
 goto _LL2D;case 13U: _LL2C: _LL2D:
# 863
 Cyc_Absyndump_dump_nospace(({const char*_tmp1AC="){";_tag_fat(_tmp1AC,sizeof(char),3U);}));Cyc_Absyndump_dumpstmt(s1,0,0);Cyc_Absyndump_dump_char((int)'}');goto _LL25;default: _LL2E: _LL2F:
 Cyc_Absyndump_dump_char((int)')');Cyc_Absyndump_dumpstmt(s1,0,varsinblock);}_LL25:;}
# 866
{void*_stmttmp10=s2->r;void*_tmp1AD=_stmttmp10;if(((struct Cyc_Absyn_Skip_s_Absyn_Raw_stmt_struct*)_tmp1AD)->tag == 0U){_LL31: _LL32:
 goto _LL30;}else{_LL33: _LL34:
 Cyc_Absyndump_dump(({const char*_tmp1AE="else{";_tag_fat(_tmp1AE,sizeof(char),6U);}));Cyc_Absyndump_dumpstmt(s2,0,0);Cyc_Absyndump_dump_char((int)'}');goto _LL30;}_LL30:;}
# 870
goto _LL0;}case 5U: _LLD: _tmp19F=(((struct Cyc_Absyn_While_s_Absyn_Raw_stmt_struct*)_tmp18E)->f1).f1;_tmp1A0=((struct Cyc_Absyn_While_s_Absyn_Raw_stmt_struct*)_tmp18E)->f2;_LLE: {struct Cyc_Absyn_Exp*e=_tmp19F;struct Cyc_Absyn_Stmt*s1=_tmp1A0;
# 873
Cyc_Absyndump_dump(({const char*_tmp1AF="while(";_tag_fat(_tmp1AF,sizeof(char),7U);}));Cyc_Absyndump_dumpexp(e);Cyc_Absyndump_dump_nospace(({const char*_tmp1B0="){";_tag_fat(_tmp1B0,sizeof(char),3U);}));
Cyc_Absyndump_dumpstmt(s1,0,0);Cyc_Absyndump_dump_char((int)'}');
goto _LL0;}case 9U: _LLF: _tmp19B=((struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct*)_tmp18E)->f1;_tmp19C=(((struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct*)_tmp18E)->f2).f1;_tmp19D=(((struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct*)_tmp18E)->f3).f1;_tmp19E=((struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct*)_tmp18E)->f4;_LL10: {struct Cyc_Absyn_Exp*e1=_tmp19B;struct Cyc_Absyn_Exp*e2=_tmp19C;struct Cyc_Absyn_Exp*e3=_tmp19D;struct Cyc_Absyn_Stmt*s1=_tmp19E;
# 877
Cyc_Absyndump_dump(({const char*_tmp1B1="for(";_tag_fat(_tmp1B1,sizeof(char),5U);}));Cyc_Absyndump_dumpexp(e1);Cyc_Absyndump_dump_semi();Cyc_Absyndump_dumpexp(e2);Cyc_Absyndump_dump_semi();Cyc_Absyndump_dumpexp(e3);
Cyc_Absyndump_dump_nospace(({const char*_tmp1B2="){";_tag_fat(_tmp1B2,sizeof(char),3U);}));Cyc_Absyndump_dumpstmt(s1,0,0);Cyc_Absyndump_dump_char((int)'}');
goto _LL0;}case 14U: _LL11: _tmp199=((struct Cyc_Absyn_Do_s_Absyn_Raw_stmt_struct*)_tmp18E)->f1;_tmp19A=(((struct Cyc_Absyn_Do_s_Absyn_Raw_stmt_struct*)_tmp18E)->f2).f1;_LL12: {struct Cyc_Absyn_Stmt*s1=_tmp199;struct Cyc_Absyn_Exp*e=_tmp19A;
# 881
Cyc_Absyndump_dump(({const char*_tmp1B3="do{";_tag_fat(_tmp1B3,sizeof(char),4U);}));Cyc_Absyndump_dumpstmt(s1,0,0);
Cyc_Absyndump_dump_nospace(({const char*_tmp1B4="}while(";_tag_fat(_tmp1B4,sizeof(char),8U);}));Cyc_Absyndump_dumpexp(e);Cyc_Absyndump_dump_nospace(({const char*_tmp1B5=");";_tag_fat(_tmp1B5,sizeof(char),3U);}));
goto _LL0;}case 6U: _LL13: _LL14:
# 885
 Cyc_Absyndump_dump(({const char*_tmp1B6="break;";_tag_fat(_tmp1B6,sizeof(char),7U);}));goto _LL0;case 7U: _LL15: _LL16:
 Cyc_Absyndump_dump(({const char*_tmp1B7="continue;";_tag_fat(_tmp1B7,sizeof(char),10U);}));goto _LL0;case 8U: _LL17: _tmp198=((struct Cyc_Absyn_Goto_s_Absyn_Raw_stmt_struct*)_tmp18E)->f1;_LL18: {struct _fat_ptr*x=_tmp198;
Cyc_Absyndump_dump(({const char*_tmp1B8="goto";_tag_fat(_tmp1B8,sizeof(char),5U);}));Cyc_Absyndump_dump_str(x);Cyc_Absyndump_dump_semi();goto _LL0;}case 10U: _LL19: _tmp196=((struct Cyc_Absyn_Switch_s_Absyn_Raw_stmt_struct*)_tmp18E)->f1;_tmp197=((struct Cyc_Absyn_Switch_s_Absyn_Raw_stmt_struct*)_tmp18E)->f2;_LL1A: {struct Cyc_Absyn_Exp*e=_tmp196;struct Cyc_List_List*ss=_tmp197;
# 890
Cyc_Absyndump_dump(({const char*_tmp1B9="switch(";_tag_fat(_tmp1B9,sizeof(char),8U);}));Cyc_Absyndump_dumpexp(e);Cyc_Absyndump_dump_nospace(({const char*_tmp1BA="){";_tag_fat(_tmp1BA,sizeof(char),3U);}));
Cyc_Absyndump_dumpswitchclauses(ss,varsinblock);
Cyc_Absyndump_dump_char((int)'}');
goto _LL0;}case 15U: _LL1B: _tmp194=((struct Cyc_Absyn_TryCatch_s_Absyn_Raw_stmt_struct*)_tmp18E)->f1;_tmp195=((struct Cyc_Absyn_TryCatch_s_Absyn_Raw_stmt_struct*)_tmp18E)->f2;_LL1C: {struct Cyc_Absyn_Stmt*s1=_tmp194;struct Cyc_List_List*ss=_tmp195;
# 895
Cyc_Absyndump_dump(({const char*_tmp1BB="try";_tag_fat(_tmp1BB,sizeof(char),4U);}));Cyc_Absyndump_dumpstmt(s1,0,varsinblock);
Cyc_Absyndump_dump(({const char*_tmp1BC="catch{";_tag_fat(_tmp1BC,sizeof(char),7U);}));
Cyc_Absyndump_dumpswitchclauses(ss,varsinblock);Cyc_Absyndump_dump_char((int)'}');
goto _LL0;}case 12U: _LL1D: _tmp192=((struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct*)_tmp18E)->f1;_tmp193=((struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct*)_tmp18E)->f2;_LL1E: {struct Cyc_Absyn_Decl*d=_tmp192;struct Cyc_Absyn_Stmt*s1=_tmp193;
# 901
struct _tuple13 _stmttmp11=Cyc_Absynpp_shadows(d,varsinblock);struct Cyc_List_List*_tmp1BE;int _tmp1BD;_LL36: _tmp1BD=_stmttmp11.f1;_tmp1BE=_stmttmp11.f2;_LL37: {int newblock=_tmp1BD;struct Cyc_List_List*newvarsinblock=_tmp1BE;
if(newblock){
if(expstmt)Cyc_Absyndump_dump(({const char*_tmp1BF="({";_tag_fat(_tmp1BF,sizeof(char),3U);}));else{Cyc_Absyndump_dump(({const char*_tmp1C0="{";_tag_fat(_tmp1C0,sizeof(char),2U);}));}
Cyc_Absyndump_dumpdecl(d);
Cyc_Absyndump_dumpstmt(s1,expstmt,0);
if(expstmt)Cyc_Absyndump_dump_nospace(({const char*_tmp1C1="});";_tag_fat(_tmp1C1,sizeof(char),4U);}));else{Cyc_Absyndump_dump(({const char*_tmp1C2="}";_tag_fat(_tmp1C2,sizeof(char),2U);}));}}else{
# 908
Cyc_Absyndump_dumpdecl(d);Cyc_Absyndump_dumpstmt(s1,expstmt,newvarsinblock);}
# 910
goto _LL0;}}case 13U: _LL1F: _tmp190=((struct Cyc_Absyn_Label_s_Absyn_Raw_stmt_struct*)_tmp18E)->f1;_tmp191=((struct Cyc_Absyn_Label_s_Absyn_Raw_stmt_struct*)_tmp18E)->f2;_LL20: {struct _fat_ptr*x=_tmp190;struct Cyc_Absyn_Stmt*s1=_tmp191;
# 916
if(Cyc_Absynpp_is_declaration(s1)){
Cyc_Absyndump_dump_str(x);
if(expstmt)Cyc_Absyndump_dump_nospace(({const char*_tmp1C3=": ({";_tag_fat(_tmp1C3,sizeof(char),5U);}));else{Cyc_Absyndump_dump_nospace(({const char*_tmp1C4=": {";_tag_fat(_tmp1C4,sizeof(char),4U);}));}
Cyc_Absyndump_dumpstmt(s1,expstmt,0);
if(expstmt)Cyc_Absyndump_dump_nospace(({const char*_tmp1C5="});";_tag_fat(_tmp1C5,sizeof(char),4U);}));else{Cyc_Absyndump_dump_char((int)'}');}}else{
# 922
Cyc_Absyndump_dump_str(x);Cyc_Absyndump_dump_char((int)':');Cyc_Absyndump_dumpstmt(s1,expstmt,varsinblock);}
# 924
goto _LL0;}default: if(((struct Cyc_Absyn_Fallthru_s_Absyn_Raw_stmt_struct*)_tmp18E)->f1 == 0){_LL21: _LL22:
# 926
 Cyc_Absyndump_dump(({const char*_tmp1C6="fallthru;";_tag_fat(_tmp1C6,sizeof(char),10U);}));goto _LL0;}else{_LL23: _tmp18F=((struct Cyc_Absyn_Fallthru_s_Absyn_Raw_stmt_struct*)_tmp18E)->f1;_LL24: {struct Cyc_List_List*es=_tmp18F;
# 928
Cyc_Absyndump_dump(({const char*_tmp1C7="fallthru(";_tag_fat(_tmp1C7,sizeof(char),10U);}));Cyc_Absyndump_dumpexps_prec(20,es);Cyc_Absyndump_dump_nospace(({const char*_tmp1C8=");";_tag_fat(_tmp1C8,sizeof(char),3U);}));goto _LL0;}}}_LL0:;}}struct _tuple21{struct Cyc_List_List*f1;struct Cyc_Absyn_Pat*f2;};
# 96
static void Cyc_Absyndump_dumpdp(struct _tuple21*dp){
# 933
({struct Cyc_List_List*_tmp350=(*dp).f1;struct _fat_ptr _tmp34F=({const char*_tmp1C9="";_tag_fat(_tmp1C9,sizeof(char),1U);});struct _fat_ptr _tmp34E=({const char*_tmp1CA="=";_tag_fat(_tmp1CA,sizeof(char),2U);});Cyc_Absyndump_egroup(Cyc_Absyndump_dumpdesignator,_tmp350,_tmp34F,_tmp34E,({const char*_tmp1CB="=";_tag_fat(_tmp1CB,sizeof(char),2U);}));});
Cyc_Absyndump_dumppat((*dp).f2);}
# 96
static struct _fat_ptr Cyc_Absyndump_pat_term(int dots){
# 936
return dots?({const char*_tmp1CC="...)";_tag_fat(_tmp1CC,sizeof(char),5U);}):({const char*_tmp1CD=")";_tag_fat(_tmp1CD,sizeof(char),2U);});}
# 96
static void Cyc_Absyndump_dumppat(struct Cyc_Absyn_Pat*p){
# 939
void*_stmttmp12=p->r;void*_tmp1CE=_stmttmp12;struct Cyc_Absyn_Exp*_tmp1CF;int _tmp1D2;struct Cyc_List_List*_tmp1D1;struct Cyc_Absyn_Datatypefield*_tmp1D0;int _tmp1D5;struct Cyc_List_List*_tmp1D4;struct Cyc_List_List*_tmp1D3;int _tmp1D9;struct Cyc_List_List*_tmp1D8;struct Cyc_List_List*_tmp1D7;union Cyc_Absyn_AggrInfo _tmp1D6;int _tmp1DC;struct Cyc_List_List*_tmp1DB;struct _tuple0*_tmp1DA;struct _tuple0*_tmp1DD;struct Cyc_Absyn_Vardecl*_tmp1DF;struct Cyc_Absyn_Tvar*_tmp1DE;struct Cyc_Absyn_Pat*_tmp1E0;int _tmp1E2;struct Cyc_List_List*_tmp1E1;struct Cyc_Absyn_Vardecl*_tmp1E4;struct Cyc_Absyn_Tvar*_tmp1E3;struct Cyc_Absyn_Pat*_tmp1E6;struct Cyc_Absyn_Vardecl*_tmp1E5;struct Cyc_Absyn_Vardecl*_tmp1E7;struct Cyc_Absyn_Pat*_tmp1E9;struct Cyc_Absyn_Vardecl*_tmp1E8;struct Cyc_Absyn_Vardecl*_tmp1EA;char _tmp1EB;struct Cyc_Absyn_Enumfield*_tmp1EC;struct Cyc_Absyn_Enumfield*_tmp1ED;struct _fat_ptr _tmp1EE;int _tmp1EF;int _tmp1F0;int _tmp1F1;switch(*((int*)_tmp1CE)){case 0U: _LL1: _LL2:
 Cyc_Absyndump_dump_char((int)'_');goto _LL0;case 9U: _LL3: _LL4:
 Cyc_Absyndump_dump(({const char*_tmp1F2="NULL";_tag_fat(_tmp1F2,sizeof(char),5U);}));goto _LL0;case 10U: switch(((struct Cyc_Absyn_Int_p_Absyn_Raw_pat_struct*)_tmp1CE)->f1){case Cyc_Absyn_None: _LL5: _tmp1F1=((struct Cyc_Absyn_Int_p_Absyn_Raw_pat_struct*)_tmp1CE)->f2;_LL6: {int i=_tmp1F1;
_tmp1F0=i;goto _LL8;}case Cyc_Absyn_Signed: _LL7: _tmp1F0=((struct Cyc_Absyn_Int_p_Absyn_Raw_pat_struct*)_tmp1CE)->f2;_LL8: {int i=_tmp1F0;
Cyc_Absyndump_dump((struct _fat_ptr)({struct Cyc_Int_pa_PrintArg_struct _tmp1F5=({struct Cyc_Int_pa_PrintArg_struct _tmp2ED;_tmp2ED.tag=1U,_tmp2ED.f1=(unsigned long)i;_tmp2ED;});void*_tmp1F3[1U];_tmp1F3[0]=& _tmp1F5;({struct _fat_ptr _tmp351=({const char*_tmp1F4="%d";_tag_fat(_tmp1F4,sizeof(char),3U);});Cyc_aprintf(_tmp351,_tag_fat(_tmp1F3,sizeof(void*),1U));});}));goto _LL0;}default: _LL9: _tmp1EF=((struct Cyc_Absyn_Int_p_Absyn_Raw_pat_struct*)_tmp1CE)->f2;_LLA: {int i=_tmp1EF;
Cyc_Absyndump_dump((struct _fat_ptr)({struct Cyc_Int_pa_PrintArg_struct _tmp1F8=({struct Cyc_Int_pa_PrintArg_struct _tmp2EE;_tmp2EE.tag=1U,_tmp2EE.f1=(unsigned)i;_tmp2EE;});void*_tmp1F6[1U];_tmp1F6[0]=& _tmp1F8;({struct _fat_ptr _tmp352=({const char*_tmp1F7="%u";_tag_fat(_tmp1F7,sizeof(char),3U);});Cyc_aprintf(_tmp352,_tag_fat(_tmp1F6,sizeof(void*),1U));});}));goto _LL0;}}case 12U: _LLB: _tmp1EE=((struct Cyc_Absyn_Float_p_Absyn_Raw_pat_struct*)_tmp1CE)->f1;_LLC: {struct _fat_ptr x=_tmp1EE;
Cyc_Absyndump_dump(x);goto _LL0;}case 13U: _LLD: _tmp1ED=((struct Cyc_Absyn_Enum_p_Absyn_Raw_pat_struct*)_tmp1CE)->f2;_LLE: {struct Cyc_Absyn_Enumfield*ef=_tmp1ED;
_tmp1EC=ef;goto _LL10;}case 14U: _LLF: _tmp1EC=((struct Cyc_Absyn_AnonEnum_p_Absyn_Raw_pat_struct*)_tmp1CE)->f2;_LL10: {struct Cyc_Absyn_Enumfield*ef=_tmp1EC;
Cyc_Absyndump_dumpqvar(ef->name);goto _LL0;}case 11U: _LL11: _tmp1EB=((struct Cyc_Absyn_Char_p_Absyn_Raw_pat_struct*)_tmp1CE)->f1;_LL12: {char ch=_tmp1EB;
# 949
Cyc_Absyndump_dump(({const char*_tmp1F9="'";_tag_fat(_tmp1F9,sizeof(char),2U);}));Cyc_Absyndump_dump_nospace(Cyc_Absynpp_char_escape(ch));Cyc_Absyndump_dump_nospace(({const char*_tmp1FA="'";_tag_fat(_tmp1FA,sizeof(char),2U);}));goto _LL0;}case 3U: if(((struct Cyc_Absyn_Wild_p_Absyn_Raw_pat_struct*)((struct Cyc_Absyn_Pat*)((struct Cyc_Absyn_Reference_p_Absyn_Raw_pat_struct*)_tmp1CE)->f2)->r)->tag == 0U){_LL13: _tmp1EA=((struct Cyc_Absyn_Reference_p_Absyn_Raw_pat_struct*)_tmp1CE)->f1;_LL14: {struct Cyc_Absyn_Vardecl*vd=_tmp1EA;
Cyc_Absyndump_dump(({const char*_tmp1FB="*";_tag_fat(_tmp1FB,sizeof(char),2U);}));_tmp1E7=vd;goto _LL16;}}else{_LL17: _tmp1E8=((struct Cyc_Absyn_Reference_p_Absyn_Raw_pat_struct*)_tmp1CE)->f1;_tmp1E9=((struct Cyc_Absyn_Reference_p_Absyn_Raw_pat_struct*)_tmp1CE)->f2;_LL18: {struct Cyc_Absyn_Vardecl*vd=_tmp1E8;struct Cyc_Absyn_Pat*p2=_tmp1E9;
# 952
Cyc_Absyndump_dump(({const char*_tmp1FC="*";_tag_fat(_tmp1FC,sizeof(char),2U);}));_tmp1E5=vd;_tmp1E6=p2;goto _LL1A;}}case 1U: if(((struct Cyc_Absyn_Wild_p_Absyn_Raw_pat_struct*)((struct Cyc_Absyn_Pat*)((struct Cyc_Absyn_Var_p_Absyn_Raw_pat_struct*)_tmp1CE)->f2)->r)->tag == 0U){_LL15: _tmp1E7=((struct Cyc_Absyn_Var_p_Absyn_Raw_pat_struct*)_tmp1CE)->f1;_LL16: {struct Cyc_Absyn_Vardecl*vd=_tmp1E7;
# 951
Cyc_Absyndump_dumpqvar(vd->name);goto _LL0;}}else{_LL19: _tmp1E5=((struct Cyc_Absyn_Var_p_Absyn_Raw_pat_struct*)_tmp1CE)->f1;_tmp1E6=((struct Cyc_Absyn_Var_p_Absyn_Raw_pat_struct*)_tmp1CE)->f2;_LL1A: {struct Cyc_Absyn_Vardecl*vd=_tmp1E5;struct Cyc_Absyn_Pat*p2=_tmp1E6;
# 953
Cyc_Absyndump_dumpqvar(vd->name);Cyc_Absyndump_dump(({const char*_tmp1FD=" as ";_tag_fat(_tmp1FD,sizeof(char),5U);}));Cyc_Absyndump_dumppat(p2);goto _LL0;}}case 2U: _LL1B: _tmp1E3=((struct Cyc_Absyn_AliasVar_p_Absyn_Raw_pat_struct*)_tmp1CE)->f1;_tmp1E4=((struct Cyc_Absyn_AliasVar_p_Absyn_Raw_pat_struct*)_tmp1CE)->f2;_LL1C: {struct Cyc_Absyn_Tvar*tv=_tmp1E3;struct Cyc_Absyn_Vardecl*vd=_tmp1E4;
# 955
Cyc_Absyndump_dump(({const char*_tmp1FE="alias";_tag_fat(_tmp1FE,sizeof(char),6U);}));Cyc_Absyndump_dump(({const char*_tmp1FF="<";_tag_fat(_tmp1FF,sizeof(char),2U);}));Cyc_Absyndump_dumptvar(tv);Cyc_Absyndump_dump(({const char*_tmp200=">";_tag_fat(_tmp200,sizeof(char),2U);}));
Cyc_Absyndump_dumpvardecl(vd,p->loc,0);
goto _LL0;}case 5U: _LL1D: _tmp1E1=((struct Cyc_Absyn_Tuple_p_Absyn_Raw_pat_struct*)_tmp1CE)->f1;_tmp1E2=((struct Cyc_Absyn_Tuple_p_Absyn_Raw_pat_struct*)_tmp1CE)->f2;_LL1E: {struct Cyc_List_List*ts=_tmp1E1;int dots=_tmp1E2;
({void(*_tmp356)(void(*f)(struct Cyc_Absyn_Pat*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep)=({void(*_tmp201)(void(*f)(struct Cyc_Absyn_Pat*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep)=(void(*)(void(*f)(struct Cyc_Absyn_Pat*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep))Cyc_Absyndump_group;_tmp201;});struct Cyc_List_List*_tmp355=ts;struct _fat_ptr _tmp354=({const char*_tmp202="$(";_tag_fat(_tmp202,sizeof(char),3U);});struct _fat_ptr _tmp353=Cyc_Absyndump_pat_term(dots);_tmp356(Cyc_Absyndump_dumppat,_tmp355,_tmp354,_tmp353,({const char*_tmp203=",";_tag_fat(_tmp203,sizeof(char),2U);}));});goto _LL0;}case 6U: _LL1F: _tmp1E0=((struct Cyc_Absyn_Pointer_p_Absyn_Raw_pat_struct*)_tmp1CE)->f1;_LL20: {struct Cyc_Absyn_Pat*p2=_tmp1E0;
Cyc_Absyndump_dump(({const char*_tmp204="&";_tag_fat(_tmp204,sizeof(char),2U);}));Cyc_Absyndump_dumppat(p2);goto _LL0;}case 4U: _LL21: _tmp1DE=((struct Cyc_Absyn_TagInt_p_Absyn_Raw_pat_struct*)_tmp1CE)->f1;_tmp1DF=((struct Cyc_Absyn_TagInt_p_Absyn_Raw_pat_struct*)_tmp1CE)->f2;_LL22: {struct Cyc_Absyn_Tvar*tv=_tmp1DE;struct Cyc_Absyn_Vardecl*vd=_tmp1DF;
# 961
Cyc_Absyndump_dumpqvar(vd->name);Cyc_Absyndump_dump_char((int)'<');Cyc_Absyndump_dumptvar(tv);Cyc_Absyndump_dump_char((int)'>');goto _LL0;}case 15U: _LL23: _tmp1DD=((struct Cyc_Absyn_UnknownId_p_Absyn_Raw_pat_struct*)_tmp1CE)->f1;_LL24: {struct _tuple0*q=_tmp1DD;
Cyc_Absyndump_dumpqvar(q);goto _LL0;}case 16U: _LL25: _tmp1DA=((struct Cyc_Absyn_UnknownCall_p_Absyn_Raw_pat_struct*)_tmp1CE)->f1;_tmp1DB=((struct Cyc_Absyn_UnknownCall_p_Absyn_Raw_pat_struct*)_tmp1CE)->f2;_tmp1DC=((struct Cyc_Absyn_UnknownCall_p_Absyn_Raw_pat_struct*)_tmp1CE)->f3;_LL26: {struct _tuple0*q=_tmp1DA;struct Cyc_List_List*ps=_tmp1DB;int dots=_tmp1DC;
# 964
Cyc_Absyndump_dumpqvar(q);({void(*_tmp35A)(void(*f)(struct Cyc_Absyn_Pat*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep)=({void(*_tmp205)(void(*f)(struct Cyc_Absyn_Pat*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep)=(void(*)(void(*f)(struct Cyc_Absyn_Pat*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep))Cyc_Absyndump_group;_tmp205;});struct Cyc_List_List*_tmp359=ps;struct _fat_ptr _tmp358=({const char*_tmp206="(";_tag_fat(_tmp206,sizeof(char),2U);});struct _fat_ptr _tmp357=Cyc_Absyndump_pat_term(dots);_tmp35A(Cyc_Absyndump_dumppat,_tmp359,_tmp358,_tmp357,({const char*_tmp207=",";_tag_fat(_tmp207,sizeof(char),2U);}));});goto _LL0;}case 7U: if(((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmp1CE)->f1 != 0){_LL27: _tmp1D6=*((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmp1CE)->f1;_tmp1D7=((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmp1CE)->f2;_tmp1D8=((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmp1CE)->f3;_tmp1D9=((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmp1CE)->f4;_LL28: {union Cyc_Absyn_AggrInfo info=_tmp1D6;struct Cyc_List_List*exists=_tmp1D7;struct Cyc_List_List*dps=_tmp1D8;int dots=_tmp1D9;
# 966
Cyc_Absyndump_dumpqvar((Cyc_Absyn_aggr_kinded_name(info)).f2);
_tmp1D3=exists;_tmp1D4=dps;_tmp1D5=dots;goto _LL2A;}}else{_LL29: _tmp1D3=((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmp1CE)->f2;_tmp1D4=((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmp1CE)->f3;_tmp1D5=((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmp1CE)->f4;_LL2A: {struct Cyc_List_List*exists=_tmp1D3;struct Cyc_List_List*dps=_tmp1D4;int dots=_tmp1D5;
# 969
Cyc_Absyndump_dump_char((int)'{');
({void(*_tmp35E)(void(*f)(struct Cyc_Absyn_Tvar*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep)=({void(*_tmp208)(void(*f)(struct Cyc_Absyn_Tvar*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep)=(void(*)(void(*f)(struct Cyc_Absyn_Tvar*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep))Cyc_Absyndump_egroup;_tmp208;});struct Cyc_List_List*_tmp35D=exists;struct _fat_ptr _tmp35C=({const char*_tmp209="<";_tag_fat(_tmp209,sizeof(char),2U);});struct _fat_ptr _tmp35B=({const char*_tmp20A=">";_tag_fat(_tmp20A,sizeof(char),2U);});_tmp35E(Cyc_Absyndump_dumptvar,_tmp35D,_tmp35C,_tmp35B,({const char*_tmp20B=",";_tag_fat(_tmp20B,sizeof(char),2U);}));});
({void(*_tmp362)(void(*f)(struct _tuple21*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep)=({void(*_tmp20C)(void(*f)(struct _tuple21*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep)=(void(*)(void(*f)(struct _tuple21*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep))Cyc_Absyndump_group;_tmp20C;});struct Cyc_List_List*_tmp361=dps;struct _fat_ptr _tmp360=({const char*_tmp20D="";_tag_fat(_tmp20D,sizeof(char),1U);});struct _fat_ptr _tmp35F=Cyc_Absyndump_pat_term(dots);_tmp362(Cyc_Absyndump_dumpdp,_tmp361,_tmp360,_tmp35F,({const char*_tmp20E=",";_tag_fat(_tmp20E,sizeof(char),2U);}));});
goto _LL0;}}case 8U: _LL2B: _tmp1D0=((struct Cyc_Absyn_Datatype_p_Absyn_Raw_pat_struct*)_tmp1CE)->f2;_tmp1D1=((struct Cyc_Absyn_Datatype_p_Absyn_Raw_pat_struct*)_tmp1CE)->f3;_tmp1D2=((struct Cyc_Absyn_Datatype_p_Absyn_Raw_pat_struct*)_tmp1CE)->f4;_LL2C: {struct Cyc_Absyn_Datatypefield*tuf=_tmp1D0;struct Cyc_List_List*ps=_tmp1D1;int dots=_tmp1D2;
# 974
Cyc_Absyndump_dumpqvar(tuf->name);
if(ps != 0)({void(*_tmp366)(void(*f)(struct Cyc_Absyn_Pat*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep)=({void(*_tmp20F)(void(*f)(struct Cyc_Absyn_Pat*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep)=(void(*)(void(*f)(struct Cyc_Absyn_Pat*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep))Cyc_Absyndump_group;_tmp20F;});struct Cyc_List_List*_tmp365=ps;struct _fat_ptr _tmp364=({const char*_tmp210="(";_tag_fat(_tmp210,sizeof(char),2U);});struct _fat_ptr _tmp363=Cyc_Absyndump_pat_term(dots);_tmp366(Cyc_Absyndump_dumppat,_tmp365,_tmp364,_tmp363,({const char*_tmp211=",";_tag_fat(_tmp211,sizeof(char),2U);}));});goto _LL0;}default: _LL2D: _tmp1CF=((struct Cyc_Absyn_Exp_p_Absyn_Raw_pat_struct*)_tmp1CE)->f1;_LL2E: {struct Cyc_Absyn_Exp*e=_tmp1CF;
# 977
Cyc_Absyndump_dumpexp(e);goto _LL0;}}_LL0:;}
# 96
static void Cyc_Absyndump_dumpdatatypefield(struct Cyc_Absyn_Datatypefield*ef){
# 982
Cyc_Absyndump_dumpqvar(ef->name);
if(ef->typs != 0)
Cyc_Absyndump_dumpargs(ef->typs);}
# 96
static void Cyc_Absyndump_dumpdatatypefields(struct Cyc_List_List*fields){
# 987
({void(*_tmp368)(void(*f)(struct Cyc_Absyn_Datatypefield*),struct Cyc_List_List*l,struct _fat_ptr sep)=({void(*_tmp212)(void(*f)(struct Cyc_Absyn_Datatypefield*),struct Cyc_List_List*l,struct _fat_ptr sep)=(void(*)(void(*f)(struct Cyc_Absyn_Datatypefield*),struct Cyc_List_List*l,struct _fat_ptr sep))Cyc_Absyndump_dump_sep;_tmp212;});struct Cyc_List_List*_tmp367=fields;_tmp368(Cyc_Absyndump_dumpdatatypefield,_tmp367,({const char*_tmp213=",";_tag_fat(_tmp213,sizeof(char),2U);}));});}
# 96
static void Cyc_Absyndump_dumpenumfield(struct Cyc_Absyn_Enumfield*ef){
# 990
Cyc_Absyndump_dumpqvar(ef->name);
if(ef->tag != 0){
Cyc_Absyndump_dump(({const char*_tmp214="=";_tag_fat(_tmp214,sizeof(char),2U);}));Cyc_Absyndump_dumpexp((struct Cyc_Absyn_Exp*)_check_null(ef->tag));}}
# 96
static void Cyc_Absyndump_dumpenumfields(struct Cyc_List_List*fields){
# 996
({void(*_tmp36A)(void(*f)(struct Cyc_Absyn_Enumfield*),struct Cyc_List_List*l,struct _fat_ptr sep)=({void(*_tmp215)(void(*f)(struct Cyc_Absyn_Enumfield*),struct Cyc_List_List*l,struct _fat_ptr sep)=(void(*)(void(*f)(struct Cyc_Absyn_Enumfield*),struct Cyc_List_List*l,struct _fat_ptr sep))Cyc_Absyndump_dump_sep;_tmp215;});struct Cyc_List_List*_tmp369=fields;_tmp36A(Cyc_Absyndump_dumpenumfield,_tmp369,({const char*_tmp216=",";_tag_fat(_tmp216,sizeof(char),2U);}));});}
# 96
static void Cyc_Absyndump_dumpaggrfields(struct Cyc_List_List*fields){
# 1000
for(0;fields != 0;fields=fields->tl){
struct Cyc_Absyn_Aggrfield*_stmttmp13=(struct Cyc_Absyn_Aggrfield*)fields->hd;struct Cyc_Absyn_Exp*_tmp21C;struct Cyc_List_List*_tmp21B;struct Cyc_Absyn_Exp*_tmp21A;void*_tmp219;struct Cyc_Absyn_Tqual _tmp218;struct _fat_ptr*_tmp217;_LL1: _tmp217=_stmttmp13->name;_tmp218=_stmttmp13->tq;_tmp219=_stmttmp13->type;_tmp21A=_stmttmp13->width;_tmp21B=_stmttmp13->attributes;_tmp21C=_stmttmp13->requires_clause;_LL2: {struct _fat_ptr*name=_tmp217;struct Cyc_Absyn_Tqual tq=_tmp218;void*type=_tmp219;struct Cyc_Absyn_Exp*width=_tmp21A;struct Cyc_List_List*atts=_tmp21B;struct Cyc_Absyn_Exp*req=_tmp21C;
{enum Cyc_Cyclone_C_Compilers _tmp21D=Cyc_Cyclone_c_compiler;if(_tmp21D == Cyc_Cyclone_Gcc_c){_LL4: _LL5:
({void(*_tmp36D)(struct Cyc_Absyn_Tqual,void*,void(*f)(struct _fat_ptr*),struct _fat_ptr*)=({void(*_tmp21E)(struct Cyc_Absyn_Tqual,void*,void(*f)(struct _fat_ptr*),struct _fat_ptr*)=(void(*)(struct Cyc_Absyn_Tqual,void*,void(*f)(struct _fat_ptr*),struct _fat_ptr*))Cyc_Absyndump_dumptqtd;_tmp21E;});struct Cyc_Absyn_Tqual _tmp36C=tq;void*_tmp36B=type;_tmp36D(_tmp36C,_tmp36B,Cyc_Absyndump_dump_str,name);});Cyc_Absyndump_dumpatts(atts);goto _LL3;}else{_LL6: _LL7:
 Cyc_Absyndump_dumpatts(atts);({void(*_tmp370)(struct Cyc_Absyn_Tqual,void*,void(*f)(struct _fat_ptr*),struct _fat_ptr*)=({void(*_tmp21F)(struct Cyc_Absyn_Tqual,void*,void(*f)(struct _fat_ptr*),struct _fat_ptr*)=(void(*)(struct Cyc_Absyn_Tqual,void*,void(*f)(struct _fat_ptr*),struct _fat_ptr*))Cyc_Absyndump_dumptqtd;_tmp21F;});struct Cyc_Absyn_Tqual _tmp36F=tq;void*_tmp36E=type;_tmp370(_tmp36F,_tmp36E,Cyc_Absyndump_dump_str,name);});goto _LL3;}_LL3:;}
# 1006
if((unsigned)req){
Cyc_Absyndump_dump(({const char*_tmp220="@requires ";_tag_fat(_tmp220,sizeof(char),11U);}));Cyc_Absyndump_dumpexp(req);}
# 1006
if(width != 0){
# 1010
Cyc_Absyndump_dump_char((int)':');Cyc_Absyndump_dumpexp(width);}
# 1006
Cyc_Absyndump_dump_semi();}}}
# 96
static void Cyc_Absyndump_dumptypedefname(struct Cyc_Absyn_Typedefdecl*td){
# 1017
Cyc_Absyndump_dumpqvar(td->name);
Cyc_Absyndump_dumptvars(td->tvs);}
# 96
static void Cyc_Absyndump_dump_atts_qvar(struct Cyc_Absyn_Fndecl*fd){
# 1022
Cyc_Absyndump_dumpatts((fd->i).attributes);
Cyc_Absyndump_dumpqvar(fd->name);}struct _tuple22{void*f1;struct _tuple0*f2;};
# 96
static void Cyc_Absyndump_dump_callconv_qvar(struct _tuple22*pr){
# 1026
{void*_stmttmp14=(*pr).f1;void*_tmp221=_stmttmp14;switch(*((int*)_tmp221)){case 11U: _LL1: _LL2:
 goto _LL0;case 1U: _LL3: _LL4:
 Cyc_Absyndump_dump(({const char*_tmp222="_stdcall";_tag_fat(_tmp222,sizeof(char),9U);}));goto _LL0;case 2U: _LL5: _LL6:
 Cyc_Absyndump_dump(({const char*_tmp223="_cdecl";_tag_fat(_tmp223,sizeof(char),7U);}));goto _LL0;case 3U: _LL7: _LL8:
 Cyc_Absyndump_dump(({const char*_tmp224="_fastcall";_tag_fat(_tmp224,sizeof(char),10U);}));goto _LL0;default: _LL9: _LLA:
 goto _LL0;}_LL0:;}
# 1033
Cyc_Absyndump_dumpqvar((*pr).f2);}
# 96
static void Cyc_Absyndump_dump_callconv_fdqvar(struct Cyc_Absyn_Fndecl*fd){
# 1036
Cyc_Absyndump_dump_callconv((fd->i).attributes);
Cyc_Absyndump_dumpqvar(fd->name);}
# 96
static void Cyc_Absyndump_dumpid(struct Cyc_Absyn_Vardecl*vd){
# 1040
Cyc_Absyndump_dumpqvar(vd->name);}
# 96
static int Cyc_Absyndump_is_char_ptr(void*t){
# 1045
void*_tmp225=t;void*_tmp226;void*_tmp227;switch(*((int*)_tmp225)){case 1U: _LL1: _tmp227=(void*)((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp225)->f2;if(_tmp227 != 0){_LL2: {void*def=_tmp227;
return Cyc_Absyndump_is_char_ptr(def);}}else{goto _LL5;}case 3U: _LL3: _tmp226=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp225)->f1).elt_type;_LL4: {void*elt_typ=_tmp226;
# 1048
while(1){
void*_tmp228=elt_typ;void*_tmp229;void*_tmp22A;switch(*((int*)_tmp228)){case 1U: _LL8: _tmp22A=(void*)((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp228)->f2;if(_tmp22A != 0){_LL9: {void*t=_tmp22A;
elt_typ=t;goto _LL7;}}else{goto _LLE;}case 8U: _LLA: _tmp229=(void*)((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmp228)->f4;if(_tmp229 != 0){_LLB: {void*t=_tmp229;
elt_typ=t;goto _LL7;}}else{goto _LLE;}case 0U: if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp228)->f1)->tag == 1U){if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp228)->f1)->f2 == Cyc_Absyn_Char_sz){_LLC: _LLD:
 return 1;}else{goto _LLE;}}else{goto _LLE;}default: _LLE: _LLF:
 return 0;}_LL7:;}}default: _LL5: _LL6:
# 1055
 return 0;}_LL0:;}
# 96
static void Cyc_Absyndump_dumpvardecl(struct Cyc_Absyn_Vardecl*vd,unsigned loc,int do_semi){
# 1060
struct Cyc_List_List*_tmp231;struct Cyc_Absyn_Exp*_tmp230;void*_tmp22F;struct Cyc_Absyn_Tqual _tmp22E;unsigned _tmp22D;struct _tuple0*_tmp22C;enum Cyc_Absyn_Scope _tmp22B;_LL1: _tmp22B=vd->sc;_tmp22C=vd->name;_tmp22D=vd->varloc;_tmp22E=vd->tq;_tmp22F=vd->type;_tmp230=vd->initializer;_tmp231=vd->attributes;_LL2: {enum Cyc_Absyn_Scope sc=_tmp22B;struct _tuple0*name=_tmp22C;unsigned varloc=_tmp22D;struct Cyc_Absyn_Tqual tq=_tmp22E;void*type=_tmp22F;struct Cyc_Absyn_Exp*initializer=_tmp230;struct Cyc_List_List*atts=_tmp231;
{enum Cyc_Cyclone_C_Compilers _tmp232=Cyc_Cyclone_c_compiler;if(_tmp232 == Cyc_Cyclone_Gcc_c){_LL4: _LL5:
# 1064
 if((int)sc == (int)3U && Cyc_Absyndump_qvar_to_Cids){
void*_stmttmp15=Cyc_Tcutil_compress(type);void*_tmp233=_stmttmp15;if(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp233)->tag == 5U){_LL9: _LLA:
 goto _LL8;}else{_LLB: _LLC:
 Cyc_Absyndump_dumpscope(sc);}_LL8:;}else{
# 1070
Cyc_Absyndump_dumpscope(sc);}
({void(*_tmp373)(struct Cyc_Absyn_Tqual,void*,void(*f)(struct _tuple0*),struct _tuple0*)=({void(*_tmp234)(struct Cyc_Absyn_Tqual,void*,void(*f)(struct _tuple0*),struct _tuple0*)=(void(*)(struct Cyc_Absyn_Tqual,void*,void(*f)(struct _tuple0*),struct _tuple0*))Cyc_Absyndump_dumptqtd;_tmp234;});struct Cyc_Absyn_Tqual _tmp372=tq;void*_tmp371=type;_tmp373(_tmp372,_tmp371,Cyc_Absyndump_dumpqvar,name);});
Cyc_Absyndump_dumpatts(atts);
goto _LL3;}else{_LL6: _LL7:
# 1075
 Cyc_Absyndump_dumpatts(atts);
Cyc_Absyndump_dumpscope(sc);{
struct _RegionHandle _tmp235=_new_region("temp");struct _RegionHandle*temp=& _tmp235;_push_region(temp);
{int is_cp=Cyc_Absyndump_is_char_ptr(type);
struct _tuple12 _stmttmp16=Cyc_Absynpp_to_tms(temp,tq,type);struct Cyc_List_List*_tmp238;void*_tmp237;struct Cyc_Absyn_Tqual _tmp236;_LLE: _tmp236=_stmttmp16.f1;_tmp237=_stmttmp16.f2;_tmp238=_stmttmp16.f3;_LLF: {struct Cyc_Absyn_Tqual tq=_tmp236;void*t=_tmp237;struct Cyc_List_List*tms=_tmp238;
# 1081
void*call_conv=(void*)& Cyc_Absyn_Unused_att_val;
{struct Cyc_List_List*tms2=tms;for(0;tms2 != 0;tms2=tms2->tl){
void*_stmttmp17=(void*)tms2->hd;void*_tmp239=_stmttmp17;struct Cyc_List_List*_tmp23A;if(((struct Cyc_Absyn_Attributes_mod_Absyn_Type_modifier_struct*)_tmp239)->tag == 5U){_LL11: _tmp23A=((struct Cyc_Absyn_Attributes_mod_Absyn_Type_modifier_struct*)_tmp239)->f2;_LL12: {struct Cyc_List_List*atts=_tmp23A;
# 1085
for(0;atts != 0;atts=atts->tl){
void*_stmttmp18=(void*)atts->hd;void*_tmp23B=_stmttmp18;switch(*((int*)_tmp23B)){case 1U: _LL16: _LL17:
 call_conv=(void*)& Cyc_Absyn_Stdcall_att_val;goto _LL15;case 2U: _LL18: _LL19:
 call_conv=(void*)& Cyc_Absyn_Cdecl_att_val;goto _LL15;case 3U: _LL1A: _LL1B:
 call_conv=(void*)& Cyc_Absyn_Fastcall_att_val;goto _LL15;default: _LL1C: _LL1D:
 goto _LL15;}_LL15:;}
# 1092
goto _LL10;}}else{_LL13: _LL14:
 goto _LL10;}_LL10:;}}
# 1095
Cyc_Absyndump_dumptq(tq);
Cyc_Absyndump_dumpntyp(t);{
struct _tuple22 pr=({struct _tuple22 _tmp2EF;_tmp2EF.f1=call_conv,_tmp2EF.f2=name;_tmp2EF;});
({void(*_tmp375)(int is_char_ptr,struct Cyc_List_List*,void(*f)(struct _tuple22*),struct _tuple22*a)=({void(*_tmp23C)(int is_char_ptr,struct Cyc_List_List*,void(*f)(struct _tuple22*),struct _tuple22*a)=(void(*)(int is_char_ptr,struct Cyc_List_List*,void(*f)(struct _tuple22*),struct _tuple22*a))Cyc_Absyndump_dumptms;_tmp23C;});int _tmp374=is_cp;_tmp375(_tmp374,Cyc_List_imp_rev(tms),Cyc_Absyndump_dump_callconv_qvar,& pr);});
_npop_handler(0U);goto _LL3;}}}
# 1078
;_pop_region(temp);}}_LL3:;}
# 1102
if(initializer != 0){
Cyc_Absyndump_dump_char((int)'=');Cyc_Absyndump_dumpexp(initializer);}
# 1102
if(do_semi)
# 1105
Cyc_Absyndump_dump_semi();}}
# 96
static void Cyc_Absyndump_dump_aggrdecl(struct Cyc_Absyn_Aggrdecl*ad){
# 1109
Cyc_Absyndump_dumpscope(ad->sc);
if(ad->impl != 0 &&((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->tagged)
Cyc_Absyndump_dump(({const char*_tmp23D="@tagged ";_tag_fat(_tmp23D,sizeof(char),9U);}));
# 1110
Cyc_Absyndump_dumpaggr_kind(ad->kind);
# 1112
Cyc_Absyndump_dumpqvar(ad->name);Cyc_Absyndump_dumpkindedtvars(ad->tvs);
if(ad->impl != 0){
Cyc_Absyndump_dump_char((int)'{');
({void(*_tmp379)(void(*f)(struct Cyc_Absyn_Tvar*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep)=({void(*_tmp23E)(void(*f)(struct Cyc_Absyn_Tvar*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep)=(void(*)(void(*f)(struct Cyc_Absyn_Tvar*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep))Cyc_Absyndump_egroup;_tmp23E;});struct Cyc_List_List*_tmp378=((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->exist_vars;struct _fat_ptr _tmp377=({const char*_tmp23F="<";_tag_fat(_tmp23F,sizeof(char),2U);});struct _fat_ptr _tmp376=({const char*_tmp240=">";_tag_fat(_tmp240,sizeof(char),2U);});_tmp379(Cyc_Absyndump_dumpkindedtvar,_tmp378,_tmp377,_tmp376,({const char*_tmp241=",";_tag_fat(_tmp241,sizeof(char),2U);}));});
if(((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->rgn_po != 0){
Cyc_Absyndump_dump_char((int)':');Cyc_Absyndump_dump_rgnpo(((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->rgn_po);}
# 1116
Cyc_Absyndump_dumpaggrfields(((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->fields);
# 1120
Cyc_Absyndump_dump_char((int)'}');
Cyc_Absyndump_dumpatts(ad->attributes);}}
# 96
static void Cyc_Absyndump_dump_enumdecl(struct Cyc_Absyn_Enumdecl*ed){
# 1125
struct Cyc_Core_Opt*_tmp244;struct _tuple0*_tmp243;enum Cyc_Absyn_Scope _tmp242;_LL1: _tmp242=ed->sc;_tmp243=ed->name;_tmp244=ed->fields;_LL2: {enum Cyc_Absyn_Scope sc=_tmp242;struct _tuple0*nm=_tmp243;struct Cyc_Core_Opt*fields=_tmp244;
Cyc_Absyndump_dumpscope(sc);Cyc_Absyndump_dump(({const char*_tmp245="enum ";_tag_fat(_tmp245,sizeof(char),6U);}));Cyc_Absyndump_dumpqvar(nm);
if(fields != 0){
Cyc_Absyndump_dump_char((int)'{');Cyc_Absyndump_dumpenumfields((struct Cyc_List_List*)fields->v);Cyc_Absyndump_dump_char((int)'}');}}}
# 96
static void Cyc_Absyndump_dump_datatypedecl(struct Cyc_Absyn_Datatypedecl*dd){
# 1132
int _tmp24A;struct Cyc_Core_Opt*_tmp249;struct Cyc_List_List*_tmp248;struct _tuple0*_tmp247;enum Cyc_Absyn_Scope _tmp246;_LL1: _tmp246=dd->sc;_tmp247=dd->name;_tmp248=dd->tvs;_tmp249=dd->fields;_tmp24A=dd->is_extensible;_LL2: {enum Cyc_Absyn_Scope sc=_tmp246;struct _tuple0*name=_tmp247;struct Cyc_List_List*tvs=_tmp248;struct Cyc_Core_Opt*fields=_tmp249;int is_x=_tmp24A;
Cyc_Absyndump_dumpscope(sc);
if(is_x)
Cyc_Absyndump_dump(({const char*_tmp24B="@extensible ";_tag_fat(_tmp24B,sizeof(char),13U);}));
# 1134
Cyc_Absyndump_dump(({const char*_tmp24C="datatype ";_tag_fat(_tmp24C,sizeof(char),10U);}));
# 1136
Cyc_Absyndump_dumpqvar(name);Cyc_Absyndump_dumptvars(tvs);
if(fields != 0){
Cyc_Absyndump_dump_char((int)'{');Cyc_Absyndump_dumpdatatypefields((struct Cyc_List_List*)fields->v);Cyc_Absyndump_dump_char((int)'}');}}}struct _tuple23{unsigned f1;struct _tuple0*f2;int f3;};
# 96
static void Cyc_Absyndump_dumpexport(struct _tuple23*tup){
# 1143
Cyc_Absyndump_dumpqvar((*tup).f2);}
# 96
static void Cyc_Absyndump_dump_braced_decls(struct Cyc_List_List*tdl){
# 1146
Cyc_Absyndump_dump_char((int)'{');
for(0;tdl != 0;tdl=tdl->tl){
Cyc_Absyndump_dumpdecl((struct Cyc_Absyn_Decl*)tdl->hd);}
Cyc_Absyndump_dump_char((int)'}');}
# 96
static void Cyc_Absyndump_dumpdecl(struct Cyc_Absyn_Decl*d){
# 1153
Cyc_Absyndump_dumploc(d->loc);{
void*_stmttmp19=d->r;void*_tmp24D=_stmttmp19;struct _tuple10*_tmp251;struct Cyc_List_List*_tmp250;struct Cyc_List_List*_tmp24F;struct Cyc_List_List*_tmp24E;struct Cyc_List_List*_tmp252;struct Cyc_List_List*_tmp254;struct _tuple0*_tmp253;struct Cyc_List_List*_tmp256;struct _fat_ptr*_tmp255;struct Cyc_Absyn_Typedefdecl*_tmp257;struct Cyc_Absyn_Exp*_tmp25A;struct Cyc_Absyn_Vardecl*_tmp259;struct Cyc_Absyn_Tvar*_tmp258;struct Cyc_Absyn_Exp*_tmp25C;struct Cyc_Absyn_Pat*_tmp25B;struct Cyc_List_List*_tmp25D;struct Cyc_Absyn_Enumdecl*_tmp25E;struct Cyc_Absyn_Datatypedecl*_tmp25F;struct Cyc_Absyn_Aggrdecl*_tmp260;struct Cyc_Absyn_Vardecl*_tmp261;struct Cyc_Absyn_Fndecl*_tmp262;switch(*((int*)_tmp24D)){case 1U: _LL1: _tmp262=((struct Cyc_Absyn_Fn_d_Absyn_Raw_decl_struct*)_tmp24D)->f1;_LL2: {struct Cyc_Absyn_Fndecl*fd=_tmp262;
# 1156
{enum Cyc_Cyclone_C_Compilers _tmp263=Cyc_Cyclone_c_compiler;if(_tmp263 == Cyc_Cyclone_Vc_c){_LL24: _LL25:
 Cyc_Absyndump_dumpatts((fd->i).attributes);goto _LL23;}else{_LL26: _LL27:
 goto _LL23;}_LL23:;}
# 1160
if(fd->is_inline){
enum Cyc_Cyclone_C_Compilers _tmp264=Cyc_Cyclone_c_compiler;if(_tmp264 == Cyc_Cyclone_Vc_c){_LL29: _LL2A:
 Cyc_Absyndump_dump(({const char*_tmp265="__inline";_tag_fat(_tmp265,sizeof(char),9U);}));goto _LL28;}else{_LL2B: _LL2C:
 Cyc_Absyndump_dump(({const char*_tmp266="inline";_tag_fat(_tmp266,sizeof(char),7U);}));goto _LL28;}_LL28:;}
# 1160
Cyc_Absyndump_dumpscope(fd->sc);{
# 1167
struct Cyc_Absyn_FnInfo type_info=fd->i;
type_info.attributes=0;{
void*t=(void*)({struct Cyc_Absyn_FnType_Absyn_Type_struct*_tmp26F=_cycalloc(sizeof(*_tmp26F));_tmp26F->tag=5U,_tmp26F->f1=type_info;_tmp26F;});
if(fd->cached_type != 0){
void*_stmttmp1A=Cyc_Tcutil_compress((void*)_check_null(fd->cached_type));void*_tmp267=_stmttmp1A;struct Cyc_Absyn_FnInfo _tmp268;if(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp267)->tag == 5U){_LL2E: _tmp268=((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp267)->f1;_LL2F: {struct Cyc_Absyn_FnInfo i=_tmp268;
# 1173
({struct Cyc_List_List*_tmp37A=Cyc_List_append((fd->i).attributes,i.attributes);(fd->i).attributes=_tmp37A;});goto _LL2D;}}else{_LL30: _LL31:
({void*_tmp269=0U;({int(*_tmp37C)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp26B)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmp26B;});struct _fat_ptr _tmp37B=({const char*_tmp26A="function has non-function type";_tag_fat(_tmp26A,sizeof(char),31U);});_tmp37C(_tmp37B,_tag_fat(_tmp269,sizeof(void*),0U));});});}_LL2D:;}
# 1170
{enum Cyc_Cyclone_C_Compilers _tmp26C=Cyc_Cyclone_c_compiler;if(_tmp26C == Cyc_Cyclone_Gcc_c){_LL33: _LL34:
# 1178
({void(*_tmp37F)(struct Cyc_Absyn_Tqual,void*,void(*f)(struct Cyc_Absyn_Fndecl*),struct Cyc_Absyn_Fndecl*)=({void(*_tmp26D)(struct Cyc_Absyn_Tqual,void*,void(*f)(struct Cyc_Absyn_Fndecl*),struct Cyc_Absyn_Fndecl*)=(void(*)(struct Cyc_Absyn_Tqual,void*,void(*f)(struct Cyc_Absyn_Fndecl*),struct Cyc_Absyn_Fndecl*))Cyc_Absyndump_dumptqtd;_tmp26D;});struct Cyc_Absyn_Tqual _tmp37E=Cyc_Absyn_empty_tqual(0U);void*_tmp37D=t;_tmp37F(_tmp37E,_tmp37D,Cyc_Absyndump_dump_atts_qvar,fd);});goto _LL32;}else{_LL35: _LL36:
({void(*_tmp382)(struct Cyc_Absyn_Tqual,void*,void(*f)(struct Cyc_Absyn_Fndecl*),struct Cyc_Absyn_Fndecl*)=({void(*_tmp26E)(struct Cyc_Absyn_Tqual,void*,void(*f)(struct Cyc_Absyn_Fndecl*),struct Cyc_Absyn_Fndecl*)=(void(*)(struct Cyc_Absyn_Tqual,void*,void(*f)(struct Cyc_Absyn_Fndecl*),struct Cyc_Absyn_Fndecl*))Cyc_Absyndump_dumptqtd;_tmp26E;});struct Cyc_Absyn_Tqual _tmp381=Cyc_Absyn_empty_tqual(0U);void*_tmp380=t;_tmp382(_tmp381,_tmp380,Cyc_Absyndump_dump_callconv_fdqvar,fd);});goto _LL32;}_LL32:;}
# 1181
Cyc_Absyndump_dump_char((int)'{');
Cyc_Absyndump_dumpstmt(fd->body,0,0);
Cyc_Absyndump_dump_char((int)'}');
goto _LL0;}}}case 0U: _LL3: _tmp261=((struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct*)_tmp24D)->f1;_LL4: {struct Cyc_Absyn_Vardecl*vd=_tmp261;
Cyc_Absyndump_dumpvardecl(vd,d->loc,1);goto _LL0;}case 5U: _LL5: _tmp260=((struct Cyc_Absyn_Aggr_d_Absyn_Raw_decl_struct*)_tmp24D)->f1;_LL6: {struct Cyc_Absyn_Aggrdecl*ad=_tmp260;
Cyc_Absyndump_dump_aggrdecl(ad);Cyc_Absyndump_dump_semi();goto _LL0;}case 6U: _LL7: _tmp25F=((struct Cyc_Absyn_Datatype_d_Absyn_Raw_decl_struct*)_tmp24D)->f1;_LL8: {struct Cyc_Absyn_Datatypedecl*dd=_tmp25F;
Cyc_Absyndump_dump_datatypedecl(dd);Cyc_Absyndump_dump_semi();goto _LL0;}case 7U: _LL9: _tmp25E=((struct Cyc_Absyn_Enum_d_Absyn_Raw_decl_struct*)_tmp24D)->f1;_LLA: {struct Cyc_Absyn_Enumdecl*ed=_tmp25E;
Cyc_Absyndump_dump_enumdecl(ed);Cyc_Absyndump_dump_semi();goto _LL0;}case 3U: _LLB: _tmp25D=((struct Cyc_Absyn_Letv_d_Absyn_Raw_decl_struct*)_tmp24D)->f1;_LLC: {struct Cyc_List_List*vds=_tmp25D;
({void(*_tmp386)(void(*f)(struct Cyc_Absyn_Vardecl*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep)=({void(*_tmp270)(void(*f)(struct Cyc_Absyn_Vardecl*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep)=(void(*)(void(*f)(struct Cyc_Absyn_Vardecl*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep))Cyc_Absyndump_group;_tmp270;});struct Cyc_List_List*_tmp385=vds;struct _fat_ptr _tmp384=({const char*_tmp271=" let ";_tag_fat(_tmp271,sizeof(char),6U);});struct _fat_ptr _tmp383=({const char*_tmp272=";";_tag_fat(_tmp272,sizeof(char),2U);});_tmp386(Cyc_Absyndump_dumpid,_tmp385,_tmp384,_tmp383,({const char*_tmp273=",";_tag_fat(_tmp273,sizeof(char),2U);}));});goto _LL0;}case 2U: _LLD: _tmp25B=((struct Cyc_Absyn_Let_d_Absyn_Raw_decl_struct*)_tmp24D)->f1;_tmp25C=((struct Cyc_Absyn_Let_d_Absyn_Raw_decl_struct*)_tmp24D)->f3;_LLE: {struct Cyc_Absyn_Pat*p=_tmp25B;struct Cyc_Absyn_Exp*e=_tmp25C;
# 1191
Cyc_Absyndump_dump(({const char*_tmp274="let";_tag_fat(_tmp274,sizeof(char),4U);}));Cyc_Absyndump_dumppat(p);Cyc_Absyndump_dump_char((int)'=');Cyc_Absyndump_dumpexp(e);Cyc_Absyndump_dump_semi();goto _LL0;}case 4U: _LLF: _tmp258=((struct Cyc_Absyn_Region_d_Absyn_Raw_decl_struct*)_tmp24D)->f1;_tmp259=((struct Cyc_Absyn_Region_d_Absyn_Raw_decl_struct*)_tmp24D)->f2;_tmp25A=((struct Cyc_Absyn_Region_d_Absyn_Raw_decl_struct*)_tmp24D)->f3;_LL10: {struct Cyc_Absyn_Tvar*tv=_tmp258;struct Cyc_Absyn_Vardecl*vd=_tmp259;struct Cyc_Absyn_Exp*open_exp_opt=_tmp25A;
# 1193
Cyc_Absyndump_dump(({const char*_tmp275="region";_tag_fat(_tmp275,sizeof(char),7U);}));Cyc_Absyndump_dump(({const char*_tmp276="<";_tag_fat(_tmp276,sizeof(char),2U);}));Cyc_Absyndump_dumptvar(tv);Cyc_Absyndump_dump(({const char*_tmp277="> ";_tag_fat(_tmp277,sizeof(char),3U);}));Cyc_Absyndump_dumpqvar(vd->name);
if((unsigned)open_exp_opt){
Cyc_Absyndump_dump(({const char*_tmp278=" = open(";_tag_fat(_tmp278,sizeof(char),9U);}));Cyc_Absyndump_dumpexp(open_exp_opt);Cyc_Absyndump_dump(({const char*_tmp279=")";_tag_fat(_tmp279,sizeof(char),2U);}));}
# 1194
Cyc_Absyndump_dump_semi();
# 1198
goto _LL0;}case 8U: _LL11: _tmp257=((struct Cyc_Absyn_Typedef_d_Absyn_Raw_decl_struct*)_tmp24D)->f1;_LL12: {struct Cyc_Absyn_Typedefdecl*td=_tmp257;
# 1204
Cyc_Absyndump_dump(({const char*_tmp27A="typedef";_tag_fat(_tmp27A,sizeof(char),8U);}));{
void*t=td->defn == 0?Cyc_Absyn_new_evar(td->kind,0):(void*)_check_null(td->defn);
({void(*_tmp389)(struct Cyc_Absyn_Tqual,void*,void(*f)(struct Cyc_Absyn_Typedefdecl*),struct Cyc_Absyn_Typedefdecl*)=({void(*_tmp27B)(struct Cyc_Absyn_Tqual,void*,void(*f)(struct Cyc_Absyn_Typedefdecl*),struct Cyc_Absyn_Typedefdecl*)=(void(*)(struct Cyc_Absyn_Tqual,void*,void(*f)(struct Cyc_Absyn_Typedefdecl*),struct Cyc_Absyn_Typedefdecl*))Cyc_Absyndump_dumptqtd;_tmp27B;});struct Cyc_Absyn_Tqual _tmp388=td->tq;void*_tmp387=t;_tmp389(_tmp388,_tmp387,Cyc_Absyndump_dumptypedefname,td);});
Cyc_Absyndump_dumpatts(td->atts);
Cyc_Absyndump_dump_semi();
# 1210
goto _LL0;}}case 9U: _LL13: _tmp255=((struct Cyc_Absyn_Namespace_d_Absyn_Raw_decl_struct*)_tmp24D)->f1;_tmp256=((struct Cyc_Absyn_Namespace_d_Absyn_Raw_decl_struct*)_tmp24D)->f2;_LL14: {struct _fat_ptr*v=_tmp255;struct Cyc_List_List*tdl=_tmp256;
# 1212
Cyc_Absyndump_dump(({const char*_tmp27C="namespace";_tag_fat(_tmp27C,sizeof(char),10U);}));Cyc_Absyndump_dump_str(v);Cyc_Absyndump_dump_braced_decls(tdl);goto _LL0;}case 10U: _LL15: _tmp253=((struct Cyc_Absyn_Using_d_Absyn_Raw_decl_struct*)_tmp24D)->f1;_tmp254=((struct Cyc_Absyn_Using_d_Absyn_Raw_decl_struct*)_tmp24D)->f2;_LL16: {struct _tuple0*q=_tmp253;struct Cyc_List_List*tdl=_tmp254;
# 1214
Cyc_Absyndump_dump(({const char*_tmp27D="using";_tag_fat(_tmp27D,sizeof(char),6U);}));Cyc_Absyndump_dumpqvar(q);Cyc_Absyndump_dump_braced_decls(tdl);goto _LL0;}case 11U: _LL17: _tmp252=((struct Cyc_Absyn_ExternC_d_Absyn_Raw_decl_struct*)_tmp24D)->f1;_LL18: {struct Cyc_List_List*tdl=_tmp252;
# 1216
Cyc_Absyndump_dump(({const char*_tmp27E="extern \"C\" ";_tag_fat(_tmp27E,sizeof(char),12U);}));Cyc_Absyndump_dump_braced_decls(tdl);goto _LL0;}case 12U: _LL19: _tmp24E=((struct Cyc_Absyn_ExternCinclude_d_Absyn_Raw_decl_struct*)_tmp24D)->f1;_tmp24F=((struct Cyc_Absyn_ExternCinclude_d_Absyn_Raw_decl_struct*)_tmp24D)->f2;_tmp250=((struct Cyc_Absyn_ExternCinclude_d_Absyn_Raw_decl_struct*)_tmp24D)->f3;_tmp251=((struct Cyc_Absyn_ExternCinclude_d_Absyn_Raw_decl_struct*)_tmp24D)->f4;_LL1A: {struct Cyc_List_List*tdl=_tmp24E;struct Cyc_List_List*ovrs=_tmp24F;struct Cyc_List_List*exs=_tmp250;struct _tuple10*wc=_tmp251;
# 1218
Cyc_Absyndump_dump(({const char*_tmp27F="extern \"C include\" ";_tag_fat(_tmp27F,sizeof(char),20U);}));Cyc_Absyndump_dump_braced_decls(tdl);
if(ovrs != 0){
Cyc_Absyndump_dump(({const char*_tmp280=" cyclone_override ";_tag_fat(_tmp280,sizeof(char),19U);}));
Cyc_Absyndump_dump_braced_decls(ovrs);}
# 1219
if((unsigned)wc){
# 1224
Cyc_Absyndump_dump(({const char*_tmp281=" export { * }\n";_tag_fat(_tmp281,sizeof(char),15U);}));{
struct Cyc_List_List*_tmp282;_LL38: _tmp282=wc->f2;_LL39: {struct Cyc_List_List*hides=_tmp282;
if((unsigned)hides)
({void(*_tmp38D)(void(*f)(struct _tuple0*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep)=({void(*_tmp283)(void(*f)(struct _tuple0*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep)=(void(*)(void(*f)(struct _tuple0*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep))Cyc_Absyndump_egroup;_tmp283;});struct Cyc_List_List*_tmp38C=hides;struct _fat_ptr _tmp38B=({const char*_tmp284=" hide {";_tag_fat(_tmp284,sizeof(char),8U);});struct _fat_ptr _tmp38A=({const char*_tmp285="}";_tag_fat(_tmp285,sizeof(char),2U);});_tmp38D(Cyc_Absyndump_dumpqvar,_tmp38C,_tmp38B,_tmp38A,({const char*_tmp286=",";_tag_fat(_tmp286,sizeof(char),2U);}));});}}}else{
# 1231
({void(*_tmp391)(void(*f)(struct _tuple23*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep)=({void(*_tmp287)(void(*f)(struct _tuple23*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep)=(void(*)(void(*f)(struct _tuple23*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep))Cyc_Absyndump_egroup;_tmp287;});struct Cyc_List_List*_tmp390=exs;struct _fat_ptr _tmp38F=({const char*_tmp288=" export {";_tag_fat(_tmp288,sizeof(char),10U);});struct _fat_ptr _tmp38E=({const char*_tmp289="}";_tag_fat(_tmp289,sizeof(char),2U);});_tmp391(Cyc_Absyndump_dumpexport,_tmp390,_tmp38F,_tmp38E,({const char*_tmp28A=",";_tag_fat(_tmp28A,sizeof(char),2U);}));});}
# 1233
goto _LL0;}case 13U: _LL1B: _LL1C:
 Cyc_Absyndump_dump(({const char*_tmp28B=" __cyclone_port_on__; ";_tag_fat(_tmp28B,sizeof(char),23U);}));goto _LL0;case 14U: _LL1D: _LL1E:
 Cyc_Absyndump_dump(({const char*_tmp28C=" __cyclone_port_off__; ";_tag_fat(_tmp28C,sizeof(char),24U);}));goto _LL0;case 15U: _LL1F: _LL20:
 Cyc_Absyndump_dump(({const char*_tmp28D=" __tempest_on__; ";_tag_fat(_tmp28D,sizeof(char),18U);}));goto _LL0;default: _LL21: _LL22:
 Cyc_Absyndump_dump(({const char*_tmp28E=" __tempest_off__; ";_tag_fat(_tmp28E,sizeof(char),19U);}));goto _LL0;}_LL0:;}}
# 96
static void Cyc_Absyndump_dump_upperbound(struct Cyc_Absyn_Exp*e){
# 1242
struct _tuple14 pr=Cyc_Evexp_eval_const_uint_exp(e);
if(pr.f1 != (unsigned)1 || !pr.f2){
Cyc_Absyndump_dump_char((int)'{');Cyc_Absyndump_dumpexp(e);Cyc_Absyndump_dump_char((int)'}');}}
# 96
static void Cyc_Absyndump_dumptms(int is_char_ptr,struct Cyc_List_List*tms,void(*f)(void*),void*a){
# 1250
if(tms == 0){
f(a);
return;}{
# 1250
void*_stmttmp1B=(void*)tms->hd;void*_tmp28F=_stmttmp1B;struct Cyc_Absyn_Tqual _tmp294;void*_tmp293;void*_tmp292;void*_tmp291;void*_tmp290;if(((struct Cyc_Absyn_Pointer_mod_Absyn_Type_modifier_struct*)_tmp28F)->tag == 2U){_LL1: _tmp290=(((struct Cyc_Absyn_Pointer_mod_Absyn_Type_modifier_struct*)_tmp28F)->f1).rgn;_tmp291=(((struct Cyc_Absyn_Pointer_mod_Absyn_Type_modifier_struct*)_tmp28F)->f1).nullable;_tmp292=(((struct Cyc_Absyn_Pointer_mod_Absyn_Type_modifier_struct*)_tmp28F)->f1).bounds;_tmp293=(((struct Cyc_Absyn_Pointer_mod_Absyn_Type_modifier_struct*)_tmp28F)->f1).zero_term;_tmp294=((struct Cyc_Absyn_Pointer_mod_Absyn_Type_modifier_struct*)_tmp28F)->f2;_LL2: {void*rgn=_tmp290;void*nullable=_tmp291;void*bd=_tmp292;void*zt=_tmp293;struct Cyc_Absyn_Tqual tq2=_tmp294;
# 1258
{void*_stmttmp1C=Cyc_Tcutil_compress(bd);void*_tmp295=_stmttmp1C;struct Cyc_Absyn_Exp*_tmp296;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp295)->tag == 0U)switch(*((int*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp295)->f1)){case 14U: _LL6: _LL7:
 Cyc_Absyndump_dump_char((int)'?');goto _LL5;case 13U: if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp295)->f2 != 0){if(((struct Cyc_Absyn_ValueofType_Absyn_Type_struct*)((struct Cyc_List_List*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp295)->f2)->hd)->tag == 9U){_LL8: _tmp296=((struct Cyc_Absyn_ValueofType_Absyn_Type_struct*)(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp295)->f2)->hd)->f1;_LL9: {struct Cyc_Absyn_Exp*e=_tmp296;
# 1261
Cyc_Absyndump_dump_char((int)(Cyc_Absyn_type2bool(1,nullable)?'*':'@'));Cyc_Absyndump_dump_upperbound(e);goto _LL5;}}else{goto _LLA;}}else{goto _LLA;}default: goto _LLA;}else{_LLA: _LLB:
# 1263
 Cyc_Absyndump_dump_char((int)(Cyc_Absyn_type2bool(1,nullable)?'*':'@'));}_LL5:;}
# 1265
if((!Cyc_Absyndump_qvar_to_Cids && !is_char_ptr)&& Cyc_Absyn_type2bool(0,zt))Cyc_Absyndump_dump(({const char*_tmp297="@zeroterm";_tag_fat(_tmp297,sizeof(char),10U);}));if(
(!Cyc_Absyndump_qvar_to_Cids && is_char_ptr)&& !Cyc_Absyn_type2bool(0,zt))Cyc_Absyndump_dump(({const char*_tmp298="@nozeroterm";_tag_fat(_tmp298,sizeof(char),12U);}));
# 1265
{void*_stmttmp1D=
# 1267
Cyc_Tcutil_compress(rgn);
# 1265
void*_tmp299=_stmttmp1D;struct Cyc_Absyn_Tvar*_tmp29A;switch(*((int*)_tmp299)){case 0U: switch(*((int*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp299)->f1)){case 5U: _LLD: _LLE:
# 1268
 if(!Cyc_Absyndump_qvar_to_Cids)Cyc_Absyndump_dump(({const char*_tmp29B="`H";_tag_fat(_tmp29B,sizeof(char),3U);}));goto _LLC;case 6U: _LLF: _LL10:
 Cyc_Absyndump_dump(({const char*_tmp29C="`U";_tag_fat(_tmp29C,sizeof(char),3U);}));goto _LLC;case 7U: _LL11: _LL12:
 Cyc_Absyndump_dump(({const char*_tmp29D="`RC";_tag_fat(_tmp29D,sizeof(char),4U);}));goto _LLC;default: goto _LL19;}case 2U: _LL13: _tmp29A=((struct Cyc_Absyn_VarType_Absyn_Type_struct*)_tmp299)->f1;_LL14: {struct Cyc_Absyn_Tvar*tv=_tmp29A;
Cyc_Absyndump_dumptvar(tv);goto _LLC;}case 1U: if(((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp299)->f2 == 0){_LL15: _LL16:
 Cyc_Absyndump_dumpntyp(Cyc_Tcutil_compress(rgn));goto _LLC;}else{goto _LL19;}case 8U: _LL17: _LL18:
 Cyc_Absyndump_dump(({const char*_tmp29E="@region(";_tag_fat(_tmp29E,sizeof(char),9U);}));Cyc_Absyndump_dumptyp(rgn);Cyc_Absyndump_dump(({const char*_tmp29F=")";_tag_fat(_tmp29F,sizeof(char),2U);}));goto _LLC;default: _LL19: _LL1A:
({void*_tmp2A0=0U;({struct _fat_ptr _tmp392=({const char*_tmp2A1="dumptms: bad rgn type in Pointer_mod";_tag_fat(_tmp2A1,sizeof(char),37U);});Cyc_Warn_impos(_tmp392,_tag_fat(_tmp2A0,sizeof(void*),0U));});});}_LLC:;}
# 1276
Cyc_Absyndump_dumptq(tq2);
Cyc_Absyndump_dumptms(0,tms->tl,f,a);
return;}}else{_LL3: _LL4: {
# 1281
int next_is_pointer=0;
if(tms->tl != 0){
void*_stmttmp1E=(void*)((struct Cyc_List_List*)_check_null(tms->tl))->hd;void*_tmp2A2=_stmttmp1E;if(((struct Cyc_Absyn_Pointer_mod_Absyn_Type_modifier_struct*)_tmp2A2)->tag == 2U){_LL1C: _LL1D:
 next_is_pointer=1;goto _LL1B;}else{_LL1E: _LL1F:
 goto _LL1B;}_LL1B:;}
# 1282
if(next_is_pointer)
# 1288
Cyc_Absyndump_dump_char((int)'(');
# 1282
Cyc_Absyndump_dumptms(0,tms->tl,f,a);
# 1290
if(next_is_pointer)
Cyc_Absyndump_dump_char((int)')');
# 1290
{void*_stmttmp1F=(void*)tms->hd;void*_tmp2A3=_stmttmp1F;struct Cyc_List_List*_tmp2A4;int _tmp2A7;unsigned _tmp2A6;struct Cyc_List_List*_tmp2A5;unsigned _tmp2A9;struct Cyc_List_List*_tmp2A8;struct Cyc_Absyn_Exp*_tmp2B0;struct Cyc_Absyn_Exp*_tmp2AF;struct Cyc_List_List*_tmp2AE;void*_tmp2AD;struct Cyc_Absyn_VarargInfo*_tmp2AC;int _tmp2AB;struct Cyc_List_List*_tmp2AA;void*_tmp2B2;struct Cyc_Absyn_Exp*_tmp2B1;void*_tmp2B3;switch(*((int*)_tmp2A3)){case 0U: _LL21: _tmp2B3=(void*)((struct Cyc_Absyn_Carray_mod_Absyn_Type_modifier_struct*)_tmp2A3)->f1;_LL22: {void*zeroterm=_tmp2B3;
# 1294
Cyc_Absyndump_dump(({const char*_tmp2B4="[]";_tag_fat(_tmp2B4,sizeof(char),3U);}));
if(Cyc_Absyn_type2bool(0,zeroterm))Cyc_Absyndump_dump(({const char*_tmp2B5="@zeroterm";_tag_fat(_tmp2B5,sizeof(char),10U);}));goto _LL20;}case 1U: _LL23: _tmp2B1=((struct Cyc_Absyn_ConstArray_mod_Absyn_Type_modifier_struct*)_tmp2A3)->f1;_tmp2B2=(void*)((struct Cyc_Absyn_ConstArray_mod_Absyn_Type_modifier_struct*)_tmp2A3)->f2;_LL24: {struct Cyc_Absyn_Exp*e=_tmp2B1;void*zeroterm=_tmp2B2;
# 1298
Cyc_Absyndump_dump_char((int)'[');Cyc_Absyndump_dumpexp(e);Cyc_Absyndump_dump_char((int)']');
if(Cyc_Absyn_type2bool(0,zeroterm))Cyc_Absyndump_dump(({const char*_tmp2B6="@zeroterm";_tag_fat(_tmp2B6,sizeof(char),10U);}));goto _LL20;}case 3U: if(((struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct*)((struct Cyc_Absyn_Function_mod_Absyn_Type_modifier_struct*)_tmp2A3)->f1)->tag == 1U){_LL25: _tmp2AA=((struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct*)((struct Cyc_Absyn_Function_mod_Absyn_Type_modifier_struct*)_tmp2A3)->f1)->f1;_tmp2AB=((struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct*)((struct Cyc_Absyn_Function_mod_Absyn_Type_modifier_struct*)_tmp2A3)->f1)->f2;_tmp2AC=((struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct*)((struct Cyc_Absyn_Function_mod_Absyn_Type_modifier_struct*)_tmp2A3)->f1)->f3;_tmp2AD=(void*)((struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct*)((struct Cyc_Absyn_Function_mod_Absyn_Type_modifier_struct*)_tmp2A3)->f1)->f4;_tmp2AE=((struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct*)((struct Cyc_Absyn_Function_mod_Absyn_Type_modifier_struct*)_tmp2A3)->f1)->f5;_tmp2AF=((struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct*)((struct Cyc_Absyn_Function_mod_Absyn_Type_modifier_struct*)_tmp2A3)->f1)->f6;_tmp2B0=((struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct*)((struct Cyc_Absyn_Function_mod_Absyn_Type_modifier_struct*)_tmp2A3)->f1)->f7;_LL26: {struct Cyc_List_List*args=_tmp2AA;int c_varargs=_tmp2AB;struct Cyc_Absyn_VarargInfo*cyc_varargs=_tmp2AC;void*effopt=_tmp2AD;struct Cyc_List_List*rgn_po=_tmp2AE;struct Cyc_Absyn_Exp*req=_tmp2AF;struct Cyc_Absyn_Exp*ens=_tmp2B0;
# 1302
Cyc_Absyndump_dumpfunargs(args,c_varargs,cyc_varargs,effopt,rgn_po,req,ens);goto _LL20;}}else{_LL27: _tmp2A8=((struct Cyc_Absyn_NoTypes_Absyn_Funcparams_struct*)((struct Cyc_Absyn_Function_mod_Absyn_Type_modifier_struct*)_tmp2A3)->f1)->f1;_tmp2A9=((struct Cyc_Absyn_NoTypes_Absyn_Funcparams_struct*)((struct Cyc_Absyn_Function_mod_Absyn_Type_modifier_struct*)_tmp2A3)->f1)->f2;_LL28: {struct Cyc_List_List*sl=_tmp2A8;unsigned loc=_tmp2A9;
# 1304
({void(*_tmp396)(void(*f)(struct _fat_ptr*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep)=({void(*_tmp2B7)(void(*f)(struct _fat_ptr*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep)=(void(*)(void(*f)(struct _fat_ptr*),struct Cyc_List_List*l,struct _fat_ptr start,struct _fat_ptr end,struct _fat_ptr sep))Cyc_Absyndump_group;_tmp2B7;});struct Cyc_List_List*_tmp395=sl;struct _fat_ptr _tmp394=({const char*_tmp2B8="(";_tag_fat(_tmp2B8,sizeof(char),2U);});struct _fat_ptr _tmp393=({const char*_tmp2B9=")";_tag_fat(_tmp2B9,sizeof(char),2U);});_tmp396(Cyc_Absyndump_dump_str,_tmp395,_tmp394,_tmp393,({const char*_tmp2BA=",";_tag_fat(_tmp2BA,sizeof(char),2U);}));});goto _LL20;}}case 4U: _LL29: _tmp2A5=((struct Cyc_Absyn_TypeParams_mod_Absyn_Type_modifier_struct*)_tmp2A3)->f1;_tmp2A6=((struct Cyc_Absyn_TypeParams_mod_Absyn_Type_modifier_struct*)_tmp2A3)->f2;_tmp2A7=((struct Cyc_Absyn_TypeParams_mod_Absyn_Type_modifier_struct*)_tmp2A3)->f3;_LL2A: {struct Cyc_List_List*ts=_tmp2A5;unsigned loc=_tmp2A6;int print_kinds=_tmp2A7;
# 1306
if(print_kinds)Cyc_Absyndump_dumpkindedtvars(ts);else{Cyc_Absyndump_dumptvars(ts);}goto _LL20;}case 5U: _LL2B: _tmp2A4=((struct Cyc_Absyn_Attributes_mod_Absyn_Type_modifier_struct*)_tmp2A3)->f2;_LL2C: {struct Cyc_List_List*atts=_tmp2A4;
Cyc_Absyndump_dumpatts(atts);goto _LL20;}default: _LL2D: _LL2E:
({void*_tmp2BB=0U;({struct _fat_ptr _tmp397=({const char*_tmp2BC="dumptms";_tag_fat(_tmp2BC,sizeof(char),8U);});Cyc_Warn_impos(_tmp397,_tag_fat(_tmp2BB,sizeof(void*),0U));});});}_LL20:;}
# 1310
return;}}_LL0:;}}
# 96
static void Cyc_Absyndump_dumptqtd(struct Cyc_Absyn_Tqual tq,void*t,void(*f)(void*),void*a){
# 1315
int cp=Cyc_Absyndump_is_char_ptr(t);
struct _RegionHandle _tmp2BD=_new_region("temp");struct _RegionHandle*temp=& _tmp2BD;_push_region(temp);
{struct _tuple12 _stmttmp20=Cyc_Absynpp_to_tms(temp,tq,t);struct Cyc_List_List*_tmp2C0;void*_tmp2BF;struct Cyc_Absyn_Tqual _tmp2BE;_LL1: _tmp2BE=_stmttmp20.f1;_tmp2BF=_stmttmp20.f2;_tmp2C0=_stmttmp20.f3;_LL2: {struct Cyc_Absyn_Tqual tq=_tmp2BE;void*t=_tmp2BF;struct Cyc_List_List*tms=_tmp2C0;
Cyc_Absyndump_dumptq(tq);
Cyc_Absyndump_dumpntyp(t);
({int _tmp39A=cp;struct Cyc_List_List*_tmp399=Cyc_List_imp_rev(tms);void(*_tmp398)(void*)=f;Cyc_Absyndump_dumptms(_tmp39A,_tmp399,_tmp398,a);});}}
# 1317
;_pop_region(temp);}
# 96
void Cyc_Absyndump_dumpdecllist2file(struct Cyc_List_List*tdl,struct Cyc___cycFILE*f){
# 1325
*Cyc_Absyndump_dump_file=f;
for(0;tdl != 0;tdl=tdl->tl){
Cyc_Absyndump_dumpdecl((struct Cyc_Absyn_Decl*)tdl->hd);}
({void*_tmp2C1=0U;({struct Cyc___cycFILE*_tmp39C=f;struct _fat_ptr _tmp39B=({const char*_tmp2C2="\n";_tag_fat(_tmp2C2,sizeof(char),2U);});Cyc_fprintf(_tmp39C,_tmp39B,_tag_fat(_tmp2C1,sizeof(void*),0U));});});}
# 96
static void Cyc_Absyndump_dump_decl_interface(struct Cyc_Absyn_Decl*d){
# 1332
void*_stmttmp21=d->r;void*_tmp2C3=_stmttmp21;struct _tuple10*_tmp2C7;struct Cyc_List_List*_tmp2C6;struct Cyc_List_List*_tmp2C5;struct Cyc_List_List*_tmp2C4;struct Cyc_List_List*_tmp2C8;struct Cyc_List_List*_tmp2C9;struct Cyc_List_List*_tmp2CB;struct _fat_ptr*_tmp2CA;struct Cyc_Absyn_Typedefdecl*_tmp2CC;struct Cyc_Absyn_Enumdecl*_tmp2CD;struct Cyc_Absyn_Datatypedecl*_tmp2CE;struct Cyc_Absyn_Aggrdecl*_tmp2CF;struct Cyc_Absyn_Fndecl*_tmp2D0;struct Cyc_Absyn_Vardecl*_tmp2D1;switch(*((int*)_tmp2C3)){case 0U: _LL1: _tmp2D1=((struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct*)_tmp2C3)->f1;_LL2: {struct Cyc_Absyn_Vardecl*vd=_tmp2D1;
# 1334
if((int)vd->sc == (int)Cyc_Absyn_Static)return;{struct Cyc_Absyn_Exp*init=vd->initializer;
# 1336
vd->initializer=0;
if((int)vd->sc == (int)Cyc_Absyn_Public)
Cyc_Absyndump_dump(({const char*_tmp2D2="extern ";_tag_fat(_tmp2D2,sizeof(char),8U);}));
# 1337
Cyc_Absyndump_dumpvardecl(vd,d->loc,1);
# 1340
Cyc_Absyndump_dump(({const char*_tmp2D3="\n";_tag_fat(_tmp2D3,sizeof(char),2U);}));
vd->initializer=init;
goto _LL0;}}case 1U: _LL3: _tmp2D0=((struct Cyc_Absyn_Fn_d_Absyn_Raw_decl_struct*)_tmp2C3)->f1;_LL4: {struct Cyc_Absyn_Fndecl*fd=_tmp2D0;
# 1344
if((int)fd->sc == (int)Cyc_Absyn_Static)return;Cyc_Absyndump_dumpscope(fd->sc);{
# 1346
struct Cyc_Absyn_FnInfo type_info=fd->i;
type_info.attributes=0;{
void*t=(void*)({struct Cyc_Absyn_FnType_Absyn_Type_struct*_tmp2D6=_cycalloc(sizeof(*_tmp2D6));_tmp2D6->tag=5U,_tmp2D6->f1=type_info;_tmp2D6;});
({void(*_tmp39F)(struct Cyc_Absyn_Tqual tq,void*t,void(*f)(struct Cyc_Absyn_Fndecl*),struct Cyc_Absyn_Fndecl*a)=({void(*_tmp2D4)(struct Cyc_Absyn_Tqual tq,void*t,void(*f)(struct Cyc_Absyn_Fndecl*),struct Cyc_Absyn_Fndecl*a)=(void(*)(struct Cyc_Absyn_Tqual tq,void*t,void(*f)(struct Cyc_Absyn_Fndecl*),struct Cyc_Absyn_Fndecl*a))Cyc_Absyndump_dumptqtd;_tmp2D4;});struct Cyc_Absyn_Tqual _tmp39E=Cyc_Absyn_empty_tqual(0U);void*_tmp39D=t;_tmp39F(_tmp39E,_tmp39D,Cyc_Absyndump_dump_atts_qvar,fd);});
Cyc_Absyndump_dump(({const char*_tmp2D5=";\n";_tag_fat(_tmp2D5,sizeof(char),3U);}));
goto _LL0;}}}case 5U: _LL5: _tmp2CF=((struct Cyc_Absyn_Aggr_d_Absyn_Raw_decl_struct*)_tmp2C3)->f1;_LL6: {struct Cyc_Absyn_Aggrdecl*ad=_tmp2CF;
# 1353
if((int)ad->sc == (int)Cyc_Absyn_Static)return;{struct Cyc_Absyn_AggrdeclImpl*impl=ad->impl;
# 1355
if((int)ad->sc == (int)Cyc_Absyn_Abstract)ad->impl=0;Cyc_Absyndump_dump_aggrdecl(ad);
# 1357
ad->impl=impl;
Cyc_Absyndump_dump(({const char*_tmp2D7=";\n";_tag_fat(_tmp2D7,sizeof(char),3U);}));
goto _LL0;}}case 6U: _LL7: _tmp2CE=((struct Cyc_Absyn_Datatype_d_Absyn_Raw_decl_struct*)_tmp2C3)->f1;_LL8: {struct Cyc_Absyn_Datatypedecl*dd=_tmp2CE;
Cyc_Absyndump_dump_datatypedecl(dd);Cyc_Absyndump_dump(({const char*_tmp2D8=";\n";_tag_fat(_tmp2D8,sizeof(char),3U);}));goto _LL0;}case 7U: _LL9: _tmp2CD=((struct Cyc_Absyn_Enum_d_Absyn_Raw_decl_struct*)_tmp2C3)->f1;_LLA: {struct Cyc_Absyn_Enumdecl*ed=_tmp2CD;
Cyc_Absyndump_dump_enumdecl(ed);Cyc_Absyndump_dump(({const char*_tmp2D9=";\n";_tag_fat(_tmp2D9,sizeof(char),3U);}));goto _LL0;}case 8U: _LLB: _tmp2CC=((struct Cyc_Absyn_Typedef_d_Absyn_Raw_decl_struct*)_tmp2C3)->f1;_LLC: {struct Cyc_Absyn_Typedefdecl*td=_tmp2CC;
# 1363
if(td->defn == 0){
Cyc_Absyndump_dumpdecl(d);
Cyc_Absyndump_dump(({const char*_tmp2DA="\n";_tag_fat(_tmp2DA,sizeof(char),2U);}));}
# 1363
goto _LL0;}case 9U: _LLD: _tmp2CA=((struct Cyc_Absyn_Namespace_d_Absyn_Raw_decl_struct*)_tmp2C3)->f1;_tmp2CB=((struct Cyc_Absyn_Namespace_d_Absyn_Raw_decl_struct*)_tmp2C3)->f2;_LLE: {struct _fat_ptr*v=_tmp2CA;struct Cyc_List_List*ds=_tmp2CB;
# 1369
Cyc_Absyndump_dump(({const char*_tmp2DB="namespace ";_tag_fat(_tmp2DB,sizeof(char),11U);}));
Cyc_Absyndump_dump_str(v);
Cyc_Absyndump_dump(({const char*_tmp2DC="{\n";_tag_fat(_tmp2DC,sizeof(char),3U);}));
for(0;ds != 0;ds=ds->tl){
Cyc_Absyndump_dump_decl_interface((struct Cyc_Absyn_Decl*)ds->hd);}
Cyc_Absyndump_dump(({const char*_tmp2DD="}\n";_tag_fat(_tmp2DD,sizeof(char),3U);}));
goto _LL0;}case 10U: _LLF: _tmp2C9=((struct Cyc_Absyn_Using_d_Absyn_Raw_decl_struct*)_tmp2C3)->f2;_LL10: {struct Cyc_List_List*ds=_tmp2C9;
# 1377
for(0;ds != 0;ds=ds->tl){
Cyc_Absyndump_dump_decl_interface((struct Cyc_Absyn_Decl*)ds->hd);}
goto _LL0;}case 11U: _LL11: _tmp2C8=((struct Cyc_Absyn_ExternC_d_Absyn_Raw_decl_struct*)_tmp2C3)->f1;_LL12: {struct Cyc_List_List*ds=_tmp2C8;
# 1381
Cyc_Absyndump_dump(({const char*_tmp2DE="extern \"C\" {";_tag_fat(_tmp2DE,sizeof(char),13U);}));
for(0;ds != 0;ds=ds->tl){
Cyc_Absyndump_dump_decl_interface((struct Cyc_Absyn_Decl*)ds->hd);}
Cyc_Absyndump_dump(({const char*_tmp2DF="}\n";_tag_fat(_tmp2DF,sizeof(char),3U);}));
goto _LL0;}case 12U: _LL13: _tmp2C4=((struct Cyc_Absyn_ExternCinclude_d_Absyn_Raw_decl_struct*)_tmp2C3)->f1;_tmp2C5=((struct Cyc_Absyn_ExternCinclude_d_Absyn_Raw_decl_struct*)_tmp2C3)->f2;_tmp2C6=((struct Cyc_Absyn_ExternCinclude_d_Absyn_Raw_decl_struct*)_tmp2C3)->f3;_tmp2C7=((struct Cyc_Absyn_ExternCinclude_d_Absyn_Raw_decl_struct*)_tmp2C3)->f4;_LL14: {struct Cyc_List_List*ds=_tmp2C4;struct Cyc_List_List*ovrs=_tmp2C5;struct Cyc_List_List*exs=_tmp2C6;struct _tuple10*wc=_tmp2C7;
# 1402 "absyndump.cyc"
goto _LL0;}default: _LL15: _LL16:
({void*_tmp2E0=0U;({int(*_tmp3A1)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp2E2)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmp2E2;});struct _fat_ptr _tmp3A0=({const char*_tmp2E1="bad top-level declaration";_tag_fat(_tmp2E1,sizeof(char),26U);});_tmp3A1(_tmp3A0,_tag_fat(_tmp2E0,sizeof(void*),0U));});});}_LL0:;}
# 96 "absyndump.cyc"
void Cyc_Absyndump_dump_interface(struct Cyc_List_List*ds,struct Cyc___cycFILE*f){
# 1408 "absyndump.cyc"
*Cyc_Absyndump_dump_file=f;
for(0;ds != 0;ds=ds->tl){
Cyc_Absyndump_dump_decl_interface((struct Cyc_Absyn_Decl*)ds->hd);}}
