#include <setjmp.h>
/* This is a C header file to be used by the output of the Cyclone to
   C translator.  The corresponding definitions are in file lib/runtime_*.c */
#ifndef _CYC_INCLUDE_H_
#define _CYC_INCLUDE_H_

/* Need one of these per thread (see runtime_stack.c). The runtime maintains 
   a stack that contains either _handler_cons structs or _RegionHandle structs.
   The tag is 0 for a handler_cons and 1 for a region handle.  */
struct _RuntimeStack {
  int tag; 
  struct _RuntimeStack *next;
  void (*cleanup)(struct _RuntimeStack *frame);
};

#ifndef offsetof
/* should be size_t, but int is fine. */
#define offsetof(t,n) ((int)(&(((t *)0)->n)))
#endif

/* Fat pointers */
struct _fat_ptr {
  unsigned char *curr; 
  unsigned char *base; 
  unsigned char *last_plus_one; 
};  

/* Discriminated Unions */
struct _xtunion_struct { char *tag; };

/* Regions */
struct _RegionPage
#ifdef CYC_REGION_PROFILE
{ unsigned total_bytes;
  unsigned free_bytes;
  /* MWH: wish we didn't have to include the stuff below ... */
  struct _RegionPage *next;
  char data[1];
}
#endif
; // abstract -- defined in runtime_memory.c
struct _RegionHandle {
  struct _RuntimeStack s;
  struct _RegionPage *curr;
  char               *offset;
  char               *last_plus_one;
  struct _DynRegionHandle *sub_regions;
#ifdef CYC_REGION_PROFILE
  const char         *name;
#else
  unsigned used_bytes;
  unsigned wasted_bytes;
#endif
};
struct _DynRegionFrame {
  struct _RuntimeStack s;
  struct _DynRegionHandle *x;
};
// A dynamic region is just a region handle.  The wrapper struct is for type
// abstraction.
struct Cyc_Core_DynamicRegion {
  struct _RegionHandle h;
};

struct _RegionHandle _new_region(const char*);
void* _region_malloc(struct _RegionHandle*, unsigned);
void* _region_calloc(struct _RegionHandle*, unsigned t, unsigned n);
void   _free_region(struct _RegionHandle*);
struct _RegionHandle*_open_dynregion(struct _DynRegionFrame*,struct _DynRegionHandle*);
void   _pop_dynregion();

/* Exceptions */
struct _handler_cons {
  struct _RuntimeStack s;
  jmp_buf handler;
};
void _push_handler(struct _handler_cons *);
void _push_region(struct _RegionHandle *);
void _npop_handler(int);
void _pop_handler();
void _pop_region();

#ifndef _throw
void* _throw_null_fn(const char*,unsigned);
void* _throw_arraybounds_fn(const char*,unsigned);
void* _throw_badalloc_fn(const char*,unsigned);
void* _throw_match_fn(const char*,unsigned);
void* _throw_fn(void*,const char*,unsigned);
void* _rethrow(void*);
#define _throw_null() (_throw_null_fn(__FILE__,__LINE__))
#define _throw_arraybounds() (_throw_arraybounds_fn(__FILE__,__LINE__))
#define _throw_badalloc() (_throw_badalloc_fn(__FILE__,__LINE__))
#define _throw_match() (_throw_match_fn(__FILE__,__LINE__))
#define _throw(e) (_throw_fn((e),__FILE__,__LINE__))
#endif

struct _xtunion_struct* Cyc_Core_get_exn_thrown();
/* Built-in Exceptions */
struct Cyc_Null_Exception_exn_struct { char *tag; };
struct Cyc_Array_bounds_exn_struct { char *tag; };
struct Cyc_Match_Exception_exn_struct { char *tag; };
struct Cyc_Bad_alloc_exn_struct { char *tag; };
extern char Cyc_Null_Exception[];
extern char Cyc_Array_bounds[];
extern char Cyc_Match_Exception[];
extern char Cyc_Bad_alloc[];

/* Built-in Run-time Checks and company */
#ifdef CYC_ANSI_OUTPUT
#define _INLINE  
#else
#define _INLINE inline
#endif

#ifdef NO_CYC_NULL_CHECKS
#define _check_null(ptr) (ptr)
#else
#define _check_null(ptr) \
  ({ void*_cks_null = (void*)(ptr); \
     if (!_cks_null) _throw_null(); \
     _cks_null; })
#endif

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index)\
   (((char*)ptr) + (elt_sz)*(index))
#ifdef NO_CYC_NULL_CHECKS
#define _check_known_subscript_null _check_known_subscript_notnull
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr);\
  int _index = (index);\
  if (!_cks_ptr) _throw_null(); \
  _cks_ptr + (elt_sz)*_index; })
#endif
#define _zero_arr_plus_fn(orig_x,orig_sz,orig_i,f,l) ((orig_x)+(orig_i))
#define _zero_arr_plus_char_fn _zero_arr_plus_fn
#define _zero_arr_plus_short_fn _zero_arr_plus_fn
#define _zero_arr_plus_int_fn _zero_arr_plus_fn
#define _zero_arr_plus_float_fn _zero_arr_plus_fn
#define _zero_arr_plus_double_fn _zero_arr_plus_fn
#define _zero_arr_plus_longdouble_fn _zero_arr_plus_fn
#define _zero_arr_plus_voidstar_fn _zero_arr_plus_fn
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (!_cks_ptr) _throw_null(); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })

/* _zero_arr_plus_*_fn(x,sz,i,filename,lineno) adds i to zero-terminated ptr
   x that has at least sz elements */
char* _zero_arr_plus_char_fn(char*,unsigned,int,const char*,unsigned);
short* _zero_arr_plus_short_fn(short*,unsigned,int,const char*,unsigned);
int* _zero_arr_plus_int_fn(int*,unsigned,int,const char*,unsigned);
float* _zero_arr_plus_float_fn(float*,unsigned,int,const char*,unsigned);
double* _zero_arr_plus_double_fn(double*,unsigned,int,const char*,unsigned);
long double* _zero_arr_plus_longdouble_fn(long double*,unsigned,int,const char*, unsigned);
void** _zero_arr_plus_voidstar_fn(void**,unsigned,int,const char*,unsigned);
#endif

/* _get_zero_arr_size_*(x,sz) returns the number of elements in a
   zero-terminated array that is NULL or has at least sz elements */
int _get_zero_arr_size_char(const char*,unsigned);
int _get_zero_arr_size_short(const short*,unsigned);
int _get_zero_arr_size_int(const int*,unsigned);
int _get_zero_arr_size_float(const float*,unsigned);
int _get_zero_arr_size_double(const double*,unsigned);
int _get_zero_arr_size_longdouble(const long double*,unsigned);
int _get_zero_arr_size_voidstar(const void**,unsigned);

/* _zero_arr_inplace_plus_*_fn(x,i,filename,lineno) sets
   zero-terminated pointer *x to *x + i */
char* _zero_arr_inplace_plus_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_short_fn(short**,int,const char*,unsigned);
int* _zero_arr_inplace_plus_int(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_longdouble_fn(long double**,int,const char*,unsigned);
void** _zero_arr_inplace_plus_voidstar_fn(void***,int,const char*,unsigned);
/* like the previous functions, but does post-addition (as in e++) */
char* _zero_arr_inplace_plus_post_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_post_short_fn(short**x,int,const char*,unsigned);
int* _zero_arr_inplace_plus_post_int_fn(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_post_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_post_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_post_longdouble_fn(long double**,int,const char *,unsigned);
void** _zero_arr_inplace_plus_post_voidstar_fn(void***,int,const char*,unsigned);
#define _zero_arr_plus_char(x,s,i) \
  (_zero_arr_plus_char_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_short(x,s,i) \
  (_zero_arr_plus_short_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_int(x,s,i) \
  (_zero_arr_plus_int_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_float(x,s,i) \
  (_zero_arr_plus_float_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_double(x,s,i) \
  (_zero_arr_plus_double_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_longdouble(x,s,i) \
  (_zero_arr_plus_longdouble_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_voidstar(x,s,i) \
  (_zero_arr_plus_voidstar_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_inplace_plus_char(x,i) \
  _zero_arr_inplace_plus_char_fn((char **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_short(x,i) \
  _zero_arr_inplace_plus_short_fn((short **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_int(x,i) \
  _zero_arr_inplace_plus_int_fn((int **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_float(x,i) \
  _zero_arr_inplace_plus_float_fn((float **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_double(x,i) \
  _zero_arr_inplace_plus_double_fn((double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_longdouble(x,i) \
  _zero_arr_inplace_plus_longdouble_fn((long double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_voidstar(x,i) \
  _zero_arr_inplace_plus_voidstar_fn((void ***)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_char(x,i) \
  _zero_arr_inplace_plus_post_char_fn((char **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_short(x,i) \
  _zero_arr_inplace_plus_post_short_fn((short **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_int(x,i) \
  _zero_arr_inplace_plus_post_int_fn((int **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_float(x,i) \
  _zero_arr_inplace_plus_post_float_fn((float **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_double(x,i) \
  _zero_arr_inplace_plus_post_double_fn((double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_longdouble(x,i) \
  _zero_arr_inplace_plus_post_longdouble_fn((long double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_voidstar(x,i) \
  _zero_arr_inplace_plus_post_voidstar_fn((void***)(x),(i),__FILE__,__LINE__)

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_fat_subscript(arr,elt_sz,index) ((arr).curr + (elt_sz) * (index))
#define _untag_fat_ptr(arr,elt_sz,num_elts) ((arr).curr)
#else
#define _check_fat_subscript(arr,elt_sz,index) ({ \
  struct _fat_ptr _cus_arr = (arr); \
  unsigned char *_cus_ans = _cus_arr.curr + (elt_sz) * (index); \
  /* JGM: not needed! if (!_cus_arr.base) _throw_null();*/ \
  if (_cus_ans < _cus_arr.base || _cus_ans >= _cus_arr.last_plus_one) \
    _throw_arraybounds(); \
  _cus_ans; })
#define _untag_fat_ptr(arr,elt_sz,num_elts) ({ \
  struct _fat_ptr _arr = (arr); \
  unsigned char *_curr = _arr.curr; \
  if ((_curr < _arr.base || _curr + (elt_sz) * (num_elts) > _arr.last_plus_one) &&\
      _curr != (unsigned char *)0) \
    _throw_arraybounds(); \
  _curr; })
#endif

#define _tag_fat(tcurr,elt_sz,num_elts) ({ \
  struct _fat_ptr _ans; \
  unsigned _num_elts = (num_elts);\
  _ans.base = _ans.curr = (void*)(tcurr); \
  /* JGM: if we're tagging NULL, ignore num_elts */ \
  _ans.last_plus_one = _ans.base ? (_ans.base + (elt_sz) * _num_elts) : 0; \
  _ans; })

#define _get_fat_size(arr,elt_sz) \
  ({struct _fat_ptr _arr = (arr); \
    unsigned char *_arr_curr=_arr.curr; \
    unsigned char *_arr_last=_arr.last_plus_one; \
    (_arr_curr < _arr.base || _arr_curr >= _arr_last) ? 0 : \
    ((_arr_last - _arr_curr) / (elt_sz));})

#define _fat_ptr_plus(arr,elt_sz,change) ({ \
  struct _fat_ptr _ans = (arr); \
  int _change = (change);\
  _ans.curr += (elt_sz) * _change;\
  _ans; })
#define _fat_ptr_inplace_plus(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  _arr_ptr->curr += (elt_sz) * (change);\
  *_arr_ptr; })
#define _fat_ptr_inplace_plus_post(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  struct _fat_ptr _ans = *_arr_ptr; \
  _arr_ptr->curr += (elt_sz) * (change);\
  _ans; })

//Not a macro since initialization order matters. Defined in runtime_zeroterm.c.
struct _fat_ptr _fat_ptr_decrease_size(struct _fat_ptr,unsigned sz,unsigned numelts);

/* Allocation */
void* GC_malloc(int);
void* GC_malloc_atomic(int);
void* GC_calloc(unsigned,unsigned);
void* GC_calloc_atomic(unsigned,unsigned);
// bound the allocation size to be < MAX_ALLOC_SIZE. See macros below for usage.
#define MAX_MALLOC_SIZE (1 << 28)
void* _bounded_GC_malloc(int,const char*,int);
void* _bounded_GC_malloc_atomic(int,const char*,int);
void* _bounded_GC_calloc(unsigned,unsigned,const char*,int);
void* _bounded_GC_calloc_atomic(unsigned,unsigned,const char*,int);
/* these macros are overridden below ifdef CYC_REGION_PROFILE */
#ifndef CYC_REGION_PROFILE
#define _cycalloc(n) _bounded_GC_malloc(n,__FILE__,__LINE__)
#define _cycalloc_atomic(n) _bounded_GC_malloc_atomic(n,__FILE__,__LINE__)
#define _cyccalloc(n,s) _bounded_GC_calloc(n,s,__FILE__,__LINE__)
#define _cyccalloc_atomic(n,s) _bounded_GC_calloc_atomic(n,s,__FILE__,__LINE__)
#endif

static _INLINE unsigned int _check_times(unsigned x, unsigned y) {
  unsigned long long whole_ans = 
    ((unsigned long long) x)*((unsigned long long)y);
  unsigned word_ans = (unsigned)whole_ans;
  if(word_ans < whole_ans || word_ans > MAX_MALLOC_SIZE)
    _throw_badalloc();
  return word_ans;
}

#define _CYC_MAX_REGION_CONST 2
#define _CYC_MIN_ALIGNMENT (sizeof(double))

#ifdef CYC_REGION_PROFILE
extern int rgn_total_bytes;
#endif

static _INLINE void *_fast_region_malloc(struct _RegionHandle *r, unsigned orig_s) {  
  if (r > (struct _RegionHandle *)_CYC_MAX_REGION_CONST && r->curr != 0) { 
#ifdef CYC_NOALIGN
    unsigned s =  orig_s;
#else
    unsigned s =  (orig_s + _CYC_MIN_ALIGNMENT - 1) & (~(_CYC_MIN_ALIGNMENT -1)); 
#endif
    char *result; 
    result = r->offset; 
    if (s <= (r->last_plus_one - result)) {
      r->offset = result + s; 
#ifdef CYC_REGION_PROFILE
    r->curr->free_bytes = r->curr->free_bytes - s;
    rgn_total_bytes += s;
#endif
      return result;
    }
  } 
  return _region_malloc(r,orig_s); 
}

#ifdef CYC_REGION_PROFILE
/* see macros below for usage. defined in runtime_memory.c */
void* _profile_GC_malloc(int,const char*,const char*,int);
void* _profile_GC_malloc_atomic(int,const char*,const char*,int);
void* _profile_GC_calloc(unsigned,unsigned,const char*,const char*,int);
void* _profile_GC_calloc_atomic(unsigned,unsigned,const char*,const char*,int);
void* _profile_region_malloc(struct _RegionHandle*,unsigned,const char*,const char*,int);
void* _profile_region_calloc(struct _RegionHandle*,unsigned,unsigned,const char *,const char*,int);
struct _RegionHandle _profile_new_region(const char*,const char*,const char*,int);
void _profile_free_region(struct _RegionHandle*,const char*,const char*,int);
#ifndef RUNTIME_CYC
#define _new_region(n) _profile_new_region(n,__FILE__,__FUNCTION__,__LINE__)
#define _free_region(r) _profile_free_region(r,__FILE__,__FUNCTION__,__LINE__)
#define _region_malloc(rh,n) _profile_region_malloc(rh,n,__FILE__,__FUNCTION__,__LINE__)
#define _region_calloc(rh,n,t) _profile_region_calloc(rh,n,t,__FILE__,__FUNCTION__,__LINE__)
#  endif
#define _cycalloc(n) _profile_GC_malloc(n,__FILE__,__FUNCTION__,__LINE__)
#define _cycalloc_atomic(n) _profile_GC_malloc_atomic(n,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc(n,s) _profile_GC_calloc(n,s,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc_atomic(n,s) _profile_GC_calloc_atomic(n,s,__FILE__,__FUNCTION__,__LINE__)
#endif
#endif
 struct Cyc_Core_Opt{void*v;};
# 95 "core.h"
struct _fat_ptr Cyc_Core_new_string(unsigned);extern char Cyc_Core_Invalid_argument[17U];struct Cyc_Core_Invalid_argument_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Failure[8U];struct Cyc_Core_Failure_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Impossible[11U];struct Cyc_Core_Impossible_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Not_found[10U];struct Cyc_Core_Not_found_exn_struct{char*tag;};extern char Cyc_Core_Unreachable[12U];struct Cyc_Core_Unreachable_exn_struct{char*tag;struct _fat_ptr f1;};
# 168
extern struct _RegionHandle*Cyc_Core_unique_region;struct Cyc_Core_DynamicRegion;struct Cyc_Core_NewDynamicRegion{struct Cyc_Core_DynamicRegion*key;};struct Cyc_Core_ThinRes{void*arr;unsigned nelts;};struct Cyc___cycFILE;
# 53 "cycboot.h"
extern struct Cyc___cycFILE*Cyc_stderr;struct Cyc_String_pa_PrintArg_struct{int tag;struct _fat_ptr f1;};struct Cyc_Int_pa_PrintArg_struct{int tag;unsigned long f1;};struct Cyc_Double_pa_PrintArg_struct{int tag;double f1;};struct Cyc_LongDouble_pa_PrintArg_struct{int tag;long double f1;};struct Cyc_ShortPtr_pa_PrintArg_struct{int tag;short*f1;};struct Cyc_IntPtr_pa_PrintArg_struct{int tag;unsigned long*f1;};
# 73
struct _fat_ptr Cyc_aprintf(struct _fat_ptr,struct _fat_ptr);
# 100
int Cyc_fprintf(struct Cyc___cycFILE*,struct _fat_ptr,struct _fat_ptr);struct Cyc_ShortPtr_sa_ScanfArg_struct{int tag;short*f1;};struct Cyc_UShortPtr_sa_ScanfArg_struct{int tag;unsigned short*f1;};struct Cyc_IntPtr_sa_ScanfArg_struct{int tag;int*f1;};struct Cyc_UIntPtr_sa_ScanfArg_struct{int tag;unsigned*f1;};struct Cyc_StringPtr_sa_ScanfArg_struct{int tag;struct _fat_ptr f1;};struct Cyc_DoublePtr_sa_ScanfArg_struct{int tag;double*f1;};struct Cyc_FloatPtr_sa_ScanfArg_struct{int tag;float*f1;};struct Cyc_CharPtr_sa_ScanfArg_struct{int tag;struct _fat_ptr f1;};extern char Cyc_FileCloseError[15U];struct Cyc_FileCloseError_exn_struct{char*tag;};extern char Cyc_FileOpenError[14U];struct Cyc_FileOpenError_exn_struct{char*tag;struct _fat_ptr f1;};struct Cyc_Buffer_t;struct Cyc_List_List{void*hd;struct Cyc_List_List*tl;};
# 76 "list.h"
struct Cyc_List_List*Cyc_List_map(void*(*f)(void*),struct Cyc_List_List*x);
# 83
struct Cyc_List_List*Cyc_List_map_c(void*(*f)(void*,void*),void*env,struct Cyc_List_List*x);extern char Cyc_List_List_mismatch[14U];struct Cyc_List_List_mismatch_exn_struct{char*tag;};
# 133
void Cyc_List_iter(void(*f)(void*),struct Cyc_List_List*x);
# 178
struct Cyc_List_List*Cyc_List_imp_rev(struct Cyc_List_List*x);
# 184
struct Cyc_List_List*Cyc_List_append(struct Cyc_List_List*x,struct Cyc_List_List*y);
# 195
struct Cyc_List_List*Cyc_List_imp_append(struct Cyc_List_List*x,struct Cyc_List_List*y);extern char Cyc_List_Nth[4U];struct Cyc_List_Nth_exn_struct{char*tag;};
# 258
int Cyc_List_exists(int(*pred)(void*),struct Cyc_List_List*x);
# 322
int Cyc_List_mem(int(*compare)(void*,void*),struct Cyc_List_List*l,void*x);
# 383
int Cyc_List_list_cmp(int(*cmp)(void*,void*),struct Cyc_List_List*l1,struct Cyc_List_List*l2);
# 387
int Cyc_List_list_prefix(int(*cmp)(void*,void*),struct Cyc_List_List*l1,struct Cyc_List_List*l2);
# 38 "string.h"
unsigned long Cyc_strlen(struct _fat_ptr s);
# 50 "string.h"
int Cyc_strptrcmp(struct _fat_ptr*s1,struct _fat_ptr*s2);
# 62
struct _fat_ptr Cyc_strconcat(struct _fat_ptr,struct _fat_ptr);
# 66
struct _fat_ptr Cyc_str_sepstr(struct Cyc_List_List*,struct _fat_ptr);struct Cyc_Position_Error;struct Cyc_Relations_Reln;struct _union_Nmspace_Rel_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_Abs_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_C_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_Loc_n{int tag;int val;};union Cyc_Absyn_Nmspace{struct _union_Nmspace_Rel_n Rel_n;struct _union_Nmspace_Abs_n Abs_n;struct _union_Nmspace_C_n C_n;struct _union_Nmspace_Loc_n Loc_n;};struct _tuple0{union Cyc_Absyn_Nmspace f1;struct _fat_ptr*f2;};
# 161 "absyn.h"
enum Cyc_Absyn_Scope{Cyc_Absyn_Static =0U,Cyc_Absyn_Abstract =1U,Cyc_Absyn_Public =2U,Cyc_Absyn_Extern =3U,Cyc_Absyn_ExternC =4U,Cyc_Absyn_Register =5U};struct Cyc_Absyn_Tqual{int print_const: 1;int q_volatile: 1;int q_restrict: 1;int real_const: 1;unsigned loc;};
# 182
enum Cyc_Absyn_Size_of{Cyc_Absyn_Char_sz =0U,Cyc_Absyn_Short_sz =1U,Cyc_Absyn_Int_sz =2U,Cyc_Absyn_Long_sz =3U,Cyc_Absyn_LongLong_sz =4U};
enum Cyc_Absyn_Sign{Cyc_Absyn_Signed =0U,Cyc_Absyn_Unsigned =1U,Cyc_Absyn_None =2U};
enum Cyc_Absyn_AggrKind{Cyc_Absyn_StructA =0U,Cyc_Absyn_UnionA =1U};
# 187
enum Cyc_Absyn_AliasQual{Cyc_Absyn_Aliasable =0U,Cyc_Absyn_Unique =1U,Cyc_Absyn_Top =2U};
# 192
enum Cyc_Absyn_KindQual{Cyc_Absyn_AnyKind =0U,Cyc_Absyn_MemKind =1U,Cyc_Absyn_BoxKind =2U,Cyc_Absyn_RgnKind =3U,Cyc_Absyn_EffKind =4U,Cyc_Absyn_IntKind =5U,Cyc_Absyn_BoolKind =6U,Cyc_Absyn_PtrBndKind =7U};struct Cyc_Absyn_Kind{enum Cyc_Absyn_KindQual kind;enum Cyc_Absyn_AliasQual aliasqual;};struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct{int tag;struct Cyc_Absyn_Kind*f1;};struct Cyc_Absyn_Unknown_kb_Absyn_KindBound_struct{int tag;struct Cyc_Core_Opt*f1;};struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct{int tag;struct Cyc_Core_Opt*f1;struct Cyc_Absyn_Kind*f2;};struct Cyc_Absyn_Tvar{struct _fat_ptr*name;int identity;void*kind;};struct Cyc_Absyn_PtrLoc{unsigned ptr_loc;unsigned rgn_loc;unsigned zt_loc;};struct Cyc_Absyn_PtrAtts{void*rgn;void*nullable;void*bounds;void*zero_term;struct Cyc_Absyn_PtrLoc*ptrloc;};struct Cyc_Absyn_PtrInfo{void*elt_type;struct Cyc_Absyn_Tqual elt_tq;struct Cyc_Absyn_PtrAtts ptr_atts;};struct Cyc_Absyn_VarargInfo{struct _fat_ptr*name;struct Cyc_Absyn_Tqual tq;void*type;int inject;};struct Cyc_Absyn_Star_Absyn_Cap_struct{int tag;};struct Cyc_Absyn_Bot_Absyn_Cap_struct{int tag;};struct Cyc_Absyn_Nat_Absyn_Cap_struct{int tag;struct Cyc_Absyn_Exp*f1;int f2;};struct Cyc_Absyn_RgnEffect{void*name;void*rgn;void*lock;void*unadopt;void*parent;};struct Cyc_Absyn_FnInfo{struct Cyc_List_List*tvars;void*effect;struct Cyc_Absyn_Tqual ret_tqual;void*ret_type;struct Cyc_List_List*args;int c_varargs;struct Cyc_Absyn_VarargInfo*cyc_varargs;struct Cyc_List_List*rgn_po;struct Cyc_List_List*attributes;struct Cyc_Absyn_Exp*requires_clause;struct Cyc_List_List*requires_relns;struct Cyc_Absyn_Exp*ensures_clause;struct Cyc_List_List*ensures_relns;struct Cyc_List_List*ieffect;struct Cyc_List_List*oeffect;};struct Cyc_Absyn_UnknownDatatypeInfo{struct _tuple0*name;int is_extensible;};struct _union_DatatypeInfo_UnknownDatatype{int tag;struct Cyc_Absyn_UnknownDatatypeInfo val;};struct _union_DatatypeInfo_KnownDatatype{int tag;struct Cyc_Absyn_Datatypedecl**val;};union Cyc_Absyn_DatatypeInfo{struct _union_DatatypeInfo_UnknownDatatype UnknownDatatype;struct _union_DatatypeInfo_KnownDatatype KnownDatatype;};struct Cyc_Absyn_UnknownDatatypeFieldInfo{struct _tuple0*datatype_name;struct _tuple0*field_name;int is_extensible;};struct _union_DatatypeFieldInfo_UnknownDatatypefield{int tag;struct Cyc_Absyn_UnknownDatatypeFieldInfo val;};struct _tuple1{struct Cyc_Absyn_Datatypedecl*f1;struct Cyc_Absyn_Datatypefield*f2;};struct _union_DatatypeFieldInfo_KnownDatatypefield{int tag;struct _tuple1 val;};union Cyc_Absyn_DatatypeFieldInfo{struct _union_DatatypeFieldInfo_UnknownDatatypefield UnknownDatatypefield;struct _union_DatatypeFieldInfo_KnownDatatypefield KnownDatatypefield;};struct _tuple2{enum Cyc_Absyn_AggrKind f1;struct _tuple0*f2;struct Cyc_Core_Opt*f3;};struct _union_AggrInfo_UnknownAggr{int tag;struct _tuple2 val;};struct _union_AggrInfo_KnownAggr{int tag;struct Cyc_Absyn_Aggrdecl**val;};union Cyc_Absyn_AggrInfo{struct _union_AggrInfo_UnknownAggr UnknownAggr;struct _union_AggrInfo_KnownAggr KnownAggr;};struct Cyc_Absyn_ArrayInfo{void*elt_type;struct Cyc_Absyn_Tqual tq;struct Cyc_Absyn_Exp*num_elts;void*zero_term;unsigned zt_loc;};struct Cyc_Absyn_Aggr_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Aggrdecl*f1;};struct Cyc_Absyn_Enum_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;};struct Cyc_Absyn_Datatype_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;};struct Cyc_Absyn_TypeDecl{void*r;unsigned loc;};struct Cyc_Absyn_VoidCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_IntCon_Absyn_TyCon_struct{int tag;enum Cyc_Absyn_Sign f1;enum Cyc_Absyn_Size_of f2;};struct Cyc_Absyn_FloatCon_Absyn_TyCon_struct{int tag;int f1;};struct Cyc_Absyn_RgnHandleCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_TagCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_HeapCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_UniqueCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_RefCntCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_AccessCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_JoinCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_RgnsCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_TrueCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_FalseCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_ThinCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_FatCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_EnumCon_Absyn_TyCon_struct{int tag;struct _tuple0*f1;struct Cyc_Absyn_Enumdecl*f2;};struct Cyc_Absyn_AnonEnumCon_Absyn_TyCon_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_BuiltinCon_Absyn_TyCon_struct{int tag;struct _fat_ptr f1;struct Cyc_Absyn_Kind*f2;};struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_DatatypeInfo f1;};struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_DatatypeFieldInfo f1;};struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_AggrInfo f1;};struct Cyc_Absyn_AppType_Absyn_Type_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Evar_Absyn_Type_struct{int tag;struct Cyc_Core_Opt*f1;void*f2;int f3;struct Cyc_Core_Opt*f4;};struct Cyc_Absyn_VarType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Tvar*f1;};struct Cyc_Absyn_PointerType_Absyn_Type_struct{int tag;struct Cyc_Absyn_PtrInfo f1;};struct Cyc_Absyn_ArrayType_Absyn_Type_struct{int tag;struct Cyc_Absyn_ArrayInfo f1;};struct Cyc_Absyn_FnType_Absyn_Type_struct{int tag;struct Cyc_Absyn_FnInfo f1;};struct Cyc_Absyn_TupleType_Absyn_Type_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct{int tag;enum Cyc_Absyn_AggrKind f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_TypedefType_Absyn_Type_struct{int tag;struct _tuple0*f1;struct Cyc_List_List*f2;struct Cyc_Absyn_Typedefdecl*f3;void*f4;};struct Cyc_Absyn_ValueofType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct{int tag;struct Cyc_Absyn_TypeDecl*f1;void**f2;};struct Cyc_Absyn_TypeofType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_NoTypes_Absyn_Funcparams_struct{int tag;struct Cyc_List_List*f1;unsigned f2;};struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct{int tag;struct Cyc_List_List*f1;int f2;struct Cyc_Absyn_VarargInfo*f3;void*f4;struct Cyc_List_List*f5;struct Cyc_Absyn_Exp*f6;struct Cyc_Absyn_Exp*f7;struct Cyc_List_List*f8;struct Cyc_List_List*f9;};
# 437 "absyn.h"
enum Cyc_Absyn_Format_Type{Cyc_Absyn_Printf_ft =0U,Cyc_Absyn_Scanf_ft =1U};struct Cyc_Absyn_Regparm_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Stdcall_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Cdecl_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Fastcall_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Noreturn_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Const_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Aligned_att_Absyn_Attribute_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Packed_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Section_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Nocommon_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Shared_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Unused_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Weak_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Dllimport_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Dllexport_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_instrument_function_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Constructor_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Destructor_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_check_memory_usage_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Format_att_Absyn_Attribute_struct{int tag;enum Cyc_Absyn_Format_Type f1;int f2;int f3;};struct Cyc_Absyn_Initializes_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Noliveunique_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Consume_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Pure_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Mode_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Alias_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Always_inline_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_throw_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Non_null_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Deprecated_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Carray_mod_Absyn_Type_modifier_struct{int tag;void*f1;unsigned f2;};struct Cyc_Absyn_ConstArray_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_Absyn_Exp*f1;void*f2;unsigned f3;};struct Cyc_Absyn_Pointer_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_Absyn_PtrAtts f1;struct Cyc_Absyn_Tqual f2;};struct Cyc_Absyn_Function_mod_Absyn_Type_modifier_struct{int tag;void*f1;};struct Cyc_Absyn_TypeParams_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_List_List*f1;unsigned f2;int f3;};struct Cyc_Absyn_Attributes_mod_Absyn_Type_modifier_struct{int tag;unsigned f1;struct Cyc_List_List*f2;};struct _union_Cnst_Null_c{int tag;int val;};struct _tuple3{enum Cyc_Absyn_Sign f1;char f2;};struct _union_Cnst_Char_c{int tag;struct _tuple3 val;};struct _union_Cnst_Wchar_c{int tag;struct _fat_ptr val;};struct _tuple4{enum Cyc_Absyn_Sign f1;short f2;};struct _union_Cnst_Short_c{int tag;struct _tuple4 val;};struct _tuple5{enum Cyc_Absyn_Sign f1;int f2;};struct _union_Cnst_Int_c{int tag;struct _tuple5 val;};struct _tuple6{enum Cyc_Absyn_Sign f1;long long f2;};struct _union_Cnst_LongLong_c{int tag;struct _tuple6 val;};struct _tuple7{struct _fat_ptr f1;int f2;};struct _union_Cnst_Float_c{int tag;struct _tuple7 val;};struct _union_Cnst_String_c{int tag;struct _fat_ptr val;};struct _union_Cnst_Wstring_c{int tag;struct _fat_ptr val;};union Cyc_Absyn_Cnst{struct _union_Cnst_Null_c Null_c;struct _union_Cnst_Char_c Char_c;struct _union_Cnst_Wchar_c Wchar_c;struct _union_Cnst_Short_c Short_c;struct _union_Cnst_Int_c Int_c;struct _union_Cnst_LongLong_c LongLong_c;struct _union_Cnst_Float_c Float_c;struct _union_Cnst_String_c String_c;struct _union_Cnst_Wstring_c Wstring_c;};
# 532
enum Cyc_Absyn_Primop{Cyc_Absyn_Plus =0U,Cyc_Absyn_Times =1U,Cyc_Absyn_Minus =2U,Cyc_Absyn_Div =3U,Cyc_Absyn_Mod =4U,Cyc_Absyn_Eq =5U,Cyc_Absyn_Neq =6U,Cyc_Absyn_Gt =7U,Cyc_Absyn_Lt =8U,Cyc_Absyn_Gte =9U,Cyc_Absyn_Lte =10U,Cyc_Absyn_Not =11U,Cyc_Absyn_Bitnot =12U,Cyc_Absyn_Bitand =13U,Cyc_Absyn_Bitor =14U,Cyc_Absyn_Bitxor =15U,Cyc_Absyn_Bitlshift =16U,Cyc_Absyn_Bitlrshift =17U,Cyc_Absyn_Numelts =18U};
# 539
enum Cyc_Absyn_Incrementor{Cyc_Absyn_PreInc =0U,Cyc_Absyn_PostInc =1U,Cyc_Absyn_PreDec =2U,Cyc_Absyn_PostDec =3U};struct Cyc_Absyn_VarargCallInfo{int num_varargs;struct Cyc_List_List*injectors;struct Cyc_Absyn_VarargInfo*vai;};struct Cyc_Absyn_StructField_Absyn_OffsetofField_struct{int tag;struct _fat_ptr*f1;};struct Cyc_Absyn_TupleIndex_Absyn_OffsetofField_struct{int tag;unsigned f1;};
# 557
enum Cyc_Absyn_Coercion{Cyc_Absyn_Unknown_coercion =0U,Cyc_Absyn_No_coercion =1U,Cyc_Absyn_Null_to_NonNull =2U,Cyc_Absyn_Other_coercion =3U};struct Cyc_Absyn_MallocInfo{int is_calloc;struct Cyc_Absyn_Exp*rgn;void**elt_type;struct Cyc_Absyn_Exp*num_elts;int fat_result;int inline_call;};struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct{int tag;union Cyc_Absyn_Cnst f1;};struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Pragma_e_Absyn_Raw_exp_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct{int tag;enum Cyc_Absyn_Primop f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_AssignOp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Core_Opt*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_Increment_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;enum Cyc_Absyn_Incrementor f2;};struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;struct Cyc_Absyn_VarargCallInfo*f3;int f4;};struct Cyc_Absyn_Throw_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;int f2;};struct Cyc_Absyn_NoInstantiate_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Instantiate_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_Absyn_Exp*f2;int f3;enum Cyc_Absyn_Coercion f4;};struct Cyc_Absyn_Address_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_New_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Sizeoftype_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Sizeofexp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Offsetof_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Deref_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;int f3;int f4;};struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;int f3;int f4;};struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Tuple_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;};struct _tuple8{struct _fat_ptr*f1;struct Cyc_Absyn_Tqual f2;void*f3;};struct Cyc_Absyn_CompoundLit_e_Absyn_Raw_exp_struct{int tag;struct _tuple8*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Array_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Exp*f2;struct Cyc_Absyn_Exp*f3;int f4;};struct Cyc_Absyn_ComprehensionNoinit_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;void*f2;int f3;};struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct{int tag;struct _tuple0*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;struct Cyc_Absyn_Aggrdecl*f4;};struct Cyc_Absyn_AnonStruct_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Datatype_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;struct Cyc_Absyn_Datatypedecl*f2;struct Cyc_Absyn_Datatypefield*f3;};struct Cyc_Absyn_Enum_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_AnonEnum_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_Spawn_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_MallocInfo f1;};struct Cyc_Absyn_Swap_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_UnresolvedMem_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Core_Opt*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_StmtExp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Stmt*f1;};struct Cyc_Absyn_Tagcheck_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;};struct Cyc_Absyn_Valueof_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Asm_e_Absyn_Raw_exp_struct{int tag;int f1;struct _fat_ptr f2;struct Cyc_List_List*f3;struct Cyc_List_List*f4;struct Cyc_List_List*f5;};struct Cyc_Absyn_Extension_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Exp{void*topt;void*r;unsigned loc;void*annot;};struct Cyc_Absyn_Skip_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Exp_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Seq_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Return_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_IfThenElse_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Stmt*f2;struct Cyc_Absyn_Stmt*f3;};struct _tuple9{struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_While_s_Absyn_Raw_stmt_struct{int tag;struct _tuple9 f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Break_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Continue_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Goto_s_Absyn_Raw_stmt_struct{int tag;struct _fat_ptr*f1;};struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _tuple9 f2;struct _tuple9 f3;struct Cyc_Absyn_Stmt*f4;};struct Cyc_Absyn_Switch_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;void*f3;};struct Cyc_Absyn_Fallthru_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_List_List*f1;struct Cyc_Absyn_Switch_clause**f2;};struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Decl*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Label_s_Absyn_Raw_stmt_struct{int tag;struct _fat_ptr*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Do_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct _tuple9 f2;};struct Cyc_Absyn_TryCatch_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct Cyc_List_List*f2;void*f3;};struct Cyc_Absyn_Stmt{void*r;unsigned loc;void*annot;};struct Cyc_Absyn_Wild_p_Absyn_Raw_pat_struct{int tag;};struct Cyc_Absyn_Var_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Pat*f2;};struct Cyc_Absyn_AliasVar_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;};struct Cyc_Absyn_Reference_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Pat*f2;};struct Cyc_Absyn_TagInt_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;};struct Cyc_Absyn_Tuple_p_Absyn_Raw_pat_struct{int tag;struct Cyc_List_List*f1;int f2;};struct Cyc_Absyn_Pointer_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Pat*f1;};struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct{int tag;union Cyc_Absyn_AggrInfo*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;int f4;};struct Cyc_Absyn_Datatype_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;struct Cyc_Absyn_Datatypefield*f2;struct Cyc_List_List*f3;int f4;};struct Cyc_Absyn_Null_p_Absyn_Raw_pat_struct{int tag;};struct Cyc_Absyn_Int_p_Absyn_Raw_pat_struct{int tag;enum Cyc_Absyn_Sign f1;int f2;};struct Cyc_Absyn_Char_p_Absyn_Raw_pat_struct{int tag;char f1;};struct Cyc_Absyn_Float_p_Absyn_Raw_pat_struct{int tag;struct _fat_ptr f1;int f2;};struct Cyc_Absyn_Enum_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_AnonEnum_p_Absyn_Raw_pat_struct{int tag;void*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_UnknownId_p_Absyn_Raw_pat_struct{int tag;struct _tuple0*f1;};struct Cyc_Absyn_UnknownCall_p_Absyn_Raw_pat_struct{int tag;struct _tuple0*f1;struct Cyc_List_List*f2;int f3;};struct Cyc_Absyn_Exp_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Exp*f1;};
# 725 "absyn.h"
extern struct Cyc_Absyn_Wild_p_Absyn_Raw_pat_struct Cyc_Absyn_Wild_p_val;struct Cyc_Absyn_Pat{void*r;void*topt;unsigned loc;};struct Cyc_Absyn_Switch_clause{struct Cyc_Absyn_Pat*pattern;struct Cyc_Core_Opt*pat_vars;struct Cyc_Absyn_Exp*where_clause;struct Cyc_Absyn_Stmt*body;unsigned loc;};struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct{int tag;struct _tuple0*f1;};struct Cyc_Absyn_Global_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Funname_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Fndecl*f1;};struct Cyc_Absyn_Param_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Local_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Pat_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Vardecl{enum Cyc_Absyn_Scope sc;struct _tuple0*name;unsigned varloc;struct Cyc_Absyn_Tqual tq;void*type;struct Cyc_Absyn_Exp*initializer;void*rgn;struct Cyc_List_List*attributes;int escapes;};struct Cyc_Absyn_Fndecl{enum Cyc_Absyn_Scope sc;int is_inline;struct _tuple0*name;struct Cyc_Absyn_Stmt*body;struct Cyc_Absyn_FnInfo i;void*cached_type;struct Cyc_Core_Opt*param_vardecls;struct Cyc_Absyn_Vardecl*fn_vardecl;};struct Cyc_Absyn_Aggrfield{struct _fat_ptr*name;struct Cyc_Absyn_Tqual tq;void*type;struct Cyc_Absyn_Exp*width;struct Cyc_List_List*attributes;struct Cyc_Absyn_Exp*requires_clause;};struct Cyc_Absyn_AggrdeclImpl{struct Cyc_List_List*exist_vars;struct Cyc_List_List*rgn_po;struct Cyc_List_List*fields;int tagged;};struct Cyc_Absyn_Aggrdecl{enum Cyc_Absyn_AggrKind kind;enum Cyc_Absyn_Scope sc;struct _tuple0*name;struct Cyc_List_List*tvs;struct Cyc_Absyn_AggrdeclImpl*impl;struct Cyc_List_List*attributes;int expected_mem_kind;};struct Cyc_Absyn_Datatypefield{struct _tuple0*name;struct Cyc_List_List*typs;unsigned loc;enum Cyc_Absyn_Scope sc;};struct Cyc_Absyn_Datatypedecl{enum Cyc_Absyn_Scope sc;struct _tuple0*name;struct Cyc_List_List*tvs;struct Cyc_Core_Opt*fields;int is_extensible;};struct Cyc_Absyn_Enumfield{struct _tuple0*name;struct Cyc_Absyn_Exp*tag;unsigned loc;};struct Cyc_Absyn_Enumdecl{enum Cyc_Absyn_Scope sc;struct _tuple0*name;struct Cyc_Core_Opt*fields;};struct Cyc_Absyn_Typedefdecl{struct _tuple0*name;struct Cyc_Absyn_Tqual tq;struct Cyc_List_List*tvs;struct Cyc_Core_Opt*kind;void*defn;struct Cyc_List_List*atts;int extern_c;};struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Fn_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Fndecl*f1;};struct Cyc_Absyn_Let_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Pat*f1;struct Cyc_Core_Opt*f2;struct Cyc_Absyn_Exp*f3;void*f4;};struct Cyc_Absyn_Letv_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_Region_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_Aggr_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Aggrdecl*f1;};struct Cyc_Absyn_Datatype_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;};struct Cyc_Absyn_Enum_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;};struct Cyc_Absyn_Typedef_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Typedefdecl*f1;};struct Cyc_Absyn_Namespace_d_Absyn_Raw_decl_struct{int tag;struct _fat_ptr*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Using_d_Absyn_Raw_decl_struct{int tag;struct _tuple0*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_ExternC_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;};struct _tuple10{unsigned f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_ExternCinclude_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;struct _tuple10*f4;};struct Cyc_Absyn_Porton_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Portoff_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Tempeston_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Tempestoff_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Decl{void*r;unsigned loc;};struct Cyc_Absyn_ArrayElement_Absyn_Designator_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_FieldName_Absyn_Designator_struct{int tag;struct _fat_ptr*f1;};extern char Cyc_Absyn_EmptyAnnot[11U];struct Cyc_Absyn_EmptyAnnot_Absyn_AbsynAnnot_struct{char*tag;};
# 908
int Cyc_Absyn_qvar_cmp(struct _tuple0*,struct _tuple0*);
# 917
struct Cyc_Absyn_Tqual Cyc_Absyn_empty_tqual(unsigned);
# 922
void*Cyc_Absyn_compress_kb(void*);
# 928
int Cyc_Absyn_type2bool(int def,void*);
# 933
void*Cyc_Absyn_new_evar(struct Cyc_Core_Opt*k,struct Cyc_Core_Opt*tenv);
# 1037
struct Cyc_Absyn_Exp*Cyc_Absyn_times_exp(struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*,unsigned);
# 1060
struct Cyc_Absyn_Exp*Cyc_Absyn_sizeoftype_exp(void*t,unsigned);
# 1164
struct _fat_ptr Cyc_Absyn_attribute2string(void*);struct _tuple11{enum Cyc_Absyn_AggrKind f1;struct _tuple0*f2;};
# 1174
struct _tuple11 Cyc_Absyn_aggr_kinded_name(union Cyc_Absyn_AggrInfo);
# 1182
struct _tuple0*Cyc_Absyn_binding2qvar(void*);struct _tuple12{unsigned f1;int f2;};
# 28 "evexp.h"
struct _tuple12 Cyc_Evexp_eval_const_uint_exp(struct Cyc_Absyn_Exp*e);struct _union_RelnOp_RConst{int tag;unsigned val;};struct _union_RelnOp_RVar{int tag;struct Cyc_Absyn_Vardecl*val;};struct _union_RelnOp_RNumelts{int tag;struct Cyc_Absyn_Vardecl*val;};struct _union_RelnOp_RType{int tag;void*val;};struct _union_RelnOp_RParam{int tag;unsigned val;};struct _union_RelnOp_RParamNumelts{int tag;unsigned val;};struct _union_RelnOp_RReturn{int tag;unsigned val;};union Cyc_Relations_RelnOp{struct _union_RelnOp_RConst RConst;struct _union_RelnOp_RVar RVar;struct _union_RelnOp_RNumelts RNumelts;struct _union_RelnOp_RType RType;struct _union_RelnOp_RParam RParam;struct _union_RelnOp_RParamNumelts RParamNumelts;struct _union_RelnOp_RReturn RReturn;};
# 50 "relations-ap.h"
enum Cyc_Relations_Relation{Cyc_Relations_Req =0U,Cyc_Relations_Rneq =1U,Cyc_Relations_Rlte =2U,Cyc_Relations_Rlt =3U};struct Cyc_Relations_Reln{union Cyc_Relations_RelnOp rop1;enum Cyc_Relations_Relation relation;union Cyc_Relations_RelnOp rop2;};struct Cyc_Iter_Iter{void*env;int(*next)(void*env,void*dest);};struct Cyc_Dict_T;struct Cyc_Dict_Dict{int(*rel)(void*,void*);struct _RegionHandle*r;const struct Cyc_Dict_T*t;};extern char Cyc_Dict_Present[8U];struct Cyc_Dict_Present_exn_struct{char*tag;};extern char Cyc_Dict_Absent[7U];struct Cyc_Dict_Absent_exn_struct{char*tag;};struct Cyc_RgnOrder_RgnPO;extern char Cyc_Tcenv_Env_error[10U];struct Cyc_Tcenv_Env_error_exn_struct{char*tag;};struct Cyc_Tcenv_Genv{struct Cyc_Dict_Dict aggrdecls;struct Cyc_Dict_Dict datatypedecls;struct Cyc_Dict_Dict enumdecls;struct Cyc_Dict_Dict typedefs;struct Cyc_Dict_Dict ordinaries;};struct Cyc_Tcenv_Fenv;struct Cyc_Tcenv_Tenv{struct Cyc_List_List*ns;struct Cyc_Tcenv_Genv*ae;struct Cyc_Tcenv_Fenv*le;int allow_valueof: 1;int in_extern_c_include: 1;int in_tempest: 1;int tempest_generalize: 1;};
# 88 "tcenv.h"
enum Cyc_Tcenv_NewStatus{Cyc_Tcenv_NoneNew =0U,Cyc_Tcenv_InNew =1U,Cyc_Tcenv_InNewAggr =2U};
# 107 "tcutil.h"
void*Cyc_Tcutil_compress(void*);
# 263
int Cyc_Tcutil_is_temp_tvar(struct Cyc_Absyn_Tvar*);
# 40 "warn.h"
void*Cyc_Warn_impos(struct _fat_ptr fmt,struct _fat_ptr ap);struct Cyc_Warn_String_Warn_Warg_struct{int tag;struct _fat_ptr f1;};struct Cyc_Warn_Exp_Warn_Warg_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Warn_Typ_Warn_Warg_struct{int tag;void*f1;};struct Cyc_Warn_Qvar_Warn_Warg_struct{int tag;struct _tuple0*f1;};
# 39 "pp.h"
extern int Cyc_PP_tex_output;struct Cyc_PP_Ppstate;struct Cyc_PP_Out;struct Cyc_PP_Doc;
# 50
void Cyc_PP_file_of_doc(struct Cyc_PP_Doc*d,int w,struct Cyc___cycFILE*f);
# 53
struct _fat_ptr Cyc_PP_string_of_doc(struct Cyc_PP_Doc*d,int w);
# 67 "pp.h"
struct Cyc_PP_Doc*Cyc_PP_nil_doc();
# 69
struct Cyc_PP_Doc*Cyc_PP_blank_doc();
# 72
struct Cyc_PP_Doc*Cyc_PP_line_doc();
# 78
struct Cyc_PP_Doc*Cyc_PP_text(struct _fat_ptr s);
# 80
struct Cyc_PP_Doc*Cyc_PP_textptr(struct _fat_ptr*p);
# 83
struct Cyc_PP_Doc*Cyc_PP_text_width(struct _fat_ptr s,int w);
# 91
struct Cyc_PP_Doc*Cyc_PP_nest(int k,struct Cyc_PP_Doc*d);
# 94
struct Cyc_PP_Doc*Cyc_PP_cat(struct _fat_ptr);
# 108
struct Cyc_PP_Doc*Cyc_PP_seq(struct _fat_ptr sep,struct Cyc_List_List*l);
# 112
struct Cyc_PP_Doc*Cyc_PP_ppseq(struct Cyc_PP_Doc*(*pp)(void*),struct _fat_ptr sep,struct Cyc_List_List*l);
# 117
struct Cyc_PP_Doc*Cyc_PP_seql(struct _fat_ptr sep,struct Cyc_List_List*l0);
# 120
struct Cyc_PP_Doc*Cyc_PP_ppseql(struct Cyc_PP_Doc*(*pp)(void*),struct _fat_ptr sep,struct Cyc_List_List*l);
# 123
struct Cyc_PP_Doc*Cyc_PP_group(struct _fat_ptr start,struct _fat_ptr stop,struct _fat_ptr sep,struct Cyc_List_List*l);
# 129
struct Cyc_PP_Doc*Cyc_PP_egroup(struct _fat_ptr start,struct _fat_ptr stop,struct _fat_ptr sep,struct Cyc_List_List*l);struct Cyc_Absynpp_Params{int expand_typedefs;int qvar_to_Cids;int add_cyc_prefix;int to_VC;int decls_first;int rewrite_temp_tvars;int print_all_tvars;int print_all_kinds;int print_all_effects;int print_using_stmts;int print_externC_stmts;int print_full_evars;int print_zeroterm;int generate_line_directives;int use_curr_namespace;struct Cyc_List_List*curr_namespace;};
# 51 "absynpp.h"
extern int Cyc_Absynpp_print_for_cycdoc;
# 59
struct Cyc_PP_Doc*Cyc_Absynpp_decl2doc(struct Cyc_Absyn_Decl*d);
# 61
struct _fat_ptr Cyc_Absynpp_longlong2string(unsigned long long);
struct _fat_ptr Cyc_Absynpp_typ2string(void*);
# 64
struct _fat_ptr Cyc_Absynpp_kind2string(struct Cyc_Absyn_Kind*);
# 69
struct _fat_ptr Cyc_Absynpp_qvar2string(struct _tuple0*);
# 76
int Cyc_Absynpp_is_anon_aggrtype(void*t);
extern struct _fat_ptr Cyc_Absynpp_cyc_string;
extern struct _fat_ptr*Cyc_Absynpp_cyc_stringptr;
int Cyc_Absynpp_exp_prec(struct Cyc_Absyn_Exp*);
struct _fat_ptr Cyc_Absynpp_char_escape(char);
struct _fat_ptr Cyc_Absynpp_string_escape(struct _fat_ptr);
struct _fat_ptr Cyc_Absynpp_prim2str(enum Cyc_Absyn_Primop p);
int Cyc_Absynpp_is_declaration(struct Cyc_Absyn_Stmt*s);struct _tuple13{struct Cyc_Absyn_Tqual f1;void*f2;struct Cyc_List_List*f3;};
struct _tuple13 Cyc_Absynpp_to_tms(struct _RegionHandle*,struct Cyc_Absyn_Tqual tq,void*t);struct _tuple14{int f1;struct Cyc_List_List*f2;};
# 93 "absynpp.h"
struct _tuple14 Cyc_Absynpp_shadows(struct Cyc_Absyn_Decl*d,struct Cyc_List_List*varsinblock);
# 26 "cyclone.h"
enum Cyc_Cyclone_C_Compilers{Cyc_Cyclone_Gcc_c =0U,Cyc_Cyclone_Vc_c =1U};
# 32
extern enum Cyc_Cyclone_C_Compilers Cyc_Cyclone_c_compiler;struct _tuple15{struct Cyc_List_List*f1;struct Cyc_Absyn_Pat*f2;};
# 39 "absynpp.cyc"
struct Cyc_PP_Doc*Cyc_Absynpp_dp2doc(struct _tuple15*dp);
struct Cyc_PP_Doc*Cyc_Absynpp_switchclauses2doc(struct Cyc_List_List*cs);
struct Cyc_PP_Doc*Cyc_Absynpp_typ2doc(void*);
struct Cyc_PP_Doc*Cyc_Absynpp_aggrfields2doc(struct Cyc_List_List*fields);
struct Cyc_PP_Doc*Cyc_Absynpp_scope2doc(enum Cyc_Absyn_Scope);
struct Cyc_PP_Doc*Cyc_Absynpp_stmt2doc(struct Cyc_Absyn_Stmt*,int expstmt,struct Cyc_List_List*varsinblock);
struct Cyc_PP_Doc*Cyc_Absynpp_exp2doc(struct Cyc_Absyn_Exp*);
struct Cyc_PP_Doc*Cyc_Absynpp_exp2doc_prec(int inprec,struct Cyc_Absyn_Exp*e);
struct Cyc_PP_Doc*Cyc_Absynpp_exps2doc_prec(int inprec,struct Cyc_List_List*es);
struct Cyc_PP_Doc*Cyc_Absynpp_qvar2doc(struct _tuple0*);
struct Cyc_PP_Doc*Cyc_Absynpp_typedef_name2doc(struct _tuple0*);
struct Cyc_PP_Doc*Cyc_Absynpp_cnst2doc(union Cyc_Absyn_Cnst);
struct Cyc_PP_Doc*Cyc_Absynpp_prim2doc(enum Cyc_Absyn_Primop);
struct Cyc_PP_Doc*Cyc_Absynpp_primapp2doc(int inprec,enum Cyc_Absyn_Primop p,struct Cyc_List_List*es);struct _tuple16{struct Cyc_List_List*f1;struct Cyc_Absyn_Exp*f2;};
struct Cyc_PP_Doc*Cyc_Absynpp_de2doc(struct _tuple16*de);
struct Cyc_PP_Doc*Cyc_Absynpp_tqtd2doc(struct Cyc_Absyn_Tqual tq,void*t,struct Cyc_Core_Opt*dopt);
struct Cyc_PP_Doc*Cyc_Absynpp_funargs2doc(struct Cyc_List_List*args,int c_varargs,struct Cyc_Absyn_VarargInfo*cyc_varargs,void*effopt,struct Cyc_List_List*rgn_po,struct Cyc_Absyn_Exp*requires,struct Cyc_Absyn_Exp*ensures,struct Cyc_List_List*ieff,struct Cyc_List_List*oeff);
# 63
struct Cyc_PP_Doc*Cyc_Absynpp_datatypefields2doc(struct Cyc_List_List*fields);
struct Cyc_PP_Doc*Cyc_Absynpp_enumfields2doc(struct Cyc_List_List*fs);
struct Cyc_PP_Doc*Cyc_Absynpp_vardecl2doc(struct Cyc_Absyn_Vardecl*vd);
struct Cyc_PP_Doc*Cyc_Absynpp_aggrdecl2doc(struct Cyc_Absyn_Aggrdecl*ad);
struct Cyc_PP_Doc*Cyc_Absynpp_enumdecl2doc(struct Cyc_Absyn_Enumdecl*ad);
struct Cyc_PP_Doc*Cyc_Absynpp_datatypedecl2doc(struct Cyc_Absyn_Datatypedecl*ad);
# 70
static int Cyc_Absynpp_expand_typedefs;
# 74
static int Cyc_Absynpp_qvar_to_Cids;static char _tmp0[4U]="Cyc";
# 76
struct _fat_ptr Cyc_Absynpp_cyc_string={_tmp0,_tmp0,_tmp0 + 4U};
struct _fat_ptr*Cyc_Absynpp_cyc_stringptr=& Cyc_Absynpp_cyc_string;
# 79
static int Cyc_Absynpp_add_cyc_prefix;
# 83
static int Cyc_Absynpp_to_VC;
# 86
static int Cyc_Absynpp_decls_first;
# 90
static int Cyc_Absynpp_rewrite_temp_tvars;
# 93
static int Cyc_Absynpp_print_all_tvars;
# 96
static int Cyc_Absynpp_print_all_kinds;
# 99
static int Cyc_Absynpp_print_all_effects;
# 102
static int Cyc_Absynpp_print_using_stmts;
# 107
static int Cyc_Absynpp_print_externC_stmts;
# 111
static int Cyc_Absynpp_print_full_evars;
# 114
static int Cyc_Absynpp_generate_line_directives;
# 117
static int Cyc_Absynpp_use_curr_namespace;
# 120
static int Cyc_Absynpp_print_zeroterm;
# 123
static struct Cyc_List_List*Cyc_Absynpp_curr_namespace=0;
# 126
int Cyc_Absynpp_print_for_cycdoc=0;
# 147
void Cyc_Absynpp_set_params(struct Cyc_Absynpp_Params*fs){
Cyc_Absynpp_expand_typedefs=fs->expand_typedefs;
Cyc_Absynpp_qvar_to_Cids=fs->qvar_to_Cids;
Cyc_Absynpp_add_cyc_prefix=fs->add_cyc_prefix;
Cyc_Absynpp_to_VC=fs->to_VC;
Cyc_Absynpp_decls_first=fs->decls_first;
Cyc_Absynpp_rewrite_temp_tvars=fs->rewrite_temp_tvars;
Cyc_Absynpp_print_all_tvars=fs->print_all_tvars;
Cyc_Absynpp_print_all_kinds=fs->print_all_kinds;
Cyc_Absynpp_print_all_effects=fs->print_all_effects;
Cyc_Absynpp_print_using_stmts=fs->print_using_stmts;
Cyc_Absynpp_print_externC_stmts=fs->print_externC_stmts;
Cyc_Absynpp_print_full_evars=fs->print_full_evars;
Cyc_Absynpp_print_zeroterm=fs->print_zeroterm;
Cyc_Absynpp_generate_line_directives=fs->generate_line_directives;
Cyc_Absynpp_use_curr_namespace=fs->use_curr_namespace;
Cyc_Absynpp_curr_namespace=fs->curr_namespace;}
# 147
struct Cyc_Absynpp_Params Cyc_Absynpp_cyc_params_r={0,0,0,0,0,1,0,0,0,1,1,0,1,0,1,0};
# 186
struct Cyc_Absynpp_Params Cyc_Absynpp_cyci_params_r={1,0,1,0,0,1,0,0,0,1,1,0,1,0,1,0};
# 206
struct Cyc_Absynpp_Params Cyc_Absynpp_c_params_r={1,1,1,0,1,0,0,0,0,0,0,0,0,1,0,0};
# 226
struct Cyc_Absynpp_Params Cyc_Absynpp_tc_params_r={0,0,0,0,0,1,0,0,0,1,1,0,1,0,0,0};
# 247
static void Cyc_Absynpp_curr_namespace_add(struct _fat_ptr*v){
Cyc_Absynpp_curr_namespace=({struct Cyc_List_List*_tmp56D=Cyc_Absynpp_curr_namespace;Cyc_List_imp_append(_tmp56D,({struct Cyc_List_List*_tmp1=_cycalloc(sizeof(*_tmp1));_tmp1->hd=v,_tmp1->tl=0;_tmp1;}));});}
# 247
static void Cyc_Absynpp_suppr_last(struct Cyc_List_List**l){
# 252
if(((struct Cyc_List_List*)_check_null(*l))->tl == 0)
*l=0;else{
# 255
Cyc_Absynpp_suppr_last(&((struct Cyc_List_List*)_check_null(*l))->tl);}}
# 247
static void Cyc_Absynpp_curr_namespace_drop(){
# 260
Cyc_Absynpp_suppr_last(& Cyc_Absynpp_curr_namespace);}
# 247
struct _fat_ptr Cyc_Absynpp_char_escape(char c){
# 264
char _tmp2=c;switch(_tmp2){case 7U: _LL1: _LL2:
 return({const char*_tmp3="\\a";_tag_fat(_tmp3,sizeof(char),3U);});case 8U: _LL3: _LL4:
 return({const char*_tmp4="\\b";_tag_fat(_tmp4,sizeof(char),3U);});case 12U: _LL5: _LL6:
 return({const char*_tmp5="\\f";_tag_fat(_tmp5,sizeof(char),3U);});case 10U: _LL7: _LL8:
 return({const char*_tmp6="\\n";_tag_fat(_tmp6,sizeof(char),3U);});case 13U: _LL9: _LLA:
 return({const char*_tmp7="\\r";_tag_fat(_tmp7,sizeof(char),3U);});case 9U: _LLB: _LLC:
 return({const char*_tmp8="\\t";_tag_fat(_tmp8,sizeof(char),3U);});case 11U: _LLD: _LLE:
 return({const char*_tmp9="\\v";_tag_fat(_tmp9,sizeof(char),3U);});case 92U: _LLF: _LL10:
 return({const char*_tmpA="\\\\";_tag_fat(_tmpA,sizeof(char),3U);});case 34U: _LL11: _LL12:
 return({const char*_tmpB="\"";_tag_fat(_tmpB,sizeof(char),2U);});case 39U: _LL13: _LL14:
 return({const char*_tmpC="\\'";_tag_fat(_tmpC,sizeof(char),3U);});default: _LL15: _LL16:
# 276
 if((int)c >= (int)' ' &&(int)c <= (int)'~'){
struct _fat_ptr t=Cyc_Core_new_string(2U);
({struct _fat_ptr _tmpD=_fat_ptr_plus(t,sizeof(char),0);char _tmpE=*((char*)_check_fat_subscript(_tmpD,sizeof(char),0U));char _tmpF=c;if(_get_fat_size(_tmpD,sizeof(char))== 1U &&(_tmpE == 0 && _tmpF != 0))_throw_arraybounds();*((char*)_tmpD.curr)=_tmpF;});
return(struct _fat_ptr)t;}else{
# 281
struct _fat_ptr t=Cyc_Core_new_string(5U);
int j=0;
({struct _fat_ptr _tmp10=_fat_ptr_plus(t,sizeof(char),j ++);char _tmp11=*((char*)_check_fat_subscript(_tmp10,sizeof(char),0U));char _tmp12='\\';if(_get_fat_size(_tmp10,sizeof(char))== 1U &&(_tmp11 == 0 && _tmp12 != 0))_throw_arraybounds();*((char*)_tmp10.curr)=_tmp12;});
({struct _fat_ptr _tmp13=_fat_ptr_plus(t,sizeof(char),j ++);char _tmp14=*((char*)_check_fat_subscript(_tmp13,sizeof(char),0U));char _tmp15=(char)((int)'0' + ((int)((unsigned char)c)>> 6 & 3));if(_get_fat_size(_tmp13,sizeof(char))== 1U &&(_tmp14 == 0 && _tmp15 != 0))_throw_arraybounds();*((char*)_tmp13.curr)=_tmp15;});
({struct _fat_ptr _tmp16=_fat_ptr_plus(t,sizeof(char),j ++);char _tmp17=*((char*)_check_fat_subscript(_tmp16,sizeof(char),0U));char _tmp18=(char)((int)'0' + ((int)c >> 3 & 7));if(_get_fat_size(_tmp16,sizeof(char))== 1U &&(_tmp17 == 0 && _tmp18 != 0))_throw_arraybounds();*((char*)_tmp16.curr)=_tmp18;});
({struct _fat_ptr _tmp19=_fat_ptr_plus(t,sizeof(char),j ++);char _tmp1A=*((char*)_check_fat_subscript(_tmp19,sizeof(char),0U));char _tmp1B=(char)((int)'0' + ((int)c & 7));if(_get_fat_size(_tmp19,sizeof(char))== 1U &&(_tmp1A == 0 && _tmp1B != 0))_throw_arraybounds();*((char*)_tmp19.curr)=_tmp1B;});
return(struct _fat_ptr)t;}}_LL0:;}
# 247
static int Cyc_Absynpp_special(struct _fat_ptr s){
# 293
int sz=(int)(_get_fat_size(s,sizeof(char))- (unsigned)1);
{int i=0;for(0;i < sz;++ i){
char c=*((const char*)_check_fat_subscript(s,sizeof(char),i));
if((((int)c <= (int)' ' ||(int)c >= (int)'~')||(int)c == (int)'"')||(int)c == (int)'\\')
return 1;}}
# 299
return 0;}
# 247
struct _fat_ptr Cyc_Absynpp_string_escape(struct _fat_ptr s){
# 303
if(!Cyc_Absynpp_special(s))return s;{int n=(int)(
# 305
_get_fat_size(s,sizeof(char))- (unsigned)1);
# 307
if(n > 0 &&(int)*((const char*)_check_fat_subscript(s,sizeof(char),n))== (int)'\000')-- n;{int len=0;
# 310
{int i=0;for(0;i <= n;++ i){
char _stmttmp0=*((const char*)_check_fat_subscript(s,sizeof(char),i));char _tmp1C=_stmttmp0;char _tmp1D;switch(_tmp1C){case 7U: _LL1: _LL2:
 goto _LL4;case 8U: _LL3: _LL4:
 goto _LL6;case 12U: _LL5: _LL6:
 goto _LL8;case 10U: _LL7: _LL8:
 goto _LLA;case 13U: _LL9: _LLA:
 goto _LLC;case 9U: _LLB: _LLC:
 goto _LLE;case 11U: _LLD: _LLE:
 goto _LL10;case 92U: _LLF: _LL10:
 goto _LL12;case 34U: _LL11: _LL12:
 len +=2;goto _LL0;default: _LL13: _tmp1D=_tmp1C;_LL14: {char c=_tmp1D;
# 322
if((int)c >= (int)' ' &&(int)c <= (int)'~')++ len;else{
len +=4;}
goto _LL0;}}_LL0:;}}{
# 327
struct _fat_ptr t=Cyc_Core_new_string((unsigned)(len + 1));
int j=0;
{int i=0;for(0;i <= n;++ i){
char _stmttmp1=*((const char*)_check_fat_subscript(s,sizeof(char),i));char _tmp1E=_stmttmp1;char _tmp1F;switch(_tmp1E){case 7U: _LL16: _LL17:
({struct _fat_ptr _tmp20=_fat_ptr_plus(t,sizeof(char),j ++);char _tmp21=*((char*)_check_fat_subscript(_tmp20,sizeof(char),0U));char _tmp22='\\';if(_get_fat_size(_tmp20,sizeof(char))== 1U &&(_tmp21 == 0 && _tmp22 != 0))_throw_arraybounds();*((char*)_tmp20.curr)=_tmp22;});({struct _fat_ptr _tmp23=_fat_ptr_plus(t,sizeof(char),j ++);char _tmp24=*((char*)_check_fat_subscript(_tmp23,sizeof(char),0U));char _tmp25='a';if(_get_fat_size(_tmp23,sizeof(char))== 1U &&(_tmp24 == 0 && _tmp25 != 0))_throw_arraybounds();*((char*)_tmp23.curr)=_tmp25;});goto _LL15;case 8U: _LL18: _LL19:
({struct _fat_ptr _tmp26=_fat_ptr_plus(t,sizeof(char),j ++);char _tmp27=*((char*)_check_fat_subscript(_tmp26,sizeof(char),0U));char _tmp28='\\';if(_get_fat_size(_tmp26,sizeof(char))== 1U &&(_tmp27 == 0 && _tmp28 != 0))_throw_arraybounds();*((char*)_tmp26.curr)=_tmp28;});({struct _fat_ptr _tmp29=_fat_ptr_plus(t,sizeof(char),j ++);char _tmp2A=*((char*)_check_fat_subscript(_tmp29,sizeof(char),0U));char _tmp2B='b';if(_get_fat_size(_tmp29,sizeof(char))== 1U &&(_tmp2A == 0 && _tmp2B != 0))_throw_arraybounds();*((char*)_tmp29.curr)=_tmp2B;});goto _LL15;case 12U: _LL1A: _LL1B:
({struct _fat_ptr _tmp2C=_fat_ptr_plus(t,sizeof(char),j ++);char _tmp2D=*((char*)_check_fat_subscript(_tmp2C,sizeof(char),0U));char _tmp2E='\\';if(_get_fat_size(_tmp2C,sizeof(char))== 1U &&(_tmp2D == 0 && _tmp2E != 0))_throw_arraybounds();*((char*)_tmp2C.curr)=_tmp2E;});({struct _fat_ptr _tmp2F=_fat_ptr_plus(t,sizeof(char),j ++);char _tmp30=*((char*)_check_fat_subscript(_tmp2F,sizeof(char),0U));char _tmp31='f';if(_get_fat_size(_tmp2F,sizeof(char))== 1U &&(_tmp30 == 0 && _tmp31 != 0))_throw_arraybounds();*((char*)_tmp2F.curr)=_tmp31;});goto _LL15;case 10U: _LL1C: _LL1D:
({struct _fat_ptr _tmp32=_fat_ptr_plus(t,sizeof(char),j ++);char _tmp33=*((char*)_check_fat_subscript(_tmp32,sizeof(char),0U));char _tmp34='\\';if(_get_fat_size(_tmp32,sizeof(char))== 1U &&(_tmp33 == 0 && _tmp34 != 0))_throw_arraybounds();*((char*)_tmp32.curr)=_tmp34;});({struct _fat_ptr _tmp35=_fat_ptr_plus(t,sizeof(char),j ++);char _tmp36=*((char*)_check_fat_subscript(_tmp35,sizeof(char),0U));char _tmp37='n';if(_get_fat_size(_tmp35,sizeof(char))== 1U &&(_tmp36 == 0 && _tmp37 != 0))_throw_arraybounds();*((char*)_tmp35.curr)=_tmp37;});goto _LL15;case 13U: _LL1E: _LL1F:
({struct _fat_ptr _tmp38=_fat_ptr_plus(t,sizeof(char),j ++);char _tmp39=*((char*)_check_fat_subscript(_tmp38,sizeof(char),0U));char _tmp3A='\\';if(_get_fat_size(_tmp38,sizeof(char))== 1U &&(_tmp39 == 0 && _tmp3A != 0))_throw_arraybounds();*((char*)_tmp38.curr)=_tmp3A;});({struct _fat_ptr _tmp3B=_fat_ptr_plus(t,sizeof(char),j ++);char _tmp3C=*((char*)_check_fat_subscript(_tmp3B,sizeof(char),0U));char _tmp3D='r';if(_get_fat_size(_tmp3B,sizeof(char))== 1U &&(_tmp3C == 0 && _tmp3D != 0))_throw_arraybounds();*((char*)_tmp3B.curr)=_tmp3D;});goto _LL15;case 9U: _LL20: _LL21:
({struct _fat_ptr _tmp3E=_fat_ptr_plus(t,sizeof(char),j ++);char _tmp3F=*((char*)_check_fat_subscript(_tmp3E,sizeof(char),0U));char _tmp40='\\';if(_get_fat_size(_tmp3E,sizeof(char))== 1U &&(_tmp3F == 0 && _tmp40 != 0))_throw_arraybounds();*((char*)_tmp3E.curr)=_tmp40;});({struct _fat_ptr _tmp41=_fat_ptr_plus(t,sizeof(char),j ++);char _tmp42=*((char*)_check_fat_subscript(_tmp41,sizeof(char),0U));char _tmp43='t';if(_get_fat_size(_tmp41,sizeof(char))== 1U &&(_tmp42 == 0 && _tmp43 != 0))_throw_arraybounds();*((char*)_tmp41.curr)=_tmp43;});goto _LL15;case 11U: _LL22: _LL23:
({struct _fat_ptr _tmp44=_fat_ptr_plus(t,sizeof(char),j ++);char _tmp45=*((char*)_check_fat_subscript(_tmp44,sizeof(char),0U));char _tmp46='\\';if(_get_fat_size(_tmp44,sizeof(char))== 1U &&(_tmp45 == 0 && _tmp46 != 0))_throw_arraybounds();*((char*)_tmp44.curr)=_tmp46;});({struct _fat_ptr _tmp47=_fat_ptr_plus(t,sizeof(char),j ++);char _tmp48=*((char*)_check_fat_subscript(_tmp47,sizeof(char),0U));char _tmp49='v';if(_get_fat_size(_tmp47,sizeof(char))== 1U &&(_tmp48 == 0 && _tmp49 != 0))_throw_arraybounds();*((char*)_tmp47.curr)=_tmp49;});goto _LL15;case 92U: _LL24: _LL25:
({struct _fat_ptr _tmp4A=_fat_ptr_plus(t,sizeof(char),j ++);char _tmp4B=*((char*)_check_fat_subscript(_tmp4A,sizeof(char),0U));char _tmp4C='\\';if(_get_fat_size(_tmp4A,sizeof(char))== 1U &&(_tmp4B == 0 && _tmp4C != 0))_throw_arraybounds();*((char*)_tmp4A.curr)=_tmp4C;});({struct _fat_ptr _tmp4D=_fat_ptr_plus(t,sizeof(char),j ++);char _tmp4E=*((char*)_check_fat_subscript(_tmp4D,sizeof(char),0U));char _tmp4F='\\';if(_get_fat_size(_tmp4D,sizeof(char))== 1U &&(_tmp4E == 0 && _tmp4F != 0))_throw_arraybounds();*((char*)_tmp4D.curr)=_tmp4F;});goto _LL15;case 34U: _LL26: _LL27:
({struct _fat_ptr _tmp50=_fat_ptr_plus(t,sizeof(char),j ++);char _tmp51=*((char*)_check_fat_subscript(_tmp50,sizeof(char),0U));char _tmp52='\\';if(_get_fat_size(_tmp50,sizeof(char))== 1U &&(_tmp51 == 0 && _tmp52 != 0))_throw_arraybounds();*((char*)_tmp50.curr)=_tmp52;});({struct _fat_ptr _tmp53=_fat_ptr_plus(t,sizeof(char),j ++);char _tmp54=*((char*)_check_fat_subscript(_tmp53,sizeof(char),0U));char _tmp55='"';if(_get_fat_size(_tmp53,sizeof(char))== 1U &&(_tmp54 == 0 && _tmp55 != 0))_throw_arraybounds();*((char*)_tmp53.curr)=_tmp55;});goto _LL15;default: _LL28: _tmp1F=_tmp1E;_LL29: {char c=_tmp1F;
# 341
if((int)c >= (int)' ' &&(int)c <= (int)'~')({struct _fat_ptr _tmp56=_fat_ptr_plus(t,sizeof(char),j ++);char _tmp57=*((char*)_check_fat_subscript(_tmp56,sizeof(char),0U));char _tmp58=c;if(_get_fat_size(_tmp56,sizeof(char))== 1U &&(_tmp57 == 0 && _tmp58 != 0))_throw_arraybounds();*((char*)_tmp56.curr)=_tmp58;});else{
# 343
({struct _fat_ptr _tmp59=_fat_ptr_plus(t,sizeof(char),j ++);char _tmp5A=*((char*)_check_fat_subscript(_tmp59,sizeof(char),0U));char _tmp5B='\\';if(_get_fat_size(_tmp59,sizeof(char))== 1U &&(_tmp5A == 0 && _tmp5B != 0))_throw_arraybounds();*((char*)_tmp59.curr)=_tmp5B;});
({struct _fat_ptr _tmp5C=_fat_ptr_plus(t,sizeof(char),j ++);char _tmp5D=*((char*)_check_fat_subscript(_tmp5C,sizeof(char),0U));char _tmp5E=(char)((int)'0' + ((int)c >> 6 & 7));if(_get_fat_size(_tmp5C,sizeof(char))== 1U &&(_tmp5D == 0 && _tmp5E != 0))_throw_arraybounds();*((char*)_tmp5C.curr)=_tmp5E;});
({struct _fat_ptr _tmp5F=_fat_ptr_plus(t,sizeof(char),j ++);char _tmp60=*((char*)_check_fat_subscript(_tmp5F,sizeof(char),0U));char _tmp61=(char)((int)'0' + ((int)c >> 3 & 7));if(_get_fat_size(_tmp5F,sizeof(char))== 1U &&(_tmp60 == 0 && _tmp61 != 0))_throw_arraybounds();*((char*)_tmp5F.curr)=_tmp61;});
({struct _fat_ptr _tmp62=_fat_ptr_plus(t,sizeof(char),j ++);char _tmp63=*((char*)_check_fat_subscript(_tmp62,sizeof(char),0U));char _tmp64=(char)((int)'0' + ((int)c & 7));if(_get_fat_size(_tmp62,sizeof(char))== 1U &&(_tmp63 == 0 && _tmp64 != 0))_throw_arraybounds();*((char*)_tmp62.curr)=_tmp64;});}
# 348
goto _LL15;}}_LL15:;}}
# 350
return(struct _fat_ptr)t;}}}}static char _tmp65[9U]="restrict";
# 247
static struct _fat_ptr Cyc_Absynpp_restrict_string={_tmp65,_tmp65,_tmp65 + 9U};static char _tmp66[9U]="volatile";
# 354
static struct _fat_ptr Cyc_Absynpp_volatile_string={_tmp66,_tmp66,_tmp66 + 9U};static char _tmp67[6U]="const";
static struct _fat_ptr Cyc_Absynpp_const_str={_tmp67,_tmp67,_tmp67 + 6U};
static struct _fat_ptr*Cyc_Absynpp_restrict_sp=& Cyc_Absynpp_restrict_string;
static struct _fat_ptr*Cyc_Absynpp_volatile_sp=& Cyc_Absynpp_volatile_string;
static struct _fat_ptr*Cyc_Absynpp_const_sp=& Cyc_Absynpp_const_str;
# 360
struct Cyc_PP_Doc*Cyc_Absynpp_tqual2doc(struct Cyc_Absyn_Tqual tq){
struct Cyc_List_List*l=0;
# 363
if(tq.q_restrict)l=({struct Cyc_List_List*_tmp68=_cycalloc(sizeof(*_tmp68));_tmp68->hd=Cyc_Absynpp_restrict_sp,_tmp68->tl=l;_tmp68;});if(tq.q_volatile)
l=({struct Cyc_List_List*_tmp69=_cycalloc(sizeof(*_tmp69));_tmp69->hd=Cyc_Absynpp_volatile_sp,_tmp69->tl=l;_tmp69;});
# 363
if(tq.print_const)
# 365
l=({struct Cyc_List_List*_tmp6A=_cycalloc(sizeof(*_tmp6A));_tmp6A->hd=Cyc_Absynpp_const_sp,_tmp6A->tl=l;_tmp6A;});
# 363
return({
# 366
struct _fat_ptr _tmp571=({const char*_tmp6B="";_tag_fat(_tmp6B,sizeof(char),1U);});struct _fat_ptr _tmp570=({const char*_tmp6C=" ";_tag_fat(_tmp6C,sizeof(char),2U);});struct _fat_ptr _tmp56F=({const char*_tmp6D=" ";_tag_fat(_tmp6D,sizeof(char),2U);});Cyc_PP_egroup(_tmp571,_tmp570,_tmp56F,({struct Cyc_List_List*(*_tmp56E)(struct Cyc_PP_Doc*(*f)(struct _fat_ptr*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp6E)(struct Cyc_PP_Doc*(*f)(struct _fat_ptr*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_PP_Doc*(*f)(struct _fat_ptr*),struct Cyc_List_List*x))Cyc_List_map;_tmp6E;});_tmp56E(Cyc_PP_textptr,l);}));});}
# 360
struct _fat_ptr Cyc_Absynpp_kind2string(struct Cyc_Absyn_Kind*ka){
# 370
enum Cyc_Absyn_AliasQual _tmp70;enum Cyc_Absyn_KindQual _tmp6F;_LL1: _tmp6F=ka->kind;_tmp70=ka->aliasqual;_LL2: {enum Cyc_Absyn_KindQual k=_tmp6F;enum Cyc_Absyn_AliasQual a=_tmp70;
enum Cyc_Absyn_KindQual _tmp71=k;switch(_tmp71){case Cyc_Absyn_AnyKind: _LL4: _LL5: {
# 373
enum Cyc_Absyn_AliasQual _tmp72=a;switch(_tmp72){case Cyc_Absyn_Aliasable: _LL17: _LL18:
 return({const char*_tmp73="A";_tag_fat(_tmp73,sizeof(char),2U);});case Cyc_Absyn_Unique: _LL19: _LL1A:
 return({const char*_tmp74="UA";_tag_fat(_tmp74,sizeof(char),3U);});case Cyc_Absyn_Top: _LL1B: _LL1C:
 goto _LL1E;default: _LL1D: _LL1E: return({const char*_tmp75="TA";_tag_fat(_tmp75,sizeof(char),3U);});}_LL16:;}case Cyc_Absyn_MemKind: _LL6: _LL7: {
# 379
enum Cyc_Absyn_AliasQual _tmp76=a;switch(_tmp76){case Cyc_Absyn_Aliasable: _LL20: _LL21:
 return({const char*_tmp77="M";_tag_fat(_tmp77,sizeof(char),2U);});case Cyc_Absyn_Unique: _LL22: _LL23:
 return({const char*_tmp78="UM";_tag_fat(_tmp78,sizeof(char),3U);});case Cyc_Absyn_Top: _LL24: _LL25:
 goto _LL27;default: _LL26: _LL27: return({const char*_tmp79="TM";_tag_fat(_tmp79,sizeof(char),3U);});}_LL1F:;}case Cyc_Absyn_BoxKind: _LL8: _LL9: {
# 385
enum Cyc_Absyn_AliasQual _tmp7A=a;switch(_tmp7A){case Cyc_Absyn_Aliasable: _LL29: _LL2A:
 return({const char*_tmp7B="B";_tag_fat(_tmp7B,sizeof(char),2U);});case Cyc_Absyn_Unique: _LL2B: _LL2C:
 return({const char*_tmp7C="UB";_tag_fat(_tmp7C,sizeof(char),3U);});case Cyc_Absyn_Top: _LL2D: _LL2E:
 goto _LL30;default: _LL2F: _LL30: return({const char*_tmp7D="TB";_tag_fat(_tmp7D,sizeof(char),3U);});}_LL28:;}case Cyc_Absyn_RgnKind: _LLA: _LLB: {
# 391
enum Cyc_Absyn_AliasQual _tmp7E=a;switch(_tmp7E){case Cyc_Absyn_Aliasable: _LL32: _LL33:
 return({const char*_tmp7F="R";_tag_fat(_tmp7F,sizeof(char),2U);});case Cyc_Absyn_Unique: _LL34: _LL35:
 return({const char*_tmp80="UR";_tag_fat(_tmp80,sizeof(char),3U);});case Cyc_Absyn_Top: _LL36: _LL37:
 goto _LL39;default: _LL38: _LL39: return({const char*_tmp81="TR";_tag_fat(_tmp81,sizeof(char),3U);});}_LL31:;}case Cyc_Absyn_EffKind: _LLC: _LLD:
# 396
 return({const char*_tmp82="E";_tag_fat(_tmp82,sizeof(char),2U);});case Cyc_Absyn_IntKind: _LLE: _LLF:
 return({const char*_tmp83="I";_tag_fat(_tmp83,sizeof(char),2U);});case Cyc_Absyn_BoolKind: _LL10: _LL11:
 return({const char*_tmp84="BOOL";_tag_fat(_tmp84,sizeof(char),5U);});case Cyc_Absyn_PtrBndKind: _LL12: _LL13:
 goto _LL15;default: _LL14: _LL15: return({const char*_tmp85="PTRBND";_tag_fat(_tmp85,sizeof(char),7U);});}_LL3:;}}
# 360
struct Cyc_PP_Doc*Cyc_Absynpp_kind2doc(struct Cyc_Absyn_Kind*k){
# 402
return Cyc_PP_text(Cyc_Absynpp_kind2string(k));}
# 360
struct _fat_ptr Cyc_Absynpp_kindbound2string(void*c){
# 405
void*_stmttmp2=Cyc_Absyn_compress_kb(c);void*_tmp86=_stmttmp2;struct Cyc_Absyn_Kind*_tmp87;struct Cyc_Absyn_Kind*_tmp88;switch(*((int*)_tmp86)){case 0U: _LL1: _tmp88=((struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct*)_tmp86)->f1;_LL2: {struct Cyc_Absyn_Kind*k=_tmp88;
return Cyc_Absynpp_kind2string(k);}case 1U: _LL3: _LL4:
# 408
 if(Cyc_PP_tex_output)
return({const char*_tmp89="{?}";_tag_fat(_tmp89,sizeof(char),4U);});else{
return({const char*_tmp8A="?";_tag_fat(_tmp8A,sizeof(char),2U);});}default: _LL5: _tmp87=((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp86)->f2;_LL6: {struct Cyc_Absyn_Kind*k=_tmp87;
return(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp8D=({struct Cyc_String_pa_PrintArg_struct _tmp54E;_tmp54E.tag=0U,({struct _fat_ptr _tmp572=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_kind2string(k));_tmp54E.f1=_tmp572;});_tmp54E;});void*_tmp8B[1U];_tmp8B[0]=& _tmp8D;({struct _fat_ptr _tmp573=({const char*_tmp8C="<=%s";_tag_fat(_tmp8C,sizeof(char),5U);});Cyc_aprintf(_tmp573,_tag_fat(_tmp8B,sizeof(void*),1U));});});}}_LL0:;}
# 360
struct Cyc_PP_Doc*Cyc_Absynpp_kindbound2doc(void*c){
# 416
void*_stmttmp3=Cyc_Absyn_compress_kb(c);void*_tmp8E=_stmttmp3;struct Cyc_Absyn_Kind*_tmp8F;struct Cyc_Absyn_Kind*_tmp90;switch(*((int*)_tmp8E)){case 0U: _LL1: _tmp90=((struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct*)_tmp8E)->f1;_LL2: {struct Cyc_Absyn_Kind*k=_tmp90;
return Cyc_PP_text(Cyc_Absynpp_kind2string(k));}case 1U: _LL3: _LL4:
# 419
 if(Cyc_PP_tex_output)
return Cyc_PP_text_width(({const char*_tmp91="{?}";_tag_fat(_tmp91,sizeof(char),4U);}),1);else{
return Cyc_PP_text(({const char*_tmp92="?";_tag_fat(_tmp92,sizeof(char),2U);}));}default: _LL5: _tmp8F=((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp8E)->f2;_LL6: {struct Cyc_Absyn_Kind*k=_tmp8F;
return Cyc_PP_text(Cyc_Absynpp_kind2string(k));}}_LL0:;}
# 360
struct Cyc_PP_Doc*Cyc_Absynpp_tps2doc(struct Cyc_List_List*ts){
# 427
return({struct _fat_ptr _tmp577=({const char*_tmp93="<";_tag_fat(_tmp93,sizeof(char),2U);});struct _fat_ptr _tmp576=({const char*_tmp94=">";_tag_fat(_tmp94,sizeof(char),2U);});struct _fat_ptr _tmp575=({const char*_tmp95=",";_tag_fat(_tmp95,sizeof(char),2U);});Cyc_PP_egroup(_tmp577,_tmp576,_tmp575,({struct Cyc_List_List*(*_tmp574)(struct Cyc_PP_Doc*(*f)(void*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp96)(struct Cyc_PP_Doc*(*f)(void*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_PP_Doc*(*f)(void*),struct Cyc_List_List*x))Cyc_List_map;_tmp96;});_tmp574(Cyc_Absynpp_typ2doc,ts);}));});}
# 360
struct Cyc_PP_Doc*Cyc_Absynpp_tvar2doc(struct Cyc_Absyn_Tvar*tv){
# 431
struct _fat_ptr n=*tv->name;
# 434
if(Cyc_Absynpp_rewrite_temp_tvars && Cyc_Tcutil_is_temp_tvar(tv)){
# 436
struct _fat_ptr kstring=({const char*_tmp9E="K";_tag_fat(_tmp9E,sizeof(char),2U);});
{void*_stmttmp4=Cyc_Absyn_compress_kb(tv->kind);void*_tmp97=_stmttmp4;struct Cyc_Absyn_Kind*_tmp98;struct Cyc_Absyn_Kind*_tmp99;switch(*((int*)_tmp97)){case 2U: _LL1: _tmp99=((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp97)->f2;_LL2: {struct Cyc_Absyn_Kind*k=_tmp99;
_tmp98=k;goto _LL4;}case 0U: _LL3: _tmp98=((struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct*)_tmp97)->f1;_LL4: {struct Cyc_Absyn_Kind*k=_tmp98;
kstring=Cyc_Absynpp_kind2string(k);goto _LL0;}default: _LL5: _LL6:
 goto _LL0;}_LL0:;}
# 442
return Cyc_PP_text((struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp9C=({struct Cyc_String_pa_PrintArg_struct _tmp550;_tmp550.tag=0U,_tmp550.f1=(struct _fat_ptr)((struct _fat_ptr)kstring);_tmp550;});struct Cyc_String_pa_PrintArg_struct _tmp9D=({struct Cyc_String_pa_PrintArg_struct _tmp54F;_tmp54F.tag=0U,({struct _fat_ptr _tmp578=(struct _fat_ptr)((struct _fat_ptr)_fat_ptr_plus(n,sizeof(char),1));_tmp54F.f1=_tmp578;});_tmp54F;});void*_tmp9A[2U];_tmp9A[0]=& _tmp9C,_tmp9A[1]=& _tmp9D;({struct _fat_ptr _tmp579=({const char*_tmp9B="`G%s%s";_tag_fat(_tmp9B,sizeof(char),7U);});Cyc_aprintf(_tmp579,_tag_fat(_tmp9A,sizeof(void*),2U));});}));}
# 434
return
# 444
 Cyc_PP_text(n);}
# 360
struct Cyc_PP_Doc*Cyc_Absynpp_ktvar2doc(struct Cyc_Absyn_Tvar*tv){
# 448
void*_stmttmp5=Cyc_Absyn_compress_kb(tv->kind);void*_tmp9F=_stmttmp5;struct Cyc_Absyn_Kind*_tmpA0;struct Cyc_Absyn_Kind*_tmpA1;switch(*((int*)_tmp9F)){case 1U: _LL1: _LL2:
 goto _LL4;case 0U: if(((struct Cyc_Absyn_Kind*)((struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct*)_tmp9F)->f1)->kind == Cyc_Absyn_BoxKind){if(((struct Cyc_Absyn_Kind*)((struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct*)_tmp9F)->f1)->aliasqual == Cyc_Absyn_Aliasable){_LL3: _LL4:
 return Cyc_Absynpp_tvar2doc(tv);}else{goto _LL7;}}else{_LL7: _tmpA1=((struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct*)_tmp9F)->f1;_LL8: {struct Cyc_Absyn_Kind*k=_tmpA1;
# 452
return({struct Cyc_PP_Doc*_tmpA2[3U];({struct Cyc_PP_Doc*_tmp57C=Cyc_Absynpp_tvar2doc(tv);_tmpA2[0]=_tmp57C;}),({struct Cyc_PP_Doc*_tmp57B=Cyc_PP_text(({const char*_tmpA3="::";_tag_fat(_tmpA3,sizeof(char),3U);}));_tmpA2[1]=_tmp57B;}),({struct Cyc_PP_Doc*_tmp57A=Cyc_Absynpp_kind2doc(k);_tmpA2[2]=_tmp57A;});Cyc_PP_cat(_tag_fat(_tmpA2,sizeof(struct Cyc_PP_Doc*),3U));});}}default: _LL5: _tmpA0=((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp9F)->f2;_LL6: {struct Cyc_Absyn_Kind*k=_tmpA0;
# 451
_tmpA1=k;goto _LL8;}}_LL0:;}
# 360
struct Cyc_PP_Doc*Cyc_Absynpp_ktvars2doc(struct Cyc_List_List*tvs){
# 457
return({struct _fat_ptr _tmp580=({const char*_tmpA4="<";_tag_fat(_tmpA4,sizeof(char),2U);});struct _fat_ptr _tmp57F=({const char*_tmpA5=">";_tag_fat(_tmpA5,sizeof(char),2U);});struct _fat_ptr _tmp57E=({const char*_tmpA6=",";_tag_fat(_tmpA6,sizeof(char),2U);});Cyc_PP_egroup(_tmp580,_tmp57F,_tmp57E,({struct Cyc_List_List*(*_tmp57D)(struct Cyc_PP_Doc*(*f)(struct Cyc_Absyn_Tvar*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmpA7)(struct Cyc_PP_Doc*(*f)(struct Cyc_Absyn_Tvar*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_PP_Doc*(*f)(struct Cyc_Absyn_Tvar*),struct Cyc_List_List*x))Cyc_List_map;_tmpA7;});_tmp57D(Cyc_Absynpp_ktvar2doc,tvs);}));});}
# 360
struct Cyc_PP_Doc*Cyc_Absynpp_tvars2doc(struct Cyc_List_List*tvs){
# 461
if(Cyc_Absynpp_print_all_kinds)
return Cyc_Absynpp_ktvars2doc(tvs);
# 461
return({
# 463
struct _fat_ptr _tmp584=({const char*_tmpA8="<";_tag_fat(_tmpA8,sizeof(char),2U);});struct _fat_ptr _tmp583=({const char*_tmpA9=">";_tag_fat(_tmpA9,sizeof(char),2U);});struct _fat_ptr _tmp582=({const char*_tmpAA=",";_tag_fat(_tmpAA,sizeof(char),2U);});Cyc_PP_egroup(_tmp584,_tmp583,_tmp582,({struct Cyc_List_List*(*_tmp581)(struct Cyc_PP_Doc*(*f)(struct Cyc_Absyn_Tvar*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmpAB)(struct Cyc_PP_Doc*(*f)(struct Cyc_Absyn_Tvar*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_PP_Doc*(*f)(struct Cyc_Absyn_Tvar*),struct Cyc_List_List*x))Cyc_List_map;_tmpAB;});_tmp581(Cyc_Absynpp_tvar2doc,tvs);}));});}struct _tuple17{struct Cyc_Absyn_Tqual f1;void*f2;};
# 360
struct Cyc_PP_Doc*Cyc_Absynpp_arg2doc(struct _tuple17*t){
# 467
return Cyc_Absynpp_tqtd2doc((*t).f1,(*t).f2,0);}
# 360
struct Cyc_PP_Doc*Cyc_Absynpp_args2doc(struct Cyc_List_List*ts){
# 471
return({struct _fat_ptr _tmp588=({const char*_tmpAC="(";_tag_fat(_tmpAC,sizeof(char),2U);});struct _fat_ptr _tmp587=({const char*_tmpAD=")";_tag_fat(_tmpAD,sizeof(char),2U);});struct _fat_ptr _tmp586=({const char*_tmpAE=",";_tag_fat(_tmpAE,sizeof(char),2U);});Cyc_PP_group(_tmp588,_tmp587,_tmp586,({struct Cyc_List_List*(*_tmp585)(struct Cyc_PP_Doc*(*f)(struct _tuple17*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmpAF)(struct Cyc_PP_Doc*(*f)(struct _tuple17*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_PP_Doc*(*f)(struct _tuple17*),struct Cyc_List_List*x))Cyc_List_map;_tmpAF;});_tmp585(Cyc_Absynpp_arg2doc,ts);}));});}
# 360
struct Cyc_PP_Doc*Cyc_Absynpp_noncallatt2doc(void*att){
# 475
void*_tmpB0=att;switch(*((int*)_tmpB0)){case 1U: _LL1: _LL2:
 goto _LL4;case 2U: _LL3: _LL4:
 goto _LL6;case 3U: _LL5: _LL6:
 return Cyc_PP_nil_doc();default: _LL7: _LL8:
 return Cyc_PP_text(Cyc_Absyn_attribute2string(att));}_LL0:;}
# 360
struct Cyc_PP_Doc*Cyc_Absynpp_callconv2doc(struct Cyc_List_List*atts){
# 484
for(0;atts != 0;atts=atts->tl){
void*_stmttmp6=(void*)atts->hd;void*_tmpB1=_stmttmp6;switch(*((int*)_tmpB1)){case 1U: _LL1: _LL2:
 return Cyc_PP_text(({const char*_tmpB2=" _stdcall ";_tag_fat(_tmpB2,sizeof(char),11U);}));case 2U: _LL3: _LL4:
 return Cyc_PP_text(({const char*_tmpB3=" _cdecl ";_tag_fat(_tmpB3,sizeof(char),9U);}));case 3U: _LL5: _LL6:
 return Cyc_PP_text(({const char*_tmpB4=" _fastcall ";_tag_fat(_tmpB4,sizeof(char),12U);}));default: _LL7: _LL8:
 goto _LL0;}_LL0:;}
# 491
return Cyc_PP_nil_doc();}
# 360
struct Cyc_PP_Doc*Cyc_Absynpp_noncallconv2doc(struct Cyc_List_List*atts){
# 495
int hasatt=0;
{struct Cyc_List_List*atts2=atts;for(0;atts2 != 0;atts2=atts2->tl){
void*_stmttmp7=(void*)atts2->hd;void*_tmpB5=_stmttmp7;switch(*((int*)_tmpB5)){case 1U: _LL1: _LL2:
 goto _LL4;case 2U: _LL3: _LL4:
 goto _LL6;case 3U: _LL5: _LL6:
 goto _LL0;default: _LL7: _LL8:
 hasatt=1;goto _LL0;}_LL0:;}}
# 503
if(!hasatt)
return Cyc_PP_nil_doc();
# 503
return({struct Cyc_PP_Doc*_tmpB6[3U];({
# 505
struct Cyc_PP_Doc*_tmp58F=Cyc_PP_text(({const char*_tmpB7=" __declspec(";_tag_fat(_tmpB7,sizeof(char),13U);}));_tmpB6[0]=_tmp58F;}),({
struct Cyc_PP_Doc*_tmp58E=({struct _fat_ptr _tmp58D=({const char*_tmpB8="";_tag_fat(_tmpB8,sizeof(char),1U);});struct _fat_ptr _tmp58C=({const char*_tmpB9="";_tag_fat(_tmpB9,sizeof(char),1U);});struct _fat_ptr _tmp58B=({const char*_tmpBA=" ";_tag_fat(_tmpBA,sizeof(char),2U);});Cyc_PP_group(_tmp58D,_tmp58C,_tmp58B,({struct Cyc_List_List*(*_tmp58A)(struct Cyc_PP_Doc*(*f)(void*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmpBB)(struct Cyc_PP_Doc*(*f)(void*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_PP_Doc*(*f)(void*),struct Cyc_List_List*x))Cyc_List_map;_tmpBB;});_tmp58A(Cyc_Absynpp_noncallatt2doc,atts);}));});_tmpB6[1]=_tmp58E;}),({
struct Cyc_PP_Doc*_tmp589=Cyc_PP_text(({const char*_tmpBC=")";_tag_fat(_tmpBC,sizeof(char),2U);}));_tmpB6[2]=_tmp589;});Cyc_PP_cat(_tag_fat(_tmpB6,sizeof(struct Cyc_PP_Doc*),3U));});}
# 360
struct Cyc_PP_Doc*Cyc_Absynpp_att2doc(void*a){
# 511
return Cyc_PP_text(Cyc_Absyn_attribute2string(a));}
# 360
struct Cyc_PP_Doc*Cyc_Absynpp_atts2doc(struct Cyc_List_List*atts){
# 515
if(atts == 0)return Cyc_PP_nil_doc();{enum Cyc_Cyclone_C_Compilers _tmpBD=Cyc_Cyclone_c_compiler;if(_tmpBD == Cyc_Cyclone_Vc_c){_LL1: _LL2:
# 517
 return Cyc_Absynpp_noncallconv2doc(atts);}else{_LL3: _LL4:
# 519
 return({struct Cyc_PP_Doc*_tmpBE[2U];({struct Cyc_PP_Doc*_tmp595=Cyc_PP_text(({const char*_tmpBF=" __attribute__";_tag_fat(_tmpBF,sizeof(char),15U);}));_tmpBE[0]=_tmp595;}),({
struct Cyc_PP_Doc*_tmp594=({struct _fat_ptr _tmp593=({const char*_tmpC0="((";_tag_fat(_tmpC0,sizeof(char),3U);});struct _fat_ptr _tmp592=({const char*_tmpC1="))";_tag_fat(_tmpC1,sizeof(char),3U);});struct _fat_ptr _tmp591=({const char*_tmpC2=",";_tag_fat(_tmpC2,sizeof(char),2U);});Cyc_PP_group(_tmp593,_tmp592,_tmp591,({struct Cyc_List_List*(*_tmp590)(struct Cyc_PP_Doc*(*f)(void*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmpC3)(struct Cyc_PP_Doc*(*f)(void*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_PP_Doc*(*f)(void*),struct Cyc_List_List*x))Cyc_List_map;_tmpC3;});_tmp590(Cyc_Absynpp_att2doc,atts);}));});_tmpBE[1]=_tmp594;});Cyc_PP_cat(_tag_fat(_tmpBE,sizeof(struct Cyc_PP_Doc*),2U));});}_LL0:;}}
# 360
int Cyc_Absynpp_next_is_pointer(struct Cyc_List_List*tms){
# 525
if(tms == 0)return 0;{void*_stmttmp8=(void*)tms->hd;void*_tmpC4=_stmttmp8;switch(*((int*)_tmpC4)){case 2U: _LL1: _LL2:
# 527
 return 1;case 5U: _LL3: _LL4: {
# 529
enum Cyc_Cyclone_C_Compilers _tmpC5=Cyc_Cyclone_c_compiler;if(_tmpC5 == Cyc_Cyclone_Gcc_c){_LL8: _LL9:
 return 0;}else{_LLA: _LLB:
 return Cyc_Absynpp_next_is_pointer(tms->tl);}_LL7:;}default: _LL5: _LL6:
# 533
 return 0;}_LL0:;}}
# 360
struct Cyc_PP_Doc*Cyc_Absynpp_ntyp2doc(void*t);
# 540
static struct Cyc_PP_Doc*Cyc_Absynpp_cache_question=0;
static struct Cyc_PP_Doc*Cyc_Absynpp_question(){
if(!((unsigned)Cyc_Absynpp_cache_question)){
if(Cyc_PP_tex_output)
Cyc_Absynpp_cache_question=Cyc_PP_text_width(({const char*_tmpC6="{?}";_tag_fat(_tmpC6,sizeof(char),4U);}),1);else{
Cyc_Absynpp_cache_question=Cyc_PP_text(({const char*_tmpC7="?";_tag_fat(_tmpC7,sizeof(char),2U);}));}}
# 542
return(struct Cyc_PP_Doc*)_check_null(Cyc_Absynpp_cache_question);}
# 541
static struct Cyc_PP_Doc*Cyc_Absynpp_cache_lb=0;
# 550
static struct Cyc_PP_Doc*Cyc_Absynpp_lb(){
if(!((unsigned)Cyc_Absynpp_cache_lb)){
if(Cyc_PP_tex_output)
Cyc_Absynpp_cache_lb=Cyc_PP_text_width(({const char*_tmpC8="{\\lb}";_tag_fat(_tmpC8,sizeof(char),6U);}),1);else{
Cyc_Absynpp_cache_lb=Cyc_PP_text(({const char*_tmpC9="{";_tag_fat(_tmpC9,sizeof(char),2U);}));}}
# 551
return(struct Cyc_PP_Doc*)_check_null(Cyc_Absynpp_cache_lb);}
# 550
static struct Cyc_PP_Doc*Cyc_Absynpp_cache_rb=0;
# 559
static struct Cyc_PP_Doc*Cyc_Absynpp_rb(){
if(!((unsigned)Cyc_Absynpp_cache_rb)){
if(Cyc_PP_tex_output)
Cyc_Absynpp_cache_rb=Cyc_PP_text_width(({const char*_tmpCA="{\\rb}";_tag_fat(_tmpCA,sizeof(char),6U);}),1);else{
Cyc_Absynpp_cache_rb=Cyc_PP_text(({const char*_tmpCB="}";_tag_fat(_tmpCB,sizeof(char),2U);}));}}
# 560
return(struct Cyc_PP_Doc*)_check_null(Cyc_Absynpp_cache_rb);}
# 559
static struct Cyc_PP_Doc*Cyc_Absynpp_cache_dollar=0;
# 568
static struct Cyc_PP_Doc*Cyc_Absynpp_dollar(){
if(!((unsigned)Cyc_Absynpp_cache_dollar)){
if(Cyc_PP_tex_output)
Cyc_Absynpp_cache_dollar=Cyc_PP_text_width(({const char*_tmpCC="\\$";_tag_fat(_tmpCC,sizeof(char),3U);}),1);else{
Cyc_Absynpp_cache_dollar=Cyc_PP_text(({const char*_tmpCD="$";_tag_fat(_tmpCD,sizeof(char),2U);}));}}
# 569
return(struct Cyc_PP_Doc*)_check_null(Cyc_Absynpp_cache_dollar);}
# 568
struct Cyc_PP_Doc*Cyc_Absynpp_group_braces(struct _fat_ptr sep,struct Cyc_List_List*ss){
# 577
return({struct Cyc_PP_Doc*_tmpCE[3U];({struct Cyc_PP_Doc*_tmp598=Cyc_Absynpp_lb();_tmpCE[0]=_tmp598;}),({struct Cyc_PP_Doc*_tmp597=Cyc_PP_seq(sep,ss);_tmpCE[1]=_tmp597;}),({struct Cyc_PP_Doc*_tmp596=Cyc_Absynpp_rb();_tmpCE[2]=_tmp596;});Cyc_PP_cat(_tag_fat(_tmpCE,sizeof(struct Cyc_PP_Doc*),3U));});}
# 568
static void Cyc_Absynpp_print_tms(struct Cyc_List_List*tms){
# 582
for(0;tms != 0;tms=tms->tl){
void*_stmttmp9=(void*)tms->hd;void*_tmpCF=_stmttmp9;struct Cyc_List_List*_tmpD0;switch(*((int*)_tmpCF)){case 0U: _LL1: _LL2:
({void*_tmpD1=0U;({struct Cyc___cycFILE*_tmp59A=Cyc_stderr;struct _fat_ptr _tmp599=({const char*_tmpD2="Carray_mod ";_tag_fat(_tmpD2,sizeof(char),12U);});Cyc_fprintf(_tmp59A,_tmp599,_tag_fat(_tmpD1,sizeof(void*),0U));});});goto _LL0;case 1U: _LL3: _LL4:
({void*_tmpD3=0U;({struct Cyc___cycFILE*_tmp59C=Cyc_stderr;struct _fat_ptr _tmp59B=({const char*_tmpD4="ConstArray_mod ";_tag_fat(_tmpD4,sizeof(char),16U);});Cyc_fprintf(_tmp59C,_tmp59B,_tag_fat(_tmpD3,sizeof(void*),0U));});});goto _LL0;case 3U: if(((struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct*)((struct Cyc_Absyn_Function_mod_Absyn_Type_modifier_struct*)_tmpCF)->f1)->tag == 1U){_LL5: _tmpD0=((struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct*)((struct Cyc_Absyn_Function_mod_Absyn_Type_modifier_struct*)_tmpCF)->f1)->f1;_LL6: {struct Cyc_List_List*ys=_tmpD0;
# 587
({void*_tmpD5=0U;({struct Cyc___cycFILE*_tmp59E=Cyc_stderr;struct _fat_ptr _tmp59D=({const char*_tmpD6="Function_mod(";_tag_fat(_tmpD6,sizeof(char),14U);});Cyc_fprintf(_tmp59E,_tmp59D,_tag_fat(_tmpD5,sizeof(void*),0U));});});
for(0;ys != 0;ys=ys->tl){
struct _fat_ptr*v=(*((struct _tuple8*)ys->hd)).f1;
if(v == 0)({void*_tmpD7=0U;({struct Cyc___cycFILE*_tmp5A0=Cyc_stderr;struct _fat_ptr _tmp59F=({const char*_tmpD8="?";_tag_fat(_tmpD8,sizeof(char),2U);});Cyc_fprintf(_tmp5A0,_tmp59F,_tag_fat(_tmpD7,sizeof(void*),0U));});});else{
({void*_tmpD9=0U;({struct Cyc___cycFILE*_tmp5A2=Cyc_stderr;struct _fat_ptr _tmp5A1=*v;Cyc_fprintf(_tmp5A2,_tmp5A1,_tag_fat(_tmpD9,sizeof(void*),0U));});});}
if(ys->tl != 0)({void*_tmpDA=0U;({struct Cyc___cycFILE*_tmp5A4=Cyc_stderr;struct _fat_ptr _tmp5A3=({const char*_tmpDB=",";_tag_fat(_tmpDB,sizeof(char),2U);});Cyc_fprintf(_tmp5A4,_tmp5A3,_tag_fat(_tmpDA,sizeof(void*),0U));});});}
# 594
({void*_tmpDC=0U;({struct Cyc___cycFILE*_tmp5A6=Cyc_stderr;struct _fat_ptr _tmp5A5=({const char*_tmpDD=") ";_tag_fat(_tmpDD,sizeof(char),3U);});Cyc_fprintf(_tmp5A6,_tmp5A5,_tag_fat(_tmpDC,sizeof(void*),0U));});});
goto _LL0;}}else{_LL7: _LL8:
# 597
({void*_tmpDE=0U;({struct Cyc___cycFILE*_tmp5A8=Cyc_stderr;struct _fat_ptr _tmp5A7=({const char*_tmpDF="Function_mod()";_tag_fat(_tmpDF,sizeof(char),15U);});Cyc_fprintf(_tmp5A8,_tmp5A7,_tag_fat(_tmpDE,sizeof(void*),0U));});});goto _LL0;}case 5U: _LL9: _LLA:
({void*_tmpE0=0U;({struct Cyc___cycFILE*_tmp5AA=Cyc_stderr;struct _fat_ptr _tmp5A9=({const char*_tmpE1="Attributes_mod ";_tag_fat(_tmpE1,sizeof(char),16U);});Cyc_fprintf(_tmp5AA,_tmp5A9,_tag_fat(_tmpE0,sizeof(void*),0U));});});goto _LL0;case 4U: _LLB: _LLC:
({void*_tmpE2=0U;({struct Cyc___cycFILE*_tmp5AC=Cyc_stderr;struct _fat_ptr _tmp5AB=({const char*_tmpE3="TypeParams_mod ";_tag_fat(_tmpE3,sizeof(char),16U);});Cyc_fprintf(_tmp5AC,_tmp5AB,_tag_fat(_tmpE2,sizeof(void*),0U));});});goto _LL0;default: _LLD: _LLE:
({void*_tmpE4=0U;({struct Cyc___cycFILE*_tmp5AE=Cyc_stderr;struct _fat_ptr _tmp5AD=({const char*_tmpE5="Pointer_mod ";_tag_fat(_tmpE5,sizeof(char),13U);});Cyc_fprintf(_tmp5AE,_tmp5AD,_tag_fat(_tmpE4,sizeof(void*),0U));});});goto _LL0;}_LL0:;}
# 602
({void*_tmpE6=0U;({struct Cyc___cycFILE*_tmp5B0=Cyc_stderr;struct _fat_ptr _tmp5AF=({const char*_tmpE7="\n";_tag_fat(_tmpE7,sizeof(char),2U);});Cyc_fprintf(_tmp5B0,_tmp5AF,_tag_fat(_tmpE6,sizeof(void*),0U));});});}
# 568
struct Cyc_PP_Doc*Cyc_Absynpp_rgn2doc(void*t);
# 607
struct Cyc_PP_Doc*Cyc_Absynpp_dtms2doc(int is_char_ptr,struct Cyc_PP_Doc*d,struct Cyc_List_List*tms){
if(tms == 0)return d;{struct Cyc_PP_Doc*rest=
Cyc_Absynpp_dtms2doc(0,d,tms->tl);
struct Cyc_PP_Doc*p_rest=({struct Cyc_PP_Doc*_tmp129[3U];({struct Cyc_PP_Doc*_tmp5B2=Cyc_PP_text(({const char*_tmp12A="(";_tag_fat(_tmp12A,sizeof(char),2U);}));_tmp129[0]=_tmp5B2;}),_tmp129[1]=rest,({struct Cyc_PP_Doc*_tmp5B1=Cyc_PP_text(({const char*_tmp12B=")";_tag_fat(_tmp12B,sizeof(char),2U);}));_tmp129[2]=_tmp5B1;});Cyc_PP_cat(_tag_fat(_tmp129,sizeof(struct Cyc_PP_Doc*),3U));});
void*_stmttmpA=(void*)tms->hd;void*_tmpE8=_stmttmpA;struct Cyc_Absyn_Tqual _tmpED;void*_tmpEC;void*_tmpEB;void*_tmpEA;void*_tmpE9;int _tmpF0;unsigned _tmpEF;struct Cyc_List_List*_tmpEE;struct Cyc_List_List*_tmpF1;void*_tmpF2;void*_tmpF4;struct Cyc_Absyn_Exp*_tmpF3;void*_tmpF5;switch(*((int*)_tmpE8)){case 0U: _LL1: _tmpF5=(void*)((struct Cyc_Absyn_Carray_mod_Absyn_Type_modifier_struct*)_tmpE8)->f1;_LL2: {void*zeroterm=_tmpF5;
# 613
if(Cyc_Absynpp_next_is_pointer(tms->tl))rest=p_rest;return({struct Cyc_PP_Doc*_tmpF6[2U];_tmpF6[0]=rest,
Cyc_Absyn_type2bool(0,zeroterm)?({
struct Cyc_PP_Doc*_tmp5B4=Cyc_PP_text(({const char*_tmpF7="[]@zeroterm";_tag_fat(_tmpF7,sizeof(char),12U);}));_tmpF6[1]=_tmp5B4;}):({struct Cyc_PP_Doc*_tmp5B3=Cyc_PP_text(({const char*_tmpF8="[]";_tag_fat(_tmpF8,sizeof(char),3U);}));_tmpF6[1]=_tmp5B3;});Cyc_PP_cat(_tag_fat(_tmpF6,sizeof(struct Cyc_PP_Doc*),2U));});}case 1U: _LL3: _tmpF3=((struct Cyc_Absyn_ConstArray_mod_Absyn_Type_modifier_struct*)_tmpE8)->f1;_tmpF4=(void*)((struct Cyc_Absyn_ConstArray_mod_Absyn_Type_modifier_struct*)_tmpE8)->f2;_LL4: {struct Cyc_Absyn_Exp*e=_tmpF3;void*zeroterm=_tmpF4;
# 617
if(Cyc_Absynpp_next_is_pointer(tms->tl))rest=p_rest;return({struct Cyc_PP_Doc*_tmpF9[4U];_tmpF9[0]=rest,({
struct Cyc_PP_Doc*_tmp5B8=Cyc_PP_text(({const char*_tmpFA="[";_tag_fat(_tmpFA,sizeof(char),2U);}));_tmpF9[1]=_tmp5B8;}),({struct Cyc_PP_Doc*_tmp5B7=Cyc_Absynpp_exp2doc(e);_tmpF9[2]=_tmp5B7;}),
Cyc_Absyn_type2bool(0,zeroterm)?({struct Cyc_PP_Doc*_tmp5B6=Cyc_PP_text(({const char*_tmpFB="]@zeroterm";_tag_fat(_tmpFB,sizeof(char),11U);}));_tmpF9[3]=_tmp5B6;}):({struct Cyc_PP_Doc*_tmp5B5=Cyc_PP_text(({const char*_tmpFC="]";_tag_fat(_tmpFC,sizeof(char),2U);}));_tmpF9[3]=_tmp5B5;});Cyc_PP_cat(_tag_fat(_tmpF9,sizeof(struct Cyc_PP_Doc*),4U));});}case 3U: _LL5: _tmpF2=(void*)((struct Cyc_Absyn_Function_mod_Absyn_Type_modifier_struct*)_tmpE8)->f1;_LL6: {void*args=_tmpF2;
# 621
if(Cyc_Absynpp_next_is_pointer(tms->tl))rest=p_rest;{void*_tmpFD=args;unsigned _tmpFF;struct Cyc_List_List*_tmpFE;struct Cyc_List_List*_tmp108;struct Cyc_List_List*_tmp107;struct Cyc_Absyn_Exp*_tmp106;struct Cyc_Absyn_Exp*_tmp105;struct Cyc_List_List*_tmp104;void*_tmp103;struct Cyc_Absyn_VarargInfo*_tmp102;int _tmp101;struct Cyc_List_List*_tmp100;if(((struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct*)_tmpFD)->tag == 1U){_LLE: _tmp100=((struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct*)_tmpFD)->f1;_tmp101=((struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct*)_tmpFD)->f2;_tmp102=((struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct*)_tmpFD)->f3;_tmp103=(void*)((struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct*)_tmpFD)->f4;_tmp104=((struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct*)_tmpFD)->f5;_tmp105=((struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct*)_tmpFD)->f6;_tmp106=((struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct*)_tmpFD)->f7;_tmp107=((struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct*)_tmpFD)->f8;_tmp108=((struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct*)_tmpFD)->f9;_LLF: {struct Cyc_List_List*args2=_tmp100;int c_varargs=_tmp101;struct Cyc_Absyn_VarargInfo*cyc_varargs=_tmp102;void*effopt=_tmp103;struct Cyc_List_List*rgn_po=_tmp104;struct Cyc_Absyn_Exp*req=_tmp105;struct Cyc_Absyn_Exp*ens=_tmp106;struct Cyc_List_List*ieff=_tmp107;struct Cyc_List_List*oeff=_tmp108;
# 624
return({struct Cyc_PP_Doc*_tmp109[2U];_tmp109[0]=rest,({struct Cyc_PP_Doc*_tmp5B9=Cyc_Absynpp_funargs2doc(args2,c_varargs,cyc_varargs,effopt,rgn_po,req,ens,ieff,oeff);_tmp109[1]=_tmp5B9;});Cyc_PP_cat(_tag_fat(_tmp109,sizeof(struct Cyc_PP_Doc*),2U));});}}else{_LL10: _tmpFE=((struct Cyc_Absyn_NoTypes_Absyn_Funcparams_struct*)_tmpFD)->f1;_tmpFF=((struct Cyc_Absyn_NoTypes_Absyn_Funcparams_struct*)_tmpFD)->f2;_LL11: {struct Cyc_List_List*sl=_tmpFE;unsigned loc=_tmpFF;
# 626
return({struct Cyc_PP_Doc*_tmp10A[2U];_tmp10A[0]=rest,({struct Cyc_PP_Doc*_tmp5BE=({struct _fat_ptr _tmp5BD=({const char*_tmp10B="(";_tag_fat(_tmp10B,sizeof(char),2U);});struct _fat_ptr _tmp5BC=({const char*_tmp10C=")";_tag_fat(_tmp10C,sizeof(char),2U);});struct _fat_ptr _tmp5BB=({const char*_tmp10D=",";_tag_fat(_tmp10D,sizeof(char),2U);});Cyc_PP_group(_tmp5BD,_tmp5BC,_tmp5BB,({struct Cyc_List_List*(*_tmp5BA)(struct Cyc_PP_Doc*(*f)(struct _fat_ptr*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp10E)(struct Cyc_PP_Doc*(*f)(struct _fat_ptr*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_PP_Doc*(*f)(struct _fat_ptr*),struct Cyc_List_List*x))Cyc_List_map;_tmp10E;});_tmp5BA(Cyc_PP_textptr,sl);}));});_tmp10A[1]=_tmp5BE;});Cyc_PP_cat(_tag_fat(_tmp10A,sizeof(struct Cyc_PP_Doc*),2U));});}}_LLD:;}}case 5U: _LL7: _tmpF1=((struct Cyc_Absyn_Attributes_mod_Absyn_Type_modifier_struct*)_tmpE8)->f2;_LL8: {struct Cyc_List_List*atts=_tmpF1;
# 629
enum Cyc_Cyclone_C_Compilers _tmp10F=Cyc_Cyclone_c_compiler;if(_tmp10F == Cyc_Cyclone_Gcc_c){_LL13: _LL14:
# 631
 if(Cyc_Absynpp_next_is_pointer(tms->tl))
rest=p_rest;
# 631
return({struct Cyc_PP_Doc*_tmp110[2U];_tmp110[0]=rest,({
# 633
struct Cyc_PP_Doc*_tmp5BF=Cyc_Absynpp_atts2doc(atts);_tmp110[1]=_tmp5BF;});Cyc_PP_cat(_tag_fat(_tmp110,sizeof(struct Cyc_PP_Doc*),2U));});}else{_LL15: _LL16:
# 635
 if(Cyc_Absynpp_next_is_pointer(tms->tl))
return({struct Cyc_PP_Doc*_tmp111[2U];({struct Cyc_PP_Doc*_tmp5C0=Cyc_Absynpp_callconv2doc(atts);_tmp111[0]=_tmp5C0;}),_tmp111[1]=rest;Cyc_PP_cat(_tag_fat(_tmp111,sizeof(struct Cyc_PP_Doc*),2U));});
# 635
return rest;}_LL12:;}case 4U: _LL9: _tmpEE=((struct Cyc_Absyn_TypeParams_mod_Absyn_Type_modifier_struct*)_tmpE8)->f1;_tmpEF=((struct Cyc_Absyn_TypeParams_mod_Absyn_Type_modifier_struct*)_tmpE8)->f2;_tmpF0=((struct Cyc_Absyn_TypeParams_mod_Absyn_Type_modifier_struct*)_tmpE8)->f3;_LLA: {struct Cyc_List_List*ts=_tmpEE;unsigned loc=_tmpEF;int print_kinds=_tmpF0;
# 640
if(Cyc_Absynpp_next_is_pointer(tms->tl))rest=p_rest;if(print_kinds)
# 642
return({struct Cyc_PP_Doc*_tmp112[2U];_tmp112[0]=rest,({struct Cyc_PP_Doc*_tmp5C1=Cyc_Absynpp_ktvars2doc(ts);_tmp112[1]=_tmp5C1;});Cyc_PP_cat(_tag_fat(_tmp112,sizeof(struct Cyc_PP_Doc*),2U));});else{
# 644
return({struct Cyc_PP_Doc*_tmp113[2U];_tmp113[0]=rest,({struct Cyc_PP_Doc*_tmp5C2=Cyc_Absynpp_tvars2doc(ts);_tmp113[1]=_tmp5C2;});Cyc_PP_cat(_tag_fat(_tmp113,sizeof(struct Cyc_PP_Doc*),2U));});}}default: _LLB: _tmpE9=(((struct Cyc_Absyn_Pointer_mod_Absyn_Type_modifier_struct*)_tmpE8)->f1).rgn;_tmpEA=(((struct Cyc_Absyn_Pointer_mod_Absyn_Type_modifier_struct*)_tmpE8)->f1).nullable;_tmpEB=(((struct Cyc_Absyn_Pointer_mod_Absyn_Type_modifier_struct*)_tmpE8)->f1).bounds;_tmpEC=(((struct Cyc_Absyn_Pointer_mod_Absyn_Type_modifier_struct*)_tmpE8)->f1).zero_term;_tmpED=((struct Cyc_Absyn_Pointer_mod_Absyn_Type_modifier_struct*)_tmpE8)->f2;_LLC: {void*rgn=_tmpE9;void*nullable=_tmpEA;void*bd=_tmpEB;void*zt=_tmpEC;struct Cyc_Absyn_Tqual tq2=_tmpED;
# 648
struct Cyc_PP_Doc*ptr;
struct Cyc_PP_Doc*mt=Cyc_PP_nil_doc();
struct Cyc_PP_Doc*ztd=mt;
struct Cyc_PP_Doc*rgd=mt;
struct Cyc_PP_Doc*tqd=Cyc_Absynpp_tqual2doc(tq2);
{void*_stmttmpB=Cyc_Tcutil_compress(bd);void*_tmp114=_stmttmpB;void*_tmp115;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp114)->tag == 0U)switch(*((int*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp114)->f1)){case 14U: _LL18: _LL19:
 ptr=Cyc_Absynpp_question();goto _LL17;case 13U: if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp114)->f2 != 0){_LL1A: _tmp115=(void*)(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp114)->f2)->hd;_LL1B: {void*targ=_tmp115;
# 656
{void*_stmttmpC=Cyc_Tcutil_compress(targ);void*_tmp116=_stmttmpC;struct Cyc_Absyn_Exp*_tmp117;if(((struct Cyc_Absyn_ValueofType_Absyn_Type_struct*)_tmp116)->tag == 9U){_LL1F: _tmp117=((struct Cyc_Absyn_ValueofType_Absyn_Type_struct*)_tmp116)->f1;_LL20: {struct Cyc_Absyn_Exp*e=_tmp117;
# 658
ptr=Cyc_PP_text(Cyc_Absyn_type2bool(1,nullable)?({const char*_tmp118="*";_tag_fat(_tmp118,sizeof(char),2U);}):({const char*_tmp119="@";_tag_fat(_tmp119,sizeof(char),2U);}));{
struct _tuple12 _stmttmpD=Cyc_Evexp_eval_const_uint_exp(e);int _tmp11B;unsigned _tmp11A;_LL24: _tmp11A=_stmttmpD.f1;_tmp11B=_stmttmpD.f2;_LL25: {unsigned val=_tmp11A;int known=_tmp11B;
if(!known || val != (unsigned)1)
ptr=({struct Cyc_PP_Doc*_tmp11C[4U];_tmp11C[0]=ptr,({struct Cyc_PP_Doc*_tmp5C5=Cyc_Absynpp_lb();_tmp11C[1]=_tmp5C5;}),({struct Cyc_PP_Doc*_tmp5C4=Cyc_Absynpp_exp2doc(e);_tmp11C[2]=_tmp5C4;}),({struct Cyc_PP_Doc*_tmp5C3=Cyc_Absynpp_rb();_tmp11C[3]=_tmp5C3;});Cyc_PP_cat(_tag_fat(_tmp11C,sizeof(struct Cyc_PP_Doc*),4U));});
# 660
goto _LL1E;}}}}else{_LL21: _LL22:
# 664
 ptr=Cyc_PP_text(Cyc_Absyn_type2bool(1,nullable)?({const char*_tmp11D="*";_tag_fat(_tmp11D,sizeof(char),2U);}):({const char*_tmp11E="@";_tag_fat(_tmp11E,sizeof(char),2U);}));
ptr=({struct Cyc_PP_Doc*_tmp11F[4U];_tmp11F[0]=ptr,({struct Cyc_PP_Doc*_tmp5C8=Cyc_Absynpp_lb();_tmp11F[1]=_tmp5C8;}),({struct Cyc_PP_Doc*_tmp5C7=Cyc_Absynpp_typ2doc(targ);_tmp11F[2]=_tmp5C7;}),({struct Cyc_PP_Doc*_tmp5C6=Cyc_Absynpp_rb();_tmp11F[3]=_tmp5C6;});Cyc_PP_cat(_tag_fat(_tmp11F,sizeof(struct Cyc_PP_Doc*),4U));});
goto _LL1E;}_LL1E:;}
# 668
goto _LL17;}}else{goto _LL1C;}default: goto _LL1C;}else{_LL1C: _LL1D:
# 670
 ptr=Cyc_PP_text(Cyc_Absyn_type2bool(1,nullable)?({const char*_tmp120="*";_tag_fat(_tmp120,sizeof(char),2U);}):({const char*_tmp121="@";_tag_fat(_tmp121,sizeof(char),2U);}));
ptr=({struct Cyc_PP_Doc*_tmp122[4U];_tmp122[0]=ptr,({struct Cyc_PP_Doc*_tmp5CB=Cyc_Absynpp_lb();_tmp122[1]=_tmp5CB;}),({struct Cyc_PP_Doc*_tmp5CA=Cyc_Absynpp_typ2doc(bd);_tmp122[2]=_tmp5CA;}),({struct Cyc_PP_Doc*_tmp5C9=Cyc_Absynpp_rb();_tmp122[3]=_tmp5C9;});Cyc_PP_cat(_tag_fat(_tmp122,sizeof(struct Cyc_PP_Doc*),4U));});
goto _LL17;}_LL17:;}
# 675
if(Cyc_Absynpp_print_zeroterm){
if(!is_char_ptr && Cyc_Absyn_type2bool(0,zt))
ztd=Cyc_PP_text(({const char*_tmp123="@zeroterm";_tag_fat(_tmp123,sizeof(char),10U);}));else{
if(is_char_ptr && !Cyc_Absyn_type2bool(0,zt))
ztd=Cyc_PP_text(({const char*_tmp124="@nozeroterm";_tag_fat(_tmp124,sizeof(char),12U);}));}}
# 675
{void*_stmttmpE=
# 681
Cyc_Tcutil_compress(rgn);
# 675
void*_tmp125=_stmttmpE;switch(*((int*)_tmp125)){case 0U: if(((struct Cyc_Absyn_HeapCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp125)->f1)->tag == 5U){_LL27: _LL28:
# 682
 goto _LL26;}else{goto _LL2B;}case 1U: _LL29: if(Cyc_Absynpp_print_for_cycdoc){_LL2A:
 goto _LL26;}else{goto _LL2B;}default: _LL2B: _LL2C:
 rgd=Cyc_Absynpp_rgn2doc(rgn);}_LL26:;}{
# 686
struct Cyc_PP_Doc*spacer1=tqd != mt &&(ztd != mt || rgd != mt)?Cyc_PP_text(({const char*_tmp128=" ";_tag_fat(_tmp128,sizeof(char),2U);})): mt;
struct Cyc_PP_Doc*spacer2=rest != mt?Cyc_PP_text(({const char*_tmp127=" ";_tag_fat(_tmp127,sizeof(char),2U);})): mt;
return({struct Cyc_PP_Doc*_tmp126[7U];_tmp126[0]=ptr,_tmp126[1]=ztd,_tmp126[2]=rgd,_tmp126[3]=spacer1,_tmp126[4]=tqd,_tmp126[5]=spacer2,_tmp126[6]=rest;Cyc_PP_cat(_tag_fat(_tmp126,sizeof(struct Cyc_PP_Doc*),7U));});}}}_LL0:;}}
# 607
struct Cyc_PP_Doc*Cyc_Absynpp_rgn2doc(void*t){
# 693
void*_stmttmpF=Cyc_Tcutil_compress(t);void*_tmp12C=_stmttmpF;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp12C)->tag == 0U)switch(*((int*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp12C)->f1)){case 5U: _LL1: _LL2:
 return Cyc_PP_text(({const char*_tmp12D="`H";_tag_fat(_tmp12D,sizeof(char),3U);}));case 6U: _LL3: _LL4:
 return Cyc_PP_text(({const char*_tmp12E="`U";_tag_fat(_tmp12E,sizeof(char),3U);}));case 7U: _LL5: _LL6:
 return Cyc_PP_text(({const char*_tmp12F="`RC";_tag_fat(_tmp12F,sizeof(char),4U);}));default: goto _LL7;}else{_LL7: _LL8:
 return Cyc_Absynpp_ntyp2doc(t);}_LL0:;}
# 607
static void Cyc_Absynpp_effects2docs(struct Cyc_List_List**rgions,struct Cyc_List_List**effects,void*t){
# 706
void*_stmttmp10=Cyc_Tcutil_compress(t);void*_tmp130=_stmttmp10;struct Cyc_List_List*_tmp131;void*_tmp132;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp130)->tag == 0U)switch(*((int*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp130)->f1)){case 8U: if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp130)->f2 != 0){_LL1: _tmp132=(void*)(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp130)->f2)->hd;_LL2: {void*r=_tmp132;
({struct Cyc_List_List*_tmp5CD=({struct Cyc_List_List*_tmp133=_cycalloc(sizeof(*_tmp133));({struct Cyc_PP_Doc*_tmp5CC=Cyc_Absynpp_rgn2doc(r);_tmp133->hd=_tmp5CC;}),_tmp133->tl=*rgions;_tmp133;});*rgions=_tmp5CD;});goto _LL0;}}else{goto _LL5;}case 9U: _LL3: _tmp131=((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp130)->f2;_LL4: {struct Cyc_List_List*ts=_tmp131;
# 709
for(0;ts != 0;ts=ts->tl){
Cyc_Absynpp_effects2docs(rgions,effects,(void*)ts->hd);}
# 712
goto _LL0;}default: goto _LL5;}else{_LL5: _LL6:
({struct Cyc_List_List*_tmp5CF=({struct Cyc_List_List*_tmp134=_cycalloc(sizeof(*_tmp134));({struct Cyc_PP_Doc*_tmp5CE=Cyc_Absynpp_typ2doc(t);_tmp134->hd=_tmp5CE;}),_tmp134->tl=*effects;_tmp134;});*effects=_tmp5CF;});goto _LL0;}_LL0:;}
# 607
struct Cyc_PP_Doc*Cyc_Absynpp_eff2doc(void*t){
# 718
struct Cyc_List_List*rgions=0;struct Cyc_List_List*effects=0;
Cyc_Absynpp_effects2docs(& rgions,& effects,t);
rgions=Cyc_List_imp_rev(rgions);
effects=Cyc_List_imp_rev(effects);
if(rgions == 0 && effects != 0)
return({struct _fat_ptr _tmp5D2=({const char*_tmp135="";_tag_fat(_tmp135,sizeof(char),1U);});struct _fat_ptr _tmp5D1=({const char*_tmp136="";_tag_fat(_tmp136,sizeof(char),1U);});struct _fat_ptr _tmp5D0=({const char*_tmp137="+";_tag_fat(_tmp137,sizeof(char),2U);});Cyc_PP_group(_tmp5D2,_tmp5D1,_tmp5D0,effects);});else{
# 725
struct Cyc_PP_Doc*doc1=({struct _fat_ptr _tmp5D3=({const char*_tmp13C=",";_tag_fat(_tmp13C,sizeof(char),2U);});Cyc_Absynpp_group_braces(_tmp5D3,rgions);});
return({struct _fat_ptr _tmp5D7=({const char*_tmp138="";_tag_fat(_tmp138,sizeof(char),1U);});struct _fat_ptr _tmp5D6=({const char*_tmp139="";_tag_fat(_tmp139,sizeof(char),1U);});struct _fat_ptr _tmp5D5=({const char*_tmp13A="+";_tag_fat(_tmp13A,sizeof(char),2U);});Cyc_PP_group(_tmp5D7,_tmp5D6,_tmp5D5,({struct Cyc_List_List*_tmp5D4=effects;Cyc_List_imp_append(_tmp5D4,({struct Cyc_List_List*_tmp13B=_cycalloc(sizeof(*_tmp13B));_tmp13B->hd=doc1,_tmp13B->tl=0;_tmp13B;}));}));});}}
# 607
struct Cyc_PP_Doc*Cyc_Absynpp_aggr_kind2doc(enum Cyc_Absyn_AggrKind k){
# 731
enum Cyc_Absyn_AggrKind _tmp13D=k;if(_tmp13D == Cyc_Absyn_StructA){_LL1: _LL2:
 return Cyc_PP_text(({const char*_tmp13E="struct ";_tag_fat(_tmp13E,sizeof(char),8U);}));}else{_LL3: _LL4:
 return Cyc_PP_text(({const char*_tmp13F="union ";_tag_fat(_tmp13F,sizeof(char),7U);}));}_LL0:;}
# 607
struct Cyc_PP_Doc*Cyc_Absynpp_ntyp2doc(void*t){
# 739
struct Cyc_PP_Doc*s;
{void*_tmp140=t;struct Cyc_Absyn_Datatypedecl*_tmp141;struct Cyc_Absyn_Enumdecl*_tmp142;struct Cyc_Absyn_Aggrdecl*_tmp143;struct Cyc_Absyn_Typedefdecl*_tmp146;struct Cyc_List_List*_tmp145;struct _tuple0*_tmp144;struct Cyc_Absyn_Exp*_tmp147;struct Cyc_Absyn_Exp*_tmp148;struct Cyc_List_List*_tmp14A;enum Cyc_Absyn_AggrKind _tmp149;struct Cyc_List_List*_tmp14B;struct Cyc_Absyn_Tvar*_tmp14C;struct Cyc_Core_Opt*_tmp150;int _tmp14F;void*_tmp14E;struct Cyc_Core_Opt*_tmp14D;void*_tmp151;struct Cyc_List_List*_tmp152;struct Cyc_List_List*_tmp153;struct Cyc_List_List*_tmp154;struct _fat_ptr _tmp155;struct _tuple0*_tmp156;struct Cyc_List_List*_tmp157;struct Cyc_List_List*_tmp159;union Cyc_Absyn_AggrInfo _tmp158;int _tmp15A;enum Cyc_Absyn_Size_of _tmp15C;enum Cyc_Absyn_Sign _tmp15B;struct Cyc_List_List*_tmp15E;union Cyc_Absyn_DatatypeFieldInfo _tmp15D;struct Cyc_List_List*_tmp160;union Cyc_Absyn_DatatypeInfo _tmp15F;switch(*((int*)_tmp140)){case 4U: _LL1: _LL2:
# 742
 return Cyc_PP_text(({const char*_tmp161="[[[array]]]";_tag_fat(_tmp161,sizeof(char),12U);}));case 5U: _LL3: _LL4:
 return Cyc_PP_nil_doc();case 3U: _LL5: _LL6:
 return Cyc_PP_nil_doc();case 0U: switch(*((int*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp140)->f1)){case 0U: _LL7: _LL8:
# 746
 s=Cyc_PP_text(({const char*_tmp162="void";_tag_fat(_tmp162,sizeof(char),5U);}));goto _LL0;case 18U: _LLD: _tmp15F=((struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp140)->f1)->f1;_tmp160=((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp140)->f2;_LLE: {union Cyc_Absyn_DatatypeInfo tu_info=_tmp15F;struct Cyc_List_List*ts=_tmp160;
# 762
{union Cyc_Absyn_DatatypeInfo _tmp16A=tu_info;int _tmp16C;struct _tuple0*_tmp16B;int _tmp16E;struct _tuple0*_tmp16D;if((_tmp16A.UnknownDatatype).tag == 1){_LL48: _tmp16D=((_tmp16A.UnknownDatatype).val).name;_tmp16E=((_tmp16A.UnknownDatatype).val).is_extensible;_LL49: {struct _tuple0*n=_tmp16D;int is_x=_tmp16E;
_tmp16B=n;_tmp16C=is_x;goto _LL4B;}}else{_LL4A: _tmp16B=(*(_tmp16A.KnownDatatype).val)->name;_tmp16C=(*(_tmp16A.KnownDatatype).val)->is_extensible;_LL4B: {struct _tuple0*n=_tmp16B;int is_x=_tmp16C;
# 765
struct Cyc_PP_Doc*kw=Cyc_PP_text(({const char*_tmp171="datatype ";_tag_fat(_tmp171,sizeof(char),10U);}));
struct Cyc_PP_Doc*ext=is_x?Cyc_PP_text(({const char*_tmp170="@extensible ";_tag_fat(_tmp170,sizeof(char),13U);})): Cyc_PP_nil_doc();
s=({struct Cyc_PP_Doc*_tmp16F[4U];_tmp16F[0]=ext,_tmp16F[1]=kw,({struct Cyc_PP_Doc*_tmp5D9=Cyc_Absynpp_qvar2doc(n);_tmp16F[2]=_tmp5D9;}),({struct Cyc_PP_Doc*_tmp5D8=Cyc_Absynpp_tps2doc(ts);_tmp16F[3]=_tmp5D8;});Cyc_PP_cat(_tag_fat(_tmp16F,sizeof(struct Cyc_PP_Doc*),4U));});
goto _LL47;}}_LL47:;}
# 770
goto _LL0;}case 19U: _LLF: _tmp15D=((struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp140)->f1)->f1;_tmp15E=((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp140)->f2;_LL10: {union Cyc_Absyn_DatatypeFieldInfo tuf_info=_tmp15D;struct Cyc_List_List*ts=_tmp15E;
# 772
{union Cyc_Absyn_DatatypeFieldInfo _tmp172=tuf_info;struct _tuple0*_tmp175;int _tmp174;struct _tuple0*_tmp173;int _tmp178;struct _tuple0*_tmp177;struct _tuple0*_tmp176;if((_tmp172.UnknownDatatypefield).tag == 1){_LL4D: _tmp176=((_tmp172.UnknownDatatypefield).val).datatype_name;_tmp177=((_tmp172.UnknownDatatypefield).val).field_name;_tmp178=((_tmp172.UnknownDatatypefield).val).is_extensible;_LL4E: {struct _tuple0*tname=_tmp176;struct _tuple0*fname=_tmp177;int is_x=_tmp178;
# 774
_tmp173=tname;_tmp174=is_x;_tmp175=fname;goto _LL50;}}else{_LL4F: _tmp173=(((_tmp172.KnownDatatypefield).val).f1)->name;_tmp174=(((_tmp172.KnownDatatypefield).val).f1)->is_extensible;_tmp175=(((_tmp172.KnownDatatypefield).val).f2)->name;_LL50: {struct _tuple0*tname=_tmp173;int is_x=_tmp174;struct _tuple0*fname=_tmp175;
# 777
struct Cyc_PP_Doc*kw=Cyc_PP_text(is_x?({const char*_tmp17B="@extensible datatype ";_tag_fat(_tmp17B,sizeof(char),22U);}):({const char*_tmp17C="datatype ";_tag_fat(_tmp17C,sizeof(char),10U);}));
s=({struct Cyc_PP_Doc*_tmp179[4U];_tmp179[0]=kw,({struct Cyc_PP_Doc*_tmp5DC=Cyc_Absynpp_qvar2doc(tname);_tmp179[1]=_tmp5DC;}),({struct Cyc_PP_Doc*_tmp5DB=Cyc_PP_text(({const char*_tmp17A=".";_tag_fat(_tmp17A,sizeof(char),2U);}));_tmp179[2]=_tmp5DB;}),({struct Cyc_PP_Doc*_tmp5DA=Cyc_Absynpp_qvar2doc(fname);_tmp179[3]=_tmp5DA;});Cyc_PP_cat(_tag_fat(_tmp179,sizeof(struct Cyc_PP_Doc*),4U));});
goto _LL4C;}}_LL4C:;}
# 781
goto _LL0;}case 1U: _LL11: _tmp15B=((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp140)->f1)->f1;_tmp15C=((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp140)->f1)->f2;_LL12: {enum Cyc_Absyn_Sign sn=_tmp15B;enum Cyc_Absyn_Size_of sz=_tmp15C;
# 783
struct _fat_ptr sns;
struct _fat_ptr ts;
{enum Cyc_Absyn_Sign _tmp17D=sn;switch(_tmp17D){case Cyc_Absyn_None: _LL52: _LL53:
 goto _LL55;case Cyc_Absyn_Signed: _LL54: _LL55:
 sns=({const char*_tmp17E="";_tag_fat(_tmp17E,sizeof(char),1U);});goto _LL51;default: _LL56: _LL57:
 sns=({const char*_tmp17F="unsigned ";_tag_fat(_tmp17F,sizeof(char),10U);});goto _LL51;}_LL51:;}
# 790
{enum Cyc_Absyn_Size_of _tmp180=sz;switch(_tmp180){case Cyc_Absyn_Char_sz: _LL59: _LL5A:
# 792
{enum Cyc_Absyn_Sign _tmp181=sn;switch(_tmp181){case Cyc_Absyn_None: _LL66: _LL67:
 sns=({const char*_tmp182="";_tag_fat(_tmp182,sizeof(char),1U);});goto _LL65;case Cyc_Absyn_Signed: _LL68: _LL69:
 sns=({const char*_tmp183="signed ";_tag_fat(_tmp183,sizeof(char),8U);});goto _LL65;default: _LL6A: _LL6B:
 sns=({const char*_tmp184="unsigned ";_tag_fat(_tmp184,sizeof(char),10U);});goto _LL65;}_LL65:;}
# 797
ts=({const char*_tmp185="char";_tag_fat(_tmp185,sizeof(char),5U);});
goto _LL58;case Cyc_Absyn_Short_sz: _LL5B: _LL5C:
 ts=({const char*_tmp186="short";_tag_fat(_tmp186,sizeof(char),6U);});goto _LL58;case Cyc_Absyn_Int_sz: _LL5D: _LL5E:
 ts=({const char*_tmp187="int";_tag_fat(_tmp187,sizeof(char),4U);});goto _LL58;case Cyc_Absyn_Long_sz: _LL5F: _LL60:
 ts=({const char*_tmp188="long";_tag_fat(_tmp188,sizeof(char),5U);});goto _LL58;case Cyc_Absyn_LongLong_sz: _LL61: _LL62:
 goto _LL64;default: _LL63: _LL64:
# 804
{enum Cyc_Cyclone_C_Compilers _tmp189=Cyc_Cyclone_c_compiler;if(_tmp189 == Cyc_Cyclone_Gcc_c){_LL6D: _LL6E:
 ts=({const char*_tmp18A="long long";_tag_fat(_tmp18A,sizeof(char),10U);});goto _LL6C;}else{_LL6F: _LL70:
 ts=({const char*_tmp18B="__int64";_tag_fat(_tmp18B,sizeof(char),8U);});goto _LL6C;}_LL6C:;}
# 808
goto _LL58;}_LL58:;}
# 810
s=Cyc_PP_text((struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp18E=({struct Cyc_String_pa_PrintArg_struct _tmp552;_tmp552.tag=0U,_tmp552.f1=(struct _fat_ptr)((struct _fat_ptr)sns);_tmp552;});struct Cyc_String_pa_PrintArg_struct _tmp18F=({struct Cyc_String_pa_PrintArg_struct _tmp551;_tmp551.tag=0U,_tmp551.f1=(struct _fat_ptr)((struct _fat_ptr)ts);_tmp551;});void*_tmp18C[2U];_tmp18C[0]=& _tmp18E,_tmp18C[1]=& _tmp18F;({struct _fat_ptr _tmp5DD=({const char*_tmp18D="%s%s";_tag_fat(_tmp18D,sizeof(char),5U);});Cyc_aprintf(_tmp5DD,_tag_fat(_tmp18C,sizeof(void*),2U));});}));
goto _LL0;}case 2U: _LL13: _tmp15A=((struct Cyc_Absyn_FloatCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp140)->f1)->f1;_LL14: {int i=_tmp15A;
# 813
{int _tmp190=i;switch(_tmp190){case 0U: _LL72: _LL73:
 s=Cyc_PP_text(({const char*_tmp191="float";_tag_fat(_tmp191,sizeof(char),6U);}));goto _LL71;case 1U: _LL74: _LL75:
 s=Cyc_PP_text(({const char*_tmp192="double";_tag_fat(_tmp192,sizeof(char),7U);}));goto _LL71;default: _LL76: _LL77:
 s=Cyc_PP_text(({const char*_tmp193="long double";_tag_fat(_tmp193,sizeof(char),12U);}));goto _LL71;}_LL71:;}
# 818
goto _LL0;}case 20U: _LL17: _tmp158=((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp140)->f1)->f1;_tmp159=((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp140)->f2;_LL18: {union Cyc_Absyn_AggrInfo info=_tmp158;struct Cyc_List_List*ts=_tmp159;
# 821
struct _tuple11 _stmttmp11=Cyc_Absyn_aggr_kinded_name(info);struct _tuple0*_tmp196;enum Cyc_Absyn_AggrKind _tmp195;_LL79: _tmp195=_stmttmp11.f1;_tmp196=_stmttmp11.f2;_LL7A: {enum Cyc_Absyn_AggrKind k=_tmp195;struct _tuple0*n=_tmp196;
s=({struct Cyc_PP_Doc*_tmp197[3U];({struct Cyc_PP_Doc*_tmp5E0=Cyc_Absynpp_aggr_kind2doc(k);_tmp197[0]=_tmp5E0;}),({struct Cyc_PP_Doc*_tmp5DF=Cyc_Absynpp_qvar2doc(n);_tmp197[1]=_tmp5DF;}),({struct Cyc_PP_Doc*_tmp5DE=Cyc_Absynpp_tps2doc(ts);_tmp197[2]=_tmp5DE;});Cyc_PP_cat(_tag_fat(_tmp197,sizeof(struct Cyc_PP_Doc*),3U));});
goto _LL0;}}case 16U: _LL1B: _tmp157=((struct Cyc_Absyn_AnonEnumCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp140)->f1)->f1;_LL1C: {struct Cyc_List_List*fs=_tmp157;
# 830
s=({struct Cyc_PP_Doc*_tmp199[4U];({struct Cyc_PP_Doc*_tmp5E4=Cyc_PP_text(({const char*_tmp19A="enum ";_tag_fat(_tmp19A,sizeof(char),6U);}));_tmp199[0]=_tmp5E4;}),({struct Cyc_PP_Doc*_tmp5E3=Cyc_Absynpp_lb();_tmp199[1]=_tmp5E3;}),({struct Cyc_PP_Doc*_tmp5E2=Cyc_PP_nest(2,Cyc_Absynpp_enumfields2doc(fs));_tmp199[2]=_tmp5E2;}),({struct Cyc_PP_Doc*_tmp5E1=Cyc_Absynpp_rb();_tmp199[3]=_tmp5E1;});Cyc_PP_cat(_tag_fat(_tmp199,sizeof(struct Cyc_PP_Doc*),4U));});goto _LL0;}case 15U: _LL1D: _tmp156=((struct Cyc_Absyn_EnumCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp140)->f1)->f1;_LL1E: {struct _tuple0*n=_tmp156;
s=({struct Cyc_PP_Doc*_tmp19B[2U];({struct Cyc_PP_Doc*_tmp5E6=Cyc_PP_text(({const char*_tmp19C="enum ";_tag_fat(_tmp19C,sizeof(char),6U);}));_tmp19B[0]=_tmp5E6;}),({struct Cyc_PP_Doc*_tmp5E5=Cyc_Absynpp_qvar2doc(n);_tmp19B[1]=_tmp5E5;});Cyc_PP_cat(_tag_fat(_tmp19B,sizeof(struct Cyc_PP_Doc*),2U));});goto _LL0;}case 17U: _LL23: _tmp155=((struct Cyc_Absyn_BuiltinCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp140)->f1)->f1;_LL24: {struct _fat_ptr t=_tmp155;
# 834
s=Cyc_PP_text(t);goto _LL0;}case 3U: _LL27: _tmp154=((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp140)->f2;_LL28: {struct Cyc_List_List*ts=_tmp154;
# 844
s=({struct Cyc_PP_Doc*_tmp1A4[3U];({struct Cyc_PP_Doc*_tmp5E9=Cyc_PP_text(({const char*_tmp1A5="region_t<";_tag_fat(_tmp1A5,sizeof(char),10U);}));_tmp1A4[0]=_tmp5E9;}),({struct Cyc_PP_Doc*_tmp5E8=Cyc_Absynpp_rgn2doc((void*)((struct Cyc_List_List*)_check_null(ts))->hd);_tmp1A4[1]=_tmp5E8;}),({struct Cyc_PP_Doc*_tmp5E7=Cyc_PP_text(({const char*_tmp1A6=">";_tag_fat(_tmp1A6,sizeof(char),2U);}));_tmp1A4[2]=_tmp5E7;});Cyc_PP_cat(_tag_fat(_tmp1A4,sizeof(struct Cyc_PP_Doc*),3U));});goto _LL0;}case 4U: _LL29: _tmp153=((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp140)->f2;_LL2A: {struct Cyc_List_List*ts=_tmp153;
# 846
s=({struct Cyc_PP_Doc*_tmp1A7[3U];({struct Cyc_PP_Doc*_tmp5EC=Cyc_PP_text(({const char*_tmp1A8="tag_t<";_tag_fat(_tmp1A8,sizeof(char),7U);}));_tmp1A7[0]=_tmp5EC;}),({struct Cyc_PP_Doc*_tmp5EB=Cyc_Absynpp_typ2doc((void*)((struct Cyc_List_List*)_check_null(ts))->hd);_tmp1A7[1]=_tmp5EB;}),({struct Cyc_PP_Doc*_tmp5EA=Cyc_PP_text(({const char*_tmp1A9=">";_tag_fat(_tmp1A9,sizeof(char),2U);}));_tmp1A7[2]=_tmp5EA;});Cyc_PP_cat(_tag_fat(_tmp1A7,sizeof(struct Cyc_PP_Doc*),3U));});goto _LL0;}case 6U: _LL2B: _LL2C:
 goto _LL2E;case 5U: _LL2D: _LL2E:
 goto _LL30;case 7U: _LL2F: _LL30:
 s=Cyc_Absynpp_rgn2doc(t);goto _LL0;case 10U: _LL31: _tmp152=((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp140)->f2;_LL32: {struct Cyc_List_List*ts=_tmp152;
# 851
s=({struct Cyc_PP_Doc*_tmp1AA[3U];({struct Cyc_PP_Doc*_tmp5EF=Cyc_PP_text(({const char*_tmp1AB="regions(";_tag_fat(_tmp1AB,sizeof(char),9U);}));_tmp1AA[0]=_tmp5EF;}),({struct Cyc_PP_Doc*_tmp5EE=Cyc_Absynpp_typ2doc((void*)((struct Cyc_List_List*)_check_null(ts))->hd);_tmp1AA[1]=_tmp5EE;}),({struct Cyc_PP_Doc*_tmp5ED=Cyc_PP_text(({const char*_tmp1AC=")";_tag_fat(_tmp1AC,sizeof(char),2U);}));_tmp1AA[2]=_tmp5ED;});Cyc_PP_cat(_tag_fat(_tmp1AA,sizeof(struct Cyc_PP_Doc*),3U));});goto _LL0;}case 8U: _LL33: _LL34:
 goto _LL36;case 9U: _LL35: _LL36:
 s=Cyc_Absynpp_eff2doc(t);goto _LL0;case 11U: _LL3D: _LL3E:
# 857
 s=Cyc_PP_text(({const char*_tmp1AD="@true";_tag_fat(_tmp1AD,sizeof(char),6U);}));goto _LL0;case 12U: _LL3F: _LL40:
 s=Cyc_PP_text(({const char*_tmp1AE="@false";_tag_fat(_tmp1AE,sizeof(char),7U);}));goto _LL0;case 13U: if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp140)->f2 != 0){_LL41: _tmp151=(void*)(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp140)->f2)->hd;_LL42: {void*t=_tmp151;
# 860
s=({struct Cyc_PP_Doc*_tmp1AF[4U];({struct Cyc_PP_Doc*_tmp5F3=Cyc_PP_text(({const char*_tmp1B0="@thin @numelts";_tag_fat(_tmp1B0,sizeof(char),15U);}));_tmp1AF[0]=_tmp5F3;}),({struct Cyc_PP_Doc*_tmp5F2=Cyc_Absynpp_lb();_tmp1AF[1]=_tmp5F2;}),({struct Cyc_PP_Doc*_tmp5F1=Cyc_Absynpp_typ2doc(t);_tmp1AF[2]=_tmp5F1;}),({struct Cyc_PP_Doc*_tmp5F0=Cyc_Absynpp_rb();_tmp1AF[3]=_tmp5F0;});Cyc_PP_cat(_tag_fat(_tmp1AF,sizeof(struct Cyc_PP_Doc*),4U));});goto _LL0;}}else{_LL43: _LL44:
# 862
 s=Cyc_PP_text(({const char*_tmp1B1="@thin";_tag_fat(_tmp1B1,sizeof(char),6U);}));goto _LL0;}default: _LL45: _LL46:
 s=Cyc_PP_text(({const char*_tmp1B2="@fat";_tag_fat(_tmp1B2,sizeof(char),5U);}));goto _LL0;}case 1U: _LL9: _tmp14D=((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp140)->f1;_tmp14E=(void*)((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp140)->f2;_tmp14F=((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp140)->f3;_tmp150=((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp140)->f4;_LLA: {struct Cyc_Core_Opt*k=_tmp14D;void*topt=_tmp14E;int i=_tmp14F;struct Cyc_Core_Opt*tvs=_tmp150;
# 748
if(topt != 0)
# 750
return Cyc_Absynpp_ntyp2doc(topt);else{
# 752
struct _fat_ptr kindstring=k == 0?({const char*_tmp167="K";_tag_fat(_tmp167,sizeof(char),2U);}): Cyc_Absynpp_kind2string((struct Cyc_Absyn_Kind*)k->v);
s=Cyc_PP_text((struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp165=({struct Cyc_String_pa_PrintArg_struct _tmp554;_tmp554.tag=0U,_tmp554.f1=(struct _fat_ptr)((struct _fat_ptr)kindstring);_tmp554;});struct Cyc_Int_pa_PrintArg_struct _tmp166=({struct Cyc_Int_pa_PrintArg_struct _tmp553;_tmp553.tag=1U,_tmp553.f1=(unsigned long)i;_tmp553;});void*_tmp163[2U];_tmp163[0]=& _tmp165,_tmp163[1]=& _tmp166;({struct _fat_ptr _tmp5F4=({const char*_tmp164="`E%s%d";_tag_fat(_tmp164,sizeof(char),7U);});Cyc_aprintf(_tmp5F4,_tag_fat(_tmp163,sizeof(void*),2U));});}));}
# 755
goto _LL0;}case 2U: _LLB: _tmp14C=((struct Cyc_Absyn_VarType_Absyn_Type_struct*)_tmp140)->f1;_LLC: {struct Cyc_Absyn_Tvar*tv=_tmp14C;
# 757
s=Cyc_Absynpp_tvar2doc(tv);
if(Cyc_Absynpp_print_all_kinds)
s=({struct Cyc_PP_Doc*_tmp168[3U];_tmp168[0]=s,({struct Cyc_PP_Doc*_tmp5F6=Cyc_PP_text(({const char*_tmp169="::";_tag_fat(_tmp169,sizeof(char),3U);}));_tmp168[1]=_tmp5F6;}),({struct Cyc_PP_Doc*_tmp5F5=Cyc_Absynpp_kindbound2doc(tv->kind);_tmp168[2]=_tmp5F5;});Cyc_PP_cat(_tag_fat(_tmp168,sizeof(struct Cyc_PP_Doc*),3U));});
# 758
goto _LL0;}case 6U: _LL15: _tmp14B=((struct Cyc_Absyn_TupleType_Absyn_Type_struct*)_tmp140)->f1;_LL16: {struct Cyc_List_List*ts=_tmp14B;
# 819
s=({struct Cyc_PP_Doc*_tmp194[2U];({struct Cyc_PP_Doc*_tmp5F8=Cyc_Absynpp_dollar();_tmp194[0]=_tmp5F8;}),({struct Cyc_PP_Doc*_tmp5F7=Cyc_Absynpp_args2doc(ts);_tmp194[1]=_tmp5F7;});Cyc_PP_cat(_tag_fat(_tmp194,sizeof(struct Cyc_PP_Doc*),2U));});goto _LL0;}case 7U: _LL19: _tmp149=((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmp140)->f1;_tmp14A=((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmp140)->f2;_LL1A: {enum Cyc_Absyn_AggrKind k=_tmp149;struct Cyc_List_List*fs=_tmp14A;
# 825
s=({struct Cyc_PP_Doc*_tmp198[4U];({struct Cyc_PP_Doc*_tmp5FC=Cyc_Absynpp_aggr_kind2doc(k);_tmp198[0]=_tmp5FC;}),({struct Cyc_PP_Doc*_tmp5FB=Cyc_Absynpp_lb();_tmp198[1]=_tmp5FB;}),({
struct Cyc_PP_Doc*_tmp5FA=Cyc_PP_nest(2,Cyc_Absynpp_aggrfields2doc(fs));_tmp198[2]=_tmp5FA;}),({
struct Cyc_PP_Doc*_tmp5F9=Cyc_Absynpp_rb();_tmp198[3]=_tmp5F9;});Cyc_PP_cat(_tag_fat(_tmp198,sizeof(struct Cyc_PP_Doc*),4U));});
goto _LL0;}case 9U: _LL1F: _tmp148=((struct Cyc_Absyn_ValueofType_Absyn_Type_struct*)_tmp140)->f1;_LL20: {struct Cyc_Absyn_Exp*e=_tmp148;
# 832
s=({struct Cyc_PP_Doc*_tmp19D[3U];({struct Cyc_PP_Doc*_tmp5FF=Cyc_PP_text(({const char*_tmp19E="valueof_t(";_tag_fat(_tmp19E,sizeof(char),11U);}));_tmp19D[0]=_tmp5FF;}),({struct Cyc_PP_Doc*_tmp5FE=Cyc_Absynpp_exp2doc(e);_tmp19D[1]=_tmp5FE;}),({struct Cyc_PP_Doc*_tmp5FD=Cyc_PP_text(({const char*_tmp19F=")";_tag_fat(_tmp19F,sizeof(char),2U);}));_tmp19D[2]=_tmp5FD;});Cyc_PP_cat(_tag_fat(_tmp19D,sizeof(struct Cyc_PP_Doc*),3U));});goto _LL0;}case 11U: _LL21: _tmp147=((struct Cyc_Absyn_TypeofType_Absyn_Type_struct*)_tmp140)->f1;_LL22: {struct Cyc_Absyn_Exp*e=_tmp147;
s=({struct Cyc_PP_Doc*_tmp1A0[3U];({struct Cyc_PP_Doc*_tmp602=Cyc_PP_text(({const char*_tmp1A1="typeof(";_tag_fat(_tmp1A1,sizeof(char),8U);}));_tmp1A0[0]=_tmp602;}),({struct Cyc_PP_Doc*_tmp601=Cyc_Absynpp_exp2doc(e);_tmp1A0[1]=_tmp601;}),({struct Cyc_PP_Doc*_tmp600=Cyc_PP_text(({const char*_tmp1A2=")";_tag_fat(_tmp1A2,sizeof(char),2U);}));_tmp1A0[2]=_tmp600;});Cyc_PP_cat(_tag_fat(_tmp1A0,sizeof(struct Cyc_PP_Doc*),3U));});goto _LL0;}case 8U: _LL25: _tmp144=((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmp140)->f1;_tmp145=((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmp140)->f2;_tmp146=((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmp140)->f3;_LL26: {struct _tuple0*n=_tmp144;struct Cyc_List_List*ts=_tmp145;struct Cyc_Absyn_Typedefdecl*kopt=_tmp146;
# 841
s=({struct Cyc_PP_Doc*_tmp1A3[2U];({struct Cyc_PP_Doc*_tmp604=Cyc_Absynpp_qvar2doc(n);_tmp1A3[0]=_tmp604;}),({struct Cyc_PP_Doc*_tmp603=Cyc_Absynpp_tps2doc(ts);_tmp1A3[1]=_tmp603;});Cyc_PP_cat(_tag_fat(_tmp1A3,sizeof(struct Cyc_PP_Doc*),2U));});
goto _LL0;}default: switch(*((int*)((struct Cyc_Absyn_TypeDecl*)((struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct*)_tmp140)->f1)->r)){case 0U: _LL37: _tmp143=((struct Cyc_Absyn_Aggr_td_Absyn_Raw_typedecl_struct*)(((struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct*)_tmp140)->f1)->r)->f1;_LL38: {struct Cyc_Absyn_Aggrdecl*d=_tmp143;
# 854
s=Cyc_Absynpp_aggrdecl2doc(d);goto _LL0;}case 1U: _LL39: _tmp142=((struct Cyc_Absyn_Enum_td_Absyn_Raw_typedecl_struct*)(((struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct*)_tmp140)->f1)->r)->f1;_LL3A: {struct Cyc_Absyn_Enumdecl*d=_tmp142;
s=Cyc_Absynpp_enumdecl2doc(d);goto _LL0;}default: _LL3B: _tmp141=((struct Cyc_Absyn_Datatype_td_Absyn_Raw_typedecl_struct*)(((struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct*)_tmp140)->f1)->r)->f1;_LL3C: {struct Cyc_Absyn_Datatypedecl*d=_tmp141;
s=Cyc_Absynpp_datatypedecl2doc(d);goto _LL0;}}}_LL0:;}
# 865
return s;}
# 607
struct Cyc_PP_Doc*Cyc_Absynpp_vo2doc(struct _fat_ptr*vo){
# 869
return vo == 0?Cyc_PP_nil_doc(): Cyc_PP_text(*vo);}struct _tuple18{void*f1;void*f2;};
# 607
struct Cyc_PP_Doc*Cyc_Absynpp_rgn_cmp2doc(struct _tuple18*cmp){
# 873
void*_tmp1B4;void*_tmp1B3;_LL1: _tmp1B3=cmp->f1;_tmp1B4=cmp->f2;_LL2: {void*r1=_tmp1B3;void*r2=_tmp1B4;
return({struct Cyc_PP_Doc*_tmp1B5[3U];({struct Cyc_PP_Doc*_tmp607=Cyc_Absynpp_rgn2doc(r1);_tmp1B5[0]=_tmp607;}),({struct Cyc_PP_Doc*_tmp606=Cyc_PP_text(({const char*_tmp1B6=" > ";_tag_fat(_tmp1B6,sizeof(char),4U);}));_tmp1B5[1]=_tmp606;}),({struct Cyc_PP_Doc*_tmp605=Cyc_Absynpp_rgn2doc(r2);_tmp1B5[2]=_tmp605;});Cyc_PP_cat(_tag_fat(_tmp1B5,sizeof(struct Cyc_PP_Doc*),3U));});}}
# 607
struct Cyc_PP_Doc*Cyc_Absynpp_rgnpo2doc(struct Cyc_List_List*po){
# 878
return({struct _fat_ptr _tmp60B=({const char*_tmp1B7="";_tag_fat(_tmp1B7,sizeof(char),1U);});struct _fat_ptr _tmp60A=({const char*_tmp1B8="";_tag_fat(_tmp1B8,sizeof(char),1U);});struct _fat_ptr _tmp609=({const char*_tmp1B9=",";_tag_fat(_tmp1B9,sizeof(char),2U);});Cyc_PP_group(_tmp60B,_tmp60A,_tmp609,({struct Cyc_List_List*(*_tmp608)(struct Cyc_PP_Doc*(*f)(struct _tuple18*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp1BA)(struct Cyc_PP_Doc*(*f)(struct _tuple18*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_PP_Doc*(*f)(struct _tuple18*),struct Cyc_List_List*x))Cyc_List_map;_tmp1BA;});_tmp608(Cyc_Absynpp_rgn_cmp2doc,po);}));});}
# 607
struct Cyc_PP_Doc*Cyc_Absynpp_funarg2doc(struct _tuple8*t){
# 882
struct _fat_ptr*vo=(*t).f1;
struct Cyc_Core_Opt*dopt=vo == 0?0:({struct Cyc_Core_Opt*_tmp1BB=_cycalloc(sizeof(*_tmp1BB));({struct Cyc_PP_Doc*_tmp60C=Cyc_PP_text(*vo);_tmp1BB->v=_tmp60C;});_tmp1BB;});
return Cyc_Absynpp_tqtd2doc((*t).f2,(*t).f3,dopt);}
# 607
struct Cyc_PP_Doc*Cyc_Absynpp_funargs2doc(struct Cyc_List_List*args,int c_varargs,struct Cyc_Absyn_VarargInfo*cyc_varargs,void*effopt,struct Cyc_List_List*rgn_po,struct Cyc_Absyn_Exp*req,struct Cyc_Absyn_Exp*ens,struct Cyc_List_List*ieff,struct Cyc_List_List*oeff){
# 895
struct Cyc_List_List*arg_docs=({struct Cyc_List_List*(*_tmp60D)(struct Cyc_PP_Doc*(*f)(struct _tuple8*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp1D4)(struct Cyc_PP_Doc*(*f)(struct _tuple8*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_PP_Doc*(*f)(struct _tuple8*),struct Cyc_List_List*x))Cyc_List_map;_tmp1D4;});_tmp60D(Cyc_Absynpp_funarg2doc,args);});
struct Cyc_PP_Doc*eff_doc;
if(c_varargs)
arg_docs=({struct Cyc_List_List*_tmp60F=arg_docs;Cyc_List_append(_tmp60F,({struct Cyc_List_List*_tmp1BD=_cycalloc(sizeof(*_tmp1BD));({struct Cyc_PP_Doc*_tmp60E=Cyc_PP_text(({const char*_tmp1BC="...";_tag_fat(_tmp1BC,sizeof(char),4U);}));_tmp1BD->hd=_tmp60E;}),_tmp1BD->tl=0;_tmp1BD;}));});else{
if(cyc_varargs != 0){
struct Cyc_PP_Doc*varargs_doc=({struct Cyc_PP_Doc*_tmp1BF[3U];({struct Cyc_PP_Doc*_tmp613=Cyc_PP_text(({const char*_tmp1C0="...";_tag_fat(_tmp1C0,sizeof(char),4U);}));_tmp1BF[0]=_tmp613;}),
cyc_varargs->inject?({struct Cyc_PP_Doc*_tmp612=Cyc_PP_text(({const char*_tmp1C1=" inject ";_tag_fat(_tmp1C1,sizeof(char),9U);}));_tmp1BF[1]=_tmp612;}):({struct Cyc_PP_Doc*_tmp611=Cyc_PP_text(({const char*_tmp1C2=" ";_tag_fat(_tmp1C2,sizeof(char),2U);}));_tmp1BF[1]=_tmp611;}),({
struct Cyc_PP_Doc*_tmp610=Cyc_Absynpp_funarg2doc(({struct _tuple8*_tmp1C3=_cycalloc(sizeof(*_tmp1C3));_tmp1C3->f1=cyc_varargs->name,_tmp1C3->f2=cyc_varargs->tq,_tmp1C3->f3=cyc_varargs->type;_tmp1C3;}));_tmp1BF[2]=_tmp610;});Cyc_PP_cat(_tag_fat(_tmp1BF,sizeof(struct Cyc_PP_Doc*),3U));});
# 904
arg_docs=({struct Cyc_List_List*_tmp614=arg_docs;Cyc_List_append(_tmp614,({struct Cyc_List_List*_tmp1BE=_cycalloc(sizeof(*_tmp1BE));_tmp1BE->hd=varargs_doc,_tmp1BE->tl=0;_tmp1BE;}));});}}{
# 897
struct Cyc_PP_Doc*arg_doc=({
# 906
struct _fat_ptr _tmp617=({const char*_tmp1D1="";_tag_fat(_tmp1D1,sizeof(char),1U);});struct _fat_ptr _tmp616=({const char*_tmp1D2="";_tag_fat(_tmp1D2,sizeof(char),1U);});struct _fat_ptr _tmp615=({const char*_tmp1D3=",";_tag_fat(_tmp1D3,sizeof(char),2U);});Cyc_PP_group(_tmp617,_tmp616,_tmp615,arg_docs);});
if(effopt != 0 && Cyc_Absynpp_print_all_effects)
arg_doc=({struct Cyc_PP_Doc*_tmp1C4[3U];_tmp1C4[0]=arg_doc,({struct Cyc_PP_Doc*_tmp619=Cyc_PP_text(({const char*_tmp1C5=";";_tag_fat(_tmp1C5,sizeof(char),2U);}));_tmp1C4[1]=_tmp619;}),({struct Cyc_PP_Doc*_tmp618=Cyc_Absynpp_eff2doc(effopt);_tmp1C4[2]=_tmp618;});Cyc_PP_cat(_tag_fat(_tmp1C4,sizeof(struct Cyc_PP_Doc*),3U));});
# 907
if(rgn_po != 0)
# 910
arg_doc=({struct Cyc_PP_Doc*_tmp1C6[3U];_tmp1C6[0]=arg_doc,({struct Cyc_PP_Doc*_tmp61B=Cyc_PP_text(({const char*_tmp1C7=":";_tag_fat(_tmp1C7,sizeof(char),2U);}));_tmp1C6[1]=_tmp61B;}),({struct Cyc_PP_Doc*_tmp61A=Cyc_Absynpp_rgnpo2doc(rgn_po);_tmp1C6[2]=_tmp61A;});Cyc_PP_cat(_tag_fat(_tmp1C6,sizeof(struct Cyc_PP_Doc*),3U));});{
# 907
struct Cyc_PP_Doc*res=({struct Cyc_PP_Doc*_tmp1CE[3U];({
# 911
struct Cyc_PP_Doc*_tmp61D=Cyc_PP_text(({const char*_tmp1CF="(";_tag_fat(_tmp1CF,sizeof(char),2U);}));_tmp1CE[0]=_tmp61D;}),_tmp1CE[1]=arg_doc,({struct Cyc_PP_Doc*_tmp61C=Cyc_PP_text(({const char*_tmp1D0=")";_tag_fat(_tmp1D0,sizeof(char),2U);}));_tmp1CE[2]=_tmp61C;});Cyc_PP_cat(_tag_fat(_tmp1CE,sizeof(struct Cyc_PP_Doc*),3U));});
if(req != 0)
res=({struct Cyc_PP_Doc*_tmp1C8[4U];_tmp1C8[0]=res,({struct Cyc_PP_Doc*_tmp620=Cyc_PP_text(({const char*_tmp1C9=" @requires(";_tag_fat(_tmp1C9,sizeof(char),12U);}));_tmp1C8[1]=_tmp620;}),({struct Cyc_PP_Doc*_tmp61F=Cyc_Absynpp_exp2doc(req);_tmp1C8[2]=_tmp61F;}),({struct Cyc_PP_Doc*_tmp61E=Cyc_PP_text(({const char*_tmp1CA=")";_tag_fat(_tmp1CA,sizeof(char),2U);}));_tmp1C8[3]=_tmp61E;});Cyc_PP_cat(_tag_fat(_tmp1C8,sizeof(struct Cyc_PP_Doc*),4U));});
# 912
if(ens != 0)
# 915
res=({struct Cyc_PP_Doc*_tmp1CB[4U];_tmp1CB[0]=res,({struct Cyc_PP_Doc*_tmp623=Cyc_PP_text(({const char*_tmp1CC=" @ensures(";_tag_fat(_tmp1CC,sizeof(char),11U);}));_tmp1CB[1]=_tmp623;}),({struct Cyc_PP_Doc*_tmp622=Cyc_Absynpp_exp2doc(ens);_tmp1CB[2]=_tmp622;}),({struct Cyc_PP_Doc*_tmp621=Cyc_PP_text(({const char*_tmp1CD=")";_tag_fat(_tmp1CD,sizeof(char),2U);}));_tmp1CB[3]=_tmp621;});Cyc_PP_cat(_tag_fat(_tmp1CB,sizeof(struct Cyc_PP_Doc*),4U));});
# 912
return res;}}}
# 607
struct Cyc_PP_Doc*Cyc_Absynpp_var2doc(struct _fat_ptr*v){
# 919
return Cyc_PP_text(*v);}
# 607
struct _fat_ptr Cyc_Absynpp_qvar2string(struct _tuple0*q){
# 922
struct Cyc_List_List*prefix=0;
int match;
{union Cyc_Absyn_Nmspace _stmttmp12=(*q).f1;union Cyc_Absyn_Nmspace _tmp1D5=_stmttmp12;struct Cyc_List_List*_tmp1D6;struct Cyc_List_List*_tmp1D7;struct Cyc_List_List*_tmp1D8;switch((_tmp1D5.C_n).tag){case 4U: _LL1: _LL2:
 _tmp1D8=0;goto _LL4;case 1U: _LL3: _tmp1D8=(_tmp1D5.Rel_n).val;_LL4: {struct Cyc_List_List*x=_tmp1D8;
# 927
match=0;
prefix=x;
goto _LL0;}case 3U: _LL5: _tmp1D7=(_tmp1D5.C_n).val;_LL6: {struct Cyc_List_List*x=_tmp1D7;
# 931
match=Cyc_Absynpp_use_curr_namespace &&({int(*_tmp625)(int(*cmp)(struct _fat_ptr*,struct _fat_ptr*),struct Cyc_List_List*l1,struct Cyc_List_List*l2)=({int(*_tmp1D9)(int(*cmp)(struct _fat_ptr*,struct _fat_ptr*),struct Cyc_List_List*l1,struct Cyc_List_List*l2)=(int(*)(int(*cmp)(struct _fat_ptr*,struct _fat_ptr*),struct Cyc_List_List*l1,struct Cyc_List_List*l2))Cyc_List_list_prefix;_tmp1D9;});struct Cyc_List_List*_tmp624=x;_tmp625(Cyc_strptrcmp,_tmp624,Cyc_Absynpp_curr_namespace);});
# 933
goto _LL0;}default: _LL7: _tmp1D6=(_tmp1D5.Abs_n).val;_LL8: {struct Cyc_List_List*x=_tmp1D6;
# 935
match=Cyc_Absynpp_use_curr_namespace &&({int(*_tmp627)(int(*cmp)(struct _fat_ptr*,struct _fat_ptr*),struct Cyc_List_List*l1,struct Cyc_List_List*l2)=({int(*_tmp1DA)(int(*cmp)(struct _fat_ptr*,struct _fat_ptr*),struct Cyc_List_List*l1,struct Cyc_List_List*l2)=(int(*)(int(*cmp)(struct _fat_ptr*,struct _fat_ptr*),struct Cyc_List_List*l1,struct Cyc_List_List*l2))Cyc_List_list_prefix;_tmp1DA;});struct Cyc_List_List*_tmp626=x;_tmp627(Cyc_strptrcmp,_tmp626,Cyc_Absynpp_curr_namespace);});
prefix=Cyc_Absynpp_qvar_to_Cids && Cyc_Absynpp_add_cyc_prefix?({struct Cyc_List_List*_tmp1DB=_cycalloc(sizeof(*_tmp1DB));_tmp1DB->hd=Cyc_Absynpp_cyc_stringptr,_tmp1DB->tl=x;_tmp1DB;}): x;
goto _LL0;}}_LL0:;}
# 939
if(Cyc_Absynpp_qvar_to_Cids)
return(struct _fat_ptr)({struct Cyc_List_List*_tmp629=({struct Cyc_List_List*_tmp628=prefix;Cyc_List_append(_tmp628,({struct Cyc_List_List*_tmp1DC=_cycalloc(sizeof(*_tmp1DC));
_tmp1DC->hd=(*q).f2,_tmp1DC->tl=0;_tmp1DC;}));});
# 940
Cyc_str_sepstr(_tmp629,({const char*_tmp1DD="_";_tag_fat(_tmp1DD,sizeof(char),2U);}));});else{
# 944
if(match)
return*(*q).f2;else{
# 947
return(struct _fat_ptr)({struct Cyc_List_List*_tmp62B=({struct Cyc_List_List*_tmp62A=prefix;Cyc_List_append(_tmp62A,({struct Cyc_List_List*_tmp1DE=_cycalloc(sizeof(*_tmp1DE));_tmp1DE->hd=(*q).f2,_tmp1DE->tl=0;_tmp1DE;}));});Cyc_str_sepstr(_tmp62B,({const char*_tmp1DF="::";_tag_fat(_tmp1DF,sizeof(char),3U);}));});}}}
# 607
struct Cyc_PP_Doc*Cyc_Absynpp_qvar2doc(struct _tuple0*q){
# 952
return Cyc_PP_text(Cyc_Absynpp_qvar2string(q));}
# 607
struct Cyc_PP_Doc*Cyc_Absynpp_qvar2bolddoc(struct _tuple0*q){
# 956
struct _fat_ptr qs=Cyc_Absynpp_qvar2string(q);
if(Cyc_PP_tex_output)
# 959
return({struct _fat_ptr _tmp62E=(struct _fat_ptr)({struct _fat_ptr _tmp62D=(struct _fat_ptr)({struct _fat_ptr _tmp62C=({const char*_tmp1E0="\\textbf{";_tag_fat(_tmp1E0,sizeof(char),9U);});Cyc_strconcat(_tmp62C,(struct _fat_ptr)qs);});Cyc_strconcat(_tmp62D,({const char*_tmp1E1="}";_tag_fat(_tmp1E1,sizeof(char),2U);}));});Cyc_PP_text_width(_tmp62E,(int)
Cyc_strlen((struct _fat_ptr)qs));});else{
return Cyc_PP_text(qs);}}
# 607
struct _fat_ptr Cyc_Absynpp_typedef_name2string(struct _tuple0*v){
# 966
if(Cyc_Absynpp_qvar_to_Cids)return Cyc_Absynpp_qvar2string(v);if(Cyc_Absynpp_use_curr_namespace){
# 970
union Cyc_Absyn_Nmspace _stmttmp13=(*v).f1;union Cyc_Absyn_Nmspace _tmp1E2=_stmttmp13;struct Cyc_List_List*_tmp1E3;struct Cyc_List_List*_tmp1E4;switch((_tmp1E2.C_n).tag){case 4U: _LL1: _LL2:
 goto _LL4;case 1U: if((_tmp1E2.Rel_n).val == 0){_LL3: _LL4:
 return*(*v).f2;}else{_LL9: _LLA:
# 980
 return(struct _fat_ptr)({struct _fat_ptr _tmp62F=({const char*_tmp1E6="/* bad namespace : */ ";_tag_fat(_tmp1E6,sizeof(char),23U);});Cyc_strconcat(_tmp62F,(struct _fat_ptr)Cyc_Absynpp_qvar2string(v));});}case 3U: _LL5: _tmp1E4=(_tmp1E2.C_n).val;_LL6: {struct Cyc_List_List*l=_tmp1E4;
# 973
_tmp1E3=l;goto _LL8;}default: _LL7: _tmp1E3=(_tmp1E2.Abs_n).val;_LL8: {struct Cyc_List_List*l=_tmp1E3;
# 975
if(({int(*_tmp631)(int(*cmp)(struct _fat_ptr*,struct _fat_ptr*),struct Cyc_List_List*l1,struct Cyc_List_List*l2)=({int(*_tmp1E5)(int(*cmp)(struct _fat_ptr*,struct _fat_ptr*),struct Cyc_List_List*l1,struct Cyc_List_List*l2)=(int(*)(int(*cmp)(struct _fat_ptr*,struct _fat_ptr*),struct Cyc_List_List*l1,struct Cyc_List_List*l2))Cyc_List_list_cmp;_tmp1E5;});struct Cyc_List_List*_tmp630=l;_tmp631(Cyc_strptrcmp,_tmp630,Cyc_Absynpp_curr_namespace);})== 0)
return*(*v).f2;else{
# 978
goto _LLA;}}}_LL0:;}else{
# 983
return*(*v).f2;}}
# 607
struct Cyc_PP_Doc*Cyc_Absynpp_typedef_name2doc(struct _tuple0*v){
# 987
return Cyc_PP_text(Cyc_Absynpp_typedef_name2string(v));}
# 607
struct Cyc_PP_Doc*Cyc_Absynpp_typedef_name2bolddoc(struct _tuple0*v){
# 990
struct _fat_ptr vs=Cyc_Absynpp_typedef_name2string(v);
if(Cyc_PP_tex_output)
# 993
return({struct _fat_ptr _tmp634=(struct _fat_ptr)({struct _fat_ptr _tmp633=(struct _fat_ptr)({struct _fat_ptr _tmp632=({const char*_tmp1E7="\\textbf{";_tag_fat(_tmp1E7,sizeof(char),9U);});Cyc_strconcat(_tmp632,(struct _fat_ptr)vs);});Cyc_strconcat(_tmp633,({const char*_tmp1E8="}";_tag_fat(_tmp1E8,sizeof(char),2U);}));});Cyc_PP_text_width(_tmp634,(int)
Cyc_strlen((struct _fat_ptr)vs));});else{
return Cyc_PP_text(vs);}}
# 607
struct Cyc_PP_Doc*Cyc_Absynpp_typ2doc(void*t){
# 999
return({struct Cyc_Absyn_Tqual _tmp635=Cyc_Absyn_empty_tqual(0U);Cyc_Absynpp_tqtd2doc(_tmp635,t,0);});}
# 607
static struct Cyc_PP_Doc*Cyc_Absynpp_offsetof_field_to_doc(void*f){
# 1003
void*_tmp1E9=f;unsigned _tmp1EA;struct _fat_ptr*_tmp1EB;if(((struct Cyc_Absyn_StructField_Absyn_OffsetofField_struct*)_tmp1E9)->tag == 0U){_LL1: _tmp1EB=((struct Cyc_Absyn_StructField_Absyn_OffsetofField_struct*)_tmp1E9)->f1;_LL2: {struct _fat_ptr*n=_tmp1EB;
return Cyc_PP_textptr(n);}}else{_LL3: _tmp1EA=((struct Cyc_Absyn_TupleIndex_Absyn_OffsetofField_struct*)_tmp1E9)->f1;_LL4: {unsigned n=_tmp1EA;
return Cyc_PP_text((struct _fat_ptr)({struct Cyc_Int_pa_PrintArg_struct _tmp1EE=({struct Cyc_Int_pa_PrintArg_struct _tmp555;_tmp555.tag=1U,_tmp555.f1=(unsigned long)((int)n);_tmp555;});void*_tmp1EC[1U];_tmp1EC[0]=& _tmp1EE;({struct _fat_ptr _tmp636=({const char*_tmp1ED="%d";_tag_fat(_tmp1ED,sizeof(char),3U);});Cyc_aprintf(_tmp636,_tag_fat(_tmp1EC,sizeof(void*),1U));});}));}}_LL0:;}
# 607
int Cyc_Absynpp_exp_prec(struct Cyc_Absyn_Exp*e){
# 1017 "absynpp.cyc"
void*_stmttmp14=e->r;void*_tmp1EF=_stmttmp14;struct Cyc_Absyn_Exp*_tmp1F0;struct Cyc_Absyn_Exp*_tmp1F1;enum Cyc_Absyn_Primop _tmp1F2;switch(*((int*)_tmp1EF)){case 0U: _LL1: _LL2:
 goto _LL4;case 1U: _LL3: _LL4:
 return 10000;case 3U: _LL5: _tmp1F2=((struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct*)_tmp1EF)->f1;_LL6: {enum Cyc_Absyn_Primop p=_tmp1F2;
# 1021
enum Cyc_Absyn_Primop _tmp1F3=p;switch(_tmp1F3){case Cyc_Absyn_Plus: _LL58: _LL59:
 return 100;case Cyc_Absyn_Times: _LL5A: _LL5B:
 return 110;case Cyc_Absyn_Minus: _LL5C: _LL5D:
 return 100;case Cyc_Absyn_Div: _LL5E: _LL5F:
 goto _LL61;case Cyc_Absyn_Mod: _LL60: _LL61:
 return 110;case Cyc_Absyn_Eq: _LL62: _LL63:
 goto _LL65;case Cyc_Absyn_Neq: _LL64: _LL65:
 return 70;case Cyc_Absyn_Gt: _LL66: _LL67:
 goto _LL69;case Cyc_Absyn_Lt: _LL68: _LL69:
 goto _LL6B;case Cyc_Absyn_Gte: _LL6A: _LL6B:
 goto _LL6D;case Cyc_Absyn_Lte: _LL6C: _LL6D:
 return 80;case Cyc_Absyn_Not: _LL6E: _LL6F:
 goto _LL71;case Cyc_Absyn_Bitnot: _LL70: _LL71:
 return 130;case Cyc_Absyn_Bitand: _LL72: _LL73:
 return 60;case Cyc_Absyn_Bitor: _LL74: _LL75:
 return 40;case Cyc_Absyn_Bitxor: _LL76: _LL77:
 return 50;case Cyc_Absyn_Bitlshift: _LL78: _LL79:
 return 90;case Cyc_Absyn_Bitlrshift: _LL7A: _LL7B:
 return 80;case Cyc_Absyn_Numelts: _LL7C: _LL7D:
 return 140;default: _LL7E: _LL7F:
 return 140;}_LL57:;}case 4U: _LL7: _LL8:
# 1043
 return 20;case 5U: _LL9: _LLA:
 return 130;case 6U: _LLB: _LLC:
 return 30;case 7U: _LLD: _LLE:
 return 35;case 8U: _LLF: _LL10:
 return 30;case 9U: _LL11: _LL12:
 return - 10;case 10U: _LL13: _LL14:
 return 140;case 2U: _LL15: _LL16:
 return 140;case 11U: _LL17: _LL18:
 return 130;case 12U: _LL19: _tmp1F1=((struct Cyc_Absyn_NoInstantiate_e_Absyn_Raw_exp_struct*)_tmp1EF)->f1;_LL1A: {struct Cyc_Absyn_Exp*e2=_tmp1F1;
return Cyc_Absynpp_exp_prec(e2);}case 13U: _LL1B: _tmp1F0=((struct Cyc_Absyn_Instantiate_e_Absyn_Raw_exp_struct*)_tmp1EF)->f1;_LL1C: {struct Cyc_Absyn_Exp*e2=_tmp1F0;
return Cyc_Absynpp_exp_prec(e2);}case 14U: _LL1D: _LL1E:
 return 120;case 16U: _LL1F: _LL20:
 return 15;case 15U: _LL21: _LL22:
 goto _LL24;case 17U: _LL23: _LL24:
 goto _LL26;case 18U: _LL25: _LL26:
 goto _LL28;case 40U: _LL27: _LL28:
 goto _LL2A;case 41U: _LL29: _LL2A:
 goto _LL2C;case 39U: _LL2B: _LL2C:
 goto _LL2E;case 19U: _LL2D: _LL2E:
 goto _LL30;case 20U: _LL2F: _LL30:
 goto _LL32;case 42U: _LL31: _LL32:
 return 130;case 21U: _LL33: _LL34:
 goto _LL36;case 22U: _LL35: _LL36:
 goto _LL38;case 23U: _LL37: _LL38:
 return 140;case 24U: _LL39: _LL3A:
 return 150;case 25U: _LL3B: _LL3C:
 goto _LL3E;case 26U: _LL3D: _LL3E:
 goto _LL40;case 27U: _LL3F: _LL40:
 goto _LL42;case 28U: _LL41: _LL42:
 goto _LL44;case 29U: _LL43: _LL44:
 goto _LL46;case 30U: _LL45: _LL46:
 goto _LL48;case 31U: _LL47: _LL48:
 goto _LL4A;case 32U: _LL49: _LL4A:
 goto _LL4C;case 33U: _LL4B: _LL4C:
 goto _LL4E;case 34U: _LL4D: _LL4E:
 goto _LL50;case 35U: _LL4F: _LL50:
 goto _LL52;case 36U: _LL51: _LL52:
 goto _LL54;case 37U: _LL53: _LL54:
 return 140;default: _LL55: _LL56:
 return 10000;}_LL0:;}
# 607 "absynpp.cyc"
struct Cyc_PP_Doc*Cyc_Absynpp_exp2doc(struct Cyc_Absyn_Exp*e){
# 1087 "absynpp.cyc"
return Cyc_Absynpp_exp2doc_prec(0,e);}struct _tuple19{struct _fat_ptr f1;struct Cyc_Absyn_Exp*f2;};
# 607 "absynpp.cyc"
static struct Cyc_PP_Doc*Cyc_Absynpp_asm_iolist_doc_in(struct Cyc_List_List*o){
# 1091 "absynpp.cyc"
struct Cyc_PP_Doc*s=Cyc_PP_nil_doc();
while((unsigned)o){
struct _tuple19*_stmttmp15=(struct _tuple19*)o->hd;struct Cyc_Absyn_Exp*_tmp1F5;struct _fat_ptr _tmp1F4;_LL1: _tmp1F4=_stmttmp15->f1;_tmp1F5=_stmttmp15->f2;_LL2: {struct _fat_ptr c=_tmp1F4;struct Cyc_Absyn_Exp*e=_tmp1F5;
s=({struct Cyc_PP_Doc*_tmp1F6[5U];({struct Cyc_PP_Doc*_tmp63B=Cyc_PP_text(({const char*_tmp1F7="\"";_tag_fat(_tmp1F7,sizeof(char),2U);}));_tmp1F6[0]=_tmp63B;}),({struct Cyc_PP_Doc*_tmp63A=Cyc_PP_text(c);_tmp1F6[1]=_tmp63A;}),({struct Cyc_PP_Doc*_tmp639=Cyc_PP_text(({const char*_tmp1F8="\" (";_tag_fat(_tmp1F8,sizeof(char),4U);}));_tmp1F6[2]=_tmp639;}),({struct Cyc_PP_Doc*_tmp638=Cyc_Absynpp_exp2doc(e);_tmp1F6[3]=_tmp638;}),({struct Cyc_PP_Doc*_tmp637=Cyc_PP_text(({const char*_tmp1F9=")";_tag_fat(_tmp1F9,sizeof(char),2U);}));_tmp1F6[4]=_tmp637;});Cyc_PP_cat(_tag_fat(_tmp1F6,sizeof(struct Cyc_PP_Doc*),5U));});
o=o->tl;
if((unsigned)o)
s=({struct Cyc_PP_Doc*_tmp1FA[2U];_tmp1FA[0]=s,({struct Cyc_PP_Doc*_tmp63C=Cyc_PP_text(({const char*_tmp1FB=",";_tag_fat(_tmp1FB,sizeof(char),2U);}));_tmp1FA[1]=_tmp63C;});Cyc_PP_cat(_tag_fat(_tmp1FA,sizeof(struct Cyc_PP_Doc*),2U));});}}
# 1100
return s;}
# 607 "absynpp.cyc"
static struct Cyc_PP_Doc*Cyc_Absynpp_asm_iolist_doc(struct Cyc_List_List*o,struct Cyc_List_List*i,struct Cyc_List_List*cl){
# 1104 "absynpp.cyc"
struct Cyc_PP_Doc*s=Cyc_PP_nil_doc();
if((unsigned)o)
s=({struct Cyc_PP_Doc*_tmp1FC[2U];({struct Cyc_PP_Doc*_tmp63E=Cyc_PP_text(({const char*_tmp1FD=": ";_tag_fat(_tmp1FD,sizeof(char),3U);}));_tmp1FC[0]=_tmp63E;}),({struct Cyc_PP_Doc*_tmp63D=Cyc_Absynpp_asm_iolist_doc_in(o);_tmp1FC[1]=_tmp63D;});Cyc_PP_cat(_tag_fat(_tmp1FC,sizeof(struct Cyc_PP_Doc*),2U));});
# 1105
if((unsigned)i){
# 1109
if(!((unsigned)o))
s=({struct Cyc_PP_Doc*_tmp1FE[3U];_tmp1FE[0]=s,({struct Cyc_PP_Doc*_tmp640=Cyc_PP_text(({const char*_tmp1FF=": : ";_tag_fat(_tmp1FF,sizeof(char),5U);}));_tmp1FE[1]=_tmp640;}),({struct Cyc_PP_Doc*_tmp63F=Cyc_Absynpp_asm_iolist_doc_in(i);_tmp1FE[2]=_tmp63F;});Cyc_PP_cat(_tag_fat(_tmp1FE,sizeof(struct Cyc_PP_Doc*),3U));});else{
# 1112
s=({struct Cyc_PP_Doc*_tmp200[3U];_tmp200[0]=s,({struct Cyc_PP_Doc*_tmp642=Cyc_PP_text(({const char*_tmp201=" : ";_tag_fat(_tmp201,sizeof(char),4U);}));_tmp200[1]=_tmp642;}),({struct Cyc_PP_Doc*_tmp641=Cyc_Absynpp_asm_iolist_doc_in(i);_tmp200[2]=_tmp641;});Cyc_PP_cat(_tag_fat(_tmp200,sizeof(struct Cyc_PP_Doc*),3U));});}}
# 1105
if((unsigned)cl){
# 1115
int ncol=(unsigned)i?2:((unsigned)o?1: 0);
s=({struct Cyc_PP_Doc*_tmp202[2U];_tmp202[0]=s,ncol == 0?({struct Cyc_PP_Doc*_tmp645=Cyc_PP_text(({const char*_tmp203=": : :";_tag_fat(_tmp203,sizeof(char),6U);}));_tmp202[1]=_tmp645;}):(ncol == 1?({struct Cyc_PP_Doc*_tmp644=Cyc_PP_text(({const char*_tmp204=" : : ";_tag_fat(_tmp204,sizeof(char),6U);}));_tmp202[1]=_tmp644;}):({struct Cyc_PP_Doc*_tmp643=Cyc_PP_text(({const char*_tmp205=" : ";_tag_fat(_tmp205,sizeof(char),4U);}));_tmp202[1]=_tmp643;}));Cyc_PP_cat(_tag_fat(_tmp202,sizeof(struct Cyc_PP_Doc*),2U));});
while(cl != 0){
s=({struct Cyc_PP_Doc*_tmp206[4U];_tmp206[0]=s,({struct Cyc_PP_Doc*_tmp648=Cyc_PP_text(({const char*_tmp207="\"";_tag_fat(_tmp207,sizeof(char),2U);}));_tmp206[1]=_tmp648;}),({struct Cyc_PP_Doc*_tmp647=Cyc_PP_text(*((struct _fat_ptr*)cl->hd));_tmp206[2]=_tmp647;}),({struct Cyc_PP_Doc*_tmp646=Cyc_PP_text(({const char*_tmp208="\"";_tag_fat(_tmp208,sizeof(char),2U);}));_tmp206[3]=_tmp646;});Cyc_PP_cat(_tag_fat(_tmp206,sizeof(struct Cyc_PP_Doc*),4U));});
cl=cl->tl;
if((unsigned)cl)
s=({struct Cyc_PP_Doc*_tmp209[2U];_tmp209[0]=s,({struct Cyc_PP_Doc*_tmp649=Cyc_PP_text(({const char*_tmp20A=", ";_tag_fat(_tmp20A,sizeof(char),3U);}));_tmp209[1]=_tmp649;});Cyc_PP_cat(_tag_fat(_tmp209,sizeof(struct Cyc_PP_Doc*),2U));});}}
# 1105
return s;}
# 607 "absynpp.cyc"
struct Cyc_PP_Doc*Cyc_Absynpp_exp2doc_prec(int inprec,struct Cyc_Absyn_Exp*e){
# 1128 "absynpp.cyc"
int myprec=Cyc_Absynpp_exp_prec(e);
struct Cyc_PP_Doc*s;
{void*_stmttmp16=e->r;void*_tmp20B=_stmttmp16;struct Cyc_Absyn_Stmt*_tmp20C;struct Cyc_List_List*_tmp20E;struct Cyc_Core_Opt*_tmp20D;struct Cyc_Absyn_Exp*_tmp210;struct Cyc_Absyn_Exp*_tmp20F;int _tmp215;struct Cyc_Absyn_Exp*_tmp214;void**_tmp213;struct Cyc_Absyn_Exp*_tmp212;int _tmp211;struct Cyc_Absyn_Exp*_tmp217;struct Cyc_Absyn_Exp*_tmp216;struct Cyc_Absyn_Enumfield*_tmp218;struct Cyc_Absyn_Enumfield*_tmp219;struct Cyc_Absyn_Datatypefield*_tmp21B;struct Cyc_List_List*_tmp21A;struct Cyc_List_List*_tmp21C;struct Cyc_List_List*_tmp21F;struct Cyc_List_List*_tmp21E;struct _tuple0*_tmp21D;void*_tmp221;struct Cyc_Absyn_Exp*_tmp220;struct Cyc_Absyn_Exp*_tmp224;struct Cyc_Absyn_Exp*_tmp223;struct Cyc_Absyn_Vardecl*_tmp222;struct Cyc_List_List*_tmp225;struct Cyc_List_List*_tmp227;struct _tuple8*_tmp226;struct Cyc_List_List*_tmp228;struct Cyc_Absyn_Exp*_tmp22A;struct Cyc_Absyn_Exp*_tmp229;struct _fat_ptr*_tmp22C;struct Cyc_Absyn_Exp*_tmp22B;struct _fat_ptr*_tmp22E;struct Cyc_Absyn_Exp*_tmp22D;struct Cyc_Absyn_Exp*_tmp22F;struct Cyc_List_List*_tmp231;void*_tmp230;struct _fat_ptr*_tmp233;struct Cyc_Absyn_Exp*_tmp232;struct Cyc_List_List*_tmp238;struct Cyc_List_List*_tmp237;struct Cyc_List_List*_tmp236;struct _fat_ptr _tmp235;int _tmp234;void*_tmp239;struct Cyc_Absyn_Exp*_tmp23A;struct Cyc_Absyn_Exp*_tmp23B;void*_tmp23C;struct Cyc_Absyn_Exp*_tmp23E;struct Cyc_Absyn_Exp*_tmp23D;struct Cyc_Absyn_Exp*_tmp23F;struct Cyc_Absyn_Exp*_tmp241;void*_tmp240;struct Cyc_Absyn_Exp*_tmp242;struct Cyc_Absyn_Exp*_tmp243;struct Cyc_Absyn_Exp*_tmp244;struct Cyc_List_List*_tmp246;struct Cyc_Absyn_Exp*_tmp245;struct Cyc_Absyn_Exp*_tmp248;struct Cyc_Absyn_Exp*_tmp247;struct Cyc_Absyn_Exp*_tmp24A;struct Cyc_Absyn_Exp*_tmp249;struct Cyc_Absyn_Exp*_tmp24C;struct Cyc_Absyn_Exp*_tmp24B;struct Cyc_Absyn_Exp*_tmp24F;struct Cyc_Absyn_Exp*_tmp24E;struct Cyc_Absyn_Exp*_tmp24D;enum Cyc_Absyn_Incrementor _tmp251;struct Cyc_Absyn_Exp*_tmp250;struct Cyc_Absyn_Exp*_tmp254;struct Cyc_Core_Opt*_tmp253;struct Cyc_Absyn_Exp*_tmp252;struct Cyc_List_List*_tmp256;enum Cyc_Absyn_Primop _tmp255;struct _fat_ptr _tmp257;void*_tmp258;union Cyc_Absyn_Cnst _tmp259;switch(*((int*)_tmp20B)){case 0U: _LL1: _tmp259=((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmp20B)->f1;_LL2: {union Cyc_Absyn_Cnst c=_tmp259;
s=Cyc_Absynpp_cnst2doc(c);goto _LL0;}case 1U: _LL3: _tmp258=(void*)((struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct*)_tmp20B)->f1;_LL4: {void*b=_tmp258;
s=Cyc_Absynpp_qvar2doc(Cyc_Absyn_binding2qvar(b));goto _LL0;}case 2U: _LL5: _tmp257=((struct Cyc_Absyn_Pragma_e_Absyn_Raw_exp_struct*)_tmp20B)->f1;_LL6: {struct _fat_ptr p=_tmp257;
# 1134
s=({struct Cyc_PP_Doc*_tmp25A[4U];({struct Cyc_PP_Doc*_tmp64D=Cyc_PP_text(({const char*_tmp25B="__cyclone_pragma__";_tag_fat(_tmp25B,sizeof(char),19U);}));_tmp25A[0]=_tmp64D;}),({struct Cyc_PP_Doc*_tmp64C=Cyc_PP_text(({const char*_tmp25C="(";_tag_fat(_tmp25C,sizeof(char),2U);}));_tmp25A[1]=_tmp64C;}),({struct Cyc_PP_Doc*_tmp64B=Cyc_PP_text(p);_tmp25A[2]=_tmp64B;}),({struct Cyc_PP_Doc*_tmp64A=Cyc_PP_text(({const char*_tmp25D=")";_tag_fat(_tmp25D,sizeof(char),2U);}));_tmp25A[3]=_tmp64A;});Cyc_PP_cat(_tag_fat(_tmp25A,sizeof(struct Cyc_PP_Doc*),4U));});goto _LL0;}case 3U: _LL7: _tmp255=((struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct*)_tmp20B)->f1;_tmp256=((struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct*)_tmp20B)->f2;_LL8: {enum Cyc_Absyn_Primop p=_tmp255;struct Cyc_List_List*es=_tmp256;
s=Cyc_Absynpp_primapp2doc(myprec,p,es);goto _LL0;}case 4U: _LL9: _tmp252=((struct Cyc_Absyn_AssignOp_e_Absyn_Raw_exp_struct*)_tmp20B)->f1;_tmp253=((struct Cyc_Absyn_AssignOp_e_Absyn_Raw_exp_struct*)_tmp20B)->f2;_tmp254=((struct Cyc_Absyn_AssignOp_e_Absyn_Raw_exp_struct*)_tmp20B)->f3;_LLA: {struct Cyc_Absyn_Exp*e1=_tmp252;struct Cyc_Core_Opt*popt=_tmp253;struct Cyc_Absyn_Exp*e2=_tmp254;
# 1137
s=({struct Cyc_PP_Doc*_tmp25E[5U];({struct Cyc_PP_Doc*_tmp653=Cyc_Absynpp_exp2doc_prec(myprec,e1);_tmp25E[0]=_tmp653;}),({
struct Cyc_PP_Doc*_tmp652=Cyc_PP_text(({const char*_tmp25F=" ";_tag_fat(_tmp25F,sizeof(char),2U);}));_tmp25E[1]=_tmp652;}),
popt == 0?({struct Cyc_PP_Doc*_tmp651=Cyc_PP_nil_doc();_tmp25E[2]=_tmp651;}):({struct Cyc_PP_Doc*_tmp650=Cyc_Absynpp_prim2doc((enum Cyc_Absyn_Primop)popt->v);_tmp25E[2]=_tmp650;}),({
struct Cyc_PP_Doc*_tmp64F=Cyc_PP_text(({const char*_tmp260="= ";_tag_fat(_tmp260,sizeof(char),3U);}));_tmp25E[3]=_tmp64F;}),({
struct Cyc_PP_Doc*_tmp64E=Cyc_Absynpp_exp2doc_prec(myprec,e2);_tmp25E[4]=_tmp64E;});Cyc_PP_cat(_tag_fat(_tmp25E,sizeof(struct Cyc_PP_Doc*),5U));});
goto _LL0;}case 5U: _LLB: _tmp250=((struct Cyc_Absyn_Increment_e_Absyn_Raw_exp_struct*)_tmp20B)->f1;_tmp251=((struct Cyc_Absyn_Increment_e_Absyn_Raw_exp_struct*)_tmp20B)->f2;_LLC: {struct Cyc_Absyn_Exp*e2=_tmp250;enum Cyc_Absyn_Incrementor i=_tmp251;
# 1144
struct Cyc_PP_Doc*es=Cyc_Absynpp_exp2doc_prec(myprec,e2);
{enum Cyc_Absyn_Incrementor _tmp261=i;switch(_tmp261){case Cyc_Absyn_PreInc: _LL58: _LL59:
 s=({struct Cyc_PP_Doc*_tmp262[2U];({struct Cyc_PP_Doc*_tmp654=Cyc_PP_text(({const char*_tmp263="++";_tag_fat(_tmp263,sizeof(char),3U);}));_tmp262[0]=_tmp654;}),_tmp262[1]=es;Cyc_PP_cat(_tag_fat(_tmp262,sizeof(struct Cyc_PP_Doc*),2U));});goto _LL57;case Cyc_Absyn_PreDec: _LL5A: _LL5B:
 s=({struct Cyc_PP_Doc*_tmp264[2U];({struct Cyc_PP_Doc*_tmp655=Cyc_PP_text(({const char*_tmp265="--";_tag_fat(_tmp265,sizeof(char),3U);}));_tmp264[0]=_tmp655;}),_tmp264[1]=es;Cyc_PP_cat(_tag_fat(_tmp264,sizeof(struct Cyc_PP_Doc*),2U));});goto _LL57;case Cyc_Absyn_PostInc: _LL5C: _LL5D:
 s=({struct Cyc_PP_Doc*_tmp266[2U];_tmp266[0]=es,({struct Cyc_PP_Doc*_tmp656=Cyc_PP_text(({const char*_tmp267="++";_tag_fat(_tmp267,sizeof(char),3U);}));_tmp266[1]=_tmp656;});Cyc_PP_cat(_tag_fat(_tmp266,sizeof(struct Cyc_PP_Doc*),2U));});goto _LL57;case Cyc_Absyn_PostDec: _LL5E: _LL5F:
 goto _LL61;default: _LL60: _LL61:
 s=({struct Cyc_PP_Doc*_tmp268[2U];_tmp268[0]=es,({struct Cyc_PP_Doc*_tmp657=Cyc_PP_text(({const char*_tmp269="--";_tag_fat(_tmp269,sizeof(char),3U);}));_tmp268[1]=_tmp657;});Cyc_PP_cat(_tag_fat(_tmp268,sizeof(struct Cyc_PP_Doc*),2U));});goto _LL57;}_LL57:;}
# 1152
goto _LL0;}case 6U: _LLD: _tmp24D=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmp20B)->f1;_tmp24E=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmp20B)->f2;_tmp24F=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmp20B)->f3;_LLE: {struct Cyc_Absyn_Exp*e1=_tmp24D;struct Cyc_Absyn_Exp*e2=_tmp24E;struct Cyc_Absyn_Exp*e3=_tmp24F;
# 1154
s=({struct Cyc_PP_Doc*_tmp26A[5U];({struct Cyc_PP_Doc*_tmp65C=Cyc_Absynpp_exp2doc_prec(myprec,e1);_tmp26A[0]=_tmp65C;}),({struct Cyc_PP_Doc*_tmp65B=Cyc_PP_text(({const char*_tmp26B=" ? ";_tag_fat(_tmp26B,sizeof(char),4U);}));_tmp26A[1]=_tmp65B;}),({struct Cyc_PP_Doc*_tmp65A=Cyc_Absynpp_exp2doc_prec(0,e2);_tmp26A[2]=_tmp65A;}),({
struct Cyc_PP_Doc*_tmp659=Cyc_PP_text(({const char*_tmp26C=" : ";_tag_fat(_tmp26C,sizeof(char),4U);}));_tmp26A[3]=_tmp659;}),({struct Cyc_PP_Doc*_tmp658=Cyc_Absynpp_exp2doc_prec(myprec,e3);_tmp26A[4]=_tmp658;});Cyc_PP_cat(_tag_fat(_tmp26A,sizeof(struct Cyc_PP_Doc*),5U));});
goto _LL0;}case 7U: _LLF: _tmp24B=((struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct*)_tmp20B)->f1;_tmp24C=((struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct*)_tmp20B)->f2;_LL10: {struct Cyc_Absyn_Exp*e1=_tmp24B;struct Cyc_Absyn_Exp*e2=_tmp24C;
# 1158
s=({struct Cyc_PP_Doc*_tmp26D[3U];({struct Cyc_PP_Doc*_tmp65F=Cyc_Absynpp_exp2doc_prec(myprec,e1);_tmp26D[0]=_tmp65F;}),({struct Cyc_PP_Doc*_tmp65E=Cyc_PP_text(({const char*_tmp26E=" && ";_tag_fat(_tmp26E,sizeof(char),5U);}));_tmp26D[1]=_tmp65E;}),({struct Cyc_PP_Doc*_tmp65D=Cyc_Absynpp_exp2doc_prec(myprec,e2);_tmp26D[2]=_tmp65D;});Cyc_PP_cat(_tag_fat(_tmp26D,sizeof(struct Cyc_PP_Doc*),3U));});
goto _LL0;}case 8U: _LL11: _tmp249=((struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct*)_tmp20B)->f1;_tmp24A=((struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct*)_tmp20B)->f2;_LL12: {struct Cyc_Absyn_Exp*e1=_tmp249;struct Cyc_Absyn_Exp*e2=_tmp24A;
# 1161
s=({struct Cyc_PP_Doc*_tmp26F[3U];({struct Cyc_PP_Doc*_tmp662=Cyc_Absynpp_exp2doc_prec(myprec,e1);_tmp26F[0]=_tmp662;}),({struct Cyc_PP_Doc*_tmp661=Cyc_PP_text(({const char*_tmp270=" || ";_tag_fat(_tmp270,sizeof(char),5U);}));_tmp26F[1]=_tmp661;}),({struct Cyc_PP_Doc*_tmp660=Cyc_Absynpp_exp2doc_prec(myprec,e2);_tmp26F[2]=_tmp660;});Cyc_PP_cat(_tag_fat(_tmp26F,sizeof(struct Cyc_PP_Doc*),3U));});
goto _LL0;}case 9U: _LL13: _tmp247=((struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct*)_tmp20B)->f1;_tmp248=((struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct*)_tmp20B)->f2;_LL14: {struct Cyc_Absyn_Exp*e1=_tmp247;struct Cyc_Absyn_Exp*e2=_tmp248;
# 1166
s=({struct Cyc_PP_Doc*_tmp271[3U];({struct Cyc_PP_Doc*_tmp665=Cyc_Absynpp_exp2doc_prec(myprec - 1,e1);_tmp271[0]=_tmp665;}),({struct Cyc_PP_Doc*_tmp664=Cyc_PP_text(({const char*_tmp272=", ";_tag_fat(_tmp272,sizeof(char),3U);}));_tmp271[1]=_tmp664;}),({struct Cyc_PP_Doc*_tmp663=Cyc_Absynpp_exp2doc_prec(myprec - 1,e2);_tmp271[2]=_tmp663;});Cyc_PP_cat(_tag_fat(_tmp271,sizeof(struct Cyc_PP_Doc*),3U));});
goto _LL0;}case 10U: _LL15: _tmp245=((struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct*)_tmp20B)->f1;_tmp246=((struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct*)_tmp20B)->f2;_LL16: {struct Cyc_Absyn_Exp*e1=_tmp245;struct Cyc_List_List*es=_tmp246;
# 1169
s=({struct Cyc_PP_Doc*_tmp273[4U];({struct Cyc_PP_Doc*_tmp669=Cyc_Absynpp_exp2doc_prec(myprec,e1);_tmp273[0]=_tmp669;}),({
struct Cyc_PP_Doc*_tmp668=Cyc_PP_text(({const char*_tmp274="(";_tag_fat(_tmp274,sizeof(char),2U);}));_tmp273[1]=_tmp668;}),({
struct Cyc_PP_Doc*_tmp667=Cyc_Absynpp_exps2doc_prec(20,es);_tmp273[2]=_tmp667;}),({
struct Cyc_PP_Doc*_tmp666=Cyc_PP_text(({const char*_tmp275=")";_tag_fat(_tmp275,sizeof(char),2U);}));_tmp273[3]=_tmp666;});Cyc_PP_cat(_tag_fat(_tmp273,sizeof(struct Cyc_PP_Doc*),4U));});
goto _LL0;}case 11U: _LL17: _tmp244=((struct Cyc_Absyn_Throw_e_Absyn_Raw_exp_struct*)_tmp20B)->f1;_LL18: {struct Cyc_Absyn_Exp*e1=_tmp244;
s=({struct Cyc_PP_Doc*_tmp276[2U];({struct Cyc_PP_Doc*_tmp66B=Cyc_PP_text(({const char*_tmp277="throw ";_tag_fat(_tmp277,sizeof(char),7U);}));_tmp276[0]=_tmp66B;}),({struct Cyc_PP_Doc*_tmp66A=Cyc_Absynpp_exp2doc_prec(myprec,e1);_tmp276[1]=_tmp66A;});Cyc_PP_cat(_tag_fat(_tmp276,sizeof(struct Cyc_PP_Doc*),2U));});goto _LL0;}case 12U: _LL19: _tmp243=((struct Cyc_Absyn_NoInstantiate_e_Absyn_Raw_exp_struct*)_tmp20B)->f1;_LL1A: {struct Cyc_Absyn_Exp*e1=_tmp243;
s=Cyc_Absynpp_exp2doc_prec(inprec,e1);goto _LL0;}case 13U: _LL1B: _tmp242=((struct Cyc_Absyn_Instantiate_e_Absyn_Raw_exp_struct*)_tmp20B)->f1;_LL1C: {struct Cyc_Absyn_Exp*e1=_tmp242;
s=Cyc_Absynpp_exp2doc_prec(inprec,e1);goto _LL0;}case 14U: _LL1D: _tmp240=(void*)((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp20B)->f1;_tmp241=((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp20B)->f2;_LL1E: {void*t=_tmp240;struct Cyc_Absyn_Exp*e1=_tmp241;
# 1178
s=({struct Cyc_PP_Doc*_tmp278[4U];({struct Cyc_PP_Doc*_tmp66F=Cyc_PP_text(({const char*_tmp279="(";_tag_fat(_tmp279,sizeof(char),2U);}));_tmp278[0]=_tmp66F;}),({struct Cyc_PP_Doc*_tmp66E=Cyc_Absynpp_typ2doc(t);_tmp278[1]=_tmp66E;}),({struct Cyc_PP_Doc*_tmp66D=Cyc_PP_text(({const char*_tmp27A=")";_tag_fat(_tmp27A,sizeof(char),2U);}));_tmp278[2]=_tmp66D;}),({struct Cyc_PP_Doc*_tmp66C=Cyc_Absynpp_exp2doc_prec(myprec,e1);_tmp278[3]=_tmp66C;});Cyc_PP_cat(_tag_fat(_tmp278,sizeof(struct Cyc_PP_Doc*),4U));});goto _LL0;}case 15U: _LL1F: _tmp23F=((struct Cyc_Absyn_Address_e_Absyn_Raw_exp_struct*)_tmp20B)->f1;_LL20: {struct Cyc_Absyn_Exp*e1=_tmp23F;
s=({struct Cyc_PP_Doc*_tmp27B[2U];({struct Cyc_PP_Doc*_tmp671=Cyc_PP_text(({const char*_tmp27C="&";_tag_fat(_tmp27C,sizeof(char),2U);}));_tmp27B[0]=_tmp671;}),({struct Cyc_PP_Doc*_tmp670=Cyc_Absynpp_exp2doc_prec(myprec,e1);_tmp27B[1]=_tmp670;});Cyc_PP_cat(_tag_fat(_tmp27B,sizeof(struct Cyc_PP_Doc*),2U));});goto _LL0;}case 16U: _LL21: _tmp23D=((struct Cyc_Absyn_New_e_Absyn_Raw_exp_struct*)_tmp20B)->f1;_tmp23E=((struct Cyc_Absyn_New_e_Absyn_Raw_exp_struct*)_tmp20B)->f2;_LL22: {struct Cyc_Absyn_Exp*ropt=_tmp23D;struct Cyc_Absyn_Exp*e1=_tmp23E;
# 1181
if(ropt == 0)
s=({struct Cyc_PP_Doc*_tmp27D[2U];({struct Cyc_PP_Doc*_tmp673=Cyc_PP_text(({const char*_tmp27E="new ";_tag_fat(_tmp27E,sizeof(char),5U);}));_tmp27D[0]=_tmp673;}),({struct Cyc_PP_Doc*_tmp672=Cyc_Absynpp_exp2doc_prec(myprec,e1);_tmp27D[1]=_tmp672;});Cyc_PP_cat(_tag_fat(_tmp27D,sizeof(struct Cyc_PP_Doc*),2U));});else{
# 1184
s=({struct Cyc_PP_Doc*_tmp27F[4U];({struct Cyc_PP_Doc*_tmp677=Cyc_PP_text(({const char*_tmp280="rnew(";_tag_fat(_tmp280,sizeof(char),6U);}));_tmp27F[0]=_tmp677;}),({struct Cyc_PP_Doc*_tmp676=Cyc_Absynpp_exp2doc(ropt);_tmp27F[1]=_tmp676;}),({struct Cyc_PP_Doc*_tmp675=Cyc_PP_text(({const char*_tmp281=") ";_tag_fat(_tmp281,sizeof(char),3U);}));_tmp27F[2]=_tmp675;}),({struct Cyc_PP_Doc*_tmp674=Cyc_Absynpp_exp2doc_prec(myprec,e1);_tmp27F[3]=_tmp674;});Cyc_PP_cat(_tag_fat(_tmp27F,sizeof(struct Cyc_PP_Doc*),4U));});}
goto _LL0;}case 17U: _LL23: _tmp23C=(void*)((struct Cyc_Absyn_Sizeoftype_e_Absyn_Raw_exp_struct*)_tmp20B)->f1;_LL24: {void*t=_tmp23C;
s=({struct Cyc_PP_Doc*_tmp282[3U];({struct Cyc_PP_Doc*_tmp67A=Cyc_PP_text(({const char*_tmp283="sizeof(";_tag_fat(_tmp283,sizeof(char),8U);}));_tmp282[0]=_tmp67A;}),({struct Cyc_PP_Doc*_tmp679=Cyc_Absynpp_typ2doc(t);_tmp282[1]=_tmp679;}),({struct Cyc_PP_Doc*_tmp678=Cyc_PP_text(({const char*_tmp284=")";_tag_fat(_tmp284,sizeof(char),2U);}));_tmp282[2]=_tmp678;});Cyc_PP_cat(_tag_fat(_tmp282,sizeof(struct Cyc_PP_Doc*),3U));});goto _LL0;}case 18U: _LL25: _tmp23B=((struct Cyc_Absyn_Sizeofexp_e_Absyn_Raw_exp_struct*)_tmp20B)->f1;_LL26: {struct Cyc_Absyn_Exp*e1=_tmp23B;
s=({struct Cyc_PP_Doc*_tmp285[3U];({struct Cyc_PP_Doc*_tmp67D=Cyc_PP_text(({const char*_tmp286="sizeof(";_tag_fat(_tmp286,sizeof(char),8U);}));_tmp285[0]=_tmp67D;}),({struct Cyc_PP_Doc*_tmp67C=Cyc_Absynpp_exp2doc(e1);_tmp285[1]=_tmp67C;}),({struct Cyc_PP_Doc*_tmp67B=Cyc_PP_text(({const char*_tmp287=")";_tag_fat(_tmp287,sizeof(char),2U);}));_tmp285[2]=_tmp67B;});Cyc_PP_cat(_tag_fat(_tmp285,sizeof(struct Cyc_PP_Doc*),3U));});goto _LL0;}case 42U: _LL27: _tmp23A=((struct Cyc_Absyn_Extension_e_Absyn_Raw_exp_struct*)_tmp20B)->f1;_LL28: {struct Cyc_Absyn_Exp*e=_tmp23A;
# 1189
s=({struct Cyc_PP_Doc*_tmp288[3U];({struct Cyc_PP_Doc*_tmp680=Cyc_PP_text(({const char*_tmp289="__extension__(";_tag_fat(_tmp289,sizeof(char),15U);}));_tmp288[0]=_tmp680;}),({struct Cyc_PP_Doc*_tmp67F=Cyc_Absynpp_exp2doc(e);_tmp288[1]=_tmp67F;}),({struct Cyc_PP_Doc*_tmp67E=Cyc_PP_text(({const char*_tmp28A=")";_tag_fat(_tmp28A,sizeof(char),2U);}));_tmp288[2]=_tmp67E;});Cyc_PP_cat(_tag_fat(_tmp288,sizeof(struct Cyc_PP_Doc*),3U));});goto _LL0;}case 40U: _LL29: _tmp239=(void*)((struct Cyc_Absyn_Valueof_e_Absyn_Raw_exp_struct*)_tmp20B)->f1;_LL2A: {void*t=_tmp239;
s=({struct Cyc_PP_Doc*_tmp28B[3U];({struct Cyc_PP_Doc*_tmp683=Cyc_PP_text(({const char*_tmp28C="valueof(";_tag_fat(_tmp28C,sizeof(char),9U);}));_tmp28B[0]=_tmp683;}),({struct Cyc_PP_Doc*_tmp682=Cyc_Absynpp_typ2doc(t);_tmp28B[1]=_tmp682;}),({struct Cyc_PP_Doc*_tmp681=Cyc_PP_text(({const char*_tmp28D=")";_tag_fat(_tmp28D,sizeof(char),2U);}));_tmp28B[2]=_tmp681;});Cyc_PP_cat(_tag_fat(_tmp28B,sizeof(struct Cyc_PP_Doc*),3U));});goto _LL0;}case 41U: _LL2B: _tmp234=((struct Cyc_Absyn_Asm_e_Absyn_Raw_exp_struct*)_tmp20B)->f1;_tmp235=((struct Cyc_Absyn_Asm_e_Absyn_Raw_exp_struct*)_tmp20B)->f2;_tmp236=((struct Cyc_Absyn_Asm_e_Absyn_Raw_exp_struct*)_tmp20B)->f3;_tmp237=((struct Cyc_Absyn_Asm_e_Absyn_Raw_exp_struct*)_tmp20B)->f4;_tmp238=((struct Cyc_Absyn_Asm_e_Absyn_Raw_exp_struct*)_tmp20B)->f5;_LL2C: {int vol=_tmp234;struct _fat_ptr t=_tmp235;struct Cyc_List_List*o=_tmp236;struct Cyc_List_List*i=_tmp237;struct Cyc_List_List*cl=_tmp238;
# 1192
if(vol)
s=({struct Cyc_PP_Doc*_tmp28E[7U];({struct Cyc_PP_Doc*_tmp68A=Cyc_PP_text(({const char*_tmp28F="__asm__";_tag_fat(_tmp28F,sizeof(char),8U);}));_tmp28E[0]=_tmp68A;}),({struct Cyc_PP_Doc*_tmp689=Cyc_PP_text(({const char*_tmp290=" volatile (";_tag_fat(_tmp290,sizeof(char),12U);}));_tmp28E[1]=_tmp689;}),({struct Cyc_PP_Doc*_tmp688=Cyc_PP_text(({const char*_tmp291="\"";_tag_fat(_tmp291,sizeof(char),2U);}));_tmp28E[2]=_tmp688;}),({struct Cyc_PP_Doc*_tmp687=Cyc_PP_text(t);_tmp28E[3]=_tmp687;}),({struct Cyc_PP_Doc*_tmp686=Cyc_PP_text(({const char*_tmp292="\"";_tag_fat(_tmp292,sizeof(char),2U);}));_tmp28E[4]=_tmp686;}),({struct Cyc_PP_Doc*_tmp685=Cyc_Absynpp_asm_iolist_doc(o,i,cl);_tmp28E[5]=_tmp685;}),({struct Cyc_PP_Doc*_tmp684=Cyc_PP_text(({const char*_tmp293=")";_tag_fat(_tmp293,sizeof(char),2U);}));_tmp28E[6]=_tmp684;});Cyc_PP_cat(_tag_fat(_tmp28E,sizeof(struct Cyc_PP_Doc*),7U));});else{
# 1195
s=({struct Cyc_PP_Doc*_tmp294[6U];({struct Cyc_PP_Doc*_tmp690=Cyc_PP_text(({const char*_tmp295="__asm__(";_tag_fat(_tmp295,sizeof(char),9U);}));_tmp294[0]=_tmp690;}),({struct Cyc_PP_Doc*_tmp68F=Cyc_PP_text(({const char*_tmp296="\"";_tag_fat(_tmp296,sizeof(char),2U);}));_tmp294[1]=_tmp68F;}),({struct Cyc_PP_Doc*_tmp68E=Cyc_PP_text(t);_tmp294[2]=_tmp68E;}),({struct Cyc_PP_Doc*_tmp68D=Cyc_PP_text(({const char*_tmp297="\"";_tag_fat(_tmp297,sizeof(char),2U);}));_tmp294[3]=_tmp68D;}),({struct Cyc_PP_Doc*_tmp68C=Cyc_Absynpp_asm_iolist_doc(o,i,cl);_tmp294[4]=_tmp68C;}),({struct Cyc_PP_Doc*_tmp68B=Cyc_PP_text(({const char*_tmp298=")";_tag_fat(_tmp298,sizeof(char),2U);}));_tmp294[5]=_tmp68B;});Cyc_PP_cat(_tag_fat(_tmp294,sizeof(struct Cyc_PP_Doc*),6U));});}
goto _LL0;}case 39U: _LL2D: _tmp232=((struct Cyc_Absyn_Tagcheck_e_Absyn_Raw_exp_struct*)_tmp20B)->f1;_tmp233=((struct Cyc_Absyn_Tagcheck_e_Absyn_Raw_exp_struct*)_tmp20B)->f2;_LL2E: {struct Cyc_Absyn_Exp*e=_tmp232;struct _fat_ptr*f=_tmp233;
# 1198
s=({struct Cyc_PP_Doc*_tmp299[5U];({struct Cyc_PP_Doc*_tmp695=Cyc_PP_text(({const char*_tmp29A="tagcheck(";_tag_fat(_tmp29A,sizeof(char),10U);}));_tmp299[0]=_tmp695;}),({struct Cyc_PP_Doc*_tmp694=Cyc_Absynpp_exp2doc(e);_tmp299[1]=_tmp694;}),({struct Cyc_PP_Doc*_tmp693=Cyc_PP_text(({const char*_tmp29B=".";_tag_fat(_tmp29B,sizeof(char),2U);}));_tmp299[2]=_tmp693;}),({struct Cyc_PP_Doc*_tmp692=Cyc_PP_textptr(f);_tmp299[3]=_tmp692;}),({struct Cyc_PP_Doc*_tmp691=Cyc_PP_text(({const char*_tmp29C=")";_tag_fat(_tmp29C,sizeof(char),2U);}));_tmp299[4]=_tmp691;});Cyc_PP_cat(_tag_fat(_tmp299,sizeof(struct Cyc_PP_Doc*),5U));});
goto _LL0;}case 19U: _LL2F: _tmp230=(void*)((struct Cyc_Absyn_Offsetof_e_Absyn_Raw_exp_struct*)_tmp20B)->f1;_tmp231=((struct Cyc_Absyn_Offsetof_e_Absyn_Raw_exp_struct*)_tmp20B)->f2;_LL30: {void*t=_tmp230;struct Cyc_List_List*l=_tmp231;
# 1201
s=({struct Cyc_PP_Doc*_tmp29D[5U];({struct Cyc_PP_Doc*_tmp69C=Cyc_PP_text(({const char*_tmp29E="offsetof(";_tag_fat(_tmp29E,sizeof(char),10U);}));_tmp29D[0]=_tmp69C;}),({struct Cyc_PP_Doc*_tmp69B=Cyc_Absynpp_typ2doc(t);_tmp29D[1]=_tmp69B;}),({struct Cyc_PP_Doc*_tmp69A=Cyc_PP_text(({const char*_tmp29F=",";_tag_fat(_tmp29F,sizeof(char),2U);}));_tmp29D[2]=_tmp69A;}),({
struct Cyc_PP_Doc*_tmp699=({struct _fat_ptr _tmp698=({const char*_tmp2A0=".";_tag_fat(_tmp2A0,sizeof(char),2U);});Cyc_PP_seq(_tmp698,({struct Cyc_List_List*(*_tmp697)(struct Cyc_PP_Doc*(*f)(void*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp2A1)(struct Cyc_PP_Doc*(*f)(void*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_PP_Doc*(*f)(void*),struct Cyc_List_List*x))Cyc_List_map;_tmp2A1;});_tmp697(Cyc_Absynpp_offsetof_field_to_doc,l);}));});_tmp29D[3]=_tmp699;}),({struct Cyc_PP_Doc*_tmp696=Cyc_PP_text(({const char*_tmp2A2=")";_tag_fat(_tmp2A2,sizeof(char),2U);}));_tmp29D[4]=_tmp696;});Cyc_PP_cat(_tag_fat(_tmp29D,sizeof(struct Cyc_PP_Doc*),5U));});
goto _LL0;}case 20U: _LL31: _tmp22F=((struct Cyc_Absyn_Deref_e_Absyn_Raw_exp_struct*)_tmp20B)->f1;_LL32: {struct Cyc_Absyn_Exp*e1=_tmp22F;
s=({struct Cyc_PP_Doc*_tmp2A3[2U];({struct Cyc_PP_Doc*_tmp69E=Cyc_PP_text(({const char*_tmp2A4="*";_tag_fat(_tmp2A4,sizeof(char),2U);}));_tmp2A3[0]=_tmp69E;}),({struct Cyc_PP_Doc*_tmp69D=Cyc_Absynpp_exp2doc_prec(myprec,e1);_tmp2A3[1]=_tmp69D;});Cyc_PP_cat(_tag_fat(_tmp2A3,sizeof(struct Cyc_PP_Doc*),2U));});goto _LL0;}case 21U: _LL33: _tmp22D=((struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct*)_tmp20B)->f1;_tmp22E=((struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct*)_tmp20B)->f2;_LL34: {struct Cyc_Absyn_Exp*e1=_tmp22D;struct _fat_ptr*n=_tmp22E;
# 1206
s=({struct Cyc_PP_Doc*_tmp2A5[3U];({struct Cyc_PP_Doc*_tmp6A1=Cyc_Absynpp_exp2doc_prec(myprec,e1);_tmp2A5[0]=_tmp6A1;}),({struct Cyc_PP_Doc*_tmp6A0=Cyc_PP_text(({const char*_tmp2A6=".";_tag_fat(_tmp2A6,sizeof(char),2U);}));_tmp2A5[1]=_tmp6A0;}),({struct Cyc_PP_Doc*_tmp69F=Cyc_PP_textptr(n);_tmp2A5[2]=_tmp69F;});Cyc_PP_cat(_tag_fat(_tmp2A5,sizeof(struct Cyc_PP_Doc*),3U));});goto _LL0;}case 22U: _LL35: _tmp22B=((struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct*)_tmp20B)->f1;_tmp22C=((struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct*)_tmp20B)->f2;_LL36: {struct Cyc_Absyn_Exp*e1=_tmp22B;struct _fat_ptr*n=_tmp22C;
# 1208
s=({struct Cyc_PP_Doc*_tmp2A7[3U];({struct Cyc_PP_Doc*_tmp6A4=Cyc_Absynpp_exp2doc_prec(myprec,e1);_tmp2A7[0]=_tmp6A4;}),({struct Cyc_PP_Doc*_tmp6A3=Cyc_PP_text(({const char*_tmp2A8="->";_tag_fat(_tmp2A8,sizeof(char),3U);}));_tmp2A7[1]=_tmp6A3;}),({struct Cyc_PP_Doc*_tmp6A2=Cyc_PP_textptr(n);_tmp2A7[2]=_tmp6A2;});Cyc_PP_cat(_tag_fat(_tmp2A7,sizeof(struct Cyc_PP_Doc*),3U));});goto _LL0;}case 23U: _LL37: _tmp229=((struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct*)_tmp20B)->f1;_tmp22A=((struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct*)_tmp20B)->f2;_LL38: {struct Cyc_Absyn_Exp*e1=_tmp229;struct Cyc_Absyn_Exp*e2=_tmp22A;
# 1210
s=({struct Cyc_PP_Doc*_tmp2A9[4U];({struct Cyc_PP_Doc*_tmp6A8=Cyc_Absynpp_exp2doc_prec(myprec,e1);_tmp2A9[0]=_tmp6A8;}),({struct Cyc_PP_Doc*_tmp6A7=Cyc_PP_text(({const char*_tmp2AA="[";_tag_fat(_tmp2AA,sizeof(char),2U);}));_tmp2A9[1]=_tmp6A7;}),({struct Cyc_PP_Doc*_tmp6A6=Cyc_Absynpp_exp2doc(e2);_tmp2A9[2]=_tmp6A6;}),({struct Cyc_PP_Doc*_tmp6A5=Cyc_PP_text(({const char*_tmp2AB="]";_tag_fat(_tmp2AB,sizeof(char),2U);}));_tmp2A9[3]=_tmp6A5;});Cyc_PP_cat(_tag_fat(_tmp2A9,sizeof(struct Cyc_PP_Doc*),4U));});goto _LL0;}case 24U: _LL39: _tmp228=((struct Cyc_Absyn_Tuple_e_Absyn_Raw_exp_struct*)_tmp20B)->f1;_LL3A: {struct Cyc_List_List*es=_tmp228;
# 1212
s=({struct Cyc_PP_Doc*_tmp2AC[4U];({struct Cyc_PP_Doc*_tmp6AC=Cyc_Absynpp_dollar();_tmp2AC[0]=_tmp6AC;}),({struct Cyc_PP_Doc*_tmp6AB=Cyc_PP_text(({const char*_tmp2AD="(";_tag_fat(_tmp2AD,sizeof(char),2U);}));_tmp2AC[1]=_tmp6AB;}),({struct Cyc_PP_Doc*_tmp6AA=Cyc_Absynpp_exps2doc_prec(20,es);_tmp2AC[2]=_tmp6AA;}),({struct Cyc_PP_Doc*_tmp6A9=Cyc_PP_text(({const char*_tmp2AE=")";_tag_fat(_tmp2AE,sizeof(char),2U);}));_tmp2AC[3]=_tmp6A9;});Cyc_PP_cat(_tag_fat(_tmp2AC,sizeof(struct Cyc_PP_Doc*),4U));});goto _LL0;}case 25U: _LL3B: _tmp226=((struct Cyc_Absyn_CompoundLit_e_Absyn_Raw_exp_struct*)_tmp20B)->f1;_tmp227=((struct Cyc_Absyn_CompoundLit_e_Absyn_Raw_exp_struct*)_tmp20B)->f2;_LL3C: {struct _tuple8*vat=_tmp226;struct Cyc_List_List*des=_tmp227;
# 1214
s=({struct Cyc_PP_Doc*_tmp2AF[4U];({struct Cyc_PP_Doc*_tmp6B2=Cyc_PP_text(({const char*_tmp2B0="(";_tag_fat(_tmp2B0,sizeof(char),2U);}));_tmp2AF[0]=_tmp6B2;}),({struct Cyc_PP_Doc*_tmp6B1=Cyc_Absynpp_typ2doc((*vat).f3);_tmp2AF[1]=_tmp6B1;}),({struct Cyc_PP_Doc*_tmp6B0=Cyc_PP_text(({const char*_tmp2B1=")";_tag_fat(_tmp2B1,sizeof(char),2U);}));_tmp2AF[2]=_tmp6B0;}),({
struct Cyc_PP_Doc*_tmp6AF=({struct _fat_ptr _tmp6AE=({const char*_tmp2B2=",";_tag_fat(_tmp2B2,sizeof(char),2U);});Cyc_Absynpp_group_braces(_tmp6AE,({struct Cyc_List_List*(*_tmp6AD)(struct Cyc_PP_Doc*(*f)(struct _tuple16*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp2B3)(struct Cyc_PP_Doc*(*f)(struct _tuple16*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_PP_Doc*(*f)(struct _tuple16*),struct Cyc_List_List*x))Cyc_List_map;_tmp2B3;});_tmp6AD(Cyc_Absynpp_de2doc,des);}));});_tmp2AF[3]=_tmp6AF;});Cyc_PP_cat(_tag_fat(_tmp2AF,sizeof(struct Cyc_PP_Doc*),4U));});
goto _LL0;}case 26U: _LL3D: _tmp225=((struct Cyc_Absyn_Array_e_Absyn_Raw_exp_struct*)_tmp20B)->f1;_LL3E: {struct Cyc_List_List*des=_tmp225;
s=({struct _fat_ptr _tmp6B4=({const char*_tmp2B4=",";_tag_fat(_tmp2B4,sizeof(char),2U);});Cyc_Absynpp_group_braces(_tmp6B4,({struct Cyc_List_List*(*_tmp6B3)(struct Cyc_PP_Doc*(*f)(struct _tuple16*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp2B5)(struct Cyc_PP_Doc*(*f)(struct _tuple16*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_PP_Doc*(*f)(struct _tuple16*),struct Cyc_List_List*x))Cyc_List_map;_tmp2B5;});_tmp6B3(Cyc_Absynpp_de2doc,des);}));});goto _LL0;}case 27U: _LL3F: _tmp222=((struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct*)_tmp20B)->f1;_tmp223=((struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct*)_tmp20B)->f2;_tmp224=((struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct*)_tmp20B)->f3;_LL40: {struct Cyc_Absyn_Vardecl*vd=_tmp222;struct Cyc_Absyn_Exp*e1=_tmp223;struct Cyc_Absyn_Exp*e2=_tmp224;
# 1219
s=({struct Cyc_PP_Doc*_tmp2B6[8U];({struct Cyc_PP_Doc*_tmp6BC=Cyc_Absynpp_lb();_tmp2B6[0]=_tmp6BC;}),({struct Cyc_PP_Doc*_tmp6BB=Cyc_PP_text(({const char*_tmp2B7="for ";_tag_fat(_tmp2B7,sizeof(char),5U);}));_tmp2B6[1]=_tmp6BB;}),({
struct Cyc_PP_Doc*_tmp6BA=Cyc_PP_text(*(*vd->name).f2);_tmp2B6[2]=_tmp6BA;}),({struct Cyc_PP_Doc*_tmp6B9=Cyc_PP_text(({const char*_tmp2B8=" < ";_tag_fat(_tmp2B8,sizeof(char),4U);}));_tmp2B6[3]=_tmp6B9;}),({struct Cyc_PP_Doc*_tmp6B8=Cyc_Absynpp_exp2doc(e1);_tmp2B6[4]=_tmp6B8;}),({struct Cyc_PP_Doc*_tmp6B7=Cyc_PP_text(({const char*_tmp2B9=" : ";_tag_fat(_tmp2B9,sizeof(char),4U);}));_tmp2B6[5]=_tmp6B7;}),({
struct Cyc_PP_Doc*_tmp6B6=Cyc_Absynpp_exp2doc(e2);_tmp2B6[6]=_tmp6B6;}),({struct Cyc_PP_Doc*_tmp6B5=Cyc_Absynpp_rb();_tmp2B6[7]=_tmp6B5;});Cyc_PP_cat(_tag_fat(_tmp2B6,sizeof(struct Cyc_PP_Doc*),8U));});
goto _LL0;}case 28U: _LL41: _tmp220=((struct Cyc_Absyn_ComprehensionNoinit_e_Absyn_Raw_exp_struct*)_tmp20B)->f1;_tmp221=(void*)((struct Cyc_Absyn_ComprehensionNoinit_e_Absyn_Raw_exp_struct*)_tmp20B)->f2;_LL42: {struct Cyc_Absyn_Exp*e=_tmp220;void*t=_tmp221;
# 1224
s=({struct Cyc_PP_Doc*_tmp2BA[7U];({struct Cyc_PP_Doc*_tmp6C3=Cyc_Absynpp_lb();_tmp2BA[0]=_tmp6C3;}),({struct Cyc_PP_Doc*_tmp6C2=Cyc_PP_text(({const char*_tmp2BB="for x ";_tag_fat(_tmp2BB,sizeof(char),7U);}));_tmp2BA[1]=_tmp6C2;}),({
struct Cyc_PP_Doc*_tmp6C1=Cyc_PP_text(({const char*_tmp2BC=" < ";_tag_fat(_tmp2BC,sizeof(char),4U);}));_tmp2BA[2]=_tmp6C1;}),({
struct Cyc_PP_Doc*_tmp6C0=Cyc_Absynpp_exp2doc(e);_tmp2BA[3]=_tmp6C0;}),({
struct Cyc_PP_Doc*_tmp6BF=Cyc_PP_text(({const char*_tmp2BD=" : ";_tag_fat(_tmp2BD,sizeof(char),4U);}));_tmp2BA[4]=_tmp6BF;}),({
struct Cyc_PP_Doc*_tmp6BE=Cyc_Absynpp_typ2doc(t);_tmp2BA[5]=_tmp6BE;}),({
struct Cyc_PP_Doc*_tmp6BD=Cyc_Absynpp_rb();_tmp2BA[6]=_tmp6BD;});Cyc_PP_cat(_tag_fat(_tmp2BA,sizeof(struct Cyc_PP_Doc*),7U));});
goto _LL0;}case 29U: _LL43: _tmp21D=((struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct*)_tmp20B)->f1;_tmp21E=((struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct*)_tmp20B)->f2;_tmp21F=((struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct*)_tmp20B)->f3;_LL44: {struct _tuple0*n=_tmp21D;struct Cyc_List_List*ts=_tmp21E;struct Cyc_List_List*des=_tmp21F;
# 1232
struct Cyc_List_List*des_doc=({struct Cyc_List_List*(*_tmp6C4)(struct Cyc_PP_Doc*(*f)(struct _tuple16*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp2C1)(struct Cyc_PP_Doc*(*f)(struct _tuple16*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_PP_Doc*(*f)(struct _tuple16*),struct Cyc_List_List*x))Cyc_List_map;_tmp2C1;});_tmp6C4(Cyc_Absynpp_de2doc,des);});
s=({struct Cyc_PP_Doc*_tmp2BE[2U];({struct Cyc_PP_Doc*_tmp6C8=Cyc_Absynpp_qvar2doc(n);_tmp2BE[0]=_tmp6C8;}),({
struct Cyc_PP_Doc*_tmp6C7=({struct _fat_ptr _tmp6C6=({const char*_tmp2BF=",";_tag_fat(_tmp2BF,sizeof(char),2U);});Cyc_Absynpp_group_braces(_tmp6C6,
ts != 0?({struct Cyc_List_List*_tmp2C0=_cycalloc(sizeof(*_tmp2C0));({struct Cyc_PP_Doc*_tmp6C5=Cyc_Absynpp_tps2doc(ts);_tmp2C0->hd=_tmp6C5;}),_tmp2C0->tl=des_doc;_tmp2C0;}): des_doc);});
# 1234
_tmp2BE[1]=_tmp6C7;});Cyc_PP_cat(_tag_fat(_tmp2BE,sizeof(struct Cyc_PP_Doc*),2U));});
# 1236
goto _LL0;}case 30U: _LL45: _tmp21C=((struct Cyc_Absyn_AnonStruct_e_Absyn_Raw_exp_struct*)_tmp20B)->f2;_LL46: {struct Cyc_List_List*des=_tmp21C;
s=({struct _fat_ptr _tmp6CA=({const char*_tmp2C2=",";_tag_fat(_tmp2C2,sizeof(char),2U);});Cyc_Absynpp_group_braces(_tmp6CA,({struct Cyc_List_List*(*_tmp6C9)(struct Cyc_PP_Doc*(*f)(struct _tuple16*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp2C3)(struct Cyc_PP_Doc*(*f)(struct _tuple16*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_PP_Doc*(*f)(struct _tuple16*),struct Cyc_List_List*x))Cyc_List_map;_tmp2C3;});_tmp6C9(Cyc_Absynpp_de2doc,des);}));});goto _LL0;}case 31U: _LL47: _tmp21A=((struct Cyc_Absyn_Datatype_e_Absyn_Raw_exp_struct*)_tmp20B)->f1;_tmp21B=((struct Cyc_Absyn_Datatype_e_Absyn_Raw_exp_struct*)_tmp20B)->f3;_LL48: {struct Cyc_List_List*es=_tmp21A;struct Cyc_Absyn_Datatypefield*tuf=_tmp21B;
# 1239
if(es == 0)
# 1241
s=Cyc_Absynpp_qvar2doc(tuf->name);else{
# 1243
s=({struct Cyc_PP_Doc*_tmp2C4[2U];({struct Cyc_PP_Doc*_tmp6D0=Cyc_Absynpp_qvar2doc(tuf->name);_tmp2C4[0]=_tmp6D0;}),({
struct Cyc_PP_Doc*_tmp6CF=({struct _fat_ptr _tmp6CE=({const char*_tmp2C5="(";_tag_fat(_tmp2C5,sizeof(char),2U);});struct _fat_ptr _tmp6CD=({const char*_tmp2C6=")";_tag_fat(_tmp2C6,sizeof(char),2U);});struct _fat_ptr _tmp6CC=({const char*_tmp2C7=",";_tag_fat(_tmp2C7,sizeof(char),2U);});Cyc_PP_egroup(_tmp6CE,_tmp6CD,_tmp6CC,({struct Cyc_List_List*(*_tmp6CB)(struct Cyc_PP_Doc*(*f)(struct Cyc_Absyn_Exp*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp2C8)(struct Cyc_PP_Doc*(*f)(struct Cyc_Absyn_Exp*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_PP_Doc*(*f)(struct Cyc_Absyn_Exp*),struct Cyc_List_List*x))Cyc_List_map;_tmp2C8;});_tmp6CB(Cyc_Absynpp_exp2doc,es);}));});_tmp2C4[1]=_tmp6CF;});Cyc_PP_cat(_tag_fat(_tmp2C4,sizeof(struct Cyc_PP_Doc*),2U));});}
goto _LL0;}case 32U: _LL49: _tmp219=((struct Cyc_Absyn_Enum_e_Absyn_Raw_exp_struct*)_tmp20B)->f2;_LL4A: {struct Cyc_Absyn_Enumfield*ef=_tmp219;
_tmp218=ef;goto _LL4C;}case 33U: _LL4B: _tmp218=((struct Cyc_Absyn_AnonEnum_e_Absyn_Raw_exp_struct*)_tmp20B)->f2;_LL4C: {struct Cyc_Absyn_Enumfield*ef=_tmp218;
s=Cyc_Absynpp_qvar2doc(ef->name);goto _LL0;}case 34U: _LL4D: _tmp216=((struct Cyc_Absyn_Spawn_e_Absyn_Raw_exp_struct*)_tmp20B)->f1;_tmp217=((struct Cyc_Absyn_Spawn_e_Absyn_Raw_exp_struct*)_tmp20B)->f2;_LL4E: {struct Cyc_Absyn_Exp*e1=_tmp216;struct Cyc_Absyn_Exp*e2=_tmp217;
# 1250
s=({struct Cyc_PP_Doc*_tmp2C9[5U];({struct Cyc_PP_Doc*_tmp6D5=Cyc_PP_text(({const char*_tmp2CA="spawn(";_tag_fat(_tmp2CA,sizeof(char),7U);}));_tmp2C9[0]=_tmp6D5;}),({struct Cyc_PP_Doc*_tmp6D4=Cyc_Absynpp_exp2doc(e1);_tmp2C9[1]=_tmp6D4;}),({struct Cyc_PP_Doc*_tmp6D3=Cyc_PP_text(({const char*_tmp2CB=",";_tag_fat(_tmp2CB,sizeof(char),2U);}));_tmp2C9[2]=_tmp6D3;}),({struct Cyc_PP_Doc*_tmp6D2=Cyc_Absynpp_exp2doc(e2);_tmp2C9[3]=_tmp6D2;}),({struct Cyc_PP_Doc*_tmp6D1=Cyc_PP_text(({const char*_tmp2CC=")";_tag_fat(_tmp2CC,sizeof(char),2U);}));_tmp2C9[4]=_tmp6D1;});Cyc_PP_cat(_tag_fat(_tmp2C9,sizeof(struct Cyc_PP_Doc*),5U));});
goto _LL0;}case 35U: _LL4F: _tmp211=(((struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct*)_tmp20B)->f1).is_calloc;_tmp212=(((struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct*)_tmp20B)->f1).rgn;_tmp213=(((struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct*)_tmp20B)->f1).elt_type;_tmp214=(((struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct*)_tmp20B)->f1).num_elts;_tmp215=(((struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct*)_tmp20B)->f1).inline_call;_LL50: {int is_calloc=_tmp211;struct Cyc_Absyn_Exp*rgnopt=_tmp212;void**topt=_tmp213;struct Cyc_Absyn_Exp*e=_tmp214;int inline_call=_tmp215;
# 1256
if(is_calloc){
# 1258
struct Cyc_Absyn_Exp*st=Cyc_Absyn_sizeoftype_exp(*((void**)_check_null(topt)),0U);
if(rgnopt == 0)
s=({struct Cyc_PP_Doc*_tmp2CD[5U];({struct Cyc_PP_Doc*_tmp6DA=Cyc_PP_text(({const char*_tmp2CE="calloc(";_tag_fat(_tmp2CE,sizeof(char),8U);}));_tmp2CD[0]=_tmp6DA;}),({struct Cyc_PP_Doc*_tmp6D9=Cyc_Absynpp_exp2doc(e);_tmp2CD[1]=_tmp6D9;}),({struct Cyc_PP_Doc*_tmp6D8=Cyc_PP_text(({const char*_tmp2CF=",";_tag_fat(_tmp2CF,sizeof(char),2U);}));_tmp2CD[2]=_tmp6D8;}),({struct Cyc_PP_Doc*_tmp6D7=Cyc_Absynpp_exp2doc(st);_tmp2CD[3]=_tmp6D7;}),({struct Cyc_PP_Doc*_tmp6D6=Cyc_PP_text(({const char*_tmp2D0=")";_tag_fat(_tmp2D0,sizeof(char),2U);}));_tmp2CD[4]=_tmp6D6;});Cyc_PP_cat(_tag_fat(_tmp2CD,sizeof(struct Cyc_PP_Doc*),5U));});else{
# 1262
s=({struct Cyc_PP_Doc*_tmp2D1[7U];({struct Cyc_PP_Doc*_tmp6E1=Cyc_PP_text(({const char*_tmp2D2="rcalloc(";_tag_fat(_tmp2D2,sizeof(char),9U);}));_tmp2D1[0]=_tmp6E1;}),({struct Cyc_PP_Doc*_tmp6E0=Cyc_Absynpp_exp2doc(rgnopt);_tmp2D1[1]=_tmp6E0;}),({struct Cyc_PP_Doc*_tmp6DF=Cyc_PP_text(({const char*_tmp2D3=",";_tag_fat(_tmp2D3,sizeof(char),2U);}));_tmp2D1[2]=_tmp6DF;}),({
struct Cyc_PP_Doc*_tmp6DE=Cyc_Absynpp_exp2doc(e);_tmp2D1[3]=_tmp6DE;}),({struct Cyc_PP_Doc*_tmp6DD=Cyc_PP_text(({const char*_tmp2D4=",";_tag_fat(_tmp2D4,sizeof(char),2U);}));_tmp2D1[4]=_tmp6DD;}),({struct Cyc_PP_Doc*_tmp6DC=Cyc_Absynpp_exp2doc(st);_tmp2D1[5]=_tmp6DC;}),({struct Cyc_PP_Doc*_tmp6DB=Cyc_PP_text(({const char*_tmp2D5=")";_tag_fat(_tmp2D5,sizeof(char),2U);}));_tmp2D1[6]=_tmp6DB;});Cyc_PP_cat(_tag_fat(_tmp2D1,sizeof(struct Cyc_PP_Doc*),7U));});}}else{
# 1265
struct Cyc_Absyn_Exp*new_e;
# 1267
if(topt == 0)
new_e=e;else{
# 1270
new_e=({struct Cyc_Absyn_Exp*_tmp6E2=Cyc_Absyn_sizeoftype_exp(*topt,0U);Cyc_Absyn_times_exp(_tmp6E2,e,0U);});}
# 1272
if(rgnopt == 0)
s=({struct Cyc_PP_Doc*_tmp2D6[3U];({struct Cyc_PP_Doc*_tmp6E5=Cyc_PP_text(({const char*_tmp2D7="malloc(";_tag_fat(_tmp2D7,sizeof(char),8U);}));_tmp2D6[0]=_tmp6E5;}),({struct Cyc_PP_Doc*_tmp6E4=Cyc_Absynpp_exp2doc(new_e);_tmp2D6[1]=_tmp6E4;}),({struct Cyc_PP_Doc*_tmp6E3=Cyc_PP_text(({const char*_tmp2D8=")";_tag_fat(_tmp2D8,sizeof(char),2U);}));_tmp2D6[2]=_tmp6E3;});Cyc_PP_cat(_tag_fat(_tmp2D6,sizeof(struct Cyc_PP_Doc*),3U));});else{
# 1275
if(inline_call)
s=({struct Cyc_PP_Doc*_tmp2D9[5U];({struct Cyc_PP_Doc*_tmp6EA=Cyc_PP_text(({const char*_tmp2DA="rmalloc_inline(";_tag_fat(_tmp2DA,sizeof(char),16U);}));_tmp2D9[0]=_tmp6EA;}),({struct Cyc_PP_Doc*_tmp6E9=Cyc_Absynpp_exp2doc(rgnopt);_tmp2D9[1]=_tmp6E9;}),({struct Cyc_PP_Doc*_tmp6E8=Cyc_PP_text(({const char*_tmp2DB=",";_tag_fat(_tmp2DB,sizeof(char),2U);}));_tmp2D9[2]=_tmp6E8;}),({
struct Cyc_PP_Doc*_tmp6E7=Cyc_Absynpp_exp2doc(new_e);_tmp2D9[3]=_tmp6E7;}),({struct Cyc_PP_Doc*_tmp6E6=Cyc_PP_text(({const char*_tmp2DC=")";_tag_fat(_tmp2DC,sizeof(char),2U);}));_tmp2D9[4]=_tmp6E6;});Cyc_PP_cat(_tag_fat(_tmp2D9,sizeof(struct Cyc_PP_Doc*),5U));});else{
# 1279
s=({struct Cyc_PP_Doc*_tmp2DD[5U];({struct Cyc_PP_Doc*_tmp6EF=Cyc_PP_text(({const char*_tmp2DE="rmalloc(";_tag_fat(_tmp2DE,sizeof(char),9U);}));_tmp2DD[0]=_tmp6EF;}),({struct Cyc_PP_Doc*_tmp6EE=Cyc_Absynpp_exp2doc(rgnopt);_tmp2DD[1]=_tmp6EE;}),({struct Cyc_PP_Doc*_tmp6ED=Cyc_PP_text(({const char*_tmp2DF=",";_tag_fat(_tmp2DF,sizeof(char),2U);}));_tmp2DD[2]=_tmp6ED;}),({
struct Cyc_PP_Doc*_tmp6EC=Cyc_Absynpp_exp2doc(new_e);_tmp2DD[3]=_tmp6EC;}),({struct Cyc_PP_Doc*_tmp6EB=Cyc_PP_text(({const char*_tmp2E0=")";_tag_fat(_tmp2E0,sizeof(char),2U);}));_tmp2DD[4]=_tmp6EB;});Cyc_PP_cat(_tag_fat(_tmp2DD,sizeof(struct Cyc_PP_Doc*),5U));});}}}
# 1283
goto _LL0;}case 36U: _LL51: _tmp20F=((struct Cyc_Absyn_Swap_e_Absyn_Raw_exp_struct*)_tmp20B)->f1;_tmp210=((struct Cyc_Absyn_Swap_e_Absyn_Raw_exp_struct*)_tmp20B)->f2;_LL52: {struct Cyc_Absyn_Exp*e1=_tmp20F;struct Cyc_Absyn_Exp*e2=_tmp210;
# 1285
s=({struct Cyc_PP_Doc*_tmp2E1[3U];({struct Cyc_PP_Doc*_tmp6F2=Cyc_Absynpp_exp2doc_prec(myprec,e1);_tmp2E1[0]=_tmp6F2;}),({struct Cyc_PP_Doc*_tmp6F1=Cyc_PP_text(({const char*_tmp2E2=" :=: ";_tag_fat(_tmp2E2,sizeof(char),6U);}));_tmp2E1[1]=_tmp6F1;}),({struct Cyc_PP_Doc*_tmp6F0=Cyc_Absynpp_exp2doc_prec(myprec,e2);_tmp2E1[2]=_tmp6F0;});Cyc_PP_cat(_tag_fat(_tmp2E1,sizeof(struct Cyc_PP_Doc*),3U));});
goto _LL0;}case 37U: _LL53: _tmp20D=((struct Cyc_Absyn_UnresolvedMem_e_Absyn_Raw_exp_struct*)_tmp20B)->f1;_tmp20E=((struct Cyc_Absyn_UnresolvedMem_e_Absyn_Raw_exp_struct*)_tmp20B)->f2;_LL54: {struct Cyc_Core_Opt*n=_tmp20D;struct Cyc_List_List*des=_tmp20E;
# 1288
s=({struct _fat_ptr _tmp6F4=({const char*_tmp2E3=",";_tag_fat(_tmp2E3,sizeof(char),2U);});Cyc_Absynpp_group_braces(_tmp6F4,({struct Cyc_List_List*(*_tmp6F3)(struct Cyc_PP_Doc*(*f)(struct _tuple16*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp2E4)(struct Cyc_PP_Doc*(*f)(struct _tuple16*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_PP_Doc*(*f)(struct _tuple16*),struct Cyc_List_List*x))Cyc_List_map;_tmp2E4;});_tmp6F3(Cyc_Absynpp_de2doc,des);}));});
goto _LL0;}default: _LL55: _tmp20C=((struct Cyc_Absyn_StmtExp_e_Absyn_Raw_exp_struct*)_tmp20B)->f1;_LL56: {struct Cyc_Absyn_Stmt*s2=_tmp20C;
# 1291
s=({struct Cyc_PP_Doc*_tmp2E5[7U];({struct Cyc_PP_Doc*_tmp6FB=Cyc_PP_text(({const char*_tmp2E6="(";_tag_fat(_tmp2E6,sizeof(char),2U);}));_tmp2E5[0]=_tmp6FB;}),({struct Cyc_PP_Doc*_tmp6FA=Cyc_Absynpp_lb();_tmp2E5[1]=_tmp6FA;}),({struct Cyc_PP_Doc*_tmp6F9=Cyc_PP_blank_doc();_tmp2E5[2]=_tmp6F9;}),({
struct Cyc_PP_Doc*_tmp6F8=Cyc_PP_nest(2,Cyc_Absynpp_stmt2doc(s2,1,0));_tmp2E5[3]=_tmp6F8;}),({
struct Cyc_PP_Doc*_tmp6F7=Cyc_PP_blank_doc();_tmp2E5[4]=_tmp6F7;}),({struct Cyc_PP_Doc*_tmp6F6=Cyc_Absynpp_rb();_tmp2E5[5]=_tmp6F6;}),({struct Cyc_PP_Doc*_tmp6F5=Cyc_PP_text(({const char*_tmp2E7=")";_tag_fat(_tmp2E7,sizeof(char),2U);}));_tmp2E5[6]=_tmp6F5;});Cyc_PP_cat(_tag_fat(_tmp2E5,sizeof(struct Cyc_PP_Doc*),7U));});
goto _LL0;}}_LL0:;}
# 1296
if(inprec >= myprec)
s=({struct Cyc_PP_Doc*_tmp2E8[3U];({struct Cyc_PP_Doc*_tmp6FD=Cyc_PP_text(({const char*_tmp2E9="(";_tag_fat(_tmp2E9,sizeof(char),2U);}));_tmp2E8[0]=_tmp6FD;}),_tmp2E8[1]=s,({struct Cyc_PP_Doc*_tmp6FC=Cyc_PP_text(({const char*_tmp2EA=")";_tag_fat(_tmp2EA,sizeof(char),2U);}));_tmp2E8[2]=_tmp6FC;});Cyc_PP_cat(_tag_fat(_tmp2E8,sizeof(struct Cyc_PP_Doc*),3U));});
# 1296
return s;}
# 607 "absynpp.cyc"
struct Cyc_PP_Doc*Cyc_Absynpp_designator2doc(void*d){
# 1302 "absynpp.cyc"
void*_tmp2EB=d;struct _fat_ptr*_tmp2EC;struct Cyc_Absyn_Exp*_tmp2ED;if(((struct Cyc_Absyn_ArrayElement_Absyn_Designator_struct*)_tmp2EB)->tag == 0U){_LL1: _tmp2ED=((struct Cyc_Absyn_ArrayElement_Absyn_Designator_struct*)_tmp2EB)->f1;_LL2: {struct Cyc_Absyn_Exp*e=_tmp2ED;
return({struct Cyc_PP_Doc*_tmp2EE[3U];({struct Cyc_PP_Doc*_tmp700=Cyc_PP_text(({const char*_tmp2EF=".[";_tag_fat(_tmp2EF,sizeof(char),3U);}));_tmp2EE[0]=_tmp700;}),({struct Cyc_PP_Doc*_tmp6FF=Cyc_Absynpp_exp2doc(e);_tmp2EE[1]=_tmp6FF;}),({struct Cyc_PP_Doc*_tmp6FE=Cyc_PP_text(({const char*_tmp2F0="]";_tag_fat(_tmp2F0,sizeof(char),2U);}));_tmp2EE[2]=_tmp6FE;});Cyc_PP_cat(_tag_fat(_tmp2EE,sizeof(struct Cyc_PP_Doc*),3U));});}}else{_LL3: _tmp2EC=((struct Cyc_Absyn_FieldName_Absyn_Designator_struct*)_tmp2EB)->f1;_LL4: {struct _fat_ptr*v=_tmp2EC;
return({struct Cyc_PP_Doc*_tmp2F1[2U];({struct Cyc_PP_Doc*_tmp702=Cyc_PP_text(({const char*_tmp2F2=".";_tag_fat(_tmp2F2,sizeof(char),2U);}));_tmp2F1[0]=_tmp702;}),({struct Cyc_PP_Doc*_tmp701=Cyc_PP_textptr(v);_tmp2F1[1]=_tmp701;});Cyc_PP_cat(_tag_fat(_tmp2F1,sizeof(struct Cyc_PP_Doc*),2U));});}}_LL0:;}
# 607 "absynpp.cyc"
struct Cyc_PP_Doc*Cyc_Absynpp_de2doc(struct _tuple16*de){
# 1309 "absynpp.cyc"
if((*de).f1 == 0)return Cyc_Absynpp_exp2doc((*de).f2);else{
return({struct Cyc_PP_Doc*_tmp2F3[2U];({struct Cyc_PP_Doc*_tmp708=({struct _fat_ptr _tmp707=({const char*_tmp2F4="";_tag_fat(_tmp2F4,sizeof(char),1U);});struct _fat_ptr _tmp706=({const char*_tmp2F5="=";_tag_fat(_tmp2F5,sizeof(char),2U);});struct _fat_ptr _tmp705=({const char*_tmp2F6="=";_tag_fat(_tmp2F6,sizeof(char),2U);});Cyc_PP_egroup(_tmp707,_tmp706,_tmp705,({struct Cyc_List_List*(*_tmp704)(struct Cyc_PP_Doc*(*f)(void*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp2F7)(struct Cyc_PP_Doc*(*f)(void*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_PP_Doc*(*f)(void*),struct Cyc_List_List*x))Cyc_List_map;_tmp2F7;});_tmp704(Cyc_Absynpp_designator2doc,(*de).f1);}));});_tmp2F3[0]=_tmp708;}),({
struct Cyc_PP_Doc*_tmp703=Cyc_Absynpp_exp2doc((*de).f2);_tmp2F3[1]=_tmp703;});Cyc_PP_cat(_tag_fat(_tmp2F3,sizeof(struct Cyc_PP_Doc*),2U));});}}
# 607 "absynpp.cyc"
struct Cyc_PP_Doc*Cyc_Absynpp_exps2doc_prec(int inprec,struct Cyc_List_List*es){
# 1315 "absynpp.cyc"
return({struct _fat_ptr _tmp70D=({const char*_tmp2F8="";_tag_fat(_tmp2F8,sizeof(char),1U);});struct _fat_ptr _tmp70C=({const char*_tmp2F9="";_tag_fat(_tmp2F9,sizeof(char),1U);});struct _fat_ptr _tmp70B=({const char*_tmp2FA=",";_tag_fat(_tmp2FA,sizeof(char),2U);});Cyc_PP_group(_tmp70D,_tmp70C,_tmp70B,({struct Cyc_List_List*(*_tmp70A)(struct Cyc_PP_Doc*(*f)(int,struct Cyc_Absyn_Exp*),int env,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp2FB)(struct Cyc_PP_Doc*(*f)(int,struct Cyc_Absyn_Exp*),int env,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_PP_Doc*(*f)(int,struct Cyc_Absyn_Exp*),int env,struct Cyc_List_List*x))Cyc_List_map_c;_tmp2FB;});int _tmp709=inprec;_tmp70A(Cyc_Absynpp_exp2doc_prec,_tmp709,es);}));});}
# 607 "absynpp.cyc"
struct _fat_ptr Cyc_Absynpp_longlong2string(unsigned long long i){
# 1319 "absynpp.cyc"
struct _fat_ptr x=({char*_tmp310=({unsigned _tmp30F=28U + 1U;char*_tmp30E=_cycalloc_atomic(_check_times(_tmp30F,sizeof(char)));({{unsigned _tmp556=28U;unsigned i;for(i=0;i < _tmp556;++ i){_tmp30E[i]='z';}_tmp30E[_tmp556]=0;}0;});_tmp30E;});_tag_fat(_tmp310,sizeof(char),29U);});
({struct _fat_ptr _tmp2FC=_fat_ptr_plus(x,sizeof(char),27);char _tmp2FD=*((char*)_check_fat_subscript(_tmp2FC,sizeof(char),0U));char _tmp2FE='\000';if(_get_fat_size(_tmp2FC,sizeof(char))== 1U &&(_tmp2FD == 0 && _tmp2FE != 0))_throw_arraybounds();*((char*)_tmp2FC.curr)=_tmp2FE;});
({struct _fat_ptr _tmp2FF=_fat_ptr_plus(x,sizeof(char),26);char _tmp300=*((char*)_check_fat_subscript(_tmp2FF,sizeof(char),0U));char _tmp301='L';if(_get_fat_size(_tmp2FF,sizeof(char))== 1U &&(_tmp300 == 0 && _tmp301 != 0))_throw_arraybounds();*((char*)_tmp2FF.curr)=_tmp301;});
({struct _fat_ptr _tmp302=_fat_ptr_plus(x,sizeof(char),25);char _tmp303=*((char*)_check_fat_subscript(_tmp302,sizeof(char),0U));char _tmp304='L';if(_get_fat_size(_tmp302,sizeof(char))== 1U &&(_tmp303 == 0 && _tmp304 != 0))_throw_arraybounds();*((char*)_tmp302.curr)=_tmp304;});
({struct _fat_ptr _tmp305=_fat_ptr_plus(x,sizeof(char),24);char _tmp306=*((char*)_check_fat_subscript(_tmp305,sizeof(char),0U));char _tmp307='U';if(_get_fat_size(_tmp305,sizeof(char))== 1U &&(_tmp306 == 0 && _tmp307 != 0))_throw_arraybounds();*((char*)_tmp305.curr)=_tmp307;});
({struct _fat_ptr _tmp308=_fat_ptr_plus(x,sizeof(char),23);char _tmp309=*((char*)_check_fat_subscript(_tmp308,sizeof(char),0U));char _tmp30A='0';if(_get_fat_size(_tmp308,sizeof(char))== 1U &&(_tmp309 == 0 && _tmp30A != 0))_throw_arraybounds();*((char*)_tmp308.curr)=_tmp30A;});{
int index=23;
while(i != (unsigned long long)0){
char c=(char)((unsigned long long)'0' + i % (unsigned long long)10);
({struct _fat_ptr _tmp30B=_fat_ptr_plus(x,sizeof(char),index);char _tmp30C=*((char*)_check_fat_subscript(_tmp30B,sizeof(char),0U));char _tmp30D=c;if(_get_fat_size(_tmp30B,sizeof(char))== 1U &&(_tmp30C == 0 && _tmp30D != 0))_throw_arraybounds();*((char*)_tmp30B.curr)=_tmp30D;});
i=i / (unsigned long long)10;
-- index;}
# 1332
return(struct _fat_ptr)_fat_ptr_plus(_fat_ptr_plus(x,sizeof(char),index),sizeof(char),1);}}
# 607 "absynpp.cyc"
struct Cyc_PP_Doc*Cyc_Absynpp_cnst2doc(union Cyc_Absyn_Cnst c){
# 1337 "absynpp.cyc"
union Cyc_Absyn_Cnst _tmp311=c;struct _fat_ptr _tmp312;struct _fat_ptr _tmp313;struct _fat_ptr _tmp314;long long _tmp316;enum Cyc_Absyn_Sign _tmp315;int _tmp318;enum Cyc_Absyn_Sign _tmp317;short _tmp31A;enum Cyc_Absyn_Sign _tmp319;struct _fat_ptr _tmp31B;char _tmp31D;enum Cyc_Absyn_Sign _tmp31C;switch((_tmp311.String_c).tag){case 2U: _LL1: _tmp31C=((_tmp311.Char_c).val).f1;_tmp31D=((_tmp311.Char_c).val).f2;_LL2: {enum Cyc_Absyn_Sign sg=_tmp31C;char ch=_tmp31D;
return Cyc_PP_text((struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp320=({struct Cyc_String_pa_PrintArg_struct _tmp557;_tmp557.tag=0U,({struct _fat_ptr _tmp70E=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_char_escape(ch));_tmp557.f1=_tmp70E;});_tmp557;});void*_tmp31E[1U];_tmp31E[0]=& _tmp320;({struct _fat_ptr _tmp70F=({const char*_tmp31F="'%s'";_tag_fat(_tmp31F,sizeof(char),5U);});Cyc_aprintf(_tmp70F,_tag_fat(_tmp31E,sizeof(void*),1U));});}));}case 3U: _LL3: _tmp31B=(_tmp311.Wchar_c).val;_LL4: {struct _fat_ptr s=_tmp31B;
return Cyc_PP_text((struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp323=({struct Cyc_String_pa_PrintArg_struct _tmp558;_tmp558.tag=0U,_tmp558.f1=(struct _fat_ptr)((struct _fat_ptr)s);_tmp558;});void*_tmp321[1U];_tmp321[0]=& _tmp323;({struct _fat_ptr _tmp710=({const char*_tmp322="L'%s'";_tag_fat(_tmp322,sizeof(char),6U);});Cyc_aprintf(_tmp710,_tag_fat(_tmp321,sizeof(void*),1U));});}));}case 4U: _LL5: _tmp319=((_tmp311.Short_c).val).f1;_tmp31A=((_tmp311.Short_c).val).f2;_LL6: {enum Cyc_Absyn_Sign sg=_tmp319;short s=_tmp31A;
return Cyc_PP_text((struct _fat_ptr)({struct Cyc_Int_pa_PrintArg_struct _tmp326=({struct Cyc_Int_pa_PrintArg_struct _tmp559;_tmp559.tag=1U,_tmp559.f1=(unsigned long)((int)s);_tmp559;});void*_tmp324[1U];_tmp324[0]=& _tmp326;({struct _fat_ptr _tmp711=({const char*_tmp325="%d";_tag_fat(_tmp325,sizeof(char),3U);});Cyc_aprintf(_tmp711,_tag_fat(_tmp324,sizeof(void*),1U));});}));}case 5U: _LL7: _tmp317=((_tmp311.Int_c).val).f1;_tmp318=((_tmp311.Int_c).val).f2;_LL8: {enum Cyc_Absyn_Sign sn=_tmp317;int i=_tmp318;
# 1342
if((int)sn == (int)1U)return Cyc_PP_text((struct _fat_ptr)({struct Cyc_Int_pa_PrintArg_struct _tmp329=({struct Cyc_Int_pa_PrintArg_struct _tmp55A;_tmp55A.tag=1U,_tmp55A.f1=(unsigned)i;_tmp55A;});void*_tmp327[1U];_tmp327[0]=& _tmp329;({struct _fat_ptr _tmp712=({const char*_tmp328="%uU";_tag_fat(_tmp328,sizeof(char),4U);});Cyc_aprintf(_tmp712,_tag_fat(_tmp327,sizeof(void*),1U));});}));else{
return Cyc_PP_text((struct _fat_ptr)({struct Cyc_Int_pa_PrintArg_struct _tmp32C=({struct Cyc_Int_pa_PrintArg_struct _tmp55B;_tmp55B.tag=1U,_tmp55B.f1=(unsigned long)i;_tmp55B;});void*_tmp32A[1U];_tmp32A[0]=& _tmp32C;({struct _fat_ptr _tmp713=({const char*_tmp32B="%d";_tag_fat(_tmp32B,sizeof(char),3U);});Cyc_aprintf(_tmp713,_tag_fat(_tmp32A,sizeof(void*),1U));});}));}}case 6U: _LL9: _tmp315=((_tmp311.LongLong_c).val).f1;_tmp316=((_tmp311.LongLong_c).val).f2;_LLA: {enum Cyc_Absyn_Sign sg=_tmp315;long long i=_tmp316;
# 1346
return Cyc_PP_text(Cyc_Absynpp_longlong2string((unsigned long long)i));}case 7U: _LLB: _tmp314=((_tmp311.Float_c).val).f1;_LLC: {struct _fat_ptr x=_tmp314;
return Cyc_PP_text(x);}case 1U: _LLD: _LLE:
 return Cyc_PP_text(({const char*_tmp32D="NULL";_tag_fat(_tmp32D,sizeof(char),5U);}));case 8U: _LLF: _tmp313=(_tmp311.String_c).val;_LL10: {struct _fat_ptr s=_tmp313;
return({struct Cyc_PP_Doc*_tmp32E[3U];({struct Cyc_PP_Doc*_tmp716=Cyc_PP_text(({const char*_tmp32F="\"";_tag_fat(_tmp32F,sizeof(char),2U);}));_tmp32E[0]=_tmp716;}),({struct Cyc_PP_Doc*_tmp715=Cyc_PP_text(Cyc_Absynpp_string_escape(s));_tmp32E[1]=_tmp715;}),({struct Cyc_PP_Doc*_tmp714=Cyc_PP_text(({const char*_tmp330="\"";_tag_fat(_tmp330,sizeof(char),2U);}));_tmp32E[2]=_tmp714;});Cyc_PP_cat(_tag_fat(_tmp32E,sizeof(struct Cyc_PP_Doc*),3U));});}default: _LL11: _tmp312=(_tmp311.Wstring_c).val;_LL12: {struct _fat_ptr s=_tmp312;
return({struct Cyc_PP_Doc*_tmp331[3U];({struct Cyc_PP_Doc*_tmp719=Cyc_PP_text(({const char*_tmp332="L\"";_tag_fat(_tmp332,sizeof(char),3U);}));_tmp331[0]=_tmp719;}),({struct Cyc_PP_Doc*_tmp718=Cyc_PP_text(s);_tmp331[1]=_tmp718;}),({struct Cyc_PP_Doc*_tmp717=Cyc_PP_text(({const char*_tmp333="\"";_tag_fat(_tmp333,sizeof(char),2U);}));_tmp331[2]=_tmp717;});Cyc_PP_cat(_tag_fat(_tmp331,sizeof(struct Cyc_PP_Doc*),3U));});}}_LL0:;}
# 607 "absynpp.cyc"
struct Cyc_PP_Doc*Cyc_Absynpp_primapp2doc(int inprec,enum Cyc_Absyn_Primop p,struct Cyc_List_List*es){
# 1355 "absynpp.cyc"
struct Cyc_PP_Doc*ps=Cyc_Absynpp_prim2doc(p);
if((int)p == (int)18U){
if(es == 0 || es->tl != 0)
(int)_throw((void*)({struct Cyc_Core_Failure_exn_struct*_tmp337=_cycalloc(sizeof(*_tmp337));_tmp337->tag=Cyc_Core_Failure,({struct _fat_ptr _tmp71C=(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp336=({struct Cyc_String_pa_PrintArg_struct _tmp55C;_tmp55C.tag=0U,({struct _fat_ptr _tmp71A=(struct _fat_ptr)((struct _fat_ptr)Cyc_PP_string_of_doc(ps,72));_tmp55C.f1=_tmp71A;});_tmp55C;});void*_tmp334[1U];_tmp334[0]=& _tmp336;({struct _fat_ptr _tmp71B=({const char*_tmp335="Absynpp::primapp2doc Numelts: %s with bad args";_tag_fat(_tmp335,sizeof(char),47U);});Cyc_aprintf(_tmp71B,_tag_fat(_tmp334,sizeof(void*),1U));});});_tmp337->f1=_tmp71C;});_tmp337;}));
# 1357
return({struct Cyc_PP_Doc*_tmp338[3U];({
# 1360
struct Cyc_PP_Doc*_tmp71F=Cyc_PP_text(({const char*_tmp339="numelts(";_tag_fat(_tmp339,sizeof(char),9U);}));_tmp338[0]=_tmp71F;}),({struct Cyc_PP_Doc*_tmp71E=Cyc_Absynpp_exp2doc((struct Cyc_Absyn_Exp*)es->hd);_tmp338[1]=_tmp71E;}),({struct Cyc_PP_Doc*_tmp71D=Cyc_PP_text(({const char*_tmp33A=")";_tag_fat(_tmp33A,sizeof(char),2U);}));_tmp338[2]=_tmp71D;});Cyc_PP_cat(_tag_fat(_tmp338,sizeof(struct Cyc_PP_Doc*),3U));});}else{
# 1362
struct Cyc_List_List*ds=({struct Cyc_List_List*(*_tmp721)(struct Cyc_PP_Doc*(*f)(int,struct Cyc_Absyn_Exp*),int env,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp348)(struct Cyc_PP_Doc*(*f)(int,struct Cyc_Absyn_Exp*),int env,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_PP_Doc*(*f)(int,struct Cyc_Absyn_Exp*),int env,struct Cyc_List_List*x))Cyc_List_map_c;_tmp348;});int _tmp720=inprec;_tmp721(Cyc_Absynpp_exp2doc_prec,_tmp720,es);});
if(ds == 0)
(int)_throw((void*)({struct Cyc_Core_Failure_exn_struct*_tmp33E=_cycalloc(sizeof(*_tmp33E));_tmp33E->tag=Cyc_Core_Failure,({struct _fat_ptr _tmp724=(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp33D=({struct Cyc_String_pa_PrintArg_struct _tmp55D;_tmp55D.tag=0U,({struct _fat_ptr _tmp722=(struct _fat_ptr)((struct _fat_ptr)Cyc_PP_string_of_doc(ps,72));_tmp55D.f1=_tmp722;});_tmp55D;});void*_tmp33B[1U];_tmp33B[0]=& _tmp33D;({struct _fat_ptr _tmp723=({const char*_tmp33C="Absynpp::primapp2doc: %s with no args";_tag_fat(_tmp33C,sizeof(char),38U);});Cyc_aprintf(_tmp723,_tag_fat(_tmp33B,sizeof(void*),1U));});});_tmp33E->f1=_tmp724;});_tmp33E;}));else{
# 1366
if(ds->tl == 0)
return({struct Cyc_PP_Doc*_tmp33F[3U];_tmp33F[0]=ps,({struct Cyc_PP_Doc*_tmp725=Cyc_PP_text(({const char*_tmp340=" ";_tag_fat(_tmp340,sizeof(char),2U);}));_tmp33F[1]=_tmp725;}),_tmp33F[2]=(struct Cyc_PP_Doc*)ds->hd;Cyc_PP_cat(_tag_fat(_tmp33F,sizeof(struct Cyc_PP_Doc*),3U));});else{
if(((struct Cyc_List_List*)_check_null(ds->tl))->tl != 0)
(int)_throw((void*)({struct Cyc_Core_Failure_exn_struct*_tmp344=_cycalloc(sizeof(*_tmp344));_tmp344->tag=Cyc_Core_Failure,({struct _fat_ptr _tmp728=(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp343=({struct Cyc_String_pa_PrintArg_struct _tmp55E;_tmp55E.tag=0U,({struct _fat_ptr _tmp726=(struct _fat_ptr)((struct _fat_ptr)Cyc_PP_string_of_doc(ps,72));_tmp55E.f1=_tmp726;});_tmp55E;});void*_tmp341[1U];_tmp341[0]=& _tmp343;({struct _fat_ptr _tmp727=({const char*_tmp342="Absynpp::primapp2doc: %s with more than 2 args";_tag_fat(_tmp342,sizeof(char),47U);});Cyc_aprintf(_tmp727,_tag_fat(_tmp341,sizeof(void*),1U));});});_tmp344->f1=_tmp728;});_tmp344;}));else{
# 1372
return({struct Cyc_PP_Doc*_tmp345[5U];_tmp345[0]=(struct Cyc_PP_Doc*)ds->hd,({struct Cyc_PP_Doc*_tmp72A=Cyc_PP_text(({const char*_tmp346=" ";_tag_fat(_tmp346,sizeof(char),2U);}));_tmp345[1]=_tmp72A;}),_tmp345[2]=ps,({struct Cyc_PP_Doc*_tmp729=Cyc_PP_text(({const char*_tmp347=" ";_tag_fat(_tmp347,sizeof(char),2U);}));_tmp345[3]=_tmp729;}),_tmp345[4]=(struct Cyc_PP_Doc*)((struct Cyc_List_List*)_check_null(ds->tl))->hd;Cyc_PP_cat(_tag_fat(_tmp345,sizeof(struct Cyc_PP_Doc*),5U));});}}}}}
# 607 "absynpp.cyc"
struct _fat_ptr Cyc_Absynpp_prim2str(enum Cyc_Absyn_Primop p){
# 1377 "absynpp.cyc"
enum Cyc_Absyn_Primop _tmp349=p;switch(_tmp349){case Cyc_Absyn_Plus: _LL1: _LL2:
 return({const char*_tmp34A="+";_tag_fat(_tmp34A,sizeof(char),2U);});case Cyc_Absyn_Times: _LL3: _LL4:
 return({const char*_tmp34B="*";_tag_fat(_tmp34B,sizeof(char),2U);});case Cyc_Absyn_Minus: _LL5: _LL6:
 return({const char*_tmp34C="-";_tag_fat(_tmp34C,sizeof(char),2U);});case Cyc_Absyn_Div: _LL7: _LL8:
 return({const char*_tmp34D="/";_tag_fat(_tmp34D,sizeof(char),2U);});case Cyc_Absyn_Mod: _LL9: _LLA:
 return Cyc_Absynpp_print_for_cycdoc?({const char*_tmp34E="\\%";_tag_fat(_tmp34E,sizeof(char),3U);}):({const char*_tmp34F="%";_tag_fat(_tmp34F,sizeof(char),2U);});case Cyc_Absyn_Eq: _LLB: _LLC:
 return({const char*_tmp350="==";_tag_fat(_tmp350,sizeof(char),3U);});case Cyc_Absyn_Neq: _LLD: _LLE:
 return({const char*_tmp351="!=";_tag_fat(_tmp351,sizeof(char),3U);});case Cyc_Absyn_Gt: _LLF: _LL10:
 return({const char*_tmp352=">";_tag_fat(_tmp352,sizeof(char),2U);});case Cyc_Absyn_Lt: _LL11: _LL12:
 return({const char*_tmp353="<";_tag_fat(_tmp353,sizeof(char),2U);});case Cyc_Absyn_Gte: _LL13: _LL14:
 return({const char*_tmp354=">=";_tag_fat(_tmp354,sizeof(char),3U);});case Cyc_Absyn_Lte: _LL15: _LL16:
 return({const char*_tmp355="<=";_tag_fat(_tmp355,sizeof(char),3U);});case Cyc_Absyn_Not: _LL17: _LL18:
 return({const char*_tmp356="!";_tag_fat(_tmp356,sizeof(char),2U);});case Cyc_Absyn_Bitnot: _LL19: _LL1A:
 return({const char*_tmp357="~";_tag_fat(_tmp357,sizeof(char),2U);});case Cyc_Absyn_Bitand: _LL1B: _LL1C:
 return({const char*_tmp358="&";_tag_fat(_tmp358,sizeof(char),2U);});case Cyc_Absyn_Bitor: _LL1D: _LL1E:
 return({const char*_tmp359="|";_tag_fat(_tmp359,sizeof(char),2U);});case Cyc_Absyn_Bitxor: _LL1F: _LL20:
 return({const char*_tmp35A="^";_tag_fat(_tmp35A,sizeof(char),2U);});case Cyc_Absyn_Bitlshift: _LL21: _LL22:
 return({const char*_tmp35B="<<";_tag_fat(_tmp35B,sizeof(char),3U);});case Cyc_Absyn_Bitlrshift: _LL23: _LL24:
 return({const char*_tmp35C=">>";_tag_fat(_tmp35C,sizeof(char),3U);});case Cyc_Absyn_Numelts: _LL25: _LL26:
 return({const char*_tmp35D="numelts";_tag_fat(_tmp35D,sizeof(char),8U);});default: _LL27: _LL28:
 return({const char*_tmp35E="?";_tag_fat(_tmp35E,sizeof(char),2U);});}_LL0:;}
# 607 "absynpp.cyc"
struct Cyc_PP_Doc*Cyc_Absynpp_prim2doc(enum Cyc_Absyn_Primop p){
# 1402 "absynpp.cyc"
return Cyc_PP_text(Cyc_Absynpp_prim2str(p));}
# 607 "absynpp.cyc"
int Cyc_Absynpp_is_declaration(struct Cyc_Absyn_Stmt*s){
# 1406 "absynpp.cyc"
void*_stmttmp17=s->r;void*_tmp35F=_stmttmp17;if(((struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct*)_tmp35F)->tag == 12U){_LL1: _LL2:
 return 1;}else{_LL3: _LL4:
 return 0;}_LL0:;}
# 607 "absynpp.cyc"
struct _tuple14 Cyc_Absynpp_shadows(struct Cyc_Absyn_Decl*d,struct Cyc_List_List*varsinblock){
# 1421 "absynpp.cyc"
void*_stmttmp18=d->r;void*_tmp360=_stmttmp18;struct Cyc_Absyn_Vardecl*_tmp361;if(((struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct*)_tmp360)->tag == 0U){_LL1: _tmp361=((struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct*)_tmp360)->f1;_LL2: {struct Cyc_Absyn_Vardecl*vd=_tmp361;
# 1423
if(({int(*_tmp72C)(int(*compare)(struct _tuple0*,struct _tuple0*),struct Cyc_List_List*l,struct _tuple0*x)=({int(*_tmp362)(int(*compare)(struct _tuple0*,struct _tuple0*),struct Cyc_List_List*l,struct _tuple0*x)=(int(*)(int(*compare)(struct _tuple0*,struct _tuple0*),struct Cyc_List_List*l,struct _tuple0*x))Cyc_List_mem;_tmp362;});struct Cyc_List_List*_tmp72B=varsinblock;_tmp72C(Cyc_Absyn_qvar_cmp,_tmp72B,vd->name);}))
return({struct _tuple14 _tmp55F;_tmp55F.f1=1,({struct Cyc_List_List*_tmp72D=({struct Cyc_List_List*_tmp363=_cycalloc(sizeof(*_tmp363));_tmp363->hd=vd->name,_tmp363->tl=0;_tmp363;});_tmp55F.f2=_tmp72D;});_tmp55F;});else{
# 1426
return({struct _tuple14 _tmp560;_tmp560.f1=0,({struct Cyc_List_List*_tmp72E=({struct Cyc_List_List*_tmp364=_cycalloc(sizeof(*_tmp364));_tmp364->hd=vd->name,_tmp364->tl=varsinblock;_tmp364;});_tmp560.f2=_tmp72E;});_tmp560;});}}}else{_LL3: _LL4:
# 1428
 return({struct _tuple14 _tmp561;_tmp561.f1=0,_tmp561.f2=varsinblock;_tmp561;});}_LL0:;}
# 607 "absynpp.cyc"
struct Cyc_PP_Doc*Cyc_Absynpp_block(int stmtexp,struct Cyc_PP_Doc*d){
# 1433 "absynpp.cyc"
if(stmtexp)
return({struct Cyc_PP_Doc*_tmp365[8U];({struct Cyc_PP_Doc*_tmp736=Cyc_PP_text(({const char*_tmp366="(";_tag_fat(_tmp366,sizeof(char),2U);}));_tmp365[0]=_tmp736;}),({struct Cyc_PP_Doc*_tmp735=Cyc_Absynpp_lb();_tmp365[1]=_tmp735;}),({struct Cyc_PP_Doc*_tmp734=Cyc_PP_blank_doc();_tmp365[2]=_tmp734;}),({struct Cyc_PP_Doc*_tmp733=Cyc_PP_nest(2,d);_tmp365[3]=_tmp733;}),({struct Cyc_PP_Doc*_tmp732=Cyc_PP_line_doc();_tmp365[4]=_tmp732;}),({struct Cyc_PP_Doc*_tmp731=Cyc_Absynpp_rb();_tmp365[5]=_tmp731;}),({
struct Cyc_PP_Doc*_tmp730=Cyc_PP_text(({const char*_tmp367=");";_tag_fat(_tmp367,sizeof(char),3U);}));_tmp365[6]=_tmp730;}),({struct Cyc_PP_Doc*_tmp72F=Cyc_PP_line_doc();_tmp365[7]=_tmp72F;});Cyc_PP_cat(_tag_fat(_tmp365,sizeof(struct Cyc_PP_Doc*),8U));});else{
# 1437
return({struct Cyc_PP_Doc*_tmp368[6U];({struct Cyc_PP_Doc*_tmp73C=Cyc_Absynpp_lb();_tmp368[0]=_tmp73C;}),({struct Cyc_PP_Doc*_tmp73B=Cyc_PP_blank_doc();_tmp368[1]=_tmp73B;}),({struct Cyc_PP_Doc*_tmp73A=Cyc_PP_nest(2,d);_tmp368[2]=_tmp73A;}),({struct Cyc_PP_Doc*_tmp739=Cyc_PP_line_doc();_tmp368[3]=_tmp739;}),({struct Cyc_PP_Doc*_tmp738=Cyc_Absynpp_rb();_tmp368[4]=_tmp738;}),({struct Cyc_PP_Doc*_tmp737=Cyc_PP_line_doc();_tmp368[5]=_tmp737;});Cyc_PP_cat(_tag_fat(_tmp368,sizeof(struct Cyc_PP_Doc*),6U));});}}
# 607 "absynpp.cyc"
struct Cyc_PP_Doc*Cyc_Absynpp_stmt2doc(struct Cyc_Absyn_Stmt*st,int stmtexp,struct Cyc_List_List*varsinblock){
# 1441 "absynpp.cyc"
struct Cyc_PP_Doc*s;
{void*_stmttmp19=st->r;void*_tmp369=_stmttmp19;struct Cyc_List_List*_tmp36B;struct Cyc_Absyn_Stmt*_tmp36A;struct Cyc_Absyn_Exp*_tmp36D;struct Cyc_Absyn_Stmt*_tmp36C;struct Cyc_Absyn_Stmt*_tmp36F;struct _fat_ptr*_tmp36E;struct Cyc_Absyn_Stmt*_tmp371;struct Cyc_Absyn_Decl*_tmp370;struct Cyc_List_List*_tmp372;struct Cyc_List_List*_tmp374;struct Cyc_Absyn_Exp*_tmp373;struct Cyc_Absyn_Stmt*_tmp378;struct Cyc_Absyn_Exp*_tmp377;struct Cyc_Absyn_Exp*_tmp376;struct Cyc_Absyn_Exp*_tmp375;struct _fat_ptr*_tmp379;struct Cyc_Absyn_Stmt*_tmp37B;struct Cyc_Absyn_Exp*_tmp37A;struct Cyc_Absyn_Stmt*_tmp37E;struct Cyc_Absyn_Stmt*_tmp37D;struct Cyc_Absyn_Exp*_tmp37C;struct Cyc_Absyn_Exp*_tmp37F;struct Cyc_Absyn_Stmt*_tmp381;struct Cyc_Absyn_Stmt*_tmp380;struct Cyc_Absyn_Exp*_tmp382;switch(*((int*)_tmp369)){case 0U: _LL1: _LL2:
 s=Cyc_PP_text(({const char*_tmp383=";";_tag_fat(_tmp383,sizeof(char),2U);}));goto _LL0;case 1U: _LL3: _tmp382=((struct Cyc_Absyn_Exp_s_Absyn_Raw_stmt_struct*)_tmp369)->f1;_LL4: {struct Cyc_Absyn_Exp*e=_tmp382;
s=({struct Cyc_PP_Doc*_tmp384[2U];({struct Cyc_PP_Doc*_tmp73E=Cyc_Absynpp_exp2doc(e);_tmp384[0]=_tmp73E;}),({struct Cyc_PP_Doc*_tmp73D=Cyc_PP_text(({const char*_tmp385=";";_tag_fat(_tmp385,sizeof(char),2U);}));_tmp384[1]=_tmp73D;});Cyc_PP_cat(_tag_fat(_tmp384,sizeof(struct Cyc_PP_Doc*),2U));});goto _LL0;}case 2U: _LL5: _tmp380=((struct Cyc_Absyn_Seq_s_Absyn_Raw_stmt_struct*)_tmp369)->f1;_tmp381=((struct Cyc_Absyn_Seq_s_Absyn_Raw_stmt_struct*)_tmp369)->f2;_LL6: {struct Cyc_Absyn_Stmt*s1=_tmp380;struct Cyc_Absyn_Stmt*s2=_tmp381;
# 1446
if(Cyc_Absynpp_decls_first){
if(Cyc_Absynpp_is_declaration(s1))
s=({struct Cyc_PP_Doc*_tmp386[2U];({struct Cyc_PP_Doc*_tmp742=Cyc_Absynpp_block(0,Cyc_Absynpp_stmt2doc(s1,0,0));_tmp386[0]=_tmp742;}),
Cyc_Absynpp_is_declaration(s2)?({
struct Cyc_PP_Doc*_tmp741=({int _tmp740=stmtexp;Cyc_Absynpp_block(_tmp740,Cyc_Absynpp_stmt2doc(s2,stmtexp,0));});_tmp386[1]=_tmp741;}):({
struct Cyc_PP_Doc*_tmp73F=Cyc_Absynpp_stmt2doc(s2,stmtexp,varsinblock);_tmp386[1]=_tmp73F;});Cyc_PP_cat(_tag_fat(_tmp386,sizeof(struct Cyc_PP_Doc*),2U));});else{
if(Cyc_Absynpp_is_declaration(s2))
s=({struct Cyc_PP_Doc*_tmp387[3U];({struct Cyc_PP_Doc*_tmp746=Cyc_Absynpp_stmt2doc(s1,0,varsinblock);_tmp387[0]=_tmp746;}),({
struct Cyc_PP_Doc*_tmp745=Cyc_PP_line_doc();_tmp387[1]=_tmp745;}),({
struct Cyc_PP_Doc*_tmp744=({int _tmp743=stmtexp;Cyc_Absynpp_block(_tmp743,Cyc_Absynpp_stmt2doc(s2,stmtexp,0));});_tmp387[2]=_tmp744;});Cyc_PP_cat(_tag_fat(_tmp387,sizeof(struct Cyc_PP_Doc*),3U));});else{
# 1457
s=({struct Cyc_PP_Doc*_tmp388[3U];({struct Cyc_PP_Doc*_tmp749=Cyc_Absynpp_stmt2doc(s1,0,varsinblock);_tmp388[0]=_tmp749;}),({struct Cyc_PP_Doc*_tmp748=Cyc_PP_line_doc();_tmp388[1]=_tmp748;}),({
struct Cyc_PP_Doc*_tmp747=Cyc_Absynpp_stmt2doc(s2,stmtexp,varsinblock);_tmp388[2]=_tmp747;});Cyc_PP_cat(_tag_fat(_tmp388,sizeof(struct Cyc_PP_Doc*),3U));});}}}else{
# 1461
s=({struct Cyc_PP_Doc*_tmp389[3U];({struct Cyc_PP_Doc*_tmp74C=Cyc_Absynpp_stmt2doc(s1,0,0);_tmp389[0]=_tmp74C;}),({struct Cyc_PP_Doc*_tmp74B=Cyc_PP_line_doc();_tmp389[1]=_tmp74B;}),({struct Cyc_PP_Doc*_tmp74A=Cyc_Absynpp_stmt2doc(s2,stmtexp,0);_tmp389[2]=_tmp74A;});Cyc_PP_cat(_tag_fat(_tmp389,sizeof(struct Cyc_PP_Doc*),3U));});}
goto _LL0;}case 3U: _LL7: _tmp37F=((struct Cyc_Absyn_Return_s_Absyn_Raw_stmt_struct*)_tmp369)->f1;_LL8: {struct Cyc_Absyn_Exp*eopt=_tmp37F;
# 1464
if(eopt == 0)
s=Cyc_PP_text(({const char*_tmp38A="return;";_tag_fat(_tmp38A,sizeof(char),8U);}));else{
# 1467
s=({struct Cyc_PP_Doc*_tmp38B[3U];({struct Cyc_PP_Doc*_tmp750=Cyc_PP_text(({const char*_tmp38C="return ";_tag_fat(_tmp38C,sizeof(char),8U);}));_tmp38B[0]=_tmp750;}),
eopt == 0?({struct Cyc_PP_Doc*_tmp74F=Cyc_PP_nil_doc();_tmp38B[1]=_tmp74F;}):({struct Cyc_PP_Doc*_tmp74E=Cyc_Absynpp_exp2doc(eopt);_tmp38B[1]=_tmp74E;}),({
struct Cyc_PP_Doc*_tmp74D=Cyc_PP_text(({const char*_tmp38D=";";_tag_fat(_tmp38D,sizeof(char),2U);}));_tmp38B[2]=_tmp74D;});Cyc_PP_cat(_tag_fat(_tmp38B,sizeof(struct Cyc_PP_Doc*),3U));});}
goto _LL0;}case 4U: _LL9: _tmp37C=((struct Cyc_Absyn_IfThenElse_s_Absyn_Raw_stmt_struct*)_tmp369)->f1;_tmp37D=((struct Cyc_Absyn_IfThenElse_s_Absyn_Raw_stmt_struct*)_tmp369)->f2;_tmp37E=((struct Cyc_Absyn_IfThenElse_s_Absyn_Raw_stmt_struct*)_tmp369)->f3;_LLA: {struct Cyc_Absyn_Exp*e=_tmp37C;struct Cyc_Absyn_Stmt*s1=_tmp37D;struct Cyc_Absyn_Stmt*s2=_tmp37E;
# 1472
int print_else;
{void*_stmttmp1A=s2->r;void*_tmp38E=_stmttmp1A;if(((struct Cyc_Absyn_Skip_s_Absyn_Raw_stmt_struct*)_tmp38E)->tag == 0U){_LL24: _LL25:
 print_else=0;goto _LL23;}else{_LL26: _LL27:
 print_else=1;goto _LL23;}_LL23:;}
# 1477
s=({struct Cyc_PP_Doc*_tmp38F[5U];({struct Cyc_PP_Doc*_tmp759=Cyc_PP_text(({const char*_tmp390="if (";_tag_fat(_tmp390,sizeof(char),5U);}));_tmp38F[0]=_tmp759;}),({
struct Cyc_PP_Doc*_tmp758=Cyc_Absynpp_exp2doc(e);_tmp38F[1]=_tmp758;}),({
struct Cyc_PP_Doc*_tmp757=Cyc_PP_text(({const char*_tmp391=") ";_tag_fat(_tmp391,sizeof(char),3U);}));_tmp38F[2]=_tmp757;}),({
struct Cyc_PP_Doc*_tmp756=Cyc_Absynpp_block(0,Cyc_Absynpp_stmt2doc(s1,0,0));_tmp38F[3]=_tmp756;}),
print_else?({
struct Cyc_PP_Doc*_tmp755=({struct Cyc_PP_Doc*_tmp392[3U];({struct Cyc_PP_Doc*_tmp754=Cyc_PP_line_doc();_tmp392[0]=_tmp754;}),({
struct Cyc_PP_Doc*_tmp753=Cyc_PP_text(({const char*_tmp393="else ";_tag_fat(_tmp393,sizeof(char),6U);}));_tmp392[1]=_tmp753;}),({
struct Cyc_PP_Doc*_tmp752=Cyc_Absynpp_block(0,Cyc_Absynpp_stmt2doc(s2,0,0));_tmp392[2]=_tmp752;});Cyc_PP_cat(_tag_fat(_tmp392,sizeof(struct Cyc_PP_Doc*),3U));});
# 1482
_tmp38F[4]=_tmp755;}):({
# 1485
struct Cyc_PP_Doc*_tmp751=Cyc_PP_nil_doc();_tmp38F[4]=_tmp751;});Cyc_PP_cat(_tag_fat(_tmp38F,sizeof(struct Cyc_PP_Doc*),5U));});
goto _LL0;}case 5U: _LLB: _tmp37A=(((struct Cyc_Absyn_While_s_Absyn_Raw_stmt_struct*)_tmp369)->f1).f1;_tmp37B=((struct Cyc_Absyn_While_s_Absyn_Raw_stmt_struct*)_tmp369)->f2;_LLC: {struct Cyc_Absyn_Exp*e=_tmp37A;struct Cyc_Absyn_Stmt*s1=_tmp37B;
# 1488
s=({struct Cyc_PP_Doc*_tmp394[4U];({struct Cyc_PP_Doc*_tmp75D=Cyc_PP_text(({const char*_tmp395="while (";_tag_fat(_tmp395,sizeof(char),8U);}));_tmp394[0]=_tmp75D;}),({
struct Cyc_PP_Doc*_tmp75C=Cyc_Absynpp_exp2doc(e);_tmp394[1]=_tmp75C;}),({
struct Cyc_PP_Doc*_tmp75B=Cyc_PP_text(({const char*_tmp396=") ";_tag_fat(_tmp396,sizeof(char),3U);}));_tmp394[2]=_tmp75B;}),({
struct Cyc_PP_Doc*_tmp75A=Cyc_Absynpp_block(0,Cyc_Absynpp_stmt2doc(s1,0,0));_tmp394[3]=_tmp75A;});Cyc_PP_cat(_tag_fat(_tmp394,sizeof(struct Cyc_PP_Doc*),4U));});
goto _LL0;}case 6U: _LLD: _LLE:
 s=Cyc_PP_text(({const char*_tmp397="break;";_tag_fat(_tmp397,sizeof(char),7U);}));goto _LL0;case 7U: _LLF: _LL10:
 s=Cyc_PP_text(({const char*_tmp398="continue;";_tag_fat(_tmp398,sizeof(char),10U);}));goto _LL0;case 8U: _LL11: _tmp379=((struct Cyc_Absyn_Goto_s_Absyn_Raw_stmt_struct*)_tmp369)->f1;_LL12: {struct _fat_ptr*x=_tmp379;
s=Cyc_PP_text((struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp39B=({struct Cyc_String_pa_PrintArg_struct _tmp562;_tmp562.tag=0U,_tmp562.f1=(struct _fat_ptr)((struct _fat_ptr)*x);_tmp562;});void*_tmp399[1U];_tmp399[0]=& _tmp39B;({struct _fat_ptr _tmp75E=({const char*_tmp39A="goto %s;";_tag_fat(_tmp39A,sizeof(char),9U);});Cyc_aprintf(_tmp75E,_tag_fat(_tmp399,sizeof(void*),1U));});}));goto _LL0;}case 9U: _LL13: _tmp375=((struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct*)_tmp369)->f1;_tmp376=(((struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct*)_tmp369)->f2).f1;_tmp377=(((struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct*)_tmp369)->f3).f1;_tmp378=((struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct*)_tmp369)->f4;_LL14: {struct Cyc_Absyn_Exp*e1=_tmp375;struct Cyc_Absyn_Exp*e2=_tmp376;struct Cyc_Absyn_Exp*e3=_tmp377;struct Cyc_Absyn_Stmt*s1=_tmp378;
# 1497
s=({struct Cyc_PP_Doc*_tmp39C[8U];({struct Cyc_PP_Doc*_tmp766=Cyc_PP_text(({const char*_tmp39D="for(";_tag_fat(_tmp39D,sizeof(char),5U);}));_tmp39C[0]=_tmp766;}),({
struct Cyc_PP_Doc*_tmp765=Cyc_Absynpp_exp2doc(e1);_tmp39C[1]=_tmp765;}),({
struct Cyc_PP_Doc*_tmp764=Cyc_PP_text(({const char*_tmp39E="; ";_tag_fat(_tmp39E,sizeof(char),3U);}));_tmp39C[2]=_tmp764;}),({
struct Cyc_PP_Doc*_tmp763=Cyc_Absynpp_exp2doc(e2);_tmp39C[3]=_tmp763;}),({
struct Cyc_PP_Doc*_tmp762=Cyc_PP_text(({const char*_tmp39F="; ";_tag_fat(_tmp39F,sizeof(char),3U);}));_tmp39C[4]=_tmp762;}),({
struct Cyc_PP_Doc*_tmp761=Cyc_Absynpp_exp2doc(e3);_tmp39C[5]=_tmp761;}),({
struct Cyc_PP_Doc*_tmp760=Cyc_PP_text(({const char*_tmp3A0=") ";_tag_fat(_tmp3A0,sizeof(char),3U);}));_tmp39C[6]=_tmp760;}),({
struct Cyc_PP_Doc*_tmp75F=Cyc_Absynpp_block(0,Cyc_Absynpp_stmt2doc(s1,0,0));_tmp39C[7]=_tmp75F;});Cyc_PP_cat(_tag_fat(_tmp39C,sizeof(struct Cyc_PP_Doc*),8U));});
goto _LL0;}case 10U: _LL15: _tmp373=((struct Cyc_Absyn_Switch_s_Absyn_Raw_stmt_struct*)_tmp369)->f1;_tmp374=((struct Cyc_Absyn_Switch_s_Absyn_Raw_stmt_struct*)_tmp369)->f2;_LL16: {struct Cyc_Absyn_Exp*e=_tmp373;struct Cyc_List_List*ss=_tmp374;
# 1507
s=({struct Cyc_PP_Doc*_tmp3A1[8U];({struct Cyc_PP_Doc*_tmp76E=Cyc_PP_text(({const char*_tmp3A2="switch (";_tag_fat(_tmp3A2,sizeof(char),9U);}));_tmp3A1[0]=_tmp76E;}),({
struct Cyc_PP_Doc*_tmp76D=Cyc_Absynpp_exp2doc(e);_tmp3A1[1]=_tmp76D;}),({
struct Cyc_PP_Doc*_tmp76C=Cyc_PP_text(({const char*_tmp3A3=") ";_tag_fat(_tmp3A3,sizeof(char),3U);}));_tmp3A1[2]=_tmp76C;}),({
struct Cyc_PP_Doc*_tmp76B=Cyc_Absynpp_lb();_tmp3A1[3]=_tmp76B;}),({
struct Cyc_PP_Doc*_tmp76A=Cyc_PP_line_doc();_tmp3A1[4]=_tmp76A;}),({
struct Cyc_PP_Doc*_tmp769=Cyc_Absynpp_switchclauses2doc(ss);_tmp3A1[5]=_tmp769;}),({
struct Cyc_PP_Doc*_tmp768=Cyc_PP_line_doc();_tmp3A1[6]=_tmp768;}),({
struct Cyc_PP_Doc*_tmp767=Cyc_Absynpp_rb();_tmp3A1[7]=_tmp767;});Cyc_PP_cat(_tag_fat(_tmp3A1,sizeof(struct Cyc_PP_Doc*),8U));});
goto _LL0;}case 11U: if(((struct Cyc_Absyn_Fallthru_s_Absyn_Raw_stmt_struct*)_tmp369)->f1 == 0){_LL17: _LL18:
 s=Cyc_PP_text(({const char*_tmp3A4="fallthru;";_tag_fat(_tmp3A4,sizeof(char),10U);}));goto _LL0;}else{_LL19: _tmp372=((struct Cyc_Absyn_Fallthru_s_Absyn_Raw_stmt_struct*)_tmp369)->f1;_LL1A: {struct Cyc_List_List*es=_tmp372;
# 1518
s=({struct Cyc_PP_Doc*_tmp3A5[3U];({struct Cyc_PP_Doc*_tmp771=Cyc_PP_text(({const char*_tmp3A6="fallthru(";_tag_fat(_tmp3A6,sizeof(char),10U);}));_tmp3A5[0]=_tmp771;}),({struct Cyc_PP_Doc*_tmp770=Cyc_Absynpp_exps2doc_prec(20,es);_tmp3A5[1]=_tmp770;}),({struct Cyc_PP_Doc*_tmp76F=Cyc_PP_text(({const char*_tmp3A7=");";_tag_fat(_tmp3A7,sizeof(char),3U);}));_tmp3A5[2]=_tmp76F;});Cyc_PP_cat(_tag_fat(_tmp3A5,sizeof(struct Cyc_PP_Doc*),3U));});goto _LL0;}}case 12U: _LL1B: _tmp370=((struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct*)_tmp369)->f1;_tmp371=((struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct*)_tmp369)->f2;_LL1C: {struct Cyc_Absyn_Decl*d=_tmp370;struct Cyc_Absyn_Stmt*s1=_tmp371;
# 1520
struct _tuple14 _stmttmp1B=Cyc_Absynpp_shadows(d,varsinblock);struct Cyc_List_List*_tmp3A9;int _tmp3A8;_LL29: _tmp3A8=_stmttmp1B.f1;_tmp3A9=_stmttmp1B.f2;_LL2A: {int newblock=_tmp3A8;struct Cyc_List_List*newvarsinblock=_tmp3A9;
s=({struct Cyc_PP_Doc*_tmp3AA[3U];({struct Cyc_PP_Doc*_tmp774=Cyc_Absynpp_decl2doc(d);_tmp3AA[0]=_tmp774;}),({struct Cyc_PP_Doc*_tmp773=Cyc_PP_line_doc();_tmp3AA[1]=_tmp773;}),({struct Cyc_PP_Doc*_tmp772=Cyc_Absynpp_stmt2doc(s1,stmtexp,newvarsinblock);_tmp3AA[2]=_tmp772;});Cyc_PP_cat(_tag_fat(_tmp3AA,sizeof(struct Cyc_PP_Doc*),3U));});
if(newblock)s=Cyc_Absynpp_block(stmtexp,s);goto _LL0;}}case 13U: _LL1D: _tmp36E=((struct Cyc_Absyn_Label_s_Absyn_Raw_stmt_struct*)_tmp369)->f1;_tmp36F=((struct Cyc_Absyn_Label_s_Absyn_Raw_stmt_struct*)_tmp369)->f2;_LL1E: {struct _fat_ptr*x=_tmp36E;struct Cyc_Absyn_Stmt*s1=_tmp36F;
# 1525
if(Cyc_Absynpp_decls_first && Cyc_Absynpp_is_declaration(s1))
s=({struct Cyc_PP_Doc*_tmp3AB[3U];({struct Cyc_PP_Doc*_tmp778=Cyc_PP_textptr(x);_tmp3AB[0]=_tmp778;}),({struct Cyc_PP_Doc*_tmp777=Cyc_PP_text(({const char*_tmp3AC=": ";_tag_fat(_tmp3AC,sizeof(char),3U);}));_tmp3AB[1]=_tmp777;}),({struct Cyc_PP_Doc*_tmp776=({int _tmp775=stmtexp;Cyc_Absynpp_block(_tmp775,Cyc_Absynpp_stmt2doc(s1,stmtexp,0));});_tmp3AB[2]=_tmp776;});Cyc_PP_cat(_tag_fat(_tmp3AB,sizeof(struct Cyc_PP_Doc*),3U));});else{
# 1528
s=({struct Cyc_PP_Doc*_tmp3AD[3U];({struct Cyc_PP_Doc*_tmp77B=Cyc_PP_textptr(x);_tmp3AD[0]=_tmp77B;}),({struct Cyc_PP_Doc*_tmp77A=Cyc_PP_text(({const char*_tmp3AE=": ";_tag_fat(_tmp3AE,sizeof(char),3U);}));_tmp3AD[1]=_tmp77A;}),({struct Cyc_PP_Doc*_tmp779=Cyc_Absynpp_stmt2doc(s1,stmtexp,varsinblock);_tmp3AD[2]=_tmp779;});Cyc_PP_cat(_tag_fat(_tmp3AD,sizeof(struct Cyc_PP_Doc*),3U));});}
goto _LL0;}case 14U: _LL1F: _tmp36C=((struct Cyc_Absyn_Do_s_Absyn_Raw_stmt_struct*)_tmp369)->f1;_tmp36D=(((struct Cyc_Absyn_Do_s_Absyn_Raw_stmt_struct*)_tmp369)->f2).f1;_LL20: {struct Cyc_Absyn_Stmt*s1=_tmp36C;struct Cyc_Absyn_Exp*e=_tmp36D;
# 1531
s=({struct Cyc_PP_Doc*_tmp3AF[5U];({struct Cyc_PP_Doc*_tmp780=Cyc_PP_text(({const char*_tmp3B0="do ";_tag_fat(_tmp3B0,sizeof(char),4U);}));_tmp3AF[0]=_tmp780;}),({
struct Cyc_PP_Doc*_tmp77F=Cyc_Absynpp_block(0,Cyc_Absynpp_stmt2doc(s1,0,0));_tmp3AF[1]=_tmp77F;}),({
struct Cyc_PP_Doc*_tmp77E=Cyc_PP_text(({const char*_tmp3B1=" while (";_tag_fat(_tmp3B1,sizeof(char),9U);}));_tmp3AF[2]=_tmp77E;}),({
struct Cyc_PP_Doc*_tmp77D=Cyc_Absynpp_exp2doc(e);_tmp3AF[3]=_tmp77D;}),({
struct Cyc_PP_Doc*_tmp77C=Cyc_PP_text(({const char*_tmp3B2=");";_tag_fat(_tmp3B2,sizeof(char),3U);}));_tmp3AF[4]=_tmp77C;});Cyc_PP_cat(_tag_fat(_tmp3AF,sizeof(struct Cyc_PP_Doc*),5U));});
goto _LL0;}default: _LL21: _tmp36A=((struct Cyc_Absyn_TryCatch_s_Absyn_Raw_stmt_struct*)_tmp369)->f1;_tmp36B=((struct Cyc_Absyn_TryCatch_s_Absyn_Raw_stmt_struct*)_tmp369)->f2;_LL22: {struct Cyc_Absyn_Stmt*s1=_tmp36A;struct Cyc_List_List*ss=_tmp36B;
# 1538
s=({struct Cyc_PP_Doc*_tmp3B3[4U];({struct Cyc_PP_Doc*_tmp784=Cyc_PP_text(({const char*_tmp3B4="try ";_tag_fat(_tmp3B4,sizeof(char),5U);}));_tmp3B3[0]=_tmp784;}),({
struct Cyc_PP_Doc*_tmp783=Cyc_Absynpp_block(0,Cyc_Absynpp_stmt2doc(s1,0,0));_tmp3B3[1]=_tmp783;}),({
struct Cyc_PP_Doc*_tmp782=Cyc_PP_text(({const char*_tmp3B5=" catch ";_tag_fat(_tmp3B5,sizeof(char),8U);}));_tmp3B3[2]=_tmp782;}),({
struct Cyc_PP_Doc*_tmp781=Cyc_Absynpp_block(0,Cyc_Absynpp_switchclauses2doc(ss));_tmp3B3[3]=_tmp781;});Cyc_PP_cat(_tag_fat(_tmp3B3,sizeof(struct Cyc_PP_Doc*),4U));});
goto _LL0;}}_LL0:;}
# 1544
return s;}
# 607 "absynpp.cyc"
struct Cyc_PP_Doc*Cyc_Absynpp_pat2doc(struct Cyc_Absyn_Pat*p){
# 1548 "absynpp.cyc"
struct Cyc_PP_Doc*s;
{void*_stmttmp1C=p->r;void*_tmp3B6=_stmttmp1C;struct Cyc_Absyn_Exp*_tmp3B7;int _tmp3BA;struct Cyc_List_List*_tmp3B9;struct Cyc_Absyn_Datatypefield*_tmp3B8;struct Cyc_Absyn_Datatypefield*_tmp3BB;struct Cyc_Absyn_Enumfield*_tmp3BC;struct Cyc_Absyn_Enumfield*_tmp3BD;int _tmp3C0;struct Cyc_List_List*_tmp3BF;struct Cyc_List_List*_tmp3BE;int _tmp3C4;struct Cyc_List_List*_tmp3C3;struct Cyc_List_List*_tmp3C2;union Cyc_Absyn_AggrInfo _tmp3C1;int _tmp3C7;struct Cyc_List_List*_tmp3C6;struct _tuple0*_tmp3C5;struct _tuple0*_tmp3C8;struct Cyc_Absyn_Pat*_tmp3CA;struct Cyc_Absyn_Vardecl*_tmp3C9;struct Cyc_Absyn_Vardecl*_tmp3CB;struct Cyc_Absyn_Pat*_tmp3CC;int _tmp3CE;struct Cyc_List_List*_tmp3CD;struct Cyc_Absyn_Vardecl*_tmp3D0;struct Cyc_Absyn_Tvar*_tmp3CF;struct Cyc_Absyn_Vardecl*_tmp3D2;struct Cyc_Absyn_Tvar*_tmp3D1;struct Cyc_Absyn_Pat*_tmp3D4;struct Cyc_Absyn_Vardecl*_tmp3D3;struct Cyc_Absyn_Vardecl*_tmp3D5;struct _fat_ptr _tmp3D6;char _tmp3D7;int _tmp3D9;enum Cyc_Absyn_Sign _tmp3D8;switch(*((int*)_tmp3B6)){case 0U: _LL1: _LL2:
 s=Cyc_PP_text(({const char*_tmp3DA="_";_tag_fat(_tmp3DA,sizeof(char),2U);}));goto _LL0;case 9U: _LL3: _LL4:
 s=Cyc_PP_text(({const char*_tmp3DB="NULL";_tag_fat(_tmp3DB,sizeof(char),5U);}));goto _LL0;case 10U: _LL5: _tmp3D8=((struct Cyc_Absyn_Int_p_Absyn_Raw_pat_struct*)_tmp3B6)->f1;_tmp3D9=((struct Cyc_Absyn_Int_p_Absyn_Raw_pat_struct*)_tmp3B6)->f2;_LL6: {enum Cyc_Absyn_Sign sg=_tmp3D8;int i=_tmp3D9;
# 1553
if((int)sg != (int)1U)
s=Cyc_PP_text((struct _fat_ptr)({struct Cyc_Int_pa_PrintArg_struct _tmp3DE=({struct Cyc_Int_pa_PrintArg_struct _tmp563;_tmp563.tag=1U,_tmp563.f1=(unsigned long)i;_tmp563;});void*_tmp3DC[1U];_tmp3DC[0]=& _tmp3DE;({struct _fat_ptr _tmp785=({const char*_tmp3DD="%d";_tag_fat(_tmp3DD,sizeof(char),3U);});Cyc_aprintf(_tmp785,_tag_fat(_tmp3DC,sizeof(void*),1U));});}));else{
s=Cyc_PP_text((struct _fat_ptr)({struct Cyc_Int_pa_PrintArg_struct _tmp3E1=({struct Cyc_Int_pa_PrintArg_struct _tmp564;_tmp564.tag=1U,_tmp564.f1=(unsigned)i;_tmp564;});void*_tmp3DF[1U];_tmp3DF[0]=& _tmp3E1;({struct _fat_ptr _tmp786=({const char*_tmp3E0="%u";_tag_fat(_tmp3E0,sizeof(char),3U);});Cyc_aprintf(_tmp786,_tag_fat(_tmp3DF,sizeof(void*),1U));});}));}
goto _LL0;}case 11U: _LL7: _tmp3D7=((struct Cyc_Absyn_Char_p_Absyn_Raw_pat_struct*)_tmp3B6)->f1;_LL8: {char ch=_tmp3D7;
s=Cyc_PP_text((struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp3E4=({struct Cyc_String_pa_PrintArg_struct _tmp565;_tmp565.tag=0U,({struct _fat_ptr _tmp787=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_char_escape(ch));_tmp565.f1=_tmp787;});_tmp565;});void*_tmp3E2[1U];_tmp3E2[0]=& _tmp3E4;({struct _fat_ptr _tmp788=({const char*_tmp3E3="'%s'";_tag_fat(_tmp3E3,sizeof(char),5U);});Cyc_aprintf(_tmp788,_tag_fat(_tmp3E2,sizeof(void*),1U));});}));goto _LL0;}case 12U: _LL9: _tmp3D6=((struct Cyc_Absyn_Float_p_Absyn_Raw_pat_struct*)_tmp3B6)->f1;_LLA: {struct _fat_ptr x=_tmp3D6;
s=Cyc_PP_text(x);goto _LL0;}case 1U: if(((struct Cyc_Absyn_Wild_p_Absyn_Raw_pat_struct*)((struct Cyc_Absyn_Pat*)((struct Cyc_Absyn_Var_p_Absyn_Raw_pat_struct*)_tmp3B6)->f2)->r)->tag == 0U){_LLB: _tmp3D5=((struct Cyc_Absyn_Var_p_Absyn_Raw_pat_struct*)_tmp3B6)->f1;_LLC: {struct Cyc_Absyn_Vardecl*vd=_tmp3D5;
# 1560
s=Cyc_Absynpp_qvar2doc(vd->name);goto _LL0;}}else{_LLD: _tmp3D3=((struct Cyc_Absyn_Var_p_Absyn_Raw_pat_struct*)_tmp3B6)->f1;_tmp3D4=((struct Cyc_Absyn_Var_p_Absyn_Raw_pat_struct*)_tmp3B6)->f2;_LLE: {struct Cyc_Absyn_Vardecl*vd=_tmp3D3;struct Cyc_Absyn_Pat*p2=_tmp3D4;
# 1562
s=({struct Cyc_PP_Doc*_tmp3E5[3U];({struct Cyc_PP_Doc*_tmp78B=Cyc_Absynpp_qvar2doc(vd->name);_tmp3E5[0]=_tmp78B;}),({struct Cyc_PP_Doc*_tmp78A=Cyc_PP_text(({const char*_tmp3E6=" as ";_tag_fat(_tmp3E6,sizeof(char),5U);}));_tmp3E5[1]=_tmp78A;}),({struct Cyc_PP_Doc*_tmp789=Cyc_Absynpp_pat2doc(p2);_tmp3E5[2]=_tmp789;});Cyc_PP_cat(_tag_fat(_tmp3E5,sizeof(struct Cyc_PP_Doc*),3U));});goto _LL0;}}case 2U: _LLF: _tmp3D1=((struct Cyc_Absyn_AliasVar_p_Absyn_Raw_pat_struct*)_tmp3B6)->f1;_tmp3D2=((struct Cyc_Absyn_AliasVar_p_Absyn_Raw_pat_struct*)_tmp3B6)->f2;_LL10: {struct Cyc_Absyn_Tvar*tv=_tmp3D1;struct Cyc_Absyn_Vardecl*vd=_tmp3D2;
# 1564
s=({struct Cyc_PP_Doc*_tmp3E7[5U];({struct Cyc_PP_Doc*_tmp790=Cyc_PP_text(({const char*_tmp3E8="alias";_tag_fat(_tmp3E8,sizeof(char),6U);}));_tmp3E7[0]=_tmp790;}),({struct Cyc_PP_Doc*_tmp78F=Cyc_PP_text(({const char*_tmp3E9=" <";_tag_fat(_tmp3E9,sizeof(char),3U);}));_tmp3E7[1]=_tmp78F;}),({struct Cyc_PP_Doc*_tmp78E=Cyc_Absynpp_tvar2doc(tv);_tmp3E7[2]=_tmp78E;}),({struct Cyc_PP_Doc*_tmp78D=Cyc_PP_text(({const char*_tmp3EA="> ";_tag_fat(_tmp3EA,sizeof(char),3U);}));_tmp3E7[3]=_tmp78D;}),({
struct Cyc_PP_Doc*_tmp78C=Cyc_Absynpp_vardecl2doc(vd);_tmp3E7[4]=_tmp78C;});Cyc_PP_cat(_tag_fat(_tmp3E7,sizeof(struct Cyc_PP_Doc*),5U));});
goto _LL0;}case 4U: _LL11: _tmp3CF=((struct Cyc_Absyn_TagInt_p_Absyn_Raw_pat_struct*)_tmp3B6)->f1;_tmp3D0=((struct Cyc_Absyn_TagInt_p_Absyn_Raw_pat_struct*)_tmp3B6)->f2;_LL12: {struct Cyc_Absyn_Tvar*tv=_tmp3CF;struct Cyc_Absyn_Vardecl*vd=_tmp3D0;
# 1568
s=({struct Cyc_PP_Doc*_tmp3EB[4U];({struct Cyc_PP_Doc*_tmp794=Cyc_Absynpp_qvar2doc(vd->name);_tmp3EB[0]=_tmp794;}),({struct Cyc_PP_Doc*_tmp793=Cyc_PP_text(({const char*_tmp3EC="<";_tag_fat(_tmp3EC,sizeof(char),2U);}));_tmp3EB[1]=_tmp793;}),({struct Cyc_PP_Doc*_tmp792=Cyc_Absynpp_tvar2doc(tv);_tmp3EB[2]=_tmp792;}),({struct Cyc_PP_Doc*_tmp791=Cyc_PP_text(({const char*_tmp3ED=">";_tag_fat(_tmp3ED,sizeof(char),2U);}));_tmp3EB[3]=_tmp791;});Cyc_PP_cat(_tag_fat(_tmp3EB,sizeof(struct Cyc_PP_Doc*),4U));});
goto _LL0;}case 5U: _LL13: _tmp3CD=((struct Cyc_Absyn_Tuple_p_Absyn_Raw_pat_struct*)_tmp3B6)->f1;_tmp3CE=((struct Cyc_Absyn_Tuple_p_Absyn_Raw_pat_struct*)_tmp3B6)->f2;_LL14: {struct Cyc_List_List*ts=_tmp3CD;int dots=_tmp3CE;
# 1571
s=({struct Cyc_PP_Doc*_tmp3EE[4U];({struct Cyc_PP_Doc*_tmp79B=Cyc_Absynpp_dollar();_tmp3EE[0]=_tmp79B;}),({struct Cyc_PP_Doc*_tmp79A=Cyc_PP_text(({const char*_tmp3EF="(";_tag_fat(_tmp3EF,sizeof(char),2U);}));_tmp3EE[1]=_tmp79A;}),({struct Cyc_PP_Doc*_tmp799=({struct Cyc_PP_Doc*(*_tmp798)(struct Cyc_PP_Doc*(*pp)(struct Cyc_Absyn_Pat*),struct _fat_ptr sep,struct Cyc_List_List*l)=({struct Cyc_PP_Doc*(*_tmp3F0)(struct Cyc_PP_Doc*(*pp)(struct Cyc_Absyn_Pat*),struct _fat_ptr sep,struct Cyc_List_List*l)=(struct Cyc_PP_Doc*(*)(struct Cyc_PP_Doc*(*pp)(struct Cyc_Absyn_Pat*),struct _fat_ptr sep,struct Cyc_List_List*l))Cyc_PP_ppseq;_tmp3F0;});struct _fat_ptr _tmp797=({const char*_tmp3F1=",";_tag_fat(_tmp3F1,sizeof(char),2U);});_tmp798(Cyc_Absynpp_pat2doc,_tmp797,ts);});_tmp3EE[2]=_tmp799;}),
dots?({struct Cyc_PP_Doc*_tmp796=Cyc_PP_text(({const char*_tmp3F2=", ...)";_tag_fat(_tmp3F2,sizeof(char),7U);}));_tmp3EE[3]=_tmp796;}):({struct Cyc_PP_Doc*_tmp795=Cyc_PP_text(({const char*_tmp3F3=")";_tag_fat(_tmp3F3,sizeof(char),2U);}));_tmp3EE[3]=_tmp795;});Cyc_PP_cat(_tag_fat(_tmp3EE,sizeof(struct Cyc_PP_Doc*),4U));});
goto _LL0;}case 6U: _LL15: _tmp3CC=((struct Cyc_Absyn_Pointer_p_Absyn_Raw_pat_struct*)_tmp3B6)->f1;_LL16: {struct Cyc_Absyn_Pat*p2=_tmp3CC;
# 1575
s=({struct Cyc_PP_Doc*_tmp3F4[2U];({struct Cyc_PP_Doc*_tmp79D=Cyc_PP_text(({const char*_tmp3F5="&";_tag_fat(_tmp3F5,sizeof(char),2U);}));_tmp3F4[0]=_tmp79D;}),({struct Cyc_PP_Doc*_tmp79C=Cyc_Absynpp_pat2doc(p2);_tmp3F4[1]=_tmp79C;});Cyc_PP_cat(_tag_fat(_tmp3F4,sizeof(struct Cyc_PP_Doc*),2U));});
goto _LL0;}case 3U: if(((struct Cyc_Absyn_Wild_p_Absyn_Raw_pat_struct*)((struct Cyc_Absyn_Pat*)((struct Cyc_Absyn_Reference_p_Absyn_Raw_pat_struct*)_tmp3B6)->f2)->r)->tag == 0U){_LL17: _tmp3CB=((struct Cyc_Absyn_Reference_p_Absyn_Raw_pat_struct*)_tmp3B6)->f1;_LL18: {struct Cyc_Absyn_Vardecl*vd=_tmp3CB;
# 1578
s=({struct Cyc_PP_Doc*_tmp3F6[2U];({struct Cyc_PP_Doc*_tmp79F=Cyc_PP_text(({const char*_tmp3F7="*";_tag_fat(_tmp3F7,sizeof(char),2U);}));_tmp3F6[0]=_tmp79F;}),({struct Cyc_PP_Doc*_tmp79E=Cyc_Absynpp_qvar2doc(vd->name);_tmp3F6[1]=_tmp79E;});Cyc_PP_cat(_tag_fat(_tmp3F6,sizeof(struct Cyc_PP_Doc*),2U));});
goto _LL0;}}else{_LL19: _tmp3C9=((struct Cyc_Absyn_Reference_p_Absyn_Raw_pat_struct*)_tmp3B6)->f1;_tmp3CA=((struct Cyc_Absyn_Reference_p_Absyn_Raw_pat_struct*)_tmp3B6)->f2;_LL1A: {struct Cyc_Absyn_Vardecl*vd=_tmp3C9;struct Cyc_Absyn_Pat*p2=_tmp3CA;
# 1581
s=({struct Cyc_PP_Doc*_tmp3F8[4U];({struct Cyc_PP_Doc*_tmp7A3=Cyc_PP_text(({const char*_tmp3F9="*";_tag_fat(_tmp3F9,sizeof(char),2U);}));_tmp3F8[0]=_tmp7A3;}),({struct Cyc_PP_Doc*_tmp7A2=Cyc_Absynpp_qvar2doc(vd->name);_tmp3F8[1]=_tmp7A2;}),({struct Cyc_PP_Doc*_tmp7A1=Cyc_PP_text(({const char*_tmp3FA=" as ";_tag_fat(_tmp3FA,sizeof(char),5U);}));_tmp3F8[2]=_tmp7A1;}),({struct Cyc_PP_Doc*_tmp7A0=Cyc_Absynpp_pat2doc(p2);_tmp3F8[3]=_tmp7A0;});Cyc_PP_cat(_tag_fat(_tmp3F8,sizeof(struct Cyc_PP_Doc*),4U));});
goto _LL0;}}case 15U: _LL1B: _tmp3C8=((struct Cyc_Absyn_UnknownId_p_Absyn_Raw_pat_struct*)_tmp3B6)->f1;_LL1C: {struct _tuple0*q=_tmp3C8;
# 1584
s=Cyc_Absynpp_qvar2doc(q);
goto _LL0;}case 16U: _LL1D: _tmp3C5=((struct Cyc_Absyn_UnknownCall_p_Absyn_Raw_pat_struct*)_tmp3B6)->f1;_tmp3C6=((struct Cyc_Absyn_UnknownCall_p_Absyn_Raw_pat_struct*)_tmp3B6)->f2;_tmp3C7=((struct Cyc_Absyn_UnknownCall_p_Absyn_Raw_pat_struct*)_tmp3B6)->f3;_LL1E: {struct _tuple0*q=_tmp3C5;struct Cyc_List_List*ps=_tmp3C6;int dots=_tmp3C7;
# 1587
struct _fat_ptr term=dots?({const char*_tmp3FF=", ...)";_tag_fat(_tmp3FF,sizeof(char),7U);}):({const char*_tmp400=")";_tag_fat(_tmp400,sizeof(char),2U);});
s=({struct Cyc_PP_Doc*_tmp3FB[2U];({struct Cyc_PP_Doc*_tmp7A9=Cyc_Absynpp_qvar2doc(q);_tmp3FB[0]=_tmp7A9;}),({struct Cyc_PP_Doc*_tmp7A8=({struct _fat_ptr _tmp7A7=({const char*_tmp3FC="(";_tag_fat(_tmp3FC,sizeof(char),2U);});struct _fat_ptr _tmp7A6=term;struct _fat_ptr _tmp7A5=({const char*_tmp3FD=",";_tag_fat(_tmp3FD,sizeof(char),2U);});Cyc_PP_group(_tmp7A7,_tmp7A6,_tmp7A5,({struct Cyc_List_List*(*_tmp7A4)(struct Cyc_PP_Doc*(*f)(struct Cyc_Absyn_Pat*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp3FE)(struct Cyc_PP_Doc*(*f)(struct Cyc_Absyn_Pat*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_PP_Doc*(*f)(struct Cyc_Absyn_Pat*),struct Cyc_List_List*x))Cyc_List_map;_tmp3FE;});_tmp7A4(Cyc_Absynpp_pat2doc,ps);}));});_tmp3FB[1]=_tmp7A8;});Cyc_PP_cat(_tag_fat(_tmp3FB,sizeof(struct Cyc_PP_Doc*),2U));});
goto _LL0;}case 7U: if(((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmp3B6)->f1 != 0){_LL1F: _tmp3C1=*((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmp3B6)->f1;_tmp3C2=((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmp3B6)->f2;_tmp3C3=((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmp3B6)->f3;_tmp3C4=((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmp3B6)->f4;_LL20: {union Cyc_Absyn_AggrInfo info=_tmp3C1;struct Cyc_List_List*exists=_tmp3C2;struct Cyc_List_List*dps=_tmp3C3;int dots=_tmp3C4;
# 1591
struct _fat_ptr term=dots?({const char*_tmp40A=", ...}";_tag_fat(_tmp40A,sizeof(char),7U);}):({const char*_tmp40B="}";_tag_fat(_tmp40B,sizeof(char),2U);});
struct _tuple11 _stmttmp1D=Cyc_Absyn_aggr_kinded_name(info);struct _tuple0*_tmp401;_LL2E: _tmp401=_stmttmp1D.f2;_LL2F: {struct _tuple0*n=_tmp401;
s=({struct Cyc_PP_Doc*_tmp402[4U];({struct Cyc_PP_Doc*_tmp7B5=Cyc_Absynpp_qvar2doc(n);_tmp402[0]=_tmp7B5;}),({struct Cyc_PP_Doc*_tmp7B4=Cyc_Absynpp_lb();_tmp402[1]=_tmp7B4;}),({
struct Cyc_PP_Doc*_tmp7B3=({struct _fat_ptr _tmp7B2=({const char*_tmp403="[";_tag_fat(_tmp403,sizeof(char),2U);});struct _fat_ptr _tmp7B1=({const char*_tmp404="]";_tag_fat(_tmp404,sizeof(char),2U);});struct _fat_ptr _tmp7B0=({const char*_tmp405=",";_tag_fat(_tmp405,sizeof(char),2U);});Cyc_PP_egroup(_tmp7B2,_tmp7B1,_tmp7B0,({struct Cyc_List_List*(*_tmp7AF)(struct Cyc_PP_Doc*(*f)(struct Cyc_Absyn_Tvar*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp406)(struct Cyc_PP_Doc*(*f)(struct Cyc_Absyn_Tvar*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_PP_Doc*(*f)(struct Cyc_Absyn_Tvar*),struct Cyc_List_List*x))Cyc_List_map;_tmp406;});_tmp7AF(Cyc_Absynpp_tvar2doc,exists);}));});_tmp402[2]=_tmp7B3;}),({
struct Cyc_PP_Doc*_tmp7AE=({struct _fat_ptr _tmp7AD=({const char*_tmp407="";_tag_fat(_tmp407,sizeof(char),1U);});struct _fat_ptr _tmp7AC=term;struct _fat_ptr _tmp7AB=({const char*_tmp408=",";_tag_fat(_tmp408,sizeof(char),2U);});Cyc_PP_group(_tmp7AD,_tmp7AC,_tmp7AB,({struct Cyc_List_List*(*_tmp7AA)(struct Cyc_PP_Doc*(*f)(struct _tuple15*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp409)(struct Cyc_PP_Doc*(*f)(struct _tuple15*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_PP_Doc*(*f)(struct _tuple15*),struct Cyc_List_List*x))Cyc_List_map;_tmp409;});_tmp7AA(Cyc_Absynpp_dp2doc,dps);}));});_tmp402[3]=_tmp7AE;});Cyc_PP_cat(_tag_fat(_tmp402,sizeof(struct Cyc_PP_Doc*),4U));});
goto _LL0;}}}else{_LL21: _tmp3BE=((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmp3B6)->f2;_tmp3BF=((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmp3B6)->f3;_tmp3C0=((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmp3B6)->f4;_LL22: {struct Cyc_List_List*exists=_tmp3BE;struct Cyc_List_List*dps=_tmp3BF;int dots=_tmp3C0;
# 1598
struct _fat_ptr term=dots?({const char*_tmp414=", ...}";_tag_fat(_tmp414,sizeof(char),7U);}):({const char*_tmp415="}";_tag_fat(_tmp415,sizeof(char),2U);});
s=({struct Cyc_PP_Doc*_tmp40C[3U];({struct Cyc_PP_Doc*_tmp7C0=Cyc_Absynpp_lb();_tmp40C[0]=_tmp7C0;}),({
struct Cyc_PP_Doc*_tmp7BF=({struct _fat_ptr _tmp7BE=({const char*_tmp40D="[";_tag_fat(_tmp40D,sizeof(char),2U);});struct _fat_ptr _tmp7BD=({const char*_tmp40E="]";_tag_fat(_tmp40E,sizeof(char),2U);});struct _fat_ptr _tmp7BC=({const char*_tmp40F=",";_tag_fat(_tmp40F,sizeof(char),2U);});Cyc_PP_egroup(_tmp7BE,_tmp7BD,_tmp7BC,({struct Cyc_List_List*(*_tmp7BB)(struct Cyc_PP_Doc*(*f)(struct Cyc_Absyn_Tvar*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp410)(struct Cyc_PP_Doc*(*f)(struct Cyc_Absyn_Tvar*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_PP_Doc*(*f)(struct Cyc_Absyn_Tvar*),struct Cyc_List_List*x))Cyc_List_map;_tmp410;});_tmp7BB(Cyc_Absynpp_tvar2doc,exists);}));});_tmp40C[1]=_tmp7BF;}),({
struct Cyc_PP_Doc*_tmp7BA=({struct _fat_ptr _tmp7B9=({const char*_tmp411="";_tag_fat(_tmp411,sizeof(char),1U);});struct _fat_ptr _tmp7B8=term;struct _fat_ptr _tmp7B7=({const char*_tmp412=",";_tag_fat(_tmp412,sizeof(char),2U);});Cyc_PP_group(_tmp7B9,_tmp7B8,_tmp7B7,({struct Cyc_List_List*(*_tmp7B6)(struct Cyc_PP_Doc*(*f)(struct _tuple15*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp413)(struct Cyc_PP_Doc*(*f)(struct _tuple15*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_PP_Doc*(*f)(struct _tuple15*),struct Cyc_List_List*x))Cyc_List_map;_tmp413;});_tmp7B6(Cyc_Absynpp_dp2doc,dps);}));});_tmp40C[2]=_tmp7BA;});Cyc_PP_cat(_tag_fat(_tmp40C,sizeof(struct Cyc_PP_Doc*),3U));});
goto _LL0;}}case 13U: _LL23: _tmp3BD=((struct Cyc_Absyn_Enum_p_Absyn_Raw_pat_struct*)_tmp3B6)->f2;_LL24: {struct Cyc_Absyn_Enumfield*ef=_tmp3BD;
_tmp3BC=ef;goto _LL26;}case 14U: _LL25: _tmp3BC=((struct Cyc_Absyn_AnonEnum_p_Absyn_Raw_pat_struct*)_tmp3B6)->f2;_LL26: {struct Cyc_Absyn_Enumfield*ef=_tmp3BC;
s=Cyc_Absynpp_qvar2doc(ef->name);goto _LL0;}case 8U: if(((struct Cyc_Absyn_Datatype_p_Absyn_Raw_pat_struct*)_tmp3B6)->f3 == 0){_LL27: _tmp3BB=((struct Cyc_Absyn_Datatype_p_Absyn_Raw_pat_struct*)_tmp3B6)->f2;_LL28: {struct Cyc_Absyn_Datatypefield*tuf=_tmp3BB;
s=Cyc_Absynpp_qvar2doc(tuf->name);goto _LL0;}}else{_LL29: _tmp3B8=((struct Cyc_Absyn_Datatype_p_Absyn_Raw_pat_struct*)_tmp3B6)->f2;_tmp3B9=((struct Cyc_Absyn_Datatype_p_Absyn_Raw_pat_struct*)_tmp3B6)->f3;_tmp3BA=((struct Cyc_Absyn_Datatype_p_Absyn_Raw_pat_struct*)_tmp3B6)->f4;_LL2A: {struct Cyc_Absyn_Datatypefield*tuf=_tmp3B8;struct Cyc_List_List*ps=_tmp3B9;int dots=_tmp3BA;
# 1607
struct _fat_ptr term=dots?({const char*_tmp41A=", ...)";_tag_fat(_tmp41A,sizeof(char),7U);}):({const char*_tmp41B=")";_tag_fat(_tmp41B,sizeof(char),2U);});
s=({struct Cyc_PP_Doc*_tmp416[2U];({struct Cyc_PP_Doc*_tmp7C6=Cyc_Absynpp_qvar2doc(tuf->name);_tmp416[0]=_tmp7C6;}),({struct Cyc_PP_Doc*_tmp7C5=({struct _fat_ptr _tmp7C4=({const char*_tmp417="(";_tag_fat(_tmp417,sizeof(char),2U);});struct _fat_ptr _tmp7C3=term;struct _fat_ptr _tmp7C2=({const char*_tmp418=",";_tag_fat(_tmp418,sizeof(char),2U);});Cyc_PP_egroup(_tmp7C4,_tmp7C3,_tmp7C2,({struct Cyc_List_List*(*_tmp7C1)(struct Cyc_PP_Doc*(*f)(struct Cyc_Absyn_Pat*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp419)(struct Cyc_PP_Doc*(*f)(struct Cyc_Absyn_Pat*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_PP_Doc*(*f)(struct Cyc_Absyn_Pat*),struct Cyc_List_List*x))Cyc_List_map;_tmp419;});_tmp7C1(Cyc_Absynpp_pat2doc,ps);}));});_tmp416[1]=_tmp7C5;});Cyc_PP_cat(_tag_fat(_tmp416,sizeof(struct Cyc_PP_Doc*),2U));});
goto _LL0;}}default: _LL2B: _tmp3B7=((struct Cyc_Absyn_Exp_p_Absyn_Raw_pat_struct*)_tmp3B6)->f1;_LL2C: {struct Cyc_Absyn_Exp*e=_tmp3B7;
# 1611
s=Cyc_Absynpp_exp2doc(e);goto _LL0;}}_LL0:;}
# 1613
return s;}
# 607 "absynpp.cyc"
struct Cyc_PP_Doc*Cyc_Absynpp_dp2doc(struct _tuple15*dp){
# 1617 "absynpp.cyc"
return({struct Cyc_PP_Doc*_tmp41C[2U];({struct Cyc_PP_Doc*_tmp7CC=({struct _fat_ptr _tmp7CB=({const char*_tmp41D="";_tag_fat(_tmp41D,sizeof(char),1U);});struct _fat_ptr _tmp7CA=({const char*_tmp41E="=";_tag_fat(_tmp41E,sizeof(char),2U);});struct _fat_ptr _tmp7C9=({const char*_tmp41F="=";_tag_fat(_tmp41F,sizeof(char),2U);});Cyc_PP_egroup(_tmp7CB,_tmp7CA,_tmp7C9,({struct Cyc_List_List*(*_tmp7C8)(struct Cyc_PP_Doc*(*f)(void*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp420)(struct Cyc_PP_Doc*(*f)(void*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_PP_Doc*(*f)(void*),struct Cyc_List_List*x))Cyc_List_map;_tmp420;});_tmp7C8(Cyc_Absynpp_designator2doc,(*dp).f1);}));});_tmp41C[0]=_tmp7CC;}),({
struct Cyc_PP_Doc*_tmp7C7=Cyc_Absynpp_pat2doc((*dp).f2);_tmp41C[1]=_tmp7C7;});Cyc_PP_cat(_tag_fat(_tmp41C,sizeof(struct Cyc_PP_Doc*),2U));});}
# 607 "absynpp.cyc"
struct Cyc_PP_Doc*Cyc_Absynpp_switchclause2doc(struct Cyc_Absyn_Switch_clause*c){
# 1623 "absynpp.cyc"
struct Cyc_PP_Doc*body=Cyc_Absynpp_stmt2doc(c->body,0,0);
if(Cyc_Absynpp_decls_first && Cyc_Absynpp_is_declaration(c->body))
body=Cyc_Absynpp_block(0,body);
# 1624
if(
# 1626
c->where_clause == 0 &&(c->pattern)->r == (void*)& Cyc_Absyn_Wild_p_val)
return({struct Cyc_PP_Doc*_tmp421[2U];({struct Cyc_PP_Doc*_tmp7CF=Cyc_PP_text(({const char*_tmp422="default: ";_tag_fat(_tmp422,sizeof(char),10U);}));_tmp421[0]=_tmp7CF;}),({
struct Cyc_PP_Doc*_tmp7CE=Cyc_PP_nest(2,({struct Cyc_PP_Doc*_tmp423[2U];({struct Cyc_PP_Doc*_tmp7CD=Cyc_PP_line_doc();_tmp423[0]=_tmp7CD;}),_tmp423[1]=body;Cyc_PP_cat(_tag_fat(_tmp423,sizeof(struct Cyc_PP_Doc*),2U));}));_tmp421[1]=_tmp7CE;});Cyc_PP_cat(_tag_fat(_tmp421,sizeof(struct Cyc_PP_Doc*),2U));});else{
if(c->where_clause == 0)
return({struct Cyc_PP_Doc*_tmp424[4U];({struct Cyc_PP_Doc*_tmp7D4=Cyc_PP_text(({const char*_tmp425="case ";_tag_fat(_tmp425,sizeof(char),6U);}));_tmp424[0]=_tmp7D4;}),({
struct Cyc_PP_Doc*_tmp7D3=Cyc_Absynpp_pat2doc(c->pattern);_tmp424[1]=_tmp7D3;}),({
struct Cyc_PP_Doc*_tmp7D2=Cyc_PP_text(({const char*_tmp426=": ";_tag_fat(_tmp426,sizeof(char),3U);}));_tmp424[2]=_tmp7D2;}),({
struct Cyc_PP_Doc*_tmp7D1=Cyc_PP_nest(2,({struct Cyc_PP_Doc*_tmp427[2U];({struct Cyc_PP_Doc*_tmp7D0=Cyc_PP_line_doc();_tmp427[0]=_tmp7D0;}),_tmp427[1]=body;Cyc_PP_cat(_tag_fat(_tmp427,sizeof(struct Cyc_PP_Doc*),2U));}));_tmp424[3]=_tmp7D1;});Cyc_PP_cat(_tag_fat(_tmp424,sizeof(struct Cyc_PP_Doc*),4U));});else{
# 1635
return({struct Cyc_PP_Doc*_tmp428[6U];({struct Cyc_PP_Doc*_tmp7DB=Cyc_PP_text(({const char*_tmp429="case ";_tag_fat(_tmp429,sizeof(char),6U);}));_tmp428[0]=_tmp7DB;}),({
struct Cyc_PP_Doc*_tmp7DA=Cyc_Absynpp_pat2doc(c->pattern);_tmp428[1]=_tmp7DA;}),({
struct Cyc_PP_Doc*_tmp7D9=Cyc_PP_text(({const char*_tmp42A=" && ";_tag_fat(_tmp42A,sizeof(char),5U);}));_tmp428[2]=_tmp7D9;}),({
struct Cyc_PP_Doc*_tmp7D8=Cyc_Absynpp_exp2doc((struct Cyc_Absyn_Exp*)_check_null(c->where_clause));_tmp428[3]=_tmp7D8;}),({
struct Cyc_PP_Doc*_tmp7D7=Cyc_PP_text(({const char*_tmp42B=": ";_tag_fat(_tmp42B,sizeof(char),3U);}));_tmp428[4]=_tmp7D7;}),({
struct Cyc_PP_Doc*_tmp7D6=Cyc_PP_nest(2,({struct Cyc_PP_Doc*_tmp42C[2U];({struct Cyc_PP_Doc*_tmp7D5=Cyc_PP_line_doc();_tmp42C[0]=_tmp7D5;}),_tmp42C[1]=body;Cyc_PP_cat(_tag_fat(_tmp42C,sizeof(struct Cyc_PP_Doc*),2U));}));_tmp428[5]=_tmp7D6;});Cyc_PP_cat(_tag_fat(_tmp428,sizeof(struct Cyc_PP_Doc*),6U));});}}}
# 607 "absynpp.cyc"
struct Cyc_PP_Doc*Cyc_Absynpp_switchclauses2doc(struct Cyc_List_List*cs){
# 1644 "absynpp.cyc"
return({struct Cyc_PP_Doc*(*_tmp7DD)(struct Cyc_PP_Doc*(*pp)(struct Cyc_Absyn_Switch_clause*),struct _fat_ptr sep,struct Cyc_List_List*l)=({struct Cyc_PP_Doc*(*_tmp42D)(struct Cyc_PP_Doc*(*pp)(struct Cyc_Absyn_Switch_clause*),struct _fat_ptr sep,struct Cyc_List_List*l)=(struct Cyc_PP_Doc*(*)(struct Cyc_PP_Doc*(*pp)(struct Cyc_Absyn_Switch_clause*),struct _fat_ptr sep,struct Cyc_List_List*l))Cyc_PP_ppseql;_tmp42D;});struct _fat_ptr _tmp7DC=({const char*_tmp42E="";_tag_fat(_tmp42E,sizeof(char),1U);});_tmp7DD(Cyc_Absynpp_switchclause2doc,_tmp7DC,cs);});}
# 607 "absynpp.cyc"
struct Cyc_PP_Doc*Cyc_Absynpp_enumfield2doc(struct Cyc_Absyn_Enumfield*f){
# 1648 "absynpp.cyc"
if(f->tag == 0)
return Cyc_Absynpp_qvar2doc(f->name);else{
# 1651
return({struct Cyc_PP_Doc*_tmp42F[3U];({struct Cyc_PP_Doc*_tmp7E0=Cyc_Absynpp_qvar2doc(f->name);_tmp42F[0]=_tmp7E0;}),({struct Cyc_PP_Doc*_tmp7DF=Cyc_PP_text(({const char*_tmp430=" = ";_tag_fat(_tmp430,sizeof(char),4U);}));_tmp42F[1]=_tmp7DF;}),({struct Cyc_PP_Doc*_tmp7DE=Cyc_Absynpp_exp2doc((struct Cyc_Absyn_Exp*)_check_null(f->tag));_tmp42F[2]=_tmp7DE;});Cyc_PP_cat(_tag_fat(_tmp42F,sizeof(struct Cyc_PP_Doc*),3U));});}}
# 607 "absynpp.cyc"
struct Cyc_PP_Doc*Cyc_Absynpp_enumfields2doc(struct Cyc_List_List*fs){
# 1655 "absynpp.cyc"
return({struct Cyc_PP_Doc*(*_tmp7E2)(struct Cyc_PP_Doc*(*pp)(struct Cyc_Absyn_Enumfield*),struct _fat_ptr sep,struct Cyc_List_List*l)=({struct Cyc_PP_Doc*(*_tmp431)(struct Cyc_PP_Doc*(*pp)(struct Cyc_Absyn_Enumfield*),struct _fat_ptr sep,struct Cyc_List_List*l)=(struct Cyc_PP_Doc*(*)(struct Cyc_PP_Doc*(*pp)(struct Cyc_Absyn_Enumfield*),struct _fat_ptr sep,struct Cyc_List_List*l))Cyc_PP_ppseql;_tmp431;});struct _fat_ptr _tmp7E1=({const char*_tmp432=",";_tag_fat(_tmp432,sizeof(char),2U);});_tmp7E2(Cyc_Absynpp_enumfield2doc,_tmp7E1,fs);});}
# 607 "absynpp.cyc"
static struct Cyc_PP_Doc*Cyc_Absynpp_id2doc(struct Cyc_Absyn_Vardecl*v){
# 1659 "absynpp.cyc"
return Cyc_Absynpp_qvar2doc(v->name);}
# 607 "absynpp.cyc"
static struct Cyc_PP_Doc*Cyc_Absynpp_ids2doc(struct Cyc_List_List*vds){
# 1663 "absynpp.cyc"
return({struct Cyc_PP_Doc*(*_tmp7E4)(struct Cyc_PP_Doc*(*pp)(struct Cyc_Absyn_Vardecl*),struct _fat_ptr sep,struct Cyc_List_List*l)=({struct Cyc_PP_Doc*(*_tmp433)(struct Cyc_PP_Doc*(*pp)(struct Cyc_Absyn_Vardecl*),struct _fat_ptr sep,struct Cyc_List_List*l)=(struct Cyc_PP_Doc*(*)(struct Cyc_PP_Doc*(*pp)(struct Cyc_Absyn_Vardecl*),struct _fat_ptr sep,struct Cyc_List_List*l))Cyc_PP_ppseq;_tmp433;});struct _fat_ptr _tmp7E3=({const char*_tmp434=",";_tag_fat(_tmp434,sizeof(char),2U);});_tmp7E4(Cyc_Absynpp_id2doc,_tmp7E3,vds);});}
# 607 "absynpp.cyc"
struct Cyc_PP_Doc*Cyc_Absynpp_vardecl2doc(struct Cyc_Absyn_Vardecl*vd){
# 1667 "absynpp.cyc"
struct Cyc_List_List*_tmp43B;struct Cyc_Absyn_Exp*_tmp43A;void*_tmp439;struct Cyc_Absyn_Tqual _tmp438;unsigned _tmp437;struct _tuple0*_tmp436;enum Cyc_Absyn_Scope _tmp435;_LL1: _tmp435=vd->sc;_tmp436=vd->name;_tmp437=vd->varloc;_tmp438=vd->tq;_tmp439=vd->type;_tmp43A=vd->initializer;_tmp43B=vd->attributes;_LL2: {enum Cyc_Absyn_Scope sc=_tmp435;struct _tuple0*name=_tmp436;unsigned varloc=_tmp437;struct Cyc_Absyn_Tqual tq=_tmp438;void*type=_tmp439;struct Cyc_Absyn_Exp*initializer=_tmp43A;struct Cyc_List_List*atts=_tmp43B;
struct Cyc_PP_Doc*s;
struct Cyc_PP_Doc*sn=Cyc_Absynpp_typedef_name2bolddoc(name);
struct Cyc_PP_Doc*attsdoc=Cyc_Absynpp_atts2doc(atts);
struct Cyc_PP_Doc*beforenamedoc;
{enum Cyc_Cyclone_C_Compilers _tmp43C=Cyc_Cyclone_c_compiler;if(_tmp43C == Cyc_Cyclone_Gcc_c){_LL4: _LL5:
 beforenamedoc=attsdoc;goto _LL3;}else{_LL6: _LL7:
# 1675
{void*_stmttmp1E=Cyc_Tcutil_compress(type);void*_tmp43D=_stmttmp1E;struct Cyc_List_List*_tmp43E;if(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp43D)->tag == 5U){_LL9: _tmp43E=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp43D)->f1).attributes;_LLA: {struct Cyc_List_List*atts2=_tmp43E;
# 1677
beforenamedoc=Cyc_Absynpp_callconv2doc(atts2);
goto _LL8;}}else{_LLB: _LLC:
 beforenamedoc=Cyc_PP_nil_doc();goto _LL8;}_LL8:;}
# 1681
goto _LL3;}_LL3:;}{
# 1684
struct Cyc_PP_Doc*tmp_doc;
{enum Cyc_Cyclone_C_Compilers _tmp43F=Cyc_Cyclone_c_compiler;if(_tmp43F == Cyc_Cyclone_Gcc_c){_LLE: _LLF:
 tmp_doc=Cyc_PP_nil_doc();goto _LLD;}else{_LL10: _LL11:
 tmp_doc=attsdoc;goto _LLD;}_LLD:;}
# 1689
s=({struct Cyc_PP_Doc*_tmp440[5U];_tmp440[0]=tmp_doc,({
# 1691
struct Cyc_PP_Doc*_tmp7EE=Cyc_Absynpp_scope2doc(sc);_tmp440[1]=_tmp7EE;}),({
struct Cyc_PP_Doc*_tmp7ED=({struct Cyc_Absyn_Tqual _tmp7EC=tq;void*_tmp7EB=type;Cyc_Absynpp_tqtd2doc(_tmp7EC,_tmp7EB,({struct Cyc_Core_Opt*_tmp442=_cycalloc(sizeof(*_tmp442));({struct Cyc_PP_Doc*_tmp7EA=({struct Cyc_PP_Doc*_tmp441[2U];_tmp441[0]=beforenamedoc,_tmp441[1]=sn;Cyc_PP_cat(_tag_fat(_tmp441,sizeof(struct Cyc_PP_Doc*),2U));});_tmp442->v=_tmp7EA;});_tmp442;}));});_tmp440[2]=_tmp7ED;}),
initializer == 0?({
struct Cyc_PP_Doc*_tmp7E9=Cyc_PP_nil_doc();_tmp440[3]=_tmp7E9;}):({
struct Cyc_PP_Doc*_tmp7E8=({struct Cyc_PP_Doc*_tmp443[2U];({struct Cyc_PP_Doc*_tmp7E7=Cyc_PP_text(({const char*_tmp444=" = ";_tag_fat(_tmp444,sizeof(char),4U);}));_tmp443[0]=_tmp7E7;}),({struct Cyc_PP_Doc*_tmp7E6=Cyc_Absynpp_exp2doc(initializer);_tmp443[1]=_tmp7E6;});Cyc_PP_cat(_tag_fat(_tmp443,sizeof(struct Cyc_PP_Doc*),2U));});_tmp440[3]=_tmp7E8;}),({
struct Cyc_PP_Doc*_tmp7E5=Cyc_PP_text(({const char*_tmp445=";";_tag_fat(_tmp445,sizeof(char),2U);}));_tmp440[4]=_tmp7E5;});Cyc_PP_cat(_tag_fat(_tmp440,sizeof(struct Cyc_PP_Doc*),5U));});
return s;}}}struct _tuple20{unsigned f1;struct _tuple0*f2;int f3;};
# 607 "absynpp.cyc"
struct Cyc_PP_Doc*Cyc_Absynpp_export2doc(struct _tuple20*x){
# 1701 "absynpp.cyc"
struct _tuple20 _stmttmp1F=*x;struct _tuple0*_tmp446;_LL1: _tmp446=_stmttmp1F.f2;_LL2: {struct _tuple0*v=_tmp446;
return Cyc_Absynpp_qvar2doc(v);}}
# 607 "absynpp.cyc"
struct Cyc_PP_Doc*Cyc_Absynpp_aggrdecl2doc(struct Cyc_Absyn_Aggrdecl*ad){
# 1706 "absynpp.cyc"
if(ad->impl == 0)
return({struct Cyc_PP_Doc*_tmp447[4U];({struct Cyc_PP_Doc*_tmp7F2=Cyc_Absynpp_scope2doc(ad->sc);_tmp447[0]=_tmp7F2;}),({
struct Cyc_PP_Doc*_tmp7F1=Cyc_Absynpp_aggr_kind2doc(ad->kind);_tmp447[1]=_tmp7F1;}),({
struct Cyc_PP_Doc*_tmp7F0=Cyc_Absynpp_qvar2bolddoc(ad->name);_tmp447[2]=_tmp7F0;}),({
struct Cyc_PP_Doc*_tmp7EF=Cyc_Absynpp_ktvars2doc(ad->tvs);_tmp447[3]=_tmp7EF;});Cyc_PP_cat(_tag_fat(_tmp447,sizeof(struct Cyc_PP_Doc*),4U));});else{
# 1712
return({struct Cyc_PP_Doc*_tmp448[12U];({struct Cyc_PP_Doc*_tmp803=Cyc_Absynpp_scope2doc(ad->sc);_tmp448[0]=_tmp803;}),({
struct Cyc_PP_Doc*_tmp802=Cyc_Absynpp_aggr_kind2doc(ad->kind);_tmp448[1]=_tmp802;}),({
struct Cyc_PP_Doc*_tmp801=Cyc_Absynpp_qvar2bolddoc(ad->name);_tmp448[2]=_tmp801;}),({
struct Cyc_PP_Doc*_tmp800=Cyc_Absynpp_ktvars2doc(ad->tvs);_tmp448[3]=_tmp800;}),({
struct Cyc_PP_Doc*_tmp7FF=Cyc_PP_blank_doc();_tmp448[4]=_tmp7FF;}),({struct Cyc_PP_Doc*_tmp7FE=Cyc_Absynpp_lb();_tmp448[5]=_tmp7FE;}),({
struct Cyc_PP_Doc*_tmp7FD=Cyc_Absynpp_ktvars2doc(((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->exist_vars);_tmp448[6]=_tmp7FD;}),
((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->rgn_po == 0?({struct Cyc_PP_Doc*_tmp7FC=Cyc_PP_nil_doc();_tmp448[7]=_tmp7FC;}):({
struct Cyc_PP_Doc*_tmp7FB=({struct Cyc_PP_Doc*_tmp449[2U];({struct Cyc_PP_Doc*_tmp7FA=Cyc_PP_text(({const char*_tmp44A=":";_tag_fat(_tmp44A,sizeof(char),2U);}));_tmp449[0]=_tmp7FA;}),({struct Cyc_PP_Doc*_tmp7F9=Cyc_Absynpp_rgnpo2doc(((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->rgn_po);_tmp449[1]=_tmp7F9;});Cyc_PP_cat(_tag_fat(_tmp449,sizeof(struct Cyc_PP_Doc*),2U));});_tmp448[7]=_tmp7FB;}),({
struct Cyc_PP_Doc*_tmp7F8=Cyc_PP_nest(2,({struct Cyc_PP_Doc*_tmp44B[2U];({struct Cyc_PP_Doc*_tmp7F7=Cyc_PP_line_doc();_tmp44B[0]=_tmp7F7;}),({struct Cyc_PP_Doc*_tmp7F6=Cyc_Absynpp_aggrfields2doc(((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->fields);_tmp44B[1]=_tmp7F6;});Cyc_PP_cat(_tag_fat(_tmp44B,sizeof(struct Cyc_PP_Doc*),2U));}));_tmp448[8]=_tmp7F8;}),({
struct Cyc_PP_Doc*_tmp7F5=Cyc_PP_line_doc();_tmp448[9]=_tmp7F5;}),({
struct Cyc_PP_Doc*_tmp7F4=Cyc_Absynpp_rb();_tmp448[10]=_tmp7F4;}),({
struct Cyc_PP_Doc*_tmp7F3=Cyc_Absynpp_atts2doc(ad->attributes);_tmp448[11]=_tmp7F3;});Cyc_PP_cat(_tag_fat(_tmp448,sizeof(struct Cyc_PP_Doc*),12U));});}}
# 607 "absynpp.cyc"
struct Cyc_PP_Doc*Cyc_Absynpp_datatypedecl2doc(struct Cyc_Absyn_Datatypedecl*dd){
# 1727 "absynpp.cyc"
int _tmp450;struct Cyc_Core_Opt*_tmp44F;struct Cyc_List_List*_tmp44E;struct _tuple0*_tmp44D;enum Cyc_Absyn_Scope _tmp44C;_LL1: _tmp44C=dd->sc;_tmp44D=dd->name;_tmp44E=dd->tvs;_tmp44F=dd->fields;_tmp450=dd->is_extensible;_LL2: {enum Cyc_Absyn_Scope sc=_tmp44C;struct _tuple0*name=_tmp44D;struct Cyc_List_List*tvs=_tmp44E;struct Cyc_Core_Opt*fields=_tmp44F;int is_x=_tmp450;
if(fields == 0)
return({struct Cyc_PP_Doc*_tmp451[5U];({struct Cyc_PP_Doc*_tmp80A=Cyc_Absynpp_scope2doc(sc);_tmp451[0]=_tmp80A;}),
is_x?({struct Cyc_PP_Doc*_tmp809=Cyc_PP_text(({const char*_tmp452="@extensible ";_tag_fat(_tmp452,sizeof(char),13U);}));_tmp451[1]=_tmp809;}):({struct Cyc_PP_Doc*_tmp808=Cyc_PP_blank_doc();_tmp451[1]=_tmp808;}),({
struct Cyc_PP_Doc*_tmp807=Cyc_PP_text(({const char*_tmp453="datatype ";_tag_fat(_tmp453,sizeof(char),10U);}));_tmp451[2]=_tmp807;}),
is_x?({struct Cyc_PP_Doc*_tmp806=Cyc_Absynpp_qvar2bolddoc(name);_tmp451[3]=_tmp806;}):({struct Cyc_PP_Doc*_tmp805=Cyc_Absynpp_typedef_name2bolddoc(name);_tmp451[3]=_tmp805;}),({
struct Cyc_PP_Doc*_tmp804=Cyc_Absynpp_ktvars2doc(tvs);_tmp451[4]=_tmp804;});Cyc_PP_cat(_tag_fat(_tmp451,sizeof(struct Cyc_PP_Doc*),5U));});else{
# 1735
return({struct Cyc_PP_Doc*_tmp454[10U];({struct Cyc_PP_Doc*_tmp818=Cyc_Absynpp_scope2doc(sc);_tmp454[0]=_tmp818;}),
is_x?({struct Cyc_PP_Doc*_tmp817=Cyc_PP_text(({const char*_tmp455="@extensible ";_tag_fat(_tmp455,sizeof(char),13U);}));_tmp454[1]=_tmp817;}):({struct Cyc_PP_Doc*_tmp816=Cyc_PP_blank_doc();_tmp454[1]=_tmp816;}),({
struct Cyc_PP_Doc*_tmp815=Cyc_PP_text(({const char*_tmp456="datatype ";_tag_fat(_tmp456,sizeof(char),10U);}));_tmp454[2]=_tmp815;}),
is_x?({struct Cyc_PP_Doc*_tmp814=Cyc_Absynpp_qvar2bolddoc(name);_tmp454[3]=_tmp814;}):({struct Cyc_PP_Doc*_tmp813=Cyc_Absynpp_typedef_name2bolddoc(name);_tmp454[3]=_tmp813;}),({
struct Cyc_PP_Doc*_tmp812=Cyc_Absynpp_ktvars2doc(tvs);_tmp454[4]=_tmp812;}),({
struct Cyc_PP_Doc*_tmp811=Cyc_PP_blank_doc();_tmp454[5]=_tmp811;}),({struct Cyc_PP_Doc*_tmp810=Cyc_Absynpp_lb();_tmp454[6]=_tmp810;}),({
struct Cyc_PP_Doc*_tmp80F=Cyc_PP_nest(2,({struct Cyc_PP_Doc*_tmp457[2U];({struct Cyc_PP_Doc*_tmp80E=Cyc_PP_line_doc();_tmp457[0]=_tmp80E;}),({struct Cyc_PP_Doc*_tmp80D=Cyc_Absynpp_datatypefields2doc((struct Cyc_List_List*)fields->v);_tmp457[1]=_tmp80D;});Cyc_PP_cat(_tag_fat(_tmp457,sizeof(struct Cyc_PP_Doc*),2U));}));_tmp454[7]=_tmp80F;}),({
struct Cyc_PP_Doc*_tmp80C=Cyc_PP_line_doc();_tmp454[8]=_tmp80C;}),({
struct Cyc_PP_Doc*_tmp80B=Cyc_Absynpp_rb();_tmp454[9]=_tmp80B;});Cyc_PP_cat(_tag_fat(_tmp454,sizeof(struct Cyc_PP_Doc*),10U));});}}}
# 607 "absynpp.cyc"
struct Cyc_PP_Doc*Cyc_Absynpp_enumdecl2doc(struct Cyc_Absyn_Enumdecl*ed){
# 1747 "absynpp.cyc"
struct Cyc_Core_Opt*_tmp45A;struct _tuple0*_tmp459;enum Cyc_Absyn_Scope _tmp458;_LL1: _tmp458=ed->sc;_tmp459=ed->name;_tmp45A=ed->fields;_LL2: {enum Cyc_Absyn_Scope sc=_tmp458;struct _tuple0*n=_tmp459;struct Cyc_Core_Opt*fields=_tmp45A;
if(fields == 0)
return({struct Cyc_PP_Doc*_tmp45B[3U];({struct Cyc_PP_Doc*_tmp81B=Cyc_Absynpp_scope2doc(sc);_tmp45B[0]=_tmp81B;}),({
struct Cyc_PP_Doc*_tmp81A=Cyc_PP_text(({const char*_tmp45C="enum ";_tag_fat(_tmp45C,sizeof(char),6U);}));_tmp45B[1]=_tmp81A;}),({
struct Cyc_PP_Doc*_tmp819=Cyc_Absynpp_typedef_name2bolddoc(n);_tmp45B[2]=_tmp819;});Cyc_PP_cat(_tag_fat(_tmp45B,sizeof(struct Cyc_PP_Doc*),3U));});else{
# 1754
return({struct Cyc_PP_Doc*_tmp45D[8U];({struct Cyc_PP_Doc*_tmp825=Cyc_Absynpp_scope2doc(sc);_tmp45D[0]=_tmp825;}),({
struct Cyc_PP_Doc*_tmp824=Cyc_PP_text(({const char*_tmp45E="enum ";_tag_fat(_tmp45E,sizeof(char),6U);}));_tmp45D[1]=_tmp824;}),({
struct Cyc_PP_Doc*_tmp823=Cyc_Absynpp_qvar2bolddoc(n);_tmp45D[2]=_tmp823;}),({
struct Cyc_PP_Doc*_tmp822=Cyc_PP_blank_doc();_tmp45D[3]=_tmp822;}),({struct Cyc_PP_Doc*_tmp821=Cyc_Absynpp_lb();_tmp45D[4]=_tmp821;}),({
struct Cyc_PP_Doc*_tmp820=Cyc_PP_nest(2,({struct Cyc_PP_Doc*_tmp45F[2U];({struct Cyc_PP_Doc*_tmp81F=Cyc_PP_line_doc();_tmp45F[0]=_tmp81F;}),({struct Cyc_PP_Doc*_tmp81E=Cyc_Absynpp_enumfields2doc((struct Cyc_List_List*)fields->v);_tmp45F[1]=_tmp81E;});Cyc_PP_cat(_tag_fat(_tmp45F,sizeof(struct Cyc_PP_Doc*),2U));}));_tmp45D[5]=_tmp820;}),({
struct Cyc_PP_Doc*_tmp81D=Cyc_PP_line_doc();_tmp45D[6]=_tmp81D;}),({
struct Cyc_PP_Doc*_tmp81C=Cyc_Absynpp_rb();_tmp45D[7]=_tmp81C;});Cyc_PP_cat(_tag_fat(_tmp45D,sizeof(struct Cyc_PP_Doc*),8U));});}}}
# 607 "absynpp.cyc"
struct Cyc_PP_Doc*Cyc_Absynpp_decl2doc(struct Cyc_Absyn_Decl*d){
# 1764 "absynpp.cyc"
struct Cyc_PP_Doc*s;
{void*_stmttmp20=d->r;void*_tmp460=_stmttmp20;struct _tuple10*_tmp464;struct Cyc_List_List*_tmp463;struct Cyc_List_List*_tmp462;struct Cyc_List_List*_tmp461;struct Cyc_List_List*_tmp465;struct Cyc_List_List*_tmp467;struct _tuple0*_tmp466;struct Cyc_List_List*_tmp469;struct _fat_ptr*_tmp468;struct Cyc_Absyn_Typedefdecl*_tmp46A;struct Cyc_Absyn_Enumdecl*_tmp46B;struct Cyc_List_List*_tmp46C;struct Cyc_Absyn_Exp*_tmp46E;struct Cyc_Absyn_Pat*_tmp46D;struct Cyc_Absyn_Datatypedecl*_tmp46F;struct Cyc_Absyn_Exp*_tmp472;struct Cyc_Absyn_Vardecl*_tmp471;struct Cyc_Absyn_Tvar*_tmp470;struct Cyc_Absyn_Vardecl*_tmp473;struct Cyc_Absyn_Aggrdecl*_tmp474;struct Cyc_Absyn_Fndecl*_tmp475;switch(*((int*)_tmp460)){case 1U: _LL1: _tmp475=((struct Cyc_Absyn_Fn_d_Absyn_Raw_decl_struct*)_tmp460)->f1;_LL2: {struct Cyc_Absyn_Fndecl*fd=_tmp475;
# 1767
struct Cyc_Absyn_FnInfo type_info=fd->i;
type_info.attributes=0;{
void*t=(void*)({struct Cyc_Absyn_FnType_Absyn_Type_struct*_tmp486=_cycalloc(sizeof(*_tmp486));_tmp486->tag=5U,_tmp486->f1=type_info;_tmp486;});
if(fd->cached_type != 0){
void*_stmttmp21=Cyc_Tcutil_compress((void*)_check_null(fd->cached_type));void*_tmp476=_stmttmp21;struct Cyc_Absyn_FnInfo _tmp477;if(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp476)->tag == 5U){_LL24: _tmp477=((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp476)->f1;_LL25: {struct Cyc_Absyn_FnInfo i=_tmp477;
# 1773
({struct Cyc_List_List*_tmp826=Cyc_List_append((fd->i).attributes,i.attributes);(fd->i).attributes=_tmp826;});goto _LL23;}}else{_LL26: _LL27:
({void*_tmp478=0U;({int(*_tmp828)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp47A)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmp47A;});struct _fat_ptr _tmp827=({const char*_tmp479="function has non-function type";_tag_fat(_tmp479,sizeof(char),31U);});_tmp828(_tmp827,_tag_fat(_tmp478,sizeof(void*),0U));});});}_LL23:;}{
# 1770
struct Cyc_PP_Doc*attsdoc=
# 1776
Cyc_Absynpp_atts2doc((fd->i).attributes);
struct Cyc_PP_Doc*inlinedoc;
if(fd->is_inline){
enum Cyc_Cyclone_C_Compilers _tmp47B=Cyc_Cyclone_c_compiler;if(_tmp47B == Cyc_Cyclone_Gcc_c){_LL29: _LL2A:
 inlinedoc=Cyc_PP_text(({const char*_tmp47C="inline ";_tag_fat(_tmp47C,sizeof(char),8U);}));goto _LL28;}else{_LL2B: _LL2C:
 inlinedoc=Cyc_PP_text(({const char*_tmp47D="__inline ";_tag_fat(_tmp47D,sizeof(char),10U);}));goto _LL28;}_LL28:;}else{
# 1784
inlinedoc=Cyc_PP_nil_doc();}{
struct Cyc_PP_Doc*scopedoc=Cyc_Absynpp_scope2doc(fd->sc);
struct Cyc_PP_Doc*beforenamedoc;
{enum Cyc_Cyclone_C_Compilers _tmp47E=Cyc_Cyclone_c_compiler;if(_tmp47E == Cyc_Cyclone_Gcc_c){_LL2E: _LL2F:
 beforenamedoc=attsdoc;goto _LL2D;}else{_LL30: _LL31:
 beforenamedoc=Cyc_Absynpp_callconv2doc((fd->i).attributes);goto _LL2D;}_LL2D:;}{
# 1791
struct Cyc_PP_Doc*namedoc=Cyc_Absynpp_typedef_name2doc(fd->name);
struct Cyc_PP_Doc*tqtddoc=({struct Cyc_Absyn_Tqual _tmp82B=Cyc_Absyn_empty_tqual(0U);void*_tmp82A=t;Cyc_Absynpp_tqtd2doc(_tmp82B,_tmp82A,({struct Cyc_Core_Opt*_tmp485=_cycalloc(sizeof(*_tmp485));({
struct Cyc_PP_Doc*_tmp829=({struct Cyc_PP_Doc*_tmp484[2U];_tmp484[0]=beforenamedoc,_tmp484[1]=namedoc;Cyc_PP_cat(_tag_fat(_tmp484,sizeof(struct Cyc_PP_Doc*),2U));});_tmp485->v=_tmp829;});_tmp485;}));});
# 1799
struct Cyc_PP_Doc*bodydoc=({struct Cyc_PP_Doc*_tmp482[5U];({struct Cyc_PP_Doc*_tmp832=Cyc_PP_blank_doc();_tmp482[0]=_tmp832;}),({struct Cyc_PP_Doc*_tmp831=Cyc_Absynpp_lb();_tmp482[1]=_tmp831;}),({
struct Cyc_PP_Doc*_tmp830=Cyc_PP_nest(2,({struct Cyc_PP_Doc*_tmp483[2U];({struct Cyc_PP_Doc*_tmp82F=Cyc_PP_line_doc();_tmp483[0]=_tmp82F;}),({struct Cyc_PP_Doc*_tmp82E=Cyc_Absynpp_stmt2doc(fd->body,0,0);_tmp483[1]=_tmp82E;});Cyc_PP_cat(_tag_fat(_tmp483,sizeof(struct Cyc_PP_Doc*),2U));}));_tmp482[2]=_tmp830;}),({
struct Cyc_PP_Doc*_tmp82D=Cyc_PP_line_doc();_tmp482[3]=_tmp82D;}),({
struct Cyc_PP_Doc*_tmp82C=Cyc_Absynpp_rb();_tmp482[4]=_tmp82C;});Cyc_PP_cat(_tag_fat(_tmp482,sizeof(struct Cyc_PP_Doc*),5U));});
s=({struct Cyc_PP_Doc*_tmp47F[4U];_tmp47F[0]=inlinedoc,_tmp47F[1]=scopedoc,_tmp47F[2]=tqtddoc,_tmp47F[3]=bodydoc;Cyc_PP_cat(_tag_fat(_tmp47F,sizeof(struct Cyc_PP_Doc*),4U));});
# 1805
{enum Cyc_Cyclone_C_Compilers _tmp480=Cyc_Cyclone_c_compiler;if(_tmp480 == Cyc_Cyclone_Vc_c){_LL33: _LL34:
 s=({struct Cyc_PP_Doc*_tmp481[2U];_tmp481[0]=attsdoc,_tmp481[1]=s;Cyc_PP_cat(_tag_fat(_tmp481,sizeof(struct Cyc_PP_Doc*),2U));});goto _LL32;}else{_LL35: _LL36:
 goto _LL32;}_LL32:;}
# 1810
goto _LL0;}}}}}case 5U: _LL3: _tmp474=((struct Cyc_Absyn_Aggr_d_Absyn_Raw_decl_struct*)_tmp460)->f1;_LL4: {struct Cyc_Absyn_Aggrdecl*ad=_tmp474;
# 1813
s=({struct Cyc_PP_Doc*_tmp487[2U];({struct Cyc_PP_Doc*_tmp834=Cyc_Absynpp_aggrdecl2doc(ad);_tmp487[0]=_tmp834;}),({struct Cyc_PP_Doc*_tmp833=Cyc_PP_text(({const char*_tmp488=";";_tag_fat(_tmp488,sizeof(char),2U);}));_tmp487[1]=_tmp833;});Cyc_PP_cat(_tag_fat(_tmp487,sizeof(struct Cyc_PP_Doc*),2U));});
goto _LL0;}case 0U: _LL5: _tmp473=((struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct*)_tmp460)->f1;_LL6: {struct Cyc_Absyn_Vardecl*vd=_tmp473;
# 1816
s=Cyc_Absynpp_vardecl2doc(vd);
goto _LL0;}case 4U: _LL7: _tmp470=((struct Cyc_Absyn_Region_d_Absyn_Raw_decl_struct*)_tmp460)->f1;_tmp471=((struct Cyc_Absyn_Region_d_Absyn_Raw_decl_struct*)_tmp460)->f2;_tmp472=((struct Cyc_Absyn_Region_d_Absyn_Raw_decl_struct*)_tmp460)->f3;_LL8: {struct Cyc_Absyn_Tvar*tv=_tmp470;struct Cyc_Absyn_Vardecl*vd=_tmp471;struct Cyc_Absyn_Exp*open_exp_opt=_tmp472;
# 1819
s=({struct Cyc_PP_Doc*_tmp489[7U];({struct Cyc_PP_Doc*_tmp83F=Cyc_PP_text(({const char*_tmp48A="region";_tag_fat(_tmp48A,sizeof(char),7U);}));_tmp489[0]=_tmp83F;}),({
struct Cyc_PP_Doc*_tmp83E=Cyc_PP_text(({const char*_tmp48B="<";_tag_fat(_tmp48B,sizeof(char),2U);}));_tmp489[1]=_tmp83E;}),({
struct Cyc_PP_Doc*_tmp83D=Cyc_Absynpp_tvar2doc(tv);_tmp489[2]=_tmp83D;}),({
struct Cyc_PP_Doc*_tmp83C=Cyc_PP_text(({const char*_tmp48C=">";_tag_fat(_tmp48C,sizeof(char),2U);}));_tmp489[3]=_tmp83C;}),({
struct Cyc_PP_Doc*_tmp83B=Cyc_Absynpp_qvar2doc(vd->name);_tmp489[4]=_tmp83B;}),
(unsigned)open_exp_opt?({struct Cyc_PP_Doc*_tmp83A=({struct Cyc_PP_Doc*_tmp48D[3U];({struct Cyc_PP_Doc*_tmp839=Cyc_PP_text(({const char*_tmp48E=" = open(";_tag_fat(_tmp48E,sizeof(char),9U);}));_tmp48D[0]=_tmp839;}),({struct Cyc_PP_Doc*_tmp838=Cyc_Absynpp_exp2doc(open_exp_opt);_tmp48D[1]=_tmp838;}),({
struct Cyc_PP_Doc*_tmp837=Cyc_PP_text(({const char*_tmp48F=")";_tag_fat(_tmp48F,sizeof(char),2U);}));_tmp48D[2]=_tmp837;});Cyc_PP_cat(_tag_fat(_tmp48D,sizeof(struct Cyc_PP_Doc*),3U));});
# 1824
_tmp489[5]=_tmp83A;}):({
struct Cyc_PP_Doc*_tmp836=Cyc_PP_nil_doc();_tmp489[5]=_tmp836;}),({
struct Cyc_PP_Doc*_tmp835=Cyc_PP_text(({const char*_tmp490=";";_tag_fat(_tmp490,sizeof(char),2U);}));_tmp489[6]=_tmp835;});Cyc_PP_cat(_tag_fat(_tmp489,sizeof(struct Cyc_PP_Doc*),7U));});
goto _LL0;}case 6U: _LL9: _tmp46F=((struct Cyc_Absyn_Datatype_d_Absyn_Raw_decl_struct*)_tmp460)->f1;_LLA: {struct Cyc_Absyn_Datatypedecl*dd=_tmp46F;
# 1829
s=({struct Cyc_PP_Doc*_tmp491[2U];({struct Cyc_PP_Doc*_tmp841=Cyc_Absynpp_datatypedecl2doc(dd);_tmp491[0]=_tmp841;}),({struct Cyc_PP_Doc*_tmp840=Cyc_PP_text(({const char*_tmp492=";";_tag_fat(_tmp492,sizeof(char),2U);}));_tmp491[1]=_tmp840;});Cyc_PP_cat(_tag_fat(_tmp491,sizeof(struct Cyc_PP_Doc*),2U));});
goto _LL0;}case 2U: _LLB: _tmp46D=((struct Cyc_Absyn_Let_d_Absyn_Raw_decl_struct*)_tmp460)->f1;_tmp46E=((struct Cyc_Absyn_Let_d_Absyn_Raw_decl_struct*)_tmp460)->f3;_LLC: {struct Cyc_Absyn_Pat*p=_tmp46D;struct Cyc_Absyn_Exp*e=_tmp46E;
# 1832
s=({struct Cyc_PP_Doc*_tmp493[5U];({struct Cyc_PP_Doc*_tmp846=Cyc_PP_text(({const char*_tmp494="let ";_tag_fat(_tmp494,sizeof(char),5U);}));_tmp493[0]=_tmp846;}),({
struct Cyc_PP_Doc*_tmp845=Cyc_Absynpp_pat2doc(p);_tmp493[1]=_tmp845;}),({
struct Cyc_PP_Doc*_tmp844=Cyc_PP_text(({const char*_tmp495=" = ";_tag_fat(_tmp495,sizeof(char),4U);}));_tmp493[2]=_tmp844;}),({
struct Cyc_PP_Doc*_tmp843=Cyc_Absynpp_exp2doc(e);_tmp493[3]=_tmp843;}),({
struct Cyc_PP_Doc*_tmp842=Cyc_PP_text(({const char*_tmp496=";";_tag_fat(_tmp496,sizeof(char),2U);}));_tmp493[4]=_tmp842;});Cyc_PP_cat(_tag_fat(_tmp493,sizeof(struct Cyc_PP_Doc*),5U));});
goto _LL0;}case 3U: _LLD: _tmp46C=((struct Cyc_Absyn_Letv_d_Absyn_Raw_decl_struct*)_tmp460)->f1;_LLE: {struct Cyc_List_List*vds=_tmp46C;
# 1839
s=({struct Cyc_PP_Doc*_tmp497[3U];({struct Cyc_PP_Doc*_tmp849=Cyc_PP_text(({const char*_tmp498="let ";_tag_fat(_tmp498,sizeof(char),5U);}));_tmp497[0]=_tmp849;}),({struct Cyc_PP_Doc*_tmp848=Cyc_Absynpp_ids2doc(vds);_tmp497[1]=_tmp848;}),({struct Cyc_PP_Doc*_tmp847=Cyc_PP_text(({const char*_tmp499=";";_tag_fat(_tmp499,sizeof(char),2U);}));_tmp497[2]=_tmp847;});Cyc_PP_cat(_tag_fat(_tmp497,sizeof(struct Cyc_PP_Doc*),3U));});
goto _LL0;}case 7U: _LLF: _tmp46B=((struct Cyc_Absyn_Enum_d_Absyn_Raw_decl_struct*)_tmp460)->f1;_LL10: {struct Cyc_Absyn_Enumdecl*ed=_tmp46B;
# 1842
s=({struct Cyc_PP_Doc*_tmp49A[2U];({struct Cyc_PP_Doc*_tmp84B=Cyc_Absynpp_enumdecl2doc(ed);_tmp49A[0]=_tmp84B;}),({struct Cyc_PP_Doc*_tmp84A=Cyc_PP_text(({const char*_tmp49B=";";_tag_fat(_tmp49B,sizeof(char),2U);}));_tmp49A[1]=_tmp84A;});Cyc_PP_cat(_tag_fat(_tmp49A,sizeof(struct Cyc_PP_Doc*),2U));});
goto _LL0;}case 8U: _LL11: _tmp46A=((struct Cyc_Absyn_Typedef_d_Absyn_Raw_decl_struct*)_tmp460)->f1;_LL12: {struct Cyc_Absyn_Typedefdecl*td=_tmp46A;
# 1845
void*t;
if(td->defn != 0)
t=(void*)_check_null(td->defn);else{
# 1849
t=Cyc_Absyn_new_evar(td->kind,0);}
s=({struct Cyc_PP_Doc*_tmp49C[4U];({struct Cyc_PP_Doc*_tmp854=Cyc_PP_text(({const char*_tmp49D="typedef ";_tag_fat(_tmp49D,sizeof(char),9U);}));_tmp49C[0]=_tmp854;}),({
struct Cyc_PP_Doc*_tmp853=({struct Cyc_Absyn_Tqual _tmp852=td->tq;void*_tmp851=t;Cyc_Absynpp_tqtd2doc(_tmp852,_tmp851,({struct Cyc_Core_Opt*_tmp49F=_cycalloc(sizeof(*_tmp49F));({
# 1853
struct Cyc_PP_Doc*_tmp850=({struct Cyc_PP_Doc*_tmp49E[2U];({struct Cyc_PP_Doc*_tmp84F=Cyc_Absynpp_typedef_name2bolddoc(td->name);_tmp49E[0]=_tmp84F;}),({
struct Cyc_PP_Doc*_tmp84E=Cyc_Absynpp_tvars2doc(td->tvs);_tmp49E[1]=_tmp84E;});Cyc_PP_cat(_tag_fat(_tmp49E,sizeof(struct Cyc_PP_Doc*),2U));});
# 1853
_tmp49F->v=_tmp850;});_tmp49F;}));});
# 1851
_tmp49C[1]=_tmp853;}),({
# 1856
struct Cyc_PP_Doc*_tmp84D=Cyc_Absynpp_atts2doc(td->atts);_tmp49C[2]=_tmp84D;}),({
struct Cyc_PP_Doc*_tmp84C=Cyc_PP_text(({const char*_tmp4A0=";";_tag_fat(_tmp4A0,sizeof(char),2U);}));_tmp49C[3]=_tmp84C;});Cyc_PP_cat(_tag_fat(_tmp49C,sizeof(struct Cyc_PP_Doc*),4U));});
# 1859
goto _LL0;}case 9U: _LL13: _tmp468=((struct Cyc_Absyn_Namespace_d_Absyn_Raw_decl_struct*)_tmp460)->f1;_tmp469=((struct Cyc_Absyn_Namespace_d_Absyn_Raw_decl_struct*)_tmp460)->f2;_LL14: {struct _fat_ptr*v=_tmp468;struct Cyc_List_List*tdl=_tmp469;
# 1861
if(Cyc_Absynpp_use_curr_namespace)Cyc_Absynpp_curr_namespace_add(v);s=({struct Cyc_PP_Doc*_tmp4A1[8U];({
struct Cyc_PP_Doc*_tmp85E=Cyc_PP_text(({const char*_tmp4A2="namespace ";_tag_fat(_tmp4A2,sizeof(char),11U);}));_tmp4A1[0]=_tmp85E;}),({
struct Cyc_PP_Doc*_tmp85D=Cyc_PP_textptr(v);_tmp4A1[1]=_tmp85D;}),({
struct Cyc_PP_Doc*_tmp85C=Cyc_PP_blank_doc();_tmp4A1[2]=_tmp85C;}),({struct Cyc_PP_Doc*_tmp85B=Cyc_Absynpp_lb();_tmp4A1[3]=_tmp85B;}),({
struct Cyc_PP_Doc*_tmp85A=Cyc_PP_line_doc();_tmp4A1[4]=_tmp85A;}),({
struct Cyc_PP_Doc*_tmp859=({struct Cyc_PP_Doc*(*_tmp858)(struct Cyc_PP_Doc*(*pp)(struct Cyc_Absyn_Decl*),struct _fat_ptr sep,struct Cyc_List_List*l)=({struct Cyc_PP_Doc*(*_tmp4A3)(struct Cyc_PP_Doc*(*pp)(struct Cyc_Absyn_Decl*),struct _fat_ptr sep,struct Cyc_List_List*l)=(struct Cyc_PP_Doc*(*)(struct Cyc_PP_Doc*(*pp)(struct Cyc_Absyn_Decl*),struct _fat_ptr sep,struct Cyc_List_List*l))Cyc_PP_ppseql;_tmp4A3;});struct _fat_ptr _tmp857=({const char*_tmp4A4="";_tag_fat(_tmp4A4,sizeof(char),1U);});_tmp858(Cyc_Absynpp_decl2doc,_tmp857,tdl);});_tmp4A1[5]=_tmp859;}),({
struct Cyc_PP_Doc*_tmp856=Cyc_PP_line_doc();_tmp4A1[6]=_tmp856;}),({
struct Cyc_PP_Doc*_tmp855=Cyc_Absynpp_rb();_tmp4A1[7]=_tmp855;});Cyc_PP_cat(_tag_fat(_tmp4A1,sizeof(struct Cyc_PP_Doc*),8U));});
if(Cyc_Absynpp_use_curr_namespace)Cyc_Absynpp_curr_namespace_drop();goto _LL0;}case 10U: _LL15: _tmp466=((struct Cyc_Absyn_Using_d_Absyn_Raw_decl_struct*)_tmp460)->f1;_tmp467=((struct Cyc_Absyn_Using_d_Absyn_Raw_decl_struct*)_tmp460)->f2;_LL16: {struct _tuple0*q=_tmp466;struct Cyc_List_List*tdl=_tmp467;
# 1872
if(Cyc_Absynpp_print_using_stmts)
s=({struct Cyc_PP_Doc*_tmp4A5[8U];({struct Cyc_PP_Doc*_tmp868=Cyc_PP_text(({const char*_tmp4A6="using ";_tag_fat(_tmp4A6,sizeof(char),7U);}));_tmp4A5[0]=_tmp868;}),({
struct Cyc_PP_Doc*_tmp867=Cyc_Absynpp_qvar2doc(q);_tmp4A5[1]=_tmp867;}),({
struct Cyc_PP_Doc*_tmp866=Cyc_PP_blank_doc();_tmp4A5[2]=_tmp866;}),({struct Cyc_PP_Doc*_tmp865=Cyc_Absynpp_lb();_tmp4A5[3]=_tmp865;}),({
struct Cyc_PP_Doc*_tmp864=Cyc_PP_line_doc();_tmp4A5[4]=_tmp864;}),({
struct Cyc_PP_Doc*_tmp863=({struct Cyc_PP_Doc*(*_tmp862)(struct Cyc_PP_Doc*(*pp)(struct Cyc_Absyn_Decl*),struct _fat_ptr sep,struct Cyc_List_List*l)=({struct Cyc_PP_Doc*(*_tmp4A7)(struct Cyc_PP_Doc*(*pp)(struct Cyc_Absyn_Decl*),struct _fat_ptr sep,struct Cyc_List_List*l)=(struct Cyc_PP_Doc*(*)(struct Cyc_PP_Doc*(*pp)(struct Cyc_Absyn_Decl*),struct _fat_ptr sep,struct Cyc_List_List*l))Cyc_PP_ppseql;_tmp4A7;});struct _fat_ptr _tmp861=({const char*_tmp4A8="";_tag_fat(_tmp4A8,sizeof(char),1U);});_tmp862(Cyc_Absynpp_decl2doc,_tmp861,tdl);});_tmp4A5[5]=_tmp863;}),({
struct Cyc_PP_Doc*_tmp860=Cyc_PP_line_doc();_tmp4A5[6]=_tmp860;}),({
struct Cyc_PP_Doc*_tmp85F=Cyc_Absynpp_rb();_tmp4A5[7]=_tmp85F;});Cyc_PP_cat(_tag_fat(_tmp4A5,sizeof(struct Cyc_PP_Doc*),8U));});else{
# 1881
s=({struct Cyc_PP_Doc*_tmp4A9[11U];({struct Cyc_PP_Doc*_tmp875=Cyc_PP_text(({const char*_tmp4AA="/* using ";_tag_fat(_tmp4AA,sizeof(char),10U);}));_tmp4A9[0]=_tmp875;}),({
struct Cyc_PP_Doc*_tmp874=Cyc_Absynpp_qvar2doc(q);_tmp4A9[1]=_tmp874;}),({
struct Cyc_PP_Doc*_tmp873=Cyc_PP_blank_doc();_tmp4A9[2]=_tmp873;}),({
struct Cyc_PP_Doc*_tmp872=Cyc_Absynpp_lb();_tmp4A9[3]=_tmp872;}),({
struct Cyc_PP_Doc*_tmp871=Cyc_PP_text(({const char*_tmp4AB=" */";_tag_fat(_tmp4AB,sizeof(char),4U);}));_tmp4A9[4]=_tmp871;}),({
struct Cyc_PP_Doc*_tmp870=Cyc_PP_line_doc();_tmp4A9[5]=_tmp870;}),({
struct Cyc_PP_Doc*_tmp86F=({struct Cyc_PP_Doc*(*_tmp86E)(struct Cyc_PP_Doc*(*pp)(struct Cyc_Absyn_Decl*),struct _fat_ptr sep,struct Cyc_List_List*l)=({struct Cyc_PP_Doc*(*_tmp4AC)(struct Cyc_PP_Doc*(*pp)(struct Cyc_Absyn_Decl*),struct _fat_ptr sep,struct Cyc_List_List*l)=(struct Cyc_PP_Doc*(*)(struct Cyc_PP_Doc*(*pp)(struct Cyc_Absyn_Decl*),struct _fat_ptr sep,struct Cyc_List_List*l))Cyc_PP_ppseql;_tmp4AC;});struct _fat_ptr _tmp86D=({const char*_tmp4AD="";_tag_fat(_tmp4AD,sizeof(char),1U);});_tmp86E(Cyc_Absynpp_decl2doc,_tmp86D,tdl);});_tmp4A9[6]=_tmp86F;}),({
struct Cyc_PP_Doc*_tmp86C=Cyc_PP_line_doc();_tmp4A9[7]=_tmp86C;}),({
struct Cyc_PP_Doc*_tmp86B=Cyc_PP_text(({const char*_tmp4AE="/* ";_tag_fat(_tmp4AE,sizeof(char),4U);}));_tmp4A9[8]=_tmp86B;}),({
struct Cyc_PP_Doc*_tmp86A=Cyc_Absynpp_rb();_tmp4A9[9]=_tmp86A;}),({
struct Cyc_PP_Doc*_tmp869=Cyc_PP_text(({const char*_tmp4AF=" */";_tag_fat(_tmp4AF,sizeof(char),4U);}));_tmp4A9[10]=_tmp869;});Cyc_PP_cat(_tag_fat(_tmp4A9,sizeof(struct Cyc_PP_Doc*),11U));});}
goto _LL0;}case 11U: _LL17: _tmp465=((struct Cyc_Absyn_ExternC_d_Absyn_Raw_decl_struct*)_tmp460)->f1;_LL18: {struct Cyc_List_List*tdl=_tmp465;
# 1894
if(Cyc_Absynpp_print_externC_stmts)
s=({struct Cyc_PP_Doc*_tmp4B0[6U];({struct Cyc_PP_Doc*_tmp87D=Cyc_PP_text(({const char*_tmp4B1="extern \"C\" ";_tag_fat(_tmp4B1,sizeof(char),12U);}));_tmp4B0[0]=_tmp87D;}),({
struct Cyc_PP_Doc*_tmp87C=Cyc_Absynpp_lb();_tmp4B0[1]=_tmp87C;}),({
struct Cyc_PP_Doc*_tmp87B=Cyc_PP_line_doc();_tmp4B0[2]=_tmp87B;}),({
struct Cyc_PP_Doc*_tmp87A=({struct Cyc_PP_Doc*(*_tmp879)(struct Cyc_PP_Doc*(*pp)(struct Cyc_Absyn_Decl*),struct _fat_ptr sep,struct Cyc_List_List*l)=({struct Cyc_PP_Doc*(*_tmp4B2)(struct Cyc_PP_Doc*(*pp)(struct Cyc_Absyn_Decl*),struct _fat_ptr sep,struct Cyc_List_List*l)=(struct Cyc_PP_Doc*(*)(struct Cyc_PP_Doc*(*pp)(struct Cyc_Absyn_Decl*),struct _fat_ptr sep,struct Cyc_List_List*l))Cyc_PP_ppseql;_tmp4B2;});struct _fat_ptr _tmp878=({const char*_tmp4B3="";_tag_fat(_tmp4B3,sizeof(char),1U);});_tmp879(Cyc_Absynpp_decl2doc,_tmp878,tdl);});_tmp4B0[3]=_tmp87A;}),({
struct Cyc_PP_Doc*_tmp877=Cyc_PP_line_doc();_tmp4B0[4]=_tmp877;}),({
struct Cyc_PP_Doc*_tmp876=Cyc_Absynpp_rb();_tmp4B0[5]=_tmp876;});Cyc_PP_cat(_tag_fat(_tmp4B0,sizeof(struct Cyc_PP_Doc*),6U));});else{
# 1902
s=({struct Cyc_PP_Doc*_tmp4B4[9U];({struct Cyc_PP_Doc*_tmp888=Cyc_PP_text(({const char*_tmp4B5="/* extern \"C\" ";_tag_fat(_tmp4B5,sizeof(char),15U);}));_tmp4B4[0]=_tmp888;}),({
struct Cyc_PP_Doc*_tmp887=Cyc_Absynpp_lb();_tmp4B4[1]=_tmp887;}),({
struct Cyc_PP_Doc*_tmp886=Cyc_PP_text(({const char*_tmp4B6=" */";_tag_fat(_tmp4B6,sizeof(char),4U);}));_tmp4B4[2]=_tmp886;}),({
struct Cyc_PP_Doc*_tmp885=Cyc_PP_line_doc();_tmp4B4[3]=_tmp885;}),({
struct Cyc_PP_Doc*_tmp884=({struct Cyc_PP_Doc*(*_tmp883)(struct Cyc_PP_Doc*(*pp)(struct Cyc_Absyn_Decl*),struct _fat_ptr sep,struct Cyc_List_List*l)=({struct Cyc_PP_Doc*(*_tmp4B7)(struct Cyc_PP_Doc*(*pp)(struct Cyc_Absyn_Decl*),struct _fat_ptr sep,struct Cyc_List_List*l)=(struct Cyc_PP_Doc*(*)(struct Cyc_PP_Doc*(*pp)(struct Cyc_Absyn_Decl*),struct _fat_ptr sep,struct Cyc_List_List*l))Cyc_PP_ppseql;_tmp4B7;});struct _fat_ptr _tmp882=({const char*_tmp4B8="";_tag_fat(_tmp4B8,sizeof(char),1U);});_tmp883(Cyc_Absynpp_decl2doc,_tmp882,tdl);});_tmp4B4[4]=_tmp884;}),({
struct Cyc_PP_Doc*_tmp881=Cyc_PP_line_doc();_tmp4B4[5]=_tmp881;}),({
struct Cyc_PP_Doc*_tmp880=Cyc_PP_text(({const char*_tmp4B9="/* ";_tag_fat(_tmp4B9,sizeof(char),4U);}));_tmp4B4[6]=_tmp880;}),({
struct Cyc_PP_Doc*_tmp87F=Cyc_Absynpp_rb();_tmp4B4[7]=_tmp87F;}),({
struct Cyc_PP_Doc*_tmp87E=Cyc_PP_text(({const char*_tmp4BA=" */";_tag_fat(_tmp4BA,sizeof(char),4U);}));_tmp4B4[8]=_tmp87E;});Cyc_PP_cat(_tag_fat(_tmp4B4,sizeof(struct Cyc_PP_Doc*),9U));});}
goto _LL0;}case 12U: _LL19: _tmp461=((struct Cyc_Absyn_ExternCinclude_d_Absyn_Raw_decl_struct*)_tmp460)->f1;_tmp462=((struct Cyc_Absyn_ExternCinclude_d_Absyn_Raw_decl_struct*)_tmp460)->f2;_tmp463=((struct Cyc_Absyn_ExternCinclude_d_Absyn_Raw_decl_struct*)_tmp460)->f3;_tmp464=((struct Cyc_Absyn_ExternCinclude_d_Absyn_Raw_decl_struct*)_tmp460)->f4;_LL1A: {struct Cyc_List_List*tdl=_tmp461;struct Cyc_List_List*ovrs=_tmp462;struct Cyc_List_List*exs=_tmp463;struct _tuple10*wc=_tmp464;
# 1913
if(Cyc_Absynpp_print_externC_stmts){
struct Cyc_PP_Doc*exs_doc;
struct Cyc_PP_Doc*ovrs_doc;
if(exs != 0)
exs_doc=({struct Cyc_PP_Doc*_tmp4BB[7U];({struct Cyc_PP_Doc*_tmp891=Cyc_Absynpp_rb();_tmp4BB[0]=_tmp891;}),({struct Cyc_PP_Doc*_tmp890=Cyc_PP_text(({const char*_tmp4BC=" export ";_tag_fat(_tmp4BC,sizeof(char),9U);}));_tmp4BB[1]=_tmp890;}),({struct Cyc_PP_Doc*_tmp88F=Cyc_Absynpp_lb();_tmp4BB[2]=_tmp88F;}),({
struct Cyc_PP_Doc*_tmp88E=Cyc_PP_line_doc();_tmp4BB[3]=_tmp88E;}),({struct Cyc_PP_Doc*_tmp88D=({struct Cyc_PP_Doc*(*_tmp88C)(struct Cyc_PP_Doc*(*pp)(struct _tuple20*),struct _fat_ptr sep,struct Cyc_List_List*l)=({struct Cyc_PP_Doc*(*_tmp4BD)(struct Cyc_PP_Doc*(*pp)(struct _tuple20*),struct _fat_ptr sep,struct Cyc_List_List*l)=(struct Cyc_PP_Doc*(*)(struct Cyc_PP_Doc*(*pp)(struct _tuple20*),struct _fat_ptr sep,struct Cyc_List_List*l))Cyc_PP_ppseql;_tmp4BD;});struct _fat_ptr _tmp88B=({const char*_tmp4BE=",";_tag_fat(_tmp4BE,sizeof(char),2U);});_tmp88C(Cyc_Absynpp_export2doc,_tmp88B,exs);});_tmp4BB[4]=_tmp88D;}),({
struct Cyc_PP_Doc*_tmp88A=Cyc_PP_line_doc();_tmp4BB[5]=_tmp88A;}),({struct Cyc_PP_Doc*_tmp889=Cyc_Absynpp_rb();_tmp4BB[6]=_tmp889;});Cyc_PP_cat(_tag_fat(_tmp4BB,sizeof(struct Cyc_PP_Doc*),7U));});else{
# 1921
exs_doc=Cyc_Absynpp_rb();}
if(ovrs != 0)
ovrs_doc=({struct Cyc_PP_Doc*_tmp4BF[7U];({struct Cyc_PP_Doc*_tmp89A=Cyc_Absynpp_rb();_tmp4BF[0]=_tmp89A;}),({struct Cyc_PP_Doc*_tmp899=Cyc_PP_text(({const char*_tmp4C0=" cycdef ";_tag_fat(_tmp4C0,sizeof(char),9U);}));_tmp4BF[1]=_tmp899;}),({struct Cyc_PP_Doc*_tmp898=Cyc_Absynpp_lb();_tmp4BF[2]=_tmp898;}),({
struct Cyc_PP_Doc*_tmp897=Cyc_PP_line_doc();_tmp4BF[3]=_tmp897;}),({struct Cyc_PP_Doc*_tmp896=({struct Cyc_PP_Doc*(*_tmp895)(struct Cyc_PP_Doc*(*pp)(struct Cyc_Absyn_Decl*),struct _fat_ptr sep,struct Cyc_List_List*l)=({struct Cyc_PP_Doc*(*_tmp4C1)(struct Cyc_PP_Doc*(*pp)(struct Cyc_Absyn_Decl*),struct _fat_ptr sep,struct Cyc_List_List*l)=(struct Cyc_PP_Doc*(*)(struct Cyc_PP_Doc*(*pp)(struct Cyc_Absyn_Decl*),struct _fat_ptr sep,struct Cyc_List_List*l))Cyc_PP_ppseql;_tmp4C1;});struct _fat_ptr _tmp894=({const char*_tmp4C2="";_tag_fat(_tmp4C2,sizeof(char),1U);});_tmp895(Cyc_Absynpp_decl2doc,_tmp894,ovrs);});_tmp4BF[4]=_tmp896;}),({
struct Cyc_PP_Doc*_tmp893=Cyc_PP_line_doc();_tmp4BF[5]=_tmp893;}),({struct Cyc_PP_Doc*_tmp892=Cyc_Absynpp_rb();_tmp4BF[6]=_tmp892;});Cyc_PP_cat(_tag_fat(_tmp4BF,sizeof(struct Cyc_PP_Doc*),7U));});else{
# 1927
ovrs_doc=Cyc_Absynpp_rb();}
s=({struct Cyc_PP_Doc*_tmp4C3[6U];({struct Cyc_PP_Doc*_tmp8A1=Cyc_PP_text(({const char*_tmp4C4="extern \"C include\" ";_tag_fat(_tmp4C4,sizeof(char),20U);}));_tmp4C3[0]=_tmp8A1;}),({
struct Cyc_PP_Doc*_tmp8A0=Cyc_Absynpp_lb();_tmp4C3[1]=_tmp8A0;}),({
struct Cyc_PP_Doc*_tmp89F=Cyc_PP_line_doc();_tmp4C3[2]=_tmp89F;}),({
struct Cyc_PP_Doc*_tmp89E=({struct Cyc_PP_Doc*(*_tmp89D)(struct Cyc_PP_Doc*(*pp)(struct Cyc_Absyn_Decl*),struct _fat_ptr sep,struct Cyc_List_List*l)=({struct Cyc_PP_Doc*(*_tmp4C5)(struct Cyc_PP_Doc*(*pp)(struct Cyc_Absyn_Decl*),struct _fat_ptr sep,struct Cyc_List_List*l)=(struct Cyc_PP_Doc*(*)(struct Cyc_PP_Doc*(*pp)(struct Cyc_Absyn_Decl*),struct _fat_ptr sep,struct Cyc_List_List*l))Cyc_PP_ppseql;_tmp4C5;});struct _fat_ptr _tmp89C=({const char*_tmp4C6="";_tag_fat(_tmp4C6,sizeof(char),1U);});_tmp89D(Cyc_Absynpp_decl2doc,_tmp89C,tdl);});_tmp4C3[3]=_tmp89E;}),({
struct Cyc_PP_Doc*_tmp89B=Cyc_PP_line_doc();_tmp4C3[4]=_tmp89B;}),_tmp4C3[5]=exs_doc;Cyc_PP_cat(_tag_fat(_tmp4C3,sizeof(struct Cyc_PP_Doc*),6U));});}else{
# 1935
s=({struct Cyc_PP_Doc*_tmp4C7[9U];({struct Cyc_PP_Doc*_tmp8AC=Cyc_PP_text(({const char*_tmp4C8="/* extern \"C include\" ";_tag_fat(_tmp4C8,sizeof(char),23U);}));_tmp4C7[0]=_tmp8AC;}),({
struct Cyc_PP_Doc*_tmp8AB=Cyc_Absynpp_lb();_tmp4C7[1]=_tmp8AB;}),({
struct Cyc_PP_Doc*_tmp8AA=Cyc_PP_text(({const char*_tmp4C9=" */";_tag_fat(_tmp4C9,sizeof(char),4U);}));_tmp4C7[2]=_tmp8AA;}),({
struct Cyc_PP_Doc*_tmp8A9=Cyc_PP_line_doc();_tmp4C7[3]=_tmp8A9;}),({
struct Cyc_PP_Doc*_tmp8A8=({struct Cyc_PP_Doc*(*_tmp8A7)(struct Cyc_PP_Doc*(*pp)(struct Cyc_Absyn_Decl*),struct _fat_ptr sep,struct Cyc_List_List*l)=({struct Cyc_PP_Doc*(*_tmp4CA)(struct Cyc_PP_Doc*(*pp)(struct Cyc_Absyn_Decl*),struct _fat_ptr sep,struct Cyc_List_List*l)=(struct Cyc_PP_Doc*(*)(struct Cyc_PP_Doc*(*pp)(struct Cyc_Absyn_Decl*),struct _fat_ptr sep,struct Cyc_List_List*l))Cyc_PP_ppseql;_tmp4CA;});struct _fat_ptr _tmp8A6=({const char*_tmp4CB="";_tag_fat(_tmp4CB,sizeof(char),1U);});_tmp8A7(Cyc_Absynpp_decl2doc,_tmp8A6,tdl);});_tmp4C7[4]=_tmp8A8;}),({
struct Cyc_PP_Doc*_tmp8A5=Cyc_PP_line_doc();_tmp4C7[5]=_tmp8A5;}),({
struct Cyc_PP_Doc*_tmp8A4=Cyc_PP_text(({const char*_tmp4CC="/* ";_tag_fat(_tmp4CC,sizeof(char),4U);}));_tmp4C7[6]=_tmp8A4;}),({
struct Cyc_PP_Doc*_tmp8A3=Cyc_Absynpp_rb();_tmp4C7[7]=_tmp8A3;}),({
struct Cyc_PP_Doc*_tmp8A2=Cyc_PP_text(({const char*_tmp4CD=" */";_tag_fat(_tmp4CD,sizeof(char),4U);}));_tmp4C7[8]=_tmp8A2;});Cyc_PP_cat(_tag_fat(_tmp4C7,sizeof(struct Cyc_PP_Doc*),9U));});}
goto _LL0;}case 13U: _LL1B: _LL1C:
# 1946
 s=({struct Cyc_PP_Doc*_tmp4CE[2U];({struct Cyc_PP_Doc*_tmp8AE=Cyc_PP_text(({const char*_tmp4CF="__cyclone_port_on__;";_tag_fat(_tmp4CF,sizeof(char),21U);}));_tmp4CE[0]=_tmp8AE;}),({struct Cyc_PP_Doc*_tmp8AD=Cyc_Absynpp_lb();_tmp4CE[1]=_tmp8AD;});Cyc_PP_cat(_tag_fat(_tmp4CE,sizeof(struct Cyc_PP_Doc*),2U));});
goto _LL0;case 14U: _LL1D: _LL1E:
# 1949
 s=({struct Cyc_PP_Doc*_tmp4D0[2U];({struct Cyc_PP_Doc*_tmp8B0=Cyc_PP_text(({const char*_tmp4D1="__cyclone_port_off__;";_tag_fat(_tmp4D1,sizeof(char),22U);}));_tmp4D0[0]=_tmp8B0;}),({struct Cyc_PP_Doc*_tmp8AF=Cyc_Absynpp_lb();_tmp4D0[1]=_tmp8AF;});Cyc_PP_cat(_tag_fat(_tmp4D0,sizeof(struct Cyc_PP_Doc*),2U));});
goto _LL0;case 15U: _LL1F: _LL20:
# 1952
 s=({struct Cyc_PP_Doc*_tmp4D2[2U];({struct Cyc_PP_Doc*_tmp8B2=Cyc_PP_text(({const char*_tmp4D3="__tempest_on__;";_tag_fat(_tmp4D3,sizeof(char),16U);}));_tmp4D2[0]=_tmp8B2;}),({struct Cyc_PP_Doc*_tmp8B1=Cyc_Absynpp_lb();_tmp4D2[1]=_tmp8B1;});Cyc_PP_cat(_tag_fat(_tmp4D2,sizeof(struct Cyc_PP_Doc*),2U));});
goto _LL0;default: _LL21: _LL22:
# 1955
 s=({struct Cyc_PP_Doc*_tmp4D4[2U];({struct Cyc_PP_Doc*_tmp8B4=Cyc_PP_text(({const char*_tmp4D5="__tempest_off__;";_tag_fat(_tmp4D5,sizeof(char),17U);}));_tmp4D4[0]=_tmp8B4;}),({struct Cyc_PP_Doc*_tmp8B3=Cyc_Absynpp_lb();_tmp4D4[1]=_tmp8B3;});Cyc_PP_cat(_tag_fat(_tmp4D4,sizeof(struct Cyc_PP_Doc*),2U));});
goto _LL0;}_LL0:;}
# 1959
return s;}
# 607 "absynpp.cyc"
struct Cyc_PP_Doc*Cyc_Absynpp_scope2doc(enum Cyc_Absyn_Scope sc){
# 1963 "absynpp.cyc"
if(Cyc_Absynpp_print_for_cycdoc)return Cyc_PP_nil_doc();{enum Cyc_Absyn_Scope _tmp4D6=sc;switch(_tmp4D6){case Cyc_Absyn_Static: _LL1: _LL2:
# 1965
 return Cyc_PP_text(({const char*_tmp4D7="static ";_tag_fat(_tmp4D7,sizeof(char),8U);}));case Cyc_Absyn_Public: _LL3: _LL4:
 return Cyc_PP_nil_doc();case Cyc_Absyn_Extern: _LL5: _LL6:
 return Cyc_PP_text(({const char*_tmp4D8="extern ";_tag_fat(_tmp4D8,sizeof(char),8U);}));case Cyc_Absyn_ExternC: _LL7: _LL8:
 return Cyc_PP_text(({const char*_tmp4D9="extern \"C\" ";_tag_fat(_tmp4D9,sizeof(char),12U);}));case Cyc_Absyn_Abstract: _LL9: _LLA:
 return Cyc_PP_text(({const char*_tmp4DA="abstract ";_tag_fat(_tmp4DA,sizeof(char),10U);}));case Cyc_Absyn_Register: _LLB: _LLC:
 return Cyc_PP_text(({const char*_tmp4DB="register ";_tag_fat(_tmp4DB,sizeof(char),10U);}));default: _LLD: _LLE:
 return Cyc_PP_nil_doc();}_LL0:;}}
# 607 "absynpp.cyc"
int Cyc_Absynpp_exists_temp_tvar_in_effect(void*t){
# 1977 "absynpp.cyc"
void*_stmttmp22=Cyc_Tcutil_compress(t);void*_tmp4DC=_stmttmp22;struct Cyc_List_List*_tmp4DD;struct Cyc_Absyn_Tvar*_tmp4DE;switch(*((int*)_tmp4DC)){case 2U: _LL1: _tmp4DE=((struct Cyc_Absyn_VarType_Absyn_Type_struct*)_tmp4DC)->f1;_LL2: {struct Cyc_Absyn_Tvar*tv=_tmp4DE;
return Cyc_Tcutil_is_temp_tvar(tv);}case 0U: if(((struct Cyc_Absyn_JoinCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp4DC)->f1)->tag == 9U){_LL3: _tmp4DD=((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp4DC)->f2;_LL4: {struct Cyc_List_List*l=_tmp4DD;
return Cyc_List_exists(Cyc_Absynpp_exists_temp_tvar_in_effect,l);}}else{goto _LL5;}default: _LL5: _LL6:
 return 0;}_LL0:;}
# 607 "absynpp.cyc"
int Cyc_Absynpp_is_anon_aggrtype(void*t){
# 1989 "absynpp.cyc"
void*_tmp4DF=t;void*_tmp4E1;struct Cyc_Absyn_Typedefdecl*_tmp4E0;switch(*((int*)_tmp4DF)){case 7U: _LL1: _LL2:
 return 1;case 0U: if(((struct Cyc_Absyn_AnonEnumCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp4DF)->f1)->tag == 16U){_LL3: _LL4:
 return 1;}else{goto _LL7;}case 8U: _LL5: _tmp4E0=((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmp4DF)->f3;_tmp4E1=(void*)((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmp4DF)->f4;if(_tmp4E1 != 0){_LL6: {struct Cyc_Absyn_Typedefdecl*td=_tmp4E0;void*x=_tmp4E1;
# 1995
return Cyc_Absynpp_is_anon_aggrtype(x);}}else{goto _LL7;}default: _LL7: _LL8:
 return 0;}_LL0:;}
# 607 "absynpp.cyc"
static struct Cyc_List_List*Cyc_Absynpp_bubble_attributes(struct _RegionHandle*r,void*atts,struct Cyc_List_List*tms){
# 2005 "absynpp.cyc"
if(tms != 0 && tms->tl != 0){
struct _tuple18 _stmttmp23=({struct _tuple18 _tmp566;_tmp566.f1=(void*)tms->hd,_tmp566.f2=(void*)((struct Cyc_List_List*)_check_null(tms->tl))->hd;_tmp566;});struct _tuple18 _tmp4E2=_stmttmp23;if(((struct Cyc_Absyn_Pointer_mod_Absyn_Type_modifier_struct*)_tmp4E2.f1)->tag == 2U){if(((struct Cyc_Absyn_Function_mod_Absyn_Type_modifier_struct*)_tmp4E2.f2)->tag == 3U){_LL1: _LL2:
# 2008
 return({struct Cyc_List_List*_tmp4E4=_region_malloc(r,sizeof(*_tmp4E4));_tmp4E4->hd=(void*)tms->hd,({struct Cyc_List_List*_tmp8B6=({struct Cyc_List_List*_tmp4E3=_region_malloc(r,sizeof(*_tmp4E3));_tmp4E3->hd=(void*)((struct Cyc_List_List*)_check_null(tms->tl))->hd,({struct Cyc_List_List*_tmp8B5=Cyc_Absynpp_bubble_attributes(r,atts,((struct Cyc_List_List*)_check_null(tms->tl))->tl);_tmp4E3->tl=_tmp8B5;});_tmp4E3;});_tmp4E4->tl=_tmp8B6;});_tmp4E4;});}else{goto _LL3;}}else{_LL3: _LL4:
 return({struct Cyc_List_List*_tmp4E5=_region_malloc(r,sizeof(*_tmp4E5));_tmp4E5->hd=atts,_tmp4E5->tl=tms;_tmp4E5;});}_LL0:;}else{
# 2011
return({struct Cyc_List_List*_tmp4E6=_region_malloc(r,sizeof(*_tmp4E6));_tmp4E6->hd=atts,_tmp4E6->tl=tms;_tmp4E6;});}}
# 607 "absynpp.cyc"
static void Cyc_Absynpp_rewrite_temp_tvar(struct Cyc_Absyn_Tvar*t){
# 2015 "absynpp.cyc"
if(!Cyc_Tcutil_is_temp_tvar(t))return;{struct _fat_ptr s=({
struct _fat_ptr _tmp8B7=({const char*_tmp4EC="`";_tag_fat(_tmp4EC,sizeof(char),2U);});Cyc_strconcat(_tmp8B7,(struct _fat_ptr)*t->name);});
({struct _fat_ptr _tmp4E7=_fat_ptr_plus(s,sizeof(char),1);char _tmp4E8=*((char*)_check_fat_subscript(_tmp4E7,sizeof(char),0U));char _tmp4E9='t';if(_get_fat_size(_tmp4E7,sizeof(char))== 1U &&(_tmp4E8 == 0 && _tmp4E9 != 0))_throw_arraybounds();*((char*)_tmp4E7.curr)=_tmp4E9;});
({struct _fat_ptr*_tmp8B8=({unsigned _tmp4EB=1;struct _fat_ptr*_tmp4EA=_cycalloc(_check_times(_tmp4EB,sizeof(struct _fat_ptr)));_tmp4EA[0]=(struct _fat_ptr)s;_tmp4EA;});t->name=_tmp8B8;});}}
# 607 "absynpp.cyc"
struct _tuple13 Cyc_Absynpp_to_tms(struct _RegionHandle*r,struct Cyc_Absyn_Tqual tq,void*t){
# 2025 "absynpp.cyc"
void*_tmp4ED=t;void*_tmp4F1;struct Cyc_Absyn_Typedefdecl*_tmp4F0;struct Cyc_List_List*_tmp4EF;struct _tuple0*_tmp4EE;int _tmp4F4;void*_tmp4F3;struct Cyc_Core_Opt*_tmp4F2;struct Cyc_Absyn_Exp*_tmp4FF;struct Cyc_Absyn_Exp*_tmp4FE;struct Cyc_List_List*_tmp4FD;struct Cyc_List_List*_tmp4FC;struct Cyc_Absyn_VarargInfo*_tmp4FB;int _tmp4FA;struct Cyc_List_List*_tmp4F9;void*_tmp4F8;struct Cyc_Absyn_Tqual _tmp4F7;void*_tmp4F6;struct Cyc_List_List*_tmp4F5;struct Cyc_Absyn_PtrAtts _tmp502;struct Cyc_Absyn_Tqual _tmp501;void*_tmp500;unsigned _tmp507;void*_tmp506;struct Cyc_Absyn_Exp*_tmp505;struct Cyc_Absyn_Tqual _tmp504;void*_tmp503;switch(*((int*)_tmp4ED)){case 4U: _LL1: _tmp503=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp4ED)->f1).elt_type;_tmp504=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp4ED)->f1).tq;_tmp505=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp4ED)->f1).num_elts;_tmp506=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp4ED)->f1).zero_term;_tmp507=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp4ED)->f1).zt_loc;_LL2: {void*t2=_tmp503;struct Cyc_Absyn_Tqual tq2=_tmp504;struct Cyc_Absyn_Exp*e=_tmp505;void*zeroterm=_tmp506;unsigned ztl=_tmp507;
# 2028
struct _tuple13 _stmttmp24=Cyc_Absynpp_to_tms(r,tq2,t2);struct Cyc_List_List*_tmp50A;void*_tmp509;struct Cyc_Absyn_Tqual _tmp508;_LLE: _tmp508=_stmttmp24.f1;_tmp509=_stmttmp24.f2;_tmp50A=_stmttmp24.f3;_LLF: {struct Cyc_Absyn_Tqual tq3=_tmp508;void*t3=_tmp509;struct Cyc_List_List*tml3=_tmp50A;
void*tm;
if(e == 0)
tm=(void*)({struct Cyc_Absyn_Carray_mod_Absyn_Type_modifier_struct*_tmp50B=_region_malloc(r,sizeof(*_tmp50B));_tmp50B->tag=0U,_tmp50B->f1=zeroterm,_tmp50B->f2=ztl;_tmp50B;});else{
# 2033
tm=(void*)({struct Cyc_Absyn_ConstArray_mod_Absyn_Type_modifier_struct*_tmp50C=_region_malloc(r,sizeof(*_tmp50C));_tmp50C->tag=1U,_tmp50C->f1=e,_tmp50C->f2=zeroterm,_tmp50C->f3=ztl;_tmp50C;});}
return({struct _tuple13 _tmp567;_tmp567.f1=tq3,_tmp567.f2=t3,({struct Cyc_List_List*_tmp8B9=({struct Cyc_List_List*_tmp50D=_region_malloc(r,sizeof(*_tmp50D));_tmp50D->hd=tm,_tmp50D->tl=tml3;_tmp50D;});_tmp567.f3=_tmp8B9;});_tmp567;});}}case 3U: _LL3: _tmp500=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp4ED)->f1).elt_type;_tmp501=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp4ED)->f1).elt_tq;_tmp502=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp4ED)->f1).ptr_atts;_LL4: {void*t2=_tmp500;struct Cyc_Absyn_Tqual tq2=_tmp501;struct Cyc_Absyn_PtrAtts ptratts=_tmp502;
# 2037
struct _tuple13 _stmttmp25=Cyc_Absynpp_to_tms(r,tq2,t2);struct Cyc_List_List*_tmp510;void*_tmp50F;struct Cyc_Absyn_Tqual _tmp50E;_LL11: _tmp50E=_stmttmp25.f1;_tmp50F=_stmttmp25.f2;_tmp510=_stmttmp25.f3;_LL12: {struct Cyc_Absyn_Tqual tq3=_tmp50E;void*t3=_tmp50F;struct Cyc_List_List*tml3=_tmp510;
tml3=({struct Cyc_List_List*_tmp512=_region_malloc(r,sizeof(*_tmp512));({void*_tmp8BA=(void*)({struct Cyc_Absyn_Pointer_mod_Absyn_Type_modifier_struct*_tmp511=_region_malloc(r,sizeof(*_tmp511));_tmp511->tag=2U,_tmp511->f1=ptratts,_tmp511->f2=tq;_tmp511;});_tmp512->hd=_tmp8BA;}),_tmp512->tl=tml3;_tmp512;});
return({struct _tuple13 _tmp568;_tmp568.f1=tq3,_tmp568.f2=t3,_tmp568.f3=tml3;_tmp568;});}}case 5U: _LL5: _tmp4F5=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp4ED)->f1).tvars;_tmp4F6=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp4ED)->f1).effect;_tmp4F7=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp4ED)->f1).ret_tqual;_tmp4F8=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp4ED)->f1).ret_type;_tmp4F9=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp4ED)->f1).args;_tmp4FA=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp4ED)->f1).c_varargs;_tmp4FB=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp4ED)->f1).cyc_varargs;_tmp4FC=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp4ED)->f1).rgn_po;_tmp4FD=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp4ED)->f1).attributes;_tmp4FE=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp4ED)->f1).requires_clause;_tmp4FF=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp4ED)->f1).ensures_clause;_LL6: {struct Cyc_List_List*typvars=_tmp4F5;void*effopt=_tmp4F6;struct Cyc_Absyn_Tqual t2qual=_tmp4F7;void*t2=_tmp4F8;struct Cyc_List_List*args=_tmp4F9;int c_varargs=_tmp4FA;struct Cyc_Absyn_VarargInfo*cyc_varargs=_tmp4FB;struct Cyc_List_List*rgn_po=_tmp4FC;struct Cyc_List_List*fn_atts=_tmp4FD;struct Cyc_Absyn_Exp*req=_tmp4FE;struct Cyc_Absyn_Exp*ens=_tmp4FF;
# 2043
if(!Cyc_Absynpp_print_all_tvars){
# 2047
if(effopt == 0 || Cyc_Absynpp_exists_temp_tvar_in_effect(effopt)){
effopt=0;
typvars=0;}}else{
# 2052
if(Cyc_Absynpp_rewrite_temp_tvars)
# 2055
({void(*_tmp8BB)(void(*f)(struct Cyc_Absyn_Tvar*),struct Cyc_List_List*x)=({void(*_tmp513)(void(*f)(struct Cyc_Absyn_Tvar*),struct Cyc_List_List*x)=(void(*)(void(*f)(struct Cyc_Absyn_Tvar*),struct Cyc_List_List*x))Cyc_List_iter;_tmp513;});_tmp8BB(Cyc_Absynpp_rewrite_temp_tvar,typvars);});}{
# 2060
struct _tuple13 _stmttmp26=Cyc_Absynpp_to_tms(r,t2qual,t2);struct Cyc_List_List*_tmp516;void*_tmp515;struct Cyc_Absyn_Tqual _tmp514;_LL14: _tmp514=_stmttmp26.f1;_tmp515=_stmttmp26.f2;_tmp516=_stmttmp26.f3;_LL15: {struct Cyc_Absyn_Tqual tq3=_tmp514;void*t3=_tmp515;struct Cyc_List_List*tml3=_tmp516;
struct Cyc_List_List*tms=tml3;
# 2071 "absynpp.cyc"
{enum Cyc_Cyclone_C_Compilers _tmp517=Cyc_Cyclone_c_compiler;if(_tmp517 == Cyc_Cyclone_Gcc_c){_LL17: _LL18:
# 2073
 if(fn_atts != 0)
# 2076
tms=({struct _RegionHandle*_tmp8BD=r;void*_tmp8BC=(void*)({struct Cyc_Absyn_Attributes_mod_Absyn_Type_modifier_struct*_tmp518=_region_malloc(r,sizeof(*_tmp518));_tmp518->tag=5U,_tmp518->f1=0U,_tmp518->f2=fn_atts;_tmp518;});Cyc_Absynpp_bubble_attributes(_tmp8BD,_tmp8BC,tms);});
# 2073
tms=({struct Cyc_List_List*_tmp51B=_region_malloc(r,sizeof(*_tmp51B));
# 2078
({void*_tmp8BF=(void*)({struct Cyc_Absyn_Function_mod_Absyn_Type_modifier_struct*_tmp51A=_region_malloc(r,sizeof(*_tmp51A));
_tmp51A->tag=3U,({void*_tmp8BE=(void*)({struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct*_tmp519=_region_malloc(r,sizeof(*_tmp519));_tmp519->tag=1U,_tmp519->f1=args,_tmp519->f2=c_varargs,_tmp519->f3=cyc_varargs,_tmp519->f4=effopt,_tmp519->f5=rgn_po,_tmp519->f6=req,_tmp519->f7=ens,_tmp519->f8=0,_tmp519->f9=0;_tmp519;});_tmp51A->f1=_tmp8BE;});_tmp51A;});
# 2078
_tmp51B->hd=_tmp8BF;}),_tmp51B->tl=tms;_tmp51B;});
# 2082
goto _LL16;}else{_LL19: _LL1A:
# 2084
 tms=({struct Cyc_List_List*_tmp51E=_region_malloc(r,sizeof(*_tmp51E));({void*_tmp8C1=(void*)({struct Cyc_Absyn_Function_mod_Absyn_Type_modifier_struct*_tmp51D=_region_malloc(r,sizeof(*_tmp51D));
_tmp51D->tag=3U,({void*_tmp8C0=(void*)({struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct*_tmp51C=_region_malloc(r,sizeof(*_tmp51C));_tmp51C->tag=1U,_tmp51C->f1=args,_tmp51C->f2=c_varargs,_tmp51C->f3=cyc_varargs,_tmp51C->f4=effopt,_tmp51C->f5=rgn_po,_tmp51C->f6=req,_tmp51C->f7=ens,_tmp51C->f8=0,_tmp51C->f9=0;_tmp51C;});_tmp51D->f1=_tmp8C0;});_tmp51D;});
# 2084
_tmp51E->hd=_tmp8C1;}),_tmp51E->tl=tms;_tmp51E;});
# 2088
for(0;fn_atts != 0;fn_atts=fn_atts->tl){
void*_stmttmp27=(void*)fn_atts->hd;void*_tmp51F=_stmttmp27;switch(*((int*)_tmp51F)){case 1U: _LL1C: _LL1D:
 goto _LL1F;case 2U: _LL1E: _LL1F:
 goto _LL21;case 3U: _LL20: _LL21:
# 2093
 tms=({struct Cyc_List_List*_tmp522=_region_malloc(r,sizeof(*_tmp522));({void*_tmp8C3=(void*)({struct Cyc_Absyn_Attributes_mod_Absyn_Type_modifier_struct*_tmp521=_region_malloc(r,sizeof(*_tmp521));_tmp521->tag=5U,_tmp521->f1=0U,({struct Cyc_List_List*_tmp8C2=({struct Cyc_List_List*_tmp520=_cycalloc(sizeof(*_tmp520));_tmp520->hd=(void*)fn_atts->hd,_tmp520->tl=0;_tmp520;});_tmp521->f2=_tmp8C2;});_tmp521;});_tmp522->hd=_tmp8C3;}),_tmp522->tl=tms;_tmp522;});
goto AfterAtts;default: _LL22: _LL23:
 goto _LL1B;}_LL1B:;}
# 2097
goto _LL16;}_LL16:;}
# 2100
AfterAtts:
 if(typvars != 0)
tms=({struct Cyc_List_List*_tmp524=_region_malloc(r,sizeof(*_tmp524));({void*_tmp8C4=(void*)({struct Cyc_Absyn_TypeParams_mod_Absyn_Type_modifier_struct*_tmp523=_region_malloc(r,sizeof(*_tmp523));_tmp523->tag=4U,_tmp523->f1=typvars,_tmp523->f2=0U,_tmp523->f3=1;_tmp523;});_tmp524->hd=_tmp8C4;}),_tmp524->tl=tms;_tmp524;});
# 2100
return({struct _tuple13 _tmp569;_tmp569.f1=tq3,_tmp569.f2=t3,_tmp569.f3=tms;_tmp569;});}}}case 1U: _LL7: _tmp4F2=((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp4ED)->f1;_tmp4F3=(void*)((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp4ED)->f2;_tmp4F4=((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp4ED)->f3;_LL8: {struct Cyc_Core_Opt*k=_tmp4F2;void*topt=_tmp4F3;int i=_tmp4F4;
# 2106
if(topt == 0)
return({struct _tuple13 _tmp56A;_tmp56A.f1=tq,_tmp56A.f2=t,_tmp56A.f3=0;_tmp56A;});else{
# 2109
return Cyc_Absynpp_to_tms(r,tq,topt);}}case 8U: _LL9: _tmp4EE=((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmp4ED)->f1;_tmp4EF=((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmp4ED)->f2;_tmp4F0=((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmp4ED)->f3;_tmp4F1=(void*)((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmp4ED)->f4;_LLA: {struct _tuple0*n=_tmp4EE;struct Cyc_List_List*ts=_tmp4EF;struct Cyc_Absyn_Typedefdecl*td=_tmp4F0;void*topt=_tmp4F1;
# 2115
if(topt == 0 || !Cyc_Absynpp_expand_typedefs)
return({struct _tuple13 _tmp56B;_tmp56B.f1=tq,_tmp56B.f2=t,_tmp56B.f3=0;_tmp56B;});else{
# 2118
if(tq.real_const)
tq.print_const=tq.real_const;
# 2118
return
# 2120
 Cyc_Absynpp_to_tms(r,tq,topt);}}default: _LLB: _LLC:
# 2123
 return({struct _tuple13 _tmp56C;_tmp56C.f1=tq,_tmp56C.f2=t,_tmp56C.f3=0;_tmp56C;});}_LL0:;}
# 607 "absynpp.cyc"
static int Cyc_Absynpp_is_char_ptr(void*t){
# 2129 "absynpp.cyc"
void*_tmp525=t;void*_tmp526;void*_tmp527;switch(*((int*)_tmp525)){case 1U: _LL1: _tmp527=(void*)((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp525)->f2;if(_tmp527 != 0){_LL2: {void*def=_tmp527;
return Cyc_Absynpp_is_char_ptr(def);}}else{goto _LL5;}case 3U: _LL3: _tmp526=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp525)->f1).elt_type;_LL4: {void*elt_typ=_tmp526;
# 2132
L: {
void*_tmp528=elt_typ;void*_tmp529;void*_tmp52A;switch(*((int*)_tmp528)){case 1U: _LL8: _tmp52A=(void*)((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp528)->f2;if(_tmp52A != 0){_LL9: {void*t=_tmp52A;
elt_typ=t;goto L;}}else{goto _LLE;}case 8U: _LLA: _tmp529=(void*)((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmp528)->f4;if(_tmp529 != 0){_LLB: {void*t=_tmp529;
elt_typ=t;goto L;}}else{goto _LLE;}case 0U: if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp528)->f1)->tag == 1U){if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp528)->f1)->f2 == Cyc_Absyn_Char_sz){_LLC: _LLD:
 return 1;}else{goto _LLE;}}else{goto _LLE;}default: _LLE: _LLF:
 return 0;}_LL7:;}}default: _LL5: _LL6:
# 2139
 return 0;}_LL0:;}
# 607 "absynpp.cyc"
struct Cyc_PP_Doc*Cyc_Absynpp_tqtd2doc(struct Cyc_Absyn_Tqual tq,void*typ,struct Cyc_Core_Opt*dopt){
# 2144 "absynpp.cyc"
struct _RegionHandle _tmp52B=_new_region("temp");struct _RegionHandle*temp=& _tmp52B;_push_region(temp);
{struct _tuple13 _stmttmp28=Cyc_Absynpp_to_tms(temp,tq,typ);struct Cyc_List_List*_tmp52E;void*_tmp52D;struct Cyc_Absyn_Tqual _tmp52C;_LL1: _tmp52C=_stmttmp28.f1;_tmp52D=_stmttmp28.f2;_tmp52E=_stmttmp28.f3;_LL2: {struct Cyc_Absyn_Tqual tq=_tmp52C;void*t=_tmp52D;struct Cyc_List_List*tms=_tmp52E;
tms=Cyc_List_imp_rev(tms);
if(tms == 0 && dopt == 0){
struct Cyc_PP_Doc*_tmp530=({struct Cyc_PP_Doc*_tmp52F[2U];({struct Cyc_PP_Doc*_tmp8C6=Cyc_Absynpp_tqual2doc(tq);_tmp52F[0]=_tmp8C6;}),({struct Cyc_PP_Doc*_tmp8C5=Cyc_Absynpp_ntyp2doc(t);_tmp52F[1]=_tmp8C5;});Cyc_PP_cat(_tag_fat(_tmp52F,sizeof(struct Cyc_PP_Doc*),2U));});_npop_handler(0U);return _tmp530;}else{
# 2150
struct Cyc_PP_Doc*_tmp533=({struct Cyc_PP_Doc*_tmp531[4U];({
struct Cyc_PP_Doc*_tmp8CC=Cyc_Absynpp_tqual2doc(tq);_tmp531[0]=_tmp8CC;}),({
struct Cyc_PP_Doc*_tmp8CB=Cyc_Absynpp_ntyp2doc(t);_tmp531[1]=_tmp8CB;}),({
struct Cyc_PP_Doc*_tmp8CA=Cyc_PP_text(({const char*_tmp532=" ";_tag_fat(_tmp532,sizeof(char),2U);}));_tmp531[2]=_tmp8CA;}),({
struct Cyc_PP_Doc*_tmp8C9=({int _tmp8C8=Cyc_Absynpp_is_char_ptr(typ);struct Cyc_PP_Doc*_tmp8C7=dopt == 0?Cyc_PP_nil_doc():(struct Cyc_PP_Doc*)dopt->v;Cyc_Absynpp_dtms2doc(_tmp8C8,_tmp8C7,tms);});_tmp531[3]=_tmp8C9;});Cyc_PP_cat(_tag_fat(_tmp531,sizeof(struct Cyc_PP_Doc*),4U));});_npop_handler(0U);return _tmp533;}}}
# 2145
;_pop_region(temp);}
# 607 "absynpp.cyc"
struct Cyc_PP_Doc*Cyc_Absynpp_aggrfield2doc(struct Cyc_Absyn_Aggrfield*f){
# 2159 "absynpp.cyc"
struct Cyc_PP_Doc*requires_doc;
struct Cyc_Absyn_Exp*req=f->requires_clause;
if((unsigned)req)
requires_doc=({struct Cyc_PP_Doc*_tmp534[2U];({struct Cyc_PP_Doc*_tmp8CE=Cyc_PP_text(({const char*_tmp535="@requires ";_tag_fat(_tmp535,sizeof(char),11U);}));_tmp534[0]=_tmp8CE;}),({struct Cyc_PP_Doc*_tmp8CD=Cyc_Absynpp_exp2doc(req);_tmp534[1]=_tmp8CD;});Cyc_PP_cat(_tag_fat(_tmp534,sizeof(struct Cyc_PP_Doc*),2U));});else{
# 2164
requires_doc=Cyc_PP_nil_doc();}{
# 2166
enum Cyc_Cyclone_C_Compilers _tmp536=Cyc_Cyclone_c_compiler;if(_tmp536 == Cyc_Cyclone_Gcc_c){_LL1: _LL2:
# 2169
 if(f->width != 0)
return({struct Cyc_PP_Doc*_tmp537[5U];({struct Cyc_PP_Doc*_tmp8D6=({struct Cyc_Absyn_Tqual _tmp8D5=f->tq;void*_tmp8D4=f->type;Cyc_Absynpp_tqtd2doc(_tmp8D5,_tmp8D4,({struct Cyc_Core_Opt*_tmp538=_cycalloc(sizeof(*_tmp538));({struct Cyc_PP_Doc*_tmp8D3=Cyc_PP_textptr(f->name);_tmp538->v=_tmp8D3;});_tmp538;}));});_tmp537[0]=_tmp8D6;}),({
struct Cyc_PP_Doc*_tmp8D2=Cyc_PP_text(({const char*_tmp539=":";_tag_fat(_tmp539,sizeof(char),2U);}));_tmp537[1]=_tmp8D2;}),({struct Cyc_PP_Doc*_tmp8D1=Cyc_Absynpp_exp2doc((struct Cyc_Absyn_Exp*)_check_null(f->width));_tmp537[2]=_tmp8D1;}),({
struct Cyc_PP_Doc*_tmp8D0=Cyc_Absynpp_atts2doc(f->attributes);_tmp537[3]=_tmp8D0;}),({struct Cyc_PP_Doc*_tmp8CF=Cyc_PP_text(({const char*_tmp53A=";";_tag_fat(_tmp53A,sizeof(char),2U);}));_tmp537[4]=_tmp8CF;});Cyc_PP_cat(_tag_fat(_tmp537,sizeof(struct Cyc_PP_Doc*),5U));});else{
# 2174
return({struct Cyc_PP_Doc*_tmp53B[4U];({struct Cyc_PP_Doc*_tmp8DC=({struct Cyc_Absyn_Tqual _tmp8DB=f->tq;void*_tmp8DA=f->type;Cyc_Absynpp_tqtd2doc(_tmp8DB,_tmp8DA,({struct Cyc_Core_Opt*_tmp53C=_cycalloc(sizeof(*_tmp53C));({struct Cyc_PP_Doc*_tmp8D9=Cyc_PP_textptr(f->name);_tmp53C->v=_tmp8D9;});_tmp53C;}));});_tmp53B[0]=_tmp8DC;}),({
struct Cyc_PP_Doc*_tmp8D8=Cyc_Absynpp_atts2doc(f->attributes);_tmp53B[1]=_tmp8D8;}),_tmp53B[2]=requires_doc,({struct Cyc_PP_Doc*_tmp8D7=Cyc_PP_text(({const char*_tmp53D=";";_tag_fat(_tmp53D,sizeof(char),2U);}));_tmp53B[3]=_tmp8D7;});Cyc_PP_cat(_tag_fat(_tmp53B,sizeof(struct Cyc_PP_Doc*),4U));});}}else{_LL3: _LL4:
# 2177
 if(f->width != 0)
return({struct Cyc_PP_Doc*_tmp53E[5U];({struct Cyc_PP_Doc*_tmp8E4=Cyc_Absynpp_atts2doc(f->attributes);_tmp53E[0]=_tmp8E4;}),({
struct Cyc_PP_Doc*_tmp8E3=({struct Cyc_Absyn_Tqual _tmp8E2=f->tq;void*_tmp8E1=f->type;Cyc_Absynpp_tqtd2doc(_tmp8E2,_tmp8E1,({struct Cyc_Core_Opt*_tmp53F=_cycalloc(sizeof(*_tmp53F));({struct Cyc_PP_Doc*_tmp8E0=Cyc_PP_textptr(f->name);_tmp53F->v=_tmp8E0;});_tmp53F;}));});_tmp53E[1]=_tmp8E3;}),({
struct Cyc_PP_Doc*_tmp8DF=Cyc_PP_text(({const char*_tmp540=":";_tag_fat(_tmp540,sizeof(char),2U);}));_tmp53E[2]=_tmp8DF;}),({struct Cyc_PP_Doc*_tmp8DE=Cyc_Absynpp_exp2doc((struct Cyc_Absyn_Exp*)_check_null(f->width));_tmp53E[3]=_tmp8DE;}),({struct Cyc_PP_Doc*_tmp8DD=Cyc_PP_text(({const char*_tmp541=";";_tag_fat(_tmp541,sizeof(char),2U);}));_tmp53E[4]=_tmp8DD;});Cyc_PP_cat(_tag_fat(_tmp53E,sizeof(struct Cyc_PP_Doc*),5U));});else{
# 2182
return({struct Cyc_PP_Doc*_tmp542[4U];({struct Cyc_PP_Doc*_tmp8EA=Cyc_Absynpp_atts2doc(f->attributes);_tmp542[0]=_tmp8EA;}),({
struct Cyc_PP_Doc*_tmp8E9=({struct Cyc_Absyn_Tqual _tmp8E8=f->tq;void*_tmp8E7=f->type;Cyc_Absynpp_tqtd2doc(_tmp8E8,_tmp8E7,({struct Cyc_Core_Opt*_tmp543=_cycalloc(sizeof(*_tmp543));({struct Cyc_PP_Doc*_tmp8E6=Cyc_PP_textptr(f->name);_tmp543->v=_tmp8E6;});_tmp543;}));});_tmp542[1]=_tmp8E9;}),_tmp542[2]=requires_doc,({
struct Cyc_PP_Doc*_tmp8E5=Cyc_PP_text(({const char*_tmp544=";";_tag_fat(_tmp544,sizeof(char),2U);}));_tmp542[3]=_tmp8E5;});Cyc_PP_cat(_tag_fat(_tmp542,sizeof(struct Cyc_PP_Doc*),4U));});}}_LL0:;}}
# 607 "absynpp.cyc"
struct Cyc_PP_Doc*Cyc_Absynpp_aggrfields2doc(struct Cyc_List_List*fields){
# 2190 "absynpp.cyc"
return({struct Cyc_PP_Doc*(*_tmp8EC)(struct Cyc_PP_Doc*(*pp)(struct Cyc_Absyn_Aggrfield*),struct _fat_ptr sep,struct Cyc_List_List*l)=({struct Cyc_PP_Doc*(*_tmp545)(struct Cyc_PP_Doc*(*pp)(struct Cyc_Absyn_Aggrfield*),struct _fat_ptr sep,struct Cyc_List_List*l)=(struct Cyc_PP_Doc*(*)(struct Cyc_PP_Doc*(*pp)(struct Cyc_Absyn_Aggrfield*),struct _fat_ptr sep,struct Cyc_List_List*l))Cyc_PP_ppseql;_tmp545;});struct _fat_ptr _tmp8EB=({const char*_tmp546="";_tag_fat(_tmp546,sizeof(char),1U);});_tmp8EC(Cyc_Absynpp_aggrfield2doc,_tmp8EB,fields);});}
# 607 "absynpp.cyc"
struct Cyc_PP_Doc*Cyc_Absynpp_datatypefield2doc(struct Cyc_Absyn_Datatypefield*f){
# 2194 "absynpp.cyc"
return({struct Cyc_PP_Doc*_tmp547[3U];({struct Cyc_PP_Doc*_tmp8F0=Cyc_Absynpp_scope2doc(f->sc);_tmp547[0]=_tmp8F0;}),({struct Cyc_PP_Doc*_tmp8EF=Cyc_Absynpp_typedef_name2doc(f->name);_tmp547[1]=_tmp8EF;}),
f->typs == 0?({struct Cyc_PP_Doc*_tmp8EE=Cyc_PP_nil_doc();_tmp547[2]=_tmp8EE;}):({struct Cyc_PP_Doc*_tmp8ED=Cyc_Absynpp_args2doc(f->typs);_tmp547[2]=_tmp8ED;});Cyc_PP_cat(_tag_fat(_tmp547,sizeof(struct Cyc_PP_Doc*),3U));});}
# 607 "absynpp.cyc"
struct Cyc_PP_Doc*Cyc_Absynpp_datatypefields2doc(struct Cyc_List_List*fields){
# 2199 "absynpp.cyc"
return({struct Cyc_PP_Doc*(*_tmp8F2)(struct Cyc_PP_Doc*(*pp)(struct Cyc_Absyn_Datatypefield*),struct _fat_ptr sep,struct Cyc_List_List*l)=({struct Cyc_PP_Doc*(*_tmp548)(struct Cyc_PP_Doc*(*pp)(struct Cyc_Absyn_Datatypefield*),struct _fat_ptr sep,struct Cyc_List_List*l)=(struct Cyc_PP_Doc*(*)(struct Cyc_PP_Doc*(*pp)(struct Cyc_Absyn_Datatypefield*),struct _fat_ptr sep,struct Cyc_List_List*l))Cyc_PP_ppseql;_tmp548;});struct _fat_ptr _tmp8F1=({const char*_tmp549=",";_tag_fat(_tmp549,sizeof(char),2U);});_tmp8F2(Cyc_Absynpp_datatypefield2doc,_tmp8F1,fields);});}
# 607 "absynpp.cyc"
void Cyc_Absynpp_decllist2file(struct Cyc_List_List*tdl,struct Cyc___cycFILE*f){
# 2208 "absynpp.cyc"
for(0;tdl != 0;tdl=tdl->tl){
({struct Cyc_PP_Doc*_tmp8F3=Cyc_Absynpp_decl2doc((struct Cyc_Absyn_Decl*)tdl->hd);Cyc_PP_file_of_doc(_tmp8F3,72,f);});
({void*_tmp54A=0U;({struct Cyc___cycFILE*_tmp8F5=f;struct _fat_ptr _tmp8F4=({const char*_tmp54B="\n";_tag_fat(_tmp54B,sizeof(char),2U);});Cyc_fprintf(_tmp8F5,_tmp8F4,_tag_fat(_tmp54A,sizeof(void*),0U));});});}}
# 607 "absynpp.cyc"
struct _fat_ptr Cyc_Absynpp_decllist2string(struct Cyc_List_List*tdl){
# 2215 "absynpp.cyc"
return Cyc_PP_string_of_doc(({struct _fat_ptr _tmp8F7=({const char*_tmp54C="";_tag_fat(_tmp54C,sizeof(char),1U);});Cyc_PP_seql(_tmp8F7,({struct Cyc_List_List*(*_tmp8F6)(struct Cyc_PP_Doc*(*f)(struct Cyc_Absyn_Decl*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp54D)(struct Cyc_PP_Doc*(*f)(struct Cyc_Absyn_Decl*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_PP_Doc*(*f)(struct Cyc_Absyn_Decl*),struct Cyc_List_List*x))Cyc_List_map;_tmp54D;});_tmp8F6(Cyc_Absynpp_decl2doc,tdl);}));}),72);}
# 607 "absynpp.cyc"
struct _fat_ptr Cyc_Absynpp_exp2string(struct Cyc_Absyn_Exp*e){
# 2217 "absynpp.cyc"
return Cyc_PP_string_of_doc(Cyc_Absynpp_exp2doc(e),72);}
# 607 "absynpp.cyc"
struct _fat_ptr Cyc_Absynpp_stmt2string(struct Cyc_Absyn_Stmt*s){
# 2218 "absynpp.cyc"
return Cyc_PP_string_of_doc(Cyc_Absynpp_stmt2doc(s,0,0),72);}
# 607 "absynpp.cyc"
struct _fat_ptr Cyc_Absynpp_typ2string(void*t){
# 2219 "absynpp.cyc"
return Cyc_PP_string_of_doc(Cyc_Absynpp_typ2doc(t),72);}
# 607 "absynpp.cyc"
struct _fat_ptr Cyc_Absynpp_tvar2string(struct Cyc_Absyn_Tvar*t){
# 2220 "absynpp.cyc"
return Cyc_PP_string_of_doc(Cyc_Absynpp_tvar2doc(t),72);}
# 607 "absynpp.cyc"
struct _fat_ptr Cyc_Absynpp_typ2cstring(void*t){
# 2222 "absynpp.cyc"
int old_qvar_to_Cids=Cyc_Absynpp_qvar_to_Cids;
int old_add_cyc_prefix=Cyc_Absynpp_add_cyc_prefix;
Cyc_Absynpp_qvar_to_Cids=1;
Cyc_Absynpp_add_cyc_prefix=0;{
struct _fat_ptr s=Cyc_Absynpp_typ2string(t);
Cyc_Absynpp_qvar_to_Cids=old_qvar_to_Cids;
Cyc_Absynpp_add_cyc_prefix=old_add_cyc_prefix;
return s;}}
# 607 "absynpp.cyc"
struct _fat_ptr Cyc_Absynpp_prim2string(enum Cyc_Absyn_Primop p){
# 2231 "absynpp.cyc"
return Cyc_PP_string_of_doc(Cyc_Absynpp_prim2doc(p),72);}
# 607 "absynpp.cyc"
struct _fat_ptr Cyc_Absynpp_pat2string(struct Cyc_Absyn_Pat*p){
# 2232 "absynpp.cyc"
return Cyc_PP_string_of_doc(Cyc_Absynpp_pat2doc(p),72);}
# 607 "absynpp.cyc"
struct _fat_ptr Cyc_Absynpp_scope2string(enum Cyc_Absyn_Scope sc){
# 2233 "absynpp.cyc"
return Cyc_PP_string_of_doc(Cyc_Absynpp_scope2doc(sc),72);}
# 607 "absynpp.cyc"
struct _fat_ptr Cyc_Absynpp_cnst2string(union Cyc_Absyn_Cnst c){
# 2234 "absynpp.cyc"
return Cyc_PP_string_of_doc(Cyc_Absynpp_cnst2doc(c),72);}
