#include <setjmp.h>
/* This is a C header file to be used by the output of the Cyclone to
   C translator.  The corresponding definitions are in file lib/runtime_*.c */
#ifndef _CYC_INCLUDE_H_
#define _CYC_INCLUDE_H_

/* Need one of these per thread (see runtime_stack.c). The runtime maintains 
   a stack that contains either _handler_cons structs or _RegionHandle structs.
   The tag is 0 for a handler_cons and 1 for a region handle.  */
struct _RuntimeStack {
  int tag; 
  struct _RuntimeStack *next;
  void (*cleanup)(struct _RuntimeStack *frame);
};

#ifndef offsetof
/* should be size_t, but int is fine. */
#define offsetof(t,n) ((int)(&(((t *)0)->n)))
#endif

/* Fat pointers */
struct _fat_ptr {
  unsigned char *curr; 
  unsigned char *base; 
  unsigned char *last_plus_one; 
};  

/* Discriminated Unions */
struct _xtunion_struct { char *tag; };

/* Regions */
struct _RegionPage
#ifdef CYC_REGION_PROFILE
{ unsigned total_bytes;
  unsigned free_bytes;
  /* MWH: wish we didn't have to include the stuff below ... */
  struct _RegionPage *next;
  char data[1];
}
#endif
; // abstract -- defined in runtime_memory.c
struct _RegionHandle {
  struct _RuntimeStack s;
  struct _RegionPage *curr;
  char               *offset;
  char               *last_plus_one;
  struct _DynRegionHandle *sub_regions;
#ifdef CYC_REGION_PROFILE
  const char         *name;
#else
  unsigned used_bytes;
  unsigned wasted_bytes;
#endif
};
struct _DynRegionFrame {
  struct _RuntimeStack s;
  struct _DynRegionHandle *x;
};
// A dynamic region is just a region handle.  The wrapper struct is for type
// abstraction.
struct Cyc_Core_DynamicRegion {
  struct _RegionHandle h;
};

struct _RegionHandle _new_region(const char*);
void* _region_malloc(struct _RegionHandle*, unsigned);
void* _region_calloc(struct _RegionHandle*, unsigned t, unsigned n);
void   _free_region(struct _RegionHandle*);
struct _RegionHandle*_open_dynregion(struct _DynRegionFrame*,struct _DynRegionHandle*);
void   _pop_dynregion();

/* Exceptions */
struct _handler_cons {
  struct _RuntimeStack s;
  jmp_buf handler;
};
void _push_handler(struct _handler_cons *);
void _push_region(struct _RegionHandle *);
void _npop_handler(int);
void _pop_handler();
void _pop_region();

#ifndef _throw
void* _throw_null_fn(const char*,unsigned);
void* _throw_arraybounds_fn(const char*,unsigned);
void* _throw_badalloc_fn(const char*,unsigned);
void* _throw_match_fn(const char*,unsigned);
void* _throw_fn(void*,const char*,unsigned);
void* _rethrow(void*);
#define _throw_null() (_throw_null_fn(__FILE__,__LINE__))
#define _throw_arraybounds() (_throw_arraybounds_fn(__FILE__,__LINE__))
#define _throw_badalloc() (_throw_badalloc_fn(__FILE__,__LINE__))
#define _throw_match() (_throw_match_fn(__FILE__,__LINE__))
#define _throw(e) (_throw_fn((e),__FILE__,__LINE__))
#endif

struct _xtunion_struct* Cyc_Core_get_exn_thrown();
/* Built-in Exceptions */
struct Cyc_Null_Exception_exn_struct { char *tag; };
struct Cyc_Array_bounds_exn_struct { char *tag; };
struct Cyc_Match_Exception_exn_struct { char *tag; };
struct Cyc_Bad_alloc_exn_struct { char *tag; };
extern char Cyc_Null_Exception[];
extern char Cyc_Array_bounds[];
extern char Cyc_Match_Exception[];
extern char Cyc_Bad_alloc[];

/* Built-in Run-time Checks and company */
#ifdef CYC_ANSI_OUTPUT
#define _INLINE  
#else
#define _INLINE inline
#endif

#ifdef NO_CYC_NULL_CHECKS
#define _check_null(ptr) (ptr)
#else
#define _check_null(ptr) \
  ({ void*_cks_null = (void*)(ptr); \
     if (!_cks_null) _throw_null(); \
     _cks_null; })
#endif

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index)\
   (((char*)ptr) + (elt_sz)*(index))
#ifdef NO_CYC_NULL_CHECKS
#define _check_known_subscript_null _check_known_subscript_notnull
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr);\
  int _index = (index);\
  if (!_cks_ptr) _throw_null(); \
  _cks_ptr + (elt_sz)*_index; })
#endif
#define _zero_arr_plus_fn(orig_x,orig_sz,orig_i,f,l) ((orig_x)+(orig_i))
#define _zero_arr_plus_char_fn _zero_arr_plus_fn
#define _zero_arr_plus_short_fn _zero_arr_plus_fn
#define _zero_arr_plus_int_fn _zero_arr_plus_fn
#define _zero_arr_plus_float_fn _zero_arr_plus_fn
#define _zero_arr_plus_double_fn _zero_arr_plus_fn
#define _zero_arr_plus_longdouble_fn _zero_arr_plus_fn
#define _zero_arr_plus_voidstar_fn _zero_arr_plus_fn
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (!_cks_ptr) _throw_null(); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })

/* _zero_arr_plus_*_fn(x,sz,i,filename,lineno) adds i to zero-terminated ptr
   x that has at least sz elements */
char* _zero_arr_plus_char_fn(char*,unsigned,int,const char*,unsigned);
short* _zero_arr_plus_short_fn(short*,unsigned,int,const char*,unsigned);
int* _zero_arr_plus_int_fn(int*,unsigned,int,const char*,unsigned);
float* _zero_arr_plus_float_fn(float*,unsigned,int,const char*,unsigned);
double* _zero_arr_plus_double_fn(double*,unsigned,int,const char*,unsigned);
long double* _zero_arr_plus_longdouble_fn(long double*,unsigned,int,const char*, unsigned);
void** _zero_arr_plus_voidstar_fn(void**,unsigned,int,const char*,unsigned);
#endif

/* _get_zero_arr_size_*(x,sz) returns the number of elements in a
   zero-terminated array that is NULL or has at least sz elements */
int _get_zero_arr_size_char(const char*,unsigned);
int _get_zero_arr_size_short(const short*,unsigned);
int _get_zero_arr_size_int(const int*,unsigned);
int _get_zero_arr_size_float(const float*,unsigned);
int _get_zero_arr_size_double(const double*,unsigned);
int _get_zero_arr_size_longdouble(const long double*,unsigned);
int _get_zero_arr_size_voidstar(const void**,unsigned);

/* _zero_arr_inplace_plus_*_fn(x,i,filename,lineno) sets
   zero-terminated pointer *x to *x + i */
char* _zero_arr_inplace_plus_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_short_fn(short**,int,const char*,unsigned);
int* _zero_arr_inplace_plus_int(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_longdouble_fn(long double**,int,const char*,unsigned);
void** _zero_arr_inplace_plus_voidstar_fn(void***,int,const char*,unsigned);
/* like the previous functions, but does post-addition (as in e++) */
char* _zero_arr_inplace_plus_post_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_post_short_fn(short**x,int,const char*,unsigned);
int* _zero_arr_inplace_plus_post_int_fn(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_post_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_post_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_post_longdouble_fn(long double**,int,const char *,unsigned);
void** _zero_arr_inplace_plus_post_voidstar_fn(void***,int,const char*,unsigned);
#define _zero_arr_plus_char(x,s,i) \
  (_zero_arr_plus_char_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_short(x,s,i) \
  (_zero_arr_plus_short_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_int(x,s,i) \
  (_zero_arr_plus_int_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_float(x,s,i) \
  (_zero_arr_plus_float_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_double(x,s,i) \
  (_zero_arr_plus_double_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_longdouble(x,s,i) \
  (_zero_arr_plus_longdouble_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_voidstar(x,s,i) \
  (_zero_arr_plus_voidstar_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_inplace_plus_char(x,i) \
  _zero_arr_inplace_plus_char_fn((char **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_short(x,i) \
  _zero_arr_inplace_plus_short_fn((short **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_int(x,i) \
  _zero_arr_inplace_plus_int_fn((int **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_float(x,i) \
  _zero_arr_inplace_plus_float_fn((float **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_double(x,i) \
  _zero_arr_inplace_plus_double_fn((double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_longdouble(x,i) \
  _zero_arr_inplace_plus_longdouble_fn((long double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_voidstar(x,i) \
  _zero_arr_inplace_plus_voidstar_fn((void ***)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_char(x,i) \
  _zero_arr_inplace_plus_post_char_fn((char **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_short(x,i) \
  _zero_arr_inplace_plus_post_short_fn((short **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_int(x,i) \
  _zero_arr_inplace_plus_post_int_fn((int **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_float(x,i) \
  _zero_arr_inplace_plus_post_float_fn((float **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_double(x,i) \
  _zero_arr_inplace_plus_post_double_fn((double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_longdouble(x,i) \
  _zero_arr_inplace_plus_post_longdouble_fn((long double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_voidstar(x,i) \
  _zero_arr_inplace_plus_post_voidstar_fn((void***)(x),(i),__FILE__,__LINE__)

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_fat_subscript(arr,elt_sz,index) ((arr).curr + (elt_sz) * (index))
#define _untag_fat_ptr(arr,elt_sz,num_elts) ((arr).curr)
#else
#define _check_fat_subscript(arr,elt_sz,index) ({ \
  struct _fat_ptr _cus_arr = (arr); \
  unsigned char *_cus_ans = _cus_arr.curr + (elt_sz) * (index); \
  /* JGM: not needed! if (!_cus_arr.base) _throw_null();*/ \
  if (_cus_ans < _cus_arr.base || _cus_ans >= _cus_arr.last_plus_one) \
    _throw_arraybounds(); \
  _cus_ans; })
#define _untag_fat_ptr(arr,elt_sz,num_elts) ({ \
  struct _fat_ptr _arr = (arr); \
  unsigned char *_curr = _arr.curr; \
  if ((_curr < _arr.base || _curr + (elt_sz) * (num_elts) > _arr.last_plus_one) &&\
      _curr != (unsigned char *)0) \
    _throw_arraybounds(); \
  _curr; })
#endif

#define _tag_fat(tcurr,elt_sz,num_elts) ({ \
  struct _fat_ptr _ans; \
  unsigned _num_elts = (num_elts);\
  _ans.base = _ans.curr = (void*)(tcurr); \
  /* JGM: if we're tagging NULL, ignore num_elts */ \
  _ans.last_plus_one = _ans.base ? (_ans.base + (elt_sz) * _num_elts) : 0; \
  _ans; })

#define _get_fat_size(arr,elt_sz) \
  ({struct _fat_ptr _arr = (arr); \
    unsigned char *_arr_curr=_arr.curr; \
    unsigned char *_arr_last=_arr.last_plus_one; \
    (_arr_curr < _arr.base || _arr_curr >= _arr_last) ? 0 : \
    ((_arr_last - _arr_curr) / (elt_sz));})

#define _fat_ptr_plus(arr,elt_sz,change) ({ \
  struct _fat_ptr _ans = (arr); \
  int _change = (change);\
  _ans.curr += (elt_sz) * _change;\
  _ans; })
#define _fat_ptr_inplace_plus(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  _arr_ptr->curr += (elt_sz) * (change);\
  *_arr_ptr; })
#define _fat_ptr_inplace_plus_post(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  struct _fat_ptr _ans = *_arr_ptr; \
  _arr_ptr->curr += (elt_sz) * (change);\
  _ans; })

//Not a macro since initialization order matters. Defined in runtime_zeroterm.c.
struct _fat_ptr _fat_ptr_decrease_size(struct _fat_ptr,unsigned sz,unsigned numelts);

/* Allocation */
void* GC_malloc(int);
void* GC_malloc_atomic(int);
void* GC_calloc(unsigned,unsigned);
void* GC_calloc_atomic(unsigned,unsigned);
// bound the allocation size to be < MAX_ALLOC_SIZE. See macros below for usage.
#define MAX_MALLOC_SIZE (1 << 28)
void* _bounded_GC_malloc(int,const char*,int);
void* _bounded_GC_malloc_atomic(int,const char*,int);
void* _bounded_GC_calloc(unsigned,unsigned,const char*,int);
void* _bounded_GC_calloc_atomic(unsigned,unsigned,const char*,int);
/* these macros are overridden below ifdef CYC_REGION_PROFILE */
#ifndef CYC_REGION_PROFILE
#define _cycalloc(n) _bounded_GC_malloc(n,__FILE__,__LINE__)
#define _cycalloc_atomic(n) _bounded_GC_malloc_atomic(n,__FILE__,__LINE__)
#define _cyccalloc(n,s) _bounded_GC_calloc(n,s,__FILE__,__LINE__)
#define _cyccalloc_atomic(n,s) _bounded_GC_calloc_atomic(n,s,__FILE__,__LINE__)
#endif

static _INLINE unsigned int _check_times(unsigned x, unsigned y) {
  unsigned long long whole_ans = 
    ((unsigned long long) x)*((unsigned long long)y);
  unsigned word_ans = (unsigned)whole_ans;
  if(word_ans < whole_ans || word_ans > MAX_MALLOC_SIZE)
    _throw_badalloc();
  return word_ans;
}

#define _CYC_MAX_REGION_CONST 2
#define _CYC_MIN_ALIGNMENT (sizeof(double))

#ifdef CYC_REGION_PROFILE
extern int rgn_total_bytes;
#endif

static _INLINE void *_fast_region_malloc(struct _RegionHandle *r, unsigned orig_s) {  
  if (r > (struct _RegionHandle *)_CYC_MAX_REGION_CONST && r->curr != 0) { 
#ifdef CYC_NOALIGN
    unsigned s =  orig_s;
#else
    unsigned s =  (orig_s + _CYC_MIN_ALIGNMENT - 1) & (~(_CYC_MIN_ALIGNMENT -1)); 
#endif
    char *result; 
    result = r->offset; 
    if (s <= (r->last_plus_one - result)) {
      r->offset = result + s; 
#ifdef CYC_REGION_PROFILE
    r->curr->free_bytes = r->curr->free_bytes - s;
    rgn_total_bytes += s;
#endif
      return result;
    }
  } 
  return _region_malloc(r,orig_s); 
}

#ifdef CYC_REGION_PROFILE
/* see macros below for usage. defined in runtime_memory.c */
void* _profile_GC_malloc(int,const char*,const char*,int);
void* _profile_GC_malloc_atomic(int,const char*,const char*,int);
void* _profile_GC_calloc(unsigned,unsigned,const char*,const char*,int);
void* _profile_GC_calloc_atomic(unsigned,unsigned,const char*,const char*,int);
void* _profile_region_malloc(struct _RegionHandle*,unsigned,const char*,const char*,int);
void* _profile_region_calloc(struct _RegionHandle*,unsigned,unsigned,const char *,const char*,int);
struct _RegionHandle _profile_new_region(const char*,const char*,const char*,int);
void _profile_free_region(struct _RegionHandle*,const char*,const char*,int);
#ifndef RUNTIME_CYC
#define _new_region(n) _profile_new_region(n,__FILE__,__FUNCTION__,__LINE__)
#define _free_region(r) _profile_free_region(r,__FILE__,__FUNCTION__,__LINE__)
#define _region_malloc(rh,n) _profile_region_malloc(rh,n,__FILE__,__FUNCTION__,__LINE__)
#define _region_calloc(rh,n,t) _profile_region_calloc(rh,n,t,__FILE__,__FUNCTION__,__LINE__)
#  endif
#define _cycalloc(n) _profile_GC_malloc(n,__FILE__,__FUNCTION__,__LINE__)
#define _cycalloc_atomic(n) _profile_GC_malloc_atomic(n,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc(n,s) _profile_GC_calloc(n,s,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc_atomic(n,s) _profile_GC_calloc_atomic(n,s,__FILE__,__FUNCTION__,__LINE__)
#endif
#endif
 struct Cyc_Core_Opt{void*v;};extern char Cyc_Core_Invalid_argument[17U];struct Cyc_Core_Invalid_argument_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Failure[8U];struct Cyc_Core_Failure_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Impossible[11U];struct Cyc_Core_Impossible_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Not_found[10U];struct Cyc_Core_Not_found_exn_struct{char*tag;};extern char Cyc_Core_Unreachable[12U];struct Cyc_Core_Unreachable_exn_struct{char*tag;struct _fat_ptr f1;};
# 168 "core.h"
extern struct _RegionHandle*Cyc_Core_unique_region;struct Cyc_Core_DynamicRegion;struct Cyc_Core_NewDynamicRegion{struct Cyc_Core_DynamicRegion*key;};struct Cyc_Core_ThinRes{void*arr;unsigned nelts;};struct Cyc_List_List{void*hd;struct Cyc_List_List*tl;};extern char Cyc_List_List_mismatch[14U];struct Cyc_List_List_mismatch_exn_struct{char*tag;};
# 172 "list.h"
struct Cyc_List_List*Cyc_List_rev(struct Cyc_List_List*x);
# 178
struct Cyc_List_List*Cyc_List_imp_rev(struct Cyc_List_List*x);
# 184
struct Cyc_List_List*Cyc_List_append(struct Cyc_List_List*x,struct Cyc_List_List*y);extern char Cyc_List_Nth[4U];struct Cyc_List_Nth_exn_struct{char*tag;};
# 322
int Cyc_List_mem(int(*compare)(void*,void*),struct Cyc_List_List*l,void*x);struct Cyc___cycFILE;struct Cyc_String_pa_PrintArg_struct{int tag;struct _fat_ptr f1;};struct Cyc_Int_pa_PrintArg_struct{int tag;unsigned long f1;};struct Cyc_Double_pa_PrintArg_struct{int tag;double f1;};struct Cyc_LongDouble_pa_PrintArg_struct{int tag;long double f1;};struct Cyc_ShortPtr_pa_PrintArg_struct{int tag;short*f1;};struct Cyc_IntPtr_pa_PrintArg_struct{int tag;unsigned long*f1;};struct Cyc_ShortPtr_sa_ScanfArg_struct{int tag;short*f1;};struct Cyc_UShortPtr_sa_ScanfArg_struct{int tag;unsigned short*f1;};struct Cyc_IntPtr_sa_ScanfArg_struct{int tag;int*f1;};struct Cyc_UIntPtr_sa_ScanfArg_struct{int tag;unsigned*f1;};struct Cyc_StringPtr_sa_ScanfArg_struct{int tag;struct _fat_ptr f1;};struct Cyc_DoublePtr_sa_ScanfArg_struct{int tag;double*f1;};struct Cyc_FloatPtr_sa_ScanfArg_struct{int tag;float*f1;};struct Cyc_CharPtr_sa_ScanfArg_struct{int tag;struct _fat_ptr f1;};extern char Cyc_FileCloseError[15U];struct Cyc_FileCloseError_exn_struct{char*tag;};extern char Cyc_FileOpenError[14U];struct Cyc_FileOpenError_exn_struct{char*tag;struct _fat_ptr f1;};
# 50 "string.h"
int Cyc_strptrcmp(struct _fat_ptr*s1,struct _fat_ptr*s2);
# 66
struct _fat_ptr Cyc_str_sepstr(struct Cyc_List_List*,struct _fat_ptr);struct Cyc_Iter_Iter{void*env;int(*next)(void*env,void*dest);};struct Cyc_Dict_T;struct Cyc_Dict_Dict{int(*rel)(void*,void*);struct _RegionHandle*r;const struct Cyc_Dict_T*t;};extern char Cyc_Dict_Present[8U];struct Cyc_Dict_Present_exn_struct{char*tag;};extern char Cyc_Dict_Absent[7U];struct Cyc_Dict_Absent_exn_struct{char*tag;};
# 62 "dict.h"
struct Cyc_Dict_Dict Cyc_Dict_empty(int(*cmp)(void*,void*));
# 83
int Cyc_Dict_member(struct Cyc_Dict_Dict d,void*k);
# 87
struct Cyc_Dict_Dict Cyc_Dict_insert(struct Cyc_Dict_Dict d,void*k,void*v);
# 101
struct Cyc_Dict_Dict Cyc_Dict_singleton(int(*cmp)(void*,void*),void*k,void*v);
# 110
void*Cyc_Dict_lookup(struct Cyc_Dict_Dict d,void*k);
# 149 "dict.h"
void Cyc_Dict_iter_c(void(*f)(void*,void*,void*),void*env,struct Cyc_Dict_Dict d);
# 257
struct Cyc_Dict_Dict Cyc_Dict_difference(struct Cyc_Dict_Dict d1,struct Cyc_Dict_Dict d2);struct Cyc_Set_Set;extern char Cyc_Set_Absent[7U];struct Cyc_Set_Absent_exn_struct{char*tag;};struct Cyc_Position_Error;struct Cyc_Relations_Reln;struct _union_Nmspace_Rel_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_Abs_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_C_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_Loc_n{int tag;int val;};union Cyc_Absyn_Nmspace{struct _union_Nmspace_Rel_n Rel_n;struct _union_Nmspace_Abs_n Abs_n;struct _union_Nmspace_C_n C_n;struct _union_Nmspace_Loc_n Loc_n;};
# 95 "absyn.h"
extern union Cyc_Absyn_Nmspace Cyc_Absyn_Loc_n;
union Cyc_Absyn_Nmspace Cyc_Absyn_Rel_n(struct Cyc_List_List*);
# 98
union Cyc_Absyn_Nmspace Cyc_Absyn_Abs_n(struct Cyc_List_List*ns,int C_scope);struct _tuple0{union Cyc_Absyn_Nmspace f1;struct _fat_ptr*f2;};
# 161
enum Cyc_Absyn_Scope{Cyc_Absyn_Static =0U,Cyc_Absyn_Abstract =1U,Cyc_Absyn_Public =2U,Cyc_Absyn_Extern =3U,Cyc_Absyn_ExternC =4U,Cyc_Absyn_Register =5U};struct Cyc_Absyn_Tqual{int print_const: 1;int q_volatile: 1;int q_restrict: 1;int real_const: 1;unsigned loc;};
# 182
enum Cyc_Absyn_Size_of{Cyc_Absyn_Char_sz =0U,Cyc_Absyn_Short_sz =1U,Cyc_Absyn_Int_sz =2U,Cyc_Absyn_Long_sz =3U,Cyc_Absyn_LongLong_sz =4U};
enum Cyc_Absyn_Sign{Cyc_Absyn_Signed =0U,Cyc_Absyn_Unsigned =1U,Cyc_Absyn_None =2U};
enum Cyc_Absyn_AggrKind{Cyc_Absyn_StructA =0U,Cyc_Absyn_UnionA =1U};
# 187
enum Cyc_Absyn_AliasQual{Cyc_Absyn_Aliasable =0U,Cyc_Absyn_Unique =1U,Cyc_Absyn_Top =2U};
# 192
enum Cyc_Absyn_KindQual{Cyc_Absyn_AnyKind =0U,Cyc_Absyn_MemKind =1U,Cyc_Absyn_BoxKind =2U,Cyc_Absyn_RgnKind =3U,Cyc_Absyn_EffKind =4U,Cyc_Absyn_IntKind =5U,Cyc_Absyn_BoolKind =6U,Cyc_Absyn_PtrBndKind =7U};struct Cyc_Absyn_Kind{enum Cyc_Absyn_KindQual kind;enum Cyc_Absyn_AliasQual aliasqual;};struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct{int tag;struct Cyc_Absyn_Kind*f1;};struct Cyc_Absyn_Unknown_kb_Absyn_KindBound_struct{int tag;struct Cyc_Core_Opt*f1;};struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct{int tag;struct Cyc_Core_Opt*f1;struct Cyc_Absyn_Kind*f2;};struct Cyc_Absyn_Tvar{struct _fat_ptr*name;int identity;void*kind;};struct Cyc_Absyn_PtrLoc{unsigned ptr_loc;unsigned rgn_loc;unsigned zt_loc;};struct Cyc_Absyn_PtrAtts{void*rgn;void*nullable;void*bounds;void*zero_term;struct Cyc_Absyn_PtrLoc*ptrloc;};struct Cyc_Absyn_PtrInfo{void*elt_type;struct Cyc_Absyn_Tqual elt_tq;struct Cyc_Absyn_PtrAtts ptr_atts;};struct Cyc_Absyn_VarargInfo{struct _fat_ptr*name;struct Cyc_Absyn_Tqual tq;void*type;int inject;};struct Cyc_Absyn_Star_Absyn_Cap_struct{int tag;};struct Cyc_Absyn_Bot_Absyn_Cap_struct{int tag;};struct Cyc_Absyn_Nat_Absyn_Cap_struct{int tag;struct Cyc_Absyn_Exp*f1;int f2;};struct Cyc_Absyn_RgnEffect{void*name;void*rgn;void*lock;void*unadopt;void*parent;};struct Cyc_Absyn_FnInfo{struct Cyc_List_List*tvars;void*effect;struct Cyc_Absyn_Tqual ret_tqual;void*ret_type;struct Cyc_List_List*args;int c_varargs;struct Cyc_Absyn_VarargInfo*cyc_varargs;struct Cyc_List_List*rgn_po;struct Cyc_List_List*attributes;struct Cyc_Absyn_Exp*requires_clause;struct Cyc_List_List*requires_relns;struct Cyc_Absyn_Exp*ensures_clause;struct Cyc_List_List*ensures_relns;struct Cyc_List_List*ieffect;struct Cyc_List_List*oeffect;};struct Cyc_Absyn_UnknownDatatypeInfo{struct _tuple0*name;int is_extensible;};struct _union_DatatypeInfo_UnknownDatatype{int tag;struct Cyc_Absyn_UnknownDatatypeInfo val;};struct _union_DatatypeInfo_KnownDatatype{int tag;struct Cyc_Absyn_Datatypedecl**val;};union Cyc_Absyn_DatatypeInfo{struct _union_DatatypeInfo_UnknownDatatype UnknownDatatype;struct _union_DatatypeInfo_KnownDatatype KnownDatatype;};
# 311
union Cyc_Absyn_DatatypeInfo Cyc_Absyn_UnknownDatatype(struct Cyc_Absyn_UnknownDatatypeInfo);struct Cyc_Absyn_UnknownDatatypeFieldInfo{struct _tuple0*datatype_name;struct _tuple0*field_name;int is_extensible;};struct _union_DatatypeFieldInfo_UnknownDatatypefield{int tag;struct Cyc_Absyn_UnknownDatatypeFieldInfo val;};struct _tuple1{struct Cyc_Absyn_Datatypedecl*f1;struct Cyc_Absyn_Datatypefield*f2;};struct _union_DatatypeFieldInfo_KnownDatatypefield{int tag;struct _tuple1 val;};union Cyc_Absyn_DatatypeFieldInfo{struct _union_DatatypeFieldInfo_UnknownDatatypefield UnknownDatatypefield;struct _union_DatatypeFieldInfo_KnownDatatypefield KnownDatatypefield;};
# 324
union Cyc_Absyn_DatatypeFieldInfo Cyc_Absyn_UnknownDatatypefield(struct Cyc_Absyn_UnknownDatatypeFieldInfo);struct _tuple2{enum Cyc_Absyn_AggrKind f1;struct _tuple0*f2;struct Cyc_Core_Opt*f3;};struct _union_AggrInfo_UnknownAggr{int tag;struct _tuple2 val;};struct _union_AggrInfo_KnownAggr{int tag;struct Cyc_Absyn_Aggrdecl**val;};union Cyc_Absyn_AggrInfo{struct _union_AggrInfo_UnknownAggr UnknownAggr;struct _union_AggrInfo_KnownAggr KnownAggr;};
# 331
union Cyc_Absyn_AggrInfo Cyc_Absyn_UnknownAggr(enum Cyc_Absyn_AggrKind,struct _tuple0*,struct Cyc_Core_Opt*);
union Cyc_Absyn_AggrInfo Cyc_Absyn_KnownAggr(struct Cyc_Absyn_Aggrdecl**);struct Cyc_Absyn_ArrayInfo{void*elt_type;struct Cyc_Absyn_Tqual tq;struct Cyc_Absyn_Exp*num_elts;void*zero_term;unsigned zt_loc;};struct Cyc_Absyn_Aggr_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Aggrdecl*f1;};struct Cyc_Absyn_Enum_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;};struct Cyc_Absyn_Datatype_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;};struct Cyc_Absyn_TypeDecl{void*r;unsigned loc;};struct Cyc_Absyn_VoidCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_IntCon_Absyn_TyCon_struct{int tag;enum Cyc_Absyn_Sign f1;enum Cyc_Absyn_Size_of f2;};struct Cyc_Absyn_FloatCon_Absyn_TyCon_struct{int tag;int f1;};struct Cyc_Absyn_RgnHandleCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_TagCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_HeapCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_UniqueCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_RefCntCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_AccessCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_JoinCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_RgnsCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_TrueCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_FalseCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_ThinCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_FatCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_EnumCon_Absyn_TyCon_struct{int tag;struct _tuple0*f1;struct Cyc_Absyn_Enumdecl*f2;};struct Cyc_Absyn_AnonEnumCon_Absyn_TyCon_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_BuiltinCon_Absyn_TyCon_struct{int tag;struct _fat_ptr f1;struct Cyc_Absyn_Kind*f2;};struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_DatatypeInfo f1;};struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_DatatypeFieldInfo f1;};struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_AggrInfo f1;};struct Cyc_Absyn_AppType_Absyn_Type_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Evar_Absyn_Type_struct{int tag;struct Cyc_Core_Opt*f1;void*f2;int f3;struct Cyc_Core_Opt*f4;};struct Cyc_Absyn_VarType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Tvar*f1;};struct Cyc_Absyn_PointerType_Absyn_Type_struct{int tag;struct Cyc_Absyn_PtrInfo f1;};struct Cyc_Absyn_ArrayType_Absyn_Type_struct{int tag;struct Cyc_Absyn_ArrayInfo f1;};struct Cyc_Absyn_FnType_Absyn_Type_struct{int tag;struct Cyc_Absyn_FnInfo f1;};struct Cyc_Absyn_TupleType_Absyn_Type_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct{int tag;enum Cyc_Absyn_AggrKind f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_TypedefType_Absyn_Type_struct{int tag;struct _tuple0*f1;struct Cyc_List_List*f2;struct Cyc_Absyn_Typedefdecl*f3;void*f4;};struct Cyc_Absyn_ValueofType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct{int tag;struct Cyc_Absyn_TypeDecl*f1;void**f2;};struct Cyc_Absyn_TypeofType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_NoTypes_Absyn_Funcparams_struct{int tag;struct Cyc_List_List*f1;unsigned f2;};struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct{int tag;struct Cyc_List_List*f1;int f2;struct Cyc_Absyn_VarargInfo*f3;void*f4;struct Cyc_List_List*f5;struct Cyc_Absyn_Exp*f6;struct Cyc_Absyn_Exp*f7;struct Cyc_List_List*f8;struct Cyc_List_List*f9;};
# 437 "absyn.h"
enum Cyc_Absyn_Format_Type{Cyc_Absyn_Printf_ft =0U,Cyc_Absyn_Scanf_ft =1U};struct Cyc_Absyn_Regparm_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Stdcall_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Cdecl_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Fastcall_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Noreturn_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Const_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Aligned_att_Absyn_Attribute_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Packed_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Section_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Nocommon_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Shared_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Unused_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Weak_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Dllimport_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Dllexport_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_instrument_function_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Constructor_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Destructor_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_check_memory_usage_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Format_att_Absyn_Attribute_struct{int tag;enum Cyc_Absyn_Format_Type f1;int f2;int f3;};struct Cyc_Absyn_Initializes_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Noliveunique_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Consume_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Pure_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Mode_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Alias_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Always_inline_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_throw_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Non_null_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Deprecated_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Carray_mod_Absyn_Type_modifier_struct{int tag;void*f1;unsigned f2;};struct Cyc_Absyn_ConstArray_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_Absyn_Exp*f1;void*f2;unsigned f3;};struct Cyc_Absyn_Pointer_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_Absyn_PtrAtts f1;struct Cyc_Absyn_Tqual f2;};struct Cyc_Absyn_Function_mod_Absyn_Type_modifier_struct{int tag;void*f1;};struct Cyc_Absyn_TypeParams_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_List_List*f1;unsigned f2;int f3;};struct Cyc_Absyn_Attributes_mod_Absyn_Type_modifier_struct{int tag;unsigned f1;struct Cyc_List_List*f2;};struct _union_Cnst_Null_c{int tag;int val;};struct _tuple3{enum Cyc_Absyn_Sign f1;char f2;};struct _union_Cnst_Char_c{int tag;struct _tuple3 val;};struct _union_Cnst_Wchar_c{int tag;struct _fat_ptr val;};struct _tuple4{enum Cyc_Absyn_Sign f1;short f2;};struct _union_Cnst_Short_c{int tag;struct _tuple4 val;};struct _tuple5{enum Cyc_Absyn_Sign f1;int f2;};struct _union_Cnst_Int_c{int tag;struct _tuple5 val;};struct _tuple6{enum Cyc_Absyn_Sign f1;long long f2;};struct _union_Cnst_LongLong_c{int tag;struct _tuple6 val;};struct _tuple7{struct _fat_ptr f1;int f2;};struct _union_Cnst_Float_c{int tag;struct _tuple7 val;};struct _union_Cnst_String_c{int tag;struct _fat_ptr val;};struct _union_Cnst_Wstring_c{int tag;struct _fat_ptr val;};union Cyc_Absyn_Cnst{struct _union_Cnst_Null_c Null_c;struct _union_Cnst_Char_c Char_c;struct _union_Cnst_Wchar_c Wchar_c;struct _union_Cnst_Short_c Short_c;struct _union_Cnst_Int_c Int_c;struct _union_Cnst_LongLong_c LongLong_c;struct _union_Cnst_Float_c Float_c;struct _union_Cnst_String_c String_c;struct _union_Cnst_Wstring_c Wstring_c;};
# 532
enum Cyc_Absyn_Primop{Cyc_Absyn_Plus =0U,Cyc_Absyn_Times =1U,Cyc_Absyn_Minus =2U,Cyc_Absyn_Div =3U,Cyc_Absyn_Mod =4U,Cyc_Absyn_Eq =5U,Cyc_Absyn_Neq =6U,Cyc_Absyn_Gt =7U,Cyc_Absyn_Lt =8U,Cyc_Absyn_Gte =9U,Cyc_Absyn_Lte =10U,Cyc_Absyn_Not =11U,Cyc_Absyn_Bitnot =12U,Cyc_Absyn_Bitand =13U,Cyc_Absyn_Bitor =14U,Cyc_Absyn_Bitxor =15U,Cyc_Absyn_Bitlshift =16U,Cyc_Absyn_Bitlrshift =17U,Cyc_Absyn_Numelts =18U};
# 539
enum Cyc_Absyn_Incrementor{Cyc_Absyn_PreInc =0U,Cyc_Absyn_PostInc =1U,Cyc_Absyn_PreDec =2U,Cyc_Absyn_PostDec =3U};struct Cyc_Absyn_VarargCallInfo{int num_varargs;struct Cyc_List_List*injectors;struct Cyc_Absyn_VarargInfo*vai;};struct Cyc_Absyn_StructField_Absyn_OffsetofField_struct{int tag;struct _fat_ptr*f1;};struct Cyc_Absyn_TupleIndex_Absyn_OffsetofField_struct{int tag;unsigned f1;};
# 557
enum Cyc_Absyn_Coercion{Cyc_Absyn_Unknown_coercion =0U,Cyc_Absyn_No_coercion =1U,Cyc_Absyn_Null_to_NonNull =2U,Cyc_Absyn_Other_coercion =3U};struct Cyc_Absyn_MallocInfo{int is_calloc;struct Cyc_Absyn_Exp*rgn;void**elt_type;struct Cyc_Absyn_Exp*num_elts;int fat_result;int inline_call;};struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct{int tag;union Cyc_Absyn_Cnst f1;};struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Pragma_e_Absyn_Raw_exp_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct{int tag;enum Cyc_Absyn_Primop f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_AssignOp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Core_Opt*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_Increment_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;enum Cyc_Absyn_Incrementor f2;};struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;struct Cyc_Absyn_VarargCallInfo*f3;int f4;};struct Cyc_Absyn_Throw_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;int f2;};struct Cyc_Absyn_NoInstantiate_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Instantiate_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_Absyn_Exp*f2;int f3;enum Cyc_Absyn_Coercion f4;};struct Cyc_Absyn_Address_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_New_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Sizeoftype_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Sizeofexp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Offsetof_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Deref_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;int f3;int f4;};struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;int f3;int f4;};struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Tuple_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;};struct _tuple8{struct _fat_ptr*f1;struct Cyc_Absyn_Tqual f2;void*f3;};struct Cyc_Absyn_CompoundLit_e_Absyn_Raw_exp_struct{int tag;struct _tuple8*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Array_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Exp*f2;struct Cyc_Absyn_Exp*f3;int f4;};struct Cyc_Absyn_ComprehensionNoinit_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;void*f2;int f3;};struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct{int tag;struct _tuple0*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;struct Cyc_Absyn_Aggrdecl*f4;};struct Cyc_Absyn_AnonStruct_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Datatype_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;struct Cyc_Absyn_Datatypedecl*f2;struct Cyc_Absyn_Datatypefield*f3;};struct Cyc_Absyn_Enum_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_AnonEnum_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_Spawn_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_MallocInfo f1;};struct Cyc_Absyn_Swap_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_UnresolvedMem_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Core_Opt*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_StmtExp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Stmt*f1;};struct Cyc_Absyn_Tagcheck_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;};struct Cyc_Absyn_Valueof_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Asm_e_Absyn_Raw_exp_struct{int tag;int f1;struct _fat_ptr f2;struct Cyc_List_List*f3;struct Cyc_List_List*f4;struct Cyc_List_List*f5;};struct Cyc_Absyn_Extension_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Exp{void*topt;void*r;unsigned loc;void*annot;};struct Cyc_Absyn_Skip_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Exp_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Seq_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Return_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_IfThenElse_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Stmt*f2;struct Cyc_Absyn_Stmt*f3;};struct _tuple9{struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_While_s_Absyn_Raw_stmt_struct{int tag;struct _tuple9 f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Break_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Continue_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Goto_s_Absyn_Raw_stmt_struct{int tag;struct _fat_ptr*f1;};struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _tuple9 f2;struct _tuple9 f3;struct Cyc_Absyn_Stmt*f4;};struct Cyc_Absyn_Switch_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;void*f3;};struct Cyc_Absyn_Fallthru_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_List_List*f1;struct Cyc_Absyn_Switch_clause**f2;};struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Decl*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Label_s_Absyn_Raw_stmt_struct{int tag;struct _fat_ptr*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Do_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct _tuple9 f2;};struct Cyc_Absyn_TryCatch_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct Cyc_List_List*f2;void*f3;};struct Cyc_Absyn_Stmt{void*r;unsigned loc;void*annot;};struct Cyc_Absyn_Wild_p_Absyn_Raw_pat_struct{int tag;};struct Cyc_Absyn_Var_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Pat*f2;};struct Cyc_Absyn_AliasVar_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;};struct Cyc_Absyn_Reference_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Pat*f2;};struct Cyc_Absyn_TagInt_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;};struct Cyc_Absyn_Tuple_p_Absyn_Raw_pat_struct{int tag;struct Cyc_List_List*f1;int f2;};struct Cyc_Absyn_Pointer_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Pat*f1;};struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct{int tag;union Cyc_Absyn_AggrInfo*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;int f4;};struct Cyc_Absyn_Datatype_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;struct Cyc_Absyn_Datatypefield*f2;struct Cyc_List_List*f3;int f4;};struct Cyc_Absyn_Null_p_Absyn_Raw_pat_struct{int tag;};struct Cyc_Absyn_Int_p_Absyn_Raw_pat_struct{int tag;enum Cyc_Absyn_Sign f1;int f2;};struct Cyc_Absyn_Char_p_Absyn_Raw_pat_struct{int tag;char f1;};struct Cyc_Absyn_Float_p_Absyn_Raw_pat_struct{int tag;struct _fat_ptr f1;int f2;};struct Cyc_Absyn_Enum_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_AnonEnum_p_Absyn_Raw_pat_struct{int tag;void*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_UnknownId_p_Absyn_Raw_pat_struct{int tag;struct _tuple0*f1;};struct Cyc_Absyn_UnknownCall_p_Absyn_Raw_pat_struct{int tag;struct _tuple0*f1;struct Cyc_List_List*f2;int f3;};struct Cyc_Absyn_Exp_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Exp*f1;};
# 725 "absyn.h"
extern struct Cyc_Absyn_Wild_p_Absyn_Raw_pat_struct Cyc_Absyn_Wild_p_val;struct Cyc_Absyn_Pat{void*r;void*topt;unsigned loc;};struct Cyc_Absyn_Switch_clause{struct Cyc_Absyn_Pat*pattern;struct Cyc_Core_Opt*pat_vars;struct Cyc_Absyn_Exp*where_clause;struct Cyc_Absyn_Stmt*body;unsigned loc;};struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct{int tag;struct _tuple0*f1;};struct Cyc_Absyn_Global_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Funname_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Fndecl*f1;};struct Cyc_Absyn_Param_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Local_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Pat_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Vardecl{enum Cyc_Absyn_Scope sc;struct _tuple0*name;unsigned varloc;struct Cyc_Absyn_Tqual tq;void*type;struct Cyc_Absyn_Exp*initializer;void*rgn;struct Cyc_List_List*attributes;int escapes;};struct Cyc_Absyn_Fndecl{enum Cyc_Absyn_Scope sc;int is_inline;struct _tuple0*name;struct Cyc_Absyn_Stmt*body;struct Cyc_Absyn_FnInfo i;void*cached_type;struct Cyc_Core_Opt*param_vardecls;struct Cyc_Absyn_Vardecl*fn_vardecl;};struct Cyc_Absyn_Aggrfield{struct _fat_ptr*name;struct Cyc_Absyn_Tqual tq;void*type;struct Cyc_Absyn_Exp*width;struct Cyc_List_List*attributes;struct Cyc_Absyn_Exp*requires_clause;};struct Cyc_Absyn_AggrdeclImpl{struct Cyc_List_List*exist_vars;struct Cyc_List_List*rgn_po;struct Cyc_List_List*fields;int tagged;};struct Cyc_Absyn_Aggrdecl{enum Cyc_Absyn_AggrKind kind;enum Cyc_Absyn_Scope sc;struct _tuple0*name;struct Cyc_List_List*tvs;struct Cyc_Absyn_AggrdeclImpl*impl;struct Cyc_List_List*attributes;int expected_mem_kind;};struct Cyc_Absyn_Datatypefield{struct _tuple0*name;struct Cyc_List_List*typs;unsigned loc;enum Cyc_Absyn_Scope sc;};struct Cyc_Absyn_Datatypedecl{enum Cyc_Absyn_Scope sc;struct _tuple0*name;struct Cyc_List_List*tvs;struct Cyc_Core_Opt*fields;int is_extensible;};struct Cyc_Absyn_Enumfield{struct _tuple0*name;struct Cyc_Absyn_Exp*tag;unsigned loc;};struct Cyc_Absyn_Enumdecl{enum Cyc_Absyn_Scope sc;struct _tuple0*name;struct Cyc_Core_Opt*fields;};struct Cyc_Absyn_Typedefdecl{struct _tuple0*name;struct Cyc_Absyn_Tqual tq;struct Cyc_List_List*tvs;struct Cyc_Core_Opt*kind;void*defn;struct Cyc_List_List*atts;int extern_c;};struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Fn_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Fndecl*f1;};struct Cyc_Absyn_Let_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Pat*f1;struct Cyc_Core_Opt*f2;struct Cyc_Absyn_Exp*f3;void*f4;};struct Cyc_Absyn_Letv_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_Region_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_Aggr_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Aggrdecl*f1;};struct Cyc_Absyn_Datatype_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;};struct Cyc_Absyn_Enum_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;};struct Cyc_Absyn_Typedef_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Typedefdecl*f1;};struct Cyc_Absyn_Namespace_d_Absyn_Raw_decl_struct{int tag;struct _fat_ptr*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Using_d_Absyn_Raw_decl_struct{int tag;struct _tuple0*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_ExternC_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;};struct _tuple10{unsigned f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_ExternCinclude_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;struct _tuple10*f4;};struct Cyc_Absyn_Porton_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Portoff_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Tempeston_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Tempestoff_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Decl{void*r;unsigned loc;};struct Cyc_Absyn_ArrayElement_Absyn_Designator_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_FieldName_Absyn_Designator_struct{int tag;struct _fat_ptr*f1;};extern char Cyc_Absyn_EmptyAnnot[11U];struct Cyc_Absyn_EmptyAnnot_Absyn_AbsynAnnot_struct{char*tag;};
# 908
int Cyc_Absyn_qvar_cmp(struct _tuple0*,struct _tuple0*);
int Cyc_Absyn_varlist_cmp(struct Cyc_List_List*,struct Cyc_List_List*);
# 945
extern void*Cyc_Absyn_heap_rgn_type;
# 949
extern void*Cyc_Absyn_false_type;
# 951
extern void*Cyc_Absyn_void_type;
# 967
struct Cyc_Absyn_Datatypedecl*Cyc_Absyn_exn_tud();
# 996
void*Cyc_Absyn_fatptr_type(void*t,void*rgn,struct Cyc_Absyn_Tqual,void*zeroterm);
# 1103
struct Cyc_Absyn_Pat*Cyc_Absyn_new_pat(void*,unsigned);
# 1112
struct Cyc_Absyn_Vardecl*Cyc_Absyn_new_vardecl(unsigned varloc,struct _tuple0*,void*,struct Cyc_Absyn_Exp*init);
# 35 "warn.h"
void Cyc_Warn_err(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap);
# 40
void*Cyc_Warn_impos(struct _fat_ptr fmt,struct _fat_ptr ap);struct Cyc_Warn_String_Warn_Warg_struct{int tag;struct _fat_ptr f1;};struct Cyc_Warn_Exp_Warn_Warg_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Warn_Typ_Warn_Warg_struct{int tag;void*f1;};struct Cyc_Warn_Qvar_Warn_Warg_struct{int tag;struct _tuple0*f1;};
# 58
void Cyc_Warn_err2(unsigned,struct _fat_ptr);
# 60
void Cyc_Warn_warn2(unsigned,struct _fat_ptr);
# 29 "binding.h"
extern int Cyc_Binding_warn_override;struct Cyc_Binding_Namespace_Binding_NSDirective_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Binding_Using_Binding_NSDirective_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Binding_NSCtxt{struct Cyc_List_List*curr_ns;struct Cyc_List_List*availables;struct Cyc_Dict_Dict ns_data;};
# 46
struct Cyc_Binding_NSCtxt*Cyc_Binding_mt_nsctxt(void*,void*(*mkdata)(void*));
void Cyc_Binding_enter_ns(struct Cyc_Binding_NSCtxt*,struct _fat_ptr*,void*,void*(*mkdata)(void*));
struct Cyc_List_List*Cyc_Binding_enter_using(unsigned loc,struct Cyc_Binding_NSCtxt*ctxt,struct _tuple0*usename);
void Cyc_Binding_leave_ns(struct Cyc_Binding_NSCtxt*ctxt);
void Cyc_Binding_leave_using(struct Cyc_Binding_NSCtxt*ctxt);
struct Cyc_List_List*Cyc_Binding_resolve_rel_ns(unsigned,struct Cyc_Binding_NSCtxt*,struct Cyc_List_List*);
# 34 "cifc.h"
void Cyc_Cifc_user_overrides(unsigned,struct Cyc_List_List*ds,struct Cyc_List_List*ovrs);char Cyc_Binding_BindingError[13U]="BindingError";struct Cyc_Binding_BindingError_exn_struct{char*tag;};
# 44 "binding.cyc"
int Cyc_Binding_warn_override=0;
# 58
struct Cyc_Binding_NSCtxt*Cyc_Binding_mt_nsctxt(void*env,void*(*mkdata)(void*)){
void*data=mkdata(env);
return({struct Cyc_Binding_NSCtxt*_tmp3=_cycalloc(sizeof(*_tmp3));_tmp3->curr_ns=0,({
struct Cyc_List_List*_tmp2E5=({struct Cyc_List_List*_tmp1=_cycalloc(sizeof(*_tmp1));({void*_tmp2E4=(void*)({struct Cyc_Binding_Namespace_Binding_NSDirective_struct*_tmp0=_cycalloc(sizeof(*_tmp0));_tmp0->tag=0U,_tmp0->f1=0;_tmp0;});_tmp1->hd=_tmp2E4;}),_tmp1->tl=0;_tmp1;});_tmp3->availables=_tmp2E5;}),({
struct Cyc_Dict_Dict _tmp2E3=({struct Cyc_Dict_Dict(*_tmp2E2)(int(*cmp)(struct Cyc_List_List*,struct Cyc_List_List*),struct Cyc_List_List*k,void*v)=({struct Cyc_Dict_Dict(*_tmp2)(int(*cmp)(struct Cyc_List_List*,struct Cyc_List_List*),struct Cyc_List_List*k,void*v)=(struct Cyc_Dict_Dict(*)(int(*cmp)(struct Cyc_List_List*,struct Cyc_List_List*),struct Cyc_List_List*k,void*v))Cyc_Dict_singleton;_tmp2;});_tmp2E2(Cyc_Absyn_varlist_cmp,0,data);});_tmp3->ns_data=_tmp2E3;});_tmp3;});}
# 58
void*Cyc_Binding_get_ns_data(struct Cyc_Binding_NSCtxt*ctxt,union Cyc_Absyn_Nmspace abs_ns){
# 65
union Cyc_Absyn_Nmspace _tmp4=abs_ns;struct Cyc_List_List*_tmp5;struct Cyc_List_List*_tmp6;switch((_tmp4.Abs_n).tag){case 3U: _LL1: _tmp6=(_tmp4.C_n).val;_LL2: {struct Cyc_List_List*vs=_tmp6;
_tmp5=vs;goto _LL4;}case 2U: _LL3: _tmp5=(_tmp4.Abs_n).val;_LL4: {struct Cyc_List_List*vs=_tmp5;
return({void*(*_tmp2E7)(struct Cyc_Dict_Dict d,struct Cyc_List_List*k)=({void*(*_tmp7)(struct Cyc_Dict_Dict d,struct Cyc_List_List*k)=(void*(*)(struct Cyc_Dict_Dict d,struct Cyc_List_List*k))Cyc_Dict_lookup;_tmp7;});struct Cyc_Dict_Dict _tmp2E6=ctxt->ns_data;_tmp2E7(_tmp2E6,vs);});}default: _LL5: _LL6:
({void*_tmp8=0U;({struct _fat_ptr _tmp2E8=({const char*_tmp9="Binding:get_ns_data: relative ns";_tag_fat(_tmp9,sizeof(char),33U);});Cyc_Warn_impos(_tmp2E8,_tag_fat(_tmp8,sizeof(void*),0U));});});}_LL0:;}
# 58
struct Cyc_List_List*Cyc_Binding_resolve_rel_ns(unsigned loc,struct Cyc_Binding_NSCtxt*ctxt,struct Cyc_List_List*rel_ns){
# 85 "binding.cyc"
struct Cyc_List_List*fullname=Cyc_List_append(ctxt->curr_ns,rel_ns);
if(({int(*_tmp2EA)(struct Cyc_Dict_Dict d,struct Cyc_List_List*k)=({int(*_tmpA)(struct Cyc_Dict_Dict d,struct Cyc_List_List*k)=(int(*)(struct Cyc_Dict_Dict d,struct Cyc_List_List*k))Cyc_Dict_member;_tmpA;});struct Cyc_Dict_Dict _tmp2E9=ctxt->ns_data;_tmp2EA(_tmp2E9,fullname);}))
return fullname;
# 86
{
# 88
struct Cyc_List_List*as=ctxt->availables;
# 86
for(0;as != 0;as=as->tl){
# 89
void*_stmttmp0=(void*)as->hd;void*_tmpB=_stmttmp0;struct Cyc_List_List*_tmpC;struct Cyc_List_List*_tmpD;if(((struct Cyc_Binding_Using_Binding_NSDirective_struct*)_tmpB)->tag == 1U){_LL1: _tmpD=((struct Cyc_Binding_Using_Binding_NSDirective_struct*)_tmpB)->f1;_LL2: {struct Cyc_List_List*ns=_tmpD;
_tmpC=ns;goto _LL4;}}else{_LL3: _tmpC=((struct Cyc_Binding_Namespace_Binding_NSDirective_struct*)_tmpB)->f1;_LL4: {struct Cyc_List_List*ns=_tmpC;
# 92
struct Cyc_List_List*fullname=Cyc_List_append(ns,rel_ns);
if(({int(*_tmp2EC)(struct Cyc_Dict_Dict d,struct Cyc_List_List*k)=({int(*_tmpE)(struct Cyc_Dict_Dict d,struct Cyc_List_List*k)=(int(*)(struct Cyc_Dict_Dict d,struct Cyc_List_List*k))Cyc_Dict_member;_tmpE;});struct Cyc_Dict_Dict _tmp2EB=ctxt->ns_data;_tmp2EC(_tmp2EB,fullname);}))
return fullname;
# 93
goto _LL0;}}_LL0:;}}
# 97
({struct Cyc_Warn_String_Warn_Warg_struct _tmp10=({struct Cyc_Warn_String_Warn_Warg_struct _tmp2B5;_tmp2B5.tag=0U,({struct _fat_ptr _tmp2ED=({const char*_tmp15="namespace ";_tag_fat(_tmp15,sizeof(char),11U);});_tmp2B5.f1=_tmp2ED;});_tmp2B5;});struct Cyc_Warn_String_Warn_Warg_struct _tmp11=({struct Cyc_Warn_String_Warn_Warg_struct _tmp2B4;_tmp2B4.tag=0U,({struct _fat_ptr _tmp2EF=(struct _fat_ptr)({struct Cyc_List_List*_tmp2EE=rel_ns;Cyc_str_sepstr(_tmp2EE,({const char*_tmp14="::";_tag_fat(_tmp14,sizeof(char),3U);}));});_tmp2B4.f1=_tmp2EF;});_tmp2B4;});struct Cyc_Warn_String_Warn_Warg_struct _tmp12=({struct Cyc_Warn_String_Warn_Warg_struct _tmp2B3;_tmp2B3.tag=0U,({struct _fat_ptr _tmp2F0=({const char*_tmp13=" not found";_tag_fat(_tmp13,sizeof(char),11U);});_tmp2B3.f1=_tmp2F0;});_tmp2B3;});void*_tmpF[3U];_tmpF[0]=& _tmp10,_tmpF[1]=& _tmp11,_tmpF[2]=& _tmp12;({unsigned _tmp2F1=loc;Cyc_Warn_err2(_tmp2F1,_tag_fat(_tmpF,sizeof(void*),3U));});});
(int)_throw((void*)({struct Cyc_Binding_BindingError_exn_struct*_tmp16=_cycalloc(sizeof(*_tmp16));_tmp16->tag=Cyc_Binding_BindingError;_tmp16;}));}
# 58 "binding.cyc"
void*Cyc_Binding_resolve_lookup(unsigned loc,struct Cyc_Binding_NSCtxt*ctxt,struct _tuple0*qv,void*(*lookup)(void*,struct _fat_ptr*)){
# 103 "binding.cyc"
struct _fat_ptr*_tmp18;union Cyc_Absyn_Nmspace _tmp17;_LL1: _tmp17=qv->f1;_tmp18=qv->f2;_LL2: {union Cyc_Absyn_Nmspace ns=_tmp17;struct _fat_ptr*v=_tmp18;
union Cyc_Absyn_Nmspace _tmp19=ns;struct Cyc_List_List*_tmp1A;struct Cyc_List_List*_tmp1B;struct Cyc_List_List*_tmp1C;switch((_tmp19.Abs_n).tag){case 1U: if((_tmp19.Rel_n).val == 0){_LL4: _LL5:
# 106
{struct _handler_cons _tmp1D;_push_handler(& _tmp1D);{int _tmp1F=0;if(setjmp(_tmp1D.handler))_tmp1F=1;if(!_tmp1F){{void*_tmp21=({void*(*_tmp2F5)(void*,struct _fat_ptr*)=lookup;void*_tmp2F4=({void*(*_tmp2F3)(struct Cyc_Dict_Dict d,struct Cyc_List_List*k)=({void*(*_tmp20)(struct Cyc_Dict_Dict d,struct Cyc_List_List*k)=(void*(*)(struct Cyc_Dict_Dict d,struct Cyc_List_List*k))Cyc_Dict_lookup;_tmp20;});struct Cyc_Dict_Dict _tmp2F2=ctxt->ns_data;_tmp2F3(_tmp2F2,ctxt->curr_ns);});_tmp2F5(_tmp2F4,v);});_npop_handler(0U);return _tmp21;};_pop_handler();}else{void*_tmp1E=(void*)Cyc_Core_get_exn_thrown();void*_tmp22=_tmp1E;void*_tmp23;if(((struct Cyc_Dict_Absent_exn_struct*)_tmp22)->tag == Cyc_Dict_Absent){_LLF: _LL10:
 goto _LLE;}else{_LL11: _tmp23=_tmp22;_LL12: {void*exn=_tmp23;(int)_rethrow(exn);}}_LLE:;}}}
{struct Cyc_List_List*as=ctxt->availables;for(0;as != 0;as=as->tl){
void*_stmttmp1=(void*)as->hd;void*_tmp24=_stmttmp1;struct Cyc_List_List*_tmp25;struct Cyc_List_List*_tmp26;if(((struct Cyc_Binding_Using_Binding_NSDirective_struct*)_tmp24)->tag == 1U){_LL14: _tmp26=((struct Cyc_Binding_Using_Binding_NSDirective_struct*)_tmp24)->f1;_LL15: {struct Cyc_List_List*ns=_tmp26;
_tmp25=ns;goto _LL17;}}else{_LL16: _tmp25=((struct Cyc_Binding_Namespace_Binding_NSDirective_struct*)_tmp24)->f1;_LL17: {struct Cyc_List_List*ns=_tmp25;
# 112
{struct _handler_cons _tmp27;_push_handler(& _tmp27);{int _tmp29=0;if(setjmp(_tmp27.handler))_tmp29=1;if(!_tmp29){{void*_tmp2B=({void*(*_tmp2F9)(void*,struct _fat_ptr*)=lookup;void*_tmp2F8=({void*(*_tmp2F7)(struct Cyc_Dict_Dict d,struct Cyc_List_List*k)=({void*(*_tmp2A)(struct Cyc_Dict_Dict d,struct Cyc_List_List*k)=(void*(*)(struct Cyc_Dict_Dict d,struct Cyc_List_List*k))Cyc_Dict_lookup;_tmp2A;});struct Cyc_Dict_Dict _tmp2F6=ctxt->ns_data;_tmp2F7(_tmp2F6,ns);});_tmp2F9(_tmp2F8,v);});_npop_handler(0U);return _tmp2B;};_pop_handler();}else{void*_tmp28=(void*)Cyc_Core_get_exn_thrown();void*_tmp2C=_tmp28;void*_tmp2D;if(((struct Cyc_Dict_Absent_exn_struct*)_tmp2C)->tag == Cyc_Dict_Absent){_LL19: _LL1A:
 goto _LL18;}else{_LL1B: _tmp2D=_tmp2C;_LL1C: {void*exn=_tmp2D;(int)_rethrow(exn);}}_LL18:;}}}
goto _LL13;}}_LL13:;}}
# 116
(int)_throw((void*)({struct Cyc_Binding_BindingError_exn_struct*_tmp2E=_cycalloc(sizeof(*_tmp2E));_tmp2E->tag=Cyc_Binding_BindingError;_tmp2E;}));}else{_LL6: _tmp1C=(_tmp19.Rel_n).val;_LL7: {struct Cyc_List_List*ns=_tmp1C;
# 118
struct _handler_cons _tmp2F;_push_handler(& _tmp2F);{int _tmp31=0;if(setjmp(_tmp2F.handler))_tmp31=1;if(!_tmp31){
{struct Cyc_List_List*abs_ns=Cyc_Binding_resolve_rel_ns(loc,ctxt,ns);
void*_tmp33=({void*(*_tmp2FD)(void*,struct _fat_ptr*)=lookup;void*_tmp2FC=({void*(*_tmp2FB)(struct Cyc_Dict_Dict d,struct Cyc_List_List*k)=({void*(*_tmp32)(struct Cyc_Dict_Dict d,struct Cyc_List_List*k)=(void*(*)(struct Cyc_Dict_Dict d,struct Cyc_List_List*k))Cyc_Dict_lookup;_tmp32;});struct Cyc_Dict_Dict _tmp2FA=ctxt->ns_data;_tmp2FB(_tmp2FA,abs_ns);});_tmp2FD(_tmp2FC,v);});_npop_handler(0U);return _tmp33;}
# 119
;_pop_handler();}else{void*_tmp30=(void*)Cyc_Core_get_exn_thrown();void*_tmp34=_tmp30;void*_tmp35;if(((struct Cyc_Dict_Absent_exn_struct*)_tmp34)->tag == Cyc_Dict_Absent){_LL1E: _LL1F:
# 121
(int)_throw((void*)({struct Cyc_Binding_BindingError_exn_struct*_tmp36=_cycalloc(sizeof(*_tmp36));_tmp36->tag=Cyc_Binding_BindingError;_tmp36;}));}else{_LL20: _tmp35=_tmp34;_LL21: {void*exn=_tmp35;(int)_rethrow(exn);}}_LL1D:;}}}}case 3U: _LL8: _tmp1B=(_tmp19.C_n).val;_LL9: {struct Cyc_List_List*ns=_tmp1B;
_tmp1A=ns;goto _LLB;}case 2U: _LLA: _tmp1A=(_tmp19.Abs_n).val;_LLB: {struct Cyc_List_List*ns=_tmp1A;
# 124
struct _handler_cons _tmp37;_push_handler(& _tmp37);{int _tmp39=0;if(setjmp(_tmp37.handler))_tmp39=1;if(!_tmp39){{void*_tmp3B=({void*(*_tmp301)(void*,struct _fat_ptr*)=lookup;void*_tmp300=({void*(*_tmp2FF)(struct Cyc_Dict_Dict d,struct Cyc_List_List*k)=({void*(*_tmp3A)(struct Cyc_Dict_Dict d,struct Cyc_List_List*k)=(void*(*)(struct Cyc_Dict_Dict d,struct Cyc_List_List*k))Cyc_Dict_lookup;_tmp3A;});struct Cyc_Dict_Dict _tmp2FE=ctxt->ns_data;_tmp2FF(_tmp2FE,ns);});_tmp301(_tmp300,v);});_npop_handler(0U);return _tmp3B;};_pop_handler();}else{void*_tmp38=(void*)Cyc_Core_get_exn_thrown();void*_tmp3C=_tmp38;void*_tmp3D;if(((struct Cyc_Dict_Absent_exn_struct*)_tmp3C)->tag == Cyc_Dict_Absent){_LL23: _LL24:
(int)_throw((void*)({struct Cyc_Binding_BindingError_exn_struct*_tmp3E=_cycalloc(sizeof(*_tmp3E));_tmp3E->tag=Cyc_Binding_BindingError;_tmp3E;}));}else{_LL25: _tmp3D=_tmp3C;_LL26: {void*exn=_tmp3D;(int)_rethrow(exn);}}_LL22:;}}}default: _LLC: _LLD:
({void*_tmp3F=0U;({struct _fat_ptr _tmp302=({const char*_tmp40="lookup local in global";_tag_fat(_tmp40,sizeof(char),23U);});Cyc_Warn_impos(_tmp302,_tag_fat(_tmp3F,sizeof(void*),0U));});});}_LL3:;}}
# 58 "binding.cyc"
void Cyc_Binding_enter_ns(struct Cyc_Binding_NSCtxt*ctxt,struct _fat_ptr*subname,void*env,void*(*mkdata)(void*)){
# 131 "binding.cyc"
struct Cyc_List_List*ns=ctxt->curr_ns;
struct Cyc_List_List*ns2=({struct Cyc_List_List*_tmp303=ns;Cyc_List_append(_tmp303,({struct Cyc_List_List*_tmp45=_cycalloc(sizeof(*_tmp45));_tmp45->hd=subname,_tmp45->tl=0;_tmp45;}));});
if(!({int(*_tmp305)(struct Cyc_Dict_Dict d,struct Cyc_List_List*k)=({int(*_tmp41)(struct Cyc_Dict_Dict d,struct Cyc_List_List*k)=(int(*)(struct Cyc_Dict_Dict d,struct Cyc_List_List*k))Cyc_Dict_member;_tmp41;});struct Cyc_Dict_Dict _tmp304=ctxt->ns_data;_tmp305(_tmp304,ns2);}))
({struct Cyc_Dict_Dict _tmp309=({struct Cyc_Dict_Dict(*_tmp308)(struct Cyc_Dict_Dict d,struct Cyc_List_List*k,void*v)=({struct Cyc_Dict_Dict(*_tmp42)(struct Cyc_Dict_Dict d,struct Cyc_List_List*k,void*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct Cyc_List_List*k,void*v))Cyc_Dict_insert;_tmp42;});struct Cyc_Dict_Dict _tmp307=ctxt->ns_data;struct Cyc_List_List*_tmp306=ns2;_tmp308(_tmp307,_tmp306,mkdata(env));});ctxt->ns_data=_tmp309;});
# 133
ctxt->curr_ns=ns2;
# 136
({struct Cyc_List_List*_tmp30B=({struct Cyc_List_List*_tmp44=_cycalloc(sizeof(*_tmp44));({void*_tmp30A=(void*)({struct Cyc_Binding_Namespace_Binding_NSDirective_struct*_tmp43=_cycalloc(sizeof(*_tmp43));_tmp43->tag=0U,_tmp43->f1=ns2;_tmp43;});_tmp44->hd=_tmp30A;}),_tmp44->tl=ctxt->availables;_tmp44;});ctxt->availables=_tmp30B;});}
# 58 "binding.cyc"
void Cyc_Binding_leave_ns(struct Cyc_Binding_NSCtxt*ctxt){
# 139 "binding.cyc"
if(ctxt->availables == 0)
({void*_tmp46=0U;({struct _fat_ptr _tmp30C=({const char*_tmp47="leaving topmost namespace";_tag_fat(_tmp47,sizeof(char),26U);});Cyc_Warn_impos(_tmp30C,_tag_fat(_tmp46,sizeof(void*),0U));});});{
# 139
void*_stmttmp2=(void*)((struct Cyc_List_List*)_check_null(ctxt->availables))->hd;void*_tmp48=_stmttmp2;if(((struct Cyc_Binding_Namespace_Binding_NSDirective_struct*)_tmp48)->tag == 0U){_LL1: _LL2:
# 143
 ctxt->availables=((struct Cyc_List_List*)_check_null(ctxt->availables))->tl;
({struct Cyc_List_List*_tmp30D=Cyc_List_rev(((struct Cyc_List_List*)_check_null(Cyc_List_rev(ctxt->curr_ns)))->tl);ctxt->curr_ns=_tmp30D;});
goto _LL0;}else{_LL3: _LL4:
# 147
({void*_tmp49=0U;({struct _fat_ptr _tmp30E=({const char*_tmp4A="leaving using as namespace";_tag_fat(_tmp4A,sizeof(char),27U);});Cyc_Warn_impos(_tmp30E,_tag_fat(_tmp49,sizeof(void*),0U));});});}_LL0:;}}
# 58 "binding.cyc"
struct Cyc_List_List*Cyc_Binding_enter_using(unsigned loc,struct Cyc_Binding_NSCtxt*ctxt,struct _tuple0*usename){
# 151 "binding.cyc"
struct _fat_ptr*_tmp4C;union Cyc_Absyn_Nmspace _tmp4B;_LL1: _tmp4B=usename->f1;_tmp4C=usename->f2;_LL2: {union Cyc_Absyn_Nmspace nsl=_tmp4B;struct _fat_ptr*v=_tmp4C;
struct Cyc_List_List*ns;
{union Cyc_Absyn_Nmspace _tmp4D=nsl;struct Cyc_List_List*_tmp4E;struct Cyc_List_List*_tmp4F;switch((_tmp4D.Loc_n).tag){case 1U: _LL4: _tmp4F=(_tmp4D.Rel_n).val;_LL5: {struct Cyc_List_List*vs=_tmp4F;
# 155
ns=({unsigned _tmp311=loc;struct Cyc_Binding_NSCtxt*_tmp310=ctxt;Cyc_Binding_resolve_rel_ns(_tmp311,_tmp310,({struct Cyc_List_List*_tmp30F=vs;Cyc_List_append(_tmp30F,({struct Cyc_List_List*_tmp50=_cycalloc(sizeof(*_tmp50));_tmp50->hd=v,_tmp50->tl=0;_tmp50;}));}));});{
struct Cyc_List_List*abs_vs=Cyc_List_rev(((struct Cyc_List_List*)_check_null(Cyc_List_rev(ns)))->tl);
({union Cyc_Absyn_Nmspace _tmp312=Cyc_Absyn_Abs_n(abs_vs,0);(*usename).f1=_tmp312;});
goto _LL3;}}case 2U: _LL6: _tmp4E=(_tmp4D.Abs_n).val;_LL7: {struct Cyc_List_List*vs=_tmp4E;
# 160
ns=({struct Cyc_List_List*_tmp313=vs;Cyc_List_append(_tmp313,({struct Cyc_List_List*_tmp51=_cycalloc(sizeof(*_tmp51));_tmp51->hd=v,_tmp51->tl=0;_tmp51;}));});
goto _LL3;}case 4U: _LL8: _LL9:
({void*_tmp52=0U;({struct _fat_ptr _tmp314=({const char*_tmp53="enter_using, Loc_n";_tag_fat(_tmp53,sizeof(char),19U);});Cyc_Warn_impos(_tmp314,_tag_fat(_tmp52,sizeof(void*),0U));});});default: _LLA: _LLB:
({void*_tmp54=0U;({struct _fat_ptr _tmp315=({const char*_tmp55="enter_using, C_n";_tag_fat(_tmp55,sizeof(char),17U);});Cyc_Warn_impos(_tmp315,_tag_fat(_tmp54,sizeof(void*),0U));});});}_LL3:;}
# 165
({struct Cyc_List_List*_tmp317=({struct Cyc_List_List*_tmp57=_cycalloc(sizeof(*_tmp57));({void*_tmp316=(void*)({struct Cyc_Binding_Using_Binding_NSDirective_struct*_tmp56=_cycalloc(sizeof(*_tmp56));_tmp56->tag=1U,_tmp56->f1=ns;_tmp56;});_tmp57->hd=_tmp316;}),_tmp57->tl=ctxt->availables;_tmp57;});ctxt->availables=_tmp317;});
return ns;}}
# 58 "binding.cyc"
void Cyc_Binding_leave_using(struct Cyc_Binding_NSCtxt*ctxt){
# 169 "binding.cyc"
if(ctxt->availables == 0)
({void*_tmp58=0U;({struct _fat_ptr _tmp318=({const char*_tmp59="leaving topmost namespace as a using";_tag_fat(_tmp59,sizeof(char),37U);});Cyc_Warn_impos(_tmp318,_tag_fat(_tmp58,sizeof(void*),0U));});});{
# 169
void*_stmttmp3=(void*)((struct Cyc_List_List*)_check_null(ctxt->availables))->hd;void*_tmp5A=_stmttmp3;if(((struct Cyc_Binding_Using_Binding_NSDirective_struct*)_tmp5A)->tag == 1U){_LL1: _LL2:
# 172
 ctxt->availables=((struct Cyc_List_List*)_check_null(ctxt->availables))->tl;goto _LL0;}else{_LL3: _LL4:
({void*_tmp5B=0U;({struct _fat_ptr _tmp319=({const char*_tmp5C="leaving namespace as using";_tag_fat(_tmp5C,sizeof(char),27U);});Cyc_Warn_impos(_tmp319,_tag_fat(_tmp5B,sizeof(void*),0U));});});}_LL0:;}}struct Cyc_Binding_VarRes_Binding_Resolved_struct{int tag;void*f1;};struct Cyc_Binding_AggrRes_Binding_Resolved_struct{int tag;struct Cyc_Absyn_Aggrdecl*f1;};struct Cyc_Binding_DatatypeRes_Binding_Resolved_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;struct Cyc_Absyn_Datatypefield*f2;};struct Cyc_Binding_EnumRes_Binding_Resolved_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Binding_AnonEnumRes_Binding_Resolved_struct{int tag;void*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Binding_ResolveNSEnv{struct Cyc_Dict_Dict aggrdecls;struct Cyc_Dict_Dict datatypedecls;struct Cyc_Dict_Dict enumdecls;struct Cyc_Dict_Dict typedefs;struct Cyc_Dict_Dict ordinaries;};struct Cyc_Binding_Env{int in_cinclude;struct Cyc_Binding_NSCtxt*ns;struct Cyc_Dict_Dict*local_vars;};
# 212 "binding.cyc"
inline static int Cyc_Binding_in_cinclude(struct Cyc_Binding_Env*env){
return env->in_cinclude;}
# 212
inline static int Cyc_Binding_at_toplevel(struct Cyc_Binding_Env*env){
# 216
return env->local_vars == 0;}
# 212
static struct Cyc_Binding_ResolveNSEnv*Cyc_Binding_mt_renv(int ignore){
# 219
return({struct Cyc_Binding_ResolveNSEnv*_tmp62=_cycalloc(sizeof(*_tmp62));({struct Cyc_Dict_Dict _tmp31E=({struct Cyc_Dict_Dict(*_tmp5D)(int(*cmp)(struct _fat_ptr*,struct _fat_ptr*))=(struct Cyc_Dict_Dict(*)(int(*cmp)(struct _fat_ptr*,struct _fat_ptr*)))Cyc_Dict_empty;_tmp5D;})(Cyc_strptrcmp);_tmp62->aggrdecls=_tmp31E;}),({
struct Cyc_Dict_Dict _tmp31D=({struct Cyc_Dict_Dict(*_tmp5E)(int(*cmp)(struct _fat_ptr*,struct _fat_ptr*))=(struct Cyc_Dict_Dict(*)(int(*cmp)(struct _fat_ptr*,struct _fat_ptr*)))Cyc_Dict_empty;_tmp5E;})(Cyc_strptrcmp);_tmp62->datatypedecls=_tmp31D;}),({
struct Cyc_Dict_Dict _tmp31C=({struct Cyc_Dict_Dict(*_tmp5F)(int(*cmp)(struct _fat_ptr*,struct _fat_ptr*))=(struct Cyc_Dict_Dict(*)(int(*cmp)(struct _fat_ptr*,struct _fat_ptr*)))Cyc_Dict_empty;_tmp5F;})(Cyc_strptrcmp);_tmp62->enumdecls=_tmp31C;}),({
struct Cyc_Dict_Dict _tmp31B=({struct Cyc_Dict_Dict(*_tmp60)(int(*cmp)(struct _fat_ptr*,struct _fat_ptr*))=(struct Cyc_Dict_Dict(*)(int(*cmp)(struct _fat_ptr*,struct _fat_ptr*)))Cyc_Dict_empty;_tmp60;})(Cyc_strptrcmp);_tmp62->typedefs=_tmp31B;}),({
struct Cyc_Dict_Dict _tmp31A=({struct Cyc_Dict_Dict(*_tmp61)(int(*cmp)(struct _fat_ptr*,struct _fat_ptr*))=(struct Cyc_Dict_Dict(*)(int(*cmp)(struct _fat_ptr*,struct _fat_ptr*)))Cyc_Dict_empty;_tmp61;})(Cyc_strptrcmp);_tmp62->ordinaries=_tmp31A;});_tmp62;});}
# 212
static struct Cyc_Absyn_Aggrdecl*Cyc_Binding_lookup_aggrdecl(struct Cyc_Binding_ResolveNSEnv*renv,struct _fat_ptr*v){
# 227
return({struct Cyc_Absyn_Aggrdecl*(*_tmp320)(struct Cyc_Dict_Dict d,struct _fat_ptr*k)=({struct Cyc_Absyn_Aggrdecl*(*_tmp63)(struct Cyc_Dict_Dict d,struct _fat_ptr*k)=(struct Cyc_Absyn_Aggrdecl*(*)(struct Cyc_Dict_Dict d,struct _fat_ptr*k))Cyc_Dict_lookup;_tmp63;});struct Cyc_Dict_Dict _tmp31F=renv->aggrdecls;_tmp320(_tmp31F,v);});}
# 212
static struct Cyc_List_List*Cyc_Binding_lookup_datatypedecl(struct Cyc_Binding_ResolveNSEnv*renv,struct _fat_ptr*v){
# 230
return({struct Cyc_List_List*(*_tmp322)(struct Cyc_Dict_Dict d,struct _fat_ptr*k)=({struct Cyc_List_List*(*_tmp64)(struct Cyc_Dict_Dict d,struct _fat_ptr*k)=(struct Cyc_List_List*(*)(struct Cyc_Dict_Dict d,struct _fat_ptr*k))Cyc_Dict_lookup;_tmp64;});struct Cyc_Dict_Dict _tmp321=renv->datatypedecls;_tmp322(_tmp321,v);});}
# 212
static struct Cyc_Absyn_Enumdecl*Cyc_Binding_lookup_enumdecl(struct Cyc_Binding_ResolveNSEnv*renv,struct _fat_ptr*v){
# 233
return({struct Cyc_Absyn_Enumdecl*(*_tmp324)(struct Cyc_Dict_Dict d,struct _fat_ptr*k)=({struct Cyc_Absyn_Enumdecl*(*_tmp65)(struct Cyc_Dict_Dict d,struct _fat_ptr*k)=(struct Cyc_Absyn_Enumdecl*(*)(struct Cyc_Dict_Dict d,struct _fat_ptr*k))Cyc_Dict_lookup;_tmp65;});struct Cyc_Dict_Dict _tmp323=renv->enumdecls;_tmp324(_tmp323,v);});}
# 212
static struct Cyc_Absyn_Typedefdecl*Cyc_Binding_lookup_typedefdecl(struct Cyc_Binding_ResolveNSEnv*renv,struct _fat_ptr*v){
# 236
return({struct Cyc_Absyn_Typedefdecl*(*_tmp326)(struct Cyc_Dict_Dict d,struct _fat_ptr*k)=({struct Cyc_Absyn_Typedefdecl*(*_tmp66)(struct Cyc_Dict_Dict d,struct _fat_ptr*k)=(struct Cyc_Absyn_Typedefdecl*(*)(struct Cyc_Dict_Dict d,struct _fat_ptr*k))Cyc_Dict_lookup;_tmp66;});struct Cyc_Dict_Dict _tmp325=renv->typedefs;_tmp326(_tmp325,v);});}
# 212
static void*Cyc_Binding_lookup_ordinary_global(struct Cyc_Binding_ResolveNSEnv*renv,struct _fat_ptr*v){
# 239
return({void*(*_tmp328)(struct Cyc_Dict_Dict d,struct _fat_ptr*k)=({void*(*_tmp67)(struct Cyc_Dict_Dict d,struct _fat_ptr*k)=(void*(*)(struct Cyc_Dict_Dict d,struct _fat_ptr*k))Cyc_Dict_lookup;_tmp67;});struct Cyc_Dict_Dict _tmp327=renv->ordinaries;_tmp328(_tmp327,v);});}
# 212
static void*Cyc_Binding_lookup_ordinary(unsigned loc,struct Cyc_Binding_Env*env,struct _tuple0*qv){
# 242
struct _fat_ptr*_tmp69;union Cyc_Absyn_Nmspace _tmp68;_LL1: _tmp68=qv->f1;_tmp69=qv->f2;_LL2: {union Cyc_Absyn_Nmspace nsl=_tmp68;struct _fat_ptr*v=_tmp69;
union Cyc_Absyn_Nmspace _tmp6A=nsl;switch((_tmp6A.Rel_n).tag){case 4U: _LL4: _LL5:
# 245
 if(Cyc_Binding_at_toplevel(env)|| !({int(*_tmp32A)(struct Cyc_Dict_Dict d,struct _fat_ptr*k)=({int(*_tmp6B)(struct Cyc_Dict_Dict d,struct _fat_ptr*k)=(int(*)(struct Cyc_Dict_Dict d,struct _fat_ptr*k))Cyc_Dict_member;_tmp6B;});struct Cyc_Dict_Dict _tmp329=*((struct Cyc_Dict_Dict*)_check_null(env->local_vars));_tmp32A(_tmp329,v);}))
(int)_throw((void*)({struct Cyc_Binding_BindingError_exn_struct*_tmp6C=_cycalloc(sizeof(*_tmp6C));_tmp6C->tag=Cyc_Binding_BindingError;_tmp6C;}));
# 245
return({
# 247
void*(*_tmp32C)(struct Cyc_Dict_Dict d,struct _fat_ptr*k)=({void*(*_tmp6D)(struct Cyc_Dict_Dict d,struct _fat_ptr*k)=(void*(*)(struct Cyc_Dict_Dict d,struct _fat_ptr*k))Cyc_Dict_lookup;_tmp6D;});struct Cyc_Dict_Dict _tmp32B=*((struct Cyc_Dict_Dict*)_check_null(env->local_vars));_tmp32C(_tmp32B,v);});case 1U: if((_tmp6A.Rel_n).val == 0){_LL6: _LL7:
# 249
 if(!Cyc_Binding_at_toplevel(env)&&({int(*_tmp32E)(struct Cyc_Dict_Dict d,struct _fat_ptr*k)=({int(*_tmp6E)(struct Cyc_Dict_Dict d,struct _fat_ptr*k)=(int(*)(struct Cyc_Dict_Dict d,struct _fat_ptr*k))Cyc_Dict_member;_tmp6E;});struct Cyc_Dict_Dict _tmp32D=*((struct Cyc_Dict_Dict*)_check_null(env->local_vars));_tmp32E(_tmp32D,v);}))
return({void*(*_tmp330)(struct Cyc_Dict_Dict d,struct _fat_ptr*k)=({void*(*_tmp6F)(struct Cyc_Dict_Dict d,struct _fat_ptr*k)=(void*(*)(struct Cyc_Dict_Dict d,struct _fat_ptr*k))Cyc_Dict_lookup;_tmp6F;});struct Cyc_Dict_Dict _tmp32F=*((struct Cyc_Dict_Dict*)_check_null(env->local_vars));_tmp330(_tmp32F,v);});
# 249
goto _LL9;}else{goto _LL8;}default: _LL8: _LL9:
# 253
 return({void*(*_tmp333)(unsigned loc,struct Cyc_Binding_NSCtxt*ctxt,struct _tuple0*qv,void*(*lookup)(struct Cyc_Binding_ResolveNSEnv*,struct _fat_ptr*))=({void*(*_tmp70)(unsigned loc,struct Cyc_Binding_NSCtxt*ctxt,struct _tuple0*qv,void*(*lookup)(struct Cyc_Binding_ResolveNSEnv*,struct _fat_ptr*))=(void*(*)(unsigned loc,struct Cyc_Binding_NSCtxt*ctxt,struct _tuple0*qv,void*(*lookup)(struct Cyc_Binding_ResolveNSEnv*,struct _fat_ptr*)))Cyc_Binding_resolve_lookup;_tmp70;});unsigned _tmp332=loc;struct Cyc_Binding_NSCtxt*_tmp331=env->ns;_tmp333(_tmp332,_tmp331,qv,Cyc_Binding_lookup_ordinary_global);});}_LL3:;}}
# 212
void Cyc_Binding_resolve_decl(struct Cyc_Binding_Env*,struct Cyc_Absyn_Decl*);
# 258
void Cyc_Binding_resolve_decls(struct Cyc_Binding_Env*,struct Cyc_List_List*);
void Cyc_Binding_resolve_stmt(struct Cyc_Binding_Env*,struct Cyc_Absyn_Stmt*);
void Cyc_Binding_resolve_pat(struct Cyc_Binding_Env*,struct Cyc_Absyn_Pat*);
void Cyc_Binding_resolve_exp(struct Cyc_Binding_Env*,struct Cyc_Absyn_Exp*);
void Cyc_Binding_resolve_type(unsigned,struct Cyc_Binding_Env*,void*);
void Cyc_Binding_resolve_rgnpo(unsigned,struct Cyc_Binding_Env*,struct Cyc_List_List*);
void Cyc_Binding_resolve_scs(struct Cyc_Binding_Env*,struct Cyc_List_List*);
void Cyc_Binding_resolve_aggrfields(unsigned,struct Cyc_Binding_Env*,struct Cyc_List_List*);
void Cyc_Binding_resolve_function_stuff(unsigned loc,struct Cyc_Binding_Env*env,struct Cyc_Absyn_FnInfo i);
void Cyc_Binding_resolve_asm_iolist(struct Cyc_Binding_Env*,struct Cyc_List_List*l);
struct Cyc_List_List*Cyc_Binding_get_fun_vardecls(int,unsigned,struct Cyc_Binding_Env*,struct Cyc_List_List*,struct Cyc_Absyn_VarargInfo*);
# 273
void Cyc_Binding_absolutize_decl(unsigned loc,struct Cyc_Binding_Env*env,struct _tuple0*qv,enum Cyc_Absyn_Scope sc){
union Cyc_Absyn_Nmspace _stmttmp4=(*qv).f1;union Cyc_Absyn_Nmspace _tmp71=_stmttmp4;switch((_tmp71.Abs_n).tag){case 1U: if((_tmp71.Rel_n).val == 0){_LL1: _LL2:
# 276
 if(Cyc_Binding_at_toplevel(env))
({union Cyc_Absyn_Nmspace _tmp335=({struct Cyc_List_List*_tmp334=(env->ns)->curr_ns;Cyc_Absyn_Abs_n(_tmp334,Cyc_Binding_in_cinclude(env)||(int)sc == (int)Cyc_Absyn_ExternC);});(*qv).f1=_tmp335;});else{
# 279
(*qv).f1=Cyc_Absyn_Loc_n;}
goto _LL0;}else{goto _LL7;}case 4U: _LL3: _LL4:
 goto _LL0;case 2U: _LL5: _LL6:
# 283
 if(!Cyc_Binding_at_toplevel(env))
goto _LL8;
# 283
goto _LL0;default: _LL7: _LL8:
# 286
({struct Cyc_Warn_String_Warn_Warg_struct _tmp73=({struct Cyc_Warn_String_Warn_Warg_struct _tmp2B8;_tmp2B8.tag=0U,({struct _fat_ptr _tmp336=({const char*_tmp77="qualified names in declarations unimplemented (";_tag_fat(_tmp77,sizeof(char),48U);});_tmp2B8.f1=_tmp336;});_tmp2B8;});struct Cyc_Warn_Qvar_Warn_Warg_struct _tmp74=({struct Cyc_Warn_Qvar_Warn_Warg_struct _tmp2B7;_tmp2B7.tag=3U,_tmp2B7.f1=qv;_tmp2B7;});struct Cyc_Warn_String_Warn_Warg_struct _tmp75=({struct Cyc_Warn_String_Warn_Warg_struct _tmp2B6;_tmp2B6.tag=0U,({struct _fat_ptr _tmp337=({const char*_tmp76=")";_tag_fat(_tmp76,sizeof(char),2U);});_tmp2B6.f1=_tmp337;});_tmp2B6;});void*_tmp72[3U];_tmp72[0]=& _tmp73,_tmp72[1]=& _tmp74,_tmp72[2]=& _tmp75;({unsigned _tmp338=loc;Cyc_Warn_err2(_tmp338,_tag_fat(_tmp72,sizeof(void*),3U));});});}_LL0:;}
# 273
void Cyc_Binding_absolutize_topdecl(unsigned loc,struct Cyc_Binding_Env*env,struct _tuple0*qv,enum Cyc_Absyn_Scope sc){
# 293
struct Cyc_Dict_Dict*old_locals=env->local_vars;
env->local_vars=0;
Cyc_Binding_absolutize_decl(loc,env,qv,sc);
env->local_vars=old_locals;}
# 273
void Cyc_Binding_check_warn_override(unsigned loc,struct Cyc_Binding_Env*env,struct _tuple0*q){
# 300
struct _fat_ptr hides_what;
struct _handler_cons _tmp78;_push_handler(& _tmp78);{int _tmp7A=0;if(setjmp(_tmp78.handler))_tmp7A=1;if(!_tmp7A){
{void*_stmttmp5=Cyc_Binding_lookup_ordinary(loc,env,q);void*_tmp7B=_stmttmp5;void*_tmp7C;switch(*((int*)_tmp7B)){case 0U: _LL1: _tmp7C=(void*)((struct Cyc_Binding_VarRes_Binding_Resolved_struct*)_tmp7B)->f1;_LL2: {void*b=_tmp7C;
# 304
if(Cyc_Binding_at_toplevel(env)){
_npop_handler(0U);return;}
# 304
{void*_tmp7D=b;switch(*((int*)_tmp7D)){case 1U: _LLC: _LLD:
# 307
 hides_what=({const char*_tmp7E="global variable";_tag_fat(_tmp7E,sizeof(char),16U);});goto _LLB;case 4U: _LLE: _LLF:
 hides_what=({const char*_tmp7F="local variable";_tag_fat(_tmp7F,sizeof(char),15U);});goto _LLB;case 3U: _LL10: _LL11:
 hides_what=({const char*_tmp80="parameter";_tag_fat(_tmp80,sizeof(char),10U);});goto _LLB;case 5U: _LL12: _LL13:
 hides_what=({const char*_tmp81="pattern variable";_tag_fat(_tmp81,sizeof(char),17U);});goto _LLB;case 2U: _LL14: _LL15:
 hides_what=({const char*_tmp82="function";_tag_fat(_tmp82,sizeof(char),9U);});goto _LLB;default: _LL16: _LL17:
({void*_tmp83=0U;({int(*_tmp33A)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp85)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmp85;});struct _fat_ptr _tmp339=({const char*_tmp84="shadowing free variable!";_tag_fat(_tmp84,sizeof(char),25U);});_tmp33A(_tmp339,_tag_fat(_tmp83,sizeof(void*),0U));});});}_LLB:;}
# 314
goto _LL0;}case 1U: _LL3: _LL4:
 hides_what=({const char*_tmp86="struct constructor";_tag_fat(_tmp86,sizeof(char),19U);});goto _LL0;case 2U: _LL5: _LL6:
 hides_what=({const char*_tmp87="datatype constructor";_tag_fat(_tmp87,sizeof(char),21U);});goto _LL0;case 3U: _LL7: _LL8:
 goto _LLA;default: _LL9: _LLA:
 hides_what=({const char*_tmp88="enum tag";_tag_fat(_tmp88,sizeof(char),9U);});goto _LL0;}_LL0:;}
# 320
({struct Cyc_Warn_String_Warn_Warg_struct _tmp8A=({struct Cyc_Warn_String_Warn_Warg_struct _tmp2BA;_tmp2BA.tag=0U,({struct _fat_ptr _tmp33B=({const char*_tmp8C="declaration hides ";_tag_fat(_tmp8C,sizeof(char),19U);});_tmp2BA.f1=_tmp33B;});_tmp2BA;});struct Cyc_Warn_String_Warn_Warg_struct _tmp8B=({struct Cyc_Warn_String_Warn_Warg_struct _tmp2B9;_tmp2B9.tag=0U,_tmp2B9.f1=hides_what;_tmp2B9;});void*_tmp89[2U];_tmp89[0]=& _tmp8A,_tmp89[1]=& _tmp8B;({unsigned _tmp33C=loc;Cyc_Warn_warn2(_tmp33C,_tag_fat(_tmp89,sizeof(void*),2U));});});
_npop_handler(0U);return;
# 302
;_pop_handler();}else{void*_tmp79=(void*)Cyc_Core_get_exn_thrown();void*_tmp8D=_tmp79;void*_tmp8E;if(((struct Cyc_Binding_BindingError_exn_struct*)_tmp8D)->tag == Cyc_Binding_BindingError){_LL19: _LL1A:
# 322
 return;}else{_LL1B: _tmp8E=_tmp8D;_LL1C: {void*exn=_tmp8E;(int)_rethrow(exn);}}_LL18:;}}}
# 273
void Cyc_Binding_resolve_and_add_vardecl(unsigned loc,struct Cyc_Binding_Env*env,struct Cyc_Absyn_Vardecl*vd){
# 326
Cyc_Binding_absolutize_decl(loc,env,vd->name,vd->sc);{
struct _tuple0*_stmttmp6=vd->name;struct _fat_ptr*_tmp90;union Cyc_Absyn_Nmspace _tmp8F;_LL1: _tmp8F=_stmttmp6->f1;_tmp90=_stmttmp6->f2;_LL2: {union Cyc_Absyn_Nmspace decl_ns=_tmp8F;struct _fat_ptr*decl_name=_tmp90;
Cyc_Binding_resolve_type(loc,env,vd->type);
if(Cyc_Binding_warn_override)
Cyc_Binding_check_warn_override(loc,env,vd->name);
# 329
if(!
# 331
Cyc_Binding_at_toplevel(env))
({struct Cyc_Dict_Dict _tmp341=({struct Cyc_Dict_Dict(*_tmp340)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,void*v)=({struct Cyc_Dict_Dict(*_tmp91)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,void*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,void*v))Cyc_Dict_insert;_tmp91;});struct Cyc_Dict_Dict _tmp33F=*((struct Cyc_Dict_Dict*)_check_null(env->local_vars));struct _fat_ptr*_tmp33E=decl_name;_tmp340(_tmp33F,_tmp33E,(void*)({struct Cyc_Binding_VarRes_Binding_Resolved_struct*_tmp93=_cycalloc(sizeof(*_tmp93));
_tmp93->tag=0U,({void*_tmp33D=(void*)({struct Cyc_Absyn_Local_b_Absyn_Binding_struct*_tmp92=_cycalloc(sizeof(*_tmp92));_tmp92->tag=4U,_tmp92->f1=vd;_tmp92;});_tmp93->f1=_tmp33D;});_tmp93;}));});
# 332
*((struct Cyc_Dict_Dict*)_check_null(env->local_vars))=_tmp341;});else{
# 335
struct Cyc_Binding_ResolveNSEnv*decl_ns_data=({struct Cyc_Binding_ResolveNSEnv*(*_tmp343)(struct Cyc_Binding_NSCtxt*ctxt,union Cyc_Absyn_Nmspace abs_ns)=({struct Cyc_Binding_ResolveNSEnv*(*_tmp97)(struct Cyc_Binding_NSCtxt*ctxt,union Cyc_Absyn_Nmspace abs_ns)=(struct Cyc_Binding_ResolveNSEnv*(*)(struct Cyc_Binding_NSCtxt*ctxt,union Cyc_Absyn_Nmspace abs_ns))Cyc_Binding_get_ns_data;_tmp97;});struct Cyc_Binding_NSCtxt*_tmp342=env->ns;_tmp343(_tmp342,decl_ns);});
({struct Cyc_Dict_Dict _tmp348=({struct Cyc_Dict_Dict(*_tmp347)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,void*v)=({struct Cyc_Dict_Dict(*_tmp94)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,void*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,void*v))Cyc_Dict_insert;_tmp94;});struct Cyc_Dict_Dict _tmp346=decl_ns_data->ordinaries;struct _fat_ptr*_tmp345=decl_name;_tmp347(_tmp346,_tmp345,(void*)({struct Cyc_Binding_VarRes_Binding_Resolved_struct*_tmp96=_cycalloc(sizeof(*_tmp96));
# 338
_tmp96->tag=0U,({void*_tmp344=(void*)({struct Cyc_Absyn_Global_b_Absyn_Binding_struct*_tmp95=_cycalloc(sizeof(*_tmp95));_tmp95->tag=1U,_tmp95->f1=vd;_tmp95;});_tmp96->f1=_tmp344;});_tmp96;}));});
# 336
decl_ns_data->ordinaries=_tmp348;});}}}}
# 273
void Cyc_Binding_resolve_stmt(struct Cyc_Binding_Env*env,struct Cyc_Absyn_Stmt*s){
# 343
void*_stmttmp7=s->r;void*_tmp98=_stmttmp7;struct Cyc_List_List*_tmp9A;struct Cyc_Absyn_Stmt*_tmp99;struct Cyc_List_List*_tmp9C;struct Cyc_Absyn_Exp*_tmp9B;struct Cyc_Absyn_Stmt*_tmp9E;struct Cyc_Absyn_Decl*_tmp9D;struct Cyc_Absyn_Stmt*_tmp9F;struct Cyc_List_List*_tmpA0;struct Cyc_Absyn_Stmt*_tmpA4;struct Cyc_Absyn_Exp*_tmpA3;struct Cyc_Absyn_Exp*_tmpA2;struct Cyc_Absyn_Exp*_tmpA1;struct Cyc_Absyn_Stmt*_tmpA6;struct Cyc_Absyn_Exp*_tmpA5;struct Cyc_Absyn_Exp*_tmpA8;struct Cyc_Absyn_Stmt*_tmpA7;struct Cyc_Absyn_Stmt*_tmpAB;struct Cyc_Absyn_Stmt*_tmpAA;struct Cyc_Absyn_Exp*_tmpA9;struct Cyc_Absyn_Exp*_tmpAC;struct Cyc_Absyn_Stmt*_tmpAE;struct Cyc_Absyn_Stmt*_tmpAD;struct Cyc_Absyn_Exp*_tmpAF;switch(*((int*)_tmp98)){case 0U: _LL1: _LL2:
 return;case 1U: _LL3: _tmpAF=((struct Cyc_Absyn_Exp_s_Absyn_Raw_stmt_struct*)_tmp98)->f1;_LL4: {struct Cyc_Absyn_Exp*e=_tmpAF;
Cyc_Binding_resolve_exp(env,e);return;}case 2U: _LL5: _tmpAD=((struct Cyc_Absyn_Seq_s_Absyn_Raw_stmt_struct*)_tmp98)->f1;_tmpAE=((struct Cyc_Absyn_Seq_s_Absyn_Raw_stmt_struct*)_tmp98)->f2;_LL6: {struct Cyc_Absyn_Stmt*s1=_tmpAD;struct Cyc_Absyn_Stmt*s2=_tmpAE;
Cyc_Binding_resolve_stmt(env,s1);Cyc_Binding_resolve_stmt(env,s2);return;}case 3U: _LL7: _tmpAC=((struct Cyc_Absyn_Return_s_Absyn_Raw_stmt_struct*)_tmp98)->f1;_LL8: {struct Cyc_Absyn_Exp*eopt=_tmpAC;
if((unsigned)eopt)Cyc_Binding_resolve_exp(env,eopt);return;}case 4U: _LL9: _tmpA9=((struct Cyc_Absyn_IfThenElse_s_Absyn_Raw_stmt_struct*)_tmp98)->f1;_tmpAA=((struct Cyc_Absyn_IfThenElse_s_Absyn_Raw_stmt_struct*)_tmp98)->f2;_tmpAB=((struct Cyc_Absyn_IfThenElse_s_Absyn_Raw_stmt_struct*)_tmp98)->f3;_LLA: {struct Cyc_Absyn_Exp*e=_tmpA9;struct Cyc_Absyn_Stmt*s1=_tmpAA;struct Cyc_Absyn_Stmt*s2=_tmpAB;
# 349
Cyc_Binding_resolve_exp(env,e);Cyc_Binding_resolve_stmt(env,s1);Cyc_Binding_resolve_stmt(env,s2);return;}case 14U: _LLB: _tmpA7=((struct Cyc_Absyn_Do_s_Absyn_Raw_stmt_struct*)_tmp98)->f1;_tmpA8=(((struct Cyc_Absyn_Do_s_Absyn_Raw_stmt_struct*)_tmp98)->f2).f1;_LLC: {struct Cyc_Absyn_Stmt*s1=_tmpA7;struct Cyc_Absyn_Exp*e=_tmpA8;
_tmpA5=e;_tmpA6=s1;goto _LLE;}case 5U: _LLD: _tmpA5=(((struct Cyc_Absyn_While_s_Absyn_Raw_stmt_struct*)_tmp98)->f1).f1;_tmpA6=((struct Cyc_Absyn_While_s_Absyn_Raw_stmt_struct*)_tmp98)->f2;_LLE: {struct Cyc_Absyn_Exp*e=_tmpA5;struct Cyc_Absyn_Stmt*s1=_tmpA6;
Cyc_Binding_resolve_exp(env,e);Cyc_Binding_resolve_stmt(env,s1);return;}case 9U: _LLF: _tmpA1=((struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct*)_tmp98)->f1;_tmpA2=(((struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct*)_tmp98)->f2).f1;_tmpA3=(((struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct*)_tmp98)->f3).f1;_tmpA4=((struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct*)_tmp98)->f4;_LL10: {struct Cyc_Absyn_Exp*e1=_tmpA1;struct Cyc_Absyn_Exp*e2=_tmpA2;struct Cyc_Absyn_Exp*e3=_tmpA3;struct Cyc_Absyn_Stmt*s1=_tmpA4;
# 353
Cyc_Binding_resolve_exp(env,e1);Cyc_Binding_resolve_exp(env,e2);Cyc_Binding_resolve_exp(env,e3);
Cyc_Binding_resolve_stmt(env,s1);
return;}case 6U: _LL11: _LL12:
 goto _LL14;case 7U: _LL13: _LL14:
 goto _LL16;case 8U: _LL15: _LL16:
 return;case 11U: _LL17: _tmpA0=((struct Cyc_Absyn_Fallthru_s_Absyn_Raw_stmt_struct*)_tmp98)->f1;_LL18: {struct Cyc_List_List*es=_tmpA0;
# 360
for(0;es != 0;es=es->tl){
Cyc_Binding_resolve_exp(env,(struct Cyc_Absyn_Exp*)es->hd);}
return;}case 13U: _LL19: _tmp9F=((struct Cyc_Absyn_Label_s_Absyn_Raw_stmt_struct*)_tmp98)->f2;_LL1A: {struct Cyc_Absyn_Stmt*s1=_tmp9F;
Cyc_Binding_resolve_stmt(env,s1);return;}case 12U: _LL1B: _tmp9D=((struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct*)_tmp98)->f1;_tmp9E=((struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct*)_tmp98)->f2;_LL1C: {struct Cyc_Absyn_Decl*d=_tmp9D;struct Cyc_Absyn_Stmt*s1=_tmp9E;
# 365
struct Cyc_Dict_Dict old_locals=*((struct Cyc_Dict_Dict*)_check_null(env->local_vars));
Cyc_Binding_resolve_decl(env,d);
Cyc_Binding_resolve_stmt(env,s1);
*((struct Cyc_Dict_Dict*)_check_null(env->local_vars))=old_locals;
return;}case 10U: _LL1D: _tmp9B=((struct Cyc_Absyn_Switch_s_Absyn_Raw_stmt_struct*)_tmp98)->f1;_tmp9C=((struct Cyc_Absyn_Switch_s_Absyn_Raw_stmt_struct*)_tmp98)->f2;_LL1E: {struct Cyc_Absyn_Exp*e=_tmp9B;struct Cyc_List_List*scs=_tmp9C;
Cyc_Binding_resolve_exp(env,e);Cyc_Binding_resolve_scs(env,scs);return;}default: _LL1F: _tmp99=((struct Cyc_Absyn_TryCatch_s_Absyn_Raw_stmt_struct*)_tmp98)->f1;_tmp9A=((struct Cyc_Absyn_TryCatch_s_Absyn_Raw_stmt_struct*)_tmp98)->f2;_LL20: {struct Cyc_Absyn_Stmt*s1=_tmp99;struct Cyc_List_List*scs=_tmp9A;
Cyc_Binding_resolve_stmt(env,s1);Cyc_Binding_resolve_scs(env,scs);return;}}_LL0:;}struct _tuple11{struct Cyc_List_List*f1;struct Cyc_Absyn_Exp*f2;};
# 273
void Cyc_Binding_resolve_exp(struct Cyc_Binding_Env*env,struct Cyc_Absyn_Exp*e){
# 375
void*_stmttmp8=e->r;void*_tmpB0=_stmttmp8;struct Cyc_List_List*_tmpB5;struct Cyc_List_List*_tmpB4;struct Cyc_List_List*_tmpB3;struct _fat_ptr _tmpB2;int _tmpB1;struct Cyc_List_List*_tmpB7;void*_tmpB6;struct Cyc_Absyn_Datatypefield*_tmpBA;struct Cyc_Absyn_Datatypedecl*_tmpB9;struct Cyc_List_List*_tmpB8;struct Cyc_Absyn_Enumfield*_tmpBC;struct Cyc_Absyn_Enumdecl*_tmpBB;struct Cyc_Absyn_Enumfield*_tmpBE;void*_tmpBD;struct Cyc_List_List*_tmpBF;struct Cyc_List_List*_tmpC1;void*_tmpC0;void*_tmpC2;void*_tmpC3;void*_tmpC4;struct Cyc_Absyn_Stmt*_tmpC5;struct Cyc_Absyn_Exp*_tmpC7;void*_tmpC6;struct Cyc_List_List*_tmpC9;struct Cyc_Absyn_Exp*_tmpC8;struct Cyc_Absyn_Exp*_tmpCC;struct Cyc_Absyn_Exp*_tmpCB;struct Cyc_Absyn_Exp*_tmpCA;struct Cyc_Absyn_Exp*_tmpCE;struct Cyc_Absyn_Exp*_tmpCD;struct Cyc_Absyn_Exp*_tmpD1;void**_tmpD0;struct Cyc_Absyn_Exp*_tmpCF;struct Cyc_Absyn_Exp*_tmpD3;struct Cyc_Absyn_Exp*_tmpD2;struct Cyc_Absyn_Exp*_tmpD5;struct Cyc_Absyn_Exp*_tmpD4;struct Cyc_Absyn_Exp*_tmpD7;struct Cyc_Absyn_Exp*_tmpD6;struct Cyc_Absyn_Exp*_tmpD9;struct Cyc_Absyn_Exp*_tmpD8;struct Cyc_Absyn_Exp*_tmpDB;struct Cyc_Absyn_Exp*_tmpDA;struct Cyc_Absyn_Exp*_tmpDD;struct Cyc_Absyn_Exp*_tmpDC;struct Cyc_Absyn_Exp*_tmpDF;struct Cyc_Absyn_Exp*_tmpDE;struct Cyc_Absyn_Exp*_tmpE0;struct Cyc_Absyn_Exp*_tmpE1;struct Cyc_Absyn_Exp*_tmpE2;struct Cyc_Absyn_Exp*_tmpE3;struct Cyc_Absyn_Exp*_tmpE4;struct Cyc_Absyn_Exp*_tmpE5;struct Cyc_Absyn_Exp*_tmpE6;struct Cyc_Absyn_Exp*_tmpE7;struct Cyc_Absyn_Exp*_tmpE8;struct Cyc_Absyn_Exp*_tmpE9;struct Cyc_List_List*_tmpEA;struct Cyc_List_List*_tmpEB;void*_tmpED;struct Cyc_Absyn_Exp*_tmpEC;struct Cyc_List_List*_tmpEE;struct Cyc_Absyn_Exp*_tmpF1;struct Cyc_Absyn_Exp*_tmpF0;struct Cyc_Absyn_Vardecl*_tmpEF;struct Cyc_Absyn_Aggrdecl**_tmpF5;struct Cyc_List_List*_tmpF4;struct Cyc_List_List*_tmpF3;struct _tuple0**_tmpF2;int*_tmpF8;struct Cyc_List_List*_tmpF7;struct Cyc_Absyn_Exp*_tmpF6;void**_tmpF9;switch(*((int*)_tmpB0)){case 1U: _LL1: _tmpF9=(void**)&((struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct*)_tmpB0)->f1;_LL2: {void**b=_tmpF9;
# 377
void*_stmttmp9=*b;void*_tmpFA=_stmttmp9;struct _tuple0*_tmpFB;if(((struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct*)_tmpFA)->tag == 0U){_LL58: _tmpFB=((struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct*)_tmpFA)->f1;_LL59: {struct _tuple0*q=_tmpFB;
# 379
struct _handler_cons _tmpFC;_push_handler(& _tmpFC);{int _tmpFE=0;if(setjmp(_tmpFC.handler))_tmpFE=1;if(!_tmpFE){{void*_stmttmpA=Cyc_Binding_lookup_ordinary(e->loc,env,q);void*_tmpFF=_stmttmpA;struct Cyc_Absyn_Aggrdecl*_tmp100;struct Cyc_Absyn_Enumfield*_tmp102;void*_tmp101;struct Cyc_Absyn_Enumfield*_tmp104;struct Cyc_Absyn_Enumdecl*_tmp103;struct Cyc_Absyn_Datatypefield*_tmp106;struct Cyc_Absyn_Datatypedecl*_tmp105;void*_tmp107;switch(*((int*)_tmpFF)){case 0U: _LL5D: _tmp107=(void*)((struct Cyc_Binding_VarRes_Binding_Resolved_struct*)_tmpFF)->f1;_LL5E: {void*bnd=_tmp107;
*b=bnd;_npop_handler(0U);return;}case 2U: _LL5F: _tmp105=((struct Cyc_Binding_DatatypeRes_Binding_Resolved_struct*)_tmpFF)->f1;_tmp106=((struct Cyc_Binding_DatatypeRes_Binding_Resolved_struct*)_tmpFF)->f2;_LL60: {struct Cyc_Absyn_Datatypedecl*tud=_tmp105;struct Cyc_Absyn_Datatypefield*tuf=_tmp106;
({void*_tmp349=(void*)({struct Cyc_Absyn_Datatype_e_Absyn_Raw_exp_struct*_tmp108=_cycalloc(sizeof(*_tmp108));_tmp108->tag=31U,_tmp108->f1=0,_tmp108->f2=tud,_tmp108->f3=tuf;_tmp108;});e->r=_tmp349;});_npop_handler(0U);return;}case 3U: _LL61: _tmp103=((struct Cyc_Binding_EnumRes_Binding_Resolved_struct*)_tmpFF)->f1;_tmp104=((struct Cyc_Binding_EnumRes_Binding_Resolved_struct*)_tmpFF)->f2;_LL62: {struct Cyc_Absyn_Enumdecl*ed=_tmp103;struct Cyc_Absyn_Enumfield*ef=_tmp104;
({void*_tmp34A=(void*)({struct Cyc_Absyn_Enum_e_Absyn_Raw_exp_struct*_tmp109=_cycalloc(sizeof(*_tmp109));_tmp109->tag=32U,_tmp109->f1=ed,_tmp109->f2=ef;_tmp109;});e->r=_tmp34A;});_npop_handler(0U);return;}case 4U: _LL63: _tmp101=(void*)((struct Cyc_Binding_AnonEnumRes_Binding_Resolved_struct*)_tmpFF)->f1;_tmp102=((struct Cyc_Binding_AnonEnumRes_Binding_Resolved_struct*)_tmpFF)->f2;_LL64: {void*t=_tmp101;struct Cyc_Absyn_Enumfield*ef=_tmp102;
({void*_tmp34B=(void*)({struct Cyc_Absyn_AnonEnum_e_Absyn_Raw_exp_struct*_tmp10A=_cycalloc(sizeof(*_tmp10A));_tmp10A->tag=33U,_tmp10A->f1=t,_tmp10A->f2=ef;_tmp10A;});e->r=_tmp34B;});_npop_handler(0U);return;}default: _LL65: _tmp100=((struct Cyc_Binding_AggrRes_Binding_Resolved_struct*)_tmpFF)->f1;_LL66: {struct Cyc_Absyn_Aggrdecl*ad=_tmp100;
({struct Cyc_Warn_String_Warn_Warg_struct _tmp10C=({struct Cyc_Warn_String_Warn_Warg_struct _tmp2BC;_tmp2BC.tag=0U,({struct _fat_ptr _tmp34C=({const char*_tmp10E="bad occurrence of type name ";_tag_fat(_tmp10E,sizeof(char),29U);});_tmp2BC.f1=_tmp34C;});_tmp2BC;});struct Cyc_Warn_Qvar_Warn_Warg_struct _tmp10D=({struct Cyc_Warn_Qvar_Warn_Warg_struct _tmp2BB;_tmp2BB.tag=3U,_tmp2BB.f1=q;_tmp2BB;});void*_tmp10B[2U];_tmp10B[0]=& _tmp10C,_tmp10B[1]=& _tmp10D;({unsigned _tmp34D=e->loc;Cyc_Warn_err2(_tmp34D,_tag_fat(_tmp10B,sizeof(void*),2U));});});_npop_handler(0U);return;}}_LL5C:;}
# 379
;_pop_handler();}else{void*_tmpFD=(void*)Cyc_Core_get_exn_thrown();void*_tmp10F=_tmpFD;void*_tmp110;if(((struct Cyc_Binding_BindingError_exn_struct*)_tmp10F)->tag == Cyc_Binding_BindingError){_LL68: _LL69:
# 387
({struct Cyc_Warn_String_Warn_Warg_struct _tmp112=({struct Cyc_Warn_String_Warn_Warg_struct _tmp2BE;_tmp2BE.tag=0U,({struct _fat_ptr _tmp34E=({const char*_tmp114="undeclared identifier ";_tag_fat(_tmp114,sizeof(char),23U);});_tmp2BE.f1=_tmp34E;});_tmp2BE;});struct Cyc_Warn_Qvar_Warn_Warg_struct _tmp113=({struct Cyc_Warn_Qvar_Warn_Warg_struct _tmp2BD;_tmp2BD.tag=3U,_tmp2BD.f1=q;_tmp2BD;});void*_tmp111[2U];_tmp111[0]=& _tmp112,_tmp111[1]=& _tmp113;({unsigned _tmp34F=e->loc;Cyc_Warn_err2(_tmp34F,_tag_fat(_tmp111,sizeof(void*),2U));});});return;}else{_LL6A: _tmp110=_tmp10F;_LL6B: {void*exn=_tmp110;(int)_rethrow(exn);}}_LL67:;}}}}else{_LL5A: _LL5B:
# 389
 return;}_LL57:;}case 10U: _LL3: _tmpF6=((struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct*)_tmpB0)->f1;_tmpF7=((struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct*)_tmpB0)->f2;_tmpF8=(int*)&((struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct*)_tmpB0)->f4;_LL4: {struct Cyc_Absyn_Exp*e1=_tmpF6;struct Cyc_List_List*es=_tmpF7;int*b=_tmpF8;
# 392
*b=1;
# 394
{struct Cyc_List_List*es2=es;for(0;es2 != 0;es2=es2->tl){
Cyc_Binding_resolve_exp(env,(struct Cyc_Absyn_Exp*)es2->hd);}}{
void*_stmttmpB=e1->r;void*_tmp115=_stmttmpB;void**_tmp116;if(((struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct*)_tmp115)->tag == 1U){_LL6D: _tmp116=(void**)&((struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct*)_tmp115)->f1;_LL6E: {void**b=_tmp116;
# 398
void*_stmttmpC=*b;void*_tmp117=_stmttmpC;struct _tuple0*_tmp118;if(((struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct*)_tmp117)->tag == 0U){_LL72: _tmp118=((struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct*)_tmp117)->f1;_LL73: {struct _tuple0*q=_tmp118;
# 400
struct _handler_cons _tmp119;_push_handler(& _tmp119);{int _tmp11B=0;if(setjmp(_tmp119.handler))_tmp11B=1;if(!_tmp11B){{void*_stmttmpD=Cyc_Binding_lookup_ordinary(e1->loc,env,q);void*_tmp11C=_stmttmpD;struct Cyc_Absyn_Datatypefield*_tmp11E;struct Cyc_Absyn_Datatypedecl*_tmp11D;struct Cyc_Absyn_Aggrdecl*_tmp11F;void*_tmp120;switch(*((int*)_tmp11C)){case 0U: _LL77: _tmp120=(void*)((struct Cyc_Binding_VarRes_Binding_Resolved_struct*)_tmp11C)->f1;_LL78: {void*bnd=_tmp120;
# 402
*b=bnd;_npop_handler(0U);return;}case 1U: _LL79: _tmp11F=((struct Cyc_Binding_AggrRes_Binding_Resolved_struct*)_tmp11C)->f1;_LL7A: {struct Cyc_Absyn_Aggrdecl*ad=_tmp11F;
# 404
struct Cyc_List_List*dles=0;
for(0;es != 0;es=es->tl){
dles=({struct Cyc_List_List*_tmp122=_cycalloc(sizeof(*_tmp122));({struct _tuple11*_tmp350=({struct _tuple11*_tmp121=_cycalloc(sizeof(*_tmp121));_tmp121->f1=0,_tmp121->f2=(struct Cyc_Absyn_Exp*)es->hd;_tmp121;});_tmp122->hd=_tmp350;}),_tmp122->tl=dles;_tmp122;});}
({void*_tmp352=(void*)({struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct*_tmp123=_cycalloc(sizeof(*_tmp123));_tmp123->tag=29U,_tmp123->f1=ad->name,_tmp123->f2=0,({struct Cyc_List_List*_tmp351=Cyc_List_imp_rev(dles);_tmp123->f3=_tmp351;}),_tmp123->f4=ad;_tmp123;});e->r=_tmp352;});
_npop_handler(0U);return;}case 2U: _LL7B: _tmp11D=((struct Cyc_Binding_DatatypeRes_Binding_Resolved_struct*)_tmp11C)->f1;_tmp11E=((struct Cyc_Binding_DatatypeRes_Binding_Resolved_struct*)_tmp11C)->f2;_LL7C: {struct Cyc_Absyn_Datatypedecl*tud=_tmp11D;struct Cyc_Absyn_Datatypefield*tuf=_tmp11E;
# 410
if(tuf->typs == 0)
# 412
({struct Cyc_Warn_Qvar_Warn_Warg_struct _tmp125=({struct Cyc_Warn_Qvar_Warn_Warg_struct _tmp2C0;_tmp2C0.tag=3U,_tmp2C0.f1=tuf->name;_tmp2C0;});struct Cyc_Warn_String_Warn_Warg_struct _tmp126=({struct Cyc_Warn_String_Warn_Warg_struct _tmp2BF;_tmp2BF.tag=0U,({struct _fat_ptr _tmp353=({const char*_tmp127=" is a constant, not a function";_tag_fat(_tmp127,sizeof(char),31U);});_tmp2BF.f1=_tmp353;});_tmp2BF;});void*_tmp124[2U];_tmp124[0]=& _tmp125,_tmp124[1]=& _tmp126;({unsigned _tmp354=e->loc;Cyc_Warn_err2(_tmp354,_tag_fat(_tmp124,sizeof(void*),2U));});});
# 410
({void*_tmp355=(void*)({struct Cyc_Absyn_Datatype_e_Absyn_Raw_exp_struct*_tmp128=_cycalloc(sizeof(*_tmp128));
# 413
_tmp128->tag=31U,_tmp128->f1=es,_tmp128->f2=tud,_tmp128->f3=tuf;_tmp128;});
# 410
e->r=_tmp355;});
# 414
_npop_handler(0U);return;}case 4U: _LL7D: _LL7E:
 goto _LL80;default: _LL7F: _LL80:
# 417
({struct Cyc_Warn_Qvar_Warn_Warg_struct _tmp12A=({struct Cyc_Warn_Qvar_Warn_Warg_struct _tmp2C2;_tmp2C2.tag=3U,_tmp2C2.f1=q;_tmp2C2;});struct Cyc_Warn_String_Warn_Warg_struct _tmp12B=({struct Cyc_Warn_String_Warn_Warg_struct _tmp2C1;_tmp2C1.tag=0U,({struct _fat_ptr _tmp356=({const char*_tmp12C=" is an enum constructor, not a function";_tag_fat(_tmp12C,sizeof(char),40U);});_tmp2C1.f1=_tmp356;});_tmp2C1;});void*_tmp129[2U];_tmp129[0]=& _tmp12A,_tmp129[1]=& _tmp12B;({unsigned _tmp357=e->loc;Cyc_Warn_err2(_tmp357,_tag_fat(_tmp129,sizeof(void*),2U));});});_npop_handler(0U);return;}_LL76:;}
# 400
;_pop_handler();}else{void*_tmp11A=(void*)Cyc_Core_get_exn_thrown();void*_tmp12D=_tmp11A;void*_tmp12E;if(((struct Cyc_Binding_BindingError_exn_struct*)_tmp12D)->tag == Cyc_Binding_BindingError){_LL82: _LL83:
# 419
({struct Cyc_Warn_String_Warn_Warg_struct _tmp130=({struct Cyc_Warn_String_Warn_Warg_struct _tmp2C4;_tmp2C4.tag=0U,({struct _fat_ptr _tmp358=({const char*_tmp132="undeclared identifier ";_tag_fat(_tmp132,sizeof(char),23U);});_tmp2C4.f1=_tmp358;});_tmp2C4;});struct Cyc_Warn_Qvar_Warn_Warg_struct _tmp131=({struct Cyc_Warn_Qvar_Warn_Warg_struct _tmp2C3;_tmp2C3.tag=3U,_tmp2C3.f1=q;_tmp2C3;});void*_tmp12F[2U];_tmp12F[0]=& _tmp130,_tmp12F[1]=& _tmp131;({unsigned _tmp359=e->loc;Cyc_Warn_err2(_tmp359,_tag_fat(_tmp12F,sizeof(void*),2U));});});return;}else{_LL84: _tmp12E=_tmp12D;_LL85: {void*exn=_tmp12E;(int)_rethrow(exn);}}_LL81:;}}}}else{_LL74: _LL75:
# 421
 return;}_LL71:;}}else{_LL6F: _LL70:
# 423
 Cyc_Binding_resolve_exp(env,e1);return;}_LL6C:;}}case 29U: _LL5: _tmpF2=(struct _tuple0**)&((struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct*)_tmpB0)->f1;_tmpF3=((struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct*)_tmpB0)->f2;_tmpF4=((struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct*)_tmpB0)->f3;_tmpF5=(struct Cyc_Absyn_Aggrdecl**)&((struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct*)_tmpB0)->f4;_LL6: {struct _tuple0**tn=_tmpF2;struct Cyc_List_List*ts=_tmpF3;struct Cyc_List_List*dles=_tmpF4;struct Cyc_Absyn_Aggrdecl**adopt=_tmpF5;
# 427
for(0;dles != 0;dles=dles->tl){
Cyc_Binding_resolve_exp(env,(*((struct _tuple11*)dles->hd)).f2);}{
struct _handler_cons _tmp133;_push_handler(& _tmp133);{int _tmp135=0;if(setjmp(_tmp133.handler))_tmp135=1;if(!_tmp135){
({struct Cyc_Absyn_Aggrdecl*_tmp35D=({struct Cyc_Absyn_Aggrdecl*(*_tmp35C)(unsigned loc,struct Cyc_Binding_NSCtxt*ctxt,struct _tuple0*qv,struct Cyc_Absyn_Aggrdecl*(*lookup)(struct Cyc_Binding_ResolveNSEnv*,struct _fat_ptr*))=({struct Cyc_Absyn_Aggrdecl*(*_tmp136)(unsigned loc,struct Cyc_Binding_NSCtxt*ctxt,struct _tuple0*qv,struct Cyc_Absyn_Aggrdecl*(*lookup)(struct Cyc_Binding_ResolveNSEnv*,struct _fat_ptr*))=(struct Cyc_Absyn_Aggrdecl*(*)(unsigned loc,struct Cyc_Binding_NSCtxt*ctxt,struct _tuple0*qv,struct Cyc_Absyn_Aggrdecl*(*lookup)(struct Cyc_Binding_ResolveNSEnv*,struct _fat_ptr*)))Cyc_Binding_resolve_lookup;_tmp136;});unsigned _tmp35B=e->loc;struct Cyc_Binding_NSCtxt*_tmp35A=env->ns;_tmp35C(_tmp35B,_tmp35A,*tn,Cyc_Binding_lookup_aggrdecl);});*adopt=_tmp35D;});
*tn=((struct Cyc_Absyn_Aggrdecl*)_check_null(*adopt))->name;
_npop_handler(0U);return;
# 430
;_pop_handler();}else{void*_tmp134=(void*)Cyc_Core_get_exn_thrown();void*_tmp137=_tmp134;void*_tmp138;if(((struct Cyc_Binding_BindingError_exn_struct*)_tmp137)->tag == Cyc_Binding_BindingError){_LL87: _LL88:
# 434
({struct Cyc_Warn_String_Warn_Warg_struct _tmp13A=({struct Cyc_Warn_String_Warn_Warg_struct _tmp2C6;_tmp2C6.tag=0U,({struct _fat_ptr _tmp35E=({const char*_tmp13C="unbound struct/union name ";_tag_fat(_tmp13C,sizeof(char),27U);});_tmp2C6.f1=_tmp35E;});_tmp2C6;});struct Cyc_Warn_Qvar_Warn_Warg_struct _tmp13B=({struct Cyc_Warn_Qvar_Warn_Warg_struct _tmp2C5;_tmp2C5.tag=3U,_tmp2C5.f1=*tn;_tmp2C5;});void*_tmp139[2U];_tmp139[0]=& _tmp13A,_tmp139[1]=& _tmp13B;({unsigned _tmp35F=e->loc;Cyc_Warn_err2(_tmp35F,_tag_fat(_tmp139,sizeof(void*),2U));});});
return;}else{_LL89: _tmp138=_tmp137;_LL8A: {void*exn=_tmp138;(int)_rethrow(exn);}}_LL86:;}}}}case 27U: _LL7: _tmpEF=((struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct*)_tmpB0)->f1;_tmpF0=((struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct*)_tmpB0)->f2;_tmpF1=((struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct*)_tmpB0)->f3;_LL8: {struct Cyc_Absyn_Vardecl*vd=_tmpEF;struct Cyc_Absyn_Exp*e1=_tmpF0;struct Cyc_Absyn_Exp*e2=_tmpF1;
# 439
Cyc_Binding_resolve_exp(env,e1);
if(Cyc_Binding_at_toplevel(env)){
({struct Cyc_Dict_Dict*_tmp361=({struct Cyc_Dict_Dict*_tmp13E=_cycalloc(sizeof(*_tmp13E));({struct Cyc_Dict_Dict _tmp360=({struct Cyc_Dict_Dict(*_tmp13D)(int(*cmp)(struct _fat_ptr*,struct _fat_ptr*))=(struct Cyc_Dict_Dict(*)(int(*cmp)(struct _fat_ptr*,struct _fat_ptr*)))Cyc_Dict_empty;_tmp13D;})(Cyc_strptrcmp);*_tmp13E=_tmp360;});_tmp13E;});env->local_vars=_tmp361;});
Cyc_Binding_resolve_and_add_vardecl(e->loc,env,vd);
Cyc_Binding_resolve_exp(env,e2);
env->local_vars=0;
return;}{
# 440
struct Cyc_Dict_Dict old_locals=*((struct Cyc_Dict_Dict*)_check_null(env->local_vars));
# 448
Cyc_Binding_resolve_and_add_vardecl(e->loc,env,vd);
Cyc_Binding_resolve_exp(env,e2);
*((struct Cyc_Dict_Dict*)_check_null(env->local_vars))=old_locals;
return;}}case 37U: _LL9: _tmpEE=((struct Cyc_Absyn_UnresolvedMem_e_Absyn_Raw_exp_struct*)_tmpB0)->f2;_LLA: {struct Cyc_List_List*dles=_tmpEE;
# 455
for(0;dles != 0;dles=dles->tl){
Cyc_Binding_resolve_exp(env,(*((struct _tuple11*)dles->hd)).f2);}
return;}case 28U: _LLB: _tmpEC=((struct Cyc_Absyn_ComprehensionNoinit_e_Absyn_Raw_exp_struct*)_tmpB0)->f1;_tmpED=(void*)((struct Cyc_Absyn_ComprehensionNoinit_e_Absyn_Raw_exp_struct*)_tmpB0)->f2;_LLC: {struct Cyc_Absyn_Exp*e1=_tmpEC;void*t=_tmpED;
# 461
Cyc_Binding_resolve_exp(env,e1);
Cyc_Binding_resolve_type(e->loc,env,t);
return;}case 2U: _LLD: _LLE:
# 465
 goto _LL10;case 0U: _LLF: _LL10:
 return;case 24U: _LL11: _tmpEB=((struct Cyc_Absyn_Tuple_e_Absyn_Raw_exp_struct*)_tmpB0)->f1;_LL12: {struct Cyc_List_List*es=_tmpEB;
# 468
_tmpEA=es;goto _LL14;}case 3U: _LL13: _tmpEA=((struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct*)_tmpB0)->f2;_LL14: {struct Cyc_List_List*es=_tmpEA;
# 470
for(0;es != 0;es=es->tl){
Cyc_Binding_resolve_exp(env,(struct Cyc_Absyn_Exp*)es->hd);}
return;}case 42U: _LL15: _tmpE9=((struct Cyc_Absyn_Extension_e_Absyn_Raw_exp_struct*)_tmpB0)->f1;_LL16: {struct Cyc_Absyn_Exp*e1=_tmpE9;
# 474
_tmpE8=e1;goto _LL18;}case 39U: _LL17: _tmpE8=((struct Cyc_Absyn_Tagcheck_e_Absyn_Raw_exp_struct*)_tmpB0)->f1;_LL18: {struct Cyc_Absyn_Exp*e1=_tmpE8;
_tmpE7=e1;goto _LL1A;}case 12U: _LL19: _tmpE7=((struct Cyc_Absyn_NoInstantiate_e_Absyn_Raw_exp_struct*)_tmpB0)->f1;_LL1A: {struct Cyc_Absyn_Exp*e1=_tmpE7;
_tmpE6=e1;goto _LL1C;}case 18U: _LL1B: _tmpE6=((struct Cyc_Absyn_Sizeofexp_e_Absyn_Raw_exp_struct*)_tmpB0)->f1;_LL1C: {struct Cyc_Absyn_Exp*e1=_tmpE6;
_tmpE5=e1;goto _LL1E;}case 11U: _LL1D: _tmpE5=((struct Cyc_Absyn_Throw_e_Absyn_Raw_exp_struct*)_tmpB0)->f1;_LL1E: {struct Cyc_Absyn_Exp*e1=_tmpE5;
_tmpE4=e1;goto _LL20;}case 5U: _LL1F: _tmpE4=((struct Cyc_Absyn_Increment_e_Absyn_Raw_exp_struct*)_tmpB0)->f1;_LL20: {struct Cyc_Absyn_Exp*e1=_tmpE4;
_tmpE3=e1;goto _LL22;}case 22U: _LL21: _tmpE3=((struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct*)_tmpB0)->f1;_LL22: {struct Cyc_Absyn_Exp*e1=_tmpE3;
_tmpE2=e1;goto _LL24;}case 21U: _LL23: _tmpE2=((struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct*)_tmpB0)->f1;_LL24: {struct Cyc_Absyn_Exp*e1=_tmpE2;
_tmpE1=e1;goto _LL26;}case 15U: _LL25: _tmpE1=((struct Cyc_Absyn_Address_e_Absyn_Raw_exp_struct*)_tmpB0)->f1;_LL26: {struct Cyc_Absyn_Exp*e1=_tmpE1;
_tmpE0=e1;goto _LL28;}case 20U: _LL27: _tmpE0=((struct Cyc_Absyn_Deref_e_Absyn_Raw_exp_struct*)_tmpB0)->f1;_LL28: {struct Cyc_Absyn_Exp*e1=_tmpE0;
Cyc_Binding_resolve_exp(env,e1);return;}case 36U: _LL29: _tmpDE=((struct Cyc_Absyn_Swap_e_Absyn_Raw_exp_struct*)_tmpB0)->f1;_tmpDF=((struct Cyc_Absyn_Swap_e_Absyn_Raw_exp_struct*)_tmpB0)->f2;_LL2A: {struct Cyc_Absyn_Exp*e1=_tmpDE;struct Cyc_Absyn_Exp*e2=_tmpDF;
# 485
_tmpDC=e1;_tmpDD=e2;goto _LL2C;}case 9U: _LL2B: _tmpDC=((struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct*)_tmpB0)->f1;_tmpDD=((struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct*)_tmpB0)->f2;_LL2C: {struct Cyc_Absyn_Exp*e1=_tmpDC;struct Cyc_Absyn_Exp*e2=_tmpDD;
_tmpDA=e1;_tmpDB=e2;goto _LL2E;}case 4U: _LL2D: _tmpDA=((struct Cyc_Absyn_AssignOp_e_Absyn_Raw_exp_struct*)_tmpB0)->f1;_tmpDB=((struct Cyc_Absyn_AssignOp_e_Absyn_Raw_exp_struct*)_tmpB0)->f3;_LL2E: {struct Cyc_Absyn_Exp*e1=_tmpDA;struct Cyc_Absyn_Exp*e2=_tmpDB;
_tmpD8=e1;_tmpD9=e2;goto _LL30;}case 23U: _LL2F: _tmpD8=((struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct*)_tmpB0)->f1;_tmpD9=((struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct*)_tmpB0)->f2;_LL30: {struct Cyc_Absyn_Exp*e1=_tmpD8;struct Cyc_Absyn_Exp*e2=_tmpD9;
_tmpD6=e1;_tmpD7=e2;goto _LL32;}case 7U: _LL31: _tmpD6=((struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct*)_tmpB0)->f1;_tmpD7=((struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct*)_tmpB0)->f2;_LL32: {struct Cyc_Absyn_Exp*e1=_tmpD6;struct Cyc_Absyn_Exp*e2=_tmpD7;
_tmpD4=e1;_tmpD5=e2;goto _LL34;}case 8U: _LL33: _tmpD4=((struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct*)_tmpB0)->f1;_tmpD5=((struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct*)_tmpB0)->f2;_LL34: {struct Cyc_Absyn_Exp*e1=_tmpD4;struct Cyc_Absyn_Exp*e2=_tmpD5;
Cyc_Binding_resolve_exp(env,e1);Cyc_Binding_resolve_exp(env,e2);return;}case 34U: _LL35: _tmpD2=((struct Cyc_Absyn_Spawn_e_Absyn_Raw_exp_struct*)_tmpB0)->f1;_tmpD3=((struct Cyc_Absyn_Spawn_e_Absyn_Raw_exp_struct*)_tmpB0)->f2;_LL36: {struct Cyc_Absyn_Exp*e1=_tmpD2;struct Cyc_Absyn_Exp*e2=_tmpD3;
# 493
Cyc_Binding_resolve_exp(env,e1);
Cyc_Binding_resolve_exp(env,e2);
return;}case 35U: _LL37: _tmpCF=(((struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct*)_tmpB0)->f1).rgn;_tmpD0=(((struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct*)_tmpB0)->f1).elt_type;_tmpD1=(((struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct*)_tmpB0)->f1).num_elts;_LL38: {struct Cyc_Absyn_Exp*eo=_tmpCF;void**to=_tmpD0;struct Cyc_Absyn_Exp*e1=_tmpD1;
# 497
if(eo != 0)Cyc_Binding_resolve_exp(env,eo);if(to != 0)
Cyc_Binding_resolve_type(e->loc,env,*to);
# 497
Cyc_Binding_resolve_exp(env,e1);
# 500
return;}case 16U: _LL39: _tmpCD=((struct Cyc_Absyn_New_e_Absyn_Raw_exp_struct*)_tmpB0)->f1;_tmpCE=((struct Cyc_Absyn_New_e_Absyn_Raw_exp_struct*)_tmpB0)->f2;_LL3A: {struct Cyc_Absyn_Exp*eo=_tmpCD;struct Cyc_Absyn_Exp*e2=_tmpCE;
# 503
if(eo != 0)Cyc_Binding_resolve_exp(env,eo);Cyc_Binding_resolve_exp(env,e2);
# 505
return;}case 6U: _LL3B: _tmpCA=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmpB0)->f1;_tmpCB=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmpB0)->f2;_tmpCC=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmpB0)->f3;_LL3C: {struct Cyc_Absyn_Exp*e1=_tmpCA;struct Cyc_Absyn_Exp*e2=_tmpCB;struct Cyc_Absyn_Exp*e3=_tmpCC;
# 508
Cyc_Binding_resolve_exp(env,e1);Cyc_Binding_resolve_exp(env,e2);Cyc_Binding_resolve_exp(env,e3);return;}case 13U: _LL3D: _tmpC8=((struct Cyc_Absyn_Instantiate_e_Absyn_Raw_exp_struct*)_tmpB0)->f1;_tmpC9=((struct Cyc_Absyn_Instantiate_e_Absyn_Raw_exp_struct*)_tmpB0)->f2;_LL3E: {struct Cyc_Absyn_Exp*e1=_tmpC8;struct Cyc_List_List*ts=_tmpC9;
# 511
Cyc_Binding_resolve_exp(env,e1);
for(0;ts != 0;ts=ts->tl){
Cyc_Binding_resolve_type(e->loc,env,(void*)ts->hd);}
return;}case 14U: _LL3F: _tmpC6=(void*)((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmpB0)->f1;_tmpC7=((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmpB0)->f2;_LL40: {void*t=_tmpC6;struct Cyc_Absyn_Exp*e1=_tmpC7;
# 517
Cyc_Binding_resolve_exp(env,e1);Cyc_Binding_resolve_type(e->loc,env,t);return;}case 38U: _LL41: _tmpC5=((struct Cyc_Absyn_StmtExp_e_Absyn_Raw_exp_struct*)_tmpB0)->f1;_LL42: {struct Cyc_Absyn_Stmt*s=_tmpC5;
# 519
Cyc_Binding_resolve_stmt(env,s);return;}case 40U: _LL43: _tmpC4=(void*)((struct Cyc_Absyn_Valueof_e_Absyn_Raw_exp_struct*)_tmpB0)->f1;_LL44: {void*t=_tmpC4;
# 521
_tmpC3=t;goto _LL46;}case 19U: _LL45: _tmpC3=(void*)((struct Cyc_Absyn_Offsetof_e_Absyn_Raw_exp_struct*)_tmpB0)->f1;_LL46: {void*t=_tmpC3;
_tmpC2=t;goto _LL48;}case 17U: _LL47: _tmpC2=(void*)((struct Cyc_Absyn_Sizeoftype_e_Absyn_Raw_exp_struct*)_tmpB0)->f1;_LL48: {void*t=_tmpC2;
Cyc_Binding_resolve_type(e->loc,env,t);return;}case 25U: _LL49: _tmpC0=(((struct Cyc_Absyn_CompoundLit_e_Absyn_Raw_exp_struct*)_tmpB0)->f1)->f3;_tmpC1=((struct Cyc_Absyn_CompoundLit_e_Absyn_Raw_exp_struct*)_tmpB0)->f2;_LL4A: {void*t=_tmpC0;struct Cyc_List_List*dles=_tmpC1;
# 526
Cyc_Binding_resolve_type(e->loc,env,t);
_tmpBF=dles;goto _LL4C;}case 26U: _LL4B: _tmpBF=((struct Cyc_Absyn_Array_e_Absyn_Raw_exp_struct*)_tmpB0)->f1;_LL4C: {struct Cyc_List_List*dles=_tmpBF;
# 530
for(0;dles != 0;dles=dles->tl){
Cyc_Binding_resolve_exp(env,(*((struct _tuple11*)dles->hd)).f2);}
return;}case 33U: _LL4D: _tmpBD=(void*)((struct Cyc_Absyn_AnonEnum_e_Absyn_Raw_exp_struct*)_tmpB0)->f1;_tmpBE=((struct Cyc_Absyn_AnonEnum_e_Absyn_Raw_exp_struct*)_tmpB0)->f2;_LL4E: {void*t=_tmpBD;struct Cyc_Absyn_Enumfield*ef=_tmpBE;
# 535
Cyc_Binding_resolve_type(e->loc,env,t);return;}case 32U: _LL4F: _tmpBB=((struct Cyc_Absyn_Enum_e_Absyn_Raw_exp_struct*)_tmpB0)->f1;_tmpBC=((struct Cyc_Absyn_Enum_e_Absyn_Raw_exp_struct*)_tmpB0)->f2;_LL50: {struct Cyc_Absyn_Enumdecl*ed=_tmpBB;struct Cyc_Absyn_Enumfield*ef=_tmpBC;
return;}case 31U: _LL51: _tmpB8=((struct Cyc_Absyn_Datatype_e_Absyn_Raw_exp_struct*)_tmpB0)->f1;_tmpB9=((struct Cyc_Absyn_Datatype_e_Absyn_Raw_exp_struct*)_tmpB0)->f2;_tmpBA=((struct Cyc_Absyn_Datatype_e_Absyn_Raw_exp_struct*)_tmpB0)->f3;_LL52: {struct Cyc_List_List*es=_tmpB8;struct Cyc_Absyn_Datatypedecl*tud=_tmpB9;struct Cyc_Absyn_Datatypefield*tuf=_tmpBA;
# 538
for(0;es != 0;es=es->tl){
Cyc_Binding_resolve_exp(env,(struct Cyc_Absyn_Exp*)es->hd);}
return;}case 30U: _LL53: _tmpB6=(void*)((struct Cyc_Absyn_AnonStruct_e_Absyn_Raw_exp_struct*)_tmpB0)->f1;_tmpB7=((struct Cyc_Absyn_AnonStruct_e_Absyn_Raw_exp_struct*)_tmpB0)->f2;_LL54: {void*t=_tmpB6;struct Cyc_List_List*dles=_tmpB7;
# 542
Cyc_Binding_resolve_type(e->loc,env,t);
for(0;dles != 0;dles=dles->tl){
Cyc_Binding_resolve_exp(env,(*((struct _tuple11*)dles->hd)).f2);}
return;}default: _LL55: _tmpB1=((struct Cyc_Absyn_Asm_e_Absyn_Raw_exp_struct*)_tmpB0)->f1;_tmpB2=((struct Cyc_Absyn_Asm_e_Absyn_Raw_exp_struct*)_tmpB0)->f2;_tmpB3=((struct Cyc_Absyn_Asm_e_Absyn_Raw_exp_struct*)_tmpB0)->f3;_tmpB4=((struct Cyc_Absyn_Asm_e_Absyn_Raw_exp_struct*)_tmpB0)->f4;_tmpB5=((struct Cyc_Absyn_Asm_e_Absyn_Raw_exp_struct*)_tmpB0)->f5;_LL56: {int v=_tmpB1;struct _fat_ptr t=_tmpB2;struct Cyc_List_List*o=_tmpB3;struct Cyc_List_List*i=_tmpB4;struct Cyc_List_List*c=_tmpB5;
# 548
Cyc_Binding_resolve_asm_iolist(env,o);
Cyc_Binding_resolve_asm_iolist(env,i);
goto _LL0;}}_LL0:;}struct _tuple12{struct _fat_ptr f1;struct Cyc_Absyn_Exp*f2;};
# 273
void Cyc_Binding_resolve_asm_iolist(struct Cyc_Binding_Env*env,struct Cyc_List_List*l){
# 555
while((unsigned)l){
struct _tuple12*_stmttmpE=(struct _tuple12*)l->hd;struct Cyc_Absyn_Exp*_tmp13F;_LL1: _tmp13F=_stmttmpE->f2;_LL2: {struct Cyc_Absyn_Exp*e=_tmp13F;
Cyc_Binding_resolve_exp(env,e);
l=l->tl;}}}
# 273
void Cyc_Binding_resolve_scs(struct Cyc_Binding_Env*env,struct Cyc_List_List*scs){
# 563
struct Cyc_Dict_Dict old_locals=*((struct Cyc_Dict_Dict*)_check_null(env->local_vars));
for(0;scs != 0;scs=scs->tl){
struct Cyc_Absyn_Switch_clause*_stmttmpF=(struct Cyc_Absyn_Switch_clause*)scs->hd;struct Cyc_Absyn_Stmt*_tmp142;struct Cyc_Absyn_Exp*_tmp141;struct Cyc_Absyn_Pat*_tmp140;_LL1: _tmp140=_stmttmpF->pattern;_tmp141=_stmttmpF->where_clause;_tmp142=_stmttmpF->body;_LL2: {struct Cyc_Absyn_Pat*pattern=_tmp140;struct Cyc_Absyn_Exp*where_clause=_tmp141;struct Cyc_Absyn_Stmt*body=_tmp142;
Cyc_Binding_resolve_pat(env,pattern);
if(where_clause != 0)
Cyc_Binding_resolve_exp(env,where_clause);
# 567
Cyc_Binding_resolve_stmt(env,body);
# 570
*((struct Cyc_Dict_Dict*)_check_null(env->local_vars))=old_locals;}}
# 572
return;}
# 273
void Cyc_Binding_resolve_aggrfields(unsigned loc,struct Cyc_Binding_Env*env,struct Cyc_List_List*fs){
# 575
for(0;fs != 0;fs=fs->tl){
struct Cyc_Absyn_Aggrfield*_stmttmp10=(struct Cyc_Absyn_Aggrfield*)fs->hd;struct Cyc_Absyn_Exp*_tmp148;struct Cyc_List_List*_tmp147;struct Cyc_Absyn_Exp*_tmp146;void*_tmp145;struct Cyc_Absyn_Tqual _tmp144;struct _fat_ptr*_tmp143;_LL1: _tmp143=_stmttmp10->name;_tmp144=_stmttmp10->tq;_tmp145=_stmttmp10->type;_tmp146=_stmttmp10->width;_tmp147=_stmttmp10->attributes;_tmp148=_stmttmp10->requires_clause;_LL2: {struct _fat_ptr*fn=_tmp143;struct Cyc_Absyn_Tqual tq=_tmp144;void*t=_tmp145;struct Cyc_Absyn_Exp*width=_tmp146;struct Cyc_List_List*atts=_tmp147;struct Cyc_Absyn_Exp*requires_clause=_tmp148;
Cyc_Binding_resolve_type(loc,env,t);
if(width != 0)
Cyc_Binding_resolve_exp(env,width);
# 578
if(requires_clause != 0)
# 581
Cyc_Binding_resolve_exp(env,requires_clause);}}
# 583
return;}
# 273
struct Cyc_List_List*Cyc_Binding_get_fun_vardecls(int need_va_name,unsigned loc,struct Cyc_Binding_Env*env,struct Cyc_List_List*args,struct Cyc_Absyn_VarargInfo*vai){
# 592
struct Cyc_List_List*vds=0;
for(0;args != 0;args=args->tl){
struct _tuple8*_stmttmp11=(struct _tuple8*)args->hd;void*_tmp14B;struct Cyc_Absyn_Tqual _tmp14A;struct _fat_ptr*_tmp149;_LL1: _tmp149=_stmttmp11->f1;_tmp14A=_stmttmp11->f2;_tmp14B=_stmttmp11->f3;_LL2: {struct _fat_ptr*a=_tmp149;struct Cyc_Absyn_Tqual tq=_tmp14A;void*t=_tmp14B;
if(a == 0)
continue;{
# 595
struct Cyc_Absyn_Vardecl*vd=({
# 597
struct _tuple0*_tmp362=({struct _tuple0*_tmp14D=_cycalloc(sizeof(*_tmp14D));_tmp14D->f1=Cyc_Absyn_Loc_n,_tmp14D->f2=a;_tmp14D;});Cyc_Absyn_new_vardecl(0U,_tmp362,t,0);});
vd->tq=tq;
vds=({struct Cyc_List_List*_tmp14C=_cycalloc(sizeof(*_tmp14C));_tmp14C->hd=vd,_tmp14C->tl=vds;_tmp14C;});}}}
# 601
if(vai != 0){
struct Cyc_Absyn_VarargInfo _stmttmp12=*vai;int _tmp151;void*_tmp150;struct Cyc_Absyn_Tqual _tmp14F;struct _fat_ptr*_tmp14E;_LL4: _tmp14E=_stmttmp12.name;_tmp14F=_stmttmp12.tq;_tmp150=_stmttmp12.type;_tmp151=_stmttmp12.inject;_LL5: {struct _fat_ptr*v=_tmp14E;struct Cyc_Absyn_Tqual tq=_tmp14F;void*t=_tmp150;int i=_tmp151;
if(v == 0){
if(need_va_name)
({struct Cyc_Warn_String_Warn_Warg_struct _tmp153=({struct Cyc_Warn_String_Warn_Warg_struct _tmp2C7;_tmp2C7.tag=0U,({struct _fat_ptr _tmp363=({const char*_tmp154="missing name for vararg";_tag_fat(_tmp154,sizeof(char),24U);});_tmp2C7.f1=_tmp363;});_tmp2C7;});void*_tmp152[1U];_tmp152[0]=& _tmp153;({unsigned _tmp364=loc;Cyc_Warn_err2(_tmp364,_tag_fat(_tmp152,sizeof(void*),1U));});});}else{
# 608
void*typ=Cyc_Absyn_fatptr_type(t,Cyc_Absyn_heap_rgn_type,tq,Cyc_Absyn_false_type);
struct Cyc_Absyn_Vardecl*vd=({struct _tuple0*_tmp365=({struct _tuple0*_tmp156=_cycalloc(sizeof(*_tmp156));_tmp156->f1=Cyc_Absyn_Loc_n,_tmp156->f2=v;_tmp156;});Cyc_Absyn_new_vardecl(0U,_tmp365,typ,0);});
vds=({struct Cyc_List_List*_tmp155=_cycalloc(sizeof(*_tmp155));_tmp155->hd=vd,_tmp155->tl=vds;_tmp155;});}}}
# 601
vds=
# 613
Cyc_List_imp_rev(vds);
return vds;}
# 273
void Cyc_Binding_resolve_function_stuff(unsigned loc,struct Cyc_Binding_Env*env,struct Cyc_Absyn_FnInfo i){
# 618
if(i.effect != 0)
Cyc_Binding_resolve_type(loc,env,i.effect);
# 618
Cyc_Binding_resolve_type(loc,env,i.ret_type);
# 621
{struct Cyc_List_List*args=i.args;for(0;args != 0;args=args->tl){
Cyc_Binding_resolve_type(loc,env,(*((struct _tuple8*)args->hd)).f3);}}
if(i.cyc_varargs != 0)
Cyc_Binding_resolve_type(loc,env,(i.cyc_varargs)->type);
# 623
Cyc_Binding_resolve_rgnpo(loc,env,i.rgn_po);
# 626
if(i.requires_clause != 0)
Cyc_Binding_resolve_exp(env,i.requires_clause);
# 626
if(i.ensures_clause != 0){
# 630
struct Cyc_Dict_Dict locs=*((struct Cyc_Dict_Dict*)_check_null(env->local_vars));
struct _fat_ptr*v=({struct _fat_ptr*_tmp15C=_cycalloc(sizeof(*_tmp15C));({struct _fat_ptr _tmp366=({const char*_tmp15B="return_value";_tag_fat(_tmp15B,sizeof(char),13U);});*_tmp15C=_tmp366;});_tmp15C;});
struct Cyc_Absyn_Vardecl*vd=({struct _tuple0*_tmp367=({struct _tuple0*_tmp15A=_cycalloc(sizeof(*_tmp15A));_tmp15A->f1=Cyc_Absyn_Loc_n,_tmp15A->f2=v;_tmp15A;});Cyc_Absyn_new_vardecl(0U,_tmp367,i.ret_type,0);});
({struct Cyc_Dict_Dict _tmp36C=({struct Cyc_Dict_Dict(*_tmp36B)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,void*v)=({struct Cyc_Dict_Dict(*_tmp157)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,void*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,void*v))Cyc_Dict_insert;_tmp157;});struct Cyc_Dict_Dict _tmp36A=*((struct Cyc_Dict_Dict*)_check_null(env->local_vars));struct _fat_ptr*_tmp369=v;_tmp36B(_tmp36A,_tmp369,(void*)({struct Cyc_Binding_VarRes_Binding_Resolved_struct*_tmp159=_cycalloc(sizeof(*_tmp159));
_tmp159->tag=0U,({void*_tmp368=(void*)({struct Cyc_Absyn_Local_b_Absyn_Binding_struct*_tmp158=_cycalloc(sizeof(*_tmp158));_tmp158->tag=4U,_tmp158->f1=vd;_tmp158;});_tmp159->f1=_tmp368;});_tmp159;}));});
# 633
*((struct Cyc_Dict_Dict*)_check_null(env->local_vars))=_tmp36C;});
# 635
Cyc_Binding_resolve_exp(env,i.ensures_clause);
*((struct Cyc_Dict_Dict*)_check_null(env->local_vars))=locs;}}struct _tuple13{struct Cyc_Absyn_Tqual f1;void*f2;};
# 273
void Cyc_Binding_resolve_type(unsigned loc,struct Cyc_Binding_Env*env,void*t){
# 646
void*_tmp15D=t;struct Cyc_Absyn_Exp*_tmp15E;struct Cyc_Absyn_Exp*_tmp15F;struct Cyc_Absyn_FnInfo _tmp160;struct Cyc_Absyn_Exp*_tmp162;void*_tmp161;struct Cyc_List_List*_tmp163;void*_tmp166;void*_tmp165;void*_tmp164;void*_tmp167;struct Cyc_List_List*_tmp168;void***_tmp16A;struct Cyc_Absyn_TypeDecl*_tmp169;struct Cyc_Absyn_Typedefdecl**_tmp16D;struct Cyc_List_List*_tmp16C;struct _tuple0**_tmp16B;struct Cyc_List_List*_tmp16F;void*_tmp16E;struct Cyc_List_List*_tmp170;struct Cyc_Absyn_Enumdecl*_tmp172;struct _tuple0**_tmp171;struct Cyc_List_List*_tmp174;union Cyc_Absyn_AggrInfo*_tmp173;struct Cyc_List_List*_tmp176;union Cyc_Absyn_DatatypeFieldInfo*_tmp175;struct Cyc_List_List*_tmp178;union Cyc_Absyn_DatatypeInfo*_tmp177;switch(*((int*)_tmp15D)){case 0U: switch(*((int*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp15D)->f1)){case 18U: _LL1: _tmp177=(union Cyc_Absyn_DatatypeInfo*)&((struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp15D)->f1)->f1;_tmp178=((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp15D)->f2;_LL2: {union Cyc_Absyn_DatatypeInfo*info=_tmp177;struct Cyc_List_List*targs=_tmp178;
# 648
for(0;targs != 0;targs=targs->tl){
Cyc_Binding_resolve_type(loc,env,(void*)targs->hd);}{
union Cyc_Absyn_DatatypeInfo _stmttmp13=*info;union Cyc_Absyn_DatatypeInfo _tmp179=_stmttmp13;int _tmp17B;struct _tuple0*_tmp17A;if((_tmp179.UnknownDatatype).tag == 1){_LL26: _tmp17A=((_tmp179.UnknownDatatype).val).name;_tmp17B=((_tmp179.UnknownDatatype).val).is_extensible;_LL27: {struct _tuple0*qv=_tmp17A;int b=_tmp17B;
# 652
struct _handler_cons _tmp17C;_push_handler(& _tmp17C);{int _tmp17E=0;if(setjmp(_tmp17C.handler))_tmp17E=1;if(!_tmp17E){
{struct Cyc_Absyn_Datatypedecl*tud=(struct Cyc_Absyn_Datatypedecl*)({struct Cyc_List_List*(*_tmp36F)(unsigned loc,struct Cyc_Binding_NSCtxt*ctxt,struct _tuple0*qv,struct Cyc_List_List*(*lookup)(struct Cyc_Binding_ResolveNSEnv*,struct _fat_ptr*))=({struct Cyc_List_List*(*_tmp17F)(unsigned loc,struct Cyc_Binding_NSCtxt*ctxt,struct _tuple0*qv,struct Cyc_List_List*(*lookup)(struct Cyc_Binding_ResolveNSEnv*,struct _fat_ptr*))=(struct Cyc_List_List*(*)(unsigned loc,struct Cyc_Binding_NSCtxt*ctxt,struct _tuple0*qv,struct Cyc_List_List*(*lookup)(struct Cyc_Binding_ResolveNSEnv*,struct _fat_ptr*)))Cyc_Binding_resolve_lookup;_tmp17F;});unsigned _tmp36E=loc;struct Cyc_Binding_NSCtxt*_tmp36D=env->ns;_tmp36F(_tmp36E,_tmp36D,qv,Cyc_Binding_lookup_datatypedecl);})->hd;
({union Cyc_Absyn_DatatypeInfo _tmp370=Cyc_Absyn_UnknownDatatype(({struct Cyc_Absyn_UnknownDatatypeInfo _tmp2C8;_tmp2C8.name=tud->name,_tmp2C8.is_extensible=b;_tmp2C8;}));*info=_tmp370;});
_npop_handler(0U);return;}
# 653
;_pop_handler();}else{void*_tmp17D=(void*)Cyc_Core_get_exn_thrown();void*_tmp180=_tmp17D;void*_tmp181;if(((struct Cyc_Binding_BindingError_exn_struct*)_tmp180)->tag == Cyc_Binding_BindingError){_LL2B: _LL2C:
# 657
 Cyc_Binding_absolutize_topdecl(loc,env,qv,Cyc_Absyn_Public);return;}else{_LL2D: _tmp181=_tmp180;_LL2E: {void*exn=_tmp181;(int)_rethrow(exn);}}_LL2A:;}}}}else{_LL28: _LL29:
 return;}_LL25:;}}case 19U: _LL3: _tmp175=(union Cyc_Absyn_DatatypeFieldInfo*)&((struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp15D)->f1)->f1;_tmp176=((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp15D)->f2;_LL4: {union Cyc_Absyn_DatatypeFieldInfo*info=_tmp175;struct Cyc_List_List*targs=_tmp176;
# 661
for(0;targs != 0;targs=targs->tl){
Cyc_Binding_resolve_type(loc,env,(void*)targs->hd);}{
union Cyc_Absyn_DatatypeFieldInfo _stmttmp14=*info;union Cyc_Absyn_DatatypeFieldInfo _tmp182=_stmttmp14;int _tmp185;struct _tuple0*_tmp184;struct _tuple0*_tmp183;if((_tmp182.UnknownDatatypefield).tag == 1){_LL30: _tmp183=((_tmp182.UnknownDatatypefield).val).datatype_name;_tmp184=((_tmp182.UnknownDatatypefield).val).field_name;_tmp185=((_tmp182.UnknownDatatypefield).val).is_extensible;_LL31: {struct _tuple0*qvd=_tmp183;struct _tuple0*qvf=_tmp184;int b=_tmp185;
# 667
{struct _handler_cons _tmp186;_push_handler(& _tmp186);{int _tmp188=0;if(setjmp(_tmp186.handler))_tmp188=1;if(!_tmp188){
{void*_stmttmp15=Cyc_Binding_lookup_ordinary(loc,env,qvf);void*_tmp189=_stmttmp15;struct Cyc_Absyn_Datatypefield*_tmp18B;struct Cyc_Absyn_Datatypedecl*_tmp18A;if(((struct Cyc_Binding_DatatypeRes_Binding_Resolved_struct*)_tmp189)->tag == 2U){_LL35: _tmp18A=((struct Cyc_Binding_DatatypeRes_Binding_Resolved_struct*)_tmp189)->f1;_tmp18B=((struct Cyc_Binding_DatatypeRes_Binding_Resolved_struct*)_tmp189)->f2;_LL36: {struct Cyc_Absyn_Datatypedecl*tud=_tmp18A;struct Cyc_Absyn_Datatypefield*tuf=_tmp18B;
# 670
struct Cyc_Absyn_Datatypedecl*tud2=(struct Cyc_Absyn_Datatypedecl*)({struct Cyc_List_List*(*_tmp373)(unsigned loc,struct Cyc_Binding_NSCtxt*ctxt,struct _tuple0*qv,struct Cyc_List_List*(*lookup)(struct Cyc_Binding_ResolveNSEnv*,struct _fat_ptr*))=({struct Cyc_List_List*(*_tmp196)(unsigned loc,struct Cyc_Binding_NSCtxt*ctxt,struct _tuple0*qv,struct Cyc_List_List*(*lookup)(struct Cyc_Binding_ResolveNSEnv*,struct _fat_ptr*))=(struct Cyc_List_List*(*)(unsigned loc,struct Cyc_Binding_NSCtxt*ctxt,struct _tuple0*qv,struct Cyc_List_List*(*lookup)(struct Cyc_Binding_ResolveNSEnv*,struct _fat_ptr*)))Cyc_Binding_resolve_lookup;_tmp196;});unsigned _tmp372=loc;struct Cyc_Binding_NSCtxt*_tmp371=env->ns;_tmp373(_tmp372,_tmp371,qvd,Cyc_Binding_lookup_datatypedecl);})->hd;
if(Cyc_Absyn_qvar_cmp(tud->name,tud2->name)!= 0){
({struct Cyc_Warn_Qvar_Warn_Warg_struct _tmp18D=({struct Cyc_Warn_Qvar_Warn_Warg_struct _tmp2CE;_tmp2CE.tag=3U,_tmp2CE.f1=tuf->name;_tmp2CE;});struct Cyc_Warn_String_Warn_Warg_struct _tmp18E=({struct Cyc_Warn_String_Warn_Warg_struct _tmp2CD;_tmp2CD.tag=0U,({struct _fat_ptr _tmp374=({const char*_tmp195=" is a variant of ";_tag_fat(_tmp195,sizeof(char),18U);});_tmp2CD.f1=_tmp374;});_tmp2CD;});struct Cyc_Warn_Qvar_Warn_Warg_struct _tmp18F=({struct Cyc_Warn_Qvar_Warn_Warg_struct _tmp2CC;_tmp2CC.tag=3U,_tmp2CC.f1=tud2->name;_tmp2CC;});struct Cyc_Warn_String_Warn_Warg_struct _tmp190=({struct Cyc_Warn_String_Warn_Warg_struct _tmp2CB;_tmp2CB.tag=0U,({struct _fat_ptr _tmp375=({const char*_tmp194=" not ";_tag_fat(_tmp194,sizeof(char),6U);});_tmp2CB.f1=_tmp375;});_tmp2CB;});struct Cyc_Warn_Qvar_Warn_Warg_struct _tmp191=({struct Cyc_Warn_Qvar_Warn_Warg_struct _tmp2CA;_tmp2CA.tag=3U,_tmp2CA.f1=tud->name;_tmp2CA;});struct Cyc_Warn_String_Warn_Warg_struct _tmp192=({struct Cyc_Warn_String_Warn_Warg_struct _tmp2C9;_tmp2C9.tag=0U,({
struct _fat_ptr _tmp376=({const char*_tmp193=" (shadowing not yet implemented properly)";_tag_fat(_tmp193,sizeof(char),42U);});_tmp2C9.f1=_tmp376;});_tmp2C9;});void*_tmp18C[6U];_tmp18C[0]=& _tmp18D,_tmp18C[1]=& _tmp18E,_tmp18C[2]=& _tmp18F,_tmp18C[3]=& _tmp190,_tmp18C[4]=& _tmp191,_tmp18C[5]=& _tmp192;({unsigned _tmp377=loc;Cyc_Warn_err2(_tmp377,_tag_fat(_tmp18C,sizeof(void*),6U));});});
_npop_handler(0U);return;}
# 671
({union Cyc_Absyn_DatatypeFieldInfo _tmp378=
# 676
Cyc_Absyn_UnknownDatatypefield(({struct Cyc_Absyn_UnknownDatatypeFieldInfo _tmp2CF;_tmp2CF.datatype_name=tud->name,_tmp2CF.field_name=tuf->name,_tmp2CF.is_extensible=b;_tmp2CF;}));
# 671
*info=_tmp378;});
# 678
_npop_handler(0U);return;}}else{_LL37: _LL38:
 goto _LL34;}_LL34:;}
# 668
;_pop_handler();}else{void*_tmp187=(void*)Cyc_Core_get_exn_thrown();void*_tmp197=_tmp187;void*_tmp198;if(((struct Cyc_Binding_BindingError_exn_struct*)_tmp197)->tag == Cyc_Binding_BindingError){_LL3A: _LL3B:
# 681
 goto _LL39;}else{_LL3C: _tmp198=_tmp197;_LL3D: {void*exn=_tmp198;(int)_rethrow(exn);}}_LL39:;}}}
({struct Cyc_Warn_Qvar_Warn_Warg_struct _tmp19A=({struct Cyc_Warn_Qvar_Warn_Warg_struct _tmp2D1;_tmp2D1.tag=3U,_tmp2D1.f1=qvf;_tmp2D1;});struct Cyc_Warn_String_Warn_Warg_struct _tmp19B=({struct Cyc_Warn_String_Warn_Warg_struct _tmp2D0;_tmp2D0.tag=0U,({struct _fat_ptr _tmp379=({const char*_tmp19C=" is not a datatype field";_tag_fat(_tmp19C,sizeof(char),25U);});_tmp2D0.f1=_tmp379;});_tmp2D0;});void*_tmp199[2U];_tmp199[0]=& _tmp19A,_tmp199[1]=& _tmp19B;({unsigned _tmp37A=loc;Cyc_Warn_err2(_tmp37A,_tag_fat(_tmp199,sizeof(void*),2U));});});return;}}else{_LL32: _LL33:
 return;}_LL2F:;}}case 20U: _LL5: _tmp173=(union Cyc_Absyn_AggrInfo*)&((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp15D)->f1)->f1;_tmp174=((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp15D)->f2;_LL6: {union Cyc_Absyn_AggrInfo*info=_tmp173;struct Cyc_List_List*targs=_tmp174;
# 687
for(0;targs != 0;targs=targs->tl){
Cyc_Binding_resolve_type(loc,env,(void*)targs->hd);}{
union Cyc_Absyn_AggrInfo _stmttmp16=*info;union Cyc_Absyn_AggrInfo _tmp19D=_stmttmp16;struct Cyc_Core_Opt*_tmp1A0;struct _tuple0*_tmp19F;enum Cyc_Absyn_AggrKind _tmp19E;if((_tmp19D.UnknownAggr).tag == 1){_LL3F: _tmp19E=((_tmp19D.UnknownAggr).val).f1;_tmp19F=((_tmp19D.UnknownAggr).val).f2;_tmp1A0=((_tmp19D.UnknownAggr).val).f3;_LL40: {enum Cyc_Absyn_AggrKind ak=_tmp19E;struct _tuple0*qv=_tmp19F;struct Cyc_Core_Opt*bo=_tmp1A0;
# 691
struct _handler_cons _tmp1A1;_push_handler(& _tmp1A1);{int _tmp1A3=0;if(setjmp(_tmp1A1.handler))_tmp1A3=1;if(!_tmp1A3){
{struct Cyc_Absyn_Aggrdecl*ad=({struct Cyc_Absyn_Aggrdecl*(*_tmp37D)(unsigned loc,struct Cyc_Binding_NSCtxt*ctxt,struct _tuple0*qv,struct Cyc_Absyn_Aggrdecl*(*lookup)(struct Cyc_Binding_ResolveNSEnv*,struct _fat_ptr*))=({struct Cyc_Absyn_Aggrdecl*(*_tmp1AA)(unsigned loc,struct Cyc_Binding_NSCtxt*ctxt,struct _tuple0*qv,struct Cyc_Absyn_Aggrdecl*(*lookup)(struct Cyc_Binding_ResolveNSEnv*,struct _fat_ptr*))=(struct Cyc_Absyn_Aggrdecl*(*)(unsigned loc,struct Cyc_Binding_NSCtxt*ctxt,struct _tuple0*qv,struct Cyc_Absyn_Aggrdecl*(*lookup)(struct Cyc_Binding_ResolveNSEnv*,struct _fat_ptr*)))Cyc_Binding_resolve_lookup;_tmp1AA;});unsigned _tmp37C=loc;struct Cyc_Binding_NSCtxt*_tmp37B=env->ns;_tmp37D(_tmp37C,_tmp37B,qv,Cyc_Binding_lookup_aggrdecl);});
if((int)ad->kind != (int)ak)
({struct Cyc_Warn_String_Warn_Warg_struct _tmp1A5=({struct Cyc_Warn_String_Warn_Warg_struct _tmp2D2;_tmp2D2.tag=0U,({struct _fat_ptr _tmp37E=({const char*_tmp1A6="struct vs. union mismatch with earlier declaration";_tag_fat(_tmp1A6,sizeof(char),51U);});_tmp2D2.f1=_tmp37E;});_tmp2D2;});void*_tmp1A4[1U];_tmp1A4[0]=& _tmp1A5;({unsigned _tmp37F=loc;Cyc_Warn_err2(_tmp37F,_tag_fat(_tmp1A4,sizeof(void*),1U));});});
# 693
if(
# 695
(((int)ak == (int)1U && bo != 0)&& ad->impl != 0)&&((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->tagged != (int)bo->v)
# 697
({struct Cyc_Warn_String_Warn_Warg_struct _tmp1A8=({struct Cyc_Warn_String_Warn_Warg_struct _tmp2D3;_tmp2D3.tag=0U,({struct _fat_ptr _tmp380=({const char*_tmp1A9="@tagged mismatch with earlier declaration";_tag_fat(_tmp1A9,sizeof(char),42U);});_tmp2D3.f1=_tmp380;});_tmp2D3;});void*_tmp1A7[1U];_tmp1A7[0]=& _tmp1A8;({unsigned _tmp381=loc;Cyc_Warn_err2(_tmp381,_tag_fat(_tmp1A7,sizeof(void*),1U));});});
# 693
({union Cyc_Absyn_AggrInfo _tmp382=
# 698
Cyc_Absyn_UnknownAggr(ak,ad->name,bo);
# 693
*info=_tmp382;});
# 699
_npop_handler(0U);return;}
# 692
;_pop_handler();}else{void*_tmp1A2=(void*)Cyc_Core_get_exn_thrown();void*_tmp1AB=_tmp1A2;void*_tmp1AC;if(((struct Cyc_Binding_BindingError_exn_struct*)_tmp1AB)->tag == Cyc_Binding_BindingError){_LL44: _LL45:
# 701
 Cyc_Binding_absolutize_topdecl(loc,env,qv,Cyc_Absyn_Public);return;}else{_LL46: _tmp1AC=_tmp1AB;_LL47: {void*exn=_tmp1AC;(int)_rethrow(exn);}}_LL43:;}}}}else{_LL41: _LL42:
 return;}_LL3E:;}}case 15U: _LL7: _tmp171=(struct _tuple0**)&((struct Cyc_Absyn_EnumCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp15D)->f1)->f1;_tmp172=((struct Cyc_Absyn_EnumCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp15D)->f1)->f2;_LL8: {struct _tuple0**qv=_tmp171;struct Cyc_Absyn_Enumdecl*edo=_tmp172;
# 705
if(edo != 0)
return;{
# 705
struct _handler_cons _tmp1AD;_push_handler(& _tmp1AD);{int _tmp1AF=0;if(setjmp(_tmp1AD.handler))_tmp1AF=1;if(!_tmp1AF){
# 708
{struct Cyc_Absyn_Enumdecl*ed=({struct Cyc_Absyn_Enumdecl*(*_tmp385)(unsigned loc,struct Cyc_Binding_NSCtxt*ctxt,struct _tuple0*qv,struct Cyc_Absyn_Enumdecl*(*lookup)(struct Cyc_Binding_ResolveNSEnv*,struct _fat_ptr*))=({struct Cyc_Absyn_Enumdecl*(*_tmp1B0)(unsigned loc,struct Cyc_Binding_NSCtxt*ctxt,struct _tuple0*qv,struct Cyc_Absyn_Enumdecl*(*lookup)(struct Cyc_Binding_ResolveNSEnv*,struct _fat_ptr*))=(struct Cyc_Absyn_Enumdecl*(*)(unsigned loc,struct Cyc_Binding_NSCtxt*ctxt,struct _tuple0*qv,struct Cyc_Absyn_Enumdecl*(*lookup)(struct Cyc_Binding_ResolveNSEnv*,struct _fat_ptr*)))Cyc_Binding_resolve_lookup;_tmp1B0;});unsigned _tmp384=loc;struct Cyc_Binding_NSCtxt*_tmp383=env->ns;_tmp385(_tmp384,_tmp383,*qv,Cyc_Binding_lookup_enumdecl);});
*qv=ed->name;
_npop_handler(0U);return;}
# 708
;_pop_handler();}else{void*_tmp1AE=(void*)Cyc_Core_get_exn_thrown();void*_tmp1B1=_tmp1AE;void*_tmp1B2;if(((struct Cyc_Binding_BindingError_exn_struct*)_tmp1B1)->tag == Cyc_Binding_BindingError){_LL49: _LL4A:
# 712
 Cyc_Binding_absolutize_topdecl(loc,env,*qv,Cyc_Absyn_Public);return;}else{_LL4B: _tmp1B2=_tmp1B1;_LL4C: {void*exn=_tmp1B2;(int)_rethrow(exn);}}_LL48:;}}}}case 16U: _LLF: _tmp170=((struct Cyc_Absyn_AnonEnumCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp15D)->f1)->f1;_LL10: {struct Cyc_List_List*fs=_tmp170;
# 750
struct Cyc_Binding_ResolveNSEnv*decl_ns_data=({struct Cyc_Binding_ResolveNSEnv*(*_tmp388)(struct Cyc_Binding_NSCtxt*ctxt,union Cyc_Absyn_Nmspace abs_ns)=({struct Cyc_Binding_ResolveNSEnv*(*_tmp1C9)(struct Cyc_Binding_NSCtxt*ctxt,union Cyc_Absyn_Nmspace abs_ns)=(struct Cyc_Binding_ResolveNSEnv*(*)(struct Cyc_Binding_NSCtxt*ctxt,union Cyc_Absyn_Nmspace abs_ns))Cyc_Binding_get_ns_data;_tmp1C9;});struct Cyc_Binding_NSCtxt*_tmp387=env->ns;_tmp388(_tmp387,({struct Cyc_List_List*_tmp386=(env->ns)->curr_ns;Cyc_Absyn_Abs_n(_tmp386,
Cyc_Binding_in_cinclude(env));}));});
for(0;fs != 0;fs=fs->tl){
struct Cyc_Absyn_Enumfield*f=(struct Cyc_Absyn_Enumfield*)fs->hd;
if(f->tag != 0)
Cyc_Binding_resolve_exp(env,(struct Cyc_Absyn_Exp*)_check_null(f->tag));
# 754
({struct Cyc_Dict_Dict _tmp38C=({
# 756
struct Cyc_Dict_Dict(*_tmp38B)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,void*v)=({struct Cyc_Dict_Dict(*_tmp1C7)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,void*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,void*v))Cyc_Dict_insert;_tmp1C7;});struct Cyc_Dict_Dict _tmp38A=decl_ns_data->ordinaries;struct _fat_ptr*_tmp389=(*f->name).f2;_tmp38B(_tmp38A,_tmp389,(void*)({struct Cyc_Binding_AnonEnumRes_Binding_Resolved_struct*_tmp1C8=_cycalloc(sizeof(*_tmp1C8));_tmp1C8->tag=4U,_tmp1C8->f1=t,_tmp1C8->f2=f;_tmp1C8;}));});
# 754
decl_ns_data->ordinaries=_tmp38C;});}
# 758
return;}default: if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp15D)->f2 == 0){_LL15: _LL16:
# 762
 return;}else{_LL17: _tmp16E=(void*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp15D)->f1;_tmp16F=((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp15D)->f2;_LL18: {void*c=_tmp16E;struct Cyc_List_List*ts=_tmp16F;
# 764
for(0;ts != 0;ts=ts->tl){
Cyc_Binding_resolve_type(loc,env,(void*)ts->hd);}
return;}}}case 8U: _LL9: _tmp16B=(struct _tuple0**)&((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmp15D)->f1;_tmp16C=((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmp15D)->f2;_tmp16D=(struct Cyc_Absyn_Typedefdecl**)&((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmp15D)->f3;_LLA: {struct _tuple0**tdn=_tmp16B;struct Cyc_List_List*targs=_tmp16C;struct Cyc_Absyn_Typedefdecl**tdo=_tmp16D;
# 715
for(0;targs != 0;targs=targs->tl){
Cyc_Binding_resolve_type(loc,env,(void*)targs->hd);}
{struct _handler_cons _tmp1B3;_push_handler(& _tmp1B3);{int _tmp1B5=0;if(setjmp(_tmp1B3.handler))_tmp1B5=1;if(!_tmp1B5){
{struct Cyc_Absyn_Typedefdecl*td=({struct Cyc_Absyn_Typedefdecl*(*_tmp38F)(unsigned loc,struct Cyc_Binding_NSCtxt*ctxt,struct _tuple0*qv,struct Cyc_Absyn_Typedefdecl*(*lookup)(struct Cyc_Binding_ResolveNSEnv*,struct _fat_ptr*))=({struct Cyc_Absyn_Typedefdecl*(*_tmp1B6)(unsigned loc,struct Cyc_Binding_NSCtxt*ctxt,struct _tuple0*qv,struct Cyc_Absyn_Typedefdecl*(*lookup)(struct Cyc_Binding_ResolveNSEnv*,struct _fat_ptr*))=(struct Cyc_Absyn_Typedefdecl*(*)(unsigned loc,struct Cyc_Binding_NSCtxt*ctxt,struct _tuple0*qv,struct Cyc_Absyn_Typedefdecl*(*lookup)(struct Cyc_Binding_ResolveNSEnv*,struct _fat_ptr*)))Cyc_Binding_resolve_lookup;_tmp1B6;});unsigned _tmp38E=loc;struct Cyc_Binding_NSCtxt*_tmp38D=env->ns;_tmp38F(_tmp38E,_tmp38D,*tdn,Cyc_Binding_lookup_typedefdecl);});
# 720
*tdn=td->name;
_npop_handler(0U);return;}
# 718
;_pop_handler();}else{void*_tmp1B4=(void*)Cyc_Core_get_exn_thrown();void*_tmp1B7=_tmp1B4;void*_tmp1B8;if(((struct Cyc_Binding_BindingError_exn_struct*)_tmp1B7)->tag == Cyc_Binding_BindingError){_LL4E: _LL4F:
# 722
 goto _LL4D;}else{_LL50: _tmp1B8=_tmp1B7;_LL51: {void*exn=_tmp1B8;(int)_rethrow(exn);}}_LL4D:;}}}
({struct Cyc_Warn_String_Warn_Warg_struct _tmp1BA=({struct Cyc_Warn_String_Warn_Warg_struct _tmp2D5;_tmp2D5.tag=0U,({struct _fat_ptr _tmp390=({const char*_tmp1BC="unbound typdef name ";_tag_fat(_tmp1BC,sizeof(char),21U);});_tmp2D5.f1=_tmp390;});_tmp2D5;});struct Cyc_Warn_Qvar_Warn_Warg_struct _tmp1BB=({struct Cyc_Warn_Qvar_Warn_Warg_struct _tmp2D4;_tmp2D4.tag=3U,_tmp2D4.f1=*tdn;_tmp2D4;});void*_tmp1B9[2U];_tmp1B9[0]=& _tmp1BA,_tmp1B9[1]=& _tmp1BB;({unsigned _tmp391=loc;Cyc_Warn_err2(_tmp391,_tag_fat(_tmp1B9,sizeof(void*),2U));});});
return;}case 10U: _LLB: _tmp169=((struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct*)_tmp15D)->f1;_tmp16A=(void***)&((struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct*)_tmp15D)->f2;_LLC: {struct Cyc_Absyn_TypeDecl*td=_tmp169;void***to=_tmp16A;
# 728
struct Cyc_Dict_Dict*old_locals=env->local_vars;
env->local_vars=0;
{void*_stmttmp17=td->r;void*_tmp1BD=_stmttmp17;struct Cyc_Absyn_Datatypedecl*_tmp1BE;struct Cyc_Absyn_Enumdecl*_tmp1BF;struct Cyc_Absyn_Aggrdecl*_tmp1C0;switch(*((int*)_tmp1BD)){case 0U: _LL53: _tmp1C0=((struct Cyc_Absyn_Aggr_td_Absyn_Raw_typedecl_struct*)_tmp1BD)->f1;_LL54: {struct Cyc_Absyn_Aggrdecl*ad=_tmp1C0;
# 732
({struct Cyc_Binding_Env*_tmp393=env;Cyc_Binding_resolve_decl(_tmp393,({struct Cyc_Absyn_Decl*_tmp1C2=_cycalloc(sizeof(*_tmp1C2));({void*_tmp392=(void*)({struct Cyc_Absyn_Aggr_d_Absyn_Raw_decl_struct*_tmp1C1=_cycalloc(sizeof(*_tmp1C1));_tmp1C1->tag=5U,_tmp1C1->f1=ad;_tmp1C1;});_tmp1C2->r=_tmp392;}),_tmp1C2->loc=td->loc;_tmp1C2;}));});goto _LL52;}case 1U: _LL55: _tmp1BF=((struct Cyc_Absyn_Enum_td_Absyn_Raw_typedecl_struct*)_tmp1BD)->f1;_LL56: {struct Cyc_Absyn_Enumdecl*ed=_tmp1BF;
# 734
({struct Cyc_Binding_Env*_tmp395=env;Cyc_Binding_resolve_decl(_tmp395,({struct Cyc_Absyn_Decl*_tmp1C4=_cycalloc(sizeof(*_tmp1C4));({void*_tmp394=(void*)({struct Cyc_Absyn_Enum_d_Absyn_Raw_decl_struct*_tmp1C3=_cycalloc(sizeof(*_tmp1C3));_tmp1C3->tag=7U,_tmp1C3->f1=ed;_tmp1C3;});_tmp1C4->r=_tmp394;}),_tmp1C4->loc=td->loc;_tmp1C4;}));});goto _LL52;}default: _LL57: _tmp1BE=((struct Cyc_Absyn_Datatype_td_Absyn_Raw_typedecl_struct*)_tmp1BD)->f1;_LL58: {struct Cyc_Absyn_Datatypedecl*tud=_tmp1BE;
# 736
({struct Cyc_Binding_Env*_tmp397=env;Cyc_Binding_resolve_decl(_tmp397,({struct Cyc_Absyn_Decl*_tmp1C6=_cycalloc(sizeof(*_tmp1C6));({void*_tmp396=(void*)({struct Cyc_Absyn_Datatype_d_Absyn_Raw_decl_struct*_tmp1C5=_cycalloc(sizeof(*_tmp1C5));_tmp1C5->tag=6U,_tmp1C5->f1=tud;_tmp1C5;});_tmp1C6->r=_tmp396;}),_tmp1C6->loc=td->loc;_tmp1C6;}));});goto _LL52;}}_LL52:;}
# 738
env->local_vars=old_locals;
return;}case 7U: _LLD: _tmp168=((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmp15D)->f2;_LLE: {struct Cyc_List_List*fs=_tmp168;
# 745
Cyc_Binding_resolve_aggrfields(loc,env,fs);
return;}case 1U: _LL11: _tmp167=(void*)((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp15D)->f2;_LL12: {void*to=_tmp167;
# 759
if(to != 0)Cyc_Binding_resolve_type(loc,env,to);return;}case 2U: _LL13: _LL14:
# 761
 goto _LL16;case 3U: _LL19: _tmp164=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp15D)->f1).elt_type;_tmp165=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp15D)->f1).ptr_atts).rgn;_tmp166=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp15D)->f1).ptr_atts).bounds;_LL1A: {void*t1=_tmp164;void*t2=_tmp165;void*bds=_tmp166;
# 769
Cyc_Binding_resolve_type(loc,env,t1);
Cyc_Binding_resolve_type(loc,env,t2);
Cyc_Binding_resolve_type(loc,env,bds);
return;}case 6U: _LL1B: _tmp163=((struct Cyc_Absyn_TupleType_Absyn_Type_struct*)_tmp15D)->f1;_LL1C: {struct Cyc_List_List*tqts=_tmp163;
# 775
for(0;tqts != 0;tqts=tqts->tl){
Cyc_Binding_resolve_type(loc,env,(*((struct _tuple13*)tqts->hd)).f2);}
return;}case 4U: _LL1D: _tmp161=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp15D)->f1).elt_type;_tmp162=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp15D)->f1).num_elts;_LL1E: {void*t1=_tmp161;struct Cyc_Absyn_Exp*eo=_tmp162;
# 779
Cyc_Binding_resolve_type(loc,env,t1);
if(eo != 0)Cyc_Binding_resolve_exp(env,eo);return;}case 5U: _LL1F: _tmp160=((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp15D)->f1;_LL20: {struct Cyc_Absyn_FnInfo i=_tmp160;
# 786
struct Cyc_List_List*vds=Cyc_Binding_get_fun_vardecls(0,loc,env,i.args,i.cyc_varargs);
# 788
struct Cyc_Dict_Dict*old_locals=env->local_vars;
if(old_locals != 0)
({struct Cyc_Dict_Dict*_tmp398=({struct Cyc_Dict_Dict*_tmp1CA=_cycalloc(sizeof(*_tmp1CA));*_tmp1CA=*((struct Cyc_Dict_Dict*)_check_null(env->local_vars));_tmp1CA;});env->local_vars=_tmp398;});else{
# 792
({struct Cyc_Dict_Dict*_tmp39A=({struct Cyc_Dict_Dict*_tmp1CC=_cycalloc(sizeof(*_tmp1CC));({struct Cyc_Dict_Dict _tmp399=({struct Cyc_Dict_Dict(*_tmp1CB)(int(*cmp)(struct _fat_ptr*,struct _fat_ptr*))=(struct Cyc_Dict_Dict(*)(int(*cmp)(struct _fat_ptr*,struct _fat_ptr*)))Cyc_Dict_empty;_tmp1CB;})(Cyc_strptrcmp);*_tmp1CC=_tmp399;});_tmp1CC;});env->local_vars=_tmp39A;});}
{struct Cyc_List_List*vds1=vds;for(0;vds1 != 0;vds1=vds1->tl){
({struct Cyc_Dict_Dict _tmp39F=({struct Cyc_Dict_Dict(*_tmp39E)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,void*v)=({struct Cyc_Dict_Dict(*_tmp1CD)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,void*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,void*v))Cyc_Dict_insert;_tmp1CD;});struct Cyc_Dict_Dict _tmp39D=*((struct Cyc_Dict_Dict*)_check_null(env->local_vars));struct _fat_ptr*_tmp39C=(*((struct Cyc_Absyn_Vardecl*)vds1->hd)->name).f2;_tmp39E(_tmp39D,_tmp39C,(void*)({struct Cyc_Binding_VarRes_Binding_Resolved_struct*_tmp1CF=_cycalloc(sizeof(*_tmp1CF));
_tmp1CF->tag=0U,({void*_tmp39B=(void*)({struct Cyc_Absyn_Param_b_Absyn_Binding_struct*_tmp1CE=_cycalloc(sizeof(*_tmp1CE));_tmp1CE->tag=3U,_tmp1CE->f1=(struct Cyc_Absyn_Vardecl*)vds1->hd;_tmp1CE;});_tmp1CF->f1=_tmp39B;});_tmp1CF;}));});
# 794
*((struct Cyc_Dict_Dict*)_check_null(env->local_vars))=_tmp39F;});}}
# 797
Cyc_Binding_resolve_function_stuff(loc,env,i);
env->local_vars=old_locals;
return;}case 9U: _LL21: _tmp15F=((struct Cyc_Absyn_ValueofType_Absyn_Type_struct*)_tmp15D)->f1;_LL22: {struct Cyc_Absyn_Exp*e=_tmp15F;
# 801
_tmp15E=e;goto _LL24;}default: _LL23: _tmp15E=((struct Cyc_Absyn_TypeofType_Absyn_Type_struct*)_tmp15D)->f1;_LL24: {struct Cyc_Absyn_Exp*e=_tmp15E;
Cyc_Binding_resolve_exp(env,e);return;}}_LL0:;}struct _tuple14{struct Cyc_List_List*f1;struct Cyc_Absyn_Pat*f2;};
# 273
void Cyc_Binding_resolve_pat(struct Cyc_Binding_Env*env,struct Cyc_Absyn_Pat*p){
# 807
unsigned _tmp1D1;void*_tmp1D0;_LL1: _tmp1D0=p->r;_tmp1D1=p->loc;_LL2: {void*r=_tmp1D0;unsigned loc=_tmp1D1;
void*_tmp1D2=r;struct Cyc_List_List*_tmp1D3;struct Cyc_Absyn_Exp*_tmp1D4;struct Cyc_List_List*_tmp1D5;struct Cyc_Absyn_Pat*_tmp1D6;struct Cyc_Absyn_Vardecl*_tmp1D7;struct Cyc_Absyn_Vardecl*_tmp1D8;struct Cyc_Absyn_Pat*_tmp1DA;struct Cyc_Absyn_Vardecl*_tmp1D9;struct Cyc_Absyn_Pat*_tmp1DC;struct Cyc_Absyn_Vardecl*_tmp1DB;struct Cyc_List_List*_tmp1DD;struct Cyc_List_List*_tmp1DE;int _tmp1E2;struct Cyc_List_List*_tmp1E1;struct Cyc_List_List*_tmp1E0;struct _tuple0*_tmp1DF;int _tmp1E5;struct Cyc_List_List*_tmp1E4;struct _tuple0*_tmp1E3;struct _tuple0*_tmp1E6;switch(*((int*)_tmp1D2)){case 15U: _LL4: _tmp1E6=((struct Cyc_Absyn_UnknownId_p_Absyn_Raw_pat_struct*)_tmp1D2)->f1;_LL5: {struct _tuple0*qv=_tmp1E6;
# 810
{struct _handler_cons _tmp1E7;_push_handler(& _tmp1E7);{int _tmp1E9=0;if(setjmp(_tmp1E7.handler))_tmp1E9=1;if(!_tmp1E9){{void*_stmttmp18=Cyc_Binding_lookup_ordinary(loc,env,qv);void*_tmp1EA=_stmttmp18;struct Cyc_Absyn_Enumfield*_tmp1EC;void*_tmp1EB;struct Cyc_Absyn_Enumfield*_tmp1EE;struct Cyc_Absyn_Enumdecl*_tmp1ED;struct Cyc_Absyn_Datatypefield*_tmp1F0;struct Cyc_Absyn_Datatypedecl*_tmp1EF;switch(*((int*)_tmp1EA)){case 0U: _LL2D: _LL2E:
 goto _LL2C;case 1U: _LL2F: _LL30:
# 813
({struct Cyc_Warn_String_Warn_Warg_struct _tmp1F2=({struct Cyc_Warn_String_Warn_Warg_struct _tmp2D6;_tmp2D6.tag=0U,({struct _fat_ptr _tmp3A0=({const char*_tmp1F3="struct tag used without arguments in pattern";_tag_fat(_tmp1F3,sizeof(char),45U);});_tmp2D6.f1=_tmp3A0;});_tmp2D6;});void*_tmp1F1[1U];_tmp1F1[0]=& _tmp1F2;({unsigned _tmp3A1=loc;Cyc_Warn_err2(_tmp3A1,_tag_fat(_tmp1F1,sizeof(void*),1U));});});_npop_handler(0U);return;case 2U: _LL31: _tmp1EF=((struct Cyc_Binding_DatatypeRes_Binding_Resolved_struct*)_tmp1EA)->f1;_tmp1F0=((struct Cyc_Binding_DatatypeRes_Binding_Resolved_struct*)_tmp1EA)->f2;_LL32: {struct Cyc_Absyn_Datatypedecl*tud=_tmp1EF;struct Cyc_Absyn_Datatypefield*tuf=_tmp1F0;
# 815
({void*_tmp3A2=(void*)({struct Cyc_Absyn_Datatype_p_Absyn_Raw_pat_struct*_tmp1F4=_cycalloc(sizeof(*_tmp1F4));_tmp1F4->tag=8U,_tmp1F4->f1=tud,_tmp1F4->f2=tuf,_tmp1F4->f3=0,_tmp1F4->f4=0;_tmp1F4;});p->r=_tmp3A2;});_npop_handler(0U);return;}case 3U: _LL33: _tmp1ED=((struct Cyc_Binding_EnumRes_Binding_Resolved_struct*)_tmp1EA)->f1;_tmp1EE=((struct Cyc_Binding_EnumRes_Binding_Resolved_struct*)_tmp1EA)->f2;_LL34: {struct Cyc_Absyn_Enumdecl*ed=_tmp1ED;struct Cyc_Absyn_Enumfield*ef=_tmp1EE;
# 817
({void*_tmp3A3=(void*)({struct Cyc_Absyn_Enum_p_Absyn_Raw_pat_struct*_tmp1F5=_cycalloc(sizeof(*_tmp1F5));_tmp1F5->tag=13U,_tmp1F5->f1=ed,_tmp1F5->f2=ef;_tmp1F5;});p->r=_tmp3A3;});_npop_handler(0U);return;}default: _LL35: _tmp1EB=(void*)((struct Cyc_Binding_AnonEnumRes_Binding_Resolved_struct*)_tmp1EA)->f1;_tmp1EC=((struct Cyc_Binding_AnonEnumRes_Binding_Resolved_struct*)_tmp1EA)->f2;_LL36: {void*t=_tmp1EB;struct Cyc_Absyn_Enumfield*ef=_tmp1EC;
# 819
({void*_tmp3A4=(void*)({struct Cyc_Absyn_AnonEnum_p_Absyn_Raw_pat_struct*_tmp1F6=_cycalloc(sizeof(*_tmp1F6));_tmp1F6->tag=14U,_tmp1F6->f1=t,_tmp1F6->f2=ef;_tmp1F6;});p->r=_tmp3A4;});_npop_handler(0U);return;}}_LL2C:;}
# 810
;_pop_handler();}else{void*_tmp1E8=(void*)Cyc_Core_get_exn_thrown();void*_tmp1F7=_tmp1E8;void*_tmp1F8;if(((struct Cyc_Binding_BindingError_exn_struct*)_tmp1F7)->tag == Cyc_Binding_BindingError){_LL38: _LL39:
# 820
 goto _LL37;}else{_LL3A: _tmp1F8=_tmp1F7;_LL3B: {void*exn=_tmp1F8;(int)_rethrow(exn);}}_LL37:;}}}{
struct Cyc_Absyn_Vardecl*vd=Cyc_Absyn_new_vardecl(0U,qv,Cyc_Absyn_void_type,0);
Cyc_Binding_resolve_and_add_vardecl(loc,env,vd);
({void*_tmp3A6=(void*)({struct Cyc_Absyn_Var_p_Absyn_Raw_pat_struct*_tmp1F9=_cycalloc(sizeof(*_tmp1F9));_tmp1F9->tag=1U,_tmp1F9->f1=vd,({struct Cyc_Absyn_Pat*_tmp3A5=Cyc_Absyn_new_pat((void*)& Cyc_Absyn_Wild_p_val,0U);_tmp1F9->f2=_tmp3A5;});_tmp1F9;});p->r=_tmp3A6;});
return;}}case 16U: _LL6: _tmp1E3=((struct Cyc_Absyn_UnknownCall_p_Absyn_Raw_pat_struct*)_tmp1D2)->f1;_tmp1E4=((struct Cyc_Absyn_UnknownCall_p_Absyn_Raw_pat_struct*)_tmp1D2)->f2;_tmp1E5=((struct Cyc_Absyn_UnknownCall_p_Absyn_Raw_pat_struct*)_tmp1D2)->f3;_LL7: {struct _tuple0*qv=_tmp1E3;struct Cyc_List_List*ps=_tmp1E4;int dots=_tmp1E5;
# 827
{struct Cyc_List_List*ps2=ps;for(0;ps2 != 0;ps2=ps2->tl){
Cyc_Binding_resolve_pat(env,(struct Cyc_Absyn_Pat*)ps2->hd);}}
{struct _handler_cons _tmp1FA;_push_handler(& _tmp1FA);{int _tmp1FC=0;if(setjmp(_tmp1FA.handler))_tmp1FC=1;if(!_tmp1FC){{void*_stmttmp19=Cyc_Binding_lookup_ordinary(loc,env,qv);void*_tmp1FD=_stmttmp19;struct Cyc_Absyn_Datatypefield*_tmp1FF;struct Cyc_Absyn_Datatypedecl*_tmp1FE;struct Cyc_Absyn_Aggrdecl*_tmp200;switch(*((int*)_tmp1FD)){case 0U: _LL3D: _LL3E:
 goto _LL3C;case 1U: _LL3F: _tmp200=((struct Cyc_Binding_AggrRes_Binding_Resolved_struct*)_tmp1FD)->f1;_LL40: {struct Cyc_Absyn_Aggrdecl*ad=_tmp200;
# 832
struct Cyc_List_List*new_ps=0;
for(0;ps != 0;ps=ps->tl){
new_ps=({struct Cyc_List_List*_tmp202=_cycalloc(sizeof(*_tmp202));({struct _tuple14*_tmp3A7=({struct _tuple14*_tmp201=_cycalloc(sizeof(*_tmp201));_tmp201->f1=0,_tmp201->f2=(struct Cyc_Absyn_Pat*)ps->hd;_tmp201;});_tmp202->hd=_tmp3A7;}),_tmp202->tl=new_ps;_tmp202;});}
({void*_tmp3AB=(void*)({struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*_tmp205=_cycalloc(sizeof(*_tmp205));_tmp205->tag=7U,({union Cyc_Absyn_AggrInfo*_tmp3AA=({union Cyc_Absyn_AggrInfo*_tmp204=_cycalloc(sizeof(*_tmp204));({union Cyc_Absyn_AggrInfo _tmp3A9=Cyc_Absyn_KnownAggr(({struct Cyc_Absyn_Aggrdecl**_tmp203=_cycalloc(sizeof(*_tmp203));*_tmp203=ad;_tmp203;}));*_tmp204=_tmp3A9;});_tmp204;});_tmp205->f1=_tmp3AA;}),_tmp205->f2=0,({struct Cyc_List_List*_tmp3A8=Cyc_List_imp_rev(new_ps);_tmp205->f3=_tmp3A8;}),_tmp205->f4=dots;_tmp205;});p->r=_tmp3AB;});
_npop_handler(0U);return;}case 2U: _LL41: _tmp1FE=((struct Cyc_Binding_DatatypeRes_Binding_Resolved_struct*)_tmp1FD)->f1;_tmp1FF=((struct Cyc_Binding_DatatypeRes_Binding_Resolved_struct*)_tmp1FD)->f2;_LL42: {struct Cyc_Absyn_Datatypedecl*tud=_tmp1FE;struct Cyc_Absyn_Datatypefield*tuf=_tmp1FF;
# 838
({void*_tmp3AC=(void*)({struct Cyc_Absyn_Datatype_p_Absyn_Raw_pat_struct*_tmp206=_cycalloc(sizeof(*_tmp206));_tmp206->tag=8U,_tmp206->f1=tud,_tmp206->f2=tuf,_tmp206->f3=ps,_tmp206->f4=dots;_tmp206;});p->r=_tmp3AC;});_npop_handler(0U);return;}case 3U: _LL43: _LL44:
 goto _LL46;default: _LL45: _LL46:
# 841
({struct Cyc_Warn_String_Warn_Warg_struct _tmp208=({struct Cyc_Warn_String_Warn_Warg_struct _tmp2D7;_tmp2D7.tag=0U,({struct _fat_ptr _tmp3AD=({const char*_tmp209="enum tag used with arguments in pattern";_tag_fat(_tmp209,sizeof(char),40U);});_tmp2D7.f1=_tmp3AD;});_tmp2D7;});void*_tmp207[1U];_tmp207[0]=& _tmp208;({unsigned _tmp3AE=loc;Cyc_Warn_err2(_tmp3AE,_tag_fat(_tmp207,sizeof(void*),1U));});});_npop_handler(0U);return;}_LL3C:;}
# 829
;_pop_handler();}else{void*_tmp1FB=(void*)Cyc_Core_get_exn_thrown();void*_tmp20A=_tmp1FB;void*_tmp20B;if(((struct Cyc_Binding_BindingError_exn_struct*)_tmp20A)->tag == Cyc_Binding_BindingError){_LL48: _LL49:
# 842
 goto _LL47;}else{_LL4A: _tmp20B=_tmp20A;_LL4B: {void*exn=_tmp20B;(int)_rethrow(exn);}}_LL47:;}}}
({struct Cyc_Warn_Qvar_Warn_Warg_struct _tmp20D=({struct Cyc_Warn_Qvar_Warn_Warg_struct _tmp2D9;_tmp2D9.tag=3U,_tmp2D9.f1=qv;_tmp2D9;});struct Cyc_Warn_String_Warn_Warg_struct _tmp20E=({struct Cyc_Warn_String_Warn_Warg_struct _tmp2D8;_tmp2D8.tag=0U,({struct _fat_ptr _tmp3AF=({const char*_tmp20F=" is not a constructor in pattern";_tag_fat(_tmp20F,sizeof(char),33U);});_tmp2D8.f1=_tmp3AF;});_tmp2D8;});void*_tmp20C[2U];_tmp20C[0]=& _tmp20D,_tmp20C[1]=& _tmp20E;({unsigned _tmp3B0=loc;Cyc_Warn_err2(_tmp3B0,_tag_fat(_tmp20C,sizeof(void*),2U));});});return;}case 7U: if(((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmp1D2)->f1 != 0){if((((union Cyc_Absyn_AggrInfo*)((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmp1D2)->f1)->UnknownAggr).tag == 1){_LL8: _tmp1DF=(((((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmp1D2)->f1)->UnknownAggr).val).f2;_tmp1E0=((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmp1D2)->f2;_tmp1E1=((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmp1D2)->f3;_tmp1E2=((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmp1D2)->f4;_LL9: {struct _tuple0*n=_tmp1DF;struct Cyc_List_List*exist_ts=_tmp1E0;struct Cyc_List_List*dps=_tmp1E1;int dots=_tmp1E2;
# 846
{struct Cyc_List_List*dps2=dps;for(0;dps2 != 0;dps2=dps2->tl){
Cyc_Binding_resolve_pat(env,(*((struct _tuple14*)dps2->hd)).f2);}}
{struct _handler_cons _tmp210;_push_handler(& _tmp210);{int _tmp212=0;if(setjmp(_tmp210.handler))_tmp212=1;if(!_tmp212){
{struct Cyc_Absyn_Aggrdecl*ad=({struct Cyc_Absyn_Aggrdecl*(*_tmp3B3)(unsigned loc,struct Cyc_Binding_NSCtxt*ctxt,struct _tuple0*qv,struct Cyc_Absyn_Aggrdecl*(*lookup)(struct Cyc_Binding_ResolveNSEnv*,struct _fat_ptr*))=({struct Cyc_Absyn_Aggrdecl*(*_tmp216)(unsigned loc,struct Cyc_Binding_NSCtxt*ctxt,struct _tuple0*qv,struct Cyc_Absyn_Aggrdecl*(*lookup)(struct Cyc_Binding_ResolveNSEnv*,struct _fat_ptr*))=(struct Cyc_Absyn_Aggrdecl*(*)(unsigned loc,struct Cyc_Binding_NSCtxt*ctxt,struct _tuple0*qv,struct Cyc_Absyn_Aggrdecl*(*lookup)(struct Cyc_Binding_ResolveNSEnv*,struct _fat_ptr*)))Cyc_Binding_resolve_lookup;_tmp216;});unsigned _tmp3B2=loc;struct Cyc_Binding_NSCtxt*_tmp3B1=env->ns;_tmp3B3(_tmp3B2,_tmp3B1,n,Cyc_Binding_lookup_aggrdecl);});
({void*_tmp3B6=(void*)({struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*_tmp215=_cycalloc(sizeof(*_tmp215));_tmp215->tag=7U,({union Cyc_Absyn_AggrInfo*_tmp3B5=({union Cyc_Absyn_AggrInfo*_tmp214=_cycalloc(sizeof(*_tmp214));({union Cyc_Absyn_AggrInfo _tmp3B4=Cyc_Absyn_KnownAggr(({struct Cyc_Absyn_Aggrdecl**_tmp213=_cycalloc(sizeof(*_tmp213));*_tmp213=ad;_tmp213;}));*_tmp214=_tmp3B4;});_tmp214;});_tmp215->f1=_tmp3B5;}),_tmp215->f2=exist_ts,_tmp215->f3=dps,_tmp215->f4=dots;_tmp215;});p->r=_tmp3B6;});}
# 849
;_pop_handler();}else{void*_tmp211=(void*)Cyc_Core_get_exn_thrown();void*_tmp217=_tmp211;void*_tmp218;if(((struct Cyc_Binding_BindingError_exn_struct*)_tmp217)->tag == Cyc_Binding_BindingError){_LL4D: _LL4E:
# 852
({struct Cyc_Warn_String_Warn_Warg_struct _tmp21A=({struct Cyc_Warn_String_Warn_Warg_struct _tmp2DA;_tmp2DA.tag=0U,({struct _fat_ptr _tmp3B7=({const char*_tmp21B="non-aggregate name has designator patterns";_tag_fat(_tmp21B,sizeof(char),43U);});_tmp2DA.f1=_tmp3B7;});_tmp2DA;});void*_tmp219[1U];_tmp219[0]=& _tmp21A;({unsigned _tmp3B8=loc;Cyc_Warn_err2(_tmp3B8,_tag_fat(_tmp219,sizeof(void*),1U));});});return;}else{_LL4F: _tmp218=_tmp217;_LL50: {void*exn=_tmp218;(int)_rethrow(exn);}}_LL4C:;}}}
# 854
return;}}else{_LL24: _tmp1DE=((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmp1D2)->f3;_LL25: {struct Cyc_List_List*dps=_tmp1DE;
# 883
for(0;dps != 0;dps=dps->tl){
Cyc_Binding_resolve_pat(env,(*((struct _tuple14*)dps->hd)).f2);}
return;}}}else{_LL22: _tmp1DD=((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmp1D2)->f3;_LL23: {struct Cyc_List_List*dps=_tmp1DD;
# 881
_tmp1DE=dps;goto _LL25;}}case 0U: _LLA: _LLB:
# 856
 return;case 3U: _LLC: _tmp1DB=((struct Cyc_Absyn_Reference_p_Absyn_Raw_pat_struct*)_tmp1D2)->f1;_tmp1DC=((struct Cyc_Absyn_Reference_p_Absyn_Raw_pat_struct*)_tmp1D2)->f2;_LLD: {struct Cyc_Absyn_Vardecl*vd=_tmp1DB;struct Cyc_Absyn_Pat*p2=_tmp1DC;
# 858
_tmp1D9=vd;_tmp1DA=p2;goto _LLF;}case 1U: _LLE: _tmp1D9=((struct Cyc_Absyn_Var_p_Absyn_Raw_pat_struct*)_tmp1D2)->f1;_tmp1DA=((struct Cyc_Absyn_Var_p_Absyn_Raw_pat_struct*)_tmp1D2)->f2;_LLF: {struct Cyc_Absyn_Vardecl*vd=_tmp1D9;struct Cyc_Absyn_Pat*p2=_tmp1DA;
# 860
Cyc_Binding_resolve_pat(env,p2);
_tmp1D8=vd;goto _LL11;}case 4U: _LL10: _tmp1D8=((struct Cyc_Absyn_TagInt_p_Absyn_Raw_pat_struct*)_tmp1D2)->f2;_LL11: {struct Cyc_Absyn_Vardecl*vd=_tmp1D8;
_tmp1D7=vd;goto _LL13;}case 2U: _LL12: _tmp1D7=((struct Cyc_Absyn_AliasVar_p_Absyn_Raw_pat_struct*)_tmp1D2)->f2;_LL13: {struct Cyc_Absyn_Vardecl*vd=_tmp1D7;
# 864
Cyc_Binding_resolve_and_add_vardecl(loc,env,vd);goto _LL3;}case 6U: _LL14: _tmp1D6=((struct Cyc_Absyn_Pointer_p_Absyn_Raw_pat_struct*)_tmp1D2)->f1;_LL15: {struct Cyc_Absyn_Pat*p2=_tmp1D6;
# 866
Cyc_Binding_resolve_pat(env,p2);return;}case 5U: _LL16: _tmp1D5=((struct Cyc_Absyn_Tuple_p_Absyn_Raw_pat_struct*)_tmp1D2)->f1;_LL17: {struct Cyc_List_List*ps=_tmp1D5;
# 868
for(0;ps != 0;ps=ps->tl){
Cyc_Binding_resolve_pat(env,(struct Cyc_Absyn_Pat*)ps->hd);}
goto _LL3;}case 17U: _LL18: _tmp1D4=((struct Cyc_Absyn_Exp_p_Absyn_Raw_pat_struct*)_tmp1D2)->f1;_LL19: {struct Cyc_Absyn_Exp*e=_tmp1D4;
# 872
Cyc_Binding_resolve_exp(env,e);return;}case 9U: _LL1A: _LL1B:
# 874
 goto _LL1D;case 10U: _LL1C: _LL1D:
 goto _LL1F;case 11U: _LL1E: _LL1F:
 goto _LL21;case 12U: _LL20: _LL21:
 return;case 8U: _LL26: _tmp1D3=((struct Cyc_Absyn_Datatype_p_Absyn_Raw_pat_struct*)_tmp1D2)->f3;_LL27: {struct Cyc_List_List*ps=_tmp1D3;
# 887
for(0;ps != 0;ps=ps->tl){
Cyc_Binding_resolve_pat(env,(struct Cyc_Absyn_Pat*)ps->hd);}
return;}case 13U: _LL28: _LL29:
 goto _LL2B;default: _LL2A: _LL2B:
 return;}_LL3:;}}struct _tuple15{void*f1;void*f2;};
# 273
void Cyc_Binding_resolve_rgnpo(unsigned loc,struct Cyc_Binding_Env*env,struct Cyc_List_List*po){
# 896
for(0;po != 0;po=po->tl){
Cyc_Binding_resolve_type(loc,env,(*((struct _tuple15*)po->hd)).f1);
Cyc_Binding_resolve_type(loc,env,(*((struct _tuple15*)po->hd)).f2);}}struct _tuple16{struct Cyc_List_List**f1;struct Cyc_Dict_Dict*f2;struct Cyc_Binding_Env*f3;struct _tuple10*f4;};struct _tuple17{unsigned f1;struct _tuple0*f2;int f3;};
# 273
void Cyc_Binding_export_all_symbols(struct _tuple16*cenv,struct _fat_ptr*name,void*res){
# 906
struct Cyc_List_List*_tmp220;unsigned _tmp21F;struct Cyc_Binding_Env*_tmp21E;struct Cyc_Dict_Dict*_tmp21D;struct Cyc_List_List**_tmp21C;_LL1: _tmp21C=cenv->f1;_tmp21D=cenv->f2;_tmp21E=cenv->f3;_tmp21F=(cenv->f4)->f1;_tmp220=(cenv->f4)->f2;_LL2: {struct Cyc_List_List**exlist_ptr=_tmp21C;struct Cyc_Dict_Dict*out_dict=_tmp21D;struct Cyc_Binding_Env*env=_tmp21E;unsigned wcloc=_tmp21F;struct Cyc_List_List*hidelist=_tmp220;
void*_tmp221=res;struct Cyc_Absyn_Datatypefield*_tmp223;struct Cyc_Absyn_Datatypedecl*_tmp222;struct Cyc_Absyn_Enumfield*_tmp225;void*_tmp224;struct Cyc_Absyn_Aggrdecl*_tmp226;struct Cyc_Absyn_Enumfield*_tmp228;struct Cyc_Absyn_Enumdecl*_tmp227;void*_tmp229;switch(*((int*)_tmp221)){case 0U: _LL4: _tmp229=(void*)((struct Cyc_Binding_VarRes_Binding_Resolved_struct*)_tmp221)->f1;_LL5: {void*bnd=_tmp229;
# 909
struct _tuple0*qv=({struct _tuple0*_tmp22E=_cycalloc(sizeof(*_tmp22E));({union Cyc_Absyn_Nmspace _tmp3B9=Cyc_Absyn_Rel_n(0);_tmp22E->f1=_tmp3B9;}),_tmp22E->f2=name;_tmp22E;});
if(({int(*_tmp3BB)(int(*compare)(struct _tuple0*,struct _tuple0*),struct Cyc_List_List*l,struct _tuple0*x)=({int(*_tmp22A)(int(*compare)(struct _tuple0*,struct _tuple0*),struct Cyc_List_List*l,struct _tuple0*x)=(int(*)(int(*compare)(struct _tuple0*,struct _tuple0*),struct Cyc_List_List*l,struct _tuple0*x))Cyc_List_mem;_tmp22A;});struct Cyc_List_List*_tmp3BA=hidelist;_tmp3BB(Cyc_Absyn_qvar_cmp,_tmp3BA,qv);}))
# 912
return;
# 910
Cyc_Binding_absolutize_decl(wcloc,env,qv,Cyc_Absyn_ExternC);{
# 915
struct _tuple17*ex_sym=({struct _tuple17*_tmp22D=_cycalloc(sizeof(*_tmp22D));_tmp22D->f1=wcloc,_tmp22D->f2=qv,_tmp22D->f3=0;_tmp22D;});
({struct Cyc_List_List*_tmp3BC=({struct Cyc_List_List*_tmp22B=_cycalloc(sizeof(*_tmp22B));_tmp22B->hd=ex_sym,_tmp22B->tl=*exlist_ptr;_tmp22B;});*exlist_ptr=_tmp3BC;});
({struct Cyc_Dict_Dict _tmp3C0=({struct Cyc_Dict_Dict(*_tmp3BF)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,void*v)=({struct Cyc_Dict_Dict(*_tmp22C)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,void*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,void*v))Cyc_Dict_insert;_tmp22C;});struct Cyc_Dict_Dict _tmp3BE=*out_dict;struct _fat_ptr*_tmp3BD=name;_tmp3BF(_tmp3BE,_tmp3BD,res);});*out_dict=_tmp3C0;});
goto _LL3;}}case 3U: _LL6: _tmp227=((struct Cyc_Binding_EnumRes_Binding_Resolved_struct*)_tmp221)->f1;_tmp228=((struct Cyc_Binding_EnumRes_Binding_Resolved_struct*)_tmp221)->f2;_LL7: {struct Cyc_Absyn_Enumdecl*ed=_tmp227;struct Cyc_Absyn_Enumfield*ef=_tmp228;
# 920
({struct Cyc_Dict_Dict _tmp3C4=({struct Cyc_Dict_Dict(*_tmp3C3)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,void*v)=({struct Cyc_Dict_Dict(*_tmp22F)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,void*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,void*v))Cyc_Dict_insert;_tmp22F;});struct Cyc_Dict_Dict _tmp3C2=*out_dict;struct _fat_ptr*_tmp3C1=name;_tmp3C3(_tmp3C2,_tmp3C1,res);});*out_dict=_tmp3C4;});goto _LL3;}case 1U: _LL8: _tmp226=((struct Cyc_Binding_AggrRes_Binding_Resolved_struct*)_tmp221)->f1;_LL9: {struct Cyc_Absyn_Aggrdecl*ad=_tmp226;
# 922
goto _LL3;}case 4U: _LLA: _tmp224=(void*)((struct Cyc_Binding_AnonEnumRes_Binding_Resolved_struct*)_tmp221)->f1;_tmp225=((struct Cyc_Binding_AnonEnumRes_Binding_Resolved_struct*)_tmp221)->f2;_LLB: {void*t=_tmp224;struct Cyc_Absyn_Enumfield*ef=_tmp225;
goto _LL3;}default: _LLC: _tmp222=((struct Cyc_Binding_DatatypeRes_Binding_Resolved_struct*)_tmp221)->f1;_tmp223=((struct Cyc_Binding_DatatypeRes_Binding_Resolved_struct*)_tmp221)->f2;_LLD: {struct Cyc_Absyn_Datatypedecl*dd=_tmp222;struct Cyc_Absyn_Datatypefield*df=_tmp223;
# 925
({void*_tmp230=0U;({unsigned _tmp3C6=wcloc;struct _fat_ptr _tmp3C5=({const char*_tmp231="Unexpected binding from extern C Include";_tag_fat(_tmp231,sizeof(char),41U);});Cyc_Warn_err(_tmp3C6,_tmp3C5,_tag_fat(_tmp230,sizeof(void*),0U));});});
goto _LL3;}}_LL3:;}}
# 273
void Cyc_Binding_resolve_decl(struct Cyc_Binding_Env*env,struct Cyc_Absyn_Decl*d){
# 941 "binding.cyc"
unsigned loc=d->loc;
void*_stmttmp1A=d->r;void*_tmp232=_stmttmp1A;struct _tuple10*_tmp236;struct Cyc_List_List**_tmp235;struct Cyc_List_List*_tmp234;struct Cyc_List_List*_tmp233;struct Cyc_List_List*_tmp237;struct Cyc_List_List*_tmp239;struct _tuple0*_tmp238;struct Cyc_List_List*_tmp23B;struct _fat_ptr*_tmp23A;struct Cyc_Absyn_Datatypedecl*_tmp23C;struct Cyc_Absyn_Enumdecl*_tmp23D;struct Cyc_Absyn_Aggrdecl*_tmp23E;struct Cyc_Absyn_Typedefdecl*_tmp23F;struct Cyc_Absyn_Exp*_tmp242;struct Cyc_Absyn_Vardecl*_tmp241;struct Cyc_Absyn_Tvar*_tmp240;struct Cyc_List_List*_tmp243;struct Cyc_Absyn_Exp*_tmp245;struct Cyc_Absyn_Pat*_tmp244;struct Cyc_Absyn_Fndecl*_tmp246;struct Cyc_Absyn_Vardecl*_tmp247;switch(*((int*)_tmp232)){case 0U: _LL1: _tmp247=((struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct*)_tmp232)->f1;_LL2: {struct Cyc_Absyn_Vardecl*vd=_tmp247;
# 945
Cyc_Binding_resolve_and_add_vardecl(loc,env,vd);
# 950
if(vd->initializer != 0 && !Cyc_Binding_in_cinclude(env))
Cyc_Binding_resolve_exp(env,(struct Cyc_Absyn_Exp*)_check_null(vd->initializer));
# 950
goto _LL0;}case 1U: _LL3: _tmp246=((struct Cyc_Absyn_Fn_d_Absyn_Raw_decl_struct*)_tmp232)->f1;_LL4: {struct Cyc_Absyn_Fndecl*fd=_tmp246;
# 955
Cyc_Binding_absolutize_decl(loc,env,fd->name,fd->sc);{
struct _tuple0*_stmttmp1B=fd->name;struct _fat_ptr*_tmp249;union Cyc_Absyn_Nmspace _tmp248;_LL24: _tmp248=_stmttmp1B->f1;_tmp249=_stmttmp1B->f2;_LL25: {union Cyc_Absyn_Nmspace decl_ns=_tmp248;struct _fat_ptr*decl_name=_tmp249;
# 958
struct Cyc_List_List*vds=Cyc_Binding_get_fun_vardecls(1,loc,env,(fd->i).args,(fd->i).cyc_varargs);
({struct Cyc_Core_Opt*_tmp3C7=({struct Cyc_Core_Opt*_tmp24A=_cycalloc(sizeof(*_tmp24A));_tmp24A->v=vds;_tmp24A;});fd->param_vardecls=_tmp3C7;});
# 961
if(Cyc_Binding_warn_override)
Cyc_Binding_check_warn_override(loc,env,fd->name);{
# 961
struct Cyc_Dict_Dict*old_locals=env->local_vars;
# 965
if(old_locals != 0)
({struct Cyc_Dict_Dict*_tmp3C8=({struct Cyc_Dict_Dict*_tmp24B=_cycalloc(sizeof(*_tmp24B));*_tmp24B=*((struct Cyc_Dict_Dict*)_check_null(env->local_vars));_tmp24B;});env->local_vars=_tmp3C8;});else{
# 968
({struct Cyc_Dict_Dict*_tmp3CA=({struct Cyc_Dict_Dict*_tmp24D=_cycalloc(sizeof(*_tmp24D));({struct Cyc_Dict_Dict _tmp3C9=({struct Cyc_Dict_Dict(*_tmp24C)(int(*cmp)(struct _fat_ptr*,struct _fat_ptr*))=(struct Cyc_Dict_Dict(*)(int(*cmp)(struct _fat_ptr*,struct _fat_ptr*)))Cyc_Dict_empty;_tmp24C;})(Cyc_strptrcmp);*_tmp24D=_tmp3C9;});_tmp24D;});env->local_vars=_tmp3CA;});}
{struct Cyc_List_List*vds1=vds;for(0;vds1 != 0;vds1=vds1->tl){
({struct Cyc_Dict_Dict _tmp3CF=({struct Cyc_Dict_Dict(*_tmp3CE)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,void*v)=({struct Cyc_Dict_Dict(*_tmp24E)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,void*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,void*v))Cyc_Dict_insert;_tmp24E;});struct Cyc_Dict_Dict _tmp3CD=*((struct Cyc_Dict_Dict*)_check_null(env->local_vars));struct _fat_ptr*_tmp3CC=(*((struct Cyc_Absyn_Vardecl*)vds1->hd)->name).f2;_tmp3CE(_tmp3CD,_tmp3CC,(void*)({struct Cyc_Binding_VarRes_Binding_Resolved_struct*_tmp250=_cycalloc(sizeof(*_tmp250));
_tmp250->tag=0U,({void*_tmp3CB=(void*)({struct Cyc_Absyn_Param_b_Absyn_Binding_struct*_tmp24F=_cycalloc(sizeof(*_tmp24F));_tmp24F->tag=3U,_tmp24F->f1=(struct Cyc_Absyn_Vardecl*)vds1->hd;_tmp24F;});_tmp250->f1=_tmp3CB;});_tmp250;}));});
# 970
*((struct Cyc_Dict_Dict*)_check_null(env->local_vars))=_tmp3CF;});}}
# 973
Cyc_Binding_resolve_function_stuff(loc,env,fd->i);
# 975
if(old_locals != 0){
# 977
({struct Cyc_Dict_Dict _tmp3D4=({struct Cyc_Dict_Dict(*_tmp3D3)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,void*v)=({struct Cyc_Dict_Dict(*_tmp251)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,void*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,void*v))Cyc_Dict_insert;_tmp251;});struct Cyc_Dict_Dict _tmp3D2=*old_locals;struct _fat_ptr*_tmp3D1=decl_name;_tmp3D3(_tmp3D2,_tmp3D1,(void*)({struct Cyc_Binding_VarRes_Binding_Resolved_struct*_tmp253=_cycalloc(sizeof(*_tmp253));
_tmp253->tag=0U,({void*_tmp3D0=(void*)({struct Cyc_Absyn_Funname_b_Absyn_Binding_struct*_tmp252=_cycalloc(sizeof(*_tmp252));_tmp252->tag=2U,_tmp252->f1=fd;_tmp252;});_tmp253->f1=_tmp3D0;});_tmp253;}));});
# 977
*old_locals=_tmp3D4;});
# 979
({struct Cyc_Dict_Dict _tmp3D9=({struct Cyc_Dict_Dict(*_tmp3D8)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,void*v)=({struct Cyc_Dict_Dict(*_tmp254)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,void*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,void*v))Cyc_Dict_insert;_tmp254;});struct Cyc_Dict_Dict _tmp3D7=*((struct Cyc_Dict_Dict*)_check_null(env->local_vars));struct _fat_ptr*_tmp3D6=decl_name;_tmp3D8(_tmp3D7,_tmp3D6,(void*)({struct Cyc_Binding_VarRes_Binding_Resolved_struct*_tmp256=_cycalloc(sizeof(*_tmp256));
_tmp256->tag=0U,({void*_tmp3D5=(void*)({struct Cyc_Absyn_Funname_b_Absyn_Binding_struct*_tmp255=_cycalloc(sizeof(*_tmp255));_tmp255->tag=2U,_tmp255->f1=fd;_tmp255;});_tmp256->f1=_tmp3D5;});_tmp256;}));});
# 979
*((struct Cyc_Dict_Dict*)_check_null(env->local_vars))=_tmp3D9;});}else{
# 982
struct Cyc_Binding_ResolveNSEnv*decl_ns_data=({struct Cyc_Binding_ResolveNSEnv*(*_tmp3DB)(struct Cyc_Binding_NSCtxt*ctxt,union Cyc_Absyn_Nmspace abs_ns)=({struct Cyc_Binding_ResolveNSEnv*(*_tmp25A)(struct Cyc_Binding_NSCtxt*ctxt,union Cyc_Absyn_Nmspace abs_ns)=(struct Cyc_Binding_ResolveNSEnv*(*)(struct Cyc_Binding_NSCtxt*ctxt,union Cyc_Absyn_Nmspace abs_ns))Cyc_Binding_get_ns_data;_tmp25A;});struct Cyc_Binding_NSCtxt*_tmp3DA=env->ns;_tmp3DB(_tmp3DA,decl_ns);});
({struct Cyc_Dict_Dict _tmp3E0=({struct Cyc_Dict_Dict(*_tmp3DF)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,void*v)=({struct Cyc_Dict_Dict(*_tmp257)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,void*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,void*v))Cyc_Dict_insert;_tmp257;});struct Cyc_Dict_Dict _tmp3DE=decl_ns_data->ordinaries;struct _fat_ptr*_tmp3DD=decl_name;_tmp3DF(_tmp3DE,_tmp3DD,(void*)({struct Cyc_Binding_VarRes_Binding_Resolved_struct*_tmp259=_cycalloc(sizeof(*_tmp259));
# 985
_tmp259->tag=0U,({void*_tmp3DC=(void*)({struct Cyc_Absyn_Funname_b_Absyn_Binding_struct*_tmp258=_cycalloc(sizeof(*_tmp258));_tmp258->tag=2U,_tmp258->f1=fd;_tmp258;});_tmp259->f1=_tmp3DC;});_tmp259;}));});
# 983
decl_ns_data->ordinaries=_tmp3E0;});}
# 988
if(!Cyc_Binding_in_cinclude(env))
Cyc_Binding_resolve_stmt(env,fd->body);
# 988
env->local_vars=old_locals;
# 992
goto _LL0;}}}}case 2U: _LL5: _tmp244=((struct Cyc_Absyn_Let_d_Absyn_Raw_decl_struct*)_tmp232)->f1;_tmp245=((struct Cyc_Absyn_Let_d_Absyn_Raw_decl_struct*)_tmp232)->f3;_LL6: {struct Cyc_Absyn_Pat*p=_tmp244;struct Cyc_Absyn_Exp*exp=_tmp245;
# 995
if(Cyc_Binding_at_toplevel(env)){
({struct Cyc_Warn_String_Warn_Warg_struct _tmp25C=({struct Cyc_Warn_String_Warn_Warg_struct _tmp2DB;_tmp2DB.tag=0U,({struct _fat_ptr _tmp3E1=({const char*_tmp25D="let not allowed at top-level";_tag_fat(_tmp25D,sizeof(char),29U);});_tmp2DB.f1=_tmp3E1;});_tmp2DB;});void*_tmp25B[1U];_tmp25B[0]=& _tmp25C;({unsigned _tmp3E2=loc;Cyc_Warn_err2(_tmp3E2,_tag_fat(_tmp25B,sizeof(void*),1U));});});
goto _LL0;}
# 995
Cyc_Binding_resolve_exp(env,exp);
# 1000
Cyc_Binding_resolve_pat(env,p);
goto _LL0;}case 3U: _LL7: _tmp243=((struct Cyc_Absyn_Letv_d_Absyn_Raw_decl_struct*)_tmp232)->f1;_LL8: {struct Cyc_List_List*vds=_tmp243;
# 1004
for(0;vds != 0;vds=vds->tl){
Cyc_Binding_resolve_and_add_vardecl(loc,env,(struct Cyc_Absyn_Vardecl*)vds->hd);}
goto _LL0;}case 4U: _LL9: _tmp240=((struct Cyc_Absyn_Region_d_Absyn_Raw_decl_struct*)_tmp232)->f1;_tmp241=((struct Cyc_Absyn_Region_d_Absyn_Raw_decl_struct*)_tmp232)->f2;_tmp242=((struct Cyc_Absyn_Region_d_Absyn_Raw_decl_struct*)_tmp232)->f3;_LLA: {struct Cyc_Absyn_Tvar*tv=_tmp240;struct Cyc_Absyn_Vardecl*vd=_tmp241;struct Cyc_Absyn_Exp*open_exp_opt=_tmp242;
# 1009
if(open_exp_opt != 0)
Cyc_Binding_resolve_exp(env,open_exp_opt);
# 1009
Cyc_Binding_resolve_and_add_vardecl(loc,env,vd);
# 1012
goto _LL0;}case 8U: _LLB: _tmp23F=((struct Cyc_Absyn_Typedef_d_Absyn_Raw_decl_struct*)_tmp232)->f1;_LLC: {struct Cyc_Absyn_Typedefdecl*td=_tmp23F;
# 1015
if(!Cyc_Binding_at_toplevel(env)){
({struct Cyc_Warn_String_Warn_Warg_struct _tmp25F=({struct Cyc_Warn_String_Warn_Warg_struct _tmp2DC;_tmp2DC.tag=0U,({struct _fat_ptr _tmp3E3=({const char*_tmp260="nested type definitions are not yet supported";_tag_fat(_tmp260,sizeof(char),46U);});_tmp2DC.f1=_tmp3E3;});_tmp2DC;});void*_tmp25E[1U];_tmp25E[0]=& _tmp25F;({unsigned _tmp3E4=loc;Cyc_Warn_err2(_tmp3E4,_tag_fat(_tmp25E,sizeof(void*),1U));});});
goto _LL0;}
# 1015
Cyc_Binding_absolutize_decl(loc,env,td->name,
# 1019
td->extern_c?Cyc_Absyn_ExternC: Cyc_Absyn_Public);{
struct _tuple0*_stmttmp1C=td->name;struct _fat_ptr*_tmp262;union Cyc_Absyn_Nmspace _tmp261;_LL27: _tmp261=_stmttmp1C->f1;_tmp262=_stmttmp1C->f2;_LL28: {union Cyc_Absyn_Nmspace decl_ns=_tmp261;struct _fat_ptr*decl_name=_tmp262;
# 1022
struct Cyc_Binding_ResolveNSEnv*decl_ns_data=({struct Cyc_Binding_ResolveNSEnv*(*_tmp3E6)(struct Cyc_Binding_NSCtxt*ctxt,union Cyc_Absyn_Nmspace abs_ns)=({struct Cyc_Binding_ResolveNSEnv*(*_tmp264)(struct Cyc_Binding_NSCtxt*ctxt,union Cyc_Absyn_Nmspace abs_ns)=(struct Cyc_Binding_ResolveNSEnv*(*)(struct Cyc_Binding_NSCtxt*ctxt,union Cyc_Absyn_Nmspace abs_ns))Cyc_Binding_get_ns_data;_tmp264;});struct Cyc_Binding_NSCtxt*_tmp3E5=env->ns;_tmp3E6(_tmp3E5,decl_ns);});
if(td->defn != 0)
Cyc_Binding_resolve_type(loc,env,(void*)_check_null(td->defn));
# 1023
({struct Cyc_Dict_Dict _tmp3EA=({
# 1026
struct Cyc_Dict_Dict(*_tmp3E9)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,struct Cyc_Absyn_Typedefdecl*v)=({struct Cyc_Dict_Dict(*_tmp263)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,struct Cyc_Absyn_Typedefdecl*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,struct Cyc_Absyn_Typedefdecl*v))Cyc_Dict_insert;_tmp263;});struct Cyc_Dict_Dict _tmp3E8=decl_ns_data->typedefs;struct _fat_ptr*_tmp3E7=decl_name;_tmp3E9(_tmp3E8,_tmp3E7,td);});
# 1023
decl_ns_data->typedefs=_tmp3EA;});
# 1028
goto _LL0;}}}case 5U: _LLD: _tmp23E=((struct Cyc_Absyn_Aggr_d_Absyn_Raw_decl_struct*)_tmp232)->f1;_LLE: {struct Cyc_Absyn_Aggrdecl*ad=_tmp23E;
# 1031
if(!Cyc_Binding_at_toplevel(env)){
({struct Cyc_Warn_String_Warn_Warg_struct _tmp266=({struct Cyc_Warn_String_Warn_Warg_struct _tmp2DD;_tmp2DD.tag=0U,({struct _fat_ptr _tmp3EB=({const char*_tmp267="nested type definitions are not yet supported";_tag_fat(_tmp267,sizeof(char),46U);});_tmp2DD.f1=_tmp3EB;});_tmp2DD;});void*_tmp265[1U];_tmp265[0]=& _tmp266;({unsigned _tmp3EC=loc;Cyc_Warn_err2(_tmp3EC,_tag_fat(_tmp265,sizeof(void*),1U));});});
goto _LL0;}
# 1031
Cyc_Binding_absolutize_decl(loc,env,ad->name,ad->sc);{
# 1036
struct _tuple0*_stmttmp1D=ad->name;struct _fat_ptr*_tmp269;union Cyc_Absyn_Nmspace _tmp268;_LL2A: _tmp268=_stmttmp1D->f1;_tmp269=_stmttmp1D->f2;_LL2B: {union Cyc_Absyn_Nmspace decl_ns=_tmp268;struct _fat_ptr*decl_name=_tmp269;
# 1038
struct Cyc_Binding_ResolveNSEnv*decl_ns_data=({struct Cyc_Binding_ResolveNSEnv*(*_tmp3EE)(struct Cyc_Binding_NSCtxt*ctxt,union Cyc_Absyn_Nmspace abs_ns)=({struct Cyc_Binding_ResolveNSEnv*(*_tmp270)(struct Cyc_Binding_NSCtxt*ctxt,union Cyc_Absyn_Nmspace abs_ns)=(struct Cyc_Binding_ResolveNSEnv*(*)(struct Cyc_Binding_NSCtxt*ctxt,union Cyc_Absyn_Nmspace abs_ns))Cyc_Binding_get_ns_data;_tmp270;});struct Cyc_Binding_NSCtxt*_tmp3ED=env->ns;_tmp3EE(_tmp3ED,decl_ns);});
# 1040
if(({int(*_tmp3F0)(struct Cyc_Dict_Dict d,struct _fat_ptr*k)=({int(*_tmp26A)(struct Cyc_Dict_Dict d,struct _fat_ptr*k)=(int(*)(struct Cyc_Dict_Dict d,struct _fat_ptr*k))Cyc_Dict_member;_tmp26A;});struct Cyc_Dict_Dict _tmp3EF=decl_ns_data->aggrdecls;_tmp3F0(_tmp3EF,decl_name);})&& ad->impl == 0)
goto _LL0;
# 1040
({struct Cyc_Dict_Dict _tmp3F4=({
# 1043
struct Cyc_Dict_Dict(*_tmp3F3)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,struct Cyc_Absyn_Aggrdecl*v)=({struct Cyc_Dict_Dict(*_tmp26B)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,struct Cyc_Absyn_Aggrdecl*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,struct Cyc_Absyn_Aggrdecl*v))Cyc_Dict_insert;_tmp26B;});struct Cyc_Dict_Dict _tmp3F2=decl_ns_data->aggrdecls;struct _fat_ptr*_tmp3F1=decl_name;_tmp3F3(_tmp3F2,_tmp3F1,ad);});
# 1040
decl_ns_data->aggrdecls=_tmp3F4;});
# 1045
({struct Cyc_Dict_Dict _tmp3F8=({struct Cyc_Dict_Dict(*_tmp3F7)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,void*v)=({struct Cyc_Dict_Dict(*_tmp26C)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,void*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,void*v))Cyc_Dict_insert;_tmp26C;});struct Cyc_Dict_Dict _tmp3F6=decl_ns_data->ordinaries;struct _fat_ptr*_tmp3F5=decl_name;_tmp3F7(_tmp3F6,_tmp3F5,(void*)({struct Cyc_Binding_AggrRes_Binding_Resolved_struct*_tmp26D=_cycalloc(sizeof(*_tmp26D));
_tmp26D->tag=1U,_tmp26D->f1=ad;_tmp26D;}));});
# 1045
decl_ns_data->ordinaries=_tmp3F8;});
# 1047
if(ad->impl != 0){
struct Cyc_Absyn_AggrdeclImpl*_stmttmp1E=(struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl);struct Cyc_List_List*_tmp26F;struct Cyc_List_List*_tmp26E;_LL2D: _tmp26E=_stmttmp1E->rgn_po;_tmp26F=_stmttmp1E->fields;_LL2E: {struct Cyc_List_List*rpo=_tmp26E;struct Cyc_List_List*fs=_tmp26F;
Cyc_Binding_resolve_rgnpo(loc,env,rpo);
Cyc_Binding_resolve_aggrfields(loc,env,fs);}}
# 1047
goto _LL0;}}}case 7U: _LLF: _tmp23D=((struct Cyc_Absyn_Enum_d_Absyn_Raw_decl_struct*)_tmp232)->f1;_LL10: {struct Cyc_Absyn_Enumdecl*ed=_tmp23D;
# 1055
if(!Cyc_Binding_at_toplevel(env)){
({struct Cyc_Warn_String_Warn_Warg_struct _tmp272=({struct Cyc_Warn_String_Warn_Warg_struct _tmp2DE;_tmp2DE.tag=0U,({struct _fat_ptr _tmp3F9=({const char*_tmp273="nested type definitions are not yet supported";_tag_fat(_tmp273,sizeof(char),46U);});_tmp2DE.f1=_tmp3F9;});_tmp2DE;});void*_tmp271[1U];_tmp271[0]=& _tmp272;({unsigned _tmp3FA=loc;Cyc_Warn_err2(_tmp3FA,_tag_fat(_tmp271,sizeof(void*),1U));});});
goto _LL0;}
# 1055
Cyc_Binding_absolutize_decl(loc,env,ed->name,ed->sc);{
# 1060
struct _tuple0*_stmttmp1F=ed->name;struct _fat_ptr*_tmp275;union Cyc_Absyn_Nmspace _tmp274;_LL30: _tmp274=_stmttmp1F->f1;_tmp275=_stmttmp1F->f2;_LL31: {union Cyc_Absyn_Nmspace decl_ns=_tmp274;struct _fat_ptr*decl_name=_tmp275;
# 1062
struct Cyc_Binding_ResolveNSEnv*decl_ns_data=({struct Cyc_Binding_ResolveNSEnv*(*_tmp3FC)(struct Cyc_Binding_NSCtxt*ctxt,union Cyc_Absyn_Nmspace abs_ns)=({struct Cyc_Binding_ResolveNSEnv*(*_tmp27A)(struct Cyc_Binding_NSCtxt*ctxt,union Cyc_Absyn_Nmspace abs_ns)=(struct Cyc_Binding_ResolveNSEnv*(*)(struct Cyc_Binding_NSCtxt*ctxt,union Cyc_Absyn_Nmspace abs_ns))Cyc_Binding_get_ns_data;_tmp27A;});struct Cyc_Binding_NSCtxt*_tmp3FB=env->ns;_tmp3FC(_tmp3FB,decl_ns);});
# 1064
if(({int(*_tmp3FE)(struct Cyc_Dict_Dict d,struct _fat_ptr*k)=({int(*_tmp276)(struct Cyc_Dict_Dict d,struct _fat_ptr*k)=(int(*)(struct Cyc_Dict_Dict d,struct _fat_ptr*k))Cyc_Dict_member;_tmp276;});struct Cyc_Dict_Dict _tmp3FD=decl_ns_data->enumdecls;_tmp3FE(_tmp3FD,decl_name);})&& ed->fields == 0)
goto _LL0;
# 1064
({struct Cyc_Dict_Dict _tmp402=({
# 1068
struct Cyc_Dict_Dict(*_tmp401)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,struct Cyc_Absyn_Enumdecl*v)=({struct Cyc_Dict_Dict(*_tmp277)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,struct Cyc_Absyn_Enumdecl*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,struct Cyc_Absyn_Enumdecl*v))Cyc_Dict_insert;_tmp277;});struct Cyc_Dict_Dict _tmp400=decl_ns_data->enumdecls;struct _fat_ptr*_tmp3FF=decl_name;_tmp401(_tmp400,_tmp3FF,ed);});
# 1064
decl_ns_data->enumdecls=_tmp402;});
# 1070
if(ed->fields != 0){
# 1072
struct Cyc_List_List*fs=(struct Cyc_List_List*)((struct Cyc_Core_Opt*)_check_null(ed->fields))->v;for(0;fs != 0;fs=fs->tl){
struct Cyc_Absyn_Enumfield*f=(struct Cyc_Absyn_Enumfield*)fs->hd;
Cyc_Binding_absolutize_decl(f->loc,env,f->name,ed->sc);
if(f->tag != 0)
Cyc_Binding_resolve_exp(env,(struct Cyc_Absyn_Exp*)_check_null(f->tag));
# 1075
({struct Cyc_Dict_Dict _tmp406=({
# 1077
struct Cyc_Dict_Dict(*_tmp405)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,void*v)=({struct Cyc_Dict_Dict(*_tmp278)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,void*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,void*v))Cyc_Dict_insert;_tmp278;});struct Cyc_Dict_Dict _tmp404=decl_ns_data->ordinaries;struct _fat_ptr*_tmp403=(*f->name).f2;_tmp405(_tmp404,_tmp403,(void*)({struct Cyc_Binding_EnumRes_Binding_Resolved_struct*_tmp279=_cycalloc(sizeof(*_tmp279));
# 1079
_tmp279->tag=3U,_tmp279->f1=ed,_tmp279->f2=f;_tmp279;}));});
# 1075
decl_ns_data->ordinaries=_tmp406;});}}
# 1070
goto _LL0;}}}case 6U: _LL11: _tmp23C=((struct Cyc_Absyn_Datatype_d_Absyn_Raw_decl_struct*)_tmp232)->f1;_LL12: {struct Cyc_Absyn_Datatypedecl*tud=_tmp23C;
# 1085
{struct _handler_cons _tmp27B;_push_handler(& _tmp27B);{int _tmp27D=0;if(setjmp(_tmp27B.handler))_tmp27D=1;if(!_tmp27D){
{struct Cyc_List_List*decls=({struct Cyc_List_List*(*_tmp409)(unsigned loc,struct Cyc_Binding_NSCtxt*ctxt,struct _tuple0*qv,struct Cyc_List_List*(*lookup)(struct Cyc_Binding_ResolveNSEnv*,struct _fat_ptr*))=({struct Cyc_List_List*(*_tmp284)(unsigned loc,struct Cyc_Binding_NSCtxt*ctxt,struct _tuple0*qv,struct Cyc_List_List*(*lookup)(struct Cyc_Binding_ResolveNSEnv*,struct _fat_ptr*))=(struct Cyc_List_List*(*)(unsigned loc,struct Cyc_Binding_NSCtxt*ctxt,struct _tuple0*qv,struct Cyc_List_List*(*lookup)(struct Cyc_Binding_ResolveNSEnv*,struct _fat_ptr*)))Cyc_Binding_resolve_lookup;_tmp284;});unsigned _tmp408=loc;struct Cyc_Binding_NSCtxt*_tmp407=env->ns;_tmp409(_tmp408,_tmp407,tud->name,Cyc_Binding_lookup_datatypedecl);});
struct Cyc_Absyn_Datatypedecl*last_decl=(struct Cyc_Absyn_Datatypedecl*)decls->hd;
if(!last_decl->is_extensible)
(int)_throw((void*)({struct Cyc_Binding_BindingError_exn_struct*_tmp27E=_cycalloc(sizeof(*_tmp27E));_tmp27E->tag=Cyc_Binding_BindingError;_tmp27E;}));
# 1088
tud->name=last_decl->name;
# 1091
tud->is_extensible=1;{
struct _tuple0*_stmttmp20=tud->name;struct _fat_ptr*_tmp280;union Cyc_Absyn_Nmspace _tmp27F;_LL33: _tmp27F=_stmttmp20->f1;_tmp280=_stmttmp20->f2;_LL34: {union Cyc_Absyn_Nmspace decl_ns=_tmp27F;struct _fat_ptr*decl_name=_tmp280;
struct Cyc_Binding_ResolveNSEnv*decl_ns_data=({struct Cyc_Binding_ResolveNSEnv*(*_tmp40B)(struct Cyc_Binding_NSCtxt*ctxt,union Cyc_Absyn_Nmspace abs_ns)=({struct Cyc_Binding_ResolveNSEnv*(*_tmp283)(struct Cyc_Binding_NSCtxt*ctxt,union Cyc_Absyn_Nmspace abs_ns)=(struct Cyc_Binding_ResolveNSEnv*(*)(struct Cyc_Binding_NSCtxt*ctxt,union Cyc_Absyn_Nmspace abs_ns))Cyc_Binding_get_ns_data;_tmp283;});struct Cyc_Binding_NSCtxt*_tmp40A=env->ns;_tmp40B(_tmp40A,decl_ns);});
({struct Cyc_Dict_Dict _tmp40F=({struct Cyc_Dict_Dict(*_tmp40E)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,struct Cyc_List_List*v)=({struct Cyc_Dict_Dict(*_tmp281)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,struct Cyc_List_List*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,struct Cyc_List_List*v))Cyc_Dict_insert;_tmp281;});struct Cyc_Dict_Dict _tmp40D=decl_ns_data->datatypedecls;struct _fat_ptr*_tmp40C=decl_name;_tmp40E(_tmp40D,_tmp40C,({struct Cyc_List_List*_tmp282=_cycalloc(sizeof(*_tmp282));
_tmp282->hd=tud,_tmp282->tl=decls;_tmp282;}));});
# 1094
decl_ns_data->datatypedecls=_tmp40F;});}}}
# 1086
;_pop_handler();}else{void*_tmp27C=(void*)Cyc_Core_get_exn_thrown();void*_tmp285=_tmp27C;void*_tmp286;if(((struct Cyc_Binding_BindingError_exn_struct*)_tmp285)->tag == Cyc_Binding_BindingError){_LL36: _LL37:
# 1098
 Cyc_Binding_absolutize_topdecl(loc,env,tud->name,tud->sc);{
struct _tuple0*_stmttmp21=tud->name;struct _fat_ptr*_tmp288;union Cyc_Absyn_Nmspace _tmp287;_LL3B: _tmp287=_stmttmp21->f1;_tmp288=_stmttmp21->f2;_LL3C: {union Cyc_Absyn_Nmspace decl_ns=_tmp287;struct _fat_ptr*decl_name=_tmp288;
struct Cyc_Binding_ResolveNSEnv*decl_ns_data=({struct Cyc_Binding_ResolveNSEnv*(*_tmp411)(struct Cyc_Binding_NSCtxt*ctxt,union Cyc_Absyn_Nmspace abs_ns)=({struct Cyc_Binding_ResolveNSEnv*(*_tmp28C)(struct Cyc_Binding_NSCtxt*ctxt,union Cyc_Absyn_Nmspace abs_ns)=(struct Cyc_Binding_ResolveNSEnv*(*)(struct Cyc_Binding_NSCtxt*ctxt,union Cyc_Absyn_Nmspace abs_ns))Cyc_Binding_get_ns_data;_tmp28C;});struct Cyc_Binding_NSCtxt*_tmp410=env->ns;_tmp411(_tmp410,decl_ns);});
# 1102
if(({int(*_tmp413)(struct Cyc_Dict_Dict d,struct _fat_ptr*k)=({int(*_tmp289)(struct Cyc_Dict_Dict d,struct _fat_ptr*k)=(int(*)(struct Cyc_Dict_Dict d,struct _fat_ptr*k))Cyc_Dict_member;_tmp289;});struct Cyc_Dict_Dict _tmp412=decl_ns_data->datatypedecls;_tmp413(_tmp412,decl_name);})&& tud->fields == 0)
# 1104
goto _LL35;
# 1102
({struct Cyc_Dict_Dict _tmp417=({
# 1105
struct Cyc_Dict_Dict(*_tmp416)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,struct Cyc_List_List*v)=({struct Cyc_Dict_Dict(*_tmp28A)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,struct Cyc_List_List*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,struct Cyc_List_List*v))Cyc_Dict_insert;_tmp28A;});struct Cyc_Dict_Dict _tmp415=decl_ns_data->datatypedecls;struct _fat_ptr*_tmp414=decl_name;_tmp416(_tmp415,_tmp414,({struct Cyc_List_List*_tmp28B=_cycalloc(sizeof(*_tmp28B));
_tmp28B->hd=tud,_tmp28B->tl=0;_tmp28B;}));});
# 1102
decl_ns_data->datatypedecls=_tmp417;});
# 1107
goto _LL35;}}}else{_LL38: _tmp286=_tmp285;_LL39: {void*exn=_tmp286;(int)_rethrow(exn);}}_LL35:;}}}{
# 1110
struct _tuple0*_stmttmp22=tud->name;struct _fat_ptr*_tmp28E;union Cyc_Absyn_Nmspace _tmp28D;_LL3E: _tmp28D=_stmttmp22->f1;_tmp28E=_stmttmp22->f2;_LL3F: {union Cyc_Absyn_Nmspace decl_ns=_tmp28D;struct _fat_ptr*decl_name=_tmp28E;
if(tud->fields != 0){
struct Cyc_List_List*fs=(struct Cyc_List_List*)((struct Cyc_Core_Opt*)_check_null(tud->fields))->v;for(0;fs != 0;fs=fs->tl){
struct Cyc_Absyn_Datatypefield*f=(struct Cyc_Absyn_Datatypefield*)fs->hd;
{struct Cyc_List_List*tqts=f->typs;for(0;tqts != 0;tqts=tqts->tl){
Cyc_Binding_resolve_type(loc,env,(*((struct _tuple13*)tqts->hd)).f2);}}
{union Cyc_Absyn_Nmspace _stmttmp23=(*f->name).f1;union Cyc_Absyn_Nmspace _tmp28F=_stmttmp23;switch((_tmp28F.Abs_n).tag){case 1U: if((_tmp28F.Rel_n).val == 0){_LL41: _LL42:
# 1118
 if(tud->is_extensible)
({union Cyc_Absyn_Nmspace _tmp418=Cyc_Absyn_Abs_n((env->ns)->curr_ns,0);(*f->name).f1=_tmp418;});else{
# 1121
(*f->name).f1=(*tud->name).f1;}
goto _LL40;}else{_LL43: _LL44:
# 1124
({struct Cyc_Warn_String_Warn_Warg_struct _tmp291=({struct Cyc_Warn_String_Warn_Warg_struct _tmp2DF;_tmp2DF.tag=0U,({struct _fat_ptr _tmp419=({const char*_tmp292="qualified datatype field declaratations not allowed";_tag_fat(_tmp292,sizeof(char),52U);});_tmp2DF.f1=_tmp419;});_tmp2DF;});void*_tmp290[1U];_tmp290[0]=& _tmp291;({unsigned _tmp41A=f->loc;Cyc_Warn_err2(_tmp41A,_tag_fat(_tmp290,sizeof(void*),1U));});});
return;}case 2U: _LL45: _LL46:
 goto _LL40;default: _LL47: _LL48:
({void*_tmp293=0U;({int(*_tmp41C)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp295)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmp295;});struct _fat_ptr _tmp41B=({const char*_tmp294="datatype field Loc_n or C_n";_tag_fat(_tmp294,sizeof(char),28U);});_tmp41C(_tmp41B,_tag_fat(_tmp293,sizeof(void*),0U));});});}_LL40:;}{
# 1129
struct Cyc_Binding_ResolveNSEnv*ns_data=({struct Cyc_Binding_ResolveNSEnv*(*_tmp41E)(struct Cyc_Binding_NSCtxt*ctxt,union Cyc_Absyn_Nmspace abs_ns)=({struct Cyc_Binding_ResolveNSEnv*(*_tmp298)(struct Cyc_Binding_NSCtxt*ctxt,union Cyc_Absyn_Nmspace abs_ns)=(struct Cyc_Binding_ResolveNSEnv*(*)(struct Cyc_Binding_NSCtxt*ctxt,union Cyc_Absyn_Nmspace abs_ns))Cyc_Binding_get_ns_data;_tmp298;});struct Cyc_Binding_NSCtxt*_tmp41D=env->ns;_tmp41E(_tmp41D,(*f->name).f1);});
({struct Cyc_Dict_Dict _tmp422=({struct Cyc_Dict_Dict(*_tmp421)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,void*v)=({struct Cyc_Dict_Dict(*_tmp296)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,void*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,void*v))Cyc_Dict_insert;_tmp296;});struct Cyc_Dict_Dict _tmp420=ns_data->ordinaries;struct _fat_ptr*_tmp41F=(*f->name).f2;_tmp421(_tmp420,_tmp41F,(void*)({struct Cyc_Binding_DatatypeRes_Binding_Resolved_struct*_tmp297=_cycalloc(sizeof(*_tmp297));
# 1132
_tmp297->tag=2U,_tmp297->f1=tud,_tmp297->f2=f;_tmp297;}));});
# 1130
ns_data->ordinaries=_tmp422;});}}}
# 1111
goto _LL0;}}}case 9U: _LL13: _tmp23A=((struct Cyc_Absyn_Namespace_d_Absyn_Raw_decl_struct*)_tmp232)->f1;_tmp23B=((struct Cyc_Absyn_Namespace_d_Absyn_Raw_decl_struct*)_tmp232)->f2;_LL14: {struct _fat_ptr*v=_tmp23A;struct Cyc_List_List*ds2=_tmp23B;
# 1137
({void(*_tmp424)(struct Cyc_Binding_NSCtxt*ctxt,struct _fat_ptr*subname,int env,struct Cyc_Binding_ResolveNSEnv*(*mkdata)(int))=({void(*_tmp299)(struct Cyc_Binding_NSCtxt*ctxt,struct _fat_ptr*subname,int env,struct Cyc_Binding_ResolveNSEnv*(*mkdata)(int))=(void(*)(struct Cyc_Binding_NSCtxt*ctxt,struct _fat_ptr*subname,int env,struct Cyc_Binding_ResolveNSEnv*(*mkdata)(int)))Cyc_Binding_enter_ns;_tmp299;});struct Cyc_Binding_NSCtxt*_tmp423=env->ns;_tmp424(_tmp423,v,1,Cyc_Binding_mt_renv);});
Cyc_Binding_resolve_decls(env,ds2);
Cyc_Binding_leave_ns(env->ns);
goto _LL0;}case 10U: _LL15: _tmp238=((struct Cyc_Absyn_Using_d_Absyn_Raw_decl_struct*)_tmp232)->f1;_tmp239=((struct Cyc_Absyn_Using_d_Absyn_Raw_decl_struct*)_tmp232)->f2;_LL16: {struct _tuple0*qv=_tmp238;struct Cyc_List_List*ds2=_tmp239;
# 1142
Cyc_Binding_enter_using(d->loc,env->ns,qv);
Cyc_Binding_resolve_decls(env,ds2);
Cyc_Binding_leave_using(env->ns);
goto _LL0;}case 11U: _LL17: _tmp237=((struct Cyc_Absyn_ExternC_d_Absyn_Raw_decl_struct*)_tmp232)->f1;_LL18: {struct Cyc_List_List*ds2=_tmp237;
# 1148
int old=env->in_cinclude;
env->in_cinclude=1;
Cyc_Binding_resolve_decls(env,ds2);
env->in_cinclude=old;
goto _LL0;}case 12U: _LL19: _tmp233=((struct Cyc_Absyn_ExternCinclude_d_Absyn_Raw_decl_struct*)_tmp232)->f1;_tmp234=((struct Cyc_Absyn_ExternCinclude_d_Absyn_Raw_decl_struct*)_tmp232)->f2;_tmp235=(struct Cyc_List_List**)&((struct Cyc_Absyn_ExternCinclude_d_Absyn_Raw_decl_struct*)_tmp232)->f3;_tmp236=((struct Cyc_Absyn_ExternCinclude_d_Absyn_Raw_decl_struct*)_tmp232)->f4;_LL1A: {struct Cyc_List_List*ds2=_tmp233;struct Cyc_List_List*ovrs=_tmp234;struct Cyc_List_List**exports=_tmp235;struct _tuple10*hides=_tmp236;
# 1159
{struct Cyc_List_List*exs=*exports;for(0;exs != 0;exs=exs->tl){
struct _tuple17*_stmttmp24=(struct _tuple17*)exs->hd;struct _tuple0*_tmp29B;unsigned _tmp29A;_LL4A: _tmp29A=_stmttmp24->f1;_tmp29B=_stmttmp24->f2;_LL4B: {unsigned loc=_tmp29A;struct _tuple0*qv=_tmp29B;
Cyc_Binding_absolutize_decl(loc,env,qv,Cyc_Absyn_ExternC);}}}
# 1167
if(!Cyc_Binding_at_toplevel(env)){
({struct Cyc_Warn_String_Warn_Warg_struct _tmp29D=({struct Cyc_Warn_String_Warn_Warg_struct _tmp2E0;_tmp2E0.tag=0U,({struct _fat_ptr _tmp425=({const char*_tmp29E="extern \"C include\" not at toplevel";_tag_fat(_tmp29E,sizeof(char),35U);});_tmp2E0.f1=_tmp425;});_tmp2E0;});void*_tmp29C[1U];_tmp29C[0]=& _tmp29D;({unsigned _tmp426=loc;Cyc_Warn_err2(_tmp426,_tag_fat(_tmp29C,sizeof(void*),1U));});});
goto _LL0;}{
# 1167
struct Cyc_Binding_ResolveNSEnv*data=({
# 1171
struct Cyc_Binding_ResolveNSEnv*(*_tmp428)(struct Cyc_Dict_Dict d,struct Cyc_List_List*k)=({struct Cyc_Binding_ResolveNSEnv*(*_tmp2AE)(struct Cyc_Dict_Dict d,struct Cyc_List_List*k)=(struct Cyc_Binding_ResolveNSEnv*(*)(struct Cyc_Dict_Dict d,struct Cyc_List_List*k))Cyc_Dict_lookup;_tmp2AE;});struct Cyc_Dict_Dict _tmp427=(env->ns)->ns_data;_tmp428(_tmp427,(env->ns)->curr_ns);});
struct Cyc_Dict_Dict old_dict=data->ordinaries;
int old=env->in_cinclude;
env->in_cinclude=1;
Cyc_Binding_resolve_decls(env,ds2);
Cyc_Binding_resolve_decls(env,ovrs);{
struct Cyc_Dict_Dict new_dict=data->ordinaries;
struct Cyc_Dict_Dict out_dict=old_dict;
if((*hides).f1 > (unsigned)0){
if((unsigned)*exports)
({void*_tmp29F=0U;({unsigned _tmp42A=(*hides).f1;struct _fat_ptr _tmp429=({const char*_tmp2A0="export wildcard expects empty export list";_tag_fat(_tmp2A0,sizeof(char),42U);});Cyc_Warn_err(_tmp42A,_tmp429,_tag_fat(_tmp29F,sizeof(void*),0U));});});
# 1180
env->in_cinclude=old;
# 1183
({void(*_tmp42C)(void(*f)(struct _tuple16*,struct _fat_ptr*,void*),struct _tuple16*env,struct Cyc_Dict_Dict d)=({void(*_tmp2A1)(void(*f)(struct _tuple16*,struct _fat_ptr*,void*),struct _tuple16*env,struct Cyc_Dict_Dict d)=(void(*)(void(*f)(struct _tuple16*,struct _fat_ptr*,void*),struct _tuple16*env,struct Cyc_Dict_Dict d))Cyc_Dict_iter_c;_tmp2A1;});struct _tuple16*_tmp42B=({struct _tuple16*_tmp2A2=_cycalloc(sizeof(*_tmp2A2));_tmp2A2->f1=exports,_tmp2A2->f2=& out_dict,_tmp2A2->f3=env,_tmp2A2->f4=hides;_tmp2A2;});_tmp42C(Cyc_Binding_export_all_symbols,_tmp42B,
Cyc_Dict_difference(new_dict,old_dict));});}else{
# 1187
struct Cyc_List_List*exs=*exports;for(0;exs != 0;exs=exs->tl){
struct _tuple17*_stmttmp25=(struct _tuple17*)exs->hd;struct _fat_ptr*_tmp2A4;unsigned _tmp2A3;_LL4D: _tmp2A3=_stmttmp25->f1;_tmp2A4=(_stmttmp25->f2)->f2;_LL4E: {unsigned loc=_tmp2A3;struct _fat_ptr*v=_tmp2A4;
if(!({int(*_tmp435)(struct Cyc_Dict_Dict d,struct _fat_ptr*k)=({int(*_tmp2A5)(struct Cyc_Dict_Dict d,struct _fat_ptr*k)=(int(*)(struct Cyc_Dict_Dict d,struct _fat_ptr*k))Cyc_Dict_member;_tmp2A5;});struct Cyc_Dict_Dict _tmp434=new_dict;_tmp435(_tmp434,v);})||
({int(*_tmp433)(struct Cyc_Dict_Dict d,struct _fat_ptr*k)=({int(*_tmp2A6)(struct Cyc_Dict_Dict d,struct _fat_ptr*k)=(int(*)(struct Cyc_Dict_Dict d,struct _fat_ptr*k))Cyc_Dict_member;_tmp2A6;});struct Cyc_Dict_Dict _tmp432=old_dict;_tmp433(_tmp432,v);})&&({
void*_tmp431=({void*(*_tmp42E)(struct Cyc_Dict_Dict d,struct _fat_ptr*k)=({void*(*_tmp2A7)(struct Cyc_Dict_Dict d,struct _fat_ptr*k)=(void*(*)(struct Cyc_Dict_Dict d,struct _fat_ptr*k))Cyc_Dict_lookup;_tmp2A7;});struct Cyc_Dict_Dict _tmp42D=old_dict;_tmp42E(_tmp42D,v);});_tmp431 == ({void*(*_tmp430)(struct Cyc_Dict_Dict d,struct _fat_ptr*k)=({void*(*_tmp2A8)(struct Cyc_Dict_Dict d,struct _fat_ptr*k)=(void*(*)(struct Cyc_Dict_Dict d,struct _fat_ptr*k))Cyc_Dict_lookup;_tmp2A8;});struct Cyc_Dict_Dict _tmp42F=new_dict;_tmp430(_tmp42F,v);});}))
({struct Cyc_String_pa_PrintArg_struct _tmp2AB=({struct Cyc_String_pa_PrintArg_struct _tmp2E1;_tmp2E1.tag=0U,_tmp2E1.f1=(struct _fat_ptr)((struct _fat_ptr)*v);_tmp2E1;});void*_tmp2A9[1U];_tmp2A9[0]=& _tmp2AB;({unsigned _tmp437=loc;struct _fat_ptr _tmp436=({const char*_tmp2AA="%s is exported but not defined";_tag_fat(_tmp2AA,sizeof(char),31U);});Cyc_Warn_err(_tmp437,_tmp436,_tag_fat(_tmp2A9,sizeof(void*),1U));});});
# 1189
out_dict=({
# 1193
struct Cyc_Dict_Dict(*_tmp43C)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,void*v)=({struct Cyc_Dict_Dict(*_tmp2AC)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,void*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,void*v))Cyc_Dict_insert;_tmp2AC;});struct Cyc_Dict_Dict _tmp43B=out_dict;struct _fat_ptr*_tmp43A=v;_tmp43C(_tmp43B,_tmp43A,({void*(*_tmp439)(struct Cyc_Dict_Dict d,struct _fat_ptr*k)=({void*(*_tmp2AD)(struct Cyc_Dict_Dict d,struct _fat_ptr*k)=(void*(*)(struct Cyc_Dict_Dict d,struct _fat_ptr*k))Cyc_Dict_lookup;_tmp2AD;});struct Cyc_Dict_Dict _tmp438=new_dict;_tmp439(_tmp438,v);}));});}}}
# 1196
data->ordinaries=out_dict;
env->in_cinclude=old;
# 1200
Cyc_Cifc_user_overrides(loc,ds2,ovrs);
goto _LL0;}}}case 13U: _LL1B: _LL1C:
# 1203
 goto _LL1E;case 14U: _LL1D: _LL1E:
 goto _LL20;case 15U: _LL1F: _LL20:
 goto _LL22;default: _LL21: _LL22:
 goto _LL0;}_LL0:;}
# 273 "binding.cyc"
void Cyc_Binding_resolve_decls(struct Cyc_Binding_Env*env,struct Cyc_List_List*tds){
# 1211 "binding.cyc"
for(0;tds != 0;tds=tds->tl){
Cyc_Binding_resolve_decl(env,(struct Cyc_Absyn_Decl*)tds->hd);}}
# 273 "binding.cyc"
void Cyc_Binding_resolve_all(struct Cyc_List_List*tds){
# 1216 "binding.cyc"
struct Cyc_Binding_Env*env=({struct Cyc_Binding_Env*_tmp2B2=_cycalloc(sizeof(*_tmp2B2));_tmp2B2->in_cinclude=0,({struct Cyc_Binding_NSCtxt*_tmp43D=({struct Cyc_Binding_NSCtxt*(*_tmp2B1)(int env,struct Cyc_Binding_ResolveNSEnv*(*mkdata)(int))=(struct Cyc_Binding_NSCtxt*(*)(int env,struct Cyc_Binding_ResolveNSEnv*(*mkdata)(int)))Cyc_Binding_mt_nsctxt;_tmp2B1;})(1,Cyc_Binding_mt_renv);_tmp2B2->ns=_tmp43D;}),_tmp2B2->local_vars=0;_tmp2B2;});
({struct Cyc_Binding_Env*_tmp440=env;Cyc_Binding_resolve_decl(_tmp440,({struct Cyc_Absyn_Decl*_tmp2B0=_cycalloc(sizeof(*_tmp2B0));({void*_tmp43F=(void*)({struct Cyc_Absyn_Datatype_d_Absyn_Raw_decl_struct*_tmp2AF=_cycalloc(sizeof(*_tmp2AF));_tmp2AF->tag=6U,({struct Cyc_Absyn_Datatypedecl*_tmp43E=Cyc_Absyn_exn_tud();_tmp2AF->f1=_tmp43E;});_tmp2AF;});_tmp2B0->r=_tmp43F;}),_tmp2B0->loc=0U;_tmp2B0;}));});
Cyc_Binding_resolve_decls(env,tds);}
