#include <setjmp.h>
/* This is a C header file to be used by the output of the Cyclone to
   C translator.  The corresponding definitions are in file lib/runtime_*.c */
#ifndef _CYC_INCLUDE_H_
#define _CYC_INCLUDE_H_

/* Need one of these per thread (see runtime_stack.c). The runtime maintains 
   a stack that contains either _handler_cons structs or _RegionHandle structs.
   The tag is 0 for a handler_cons and 1 for a region handle.  */
struct _RuntimeStack {
  int tag; 
  struct _RuntimeStack *next;
  void (*cleanup)(struct _RuntimeStack *frame);
};

#ifndef offsetof
/* should be size_t, but int is fine. */
#define offsetof(t,n) ((int)(&(((t *)0)->n)))
#endif

/* Fat pointers */
struct _fat_ptr {
  unsigned char *curr; 
  unsigned char *base; 
  unsigned char *last_plus_one; 
};  

/* Discriminated Unions */
struct _xtunion_struct { char *tag; };

/* Regions */
struct _RegionPage
#ifdef CYC_REGION_PROFILE
{ unsigned total_bytes;
  unsigned free_bytes;
  /* MWH: wish we didn't have to include the stuff below ... */
  struct _RegionPage *next;
  char data[1];
}
#endif
; // abstract -- defined in runtime_memory.c
struct _RegionHandle {
  struct _RuntimeStack s;
  struct _RegionPage *curr;
  char               *offset;
  char               *last_plus_one;
  struct _DynRegionHandle *sub_regions;
#ifdef CYC_REGION_PROFILE
  const char         *name;
#else
  unsigned used_bytes;
  unsigned wasted_bytes;
#endif
};
struct _DynRegionFrame {
  struct _RuntimeStack s;
  struct _DynRegionHandle *x;
};
// A dynamic region is just a region handle.  The wrapper struct is for type
// abstraction.
struct Cyc_Core_DynamicRegion {
  struct _RegionHandle h;
};

struct _RegionHandle _new_region(const char*);
void* _region_malloc(struct _RegionHandle*, unsigned);
void* _region_calloc(struct _RegionHandle*, unsigned t, unsigned n);
void   _free_region(struct _RegionHandle*);
struct _RegionHandle*_open_dynregion(struct _DynRegionFrame*,struct _DynRegionHandle*);
void   _pop_dynregion();

/* Exceptions */
struct _handler_cons {
  struct _RuntimeStack s;
  jmp_buf handler;
};
void _push_handler(struct _handler_cons *);
void _push_region(struct _RegionHandle *);
void _npop_handler(int);
void _pop_handler();
void _pop_region();

#ifndef _throw
void* _throw_null_fn(const char*,unsigned);
void* _throw_arraybounds_fn(const char*,unsigned);
void* _throw_badalloc_fn(const char*,unsigned);
void* _throw_match_fn(const char*,unsigned);
void* _throw_fn(void*,const char*,unsigned);
void* _rethrow(void*);
#define _throw_null() (_throw_null_fn(__FILE__,__LINE__))
#define _throw_arraybounds() (_throw_arraybounds_fn(__FILE__,__LINE__))
#define _throw_badalloc() (_throw_badalloc_fn(__FILE__,__LINE__))
#define _throw_match() (_throw_match_fn(__FILE__,__LINE__))
#define _throw(e) (_throw_fn((e),__FILE__,__LINE__))
#endif

struct _xtunion_struct* Cyc_Core_get_exn_thrown();
/* Built-in Exceptions */
struct Cyc_Null_Exception_exn_struct { char *tag; };
struct Cyc_Array_bounds_exn_struct { char *tag; };
struct Cyc_Match_Exception_exn_struct { char *tag; };
struct Cyc_Bad_alloc_exn_struct { char *tag; };
extern char Cyc_Null_Exception[];
extern char Cyc_Array_bounds[];
extern char Cyc_Match_Exception[];
extern char Cyc_Bad_alloc[];

/* Built-in Run-time Checks and company */
#ifdef CYC_ANSI_OUTPUT
#define _INLINE  
#else
#define _INLINE inline
#endif

#ifdef NO_CYC_NULL_CHECKS
#define _check_null(ptr) (ptr)
#else
#define _check_null(ptr) \
  ({ void*_cks_null = (void*)(ptr); \
     if (!_cks_null) _throw_null(); \
     _cks_null; })
#endif

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index)\
   (((char*)ptr) + (elt_sz)*(index))
#ifdef NO_CYC_NULL_CHECKS
#define _check_known_subscript_null _check_known_subscript_notnull
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr);\
  int _index = (index);\
  if (!_cks_ptr) _throw_null(); \
  _cks_ptr + (elt_sz)*_index; })
#endif
#define _zero_arr_plus_fn(orig_x,orig_sz,orig_i,f,l) ((orig_x)+(orig_i))
#define _zero_arr_plus_char_fn _zero_arr_plus_fn
#define _zero_arr_plus_short_fn _zero_arr_plus_fn
#define _zero_arr_plus_int_fn _zero_arr_plus_fn
#define _zero_arr_plus_float_fn _zero_arr_plus_fn
#define _zero_arr_plus_double_fn _zero_arr_plus_fn
#define _zero_arr_plus_longdouble_fn _zero_arr_plus_fn
#define _zero_arr_plus_voidstar_fn _zero_arr_plus_fn
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (!_cks_ptr) _throw_null(); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })

/* _zero_arr_plus_*_fn(x,sz,i,filename,lineno) adds i to zero-terminated ptr
   x that has at least sz elements */
char* _zero_arr_plus_char_fn(char*,unsigned,int,const char*,unsigned);
short* _zero_arr_plus_short_fn(short*,unsigned,int,const char*,unsigned);
int* _zero_arr_plus_int_fn(int*,unsigned,int,const char*,unsigned);
float* _zero_arr_plus_float_fn(float*,unsigned,int,const char*,unsigned);
double* _zero_arr_plus_double_fn(double*,unsigned,int,const char*,unsigned);
long double* _zero_arr_plus_longdouble_fn(long double*,unsigned,int,const char*, unsigned);
void** _zero_arr_plus_voidstar_fn(void**,unsigned,int,const char*,unsigned);
#endif

/* _get_zero_arr_size_*(x,sz) returns the number of elements in a
   zero-terminated array that is NULL or has at least sz elements */
int _get_zero_arr_size_char(const char*,unsigned);
int _get_zero_arr_size_short(const short*,unsigned);
int _get_zero_arr_size_int(const int*,unsigned);
int _get_zero_arr_size_float(const float*,unsigned);
int _get_zero_arr_size_double(const double*,unsigned);
int _get_zero_arr_size_longdouble(const long double*,unsigned);
int _get_zero_arr_size_voidstar(const void**,unsigned);

/* _zero_arr_inplace_plus_*_fn(x,i,filename,lineno) sets
   zero-terminated pointer *x to *x + i */
char* _zero_arr_inplace_plus_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_short_fn(short**,int,const char*,unsigned);
int* _zero_arr_inplace_plus_int(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_longdouble_fn(long double**,int,const char*,unsigned);
void** _zero_arr_inplace_plus_voidstar_fn(void***,int,const char*,unsigned);
/* like the previous functions, but does post-addition (as in e++) */
char* _zero_arr_inplace_plus_post_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_post_short_fn(short**x,int,const char*,unsigned);
int* _zero_arr_inplace_plus_post_int_fn(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_post_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_post_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_post_longdouble_fn(long double**,int,const char *,unsigned);
void** _zero_arr_inplace_plus_post_voidstar_fn(void***,int,const char*,unsigned);
#define _zero_arr_plus_char(x,s,i) \
  (_zero_arr_plus_char_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_short(x,s,i) \
  (_zero_arr_plus_short_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_int(x,s,i) \
  (_zero_arr_plus_int_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_float(x,s,i) \
  (_zero_arr_plus_float_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_double(x,s,i) \
  (_zero_arr_plus_double_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_longdouble(x,s,i) \
  (_zero_arr_plus_longdouble_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_voidstar(x,s,i) \
  (_zero_arr_plus_voidstar_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_inplace_plus_char(x,i) \
  _zero_arr_inplace_plus_char_fn((char **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_short(x,i) \
  _zero_arr_inplace_plus_short_fn((short **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_int(x,i) \
  _zero_arr_inplace_plus_int_fn((int **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_float(x,i) \
  _zero_arr_inplace_plus_float_fn((float **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_double(x,i) \
  _zero_arr_inplace_plus_double_fn((double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_longdouble(x,i) \
  _zero_arr_inplace_plus_longdouble_fn((long double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_voidstar(x,i) \
  _zero_arr_inplace_plus_voidstar_fn((void ***)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_char(x,i) \
  _zero_arr_inplace_plus_post_char_fn((char **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_short(x,i) \
  _zero_arr_inplace_plus_post_short_fn((short **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_int(x,i) \
  _zero_arr_inplace_plus_post_int_fn((int **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_float(x,i) \
  _zero_arr_inplace_plus_post_float_fn((float **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_double(x,i) \
  _zero_arr_inplace_plus_post_double_fn((double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_longdouble(x,i) \
  _zero_arr_inplace_plus_post_longdouble_fn((long double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_voidstar(x,i) \
  _zero_arr_inplace_plus_post_voidstar_fn((void***)(x),(i),__FILE__,__LINE__)

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_fat_subscript(arr,elt_sz,index) ((arr).curr + (elt_sz) * (index))
#define _untag_fat_ptr(arr,elt_sz,num_elts) ((arr).curr)
#else
#define _check_fat_subscript(arr,elt_sz,index) ({ \
  struct _fat_ptr _cus_arr = (arr); \
  unsigned char *_cus_ans = _cus_arr.curr + (elt_sz) * (index); \
  /* JGM: not needed! if (!_cus_arr.base) _throw_null();*/ \
  if (_cus_ans < _cus_arr.base || _cus_ans >= _cus_arr.last_plus_one) \
    _throw_arraybounds(); \
  _cus_ans; })
#define _untag_fat_ptr(arr,elt_sz,num_elts) ({ \
  struct _fat_ptr _arr = (arr); \
  unsigned char *_curr = _arr.curr; \
  if ((_curr < _arr.base || _curr + (elt_sz) * (num_elts) > _arr.last_plus_one) &&\
      _curr != (unsigned char *)0) \
    _throw_arraybounds(); \
  _curr; })
#endif

#define _tag_fat(tcurr,elt_sz,num_elts) ({ \
  struct _fat_ptr _ans; \
  unsigned _num_elts = (num_elts);\
  _ans.base = _ans.curr = (void*)(tcurr); \
  /* JGM: if we're tagging NULL, ignore num_elts */ \
  _ans.last_plus_one = _ans.base ? (_ans.base + (elt_sz) * _num_elts) : 0; \
  _ans; })

#define _get_fat_size(arr,elt_sz) \
  ({struct _fat_ptr _arr = (arr); \
    unsigned char *_arr_curr=_arr.curr; \
    unsigned char *_arr_last=_arr.last_plus_one; \
    (_arr_curr < _arr.base || _arr_curr >= _arr_last) ? 0 : \
    ((_arr_last - _arr_curr) / (elt_sz));})

#define _fat_ptr_plus(arr,elt_sz,change) ({ \
  struct _fat_ptr _ans = (arr); \
  int _change = (change);\
  _ans.curr += (elt_sz) * _change;\
  _ans; })
#define _fat_ptr_inplace_plus(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  _arr_ptr->curr += (elt_sz) * (change);\
  *_arr_ptr; })
#define _fat_ptr_inplace_plus_post(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  struct _fat_ptr _ans = *_arr_ptr; \
  _arr_ptr->curr += (elt_sz) * (change);\
  _ans; })

//Not a macro since initialization order matters. Defined in runtime_zeroterm.c.
struct _fat_ptr _fat_ptr_decrease_size(struct _fat_ptr,unsigned sz,unsigned numelts);

/* Allocation */
void* GC_malloc(int);
void* GC_malloc_atomic(int);
void* GC_calloc(unsigned,unsigned);
void* GC_calloc_atomic(unsigned,unsigned);
// bound the allocation size to be < MAX_ALLOC_SIZE. See macros below for usage.
#define MAX_MALLOC_SIZE (1 << 28)
void* _bounded_GC_malloc(int,const char*,int);
void* _bounded_GC_malloc_atomic(int,const char*,int);
void* _bounded_GC_calloc(unsigned,unsigned,const char*,int);
void* _bounded_GC_calloc_atomic(unsigned,unsigned,const char*,int);
/* these macros are overridden below ifdef CYC_REGION_PROFILE */
#ifndef CYC_REGION_PROFILE
#define _cycalloc(n) _bounded_GC_malloc(n,__FILE__,__LINE__)
#define _cycalloc_atomic(n) _bounded_GC_malloc_atomic(n,__FILE__,__LINE__)
#define _cyccalloc(n,s) _bounded_GC_calloc(n,s,__FILE__,__LINE__)
#define _cyccalloc_atomic(n,s) _bounded_GC_calloc_atomic(n,s,__FILE__,__LINE__)
#endif

static _INLINE unsigned int _check_times(unsigned x, unsigned y) {
  unsigned long long whole_ans = 
    ((unsigned long long) x)*((unsigned long long)y);
  unsigned word_ans = (unsigned)whole_ans;
  if(word_ans < whole_ans || word_ans > MAX_MALLOC_SIZE)
    _throw_badalloc();
  return word_ans;
}

#define _CYC_MAX_REGION_CONST 2
#define _CYC_MIN_ALIGNMENT (sizeof(double))

#ifdef CYC_REGION_PROFILE
extern int rgn_total_bytes;
#endif

static _INLINE void *_fast_region_malloc(struct _RegionHandle *r, unsigned orig_s) {  
  if (r > (struct _RegionHandle *)_CYC_MAX_REGION_CONST && r->curr != 0) { 
#ifdef CYC_NOALIGN
    unsigned s =  orig_s;
#else
    unsigned s =  (orig_s + _CYC_MIN_ALIGNMENT - 1) & (~(_CYC_MIN_ALIGNMENT -1)); 
#endif
    char *result; 
    result = r->offset; 
    if (s <= (r->last_plus_one - result)) {
      r->offset = result + s; 
#ifdef CYC_REGION_PROFILE
    r->curr->free_bytes = r->curr->free_bytes - s;
    rgn_total_bytes += s;
#endif
      return result;
    }
  } 
  return _region_malloc(r,orig_s); 
}

#ifdef CYC_REGION_PROFILE
/* see macros below for usage. defined in runtime_memory.c */
void* _profile_GC_malloc(int,const char*,const char*,int);
void* _profile_GC_malloc_atomic(int,const char*,const char*,int);
void* _profile_GC_calloc(unsigned,unsigned,const char*,const char*,int);
void* _profile_GC_calloc_atomic(unsigned,unsigned,const char*,const char*,int);
void* _profile_region_malloc(struct _RegionHandle*,unsigned,const char*,const char*,int);
void* _profile_region_calloc(struct _RegionHandle*,unsigned,unsigned,const char *,const char*,int);
struct _RegionHandle _profile_new_region(const char*,const char*,const char*,int);
void _profile_free_region(struct _RegionHandle*,const char*,const char*,int);
#ifndef RUNTIME_CYC
#define _new_region(n) _profile_new_region(n,__FILE__,__FUNCTION__,__LINE__)
#define _free_region(r) _profile_free_region(r,__FILE__,__FUNCTION__,__LINE__)
#define _region_malloc(rh,n) _profile_region_malloc(rh,n,__FILE__,__FUNCTION__,__LINE__)
#define _region_calloc(rh,n,t) _profile_region_calloc(rh,n,t,__FILE__,__FUNCTION__,__LINE__)
#  endif
#define _cycalloc(n) _profile_GC_malloc(n,__FILE__,__FUNCTION__,__LINE__)
#define _cycalloc_atomic(n) _profile_GC_malloc_atomic(n,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc(n,s) _profile_GC_calloc(n,s,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc_atomic(n,s) _profile_GC_calloc_atomic(n,s,__FILE__,__FUNCTION__,__LINE__)
#endif
#endif
 struct Cyc_Core_Opt{void*v;};extern char Cyc_Core_Invalid_argument[17U];struct Cyc_Core_Invalid_argument_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Failure[8U];struct Cyc_Core_Failure_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Impossible[11U];struct Cyc_Core_Impossible_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Not_found[10U];struct Cyc_Core_Not_found_exn_struct{char*tag;};extern char Cyc_Core_Unreachable[12U];struct Cyc_Core_Unreachable_exn_struct{char*tag;struct _fat_ptr f1;};
# 165 "core.h"
extern struct _RegionHandle*Cyc_Core_heap_region;
# 168
extern struct _RegionHandle*Cyc_Core_unique_region;struct Cyc_Core_DynamicRegion;struct Cyc_Core_NewDynamicRegion{struct Cyc_Core_DynamicRegion*key;};
# 275 "core.h"
void Cyc_Core_rethrow(void*);
# 281
const char*Cyc_Core_get_exn_filename();
# 288
int Cyc_Core_get_exn_lineno();struct Cyc_Core_ThinRes{void*arr;unsigned nelts;};
# 38 "cycboot.h"
int Cyc_open(const char*,int,struct _fat_ptr);struct Cyc___cycFILE;
# 51
extern struct Cyc___cycFILE*Cyc_stdout;
# 53
extern struct Cyc___cycFILE*Cyc_stderr;struct Cyc_String_pa_PrintArg_struct{int tag;struct _fat_ptr f1;};struct Cyc_Int_pa_PrintArg_struct{int tag;unsigned long f1;};struct Cyc_Double_pa_PrintArg_struct{int tag;double f1;};struct Cyc_LongDouble_pa_PrintArg_struct{int tag;long double f1;};struct Cyc_ShortPtr_pa_PrintArg_struct{int tag;short*f1;};struct Cyc_IntPtr_pa_PrintArg_struct{int tag;unsigned long*f1;};
# 73
struct _fat_ptr Cyc_aprintf(struct _fat_ptr,struct _fat_ptr);
# 79
int Cyc_fclose(struct Cyc___cycFILE*);
# 88
int Cyc_fflush(struct Cyc___cycFILE*);
# 98
struct Cyc___cycFILE*Cyc_fopen(const char*,const char*);
# 100
int Cyc_fprintf(struct Cyc___cycFILE*,struct _fat_ptr,struct _fat_ptr);
# 104
int Cyc_fputc(int,struct Cyc___cycFILE*);
# 106
int Cyc_fputs(const char*,struct Cyc___cycFILE*);struct Cyc_ShortPtr_sa_ScanfArg_struct{int tag;short*f1;};struct Cyc_UShortPtr_sa_ScanfArg_struct{int tag;unsigned short*f1;};struct Cyc_IntPtr_sa_ScanfArg_struct{int tag;int*f1;};struct Cyc_UIntPtr_sa_ScanfArg_struct{int tag;unsigned*f1;};struct Cyc_StringPtr_sa_ScanfArg_struct{int tag;struct _fat_ptr f1;};struct Cyc_DoublePtr_sa_ScanfArg_struct{int tag;double*f1;};struct Cyc_FloatPtr_sa_ScanfArg_struct{int tag;float*f1;};struct Cyc_CharPtr_sa_ScanfArg_struct{int tag;struct _fat_ptr f1;};
# 224 "cycboot.h"
int Cyc_vfprintf(struct Cyc___cycFILE*,struct _fat_ptr,struct _fat_ptr);extern char Cyc_FileCloseError[15U];struct Cyc_FileCloseError_exn_struct{char*tag;};extern char Cyc_FileOpenError[14U];struct Cyc_FileOpenError_exn_struct{char*tag;struct _fat_ptr f1;};
# 300 "cycboot.h"
int isspace(int);
# 310
int toupper(int);
# 318
int system(const char*);
void exit(int);
# 323
int mkdir(const char*pathname,unsigned short mode);
# 326
int close(int);
int chdir(const char*);
struct _fat_ptr Cyc_getcwd(struct _fat_ptr buf,unsigned long size);extern char Cyc_Lexing_Error[6U];struct Cyc_Lexing_Error_exn_struct{char*tag;struct _fat_ptr f1;};struct Cyc_Lexing_lexbuf{void(*refill_buff)(struct Cyc_Lexing_lexbuf*);void*refill_state;struct _fat_ptr lex_buffer;int lex_buffer_len;int lex_abs_pos;int lex_start_pos;int lex_curr_pos;int lex_last_pos;int lex_last_action;int lex_eof_reached;};struct Cyc_Lexing_function_lexbuf_state{int(*read_fun)(struct _fat_ptr,int,void*);void*read_fun_state;};struct Cyc_Lexing_lex_tables{struct _fat_ptr lex_base;struct _fat_ptr lex_backtrk;struct _fat_ptr lex_default;struct _fat_ptr lex_trans;struct _fat_ptr lex_check;};
# 78 "lexing.h"
struct Cyc_Lexing_lexbuf*Cyc_Lexing_from_file(struct Cyc___cycFILE*);
# 82
struct _fat_ptr Cyc_Lexing_lexeme(struct Cyc_Lexing_lexbuf*);
char Cyc_Lexing_lexeme_char(struct Cyc_Lexing_lexbuf*,int);
int Cyc_Lexing_lexeme_start(struct Cyc_Lexing_lexbuf*);
int Cyc_Lexing_lexeme_end(struct Cyc_Lexing_lexbuf*);struct Cyc_List_List{void*hd;struct Cyc_List_List*tl;};
# 54 "list.h"
struct Cyc_List_List*Cyc_List_list(struct _fat_ptr);
# 76
struct Cyc_List_List*Cyc_List_map(void*(*f)(void*),struct Cyc_List_List*x);extern char Cyc_List_List_mismatch[14U];struct Cyc_List_List_mismatch_exn_struct{char*tag;};
# 172
struct Cyc_List_List*Cyc_List_rev(struct Cyc_List_List*x);
# 178
struct Cyc_List_List*Cyc_List_imp_rev(struct Cyc_List_List*x);
# 184
struct Cyc_List_List*Cyc_List_append(struct Cyc_List_List*x,struct Cyc_List_List*y);extern char Cyc_List_Nth[4U];struct Cyc_List_Nth_exn_struct{char*tag;};struct _tuple0{struct Cyc_List_List*f1;struct Cyc_List_List*f2;};
# 294
struct _tuple0 Cyc_List_split(struct Cyc_List_List*x);
# 322
int Cyc_List_mem(int(*compare)(void*,void*),struct Cyc_List_List*l,void*x);struct Cyc_Iter_Iter{void*env;int(*next)(void*env,void*dest);};
# 37 "iter.h"
int Cyc_Iter_next(struct Cyc_Iter_Iter,void*);struct Cyc_Set_Set;
# 51 "set.h"
struct Cyc_Set_Set*Cyc_Set_empty(int(*cmp)(void*,void*));
# 63
struct Cyc_Set_Set*Cyc_Set_insert(struct Cyc_Set_Set*s,void*elt);
# 75
struct Cyc_Set_Set*Cyc_Set_union_two(struct Cyc_Set_Set*s1,struct Cyc_Set_Set*s2);
# 82
struct Cyc_Set_Set*Cyc_Set_diff(struct Cyc_Set_Set*s1,struct Cyc_Set_Set*s2);
# 85
struct Cyc_Set_Set*Cyc_Set_delete(struct Cyc_Set_Set*s,void*elt);
# 94
int Cyc_Set_cardinality(struct Cyc_Set_Set*s);
# 100
int Cyc_Set_member(struct Cyc_Set_Set*s,void*elt);extern char Cyc_Set_Absent[7U];struct Cyc_Set_Absent_exn_struct{char*tag;};
# 141
struct Cyc_Iter_Iter Cyc_Set_make_iter(struct _RegionHandle*rgn,struct Cyc_Set_Set*s);
# 38 "string.h"
unsigned long Cyc_strlen(struct _fat_ptr s);
# 49 "string.h"
int Cyc_strcmp(struct _fat_ptr s1,struct _fat_ptr s2);
int Cyc_strptrcmp(struct _fat_ptr*s1,struct _fat_ptr*s2);
int Cyc_strncmp(struct _fat_ptr s1,struct _fat_ptr s2,unsigned long len);
# 62
struct _fat_ptr Cyc_strconcat(struct _fat_ptr,struct _fat_ptr);
# 64
struct _fat_ptr Cyc_strconcat_l(struct Cyc_List_List*);
# 66
struct _fat_ptr Cyc_str_sepstr(struct Cyc_List_List*,struct _fat_ptr);
# 104 "string.h"
struct _fat_ptr Cyc_strdup(struct _fat_ptr src);
# 109
struct _fat_ptr Cyc_substring(struct _fat_ptr,int ofs,unsigned long n);struct Cyc_Hashtable_Table;
# 39 "hashtable.h"
struct Cyc_Hashtable_Table*Cyc_Hashtable_create(int sz,int(*cmp)(void*,void*),int(*hash)(void*));
# 50
void Cyc_Hashtable_insert(struct Cyc_Hashtable_Table*t,void*key,void*val);
# 52
void*Cyc_Hashtable_lookup(struct Cyc_Hashtable_Table*t,void*key);
# 78
int Cyc_Hashtable_hash_stringptr(struct _fat_ptr*p);
# 30 "filename.h"
struct _fat_ptr Cyc_Filename_concat(struct _fat_ptr,struct _fat_ptr);
# 34
struct _fat_ptr Cyc_Filename_chop_extension(struct _fat_ptr);
# 40
struct _fat_ptr Cyc_Filename_dirname(struct _fat_ptr);
# 43
struct _fat_ptr Cyc_Filename_basename(struct _fat_ptr);extern char Cyc_Arg_Bad[4U];struct Cyc_Arg_Bad_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Arg_Error[6U];struct Cyc_Arg_Error_exn_struct{char*tag;};struct Cyc_Arg_Unit_spec_Arg_Spec_struct{int tag;void(*f1)();};struct Cyc_Arg_Flag_spec_Arg_Spec_struct{int tag;void(*f1)(struct _fat_ptr);};struct Cyc_Arg_FlagString_spec_Arg_Spec_struct{int tag;void(*f1)(struct _fat_ptr,struct _fat_ptr);};struct Cyc_Arg_Set_spec_Arg_Spec_struct{int tag;int*f1;};struct Cyc_Arg_Clear_spec_Arg_Spec_struct{int tag;int*f1;};struct Cyc_Arg_String_spec_Arg_Spec_struct{int tag;void(*f1)(struct _fat_ptr);};struct Cyc_Arg_Int_spec_Arg_Spec_struct{int tag;void(*f1)(int);};struct Cyc_Arg_Rest_spec_Arg_Spec_struct{int tag;void(*f1)(struct _fat_ptr);};
# 66 "arg.h"
void Cyc_Arg_usage(struct Cyc_List_List*,struct _fat_ptr);
# 69
extern int Cyc_Arg_current;
# 71
void Cyc_Arg_parse(struct Cyc_List_List*specs,void(*anonfun)(struct _fat_ptr),int(*anonflagfun)(struct _fat_ptr),struct _fat_ptr errmsg,struct _fat_ptr args);struct Cyc_Buffer_t;
# 50 "buffer.h"
struct Cyc_Buffer_t*Cyc_Buffer_create(unsigned n);
# 58
struct _fat_ptr Cyc_Buffer_contents(struct Cyc_Buffer_t*);
# 81
void Cyc_Buffer_add_char(struct Cyc_Buffer_t*,char);
# 92 "buffer.h"
void Cyc_Buffer_add_string(struct Cyc_Buffer_t*,struct _fat_ptr);struct Cyc_PP_Ppstate;struct Cyc_PP_Out;struct Cyc_PP_Doc;
# 28 "position.h"
void Cyc_Position_reset_position(struct _fat_ptr);struct Cyc_Position_Error;struct Cyc_Relations_Reln;struct _union_Nmspace_Rel_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_Abs_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_C_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_Loc_n{int tag;int val;};union Cyc_Absyn_Nmspace{struct _union_Nmspace_Rel_n Rel_n;struct _union_Nmspace_Abs_n Abs_n;struct _union_Nmspace_C_n C_n;struct _union_Nmspace_Loc_n Loc_n;};struct _tuple1{union Cyc_Absyn_Nmspace f1;struct _fat_ptr*f2;};
# 161 "absyn.h"
enum Cyc_Absyn_Scope{Cyc_Absyn_Static =0U,Cyc_Absyn_Abstract =1U,Cyc_Absyn_Public =2U,Cyc_Absyn_Extern =3U,Cyc_Absyn_ExternC =4U,Cyc_Absyn_Register =5U};struct Cyc_Absyn_Tqual{int print_const: 1;int q_volatile: 1;int q_restrict: 1;int real_const: 1;unsigned loc;};
# 182
enum Cyc_Absyn_Size_of{Cyc_Absyn_Char_sz =0U,Cyc_Absyn_Short_sz =1U,Cyc_Absyn_Int_sz =2U,Cyc_Absyn_Long_sz =3U,Cyc_Absyn_LongLong_sz =4U};
enum Cyc_Absyn_Sign{Cyc_Absyn_Signed =0U,Cyc_Absyn_Unsigned =1U,Cyc_Absyn_None =2U};
enum Cyc_Absyn_AggrKind{Cyc_Absyn_StructA =0U,Cyc_Absyn_UnionA =1U};
# 187
enum Cyc_Absyn_AliasQual{Cyc_Absyn_Aliasable =0U,Cyc_Absyn_Unique =1U,Cyc_Absyn_Top =2U};
# 192
enum Cyc_Absyn_KindQual{Cyc_Absyn_AnyKind =0U,Cyc_Absyn_MemKind =1U,Cyc_Absyn_BoxKind =2U,Cyc_Absyn_RgnKind =3U,Cyc_Absyn_EffKind =4U,Cyc_Absyn_IntKind =5U,Cyc_Absyn_BoolKind =6U,Cyc_Absyn_PtrBndKind =7U};struct Cyc_Absyn_Kind{enum Cyc_Absyn_KindQual kind;enum Cyc_Absyn_AliasQual aliasqual;};struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct{int tag;struct Cyc_Absyn_Kind*f1;};struct Cyc_Absyn_Unknown_kb_Absyn_KindBound_struct{int tag;struct Cyc_Core_Opt*f1;};struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct{int tag;struct Cyc_Core_Opt*f1;struct Cyc_Absyn_Kind*f2;};struct Cyc_Absyn_Tvar{struct _fat_ptr*name;int identity;void*kind;};struct Cyc_Absyn_PtrLoc{unsigned ptr_loc;unsigned rgn_loc;unsigned zt_loc;};struct Cyc_Absyn_PtrAtts{void*rgn;void*nullable;void*bounds;void*zero_term;struct Cyc_Absyn_PtrLoc*ptrloc;};struct Cyc_Absyn_PtrInfo{void*elt_type;struct Cyc_Absyn_Tqual elt_tq;struct Cyc_Absyn_PtrAtts ptr_atts;};struct Cyc_Absyn_VarargInfo{struct _fat_ptr*name;struct Cyc_Absyn_Tqual tq;void*type;int inject;};struct Cyc_Absyn_Star_Absyn_Cap_struct{int tag;};struct Cyc_Absyn_Bot_Absyn_Cap_struct{int tag;};struct Cyc_Absyn_Nat_Absyn_Cap_struct{int tag;struct Cyc_Absyn_Exp*f1;int f2;};struct Cyc_Absyn_RgnEffect{void*name;void*rgn;void*lock;void*unadopt;void*parent;};struct Cyc_Absyn_FnInfo{struct Cyc_List_List*tvars;void*effect;struct Cyc_Absyn_Tqual ret_tqual;void*ret_type;struct Cyc_List_List*args;int c_varargs;struct Cyc_Absyn_VarargInfo*cyc_varargs;struct Cyc_List_List*rgn_po;struct Cyc_List_List*attributes;struct Cyc_Absyn_Exp*requires_clause;struct Cyc_List_List*requires_relns;struct Cyc_Absyn_Exp*ensures_clause;struct Cyc_List_List*ensures_relns;struct Cyc_List_List*ieffect;struct Cyc_List_List*oeffect;};struct Cyc_Absyn_UnknownDatatypeInfo{struct _tuple1*name;int is_extensible;};struct _union_DatatypeInfo_UnknownDatatype{int tag;struct Cyc_Absyn_UnknownDatatypeInfo val;};struct _union_DatatypeInfo_KnownDatatype{int tag;struct Cyc_Absyn_Datatypedecl**val;};union Cyc_Absyn_DatatypeInfo{struct _union_DatatypeInfo_UnknownDatatype UnknownDatatype;struct _union_DatatypeInfo_KnownDatatype KnownDatatype;};struct Cyc_Absyn_UnknownDatatypeFieldInfo{struct _tuple1*datatype_name;struct _tuple1*field_name;int is_extensible;};struct _union_DatatypeFieldInfo_UnknownDatatypefield{int tag;struct Cyc_Absyn_UnknownDatatypeFieldInfo val;};struct _tuple2{struct Cyc_Absyn_Datatypedecl*f1;struct Cyc_Absyn_Datatypefield*f2;};struct _union_DatatypeFieldInfo_KnownDatatypefield{int tag;struct _tuple2 val;};union Cyc_Absyn_DatatypeFieldInfo{struct _union_DatatypeFieldInfo_UnknownDatatypefield UnknownDatatypefield;struct _union_DatatypeFieldInfo_KnownDatatypefield KnownDatatypefield;};struct _tuple3{enum Cyc_Absyn_AggrKind f1;struct _tuple1*f2;struct Cyc_Core_Opt*f3;};struct _union_AggrInfo_UnknownAggr{int tag;struct _tuple3 val;};struct _union_AggrInfo_KnownAggr{int tag;struct Cyc_Absyn_Aggrdecl**val;};union Cyc_Absyn_AggrInfo{struct _union_AggrInfo_UnknownAggr UnknownAggr;struct _union_AggrInfo_KnownAggr KnownAggr;};struct Cyc_Absyn_ArrayInfo{void*elt_type;struct Cyc_Absyn_Tqual tq;struct Cyc_Absyn_Exp*num_elts;void*zero_term;unsigned zt_loc;};struct Cyc_Absyn_Aggr_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Aggrdecl*f1;};struct Cyc_Absyn_Enum_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;};struct Cyc_Absyn_Datatype_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;};struct Cyc_Absyn_TypeDecl{void*r;unsigned loc;};struct Cyc_Absyn_VoidCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_IntCon_Absyn_TyCon_struct{int tag;enum Cyc_Absyn_Sign f1;enum Cyc_Absyn_Size_of f2;};struct Cyc_Absyn_FloatCon_Absyn_TyCon_struct{int tag;int f1;};struct Cyc_Absyn_RgnHandleCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_TagCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_HeapCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_UniqueCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_RefCntCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_AccessCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_JoinCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_RgnsCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_TrueCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_FalseCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_ThinCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_FatCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_EnumCon_Absyn_TyCon_struct{int tag;struct _tuple1*f1;struct Cyc_Absyn_Enumdecl*f2;};struct Cyc_Absyn_AnonEnumCon_Absyn_TyCon_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_BuiltinCon_Absyn_TyCon_struct{int tag;struct _fat_ptr f1;struct Cyc_Absyn_Kind*f2;};struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_DatatypeInfo f1;};struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_DatatypeFieldInfo f1;};struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_AggrInfo f1;};struct Cyc_Absyn_AppType_Absyn_Type_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Evar_Absyn_Type_struct{int tag;struct Cyc_Core_Opt*f1;void*f2;int f3;struct Cyc_Core_Opt*f4;};struct Cyc_Absyn_VarType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Tvar*f1;};struct Cyc_Absyn_PointerType_Absyn_Type_struct{int tag;struct Cyc_Absyn_PtrInfo f1;};struct Cyc_Absyn_ArrayType_Absyn_Type_struct{int tag;struct Cyc_Absyn_ArrayInfo f1;};struct Cyc_Absyn_FnType_Absyn_Type_struct{int tag;struct Cyc_Absyn_FnInfo f1;};struct Cyc_Absyn_TupleType_Absyn_Type_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct{int tag;enum Cyc_Absyn_AggrKind f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_TypedefType_Absyn_Type_struct{int tag;struct _tuple1*f1;struct Cyc_List_List*f2;struct Cyc_Absyn_Typedefdecl*f3;void*f4;};struct Cyc_Absyn_ValueofType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct{int tag;struct Cyc_Absyn_TypeDecl*f1;void**f2;};struct Cyc_Absyn_TypeofType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_NoTypes_Absyn_Funcparams_struct{int tag;struct Cyc_List_List*f1;unsigned f2;};struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct{int tag;struct Cyc_List_List*f1;int f2;struct Cyc_Absyn_VarargInfo*f3;void*f4;struct Cyc_List_List*f5;struct Cyc_Absyn_Exp*f6;struct Cyc_Absyn_Exp*f7;struct Cyc_List_List*f8;struct Cyc_List_List*f9;};
# 437 "absyn.h"
enum Cyc_Absyn_Format_Type{Cyc_Absyn_Printf_ft =0U,Cyc_Absyn_Scanf_ft =1U};struct Cyc_Absyn_Regparm_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Stdcall_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Cdecl_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Fastcall_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Noreturn_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Const_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Aligned_att_Absyn_Attribute_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Packed_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Section_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Nocommon_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Shared_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Unused_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Weak_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Dllimport_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Dllexport_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_instrument_function_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Constructor_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Destructor_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_check_memory_usage_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Format_att_Absyn_Attribute_struct{int tag;enum Cyc_Absyn_Format_Type f1;int f2;int f3;};struct Cyc_Absyn_Initializes_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Noliveunique_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Consume_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Pure_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Mode_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Alias_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Always_inline_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_throw_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Non_null_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Deprecated_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Carray_mod_Absyn_Type_modifier_struct{int tag;void*f1;unsigned f2;};struct Cyc_Absyn_ConstArray_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_Absyn_Exp*f1;void*f2;unsigned f3;};struct Cyc_Absyn_Pointer_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_Absyn_PtrAtts f1;struct Cyc_Absyn_Tqual f2;};struct Cyc_Absyn_Function_mod_Absyn_Type_modifier_struct{int tag;void*f1;};struct Cyc_Absyn_TypeParams_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_List_List*f1;unsigned f2;int f3;};struct Cyc_Absyn_Attributes_mod_Absyn_Type_modifier_struct{int tag;unsigned f1;struct Cyc_List_List*f2;};struct _union_Cnst_Null_c{int tag;int val;};struct _tuple4{enum Cyc_Absyn_Sign f1;char f2;};struct _union_Cnst_Char_c{int tag;struct _tuple4 val;};struct _union_Cnst_Wchar_c{int tag;struct _fat_ptr val;};struct _tuple5{enum Cyc_Absyn_Sign f1;short f2;};struct _union_Cnst_Short_c{int tag;struct _tuple5 val;};struct _tuple6{enum Cyc_Absyn_Sign f1;int f2;};struct _union_Cnst_Int_c{int tag;struct _tuple6 val;};struct _tuple7{enum Cyc_Absyn_Sign f1;long long f2;};struct _union_Cnst_LongLong_c{int tag;struct _tuple7 val;};struct _tuple8{struct _fat_ptr f1;int f2;};struct _union_Cnst_Float_c{int tag;struct _tuple8 val;};struct _union_Cnst_String_c{int tag;struct _fat_ptr val;};struct _union_Cnst_Wstring_c{int tag;struct _fat_ptr val;};union Cyc_Absyn_Cnst{struct _union_Cnst_Null_c Null_c;struct _union_Cnst_Char_c Char_c;struct _union_Cnst_Wchar_c Wchar_c;struct _union_Cnst_Short_c Short_c;struct _union_Cnst_Int_c Int_c;struct _union_Cnst_LongLong_c LongLong_c;struct _union_Cnst_Float_c Float_c;struct _union_Cnst_String_c String_c;struct _union_Cnst_Wstring_c Wstring_c;};
# 532
enum Cyc_Absyn_Primop{Cyc_Absyn_Plus =0U,Cyc_Absyn_Times =1U,Cyc_Absyn_Minus =2U,Cyc_Absyn_Div =3U,Cyc_Absyn_Mod =4U,Cyc_Absyn_Eq =5U,Cyc_Absyn_Neq =6U,Cyc_Absyn_Gt =7U,Cyc_Absyn_Lt =8U,Cyc_Absyn_Gte =9U,Cyc_Absyn_Lte =10U,Cyc_Absyn_Not =11U,Cyc_Absyn_Bitnot =12U,Cyc_Absyn_Bitand =13U,Cyc_Absyn_Bitor =14U,Cyc_Absyn_Bitxor =15U,Cyc_Absyn_Bitlshift =16U,Cyc_Absyn_Bitlrshift =17U,Cyc_Absyn_Numelts =18U};
# 539
enum Cyc_Absyn_Incrementor{Cyc_Absyn_PreInc =0U,Cyc_Absyn_PostInc =1U,Cyc_Absyn_PreDec =2U,Cyc_Absyn_PostDec =3U};struct Cyc_Absyn_VarargCallInfo{int num_varargs;struct Cyc_List_List*injectors;struct Cyc_Absyn_VarargInfo*vai;};struct Cyc_Absyn_StructField_Absyn_OffsetofField_struct{int tag;struct _fat_ptr*f1;};struct Cyc_Absyn_TupleIndex_Absyn_OffsetofField_struct{int tag;unsigned f1;};
# 557
enum Cyc_Absyn_Coercion{Cyc_Absyn_Unknown_coercion =0U,Cyc_Absyn_No_coercion =1U,Cyc_Absyn_Null_to_NonNull =2U,Cyc_Absyn_Other_coercion =3U};struct Cyc_Absyn_MallocInfo{int is_calloc;struct Cyc_Absyn_Exp*rgn;void**elt_type;struct Cyc_Absyn_Exp*num_elts;int fat_result;int inline_call;};struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct{int tag;union Cyc_Absyn_Cnst f1;};struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Pragma_e_Absyn_Raw_exp_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct{int tag;enum Cyc_Absyn_Primop f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_AssignOp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Core_Opt*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_Increment_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;enum Cyc_Absyn_Incrementor f2;};struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;struct Cyc_Absyn_VarargCallInfo*f3;int f4;};struct Cyc_Absyn_Throw_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;int f2;};struct Cyc_Absyn_NoInstantiate_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Instantiate_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_Absyn_Exp*f2;int f3;enum Cyc_Absyn_Coercion f4;};struct Cyc_Absyn_Address_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_New_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Sizeoftype_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Sizeofexp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Offsetof_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Deref_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;int f3;int f4;};struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;int f3;int f4;};struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Tuple_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;};struct _tuple9{struct _fat_ptr*f1;struct Cyc_Absyn_Tqual f2;void*f3;};struct Cyc_Absyn_CompoundLit_e_Absyn_Raw_exp_struct{int tag;struct _tuple9*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Array_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Exp*f2;struct Cyc_Absyn_Exp*f3;int f4;};struct Cyc_Absyn_ComprehensionNoinit_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;void*f2;int f3;};struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct{int tag;struct _tuple1*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;struct Cyc_Absyn_Aggrdecl*f4;};struct Cyc_Absyn_AnonStruct_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Datatype_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;struct Cyc_Absyn_Datatypedecl*f2;struct Cyc_Absyn_Datatypefield*f3;};struct Cyc_Absyn_Enum_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_AnonEnum_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_Spawn_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_MallocInfo f1;};struct Cyc_Absyn_Swap_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_UnresolvedMem_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Core_Opt*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_StmtExp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Stmt*f1;};struct Cyc_Absyn_Tagcheck_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;};struct Cyc_Absyn_Valueof_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Asm_e_Absyn_Raw_exp_struct{int tag;int f1;struct _fat_ptr f2;struct Cyc_List_List*f3;struct Cyc_List_List*f4;struct Cyc_List_List*f5;};struct Cyc_Absyn_Extension_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Exp{void*topt;void*r;unsigned loc;void*annot;};struct Cyc_Absyn_Skip_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Exp_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Seq_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Return_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_IfThenElse_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Stmt*f2;struct Cyc_Absyn_Stmt*f3;};struct _tuple10{struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_While_s_Absyn_Raw_stmt_struct{int tag;struct _tuple10 f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Break_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Continue_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Goto_s_Absyn_Raw_stmt_struct{int tag;struct _fat_ptr*f1;};struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _tuple10 f2;struct _tuple10 f3;struct Cyc_Absyn_Stmt*f4;};struct Cyc_Absyn_Switch_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;void*f3;};struct Cyc_Absyn_Fallthru_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_List_List*f1;struct Cyc_Absyn_Switch_clause**f2;};struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Decl*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Label_s_Absyn_Raw_stmt_struct{int tag;struct _fat_ptr*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Do_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct _tuple10 f2;};struct Cyc_Absyn_TryCatch_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct Cyc_List_List*f2;void*f3;};struct Cyc_Absyn_Stmt{void*r;unsigned loc;void*annot;};struct Cyc_Absyn_Wild_p_Absyn_Raw_pat_struct{int tag;};struct Cyc_Absyn_Var_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Pat*f2;};struct Cyc_Absyn_AliasVar_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;};struct Cyc_Absyn_Reference_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Pat*f2;};struct Cyc_Absyn_TagInt_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;};struct Cyc_Absyn_Tuple_p_Absyn_Raw_pat_struct{int tag;struct Cyc_List_List*f1;int f2;};struct Cyc_Absyn_Pointer_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Pat*f1;};struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct{int tag;union Cyc_Absyn_AggrInfo*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;int f4;};struct Cyc_Absyn_Datatype_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;struct Cyc_Absyn_Datatypefield*f2;struct Cyc_List_List*f3;int f4;};struct Cyc_Absyn_Null_p_Absyn_Raw_pat_struct{int tag;};struct Cyc_Absyn_Int_p_Absyn_Raw_pat_struct{int tag;enum Cyc_Absyn_Sign f1;int f2;};struct Cyc_Absyn_Char_p_Absyn_Raw_pat_struct{int tag;char f1;};struct Cyc_Absyn_Float_p_Absyn_Raw_pat_struct{int tag;struct _fat_ptr f1;int f2;};struct Cyc_Absyn_Enum_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_AnonEnum_p_Absyn_Raw_pat_struct{int tag;void*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_UnknownId_p_Absyn_Raw_pat_struct{int tag;struct _tuple1*f1;};struct Cyc_Absyn_UnknownCall_p_Absyn_Raw_pat_struct{int tag;struct _tuple1*f1;struct Cyc_List_List*f2;int f3;};struct Cyc_Absyn_Exp_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Pat{void*r;void*topt;unsigned loc;};struct Cyc_Absyn_Switch_clause{struct Cyc_Absyn_Pat*pattern;struct Cyc_Core_Opt*pat_vars;struct Cyc_Absyn_Exp*where_clause;struct Cyc_Absyn_Stmt*body;unsigned loc;};struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct{int tag;struct _tuple1*f1;};struct Cyc_Absyn_Global_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Funname_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Fndecl*f1;};struct Cyc_Absyn_Param_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Local_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Pat_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Vardecl{enum Cyc_Absyn_Scope sc;struct _tuple1*name;unsigned varloc;struct Cyc_Absyn_Tqual tq;void*type;struct Cyc_Absyn_Exp*initializer;void*rgn;struct Cyc_List_List*attributes;int escapes;};struct Cyc_Absyn_Fndecl{enum Cyc_Absyn_Scope sc;int is_inline;struct _tuple1*name;struct Cyc_Absyn_Stmt*body;struct Cyc_Absyn_FnInfo i;void*cached_type;struct Cyc_Core_Opt*param_vardecls;struct Cyc_Absyn_Vardecl*fn_vardecl;};struct Cyc_Absyn_Aggrfield{struct _fat_ptr*name;struct Cyc_Absyn_Tqual tq;void*type;struct Cyc_Absyn_Exp*width;struct Cyc_List_List*attributes;struct Cyc_Absyn_Exp*requires_clause;};struct Cyc_Absyn_AggrdeclImpl{struct Cyc_List_List*exist_vars;struct Cyc_List_List*rgn_po;struct Cyc_List_List*fields;int tagged;};struct Cyc_Absyn_Aggrdecl{enum Cyc_Absyn_AggrKind kind;enum Cyc_Absyn_Scope sc;struct _tuple1*name;struct Cyc_List_List*tvs;struct Cyc_Absyn_AggrdeclImpl*impl;struct Cyc_List_List*attributes;int expected_mem_kind;};struct Cyc_Absyn_Datatypefield{struct _tuple1*name;struct Cyc_List_List*typs;unsigned loc;enum Cyc_Absyn_Scope sc;};struct Cyc_Absyn_Datatypedecl{enum Cyc_Absyn_Scope sc;struct _tuple1*name;struct Cyc_List_List*tvs;struct Cyc_Core_Opt*fields;int is_extensible;};struct Cyc_Absyn_Enumfield{struct _tuple1*name;struct Cyc_Absyn_Exp*tag;unsigned loc;};struct Cyc_Absyn_Enumdecl{enum Cyc_Absyn_Scope sc;struct _tuple1*name;struct Cyc_Core_Opt*fields;};struct Cyc_Absyn_Typedefdecl{struct _tuple1*name;struct Cyc_Absyn_Tqual tq;struct Cyc_List_List*tvs;struct Cyc_Core_Opt*kind;void*defn;struct Cyc_List_List*atts;int extern_c;};struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Fn_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Fndecl*f1;};struct Cyc_Absyn_Let_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Pat*f1;struct Cyc_Core_Opt*f2;struct Cyc_Absyn_Exp*f3;void*f4;};struct Cyc_Absyn_Letv_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_Region_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_Aggr_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Aggrdecl*f1;};struct Cyc_Absyn_Datatype_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;};struct Cyc_Absyn_Enum_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;};struct Cyc_Absyn_Typedef_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Typedefdecl*f1;};struct Cyc_Absyn_Namespace_d_Absyn_Raw_decl_struct{int tag;struct _fat_ptr*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Using_d_Absyn_Raw_decl_struct{int tag;struct _tuple1*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_ExternC_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;};struct _tuple11{unsigned f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_ExternCinclude_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;struct _tuple11*f4;};struct Cyc_Absyn_Porton_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Portoff_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Tempeston_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Tempestoff_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Decl{void*r;unsigned loc;};struct Cyc_Absyn_ArrayElement_Absyn_Designator_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_FieldName_Absyn_Designator_struct{int tag;struct _fat_ptr*f1;};extern char Cyc_Absyn_EmptyAnnot[11U];struct Cyc_Absyn_EmptyAnnot_Absyn_AbsynAnnot_struct{char*tag;};
# 1107 "absyn.h"
struct Cyc_Absyn_Decl*Cyc_Absyn_new_decl(void*,unsigned);
# 1160
struct Cyc_Absyn_Decl*Cyc_Absyn_lookup_decl(struct Cyc_List_List*decls,struct _fat_ptr*name);struct _tuple12{enum Cyc_Absyn_AggrKind f1;struct _tuple1*f2;};
# 1174
struct _tuple12 Cyc_Absyn_aggr_kinded_name(union Cyc_Absyn_AggrInfo);
# 1182
struct _tuple1*Cyc_Absyn_binding2qvar(void*);struct Cyc_Absynpp_Params{int expand_typedefs;int qvar_to_Cids;int add_cyc_prefix;int to_VC;int decls_first;int rewrite_temp_tvars;int print_all_tvars;int print_all_kinds;int print_all_effects;int print_using_stmts;int print_externC_stmts;int print_full_evars;int print_zeroterm;int generate_line_directives;int use_curr_namespace;struct Cyc_List_List*curr_namespace;};
# 53 "absynpp.h"
void Cyc_Absynpp_set_params(struct Cyc_Absynpp_Params*fs);
# 55
extern struct Cyc_Absynpp_Params Cyc_Absynpp_cyc_params_r;
# 57
void Cyc_Absynpp_decllist2file(struct Cyc_List_List*tdl,struct Cyc___cycFILE*f);
# 62
struct _fat_ptr Cyc_Absynpp_typ2string(void*);
# 28 "parse.h"
struct Cyc_List_List*Cyc_Parse_parse_file(struct Cyc___cycFILE*f,struct _fat_ptr name);extern char Cyc_Parse_Exit[5U];struct Cyc_Parse_Exit_exn_struct{char*tag;};struct Cyc_FlatList{struct Cyc_FlatList*tl;void*hd[0U] __attribute__((aligned )) ;};struct Cyc_Type_specifier{int Signed_spec: 1;int Unsigned_spec: 1;int Short_spec: 1;int Long_spec: 1;int Long_Long_spec: 1;int Valid_type_spec: 1;void*Type_spec;unsigned loc;};struct Cyc_Declarator{struct _tuple1*id;struct Cyc_List_List*tms;};struct _tuple14{struct Cyc_Declarator f1;struct Cyc_Absyn_Exp*f2;};struct _tuple13{struct _tuple13*tl;struct _tuple14 hd  __attribute__((aligned )) ;};
# 52
enum Cyc_Storage_class{Cyc_Typedef_sc =0U,Cyc_Extern_sc =1U,Cyc_ExternC_sc =2U,Cyc_Static_sc =3U,Cyc_Auto_sc =4U,Cyc_Register_sc =5U,Cyc_Abstract_sc =6U};struct Cyc_Declaration_spec{enum Cyc_Storage_class*sc;struct Cyc_Absyn_Tqual tq;struct Cyc_Type_specifier type_specs;int is_inline;struct Cyc_List_List*attributes;};struct Cyc_Abstractdeclarator{struct Cyc_List_List*tms;};struct Cyc_Numelts_ptrqual_Pointer_qual_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Region_ptrqual_Pointer_qual_struct{int tag;void*f1;};struct Cyc_Thin_ptrqual_Pointer_qual_struct{int tag;};struct Cyc_Fat_ptrqual_Pointer_qual_struct{int tag;};struct Cyc_Zeroterm_ptrqual_Pointer_qual_struct{int tag;};struct Cyc_Nozeroterm_ptrqual_Pointer_qual_struct{int tag;};struct Cyc_Notnull_ptrqual_Pointer_qual_struct{int tag;};struct Cyc_Nullable_ptrqual_Pointer_qual_struct{int tag;};struct _union_YYSTYPE_Int_tok{int tag;union Cyc_Absyn_Cnst val;};struct _union_YYSTYPE_Char_tok{int tag;char val;};struct _union_YYSTYPE_String_tok{int tag;struct _fat_ptr val;};struct _union_YYSTYPE_Stringopt_tok{int tag;struct Cyc_Core_Opt*val;};struct _union_YYSTYPE_QualId_tok{int tag;struct _tuple1*val;};struct _tuple15{int f1;struct _fat_ptr f2;};struct _union_YYSTYPE_Asm_tok{int tag;struct _tuple15 val;};struct _union_YYSTYPE_Exp_tok{int tag;struct Cyc_Absyn_Exp*val;};struct _union_YYSTYPE_Stmt_tok{int tag;struct Cyc_Absyn_Stmt*val;};struct _tuple16{unsigned f1;void*f2;void*f3;};struct _union_YYSTYPE_YY1{int tag;struct _tuple16*val;};struct _union_YYSTYPE_YY2{int tag;void*val;};struct _union_YYSTYPE_YY3{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY4{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY5{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY6{int tag;enum Cyc_Absyn_Primop val;};struct _union_YYSTYPE_YY7{int tag;struct Cyc_Core_Opt*val;};struct _union_YYSTYPE_YY8{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY9{int tag;struct Cyc_Absyn_Pat*val;};struct _tuple17{struct Cyc_List_List*f1;int f2;};struct _union_YYSTYPE_YY10{int tag;struct _tuple17*val;};struct _union_YYSTYPE_YY11{int tag;struct Cyc_List_List*val;};struct _tuple18{struct Cyc_List_List*f1;struct Cyc_Absyn_Pat*f2;};struct _union_YYSTYPE_YY12{int tag;struct _tuple18*val;};struct _union_YYSTYPE_YY13{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY14{int tag;struct _tuple17*val;};struct _union_YYSTYPE_YY15{int tag;struct Cyc_Absyn_Fndecl*val;};struct _union_YYSTYPE_YY16{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY17{int tag;struct Cyc_Declaration_spec val;};struct _union_YYSTYPE_YY18{int tag;struct _tuple14 val;};struct _union_YYSTYPE_YY19{int tag;struct _tuple13*val;};struct _union_YYSTYPE_YY20{int tag;enum Cyc_Storage_class*val;};struct _union_YYSTYPE_YY21{int tag;struct Cyc_Type_specifier val;};struct _union_YYSTYPE_YY22{int tag;enum Cyc_Absyn_AggrKind val;};struct _union_YYSTYPE_YY23{int tag;struct Cyc_Absyn_Tqual val;};struct _union_YYSTYPE_YY24{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY25{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY26{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY27{int tag;struct Cyc_Declarator val;};struct _tuple19{struct Cyc_Declarator f1;struct Cyc_Absyn_Exp*f2;struct Cyc_Absyn_Exp*f3;};struct _union_YYSTYPE_YY28{int tag;struct _tuple19*val;};struct _union_YYSTYPE_YY29{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY30{int tag;struct Cyc_Abstractdeclarator val;};struct _union_YYSTYPE_YY31{int tag;int val;};struct _union_YYSTYPE_YY32{int tag;enum Cyc_Absyn_Scope val;};struct _union_YYSTYPE_YY33{int tag;struct Cyc_Absyn_Datatypefield*val;};struct _union_YYSTYPE_YY34{int tag;struct Cyc_List_List*val;};struct _tuple20{struct Cyc_Absyn_Tqual f1;struct Cyc_Type_specifier f2;struct Cyc_List_List*f3;};struct _union_YYSTYPE_YY35{int tag;struct _tuple20 val;};struct _union_YYSTYPE_YY36{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY37{int tag;struct _tuple9*val;};struct _union_YYSTYPE_YY38{int tag;struct Cyc_List_List*val;};struct _tuple21{struct Cyc_List_List*f1;int f2;struct Cyc_Absyn_VarargInfo*f3;void*f4;struct Cyc_List_List*f5;};struct _union_YYSTYPE_YY39{int tag;struct _tuple21*val;};struct _union_YYSTYPE_YY40{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY41{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY42{int tag;void*val;};struct _union_YYSTYPE_YY43{int tag;struct Cyc_Absyn_Kind*val;};struct _union_YYSTYPE_YY44{int tag;void*val;};struct _union_YYSTYPE_YY45{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY46{int tag;void*val;};struct _union_YYSTYPE_YY47{int tag;struct Cyc_Absyn_Enumfield*val;};struct _union_YYSTYPE_YY48{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY49{int tag;void*val;};struct _union_YYSTYPE_YY50{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY51{int tag;void*val;};struct _union_YYSTYPE_YY52{int tag;struct Cyc_List_List*val;};struct _tuple22{struct Cyc_List_List*f1;unsigned f2;};struct _union_YYSTYPE_YY53{int tag;struct _tuple22*val;};struct _union_YYSTYPE_YY54{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY55{int tag;void*val;};struct _union_YYSTYPE_YY56{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY57{int tag;struct Cyc_Absyn_Exp*val;};struct _union_YYSTYPE_YY58{int tag;void*val;};struct _tuple23{struct Cyc_List_List*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;};struct _union_YYSTYPE_YY59{int tag;struct _tuple23*val;};struct _union_YYSTYPE_YY60{int tag;struct _tuple0*val;};struct _union_YYSTYPE_YY61{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY62{int tag;struct Cyc_List_List*val;};struct _tuple24{struct _fat_ptr f1;struct Cyc_Absyn_Exp*f2;};struct _union_YYSTYPE_YY63{int tag;struct _tuple24*val;};struct _union_YYSTYPE_YY64{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YYINITIALSVAL{int tag;int val;};union Cyc_YYSTYPE{struct _union_YYSTYPE_Int_tok Int_tok;struct _union_YYSTYPE_Char_tok Char_tok;struct _union_YYSTYPE_String_tok String_tok;struct _union_YYSTYPE_Stringopt_tok Stringopt_tok;struct _union_YYSTYPE_QualId_tok QualId_tok;struct _union_YYSTYPE_Asm_tok Asm_tok;struct _union_YYSTYPE_Exp_tok Exp_tok;struct _union_YYSTYPE_Stmt_tok Stmt_tok;struct _union_YYSTYPE_YY1 YY1;struct _union_YYSTYPE_YY2 YY2;struct _union_YYSTYPE_YY3 YY3;struct _union_YYSTYPE_YY4 YY4;struct _union_YYSTYPE_YY5 YY5;struct _union_YYSTYPE_YY6 YY6;struct _union_YYSTYPE_YY7 YY7;struct _union_YYSTYPE_YY8 YY8;struct _union_YYSTYPE_YY9 YY9;struct _union_YYSTYPE_YY10 YY10;struct _union_YYSTYPE_YY11 YY11;struct _union_YYSTYPE_YY12 YY12;struct _union_YYSTYPE_YY13 YY13;struct _union_YYSTYPE_YY14 YY14;struct _union_YYSTYPE_YY15 YY15;struct _union_YYSTYPE_YY16 YY16;struct _union_YYSTYPE_YY17 YY17;struct _union_YYSTYPE_YY18 YY18;struct _union_YYSTYPE_YY19 YY19;struct _union_YYSTYPE_YY20 YY20;struct _union_YYSTYPE_YY21 YY21;struct _union_YYSTYPE_YY22 YY22;struct _union_YYSTYPE_YY23 YY23;struct _union_YYSTYPE_YY24 YY24;struct _union_YYSTYPE_YY25 YY25;struct _union_YYSTYPE_YY26 YY26;struct _union_YYSTYPE_YY27 YY27;struct _union_YYSTYPE_YY28 YY28;struct _union_YYSTYPE_YY29 YY29;struct _union_YYSTYPE_YY30 YY30;struct _union_YYSTYPE_YY31 YY31;struct _union_YYSTYPE_YY32 YY32;struct _union_YYSTYPE_YY33 YY33;struct _union_YYSTYPE_YY34 YY34;struct _union_YYSTYPE_YY35 YY35;struct _union_YYSTYPE_YY36 YY36;struct _union_YYSTYPE_YY37 YY37;struct _union_YYSTYPE_YY38 YY38;struct _union_YYSTYPE_YY39 YY39;struct _union_YYSTYPE_YY40 YY40;struct _union_YYSTYPE_YY41 YY41;struct _union_YYSTYPE_YY42 YY42;struct _union_YYSTYPE_YY43 YY43;struct _union_YYSTYPE_YY44 YY44;struct _union_YYSTYPE_YY45 YY45;struct _union_YYSTYPE_YY46 YY46;struct _union_YYSTYPE_YY47 YY47;struct _union_YYSTYPE_YY48 YY48;struct _union_YYSTYPE_YY49 YY49;struct _union_YYSTYPE_YY50 YY50;struct _union_YYSTYPE_YY51 YY51;struct _union_YYSTYPE_YY52 YY52;struct _union_YYSTYPE_YY53 YY53;struct _union_YYSTYPE_YY54 YY54;struct _union_YYSTYPE_YY55 YY55;struct _union_YYSTYPE_YY56 YY56;struct _union_YYSTYPE_YY57 YY57;struct _union_YYSTYPE_YY58 YY58;struct _union_YYSTYPE_YY59 YY59;struct _union_YYSTYPE_YY60 YY60;struct _union_YYSTYPE_YY61 YY61;struct _union_YYSTYPE_YY62 YY62;struct _union_YYSTYPE_YY63 YY63;struct _union_YYSTYPE_YY64 YY64;struct _union_YYSTYPE_YYINITIALSVAL YYINITIALSVAL;};struct Cyc_Yyltype{int timestamp;int first_line;int first_column;int last_line;int last_column;};struct Cyc_Dict_T;struct Cyc_Dict_Dict{int(*rel)(void*,void*);struct _RegionHandle*r;const struct Cyc_Dict_T*t;};extern char Cyc_Dict_Present[8U];struct Cyc_Dict_Present_exn_struct{char*tag;};extern char Cyc_Dict_Absent[7U];struct Cyc_Dict_Absent_exn_struct{char*tag;};struct Cyc_RgnOrder_RgnPO;extern char Cyc_Tcenv_Env_error[10U];struct Cyc_Tcenv_Env_error_exn_struct{char*tag;};struct Cyc_Tcenv_Genv{struct Cyc_Dict_Dict aggrdecls;struct Cyc_Dict_Dict datatypedecls;struct Cyc_Dict_Dict enumdecls;struct Cyc_Dict_Dict typedefs;struct Cyc_Dict_Dict ordinaries;};struct Cyc_Tcenv_Fenv;struct Cyc_Tcenv_Tenv{struct Cyc_List_List*ns;struct Cyc_Tcenv_Genv*ae;struct Cyc_Tcenv_Fenv*le;int allow_valueof: 1;int in_extern_c_include: 1;int in_tempest: 1;int tempest_generalize: 1;};
# 68 "tcenv.h"
struct Cyc_Tcenv_Tenv*Cyc_Tcenv_tc_init();
# 88
enum Cyc_Tcenv_NewStatus{Cyc_Tcenv_NoneNew =0U,Cyc_Tcenv_InNew =1U,Cyc_Tcenv_InNewAggr =2U};
# 33 "tc.h"
void Cyc_Tc_tc(struct Cyc_Tcenv_Tenv*te,int var_default_init,struct Cyc_List_List*ds);extern char Cyc_Tcdecl_Incompatible[13U];struct Cyc_Tcdecl_Incompatible_exn_struct{char*tag;};struct Cyc_Tcdecl_Xdatatypefielddecl{struct Cyc_Absyn_Datatypedecl*base;struct Cyc_Absyn_Datatypefield*field;};
# 32 "cifc.h"
void Cyc_Cifc_merge_sys_user_decl(unsigned,int is_buildlib,struct Cyc_Absyn_Decl*user_decl,struct Cyc_Absyn_Decl*c_decl);struct _union_RelnOp_RConst{int tag;unsigned val;};struct _union_RelnOp_RVar{int tag;struct Cyc_Absyn_Vardecl*val;};struct _union_RelnOp_RNumelts{int tag;struct Cyc_Absyn_Vardecl*val;};struct _union_RelnOp_RType{int tag;void*val;};struct _union_RelnOp_RParam{int tag;unsigned val;};struct _union_RelnOp_RParamNumelts{int tag;unsigned val;};struct _union_RelnOp_RReturn{int tag;unsigned val;};union Cyc_Relations_RelnOp{struct _union_RelnOp_RConst RConst;struct _union_RelnOp_RVar RVar;struct _union_RelnOp_RNumelts RNumelts;struct _union_RelnOp_RType RType;struct _union_RelnOp_RParam RParam;struct _union_RelnOp_RParamNumelts RParamNumelts;struct _union_RelnOp_RReturn RReturn;};
# 50 "relations-ap.h"
enum Cyc_Relations_Relation{Cyc_Relations_Req =0U,Cyc_Relations_Rneq =1U,Cyc_Relations_Rlte =2U,Cyc_Relations_Rlt =3U};struct Cyc_Relations_Reln{union Cyc_Relations_RelnOp rop1;enum Cyc_Relations_Relation relation;union Cyc_Relations_RelnOp rop2;};
# 30 "binding.h"
void Cyc_Binding_resolve_all(struct Cyc_List_List*tds);struct Cyc_Binding_Namespace_Binding_NSDirective_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Binding_Using_Binding_NSDirective_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Binding_NSCtxt{struct Cyc_List_List*curr_ns;struct Cyc_List_List*availables;struct Cyc_Dict_Dict ns_data;};
# 29 "specsfile.h"
struct _fat_ptr Cyc_Specsfile_target_arch;
void Cyc_Specsfile_set_target_arch(struct _fat_ptr s);
struct Cyc_List_List*Cyc_Specsfile_cyclone_exec_path;
void Cyc_Specsfile_add_cyclone_exec_path(struct _fat_ptr s);
# 34
struct Cyc_List_List*Cyc_Specsfile_read_specs(struct _fat_ptr file);
# 36
struct _fat_ptr Cyc_Specsfile_get_spec(struct Cyc_List_List*specs,struct _fat_ptr spec_name);
struct Cyc_List_List*Cyc_Specsfile_cyclone_arch_path;
struct _fat_ptr Cyc_Specsfile_def_lib_path;
struct _fat_ptr Cyc_Specsfile_parse_b(struct Cyc_List_List*specs,void(*anonfun)(struct _fat_ptr),int(*anonflagfun)(struct _fat_ptr),struct _fat_ptr errmsg,struct _fat_ptr argv);
# 44
struct _fat_ptr Cyc_Specsfile_find_in_arch_path(struct _fat_ptr s);
# 80 "buildlib.cyl"
void Cyc_Lex_lex_init(int use_cyclone_keywords);static char _tmp0[4U]="gcc";
# 90
static struct _fat_ptr Cyc_cyclone_cc={_tmp0,_tmp0,_tmp0 + 4U};static char _tmp1[1U]="";
static struct _fat_ptr Cyc_target_cflags={_tmp1,_tmp1,_tmp1 + 1U};
# 93
static int Cyc_do_setjmp=0;
static int Cyc_verbose=0;
int Cyc_noexpand_r=1;
# 97
struct Cyc___cycFILE*Cyc_log_file=0;
struct Cyc___cycFILE*Cyc_cstubs_file=0;
struct Cyc___cycFILE*Cyc_cycstubs_file=0;
int Cyc_count=0;
# 102
int Cyc_log(struct _fat_ptr fmt,struct _fat_ptr ap){
# 105
if(Cyc_log_file == 0){
({void*_tmp2=0U;({struct Cyc___cycFILE*_tmp3C9=Cyc_stderr;struct _fat_ptr _tmp3C8=({const char*_tmp3="Internal error: log file is NULL\n";_tag_fat(_tmp3,sizeof(char),34U);});Cyc_fprintf(_tmp3C9,_tmp3C8,_tag_fat(_tmp2,sizeof(void*),0U));});});
exit(1);}{
# 105
int x=
# 109
Cyc_vfprintf((struct Cyc___cycFILE*)_check_null(Cyc_log_file),fmt,ap);
Cyc_fflush((struct Cyc___cycFILE*)_check_null(Cyc_log_file));
return x;}}
# 102
static struct _fat_ptr*Cyc_current_source=0;
# 115
static struct Cyc_List_List*Cyc_current_args=0;
static struct Cyc_Set_Set**Cyc_current_targets=0;
static void Cyc_add_target(struct _fat_ptr*sptr){
Cyc_current_targets=({struct Cyc_Set_Set**_tmp5=_cycalloc(sizeof(*_tmp5));({struct Cyc_Set_Set*_tmp3CC=({struct Cyc_Set_Set*(*_tmp3CB)(struct Cyc_Set_Set*s,struct _fat_ptr*elt)=({struct Cyc_Set_Set*(*_tmp4)(struct Cyc_Set_Set*s,struct _fat_ptr*elt)=(struct Cyc_Set_Set*(*)(struct Cyc_Set_Set*s,struct _fat_ptr*elt))Cyc_Set_insert;_tmp4;});struct Cyc_Set_Set*_tmp3CA=*((struct Cyc_Set_Set**)_check_null(Cyc_current_targets));_tmp3CB(_tmp3CA,sptr);});*_tmp5=_tmp3CC;});_tmp5;});}struct _tuple25{struct _fat_ptr*f1;struct Cyc_Set_Set*f2;};
# 122
struct _tuple25*Cyc_line(struct Cyc_Lexing_lexbuf*);
int Cyc_macroname(struct Cyc_Lexing_lexbuf*);
int Cyc_args(struct Cyc_Lexing_lexbuf*);
int Cyc_token(struct Cyc_Lexing_lexbuf*);
int Cyc_string(struct Cyc_Lexing_lexbuf*);
# 128
struct Cyc___cycFILE*Cyc_slurp_out=0;
int Cyc_slurp(struct Cyc_Lexing_lexbuf*);
int Cyc_slurp_string(struct Cyc_Lexing_lexbuf*);
# 132
int Cyc_asm_string(struct Cyc_Lexing_lexbuf*);
int Cyc_asm_comment(struct Cyc_Lexing_lexbuf*);struct _tuple26{struct _fat_ptr f1;struct _fat_ptr*f2;};
# 136
struct _tuple26*Cyc_suck_line(struct Cyc_Lexing_lexbuf*);
int Cyc_suck_macroname(struct Cyc_Lexing_lexbuf*);
int Cyc_suck_restofline(struct Cyc_Lexing_lexbuf*);
struct _fat_ptr Cyc_current_line={(void*)0,(void*)0,(void*)(0 + 0)};struct _tuple27{struct _fat_ptr f1;struct _fat_ptr f2;};struct _tuple28{struct _fat_ptr*f1;struct _fat_ptr*f2;};struct _tuple29{struct _fat_ptr f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;struct Cyc_List_List*f4;struct Cyc_List_List*f5;struct Cyc_List_List*f6;struct Cyc_List_List*f7;struct Cyc_List_List*f8;};
# 152
struct _tuple29*Cyc_spec(struct Cyc_Lexing_lexbuf*);
int Cyc_commands(struct Cyc_Lexing_lexbuf*);
int Cyc_snarfsymbols(struct Cyc_Lexing_lexbuf*);
int Cyc_block(struct Cyc_Lexing_lexbuf*);
int Cyc_block_string(struct Cyc_Lexing_lexbuf*);
int Cyc_block_comment(struct Cyc_Lexing_lexbuf*);
struct _fat_ptr Cyc_current_headerfile={(void*)0,(void*)0,(void*)(0 + 0)};
struct Cyc_List_List*Cyc_snarfed_symbols=0;
struct Cyc_List_List*Cyc_current_symbols=0;
struct Cyc_List_List*Cyc_current_user_defs=0;
struct Cyc_List_List*Cyc_current_cstubs=0;
struct Cyc_List_List*Cyc_current_cycstubs=0;
struct Cyc_List_List*Cyc_current_hstubs=0;
struct Cyc_List_List*Cyc_current_omit_symbols=0;
struct Cyc_List_List*Cyc_current_cpp=0;
struct Cyc_Buffer_t*Cyc_specbuf=0;
struct _fat_ptr Cyc_current_symbol={(void*)0,(void*)0,(void*)(0 + 0)};
int Cyc_rename_current_symbol=0;
int Cyc_braces_to_match=0;
int Cyc_parens_to_match=0;
# 173
int Cyc_numdef=0;
# 175
static struct Cyc_List_List*Cyc_cppargs=0;static char _tmp6[14U]="BUILDLIB_sym_";
# 177
struct _fat_ptr Cyc_user_prefix={_tmp6,_tmp6,_tmp6 + 14U};
static struct _fat_ptr*Cyc_add_user_prefix(struct _fat_ptr*symbol){
struct _fat_ptr s=(struct _fat_ptr)Cyc_strconcat((struct _fat_ptr)Cyc_user_prefix,(struct _fat_ptr)*symbol);
return({struct _fat_ptr*_tmp7=_cycalloc(sizeof(*_tmp7));*_tmp7=s;_tmp7;});}
# 178
static struct _fat_ptr Cyc_remove_user_prefix(struct _fat_ptr symbol){
# 183
unsigned prefix_len=Cyc_strlen((struct _fat_ptr)Cyc_user_prefix);
if(Cyc_strncmp((struct _fat_ptr)symbol,(struct _fat_ptr)Cyc_user_prefix,prefix_len)!= 0){
({struct Cyc_String_pa_PrintArg_struct _tmpA=({struct Cyc_String_pa_PrintArg_struct _tmp368;_tmp368.tag=0U,_tmp368.f1=(struct _fat_ptr)((struct _fat_ptr)symbol);_tmp368;});void*_tmp8[1U];_tmp8[0]=& _tmpA;({struct Cyc___cycFILE*_tmp3CE=Cyc_stderr;struct _fat_ptr _tmp3CD=({const char*_tmp9="Internal error: bad user type name %s\n";_tag_fat(_tmp9,sizeof(char),39U);});Cyc_fprintf(_tmp3CE,_tmp3CD,_tag_fat(_tmp8,sizeof(void*),1U));});});
return symbol;}
# 184
return(struct _fat_ptr)({
# 188
struct _fat_ptr _tmp3D1=(struct _fat_ptr)symbol;int _tmp3D0=(int)prefix_len;Cyc_substring(_tmp3D1,_tmp3D0,({unsigned long _tmp3CF=Cyc_strlen((struct _fat_ptr)symbol);_tmp3CF - prefix_len;}));});}
# 178
static void Cyc_rename_decl(struct Cyc_Absyn_Decl*d){
# 192
void*_stmttmp0=d->r;void*_tmpB=_stmttmp0;struct Cyc_Absyn_Typedefdecl*_tmpC;struct Cyc_Absyn_Enumdecl*_tmpD;struct Cyc_Absyn_Aggrdecl*_tmpE;switch(*((int*)_tmpB)){case 5U: _LL1: _tmpE=((struct Cyc_Absyn_Aggr_d_Absyn_Raw_decl_struct*)_tmpB)->f1;_LL2: {struct Cyc_Absyn_Aggrdecl*ad=_tmpE;
# 194
({struct _fat_ptr*_tmp3D3=({struct _fat_ptr*_tmpF=_cycalloc(sizeof(*_tmpF));({struct _fat_ptr _tmp3D2=Cyc_remove_user_prefix(*(*ad->name).f2);*_tmpF=_tmp3D2;});_tmpF;});(*ad->name).f2=_tmp3D3;});
goto _LL0;}case 7U: _LL3: _tmpD=((struct Cyc_Absyn_Enum_d_Absyn_Raw_decl_struct*)_tmpB)->f1;_LL4: {struct Cyc_Absyn_Enumdecl*ed=_tmpD;
# 197
({struct _fat_ptr*_tmp3D5=({struct _fat_ptr*_tmp10=_cycalloc(sizeof(*_tmp10));({struct _fat_ptr _tmp3D4=Cyc_remove_user_prefix(*(*ed->name).f2);*_tmp10=_tmp3D4;});_tmp10;});(*ed->name).f2=_tmp3D5;});
goto _LL0;}case 8U: _LL5: _tmpC=((struct Cyc_Absyn_Typedef_d_Absyn_Raw_decl_struct*)_tmpB)->f1;_LL6: {struct Cyc_Absyn_Typedefdecl*td=_tmpC;
# 200
({struct _fat_ptr*_tmp3D7=({struct _fat_ptr*_tmp11=_cycalloc(sizeof(*_tmp11));({struct _fat_ptr _tmp3D6=Cyc_remove_user_prefix(*(*td->name).f2);*_tmp11=_tmp3D6;});_tmp11;});(*td->name).f2=_tmp3D7;});
goto _LL0;}case 1U: _LL7: _LL8:
# 203
 goto _LLA;case 0U: _LL9: _LLA:
 goto _LLC;case 4U: _LLB: _LLC:
 goto _LLE;case 6U: _LLD: _LLE:
 goto _LL10;case 2U: _LLF: _LL10:
 goto _LL12;case 3U: _LL11: _LL12:
 goto _LL14;case 9U: _LL13: _LL14:
 goto _LL16;case 10U: _LL15: _LL16:
 goto _LL18;case 11U: _LL17: _LL18:
 goto _LL1A;case 12U: _LL19: _LL1A:
 goto _LL1C;case 13U: _LL1B: _LL1C:
 goto _LL1E;case 14U: _LL1D: _LL1E:
 goto _LL20;case 15U: _LL1F: _LL20:
 goto _LL22;default: _LL21: _LL22:
# 217
({void*_tmp12=0U;({struct Cyc___cycFILE*_tmp3D9=Cyc_stderr;struct _fat_ptr _tmp3D8=({const char*_tmp13="Error in .cys file: bad user-defined type definition\n";_tag_fat(_tmp13,sizeof(char),54U);});Cyc_fprintf(_tmp3D9,_tmp3D8,_tag_fat(_tmp12,sizeof(void*),0U));});});
exit(1);}_LL0:;}
# 224
const int Cyc_lex_base[489U]={0,0,75,192,305,310,311,166,312,91,27,384,28,523,637,715,829,325,92,- 3,0,- 1,- 2,- 8,- 3,1,- 2,323,- 4,2,166,- 5,605,907,312,- 6,- 8,- 4,16,945,- 3,1021,29,11,1135,- 4,13,1250,223,- 14,317,12,- 2,216,20,27,29,34,50,49,71,55,66,101,101,100,108,95,370,386,112,104,105,123,126,375,399,112,134,190,381,1365,1480,414,205,213,233,219,224,224,245,498,1594,1709,- 9,525,- 10,233,253,507,1823,1938,- 7,654,684,- 11,432,514,516,2015,2092,2169,2250,434,465,531,2325,253,253,253,251,247,257,0,13,4,2406,5,628,2414,2479,660,- 4,- 3,49,467,6,2440,7,701,2502,2540,793,- 28,1162,1167,273,264,267,271,281,286,292,294,- 25,752,276,276,287,280,291,305,305,308,304,314,316,324,326,- 23,311,312,315,325,377,394,- 27,404,408,404,414,423,424,- 19,405,427,421,427,438,434,437,454,455,441,437,437,459,461,- 21,453,451,444,457,454,470,464,486,489,477,481,472,498,- 26,499,502,512,521,532,535,551,570,571,1141,4,592,573,584,577,577,594,599,602,584,600,617,5,20,- 16,622,617,608,609,612,630,619,615,621,637,621,636,629,624,648,627,635,641,646,648,664,665,23,51,- 18,651,652,656,666,667,681,683,1,53,823,53,765,708,708,716,743,820,799,759,760,788,785,764,766,821,824,827,774,775,1308,798,799,800,801,764,773,788,794,795,850,851,879,- 15,826,827,882,884,912,870,871,926,927,928,- 12,875,883,938,939,940,887,888,943,944,945,- 13,875,874,888,891,894,876,902,936,942,943,998,1000,- 17,1278,971,983,980,979,978,973,1002,1005,1006,1004,1018,1372,1038,1039,1038,1053,1482,1397,1056,1058,1048,1049,1047,1060,1596,1052,1053,1051,1064,1601,- 7,- 8,8,1260,2572,9,1449,2596,2634,1567,1280,- 49,1253,- 2,1103,- 4,1104,1132,1369,1105,1135,1222,1675,1107,2661,2704,1111,1166,1113,1136,2774,1118,1217,- 36,- 42,- 37,2849,1121,- 40,1134,- 45,- 39,- 48,2924,2953,1694,1192,1203,1789,2963,2993,1808,1904,3026,3057,3095,1307,1317,3165,3203,1309,1320,1409,1431,1423,1433,- 6,- 34,1183,3135,- 47,- 30,- 32,- 46,- 29,- 31,- 33,1192,3243,1229,1233,1923,1235,1236,1241,1247,1256,1257,1258,1269,1274,3316,3400,- 22,2469,1303,- 20,- 24,- 41,- 38,- 35,1391,3482,2,1297,1298,1305,3565,1342,15,1284,1285,1286,1285,1285,1297,4};
const int Cyc_lex_backtrk[489U]={- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,0,- 1,- 1,- 1,- 1,- 1,- 1,- 1,2,- 1,- 1,- 1,- 1,2,- 1,8,- 1,3,5,- 1,- 1,6,5,- 1,- 1,- 1,7,6,- 1,6,5,2,0,- 1,- 1,0,2,- 1,12,13,- 1,13,13,13,13,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,11,12,2,4,4,- 1,0,0,0,2,0,- 1,- 1,- 1,- 1,- 1,- 1,- 1,2,2,8,3,5,- 1,6,5,- 1,- 1,6,5,2,8,3,5,- 1,6,5,- 1,27,27,27,27,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,23,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,19,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,21,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,6,1,9,2,4,- 1,5,4,- 1,- 1,2,- 1,48,- 1,48,48,48,48,48,48,48,48,5,7,48,48,48,48,0,48,48,- 1,- 1,- 1,0,43,- 1,42,- 1,- 1,- 1,9,7,- 1,7,7,- 1,8,9,- 1,- 1,9,5,6,5,5,- 1,4,4,4,6,6,5,5,- 1,- 1,- 1,9,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,2,- 1,1,- 1,- 1,- 1,2,1,1,- 1,- 1,- 1,- 1,- 1,- 1,- 1};
const int Cyc_lex_default[489U]={- 1,- 1,- 1,383,372,143,23,102,23,19,- 1,- 1,12,31,49,35,36,23,19,0,- 1,0,0,0,0,- 1,0,- 1,0,- 1,- 1,0,- 1,- 1,- 1,0,0,0,- 1,- 1,0,- 1,42,- 1,- 1,0,- 1,- 1,- 1,0,- 1,- 1,0,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,0,- 1,0,- 1,- 1,- 1,- 1,- 1,0,- 1,- 1,0,106,- 1,- 1,- 1,- 1,- 1,113,113,113,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,0,0,- 1,135,- 1,- 1,- 1,- 1,- 1,- 1,- 1,0,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,0,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,0,- 1,- 1,- 1,- 1,- 1,- 1,0,- 1,- 1,- 1,- 1,- 1,- 1,0,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,0,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,0,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,0,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,0,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,272,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,0,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,0,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,0,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,0,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,0,0,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,0,- 1,0,- 1,0,- 1,- 1,449,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,0,0,0,- 1,- 1,0,- 1,0,0,0,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,0,0,- 1,- 1,0,0,0,0,0,0,0,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,0,- 1,- 1,0,0,0,0,0,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1};
const int Cyc_lex_trans[3822U]={0,0,0,0,0,0,0,0,0,0,22,19,28,481,19,28,19,28,36,19,48,48,46,46,48,22,46,0,0,0,0,0,21,271,21,482,21,22,- 1,- 1,22,- 1,- 1,48,224,46,236,22,479,479,479,479,479,479,479,479,479,479,31,106,22,237,117,42,261,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,31,262,272,274,479,135,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,474,474,474,474,474,474,474,474,474,474,124,20,77,22,70,57,58,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,59,60,61,62,474,63,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,64,65,132,384,385,384,384,385,133,22,66,67,68,71,72,134,34,34,34,34,34,34,34,34,73,74,384,386,387,75,78,388,389,390,48,48,391,392,48,393,394,395,396,397,397,397,397,397,397,397,397,397,398,79,399,400,401,48,19,402,402,402,402,402,402,402,402,402,402,402,402,402,402,402,402,402,402,402,402,402,402,402,402,402,402,19,- 1,- 1,403,402,80,402,402,402,402,402,402,402,402,402,402,402,402,402,402,402,402,402,402,402,402,402,402,402,402,402,402,373,404,104,374,144,144,24,24,144,136,125,107,107,84,97,107,85,87,28,88,24,29,86,25,375,89,90,144,91,22,26,26,21,21,107,98,99,145,118,119,120,121,122,123,26,35,35,35,35,35,35,35,35,155,148,149,30,30,30,30,30,30,30,30,69,69,150,151,69,76,76,152,153,76,154,81,81,213,199,81,69,69,376,193,69,184,177,69,137,126,146,170,76,76,76,163,164,76,81,165,166,167,27,69,168,31,169,21,83,83,147,171,83,172,173,174,76,116,116,116,116,116,116,116,116,116,116,- 1,32,- 1,- 1,83,- 1,22,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,- 1,175,- 1,- 1,116,- 1,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,92,92,28,176,92,178,179,180,181,100,100,182,183,100,185,19,107,107,115,115,107,186,115,92,187,108,108,95,95,108,35,95,100,115,115,188,189,115,190,107,191,115,192,36,194,195,196,197,108,198,95,200,201,202,94,203,115,204,205,21,21,21,109,110,109,109,109,109,109,109,109,109,109,109,21,206,207,208,209,210,211,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,212,214,215,216,109,217,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,50,50,96,218,50,219,220,33,33,33,33,33,33,33,33,33,33,103,103,221,222,103,238,50,33,33,33,33,33,33,131,131,131,131,131,131,131,131,51,227,103,225,- 1,228,- 1,229,226,104,104,230,52,104,231,232,233,234,33,33,33,33,33,33,35,35,35,35,35,35,35,35,104,235,328,275,263,- 1,239,- 1,43,43,243,244,43,245,246,240,241,247,248,249,53,250,242,251,252,54,55,253,254,255,256,43,56,142,142,142,142,142,142,142,142,257,258,259,260,264,265,44,44,44,44,44,44,44,44,44,44,266,267,268,269,36,270,28,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,94,105,276,277,44,278,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,279,290,45,35,35,35,35,35,35,35,35,156,157,158,280,159,282,283,284,160,285,286,281,287,288,37,273,289,161,162,49,291,292,322,317,311,306,298,38,39,39,39,39,39,39,39,39,39,39,299,300,301,302,303,304,21,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,305,307,308,309,39,310,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,40,105,22,33,33,33,33,33,33,33,33,33,33,312,313,314,315,316,318,31,33,33,33,33,33,33,319,320,321,96,323,324,325,326,327,329,330,331,332,333,334,41,41,41,41,41,41,41,41,41,41,335,33,33,33,33,33,33,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,336,337,338,339,41,340,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,354,41,41,41,41,41,41,41,41,41,41,- 1,349,345,346,347,348,21,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,102,350,351,352,41,353,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,46,46,355,356,46,357,222,222,358,367,222,362,363,364,365,366,368,369,370,371,212,472,448,46,439,414,470,359,359,222,154,359,341,341,446,407,341,223,410,47,47,47,47,47,47,47,47,47,47,471,359,413,447,143,409,341,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,411,176,412,442,47,469,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,22,46,46,360,473,46,473,473,342,443,468,382,382,361,183,382,198,262,343,36,405,36,105,344,46,444,445,473,316,341,341,382,382,341,382,382,21,327,49,305,47,47,47,47,47,47,47,47,47,47,237,36,341,36,382,340,21,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,406,465,477,478,47,28,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,22,81,81,293,342,81,- 1,294,353,353,21,483,353,484,343,485,295,486,296,35,35,31,487,31,81,488,0,473,0,473,473,353,0,359,359,- 1,0,359,0,19,82,82,82,82,82,82,82,82,82,82,473,35,35,31,297,31,359,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,0,0,0,0,82,450,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,31,83,83,358,358,83,102,358,360,381,381,381,381,381,381,381,381,0,361,102,438,438,0,0,83,0,358,0,0,0,0,0,0,0,31,0,0,0,102,0,82,82,82,82,82,82,82,82,82,82,0,102,438,438,0,0,0,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,0,0,0,0,82,0,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,92,92,366,366,92,0,366,371,371,0,0,371,31,31,31,31,31,31,31,31,0,0,- 1,92,0,366,0,0,0,0,371,0,0,28,0,0,0,0,35,93,93,93,93,93,93,93,93,93,93,0,0,0,0,0,0,0,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,0,0,0,0,93,0,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,94,95,95,0,440,95,441,441,441,441,441,441,441,441,441,441,0,0,0,0,420,0,420,0,95,421,421,421,421,421,421,421,421,421,421,0,0,0,0,0,93,93,93,93,93,93,93,93,93,93,0,0,0,0,0,0,0,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,0,0,0,0,93,0,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,100,100,0,0,100,421,421,421,421,421,421,421,421,421,421,0,0,0,0,424,0,424,0,100,425,425,425,425,425,425,425,425,425,425,0,0,0,0,0,101,101,101,101,101,101,101,101,101,101,0,0,0,0,0,0,0,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,0,0,0,0,101,0,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,102,103,103,0,0,103,425,425,425,425,425,425,425,425,425,425,465,0,0,0,0,0,0,0,103,466,466,466,466,466,466,466,466,0,0,0,0,0,0,0,101,101,101,101,101,101,101,101,101,101,0,0,0,0,0,0,0,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,0,0,0,0,101,0,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,111,111,111,111,111,111,111,111,111,111,111,111,22,0,0,0,0,0,0,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,0,0,0,0,111,0,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,112,111,111,111,111,111,111,111,111,111,111,22,0,0,0,0,0,0,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,0,0,0,0,111,0,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,22,0,0,0,0,0,0,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,- 1,0,0,- 1,111,0,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,0,0,0,0,112,112,112,112,112,112,112,112,112,112,112,112,114,0,0,0,0,0,0,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,0,0,0,0,112,0,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,116,116,116,116,116,116,116,116,116,116,0,0,0,0,0,0,0,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,28,0,0,127,116,0,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,0,0,28,0,0,138,128,128,128,128,128,128,128,128,130,130,130,130,130,130,130,130,130,130,0,0,0,0,0,0,0,130,130,130,130,130,130,0,0,0,139,139,139,139,139,139,139,139,0,0,0,0,0,0,0,31,0,0,- 1,0,465,0,0,130,130,130,130,130,130,467,467,467,467,467,467,467,467,0,129,130,130,130,130,130,130,130,130,130,130,31,0,0,0,0,0,0,130,130,130,130,130,130,141,141,141,141,141,141,141,141,141,141,140,0,0,0,0,0,0,141,141,141,141,141,141,0,0,0,130,130,130,130,130,130,19,0,0,377,0,0,141,141,141,141,141,141,141,141,141,141,0,141,141,141,141,141,141,141,141,141,141,141,141,0,0,0,0,0,0,0,0,0,378,378,378,378,378,378,378,378,0,0,0,0,0,0,0,0,0,141,141,141,141,141,141,0,380,380,380,380,380,380,380,380,380,380,0,0,0,0,0,0,0,380,380,380,380,380,380,0,0,28,0,0,0,0,0,0,0,0,0,0,0,0,380,380,380,380,380,380,380,380,380,380,379,380,380,380,380,380,380,380,380,380,380,380,380,0,0,415,0,426,426,426,426,426,426,426,426,427,427,0,0,0,0,0,0,0,0,0,0,0,417,380,380,380,380,380,380,428,0,0,0,0,0,0,0,0,429,0,0,430,415,0,416,416,416,416,416,416,416,416,416,416,417,0,0,0,0,0,0,428,0,0,0,417,0,0,0,0,429,0,418,430,0,0,0,0,0,0,0,419,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,417,0,0,0,0,0,0,418,0,0,0,0,0,0,0,0,419,408,408,408,408,408,408,408,408,408,408,0,0,0,0,0,0,0,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,0,0,0,0,408,0,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,0,0,0,0,0,0,0,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,0,0,0,0,408,0,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,422,422,422,422,422,422,422,422,422,422,0,0,0,0,0,0,0,0,0,0,0,423,96,0,0,0,0,415,96,416,416,416,416,416,416,416,416,416,416,421,421,421,421,421,421,421,421,421,421,0,417,0,0,423,96,0,0,418,0,0,96,94,0,0,0,0,419,94,0,422,422,422,422,422,422,422,422,422,422,0,0,0,417,0,0,0,0,0,0,418,423,96,0,94,0,0,0,96,419,94,0,0,425,425,425,425,425,425,425,425,425,425,0,0,0,0,0,0,0,0,0,0,423,96,96,0,0,0,0,96,96,415,0,426,426,426,426,426,426,426,426,427,427,0,0,0,0,0,0,0,0,0,0,0,417,0,96,0,0,0,0,436,96,0,0,0,0,0,0,415,437,427,427,427,427,427,427,427,427,427,427,0,0,0,0,0,417,0,0,0,0,0,417,436,0,0,0,0,0,434,0,0,437,0,0,0,0,0,435,0,0,441,441,441,441,441,441,441,441,441,441,0,0,0,417,0,0,0,0,0,0,434,423,96,0,0,0,0,0,96,435,431,431,431,431,431,431,431,431,431,431,0,0,0,0,0,0,0,431,431,431,431,431,431,423,96,0,0,0,0,0,96,0,0,0,0,0,0,0,431,431,431,431,431,431,431,431,431,431,0,431,431,431,431,431,431,431,431,431,431,431,431,0,0,0,451,0,432,0,0,452,0,0,0,0,0,433,0,0,453,453,453,453,453,453,453,453,0,431,431,431,431,431,431,454,0,0,0,0,432,0,0,0,0,0,0,0,0,433,0,0,0,0,0,0,0,0,0,0,0,0,0,0,455,0,0,0,0,456,457,0,0,0,458,0,0,0,0,0,0,0,459,0,0,0,460,0,461,0,462,0,463,464,464,464,464,464,464,464,464,464,464,0,0,0,0,0,0,0,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,0,0,0,0,0,0,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,169,0,0,0,0,0,0,0,0,464,464,464,464,464,464,464,464,464,464,0,0,0,0,0,0,0,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,0,0,0,0,0,0,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,19,0,0,475,0,476,0,474,474,474,474,474,474,474,474,474,474,0,0,0,0,0,0,0,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,0,0,0,0,474,0,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,480,0,0,0,0,0,0,0,479,479,479,479,479,479,479,479,479,479,0,0,0,0,0,0,0,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,0,0,0,0,479,0,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
const int Cyc_lex_check[3822U]={- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,0,25,29,0,125,127,136,138,374,377,43,43,46,46,43,481,46,- 1,- 1,- 1,- 1,- 1,123,270,475,0,488,10,12,42,10,12,42,43,223,46,235,20,1,1,1,1,1,1,1,1,1,1,38,51,124,236,10,38,260,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,134,261,271,273,1,134,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,2,2,2,2,2,2,2,9,18,54,46,55,56,57,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,58,59,60,61,2,62,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,63,64,7,3,3,3,3,3,7,7,65,66,67,70,71,7,30,30,30,30,30,30,30,30,72,73,3,3,3,74,77,3,3,3,48,48,3,3,48,3,3,3,3,3,3,3,3,3,3,3,3,3,3,78,3,3,3,48,0,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,10,12,42,3,3,79,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,4,3,84,4,5,5,6,8,5,6,8,50,50,53,85,50,53,86,27,87,17,27,53,17,4,88,89,5,90,5,6,8,9,18,50,97,98,5,117,118,119,120,121,122,17,34,34,34,34,34,34,34,34,146,147,148,27,27,27,27,27,27,27,27,68,68,149,150,68,75,75,151,152,75,153,80,80,156,157,80,69,69,4,158,69,159,160,68,6,8,5,161,75,76,76,162,163,76,80,164,165,166,17,69,167,27,168,7,83,83,5,170,83,171,172,173,76,11,11,11,11,11,11,11,11,11,11,106,27,113,106,83,113,3,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,114,174,135,114,11,135,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,91,91,69,175,91,177,178,179,180,99,99,181,182,99,184,76,107,107,108,108,107,185,108,91,186,13,13,95,95,13,83,95,99,115,115,187,188,115,189,107,190,108,191,192,193,194,195,196,13,197,95,199,200,201,4,202,115,203,204,5,6,8,13,13,13,13,13,13,13,13,13,13,13,13,17,205,206,207,208,209,210,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,211,213,214,215,13,216,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,14,14,95,217,14,218,219,32,32,32,32,32,32,32,32,32,32,103,103,220,221,103,225,14,32,32,32,32,32,32,128,128,128,128,128,128,128,128,14,226,103,224,106,227,113,228,224,104,104,229,14,104,230,231,232,233,32,32,32,32,32,32,131,131,131,131,131,131,131,131,104,234,239,240,241,114,238,135,15,15,242,243,15,244,245,238,238,246,247,248,14,249,238,250,251,14,14,252,253,254,255,15,14,139,139,139,139,139,139,139,139,256,257,258,259,263,264,15,15,15,15,15,15,15,15,15,15,265,266,267,268,103,269,13,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,274,104,275,276,15,277,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,278,280,15,142,142,142,142,142,142,142,142,155,155,155,279,155,281,282,283,155,284,285,279,286,287,16,272,288,155,155,289,290,291,293,294,295,296,297,16,16,16,16,16,16,16,16,16,16,16,298,299,300,301,302,303,14,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,304,306,307,308,16,309,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,310,16,33,33,33,33,33,33,33,33,33,33,311,312,313,314,315,317,15,33,33,33,33,33,33,318,319,320,321,322,323,324,325,326,328,329,330,331,332,333,39,39,39,39,39,39,39,39,39,39,334,33,33,33,33,33,33,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,335,336,337,338,39,339,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,39,342,41,41,41,41,41,41,41,41,41,41,272,343,344,345,346,347,16,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,348,349,350,351,41,352,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,41,44,44,354,355,44,356,222,222,357,360,222,361,362,363,364,365,367,368,369,370,386,388,391,44,395,398,389,144,144,222,400,144,145,145,392,403,145,222,409,44,44,44,44,44,44,44,44,44,44,389,144,411,392,401,401,145,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,399,399,399,440,44,449,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,47,47,144,384,47,384,384,145,393,451,375,375,144,452,375,454,455,145,418,404,419,456,145,47,393,393,384,457,341,341,382,382,341,375,382,375,458,459,460,47,47,47,47,47,47,47,47,47,47,461,418,341,419,382,462,382,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,404,467,476,477,47,478,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,81,81,292,341,81,390,292,353,353,480,482,353,483,341,484,292,485,292,428,429,432,486,433,81,487,- 1,473,- 1,473,473,353,- 1,359,359,390,- 1,359,- 1,353,81,81,81,81,81,81,81,81,81,81,473,428,429,432,292,433,359,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,- 1,- 1,- 1,- 1,81,390,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,82,82,358,358,82,434,358,359,378,378,378,378,378,378,378,378,- 1,359,435,436,437,- 1,- 1,82,- 1,358,- 1,- 1,- 1,- 1,- 1,- 1,- 1,358,- 1,- 1,- 1,434,- 1,82,82,82,82,82,82,82,82,82,82,- 1,435,436,437,- 1,- 1,- 1,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,- 1,- 1,- 1,- 1,82,- 1,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,92,92,366,366,92,- 1,366,371,371,- 1,- 1,371,381,381,381,381,381,381,381,381,- 1,- 1,390,92,- 1,366,- 1,- 1,- 1,- 1,371,- 1,- 1,366,- 1,- 1,- 1,- 1,371,92,92,92,92,92,92,92,92,92,92,- 1,- 1,- 1,- 1,- 1,- 1,- 1,92,92,92,92,92,92,92,92,92,92,92,92,92,92,92,92,92,92,92,92,92,92,92,92,92,92,- 1,- 1,- 1,- 1,92,- 1,92,92,92,92,92,92,92,92,92,92,92,92,92,92,92,92,92,92,92,92,92,92,92,92,92,92,92,93,93,- 1,394,93,394,394,394,394,394,394,394,394,394,394,- 1,- 1,- 1,- 1,417,- 1,417,- 1,93,417,417,417,417,417,417,417,417,417,417,- 1,- 1,- 1,- 1,- 1,93,93,93,93,93,93,93,93,93,93,- 1,- 1,- 1,- 1,- 1,- 1,- 1,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,- 1,- 1,- 1,- 1,93,- 1,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,100,100,- 1,- 1,100,420,420,420,420,420,420,420,420,420,420,- 1,- 1,- 1,- 1,423,- 1,423,- 1,100,423,423,423,423,423,423,423,423,423,423,- 1,- 1,- 1,- 1,- 1,100,100,100,100,100,100,100,100,100,100,- 1,- 1,- 1,- 1,- 1,- 1,- 1,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,- 1,- 1,- 1,- 1,100,- 1,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,101,101,- 1,- 1,101,424,424,424,424,424,424,424,424,424,424,453,- 1,- 1,- 1,- 1,- 1,- 1,- 1,101,453,453,453,453,453,453,453,453,- 1,- 1,- 1,- 1,- 1,- 1,- 1,101,101,101,101,101,101,101,101,101,101,- 1,- 1,- 1,- 1,- 1,- 1,- 1,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,- 1,- 1,- 1,- 1,101,- 1,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,109,109,109,109,109,109,109,109,109,109,109,109,109,- 1,- 1,- 1,- 1,- 1,- 1,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,- 1,- 1,- 1,- 1,109,- 1,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,110,110,110,110,110,110,110,110,110,110,110,110,110,- 1,- 1,- 1,- 1,- 1,- 1,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,- 1,- 1,- 1,- 1,110,- 1,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,111,111,111,111,111,111,111,111,111,111,111,111,111,- 1,- 1,- 1,- 1,- 1,- 1,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,112,- 1,- 1,112,111,- 1,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,- 1,- 1,- 1,- 1,112,112,112,112,112,112,112,112,112,112,112,112,112,- 1,- 1,- 1,- 1,- 1,- 1,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,- 1,- 1,- 1,- 1,112,- 1,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,116,116,116,116,116,116,116,116,116,116,- 1,- 1,- 1,- 1,- 1,- 1,- 1,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,126,- 1,- 1,126,116,- 1,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,- 1,- 1,137,- 1,- 1,137,126,126,126,126,126,126,126,126,129,129,129,129,129,129,129,129,129,129,- 1,- 1,- 1,- 1,- 1,- 1,- 1,129,129,129,129,129,129,- 1,- 1,- 1,137,137,137,137,137,137,137,137,- 1,- 1,- 1,- 1,- 1,- 1,- 1,126,- 1,- 1,112,- 1,466,- 1,- 1,129,129,129,129,129,129,466,466,466,466,466,466,466,466,- 1,126,130,130,130,130,130,130,130,130,130,130,137,- 1,- 1,- 1,- 1,- 1,- 1,130,130,130,130,130,130,140,140,140,140,140,140,140,140,140,140,137,- 1,- 1,- 1,- 1,- 1,- 1,140,140,140,140,140,140,- 1,- 1,- 1,130,130,130,130,130,130,376,- 1,- 1,376,- 1,- 1,141,141,141,141,141,141,141,141,141,141,- 1,140,140,140,140,140,140,141,141,141,141,141,141,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,376,376,376,376,376,376,376,376,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,141,141,141,141,141,141,- 1,379,379,379,379,379,379,379,379,379,379,- 1,- 1,- 1,- 1,- 1,- 1,- 1,379,379,379,379,379,379,- 1,- 1,376,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,380,380,380,380,380,380,380,380,380,380,376,379,379,379,379,379,379,380,380,380,380,380,380,- 1,- 1,396,- 1,396,396,396,396,396,396,396,396,396,396,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,396,380,380,380,380,380,380,396,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,396,- 1,- 1,396,397,- 1,397,397,397,397,397,397,397,397,397,397,396,- 1,- 1,- 1,- 1,- 1,- 1,396,- 1,- 1,- 1,397,- 1,- 1,- 1,- 1,396,- 1,397,396,- 1,- 1,- 1,- 1,- 1,- 1,- 1,397,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,397,- 1,- 1,- 1,- 1,- 1,- 1,397,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,397,402,402,402,402,402,402,402,402,402,402,- 1,- 1,- 1,- 1,- 1,- 1,- 1,402,402,402,402,402,402,402,402,402,402,402,402,402,402,402,402,402,402,402,402,402,402,402,402,402,402,- 1,- 1,- 1,- 1,402,- 1,402,402,402,402,402,402,402,402,402,402,402,402,402,402,402,402,402,402,402,402,402,402,402,402,402,402,408,408,408,408,408,408,408,408,408,408,- 1,- 1,- 1,- 1,- 1,- 1,- 1,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,- 1,- 1,- 1,- 1,408,- 1,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,415,415,415,415,415,415,415,415,415,415,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,415,415,- 1,- 1,- 1,- 1,416,415,416,416,416,416,416,416,416,416,416,416,421,421,421,421,421,421,421,421,421,421,- 1,416,- 1,- 1,415,415,- 1,- 1,416,- 1,- 1,415,421,- 1,- 1,- 1,- 1,416,421,- 1,422,422,422,422,422,422,422,422,422,422,- 1,- 1,- 1,416,- 1,- 1,- 1,- 1,- 1,- 1,416,422,422,- 1,421,- 1,- 1,- 1,422,416,421,- 1,- 1,425,425,425,425,425,425,425,425,425,425,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,422,422,425,- 1,- 1,- 1,- 1,422,425,426,- 1,426,426,426,426,426,426,426,426,426,426,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,426,- 1,425,- 1,- 1,- 1,- 1,426,425,- 1,- 1,- 1,- 1,- 1,- 1,427,426,427,427,427,427,427,427,427,427,427,427,- 1,- 1,- 1,- 1,- 1,426,- 1,- 1,- 1,- 1,- 1,427,426,- 1,- 1,- 1,- 1,- 1,427,- 1,- 1,426,- 1,- 1,- 1,- 1,- 1,427,- 1,- 1,441,441,441,441,441,441,441,441,441,441,- 1,- 1,- 1,427,- 1,- 1,- 1,- 1,- 1,- 1,427,441,441,- 1,- 1,- 1,- 1,- 1,441,427,430,430,430,430,430,430,430,430,430,430,- 1,- 1,- 1,- 1,- 1,- 1,- 1,430,430,430,430,430,430,441,441,- 1,- 1,- 1,- 1,- 1,441,- 1,- 1,- 1,- 1,- 1,- 1,- 1,431,431,431,431,431,431,431,431,431,431,- 1,430,430,430,430,430,430,431,431,431,431,431,431,- 1,- 1,- 1,450,- 1,431,- 1,- 1,450,- 1,- 1,- 1,- 1,- 1,431,- 1,- 1,450,450,450,450,450,450,450,450,- 1,431,431,431,431,431,431,450,- 1,- 1,- 1,- 1,431,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,431,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,450,- 1,- 1,- 1,- 1,450,450,- 1,- 1,- 1,450,- 1,- 1,- 1,- 1,- 1,- 1,- 1,450,- 1,- 1,- 1,450,- 1,450,- 1,450,- 1,450,463,463,463,463,463,463,463,463,463,463,- 1,- 1,- 1,- 1,- 1,- 1,- 1,463,463,463,463,463,463,463,463,463,463,463,463,463,463,463,463,463,463,463,463,463,463,463,463,463,463,- 1,- 1,- 1,- 1,- 1,- 1,463,463,463,463,463,463,463,463,463,463,463,463,463,463,463,463,463,463,463,463,463,463,463,463,463,463,464,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,464,464,464,464,464,464,464,464,464,464,- 1,- 1,- 1,- 1,- 1,- 1,- 1,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,- 1,- 1,- 1,- 1,- 1,- 1,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,464,474,- 1,- 1,474,- 1,474,- 1,474,474,474,474,474,474,474,474,474,474,- 1,- 1,- 1,- 1,- 1,- 1,- 1,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,- 1,- 1,- 1,- 1,474,- 1,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,474,479,- 1,- 1,- 1,- 1,- 1,- 1,- 1,479,479,479,479,479,479,479,479,479,479,- 1,- 1,- 1,- 1,- 1,- 1,- 1,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,- 1,- 1,- 1,- 1,479,- 1,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,479,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1};
int Cyc_lex_engine(int start_state,struct Cyc_Lexing_lexbuf*lbuf){
# 231
int state;int base;int backtrk;
int c;
state=start_state;
# 235
if(state >= 0){
({int _tmp3DA=lbuf->lex_start_pos=lbuf->lex_curr_pos;lbuf->lex_last_pos=_tmp3DA;});
lbuf->lex_last_action=- 1;}else{
# 239
state=(- state)- 1;}
# 241
while(1){
base=*((const int*)_check_known_subscript_notnull(Cyc_lex_base,489U,sizeof(int),state));
if(base < 0)return(- base)- 1;backtrk=Cyc_lex_backtrk[state];
# 245
if(backtrk >= 0){
lbuf->lex_last_pos=lbuf->lex_curr_pos;
lbuf->lex_last_action=backtrk;}
# 245
if(lbuf->lex_curr_pos >= lbuf->lex_buffer_len){
# 250
if(!lbuf->lex_eof_reached)
return(- state)- 1;else{
# 253
c=256;}}else{
# 255
c=(int)*((char*)_check_fat_subscript(lbuf->lex_buffer,sizeof(char),lbuf->lex_curr_pos ++));
if(c == -1)c=256;}
# 258
if(*((const int*)_check_known_subscript_notnull(Cyc_lex_check,3822U,sizeof(int),base + c))== state)
state=*((const int*)_check_known_subscript_notnull(Cyc_lex_trans,3822U,sizeof(int),base + c));else{
# 261
state=Cyc_lex_default[state];}
if(state < 0){
lbuf->lex_curr_pos=lbuf->lex_last_pos;
if(lbuf->lex_last_action == - 1)
(int)_throw((void*)({struct Cyc_Lexing_Error_exn_struct*_tmp15=_cycalloc(sizeof(*_tmp15));_tmp15->tag=Cyc_Lexing_Error,({struct _fat_ptr _tmp3DB=({const char*_tmp14="empty token";_tag_fat(_tmp14,sizeof(char),12U);});_tmp15->f1=_tmp3DB;});_tmp15;}));else{
# 267
return lbuf->lex_last_action;}}else{
# 270
if(c == 256)lbuf->lex_eof_reached=0;}}}
# 229
struct _tuple25*Cyc_line_rec(struct Cyc_Lexing_lexbuf*lexbuf,int lexstate){
# 275
lexstate=Cyc_lex_engine(lexstate,lexbuf);
{int _tmp16=lexstate;switch(_tmp16){case 0U: _LL1: _LL2:
# 233 "buildlib.cyl"
({struct Cyc_Int_pa_PrintArg_struct _tmp19=({struct Cyc_Int_pa_PrintArg_struct _tmp369;_tmp369.tag=1U,_tmp369.f1=(unsigned long)Cyc_count ++;_tmp369;});void*_tmp17[1U];_tmp17[0]=& _tmp19;({struct _fat_ptr _tmp3DC=({const char*_tmp18="Count = %d\n";_tag_fat(_tmp18,sizeof(char),12U);});Cyc_log(_tmp3DC,_tag_fat(_tmp17,sizeof(void*),1U));});});
Cyc_macroname(lexbuf);
for(0;Cyc_current_args != 0;Cyc_current_args=((struct Cyc_List_List*)_check_null(Cyc_current_args))->tl){
Cyc_current_targets=({struct Cyc_Set_Set**_tmp1B=_cycalloc(sizeof(*_tmp1B));({struct Cyc_Set_Set*_tmp3DF=({struct Cyc_Set_Set*(*_tmp3DE)(struct Cyc_Set_Set*s,struct _fat_ptr*elt)=({struct Cyc_Set_Set*(*_tmp1A)(struct Cyc_Set_Set*s,struct _fat_ptr*elt)=(struct Cyc_Set_Set*(*)(struct Cyc_Set_Set*s,struct _fat_ptr*elt))Cyc_Set_delete;_tmp1A;});struct Cyc_Set_Set*_tmp3DD=*((struct Cyc_Set_Set**)_check_null(Cyc_current_targets));_tmp3DE(_tmp3DD,(struct _fat_ptr*)((struct Cyc_List_List*)_check_null(Cyc_current_args))->hd);});*_tmp1B=_tmp3DF;});_tmp1B;});}
# 239
return({struct _tuple25*_tmp1C=_cycalloc(sizeof(*_tmp1C));_tmp1C->f1=(struct _fat_ptr*)_check_null(Cyc_current_source),_tmp1C->f2=*((struct Cyc_Set_Set**)_check_null(Cyc_current_targets));_tmp1C;});case 1U: _LL3: _LL4:
# 242 "buildlib.cyl"
 return Cyc_line(lexbuf);case 2U: _LL5: _LL6:
# 244
 return 0;default: _LL7: _LL8:
(lexbuf->refill_buff)(lexbuf);
return Cyc_line_rec(lexbuf,lexstate);}_LL0:;}
# 248
(int)_throw((void*)({struct Cyc_Lexing_Error_exn_struct*_tmp1E=_cycalloc(sizeof(*_tmp1E));_tmp1E->tag=Cyc_Lexing_Error,({struct _fat_ptr _tmp3E0=({const char*_tmp1D="some action didn't return!";_tag_fat(_tmp1D,sizeof(char),27U);});_tmp1E->f1=_tmp3E0;});_tmp1E;}));}
# 229 "buildlib.cyl"
struct _tuple25*Cyc_line(struct Cyc_Lexing_lexbuf*lexbuf){
# 250 "buildlib.cyl"
return Cyc_line_rec(lexbuf,0);}
# 229 "buildlib.cyl"
int Cyc_macroname_rec(struct Cyc_Lexing_lexbuf*lexbuf,int lexstate){
# 252 "buildlib.cyl"
lexstate=Cyc_lex_engine(lexstate,lexbuf);
{int _tmp1F=lexstate;switch(_tmp1F){case 0U: _LL1: _LL2:
# 248 "buildlib.cyl"
 Cyc_current_source=({struct _fat_ptr*_tmp20=_cycalloc(sizeof(*_tmp20));({struct _fat_ptr _tmp3E3=(struct _fat_ptr)({struct _fat_ptr _tmp3E2=(struct _fat_ptr)Cyc_Lexing_lexeme(lexbuf);Cyc_substring(_tmp3E2,0,(unsigned long)(({
int _tmp3E1=Cyc_Lexing_lexeme_end(lexbuf);_tmp3E1 - Cyc_Lexing_lexeme_start(lexbuf);})- 2));});
# 248
*_tmp20=_tmp3E3;});_tmp20;});
# 250
Cyc_current_args=0;
Cyc_current_targets=({struct Cyc_Set_Set**_tmp22=_cycalloc(sizeof(*_tmp22));({struct Cyc_Set_Set*_tmp3E4=({struct Cyc_Set_Set*(*_tmp21)(int(*cmp)(struct _fat_ptr*,struct _fat_ptr*))=(struct Cyc_Set_Set*(*)(int(*cmp)(struct _fat_ptr*,struct _fat_ptr*)))Cyc_Set_empty;_tmp21;})(Cyc_strptrcmp);*_tmp22=_tmp3E4;});_tmp22;});
Cyc_token(lexbuf);
return 0;case 1U: _LL3: _LL4:
# 256
 Cyc_current_source=({struct _fat_ptr*_tmp23=_cycalloc(sizeof(*_tmp23));({struct _fat_ptr _tmp3E7=(struct _fat_ptr)({struct _fat_ptr _tmp3E6=(struct _fat_ptr)Cyc_Lexing_lexeme(lexbuf);Cyc_substring(_tmp3E6,0,(unsigned long)(({
int _tmp3E5=Cyc_Lexing_lexeme_end(lexbuf);_tmp3E5 - Cyc_Lexing_lexeme_start(lexbuf);})- 1));});
# 256
*_tmp23=_tmp3E7;});_tmp23;});
# 258
Cyc_current_args=0;
Cyc_current_targets=({struct Cyc_Set_Set**_tmp25=_cycalloc(sizeof(*_tmp25));({struct Cyc_Set_Set*_tmp3E8=({struct Cyc_Set_Set*(*_tmp24)(int(*cmp)(struct _fat_ptr*,struct _fat_ptr*))=(struct Cyc_Set_Set*(*)(int(*cmp)(struct _fat_ptr*,struct _fat_ptr*)))Cyc_Set_empty;_tmp24;})(Cyc_strptrcmp);*_tmp25=_tmp3E8;});_tmp25;});
Cyc_args(lexbuf);
return 0;case 2U: _LL5: _LL6:
# 264
 Cyc_current_source=({struct _fat_ptr*_tmp26=_cycalloc(sizeof(*_tmp26));({struct _fat_ptr _tmp3E9=(struct _fat_ptr)Cyc_Lexing_lexeme(lexbuf);*_tmp26=_tmp3E9;});_tmp26;});
Cyc_current_args=0;
Cyc_current_targets=({struct Cyc_Set_Set**_tmp28=_cycalloc(sizeof(*_tmp28));({struct Cyc_Set_Set*_tmp3EA=({struct Cyc_Set_Set*(*_tmp27)(int(*cmp)(struct _fat_ptr*,struct _fat_ptr*))=(struct Cyc_Set_Set*(*)(int(*cmp)(struct _fat_ptr*,struct _fat_ptr*)))Cyc_Set_empty;_tmp27;})(Cyc_strptrcmp);*_tmp28=_tmp3EA;});_tmp28;});
Cyc_token(lexbuf);
return 0;default: _LL7: _LL8:
# 270
(lexbuf->refill_buff)(lexbuf);
return Cyc_macroname_rec(lexbuf,lexstate);}_LL0:;}
# 273
(int)_throw((void*)({struct Cyc_Lexing_Error_exn_struct*_tmp2A=_cycalloc(sizeof(*_tmp2A));_tmp2A->tag=Cyc_Lexing_Error,({struct _fat_ptr _tmp3EB=({const char*_tmp29="some action didn't return!";_tag_fat(_tmp29,sizeof(char),27U);});_tmp2A->f1=_tmp3EB;});_tmp2A;}));}
# 229 "buildlib.cyl"
int Cyc_macroname(struct Cyc_Lexing_lexbuf*lexbuf){
# 275 "buildlib.cyl"
return Cyc_macroname_rec(lexbuf,1);}
# 229 "buildlib.cyl"
int Cyc_args_rec(struct Cyc_Lexing_lexbuf*lexbuf,int lexstate){
# 277 "buildlib.cyl"
lexstate=Cyc_lex_engine(lexstate,lexbuf);
{int _tmp2B=lexstate;switch(_tmp2B){case 0U: _LL1: _LL2: {
# 273 "buildlib.cyl"
struct _fat_ptr*a=({struct _fat_ptr*_tmp2D=_cycalloc(sizeof(*_tmp2D));({struct _fat_ptr _tmp3EE=(struct _fat_ptr)({struct _fat_ptr _tmp3ED=(struct _fat_ptr)Cyc_Lexing_lexeme(lexbuf);Cyc_substring(_tmp3ED,0,(unsigned long)(({
int _tmp3EC=Cyc_Lexing_lexeme_end(lexbuf);_tmp3EC - Cyc_Lexing_lexeme_start(lexbuf);})- 2));});
# 273
*_tmp2D=_tmp3EE;});_tmp2D;});
# 275
Cyc_current_args=({struct Cyc_List_List*_tmp2C=_cycalloc(sizeof(*_tmp2C));_tmp2C->hd=a,_tmp2C->tl=Cyc_current_args;_tmp2C;});
return Cyc_args(lexbuf);}case 1U: _LL3: _LL4: {
# 279
struct _fat_ptr*a=({struct _fat_ptr*_tmp2F=_cycalloc(sizeof(*_tmp2F));({struct _fat_ptr _tmp3F1=(struct _fat_ptr)({struct _fat_ptr _tmp3F0=(struct _fat_ptr)Cyc_Lexing_lexeme(lexbuf);Cyc_substring(_tmp3F0,0,(unsigned long)(({
int _tmp3EF=Cyc_Lexing_lexeme_end(lexbuf);_tmp3EF - Cyc_Lexing_lexeme_start(lexbuf);})- 1));});
# 279
*_tmp2F=_tmp3F1;});_tmp2F;});
# 281
Cyc_current_args=({struct Cyc_List_List*_tmp2E=_cycalloc(sizeof(*_tmp2E));_tmp2E->hd=a,_tmp2E->tl=Cyc_current_args;_tmp2E;});
return Cyc_args(lexbuf);}case 2U: _LL5: _LL6: {
# 285
struct _fat_ptr*a=({struct _fat_ptr*_tmp31=_cycalloc(sizeof(*_tmp31));({struct _fat_ptr _tmp3F4=(struct _fat_ptr)({struct _fat_ptr _tmp3F3=(struct _fat_ptr)Cyc_Lexing_lexeme(lexbuf);Cyc_substring(_tmp3F3,0,(unsigned long)(({
int _tmp3F2=Cyc_Lexing_lexeme_end(lexbuf);_tmp3F2 - Cyc_Lexing_lexeme_start(lexbuf);})- 1));});
# 285
*_tmp31=_tmp3F4;});_tmp31;});
# 287
Cyc_current_args=({struct Cyc_List_List*_tmp30=_cycalloc(sizeof(*_tmp30));_tmp30->hd=a,_tmp30->tl=Cyc_current_args;_tmp30;});
return Cyc_token(lexbuf);}case 3U: _LL7: _LL8: {
# 291
struct _fat_ptr*a=({struct _fat_ptr*_tmp33=_cycalloc(sizeof(*_tmp33));({struct _fat_ptr _tmp3F7=(struct _fat_ptr)({struct _fat_ptr _tmp3F6=(struct _fat_ptr)Cyc_Lexing_lexeme(lexbuf);Cyc_substring(_tmp3F6,0,(unsigned long)(({
int _tmp3F5=Cyc_Lexing_lexeme_end(lexbuf);_tmp3F5 - Cyc_Lexing_lexeme_start(lexbuf);})- 1));});
# 291
*_tmp33=_tmp3F7;});_tmp33;});
# 293
Cyc_current_args=({struct Cyc_List_List*_tmp32=_cycalloc(sizeof(*_tmp32));_tmp32->hd=a,_tmp32->tl=Cyc_current_args;_tmp32;});
return Cyc_token(lexbuf);}default: _LL9: _LLA:
# 296
(lexbuf->refill_buff)(lexbuf);
return Cyc_args_rec(lexbuf,lexstate);}_LL0:;}
# 299
(int)_throw((void*)({struct Cyc_Lexing_Error_exn_struct*_tmp35=_cycalloc(sizeof(*_tmp35));_tmp35->tag=Cyc_Lexing_Error,({struct _fat_ptr _tmp3F8=({const char*_tmp34="some action didn't return!";_tag_fat(_tmp34,sizeof(char),27U);});_tmp35->f1=_tmp3F8;});_tmp35;}));}
# 229 "buildlib.cyl"
int Cyc_args(struct Cyc_Lexing_lexbuf*lexbuf){
# 301 "buildlib.cyl"
return Cyc_args_rec(lexbuf,2);}
# 229 "buildlib.cyl"
int Cyc_token_rec(struct Cyc_Lexing_lexbuf*lexbuf,int lexstate){
# 303 "buildlib.cyl"
lexstate=Cyc_lex_engine(lexstate,lexbuf);
{int _tmp36=lexstate;switch(_tmp36){case 0U: _LL1: _LL2:
# 300 "buildlib.cyl"
 Cyc_add_target(({struct _fat_ptr*_tmp37=_cycalloc(sizeof(*_tmp37));({struct _fat_ptr _tmp3F9=(struct _fat_ptr)Cyc_Lexing_lexeme(lexbuf);*_tmp37=_tmp3F9;});_tmp37;}));return Cyc_token(lexbuf);case 1U: _LL3: _LL4:
# 303 "buildlib.cyl"
 return 0;case 2U: _LL5: _LL6:
# 306 "buildlib.cyl"
 return Cyc_token(lexbuf);case 3U: _LL7: _LL8:
# 309 "buildlib.cyl"
 Cyc_string(lexbuf);return Cyc_token(lexbuf);case 4U: _LL9: _LLA:
# 312 "buildlib.cyl"
 return Cyc_token(lexbuf);case 5U: _LLB: _LLC:
# 314
 return Cyc_token(lexbuf);case 6U: _LLD: _LLE:
# 316
 return Cyc_token(lexbuf);case 7U: _LLF: _LL10:
# 318
 return Cyc_token(lexbuf);case 8U: _LL11: _LL12:
# 321 "buildlib.cyl"
 return Cyc_token(lexbuf);case 9U: _LL13: _LL14:
# 324 "buildlib.cyl"
 return Cyc_token(lexbuf);case 10U: _LL15: _LL16:
# 327 "buildlib.cyl"
 return Cyc_token(lexbuf);case 11U: _LL17: _LL18:
# 329
 return Cyc_token(lexbuf);case 12U: _LL19: _LL1A:
# 331
 return Cyc_token(lexbuf);case 13U: _LL1B: _LL1C:
# 333
 return Cyc_token(lexbuf);case 14U: _LL1D: _LL1E:
# 335
 return Cyc_token(lexbuf);case 15U: _LL1F: _LL20:
# 337
 return Cyc_token(lexbuf);case 16U: _LL21: _LL22:
# 339
 return Cyc_token(lexbuf);case 17U: _LL23: _LL24:
# 341
 return Cyc_token(lexbuf);case 18U: _LL25: _LL26:
# 343
 return Cyc_token(lexbuf);case 19U: _LL27: _LL28:
# 345
 return Cyc_token(lexbuf);case 20U: _LL29: _LL2A:
# 347
 return Cyc_token(lexbuf);case 21U: _LL2B: _LL2C:
# 349
 return Cyc_token(lexbuf);case 22U: _LL2D: _LL2E:
# 351
 return Cyc_token(lexbuf);case 23U: _LL2F: _LL30:
# 353
 return Cyc_token(lexbuf);case 24U: _LL31: _LL32:
# 356 "buildlib.cyl"
 return Cyc_token(lexbuf);case 25U: _LL33: _LL34:
# 358
 return Cyc_token(lexbuf);case 26U: _LL35: _LL36:
# 360
 return Cyc_token(lexbuf);case 27U: _LL37: _LL38:
# 362
 return Cyc_token(lexbuf);case 28U: _LL39: _LL3A:
# 364
 return Cyc_token(lexbuf);case 29U: _LL3B: _LL3C:
# 366
 return Cyc_token(lexbuf);case 30U: _LL3D: _LL3E:
# 368
 return Cyc_token(lexbuf);case 31U: _LL3F: _LL40:
# 370
 return Cyc_token(lexbuf);case 32U: _LL41: _LL42:
# 372
 return Cyc_token(lexbuf);case 33U: _LL43: _LL44:
# 374
 return Cyc_token(lexbuf);case 34U: _LL45: _LL46:
# 376
 return Cyc_token(lexbuf);case 35U: _LL47: _LL48:
# 378
 return Cyc_token(lexbuf);case 36U: _LL49: _LL4A:
# 380
 return Cyc_token(lexbuf);case 37U: _LL4B: _LL4C:
# 382
 return Cyc_token(lexbuf);case 38U: _LL4D: _LL4E:
# 384
 return Cyc_token(lexbuf);case 39U: _LL4F: _LL50:
# 386
 return Cyc_token(lexbuf);case 40U: _LL51: _LL52:
# 388
 return Cyc_token(lexbuf);case 41U: _LL53: _LL54:
# 390
 return Cyc_token(lexbuf);case 42U: _LL55: _LL56:
# 392
 return Cyc_token(lexbuf);case 43U: _LL57: _LL58:
# 394
 return Cyc_token(lexbuf);case 44U: _LL59: _LL5A:
# 396
 return Cyc_token(lexbuf);case 45U: _LL5B: _LL5C:
# 398
 return Cyc_token(lexbuf);case 46U: _LL5D: _LL5E:
# 400
 return Cyc_token(lexbuf);case 47U: _LL5F: _LL60:
# 402
 return Cyc_token(lexbuf);case 48U: _LL61: _LL62:
# 405 "buildlib.cyl"
 return Cyc_token(lexbuf);default: _LL63: _LL64:
(lexbuf->refill_buff)(lexbuf);
return Cyc_token_rec(lexbuf,lexstate);}_LL0:;}
# 409
(int)_throw((void*)({struct Cyc_Lexing_Error_exn_struct*_tmp39=_cycalloc(sizeof(*_tmp39));_tmp39->tag=Cyc_Lexing_Error,({struct _fat_ptr _tmp3FA=({const char*_tmp38="some action didn't return!";_tag_fat(_tmp38,sizeof(char),27U);});_tmp39->f1=_tmp3FA;});_tmp39;}));}
# 229 "buildlib.cyl"
int Cyc_token(struct Cyc_Lexing_lexbuf*lexbuf){
# 411 "buildlib.cyl"
return Cyc_token_rec(lexbuf,3);}
# 229 "buildlib.cyl"
int Cyc_string_rec(struct Cyc_Lexing_lexbuf*lexbuf,int lexstate){
# 413 "buildlib.cyl"
lexstate=Cyc_lex_engine(lexstate,lexbuf);
{int _tmp3A=lexstate;switch(_tmp3A){case 0U: _LL1: _LL2:
# 410 "buildlib.cyl"
 return Cyc_string(lexbuf);case 1U: _LL3: _LL4:
# 411 "buildlib.cyl"
 return 0;case 2U: _LL5: _LL6:
# 412 "buildlib.cyl"
 return Cyc_string(lexbuf);case 3U: _LL7: _LL8:
# 413 "buildlib.cyl"
 return Cyc_string(lexbuf);case 4U: _LL9: _LLA:
# 416 "buildlib.cyl"
 return Cyc_string(lexbuf);case 5U: _LLB: _LLC:
# 419 "buildlib.cyl"
 return Cyc_string(lexbuf);case 6U: _LLD: _LLE:
# 421
 return Cyc_string(lexbuf);case 7U: _LLF: _LL10:
# 422 "buildlib.cyl"
 return 0;case 8U: _LL11: _LL12:
# 423 "buildlib.cyl"
 return 0;case 9U: _LL13: _LL14:
# 424 "buildlib.cyl"
 return Cyc_string(lexbuf);default: _LL15: _LL16:
(lexbuf->refill_buff)(lexbuf);
return Cyc_string_rec(lexbuf,lexstate);}_LL0:;}
# 428
(int)_throw((void*)({struct Cyc_Lexing_Error_exn_struct*_tmp3C=_cycalloc(sizeof(*_tmp3C));_tmp3C->tag=Cyc_Lexing_Error,({struct _fat_ptr _tmp3FB=({const char*_tmp3B="some action didn't return!";_tag_fat(_tmp3B,sizeof(char),27U);});_tmp3C->f1=_tmp3FB;});_tmp3C;}));}
# 229 "buildlib.cyl"
int Cyc_string(struct Cyc_Lexing_lexbuf*lexbuf){
# 430 "buildlib.cyl"
return Cyc_string_rec(lexbuf,4);}
# 229 "buildlib.cyl"
int Cyc_slurp_rec(struct Cyc_Lexing_lexbuf*lexbuf,int lexstate){
# 432 "buildlib.cyl"
lexstate=Cyc_lex_engine(lexstate,lexbuf);
{int _tmp3D=lexstate;switch(_tmp3D){case 0U: _LL1: _LL2:
# 433 "buildlib.cyl"
 return 0;case 1U: _LL3: _LL4:
# 435
 Cyc_fputc((int)'"',(struct Cyc___cycFILE*)_check_null(Cyc_slurp_out));
while(Cyc_slurp_string(lexbuf)){;}
return 1;case 2U: _LL5: _LL6:
# 442 "buildlib.cyl"
 Cyc_fputs("*__IGNORE_FOR_CYCLONE_MALLOC(",(struct Cyc___cycFILE*)_check_null(Cyc_slurp_out));
({void*_tmp3E=0U;({struct _fat_ptr _tmp3FC=({const char*_tmp3F="Warning: declaration of malloc sidestepped\n";_tag_fat(_tmp3F,sizeof(char),44U);});Cyc_log(_tmp3FC,_tag_fat(_tmp3E,sizeof(void*),0U));});});
return 1;case 3U: _LL7: _LL8:
# 448 "buildlib.cyl"
 Cyc_fputs(" __IGNORE_FOR_CYCLONE_MALLOC(",(struct Cyc___cycFILE*)_check_null(Cyc_slurp_out));
({void*_tmp40=0U;({struct _fat_ptr _tmp3FD=({const char*_tmp41="Warning: declaration of malloc sidestepped\n";_tag_fat(_tmp41,sizeof(char),44U);});Cyc_log(_tmp3FD,_tag_fat(_tmp40,sizeof(void*),0U));});});
return 1;case 4U: _LL9: _LLA:
# 454 "buildlib.cyl"
 Cyc_fputs("*__IGNORE_FOR_CYCLONE_CALLOC(",(struct Cyc___cycFILE*)_check_null(Cyc_slurp_out));
({void*_tmp42=0U;({struct _fat_ptr _tmp3FE=({const char*_tmp43="Warning: declaration of calloc sidestepped\n";_tag_fat(_tmp43,sizeof(char),44U);});Cyc_log(_tmp3FE,_tag_fat(_tmp42,sizeof(void*),0U));});});
return 1;case 5U: _LLB: _LLC:
# 460 "buildlib.cyl"
 Cyc_fputs(" __IGNORE_FOR_CYCLONE_CALLOC(",(struct Cyc___cycFILE*)_check_null(Cyc_slurp_out));
({void*_tmp44=0U;({struct _fat_ptr _tmp3FF=({const char*_tmp45="Warning: declaration of calloc sidestepped\n";_tag_fat(_tmp45,sizeof(char),44U);});Cyc_log(_tmp3FF,_tag_fat(_tmp44,sizeof(void*),0U));});});
return 1;case 6U: _LLD: _LLE:
# 464
 Cyc_fputs("__region",(struct Cyc___cycFILE*)_check_null(Cyc_slurp_out));
({void*_tmp46=0U;({struct _fat_ptr _tmp400=({const char*_tmp47="Warning: use of region sidestepped\n";_tag_fat(_tmp47,sizeof(char),36U);});Cyc_log(_tmp400,_tag_fat(_tmp46,sizeof(void*),0U));});});
return 1;case 7U: _LLF: _LL10:
# 468
({void*_tmp48=0U;({struct _fat_ptr _tmp401=({const char*_tmp49="Warning: use of __extension__ deleted\n";_tag_fat(_tmp49,sizeof(char),39U);});Cyc_log(_tmp401,_tag_fat(_tmp48,sizeof(void*),0U));});});
return 1;case 8U: _LL11: _LL12:
# 472 "buildlib.cyl"
({void*_tmp4A=0U;({struct _fat_ptr _tmp402=({const char*_tmp4B="Warning: use of nonnull attribute deleted\n";_tag_fat(_tmp4B,sizeof(char),43U);});Cyc_log(_tmp402,_tag_fat(_tmp4A,sizeof(void*),0U));});});
return 1;case 9U: _LL13: _LL14:
# 477 "buildlib.cyl"
({void*_tmp4C=0U;({struct _fat_ptr _tmp403=({const char*_tmp4D="Warning: use of mode HI deleted\n";_tag_fat(_tmp4D,sizeof(char),33U);});Cyc_log(_tmp403,_tag_fat(_tmp4C,sizeof(void*),0U));});});
return 1;case 10U: _LL15: _LL16:
# 480
({void*_tmp4E=0U;({struct _fat_ptr _tmp404=({const char*_tmp4F="Warning: use of mode SI deleted\n";_tag_fat(_tmp4F,sizeof(char),33U);});Cyc_log(_tmp404,_tag_fat(_tmp4E,sizeof(void*),0U));});});
return 1;case 11U: _LL17: _LL18:
# 483
({void*_tmp50=0U;({struct _fat_ptr _tmp405=({const char*_tmp51="Warning: use of mode QI deleted\n";_tag_fat(_tmp51,sizeof(char),33U);});Cyc_log(_tmp405,_tag_fat(_tmp50,sizeof(void*),0U));});});
return 1;case 12U: _LL19: _LL1A:
# 486
({void*_tmp52=0U;({struct _fat_ptr _tmp406=({const char*_tmp53="Warning: use of mode DI deleted\n";_tag_fat(_tmp53,sizeof(char),33U);});Cyc_log(_tmp406,_tag_fat(_tmp52,sizeof(void*),0U));});});
return 1;case 13U: _LL1B: _LL1C:
# 489
({void*_tmp54=0U;({struct _fat_ptr _tmp407=({const char*_tmp55="Warning: use of mode DI deleted\n";_tag_fat(_tmp55,sizeof(char),33U);});Cyc_log(_tmp407,_tag_fat(_tmp54,sizeof(void*),0U));});});
return 1;case 14U: _LL1D: _LL1E:
# 492
({void*_tmp56=0U;({struct _fat_ptr _tmp408=({const char*_tmp57="Warning: use of mode word deleted\n";_tag_fat(_tmp57,sizeof(char),35U);});Cyc_log(_tmp408,_tag_fat(_tmp56,sizeof(void*),0U));});});
return 1;case 15U: _LL1F: _LL20:
# 495
({void*_tmp58=0U;({struct _fat_ptr _tmp409=({const char*_tmp59="Warning: use of __attribute__((deprecated)) deleted\n";_tag_fat(_tmp59,sizeof(char),53U);});Cyc_log(_tmp409,_tag_fat(_tmp58,sizeof(void*),0U));});});
return 1;case 16U: _LL21: _LL22:
# 498
({void*_tmp5A=0U;({struct _fat_ptr _tmp40A=({const char*_tmp5B="Warning: use of __attribute__((__deprecated__)) deleted\n";_tag_fat(_tmp5B,sizeof(char),57U);});Cyc_log(_tmp40A,_tag_fat(_tmp5A,sizeof(void*),0U));});});
return 1;case 17U: _LL23: _LL24:
# 501
({void*_tmp5C=0U;({struct _fat_ptr _tmp40B=({const char*_tmp5D="Warning: use of __attribute__((__transparent_union__)) deleted\n";_tag_fat(_tmp5D,sizeof(char),64U);});Cyc_log(_tmp40B,_tag_fat(_tmp5C,sizeof(void*),0U));});});
return 1;case 18U: _LL25: _LL26:
# 504
 Cyc_fputs("inline",(struct Cyc___cycFILE*)_check_null(Cyc_slurp_out));return 1;case 19U: _LL27: _LL28:
# 506
 Cyc_fputs("inline",(struct Cyc___cycFILE*)_check_null(Cyc_slurp_out));return 1;case 20U: _LL29: _LL2A:
# 508
 Cyc_fputs("const",(struct Cyc___cycFILE*)_check_null(Cyc_slurp_out));return 1;case 21U: _LL2B: _LL2C:
# 510
 Cyc_fputs("const",(struct Cyc___cycFILE*)_check_null(Cyc_slurp_out));return 1;case 22U: _LL2D: _LL2E:
# 512
 Cyc_fputs("signed",(struct Cyc___cycFILE*)_check_null(Cyc_slurp_out));return 1;case 23U: _LL2F: _LL30:
# 514
 Cyc_fputs("signed",(struct Cyc___cycFILE*)_check_null(Cyc_slurp_out));return 1;case 24U: _LL31: _LL32:
# 516
 Cyc_fputs("signed",(struct Cyc___cycFILE*)_check_null(Cyc_slurp_out));return 1;case 25U: _LL33: _LL34:
# 521 "buildlib.cyl"
 Cyc_fputs("int",(struct Cyc___cycFILE*)_check_null(Cyc_slurp_out));return 1;case 26U: _LL35: _LL36:
# 523
 return 1;case 27U: _LL37: _LL38:
# 525
({int _tmp40C=(int)Cyc_Lexing_lexeme_char(lexbuf,0);Cyc_fputc(_tmp40C,(struct Cyc___cycFILE*)_check_null(Cyc_slurp_out));});return 1;default: _LL39: _LL3A:
(lexbuf->refill_buff)(lexbuf);
return Cyc_slurp_rec(lexbuf,lexstate);}_LL0:;}
# 529
(int)_throw((void*)({struct Cyc_Lexing_Error_exn_struct*_tmp5F=_cycalloc(sizeof(*_tmp5F));_tmp5F->tag=Cyc_Lexing_Error,({struct _fat_ptr _tmp40D=({const char*_tmp5E="some action didn't return!";_tag_fat(_tmp5E,sizeof(char),27U);});_tmp5F->f1=_tmp40D;});_tmp5F;}));}
# 229 "buildlib.cyl"
int Cyc_slurp(struct Cyc_Lexing_lexbuf*lexbuf){
# 531 "buildlib.cyl"
return Cyc_slurp_rec(lexbuf,5);}
# 229 "buildlib.cyl"
int Cyc_slurp_string_rec(struct Cyc_Lexing_lexbuf*lexbuf,int lexstate){
# 533 "buildlib.cyl"
lexstate=Cyc_lex_engine(lexstate,lexbuf);
{int _tmp60=lexstate;switch(_tmp60){case 0U: _LL1: _LL2:
# 530 "buildlib.cyl"
 return 0;case 1U: _LL3: _LL4:
# 532
 Cyc_fputc((int)'"',(struct Cyc___cycFILE*)_check_null(Cyc_slurp_out));return 0;case 2U: _LL5: _LL6:
# 534
({void*_tmp61=0U;({struct _fat_ptr _tmp40E=({const char*_tmp62="Warning: unclosed string\n";_tag_fat(_tmp62,sizeof(char),26U);});Cyc_log(_tmp40E,_tag_fat(_tmp61,sizeof(void*),0U));});});
({struct Cyc_String_pa_PrintArg_struct _tmp65=({struct Cyc_String_pa_PrintArg_struct _tmp36A;_tmp36A.tag=0U,({struct _fat_ptr _tmp40F=(struct _fat_ptr)((struct _fat_ptr)Cyc_Lexing_lexeme(lexbuf));_tmp36A.f1=_tmp40F;});_tmp36A;});void*_tmp63[1U];_tmp63[0]=& _tmp65;({struct Cyc___cycFILE*_tmp411=(struct Cyc___cycFILE*)_check_null(Cyc_slurp_out);struct _fat_ptr _tmp410=({const char*_tmp64="%s";_tag_fat(_tmp64,sizeof(char),3U);});Cyc_fprintf(_tmp411,_tmp410,_tag_fat(_tmp63,sizeof(void*),1U));});});return 1;case 3U: _LL7: _LL8:
# 537
({struct Cyc_String_pa_PrintArg_struct _tmp68=({struct Cyc_String_pa_PrintArg_struct _tmp36B;_tmp36B.tag=0U,({struct _fat_ptr _tmp412=(struct _fat_ptr)((struct _fat_ptr)Cyc_Lexing_lexeme(lexbuf));_tmp36B.f1=_tmp412;});_tmp36B;});void*_tmp66[1U];_tmp66[0]=& _tmp68;({struct Cyc___cycFILE*_tmp414=(struct Cyc___cycFILE*)_check_null(Cyc_slurp_out);struct _fat_ptr _tmp413=({const char*_tmp67="%s";_tag_fat(_tmp67,sizeof(char),3U);});Cyc_fprintf(_tmp414,_tmp413,_tag_fat(_tmp66,sizeof(void*),1U));});});return 1;case 4U: _LL9: _LLA:
# 539
({struct Cyc_String_pa_PrintArg_struct _tmp6B=({struct Cyc_String_pa_PrintArg_struct _tmp36C;_tmp36C.tag=0U,({struct _fat_ptr _tmp415=(struct _fat_ptr)((struct _fat_ptr)Cyc_Lexing_lexeme(lexbuf));_tmp36C.f1=_tmp415;});_tmp36C;});void*_tmp69[1U];_tmp69[0]=& _tmp6B;({struct Cyc___cycFILE*_tmp417=(struct Cyc___cycFILE*)_check_null(Cyc_slurp_out);struct _fat_ptr _tmp416=({const char*_tmp6A="%s";_tag_fat(_tmp6A,sizeof(char),3U);});Cyc_fprintf(_tmp417,_tmp416,_tag_fat(_tmp69,sizeof(void*),1U));});});return 1;case 5U: _LLB: _LLC:
# 541
({struct Cyc_String_pa_PrintArg_struct _tmp6E=({struct Cyc_String_pa_PrintArg_struct _tmp36D;_tmp36D.tag=0U,({struct _fat_ptr _tmp418=(struct _fat_ptr)((struct _fat_ptr)Cyc_Lexing_lexeme(lexbuf));_tmp36D.f1=_tmp418;});_tmp36D;});void*_tmp6C[1U];_tmp6C[0]=& _tmp6E;({struct Cyc___cycFILE*_tmp41A=(struct Cyc___cycFILE*)_check_null(Cyc_slurp_out);struct _fat_ptr _tmp419=({const char*_tmp6D="%s";_tag_fat(_tmp6D,sizeof(char),3U);});Cyc_fprintf(_tmp41A,_tmp419,_tag_fat(_tmp6C,sizeof(void*),1U));});});return 1;case 6U: _LLD: _LLE:
# 543
({struct Cyc_String_pa_PrintArg_struct _tmp71=({struct Cyc_String_pa_PrintArg_struct _tmp36E;_tmp36E.tag=0U,({struct _fat_ptr _tmp41B=(struct _fat_ptr)((struct _fat_ptr)Cyc_Lexing_lexeme(lexbuf));_tmp36E.f1=_tmp41B;});_tmp36E;});void*_tmp6F[1U];_tmp6F[0]=& _tmp71;({struct Cyc___cycFILE*_tmp41D=(struct Cyc___cycFILE*)_check_null(Cyc_slurp_out);struct _fat_ptr _tmp41C=({const char*_tmp70="%s";_tag_fat(_tmp70,sizeof(char),3U);});Cyc_fprintf(_tmp41D,_tmp41C,_tag_fat(_tmp6F,sizeof(void*),1U));});});return 1;case 7U: _LLF: _LL10:
# 545
({struct Cyc_String_pa_PrintArg_struct _tmp74=({struct Cyc_String_pa_PrintArg_struct _tmp36F;_tmp36F.tag=0U,({struct _fat_ptr _tmp41E=(struct _fat_ptr)((struct _fat_ptr)Cyc_Lexing_lexeme(lexbuf));_tmp36F.f1=_tmp41E;});_tmp36F;});void*_tmp72[1U];_tmp72[0]=& _tmp74;({struct Cyc___cycFILE*_tmp420=(struct Cyc___cycFILE*)_check_null(Cyc_slurp_out);struct _fat_ptr _tmp41F=({const char*_tmp73="%s";_tag_fat(_tmp73,sizeof(char),3U);});Cyc_fprintf(_tmp420,_tmp41F,_tag_fat(_tmp72,sizeof(void*),1U));});});return 1;case 8U: _LL11: _LL12:
# 547
({struct Cyc_String_pa_PrintArg_struct _tmp77=({struct Cyc_String_pa_PrintArg_struct _tmp370;_tmp370.tag=0U,({struct _fat_ptr _tmp421=(struct _fat_ptr)((struct _fat_ptr)Cyc_Lexing_lexeme(lexbuf));_tmp370.f1=_tmp421;});_tmp370;});void*_tmp75[1U];_tmp75[0]=& _tmp77;({struct Cyc___cycFILE*_tmp423=(struct Cyc___cycFILE*)_check_null(Cyc_slurp_out);struct _fat_ptr _tmp422=({const char*_tmp76="%s";_tag_fat(_tmp76,sizeof(char),3U);});Cyc_fprintf(_tmp423,_tmp422,_tag_fat(_tmp75,sizeof(void*),1U));});});return 1;default: _LL13: _LL14:
(lexbuf->refill_buff)(lexbuf);
return Cyc_slurp_string_rec(lexbuf,lexstate);}_LL0:;}
# 551
(int)_throw((void*)({struct Cyc_Lexing_Error_exn_struct*_tmp79=_cycalloc(sizeof(*_tmp79));_tmp79->tag=Cyc_Lexing_Error,({struct _fat_ptr _tmp424=({const char*_tmp78="some action didn't return!";_tag_fat(_tmp78,sizeof(char),27U);});_tmp79->f1=_tmp424;});_tmp79;}));}
# 229 "buildlib.cyl"
int Cyc_slurp_string(struct Cyc_Lexing_lexbuf*lexbuf){
# 553 "buildlib.cyl"
return Cyc_slurp_string_rec(lexbuf,6);}
# 229 "buildlib.cyl"
int Cyc_asmtok_rec(struct Cyc_Lexing_lexbuf*lexbuf,int lexstate){
# 555 "buildlib.cyl"
lexstate=Cyc_lex_engine(lexstate,lexbuf);
{int _tmp7A=lexstate;switch(_tmp7A){case 0U: _LL1: _LL2:
# 557 "buildlib.cyl"
 return 0;case 1U: _LL3: _LL4:
# 559
 if(Cyc_parens_to_match == 1)return 0;-- Cyc_parens_to_match;
# 561
return 1;case 2U: _LL5: _LL6:
# 563
 ++ Cyc_parens_to_match;
return 1;case 3U: _LL7: _LL8:
# 566
 while(Cyc_asm_string(lexbuf)){;}
return 1;case 4U: _LL9: _LLA:
# 569
 while(Cyc_asm_comment(lexbuf)){;}
return 1;case 5U: _LLB: _LLC:
# 572
 return 1;case 6U: _LLD: _LLE:
# 574
 return 1;default: _LLF: _LL10:
(lexbuf->refill_buff)(lexbuf);
return Cyc_asmtok_rec(lexbuf,lexstate);}_LL0:;}
# 578
(int)_throw((void*)({struct Cyc_Lexing_Error_exn_struct*_tmp7C=_cycalloc(sizeof(*_tmp7C));_tmp7C->tag=Cyc_Lexing_Error,({struct _fat_ptr _tmp425=({const char*_tmp7B="some action didn't return!";_tag_fat(_tmp7B,sizeof(char),27U);});_tmp7C->f1=_tmp425;});_tmp7C;}));}
# 229 "buildlib.cyl"
int Cyc_asmtok(struct Cyc_Lexing_lexbuf*lexbuf){
# 580 "buildlib.cyl"
return Cyc_asmtok_rec(lexbuf,7);}
# 229 "buildlib.cyl"
int Cyc_asm_string_rec(struct Cyc_Lexing_lexbuf*lexbuf,int lexstate){
# 582 "buildlib.cyl"
lexstate=Cyc_lex_engine(lexstate,lexbuf);
{int _tmp7D=lexstate;switch(_tmp7D){case 0U: _LL1: _LL2:
# 578 "buildlib.cyl"
({void*_tmp7E=0U;({struct _fat_ptr _tmp426=({const char*_tmp7F="Warning: unclosed string\n";_tag_fat(_tmp7F,sizeof(char),26U);});Cyc_log(_tmp426,_tag_fat(_tmp7E,sizeof(void*),0U));});});return 0;case 1U: _LL3: _LL4:
# 580
 return 0;case 2U: _LL5: _LL6:
# 582
({void*_tmp80=0U;({struct _fat_ptr _tmp427=({const char*_tmp81="Warning: unclosed string\n";_tag_fat(_tmp81,sizeof(char),26U);});Cyc_log(_tmp427,_tag_fat(_tmp80,sizeof(void*),0U));});});return 1;case 3U: _LL7: _LL8:
# 584
 return 1;case 4U: _LL9: _LLA:
# 586
 return 1;case 5U: _LLB: _LLC:
# 588
 return 1;case 6U: _LLD: _LLE:
# 590
 return 1;case 7U: _LLF: _LL10:
# 592
 return 1;case 8U: _LL11: _LL12:
# 594
 return 1;default: _LL13: _LL14:
(lexbuf->refill_buff)(lexbuf);
return Cyc_asm_string_rec(lexbuf,lexstate);}_LL0:;}
# 598
(int)_throw((void*)({struct Cyc_Lexing_Error_exn_struct*_tmp83=_cycalloc(sizeof(*_tmp83));_tmp83->tag=Cyc_Lexing_Error,({struct _fat_ptr _tmp428=({const char*_tmp82="some action didn't return!";_tag_fat(_tmp82,sizeof(char),27U);});_tmp83->f1=_tmp428;});_tmp83;}));}
# 229 "buildlib.cyl"
int Cyc_asm_string(struct Cyc_Lexing_lexbuf*lexbuf){
# 600 "buildlib.cyl"
return Cyc_asm_string_rec(lexbuf,8);}
# 229 "buildlib.cyl"
int Cyc_asm_comment_rec(struct Cyc_Lexing_lexbuf*lexbuf,int lexstate){
# 602 "buildlib.cyl"
lexstate=Cyc_lex_engine(lexstate,lexbuf);
{int _tmp84=lexstate;switch(_tmp84){case 0U: _LL1: _LL2:
# 598 "buildlib.cyl"
({void*_tmp85=0U;({struct _fat_ptr _tmp429=({const char*_tmp86="Warning: unclosed comment\n";_tag_fat(_tmp86,sizeof(char),27U);});Cyc_log(_tmp429,_tag_fat(_tmp85,sizeof(void*),0U));});});return 0;case 1U: _LL3: _LL4:
# 600
 return 0;case 2U: _LL5: _LL6:
# 602
 return 1;default: _LL7: _LL8:
(lexbuf->refill_buff)(lexbuf);
return Cyc_asm_comment_rec(lexbuf,lexstate);}_LL0:;}
# 606
(int)_throw((void*)({struct Cyc_Lexing_Error_exn_struct*_tmp88=_cycalloc(sizeof(*_tmp88));_tmp88->tag=Cyc_Lexing_Error,({struct _fat_ptr _tmp42A=({const char*_tmp87="some action didn't return!";_tag_fat(_tmp87,sizeof(char),27U);});_tmp88->f1=_tmp42A;});_tmp88;}));}
# 229 "buildlib.cyl"
int Cyc_asm_comment(struct Cyc_Lexing_lexbuf*lexbuf){
# 608 "buildlib.cyl"
return Cyc_asm_comment_rec(lexbuf,9);}
# 229 "buildlib.cyl"
struct _tuple26*Cyc_suck_line_rec(struct Cyc_Lexing_lexbuf*lexbuf,int lexstate){
# 610 "buildlib.cyl"
lexstate=Cyc_lex_engine(lexstate,lexbuf);
{int _tmp89=lexstate;switch(_tmp89){case 0U: _LL1: _LL2:
# 610 "buildlib.cyl"
 Cyc_current_line=({const char*_tmp8A="#define ";_tag_fat(_tmp8A,sizeof(char),9U);});
Cyc_suck_macroname(lexbuf);
return({struct _tuple26*_tmp8B=_cycalloc(sizeof(*_tmp8B));_tmp8B->f1=Cyc_current_line,_tmp8B->f2=(struct _fat_ptr*)_check_null(Cyc_current_source);_tmp8B;});case 1U: _LL3: _LL4:
# 614
 return Cyc_suck_line(lexbuf);case 2U: _LL5: _LL6:
# 616
 return 0;default: _LL7: _LL8:
(lexbuf->refill_buff)(lexbuf);
return Cyc_suck_line_rec(lexbuf,lexstate);}_LL0:;}
# 620
(int)_throw((void*)({struct Cyc_Lexing_Error_exn_struct*_tmp8D=_cycalloc(sizeof(*_tmp8D));_tmp8D->tag=Cyc_Lexing_Error,({struct _fat_ptr _tmp42B=({const char*_tmp8C="some action didn't return!";_tag_fat(_tmp8C,sizeof(char),27U);});_tmp8D->f1=_tmp42B;});_tmp8D;}));}
# 229 "buildlib.cyl"
struct _tuple26*Cyc_suck_line(struct Cyc_Lexing_lexbuf*lexbuf){
# 622 "buildlib.cyl"
return Cyc_suck_line_rec(lexbuf,10);}
# 229 "buildlib.cyl"
int Cyc_suck_macroname_rec(struct Cyc_Lexing_lexbuf*lexbuf,int lexstate){
# 624 "buildlib.cyl"
lexstate=Cyc_lex_engine(lexstate,lexbuf);
{int _tmp8E=lexstate;if(_tmp8E == 0){_LL1: _LL2:
# 620 "buildlib.cyl"
 Cyc_current_source=({struct _fat_ptr*_tmp8F=_cycalloc(sizeof(*_tmp8F));({struct _fat_ptr _tmp42C=(struct _fat_ptr)Cyc_Lexing_lexeme(lexbuf);*_tmp8F=_tmp42C;});_tmp8F;});
Cyc_current_line=(struct _fat_ptr)Cyc_strconcat((struct _fat_ptr)Cyc_current_line,(struct _fat_ptr)*((struct _fat_ptr*)_check_null(Cyc_current_source)));
return Cyc_suck_restofline(lexbuf);}else{_LL3: _LL4:
# 624
(lexbuf->refill_buff)(lexbuf);
return Cyc_suck_macroname_rec(lexbuf,lexstate);}_LL0:;}
# 627
(int)_throw((void*)({struct Cyc_Lexing_Error_exn_struct*_tmp91=_cycalloc(sizeof(*_tmp91));_tmp91->tag=Cyc_Lexing_Error,({struct _fat_ptr _tmp42D=({const char*_tmp90="some action didn't return!";_tag_fat(_tmp90,sizeof(char),27U);});_tmp91->f1=_tmp42D;});_tmp91;}));}
# 229 "buildlib.cyl"
int Cyc_suck_macroname(struct Cyc_Lexing_lexbuf*lexbuf){
# 629 "buildlib.cyl"
return Cyc_suck_macroname_rec(lexbuf,11);}
# 229 "buildlib.cyl"
int Cyc_suck_restofline_rec(struct Cyc_Lexing_lexbuf*lexbuf,int lexstate){
# 631 "buildlib.cyl"
lexstate=Cyc_lex_engine(lexstate,lexbuf);
{int _tmp92=lexstate;if(_tmp92 == 0){_LL1: _LL2:
# 627 "buildlib.cyl"
 Cyc_current_line=(struct _fat_ptr)({struct _fat_ptr _tmp42E=(struct _fat_ptr)Cyc_current_line;Cyc_strconcat(_tmp42E,(struct _fat_ptr)Cyc_Lexing_lexeme(lexbuf));});return 0;}else{_LL3: _LL4:
(lexbuf->refill_buff)(lexbuf);
return Cyc_suck_restofline_rec(lexbuf,lexstate);}_LL0:;}
# 631
(int)_throw((void*)({struct Cyc_Lexing_Error_exn_struct*_tmp94=_cycalloc(sizeof(*_tmp94));_tmp94->tag=Cyc_Lexing_Error,({struct _fat_ptr _tmp42F=({const char*_tmp93="some action didn't return!";_tag_fat(_tmp93,sizeof(char),27U);});_tmp94->f1=_tmp42F;});_tmp94;}));}
# 229 "buildlib.cyl"
int Cyc_suck_restofline(struct Cyc_Lexing_lexbuf*lexbuf){
# 633 "buildlib.cyl"
return Cyc_suck_restofline_rec(lexbuf,12);}
# 229 "buildlib.cyl"
struct _tuple29*Cyc_spec_rec(struct Cyc_Lexing_lexbuf*lexbuf,int lexstate){
# 635 "buildlib.cyl"
lexstate=Cyc_lex_engine(lexstate,lexbuf);
{int _tmp95=lexstate;switch(_tmp95){case 0U: _LL1: _LL2:
# 634 "buildlib.cyl"
 return Cyc_spec(lexbuf);case 1U: _LL3: _LL4:
# 636
 Cyc_current_headerfile=(struct _fat_ptr)({
struct _fat_ptr _tmp431=(struct _fat_ptr)Cyc_Lexing_lexeme(lexbuf);Cyc_substring(_tmp431,0,(unsigned long)(({
int _tmp430=Cyc_Lexing_lexeme_end(lexbuf);_tmp430 - Cyc_Lexing_lexeme_start(lexbuf);})- 1));});
Cyc_current_symbols=0;
Cyc_current_user_defs=0;
Cyc_current_omit_symbols=0;
Cyc_current_cstubs=0;
Cyc_current_cycstubs=0;
Cyc_current_hstubs=0;
Cyc_current_cpp=0;
while(Cyc_commands(lexbuf)){;}
Cyc_current_hstubs=Cyc_List_imp_rev(Cyc_current_hstubs);
Cyc_current_cstubs=Cyc_List_imp_rev(Cyc_current_cstubs);
Cyc_current_cycstubs=Cyc_List_imp_rev(Cyc_current_cycstubs);
Cyc_current_cpp=Cyc_List_imp_rev(Cyc_current_cpp);
return({struct _tuple29*_tmp96=_cycalloc(sizeof(*_tmp96));_tmp96->f1=Cyc_current_headerfile,_tmp96->f2=Cyc_current_symbols,_tmp96->f3=Cyc_current_user_defs,_tmp96->f4=Cyc_current_omit_symbols,_tmp96->f5=Cyc_current_hstubs,_tmp96->f6=Cyc_current_cstubs,_tmp96->f7=Cyc_current_cycstubs,_tmp96->f8=Cyc_current_cpp;_tmp96;});case 2U: _LL5: _LL6:
# 661
 return Cyc_spec(lexbuf);case 3U: _LL7: _LL8:
# 663
 return 0;case 4U: _LL9: _LLA:
# 665
({struct Cyc_Int_pa_PrintArg_struct _tmp99=({struct Cyc_Int_pa_PrintArg_struct _tmp371;_tmp371.tag=1U,({
# 667
unsigned long _tmp432=(unsigned long)((int)Cyc_Lexing_lexeme_char(lexbuf,0));_tmp371.f1=_tmp432;});_tmp371;});void*_tmp97[1U];_tmp97[0]=& _tmp99;({struct Cyc___cycFILE*_tmp434=Cyc_stderr;struct _fat_ptr _tmp433=({const char*_tmp98="Error in .cys file: expected header file name, found '%c' instead\n";_tag_fat(_tmp98,sizeof(char),67U);});Cyc_fprintf(_tmp434,_tmp433,_tag_fat(_tmp97,sizeof(void*),1U));});});
return 0;default: _LLB: _LLC:
(lexbuf->refill_buff)(lexbuf);
return Cyc_spec_rec(lexbuf,lexstate);}_LL0:;}
# 672
(int)_throw((void*)({struct Cyc_Lexing_Error_exn_struct*_tmp9B=_cycalloc(sizeof(*_tmp9B));_tmp9B->tag=Cyc_Lexing_Error,({struct _fat_ptr _tmp435=({const char*_tmp9A="some action didn't return!";_tag_fat(_tmp9A,sizeof(char),27U);});_tmp9B->f1=_tmp435;});_tmp9B;}));}
# 229 "buildlib.cyl"
struct _tuple29*Cyc_spec(struct Cyc_Lexing_lexbuf*lexbuf){
# 674 "buildlib.cyl"
return Cyc_spec_rec(lexbuf,13);}
# 229 "buildlib.cyl"
int Cyc_commands_rec(struct Cyc_Lexing_lexbuf*lexbuf,int lexstate){
# 676 "buildlib.cyl"
lexstate=Cyc_lex_engine(lexstate,lexbuf);
{int _tmp9C=lexstate;switch(_tmp9C){case 0U: _LL1: _LL2:
# 672 "buildlib.cyl"
 return 0;case 1U: _LL3: _LL4:
# 674
 return 0;case 2U: _LL5: _LL6:
# 676
 Cyc_snarfed_symbols=0;
while(Cyc_snarfsymbols(lexbuf)){;}
Cyc_current_symbols=Cyc_List_append(Cyc_snarfed_symbols,Cyc_current_symbols);
return 1;case 3U: _LL7: _LL8:
# 681
 Cyc_snarfed_symbols=0;{
struct Cyc_List_List*tmp_user_defs=Cyc_current_user_defs;
while(Cyc_snarfsymbols(lexbuf)){;}
if(tmp_user_defs != Cyc_current_user_defs){
({void*_tmp9D=0U;({struct Cyc___cycFILE*_tmp437=Cyc_stderr;struct _fat_ptr _tmp436=({const char*_tmp9E="Error in .cys file: got optional definition in omitsymbols\n";_tag_fat(_tmp9E,sizeof(char),60U);});Cyc_fprintf(_tmp437,_tmp436,_tag_fat(_tmp9D,sizeof(void*),0U));});});
# 687
return 0;}
# 684
Cyc_current_omit_symbols=
# 689
Cyc_List_append(Cyc_snarfed_symbols,Cyc_current_omit_symbols);
return 1;}case 4U: _LL9: _LLA:
# 692
 Cyc_braces_to_match=1;
Cyc_specbuf=Cyc_Buffer_create(255U);
while(Cyc_block(lexbuf)){;}{
struct _tuple27*x=({struct _tuple27*_tmpA0=_cycalloc(sizeof(*_tmpA0));({struct _fat_ptr _tmp439=(struct _fat_ptr)_tag_fat(0,0,0);_tmpA0->f1=_tmp439;}),({
struct _fat_ptr _tmp438=(struct _fat_ptr)Cyc_Buffer_contents((struct Cyc_Buffer_t*)_check_null(Cyc_specbuf));_tmpA0->f2=_tmp438;});_tmpA0;});
Cyc_current_hstubs=({struct Cyc_List_List*_tmp9F=_cycalloc(sizeof(*_tmp9F));_tmp9F->hd=x,_tmp9F->tl=Cyc_current_hstubs;_tmp9F;});
return 1;}case 5U: _LLB: _LLC: {
# 700
struct _fat_ptr s=Cyc_Lexing_lexeme(lexbuf);
_fat_ptr_inplace_plus(& s,sizeof(char),(int)Cyc_strlen(({const char*_tmpA1="hstub";_tag_fat(_tmpA1,sizeof(char),6U);})));
while(isspace((int)*((char*)_check_fat_subscript(s,sizeof(char),0U)))){_fat_ptr_inplace_plus(& s,sizeof(char),1);}{
struct _fat_ptr t=s;
while(!isspace((int)*((char*)_check_fat_subscript(t,sizeof(char),0U)))){_fat_ptr_inplace_plus(& t,sizeof(char),1);}{
struct _fat_ptr symbol=Cyc_substring((struct _fat_ptr)s,0,(unsigned long)((t.curr - s.curr)/ sizeof(char)));
Cyc_braces_to_match=1;
Cyc_specbuf=Cyc_Buffer_create(255U);
while(Cyc_block(lexbuf)){;}{
struct _tuple27*x=({struct _tuple27*_tmpA3=_cycalloc(sizeof(*_tmpA3));_tmpA3->f1=(struct _fat_ptr)symbol,({
struct _fat_ptr _tmp43A=(struct _fat_ptr)Cyc_Buffer_contents((struct Cyc_Buffer_t*)_check_null(Cyc_specbuf));_tmpA3->f2=_tmp43A;});_tmpA3;});
Cyc_current_hstubs=({struct Cyc_List_List*_tmpA2=_cycalloc(sizeof(*_tmpA2));_tmpA2->hd=x,_tmpA2->tl=Cyc_current_hstubs;_tmpA2;});
return 1;}}}}case 6U: _LLD: _LLE:
# 714
 Cyc_braces_to_match=1;
Cyc_specbuf=Cyc_Buffer_create(255U);
while(Cyc_block(lexbuf)){;}{
struct _tuple27*x=({struct _tuple27*_tmpA5=_cycalloc(sizeof(*_tmpA5));({struct _fat_ptr _tmp43C=(struct _fat_ptr)_tag_fat(0,0,0);_tmpA5->f1=_tmp43C;}),({
struct _fat_ptr _tmp43B=(struct _fat_ptr)Cyc_Buffer_contents((struct Cyc_Buffer_t*)_check_null(Cyc_specbuf));_tmpA5->f2=_tmp43B;});_tmpA5;});
Cyc_current_cstubs=({struct Cyc_List_List*_tmpA4=_cycalloc(sizeof(*_tmpA4));_tmpA4->hd=x,_tmpA4->tl=Cyc_current_cstubs;_tmpA4;});
return 1;}case 7U: _LLF: _LL10: {
# 722
struct _fat_ptr s=Cyc_Lexing_lexeme(lexbuf);
_fat_ptr_inplace_plus(& s,sizeof(char),(int)Cyc_strlen(({const char*_tmpA6="cstub";_tag_fat(_tmpA6,sizeof(char),6U);})));
while(isspace((int)*((char*)_check_fat_subscript(s,sizeof(char),0U)))){_fat_ptr_inplace_plus(& s,sizeof(char),1);}{
struct _fat_ptr t=s;
while(!isspace((int)*((char*)_check_fat_subscript(t,sizeof(char),0U)))){_fat_ptr_inplace_plus(& t,sizeof(char),1);}{
struct _fat_ptr symbol=Cyc_substring((struct _fat_ptr)s,0,(unsigned long)((t.curr - s.curr)/ sizeof(char)));
Cyc_braces_to_match=1;
Cyc_specbuf=Cyc_Buffer_create(255U);
while(Cyc_block(lexbuf)){;}{
struct _tuple27*x=({struct _tuple27*_tmpA8=_cycalloc(sizeof(*_tmpA8));_tmpA8->f1=(struct _fat_ptr)symbol,({
struct _fat_ptr _tmp43D=(struct _fat_ptr)Cyc_Buffer_contents((struct Cyc_Buffer_t*)_check_null(Cyc_specbuf));_tmpA8->f2=_tmp43D;});_tmpA8;});
Cyc_current_cstubs=({struct Cyc_List_List*_tmpA7=_cycalloc(sizeof(*_tmpA7));_tmpA7->hd=x,_tmpA7->tl=Cyc_current_cstubs;_tmpA7;});
return 1;}}}}case 8U: _LL11: _LL12:
# 736
 Cyc_braces_to_match=1;
Cyc_specbuf=Cyc_Buffer_create(255U);
while(Cyc_block(lexbuf)){;}{
struct _tuple27*x=({struct _tuple27*_tmpAA=_cycalloc(sizeof(*_tmpAA));({struct _fat_ptr _tmp43F=(struct _fat_ptr)_tag_fat(0,0,0);_tmpAA->f1=_tmp43F;}),({
struct _fat_ptr _tmp43E=(struct _fat_ptr)Cyc_Buffer_contents((struct Cyc_Buffer_t*)_check_null(Cyc_specbuf));_tmpAA->f2=_tmp43E;});_tmpAA;});
Cyc_current_cycstubs=({struct Cyc_List_List*_tmpA9=_cycalloc(sizeof(*_tmpA9));_tmpA9->hd=x,_tmpA9->tl=Cyc_current_cycstubs;_tmpA9;});
return 1;}case 9U: _LL13: _LL14: {
# 744
struct _fat_ptr s=Cyc_Lexing_lexeme(lexbuf);
_fat_ptr_inplace_plus(& s,sizeof(char),(int)Cyc_strlen(({const char*_tmpAB="cycstub";_tag_fat(_tmpAB,sizeof(char),8U);})));
while(isspace((int)*((char*)_check_fat_subscript(s,sizeof(char),0U)))){_fat_ptr_inplace_plus(& s,sizeof(char),1);}{
struct _fat_ptr t=s;
while(!isspace((int)*((char*)_check_fat_subscript(t,sizeof(char),0U)))){_fat_ptr_inplace_plus(& t,sizeof(char),1);}{
struct _fat_ptr symbol=Cyc_substring((struct _fat_ptr)s,0,(unsigned long)((t.curr - s.curr)/ sizeof(char)));
Cyc_braces_to_match=1;
Cyc_specbuf=Cyc_Buffer_create(255U);
while(Cyc_block(lexbuf)){;}{
struct _tuple27*x=({struct _tuple27*_tmpAD=_cycalloc(sizeof(*_tmpAD));_tmpAD->f1=(struct _fat_ptr)symbol,({
struct _fat_ptr _tmp440=(struct _fat_ptr)Cyc_Buffer_contents((struct Cyc_Buffer_t*)_check_null(Cyc_specbuf));_tmpAD->f2=_tmp440;});_tmpAD;});
Cyc_current_cycstubs=({struct Cyc_List_List*_tmpAC=_cycalloc(sizeof(*_tmpAC));_tmpAC->hd=x,_tmpAC->tl=Cyc_current_cycstubs;_tmpAC;});
return 1;}}}}case 10U: _LL15: _LL16:
# 758
 Cyc_braces_to_match=1;
Cyc_specbuf=Cyc_Buffer_create(255U);
while(Cyc_block(lexbuf)){;}{
struct _fat_ptr*x=({struct _fat_ptr*_tmpAF=_cycalloc(sizeof(*_tmpAF));({struct _fat_ptr _tmp441=(struct _fat_ptr)Cyc_Buffer_contents((struct Cyc_Buffer_t*)_check_null(Cyc_specbuf));*_tmpAF=_tmp441;});_tmpAF;});
Cyc_current_cpp=({struct Cyc_List_List*_tmpAE=_cycalloc(sizeof(*_tmpAE));_tmpAE->hd=x,_tmpAE->tl=Cyc_current_cpp;_tmpAE;});
return 1;}case 11U: _LL17: _LL18:
# 765
 return 1;case 12U: _LL19: _LL1A:
# 767
 return 1;case 13U: _LL1B: _LL1C:
# 769
({struct Cyc_Int_pa_PrintArg_struct _tmpB2=({struct Cyc_Int_pa_PrintArg_struct _tmp372;_tmp372.tag=1U,({
# 771
unsigned long _tmp442=(unsigned long)((int)Cyc_Lexing_lexeme_char(lexbuf,0));_tmp372.f1=_tmp442;});_tmp372;});void*_tmpB0[1U];_tmpB0[0]=& _tmpB2;({struct Cyc___cycFILE*_tmp444=Cyc_stderr;struct _fat_ptr _tmp443=({const char*_tmpB1="Error in .cys file: expected command, found '%c' instead\n";_tag_fat(_tmpB1,sizeof(char),58U);});Cyc_fprintf(_tmp444,_tmp443,_tag_fat(_tmpB0,sizeof(void*),1U));});});
return 0;default: _LL1D: _LL1E:
(lexbuf->refill_buff)(lexbuf);
return Cyc_commands_rec(lexbuf,lexstate);}_LL0:;}
# 776
(int)_throw((void*)({struct Cyc_Lexing_Error_exn_struct*_tmpB4=_cycalloc(sizeof(*_tmpB4));_tmpB4->tag=Cyc_Lexing_Error,({struct _fat_ptr _tmp445=({const char*_tmpB3="some action didn't return!";_tag_fat(_tmpB3,sizeof(char),27U);});_tmpB4->f1=_tmp445;});_tmpB4;}));}
# 229 "buildlib.cyl"
int Cyc_commands(struct Cyc_Lexing_lexbuf*lexbuf){
# 778 "buildlib.cyl"
return Cyc_commands_rec(lexbuf,14);}
# 229 "buildlib.cyl"
int Cyc_snarfsymbols_rec(struct Cyc_Lexing_lexbuf*lexbuf,int lexstate){
# 780 "buildlib.cyl"
lexstate=Cyc_lex_engine(lexstate,lexbuf);
{int _tmpB5=lexstate;switch(_tmpB5){case 0U: _LL1: _LL2:
# 782 "buildlib.cyl"
 Cyc_snarfed_symbols=({struct Cyc_List_List*_tmpB7=_cycalloc(sizeof(*_tmpB7));({struct _fat_ptr*_tmp447=({struct _fat_ptr*_tmpB6=_cycalloc(sizeof(*_tmpB6));({struct _fat_ptr _tmp446=(struct _fat_ptr)Cyc_Lexing_lexeme(lexbuf);*_tmpB6=_tmp446;});_tmpB6;});_tmpB7->hd=_tmp447;}),_tmpB7->tl=Cyc_snarfed_symbols;_tmpB7;});
return 1;case 1U: _LL3: _LL4: {
# 785
struct _fat_ptr s=Cyc_Lexing_lexeme(lexbuf);
struct _fat_ptr t=s;
while(!isspace((int)*((char*)_check_fat_subscript(t,sizeof(char),0U)))){_fat_ptr_inplace_plus(& t,sizeof(char),1);}
Cyc_current_symbol=(struct _fat_ptr)Cyc_substring((struct _fat_ptr)s,0,(unsigned long)((t.curr - s.curr)/ sizeof(char)));
Cyc_rename_current_symbol=1;
Cyc_braces_to_match=1;
Cyc_specbuf=Cyc_Buffer_create(255U);
while(Cyc_block(lexbuf)){;}
# 794
Cyc_rename_current_symbol=0;{
struct _tuple28*user_def=({struct _tuple28*_tmpBD=_cycalloc(sizeof(*_tmpBD));({struct _fat_ptr*_tmp44A=({struct _fat_ptr*_tmpBB=_cycalloc(sizeof(*_tmpBB));*_tmpBB=Cyc_current_symbol;_tmpBB;});_tmpBD->f1=_tmp44A;}),({
struct _fat_ptr*_tmp449=({struct _fat_ptr*_tmpBC=_cycalloc(sizeof(*_tmpBC));({struct _fat_ptr _tmp448=(struct _fat_ptr)Cyc_Buffer_contents((struct Cyc_Buffer_t*)_check_null(Cyc_specbuf));*_tmpBC=_tmp448;});_tmpBC;});_tmpBD->f2=_tmp449;});_tmpBD;});
Cyc_snarfed_symbols=({struct Cyc_List_List*_tmpB9=_cycalloc(sizeof(*_tmpB9));({struct _fat_ptr*_tmp44B=({struct _fat_ptr*_tmpB8=_cycalloc(sizeof(*_tmpB8));*_tmpB8=(struct _fat_ptr)Cyc_current_symbol;_tmpB8;});_tmpB9->hd=_tmp44B;}),_tmpB9->tl=Cyc_snarfed_symbols;_tmpB9;});
Cyc_current_user_defs=({struct Cyc_List_List*_tmpBA=_cycalloc(sizeof(*_tmpBA));_tmpBA->hd=user_def,_tmpBA->tl=Cyc_current_user_defs;_tmpBA;});
return 1;}}case 2U: _LL5: _LL6:
# 801
 return 1;case 3U: _LL7: _LL8:
# 803
 return 0;case 4U: _LL9: _LLA:
# 805
({void*_tmpBE=0U;({struct Cyc___cycFILE*_tmp44D=Cyc_stderr;struct _fat_ptr _tmp44C=({const char*_tmpBF="Error in .cys file: unexpected end-of-file\n";_tag_fat(_tmpBF,sizeof(char),44U);});Cyc_fprintf(_tmp44D,_tmp44C,_tag_fat(_tmpBE,sizeof(void*),0U));});});
# 807
return 0;case 5U: _LLB: _LLC:
# 809
({struct Cyc_Int_pa_PrintArg_struct _tmpC2=({struct Cyc_Int_pa_PrintArg_struct _tmp373;_tmp373.tag=1U,({
# 811
unsigned long _tmp44E=(unsigned long)((int)Cyc_Lexing_lexeme_char(lexbuf,0));_tmp373.f1=_tmp44E;});_tmp373;});void*_tmpC0[1U];_tmpC0[0]=& _tmpC2;({struct Cyc___cycFILE*_tmp450=Cyc_stderr;struct _fat_ptr _tmp44F=({const char*_tmpC1="Error in .cys file: expected symbol, found '%c' instead\n";_tag_fat(_tmpC1,sizeof(char),57U);});Cyc_fprintf(_tmp450,_tmp44F,_tag_fat(_tmpC0,sizeof(void*),1U));});});
return 0;default: _LLD: _LLE:
(lexbuf->refill_buff)(lexbuf);
return Cyc_snarfsymbols_rec(lexbuf,lexstate);}_LL0:;}
# 816
(int)_throw((void*)({struct Cyc_Lexing_Error_exn_struct*_tmpC4=_cycalloc(sizeof(*_tmpC4));_tmpC4->tag=Cyc_Lexing_Error,({struct _fat_ptr _tmp451=({const char*_tmpC3="some action didn't return!";_tag_fat(_tmpC3,sizeof(char),27U);});_tmpC4->f1=_tmp451;});_tmpC4;}));}
# 229 "buildlib.cyl"
int Cyc_snarfsymbols(struct Cyc_Lexing_lexbuf*lexbuf){
# 818 "buildlib.cyl"
return Cyc_snarfsymbols_rec(lexbuf,15);}
# 229 "buildlib.cyl"
int Cyc_block_rec(struct Cyc_Lexing_lexbuf*lexbuf,int lexstate){
# 820 "buildlib.cyl"
lexstate=Cyc_lex_engine(lexstate,lexbuf);
{int _tmpC5=lexstate;switch(_tmpC5){case 0U: _LL1: _LL2:
# 822 "buildlib.cyl"
({void*_tmpC6=0U;({struct _fat_ptr _tmp452=({const char*_tmpC7="Warning: unclosed brace\n";_tag_fat(_tmpC7,sizeof(char),25U);});Cyc_log(_tmp452,_tag_fat(_tmpC6,sizeof(void*),0U));});});return 0;case 1U: _LL3: _LL4:
# 824
 if(Cyc_braces_to_match == 1)return 0;-- Cyc_braces_to_match;
# 826
Cyc_Buffer_add_char((struct Cyc_Buffer_t*)_check_null(Cyc_specbuf),'}');
return 1;case 2U: _LL5: _LL6:
# 829
 ++ Cyc_braces_to_match;
Cyc_Buffer_add_char((struct Cyc_Buffer_t*)_check_null(Cyc_specbuf),'{');
return 1;case 3U: _LL7: _LL8:
# 833
 Cyc_Buffer_add_char((struct Cyc_Buffer_t*)_check_null(Cyc_specbuf),'"');
while(Cyc_block_string(lexbuf)){;}
return 1;case 4U: _LL9: _LLA:
# 837
({struct Cyc_Buffer_t*_tmp453=(struct Cyc_Buffer_t*)_check_null(Cyc_specbuf);Cyc_Buffer_add_string(_tmp453,({const char*_tmpC8="/*";_tag_fat(_tmpC8,sizeof(char),3U);}));});
while(Cyc_block_comment(lexbuf)){;}
return 1;case 5U: _LLB: _LLC:
# 841
({struct Cyc_Buffer_t*_tmp454=(struct Cyc_Buffer_t*)_check_null(Cyc_specbuf);Cyc_Buffer_add_string(_tmp454,(struct _fat_ptr)Cyc_Lexing_lexeme(lexbuf));});
return 1;case 6U: _LLD: _LLE: {
# 844
struct _fat_ptr symbol=(struct _fat_ptr)Cyc_Lexing_lexeme(lexbuf);
if(Cyc_rename_current_symbol && !Cyc_strcmp((struct _fat_ptr)symbol,(struct _fat_ptr)Cyc_current_symbol))
({struct Cyc_Buffer_t*_tmp455=(struct Cyc_Buffer_t*)_check_null(Cyc_specbuf);Cyc_Buffer_add_string(_tmp455,*Cyc_add_user_prefix(({struct _fat_ptr*_tmpC9=_cycalloc(sizeof(*_tmpC9));*_tmpC9=symbol;_tmpC9;})));});else{
# 848
Cyc_Buffer_add_string((struct Cyc_Buffer_t*)_check_null(Cyc_specbuf),symbol);}
return 1;}case 7U: _LLF: _LL10:
# 851
({struct Cyc_Buffer_t*_tmp456=(struct Cyc_Buffer_t*)_check_null(Cyc_specbuf);Cyc_Buffer_add_char(_tmp456,Cyc_Lexing_lexeme_char(lexbuf,0));});
return 1;default: _LL11: _LL12:
(lexbuf->refill_buff)(lexbuf);
return Cyc_block_rec(lexbuf,lexstate);}_LL0:;}
# 856
(int)_throw((void*)({struct Cyc_Lexing_Error_exn_struct*_tmpCB=_cycalloc(sizeof(*_tmpCB));_tmpCB->tag=Cyc_Lexing_Error,({struct _fat_ptr _tmp457=({const char*_tmpCA="some action didn't return!";_tag_fat(_tmpCA,sizeof(char),27U);});_tmpCB->f1=_tmp457;});_tmpCB;}));}
# 229 "buildlib.cyl"
int Cyc_block(struct Cyc_Lexing_lexbuf*lexbuf){
# 858 "buildlib.cyl"
return Cyc_block_rec(lexbuf,16);}
# 229 "buildlib.cyl"
int Cyc_block_string_rec(struct Cyc_Lexing_lexbuf*lexbuf,int lexstate){
# 860 "buildlib.cyl"
lexstate=Cyc_lex_engine(lexstate,lexbuf);
{int _tmpCC=lexstate;switch(_tmpCC){case 0U: _LL1: _LL2:
# 856 "buildlib.cyl"
({void*_tmpCD=0U;({struct _fat_ptr _tmp458=({const char*_tmpCE="Warning: unclosed string\n";_tag_fat(_tmpCE,sizeof(char),26U);});Cyc_log(_tmp458,_tag_fat(_tmpCD,sizeof(void*),0U));});});return 0;case 1U: _LL3: _LL4:
# 858
 Cyc_Buffer_add_char((struct Cyc_Buffer_t*)_check_null(Cyc_specbuf),'"');return 0;case 2U: _LL5: _LL6:
# 860
({void*_tmpCF=0U;({struct _fat_ptr _tmp459=({const char*_tmpD0="Warning: unclosed string\n";_tag_fat(_tmpD0,sizeof(char),26U);});Cyc_log(_tmp459,_tag_fat(_tmpCF,sizeof(void*),0U));});});
({struct Cyc_Buffer_t*_tmp45A=(struct Cyc_Buffer_t*)_check_null(Cyc_specbuf);Cyc_Buffer_add_string(_tmp45A,(struct _fat_ptr)Cyc_Lexing_lexeme(lexbuf));});
return 1;case 3U: _LL7: _LL8:
# 864
({struct Cyc_Buffer_t*_tmp45B=(struct Cyc_Buffer_t*)_check_null(Cyc_specbuf);Cyc_Buffer_add_string(_tmp45B,(struct _fat_ptr)Cyc_Lexing_lexeme(lexbuf));});
return 1;case 4U: _LL9: _LLA:
# 867
({struct Cyc_Buffer_t*_tmp45C=(struct Cyc_Buffer_t*)_check_null(Cyc_specbuf);Cyc_Buffer_add_string(_tmp45C,(struct _fat_ptr)Cyc_Lexing_lexeme(lexbuf));});
return 1;case 5U: _LLB: _LLC:
# 870
({struct Cyc_Buffer_t*_tmp45D=(struct Cyc_Buffer_t*)_check_null(Cyc_specbuf);Cyc_Buffer_add_string(_tmp45D,(struct _fat_ptr)Cyc_Lexing_lexeme(lexbuf));});
return 1;case 6U: _LLD: _LLE:
# 873
({struct Cyc_Buffer_t*_tmp45E=(struct Cyc_Buffer_t*)_check_null(Cyc_specbuf);Cyc_Buffer_add_string(_tmp45E,(struct _fat_ptr)Cyc_Lexing_lexeme(lexbuf));});
return 1;case 7U: _LLF: _LL10:
# 876
({struct Cyc_Buffer_t*_tmp45F=(struct Cyc_Buffer_t*)_check_null(Cyc_specbuf);Cyc_Buffer_add_string(_tmp45F,(struct _fat_ptr)Cyc_Lexing_lexeme(lexbuf));});
return 1;case 8U: _LL11: _LL12:
# 879
({struct Cyc_Buffer_t*_tmp460=(struct Cyc_Buffer_t*)_check_null(Cyc_specbuf);Cyc_Buffer_add_string(_tmp460,(struct _fat_ptr)Cyc_Lexing_lexeme(lexbuf));});
return 1;default: _LL13: _LL14:
(lexbuf->refill_buff)(lexbuf);
return Cyc_block_string_rec(lexbuf,lexstate);}_LL0:;}
# 884
(int)_throw((void*)({struct Cyc_Lexing_Error_exn_struct*_tmpD2=_cycalloc(sizeof(*_tmpD2));_tmpD2->tag=Cyc_Lexing_Error,({struct _fat_ptr _tmp461=({const char*_tmpD1="some action didn't return!";_tag_fat(_tmpD1,sizeof(char),27U);});_tmpD2->f1=_tmp461;});_tmpD2;}));}
# 229 "buildlib.cyl"
int Cyc_block_string(struct Cyc_Lexing_lexbuf*lexbuf){
# 886 "buildlib.cyl"
return Cyc_block_string_rec(lexbuf,17);}
# 229 "buildlib.cyl"
int Cyc_block_comment_rec(struct Cyc_Lexing_lexbuf*lexbuf,int lexstate){
# 888 "buildlib.cyl"
lexstate=Cyc_lex_engine(lexstate,lexbuf);
{int _tmpD3=lexstate;switch(_tmpD3){case 0U: _LL1: _LL2:
# 884 "buildlib.cyl"
({void*_tmpD4=0U;({struct _fat_ptr _tmp462=({const char*_tmpD5="Warning: unclosed comment\n";_tag_fat(_tmpD5,sizeof(char),27U);});Cyc_log(_tmp462,_tag_fat(_tmpD4,sizeof(void*),0U));});});return 0;case 1U: _LL3: _LL4:
# 886
({struct Cyc_Buffer_t*_tmp463=(struct Cyc_Buffer_t*)_check_null(Cyc_specbuf);Cyc_Buffer_add_string(_tmp463,({const char*_tmpD6="*/";_tag_fat(_tmpD6,sizeof(char),3U);}));});return 0;case 2U: _LL5: _LL6:
# 888
({struct Cyc_Buffer_t*_tmp464=(struct Cyc_Buffer_t*)_check_null(Cyc_specbuf);Cyc_Buffer_add_string(_tmp464,(struct _fat_ptr)Cyc_Lexing_lexeme(lexbuf));});
return 1;default: _LL7: _LL8:
(lexbuf->refill_buff)(lexbuf);
return Cyc_block_comment_rec(lexbuf,lexstate);}_LL0:;}
# 893
(int)_throw((void*)({struct Cyc_Lexing_Error_exn_struct*_tmpD8=_cycalloc(sizeof(*_tmpD8));_tmpD8->tag=Cyc_Lexing_Error,({struct _fat_ptr _tmp465=({const char*_tmpD7="some action didn't return!";_tag_fat(_tmpD7,sizeof(char),27U);});_tmpD8->f1=_tmp465;});_tmpD8;}));}
# 229 "buildlib.cyl"
int Cyc_block_comment(struct Cyc_Lexing_lexbuf*lexbuf){
# 895 "buildlib.cyl"
return Cyc_block_comment_rec(lexbuf,18);}
# 901 "buildlib.cyl"
void Cyc_scan_type(void*t,struct Cyc_Hashtable_Table*dep);struct _tuple30{struct Cyc_List_List*f1;struct Cyc_Absyn_Exp*f2;};
void Cyc_scan_exp(struct Cyc_Absyn_Exp*e,struct Cyc_Hashtable_Table*dep){
void*_stmttmp1=((struct Cyc_Absyn_Exp*)_check_null(e))->r;void*_tmpD9=_stmttmp1;struct Cyc_List_List*_tmpDA;struct Cyc_List_List*_tmpDC;void*_tmpDB;struct _fat_ptr*_tmpDE;struct Cyc_Absyn_Exp*_tmpDD;struct _fat_ptr*_tmpE0;struct Cyc_Absyn_Exp*_tmpDF;void*_tmpE1;void*_tmpE2;struct Cyc_Absyn_Exp*_tmpE3;struct Cyc_Absyn_Exp*_tmpE7;void**_tmpE6;struct Cyc_Absyn_Exp*_tmpE5;int _tmpE4;struct Cyc_Absyn_Exp*_tmpE9;void*_tmpE8;struct Cyc_List_List*_tmpEB;struct Cyc_Absyn_Exp*_tmpEA;struct Cyc_Absyn_Exp*_tmpED;struct Cyc_Absyn_Exp*_tmpEC;struct Cyc_Absyn_Exp*_tmpEF;struct Cyc_Absyn_Exp*_tmpEE;struct Cyc_Absyn_Exp*_tmpF2;struct Cyc_Absyn_Exp*_tmpF1;struct Cyc_Absyn_Exp*_tmpF0;struct Cyc_Absyn_Exp*_tmpF3;struct Cyc_Absyn_Exp*_tmpF4;struct Cyc_Absyn_Exp*_tmpF5;struct Cyc_Absyn_Exp*_tmpF6;struct Cyc_Absyn_Exp*_tmpF7;struct Cyc_Absyn_Exp*_tmpF9;struct Cyc_Absyn_Exp*_tmpF8;struct Cyc_Absyn_Exp*_tmpFB;struct Cyc_Absyn_Exp*_tmpFA;struct Cyc_Absyn_Exp*_tmpFD;struct Cyc_Absyn_Exp*_tmpFC;struct Cyc_List_List*_tmpFE;void*_tmpFF;switch(*((int*)_tmpD9)){case 1U: _LL1: _tmpFF=(void*)((struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct*)_tmpD9)->f1;_LL2: {void*b=_tmpFF;
# 905
struct _fat_ptr*v=(*Cyc_Absyn_binding2qvar(b)).f2;
Cyc_add_target(v);
return;}case 3U: _LL3: _tmpFE=((struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct*)_tmpD9)->f2;_LL4: {struct Cyc_List_List*x=_tmpFE;
# 909
for(0;x != 0;x=x->tl){
Cyc_scan_exp((struct Cyc_Absyn_Exp*)x->hd,dep);}
# 912
return;}case 23U: _LL5: _tmpFC=((struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct*)_tmpD9)->f1;_tmpFD=((struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct*)_tmpD9)->f2;_LL6: {struct Cyc_Absyn_Exp*e1=_tmpFC;struct Cyc_Absyn_Exp*e2=_tmpFD;
# 914
_tmpFA=e1;_tmpFB=e2;goto _LL8;}case 9U: _LL7: _tmpFA=((struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct*)_tmpD9)->f1;_tmpFB=((struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct*)_tmpD9)->f2;_LL8: {struct Cyc_Absyn_Exp*e1=_tmpFA;struct Cyc_Absyn_Exp*e2=_tmpFB;
# 916
_tmpF8=e1;_tmpF9=e2;goto _LLA;}case 4U: _LL9: _tmpF8=((struct Cyc_Absyn_AssignOp_e_Absyn_Raw_exp_struct*)_tmpD9)->f1;_tmpF9=((struct Cyc_Absyn_AssignOp_e_Absyn_Raw_exp_struct*)_tmpD9)->f3;_LLA: {struct Cyc_Absyn_Exp*e1=_tmpF8;struct Cyc_Absyn_Exp*e2=_tmpF9;
# 918
Cyc_scan_exp(e1,dep);
Cyc_scan_exp(e2,dep);
return;}case 42U: _LLB: _tmpF7=((struct Cyc_Absyn_Extension_e_Absyn_Raw_exp_struct*)_tmpD9)->f1;_LLC: {struct Cyc_Absyn_Exp*e1=_tmpF7;
_tmpF6=e1;goto _LLE;}case 20U: _LLD: _tmpF6=((struct Cyc_Absyn_Deref_e_Absyn_Raw_exp_struct*)_tmpD9)->f1;_LLE: {struct Cyc_Absyn_Exp*e1=_tmpF6;
# 923
_tmpF5=e1;goto _LL10;}case 18U: _LLF: _tmpF5=((struct Cyc_Absyn_Sizeofexp_e_Absyn_Raw_exp_struct*)_tmpD9)->f1;_LL10: {struct Cyc_Absyn_Exp*e1=_tmpF5;
# 925
_tmpF4=e1;goto _LL12;}case 15U: _LL11: _tmpF4=((struct Cyc_Absyn_Address_e_Absyn_Raw_exp_struct*)_tmpD9)->f1;_LL12: {struct Cyc_Absyn_Exp*e1=_tmpF4;
# 927
_tmpF3=e1;goto _LL14;}case 5U: _LL13: _tmpF3=((struct Cyc_Absyn_Increment_e_Absyn_Raw_exp_struct*)_tmpD9)->f1;_LL14: {struct Cyc_Absyn_Exp*e1=_tmpF3;
# 929
Cyc_scan_exp(e1,dep);
return;}case 6U: _LL15: _tmpF0=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmpD9)->f1;_tmpF1=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmpD9)->f2;_tmpF2=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmpD9)->f3;_LL16: {struct Cyc_Absyn_Exp*e1=_tmpF0;struct Cyc_Absyn_Exp*e2=_tmpF1;struct Cyc_Absyn_Exp*e3=_tmpF2;
# 932
Cyc_scan_exp(e1,dep);
Cyc_scan_exp(e2,dep);
Cyc_scan_exp(e3,dep);
return;}case 7U: _LL17: _tmpEE=((struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct*)_tmpD9)->f1;_tmpEF=((struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct*)_tmpD9)->f2;_LL18: {struct Cyc_Absyn_Exp*e1=_tmpEE;struct Cyc_Absyn_Exp*e2=_tmpEF;
_tmpEC=e1;_tmpED=e2;goto _LL1A;}case 8U: _LL19: _tmpEC=((struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct*)_tmpD9)->f1;_tmpED=((struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct*)_tmpD9)->f2;_LL1A: {struct Cyc_Absyn_Exp*e1=_tmpEC;struct Cyc_Absyn_Exp*e2=_tmpED;
# 938
Cyc_scan_exp(e1,dep);
Cyc_scan_exp(e2,dep);
return;}case 10U: _LL1B: _tmpEA=((struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct*)_tmpD9)->f1;_tmpEB=((struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct*)_tmpD9)->f2;_LL1C: {struct Cyc_Absyn_Exp*e1=_tmpEA;struct Cyc_List_List*x=_tmpEB;
# 942
Cyc_scan_exp(e1,dep);
for(0;x != 0;x=x->tl){
Cyc_scan_exp((struct Cyc_Absyn_Exp*)x->hd,dep);}
# 946
return;}case 14U: _LL1D: _tmpE8=(void*)((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmpD9)->f1;_tmpE9=((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmpD9)->f2;_LL1E: {void*t1=_tmpE8;struct Cyc_Absyn_Exp*e1=_tmpE9;
# 948
Cyc_scan_type(t1,dep);
Cyc_scan_exp(e1,dep);
return;}case 35U: _LL1F: _tmpE4=(((struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct*)_tmpD9)->f1).is_calloc;_tmpE5=(((struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct*)_tmpD9)->f1).rgn;_tmpE6=(((struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct*)_tmpD9)->f1).elt_type;_tmpE7=(((struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct*)_tmpD9)->f1).num_elts;_LL20: {int iscalloc=_tmpE4;struct Cyc_Absyn_Exp*ropt=_tmpE5;void**topt=_tmpE6;struct Cyc_Absyn_Exp*e=_tmpE7;
# 952
if(ropt != 0)Cyc_scan_exp(ropt,dep);if(topt != 0)
Cyc_scan_type(*topt,dep);
# 952
Cyc_scan_exp(e,dep);
# 955
return;}case 39U: _LL21: _tmpE3=((struct Cyc_Absyn_Tagcheck_e_Absyn_Raw_exp_struct*)_tmpD9)->f1;_LL22: {struct Cyc_Absyn_Exp*e=_tmpE3;
# 957
Cyc_scan_exp(e,dep);return;}case 40U: _LL23: _tmpE2=(void*)((struct Cyc_Absyn_Valueof_e_Absyn_Raw_exp_struct*)_tmpD9)->f1;_LL24: {void*t1=_tmpE2;
_tmpE1=t1;goto _LL26;}case 17U: _LL25: _tmpE1=(void*)((struct Cyc_Absyn_Sizeoftype_e_Absyn_Raw_exp_struct*)_tmpD9)->f1;_LL26: {void*t1=_tmpE1;
# 960
Cyc_scan_type(t1,dep);
return;}case 21U: _LL27: _tmpDF=((struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct*)_tmpD9)->f1;_tmpE0=((struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct*)_tmpD9)->f2;_LL28: {struct Cyc_Absyn_Exp*e1=_tmpDF;struct _fat_ptr*fn=_tmpE0;
# 963
_tmpDD=e1;_tmpDE=fn;goto _LL2A;}case 22U: _LL29: _tmpDD=((struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct*)_tmpD9)->f1;_tmpDE=((struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct*)_tmpD9)->f2;_LL2A: {struct Cyc_Absyn_Exp*e1=_tmpDD;struct _fat_ptr*fn=_tmpDE;
# 965
Cyc_scan_exp(e1,dep);
Cyc_add_target(fn);
return;}case 19U: _LL2B: _tmpDB=(void*)((struct Cyc_Absyn_Offsetof_e_Absyn_Raw_exp_struct*)_tmpD9)->f1;_tmpDC=((struct Cyc_Absyn_Offsetof_e_Absyn_Raw_exp_struct*)_tmpD9)->f2;_LL2C: {void*t1=_tmpDB;struct Cyc_List_List*f=_tmpDC;
# 969
Cyc_scan_type(t1,dep);
# 971
{void*_stmttmp2=(void*)((struct Cyc_List_List*)_check_null(f))->hd;void*_tmp100=_stmttmp2;struct _fat_ptr*_tmp101;if(((struct Cyc_Absyn_StructField_Absyn_OffsetofField_struct*)_tmp100)->tag == 0U){_LL58: _tmp101=((struct Cyc_Absyn_StructField_Absyn_OffsetofField_struct*)_tmp100)->f1;_LL59: {struct _fat_ptr*fn=_tmp101;
Cyc_add_target(fn);goto _LL57;}}else{_LL5A: _LL5B:
 goto _LL57;}_LL57:;}
# 975
return;}case 0U: _LL2D: _LL2E:
# 977
 return;case 37U: _LL2F: _tmpDA=((struct Cyc_Absyn_UnresolvedMem_e_Absyn_Raw_exp_struct*)_tmpD9)->f2;_LL30: {struct Cyc_List_List*x=_tmpDA;
# 979
for(0;x != 0;x=x->tl){
struct _tuple30*_stmttmp3=(struct _tuple30*)x->hd;struct Cyc_Absyn_Exp*_tmp102;_LL5D: _tmp102=_stmttmp3->f2;_LL5E: {struct Cyc_Absyn_Exp*e1=_tmp102;
Cyc_scan_exp(e1,dep);}}
# 983
return;}case 41U: _LL31: _LL32:
 return;case 2U: _LL33: _LL34:
# 986
({void*_tmp103=0U;({struct Cyc___cycFILE*_tmp467=Cyc_stderr;struct _fat_ptr _tmp466=({const char*_tmp104="Error: unexpected Pragma_e\n";_tag_fat(_tmp104,sizeof(char),28U);});Cyc_fprintf(_tmp467,_tmp466,_tag_fat(_tmp103,sizeof(void*),0U));});});
exit(1);return;case 36U: _LL35: _LL36:
# 989
({void*_tmp105=0U;({struct Cyc___cycFILE*_tmp469=Cyc_stderr;struct _fat_ptr _tmp468=({const char*_tmp106="Error: unexpected Swap_e\n";_tag_fat(_tmp106,sizeof(char),26U);});Cyc_fprintf(_tmp469,_tmp468,_tag_fat(_tmp105,sizeof(void*),0U));});});
exit(1);return;case 38U: _LL37: _LL38:
# 992
({void*_tmp107=0U;({struct Cyc___cycFILE*_tmp46B=Cyc_stderr;struct _fat_ptr _tmp46A=({const char*_tmp108="Error: unexpected Stmt_e\n";_tag_fat(_tmp108,sizeof(char),26U);});Cyc_fprintf(_tmp46B,_tmp46A,_tag_fat(_tmp107,sizeof(void*),0U));});});
exit(1);return;case 11U: _LL39: _LL3A:
# 995
({void*_tmp109=0U;({struct Cyc___cycFILE*_tmp46D=Cyc_stderr;struct _fat_ptr _tmp46C=({const char*_tmp10A="Error: unexpected Throw_e\n";_tag_fat(_tmp10A,sizeof(char),27U);});Cyc_fprintf(_tmp46D,_tmp46C,_tag_fat(_tmp109,sizeof(void*),0U));});});
exit(1);return;case 12U: _LL3B: _LL3C:
# 998
({void*_tmp10B=0U;({struct Cyc___cycFILE*_tmp46F=Cyc_stderr;struct _fat_ptr _tmp46E=({const char*_tmp10C="Error: unexpected NoInstantiate_e\n";_tag_fat(_tmp10C,sizeof(char),35U);});Cyc_fprintf(_tmp46F,_tmp46E,_tag_fat(_tmp10B,sizeof(void*),0U));});});
exit(1);return;case 13U: _LL3D: _LL3E:
# 1001
({void*_tmp10D=0U;({struct Cyc___cycFILE*_tmp471=Cyc_stderr;struct _fat_ptr _tmp470=({const char*_tmp10E="Error: unexpected Instantiate_e\n";_tag_fat(_tmp10E,sizeof(char),33U);});Cyc_fprintf(_tmp471,_tmp470,_tag_fat(_tmp10D,sizeof(void*),0U));});});
exit(1);return;case 16U: _LL3F: _LL40:
# 1004
({void*_tmp10F=0U;({struct Cyc___cycFILE*_tmp473=Cyc_stderr;struct _fat_ptr _tmp472=({const char*_tmp110="Error: unexpected New_e\n";_tag_fat(_tmp110,sizeof(char),25U);});Cyc_fprintf(_tmp473,_tmp472,_tag_fat(_tmp10F,sizeof(void*),0U));});});
exit(1);return;case 24U: _LL41: _LL42:
# 1007
({void*_tmp111=0U;({struct Cyc___cycFILE*_tmp475=Cyc_stderr;struct _fat_ptr _tmp474=({const char*_tmp112="Error: unexpected Tuple_e\n";_tag_fat(_tmp112,sizeof(char),27U);});Cyc_fprintf(_tmp475,_tmp474,_tag_fat(_tmp111,sizeof(void*),0U));});});
exit(1);return;case 34U: _LL43: _LL44:
# 1010
({void*_tmp113=0U;({struct Cyc___cycFILE*_tmp477=Cyc_stderr;struct _fat_ptr _tmp476=({const char*_tmp114="Error: unexpected Spawn_e\n";_tag_fat(_tmp114,sizeof(char),27U);});Cyc_fprintf(_tmp477,_tmp476,_tag_fat(_tmp113,sizeof(void*),0U));});});
exit(1);return;case 25U: _LL45: _LL46:
# 1013
({void*_tmp115=0U;({struct Cyc___cycFILE*_tmp479=Cyc_stderr;struct _fat_ptr _tmp478=({const char*_tmp116="Error: unexpected CompoundLit_e\n";_tag_fat(_tmp116,sizeof(char),33U);});Cyc_fprintf(_tmp479,_tmp478,_tag_fat(_tmp115,sizeof(void*),0U));});});
exit(1);return;case 26U: _LL47: _LL48:
# 1016
({void*_tmp117=0U;({struct Cyc___cycFILE*_tmp47B=Cyc_stderr;struct _fat_ptr _tmp47A=({const char*_tmp118="Error: unexpected Array_e\n";_tag_fat(_tmp118,sizeof(char),27U);});Cyc_fprintf(_tmp47B,_tmp47A,_tag_fat(_tmp117,sizeof(void*),0U));});});
exit(1);return;case 27U: _LL49: _LL4A:
# 1019
({void*_tmp119=0U;({struct Cyc___cycFILE*_tmp47D=Cyc_stderr;struct _fat_ptr _tmp47C=({const char*_tmp11A="Error: unexpected Comprehension_e\n";_tag_fat(_tmp11A,sizeof(char),35U);});Cyc_fprintf(_tmp47D,_tmp47C,_tag_fat(_tmp119,sizeof(void*),0U));});});
exit(1);return;case 28U: _LL4B: _LL4C:
# 1022
({void*_tmp11B=0U;({struct Cyc___cycFILE*_tmp47F=Cyc_stderr;struct _fat_ptr _tmp47E=({const char*_tmp11C="Error: unexpected ComprehensionNoinit_e\n";_tag_fat(_tmp11C,sizeof(char),41U);});Cyc_fprintf(_tmp47F,_tmp47E,_tag_fat(_tmp11B,sizeof(void*),0U));});});
exit(1);return;case 29U: _LL4D: _LL4E:
# 1025
({void*_tmp11D=0U;({struct Cyc___cycFILE*_tmp481=Cyc_stderr;struct _fat_ptr _tmp480=({const char*_tmp11E="Error: unexpected Aggregate_e\n";_tag_fat(_tmp11E,sizeof(char),31U);});Cyc_fprintf(_tmp481,_tmp480,_tag_fat(_tmp11D,sizeof(void*),0U));});});
exit(1);return;case 30U: _LL4F: _LL50:
# 1028
({void*_tmp11F=0U;({struct Cyc___cycFILE*_tmp483=Cyc_stderr;struct _fat_ptr _tmp482=({const char*_tmp120="Error: unexpected AnonStruct_e\n";_tag_fat(_tmp120,sizeof(char),32U);});Cyc_fprintf(_tmp483,_tmp482,_tag_fat(_tmp11F,sizeof(void*),0U));});});
exit(1);return;case 31U: _LL51: _LL52:
# 1031
({void*_tmp121=0U;({struct Cyc___cycFILE*_tmp485=Cyc_stderr;struct _fat_ptr _tmp484=({const char*_tmp122="Error: unexpected Datatype_e\n";_tag_fat(_tmp122,sizeof(char),30U);});Cyc_fprintf(_tmp485,_tmp484,_tag_fat(_tmp121,sizeof(void*),0U));});});
exit(1);return;case 32U: _LL53: _LL54:
# 1034
({void*_tmp123=0U;({struct Cyc___cycFILE*_tmp487=Cyc_stderr;struct _fat_ptr _tmp486=({const char*_tmp124="Error: unexpected Enum_e\n";_tag_fat(_tmp124,sizeof(char),26U);});Cyc_fprintf(_tmp487,_tmp486,_tag_fat(_tmp123,sizeof(void*),0U));});});
exit(1);return;default: _LL55: _LL56:
# 1037
({void*_tmp125=0U;({struct Cyc___cycFILE*_tmp489=Cyc_stderr;struct _fat_ptr _tmp488=({const char*_tmp126="Error: unexpected AnonEnum_e\n";_tag_fat(_tmp126,sizeof(char),30U);});Cyc_fprintf(_tmp489,_tmp488,_tag_fat(_tmp125,sizeof(void*),0U));});});
exit(1);return;}_LL0:;}
# 902
void Cyc_scan_exp_opt(struct Cyc_Absyn_Exp*eo,struct Cyc_Hashtable_Table*dep){
# 1043
if((unsigned)eo)Cyc_scan_exp(eo,dep);return;}
# 902
void Cyc_scan_decl(struct Cyc_Absyn_Decl*d,struct Cyc_Hashtable_Table*dep);
# 1048
void Cyc_scan_type(void*t,struct Cyc_Hashtable_Table*dep){
void*_tmp127=t;struct Cyc_Absyn_Datatypedecl*_tmp128;struct Cyc_Absyn_Enumdecl*_tmp129;struct Cyc_Absyn_Aggrdecl*_tmp12A;struct _fat_ptr*_tmp12B;struct Cyc_List_List*_tmp12C;struct Cyc_Absyn_FnInfo _tmp12D;struct Cyc_Absyn_Exp*_tmp12E;void*_tmp131;struct Cyc_Absyn_Exp*_tmp130;void*_tmp12F;struct Cyc_Absyn_PtrInfo _tmp132;struct Cyc_List_List*_tmp134;void*_tmp133;switch(*((int*)_tmp127)){case 0U: _LL1: _tmp133=(void*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp127)->f1;_tmp134=((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp127)->f2;_LL2: {void*c=_tmp133;struct Cyc_List_List*ts=_tmp134;
# 1051
void*_tmp135=c;struct _fat_ptr*_tmp136;union Cyc_Absyn_AggrInfo _tmp137;switch(*((int*)_tmp135)){case 0U: _LL1E: _LL1F:
 goto _LL21;case 1U: _LL20: _LL21:
 goto _LL23;case 17U: _LL22: _LL23:
 goto _LL25;case 2U: _LL24: _LL25:
 goto _LL27;case 16U: _LL26: _LL27:
# 1057
 return;case 20U: _LL28: _tmp137=((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)_tmp135)->f1;_LL29: {union Cyc_Absyn_AggrInfo info=_tmp137;
# 1059
struct _tuple12 _stmttmp4=Cyc_Absyn_aggr_kinded_name(info);struct _fat_ptr*_tmp138;_LL49: _tmp138=(_stmttmp4.f2)->f2;_LL4A: {struct _fat_ptr*v=_tmp138;
_tmp136=v;goto _LL2B;}}case 15U: _LL2A: _tmp136=(((struct Cyc_Absyn_EnumCon_Absyn_TyCon_struct*)_tmp135)->f1)->f2;_LL2B: {struct _fat_ptr*v=_tmp136;
Cyc_add_target(v);return;}case 18U: _LL2C: _LL2D:
 goto _LL2F;case 19U: _LL2E: _LL2F: goto _LL31;case 3U: _LL30: _LL31:
 goto _LL33;case 5U: _LL32: _LL33: goto _LL35;case 6U: _LL34: _LL35:
 goto _LL37;case 7U: _LL36: _LL37: goto _LL39;case 8U: _LL38: _LL39:
 goto _LL3B;case 9U: _LL3A: _LL3B: goto _LL3D;case 10U: _LL3C: _LL3D:
 goto _LL3F;case 4U: _LL3E: _LL3F: goto _LL41;case 11U: _LL40: _LL41:
 goto _LL43;case 12U: _LL42: _LL43: goto _LL45;case 13U: _LL44: _LL45:
 goto _LL47;default: _LL46: _LL47:
({struct Cyc_String_pa_PrintArg_struct _tmp13B=({struct Cyc_String_pa_PrintArg_struct _tmp374;_tmp374.tag=0U,({struct _fat_ptr _tmp48A=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t));_tmp374.f1=_tmp48A;});_tmp374;});void*_tmp139[1U];_tmp139[0]=& _tmp13B;({struct Cyc___cycFILE*_tmp48C=Cyc_stderr;struct _fat_ptr _tmp48B=({const char*_tmp13A="Error: unexpected type %s\n";_tag_fat(_tmp13A,sizeof(char),27U);});Cyc_fprintf(_tmp48C,_tmp48B,_tag_fat(_tmp139,sizeof(void*),1U));});});
exit(1);return;}_LL1D:;}case 3U: _LL3: _tmp132=((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp127)->f1;_LL4: {struct Cyc_Absyn_PtrInfo x=_tmp132;
# 1074
Cyc_scan_type(x.elt_type,dep);
return;}case 4U: _LL5: _tmp12F=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp127)->f1).elt_type;_tmp130=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp127)->f1).num_elts;_tmp131=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp127)->f1).zero_term;_LL6: {void*t=_tmp12F;struct Cyc_Absyn_Exp*sz=_tmp130;void*zt=_tmp131;
# 1077
Cyc_scan_type(t,dep);
Cyc_scan_exp_opt(sz,dep);
return;}case 11U: _LL7: _tmp12E=((struct Cyc_Absyn_TypeofType_Absyn_Type_struct*)_tmp127)->f1;_LL8: {struct Cyc_Absyn_Exp*e=_tmp12E;
# 1081
Cyc_scan_exp(e,dep);
return;}case 5U: _LL9: _tmp12D=((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp127)->f1;_LLA: {struct Cyc_Absyn_FnInfo x=_tmp12D;
# 1084
Cyc_scan_type(x.ret_type,dep);
{struct Cyc_List_List*a=x.args;for(0;a != 0;a=a->tl){
struct _tuple9*_stmttmp5=(struct _tuple9*)a->hd;void*_tmp13C;_LL4C: _tmp13C=_stmttmp5->f3;_LL4D: {void*argt=_tmp13C;
Cyc_scan_type(argt,dep);}}}
# 1089
if(x.cyc_varargs != 0)
Cyc_scan_type((x.cyc_varargs)->type,dep);
# 1089
return;}case 7U: _LLB: _tmp12C=((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmp127)->f2;_LLC: {struct Cyc_List_List*sfs=_tmp12C;
# 1093
for(0;sfs != 0;sfs=sfs->tl){
Cyc_scan_type(((struct Cyc_Absyn_Aggrfield*)sfs->hd)->type,dep);
Cyc_scan_exp_opt(((struct Cyc_Absyn_Aggrfield*)sfs->hd)->width,dep);}
# 1097
return;}case 8U: _LLD: _tmp12B=(((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmp127)->f1)->f2;_LLE: {struct _fat_ptr*v=_tmp12B;
# 1099
Cyc_add_target(v);
return;}case 10U: switch(*((int*)((struct Cyc_Absyn_TypeDecl*)((struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct*)_tmp127)->f1)->r)){case 0U: _LLF: _tmp12A=((struct Cyc_Absyn_Aggr_td_Absyn_Raw_typedecl_struct*)(((struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct*)_tmp127)->f1)->r)->f1;_LL10: {struct Cyc_Absyn_Aggrdecl*x=_tmp12A;
# 1103
({struct Cyc_Absyn_Decl*_tmp48D=Cyc_Absyn_new_decl((void*)({struct Cyc_Absyn_Aggr_d_Absyn_Raw_decl_struct*_tmp13D=_cycalloc(sizeof(*_tmp13D));_tmp13D->tag=5U,_tmp13D->f1=x;_tmp13D;}),0U);Cyc_scan_decl(_tmp48D,dep);});{
struct _tuple1*_stmttmp6=x->name;struct _fat_ptr*_tmp13E;_LL4F: _tmp13E=_stmttmp6->f2;_LL50: {struct _fat_ptr*n=_tmp13E;
Cyc_add_target(n);
return;}}}case 1U: _LL11: _tmp129=((struct Cyc_Absyn_Enum_td_Absyn_Raw_typedecl_struct*)(((struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct*)_tmp127)->f1)->r)->f1;_LL12: {struct Cyc_Absyn_Enumdecl*x=_tmp129;
# 1109
({struct Cyc_Absyn_Decl*_tmp48E=Cyc_Absyn_new_decl((void*)({struct Cyc_Absyn_Enum_d_Absyn_Raw_decl_struct*_tmp13F=_cycalloc(sizeof(*_tmp13F));_tmp13F->tag=7U,_tmp13F->f1=x;_tmp13F;}),0U);Cyc_scan_decl(_tmp48E,dep);});{
struct _tuple1*_stmttmp7=x->name;struct _fat_ptr*_tmp140;_LL52: _tmp140=_stmttmp7->f2;_LL53: {struct _fat_ptr*n=_tmp140;
Cyc_add_target(n);
return;}}}default: _LL13: _tmp128=((struct Cyc_Absyn_Datatype_td_Absyn_Raw_typedecl_struct*)(((struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct*)_tmp127)->f1)->r)->f1;_LL14: {struct Cyc_Absyn_Datatypedecl*dd=_tmp128;
# 1115
({void*_tmp141=0U;({struct Cyc___cycFILE*_tmp490=Cyc_stderr;struct _fat_ptr _tmp48F=({const char*_tmp142="Error: unexpected Datatype declaration\n";_tag_fat(_tmp142,sizeof(char),40U);});Cyc_fprintf(_tmp490,_tmp48F,_tag_fat(_tmp141,sizeof(void*),0U));});});
exit(1);return;}}case 1U: _LL15: _LL16:
# 1118
({void*_tmp143=0U;({struct Cyc___cycFILE*_tmp492=Cyc_stderr;struct _fat_ptr _tmp491=({const char*_tmp144="Error: unexpected Evar\n";_tag_fat(_tmp144,sizeof(char),24U);});Cyc_fprintf(_tmp492,_tmp491,_tag_fat(_tmp143,sizeof(void*),0U));});});
exit(1);return;case 2U: _LL17: _LL18:
# 1121
({void*_tmp145=0U;({struct Cyc___cycFILE*_tmp494=Cyc_stderr;struct _fat_ptr _tmp493=({const char*_tmp146="Error: unexpected VarType\n";_tag_fat(_tmp146,sizeof(char),27U);});Cyc_fprintf(_tmp494,_tmp493,_tag_fat(_tmp145,sizeof(void*),0U));});});
exit(1);return;case 6U: _LL19: _LL1A:
# 1124
({void*_tmp147=0U;({struct Cyc___cycFILE*_tmp496=Cyc_stderr;struct _fat_ptr _tmp495=({const char*_tmp148="Error: unexpected TupleType\n";_tag_fat(_tmp148,sizeof(char),29U);});Cyc_fprintf(_tmp496,_tmp495,_tag_fat(_tmp147,sizeof(void*),0U));});});
exit(1);return;default: _LL1B: _LL1C:
# 1127
({void*_tmp149=0U;({struct Cyc___cycFILE*_tmp498=Cyc_stderr;struct _fat_ptr _tmp497=({const char*_tmp14A="Error: unexpected valueof_t\n";_tag_fat(_tmp14A,sizeof(char),29U);});Cyc_fprintf(_tmp498,_tmp497,_tag_fat(_tmp149,sizeof(void*),0U));});});
exit(1);return;}_LL0:;}
# 1048
void Cyc_scan_decl(struct Cyc_Absyn_Decl*d,struct Cyc_Hashtable_Table*dep){
# 1133
struct Cyc_Set_Set**saved_targets=Cyc_current_targets;
struct _fat_ptr*saved_source=Cyc_current_source;
Cyc_current_targets=({struct Cyc_Set_Set**_tmp14C=_cycalloc(sizeof(*_tmp14C));({struct Cyc_Set_Set*_tmp499=({struct Cyc_Set_Set*(*_tmp14B)(int(*cmp)(struct _fat_ptr*,struct _fat_ptr*))=(struct Cyc_Set_Set*(*)(int(*cmp)(struct _fat_ptr*,struct _fat_ptr*)))Cyc_Set_empty;_tmp14B;})(Cyc_strptrcmp);*_tmp14C=_tmp499;});_tmp14C;});
{void*_stmttmp8=d->r;void*_tmp14D=_stmttmp8;struct Cyc_Absyn_Typedefdecl*_tmp14E;struct Cyc_Absyn_Enumdecl*_tmp14F;struct Cyc_Absyn_Aggrdecl*_tmp150;struct Cyc_Absyn_Fndecl*_tmp151;struct Cyc_Absyn_Vardecl*_tmp152;switch(*((int*)_tmp14D)){case 0U: _LL1: _tmp152=((struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct*)_tmp14D)->f1;_LL2: {struct Cyc_Absyn_Vardecl*x=_tmp152;
# 1138
struct _tuple1*_stmttmp9=x->name;struct _fat_ptr*_tmp153;_LL24: _tmp153=_stmttmp9->f2;_LL25: {struct _fat_ptr*v=_tmp153;
Cyc_current_source=v;
Cyc_scan_type(x->type,dep);
Cyc_scan_exp_opt(x->initializer,dep);
goto _LL0;}}case 1U: _LL3: _tmp151=((struct Cyc_Absyn_Fn_d_Absyn_Raw_decl_struct*)_tmp14D)->f1;_LL4: {struct Cyc_Absyn_Fndecl*x=_tmp151;
# 1144
struct _tuple1*_stmttmpA=x->name;struct _fat_ptr*_tmp154;_LL27: _tmp154=_stmttmpA->f2;_LL28: {struct _fat_ptr*v=_tmp154;
Cyc_current_source=v;
Cyc_scan_type((x->i).ret_type,dep);
{struct Cyc_List_List*a=(x->i).args;for(0;a != 0;a=a->tl){
struct _tuple9*_stmttmpB=(struct _tuple9*)a->hd;void*_tmp155;_LL2A: _tmp155=_stmttmpB->f3;_LL2B: {void*t1=_tmp155;
Cyc_scan_type(t1,dep);}}}
# 1151
if((x->i).cyc_varargs != 0)
Cyc_scan_type(((struct Cyc_Absyn_VarargInfo*)_check_null((x->i).cyc_varargs))->type,dep);
# 1151
if(x->is_inline)
# 1154
({void*_tmp156=0U;({struct Cyc___cycFILE*_tmp49B=Cyc_stderr;struct _fat_ptr _tmp49A=({const char*_tmp157="Warning: ignoring inline function\n";_tag_fat(_tmp157,sizeof(char),35U);});Cyc_fprintf(_tmp49B,_tmp49A,_tag_fat(_tmp156,sizeof(void*),0U));});});
# 1151
goto _LL0;}}case 5U: _LL5: _tmp150=((struct Cyc_Absyn_Aggr_d_Absyn_Raw_decl_struct*)_tmp14D)->f1;_LL6: {struct Cyc_Absyn_Aggrdecl*x=_tmp150;
# 1157
struct _tuple1*_stmttmpC=x->name;struct _fat_ptr*_tmp158;_LL2D: _tmp158=_stmttmpC->f2;_LL2E: {struct _fat_ptr*v=_tmp158;
Cyc_current_source=v;
if((unsigned)x->impl){
{struct Cyc_List_List*fs=((struct Cyc_Absyn_AggrdeclImpl*)_check_null(x->impl))->fields;for(0;fs != 0;fs=fs->tl){
struct Cyc_Absyn_Aggrfield*f=(struct Cyc_Absyn_Aggrfield*)fs->hd;
Cyc_scan_type(f->type,dep);
Cyc_scan_exp_opt(f->width,dep);}}{
# 1167
struct Cyc_List_List*fs=((struct Cyc_Absyn_AggrdeclImpl*)_check_null(x->impl))->fields;for(0;fs != 0;fs=fs->tl){;}}}
# 1159
goto _LL0;}}case 7U: _LL7: _tmp14F=((struct Cyc_Absyn_Enum_d_Absyn_Raw_decl_struct*)_tmp14D)->f1;_LL8: {struct Cyc_Absyn_Enumdecl*x=_tmp14F;
# 1173
struct _tuple1*_stmttmpD=x->name;struct _fat_ptr*_tmp159;_LL30: _tmp159=_stmttmpD->f2;_LL31: {struct _fat_ptr*v=_tmp159;
Cyc_current_source=v;
if((unsigned)x->fields){
{struct Cyc_List_List*fs=(struct Cyc_List_List*)((struct Cyc_Core_Opt*)_check_null(x->fields))->v;for(0;fs != 0;fs=fs->tl){
struct Cyc_Absyn_Enumfield*f=(struct Cyc_Absyn_Enumfield*)fs->hd;
Cyc_scan_exp_opt(f->tag,dep);}}{
# 1182
struct Cyc_List_List*fs=(struct Cyc_List_List*)((struct Cyc_Core_Opt*)_check_null(x->fields))->v;for(0;fs != 0;fs=fs->tl){;}}}
# 1175
goto _LL0;}}case 8U: _LL9: _tmp14E=((struct Cyc_Absyn_Typedef_d_Absyn_Raw_decl_struct*)_tmp14D)->f1;_LLA: {struct Cyc_Absyn_Typedefdecl*x=_tmp14E;
# 1188
struct _tuple1*_stmttmpE=x->name;struct _fat_ptr*_tmp15A;_LL33: _tmp15A=_stmttmpE->f2;_LL34: {struct _fat_ptr*v=_tmp15A;
Cyc_current_source=v;
if((unsigned)x->defn)
Cyc_scan_type((void*)_check_null(x->defn),dep);
# 1190
goto _LL0;}}case 4U: _LLB: _LLC:
# 1194
({void*_tmp15B=0U;({struct Cyc___cycFILE*_tmp49D=Cyc_stderr;struct _fat_ptr _tmp49C=({const char*_tmp15C="Error: unexpected region declaration";_tag_fat(_tmp15C,sizeof(char),37U);});Cyc_fprintf(_tmp49D,_tmp49C,_tag_fat(_tmp15B,sizeof(void*),0U));});});
exit(1);case 13U: _LLD: _LLE:
# 1197
({void*_tmp15D=0U;({struct Cyc___cycFILE*_tmp49F=Cyc_stderr;struct _fat_ptr _tmp49E=({const char*_tmp15E="Error: unexpected __cyclone_port_on__";_tag_fat(_tmp15E,sizeof(char),38U);});Cyc_fprintf(_tmp49F,_tmp49E,_tag_fat(_tmp15D,sizeof(void*),0U));});});
exit(1);case 14U: _LLF: _LL10:
# 1200
({void*_tmp15F=0U;({struct Cyc___cycFILE*_tmp4A1=Cyc_stderr;struct _fat_ptr _tmp4A0=({const char*_tmp160="Error: unexpected __cyclone_port_off__";_tag_fat(_tmp160,sizeof(char),39U);});Cyc_fprintf(_tmp4A1,_tmp4A0,_tag_fat(_tmp15F,sizeof(void*),0U));});});
exit(1);case 15U: _LL11: _LL12:
# 1203
({void*_tmp161=0U;({struct Cyc___cycFILE*_tmp4A3=Cyc_stderr;struct _fat_ptr _tmp4A2=({const char*_tmp162="Error: unexpected __tempest_on__";_tag_fat(_tmp162,sizeof(char),33U);});Cyc_fprintf(_tmp4A3,_tmp4A2,_tag_fat(_tmp161,sizeof(void*),0U));});});
exit(1);case 16U: _LL13: _LL14:
# 1206
({void*_tmp163=0U;({struct Cyc___cycFILE*_tmp4A5=Cyc_stderr;struct _fat_ptr _tmp4A4=({const char*_tmp164="Error: unexpected __tempest_off__";_tag_fat(_tmp164,sizeof(char),34U);});Cyc_fprintf(_tmp4A5,_tmp4A4,_tag_fat(_tmp163,sizeof(void*),0U));});});
exit(1);case 2U: _LL15: _LL16:
# 1209
({void*_tmp165=0U;({struct Cyc___cycFILE*_tmp4A7=Cyc_stderr;struct _fat_ptr _tmp4A6=({const char*_tmp166="Error: unexpected let declaration\n";_tag_fat(_tmp166,sizeof(char),35U);});Cyc_fprintf(_tmp4A7,_tmp4A6,_tag_fat(_tmp165,sizeof(void*),0U));});});
exit(1);case 6U: _LL17: _LL18:
# 1212
({void*_tmp167=0U;({struct Cyc___cycFILE*_tmp4A9=Cyc_stderr;struct _fat_ptr _tmp4A8=({const char*_tmp168="Error: unexpected datatype declaration\n";_tag_fat(_tmp168,sizeof(char),40U);});Cyc_fprintf(_tmp4A9,_tmp4A8,_tag_fat(_tmp167,sizeof(void*),0U));});});
exit(1);case 3U: _LL19: _LL1A:
# 1215
({void*_tmp169=0U;({struct Cyc___cycFILE*_tmp4AB=Cyc_stderr;struct _fat_ptr _tmp4AA=({const char*_tmp16A="Error: unexpected let declaration\n";_tag_fat(_tmp16A,sizeof(char),35U);});Cyc_fprintf(_tmp4AB,_tmp4AA,_tag_fat(_tmp169,sizeof(void*),0U));});});
exit(1);case 9U: _LL1B: _LL1C:
# 1218
({void*_tmp16B=0U;({struct Cyc___cycFILE*_tmp4AD=Cyc_stderr;struct _fat_ptr _tmp4AC=({const char*_tmp16C="Error: unexpected namespace declaration\n";_tag_fat(_tmp16C,sizeof(char),41U);});Cyc_fprintf(_tmp4AD,_tmp4AC,_tag_fat(_tmp16B,sizeof(void*),0U));});});
exit(1);case 10U: _LL1D: _LL1E:
# 1221
({void*_tmp16D=0U;({struct Cyc___cycFILE*_tmp4AF=Cyc_stderr;struct _fat_ptr _tmp4AE=({const char*_tmp16E="Error: unexpected using declaration\n";_tag_fat(_tmp16E,sizeof(char),37U);});Cyc_fprintf(_tmp4AF,_tmp4AE,_tag_fat(_tmp16D,sizeof(void*),0U));});});
exit(1);case 11U: _LL1F: _LL20:
# 1224
({void*_tmp16F=0U;({struct Cyc___cycFILE*_tmp4B1=Cyc_stderr;struct _fat_ptr _tmp4B0=({const char*_tmp170="Error: unexpected extern \"C\" declaration\n";_tag_fat(_tmp170,sizeof(char),42U);});Cyc_fprintf(_tmp4B1,_tmp4B0,_tag_fat(_tmp16F,sizeof(void*),0U));});});
exit(1);default: _LL21: _LL22:
# 1227
({void*_tmp171=0U;({struct Cyc___cycFILE*_tmp4B3=Cyc_stderr;struct _fat_ptr _tmp4B2=({const char*_tmp172="Error: unexpected extern \"C include\" declaration\n";_tag_fat(_tmp172,sizeof(char),50U);});Cyc_fprintf(_tmp4B3,_tmp4B2,_tag_fat(_tmp171,sizeof(void*),0U));});});
exit(1);}_LL0:;}{
# 1235
struct Cyc_Set_Set*old;
struct _fat_ptr*name=(struct _fat_ptr*)_check_null(Cyc_current_source);
{struct _handler_cons _tmp173;_push_handler(& _tmp173);{int _tmp175=0;if(setjmp(_tmp173.handler))_tmp175=1;if(!_tmp175){
old=({struct Cyc_Set_Set*(*_tmp4B5)(struct Cyc_Hashtable_Table*t,struct _fat_ptr*key)=({struct Cyc_Set_Set*(*_tmp176)(struct Cyc_Hashtable_Table*t,struct _fat_ptr*key)=(struct Cyc_Set_Set*(*)(struct Cyc_Hashtable_Table*t,struct _fat_ptr*key))Cyc_Hashtable_lookup;_tmp176;});struct Cyc_Hashtable_Table*_tmp4B4=dep;_tmp4B5(_tmp4B4,name);});;_pop_handler();}else{void*_tmp174=(void*)Cyc_Core_get_exn_thrown();void*_tmp177=_tmp174;void*_tmp178;if(((struct Cyc_Core_Not_found_exn_struct*)_tmp177)->tag == Cyc_Core_Not_found){_LL36: _LL37:
# 1240
 old=({struct Cyc_Set_Set*(*_tmp179)(int(*cmp)(struct _fat_ptr*,struct _fat_ptr*))=(struct Cyc_Set_Set*(*)(int(*cmp)(struct _fat_ptr*,struct _fat_ptr*)))Cyc_Set_empty;_tmp179;})(Cyc_strptrcmp);goto _LL35;}else{_LL38: _tmp178=_tmp177;_LL39: {void*exn=_tmp178;(int)_rethrow(exn);}}_LL35:;}}}{
# 1242
struct Cyc_Set_Set*targets=Cyc_Set_union_two(*((struct Cyc_Set_Set**)_check_null(Cyc_current_targets)),old);
({void(*_tmp4B8)(struct Cyc_Hashtable_Table*t,struct _fat_ptr*key,struct Cyc_Set_Set*val)=({void(*_tmp17A)(struct Cyc_Hashtable_Table*t,struct _fat_ptr*key,struct Cyc_Set_Set*val)=(void(*)(struct Cyc_Hashtable_Table*t,struct _fat_ptr*key,struct Cyc_Set_Set*val))Cyc_Hashtable_insert;_tmp17A;});struct Cyc_Hashtable_Table*_tmp4B7=dep;struct _fat_ptr*_tmp4B6=name;_tmp4B8(_tmp4B7,_tmp4B6,targets);});
# 1245
Cyc_current_targets=saved_targets;
Cyc_current_source=saved_source;}}}
# 1048
struct Cyc_Hashtable_Table*Cyc_new_deps(){
# 1250
return({struct Cyc_Hashtable_Table*(*_tmp17B)(int sz,int(*cmp)(struct _fat_ptr*,struct _fat_ptr*),int(*hash)(struct _fat_ptr*))=(struct Cyc_Hashtable_Table*(*)(int sz,int(*cmp)(struct _fat_ptr*,struct _fat_ptr*),int(*hash)(struct _fat_ptr*)))Cyc_Hashtable_create;_tmp17B;})(107,Cyc_strptrcmp,Cyc_Hashtable_hash_stringptr);}
# 1048
struct Cyc_Set_Set*Cyc_find(struct Cyc_Hashtable_Table*t,struct _fat_ptr*x){
# 1254
struct _handler_cons _tmp17C;_push_handler(& _tmp17C);{int _tmp17E=0;if(setjmp(_tmp17C.handler))_tmp17E=1;if(!_tmp17E){{struct Cyc_Set_Set*_tmp180=({struct Cyc_Set_Set*(*_tmp4BA)(struct Cyc_Hashtable_Table*t,struct _fat_ptr*key)=({struct Cyc_Set_Set*(*_tmp17F)(struct Cyc_Hashtable_Table*t,struct _fat_ptr*key)=(struct Cyc_Set_Set*(*)(struct Cyc_Hashtable_Table*t,struct _fat_ptr*key))Cyc_Hashtable_lookup;_tmp17F;});struct Cyc_Hashtable_Table*_tmp4B9=t;_tmp4BA(_tmp4B9,x);});_npop_handler(0U);return _tmp180;};_pop_handler();}else{void*_tmp17D=(void*)Cyc_Core_get_exn_thrown();void*_tmp181=_tmp17D;void*_tmp182;if(((struct Cyc_Core_Not_found_exn_struct*)_tmp181)->tag == Cyc_Core_Not_found){_LL1: _LL2:
# 1256
 return({struct Cyc_Set_Set*(*_tmp183)(int(*cmp)(struct _fat_ptr*,struct _fat_ptr*))=(struct Cyc_Set_Set*(*)(int(*cmp)(struct _fat_ptr*,struct _fat_ptr*)))Cyc_Set_empty;_tmp183;})(Cyc_strptrcmp);}else{_LL3: _tmp182=_tmp181;_LL4: {void*exn=_tmp182;(int)_rethrow(exn);}}_LL0:;}}}
# 1048
struct Cyc_Set_Set*Cyc_reachable(struct Cyc_List_List*init,struct Cyc_Hashtable_Table*t){
# 1270 "buildlib.cyl"
struct Cyc_Set_Set*emptyset=({struct Cyc_Set_Set*(*_tmp188)(int(*cmp)(struct _fat_ptr*,struct _fat_ptr*))=(struct Cyc_Set_Set*(*)(int(*cmp)(struct _fat_ptr*,struct _fat_ptr*)))Cyc_Set_empty;_tmp188;})(Cyc_strptrcmp);
struct Cyc_Set_Set*curr;
for(curr=emptyset;init != 0;init=init->tl){
curr=({struct Cyc_Set_Set*(*_tmp4BC)(struct Cyc_Set_Set*s,struct _fat_ptr*elt)=({struct Cyc_Set_Set*(*_tmp184)(struct Cyc_Set_Set*s,struct _fat_ptr*elt)=(struct Cyc_Set_Set*(*)(struct Cyc_Set_Set*s,struct _fat_ptr*elt))Cyc_Set_insert;_tmp184;});struct Cyc_Set_Set*_tmp4BB=curr;_tmp4BC(_tmp4BB,(struct _fat_ptr*)init->hd);});}{
# 1275
struct Cyc_Set_Set*delta=curr;
# 1277
struct _fat_ptr*sptr=({struct _fat_ptr*_tmp187=_cycalloc(sizeof(*_tmp187));({struct _fat_ptr _tmp4BD=({const char*_tmp186="";_tag_fat(_tmp186,sizeof(char),1U);});*_tmp187=_tmp4BD;});_tmp187;});
while(Cyc_Set_cardinality(delta)> 0){
struct Cyc_Set_Set*next=emptyset;
struct Cyc_Iter_Iter iter=Cyc_Set_make_iter(Cyc_Core_heap_region,delta);
while(({int(*_tmp4BE)(struct Cyc_Iter_Iter,struct _fat_ptr**)=({int(*_tmp185)(struct Cyc_Iter_Iter,struct _fat_ptr**)=(int(*)(struct Cyc_Iter_Iter,struct _fat_ptr**))Cyc_Iter_next;_tmp185;});_tmp4BE(iter,& sptr);})){
next=({struct Cyc_Set_Set*_tmp4BF=next;Cyc_Set_union_two(_tmp4BF,Cyc_find(t,sptr));});}
delta=Cyc_Set_diff(next,curr);
curr=Cyc_Set_union_two(curr,delta);}
# 1286
return curr;}}
# 1048 "buildlib.cyl"
enum Cyc_buildlib_mode{Cyc_NORMAL =0U,Cyc_GATHER =1U,Cyc_GATHERSCRIPT =2U,Cyc_FINISH =3U};
# 1290 "buildlib.cyl"
static enum Cyc_buildlib_mode Cyc_mode=Cyc_NORMAL;
static int Cyc_gathering(){
return(int)Cyc_mode == (int)Cyc_GATHER ||(int)Cyc_mode == (int)Cyc_GATHERSCRIPT;}
# 1291
static struct Cyc___cycFILE*Cyc_script_file=0;
# 1296
int Cyc_prscript(struct _fat_ptr fmt,struct _fat_ptr ap){
# 1299
if(Cyc_script_file == 0){
({void*_tmp189=0U;({struct Cyc___cycFILE*_tmp4C1=Cyc_stderr;struct _fat_ptr _tmp4C0=({const char*_tmp18A="Internal error: script file is NULL\n";_tag_fat(_tmp18A,sizeof(char),37U);});Cyc_fprintf(_tmp4C1,_tmp4C0,_tag_fat(_tmp189,sizeof(void*),0U));});});
exit(1);}
# 1299
return
# 1303
 Cyc_vfprintf((struct Cyc___cycFILE*)_check_null(Cyc_script_file),fmt,ap);}
# 1296
int Cyc_force_directory(struct _fat_ptr d){
# 1307
if((int)Cyc_mode == (int)Cyc_GATHERSCRIPT)
({struct Cyc_String_pa_PrintArg_struct _tmp18D=({struct Cyc_String_pa_PrintArg_struct _tmp376;_tmp376.tag=0U,_tmp376.f1=(struct _fat_ptr)((struct _fat_ptr)d);_tmp376;});struct Cyc_String_pa_PrintArg_struct _tmp18E=({struct Cyc_String_pa_PrintArg_struct _tmp375;_tmp375.tag=0U,_tmp375.f1=(struct _fat_ptr)((struct _fat_ptr)d);_tmp375;});void*_tmp18B[2U];_tmp18B[0]=& _tmp18D,_tmp18B[1]=& _tmp18E;({struct _fat_ptr _tmp4C2=({const char*_tmp18C="if ! test -e %s; then mkdir %s; fi\n";_tag_fat(_tmp18C,sizeof(char),36U);});Cyc_prscript(_tmp4C2,_tag_fat(_tmp18B,sizeof(void*),2U));});});else{
# 1313
int fd=({void*_tmp192=0U;({const char*_tmp4C3=(const char*)_untag_fat_ptr(d,sizeof(char),1U);Cyc_open(_tmp4C3,0,_tag_fat(_tmp192,sizeof(unsigned short),0U));});});
if(fd == -1){
if(mkdir((const char*)_check_null(_untag_fat_ptr(d,sizeof(char),1U)),448)== - 1){
({struct Cyc_String_pa_PrintArg_struct _tmp191=({struct Cyc_String_pa_PrintArg_struct _tmp377;_tmp377.tag=0U,_tmp377.f1=(struct _fat_ptr)((struct _fat_ptr)d);_tmp377;});void*_tmp18F[1U];_tmp18F[0]=& _tmp191;({struct Cyc___cycFILE*_tmp4C5=Cyc_stderr;struct _fat_ptr _tmp4C4=({const char*_tmp190="Error: could not create directory %s\n";_tag_fat(_tmp190,sizeof(char),38U);});Cyc_fprintf(_tmp4C5,_tmp4C4,_tag_fat(_tmp18F,sizeof(void*),1U));});});
return 1;}}else{
# 1320
close(fd);}}
# 1322
return 0;}
# 1296
int Cyc_force_directory_prefixes(struct _fat_ptr file){
# 1329
struct _fat_ptr curr=Cyc_strdup((struct _fat_ptr)file);
# 1331
struct Cyc_List_List*x=0;
while(1){
curr=Cyc_Filename_dirname((struct _fat_ptr)curr);
if(Cyc_strlen((struct _fat_ptr)curr)== (unsigned long)0)break;x=({struct Cyc_List_List*_tmp194=_cycalloc(sizeof(*_tmp194));
({struct _fat_ptr*_tmp4C6=({struct _fat_ptr*_tmp193=_cycalloc(sizeof(*_tmp193));*_tmp193=(struct _fat_ptr)curr;_tmp193;});_tmp194->hd=_tmp4C6;}),_tmp194->tl=x;_tmp194;});}
# 1338
for(0;x != 0;x=x->tl){
if(Cyc_force_directory(*((struct _fat_ptr*)x->hd)))return 1;}
# 1341
return 0;}char Cyc_NO_SUPPORT[11U]="NO_SUPPORT";struct Cyc_NO_SUPPORT_exn_struct{char*tag;struct _fat_ptr f1;};
# 1348
static int Cyc_is_other_special(char c){
char _tmp195=c;switch(_tmp195){case 92U: _LL1: _LL2:
 goto _LL4;case 34U: _LL3: _LL4:
 goto _LL6;case 59U: _LL5: _LL6:
 goto _LL8;case 38U: _LL7: _LL8:
 goto _LLA;case 40U: _LL9: _LLA:
 goto _LLC;case 41U: _LLB: _LLC:
 goto _LLE;case 124U: _LLD: _LLE:
 goto _LL10;case 94U: _LLF: _LL10:
 goto _LL12;case 60U: _LL11: _LL12:
 goto _LL14;case 62U: _LL13: _LL14:
 goto _LL16;case 10U: _LL15: _LL16:
# 1363
 goto _LL18;case 9U: _LL17: _LL18:
 return 1;default: _LL19: _LL1A:
 return 0;}_LL0:;}
# 1348
static struct _fat_ptr Cyc_sh_escape_string(struct _fat_ptr s){
# 1370
unsigned long len=Cyc_strlen((struct _fat_ptr)s);
# 1373
int single_quotes=0;
int other_special=0;
{int i=0;for(0;(unsigned long)i < len;++ i){
char c=*((const char*)_check_fat_subscript(s,sizeof(char),i));
if((int)c == (int)'\'')++ single_quotes;else{
if(Cyc_is_other_special(c))++ other_special;}}}
# 1382
if(single_quotes == 0 && other_special == 0)
return s;
# 1382
if(single_quotes == 0)
# 1387
return(struct _fat_ptr)Cyc_strconcat_l(({struct _fat_ptr*_tmp196[3U];({struct _fat_ptr*_tmp4CB=({struct _fat_ptr*_tmp198=_cycalloc(sizeof(*_tmp198));({struct _fat_ptr _tmp4CA=({const char*_tmp197="'";_tag_fat(_tmp197,sizeof(char),2U);});*_tmp198=_tmp4CA;});_tmp198;});_tmp196[0]=_tmp4CB;}),({struct _fat_ptr*_tmp4C9=({struct _fat_ptr*_tmp199=_cycalloc(sizeof(*_tmp199));*_tmp199=(struct _fat_ptr)s;_tmp199;});_tmp196[1]=_tmp4C9;}),({struct _fat_ptr*_tmp4C8=({struct _fat_ptr*_tmp19B=_cycalloc(sizeof(*_tmp19B));({struct _fat_ptr _tmp4C7=({const char*_tmp19A="'";_tag_fat(_tmp19A,sizeof(char),2U);});*_tmp19B=_tmp4C7;});_tmp19B;});_tmp196[2]=_tmp4C8;});Cyc_List_list(_tag_fat(_tmp196,sizeof(struct _fat_ptr*),3U));}));{
# 1382
unsigned long len2=(len + (unsigned long)single_quotes)+ (unsigned long)other_special;
# 1391
struct _fat_ptr s2=({unsigned _tmp1A3=(len2 + (unsigned long)1)+ 1U;char*_tmp1A2=_cycalloc_atomic(_check_times(_tmp1A3,sizeof(char)));({{unsigned _tmp378=len2 + (unsigned long)1;unsigned i;for(i=0;i < _tmp378;++ i){_tmp1A2[i]='\000';}_tmp1A2[_tmp378]=0;}0;});_tag_fat(_tmp1A2,sizeof(char),_tmp1A3);});
int i=0;
int j=0;
for(0;(unsigned long)i < len;++ i){
char c=*((const char*)_check_fat_subscript(s,sizeof(char),i));
if((int)c == (int)'\'' || Cyc_is_other_special(c))
({struct _fat_ptr _tmp19C=_fat_ptr_plus(s2,sizeof(char),j ++);char _tmp19D=*((char*)_check_fat_subscript(_tmp19C,sizeof(char),0U));char _tmp19E='\\';if(_get_fat_size(_tmp19C,sizeof(char))== 1U &&(_tmp19D == 0 && _tmp19E != 0))_throw_arraybounds();*((char*)_tmp19C.curr)=_tmp19E;});
# 1396
({struct _fat_ptr _tmp19F=_fat_ptr_plus(s2,sizeof(char),j ++);char _tmp1A0=*((char*)_check_fat_subscript(_tmp19F,sizeof(char),0U));char _tmp1A1=c;if(_get_fat_size(_tmp19F,sizeof(char))== 1U &&(_tmp1A0 == 0 && _tmp1A1 != 0))_throw_arraybounds();*((char*)_tmp19F.curr)=_tmp1A1;});}
# 1400
return(struct _fat_ptr)s2;}}
# 1348
static struct _fat_ptr*Cyc_sh_escape_stringptr(struct _fat_ptr*sp){
# 1403
return({struct _fat_ptr*_tmp1A4=_cycalloc(sizeof(*_tmp1A4));({struct _fat_ptr _tmp4CC=Cyc_sh_escape_string(*sp);*_tmp1A4=_tmp4CC;});_tmp1A4;});}
# 1348
int Cyc_process_file(const char*filename,struct Cyc_List_List*start_symbols,struct Cyc_List_List*user_defs,struct Cyc_List_List*omit_symbols,struct Cyc_List_List*hstubs,struct Cyc_List_List*cstubs,struct Cyc_List_List*cycstubs,struct Cyc_List_List*cpp_insert){
# 1415
struct Cyc___cycFILE*maybe;
struct Cyc___cycFILE*in_file;
struct Cyc___cycFILE*out_file;
int errorcode=0;
# 1420
({struct Cyc_String_pa_PrintArg_struct _tmp1A7=({struct Cyc_String_pa_PrintArg_struct _tmp379;_tmp379.tag=0U,({
struct _fat_ptr _tmp4CD=(struct _fat_ptr)({const char*_tmp1A8=filename;_tag_fat(_tmp1A8,sizeof(char),_get_zero_arr_size_char((void*)_tmp1A8,1U));});_tmp379.f1=_tmp4CD;});_tmp379;});void*_tmp1A5[1U];_tmp1A5[0]=& _tmp1A7;({struct Cyc___cycFILE*_tmp4CF=Cyc_stderr;struct _fat_ptr _tmp4CE=({const char*_tmp1A6="********************************* %s...\n";_tag_fat(_tmp1A6,sizeof(char),41U);});Cyc_fprintf(_tmp4CF,_tmp4CE,_tag_fat(_tmp1A5,sizeof(void*),1U));});});
# 1423
if(!Cyc_gathering())({struct Cyc_String_pa_PrintArg_struct _tmp1AB=({struct Cyc_String_pa_PrintArg_struct _tmp37A;_tmp37A.tag=0U,({struct _fat_ptr _tmp4D0=(struct _fat_ptr)({const char*_tmp1AC=filename;_tag_fat(_tmp1AC,sizeof(char),_get_zero_arr_size_char((void*)_tmp1AC,1U));});_tmp37A.f1=_tmp4D0;});_tmp37A;});void*_tmp1A9[1U];_tmp1A9[0]=& _tmp1AB;({struct _fat_ptr _tmp4D1=({const char*_tmp1AA="\n%s:\n";_tag_fat(_tmp1AA,sizeof(char),6U);});Cyc_log(_tmp4D1,_tag_fat(_tmp1A9,sizeof(void*),1U));});});{struct _fat_ptr basename=
# 1435 "buildlib.cyl"
Cyc_Filename_basename(({const char*_tmp2CC=filename;_tag_fat(_tmp2CC,sizeof(char),_get_zero_arr_size_char((void*)_tmp2CC,1U));}));
struct _fat_ptr dirname=Cyc_Filename_dirname(({const char*_tmp2CB=filename;_tag_fat(_tmp2CB,sizeof(char),_get_zero_arr_size_char((void*)_tmp2CB,1U));}));
struct _fat_ptr choppedname=Cyc_Filename_chop_extension((struct _fat_ptr)basename);
const char*cppinfile=(const char*)_check_null(_untag_fat_ptr(({struct _fat_ptr _tmp4D2=(struct _fat_ptr)choppedname;Cyc_strconcat(_tmp4D2,({const char*_tmp2CA=".iA";_tag_fat(_tmp2CA,sizeof(char),4U);}));}),sizeof(char),1U));
# 1440
const char*macrosfile=(const char*)_check_null(_untag_fat_ptr(_get_fat_size(dirname,sizeof(char))== (unsigned)0?({struct Cyc_String_pa_PrintArg_struct _tmp2C6=({struct Cyc_String_pa_PrintArg_struct _tmp3B0;_tmp3B0.tag=0U,_tmp3B0.f1=(struct _fat_ptr)((struct _fat_ptr)choppedname);_tmp3B0;});void*_tmp2C4[1U];_tmp2C4[0]=& _tmp2C6;({struct _fat_ptr _tmp4D5=({const char*_tmp2C5="%s.iB";_tag_fat(_tmp2C5,sizeof(char),6U);});Cyc_aprintf(_tmp4D5,_tag_fat(_tmp2C4,sizeof(void*),1U));});}):({struct _fat_ptr _tmp4D4=(struct _fat_ptr)dirname;Cyc_Filename_concat(_tmp4D4,(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp2C9=({struct Cyc_String_pa_PrintArg_struct _tmp3B1;_tmp3B1.tag=0U,_tmp3B1.f1=(struct _fat_ptr)((struct _fat_ptr)choppedname);_tmp3B1;});void*_tmp2C7[1U];_tmp2C7[0]=& _tmp2C9;({struct _fat_ptr _tmp4D3=({const char*_tmp2C8="%s.iB";_tag_fat(_tmp2C8,sizeof(char),6U);});Cyc_aprintf(_tmp4D3,_tag_fat(_tmp2C7,sizeof(void*),1U));});}));}),sizeof(char),1U));
# 1445
const char*declsfile=(const char*)_check_null(_untag_fat_ptr(_get_fat_size(dirname,sizeof(char))== (unsigned)0?({struct Cyc_String_pa_PrintArg_struct _tmp2C0=({struct Cyc_String_pa_PrintArg_struct _tmp3AE;_tmp3AE.tag=0U,_tmp3AE.f1=(struct _fat_ptr)((struct _fat_ptr)choppedname);_tmp3AE;});void*_tmp2BE[1U];_tmp2BE[0]=& _tmp2C0;({struct _fat_ptr _tmp4D8=({const char*_tmp2BF="%s.iC";_tag_fat(_tmp2BF,sizeof(char),6U);});Cyc_aprintf(_tmp4D8,_tag_fat(_tmp2BE,sizeof(void*),1U));});}):({struct _fat_ptr _tmp4D7=(struct _fat_ptr)dirname;Cyc_Filename_concat(_tmp4D7,(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp2C3=({struct Cyc_String_pa_PrintArg_struct _tmp3AF;_tmp3AF.tag=0U,_tmp3AF.f1=(struct _fat_ptr)((struct _fat_ptr)choppedname);_tmp3AF;});void*_tmp2C1[1U];_tmp2C1[0]=& _tmp2C3;({struct _fat_ptr _tmp4D6=({const char*_tmp2C2="%s.iC";_tag_fat(_tmp2C2,sizeof(char),6U);});Cyc_aprintf(_tmp4D6,_tag_fat(_tmp2C1,sizeof(void*),1U));});}));}),sizeof(char),1U));
# 1450
const char*filtereddeclsfile=(const char*)_check_null(_untag_fat_ptr(_get_fat_size(dirname,sizeof(char))== (unsigned)0?({struct Cyc_String_pa_PrintArg_struct _tmp2BA=({struct Cyc_String_pa_PrintArg_struct _tmp3AC;_tmp3AC.tag=0U,_tmp3AC.f1=(struct _fat_ptr)((struct _fat_ptr)choppedname);_tmp3AC;});void*_tmp2B8[1U];_tmp2B8[0]=& _tmp2BA;({struct _fat_ptr _tmp4DB=({const char*_tmp2B9="%s.iD";_tag_fat(_tmp2B9,sizeof(char),6U);});Cyc_aprintf(_tmp4DB,_tag_fat(_tmp2B8,sizeof(void*),1U));});}):({struct _fat_ptr _tmp4DA=(struct _fat_ptr)dirname;Cyc_Filename_concat(_tmp4DA,(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp2BD=({struct Cyc_String_pa_PrintArg_struct _tmp3AD;_tmp3AD.tag=0U,_tmp3AD.f1=(struct _fat_ptr)((struct _fat_ptr)choppedname);_tmp3AD;});void*_tmp2BB[1U];_tmp2BB[0]=& _tmp2BD;({struct _fat_ptr _tmp4D9=({const char*_tmp2BC="%s.iD";_tag_fat(_tmp2BC,sizeof(char),6U);});Cyc_aprintf(_tmp4D9,_tag_fat(_tmp2BB,sizeof(void*),1U));});}));}),sizeof(char),1U));
# 1455
{struct _handler_cons _tmp1AD;_push_handler(& _tmp1AD);{int _tmp1AF=0;if(setjmp(_tmp1AD.handler))_tmp1AF=1;if(!_tmp1AF){
# 1458
if(Cyc_force_directory_prefixes(({const char*_tmp1B0=filename;_tag_fat(_tmp1B0,sizeof(char),_get_zero_arr_size_char((void*)_tmp1B0,1U));}))){
int _tmp1B1=1;_npop_handler(0U);return _tmp1B1;}
# 1458
if((int)Cyc_mode != (int)Cyc_FINISH){
# 1464
if((int)Cyc_mode == (int)Cyc_GATHERSCRIPT){
({struct Cyc_String_pa_PrintArg_struct _tmp1B4=({struct Cyc_String_pa_PrintArg_struct _tmp37B;_tmp37B.tag=0U,({struct _fat_ptr _tmp4DC=(struct _fat_ptr)({const char*_tmp1B5=cppinfile;_tag_fat(_tmp1B5,sizeof(char),_get_zero_arr_size_char((void*)_tmp1B5,1U));});_tmp37B.f1=_tmp4DC;});_tmp37B;});void*_tmp1B2[1U];_tmp1B2[0]=& _tmp1B4;({struct _fat_ptr _tmp4DD=({const char*_tmp1B3="cat >%s <<XXX\n";_tag_fat(_tmp1B3,sizeof(char),15U);});Cyc_prscript(_tmp4DD,_tag_fat(_tmp1B2,sizeof(void*),1U));});});
{struct Cyc_List_List*l=cpp_insert;for(0;l != 0;l=l->tl){
({struct Cyc_String_pa_PrintArg_struct _tmp1B8=({struct Cyc_String_pa_PrintArg_struct _tmp37C;_tmp37C.tag=0U,_tmp37C.f1=(struct _fat_ptr)((struct _fat_ptr)*((struct _fat_ptr*)l->hd));_tmp37C;});void*_tmp1B6[1U];_tmp1B6[0]=& _tmp1B8;({struct _fat_ptr _tmp4DE=({const char*_tmp1B7="%s";_tag_fat(_tmp1B7,sizeof(char),3U);});Cyc_prscript(_tmp4DE,_tag_fat(_tmp1B6,sizeof(void*),1U));});});}}
({struct Cyc_String_pa_PrintArg_struct _tmp1BB=({struct Cyc_String_pa_PrintArg_struct _tmp37D;_tmp37D.tag=0U,({struct _fat_ptr _tmp4DF=(struct _fat_ptr)({const char*_tmp1BC=filename;_tag_fat(_tmp1BC,sizeof(char),_get_zero_arr_size_char((void*)_tmp1BC,1U));});_tmp37D.f1=_tmp4DF;});_tmp37D;});void*_tmp1B9[1U];_tmp1B9[0]=& _tmp1BB;({struct _fat_ptr _tmp4E0=({const char*_tmp1BA="#include <%s>\n";_tag_fat(_tmp1BA,sizeof(char),15U);});Cyc_prscript(_tmp4E0,_tag_fat(_tmp1B9,sizeof(void*),1U));});});
({void*_tmp1BD=0U;({struct _fat_ptr _tmp4E1=({const char*_tmp1BE="XXX\n";_tag_fat(_tmp1BE,sizeof(char),5U);});Cyc_prscript(_tmp4E1,_tag_fat(_tmp1BD,sizeof(void*),0U));});});
({struct Cyc_String_pa_PrintArg_struct _tmp1C1=({struct Cyc_String_pa_PrintArg_struct _tmp380;_tmp380.tag=0U,_tmp380.f1=(struct _fat_ptr)((struct _fat_ptr)Cyc_target_cflags);_tmp380;});struct Cyc_String_pa_PrintArg_struct _tmp1C2=({struct Cyc_String_pa_PrintArg_struct _tmp37F;_tmp37F.tag=0U,({struct _fat_ptr _tmp4E2=(struct _fat_ptr)({const char*_tmp1C5=macrosfile;_tag_fat(_tmp1C5,sizeof(char),_get_zero_arr_size_char((void*)_tmp1C5,1U));});_tmp37F.f1=_tmp4E2;});_tmp37F;});struct Cyc_String_pa_PrintArg_struct _tmp1C3=({struct Cyc_String_pa_PrintArg_struct _tmp37E;_tmp37E.tag=0U,({struct _fat_ptr _tmp4E3=(struct _fat_ptr)({const char*_tmp1C4=cppinfile;_tag_fat(_tmp1C4,sizeof(char),_get_zero_arr_size_char((void*)_tmp1C4,1U));});_tmp37E.f1=_tmp4E3;});_tmp37E;});void*_tmp1BF[3U];_tmp1BF[0]=& _tmp1C1,_tmp1BF[1]=& _tmp1C2,_tmp1BF[2]=& _tmp1C3;({struct _fat_ptr _tmp4E4=({const char*_tmp1C0="$GCC %s -E -dM -o %s -x c %s && \\\n";_tag_fat(_tmp1C0,sizeof(char),35U);});Cyc_prscript(_tmp4E4,_tag_fat(_tmp1BF,sizeof(void*),3U));});});
({struct Cyc_String_pa_PrintArg_struct _tmp1C8=({struct Cyc_String_pa_PrintArg_struct _tmp383;_tmp383.tag=0U,_tmp383.f1=(struct _fat_ptr)((struct _fat_ptr)Cyc_target_cflags);_tmp383;});struct Cyc_String_pa_PrintArg_struct _tmp1C9=({struct Cyc_String_pa_PrintArg_struct _tmp382;_tmp382.tag=0U,({struct _fat_ptr _tmp4E5=(struct _fat_ptr)({const char*_tmp1CC=declsfile;_tag_fat(_tmp1CC,sizeof(char),_get_zero_arr_size_char((void*)_tmp1CC,1U));});_tmp382.f1=_tmp4E5;});_tmp382;});struct Cyc_String_pa_PrintArg_struct _tmp1CA=({struct Cyc_String_pa_PrintArg_struct _tmp381;_tmp381.tag=0U,({struct _fat_ptr _tmp4E6=(struct _fat_ptr)({const char*_tmp1CB=cppinfile;_tag_fat(_tmp1CB,sizeof(char),_get_zero_arr_size_char((void*)_tmp1CB,1U));});_tmp381.f1=_tmp4E6;});_tmp381;});void*_tmp1C6[3U];_tmp1C6[0]=& _tmp1C8,_tmp1C6[1]=& _tmp1C9,_tmp1C6[2]=& _tmp1CA;({struct _fat_ptr _tmp4E7=({const char*_tmp1C7="$GCC %s -E     -o %s -x c %s;\n";_tag_fat(_tmp1C7,sizeof(char),31U);});Cyc_prscript(_tmp4E7,_tag_fat(_tmp1C6,sizeof(void*),3U));});});
({struct Cyc_String_pa_PrintArg_struct _tmp1CF=({struct Cyc_String_pa_PrintArg_struct _tmp384;_tmp384.tag=0U,({struct _fat_ptr _tmp4E8=(struct _fat_ptr)({const char*_tmp1D0=cppinfile;_tag_fat(_tmp1D0,sizeof(char),_get_zero_arr_size_char((void*)_tmp1D0,1U));});_tmp384.f1=_tmp4E8;});_tmp384;});void*_tmp1CD[1U];_tmp1CD[0]=& _tmp1CF;({struct _fat_ptr _tmp4E9=({const char*_tmp1CE="rm %s\n";_tag_fat(_tmp1CE,sizeof(char),7U);});Cyc_prscript(_tmp4E9,_tag_fat(_tmp1CD,sizeof(void*),1U));});});}else{
# 1475
maybe=Cyc_fopen(cppinfile,"w");
if(!((unsigned)maybe)){
({struct Cyc_String_pa_PrintArg_struct _tmp1D3=({struct Cyc_String_pa_PrintArg_struct _tmp385;_tmp385.tag=0U,({struct _fat_ptr _tmp4EA=(struct _fat_ptr)({const char*_tmp1D4=cppinfile;_tag_fat(_tmp1D4,sizeof(char),_get_zero_arr_size_char((void*)_tmp1D4,1U));});_tmp385.f1=_tmp4EA;});_tmp385;});void*_tmp1D1[1U];_tmp1D1[0]=& _tmp1D3;({struct Cyc___cycFILE*_tmp4EC=Cyc_stderr;struct _fat_ptr _tmp4EB=({const char*_tmp1D2="Error: could not create file %s\n";_tag_fat(_tmp1D2,sizeof(char),33U);});Cyc_fprintf(_tmp4EC,_tmp4EB,_tag_fat(_tmp1D1,sizeof(void*),1U));});});{
int _tmp1D5=1;_npop_handler(0U);return _tmp1D5;}}
# 1476
if(Cyc_verbose)
# 1481
({struct Cyc_String_pa_PrintArg_struct _tmp1D8=({struct Cyc_String_pa_PrintArg_struct _tmp386;_tmp386.tag=0U,({struct _fat_ptr _tmp4ED=(struct _fat_ptr)({const char*_tmp1D9=cppinfile;_tag_fat(_tmp1D9,sizeof(char),_get_zero_arr_size_char((void*)_tmp1D9,1U));});_tmp386.f1=_tmp4ED;});_tmp386;});void*_tmp1D6[1U];_tmp1D6[0]=& _tmp1D8;({struct Cyc___cycFILE*_tmp4EF=Cyc_stderr;struct _fat_ptr _tmp4EE=({const char*_tmp1D7="Creating %s\n";_tag_fat(_tmp1D7,sizeof(char),13U);});Cyc_fprintf(_tmp4EF,_tmp4EE,_tag_fat(_tmp1D6,sizeof(void*),1U));});});
# 1476
out_file=maybe;
# 1483
{struct Cyc_List_List*l=cpp_insert;for(0;l != 0;l=l->tl){
Cyc_fputs((const char*)_check_null(_untag_fat_ptr(*((struct _fat_ptr*)l->hd),sizeof(char),1U)),out_file);}}
# 1486
({struct Cyc_String_pa_PrintArg_struct _tmp1DC=({struct Cyc_String_pa_PrintArg_struct _tmp387;_tmp387.tag=0U,({struct _fat_ptr _tmp4F0=(struct _fat_ptr)({const char*_tmp1DD=filename;_tag_fat(_tmp1DD,sizeof(char),_get_zero_arr_size_char((void*)_tmp1DD,1U));});_tmp387.f1=_tmp4F0;});_tmp387;});void*_tmp1DA[1U];_tmp1DA[0]=& _tmp1DC;({struct Cyc___cycFILE*_tmp4F2=out_file;struct _fat_ptr _tmp4F1=({const char*_tmp1DB="#include <%s>\n";_tag_fat(_tmp1DB,sizeof(char),15U);});Cyc_fprintf(_tmp4F2,_tmp4F1,_tag_fat(_tmp1DA,sizeof(void*),1U));});});
Cyc_fclose(out_file);{
struct _fat_ptr cppargs_string=({
struct Cyc_List_List*_tmp4F7=({struct Cyc_List_List*_tmp1FB=_cycalloc(sizeof(*_tmp1FB));({struct _fat_ptr*_tmp4F6=({struct _fat_ptr*_tmp1F9=_cycalloc(sizeof(*_tmp1F9));({struct _fat_ptr _tmp4F5=(struct _fat_ptr)({const char*_tmp1F8="";_tag_fat(_tmp1F8,sizeof(char),1U);});*_tmp1F9=_tmp4F5;});_tmp1F9;});_tmp1FB->hd=_tmp4F6;}),({
struct Cyc_List_List*_tmp4F4=({struct Cyc_List_List*(*_tmp4F3)(struct _fat_ptr*(*f)(struct _fat_ptr*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp1FA)(struct _fat_ptr*(*f)(struct _fat_ptr*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct _fat_ptr*(*f)(struct _fat_ptr*),struct Cyc_List_List*x))Cyc_List_map;_tmp1FA;});_tmp4F3(Cyc_sh_escape_stringptr,Cyc_List_rev(Cyc_cppargs));});_tmp1FB->tl=_tmp4F4;});_tmp1FB;});
# 1489
Cyc_str_sepstr(_tmp4F7,({const char*_tmp1FC=" ";_tag_fat(_tmp1FC,sizeof(char),2U);}));});
# 1492
char*cmd=(char*)_check_null(_untag_fat_ptr(({struct Cyc_String_pa_PrintArg_struct _tmp1F1=({struct Cyc_String_pa_PrintArg_struct _tmp393;_tmp393.tag=0U,_tmp393.f1=(struct _fat_ptr)((struct _fat_ptr)Cyc_cyclone_cc);_tmp393;});struct Cyc_String_pa_PrintArg_struct _tmp1F2=({struct Cyc_String_pa_PrintArg_struct _tmp392;_tmp392.tag=0U,_tmp392.f1=(struct _fat_ptr)((struct _fat_ptr)Cyc_target_cflags);_tmp392;});struct Cyc_String_pa_PrintArg_struct _tmp1F3=({struct Cyc_String_pa_PrintArg_struct _tmp391;_tmp391.tag=0U,_tmp391.f1=(struct _fat_ptr)((struct _fat_ptr)cppargs_string);_tmp391;});struct Cyc_String_pa_PrintArg_struct _tmp1F4=({struct Cyc_String_pa_PrintArg_struct _tmp390;_tmp390.tag=0U,({struct _fat_ptr _tmp4F8=(struct _fat_ptr)({const char*_tmp1F7=macrosfile;_tag_fat(_tmp1F7,sizeof(char),_get_zero_arr_size_char((void*)_tmp1F7,1U));});_tmp390.f1=_tmp4F8;});_tmp390;});struct Cyc_String_pa_PrintArg_struct _tmp1F5=({struct Cyc_String_pa_PrintArg_struct _tmp38F;_tmp38F.tag=0U,({struct _fat_ptr _tmp4F9=(struct _fat_ptr)({const char*_tmp1F6=cppinfile;_tag_fat(_tmp1F6,sizeof(char),_get_zero_arr_size_char((void*)_tmp1F6,1U));});_tmp38F.f1=_tmp4F9;});_tmp38F;});void*_tmp1EF[5U];_tmp1EF[0]=& _tmp1F1,_tmp1EF[1]=& _tmp1F2,_tmp1EF[2]=& _tmp1F3,_tmp1EF[3]=& _tmp1F4,_tmp1EF[4]=& _tmp1F5;({struct _fat_ptr _tmp4FA=({const char*_tmp1F0="%s %s %s -E -dM -o %s -x c %s";_tag_fat(_tmp1F0,sizeof(char),30U);});Cyc_aprintf(_tmp4FA,_tag_fat(_tmp1EF,sizeof(void*),5U));});}),sizeof(char),1U));
# 1495
if(Cyc_verbose)
({struct Cyc_String_pa_PrintArg_struct _tmp1E0=({struct Cyc_String_pa_PrintArg_struct _tmp388;_tmp388.tag=0U,({struct _fat_ptr _tmp4FB=(struct _fat_ptr)({char*_tmp1E1=cmd;_tag_fat(_tmp1E1,sizeof(char),_get_zero_arr_size_char((void*)_tmp1E1,1U));});_tmp388.f1=_tmp4FB;});_tmp388;});void*_tmp1DE[1U];_tmp1DE[0]=& _tmp1E0;({struct Cyc___cycFILE*_tmp4FD=Cyc_stderr;struct _fat_ptr _tmp4FC=({const char*_tmp1DF="%s\n";_tag_fat(_tmp1DF,sizeof(char),4U);});Cyc_fprintf(_tmp4FD,_tmp4FC,_tag_fat(_tmp1DE,sizeof(void*),1U));});});
# 1495
if(!
# 1497
system((const char*)cmd)){
# 1500
cmd=(char*)_check_null(_untag_fat_ptr(({struct Cyc_String_pa_PrintArg_struct _tmp1E4=({struct Cyc_String_pa_PrintArg_struct _tmp38D;_tmp38D.tag=0U,_tmp38D.f1=(struct _fat_ptr)((struct _fat_ptr)Cyc_cyclone_cc);_tmp38D;});struct Cyc_String_pa_PrintArg_struct _tmp1E5=({struct Cyc_String_pa_PrintArg_struct _tmp38C;_tmp38C.tag=0U,_tmp38C.f1=(struct _fat_ptr)((struct _fat_ptr)Cyc_target_cflags);_tmp38C;});struct Cyc_String_pa_PrintArg_struct _tmp1E6=({struct Cyc_String_pa_PrintArg_struct _tmp38B;_tmp38B.tag=0U,_tmp38B.f1=(struct _fat_ptr)((struct _fat_ptr)cppargs_string);_tmp38B;});struct Cyc_String_pa_PrintArg_struct _tmp1E7=({struct Cyc_String_pa_PrintArg_struct _tmp38A;_tmp38A.tag=0U,({struct _fat_ptr _tmp4FE=(struct _fat_ptr)({const char*_tmp1EA=declsfile;_tag_fat(_tmp1EA,sizeof(char),_get_zero_arr_size_char((void*)_tmp1EA,1U));});_tmp38A.f1=_tmp4FE;});_tmp38A;});struct Cyc_String_pa_PrintArg_struct _tmp1E8=({struct Cyc_String_pa_PrintArg_struct _tmp389;_tmp389.tag=0U,({struct _fat_ptr _tmp4FF=(struct _fat_ptr)({const char*_tmp1E9=cppinfile;_tag_fat(_tmp1E9,sizeof(char),_get_zero_arr_size_char((void*)_tmp1E9,1U));});_tmp389.f1=_tmp4FF;});_tmp389;});void*_tmp1E2[5U];_tmp1E2[0]=& _tmp1E4,_tmp1E2[1]=& _tmp1E5,_tmp1E2[2]=& _tmp1E6,_tmp1E2[3]=& _tmp1E7,_tmp1E2[4]=& _tmp1E8;({struct _fat_ptr _tmp500=({const char*_tmp1E3="%s %s %s -E -o %s -x c %s";_tag_fat(_tmp1E3,sizeof(char),26U);});Cyc_aprintf(_tmp500,_tag_fat(_tmp1E2,sizeof(void*),5U));});}),sizeof(char),1U));
# 1503
if(Cyc_verbose)
({struct Cyc_String_pa_PrintArg_struct _tmp1ED=({struct Cyc_String_pa_PrintArg_struct _tmp38E;_tmp38E.tag=0U,({struct _fat_ptr _tmp501=(struct _fat_ptr)({char*_tmp1EE=cmd;_tag_fat(_tmp1EE,sizeof(char),_get_zero_arr_size_char((void*)_tmp1EE,1U));});_tmp38E.f1=_tmp501;});_tmp38E;});void*_tmp1EB[1U];_tmp1EB[0]=& _tmp1ED;({struct Cyc___cycFILE*_tmp503=Cyc_stderr;struct _fat_ptr _tmp502=({const char*_tmp1EC="%s\n";_tag_fat(_tmp1EC,sizeof(char),4U);});Cyc_fprintf(_tmp503,_tmp502,_tag_fat(_tmp1EB,sizeof(void*),1U));});});
# 1503
system((const char*)cmd);}}}}
# 1458
if(
# 1510
Cyc_gathering()){int _tmp1FD=0;_npop_handler(0U);return _tmp1FD;}{
# 1458
struct Cyc_Hashtable_Table*t=
# 1513
Cyc_new_deps();
maybe=Cyc_fopen(macrosfile,"r");
if(!((unsigned)maybe))(int)_throw((void*)({struct Cyc_NO_SUPPORT_exn_struct*_tmp202=_cycalloc(sizeof(*_tmp202));_tmp202->tag=Cyc_NO_SUPPORT,({struct _fat_ptr _tmp506=(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp200=({struct Cyc_String_pa_PrintArg_struct _tmp394;_tmp394.tag=0U,({struct _fat_ptr _tmp504=(struct _fat_ptr)({const char*_tmp201=macrosfile;_tag_fat(_tmp201,sizeof(char),_get_zero_arr_size_char((void*)_tmp201,1U));});_tmp394.f1=_tmp504;});_tmp394;});void*_tmp1FE[1U];_tmp1FE[0]=& _tmp200;({struct _fat_ptr _tmp505=({const char*_tmp1FF="can't open macrosfile %s";_tag_fat(_tmp1FF,sizeof(char),25U);});Cyc_aprintf(_tmp505,_tag_fat(_tmp1FE,sizeof(void*),1U));});});_tmp202->f1=_tmp506;});_tmp202;}));in_file=maybe;{
# 1519
struct Cyc_Lexing_lexbuf*l=Cyc_Lexing_from_file(in_file);
struct _tuple25*entry;
while((entry=Cyc_line(l))!= 0){
struct _tuple25*_stmttmpF=(struct _tuple25*)_check_null(entry);struct Cyc_Set_Set*_tmp204;struct _fat_ptr*_tmp203;_LL1: _tmp203=_stmttmpF->f1;_tmp204=_stmttmpF->f2;_LL2: {struct _fat_ptr*name=_tmp203;struct Cyc_Set_Set*uses=_tmp204;
({void(*_tmp509)(struct Cyc_Hashtable_Table*t,struct _fat_ptr*key,struct Cyc_Set_Set*val)=({void(*_tmp205)(struct Cyc_Hashtable_Table*t,struct _fat_ptr*key,struct Cyc_Set_Set*val)=(void(*)(struct Cyc_Hashtable_Table*t,struct _fat_ptr*key,struct Cyc_Set_Set*val))Cyc_Hashtable_insert;_tmp205;});struct Cyc_Hashtable_Table*_tmp508=t;struct _fat_ptr*_tmp507=name;_tmp509(_tmp508,_tmp507,uses);});}}
# 1527
Cyc_fclose(in_file);
# 1530
maybe=Cyc_fopen(declsfile,"r");
if(!((unsigned)maybe))(int)_throw((void*)({struct Cyc_NO_SUPPORT_exn_struct*_tmp20A=_cycalloc(sizeof(*_tmp20A));_tmp20A->tag=Cyc_NO_SUPPORT,({struct _fat_ptr _tmp50C=(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp208=({struct Cyc_String_pa_PrintArg_struct _tmp395;_tmp395.tag=0U,({struct _fat_ptr _tmp50A=(struct _fat_ptr)({const char*_tmp209=declsfile;_tag_fat(_tmp209,sizeof(char),_get_zero_arr_size_char((void*)_tmp209,1U));});_tmp395.f1=_tmp50A;});_tmp395;});void*_tmp206[1U];_tmp206[0]=& _tmp208;({struct _fat_ptr _tmp50B=({const char*_tmp207="can't open declsfile %s";_tag_fat(_tmp207,sizeof(char),24U);});Cyc_aprintf(_tmp50B,_tag_fat(_tmp206,sizeof(void*),1U));});});_tmp20A->f1=_tmp50C;});_tmp20A;}));in_file=maybe;
# 1535
l=Cyc_Lexing_from_file(in_file);
Cyc_slurp_out=Cyc_fopen(filtereddeclsfile,"w");
if(!((unsigned)Cyc_slurp_out)){int _tmp20B=1;_npop_handler(0U);return _tmp20B;}while(
Cyc_slurp(l)){;}{
# 1540
struct Cyc_List_List*x=user_defs;
while(x != 0){
struct _tuple28*_stmttmp10=(struct _tuple28*)x->hd;struct _fat_ptr*_tmp20C;_LL4: _tmp20C=_stmttmp10->f2;_LL5: {struct _fat_ptr*s=_tmp20C;
({struct Cyc_String_pa_PrintArg_struct _tmp20F=({struct Cyc_String_pa_PrintArg_struct _tmp396;_tmp396.tag=0U,_tmp396.f1=(struct _fat_ptr)((struct _fat_ptr)*s);_tmp396;});void*_tmp20D[1U];_tmp20D[0]=& _tmp20F;({struct Cyc___cycFILE*_tmp50E=(struct Cyc___cycFILE*)_check_null(Cyc_slurp_out);struct _fat_ptr _tmp50D=({const char*_tmp20E="%s";_tag_fat(_tmp20E,sizeof(char),3U);});Cyc_fprintf(_tmp50E,_tmp50D,_tag_fat(_tmp20D,sizeof(void*),1U));});});
x=x->tl;}}
# 1546
Cyc_fclose(in_file);
Cyc_fclose((struct Cyc___cycFILE*)_check_null(Cyc_slurp_out));
if((int)Cyc_mode != (int)Cyc_FINISH)
;
# 1548
maybe=
# 1552
Cyc_fopen(filtereddeclsfile,"r");
if(!((unsigned)maybe)){int _tmp210=1;_npop_handler(0U);return _tmp210;}in_file=maybe;
# 1555
Cyc_Position_reset_position(({const char*_tmp211=filtereddeclsfile;_tag_fat(_tmp211,sizeof(char),_get_zero_arr_size_char((void*)_tmp211,1U));}));
Cyc_Lex_lex_init(0);{
struct Cyc_List_List*decls=({struct Cyc___cycFILE*_tmp50F=in_file;Cyc_Parse_parse_file(_tmp50F,({const char*_tmp28C=filtereddeclsfile;_tag_fat(_tmp28C,sizeof(char),_get_zero_arr_size_char((void*)_tmp28C,1U));}));});
Cyc_Lex_lex_init(0);
Cyc_fclose(in_file);
# 1562
{struct Cyc_List_List*d=decls;for(0;d != 0;d=d->tl){
Cyc_scan_decl((struct Cyc_Absyn_Decl*)d->hd,t);}}{
# 1566
struct Cyc_List_List*user_symbols=({struct Cyc_List_List*(*_tmp510)(struct _fat_ptr*(*f)(struct _fat_ptr*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp28B)(struct _fat_ptr*(*f)(struct _fat_ptr*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct _fat_ptr*(*f)(struct _fat_ptr*),struct Cyc_List_List*x))Cyc_List_map;_tmp28B;});_tmp510(Cyc_add_user_prefix,(Cyc_List_split(user_defs)).f1);});
struct Cyc_Set_Set*reachable_set=({struct Cyc_List_List*_tmp511=Cyc_List_append(start_symbols,user_symbols);Cyc_reachable(_tmp511,t);});
# 1570
struct Cyc_List_List*reachable_decls=0;
struct Cyc_List_List*user_decls=0;
struct Cyc_Set_Set*defined_symbols=({struct Cyc_Set_Set*(*_tmp28A)(int(*cmp)(struct _fat_ptr*,struct _fat_ptr*))=(struct Cyc_Set_Set*(*)(int(*cmp)(struct _fat_ptr*,struct _fat_ptr*)))Cyc_Set_empty;_tmp28A;})(Cyc_strptrcmp);
{struct Cyc_List_List*d=decls;for(0;d != 0;d=d->tl){
struct Cyc_Absyn_Decl*decl=(struct Cyc_Absyn_Decl*)d->hd;
struct _fat_ptr*name;
{void*_stmttmp11=decl->r;void*_tmp212=_stmttmp11;struct Cyc_Absyn_Typedefdecl*_tmp213;struct Cyc_Absyn_Enumdecl*_tmp214;struct Cyc_Absyn_Aggrdecl*_tmp215;struct Cyc_Absyn_Fndecl*_tmp216;struct Cyc_Absyn_Vardecl*_tmp217;switch(*((int*)_tmp212)){case 0U: _LL7: _tmp217=((struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct*)_tmp212)->f1;_LL8: {struct Cyc_Absyn_Vardecl*x=_tmp217;
# 1578
struct _tuple1*_stmttmp12=x->name;struct _fat_ptr*_tmp218;_LL2A: _tmp218=_stmttmp12->f2;_LL2B: {struct _fat_ptr*v=_tmp218;
defined_symbols=({struct Cyc_Set_Set*(*_tmp513)(struct Cyc_Set_Set*s,struct _fat_ptr*elt)=({struct Cyc_Set_Set*(*_tmp219)(struct Cyc_Set_Set*s,struct _fat_ptr*elt)=(struct Cyc_Set_Set*(*)(struct Cyc_Set_Set*s,struct _fat_ptr*elt))Cyc_Set_insert;_tmp219;});struct Cyc_Set_Set*_tmp512=defined_symbols;_tmp513(_tmp512,v);});
if(({int(*_tmp515)(int(*compare)(struct _fat_ptr*,struct _fat_ptr*),struct Cyc_List_List*l,struct _fat_ptr*x)=({int(*_tmp21A)(int(*compare)(struct _fat_ptr*,struct _fat_ptr*),struct Cyc_List_List*l,struct _fat_ptr*x)=(int(*)(int(*compare)(struct _fat_ptr*,struct _fat_ptr*),struct Cyc_List_List*l,struct _fat_ptr*x))Cyc_List_mem;_tmp21A;});struct Cyc_List_List*_tmp514=omit_symbols;_tmp515(Cyc_strptrcmp,_tmp514,v);}))name=0;else{
name=v;}
goto _LL6;}}case 1U: _LL9: _tmp216=((struct Cyc_Absyn_Fn_d_Absyn_Raw_decl_struct*)_tmp212)->f1;_LLA: {struct Cyc_Absyn_Fndecl*x=_tmp216;
# 1584
struct _tuple1*_stmttmp13=x->name;struct _fat_ptr*_tmp21B;_LL2D: _tmp21B=_stmttmp13->f2;_LL2E: {struct _fat_ptr*v=_tmp21B;
defined_symbols=({struct Cyc_Set_Set*(*_tmp517)(struct Cyc_Set_Set*s,struct _fat_ptr*elt)=({struct Cyc_Set_Set*(*_tmp21C)(struct Cyc_Set_Set*s,struct _fat_ptr*elt)=(struct Cyc_Set_Set*(*)(struct Cyc_Set_Set*s,struct _fat_ptr*elt))Cyc_Set_insert;_tmp21C;});struct Cyc_Set_Set*_tmp516=defined_symbols;_tmp517(_tmp516,v);});
if(({int(*_tmp519)(int(*compare)(struct _fat_ptr*,struct _fat_ptr*),struct Cyc_List_List*l,struct _fat_ptr*x)=({int(*_tmp21D)(int(*compare)(struct _fat_ptr*,struct _fat_ptr*),struct Cyc_List_List*l,struct _fat_ptr*x)=(int(*)(int(*compare)(struct _fat_ptr*,struct _fat_ptr*),struct Cyc_List_List*l,struct _fat_ptr*x))Cyc_List_mem;_tmp21D;});struct Cyc_List_List*_tmp518=omit_symbols;_tmp519(Cyc_strptrcmp,_tmp518,v);}))name=0;else{
name=v;}
goto _LL6;}}case 5U: _LLB: _tmp215=((struct Cyc_Absyn_Aggr_d_Absyn_Raw_decl_struct*)_tmp212)->f1;_LLC: {struct Cyc_Absyn_Aggrdecl*x=_tmp215;
# 1590
struct _tuple1*_stmttmp14=x->name;struct _fat_ptr*_tmp21E;_LL30: _tmp21E=_stmttmp14->f2;_LL31: {struct _fat_ptr*v=_tmp21E;
name=v;
goto _LL6;}}case 7U: _LLD: _tmp214=((struct Cyc_Absyn_Enum_d_Absyn_Raw_decl_struct*)_tmp212)->f1;_LLE: {struct Cyc_Absyn_Enumdecl*x=_tmp214;
# 1594
struct _tuple1*_stmttmp15=x->name;struct _fat_ptr*_tmp21F;_LL33: _tmp21F=_stmttmp15->f2;_LL34: {struct _fat_ptr*v=_tmp21F;
name=v;
# 1598
if(name != 0 &&({int(*_tmp51B)(struct Cyc_Set_Set*s,struct _fat_ptr*elt)=({int(*_tmp220)(struct Cyc_Set_Set*s,struct _fat_ptr*elt)=(int(*)(struct Cyc_Set_Set*s,struct _fat_ptr*elt))Cyc_Set_member;_tmp220;});struct Cyc_Set_Set*_tmp51A=reachable_set;_tmp51B(_tmp51A,name);}))
reachable_decls=({struct Cyc_List_List*_tmp221=_cycalloc(sizeof(*_tmp221));_tmp221->hd=decl,_tmp221->tl=reachable_decls;_tmp221;});else{
# 1601
if((unsigned)x->fields){
struct Cyc_List_List*fs=(struct Cyc_List_List*)((struct Cyc_Core_Opt*)_check_null(x->fields))->v;for(0;fs != 0;fs=fs->tl){
struct Cyc_Absyn_Enumfield*f=(struct Cyc_Absyn_Enumfield*)fs->hd;
struct _tuple1*_stmttmp16=f->name;struct _fat_ptr*_tmp222;_LL36: _tmp222=_stmttmp16->f2;_LL37: {struct _fat_ptr*v=_tmp222;
if(({int(*_tmp51D)(struct Cyc_Set_Set*s,struct _fat_ptr*elt)=({int(*_tmp223)(struct Cyc_Set_Set*s,struct _fat_ptr*elt)=(int(*)(struct Cyc_Set_Set*s,struct _fat_ptr*elt))Cyc_Set_member;_tmp223;});struct Cyc_Set_Set*_tmp51C=reachable_set;_tmp51D(_tmp51C,v);})){
reachable_decls=({struct Cyc_List_List*_tmp224=_cycalloc(sizeof(*_tmp224));_tmp224->hd=decl,_tmp224->tl=reachable_decls;_tmp224;});
break;}}}}}
# 1598
name=0;
# 1612
goto _LL6;}}case 8U: _LLF: _tmp213=((struct Cyc_Absyn_Typedef_d_Absyn_Raw_decl_struct*)_tmp212)->f1;_LL10: {struct Cyc_Absyn_Typedefdecl*x=_tmp213;
# 1614
struct _tuple1*_stmttmp17=x->name;struct _fat_ptr*_tmp225;_LL39: _tmp225=_stmttmp17->f2;_LL3A: {struct _fat_ptr*v=_tmp225;
name=v;
goto _LL6;}}case 13U: _LL11: _LL12:
 goto _LL14;case 14U: _LL13: _LL14:
 goto _LL16;case 15U: _LL15: _LL16:
 goto _LL18;case 16U: _LL17: _LL18:
 goto _LL1A;case 2U: _LL19: _LL1A:
 goto _LL1C;case 6U: _LL1B: _LL1C:
 goto _LL1E;case 3U: _LL1D: _LL1E:
 goto _LL20;case 9U: _LL1F: _LL20:
 goto _LL22;case 10U: _LL21: _LL22:
 goto _LL24;case 11U: _LL23: _LL24:
 goto _LL26;case 12U: _LL25: _LL26:
 goto _LL28;default: _LL27: _LL28:
# 1629
 name=0;
goto _LL6;}_LL6:;}
# 1633
if(name != 0 &&({int(*_tmp51F)(struct Cyc_Set_Set*s,struct _fat_ptr*elt)=({int(*_tmp226)(struct Cyc_Set_Set*s,struct _fat_ptr*elt)=(int(*)(struct Cyc_Set_Set*s,struct _fat_ptr*elt))Cyc_Set_member;_tmp226;});struct Cyc_Set_Set*_tmp51E=reachable_set;_tmp51F(_tmp51E,name);})){
if(({struct _fat_ptr _tmp521=(struct _fat_ptr)*name;struct _fat_ptr _tmp520=(struct _fat_ptr)Cyc_user_prefix;Cyc_strncmp(_tmp521,_tmp520,Cyc_strlen((struct _fat_ptr)Cyc_user_prefix));})!= 0)
reachable_decls=({struct Cyc_List_List*_tmp227=_cycalloc(sizeof(*_tmp227));_tmp227->hd=decl,_tmp227->tl=reachable_decls;_tmp227;});else{
# 1638
Cyc_rename_decl(decl);
user_decls=({struct Cyc_List_List*_tmp228=_cycalloc(sizeof(*_tmp228));_tmp228->hd=decl,_tmp228->tl=user_decls;_tmp228;});}}}}
# 1645
if(!Cyc_do_setjmp){
maybe=Cyc_fopen(filename,"w");
if(!((unsigned)maybe)){int _tmp229=1;_npop_handler(0U);return _tmp229;}out_file=maybe;}else{
# 1649
out_file=Cyc_stdout;}{
struct _fat_ptr ifdefmacro=({struct Cyc_String_pa_PrintArg_struct _tmp288=({struct Cyc_String_pa_PrintArg_struct _tmp3A2;_tmp3A2.tag=0U,({struct _fat_ptr _tmp522=(struct _fat_ptr)({const char*_tmp289=filename;_tag_fat(_tmp289,sizeof(char),_get_zero_arr_size_char((void*)_tmp289,1U));});_tmp3A2.f1=_tmp522;});_tmp3A2;});void*_tmp286[1U];_tmp286[0]=& _tmp288;({struct _fat_ptr _tmp523=({const char*_tmp287="_%s_";_tag_fat(_tmp287,sizeof(char),5U);});Cyc_aprintf(_tmp523,_tag_fat(_tmp286,sizeof(void*),1U));});});
{int j=0;for(0;(unsigned)j < _get_fat_size(ifdefmacro,sizeof(char));++ j){
if((int)((char*)ifdefmacro.curr)[j]== (int)'.' ||(int)((char*)ifdefmacro.curr)[j]== (int)'/')
({struct _fat_ptr _tmp22A=_fat_ptr_plus(ifdefmacro,sizeof(char),j);char _tmp22B=*((char*)_check_fat_subscript(_tmp22A,sizeof(char),0U));char _tmp22C='_';if(_get_fat_size(_tmp22A,sizeof(char))== 1U &&(_tmp22B == 0 && _tmp22C != 0))_throw_arraybounds();*((char*)_tmp22A.curr)=_tmp22C;});else{
if((int)((char*)ifdefmacro.curr)[j]!= (int)'_' &&(int)((char*)ifdefmacro.curr)[j]!= (int)'/')
({struct _fat_ptr _tmp22D=_fat_ptr_plus(ifdefmacro,sizeof(char),j);char _tmp22E=*((char*)_check_fat_subscript(_tmp22D,sizeof(char),0U));char _tmp22F=(char)toupper((int)((char*)ifdefmacro.curr)[j]);if(_get_fat_size(_tmp22D,sizeof(char))== 1U &&(_tmp22E == 0 && _tmp22F != 0))_throw_arraybounds();*((char*)_tmp22D.curr)=_tmp22F;});}}}
# 1657
({struct Cyc_String_pa_PrintArg_struct _tmp232=({struct Cyc_String_pa_PrintArg_struct _tmp398;_tmp398.tag=0U,_tmp398.f1=(struct _fat_ptr)((struct _fat_ptr)ifdefmacro);_tmp398;});struct Cyc_String_pa_PrintArg_struct _tmp233=({struct Cyc_String_pa_PrintArg_struct _tmp397;_tmp397.tag=0U,_tmp397.f1=(struct _fat_ptr)((struct _fat_ptr)ifdefmacro);_tmp397;});void*_tmp230[2U];_tmp230[0]=& _tmp232,_tmp230[1]=& _tmp233;({struct Cyc___cycFILE*_tmp525=out_file;struct _fat_ptr _tmp524=({const char*_tmp231="#ifndef %s\n#define %s\n";_tag_fat(_tmp231,sizeof(char),23U);});Cyc_fprintf(_tmp525,_tmp524,_tag_fat(_tmp230,sizeof(void*),2U));});});{
# 1664
struct Cyc_List_List*print_decls=0;
struct Cyc_List_List*names=0;
{struct Cyc_List_List*d=reachable_decls;for(0;d != 0;d=d->tl){
struct Cyc_Absyn_Decl*decl=(struct Cyc_Absyn_Decl*)d->hd;
int anon_enum=0;
struct _fat_ptr*name;
{void*_stmttmp18=decl->r;void*_tmp234=_stmttmp18;struct Cyc_Absyn_Typedefdecl*_tmp235;struct Cyc_Absyn_Enumdecl*_tmp236;struct Cyc_Absyn_Aggrdecl*_tmp237;struct Cyc_Absyn_Fndecl*_tmp238;struct Cyc_Absyn_Vardecl*_tmp239;switch(*((int*)_tmp234)){case 0U: _LL3C: _tmp239=((struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct*)_tmp234)->f1;_LL3D: {struct Cyc_Absyn_Vardecl*x=_tmp239;
# 1672
struct _tuple1*_stmttmp19=x->name;struct _fat_ptr*_tmp23A;_LL5F: _tmp23A=_stmttmp19->f2;_LL60: {struct _fat_ptr*v=_tmp23A;
name=v;
goto _LL3B;}}case 1U: _LL3E: _tmp238=((struct Cyc_Absyn_Fn_d_Absyn_Raw_decl_struct*)_tmp234)->f1;_LL3F: {struct Cyc_Absyn_Fndecl*x=_tmp238;
# 1676
if(x->is_inline){name=0;goto _LL3B;}{struct _tuple1*_stmttmp1A=x->name;struct _fat_ptr*_tmp23B;_LL62: _tmp23B=_stmttmp1A->f2;_LL63: {struct _fat_ptr*v=_tmp23B;
# 1678
name=v;
goto _LL3B;}}}case 5U: _LL40: _tmp237=((struct Cyc_Absyn_Aggr_d_Absyn_Raw_decl_struct*)_tmp234)->f1;_LL41: {struct Cyc_Absyn_Aggrdecl*x=_tmp237;
# 1681
struct _tuple1*_stmttmp1B=x->name;struct _fat_ptr*_tmp23C;_LL65: _tmp23C=_stmttmp1B->f2;_LL66: {struct _fat_ptr*v=_tmp23C;
name=v;
goto _LL3B;}}case 7U: _LL42: _tmp236=((struct Cyc_Absyn_Enum_d_Absyn_Raw_decl_struct*)_tmp234)->f1;_LL43: {struct Cyc_Absyn_Enumdecl*x=_tmp236;
# 1685
struct _tuple1*_stmttmp1C=x->name;struct _fat_ptr*_tmp23D;_LL68: _tmp23D=_stmttmp1C->f2;_LL69: {struct _fat_ptr*v=_tmp23D;
name=v;
goto _LL3B;}}case 8U: _LL44: _tmp235=((struct Cyc_Absyn_Typedef_d_Absyn_Raw_decl_struct*)_tmp234)->f1;_LL45: {struct Cyc_Absyn_Typedefdecl*x=_tmp235;
# 1689
struct _tuple1*_stmttmp1D=x->name;struct _fat_ptr*_tmp23E;_LL6B: _tmp23E=_stmttmp1D->f2;_LL6C: {struct _fat_ptr*v=_tmp23E;
name=v;
goto _LL3B;}}case 4U: _LL46: _LL47:
 goto _LL49;case 13U: _LL48: _LL49:
 goto _LL4B;case 14U: _LL4A: _LL4B:
 goto _LL4D;case 15U: _LL4C: _LL4D:
 goto _LL4F;case 16U: _LL4E: _LL4F:
 goto _LL51;case 2U: _LL50: _LL51:
 goto _LL53;case 6U: _LL52: _LL53:
 goto _LL55;case 3U: _LL54: _LL55:
 goto _LL57;case 9U: _LL56: _LL57:
 goto _LL59;case 10U: _LL58: _LL59:
 goto _LL5B;case 11U: _LL5A: _LL5B:
 goto _LL5D;default: _LL5C: _LL5D:
# 1704
 name=0;
goto _LL3B;}_LL3B:;}
# 1707
if(!((unsigned)name)&& !anon_enum)continue;if(({
# 1712
int(*_tmp527)(struct Cyc_Set_Set*s,struct _fat_ptr*elt)=({int(*_tmp23F)(struct Cyc_Set_Set*s,struct _fat_ptr*elt)=(int(*)(struct Cyc_Set_Set*s,struct _fat_ptr*elt))Cyc_Set_member;_tmp23F;});struct Cyc_Set_Set*_tmp526=reachable_set;_tmp527(_tmp526,Cyc_add_user_prefix(name));})){
struct Cyc_Absyn_Decl*user_decl=Cyc_Absyn_lookup_decl(user_decls,name);
if(user_decl == 0)
(int)_throw((void*)({struct Cyc_Core_Impossible_exn_struct*_tmp241=_cycalloc(sizeof(*_tmp241));_tmp241->tag=Cyc_Core_Impossible,({struct _fat_ptr _tmp528=({const char*_tmp240="Internal Error: bad user-def name";_tag_fat(_tmp240,sizeof(char),34U);});_tmp241->f1=_tmp528;});_tmp241;}));else{
# 1718
void*_stmttmp1E=user_decl->r;void*_tmp242=_stmttmp1E;switch(*((int*)_tmp242)){case 0U: _LL6E: _LL6F:
 goto _LL71;case 1U: _LL70: _LL71:
# 1721
(int)_throw((void*)({struct Cyc_NO_SUPPORT_exn_struct*_tmp244=_cycalloc(sizeof(*_tmp244));_tmp244->tag=Cyc_NO_SUPPORT,({struct _fat_ptr _tmp529=({const char*_tmp243="user defintions for function or variable decls";_tag_fat(_tmp243,sizeof(char),47U);});_tmp244->f1=_tmp529;});_tmp244;}));default: _LL72: _LL73:
 goto _LL6D;}_LL6D:;}
# 1725
Cyc_Cifc_merge_sys_user_decl(0U,1,user_decl,decl);
print_decls=({struct Cyc_List_List*_tmp245=_cycalloc(sizeof(*_tmp245));_tmp245->hd=decl,_tmp245->tl=print_decls;_tmp245;});}else{
# 1729
print_decls=({struct Cyc_List_List*_tmp246=_cycalloc(sizeof(*_tmp246));_tmp246->hd=decl,_tmp246->tl=print_decls;_tmp246;});}
names=({struct Cyc_List_List*_tmp247=_cycalloc(sizeof(*_tmp247));_tmp247->hd=name,_tmp247->tl=names;_tmp247;});}}
# 1734
{struct _handler_cons _tmp248;_push_handler(& _tmp248);{int _tmp24A=0;if(setjmp(_tmp248.handler))_tmp24A=1;if(!_tmp24A){
Cyc_Binding_resolve_all(print_decls);
({struct Cyc_Tcenv_Tenv*_tmp52A=Cyc_Tcenv_tc_init();Cyc_Tc_tc(_tmp52A,1,print_decls);});
# 1735
;_pop_handler();}else{void*_tmp249=(void*)Cyc_Core_get_exn_thrown();void*_tmp24B=_tmp249;_LL75: _LL76:
# 1738
(int)_throw((void*)({struct Cyc_NO_SUPPORT_exn_struct*_tmp24D=_cycalloc(sizeof(*_tmp24D));_tmp24D->tag=Cyc_NO_SUPPORT,({struct _fat_ptr _tmp52B=({const char*_tmp24C="can't typecheck acquired declarations";_tag_fat(_tmp24C,sizeof(char),38U);});_tmp24D->f1=_tmp52B;});_tmp24D;}));_LL74:;}}}
# 1742
{struct _tuple0 _stmttmp1F=({struct _tuple0 _tmp39C;_tmp39C.f1=print_decls,_tmp39C.f2=names;_tmp39C;});struct Cyc_List_List*_tmp24F;struct Cyc_List_List*_tmp24E;_LL7A: _tmp24E=_stmttmp1F.f1;_tmp24F=_stmttmp1F.f2;_LL7B: {struct Cyc_List_List*d=_tmp24E;struct Cyc_List_List*n=_tmp24F;for(0;
d != 0 && n != 0;(d=d->tl,n=n->tl)){
struct Cyc_Absyn_Decl*decl=(struct Cyc_Absyn_Decl*)d->hd;
struct _fat_ptr*name=(struct _fat_ptr*)n->hd;
int anon_enum=0;
if(!((unsigned)name))
anon_enum=1;
# 1747
Cyc_Absynpp_set_params(& Cyc_Absynpp_cyc_params_r);
# 1752
if((unsigned)name){
ifdefmacro=({struct Cyc_String_pa_PrintArg_struct _tmp252=({struct Cyc_String_pa_PrintArg_struct _tmp399;_tmp399.tag=0U,_tmp399.f1=(struct _fat_ptr)((struct _fat_ptr)*name);_tmp399;});void*_tmp250[1U];_tmp250[0]=& _tmp252;({struct _fat_ptr _tmp52C=({const char*_tmp251="_%s_def_";_tag_fat(_tmp251,sizeof(char),9U);});Cyc_aprintf(_tmp52C,_tag_fat(_tmp250,sizeof(void*),1U));});});
({struct Cyc_String_pa_PrintArg_struct _tmp255=({struct Cyc_String_pa_PrintArg_struct _tmp39A;_tmp39A.tag=0U,_tmp39A.f1=(struct _fat_ptr)((struct _fat_ptr)ifdefmacro);_tmp39A;});void*_tmp253[1U];_tmp253[0]=& _tmp255;({struct Cyc___cycFILE*_tmp52E=out_file;struct _fat_ptr _tmp52D=({const char*_tmp254="#ifndef %s\n";_tag_fat(_tmp254,sizeof(char),12U);});Cyc_fprintf(_tmp52E,_tmp52D,_tag_fat(_tmp253,sizeof(void*),1U));});});
({struct Cyc_String_pa_PrintArg_struct _tmp258=({struct Cyc_String_pa_PrintArg_struct _tmp39B;_tmp39B.tag=0U,_tmp39B.f1=(struct _fat_ptr)((struct _fat_ptr)ifdefmacro);_tmp39B;});void*_tmp256[1U];_tmp256[0]=& _tmp258;({struct Cyc___cycFILE*_tmp530=out_file;struct _fat_ptr _tmp52F=({const char*_tmp257="#define %s\n";_tag_fat(_tmp257,sizeof(char),12U);});Cyc_fprintf(_tmp530,_tmp52F,_tag_fat(_tmp256,sizeof(void*),1U));});});
# 1757
({struct Cyc_List_List*_tmp531=({struct Cyc_Absyn_Decl*_tmp259[1U];_tmp259[0]=decl;Cyc_List_list(_tag_fat(_tmp259,sizeof(struct Cyc_Absyn_Decl*),1U));});Cyc_Absynpp_decllist2file(_tmp531,out_file);});
({void*_tmp25A=0U;({struct Cyc___cycFILE*_tmp533=out_file;struct _fat_ptr _tmp532=({const char*_tmp25B="#endif\n";_tag_fat(_tmp25B,sizeof(char),8U);});Cyc_fprintf(_tmp533,_tmp532,_tag_fat(_tmp25A,sizeof(void*),0U));});});}else{
# 1762
({struct Cyc_List_List*_tmp534=({struct Cyc_Absyn_Decl*_tmp25C[1U];_tmp25C[0]=decl;Cyc_List_list(_tag_fat(_tmp25C,sizeof(struct Cyc_Absyn_Decl*),1U));});Cyc_Absynpp_decllist2file(_tmp534,out_file);});}}}}
# 1767
maybe=Cyc_fopen(macrosfile,"r");
if(!((unsigned)maybe))(int)_throw((void*)({struct Cyc_NO_SUPPORT_exn_struct*_tmp261=_cycalloc(sizeof(*_tmp261));_tmp261->tag=Cyc_NO_SUPPORT,({struct _fat_ptr _tmp537=(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp25F=({struct Cyc_String_pa_PrintArg_struct _tmp39D;_tmp39D.tag=0U,({struct _fat_ptr _tmp535=(struct _fat_ptr)({const char*_tmp260=macrosfile;_tag_fat(_tmp260,sizeof(char),_get_zero_arr_size_char((void*)_tmp260,1U));});_tmp39D.f1=_tmp535;});_tmp39D;});void*_tmp25D[1U];_tmp25D[0]=& _tmp25F;({struct _fat_ptr _tmp536=({const char*_tmp25E="can't open macrosfile %s";_tag_fat(_tmp25E,sizeof(char),25U);});Cyc_aprintf(_tmp536,_tag_fat(_tmp25D,sizeof(void*),1U));});});_tmp261->f1=_tmp537;});_tmp261;}));in_file=maybe;
# 1771
l=Cyc_Lexing_from_file(in_file);{
struct _tuple26*entry2;
while((entry2=Cyc_suck_line(l))!= 0){
struct _tuple26*_stmttmp20=(struct _tuple26*)_check_null(entry2);struct _fat_ptr*_tmp263;struct _fat_ptr _tmp262;_LL7D: _tmp262=_stmttmp20->f1;_tmp263=_stmttmp20->f2;_LL7E: {struct _fat_ptr line=_tmp262;struct _fat_ptr*name=_tmp263;
if(({int(*_tmp539)(struct Cyc_Set_Set*s,struct _fat_ptr*elt)=({int(*_tmp264)(struct Cyc_Set_Set*s,struct _fat_ptr*elt)=(int(*)(struct Cyc_Set_Set*s,struct _fat_ptr*elt))Cyc_Set_member;_tmp264;});struct Cyc_Set_Set*_tmp538=reachable_set;_tmp539(_tmp538,name);})){
({struct Cyc_String_pa_PrintArg_struct _tmp267=({struct Cyc_String_pa_PrintArg_struct _tmp39E;_tmp39E.tag=0U,_tmp39E.f1=(struct _fat_ptr)((struct _fat_ptr)*name);_tmp39E;});void*_tmp265[1U];_tmp265[0]=& _tmp267;({struct Cyc___cycFILE*_tmp53B=out_file;struct _fat_ptr _tmp53A=({const char*_tmp266="#ifndef %s\n";_tag_fat(_tmp266,sizeof(char),12U);});Cyc_fprintf(_tmp53B,_tmp53A,_tag_fat(_tmp265,sizeof(void*),1U));});});
({struct Cyc_String_pa_PrintArg_struct _tmp26A=({struct Cyc_String_pa_PrintArg_struct _tmp39F;_tmp39F.tag=0U,_tmp39F.f1=(struct _fat_ptr)((struct _fat_ptr)line);_tmp39F;});void*_tmp268[1U];_tmp268[0]=& _tmp26A;({struct Cyc___cycFILE*_tmp53D=out_file;struct _fat_ptr _tmp53C=({const char*_tmp269="%s\n";_tag_fat(_tmp269,sizeof(char),4U);});Cyc_fprintf(_tmp53D,_tmp53C,_tag_fat(_tmp268,sizeof(void*),1U));});});
({void*_tmp26B=0U;({struct Cyc___cycFILE*_tmp53F=out_file;struct _fat_ptr _tmp53E=({const char*_tmp26C="#endif\n";_tag_fat(_tmp26C,sizeof(char),8U);});Cyc_fprintf(_tmp53F,_tmp53E,_tag_fat(_tmp26B,sizeof(void*),0U));});});}}}
# 1781
Cyc_fclose(in_file);
if((int)Cyc_mode != (int)Cyc_FINISH);if(hstubs != 0){
# 1785
struct Cyc_List_List*x=hstubs;for(0;x != 0;x=x->tl){
struct _tuple27*_stmttmp21=(struct _tuple27*)x->hd;struct _fat_ptr _tmp26E;struct _fat_ptr _tmp26D;_LL80: _tmp26D=_stmttmp21->f1;_tmp26E=_stmttmp21->f2;_LL81: {struct _fat_ptr symbol=_tmp26D;struct _fat_ptr text=_tmp26E;
if(({char*_tmp543=(char*)text.curr;_tmp543 != (char*)(_tag_fat(0,0,0)).curr;})&&(
({char*_tmp542=(char*)symbol.curr;_tmp542 == (char*)(_tag_fat(0,0,0)).curr;})||({int(*_tmp541)(struct Cyc_Set_Set*s,struct _fat_ptr*elt)=({int(*_tmp26F)(struct Cyc_Set_Set*s,struct _fat_ptr*elt)=(int(*)(struct Cyc_Set_Set*s,struct _fat_ptr*elt))Cyc_Set_member;_tmp26F;});struct Cyc_Set_Set*_tmp540=defined_symbols;_tmp541(_tmp540,({struct _fat_ptr*_tmp270=_cycalloc(sizeof(*_tmp270));*_tmp270=symbol;_tmp270;}));})))
# 1790
Cyc_fputs((const char*)_untag_fat_ptr(text,sizeof(char),1U),out_file);else{
# 1792
({struct Cyc_String_pa_PrintArg_struct _tmp273=({struct Cyc_String_pa_PrintArg_struct _tmp3A0;_tmp3A0.tag=0U,_tmp3A0.f1=(struct _fat_ptr)((struct _fat_ptr)symbol);_tmp3A0;});void*_tmp271[1U];_tmp271[0]=& _tmp273;({struct _fat_ptr _tmp544=({const char*_tmp272="%s is not supported on this platform\n";_tag_fat(_tmp272,sizeof(char),38U);});Cyc_log(_tmp544,_tag_fat(_tmp271,sizeof(void*),1U));});});}}}}
# 1782
({void*_tmp274=0U;({struct Cyc___cycFILE*_tmp546=out_file;struct _fat_ptr _tmp545=({const char*_tmp275="#endif\n";_tag_fat(_tmp275,sizeof(char),8U);});Cyc_fprintf(_tmp546,_tmp545,_tag_fat(_tmp274,sizeof(void*),0U));});});
# 1796
if(Cyc_do_setjmp){int _tmp276=0;_npop_handler(0U);return _tmp276;}else{
Cyc_fclose(out_file);}
# 1800
if(cstubs != 0){
out_file=(struct Cyc___cycFILE*)_check_null(Cyc_cstubs_file);{
struct Cyc_List_List*x=cstubs;for(0;x != 0;x=x->tl){
struct _tuple27*_stmttmp22=(struct _tuple27*)x->hd;struct _fat_ptr _tmp278;struct _fat_ptr _tmp277;_LL83: _tmp277=_stmttmp22->f1;_tmp278=_stmttmp22->f2;_LL84: {struct _fat_ptr symbol=_tmp277;struct _fat_ptr text=_tmp278;
if(({char*_tmp54A=(char*)text.curr;_tmp54A != (char*)(_tag_fat(0,0,0)).curr;})&&(
({char*_tmp549=(char*)symbol.curr;_tmp549 == (char*)(_tag_fat(0,0,0)).curr;})||({int(*_tmp548)(struct Cyc_Set_Set*s,struct _fat_ptr*elt)=({int(*_tmp279)(struct Cyc_Set_Set*s,struct _fat_ptr*elt)=(int(*)(struct Cyc_Set_Set*s,struct _fat_ptr*elt))Cyc_Set_member;_tmp279;});struct Cyc_Set_Set*_tmp547=defined_symbols;_tmp548(_tmp547,({struct _fat_ptr*_tmp27A=_cycalloc(sizeof(*_tmp27A));*_tmp27A=symbol;_tmp27A;}));})))
Cyc_fputs((const char*)_untag_fat_ptr(text,sizeof(char),1U),out_file);}}}}
# 1800
out_file=(struct Cyc___cycFILE*)_check_null(Cyc_cycstubs_file);
# 1813
({struct Cyc_String_pa_PrintArg_struct _tmp27D=({struct Cyc_String_pa_PrintArg_struct _tmp3A1;_tmp3A1.tag=0U,({struct _fat_ptr _tmp54B=(struct _fat_ptr)({const char*_tmp27E=filename;_tag_fat(_tmp27E,sizeof(char),_get_zero_arr_size_char((void*)_tmp27E,1U));});_tmp3A1.f1=_tmp54B;});_tmp3A1;});void*_tmp27B[1U];_tmp27B[0]=& _tmp27D;({struct Cyc___cycFILE*_tmp54D=out_file;struct _fat_ptr _tmp54C=({const char*_tmp27C="#include <%s>\n\n";_tag_fat(_tmp27C,sizeof(char),16U);});Cyc_fprintf(_tmp54D,_tmp54C,_tag_fat(_tmp27B,sizeof(void*),1U));});});
if(cycstubs != 0){
out_file=(struct Cyc___cycFILE*)_check_null(Cyc_cycstubs_file);
{struct Cyc_List_List*x=cycstubs;for(0;x != 0;x=x->tl){
struct _tuple27*_stmttmp23=(struct _tuple27*)x->hd;struct _fat_ptr _tmp280;struct _fat_ptr _tmp27F;_LL86: _tmp27F=_stmttmp23->f1;_tmp280=_stmttmp23->f2;_LL87: {struct _fat_ptr symbol=_tmp27F;struct _fat_ptr text=_tmp280;
if(({char*_tmp551=(char*)text.curr;_tmp551 != (char*)(_tag_fat(0,0,0)).curr;})&&(
({char*_tmp550=(char*)symbol.curr;_tmp550 == (char*)(_tag_fat(0,0,0)).curr;})||({int(*_tmp54F)(struct Cyc_Set_Set*s,struct _fat_ptr*elt)=({int(*_tmp281)(struct Cyc_Set_Set*s,struct _fat_ptr*elt)=(int(*)(struct Cyc_Set_Set*s,struct _fat_ptr*elt))Cyc_Set_member;_tmp281;});struct Cyc_Set_Set*_tmp54E=defined_symbols;_tmp54F(_tmp54E,({struct _fat_ptr*_tmp282=_cycalloc(sizeof(*_tmp282));*_tmp282=symbol;_tmp282;}));})))
Cyc_fputs((const char*)_untag_fat_ptr(text,sizeof(char),1U),out_file);}}}
# 1822
({void*_tmp283=0U;({struct Cyc___cycFILE*_tmp553=out_file;struct _fat_ptr _tmp552=({const char*_tmp284="\n";_tag_fat(_tmp284,sizeof(char),2U);});Cyc_fprintf(_tmp553,_tmp552,_tag_fat(_tmp283,sizeof(void*),0U));});});}{
# 1814
int _tmp285=0;_npop_handler(0U);return _tmp285;}}}}}}}}}
# 1458
;_pop_handler();}else{void*_tmp1AE=(void*)Cyc_Core_get_exn_thrown();void*_tmp28D=_tmp1AE;void*_tmp28E;struct _fat_ptr _tmp28F;struct _fat_ptr _tmp290;struct _fat_ptr _tmp291;struct _fat_ptr _tmp292;if(((struct Cyc_Core_Impossible_exn_struct*)_tmp28D)->tag == Cyc_Core_Impossible){_LL89: _tmp292=((struct Cyc_Core_Impossible_exn_struct*)_tmp28D)->f1;_LL8A: {struct _fat_ptr s=_tmp292;
# 1829
({struct Cyc_String_pa_PrintArg_struct _tmp295=({struct Cyc_String_pa_PrintArg_struct _tmp3A3;_tmp3A3.tag=0U,_tmp3A3.f1=(struct _fat_ptr)((struct _fat_ptr)s);_tmp3A3;});void*_tmp293[1U];_tmp293[0]=& _tmp295;({struct Cyc___cycFILE*_tmp555=Cyc_stderr;struct _fat_ptr _tmp554=({const char*_tmp294="Got Core::Impossible(%s)\n";_tag_fat(_tmp294,sizeof(char),26U);});Cyc_fprintf(_tmp555,_tmp554,_tag_fat(_tmp293,sizeof(void*),1U));});});goto _LL88;}}else{if(((struct Cyc_Dict_Absent_exn_struct*)_tmp28D)->tag == Cyc_Dict_Absent){_LL8B: _LL8C:
# 1831
({void*_tmp296=0U;({struct Cyc___cycFILE*_tmp557=Cyc_stderr;struct _fat_ptr _tmp556=({const char*_tmp297="Got Dict::Absent\n";_tag_fat(_tmp297,sizeof(char),18U);});Cyc_fprintf(_tmp557,_tmp556,_tag_fat(_tmp296,sizeof(void*),0U));});});goto _LL88;}else{if(((struct Cyc_Core_Failure_exn_struct*)_tmp28D)->tag == Cyc_Core_Failure){_LL8D: _tmp291=((struct Cyc_Core_Failure_exn_struct*)_tmp28D)->f1;_LL8E: {struct _fat_ptr s=_tmp291;
# 1833
({struct Cyc_String_pa_PrintArg_struct _tmp29A=({struct Cyc_String_pa_PrintArg_struct _tmp3A5;_tmp3A5.tag=0U,({
struct _fat_ptr _tmp558=(struct _fat_ptr)({const char*_tmp29C=Cyc_Core_get_exn_filename();_tag_fat(_tmp29C,sizeof(char),_get_zero_arr_size_char((void*)_tmp29C,1U));});_tmp3A5.f1=_tmp558;});_tmp3A5;});struct Cyc_Int_pa_PrintArg_struct _tmp29B=({struct Cyc_Int_pa_PrintArg_struct _tmp3A4;_tmp3A4.tag=1U,({unsigned long _tmp559=(unsigned long)Cyc_Core_get_exn_lineno();_tmp3A4.f1=_tmp559;});_tmp3A4;});void*_tmp298[2U];_tmp298[0]=& _tmp29A,_tmp298[1]=& _tmp29B;({struct Cyc___cycFILE*_tmp55B=Cyc_stderr;struct _fat_ptr _tmp55A=({const char*_tmp299="exception from file %s, line %d was thrown\n";_tag_fat(_tmp299,sizeof(char),44U);});Cyc_fprintf(_tmp55B,_tmp55A,_tag_fat(_tmp298,sizeof(void*),2U));});});
({struct Cyc_String_pa_PrintArg_struct _tmp29F=({struct Cyc_String_pa_PrintArg_struct _tmp3A6;_tmp3A6.tag=0U,_tmp3A6.f1=(struct _fat_ptr)((struct _fat_ptr)s);_tmp3A6;});void*_tmp29D[1U];_tmp29D[0]=& _tmp29F;({struct Cyc___cycFILE*_tmp55D=Cyc_stderr;struct _fat_ptr _tmp55C=({const char*_tmp29E="Got Core::Failure(%s)\n";_tag_fat(_tmp29E,sizeof(char),23U);});Cyc_fprintf(_tmp55D,_tmp55C,_tag_fat(_tmp29D,sizeof(void*),1U));});});goto _LL88;}}else{if(((struct Cyc_Core_Invalid_argument_exn_struct*)_tmp28D)->tag == Cyc_Core_Invalid_argument){_LL8F: _tmp290=((struct Cyc_Core_Invalid_argument_exn_struct*)_tmp28D)->f1;_LL90: {struct _fat_ptr s=_tmp290;
# 1837
({struct Cyc_String_pa_PrintArg_struct _tmp2A2=({struct Cyc_String_pa_PrintArg_struct _tmp3A7;_tmp3A7.tag=0U,_tmp3A7.f1=(struct _fat_ptr)((struct _fat_ptr)s);_tmp3A7;});void*_tmp2A0[1U];_tmp2A0[0]=& _tmp2A2;({struct Cyc___cycFILE*_tmp55F=Cyc_stderr;struct _fat_ptr _tmp55E=({const char*_tmp2A1="Got Invalid_argument(%s)\n";_tag_fat(_tmp2A1,sizeof(char),26U);});Cyc_fprintf(_tmp55F,_tmp55E,_tag_fat(_tmp2A0,sizeof(void*),1U));});});goto _LL88;}}else{if(((struct Cyc_Core_Not_found_exn_struct*)_tmp28D)->tag == Cyc_Core_Not_found){_LL91: _LL92:
# 1839
({void*_tmp2A3=0U;({struct Cyc___cycFILE*_tmp561=Cyc_stderr;struct _fat_ptr _tmp560=({const char*_tmp2A4="Got Not_found\n";_tag_fat(_tmp2A4,sizeof(char),15U);});Cyc_fprintf(_tmp561,_tmp560,_tag_fat(_tmp2A3,sizeof(void*),0U));});});goto _LL88;}else{if(((struct Cyc_NO_SUPPORT_exn_struct*)_tmp28D)->tag == Cyc_NO_SUPPORT){_LL93: _tmp28F=((struct Cyc_NO_SUPPORT_exn_struct*)_tmp28D)->f1;_LL94: {struct _fat_ptr s=_tmp28F;
# 1841
({struct Cyc_String_pa_PrintArg_struct _tmp2A7=({struct Cyc_String_pa_PrintArg_struct _tmp3A8;_tmp3A8.tag=0U,_tmp3A8.f1=(struct _fat_ptr)((struct _fat_ptr)s);_tmp3A8;});void*_tmp2A5[1U];_tmp2A5[0]=& _tmp2A7;({struct Cyc___cycFILE*_tmp563=Cyc_stderr;struct _fat_ptr _tmp562=({const char*_tmp2A6="No support because %s\n";_tag_fat(_tmp2A6,sizeof(char),23U);});Cyc_fprintf(_tmp563,_tmp562,_tag_fat(_tmp2A5,sizeof(void*),1U));});});goto _LL88;}}else{_LL95: _tmp28E=_tmp28D;_LL96: {void*x=_tmp28E;
# 1843
({void*_tmp2A8=0U;({struct Cyc___cycFILE*_tmp565=Cyc_stderr;struct _fat_ptr _tmp564=({const char*_tmp2A9="Got unknown exception\n";_tag_fat(_tmp2A9,sizeof(char),23U);});Cyc_fprintf(_tmp565,_tmp564,_tag_fat(_tmp2A8,sizeof(void*),0U));});});
Cyc_Core_rethrow(x);}}}}}}}_LL88:;}}}
# 1849
maybe=Cyc_fopen(filename,"w");
if(!((unsigned)maybe)){
({struct Cyc_String_pa_PrintArg_struct _tmp2AC=({struct Cyc_String_pa_PrintArg_struct _tmp3A9;_tmp3A9.tag=0U,({struct _fat_ptr _tmp566=(struct _fat_ptr)({const char*_tmp2AD=filename;_tag_fat(_tmp2AD,sizeof(char),_get_zero_arr_size_char((void*)_tmp2AD,1U));});_tmp3A9.f1=_tmp566;});_tmp3A9;});void*_tmp2AA[1U];_tmp2AA[0]=& _tmp2AC;({struct Cyc___cycFILE*_tmp568=Cyc_stderr;struct _fat_ptr _tmp567=({const char*_tmp2AB="Error: could not create file %s\n";_tag_fat(_tmp2AB,sizeof(char),33U);});Cyc_fprintf(_tmp568,_tmp567,_tag_fat(_tmp2AA,sizeof(void*),1U));});});
return 1;}
# 1850
out_file=maybe;
# 1855
({struct Cyc_String_pa_PrintArg_struct _tmp2B0=({struct Cyc_String_pa_PrintArg_struct _tmp3AA;_tmp3AA.tag=0U,({
# 1857
struct _fat_ptr _tmp569=(struct _fat_ptr)({const char*_tmp2B1=filename;_tag_fat(_tmp2B1,sizeof(char),_get_zero_arr_size_char((void*)_tmp2B1,1U));});_tmp3AA.f1=_tmp569;});_tmp3AA;});void*_tmp2AE[1U];_tmp2AE[0]=& _tmp2B0;({struct Cyc___cycFILE*_tmp56B=out_file;struct _fat_ptr _tmp56A=({const char*_tmp2AF="#error -- %s is not supported on this platform\n";_tag_fat(_tmp2AF,sizeof(char),48U);});Cyc_fprintf(_tmp56B,_tmp56A,_tag_fat(_tmp2AE,sizeof(void*),1U));});});
Cyc_fclose(out_file);
({struct Cyc_String_pa_PrintArg_struct _tmp2B4=({struct Cyc_String_pa_PrintArg_struct _tmp3AB;_tmp3AB.tag=0U,({
struct _fat_ptr _tmp56C=(struct _fat_ptr)({const char*_tmp2B5=filename;_tag_fat(_tmp2B5,sizeof(char),_get_zero_arr_size_char((void*)_tmp2B5,1U));});_tmp3AB.f1=_tmp56C;});_tmp3AB;});void*_tmp2B2[1U];_tmp2B2[0]=& _tmp2B4;({struct Cyc___cycFILE*_tmp56E=Cyc_stderr;struct _fat_ptr _tmp56D=({const char*_tmp2B3="Warning: %s will not be supported on this platform\n";_tag_fat(_tmp2B3,sizeof(char),52U);});Cyc_fprintf(_tmp56E,_tmp56D,_tag_fat(_tmp2B2,sizeof(void*),1U));});});
({void*_tmp2B6=0U;({struct _fat_ptr _tmp56F=({const char*_tmp2B7="Not supported on this platform\n";_tag_fat(_tmp2B7,sizeof(char),32U);});Cyc_log(_tmp56F,_tag_fat(_tmp2B6,sizeof(void*),0U));});});
# 1869
return 0;}}
# 1348 "buildlib.cyl"
int Cyc_process_specfile(const char*file,const char*dir){
# 1874 "buildlib.cyl"
if(Cyc_verbose)
({struct Cyc_String_pa_PrintArg_struct _tmp2CF=({struct Cyc_String_pa_PrintArg_struct _tmp3B2;_tmp3B2.tag=0U,({struct _fat_ptr _tmp570=(struct _fat_ptr)({const char*_tmp2D0=file;_tag_fat(_tmp2D0,sizeof(char),_get_zero_arr_size_char((void*)_tmp2D0,1U));});_tmp3B2.f1=_tmp570;});_tmp3B2;});void*_tmp2CD[1U];_tmp2CD[0]=& _tmp2CF;({struct Cyc___cycFILE*_tmp572=Cyc_stderr;struct _fat_ptr _tmp571=({const char*_tmp2CE="Processing %s\n";_tag_fat(_tmp2CE,sizeof(char),15U);});Cyc_fprintf(_tmp572,_tmp571,_tag_fat(_tmp2CD,sizeof(void*),1U));});});{
# 1874
struct Cyc___cycFILE*maybe=
# 1876
Cyc_fopen(file,"r");
if(!((unsigned)maybe)){
({struct Cyc_String_pa_PrintArg_struct _tmp2D3=({struct Cyc_String_pa_PrintArg_struct _tmp3B3;_tmp3B3.tag=0U,({struct _fat_ptr _tmp573=(struct _fat_ptr)({const char*_tmp2D4=file;_tag_fat(_tmp2D4,sizeof(char),_get_zero_arr_size_char((void*)_tmp2D4,1U));});_tmp3B3.f1=_tmp573;});_tmp3B3;});void*_tmp2D1[1U];_tmp2D1[0]=& _tmp2D3;({struct Cyc___cycFILE*_tmp575=Cyc_stderr;struct _fat_ptr _tmp574=({const char*_tmp2D2="Error: could not open %s\n";_tag_fat(_tmp2D2,sizeof(char),26U);});Cyc_fprintf(_tmp575,_tmp574,_tag_fat(_tmp2D1,sizeof(void*),1U));});});
return 1;}{
# 1877
struct Cyc___cycFILE*in_file=maybe;
# 1885
struct _fat_ptr buf=({char*_tmp2ED=({unsigned _tmp2EC=1024U + 1U;char*_tmp2EB=_cycalloc_atomic(_check_times(_tmp2EC,sizeof(char)));({{unsigned _tmp3B9=1024U;unsigned i;for(i=0;i < _tmp3B9;++ i){_tmp2EB[i]='\000';}_tmp2EB[_tmp3B9]=0;}0;});_tmp2EB;});_tag_fat(_tmp2ED,sizeof(char),1025U);});
struct _fat_ptr cwd=Cyc_getcwd(buf,_get_fat_size(buf,sizeof(char)));
if((int)Cyc_mode != (int)Cyc_GATHERSCRIPT){
if(chdir(dir)){
({struct Cyc_String_pa_PrintArg_struct _tmp2D7=({struct Cyc_String_pa_PrintArg_struct _tmp3B4;_tmp3B4.tag=0U,({struct _fat_ptr _tmp576=(struct _fat_ptr)({const char*_tmp2D8=dir;_tag_fat(_tmp2D8,sizeof(char),_get_zero_arr_size_char((void*)_tmp2D8,1U));});_tmp3B4.f1=_tmp576;});_tmp3B4;});void*_tmp2D5[1U];_tmp2D5[0]=& _tmp2D7;({struct Cyc___cycFILE*_tmp578=Cyc_stderr;struct _fat_ptr _tmp577=({const char*_tmp2D6="Error: can't change directory to %s\n";_tag_fat(_tmp2D6,sizeof(char),37U);});Cyc_fprintf(_tmp578,_tmp577,_tag_fat(_tmp2D5,sizeof(void*),1U));});});
return 1;}}
# 1887
if((int)Cyc_mode == (int)Cyc_GATHER){
# 1895
struct _fat_ptr cmd=({struct Cyc_String_pa_PrintArg_struct _tmp2DE=({struct Cyc_String_pa_PrintArg_struct _tmp3B7;_tmp3B7.tag=0U,_tmp3B7.f1=(struct _fat_ptr)((struct _fat_ptr)Cyc_cyclone_cc);_tmp3B7;});struct Cyc_String_pa_PrintArg_struct _tmp2DF=({struct Cyc_String_pa_PrintArg_struct _tmp3B6;_tmp3B6.tag=0U,_tmp3B6.f1=(struct _fat_ptr)((struct _fat_ptr)Cyc_target_cflags);_tmp3B6;});void*_tmp2DC[2U];_tmp2DC[0]=& _tmp2DE,_tmp2DC[1]=& _tmp2DF;({struct _fat_ptr _tmp579=({const char*_tmp2DD="echo | %s %s -E -dM - -o INITMACROS.h\n";_tag_fat(_tmp2DD,sizeof(char),39U);});Cyc_aprintf(_tmp579,_tag_fat(_tmp2DC,sizeof(void*),2U));});});
# 1897
if(Cyc_verbose)
({struct Cyc_String_pa_PrintArg_struct _tmp2DB=({struct Cyc_String_pa_PrintArg_struct _tmp3B5;_tmp3B5.tag=0U,_tmp3B5.f1=(struct _fat_ptr)((struct _fat_ptr)cmd);_tmp3B5;});void*_tmp2D9[1U];_tmp2D9[0]=& _tmp2DB;({struct Cyc___cycFILE*_tmp57B=Cyc_stderr;struct _fat_ptr _tmp57A=({const char*_tmp2DA="%s\n";_tag_fat(_tmp2DA,sizeof(char),4U);});Cyc_fprintf(_tmp57B,_tmp57A,_tag_fat(_tmp2D9,sizeof(void*),1U));});});
# 1897
system((const char*)_check_null(_untag_fat_ptr(cmd,sizeof(char),1U)));}{
# 1887
struct Cyc_Lexing_lexbuf*l=
# 1902
Cyc_Lexing_from_file(in_file);
struct _tuple29*entry;
while((entry=Cyc_spec(l))!= 0){
struct _tuple29*_stmttmp24=(struct _tuple29*)_check_null(entry);struct Cyc_List_List*_tmp2E7;struct Cyc_List_List*_tmp2E6;struct Cyc_List_List*_tmp2E5;struct Cyc_List_List*_tmp2E4;struct Cyc_List_List*_tmp2E3;struct Cyc_List_List*_tmp2E2;struct Cyc_List_List*_tmp2E1;struct _fat_ptr _tmp2E0;_LL1: _tmp2E0=_stmttmp24->f1;_tmp2E1=_stmttmp24->f2;_tmp2E2=_stmttmp24->f3;_tmp2E3=_stmttmp24->f4;_tmp2E4=_stmttmp24->f5;_tmp2E5=_stmttmp24->f6;_tmp2E6=_stmttmp24->f7;_tmp2E7=_stmttmp24->f8;_LL2: {struct _fat_ptr headerfile=_tmp2E0;struct Cyc_List_List*start_symbols=_tmp2E1;struct Cyc_List_List*user_defs=_tmp2E2;struct Cyc_List_List*omit_symbols=_tmp2E3;struct Cyc_List_List*hstubs=_tmp2E4;struct Cyc_List_List*cstubs=_tmp2E5;struct Cyc_List_List*cycstubs=_tmp2E6;struct Cyc_List_List*cpp_insert=_tmp2E7;
# 1907
if(Cyc_process_file((const char*)_check_null(_untag_fat_ptr(headerfile,sizeof(char),1U)),start_symbols,user_defs,omit_symbols,hstubs,cstubs,cycstubs,cpp_insert))
# 1909
return 1;}}
# 1911
Cyc_fclose(in_file);
# 1913
if((int)Cyc_mode != (int)Cyc_GATHERSCRIPT){
if(chdir((const char*)((char*)_check_null(_untag_fat_ptr(cwd,sizeof(char),1U))))){
({struct Cyc_String_pa_PrintArg_struct _tmp2EA=({struct Cyc_String_pa_PrintArg_struct _tmp3B8;_tmp3B8.tag=0U,_tmp3B8.f1=(struct _fat_ptr)((struct _fat_ptr)cwd);_tmp3B8;});void*_tmp2E8[1U];_tmp2E8[0]=& _tmp2EA;({struct Cyc___cycFILE*_tmp57D=Cyc_stderr;struct _fat_ptr _tmp57C=({const char*_tmp2E9="Error: could not change directory to %s\n";_tag_fat(_tmp2E9,sizeof(char),41U);});Cyc_fprintf(_tmp57D,_tmp57C,_tag_fat(_tmp2E8,sizeof(void*),1U));});});
return 1;}}
# 1913
return 0;}}}}
# 1348 "buildlib.cyl"
int Cyc_process_setjmp(const char*dir){
# 1926 "buildlib.cyl"
struct _fat_ptr buf=({char*_tmp2FE=({unsigned _tmp2FD=1024U + 1U;char*_tmp2FC=_cycalloc_atomic(_check_times(_tmp2FD,sizeof(char)));({{unsigned _tmp3BC=1024U;unsigned i;for(i=0;i < _tmp3BC;++ i){_tmp2FC[i]='\000';}_tmp2FC[_tmp3BC]=0;}0;});_tmp2FC;});_tag_fat(_tmp2FE,sizeof(char),1025U);});
struct _fat_ptr cwd=Cyc_getcwd(buf,_get_fat_size(buf,sizeof(char)));
if(chdir(dir)){
({struct Cyc_String_pa_PrintArg_struct _tmp2F0=({struct Cyc_String_pa_PrintArg_struct _tmp3BA;_tmp3BA.tag=0U,({struct _fat_ptr _tmp57E=(struct _fat_ptr)({const char*_tmp2F1=dir;_tag_fat(_tmp2F1,sizeof(char),_get_zero_arr_size_char((void*)_tmp2F1,1U));});_tmp3BA.f1=_tmp57E;});_tmp3BA;});void*_tmp2EE[1U];_tmp2EE[0]=& _tmp2F0;({struct Cyc___cycFILE*_tmp580=Cyc_stderr;struct _fat_ptr _tmp57F=({const char*_tmp2EF="Error: can't change directory to %s\n";_tag_fat(_tmp2EF,sizeof(char),37U);});Cyc_fprintf(_tmp580,_tmp57F,_tag_fat(_tmp2EE,sizeof(void*),1U));});});
return 1;}
# 1928
if(({
# 1932
struct Cyc_List_List*_tmp586=({struct _fat_ptr*_tmp2F2[1U];({struct _fat_ptr*_tmp582=({struct _fat_ptr*_tmp2F4=_cycalloc(sizeof(*_tmp2F4));({struct _fat_ptr _tmp581=({const char*_tmp2F3="jmp_buf";_tag_fat(_tmp2F3,sizeof(char),8U);});*_tmp2F4=_tmp581;});_tmp2F4;});_tmp2F2[0]=_tmp582;});Cyc_List_list(_tag_fat(_tmp2F2,sizeof(struct _fat_ptr*),1U));});Cyc_process_file("setjmp.h",_tmp586,0,0,({struct _tuple27*_tmp2F5[1U];({struct _tuple27*_tmp585=({struct _tuple27*_tmp2F8=_cycalloc(sizeof(*_tmp2F8));
({struct _fat_ptr _tmp584=({const char*_tmp2F6="setjmp";_tag_fat(_tmp2F6,sizeof(char),7U);});_tmp2F8->f1=_tmp584;}),({struct _fat_ptr _tmp583=({const char*_tmp2F7="extern int setjmp(jmp_buf);\n";_tag_fat(_tmp2F7,sizeof(char),29U);});_tmp2F8->f2=_tmp583;});_tmp2F8;});_tmp2F5[0]=_tmp585;});Cyc_List_list(_tag_fat(_tmp2F5,sizeof(struct _tuple27*),1U));}),0,0,0);}))
# 1935
return 1;
# 1928
if(
# 1936
chdir((const char*)((char*)_check_null(_untag_fat_ptr(cwd,sizeof(char),1U))))){
({struct Cyc_String_pa_PrintArg_struct _tmp2FB=({struct Cyc_String_pa_PrintArg_struct _tmp3BB;_tmp3BB.tag=0U,_tmp3BB.f1=(struct _fat_ptr)((struct _fat_ptr)cwd);_tmp3BB;});void*_tmp2F9[1U];_tmp2F9[0]=& _tmp2FB;({struct Cyc___cycFILE*_tmp588=Cyc_stderr;struct _fat_ptr _tmp587=({const char*_tmp2FA="Error: could not change directory to %s\n";_tag_fat(_tmp2FA,sizeof(char),41U);});Cyc_fprintf(_tmp588,_tmp587,_tag_fat(_tmp2F9,sizeof(void*),1U));});});
return 1;}
# 1928
return 0;}static char _tmp2FF[13U]="BUILDLIB.OUT";
# 1348 "buildlib.cyl"
static struct _fat_ptr Cyc_output_dir={_tmp2FF,_tmp2FF,_tmp2FF + 13U};
# 1945 "buildlib.cyl"
static void Cyc_set_output_dir(struct _fat_ptr s){
Cyc_output_dir=s;}
# 1945
static struct Cyc_List_List*Cyc_spec_files=0;
# 1949
static void Cyc_add_spec_file(struct _fat_ptr s){
Cyc_spec_files=({struct Cyc_List_List*_tmp300=_cycalloc(sizeof(*_tmp300));_tmp300->hd=(const char*)_check_null(_untag_fat_ptr(s,sizeof(char),1U)),_tmp300->tl=Cyc_spec_files;_tmp300;});}
# 1949
static int Cyc_no_other(struct _fat_ptr s){
# 1952
return 0;}
# 1949
static void Cyc_set_GATHER(){
# 1954
Cyc_mode=Cyc_GATHER;}
# 1949
static void Cyc_set_GATHERSCRIPT(){
# 1957
Cyc_mode=Cyc_GATHERSCRIPT;}
# 1949
static void Cyc_set_FINISH(){
# 1960
Cyc_mode=Cyc_FINISH;}
# 1949
static void Cyc_add_cpparg(struct _fat_ptr s){
# 1963
Cyc_cppargs=({struct Cyc_List_List*_tmp302=_cycalloc(sizeof(*_tmp302));({struct _fat_ptr*_tmp589=({struct _fat_ptr*_tmp301=_cycalloc(sizeof(*_tmp301));*_tmp301=s;_tmp301;});_tmp302->hd=_tmp589;}),_tmp302->tl=Cyc_cppargs;_tmp302;});}
# 1949
static int Cyc_badparse=0;
# 1966
static void Cyc_unsupported_option(struct _fat_ptr s){
({struct Cyc_String_pa_PrintArg_struct _tmp305=({struct Cyc_String_pa_PrintArg_struct _tmp3BD;_tmp3BD.tag=0U,_tmp3BD.f1=(struct _fat_ptr)((struct _fat_ptr)s);_tmp3BD;});void*_tmp303[1U];_tmp303[0]=& _tmp305;({struct Cyc___cycFILE*_tmp58B=Cyc_stderr;struct _fat_ptr _tmp58A=({const char*_tmp304="Unsupported option %s\n";_tag_fat(_tmp304,sizeof(char),23U);});Cyc_fprintf(_tmp58B,_tmp58A,_tag_fat(_tmp303,sizeof(void*),1U));});});
Cyc_badparse=1;}
# 1974
void GC_blacklist_warn_clear();struct _tuple31{struct _fat_ptr f1;int f2;struct _fat_ptr f3;void*f4;struct _fat_ptr f5;};
int Cyc_main(int argc,struct _fat_ptr argv){
GC_blacklist_warn_clear();{
# 1978
struct Cyc_List_List*options=({struct _tuple31*_tmp33A[9U];({
struct _tuple31*_tmp5B8=({struct _tuple31*_tmp33F=_cycalloc(sizeof(*_tmp33F));({struct _fat_ptr _tmp5B7=({const char*_tmp33B="-d";_tag_fat(_tmp33B,sizeof(char),3U);});_tmp33F->f1=_tmp5B7;}),_tmp33F->f2=0,({struct _fat_ptr _tmp5B6=({const char*_tmp33C=" <file>";_tag_fat(_tmp33C,sizeof(char),8U);});_tmp33F->f3=_tmp5B6;}),({
void*_tmp5B5=(void*)({struct Cyc_Arg_String_spec_Arg_Spec_struct*_tmp33D=_cycalloc(sizeof(*_tmp33D));_tmp33D->tag=5U,_tmp33D->f1=Cyc_set_output_dir;_tmp33D;});_tmp33F->f4=_tmp5B5;}),({
struct _fat_ptr _tmp5B4=({const char*_tmp33E="Set the output directory to <file>";_tag_fat(_tmp33E,sizeof(char),35U);});_tmp33F->f5=_tmp5B4;});_tmp33F;});
# 1979
_tmp33A[0]=_tmp5B8;}),({
# 1982
struct _tuple31*_tmp5B3=({struct _tuple31*_tmp344=_cycalloc(sizeof(*_tmp344));({struct _fat_ptr _tmp5B2=({const char*_tmp340="-gather";_tag_fat(_tmp340,sizeof(char),8U);});_tmp344->f1=_tmp5B2;}),_tmp344->f2=0,({struct _fat_ptr _tmp5B1=({const char*_tmp341="";_tag_fat(_tmp341,sizeof(char),1U);});_tmp344->f3=_tmp5B1;}),({
void*_tmp5B0=(void*)({struct Cyc_Arg_Unit_spec_Arg_Spec_struct*_tmp342=_cycalloc(sizeof(*_tmp342));_tmp342->tag=0U,_tmp342->f1=Cyc_set_GATHER;_tmp342;});_tmp344->f4=_tmp5B0;}),({
struct _fat_ptr _tmp5AF=({const char*_tmp343="Gather C library info but don't produce Cyclone headers";_tag_fat(_tmp343,sizeof(char),56U);});_tmp344->f5=_tmp5AF;});_tmp344;});
# 1982
_tmp33A[1]=_tmp5B3;}),({
# 1985
struct _tuple31*_tmp5AE=({struct _tuple31*_tmp349=_cycalloc(sizeof(*_tmp349));({struct _fat_ptr _tmp5AD=({const char*_tmp345="-gatherscript";_tag_fat(_tmp345,sizeof(char),14U);});_tmp349->f1=_tmp5AD;}),_tmp349->f2=0,({struct _fat_ptr _tmp5AC=({const char*_tmp346="";_tag_fat(_tmp346,sizeof(char),1U);});_tmp349->f3=_tmp5AC;}),({
void*_tmp5AB=(void*)({struct Cyc_Arg_Unit_spec_Arg_Spec_struct*_tmp347=_cycalloc(sizeof(*_tmp347));_tmp347->tag=0U,_tmp347->f1=Cyc_set_GATHERSCRIPT;_tmp347;});_tmp349->f4=_tmp5AB;}),({
struct _fat_ptr _tmp5AA=({const char*_tmp348="Produce a script to gather C library info";_tag_fat(_tmp348,sizeof(char),42U);});_tmp349->f5=_tmp5AA;});_tmp349;});
# 1985
_tmp33A[2]=_tmp5AE;}),({
# 1988
struct _tuple31*_tmp5A9=({struct _tuple31*_tmp34E=_cycalloc(sizeof(*_tmp34E));({struct _fat_ptr _tmp5A8=({const char*_tmp34A="-finish";_tag_fat(_tmp34A,sizeof(char),8U);});_tmp34E->f1=_tmp5A8;}),_tmp34E->f2=0,({struct _fat_ptr _tmp5A7=({const char*_tmp34B="";_tag_fat(_tmp34B,sizeof(char),1U);});_tmp34E->f3=_tmp5A7;}),({
void*_tmp5A6=(void*)({struct Cyc_Arg_Unit_spec_Arg_Spec_struct*_tmp34C=_cycalloc(sizeof(*_tmp34C));_tmp34C->tag=0U,_tmp34C->f1=Cyc_set_FINISH;_tmp34C;});_tmp34E->f4=_tmp5A6;}),({
struct _fat_ptr _tmp5A5=({const char*_tmp34D="Produce Cyclone headers from pre-gathered C library info";_tag_fat(_tmp34D,sizeof(char),57U);});_tmp34E->f5=_tmp5A5;});_tmp34E;});
# 1988
_tmp33A[3]=_tmp5A9;}),({
# 1991
struct _tuple31*_tmp5A4=({struct _tuple31*_tmp353=_cycalloc(sizeof(*_tmp353));({struct _fat_ptr _tmp5A3=({const char*_tmp34F="-setjmp";_tag_fat(_tmp34F,sizeof(char),8U);});_tmp353->f1=_tmp5A3;}),_tmp353->f2=0,({struct _fat_ptr _tmp5A2=({const char*_tmp350="";_tag_fat(_tmp350,sizeof(char),1U);});_tmp353->f3=_tmp5A2;}),({
void*_tmp5A1=(void*)({struct Cyc_Arg_Set_spec_Arg_Spec_struct*_tmp351=_cycalloc(sizeof(*_tmp351));_tmp351->tag=3U,_tmp351->f1=& Cyc_do_setjmp;_tmp351;});_tmp353->f4=_tmp5A1;}),({
# 1996
struct _fat_ptr _tmp5A0=({const char*_tmp352="Produce the jmp_buf and setjmp declarations on the standard output, for use by the Cyclone compiler special file cyc_setjmp.h.  Cannot be used with -gather, -gatherscript, or specfiles.";_tag_fat(_tmp352,sizeof(char),186U);});_tmp353->f5=_tmp5A0;});_tmp353;});
# 1991
_tmp33A[4]=_tmp5A4;}),({
# 1997
struct _tuple31*_tmp59F=({struct _tuple31*_tmp358=_cycalloc(sizeof(*_tmp358));({struct _fat_ptr _tmp59E=({const char*_tmp354="-b";_tag_fat(_tmp354,sizeof(char),3U);});_tmp358->f1=_tmp59E;}),_tmp358->f2=0,({struct _fat_ptr _tmp59D=({const char*_tmp355=" <machine>";_tag_fat(_tmp355,sizeof(char),11U);});_tmp358->f3=_tmp59D;}),({
void*_tmp59C=(void*)({struct Cyc_Arg_String_spec_Arg_Spec_struct*_tmp356=_cycalloc(sizeof(*_tmp356));_tmp356->tag=5U,_tmp356->f1=Cyc_Specsfile_set_target_arch;_tmp356;});_tmp358->f4=_tmp59C;}),({
struct _fat_ptr _tmp59B=({const char*_tmp357="Set the target machine for compilation to <machine>";_tag_fat(_tmp357,sizeof(char),52U);});_tmp358->f5=_tmp59B;});_tmp358;});
# 1997
_tmp33A[5]=_tmp59F;}),({
# 2000
struct _tuple31*_tmp59A=({struct _tuple31*_tmp35D=_cycalloc(sizeof(*_tmp35D));({struct _fat_ptr _tmp599=({const char*_tmp359="-B";_tag_fat(_tmp359,sizeof(char),3U);});_tmp35D->f1=_tmp599;}),_tmp35D->f2=1,({struct _fat_ptr _tmp598=({const char*_tmp35A="<file>";_tag_fat(_tmp35A,sizeof(char),7U);});_tmp35D->f3=_tmp598;}),({
void*_tmp597=(void*)({struct Cyc_Arg_Flag_spec_Arg_Spec_struct*_tmp35B=_cycalloc(sizeof(*_tmp35B));_tmp35B->tag=1U,_tmp35B->f1=Cyc_Specsfile_add_cyclone_exec_path;_tmp35B;});_tmp35D->f4=_tmp597;}),({
struct _fat_ptr _tmp596=({const char*_tmp35C="Add to the list of directories to search for compiler files";_tag_fat(_tmp35C,sizeof(char),60U);});_tmp35D->f5=_tmp596;});_tmp35D;});
# 2000
_tmp33A[6]=_tmp59A;}),({
# 2003
struct _tuple31*_tmp595=({struct _tuple31*_tmp362=_cycalloc(sizeof(*_tmp362));({struct _fat_ptr _tmp594=({const char*_tmp35E="-v";_tag_fat(_tmp35E,sizeof(char),3U);});_tmp362->f1=_tmp594;}),_tmp362->f2=0,({struct _fat_ptr _tmp593=({const char*_tmp35F="";_tag_fat(_tmp35F,sizeof(char),1U);});_tmp362->f3=_tmp593;}),({
void*_tmp592=(void*)({struct Cyc_Arg_Set_spec_Arg_Spec_struct*_tmp360=_cycalloc(sizeof(*_tmp360));_tmp360->tag=3U,_tmp360->f1=& Cyc_verbose;_tmp360;});_tmp362->f4=_tmp592;}),({
struct _fat_ptr _tmp591=({const char*_tmp361="Verbose operation";_tag_fat(_tmp361,sizeof(char),18U);});_tmp362->f5=_tmp591;});_tmp362;});
# 2003
_tmp33A[7]=_tmp595;}),({
# 2006
struct _tuple31*_tmp590=({struct _tuple31*_tmp367=_cycalloc(sizeof(*_tmp367));({struct _fat_ptr _tmp58F=({const char*_tmp363="-";_tag_fat(_tmp363,sizeof(char),2U);});_tmp367->f1=_tmp58F;}),_tmp367->f2=1,({struct _fat_ptr _tmp58E=({const char*_tmp364="";_tag_fat(_tmp364,sizeof(char),1U);});_tmp367->f3=_tmp58E;}),({
void*_tmp58D=(void*)({struct Cyc_Arg_Flag_spec_Arg_Spec_struct*_tmp365=_cycalloc(sizeof(*_tmp365));_tmp365->tag=1U,_tmp365->f1=Cyc_add_cpparg;_tmp365;});_tmp367->f4=_tmp58D;}),({
struct _fat_ptr _tmp58C=({const char*_tmp366="";_tag_fat(_tmp366,sizeof(char),1U);});_tmp367->f5=_tmp58C;});_tmp367;});
# 2006
_tmp33A[8]=_tmp590;});Cyc_List_list(_tag_fat(_tmp33A,sizeof(struct _tuple31*),9U));});
# 2011
struct _fat_ptr otherargs=({struct Cyc_List_List*_tmp5BA=options;struct _fat_ptr _tmp5B9=({const char*_tmp339="Options:";_tag_fat(_tmp339,sizeof(char),9U);});Cyc_Specsfile_parse_b(_tmp5BA,Cyc_add_spec_file,Cyc_no_other,_tmp5B9,argv);});
# 2013
Cyc_Arg_current=0;
({struct Cyc_List_List*_tmp5BC=options;struct _fat_ptr _tmp5BB=({const char*_tmp306="Options:";_tag_fat(_tmp306,sizeof(char),9U);});Cyc_Arg_parse(_tmp5BC,Cyc_add_spec_file,Cyc_no_other,_tmp5BB,otherargs);});
if((((Cyc_badparse ||
 !Cyc_do_setjmp && Cyc_spec_files == 0)||
 Cyc_do_setjmp && Cyc_spec_files != 0)||
 Cyc_do_setjmp &&(int)Cyc_mode == (int)Cyc_GATHER)||
 Cyc_do_setjmp &&(int)Cyc_mode == (int)Cyc_GATHERSCRIPT){
({struct Cyc_List_List*_tmp5BD=options;Cyc_Arg_usage(_tmp5BD,({const char*_tmp307="Usage: buildlib [options] specfile1 specfile2 ...\nOptions:";_tag_fat(_tmp307,sizeof(char),59U);}));});
# 2023
return 1;}{
# 2015
struct _fat_ptr specs_file=
# 2029
Cyc_Specsfile_find_in_arch_path(({const char*_tmp338="cycspecs";_tag_fat(_tmp338,sizeof(char),9U);}));
if(Cyc_verbose)({struct Cyc_String_pa_PrintArg_struct _tmp30A=({struct Cyc_String_pa_PrintArg_struct _tmp3BE;_tmp3BE.tag=0U,_tmp3BE.f1=(struct _fat_ptr)((struct _fat_ptr)specs_file);_tmp3BE;});void*_tmp308[1U];_tmp308[0]=& _tmp30A;({struct Cyc___cycFILE*_tmp5BF=Cyc_stderr;struct _fat_ptr _tmp5BE=({const char*_tmp309="Reading from specs file %s\n";_tag_fat(_tmp309,sizeof(char),28U);});Cyc_fprintf(_tmp5BF,_tmp5BE,_tag_fat(_tmp308,sizeof(void*),1U));});});{struct Cyc_List_List*specs=
Cyc_Specsfile_read_specs(specs_file);
Cyc_target_cflags=({struct Cyc_List_List*_tmp5C0=specs;Cyc_Specsfile_get_spec(_tmp5C0,({const char*_tmp30B="cyclone_target_cflags";_tag_fat(_tmp30B,sizeof(char),22U);}));});
if(Cyc_verbose)({struct Cyc_String_pa_PrintArg_struct _tmp30E=({struct Cyc_String_pa_PrintArg_struct _tmp3BF;_tmp3BF.tag=0U,_tmp3BF.f1=(struct _fat_ptr)((struct _fat_ptr)Cyc_target_cflags);_tmp3BF;});void*_tmp30C[1U];_tmp30C[0]=& _tmp30E;({struct Cyc___cycFILE*_tmp5C2=Cyc_stderr;struct _fat_ptr _tmp5C1=({const char*_tmp30D="Target cflags are %s\n";_tag_fat(_tmp30D,sizeof(char),22U);});Cyc_fprintf(_tmp5C2,_tmp5C1,_tag_fat(_tmp30C,sizeof(void*),1U));});});Cyc_cyclone_cc=({
struct Cyc_List_List*_tmp5C3=specs;Cyc_Specsfile_get_spec(_tmp5C3,({const char*_tmp30F="cyclone_cc";_tag_fat(_tmp30F,sizeof(char),11U);}));});
if(!((unsigned)Cyc_cyclone_cc.curr))Cyc_cyclone_cc=({const char*_tmp310="gcc";_tag_fat(_tmp310,sizeof(char),4U);});if(Cyc_verbose)
({struct Cyc_String_pa_PrintArg_struct _tmp313=({struct Cyc_String_pa_PrintArg_struct _tmp3C0;_tmp3C0.tag=0U,_tmp3C0.f1=(struct _fat_ptr)((struct _fat_ptr)Cyc_cyclone_cc);_tmp3C0;});void*_tmp311[1U];_tmp311[0]=& _tmp313;({struct Cyc___cycFILE*_tmp5C5=Cyc_stderr;struct _fat_ptr _tmp5C4=({const char*_tmp312="C compiler is %s\n";_tag_fat(_tmp312,sizeof(char),18U);});Cyc_fprintf(_tmp5C5,_tmp5C4,_tag_fat(_tmp311,sizeof(void*),1U));});});
# 2035
if((int)Cyc_mode == (int)Cyc_GATHERSCRIPT){
# 2039
if(Cyc_verbose)
({void*_tmp314=0U;({struct Cyc___cycFILE*_tmp5C7=Cyc_stderr;struct _fat_ptr _tmp5C6=({const char*_tmp315="Creating BUILDLIB.sh\n";_tag_fat(_tmp315,sizeof(char),22U);});Cyc_fprintf(_tmp5C7,_tmp5C6,_tag_fat(_tmp314,sizeof(void*),0U));});});
# 2039
Cyc_script_file=
# 2041
Cyc_fopen("BUILDLIB.sh","w");
if(!((unsigned)Cyc_script_file)){
({void*_tmp316=0U;({struct Cyc___cycFILE*_tmp5C9=Cyc_stderr;struct _fat_ptr _tmp5C8=({const char*_tmp317="Could not create file BUILDLIB.sh\n";_tag_fat(_tmp317,sizeof(char),35U);});Cyc_fprintf(_tmp5C9,_tmp5C8,_tag_fat(_tmp316,sizeof(void*),0U));});});
exit(1);}
# 2042
({void*_tmp318=0U;({struct _fat_ptr _tmp5CA=({const char*_tmp319="#!/bin/sh\n";_tag_fat(_tmp319,sizeof(char),11U);});Cyc_prscript(_tmp5CA,_tag_fat(_tmp318,sizeof(void*),0U));});});
# 2047
({void*_tmp31A=0U;({struct _fat_ptr _tmp5CB=({const char*_tmp31B="GCC=\"gcc\"\n";_tag_fat(_tmp31B,sizeof(char),11U);});Cyc_prscript(_tmp5CB,_tag_fat(_tmp31A,sizeof(void*),0U));});});}
# 2035
if(
# 2051
Cyc_force_directory_prefixes(Cyc_output_dir)|| Cyc_force_directory(Cyc_output_dir)){
({struct Cyc_String_pa_PrintArg_struct _tmp31E=({struct Cyc_String_pa_PrintArg_struct _tmp3C1;_tmp3C1.tag=0U,_tmp3C1.f1=(struct _fat_ptr)((struct _fat_ptr)Cyc_output_dir);_tmp3C1;});void*_tmp31C[1U];_tmp31C[0]=& _tmp31E;({struct Cyc___cycFILE*_tmp5CD=Cyc_stderr;struct _fat_ptr _tmp5CC=({const char*_tmp31D="Error: could not create directory %s\n";_tag_fat(_tmp31D,sizeof(char),38U);});Cyc_fprintf(_tmp5CD,_tmp5CC,_tag_fat(_tmp31C,sizeof(void*),1U));});});
return 1;}
# 2035
if(Cyc_verbose)
# 2056
({struct Cyc_String_pa_PrintArg_struct _tmp321=({struct Cyc_String_pa_PrintArg_struct _tmp3C2;_tmp3C2.tag=0U,_tmp3C2.f1=(struct _fat_ptr)((struct _fat_ptr)Cyc_output_dir);_tmp3C2;});void*_tmp31F[1U];_tmp31F[0]=& _tmp321;({struct Cyc___cycFILE*_tmp5CF=Cyc_stderr;struct _fat_ptr _tmp5CE=({const char*_tmp320="Output directory is %s\n";_tag_fat(_tmp320,sizeof(char),24U);});Cyc_fprintf(_tmp5CF,_tmp5CE,_tag_fat(_tmp31F,sizeof(void*),1U));});});
# 2035
if((int)Cyc_mode == (int)Cyc_GATHERSCRIPT){
# 2059
({struct Cyc_String_pa_PrintArg_struct _tmp324=({struct Cyc_String_pa_PrintArg_struct _tmp3C3;_tmp3C3.tag=0U,_tmp3C3.f1=(struct _fat_ptr)((struct _fat_ptr)Cyc_output_dir);_tmp3C3;});void*_tmp322[1U];_tmp322[0]=& _tmp324;({struct _fat_ptr _tmp5D0=({const char*_tmp323="cd %s\n";_tag_fat(_tmp323,sizeof(char),7U);});Cyc_prscript(_tmp5D0,_tag_fat(_tmp322,sizeof(void*),1U));});});
({struct Cyc_String_pa_PrintArg_struct _tmp327=({struct Cyc_String_pa_PrintArg_struct _tmp3C4;_tmp3C4.tag=0U,_tmp3C4.f1=(struct _fat_ptr)((struct _fat_ptr)Cyc_target_cflags);_tmp3C4;});void*_tmp325[1U];_tmp325[0]=& _tmp327;({struct _fat_ptr _tmp5D1=({const char*_tmp326="echo | $GCC %s -E -dM - -o INITMACROS.h\n";_tag_fat(_tmp326,sizeof(char),41U);});Cyc_prscript(_tmp5D1,_tag_fat(_tmp325,sizeof(void*),1U));});});}
# 2035
if(!
# 2064
Cyc_gathering()){
# 2067
Cyc_log_file=Cyc_fopen((const char*)_check_null(_untag_fat_ptr(({struct _fat_ptr _tmp5D2=Cyc_output_dir;Cyc_Filename_concat(_tmp5D2,({const char*_tmp328="BUILDLIB.LOG";_tag_fat(_tmp328,sizeof(char),13U);}));}),sizeof(char),1U)),"w");
if(!((unsigned)Cyc_log_file)){
({struct Cyc_String_pa_PrintArg_struct _tmp32B=({struct Cyc_String_pa_PrintArg_struct _tmp3C5;_tmp3C5.tag=0U,_tmp3C5.f1=(struct _fat_ptr)((struct _fat_ptr)Cyc_output_dir);_tmp3C5;});void*_tmp329[1U];_tmp329[0]=& _tmp32B;({struct Cyc___cycFILE*_tmp5D4=Cyc_stderr;struct _fat_ptr _tmp5D3=({const char*_tmp32A="Error: could not create log file in directory %s\n";_tag_fat(_tmp32A,sizeof(char),50U);});Cyc_fprintf(_tmp5D4,_tmp5D3,_tag_fat(_tmp329,sizeof(void*),1U));});});
return 1;}
# 2068
if(!Cyc_do_setjmp){
# 2075
Cyc_cstubs_file=Cyc_fopen((const char*)_check_null(_untag_fat_ptr(({struct _fat_ptr _tmp5D5=Cyc_output_dir;Cyc_Filename_concat(_tmp5D5,({const char*_tmp32C="cstubs.c";_tag_fat(_tmp32C,sizeof(char),9U);}));}),sizeof(char),1U)),"w");
if(!((unsigned)Cyc_cstubs_file)){
({struct Cyc_String_pa_PrintArg_struct _tmp32F=({struct Cyc_String_pa_PrintArg_struct _tmp3C6;_tmp3C6.tag=0U,_tmp3C6.f1=(struct _fat_ptr)((struct _fat_ptr)Cyc_output_dir);_tmp3C6;});void*_tmp32D[1U];_tmp32D[0]=& _tmp32F;({struct Cyc___cycFILE*_tmp5D7=Cyc_stderr;struct _fat_ptr _tmp5D6=({const char*_tmp32E="Error: could not create cstubs.c in directory %s\n";_tag_fat(_tmp32E,sizeof(char),50U);});Cyc_fprintf(_tmp5D7,_tmp5D6,_tag_fat(_tmp32D,sizeof(void*),1U));});});
return 1;}
# 2076
Cyc_cycstubs_file=
# 2082
Cyc_fopen((const char*)_check_null(_untag_fat_ptr(({struct _fat_ptr _tmp5D8=Cyc_output_dir;Cyc_Filename_concat(_tmp5D8,({const char*_tmp330="cycstubs.cyc";_tag_fat(_tmp330,sizeof(char),13U);}));}),sizeof(char),1U)),"w");
if(!((unsigned)Cyc_cycstubs_file)){
({struct Cyc_String_pa_PrintArg_struct _tmp333=({struct Cyc_String_pa_PrintArg_struct _tmp3C7;_tmp3C7.tag=0U,_tmp3C7.f1=(struct _fat_ptr)((struct _fat_ptr)Cyc_output_dir);_tmp3C7;});void*_tmp331[1U];_tmp331[0]=& _tmp333;({struct Cyc___cycFILE*_tmp5DA=Cyc_stderr;struct _fat_ptr _tmp5D9=({const char*_tmp332="Error: could not create cycstubs.c in directory %s\n";_tag_fat(_tmp332,sizeof(char),52U);});Cyc_fprintf(_tmp5DA,_tmp5D9,_tag_fat(_tmp331,sizeof(void*),1U));});});
# 2087
return 1;}
# 2083
({void*_tmp334=0U;({struct Cyc___cycFILE*_tmp5DC=(struct Cyc___cycFILE*)_check_null(Cyc_cycstubs_file);struct _fat_ptr _tmp5DB=({const char*_tmp335="#include <core.h>\nusing Core;\n\n";_tag_fat(_tmp335,sizeof(char),32U);});Cyc_fprintf(_tmp5DC,_tmp5DB,_tag_fat(_tmp334,sizeof(void*),0U));});});}}{
# 2035
const char*outdir=(const char*)_check_null(_untag_fat_ptr(Cyc_output_dir,sizeof(char),1U));
# 2097
if(Cyc_do_setjmp && Cyc_process_setjmp(outdir))
return 1;else{
# 2102
for(0;Cyc_spec_files != 0;Cyc_spec_files=((struct Cyc_List_List*)_check_null(Cyc_spec_files))->tl){
if(Cyc_process_specfile((const char*)((struct Cyc_List_List*)_check_null(Cyc_spec_files))->hd,outdir)){
({void*_tmp336=0U;({struct Cyc___cycFILE*_tmp5DE=Cyc_stderr;struct _fat_ptr _tmp5DD=({const char*_tmp337="FATAL ERROR -- QUIT!\n";_tag_fat(_tmp337,sizeof(char),22U);});Cyc_fprintf(_tmp5DE,_tmp5DD,_tag_fat(_tmp336,sizeof(void*),0U));});});
exit(1);}}}
# 2110
if((int)Cyc_mode == (int)Cyc_GATHERSCRIPT)
Cyc_fclose((struct Cyc___cycFILE*)_check_null(Cyc_script_file));else{
# 2113
if(!Cyc_gathering()){
Cyc_fclose((struct Cyc___cycFILE*)_check_null(Cyc_log_file));
if(!Cyc_do_setjmp){
Cyc_fclose((struct Cyc___cycFILE*)_check_null(Cyc_cstubs_file));
Cyc_fclose((struct Cyc___cycFILE*)_check_null(Cyc_cycstubs_file));}}}
# 2110
return 0;}}}}}
