#include <setjmp.h>
/* This is a C header file to be used by the output of the Cyclone to
   C translator.  The corresponding definitions are in file lib/runtime_*.c */
#ifndef _CYC_INCLUDE_H_
#define _CYC_INCLUDE_H_

/* Need one of these per thread (see runtime_stack.c). The runtime maintains 
   a stack that contains either _handler_cons structs or _RegionHandle structs.
   The tag is 0 for a handler_cons and 1 for a region handle.  */
struct _RuntimeStack {
  int tag; 
  struct _RuntimeStack *next;
  void (*cleanup)(struct _RuntimeStack *frame);
};

#ifndef offsetof
/* should be size_t, but int is fine. */
#define offsetof(t,n) ((int)(&(((t *)0)->n)))
#endif

/* Fat pointers */
struct _fat_ptr {
  unsigned char *curr; 
  unsigned char *base; 
  unsigned char *last_plus_one; 
};  

/* Discriminated Unions */
struct _xtunion_struct { char *tag; };

/* Regions */
struct _RegionPage
#ifdef CYC_REGION_PROFILE
{ unsigned total_bytes;
  unsigned free_bytes;
  /* MWH: wish we didn't have to include the stuff below ... */
  struct _RegionPage *next;
  char data[1];
}
#endif
; // abstract -- defined in runtime_memory.c
struct _RegionHandle {
  struct _RuntimeStack s;
  struct _RegionPage *curr;
  char               *offset;
  char               *last_plus_one;
  struct _DynRegionHandle *sub_regions;
#ifdef CYC_REGION_PROFILE
  const char         *name;
#else
  unsigned used_bytes;
  unsigned wasted_bytes;
#endif
};
struct _DynRegionFrame {
  struct _RuntimeStack s;
  struct _DynRegionHandle *x;
};
// A dynamic region is just a region handle.  The wrapper struct is for type
// abstraction.
struct Cyc_Core_DynamicRegion {
  struct _RegionHandle h;
};

struct _RegionHandle _new_region(const char*);
void* _region_malloc(struct _RegionHandle*, unsigned);
void* _region_calloc(struct _RegionHandle*, unsigned t, unsigned n);
void   _free_region(struct _RegionHandle*);
struct _RegionHandle*_open_dynregion(struct _DynRegionFrame*,struct _DynRegionHandle*);
void   _pop_dynregion();

/* Exceptions */
struct _handler_cons {
  struct _RuntimeStack s;
  jmp_buf handler;
};
void _push_handler(struct _handler_cons *);
void _push_region(struct _RegionHandle *);
void _npop_handler(int);
void _pop_handler();
void _pop_region();

#ifndef _throw
void* _throw_null_fn(const char*,unsigned);
void* _throw_arraybounds_fn(const char*,unsigned);
void* _throw_badalloc_fn(const char*,unsigned);
void* _throw_match_fn(const char*,unsigned);
void* _throw_fn(void*,const char*,unsigned);
void* _rethrow(void*);
#define _throw_null() (_throw_null_fn(__FILE__,__LINE__))
#define _throw_arraybounds() (_throw_arraybounds_fn(__FILE__,__LINE__))
#define _throw_badalloc() (_throw_badalloc_fn(__FILE__,__LINE__))
#define _throw_match() (_throw_match_fn(__FILE__,__LINE__))
#define _throw(e) (_throw_fn((e),__FILE__,__LINE__))
#endif

struct _xtunion_struct* Cyc_Core_get_exn_thrown();
/* Built-in Exceptions */
struct Cyc_Null_Exception_exn_struct { char *tag; };
struct Cyc_Array_bounds_exn_struct { char *tag; };
struct Cyc_Match_Exception_exn_struct { char *tag; };
struct Cyc_Bad_alloc_exn_struct { char *tag; };
extern char Cyc_Null_Exception[];
extern char Cyc_Array_bounds[];
extern char Cyc_Match_Exception[];
extern char Cyc_Bad_alloc[];

/* Built-in Run-time Checks and company */
#ifdef CYC_ANSI_OUTPUT
#define _INLINE  
#else
#define _INLINE inline
#endif

#ifdef NO_CYC_NULL_CHECKS
#define _check_null(ptr) (ptr)
#else
#define _check_null(ptr) \
  ({ void*_cks_null = (void*)(ptr); \
     if (!_cks_null) _throw_null(); \
     _cks_null; })
#endif

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index)\
   (((char*)ptr) + (elt_sz)*(index))
#ifdef NO_CYC_NULL_CHECKS
#define _check_known_subscript_null _check_known_subscript_notnull
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr);\
  int _index = (index);\
  if (!_cks_ptr) _throw_null(); \
  _cks_ptr + (elt_sz)*_index; })
#endif
#define _zero_arr_plus_fn(orig_x,orig_sz,orig_i,f,l) ((orig_x)+(orig_i))
#define _zero_arr_plus_char_fn _zero_arr_plus_fn
#define _zero_arr_plus_short_fn _zero_arr_plus_fn
#define _zero_arr_plus_int_fn _zero_arr_plus_fn
#define _zero_arr_plus_float_fn _zero_arr_plus_fn
#define _zero_arr_plus_double_fn _zero_arr_plus_fn
#define _zero_arr_plus_longdouble_fn _zero_arr_plus_fn
#define _zero_arr_plus_voidstar_fn _zero_arr_plus_fn
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (!_cks_ptr) _throw_null(); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })

/* _zero_arr_plus_*_fn(x,sz,i,filename,lineno) adds i to zero-terminated ptr
   x that has at least sz elements */
char* _zero_arr_plus_char_fn(char*,unsigned,int,const char*,unsigned);
short* _zero_arr_plus_short_fn(short*,unsigned,int,const char*,unsigned);
int* _zero_arr_plus_int_fn(int*,unsigned,int,const char*,unsigned);
float* _zero_arr_plus_float_fn(float*,unsigned,int,const char*,unsigned);
double* _zero_arr_plus_double_fn(double*,unsigned,int,const char*,unsigned);
long double* _zero_arr_plus_longdouble_fn(long double*,unsigned,int,const char*, unsigned);
void** _zero_arr_plus_voidstar_fn(void**,unsigned,int,const char*,unsigned);
#endif

/* _get_zero_arr_size_*(x,sz) returns the number of elements in a
   zero-terminated array that is NULL or has at least sz elements */
int _get_zero_arr_size_char(const char*,unsigned);
int _get_zero_arr_size_short(const short*,unsigned);
int _get_zero_arr_size_int(const int*,unsigned);
int _get_zero_arr_size_float(const float*,unsigned);
int _get_zero_arr_size_double(const double*,unsigned);
int _get_zero_arr_size_longdouble(const long double*,unsigned);
int _get_zero_arr_size_voidstar(const void**,unsigned);

/* _zero_arr_inplace_plus_*_fn(x,i,filename,lineno) sets
   zero-terminated pointer *x to *x + i */
char* _zero_arr_inplace_plus_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_short_fn(short**,int,const char*,unsigned);
int* _zero_arr_inplace_plus_int(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_longdouble_fn(long double**,int,const char*,unsigned);
void** _zero_arr_inplace_plus_voidstar_fn(void***,int,const char*,unsigned);
/* like the previous functions, but does post-addition (as in e++) */
char* _zero_arr_inplace_plus_post_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_post_short_fn(short**x,int,const char*,unsigned);
int* _zero_arr_inplace_plus_post_int_fn(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_post_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_post_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_post_longdouble_fn(long double**,int,const char *,unsigned);
void** _zero_arr_inplace_plus_post_voidstar_fn(void***,int,const char*,unsigned);
#define _zero_arr_plus_char(x,s,i) \
  (_zero_arr_plus_char_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_short(x,s,i) \
  (_zero_arr_plus_short_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_int(x,s,i) \
  (_zero_arr_plus_int_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_float(x,s,i) \
  (_zero_arr_plus_float_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_double(x,s,i) \
  (_zero_arr_plus_double_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_longdouble(x,s,i) \
  (_zero_arr_plus_longdouble_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_voidstar(x,s,i) \
  (_zero_arr_plus_voidstar_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_inplace_plus_char(x,i) \
  _zero_arr_inplace_plus_char_fn((char **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_short(x,i) \
  _zero_arr_inplace_plus_short_fn((short **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_int(x,i) \
  _zero_arr_inplace_plus_int_fn((int **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_float(x,i) \
  _zero_arr_inplace_plus_float_fn((float **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_double(x,i) \
  _zero_arr_inplace_plus_double_fn((double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_longdouble(x,i) \
  _zero_arr_inplace_plus_longdouble_fn((long double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_voidstar(x,i) \
  _zero_arr_inplace_plus_voidstar_fn((void ***)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_char(x,i) \
  _zero_arr_inplace_plus_post_char_fn((char **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_short(x,i) \
  _zero_arr_inplace_plus_post_short_fn((short **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_int(x,i) \
  _zero_arr_inplace_plus_post_int_fn((int **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_float(x,i) \
  _zero_arr_inplace_plus_post_float_fn((float **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_double(x,i) \
  _zero_arr_inplace_plus_post_double_fn((double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_longdouble(x,i) \
  _zero_arr_inplace_plus_post_longdouble_fn((long double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_voidstar(x,i) \
  _zero_arr_inplace_plus_post_voidstar_fn((void***)(x),(i),__FILE__,__LINE__)

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_fat_subscript(arr,elt_sz,index) ((arr).curr + (elt_sz) * (index))
#define _untag_fat_ptr(arr,elt_sz,num_elts) ((arr).curr)
#else
#define _check_fat_subscript(arr,elt_sz,index) ({ \
  struct _fat_ptr _cus_arr = (arr); \
  unsigned char *_cus_ans = _cus_arr.curr + (elt_sz) * (index); \
  /* JGM: not needed! if (!_cus_arr.base) _throw_null();*/ \
  if (_cus_ans < _cus_arr.base || _cus_ans >= _cus_arr.last_plus_one) \
    _throw_arraybounds(); \
  _cus_ans; })
#define _untag_fat_ptr(arr,elt_sz,num_elts) ({ \
  struct _fat_ptr _arr = (arr); \
  unsigned char *_curr = _arr.curr; \
  if ((_curr < _arr.base || _curr + (elt_sz) * (num_elts) > _arr.last_plus_one) &&\
      _curr != (unsigned char *)0) \
    _throw_arraybounds(); \
  _curr; })
#endif

#define _tag_fat(tcurr,elt_sz,num_elts) ({ \
  struct _fat_ptr _ans; \
  unsigned _num_elts = (num_elts);\
  _ans.base = _ans.curr = (void*)(tcurr); \
  /* JGM: if we're tagging NULL, ignore num_elts */ \
  _ans.last_plus_one = _ans.base ? (_ans.base + (elt_sz) * _num_elts) : 0; \
  _ans; })

#define _get_fat_size(arr,elt_sz) \
  ({struct _fat_ptr _arr = (arr); \
    unsigned char *_arr_curr=_arr.curr; \
    unsigned char *_arr_last=_arr.last_plus_one; \
    (_arr_curr < _arr.base || _arr_curr >= _arr_last) ? 0 : \
    ((_arr_last - _arr_curr) / (elt_sz));})

#define _fat_ptr_plus(arr,elt_sz,change) ({ \
  struct _fat_ptr _ans = (arr); \
  int _change = (change);\
  _ans.curr += (elt_sz) * _change;\
  _ans; })
#define _fat_ptr_inplace_plus(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  _arr_ptr->curr += (elt_sz) * (change);\
  *_arr_ptr; })
#define _fat_ptr_inplace_plus_post(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  struct _fat_ptr _ans = *_arr_ptr; \
  _arr_ptr->curr += (elt_sz) * (change);\
  _ans; })

//Not a macro since initialization order matters. Defined in runtime_zeroterm.c.
struct _fat_ptr _fat_ptr_decrease_size(struct _fat_ptr,unsigned sz,unsigned numelts);

/* Allocation */
void* GC_malloc(int);
void* GC_malloc_atomic(int);
void* GC_calloc(unsigned,unsigned);
void* GC_calloc_atomic(unsigned,unsigned);
// bound the allocation size to be < MAX_ALLOC_SIZE. See macros below for usage.
#define MAX_MALLOC_SIZE (1 << 28)
void* _bounded_GC_malloc(int,const char*,int);
void* _bounded_GC_malloc_atomic(int,const char*,int);
void* _bounded_GC_calloc(unsigned,unsigned,const char*,int);
void* _bounded_GC_calloc_atomic(unsigned,unsigned,const char*,int);
/* these macros are overridden below ifdef CYC_REGION_PROFILE */
#ifndef CYC_REGION_PROFILE
#define _cycalloc(n) _bounded_GC_malloc(n,__FILE__,__LINE__)
#define _cycalloc_atomic(n) _bounded_GC_malloc_atomic(n,__FILE__,__LINE__)
#define _cyccalloc(n,s) _bounded_GC_calloc(n,s,__FILE__,__LINE__)
#define _cyccalloc_atomic(n,s) _bounded_GC_calloc_atomic(n,s,__FILE__,__LINE__)
#endif

static _INLINE unsigned int _check_times(unsigned x, unsigned y) {
  unsigned long long whole_ans = 
    ((unsigned long long) x)*((unsigned long long)y);
  unsigned word_ans = (unsigned)whole_ans;
  if(word_ans < whole_ans || word_ans > MAX_MALLOC_SIZE)
    _throw_badalloc();
  return word_ans;
}

#define _CYC_MAX_REGION_CONST 2
#define _CYC_MIN_ALIGNMENT (sizeof(double))

#ifdef CYC_REGION_PROFILE
extern int rgn_total_bytes;
#endif

static _INLINE void *_fast_region_malloc(struct _RegionHandle *r, unsigned orig_s) {  
  if (r > (struct _RegionHandle *)_CYC_MAX_REGION_CONST && r->curr != 0) { 
#ifdef CYC_NOALIGN
    unsigned s =  orig_s;
#else
    unsigned s =  (orig_s + _CYC_MIN_ALIGNMENT - 1) & (~(_CYC_MIN_ALIGNMENT -1)); 
#endif
    char *result; 
    result = r->offset; 
    if (s <= (r->last_plus_one - result)) {
      r->offset = result + s; 
#ifdef CYC_REGION_PROFILE
    r->curr->free_bytes = r->curr->free_bytes - s;
    rgn_total_bytes += s;
#endif
      return result;
    }
  } 
  return _region_malloc(r,orig_s); 
}

#ifdef CYC_REGION_PROFILE
/* see macros below for usage. defined in runtime_memory.c */
void* _profile_GC_malloc(int,const char*,const char*,int);
void* _profile_GC_malloc_atomic(int,const char*,const char*,int);
void* _profile_GC_calloc(unsigned,unsigned,const char*,const char*,int);
void* _profile_GC_calloc_atomic(unsigned,unsigned,const char*,const char*,int);
void* _profile_region_malloc(struct _RegionHandle*,unsigned,const char*,const char*,int);
void* _profile_region_calloc(struct _RegionHandle*,unsigned,unsigned,const char *,const char*,int);
struct _RegionHandle _profile_new_region(const char*,const char*,const char*,int);
void _profile_free_region(struct _RegionHandle*,const char*,const char*,int);
#ifndef RUNTIME_CYC
#define _new_region(n) _profile_new_region(n,__FILE__,__FUNCTION__,__LINE__)
#define _free_region(r) _profile_free_region(r,__FILE__,__FUNCTION__,__LINE__)
#define _region_malloc(rh,n) _profile_region_malloc(rh,n,__FILE__,__FUNCTION__,__LINE__)
#define _region_calloc(rh,n,t) _profile_region_calloc(rh,n,t,__FILE__,__FUNCTION__,__LINE__)
#  endif
#define _cycalloc(n) _profile_GC_malloc(n,__FILE__,__FUNCTION__,__LINE__)
#define _cycalloc_atomic(n) _profile_GC_malloc_atomic(n,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc(n,s) _profile_GC_calloc(n,s,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc_atomic(n,s) _profile_GC_calloc_atomic(n,s,__FILE__,__FUNCTION__,__LINE__)
#endif
#endif
 struct Cyc_Core_Opt{void*v;};extern char Cyc_Core_Invalid_argument[17U];struct Cyc_Core_Invalid_argument_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Failure[8U];struct Cyc_Core_Failure_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Impossible[11U];struct Cyc_Core_Impossible_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Not_found[10U];struct Cyc_Core_Not_found_exn_struct{char*tag;};extern char Cyc_Core_Unreachable[12U];struct Cyc_Core_Unreachable_exn_struct{char*tag;struct _fat_ptr f1;};
# 165 "core.h"
extern struct _RegionHandle*Cyc_Core_heap_region;
# 168
extern struct _RegionHandle*Cyc_Core_unique_region;struct Cyc_Core_DynamicRegion;struct Cyc_Core_NewDynamicRegion{struct Cyc_Core_DynamicRegion*key;};struct Cyc_Core_ThinRes{void*arr;unsigned nelts;};struct Cyc_List_List{void*hd;struct Cyc_List_List*tl;};
# 61 "list.h"
int Cyc_List_length(struct Cyc_List_List*x);
# 86
struct Cyc_List_List*Cyc_List_rmap_c(struct _RegionHandle*,void*(*f)(void*,void*),void*env,struct Cyc_List_List*x);extern char Cyc_List_List_mismatch[14U];struct Cyc_List_List_mismatch_exn_struct{char*tag;};
# 178
struct Cyc_List_List*Cyc_List_imp_rev(struct Cyc_List_List*x);extern char Cyc_List_Nth[4U];struct Cyc_List_Nth_exn_struct{char*tag;};
# 322
int Cyc_List_mem(int(*compare)(void*,void*),struct Cyc_List_List*l,void*x);
# 383
int Cyc_List_list_cmp(int(*cmp)(void*,void*),struct Cyc_List_List*l1,struct Cyc_List_List*l2);
# 394
struct Cyc_List_List*Cyc_List_filter_c(int(*f)(void*,void*),void*env,struct Cyc_List_List*x);struct Cyc_Iter_Iter{void*env;int(*next)(void*env,void*dest);};
# 37 "iter.h"
int Cyc_Iter_next(struct Cyc_Iter_Iter,void*);struct Cyc_Set_Set;extern char Cyc_Set_Absent[7U];struct Cyc_Set_Absent_exn_struct{char*tag;};struct Cyc___cycFILE;
# 53 "cycboot.h"
extern struct Cyc___cycFILE*Cyc_stderr;struct Cyc_String_pa_PrintArg_struct{int tag;struct _fat_ptr f1;};struct Cyc_Int_pa_PrintArg_struct{int tag;unsigned long f1;};struct Cyc_Double_pa_PrintArg_struct{int tag;double f1;};struct Cyc_LongDouble_pa_PrintArg_struct{int tag;long double f1;};struct Cyc_ShortPtr_pa_PrintArg_struct{int tag;short*f1;};struct Cyc_IntPtr_pa_PrintArg_struct{int tag;unsigned long*f1;};
# 73
struct _fat_ptr Cyc_aprintf(struct _fat_ptr,struct _fat_ptr);
# 100
int Cyc_fprintf(struct Cyc___cycFILE*,struct _fat_ptr,struct _fat_ptr);struct Cyc_ShortPtr_sa_ScanfArg_struct{int tag;short*f1;};struct Cyc_UShortPtr_sa_ScanfArg_struct{int tag;unsigned short*f1;};struct Cyc_IntPtr_sa_ScanfArg_struct{int tag;int*f1;};struct Cyc_UIntPtr_sa_ScanfArg_struct{int tag;unsigned*f1;};struct Cyc_StringPtr_sa_ScanfArg_struct{int tag;struct _fat_ptr f1;};struct Cyc_DoublePtr_sa_ScanfArg_struct{int tag;double*f1;};struct Cyc_FloatPtr_sa_ScanfArg_struct{int tag;float*f1;};struct Cyc_CharPtr_sa_ScanfArg_struct{int tag;struct _fat_ptr f1;};extern char Cyc_FileCloseError[15U];struct Cyc_FileCloseError_exn_struct{char*tag;};extern char Cyc_FileOpenError[14U];struct Cyc_FileOpenError_exn_struct{char*tag;struct _fat_ptr f1;};struct Cyc_Dict_T;struct Cyc_Dict_Dict{int(*rel)(void*,void*);struct _RegionHandle*r;const struct Cyc_Dict_T*t;};extern char Cyc_Dict_Present[8U];struct Cyc_Dict_Present_exn_struct{char*tag;};extern char Cyc_Dict_Absent[7U];struct Cyc_Dict_Absent_exn_struct{char*tag;};
# 62 "dict.h"
struct Cyc_Dict_Dict Cyc_Dict_empty(int(*cmp)(void*,void*));
# 87
struct Cyc_Dict_Dict Cyc_Dict_insert(struct Cyc_Dict_Dict d,void*k,void*v);
# 110
void*Cyc_Dict_lookup(struct Cyc_Dict_Dict d,void*k);
# 147 "dict.h"
void Cyc_Dict_iter(void(*f)(void*,void*),struct Cyc_Dict_Dict d);
# 212
struct Cyc_Dict_Dict Cyc_Dict_intersect_c(void*(*f)(void*,void*,void*,void*),void*env,struct Cyc_Dict_Dict d1,struct Cyc_Dict_Dict d2);
# 222
int Cyc_Dict_forall_intersect(int(*f)(void*,void*,void*),struct Cyc_Dict_Dict d1,struct Cyc_Dict_Dict d2);
# 283
struct Cyc_Iter_Iter Cyc_Dict_make_iter(struct _RegionHandle*rgn,struct Cyc_Dict_Dict d);
# 50 "string.h"
int Cyc_strptrcmp(struct _fat_ptr*s1,struct _fat_ptr*s2);struct Cyc_Position_Error;struct Cyc_Relations_Reln;struct _union_Nmspace_Rel_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_Abs_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_C_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_Loc_n{int tag;int val;};union Cyc_Absyn_Nmspace{struct _union_Nmspace_Rel_n Rel_n;struct _union_Nmspace_Abs_n Abs_n;struct _union_Nmspace_C_n C_n;struct _union_Nmspace_Loc_n Loc_n;};struct _tuple0{union Cyc_Absyn_Nmspace f1;struct _fat_ptr*f2;};
# 161 "absyn.h"
enum Cyc_Absyn_Scope{Cyc_Absyn_Static =0U,Cyc_Absyn_Abstract =1U,Cyc_Absyn_Public =2U,Cyc_Absyn_Extern =3U,Cyc_Absyn_ExternC =4U,Cyc_Absyn_Register =5U};struct Cyc_Absyn_Tqual{int print_const: 1;int q_volatile: 1;int q_restrict: 1;int real_const: 1;unsigned loc;};
# 182
enum Cyc_Absyn_Size_of{Cyc_Absyn_Char_sz =0U,Cyc_Absyn_Short_sz =1U,Cyc_Absyn_Int_sz =2U,Cyc_Absyn_Long_sz =3U,Cyc_Absyn_LongLong_sz =4U};
enum Cyc_Absyn_Sign{Cyc_Absyn_Signed =0U,Cyc_Absyn_Unsigned =1U,Cyc_Absyn_None =2U};
enum Cyc_Absyn_AggrKind{Cyc_Absyn_StructA =0U,Cyc_Absyn_UnionA =1U};
# 187
enum Cyc_Absyn_AliasQual{Cyc_Absyn_Aliasable =0U,Cyc_Absyn_Unique =1U,Cyc_Absyn_Top =2U};
# 192
enum Cyc_Absyn_KindQual{Cyc_Absyn_AnyKind =0U,Cyc_Absyn_MemKind =1U,Cyc_Absyn_BoxKind =2U,Cyc_Absyn_RgnKind =3U,Cyc_Absyn_EffKind =4U,Cyc_Absyn_IntKind =5U,Cyc_Absyn_BoolKind =6U,Cyc_Absyn_PtrBndKind =7U};struct Cyc_Absyn_Kind{enum Cyc_Absyn_KindQual kind;enum Cyc_Absyn_AliasQual aliasqual;};struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct{int tag;struct Cyc_Absyn_Kind*f1;};struct Cyc_Absyn_Unknown_kb_Absyn_KindBound_struct{int tag;struct Cyc_Core_Opt*f1;};struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct{int tag;struct Cyc_Core_Opt*f1;struct Cyc_Absyn_Kind*f2;};struct Cyc_Absyn_Tvar{struct _fat_ptr*name;int identity;void*kind;};struct Cyc_Absyn_PtrLoc{unsigned ptr_loc;unsigned rgn_loc;unsigned zt_loc;};struct Cyc_Absyn_PtrAtts{void*rgn;void*nullable;void*bounds;void*zero_term;struct Cyc_Absyn_PtrLoc*ptrloc;};struct Cyc_Absyn_PtrInfo{void*elt_type;struct Cyc_Absyn_Tqual elt_tq;struct Cyc_Absyn_PtrAtts ptr_atts;};struct Cyc_Absyn_VarargInfo{struct _fat_ptr*name;struct Cyc_Absyn_Tqual tq;void*type;int inject;};struct Cyc_Absyn_Star_Absyn_Cap_struct{int tag;};struct Cyc_Absyn_Bot_Absyn_Cap_struct{int tag;};struct Cyc_Absyn_Nat_Absyn_Cap_struct{int tag;struct Cyc_Absyn_Exp*f1;int f2;};struct Cyc_Absyn_RgnEffect{void*name;void*rgn;void*lock;void*unadopt;void*parent;};struct Cyc_Absyn_FnInfo{struct Cyc_List_List*tvars;void*effect;struct Cyc_Absyn_Tqual ret_tqual;void*ret_type;struct Cyc_List_List*args;int c_varargs;struct Cyc_Absyn_VarargInfo*cyc_varargs;struct Cyc_List_List*rgn_po;struct Cyc_List_List*attributes;struct Cyc_Absyn_Exp*requires_clause;struct Cyc_List_List*requires_relns;struct Cyc_Absyn_Exp*ensures_clause;struct Cyc_List_List*ensures_relns;struct Cyc_List_List*ieffect;struct Cyc_List_List*oeffect;};struct Cyc_Absyn_UnknownDatatypeInfo{struct _tuple0*name;int is_extensible;};struct _union_DatatypeInfo_UnknownDatatype{int tag;struct Cyc_Absyn_UnknownDatatypeInfo val;};struct _union_DatatypeInfo_KnownDatatype{int tag;struct Cyc_Absyn_Datatypedecl**val;};union Cyc_Absyn_DatatypeInfo{struct _union_DatatypeInfo_UnknownDatatype UnknownDatatype;struct _union_DatatypeInfo_KnownDatatype KnownDatatype;};struct Cyc_Absyn_UnknownDatatypeFieldInfo{struct _tuple0*datatype_name;struct _tuple0*field_name;int is_extensible;};struct _union_DatatypeFieldInfo_UnknownDatatypefield{int tag;struct Cyc_Absyn_UnknownDatatypeFieldInfo val;};struct _tuple1{struct Cyc_Absyn_Datatypedecl*f1;struct Cyc_Absyn_Datatypefield*f2;};struct _union_DatatypeFieldInfo_KnownDatatypefield{int tag;struct _tuple1 val;};union Cyc_Absyn_DatatypeFieldInfo{struct _union_DatatypeFieldInfo_UnknownDatatypefield UnknownDatatypefield;struct _union_DatatypeFieldInfo_KnownDatatypefield KnownDatatypefield;};struct _tuple2{enum Cyc_Absyn_AggrKind f1;struct _tuple0*f2;struct Cyc_Core_Opt*f3;};struct _union_AggrInfo_UnknownAggr{int tag;struct _tuple2 val;};struct _union_AggrInfo_KnownAggr{int tag;struct Cyc_Absyn_Aggrdecl**val;};union Cyc_Absyn_AggrInfo{struct _union_AggrInfo_UnknownAggr UnknownAggr;struct _union_AggrInfo_KnownAggr KnownAggr;};struct Cyc_Absyn_ArrayInfo{void*elt_type;struct Cyc_Absyn_Tqual tq;struct Cyc_Absyn_Exp*num_elts;void*zero_term;unsigned zt_loc;};struct Cyc_Absyn_Aggr_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Aggrdecl*f1;};struct Cyc_Absyn_Enum_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;};struct Cyc_Absyn_Datatype_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;};struct Cyc_Absyn_TypeDecl{void*r;unsigned loc;};struct Cyc_Absyn_VoidCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_IntCon_Absyn_TyCon_struct{int tag;enum Cyc_Absyn_Sign f1;enum Cyc_Absyn_Size_of f2;};struct Cyc_Absyn_FloatCon_Absyn_TyCon_struct{int tag;int f1;};struct Cyc_Absyn_RgnHandleCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_TagCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_HeapCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_UniqueCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_RefCntCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_AccessCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_JoinCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_RgnsCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_TrueCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_FalseCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_ThinCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_FatCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_EnumCon_Absyn_TyCon_struct{int tag;struct _tuple0*f1;struct Cyc_Absyn_Enumdecl*f2;};struct Cyc_Absyn_AnonEnumCon_Absyn_TyCon_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_BuiltinCon_Absyn_TyCon_struct{int tag;struct _fat_ptr f1;struct Cyc_Absyn_Kind*f2;};struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_DatatypeInfo f1;};struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_DatatypeFieldInfo f1;};struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_AggrInfo f1;};struct Cyc_Absyn_AppType_Absyn_Type_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Evar_Absyn_Type_struct{int tag;struct Cyc_Core_Opt*f1;void*f2;int f3;struct Cyc_Core_Opt*f4;};struct Cyc_Absyn_VarType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Tvar*f1;};struct Cyc_Absyn_PointerType_Absyn_Type_struct{int tag;struct Cyc_Absyn_PtrInfo f1;};struct Cyc_Absyn_ArrayType_Absyn_Type_struct{int tag;struct Cyc_Absyn_ArrayInfo f1;};struct Cyc_Absyn_FnType_Absyn_Type_struct{int tag;struct Cyc_Absyn_FnInfo f1;};struct Cyc_Absyn_TupleType_Absyn_Type_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct{int tag;enum Cyc_Absyn_AggrKind f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_TypedefType_Absyn_Type_struct{int tag;struct _tuple0*f1;struct Cyc_List_List*f2;struct Cyc_Absyn_Typedefdecl*f3;void*f4;};struct Cyc_Absyn_ValueofType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct{int tag;struct Cyc_Absyn_TypeDecl*f1;void**f2;};struct Cyc_Absyn_TypeofType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_NoTypes_Absyn_Funcparams_struct{int tag;struct Cyc_List_List*f1;unsigned f2;};struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct{int tag;struct Cyc_List_List*f1;int f2;struct Cyc_Absyn_VarargInfo*f3;void*f4;struct Cyc_List_List*f5;struct Cyc_Absyn_Exp*f6;struct Cyc_Absyn_Exp*f7;struct Cyc_List_List*f8;struct Cyc_List_List*f9;};
# 437 "absyn.h"
enum Cyc_Absyn_Format_Type{Cyc_Absyn_Printf_ft =0U,Cyc_Absyn_Scanf_ft =1U};struct Cyc_Absyn_Regparm_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Stdcall_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Cdecl_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Fastcall_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Noreturn_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Const_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Aligned_att_Absyn_Attribute_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Packed_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Section_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Nocommon_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Shared_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Unused_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Weak_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Dllimport_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Dllexport_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_instrument_function_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Constructor_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Destructor_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_check_memory_usage_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Format_att_Absyn_Attribute_struct{int tag;enum Cyc_Absyn_Format_Type f1;int f2;int f3;};struct Cyc_Absyn_Initializes_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Noliveunique_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Consume_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Pure_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Mode_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Alias_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Always_inline_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_throw_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Non_null_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Deprecated_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Carray_mod_Absyn_Type_modifier_struct{int tag;void*f1;unsigned f2;};struct Cyc_Absyn_ConstArray_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_Absyn_Exp*f1;void*f2;unsigned f3;};struct Cyc_Absyn_Pointer_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_Absyn_PtrAtts f1;struct Cyc_Absyn_Tqual f2;};struct Cyc_Absyn_Function_mod_Absyn_Type_modifier_struct{int tag;void*f1;};struct Cyc_Absyn_TypeParams_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_List_List*f1;unsigned f2;int f3;};struct Cyc_Absyn_Attributes_mod_Absyn_Type_modifier_struct{int tag;unsigned f1;struct Cyc_List_List*f2;};struct _union_Cnst_Null_c{int tag;int val;};struct _tuple3{enum Cyc_Absyn_Sign f1;char f2;};struct _union_Cnst_Char_c{int tag;struct _tuple3 val;};struct _union_Cnst_Wchar_c{int tag;struct _fat_ptr val;};struct _tuple4{enum Cyc_Absyn_Sign f1;short f2;};struct _union_Cnst_Short_c{int tag;struct _tuple4 val;};struct _tuple5{enum Cyc_Absyn_Sign f1;int f2;};struct _union_Cnst_Int_c{int tag;struct _tuple5 val;};struct _tuple6{enum Cyc_Absyn_Sign f1;long long f2;};struct _union_Cnst_LongLong_c{int tag;struct _tuple6 val;};struct _tuple7{struct _fat_ptr f1;int f2;};struct _union_Cnst_Float_c{int tag;struct _tuple7 val;};struct _union_Cnst_String_c{int tag;struct _fat_ptr val;};struct _union_Cnst_Wstring_c{int tag;struct _fat_ptr val;};union Cyc_Absyn_Cnst{struct _union_Cnst_Null_c Null_c;struct _union_Cnst_Char_c Char_c;struct _union_Cnst_Wchar_c Wchar_c;struct _union_Cnst_Short_c Short_c;struct _union_Cnst_Int_c Int_c;struct _union_Cnst_LongLong_c LongLong_c;struct _union_Cnst_Float_c Float_c;struct _union_Cnst_String_c String_c;struct _union_Cnst_Wstring_c Wstring_c;};
# 532
enum Cyc_Absyn_Primop{Cyc_Absyn_Plus =0U,Cyc_Absyn_Times =1U,Cyc_Absyn_Minus =2U,Cyc_Absyn_Div =3U,Cyc_Absyn_Mod =4U,Cyc_Absyn_Eq =5U,Cyc_Absyn_Neq =6U,Cyc_Absyn_Gt =7U,Cyc_Absyn_Lt =8U,Cyc_Absyn_Gte =9U,Cyc_Absyn_Lte =10U,Cyc_Absyn_Not =11U,Cyc_Absyn_Bitnot =12U,Cyc_Absyn_Bitand =13U,Cyc_Absyn_Bitor =14U,Cyc_Absyn_Bitxor =15U,Cyc_Absyn_Bitlshift =16U,Cyc_Absyn_Bitlrshift =17U,Cyc_Absyn_Numelts =18U};
# 539
enum Cyc_Absyn_Incrementor{Cyc_Absyn_PreInc =0U,Cyc_Absyn_PostInc =1U,Cyc_Absyn_PreDec =2U,Cyc_Absyn_PostDec =3U};struct Cyc_Absyn_VarargCallInfo{int num_varargs;struct Cyc_List_List*injectors;struct Cyc_Absyn_VarargInfo*vai;};struct Cyc_Absyn_StructField_Absyn_OffsetofField_struct{int tag;struct _fat_ptr*f1;};struct Cyc_Absyn_TupleIndex_Absyn_OffsetofField_struct{int tag;unsigned f1;};
# 557
enum Cyc_Absyn_Coercion{Cyc_Absyn_Unknown_coercion =0U,Cyc_Absyn_No_coercion =1U,Cyc_Absyn_Null_to_NonNull =2U,Cyc_Absyn_Other_coercion =3U};struct Cyc_Absyn_MallocInfo{int is_calloc;struct Cyc_Absyn_Exp*rgn;void**elt_type;struct Cyc_Absyn_Exp*num_elts;int fat_result;int inline_call;};struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct{int tag;union Cyc_Absyn_Cnst f1;};struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Pragma_e_Absyn_Raw_exp_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct{int tag;enum Cyc_Absyn_Primop f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_AssignOp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Core_Opt*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_Increment_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;enum Cyc_Absyn_Incrementor f2;};struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;struct Cyc_Absyn_VarargCallInfo*f3;int f4;};struct Cyc_Absyn_Throw_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;int f2;};struct Cyc_Absyn_NoInstantiate_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Instantiate_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_Absyn_Exp*f2;int f3;enum Cyc_Absyn_Coercion f4;};struct Cyc_Absyn_Address_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_New_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Sizeoftype_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Sizeofexp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Offsetof_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Deref_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;int f3;int f4;};struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;int f3;int f4;};struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Tuple_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;};struct _tuple8{struct _fat_ptr*f1;struct Cyc_Absyn_Tqual f2;void*f3;};struct Cyc_Absyn_CompoundLit_e_Absyn_Raw_exp_struct{int tag;struct _tuple8*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Array_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Exp*f2;struct Cyc_Absyn_Exp*f3;int f4;};struct Cyc_Absyn_ComprehensionNoinit_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;void*f2;int f3;};struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct{int tag;struct _tuple0*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;struct Cyc_Absyn_Aggrdecl*f4;};struct Cyc_Absyn_AnonStruct_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Datatype_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;struct Cyc_Absyn_Datatypedecl*f2;struct Cyc_Absyn_Datatypefield*f3;};struct Cyc_Absyn_Enum_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_AnonEnum_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_Spawn_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_MallocInfo f1;};struct Cyc_Absyn_Swap_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_UnresolvedMem_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Core_Opt*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_StmtExp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Stmt*f1;};struct Cyc_Absyn_Tagcheck_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;};struct Cyc_Absyn_Valueof_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Asm_e_Absyn_Raw_exp_struct{int tag;int f1;struct _fat_ptr f2;struct Cyc_List_List*f3;struct Cyc_List_List*f4;struct Cyc_List_List*f5;};struct Cyc_Absyn_Extension_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Exp{void*topt;void*r;unsigned loc;void*annot;};struct Cyc_Absyn_Skip_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Exp_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Seq_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Return_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_IfThenElse_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Stmt*f2;struct Cyc_Absyn_Stmt*f3;};struct _tuple9{struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_While_s_Absyn_Raw_stmt_struct{int tag;struct _tuple9 f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Break_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Continue_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Goto_s_Absyn_Raw_stmt_struct{int tag;struct _fat_ptr*f1;};struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _tuple9 f2;struct _tuple9 f3;struct Cyc_Absyn_Stmt*f4;};struct Cyc_Absyn_Switch_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;void*f3;};struct Cyc_Absyn_Fallthru_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_List_List*f1;struct Cyc_Absyn_Switch_clause**f2;};struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Decl*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Label_s_Absyn_Raw_stmt_struct{int tag;struct _fat_ptr*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Do_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct _tuple9 f2;};struct Cyc_Absyn_TryCatch_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct Cyc_List_List*f2;void*f3;};struct Cyc_Absyn_Stmt{void*r;unsigned loc;void*annot;};struct Cyc_Absyn_Wild_p_Absyn_Raw_pat_struct{int tag;};struct Cyc_Absyn_Var_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Pat*f2;};struct Cyc_Absyn_AliasVar_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;};struct Cyc_Absyn_Reference_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Pat*f2;};struct Cyc_Absyn_TagInt_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;};struct Cyc_Absyn_Tuple_p_Absyn_Raw_pat_struct{int tag;struct Cyc_List_List*f1;int f2;};struct Cyc_Absyn_Pointer_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Pat*f1;};struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct{int tag;union Cyc_Absyn_AggrInfo*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;int f4;};struct Cyc_Absyn_Datatype_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;struct Cyc_Absyn_Datatypefield*f2;struct Cyc_List_List*f3;int f4;};struct Cyc_Absyn_Null_p_Absyn_Raw_pat_struct{int tag;};struct Cyc_Absyn_Int_p_Absyn_Raw_pat_struct{int tag;enum Cyc_Absyn_Sign f1;int f2;};struct Cyc_Absyn_Char_p_Absyn_Raw_pat_struct{int tag;char f1;};struct Cyc_Absyn_Float_p_Absyn_Raw_pat_struct{int tag;struct _fat_ptr f1;int f2;};struct Cyc_Absyn_Enum_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_AnonEnum_p_Absyn_Raw_pat_struct{int tag;void*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_UnknownId_p_Absyn_Raw_pat_struct{int tag;struct _tuple0*f1;};struct Cyc_Absyn_UnknownCall_p_Absyn_Raw_pat_struct{int tag;struct _tuple0*f1;struct Cyc_List_List*f2;int f3;};struct Cyc_Absyn_Exp_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Pat{void*r;void*topt;unsigned loc;};struct Cyc_Absyn_Switch_clause{struct Cyc_Absyn_Pat*pattern;struct Cyc_Core_Opt*pat_vars;struct Cyc_Absyn_Exp*where_clause;struct Cyc_Absyn_Stmt*body;unsigned loc;};struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct{int tag;struct _tuple0*f1;};struct Cyc_Absyn_Global_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Funname_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Fndecl*f1;};struct Cyc_Absyn_Param_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Local_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Pat_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Vardecl{enum Cyc_Absyn_Scope sc;struct _tuple0*name;unsigned varloc;struct Cyc_Absyn_Tqual tq;void*type;struct Cyc_Absyn_Exp*initializer;void*rgn;struct Cyc_List_List*attributes;int escapes;};struct Cyc_Absyn_Fndecl{enum Cyc_Absyn_Scope sc;int is_inline;struct _tuple0*name;struct Cyc_Absyn_Stmt*body;struct Cyc_Absyn_FnInfo i;void*cached_type;struct Cyc_Core_Opt*param_vardecls;struct Cyc_Absyn_Vardecl*fn_vardecl;};struct Cyc_Absyn_Aggrfield{struct _fat_ptr*name;struct Cyc_Absyn_Tqual tq;void*type;struct Cyc_Absyn_Exp*width;struct Cyc_List_List*attributes;struct Cyc_Absyn_Exp*requires_clause;};struct Cyc_Absyn_AggrdeclImpl{struct Cyc_List_List*exist_vars;struct Cyc_List_List*rgn_po;struct Cyc_List_List*fields;int tagged;};struct Cyc_Absyn_Aggrdecl{enum Cyc_Absyn_AggrKind kind;enum Cyc_Absyn_Scope sc;struct _tuple0*name;struct Cyc_List_List*tvs;struct Cyc_Absyn_AggrdeclImpl*impl;struct Cyc_List_List*attributes;int expected_mem_kind;};struct Cyc_Absyn_Datatypefield{struct _tuple0*name;struct Cyc_List_List*typs;unsigned loc;enum Cyc_Absyn_Scope sc;};struct Cyc_Absyn_Datatypedecl{enum Cyc_Absyn_Scope sc;struct _tuple0*name;struct Cyc_List_List*tvs;struct Cyc_Core_Opt*fields;int is_extensible;};struct Cyc_Absyn_Enumfield{struct _tuple0*name;struct Cyc_Absyn_Exp*tag;unsigned loc;};struct Cyc_Absyn_Enumdecl{enum Cyc_Absyn_Scope sc;struct _tuple0*name;struct Cyc_Core_Opt*fields;};struct Cyc_Absyn_Typedefdecl{struct _tuple0*name;struct Cyc_Absyn_Tqual tq;struct Cyc_List_List*tvs;struct Cyc_Core_Opt*kind;void*defn;struct Cyc_List_List*atts;int extern_c;};struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Fn_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Fndecl*f1;};struct Cyc_Absyn_Let_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Pat*f1;struct Cyc_Core_Opt*f2;struct Cyc_Absyn_Exp*f3;void*f4;};struct Cyc_Absyn_Letv_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_Region_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_Aggr_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Aggrdecl*f1;};struct Cyc_Absyn_Datatype_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;};struct Cyc_Absyn_Enum_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;};struct Cyc_Absyn_Typedef_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Typedefdecl*f1;};struct Cyc_Absyn_Namespace_d_Absyn_Raw_decl_struct{int tag;struct _fat_ptr*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Using_d_Absyn_Raw_decl_struct{int tag;struct _tuple0*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_ExternC_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;};struct _tuple10{unsigned f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_ExternCinclude_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;struct _tuple10*f4;};struct Cyc_Absyn_Porton_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Portoff_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Tempeston_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Tempestoff_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Decl{void*r;unsigned loc;};struct Cyc_Absyn_ArrayElement_Absyn_Designator_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_FieldName_Absyn_Designator_struct{int tag;struct _fat_ptr*f1;};extern char Cyc_Absyn_EmptyAnnot[11U];struct Cyc_Absyn_EmptyAnnot_Absyn_AbsynAnnot_struct{char*tag;};
# 901 "absyn.h"
extern struct Cyc_Absyn_EmptyAnnot_Absyn_AbsynAnnot_struct Cyc_Absyn_EmptyAnnot_val;
# 951
extern void*Cyc_Absyn_void_type;
# 1176
struct Cyc_Absyn_Aggrdecl*Cyc_Absyn_get_known_aggrdecl(union Cyc_Absyn_AggrInfo);
# 1178
int Cyc_Absyn_is_nontagged_nonrequire_union_type(void*);struct _union_RelnOp_RConst{int tag;unsigned val;};struct _union_RelnOp_RVar{int tag;struct Cyc_Absyn_Vardecl*val;};struct _union_RelnOp_RNumelts{int tag;struct Cyc_Absyn_Vardecl*val;};struct _union_RelnOp_RType{int tag;void*val;};struct _union_RelnOp_RParam{int tag;unsigned val;};struct _union_RelnOp_RParamNumelts{int tag;unsigned val;};struct _union_RelnOp_RReturn{int tag;unsigned val;};union Cyc_Relations_RelnOp{struct _union_RelnOp_RConst RConst;struct _union_RelnOp_RVar RVar;struct _union_RelnOp_RNumelts RNumelts;struct _union_RelnOp_RType RType;struct _union_RelnOp_RParam RParam;struct _union_RelnOp_RParamNumelts RParamNumelts;struct _union_RelnOp_RReturn RReturn;};
# 50 "relations-ap.h"
enum Cyc_Relations_Relation{Cyc_Relations_Req =0U,Cyc_Relations_Rneq =1U,Cyc_Relations_Rlte =2U,Cyc_Relations_Rlt =3U};struct Cyc_Relations_Reln{union Cyc_Relations_RelnOp rop1;enum Cyc_Relations_Relation relation;union Cyc_Relations_RelnOp rop2;};
# 114
int Cyc_Relations_relns_approx(struct Cyc_List_List*r2s,struct Cyc_List_List*r1s);
# 116
struct Cyc_List_List*Cyc_Relations_join_relns(struct _RegionHandle*r,struct Cyc_List_List*,struct Cyc_List_List*);
# 118
void Cyc_Relations_print_relns(struct Cyc___cycFILE*,struct Cyc_List_List*);struct Cyc_RgnOrder_RgnPO;extern char Cyc_Tcenv_Env_error[10U];struct Cyc_Tcenv_Env_error_exn_struct{char*tag;};struct Cyc_Tcenv_Genv{struct Cyc_Dict_Dict aggrdecls;struct Cyc_Dict_Dict datatypedecls;struct Cyc_Dict_Dict enumdecls;struct Cyc_Dict_Dict typedefs;struct Cyc_Dict_Dict ordinaries;};struct Cyc_Tcenv_Fenv;struct Cyc_Tcenv_Tenv{struct Cyc_List_List*ns;struct Cyc_Tcenv_Genv*ae;struct Cyc_Tcenv_Fenv*le;int allow_valueof: 1;int in_extern_c_include: 1;int in_tempest: 1;int tempest_generalize: 1;};
# 88 "tcenv.h"
enum Cyc_Tcenv_NewStatus{Cyc_Tcenv_NoneNew =0U,Cyc_Tcenv_InNew =1U,Cyc_Tcenv_InNewAggr =2U};
# 30 "tcutil.h"
void*Cyc_Tcutil_impos(struct _fat_ptr,struct _fat_ptr);
# 57
int Cyc_Tcutil_is_bits_only_type(void*);
# 105
struct Cyc_Absyn_Kind*Cyc_Tcutil_tvar_kind(struct Cyc_Absyn_Tvar*,struct Cyc_Absyn_Kind*def);
# 107
void*Cyc_Tcutil_compress(void*);
# 137
extern struct Cyc_Absyn_Kind Cyc_Tcutil_bk;
# 187
void*Cyc_Tcutil_rsubstitute(struct _RegionHandle*,struct Cyc_List_List*,void*);
# 193
struct Cyc_Absyn_Exp*Cyc_Tcutil_rsubsexp(struct _RegionHandle*,struct Cyc_List_List*,struct Cyc_Absyn_Exp*);
# 236
int Cyc_Tcutil_is_noalias_pointer(void*t,int must_be_unique);
# 305
int Cyc_Tcutil_force_type2bool(int desired,void*);struct Cyc_PP_Ppstate;struct Cyc_PP_Out;struct Cyc_PP_Doc;struct Cyc_Absynpp_Params{int expand_typedefs;int qvar_to_Cids;int add_cyc_prefix;int to_VC;int decls_first;int rewrite_temp_tvars;int print_all_tvars;int print_all_kinds;int print_all_effects;int print_using_stmts;int print_externC_stmts;int print_full_evars;int print_zeroterm;int generate_line_directives;int use_curr_namespace;struct Cyc_List_List*curr_namespace;};
# 62 "absynpp.h"
struct _fat_ptr Cyc_Absynpp_typ2string(void*);
# 67
struct _fat_ptr Cyc_Absynpp_exp2string(struct Cyc_Absyn_Exp*);
# 69
struct _fat_ptr Cyc_Absynpp_qvar2string(struct _tuple0*);
# 33 "warn.h"
void Cyc_Warn_verr(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap);
# 40
void*Cyc_Warn_impos(struct _fat_ptr fmt,struct _fat_ptr ap);struct Cyc_Warn_String_Warn_Warg_struct{int tag;struct _fat_ptr f1;};struct Cyc_Warn_Exp_Warn_Warg_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Warn_Typ_Warn_Warg_struct{int tag;void*f1;};struct Cyc_Warn_Qvar_Warn_Warg_struct{int tag;struct _tuple0*f1;};
# 41 "cf_flowinfo.h"
int Cyc_CfFlowInfo_anal_error;
void Cyc_CfFlowInfo_aerr(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap);struct Cyc_CfFlowInfo_VarRoot_CfFlowInfo_Root_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_CfFlowInfo_MallocPt_CfFlowInfo_Root_struct{int tag;struct Cyc_Absyn_Exp*f1;void*f2;};struct Cyc_CfFlowInfo_InitParam_CfFlowInfo_Root_struct{int tag;int f1;void*f2;};struct Cyc_CfFlowInfo_Dot_CfFlowInfo_PathCon_struct{int tag;int f1;};struct Cyc_CfFlowInfo_Star_CfFlowInfo_PathCon_struct{int tag;};struct Cyc_CfFlowInfo_Place{void*root;struct Cyc_List_List*path;};
# 74
enum Cyc_CfFlowInfo_InitLevel{Cyc_CfFlowInfo_NoneIL =0U,Cyc_CfFlowInfo_AllIL =1U};char Cyc_CfFlowInfo_IsZero[7U]="IsZero";struct Cyc_CfFlowInfo_IsZero_Absyn_AbsynAnnot_struct{char*tag;};char Cyc_CfFlowInfo_NotZero[8U]="NotZero";struct Cyc_CfFlowInfo_NotZero_Absyn_AbsynAnnot_struct{char*tag;struct Cyc_List_List*f1;};char Cyc_CfFlowInfo_UnknownZ[9U]="UnknownZ";struct Cyc_CfFlowInfo_UnknownZ_Absyn_AbsynAnnot_struct{char*tag;struct Cyc_List_List*f1;};struct _union_AbsLVal_PlaceL{int tag;struct Cyc_CfFlowInfo_Place*val;};struct _union_AbsLVal_UnknownL{int tag;int val;};union Cyc_CfFlowInfo_AbsLVal{struct _union_AbsLVal_PlaceL PlaceL;struct _union_AbsLVal_UnknownL UnknownL;};struct Cyc_CfFlowInfo_UnionRInfo{int is_union;int fieldnum;};struct Cyc_CfFlowInfo_Zero_CfFlowInfo_AbsRVal_struct{int tag;};struct Cyc_CfFlowInfo_NotZeroAll_CfFlowInfo_AbsRVal_struct{int tag;};struct Cyc_CfFlowInfo_UnknownR_CfFlowInfo_AbsRVal_struct{int tag;enum Cyc_CfFlowInfo_InitLevel f1;};struct Cyc_CfFlowInfo_Esc_CfFlowInfo_AbsRVal_struct{int tag;enum Cyc_CfFlowInfo_InitLevel f1;};struct Cyc_CfFlowInfo_AddressOf_CfFlowInfo_AbsRVal_struct{int tag;struct Cyc_CfFlowInfo_Place*f1;};struct Cyc_CfFlowInfo_UniquePtr_CfFlowInfo_AbsRVal_struct{int tag;void*f1;};struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct{int tag;struct Cyc_CfFlowInfo_UnionRInfo f1;struct _fat_ptr f2;};struct Cyc_CfFlowInfo_Consumed_CfFlowInfo_AbsRVal_struct{int tag;struct Cyc_Absyn_Exp*f1;int f2;void*f3;};struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct{int tag;struct Cyc_Absyn_Vardecl*f1;void*f2;};struct _union_FlowInfo_BottomFL{int tag;int val;};struct _tuple11{struct Cyc_Dict_Dict f1;struct Cyc_List_List*f2;};struct _union_FlowInfo_ReachableFL{int tag;struct _tuple11 val;};union Cyc_CfFlowInfo_FlowInfo{struct _union_FlowInfo_BottomFL BottomFL;struct _union_FlowInfo_ReachableFL ReachableFL;};
# 139 "cf_flowinfo.h"
union Cyc_CfFlowInfo_FlowInfo Cyc_CfFlowInfo_ReachableFL(struct Cyc_Dict_Dict,struct Cyc_List_List*);struct Cyc_CfFlowInfo_FlowEnv{void*zero;void*notzeroall;void*unknown_none;void*unknown_all;void*esc_none;void*esc_all;struct Cyc_Dict_Dict mt_flowdict;struct Cyc_CfFlowInfo_Place*dummy_place;};
# 156
int Cyc_CfFlowInfo_get_field_index_fs(struct Cyc_List_List*fs,struct _fat_ptr*f);
# 158
int Cyc_CfFlowInfo_root_cmp(void*,void*);
int Cyc_CfFlowInfo_place_cmp(struct Cyc_CfFlowInfo_Place*,struct Cyc_CfFlowInfo_Place*);
# 161
struct _fat_ptr Cyc_CfFlowInfo_aggrfields_to_aggrdict(struct Cyc_CfFlowInfo_FlowEnv*,struct Cyc_List_List*,int no_init_bits_only,void*);
# 163
void*Cyc_CfFlowInfo_make_unique_consumed(struct Cyc_CfFlowInfo_FlowEnv*fenv,void*t,struct Cyc_Absyn_Exp*consumer,int iteration,void*);
int Cyc_CfFlowInfo_is_unique_consumed(struct Cyc_Absyn_Exp*e,int env_iteration,void*r,int*needs_unconsume);
void*Cyc_CfFlowInfo_make_unique_unconsumed(struct Cyc_CfFlowInfo_FlowEnv*fenv,void*r);
# 168
enum Cyc_CfFlowInfo_InitLevel Cyc_CfFlowInfo_initlevel(struct Cyc_CfFlowInfo_FlowEnv*,struct Cyc_Dict_Dict d,void*r);
void*Cyc_CfFlowInfo_lookup_place(struct Cyc_Dict_Dict d,struct Cyc_CfFlowInfo_Place*place);
# 171
int Cyc_CfFlowInfo_is_init_pointer(void*r);
int Cyc_CfFlowInfo_flow_lessthan_approx(union Cyc_CfFlowInfo_FlowInfo f1,union Cyc_CfFlowInfo_FlowInfo f2);
# 174
void Cyc_CfFlowInfo_print_absrval(void*rval);
void Cyc_CfFlowInfo_print_initlevel(enum Cyc_CfFlowInfo_InitLevel il);
void Cyc_CfFlowInfo_print_root(void*root);
void Cyc_CfFlowInfo_print_path(struct Cyc_List_List*p);
void Cyc_CfFlowInfo_print_place(struct Cyc_CfFlowInfo_Place*p);
# 180
void Cyc_CfFlowInfo_print_flowdict(struct Cyc_Dict_Dict d);
# 201 "cf_flowinfo.h"
union Cyc_CfFlowInfo_FlowInfo Cyc_CfFlowInfo_join_flow(struct Cyc_CfFlowInfo_FlowEnv*,union Cyc_CfFlowInfo_FlowInfo,union Cyc_CfFlowInfo_FlowInfo);
# 38 "cf_flowinfo.cyc"
int Cyc_CfFlowInfo_anal_error=0;
void Cyc_CfFlowInfo_aerr(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap){
# 41
Cyc_CfFlowInfo_anal_error=1;
Cyc_Warn_verr(loc,fmt,ap);}
# 39
struct Cyc_CfFlowInfo_IsZero_Absyn_AbsynAnnot_struct Cyc_CfFlowInfo_IsZero_val={Cyc_CfFlowInfo_IsZero};
# 47
union Cyc_CfFlowInfo_AbsLVal Cyc_CfFlowInfo_PlaceL(struct Cyc_CfFlowInfo_Place*x){
return({union Cyc_CfFlowInfo_AbsLVal _tmp1D7;(_tmp1D7.PlaceL).tag=1U,(_tmp1D7.PlaceL).val=x;_tmp1D7;});}
# 47
union Cyc_CfFlowInfo_AbsLVal Cyc_CfFlowInfo_UnknownL(){
# 51
return({union Cyc_CfFlowInfo_AbsLVal _tmp1D8;(_tmp1D8.UnknownL).tag=2U,(_tmp1D8.UnknownL).val=0;_tmp1D8;});}
# 47
union Cyc_CfFlowInfo_FlowInfo Cyc_CfFlowInfo_BottomFL(){
# 53
return({union Cyc_CfFlowInfo_FlowInfo _tmp1D9;(_tmp1D9.BottomFL).tag=1U,(_tmp1D9.BottomFL).val=0;_tmp1D9;});}
# 47
union Cyc_CfFlowInfo_FlowInfo Cyc_CfFlowInfo_ReachableFL(struct Cyc_Dict_Dict fd,struct Cyc_List_List*r){
# 55
return({union Cyc_CfFlowInfo_FlowInfo _tmp1DA;(_tmp1DA.ReachableFL).tag=2U,((_tmp1DA.ReachableFL).val).f1=fd,((_tmp1DA.ReachableFL).val).f2=r;_tmp1DA;});}
# 47
struct Cyc_CfFlowInfo_FlowEnv*Cyc_CfFlowInfo_new_flow_env(){
# 59
static struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct dummy_rawexp={0U,{.Null_c={1,0}}};
static struct Cyc_Absyn_Exp dummy_exp={0,(void*)& dummy_rawexp,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};
return({struct Cyc_CfFlowInfo_FlowEnv*_tmp8=_cycalloc(sizeof(*_tmp8));
({void*_tmp214=(void*)({struct Cyc_CfFlowInfo_Zero_CfFlowInfo_AbsRVal_struct*_tmp0=_cycalloc(sizeof(*_tmp0));_tmp0->tag=0U;_tmp0;});_tmp8->zero=_tmp214;}),({
void*_tmp213=(void*)({struct Cyc_CfFlowInfo_NotZeroAll_CfFlowInfo_AbsRVal_struct*_tmp1=_cycalloc(sizeof(*_tmp1));_tmp1->tag=1U;_tmp1;});_tmp8->notzeroall=_tmp213;}),({
void*_tmp212=(void*)({struct Cyc_CfFlowInfo_UnknownR_CfFlowInfo_AbsRVal_struct*_tmp2=_cycalloc(sizeof(*_tmp2));_tmp2->tag=2U,_tmp2->f1=Cyc_CfFlowInfo_NoneIL;_tmp2;});_tmp8->unknown_none=_tmp212;}),({
void*_tmp211=(void*)({struct Cyc_CfFlowInfo_UnknownR_CfFlowInfo_AbsRVal_struct*_tmp3=_cycalloc(sizeof(*_tmp3));_tmp3->tag=2U,_tmp3->f1=Cyc_CfFlowInfo_AllIL;_tmp3;});_tmp8->unknown_all=_tmp211;}),({
void*_tmp210=(void*)({struct Cyc_CfFlowInfo_Esc_CfFlowInfo_AbsRVal_struct*_tmp4=_cycalloc(sizeof(*_tmp4));_tmp4->tag=3U,_tmp4->f1=Cyc_CfFlowInfo_NoneIL;_tmp4;});_tmp8->esc_none=_tmp210;}),({
void*_tmp20F=(void*)({struct Cyc_CfFlowInfo_Esc_CfFlowInfo_AbsRVal_struct*_tmp5=_cycalloc(sizeof(*_tmp5));_tmp5->tag=3U,_tmp5->f1=Cyc_CfFlowInfo_AllIL;_tmp5;});_tmp8->esc_all=_tmp20F;}),({
struct Cyc_Dict_Dict _tmp20E=Cyc_Dict_empty(Cyc_CfFlowInfo_root_cmp);_tmp8->mt_flowdict=_tmp20E;}),({
struct Cyc_CfFlowInfo_Place*_tmp20D=({struct Cyc_CfFlowInfo_Place*_tmp7=_cycalloc(sizeof(*_tmp7));({void*_tmp20C=(void*)({struct Cyc_CfFlowInfo_MallocPt_CfFlowInfo_Root_struct*_tmp6=_cycalloc(sizeof(*_tmp6));_tmp6->tag=1U,_tmp6->f1=& dummy_exp,_tmp6->f2=Cyc_Absyn_void_type;_tmp6;});_tmp7->root=_tmp20C;}),_tmp7->path=0;_tmp7;});_tmp8->dummy_place=_tmp20D;});_tmp8;});}
# 47
struct _fat_ptr Cyc_CfFlowInfo_place_err_string(struct Cyc_CfFlowInfo_Place*place){
# 75
struct Cyc_CfFlowInfo_Place _stmttmp0=*place;struct Cyc_List_List*_tmpA;void*_tmp9;_LL1: _tmp9=_stmttmp0.root;_tmpA=_stmttmp0.path;_LL2: {void*root=_tmp9;struct Cyc_List_List*fields=_tmpA;
void*_tmpB=root;struct Cyc_Absyn_Vardecl*_tmpC;if(((struct Cyc_CfFlowInfo_VarRoot_CfFlowInfo_Root_struct*)_tmpB)->tag == 0U){_LL4: _tmpC=((struct Cyc_CfFlowInfo_VarRoot_CfFlowInfo_Root_struct*)_tmpB)->f1;_LL5: {struct Cyc_Absyn_Vardecl*vd=_tmpC;
# 78
if(fields == 0)
return Cyc_Absynpp_qvar2string(vd->name);else{
# 81
return(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmpF=({struct Cyc_String_pa_PrintArg_struct _tmp1DB;_tmp1DB.tag=0U,({struct _fat_ptr _tmp215=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(vd->name));_tmp1DB.f1=_tmp215;});_tmp1DB;});void*_tmpD[1U];_tmpD[0]=& _tmpF;({struct _fat_ptr _tmp216=({const char*_tmpE="reachable from %s";_tag_fat(_tmpE,sizeof(char),18U);});Cyc_aprintf(_tmp216,_tag_fat(_tmpD,sizeof(void*),1U));});});}}}else{_LL6: _LL7:
({void*_tmp10=0U;({int(*_tmp218)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp12)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp12;});struct _fat_ptr _tmp217=({const char*_tmp11="error locations not for VarRoots";_tag_fat(_tmp11,sizeof(char),33U);});_tmp218(_tmp217,_tag_fat(_tmp10,sizeof(void*),0U));});});}_LL3:;}}
# 47
int Cyc_CfFlowInfo_get_field_index_fs(struct Cyc_List_List*fs,struct _fat_ptr*f){
# 88
int n=0;
for(0;(unsigned)fs;fs=fs->tl){
struct _fat_ptr*f2=((struct Cyc_Absyn_Aggrfield*)fs->hd)->name;
if(Cyc_strptrcmp(f2,f)== 0)return n;n=n + 1;}
# 94
({struct Cyc_String_pa_PrintArg_struct _tmp16=({struct Cyc_String_pa_PrintArg_struct _tmp1DC;_tmp1DC.tag=0U,_tmp1DC.f1=(struct _fat_ptr)((struct _fat_ptr)*f);_tmp1DC;});void*_tmp13[1U];_tmp13[0]=& _tmp16;({int(*_tmp21A)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp15)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmp15;});struct _fat_ptr _tmp219=({const char*_tmp14="get_field_index_fs failed: %s";_tag_fat(_tmp14,sizeof(char),30U);});_tmp21A(_tmp219,_tag_fat(_tmp13,sizeof(void*),1U));});});}
# 47
int Cyc_CfFlowInfo_get_field_index(void*t,struct _fat_ptr*f){
# 98
void*_stmttmp1=Cyc_Tcutil_compress(t);void*_tmp17=_stmttmp1;struct Cyc_List_List*_tmp18;union Cyc_Absyn_AggrInfo _tmp19;switch(*((int*)_tmp17)){case 0U: if(((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp17)->f1)->tag == 20U){_LL1: _tmp19=((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp17)->f1)->f1;_LL2: {union Cyc_Absyn_AggrInfo info=_tmp19;
# 100
struct Cyc_List_List*fs=((struct Cyc_Absyn_AggrdeclImpl*)_check_null((Cyc_Absyn_get_known_aggrdecl(info))->impl))->fields;
_tmp18=fs;goto _LL4;}}else{goto _LL5;}case 7U: _LL3: _tmp18=((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmp17)->f2;_LL4: {struct Cyc_List_List*fs=_tmp18;
# 103
return Cyc_CfFlowInfo_get_field_index_fs(fs,f);}default: _LL5: _LL6:
# 105
({struct Cyc_String_pa_PrintArg_struct _tmp1D=({struct Cyc_String_pa_PrintArg_struct _tmp1DD;_tmp1DD.tag=0U,({struct _fat_ptr _tmp21B=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t));_tmp1DD.f1=_tmp21B;});_tmp1DD;});void*_tmp1A[1U];_tmp1A[0]=& _tmp1D;({int(*_tmp21D)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp1C)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmp1C;});struct _fat_ptr _tmp21C=({const char*_tmp1B="get_field_index failed: %s";_tag_fat(_tmp1B,sizeof(char),27U);});_tmp21D(_tmp21C,_tag_fat(_tmp1A,sizeof(void*),1U));});});}_LL0:;}struct _tuple12{void*f1;void*f2;};
# 47
int Cyc_CfFlowInfo_root_cmp(void*r1,void*r2){
# 110
if((int)r1 == (int)r2)
return 0;{
# 110
struct _tuple12 _stmttmp2=({struct _tuple12 _tmp1DE;_tmp1DE.f1=r1,_tmp1DE.f2=r2;_tmp1DE;});struct _tuple12 _tmp1E=_stmttmp2;int _tmp20;int _tmp1F;struct Cyc_Absyn_Exp*_tmp22;struct Cyc_Absyn_Exp*_tmp21;struct Cyc_Absyn_Vardecl*_tmp24;struct Cyc_Absyn_Vardecl*_tmp23;if(((struct Cyc_CfFlowInfo_VarRoot_CfFlowInfo_Root_struct*)_tmp1E.f1)->tag == 0U){if(((struct Cyc_CfFlowInfo_VarRoot_CfFlowInfo_Root_struct*)_tmp1E.f2)->tag == 0U){_LL1: _tmp23=((struct Cyc_CfFlowInfo_VarRoot_CfFlowInfo_Root_struct*)_tmp1E.f1)->f1;_tmp24=((struct Cyc_CfFlowInfo_VarRoot_CfFlowInfo_Root_struct*)_tmp1E.f2)->f1;_LL2: {struct Cyc_Absyn_Vardecl*vd1=_tmp23;struct Cyc_Absyn_Vardecl*vd2=_tmp24;
# 113
return(int)vd1 - (int)vd2;}}else{_LL3: _LL4:
 return - 1;}}else{if(((struct Cyc_CfFlowInfo_VarRoot_CfFlowInfo_Root_struct*)_tmp1E.f2)->tag == 0U){_LL5: _LL6:
 return 1;}else{if(((struct Cyc_CfFlowInfo_MallocPt_CfFlowInfo_Root_struct*)_tmp1E.f1)->tag == 1U){if(((struct Cyc_CfFlowInfo_MallocPt_CfFlowInfo_Root_struct*)_tmp1E.f2)->tag == 1U){_LL7: _tmp21=((struct Cyc_CfFlowInfo_MallocPt_CfFlowInfo_Root_struct*)_tmp1E.f1)->f1;_tmp22=((struct Cyc_CfFlowInfo_MallocPt_CfFlowInfo_Root_struct*)_tmp1E.f2)->f1;_LL8: {struct Cyc_Absyn_Exp*e1=_tmp21;struct Cyc_Absyn_Exp*e2=_tmp22;
return(int)e1 - (int)e2;}}else{_LL9: _LLA:
 return - 1;}}else{if(((struct Cyc_CfFlowInfo_MallocPt_CfFlowInfo_Root_struct*)_tmp1E.f2)->tag == 1U){_LLB: _LLC:
 return 1;}else{_LLD: _tmp1F=((struct Cyc_CfFlowInfo_InitParam_CfFlowInfo_Root_struct*)_tmp1E.f1)->f1;_tmp20=((struct Cyc_CfFlowInfo_InitParam_CfFlowInfo_Root_struct*)_tmp1E.f2)->f1;_LLE: {int i1=_tmp1F;int i2=_tmp20;
return i1 - i2;}}}}}_LL0:;}}
# 47
static int Cyc_CfFlowInfo_pathcon_cmp(void*p1,void*p2){
# 124
struct _tuple12 _stmttmp3=({struct _tuple12 _tmp1DF;_tmp1DF.f1=p1,_tmp1DF.f2=p2;_tmp1DF;});struct _tuple12 _tmp25=_stmttmp3;int _tmp27;int _tmp26;if(((struct Cyc_CfFlowInfo_Dot_CfFlowInfo_PathCon_struct*)_tmp25.f1)->tag == 0U){if(((struct Cyc_CfFlowInfo_Dot_CfFlowInfo_PathCon_struct*)_tmp25.f2)->tag == 0U){_LL1: _tmp26=((struct Cyc_CfFlowInfo_Dot_CfFlowInfo_PathCon_struct*)_tmp25.f1)->f1;_tmp27=((struct Cyc_CfFlowInfo_Dot_CfFlowInfo_PathCon_struct*)_tmp25.f2)->f1;_LL2: {int i1=_tmp26;int i2=_tmp27;
# 126
if(i1 == i2)return 0;if(i1 < i2)
return - 1;else{
return 1;}}}else{_LL7: _LL8:
# 131
 return 1;}}else{if(((struct Cyc_CfFlowInfo_Star_CfFlowInfo_PathCon_struct*)_tmp25.f2)->tag == 1U){_LL3: _LL4:
# 129
 return 0;}else{_LL5: _LL6:
 return - 1;}}_LL0:;}
# 47
static int Cyc_CfFlowInfo_path_cmp(struct Cyc_List_List*path1,struct Cyc_List_List*path2){
# 136
return Cyc_List_list_cmp(Cyc_CfFlowInfo_pathcon_cmp,path1,path2);}
# 47
int Cyc_CfFlowInfo_place_cmp(struct Cyc_CfFlowInfo_Place*p1,struct Cyc_CfFlowInfo_Place*p2){
# 140
if((int)p1 == (int)p2)
return 0;{
# 140
struct Cyc_CfFlowInfo_Place _stmttmp4=*p1;struct Cyc_List_List*_tmp29;void*_tmp28;_LL1: _tmp28=_stmttmp4.root;_tmp29=_stmttmp4.path;_LL2: {void*root1=_tmp28;struct Cyc_List_List*path1=_tmp29;
# 143
struct Cyc_CfFlowInfo_Place _stmttmp5=*p2;struct Cyc_List_List*_tmp2B;void*_tmp2A;_LL4: _tmp2A=_stmttmp5.root;_tmp2B=_stmttmp5.path;_LL5: {void*root2=_tmp2A;struct Cyc_List_List*path2=_tmp2B;
int i=Cyc_CfFlowInfo_root_cmp(root1,root2);
if(i != 0)
return i;
# 145
return
# 147
 Cyc_CfFlowInfo_path_cmp(path1,path2);}}}}
# 47
static void*Cyc_CfFlowInfo_i_typ_to_absrval(struct Cyc_CfFlowInfo_FlowEnv*fenv,int allow_zeroterm,int no_init_bits_only,void*t,void*leafval);
# 165 "cf_flowinfo.cyc"
struct _fat_ptr Cyc_CfFlowInfo_aggrfields_to_aggrdict(struct Cyc_CfFlowInfo_FlowEnv*fenv,struct Cyc_List_List*fs,int no_init_bits_only,void*leafval){
# 171
unsigned sz=(unsigned)Cyc_List_length(fs);
struct _fat_ptr d=({unsigned _tmp2F=sz;void**_tmp2E=_cycalloc(_check_times(_tmp2F,sizeof(void*)));({{unsigned _tmp1E0=sz;unsigned i;for(i=0;i < _tmp1E0;++ i){_tmp2E[i]=fenv->unknown_all;}}0;});_tag_fat(_tmp2E,sizeof(void*),_tmp2F);});
{int i=0;for(0;(unsigned)i < sz;(i ++,fs=fs->tl)){
struct Cyc_Absyn_Aggrfield*_stmttmp6=(struct Cyc_Absyn_Aggrfield*)((struct Cyc_List_List*)_check_null(fs))->hd;void*_tmp2D;struct _fat_ptr*_tmp2C;_LL1: _tmp2C=_stmttmp6->name;_tmp2D=_stmttmp6->type;_LL2: {struct _fat_ptr*n=_tmp2C;void*t2=_tmp2D;
if(_get_fat_size(*n,sizeof(char))!= (unsigned)1)
({void*_tmp21E=Cyc_CfFlowInfo_i_typ_to_absrval(fenv,0,no_init_bits_only,t2,leafval);*((void**)_check_fat_subscript(d,sizeof(void*),i))=_tmp21E;});}}}
# 179
return d;}struct _tuple13{struct _RegionHandle*f1;struct Cyc_List_List*f2;};
# 165
static struct Cyc_Absyn_Aggrfield*Cyc_CfFlowInfo_substitute_field(struct _tuple13*env,struct Cyc_Absyn_Aggrfield*f){
# 187
struct Cyc_List_List*_tmp31;struct _RegionHandle*_tmp30;_LL1: _tmp30=env->f1;_tmp31=env->f2;_LL2: {struct _RegionHandle*t=_tmp30;struct Cyc_List_List*inst=_tmp31;
void*new_typ=Cyc_Tcutil_rsubstitute(t,inst,f->type);
struct Cyc_Absyn_Exp*r=f->requires_clause;
struct Cyc_Absyn_Exp*new_r=r == 0?0: Cyc_Tcutil_rsubsexp(t,inst,r);
return({struct Cyc_Absyn_Aggrfield*_tmp32=_region_malloc(t,sizeof(*_tmp32));_tmp32->name=f->name,_tmp32->tq=f->tq,_tmp32->type=new_typ,_tmp32->width=f->width,_tmp32->attributes=f->attributes,_tmp32->requires_clause=new_r;_tmp32;});}}struct _tuple14{struct Cyc_Absyn_Tvar*f1;void*f2;};
# 165
static struct _fat_ptr Cyc_CfFlowInfo_substitute_aggrfields_to_aggrdict(struct Cyc_CfFlowInfo_FlowEnv*fenv,struct Cyc_List_List*tvs,struct Cyc_List_List*targs,struct Cyc_List_List*fs,int no_init_bits_only,void*leafval){
# 204
struct _RegionHandle _tmp33=_new_region("temp");struct _RegionHandle*temp=& _tmp33;_push_region(temp);
# 208
{struct Cyc_List_List*inst=0;
for(0;tvs != 0;(tvs=tvs->tl,targs=targs->tl)){
struct Cyc_Absyn_Tvar*tv=(struct Cyc_Absyn_Tvar*)tvs->hd;
void*t=(void*)((struct Cyc_List_List*)_check_null(targs))->hd;
{struct Cyc_Absyn_Kind*_stmttmp7=Cyc_Tcutil_tvar_kind(tv,& Cyc_Tcutil_bk);struct Cyc_Absyn_Kind*_tmp34=_stmttmp7;switch(((struct Cyc_Absyn_Kind*)_tmp34)->kind){case Cyc_Absyn_RgnKind: _LL1: _LL2:
 goto _LL4;case Cyc_Absyn_EffKind: _LL3: _LL4:
 continue;default: _LL5: _LL6:
 goto _LL0;}_LL0:;}
# 217
inst=({struct Cyc_List_List*_tmp36=_region_malloc(temp,sizeof(*_tmp36));({struct _tuple14*_tmp21F=({struct _tuple14*_tmp35=_region_malloc(temp,sizeof(*_tmp35));_tmp35->f1=tv,_tmp35->f2=t;_tmp35;});_tmp36->hd=_tmp21F;}),_tmp36->tl=inst;_tmp36;});}
# 219
if(inst != 0){
struct _tuple13 env=({struct _tuple13 _tmp1E1;_tmp1E1.f1=temp,({struct Cyc_List_List*_tmp220=Cyc_List_imp_rev(inst);_tmp1E1.f2=_tmp220;});_tmp1E1;});
struct Cyc_List_List*subs_fs=({struct Cyc_List_List*(*_tmp222)(struct _RegionHandle*,struct Cyc_Absyn_Aggrfield*(*f)(struct _tuple13*,struct Cyc_Absyn_Aggrfield*),struct _tuple13*env,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp38)(struct _RegionHandle*,struct Cyc_Absyn_Aggrfield*(*f)(struct _tuple13*,struct Cyc_Absyn_Aggrfield*),struct _tuple13*env,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct _RegionHandle*,struct Cyc_Absyn_Aggrfield*(*f)(struct _tuple13*,struct Cyc_Absyn_Aggrfield*),struct _tuple13*env,struct Cyc_List_List*x))Cyc_List_rmap_c;_tmp38;});struct _RegionHandle*_tmp221=temp;_tmp222(_tmp221,Cyc_CfFlowInfo_substitute_field,& env,fs);});
struct _fat_ptr _tmp37=Cyc_CfFlowInfo_aggrfields_to_aggrdict(fenv,subs_fs,no_init_bits_only,leafval);_npop_handler(0U);return _tmp37;}else{
# 224
struct _fat_ptr _tmp39=Cyc_CfFlowInfo_aggrfields_to_aggrdict(fenv,fs,no_init_bits_only,leafval);_npop_handler(0U);return _tmp39;}}
# 208
;_pop_region(temp);}struct _tuple15{struct Cyc_Absyn_Tqual f1;void*f2;};
# 165
static void*Cyc_CfFlowInfo_i_typ_to_absrval(struct Cyc_CfFlowInfo_FlowEnv*fenv,int allow_zeroterm,int no_init_bits_only,void*t,void*leafval){
# 234
if(Cyc_Absyn_is_nontagged_nonrequire_union_type(t))return fenv->unknown_all;{void*_stmttmp8=
# 236
Cyc_Tcutil_compress(t);
# 234
void*_tmp3A=_stmttmp8;void*_tmp3B;void*_tmp3D;void*_tmp3C;struct Cyc_List_List*_tmp3F;enum Cyc_Absyn_AggrKind _tmp3E;struct Cyc_List_List*_tmp40;void*_tmp41;struct Cyc_List_List*_tmp43;union Cyc_Absyn_AggrInfo _tmp42;struct Cyc_Absyn_Datatypefield*_tmp44;switch(*((int*)_tmp3A)){case 0U: switch(*((int*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp3A)->f1)){case 19U: if(((((struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp3A)->f1)->f1).KnownDatatypefield).tag == 2){_LL1: _tmp44=(((((struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp3A)->f1)->f1).KnownDatatypefield).val).f2;_LL2: {struct Cyc_Absyn_Datatypefield*tuf=_tmp44;
# 238
if(tuf->typs == 0)
return leafval;
# 238
_tmp40=tuf->typs;goto _LL4;}}else{goto _LLF;}case 20U: _LL5: _tmp42=((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp3A)->f1)->f1;_tmp43=((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp3A)->f2;_LL6: {union Cyc_Absyn_AggrInfo info=_tmp42;struct Cyc_List_List*targs=_tmp43;
# 250
struct Cyc_Absyn_Aggrdecl*ad=Cyc_Absyn_get_known_aggrdecl(info);
if(ad->impl == 0)
goto _LL0;{
# 251
struct Cyc_List_List*fields=((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->fields;
# 254
if(targs == 0){_tmp3E=ad->kind;_tmp3F=fields;goto _LL8;}return(void*)({struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*_tmp48=_cycalloc(sizeof(*_tmp48));
_tmp48->tag=6U,(_tmp48->f1).is_union=(int)ad->kind == (int)Cyc_Absyn_UnionA,(_tmp48->f1).fieldnum=- 1,({
struct _fat_ptr _tmp223=Cyc_CfFlowInfo_substitute_aggrfields_to_aggrdict(fenv,ad->tvs,targs,fields,(int)ad->kind == (int)Cyc_Absyn_UnionA,leafval);_tmp48->f2=_tmp223;});_tmp48;});}}case 4U: if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp3A)->f2 != 0){_LLB: _tmp41=(void*)(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp3A)->f2)->hd;_LLC: {void*t=_tmp41;
# 273
return leafval;}}else{goto _LLF;}default: goto _LLF;}case 6U: _LL3: _tmp40=((struct Cyc_Absyn_TupleType_Absyn_Type_struct*)_tmp3A)->f1;_LL4: {struct Cyc_List_List*tqts=_tmp40;
# 242
unsigned sz=(unsigned)Cyc_List_length(tqts);
struct _fat_ptr d=({unsigned _tmp47=sz;void**_tmp46=_cycalloc(_check_times(_tmp47,sizeof(void*)));({{unsigned _tmp1E2=sz;unsigned i;for(i=0;i < _tmp1E2;++ i){_tmp46[i]=fenv->unknown_all;}}0;});_tag_fat(_tmp46,sizeof(void*),_tmp47);});
{int i=0;for(0;(unsigned)i < sz;++ i){
({void*_tmp224=Cyc_CfFlowInfo_i_typ_to_absrval(fenv,0,no_init_bits_only,(*((struct _tuple15*)((struct Cyc_List_List*)_check_null(tqts))->hd)).f2,leafval);*((void**)_check_fat_subscript(d,sizeof(void*),i))=_tmp224;});
tqts=tqts->tl;}}
# 248
return(void*)({struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*_tmp45=_cycalloc(sizeof(*_tmp45));_tmp45->tag=6U,(_tmp45->f1).is_union=0,(_tmp45->f1).fieldnum=- 1,_tmp45->f2=d;_tmp45;});}case 7U: _LL7: _tmp3E=((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmp3A)->f1;_tmp3F=((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmp3A)->f2;_LL8: {enum Cyc_Absyn_AggrKind k=_tmp3E;struct Cyc_List_List*fs=_tmp3F;
# 265
return(void*)({struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*_tmp49=_cycalloc(sizeof(*_tmp49));_tmp49->tag=6U,(_tmp49->f1).is_union=(int)k == (int)Cyc_Absyn_UnionA,(_tmp49->f1).fieldnum=- 1,({
struct _fat_ptr _tmp225=Cyc_CfFlowInfo_aggrfields_to_aggrdict(fenv,fs,(int)k == (int)Cyc_Absyn_UnionA,leafval);_tmp49->f2=_tmp225;});_tmp49;});}case 4U: _LL9: _tmp3C=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp3A)->f1).elt_type;_tmp3D=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp3A)->f1).zero_term;_LLA: {void*et=_tmp3C;void*zeroterm=_tmp3D;
# 268
if(Cyc_Tcutil_force_type2bool(0,zeroterm))
# 271
return(allow_zeroterm && !no_init_bits_only)&& Cyc_Tcutil_is_bits_only_type(et)?fenv->unknown_all: leafval;
# 268
goto _LL0;}case 3U: _LLD: _tmp3B=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp3A)->f1).ptr_atts).nullable;_LLE: {void*nbl=_tmp3B;
# 275
if(!Cyc_Tcutil_force_type2bool(0,nbl)){
void*_tmp4A=leafval;if(((struct Cyc_CfFlowInfo_UnknownR_CfFlowInfo_AbsRVal_struct*)_tmp4A)->tag == 2U){if(((struct Cyc_CfFlowInfo_UnknownR_CfFlowInfo_AbsRVal_struct*)_tmp4A)->f1 == Cyc_CfFlowInfo_AllIL){_LL12: _LL13:
 return fenv->notzeroall;}else{goto _LL14;}}else{_LL14: _LL15:
 goto _LL11;}_LL11:;}
# 275
goto _LL0;}default: _LLF: _LL10:
# 281
 goto _LL0;}_LL0:;}
# 284
return !no_init_bits_only && Cyc_Tcutil_is_bits_only_type(t)?fenv->unknown_all: leafval;}
# 165
void*Cyc_CfFlowInfo_typ_to_absrval(struct Cyc_CfFlowInfo_FlowEnv*fenv,void*t,int no_init_bits_only,void*leafval){
# 288
return Cyc_CfFlowInfo_i_typ_to_absrval(fenv,1,no_init_bits_only,t,leafval);}
# 165
int Cyc_CfFlowInfo_is_unique_consumed(struct Cyc_Absyn_Exp*e,int env_iteration,void*r,int*needs_unconsume){
# 294
void*_tmp4B=r;void*_tmp4C;struct _fat_ptr _tmp4F;int _tmp4E;int _tmp4D;void*_tmp52;int _tmp51;struct Cyc_Absyn_Exp*_tmp50;switch(*((int*)_tmp4B)){case 7U: _LL1: _tmp50=((struct Cyc_CfFlowInfo_Consumed_CfFlowInfo_AbsRVal_struct*)_tmp4B)->f1;_tmp51=((struct Cyc_CfFlowInfo_Consumed_CfFlowInfo_AbsRVal_struct*)_tmp4B)->f2;_tmp52=(void*)((struct Cyc_CfFlowInfo_Consumed_CfFlowInfo_AbsRVal_struct*)_tmp4B)->f3;_LL2: {struct Cyc_Absyn_Exp*consumer=_tmp50;int iteration=_tmp51;void*r=_tmp52;
# 296
if(consumer == e && iteration == env_iteration){
*needs_unconsume=1;
return 0;}
# 296
return 1;}case 6U: _LL3: _tmp4D=(((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp4B)->f1).is_union;_tmp4E=(((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp4B)->f1).fieldnum;_tmp4F=((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp4B)->f2;_LL4: {int is_union=_tmp4D;int field_no=_tmp4E;struct _fat_ptr d=_tmp4F;
# 302
if(!is_union || field_no == -1){
unsigned sz=_get_fat_size(d,sizeof(void*));
{int i=0;for(0;(unsigned)i < sz;++ i){
if(Cyc_CfFlowInfo_is_unique_consumed(e,env_iteration,((void**)d.curr)[i],needs_unconsume))
return 1;}}
# 308
return 0;}else{
# 311
return Cyc_CfFlowInfo_is_unique_consumed(e,env_iteration,*((void**)_check_fat_subscript(d,sizeof(void*),field_no)),needs_unconsume);}}case 8U: _LL5: _tmp4C=(void*)((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmp4B)->f2;_LL6: {void*r=_tmp4C;
# 313
return Cyc_CfFlowInfo_is_unique_consumed(e,env_iteration,r,needs_unconsume);}default: _LL7: _LL8:
 return 0;}_LL0:;}
# 165
void*Cyc_CfFlowInfo_make_unique_unconsumed(struct Cyc_CfFlowInfo_FlowEnv*fenv,void*r){
# 321
void*_tmp53=r;void*_tmp55;struct Cyc_Absyn_Vardecl*_tmp54;struct _fat_ptr _tmp57;struct Cyc_CfFlowInfo_UnionRInfo _tmp56;void*_tmp5A;int _tmp59;struct Cyc_Absyn_Exp*_tmp58;switch(*((int*)_tmp53)){case 7U: _LL1: _tmp58=((struct Cyc_CfFlowInfo_Consumed_CfFlowInfo_AbsRVal_struct*)_tmp53)->f1;_tmp59=((struct Cyc_CfFlowInfo_Consumed_CfFlowInfo_AbsRVal_struct*)_tmp53)->f2;_tmp5A=(void*)((struct Cyc_CfFlowInfo_Consumed_CfFlowInfo_AbsRVal_struct*)_tmp53)->f3;_LL2: {struct Cyc_Absyn_Exp*consumer=_tmp58;int iteration=_tmp59;void*r=_tmp5A;
# 323
return r;}case 6U: _LL3: _tmp56=((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp53)->f1;_tmp57=((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp53)->f2;_LL4: {struct Cyc_CfFlowInfo_UnionRInfo uinfo=_tmp56;struct _fat_ptr d=_tmp57;
# 325
unsigned sz=_get_fat_size(d,sizeof(void*));
int change=0;
struct _fat_ptr d2=({unsigned _tmp5D=sz;void**_tmp5C=_cycalloc(_check_times(_tmp5D,sizeof(void*)));({{unsigned _tmp1E3=sz;unsigned i;for(i=0;i < _tmp1E3;++ i){_tmp5C[i]=((void**)d.curr)[(int)i];}}0;});_tag_fat(_tmp5C,sizeof(void*),_tmp5D);});
{int i=0;for(0;(unsigned)i < sz;++ i){
({void*_tmp226=Cyc_CfFlowInfo_make_unique_unconsumed(fenv,((void**)d.curr)[i]);*((void**)_check_fat_subscript(d2,sizeof(void*),i))=_tmp226;});
if(((void**)d2.curr)[i]!= ((void**)d.curr)[i])
change=1;}}
# 333
if(change)
return(void*)({struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*_tmp5B=_cycalloc(sizeof(*_tmp5B));_tmp5B->tag=6U,_tmp5B->f1=uinfo,_tmp5B->f2=d2;_tmp5B;});else{
return r;}}case 8U: _LL5: _tmp54=((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmp53)->f1;_tmp55=(void*)((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmp53)->f2;_LL6: {struct Cyc_Absyn_Vardecl*n=_tmp54;void*r2=_tmp55;
# 337
void*r3=Cyc_CfFlowInfo_make_unique_unconsumed(fenv,r2);
if(r3 != r2)
return(void*)({struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*_tmp5E=_cycalloc(sizeof(*_tmp5E));_tmp5E->tag=8U,_tmp5E->f1=n,_tmp5E->f2=r3;_tmp5E;});else{
return r;}}default: _LL7: _LL8:
 return r;}_LL0:;}
# 165
void*Cyc_CfFlowInfo_make_unique_consumed(struct Cyc_CfFlowInfo_FlowEnv*fenv,void*t,struct Cyc_Absyn_Exp*consumer,int iteration,void*r){
# 348
struct _tuple12 _stmttmp9=({struct _tuple12 _tmp1E6;({void*_tmp227=Cyc_Tcutil_compress(t);_tmp1E6.f1=_tmp227;}),_tmp1E6.f2=r;_tmp1E6;});struct _tuple12 _tmp5F=_stmttmp9;struct _fat_ptr _tmp63;struct Cyc_CfFlowInfo_UnionRInfo _tmp62;struct Cyc_List_List*_tmp61;enum Cyc_Absyn_AggrKind _tmp60;struct _fat_ptr _tmp66;struct Cyc_CfFlowInfo_UnionRInfo _tmp65;union Cyc_Absyn_AggrInfo _tmp64;struct _fat_ptr _tmp69;struct Cyc_CfFlowInfo_UnionRInfo _tmp68;struct Cyc_List_List*_tmp67;void*_tmp6B;struct Cyc_Absyn_Vardecl*_tmp6A;if(((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmp5F.f2)->tag == 8U){_LL1: _tmp6A=((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmp5F.f2)->f1;_tmp6B=(void*)((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmp5F.f2)->f2;_LL2: {struct Cyc_Absyn_Vardecl*s=_tmp6A;void*r2=_tmp6B;
# 350
void*r3=Cyc_CfFlowInfo_make_unique_consumed(fenv,t,consumer,iteration,r2);
if(r3 != r2)return(void*)({struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*_tmp6C=_cycalloc(sizeof(*_tmp6C));_tmp6C->tag=8U,_tmp6C->f1=s,_tmp6C->f2=r3;_tmp6C;});else{
return r;}}}else{switch(*((int*)_tmp5F.f1)){case 6U: if(((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp5F.f2)->tag == 6U){_LL3: _tmp67=((struct Cyc_Absyn_TupleType_Absyn_Type_struct*)_tmp5F.f1)->f1;_tmp68=((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp5F.f2)->f1;_tmp69=((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp5F.f2)->f2;_LL4: {struct Cyc_List_List*tqts=_tmp67;struct Cyc_CfFlowInfo_UnionRInfo b=_tmp68;struct _fat_ptr d=_tmp69;
# 354
unsigned sz=_get_fat_size(d,sizeof(void*));
struct _fat_ptr d2=({unsigned _tmp6F=sz;void**_tmp6E=_cycalloc(_check_times(_tmp6F,sizeof(void*)));({{unsigned _tmp1E4=sz;unsigned i;for(i=0;i < _tmp1E4;++ i){_tmp6E[i]=fenv->unknown_all;}}0;});_tag_fat(_tmp6E,sizeof(void*),_tmp6F);});
{int i=0;for(0;(unsigned)i < sz;++ i){
({void*_tmp228=Cyc_CfFlowInfo_make_unique_consumed(fenv,(*((struct _tuple15*)((struct Cyc_List_List*)_check_null(tqts))->hd)).f2,consumer,iteration,((void**)d.curr)[i]);*((void**)_check_fat_subscript(d2,sizeof(void*),i))=_tmp228;});
tqts=tqts->tl;}}
# 360
return(void*)({struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*_tmp6D=_cycalloc(sizeof(*_tmp6D));_tmp6D->tag=6U,_tmp6D->f1=b,_tmp6D->f2=d2;_tmp6D;});}}else{goto _LL9;}case 0U: if(((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp5F.f1)->f1)->tag == 20U){if(((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp5F.f2)->tag == 6U){_LL5: _tmp64=((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp5F.f1)->f1)->f1;_tmp65=((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp5F.f2)->f1;_tmp66=((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp5F.f2)->f2;_LL6: {union Cyc_Absyn_AggrInfo info=_tmp64;struct Cyc_CfFlowInfo_UnionRInfo b=_tmp65;struct _fat_ptr d=_tmp66;
# 362
struct Cyc_Absyn_Aggrdecl*ad=Cyc_Absyn_get_known_aggrdecl(info);
if(ad->impl == 0)return r;_tmp60=ad->kind;_tmp61=((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->fields;_tmp62=b;_tmp63=d;goto _LL8;}}else{goto _LL9;}}else{goto _LL9;}case 7U: if(((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp5F.f2)->tag == 6U){_LL7: _tmp60=((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmp5F.f1)->f1;_tmp61=((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmp5F.f1)->f2;_tmp62=((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp5F.f2)->f1;_tmp63=((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp5F.f2)->f2;_LL8: {enum Cyc_Absyn_AggrKind k=_tmp60;struct Cyc_List_List*fs=_tmp61;struct Cyc_CfFlowInfo_UnionRInfo b=_tmp62;struct _fat_ptr d=_tmp63;
# 366
struct _fat_ptr d2=({unsigned _tmp74=_get_fat_size(d,sizeof(void*));void**_tmp73=_cycalloc(_check_times(_tmp74,sizeof(void*)));({{unsigned _tmp1E5=_get_fat_size(d,sizeof(void*));unsigned i;for(i=0;i < _tmp1E5;++ i){_tmp73[i]=((void**)d.curr)[(int)i];}}0;});_tag_fat(_tmp73,sizeof(void*),_tmp74);});
unsigned sz=(unsigned)Cyc_List_length(fs);
{int i=0;for(0;(unsigned)i < sz;(i ++,fs=fs->tl)){
struct Cyc_Absyn_Aggrfield*_stmttmpA=(struct Cyc_Absyn_Aggrfield*)((struct Cyc_List_List*)_check_null(fs))->hd;void*_tmp71;struct _fat_ptr*_tmp70;_LLC: _tmp70=_stmttmpA->name;_tmp71=_stmttmpA->type;_LLD: {struct _fat_ptr*n=_tmp70;void*t2=_tmp71;
if(_get_fat_size(*n,sizeof(char))!= (unsigned)1)
({void*_tmp229=Cyc_CfFlowInfo_make_unique_consumed(fenv,t2,consumer,iteration,*((void**)_check_fat_subscript(d,sizeof(void*),i)));*((void**)_check_fat_subscript(d2,sizeof(void*),i))=_tmp229;});}}}
# 373
return(void*)({struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*_tmp72=_cycalloc(sizeof(*_tmp72));_tmp72->tag=6U,_tmp72->f1=b,_tmp72->f2=d2;_tmp72;});}}else{goto _LL9;}default: _LL9: _LLA:
# 375
 if(Cyc_Tcutil_is_noalias_pointer(t,0))
return(void*)({struct Cyc_CfFlowInfo_Consumed_CfFlowInfo_AbsRVal_struct*_tmp75=_cycalloc(sizeof(*_tmp75));_tmp75->tag=7U,_tmp75->f1=consumer,_tmp75->f2=iteration,_tmp75->f3=r;_tmp75;});else{
return r;}}}_LL0:;}struct _tuple16{struct Cyc_CfFlowInfo_Place*f1;unsigned f2;};
# 165
static int Cyc_CfFlowInfo_prefix_of_member(struct Cyc_CfFlowInfo_Place*place,struct Cyc_Dict_Dict set){
# 383
struct _RegionHandle _tmp76=_new_region("r");struct _RegionHandle*r=& _tmp76;_push_region(r);
{struct _tuple16 elem=({struct _tuple16 _tmp1E7;_tmp1E7.f1=place,_tmp1E7.f2=0U;_tmp1E7;});
struct Cyc_Iter_Iter iter=Cyc_Dict_make_iter(r,set);
while(({int(*_tmp22A)(struct Cyc_Iter_Iter,struct _tuple16*)=({int(*_tmp77)(struct Cyc_Iter_Iter,struct _tuple16*)=(int(*)(struct Cyc_Iter_Iter,struct _tuple16*))Cyc_Iter_next;_tmp77;});_tmp22A(iter,& elem);})){
struct Cyc_CfFlowInfo_Place*place2=elem.f1;
struct Cyc_CfFlowInfo_Place _stmttmpB=*place;struct Cyc_List_List*_tmp79;void*_tmp78;_LL1: _tmp78=_stmttmpB.root;_tmp79=_stmttmpB.path;_LL2: {void*root1=_tmp78;struct Cyc_List_List*fs1=_tmp79;
struct Cyc_CfFlowInfo_Place _stmttmpC=*place2;struct Cyc_List_List*_tmp7B;void*_tmp7A;_LL4: _tmp7A=_stmttmpC.root;_tmp7B=_stmttmpC.path;_LL5: {void*root2=_tmp7A;struct Cyc_List_List*fs2=_tmp7B;
if(Cyc_CfFlowInfo_root_cmp(root1,root2)!= 0)
continue;
# 390
for(0;
# 392
fs1 != 0 && fs2 != 0;(fs1=fs1->tl,fs2=fs2->tl)){
if((void*)fs1->hd != (void*)fs2->hd)break;}
# 390
if(fs1 == 0){
# 395
int _tmp7C=1;_npop_handler(0U);return _tmp7C;}}}}{
# 397
int _tmp7D=0;_npop_handler(0U);return _tmp7D;}}
# 384
;_pop_region(r);}struct Cyc_CfFlowInfo_EscPile{struct Cyc_List_List*places;};
# 407
static void Cyc_CfFlowInfo_add_place(struct Cyc_CfFlowInfo_EscPile*pile,struct Cyc_CfFlowInfo_Place*place){
# 410
if(!({int(*_tmp22C)(int(*compare)(struct Cyc_CfFlowInfo_Place*,struct Cyc_CfFlowInfo_Place*),struct Cyc_List_List*l,struct Cyc_CfFlowInfo_Place*x)=({int(*_tmp7E)(int(*compare)(struct Cyc_CfFlowInfo_Place*,struct Cyc_CfFlowInfo_Place*),struct Cyc_List_List*l,struct Cyc_CfFlowInfo_Place*x)=(int(*)(int(*compare)(struct Cyc_CfFlowInfo_Place*,struct Cyc_CfFlowInfo_Place*),struct Cyc_List_List*l,struct Cyc_CfFlowInfo_Place*x))Cyc_List_mem;_tmp7E;});struct Cyc_List_List*_tmp22B=pile->places;_tmp22C(Cyc_CfFlowInfo_place_cmp,_tmp22B,place);}))
({struct Cyc_List_List*_tmp22D=({struct Cyc_List_List*_tmp7F=_cycalloc(sizeof(*_tmp7F));_tmp7F->hd=place,_tmp7F->tl=pile->places;_tmp7F;});pile->places=_tmp22D;});}
# 407
static void Cyc_CfFlowInfo_add_places(struct Cyc_CfFlowInfo_EscPile*pile,void*r){
# 414
void*_tmp80=r;struct _fat_ptr _tmp82;struct Cyc_CfFlowInfo_UnionRInfo _tmp81;struct Cyc_CfFlowInfo_Place*_tmp83;void*_tmp84;void*_tmp85;switch(*((int*)_tmp80)){case 7U: _LL1: _tmp85=(void*)((struct Cyc_CfFlowInfo_Consumed_CfFlowInfo_AbsRVal_struct*)_tmp80)->f3;_LL2: {void*r=_tmp85;
Cyc_CfFlowInfo_add_places(pile,r);return;}case 8U: _LL3: _tmp84=(void*)((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmp80)->f2;_LL4: {void*r=_tmp84;
Cyc_CfFlowInfo_add_places(pile,r);return;}case 4U: _LL5: _tmp83=((struct Cyc_CfFlowInfo_AddressOf_CfFlowInfo_AbsRVal_struct*)_tmp80)->f1;_LL6: {struct Cyc_CfFlowInfo_Place*p=_tmp83;
Cyc_CfFlowInfo_add_place(pile,p);return;}case 6U: _LL7: _tmp81=((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp80)->f1;_tmp82=((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp80)->f2;_LL8: {struct Cyc_CfFlowInfo_UnionRInfo b=_tmp81;struct _fat_ptr d=_tmp82;
# 419
{int i=0;for(0;(unsigned)i < _get_fat_size(d,sizeof(void*));++ i){
Cyc_CfFlowInfo_add_places(pile,((void**)d.curr)[i]);}}
return;}default: _LL9: _LLA:
 return;}_LL0:;}
# 407
static void*Cyc_CfFlowInfo_insert_place_inner(void*new_val,void*old_val){
# 430
void*_tmp86=old_val;void*_tmp88;struct Cyc_Absyn_Vardecl*_tmp87;void*_tmp8B;int _tmp8A;struct Cyc_Absyn_Exp*_tmp89;struct _fat_ptr _tmp8D;int _tmp8C;switch(*((int*)_tmp86)){case 6U: _LL1: _tmp8C=(((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp86)->f1).is_union;_tmp8D=((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp86)->f2;_LL2: {int is_union=_tmp8C;struct _fat_ptr d=_tmp8D;
# 432
struct _fat_ptr d2=({unsigned _tmp90=_get_fat_size(d,sizeof(void*));void**_tmp8F=_cycalloc(_check_times(_tmp90,sizeof(void*)));({{unsigned _tmp1E8=_get_fat_size(d,sizeof(void*));unsigned i;for(i=0;i < _tmp1E8;++ i){({
void*_tmp22E=Cyc_CfFlowInfo_insert_place_inner(new_val,((void**)d.curr)[(int)i]);_tmp8F[i]=_tmp22E;});}}0;});_tag_fat(_tmp8F,sizeof(void*),_tmp90);});
# 436
return(void*)({struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*_tmp8E=_cycalloc(sizeof(*_tmp8E));_tmp8E->tag=6U,(_tmp8E->f1).is_union=is_union,(_tmp8E->f1).fieldnum=- 1,_tmp8E->f2=d2;_tmp8E;});}case 7U: _LL3: _tmp89=((struct Cyc_CfFlowInfo_Consumed_CfFlowInfo_AbsRVal_struct*)_tmp86)->f1;_tmp8A=((struct Cyc_CfFlowInfo_Consumed_CfFlowInfo_AbsRVal_struct*)_tmp86)->f2;_tmp8B=(void*)((struct Cyc_CfFlowInfo_Consumed_CfFlowInfo_AbsRVal_struct*)_tmp86)->f3;_LL4: {struct Cyc_Absyn_Exp*e=_tmp89;int i=_tmp8A;void*rval=_tmp8B;
# 438
return(void*)({struct Cyc_CfFlowInfo_Consumed_CfFlowInfo_AbsRVal_struct*_tmp91=_cycalloc(sizeof(*_tmp91));_tmp91->tag=7U,_tmp91->f1=e,_tmp91->f2=i,({void*_tmp22F=Cyc_CfFlowInfo_insert_place_inner(new_val,rval);_tmp91->f3=_tmp22F;});_tmp91;});}case 8U: _LL5: _tmp87=((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmp86)->f1;_tmp88=(void*)((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmp86)->f2;_LL6: {struct Cyc_Absyn_Vardecl*n=_tmp87;void*rval=_tmp88;
# 440
return(void*)({struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*_tmp92=_cycalloc(sizeof(*_tmp92));_tmp92->tag=8U,_tmp92->f1=n,({void*_tmp230=Cyc_CfFlowInfo_insert_place_inner(new_val,rval);_tmp92->f2=_tmp230;});_tmp92;});}default: _LL7: _LL8:
 return new_val;}_LL0:;}
# 407
static struct _fat_ptr Cyc_CfFlowInfo_aggr_dict_insert(struct _fat_ptr d,int n,void*rval){
# 448
void*old_rval=*((void**)_check_fat_subscript(d,sizeof(void*),n));
if(old_rval == rval)return d;{struct _fat_ptr res=({unsigned _tmp94=
_get_fat_size(d,sizeof(void*));void**_tmp93=_cycalloc(_check_times(_tmp94,sizeof(void*)));({{unsigned _tmp1E9=_get_fat_size(d,sizeof(void*));unsigned i;for(i=0;i < _tmp1E9;++ i){_tmp93[i]=((void**)d.curr)[(int)i];}}0;});_tag_fat(_tmp93,sizeof(void*),_tmp94);});
*((void**)_check_fat_subscript(res,sizeof(void*),n))=rval;
return res;}}struct _tuple17{struct Cyc_List_List*f1;void*f2;};
# 407
static void*Cyc_CfFlowInfo_insert_place_outer(struct Cyc_List_List*path,void*old_val,void*new_val){
# 462
if(path == 0)
return Cyc_CfFlowInfo_insert_place_inner(new_val,old_val);{
# 462
struct _tuple17 _stmttmpD=({struct _tuple17 _tmp1EA;_tmp1EA.f1=path,_tmp1EA.f2=old_val;_tmp1EA;});struct _tuple17 _tmp95=_stmttmpD;void*_tmp97;struct Cyc_Absyn_Vardecl*_tmp96;void*_tmp99;struct Cyc_List_List*_tmp98;struct _fat_ptr _tmp9D;int _tmp9C;struct Cyc_List_List*_tmp9B;int _tmp9A;if(_tmp95.f1 != 0){if(((struct Cyc_CfFlowInfo_Dot_CfFlowInfo_PathCon_struct*)((struct Cyc_List_List*)_tmp95.f1)->hd)->tag == 0U)switch(*((int*)_tmp95.f2)){case 6U: _LL1: _tmp9A=((struct Cyc_CfFlowInfo_Dot_CfFlowInfo_PathCon_struct*)(_tmp95.f1)->hd)->f1;_tmp9B=(_tmp95.f1)->tl;_tmp9C=(((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp95.f2)->f1).is_union;_tmp9D=((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp95.f2)->f2;_LL2: {int i=_tmp9A;struct Cyc_List_List*tl=_tmp9B;int is_union=_tmp9C;struct _fat_ptr d=_tmp9D;
# 466
void*new_child=Cyc_CfFlowInfo_insert_place_outer(tl,*((void**)_check_fat_subscript(d,sizeof(void*),i)),new_val);
struct _fat_ptr new_d=Cyc_CfFlowInfo_aggr_dict_insert(d,i,new_child);
if((void**)new_d.curr == (void**)d.curr)return old_val;else{
return(void*)({struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*_tmp9E=_cycalloc(sizeof(*_tmp9E));_tmp9E->tag=6U,(_tmp9E->f1).is_union=is_union,(_tmp9E->f1).fieldnum=- 1,_tmp9E->f2=new_d;_tmp9E;});}}case 8U: goto _LL5;default: goto _LL7;}else{switch(*((int*)_tmp95.f2)){case 5U: _LL3: _tmp98=(_tmp95.f1)->tl;_tmp99=(void*)((struct Cyc_CfFlowInfo_UniquePtr_CfFlowInfo_AbsRVal_struct*)_tmp95.f2)->f1;_LL4: {struct Cyc_List_List*tl=_tmp98;void*rval=_tmp99;
# 471
void*new_rval=Cyc_CfFlowInfo_insert_place_outer(tl,rval,new_val);
if(new_rval == rval)return old_val;else{
return(void*)({struct Cyc_CfFlowInfo_UniquePtr_CfFlowInfo_AbsRVal_struct*_tmp9F=_cycalloc(sizeof(*_tmp9F));_tmp9F->tag=5U,_tmp9F->f1=new_rval;_tmp9F;});}}case 8U: goto _LL5;default: goto _LL7;}}}else{if(((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmp95.f2)->tag == 8U){_LL5: _tmp96=((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmp95.f2)->f1;_tmp97=(void*)((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmp95.f2)->f2;_LL6: {struct Cyc_Absyn_Vardecl*n=_tmp96;void*rval=_tmp97;
# 475
void*new_rval=Cyc_CfFlowInfo_insert_place_outer(path,rval,new_val);
if(new_rval == rval)return old_val;else{
return(void*)({struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*_tmpA0=_cycalloc(sizeof(*_tmpA0));_tmpA0->tag=8U,_tmpA0->f1=n,_tmpA0->f2=new_rval;_tmpA0;});}}}else{_LL7: _LL8:
({void*_tmpA1=0U;({int(*_tmp232)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmpA3)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmpA3;});struct _fat_ptr _tmp231=({const char*_tmpA2="bad insert place";_tag_fat(_tmpA2,sizeof(char),17U);});_tmp232(_tmp231,_tag_fat(_tmpA1,sizeof(void*),0U));});});}}_LL0:;}}
# 407
static struct Cyc_Dict_Dict Cyc_CfFlowInfo_escape_these(struct Cyc_CfFlowInfo_FlowEnv*fenv,struct Cyc_CfFlowInfo_EscPile*pile,struct Cyc_Dict_Dict d){
# 486
while(pile->places != 0){
struct Cyc_CfFlowInfo_Place*place=(struct Cyc_CfFlowInfo_Place*)((struct Cyc_List_List*)_check_null(pile->places))->hd;
pile->places=((struct Cyc_List_List*)_check_null(pile->places))->tl;{
void*oldval;void*newval;
{struct _handler_cons _tmpA4;_push_handler(& _tmpA4);{int _tmpA6=0;if(setjmp(_tmpA4.handler))_tmpA6=1;if(!_tmpA6){oldval=Cyc_CfFlowInfo_lookup_place(d,place);;_pop_handler();}else{void*_tmpA5=(void*)Cyc_Core_get_exn_thrown();void*_tmpA7=_tmpA5;void*_tmpA8;if(((struct Cyc_Dict_Absent_exn_struct*)_tmpA7)->tag == Cyc_Dict_Absent){_LL1: _LL2:
 continue;}else{_LL3: _tmpA8=_tmpA7;_LL4: {void*exn=_tmpA8;(int)_rethrow(exn);}}_LL0:;}}}
{enum Cyc_CfFlowInfo_InitLevel _stmttmpE=Cyc_CfFlowInfo_initlevel(fenv,d,oldval);enum Cyc_CfFlowInfo_InitLevel _tmpA9=_stmttmpE;if(_tmpA9 == Cyc_CfFlowInfo_AllIL){_LL6: _LL7:
 newval=fenv->esc_all;goto _LL5;}else{_LL8: _LL9:
 newval=fenv->esc_none;goto _LL5;}_LL5:;}
# 496
Cyc_CfFlowInfo_add_places(pile,oldval);{
struct Cyc_CfFlowInfo_Place _stmttmpF=*place;struct Cyc_List_List*_tmpAB;void*_tmpAA;_LLB: _tmpAA=_stmttmpF.root;_tmpAB=_stmttmpF.path;_LLC: {void*root=_tmpAA;struct Cyc_List_List*path=_tmpAB;
d=({struct Cyc_Dict_Dict _tmp236=d;void*_tmp235=root;Cyc_Dict_insert(_tmp236,_tmp235,({
struct Cyc_List_List*_tmp234=path;void*_tmp233=
Cyc_Dict_lookup(d,root);
# 499
Cyc_CfFlowInfo_insert_place_outer(_tmp234,_tmp233,newval);}));});}}}}
# 503
return d;}struct Cyc_CfFlowInfo_InitlevelEnv{struct Cyc_Dict_Dict d;struct Cyc_List_List*seen;};
# 513
static enum Cyc_CfFlowInfo_InitLevel Cyc_CfFlowInfo_initlevel_approx(void*r){
void*_tmpAC=r;void*_tmpAD;enum Cyc_CfFlowInfo_InitLevel _tmpAE;enum Cyc_CfFlowInfo_InitLevel _tmpAF;switch(*((int*)_tmpAC)){case 2U: _LL1: _tmpAF=((struct Cyc_CfFlowInfo_UnknownR_CfFlowInfo_AbsRVal_struct*)_tmpAC)->f1;_LL2: {enum Cyc_CfFlowInfo_InitLevel il=_tmpAF;
return il;}case 3U: _LL3: _tmpAE=((struct Cyc_CfFlowInfo_Esc_CfFlowInfo_AbsRVal_struct*)_tmpAC)->f1;_LL4: {enum Cyc_CfFlowInfo_InitLevel il=_tmpAE;
return il;}case 0U: _LL5: _LL6:
 goto _LL8;case 1U: _LL7: _LL8:
 return Cyc_CfFlowInfo_AllIL;case 7U: _LL9: _tmpAD=(void*)((struct Cyc_CfFlowInfo_Consumed_CfFlowInfo_AbsRVal_struct*)_tmpAC)->f3;_LLA: {void*r=_tmpAD;
return Cyc_CfFlowInfo_NoneIL;}default: _LLB: _LLC:
({void*_tmpB0=0U;({int(*_tmp238)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmpB2)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmpB2;});struct _fat_ptr _tmp237=({const char*_tmpB1="initlevel_approx";_tag_fat(_tmpB1,sizeof(char),17U);});_tmp238(_tmp237,_tag_fat(_tmpB0,sizeof(void*),0U));});});}_LL0:;}
# 513
static enum Cyc_CfFlowInfo_InitLevel Cyc_CfFlowInfo_initlevel_rec(struct Cyc_CfFlowInfo_InitlevelEnv*env,void*r,enum Cyc_CfFlowInfo_InitLevel acc){
# 525
enum Cyc_CfFlowInfo_InitLevel this_ans;
if((int)acc == (int)0U)return Cyc_CfFlowInfo_NoneIL;{void*_tmpB3=r;void*_tmpB4;struct Cyc_CfFlowInfo_Place*_tmpB5;struct _fat_ptr _tmpB6;struct _fat_ptr _tmpB9;int _tmpB8;int _tmpB7;void*_tmpBA;switch(*((int*)_tmpB3)){case 8U: _LL1: _tmpBA=(void*)((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmpB3)->f2;_LL2: {void*r=_tmpBA;
# 528
return Cyc_CfFlowInfo_initlevel_rec(env,r,acc);}case 6U: _LL3: _tmpB7=(((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmpB3)->f1).is_union;_tmpB8=(((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmpB3)->f1).fieldnum;_tmpB9=((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmpB3)->f2;if(_tmpB7){_LL4: {int iu=_tmpB7;int f=_tmpB8;struct _fat_ptr d=_tmpB9;
# 533
unsigned sz=_get_fat_size(d,sizeof(void*));
this_ans=0U;
if(f == -1){
int i=0;for(0;(unsigned)i < sz;++ i){
if((int)Cyc_CfFlowInfo_initlevel_rec(env,((void**)d.curr)[i],Cyc_CfFlowInfo_AllIL)== (int)Cyc_CfFlowInfo_AllIL){
this_ans=1U;
break;}}}else{
# 543
if((int)Cyc_CfFlowInfo_initlevel_rec(env,*((void**)_check_fat_subscript(d,sizeof(void*),f)),Cyc_CfFlowInfo_AllIL)== (int)Cyc_CfFlowInfo_AllIL)
this_ans=1U;}
# 535
goto _LL0;}}else{_LL5: _tmpB6=((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmpB3)->f2;_LL6: {struct _fat_ptr d=_tmpB6;
# 547
unsigned sz=_get_fat_size(d,sizeof(void*));
this_ans=1U;
{int i=0;for(0;(unsigned)i < sz;++ i){
this_ans=Cyc_CfFlowInfo_initlevel_rec(env,((void**)d.curr)[i],this_ans);}}
goto _LL0;}}case 4U: _LL7: _tmpB5=((struct Cyc_CfFlowInfo_AddressOf_CfFlowInfo_AbsRVal_struct*)_tmpB3)->f1;_LL8: {struct Cyc_CfFlowInfo_Place*p=_tmpB5;
# 553
if(({int(*_tmp23A)(int(*compare)(struct Cyc_CfFlowInfo_Place*,struct Cyc_CfFlowInfo_Place*),struct Cyc_List_List*l,struct Cyc_CfFlowInfo_Place*x)=({int(*_tmpBB)(int(*compare)(struct Cyc_CfFlowInfo_Place*,struct Cyc_CfFlowInfo_Place*),struct Cyc_List_List*l,struct Cyc_CfFlowInfo_Place*x)=(int(*)(int(*compare)(struct Cyc_CfFlowInfo_Place*,struct Cyc_CfFlowInfo_Place*),struct Cyc_List_List*l,struct Cyc_CfFlowInfo_Place*x))Cyc_List_mem;_tmpBB;});struct Cyc_List_List*_tmp239=env->seen;_tmp23A(Cyc_CfFlowInfo_place_cmp,_tmp239,p);}))
this_ans=1U;else{
# 556
({struct Cyc_List_List*_tmp23B=({struct Cyc_List_List*_tmpBC=_cycalloc(sizeof(*_tmpBC));_tmpBC->hd=p,_tmpBC->tl=env->seen;_tmpBC;});env->seen=_tmp23B;});
this_ans=({struct Cyc_CfFlowInfo_InitlevelEnv*_tmp23C=env;Cyc_CfFlowInfo_initlevel_rec(_tmp23C,Cyc_CfFlowInfo_lookup_place(env->d,p),Cyc_CfFlowInfo_AllIL);});
env->seen=((struct Cyc_List_List*)_check_null(env->seen))->tl;}
# 560
goto _LL0;}case 5U: _LL9: _tmpB4=(void*)((struct Cyc_CfFlowInfo_UniquePtr_CfFlowInfo_AbsRVal_struct*)_tmpB3)->f1;_LLA: {void*r=_tmpB4;
this_ans=Cyc_CfFlowInfo_initlevel_rec(env,r,Cyc_CfFlowInfo_AllIL);goto _LL0;}default: _LLB: _LLC:
 this_ans=Cyc_CfFlowInfo_initlevel_approx(r);goto _LL0;}_LL0:;}
# 564
return this_ans;}
# 513
enum Cyc_CfFlowInfo_InitLevel Cyc_CfFlowInfo_initlevel(struct Cyc_CfFlowInfo_FlowEnv*env0,struct Cyc_Dict_Dict d,void*r){
# 567
struct Cyc_CfFlowInfo_InitlevelEnv env=({struct Cyc_CfFlowInfo_InitlevelEnv _tmp1EB;_tmp1EB.d=d,_tmp1EB.seen=0;_tmp1EB;});
return Cyc_CfFlowInfo_initlevel_rec(& env,r,Cyc_CfFlowInfo_AllIL);}
# 513
void*Cyc_CfFlowInfo_lookup_place(struct Cyc_Dict_Dict d,struct Cyc_CfFlowInfo_Place*place){
# 572
struct Cyc_CfFlowInfo_Place _stmttmp10=*place;struct Cyc_List_List*_tmpBE;void*_tmpBD;_LL1: _tmpBD=_stmttmp10.root;_tmpBE=_stmttmp10.path;_LL2: {void*root=_tmpBD;struct Cyc_List_List*path=_tmpBE;
void*ans=Cyc_Dict_lookup(d,root);
for(0;path != 0;path=path->tl){
retry: {struct _tuple12 _stmttmp11=({struct _tuple12 _tmp1EC;_tmp1EC.f1=ans,_tmp1EC.f2=(void*)path->hd;_tmp1EC;});struct _tuple12 _tmpBF=_stmttmp11;void*_tmpC0;int _tmpC3;struct _fat_ptr _tmpC2;struct Cyc_CfFlowInfo_UnionRInfo _tmpC1;void*_tmpC4;void*_tmpC5;switch(*((int*)_tmpBF.f1)){case 8U: _LL4: _tmpC5=(void*)((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmpBF.f1)->f2;_LL5: {void*rval=_tmpC5;
# 577
ans=rval;goto retry;}case 7U: _LL6: _tmpC4=(void*)((struct Cyc_CfFlowInfo_Consumed_CfFlowInfo_AbsRVal_struct*)_tmpBF.f1)->f3;_LL7: {void*rval=_tmpC4;
# 580
ans=rval;goto retry;}case 6U: if(((struct Cyc_CfFlowInfo_Dot_CfFlowInfo_PathCon_struct*)_tmpBF.f2)->tag == 0U){_LL8: _tmpC1=((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmpBF.f1)->f1;_tmpC2=((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmpBF.f1)->f2;_tmpC3=((struct Cyc_CfFlowInfo_Dot_CfFlowInfo_PathCon_struct*)_tmpBF.f2)->f1;_LL9: {struct Cyc_CfFlowInfo_UnionRInfo is_union=_tmpC1;struct _fat_ptr d2=_tmpC2;int fname=_tmpC3;
# 582
ans=*((void**)_check_fat_subscript(d2,sizeof(void*),fname));
goto _LL3;}}else{goto _LLC;}case 5U: if(((struct Cyc_CfFlowInfo_Star_CfFlowInfo_PathCon_struct*)_tmpBF.f2)->tag == 1U){_LLA: _tmpC0=(void*)((struct Cyc_CfFlowInfo_UniquePtr_CfFlowInfo_AbsRVal_struct*)_tmpBF.f1)->f1;_LLB: {void*rval=_tmpC0;
# 585
ans=rval;
goto _LL3;}}else{goto _LLC;}default: _LLC: _LLD:
# 593
({void*_tmpC6=0U;({int(*_tmp23E)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmpC8)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmpC8;});struct _fat_ptr _tmp23D=({const char*_tmpC7="bad lookup_place";_tag_fat(_tmpC7,sizeof(char),17U);});_tmp23E(_tmp23D,_tag_fat(_tmpC6,sizeof(void*),0U));});});}_LL3:;}}
# 595
return ans;}}
# 513
static int Cyc_CfFlowInfo_is_rval_unescaped(void*rval){
# 599
void*_tmpC9=rval;struct _fat_ptr _tmpCC;int _tmpCB;int _tmpCA;void*_tmpCD;void*_tmpCE;switch(*((int*)_tmpC9)){case 3U: _LL1: _LL2:
 return 0;case 7U: _LL3: _tmpCE=(void*)((struct Cyc_CfFlowInfo_Consumed_CfFlowInfo_AbsRVal_struct*)_tmpC9)->f3;_LL4: {void*r=_tmpCE;
return Cyc_CfFlowInfo_is_rval_unescaped(r);}case 8U: _LL5: _tmpCD=(void*)((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmpC9)->f2;_LL6: {void*r=_tmpCD;
return Cyc_CfFlowInfo_is_rval_unescaped(r);}case 6U: _LL7: _tmpCA=(((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmpC9)->f1).is_union;_tmpCB=(((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmpC9)->f1).fieldnum;_tmpCC=((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmpC9)->f2;_LL8: {int is_union=_tmpCA;int field_no=_tmpCB;struct _fat_ptr d=_tmpCC;
# 604
if(is_union && field_no != -1)
return !Cyc_CfFlowInfo_is_rval_unescaped(*((void**)_check_fat_subscript(d,sizeof(void*),field_no)));else{
# 607
unsigned sz=_get_fat_size(d,sizeof(void*));
{int i=0;for(0;(unsigned)i < sz;++ i){
if(!Cyc_CfFlowInfo_is_rval_unescaped(((void**)d.curr)[i]))return 0;}}
# 608
return 1;}}default: _LL9: _LLA:
# 612
 return 1;}_LL0:;}
# 513
int Cyc_CfFlowInfo_is_unescaped(struct Cyc_Dict_Dict d,struct Cyc_CfFlowInfo_Place*place){
# 616
return Cyc_CfFlowInfo_is_rval_unescaped(Cyc_CfFlowInfo_lookup_place(d,place));}
# 513
int Cyc_CfFlowInfo_is_init_pointer(void*rval){
# 619
void*_tmpCF=rval;void*_tmpD0;switch(*((int*)_tmpCF)){case 8U: _LL1: _tmpD0=(void*)((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmpCF)->f2;_LL2: {void*r=_tmpD0;
return Cyc_CfFlowInfo_is_init_pointer(r);}case 4U: _LL3: _LL4:
 goto _LL6;case 5U: _LL5: _LL6:
 return 1;default: _LL7: _LL8:
 return 0;}_LL0:;}
# 513
struct Cyc_Dict_Dict Cyc_CfFlowInfo_escape_deref(struct Cyc_CfFlowInfo_FlowEnv*fenv,struct Cyc_Dict_Dict d,void*r){
# 629
struct _RegionHandle _tmpD1=_new_region("rgn");struct _RegionHandle*rgn=& _tmpD1;_push_region(rgn);
{struct Cyc_CfFlowInfo_EscPile*pile=({struct Cyc_CfFlowInfo_EscPile*_tmpD3=_cycalloc(sizeof(*_tmpD3));_tmpD3->places=0;_tmpD3;});
Cyc_CfFlowInfo_add_places(pile,r);{
struct Cyc_Dict_Dict _tmpD2=Cyc_CfFlowInfo_escape_these(fenv,pile,d);_npop_handler(0U);return _tmpD2;}}
# 630
;_pop_region(rgn);}struct Cyc_CfFlowInfo_AssignEnv{struct Cyc_CfFlowInfo_FlowEnv*fenv;struct Cyc_CfFlowInfo_EscPile*pile;struct Cyc_Dict_Dict d;unsigned loc;struct Cyc_CfFlowInfo_Place*root_place;};
# 642
static void*Cyc_CfFlowInfo_assign_place_inner(struct Cyc_CfFlowInfo_AssignEnv*env,void*oldval,void*newval){
# 647
struct _tuple12 _stmttmp12=({struct _tuple12 _tmp1EE;_tmp1EE.f1=oldval,_tmp1EE.f2=newval;_tmp1EE;});struct _tuple12 _tmpD4=_stmttmp12;enum Cyc_CfFlowInfo_InitLevel _tmpD5;struct _fat_ptr _tmpD9;struct Cyc_CfFlowInfo_UnionRInfo _tmpD8;struct _fat_ptr _tmpD7;struct Cyc_CfFlowInfo_UnionRInfo _tmpD6;struct Cyc_CfFlowInfo_Place*_tmpDA;void*_tmpDC;struct Cyc_Absyn_Vardecl*_tmpDB;void*_tmpDD;if(((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmpD4.f1)->tag == 8U){_LL1: _tmpDD=(void*)((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmpD4.f1)->f2;_LL2: {void*r1=_tmpDD;
# 649
return Cyc_CfFlowInfo_assign_place_inner(env,r1,newval);}}else{if(((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmpD4.f2)->tag == 8U){_LL3: _tmpDB=((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmpD4.f2)->f1;_tmpDC=(void*)((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmpD4.f2)->f2;_LL4: {struct Cyc_Absyn_Vardecl*n=_tmpDB;void*r=_tmpDC;
# 651
void*new_rval=Cyc_CfFlowInfo_assign_place_inner(env,oldval,r);
if(new_rval == r)return newval;else{
return(void*)({struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*_tmpDE=_cycalloc(sizeof(*_tmpDE));_tmpDE->tag=8U,_tmpDE->f1=n,_tmpDE->f2=new_rval;_tmpDE;});}}}else{switch(*((int*)_tmpD4.f1)){case 3U: if(((struct Cyc_CfFlowInfo_AddressOf_CfFlowInfo_AbsRVal_struct*)_tmpD4.f2)->tag == 4U){_LL5: _tmpDA=((struct Cyc_CfFlowInfo_AddressOf_CfFlowInfo_AbsRVal_struct*)_tmpD4.f2)->f1;_LL6: {struct Cyc_CfFlowInfo_Place*p=_tmpDA;
Cyc_CfFlowInfo_add_place(env->pile,p);goto _LL8;}}else{_LL7: _LL8:
# 656
 if((int)Cyc_CfFlowInfo_initlevel(env->fenv,env->d,newval)!= (int)Cyc_CfFlowInfo_AllIL)
({void*_tmpDF=0U;({unsigned _tmp240=env->loc;struct _fat_ptr _tmp23F=({const char*_tmpE0="assignment puts possibly-uninitialized data in an escaped location";_tag_fat(_tmpE0,sizeof(char),67U);});Cyc_CfFlowInfo_aerr(_tmp240,_tmp23F,_tag_fat(_tmpDF,sizeof(void*),0U));});});
# 656
return(env->fenv)->esc_all;}case 6U: switch(*((int*)_tmpD4.f2)){case 6U: _LL9: _tmpD6=((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmpD4.f1)->f1;_tmpD7=((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmpD4.f1)->f2;_tmpD8=((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmpD4.f2)->f1;_tmpD9=((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmpD4.f2)->f2;_LLA: {struct Cyc_CfFlowInfo_UnionRInfo is_union1=_tmpD6;struct _fat_ptr d1=_tmpD7;struct Cyc_CfFlowInfo_UnionRInfo is_union2=_tmpD8;struct _fat_ptr d2=_tmpD9;
# 661
struct _fat_ptr new_d=({unsigned _tmpE3=
_get_fat_size(d1,sizeof(void*));void**_tmpE2=_cycalloc(_check_times(_tmpE3,sizeof(void*)));({{unsigned _tmp1ED=_get_fat_size(d1,sizeof(void*));unsigned i;for(i=0;i < _tmp1ED;++ i){({void*_tmp241=Cyc_CfFlowInfo_assign_place_inner(env,((void**)d1.curr)[(int)i],*((void**)_check_fat_subscript(d2,sizeof(void*),(int)i)));_tmpE2[i]=_tmp241;});}}0;});_tag_fat(_tmpE2,sizeof(void*),_tmpE3);});
# 665
int change=0;
{int i=0;for(0;(unsigned)i < _get_fat_size(d1,sizeof(void*));++ i){
if(*((void**)_check_fat_subscript(new_d,sizeof(void*),i))!= ((void**)d1.curr)[i]){
change=1;break;}}}
# 666
if(!change){
# 671
if(!is_union1.is_union)return oldval;new_d=d1;}else{
# 675
change=0;
{int i=0;for(0;(unsigned)i < _get_fat_size(d1,sizeof(void*));++ i){
if(({void*_tmp242=*((void**)_check_fat_subscript(new_d,sizeof(void*),i));_tmp242 != *((void**)_check_fat_subscript(d2,sizeof(void*),i));})){
change=1;break;}}}
# 676
if(!change){
# 681
if(!is_union1.is_union)return newval;new_d=d2;}}
# 685
return(void*)({struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*_tmpE1=_cycalloc(sizeof(*_tmpE1));_tmpE1->tag=6U,_tmpE1->f1=is_union2,_tmpE1->f2=new_d;_tmpE1;});}case 3U: goto _LLB;default: goto _LLD;}default: if(((struct Cyc_CfFlowInfo_Esc_CfFlowInfo_AbsRVal_struct*)_tmpD4.f2)->tag == 3U){_LLB: _tmpD5=((struct Cyc_CfFlowInfo_Esc_CfFlowInfo_AbsRVal_struct*)_tmpD4.f2)->f1;_LLC: {enum Cyc_CfFlowInfo_InitLevel il=_tmpD5;
# 687
enum Cyc_CfFlowInfo_InitLevel _tmpE4=il;if(_tmpE4 == Cyc_CfFlowInfo_NoneIL){_LL10: _LL11:
 return(env->fenv)->unknown_none;}else{_LL12: _LL13:
 return(env->fenv)->unknown_all;}_LLF:;}}else{_LLD: _LLE:
# 691
 return newval;}}}}_LL0:;}
# 642
static int Cyc_CfFlowInfo_nprefix(int*n,void*unused){
# 698
if(*n > 0){*n=*n - 1;return 1;}else{
return 0;}}
# 642
static void*Cyc_CfFlowInfo_assign_place_outer(struct Cyc_CfFlowInfo_AssignEnv*env,struct Cyc_List_List*path,int path_prefix,void*oldval,void*newval){
# 712
if(path == 0)return Cyc_CfFlowInfo_assign_place_inner(env,oldval,newval);{struct _tuple17 _stmttmp13=({struct _tuple17 _tmp1EF;_tmp1EF.f1=path,_tmp1EF.f2=oldval;_tmp1EF;});struct _tuple17 _tmpE5=_stmttmp13;struct _fat_ptr _tmpEA;int _tmpE9;int _tmpE8;struct Cyc_List_List*_tmpE7;int _tmpE6;void*_tmpEC;struct Cyc_List_List*_tmpEB;void*_tmpEF;int _tmpEE;struct Cyc_Absyn_Exp*_tmpED;void*_tmpF1;struct Cyc_Absyn_Vardecl*_tmpF0;switch(*((int*)_tmpE5.f2)){case 8U: _LL1: _tmpF0=((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmpE5.f2)->f1;_tmpF1=(void*)((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmpE5.f2)->f2;_LL2: {struct Cyc_Absyn_Vardecl*n=_tmpF0;void*r=_tmpF1;
# 715
void*new_r=Cyc_CfFlowInfo_assign_place_outer(env,path,path_prefix,r,newval);
if(new_r == r)return oldval;else{
return(void*)({struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*_tmpF2=_cycalloc(sizeof(*_tmpF2));_tmpF2->tag=8U,_tmpF2->f1=n,_tmpF2->f2=new_r;_tmpF2;});}}case 7U: _LL3: _tmpED=((struct Cyc_CfFlowInfo_Consumed_CfFlowInfo_AbsRVal_struct*)_tmpE5.f2)->f1;_tmpEE=((struct Cyc_CfFlowInfo_Consumed_CfFlowInfo_AbsRVal_struct*)_tmpE5.f2)->f2;_tmpEF=(void*)((struct Cyc_CfFlowInfo_Consumed_CfFlowInfo_AbsRVal_struct*)_tmpE5.f2)->f3;_LL4: {struct Cyc_Absyn_Exp*x=_tmpED;int y=_tmpEE;void*r=_tmpEF;
# 719
void*new_r=Cyc_CfFlowInfo_assign_place_outer(env,path,path_prefix,r,newval);
if(new_r == r)return oldval;else{
return(void*)({struct Cyc_CfFlowInfo_Consumed_CfFlowInfo_AbsRVal_struct*_tmpF3=_cycalloc(sizeof(*_tmpF3));_tmpF3->tag=7U,_tmpF3->f1=x,_tmpF3->f2=y,_tmpF3->f3=new_r;_tmpF3;});}}default: if(_tmpE5.f1 != 0){if(((struct Cyc_CfFlowInfo_Star_CfFlowInfo_PathCon_struct*)((struct Cyc_List_List*)_tmpE5.f1)->hd)->tag == 1U){if(((struct Cyc_CfFlowInfo_UniquePtr_CfFlowInfo_AbsRVal_struct*)_tmpE5.f2)->tag == 5U){_LL5: _tmpEB=(_tmpE5.f1)->tl;_tmpEC=(void*)((struct Cyc_CfFlowInfo_UniquePtr_CfFlowInfo_AbsRVal_struct*)_tmpE5.f2)->f1;_LL6: {struct Cyc_List_List*tl=_tmpEB;void*r=_tmpEC;
# 723
void*new_r=Cyc_CfFlowInfo_assign_place_outer(env,tl,path_prefix + 1,r,newval);
if(new_r == r)return oldval;else{
return(void*)({struct Cyc_CfFlowInfo_UniquePtr_CfFlowInfo_AbsRVal_struct*_tmpF4=_cycalloc(sizeof(*_tmpF4));_tmpF4->tag=5U,_tmpF4->f1=new_r;_tmpF4;});}}}else{goto _LL9;}}else{if(((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmpE5.f2)->tag == 6U){_LL7: _tmpE6=((struct Cyc_CfFlowInfo_Dot_CfFlowInfo_PathCon_struct*)(_tmpE5.f1)->hd)->f1;_tmpE7=(_tmpE5.f1)->tl;_tmpE8=(((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmpE5.f2)->f1).is_union;_tmpE9=(((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmpE5.f2)->f1).fieldnum;_tmpEA=((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmpE5.f2)->f2;_LL8: {int fnum=_tmpE6;struct Cyc_List_List*tl=_tmpE7;int is_union=_tmpE8;int fldnum=_tmpE9;struct _fat_ptr d=_tmpEA;
# 727
void*new_child=Cyc_CfFlowInfo_assign_place_outer(env,tl,path_prefix + 1,*((void**)_check_fat_subscript(d,sizeof(void*),fnum)),newval);
# 729
struct _fat_ptr new_child_agg=Cyc_CfFlowInfo_aggr_dict_insert(d,fnum,new_child);
if((void**)new_child_agg.curr == (void**)d.curr &&(!is_union || fldnum == fnum))return oldval;d=new_child_agg;
# 734
if(is_union){
int changed=0;
int sz=(int)_get_fat_size(d,sizeof(void*));
{int i=0;for(0;i < sz;++ i){
if(i != fnum){
struct _fat_ptr new_d=({
struct _fat_ptr _tmp244=d;int _tmp243=i;Cyc_CfFlowInfo_aggr_dict_insert(_tmp244,_tmp243,
Cyc_CfFlowInfo_insert_place_inner((env->fenv)->unknown_all,*((void**)_check_fat_subscript(d,sizeof(void*),i))));});
if((void**)new_d.curr != (void**)d.curr){
d=new_d;
changed=1;}}}}
# 750
if(changed){
struct Cyc_CfFlowInfo_Place*_stmttmp14=env->root_place;struct Cyc_List_List*_tmpF6;void*_tmpF5;_LLC: _tmpF5=_stmttmp14->root;_tmpF6=_stmttmp14->path;_LLD: {void*root=_tmpF5;struct Cyc_List_List*path=_tmpF6;
struct Cyc_List_List*new_path=({struct Cyc_List_List*(*_tmp246)(int(*f)(int*,void*),int*env,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmpF8)(int(*f)(int*,void*),int*env,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(int(*f)(int*,void*),int*env,struct Cyc_List_List*x))Cyc_List_filter_c;_tmpF8;});int*_tmp245=({int*_tmpF9=_cycalloc_atomic(sizeof(*_tmpF9));*_tmpF9=path_prefix;_tmpF9;});_tmp246(Cyc_CfFlowInfo_nprefix,_tmp245,path);});
struct Cyc_CfFlowInfo_Place*curr_place=({struct Cyc_CfFlowInfo_Place*_tmpF7=_cycalloc(sizeof(*_tmpF7));_tmpF7->root=root,_tmpF7->path=new_path;_tmpF7;});;}}}
# 734
return(void*)({struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*_tmpFA=_cycalloc(sizeof(*_tmpFA));
# 756
_tmpFA->tag=6U,(_tmpFA->f1).is_union=is_union,(_tmpFA->f1).fieldnum=fnum,_tmpFA->f2=d;_tmpFA;});}}else{goto _LL9;}}}else{_LL9: _LLA:
({void*_tmpFB=0U;({int(*_tmp248)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmpFD)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmpFD;});struct _fat_ptr _tmp247=({const char*_tmpFC="bad assign place";_tag_fat(_tmpFC,sizeof(char),17U);});_tmp248(_tmp247,_tag_fat(_tmpFB,sizeof(void*),0U));});});}}_LL0:;}}
# 642
struct Cyc_Dict_Dict Cyc_CfFlowInfo_assign_place(struct Cyc_CfFlowInfo_FlowEnv*fenv,unsigned loc,struct Cyc_Dict_Dict d,struct Cyc_CfFlowInfo_Place*place,void*r){
# 768
struct Cyc_List_List*_tmpFF;void*_tmpFE;_LL1: _tmpFE=place->root;_tmpFF=place->path;_LL2: {void*root=_tmpFE;struct Cyc_List_List*path=_tmpFF;
struct Cyc_CfFlowInfo_AssignEnv env=({struct Cyc_CfFlowInfo_AssignEnv _tmp1F0;_tmp1F0.fenv=fenv,({struct Cyc_CfFlowInfo_EscPile*_tmp249=({struct Cyc_CfFlowInfo_EscPile*_tmp100=_cycalloc(sizeof(*_tmp100));_tmp100->places=0;_tmp100;});_tmp1F0.pile=_tmp249;}),_tmp1F0.d=d,_tmp1F0.loc=loc,_tmp1F0.root_place=place;_tmp1F0;});
void*newval=({struct Cyc_List_List*_tmp24B=path;void*_tmp24A=Cyc_Dict_lookup(d,root);Cyc_CfFlowInfo_assign_place_outer(& env,_tmp24B,0,_tmp24A,r);});
return({struct Cyc_CfFlowInfo_FlowEnv*_tmp24D=fenv;struct Cyc_CfFlowInfo_EscPile*_tmp24C=env.pile;Cyc_CfFlowInfo_escape_these(_tmp24D,_tmp24C,Cyc_Dict_insert(d,root,newval));});}}struct Cyc_CfFlowInfo_JoinEnv{struct Cyc_CfFlowInfo_FlowEnv*fenv;struct Cyc_CfFlowInfo_EscPile*pile;struct Cyc_Dict_Dict d1;struct Cyc_Dict_Dict d2;};
# 783
static int Cyc_CfFlowInfo_absRval_lessthan_approx(void*ignore,void*r1,void*r2);struct _tuple18{enum Cyc_CfFlowInfo_InitLevel f1;enum Cyc_CfFlowInfo_InitLevel f2;};
# 791
static void*Cyc_CfFlowInfo_join_absRval(struct Cyc_CfFlowInfo_JoinEnv*env,void*a,void*r1,void*r2){
if(r1 == r2)return r1;{struct _tuple12 _stmttmp15=({struct _tuple12 _tmp1F2;_tmp1F2.f1=r1,_tmp1F2.f2=r2;_tmp1F2;});struct _tuple12 _tmp101=_stmttmp15;struct _fat_ptr _tmp107;int _tmp106;int _tmp105;struct _fat_ptr _tmp104;int _tmp103;int _tmp102;struct Cyc_CfFlowInfo_Place*_tmp108;void*_tmp109;void*_tmp10B;void*_tmp10A;void*_tmp10C;struct Cyc_CfFlowInfo_Place*_tmp10D;struct Cyc_CfFlowInfo_Place*_tmp10E;struct Cyc_CfFlowInfo_Place*_tmp10F;struct Cyc_CfFlowInfo_Place*_tmp111;struct Cyc_CfFlowInfo_Place*_tmp110;void*_tmp114;int _tmp113;struct Cyc_Absyn_Exp*_tmp112;void*_tmp117;int _tmp116;struct Cyc_Absyn_Exp*_tmp115;void*_tmp11D;int _tmp11C;struct Cyc_Absyn_Exp*_tmp11B;void*_tmp11A;int _tmp119;struct Cyc_Absyn_Exp*_tmp118;void*_tmp11F;struct Cyc_Absyn_Vardecl*_tmp11E;void*_tmp121;struct Cyc_Absyn_Vardecl*_tmp120;void*_tmp125;struct Cyc_Absyn_Vardecl*_tmp124;void*_tmp123;struct Cyc_Absyn_Vardecl*_tmp122;if(((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmp101.f1)->tag == 8U){if(((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmp101.f2)->tag == 8U){_LL1: _tmp122=((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmp101.f1)->f1;_tmp123=(void*)((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmp101.f1)->f2;_tmp124=((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmp101.f2)->f1;_tmp125=(void*)((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmp101.f2)->f2;_LL2: {struct Cyc_Absyn_Vardecl*n1=_tmp122;void*r1=_tmp123;struct Cyc_Absyn_Vardecl*n2=_tmp124;void*r2=_tmp125;
# 797
if(n1 == n2)
return(void*)({struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*_tmp126=_cycalloc(sizeof(*_tmp126));_tmp126->tag=8U,_tmp126->f1=n1,({void*_tmp24E=Cyc_CfFlowInfo_join_absRval(env,a,r1,r2);_tmp126->f2=_tmp24E;});_tmp126;});else{
# 800
return Cyc_CfFlowInfo_join_absRval(env,a,r1,r2);}}}else{_LL3: _tmp120=((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmp101.f1)->f1;_tmp121=(void*)((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmp101.f1)->f2;_LL4: {struct Cyc_Absyn_Vardecl*n1=_tmp120;void*r1=_tmp121;
return Cyc_CfFlowInfo_join_absRval(env,a,r1,r2);}}}else{if(((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmp101.f2)->tag == 8U){_LL5: _tmp11E=((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmp101.f2)->f1;_tmp11F=(void*)((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmp101.f2)->f2;_LL6: {struct Cyc_Absyn_Vardecl*n2=_tmp11E;void*r2=_tmp11F;
return Cyc_CfFlowInfo_join_absRval(env,a,r1,r2);}}else{if(((struct Cyc_CfFlowInfo_Consumed_CfFlowInfo_AbsRVal_struct*)_tmp101.f1)->tag == 7U){if(((struct Cyc_CfFlowInfo_Consumed_CfFlowInfo_AbsRVal_struct*)_tmp101.f2)->tag == 7U){_LL7: _tmp118=((struct Cyc_CfFlowInfo_Consumed_CfFlowInfo_AbsRVal_struct*)_tmp101.f1)->f1;_tmp119=((struct Cyc_CfFlowInfo_Consumed_CfFlowInfo_AbsRVal_struct*)_tmp101.f1)->f2;_tmp11A=(void*)((struct Cyc_CfFlowInfo_Consumed_CfFlowInfo_AbsRVal_struct*)_tmp101.f1)->f3;_tmp11B=((struct Cyc_CfFlowInfo_Consumed_CfFlowInfo_AbsRVal_struct*)_tmp101.f2)->f1;_tmp11C=((struct Cyc_CfFlowInfo_Consumed_CfFlowInfo_AbsRVal_struct*)_tmp101.f2)->f2;_tmp11D=(void*)((struct Cyc_CfFlowInfo_Consumed_CfFlowInfo_AbsRVal_struct*)_tmp101.f2)->f3;_LL8: {struct Cyc_Absyn_Exp*e1=_tmp118;int i1=_tmp119;void*r1=_tmp11A;struct Cyc_Absyn_Exp*e2=_tmp11B;int i2=_tmp11C;void*r2=_tmp11D;
# 805
if(e1 == e2)
return(void*)({struct Cyc_CfFlowInfo_Consumed_CfFlowInfo_AbsRVal_struct*_tmp127=_cycalloc(sizeof(*_tmp127));_tmp127->tag=7U,_tmp127->f1=e1,i1 > i2?_tmp127->f2=i1:(_tmp127->f2=i2),({void*_tmp24F=Cyc_CfFlowInfo_join_absRval(env,a,r1,r2);_tmp127->f3=_tmp24F;});_tmp127;});
# 805
{void*_tmp128=r1;struct Cyc_CfFlowInfo_Place*_tmp129;if(((struct Cyc_CfFlowInfo_AddressOf_CfFlowInfo_AbsRVal_struct*)_tmp128)->tag == 4U){_LL22: _tmp129=((struct Cyc_CfFlowInfo_AddressOf_CfFlowInfo_AbsRVal_struct*)_tmp128)->f1;_LL23: {struct Cyc_CfFlowInfo_Place*p=_tmp129;
# 808
Cyc_CfFlowInfo_add_place(env->pile,p);goto _LL21;}}else{_LL24: _LL25:
 goto _LL21;}_LL21:;}
# 811
{void*_tmp12A=r2;struct Cyc_CfFlowInfo_Place*_tmp12B;if(((struct Cyc_CfFlowInfo_AddressOf_CfFlowInfo_AbsRVal_struct*)_tmp12A)->tag == 4U){_LL27: _tmp12B=((struct Cyc_CfFlowInfo_AddressOf_CfFlowInfo_AbsRVal_struct*)_tmp12A)->f1;_LL28: {struct Cyc_CfFlowInfo_Place*p=_tmp12B;
Cyc_CfFlowInfo_add_place(env->pile,p);goto _LL26;}}else{_LL29: _LL2A:
 goto _LL26;}_LL26:;}
# 815
goto _LL0;}}else{_LL9: _tmp115=((struct Cyc_CfFlowInfo_Consumed_CfFlowInfo_AbsRVal_struct*)_tmp101.f1)->f1;_tmp116=((struct Cyc_CfFlowInfo_Consumed_CfFlowInfo_AbsRVal_struct*)_tmp101.f1)->f2;_tmp117=(void*)((struct Cyc_CfFlowInfo_Consumed_CfFlowInfo_AbsRVal_struct*)_tmp101.f1)->f3;_LLA: {struct Cyc_Absyn_Exp*e1=_tmp115;int i1=_tmp116;void*r1=_tmp117;
# 817
return(void*)({struct Cyc_CfFlowInfo_Consumed_CfFlowInfo_AbsRVal_struct*_tmp12C=_cycalloc(sizeof(*_tmp12C));_tmp12C->tag=7U,_tmp12C->f1=e1,_tmp12C->f2=i1,({void*_tmp250=Cyc_CfFlowInfo_join_absRval(env,a,r1,r2);_tmp12C->f3=_tmp250;});_tmp12C;});}}}else{if(((struct Cyc_CfFlowInfo_Consumed_CfFlowInfo_AbsRVal_struct*)_tmp101.f2)->tag == 7U){_LLB: _tmp112=((struct Cyc_CfFlowInfo_Consumed_CfFlowInfo_AbsRVal_struct*)_tmp101.f2)->f1;_tmp113=((struct Cyc_CfFlowInfo_Consumed_CfFlowInfo_AbsRVal_struct*)_tmp101.f2)->f2;_tmp114=(void*)((struct Cyc_CfFlowInfo_Consumed_CfFlowInfo_AbsRVal_struct*)_tmp101.f2)->f3;_LLC: {struct Cyc_Absyn_Exp*e2=_tmp112;int i2=_tmp113;void*r2=_tmp114;
# 819
return(void*)({struct Cyc_CfFlowInfo_Consumed_CfFlowInfo_AbsRVal_struct*_tmp12D=_cycalloc(sizeof(*_tmp12D));_tmp12D->tag=7U,_tmp12D->f1=e2,_tmp12D->f2=i2,({void*_tmp251=Cyc_CfFlowInfo_join_absRval(env,a,r1,r2);_tmp12D->f3=_tmp251;});_tmp12D;});}}else{switch(*((int*)_tmp101.f1)){case 4U: switch(*((int*)_tmp101.f2)){case 4U: _LLD: _tmp110=((struct Cyc_CfFlowInfo_AddressOf_CfFlowInfo_AbsRVal_struct*)_tmp101.f1)->f1;_tmp111=((struct Cyc_CfFlowInfo_AddressOf_CfFlowInfo_AbsRVal_struct*)_tmp101.f2)->f1;_LLE: {struct Cyc_CfFlowInfo_Place*p1=_tmp110;struct Cyc_CfFlowInfo_Place*p2=_tmp111;
# 822
if(Cyc_CfFlowInfo_place_cmp(p1,p2)== 0)return r1;Cyc_CfFlowInfo_add_place(env->pile,p1);
# 824
Cyc_CfFlowInfo_add_place(env->pile,p2);
goto _LL0;}case 1U: _LLF: _tmp10F=((struct Cyc_CfFlowInfo_AddressOf_CfFlowInfo_AbsRVal_struct*)_tmp101.f1)->f1;_LL10: {struct Cyc_CfFlowInfo_Place*p=_tmp10F;
# 828
Cyc_CfFlowInfo_add_place(env->pile,p);{
enum Cyc_CfFlowInfo_InitLevel _stmttmp16=Cyc_CfFlowInfo_initlevel(env->fenv,env->d1,r1);enum Cyc_CfFlowInfo_InitLevel _tmp12E=_stmttmp16;if(_tmp12E == Cyc_CfFlowInfo_AllIL){_LL2C: _LL2D:
 return(env->fenv)->notzeroall;}else{_LL2E: _LL2F:
 return(env->fenv)->unknown_none;}_LL2B:;}}default: _LL19: _tmp10E=((struct Cyc_CfFlowInfo_AddressOf_CfFlowInfo_AbsRVal_struct*)_tmp101.f1)->f1;_LL1A: {struct Cyc_CfFlowInfo_Place*p=_tmp10E;
# 854
Cyc_CfFlowInfo_add_place(env->pile,p);goto _LL0;}}case 1U: switch(*((int*)_tmp101.f2)){case 4U: _LL11: _tmp10D=((struct Cyc_CfFlowInfo_AddressOf_CfFlowInfo_AbsRVal_struct*)_tmp101.f2)->f1;_LL12: {struct Cyc_CfFlowInfo_Place*p=_tmp10D;
# 834
Cyc_CfFlowInfo_add_place(env->pile,p);{
enum Cyc_CfFlowInfo_InitLevel _stmttmp17=Cyc_CfFlowInfo_initlevel(env->fenv,env->d2,r2);enum Cyc_CfFlowInfo_InitLevel _tmp12F=_stmttmp17;if(_tmp12F == Cyc_CfFlowInfo_AllIL){_LL31: _LL32:
 return(env->fenv)->notzeroall;}else{_LL33: _LL34:
 return(env->fenv)->unknown_none;}_LL30:;}}case 5U: _LL17: _tmp10C=(void*)((struct Cyc_CfFlowInfo_UniquePtr_CfFlowInfo_AbsRVal_struct*)_tmp101.f2)->f1;_LL18: {void*r2=_tmp10C;
# 849
enum Cyc_CfFlowInfo_InitLevel _stmttmp19=Cyc_CfFlowInfo_initlevel(env->fenv,env->d2,r2);enum Cyc_CfFlowInfo_InitLevel _tmp132=_stmttmp19;if(_tmp132 == Cyc_CfFlowInfo_AllIL){_LL3B: _LL3C:
 return(env->fenv)->notzeroall;}else{_LL3D: _LL3E:
 return(env->fenv)->unknown_none;}_LL3A:;}default: goto _LL1F;}case 5U: switch(*((int*)_tmp101.f2)){case 5U: _LL13: _tmp10A=(void*)((struct Cyc_CfFlowInfo_UniquePtr_CfFlowInfo_AbsRVal_struct*)_tmp101.f1)->f1;_tmp10B=(void*)((struct Cyc_CfFlowInfo_UniquePtr_CfFlowInfo_AbsRVal_struct*)_tmp101.f2)->f1;_LL14: {void*r1=_tmp10A;void*r2=_tmp10B;
# 841
return(void*)({struct Cyc_CfFlowInfo_UniquePtr_CfFlowInfo_AbsRVal_struct*_tmp130=_cycalloc(sizeof(*_tmp130));_tmp130->tag=5U,({void*_tmp252=Cyc_CfFlowInfo_join_absRval(env,a,r1,r2);_tmp130->f1=_tmp252;});_tmp130;});}case 1U: _LL15: _tmp109=(void*)((struct Cyc_CfFlowInfo_UniquePtr_CfFlowInfo_AbsRVal_struct*)_tmp101.f1)->f1;_LL16: {void*r1=_tmp109;
# 844
enum Cyc_CfFlowInfo_InitLevel _stmttmp18=Cyc_CfFlowInfo_initlevel(env->fenv,env->d1,r1);enum Cyc_CfFlowInfo_InitLevel _tmp131=_stmttmp18;if(_tmp131 == Cyc_CfFlowInfo_AllIL){_LL36: _LL37:
 return(env->fenv)->notzeroall;}else{_LL38: _LL39:
 return(env->fenv)->unknown_none;}_LL35:;}case 4U: goto _LL1B;default: goto _LL1F;}default: if(((struct Cyc_CfFlowInfo_AddressOf_CfFlowInfo_AbsRVal_struct*)_tmp101.f2)->tag == 4U){_LL1B: _tmp108=((struct Cyc_CfFlowInfo_AddressOf_CfFlowInfo_AbsRVal_struct*)_tmp101.f2)->f1;_LL1C: {struct Cyc_CfFlowInfo_Place*p=_tmp108;
# 855
Cyc_CfFlowInfo_add_place(env->pile,p);goto _LL0;}}else{if(((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp101.f1)->tag == 6U){if(((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp101.f2)->tag == 6U){_LL1D: _tmp102=(((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp101.f1)->f1).is_union;_tmp103=(((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp101.f1)->f1).fieldnum;_tmp104=((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp101.f1)->f2;_tmp105=(((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp101.f2)->f1).is_union;_tmp106=(((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp101.f2)->f1).fieldnum;_tmp107=((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp101.f2)->f2;_LL1E: {int is_union1=_tmp102;int field_no1=_tmp103;struct _fat_ptr d1=_tmp104;int is_union2=_tmp105;int field_no2=_tmp106;struct _fat_ptr d2=_tmp107;
# 858
struct _fat_ptr new_d=({unsigned _tmp136=
_get_fat_size(d1,sizeof(void*));void**_tmp135=_cycalloc(_check_times(_tmp136,sizeof(void*)));({{unsigned _tmp1F1=_get_fat_size(d1,sizeof(void*));unsigned i;for(i=0;i < _tmp1F1;++ i){({void*_tmp256=({void*(*_tmp255)(struct Cyc_CfFlowInfo_JoinEnv*env,int a,void*r1,void*r2)=({void*(*_tmp134)(struct Cyc_CfFlowInfo_JoinEnv*env,int a,void*r1,void*r2)=(void*(*)(struct Cyc_CfFlowInfo_JoinEnv*env,int a,void*r1,void*r2))Cyc_CfFlowInfo_join_absRval;_tmp134;});struct Cyc_CfFlowInfo_JoinEnv*_tmp254=env;void*_tmp253=((void**)d1.curr)[(int)i];_tmp255(_tmp254,0,_tmp253,*((void**)_check_fat_subscript(d2,sizeof(void*),(int)i)));});_tmp135[i]=_tmp256;});}}0;});_tag_fat(_tmp135,sizeof(void*),_tmp136);});
# 863
int change=0;
{int i=0;for(0;(unsigned)i < _get_fat_size(d1,sizeof(void*));++ i){
if(*((void**)_check_fat_subscript(new_d,sizeof(void*),i))!= ((void**)d1.curr)[i]){
change=1;break;}}}
# 864
if(!change){
# 869
if(!is_union1)return r1;new_d=d1;}else{
# 873
change=0;
{int i=0;for(0;(unsigned)i < _get_fat_size(d1,sizeof(void*));++ i){
if(({void*_tmp257=*((void**)_check_fat_subscript(new_d,sizeof(void*),i));_tmp257 != *((void**)_check_fat_subscript(d2,sizeof(void*),i));})){
change=1;break;}}}
# 874
if(!change){
# 879
if(!is_union1)return r2;new_d=d2;}}
# 883
return(void*)({struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*_tmp133=_cycalloc(sizeof(*_tmp133));_tmp133->tag=6U,(_tmp133->f1).is_union=is_union1,field_no1 == field_no2?(_tmp133->f1).fieldnum=field_no1:((_tmp133->f1).fieldnum=- 1),_tmp133->f2=new_d;_tmp133;});}}else{goto _LL1F;}}else{_LL1F: _LL20:
# 885
 goto _LL0;}}}}}}}_LL0:;}{
# 887
enum Cyc_CfFlowInfo_InitLevel il1=Cyc_CfFlowInfo_initlevel(env->fenv,env->d1,r1);
enum Cyc_CfFlowInfo_InitLevel il2=Cyc_CfFlowInfo_initlevel(env->fenv,env->d2,r2);
struct _tuple12 _stmttmp1A=({struct _tuple12 _tmp1F5;_tmp1F5.f1=r1,_tmp1F5.f2=r2;_tmp1F5;});struct _tuple12 _tmp137=_stmttmp1A;if(((struct Cyc_CfFlowInfo_Esc_CfFlowInfo_AbsRVal_struct*)_tmp137.f1)->tag == 3U){_LL40: _LL41:
 goto _LL43;}else{if(((struct Cyc_CfFlowInfo_Esc_CfFlowInfo_AbsRVal_struct*)_tmp137.f2)->tag == 3U){_LL42: _LL43: {
# 892
struct _tuple18 _stmttmp1B=({struct _tuple18 _tmp1F3;_tmp1F3.f1=il1,_tmp1F3.f2=il2;_tmp1F3;});struct _tuple18 _tmp138=_stmttmp1B;if(_tmp138.f2 == Cyc_CfFlowInfo_NoneIL){_LL47: _LL48:
 goto _LL4A;}else{if(_tmp138.f1 == Cyc_CfFlowInfo_NoneIL){_LL49: _LL4A: return(env->fenv)->esc_none;}else{_LL4B: _LL4C:
 return(env->fenv)->esc_all;}}_LL46:;}}else{_LL44: _LL45: {
# 897
struct _tuple18 _stmttmp1C=({struct _tuple18 _tmp1F4;_tmp1F4.f1=il1,_tmp1F4.f2=il2;_tmp1F4;});struct _tuple18 _tmp139=_stmttmp1C;if(_tmp139.f2 == Cyc_CfFlowInfo_NoneIL){_LL4E: _LL4F:
 goto _LL51;}else{if(_tmp139.f1 == Cyc_CfFlowInfo_NoneIL){_LL50: _LL51: return(env->fenv)->unknown_none;}else{_LL52: _LL53:
 return(env->fenv)->unknown_all;}}_LL4D:;}}}_LL3F:;}}struct _tuple19{union Cyc_CfFlowInfo_FlowInfo f1;union Cyc_CfFlowInfo_FlowInfo f2;};
# 791
union Cyc_CfFlowInfo_FlowInfo Cyc_CfFlowInfo_join_flow(struct Cyc_CfFlowInfo_FlowEnv*fenv,union Cyc_CfFlowInfo_FlowInfo f1,union Cyc_CfFlowInfo_FlowInfo f2){
# 906
struct _tuple19 _stmttmp1D=({struct _tuple19 _tmp1F7;_tmp1F7.f1=f1,_tmp1F7.f2=f2;_tmp1F7;});struct _tuple19 _tmp13A=_stmttmp1D;struct Cyc_List_List*_tmp13E;struct Cyc_Dict_Dict _tmp13D;struct Cyc_List_List*_tmp13C;struct Cyc_Dict_Dict _tmp13B;if(((_tmp13A.f1).BottomFL).tag == 1){_LL1: _LL2:
 return f2;}else{if(((_tmp13A.f2).BottomFL).tag == 1){_LL3: _LL4:
 return f1;}else{_LL5: _tmp13B=(((_tmp13A.f1).ReachableFL).val).f1;_tmp13C=(((_tmp13A.f1).ReachableFL).val).f2;_tmp13D=(((_tmp13A.f2).ReachableFL).val).f1;_tmp13E=(((_tmp13A.f2).ReachableFL).val).f2;_LL6: {struct Cyc_Dict_Dict d1=_tmp13B;struct Cyc_List_List*r1=_tmp13C;struct Cyc_Dict_Dict d2=_tmp13D;struct Cyc_List_List*r2=_tmp13E;
# 912
if(d1.t == d2.t && r1 == r2)return f1;if(
Cyc_CfFlowInfo_flow_lessthan_approx(f1,f2))return f2;
# 912
if(
# 914
Cyc_CfFlowInfo_flow_lessthan_approx(f2,f1))return f1;{
# 912
struct Cyc_CfFlowInfo_JoinEnv env=({struct Cyc_CfFlowInfo_JoinEnv _tmp1F6;_tmp1F6.fenv=fenv,({
# 915
struct Cyc_CfFlowInfo_EscPile*_tmp258=({struct Cyc_CfFlowInfo_EscPile*_tmp140=_cycalloc(sizeof(*_tmp140));_tmp140->places=0;_tmp140;});_tmp1F6.pile=_tmp258;}),_tmp1F6.d1=d1,_tmp1F6.d2=d2;_tmp1F6;});
struct Cyc_Dict_Dict outdict=({struct Cyc_Dict_Dict(*_tmp25A)(void*(*f)(struct Cyc_CfFlowInfo_JoinEnv*,void*,void*,void*),struct Cyc_CfFlowInfo_JoinEnv*env,struct Cyc_Dict_Dict d1,struct Cyc_Dict_Dict d2)=({struct Cyc_Dict_Dict(*_tmp13F)(void*(*f)(struct Cyc_CfFlowInfo_JoinEnv*,void*,void*,void*),struct Cyc_CfFlowInfo_JoinEnv*env,struct Cyc_Dict_Dict d1,struct Cyc_Dict_Dict d2)=(struct Cyc_Dict_Dict(*)(void*(*f)(struct Cyc_CfFlowInfo_JoinEnv*,void*,void*,void*),struct Cyc_CfFlowInfo_JoinEnv*env,struct Cyc_Dict_Dict d1,struct Cyc_Dict_Dict d2))Cyc_Dict_intersect_c;_tmp13F;});struct Cyc_Dict_Dict _tmp259=d1;_tmp25A(Cyc_CfFlowInfo_join_absRval,& env,_tmp259,d2);});
struct Cyc_List_List*r=Cyc_Relations_join_relns(Cyc_Core_heap_region,r1,r2);
return({struct Cyc_Dict_Dict _tmp25B=Cyc_CfFlowInfo_escape_these(fenv,env.pile,outdict);Cyc_CfFlowInfo_ReachableFL(_tmp25B,r);});}}}}_LL0:;}struct _tuple20{union Cyc_CfFlowInfo_FlowInfo f1;void*f2;};struct _tuple21{union Cyc_CfFlowInfo_FlowInfo f1;union Cyc_CfFlowInfo_FlowInfo f2;union Cyc_CfFlowInfo_FlowInfo f3;};
# 791
struct _tuple20 Cyc_CfFlowInfo_join_flow_and_rval(struct Cyc_CfFlowInfo_FlowEnv*fenv,struct _tuple20 pr1,struct _tuple20 pr2){
# 927
void*_tmp142;union Cyc_CfFlowInfo_FlowInfo _tmp141;_LL1: _tmp141=pr1.f1;_tmp142=pr1.f2;_LL2: {union Cyc_CfFlowInfo_FlowInfo f1=_tmp141;void*r1=_tmp142;
void*_tmp144;union Cyc_CfFlowInfo_FlowInfo _tmp143;_LL4: _tmp143=pr2.f1;_tmp144=pr2.f2;_LL5: {union Cyc_CfFlowInfo_FlowInfo f2=_tmp143;void*r2=_tmp144;
union Cyc_CfFlowInfo_FlowInfo outflow=Cyc_CfFlowInfo_join_flow(fenv,f1,f2);
struct _tuple21 _stmttmp1E=({struct _tuple21 _tmp1FE;_tmp1FE.f1=f1,_tmp1FE.f2=f2,_tmp1FE.f3=outflow;_tmp1FE;});struct _tuple21 _tmp145=_stmttmp1E;struct Cyc_List_List*_tmp149;struct Cyc_Dict_Dict _tmp148;struct Cyc_Dict_Dict _tmp147;struct Cyc_Dict_Dict _tmp146;if(((_tmp145.f1).BottomFL).tag == 1){_LL7: _LL8:
 return({struct _tuple20 _tmp1F8;_tmp1F8.f1=outflow,_tmp1F8.f2=r2;_tmp1F8;});}else{if(((_tmp145.f2).BottomFL).tag == 1){_LL9: _LLA:
 return({struct _tuple20 _tmp1F9;_tmp1F9.f1=outflow,_tmp1F9.f2=r1;_tmp1F9;});}else{if(((_tmp145.f3).ReachableFL).tag == 2){_LLB: _tmp146=(((_tmp145.f1).ReachableFL).val).f1;_tmp147=(((_tmp145.f2).ReachableFL).val).f1;_tmp148=(((_tmp145.f3).ReachableFL).val).f1;_tmp149=(((_tmp145.f3).ReachableFL).val).f2;_LLC: {struct Cyc_Dict_Dict d1=_tmp146;struct Cyc_Dict_Dict d2=_tmp147;struct Cyc_Dict_Dict outd=_tmp148;struct Cyc_List_List*relns=_tmp149;
# 935
if(({int(*_tmp25D)(int ignore,void*r1,void*r2)=({int(*_tmp14A)(int ignore,void*r1,void*r2)=(int(*)(int ignore,void*r1,void*r2))Cyc_CfFlowInfo_absRval_lessthan_approx;_tmp14A;});void*_tmp25C=r1;_tmp25D(0,_tmp25C,r2);}))return({struct _tuple20 _tmp1FA;_tmp1FA.f1=outflow,_tmp1FA.f2=r2;_tmp1FA;});if(({
int(*_tmp25F)(int ignore,void*r1,void*r2)=({int(*_tmp14B)(int ignore,void*r1,void*r2)=(int(*)(int ignore,void*r1,void*r2))Cyc_CfFlowInfo_absRval_lessthan_approx;_tmp14B;});void*_tmp25E=r2;_tmp25F(0,_tmp25E,r1);}))return({struct _tuple20 _tmp1FB;_tmp1FB.f1=outflow,_tmp1FB.f2=r1;_tmp1FB;});{
# 935
struct Cyc_CfFlowInfo_JoinEnv env=({struct Cyc_CfFlowInfo_JoinEnv _tmp1FD;_tmp1FD.fenv=fenv,({
# 937
struct Cyc_CfFlowInfo_EscPile*_tmp260=({struct Cyc_CfFlowInfo_EscPile*_tmp14D=_cycalloc(sizeof(*_tmp14D));_tmp14D->places=0;_tmp14D;});_tmp1FD.pile=_tmp260;}),_tmp1FD.d1=d1,_tmp1FD.d2=d2;_tmp1FD;});
void*outr=({void*(*_tmp262)(struct Cyc_CfFlowInfo_JoinEnv*env,int a,void*r1,void*r2)=({void*(*_tmp14C)(struct Cyc_CfFlowInfo_JoinEnv*env,int a,void*r1,void*r2)=(void*(*)(struct Cyc_CfFlowInfo_JoinEnv*env,int a,void*r1,void*r2))Cyc_CfFlowInfo_join_absRval;_tmp14C;});void*_tmp261=r1;_tmp262(& env,0,_tmp261,r2);});
return({struct _tuple20 _tmp1FC;({union Cyc_CfFlowInfo_FlowInfo _tmp264=({struct Cyc_Dict_Dict _tmp263=Cyc_CfFlowInfo_escape_these(fenv,env.pile,outd);Cyc_CfFlowInfo_ReachableFL(_tmp263,relns);});_tmp1FC.f1=_tmp264;}),_tmp1FC.f2=outr;_tmp1FC;});}}}else{_LLD: _LLE:
# 941
({void*_tmp14E=0U;({int(*_tmp266)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp150)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmp150;});struct _fat_ptr _tmp265=({const char*_tmp14F="join_flow_and_rval: BottomFL outflow";_tag_fat(_tmp14F,sizeof(char),37U);});_tmp266(_tmp265,_tag_fat(_tmp14E,sizeof(void*),0U));});});}}}_LL6:;}}}
# 791
static int Cyc_CfFlowInfo_absRval_lessthan_approx(void*ignore,void*r1,void*r2){
# 947
if(r1 == r2)return 1;{struct _tuple12 _stmttmp1F=({struct _tuple12 _tmp1FF;_tmp1FF.f1=r1,_tmp1FF.f2=r2;_tmp1FF;});struct _tuple12 _tmp151=_stmttmp1F;struct _fat_ptr _tmp157;int _tmp156;int _tmp155;struct _fat_ptr _tmp154;int _tmp153;int _tmp152;void*_tmp159;void*_tmp158;struct Cyc_CfFlowInfo_Place*_tmp15B;struct Cyc_CfFlowInfo_Place*_tmp15A;void*_tmp15F;struct Cyc_Absyn_Vardecl*_tmp15E;void*_tmp15D;struct Cyc_Absyn_Vardecl*_tmp15C;if(((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmp151.f1)->tag == 8U){if(((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmp151.f2)->tag == 8U){_LL1: _tmp15C=((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmp151.f1)->f1;_tmp15D=(void*)((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmp151.f1)->f2;_tmp15E=((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmp151.f2)->f1;_tmp15F=(void*)((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmp151.f2)->f2;_LL2: {struct Cyc_Absyn_Vardecl*n1=_tmp15C;void*r1=_tmp15D;struct Cyc_Absyn_Vardecl*n2=_tmp15E;void*r2=_tmp15F;
# 951
return n1 == n2 && Cyc_CfFlowInfo_absRval_lessthan_approx(ignore,r1,r2);}}else{_LL3: _LL4:
 goto _LL6;}}else{if(((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmp151.f2)->tag == 8U){_LL5: _LL6:
 return 0;}else{if(((struct Cyc_CfFlowInfo_AddressOf_CfFlowInfo_AbsRVal_struct*)_tmp151.f1)->tag == 4U){if(((struct Cyc_CfFlowInfo_AddressOf_CfFlowInfo_AbsRVal_struct*)_tmp151.f2)->tag == 4U){_LL7: _tmp15A=((struct Cyc_CfFlowInfo_AddressOf_CfFlowInfo_AbsRVal_struct*)_tmp151.f1)->f1;_tmp15B=((struct Cyc_CfFlowInfo_AddressOf_CfFlowInfo_AbsRVal_struct*)_tmp151.f2)->f1;_LL8: {struct Cyc_CfFlowInfo_Place*p1=_tmp15A;struct Cyc_CfFlowInfo_Place*p2=_tmp15B;
return Cyc_CfFlowInfo_place_cmp(p1,p2)== 0;}}else{_LL9: _LLA:
 goto _LLC;}}else{if(((struct Cyc_CfFlowInfo_AddressOf_CfFlowInfo_AbsRVal_struct*)_tmp151.f2)->tag == 4U){_LLB: _LLC:
 return 0;}else{if(((struct Cyc_CfFlowInfo_UniquePtr_CfFlowInfo_AbsRVal_struct*)_tmp151.f1)->tag == 5U){if(((struct Cyc_CfFlowInfo_UniquePtr_CfFlowInfo_AbsRVal_struct*)_tmp151.f2)->tag == 5U){_LLD: _tmp158=(void*)((struct Cyc_CfFlowInfo_UniquePtr_CfFlowInfo_AbsRVal_struct*)_tmp151.f1)->f1;_tmp159=(void*)((struct Cyc_CfFlowInfo_UniquePtr_CfFlowInfo_AbsRVal_struct*)_tmp151.f2)->f1;_LLE: {void*r1=_tmp158;void*r2=_tmp159;
return Cyc_CfFlowInfo_absRval_lessthan_approx(ignore,r1,r2);}}else{_LLF: _LL10:
 goto _LL12;}}else{if(((struct Cyc_CfFlowInfo_UniquePtr_CfFlowInfo_AbsRVal_struct*)_tmp151.f2)->tag == 5U){_LL11: _LL12:
 return 0;}else{if(((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp151.f1)->tag == 6U)switch(*((int*)_tmp151.f2)){case 6U: _LL13: _tmp152=(((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp151.f1)->f1).is_union;_tmp153=(((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp151.f1)->f1).fieldnum;_tmp154=((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp151.f1)->f2;_tmp155=(((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp151.f2)->f1).is_union;_tmp156=(((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp151.f2)->f1).fieldnum;_tmp157=((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp151.f2)->f2;_LL14: {int is_union1=_tmp152;int fld1=_tmp153;struct _fat_ptr d1=_tmp154;int is_union2=_tmp155;int fld2=_tmp156;struct _fat_ptr d2=_tmp157;
# 962
if(is_union1 && fld1 != fld2)return 0;if((void**)d1.curr == (void**)d2.curr)
return 1;
# 962
{
# 964
int i=0;
# 962
for(0;(unsigned)i < 
# 964
_get_fat_size(d1,sizeof(void*));++ i){
if(!({int(*_tmp268)(int ignore,void*r1,void*r2)=({int(*_tmp160)(int ignore,void*r1,void*r2)=(int(*)(int ignore,void*r1,void*r2))Cyc_CfFlowInfo_absRval_lessthan_approx;_tmp160;});void*_tmp267=((void**)d1.curr)[i];_tmp268(0,_tmp267,*((void**)_check_fat_subscript(d2,sizeof(void*),i)));}))return 0;}}
# 962
return 1;}case 0U: goto _LL15;case 1U: goto _LL17;default: goto _LL1D;}else{switch(*((int*)_tmp151.f2)){case 0U: _LL15: _LL16:
# 967
 goto _LL18;case 1U: _LL17: _LL18:
 return 0;default: if(((struct Cyc_CfFlowInfo_Esc_CfFlowInfo_AbsRVal_struct*)_tmp151.f1)->tag == 3U){if(((struct Cyc_CfFlowInfo_Esc_CfFlowInfo_AbsRVal_struct*)_tmp151.f2)->tag == 3U){_LL19: _LL1A:
 goto _LL0;}else{_LL1B: _LL1C:
 return 0;}}else{_LL1D: _LL1E:
# 977
 goto _LL0;}}}}}}}}}_LL0:;}{
# 979
struct _tuple18 _stmttmp20=({struct _tuple18 _tmp200;({enum Cyc_CfFlowInfo_InitLevel _tmp26A=Cyc_CfFlowInfo_initlevel_approx(r1);_tmp200.f1=_tmp26A;}),({enum Cyc_CfFlowInfo_InitLevel _tmp269=Cyc_CfFlowInfo_initlevel_approx(r2);_tmp200.f2=_tmp269;});_tmp200;});struct _tuple18 _tmp161=_stmttmp20;if(_tmp161.f1 == Cyc_CfFlowInfo_AllIL){_LL20: _LL21:
 goto _LL23;}else{if(_tmp161.f2 == Cyc_CfFlowInfo_NoneIL){_LL22: _LL23:
 return 1;}else{_LL24: _LL25:
 return 0;}}_LL1F:;}}
# 791
int Cyc_CfFlowInfo_flow_lessthan_approx(union Cyc_CfFlowInfo_FlowInfo f1,union Cyc_CfFlowInfo_FlowInfo f2){
# 994
struct _tuple19 _stmttmp21=({struct _tuple19 _tmp201;_tmp201.f1=f1,_tmp201.f2=f2;_tmp201;});struct _tuple19 _tmp162=_stmttmp21;struct Cyc_List_List*_tmp166;struct Cyc_Dict_Dict _tmp165;struct Cyc_List_List*_tmp164;struct Cyc_Dict_Dict _tmp163;if(((_tmp162.f1).BottomFL).tag == 1){_LL1: _LL2:
 return 1;}else{if(((_tmp162.f2).BottomFL).tag == 1){_LL3: _LL4:
 return 0;}else{_LL5: _tmp163=(((_tmp162.f1).ReachableFL).val).f1;_tmp164=(((_tmp162.f1).ReachableFL).val).f2;_tmp165=(((_tmp162.f2).ReachableFL).val).f1;_tmp166=(((_tmp162.f2).ReachableFL).val).f2;_LL6: {struct Cyc_Dict_Dict d1=_tmp163;struct Cyc_List_List*r1=_tmp164;struct Cyc_Dict_Dict d2=_tmp165;struct Cyc_List_List*r2=_tmp166;
# 998
if(d1.t == d2.t && r1 == r2)return 1;return
 Cyc_Dict_forall_intersect(Cyc_CfFlowInfo_absRval_lessthan_approx,d1,d2)&&
 Cyc_Relations_relns_approx(r1,r2);}}}_LL0:;}struct _tuple22{void*f1;struct Cyc_List_List*f2;};
# 791
struct _tuple22 Cyc_CfFlowInfo_unname_rval(void*rv){
# 1005
struct Cyc_List_List*names=0;
int done=0;
while(!done){
void*_tmp167=rv;void*_tmp169;struct Cyc_Absyn_Vardecl*_tmp168;if(((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmp167)->tag == 8U){_LL1: _tmp168=((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmp167)->f1;_tmp169=(void*)((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmp167)->f2;_LL2: {struct Cyc_Absyn_Vardecl*vd=_tmp168;void*rv2=_tmp169;
# 1010
names=({struct Cyc_List_List*_tmp16A=_cycalloc(sizeof(*_tmp16A));_tmp16A->hd=vd,_tmp16A->tl=names;_tmp16A;});rv=rv2;goto _LL0;}}else{_LL3: _LL4:
# 1012
 done=1;goto _LL0;}_LL0:;}
# 1015
return({struct _tuple22 _tmp202;_tmp202.f1=rv,_tmp202.f2=names;_tmp202;});}
# 791
void Cyc_CfFlowInfo_print_initlevel(enum Cyc_CfFlowInfo_InitLevel il){
# 1019
enum Cyc_CfFlowInfo_InitLevel _tmp16B=il;if(_tmp16B == Cyc_CfFlowInfo_NoneIL){_LL1: _LL2:
({void*_tmp16C=0U;({struct Cyc___cycFILE*_tmp26C=Cyc_stderr;struct _fat_ptr _tmp26B=({const char*_tmp16D="uninitialized";_tag_fat(_tmp16D,sizeof(char),14U);});Cyc_fprintf(_tmp26C,_tmp26B,_tag_fat(_tmp16C,sizeof(void*),0U));});});goto _LL0;}else{_LL3: _LL4:
({void*_tmp16E=0U;({struct Cyc___cycFILE*_tmp26E=Cyc_stderr;struct _fat_ptr _tmp26D=({const char*_tmp16F="all-initialized";_tag_fat(_tmp16F,sizeof(char),16U);});Cyc_fprintf(_tmp26E,_tmp26D,_tag_fat(_tmp16E,sizeof(void*),0U));});});goto _LL0;}_LL0:;}
# 791
void Cyc_CfFlowInfo_print_root(void*root){
# 1026
void*_tmp170=root;void*_tmp172;struct Cyc_Absyn_Exp*_tmp171;struct Cyc_Absyn_Vardecl*_tmp173;switch(*((int*)_tmp170)){case 0U: _LL1: _tmp173=((struct Cyc_CfFlowInfo_VarRoot_CfFlowInfo_Root_struct*)_tmp170)->f1;_LL2: {struct Cyc_Absyn_Vardecl*vd=_tmp173;
# 1028
({struct Cyc_String_pa_PrintArg_struct _tmp176=({struct Cyc_String_pa_PrintArg_struct _tmp203;_tmp203.tag=0U,({struct _fat_ptr _tmp26F=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(vd->name));_tmp203.f1=_tmp26F;});_tmp203;});void*_tmp174[1U];_tmp174[0]=& _tmp176;({struct Cyc___cycFILE*_tmp271=Cyc_stderr;struct _fat_ptr _tmp270=({const char*_tmp175="Root(%s)";_tag_fat(_tmp175,sizeof(char),9U);});Cyc_fprintf(_tmp271,_tmp270,_tag_fat(_tmp174,sizeof(void*),1U));});});goto _LL0;}case 1U: _LL3: _tmp171=((struct Cyc_CfFlowInfo_MallocPt_CfFlowInfo_Root_struct*)_tmp170)->f1;_tmp172=(void*)((struct Cyc_CfFlowInfo_MallocPt_CfFlowInfo_Root_struct*)_tmp170)->f2;_LL4: {struct Cyc_Absyn_Exp*e=_tmp171;void*t=_tmp172;
# 1030
({struct Cyc_String_pa_PrintArg_struct _tmp179=({struct Cyc_String_pa_PrintArg_struct _tmp205;_tmp205.tag=0U,({
struct _fat_ptr _tmp272=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_exp2string(e));_tmp205.f1=_tmp272;});_tmp205;});struct Cyc_String_pa_PrintArg_struct _tmp17A=({struct Cyc_String_pa_PrintArg_struct _tmp204;_tmp204.tag=0U,({struct _fat_ptr _tmp273=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t));_tmp204.f1=_tmp273;});_tmp204;});void*_tmp177[2U];_tmp177[0]=& _tmp179,_tmp177[1]=& _tmp17A;({struct Cyc___cycFILE*_tmp275=Cyc_stderr;struct _fat_ptr _tmp274=({const char*_tmp178="MallocPt(%s,%s)";_tag_fat(_tmp178,sizeof(char),16U);});Cyc_fprintf(_tmp275,_tmp274,_tag_fat(_tmp177,sizeof(void*),2U));});});goto _LL0;}default: _LL5: _LL6:
# 1033
({void*_tmp17B=0U;({struct Cyc___cycFILE*_tmp277=Cyc_stderr;struct _fat_ptr _tmp276=({const char*_tmp17C="InitParam(_,_)";_tag_fat(_tmp17C,sizeof(char),15U);});Cyc_fprintf(_tmp277,_tmp276,_tag_fat(_tmp17B,sizeof(void*),0U));});});goto _LL0;}_LL0:;}
# 791
void Cyc_CfFlowInfo_print_path(struct Cyc_List_List*p){
# 1038
{struct Cyc_List_List*x=p;for(0;x != 0;x=((struct Cyc_List_List*)_check_null(x))->tl){
void*_stmttmp22=(void*)x->hd;void*_tmp17D=_stmttmp22;int _tmp17E;if(((struct Cyc_CfFlowInfo_Dot_CfFlowInfo_PathCon_struct*)_tmp17D)->tag == 0U){_LL1: _tmp17E=((struct Cyc_CfFlowInfo_Dot_CfFlowInfo_PathCon_struct*)_tmp17D)->f1;_LL2: {int i=_tmp17E;
# 1041
({struct Cyc_Int_pa_PrintArg_struct _tmp181=({struct Cyc_Int_pa_PrintArg_struct _tmp206;_tmp206.tag=1U,_tmp206.f1=(unsigned long)i;_tmp206;});void*_tmp17F[1U];_tmp17F[0]=& _tmp181;({struct Cyc___cycFILE*_tmp279=Cyc_stderr;struct _fat_ptr _tmp278=({const char*_tmp180=".%d";_tag_fat(_tmp180,sizeof(char),4U);});Cyc_fprintf(_tmp279,_tmp278,_tag_fat(_tmp17F,sizeof(void*),1U));});});
goto _LL0;}}else{_LL3: _LL4:
# 1044
 if(x->tl != 0){
void*_stmttmp23=(void*)((struct Cyc_List_List*)_check_null(x->tl))->hd;void*_tmp182=_stmttmp23;int _tmp183;if(((struct Cyc_CfFlowInfo_Dot_CfFlowInfo_PathCon_struct*)_tmp182)->tag == 0U){_LL6: _tmp183=((struct Cyc_CfFlowInfo_Dot_CfFlowInfo_PathCon_struct*)_tmp182)->f1;_LL7: {int i=_tmp183;
# 1047
x=x->tl;
({struct Cyc_Int_pa_PrintArg_struct _tmp186=({struct Cyc_Int_pa_PrintArg_struct _tmp207;_tmp207.tag=1U,_tmp207.f1=(unsigned long)i;_tmp207;});void*_tmp184[1U];_tmp184[0]=& _tmp186;({struct Cyc___cycFILE*_tmp27B=Cyc_stderr;struct _fat_ptr _tmp27A=({const char*_tmp185="->%d";_tag_fat(_tmp185,sizeof(char),5U);});Cyc_fprintf(_tmp27B,_tmp27A,_tag_fat(_tmp184,sizeof(void*),1U));});});
continue;}}else{_LL8: _LL9:
 goto _LL5;}_LL5:;}
# 1044
({void*_tmp187=0U;({struct Cyc___cycFILE*_tmp27D=Cyc_stderr;struct _fat_ptr _tmp27C=({const char*_tmp188="*";_tag_fat(_tmp188,sizeof(char),2U);});Cyc_fprintf(_tmp27D,_tmp27C,_tag_fat(_tmp187,sizeof(void*),0U));});});}_LL0:;}}
# 1056
({void*_tmp189=0U;({struct Cyc___cycFILE*_tmp27F=Cyc_stderr;struct _fat_ptr _tmp27E=({const char*_tmp18A=" ";_tag_fat(_tmp18A,sizeof(char),2U);});Cyc_fprintf(_tmp27F,_tmp27E,_tag_fat(_tmp189,sizeof(void*),0U));});});}
# 791
void Cyc_CfFlowInfo_print_place(struct Cyc_CfFlowInfo_Place*p){
# 1060
Cyc_CfFlowInfo_print_root(p->root);
Cyc_CfFlowInfo_print_path(p->path);}
# 791
void Cyc_CfFlowInfo_print_list(struct Cyc_List_List*x,void(*pr)(void*)){
# 1065
int first=1;
while(x != 0){
if(!first){({void*_tmp18B=0U;({struct Cyc___cycFILE*_tmp281=Cyc_stderr;struct _fat_ptr _tmp280=({const char*_tmp18C=", ";_tag_fat(_tmp18C,sizeof(char),3U);});Cyc_fprintf(_tmp281,_tmp280,_tag_fat(_tmp18B,sizeof(void*),0U));});});first=0;}pr(x->hd);
# 1069
x=x->tl;}
# 1071
({void*_tmp18D=0U;({struct Cyc___cycFILE*_tmp283=Cyc_stderr;struct _fat_ptr _tmp282=({const char*_tmp18E="\n";_tag_fat(_tmp18E,sizeof(char),2U);});Cyc_fprintf(_tmp283,_tmp282,_tag_fat(_tmp18D,sizeof(void*),0U));});});}
# 791
void Cyc_CfFlowInfo_print_absrval(void*rval){
# 1075
void*_tmp18F=rval;void*_tmp191;struct Cyc_Absyn_Vardecl*_tmp190;void*_tmp194;int _tmp193;struct Cyc_Absyn_Exp*_tmp192;struct _fat_ptr _tmp197;int _tmp196;int _tmp195;void*_tmp198;struct Cyc_CfFlowInfo_Place*_tmp199;enum Cyc_CfFlowInfo_InitLevel _tmp19A;enum Cyc_CfFlowInfo_InitLevel _tmp19B;switch(*((int*)_tmp18F)){case 0U: _LL1: _LL2:
({void*_tmp19C=0U;({struct Cyc___cycFILE*_tmp285=Cyc_stderr;struct _fat_ptr _tmp284=({const char*_tmp19D="Zero";_tag_fat(_tmp19D,sizeof(char),5U);});Cyc_fprintf(_tmp285,_tmp284,_tag_fat(_tmp19C,sizeof(void*),0U));});});goto _LL0;case 1U: _LL3: _LL4:
({void*_tmp19E=0U;({struct Cyc___cycFILE*_tmp287=Cyc_stderr;struct _fat_ptr _tmp286=({const char*_tmp19F="NotZeroAll";_tag_fat(_tmp19F,sizeof(char),11U);});Cyc_fprintf(_tmp287,_tmp286,_tag_fat(_tmp19E,sizeof(void*),0U));});});goto _LL0;case 2U: _LL5: _tmp19B=((struct Cyc_CfFlowInfo_UnknownR_CfFlowInfo_AbsRVal_struct*)_tmp18F)->f1;_LL6: {enum Cyc_CfFlowInfo_InitLevel il=_tmp19B;
({void*_tmp1A0=0U;({struct Cyc___cycFILE*_tmp289=Cyc_stderr;struct _fat_ptr _tmp288=({const char*_tmp1A1="Unknown(";_tag_fat(_tmp1A1,sizeof(char),9U);});Cyc_fprintf(_tmp289,_tmp288,_tag_fat(_tmp1A0,sizeof(void*),0U));});});Cyc_CfFlowInfo_print_initlevel(il);
({void*_tmp1A2=0U;({struct Cyc___cycFILE*_tmp28B=Cyc_stderr;struct _fat_ptr _tmp28A=({const char*_tmp1A3=")";_tag_fat(_tmp1A3,sizeof(char),2U);});Cyc_fprintf(_tmp28B,_tmp28A,_tag_fat(_tmp1A2,sizeof(void*),0U));});});goto _LL0;}case 3U: _LL7: _tmp19A=((struct Cyc_CfFlowInfo_Esc_CfFlowInfo_AbsRVal_struct*)_tmp18F)->f1;_LL8: {enum Cyc_CfFlowInfo_InitLevel il=_tmp19A;
({void*_tmp1A4=0U;({struct Cyc___cycFILE*_tmp28D=Cyc_stderr;struct _fat_ptr _tmp28C=({const char*_tmp1A5="Esc(";_tag_fat(_tmp1A5,sizeof(char),5U);});Cyc_fprintf(_tmp28D,_tmp28C,_tag_fat(_tmp1A4,sizeof(void*),0U));});});Cyc_CfFlowInfo_print_initlevel(il);
({void*_tmp1A6=0U;({struct Cyc___cycFILE*_tmp28F=Cyc_stderr;struct _fat_ptr _tmp28E=({const char*_tmp1A7=")";_tag_fat(_tmp1A7,sizeof(char),2U);});Cyc_fprintf(_tmp28F,_tmp28E,_tag_fat(_tmp1A6,sizeof(void*),0U));});});goto _LL0;}case 4U: _LL9: _tmp199=((struct Cyc_CfFlowInfo_AddressOf_CfFlowInfo_AbsRVal_struct*)_tmp18F)->f1;_LLA: {struct Cyc_CfFlowInfo_Place*p=_tmp199;
({void*_tmp1A8=0U;({struct Cyc___cycFILE*_tmp291=Cyc_stderr;struct _fat_ptr _tmp290=({const char*_tmp1A9="AddrOf(";_tag_fat(_tmp1A9,sizeof(char),8U);});Cyc_fprintf(_tmp291,_tmp290,_tag_fat(_tmp1A8,sizeof(void*),0U));});});Cyc_CfFlowInfo_print_place(p);
({void*_tmp1AA=0U;({struct Cyc___cycFILE*_tmp293=Cyc_stderr;struct _fat_ptr _tmp292=({const char*_tmp1AB=")";_tag_fat(_tmp1AB,sizeof(char),2U);});Cyc_fprintf(_tmp293,_tmp292,_tag_fat(_tmp1AA,sizeof(void*),0U));});});goto _LL0;}case 5U: _LLB: _tmp198=(void*)((struct Cyc_CfFlowInfo_UniquePtr_CfFlowInfo_AbsRVal_struct*)_tmp18F)->f1;_LLC: {void*r=_tmp198;
({void*_tmp1AC=0U;({struct Cyc___cycFILE*_tmp295=Cyc_stderr;struct _fat_ptr _tmp294=({const char*_tmp1AD="UniquePtr(";_tag_fat(_tmp1AD,sizeof(char),11U);});Cyc_fprintf(_tmp295,_tmp294,_tag_fat(_tmp1AC,sizeof(void*),0U));});});Cyc_CfFlowInfo_print_absrval(r);
({void*_tmp1AE=0U;({struct Cyc___cycFILE*_tmp297=Cyc_stderr;struct _fat_ptr _tmp296=({const char*_tmp1AF=")";_tag_fat(_tmp1AF,sizeof(char),2U);});Cyc_fprintf(_tmp297,_tmp296,_tag_fat(_tmp1AE,sizeof(void*),0U));});});goto _LL0;}case 6U: _LLD: _tmp195=(((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp18F)->f1).is_union;_tmp196=(((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp18F)->f1).fieldnum;_tmp197=((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp18F)->f2;_LLE: {int is_union=_tmp195;int f=_tmp196;struct _fat_ptr d=_tmp197;
# 1087
if(is_union){
({void*_tmp1B0=0U;({struct Cyc___cycFILE*_tmp299=Cyc_stderr;struct _fat_ptr _tmp298=({const char*_tmp1B1="AggrUnion{";_tag_fat(_tmp1B1,sizeof(char),11U);});Cyc_fprintf(_tmp299,_tmp298,_tag_fat(_tmp1B0,sizeof(void*),0U));});});
if(f != -1)
({struct Cyc_Int_pa_PrintArg_struct _tmp1B4=({struct Cyc_Int_pa_PrintArg_struct _tmp208;_tmp208.tag=1U,_tmp208.f1=(unsigned long)f;_tmp208;});void*_tmp1B2[1U];_tmp1B2[0]=& _tmp1B4;({struct Cyc___cycFILE*_tmp29B=Cyc_stderr;struct _fat_ptr _tmp29A=({const char*_tmp1B3="tag == %d;";_tag_fat(_tmp1B3,sizeof(char),11U);});Cyc_fprintf(_tmp29B,_tmp29A,_tag_fat(_tmp1B2,sizeof(void*),1U));});});}else{
# 1092
({void*_tmp1B5=0U;({struct Cyc___cycFILE*_tmp29D=Cyc_stderr;struct _fat_ptr _tmp29C=({const char*_tmp1B6="AggrStruct{";_tag_fat(_tmp1B6,sizeof(char),12U);});Cyc_fprintf(_tmp29D,_tmp29C,_tag_fat(_tmp1B5,sizeof(void*),0U));});});}
{int i=0;for(0;(unsigned)i < _get_fat_size(d,sizeof(void*));++ i){
Cyc_CfFlowInfo_print_absrval(((void**)d.curr)[i]);
if((unsigned)(i + 1)< _get_fat_size(d,sizeof(void*)))({void*_tmp1B7=0U;({struct Cyc___cycFILE*_tmp29F=Cyc_stderr;struct _fat_ptr _tmp29E=({const char*_tmp1B8=",";_tag_fat(_tmp1B8,sizeof(char),2U);});Cyc_fprintf(_tmp29F,_tmp29E,_tag_fat(_tmp1B7,sizeof(void*),0U));});});}}
# 1097
({void*_tmp1B9=0U;({struct Cyc___cycFILE*_tmp2A1=Cyc_stderr;struct _fat_ptr _tmp2A0=({const char*_tmp1BA="}";_tag_fat(_tmp1BA,sizeof(char),2U);});Cyc_fprintf(_tmp2A1,_tmp2A0,_tag_fat(_tmp1B9,sizeof(void*),0U));});});
goto _LL0;}case 7U: _LLF: _tmp192=((struct Cyc_CfFlowInfo_Consumed_CfFlowInfo_AbsRVal_struct*)_tmp18F)->f1;_tmp193=((struct Cyc_CfFlowInfo_Consumed_CfFlowInfo_AbsRVal_struct*)_tmp18F)->f2;_tmp194=(void*)((struct Cyc_CfFlowInfo_Consumed_CfFlowInfo_AbsRVal_struct*)_tmp18F)->f3;_LL10: {struct Cyc_Absyn_Exp*e=_tmp192;int i=_tmp193;void*r=_tmp194;
# 1100
({struct Cyc_String_pa_PrintArg_struct _tmp1BD=({struct Cyc_String_pa_PrintArg_struct _tmp20A;_tmp20A.tag=0U,({struct _fat_ptr _tmp2A2=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_exp2string(e));_tmp20A.f1=_tmp2A2;});_tmp20A;});struct Cyc_Int_pa_PrintArg_struct _tmp1BE=({struct Cyc_Int_pa_PrintArg_struct _tmp209;_tmp209.tag=1U,_tmp209.f1=(unsigned long)i;_tmp209;});void*_tmp1BB[2U];_tmp1BB[0]=& _tmp1BD,_tmp1BB[1]=& _tmp1BE;({struct Cyc___cycFILE*_tmp2A4=Cyc_stderr;struct _fat_ptr _tmp2A3=({const char*_tmp1BC="[Consumed(%s,%d,";_tag_fat(_tmp1BC,sizeof(char),17U);});Cyc_fprintf(_tmp2A4,_tmp2A3,_tag_fat(_tmp1BB,sizeof(void*),2U));});});
Cyc_CfFlowInfo_print_absrval(r);({void*_tmp1BF=0U;({struct Cyc___cycFILE*_tmp2A6=Cyc_stderr;struct _fat_ptr _tmp2A5=({const char*_tmp1C0=")]";_tag_fat(_tmp1C0,sizeof(char),3U);});Cyc_fprintf(_tmp2A6,_tmp2A5,_tag_fat(_tmp1BF,sizeof(void*),0U));});});
goto _LL0;}default: _LL11: _tmp190=((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmp18F)->f1;_tmp191=(void*)((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmp18F)->f2;_LL12: {struct Cyc_Absyn_Vardecl*n=_tmp190;void*r=_tmp191;
# 1104
({struct Cyc_String_pa_PrintArg_struct _tmp1C3=({struct Cyc_String_pa_PrintArg_struct _tmp20B;_tmp20B.tag=0U,({struct _fat_ptr _tmp2A7=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(n->name));_tmp20B.f1=_tmp2A7;});_tmp20B;});void*_tmp1C1[1U];_tmp1C1[0]=& _tmp1C3;({struct Cyc___cycFILE*_tmp2A9=Cyc_stderr;struct _fat_ptr _tmp2A8=({const char*_tmp1C2="[NamedLocation(%s,";_tag_fat(_tmp1C2,sizeof(char),19U);});Cyc_fprintf(_tmp2A9,_tmp2A8,_tag_fat(_tmp1C1,sizeof(void*),1U));});});
Cyc_CfFlowInfo_print_absrval(r);({void*_tmp1C4=0U;({struct Cyc___cycFILE*_tmp2AB=Cyc_stderr;struct _fat_ptr _tmp2AA=({const char*_tmp1C5=")]";_tag_fat(_tmp1C5,sizeof(char),3U);});Cyc_fprintf(_tmp2AB,_tmp2AA,_tag_fat(_tmp1C4,sizeof(void*),0U));});});
goto _LL0;}}_LL0:;}
# 791
static void Cyc_CfFlowInfo_print_flow_mapping(void*root,void*rval){
# 1111
({void*_tmp1C6=0U;({struct Cyc___cycFILE*_tmp2AD=Cyc_stderr;struct _fat_ptr _tmp2AC=({const char*_tmp1C7="    ";_tag_fat(_tmp1C7,sizeof(char),5U);});Cyc_fprintf(_tmp2AD,_tmp2AC,_tag_fat(_tmp1C6,sizeof(void*),0U));});});
Cyc_CfFlowInfo_print_root(root);
({void*_tmp1C8=0U;({struct Cyc___cycFILE*_tmp2AF=Cyc_stderr;struct _fat_ptr _tmp2AE=({const char*_tmp1C9=" --> ";_tag_fat(_tmp1C9,sizeof(char),6U);});Cyc_fprintf(_tmp2AF,_tmp2AE,_tag_fat(_tmp1C8,sizeof(void*),0U));});});
Cyc_CfFlowInfo_print_absrval(rval);
({void*_tmp1CA=0U;({struct Cyc___cycFILE*_tmp2B1=Cyc_stderr;struct _fat_ptr _tmp2B0=({const char*_tmp1CB="\n";_tag_fat(_tmp1CB,sizeof(char),2U);});Cyc_fprintf(_tmp2B1,_tmp2B0,_tag_fat(_tmp1CA,sizeof(void*),0U));});});}
# 791
void Cyc_CfFlowInfo_print_flowdict(struct Cyc_Dict_Dict d){
# 1119
Cyc_Dict_iter(Cyc_CfFlowInfo_print_flow_mapping,d);}
# 791
void Cyc_CfFlowInfo_print_flow(union Cyc_CfFlowInfo_FlowInfo f){
# 1123
union Cyc_CfFlowInfo_FlowInfo _tmp1CC=f;struct Cyc_List_List*_tmp1CE;struct Cyc_Dict_Dict _tmp1CD;if((_tmp1CC.BottomFL).tag == 1){_LL1: _LL2:
({void*_tmp1CF=0U;({struct Cyc___cycFILE*_tmp2B3=Cyc_stderr;struct _fat_ptr _tmp2B2=({const char*_tmp1D0="  BottomFL\n";_tag_fat(_tmp1D0,sizeof(char),12U);});Cyc_fprintf(_tmp2B3,_tmp2B2,_tag_fat(_tmp1CF,sizeof(void*),0U));});});goto _LL0;}else{_LL3: _tmp1CD=((_tmp1CC.ReachableFL).val).f1;_tmp1CE=((_tmp1CC.ReachableFL).val).f2;_LL4: {struct Cyc_Dict_Dict fd=_tmp1CD;struct Cyc_List_List*rlns=_tmp1CE;
# 1126
({void*_tmp1D1=0U;({struct Cyc___cycFILE*_tmp2B5=Cyc_stderr;struct _fat_ptr _tmp2B4=({const char*_tmp1D2="  ReachableFL:\n";_tag_fat(_tmp1D2,sizeof(char),16U);});Cyc_fprintf(_tmp2B5,_tmp2B4,_tag_fat(_tmp1D1,sizeof(void*),0U));});});
Cyc_CfFlowInfo_print_flowdict(fd);
({void*_tmp1D3=0U;({struct Cyc___cycFILE*_tmp2B7=Cyc_stderr;struct _fat_ptr _tmp2B6=({const char*_tmp1D4="\n  Relations: ";_tag_fat(_tmp1D4,sizeof(char),15U);});Cyc_fprintf(_tmp2B7,_tmp2B6,_tag_fat(_tmp1D3,sizeof(void*),0U));});});
Cyc_Relations_print_relns(Cyc_stderr,rlns);
({void*_tmp1D5=0U;({struct Cyc___cycFILE*_tmp2B9=Cyc_stderr;struct _fat_ptr _tmp2B8=({const char*_tmp1D6="\n";_tag_fat(_tmp1D6,sizeof(char),2U);});Cyc_fprintf(_tmp2B9,_tmp2B8,_tag_fat(_tmp1D5,sizeof(void*),0U));});});
goto _LL0;}}_LL0:;}
