#include <setjmp.h>
/* This is a C header file to be used by the output of the Cyclone to
   C translator.  The corresponding definitions are in file lib/runtime_*.c */
#ifndef _CYC_INCLUDE_H_
#define _CYC_INCLUDE_H_

/* Need one of these per thread (see runtime_stack.c). The runtime maintains 
   a stack that contains either _handler_cons structs or _RegionHandle structs.
   The tag is 0 for a handler_cons and 1 for a region handle.  */
struct _RuntimeStack {
  int tag; 
  struct _RuntimeStack *next;
  void (*cleanup)(struct _RuntimeStack *frame);
};

#ifndef offsetof
/* should be size_t, but int is fine. */
#define offsetof(t,n) ((int)(&(((t *)0)->n)))
#endif

/* Fat pointers */
struct _fat_ptr {
  unsigned char *curr; 
  unsigned char *base; 
  unsigned char *last_plus_one; 
};  

/* Discriminated Unions */
struct _xtunion_struct { char *tag; };

/* Regions */
struct _RegionPage
#ifdef CYC_REGION_PROFILE
{ unsigned total_bytes;
  unsigned free_bytes;
  /* MWH: wish we didn't have to include the stuff below ... */
  struct _RegionPage *next;
  char data[1];
}
#endif
; // abstract -- defined in runtime_memory.c
struct _RegionHandle {
  struct _RuntimeStack s;
  struct _RegionPage *curr;
  char               *offset;
  char               *last_plus_one;
  struct _DynRegionHandle *sub_regions;
#ifdef CYC_REGION_PROFILE
  const char         *name;
#else
  unsigned used_bytes;
  unsigned wasted_bytes;
#endif
};
struct _DynRegionFrame {
  struct _RuntimeStack s;
  struct _DynRegionHandle *x;
};
// A dynamic region is just a region handle.  The wrapper struct is for type
// abstraction.
struct Cyc_Core_DynamicRegion {
  struct _RegionHandle h;
};

struct _RegionHandle _new_region(const char*);
void* _region_malloc(struct _RegionHandle*, unsigned);
void* _region_calloc(struct _RegionHandle*, unsigned t, unsigned n);
void   _free_region(struct _RegionHandle*);
struct _RegionHandle*_open_dynregion(struct _DynRegionFrame*,struct _DynRegionHandle*);
void   _pop_dynregion();

/* Exceptions */
struct _handler_cons {
  struct _RuntimeStack s;
  jmp_buf handler;
};
void _push_handler(struct _handler_cons *);
void _push_region(struct _RegionHandle *);
void _npop_handler(int);
void _pop_handler();
void _pop_region();

#ifndef _throw
void* _throw_null_fn(const char*,unsigned);
void* _throw_arraybounds_fn(const char*,unsigned);
void* _throw_badalloc_fn(const char*,unsigned);
void* _throw_match_fn(const char*,unsigned);
void* _throw_fn(void*,const char*,unsigned);
void* _rethrow(void*);
#define _throw_null() (_throw_null_fn(__FILE__,__LINE__))
#define _throw_arraybounds() (_throw_arraybounds_fn(__FILE__,__LINE__))
#define _throw_badalloc() (_throw_badalloc_fn(__FILE__,__LINE__))
#define _throw_match() (_throw_match_fn(__FILE__,__LINE__))
#define _throw(e) (_throw_fn((e),__FILE__,__LINE__))
#endif

struct _xtunion_struct* Cyc_Core_get_exn_thrown();
/* Built-in Exceptions */
struct Cyc_Null_Exception_exn_struct { char *tag; };
struct Cyc_Array_bounds_exn_struct { char *tag; };
struct Cyc_Match_Exception_exn_struct { char *tag; };
struct Cyc_Bad_alloc_exn_struct { char *tag; };
extern char Cyc_Null_Exception[];
extern char Cyc_Array_bounds[];
extern char Cyc_Match_Exception[];
extern char Cyc_Bad_alloc[];

/* Built-in Run-time Checks and company */
#ifdef CYC_ANSI_OUTPUT
#define _INLINE  
#else
#define _INLINE inline
#endif

#ifdef NO_CYC_NULL_CHECKS
#define _check_null(ptr) (ptr)
#else
#define _check_null(ptr) \
  ({ void*_cks_null = (void*)(ptr); \
     if (!_cks_null) _throw_null(); \
     _cks_null; })
#endif

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index)\
   (((char*)ptr) + (elt_sz)*(index))
#ifdef NO_CYC_NULL_CHECKS
#define _check_known_subscript_null _check_known_subscript_notnull
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr);\
  int _index = (index);\
  if (!_cks_ptr) _throw_null(); \
  _cks_ptr + (elt_sz)*_index; })
#endif
#define _zero_arr_plus_fn(orig_x,orig_sz,orig_i,f,l) ((orig_x)+(orig_i))
#define _zero_arr_plus_char_fn _zero_arr_plus_fn
#define _zero_arr_plus_short_fn _zero_arr_plus_fn
#define _zero_arr_plus_int_fn _zero_arr_plus_fn
#define _zero_arr_plus_float_fn _zero_arr_plus_fn
#define _zero_arr_plus_double_fn _zero_arr_plus_fn
#define _zero_arr_plus_longdouble_fn _zero_arr_plus_fn
#define _zero_arr_plus_voidstar_fn _zero_arr_plus_fn
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (!_cks_ptr) _throw_null(); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })

/* _zero_arr_plus_*_fn(x,sz,i,filename,lineno) adds i to zero-terminated ptr
   x that has at least sz elements */
char* _zero_arr_plus_char_fn(char*,unsigned,int,const char*,unsigned);
short* _zero_arr_plus_short_fn(short*,unsigned,int,const char*,unsigned);
int* _zero_arr_plus_int_fn(int*,unsigned,int,const char*,unsigned);
float* _zero_arr_plus_float_fn(float*,unsigned,int,const char*,unsigned);
double* _zero_arr_plus_double_fn(double*,unsigned,int,const char*,unsigned);
long double* _zero_arr_plus_longdouble_fn(long double*,unsigned,int,const char*, unsigned);
void** _zero_arr_plus_voidstar_fn(void**,unsigned,int,const char*,unsigned);
#endif

/* _get_zero_arr_size_*(x,sz) returns the number of elements in a
   zero-terminated array that is NULL or has at least sz elements */
int _get_zero_arr_size_char(const char*,unsigned);
int _get_zero_arr_size_short(const short*,unsigned);
int _get_zero_arr_size_int(const int*,unsigned);
int _get_zero_arr_size_float(const float*,unsigned);
int _get_zero_arr_size_double(const double*,unsigned);
int _get_zero_arr_size_longdouble(const long double*,unsigned);
int _get_zero_arr_size_voidstar(const void**,unsigned);

/* _zero_arr_inplace_plus_*_fn(x,i,filename,lineno) sets
   zero-terminated pointer *x to *x + i */
char* _zero_arr_inplace_plus_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_short_fn(short**,int,const char*,unsigned);
int* _zero_arr_inplace_plus_int(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_longdouble_fn(long double**,int,const char*,unsigned);
void** _zero_arr_inplace_plus_voidstar_fn(void***,int,const char*,unsigned);
/* like the previous functions, but does post-addition (as in e++) */
char* _zero_arr_inplace_plus_post_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_post_short_fn(short**x,int,const char*,unsigned);
int* _zero_arr_inplace_plus_post_int_fn(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_post_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_post_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_post_longdouble_fn(long double**,int,const char *,unsigned);
void** _zero_arr_inplace_plus_post_voidstar_fn(void***,int,const char*,unsigned);
#define _zero_arr_plus_char(x,s,i) \
  (_zero_arr_plus_char_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_short(x,s,i) \
  (_zero_arr_plus_short_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_int(x,s,i) \
  (_zero_arr_plus_int_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_float(x,s,i) \
  (_zero_arr_plus_float_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_double(x,s,i) \
  (_zero_arr_plus_double_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_longdouble(x,s,i) \
  (_zero_arr_plus_longdouble_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_voidstar(x,s,i) \
  (_zero_arr_plus_voidstar_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_inplace_plus_char(x,i) \
  _zero_arr_inplace_plus_char_fn((char **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_short(x,i) \
  _zero_arr_inplace_plus_short_fn((short **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_int(x,i) \
  _zero_arr_inplace_plus_int_fn((int **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_float(x,i) \
  _zero_arr_inplace_plus_float_fn((float **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_double(x,i) \
  _zero_arr_inplace_plus_double_fn((double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_longdouble(x,i) \
  _zero_arr_inplace_plus_longdouble_fn((long double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_voidstar(x,i) \
  _zero_arr_inplace_plus_voidstar_fn((void ***)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_char(x,i) \
  _zero_arr_inplace_plus_post_char_fn((char **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_short(x,i) \
  _zero_arr_inplace_plus_post_short_fn((short **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_int(x,i) \
  _zero_arr_inplace_plus_post_int_fn((int **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_float(x,i) \
  _zero_arr_inplace_plus_post_float_fn((float **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_double(x,i) \
  _zero_arr_inplace_plus_post_double_fn((double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_longdouble(x,i) \
  _zero_arr_inplace_plus_post_longdouble_fn((long double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_voidstar(x,i) \
  _zero_arr_inplace_plus_post_voidstar_fn((void***)(x),(i),__FILE__,__LINE__)

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_fat_subscript(arr,elt_sz,index) ((arr).curr + (elt_sz) * (index))
#define _untag_fat_ptr(arr,elt_sz,num_elts) ((arr).curr)
#else
#define _check_fat_subscript(arr,elt_sz,index) ({ \
  struct _fat_ptr _cus_arr = (arr); \
  unsigned char *_cus_ans = _cus_arr.curr + (elt_sz) * (index); \
  /* JGM: not needed! if (!_cus_arr.base) _throw_null();*/ \
  if (_cus_ans < _cus_arr.base || _cus_ans >= _cus_arr.last_plus_one) \
    _throw_arraybounds(); \
  _cus_ans; })
#define _untag_fat_ptr(arr,elt_sz,num_elts) ({ \
  struct _fat_ptr _arr = (arr); \
  unsigned char *_curr = _arr.curr; \
  if ((_curr < _arr.base || _curr + (elt_sz) * (num_elts) > _arr.last_plus_one) &&\
      _curr != (unsigned char *)0) \
    _throw_arraybounds(); \
  _curr; })
#endif

#define _tag_fat(tcurr,elt_sz,num_elts) ({ \
  struct _fat_ptr _ans; \
  unsigned _num_elts = (num_elts);\
  _ans.base = _ans.curr = (void*)(tcurr); \
  /* JGM: if we're tagging NULL, ignore num_elts */ \
  _ans.last_plus_one = _ans.base ? (_ans.base + (elt_sz) * _num_elts) : 0; \
  _ans; })

#define _get_fat_size(arr,elt_sz) \
  ({struct _fat_ptr _arr = (arr); \
    unsigned char *_arr_curr=_arr.curr; \
    unsigned char *_arr_last=_arr.last_plus_one; \
    (_arr_curr < _arr.base || _arr_curr >= _arr_last) ? 0 : \
    ((_arr_last - _arr_curr) / (elt_sz));})

#define _fat_ptr_plus(arr,elt_sz,change) ({ \
  struct _fat_ptr _ans = (arr); \
  int _change = (change);\
  _ans.curr += (elt_sz) * _change;\
  _ans; })
#define _fat_ptr_inplace_plus(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  _arr_ptr->curr += (elt_sz) * (change);\
  *_arr_ptr; })
#define _fat_ptr_inplace_plus_post(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  struct _fat_ptr _ans = *_arr_ptr; \
  _arr_ptr->curr += (elt_sz) * (change);\
  _ans; })

//Not a macro since initialization order matters. Defined in runtime_zeroterm.c.
struct _fat_ptr _fat_ptr_decrease_size(struct _fat_ptr,unsigned sz,unsigned numelts);

/* Allocation */
void* GC_malloc(int);
void* GC_malloc_atomic(int);
void* GC_calloc(unsigned,unsigned);
void* GC_calloc_atomic(unsigned,unsigned);
// bound the allocation size to be < MAX_ALLOC_SIZE. See macros below for usage.
#define MAX_MALLOC_SIZE (1 << 28)
void* _bounded_GC_malloc(int,const char*,int);
void* _bounded_GC_malloc_atomic(int,const char*,int);
void* _bounded_GC_calloc(unsigned,unsigned,const char*,int);
void* _bounded_GC_calloc_atomic(unsigned,unsigned,const char*,int);
/* these macros are overridden below ifdef CYC_REGION_PROFILE */
#ifndef CYC_REGION_PROFILE
#define _cycalloc(n) _bounded_GC_malloc(n,__FILE__,__LINE__)
#define _cycalloc_atomic(n) _bounded_GC_malloc_atomic(n,__FILE__,__LINE__)
#define _cyccalloc(n,s) _bounded_GC_calloc(n,s,__FILE__,__LINE__)
#define _cyccalloc_atomic(n,s) _bounded_GC_calloc_atomic(n,s,__FILE__,__LINE__)
#endif

static _INLINE unsigned int _check_times(unsigned x, unsigned y) {
  unsigned long long whole_ans = 
    ((unsigned long long) x)*((unsigned long long)y);
  unsigned word_ans = (unsigned)whole_ans;
  if(word_ans < whole_ans || word_ans > MAX_MALLOC_SIZE)
    _throw_badalloc();
  return word_ans;
}

#define _CYC_MAX_REGION_CONST 2
#define _CYC_MIN_ALIGNMENT (sizeof(double))

#ifdef CYC_REGION_PROFILE
extern int rgn_total_bytes;
#endif

static _INLINE void *_fast_region_malloc(struct _RegionHandle *r, unsigned orig_s) {  
  if (r > (struct _RegionHandle *)_CYC_MAX_REGION_CONST && r->curr != 0) { 
#ifdef CYC_NOALIGN
    unsigned s =  orig_s;
#else
    unsigned s =  (orig_s + _CYC_MIN_ALIGNMENT - 1) & (~(_CYC_MIN_ALIGNMENT -1)); 
#endif
    char *result; 
    result = r->offset; 
    if (s <= (r->last_plus_one - result)) {
      r->offset = result + s; 
#ifdef CYC_REGION_PROFILE
    r->curr->free_bytes = r->curr->free_bytes - s;
    rgn_total_bytes += s;
#endif
      return result;
    }
  } 
  return _region_malloc(r,orig_s); 
}

#ifdef CYC_REGION_PROFILE
/* see macros below for usage. defined in runtime_memory.c */
void* _profile_GC_malloc(int,const char*,const char*,int);
void* _profile_GC_malloc_atomic(int,const char*,const char*,int);
void* _profile_GC_calloc(unsigned,unsigned,const char*,const char*,int);
void* _profile_GC_calloc_atomic(unsigned,unsigned,const char*,const char*,int);
void* _profile_region_malloc(struct _RegionHandle*,unsigned,const char*,const char*,int);
void* _profile_region_calloc(struct _RegionHandle*,unsigned,unsigned,const char *,const char*,int);
struct _RegionHandle _profile_new_region(const char*,const char*,const char*,int);
void _profile_free_region(struct _RegionHandle*,const char*,const char*,int);
#ifndef RUNTIME_CYC
#define _new_region(n) _profile_new_region(n,__FILE__,__FUNCTION__,__LINE__)
#define _free_region(r) _profile_free_region(r,__FILE__,__FUNCTION__,__LINE__)
#define _region_malloc(rh,n) _profile_region_malloc(rh,n,__FILE__,__FUNCTION__,__LINE__)
#define _region_calloc(rh,n,t) _profile_region_calloc(rh,n,t,__FILE__,__FUNCTION__,__LINE__)
#  endif
#define _cycalloc(n) _profile_GC_malloc(n,__FILE__,__FUNCTION__,__LINE__)
#define _cycalloc_atomic(n) _profile_GC_malloc_atomic(n,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc(n,s) _profile_GC_calloc(n,s,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc_atomic(n,s) _profile_GC_calloc_atomic(n,s,__FILE__,__FUNCTION__,__LINE__)
#endif
#endif
 struct Cyc___cycFILE;struct Cyc_String_pa_PrintArg_struct{int tag;struct _fat_ptr f1;};struct Cyc_Int_pa_PrintArg_struct{int tag;unsigned long f1;};struct Cyc_Double_pa_PrintArg_struct{int tag;double f1;};struct Cyc_LongDouble_pa_PrintArg_struct{int tag;long double f1;};struct Cyc_ShortPtr_pa_PrintArg_struct{int tag;short*f1;};struct Cyc_IntPtr_pa_PrintArg_struct{int tag;unsigned long*f1;};
# 73 "cycboot.h"
struct _fat_ptr Cyc_aprintf(struct _fat_ptr,struct _fat_ptr);
# 100
int Cyc_fprintf(struct Cyc___cycFILE*,struct _fat_ptr,struct _fat_ptr);struct Cyc_ShortPtr_sa_ScanfArg_struct{int tag;short*f1;};struct Cyc_UShortPtr_sa_ScanfArg_struct{int tag;unsigned short*f1;};struct Cyc_IntPtr_sa_ScanfArg_struct{int tag;int*f1;};struct Cyc_UIntPtr_sa_ScanfArg_struct{int tag;unsigned*f1;};struct Cyc_StringPtr_sa_ScanfArg_struct{int tag;struct _fat_ptr f1;};struct Cyc_DoublePtr_sa_ScanfArg_struct{int tag;double*f1;};struct Cyc_FloatPtr_sa_ScanfArg_struct{int tag;float*f1;};struct Cyc_CharPtr_sa_ScanfArg_struct{int tag;struct _fat_ptr f1;};extern char Cyc_FileCloseError[15U];struct Cyc_FileCloseError_exn_struct{char*tag;};extern char Cyc_FileOpenError[14U];struct Cyc_FileOpenError_exn_struct{char*tag;struct _fat_ptr f1;};struct Cyc_Core_Opt{void*v;};
# 117 "core.h"
void*Cyc_Core_identity(void*);extern char Cyc_Core_Invalid_argument[17U];struct Cyc_Core_Invalid_argument_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Failure[8U];struct Cyc_Core_Failure_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Impossible[11U];struct Cyc_Core_Impossible_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Not_found[10U];struct Cyc_Core_Not_found_exn_struct{char*tag;};extern char Cyc_Core_Unreachable[12U];struct Cyc_Core_Unreachable_exn_struct{char*tag;struct _fat_ptr f1;};
# 168
extern struct _RegionHandle*Cyc_Core_unique_region;struct Cyc_Core_DynamicRegion;struct Cyc_Core_NewDynamicRegion{struct Cyc_Core_DynamicRegion*key;};struct Cyc_Core_ThinRes{void*arr;unsigned nelts;};struct Cyc_List_List{void*hd;struct Cyc_List_List*tl;};
# 76 "list.h"
struct Cyc_List_List*Cyc_List_map(void*(*f)(void*),struct Cyc_List_List*x);extern char Cyc_List_List_mismatch[14U];struct Cyc_List_List_mismatch_exn_struct{char*tag;};
# 145
void*Cyc_List_fold_left(void*(*f)(void*,void*),void*accum,struct Cyc_List_List*x);
# 172
struct Cyc_List_List*Cyc_List_rev(struct Cyc_List_List*x);
# 178
struct Cyc_List_List*Cyc_List_imp_rev(struct Cyc_List_List*x);
# 184
struct Cyc_List_List*Cyc_List_append(struct Cyc_List_List*x,struct Cyc_List_List*y);extern char Cyc_List_Nth[4U];struct Cyc_List_Nth_exn_struct{char*tag;};struct Cyc_Iter_Iter{void*env;int(*next)(void*env,void*dest);};struct Cyc_Dict_T;struct Cyc_Dict_Dict{int(*rel)(void*,void*);struct _RegionHandle*r;const struct Cyc_Dict_T*t;};extern char Cyc_Dict_Present[8U];struct Cyc_Dict_Present_exn_struct{char*tag;};extern char Cyc_Dict_Absent[7U];struct Cyc_Dict_Absent_exn_struct{char*tag;};
# 62 "dict.h"
struct Cyc_Dict_Dict Cyc_Dict_empty(int(*cmp)(void*,void*));
# 83
int Cyc_Dict_member(struct Cyc_Dict_Dict d,void*k);
# 87
struct Cyc_Dict_Dict Cyc_Dict_insert(struct Cyc_Dict_Dict d,void*k,void*v);
# 110
void*Cyc_Dict_lookup(struct Cyc_Dict_Dict d,void*k);
# 131 "dict.h"
void*Cyc_Dict_fold(void*(*f)(void*,void*,void*),struct Cyc_Dict_Dict d,void*accum);
# 149
void Cyc_Dict_iter_c(void(*f)(void*,void*,void*),void*env,struct Cyc_Dict_Dict d);struct Cyc_Position_Error;struct Cyc_Relations_Reln;struct _union_Nmspace_Rel_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_Abs_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_C_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_Loc_n{int tag;int val;};union Cyc_Absyn_Nmspace{struct _union_Nmspace_Rel_n Rel_n;struct _union_Nmspace_Abs_n Abs_n;struct _union_Nmspace_C_n C_n;struct _union_Nmspace_Loc_n Loc_n;};
# 96 "absyn.h"
union Cyc_Absyn_Nmspace Cyc_Absyn_Rel_n(struct Cyc_List_List*);struct _tuple0{union Cyc_Absyn_Nmspace f1;struct _fat_ptr*f2;};
# 161
enum Cyc_Absyn_Scope{Cyc_Absyn_Static =0U,Cyc_Absyn_Abstract =1U,Cyc_Absyn_Public =2U,Cyc_Absyn_Extern =3U,Cyc_Absyn_ExternC =4U,Cyc_Absyn_Register =5U};struct Cyc_Absyn_Tqual{int print_const: 1;int q_volatile: 1;int q_restrict: 1;int real_const: 1;unsigned loc;};
# 182
enum Cyc_Absyn_Size_of{Cyc_Absyn_Char_sz =0U,Cyc_Absyn_Short_sz =1U,Cyc_Absyn_Int_sz =2U,Cyc_Absyn_Long_sz =3U,Cyc_Absyn_LongLong_sz =4U};
enum Cyc_Absyn_Sign{Cyc_Absyn_Signed =0U,Cyc_Absyn_Unsigned =1U,Cyc_Absyn_None =2U};
enum Cyc_Absyn_AggrKind{Cyc_Absyn_StructA =0U,Cyc_Absyn_UnionA =1U};
# 187
enum Cyc_Absyn_AliasQual{Cyc_Absyn_Aliasable =0U,Cyc_Absyn_Unique =1U,Cyc_Absyn_Top =2U};
# 192
enum Cyc_Absyn_KindQual{Cyc_Absyn_AnyKind =0U,Cyc_Absyn_MemKind =1U,Cyc_Absyn_BoxKind =2U,Cyc_Absyn_RgnKind =3U,Cyc_Absyn_EffKind =4U,Cyc_Absyn_IntKind =5U,Cyc_Absyn_BoolKind =6U,Cyc_Absyn_PtrBndKind =7U};struct Cyc_Absyn_Kind{enum Cyc_Absyn_KindQual kind;enum Cyc_Absyn_AliasQual aliasqual;};struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct{int tag;struct Cyc_Absyn_Kind*f1;};struct Cyc_Absyn_Unknown_kb_Absyn_KindBound_struct{int tag;struct Cyc_Core_Opt*f1;};struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct{int tag;struct Cyc_Core_Opt*f1;struct Cyc_Absyn_Kind*f2;};struct Cyc_Absyn_Tvar{struct _fat_ptr*name;int identity;void*kind;};struct Cyc_Absyn_PtrLoc{unsigned ptr_loc;unsigned rgn_loc;unsigned zt_loc;};struct Cyc_Absyn_PtrAtts{void*rgn;void*nullable;void*bounds;void*zero_term;struct Cyc_Absyn_PtrLoc*ptrloc;};struct Cyc_Absyn_PtrInfo{void*elt_type;struct Cyc_Absyn_Tqual elt_tq;struct Cyc_Absyn_PtrAtts ptr_atts;};struct Cyc_Absyn_VarargInfo{struct _fat_ptr*name;struct Cyc_Absyn_Tqual tq;void*type;int inject;};struct Cyc_Absyn_Star_Absyn_Cap_struct{int tag;};struct Cyc_Absyn_Bot_Absyn_Cap_struct{int tag;};struct Cyc_Absyn_Nat_Absyn_Cap_struct{int tag;struct Cyc_Absyn_Exp*f1;int f2;};struct Cyc_Absyn_RgnEffect{void*name;void*rgn;void*lock;void*unadopt;void*parent;};struct Cyc_Absyn_FnInfo{struct Cyc_List_List*tvars;void*effect;struct Cyc_Absyn_Tqual ret_tqual;void*ret_type;struct Cyc_List_List*args;int c_varargs;struct Cyc_Absyn_VarargInfo*cyc_varargs;struct Cyc_List_List*rgn_po;struct Cyc_List_List*attributes;struct Cyc_Absyn_Exp*requires_clause;struct Cyc_List_List*requires_relns;struct Cyc_Absyn_Exp*ensures_clause;struct Cyc_List_List*ensures_relns;struct Cyc_List_List*ieffect;struct Cyc_List_List*oeffect;};struct Cyc_Absyn_UnknownDatatypeInfo{struct _tuple0*name;int is_extensible;};struct _union_DatatypeInfo_UnknownDatatype{int tag;struct Cyc_Absyn_UnknownDatatypeInfo val;};struct _union_DatatypeInfo_KnownDatatype{int tag;struct Cyc_Absyn_Datatypedecl**val;};union Cyc_Absyn_DatatypeInfo{struct _union_DatatypeInfo_UnknownDatatype UnknownDatatype;struct _union_DatatypeInfo_KnownDatatype KnownDatatype;};struct Cyc_Absyn_UnknownDatatypeFieldInfo{struct _tuple0*datatype_name;struct _tuple0*field_name;int is_extensible;};struct _union_DatatypeFieldInfo_UnknownDatatypefield{int tag;struct Cyc_Absyn_UnknownDatatypeFieldInfo val;};struct _tuple1{struct Cyc_Absyn_Datatypedecl*f1;struct Cyc_Absyn_Datatypefield*f2;};struct _union_DatatypeFieldInfo_KnownDatatypefield{int tag;struct _tuple1 val;};union Cyc_Absyn_DatatypeFieldInfo{struct _union_DatatypeFieldInfo_UnknownDatatypefield UnknownDatatypefield;struct _union_DatatypeFieldInfo_KnownDatatypefield KnownDatatypefield;};struct _tuple2{enum Cyc_Absyn_AggrKind f1;struct _tuple0*f2;struct Cyc_Core_Opt*f3;};struct _union_AggrInfo_UnknownAggr{int tag;struct _tuple2 val;};struct _union_AggrInfo_KnownAggr{int tag;struct Cyc_Absyn_Aggrdecl**val;};union Cyc_Absyn_AggrInfo{struct _union_AggrInfo_UnknownAggr UnknownAggr;struct _union_AggrInfo_KnownAggr KnownAggr;};struct Cyc_Absyn_ArrayInfo{void*elt_type;struct Cyc_Absyn_Tqual tq;struct Cyc_Absyn_Exp*num_elts;void*zero_term;unsigned zt_loc;};struct Cyc_Absyn_Aggr_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Aggrdecl*f1;};struct Cyc_Absyn_Enum_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;};struct Cyc_Absyn_Datatype_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;};struct Cyc_Absyn_TypeDecl{void*r;unsigned loc;};struct Cyc_Absyn_VoidCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_IntCon_Absyn_TyCon_struct{int tag;enum Cyc_Absyn_Sign f1;enum Cyc_Absyn_Size_of f2;};struct Cyc_Absyn_FloatCon_Absyn_TyCon_struct{int tag;int f1;};struct Cyc_Absyn_RgnHandleCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_TagCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_HeapCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_UniqueCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_RefCntCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_AccessCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_JoinCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_RgnsCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_TrueCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_FalseCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_ThinCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_FatCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_EnumCon_Absyn_TyCon_struct{int tag;struct _tuple0*f1;struct Cyc_Absyn_Enumdecl*f2;};struct Cyc_Absyn_AnonEnumCon_Absyn_TyCon_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_BuiltinCon_Absyn_TyCon_struct{int tag;struct _fat_ptr f1;struct Cyc_Absyn_Kind*f2;};struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_DatatypeInfo f1;};struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_DatatypeFieldInfo f1;};struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_AggrInfo f1;};struct Cyc_Absyn_AppType_Absyn_Type_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Evar_Absyn_Type_struct{int tag;struct Cyc_Core_Opt*f1;void*f2;int f3;struct Cyc_Core_Opt*f4;};struct Cyc_Absyn_VarType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Tvar*f1;};struct Cyc_Absyn_PointerType_Absyn_Type_struct{int tag;struct Cyc_Absyn_PtrInfo f1;};struct Cyc_Absyn_ArrayType_Absyn_Type_struct{int tag;struct Cyc_Absyn_ArrayInfo f1;};struct Cyc_Absyn_FnType_Absyn_Type_struct{int tag;struct Cyc_Absyn_FnInfo f1;};struct Cyc_Absyn_TupleType_Absyn_Type_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct{int tag;enum Cyc_Absyn_AggrKind f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_TypedefType_Absyn_Type_struct{int tag;struct _tuple0*f1;struct Cyc_List_List*f2;struct Cyc_Absyn_Typedefdecl*f3;void*f4;};struct Cyc_Absyn_ValueofType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct{int tag;struct Cyc_Absyn_TypeDecl*f1;void**f2;};struct Cyc_Absyn_TypeofType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_NoTypes_Absyn_Funcparams_struct{int tag;struct Cyc_List_List*f1;unsigned f2;};struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct{int tag;struct Cyc_List_List*f1;int f2;struct Cyc_Absyn_VarargInfo*f3;void*f4;struct Cyc_List_List*f5;struct Cyc_Absyn_Exp*f6;struct Cyc_Absyn_Exp*f7;struct Cyc_List_List*f8;struct Cyc_List_List*f9;};
# 437 "absyn.h"
enum Cyc_Absyn_Format_Type{Cyc_Absyn_Printf_ft =0U,Cyc_Absyn_Scanf_ft =1U};struct Cyc_Absyn_Regparm_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Stdcall_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Cdecl_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Fastcall_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Noreturn_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Const_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Aligned_att_Absyn_Attribute_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Packed_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Section_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Nocommon_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Shared_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Unused_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Weak_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Dllimport_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Dllexport_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_instrument_function_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Constructor_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Destructor_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_check_memory_usage_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Format_att_Absyn_Attribute_struct{int tag;enum Cyc_Absyn_Format_Type f1;int f2;int f3;};struct Cyc_Absyn_Initializes_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Noliveunique_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Consume_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Pure_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Mode_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Alias_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Always_inline_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_throw_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Non_null_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Deprecated_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Carray_mod_Absyn_Type_modifier_struct{int tag;void*f1;unsigned f2;};struct Cyc_Absyn_ConstArray_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_Absyn_Exp*f1;void*f2;unsigned f3;};struct Cyc_Absyn_Pointer_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_Absyn_PtrAtts f1;struct Cyc_Absyn_Tqual f2;};struct Cyc_Absyn_Function_mod_Absyn_Type_modifier_struct{int tag;void*f1;};struct Cyc_Absyn_TypeParams_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_List_List*f1;unsigned f2;int f3;};struct Cyc_Absyn_Attributes_mod_Absyn_Type_modifier_struct{int tag;unsigned f1;struct Cyc_List_List*f2;};struct _union_Cnst_Null_c{int tag;int val;};struct _tuple3{enum Cyc_Absyn_Sign f1;char f2;};struct _union_Cnst_Char_c{int tag;struct _tuple3 val;};struct _union_Cnst_Wchar_c{int tag;struct _fat_ptr val;};struct _tuple4{enum Cyc_Absyn_Sign f1;short f2;};struct _union_Cnst_Short_c{int tag;struct _tuple4 val;};struct _tuple5{enum Cyc_Absyn_Sign f1;int f2;};struct _union_Cnst_Int_c{int tag;struct _tuple5 val;};struct _tuple6{enum Cyc_Absyn_Sign f1;long long f2;};struct _union_Cnst_LongLong_c{int tag;struct _tuple6 val;};struct _tuple7{struct _fat_ptr f1;int f2;};struct _union_Cnst_Float_c{int tag;struct _tuple7 val;};struct _union_Cnst_String_c{int tag;struct _fat_ptr val;};struct _union_Cnst_Wstring_c{int tag;struct _fat_ptr val;};union Cyc_Absyn_Cnst{struct _union_Cnst_Null_c Null_c;struct _union_Cnst_Char_c Char_c;struct _union_Cnst_Wchar_c Wchar_c;struct _union_Cnst_Short_c Short_c;struct _union_Cnst_Int_c Int_c;struct _union_Cnst_LongLong_c LongLong_c;struct _union_Cnst_Float_c Float_c;struct _union_Cnst_String_c String_c;struct _union_Cnst_Wstring_c Wstring_c;};
# 532
enum Cyc_Absyn_Primop{Cyc_Absyn_Plus =0U,Cyc_Absyn_Times =1U,Cyc_Absyn_Minus =2U,Cyc_Absyn_Div =3U,Cyc_Absyn_Mod =4U,Cyc_Absyn_Eq =5U,Cyc_Absyn_Neq =6U,Cyc_Absyn_Gt =7U,Cyc_Absyn_Lt =8U,Cyc_Absyn_Gte =9U,Cyc_Absyn_Lte =10U,Cyc_Absyn_Not =11U,Cyc_Absyn_Bitnot =12U,Cyc_Absyn_Bitand =13U,Cyc_Absyn_Bitor =14U,Cyc_Absyn_Bitxor =15U,Cyc_Absyn_Bitlshift =16U,Cyc_Absyn_Bitlrshift =17U,Cyc_Absyn_Numelts =18U};
# 539
enum Cyc_Absyn_Incrementor{Cyc_Absyn_PreInc =0U,Cyc_Absyn_PostInc =1U,Cyc_Absyn_PreDec =2U,Cyc_Absyn_PostDec =3U};struct Cyc_Absyn_VarargCallInfo{int num_varargs;struct Cyc_List_List*injectors;struct Cyc_Absyn_VarargInfo*vai;};struct Cyc_Absyn_StructField_Absyn_OffsetofField_struct{int tag;struct _fat_ptr*f1;};struct Cyc_Absyn_TupleIndex_Absyn_OffsetofField_struct{int tag;unsigned f1;};
# 557
enum Cyc_Absyn_Coercion{Cyc_Absyn_Unknown_coercion =0U,Cyc_Absyn_No_coercion =1U,Cyc_Absyn_Null_to_NonNull =2U,Cyc_Absyn_Other_coercion =3U};struct Cyc_Absyn_MallocInfo{int is_calloc;struct Cyc_Absyn_Exp*rgn;void**elt_type;struct Cyc_Absyn_Exp*num_elts;int fat_result;int inline_call;};struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct{int tag;union Cyc_Absyn_Cnst f1;};struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Pragma_e_Absyn_Raw_exp_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct{int tag;enum Cyc_Absyn_Primop f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_AssignOp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Core_Opt*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_Increment_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;enum Cyc_Absyn_Incrementor f2;};struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;struct Cyc_Absyn_VarargCallInfo*f3;int f4;};struct Cyc_Absyn_Throw_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;int f2;};struct Cyc_Absyn_NoInstantiate_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Instantiate_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_Absyn_Exp*f2;int f3;enum Cyc_Absyn_Coercion f4;};struct Cyc_Absyn_Address_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_New_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Sizeoftype_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Sizeofexp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Offsetof_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Deref_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;int f3;int f4;};struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;int f3;int f4;};struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Tuple_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;};struct _tuple8{struct _fat_ptr*f1;struct Cyc_Absyn_Tqual f2;void*f3;};struct Cyc_Absyn_CompoundLit_e_Absyn_Raw_exp_struct{int tag;struct _tuple8*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Array_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Exp*f2;struct Cyc_Absyn_Exp*f3;int f4;};struct Cyc_Absyn_ComprehensionNoinit_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;void*f2;int f3;};struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct{int tag;struct _tuple0*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;struct Cyc_Absyn_Aggrdecl*f4;};struct Cyc_Absyn_AnonStruct_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Datatype_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;struct Cyc_Absyn_Datatypedecl*f2;struct Cyc_Absyn_Datatypefield*f3;};struct Cyc_Absyn_Enum_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_AnonEnum_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_Spawn_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_MallocInfo f1;};struct Cyc_Absyn_Swap_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_UnresolvedMem_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Core_Opt*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_StmtExp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Stmt*f1;};struct Cyc_Absyn_Tagcheck_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;};struct Cyc_Absyn_Valueof_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Asm_e_Absyn_Raw_exp_struct{int tag;int f1;struct _fat_ptr f2;struct Cyc_List_List*f3;struct Cyc_List_List*f4;struct Cyc_List_List*f5;};struct Cyc_Absyn_Extension_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Exp{void*topt;void*r;unsigned loc;void*annot;};struct Cyc_Absyn_Skip_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Exp_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Seq_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Return_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_IfThenElse_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Stmt*f2;struct Cyc_Absyn_Stmt*f3;};struct _tuple9{struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_While_s_Absyn_Raw_stmt_struct{int tag;struct _tuple9 f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Break_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Continue_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Goto_s_Absyn_Raw_stmt_struct{int tag;struct _fat_ptr*f1;};struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _tuple9 f2;struct _tuple9 f3;struct Cyc_Absyn_Stmt*f4;};struct Cyc_Absyn_Switch_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;void*f3;};struct Cyc_Absyn_Fallthru_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_List_List*f1;struct Cyc_Absyn_Switch_clause**f2;};struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Decl*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Label_s_Absyn_Raw_stmt_struct{int tag;struct _fat_ptr*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Do_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct _tuple9 f2;};struct Cyc_Absyn_TryCatch_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct Cyc_List_List*f2;void*f3;};struct Cyc_Absyn_Stmt{void*r;unsigned loc;void*annot;};struct Cyc_Absyn_Wild_p_Absyn_Raw_pat_struct{int tag;};struct Cyc_Absyn_Var_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Pat*f2;};struct Cyc_Absyn_AliasVar_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;};struct Cyc_Absyn_Reference_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Pat*f2;};struct Cyc_Absyn_TagInt_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;};struct Cyc_Absyn_Tuple_p_Absyn_Raw_pat_struct{int tag;struct Cyc_List_List*f1;int f2;};struct Cyc_Absyn_Pointer_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Pat*f1;};struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct{int tag;union Cyc_Absyn_AggrInfo*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;int f4;};struct Cyc_Absyn_Datatype_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;struct Cyc_Absyn_Datatypefield*f2;struct Cyc_List_List*f3;int f4;};struct Cyc_Absyn_Null_p_Absyn_Raw_pat_struct{int tag;};struct Cyc_Absyn_Int_p_Absyn_Raw_pat_struct{int tag;enum Cyc_Absyn_Sign f1;int f2;};struct Cyc_Absyn_Char_p_Absyn_Raw_pat_struct{int tag;char f1;};struct Cyc_Absyn_Float_p_Absyn_Raw_pat_struct{int tag;struct _fat_ptr f1;int f2;};struct Cyc_Absyn_Enum_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_AnonEnum_p_Absyn_Raw_pat_struct{int tag;void*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_UnknownId_p_Absyn_Raw_pat_struct{int tag;struct _tuple0*f1;};struct Cyc_Absyn_UnknownCall_p_Absyn_Raw_pat_struct{int tag;struct _tuple0*f1;struct Cyc_List_List*f2;int f3;};struct Cyc_Absyn_Exp_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Pat{void*r;void*topt;unsigned loc;};struct Cyc_Absyn_Switch_clause{struct Cyc_Absyn_Pat*pattern;struct Cyc_Core_Opt*pat_vars;struct Cyc_Absyn_Exp*where_clause;struct Cyc_Absyn_Stmt*body;unsigned loc;};struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct{int tag;struct _tuple0*f1;};struct Cyc_Absyn_Global_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Funname_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Fndecl*f1;};struct Cyc_Absyn_Param_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Local_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Pat_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Vardecl{enum Cyc_Absyn_Scope sc;struct _tuple0*name;unsigned varloc;struct Cyc_Absyn_Tqual tq;void*type;struct Cyc_Absyn_Exp*initializer;void*rgn;struct Cyc_List_List*attributes;int escapes;};struct Cyc_Absyn_Fndecl{enum Cyc_Absyn_Scope sc;int is_inline;struct _tuple0*name;struct Cyc_Absyn_Stmt*body;struct Cyc_Absyn_FnInfo i;void*cached_type;struct Cyc_Core_Opt*param_vardecls;struct Cyc_Absyn_Vardecl*fn_vardecl;};struct Cyc_Absyn_Aggrfield{struct _fat_ptr*name;struct Cyc_Absyn_Tqual tq;void*type;struct Cyc_Absyn_Exp*width;struct Cyc_List_List*attributes;struct Cyc_Absyn_Exp*requires_clause;};struct Cyc_Absyn_AggrdeclImpl{struct Cyc_List_List*exist_vars;struct Cyc_List_List*rgn_po;struct Cyc_List_List*fields;int tagged;};struct Cyc_Absyn_Aggrdecl{enum Cyc_Absyn_AggrKind kind;enum Cyc_Absyn_Scope sc;struct _tuple0*name;struct Cyc_List_List*tvs;struct Cyc_Absyn_AggrdeclImpl*impl;struct Cyc_List_List*attributes;int expected_mem_kind;};struct Cyc_Absyn_Datatypefield{struct _tuple0*name;struct Cyc_List_List*typs;unsigned loc;enum Cyc_Absyn_Scope sc;};struct Cyc_Absyn_Datatypedecl{enum Cyc_Absyn_Scope sc;struct _tuple0*name;struct Cyc_List_List*tvs;struct Cyc_Core_Opt*fields;int is_extensible;};struct Cyc_Absyn_Enumfield{struct _tuple0*name;struct Cyc_Absyn_Exp*tag;unsigned loc;};struct Cyc_Absyn_Enumdecl{enum Cyc_Absyn_Scope sc;struct _tuple0*name;struct Cyc_Core_Opt*fields;};struct Cyc_Absyn_Typedefdecl{struct _tuple0*name;struct Cyc_Absyn_Tqual tq;struct Cyc_List_List*tvs;struct Cyc_Core_Opt*kind;void*defn;struct Cyc_List_List*atts;int extern_c;};struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Fn_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Fndecl*f1;};struct Cyc_Absyn_Let_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Pat*f1;struct Cyc_Core_Opt*f2;struct Cyc_Absyn_Exp*f3;void*f4;};struct Cyc_Absyn_Letv_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_Region_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_Aggr_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Aggrdecl*f1;};struct Cyc_Absyn_Datatype_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;};struct Cyc_Absyn_Enum_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;};struct Cyc_Absyn_Typedef_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Typedefdecl*f1;};struct Cyc_Absyn_Namespace_d_Absyn_Raw_decl_struct{int tag;struct _fat_ptr*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Using_d_Absyn_Raw_decl_struct{int tag;struct _tuple0*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_ExternC_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;};struct _tuple10{unsigned f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_ExternCinclude_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;struct _tuple10*f4;};struct Cyc_Absyn_Porton_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Portoff_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Tempeston_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Tempestoff_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Decl{void*r;unsigned loc;};struct Cyc_Absyn_ArrayElement_Absyn_Designator_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_FieldName_Absyn_Designator_struct{int tag;struct _fat_ptr*f1;};extern char Cyc_Absyn_EmptyAnnot[11U];struct Cyc_Absyn_EmptyAnnot_Absyn_AbsynAnnot_struct{char*tag;};
# 908 "absyn.h"
int Cyc_Absyn_qvar_cmp(struct _tuple0*,struct _tuple0*);
# 917
struct Cyc_Absyn_Tqual Cyc_Absyn_empty_tqual(unsigned);
# 966
extern struct _tuple0*Cyc_Absyn_exn_name;
struct Cyc_Absyn_Datatypedecl*Cyc_Absyn_exn_tud();
# 1107
struct Cyc_Absyn_Decl*Cyc_Absyn_new_decl(void*,unsigned);
# 1112
struct Cyc_Absyn_Vardecl*Cyc_Absyn_new_vardecl(unsigned varloc,struct _tuple0*,void*,struct Cyc_Absyn_Exp*init);
# 1176
struct Cyc_Absyn_Aggrdecl*Cyc_Absyn_get_known_aggrdecl(union Cyc_Absyn_AggrInfo);struct Cyc_RgnOrder_RgnPO;extern char Cyc_Tcenv_Env_error[10U];struct Cyc_Tcenv_Env_error_exn_struct{char*tag;};struct Cyc_Tcenv_Genv{struct Cyc_Dict_Dict aggrdecls;struct Cyc_Dict_Dict datatypedecls;struct Cyc_Dict_Dict enumdecls;struct Cyc_Dict_Dict typedefs;struct Cyc_Dict_Dict ordinaries;};struct Cyc_Tcenv_Fenv;struct Cyc_Tcenv_Tenv{struct Cyc_List_List*ns;struct Cyc_Tcenv_Genv*ae;struct Cyc_Tcenv_Fenv*le;int allow_valueof: 1;int in_extern_c_include: 1;int in_tempest: 1;int tempest_generalize: 1;};
# 68 "tcenv.h"
struct Cyc_Tcenv_Tenv*Cyc_Tcenv_tc_init();
# 88
enum Cyc_Tcenv_NewStatus{Cyc_Tcenv_NoneNew =0U,Cyc_Tcenv_InNew =1U,Cyc_Tcenv_InNewAggr =2U};struct Cyc_Interface_I;
# 36 "interface.h"
struct Cyc_Interface_I*Cyc_Interface_empty();struct _tuple11{struct _fat_ptr f1;struct _fat_ptr f2;};
# 66 "interface.h"
struct Cyc_Interface_I*Cyc_Interface_merge(struct Cyc_Interface_I*i1,struct Cyc_Interface_I*i2,struct _tuple11*info);
# 75
void Cyc_Interface_print(struct Cyc_Interface_I*,struct Cyc___cycFILE*);
# 78
struct Cyc_Interface_I*Cyc_Interface_parse(struct Cyc___cycFILE*);
# 50 "string.h"
int Cyc_strptrcmp(struct _fat_ptr*s1,struct _fat_ptr*s2);struct Cyc_PP_Ppstate;struct Cyc_PP_Out;struct Cyc_PP_Doc;struct Cyc_Absynpp_Params{int expand_typedefs;int qvar_to_Cids;int add_cyc_prefix;int to_VC;int decls_first;int rewrite_temp_tvars;int print_all_tvars;int print_all_kinds;int print_all_effects;int print_using_stmts;int print_externC_stmts;int print_full_evars;int print_zeroterm;int generate_line_directives;int use_curr_namespace;struct Cyc_List_List*curr_namespace;};
# 53 "absynpp.h"
void Cyc_Absynpp_set_params(struct Cyc_Absynpp_Params*fs);
# 55
extern struct Cyc_Absynpp_Params Cyc_Absynpp_cyci_params_r;
# 57
void Cyc_Absynpp_decllist2file(struct Cyc_List_List*tdl,struct Cyc___cycFILE*f);
# 69
struct _fat_ptr Cyc_Absynpp_qvar2string(struct _tuple0*);
# 33 "tc.h"
void Cyc_Tc_tc(struct Cyc_Tcenv_Tenv*te,int var_default_init,struct Cyc_List_List*ds);extern char Cyc_Lexing_Error[6U];struct Cyc_Lexing_Error_exn_struct{char*tag;struct _fat_ptr f1;};struct Cyc_Lexing_lexbuf{void(*refill_buff)(struct Cyc_Lexing_lexbuf*);void*refill_state;struct _fat_ptr lex_buffer;int lex_buffer_len;int lex_abs_pos;int lex_start_pos;int lex_curr_pos;int lex_last_pos;int lex_last_action;int lex_eof_reached;};struct Cyc_Lexing_function_lexbuf_state{int(*read_fun)(struct _fat_ptr,int,void*);void*read_fun_state;};struct Cyc_Lexing_lex_tables{struct _fat_ptr lex_base;struct _fat_ptr lex_backtrk;struct _fat_ptr lex_default;struct _fat_ptr lex_trans;struct _fat_ptr lex_check;};
# 28 "parse.h"
struct Cyc_List_List*Cyc_Parse_parse_file(struct Cyc___cycFILE*f,struct _fat_ptr name);extern char Cyc_Parse_Exit[5U];struct Cyc_Parse_Exit_exn_struct{char*tag;};struct Cyc_FlatList{struct Cyc_FlatList*tl;void*hd[0U] __attribute__((aligned )) ;};struct Cyc_Type_specifier{int Signed_spec: 1;int Unsigned_spec: 1;int Short_spec: 1;int Long_spec: 1;int Long_Long_spec: 1;int Valid_type_spec: 1;void*Type_spec;unsigned loc;};struct Cyc_Declarator{struct _tuple0*id;struct Cyc_List_List*tms;};struct _tuple13{struct Cyc_Declarator f1;struct Cyc_Absyn_Exp*f2;};struct _tuple12{struct _tuple12*tl;struct _tuple13 hd  __attribute__((aligned )) ;};
# 52
enum Cyc_Storage_class{Cyc_Typedef_sc =0U,Cyc_Extern_sc =1U,Cyc_ExternC_sc =2U,Cyc_Static_sc =3U,Cyc_Auto_sc =4U,Cyc_Register_sc =5U,Cyc_Abstract_sc =6U};struct Cyc_Declaration_spec{enum Cyc_Storage_class*sc;struct Cyc_Absyn_Tqual tq;struct Cyc_Type_specifier type_specs;int is_inline;struct Cyc_List_List*attributes;};struct Cyc_Abstractdeclarator{struct Cyc_List_List*tms;};struct Cyc_Numelts_ptrqual_Pointer_qual_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Region_ptrqual_Pointer_qual_struct{int tag;void*f1;};struct Cyc_Thin_ptrqual_Pointer_qual_struct{int tag;};struct Cyc_Fat_ptrqual_Pointer_qual_struct{int tag;};struct Cyc_Zeroterm_ptrqual_Pointer_qual_struct{int tag;};struct Cyc_Nozeroterm_ptrqual_Pointer_qual_struct{int tag;};struct Cyc_Notnull_ptrqual_Pointer_qual_struct{int tag;};struct Cyc_Nullable_ptrqual_Pointer_qual_struct{int tag;};struct _union_YYSTYPE_Int_tok{int tag;union Cyc_Absyn_Cnst val;};struct _union_YYSTYPE_Char_tok{int tag;char val;};struct _union_YYSTYPE_String_tok{int tag;struct _fat_ptr val;};struct _union_YYSTYPE_Stringopt_tok{int tag;struct Cyc_Core_Opt*val;};struct _union_YYSTYPE_QualId_tok{int tag;struct _tuple0*val;};struct _tuple14{int f1;struct _fat_ptr f2;};struct _union_YYSTYPE_Asm_tok{int tag;struct _tuple14 val;};struct _union_YYSTYPE_Exp_tok{int tag;struct Cyc_Absyn_Exp*val;};struct _union_YYSTYPE_Stmt_tok{int tag;struct Cyc_Absyn_Stmt*val;};struct _tuple15{unsigned f1;void*f2;void*f3;};struct _union_YYSTYPE_YY1{int tag;struct _tuple15*val;};struct _union_YYSTYPE_YY2{int tag;void*val;};struct _union_YYSTYPE_YY3{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY4{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY5{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY6{int tag;enum Cyc_Absyn_Primop val;};struct _union_YYSTYPE_YY7{int tag;struct Cyc_Core_Opt*val;};struct _union_YYSTYPE_YY8{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY9{int tag;struct Cyc_Absyn_Pat*val;};struct _tuple16{struct Cyc_List_List*f1;int f2;};struct _union_YYSTYPE_YY10{int tag;struct _tuple16*val;};struct _union_YYSTYPE_YY11{int tag;struct Cyc_List_List*val;};struct _tuple17{struct Cyc_List_List*f1;struct Cyc_Absyn_Pat*f2;};struct _union_YYSTYPE_YY12{int tag;struct _tuple17*val;};struct _union_YYSTYPE_YY13{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY14{int tag;struct _tuple16*val;};struct _union_YYSTYPE_YY15{int tag;struct Cyc_Absyn_Fndecl*val;};struct _union_YYSTYPE_YY16{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY17{int tag;struct Cyc_Declaration_spec val;};struct _union_YYSTYPE_YY18{int tag;struct _tuple13 val;};struct _union_YYSTYPE_YY19{int tag;struct _tuple12*val;};struct _union_YYSTYPE_YY20{int tag;enum Cyc_Storage_class*val;};struct _union_YYSTYPE_YY21{int tag;struct Cyc_Type_specifier val;};struct _union_YYSTYPE_YY22{int tag;enum Cyc_Absyn_AggrKind val;};struct _union_YYSTYPE_YY23{int tag;struct Cyc_Absyn_Tqual val;};struct _union_YYSTYPE_YY24{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY25{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY26{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY27{int tag;struct Cyc_Declarator val;};struct _tuple18{struct Cyc_Declarator f1;struct Cyc_Absyn_Exp*f2;struct Cyc_Absyn_Exp*f3;};struct _union_YYSTYPE_YY28{int tag;struct _tuple18*val;};struct _union_YYSTYPE_YY29{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY30{int tag;struct Cyc_Abstractdeclarator val;};struct _union_YYSTYPE_YY31{int tag;int val;};struct _union_YYSTYPE_YY32{int tag;enum Cyc_Absyn_Scope val;};struct _union_YYSTYPE_YY33{int tag;struct Cyc_Absyn_Datatypefield*val;};struct _union_YYSTYPE_YY34{int tag;struct Cyc_List_List*val;};struct _tuple19{struct Cyc_Absyn_Tqual f1;struct Cyc_Type_specifier f2;struct Cyc_List_List*f3;};struct _union_YYSTYPE_YY35{int tag;struct _tuple19 val;};struct _union_YYSTYPE_YY36{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY37{int tag;struct _tuple8*val;};struct _union_YYSTYPE_YY38{int tag;struct Cyc_List_List*val;};struct _tuple20{struct Cyc_List_List*f1;int f2;struct Cyc_Absyn_VarargInfo*f3;void*f4;struct Cyc_List_List*f5;};struct _union_YYSTYPE_YY39{int tag;struct _tuple20*val;};struct _union_YYSTYPE_YY40{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY41{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY42{int tag;void*val;};struct _union_YYSTYPE_YY43{int tag;struct Cyc_Absyn_Kind*val;};struct _union_YYSTYPE_YY44{int tag;void*val;};struct _union_YYSTYPE_YY45{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY46{int tag;void*val;};struct _union_YYSTYPE_YY47{int tag;struct Cyc_Absyn_Enumfield*val;};struct _union_YYSTYPE_YY48{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY49{int tag;void*val;};struct _union_YYSTYPE_YY50{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY51{int tag;void*val;};struct _union_YYSTYPE_YY52{int tag;struct Cyc_List_List*val;};struct _tuple21{struct Cyc_List_List*f1;unsigned f2;};struct _union_YYSTYPE_YY53{int tag;struct _tuple21*val;};struct _union_YYSTYPE_YY54{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY55{int tag;void*val;};struct _union_YYSTYPE_YY56{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY57{int tag;struct Cyc_Absyn_Exp*val;};struct _union_YYSTYPE_YY58{int tag;void*val;};struct _tuple22{struct Cyc_List_List*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;};struct _union_YYSTYPE_YY59{int tag;struct _tuple22*val;};struct _tuple23{struct Cyc_List_List*f1;struct Cyc_List_List*f2;};struct _union_YYSTYPE_YY60{int tag;struct _tuple23*val;};struct _union_YYSTYPE_YY61{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY62{int tag;struct Cyc_List_List*val;};struct _tuple24{struct _fat_ptr f1;struct Cyc_Absyn_Exp*f2;};struct _union_YYSTYPE_YY63{int tag;struct _tuple24*val;};struct _union_YYSTYPE_YY64{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YYINITIALSVAL{int tag;int val;};union Cyc_YYSTYPE{struct _union_YYSTYPE_Int_tok Int_tok;struct _union_YYSTYPE_Char_tok Char_tok;struct _union_YYSTYPE_String_tok String_tok;struct _union_YYSTYPE_Stringopt_tok Stringopt_tok;struct _union_YYSTYPE_QualId_tok QualId_tok;struct _union_YYSTYPE_Asm_tok Asm_tok;struct _union_YYSTYPE_Exp_tok Exp_tok;struct _union_YYSTYPE_Stmt_tok Stmt_tok;struct _union_YYSTYPE_YY1 YY1;struct _union_YYSTYPE_YY2 YY2;struct _union_YYSTYPE_YY3 YY3;struct _union_YYSTYPE_YY4 YY4;struct _union_YYSTYPE_YY5 YY5;struct _union_YYSTYPE_YY6 YY6;struct _union_YYSTYPE_YY7 YY7;struct _union_YYSTYPE_YY8 YY8;struct _union_YYSTYPE_YY9 YY9;struct _union_YYSTYPE_YY10 YY10;struct _union_YYSTYPE_YY11 YY11;struct _union_YYSTYPE_YY12 YY12;struct _union_YYSTYPE_YY13 YY13;struct _union_YYSTYPE_YY14 YY14;struct _union_YYSTYPE_YY15 YY15;struct _union_YYSTYPE_YY16 YY16;struct _union_YYSTYPE_YY17 YY17;struct _union_YYSTYPE_YY18 YY18;struct _union_YYSTYPE_YY19 YY19;struct _union_YYSTYPE_YY20 YY20;struct _union_YYSTYPE_YY21 YY21;struct _union_YYSTYPE_YY22 YY22;struct _union_YYSTYPE_YY23 YY23;struct _union_YYSTYPE_YY24 YY24;struct _union_YYSTYPE_YY25 YY25;struct _union_YYSTYPE_YY26 YY26;struct _union_YYSTYPE_YY27 YY27;struct _union_YYSTYPE_YY28 YY28;struct _union_YYSTYPE_YY29 YY29;struct _union_YYSTYPE_YY30 YY30;struct _union_YYSTYPE_YY31 YY31;struct _union_YYSTYPE_YY32 YY32;struct _union_YYSTYPE_YY33 YY33;struct _union_YYSTYPE_YY34 YY34;struct _union_YYSTYPE_YY35 YY35;struct _union_YYSTYPE_YY36 YY36;struct _union_YYSTYPE_YY37 YY37;struct _union_YYSTYPE_YY38 YY38;struct _union_YYSTYPE_YY39 YY39;struct _union_YYSTYPE_YY40 YY40;struct _union_YYSTYPE_YY41 YY41;struct _union_YYSTYPE_YY42 YY42;struct _union_YYSTYPE_YY43 YY43;struct _union_YYSTYPE_YY44 YY44;struct _union_YYSTYPE_YY45 YY45;struct _union_YYSTYPE_YY46 YY46;struct _union_YYSTYPE_YY47 YY47;struct _union_YYSTYPE_YY48 YY48;struct _union_YYSTYPE_YY49 YY49;struct _union_YYSTYPE_YY50 YY50;struct _union_YYSTYPE_YY51 YY51;struct _union_YYSTYPE_YY52 YY52;struct _union_YYSTYPE_YY53 YY53;struct _union_YYSTYPE_YY54 YY54;struct _union_YYSTYPE_YY55 YY55;struct _union_YYSTYPE_YY56 YY56;struct _union_YYSTYPE_YY57 YY57;struct _union_YYSTYPE_YY58 YY58;struct _union_YYSTYPE_YY59 YY59;struct _union_YYSTYPE_YY60 YY60;struct _union_YYSTYPE_YY61 YY61;struct _union_YYSTYPE_YY62 YY62;struct _union_YYSTYPE_YY63 YY63;struct _union_YYSTYPE_YY64 YY64;struct _union_YYSTYPE_YYINITIALSVAL YYINITIALSVAL;};struct Cyc_Yyltype{int timestamp;int first_line;int first_column;int last_line;int last_column;};struct _union_RelnOp_RConst{int tag;unsigned val;};struct _union_RelnOp_RVar{int tag;struct Cyc_Absyn_Vardecl*val;};struct _union_RelnOp_RNumelts{int tag;struct Cyc_Absyn_Vardecl*val;};struct _union_RelnOp_RType{int tag;void*val;};struct _union_RelnOp_RParam{int tag;unsigned val;};struct _union_RelnOp_RParamNumelts{int tag;unsigned val;};struct _union_RelnOp_RReturn{int tag;unsigned val;};union Cyc_Relations_RelnOp{struct _union_RelnOp_RConst RConst;struct _union_RelnOp_RVar RVar;struct _union_RelnOp_RNumelts RNumelts;struct _union_RelnOp_RType RType;struct _union_RelnOp_RParam RParam;struct _union_RelnOp_RParamNumelts RParamNumelts;struct _union_RelnOp_RReturn RReturn;};
# 50 "relations-ap.h"
enum Cyc_Relations_Relation{Cyc_Relations_Req =0U,Cyc_Relations_Rneq =1U,Cyc_Relations_Rlte =2U,Cyc_Relations_Rlt =3U};struct Cyc_Relations_Reln{union Cyc_Relations_RelnOp rop1;enum Cyc_Relations_Relation relation;union Cyc_Relations_RelnOp rop2;};
# 31 "tcutil.h"
void Cyc_Tcutil_terr(unsigned,struct _fat_ptr,struct _fat_ptr);
# 43
int Cyc_Tcutil_is_function_type(void*);
# 107
void*Cyc_Tcutil_compress(void*);
# 137
extern struct Cyc_Absyn_Kind Cyc_Tcutil_bk;
# 174
void*Cyc_Tcutil_kind_to_bound(struct Cyc_Absyn_Kind*k);extern char Cyc_Tcdecl_Incompatible[13U];struct Cyc_Tcdecl_Incompatible_exn_struct{char*tag;};struct Cyc_Tcdecl_Xdatatypefielddecl{struct Cyc_Absyn_Datatypedecl*base;struct Cyc_Absyn_Datatypefield*field;};
# 38 "tcdecl.h"
void Cyc_Tcdecl_merr(unsigned loc,struct _fat_ptr*msg1,struct _fat_ptr fmt,struct _fat_ptr ap);
# 54 "tcdecl.h"
struct Cyc_Absyn_Aggrdecl*Cyc_Tcdecl_merge_aggrdecl(struct Cyc_Absyn_Aggrdecl*,struct Cyc_Absyn_Aggrdecl*,unsigned,struct _fat_ptr*msg);
# 57
struct Cyc_Absyn_Datatypedecl*Cyc_Tcdecl_merge_datatypedecl(struct Cyc_Absyn_Datatypedecl*,struct Cyc_Absyn_Datatypedecl*,unsigned,struct _fat_ptr*msg);
# 59
struct Cyc_Absyn_Enumdecl*Cyc_Tcdecl_merge_enumdecl(struct Cyc_Absyn_Enumdecl*,struct Cyc_Absyn_Enumdecl*,unsigned,struct _fat_ptr*msg);
# 61
struct Cyc_Absyn_Vardecl*Cyc_Tcdecl_merge_vardecl(struct Cyc_Absyn_Vardecl*,struct Cyc_Absyn_Vardecl*,unsigned,struct _fat_ptr*msg);
# 63
struct Cyc_Absyn_Typedefdecl*Cyc_Tcdecl_merge_typedefdecl(struct Cyc_Absyn_Typedefdecl*,struct Cyc_Absyn_Typedefdecl*,unsigned,struct _fat_ptr*msg);
# 68
struct Cyc_Tcdecl_Xdatatypefielddecl*Cyc_Tcdecl_merge_xdatatypefielddecl(struct Cyc_Tcdecl_Xdatatypefielddecl*,struct Cyc_Tcdecl_Xdatatypefielddecl*,unsigned,struct _fat_ptr*msg);
# 30 "binding.h"
void Cyc_Binding_resolve_all(struct Cyc_List_List*tds);struct Cyc_Binding_Namespace_Binding_NSDirective_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Binding_Using_Binding_NSDirective_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Binding_NSCtxt{struct Cyc_List_List*curr_ns;struct Cyc_List_List*availables;struct Cyc_Dict_Dict ns_data;};
# 45 "interface.cyc"
void Cyc_Lex_lex_init();struct Cyc_Interface_Ienv{struct Cyc_Dict_Dict aggrdecls;struct Cyc_Dict_Dict datatypedecls;struct Cyc_Dict_Dict enumdecls;struct Cyc_Dict_Dict typedefdecls;struct Cyc_Dict_Dict vardecls;struct Cyc_Dict_Dict xdatatypefielddecls;};struct Cyc_Interface_I{struct Cyc_Interface_Ienv*imports;struct Cyc_Interface_Ienv*exports;struct Cyc_List_List*tds;};
# 73
static struct Cyc_Interface_Ienv*Cyc_Interface_new_ienv(){
return({struct Cyc_Interface_Ienv*_tmp6=_cycalloc(sizeof(*_tmp6));({
struct Cyc_Dict_Dict _tmp238=({struct Cyc_Dict_Dict(*_tmp0)(int(*cmp)(struct _tuple0*,struct _tuple0*))=(struct Cyc_Dict_Dict(*)(int(*cmp)(struct _tuple0*,struct _tuple0*)))Cyc_Dict_empty;_tmp0;})(Cyc_Absyn_qvar_cmp);_tmp6->aggrdecls=_tmp238;}),({
struct Cyc_Dict_Dict _tmp237=({struct Cyc_Dict_Dict(*_tmp1)(int(*cmp)(struct _tuple0*,struct _tuple0*))=(struct Cyc_Dict_Dict(*)(int(*cmp)(struct _tuple0*,struct _tuple0*)))Cyc_Dict_empty;_tmp1;})(Cyc_Absyn_qvar_cmp);_tmp6->datatypedecls=_tmp237;}),({
struct Cyc_Dict_Dict _tmp236=({struct Cyc_Dict_Dict(*_tmp2)(int(*cmp)(struct _tuple0*,struct _tuple0*))=(struct Cyc_Dict_Dict(*)(int(*cmp)(struct _tuple0*,struct _tuple0*)))Cyc_Dict_empty;_tmp2;})(Cyc_Absyn_qvar_cmp);_tmp6->enumdecls=_tmp236;}),({
struct Cyc_Dict_Dict _tmp235=({struct Cyc_Dict_Dict(*_tmp3)(int(*cmp)(struct _tuple0*,struct _tuple0*))=(struct Cyc_Dict_Dict(*)(int(*cmp)(struct _tuple0*,struct _tuple0*)))Cyc_Dict_empty;_tmp3;})(Cyc_Absyn_qvar_cmp);_tmp6->typedefdecls=_tmp235;}),({
struct Cyc_Dict_Dict _tmp234=({struct Cyc_Dict_Dict(*_tmp4)(int(*cmp)(struct _tuple0*,struct _tuple0*))=(struct Cyc_Dict_Dict(*)(int(*cmp)(struct _tuple0*,struct _tuple0*)))Cyc_Dict_empty;_tmp4;})(Cyc_Absyn_qvar_cmp);_tmp6->vardecls=_tmp234;}),({
struct Cyc_Dict_Dict _tmp233=({struct Cyc_Dict_Dict(*_tmp5)(int(*cmp)(struct _tuple0*,struct _tuple0*))=(struct Cyc_Dict_Dict(*)(int(*cmp)(struct _tuple0*,struct _tuple0*)))Cyc_Dict_empty;_tmp5;})(Cyc_Absyn_qvar_cmp);_tmp6->xdatatypefielddecls=_tmp233;});_tmp6;});}
# 73
struct Cyc_Interface_I*Cyc_Interface_empty(){
# 84
return({struct Cyc_Interface_I*_tmp7=_cycalloc(sizeof(*_tmp7));({struct Cyc_Interface_Ienv*_tmp23A=Cyc_Interface_new_ienv();_tmp7->imports=_tmp23A;}),({struct Cyc_Interface_Ienv*_tmp239=Cyc_Interface_new_ienv();_tmp7->exports=_tmp239;}),_tmp7->tds=0;_tmp7;});}
# 73
struct Cyc_Interface_I*Cyc_Interface_final(){
# 89
struct Cyc_Interface_I*i=Cyc_Interface_empty();
# 91
struct Cyc_Absyn_Datatypedecl*exn_d=({struct Cyc_Absyn_Datatypedecl*_tmpC=_cycalloc(sizeof(*_tmpC));({struct Cyc_Absyn_Datatypedecl _tmp23B=*Cyc_Absyn_exn_tud();*_tmpC=_tmp23B;});_tmpC;});
exn_d->sc=Cyc_Absyn_Public;
({struct Cyc_Dict_Dict _tmp23F=({struct Cyc_Dict_Dict(*_tmp23E)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Datatypedecl*v)=({struct Cyc_Dict_Dict(*_tmp8)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Datatypedecl*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Datatypedecl*v))Cyc_Dict_insert;_tmp8;});struct Cyc_Dict_Dict _tmp23D=(i->exports)->datatypedecls;struct _tuple0*_tmp23C=Cyc_Absyn_exn_name;_tmp23E(_tmp23D,_tmp23C,exn_d);});(i->exports)->datatypedecls=_tmp23F;});
# 95
{struct Cyc_List_List*tufs=(struct Cyc_List_List*)((struct Cyc_Core_Opt*)_check_null((Cyc_Absyn_exn_tud())->fields))->v;for(0;tufs != 0;tufs=tufs->tl){
struct Cyc_Absyn_Datatypefield*exn_f=({struct Cyc_Absyn_Datatypefield*_tmpB=_cycalloc(sizeof(*_tmpB));*_tmpB=*((struct Cyc_Absyn_Datatypefield*)tufs->hd);_tmpB;});
exn_f->sc=Cyc_Absyn_Public;{
struct Cyc_Tcdecl_Xdatatypefielddecl*exn_fd=({struct Cyc_Tcdecl_Xdatatypefielddecl*_tmpA=_cycalloc(sizeof(*_tmpA));_tmpA->base=exn_d,_tmpA->field=exn_f;_tmpA;});
({struct Cyc_Dict_Dict _tmp243=({struct Cyc_Dict_Dict(*_tmp242)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Tcdecl_Xdatatypefielddecl*v)=({struct Cyc_Dict_Dict(*_tmp9)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Tcdecl_Xdatatypefielddecl*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Tcdecl_Xdatatypefielddecl*v))Cyc_Dict_insert;_tmp9;});struct Cyc_Dict_Dict _tmp241=(i->exports)->xdatatypefielddecls;struct _tuple0*_tmp240=((struct Cyc_Absyn_Datatypefield*)tufs->hd)->name;_tmp242(_tmp241,_tmp240,exn_fd);});(i->exports)->xdatatypefielddecls=_tmp243;});}}}
# 120 "interface.cyc"
i->imports=i->exports;
return i;}
# 73 "interface.cyc"
static void Cyc_Interface_err(struct _fat_ptr msg){
# 141 "interface.cyc"
({void*_tmpD=0U;({struct _fat_ptr _tmp244=msg;Cyc_Tcutil_terr(0U,_tmp244,_tag_fat(_tmpD,sizeof(void*),0U));});});}
# 73 "interface.cyc"
static void*Cyc_Interface_invalid_arg(struct _fat_ptr s){
# 144 "interface.cyc"
(int)_throw((void*)({struct Cyc_Core_Invalid_argument_exn_struct*_tmpE=_cycalloc(sizeof(*_tmpE));_tmpE->tag=Cyc_Core_Invalid_argument,_tmpE->f1=s;_tmpE;}));}
# 73 "interface.cyc"
static void Cyc_Interface_fields_err(struct _fat_ptr sc,struct _fat_ptr t,struct _tuple0*n){
# 147 "interface.cyc"
Cyc_Interface_err((struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp11=({struct Cyc_String_pa_PrintArg_struct _tmp202;_tmp202.tag=0U,_tmp202.f1=(struct _fat_ptr)((struct _fat_ptr)sc);_tmp202;});struct Cyc_String_pa_PrintArg_struct _tmp12=({struct Cyc_String_pa_PrintArg_struct _tmp201;_tmp201.tag=0U,_tmp201.f1=(struct _fat_ptr)((struct _fat_ptr)t);_tmp201;});struct Cyc_String_pa_PrintArg_struct _tmp13=({struct Cyc_String_pa_PrintArg_struct _tmp200;_tmp200.tag=0U,({
struct _fat_ptr _tmp245=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(n));_tmp200.f1=_tmp245;});_tmp200;});void*_tmpF[3U];_tmpF[0]=& _tmp11,_tmpF[1]=& _tmp12,_tmpF[2]=& _tmp13;({struct _fat_ptr _tmp246=({const char*_tmp10="fields of %s %s %s have never been defined";_tag_fat(_tmp10,sizeof(char),43U);});Cyc_aprintf(_tmp246,_tag_fat(_tmpF,sizeof(void*),3U));});}));}
# 73 "interface.cyc"
static void Cyc_Interface_body_err(struct _fat_ptr sc,struct _tuple0*n){
# 151 "interface.cyc"
Cyc_Interface_err((struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp16=({struct Cyc_String_pa_PrintArg_struct _tmp204;_tmp204.tag=0U,_tmp204.f1=(struct _fat_ptr)((struct _fat_ptr)sc);_tmp204;});struct Cyc_String_pa_PrintArg_struct _tmp17=({struct Cyc_String_pa_PrintArg_struct _tmp203;_tmp203.tag=0U,({
struct _fat_ptr _tmp247=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(n));_tmp203.f1=_tmp247;});_tmp203;});void*_tmp14[2U];_tmp14[0]=& _tmp16,_tmp14[1]=& _tmp17;({struct _fat_ptr _tmp248=({const char*_tmp15="the body of %s function %s has never been defined";_tag_fat(_tmp15,sizeof(char),50U);});Cyc_aprintf(_tmp248,_tag_fat(_tmp14,sizeof(void*),2U));});}));}
# 73 "interface.cyc"
static void Cyc_Interface_static_err(struct _fat_ptr obj1,struct _tuple0*name1,struct _fat_ptr obj2,struct _tuple0*name2){
# 157 "interface.cyc"
if(({char*_tmp249=(char*)obj1.curr;_tmp249 != (char*)(_tag_fat(0,0,0)).curr;}))
Cyc_Interface_err((struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp1A=({struct Cyc_String_pa_PrintArg_struct _tmp208;_tmp208.tag=0U,_tmp208.f1=(struct _fat_ptr)((struct _fat_ptr)obj1);_tmp208;});struct Cyc_String_pa_PrintArg_struct _tmp1B=({struct Cyc_String_pa_PrintArg_struct _tmp207;_tmp207.tag=0U,({
struct _fat_ptr _tmp24A=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(name1));_tmp207.f1=_tmp24A;});_tmp207;});struct Cyc_String_pa_PrintArg_struct _tmp1C=({struct Cyc_String_pa_PrintArg_struct _tmp206;_tmp206.tag=0U,_tmp206.f1=(struct _fat_ptr)((struct _fat_ptr)obj2);_tmp206;});struct Cyc_String_pa_PrintArg_struct _tmp1D=({struct Cyc_String_pa_PrintArg_struct _tmp205;_tmp205.tag=0U,({
struct _fat_ptr _tmp24B=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(name2));_tmp205.f1=_tmp24B;});_tmp205;});void*_tmp18[4U];_tmp18[0]=& _tmp1A,_tmp18[1]=& _tmp1B,_tmp18[2]=& _tmp1C,_tmp18[3]=& _tmp1D;({struct _fat_ptr _tmp24C=({const char*_tmp19="declaration of %s %s relies on static %s %s";_tag_fat(_tmp19,sizeof(char),44U);});Cyc_aprintf(_tmp24C,_tag_fat(_tmp18,sizeof(void*),4U));});}));}
# 73 "interface.cyc"
static void Cyc_Interface_abstract_err(struct _fat_ptr obj1,struct _tuple0*name1,struct _fat_ptr obj2,struct _tuple0*name2){
# 165 "interface.cyc"
if(({char*_tmp24D=(char*)obj1.curr;_tmp24D != (char*)(_tag_fat(0,0,0)).curr;}))
Cyc_Interface_err((struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp20=({struct Cyc_String_pa_PrintArg_struct _tmp20C;_tmp20C.tag=0U,_tmp20C.f1=(struct _fat_ptr)((struct _fat_ptr)obj1);_tmp20C;});struct Cyc_String_pa_PrintArg_struct _tmp21=({struct Cyc_String_pa_PrintArg_struct _tmp20B;_tmp20B.tag=0U,({
struct _fat_ptr _tmp24E=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(name1));_tmp20B.f1=_tmp24E;});_tmp20B;});struct Cyc_String_pa_PrintArg_struct _tmp22=({struct Cyc_String_pa_PrintArg_struct _tmp20A;_tmp20A.tag=0U,_tmp20A.f1=(struct _fat_ptr)((struct _fat_ptr)obj2);_tmp20A;});struct Cyc_String_pa_PrintArg_struct _tmp23=({struct Cyc_String_pa_PrintArg_struct _tmp209;_tmp209.tag=0U,({
struct _fat_ptr _tmp24F=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(name2));_tmp209.f1=_tmp24F;});_tmp209;});void*_tmp1E[4U];_tmp1E[0]=& _tmp20,_tmp1E[1]=& _tmp21,_tmp1E[2]=& _tmp22,_tmp1E[3]=& _tmp23;({struct _fat_ptr _tmp250=({const char*_tmp1F="declaration of %s %s relies on fields of abstract %s %s";_tag_fat(_tmp1F,sizeof(char),56U);});Cyc_aprintf(_tmp250,_tag_fat(_tmp1E,sizeof(void*),4U));});}));}struct Cyc_Interface_Seen{struct Cyc_Dict_Dict aggrs;struct Cyc_Dict_Dict datatypes;};
# 181
static struct Cyc_Interface_Seen*Cyc_Interface_new_seen(){
return({struct Cyc_Interface_Seen*_tmp26=_cycalloc(sizeof(*_tmp26));({struct Cyc_Dict_Dict _tmp252=({struct Cyc_Dict_Dict(*_tmp24)(int(*cmp)(struct _tuple0*,struct _tuple0*))=(struct Cyc_Dict_Dict(*)(int(*cmp)(struct _tuple0*,struct _tuple0*)))Cyc_Dict_empty;_tmp24;})(Cyc_Absyn_qvar_cmp);_tmp26->aggrs=_tmp252;}),({
struct Cyc_Dict_Dict _tmp251=({struct Cyc_Dict_Dict(*_tmp25)(int(*cmp)(struct _tuple0*,struct _tuple0*))=(struct Cyc_Dict_Dict(*)(int(*cmp)(struct _tuple0*,struct _tuple0*)))Cyc_Dict_empty;_tmp25;})(Cyc_Absyn_qvar_cmp);_tmp26->datatypes=_tmp251;});_tmp26;});}
# 181
static int Cyc_Interface_check_public_type(struct Cyc_Tcenv_Genv*ae,struct Cyc_Interface_Seen*seen,struct _fat_ptr obj,struct _tuple0*name,void*t);
# 190
static int Cyc_Interface_check_public_type_list(struct Cyc_Tcenv_Genv*ae,struct Cyc_Interface_Seen*seen,struct _fat_ptr obj,struct _tuple0*name,void*(*f)(void*),struct Cyc_List_List*l){
# 192
int res=1;
for(0;l != 0;l=l->tl){
if(!({struct Cyc_Tcenv_Genv*_tmp256=ae;struct Cyc_Interface_Seen*_tmp255=seen;struct _fat_ptr _tmp254=obj;struct _tuple0*_tmp253=name;Cyc_Interface_check_public_type(_tmp256,_tmp255,_tmp254,_tmp253,f(l->hd));}))
res=0;}
# 193
return res;}
# 190
static int Cyc_Interface_check_public_aggrdecl(struct Cyc_Tcenv_Genv*ae,struct Cyc_Interface_Seen*seen,struct Cyc_Absyn_Aggrdecl*d){
# 200
{struct _handler_cons _tmp27;_push_handler(& _tmp27);{int _tmp29=0;if(setjmp(_tmp27.handler))_tmp29=1;if(!_tmp29){{int _tmp2B=({int(*_tmp258)(struct Cyc_Dict_Dict d,struct _tuple0*k)=({int(*_tmp2A)(struct Cyc_Dict_Dict d,struct _tuple0*k)=(int(*)(struct Cyc_Dict_Dict d,struct _tuple0*k))Cyc_Dict_lookup;_tmp2A;});struct Cyc_Dict_Dict _tmp257=seen->aggrs;_tmp258(_tmp257,d->name);});_npop_handler(0U);return _tmp2B;};_pop_handler();}else{void*_tmp28=(void*)Cyc_Core_get_exn_thrown();void*_tmp2C=_tmp28;void*_tmp2D;if(((struct Cyc_Dict_Absent_exn_struct*)_tmp2C)->tag == Cyc_Dict_Absent){_LL1: _LL2:
 goto _LL0;}else{_LL3: _tmp2D=_tmp2C;_LL4: {void*exn=_tmp2D;(int)_rethrow(exn);}}_LL0:;}}}{
int res=1;
({struct Cyc_Dict_Dict _tmp25C=({struct Cyc_Dict_Dict(*_tmp25B)(struct Cyc_Dict_Dict d,struct _tuple0*k,int v)=({struct Cyc_Dict_Dict(*_tmp2E)(struct Cyc_Dict_Dict d,struct _tuple0*k,int v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _tuple0*k,int v))Cyc_Dict_insert;_tmp2E;});struct Cyc_Dict_Dict _tmp25A=seen->aggrs;struct _tuple0*_tmp259=d->name;_tmp25B(_tmp25A,_tmp259,res);});seen->aggrs=_tmp25C;});
if(d->impl != 0){
struct Cyc_List_List*fs=((struct Cyc_Absyn_AggrdeclImpl*)_check_null(d->impl))->fields;for(0;fs != 0;fs=fs->tl){
if(!({struct Cyc_Tcenv_Genv*_tmp260=ae;struct Cyc_Interface_Seen*_tmp25F=seen;struct _fat_ptr _tmp25E=({const char*_tmp2F="type";_tag_fat(_tmp2F,sizeof(char),5U);});struct _tuple0*_tmp25D=d->name;Cyc_Interface_check_public_type(_tmp260,_tmp25F,_tmp25E,_tmp25D,((struct Cyc_Absyn_Aggrfield*)fs->hd)->type);}))
res=0;}}
# 204
({struct Cyc_Dict_Dict _tmp264=({
# 209
struct Cyc_Dict_Dict(*_tmp263)(struct Cyc_Dict_Dict d,struct _tuple0*k,int v)=({struct Cyc_Dict_Dict(*_tmp30)(struct Cyc_Dict_Dict d,struct _tuple0*k,int v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _tuple0*k,int v))Cyc_Dict_insert;_tmp30;});struct Cyc_Dict_Dict _tmp262=seen->aggrs;struct _tuple0*_tmp261=d->name;_tmp263(_tmp262,_tmp261,res);});
# 204
seen->aggrs=_tmp264;});
# 210
return res;}}struct _tuple25{struct Cyc_Absyn_Tqual f1;void*f2;};
# 190
static void*Cyc_Interface_get_type1(struct _tuple25*x){
# 214
return(*x).f2;}
# 190
static void*Cyc_Interface_get_type2(struct _tuple8*x){
# 217
return(*x).f3;}
# 190
static int Cyc_Interface_check_public_datatypedecl(struct Cyc_Tcenv_Genv*ae,struct Cyc_Interface_Seen*seen,struct Cyc_Absyn_Datatypedecl*d){
# 221
{struct _handler_cons _tmp31;_push_handler(& _tmp31);{int _tmp33=0;if(setjmp(_tmp31.handler))_tmp33=1;if(!_tmp33){{int _tmp35=({int(*_tmp266)(struct Cyc_Dict_Dict d,struct _tuple0*k)=({int(*_tmp34)(struct Cyc_Dict_Dict d,struct _tuple0*k)=(int(*)(struct Cyc_Dict_Dict d,struct _tuple0*k))Cyc_Dict_lookup;_tmp34;});struct Cyc_Dict_Dict _tmp265=seen->datatypes;_tmp266(_tmp265,d->name);});_npop_handler(0U);return _tmp35;};_pop_handler();}else{void*_tmp32=(void*)Cyc_Core_get_exn_thrown();void*_tmp36=_tmp32;void*_tmp37;if(((struct Cyc_Dict_Absent_exn_struct*)_tmp36)->tag == Cyc_Dict_Absent){_LL1: _LL2:
 goto _LL0;}else{_LL3: _tmp37=_tmp36;_LL4: {void*exn=_tmp37;(int)_rethrow(exn);}}_LL0:;}}}{
int res=1;
({struct Cyc_Dict_Dict _tmp26A=({struct Cyc_Dict_Dict(*_tmp269)(struct Cyc_Dict_Dict d,struct _tuple0*k,int v)=({struct Cyc_Dict_Dict(*_tmp38)(struct Cyc_Dict_Dict d,struct _tuple0*k,int v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _tuple0*k,int v))Cyc_Dict_insert;_tmp38;});struct Cyc_Dict_Dict _tmp268=seen->datatypes;struct _tuple0*_tmp267=d->name;_tmp269(_tmp268,_tmp267,res);});seen->datatypes=_tmp26A;});
if(d->fields != 0){
struct Cyc_List_List*fs=(struct Cyc_List_List*)((struct Cyc_Core_Opt*)_check_null(d->fields))->v;for(0;fs != 0;fs=fs->tl){
if(!({int(*_tmp26F)(struct Cyc_Tcenv_Genv*ae,struct Cyc_Interface_Seen*seen,struct _fat_ptr obj,struct _tuple0*name,void*(*f)(struct _tuple25*),struct Cyc_List_List*l)=({int(*_tmp39)(struct Cyc_Tcenv_Genv*ae,struct Cyc_Interface_Seen*seen,struct _fat_ptr obj,struct _tuple0*name,void*(*f)(struct _tuple25*),struct Cyc_List_List*l)=(int(*)(struct Cyc_Tcenv_Genv*ae,struct Cyc_Interface_Seen*seen,struct _fat_ptr obj,struct _tuple0*name,void*(*f)(struct _tuple25*),struct Cyc_List_List*l))Cyc_Interface_check_public_type_list;_tmp39;});struct Cyc_Tcenv_Genv*_tmp26E=ae;struct Cyc_Interface_Seen*_tmp26D=seen;struct _fat_ptr _tmp26C=({const char*_tmp3A="datatype";_tag_fat(_tmp3A,sizeof(char),9U);});struct _tuple0*_tmp26B=d->name;_tmp26F(_tmp26E,_tmp26D,_tmp26C,_tmp26B,Cyc_Interface_get_type1,((struct Cyc_Absyn_Datatypefield*)fs->hd)->typs);}))
# 229
res=0;}}
# 225
({struct Cyc_Dict_Dict _tmp273=({
# 231
struct Cyc_Dict_Dict(*_tmp272)(struct Cyc_Dict_Dict d,struct _tuple0*k,int v)=({struct Cyc_Dict_Dict(*_tmp3B)(struct Cyc_Dict_Dict d,struct _tuple0*k,int v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _tuple0*k,int v))Cyc_Dict_insert;_tmp3B;});struct Cyc_Dict_Dict _tmp271=seen->datatypes;struct _tuple0*_tmp270=d->name;_tmp272(_tmp271,_tmp270,res);});
# 225
seen->datatypes=_tmp273;});
# 232
return res;}}
# 190
static int Cyc_Interface_check_public_enumdecl(struct Cyc_Tcenv_Genv*ae,struct Cyc_Interface_Seen*seen,struct Cyc_Absyn_Enumdecl*d){
# 236
return 1;}
# 190
static int Cyc_Interface_check_public_typedefdecl(struct Cyc_Tcenv_Genv*ae,struct Cyc_Interface_Seen*seen,struct Cyc_Absyn_Typedefdecl*d){
# 240
if(d->defn != 0)
return({struct Cyc_Tcenv_Genv*_tmp277=ae;struct Cyc_Interface_Seen*_tmp276=seen;struct _fat_ptr _tmp275=_tag_fat(0,0,0);struct _tuple0*_tmp274=d->name;Cyc_Interface_check_public_type(_tmp277,_tmp276,_tmp275,_tmp274,(void*)_check_null(d->defn));});else{
return 1;}}
# 190
static int Cyc_Interface_check_public_vardecl(struct Cyc_Tcenv_Genv*ae,struct Cyc_Interface_Seen*seen,struct Cyc_Absyn_Vardecl*d){
# 247
return({struct Cyc_Tcenv_Genv*_tmp27B=ae;struct Cyc_Interface_Seen*_tmp27A=seen;struct _fat_ptr _tmp279=({const char*_tmp3C="variable";_tag_fat(_tmp3C,sizeof(char),9U);});struct _tuple0*_tmp278=d->name;Cyc_Interface_check_public_type(_tmp27B,_tmp27A,_tmp279,_tmp278,d->type);});}
# 190
static int Cyc_Interface_check_public_type(struct Cyc_Tcenv_Genv*ae,struct Cyc_Interface_Seen*seen,struct _fat_ptr obj,struct _tuple0*name,void*t){
# 251
void*_stmttmp0=Cyc_Tcutil_compress(t);void*_tmp3D=_stmttmp0;struct Cyc_List_List*_tmp40;struct Cyc_Absyn_Datatypefield*_tmp3F;struct Cyc_Absyn_Datatypedecl*_tmp3E;struct Cyc_List_List*_tmp42;struct Cyc_Absyn_Datatypedecl*_tmp41;struct _tuple0*_tmp43;struct Cyc_List_List*_tmp45;union Cyc_Absyn_AggrInfo _tmp44;struct Cyc_List_List*_tmp46;struct Cyc_Absyn_VarargInfo*_tmp49;struct Cyc_List_List*_tmp48;void*_tmp47;void*_tmp4A;void*_tmp4B;void*_tmp4C;switch(*((int*)_tmp3D)){case 3U: _LL1: _tmp4C=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp3D)->f1).elt_type;_LL2: {void*t=_tmp4C;
_tmp4B=t;goto _LL4;}case 4U: _LL3: _tmp4B=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp3D)->f1).elt_type;_LL4: {void*t=_tmp4B;
_tmp4A=t;goto _LL6;}case 8U: _LL5: _tmp4A=(void*)((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmp3D)->f4;if(_tmp4A != 0){_LL6: {void*t=_tmp4A;
# 255
return Cyc_Interface_check_public_type(ae,seen,obj,name,t);}}else{goto _LL13;}case 5U: _LL7: _tmp47=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp3D)->f1).ret_type;_tmp48=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp3D)->f1).args;_tmp49=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp3D)->f1).cyc_varargs;_LL8: {void*ret=_tmp47;struct Cyc_List_List*args=_tmp48;struct Cyc_Absyn_VarargInfo*cyc_varargs=_tmp49;
# 259
int b=({int(*_tmp280)(struct Cyc_Tcenv_Genv*ae,struct Cyc_Interface_Seen*seen,struct _fat_ptr obj,struct _tuple0*name,void*(*f)(struct _tuple8*),struct Cyc_List_List*l)=({int(*_tmp4E)(struct Cyc_Tcenv_Genv*ae,struct Cyc_Interface_Seen*seen,struct _fat_ptr obj,struct _tuple0*name,void*(*f)(struct _tuple8*),struct Cyc_List_List*l)=(int(*)(struct Cyc_Tcenv_Genv*ae,struct Cyc_Interface_Seen*seen,struct _fat_ptr obj,struct _tuple0*name,void*(*f)(struct _tuple8*),struct Cyc_List_List*l))Cyc_Interface_check_public_type_list;_tmp4E;});struct Cyc_Tcenv_Genv*_tmp27F=ae;struct Cyc_Interface_Seen*_tmp27E=seen;struct _fat_ptr _tmp27D=obj;struct _tuple0*_tmp27C=name;_tmp280(_tmp27F,_tmp27E,_tmp27D,_tmp27C,Cyc_Interface_get_type2,args);})&&
 Cyc_Interface_check_public_type(ae,seen,obj,name,ret);
if(cyc_varargs != 0){
struct Cyc_Absyn_VarargInfo _stmttmp1=*cyc_varargs;void*_tmp4D;_LL16: _tmp4D=_stmttmp1.type;_LL17: {void*vt=_tmp4D;
b=Cyc_Interface_check_public_type(ae,seen,obj,name,vt);}}
# 261
return b;}case 6U: _LL9: _tmp46=((struct Cyc_Absyn_TupleType_Absyn_Type_struct*)_tmp3D)->f1;_LLA: {struct Cyc_List_List*lt=_tmp46;
# 268
return({int(*_tmp285)(struct Cyc_Tcenv_Genv*ae,struct Cyc_Interface_Seen*seen,struct _fat_ptr obj,struct _tuple0*name,void*(*f)(struct _tuple25*),struct Cyc_List_List*l)=({int(*_tmp4F)(struct Cyc_Tcenv_Genv*ae,struct Cyc_Interface_Seen*seen,struct _fat_ptr obj,struct _tuple0*name,void*(*f)(struct _tuple25*),struct Cyc_List_List*l)=(int(*)(struct Cyc_Tcenv_Genv*ae,struct Cyc_Interface_Seen*seen,struct _fat_ptr obj,struct _tuple0*name,void*(*f)(struct _tuple25*),struct Cyc_List_List*l))Cyc_Interface_check_public_type_list;_tmp4F;});struct Cyc_Tcenv_Genv*_tmp284=ae;struct Cyc_Interface_Seen*_tmp283=seen;struct _fat_ptr _tmp282=obj;struct _tuple0*_tmp281=name;_tmp285(_tmp284,_tmp283,_tmp282,_tmp281,Cyc_Interface_get_type1,lt);});}case 0U: switch(*((int*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp3D)->f1)){case 20U: _LLB: _tmp44=((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp3D)->f1)->f1;_tmp45=((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp3D)->f2;_LLC: {union Cyc_Absyn_AggrInfo info=_tmp44;struct Cyc_List_List*targs=_tmp45;
# 271
struct Cyc_Absyn_Aggrdecl*ad=Cyc_Absyn_get_known_aggrdecl(info);
if((int)ad->sc == (int)Cyc_Absyn_Static){
({struct _fat_ptr _tmp288=obj;struct _tuple0*_tmp287=name;struct _fat_ptr _tmp286=({const char*_tmp50="type";_tag_fat(_tmp50,sizeof(char),5U);});Cyc_Interface_static_err(_tmp288,_tmp287,_tmp286,ad->name);});
return 0;}
# 272
return
# 276
 Cyc_Interface_check_public_type_list(ae,seen,obj,name,Cyc_Core_identity,targs)&&
 Cyc_Interface_check_public_aggrdecl(ae,seen,ad);}case 15U: _LLD: _tmp43=((struct Cyc_Absyn_EnumCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp3D)->f1)->f1;_LLE: {struct _tuple0*name=_tmp43;
# 280
struct Cyc_Absyn_Enumdecl*ed;
{struct _handler_cons _tmp51;_push_handler(& _tmp51);{int _tmp53=0;if(setjmp(_tmp51.handler))_tmp53=1;if(!_tmp53){ed=*({struct Cyc_Absyn_Enumdecl**(*_tmp28A)(struct Cyc_Dict_Dict d,struct _tuple0*k)=({struct Cyc_Absyn_Enumdecl**(*_tmp54)(struct Cyc_Dict_Dict d,struct _tuple0*k)=(struct Cyc_Absyn_Enumdecl**(*)(struct Cyc_Dict_Dict d,struct _tuple0*k))Cyc_Dict_lookup;_tmp54;});struct Cyc_Dict_Dict _tmp289=ae->enumdecls;_tmp28A(_tmp289,name);});;_pop_handler();}else{void*_tmp52=(void*)Cyc_Core_get_exn_thrown();void*_tmp55=_tmp52;void*_tmp56;if(((struct Cyc_Dict_Absent_exn_struct*)_tmp55)->tag == Cyc_Dict_Absent){_LL19: _LL1A:
# 283
({int(*_tmp28D)(struct _fat_ptr s)=({int(*_tmp57)(struct _fat_ptr s)=(int(*)(struct _fat_ptr s))Cyc_Interface_invalid_arg;_tmp57;});_tmp28D((struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp5A=({struct Cyc_String_pa_PrintArg_struct _tmp20D;_tmp20D.tag=0U,({
struct _fat_ptr _tmp28B=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(name));_tmp20D.f1=_tmp28B;});_tmp20D;});void*_tmp58[1U];_tmp58[0]=& _tmp5A;({struct _fat_ptr _tmp28C=({const char*_tmp59="check_public_type (can't find enum %s)";_tag_fat(_tmp59,sizeof(char),39U);});Cyc_aprintf(_tmp28C,_tag_fat(_tmp58,sizeof(void*),1U));});}));});}else{_LL1B: _tmp56=_tmp55;_LL1C: {void*exn=_tmp56;(int)_rethrow(exn);}}_LL18:;}}}
# 286
if((int)ed->sc == (int)Cyc_Absyn_Static){
({struct _fat_ptr _tmp290=obj;struct _tuple0*_tmp28F=name;struct _fat_ptr _tmp28E=({const char*_tmp5B="enum";_tag_fat(_tmp5B,sizeof(char),5U);});Cyc_Interface_static_err(_tmp290,_tmp28F,_tmp28E,ed->name);});
return 0;}
# 286
return 1;}case 18U: if(((((struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp3D)->f1)->f1).KnownDatatype).tag == 2){_LLF: _tmp41=*((((struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp3D)->f1)->f1).KnownDatatype).val;_tmp42=((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp3D)->f2;_LL10: {struct Cyc_Absyn_Datatypedecl*tud0=_tmp41;struct Cyc_List_List*targs=_tmp42;
# 293
struct Cyc_Absyn_Datatypedecl*tud;
{struct _handler_cons _tmp5C;_push_handler(& _tmp5C);{int _tmp5E=0;if(setjmp(_tmp5C.handler))_tmp5E=1;if(!_tmp5E){tud=*({struct Cyc_Absyn_Datatypedecl**(*_tmp292)(struct Cyc_Dict_Dict d,struct _tuple0*k)=({struct Cyc_Absyn_Datatypedecl**(*_tmp5F)(struct Cyc_Dict_Dict d,struct _tuple0*k)=(struct Cyc_Absyn_Datatypedecl**(*)(struct Cyc_Dict_Dict d,struct _tuple0*k))Cyc_Dict_lookup;_tmp5F;});struct Cyc_Dict_Dict _tmp291=ae->datatypedecls;_tmp292(_tmp291,tud0->name);});;_pop_handler();}else{void*_tmp5D=(void*)Cyc_Core_get_exn_thrown();void*_tmp60=_tmp5D;void*_tmp61;if(((struct Cyc_Dict_Absent_exn_struct*)_tmp60)->tag == Cyc_Dict_Absent){_LL1E: _LL1F:
# 296
({int(*_tmp295)(struct _fat_ptr s)=({int(*_tmp62)(struct _fat_ptr s)=(int(*)(struct _fat_ptr s))Cyc_Interface_invalid_arg;_tmp62;});_tmp295((struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp65=({struct Cyc_String_pa_PrintArg_struct _tmp20E;_tmp20E.tag=0U,({
struct _fat_ptr _tmp293=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(tud0->name));_tmp20E.f1=_tmp293;});_tmp20E;});void*_tmp63[1U];_tmp63[0]=& _tmp65;({struct _fat_ptr _tmp294=({const char*_tmp64="check_public_type (can't find datatype %s)";_tag_fat(_tmp64,sizeof(char),43U);});Cyc_aprintf(_tmp294,_tag_fat(_tmp63,sizeof(void*),1U));});}));});}else{_LL20: _tmp61=_tmp60;_LL21: {void*exn=_tmp61;(int)_rethrow(exn);}}_LL1D:;}}}
# 299
if((int)tud->sc == (int)Cyc_Absyn_Static){
({struct _fat_ptr _tmp298=obj;struct _tuple0*_tmp297=name;struct _fat_ptr _tmp296=({const char*_tmp66="datatype";_tag_fat(_tmp66,sizeof(char),9U);});Cyc_Interface_static_err(_tmp298,_tmp297,_tmp296,tud->name);});
return 0;}
# 299
return
# 303
 Cyc_Interface_check_public_type_list(ae,seen,obj,name,Cyc_Core_identity,targs)&&
 Cyc_Interface_check_public_datatypedecl(ae,seen,tud);}}else{goto _LL13;}case 19U: if(((((struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp3D)->f1)->f1).KnownDatatypefield).tag == 2){_LL11: _tmp3E=(((((struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp3D)->f1)->f1).KnownDatatypefield).val).f1;_tmp3F=(((((struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp3D)->f1)->f1).KnownDatatypefield).val).f2;_tmp40=((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp3D)->f2;_LL12: {struct Cyc_Absyn_Datatypedecl*tud0=_tmp3E;struct Cyc_Absyn_Datatypefield*tuf0=_tmp3F;struct Cyc_List_List*targs=_tmp40;
# 307
struct Cyc_Absyn_Datatypedecl*tud;
{struct _handler_cons _tmp67;_push_handler(& _tmp67);{int _tmp69=0;if(setjmp(_tmp67.handler))_tmp69=1;if(!_tmp69){tud=*({struct Cyc_Absyn_Datatypedecl**(*_tmp29A)(struct Cyc_Dict_Dict d,struct _tuple0*k)=({struct Cyc_Absyn_Datatypedecl**(*_tmp6A)(struct Cyc_Dict_Dict d,struct _tuple0*k)=(struct Cyc_Absyn_Datatypedecl**(*)(struct Cyc_Dict_Dict d,struct _tuple0*k))Cyc_Dict_lookup;_tmp6A;});struct Cyc_Dict_Dict _tmp299=ae->datatypedecls;_tmp29A(_tmp299,tud0->name);});;_pop_handler();}else{void*_tmp68=(void*)Cyc_Core_get_exn_thrown();void*_tmp6B=_tmp68;void*_tmp6C;if(((struct Cyc_Dict_Absent_exn_struct*)_tmp6B)->tag == Cyc_Dict_Absent){_LL23: _LL24:
# 310
({int(*_tmp29D)(struct _fat_ptr s)=({int(*_tmp6D)(struct _fat_ptr s)=(int(*)(struct _fat_ptr s))Cyc_Interface_invalid_arg;_tmp6D;});_tmp29D((struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp70=({struct Cyc_String_pa_PrintArg_struct _tmp20F;_tmp20F.tag=0U,({
struct _fat_ptr _tmp29B=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(tud0->name));_tmp20F.f1=_tmp29B;});_tmp20F;});void*_tmp6E[1U];_tmp6E[0]=& _tmp70;({struct _fat_ptr _tmp29C=({const char*_tmp6F="check_public_type (can't find datatype %s and search its fields)";_tag_fat(_tmp6F,sizeof(char),65U);});Cyc_aprintf(_tmp29C,_tag_fat(_tmp6E,sizeof(void*),1U));});}));});}else{_LL25: _tmp6C=_tmp6B;_LL26: {void*exn=_tmp6C;(int)_rethrow(exn);}}_LL22:;}}}
# 313
if(tud->fields == 0)
({int(*_tmp2A0)(struct _fat_ptr s)=({int(*_tmp71)(struct _fat_ptr s)=(int(*)(struct _fat_ptr s))Cyc_Interface_invalid_arg;_tmp71;});_tmp2A0((struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp74=({struct Cyc_String_pa_PrintArg_struct _tmp210;_tmp210.tag=0U,({
struct _fat_ptr _tmp29E=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(tud->name));_tmp210.f1=_tmp29E;});_tmp210;});void*_tmp72[1U];_tmp72[0]=& _tmp74;({struct _fat_ptr _tmp29F=({const char*_tmp73="check_public_type (datatype %s has no fields)";_tag_fat(_tmp73,sizeof(char),46U);});Cyc_aprintf(_tmp29F,_tag_fat(_tmp72,sizeof(void*),1U));});}));});{
# 313
struct Cyc_Absyn_Datatypefield*tuf1=0;
# 318
{struct Cyc_List_List*fs=(struct Cyc_List_List*)((struct Cyc_Core_Opt*)_check_null(tud->fields))->v;for(0;fs != 0;fs=fs->tl){
if(Cyc_strptrcmp((*tuf0->name).f2,(*((struct Cyc_Absyn_Datatypefield*)fs->hd)->name).f2)== 0){
tuf1=(struct Cyc_Absyn_Datatypefield*)fs->hd;
break;}}}
# 318
if(tuf1 == 0)
# 325
({int(*_tmp2A3)(struct _fat_ptr s)=({int(*_tmp75)(struct _fat_ptr s)=(int(*)(struct _fat_ptr s))Cyc_Interface_invalid_arg;_tmp75;});_tmp2A3((struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp78=({struct Cyc_String_pa_PrintArg_struct _tmp211;_tmp211.tag=0U,({
struct _fat_ptr _tmp2A1=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(tuf0->name));_tmp211.f1=_tmp2A1;});_tmp211;});void*_tmp76[1U];_tmp76[0]=& _tmp78;({struct _fat_ptr _tmp2A2=({const char*_tmp77="check_public_type (can't find datatypefield %s)";_tag_fat(_tmp77,sizeof(char),48U);});Cyc_aprintf(_tmp2A2,_tag_fat(_tmp76,sizeof(void*),1U));});}));});{
# 318
struct Cyc_Absyn_Datatypefield*tuf=tuf1;
# 329
if((int)tud->sc == (int)Cyc_Absyn_Static){
({struct _fat_ptr _tmp2A6=obj;struct _tuple0*_tmp2A5=name;struct _fat_ptr _tmp2A4=({const char*_tmp79="datatype";_tag_fat(_tmp79,sizeof(char),9U);});Cyc_Interface_static_err(_tmp2A6,_tmp2A5,_tmp2A4,tud->name);});
return 0;}
# 329
if((int)tud->sc == (int)Cyc_Absyn_Abstract){
# 334
({struct _fat_ptr _tmp2A9=obj;struct _tuple0*_tmp2A8=name;struct _fat_ptr _tmp2A7=({const char*_tmp7A="datatype";_tag_fat(_tmp7A,sizeof(char),9U);});Cyc_Interface_abstract_err(_tmp2A9,_tmp2A8,_tmp2A7,tud->name);});
return 0;}
# 329
if((int)tuf->sc == (int)Cyc_Absyn_Static){
# 338
({struct _fat_ptr _tmp2AE=obj;struct _tuple0*_tmp2AD=name;struct _fat_ptr _tmp2AC=(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp7D=({struct Cyc_String_pa_PrintArg_struct _tmp212;_tmp212.tag=0U,({
struct _fat_ptr _tmp2AA=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(tuf->name));_tmp212.f1=_tmp2AA;});_tmp212;});void*_tmp7B[1U];_tmp7B[0]=& _tmp7D;({struct _fat_ptr _tmp2AB=({const char*_tmp7C="field %s of";_tag_fat(_tmp7C,sizeof(char),12U);});Cyc_aprintf(_tmp2AB,_tag_fat(_tmp7B,sizeof(void*),1U));});});
# 338
Cyc_Interface_static_err(_tmp2AE,_tmp2AD,_tmp2AC,tud->name);});
# 340
return 0;}
# 329
return
# 342
 Cyc_Interface_check_public_type_list(ae,seen,obj,name,Cyc_Core_identity,targs)&&
 Cyc_Interface_check_public_datatypedecl(ae,seen,tud);}}}}else{goto _LL13;}default: goto _LL13;}default: _LL13: _LL14:
# 345
 return 1;}_LL0:;}struct _tuple26{struct Cyc_Interface_Ienv*f1;struct Cyc_Interface_Ienv*f2;int f3;struct Cyc_Tcenv_Genv*f4;struct Cyc_Interface_Seen*f5;struct Cyc_Interface_I*f6;};
# 190
static void Cyc_Interface_extract_aggrdecl(struct _tuple26*env,struct _tuple0*x,struct Cyc_Absyn_Aggrdecl**dp){
# 353
struct Cyc_Interface_Seen*_tmp82;struct Cyc_Tcenv_Genv*_tmp81;int _tmp80;struct Cyc_Interface_Ienv*_tmp7F;struct Cyc_Interface_Ienv*_tmp7E;_LL1: _tmp7E=env->f1;_tmp7F=env->f2;_tmp80=env->f3;_tmp81=env->f4;_tmp82=env->f5;_LL2: {struct Cyc_Interface_Ienv*imp=_tmp7E;struct Cyc_Interface_Ienv*exp=_tmp7F;int check_complete_defs=_tmp80;struct Cyc_Tcenv_Genv*ae=_tmp81;struct Cyc_Interface_Seen*seen=_tmp82;
struct Cyc_Absyn_Aggrdecl*d=*dp;
enum Cyc_Absyn_Scope _stmttmp2=d->sc;enum Cyc_Absyn_Scope _tmp83=_stmttmp2;switch(_tmp83){case Cyc_Absyn_Static: _LL4: _LL5:
# 357
 if(check_complete_defs && d->impl == 0)
({struct _fat_ptr _tmp2B0=({const char*_tmp84="static";_tag_fat(_tmp84,sizeof(char),7U);});struct _fat_ptr _tmp2AF=({const char*_tmp85="struct/union";_tag_fat(_tmp85,sizeof(char),13U);});Cyc_Interface_fields_err(_tmp2B0,_tmp2AF,d->name);});
# 357
goto _LL3;case Cyc_Absyn_Abstract: _LL6: _LL7:
# 361
 if(d->impl == 0){
if(check_complete_defs)
({struct _fat_ptr _tmp2B2=({const char*_tmp86="abstract";_tag_fat(_tmp86,sizeof(char),9U);});struct _fat_ptr _tmp2B1=({const char*_tmp87="struct/union";_tag_fat(_tmp87,sizeof(char),13U);});Cyc_Interface_fields_err(_tmp2B2,_tmp2B1,d->name);});}else{
# 365
d=({struct Cyc_Absyn_Aggrdecl*_tmp88=_cycalloc(sizeof(*_tmp88));*_tmp88=*d;_tmp88;});
d->impl=0;}
# 368
if(Cyc_Interface_check_public_aggrdecl(ae,seen,d))
({struct Cyc_Dict_Dict _tmp2B6=({struct Cyc_Dict_Dict(*_tmp2B5)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Aggrdecl*v)=({struct Cyc_Dict_Dict(*_tmp89)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Aggrdecl*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Aggrdecl*v))Cyc_Dict_insert;_tmp89;});struct Cyc_Dict_Dict _tmp2B4=exp->aggrdecls;struct _tuple0*_tmp2B3=x;_tmp2B5(_tmp2B4,_tmp2B3,d);});exp->aggrdecls=_tmp2B6;});
# 368
goto _LL3;case Cyc_Absyn_Public: _LL8: _LL9:
# 372
 if(d->impl == 0){
({struct _fat_ptr _tmp2B8=({const char*_tmp8A="public";_tag_fat(_tmp8A,sizeof(char),7U);});struct _fat_ptr _tmp2B7=({const char*_tmp8B="struct/union";_tag_fat(_tmp8B,sizeof(char),13U);});Cyc_Interface_fields_err(_tmp2B8,_tmp2B7,d->name);});
d=({struct Cyc_Absyn_Aggrdecl*_tmp8C=_cycalloc(sizeof(*_tmp8C));*_tmp8C=*d;_tmp8C;});
d->sc=Cyc_Absyn_Abstract;}
# 372
if(
# 377
Cyc_Interface_check_public_aggrdecl(ae,seen,d))
({struct Cyc_Dict_Dict _tmp2BC=({struct Cyc_Dict_Dict(*_tmp2BB)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Aggrdecl*v)=({struct Cyc_Dict_Dict(*_tmp8D)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Aggrdecl*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Aggrdecl*v))Cyc_Dict_insert;_tmp8D;});struct Cyc_Dict_Dict _tmp2BA=exp->aggrdecls;struct _tuple0*_tmp2B9=x;_tmp2BB(_tmp2BA,_tmp2B9,d);});exp->aggrdecls=_tmp2BC;});
# 372
goto _LL3;case Cyc_Absyn_ExternC: _LLA: _LLB:
# 380
 goto _LLD;case Cyc_Absyn_Extern: _LLC: _LLD:
# 382
 if(Cyc_Interface_check_public_aggrdecl(ae,seen,d))
({struct Cyc_Dict_Dict _tmp2C0=({struct Cyc_Dict_Dict(*_tmp2BF)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Aggrdecl*v)=({struct Cyc_Dict_Dict(*_tmp8E)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Aggrdecl*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Aggrdecl*v))Cyc_Dict_insert;_tmp8E;});struct Cyc_Dict_Dict _tmp2BE=imp->aggrdecls;struct _tuple0*_tmp2BD=x;_tmp2BF(_tmp2BE,_tmp2BD,d);});imp->aggrdecls=_tmp2C0;});
# 382
goto _LL3;case Cyc_Absyn_Register: _LLE: _LLF:
# 385
 goto _LL11;default: _LL10: _LL11:
# 387
({int(*_tmp2C1)(struct _fat_ptr s)=({int(*_tmp8F)(struct _fat_ptr s)=(int(*)(struct _fat_ptr s))Cyc_Interface_invalid_arg;_tmp8F;});_tmp2C1(({const char*_tmp90="add_aggrdecl";_tag_fat(_tmp90,sizeof(char),13U);}));});
goto _LL3;}_LL3:;}}
# 190
static void Cyc_Interface_extract_enumdecl(struct _tuple26*env,struct _tuple0*x,struct Cyc_Absyn_Enumdecl**dp){
# 393
struct Cyc_Interface_Seen*_tmp95;struct Cyc_Tcenv_Genv*_tmp94;int _tmp93;struct Cyc_Interface_Ienv*_tmp92;struct Cyc_Interface_Ienv*_tmp91;_LL1: _tmp91=env->f1;_tmp92=env->f2;_tmp93=env->f3;_tmp94=env->f4;_tmp95=env->f5;_LL2: {struct Cyc_Interface_Ienv*imp=_tmp91;struct Cyc_Interface_Ienv*exp=_tmp92;int check_complete_defs=_tmp93;struct Cyc_Tcenv_Genv*ae=_tmp94;struct Cyc_Interface_Seen*seen=_tmp95;
struct Cyc_Absyn_Enumdecl*d=*dp;
enum Cyc_Absyn_Scope _stmttmp3=d->sc;enum Cyc_Absyn_Scope _tmp96=_stmttmp3;switch(_tmp96){case Cyc_Absyn_Static: _LL4: _LL5:
# 397
 if(check_complete_defs && d->fields == 0)
({struct _fat_ptr _tmp2C3=({const char*_tmp97="static";_tag_fat(_tmp97,sizeof(char),7U);});struct _fat_ptr _tmp2C2=({const char*_tmp98="enum";_tag_fat(_tmp98,sizeof(char),5U);});Cyc_Interface_fields_err(_tmp2C3,_tmp2C2,d->name);});
# 397
goto _LL3;case Cyc_Absyn_Abstract: _LL6: _LL7:
# 401
 if(d->fields == 0){
if(check_complete_defs)
({struct _fat_ptr _tmp2C5=({const char*_tmp99="abstract";_tag_fat(_tmp99,sizeof(char),9U);});struct _fat_ptr _tmp2C4=({const char*_tmp9A="enum";_tag_fat(_tmp9A,sizeof(char),5U);});Cyc_Interface_fields_err(_tmp2C5,_tmp2C4,d->name);});}else{
# 405
d=({struct Cyc_Absyn_Enumdecl*_tmp9B=_cycalloc(sizeof(*_tmp9B));*_tmp9B=*d;_tmp9B;});
d->fields=0;}
# 408
if(Cyc_Interface_check_public_enumdecl(ae,seen,d))
({struct Cyc_Dict_Dict _tmp2C9=({struct Cyc_Dict_Dict(*_tmp2C8)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Enumdecl*v)=({struct Cyc_Dict_Dict(*_tmp9C)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Enumdecl*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Enumdecl*v))Cyc_Dict_insert;_tmp9C;});struct Cyc_Dict_Dict _tmp2C7=exp->enumdecls;struct _tuple0*_tmp2C6=x;_tmp2C8(_tmp2C7,_tmp2C6,d);});exp->enumdecls=_tmp2C9;});
# 408
goto _LL3;case Cyc_Absyn_Public: _LL8: _LL9:
# 412
 if(d->fields == 0){
({struct _fat_ptr _tmp2CB=({const char*_tmp9D="public";_tag_fat(_tmp9D,sizeof(char),7U);});struct _fat_ptr _tmp2CA=({const char*_tmp9E="enum";_tag_fat(_tmp9E,sizeof(char),5U);});Cyc_Interface_fields_err(_tmp2CB,_tmp2CA,d->name);});
d=({struct Cyc_Absyn_Enumdecl*_tmp9F=_cycalloc(sizeof(*_tmp9F));*_tmp9F=*d;_tmp9F;});
d->sc=Cyc_Absyn_Abstract;}
# 412
if(
# 417
Cyc_Interface_check_public_enumdecl(ae,seen,d))
({struct Cyc_Dict_Dict _tmp2CF=({struct Cyc_Dict_Dict(*_tmp2CE)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Enumdecl*v)=({struct Cyc_Dict_Dict(*_tmpA0)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Enumdecl*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Enumdecl*v))Cyc_Dict_insert;_tmpA0;});struct Cyc_Dict_Dict _tmp2CD=exp->enumdecls;struct _tuple0*_tmp2CC=x;_tmp2CE(_tmp2CD,_tmp2CC,d);});exp->enumdecls=_tmp2CF;});
# 412
goto _LL3;case Cyc_Absyn_ExternC: _LLA: _LLB:
# 420
 goto _LLD;case Cyc_Absyn_Extern: _LLC: _LLD:
# 422
 if(Cyc_Interface_check_public_enumdecl(ae,seen,d))
({struct Cyc_Dict_Dict _tmp2D3=({struct Cyc_Dict_Dict(*_tmp2D2)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Enumdecl*v)=({struct Cyc_Dict_Dict(*_tmpA1)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Enumdecl*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Enumdecl*v))Cyc_Dict_insert;_tmpA1;});struct Cyc_Dict_Dict _tmp2D1=imp->enumdecls;struct _tuple0*_tmp2D0=x;_tmp2D2(_tmp2D1,_tmp2D0,d);});imp->enumdecls=_tmp2D3;});
# 422
goto _LL3;case Cyc_Absyn_Register: _LLE: _LLF:
# 425
 goto _LL11;default: _LL10: _LL11:
# 427
({int(*_tmp2D4)(struct _fat_ptr s)=({int(*_tmpA2)(struct _fat_ptr s)=(int(*)(struct _fat_ptr s))Cyc_Interface_invalid_arg;_tmpA2;});_tmp2D4(({const char*_tmpA3="add_enumdecl";_tag_fat(_tmpA3,sizeof(char),13U);}));});
goto _LL3;}_LL3:;}}
# 190
static void Cyc_Interface_extract_xdatatypefielddecl(struct Cyc_Interface_I*i,struct Cyc_Absyn_Datatypedecl*d,struct Cyc_Absyn_Datatypefield*f){
# 434
struct Cyc_Interface_Ienv*env;
{enum Cyc_Absyn_Scope _stmttmp4=f->sc;enum Cyc_Absyn_Scope _tmpA4=_stmttmp4;switch(_tmpA4){case Cyc_Absyn_Static: _LL1: _LL2:
 return;case Cyc_Absyn_Extern: _LL3: _LL4:
 env=i->imports;goto _LL0;case Cyc_Absyn_Public: _LL5: _LL6:
 env=i->exports;goto _LL0;default: _LL7: _LL8:
({int(*_tmp2D5)(struct _fat_ptr s)=({int(*_tmpA5)(struct _fat_ptr s)=(int(*)(struct _fat_ptr s))Cyc_Interface_invalid_arg;_tmpA5;});_tmp2D5(({const char*_tmpA6="add_xdatatypefielddecl";_tag_fat(_tmpA6,sizeof(char),23U);}));});}_LL0:;}
# 442
({struct Cyc_Dict_Dict _tmp2D9=({
struct Cyc_Dict_Dict(*_tmp2D8)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Tcdecl_Xdatatypefielddecl*v)=({struct Cyc_Dict_Dict(*_tmpA7)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Tcdecl_Xdatatypefielddecl*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Tcdecl_Xdatatypefielddecl*v))Cyc_Dict_insert;_tmpA7;});struct Cyc_Dict_Dict _tmp2D7=env->xdatatypefielddecls;struct _tuple0*_tmp2D6=f->name;_tmp2D8(_tmp2D7,_tmp2D6,({struct Cyc_Tcdecl_Xdatatypefielddecl*_tmpA8=_cycalloc(sizeof(*_tmpA8));
_tmpA8->base=d,_tmpA8->field=f;_tmpA8;}));});
# 442
env->xdatatypefielddecls=_tmp2D9;});}
# 190
static void Cyc_Interface_extract_datatypedecl(struct _tuple26*env,struct _tuple0*x,struct Cyc_Absyn_Datatypedecl**dp){
# 449
struct Cyc_Interface_I*_tmpAE;struct Cyc_Interface_Seen*_tmpAD;struct Cyc_Tcenv_Genv*_tmpAC;int _tmpAB;struct Cyc_Interface_Ienv*_tmpAA;struct Cyc_Interface_Ienv*_tmpA9;_LL1: _tmpA9=env->f1;_tmpAA=env->f2;_tmpAB=env->f3;_tmpAC=env->f4;_tmpAD=env->f5;_tmpAE=env->f6;_LL2: {struct Cyc_Interface_Ienv*imp=_tmpA9;struct Cyc_Interface_Ienv*exp=_tmpAA;int check_complete_defs=_tmpAB;struct Cyc_Tcenv_Genv*ae=_tmpAC;struct Cyc_Interface_Seen*seen=_tmpAD;struct Cyc_Interface_I*i=_tmpAE;
struct Cyc_Absyn_Datatypedecl*d=*dp;
# 452
enum Cyc_Absyn_Scope _stmttmp5=d->sc;enum Cyc_Absyn_Scope _tmpAF=_stmttmp5;switch(_tmpAF){case Cyc_Absyn_Static: _LL4: _LL5:
# 454
 if((!d->is_extensible && check_complete_defs)&& d->fields == 0)
({struct _fat_ptr _tmp2DB=({const char*_tmpB0="static";_tag_fat(_tmpB0,sizeof(char),7U);});struct _fat_ptr _tmp2DA=({const char*_tmpB1="datatype";_tag_fat(_tmpB1,sizeof(char),9U);});Cyc_Interface_fields_err(_tmp2DB,_tmp2DA,d->name);});
# 454
goto _LL3;case Cyc_Absyn_Abstract: _LL6: _LL7:
# 459
 if(d->fields == 0){
if(!d->is_extensible && check_complete_defs)
({struct _fat_ptr _tmp2DD=({const char*_tmpB2="abstract";_tag_fat(_tmpB2,sizeof(char),9U);});struct _fat_ptr _tmp2DC=({const char*_tmpB3="datatype";_tag_fat(_tmpB3,sizeof(char),9U);});Cyc_Interface_fields_err(_tmp2DD,_tmp2DC,d->name);});}else{
# 464
d=({struct Cyc_Absyn_Datatypedecl*_tmpB4=_cycalloc(sizeof(*_tmpB4));*_tmpB4=*d;_tmpB4;});
d->fields=0;}
# 467
if(Cyc_Interface_check_public_datatypedecl(ae,seen,d))
({struct Cyc_Dict_Dict _tmp2E1=({struct Cyc_Dict_Dict(*_tmp2E0)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Datatypedecl*v)=({struct Cyc_Dict_Dict(*_tmpB5)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Datatypedecl*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Datatypedecl*v))Cyc_Dict_insert;_tmpB5;});struct Cyc_Dict_Dict _tmp2DF=exp->datatypedecls;struct _tuple0*_tmp2DE=x;_tmp2E0(_tmp2DF,_tmp2DE,d);});exp->datatypedecls=_tmp2E1;});
# 467
goto _LL3;case Cyc_Absyn_Public: _LL8: _LL9:
# 471
 d=({struct Cyc_Absyn_Datatypedecl*_tmpB6=_cycalloc(sizeof(*_tmpB6));*_tmpB6=*d;_tmpB6;});
if(!d->is_extensible && d->fields == 0){
({struct _fat_ptr _tmp2E3=({const char*_tmpB7="public";_tag_fat(_tmpB7,sizeof(char),7U);});struct _fat_ptr _tmp2E2=({const char*_tmpB8="datatype";_tag_fat(_tmpB8,sizeof(char),9U);});Cyc_Interface_fields_err(_tmp2E3,_tmp2E2,d->name);});
d->sc=Cyc_Absyn_Abstract;}
# 472
if(
# 476
Cyc_Interface_check_public_datatypedecl(ae,seen,d)){
if(d->is_extensible && d->fields != 0){
struct Cyc_List_List*fields=(struct Cyc_List_List*)((struct Cyc_Core_Opt*)_check_null(d->fields))->v;
d->fields=0;{
struct Cyc_List_List*f=fields;for(0;f != 0;f=f->tl){
Cyc_Interface_extract_xdatatypefielddecl(i,d,(struct Cyc_Absyn_Datatypefield*)f->hd);}}}
# 477
({struct Cyc_Dict_Dict _tmp2E7=({
# 484
struct Cyc_Dict_Dict(*_tmp2E6)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Datatypedecl*v)=({struct Cyc_Dict_Dict(*_tmpB9)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Datatypedecl*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Datatypedecl*v))Cyc_Dict_insert;_tmpB9;});struct Cyc_Dict_Dict _tmp2E5=exp->datatypedecls;struct _tuple0*_tmp2E4=x;_tmp2E6(_tmp2E5,_tmp2E4,d);});
# 477
exp->datatypedecls=_tmp2E7;});}
# 472
goto _LL3;case Cyc_Absyn_ExternC: _LLA: _LLB:
# 487
({int(*_tmp2E8)(struct _fat_ptr s)=({int(*_tmpBA)(struct _fat_ptr s)=(int(*)(struct _fat_ptr s))Cyc_Interface_invalid_arg;_tmpBA;});_tmp2E8(({const char*_tmpBB="extract_datatypedecl";_tag_fat(_tmpBB,sizeof(char),21U);}));});case Cyc_Absyn_Extern: _LLC: _LLD:
# 489
 if(Cyc_Interface_check_public_datatypedecl(ae,seen,d)){
if(d->is_extensible && d->fields != 0){
d=({struct Cyc_Absyn_Datatypedecl*_tmpBC=_cycalloc(sizeof(*_tmpBC));*_tmpBC=*d;_tmpBC;});{
struct Cyc_List_List*fields=(struct Cyc_List_List*)((struct Cyc_Core_Opt*)_check_null(d->fields))->v;
d->fields=0;{
struct Cyc_List_List*f=fields;for(0;f != 0;f=f->tl){
Cyc_Interface_extract_xdatatypefielddecl(i,d,(struct Cyc_Absyn_Datatypefield*)f->hd);}}}}
# 490
({struct Cyc_Dict_Dict _tmp2EC=({
# 498
struct Cyc_Dict_Dict(*_tmp2EB)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Datatypedecl*v)=({struct Cyc_Dict_Dict(*_tmpBD)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Datatypedecl*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Datatypedecl*v))Cyc_Dict_insert;_tmpBD;});struct Cyc_Dict_Dict _tmp2EA=imp->datatypedecls;struct _tuple0*_tmp2E9=x;_tmp2EB(_tmp2EA,_tmp2E9,d);});
# 490
imp->datatypedecls=_tmp2EC;});}
# 489
goto _LL3;case Cyc_Absyn_Register: _LLE: _LLF:
# 501
 goto _LL11;default: _LL10: _LL11:
# 503
({int(*_tmp2ED)(struct _fat_ptr s)=({int(*_tmpBE)(struct _fat_ptr s)=(int(*)(struct _fat_ptr s))Cyc_Interface_invalid_arg;_tmpBE;});_tmp2ED(({const char*_tmpBF="add_datatypedecl";_tag_fat(_tmpBF,sizeof(char),17U);}));});
goto _LL3;}_LL3:;}}
# 190
static void Cyc_Interface_extract_typedef(struct _tuple26*env,struct _tuple0*x,struct Cyc_Absyn_Typedefdecl*d){
# 510
struct Cyc_Interface_Seen*_tmpC2;struct Cyc_Tcenv_Genv*_tmpC1;struct Cyc_Interface_Ienv*_tmpC0;_LL1: _tmpC0=env->f2;_tmpC1=env->f4;_tmpC2=env->f5;_LL2: {struct Cyc_Interface_Ienv*exp=_tmpC0;struct Cyc_Tcenv_Genv*ae=_tmpC1;struct Cyc_Interface_Seen*seen=_tmpC2;
if(Cyc_Interface_check_public_typedefdecl(ae,seen,d))
({struct Cyc_Dict_Dict _tmp2F1=({struct Cyc_Dict_Dict(*_tmp2F0)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Typedefdecl*v)=({struct Cyc_Dict_Dict(*_tmpC3)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Typedefdecl*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Typedefdecl*v))Cyc_Dict_insert;_tmpC3;});struct Cyc_Dict_Dict _tmp2EF=exp->typedefdecls;struct _tuple0*_tmp2EE=x;_tmp2F0(_tmp2EF,_tmp2EE,d);});exp->typedefdecls=_tmp2F1;});}}struct _tuple27{void*f1;int f2;};
# 190
static void Cyc_Interface_extract_ordinarie(struct _tuple26*env,struct _tuple0*x,struct _tuple27*v){
# 517
struct Cyc_Interface_Seen*_tmpC8;struct Cyc_Tcenv_Genv*_tmpC7;int _tmpC6;struct Cyc_Interface_Ienv*_tmpC5;struct Cyc_Interface_Ienv*_tmpC4;_LL1: _tmpC4=env->f1;_tmpC5=env->f2;_tmpC6=env->f3;_tmpC7=env->f4;_tmpC8=env->f5;_LL2: {struct Cyc_Interface_Ienv*imp=_tmpC4;struct Cyc_Interface_Ienv*exp=_tmpC5;int check_complete_defs=_tmpC6;struct Cyc_Tcenv_Genv*ae=_tmpC7;struct Cyc_Interface_Seen*seen=_tmpC8;
# 519
void*_stmttmp6=(*v).f1;void*_tmpC9=_stmttmp6;struct Cyc_Absyn_Vardecl*_tmpCA;struct Cyc_Absyn_Fndecl*_tmpCB;switch(*((int*)_tmpC9)){case 2U: _LL4: _tmpCB=((struct Cyc_Absyn_Funname_b_Absyn_Binding_struct*)_tmpC9)->f1;_LL5: {struct Cyc_Absyn_Fndecl*fd=_tmpCB;
# 521
struct Cyc_Absyn_Vardecl*vd=({struct Cyc_Absyn_Vardecl*_tmpCC=_cycalloc(sizeof(*_tmpCC));_tmpCC->sc=fd->sc,_tmpCC->name=fd->name,_tmpCC->varloc=0U,({
# 525
struct Cyc_Absyn_Tqual _tmp2F2=Cyc_Absyn_empty_tqual(0U);_tmpCC->tq=_tmp2F2;}),_tmpCC->type=(void*)_check_null(fd->cached_type),_tmpCC->initializer=0,_tmpCC->rgn=0,_tmpCC->attributes=0,_tmpCC->escapes=0;_tmpCC;});
# 532
check_complete_defs=0;
_tmpCA=vd;goto _LL7;}case 1U: _LL6: _tmpCA=((struct Cyc_Absyn_Global_b_Absyn_Binding_struct*)_tmpC9)->f1;_LL7: {struct Cyc_Absyn_Vardecl*d=_tmpCA;
# 535
if(d->initializer != 0){
d=({struct Cyc_Absyn_Vardecl*_tmpCD=_cycalloc(sizeof(*_tmpCD));*_tmpCD=*d;_tmpCD;});
d->initializer=0;}
# 535
{enum Cyc_Absyn_Scope _stmttmp7=d->sc;enum Cyc_Absyn_Scope _tmpCE=_stmttmp7;switch(_tmpCE){case Cyc_Absyn_Static: _LLB: _LLC:
# 541
 if(check_complete_defs && Cyc_Tcutil_is_function_type(d->type))
({struct _fat_ptr _tmp2F3=({const char*_tmpCF="static";_tag_fat(_tmpCF,sizeof(char),7U);});Cyc_Interface_body_err(_tmp2F3,d->name);});
# 541
goto _LLA;case Cyc_Absyn_Register: _LLD: _LLE:
# 545
 goto _LL10;case Cyc_Absyn_Abstract: _LLF: _LL10:
({int(*_tmp2F4)(struct _fat_ptr s)=({int(*_tmpD0)(struct _fat_ptr s)=(int(*)(struct _fat_ptr s))Cyc_Interface_invalid_arg;_tmpD0;});_tmp2F4(({const char*_tmpD1="extract_ordinarie";_tag_fat(_tmpD1,sizeof(char),18U);}));});case Cyc_Absyn_Public: _LL11: _LL12:
# 548
 if(check_complete_defs && Cyc_Tcutil_is_function_type(d->type))
({struct _fat_ptr _tmp2F5=({const char*_tmpD2="public";_tag_fat(_tmpD2,sizeof(char),7U);});Cyc_Interface_body_err(_tmp2F5,d->name);});
# 548
if(
# 551
Cyc_Interface_check_public_vardecl(ae,seen,d))
({struct Cyc_Dict_Dict _tmp2F9=({struct Cyc_Dict_Dict(*_tmp2F8)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Vardecl*v)=({struct Cyc_Dict_Dict(*_tmpD3)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Vardecl*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Vardecl*v))Cyc_Dict_insert;_tmpD3;});struct Cyc_Dict_Dict _tmp2F7=exp->vardecls;struct _tuple0*_tmp2F6=x;_tmp2F8(_tmp2F7,_tmp2F6,d);});exp->vardecls=_tmp2F9;});
# 548
goto _LLA;case Cyc_Absyn_ExternC: _LL13: _LL14:
# 554
 goto _LL16;case Cyc_Absyn_Extern: _LL15: _LL16:
 goto _LL18;default: _LL17: _LL18:
# 557
 if(Cyc_Interface_check_public_vardecl(ae,seen,d))
({struct Cyc_Dict_Dict _tmp2FD=({struct Cyc_Dict_Dict(*_tmp2FC)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Vardecl*v)=({struct Cyc_Dict_Dict(*_tmpD4)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Vardecl*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Vardecl*v))Cyc_Dict_insert;_tmpD4;});struct Cyc_Dict_Dict _tmp2FB=imp->vardecls;struct _tuple0*_tmp2FA=x;_tmp2FC(_tmp2FB,_tmp2FA,d);});imp->vardecls=_tmp2FD;});
# 557
goto _LLA;}_LLA:;}
# 561
goto _LL3;}default: _LL8: _LL9:
 goto _LL3;}_LL3:;}}struct _tuple28{void*f1;void*f2;};
# 190
static struct Cyc_List_List*Cyc_Interface_remove_decl_from_list(struct Cyc_List_List*l,struct Cyc_Absyn_Decl*d){
# 567
if(l == 0)return 0;{struct _tuple28 _stmttmp8=({struct _tuple28 _tmp213;_tmp213.f1=d->r,_tmp213.f2=((struct Cyc_Absyn_Decl*)l->hd)->r;_tmp213;});struct _tuple28 _tmpD5=_stmttmp8;struct Cyc_Absyn_Vardecl*_tmpD7;struct Cyc_Absyn_Vardecl**_tmpD6;struct Cyc_Absyn_Enumdecl*_tmpD9;struct Cyc_Absyn_Enumdecl*_tmpD8;struct Cyc_Absyn_Typedefdecl*_tmpDB;struct Cyc_Absyn_Typedefdecl*_tmpDA;switch(*((int*)_tmpD5.f1)){case 8U: if(((struct Cyc_Absyn_Typedef_d_Absyn_Raw_decl_struct*)_tmpD5.f2)->tag == 8U){_LL1: _tmpDA=((struct Cyc_Absyn_Typedef_d_Absyn_Raw_decl_struct*)_tmpD5.f1)->f1;_tmpDB=((struct Cyc_Absyn_Typedef_d_Absyn_Raw_decl_struct*)_tmpD5.f2)->f1;_LL2: {struct Cyc_Absyn_Typedefdecl*a1=_tmpDA;struct Cyc_Absyn_Typedefdecl*a2=_tmpDB;
# 570
if(Cyc_Absyn_qvar_cmp(a1->name,a2->name)!= 0)goto _LL0;return
 Cyc_Interface_remove_decl_from_list(l->tl,d);}}else{goto _LL7;}case 7U: if(((struct Cyc_Absyn_Enum_d_Absyn_Raw_decl_struct*)_tmpD5.f2)->tag == 7U){_LL3: _tmpD8=((struct Cyc_Absyn_Enum_d_Absyn_Raw_decl_struct*)_tmpD5.f1)->f1;_tmpD9=((struct Cyc_Absyn_Enum_d_Absyn_Raw_decl_struct*)_tmpD5.f2)->f1;_LL4: {struct Cyc_Absyn_Enumdecl*a1=_tmpD8;struct Cyc_Absyn_Enumdecl*a2=_tmpD9;
# 573
if(Cyc_Absyn_qvar_cmp(a1->name,a2->name)!= 0)goto _LL0;if((int)a1->sc == (int)Cyc_Absyn_Extern)
a1->sc=a2->sc;
# 573
return
# 575
 Cyc_Interface_remove_decl_from_list(l->tl,d);
goto _LL0;}}else{goto _LL7;}case 0U: if(((struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct*)_tmpD5.f2)->tag == 0U){_LL5: _tmpD6=(struct Cyc_Absyn_Vardecl**)&((struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct*)_tmpD5.f1)->f1;_tmpD7=((struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct*)_tmpD5.f2)->f1;_LL6: {struct Cyc_Absyn_Vardecl**vd1=_tmpD6;struct Cyc_Absyn_Vardecl*vd2=_tmpD7;
# 578
if(Cyc_Absyn_qvar_cmp((*vd1)->name,vd2->name)!= 0)goto _LL0;if((int)(*vd1)->sc == (int)Cyc_Absyn_Extern)
({struct Cyc_Absyn_Vardecl*_tmp2FE=({struct Cyc_Absyn_Vardecl*_tmpDC=_cycalloc(sizeof(*_tmpDC));*_tmpDC=*vd2;_tmpDC;});*vd1=_tmp2FE;});
# 578
return
# 580
 Cyc_Interface_remove_decl_from_list(l->tl,d);}}else{goto _LL7;}default: _LL7: _LL8:
# 582
 goto _LL0;}_LL0:;}
# 584
return({struct Cyc_List_List*_tmpDD=_cycalloc(sizeof(*_tmpDD));_tmpDD->hd=(struct Cyc_Absyn_Decl*)l->hd,({struct Cyc_List_List*_tmp2FF=Cyc_Interface_remove_decl_from_list(l->tl,d);_tmpDD->tl=_tmp2FF;});_tmpDD;});}
# 190
static struct Cyc_List_List*Cyc_Interface_uniqify_decl_list(struct Cyc_List_List*accum,struct Cyc_Absyn_Decl*d){
# 588
if(accum == 0)return({struct Cyc_List_List*_tmpDE=_cycalloc(sizeof(*_tmpDE));_tmpDE->hd=d,_tmpDE->tl=0;_tmpDE;});{struct Cyc_List_List*l;
# 590
for(l=accum;l != 0;l=l->tl){
struct _tuple28 _stmttmp9=({struct _tuple28 _tmp214;_tmp214.f1=d->r,_tmp214.f2=((struct Cyc_Absyn_Decl*)l->hd)->r;_tmp214;});struct _tuple28 _tmpDF=_stmttmp9;struct Cyc_List_List*_tmpE3;struct _fat_ptr*_tmpE2;struct Cyc_List_List**_tmpE1;struct _fat_ptr*_tmpE0;struct Cyc_Absyn_Enumdecl*_tmpE5;struct Cyc_Absyn_Enumdecl*_tmpE4;struct Cyc_Absyn_Typedefdecl*_tmpE7;struct Cyc_Absyn_Typedefdecl*_tmpE6;struct Cyc_Absyn_Vardecl**_tmpE9;struct Cyc_Absyn_Vardecl*_tmpE8;switch(*((int*)_tmpDF.f1)){case 0U: if(((struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct*)_tmpDF.f2)->tag == 0U){_LL1: _tmpE8=((struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct*)_tmpDF.f1)->f1;_tmpE9=(struct Cyc_Absyn_Vardecl**)&((struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct*)_tmpDF.f2)->f1;_LL2: {struct Cyc_Absyn_Vardecl*vd1=_tmpE8;struct Cyc_Absyn_Vardecl**vd2=_tmpE9;
# 593
if(Cyc_Absyn_qvar_cmp(vd1->name,(*vd2)->name)!= 0)goto _LL0;if((int)(*vd2)->sc == (int)Cyc_Absyn_Extern)
({struct Cyc_Absyn_Vardecl*_tmp300=({struct Cyc_Absyn_Vardecl*_tmpEA=_cycalloc(sizeof(*_tmpEA));*_tmpEA=*vd1;_tmpEA;});*vd2=_tmp300;});
# 593
return accum;}}else{goto _LL9;}case 8U: if(((struct Cyc_Absyn_Typedef_d_Absyn_Raw_decl_struct*)_tmpDF.f2)->tag == 8U){_LL3: _tmpE6=((struct Cyc_Absyn_Typedef_d_Absyn_Raw_decl_struct*)_tmpDF.f1)->f1;_tmpE7=((struct Cyc_Absyn_Typedef_d_Absyn_Raw_decl_struct*)_tmpDF.f2)->f1;_LL4: {struct Cyc_Absyn_Typedefdecl*a1=_tmpE6;struct Cyc_Absyn_Typedefdecl*a2=_tmpE7;
# 597
if(Cyc_Absyn_qvar_cmp(a1->name,a2->name)!= 0)goto _LL0;return accum;}}else{goto _LL9;}case 7U: if(((struct Cyc_Absyn_Enum_d_Absyn_Raw_decl_struct*)_tmpDF.f2)->tag == 7U){_LL5: _tmpE4=((struct Cyc_Absyn_Enum_d_Absyn_Raw_decl_struct*)_tmpDF.f1)->f1;_tmpE5=((struct Cyc_Absyn_Enum_d_Absyn_Raw_decl_struct*)_tmpDF.f2)->f1;_LL6: {struct Cyc_Absyn_Enumdecl*a1=_tmpE4;struct Cyc_Absyn_Enumdecl*a2=_tmpE5;
# 600
if(Cyc_Absyn_qvar_cmp(a1->name,a2->name)!= 0)goto _LL0;return accum;}}else{goto _LL9;}case 9U: if(((struct Cyc_Absyn_Namespace_d_Absyn_Raw_decl_struct*)_tmpDF.f2)->tag == 9U){_LL7: _tmpE0=((struct Cyc_Absyn_Namespace_d_Absyn_Raw_decl_struct*)_tmpDF.f1)->f1;_tmpE1=(struct Cyc_List_List**)&((struct Cyc_Absyn_Namespace_d_Absyn_Raw_decl_struct*)_tmpDF.f1)->f2;_tmpE2=((struct Cyc_Absyn_Namespace_d_Absyn_Raw_decl_struct*)_tmpDF.f2)->f1;_tmpE3=((struct Cyc_Absyn_Namespace_d_Absyn_Raw_decl_struct*)_tmpDF.f2)->f2;_LL8: {struct _fat_ptr*a1=_tmpE0;struct Cyc_List_List**b1=_tmpE1;struct _fat_ptr*a2=_tmpE2;struct Cyc_List_List*b2=_tmpE3;
# 603
if(Cyc_strptrcmp(a1,a2)!= 0)goto _LL0;{struct Cyc_List_List*dl=b2;
# 605
for(0;dl != 0;dl=dl->tl){
({struct Cyc_List_List*_tmp301=Cyc_Interface_remove_decl_from_list(*b1,(struct Cyc_Absyn_Decl*)dl->hd);*b1=_tmp301;});}
# 608
goto _LL0;}}}else{goto _LL9;}default: _LL9: _LLA:
# 610
 goto _LL0;}_LL0:;}
# 613
return({struct Cyc_List_List*_tmpEB=_cycalloc(sizeof(*_tmpEB));_tmpEB->hd=d,_tmpEB->tl=accum;_tmpEB;});}}
# 190
static struct Cyc_List_List*Cyc_Interface_filterstatics(struct Cyc_List_List*accum,struct Cyc_Absyn_Decl*d){
# 617
{void*_stmttmpA=d->r;void*_tmpEC=_stmttmpA;struct Cyc_List_List*_tmpED;struct Cyc_List_List*_tmpEF;struct _fat_ptr*_tmpEE;struct Cyc_Absyn_Typedefdecl*_tmpF0;struct Cyc_Absyn_Enumdecl*_tmpF1;struct Cyc_Absyn_Datatypedecl*_tmpF2;struct Cyc_Absyn_Aggrdecl*_tmpF3;struct Cyc_Absyn_Fndecl*_tmpF4;struct Cyc_Absyn_Vardecl*_tmpF5;switch(*((int*)_tmpEC)){case 0U: _LL1: _tmpF5=((struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct*)_tmpEC)->f1;_LL2: {struct Cyc_Absyn_Vardecl*vd=_tmpF5;
# 619
if((int)vd->sc == (int)Cyc_Absyn_ExternC)return accum;if((int)vd->sc == (int)Cyc_Absyn_Static)
return accum;{
# 619
struct Cyc_Absyn_Vardecl*nvd=({struct Cyc_Absyn_Vardecl*_tmpF8=_cycalloc(sizeof(*_tmpF8));*_tmpF8=*vd;_tmpF8;});
# 622
nvd->initializer=0;
if(Cyc_Tcutil_is_function_type(nvd->type)&&(int)nvd->sc != (int)Cyc_Absyn_Extern)
nvd->sc=Cyc_Absyn_Extern;
# 623
return({struct Cyc_List_List*_tmpF7=_cycalloc(sizeof(*_tmpF7));
# 625
({struct Cyc_Absyn_Decl*_tmp302=Cyc_Absyn_new_decl((void*)({struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct*_tmpF6=_cycalloc(sizeof(*_tmpF6));_tmpF6->tag=0U,_tmpF6->f1=nvd;_tmpF6;}),0U);_tmpF7->hd=_tmp302;}),_tmpF7->tl=accum;_tmpF7;});}}case 1U: _LL3: _tmpF4=((struct Cyc_Absyn_Fn_d_Absyn_Raw_decl_struct*)_tmpEC)->f1;_LL4: {struct Cyc_Absyn_Fndecl*fd=_tmpF4;
# 627
if((int)fd->sc == (int)Cyc_Absyn_Static)return accum;if((int)fd->sc == (int)Cyc_Absyn_ExternC)
return accum;{
# 627
struct Cyc_Absyn_Vardecl*vd=
# 629
Cyc_Absyn_new_vardecl(0U,fd->name,(void*)_check_null(fd->cached_type),0);
vd->sc=fd->sc;
return({struct Cyc_List_List*_tmpFA=_cycalloc(sizeof(*_tmpFA));({struct Cyc_Absyn_Decl*_tmp303=Cyc_Absyn_new_decl((void*)({struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct*_tmpF9=_cycalloc(sizeof(*_tmpF9));_tmpF9->tag=0U,_tmpF9->f1=vd;_tmpF9;}),0U);_tmpFA->hd=_tmp303;}),_tmpFA->tl=accum;_tmpFA;});}}case 5U: _LL5: _tmpF3=((struct Cyc_Absyn_Aggr_d_Absyn_Raw_decl_struct*)_tmpEC)->f1;_LL6: {struct Cyc_Absyn_Aggrdecl*a=_tmpF3;
# 634
if((int)a->sc == (int)Cyc_Absyn_ExternC)return accum;goto _LL0;}case 6U: _LL7: _tmpF2=((struct Cyc_Absyn_Datatype_d_Absyn_Raw_decl_struct*)_tmpEC)->f1;_LL8: {struct Cyc_Absyn_Datatypedecl*a=_tmpF2;
# 638
if((int)a->sc == (int)Cyc_Absyn_ExternC)return accum;goto _LL0;}case 7U: _LL9: _tmpF1=((struct Cyc_Absyn_Enum_d_Absyn_Raw_decl_struct*)_tmpEC)->f1;_LLA: {struct Cyc_Absyn_Enumdecl*a=_tmpF1;
# 641
if((int)a->sc == (int)Cyc_Absyn_Static)return accum;if((int)a->sc == (int)Cyc_Absyn_ExternC)
return accum;
# 641
goto _LL0;}case 8U: _LLB: _tmpF0=((struct Cyc_Absyn_Typedef_d_Absyn_Raw_decl_struct*)_tmpEC)->f1;_LLC: {struct Cyc_Absyn_Typedefdecl*a=_tmpF0;
# 645
goto _LL0;}case 9U: _LLD: _tmpEE=((struct Cyc_Absyn_Namespace_d_Absyn_Raw_decl_struct*)_tmpEC)->f1;_tmpEF=((struct Cyc_Absyn_Namespace_d_Absyn_Raw_decl_struct*)_tmpEC)->f2;_LLE: {struct _fat_ptr*a=_tmpEE;struct Cyc_List_List*b=_tmpEF;
# 647
struct Cyc_List_List*l=Cyc_List_rev(({struct Cyc_List_List*(*_tmp304)(struct Cyc_List_List*(*f)(struct Cyc_List_List*,struct Cyc_Absyn_Decl*),struct Cyc_List_List*accum,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmpFD)(struct Cyc_List_List*(*f)(struct Cyc_List_List*,struct Cyc_Absyn_Decl*),struct Cyc_List_List*accum,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_List_List*(*f)(struct Cyc_List_List*,struct Cyc_Absyn_Decl*),struct Cyc_List_List*accum,struct Cyc_List_List*x))Cyc_List_fold_left;_tmpFD;});_tmp304(Cyc_Interface_filterstatics,0,b);}));
return({struct Cyc_List_List*_tmpFC=_cycalloc(sizeof(*_tmpFC));({struct Cyc_Absyn_Decl*_tmp305=Cyc_Absyn_new_decl((void*)({struct Cyc_Absyn_Namespace_d_Absyn_Raw_decl_struct*_tmpFB=_cycalloc(sizeof(*_tmpFB));_tmpFB->tag=9U,_tmpFB->f1=a,_tmpFB->f2=l;_tmpFB;}),0U);_tmpFC->hd=_tmp305;}),_tmpFC->tl=accum;_tmpFC;});}case 10U: _LLF: _tmpED=((struct Cyc_Absyn_Using_d_Absyn_Raw_decl_struct*)_tmpEC)->f2;_LL10: {struct Cyc_List_List*b=_tmpED;
# 650
return({struct Cyc_List_List*_tmp307=({struct Cyc_List_List*(*_tmp306)(struct Cyc_List_List*(*f)(struct Cyc_List_List*,struct Cyc_Absyn_Decl*),struct Cyc_List_List*accum,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmpFE)(struct Cyc_List_List*(*f)(struct Cyc_List_List*,struct Cyc_Absyn_Decl*),struct Cyc_List_List*accum,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_List_List*(*f)(struct Cyc_List_List*,struct Cyc_Absyn_Decl*),struct Cyc_List_List*accum,struct Cyc_List_List*x))Cyc_List_fold_left;_tmpFE;});_tmp306(Cyc_Interface_filterstatics,0,b);});Cyc_List_append(_tmp307,accum);});}case 2U: _LL11: _LL12:
 goto _LL14;case 3U: _LL13: _LL14:
 goto _LL16;case 4U: _LL15: _LL16:
 goto _LL18;case 11U: _LL17: _LL18:
 goto _LL1A;case 12U: _LL19: _LL1A:
 goto _LL1C;case 13U: _LL1B: _LL1C:
 goto _LL1E;case 14U: _LL1D: _LL1E:
 goto _LL20;case 15U: _LL1F: _LL20:
 goto _LL22;default: _LL21: _LL22:
# 660
 return accum;}_LL0:;}
# 662
return({struct Cyc_List_List*_tmp100=_cycalloc(sizeof(*_tmp100));({struct Cyc_Absyn_Decl*_tmp308=({struct Cyc_Absyn_Decl*_tmpFF=_cycalloc(sizeof(*_tmpFF));*_tmpFF=*d;_tmpFF;});_tmp100->hd=_tmp308;}),_tmp100->tl=accum;_tmp100;});}struct _tuple29{struct Cyc_Interface_I*f1;int f2;struct Cyc_Tcenv_Genv*f3;struct Cyc_Interface_Seen*f4;};
# 190
static void Cyc_Interface_extract_f(struct _tuple29*env_f){
# 666
struct Cyc_Interface_Seen*_tmp104;struct Cyc_Tcenv_Genv*_tmp103;int _tmp102;struct Cyc_Interface_I*_tmp101;_LL1: _tmp101=env_f->f1;_tmp102=env_f->f2;_tmp103=env_f->f3;_tmp104=env_f->f4;_LL2: {struct Cyc_Interface_I*i=_tmp101;int check_complete_defs=_tmp102;struct Cyc_Tcenv_Genv*ae=_tmp103;struct Cyc_Interface_Seen*seen=_tmp104;
struct _tuple26 env=({struct _tuple26 _tmp215;_tmp215.f1=i->imports,_tmp215.f2=i->exports,_tmp215.f3=check_complete_defs,_tmp215.f4=ae,_tmp215.f5=seen,_tmp215.f6=i;_tmp215;});
({void(*_tmp309)(void(*f)(struct _tuple26*,struct _tuple0*,struct Cyc_Absyn_Aggrdecl**),struct _tuple26*env,struct Cyc_Dict_Dict d)=({void(*_tmp105)(void(*f)(struct _tuple26*,struct _tuple0*,struct Cyc_Absyn_Aggrdecl**),struct _tuple26*env,struct Cyc_Dict_Dict d)=(void(*)(void(*f)(struct _tuple26*,struct _tuple0*,struct Cyc_Absyn_Aggrdecl**),struct _tuple26*env,struct Cyc_Dict_Dict d))Cyc_Dict_iter_c;_tmp105;});_tmp309(Cyc_Interface_extract_aggrdecl,& env,ae->aggrdecls);});
({void(*_tmp30A)(void(*f)(struct _tuple26*,struct _tuple0*,struct Cyc_Absyn_Datatypedecl**),struct _tuple26*env,struct Cyc_Dict_Dict d)=({void(*_tmp106)(void(*f)(struct _tuple26*,struct _tuple0*,struct Cyc_Absyn_Datatypedecl**),struct _tuple26*env,struct Cyc_Dict_Dict d)=(void(*)(void(*f)(struct _tuple26*,struct _tuple0*,struct Cyc_Absyn_Datatypedecl**),struct _tuple26*env,struct Cyc_Dict_Dict d))Cyc_Dict_iter_c;_tmp106;});_tmp30A(Cyc_Interface_extract_datatypedecl,& env,ae->datatypedecls);});
({void(*_tmp30B)(void(*f)(struct _tuple26*,struct _tuple0*,struct Cyc_Absyn_Enumdecl**),struct _tuple26*env,struct Cyc_Dict_Dict d)=({void(*_tmp107)(void(*f)(struct _tuple26*,struct _tuple0*,struct Cyc_Absyn_Enumdecl**),struct _tuple26*env,struct Cyc_Dict_Dict d)=(void(*)(void(*f)(struct _tuple26*,struct _tuple0*,struct Cyc_Absyn_Enumdecl**),struct _tuple26*env,struct Cyc_Dict_Dict d))Cyc_Dict_iter_c;_tmp107;});_tmp30B(Cyc_Interface_extract_enumdecl,& env,ae->enumdecls);});
({void(*_tmp30C)(void(*f)(struct _tuple26*,struct _tuple0*,struct Cyc_Absyn_Typedefdecl*),struct _tuple26*env,struct Cyc_Dict_Dict d)=({void(*_tmp108)(void(*f)(struct _tuple26*,struct _tuple0*,struct Cyc_Absyn_Typedefdecl*),struct _tuple26*env,struct Cyc_Dict_Dict d)=(void(*)(void(*f)(struct _tuple26*,struct _tuple0*,struct Cyc_Absyn_Typedefdecl*),struct _tuple26*env,struct Cyc_Dict_Dict d))Cyc_Dict_iter_c;_tmp108;});_tmp30C(Cyc_Interface_extract_typedef,& env,ae->typedefs);});
({void(*_tmp30D)(void(*f)(struct _tuple26*,struct _tuple0*,struct _tuple27*),struct _tuple26*env,struct Cyc_Dict_Dict d)=({void(*_tmp109)(void(*f)(struct _tuple26*,struct _tuple0*,struct _tuple27*),struct _tuple26*env,struct Cyc_Dict_Dict d)=(void(*)(void(*f)(struct _tuple26*,struct _tuple0*,struct _tuple27*),struct _tuple26*env,struct Cyc_Dict_Dict d))Cyc_Dict_iter_c;_tmp109;});_tmp30D(Cyc_Interface_extract_ordinarie,& env,ae->ordinaries);});}}
# 190
static struct Cyc_Interface_I*Cyc_Interface_gen_extract(struct Cyc_Tcenv_Genv*ae,int check_complete_defs,struct Cyc_List_List*tds){
# 676
struct _tuple29 env=({struct _tuple29 _tmp216;({
struct Cyc_Interface_I*_tmp30F=Cyc_Interface_empty();_tmp216.f1=_tmp30F;}),_tmp216.f2=check_complete_defs,_tmp216.f3=ae,({struct Cyc_Interface_Seen*_tmp30E=Cyc_Interface_new_seen();_tmp216.f4=_tmp30E;});_tmp216;});
Cyc_Interface_extract_f(& env);{
struct Cyc_Interface_I*i=env.f1;
({struct Cyc_List_List*_tmp311=Cyc_List_rev(({struct Cyc_List_List*(*_tmp310)(struct Cyc_List_List*(*f)(struct Cyc_List_List*,struct Cyc_Absyn_Decl*),struct Cyc_List_List*accum,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp10A)(struct Cyc_List_List*(*f)(struct Cyc_List_List*,struct Cyc_Absyn_Decl*),struct Cyc_List_List*accum,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_List_List*(*f)(struct Cyc_List_List*,struct Cyc_Absyn_Decl*),struct Cyc_List_List*accum,struct Cyc_List_List*x))Cyc_List_fold_left;_tmp10A;});_tmp310(Cyc_Interface_filterstatics,0,tds);}));i->tds=_tmp311;});
({struct Cyc_List_List*_tmp313=Cyc_List_rev(({struct Cyc_List_List*(*_tmp312)(struct Cyc_List_List*(*f)(struct Cyc_List_List*,struct Cyc_Absyn_Decl*),struct Cyc_List_List*accum,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp10B)(struct Cyc_List_List*(*f)(struct Cyc_List_List*,struct Cyc_Absyn_Decl*),struct Cyc_List_List*accum,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_List_List*(*f)(struct Cyc_List_List*,struct Cyc_Absyn_Decl*),struct Cyc_List_List*accum,struct Cyc_List_List*x))Cyc_List_fold_left;_tmp10B;});_tmp312(Cyc_Interface_uniqify_decl_list,0,i->tds);}));i->tds=_tmp313;});
return i;}}
# 190
struct Cyc_Interface_I*Cyc_Interface_extract(struct Cyc_Tcenv_Genv*ae,struct Cyc_List_List*tds){
# 686
return Cyc_Interface_gen_extract(ae,1,tds);}
# 190
inline static void Cyc_Interface_check_err(struct _fat_ptr*msg1,struct _fat_ptr msg2){
# 692
({struct Cyc_String_pa_PrintArg_struct _tmp10E=({struct Cyc_String_pa_PrintArg_struct _tmp217;_tmp217.tag=0U,_tmp217.f1=(struct _fat_ptr)((struct _fat_ptr)msg2);_tmp217;});void*_tmp10C[1U];_tmp10C[0]=& _tmp10E;({struct _fat_ptr*_tmp315=msg1;struct _fat_ptr _tmp314=({const char*_tmp10D="%s";_tag_fat(_tmp10D,sizeof(char),3U);});Cyc_Tcdecl_merr(0U,_tmp315,_tmp314,_tag_fat(_tmp10C,sizeof(void*),1U));});});}struct _tuple30{int f1;struct Cyc_Dict_Dict f2;int(*f3)(void*,void*,struct _fat_ptr*);struct _fat_ptr f4;struct _fat_ptr*f5;};
# 190
static void Cyc_Interface_incl_dict_f(struct _tuple30*env,struct _tuple0*x,void*y1){
# 702
struct _fat_ptr*_tmp113;struct _fat_ptr _tmp112;int(*_tmp111)(void*,void*,struct _fat_ptr*);struct Cyc_Dict_Dict _tmp110;int*_tmp10F;_LL1: _tmp10F=(int*)& env->f1;_tmp110=env->f2;_tmp111=env->f3;_tmp112=env->f4;_tmp113=env->f5;_LL2: {int*res=_tmp10F;struct Cyc_Dict_Dict dic2=_tmp110;int(*incl_f)(void*,void*,struct _fat_ptr*)=_tmp111;struct _fat_ptr t=_tmp112;struct _fat_ptr*msg=_tmp113;
struct _handler_cons _tmp114;_push_handler(& _tmp114);{int _tmp116=0;if(setjmp(_tmp114.handler))_tmp116=1;if(!_tmp116){
{void*y2=({void*(*_tmp317)(struct Cyc_Dict_Dict d,struct _tuple0*k)=({void*(*_tmp117)(struct Cyc_Dict_Dict d,struct _tuple0*k)=(void*(*)(struct Cyc_Dict_Dict d,struct _tuple0*k))Cyc_Dict_lookup;_tmp117;});struct Cyc_Dict_Dict _tmp316=dic2;_tmp317(_tmp316,x);});
if(!incl_f(y1,y2,msg))*res=0;}
# 704
;_pop_handler();}else{void*_tmp115=(void*)Cyc_Core_get_exn_thrown();void*_tmp118=_tmp115;void*_tmp119;if(((struct Cyc_Dict_Absent_exn_struct*)_tmp118)->tag == Cyc_Dict_Absent){_LL4: _LL5:
# 708
({struct _fat_ptr*_tmp31A=msg;Cyc_Interface_check_err(_tmp31A,(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp11C=({struct Cyc_String_pa_PrintArg_struct _tmp219;_tmp219.tag=0U,_tmp219.f1=(struct _fat_ptr)((struct _fat_ptr)t);_tmp219;});struct Cyc_String_pa_PrintArg_struct _tmp11D=({struct Cyc_String_pa_PrintArg_struct _tmp218;_tmp218.tag=0U,({struct _fat_ptr _tmp318=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(x));_tmp218.f1=_tmp318;});_tmp218;});void*_tmp11A[2U];_tmp11A[0]=& _tmp11C,_tmp11A[1]=& _tmp11D;({struct _fat_ptr _tmp319=({const char*_tmp11B="%s %s is missing";_tag_fat(_tmp11B,sizeof(char),17U);});Cyc_aprintf(_tmp319,_tag_fat(_tmp11A,sizeof(void*),2U));});}));});
*res=0;
goto _LL3;}else{_LL6: _tmp119=_tmp118;_LL7: {void*exn=_tmp119;(int)_rethrow(exn);}}_LL3:;}}}}
# 190
static int Cyc_Interface_incl_dict(struct Cyc_Dict_Dict dic1,struct Cyc_Dict_Dict dic2,int(*incl_f)(void*,void*,struct _fat_ptr*),struct _fat_ptr t,struct _fat_ptr*msg){
# 719
struct _tuple30 env=({struct _tuple30 _tmp21A;_tmp21A.f1=1,_tmp21A.f2=dic2,_tmp21A.f3=incl_f,_tmp21A.f4=t,_tmp21A.f5=msg;_tmp21A;});
({void(*_tmp31B)(void(*f)(struct _tuple30*,struct _tuple0*,void*),struct _tuple30*env,struct Cyc_Dict_Dict d)=({void(*_tmp11E)(void(*f)(struct _tuple30*,struct _tuple0*,void*),struct _tuple30*env,struct Cyc_Dict_Dict d)=(void(*)(void(*f)(struct _tuple30*,struct _tuple0*,void*),struct _tuple30*env,struct Cyc_Dict_Dict d))Cyc_Dict_iter_c;_tmp11E;});_tmp31B(Cyc_Interface_incl_dict_f,& env,dic1);});
return env.f1;}
# 190
static int Cyc_Interface_incl_aggrdecl(struct Cyc_Absyn_Aggrdecl*d0,struct Cyc_Absyn_Aggrdecl*d1,struct _fat_ptr*msg){
# 736 "interface.cyc"
struct Cyc_Absyn_Aggrdecl*d=Cyc_Tcdecl_merge_aggrdecl(d0,d1,0U,msg);if(d == 0)return 0;if(d0 != d){({struct _fat_ptr*_tmp31E=msg;Cyc_Interface_check_err(_tmp31E,(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp121=({struct Cyc_String_pa_PrintArg_struct _tmp21B;_tmp21B.tag=0U,({struct _fat_ptr _tmp31C=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(d1->name));_tmp21B.f1=_tmp31C;});_tmp21B;});void*_tmp11F[1U];_tmp11F[0]=& _tmp121;({struct _fat_ptr _tmp31D=({const char*_tmp120="declaration of type %s discloses too much information";_tag_fat(_tmp120,sizeof(char),54U);});Cyc_aprintf(_tmp31D,_tag_fat(_tmp11F,sizeof(void*),1U));});}));});return 0;}return 1;}
# 190 "interface.cyc"
static int Cyc_Interface_incl_datatypedecl(struct Cyc_Absyn_Datatypedecl*d0,struct Cyc_Absyn_Datatypedecl*d1,struct _fat_ptr*msg){
# 738 "interface.cyc"
struct Cyc_Absyn_Datatypedecl*d=Cyc_Tcdecl_merge_datatypedecl(d0,d1,0U,msg);if(d == 0)return 0;if(d0 != d){({struct _fat_ptr*_tmp321=msg;Cyc_Interface_check_err(_tmp321,(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp124=({struct Cyc_String_pa_PrintArg_struct _tmp21C;_tmp21C.tag=0U,({struct _fat_ptr _tmp31F=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(d1->name));_tmp21C.f1=_tmp31F;});_tmp21C;});void*_tmp122[1U];_tmp122[0]=& _tmp124;({struct _fat_ptr _tmp320=({const char*_tmp123="declaration of datatype %s discloses too much information";_tag_fat(_tmp123,sizeof(char),58U);});Cyc_aprintf(_tmp320,_tag_fat(_tmp122,sizeof(void*),1U));});}));});return 0;}return 1;}
# 190 "interface.cyc"
static int Cyc_Interface_incl_enumdecl(struct Cyc_Absyn_Enumdecl*d0,struct Cyc_Absyn_Enumdecl*d1,struct _fat_ptr*msg){
# 740 "interface.cyc"
struct Cyc_Absyn_Enumdecl*d=Cyc_Tcdecl_merge_enumdecl(d0,d1,0U,msg);if(d == 0)return 0;if(d0 != d){({struct _fat_ptr*_tmp324=msg;Cyc_Interface_check_err(_tmp324,(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp127=({struct Cyc_String_pa_PrintArg_struct _tmp21D;_tmp21D.tag=0U,({struct _fat_ptr _tmp322=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(d1->name));_tmp21D.f1=_tmp322;});_tmp21D;});void*_tmp125[1U];_tmp125[0]=& _tmp127;({struct _fat_ptr _tmp323=({const char*_tmp126="declaration of enum %s discloses too much information";_tag_fat(_tmp126,sizeof(char),54U);});Cyc_aprintf(_tmp323,_tag_fat(_tmp125,sizeof(void*),1U));});}));});return 0;}return 1;}
# 190 "interface.cyc"
static int Cyc_Interface_incl_vardecl(struct Cyc_Absyn_Vardecl*d0,struct Cyc_Absyn_Vardecl*d1,struct _fat_ptr*msg){
# 742 "interface.cyc"
struct Cyc_Absyn_Vardecl*d=Cyc_Tcdecl_merge_vardecl(d0,d1,0U,msg);if(d == 0)return 0;if(d0 != d){({struct _fat_ptr*_tmp327=msg;Cyc_Interface_check_err(_tmp327,(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp12A=({struct Cyc_String_pa_PrintArg_struct _tmp21E;_tmp21E.tag=0U,({struct _fat_ptr _tmp325=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(d1->name));_tmp21E.f1=_tmp325;});_tmp21E;});void*_tmp128[1U];_tmp128[0]=& _tmp12A;({struct _fat_ptr _tmp326=({const char*_tmp129="declaration of variable %s discloses too much information";_tag_fat(_tmp129,sizeof(char),58U);});Cyc_aprintf(_tmp326,_tag_fat(_tmp128,sizeof(void*),1U));});}));});return 0;}return 1;}
# 190 "interface.cyc"
static int Cyc_Interface_incl_typedefdecl(struct Cyc_Absyn_Typedefdecl*d0,struct Cyc_Absyn_Typedefdecl*d1,struct _fat_ptr*msg){
# 744 "interface.cyc"
struct Cyc_Absyn_Typedefdecl*d=Cyc_Tcdecl_merge_typedefdecl(d0,d1,0U,msg);if(d == 0)return 0;if(d0 != d){({struct _fat_ptr*_tmp32A=msg;Cyc_Interface_check_err(_tmp32A,(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp12D=({struct Cyc_String_pa_PrintArg_struct _tmp21F;_tmp21F.tag=0U,({struct _fat_ptr _tmp328=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(d1->name));_tmp21F.f1=_tmp328;});_tmp21F;});void*_tmp12B[1U];_tmp12B[0]=& _tmp12D;({struct _fat_ptr _tmp329=({const char*_tmp12C="declaration of typedef %s discloses too much information";_tag_fat(_tmp12C,sizeof(char),57U);});Cyc_aprintf(_tmp329,_tag_fat(_tmp12B,sizeof(void*),1U));});}));});return 0;}return 1;}
# 190 "interface.cyc"
static int Cyc_Interface_incl_xdatatypefielddecl(struct Cyc_Tcdecl_Xdatatypefielddecl*d0,struct Cyc_Tcdecl_Xdatatypefielddecl*d1,struct _fat_ptr*msg){
# 746 "interface.cyc"
struct Cyc_Tcdecl_Xdatatypefielddecl*d=Cyc_Tcdecl_merge_xdatatypefielddecl(d0,d1,0U,msg);if(d == 0)return 0;if(d0 != d){({struct _fat_ptr*_tmp32D=msg;Cyc_Interface_check_err(_tmp32D,(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp130=({struct Cyc_String_pa_PrintArg_struct _tmp220;_tmp220.tag=0U,({struct _fat_ptr _tmp32B=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string((d1->field)->name));_tmp220.f1=_tmp32B;});_tmp220;});void*_tmp12E[1U];_tmp12E[0]=& _tmp130;({struct _fat_ptr _tmp32C=({const char*_tmp12F="declaration of xdatatypefield %s discloses too much information";_tag_fat(_tmp12F,sizeof(char),64U);});Cyc_aprintf(_tmp32C,_tag_fat(_tmp12E,sizeof(void*),1U));});}));});return 0;}return 1;}
# 190 "interface.cyc"
struct Cyc_Dict_Dict*Cyc_Interface_compat_merge_dict(struct Cyc_Dict_Dict dic1,struct Cyc_Dict_Dict dic2,struct Cyc_Dict_Dict excl,void*(*merge_f)(void*,void*,unsigned,struct _fat_ptr*),struct _fat_ptr t,struct _fat_ptr*msg);
# 760 "interface.cyc"
static int Cyc_Interface_incl_ienv(struct Cyc_Interface_Ienv*ie1,struct Cyc_Interface_Ienv*ie2,struct _fat_ptr*msg){
int r1=({int(*_tmp331)(struct Cyc_Dict_Dict dic1,struct Cyc_Dict_Dict dic2,int(*incl_f)(struct Cyc_Absyn_Aggrdecl*,struct Cyc_Absyn_Aggrdecl*,struct _fat_ptr*),struct _fat_ptr t,struct _fat_ptr*msg)=({int(*_tmp13C)(struct Cyc_Dict_Dict dic1,struct Cyc_Dict_Dict dic2,int(*incl_f)(struct Cyc_Absyn_Aggrdecl*,struct Cyc_Absyn_Aggrdecl*,struct _fat_ptr*),struct _fat_ptr t,struct _fat_ptr*msg)=(int(*)(struct Cyc_Dict_Dict dic1,struct Cyc_Dict_Dict dic2,int(*incl_f)(struct Cyc_Absyn_Aggrdecl*,struct Cyc_Absyn_Aggrdecl*,struct _fat_ptr*),struct _fat_ptr t,struct _fat_ptr*msg))Cyc_Interface_incl_dict;_tmp13C;});struct Cyc_Dict_Dict _tmp330=ie1->aggrdecls;struct Cyc_Dict_Dict _tmp32F=ie2->aggrdecls;struct _fat_ptr _tmp32E=({const char*_tmp13D="type";_tag_fat(_tmp13D,sizeof(char),5U);});_tmp331(_tmp330,_tmp32F,Cyc_Interface_incl_aggrdecl,_tmp32E,msg);});
int r2=({int(*_tmp335)(struct Cyc_Dict_Dict dic1,struct Cyc_Dict_Dict dic2,int(*incl_f)(struct Cyc_Absyn_Datatypedecl*,struct Cyc_Absyn_Datatypedecl*,struct _fat_ptr*),struct _fat_ptr t,struct _fat_ptr*msg)=({int(*_tmp13A)(struct Cyc_Dict_Dict dic1,struct Cyc_Dict_Dict dic2,int(*incl_f)(struct Cyc_Absyn_Datatypedecl*,struct Cyc_Absyn_Datatypedecl*,struct _fat_ptr*),struct _fat_ptr t,struct _fat_ptr*msg)=(int(*)(struct Cyc_Dict_Dict dic1,struct Cyc_Dict_Dict dic2,int(*incl_f)(struct Cyc_Absyn_Datatypedecl*,struct Cyc_Absyn_Datatypedecl*,struct _fat_ptr*),struct _fat_ptr t,struct _fat_ptr*msg))Cyc_Interface_incl_dict;_tmp13A;});struct Cyc_Dict_Dict _tmp334=ie1->datatypedecls;struct Cyc_Dict_Dict _tmp333=ie2->datatypedecls;struct _fat_ptr _tmp332=({const char*_tmp13B="datatype";_tag_fat(_tmp13B,sizeof(char),9U);});_tmp335(_tmp334,_tmp333,Cyc_Interface_incl_datatypedecl,_tmp332,msg);});
int r3=({int(*_tmp339)(struct Cyc_Dict_Dict dic1,struct Cyc_Dict_Dict dic2,int(*incl_f)(struct Cyc_Absyn_Enumdecl*,struct Cyc_Absyn_Enumdecl*,struct _fat_ptr*),struct _fat_ptr t,struct _fat_ptr*msg)=({int(*_tmp138)(struct Cyc_Dict_Dict dic1,struct Cyc_Dict_Dict dic2,int(*incl_f)(struct Cyc_Absyn_Enumdecl*,struct Cyc_Absyn_Enumdecl*,struct _fat_ptr*),struct _fat_ptr t,struct _fat_ptr*msg)=(int(*)(struct Cyc_Dict_Dict dic1,struct Cyc_Dict_Dict dic2,int(*incl_f)(struct Cyc_Absyn_Enumdecl*,struct Cyc_Absyn_Enumdecl*,struct _fat_ptr*),struct _fat_ptr t,struct _fat_ptr*msg))Cyc_Interface_incl_dict;_tmp138;});struct Cyc_Dict_Dict _tmp338=ie1->enumdecls;struct Cyc_Dict_Dict _tmp337=ie2->enumdecls;struct _fat_ptr _tmp336=({const char*_tmp139="enum";_tag_fat(_tmp139,sizeof(char),5U);});_tmp339(_tmp338,_tmp337,Cyc_Interface_incl_enumdecl,_tmp336,msg);});
# 765
int r4=({struct Cyc_Dict_Dict*(*_tmp33E)(struct Cyc_Dict_Dict dic1,struct Cyc_Dict_Dict dic2,struct Cyc_Dict_Dict excl,struct Cyc_Absyn_Typedefdecl*(*merge_f)(struct Cyc_Absyn_Typedefdecl*,struct Cyc_Absyn_Typedefdecl*,unsigned,struct _fat_ptr*),struct _fat_ptr t,struct _fat_ptr*msg)=({struct Cyc_Dict_Dict*(*_tmp135)(struct Cyc_Dict_Dict dic1,struct Cyc_Dict_Dict dic2,struct Cyc_Dict_Dict excl,struct Cyc_Absyn_Typedefdecl*(*merge_f)(struct Cyc_Absyn_Typedefdecl*,struct Cyc_Absyn_Typedefdecl*,unsigned,struct _fat_ptr*),struct _fat_ptr t,struct _fat_ptr*msg)=(struct Cyc_Dict_Dict*(*)(struct Cyc_Dict_Dict dic1,struct Cyc_Dict_Dict dic2,struct Cyc_Dict_Dict excl,struct Cyc_Absyn_Typedefdecl*(*merge_f)(struct Cyc_Absyn_Typedefdecl*,struct Cyc_Absyn_Typedefdecl*,unsigned,struct _fat_ptr*),struct _fat_ptr t,struct _fat_ptr*msg))Cyc_Interface_compat_merge_dict;_tmp135;});struct Cyc_Dict_Dict _tmp33D=ie1->typedefdecls;struct Cyc_Dict_Dict _tmp33C=ie2->typedefdecls;struct Cyc_Dict_Dict _tmp33B=
({struct Cyc_Dict_Dict(*_tmp136)(int(*cmp)(struct _tuple0*,struct _tuple0*))=(struct Cyc_Dict_Dict(*)(int(*cmp)(struct _tuple0*,struct _tuple0*)))Cyc_Dict_empty;_tmp136;})(Cyc_Absyn_qvar_cmp);
# 765
struct _fat_ptr _tmp33A=({const char*_tmp137="typedef";_tag_fat(_tmp137,sizeof(char),8U);});_tmp33E(_tmp33D,_tmp33C,_tmp33B,Cyc_Tcdecl_merge_typedefdecl,_tmp33A,msg);})!= 0;
# 768
int r5=({int(*_tmp342)(struct Cyc_Dict_Dict dic1,struct Cyc_Dict_Dict dic2,int(*incl_f)(struct Cyc_Absyn_Vardecl*,struct Cyc_Absyn_Vardecl*,struct _fat_ptr*),struct _fat_ptr t,struct _fat_ptr*msg)=({int(*_tmp133)(struct Cyc_Dict_Dict dic1,struct Cyc_Dict_Dict dic2,int(*incl_f)(struct Cyc_Absyn_Vardecl*,struct Cyc_Absyn_Vardecl*,struct _fat_ptr*),struct _fat_ptr t,struct _fat_ptr*msg)=(int(*)(struct Cyc_Dict_Dict dic1,struct Cyc_Dict_Dict dic2,int(*incl_f)(struct Cyc_Absyn_Vardecl*,struct Cyc_Absyn_Vardecl*,struct _fat_ptr*),struct _fat_ptr t,struct _fat_ptr*msg))Cyc_Interface_incl_dict;_tmp133;});struct Cyc_Dict_Dict _tmp341=ie1->vardecls;struct Cyc_Dict_Dict _tmp340=ie2->vardecls;struct _fat_ptr _tmp33F=({const char*_tmp134="variable";_tag_fat(_tmp134,sizeof(char),9U);});_tmp342(_tmp341,_tmp340,Cyc_Interface_incl_vardecl,_tmp33F,msg);});
int r6=({int(*_tmp346)(struct Cyc_Dict_Dict dic1,struct Cyc_Dict_Dict dic2,int(*incl_f)(struct Cyc_Tcdecl_Xdatatypefielddecl*,struct Cyc_Tcdecl_Xdatatypefielddecl*,struct _fat_ptr*),struct _fat_ptr t,struct _fat_ptr*msg)=({int(*_tmp131)(struct Cyc_Dict_Dict dic1,struct Cyc_Dict_Dict dic2,int(*incl_f)(struct Cyc_Tcdecl_Xdatatypefielddecl*,struct Cyc_Tcdecl_Xdatatypefielddecl*,struct _fat_ptr*),struct _fat_ptr t,struct _fat_ptr*msg)=(int(*)(struct Cyc_Dict_Dict dic1,struct Cyc_Dict_Dict dic2,int(*incl_f)(struct Cyc_Tcdecl_Xdatatypefielddecl*,struct Cyc_Tcdecl_Xdatatypefielddecl*,struct _fat_ptr*),struct _fat_ptr t,struct _fat_ptr*msg))Cyc_Interface_incl_dict;_tmp131;});struct Cyc_Dict_Dict _tmp345=ie1->xdatatypefielddecls;struct Cyc_Dict_Dict _tmp344=ie2->xdatatypefielddecls;struct _fat_ptr _tmp343=({const char*_tmp132="xdatatypefield";_tag_fat(_tmp132,sizeof(char),15U);});_tmp346(_tmp345,_tmp344,Cyc_Interface_incl_xdatatypefielddecl,_tmp343,msg);});
return((((r1 && r2)&& r3)&& r4)&& r5)&& r6;}
# 760
int Cyc_Interface_is_subinterface(struct Cyc_Interface_I*i1,struct Cyc_Interface_I*i2,struct _tuple11*info){
# 774
struct _handler_cons _tmp13E;_push_handler(& _tmp13E);{int _tmp140=0;if(setjmp(_tmp13E.handler))_tmp140=1;if(!_tmp140){
{int res=1;
struct _fat_ptr*msg=0;
# 778
if(info != 0)
msg=({struct _fat_ptr*_tmp145=_cycalloc(sizeof(*_tmp145));({struct _fat_ptr _tmp348=(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp143=({struct Cyc_String_pa_PrintArg_struct _tmp222;_tmp222.tag=0U,_tmp222.f1=(struct _fat_ptr)((struct _fat_ptr)(*info).f1);_tmp222;});struct Cyc_String_pa_PrintArg_struct _tmp144=({struct Cyc_String_pa_PrintArg_struct _tmp221;_tmp221.tag=0U,_tmp221.f1=(struct _fat_ptr)((struct _fat_ptr)(*info).f2);_tmp221;});void*_tmp141[2U];_tmp141[0]=& _tmp143,_tmp141[1]=& _tmp144;({struct _fat_ptr _tmp347=({const char*_tmp142="checking inclusion of %s exports into %s exports,";_tag_fat(_tmp142,sizeof(char),50U);});Cyc_aprintf(_tmp347,_tag_fat(_tmp141,sizeof(void*),2U));});});*_tmp145=_tmp348;});_tmp145;});
# 778
if(!
# 780
Cyc_Interface_incl_ienv(i1->exports,i2->exports,msg))res=0;
# 778
if(info != 0)
# 783
msg=({struct _fat_ptr*_tmp14A=_cycalloc(sizeof(*_tmp14A));({struct _fat_ptr _tmp34A=(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp148=({struct Cyc_String_pa_PrintArg_struct _tmp224;_tmp224.tag=0U,_tmp224.f1=(struct _fat_ptr)((struct _fat_ptr)(*info).f2);_tmp224;});struct Cyc_String_pa_PrintArg_struct _tmp149=({struct Cyc_String_pa_PrintArg_struct _tmp223;_tmp223.tag=0U,_tmp223.f1=(struct _fat_ptr)((struct _fat_ptr)(*info).f1);_tmp223;});void*_tmp146[2U];_tmp146[0]=& _tmp148,_tmp146[1]=& _tmp149;({struct _fat_ptr _tmp349=({const char*_tmp147="checking inclusion of %s imports into %s imports,";_tag_fat(_tmp147,sizeof(char),50U);});Cyc_aprintf(_tmp349,_tag_fat(_tmp146,sizeof(void*),2U));});});*_tmp14A=_tmp34A;});_tmp14A;});
# 778
if(!
# 784
Cyc_Interface_incl_ienv(i2->imports,i1->imports,msg))res=0;{
# 778
int _tmp14B=res;_npop_handler(0U);return _tmp14B;}}
# 775
;_pop_handler();}else{void*_tmp13F=(void*)Cyc_Core_get_exn_thrown();void*_tmp14C=_tmp13F;void*_tmp14D;if(((struct Cyc_Tcdecl_Incompatible_exn_struct*)_tmp14C)->tag == Cyc_Tcdecl_Incompatible){_LL1: _LL2:
# 788
 return 0;}else{_LL3: _tmp14D=_tmp14C;_LL4: {void*exn=_tmp14D;(int)_rethrow(exn);}}_LL0:;}}}struct _tuple31{int f1;struct Cyc_Dict_Dict f2;struct Cyc_Dict_Dict f3;struct Cyc_Dict_Dict f4;void*(*f5)(void*,void*,unsigned,struct _fat_ptr*);struct _fat_ptr f6;struct _fat_ptr*f7;};
# 760
void Cyc_Interface_compat_merge_dict_f(struct _tuple31*env,struct _tuple0*x,void*y2){
# 803
struct _fat_ptr*_tmp154;struct _fat_ptr _tmp153;void*(*_tmp152)(void*,void*,unsigned,struct _fat_ptr*);struct Cyc_Dict_Dict _tmp151;struct Cyc_Dict_Dict _tmp150;struct Cyc_Dict_Dict*_tmp14F;int*_tmp14E;_LL1: _tmp14E=(int*)& env->f1;_tmp14F=(struct Cyc_Dict_Dict*)& env->f2;_tmp150=env->f3;_tmp151=env->f4;_tmp152=env->f5;_tmp153=env->f6;_tmp154=env->f7;_LL2: {int*res=_tmp14E;struct Cyc_Dict_Dict*res_dic=_tmp14F;struct Cyc_Dict_Dict dic1=_tmp150;struct Cyc_Dict_Dict excl=_tmp151;void*(*merge_f)(void*,void*,unsigned,struct _fat_ptr*)=_tmp152;struct _fat_ptr t=_tmp153;struct _fat_ptr*msg=_tmp154;
void*y;
{struct _handler_cons _tmp155;_push_handler(& _tmp155);{int _tmp157=0;if(setjmp(_tmp155.handler))_tmp157=1;if(!_tmp157){
{void*y1=({void*(*_tmp34C)(struct Cyc_Dict_Dict d,struct _tuple0*k)=({void*(*_tmp158)(struct Cyc_Dict_Dict d,struct _tuple0*k)=(void*(*)(struct Cyc_Dict_Dict d,struct _tuple0*k))Cyc_Dict_lookup;_tmp158;});struct Cyc_Dict_Dict _tmp34B=dic1;_tmp34C(_tmp34B,x);});
# 809
void*yt=merge_f(y1,y2,0U,msg);
if(!((unsigned)yt)){
*res=0;
_npop_handler(0U);return;}
# 810
y=yt;}
# 806
;_pop_handler();}else{void*_tmp156=(void*)Cyc_Core_get_exn_thrown();void*_tmp159=_tmp156;void*_tmp15A;if(((struct Cyc_Dict_Absent_exn_struct*)_tmp159)->tag == Cyc_Dict_Absent){_LL4: _LL5:
# 817
 y=y2;
goto _LL3;}else{_LL6: _tmp15A=_tmp159;_LL7: {void*exn=_tmp15A;(int)_rethrow(exn);}}_LL3:;}}}{
# 821
struct _handler_cons _tmp15B;_push_handler(& _tmp15B);{int _tmp15D=0;if(setjmp(_tmp15B.handler))_tmp15D=1;if(!_tmp15D){
{void*ye=({void*(*_tmp34E)(struct Cyc_Dict_Dict d,struct _tuple0*k)=({void*(*_tmp162)(struct Cyc_Dict_Dict d,struct _tuple0*k)=(void*(*)(struct Cyc_Dict_Dict d,struct _tuple0*k))Cyc_Dict_lookup;_tmp162;});struct Cyc_Dict_Dict _tmp34D=excl;_tmp34E(_tmp34D,x);});
# 826
void*yt=merge_f(ye,y,0U,msg);
if(yt != ye){
if((unsigned)yt)
({struct _fat_ptr*_tmp351=msg;Cyc_Interface_check_err(_tmp351,(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp160=({struct Cyc_String_pa_PrintArg_struct _tmp226;_tmp226.tag=0U,_tmp226.f1=(struct _fat_ptr)((struct _fat_ptr)t);_tmp226;});struct Cyc_String_pa_PrintArg_struct _tmp161=({struct Cyc_String_pa_PrintArg_struct _tmp225;_tmp225.tag=0U,({
struct _fat_ptr _tmp34F=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(x));_tmp225.f1=_tmp34F;});_tmp225;});void*_tmp15E[2U];_tmp15E[0]=& _tmp160,_tmp15E[1]=& _tmp161;({struct _fat_ptr _tmp350=({const char*_tmp15F="abstract %s %s is being imported as non-abstract";_tag_fat(_tmp15F,sizeof(char),49U);});Cyc_aprintf(_tmp350,_tag_fat(_tmp15E,sizeof(void*),2U));});}));});
# 828
*res=0;}}
# 822
;_pop_handler();}else{void*_tmp15C=(void*)Cyc_Core_get_exn_thrown();void*_tmp163=_tmp15C;void*_tmp164;if(((struct Cyc_Dict_Absent_exn_struct*)_tmp163)->tag == Cyc_Dict_Absent){_LL9: _LLA:
# 835
 if(*res)
({struct Cyc_Dict_Dict _tmp355=({struct Cyc_Dict_Dict(*_tmp354)(struct Cyc_Dict_Dict d,struct _tuple0*k,void*v)=({struct Cyc_Dict_Dict(*_tmp165)(struct Cyc_Dict_Dict d,struct _tuple0*k,void*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _tuple0*k,void*v))Cyc_Dict_insert;_tmp165;});struct Cyc_Dict_Dict _tmp353=*res_dic;struct _tuple0*_tmp352=x;_tmp354(_tmp353,_tmp352,y);});*res_dic=_tmp355;});
# 835
goto _LL8;}else{_LLB: _tmp164=_tmp163;_LLC: {void*exn=_tmp164;(int)_rethrow(exn);}}_LL8:;}}}}}
# 760
struct Cyc_Dict_Dict*Cyc_Interface_compat_merge_dict(struct Cyc_Dict_Dict dic1,struct Cyc_Dict_Dict dic2,struct Cyc_Dict_Dict excl,void*(*merge_f)(void*,void*,unsigned,struct _fat_ptr*),struct _fat_ptr t,struct _fat_ptr*msg){
# 849
struct _tuple31 env=({struct _tuple31 _tmp227;_tmp227.f1=1,_tmp227.f2=dic1,_tmp227.f3=dic1,_tmp227.f4=excl,_tmp227.f5=merge_f,_tmp227.f6=t,_tmp227.f7=msg;_tmp227;});
# 851
({void(*_tmp356)(void(*f)(struct _tuple31*,struct _tuple0*,void*),struct _tuple31*env,struct Cyc_Dict_Dict d)=({void(*_tmp166)(void(*f)(struct _tuple31*,struct _tuple0*,void*),struct _tuple31*env,struct Cyc_Dict_Dict d)=(void(*)(void(*f)(struct _tuple31*,struct _tuple0*,void*),struct _tuple31*env,struct Cyc_Dict_Dict d))Cyc_Dict_iter_c;_tmp166;});_tmp356(Cyc_Interface_compat_merge_dict_f,& env,dic2);});
# 853
if(env.f1)
return({struct Cyc_Dict_Dict*_tmp167=_cycalloc(sizeof(*_tmp167));*_tmp167=env.f2;_tmp167;});
# 853
return 0;}
# 760
struct Cyc_Interface_Ienv*Cyc_Interface_compat_merge_ienv(struct Cyc_Interface_Ienv*ie1,struct Cyc_Interface_Ienv*ie2,struct Cyc_Interface_Ienv*iexcl,struct _fat_ptr*msg){
# 864
struct Cyc_Dict_Dict*r1=({struct Cyc_Dict_Dict*(*_tmp35B)(struct Cyc_Dict_Dict dic1,struct Cyc_Dict_Dict dic2,struct Cyc_Dict_Dict excl,struct Cyc_Absyn_Aggrdecl*(*merge_f)(struct Cyc_Absyn_Aggrdecl*,struct Cyc_Absyn_Aggrdecl*,unsigned,struct _fat_ptr*),struct _fat_ptr t,struct _fat_ptr*msg)=({struct Cyc_Dict_Dict*(*_tmp173)(struct Cyc_Dict_Dict dic1,struct Cyc_Dict_Dict dic2,struct Cyc_Dict_Dict excl,struct Cyc_Absyn_Aggrdecl*(*merge_f)(struct Cyc_Absyn_Aggrdecl*,struct Cyc_Absyn_Aggrdecl*,unsigned,struct _fat_ptr*),struct _fat_ptr t,struct _fat_ptr*msg)=(struct Cyc_Dict_Dict*(*)(struct Cyc_Dict_Dict dic1,struct Cyc_Dict_Dict dic2,struct Cyc_Dict_Dict excl,struct Cyc_Absyn_Aggrdecl*(*merge_f)(struct Cyc_Absyn_Aggrdecl*,struct Cyc_Absyn_Aggrdecl*,unsigned,struct _fat_ptr*),struct _fat_ptr t,struct _fat_ptr*msg))Cyc_Interface_compat_merge_dict;_tmp173;});struct Cyc_Dict_Dict _tmp35A=ie1->aggrdecls;struct Cyc_Dict_Dict _tmp359=ie2->aggrdecls;struct Cyc_Dict_Dict _tmp358=iexcl->aggrdecls;struct _fat_ptr _tmp357=({const char*_tmp174="type";_tag_fat(_tmp174,sizeof(char),5U);});_tmp35B(_tmp35A,_tmp359,_tmp358,Cyc_Tcdecl_merge_aggrdecl,_tmp357,msg);});
struct Cyc_Dict_Dict*r2=({struct Cyc_Dict_Dict*(*_tmp360)(struct Cyc_Dict_Dict dic1,struct Cyc_Dict_Dict dic2,struct Cyc_Dict_Dict excl,struct Cyc_Absyn_Datatypedecl*(*merge_f)(struct Cyc_Absyn_Datatypedecl*,struct Cyc_Absyn_Datatypedecl*,unsigned,struct _fat_ptr*),struct _fat_ptr t,struct _fat_ptr*msg)=({struct Cyc_Dict_Dict*(*_tmp171)(struct Cyc_Dict_Dict dic1,struct Cyc_Dict_Dict dic2,struct Cyc_Dict_Dict excl,struct Cyc_Absyn_Datatypedecl*(*merge_f)(struct Cyc_Absyn_Datatypedecl*,struct Cyc_Absyn_Datatypedecl*,unsigned,struct _fat_ptr*),struct _fat_ptr t,struct _fat_ptr*msg)=(struct Cyc_Dict_Dict*(*)(struct Cyc_Dict_Dict dic1,struct Cyc_Dict_Dict dic2,struct Cyc_Dict_Dict excl,struct Cyc_Absyn_Datatypedecl*(*merge_f)(struct Cyc_Absyn_Datatypedecl*,struct Cyc_Absyn_Datatypedecl*,unsigned,struct _fat_ptr*),struct _fat_ptr t,struct _fat_ptr*msg))Cyc_Interface_compat_merge_dict;_tmp171;});struct Cyc_Dict_Dict _tmp35F=ie1->datatypedecls;struct Cyc_Dict_Dict _tmp35E=ie2->datatypedecls;struct Cyc_Dict_Dict _tmp35D=iexcl->datatypedecls;struct _fat_ptr _tmp35C=({const char*_tmp172="datatype";_tag_fat(_tmp172,sizeof(char),9U);});_tmp360(_tmp35F,_tmp35E,_tmp35D,Cyc_Tcdecl_merge_datatypedecl,_tmp35C,msg);});
struct Cyc_Dict_Dict*r3=({struct Cyc_Dict_Dict*(*_tmp365)(struct Cyc_Dict_Dict dic1,struct Cyc_Dict_Dict dic2,struct Cyc_Dict_Dict excl,struct Cyc_Absyn_Enumdecl*(*merge_f)(struct Cyc_Absyn_Enumdecl*,struct Cyc_Absyn_Enumdecl*,unsigned,struct _fat_ptr*),struct _fat_ptr t,struct _fat_ptr*msg)=({struct Cyc_Dict_Dict*(*_tmp16F)(struct Cyc_Dict_Dict dic1,struct Cyc_Dict_Dict dic2,struct Cyc_Dict_Dict excl,struct Cyc_Absyn_Enumdecl*(*merge_f)(struct Cyc_Absyn_Enumdecl*,struct Cyc_Absyn_Enumdecl*,unsigned,struct _fat_ptr*),struct _fat_ptr t,struct _fat_ptr*msg)=(struct Cyc_Dict_Dict*(*)(struct Cyc_Dict_Dict dic1,struct Cyc_Dict_Dict dic2,struct Cyc_Dict_Dict excl,struct Cyc_Absyn_Enumdecl*(*merge_f)(struct Cyc_Absyn_Enumdecl*,struct Cyc_Absyn_Enumdecl*,unsigned,struct _fat_ptr*),struct _fat_ptr t,struct _fat_ptr*msg))Cyc_Interface_compat_merge_dict;_tmp16F;});struct Cyc_Dict_Dict _tmp364=ie1->enumdecls;struct Cyc_Dict_Dict _tmp363=ie2->enumdecls;struct Cyc_Dict_Dict _tmp362=iexcl->enumdecls;struct _fat_ptr _tmp361=({const char*_tmp170="enum";_tag_fat(_tmp170,sizeof(char),5U);});_tmp365(_tmp364,_tmp363,_tmp362,Cyc_Tcdecl_merge_enumdecl,_tmp361,msg);});
struct Cyc_Dict_Dict*r4=({struct Cyc_Dict_Dict*(*_tmp36A)(struct Cyc_Dict_Dict dic1,struct Cyc_Dict_Dict dic2,struct Cyc_Dict_Dict excl,struct Cyc_Absyn_Typedefdecl*(*merge_f)(struct Cyc_Absyn_Typedefdecl*,struct Cyc_Absyn_Typedefdecl*,unsigned,struct _fat_ptr*),struct _fat_ptr t,struct _fat_ptr*msg)=({struct Cyc_Dict_Dict*(*_tmp16D)(struct Cyc_Dict_Dict dic1,struct Cyc_Dict_Dict dic2,struct Cyc_Dict_Dict excl,struct Cyc_Absyn_Typedefdecl*(*merge_f)(struct Cyc_Absyn_Typedefdecl*,struct Cyc_Absyn_Typedefdecl*,unsigned,struct _fat_ptr*),struct _fat_ptr t,struct _fat_ptr*msg)=(struct Cyc_Dict_Dict*(*)(struct Cyc_Dict_Dict dic1,struct Cyc_Dict_Dict dic2,struct Cyc_Dict_Dict excl,struct Cyc_Absyn_Typedefdecl*(*merge_f)(struct Cyc_Absyn_Typedefdecl*,struct Cyc_Absyn_Typedefdecl*,unsigned,struct _fat_ptr*),struct _fat_ptr t,struct _fat_ptr*msg))Cyc_Interface_compat_merge_dict;_tmp16D;});struct Cyc_Dict_Dict _tmp369=ie1->typedefdecls;struct Cyc_Dict_Dict _tmp368=ie2->typedefdecls;struct Cyc_Dict_Dict _tmp367=iexcl->typedefdecls;struct _fat_ptr _tmp366=({const char*_tmp16E="typedef";_tag_fat(_tmp16E,sizeof(char),8U);});_tmp36A(_tmp369,_tmp368,_tmp367,Cyc_Tcdecl_merge_typedefdecl,_tmp366,msg);});
struct Cyc_Dict_Dict*r5=({struct Cyc_Dict_Dict*(*_tmp36F)(struct Cyc_Dict_Dict dic1,struct Cyc_Dict_Dict dic2,struct Cyc_Dict_Dict excl,struct Cyc_Absyn_Vardecl*(*merge_f)(struct Cyc_Absyn_Vardecl*,struct Cyc_Absyn_Vardecl*,unsigned,struct _fat_ptr*),struct _fat_ptr t,struct _fat_ptr*msg)=({struct Cyc_Dict_Dict*(*_tmp16B)(struct Cyc_Dict_Dict dic1,struct Cyc_Dict_Dict dic2,struct Cyc_Dict_Dict excl,struct Cyc_Absyn_Vardecl*(*merge_f)(struct Cyc_Absyn_Vardecl*,struct Cyc_Absyn_Vardecl*,unsigned,struct _fat_ptr*),struct _fat_ptr t,struct _fat_ptr*msg)=(struct Cyc_Dict_Dict*(*)(struct Cyc_Dict_Dict dic1,struct Cyc_Dict_Dict dic2,struct Cyc_Dict_Dict excl,struct Cyc_Absyn_Vardecl*(*merge_f)(struct Cyc_Absyn_Vardecl*,struct Cyc_Absyn_Vardecl*,unsigned,struct _fat_ptr*),struct _fat_ptr t,struct _fat_ptr*msg))Cyc_Interface_compat_merge_dict;_tmp16B;});struct Cyc_Dict_Dict _tmp36E=ie1->vardecls;struct Cyc_Dict_Dict _tmp36D=ie2->vardecls;struct Cyc_Dict_Dict _tmp36C=iexcl->vardecls;struct _fat_ptr _tmp36B=({const char*_tmp16C="variable";_tag_fat(_tmp16C,sizeof(char),9U);});_tmp36F(_tmp36E,_tmp36D,_tmp36C,Cyc_Tcdecl_merge_vardecl,_tmp36B,msg);});
struct Cyc_Dict_Dict*r6=({struct Cyc_Dict_Dict*(*_tmp374)(struct Cyc_Dict_Dict dic1,struct Cyc_Dict_Dict dic2,struct Cyc_Dict_Dict excl,struct Cyc_Tcdecl_Xdatatypefielddecl*(*merge_f)(struct Cyc_Tcdecl_Xdatatypefielddecl*,struct Cyc_Tcdecl_Xdatatypefielddecl*,unsigned,struct _fat_ptr*),struct _fat_ptr t,struct _fat_ptr*msg)=({struct Cyc_Dict_Dict*(*_tmp169)(struct Cyc_Dict_Dict dic1,struct Cyc_Dict_Dict dic2,struct Cyc_Dict_Dict excl,struct Cyc_Tcdecl_Xdatatypefielddecl*(*merge_f)(struct Cyc_Tcdecl_Xdatatypefielddecl*,struct Cyc_Tcdecl_Xdatatypefielddecl*,unsigned,struct _fat_ptr*),struct _fat_ptr t,struct _fat_ptr*msg)=(struct Cyc_Dict_Dict*(*)(struct Cyc_Dict_Dict dic1,struct Cyc_Dict_Dict dic2,struct Cyc_Dict_Dict excl,struct Cyc_Tcdecl_Xdatatypefielddecl*(*merge_f)(struct Cyc_Tcdecl_Xdatatypefielddecl*,struct Cyc_Tcdecl_Xdatatypefielddecl*,unsigned,struct _fat_ptr*),struct _fat_ptr t,struct _fat_ptr*msg))Cyc_Interface_compat_merge_dict;_tmp169;});struct Cyc_Dict_Dict _tmp373=ie1->xdatatypefielddecls;struct Cyc_Dict_Dict _tmp372=ie2->xdatatypefielddecls;struct Cyc_Dict_Dict _tmp371=iexcl->xdatatypefielddecls;struct _fat_ptr _tmp370=({const char*_tmp16A="xdatatypefield";_tag_fat(_tmp16A,sizeof(char),15U);});_tmp374(_tmp373,_tmp372,_tmp371,Cyc_Tcdecl_merge_xdatatypefielddecl,_tmp370,msg);});
if(((((!((unsigned)r1)|| !((unsigned)r2))|| !((unsigned)r3))|| !((unsigned)r4))|| !((unsigned)r5))|| !((unsigned)r6))
return 0;
# 870
return({struct Cyc_Interface_Ienv*_tmp168=_cycalloc(sizeof(*_tmp168));
# 872
_tmp168->aggrdecls=*r1,_tmp168->datatypedecls=*r2,_tmp168->enumdecls=*r3,_tmp168->typedefdecls=*r4,_tmp168->vardecls=*r5,_tmp168->xdatatypefielddecls=*r6;_tmp168;});}struct _tuple32{int f1;struct Cyc_Dict_Dict f2;struct Cyc_Dict_Dict f3;struct _fat_ptr f4;struct _fat_ptr*f5;};
# 760
void Cyc_Interface_disj_merge_dict_f(struct _tuple32*env,struct _tuple0*x,void*y){
# 879
struct _fat_ptr*_tmp179;struct _fat_ptr _tmp178;struct Cyc_Dict_Dict _tmp177;struct Cyc_Dict_Dict*_tmp176;int*_tmp175;_LL1: _tmp175=(int*)& env->f1;_tmp176=(struct Cyc_Dict_Dict*)& env->f2;_tmp177=env->f3;_tmp178=env->f4;_tmp179=env->f5;_LL2: {int*res=_tmp175;struct Cyc_Dict_Dict*res_dic=_tmp176;struct Cyc_Dict_Dict dic1=_tmp177;struct _fat_ptr t=_tmp178;struct _fat_ptr*msg=_tmp179;
if(({int(*_tmp376)(struct Cyc_Dict_Dict d,struct _tuple0*k)=({int(*_tmp17A)(struct Cyc_Dict_Dict d,struct _tuple0*k)=(int(*)(struct Cyc_Dict_Dict d,struct _tuple0*k))Cyc_Dict_member;_tmp17A;});struct Cyc_Dict_Dict _tmp375=dic1;_tmp376(_tmp375,x);})){
({struct _fat_ptr*_tmp379=msg;Cyc_Interface_check_err(_tmp379,(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp17D=({struct Cyc_String_pa_PrintArg_struct _tmp229;_tmp229.tag=0U,_tmp229.f1=(struct _fat_ptr)((struct _fat_ptr)t);_tmp229;});struct Cyc_String_pa_PrintArg_struct _tmp17E=({struct Cyc_String_pa_PrintArg_struct _tmp228;_tmp228.tag=0U,({
struct _fat_ptr _tmp377=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(x));_tmp228.f1=_tmp377;});_tmp228;});void*_tmp17B[2U];_tmp17B[0]=& _tmp17D,_tmp17B[1]=& _tmp17E;({struct _fat_ptr _tmp378=({const char*_tmp17C="%s %s is exported more than once";_tag_fat(_tmp17C,sizeof(char),33U);});Cyc_aprintf(_tmp378,_tag_fat(_tmp17B,sizeof(void*),2U));});}));});
*res=0;}else{
# 885
if(*res)({struct Cyc_Dict_Dict _tmp37D=({struct Cyc_Dict_Dict(*_tmp37C)(struct Cyc_Dict_Dict d,struct _tuple0*k,void*v)=({struct Cyc_Dict_Dict(*_tmp17F)(struct Cyc_Dict_Dict d,struct _tuple0*k,void*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _tuple0*k,void*v))Cyc_Dict_insert;_tmp17F;});struct Cyc_Dict_Dict _tmp37B=*res_dic;struct _tuple0*_tmp37A=x;_tmp37C(_tmp37B,_tmp37A,y);});*res_dic=_tmp37D;});}}}
# 760
struct Cyc_Dict_Dict*Cyc_Interface_disj_merge_dict(struct Cyc_Dict_Dict dic1,struct Cyc_Dict_Dict dic2,struct _fat_ptr t,struct _fat_ptr*msg){
# 893
struct _tuple32 env=({struct _tuple32 _tmp22A;_tmp22A.f1=1,_tmp22A.f2=dic1,_tmp22A.f3=dic1,_tmp22A.f4=t,_tmp22A.f5=msg;_tmp22A;});
({void(*_tmp37E)(void(*f)(struct _tuple32*,struct _tuple0*,void*),struct _tuple32*env,struct Cyc_Dict_Dict d)=({void(*_tmp180)(void(*f)(struct _tuple32*,struct _tuple0*,void*),struct _tuple32*env,struct Cyc_Dict_Dict d)=(void(*)(void(*f)(struct _tuple32*,struct _tuple0*,void*),struct _tuple32*env,struct Cyc_Dict_Dict d))Cyc_Dict_iter_c;_tmp180;});_tmp37E(Cyc_Interface_disj_merge_dict_f,& env,dic2);});
if(env.f1)
return({struct Cyc_Dict_Dict*_tmp181=_cycalloc(sizeof(*_tmp181));*_tmp181=env.f2;_tmp181;});
# 895
return 0;}
# 760
struct Cyc_Interface_Ienv*Cyc_Interface_disj_merge_ienv(struct Cyc_Interface_Ienv*ie1,struct Cyc_Interface_Ienv*ie2,struct _fat_ptr*msg){
# 901
struct Cyc_Dict_Dict*r1=({struct Cyc_Dict_Dict*(*_tmp383)(struct Cyc_Dict_Dict dic1,struct Cyc_Dict_Dict dic2,struct Cyc_Dict_Dict excl,struct Cyc_Absyn_Aggrdecl*(*merge_f)(struct Cyc_Absyn_Aggrdecl*,struct Cyc_Absyn_Aggrdecl*,unsigned,struct _fat_ptr*),struct _fat_ptr t,struct _fat_ptr*msg)=({struct Cyc_Dict_Dict*(*_tmp18A)(struct Cyc_Dict_Dict dic1,struct Cyc_Dict_Dict dic2,struct Cyc_Dict_Dict excl,struct Cyc_Absyn_Aggrdecl*(*merge_f)(struct Cyc_Absyn_Aggrdecl*,struct Cyc_Absyn_Aggrdecl*,unsigned,struct _fat_ptr*),struct _fat_ptr t,struct _fat_ptr*msg)=(struct Cyc_Dict_Dict*(*)(struct Cyc_Dict_Dict dic1,struct Cyc_Dict_Dict dic2,struct Cyc_Dict_Dict excl,struct Cyc_Absyn_Aggrdecl*(*merge_f)(struct Cyc_Absyn_Aggrdecl*,struct Cyc_Absyn_Aggrdecl*,unsigned,struct _fat_ptr*),struct _fat_ptr t,struct _fat_ptr*msg))Cyc_Interface_compat_merge_dict;_tmp18A;});struct Cyc_Dict_Dict _tmp382=ie1->aggrdecls;struct Cyc_Dict_Dict _tmp381=ie2->aggrdecls;struct Cyc_Dict_Dict _tmp380=
({struct Cyc_Dict_Dict(*_tmp18B)(int(*cmp)(struct _tuple0*,struct _tuple0*))=(struct Cyc_Dict_Dict(*)(int(*cmp)(struct _tuple0*,struct _tuple0*)))Cyc_Dict_empty;_tmp18B;})(Cyc_Absyn_qvar_cmp);
# 901
struct _fat_ptr _tmp37F=({const char*_tmp18C="type";_tag_fat(_tmp18C,sizeof(char),5U);});_tmp383(_tmp382,_tmp381,_tmp380,Cyc_Tcdecl_merge_aggrdecl,_tmp37F,msg);});
# 904
struct Cyc_Dict_Dict*r2=({struct Cyc_Dict_Dict _tmp386=ie1->datatypedecls;struct Cyc_Dict_Dict _tmp385=ie2->datatypedecls;struct _fat_ptr _tmp384=({const char*_tmp189="datatype";_tag_fat(_tmp189,sizeof(char),9U);});Cyc_Interface_disj_merge_dict(_tmp386,_tmp385,_tmp384,msg);});
struct Cyc_Dict_Dict*r3=({struct Cyc_Dict_Dict _tmp389=ie1->enumdecls;struct Cyc_Dict_Dict _tmp388=ie2->enumdecls;struct _fat_ptr _tmp387=({const char*_tmp188="enum";_tag_fat(_tmp188,sizeof(char),5U);});Cyc_Interface_disj_merge_dict(_tmp389,_tmp388,_tmp387,msg);});
# 907
struct Cyc_Dict_Dict*r4=({struct Cyc_Dict_Dict*(*_tmp38E)(struct Cyc_Dict_Dict dic1,struct Cyc_Dict_Dict dic2,struct Cyc_Dict_Dict excl,struct Cyc_Absyn_Typedefdecl*(*merge_f)(struct Cyc_Absyn_Typedefdecl*,struct Cyc_Absyn_Typedefdecl*,unsigned,struct _fat_ptr*),struct _fat_ptr t,struct _fat_ptr*msg)=({struct Cyc_Dict_Dict*(*_tmp185)(struct Cyc_Dict_Dict dic1,struct Cyc_Dict_Dict dic2,struct Cyc_Dict_Dict excl,struct Cyc_Absyn_Typedefdecl*(*merge_f)(struct Cyc_Absyn_Typedefdecl*,struct Cyc_Absyn_Typedefdecl*,unsigned,struct _fat_ptr*),struct _fat_ptr t,struct _fat_ptr*msg)=(struct Cyc_Dict_Dict*(*)(struct Cyc_Dict_Dict dic1,struct Cyc_Dict_Dict dic2,struct Cyc_Dict_Dict excl,struct Cyc_Absyn_Typedefdecl*(*merge_f)(struct Cyc_Absyn_Typedefdecl*,struct Cyc_Absyn_Typedefdecl*,unsigned,struct _fat_ptr*),struct _fat_ptr t,struct _fat_ptr*msg))Cyc_Interface_compat_merge_dict;_tmp185;});struct Cyc_Dict_Dict _tmp38D=ie1->typedefdecls;struct Cyc_Dict_Dict _tmp38C=ie2->typedefdecls;struct Cyc_Dict_Dict _tmp38B=
({struct Cyc_Dict_Dict(*_tmp186)(int(*cmp)(struct _tuple0*,struct _tuple0*))=(struct Cyc_Dict_Dict(*)(int(*cmp)(struct _tuple0*,struct _tuple0*)))Cyc_Dict_empty;_tmp186;})(Cyc_Absyn_qvar_cmp);
# 907
struct _fat_ptr _tmp38A=({const char*_tmp187="typedef";_tag_fat(_tmp187,sizeof(char),8U);});_tmp38E(_tmp38D,_tmp38C,_tmp38B,Cyc_Tcdecl_merge_typedefdecl,_tmp38A,msg);});
# 910
struct Cyc_Dict_Dict*r5=({struct Cyc_Dict_Dict _tmp391=ie1->vardecls;struct Cyc_Dict_Dict _tmp390=ie2->vardecls;struct _fat_ptr _tmp38F=({const char*_tmp184="variable";_tag_fat(_tmp184,sizeof(char),9U);});Cyc_Interface_disj_merge_dict(_tmp391,_tmp390,_tmp38F,msg);});
struct Cyc_Dict_Dict*r6=({struct Cyc_Dict_Dict _tmp394=ie1->xdatatypefielddecls;struct Cyc_Dict_Dict _tmp393=ie2->xdatatypefielddecls;struct _fat_ptr _tmp392=({const char*_tmp183="xdatatypefield";_tag_fat(_tmp183,sizeof(char),15U);});Cyc_Interface_disj_merge_dict(_tmp394,_tmp393,_tmp392,msg);});
# 914
if(((((!((unsigned)r1)|| !((unsigned)r2))|| !((unsigned)r3))|| !((unsigned)r4))|| !((unsigned)r5))|| !((unsigned)r6))
return 0;
# 914
return({struct Cyc_Interface_Ienv*_tmp182=_cycalloc(sizeof(*_tmp182));
# 916
_tmp182->aggrdecls=*r1,_tmp182->datatypedecls=*r2,_tmp182->enumdecls=*r3,_tmp182->typedefdecls=*r4,_tmp182->vardecls=*r5,_tmp182->xdatatypefielddecls=*r6;_tmp182;});}
# 760
struct Cyc_Interface_I*Cyc_Interface_merge(struct Cyc_Interface_I*i1,struct Cyc_Interface_I*i2,struct _tuple11*info){
# 921
struct _handler_cons _tmp18D;_push_handler(& _tmp18D);{int _tmp18F=0;if(setjmp(_tmp18D.handler))_tmp18F=1;if(!_tmp18F){
{struct _fat_ptr*msg=0;
# 924
if(info != 0)
msg=({struct _fat_ptr*_tmp194=_cycalloc(sizeof(*_tmp194));({struct _fat_ptr _tmp396=(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp192=({struct Cyc_String_pa_PrintArg_struct _tmp22C;_tmp22C.tag=0U,_tmp22C.f1=(struct _fat_ptr)((struct _fat_ptr)(*info).f1);_tmp22C;});struct Cyc_String_pa_PrintArg_struct _tmp193=({struct Cyc_String_pa_PrintArg_struct _tmp22B;_tmp22B.tag=0U,_tmp22B.f1=(struct _fat_ptr)((struct _fat_ptr)(*info).f2);_tmp22B;});void*_tmp190[2U];_tmp190[0]=& _tmp192,_tmp190[1]=& _tmp193;({struct _fat_ptr _tmp395=({const char*_tmp191="merging exports of %s and %s,";_tag_fat(_tmp191,sizeof(char),30U);});Cyc_aprintf(_tmp395,_tag_fat(_tmp190,sizeof(void*),2U));});});*_tmp194=_tmp396;});_tmp194;});{
# 924
struct Cyc_Interface_Ienv*exp=
# 928
Cyc_Interface_disj_merge_ienv(i1->exports,i2->exports,msg);
if(exp == 0){struct Cyc_Interface_I*_tmp195=0;_npop_handler(0U);return _tmp195;}if(info != 0)
# 932
msg=({struct _fat_ptr*_tmp19A=_cycalloc(sizeof(*_tmp19A));({struct _fat_ptr _tmp398=(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp198=({struct Cyc_String_pa_PrintArg_struct _tmp22E;_tmp22E.tag=0U,_tmp22E.f1=(struct _fat_ptr)((struct _fat_ptr)(*info).f1);_tmp22E;});struct Cyc_String_pa_PrintArg_struct _tmp199=({struct Cyc_String_pa_PrintArg_struct _tmp22D;_tmp22D.tag=0U,_tmp22D.f1=(struct _fat_ptr)((struct _fat_ptr)(*info).f2);_tmp22D;});void*_tmp196[2U];_tmp196[0]=& _tmp198,_tmp196[1]=& _tmp199;({struct _fat_ptr _tmp397=({const char*_tmp197="merging imports of %s and %s,";_tag_fat(_tmp197,sizeof(char),30U);});Cyc_aprintf(_tmp397,_tag_fat(_tmp196,sizeof(void*),2U));});});*_tmp19A=_tmp398;});_tmp19A;});{
# 929
struct Cyc_Interface_Ienv*imp=
# 935
Cyc_Interface_compat_merge_ienv(i1->imports,i2->imports,exp,msg);
if(imp == 0){struct Cyc_Interface_I*_tmp19B=0;_npop_handler(0U);return _tmp19B;}{struct Cyc_List_List*newtds=0;struct Cyc_List_List*l=i2->tds;
# 939
for(0;l != 0;l=l->tl){
newtds=({struct Cyc_List_List*_tmp19D=_cycalloc(sizeof(*_tmp19D));({struct Cyc_Absyn_Decl*_tmp399=({struct Cyc_Absyn_Decl*_tmp19C=_cycalloc(sizeof(*_tmp19C));*_tmp19C=*((struct Cyc_Absyn_Decl*)l->hd);_tmp19C;});_tmp19D->hd=_tmp399;}),_tmp19D->tl=newtds;_tmp19D;});}
# 942
newtds=Cyc_List_rev(({struct Cyc_List_List*(*_tmp39B)(struct Cyc_List_List*(*f)(struct Cyc_List_List*,struct Cyc_Absyn_Decl*),struct Cyc_List_List*accum,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp19E)(struct Cyc_List_List*(*f)(struct Cyc_List_List*,struct Cyc_Absyn_Decl*),struct Cyc_List_List*accum,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_List_List*(*f)(struct Cyc_List_List*,struct Cyc_Absyn_Decl*),struct Cyc_List_List*accum,struct Cyc_List_List*x))Cyc_List_fold_left;_tmp19E;});struct Cyc_List_List*_tmp39A=newtds;_tmp39B(Cyc_Interface_uniqify_decl_list,_tmp39A,i1->tds);}));{
struct Cyc_Interface_I*_tmp1A0=({struct Cyc_Interface_I*_tmp19F=_cycalloc(sizeof(*_tmp19F));_tmp19F->imports=imp,_tmp19F->exports=exp,_tmp19F->tds=newtds;_tmp19F;});_npop_handler(0U);return _tmp1A0;}}}}}
# 922
;_pop_handler();}else{void*_tmp18E=(void*)Cyc_Core_get_exn_thrown();void*_tmp1A1=_tmp18E;void*_tmp1A2;if(((struct Cyc_Tcdecl_Incompatible_exn_struct*)_tmp1A1)->tag == Cyc_Tcdecl_Incompatible){_LL1: _LL2:
# 944
 return 0;}else{_LL3: _tmp1A2=_tmp1A1;_LL4: {void*exn=_tmp1A2;(int)_rethrow(exn);}}_LL0:;}}}
# 760
struct Cyc_Interface_I*Cyc_Interface_merge_list(struct Cyc_List_List*li,struct Cyc_List_List*linfo){
# 948
if(li == 0)return Cyc_Interface_empty();{struct Cyc_Interface_I*curr_i=(struct Cyc_Interface_I*)li->hd;
# 951
struct _fat_ptr*curr_info=linfo != 0?(struct _fat_ptr*)linfo->hd: 0;
li=li->tl;
if(linfo != 0)linfo=linfo->tl;for(0;li != 0;li=li->tl){
# 956
struct Cyc_Interface_I*i=({struct Cyc_Interface_I*_tmp39D=curr_i;struct Cyc_Interface_I*_tmp39C=(struct Cyc_Interface_I*)li->hd;Cyc_Interface_merge(_tmp39D,_tmp39C,
curr_info != 0 && linfo != 0?({struct _tuple11*_tmp1A8=_cycalloc(sizeof(*_tmp1A8));_tmp1A8->f1=*curr_info,_tmp1A8->f2=*((struct _fat_ptr*)linfo->hd);_tmp1A8;}): 0);});
if(i == 0)return 0;curr_i=i;
# 960
if(curr_info != 0)
curr_info=linfo != 0?({struct _fat_ptr*_tmp1A7=_cycalloc(sizeof(*_tmp1A7));({struct _fat_ptr _tmp39F=(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp1A5=({struct Cyc_String_pa_PrintArg_struct _tmp230;_tmp230.tag=0U,_tmp230.f1=(struct _fat_ptr)((struct _fat_ptr)*curr_info);_tmp230;});struct Cyc_String_pa_PrintArg_struct _tmp1A6=({struct Cyc_String_pa_PrintArg_struct _tmp22F;_tmp22F.tag=0U,_tmp22F.f1=(struct _fat_ptr)((struct _fat_ptr)*((struct _fat_ptr*)linfo->hd));_tmp22F;});void*_tmp1A3[2U];_tmp1A3[0]=& _tmp1A5,_tmp1A3[1]=& _tmp1A6;({struct _fat_ptr _tmp39E=({const char*_tmp1A4="%s+%s";_tag_fat(_tmp1A4,sizeof(char),6U);});Cyc_aprintf(_tmp39E,_tag_fat(_tmp1A3,sizeof(void*),2U));});});*_tmp1A7=_tmp39F;});_tmp1A7;}): 0;
# 960
if(linfo != 0)
# 963
linfo=linfo->tl;}
# 965
return curr_i;}}
# 760
struct Cyc_Interface_I*Cyc_Interface_get_and_merge_list(struct Cyc_Interface_I*(*get)(void*),struct Cyc_List_List*la,struct Cyc_List_List*linfo){
# 969
if(la == 0)return Cyc_Interface_empty();{struct Cyc_Interface_I*curr_i=
# 971
get(la->hd);
struct _fat_ptr*curr_info=linfo != 0?(struct _fat_ptr*)linfo->hd: 0;
la=la->tl;
if(linfo != 0)linfo=linfo->tl;for(0;la != 0;la=la->tl){
# 977
struct Cyc_Interface_I*i=({struct Cyc_Interface_I*_tmp3A1=curr_i;struct Cyc_Interface_I*_tmp3A0=get(la->hd);Cyc_Interface_merge(_tmp3A1,_tmp3A0,
curr_info != 0 && linfo != 0?({struct _tuple11*_tmp1AE=_cycalloc(sizeof(*_tmp1AE));_tmp1AE->f1=*curr_info,_tmp1AE->f2=*((struct _fat_ptr*)linfo->hd);_tmp1AE;}): 0);});
if(i == 0)return 0;curr_i=i;
# 981
if(curr_info != 0)
curr_info=linfo != 0?({struct _fat_ptr*_tmp1AD=_cycalloc(sizeof(*_tmp1AD));({struct _fat_ptr _tmp3A3=(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp1AB=({struct Cyc_String_pa_PrintArg_struct _tmp232;_tmp232.tag=0U,_tmp232.f1=(struct _fat_ptr)((struct _fat_ptr)*curr_info);_tmp232;});struct Cyc_String_pa_PrintArg_struct _tmp1AC=({struct Cyc_String_pa_PrintArg_struct _tmp231;_tmp231.tag=0U,_tmp231.f1=(struct _fat_ptr)((struct _fat_ptr)*((struct _fat_ptr*)linfo->hd));_tmp231;});void*_tmp1A9[2U];_tmp1A9[0]=& _tmp1AB,_tmp1A9[1]=& _tmp1AC;({struct _fat_ptr _tmp3A2=({const char*_tmp1AA="%s+%s";_tag_fat(_tmp1AA,sizeof(char),6U);});Cyc_aprintf(_tmp3A2,_tag_fat(_tmp1A9,sizeof(void*),2U));});});*_tmp1AD=_tmp3A3;});_tmp1AD;}): 0;
# 981
if(linfo != 0)
# 984
linfo=linfo->tl;}
# 986
return curr_i;}}
# 760
static struct Cyc_List_List*Cyc_Interface_add_namespace(struct Cyc_List_List*tds){
# 993
struct Cyc_List_List*ans=0;
for(0;tds != 0;tds=tds->tl){
struct _tuple0*qv;
struct Cyc_Absyn_Decl*d=(struct Cyc_Absyn_Decl*)tds->hd;
{void*_stmttmpB=d->r;void*_tmp1AF=_stmttmpB;struct Cyc_Absyn_Typedefdecl*_tmp1B0;struct Cyc_Absyn_Enumdecl*_tmp1B1;struct Cyc_Absyn_Datatypedecl*_tmp1B2;struct Cyc_Absyn_Aggrdecl*_tmp1B3;struct Cyc_Absyn_Fndecl*_tmp1B4;struct Cyc_Absyn_Vardecl*_tmp1B5;switch(*((int*)_tmp1AF)){case 0U: _LL1: _tmp1B5=((struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct*)_tmp1AF)->f1;_LL2: {struct Cyc_Absyn_Vardecl*vd=_tmp1B5;
qv=vd->name;goto _LL0;}case 1U: _LL3: _tmp1B4=((struct Cyc_Absyn_Fn_d_Absyn_Raw_decl_struct*)_tmp1AF)->f1;_LL4: {struct Cyc_Absyn_Fndecl*fd=_tmp1B4;
qv=fd->name;goto _LL0;}case 5U: _LL5: _tmp1B3=((struct Cyc_Absyn_Aggr_d_Absyn_Raw_decl_struct*)_tmp1AF)->f1;_LL6: {struct Cyc_Absyn_Aggrdecl*ad=_tmp1B3;
qv=ad->name;goto _LL0;}case 6U: _LL7: _tmp1B2=((struct Cyc_Absyn_Datatype_d_Absyn_Raw_decl_struct*)_tmp1AF)->f1;_LL8: {struct Cyc_Absyn_Datatypedecl*dd=_tmp1B2;
qv=dd->name;goto _LL0;}case 7U: _LL9: _tmp1B1=((struct Cyc_Absyn_Enum_d_Absyn_Raw_decl_struct*)_tmp1AF)->f1;_LLA: {struct Cyc_Absyn_Enumdecl*ed=_tmp1B1;
qv=ed->name;goto _LL0;}case 8U: _LLB: _tmp1B0=((struct Cyc_Absyn_Typedef_d_Absyn_Raw_decl_struct*)_tmp1AF)->f1;_LLC: {struct Cyc_Absyn_Typedefdecl*td=_tmp1B0;
qv=td->name;goto _LL0;}default: _LLD: _LLE:
 Cyc_Interface_err(({const char*_tmp1B6="bad decl form in Interface::add_namespace";_tag_fat(_tmp1B6,sizeof(char),42U);}));return 0;}_LL0:;}{
# 1006
struct Cyc_List_List*vs;
{union Cyc_Absyn_Nmspace _stmttmpC=(*qv).f1;union Cyc_Absyn_Nmspace _tmp1B7=_stmttmpC;struct Cyc_List_List*_tmp1B8;struct Cyc_List_List*_tmp1B9;switch((_tmp1B7.C_n).tag){case 2U: _LL10: _tmp1B9=(_tmp1B7.Abs_n).val;_LL11: {struct Cyc_List_List*x=_tmp1B9;
vs=x;goto _LLF;}case 3U: _LL12: _tmp1B8=(_tmp1B7.C_n).val;_LL13: {struct Cyc_List_List*x=_tmp1B8;
vs=x;goto _LLF;}default: _LL14: _LL15:
 Cyc_Interface_err(({const char*_tmp1BA="bad namespace in Interface::add_namespace";_tag_fat(_tmp1BA,sizeof(char),42U);}));return 0;}_LLF:;}
# 1012
vs=Cyc_List_imp_rev(vs);
({union Cyc_Absyn_Nmspace _tmp3A4=Cyc_Absyn_Rel_n(0);(*qv).f1=_tmp3A4;});
for(0;vs != 0;vs=vs->tl){
d=({void*_tmp3A6=(void*)({struct Cyc_Absyn_Namespace_d_Absyn_Raw_decl_struct*_tmp1BC=_cycalloc(sizeof(*_tmp1BC));_tmp1BC->tag=9U,_tmp1BC->f1=(struct _fat_ptr*)vs->hd,({struct Cyc_List_List*_tmp3A5=({struct Cyc_List_List*_tmp1BB=_cycalloc(sizeof(*_tmp1BB));_tmp1BB->hd=d,_tmp1BB->tl=0;_tmp1BB;});_tmp1BC->f2=_tmp3A5;});_tmp1BC;});Cyc_Absyn_new_decl(_tmp3A6,d->loc);});}
ans=({struct Cyc_List_List*_tmp1BD=_cycalloc(sizeof(*_tmp1BD));_tmp1BD->hd=d,_tmp1BD->tl=ans;_tmp1BD;});}}
# 1018
ans=Cyc_List_imp_rev(ans);
return ans;}
# 760
static struct Cyc_List_List*Cyc_Interface_add_aggrdecl(struct _tuple0*x,struct Cyc_Absyn_Aggrdecl*d,struct Cyc_List_List*tds){
# 1023
return({struct Cyc_List_List*_tmp1BF=_cycalloc(sizeof(*_tmp1BF));({struct Cyc_Absyn_Decl*_tmp3A7=Cyc_Absyn_new_decl((void*)({struct Cyc_Absyn_Aggr_d_Absyn_Raw_decl_struct*_tmp1BE=_cycalloc(sizeof(*_tmp1BE));_tmp1BE->tag=5U,_tmp1BE->f1=d;_tmp1BE;}),0U);_tmp1BF->hd=_tmp3A7;}),_tmp1BF->tl=tds;_tmp1BF;});}
# 760
static struct Cyc_List_List*Cyc_Interface_add_aggrdecl_header(struct _tuple0*x,struct Cyc_Absyn_Aggrdecl*d,struct Cyc_List_List*tds){
# 1028
d=({struct Cyc_Absyn_Aggrdecl*_tmp1C0=_cycalloc(sizeof(*_tmp1C0));*_tmp1C0=*d;_tmp1C0;});
d->impl=0;
if((int)d->sc != (int)Cyc_Absyn_ExternC)d->sc=Cyc_Absyn_Extern;return({struct Cyc_List_List*_tmp1C2=_cycalloc(sizeof(*_tmp1C2));
({struct Cyc_Absyn_Decl*_tmp3A8=Cyc_Absyn_new_decl((void*)({struct Cyc_Absyn_Aggr_d_Absyn_Raw_decl_struct*_tmp1C1=_cycalloc(sizeof(*_tmp1C1));_tmp1C1->tag=5U,_tmp1C1->f1=d;_tmp1C1;}),0U);_tmp1C2->hd=_tmp3A8;}),_tmp1C2->tl=tds;_tmp1C2;});}
# 760
static struct Cyc_List_List*Cyc_Interface_add_datatypedecl(struct _tuple0*x,struct Cyc_Absyn_Datatypedecl*d,struct Cyc_List_List*tds){
# 1035
return({struct Cyc_List_List*_tmp1C4=_cycalloc(sizeof(*_tmp1C4));({struct Cyc_Absyn_Decl*_tmp3A9=Cyc_Absyn_new_decl((void*)({struct Cyc_Absyn_Datatype_d_Absyn_Raw_decl_struct*_tmp1C3=_cycalloc(sizeof(*_tmp1C3));_tmp1C3->tag=6U,_tmp1C3->f1=d;_tmp1C3;}),0U);_tmp1C4->hd=_tmp3A9;}),_tmp1C4->tl=tds;_tmp1C4;});}static char _tmp1C5[2U]="_";
# 760
static struct _fat_ptr Cyc_Interface_us={_tmp1C5,_tmp1C5,_tmp1C5 + 2U};
# 1039
static struct _fat_ptr*Cyc_Interface_us_p=& Cyc_Interface_us;
# 1041
static struct _tuple25*Cyc_Interface_rewrite_datatypefield_type(struct _tuple25*x){
return({struct _tuple25*_tmp1C8=_cycalloc(sizeof(*_tmp1C8));({struct Cyc_Absyn_Tqual _tmp3AD=Cyc_Absyn_empty_tqual(0U);_tmp1C8->f1=_tmp3AD;}),({
void*_tmp3AC=(void*)({struct Cyc_Absyn_VarType_Absyn_Type_struct*_tmp1C7=_cycalloc(sizeof(*_tmp1C7));_tmp1C7->tag=2U,({struct Cyc_Absyn_Tvar*_tmp3AB=({struct Cyc_Absyn_Tvar*_tmp1C6=_cycalloc(sizeof(*_tmp1C6));_tmp1C6->name=Cyc_Interface_us_p,_tmp1C6->identity=- 1,({void*_tmp3AA=Cyc_Tcutil_kind_to_bound(& Cyc_Tcutil_bk);_tmp1C6->kind=_tmp3AA;});_tmp1C6;});_tmp1C7->f1=_tmp3AB;});_tmp1C7;});_tmp1C8->f2=_tmp3AC;});_tmp1C8;});}
# 1041
static struct Cyc_Absyn_Datatypefield*Cyc_Interface_rewrite_datatypefield(struct Cyc_Absyn_Datatypefield*f){
# 1047
f=({struct Cyc_Absyn_Datatypefield*_tmp1C9=_cycalloc(sizeof(*_tmp1C9));*_tmp1C9=*f;_tmp1C9;});
({struct Cyc_List_List*_tmp3AF=({struct Cyc_List_List*(*_tmp3AE)(struct _tuple25*(*f)(struct _tuple25*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp1CA)(struct _tuple25*(*f)(struct _tuple25*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct _tuple25*(*f)(struct _tuple25*),struct Cyc_List_List*x))Cyc_List_map;_tmp1CA;});_tmp3AE(Cyc_Interface_rewrite_datatypefield_type,f->typs);});f->typs=_tmp3AF;});
return f;}
# 1041
static struct Cyc_List_List*Cyc_Interface_add_datatypedecl_header(struct _tuple0*x,struct Cyc_Absyn_Datatypedecl*d,struct Cyc_List_List*tds){
# 1054
d=({struct Cyc_Absyn_Datatypedecl*_tmp1CB=_cycalloc(sizeof(*_tmp1CB));*_tmp1CB=*d;_tmp1CB;});
# 1056
if(d->fields != 0)({struct Cyc_Core_Opt*_tmp3B2=({struct Cyc_Core_Opt*_tmp1CD=_cycalloc(sizeof(*_tmp1CD));({struct Cyc_List_List*_tmp3B1=({struct Cyc_List_List*(*_tmp3B0)(struct Cyc_Absyn_Datatypefield*(*f)(struct Cyc_Absyn_Datatypefield*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp1CC)(struct Cyc_Absyn_Datatypefield*(*f)(struct Cyc_Absyn_Datatypefield*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_Absyn_Datatypefield*(*f)(struct Cyc_Absyn_Datatypefield*),struct Cyc_List_List*x))Cyc_List_map;_tmp1CC;});_tmp3B0(Cyc_Interface_rewrite_datatypefield,(struct Cyc_List_List*)(d->fields)->v);});_tmp1CD->v=_tmp3B1;});_tmp1CD;});d->fields=_tmp3B2;});if((int)d->sc != (int)Cyc_Absyn_ExternC)
d->sc=Cyc_Absyn_Extern;
# 1056
return({struct Cyc_List_List*_tmp1CF=_cycalloc(sizeof(*_tmp1CF));
# 1058
({struct Cyc_Absyn_Decl*_tmp3B3=Cyc_Absyn_new_decl((void*)({struct Cyc_Absyn_Datatype_d_Absyn_Raw_decl_struct*_tmp1CE=_cycalloc(sizeof(*_tmp1CE));_tmp1CE->tag=6U,_tmp1CE->f1=d;_tmp1CE;}),0U);_tmp1CF->hd=_tmp3B3;}),_tmp1CF->tl=tds;_tmp1CF;});}
# 1041
static struct Cyc_List_List*Cyc_Interface_add_enumdecl(struct _tuple0*x,struct Cyc_Absyn_Enumdecl*d,struct Cyc_List_List*tds){
# 1062
return({struct Cyc_List_List*_tmp1D1=_cycalloc(sizeof(*_tmp1D1));({struct Cyc_Absyn_Decl*_tmp3B4=Cyc_Absyn_new_decl((void*)({struct Cyc_Absyn_Enum_d_Absyn_Raw_decl_struct*_tmp1D0=_cycalloc(sizeof(*_tmp1D0));_tmp1D0->tag=7U,_tmp1D0->f1=d;_tmp1D0;}),0U);_tmp1D1->hd=_tmp3B4;}),_tmp1D1->tl=tds;_tmp1D1;});}
# 1041
static struct Cyc_List_List*Cyc_Interface_add_enumdecl_header(struct _tuple0*x,struct Cyc_Absyn_Enumdecl*d,struct Cyc_List_List*tds){
# 1067
d=({struct Cyc_Absyn_Enumdecl*_tmp1D2=_cycalloc(sizeof(*_tmp1D2));*_tmp1D2=*d;_tmp1D2;});
d->fields=0;
if((int)d->sc != (int)Cyc_Absyn_ExternC)d->sc=Cyc_Absyn_Extern;return({struct Cyc_List_List*_tmp1D4=_cycalloc(sizeof(*_tmp1D4));
({struct Cyc_Absyn_Decl*_tmp3B5=Cyc_Absyn_new_decl((void*)({struct Cyc_Absyn_Enum_d_Absyn_Raw_decl_struct*_tmp1D3=_cycalloc(sizeof(*_tmp1D3));_tmp1D3->tag=7U,_tmp1D3->f1=d;_tmp1D3;}),0U);_tmp1D4->hd=_tmp3B5;}),_tmp1D4->tl=tds;_tmp1D4;});}
# 1041
static struct Cyc_List_List*Cyc_Interface_add_typedef(struct _tuple0*x,struct Cyc_Absyn_Typedefdecl*d,struct Cyc_List_List*tds){
# 1075
return({struct Cyc_List_List*_tmp1D6=_cycalloc(sizeof(*_tmp1D6));({struct Cyc_Absyn_Decl*_tmp3B6=Cyc_Absyn_new_decl((void*)({struct Cyc_Absyn_Typedef_d_Absyn_Raw_decl_struct*_tmp1D5=_cycalloc(sizeof(*_tmp1D5));_tmp1D5->tag=8U,_tmp1D5->f1=d;_tmp1D5;}),0U);_tmp1D6->hd=_tmp3B6;}),_tmp1D6->tl=tds;_tmp1D6;});}
# 1041
static struct Cyc_List_List*Cyc_Interface_add_vardecl(struct _tuple0*x,struct Cyc_Absyn_Vardecl*d,struct Cyc_List_List*tds){
# 1079
return({struct Cyc_List_List*_tmp1D8=_cycalloc(sizeof(*_tmp1D8));({struct Cyc_Absyn_Decl*_tmp3B7=Cyc_Absyn_new_decl((void*)({struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct*_tmp1D7=_cycalloc(sizeof(*_tmp1D7));_tmp1D7->tag=0U,_tmp1D7->f1=d;_tmp1D7;}),0U);_tmp1D8->hd=_tmp3B7;}),_tmp1D8->tl=tds;_tmp1D8;});}
# 1041
static struct Cyc_List_List*Cyc_Interface_add_xdatatypefielddecl(struct _tuple0*x,struct Cyc_Tcdecl_Xdatatypefielddecl*d,struct Cyc_List_List*tds){
# 1084
struct Cyc_Absyn_Datatypefield*_tmp1DA;struct Cyc_Absyn_Datatypedecl*_tmp1D9;_LL1: _tmp1D9=d->base;_tmp1DA=d->field;_LL2: {struct Cyc_Absyn_Datatypedecl*b=_tmp1D9;struct Cyc_Absyn_Datatypefield*f=_tmp1DA;
b=({struct Cyc_Absyn_Datatypedecl*_tmp1DB=_cycalloc(sizeof(*_tmp1DB));*_tmp1DB=*b;_tmp1DB;});
({struct Cyc_Core_Opt*_tmp3B9=({struct Cyc_Core_Opt*_tmp1DD=_cycalloc(sizeof(*_tmp1DD));({struct Cyc_List_List*_tmp3B8=({struct Cyc_List_List*_tmp1DC=_cycalloc(sizeof(*_tmp1DC));_tmp1DC->hd=f,_tmp1DC->tl=0;_tmp1DC;});_tmp1DD->v=_tmp3B8;});_tmp1DD;});b->fields=_tmp3B9;});
b->sc=Cyc_Absyn_Extern;
return({struct Cyc_List_List*_tmp1DF=_cycalloc(sizeof(*_tmp1DF));({struct Cyc_Absyn_Decl*_tmp3BA=Cyc_Absyn_new_decl((void*)({struct Cyc_Absyn_Datatype_d_Absyn_Raw_decl_struct*_tmp1DE=_cycalloc(sizeof(*_tmp1DE));_tmp1DE->tag=6U,_tmp1DE->f1=b;_tmp1DE;}),0U);_tmp1DF->hd=_tmp3BA;}),_tmp1DF->tl=tds;_tmp1DF;});}}
# 1041
static struct Cyc_List_List*Cyc_Interface_add_xdatatypefielddecl_header(struct _tuple0*x,struct Cyc_Tcdecl_Xdatatypefielddecl*d,struct Cyc_List_List*tds){
# 1093
struct Cyc_Absyn_Datatypefield*_tmp1E1;struct Cyc_Absyn_Datatypedecl*_tmp1E0;_LL1: _tmp1E0=d->base;_tmp1E1=d->field;_LL2: {struct Cyc_Absyn_Datatypedecl*b=_tmp1E0;struct Cyc_Absyn_Datatypefield*f=_tmp1E1;
b=({struct Cyc_Absyn_Datatypedecl*_tmp1E2=_cycalloc(sizeof(*_tmp1E2));*_tmp1E2=*b;_tmp1E2;});
f=Cyc_Interface_rewrite_datatypefield(f);
f->sc=Cyc_Absyn_Extern;
({struct Cyc_Core_Opt*_tmp3BC=({struct Cyc_Core_Opt*_tmp1E4=_cycalloc(sizeof(*_tmp1E4));({struct Cyc_List_List*_tmp3BB=({struct Cyc_List_List*_tmp1E3=_cycalloc(sizeof(*_tmp1E3));_tmp1E3->hd=f,_tmp1E3->tl=0;_tmp1E3;});_tmp1E4->v=_tmp3BB;});_tmp1E4;});b->fields=_tmp3BC;});
b->sc=Cyc_Absyn_Extern;
return({struct Cyc_List_List*_tmp1E6=_cycalloc(sizeof(*_tmp1E6));({struct Cyc_Absyn_Decl*_tmp3BD=Cyc_Absyn_new_decl((void*)({struct Cyc_Absyn_Datatype_d_Absyn_Raw_decl_struct*_tmp1E5=_cycalloc(sizeof(*_tmp1E5));_tmp1E5->tag=6U,_tmp1E5->f1=b;_tmp1E5;}),0U);_tmp1E6->hd=_tmp3BD;}),_tmp1E6->tl=tds;_tmp1E6;});}}
# 1041
static void Cyc_Interface_print_ns_headers(struct Cyc___cycFILE*f,struct Cyc_Interface_Ienv*ie){
# 1103
struct Cyc_List_List*tds=0;
tds=({struct Cyc_List_List*(*_tmp3BF)(struct Cyc_List_List*(*f)(struct _tuple0*,struct Cyc_Absyn_Aggrdecl*,struct Cyc_List_List*),struct Cyc_Dict_Dict d,struct Cyc_List_List*accum)=({struct Cyc_List_List*(*_tmp1E7)(struct Cyc_List_List*(*f)(struct _tuple0*,struct Cyc_Absyn_Aggrdecl*,struct Cyc_List_List*),struct Cyc_Dict_Dict d,struct Cyc_List_List*accum)=(struct Cyc_List_List*(*)(struct Cyc_List_List*(*f)(struct _tuple0*,struct Cyc_Absyn_Aggrdecl*,struct Cyc_List_List*),struct Cyc_Dict_Dict d,struct Cyc_List_List*accum))Cyc_Dict_fold;_tmp1E7;});struct Cyc_Dict_Dict _tmp3BE=ie->aggrdecls;_tmp3BF(Cyc_Interface_add_aggrdecl_header,_tmp3BE,tds);});
tds=({struct Cyc_List_List*(*_tmp3C1)(struct Cyc_List_List*(*f)(struct _tuple0*,struct Cyc_Absyn_Datatypedecl*,struct Cyc_List_List*),struct Cyc_Dict_Dict d,struct Cyc_List_List*accum)=({struct Cyc_List_List*(*_tmp1E8)(struct Cyc_List_List*(*f)(struct _tuple0*,struct Cyc_Absyn_Datatypedecl*,struct Cyc_List_List*),struct Cyc_Dict_Dict d,struct Cyc_List_List*accum)=(struct Cyc_List_List*(*)(struct Cyc_List_List*(*f)(struct _tuple0*,struct Cyc_Absyn_Datatypedecl*,struct Cyc_List_List*),struct Cyc_Dict_Dict d,struct Cyc_List_List*accum))Cyc_Dict_fold;_tmp1E8;});struct Cyc_Dict_Dict _tmp3C0=ie->datatypedecls;_tmp3C1(Cyc_Interface_add_datatypedecl_header,_tmp3C0,tds);});
tds=({struct Cyc_List_List*(*_tmp3C3)(struct Cyc_List_List*(*f)(struct _tuple0*,struct Cyc_Absyn_Enumdecl*,struct Cyc_List_List*),struct Cyc_Dict_Dict d,struct Cyc_List_List*accum)=({struct Cyc_List_List*(*_tmp1E9)(struct Cyc_List_List*(*f)(struct _tuple0*,struct Cyc_Absyn_Enumdecl*,struct Cyc_List_List*),struct Cyc_Dict_Dict d,struct Cyc_List_List*accum)=(struct Cyc_List_List*(*)(struct Cyc_List_List*(*f)(struct _tuple0*,struct Cyc_Absyn_Enumdecl*,struct Cyc_List_List*),struct Cyc_Dict_Dict d,struct Cyc_List_List*accum))Cyc_Dict_fold;_tmp1E9;});struct Cyc_Dict_Dict _tmp3C2=ie->enumdecls;_tmp3C3(Cyc_Interface_add_enumdecl_header,_tmp3C2,tds);});
# 1108
if(tds != 0){
tds=Cyc_List_imp_rev(tds);
tds=Cyc_Interface_add_namespace(tds);
Cyc_Absynpp_decllist2file(tds,f);}}
# 1041
static void Cyc_Interface_print_ns_xdatatypefielddecl_headers(struct Cyc___cycFILE*f,struct Cyc_Interface_Ienv*ie){
# 1116
struct Cyc_List_List*tds=({struct Cyc_List_List*(*_tmp3C4)(struct Cyc_List_List*(*f)(struct _tuple0*,struct Cyc_Tcdecl_Xdatatypefielddecl*,struct Cyc_List_List*),struct Cyc_Dict_Dict d,struct Cyc_List_List*accum)=({struct Cyc_List_List*(*_tmp1EA)(struct Cyc_List_List*(*f)(struct _tuple0*,struct Cyc_Tcdecl_Xdatatypefielddecl*,struct Cyc_List_List*),struct Cyc_Dict_Dict d,struct Cyc_List_List*accum)=(struct Cyc_List_List*(*)(struct Cyc_List_List*(*f)(struct _tuple0*,struct Cyc_Tcdecl_Xdatatypefielddecl*,struct Cyc_List_List*),struct Cyc_Dict_Dict d,struct Cyc_List_List*accum))Cyc_Dict_fold;_tmp1EA;});_tmp3C4(Cyc_Interface_add_xdatatypefielddecl_header,ie->xdatatypefielddecls,0);});
if(tds != 0){
tds=Cyc_List_imp_rev(tds);
tds=Cyc_Interface_add_namespace(tds);
Cyc_Absynpp_decllist2file(tds,f);}}
# 1041
static void Cyc_Interface_print_ns_typedefs(struct Cyc___cycFILE*f,struct Cyc_Interface_Ienv*ie){
# 1125
struct Cyc_List_List*tds=({struct Cyc_List_List*(*_tmp3C5)(struct Cyc_List_List*(*f)(struct _tuple0*,struct Cyc_Absyn_Typedefdecl*,struct Cyc_List_List*),struct Cyc_Dict_Dict d,struct Cyc_List_List*accum)=({struct Cyc_List_List*(*_tmp1EB)(struct Cyc_List_List*(*f)(struct _tuple0*,struct Cyc_Absyn_Typedefdecl*,struct Cyc_List_List*),struct Cyc_Dict_Dict d,struct Cyc_List_List*accum)=(struct Cyc_List_List*(*)(struct Cyc_List_List*(*f)(struct _tuple0*,struct Cyc_Absyn_Typedefdecl*,struct Cyc_List_List*),struct Cyc_Dict_Dict d,struct Cyc_List_List*accum))Cyc_Dict_fold;_tmp1EB;});_tmp3C5(Cyc_Interface_add_typedef,ie->typedefdecls,0);});
if(tds != 0){
tds=Cyc_List_imp_rev(tds);
tds=Cyc_Interface_add_namespace(tds);
Cyc_Absynpp_decllist2file(tds,f);}}
# 1041
static void Cyc_Interface_print_ns_decls(struct Cyc___cycFILE*f,struct Cyc_Interface_Ienv*ie){
# 1134
struct Cyc_List_List*tds=0;
tds=({struct Cyc_List_List*(*_tmp3C7)(struct Cyc_List_List*(*f)(struct _tuple0*,struct Cyc_Absyn_Aggrdecl*,struct Cyc_List_List*),struct Cyc_Dict_Dict d,struct Cyc_List_List*accum)=({struct Cyc_List_List*(*_tmp1EC)(struct Cyc_List_List*(*f)(struct _tuple0*,struct Cyc_Absyn_Aggrdecl*,struct Cyc_List_List*),struct Cyc_Dict_Dict d,struct Cyc_List_List*accum)=(struct Cyc_List_List*(*)(struct Cyc_List_List*(*f)(struct _tuple0*,struct Cyc_Absyn_Aggrdecl*,struct Cyc_List_List*),struct Cyc_Dict_Dict d,struct Cyc_List_List*accum))Cyc_Dict_fold;_tmp1EC;});struct Cyc_Dict_Dict _tmp3C6=ie->aggrdecls;_tmp3C7(Cyc_Interface_add_aggrdecl,_tmp3C6,tds);});
tds=({struct Cyc_List_List*(*_tmp3C9)(struct Cyc_List_List*(*f)(struct _tuple0*,struct Cyc_Absyn_Datatypedecl*,struct Cyc_List_List*),struct Cyc_Dict_Dict d,struct Cyc_List_List*accum)=({struct Cyc_List_List*(*_tmp1ED)(struct Cyc_List_List*(*f)(struct _tuple0*,struct Cyc_Absyn_Datatypedecl*,struct Cyc_List_List*),struct Cyc_Dict_Dict d,struct Cyc_List_List*accum)=(struct Cyc_List_List*(*)(struct Cyc_List_List*(*f)(struct _tuple0*,struct Cyc_Absyn_Datatypedecl*,struct Cyc_List_List*),struct Cyc_Dict_Dict d,struct Cyc_List_List*accum))Cyc_Dict_fold;_tmp1ED;});struct Cyc_Dict_Dict _tmp3C8=ie->datatypedecls;_tmp3C9(Cyc_Interface_add_datatypedecl,_tmp3C8,tds);});
tds=({struct Cyc_List_List*(*_tmp3CB)(struct Cyc_List_List*(*f)(struct _tuple0*,struct Cyc_Absyn_Enumdecl*,struct Cyc_List_List*),struct Cyc_Dict_Dict d,struct Cyc_List_List*accum)=({struct Cyc_List_List*(*_tmp1EE)(struct Cyc_List_List*(*f)(struct _tuple0*,struct Cyc_Absyn_Enumdecl*,struct Cyc_List_List*),struct Cyc_Dict_Dict d,struct Cyc_List_List*accum)=(struct Cyc_List_List*(*)(struct Cyc_List_List*(*f)(struct _tuple0*,struct Cyc_Absyn_Enumdecl*,struct Cyc_List_List*),struct Cyc_Dict_Dict d,struct Cyc_List_List*accum))Cyc_Dict_fold;_tmp1EE;});struct Cyc_Dict_Dict _tmp3CA=ie->enumdecls;_tmp3CB(Cyc_Interface_add_enumdecl,_tmp3CA,tds);});
tds=({struct Cyc_List_List*(*_tmp3CD)(struct Cyc_List_List*(*f)(struct _tuple0*,struct Cyc_Absyn_Vardecl*,struct Cyc_List_List*),struct Cyc_Dict_Dict d,struct Cyc_List_List*accum)=({struct Cyc_List_List*(*_tmp1EF)(struct Cyc_List_List*(*f)(struct _tuple0*,struct Cyc_Absyn_Vardecl*,struct Cyc_List_List*),struct Cyc_Dict_Dict d,struct Cyc_List_List*accum)=(struct Cyc_List_List*(*)(struct Cyc_List_List*(*f)(struct _tuple0*,struct Cyc_Absyn_Vardecl*,struct Cyc_List_List*),struct Cyc_Dict_Dict d,struct Cyc_List_List*accum))Cyc_Dict_fold;_tmp1EF;});struct Cyc_Dict_Dict _tmp3CC=ie->vardecls;_tmp3CD(Cyc_Interface_add_vardecl,_tmp3CC,tds);});
tds=({struct Cyc_List_List*(*_tmp3CF)(struct Cyc_List_List*(*f)(struct _tuple0*,struct Cyc_Tcdecl_Xdatatypefielddecl*,struct Cyc_List_List*),struct Cyc_Dict_Dict d,struct Cyc_List_List*accum)=({struct Cyc_List_List*(*_tmp1F0)(struct Cyc_List_List*(*f)(struct _tuple0*,struct Cyc_Tcdecl_Xdatatypefielddecl*,struct Cyc_List_List*),struct Cyc_Dict_Dict d,struct Cyc_List_List*accum)=(struct Cyc_List_List*(*)(struct Cyc_List_List*(*f)(struct _tuple0*,struct Cyc_Tcdecl_Xdatatypefielddecl*,struct Cyc_List_List*),struct Cyc_Dict_Dict d,struct Cyc_List_List*accum))Cyc_Dict_fold;_tmp1F0;});struct Cyc_Dict_Dict _tmp3CE=ie->xdatatypefielddecls;_tmp3CF(Cyc_Interface_add_xdatatypefielddecl,_tmp3CE,tds);});
# 1141
if(tds != 0){
tds=Cyc_List_imp_rev(tds);
tds=Cyc_Interface_add_namespace(tds);
Cyc_Absynpp_decllist2file(tds,f);}}
# 1041
void Cyc_Interface_print(struct Cyc_Interface_I*i,struct Cyc___cycFILE*f){
# 1152
Cyc_Absynpp_set_params(& Cyc_Absynpp_cyci_params_r);
({void*_tmp1F1=0U;({struct Cyc___cycFILE*_tmp3D1=f;struct _fat_ptr _tmp3D0=({const char*_tmp1F2="/****** needed (headers) ******/\n";_tag_fat(_tmp1F2,sizeof(char),34U);});Cyc_fprintf(_tmp3D1,_tmp3D0,_tag_fat(_tmp1F1,sizeof(void*),0U));});});
Cyc_Interface_print_ns_headers(f,i->imports);
# 1156
({void*_tmp1F3=0U;({struct Cyc___cycFILE*_tmp3D3=f;struct _fat_ptr _tmp3D2=({const char*_tmp1F4="\n/****** provided (headers) ******/\n";_tag_fat(_tmp1F4,sizeof(char),37U);});Cyc_fprintf(_tmp3D3,_tmp3D2,_tag_fat(_tmp1F3,sizeof(void*),0U));});});
Cyc_Interface_print_ns_headers(f,i->exports);
# 1161
({void*_tmp1F5=0U;({struct Cyc___cycFILE*_tmp3D5=f;struct _fat_ptr _tmp3D4=({const char*_tmp1F6="\n/****** needed (headers of xdatatypefielddecls) ******/\n";_tag_fat(_tmp1F6,sizeof(char),58U);});Cyc_fprintf(_tmp3D5,_tmp3D4,_tag_fat(_tmp1F5,sizeof(void*),0U));});});
Cyc_Interface_print_ns_xdatatypefielddecl_headers(f,i->imports);
# 1164
({void*_tmp1F7=0U;({struct Cyc___cycFILE*_tmp3D7=f;struct _fat_ptr _tmp3D6=({const char*_tmp1F8="\n/****** provided (headers of xdatatypefielddecls) ******/\n";_tag_fat(_tmp1F8,sizeof(char),60U);});Cyc_fprintf(_tmp3D7,_tmp3D6,_tag_fat(_tmp1F7,sizeof(void*),0U));});});
Cyc_Interface_print_ns_xdatatypefielddecl_headers(f,i->exports);
# 1168
({void*_tmp1F9=0U;({struct Cyc___cycFILE*_tmp3D9=f;struct _fat_ptr _tmp3D8=({const char*_tmp1FA="\n/****** provided (typedefs) ******/\n";_tag_fat(_tmp1FA,sizeof(char),38U);});Cyc_fprintf(_tmp3D9,_tmp3D8,_tag_fat(_tmp1F9,sizeof(void*),0U));});});
Cyc_Interface_print_ns_typedefs(f,i->exports);
# 1171
({void*_tmp1FB=0U;({struct Cyc___cycFILE*_tmp3DB=f;struct _fat_ptr _tmp3DA=({const char*_tmp1FC="\n/****** needed (declarations) ******/\n";_tag_fat(_tmp1FC,sizeof(char),40U);});Cyc_fprintf(_tmp3DB,_tmp3DA,_tag_fat(_tmp1FB,sizeof(void*),0U));});});
Cyc_Interface_print_ns_decls(f,i->imports);
# 1174
({void*_tmp1FD=0U;({struct Cyc___cycFILE*_tmp3DD=f;struct _fat_ptr _tmp3DC=({const char*_tmp1FE="\n/****** provided (declarations) ******/\n";_tag_fat(_tmp1FE,sizeof(char),42U);});Cyc_fprintf(_tmp3DD,_tmp3DC,_tag_fat(_tmp1FD,sizeof(void*),0U));});});
Cyc_Interface_print_ns_decls(f,i->exports);}
# 1041
struct Cyc_Interface_I*Cyc_Interface_parse(struct Cyc___cycFILE*f){
# 1182
Cyc_Lex_lex_init();{
struct Cyc_List_List*tds=({struct Cyc___cycFILE*_tmp3DE=f;Cyc_Parse_parse_file(_tmp3DE,({const char*_tmp1FF="unknown file name";_tag_fat(_tmp1FF,sizeof(char),18U);}));});
Cyc_Binding_resolve_all(tds);{
struct Cyc_Tcenv_Tenv*te=Cyc_Tcenv_tc_init();
Cyc_Tc_tc(te,0,tds);
return Cyc_Interface_gen_extract(te->ae,0,tds);}}}
# 1041
void Cyc_Interface_save(struct Cyc_Interface_I*i,struct Cyc___cycFILE*f){
# 1195
Cyc_Interface_print(i,f);}
# 1041
struct Cyc_Interface_I*Cyc_Interface_load(struct Cyc___cycFILE*f){
# 1201
return Cyc_Interface_parse(f);}
