#include <setjmp.h>
/* This is a C header file to be used by the output of the Cyclone to
   C translator.  The corresponding definitions are in file lib/runtime_*.c */
#ifndef _CYC_INCLUDE_H_
#define _CYC_INCLUDE_H_

/* Need one of these per thread (see runtime_stack.c). The runtime maintains 
   a stack that contains either _handler_cons structs or _RegionHandle structs.
   The tag is 0 for a handler_cons and 1 for a region handle.  */
struct _RuntimeStack {
  int tag; 
  struct _RuntimeStack *next;
  void (*cleanup)(struct _RuntimeStack *frame);
};

#ifndef offsetof
/* should be size_t, but int is fine. */
#define offsetof(t,n) ((int)(&(((t *)0)->n)))
#endif

/* Fat pointers */
struct _fat_ptr {
  unsigned char *curr; 
  unsigned char *base; 
  unsigned char *last_plus_one; 
};  

/* Discriminated Unions */
struct _xtunion_struct { char *tag; };

/* Regions */
struct _RegionPage
#ifdef CYC_REGION_PROFILE
{ unsigned total_bytes;
  unsigned free_bytes;
  /* MWH: wish we didn't have to include the stuff below ... */
  struct _RegionPage *next;
  char data[1];
}
#endif
; // abstract -- defined in runtime_memory.c
struct _RegionHandle {
  struct _RuntimeStack s;
  struct _RegionPage *curr;
  char               *offset;
  char               *last_plus_one;
  struct _DynRegionHandle *sub_regions;
#ifdef CYC_REGION_PROFILE
  const char         *name;
#else
  unsigned used_bytes;
  unsigned wasted_bytes;
#endif
};
struct _DynRegionFrame {
  struct _RuntimeStack s;
  struct _DynRegionHandle *x;
};
// A dynamic region is just a region handle.  The wrapper struct is for type
// abstraction.
struct Cyc_Core_DynamicRegion {
  struct _RegionHandle h;
};

struct _RegionHandle _new_region(const char*);
void* _region_malloc(struct _RegionHandle*, unsigned);
void* _region_calloc(struct _RegionHandle*, unsigned t, unsigned n);
void   _free_region(struct _RegionHandle*);
struct _RegionHandle*_open_dynregion(struct _DynRegionFrame*,struct _DynRegionHandle*);
void   _pop_dynregion();

/* Exceptions */
struct _handler_cons {
  struct _RuntimeStack s;
  jmp_buf handler;
};
void _push_handler(struct _handler_cons *);
void _push_region(struct _RegionHandle *);
void _npop_handler(int);
void _pop_handler();
void _pop_region();

#ifndef _throw
void* _throw_null_fn(const char*,unsigned);
void* _throw_arraybounds_fn(const char*,unsigned);
void* _throw_badalloc_fn(const char*,unsigned);
void* _throw_match_fn(const char*,unsigned);
void* _throw_fn(void*,const char*,unsigned);
void* _rethrow(void*);
#define _throw_null() (_throw_null_fn(__FILE__,__LINE__))
#define _throw_arraybounds() (_throw_arraybounds_fn(__FILE__,__LINE__))
#define _throw_badalloc() (_throw_badalloc_fn(__FILE__,__LINE__))
#define _throw_match() (_throw_match_fn(__FILE__,__LINE__))
#define _throw(e) (_throw_fn((e),__FILE__,__LINE__))
#endif

struct _xtunion_struct* Cyc_Core_get_exn_thrown();
/* Built-in Exceptions */
struct Cyc_Null_Exception_exn_struct { char *tag; };
struct Cyc_Array_bounds_exn_struct { char *tag; };
struct Cyc_Match_Exception_exn_struct { char *tag; };
struct Cyc_Bad_alloc_exn_struct { char *tag; };
extern char Cyc_Null_Exception[];
extern char Cyc_Array_bounds[];
extern char Cyc_Match_Exception[];
extern char Cyc_Bad_alloc[];

/* Built-in Run-time Checks and company */
#ifdef CYC_ANSI_OUTPUT
#define _INLINE  
#else
#define _INLINE inline
#endif

#ifdef NO_CYC_NULL_CHECKS
#define _check_null(ptr) (ptr)
#else
#define _check_null(ptr) \
  ({ void*_cks_null = (void*)(ptr); \
     if (!_cks_null) _throw_null(); \
     _cks_null; })
#endif

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index)\
   (((char*)ptr) + (elt_sz)*(index))
#ifdef NO_CYC_NULL_CHECKS
#define _check_known_subscript_null _check_known_subscript_notnull
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr);\
  int _index = (index);\
  if (!_cks_ptr) _throw_null(); \
  _cks_ptr + (elt_sz)*_index; })
#endif
#define _zero_arr_plus_fn(orig_x,orig_sz,orig_i,f,l) ((orig_x)+(orig_i))
#define _zero_arr_plus_char_fn _zero_arr_plus_fn
#define _zero_arr_plus_short_fn _zero_arr_plus_fn
#define _zero_arr_plus_int_fn _zero_arr_plus_fn
#define _zero_arr_plus_float_fn _zero_arr_plus_fn
#define _zero_arr_plus_double_fn _zero_arr_plus_fn
#define _zero_arr_plus_longdouble_fn _zero_arr_plus_fn
#define _zero_arr_plus_voidstar_fn _zero_arr_plus_fn
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (!_cks_ptr) _throw_null(); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })

/* _zero_arr_plus_*_fn(x,sz,i,filename,lineno) adds i to zero-terminated ptr
   x that has at least sz elements */
char* _zero_arr_plus_char_fn(char*,unsigned,int,const char*,unsigned);
short* _zero_arr_plus_short_fn(short*,unsigned,int,const char*,unsigned);
int* _zero_arr_plus_int_fn(int*,unsigned,int,const char*,unsigned);
float* _zero_arr_plus_float_fn(float*,unsigned,int,const char*,unsigned);
double* _zero_arr_plus_double_fn(double*,unsigned,int,const char*,unsigned);
long double* _zero_arr_plus_longdouble_fn(long double*,unsigned,int,const char*, unsigned);
void** _zero_arr_plus_voidstar_fn(void**,unsigned,int,const char*,unsigned);
#endif

/* _get_zero_arr_size_*(x,sz) returns the number of elements in a
   zero-terminated array that is NULL or has at least sz elements */
int _get_zero_arr_size_char(const char*,unsigned);
int _get_zero_arr_size_short(const short*,unsigned);
int _get_zero_arr_size_int(const int*,unsigned);
int _get_zero_arr_size_float(const float*,unsigned);
int _get_zero_arr_size_double(const double*,unsigned);
int _get_zero_arr_size_longdouble(const long double*,unsigned);
int _get_zero_arr_size_voidstar(const void**,unsigned);

/* _zero_arr_inplace_plus_*_fn(x,i,filename,lineno) sets
   zero-terminated pointer *x to *x + i */
char* _zero_arr_inplace_plus_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_short_fn(short**,int,const char*,unsigned);
int* _zero_arr_inplace_plus_int(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_longdouble_fn(long double**,int,const char*,unsigned);
void** _zero_arr_inplace_plus_voidstar_fn(void***,int,const char*,unsigned);
/* like the previous functions, but does post-addition (as in e++) */
char* _zero_arr_inplace_plus_post_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_post_short_fn(short**x,int,const char*,unsigned);
int* _zero_arr_inplace_plus_post_int_fn(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_post_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_post_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_post_longdouble_fn(long double**,int,const char *,unsigned);
void** _zero_arr_inplace_plus_post_voidstar_fn(void***,int,const char*,unsigned);
#define _zero_arr_plus_char(x,s,i) \
  (_zero_arr_plus_char_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_short(x,s,i) \
  (_zero_arr_plus_short_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_int(x,s,i) \
  (_zero_arr_plus_int_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_float(x,s,i) \
  (_zero_arr_plus_float_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_double(x,s,i) \
  (_zero_arr_plus_double_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_longdouble(x,s,i) \
  (_zero_arr_plus_longdouble_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_voidstar(x,s,i) \
  (_zero_arr_plus_voidstar_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_inplace_plus_char(x,i) \
  _zero_arr_inplace_plus_char_fn((char **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_short(x,i) \
  _zero_arr_inplace_plus_short_fn((short **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_int(x,i) \
  _zero_arr_inplace_plus_int_fn((int **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_float(x,i) \
  _zero_arr_inplace_plus_float_fn((float **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_double(x,i) \
  _zero_arr_inplace_plus_double_fn((double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_longdouble(x,i) \
  _zero_arr_inplace_plus_longdouble_fn((long double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_voidstar(x,i) \
  _zero_arr_inplace_plus_voidstar_fn((void ***)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_char(x,i) \
  _zero_arr_inplace_plus_post_char_fn((char **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_short(x,i) \
  _zero_arr_inplace_plus_post_short_fn((short **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_int(x,i) \
  _zero_arr_inplace_plus_post_int_fn((int **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_float(x,i) \
  _zero_arr_inplace_plus_post_float_fn((float **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_double(x,i) \
  _zero_arr_inplace_plus_post_double_fn((double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_longdouble(x,i) \
  _zero_arr_inplace_plus_post_longdouble_fn((long double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_voidstar(x,i) \
  _zero_arr_inplace_plus_post_voidstar_fn((void***)(x),(i),__FILE__,__LINE__)

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_fat_subscript(arr,elt_sz,index) ((arr).curr + (elt_sz) * (index))
#define _untag_fat_ptr(arr,elt_sz,num_elts) ((arr).curr)
#else
#define _check_fat_subscript(arr,elt_sz,index) ({ \
  struct _fat_ptr _cus_arr = (arr); \
  unsigned char *_cus_ans = _cus_arr.curr + (elt_sz) * (index); \
  /* JGM: not needed! if (!_cus_arr.base) _throw_null();*/ \
  if (_cus_ans < _cus_arr.base || _cus_ans >= _cus_arr.last_plus_one) \
    _throw_arraybounds(); \
  _cus_ans; })
#define _untag_fat_ptr(arr,elt_sz,num_elts) ({ \
  struct _fat_ptr _arr = (arr); \
  unsigned char *_curr = _arr.curr; \
  if ((_curr < _arr.base || _curr + (elt_sz) * (num_elts) > _arr.last_plus_one) &&\
      _curr != (unsigned char *)0) \
    _throw_arraybounds(); \
  _curr; })
#endif

#define _tag_fat(tcurr,elt_sz,num_elts) ({ \
  struct _fat_ptr _ans; \
  unsigned _num_elts = (num_elts);\
  _ans.base = _ans.curr = (void*)(tcurr); \
  /* JGM: if we're tagging NULL, ignore num_elts */ \
  _ans.last_plus_one = _ans.base ? (_ans.base + (elt_sz) * _num_elts) : 0; \
  _ans; })

#define _get_fat_size(arr,elt_sz) \
  ({struct _fat_ptr _arr = (arr); \
    unsigned char *_arr_curr=_arr.curr; \
    unsigned char *_arr_last=_arr.last_plus_one; \
    (_arr_curr < _arr.base || _arr_curr >= _arr_last) ? 0 : \
    ((_arr_last - _arr_curr) / (elt_sz));})

#define _fat_ptr_plus(arr,elt_sz,change) ({ \
  struct _fat_ptr _ans = (arr); \
  int _change = (change);\
  _ans.curr += (elt_sz) * _change;\
  _ans; })
#define _fat_ptr_inplace_plus(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  _arr_ptr->curr += (elt_sz) * (change);\
  *_arr_ptr; })
#define _fat_ptr_inplace_plus_post(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  struct _fat_ptr _ans = *_arr_ptr; \
  _arr_ptr->curr += (elt_sz) * (change);\
  _ans; })

//Not a macro since initialization order matters. Defined in runtime_zeroterm.c.
struct _fat_ptr _fat_ptr_decrease_size(struct _fat_ptr,unsigned sz,unsigned numelts);

/* Allocation */
void* GC_malloc(int);
void* GC_malloc_atomic(int);
void* GC_calloc(unsigned,unsigned);
void* GC_calloc_atomic(unsigned,unsigned);
// bound the allocation size to be < MAX_ALLOC_SIZE. See macros below for usage.
#define MAX_MALLOC_SIZE (1 << 28)
void* _bounded_GC_malloc(int,const char*,int);
void* _bounded_GC_malloc_atomic(int,const char*,int);
void* _bounded_GC_calloc(unsigned,unsigned,const char*,int);
void* _bounded_GC_calloc_atomic(unsigned,unsigned,const char*,int);
/* these macros are overridden below ifdef CYC_REGION_PROFILE */
#ifndef CYC_REGION_PROFILE
#define _cycalloc(n) _bounded_GC_malloc(n,__FILE__,__LINE__)
#define _cycalloc_atomic(n) _bounded_GC_malloc_atomic(n,__FILE__,__LINE__)
#define _cyccalloc(n,s) _bounded_GC_calloc(n,s,__FILE__,__LINE__)
#define _cyccalloc_atomic(n,s) _bounded_GC_calloc_atomic(n,s,__FILE__,__LINE__)
#endif

static _INLINE unsigned int _check_times(unsigned x, unsigned y) {
  unsigned long long whole_ans = 
    ((unsigned long long) x)*((unsigned long long)y);
  unsigned word_ans = (unsigned)whole_ans;
  if(word_ans < whole_ans || word_ans > MAX_MALLOC_SIZE)
    _throw_badalloc();
  return word_ans;
}

#define _CYC_MAX_REGION_CONST 2
#define _CYC_MIN_ALIGNMENT (sizeof(double))

#ifdef CYC_REGION_PROFILE
extern int rgn_total_bytes;
#endif

static _INLINE void *_fast_region_malloc(struct _RegionHandle *r, unsigned orig_s) {  
  if (r > (struct _RegionHandle *)_CYC_MAX_REGION_CONST && r->curr != 0) { 
#ifdef CYC_NOALIGN
    unsigned s =  orig_s;
#else
    unsigned s =  (orig_s + _CYC_MIN_ALIGNMENT - 1) & (~(_CYC_MIN_ALIGNMENT -1)); 
#endif
    char *result; 
    result = r->offset; 
    if (s <= (r->last_plus_one - result)) {
      r->offset = result + s; 
#ifdef CYC_REGION_PROFILE
    r->curr->free_bytes = r->curr->free_bytes - s;
    rgn_total_bytes += s;
#endif
      return result;
    }
  } 
  return _region_malloc(r,orig_s); 
}

#ifdef CYC_REGION_PROFILE
/* see macros below for usage. defined in runtime_memory.c */
void* _profile_GC_malloc(int,const char*,const char*,int);
void* _profile_GC_malloc_atomic(int,const char*,const char*,int);
void* _profile_GC_calloc(unsigned,unsigned,const char*,const char*,int);
void* _profile_GC_calloc_atomic(unsigned,unsigned,const char*,const char*,int);
void* _profile_region_malloc(struct _RegionHandle*,unsigned,const char*,const char*,int);
void* _profile_region_calloc(struct _RegionHandle*,unsigned,unsigned,const char *,const char*,int);
struct _RegionHandle _profile_new_region(const char*,const char*,const char*,int);
void _profile_free_region(struct _RegionHandle*,const char*,const char*,int);
#ifndef RUNTIME_CYC
#define _new_region(n) _profile_new_region(n,__FILE__,__FUNCTION__,__LINE__)
#define _free_region(r) _profile_free_region(r,__FILE__,__FUNCTION__,__LINE__)
#define _region_malloc(rh,n) _profile_region_malloc(rh,n,__FILE__,__FUNCTION__,__LINE__)
#define _region_calloc(rh,n,t) _profile_region_calloc(rh,n,t,__FILE__,__FUNCTION__,__LINE__)
#  endif
#define _cycalloc(n) _profile_GC_malloc(n,__FILE__,__FUNCTION__,__LINE__)
#define _cycalloc_atomic(n) _profile_GC_malloc_atomic(n,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc(n,s) _profile_GC_calloc(n,s,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc_atomic(n,s) _profile_GC_calloc_atomic(n,s,__FILE__,__FUNCTION__,__LINE__)
#endif
#endif
 struct Cyc_Core_Opt{void*v;};struct _tuple0{void*f1;void*f2;};
# 111 "core.h"
void*Cyc_Core_snd(struct _tuple0*);
# 126
int Cyc_Core_ptrcmp(void*,void*);extern char Cyc_Core_Invalid_argument[17U];struct Cyc_Core_Invalid_argument_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Failure[8U];struct Cyc_Core_Failure_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Impossible[11U];struct Cyc_Core_Impossible_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Not_found[10U];struct Cyc_Core_Not_found_exn_struct{char*tag;};extern char Cyc_Core_Unreachable[12U];struct Cyc_Core_Unreachable_exn_struct{char*tag;struct _fat_ptr f1;};
# 165
extern struct _RegionHandle*Cyc_Core_heap_region;
# 168
extern struct _RegionHandle*Cyc_Core_unique_region;struct Cyc_Core_DynamicRegion;struct Cyc_Core_NewDynamicRegion{struct Cyc_Core_DynamicRegion*key;};
# 275 "core.h"
void Cyc_Core_rethrow(void*);struct Cyc_Core_ThinRes{void*arr;unsigned nelts;};struct Cyc_List_List{void*hd;struct Cyc_List_List*tl;};
# 54 "list.h"
struct Cyc_List_List*Cyc_List_list(struct _fat_ptr);
# 61
int Cyc_List_length(struct Cyc_List_List*x);
# 70
struct Cyc_List_List*Cyc_List_copy(struct Cyc_List_List*x);
# 76
struct Cyc_List_List*Cyc_List_map(void*(*f)(void*),struct Cyc_List_List*x);extern char Cyc_List_List_mismatch[14U];struct Cyc_List_List_mismatch_exn_struct{char*tag;};
# 135
void Cyc_List_iter_c(void(*f)(void*,void*),void*env,struct Cyc_List_List*x);
# 172
struct Cyc_List_List*Cyc_List_rev(struct Cyc_List_List*x);
# 178
struct Cyc_List_List*Cyc_List_imp_rev(struct Cyc_List_List*x);
# 184
struct Cyc_List_List*Cyc_List_append(struct Cyc_List_List*x,struct Cyc_List_List*y);extern char Cyc_List_Nth[4U];struct Cyc_List_Nth_exn_struct{char*tag;};
# 242
void*Cyc_List_nth(struct Cyc_List_List*x,int n);
# 276
struct Cyc_List_List*Cyc_List_rzip(struct _RegionHandle*r1,struct _RegionHandle*r2,struct Cyc_List_List*x,struct Cyc_List_List*y);struct _tuple1{struct Cyc_List_List*f1;struct Cyc_List_List*f2;};
# 294
struct _tuple1 Cyc_List_split(struct Cyc_List_List*x);
# 319
int Cyc_List_memq(struct Cyc_List_List*l,void*x);
# 37 "position.h"
struct _fat_ptr Cyc_Position_string_of_segment(unsigned);struct Cyc_Position_Error;
# 47
extern int Cyc_Position_num_errors;struct Cyc_Relations_Reln;struct _union_Nmspace_Rel_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_Abs_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_C_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_Loc_n{int tag;int val;};union Cyc_Absyn_Nmspace{struct _union_Nmspace_Rel_n Rel_n;struct _union_Nmspace_Abs_n Abs_n;struct _union_Nmspace_C_n C_n;struct _union_Nmspace_Loc_n Loc_n;};struct _tuple2{union Cyc_Absyn_Nmspace f1;struct _fat_ptr*f2;};
# 161 "absyn.h"
enum Cyc_Absyn_Scope{Cyc_Absyn_Static =0U,Cyc_Absyn_Abstract =1U,Cyc_Absyn_Public =2U,Cyc_Absyn_Extern =3U,Cyc_Absyn_ExternC =4U,Cyc_Absyn_Register =5U};struct Cyc_Absyn_Tqual{int print_const: 1;int q_volatile: 1;int q_restrict: 1;int real_const: 1;unsigned loc;};
# 182
enum Cyc_Absyn_Size_of{Cyc_Absyn_Char_sz =0U,Cyc_Absyn_Short_sz =1U,Cyc_Absyn_Int_sz =2U,Cyc_Absyn_Long_sz =3U,Cyc_Absyn_LongLong_sz =4U};
enum Cyc_Absyn_Sign{Cyc_Absyn_Signed =0U,Cyc_Absyn_Unsigned =1U,Cyc_Absyn_None =2U};
enum Cyc_Absyn_AggrKind{Cyc_Absyn_StructA =0U,Cyc_Absyn_UnionA =1U};
# 187
enum Cyc_Absyn_AliasQual{Cyc_Absyn_Aliasable =0U,Cyc_Absyn_Unique =1U,Cyc_Absyn_Top =2U};
# 192
enum Cyc_Absyn_KindQual{Cyc_Absyn_AnyKind =0U,Cyc_Absyn_MemKind =1U,Cyc_Absyn_BoxKind =2U,Cyc_Absyn_RgnKind =3U,Cyc_Absyn_EffKind =4U,Cyc_Absyn_IntKind =5U,Cyc_Absyn_BoolKind =6U,Cyc_Absyn_PtrBndKind =7U};struct Cyc_Absyn_Kind{enum Cyc_Absyn_KindQual kind;enum Cyc_Absyn_AliasQual aliasqual;};struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct{int tag;struct Cyc_Absyn_Kind*f1;};struct Cyc_Absyn_Unknown_kb_Absyn_KindBound_struct{int tag;struct Cyc_Core_Opt*f1;};struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct{int tag;struct Cyc_Core_Opt*f1;struct Cyc_Absyn_Kind*f2;};struct Cyc_Absyn_Tvar{struct _fat_ptr*name;int identity;void*kind;};struct Cyc_Absyn_PtrLoc{unsigned ptr_loc;unsigned rgn_loc;unsigned zt_loc;};struct Cyc_Absyn_PtrAtts{void*rgn;void*nullable;void*bounds;void*zero_term;struct Cyc_Absyn_PtrLoc*ptrloc;};struct Cyc_Absyn_PtrInfo{void*elt_type;struct Cyc_Absyn_Tqual elt_tq;struct Cyc_Absyn_PtrAtts ptr_atts;};struct Cyc_Absyn_VarargInfo{struct _fat_ptr*name;struct Cyc_Absyn_Tqual tq;void*type;int inject;};struct Cyc_Absyn_Star_Absyn_Cap_struct{int tag;};struct Cyc_Absyn_Bot_Absyn_Cap_struct{int tag;};struct Cyc_Absyn_Nat_Absyn_Cap_struct{int tag;struct Cyc_Absyn_Exp*f1;int f2;};struct Cyc_Absyn_RgnEffect{void*name;void*rgn;void*lock;void*unadopt;void*parent;};struct Cyc_Absyn_FnInfo{struct Cyc_List_List*tvars;void*effect;struct Cyc_Absyn_Tqual ret_tqual;void*ret_type;struct Cyc_List_List*args;int c_varargs;struct Cyc_Absyn_VarargInfo*cyc_varargs;struct Cyc_List_List*rgn_po;struct Cyc_List_List*attributes;struct Cyc_Absyn_Exp*requires_clause;struct Cyc_List_List*requires_relns;struct Cyc_Absyn_Exp*ensures_clause;struct Cyc_List_List*ensures_relns;struct Cyc_List_List*ieffect;struct Cyc_List_List*oeffect;};struct Cyc_Absyn_UnknownDatatypeInfo{struct _tuple2*name;int is_extensible;};struct _union_DatatypeInfo_UnknownDatatype{int tag;struct Cyc_Absyn_UnknownDatatypeInfo val;};struct _union_DatatypeInfo_KnownDatatype{int tag;struct Cyc_Absyn_Datatypedecl**val;};union Cyc_Absyn_DatatypeInfo{struct _union_DatatypeInfo_UnknownDatatype UnknownDatatype;struct _union_DatatypeInfo_KnownDatatype KnownDatatype;};struct Cyc_Absyn_UnknownDatatypeFieldInfo{struct _tuple2*datatype_name;struct _tuple2*field_name;int is_extensible;};struct _union_DatatypeFieldInfo_UnknownDatatypefield{int tag;struct Cyc_Absyn_UnknownDatatypeFieldInfo val;};struct _tuple3{struct Cyc_Absyn_Datatypedecl*f1;struct Cyc_Absyn_Datatypefield*f2;};struct _union_DatatypeFieldInfo_KnownDatatypefield{int tag;struct _tuple3 val;};union Cyc_Absyn_DatatypeFieldInfo{struct _union_DatatypeFieldInfo_UnknownDatatypefield UnknownDatatypefield;struct _union_DatatypeFieldInfo_KnownDatatypefield KnownDatatypefield;};struct _tuple4{enum Cyc_Absyn_AggrKind f1;struct _tuple2*f2;struct Cyc_Core_Opt*f3;};struct _union_AggrInfo_UnknownAggr{int tag;struct _tuple4 val;};struct _union_AggrInfo_KnownAggr{int tag;struct Cyc_Absyn_Aggrdecl**val;};union Cyc_Absyn_AggrInfo{struct _union_AggrInfo_UnknownAggr UnknownAggr;struct _union_AggrInfo_KnownAggr KnownAggr;};struct Cyc_Absyn_ArrayInfo{void*elt_type;struct Cyc_Absyn_Tqual tq;struct Cyc_Absyn_Exp*num_elts;void*zero_term;unsigned zt_loc;};struct Cyc_Absyn_Aggr_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Aggrdecl*f1;};struct Cyc_Absyn_Enum_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;};struct Cyc_Absyn_Datatype_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;};struct Cyc_Absyn_TypeDecl{void*r;unsigned loc;};struct Cyc_Absyn_VoidCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_IntCon_Absyn_TyCon_struct{int tag;enum Cyc_Absyn_Sign f1;enum Cyc_Absyn_Size_of f2;};struct Cyc_Absyn_FloatCon_Absyn_TyCon_struct{int tag;int f1;};struct Cyc_Absyn_RgnHandleCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_TagCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_HeapCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_UniqueCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_RefCntCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_AccessCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_JoinCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_RgnsCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_TrueCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_FalseCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_ThinCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_FatCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_EnumCon_Absyn_TyCon_struct{int tag;struct _tuple2*f1;struct Cyc_Absyn_Enumdecl*f2;};struct Cyc_Absyn_AnonEnumCon_Absyn_TyCon_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_BuiltinCon_Absyn_TyCon_struct{int tag;struct _fat_ptr f1;struct Cyc_Absyn_Kind*f2;};struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_DatatypeInfo f1;};struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_DatatypeFieldInfo f1;};struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_AggrInfo f1;};struct Cyc_Absyn_AppType_Absyn_Type_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Evar_Absyn_Type_struct{int tag;struct Cyc_Core_Opt*f1;void*f2;int f3;struct Cyc_Core_Opt*f4;};struct Cyc_Absyn_VarType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Tvar*f1;};struct Cyc_Absyn_PointerType_Absyn_Type_struct{int tag;struct Cyc_Absyn_PtrInfo f1;};struct Cyc_Absyn_ArrayType_Absyn_Type_struct{int tag;struct Cyc_Absyn_ArrayInfo f1;};struct Cyc_Absyn_FnType_Absyn_Type_struct{int tag;struct Cyc_Absyn_FnInfo f1;};struct Cyc_Absyn_TupleType_Absyn_Type_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct{int tag;enum Cyc_Absyn_AggrKind f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_TypedefType_Absyn_Type_struct{int tag;struct _tuple2*f1;struct Cyc_List_List*f2;struct Cyc_Absyn_Typedefdecl*f3;void*f4;};struct Cyc_Absyn_ValueofType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct{int tag;struct Cyc_Absyn_TypeDecl*f1;void**f2;};struct Cyc_Absyn_TypeofType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_NoTypes_Absyn_Funcparams_struct{int tag;struct Cyc_List_List*f1;unsigned f2;};struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct{int tag;struct Cyc_List_List*f1;int f2;struct Cyc_Absyn_VarargInfo*f3;void*f4;struct Cyc_List_List*f5;struct Cyc_Absyn_Exp*f6;struct Cyc_Absyn_Exp*f7;struct Cyc_List_List*f8;struct Cyc_List_List*f9;};
# 437 "absyn.h"
enum Cyc_Absyn_Format_Type{Cyc_Absyn_Printf_ft =0U,Cyc_Absyn_Scanf_ft =1U};struct Cyc_Absyn_Regparm_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Stdcall_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Cdecl_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Fastcall_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Noreturn_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Const_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Aligned_att_Absyn_Attribute_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Packed_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Section_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Nocommon_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Shared_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Unused_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Weak_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Dllimport_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Dllexport_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_instrument_function_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Constructor_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Destructor_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_check_memory_usage_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Format_att_Absyn_Attribute_struct{int tag;enum Cyc_Absyn_Format_Type f1;int f2;int f3;};struct Cyc_Absyn_Initializes_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Noliveunique_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Consume_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Pure_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Mode_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Alias_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Always_inline_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_throw_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Non_null_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Deprecated_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Carray_mod_Absyn_Type_modifier_struct{int tag;void*f1;unsigned f2;};struct Cyc_Absyn_ConstArray_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_Absyn_Exp*f1;void*f2;unsigned f3;};struct Cyc_Absyn_Pointer_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_Absyn_PtrAtts f1;struct Cyc_Absyn_Tqual f2;};struct Cyc_Absyn_Function_mod_Absyn_Type_modifier_struct{int tag;void*f1;};struct Cyc_Absyn_TypeParams_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_List_List*f1;unsigned f2;int f3;};struct Cyc_Absyn_Attributes_mod_Absyn_Type_modifier_struct{int tag;unsigned f1;struct Cyc_List_List*f2;};struct _union_Cnst_Null_c{int tag;int val;};struct _tuple5{enum Cyc_Absyn_Sign f1;char f2;};struct _union_Cnst_Char_c{int tag;struct _tuple5 val;};struct _union_Cnst_Wchar_c{int tag;struct _fat_ptr val;};struct _tuple6{enum Cyc_Absyn_Sign f1;short f2;};struct _union_Cnst_Short_c{int tag;struct _tuple6 val;};struct _tuple7{enum Cyc_Absyn_Sign f1;int f2;};struct _union_Cnst_Int_c{int tag;struct _tuple7 val;};struct _tuple8{enum Cyc_Absyn_Sign f1;long long f2;};struct _union_Cnst_LongLong_c{int tag;struct _tuple8 val;};struct _tuple9{struct _fat_ptr f1;int f2;};struct _union_Cnst_Float_c{int tag;struct _tuple9 val;};struct _union_Cnst_String_c{int tag;struct _fat_ptr val;};struct _union_Cnst_Wstring_c{int tag;struct _fat_ptr val;};union Cyc_Absyn_Cnst{struct _union_Cnst_Null_c Null_c;struct _union_Cnst_Char_c Char_c;struct _union_Cnst_Wchar_c Wchar_c;struct _union_Cnst_Short_c Short_c;struct _union_Cnst_Int_c Int_c;struct _union_Cnst_LongLong_c LongLong_c;struct _union_Cnst_Float_c Float_c;struct _union_Cnst_String_c String_c;struct _union_Cnst_Wstring_c Wstring_c;};
# 532
enum Cyc_Absyn_Primop{Cyc_Absyn_Plus =0U,Cyc_Absyn_Times =1U,Cyc_Absyn_Minus =2U,Cyc_Absyn_Div =3U,Cyc_Absyn_Mod =4U,Cyc_Absyn_Eq =5U,Cyc_Absyn_Neq =6U,Cyc_Absyn_Gt =7U,Cyc_Absyn_Lt =8U,Cyc_Absyn_Gte =9U,Cyc_Absyn_Lte =10U,Cyc_Absyn_Not =11U,Cyc_Absyn_Bitnot =12U,Cyc_Absyn_Bitand =13U,Cyc_Absyn_Bitor =14U,Cyc_Absyn_Bitxor =15U,Cyc_Absyn_Bitlshift =16U,Cyc_Absyn_Bitlrshift =17U,Cyc_Absyn_Numelts =18U};
# 539
enum Cyc_Absyn_Incrementor{Cyc_Absyn_PreInc =0U,Cyc_Absyn_PostInc =1U,Cyc_Absyn_PreDec =2U,Cyc_Absyn_PostDec =3U};struct Cyc_Absyn_VarargCallInfo{int num_varargs;struct Cyc_List_List*injectors;struct Cyc_Absyn_VarargInfo*vai;};struct Cyc_Absyn_StructField_Absyn_OffsetofField_struct{int tag;struct _fat_ptr*f1;};struct Cyc_Absyn_TupleIndex_Absyn_OffsetofField_struct{int tag;unsigned f1;};
# 557
enum Cyc_Absyn_Coercion{Cyc_Absyn_Unknown_coercion =0U,Cyc_Absyn_No_coercion =1U,Cyc_Absyn_Null_to_NonNull =2U,Cyc_Absyn_Other_coercion =3U};struct Cyc_Absyn_MallocInfo{int is_calloc;struct Cyc_Absyn_Exp*rgn;void**elt_type;struct Cyc_Absyn_Exp*num_elts;int fat_result;int inline_call;};struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct{int tag;union Cyc_Absyn_Cnst f1;};struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Pragma_e_Absyn_Raw_exp_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct{int tag;enum Cyc_Absyn_Primop f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_AssignOp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Core_Opt*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_Increment_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;enum Cyc_Absyn_Incrementor f2;};struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;struct Cyc_Absyn_VarargCallInfo*f3;int f4;};struct Cyc_Absyn_Throw_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;int f2;};struct Cyc_Absyn_NoInstantiate_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Instantiate_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_Absyn_Exp*f2;int f3;enum Cyc_Absyn_Coercion f4;};struct Cyc_Absyn_Address_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_New_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Sizeoftype_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Sizeofexp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Offsetof_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Deref_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;int f3;int f4;};struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;int f3;int f4;};struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Tuple_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;};struct _tuple10{struct _fat_ptr*f1;struct Cyc_Absyn_Tqual f2;void*f3;};struct Cyc_Absyn_CompoundLit_e_Absyn_Raw_exp_struct{int tag;struct _tuple10*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Array_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Exp*f2;struct Cyc_Absyn_Exp*f3;int f4;};struct Cyc_Absyn_ComprehensionNoinit_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;void*f2;int f3;};struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct{int tag;struct _tuple2*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;struct Cyc_Absyn_Aggrdecl*f4;};struct Cyc_Absyn_AnonStruct_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Datatype_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;struct Cyc_Absyn_Datatypedecl*f2;struct Cyc_Absyn_Datatypefield*f3;};struct Cyc_Absyn_Enum_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_AnonEnum_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_Spawn_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_MallocInfo f1;};struct Cyc_Absyn_Swap_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_UnresolvedMem_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Core_Opt*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_StmtExp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Stmt*f1;};struct Cyc_Absyn_Tagcheck_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;};struct Cyc_Absyn_Valueof_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Asm_e_Absyn_Raw_exp_struct{int tag;int f1;struct _fat_ptr f2;struct Cyc_List_List*f3;struct Cyc_List_List*f4;struct Cyc_List_List*f5;};struct Cyc_Absyn_Extension_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Exp{void*topt;void*r;unsigned loc;void*annot;};struct Cyc_Absyn_Skip_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Exp_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Seq_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Return_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_IfThenElse_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Stmt*f2;struct Cyc_Absyn_Stmt*f3;};struct _tuple11{struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_While_s_Absyn_Raw_stmt_struct{int tag;struct _tuple11 f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Break_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Continue_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Goto_s_Absyn_Raw_stmt_struct{int tag;struct _fat_ptr*f1;};struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _tuple11 f2;struct _tuple11 f3;struct Cyc_Absyn_Stmt*f4;};struct Cyc_Absyn_Switch_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;void*f3;};struct Cyc_Absyn_Fallthru_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_List_List*f1;struct Cyc_Absyn_Switch_clause**f2;};struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Decl*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Label_s_Absyn_Raw_stmt_struct{int tag;struct _fat_ptr*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Do_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct _tuple11 f2;};struct Cyc_Absyn_TryCatch_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct Cyc_List_List*f2;void*f3;};struct Cyc_Absyn_Stmt{void*r;unsigned loc;void*annot;};struct Cyc_Absyn_Wild_p_Absyn_Raw_pat_struct{int tag;};struct Cyc_Absyn_Var_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Pat*f2;};struct Cyc_Absyn_AliasVar_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;};struct Cyc_Absyn_Reference_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Pat*f2;};struct Cyc_Absyn_TagInt_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;};struct Cyc_Absyn_Tuple_p_Absyn_Raw_pat_struct{int tag;struct Cyc_List_List*f1;int f2;};struct Cyc_Absyn_Pointer_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Pat*f1;};struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct{int tag;union Cyc_Absyn_AggrInfo*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;int f4;};struct Cyc_Absyn_Datatype_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;struct Cyc_Absyn_Datatypefield*f2;struct Cyc_List_List*f3;int f4;};struct Cyc_Absyn_Null_p_Absyn_Raw_pat_struct{int tag;};struct Cyc_Absyn_Int_p_Absyn_Raw_pat_struct{int tag;enum Cyc_Absyn_Sign f1;int f2;};struct Cyc_Absyn_Char_p_Absyn_Raw_pat_struct{int tag;char f1;};struct Cyc_Absyn_Float_p_Absyn_Raw_pat_struct{int tag;struct _fat_ptr f1;int f2;};struct Cyc_Absyn_Enum_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_AnonEnum_p_Absyn_Raw_pat_struct{int tag;void*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_UnknownId_p_Absyn_Raw_pat_struct{int tag;struct _tuple2*f1;};struct Cyc_Absyn_UnknownCall_p_Absyn_Raw_pat_struct{int tag;struct _tuple2*f1;struct Cyc_List_List*f2;int f3;};struct Cyc_Absyn_Exp_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Pat{void*r;void*topt;unsigned loc;};struct Cyc_Absyn_Switch_clause{struct Cyc_Absyn_Pat*pattern;struct Cyc_Core_Opt*pat_vars;struct Cyc_Absyn_Exp*where_clause;struct Cyc_Absyn_Stmt*body;unsigned loc;};struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct{int tag;struct _tuple2*f1;};struct Cyc_Absyn_Global_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Funname_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Fndecl*f1;};struct Cyc_Absyn_Param_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Local_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Pat_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Vardecl{enum Cyc_Absyn_Scope sc;struct _tuple2*name;unsigned varloc;struct Cyc_Absyn_Tqual tq;void*type;struct Cyc_Absyn_Exp*initializer;void*rgn;struct Cyc_List_List*attributes;int escapes;};struct Cyc_Absyn_Fndecl{enum Cyc_Absyn_Scope sc;int is_inline;struct _tuple2*name;struct Cyc_Absyn_Stmt*body;struct Cyc_Absyn_FnInfo i;void*cached_type;struct Cyc_Core_Opt*param_vardecls;struct Cyc_Absyn_Vardecl*fn_vardecl;};struct Cyc_Absyn_Aggrfield{struct _fat_ptr*name;struct Cyc_Absyn_Tqual tq;void*type;struct Cyc_Absyn_Exp*width;struct Cyc_List_List*attributes;struct Cyc_Absyn_Exp*requires_clause;};struct Cyc_Absyn_AggrdeclImpl{struct Cyc_List_List*exist_vars;struct Cyc_List_List*rgn_po;struct Cyc_List_List*fields;int tagged;};struct Cyc_Absyn_Aggrdecl{enum Cyc_Absyn_AggrKind kind;enum Cyc_Absyn_Scope sc;struct _tuple2*name;struct Cyc_List_List*tvs;struct Cyc_Absyn_AggrdeclImpl*impl;struct Cyc_List_List*attributes;int expected_mem_kind;};struct Cyc_Absyn_Datatypefield{struct _tuple2*name;struct Cyc_List_List*typs;unsigned loc;enum Cyc_Absyn_Scope sc;};struct Cyc_Absyn_Datatypedecl{enum Cyc_Absyn_Scope sc;struct _tuple2*name;struct Cyc_List_List*tvs;struct Cyc_Core_Opt*fields;int is_extensible;};struct Cyc_Absyn_Enumfield{struct _tuple2*name;struct Cyc_Absyn_Exp*tag;unsigned loc;};struct Cyc_Absyn_Enumdecl{enum Cyc_Absyn_Scope sc;struct _tuple2*name;struct Cyc_Core_Opt*fields;};struct Cyc_Absyn_Typedefdecl{struct _tuple2*name;struct Cyc_Absyn_Tqual tq;struct Cyc_List_List*tvs;struct Cyc_Core_Opt*kind;void*defn;struct Cyc_List_List*atts;int extern_c;};struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Fn_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Fndecl*f1;};struct Cyc_Absyn_Let_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Pat*f1;struct Cyc_Core_Opt*f2;struct Cyc_Absyn_Exp*f3;void*f4;};struct Cyc_Absyn_Letv_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_Region_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_Aggr_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Aggrdecl*f1;};struct Cyc_Absyn_Datatype_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;};struct Cyc_Absyn_Enum_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;};struct Cyc_Absyn_Typedef_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Typedefdecl*f1;};struct Cyc_Absyn_Namespace_d_Absyn_Raw_decl_struct{int tag;struct _fat_ptr*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Using_d_Absyn_Raw_decl_struct{int tag;struct _tuple2*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_ExternC_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;};struct _tuple12{unsigned f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_ExternCinclude_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;struct _tuple12*f4;};struct Cyc_Absyn_Porton_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Portoff_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Tempeston_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Tempestoff_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Decl{void*r;unsigned loc;};struct Cyc_Absyn_ArrayElement_Absyn_Designator_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_FieldName_Absyn_Designator_struct{int tag;struct _fat_ptr*f1;};extern char Cyc_Absyn_EmptyAnnot[11U];struct Cyc_Absyn_EmptyAnnot_Absyn_AbsynAnnot_struct{char*tag;};
# 951 "absyn.h"
extern void*Cyc_Absyn_void_type;
# 1021
struct Cyc_Absyn_Exp*Cyc_Absyn_signed_int_exp(int,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_uint_exp(unsigned,unsigned);
# 1033
struct Cyc_Absyn_Exp*Cyc_Absyn_prim1_exp(enum Cyc_Absyn_Primop,struct Cyc_Absyn_Exp*,unsigned);
# 1154
struct Cyc_Absyn_Aggrfield*Cyc_Absyn_lookup_field(struct Cyc_List_List*,struct _fat_ptr*);
# 1156
struct Cyc_Absyn_Aggrfield*Cyc_Absyn_lookup_decl_field(struct Cyc_Absyn_Aggrdecl*,struct _fat_ptr*);
# 1176
struct Cyc_Absyn_Aggrdecl*Cyc_Absyn_get_known_aggrdecl(union Cyc_Absyn_AggrInfo);
# 1178
int Cyc_Absyn_is_nontagged_nonrequire_union_type(void*);
# 1180
int Cyc_Absyn_is_require_union_type(void*);struct Cyc___cycFILE;
# 53 "cycboot.h"
extern struct Cyc___cycFILE*Cyc_stderr;struct Cyc_String_pa_PrintArg_struct{int tag;struct _fat_ptr f1;};struct Cyc_Int_pa_PrintArg_struct{int tag;unsigned long f1;};struct Cyc_Double_pa_PrintArg_struct{int tag;double f1;};struct Cyc_LongDouble_pa_PrintArg_struct{int tag;long double f1;};struct Cyc_ShortPtr_pa_PrintArg_struct{int tag;short*f1;};struct Cyc_IntPtr_pa_PrintArg_struct{int tag;unsigned long*f1;};
# 100
int Cyc_fprintf(struct Cyc___cycFILE*,struct _fat_ptr,struct _fat_ptr);struct Cyc_ShortPtr_sa_ScanfArg_struct{int tag;short*f1;};struct Cyc_UShortPtr_sa_ScanfArg_struct{int tag;unsigned short*f1;};struct Cyc_IntPtr_sa_ScanfArg_struct{int tag;int*f1;};struct Cyc_UIntPtr_sa_ScanfArg_struct{int tag;unsigned*f1;};struct Cyc_StringPtr_sa_ScanfArg_struct{int tag;struct _fat_ptr f1;};struct Cyc_DoublePtr_sa_ScanfArg_struct{int tag;double*f1;};struct Cyc_FloatPtr_sa_ScanfArg_struct{int tag;float*f1;};struct Cyc_CharPtr_sa_ScanfArg_struct{int tag;struct _fat_ptr f1;};extern char Cyc_FileCloseError[15U];struct Cyc_FileCloseError_exn_struct{char*tag;};extern char Cyc_FileOpenError[14U];struct Cyc_FileOpenError_exn_struct{char*tag;struct _fat_ptr f1;};struct _union_RelnOp_RConst{int tag;unsigned val;};struct _union_RelnOp_RVar{int tag;struct Cyc_Absyn_Vardecl*val;};struct _union_RelnOp_RNumelts{int tag;struct Cyc_Absyn_Vardecl*val;};struct _union_RelnOp_RType{int tag;void*val;};struct _union_RelnOp_RParam{int tag;unsigned val;};struct _union_RelnOp_RParamNumelts{int tag;unsigned val;};struct _union_RelnOp_RReturn{int tag;unsigned val;};union Cyc_Relations_RelnOp{struct _union_RelnOp_RConst RConst;struct _union_RelnOp_RVar RVar;struct _union_RelnOp_RNumelts RNumelts;struct _union_RelnOp_RType RType;struct _union_RelnOp_RParam RParam;struct _union_RelnOp_RParamNumelts RParamNumelts;struct _union_RelnOp_RReturn RReturn;};
# 41 "relations-ap.h"
union Cyc_Relations_RelnOp Cyc_Relations_RConst(unsigned);union Cyc_Relations_RelnOp Cyc_Relations_RVar(struct Cyc_Absyn_Vardecl*);union Cyc_Relations_RelnOp Cyc_Relations_RNumelts(struct Cyc_Absyn_Vardecl*);
# 50
enum Cyc_Relations_Relation{Cyc_Relations_Req =0U,Cyc_Relations_Rneq =1U,Cyc_Relations_Rlte =2U,Cyc_Relations_Rlt =3U};struct Cyc_Relations_Reln{union Cyc_Relations_RelnOp rop1;enum Cyc_Relations_Relation relation;union Cyc_Relations_RelnOp rop2;};struct _tuple13{struct Cyc_Absyn_Exp*f1;enum Cyc_Relations_Relation f2;struct Cyc_Absyn_Exp*f3;};
# 64
struct _tuple13 Cyc_Relations_primop2relation(struct Cyc_Absyn_Exp*e1,enum Cyc_Absyn_Primop p,struct Cyc_Absyn_Exp*e2);
# 68
enum Cyc_Relations_Relation Cyc_Relations_flip_relation(enum Cyc_Relations_Relation r);
# 70
struct Cyc_Relations_Reln*Cyc_Relations_negate(struct _RegionHandle*,struct Cyc_Relations_Reln*);
# 76
int Cyc_Relations_exp2relnop(struct Cyc_Absyn_Exp*e,union Cyc_Relations_RelnOp*p);
# 84
struct Cyc_List_List*Cyc_Relations_exp2relns(struct _RegionHandle*r,struct Cyc_Absyn_Exp*e);
# 87
struct Cyc_List_List*Cyc_Relations_add_relation(struct _RegionHandle*rgn,union Cyc_Relations_RelnOp rop1,enum Cyc_Relations_Relation r,union Cyc_Relations_RelnOp rop2,struct Cyc_List_List*relns);
# 100
struct Cyc_List_List*Cyc_Relations_reln_assign_var(struct _RegionHandle*,struct Cyc_List_List*,struct Cyc_Absyn_Vardecl*,struct Cyc_Absyn_Exp*);
# 103
struct Cyc_List_List*Cyc_Relations_reln_assign_exp(struct _RegionHandle*,struct Cyc_List_List*,struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*);
# 108
struct Cyc_List_List*Cyc_Relations_reln_kill_exp(struct _RegionHandle*,struct Cyc_List_List*,struct Cyc_Absyn_Exp*);
# 112
int Cyc_Relations_same_relns(struct Cyc_List_List*,struct Cyc_List_List*);
# 118
void Cyc_Relations_print_relns(struct Cyc___cycFILE*,struct Cyc_List_List*);
# 121
struct _fat_ptr Cyc_Relations_rop2string(union Cyc_Relations_RelnOp r);
struct _fat_ptr Cyc_Relations_relation2string(enum Cyc_Relations_Relation r);
struct _fat_ptr Cyc_Relations_relns2string(struct Cyc_List_List*r);
# 129
int Cyc_Relations_consistent_relations(struct Cyc_List_List*rlns);struct Cyc_Iter_Iter{void*env;int(*next)(void*env,void*dest);};struct Cyc_Dict_T;struct Cyc_Dict_Dict{int(*rel)(void*,void*);struct _RegionHandle*r;const struct Cyc_Dict_T*t;};extern char Cyc_Dict_Present[8U];struct Cyc_Dict_Present_exn_struct{char*tag;};extern char Cyc_Dict_Absent[7U];struct Cyc_Dict_Absent_exn_struct{char*tag;};
# 87 "dict.h"
struct Cyc_Dict_Dict Cyc_Dict_insert(struct Cyc_Dict_Dict d,void*k,void*v);
# 113
void*Cyc_Dict_lookup_other(struct Cyc_Dict_Dict d,int(*cmp)(void*,void*),void*k);
# 126 "dict.h"
int Cyc_Dict_lookup_bool(struct Cyc_Dict_Dict d,void*k,void**ans);
# 149
void Cyc_Dict_iter_c(void(*f)(void*,void*,void*),void*env,struct Cyc_Dict_Dict d);struct Cyc_RgnOrder_RgnPO;extern char Cyc_Tcenv_Env_error[10U];struct Cyc_Tcenv_Env_error_exn_struct{char*tag;};struct Cyc_Tcenv_Genv{struct Cyc_Dict_Dict aggrdecls;struct Cyc_Dict_Dict datatypedecls;struct Cyc_Dict_Dict enumdecls;struct Cyc_Dict_Dict typedefs;struct Cyc_Dict_Dict ordinaries;};struct Cyc_Tcenv_Fenv;struct Cyc_Tcenv_Tenv{struct Cyc_List_List*ns;struct Cyc_Tcenv_Genv*ae;struct Cyc_Tcenv_Fenv*le;int allow_valueof: 1;int in_extern_c_include: 1;int in_tempest: 1;int tempest_generalize: 1;};
# 88 "tcenv.h"
enum Cyc_Tcenv_NewStatus{Cyc_Tcenv_NoneNew =0U,Cyc_Tcenv_InNew =1U,Cyc_Tcenv_InNewAggr =2U};
# 30 "tcutil.h"
void*Cyc_Tcutil_impos(struct _fat_ptr,struct _fat_ptr);
# 36
int Cyc_Tcutil_is_void_type(void*);
# 38
int Cyc_Tcutil_is_any_int_type(void*);
int Cyc_Tcutil_is_any_float_type(void*);
# 50
int Cyc_Tcutil_is_zeroterm_pointer_type(void*);
# 52
int Cyc_Tcutil_is_bound_one(void*);
# 59
int Cyc_Tcutil_is_noreturn_fn_type(void*);
# 64
void*Cyc_Tcutil_pointer_elt_type(void*);
# 75
struct Cyc_Absyn_Exp*Cyc_Tcutil_get_type_bound(void*);
# 107
void*Cyc_Tcutil_compress(void*);
# 187
void*Cyc_Tcutil_rsubstitute(struct _RegionHandle*,struct Cyc_List_List*,void*);
# 193
struct Cyc_Absyn_Exp*Cyc_Tcutil_rsubsexp(struct _RegionHandle*,struct Cyc_List_List*,struct Cyc_Absyn_Exp*);
# 202
void*Cyc_Tcutil_fndecl2type(struct Cyc_Absyn_Fndecl*);
# 229
int Cyc_Tcutil_is_zero_ptr_deref(struct Cyc_Absyn_Exp*,void**ptr_type,int*is_fat,void**elt_type);
# 236
int Cyc_Tcutil_is_noalias_pointer(void*t,int must_be_unique);
# 246
int Cyc_Tcutil_is_noalias_pointer_or_aggr(void*);
# 291
struct Cyc_Absyn_Vardecl*Cyc_Tcutil_nonesc_vardecl(void*);
# 294
struct Cyc_List_List*Cyc_Tcutil_filter_nulls(struct Cyc_List_List*);struct _tuple14{unsigned f1;int f2;};
# 28 "evexp.h"
struct _tuple14 Cyc_Evexp_eval_const_uint_exp(struct Cyc_Absyn_Exp*e);struct Cyc_Set_Set;extern char Cyc_Set_Absent[7U];struct Cyc_Set_Absent_exn_struct{char*tag;};struct Cyc_Hashtable_Table;
# 39 "hashtable.h"
struct Cyc_Hashtable_Table*Cyc_Hashtable_create(int sz,int(*cmp)(void*,void*),int(*hash)(void*));
# 50
void Cyc_Hashtable_insert(struct Cyc_Hashtable_Table*t,void*key,void*val);
# 52
void*Cyc_Hashtable_lookup(struct Cyc_Hashtable_Table*t,void*key);
# 56
void**Cyc_Hashtable_lookup_opt(struct Cyc_Hashtable_Table*t,void*key);struct Cyc_JumpAnalysis_Jump_Anal_Result{struct Cyc_Hashtable_Table*pop_tables;struct Cyc_Hashtable_Table*succ_tables;struct Cyc_Hashtable_Table*pat_pop_tables;};
# 41 "cf_flowinfo.h"
int Cyc_CfFlowInfo_anal_error;
void Cyc_CfFlowInfo_aerr(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap);struct Cyc_CfFlowInfo_VarRoot_CfFlowInfo_Root_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_CfFlowInfo_MallocPt_CfFlowInfo_Root_struct{int tag;struct Cyc_Absyn_Exp*f1;void*f2;};struct Cyc_CfFlowInfo_InitParam_CfFlowInfo_Root_struct{int tag;int f1;void*f2;};struct Cyc_CfFlowInfo_Dot_CfFlowInfo_PathCon_struct{int tag;int f1;};struct Cyc_CfFlowInfo_Star_CfFlowInfo_PathCon_struct{int tag;};struct Cyc_CfFlowInfo_Place{void*root;struct Cyc_List_List*path;};
# 74
enum Cyc_CfFlowInfo_InitLevel{Cyc_CfFlowInfo_NoneIL =0U,Cyc_CfFlowInfo_AllIL =1U};extern char Cyc_CfFlowInfo_IsZero[7U];struct Cyc_CfFlowInfo_IsZero_Absyn_AbsynAnnot_struct{char*tag;};extern char Cyc_CfFlowInfo_NotZero[8U];struct Cyc_CfFlowInfo_NotZero_Absyn_AbsynAnnot_struct{char*tag;struct Cyc_List_List*f1;};extern char Cyc_CfFlowInfo_UnknownZ[9U];struct Cyc_CfFlowInfo_UnknownZ_Absyn_AbsynAnnot_struct{char*tag;struct Cyc_List_List*f1;};
# 87
extern struct Cyc_CfFlowInfo_IsZero_Absyn_AbsynAnnot_struct Cyc_CfFlowInfo_IsZero_val;struct _union_AbsLVal_PlaceL{int tag;struct Cyc_CfFlowInfo_Place*val;};struct _union_AbsLVal_UnknownL{int tag;int val;};union Cyc_CfFlowInfo_AbsLVal{struct _union_AbsLVal_PlaceL PlaceL;struct _union_AbsLVal_UnknownL UnknownL;};
# 94
union Cyc_CfFlowInfo_AbsLVal Cyc_CfFlowInfo_PlaceL(struct Cyc_CfFlowInfo_Place*);
union Cyc_CfFlowInfo_AbsLVal Cyc_CfFlowInfo_UnknownL();struct Cyc_CfFlowInfo_UnionRInfo{int is_union;int fieldnum;};struct Cyc_CfFlowInfo_Zero_CfFlowInfo_AbsRVal_struct{int tag;};struct Cyc_CfFlowInfo_NotZeroAll_CfFlowInfo_AbsRVal_struct{int tag;};struct Cyc_CfFlowInfo_UnknownR_CfFlowInfo_AbsRVal_struct{int tag;enum Cyc_CfFlowInfo_InitLevel f1;};struct Cyc_CfFlowInfo_Esc_CfFlowInfo_AbsRVal_struct{int tag;enum Cyc_CfFlowInfo_InitLevel f1;};struct Cyc_CfFlowInfo_AddressOf_CfFlowInfo_AbsRVal_struct{int tag;struct Cyc_CfFlowInfo_Place*f1;};struct Cyc_CfFlowInfo_UniquePtr_CfFlowInfo_AbsRVal_struct{int tag;void*f1;};struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct{int tag;struct Cyc_CfFlowInfo_UnionRInfo f1;struct _fat_ptr f2;};struct Cyc_CfFlowInfo_Consumed_CfFlowInfo_AbsRVal_struct{int tag;struct Cyc_Absyn_Exp*f1;int f2;void*f3;};struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct{int tag;struct Cyc_Absyn_Vardecl*f1;void*f2;};struct _union_FlowInfo_BottomFL{int tag;int val;};struct _tuple15{struct Cyc_Dict_Dict f1;struct Cyc_List_List*f2;};struct _union_FlowInfo_ReachableFL{int tag;struct _tuple15 val;};union Cyc_CfFlowInfo_FlowInfo{struct _union_FlowInfo_BottomFL BottomFL;struct _union_FlowInfo_ReachableFL ReachableFL;};
# 138 "cf_flowinfo.h"
union Cyc_CfFlowInfo_FlowInfo Cyc_CfFlowInfo_BottomFL();
union Cyc_CfFlowInfo_FlowInfo Cyc_CfFlowInfo_ReachableFL(struct Cyc_Dict_Dict,struct Cyc_List_List*);struct Cyc_CfFlowInfo_FlowEnv{void*zero;void*notzeroall;void*unknown_none;void*unknown_all;void*esc_none;void*esc_all;struct Cyc_Dict_Dict mt_flowdict;struct Cyc_CfFlowInfo_Place*dummy_place;};
# 153
struct Cyc_CfFlowInfo_FlowEnv*Cyc_CfFlowInfo_new_flow_env();
# 155
int Cyc_CfFlowInfo_get_field_index(void*t,struct _fat_ptr*f);
int Cyc_CfFlowInfo_get_field_index_fs(struct Cyc_List_List*fs,struct _fat_ptr*f);
# 158
int Cyc_CfFlowInfo_root_cmp(void*,void*);
# 161
struct _fat_ptr Cyc_CfFlowInfo_aggrfields_to_aggrdict(struct Cyc_CfFlowInfo_FlowEnv*,struct Cyc_List_List*,int no_init_bits_only,void*);
void*Cyc_CfFlowInfo_typ_to_absrval(struct Cyc_CfFlowInfo_FlowEnv*,void*t,int no_init_bits_only,void*leafval);
void*Cyc_CfFlowInfo_make_unique_consumed(struct Cyc_CfFlowInfo_FlowEnv*fenv,void*t,struct Cyc_Absyn_Exp*consumer,int iteration,void*);
int Cyc_CfFlowInfo_is_unique_consumed(struct Cyc_Absyn_Exp*e,int env_iteration,void*r,int*needs_unconsume);
void*Cyc_CfFlowInfo_make_unique_unconsumed(struct Cyc_CfFlowInfo_FlowEnv*fenv,void*r);struct _tuple16{void*f1;struct Cyc_List_List*f2;};
struct _tuple16 Cyc_CfFlowInfo_unname_rval(void*rv);
# 168
enum Cyc_CfFlowInfo_InitLevel Cyc_CfFlowInfo_initlevel(struct Cyc_CfFlowInfo_FlowEnv*,struct Cyc_Dict_Dict d,void*r);
void*Cyc_CfFlowInfo_lookup_place(struct Cyc_Dict_Dict d,struct Cyc_CfFlowInfo_Place*place);
# 171
int Cyc_CfFlowInfo_is_init_pointer(void*r);
int Cyc_CfFlowInfo_flow_lessthan_approx(union Cyc_CfFlowInfo_FlowInfo f1,union Cyc_CfFlowInfo_FlowInfo f2);
# 174
void Cyc_CfFlowInfo_print_absrval(void*rval);
# 181
void Cyc_CfFlowInfo_print_flow(union Cyc_CfFlowInfo_FlowInfo f);
# 196 "cf_flowinfo.h"
struct Cyc_Dict_Dict Cyc_CfFlowInfo_escape_deref(struct Cyc_CfFlowInfo_FlowEnv*fenv,struct Cyc_Dict_Dict d,void*r);
struct Cyc_Dict_Dict Cyc_CfFlowInfo_assign_place(struct Cyc_CfFlowInfo_FlowEnv*fenv,unsigned loc,struct Cyc_Dict_Dict d,struct Cyc_CfFlowInfo_Place*place,void*r);
# 201
union Cyc_CfFlowInfo_FlowInfo Cyc_CfFlowInfo_join_flow(struct Cyc_CfFlowInfo_FlowEnv*,union Cyc_CfFlowInfo_FlowInfo,union Cyc_CfFlowInfo_FlowInfo);struct _tuple17{union Cyc_CfFlowInfo_FlowInfo f1;void*f2;};
struct _tuple17 Cyc_CfFlowInfo_join_flow_and_rval(struct Cyc_CfFlowInfo_FlowEnv*,struct _tuple17 pr1,struct _tuple17 pr2);
# 32 "new_control_flow.h"
void Cyc_NewControlFlow_cf_check(struct Cyc_JumpAnalysis_Jump_Anal_Result*tables,struct Cyc_List_List*ds);
# 37
extern int Cyc_NewControlFlow_warn_lose_unique;struct Cyc_Tcpat_TcPatResult{struct _tuple1*tvars_and_bounds_opt;struct Cyc_List_List*patvars;};struct Cyc_Tcpat_WhereTest_Tcpat_PatTest_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Tcpat_EqNull_Tcpat_PatTest_struct{int tag;};struct Cyc_Tcpat_NeqNull_Tcpat_PatTest_struct{int tag;};struct Cyc_Tcpat_EqEnum_Tcpat_PatTest_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Tcpat_EqAnonEnum_Tcpat_PatTest_struct{int tag;void*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Tcpat_EqFloat_Tcpat_PatTest_struct{int tag;struct _fat_ptr f1;int f2;};struct Cyc_Tcpat_EqConst_Tcpat_PatTest_struct{int tag;unsigned f1;};struct Cyc_Tcpat_EqDatatypeTag_Tcpat_PatTest_struct{int tag;int f1;struct Cyc_Absyn_Datatypedecl*f2;struct Cyc_Absyn_Datatypefield*f3;};struct Cyc_Tcpat_EqTaggedUnion_Tcpat_PatTest_struct{int tag;struct _fat_ptr*f1;int f2;};struct Cyc_Tcpat_EqExtensibleDatatype_Tcpat_PatTest_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;struct Cyc_Absyn_Datatypefield*f2;};struct Cyc_Tcpat_Dummy_Tcpat_Access_struct{int tag;};struct Cyc_Tcpat_Deref_Tcpat_Access_struct{int tag;};struct Cyc_Tcpat_TupleField_Tcpat_Access_struct{int tag;unsigned f1;};struct Cyc_Tcpat_DatatypeField_Tcpat_Access_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;struct Cyc_Absyn_Datatypefield*f2;unsigned f3;};struct Cyc_Tcpat_AggrField_Tcpat_Access_struct{int tag;int f1;struct _fat_ptr*f2;};struct _union_PatOrWhere_pattern{int tag;struct Cyc_Absyn_Pat*val;};struct _union_PatOrWhere_where_clause{int tag;struct Cyc_Absyn_Exp*val;};union Cyc_Tcpat_PatOrWhere{struct _union_PatOrWhere_pattern pattern;struct _union_PatOrWhere_where_clause where_clause;};struct Cyc_Tcpat_PathNode{union Cyc_Tcpat_PatOrWhere orig_pat;void*access;};struct Cyc_Tcpat_Rhs{int used;unsigned pat_loc;struct Cyc_Absyn_Stmt*rhs;};struct Cyc_Tcpat_Failure_Tcpat_Decision_struct{int tag;void*f1;};struct Cyc_Tcpat_Success_Tcpat_Decision_struct{int tag;struct Cyc_Tcpat_Rhs*f1;};struct Cyc_Tcpat_SwitchDec_Tcpat_Decision_struct{int tag;struct Cyc_List_List*f1;struct Cyc_List_List*f2;void*f3;};
# 115 "tcpat.h"
int Cyc_Tcpat_has_vars(struct Cyc_Core_Opt*pat_vars);
# 29 "warn.h"
void Cyc_Warn_warn(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap);
# 40
void*Cyc_Warn_impos(struct _fat_ptr fmt,struct _fat_ptr ap);struct Cyc_Warn_String_Warn_Warg_struct{int tag;struct _fat_ptr f1;};struct Cyc_Warn_Exp_Warn_Warg_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Warn_Typ_Warn_Warg_struct{int tag;void*f1;};struct Cyc_Warn_Qvar_Warn_Warg_struct{int tag;struct _tuple2*f1;};struct Cyc_PP_Ppstate;struct Cyc_PP_Out;struct Cyc_PP_Doc;struct Cyc_Absynpp_Params{int expand_typedefs;int qvar_to_Cids;int add_cyc_prefix;int to_VC;int decls_first;int rewrite_temp_tvars;int print_all_tvars;int print_all_kinds;int print_all_effects;int print_using_stmts;int print_externC_stmts;int print_full_evars;int print_zeroterm;int generate_line_directives;int use_curr_namespace;struct Cyc_List_List*curr_namespace;};
# 67 "absynpp.h"
struct _fat_ptr Cyc_Absynpp_exp2string(struct Cyc_Absyn_Exp*);
# 69
struct _fat_ptr Cyc_Absynpp_qvar2string(struct _tuple2*);
# 51 "new_control_flow.cyc"
static int Cyc_NewControlFlow_strcmp(struct _fat_ptr s1,struct _fat_ptr s2){
if((char*)s1.curr == (char*)s2.curr)
return 0;{
# 52
int i=0;
# 55
unsigned sz1=_get_fat_size(s1,sizeof(char));
unsigned sz2=_get_fat_size(s2,sizeof(char));
unsigned minsz=sz1 < sz2?sz1: sz2;
# 59
while((unsigned)i < minsz){
char c1=*((const char*)_check_fat_subscript(s1,sizeof(char),i));
char c2=*((const char*)_check_fat_subscript(s2,sizeof(char),i));
if((int)c1 == (int)'\000'){
if((int)c2 == (int)'\000')return 0;else{
return - 1;}}else{
if((int)c2 == (int)'\000')return 1;else{
# 67
int diff=(int)c1 - (int)c2;
if(diff != 0)return diff;}}
# 70
++ i;}
# 72
if(sz1 == sz2)return 0;if(minsz < sz2){
# 74
if((int)*((const char*)_check_fat_subscript(s2,sizeof(char),i))== (int)'\000')return 0;else{
return - 1;}}else{
# 77
if((int)*((const char*)_check_fat_subscript(s1,sizeof(char),i))== (int)'\000')return 0;else{
return 1;}}}}
# 51
int Cyc_NewControlFlow_warn_lose_unique=0;struct Cyc_NewControlFlow_CFStmtAnnot{int visited;};static char Cyc_NewControlFlow_CFAnnot[8U]="CFAnnot";struct Cyc_NewControlFlow_CFAnnot_Absyn_AbsynAnnot_struct{char*tag;struct Cyc_NewControlFlow_CFStmtAnnot f1;};struct Cyc_NewControlFlow_AnalEnv{struct Cyc_JumpAnalysis_Jump_Anal_Result*all_tables;struct Cyc_Hashtable_Table*succ_table;struct Cyc_Hashtable_Table*pat_pop_table;struct Cyc_CfFlowInfo_FlowEnv*fenv;int iterate_again;int iteration_num;int in_try;union Cyc_CfFlowInfo_FlowInfo tryflow;int noreturn;void*return_type;struct Cyc_List_List*unique_pat_vars;struct Cyc_List_List*param_roots;struct Cyc_List_List*noconsume_params;struct Cyc_Hashtable_Table*flow_table;struct Cyc_List_List*return_relns;};struct _tuple18{void*f1;int f2;};
# 125 "new_control_flow.cyc"
static union Cyc_CfFlowInfo_FlowInfo Cyc_NewControlFlow_anal_stmt(struct Cyc_NewControlFlow_AnalEnv*,union Cyc_CfFlowInfo_FlowInfo,struct Cyc_Absyn_Stmt*,struct _tuple18*);
static union Cyc_CfFlowInfo_FlowInfo Cyc_NewControlFlow_anal_decl(struct Cyc_NewControlFlow_AnalEnv*,union Cyc_CfFlowInfo_FlowInfo,struct Cyc_Absyn_Decl*);struct _tuple19{union Cyc_CfFlowInfo_FlowInfo f1;union Cyc_CfFlowInfo_AbsLVal f2;};
static struct _tuple19 Cyc_NewControlFlow_anal_Lexp(struct Cyc_NewControlFlow_AnalEnv*,union Cyc_CfFlowInfo_FlowInfo,int expand_unique,int passthrough_consumes,struct Cyc_Absyn_Exp*);
static struct _tuple17 Cyc_NewControlFlow_anal_Rexp(struct Cyc_NewControlFlow_AnalEnv*,int copy_ctxt,union Cyc_CfFlowInfo_FlowInfo,struct Cyc_Absyn_Exp*);struct _tuple20{union Cyc_CfFlowInfo_FlowInfo f1;union Cyc_CfFlowInfo_FlowInfo f2;};
static struct _tuple20 Cyc_NewControlFlow_anal_test(struct Cyc_NewControlFlow_AnalEnv*,union Cyc_CfFlowInfo_FlowInfo,struct Cyc_Absyn_Exp*);
# 131
static union Cyc_CfFlowInfo_FlowInfo Cyc_NewControlFlow_expand_unique_places(struct Cyc_NewControlFlow_AnalEnv*,union Cyc_CfFlowInfo_FlowInfo inflow,struct Cyc_List_List*es);
# 135
static struct Cyc_NewControlFlow_CFStmtAnnot*Cyc_NewControlFlow_get_stmt_annot(struct Cyc_Absyn_Stmt*s){
void*_stmttmp0=s->annot;void*_tmp0=_stmttmp0;struct Cyc_NewControlFlow_CFStmtAnnot*_tmp1;if(((struct Cyc_NewControlFlow_CFAnnot_Absyn_AbsynAnnot_struct*)_tmp0)->tag == Cyc_NewControlFlow_CFAnnot){_LL1: _tmp1=(struct Cyc_NewControlFlow_CFStmtAnnot*)&((struct Cyc_NewControlFlow_CFAnnot_Absyn_AbsynAnnot_struct*)_tmp0)->f1;_LL2: {struct Cyc_NewControlFlow_CFStmtAnnot*x=_tmp1;
return x;}}else{_LL3: _LL4:
({void*_tmp4CF=(void*)({struct Cyc_NewControlFlow_CFAnnot_Absyn_AbsynAnnot_struct*_tmp2=_cycalloc(sizeof(*_tmp2));_tmp2->tag=Cyc_NewControlFlow_CFAnnot,(_tmp2->f1).visited=0;_tmp2;});s->annot=_tmp4CF;});return Cyc_NewControlFlow_get_stmt_annot(s);}_LL0:;}
# 135
static union Cyc_CfFlowInfo_FlowInfo*Cyc_NewControlFlow_get_stmt_flow(struct Cyc_NewControlFlow_AnalEnv*env,struct Cyc_Absyn_Stmt*s){
# 143
union Cyc_CfFlowInfo_FlowInfo**sflow=({union Cyc_CfFlowInfo_FlowInfo**(*_tmp4D1)(struct Cyc_Hashtable_Table*t,struct Cyc_Absyn_Stmt*key)=({union Cyc_CfFlowInfo_FlowInfo**(*_tmp5)(struct Cyc_Hashtable_Table*t,struct Cyc_Absyn_Stmt*key)=(union Cyc_CfFlowInfo_FlowInfo**(*)(struct Cyc_Hashtable_Table*t,struct Cyc_Absyn_Stmt*key))Cyc_Hashtable_lookup_opt;_tmp5;});struct Cyc_Hashtable_Table*_tmp4D0=env->flow_table;_tmp4D1(_tmp4D0,s);});
if(sflow == 0){
union Cyc_CfFlowInfo_FlowInfo*res=({union Cyc_CfFlowInfo_FlowInfo*_tmp4=_cycalloc(sizeof(*_tmp4));({union Cyc_CfFlowInfo_FlowInfo _tmp4D2=Cyc_CfFlowInfo_BottomFL();*_tmp4=_tmp4D2;});_tmp4;});
({void(*_tmp4D5)(struct Cyc_Hashtable_Table*t,struct Cyc_Absyn_Stmt*key,union Cyc_CfFlowInfo_FlowInfo*val)=({void(*_tmp3)(struct Cyc_Hashtable_Table*t,struct Cyc_Absyn_Stmt*key,union Cyc_CfFlowInfo_FlowInfo*val)=(void(*)(struct Cyc_Hashtable_Table*t,struct Cyc_Absyn_Stmt*key,union Cyc_CfFlowInfo_FlowInfo*val))Cyc_Hashtable_insert;_tmp3;});struct Cyc_Hashtable_Table*_tmp4D4=env->flow_table;struct Cyc_Absyn_Stmt*_tmp4D3=s;_tmp4D5(_tmp4D4,_tmp4D3,res);});
return res;}
# 144
return*sflow;}
# 135
static struct Cyc_List_List*Cyc_NewControlFlow_flowrelns(union Cyc_CfFlowInfo_FlowInfo f){
# 153
union Cyc_CfFlowInfo_FlowInfo _tmp6=f;struct Cyc_List_List*_tmp7;if((_tmp6.BottomFL).tag == 1){_LL1: _LL2:
 return 0;}else{_LL3: _tmp7=((_tmp6.ReachableFL).val).f2;_LL4: {struct Cyc_List_List*r=_tmp7;
return r;}}_LL0:;}struct _tuple21{struct Cyc_NewControlFlow_CFStmtAnnot*f1;union Cyc_CfFlowInfo_FlowInfo*f2;};
# 135
static struct _tuple21 Cyc_NewControlFlow_pre_stmt_check(struct Cyc_NewControlFlow_AnalEnv*env,union Cyc_CfFlowInfo_FlowInfo inflow,struct Cyc_Absyn_Stmt*s){
# 160
struct Cyc_NewControlFlow_CFStmtAnnot*annot=Cyc_NewControlFlow_get_stmt_annot(s);
union Cyc_CfFlowInfo_FlowInfo*sflow=Cyc_NewControlFlow_get_stmt_flow(env,s);
# 163
({union Cyc_CfFlowInfo_FlowInfo _tmp4D6=Cyc_CfFlowInfo_join_flow(env->fenv,inflow,*sflow);*sflow=_tmp4D6;});
# 169
annot->visited=env->iteration_num;
return({struct _tuple21 _tmp42D;_tmp42D.f1=annot,_tmp42D.f2=sflow;_tmp42D;});}
# 135
static void Cyc_NewControlFlow_update_tryflow(struct Cyc_NewControlFlow_AnalEnv*env,union Cyc_CfFlowInfo_FlowInfo new_flow){
# 180
if(env->in_try)
# 187
({union Cyc_CfFlowInfo_FlowInfo _tmp4D7=Cyc_CfFlowInfo_join_flow(env->fenv,new_flow,env->tryflow);env->tryflow=_tmp4D7;});}struct _tuple22{struct Cyc_NewControlFlow_AnalEnv*f1;unsigned f2;struct Cyc_Dict_Dict f3;};
# 135
static void Cyc_NewControlFlow_check_unique_root(struct _tuple22*ckenv,void*root,void*rval){
# 196
struct Cyc_Dict_Dict _tmpA;unsigned _tmp9;struct Cyc_NewControlFlow_AnalEnv*_tmp8;_LL1: _tmp8=ckenv->f1;_tmp9=ckenv->f2;_tmpA=ckenv->f3;_LL2: {struct Cyc_NewControlFlow_AnalEnv*env=_tmp8;unsigned loc=_tmp9;struct Cyc_Dict_Dict new_fd=_tmpA;
void*_tmpB=root;struct Cyc_Absyn_Vardecl*_tmpC;if(((struct Cyc_CfFlowInfo_VarRoot_CfFlowInfo_Root_struct*)_tmpB)->tag == 0U){_LL4: _tmpC=((struct Cyc_CfFlowInfo_VarRoot_CfFlowInfo_Root_struct*)_tmpB)->f1;_LL5: {struct Cyc_Absyn_Vardecl*vd=_tmpC;
# 199
if(!Cyc_Dict_lookup_bool(new_fd,root,& rval)&&
 Cyc_Tcutil_is_noalias_pointer_or_aggr(vd->type)){
retry: {void*_tmpD=rval;void*_tmpE;switch(*((int*)_tmpD)){case 8U: _LL9: _tmpE=(void*)((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmpD)->f2;_LLA: {void*r=_tmpE;
rval=r;goto retry;}case 7U: _LLB: _LLC:
 goto _LLE;case 2U: if(((struct Cyc_CfFlowInfo_UnknownR_CfFlowInfo_AbsRVal_struct*)_tmpD)->f1 == Cyc_CfFlowInfo_NoneIL){_LLD: _LLE:
 goto _LL10;}else{goto _LL11;}case 0U: _LLF: _LL10:
 goto _LL8;default: _LL11: _LL12:
# 208
({struct Cyc_String_pa_PrintArg_struct _tmp11=({struct Cyc_String_pa_PrintArg_struct _tmp42E;_tmp42E.tag=0U,({struct _fat_ptr _tmp4D8=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(vd->name));_tmp42E.f1=_tmp4D8;});_tmp42E;});void*_tmpF[1U];_tmpF[0]=& _tmp11;({unsigned _tmp4DA=loc;struct _fat_ptr _tmp4D9=({const char*_tmp10="alias-free pointer(s) reachable from %s may become unreachable.";_tag_fat(_tmp10,sizeof(char),64U);});Cyc_Warn_warn(_tmp4DA,_tmp4D9,_tag_fat(_tmpF,sizeof(void*),1U));});});
goto _LL8;}_LL8:;}}
# 199
goto _LL3;}}else{_LL6: _LL7:
# 213
 goto _LL3;}_LL3:;}}
# 135
static void Cyc_NewControlFlow_update_flow(struct Cyc_NewControlFlow_AnalEnv*env,struct Cyc_Absyn_Stmt*s,union Cyc_CfFlowInfo_FlowInfo flow){
# 221
struct Cyc_NewControlFlow_CFStmtAnnot*annot=Cyc_NewControlFlow_get_stmt_annot(s);
union Cyc_CfFlowInfo_FlowInfo*sflow=Cyc_NewControlFlow_get_stmt_flow(env,s);
union Cyc_CfFlowInfo_FlowInfo new_flow=Cyc_CfFlowInfo_join_flow(env->fenv,flow,*sflow);
# 226
if(Cyc_NewControlFlow_warn_lose_unique){
struct _tuple20 _stmttmp1=({struct _tuple20 _tmp430;_tmp430.f1=flow,_tmp430.f2=new_flow;_tmp430;});struct _tuple20 _tmp12=_stmttmp1;struct Cyc_Dict_Dict _tmp14;struct Cyc_Dict_Dict _tmp13;if(((_tmp12.f1).ReachableFL).tag == 2){if(((_tmp12.f2).ReachableFL).tag == 2){_LL1: _tmp13=(((_tmp12.f1).ReachableFL).val).f1;_tmp14=(((_tmp12.f2).ReachableFL).val).f1;_LL2: {struct Cyc_Dict_Dict fd=_tmp13;struct Cyc_Dict_Dict new_fd=_tmp14;
# 229
struct _tuple22 ckenv=({struct _tuple22 _tmp42F;_tmp42F.f1=env,_tmp42F.f2=s->loc,_tmp42F.f3=new_fd;_tmp42F;});
({void(*_tmp4DB)(void(*f)(struct _tuple22*,void*,void*),struct _tuple22*env,struct Cyc_Dict_Dict d)=({void(*_tmp15)(void(*f)(struct _tuple22*,void*,void*),struct _tuple22*env,struct Cyc_Dict_Dict d)=(void(*)(void(*f)(struct _tuple22*,void*,void*),struct _tuple22*env,struct Cyc_Dict_Dict d))Cyc_Dict_iter_c;_tmp15;});_tmp4DB(Cyc_NewControlFlow_check_unique_root,& ckenv,fd);});
goto _LL0;}}else{goto _LL3;}}else{_LL3: _LL4:
 goto _LL0;}_LL0:;}
# 226
if(!
# 235
Cyc_CfFlowInfo_flow_lessthan_approx(new_flow,*sflow)){
# 242
*sflow=new_flow;
# 246
if(annot->visited == env->iteration_num)
# 248
env->iterate_again=1;}}
# 135
static union Cyc_CfFlowInfo_FlowInfo Cyc_NewControlFlow_add_vars(struct Cyc_CfFlowInfo_FlowEnv*fenv,union Cyc_CfFlowInfo_FlowInfo inflow,struct Cyc_List_List*vds,void*leafval,unsigned loc,int nameit){
# 257
union Cyc_CfFlowInfo_FlowInfo _tmp16=inflow;struct Cyc_List_List*_tmp18;struct Cyc_Dict_Dict _tmp17;if((_tmp16.BottomFL).tag == 1){_LL1: _LL2:
 return Cyc_CfFlowInfo_BottomFL();}else{_LL3: _tmp17=((_tmp16.ReachableFL).val).f1;_tmp18=((_tmp16.ReachableFL).val).f2;_LL4: {struct Cyc_Dict_Dict d=_tmp17;struct Cyc_List_List*relns=_tmp18;
# 260
for(0;vds != 0;vds=vds->tl){
struct Cyc_CfFlowInfo_VarRoot_CfFlowInfo_Root_struct*root=({struct Cyc_CfFlowInfo_VarRoot_CfFlowInfo_Root_struct*_tmp1A=_cycalloc(sizeof(*_tmp1A));_tmp1A->tag=0U,_tmp1A->f1=(struct Cyc_Absyn_Vardecl*)vds->hd;_tmp1A;});
void*rval=Cyc_CfFlowInfo_typ_to_absrval(fenv,((struct Cyc_Absyn_Vardecl*)vds->hd)->type,0,leafval);
if(nameit)
rval=(void*)({struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*_tmp19=_cycalloc(sizeof(*_tmp19));_tmp19->tag=8U,_tmp19->f1=(struct Cyc_Absyn_Vardecl*)vds->hd,_tmp19->f2=rval;_tmp19;});
# 267
d=Cyc_Dict_insert(d,(void*)root,rval);}
# 269
return Cyc_CfFlowInfo_ReachableFL(d,relns);}}_LL0:;}
# 135
static int Cyc_NewControlFlow_relns_ok(struct Cyc_List_List*assume,struct Cyc_List_List*req){
# 280
for(0;(unsigned)req;req=req->tl){
struct Cyc_Relations_Reln*neg_rln=Cyc_Relations_negate(Cyc_Core_heap_region,(struct Cyc_Relations_Reln*)req->hd);
if(Cyc_Relations_consistent_relations(({struct Cyc_List_List*_tmp1B=_cycalloc(sizeof(*_tmp1B));_tmp1B->hd=neg_rln,_tmp1B->tl=assume;_tmp1B;})))
return 0;}
# 285
return 1;}
# 135
static struct Cyc_Absyn_Exp*Cyc_NewControlFlow_strip_cast(struct Cyc_Absyn_Exp*e){
# 289
void*_stmttmp2=e->r;void*_tmp1C=_stmttmp2;struct Cyc_Absyn_Exp*_tmp1D;if(((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp1C)->tag == 14U){_LL1: _tmp1D=((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp1C)->f2;_LL2: {struct Cyc_Absyn_Exp*e1=_tmp1D;
return e1;}}else{_LL3: _LL4:
 return e;}_LL0:;}
# 135
static void Cyc_NewControlFlow_check_union_requires(unsigned loc,void*t,struct _fat_ptr*f,union Cyc_CfFlowInfo_FlowInfo inflow){
# 297
union Cyc_CfFlowInfo_FlowInfo _tmp1E=inflow;struct Cyc_List_List*_tmp1F;if((_tmp1E.BottomFL).tag == 1){_LL1: _LL2:
 return;}else{_LL3: _tmp1F=((_tmp1E.ReachableFL).val).f2;_LL4: {struct Cyc_List_List*relns=_tmp1F;
# 300
{void*_stmttmp3=Cyc_Tcutil_compress(t);void*_tmp20=_stmttmp3;struct Cyc_List_List*_tmp21;struct Cyc_List_List*_tmp23;union Cyc_Absyn_AggrInfo _tmp22;switch(*((int*)_tmp20)){case 0U: if(((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp20)->f1)->tag == 20U){_LL6: _tmp22=((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp20)->f1)->f1;_tmp23=((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp20)->f2;_LL7: {union Cyc_Absyn_AggrInfo ainfo=_tmp22;struct Cyc_List_List*ts=_tmp23;
# 302
struct Cyc_Absyn_Aggrdecl*ad=Cyc_Absyn_get_known_aggrdecl(ainfo);
struct Cyc_Absyn_Aggrfield*field=Cyc_Absyn_lookup_decl_field(ad,f);
struct Cyc_Absyn_Exp*req=((struct Cyc_Absyn_Aggrfield*)_check_null(field))->requires_clause;
if(req != 0){
struct _RegionHandle _tmp24=_new_region("temp");struct _RegionHandle*temp=& _tmp24;_push_region(temp);
{struct Cyc_Absyn_Exp*ireq=({struct _RegionHandle*_tmp4DD=temp;struct Cyc_List_List*_tmp4DC=Cyc_List_rzip(temp,temp,ad->tvs,ts);Cyc_Tcutil_rsubsexp(_tmp4DD,_tmp4DC,req);});
# 309
if(!({struct Cyc_List_List*_tmp4DE=relns;Cyc_NewControlFlow_relns_ok(_tmp4DE,Cyc_Relations_exp2relns(temp,ireq));})){
({struct Cyc_String_pa_PrintArg_struct _tmp27=({struct Cyc_String_pa_PrintArg_struct _tmp432;_tmp432.tag=0U,({struct _fat_ptr _tmp4DF=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_exp2string(Cyc_NewControlFlow_strip_cast(ireq)));_tmp432.f1=_tmp4DF;});_tmp432;});struct Cyc_String_pa_PrintArg_struct _tmp28=({struct Cyc_String_pa_PrintArg_struct _tmp431;_tmp431.tag=0U,_tmp431.f1=(struct _fat_ptr)((struct _fat_ptr)*f);_tmp431;});void*_tmp25[2U];_tmp25[0]=& _tmp27,_tmp25[1]=& _tmp28;({unsigned _tmp4E1=loc;struct _fat_ptr _tmp4E0=({const char*_tmp26="unable to prove %s, required to access %s";_tag_fat(_tmp26,sizeof(char),42U);});Cyc_CfFlowInfo_aerr(_tmp4E1,_tmp4E0,_tag_fat(_tmp25,sizeof(void*),2U));});});
({void*_tmp29=0U;({struct Cyc___cycFILE*_tmp4E3=Cyc_stderr;struct _fat_ptr _tmp4E2=({const char*_tmp2A="  [recorded facts on path: ";_tag_fat(_tmp2A,sizeof(char),28U);});Cyc_fprintf(_tmp4E3,_tmp4E2,_tag_fat(_tmp29,sizeof(void*),0U));});});
Cyc_Relations_print_relns(Cyc_stderr,relns);
({void*_tmp2B=0U;({struct Cyc___cycFILE*_tmp4E5=Cyc_stderr;struct _fat_ptr _tmp4E4=({const char*_tmp2C="]\n";_tag_fat(_tmp2C,sizeof(char),3U);});Cyc_fprintf(_tmp4E5,_tmp4E4,_tag_fat(_tmp2B,sizeof(void*),0U));});});}}
# 307
;_pop_region(temp);}
# 305
goto _LL5;}}else{goto _LLA;}case 7U: _LL8: _tmp21=((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmp20)->f2;_LL9: {struct Cyc_List_List*fs=_tmp21;
# 318
struct Cyc_Absyn_Aggrfield*field=Cyc_Absyn_lookup_field(fs,f);
struct Cyc_Absyn_Exp*req=((struct Cyc_Absyn_Aggrfield*)_check_null(field))->requires_clause;
if(req != 0){
struct _RegionHandle _tmp2D=_new_region("temp");struct _RegionHandle*temp=& _tmp2D;_push_region(temp);
if(!({struct Cyc_List_List*_tmp4E6=relns;Cyc_NewControlFlow_relns_ok(_tmp4E6,Cyc_Relations_exp2relns(temp,req));})){
({struct Cyc_String_pa_PrintArg_struct _tmp30=({struct Cyc_String_pa_PrintArg_struct _tmp434;_tmp434.tag=0U,({struct _fat_ptr _tmp4E7=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_exp2string(Cyc_NewControlFlow_strip_cast(req)));_tmp434.f1=_tmp4E7;});_tmp434;});struct Cyc_String_pa_PrintArg_struct _tmp31=({struct Cyc_String_pa_PrintArg_struct _tmp433;_tmp433.tag=0U,_tmp433.f1=(struct _fat_ptr)((struct _fat_ptr)*f);_tmp433;});void*_tmp2E[2U];_tmp2E[0]=& _tmp30,_tmp2E[1]=& _tmp31;({unsigned _tmp4E9=loc;struct _fat_ptr _tmp4E8=({const char*_tmp2F="unable to prove %s, required to access %s";_tag_fat(_tmp2F,sizeof(char),42U);});Cyc_CfFlowInfo_aerr(_tmp4E9,_tmp4E8,_tag_fat(_tmp2E,sizeof(void*),2U));});});
({void*_tmp32=0U;({struct Cyc___cycFILE*_tmp4EB=Cyc_stderr;struct _fat_ptr _tmp4EA=({const char*_tmp33="  [recorded facts on path: ";_tag_fat(_tmp33,sizeof(char),28U);});Cyc_fprintf(_tmp4EB,_tmp4EA,_tag_fat(_tmp32,sizeof(void*),0U));});});
Cyc_Relations_print_relns(Cyc_stderr,relns);
({void*_tmp34=0U;({struct Cyc___cycFILE*_tmp4ED=Cyc_stderr;struct _fat_ptr _tmp4EC=({const char*_tmp35="]\n";_tag_fat(_tmp35,sizeof(char),3U);});Cyc_fprintf(_tmp4ED,_tmp4EC,_tag_fat(_tmp34,sizeof(void*),0U));});});}
# 322
;_pop_region(temp);}
# 320
goto _LL5;}default: _LLA: _LLB:
# 330
 goto _LL5;}_LL5:;}
# 332
goto _LL0;}}_LL0:;}
# 135
static union Cyc_CfFlowInfo_FlowInfo Cyc_NewControlFlow_use_Rval(struct Cyc_NewControlFlow_AnalEnv*env,unsigned loc,union Cyc_CfFlowInfo_FlowInfo inflow,void*r){
# 337
union Cyc_CfFlowInfo_FlowInfo _tmp36=inflow;struct Cyc_List_List*_tmp38;struct Cyc_Dict_Dict _tmp37;if((_tmp36.BottomFL).tag == 1){_LL1: _LL2:
 return Cyc_CfFlowInfo_BottomFL();}else{_LL3: _tmp37=((_tmp36.ReachableFL).val).f1;_tmp38=((_tmp36.ReachableFL).val).f2;_LL4: {struct Cyc_Dict_Dict d=_tmp37;struct Cyc_List_List*relns=_tmp38;
# 340
if((int)Cyc_CfFlowInfo_initlevel(env->fenv,d,r)!= (int)Cyc_CfFlowInfo_AllIL)
({void*_tmp39=0U;({unsigned _tmp4EF=loc;struct _fat_ptr _tmp4EE=({const char*_tmp3A="expression may not be fully initialized";_tag_fat(_tmp3A,sizeof(char),40U);});Cyc_CfFlowInfo_aerr(_tmp4EF,_tmp4EE,_tag_fat(_tmp39,sizeof(void*),0U));});});{
# 340
struct Cyc_Dict_Dict ans_d=
# 342
Cyc_CfFlowInfo_escape_deref(env->fenv,d,r);
if(d.t == ans_d.t)return inflow;{union Cyc_CfFlowInfo_FlowInfo ans=
Cyc_CfFlowInfo_ReachableFL(ans_d,relns);
Cyc_NewControlFlow_update_tryflow(env,ans);
return ans;}}}}_LL0:;}struct _tuple23{struct Cyc_Absyn_Tqual f1;void*f2;};
# 135
static void Cyc_NewControlFlow_check_nounique(struct Cyc_NewControlFlow_AnalEnv*env,unsigned loc,void*t,void*r){
# 351
struct _tuple0 _stmttmp4=({struct _tuple0 _tmp435;({void*_tmp4F0=Cyc_Tcutil_compress(t);_tmp435.f1=_tmp4F0;}),_tmp435.f2=r;_tmp435;});struct _tuple0 _tmp3B=_stmttmp4;struct _fat_ptr _tmp3E;struct Cyc_List_List*_tmp3D;enum Cyc_Absyn_AggrKind _tmp3C;struct _fat_ptr _tmp40;struct Cyc_List_List*_tmp3F;struct _fat_ptr _tmp43;struct Cyc_List_List*_tmp42;union Cyc_Absyn_AggrInfo _tmp41;struct _fat_ptr _tmp45;struct Cyc_Absyn_Datatypefield*_tmp44;void*_tmp46;switch(*((int*)_tmp3B.f2)){case 2U: if(((struct Cyc_CfFlowInfo_UnknownR_CfFlowInfo_AbsRVal_struct*)_tmp3B.f2)->f1 == Cyc_CfFlowInfo_NoneIL){_LL1: _LL2:
 return;}else{switch(*((int*)_tmp3B.f1)){case 0U: switch(*((int*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp3B.f1)->f1)){case 19U: if(((((struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp3B.f1)->f1)->f1).KnownDatatypefield).tag == 2)goto _LL13;else{goto _LL13;}case 20U: goto _LL13;default: goto _LL13;}case 6U: goto _LL13;case 7U: goto _LL13;case 3U: goto _LL11;default: goto _LL13;}}case 0U: _LL3: _LL4:
 return;case 7U: _LL5: _LL6:
 return;case 8U: _LL7: _tmp46=(void*)((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmp3B.f2)->f2;_LL8: {void*r=_tmp46;
Cyc_NewControlFlow_check_nounique(env,loc,t,r);return;}default: switch(*((int*)_tmp3B.f1)){case 0U: switch(*((int*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp3B.f1)->f1)){case 19U: if(((((struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp3B.f1)->f1)->f1).KnownDatatypefield).tag == 2){if(((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp3B.f2)->tag == 6U){_LL9: _tmp44=(((((struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp3B.f1)->f1)->f1).KnownDatatypefield).val).f2;_tmp45=((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp3B.f2)->f2;_LLA: {struct Cyc_Absyn_Datatypefield*tuf=_tmp44;struct _fat_ptr ad=_tmp45;
# 357
if(tuf->typs == 0)
return;
# 357
_tmp3F=tuf->typs;_tmp40=ad;goto _LLC;}}else{goto _LL13;}}else{goto _LL13;}case 20U: if(((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp3B.f2)->tag == 6U){_LLD: _tmp41=((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp3B.f1)->f1)->f1;_tmp42=((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp3B.f1)->f2;_tmp43=((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp3B.f2)->f2;_LLE: {union Cyc_Absyn_AggrInfo info=_tmp41;struct Cyc_List_List*targs=_tmp42;struct _fat_ptr d=_tmp43;
# 368
struct Cyc_Absyn_Aggrdecl*ad=Cyc_Absyn_get_known_aggrdecl(info);
if(ad->impl == 0)return;{struct Cyc_List_List*fs=((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->fields;
# 371
struct _RegionHandle _tmp47=_new_region("temp");struct _RegionHandle*temp=& _tmp47;_push_region(temp);
{struct Cyc_List_List*inst=Cyc_List_rzip(temp,temp,ad->tvs,targs);
{int i=0;for(0;(unsigned)i < _get_fat_size(d,sizeof(void*));(i ++,fs=fs->tl)){
void*t=((struct Cyc_Absyn_Aggrfield*)((struct Cyc_List_List*)_check_null(fs))->hd)->type;
if(inst != 0)t=Cyc_Tcutil_rsubstitute(temp,inst,t);Cyc_NewControlFlow_check_nounique(env,loc,t,((void**)d.curr)[i]);}}
# 378
_npop_handler(0U);return;}
# 372
;_pop_region(temp);}}}else{goto _LL13;}default: goto _LL13;}case 6U: if(((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp3B.f2)->tag == 6U){_LLB: _tmp3F=((struct Cyc_Absyn_TupleType_Absyn_Type_struct*)_tmp3B.f1)->f1;_tmp40=((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp3B.f2)->f2;_LLC: {struct Cyc_List_List*tqts=_tmp3F;struct _fat_ptr ad=_tmp40;
# 361
unsigned sz=(unsigned)Cyc_List_length(tqts);
{int i=0;for(0;(unsigned)i < sz;(i ++,tqts=tqts->tl)){
({struct Cyc_NewControlFlow_AnalEnv*_tmp4F3=env;unsigned _tmp4F2=loc;void*_tmp4F1=(*((struct _tuple23*)((struct Cyc_List_List*)_check_null(tqts))->hd)).f2;Cyc_NewControlFlow_check_nounique(_tmp4F3,_tmp4F2,_tmp4F1,*((void**)_check_fat_subscript(ad,sizeof(void*),i)));});}}
# 365
return;}}else{goto _LL13;}case 7U: if(((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp3B.f2)->tag == 6U){_LLF: _tmp3C=((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmp3B.f1)->f1;_tmp3D=((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmp3B.f1)->f2;_tmp3E=((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp3B.f2)->f2;_LL10: {enum Cyc_Absyn_AggrKind k=_tmp3C;struct Cyc_List_List*fs=_tmp3D;struct _fat_ptr d=_tmp3E;
# 381
{int i=0;for(0;(unsigned)i < _get_fat_size(d,sizeof(void*));(i ++,fs=fs->tl)){
Cyc_NewControlFlow_check_nounique(env,loc,((struct Cyc_Absyn_Aggrfield*)((struct Cyc_List_List*)_check_null(fs))->hd)->type,((void**)d.curr)[i]);}}
return;}}else{goto _LL13;}case 3U: _LL11: _LL12:
# 385
 if(Cyc_Tcutil_is_noalias_pointer(t,0))
({void*_tmp48=0U;({unsigned _tmp4F5=loc;struct _fat_ptr _tmp4F4=({const char*_tmp49="argument may still contain alias-free pointers";_tag_fat(_tmp49,sizeof(char),47U);});Cyc_Warn_warn(_tmp4F5,_tmp4F4,_tag_fat(_tmp48,sizeof(void*),0U));});});
# 385
return;default: _LL13: _LL14:
# 388
 return;}}_LL0:;}
# 135
static union Cyc_CfFlowInfo_FlowInfo Cyc_NewControlFlow_use_nounique_Rval(struct Cyc_NewControlFlow_AnalEnv*env,unsigned loc,void*t,union Cyc_CfFlowInfo_FlowInfo inflow,void*r){
# 393
union Cyc_CfFlowInfo_FlowInfo _tmp4A=inflow;struct Cyc_List_List*_tmp4C;struct Cyc_Dict_Dict _tmp4B;if((_tmp4A.BottomFL).tag == 1){_LL1: _LL2:
 return Cyc_CfFlowInfo_BottomFL();}else{_LL3: _tmp4B=((_tmp4A.ReachableFL).val).f1;_tmp4C=((_tmp4A.ReachableFL).val).f2;_LL4: {struct Cyc_Dict_Dict d=_tmp4B;struct Cyc_List_List*relns=_tmp4C;
# 396
if(!Cyc_Tcutil_is_noalias_pointer(t,0))
({void*_tmp4D=0U;({int(*_tmp4F7)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp4F)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmp4F;});struct _fat_ptr _tmp4F6=({const char*_tmp4E="noliveunique attribute requires alias-free pointer";_tag_fat(_tmp4E,sizeof(char),51U);});_tmp4F7(_tmp4F6,_tag_fat(_tmp4D,sizeof(void*),0U));});});{
# 396
void*elt_type=
# 398
Cyc_Tcutil_pointer_elt_type(t);
retry: {void*_tmp50=r;void*_tmp51;struct Cyc_CfFlowInfo_Place*_tmp52;void*_tmp53;switch(*((int*)_tmp50)){case 8U: _LL6: _tmp53=(void*)((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmp50)->f2;_LL7: {void*r2=_tmp53;
r=r2;goto retry;}case 4U: _LL8: _tmp52=((struct Cyc_CfFlowInfo_AddressOf_CfFlowInfo_AbsRVal_struct*)_tmp50)->f1;_LL9: {struct Cyc_CfFlowInfo_Place*p=_tmp52;
_tmp51=Cyc_CfFlowInfo_lookup_place(d,p);goto _LLB;}case 5U: _LLA: _tmp51=(void*)((struct Cyc_CfFlowInfo_UniquePtr_CfFlowInfo_AbsRVal_struct*)_tmp50)->f1;_LLB: {void*r=_tmp51;
Cyc_NewControlFlow_check_nounique(env,loc,elt_type,r);goto _LL5;}default: _LLC: _LLD:
# 404
 if(Cyc_Tcutil_is_noalias_pointer_or_aggr(elt_type))
({void*_tmp54=0U;({unsigned _tmp4F9=loc;struct _fat_ptr _tmp4F8=({const char*_tmp55="argument may contain live alias-free pointers";_tag_fat(_tmp55,sizeof(char),46U);});Cyc_Warn_warn(_tmp4F9,_tmp4F8,_tag_fat(_tmp54,sizeof(void*),0U));});});
# 404
return
# 406
 Cyc_NewControlFlow_use_Rval(env,loc,inflow,r);}_LL5:;}{
# 408
struct Cyc_Dict_Dict ans_d=Cyc_CfFlowInfo_escape_deref(env->fenv,d,r);
if(d.t == ans_d.t)return inflow;{union Cyc_CfFlowInfo_FlowInfo ans=
Cyc_CfFlowInfo_ReachableFL(ans_d,relns);
Cyc_NewControlFlow_update_tryflow(env,ans);
return ans;}}}}}_LL0:;}struct _tuple24{union Cyc_CfFlowInfo_FlowInfo f1;struct Cyc_List_List*f2;};
# 135
static struct _tuple24 Cyc_NewControlFlow_anal_Rexps(struct Cyc_NewControlFlow_AnalEnv*env,union Cyc_CfFlowInfo_FlowInfo inflow,struct Cyc_List_List*es,int first_is_copy,int others_are_copy){
# 423
struct Cyc_List_List*rvals=0;
if(es == 0)
return({struct _tuple24 _tmp436;_tmp436.f1=inflow,_tmp436.f2=0;_tmp436;});
# 424
do{
# 427
struct _tuple17 _stmttmp5=Cyc_NewControlFlow_anal_Rexp(env,first_is_copy,inflow,(struct Cyc_Absyn_Exp*)es->hd);void*_tmp57;union Cyc_CfFlowInfo_FlowInfo _tmp56;_LL1: _tmp56=_stmttmp5.f1;_tmp57=_stmttmp5.f2;_LL2: {union Cyc_CfFlowInfo_FlowInfo f=_tmp56;void*r=_tmp57;
inflow=f;
rvals=({struct Cyc_List_List*_tmp58=_cycalloc(sizeof(*_tmp58));_tmp58->hd=r,_tmp58->tl=rvals;_tmp58;});
es=es->tl;
first_is_copy=others_are_copy;}}while(es != 0);
# 434
Cyc_NewControlFlow_update_tryflow(env,inflow);
return({struct _tuple24 _tmp437;_tmp437.f1=inflow,({struct Cyc_List_List*_tmp4FA=Cyc_List_imp_rev(rvals);_tmp437.f2=_tmp4FA;});_tmp437;});}
# 135
static struct _tuple17 Cyc_NewControlFlow_anal_use_ints(struct Cyc_NewControlFlow_AnalEnv*env,union Cyc_CfFlowInfo_FlowInfo inflow,struct Cyc_List_List*es){
# 442
struct _tuple24 _stmttmp6=
Cyc_NewControlFlow_anal_Rexps(env,inflow,es,0,0);
# 442
struct Cyc_List_List*_tmp5A;union Cyc_CfFlowInfo_FlowInfo _tmp59;_LL1: _tmp59=_stmttmp6.f1;_tmp5A=_stmttmp6.f2;_LL2: {union Cyc_CfFlowInfo_FlowInfo afterflow=_tmp59;struct Cyc_List_List*rvals=_tmp5A;
# 444
{union Cyc_CfFlowInfo_FlowInfo _tmp5B=afterflow;struct Cyc_Dict_Dict _tmp5C;if((_tmp5B.ReachableFL).tag == 2){_LL4: _tmp5C=((_tmp5B.ReachableFL).val).f1;_LL5: {struct Cyc_Dict_Dict d=_tmp5C;
# 446
for(0;rvals != 0;(rvals=rvals->tl,es=((struct Cyc_List_List*)_check_null(es))->tl)){
if((int)Cyc_CfFlowInfo_initlevel(env->fenv,d,(void*)rvals->hd)== (int)Cyc_CfFlowInfo_NoneIL)
({void*_tmp5D=0U;({unsigned _tmp4FC=((struct Cyc_Absyn_Exp*)((struct Cyc_List_List*)_check_null(es))->hd)->loc;struct _fat_ptr _tmp4FB=({const char*_tmp5E="expression may not be initialized";_tag_fat(_tmp5E,sizeof(char),34U);});Cyc_CfFlowInfo_aerr(_tmp4FC,_tmp4FB,_tag_fat(_tmp5D,sizeof(void*),0U));});});}
# 446
goto _LL3;}}else{_LL6: _LL7:
# 450
 goto _LL3;}_LL3:;}
# 452
return({struct _tuple17 _tmp438;_tmp438.f1=afterflow,_tmp438.f2=(env->fenv)->unknown_all;_tmp438;});}}
# 135
static void*Cyc_NewControlFlow_consume_zero_rval(struct Cyc_NewControlFlow_AnalEnv*env,struct Cyc_Dict_Dict new_dict,struct Cyc_CfFlowInfo_Place*p,struct Cyc_Absyn_Exp*e,void*new_rval){
# 466
int needs_unconsume=0;
void*old_rval=Cyc_CfFlowInfo_lookup_place(new_dict,p);
if(Cyc_CfFlowInfo_is_unique_consumed(e,env->iteration_num,old_rval,& needs_unconsume))
({void*_tmp5F=0U;({int(*_tmp4FE)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp61)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmp61;});struct _fat_ptr _tmp4FD=({const char*_tmp60="consume_zero_rval";_tag_fat(_tmp60,sizeof(char),18U);});_tmp4FE(_tmp4FD,_tag_fat(_tmp5F,sizeof(void*),0U));});});else{
# 471
if(needs_unconsume)
return Cyc_CfFlowInfo_make_unique_consumed(env->fenv,(void*)_check_null(e->topt),e,env->iteration_num,new_rval);else{
# 475
return new_rval;}}}
# 135
static union Cyc_CfFlowInfo_FlowInfo Cyc_NewControlFlow_notzero(struct Cyc_NewControlFlow_AnalEnv*env,union Cyc_CfFlowInfo_FlowInfo inflow,union Cyc_CfFlowInfo_FlowInfo outflow,struct Cyc_Absyn_Exp*e,enum Cyc_CfFlowInfo_InitLevel il,struct Cyc_List_List*names){
# 493 "new_control_flow.cyc"
union Cyc_CfFlowInfo_FlowInfo _tmp62=outflow;struct Cyc_List_List*_tmp64;struct Cyc_Dict_Dict _tmp63;if((_tmp62.BottomFL).tag == 1){_LL1: _LL2:
 return outflow;}else{_LL3: _tmp63=((_tmp62.ReachableFL).val).f1;_tmp64=((_tmp62.ReachableFL).val).f2;_LL4: {struct Cyc_Dict_Dict d=_tmp63;struct Cyc_List_List*relns=_tmp64;
# 496
union Cyc_CfFlowInfo_AbsLVal _stmttmp7=(Cyc_NewControlFlow_anal_Lexp(env,inflow,0,0,e)).f2;union Cyc_CfFlowInfo_AbsLVal _tmp65=_stmttmp7;struct Cyc_CfFlowInfo_Place*_tmp66;if((_tmp65.UnknownL).tag == 2){_LL6: _LL7:
# 500
 return outflow;}else{_LL8: _tmp66=(_tmp65.PlaceL).val;_LL9: {struct Cyc_CfFlowInfo_Place*p=_tmp66;
# 504
void*nzval=(int)il == (int)1U?(env->fenv)->notzeroall:(env->fenv)->unknown_none;
for(0;names != 0;names=names->tl){
nzval=(void*)({struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*_tmp67=_cycalloc(sizeof(*_tmp67));_tmp67->tag=8U,_tmp67->f1=(struct Cyc_Absyn_Vardecl*)names->hd,_tmp67->f2=nzval;_tmp67;});}
# 508
nzval=Cyc_NewControlFlow_consume_zero_rval(env,d,p,e,nzval);{
union Cyc_CfFlowInfo_FlowInfo outflow=({struct Cyc_Dict_Dict _tmp4FF=Cyc_CfFlowInfo_assign_place(env->fenv,e->loc,d,p,nzval);Cyc_CfFlowInfo_ReachableFL(_tmp4FF,relns);});
# 513
return outflow;}}}_LL5:;}}_LL0:;}
# 135 "new_control_flow.cyc"
static struct _tuple20 Cyc_NewControlFlow_splitzero(struct Cyc_NewControlFlow_AnalEnv*env,union Cyc_CfFlowInfo_FlowInfo inflow,union Cyc_CfFlowInfo_FlowInfo outflow,struct Cyc_Absyn_Exp*e,enum Cyc_CfFlowInfo_InitLevel il,struct Cyc_List_List*names){
# 524 "new_control_flow.cyc"
union Cyc_CfFlowInfo_FlowInfo _tmp68=outflow;struct Cyc_List_List*_tmp6A;struct Cyc_Dict_Dict _tmp69;if((_tmp68.BottomFL).tag == 1){_LL1: _LL2:
 return({struct _tuple20 _tmp439;_tmp439.f1=outflow,_tmp439.f2=outflow;_tmp439;});}else{_LL3: _tmp69=((_tmp68.ReachableFL).val).f1;_tmp6A=((_tmp68.ReachableFL).val).f2;_LL4: {struct Cyc_Dict_Dict d=_tmp69;struct Cyc_List_List*relns=_tmp6A;
# 527
union Cyc_CfFlowInfo_AbsLVal _stmttmp8=(Cyc_NewControlFlow_anal_Lexp(env,inflow,0,0,e)).f2;union Cyc_CfFlowInfo_AbsLVal _tmp6B=_stmttmp8;struct Cyc_CfFlowInfo_Place*_tmp6C;if((_tmp6B.UnknownL).tag == 2){_LL6: _LL7:
 return({struct _tuple20 _tmp43A;_tmp43A.f1=outflow,_tmp43A.f2=outflow;_tmp43A;});}else{_LL8: _tmp6C=(_tmp6B.PlaceL).val;_LL9: {struct Cyc_CfFlowInfo_Place*p=_tmp6C;
# 530
void*nzval=(int)il == (int)1U?(env->fenv)->notzeroall:(env->fenv)->unknown_none;
void*zval=(env->fenv)->zero;
for(0;names != 0;names=names->tl){
nzval=(void*)({struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*_tmp6D=_cycalloc(sizeof(*_tmp6D));_tmp6D->tag=8U,_tmp6D->f1=(struct Cyc_Absyn_Vardecl*)names->hd,_tmp6D->f2=nzval;_tmp6D;});
zval=(void*)({struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*_tmp6E=_cycalloc(sizeof(*_tmp6E));_tmp6E->tag=8U,_tmp6E->f1=(struct Cyc_Absyn_Vardecl*)names->hd,_tmp6E->f2=zval;_tmp6E;});}
# 536
nzval=Cyc_NewControlFlow_consume_zero_rval(env,d,p,e,nzval);
zval=Cyc_NewControlFlow_consume_zero_rval(env,d,p,e,zval);
return({struct _tuple20 _tmp43B;({
union Cyc_CfFlowInfo_FlowInfo _tmp503=({struct Cyc_Dict_Dict _tmp502=Cyc_CfFlowInfo_assign_place(env->fenv,e->loc,d,p,nzval);Cyc_CfFlowInfo_ReachableFL(_tmp502,relns);});_tmp43B.f1=_tmp503;}),({
union Cyc_CfFlowInfo_FlowInfo _tmp501=({struct Cyc_Dict_Dict _tmp500=Cyc_CfFlowInfo_assign_place(env->fenv,e->loc,d,p,zval);Cyc_CfFlowInfo_ReachableFL(_tmp500,relns);});_tmp43B.f2=_tmp501;});_tmp43B;});}}_LL5:;}}_LL0:;}
# 135 "new_control_flow.cyc"
static struct Cyc_CfFlowInfo_NotZero_Absyn_AbsynAnnot_struct Cyc_NewControlFlow_mt_notzero_v={Cyc_CfFlowInfo_NotZero,0};
# 546 "new_control_flow.cyc"
static struct Cyc_CfFlowInfo_UnknownZ_Absyn_AbsynAnnot_struct Cyc_NewControlFlow_mt_unknownz_v={Cyc_CfFlowInfo_UnknownZ,0};
# 548
static void Cyc_NewControlFlow_update_relns(struct Cyc_Absyn_Exp*e,int possibly_null,struct Cyc_List_List*relns){
# 551
{void*_stmttmp9=e->annot;void*_tmp6F=_stmttmp9;struct Cyc_List_List*_tmp70;struct Cyc_List_List*_tmp71;if(((struct Cyc_CfFlowInfo_UnknownZ_Absyn_AbsynAnnot_struct*)_tmp6F)->tag == Cyc_CfFlowInfo_UnknownZ){_LL1: _tmp71=((struct Cyc_CfFlowInfo_UnknownZ_Absyn_AbsynAnnot_struct*)_tmp6F)->f1;_LL2: {struct Cyc_List_List*relns2=_tmp71;
# 553
if(possibly_null && Cyc_Relations_same_relns(relns,relns2))return;goto _LL0;}}else{if(((struct Cyc_CfFlowInfo_NotZero_Absyn_AbsynAnnot_struct*)_tmp6F)->tag == Cyc_CfFlowInfo_NotZero){_LL3: _tmp70=((struct Cyc_CfFlowInfo_NotZero_Absyn_AbsynAnnot_struct*)_tmp6F)->f1;_LL4: {struct Cyc_List_List*relns2=_tmp70;
# 556
if(!possibly_null && Cyc_Relations_same_relns(relns,relns2))return;goto _LL0;}}else{_LL5: _LL6:
# 558
 goto _LL0;}}_LL0:;}
# 560
if(possibly_null)
({void*_tmp504=(void*)({struct Cyc_CfFlowInfo_UnknownZ_Absyn_AbsynAnnot_struct*_tmp72=_cycalloc(sizeof(*_tmp72));_tmp72->tag=Cyc_CfFlowInfo_UnknownZ,_tmp72->f1=relns;_tmp72;});e->annot=_tmp504;});else{
# 563
({void*_tmp505=(void*)({struct Cyc_CfFlowInfo_NotZero_Absyn_AbsynAnnot_struct*_tmp73=_cycalloc(sizeof(*_tmp73));_tmp73->tag=Cyc_CfFlowInfo_NotZero,_tmp73->f1=relns;_tmp73;});e->annot=_tmp505;});}}
# 548
static struct _tuple17 Cyc_NewControlFlow_anal_derefR(struct Cyc_NewControlFlow_AnalEnv*env,union Cyc_CfFlowInfo_FlowInfo inflow,union Cyc_CfFlowInfo_FlowInfo f,struct Cyc_Absyn_Exp*e,void*r,struct Cyc_Absyn_Exp*index){
# 575
void*_stmttmpA=Cyc_Tcutil_compress((void*)_check_null(e->topt));void*_tmp74=_stmttmpA;void*_tmp76;void*_tmp75;if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp74)->tag == 3U){_LL1: _tmp75=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp74)->f1).elt_type;_tmp76=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp74)->f1).ptr_atts).bounds;_LL2: {void*elttype=_tmp75;void*bd=_tmp76;
# 577
union Cyc_CfFlowInfo_FlowInfo _tmp77=f;struct Cyc_List_List*_tmp79;struct Cyc_Dict_Dict _tmp78;if((_tmp77.BottomFL).tag == 1){_LL6: _LL7:
# 579
 return({struct _tuple17 _tmp43C;_tmp43C.f1=f,({void*_tmp506=Cyc_CfFlowInfo_typ_to_absrval(env->fenv,elttype,0,(env->fenv)->unknown_all);_tmp43C.f2=_tmp506;});_tmp43C;});}else{_LL8: _tmp78=((_tmp77.ReachableFL).val).f1;_tmp79=((_tmp77.ReachableFL).val).f2;_LL9: {struct Cyc_Dict_Dict outdict=_tmp78;struct Cyc_List_List*relns=_tmp79;
# 581
struct _tuple16 _stmttmpB=Cyc_CfFlowInfo_unname_rval(r);struct Cyc_List_List*_tmp7B;void*_tmp7A;_LLB: _tmp7A=_stmttmpB.f1;_tmp7B=_stmttmpB.f2;_LLC: {void*r=_tmp7A;struct Cyc_List_List*names=_tmp7B;
{void*_tmp7C=r;enum Cyc_CfFlowInfo_InitLevel _tmp7D;void*_tmp7E;struct Cyc_CfFlowInfo_Place*_tmp7F;void*_tmp81;struct Cyc_Absyn_Vardecl*_tmp80;switch(*((int*)_tmp7C)){case 8U: _LLE: _tmp80=((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmp7C)->f1;_tmp81=(void*)((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmp7C)->f2;_LLF: {struct Cyc_Absyn_Vardecl*n=_tmp80;void*r2=_tmp81;
# 584
({void*_tmp82=0U;({int(*_tmp508)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp84)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmp84;});struct _fat_ptr _tmp507=({const char*_tmp83="named location in anal_derefR";_tag_fat(_tmp83,sizeof(char),30U);});_tmp508(_tmp507,_tag_fat(_tmp82,sizeof(void*),0U));});});}case 1U: _LL10: _LL11:
# 586
 Cyc_NewControlFlow_update_relns(e,0,relns);
goto _LLD;case 4U: _LL12: _tmp7F=((struct Cyc_CfFlowInfo_AddressOf_CfFlowInfo_AbsRVal_struct*)_tmp7C)->f1;_LL13: {struct Cyc_CfFlowInfo_Place*p=_tmp7F;
# 589
Cyc_NewControlFlow_update_relns(e,0,relns);
if(index == 0 && Cyc_Tcutil_is_bound_one(bd))
return({struct _tuple17 _tmp43D;_tmp43D.f1=f,({void*_tmp509=Cyc_CfFlowInfo_lookup_place(outdict,p);_tmp43D.f2=_tmp509;});_tmp43D;});
# 590
goto _LLD;}case 5U: _LL14: _tmp7E=(void*)((struct Cyc_CfFlowInfo_UniquePtr_CfFlowInfo_AbsRVal_struct*)_tmp7C)->f1;_LL15: {void*r=_tmp7E;
# 595
Cyc_NewControlFlow_update_relns(e,1,relns);
if(index == 0 && Cyc_Tcutil_is_bound_one(bd))
return({struct _tuple17 _tmp43E;_tmp43E.f1=f,_tmp43E.f2=r;_tmp43E;});
# 596
goto _LLD;}case 0U: _LL16: _LL17:
# 600
 e->annot=(void*)& Cyc_CfFlowInfo_IsZero_val;
return({struct _tuple17 _tmp43F;({union Cyc_CfFlowInfo_FlowInfo _tmp50B=Cyc_CfFlowInfo_BottomFL();_tmp43F.f1=_tmp50B;}),({void*_tmp50A=Cyc_CfFlowInfo_typ_to_absrval(env->fenv,elttype,0,(env->fenv)->unknown_all);_tmp43F.f2=_tmp50A;});_tmp43F;});case 2U: _LL18: _tmp7D=((struct Cyc_CfFlowInfo_UnknownR_CfFlowInfo_AbsRVal_struct*)_tmp7C)->f1;_LL19: {enum Cyc_CfFlowInfo_InitLevel il=_tmp7D;
# 604
f=Cyc_NewControlFlow_notzero(env,inflow,f,e,il,names);
goto _LL1B;}default: _LL1A: _LL1B:
# 607
 Cyc_NewControlFlow_update_relns(e,1,relns);
goto _LLD;}_LLD:;}{
# 610
enum Cyc_CfFlowInfo_InitLevel _stmttmpC=Cyc_CfFlowInfo_initlevel(env->fenv,outdict,r);enum Cyc_CfFlowInfo_InitLevel _tmp85=_stmttmpC;if(_tmp85 == Cyc_CfFlowInfo_NoneIL){_LL1D: _LL1E: {
# 612
struct _tuple16 _stmttmpD=Cyc_CfFlowInfo_unname_rval(r);void*_tmp86;_LL22: _tmp86=_stmttmpD.f1;_LL23: {void*r=_tmp86;
{void*_tmp87=r;if(((struct Cyc_CfFlowInfo_Consumed_CfFlowInfo_AbsRVal_struct*)_tmp87)->tag == 7U){_LL25: _LL26:
# 615
({void*_tmp88=0U;({unsigned _tmp50D=e->loc;struct _fat_ptr _tmp50C=({const char*_tmp89="attempt to dereference a consumed alias-free pointer";_tag_fat(_tmp89,sizeof(char),53U);});Cyc_CfFlowInfo_aerr(_tmp50D,_tmp50C,_tag_fat(_tmp88,sizeof(void*),0U));});});
goto _LL24;}else{_LL27: _LL28:
# 618
({void*_tmp8A=0U;({unsigned _tmp50F=e->loc;struct _fat_ptr _tmp50E=({const char*_tmp8B="dereference of possibly uninitialized pointer";_tag_fat(_tmp8B,sizeof(char),46U);});Cyc_CfFlowInfo_aerr(_tmp50F,_tmp50E,_tag_fat(_tmp8A,sizeof(void*),0U));});});}_LL24:;}
# 620
goto _LL20;}}}else{_LL1F: _LL20:
# 622
 return({struct _tuple17 _tmp440;_tmp440.f1=f,({void*_tmp510=Cyc_CfFlowInfo_typ_to_absrval(env->fenv,elttype,0,(env->fenv)->unknown_all);_tmp440.f2=_tmp510;});_tmp440;});}_LL1C:;}}}}_LL5:;}}else{_LL3: _LL4:
# 625
({void*_tmp8C=0U;({int(*_tmp512)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp8E)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmp8E;});struct _fat_ptr _tmp511=({const char*_tmp8D="right deref of non-pointer-type";_tag_fat(_tmp8D,sizeof(char),32U);});_tmp512(_tmp511,_tag_fat(_tmp8C,sizeof(void*),0U));});});}_LL0:;}
# 548
static struct Cyc_List_List*Cyc_NewControlFlow_add_subscript_reln(struct Cyc_List_List*relns,struct Cyc_Absyn_Exp*e1,struct Cyc_Absyn_Exp*e2){
# 634
union Cyc_Relations_RelnOp n2=Cyc_Relations_RConst(0U);
int e2_valid_op=Cyc_Relations_exp2relnop(e2,& n2);
# 637
{void*_stmttmpE=e1->r;void*_tmp8F=_stmttmpE;void*_tmp90;if(((struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct*)_tmp8F)->tag == 1U){_LL1: _tmp90=(void*)((struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct*)_tmp8F)->f1;_LL2: {void*b1=_tmp90;
# 639
struct Cyc_Absyn_Vardecl*x=Cyc_Tcutil_nonesc_vardecl(b1);
if(x != 0){
union Cyc_Relations_RelnOp n1=Cyc_Relations_RNumelts(x);
if(e2_valid_op)
relns=Cyc_Relations_add_relation(Cyc_Core_heap_region,n2,Cyc_Relations_Rlt,n1,relns);}
# 640
goto _LL0;}}else{_LL3: _LL4:
# 646
 goto _LL0;}_LL0:;}{
# 649
struct Cyc_Absyn_Exp*bound=Cyc_Tcutil_get_type_bound((void*)_check_null(e1->topt));
if(bound != 0){
union Cyc_Relations_RelnOp rbound=Cyc_Relations_RConst(0U);
if(Cyc_Relations_exp2relnop(bound,& rbound))
relns=Cyc_Relations_add_relation(Cyc_Core_heap_region,n2,Cyc_Relations_Rlt,rbound,relns);}
# 650
return relns;}}
# 548
static union Cyc_CfFlowInfo_FlowInfo Cyc_NewControlFlow_restore_noconsume_arg(struct Cyc_NewControlFlow_AnalEnv*env,union Cyc_CfFlowInfo_FlowInfo inflow,struct Cyc_Absyn_Exp*exp,unsigned loc,void*old_rval){
# 668
struct _tuple19 _stmttmpF=Cyc_NewControlFlow_anal_Lexp(env,inflow,1,1,exp);union Cyc_CfFlowInfo_AbsLVal _tmp91;_LL1: _tmp91=_stmttmpF.f2;_LL2: {union Cyc_CfFlowInfo_AbsLVal lval=_tmp91;
{struct _tuple19 _stmttmp10=({struct _tuple19 _tmp442;_tmp442.f1=inflow,_tmp442.f2=lval;_tmp442;});struct _tuple19 _tmp92=_stmttmp10;struct Cyc_CfFlowInfo_Place*_tmp95;struct Cyc_List_List*_tmp94;struct Cyc_Dict_Dict _tmp93;if(((_tmp92.f1).ReachableFL).tag == 2){if(((_tmp92.f2).PlaceL).tag == 1){_LL4: _tmp93=(((_tmp92.f1).ReachableFL).val).f1;_tmp94=(((_tmp92.f1).ReachableFL).val).f2;_tmp95=((_tmp92.f2).PlaceL).val;_LL5: {struct Cyc_Dict_Dict fd=_tmp93;struct Cyc_List_List*relns=_tmp94;struct Cyc_CfFlowInfo_Place*p=_tmp95;
# 671
void*new_rval=Cyc_CfFlowInfo_typ_to_absrval(env->fenv,(void*)_check_null(exp->topt),0,(env->fenv)->unknown_all);
# 673
struct _tuple16 _stmttmp11=Cyc_CfFlowInfo_unname_rval(old_rval);struct Cyc_List_List*_tmp97;void*_tmp96;_LLB: _tmp96=_stmttmp11.f1;_tmp97=_stmttmp11.f2;_LLC: {void*old_rval=_tmp96;struct Cyc_List_List*names=_tmp97;
for(0;names != 0;names=names->tl){
new_rval=(void*)({struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*_tmp98=_cycalloc(sizeof(*_tmp98));_tmp98->tag=8U,_tmp98->f1=(struct Cyc_Absyn_Vardecl*)names->hd,_tmp98->f2=new_rval;_tmp98;});}
# 678
fd=Cyc_CfFlowInfo_assign_place(env->fenv,loc,fd,p,new_rval);
inflow=Cyc_CfFlowInfo_ReachableFL(fd,relns);
Cyc_NewControlFlow_update_tryflow(env,inflow);
# 684
goto _LL3;}}}else{_LL8: _LL9:
# 687
({struct Cyc_String_pa_PrintArg_struct _tmp9C=({struct Cyc_String_pa_PrintArg_struct _tmp441;_tmp441.tag=0U,({struct _fat_ptr _tmp513=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_exp2string(exp));_tmp441.f1=_tmp513;});_tmp441;});void*_tmp99[1U];_tmp99[0]=& _tmp9C;({int(*_tmp515)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp9B)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmp9B;});struct _fat_ptr _tmp514=({const char*_tmp9A="noconsume parameter %s must be l-values";_tag_fat(_tmp9A,sizeof(char),40U);});_tmp515(_tmp514,_tag_fat(_tmp99,sizeof(void*),1U));});});
goto _LL3;}}else{_LL6: _LL7:
# 685
 goto _LL3;}_LL3:;}
# 690
return inflow;}}
# 548
static struct _tuple17 Cyc_NewControlFlow_do_assign(struct Cyc_CfFlowInfo_FlowEnv*fenv,struct Cyc_NewControlFlow_AnalEnv*env,union Cyc_CfFlowInfo_FlowInfo outflow,struct Cyc_Absyn_Exp*lexp,union Cyc_CfFlowInfo_AbsLVal lval,struct Cyc_Absyn_Exp*rexp,void*rval,unsigned loc){
# 702
union Cyc_CfFlowInfo_FlowInfo _tmp9D=outflow;struct Cyc_List_List*_tmp9F;struct Cyc_Dict_Dict _tmp9E;if((_tmp9D.BottomFL).tag == 1){_LL1: _LL2:
# 704
 return({struct _tuple17 _tmp443;({union Cyc_CfFlowInfo_FlowInfo _tmp516=Cyc_CfFlowInfo_BottomFL();_tmp443.f1=_tmp516;}),_tmp443.f2=rval;_tmp443;});}else{_LL3: _tmp9E=((_tmp9D.ReachableFL).val).f1;_tmp9F=((_tmp9D.ReachableFL).val).f2;_LL4: {struct Cyc_Dict_Dict indict=_tmp9E;struct Cyc_List_List*relns=_tmp9F;
# 706
union Cyc_CfFlowInfo_AbsLVal _tmpA0=lval;struct Cyc_CfFlowInfo_Place*_tmpA1;if((_tmpA0.PlaceL).tag == 1){_LL6: _tmpA1=(_tmpA0.PlaceL).val;_LL7: {struct Cyc_CfFlowInfo_Place*p=_tmpA1;
# 714
struct Cyc_Dict_Dict outdict=Cyc_CfFlowInfo_assign_place(fenv,loc,indict,p,rval);
relns=Cyc_Relations_reln_assign_exp(Cyc_Core_heap_region,relns,lexp,rexp);
outflow=Cyc_CfFlowInfo_ReachableFL(outdict,relns);
if(Cyc_NewControlFlow_warn_lose_unique &&
 Cyc_Tcutil_is_noalias_pointer_or_aggr((void*)_check_null(lexp->topt))){
# 720
struct _tuple16 _stmttmp12=Cyc_CfFlowInfo_unname_rval(Cyc_CfFlowInfo_lookup_place(indict,p));void*_tmpA2;_LLB: _tmpA2=_stmttmp12.f1;_LLC: {void*rv=_tmpA2;
void*_tmpA3=rv;switch(*((int*)_tmpA3)){case 2U: if(((struct Cyc_CfFlowInfo_UnknownR_CfFlowInfo_AbsRVal_struct*)_tmpA3)->f1 == Cyc_CfFlowInfo_NoneIL){_LLE: _LLF:
 goto _LL11;}else{goto _LL14;}case 7U: _LL10: _LL11:
 goto _LL13;case 0U: _LL12: _LL13:
 goto _LLD;default: _LL14: _LL15:
# 726
({void*_tmpA4=0U;({unsigned _tmp518=lexp->loc;struct _fat_ptr _tmp517=({const char*_tmpA5="assignment may overwrite alias-free pointer(s)";_tag_fat(_tmpA5,sizeof(char),47U);});Cyc_Warn_warn(_tmp518,_tmp517,_tag_fat(_tmpA4,sizeof(void*),0U));});});
goto _LLD;}_LLD:;}}
# 731
Cyc_NewControlFlow_update_tryflow(env,outflow);
return({struct _tuple17 _tmp444;_tmp444.f1=outflow,_tmp444.f2=rval;_tmp444;});}}else{_LL8: _LL9:
# 737
 return({struct _tuple17 _tmp445;({union Cyc_CfFlowInfo_FlowInfo _tmp519=Cyc_NewControlFlow_use_Rval(env,rexp->loc,outflow,rval);_tmp445.f1=_tmp519;}),_tmp445.f2=rval;_tmp445;});}_LL5:;}}_LL0:;}
# 548
static union Cyc_CfFlowInfo_FlowInfo Cyc_NewControlFlow_do_initialize_var(struct Cyc_CfFlowInfo_FlowEnv*fenv,struct Cyc_NewControlFlow_AnalEnv*env,union Cyc_CfFlowInfo_FlowInfo f,struct Cyc_Absyn_Vardecl*vd,struct Cyc_Absyn_Exp*rexp,void*rval,unsigned loc){
# 750
union Cyc_CfFlowInfo_FlowInfo _tmpA6=f;struct Cyc_List_List*_tmpA8;struct Cyc_Dict_Dict _tmpA7;if((_tmpA6.BottomFL).tag == 1){_LL1: _LL2:
 return Cyc_CfFlowInfo_BottomFL();}else{_LL3: _tmpA7=((_tmpA6.ReachableFL).val).f1;_tmpA8=((_tmpA6.ReachableFL).val).f2;_LL4: {struct Cyc_Dict_Dict outdict=_tmpA7;struct Cyc_List_List*relns=_tmpA8;
# 755
outdict=({struct Cyc_CfFlowInfo_FlowEnv*_tmp51E=fenv;unsigned _tmp51D=loc;struct Cyc_Dict_Dict _tmp51C=outdict;struct Cyc_CfFlowInfo_Place*_tmp51B=({struct Cyc_CfFlowInfo_Place*_tmpAA=_cycalloc(sizeof(*_tmpAA));
({void*_tmp51A=(void*)({struct Cyc_CfFlowInfo_VarRoot_CfFlowInfo_Root_struct*_tmpA9=_cycalloc(sizeof(*_tmpA9));_tmpA9->tag=0U,_tmpA9->f1=vd;_tmpA9;});_tmpAA->root=_tmp51A;}),_tmpAA->path=0;_tmpAA;});
# 755
Cyc_CfFlowInfo_assign_place(_tmp51E,_tmp51D,_tmp51C,_tmp51B,rval);});
# 758
relns=Cyc_Relations_reln_assign_var(Cyc_Core_heap_region,relns,vd,rexp);{
union Cyc_CfFlowInfo_FlowInfo outflow=Cyc_CfFlowInfo_ReachableFL(outdict,relns);
Cyc_NewControlFlow_update_tryflow(env,outflow);
# 763
return outflow;}}}_LL0:;}struct _tuple25{struct Cyc_Absyn_Vardecl**f1;struct Cyc_Absyn_Exp*f2;};
# 548
static union Cyc_CfFlowInfo_FlowInfo Cyc_NewControlFlow_initialize_pat_vars(struct Cyc_CfFlowInfo_FlowEnv*fenv,struct Cyc_NewControlFlow_AnalEnv*env,union Cyc_CfFlowInfo_FlowInfo inflow,struct Cyc_List_List*vds,int name_locs,unsigned pat_loc,unsigned exp_loc){
# 774
if(vds == 0)return inflow;{struct Cyc_List_List*vs=
# 777
Cyc_Tcutil_filter_nulls((Cyc_List_split(vds)).f1);
struct Cyc_List_List*es=0;
{struct Cyc_List_List*x=vds;for(0;x != 0;x=x->tl){
if((*((struct _tuple25*)x->hd)).f1 == 0)es=({struct Cyc_List_List*_tmpAB=_cycalloc(sizeof(*_tmpAB));_tmpAB->hd=(struct Cyc_Absyn_Exp*)_check_null((*((struct _tuple25*)x->hd)).f2),_tmpAB->tl=es;_tmpAB;});}}
# 783
inflow=Cyc_NewControlFlow_add_vars(fenv,inflow,vs,fenv->unknown_all,pat_loc,name_locs);
# 785
inflow=Cyc_NewControlFlow_expand_unique_places(env,inflow,es);
{struct Cyc_List_List*x=es;for(0;x != 0;x=x->tl){
# 789
struct _tuple17 _stmttmp13=Cyc_NewControlFlow_anal_Rexp(env,1,inflow,(struct Cyc_Absyn_Exp*)x->hd);void*_tmpAD;union Cyc_CfFlowInfo_FlowInfo _tmpAC;_LL1: _tmpAC=_stmttmp13.f1;_tmpAD=_stmttmp13.f2;_LL2: {union Cyc_CfFlowInfo_FlowInfo f=_tmpAC;void*r=_tmpAD;
inflow=Cyc_NewControlFlow_use_Rval(env,exp_loc,f,r);}}}{
# 797
struct Cyc_List_List*_tmpAE=Cyc_List_rev(vds);struct Cyc_List_List*vds=_tmpAE;
for(0;vds != 0;vds=vds->tl){
struct _tuple25*_stmttmp14=(struct _tuple25*)vds->hd;struct Cyc_Absyn_Exp*_tmpB0;struct Cyc_Absyn_Vardecl**_tmpAF;_LL4: _tmpAF=_stmttmp14->f1;_tmpB0=_stmttmp14->f2;_LL5: {struct Cyc_Absyn_Vardecl**vd=_tmpAF;struct Cyc_Absyn_Exp*ve=_tmpB0;
if(vd != 0 && ve != 0){
if(ve->topt == 0)
({struct Cyc_String_pa_PrintArg_struct _tmpB4=({struct Cyc_String_pa_PrintArg_struct _tmp446;_tmp446.tag=0U,({
struct _fat_ptr _tmp51F=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_exp2string(ve));_tmp446.f1=_tmp51F;});_tmp446;});void*_tmpB1[1U];_tmpB1[0]=& _tmpB4;({int(*_tmp521)(struct _fat_ptr fmt,struct _fat_ptr ap)=({
# 802
int(*_tmpB3)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmpB3;});struct _fat_ptr _tmp520=({const char*_tmpB2="oops! pattern init expr %s has no type!\n";_tag_fat(_tmpB2,sizeof(char),41U);});_tmp521(_tmp520,_tag_fat(_tmpB1,sizeof(void*),1U));});});{
# 811
struct Cyc_List_List l=({struct Cyc_List_List _tmp448;_tmp448.hd=ve,_tmp448.tl=0;_tmp448;});
union Cyc_CfFlowInfo_FlowInfo f=Cyc_NewControlFlow_expand_unique_places(env,inflow,& l);
struct _tuple19 _stmttmp15=Cyc_NewControlFlow_anal_Lexp(env,f,0,0,ve);union Cyc_CfFlowInfo_AbsLVal _tmpB5;_LL7: _tmpB5=_stmttmp15.f2;_LL8: {union Cyc_CfFlowInfo_AbsLVal lval=_tmpB5;
struct _tuple17 _stmttmp16=Cyc_NewControlFlow_anal_Rexp(env,1,f,ve);void*_tmpB7;union Cyc_CfFlowInfo_FlowInfo _tmpB6;_LLA: _tmpB6=_stmttmp16.f1;_tmpB7=_stmttmp16.f2;_LLB: {union Cyc_CfFlowInfo_FlowInfo f=_tmpB6;void*rval=_tmpB7;
union Cyc_CfFlowInfo_FlowInfo _tmpB8=f;struct Cyc_List_List*_tmpBA;struct Cyc_Dict_Dict _tmpB9;if((_tmpB8.ReachableFL).tag == 2){_LLD: _tmpB9=((_tmpB8.ReachableFL).val).f1;_tmpBA=((_tmpB8.ReachableFL).val).f2;_LLE: {struct Cyc_Dict_Dict fd=_tmpB9;struct Cyc_List_List*relns=_tmpBA;
# 817
if(name_locs){
union Cyc_CfFlowInfo_AbsLVal _tmpBB=lval;struct Cyc_CfFlowInfo_Place*_tmpBC;if((_tmpBB.PlaceL).tag == 1){_LL12: _tmpBC=(_tmpBB.PlaceL).val;_LL13: {struct Cyc_CfFlowInfo_Place*p=_tmpBC;
# 820
rval=(void*)({struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*_tmpBD=_cycalloc(sizeof(*_tmpBD));_tmpBD->tag=8U,_tmpBD->f1=*vd,_tmpBD->f2=rval;_tmpBD;});{
# 823
void*new_rval=Cyc_CfFlowInfo_lookup_place(fd,p);
new_rval=(void*)({struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*_tmpBE=_cycalloc(sizeof(*_tmpBE));_tmpBE->tag=8U,_tmpBE->f1=*vd,_tmpBE->f2=new_rval;_tmpBE;});
fd=Cyc_CfFlowInfo_assign_place(fenv,exp_loc,fd,p,new_rval);
f=Cyc_CfFlowInfo_ReachableFL(fd,relns);
goto _LL11;}}}else{_LL14: _LL15:
# 830
 if(Cyc_Tcutil_is_noalias_pointer_or_aggr((void*)_check_null(ve->topt))&& !
Cyc_Tcutil_is_noalias_pointer_or_aggr((*vd)->type))
({struct Cyc_String_pa_PrintArg_struct _tmpC1=({struct Cyc_String_pa_PrintArg_struct _tmp447;_tmp447.tag=0U,({
struct _fat_ptr _tmp522=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_exp2string(ve));_tmp447.f1=_tmp522;});_tmp447;});void*_tmpBF[1U];_tmpBF[0]=& _tmpC1;({unsigned _tmp524=exp_loc;struct _fat_ptr _tmp523=({const char*_tmpC0="aliased pattern expression not an l-value: %s";_tag_fat(_tmpC0,sizeof(char),46U);});Cyc_CfFlowInfo_aerr(_tmp524,_tmp523,_tag_fat(_tmpBF,sizeof(void*),1U));});});}_LL11:;}
# 817
inflow=
# 840
Cyc_NewControlFlow_do_initialize_var(fenv,env,f,*vd,ve,rval,pat_loc);
goto _LLC;}}else{_LLF: _LL10:
# 843
 goto _LLC;}_LLC:;}}}}}}
# 848
return inflow;}}}
# 548
static int Cyc_NewControlFlow_is_local_var_rooted_path(struct Cyc_Absyn_Exp*e,int cast_ok){
# 853
void*_stmttmp17=e->r;void*_tmpC2=_stmttmp17;struct Cyc_Absyn_Exp*_tmpC3;struct Cyc_Absyn_Exp*_tmpC4;struct Cyc_Absyn_Exp*_tmpC5;struct Cyc_Absyn_Exp*_tmpC6;struct Cyc_Absyn_Exp*_tmpC7;switch(*((int*)_tmpC2)){case 1U: switch(*((int*)((struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct*)_tmpC2)->f1)){case 4U: _LL1: _LL2:
 goto _LL4;case 3U: _LL3: _LL4:
 goto _LL6;case 5U: _LL5: _LL6:
 return 1;default: goto _LL11;}case 20U: _LL7: _tmpC7=((struct Cyc_Absyn_Deref_e_Absyn_Raw_exp_struct*)_tmpC2)->f1;_LL8: {struct Cyc_Absyn_Exp*e=_tmpC7;
_tmpC6=e;goto _LLA;}case 21U: _LL9: _tmpC6=((struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct*)_tmpC2)->f1;_LLA: {struct Cyc_Absyn_Exp*e=_tmpC6;
_tmpC5=e;goto _LLC;}case 22U: _LLB: _tmpC5=((struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct*)_tmpC2)->f1;_LLC: {struct Cyc_Absyn_Exp*e=_tmpC5;
# 860
return Cyc_NewControlFlow_is_local_var_rooted_path(e,cast_ok);}case 23U: _LLD: _tmpC4=((struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct*)_tmpC2)->f1;_LLE: {struct Cyc_Absyn_Exp*e=_tmpC4;
# 862
void*_stmttmp18=Cyc_Tcutil_compress((void*)_check_null(e->topt));void*_tmpC8=_stmttmp18;if(((struct Cyc_Absyn_TupleType_Absyn_Type_struct*)_tmpC8)->tag == 6U){_LL14: _LL15:
 return Cyc_NewControlFlow_is_local_var_rooted_path(e,cast_ok);}else{_LL16: _LL17:
 return 0;}_LL13:;}case 14U: _LLF: _tmpC3=((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmpC2)->f2;_LL10: {struct Cyc_Absyn_Exp*e=_tmpC3;
# 867
if(cast_ok)return Cyc_NewControlFlow_is_local_var_rooted_path(e,cast_ok);else{
return 0;}}default: _LL11: _LL12:
 return 0;}_LL0:;}
# 548
static int Cyc_NewControlFlow_subst_param(struct Cyc_List_List*exps,union Cyc_Relations_RelnOp*rop){
# 874
union Cyc_Relations_RelnOp _stmttmp19=*rop;union Cyc_Relations_RelnOp _tmpC9=_stmttmp19;unsigned _tmpCA;unsigned _tmpCB;switch((_tmpC9.RParamNumelts).tag){case 5U: _LL1: _tmpCB=(_tmpC9.RParam).val;_LL2: {unsigned i=_tmpCB;
# 876
struct Cyc_Absyn_Exp*e=({struct Cyc_Absyn_Exp*(*_tmp526)(struct Cyc_List_List*x,int n)=({struct Cyc_Absyn_Exp*(*_tmpCC)(struct Cyc_List_List*x,int n)=(struct Cyc_Absyn_Exp*(*)(struct Cyc_List_List*x,int n))Cyc_List_nth;_tmpCC;});struct Cyc_List_List*_tmp525=exps;_tmp526(_tmp525,(int)i);});
return Cyc_Relations_exp2relnop(e,rop);}case 6U: _LL3: _tmpCA=(_tmpC9.RParamNumelts).val;_LL4: {unsigned i=_tmpCA;
# 879
struct Cyc_Absyn_Exp*e=({struct Cyc_Absyn_Exp*(*_tmp528)(struct Cyc_List_List*x,int n)=({struct Cyc_Absyn_Exp*(*_tmpCD)(struct Cyc_List_List*x,int n)=(struct Cyc_Absyn_Exp*(*)(struct Cyc_List_List*x,int n))Cyc_List_nth;_tmpCD;});struct Cyc_List_List*_tmp527=exps;_tmp528(_tmp527,(int)i);});
return({struct Cyc_Absyn_Exp*_tmp529=Cyc_Absyn_prim1_exp(Cyc_Absyn_Numelts,e,0U);Cyc_Relations_exp2relnop(_tmp529,rop);});}default: _LL5: _LL6:
 return 1;}_LL0:;}
# 548
static struct _fat_ptr Cyc_NewControlFlow_subst_param_string(struct Cyc_List_List*exps,union Cyc_Relations_RelnOp rop){
# 886
union Cyc_Relations_RelnOp _tmpCE=rop;unsigned _tmpCF;unsigned _tmpD0;switch((_tmpCE.RParamNumelts).tag){case 5U: _LL1: _tmpD0=(_tmpCE.RParam).val;_LL2: {unsigned i=_tmpD0;
# 888
return Cyc_Absynpp_exp2string(({struct Cyc_Absyn_Exp*(*_tmp52B)(struct Cyc_List_List*x,int n)=({struct Cyc_Absyn_Exp*(*_tmpD1)(struct Cyc_List_List*x,int n)=(struct Cyc_Absyn_Exp*(*)(struct Cyc_List_List*x,int n))Cyc_List_nth;_tmpD1;});struct Cyc_List_List*_tmp52A=exps;_tmp52B(_tmp52A,(int)i);}));}case 6U: _LL3: _tmpCF=(_tmpCE.RParamNumelts).val;_LL4: {unsigned i=_tmpCF;
# 890
return Cyc_Absynpp_exp2string(({struct Cyc_Absyn_Exp*(*_tmp52D)(struct Cyc_List_List*x,int n)=({struct Cyc_Absyn_Exp*(*_tmpD2)(struct Cyc_List_List*x,int n)=(struct Cyc_Absyn_Exp*(*)(struct Cyc_List_List*x,int n))Cyc_List_nth;_tmpD2;});struct Cyc_List_List*_tmp52C=exps;_tmp52D(_tmp52C,(int)i);}));}default: _LL5: _LL6:
 return Cyc_Relations_rop2string(rop);}_LL0:;}
# 548
static void Cyc_NewControlFlow_check_fn_requires(struct Cyc_NewControlFlow_AnalEnv*env,union Cyc_CfFlowInfo_FlowInfo inflow,struct Cyc_List_List*exps,struct Cyc_List_List*req,unsigned loc){
# 898
union Cyc_CfFlowInfo_FlowInfo _tmpD3=inflow;struct Cyc_List_List*_tmpD5;struct Cyc_Dict_Dict _tmpD4;if((_tmpD3.BottomFL).tag == 1){_LL1: _LL2:
 return;}else{_LL3: _tmpD4=((_tmpD3.ReachableFL).val).f1;_tmpD5=((_tmpD3.ReachableFL).val).f2;_LL4: {struct Cyc_Dict_Dict fd=_tmpD4;struct Cyc_List_List*relns=_tmpD5;
# 901
for(0;req != 0;req=req->tl){
struct Cyc_Relations_Reln*reln=(struct Cyc_Relations_Reln*)req->hd;
union Cyc_Relations_RelnOp rop1=reln->rop1;
union Cyc_Relations_RelnOp rop2=reln->rop2;
enum Cyc_Relations_Relation r=Cyc_Relations_flip_relation(reln->relation);
if((!Cyc_NewControlFlow_subst_param(exps,& rop1)|| !Cyc_NewControlFlow_subst_param(exps,& rop2))||
 Cyc_Relations_consistent_relations(Cyc_Relations_add_relation(Cyc_Core_heap_region,rop2,r,rop1,relns))){
struct _fat_ptr s1=Cyc_NewControlFlow_subst_param_string(exps,rop1);
struct _fat_ptr s2=Cyc_Relations_relation2string(reln->relation);
struct _fat_ptr s3=Cyc_NewControlFlow_subst_param_string(exps,rop2);
({struct Cyc_String_pa_PrintArg_struct _tmpD8=({struct Cyc_String_pa_PrintArg_struct _tmp44C;_tmp44C.tag=0U,_tmp44C.f1=(struct _fat_ptr)((struct _fat_ptr)s1);_tmp44C;});struct Cyc_String_pa_PrintArg_struct _tmpD9=({struct Cyc_String_pa_PrintArg_struct _tmp44B;_tmp44B.tag=0U,_tmp44B.f1=(struct _fat_ptr)((struct _fat_ptr)s2);_tmp44B;});struct Cyc_String_pa_PrintArg_struct _tmpDA=({struct Cyc_String_pa_PrintArg_struct _tmp44A;_tmp44A.tag=0U,_tmp44A.f1=(struct _fat_ptr)((struct _fat_ptr)s3);_tmp44A;});struct Cyc_String_pa_PrintArg_struct _tmpDB=({struct Cyc_String_pa_PrintArg_struct _tmp449;_tmp449.tag=0U,({
struct _fat_ptr _tmp52E=(struct _fat_ptr)((struct _fat_ptr)Cyc_Relations_relns2string(relns));_tmp449.f1=_tmp52E;});_tmp449;});void*_tmpD6[4U];_tmpD6[0]=& _tmpD8,_tmpD6[1]=& _tmpD9,_tmpD6[2]=& _tmpDA,_tmpD6[3]=& _tmpDB;({unsigned _tmp530=loc;struct _fat_ptr _tmp52F=({const char*_tmpD7="cannot prove that @requires clause %s %s %s is satisfied\n (all I know is %s)";_tag_fat(_tmpD7,sizeof(char),77U);});Cyc_CfFlowInfo_aerr(_tmp530,_tmp52F,_tag_fat(_tmpD6,sizeof(void*),4U));});});
break;}}
# 916
goto _LL0;}}_LL0:;}struct _tuple26{union Cyc_CfFlowInfo_AbsLVal f1;union Cyc_CfFlowInfo_FlowInfo f2;};struct _tuple27{struct Cyc_List_List*f1;struct Cyc_Absyn_Exp*f2;};
# 548
static struct _tuple17 Cyc_NewControlFlow_anal_Rexp(struct Cyc_NewControlFlow_AnalEnv*env,int copy_ctxt,union Cyc_CfFlowInfo_FlowInfo inflow,struct Cyc_Absyn_Exp*e){
# 925
struct Cyc_CfFlowInfo_FlowEnv*fenv=env->fenv;
struct Cyc_Dict_Dict d;
struct Cyc_List_List*relns;
# 938
{union Cyc_CfFlowInfo_FlowInfo _tmpDC=inflow;struct Cyc_List_List*_tmpDE;struct Cyc_Dict_Dict _tmpDD;if((_tmpDC.BottomFL).tag == 1){_LL1: _LL2:
 return({struct _tuple17 _tmp44D;({union Cyc_CfFlowInfo_FlowInfo _tmp531=Cyc_CfFlowInfo_BottomFL();_tmp44D.f1=_tmp531;}),_tmp44D.f2=fenv->unknown_all;_tmp44D;});}else{_LL3: _tmpDD=((_tmpDC.ReachableFL).val).f1;_tmpDE=((_tmpDC.ReachableFL).val).f2;_LL4: {struct Cyc_Dict_Dict d2=_tmpDD;struct Cyc_List_List*relns2=_tmpDE;
d=d2;relns=relns2;}}_LL0:;}
# 953 "new_control_flow.cyc"
if((copy_ctxt && Cyc_NewControlFlow_is_local_var_rooted_path(e,0))&&
 Cyc_Tcutil_is_noalias_pointer_or_aggr((void*)_check_null(e->topt))){
# 974 "new_control_flow.cyc"
struct _tuple19 _stmttmp1A=Cyc_NewControlFlow_anal_Lexp(env,inflow,1,0,e);union Cyc_CfFlowInfo_AbsLVal _tmpE0;union Cyc_CfFlowInfo_FlowInfo _tmpDF;_LL6: _tmpDF=_stmttmp1A.f1;_tmpE0=_stmttmp1A.f2;_LL7: {union Cyc_CfFlowInfo_FlowInfo flval=_tmpDF;union Cyc_CfFlowInfo_AbsLVal lval=_tmpE0;
struct _tuple19 _stmttmp1B=({struct _tuple19 _tmp451;_tmp451.f1=flval,_tmp451.f2=lval;_tmp451;});struct _tuple19 _tmpE1=_stmttmp1B;struct Cyc_CfFlowInfo_Place*_tmpE4;struct Cyc_List_List*_tmpE3;struct Cyc_Dict_Dict _tmpE2;if(((_tmpE1.f1).ReachableFL).tag == 2){if(((_tmpE1.f2).PlaceL).tag == 1){_LL9: _tmpE2=(((_tmpE1.f1).ReachableFL).val).f1;_tmpE3=(((_tmpE1.f1).ReachableFL).val).f2;_tmpE4=((_tmpE1.f2).PlaceL).val;_LLA: {struct Cyc_Dict_Dict fd=_tmpE2;struct Cyc_List_List*r=_tmpE3;struct Cyc_CfFlowInfo_Place*p=_tmpE4;
# 977
void*old_rval=Cyc_CfFlowInfo_lookup_place(fd,p);
int needs_unconsume=0;
if(Cyc_CfFlowInfo_is_unique_consumed(e,env->iteration_num,old_rval,& needs_unconsume)){
({void*_tmpE5=0U;({unsigned _tmp533=e->loc;struct _fat_ptr _tmp532=({const char*_tmpE6="expression attempts to copy a consumed alias-free value";_tag_fat(_tmpE6,sizeof(char),56U);});Cyc_CfFlowInfo_aerr(_tmp533,_tmp532,_tag_fat(_tmpE5,sizeof(void*),0U));});});
return({struct _tuple17 _tmp44E;({union Cyc_CfFlowInfo_FlowInfo _tmp534=Cyc_CfFlowInfo_BottomFL();_tmp44E.f1=_tmp534;}),_tmp44E.f2=fenv->unknown_all;_tmp44E;});}else{
# 983
if(needs_unconsume)
# 985
return({struct _tuple17 _tmp44F;_tmp44F.f1=flval,({void*_tmp535=Cyc_CfFlowInfo_make_unique_unconsumed(fenv,old_rval);_tmp44F.f2=_tmp535;});_tmp44F;});else{
# 988
void*new_rval=Cyc_CfFlowInfo_make_unique_consumed(fenv,(void*)_check_null(e->topt),e,env->iteration_num,old_rval);
struct Cyc_Dict_Dict outdict=Cyc_CfFlowInfo_assign_place(fenv,e->loc,fd,p,new_rval);
# 1000
return({struct _tuple17 _tmp450;({union Cyc_CfFlowInfo_FlowInfo _tmp536=Cyc_CfFlowInfo_ReachableFL(outdict,r);_tmp450.f1=_tmp536;}),_tmp450.f2=old_rval;_tmp450;});}}}}else{goto _LLB;}}else{_LLB: _LLC:
# 1004
 goto _LL8;}_LL8:;}}{
# 953 "new_control_flow.cyc"
void*_stmttmp1C=e->r;void*_tmpE7=_stmttmp1C;struct Cyc_Absyn_Stmt*_tmpE8;int _tmpEB;void*_tmpEA;struct Cyc_Absyn_Exp*_tmpE9;int _tmpEF;struct Cyc_Absyn_Exp*_tmpEE;struct Cyc_Absyn_Exp*_tmpED;struct Cyc_Absyn_Vardecl*_tmpEC;struct Cyc_List_List*_tmpF0;struct Cyc_List_List*_tmpF3;enum Cyc_Absyn_AggrKind _tmpF2;struct Cyc_List_List*_tmpF1;struct Cyc_List_List*_tmpF4;struct Cyc_List_List*_tmpF5;struct Cyc_Absyn_Exp*_tmpF7;struct Cyc_Absyn_Exp*_tmpF6;struct Cyc_Absyn_Exp*_tmpF9;struct Cyc_Absyn_Exp*_tmpF8;struct Cyc_Absyn_Exp*_tmpFB;struct Cyc_Absyn_Exp*_tmpFA;struct Cyc_Absyn_Exp*_tmpFE;struct Cyc_Absyn_Exp*_tmpFD;struct Cyc_Absyn_Exp*_tmpFC;struct _fat_ptr*_tmp100;struct Cyc_Absyn_Exp*_tmpFF;struct _fat_ptr*_tmp102;struct Cyc_Absyn_Exp*_tmp101;struct _fat_ptr*_tmp104;struct Cyc_Absyn_Exp*_tmp103;struct Cyc_Absyn_Exp*_tmp105;struct Cyc_Absyn_Exp*_tmp106;struct Cyc_Absyn_Exp*_tmp108;struct Cyc_Absyn_Exp*_tmp107;struct Cyc_Absyn_Exp*_tmp10A;struct Cyc_Absyn_Exp*_tmp109;int _tmp10F;struct Cyc_Absyn_Exp*_tmp10E;void**_tmp10D;struct Cyc_Absyn_Exp*_tmp10C;int _tmp10B;struct Cyc_List_List*_tmp111;struct Cyc_Absyn_Exp*_tmp110;struct Cyc_Absyn_Exp*_tmp112;struct Cyc_Absyn_Exp*_tmp114;struct Cyc_Absyn_Exp*_tmp113;struct Cyc_Absyn_Exp*_tmp116;struct Cyc_Absyn_Exp*_tmp115;struct Cyc_Absyn_Exp*_tmp118;struct Cyc_Absyn_Exp*_tmp117;struct Cyc_Absyn_Exp*_tmp11A;struct Cyc_Absyn_Exp*_tmp119;struct Cyc_Absyn_Exp*_tmp11B;struct Cyc_List_List*_tmp11D;enum Cyc_Absyn_Primop _tmp11C;struct Cyc_Absyn_Datatypedecl*_tmp11F;struct Cyc_List_List*_tmp11E;struct Cyc_Absyn_Vardecl*_tmp120;struct Cyc_Absyn_Vardecl*_tmp121;struct Cyc_Absyn_Vardecl*_tmp122;struct _fat_ptr _tmp123;struct Cyc_Absyn_Exp*_tmp124;struct Cyc_Absyn_Exp*_tmp125;struct Cyc_Absyn_Exp*_tmp126;struct Cyc_Absyn_Exp*_tmp127;struct Cyc_Absyn_Exp*_tmp128;switch(*((int*)_tmpE7)){case 14U: if(((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmpE7)->f4 == Cyc_Absyn_Null_to_NonNull){_LLE: _tmp128=((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmpE7)->f2;_LLF: {struct Cyc_Absyn_Exp*e1=_tmp128;
# 1011 "new_control_flow.cyc"
struct _tuple17 _stmttmp1D=Cyc_NewControlFlow_anal_Rexp(env,copy_ctxt,inflow,e1);void*_tmp12A;union Cyc_CfFlowInfo_FlowInfo _tmp129;_LL81: _tmp129=_stmttmp1D.f1;_tmp12A=_stmttmp1D.f2;_LL82: {union Cyc_CfFlowInfo_FlowInfo f1=_tmp129;void*r1=_tmp12A;
struct _tuple17 _stmttmp1E=Cyc_NewControlFlow_anal_derefR(env,inflow,f1,e1,r1,0);void*_tmp12C;union Cyc_CfFlowInfo_FlowInfo _tmp12B;_LL84: _tmp12B=_stmttmp1E.f1;_tmp12C=_stmttmp1E.f2;_LL85: {union Cyc_CfFlowInfo_FlowInfo f2=_tmp12B;void*r2=_tmp12C;
return({struct _tuple17 _tmp452;_tmp452.f1=f2,_tmp452.f2=r1;_tmp452;});}}}}else{_LL10: _tmp127=((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmpE7)->f2;_LL11: {struct Cyc_Absyn_Exp*e1=_tmp127;
# 1017
_tmp126=e1;goto _LL13;}}case 12U: _LL12: _tmp126=((struct Cyc_Absyn_NoInstantiate_e_Absyn_Raw_exp_struct*)_tmpE7)->f1;_LL13: {struct Cyc_Absyn_Exp*e1=_tmp126;
_tmp125=e1;goto _LL15;}case 42U: _LL14: _tmp125=((struct Cyc_Absyn_Extension_e_Absyn_Raw_exp_struct*)_tmpE7)->f1;_LL15: {struct Cyc_Absyn_Exp*e1=_tmp125;
_tmp124=e1;goto _LL17;}case 13U: _LL16: _tmp124=((struct Cyc_Absyn_Instantiate_e_Absyn_Raw_exp_struct*)_tmpE7)->f1;_LL17: {struct Cyc_Absyn_Exp*e1=_tmp124;
return Cyc_NewControlFlow_anal_Rexp(env,copy_ctxt,inflow,e1);}case 2U: _LL18: _tmp123=((struct Cyc_Absyn_Pragma_e_Absyn_Raw_exp_struct*)_tmpE7)->f1;_LL19: {struct _fat_ptr p=_tmp123;
# 1023
if(!({struct _fat_ptr _tmp537=(struct _fat_ptr)p;Cyc_NewControlFlow_strcmp(_tmp537,({const char*_tmp12D="print_flow";_tag_fat(_tmp12D,sizeof(char),11U);}));})){
struct _fat_ptr seg_str=Cyc_Position_string_of_segment(e->loc);
({struct Cyc_String_pa_PrintArg_struct _tmp130=({struct Cyc_String_pa_PrintArg_struct _tmp453;_tmp453.tag=0U,_tmp453.f1=(struct _fat_ptr)((struct _fat_ptr)seg_str);_tmp453;});void*_tmp12E[1U];_tmp12E[0]=& _tmp130;({struct Cyc___cycFILE*_tmp539=Cyc_stderr;struct _fat_ptr _tmp538=({const char*_tmp12F="%s: current flow is\n";_tag_fat(_tmp12F,sizeof(char),21U);});Cyc_fprintf(_tmp539,_tmp538,_tag_fat(_tmp12E,sizeof(void*),1U));});});
Cyc_CfFlowInfo_print_flow(inflow);
({void*_tmp131=0U;({struct Cyc___cycFILE*_tmp53B=Cyc_stderr;struct _fat_ptr _tmp53A=({const char*_tmp132="\n";_tag_fat(_tmp132,sizeof(char),2U);});Cyc_fprintf(_tmp53B,_tmp53A,_tag_fat(_tmp131,sizeof(void*),0U));});});}else{
# 1029
if(!({struct _fat_ptr _tmp53C=(struct _fat_ptr)p;Cyc_NewControlFlow_strcmp(_tmp53C,({const char*_tmp133="debug_on";_tag_fat(_tmp133,sizeof(char),9U);}));}));else{
# 1034
if(!({struct _fat_ptr _tmp53D=(struct _fat_ptr)p;Cyc_NewControlFlow_strcmp(_tmp53D,({const char*_tmp134="debug_off";_tag_fat(_tmp134,sizeof(char),10U);}));}));else{
# 1040
({struct Cyc_String_pa_PrintArg_struct _tmp137=({struct Cyc_String_pa_PrintArg_struct _tmp454;_tmp454.tag=0U,_tmp454.f1=(struct _fat_ptr)((struct _fat_ptr)p);_tmp454;});void*_tmp135[1U];_tmp135[0]=& _tmp137;({unsigned _tmp53F=e->loc;struct _fat_ptr _tmp53E=({const char*_tmp136="unknown pragma %s";_tag_fat(_tmp136,sizeof(char),18U);});Cyc_Warn_warn(_tmp53F,_tmp53E,_tag_fat(_tmp135,sizeof(void*),1U));});});}}}
return({struct _tuple17 _tmp455;_tmp455.f1=inflow,_tmp455.f2=fenv->zero;_tmp455;});}case 41U: _LL1A: _LL1B:
# 1043
 goto _LL1D;case 0U: switch(((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmpE7)->f1).Wstring_c).tag){case 1U: _LL1C: _LL1D:
 goto _LL1F;case 5U: if((((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmpE7)->f1).Int_c).val).f2 == 0){_LL1E: _LL1F:
 return({struct _tuple17 _tmp456;_tmp456.f1=inflow,_tmp456.f2=fenv->zero;_tmp456;});}else{_LL20: _LL21:
 goto _LL23;}case 8U: _LL22: _LL23:
 goto _LL25;case 9U: _LL24: _LL25:
 goto _LL27;default: _LL2A: _LL2B:
# 1052
 goto _LL2D;}case 1U: switch(*((int*)((struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct*)_tmpE7)->f1)){case 2U: _LL26: _LL27:
# 1049
 return({struct _tuple17 _tmp457;_tmp457.f1=inflow,_tmp457.f2=fenv->notzeroall;_tmp457;});case 1U: _LL36: _LL37:
# 1060
 return({struct _tuple17 _tmp458;_tmp458.f1=inflow,({void*_tmp540=Cyc_CfFlowInfo_typ_to_absrval(fenv,(void*)_check_null(e->topt),0,fenv->unknown_all);_tmp458.f2=_tmp540;});_tmp458;});case 4U: _LL38: _tmp122=((struct Cyc_Absyn_Local_b_Absyn_Binding_struct*)((struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct*)_tmpE7)->f1)->f1;_LL39: {struct Cyc_Absyn_Vardecl*vd=_tmp122;
# 1065
if((int)vd->sc == (int)Cyc_Absyn_Static)
return({struct _tuple17 _tmp459;_tmp459.f1=inflow,({void*_tmp541=Cyc_CfFlowInfo_typ_to_absrval(fenv,(void*)_check_null(e->topt),0,fenv->unknown_all);_tmp459.f2=_tmp541;});_tmp459;});
# 1065
_tmp121=vd;goto _LL3B;}case 3U: _LL3A: _tmp121=((struct Cyc_Absyn_Param_b_Absyn_Binding_struct*)((struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct*)_tmpE7)->f1)->f1;_LL3B: {struct Cyc_Absyn_Vardecl*vd=_tmp121;
# 1069
_tmp120=vd;goto _LL3D;}case 5U: _LL3C: _tmp120=((struct Cyc_Absyn_Pat_b_Absyn_Binding_struct*)((struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct*)_tmpE7)->f1)->f1;_LL3D: {struct Cyc_Absyn_Vardecl*vd=_tmp120;
# 1072
struct Cyc_CfFlowInfo_VarRoot_CfFlowInfo_Root_struct vdroot=({struct Cyc_CfFlowInfo_VarRoot_CfFlowInfo_Root_struct _tmp45B;_tmp45B.tag=0U,_tmp45B.f1=vd;_tmp45B;});
return({struct _tuple17 _tmp45A;_tmp45A.f1=inflow,({void*_tmp543=({struct Cyc_Dict_Dict _tmp542=d;Cyc_Dict_lookup_other(_tmp542,Cyc_CfFlowInfo_root_cmp,(void*)({struct Cyc_CfFlowInfo_VarRoot_CfFlowInfo_Root_struct*_tmp138=_cycalloc(sizeof(*_tmp138));*_tmp138=vdroot;_tmp138;}));});_tmp45A.f2=_tmp543;});_tmp45A;});}default: _LL78: _LL79:
# 1681
 goto _LL7B;}case 31U: if(((struct Cyc_Absyn_Datatype_e_Absyn_Raw_exp_struct*)_tmpE7)->f1 == 0){_LL28: _LL29:
# 1051
 goto _LL2B;}else{_LL66: _tmp11E=((struct Cyc_Absyn_Datatype_e_Absyn_Raw_exp_struct*)_tmpE7)->f1;_tmp11F=((struct Cyc_Absyn_Datatype_e_Absyn_Raw_exp_struct*)_tmpE7)->f2;_LL67: {struct Cyc_List_List*es=_tmp11E;struct Cyc_Absyn_Datatypedecl*tud=_tmp11F;
# 1543
_tmpF5=es;goto _LL69;}}case 18U: _LL2C: _LL2D:
# 1053
 goto _LL2F;case 17U: _LL2E: _LL2F:
 goto _LL31;case 19U: _LL30: _LL31:
 goto _LL33;case 33U: _LL32: _LL33:
 goto _LL35;case 32U: _LL34: _LL35:
 return({struct _tuple17 _tmp45C;_tmp45C.f1=inflow,_tmp45C.f2=fenv->unknown_all;_tmp45C;});case 3U: _LL3E: _tmp11C=((struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct*)_tmpE7)->f1;_tmp11D=((struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct*)_tmpE7)->f2;_LL3F: {enum Cyc_Absyn_Primop p=_tmp11C;struct Cyc_List_List*es=_tmp11D;
# 1079
struct _tuple17 _stmttmp1F=Cyc_NewControlFlow_anal_use_ints(env,inflow,es);void*_tmp13A;union Cyc_CfFlowInfo_FlowInfo _tmp139;_LL87: _tmp139=_stmttmp1F.f1;_tmp13A=_stmttmp1F.f2;_LL88: {union Cyc_CfFlowInfo_FlowInfo f=_tmp139;void*r=_tmp13A;
return({struct _tuple17 _tmp45D;_tmp45D.f1=f,_tmp45D.f2=r;_tmp45D;});}}case 5U: _LL40: _tmp11B=((struct Cyc_Absyn_Increment_e_Absyn_Raw_exp_struct*)_tmpE7)->f1;_LL41: {struct Cyc_Absyn_Exp*e1=_tmp11B;
# 1083
struct Cyc_List_List arg=({struct Cyc_List_List _tmp461;_tmp461.hd=e1,_tmp461.tl=0;_tmp461;});
struct _tuple17 _stmttmp20=Cyc_NewControlFlow_anal_use_ints(env,inflow,& arg);union Cyc_CfFlowInfo_FlowInfo _tmp13B;_LL8A: _tmp13B=_stmttmp20.f1;_LL8B: {union Cyc_CfFlowInfo_FlowInfo f=_tmp13B;
struct _tuple19 _stmttmp21=Cyc_NewControlFlow_anal_Lexp(env,f,0,0,e1);union Cyc_CfFlowInfo_AbsLVal _tmp13C;_LL8D: _tmp13C=_stmttmp21.f2;_LL8E: {union Cyc_CfFlowInfo_AbsLVal lval=_tmp13C;
int iszt=Cyc_Tcutil_is_zeroterm_pointer_type((void*)_check_null(e1->topt));
if(iszt){
# 1090
struct _tuple17 _stmttmp22=Cyc_NewControlFlow_anal_Rexp(env,0,inflow,e1);void*_tmp13E;union Cyc_CfFlowInfo_FlowInfo _tmp13D;_LL90: _tmp13D=_stmttmp22.f1;_tmp13E=_stmttmp22.f2;_LL91: {union Cyc_CfFlowInfo_FlowInfo g=_tmp13D;void*r=_tmp13E;
Cyc_NewControlFlow_anal_derefR(env,inflow,g,e1,r,0);}}
# 1087
{struct _tuple26 _stmttmp23=({struct _tuple26 _tmp45E;_tmp45E.f1=lval,_tmp45E.f2=f;_tmp45E;});struct _tuple26 _tmp13F=_stmttmp23;struct Cyc_List_List*_tmp142;struct Cyc_Dict_Dict _tmp141;struct Cyc_CfFlowInfo_Place*_tmp140;if(((_tmp13F.f1).PlaceL).tag == 1){if(((_tmp13F.f2).ReachableFL).tag == 2){_LL93: _tmp140=((_tmp13F.f1).PlaceL).val;_tmp141=(((_tmp13F.f2).ReachableFL).val).f1;_tmp142=(((_tmp13F.f2).ReachableFL).val).f2;_LL94: {struct Cyc_CfFlowInfo_Place*p=_tmp140;struct Cyc_Dict_Dict outdict=_tmp141;struct Cyc_List_List*relns=_tmp142;
# 1095
relns=Cyc_Relations_reln_kill_exp(Cyc_Core_heap_region,relns,e1);
f=({struct Cyc_Dict_Dict _tmp544=Cyc_CfFlowInfo_assign_place(fenv,e1->loc,outdict,p,fenv->unknown_all);Cyc_CfFlowInfo_ReachableFL(_tmp544,relns);});
# 1100
goto _LL92;}}else{goto _LL95;}}else{_LL95: _LL96:
 goto _LL92;}_LL92:;}
# 1104
if(iszt)
return({struct _tuple17 _tmp45F;_tmp45F.f1=f,_tmp45F.f2=fenv->notzeroall;_tmp45F;});else{
return({struct _tuple17 _tmp460;_tmp460.f1=f,_tmp460.f2=fenv->unknown_all;_tmp460;});}}}}case 4U: if(((struct Cyc_Absyn_AssignOp_e_Absyn_Raw_exp_struct*)_tmpE7)->f2 != 0){_LL42: _tmp119=((struct Cyc_Absyn_AssignOp_e_Absyn_Raw_exp_struct*)_tmpE7)->f1;_tmp11A=((struct Cyc_Absyn_AssignOp_e_Absyn_Raw_exp_struct*)_tmpE7)->f3;_LL43: {struct Cyc_Absyn_Exp*l=_tmp119;struct Cyc_Absyn_Exp*r=_tmp11A;
# 1109
if(copy_ctxt && Cyc_Tcutil_is_noalias_pointer_or_aggr((void*)_check_null(e->topt))){
({void*_tmp143=0U;({unsigned _tmp546=e->loc;struct _fat_ptr _tmp545=({const char*_tmp144="cannot track alias-free pointers through multiple assignments";_tag_fat(_tmp144,sizeof(char),62U);});Cyc_CfFlowInfo_aerr(_tmp546,_tmp545,_tag_fat(_tmp143,sizeof(void*),0U));});});
return({struct _tuple17 _tmp462;({union Cyc_CfFlowInfo_FlowInfo _tmp547=Cyc_CfFlowInfo_BottomFL();_tmp462.f1=_tmp547;}),_tmp462.f2=fenv->unknown_all;_tmp462;});}{
# 1109
struct _tuple17 _stmttmp24=
# 1113
Cyc_NewControlFlow_anal_Rexp(env,1,inflow,r);
# 1109
void*_tmp146;union Cyc_CfFlowInfo_FlowInfo _tmp145;_LL98: _tmp145=_stmttmp24.f1;_tmp146=_stmttmp24.f2;_LL99: {union Cyc_CfFlowInfo_FlowInfo right_out=_tmp145;void*rval=_tmp146;
# 1114
struct _tuple19 _stmttmp25=Cyc_NewControlFlow_anal_Lexp(env,right_out,0,0,l);union Cyc_CfFlowInfo_AbsLVal _tmp148;union Cyc_CfFlowInfo_FlowInfo _tmp147;_LL9B: _tmp147=_stmttmp25.f1;_tmp148=_stmttmp25.f2;_LL9C: {union Cyc_CfFlowInfo_FlowInfo f=_tmp147;union Cyc_CfFlowInfo_AbsLVal lval=_tmp148;
{union Cyc_CfFlowInfo_FlowInfo _tmp149=f;struct Cyc_List_List*_tmp14B;struct Cyc_Dict_Dict _tmp14A;if((_tmp149.ReachableFL).tag == 2){_LL9E: _tmp14A=((_tmp149.ReachableFL).val).f1;_tmp14B=((_tmp149.ReachableFL).val).f2;_LL9F: {struct Cyc_Dict_Dict outdict=_tmp14A;struct Cyc_List_List*relns=_tmp14B;
# 1117
{union Cyc_CfFlowInfo_AbsLVal _tmp14C=lval;struct Cyc_CfFlowInfo_Place*_tmp14D;if((_tmp14C.PlaceL).tag == 1){_LLA3: _tmp14D=(_tmp14C.PlaceL).val;_LLA4: {struct Cyc_CfFlowInfo_Place*p=_tmp14D;
# 1121
relns=Cyc_Relations_reln_kill_exp(Cyc_Core_heap_region,relns,l);
outdict=Cyc_CfFlowInfo_assign_place(fenv,l->loc,outdict,p,fenv->unknown_all);
# 1124
f=Cyc_CfFlowInfo_ReachableFL(outdict,relns);
# 1128
goto _LLA2;}}else{_LLA5: _LLA6:
# 1131
 goto _LLA2;}_LLA2:;}
# 1133
goto _LL9D;}}else{_LLA0: _LLA1:
 goto _LL9D;}_LL9D:;}
# 1136
return({struct _tuple17 _tmp463;_tmp463.f1=f,_tmp463.f2=fenv->unknown_all;_tmp463;});}}}}}else{_LL44: _tmp117=((struct Cyc_Absyn_AssignOp_e_Absyn_Raw_exp_struct*)_tmpE7)->f1;_tmp118=((struct Cyc_Absyn_AssignOp_e_Absyn_Raw_exp_struct*)_tmpE7)->f3;_LL45: {struct Cyc_Absyn_Exp*e1=_tmp117;struct Cyc_Absyn_Exp*e2=_tmp118;
# 1139
if(copy_ctxt && Cyc_Tcutil_is_noalias_pointer_or_aggr((void*)_check_null(e->topt))){
({void*_tmp14E=0U;({unsigned _tmp549=e->loc;struct _fat_ptr _tmp548=({const char*_tmp14F="cannot track alias-free pointers through multiple assignments";_tag_fat(_tmp14F,sizeof(char),62U);});Cyc_CfFlowInfo_aerr(_tmp549,_tmp548,_tag_fat(_tmp14E,sizeof(void*),0U));});});
return({struct _tuple17 _tmp464;({union Cyc_CfFlowInfo_FlowInfo _tmp54A=Cyc_CfFlowInfo_BottomFL();_tmp464.f1=_tmp54A;}),_tmp464.f2=fenv->unknown_all;_tmp464;});}
# 1145
inflow=({struct Cyc_NewControlFlow_AnalEnv*_tmp54C=env;union Cyc_CfFlowInfo_FlowInfo _tmp54B=inflow;Cyc_NewControlFlow_expand_unique_places(_tmp54C,_tmp54B,({struct Cyc_Absyn_Exp*_tmp150[2U];_tmp150[0]=e1,_tmp150[1]=e2;Cyc_List_list(_tag_fat(_tmp150,sizeof(struct Cyc_Absyn_Exp*),2U));}));});{
# 1147
struct _tuple17 _stmttmp26=Cyc_NewControlFlow_anal_Rexp(env,1,inflow,e2);void*_tmp152;union Cyc_CfFlowInfo_FlowInfo _tmp151;_LLA8: _tmp151=_stmttmp26.f1;_tmp152=_stmttmp26.f2;_LLA9: {union Cyc_CfFlowInfo_FlowInfo right_out=_tmp151;void*rval=_tmp152;
struct _tuple19 _stmttmp27=Cyc_NewControlFlow_anal_Lexp(env,right_out,0,0,e1);union Cyc_CfFlowInfo_AbsLVal _tmp154;union Cyc_CfFlowInfo_FlowInfo _tmp153;_LLAB: _tmp153=_stmttmp27.f1;_tmp154=_stmttmp27.f2;_LLAC: {union Cyc_CfFlowInfo_FlowInfo outflow=_tmp153;union Cyc_CfFlowInfo_AbsLVal lval=_tmp154;
struct _tuple17 _stmttmp28=Cyc_NewControlFlow_do_assign(fenv,env,outflow,e1,lval,e2,rval,e->loc);void*_tmp156;union Cyc_CfFlowInfo_FlowInfo _tmp155;_LLAE: _tmp155=_stmttmp28.f1;_tmp156=_stmttmp28.f2;_LLAF: {union Cyc_CfFlowInfo_FlowInfo outflow=_tmp155;void*rval=_tmp156;
# 1153
return({struct _tuple17 _tmp465;_tmp465.f1=outflow,_tmp465.f2=rval;_tmp465;});}}}}}}case 34U: _LL46: _tmp115=((struct Cyc_Absyn_Spawn_e_Absyn_Raw_exp_struct*)_tmpE7)->f1;_tmp116=((struct Cyc_Absyn_Spawn_e_Absyn_Raw_exp_struct*)_tmpE7)->f2;_LL47: {struct Cyc_Absyn_Exp*e1=_tmp115;struct Cyc_Absyn_Exp*e2=_tmp116;
# 1156
struct _tuple17 _stmttmp29=Cyc_NewControlFlow_anal_Rexp(env,0,inflow,e1);void*_tmp158;union Cyc_CfFlowInfo_FlowInfo _tmp157;_LLB1: _tmp157=_stmttmp29.f1;_tmp158=_stmttmp29.f2;_LLB2: {union Cyc_CfFlowInfo_FlowInfo f=_tmp157;void*r=_tmp158;
return Cyc_NewControlFlow_anal_Rexp(env,copy_ctxt,f,e2);}}case 9U: _LL48: _tmp113=((struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct*)_tmpE7)->f1;_tmp114=((struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct*)_tmpE7)->f2;_LL49: {struct Cyc_Absyn_Exp*e1=_tmp113;struct Cyc_Absyn_Exp*e2=_tmp114;
# 1160
struct _tuple17 _stmttmp2A=Cyc_NewControlFlow_anal_Rexp(env,0,inflow,e1);void*_tmp15A;union Cyc_CfFlowInfo_FlowInfo _tmp159;_LLB4: _tmp159=_stmttmp2A.f1;_tmp15A=_stmttmp2A.f2;_LLB5: {union Cyc_CfFlowInfo_FlowInfo f=_tmp159;void*r=_tmp15A;
return Cyc_NewControlFlow_anal_Rexp(env,copy_ctxt,f,e2);}}case 11U: _LL4A: _tmp112=((struct Cyc_Absyn_Throw_e_Absyn_Raw_exp_struct*)_tmpE7)->f1;_LL4B: {struct Cyc_Absyn_Exp*e1=_tmp112;
# 1164
struct _tuple17 _stmttmp2B=Cyc_NewControlFlow_anal_Rexp(env,1,inflow,e1);void*_tmp15C;union Cyc_CfFlowInfo_FlowInfo _tmp15B;_LLB7: _tmp15B=_stmttmp2B.f1;_tmp15C=_stmttmp2B.f2;_LLB8: {union Cyc_CfFlowInfo_FlowInfo f=_tmp15B;void*r=_tmp15C;
Cyc_NewControlFlow_use_Rval(env,e1->loc,f,r);
return({struct _tuple17 _tmp466;({union Cyc_CfFlowInfo_FlowInfo _tmp54E=Cyc_CfFlowInfo_BottomFL();_tmp466.f1=_tmp54E;}),({void*_tmp54D=Cyc_CfFlowInfo_typ_to_absrval(fenv,(void*)_check_null(e->topt),0,fenv->unknown_all);_tmp466.f2=_tmp54D;});_tmp466;});}}case 10U: _LL4C: _tmp110=((struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct*)_tmpE7)->f1;_tmp111=((struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct*)_tmpE7)->f2;_LL4D: {struct Cyc_Absyn_Exp*e1=_tmp110;struct Cyc_List_List*es=_tmp111;
# 1169
struct Cyc_List_List*orig_es=es;
struct _tuple17 _stmttmp2C=Cyc_NewControlFlow_anal_Rexp(env,0,inflow,e1);void*_tmp15E;union Cyc_CfFlowInfo_FlowInfo _tmp15D;_LLBA: _tmp15D=_stmttmp2C.f1;_tmp15E=_stmttmp2C.f2;_LLBB: {union Cyc_CfFlowInfo_FlowInfo f1=_tmp15D;void*r1=_tmp15E;
# 1172
Cyc_NewControlFlow_anal_derefR(env,inflow,f1,e1,r1,0);{
# 1175
struct _tuple24 _stmttmp2D=({struct Cyc_NewControlFlow_AnalEnv*_tmp550=env;union Cyc_CfFlowInfo_FlowInfo _tmp54F=f1;Cyc_NewControlFlow_anal_Rexps(_tmp550,_tmp54F,Cyc_List_copy(es),1,1);});struct Cyc_List_List*_tmp160;union Cyc_CfFlowInfo_FlowInfo _tmp15F;_LLBD: _tmp15F=_stmttmp2D.f1;_tmp160=_stmttmp2D.f2;_LLBE: {union Cyc_CfFlowInfo_FlowInfo fst_outflow=_tmp15F;struct Cyc_List_List*rvals=_tmp160;
# 1177
union Cyc_CfFlowInfo_FlowInfo outflow=Cyc_NewControlFlow_use_Rval(env,e1->loc,fst_outflow,r1);
# 1179
struct Cyc_List_List*init_params=0;
struct Cyc_List_List*nolive_unique_params=0;
struct Cyc_List_List*consume_params=0;
struct Cyc_List_List*requires;
struct Cyc_List_List*ensures;
{void*_stmttmp2E=Cyc_Tcutil_compress((void*)_check_null(e1->topt));void*_tmp161=_stmttmp2E;void*_tmp162;if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp161)->tag == 3U){_LLC0: _tmp162=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp161)->f1).elt_type;_LLC1: {void*t=_tmp162;
# 1186
{void*_stmttmp2F=Cyc_Tcutil_compress(t);void*_tmp163=_stmttmp2F;struct Cyc_List_List*_tmp166;struct Cyc_List_List*_tmp165;struct Cyc_List_List*_tmp164;if(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp163)->tag == 5U){_LLC5: _tmp164=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp163)->f1).attributes;_tmp165=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp163)->f1).requires_relns;_tmp166=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp163)->f1).ensures_relns;_LLC6: {struct Cyc_List_List*atts=_tmp164;struct Cyc_List_List*req=_tmp165;struct Cyc_List_List*ens=_tmp166;
# 1188
requires=req;
ensures=ens;
for(0;atts != 0;atts=atts->tl){
# 1192
void*_stmttmp30=(void*)atts->hd;void*_tmp167=_stmttmp30;int _tmp168;int _tmp169;int _tmp16A;switch(*((int*)_tmp167)){case 20U: _LLCA: _tmp16A=((struct Cyc_Absyn_Initializes_att_Absyn_Attribute_struct*)_tmp167)->f1;_LLCB: {int i=_tmp16A;
# 1194
init_params=({struct Cyc_List_List*_tmp16B=_cycalloc(sizeof(*_tmp16B));_tmp16B->hd=(void*)i,_tmp16B->tl=init_params;_tmp16B;});goto _LLC9;}case 21U: _LLCC: _tmp169=((struct Cyc_Absyn_Noliveunique_att_Absyn_Attribute_struct*)_tmp167)->f1;_LLCD: {int i=_tmp169;
# 1196
nolive_unique_params=({struct Cyc_List_List*_tmp16C=_cycalloc(sizeof(*_tmp16C));_tmp16C->hd=(void*)i,_tmp16C->tl=nolive_unique_params;_tmp16C;});
consume_params=({struct Cyc_List_List*_tmp16D=_cycalloc(sizeof(*_tmp16D));_tmp16D->hd=(void*)i,_tmp16D->tl=consume_params;_tmp16D;});
goto _LLC9;}case 22U: _LLCE: _tmp168=((struct Cyc_Absyn_Consume_att_Absyn_Attribute_struct*)_tmp167)->f1;_LLCF: {int i=_tmp168;
# 1201
consume_params=({struct Cyc_List_List*_tmp16E=_cycalloc(sizeof(*_tmp16E));_tmp16E->hd=(void*)i,_tmp16E->tl=consume_params;_tmp16E;});
goto _LLC9;}default: _LLD0: _LLD1:
 goto _LLC9;}_LLC9:;}
# 1205
goto _LLC4;}}else{_LLC7: _LLC8:
({void*_tmp16F=0U;({int(*_tmp552)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp171)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmp171;});struct _fat_ptr _tmp551=({const char*_tmp170="anal_Rexp: bad function type";_tag_fat(_tmp170,sizeof(char),29U);});_tmp552(_tmp551,_tag_fat(_tmp16F,sizeof(void*),0U));});});}_LLC4:;}
# 1208
goto _LLBF;}}else{_LLC2: _LLC3:
({void*_tmp172=0U;({int(*_tmp554)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp174)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmp174;});struct _fat_ptr _tmp553=({const char*_tmp173="anal_Rexp: bad function type";_tag_fat(_tmp173,sizeof(char),29U);});_tmp554(_tmp553,_tag_fat(_tmp172,sizeof(void*),0U));});});}_LLBF:;}
# 1211
{int i=1;for(0;rvals != 0;(rvals=rvals->tl,es=((struct Cyc_List_List*)_check_null(es))->tl,++ i)){
if(({int(*_tmp556)(struct Cyc_List_List*l,int x)=({int(*_tmp175)(struct Cyc_List_List*l,int x)=(int(*)(struct Cyc_List_List*l,int x))Cyc_List_memq;_tmp175;});struct Cyc_List_List*_tmp555=init_params;_tmp556(_tmp555,i);})){
union Cyc_CfFlowInfo_FlowInfo _tmp176=fst_outflow;struct Cyc_Dict_Dict _tmp177;if((_tmp176.BottomFL).tag == 1){_LLD3: _LLD4:
 goto _LLD2;}else{_LLD5: _tmp177=((_tmp176.ReachableFL).val).f1;_LLD6: {struct Cyc_Dict_Dict fst_d=_tmp177;
# 1216
struct _tuple16 _stmttmp31=Cyc_CfFlowInfo_unname_rval((void*)rvals->hd);void*_tmp178;_LLD8: _tmp178=_stmttmp31.f1;_LLD9: {void*r=_tmp178;
if((int)Cyc_CfFlowInfo_initlevel(env->fenv,fst_d,(void*)rvals->hd)== (int)Cyc_CfFlowInfo_NoneIL && !Cyc_CfFlowInfo_is_init_pointer((void*)rvals->hd))
({void*_tmp179=0U;({unsigned _tmp558=((struct Cyc_Absyn_Exp*)((struct Cyc_List_List*)_check_null(es))->hd)->loc;struct _fat_ptr _tmp557=({const char*_tmp17A="expression may not be initialized";_tag_fat(_tmp17A,sizeof(char),34U);});Cyc_CfFlowInfo_aerr(_tmp558,_tmp557,_tag_fat(_tmp179,sizeof(void*),0U));});});
# 1217
{union Cyc_CfFlowInfo_FlowInfo _tmp17B=outflow;struct Cyc_List_List*_tmp17D;struct Cyc_Dict_Dict _tmp17C;if((_tmp17B.BottomFL).tag == 1){_LLDB: _LLDC:
# 1220
 goto _LLDA;}else{_LLDD: _tmp17C=((_tmp17B.ReachableFL).val).f1;_tmp17D=((_tmp17B.ReachableFL).val).f2;_LLDE: {struct Cyc_Dict_Dict d=_tmp17C;struct Cyc_List_List*relns=_tmp17D;
# 1224
struct Cyc_Dict_Dict ans_d=Cyc_CfFlowInfo_escape_deref(fenv,d,(void*)rvals->hd);
{void*_stmttmp32=(void*)rvals->hd;void*_tmp17E=_stmttmp32;struct Cyc_CfFlowInfo_Place*_tmp17F;switch(*((int*)_tmp17E)){case 4U: _LLE0: _tmp17F=((struct Cyc_CfFlowInfo_AddressOf_CfFlowInfo_AbsRVal_struct*)_tmp17E)->f1;_LLE1: {struct Cyc_CfFlowInfo_Place*p=_tmp17F;
# 1227
{void*_stmttmp33=Cyc_Tcutil_compress((void*)_check_null(((struct Cyc_Absyn_Exp*)((struct Cyc_List_List*)_check_null(es))->hd)->topt));void*_tmp180=_stmttmp33;void*_tmp181;if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp180)->tag == 3U){_LLE7: _tmp181=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp180)->f1).elt_type;_LLE8: {void*t=_tmp181;
# 1229
ans_d=({struct Cyc_CfFlowInfo_FlowEnv*_tmp55C=fenv;unsigned _tmp55B=((struct Cyc_Absyn_Exp*)es->hd)->loc;struct Cyc_Dict_Dict _tmp55A=ans_d;struct Cyc_CfFlowInfo_Place*_tmp559=p;Cyc_CfFlowInfo_assign_place(_tmp55C,_tmp55B,_tmp55A,_tmp559,
Cyc_CfFlowInfo_typ_to_absrval(fenv,t,0,fenv->esc_all));});
# 1233
goto _LLE6;}}else{_LLE9: _LLEA:
({void*_tmp182=0U;({int(*_tmp55E)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp184)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmp184;});struct _fat_ptr _tmp55D=({const char*_tmp183="anal_Rexp:bad type for initialized arg";_tag_fat(_tmp183,sizeof(char),39U);});_tmp55E(_tmp55D,_tag_fat(_tmp182,sizeof(void*),0U));});});}_LLE6:;}
# 1236
goto _LLDF;}case 5U: _LLE2: _LLE3:
# 1238
({void*_tmp185=0U;({int(*_tmp560)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp187)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmp187;});struct _fat_ptr _tmp55F=({const char*_tmp186="anal_Rexp:initialize attribute on unique pointers not yet supported";_tag_fat(_tmp186,sizeof(char),68U);});_tmp560(_tmp55F,_tag_fat(_tmp185,sizeof(void*),0U));});});default: _LLE4: _LLE5:
# 1240
 goto _LLDF;}_LLDF:;}
# 1242
outflow=Cyc_CfFlowInfo_ReachableFL(ans_d,relns);
goto _LLDA;}}_LLDA:;}
# 1245
goto _LLD2;}}}_LLD2:;}else{
# 1248
if(({int(*_tmp562)(struct Cyc_List_List*l,int x)=({int(*_tmp188)(struct Cyc_List_List*l,int x)=(int(*)(struct Cyc_List_List*l,int x))Cyc_List_memq;_tmp188;});struct Cyc_List_List*_tmp561=nolive_unique_params;_tmp562(_tmp561,i);}))
# 1253
outflow=({struct Cyc_NewControlFlow_AnalEnv*_tmp566=env;unsigned _tmp565=((struct Cyc_Absyn_Exp*)((struct Cyc_List_List*)_check_null(es))->hd)->loc;void*_tmp564=(void*)_check_null(((struct Cyc_Absyn_Exp*)es->hd)->topt);union Cyc_CfFlowInfo_FlowInfo _tmp563=outflow;Cyc_NewControlFlow_use_nounique_Rval(_tmp566,_tmp565,_tmp564,_tmp563,(void*)rvals->hd);});else{
# 1263
outflow=Cyc_NewControlFlow_use_Rval(env,((struct Cyc_Absyn_Exp*)((struct Cyc_List_List*)_check_null(es))->hd)->loc,outflow,(void*)rvals->hd);
if(!({int(*_tmp568)(struct Cyc_List_List*l,int x)=({int(*_tmp189)(struct Cyc_List_List*l,int x)=(int(*)(struct Cyc_List_List*l,int x))Cyc_List_memq;_tmp189;});struct Cyc_List_List*_tmp567=consume_params;_tmp568(_tmp567,i);})&&
 Cyc_Tcutil_is_noalias_pointer((void*)_check_null(((struct Cyc_Absyn_Exp*)es->hd)->topt),0))
outflow=Cyc_NewControlFlow_restore_noconsume_arg(env,outflow,(struct Cyc_Absyn_Exp*)es->hd,((struct Cyc_Absyn_Exp*)es->hd)->loc,(void*)rvals->hd);}}}}
# 1270
Cyc_NewControlFlow_check_fn_requires(env,outflow,orig_es,requires,e->loc);
# 1273
if(Cyc_Tcutil_is_noreturn_fn_type((void*)_check_null(e1->topt)))
return({struct _tuple17 _tmp467;({union Cyc_CfFlowInfo_FlowInfo _tmp56A=Cyc_CfFlowInfo_BottomFL();_tmp467.f1=_tmp56A;}),({void*_tmp569=Cyc_CfFlowInfo_typ_to_absrval(fenv,(void*)_check_null(e->topt),0,fenv->unknown_all);_tmp467.f2=_tmp569;});_tmp467;});else{
# 1276
return({struct _tuple17 _tmp468;_tmp468.f1=outflow,({void*_tmp56B=Cyc_CfFlowInfo_typ_to_absrval(fenv,(void*)_check_null(e->topt),0,fenv->unknown_all);_tmp468.f2=_tmp56B;});_tmp468;});}}}}}case 35U: _LL4E: _tmp10B=(((struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct*)_tmpE7)->f1).is_calloc;_tmp10C=(((struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct*)_tmpE7)->f1).rgn;_tmp10D=(((struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct*)_tmpE7)->f1).elt_type;_tmp10E=(((struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct*)_tmpE7)->f1).num_elts;_tmp10F=(((struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct*)_tmpE7)->f1).fat_result;_LL4F: {int iscalloc=_tmp10B;struct Cyc_Absyn_Exp*eopt=_tmp10C;void**topt=_tmp10D;struct Cyc_Absyn_Exp*exp=_tmp10E;int isfat=_tmp10F;
# 1279
void*place_val;
if(isfat)place_val=fenv->notzeroall;else{
if(iscalloc)place_val=Cyc_CfFlowInfo_typ_to_absrval(fenv,*((void**)_check_null(topt)),0,fenv->zero);else{
place_val=Cyc_CfFlowInfo_typ_to_absrval(fenv,*((void**)_check_null(topt)),0,fenv->unknown_none);}}{
union Cyc_CfFlowInfo_FlowInfo outflow;
if(eopt != 0){
struct _tuple24 _stmttmp34=({struct Cyc_NewControlFlow_AnalEnv*_tmp56D=env;union Cyc_CfFlowInfo_FlowInfo _tmp56C=inflow;Cyc_NewControlFlow_anal_Rexps(_tmp56D,_tmp56C,({struct Cyc_Absyn_Exp*_tmp18C[2U];_tmp18C[0]=eopt,_tmp18C[1]=exp;Cyc_List_list(_tag_fat(_tmp18C,sizeof(struct Cyc_Absyn_Exp*),2U));}),1,1);});struct Cyc_List_List*_tmp18B;union Cyc_CfFlowInfo_FlowInfo _tmp18A;_LLEC: _tmp18A=_stmttmp34.f1;_tmp18B=_stmttmp34.f2;_LLED: {union Cyc_CfFlowInfo_FlowInfo f=_tmp18A;struct Cyc_List_List*rvals=_tmp18B;
outflow=f;}}else{
# 1288
struct _tuple17 _stmttmp35=Cyc_NewControlFlow_anal_Rexp(env,1,inflow,exp);union Cyc_CfFlowInfo_FlowInfo _tmp18D;_LLEF: _tmp18D=_stmttmp35.f1;_LLF0: {union Cyc_CfFlowInfo_FlowInfo f=_tmp18D;
outflow=f;}}
# 1293
if(Cyc_Tcutil_is_noalias_pointer((void*)_check_null(e->topt),1))
return({struct _tuple17 _tmp469;_tmp469.f1=outflow,({void*_tmp56E=(void*)({struct Cyc_CfFlowInfo_UniquePtr_CfFlowInfo_AbsRVal_struct*_tmp18E=_cycalloc(sizeof(*_tmp18E));_tmp18E->tag=5U,_tmp18E->f1=place_val;_tmp18E;});_tmp469.f2=_tmp56E;});_tmp469;});else{
# 1297
void*root=(void*)({struct Cyc_CfFlowInfo_MallocPt_CfFlowInfo_Root_struct*_tmp194=_cycalloc(sizeof(*_tmp194));_tmp194->tag=1U,_tmp194->f1=exp,_tmp194->f2=(void*)_check_null(e->topt);_tmp194;});
struct Cyc_CfFlowInfo_Place*place=({struct Cyc_CfFlowInfo_Place*_tmp193=_cycalloc(sizeof(*_tmp193));_tmp193->root=root,_tmp193->path=0;_tmp193;});
void*rval=(void*)({struct Cyc_CfFlowInfo_AddressOf_CfFlowInfo_AbsRVal_struct*_tmp192=_cycalloc(sizeof(*_tmp192));_tmp192->tag=4U,_tmp192->f1=place;_tmp192;});
union Cyc_CfFlowInfo_FlowInfo _tmp18F=outflow;struct Cyc_List_List*_tmp191;struct Cyc_Dict_Dict _tmp190;if((_tmp18F.BottomFL).tag == 1){_LLF2: _LLF3:
 return({struct _tuple17 _tmp46A;_tmp46A.f1=outflow,_tmp46A.f2=rval;_tmp46A;});}else{_LLF4: _tmp190=((_tmp18F.ReachableFL).val).f1;_tmp191=((_tmp18F.ReachableFL).val).f2;_LLF5: {struct Cyc_Dict_Dict d2=_tmp190;struct Cyc_List_List*relns=_tmp191;
# 1303
return({struct _tuple17 _tmp46B;({union Cyc_CfFlowInfo_FlowInfo _tmp570=({struct Cyc_Dict_Dict _tmp56F=Cyc_Dict_insert(d2,root,place_val);Cyc_CfFlowInfo_ReachableFL(_tmp56F,relns);});_tmp46B.f1=_tmp570;}),_tmp46B.f2=rval;_tmp46B;});}}_LLF1:;}}}case 36U: _LL50: _tmp109=((struct Cyc_Absyn_Swap_e_Absyn_Raw_exp_struct*)_tmpE7)->f1;_tmp10A=((struct Cyc_Absyn_Swap_e_Absyn_Raw_exp_struct*)_tmpE7)->f2;_LL51: {struct Cyc_Absyn_Exp*e1=_tmp109;struct Cyc_Absyn_Exp*e2=_tmp10A;
# 1308
void*left_rval;
void*right_rval;
union Cyc_CfFlowInfo_FlowInfo outflow;
# 1314
struct _tuple24 _stmttmp36=({struct Cyc_NewControlFlow_AnalEnv*_tmp572=env;union Cyc_CfFlowInfo_FlowInfo _tmp571=inflow;Cyc_NewControlFlow_anal_Rexps(_tmp572,_tmp571,({struct Cyc_Absyn_Exp*_tmp1AA[2U];_tmp1AA[0]=e1,_tmp1AA[1]=e2;Cyc_List_list(_tag_fat(_tmp1AA,sizeof(struct Cyc_Absyn_Exp*),2U));}),0,0);});struct Cyc_List_List*_tmp196;union Cyc_CfFlowInfo_FlowInfo _tmp195;_LLF7: _tmp195=_stmttmp36.f1;_tmp196=_stmttmp36.f2;_LLF8: {union Cyc_CfFlowInfo_FlowInfo f=_tmp195;struct Cyc_List_List*rvals=_tmp196;
left_rval=(void*)((struct Cyc_List_List*)_check_null(rvals))->hd;
right_rval=(void*)((struct Cyc_List_List*)_check_null(rvals->tl))->hd;
outflow=f;{
# 1320
void*t_ign1=Cyc_Absyn_void_type;void*t_ign2=Cyc_Absyn_void_type;
int b_ign1=0;
if(Cyc_Tcutil_is_zero_ptr_deref(e1,& t_ign1,& b_ign1,& t_ign2)){
void*_tmp197=right_rval;if(((struct Cyc_CfFlowInfo_Zero_CfFlowInfo_AbsRVal_struct*)_tmp197)->tag == 0U){_LLFA: _LLFB:
 goto _LLF9;}else{_LLFC: _LLFD:
({void*_tmp198=0U;({unsigned _tmp574=e1->loc;struct _fat_ptr _tmp573=({const char*_tmp199="cannot swap value into zeroterm array not known to be 0";_tag_fat(_tmp199,sizeof(char),56U);});Cyc_CfFlowInfo_aerr(_tmp574,_tmp573,_tag_fat(_tmp198,sizeof(void*),0U));});});}_LLF9:;}else{
# 1331
if(Cyc_Tcutil_is_zero_ptr_deref(e2,& t_ign1,& b_ign1,& t_ign2)){
void*_tmp19A=left_rval;if(((struct Cyc_CfFlowInfo_Zero_CfFlowInfo_AbsRVal_struct*)_tmp19A)->tag == 0U){_LLFF: _LL100:
 goto _LLFE;}else{_LL101: _LL102:
({void*_tmp19B=0U;({unsigned _tmp576=e2->loc;struct _fat_ptr _tmp575=({const char*_tmp19C="cannot swap value into zeroterm array not known to be 0";_tag_fat(_tmp19C,sizeof(char),56U);});Cyc_CfFlowInfo_aerr(_tmp576,_tmp575,_tag_fat(_tmp19B,sizeof(void*),0U));});});}_LLFE:;}}{
# 1322
struct _tuple19 _stmttmp37=
# 1339
Cyc_NewControlFlow_anal_Lexp(env,outflow,0,0,e1);
# 1322
union Cyc_CfFlowInfo_AbsLVal _tmp19D;_LL104: _tmp19D=_stmttmp37.f2;_LL105: {union Cyc_CfFlowInfo_AbsLVal left_lval=_tmp19D;
# 1340
struct _tuple19 _stmttmp38=Cyc_NewControlFlow_anal_Lexp(env,outflow,0,0,e2);union Cyc_CfFlowInfo_AbsLVal _tmp19E;_LL107: _tmp19E=_stmttmp38.f2;_LL108: {union Cyc_CfFlowInfo_AbsLVal right_lval=_tmp19E;
{union Cyc_CfFlowInfo_FlowInfo _tmp19F=outflow;struct Cyc_List_List*_tmp1A1;struct Cyc_Dict_Dict _tmp1A0;if((_tmp19F.ReachableFL).tag == 2){_LL10A: _tmp1A0=((_tmp19F.ReachableFL).val).f1;_tmp1A1=((_tmp19F.ReachableFL).val).f2;_LL10B: {struct Cyc_Dict_Dict outdict=_tmp1A0;struct Cyc_List_List*relns=_tmp1A1;
# 1343
{union Cyc_CfFlowInfo_AbsLVal _tmp1A2=left_lval;struct Cyc_CfFlowInfo_Place*_tmp1A3;if((_tmp1A2.PlaceL).tag == 1){_LL10F: _tmp1A3=(_tmp1A2.PlaceL).val;_LL110: {struct Cyc_CfFlowInfo_Place*lp=_tmp1A3;
# 1345
outdict=Cyc_CfFlowInfo_assign_place(fenv,e1->loc,outdict,lp,right_rval);
goto _LL10E;}}else{_LL111: _LL112:
# 1351
 if((int)Cyc_CfFlowInfo_initlevel(fenv,outdict,right_rval)!= (int)Cyc_CfFlowInfo_AllIL)
({void*_tmp1A4=0U;({unsigned _tmp578=e2->loc;struct _fat_ptr _tmp577=({const char*_tmp1A5="expression may not be fully initialized";_tag_fat(_tmp1A5,sizeof(char),40U);});Cyc_CfFlowInfo_aerr(_tmp578,_tmp577,_tag_fat(_tmp1A4,sizeof(void*),0U));});});
# 1351
goto _LL10E;}_LL10E:;}
# 1355
{union Cyc_CfFlowInfo_AbsLVal _tmp1A6=right_lval;struct Cyc_CfFlowInfo_Place*_tmp1A7;if((_tmp1A6.PlaceL).tag == 1){_LL114: _tmp1A7=(_tmp1A6.PlaceL).val;_LL115: {struct Cyc_CfFlowInfo_Place*rp=_tmp1A7;
# 1357
outdict=Cyc_CfFlowInfo_assign_place(fenv,e2->loc,outdict,rp,left_rval);
goto _LL113;}}else{_LL116: _LL117:
# 1360
 if((int)Cyc_CfFlowInfo_initlevel(fenv,outdict,left_rval)!= (int)Cyc_CfFlowInfo_AllIL)
({void*_tmp1A8=0U;({unsigned _tmp57A=e1->loc;struct _fat_ptr _tmp579=({const char*_tmp1A9="expression may not be fully initialized";_tag_fat(_tmp1A9,sizeof(char),40U);});Cyc_CfFlowInfo_aerr(_tmp57A,_tmp579,_tag_fat(_tmp1A8,sizeof(void*),0U));});});
# 1360
goto _LL113;}_LL113:;}
# 1365
relns=Cyc_Relations_reln_kill_exp(Cyc_Core_heap_region,relns,e1);
relns=Cyc_Relations_reln_kill_exp(Cyc_Core_heap_region,relns,e2);
# 1368
outflow=Cyc_CfFlowInfo_ReachableFL(outdict,relns);
Cyc_NewControlFlow_update_tryflow(env,outflow);
goto _LL109;}}else{_LL10C: _LL10D:
 goto _LL109;}_LL109:;}
# 1375
return({struct _tuple17 _tmp46C;_tmp46C.f1=outflow,_tmp46C.f2=fenv->unknown_all;_tmp46C;});}}}}}}case 16U: _LL52: _tmp107=((struct Cyc_Absyn_New_e_Absyn_Raw_exp_struct*)_tmpE7)->f1;_tmp108=((struct Cyc_Absyn_New_e_Absyn_Raw_exp_struct*)_tmpE7)->f2;_LL53: {struct Cyc_Absyn_Exp*eopt=_tmp107;struct Cyc_Absyn_Exp*e2=_tmp108;
# 1378
union Cyc_CfFlowInfo_FlowInfo outflow;
void*place_val;
if(eopt != 0){
struct _tuple24 _stmttmp39=({struct Cyc_NewControlFlow_AnalEnv*_tmp57C=env;union Cyc_CfFlowInfo_FlowInfo _tmp57B=inflow;Cyc_NewControlFlow_anal_Rexps(_tmp57C,_tmp57B,({struct Cyc_Absyn_Exp*_tmp1AD[2U];_tmp1AD[0]=eopt,_tmp1AD[1]=e2;Cyc_List_list(_tag_fat(_tmp1AD,sizeof(struct Cyc_Absyn_Exp*),2U));}),1,1);});struct Cyc_List_List*_tmp1AC;union Cyc_CfFlowInfo_FlowInfo _tmp1AB;_LL119: _tmp1AB=_stmttmp39.f1;_tmp1AC=_stmttmp39.f2;_LL11A: {union Cyc_CfFlowInfo_FlowInfo f=_tmp1AB;struct Cyc_List_List*rvals=_tmp1AC;
outflow=f;
place_val=(void*)((struct Cyc_List_List*)_check_null(((struct Cyc_List_List*)_check_null(rvals))->tl))->hd;}}else{
# 1385
struct _tuple17 _stmttmp3A=Cyc_NewControlFlow_anal_Rexp(env,1,inflow,e2);void*_tmp1AF;union Cyc_CfFlowInfo_FlowInfo _tmp1AE;_LL11C: _tmp1AE=_stmttmp3A.f1;_tmp1AF=_stmttmp3A.f2;_LL11D: {union Cyc_CfFlowInfo_FlowInfo f=_tmp1AE;void*r=_tmp1AF;
outflow=f;
place_val=r;}}
# 1390
if(Cyc_Tcutil_is_noalias_pointer((void*)_check_null(e->topt),1))
return({struct _tuple17 _tmp46D;_tmp46D.f1=outflow,({void*_tmp57D=(void*)({struct Cyc_CfFlowInfo_UniquePtr_CfFlowInfo_AbsRVal_struct*_tmp1B0=_cycalloc(sizeof(*_tmp1B0));_tmp1B0->tag=5U,_tmp1B0->f1=place_val;_tmp1B0;});_tmp46D.f2=_tmp57D;});_tmp46D;});else{
# 1394
void*root=(void*)({struct Cyc_CfFlowInfo_MallocPt_CfFlowInfo_Root_struct*_tmp1B6=_cycalloc(sizeof(*_tmp1B6));_tmp1B6->tag=1U,_tmp1B6->f1=e2,_tmp1B6->f2=(void*)_check_null(e->topt);_tmp1B6;});
struct Cyc_CfFlowInfo_Place*place=({struct Cyc_CfFlowInfo_Place*_tmp1B5=_cycalloc(sizeof(*_tmp1B5));_tmp1B5->root=root,_tmp1B5->path=0;_tmp1B5;});
void*rval=(void*)({struct Cyc_CfFlowInfo_AddressOf_CfFlowInfo_AbsRVal_struct*_tmp1B4=_cycalloc(sizeof(*_tmp1B4));_tmp1B4->tag=4U,_tmp1B4->f1=place;_tmp1B4;});
union Cyc_CfFlowInfo_FlowInfo _tmp1B1=outflow;struct Cyc_List_List*_tmp1B3;struct Cyc_Dict_Dict _tmp1B2;if((_tmp1B1.BottomFL).tag == 1){_LL11F: _LL120:
 return({struct _tuple17 _tmp46E;_tmp46E.f1=outflow,_tmp46E.f2=rval;_tmp46E;});}else{_LL121: _tmp1B2=((_tmp1B1.ReachableFL).val).f1;_tmp1B3=((_tmp1B1.ReachableFL).val).f2;_LL122: {struct Cyc_Dict_Dict d2=_tmp1B2;struct Cyc_List_List*relns=_tmp1B3;
# 1400
return({struct _tuple17 _tmp46F;({union Cyc_CfFlowInfo_FlowInfo _tmp57F=({struct Cyc_Dict_Dict _tmp57E=Cyc_Dict_insert(d2,root,place_val);Cyc_CfFlowInfo_ReachableFL(_tmp57E,relns);});_tmp46F.f1=_tmp57F;}),_tmp46F.f2=rval;_tmp46F;});}}_LL11E:;}}case 15U: _LL54: _tmp106=((struct Cyc_Absyn_Address_e_Absyn_Raw_exp_struct*)_tmpE7)->f1;_LL55: {struct Cyc_Absyn_Exp*e1=_tmp106;
# 1405
struct _tuple19 _stmttmp3B=Cyc_NewControlFlow_anal_Lexp(env,inflow,0,0,e1);union Cyc_CfFlowInfo_AbsLVal _tmp1B8;union Cyc_CfFlowInfo_FlowInfo _tmp1B7;_LL124: _tmp1B7=_stmttmp3B.f1;_tmp1B8=_stmttmp3B.f2;_LL125: {union Cyc_CfFlowInfo_FlowInfo f=_tmp1B7;union Cyc_CfFlowInfo_AbsLVal l=_tmp1B8;
union Cyc_CfFlowInfo_AbsLVal _tmp1B9=l;struct Cyc_CfFlowInfo_Place*_tmp1BA;if((_tmp1B9.UnknownL).tag == 2){_LL127: _LL128:
 return({struct _tuple17 _tmp470;_tmp470.f1=f,_tmp470.f2=fenv->notzeroall;_tmp470;});}else{_LL129: _tmp1BA=(_tmp1B9.PlaceL).val;_LL12A: {struct Cyc_CfFlowInfo_Place*p=_tmp1BA;
return({struct _tuple17 _tmp471;_tmp471.f1=f,({void*_tmp580=(void*)({struct Cyc_CfFlowInfo_AddressOf_CfFlowInfo_AbsRVal_struct*_tmp1BB=_cycalloc(sizeof(*_tmp1BB));_tmp1BB->tag=4U,_tmp1BB->f1=p;_tmp1BB;});_tmp471.f2=_tmp580;});_tmp471;});}}_LL126:;}}case 20U: _LL56: _tmp105=((struct Cyc_Absyn_Deref_e_Absyn_Raw_exp_struct*)_tmpE7)->f1;_LL57: {struct Cyc_Absyn_Exp*e1=_tmp105;
# 1412
struct _tuple17 _stmttmp3C=Cyc_NewControlFlow_anal_Rexp(env,0,inflow,e1);void*_tmp1BD;union Cyc_CfFlowInfo_FlowInfo _tmp1BC;_LL12C: _tmp1BC=_stmttmp3C.f1;_tmp1BD=_stmttmp3C.f2;_LL12D: {union Cyc_CfFlowInfo_FlowInfo f=_tmp1BC;void*r=_tmp1BD;
return Cyc_NewControlFlow_anal_derefR(env,inflow,f,e1,r,0);}}case 21U: _LL58: _tmp103=((struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct*)_tmpE7)->f1;_tmp104=((struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct*)_tmpE7)->f2;_LL59: {struct Cyc_Absyn_Exp*e1=_tmp103;struct _fat_ptr*field=_tmp104;
# 1416
struct _tuple17 _stmttmp3D=Cyc_NewControlFlow_anal_Rexp(env,0,inflow,e1);void*_tmp1BF;union Cyc_CfFlowInfo_FlowInfo _tmp1BE;_LL12F: _tmp1BE=_stmttmp3D.f1;_tmp1BF=_stmttmp3D.f2;_LL130: {union Cyc_CfFlowInfo_FlowInfo f=_tmp1BE;void*r=_tmp1BF;
void*e1_type=(void*)_check_null(e1->topt);
# 1419
if(Cyc_Absyn_is_nontagged_nonrequire_union_type(e1_type))
return({struct _tuple17 _tmp472;_tmp472.f1=f,({void*_tmp581=Cyc_CfFlowInfo_typ_to_absrval(fenv,(void*)_check_null(e->topt),0,fenv->unknown_all);_tmp472.f2=_tmp581;});_tmp472;});
# 1419
if(
# 1421
Cyc_Absyn_is_require_union_type(e1_type))
Cyc_NewControlFlow_check_union_requires(e1->loc,e1_type,field,f);{
# 1419
struct _tuple16 _stmttmp3E=
# 1423
Cyc_CfFlowInfo_unname_rval(r);
# 1419
void*_tmp1C0;_LL132: _tmp1C0=_stmttmp3E.f1;_LL133: {void*r=_tmp1C0;
# 1424
void*_tmp1C1=r;struct _fat_ptr _tmp1C4;int _tmp1C3;int _tmp1C2;if(((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp1C1)->tag == 6U){_LL135: _tmp1C2=(((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp1C1)->f1).is_union;_tmp1C3=(((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp1C1)->f1).fieldnum;_tmp1C4=((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp1C1)->f2;_LL136: {int is_union=_tmp1C2;int fnum=_tmp1C3;struct _fat_ptr rdict=_tmp1C4;
# 1426
int field_no=Cyc_CfFlowInfo_get_field_index((void*)_check_null(e1->topt),field);
if((is_union && fnum != -1)&& fnum != field_no)
return({struct _tuple17 _tmp473;_tmp473.f1=f,({void*_tmp582=Cyc_CfFlowInfo_typ_to_absrval(fenv,(void*)_check_null(e->topt),1,fenv->unknown_none);_tmp473.f2=_tmp582;});_tmp473;});
# 1427
return({struct _tuple17 _tmp474;_tmp474.f1=f,_tmp474.f2=*((void**)_check_fat_subscript(rdict,sizeof(void*),field_no));_tmp474;});}}else{_LL137: _LL138:
# 1431
({void*_tmp1C5=0U;({struct Cyc___cycFILE*_tmp584=Cyc_stderr;struct _fat_ptr _tmp583=({const char*_tmp1C6="the bad rexp is :";_tag_fat(_tmp1C6,sizeof(char),18U);});Cyc_fprintf(_tmp584,_tmp583,_tag_fat(_tmp1C5,sizeof(void*),0U));});});
Cyc_CfFlowInfo_print_absrval(r);
({struct Cyc_String_pa_PrintArg_struct _tmp1CA=({struct Cyc_String_pa_PrintArg_struct _tmp475;_tmp475.tag=0U,({struct _fat_ptr _tmp585=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_exp2string(e));_tmp475.f1=_tmp585;});_tmp475;});void*_tmp1C7[1U];_tmp1C7[0]=& _tmp1CA;({int(*_tmp587)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp1C9)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmp1C9;});struct _fat_ptr _tmp586=({const char*_tmp1C8="\nanal_Rexp: AggrMember: %s";_tag_fat(_tmp1C8,sizeof(char),27U);});_tmp587(_tmp586,_tag_fat(_tmp1C7,sizeof(void*),1U));});});}_LL134:;}}}}case 39U: _LL5A: _tmp101=((struct Cyc_Absyn_Tagcheck_e_Absyn_Raw_exp_struct*)_tmpE7)->f1;_tmp102=((struct Cyc_Absyn_Tagcheck_e_Absyn_Raw_exp_struct*)_tmpE7)->f2;_LL5B: {struct Cyc_Absyn_Exp*e1=_tmp101;struct _fat_ptr*field=_tmp102;
# 1439
struct _tuple17 _stmttmp3F=Cyc_NewControlFlow_anal_Rexp(env,0,inflow,e1);void*_tmp1CC;union Cyc_CfFlowInfo_FlowInfo _tmp1CB;_LL13A: _tmp1CB=_stmttmp3F.f1;_tmp1CC=_stmttmp3F.f2;_LL13B: {union Cyc_CfFlowInfo_FlowInfo f=_tmp1CB;void*r=_tmp1CC;
# 1441
if(Cyc_Absyn_is_nontagged_nonrequire_union_type((void*)_check_null(e1->topt)))
return({struct _tuple17 _tmp476;_tmp476.f1=f,_tmp476.f2=fenv->unknown_all;_tmp476;});{
# 1441
struct _tuple16 _stmttmp40=
# 1443
Cyc_CfFlowInfo_unname_rval(r);
# 1441
void*_tmp1CD;_LL13D: _tmp1CD=_stmttmp40.f1;_LL13E: {void*r=_tmp1CD;
# 1444
void*_tmp1CE=r;struct _fat_ptr _tmp1D1;int _tmp1D0;int _tmp1CF;if(((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp1CE)->tag == 6U){_LL140: _tmp1CF=(((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp1CE)->f1).is_union;_tmp1D0=(((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp1CE)->f1).fieldnum;_tmp1D1=((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp1CE)->f2;_LL141: {int is_union=_tmp1CF;int fnum=_tmp1D0;struct _fat_ptr rdict=_tmp1D1;
# 1446
int field_no=Cyc_CfFlowInfo_get_field_index((void*)_check_null(e1->topt),field);
if(is_union && fnum != -1){
if(fnum != field_no)
return({struct _tuple17 _tmp477;_tmp477.f1=f,_tmp477.f2=fenv->zero;_tmp477;});else{
# 1451
return({struct _tuple17 _tmp478;_tmp478.f1=f,_tmp478.f2=fenv->notzeroall;_tmp478;});}}else{
# 1453
return({struct _tuple17 _tmp479;_tmp479.f1=f,_tmp479.f2=fenv->unknown_all;_tmp479;});}}}else{_LL142: _LL143:
# 1455
({struct Cyc_String_pa_PrintArg_struct _tmp1D5=({struct Cyc_String_pa_PrintArg_struct _tmp47A;_tmp47A.tag=0U,({struct _fat_ptr _tmp588=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_exp2string(e));_tmp47A.f1=_tmp588;});_tmp47A;});void*_tmp1D2[1U];_tmp1D2[0]=& _tmp1D5;({int(*_tmp58A)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp1D4)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmp1D4;});struct _fat_ptr _tmp589=({const char*_tmp1D3="anal_Rexp: TagCheck_e: %s";_tag_fat(_tmp1D3,sizeof(char),26U);});_tmp58A(_tmp589,_tag_fat(_tmp1D2,sizeof(void*),1U));});});}_LL13F:;}}}}case 22U: _LL5C: _tmpFF=((struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct*)_tmpE7)->f1;_tmp100=((struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct*)_tmpE7)->f2;_LL5D: {struct Cyc_Absyn_Exp*e1=_tmpFF;struct _fat_ptr*field=_tmp100;
# 1459
struct _tuple17 _stmttmp41=Cyc_NewControlFlow_anal_Rexp(env,0,inflow,e1);void*_tmp1D7;union Cyc_CfFlowInfo_FlowInfo _tmp1D6;_LL145: _tmp1D6=_stmttmp41.f1;_tmp1D7=_stmttmp41.f2;_LL146: {union Cyc_CfFlowInfo_FlowInfo f1=_tmp1D6;void*r1=_tmp1D7;
# 1462
struct _tuple17 _stmttmp42=Cyc_NewControlFlow_anal_derefR(env,inflow,f1,e1,r1,0);void*_tmp1D9;union Cyc_CfFlowInfo_FlowInfo _tmp1D8;_LL148: _tmp1D8=_stmttmp42.f1;_tmp1D9=_stmttmp42.f2;_LL149: {union Cyc_CfFlowInfo_FlowInfo f2=_tmp1D8;void*r2=_tmp1D9;
# 1465
void*_stmttmp43=Cyc_Tcutil_compress((void*)_check_null(e1->topt));void*_tmp1DA=_stmttmp43;void*_tmp1DB;if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp1DA)->tag == 3U){_LL14B: _tmp1DB=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp1DA)->f1).elt_type;_LL14C: {void*t2=_tmp1DB;
# 1467
if(Cyc_Absyn_is_nontagged_nonrequire_union_type(t2))
# 1469
return({struct _tuple17 _tmp47B;_tmp47B.f1=f2,({void*_tmp58B=Cyc_CfFlowInfo_typ_to_absrval(fenv,(void*)_check_null(e->topt),0,fenv->unknown_all);_tmp47B.f2=_tmp58B;});_tmp47B;});
# 1467
if(
# 1471
Cyc_Absyn_is_require_union_type(t2))
Cyc_NewControlFlow_check_union_requires(e1->loc,t2,field,f1);{
# 1467
struct _tuple16 _stmttmp44=
# 1474
Cyc_CfFlowInfo_unname_rval(r2);
# 1467
void*_tmp1DC;_LL150: _tmp1DC=_stmttmp44.f1;_LL151: {void*r2=_tmp1DC;
# 1475
void*_tmp1DD=r2;struct _fat_ptr _tmp1E0;int _tmp1DF;int _tmp1DE;if(((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp1DD)->tag == 6U){_LL153: _tmp1DE=(((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp1DD)->f1).is_union;_tmp1DF=(((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp1DD)->f1).fieldnum;_tmp1E0=((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp1DD)->f2;_LL154: {int is_union=_tmp1DE;int fnum=_tmp1DF;struct _fat_ptr rdict=_tmp1E0;
# 1477
int field_no=Cyc_CfFlowInfo_get_field_index(t2,field);
if((is_union && fnum != -1)&& fnum != field_no)
return({struct _tuple17 _tmp47C;_tmp47C.f1=f2,({void*_tmp58C=Cyc_CfFlowInfo_typ_to_absrval(fenv,(void*)_check_null(e->topt),1,fenv->unknown_none);_tmp47C.f2=_tmp58C;});_tmp47C;});
# 1478
return({struct _tuple17 _tmp47D;_tmp47D.f1=f2,_tmp47D.f2=*((void**)_check_fat_subscript(rdict,sizeof(void*),field_no));_tmp47D;});}}else{_LL155: _LL156:
# 1481
({void*_tmp1E1=0U;({int(*_tmp58E)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp1E3)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmp1E3;});struct _fat_ptr _tmp58D=({const char*_tmp1E2="anal_Rexp: AggrArrow";_tag_fat(_tmp1E2,sizeof(char),21U);});_tmp58E(_tmp58D,_tag_fat(_tmp1E1,sizeof(void*),0U));});});}_LL152:;}}}}else{_LL14D: _LL14E:
# 1483
({void*_tmp1E4=0U;({int(*_tmp590)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp1E6)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmp1E6;});struct _fat_ptr _tmp58F=({const char*_tmp1E5="anal_Rexp: AggrArrow ptr";_tag_fat(_tmp1E5,sizeof(char),25U);});_tmp590(_tmp58F,_tag_fat(_tmp1E4,sizeof(void*),0U));});});}_LL14A:;}}}case 6U: _LL5E: _tmpFC=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmpE7)->f1;_tmpFD=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmpE7)->f2;_tmpFE=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmpE7)->f3;_LL5F: {struct Cyc_Absyn_Exp*e1=_tmpFC;struct Cyc_Absyn_Exp*e2=_tmpFD;struct Cyc_Absyn_Exp*e3=_tmpFE;
# 1487
struct _tuple20 _stmttmp45=Cyc_NewControlFlow_anal_test(env,inflow,e1);union Cyc_CfFlowInfo_FlowInfo _tmp1E8;union Cyc_CfFlowInfo_FlowInfo _tmp1E7;_LL158: _tmp1E7=_stmttmp45.f1;_tmp1E8=_stmttmp45.f2;_LL159: {union Cyc_CfFlowInfo_FlowInfo f1t=_tmp1E7;union Cyc_CfFlowInfo_FlowInfo f1f=_tmp1E8;
struct _tuple17 pr2=Cyc_NewControlFlow_anal_Rexp(env,copy_ctxt,f1t,e2);
struct _tuple17 pr3=Cyc_NewControlFlow_anal_Rexp(env,copy_ctxt,f1f,e3);
return Cyc_CfFlowInfo_join_flow_and_rval(fenv,pr2,pr3);}}case 7U: _LL60: _tmpFA=((struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct*)_tmpE7)->f1;_tmpFB=((struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct*)_tmpE7)->f2;_LL61: {struct Cyc_Absyn_Exp*e1=_tmpFA;struct Cyc_Absyn_Exp*e2=_tmpFB;
# 1493
struct _tuple20 _stmttmp46=Cyc_NewControlFlow_anal_test(env,inflow,e1);union Cyc_CfFlowInfo_FlowInfo _tmp1EA;union Cyc_CfFlowInfo_FlowInfo _tmp1E9;_LL15B: _tmp1E9=_stmttmp46.f1;_tmp1EA=_stmttmp46.f2;_LL15C: {union Cyc_CfFlowInfo_FlowInfo f1t=_tmp1E9;union Cyc_CfFlowInfo_FlowInfo f1f=_tmp1EA;
struct _tuple17 _stmttmp47=Cyc_NewControlFlow_anal_Rexp(env,copy_ctxt,f1t,e2);void*_tmp1EC;union Cyc_CfFlowInfo_FlowInfo _tmp1EB;_LL15E: _tmp1EB=_stmttmp47.f1;_tmp1EC=_stmttmp47.f2;_LL15F: {union Cyc_CfFlowInfo_FlowInfo f2t=_tmp1EB;void*f2r=_tmp1EC;
struct _tuple17 pr2=({struct _tuple17 _tmp47F;_tmp47F.f1=f2t,_tmp47F.f2=f2r;_tmp47F;});
struct _tuple17 pr3=({struct _tuple17 _tmp47E;_tmp47E.f1=f1f,_tmp47E.f2=fenv->zero;_tmp47E;});
return Cyc_CfFlowInfo_join_flow_and_rval(fenv,pr2,pr3);}}}case 8U: _LL62: _tmpF8=((struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct*)_tmpE7)->f1;_tmpF9=((struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct*)_tmpE7)->f2;_LL63: {struct Cyc_Absyn_Exp*e1=_tmpF8;struct Cyc_Absyn_Exp*e2=_tmpF9;
# 1500
struct _tuple20 _stmttmp48=Cyc_NewControlFlow_anal_test(env,inflow,e1);union Cyc_CfFlowInfo_FlowInfo _tmp1EE;union Cyc_CfFlowInfo_FlowInfo _tmp1ED;_LL161: _tmp1ED=_stmttmp48.f1;_tmp1EE=_stmttmp48.f2;_LL162: {union Cyc_CfFlowInfo_FlowInfo f1t=_tmp1ED;union Cyc_CfFlowInfo_FlowInfo f1f=_tmp1EE;
struct _tuple17 _stmttmp49=Cyc_NewControlFlow_anal_Rexp(env,copy_ctxt,f1f,e2);void*_tmp1F0;union Cyc_CfFlowInfo_FlowInfo _tmp1EF;_LL164: _tmp1EF=_stmttmp49.f1;_tmp1F0=_stmttmp49.f2;_LL165: {union Cyc_CfFlowInfo_FlowInfo f2t=_tmp1EF;void*f2r=_tmp1F0;
struct _tuple17 pr2=({struct _tuple17 _tmp481;_tmp481.f1=f2t,_tmp481.f2=f2r;_tmp481;});
struct _tuple17 pr3=({struct _tuple17 _tmp480;_tmp480.f1=f1t,_tmp480.f2=fenv->notzeroall;_tmp480;});
return Cyc_CfFlowInfo_join_flow_and_rval(fenv,pr2,pr3);}}}case 23U: _LL64: _tmpF6=((struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct*)_tmpE7)->f1;_tmpF7=((struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct*)_tmpE7)->f2;_LL65: {struct Cyc_Absyn_Exp*e1=_tmpF6;struct Cyc_Absyn_Exp*e2=_tmpF7;
# 1507
struct _tuple24 _stmttmp4A=({struct Cyc_NewControlFlow_AnalEnv*_tmp592=env;union Cyc_CfFlowInfo_FlowInfo _tmp591=inflow;Cyc_NewControlFlow_anal_Rexps(_tmp592,_tmp591,({struct Cyc_Absyn_Exp*_tmp206[2U];_tmp206[0]=e1,_tmp206[1]=e2;Cyc_List_list(_tag_fat(_tmp206,sizeof(struct Cyc_Absyn_Exp*),2U));}),0,1);});struct Cyc_List_List*_tmp1F2;union Cyc_CfFlowInfo_FlowInfo _tmp1F1;_LL167: _tmp1F1=_stmttmp4A.f1;_tmp1F2=_stmttmp4A.f2;_LL168: {union Cyc_CfFlowInfo_FlowInfo f=_tmp1F1;struct Cyc_List_List*rvals=_tmp1F2;
# 1511
union Cyc_CfFlowInfo_FlowInfo f2=f;
{union Cyc_CfFlowInfo_FlowInfo _tmp1F3=f;struct Cyc_List_List*_tmp1F5;struct Cyc_Dict_Dict _tmp1F4;if((_tmp1F3.ReachableFL).tag == 2){_LL16A: _tmp1F4=((_tmp1F3.ReachableFL).val).f1;_tmp1F5=((_tmp1F3.ReachableFL).val).f2;_LL16B: {struct Cyc_Dict_Dict d2=_tmp1F4;struct Cyc_List_List*relns=_tmp1F5;
# 1516
if((int)Cyc_CfFlowInfo_initlevel(env->fenv,d2,(void*)((struct Cyc_List_List*)_check_null(((struct Cyc_List_List*)_check_null(rvals))->tl))->hd)== (int)Cyc_CfFlowInfo_NoneIL)
({void*_tmp1F6=0U;({unsigned _tmp594=e2->loc;struct _fat_ptr _tmp593=({const char*_tmp1F7="expression may not be initialized";_tag_fat(_tmp1F7,sizeof(char),34U);});Cyc_CfFlowInfo_aerr(_tmp594,_tmp593,_tag_fat(_tmp1F6,sizeof(void*),0U));});});{
# 1516
struct Cyc_List_List*new_relns=
# 1518
Cyc_NewControlFlow_add_subscript_reln(relns,e1,e2);
if(relns != new_relns)
f2=Cyc_CfFlowInfo_ReachableFL(d2,new_relns);
# 1519
goto _LL169;}}}else{_LL16C: _LL16D:
# 1522
 goto _LL169;}_LL169:;}{
# 1524
void*_stmttmp4B=Cyc_Tcutil_compress((void*)_check_null(e1->topt));void*_tmp1F8=_stmttmp4B;struct Cyc_List_List*_tmp1F9;switch(*((int*)_tmp1F8)){case 6U: _LL16F: _tmp1F9=((struct Cyc_Absyn_TupleType_Absyn_Type_struct*)_tmp1F8)->f1;_LL170: {struct Cyc_List_List*tqts=_tmp1F9;
# 1526
struct _tuple16 _stmttmp4C=Cyc_CfFlowInfo_unname_rval((void*)((struct Cyc_List_List*)_check_null(rvals))->hd);void*_tmp1FA;_LL176: _tmp1FA=_stmttmp4C.f1;_LL177: {void*r=_tmp1FA;
void*_tmp1FB=r;struct _fat_ptr _tmp1FC;if(((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp1FB)->tag == 6U){_LL179: _tmp1FC=((struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*)_tmp1FB)->f2;_LL17A: {struct _fat_ptr rdict=_tmp1FC;
# 1529
unsigned i=(Cyc_Evexp_eval_const_uint_exp(e2)).f1;
return({struct _tuple17 _tmp482;_tmp482.f1=f2,_tmp482.f2=*((void**)_check_fat_subscript(rdict,sizeof(void*),(int)i));_tmp482;});}}else{_LL17B: _LL17C:
({void*_tmp1FD=0U;({int(*_tmp596)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp1FF)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmp1FF;});struct _fat_ptr _tmp595=({const char*_tmp1FE="anal_Rexp: Subscript";_tag_fat(_tmp1FE,sizeof(char),21U);});_tmp596(_tmp595,_tag_fat(_tmp1FD,sizeof(void*),0U));});});}_LL178:;}}case 3U: _LL171: _LL172: {
# 1534
struct _tuple17 _stmttmp4D=Cyc_NewControlFlow_anal_derefR(env,inflow,f,e1,(void*)((struct Cyc_List_List*)_check_null(rvals))->hd,e2);void*_tmp201;union Cyc_CfFlowInfo_FlowInfo _tmp200;_LL17E: _tmp200=_stmttmp4D.f1;_tmp201=_stmttmp4D.f2;_LL17F: {union Cyc_CfFlowInfo_FlowInfo f3=_tmp200;void*r=_tmp201;
union Cyc_CfFlowInfo_FlowInfo _tmp202=f3;if((_tmp202.BottomFL).tag == 1){_LL181: _LL182:
 return({struct _tuple17 _tmp483;_tmp483.f1=f3,_tmp483.f2=r;_tmp483;});}else{_LL183: _LL184:
 return({struct _tuple17 _tmp484;_tmp484.f1=f2,_tmp484.f2=r;_tmp484;});}_LL180:;}}default: _LL173: _LL174:
# 1539
({void*_tmp203=0U;({int(*_tmp598)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp205)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmp205;});struct _fat_ptr _tmp597=({const char*_tmp204="anal_Rexp: Subscript -- bad type";_tag_fat(_tmp204,sizeof(char),33U);});_tmp598(_tmp597,_tag_fat(_tmp203,sizeof(void*),0U));});});}_LL16E:;}}}case 24U: _LL68: _tmpF5=((struct Cyc_Absyn_Tuple_e_Absyn_Raw_exp_struct*)_tmpE7)->f1;_LL69: {struct Cyc_List_List*es=_tmpF5;
# 1545
struct _tuple24 _stmttmp4E=Cyc_NewControlFlow_anal_Rexps(env,inflow,es,1,1);struct Cyc_List_List*_tmp208;union Cyc_CfFlowInfo_FlowInfo _tmp207;_LL186: _tmp207=_stmttmp4E.f1;_tmp208=_stmttmp4E.f2;_LL187: {union Cyc_CfFlowInfo_FlowInfo f=_tmp207;struct Cyc_List_List*rvals=_tmp208;
struct _fat_ptr aggrdict=({unsigned _tmp20B=(unsigned)
Cyc_List_length(es);void**_tmp20A=_cycalloc(_check_times(_tmp20B,sizeof(void*)));({{unsigned _tmp486=(unsigned)Cyc_List_length(es);unsigned i;for(i=0;i < _tmp486;++ i){({
void*_tmp599=({void*temp=(void*)((struct Cyc_List_List*)_check_null(rvals))->hd;
rvals=rvals->tl;
temp;});
# 1548
_tmp20A[i]=_tmp599;});}}0;});_tag_fat(_tmp20A,sizeof(void*),_tmp20B);});
# 1552
return({struct _tuple17 _tmp485;_tmp485.f1=f,({void*_tmp59A=(void*)({struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*_tmp209=_cycalloc(sizeof(*_tmp209));_tmp209->tag=6U,(_tmp209->f1).is_union=0,(_tmp209->f1).fieldnum=- 1,_tmp209->f2=aggrdict;_tmp209;});_tmp485.f2=_tmp59A;});_tmp485;});}}case 30U: _LL6A: _tmpF4=((struct Cyc_Absyn_AnonStruct_e_Absyn_Raw_exp_struct*)_tmpE7)->f2;_LL6B: {struct Cyc_List_List*des=_tmpF4;
# 1555
struct Cyc_List_List*fs;
{void*_stmttmp4F=Cyc_Tcutil_compress((void*)_check_null(e->topt));void*_tmp20C=_stmttmp4F;struct Cyc_List_List*_tmp20D;if(((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmp20C)->tag == 7U){_LL189: _tmp20D=((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmp20C)->f2;_LL18A: {struct Cyc_List_List*f=_tmp20D;
fs=f;goto _LL188;}}else{_LL18B: _LL18C:
({void*_tmp20E=0U;({int(*_tmp59C)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp210)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmp210;});struct _fat_ptr _tmp59B=({const char*_tmp20F="anal_Rexp:anon struct has bad type";_tag_fat(_tmp20F,sizeof(char),35U);});_tmp59C(_tmp59B,_tag_fat(_tmp20E,sizeof(void*),0U));});});}_LL188:;}
# 1560
_tmpF1=des;_tmpF2=0U;_tmpF3=fs;goto _LL6D;}case 29U: if(((struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct*)_tmpE7)->f4 != 0){if(((struct Cyc_Absyn_Aggrdecl*)((struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct*)_tmpE7)->f4)->impl != 0){_LL6C: _tmpF1=((struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct*)_tmpE7)->f3;_tmpF2=(((struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct*)_tmpE7)->f4)->kind;_tmpF3=((((struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct*)_tmpE7)->f4)->impl)->fields;_LL6D: {struct Cyc_List_List*des=_tmpF1;enum Cyc_Absyn_AggrKind kind=_tmpF2;struct Cyc_List_List*fs=_tmpF3;
# 1562
void*exp_type=(void*)_check_null(e->topt);
struct _tuple24 _stmttmp50=({struct Cyc_NewControlFlow_AnalEnv*_tmp5A0=env;union Cyc_CfFlowInfo_FlowInfo _tmp59F=inflow;Cyc_NewControlFlow_anal_Rexps(_tmp5A0,_tmp59F,({struct Cyc_List_List*(*_tmp59E)(struct Cyc_Absyn_Exp*(*f)(struct _tuple27*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp219)(struct Cyc_Absyn_Exp*(*f)(struct _tuple27*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_Absyn_Exp*(*f)(struct _tuple27*),struct Cyc_List_List*x))Cyc_List_map;_tmp219;});struct Cyc_Absyn_Exp*(*_tmp59D)(struct _tuple27*)=({struct Cyc_Absyn_Exp*(*_tmp21A)(struct _tuple27*)=(struct Cyc_Absyn_Exp*(*)(struct _tuple27*))Cyc_Core_snd;_tmp21A;});_tmp59E(_tmp59D,des);}),1,1);});struct Cyc_List_List*_tmp212;union Cyc_CfFlowInfo_FlowInfo _tmp211;_LL18E: _tmp211=_stmttmp50.f1;_tmp212=_stmttmp50.f2;_LL18F: {union Cyc_CfFlowInfo_FlowInfo f=_tmp211;struct Cyc_List_List*rvals=_tmp212;
struct _fat_ptr aggrdict=
Cyc_CfFlowInfo_aggrfields_to_aggrdict(fenv,fs,(int)kind == (int)Cyc_Absyn_UnionA,fenv->unknown_all);
# 1567
int field_no=-1;
{int i=0;for(0;rvals != 0;(rvals=rvals->tl,des=des->tl,++ i)){
struct Cyc_List_List*ds=(*((struct _tuple27*)((struct Cyc_List_List*)_check_null(des))->hd)).f1;for(0;ds != 0;ds=ds->tl){
void*_stmttmp51=(void*)ds->hd;void*_tmp213=_stmttmp51;struct _fat_ptr*_tmp214;if(((struct Cyc_Absyn_ArrayElement_Absyn_Designator_struct*)_tmp213)->tag == 0U){_LL191: _LL192:
({void*_tmp215=0U;({int(*_tmp5A2)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp217)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmp217;});struct _fat_ptr _tmp5A1=({const char*_tmp216="anal_Rexp:Aggregate_e";_tag_fat(_tmp216,sizeof(char),22U);});_tmp5A2(_tmp5A1,_tag_fat(_tmp215,sizeof(void*),0U));});});}else{_LL193: _tmp214=((struct Cyc_Absyn_FieldName_Absyn_Designator_struct*)_tmp213)->f1;_LL194: {struct _fat_ptr*fld=_tmp214;
# 1574
field_no=Cyc_CfFlowInfo_get_field_index_fs(fs,fld);
*((void**)_check_fat_subscript(aggrdict,sizeof(void*),field_no))=(void*)rvals->hd;
# 1577
if((int)kind == (int)1U){
struct Cyc_Absyn_Exp*e=(*((struct _tuple27*)des->hd)).f2;
f=Cyc_NewControlFlow_use_Rval(env,e->loc,f,(void*)rvals->hd);
# 1581
Cyc_NewControlFlow_check_union_requires(e->loc,exp_type,fld,f);}}}_LL190:;}}}{
# 1584
struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*res=({struct Cyc_CfFlowInfo_Aggregate_CfFlowInfo_AbsRVal_struct*_tmp218=_cycalloc(sizeof(*_tmp218));_tmp218->tag=6U,(_tmp218->f1).is_union=(int)kind == (int)Cyc_Absyn_UnionA,(_tmp218->f1).fieldnum=field_no,_tmp218->f2=aggrdict;_tmp218;});
return({struct _tuple17 _tmp487;_tmp487.f1=f,_tmp487.f2=(void*)res;_tmp487;});}}}}else{goto _LL6E;}}else{_LL6E: _LL6F:
# 1588
({struct Cyc_String_pa_PrintArg_struct _tmp21E=({struct Cyc_String_pa_PrintArg_struct _tmp488;_tmp488.tag=0U,({struct _fat_ptr _tmp5A3=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_exp2string(e));_tmp488.f1=_tmp5A3;});_tmp488;});void*_tmp21B[1U];_tmp21B[0]=& _tmp21E;({int(*_tmp5A5)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp21D)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmp21D;});struct _fat_ptr _tmp5A4=({const char*_tmp21C="anal_Rexp:missing aggrdeclimpl in %s";_tag_fat(_tmp21C,sizeof(char),37U);});_tmp5A5(_tmp5A4,_tag_fat(_tmp21B,sizeof(void*),1U));});});}case 26U: _LL70: _tmpF0=((struct Cyc_Absyn_Array_e_Absyn_Raw_exp_struct*)_tmpE7)->f1;_LL71: {struct Cyc_List_List*dles=_tmpF0;
# 1590
struct Cyc_List_List*es=({struct Cyc_List_List*(*_tmp5A7)(struct Cyc_Absyn_Exp*(*f)(struct _tuple27*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp221)(struct Cyc_Absyn_Exp*(*f)(struct _tuple27*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_Absyn_Exp*(*f)(struct _tuple27*),struct Cyc_List_List*x))Cyc_List_map;_tmp221;});struct Cyc_Absyn_Exp*(*_tmp5A6)(struct _tuple27*)=({struct Cyc_Absyn_Exp*(*_tmp222)(struct _tuple27*)=(struct Cyc_Absyn_Exp*(*)(struct _tuple27*))Cyc_Core_snd;_tmp222;});_tmp5A7(_tmp5A6,dles);});
struct _tuple24 _stmttmp52=Cyc_NewControlFlow_anal_Rexps(env,inflow,es,1,1);struct Cyc_List_List*_tmp220;union Cyc_CfFlowInfo_FlowInfo _tmp21F;_LL196: _tmp21F=_stmttmp52.f1;_tmp220=_stmttmp52.f2;_LL197: {union Cyc_CfFlowInfo_FlowInfo outflow=_tmp21F;struct Cyc_List_List*rvals=_tmp220;
for(0;rvals != 0;(rvals=rvals->tl,es=es->tl)){
outflow=Cyc_NewControlFlow_use_Rval(env,((struct Cyc_Absyn_Exp*)((struct Cyc_List_List*)_check_null(es))->hd)->loc,outflow,(void*)rvals->hd);}
# 1595
return({struct _tuple17 _tmp489;_tmp489.f1=outflow,({void*_tmp5A8=Cyc_CfFlowInfo_typ_to_absrval(fenv,(void*)_check_null(e->topt),0,fenv->notzeroall);_tmp489.f2=_tmp5A8;});_tmp489;});}}case 27U: _LL72: _tmpEC=((struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct*)_tmpE7)->f1;_tmpED=((struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct*)_tmpE7)->f2;_tmpEE=((struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct*)_tmpE7)->f3;_tmpEF=((struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct*)_tmpE7)->f4;_LL73: {struct Cyc_Absyn_Vardecl*vd=_tmpEC;struct Cyc_Absyn_Exp*e1=_tmpED;struct Cyc_Absyn_Exp*e2=_tmpEE;int zt=_tmpEF;
# 1599
struct _tuple17 _stmttmp53=Cyc_NewControlFlow_anal_Rexp(env,1,inflow,e1);void*_tmp224;union Cyc_CfFlowInfo_FlowInfo _tmp223;_LL199: _tmp223=_stmttmp53.f1;_tmp224=_stmttmp53.f2;_LL19A: {union Cyc_CfFlowInfo_FlowInfo f1=_tmp223;void*r1=_tmp224;
union Cyc_CfFlowInfo_FlowInfo _tmp225=f1;struct Cyc_List_List*_tmp227;struct Cyc_Dict_Dict _tmp226;if((_tmp225.BottomFL).tag == 1){_LL19C: _LL19D:
 return({struct _tuple17 _tmp48A;_tmp48A.f1=f1,_tmp48A.f2=fenv->unknown_all;_tmp48A;});}else{_LL19E: _tmp226=((_tmp225.ReachableFL).val).f1;_tmp227=((_tmp225.ReachableFL).val).f2;_LL19F: {struct Cyc_Dict_Dict d1=_tmp226;struct Cyc_List_List*relns=_tmp227;
# 1603
if((int)Cyc_CfFlowInfo_initlevel(env->fenv,d1,r1)== (int)Cyc_CfFlowInfo_NoneIL)
({void*_tmp228=0U;({unsigned _tmp5AA=e1->loc;struct _fat_ptr _tmp5A9=({const char*_tmp229="expression may not be initialized";_tag_fat(_tmp229,sizeof(char),34U);});Cyc_CfFlowInfo_aerr(_tmp5AA,_tmp5A9,_tag_fat(_tmp228,sizeof(void*),0U));});});{
# 1603
struct Cyc_List_List*new_relns=relns;
# 1608
union Cyc_Relations_RelnOp n1=Cyc_Relations_RVar(vd);
union Cyc_Relations_RelnOp n2=Cyc_Relations_RConst(0U);
if(Cyc_Relations_exp2relnop(e1,& n2))
new_relns=Cyc_Relations_add_relation(Cyc_Core_heap_region,n1,Cyc_Relations_Rlt,n2,relns);
# 1610
if(relns != new_relns)
# 1613
f1=Cyc_CfFlowInfo_ReachableFL(d1,new_relns);{
# 1610
void*_tmp22A=r1;switch(*((int*)_tmp22A)){case 0U: _LL1A1: _LL1A2:
# 1617
 return({struct _tuple17 _tmp48B;_tmp48B.f1=f1,_tmp48B.f2=fenv->unknown_all;_tmp48B;});case 1U: _LL1A3: _LL1A4:
 goto _LL1A6;case 4U: _LL1A5: _LL1A6: {
# 1620
struct Cyc_List_List l=({struct Cyc_List_List _tmp48E;_tmp48E.hd=vd,_tmp48E.tl=0;_tmp48E;});
f1=Cyc_NewControlFlow_add_vars(fenv,f1,& l,fenv->unknown_all,e->loc,0);{
# 1623
struct _tuple17 _stmttmp54=Cyc_NewControlFlow_anal_Rexp(env,1,f1,e2);void*_tmp22C;union Cyc_CfFlowInfo_FlowInfo _tmp22B;_LL1AA: _tmp22B=_stmttmp54.f1;_tmp22C=_stmttmp54.f2;_LL1AB: {union Cyc_CfFlowInfo_FlowInfo f2=_tmp22B;void*r2=_tmp22C;
{union Cyc_CfFlowInfo_FlowInfo _tmp22D=f2;struct Cyc_Dict_Dict _tmp22E;if((_tmp22D.BottomFL).tag == 1){_LL1AD: _LL1AE:
 return({struct _tuple17 _tmp48C;_tmp48C.f1=f2,_tmp48C.f2=fenv->unknown_all;_tmp48C;});}else{_LL1AF: _tmp22E=((_tmp22D.ReachableFL).val).f1;_LL1B0: {struct Cyc_Dict_Dict d2=_tmp22E;
# 1627
if((int)Cyc_CfFlowInfo_initlevel(fenv,d2,r2)!= (int)Cyc_CfFlowInfo_AllIL){
({void*_tmp22F=0U;({unsigned _tmp5AC=e1->loc;struct _fat_ptr _tmp5AB=({const char*_tmp230="expression may not be initialized";_tag_fat(_tmp230,sizeof(char),34U);});Cyc_CfFlowInfo_aerr(_tmp5AC,_tmp5AB,_tag_fat(_tmp22F,sizeof(void*),0U));});});
return({struct _tuple17 _tmp48D;({union Cyc_CfFlowInfo_FlowInfo _tmp5AD=Cyc_CfFlowInfo_BottomFL();_tmp48D.f1=_tmp5AD;}),_tmp48D.f2=fenv->unknown_all;_tmp48D;});}}}_LL1AC:;}
# 1632
f1=f2;
goto _LL1A8;}}}default: _LL1A7: _LL1A8:
# 1635
 while(1){
struct Cyc_List_List l=({struct Cyc_List_List _tmp490;_tmp490.hd=vd,_tmp490.tl=0;_tmp490;});
f1=Cyc_NewControlFlow_add_vars(fenv,f1,& l,fenv->unknown_all,e->loc,0);{
struct _tuple17 _stmttmp55=Cyc_NewControlFlow_anal_Rexp(env,1,f1,e2);void*_tmp232;union Cyc_CfFlowInfo_FlowInfo _tmp231;_LL1B2: _tmp231=_stmttmp55.f1;_tmp232=_stmttmp55.f2;_LL1B3: {union Cyc_CfFlowInfo_FlowInfo f2=_tmp231;void*r2=_tmp232;
{union Cyc_CfFlowInfo_FlowInfo _tmp233=f2;struct Cyc_Dict_Dict _tmp234;if((_tmp233.BottomFL).tag == 1){_LL1B5: _LL1B6:
 goto _LL1B4;}else{_LL1B7: _tmp234=((_tmp233.ReachableFL).val).f1;_LL1B8: {struct Cyc_Dict_Dict d2=_tmp234;
# 1642
if((int)Cyc_CfFlowInfo_initlevel(fenv,d2,r2)!= (int)Cyc_CfFlowInfo_AllIL){
({void*_tmp235=0U;({unsigned _tmp5AF=e1->loc;struct _fat_ptr _tmp5AE=({const char*_tmp236="expression may not be initialized";_tag_fat(_tmp236,sizeof(char),34U);});Cyc_CfFlowInfo_aerr(_tmp5AF,_tmp5AE,_tag_fat(_tmp235,sizeof(void*),0U));});});
return({struct _tuple17 _tmp48F;({union Cyc_CfFlowInfo_FlowInfo _tmp5B0=Cyc_CfFlowInfo_BottomFL();_tmp48F.f1=_tmp5B0;}),_tmp48F.f2=fenv->unknown_all;_tmp48F;});}}}_LL1B4:;}{
# 1647
union Cyc_CfFlowInfo_FlowInfo newflow=Cyc_CfFlowInfo_join_flow(fenv,f1,f2);
if(Cyc_CfFlowInfo_flow_lessthan_approx(newflow,f1))
break;
# 1648
f1=newflow;}}}}
# 1652
return({struct _tuple17 _tmp491;_tmp491.f1=f1,_tmp491.f2=fenv->notzeroall;_tmp491;});}_LL1A0:;}}}}_LL19B:;}}case 28U: _LL74: _tmpE9=((struct Cyc_Absyn_ComprehensionNoinit_e_Absyn_Raw_exp_struct*)_tmpE7)->f1;_tmpEA=(void*)((struct Cyc_Absyn_ComprehensionNoinit_e_Absyn_Raw_exp_struct*)_tmpE7)->f2;_tmpEB=((struct Cyc_Absyn_ComprehensionNoinit_e_Absyn_Raw_exp_struct*)_tmpE7)->f3;_LL75: {struct Cyc_Absyn_Exp*exp=_tmpE9;void*typ=_tmpEA;int iszeroterm=_tmpEB;
# 1658
void*root=(void*)({struct Cyc_CfFlowInfo_MallocPt_CfFlowInfo_Root_struct*_tmp23D=_cycalloc(sizeof(*_tmp23D));_tmp23D->tag=1U,_tmp23D->f1=exp,_tmp23D->f2=(void*)_check_null(e->topt);_tmp23D;});
struct Cyc_CfFlowInfo_Place*place=({struct Cyc_CfFlowInfo_Place*_tmp23C=_cycalloc(sizeof(*_tmp23C));_tmp23C->root=root,_tmp23C->path=0;_tmp23C;});
void*rval=(void*)({struct Cyc_CfFlowInfo_AddressOf_CfFlowInfo_AbsRVal_struct*_tmp23B=_cycalloc(sizeof(*_tmp23B));_tmp23B->tag=4U,_tmp23B->f1=place;_tmp23B;});
void*place_val;
# 1666
place_val=Cyc_CfFlowInfo_typ_to_absrval(fenv,typ,0,fenv->unknown_none);{
union Cyc_CfFlowInfo_FlowInfo outflow;
struct _tuple17 _stmttmp56=Cyc_NewControlFlow_anal_Rexp(env,1,inflow,exp);union Cyc_CfFlowInfo_FlowInfo _tmp237;_LL1BA: _tmp237=_stmttmp56.f1;_LL1BB: {union Cyc_CfFlowInfo_FlowInfo f=_tmp237;
outflow=f;{
union Cyc_CfFlowInfo_FlowInfo _tmp238=outflow;struct Cyc_List_List*_tmp23A;struct Cyc_Dict_Dict _tmp239;if((_tmp238.BottomFL).tag == 1){_LL1BD: _LL1BE:
 return({struct _tuple17 _tmp492;_tmp492.f1=outflow,_tmp492.f2=rval;_tmp492;});}else{_LL1BF: _tmp239=((_tmp238.ReachableFL).val).f1;_tmp23A=((_tmp238.ReachableFL).val).f2;_LL1C0: {struct Cyc_Dict_Dict d2=_tmp239;struct Cyc_List_List*relns=_tmp23A;
# 1673
return({struct _tuple17 _tmp493;({union Cyc_CfFlowInfo_FlowInfo _tmp5B2=({struct Cyc_Dict_Dict _tmp5B1=Cyc_Dict_insert(d2,root,place_val);Cyc_CfFlowInfo_ReachableFL(_tmp5B1,relns);});_tmp493.f1=_tmp5B2;}),_tmp493.f2=rval;_tmp493;});}}_LL1BC:;}}}}case 38U: _LL76: _tmpE8=((struct Cyc_Absyn_StmtExp_e_Absyn_Raw_exp_struct*)_tmpE7)->f1;_LL77: {struct Cyc_Absyn_Stmt*s=_tmpE8;
# 1677
struct _tuple18 rval_opt=({struct _tuple18 _tmp495;_tmp495.f1=(env->fenv)->unknown_all,_tmp495.f2=copy_ctxt;_tmp495;});
union Cyc_CfFlowInfo_FlowInfo f=Cyc_NewControlFlow_anal_stmt(env,inflow,s,& rval_opt);
return({struct _tuple17 _tmp494;_tmp494.f1=f,_tmp494.f2=rval_opt.f1;_tmp494;});}case 37U: _LL7A: _LL7B:
# 1682
 goto _LL7D;case 25U: _LL7C: _LL7D:
 goto _LL7F;default: _LL7E: _LL7F:
# 1685
({void*_tmp23E=0U;({int(*_tmp5B4)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp240)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmp240;});struct _fat_ptr _tmp5B3=({const char*_tmp23F="anal_Rexp, unexpected exp form";_tag_fat(_tmp23F,sizeof(char),31U);});_tmp5B4(_tmp5B3,_tag_fat(_tmp23E,sizeof(void*),0U));});});}_LLD:;}}
# 548 "new_control_flow.cyc"
static struct _tuple19 Cyc_NewControlFlow_anal_derefL(struct Cyc_NewControlFlow_AnalEnv*env,union Cyc_CfFlowInfo_FlowInfo inflow,struct Cyc_Absyn_Exp*e,union Cyc_CfFlowInfo_FlowInfo f,void*r,int passthrough_consumes,int expanded_place_possibly_null,struct Cyc_List_List*path){
# 1706 "new_control_flow.cyc"
struct Cyc_CfFlowInfo_FlowEnv*fenv=env->fenv;
void*_stmttmp57=Cyc_Tcutil_compress((void*)_check_null(e->topt));void*_tmp241=_stmttmp57;void*_tmp242;if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp241)->tag == 3U){_LL1: _tmp242=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp241)->f1).ptr_atts).bounds;_LL2: {void*bd=_tmp242;
# 1709
union Cyc_CfFlowInfo_FlowInfo _tmp243=f;struct Cyc_List_List*_tmp245;struct Cyc_Dict_Dict _tmp244;if((_tmp243.BottomFL).tag == 1){_LL6: _LL7:
 return({struct _tuple19 _tmp496;_tmp496.f1=f,({union Cyc_CfFlowInfo_AbsLVal _tmp5B5=Cyc_CfFlowInfo_UnknownL();_tmp496.f2=_tmp5B5;});_tmp496;});}else{_LL8: _tmp244=((_tmp243.ReachableFL).val).f1;_tmp245=((_tmp243.ReachableFL).val).f2;_LL9: {struct Cyc_Dict_Dict outdict=_tmp244;struct Cyc_List_List*relns=_tmp245;
# 1713
struct _tuple16 _stmttmp58=Cyc_CfFlowInfo_unname_rval(r);struct Cyc_List_List*_tmp247;void*_tmp246;_LLB: _tmp246=_stmttmp58.f1;_tmp247=_stmttmp58.f2;_LLC: {void*r=_tmp246;struct Cyc_List_List*names=_tmp247;
retry: {void*_tmp248=r;void*_tmp249;enum Cyc_CfFlowInfo_InitLevel _tmp24A;struct Cyc_List_List*_tmp24C;void*_tmp24B;void*_tmp24D;switch(*((int*)_tmp248)){case 8U: _LLE: _LLF:
# 1716
({void*_tmp24E=0U;({int(*_tmp5B7)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp250)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmp250;});struct _fat_ptr _tmp5B6=({const char*_tmp24F="named location in anal_derefL";_tag_fat(_tmp24F,sizeof(char),30U);});_tmp5B7(_tmp5B6,_tag_fat(_tmp24E,sizeof(void*),0U));});});case 1U: _LL10: _LL11:
# 1718
({void*_tmp5B8=(void*)({struct Cyc_CfFlowInfo_NotZero_Absyn_AbsynAnnot_struct*_tmp251=_cycalloc(sizeof(*_tmp251));_tmp251->tag=Cyc_CfFlowInfo_NotZero,_tmp251->f1=relns;_tmp251;});e->annot=_tmp5B8;});goto _LLD;case 5U: _LL12: _tmp24D=(void*)((struct Cyc_CfFlowInfo_UniquePtr_CfFlowInfo_AbsRVal_struct*)_tmp248)->f1;_LL13: {void*rv=_tmp24D;
# 1721
if(expanded_place_possibly_null)
# 1724
({void*_tmp5B9=(void*)({struct Cyc_CfFlowInfo_UnknownZ_Absyn_AbsynAnnot_struct*_tmp252=_cycalloc(sizeof(*_tmp252));_tmp252->tag=Cyc_CfFlowInfo_UnknownZ,_tmp252->f1=relns;_tmp252;});e->annot=_tmp5B9;});else{
# 1727
void*_stmttmp59=e->annot;void*_tmp253=_stmttmp59;if(((struct Cyc_CfFlowInfo_UnknownZ_Absyn_AbsynAnnot_struct*)_tmp253)->tag == Cyc_CfFlowInfo_UnknownZ){_LL1F: _LL20:
# 1731
({void*_tmp5BA=(void*)({struct Cyc_CfFlowInfo_UnknownZ_Absyn_AbsynAnnot_struct*_tmp254=_cycalloc(sizeof(*_tmp254));_tmp254->tag=Cyc_CfFlowInfo_UnknownZ,_tmp254->f1=relns;_tmp254;});e->annot=_tmp5BA;});
goto _LL1E;}else{_LL21: _LL22:
# 1734
({void*_tmp5BB=(void*)({struct Cyc_CfFlowInfo_NotZero_Absyn_AbsynAnnot_struct*_tmp255=_cycalloc(sizeof(*_tmp255));_tmp255->tag=Cyc_CfFlowInfo_NotZero,_tmp255->f1=relns;_tmp255;});e->annot=_tmp5BB;});
goto _LL1E;}_LL1E:;}
# 1739
if(Cyc_Tcutil_is_bound_one(bd))return({struct _tuple19 _tmp497;_tmp497.f1=f,({union Cyc_CfFlowInfo_AbsLVal _tmp5BC=Cyc_CfFlowInfo_UnknownL();_tmp497.f2=_tmp5BC;});_tmp497;});goto _LLD;}case 4U: _LL14: _tmp24B=(((struct Cyc_CfFlowInfo_AddressOf_CfFlowInfo_AbsRVal_struct*)_tmp248)->f1)->root;_tmp24C=(((struct Cyc_CfFlowInfo_AddressOf_CfFlowInfo_AbsRVal_struct*)_tmp248)->f1)->path;_LL15: {void*root=_tmp24B;struct Cyc_List_List*path2=_tmp24C;
# 1743
({void*_tmp5BD=(void*)({struct Cyc_CfFlowInfo_NotZero_Absyn_AbsynAnnot_struct*_tmp256=_cycalloc(sizeof(*_tmp256));_tmp256->tag=Cyc_CfFlowInfo_NotZero,_tmp256->f1=relns;_tmp256;});e->annot=_tmp5BD;});
if(Cyc_Tcutil_is_bound_one(bd))
return({struct _tuple19 _tmp498;_tmp498.f1=f,({union Cyc_CfFlowInfo_AbsLVal _tmp5BF=Cyc_CfFlowInfo_PlaceL(({struct Cyc_CfFlowInfo_Place*_tmp257=_cycalloc(sizeof(*_tmp257));_tmp257->root=root,({struct Cyc_List_List*_tmp5BE=Cyc_List_append(path2,path);_tmp257->path=_tmp5BE;});_tmp257;}));_tmp498.f2=_tmp5BF;});_tmp498;});
# 1744
goto _LLD;}case 0U: _LL16: _LL17:
# 1748
 e->annot=(void*)& Cyc_CfFlowInfo_IsZero_val;
return({struct _tuple19 _tmp499;({union Cyc_CfFlowInfo_FlowInfo _tmp5C1=Cyc_CfFlowInfo_BottomFL();_tmp499.f1=_tmp5C1;}),({union Cyc_CfFlowInfo_AbsLVal _tmp5C0=Cyc_CfFlowInfo_UnknownL();_tmp499.f2=_tmp5C0;});_tmp499;});case 2U: _LL18: _tmp24A=((struct Cyc_CfFlowInfo_UnknownR_CfFlowInfo_AbsRVal_struct*)_tmp248)->f1;_LL19: {enum Cyc_CfFlowInfo_InitLevel il=_tmp24A;
# 1753
if(Cyc_Tcutil_is_bound_one(bd))
f=Cyc_NewControlFlow_notzero(env,inflow,f,e,il,names);
# 1753
({void*_tmp5C2=(void*)({struct Cyc_CfFlowInfo_UnknownZ_Absyn_AbsynAnnot_struct*_tmp258=_cycalloc(sizeof(*_tmp258));
# 1755
_tmp258->tag=Cyc_CfFlowInfo_UnknownZ,_tmp258->f1=relns;_tmp258;});
# 1753
e->annot=_tmp5C2;});
# 1756
goto _LLD;}case 7U: _LL1A: _tmp249=(void*)((struct Cyc_CfFlowInfo_Consumed_CfFlowInfo_AbsRVal_struct*)_tmp248)->f3;_LL1B: {void*r2=_tmp249;
# 1758
if(passthrough_consumes){
r=r2;goto retry;}
# 1758
goto _LL1D;}default: _LL1C: _LL1D:
# 1763
({void*_tmp5C3=(void*)({struct Cyc_CfFlowInfo_UnknownZ_Absyn_AbsynAnnot_struct*_tmp259=_cycalloc(sizeof(*_tmp259));_tmp259->tag=Cyc_CfFlowInfo_UnknownZ,_tmp259->f1=relns;_tmp259;});e->annot=_tmp5C3;});
goto _LLD;}_LLD:;}
# 1766
if((int)Cyc_CfFlowInfo_initlevel(fenv,outdict,r)== (int)Cyc_CfFlowInfo_NoneIL){
struct _tuple16 _stmttmp5A=Cyc_CfFlowInfo_unname_rval(r);void*_tmp25A;_LL24: _tmp25A=_stmttmp5A.f1;_LL25: {void*r=_tmp25A;
void*_tmp25B=r;if(((struct Cyc_CfFlowInfo_Consumed_CfFlowInfo_AbsRVal_struct*)_tmp25B)->tag == 7U){_LL27: _LL28:
# 1770
({void*_tmp25C=0U;({unsigned _tmp5C5=e->loc;struct _fat_ptr _tmp5C4=({const char*_tmp25D="attempt to dereference an alias-free that has already been copied";_tag_fat(_tmp25D,sizeof(char),66U);});Cyc_CfFlowInfo_aerr(_tmp5C5,_tmp5C4,_tag_fat(_tmp25C,sizeof(void*),0U));});});
goto _LL26;}else{_LL29: _LL2A:
# 1773
({void*_tmp25E=0U;({unsigned _tmp5C7=e->loc;struct _fat_ptr _tmp5C6=({const char*_tmp25F="dereference of possibly uninitialized pointer";_tag_fat(_tmp25F,sizeof(char),46U);});Cyc_CfFlowInfo_aerr(_tmp5C7,_tmp5C6,_tag_fat(_tmp25E,sizeof(void*),0U));});});
goto _LL26;}_LL26:;}}
# 1766
return({struct _tuple19 _tmp49A;_tmp49A.f1=f,({
# 1777
union Cyc_CfFlowInfo_AbsLVal _tmp5C8=Cyc_CfFlowInfo_UnknownL();_tmp49A.f2=_tmp5C8;});_tmp49A;});}}}_LL5:;}}else{_LL3: _LL4:
# 1779
({void*_tmp260=0U;({int(*_tmp5CA)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp262)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmp262;});struct _fat_ptr _tmp5C9=({const char*_tmp261="left deref of non-pointer-type";_tag_fat(_tmp261,sizeof(char),31U);});_tmp5CA(_tmp5C9,_tag_fat(_tmp260,sizeof(void*),0U));});});}_LL0:;}
# 548 "new_control_flow.cyc"
static struct Cyc_CfFlowInfo_Place*Cyc_NewControlFlow_make_expanded_place(struct Cyc_CfFlowInfo_Place*p,struct Cyc_List_List*path){
# 1786 "new_control_flow.cyc"
struct Cyc_List_List*_tmp264;void*_tmp263;_LL1: _tmp263=p->root;_tmp264=p->path;_LL2: {void*proot=_tmp263;struct Cyc_List_List*ppath=_tmp264;
path=({struct Cyc_List_List*_tmp266=_cycalloc(sizeof(*_tmp266));({void*_tmp5CB=(void*)({struct Cyc_CfFlowInfo_Star_CfFlowInfo_PathCon_struct*_tmp265=_cycalloc(sizeof(*_tmp265));_tmp265->tag=1U;_tmp265;});_tmp266->hd=_tmp5CB;}),_tmp266->tl=path;_tmp266;});
return({struct Cyc_CfFlowInfo_Place*_tmp267=_cycalloc(sizeof(*_tmp267));_tmp267->root=proot,({struct Cyc_List_List*_tmp5CC=Cyc_List_append(ppath,path);_tmp267->path=_tmp5CC;});_tmp267;});}}
# 548 "new_control_flow.cyc"
static struct _tuple19 Cyc_NewControlFlow_anal_Lexp_rec(struct Cyc_NewControlFlow_AnalEnv*env,union Cyc_CfFlowInfo_FlowInfo inflow,int expand_unique,int passthrough_consumes,struct Cyc_Absyn_Exp*e,struct Cyc_List_List*path){
# 1799 "new_control_flow.cyc"
struct Cyc_Dict_Dict d;
struct Cyc_CfFlowInfo_FlowEnv*fenv=env->fenv;
{union Cyc_CfFlowInfo_FlowInfo _tmp268=inflow;struct Cyc_List_List*_tmp26A;struct Cyc_Dict_Dict _tmp269;if((_tmp268.BottomFL).tag == 1){_LL1: _LL2:
 return({struct _tuple19 _tmp49B;({union Cyc_CfFlowInfo_FlowInfo _tmp5CE=Cyc_CfFlowInfo_BottomFL();_tmp49B.f1=_tmp5CE;}),({union Cyc_CfFlowInfo_AbsLVal _tmp5CD=Cyc_CfFlowInfo_UnknownL();_tmp49B.f2=_tmp5CD;});_tmp49B;});}else{_LL3: _tmp269=((_tmp268.ReachableFL).val).f1;_tmp26A=((_tmp268.ReachableFL).val).f2;_LL4: {struct Cyc_Dict_Dict d2=_tmp269;struct Cyc_List_List*relns=_tmp26A;
# 1804
d=d2;}}_LL0:;}{
# 1808
void*_stmttmp5B=e->r;void*_tmp26B=_stmttmp5B;struct _fat_ptr*_tmp26D;struct Cyc_Absyn_Exp*_tmp26C;struct Cyc_Absyn_Exp*_tmp26F;struct Cyc_Absyn_Exp*_tmp26E;struct Cyc_Absyn_Exp*_tmp270;struct _fat_ptr*_tmp272;struct Cyc_Absyn_Exp*_tmp271;struct Cyc_Absyn_Vardecl*_tmp273;struct Cyc_Absyn_Vardecl*_tmp274;struct Cyc_Absyn_Vardecl*_tmp275;struct Cyc_Absyn_Vardecl*_tmp276;struct Cyc_Absyn_Exp*_tmp277;struct Cyc_Absyn_Exp*_tmp278;struct Cyc_Absyn_Exp*_tmp279;switch(*((int*)_tmp26B)){case 14U: _LL6: _tmp279=((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp26B)->f2;_LL7: {struct Cyc_Absyn_Exp*e1=_tmp279;
_tmp278=e1;goto _LL9;}case 12U: _LL8: _tmp278=((struct Cyc_Absyn_NoInstantiate_e_Absyn_Raw_exp_struct*)_tmp26B)->f1;_LL9: {struct Cyc_Absyn_Exp*e1=_tmp278;
_tmp277=e1;goto _LLB;}case 13U: _LLA: _tmp277=((struct Cyc_Absyn_Instantiate_e_Absyn_Raw_exp_struct*)_tmp26B)->f1;_LLB: {struct Cyc_Absyn_Exp*e1=_tmp277;
return Cyc_NewControlFlow_anal_Lexp_rec(env,inflow,expand_unique,passthrough_consumes,e1,path);}case 1U: switch(*((int*)((struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct*)_tmp26B)->f1)){case 4U: _LLC: _tmp276=((struct Cyc_Absyn_Local_b_Absyn_Binding_struct*)((struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct*)_tmp26B)->f1)->f1;_LLD: {struct Cyc_Absyn_Vardecl*vd=_tmp276;
# 1814
if((int)vd->sc == (int)Cyc_Absyn_Static)
return({struct _tuple19 _tmp49C;_tmp49C.f1=inflow,({union Cyc_CfFlowInfo_AbsLVal _tmp5CF=Cyc_CfFlowInfo_UnknownL();_tmp49C.f2=_tmp5CF;});_tmp49C;});
# 1814
_tmp275=vd;goto _LLF;}case 3U: _LLE: _tmp275=((struct Cyc_Absyn_Param_b_Absyn_Binding_struct*)((struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct*)_tmp26B)->f1)->f1;_LLF: {struct Cyc_Absyn_Vardecl*vd=_tmp275;
# 1817
_tmp274=vd;goto _LL11;}case 5U: _LL10: _tmp274=((struct Cyc_Absyn_Pat_b_Absyn_Binding_struct*)((struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct*)_tmp26B)->f1)->f1;_LL11: {struct Cyc_Absyn_Vardecl*vd=_tmp274;
# 1819
return({struct _tuple19 _tmp49D;_tmp49D.f1=inflow,({union Cyc_CfFlowInfo_AbsLVal _tmp5D1=Cyc_CfFlowInfo_PlaceL(({struct Cyc_CfFlowInfo_Place*_tmp27B=_cycalloc(sizeof(*_tmp27B));({void*_tmp5D0=(void*)({struct Cyc_CfFlowInfo_VarRoot_CfFlowInfo_Root_struct*_tmp27A=_cycalloc(sizeof(*_tmp27A));_tmp27A->tag=0U,_tmp27A->f1=vd;_tmp27A;});_tmp27B->root=_tmp5D0;}),_tmp27B->path=path;_tmp27B;}));_tmp49D.f2=_tmp5D1;});_tmp49D;});}case 1U: _LL12: _tmp273=((struct Cyc_Absyn_Global_b_Absyn_Binding_struct*)((struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct*)_tmp26B)->f1)->f1;_LL13: {struct Cyc_Absyn_Vardecl*vd=_tmp273;
# 1821
return({struct _tuple19 _tmp49E;_tmp49E.f1=inflow,({union Cyc_CfFlowInfo_AbsLVal _tmp5D2=Cyc_CfFlowInfo_UnknownL();_tmp49E.f2=_tmp5D2;});_tmp49E;});}default: goto _LL1C;}case 22U: _LL14: _tmp271=((struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct*)_tmp26B)->f1;_tmp272=((struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct*)_tmp26B)->f2;_LL15: {struct Cyc_Absyn_Exp*e1=_tmp271;struct _fat_ptr*f=_tmp272;
# 1824
{void*_stmttmp5C=Cyc_Tcutil_compress((void*)_check_null(e1->topt));void*_tmp27C=_stmttmp5C;void*_tmp27D;if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp27C)->tag == 3U){_LL1F: _tmp27D=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp27C)->f1).elt_type;_LL20: {void*t2=_tmp27D;
# 1826
if(!Cyc_Absyn_is_nontagged_nonrequire_union_type(t2)){
Cyc_NewControlFlow_check_union_requires(e1->loc,t2,f,inflow);
path=({struct Cyc_List_List*_tmp27F=_cycalloc(sizeof(*_tmp27F));({void*_tmp5D4=(void*)({struct Cyc_CfFlowInfo_Dot_CfFlowInfo_PathCon_struct*_tmp27E=_cycalloc(sizeof(*_tmp27E));_tmp27E->tag=0U,({int _tmp5D3=Cyc_CfFlowInfo_get_field_index(t2,f);_tmp27E->f1=_tmp5D3;});_tmp27E;});_tmp27F->hd=_tmp5D4;}),_tmp27F->tl=path;_tmp27F;});}
# 1826
goto _LL1E;}}else{_LL21: _LL22:
# 1831
({void*_tmp280=0U;({int(*_tmp5D6)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp282)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmp282;});struct _fat_ptr _tmp5D5=({const char*_tmp281="anal_Lexp: AggrArrow ptr";_tag_fat(_tmp281,sizeof(char),25U);});_tmp5D6(_tmp5D5,_tag_fat(_tmp280,sizeof(void*),0U));});});}_LL1E:;}
# 1833
_tmp270=e1;goto _LL17;}case 20U: _LL16: _tmp270=((struct Cyc_Absyn_Deref_e_Absyn_Raw_exp_struct*)_tmp26B)->f1;_LL17: {struct Cyc_Absyn_Exp*e1=_tmp270;
# 1837
if(Cyc_Tcutil_is_noalias_pointer((void*)_check_null(e1->topt),1)){
# 1839
struct _tuple19 _stmttmp5D=Cyc_NewControlFlow_anal_Lexp(env,inflow,expand_unique,passthrough_consumes,e1);union Cyc_CfFlowInfo_AbsLVal _tmp284;union Cyc_CfFlowInfo_FlowInfo _tmp283;_LL24: _tmp283=_stmttmp5D.f1;_tmp284=_stmttmp5D.f2;_LL25: {union Cyc_CfFlowInfo_FlowInfo f=_tmp283;union Cyc_CfFlowInfo_AbsLVal lval=_tmp284;
struct _tuple19 _stmttmp5E=({struct _tuple19 _tmp4A2;_tmp4A2.f1=f,_tmp4A2.f2=lval;_tmp4A2;});struct _tuple19 _tmp285=_stmttmp5E;struct Cyc_CfFlowInfo_Place*_tmp288;struct Cyc_List_List*_tmp287;struct Cyc_Dict_Dict _tmp286;if(((_tmp285.f1).ReachableFL).tag == 2){if(((_tmp285.f2).PlaceL).tag == 1){_LL27: _tmp286=(((_tmp285.f1).ReachableFL).val).f1;_tmp287=(((_tmp285.f1).ReachableFL).val).f2;_tmp288=((_tmp285.f2).PlaceL).val;_LL28: {struct Cyc_Dict_Dict fd=_tmp286;struct Cyc_List_List*r=_tmp287;struct Cyc_CfFlowInfo_Place*p=_tmp288;
# 1842
void*old_rval=Cyc_CfFlowInfo_lookup_place(fd,p);
struct _tuple16 _stmttmp5F=Cyc_CfFlowInfo_unname_rval(old_rval);struct Cyc_List_List*_tmp28A;void*_tmp289;_LL2C: _tmp289=_stmttmp5F.f1;_tmp28A=_stmttmp5F.f2;_LL2D: {void*rval=_tmp289;struct Cyc_List_List*names=_tmp28A;
if(expand_unique){
int possibly_null=1;
void*_tmp28B=rval;void*_tmp28D;struct Cyc_Absyn_Vardecl*_tmp28C;switch(*((int*)_tmp28B)){case 8U: _LL2F: _tmp28C=((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmp28B)->f1;_tmp28D=(void*)((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmp28B)->f2;_LL30: {struct Cyc_Absyn_Vardecl*n=_tmp28C;void*r=_tmp28D;
# 1848
({void*_tmp28E=0U;({int(*_tmp5D8)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp290)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmp290;});struct _fat_ptr _tmp5D7=({const char*_tmp28F="bad named location in anal_Lexp:deref";_tag_fat(_tmp28F,sizeof(char),38U);});_tmp5D8(_tmp5D7,_tag_fat(_tmp28E,sizeof(void*),0U));});});}case 7U: switch(*((int*)((struct Cyc_CfFlowInfo_Consumed_CfFlowInfo_AbsRVal_struct*)_tmp28B)->f3)){case 4U: _LL31: _LL32:
 goto _LL34;case 5U: _LL35: _LL36:
# 1852
 goto _LL38;default: goto _LL3B;}case 4U: _LL33: _LL34:
# 1851
 return Cyc_NewControlFlow_anal_derefL(env,f,e1,f,old_rval,passthrough_consumes,0,path);case 5U: _LL37: _LL38: {
# 1856
struct _tuple19 _stmttmp60=Cyc_NewControlFlow_anal_derefL(env,f,e1,f,old_rval,passthrough_consumes,0,path);union Cyc_CfFlowInfo_FlowInfo _tmp291;_LL3E: _tmp291=_stmttmp60.f1;_LL3F: {union Cyc_CfFlowInfo_FlowInfo f=_tmp291;
return({struct _tuple19 _tmp49F;_tmp49F.f1=f,({union Cyc_CfFlowInfo_AbsLVal _tmp5D9=Cyc_CfFlowInfo_PlaceL(Cyc_NewControlFlow_make_expanded_place(p,path));_tmp49F.f2=_tmp5D9;});_tmp49F;});}}case 1U: _LL39: _LL3A:
# 1859
 possibly_null=0;goto _LL3C;default: _LL3B: _LL3C: {
# 1866
enum Cyc_CfFlowInfo_InitLevel il=Cyc_CfFlowInfo_initlevel(fenv,fd,rval);
void*leaf=(int)il == (int)1U?fenv->unknown_all: fenv->unknown_none;
void*res=(void*)({struct Cyc_CfFlowInfo_UniquePtr_CfFlowInfo_AbsRVal_struct*_tmp294=_cycalloc(sizeof(*_tmp294));_tmp294->tag=5U,({void*_tmp5DC=({struct Cyc_CfFlowInfo_FlowEnv*_tmp5DB=fenv;void*_tmp5DA=Cyc_Tcutil_pointer_elt_type((void*)_check_null(e1->topt));Cyc_CfFlowInfo_typ_to_absrval(_tmp5DB,_tmp5DA,0,leaf);});_tmp294->f1=_tmp5DC;});_tmp294;});
for(0;names != 0;names=names->tl){
res=(void*)({struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*_tmp292=_cycalloc(sizeof(*_tmp292));_tmp292->tag=8U,_tmp292->f1=(struct Cyc_Absyn_Vardecl*)names->hd,_tmp292->f2=res;_tmp292;});}
fd=Cyc_CfFlowInfo_assign_place(fenv,e->loc,fd,p,res);{
union Cyc_CfFlowInfo_FlowInfo outflow=Cyc_CfFlowInfo_ReachableFL(fd,r);
# 1879
struct _tuple19 _stmttmp61=Cyc_NewControlFlow_anal_derefL(env,outflow,e1,outflow,res,passthrough_consumes,possibly_null,path);union Cyc_CfFlowInfo_FlowInfo _tmp293;_LL41: _tmp293=_stmttmp61.f1;_LL42: {union Cyc_CfFlowInfo_FlowInfo f=_tmp293;
# 1881
return({struct _tuple19 _tmp4A0;_tmp4A0.f1=f,({union Cyc_CfFlowInfo_AbsLVal _tmp5DD=Cyc_CfFlowInfo_PlaceL(Cyc_NewControlFlow_make_expanded_place(p,path));_tmp4A0.f2=_tmp5DD;});_tmp4A0;});}}}}_LL2E:;}else{
# 1885
void*_tmp295=rval;switch(*((int*)_tmp295)){case 7U: if(((struct Cyc_CfFlowInfo_UniquePtr_CfFlowInfo_AbsRVal_struct*)((struct Cyc_CfFlowInfo_Consumed_CfFlowInfo_AbsRVal_struct*)_tmp295)->f3)->tag == 5U){_LL44: _LL45:
 goto _LL47;}else{goto _LL48;}case 5U: _LL46: _LL47: {
# 1890
struct _tuple19 _stmttmp62=Cyc_NewControlFlow_anal_derefL(env,f,e1,f,old_rval,passthrough_consumes,0,path);union Cyc_CfFlowInfo_FlowInfo _tmp296;_LL4B: _tmp296=_stmttmp62.f1;_LL4C: {union Cyc_CfFlowInfo_FlowInfo f=_tmp296;
return({struct _tuple19 _tmp4A1;_tmp4A1.f1=f,({union Cyc_CfFlowInfo_AbsLVal _tmp5DE=Cyc_CfFlowInfo_PlaceL(Cyc_NewControlFlow_make_expanded_place(p,path));_tmp4A1.f2=_tmp5DE;});_tmp4A1;});}}default: _LL48: _LL49:
# 1894
 goto _LL43;}_LL43:;}
# 1897
goto _LL26;}}}else{goto _LL29;}}else{_LL29: _LL2A:
 goto _LL26;}_LL26:;}}{
# 1902
struct _tuple17 _stmttmp63=Cyc_NewControlFlow_anal_Rexp(env,0,inflow,e1);void*_tmp298;union Cyc_CfFlowInfo_FlowInfo _tmp297;_LL4E: _tmp297=_stmttmp63.f1;_tmp298=_stmttmp63.f2;_LL4F: {union Cyc_CfFlowInfo_FlowInfo f=_tmp297;void*r=_tmp298;
# 1904
return Cyc_NewControlFlow_anal_derefL(env,inflow,e1,f,r,passthrough_consumes,0,path);}}}case 23U: _LL18: _tmp26E=((struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct*)_tmp26B)->f1;_tmp26F=((struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct*)_tmp26B)->f2;_LL19: {struct Cyc_Absyn_Exp*e1=_tmp26E;struct Cyc_Absyn_Exp*e2=_tmp26F;
# 1907
void*_stmttmp64=Cyc_Tcutil_compress((void*)_check_null(e1->topt));void*_tmp299=_stmttmp64;switch(*((int*)_tmp299)){case 6U: _LL51: _LL52: {
# 1909
unsigned fld=(Cyc_Evexp_eval_const_uint_exp(e2)).f1;
return({struct Cyc_NewControlFlow_AnalEnv*_tmp5E4=env;union Cyc_CfFlowInfo_FlowInfo _tmp5E3=inflow;int _tmp5E2=expand_unique;int _tmp5E1=passthrough_consumes;struct Cyc_Absyn_Exp*_tmp5E0=e1;Cyc_NewControlFlow_anal_Lexp_rec(_tmp5E4,_tmp5E3,_tmp5E2,_tmp5E1,_tmp5E0,({struct Cyc_List_List*_tmp29B=_cycalloc(sizeof(*_tmp29B));
({void*_tmp5DF=(void*)({struct Cyc_CfFlowInfo_Dot_CfFlowInfo_PathCon_struct*_tmp29A=_cycalloc(sizeof(*_tmp29A));_tmp29A->tag=0U,_tmp29A->f1=(int)fld;_tmp29A;});_tmp29B->hd=_tmp5DF;}),_tmp29B->tl=path;_tmp29B;}));});}case 3U: _LL53: _LL54: {
# 1913
struct _tuple24 _stmttmp65=({struct Cyc_NewControlFlow_AnalEnv*_tmp5E6=env;union Cyc_CfFlowInfo_FlowInfo _tmp5E5=inflow;Cyc_NewControlFlow_anal_Rexps(_tmp5E6,_tmp5E5,({struct Cyc_Absyn_Exp*_tmp2A6[2U];_tmp2A6[0]=e1,_tmp2A6[1]=e2;Cyc_List_list(_tag_fat(_tmp2A6,sizeof(struct Cyc_Absyn_Exp*),2U));}),0,1);});struct Cyc_List_List*_tmp29D;union Cyc_CfFlowInfo_FlowInfo _tmp29C;_LL58: _tmp29C=_stmttmp65.f1;_tmp29D=_stmttmp65.f2;_LL59: {union Cyc_CfFlowInfo_FlowInfo f=_tmp29C;struct Cyc_List_List*rvals=_tmp29D;
union Cyc_CfFlowInfo_FlowInfo f2=f;
{union Cyc_CfFlowInfo_FlowInfo _tmp29E=f;struct Cyc_List_List*_tmp2A0;struct Cyc_Dict_Dict _tmp29F;if((_tmp29E.ReachableFL).tag == 2){_LL5B: _tmp29F=((_tmp29E.ReachableFL).val).f1;_tmp2A0=((_tmp29E.ReachableFL).val).f2;_LL5C: {struct Cyc_Dict_Dict d2=_tmp29F;struct Cyc_List_List*relns=_tmp2A0;
# 1917
if((int)Cyc_CfFlowInfo_initlevel(fenv,d2,(void*)((struct Cyc_List_List*)_check_null(((struct Cyc_List_List*)_check_null(rvals))->tl))->hd)== (int)Cyc_CfFlowInfo_NoneIL)
({void*_tmp2A1=0U;({unsigned _tmp5E8=e2->loc;struct _fat_ptr _tmp5E7=({const char*_tmp2A2="expression may not be initialized";_tag_fat(_tmp2A2,sizeof(char),34U);});Cyc_CfFlowInfo_aerr(_tmp5E8,_tmp5E7,_tag_fat(_tmp2A1,sizeof(void*),0U));});});{
# 1917
struct Cyc_List_List*new_relns=
# 1919
Cyc_NewControlFlow_add_subscript_reln(relns,e1,e2);
if(relns != new_relns)
f2=Cyc_CfFlowInfo_ReachableFL(d2,new_relns);
# 1920
goto _LL5A;}}}else{_LL5D: _LL5E:
# 1923
 goto _LL5A;}_LL5A:;}{
# 1926
struct _tuple19 _stmttmp66=Cyc_NewControlFlow_anal_derefL(env,inflow,e1,f,(void*)((struct Cyc_List_List*)_check_null(rvals))->hd,passthrough_consumes,0,path);union Cyc_CfFlowInfo_AbsLVal _tmp2A4;union Cyc_CfFlowInfo_FlowInfo _tmp2A3;_LL60: _tmp2A3=_stmttmp66.f1;_tmp2A4=_stmttmp66.f2;_LL61: {union Cyc_CfFlowInfo_FlowInfo f3=_tmp2A3;union Cyc_CfFlowInfo_AbsLVal r=_tmp2A4;
union Cyc_CfFlowInfo_FlowInfo _tmp2A5=f3;if((_tmp2A5.BottomFL).tag == 1){_LL63: _LL64:
 return({struct _tuple19 _tmp4A3;_tmp4A3.f1=f3,_tmp4A3.f2=r;_tmp4A3;});}else{_LL65: _LL66:
 return({struct _tuple19 _tmp4A4;_tmp4A4.f1=f2,_tmp4A4.f2=r;_tmp4A4;});}_LL62:;}}}}default: _LL55: _LL56:
# 1931
({void*_tmp2A7=0U;({int(*_tmp5EA)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp2A9)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmp2A9;});struct _fat_ptr _tmp5E9=({const char*_tmp2A8="anal_Lexp: Subscript -- bad type";_tag_fat(_tmp2A8,sizeof(char),33U);});_tmp5EA(_tmp5E9,_tag_fat(_tmp2A7,sizeof(void*),0U));});});}_LL50:;}case 21U: _LL1A: _tmp26C=((struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct*)_tmp26B)->f1;_tmp26D=((struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct*)_tmp26B)->f2;_LL1B: {struct Cyc_Absyn_Exp*e1=_tmp26C;struct _fat_ptr*fld=_tmp26D;
# 1935
void*e1_type=(void*)_check_null(e1->topt);
if(Cyc_Absyn_is_require_union_type(e1_type))
Cyc_NewControlFlow_check_union_requires(e1->loc,e1_type,fld,inflow);
# 1936
if(
# 1939
Cyc_Absyn_is_nontagged_nonrequire_union_type(e1_type))
return({struct _tuple19 _tmp4A5;_tmp4A5.f1=inflow,({union Cyc_CfFlowInfo_AbsLVal _tmp5EB=Cyc_CfFlowInfo_UnknownL();_tmp4A5.f2=_tmp5EB;});_tmp4A5;});
# 1936
return({
# 1942
struct Cyc_NewControlFlow_AnalEnv*_tmp5F2=env;union Cyc_CfFlowInfo_FlowInfo _tmp5F1=inflow;int _tmp5F0=expand_unique;int _tmp5EF=passthrough_consumes;struct Cyc_Absyn_Exp*_tmp5EE=e1;Cyc_NewControlFlow_anal_Lexp_rec(_tmp5F2,_tmp5F1,_tmp5F0,_tmp5EF,_tmp5EE,({struct Cyc_List_List*_tmp2AB=_cycalloc(sizeof(*_tmp2AB));
({void*_tmp5ED=(void*)({struct Cyc_CfFlowInfo_Dot_CfFlowInfo_PathCon_struct*_tmp2AA=_cycalloc(sizeof(*_tmp2AA));_tmp2AA->tag=0U,({int _tmp5EC=Cyc_CfFlowInfo_get_field_index(e1_type,fld);_tmp2AA->f1=_tmp5EC;});_tmp2AA;});_tmp2AB->hd=_tmp5ED;}),_tmp2AB->tl=path;_tmp2AB;}));});}default: _LL1C: _LL1D:
# 1946
 return({struct _tuple19 _tmp4A6;({union Cyc_CfFlowInfo_FlowInfo _tmp5F4=Cyc_CfFlowInfo_BottomFL();_tmp4A6.f1=_tmp5F4;}),({union Cyc_CfFlowInfo_AbsLVal _tmp5F3=Cyc_CfFlowInfo_UnknownL();_tmp4A6.f2=_tmp5F3;});_tmp4A6;});}_LL5:;}}
# 548 "new_control_flow.cyc"
static struct _tuple19 Cyc_NewControlFlow_anal_Lexp(struct Cyc_NewControlFlow_AnalEnv*env,union Cyc_CfFlowInfo_FlowInfo inflow,int expand_unique,int passthrough_consumes,struct Cyc_Absyn_Exp*e){
# 1954 "new_control_flow.cyc"
struct _tuple19 _stmttmp67=Cyc_NewControlFlow_anal_Lexp_rec(env,inflow,expand_unique,passthrough_consumes,e,0);union Cyc_CfFlowInfo_AbsLVal _tmp2AD;union Cyc_CfFlowInfo_FlowInfo _tmp2AC;_LL1: _tmp2AC=_stmttmp67.f1;_tmp2AD=_stmttmp67.f2;_LL2: {union Cyc_CfFlowInfo_FlowInfo f=_tmp2AC;union Cyc_CfFlowInfo_AbsLVal r=_tmp2AD;
return({struct _tuple19 _tmp4A7;_tmp4A7.f1=f,_tmp4A7.f2=r;_tmp4A7;});}}
# 548 "new_control_flow.cyc"
static union Cyc_CfFlowInfo_FlowInfo Cyc_NewControlFlow_expand_unique_places(struct Cyc_NewControlFlow_AnalEnv*env,union Cyc_CfFlowInfo_FlowInfo inflow,struct Cyc_List_List*es){
# 1964 "new_control_flow.cyc"
{struct Cyc_List_List*x=es;for(0;x != 0;x=x->tl){
# 1968
if(Cyc_NewControlFlow_is_local_var_rooted_path((struct Cyc_Absyn_Exp*)x->hd,1)&&
 Cyc_Tcutil_is_noalias_pointer_or_aggr((void*)_check_null(((struct Cyc_Absyn_Exp*)x->hd)->topt))){
# 1971
struct _tuple19 _stmttmp68=Cyc_NewControlFlow_anal_Lexp(env,inflow,1,0,(struct Cyc_Absyn_Exp*)x->hd);union Cyc_CfFlowInfo_FlowInfo _tmp2AE;_LL1: _tmp2AE=_stmttmp68.f1;_LL2: {union Cyc_CfFlowInfo_FlowInfo f=_tmp2AE;
inflow=f;}}}}
# 1976
return inflow;}struct _tuple28{enum Cyc_Absyn_Primop f1;void*f2;void*f3;};
# 548 "new_control_flow.cyc"
static struct _tuple20 Cyc_NewControlFlow_anal_primop_test(struct Cyc_NewControlFlow_AnalEnv*env,union Cyc_CfFlowInfo_FlowInfo inflow,enum Cyc_Absyn_Primop p,struct Cyc_List_List*es){
# 1984 "new_control_flow.cyc"
struct Cyc_CfFlowInfo_FlowEnv*fenv=env->fenv;
# 1987
struct _tuple17 _stmttmp69=Cyc_NewControlFlow_anal_Rexp(env,0,inflow,(struct Cyc_Absyn_Exp*)((struct Cyc_List_List*)_check_null(es))->hd);void*_tmp2B0;union Cyc_CfFlowInfo_FlowInfo _tmp2AF;_LL1: _tmp2AF=_stmttmp69.f1;_tmp2B0=_stmttmp69.f2;_LL2: {union Cyc_CfFlowInfo_FlowInfo f1=_tmp2AF;void*r1=_tmp2B0;
struct _tuple17 _stmttmp6A=Cyc_NewControlFlow_anal_Rexp(env,0,f1,(struct Cyc_Absyn_Exp*)((struct Cyc_List_List*)_check_null(es->tl))->hd);void*_tmp2B2;union Cyc_CfFlowInfo_FlowInfo _tmp2B1;_LL4: _tmp2B1=_stmttmp6A.f1;_tmp2B2=_stmttmp6A.f2;_LL5: {union Cyc_CfFlowInfo_FlowInfo f2=_tmp2B1;void*r2=_tmp2B2;
union Cyc_CfFlowInfo_FlowInfo f=f2;
# 1992
union Cyc_CfFlowInfo_FlowInfo _tmp2B3=f;struct Cyc_List_List*_tmp2B5;struct Cyc_Dict_Dict _tmp2B4;if((_tmp2B3.BottomFL).tag == 1){_LL7: _LL8:
 return({struct _tuple20 _tmp4A8;_tmp4A8.f1=f,_tmp4A8.f2=f;_tmp4A8;});}else{_LL9: _tmp2B4=((_tmp2B3.ReachableFL).val).f1;_tmp2B5=((_tmp2B3.ReachableFL).val).f2;_LLA: {struct Cyc_Dict_Dict d=_tmp2B4;struct Cyc_List_List*relns=_tmp2B5;
# 1995
struct Cyc_Absyn_Exp*e1=(struct Cyc_Absyn_Exp*)es->hd;
struct Cyc_Absyn_Exp*e2=(struct Cyc_Absyn_Exp*)((struct Cyc_List_List*)_check_null(es->tl))->hd;
if((int)Cyc_CfFlowInfo_initlevel(env->fenv,d,r1)== (int)Cyc_CfFlowInfo_NoneIL && !Cyc_CfFlowInfo_is_init_pointer(r1))
({void*_tmp2B6=0U;({unsigned _tmp5F6=((struct Cyc_Absyn_Exp*)es->hd)->loc;struct _fat_ptr _tmp5F5=({const char*_tmp2B7="expression may not be initialized";_tag_fat(_tmp2B7,sizeof(char),34U);});Cyc_CfFlowInfo_aerr(_tmp5F6,_tmp5F5,_tag_fat(_tmp2B6,sizeof(void*),0U));});});
# 1997
if(
# 1999
(int)Cyc_CfFlowInfo_initlevel(env->fenv,d,r2)== (int)Cyc_CfFlowInfo_NoneIL && !Cyc_CfFlowInfo_is_init_pointer(r1))
({void*_tmp2B8=0U;({unsigned _tmp5F8=((struct Cyc_Absyn_Exp*)((struct Cyc_List_List*)_check_null(es->tl))->hd)->loc;struct _fat_ptr _tmp5F7=({const char*_tmp2B9="expression may not be initialized";_tag_fat(_tmp2B9,sizeof(char),34U);});Cyc_CfFlowInfo_aerr(_tmp5F8,_tmp5F7,_tag_fat(_tmp2B8,sizeof(void*),0U));});});{
# 1997
union Cyc_CfFlowInfo_FlowInfo ft=f;
# 2003
union Cyc_CfFlowInfo_FlowInfo ff=f;
# 2007
if((int)p == (int)5U ||(int)p == (int)6U){
struct _tuple16 _stmttmp6B=Cyc_CfFlowInfo_unname_rval(r1);struct Cyc_List_List*_tmp2BB;void*_tmp2BA;_LLC: _tmp2BA=_stmttmp6B.f1;_tmp2BB=_stmttmp6B.f2;_LLD: {void*r1=_tmp2BA;struct Cyc_List_List*r1n=_tmp2BB;
struct _tuple16 _stmttmp6C=Cyc_CfFlowInfo_unname_rval(r2);struct Cyc_List_List*_tmp2BD;void*_tmp2BC;_LLF: _tmp2BC=_stmttmp6C.f1;_tmp2BD=_stmttmp6C.f2;_LL10: {void*r2=_tmp2BC;struct Cyc_List_List*r2n=_tmp2BD;
struct _tuple0 _stmttmp6D=({struct _tuple0 _tmp4A9;_tmp4A9.f1=r1,_tmp4A9.f2=r2;_tmp4A9;});struct _tuple0 _tmp2BE=_stmttmp6D;enum Cyc_CfFlowInfo_InitLevel _tmp2BF;enum Cyc_CfFlowInfo_InitLevel _tmp2C0;switch(*((int*)_tmp2BE.f1)){case 2U: if(((struct Cyc_CfFlowInfo_Zero_CfFlowInfo_AbsRVal_struct*)_tmp2BE.f2)->tag == 0U){_LL12: _tmp2C0=((struct Cyc_CfFlowInfo_UnknownR_CfFlowInfo_AbsRVal_struct*)_tmp2BE.f1)->f1;_LL13: {enum Cyc_CfFlowInfo_InitLevel il=_tmp2C0;
# 2012
struct _tuple20 _stmttmp6E=Cyc_NewControlFlow_splitzero(env,inflow,f,e1,il,r1n);union Cyc_CfFlowInfo_FlowInfo _tmp2C2;union Cyc_CfFlowInfo_FlowInfo _tmp2C1;_LL23: _tmp2C1=_stmttmp6E.f1;_tmp2C2=_stmttmp6E.f2;_LL24: {union Cyc_CfFlowInfo_FlowInfo f1=_tmp2C1;union Cyc_CfFlowInfo_FlowInfo f2=_tmp2C2;
{enum Cyc_Absyn_Primop _tmp2C3=p;switch(_tmp2C3){case Cyc_Absyn_Eq: _LL26: _LL27:
 ft=f2;ff=f1;goto _LL25;case Cyc_Absyn_Neq: _LL28: _LL29:
 ft=f1;ff=f2;goto _LL25;default: _LL2A: _LL2B:
({void*_tmp2C4=0U;({int(*_tmp5FA)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp2C6)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmp2C6;});struct _fat_ptr _tmp5F9=({const char*_tmp2C5="anal_test, zero-split";_tag_fat(_tmp2C5,sizeof(char),22U);});_tmp5FA(_tmp5F9,_tag_fat(_tmp2C4,sizeof(void*),0U));});});}_LL25:;}
# 2018
goto _LL11;}}}else{goto _LL20;}case 0U: switch(*((int*)_tmp2BE.f2)){case 2U: _LL14: _tmp2BF=((struct Cyc_CfFlowInfo_UnknownR_CfFlowInfo_AbsRVal_struct*)_tmp2BE.f2)->f1;_LL15: {enum Cyc_CfFlowInfo_InitLevel il=_tmp2BF;
# 2020
struct _tuple20 _stmttmp6F=Cyc_NewControlFlow_splitzero(env,f2,f,e2,il,r2n);union Cyc_CfFlowInfo_FlowInfo _tmp2C8;union Cyc_CfFlowInfo_FlowInfo _tmp2C7;_LL2D: _tmp2C7=_stmttmp6F.f1;_tmp2C8=_stmttmp6F.f2;_LL2E: {union Cyc_CfFlowInfo_FlowInfo f1=_tmp2C7;union Cyc_CfFlowInfo_FlowInfo f2=_tmp2C8;
{enum Cyc_Absyn_Primop _tmp2C9=p;switch(_tmp2C9){case Cyc_Absyn_Eq: _LL30: _LL31:
 ft=f2;ff=f1;goto _LL2F;case Cyc_Absyn_Neq: _LL32: _LL33:
 ft=f1;ff=f2;goto _LL2F;default: _LL34: _LL35:
({void*_tmp2CA=0U;({int(*_tmp5FC)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp2CC)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmp2CC;});struct _fat_ptr _tmp5FB=({const char*_tmp2CB="anal_test, zero-split";_tag_fat(_tmp2CB,sizeof(char),22U);});_tmp5FC(_tmp5FB,_tag_fat(_tmp2CA,sizeof(void*),0U));});});}_LL2F:;}
# 2026
goto _LL11;}}case 0U: _LL16: _LL17:
# 2028
 if((int)p == (int)5U)ff=Cyc_CfFlowInfo_BottomFL();else{
ft=Cyc_CfFlowInfo_BottomFL();}
goto _LL11;case 1U: _LL18: _LL19:
 goto _LL1B;case 4U: _LL1A: _LL1B:
 goto _LL1D;default: goto _LL20;}case 1U: if(((struct Cyc_CfFlowInfo_Zero_CfFlowInfo_AbsRVal_struct*)_tmp2BE.f2)->tag == 0U){_LL1C: _LL1D:
 goto _LL1F;}else{goto _LL20;}case 4U: if(((struct Cyc_CfFlowInfo_Zero_CfFlowInfo_AbsRVal_struct*)_tmp2BE.f2)->tag == 0U){_LL1E: _LL1F:
# 2035
 if((int)p == (int)6U)ff=Cyc_CfFlowInfo_BottomFL();else{
ft=Cyc_CfFlowInfo_BottomFL();}
goto _LL11;}else{goto _LL20;}default: _LL20: _LL21:
 goto _LL11;}_LL11:;}}}
# 2007
{struct _tuple28 _stmttmp70=({struct _tuple28 _tmp4AB;_tmp4AB.f1=p,({
# 2048
void*_tmp5FE=Cyc_Tcutil_compress((void*)_check_null(e1->topt));_tmp4AB.f2=_tmp5FE;}),({
void*_tmp5FD=Cyc_Tcutil_compress((void*)_check_null(e2->topt));_tmp4AB.f3=_tmp5FD;});_tmp4AB;});
# 2007
struct _tuple28 _tmp2CD=_stmttmp70;switch(_tmp2CD.f1){case Cyc_Absyn_Eq: _LL37: _LL38:
# 2050
 goto _LL3A;case Cyc_Absyn_Neq: _LL39: _LL3A:
 goto _LL3C;default: if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2CD.f2)->tag == 0U){if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2CD.f2)->f1)->tag == 1U){if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2CD.f2)->f1)->f1 == Cyc_Absyn_Unsigned){_LL3B: _LL3C:
 goto _LL3E;}else{if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2CD.f3)->tag == 0U)switch(*((int*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2CD.f3)->f1)){case 1U: if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2CD.f3)->f1)->f1 == Cyc_Absyn_Unsigned)goto _LL3D;else{goto _LL43;}case 4U: goto _LL41;default: goto _LL43;}else{goto _LL43;}}}else{if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2CD.f3)->tag == 0U){if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2CD.f3)->f1)->tag == 1U){if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2CD.f3)->f1)->f1 == Cyc_Absyn_Unsigned)goto _LL3D;else{if(((struct Cyc_Absyn_TagCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2CD.f2)->f1)->tag == 4U)goto _LL3F;else{goto _LL43;}}}else{if(((struct Cyc_Absyn_TagCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2CD.f2)->f1)->tag == 4U)goto _LL3F;else{if(((struct Cyc_Absyn_TagCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2CD.f3)->f1)->tag == 4U)goto _LL41;else{goto _LL43;}}}}else{if(((struct Cyc_Absyn_TagCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2CD.f2)->f1)->tag == 4U){_LL3F: _LL40:
# 2054
 goto _LL42;}else{goto _LL43;}}}}else{if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2CD.f3)->tag == 0U)switch(*((int*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2CD.f3)->f1)){case 1U: if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2CD.f3)->f1)->f1 == Cyc_Absyn_Unsigned){_LL3D: _LL3E:
# 2053
 goto _LL40;}else{goto _LL43;}case 4U: _LL41: _LL42:
# 2055
 goto _LL36;default: goto _LL43;}else{_LL43: _LL44:
 return({struct _tuple20 _tmp4AA;_tmp4AA.f1=ft,_tmp4AA.f2=ff;_tmp4AA;});}}}_LL36:;}
# 2060
{enum Cyc_Absyn_Primop _tmp2CE=p;switch(_tmp2CE){case Cyc_Absyn_Eq: _LL46: _LL47:
 goto _LL49;case Cyc_Absyn_Neq: _LL48: _LL49: goto _LL4B;case Cyc_Absyn_Gt: _LL4A: _LL4B: goto _LL4D;case Cyc_Absyn_Gte: _LL4C: _LL4D: goto _LL4F;case Cyc_Absyn_Lt: _LL4E: _LL4F: goto _LL51;case Cyc_Absyn_Lte: _LL50: _LL51: goto _LL45;default: _LL52: _LL53:
 return({struct _tuple20 _tmp4AC;_tmp4AC.f1=ft,_tmp4AC.f2=ff;_tmp4AC;});}_LL45:;}{
# 2065
struct _RegionHandle*r=Cyc_Core_heap_region;
struct _tuple13 _stmttmp71=Cyc_Relations_primop2relation(e1,p,e2);struct Cyc_Absyn_Exp*_tmp2D1;enum Cyc_Relations_Relation _tmp2D0;struct Cyc_Absyn_Exp*_tmp2CF;_LL55: _tmp2CF=_stmttmp71.f1;_tmp2D0=_stmttmp71.f2;_tmp2D1=_stmttmp71.f3;_LL56: {struct Cyc_Absyn_Exp*e1=_tmp2CF;enum Cyc_Relations_Relation relation=_tmp2D0;struct Cyc_Absyn_Exp*e2=_tmp2D1;
union Cyc_Relations_RelnOp n1=Cyc_Relations_RConst(0U);
union Cyc_Relations_RelnOp n2=Cyc_Relations_RConst(0U);
# 2070
if(Cyc_Relations_exp2relnop(e1,& n1)&& Cyc_Relations_exp2relnop(e2,& n2)){
# 2072
struct Cyc_List_List*relns_true=Cyc_Relations_add_relation(r,n1,relation,n2,relns);
# 2076
struct Cyc_List_List*relns_false=({struct _RegionHandle*_tmp602=r;union Cyc_Relations_RelnOp _tmp601=n2;enum Cyc_Relations_Relation _tmp600=Cyc_Relations_flip_relation(relation);union Cyc_Relations_RelnOp _tmp5FF=n1;Cyc_Relations_add_relation(_tmp602,_tmp601,_tmp600,_tmp5FF,relns);});
struct _tuple20 _stmttmp72=({struct _tuple20 _tmp4B1;_tmp4B1.f1=ft,_tmp4B1.f2=ff;_tmp4B1;});struct _tuple20 _tmp2D2=_stmttmp72;struct Cyc_Dict_Dict _tmp2D3;struct Cyc_Dict_Dict _tmp2D4;struct Cyc_Dict_Dict _tmp2D6;struct Cyc_Dict_Dict _tmp2D5;if(((_tmp2D2.f1).ReachableFL).tag == 2){if(((_tmp2D2.f2).ReachableFL).tag == 2){_LL58: _tmp2D5=(((_tmp2D2.f1).ReachableFL).val).f1;_tmp2D6=(((_tmp2D2.f2).ReachableFL).val).f1;_LL59: {struct Cyc_Dict_Dict dt=_tmp2D5;struct Cyc_Dict_Dict df=_tmp2D6;
# 2079
return({struct _tuple20 _tmp4AD;({union Cyc_CfFlowInfo_FlowInfo _tmp604=Cyc_CfFlowInfo_ReachableFL(dt,relns_true);_tmp4AD.f1=_tmp604;}),({union Cyc_CfFlowInfo_FlowInfo _tmp603=Cyc_CfFlowInfo_ReachableFL(df,relns_false);_tmp4AD.f2=_tmp603;});_tmp4AD;});}}else{_LL5C: _tmp2D4=(((_tmp2D2.f1).ReachableFL).val).f1;_LL5D: {struct Cyc_Dict_Dict dt=_tmp2D4;
# 2083
return({struct _tuple20 _tmp4AE;({union Cyc_CfFlowInfo_FlowInfo _tmp605=Cyc_CfFlowInfo_ReachableFL(dt,relns_true);_tmp4AE.f1=_tmp605;}),_tmp4AE.f2=ff;_tmp4AE;});}}}else{if(((_tmp2D2.f2).ReachableFL).tag == 2){_LL5A: _tmp2D3=(((_tmp2D2.f2).ReachableFL).val).f1;_LL5B: {struct Cyc_Dict_Dict df=_tmp2D3;
# 2081
return({struct _tuple20 _tmp4AF;_tmp4AF.f1=ft,({union Cyc_CfFlowInfo_FlowInfo _tmp606=Cyc_CfFlowInfo_ReachableFL(df,relns_false);_tmp4AF.f2=_tmp606;});_tmp4AF;});}}else{_LL5E: _LL5F:
# 2085
 return({struct _tuple20 _tmp4B0;_tmp4B0.f1=ft,_tmp4B0.f2=ff;_tmp4B0;});}}_LL57:;}else{
# 2088
return({struct _tuple20 _tmp4B2;_tmp4B2.f1=ft,_tmp4B2.f2=ff;_tmp4B2;});}}}}}}_LL6:;}}}
# 548 "new_control_flow.cyc"
static struct _tuple20 Cyc_NewControlFlow_anal_test(struct Cyc_NewControlFlow_AnalEnv*env,union Cyc_CfFlowInfo_FlowInfo inflow,struct Cyc_Absyn_Exp*e){
# 2095 "new_control_flow.cyc"
struct Cyc_CfFlowInfo_FlowEnv*fenv=env->fenv;
void*_stmttmp73=e->r;void*_tmp2D7=_stmttmp73;struct Cyc_List_List*_tmp2D9;enum Cyc_Absyn_Primop _tmp2D8;struct Cyc_Absyn_Exp*_tmp2DA;struct Cyc_Absyn_Exp*_tmp2DC;struct Cyc_Absyn_Exp*_tmp2DB;struct Cyc_Absyn_Exp*_tmp2DE;struct Cyc_Absyn_Exp*_tmp2DD;struct Cyc_Absyn_Exp*_tmp2E0;struct Cyc_Absyn_Exp*_tmp2DF;struct Cyc_Absyn_Exp*_tmp2E2;struct Cyc_Absyn_Exp*_tmp2E1;struct Cyc_Absyn_Exp*_tmp2E5;struct Cyc_Absyn_Exp*_tmp2E4;struct Cyc_Absyn_Exp*_tmp2E3;switch(*((int*)_tmp2D7)){case 6U: _LL1: _tmp2E3=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmp2D7)->f1;_tmp2E4=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmp2D7)->f2;_tmp2E5=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmp2D7)->f3;_LL2: {struct Cyc_Absyn_Exp*e1=_tmp2E3;struct Cyc_Absyn_Exp*e2=_tmp2E4;struct Cyc_Absyn_Exp*e3=_tmp2E5;
# 2098
struct _tuple20 _stmttmp74=Cyc_NewControlFlow_anal_test(env,inflow,e1);union Cyc_CfFlowInfo_FlowInfo _tmp2E7;union Cyc_CfFlowInfo_FlowInfo _tmp2E6;_LL12: _tmp2E6=_stmttmp74.f1;_tmp2E7=_stmttmp74.f2;_LL13: {union Cyc_CfFlowInfo_FlowInfo f1t=_tmp2E6;union Cyc_CfFlowInfo_FlowInfo f1f=_tmp2E7;
struct _tuple20 _stmttmp75=Cyc_NewControlFlow_anal_test(env,f1t,e2);union Cyc_CfFlowInfo_FlowInfo _tmp2E9;union Cyc_CfFlowInfo_FlowInfo _tmp2E8;_LL15: _tmp2E8=_stmttmp75.f1;_tmp2E9=_stmttmp75.f2;_LL16: {union Cyc_CfFlowInfo_FlowInfo f2t=_tmp2E8;union Cyc_CfFlowInfo_FlowInfo f2f=_tmp2E9;
struct _tuple20 _stmttmp76=Cyc_NewControlFlow_anal_test(env,f1f,e3);union Cyc_CfFlowInfo_FlowInfo _tmp2EB;union Cyc_CfFlowInfo_FlowInfo _tmp2EA;_LL18: _tmp2EA=_stmttmp76.f1;_tmp2EB=_stmttmp76.f2;_LL19: {union Cyc_CfFlowInfo_FlowInfo f3t=_tmp2EA;union Cyc_CfFlowInfo_FlowInfo f3f=_tmp2EB;
return({struct _tuple20 _tmp4B3;({union Cyc_CfFlowInfo_FlowInfo _tmp608=Cyc_CfFlowInfo_join_flow(fenv,f2t,f3t);_tmp4B3.f1=_tmp608;}),({
union Cyc_CfFlowInfo_FlowInfo _tmp607=Cyc_CfFlowInfo_join_flow(fenv,f2f,f3f);_tmp4B3.f2=_tmp607;});_tmp4B3;});}}}}case 7U: _LL3: _tmp2E1=((struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct*)_tmp2D7)->f1;_tmp2E2=((struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct*)_tmp2D7)->f2;_LL4: {struct Cyc_Absyn_Exp*e1=_tmp2E1;struct Cyc_Absyn_Exp*e2=_tmp2E2;
# 2104
struct _tuple20 _stmttmp77=Cyc_NewControlFlow_anal_test(env,inflow,e1);union Cyc_CfFlowInfo_FlowInfo _tmp2ED;union Cyc_CfFlowInfo_FlowInfo _tmp2EC;_LL1B: _tmp2EC=_stmttmp77.f1;_tmp2ED=_stmttmp77.f2;_LL1C: {union Cyc_CfFlowInfo_FlowInfo f1t=_tmp2EC;union Cyc_CfFlowInfo_FlowInfo f1f=_tmp2ED;
struct _tuple20 _stmttmp78=Cyc_NewControlFlow_anal_test(env,f1t,e2);union Cyc_CfFlowInfo_FlowInfo _tmp2EF;union Cyc_CfFlowInfo_FlowInfo _tmp2EE;_LL1E: _tmp2EE=_stmttmp78.f1;_tmp2EF=_stmttmp78.f2;_LL1F: {union Cyc_CfFlowInfo_FlowInfo f2t=_tmp2EE;union Cyc_CfFlowInfo_FlowInfo f2f=_tmp2EF;
return({struct _tuple20 _tmp4B4;_tmp4B4.f1=f2t,({union Cyc_CfFlowInfo_FlowInfo _tmp609=Cyc_CfFlowInfo_join_flow(fenv,f1f,f2f);_tmp4B4.f2=_tmp609;});_tmp4B4;});}}}case 8U: _LL5: _tmp2DF=((struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct*)_tmp2D7)->f1;_tmp2E0=((struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct*)_tmp2D7)->f2;_LL6: {struct Cyc_Absyn_Exp*e1=_tmp2DF;struct Cyc_Absyn_Exp*e2=_tmp2E0;
# 2108
struct _tuple20 _stmttmp79=Cyc_NewControlFlow_anal_test(env,inflow,e1);union Cyc_CfFlowInfo_FlowInfo _tmp2F1;union Cyc_CfFlowInfo_FlowInfo _tmp2F0;_LL21: _tmp2F0=_stmttmp79.f1;_tmp2F1=_stmttmp79.f2;_LL22: {union Cyc_CfFlowInfo_FlowInfo f1t=_tmp2F0;union Cyc_CfFlowInfo_FlowInfo f1f=_tmp2F1;
struct _tuple20 _stmttmp7A=Cyc_NewControlFlow_anal_test(env,f1f,e2);union Cyc_CfFlowInfo_FlowInfo _tmp2F3;union Cyc_CfFlowInfo_FlowInfo _tmp2F2;_LL24: _tmp2F2=_stmttmp7A.f1;_tmp2F3=_stmttmp7A.f2;_LL25: {union Cyc_CfFlowInfo_FlowInfo f2t=_tmp2F2;union Cyc_CfFlowInfo_FlowInfo f2f=_tmp2F3;
return({struct _tuple20 _tmp4B5;({union Cyc_CfFlowInfo_FlowInfo _tmp60A=Cyc_CfFlowInfo_join_flow(fenv,f1t,f2t);_tmp4B5.f1=_tmp60A;}),_tmp4B5.f2=f2f;_tmp4B5;});}}}case 34U: _LL7: _tmp2DD=((struct Cyc_Absyn_Spawn_e_Absyn_Raw_exp_struct*)_tmp2D7)->f1;_tmp2DE=((struct Cyc_Absyn_Spawn_e_Absyn_Raw_exp_struct*)_tmp2D7)->f2;_LL8: {struct Cyc_Absyn_Exp*e1=_tmp2DD;struct Cyc_Absyn_Exp*e2=_tmp2DE;
# 2113
struct _tuple17 _stmttmp7B=Cyc_NewControlFlow_anal_Rexp(env,0,inflow,e1);void*_tmp2F5;union Cyc_CfFlowInfo_FlowInfo _tmp2F4;_LL27: _tmp2F4=_stmttmp7B.f1;_tmp2F5=_stmttmp7B.f2;_LL28: {union Cyc_CfFlowInfo_FlowInfo f=_tmp2F4;void*r=_tmp2F5;
return Cyc_NewControlFlow_anal_test(env,f,e2);}}case 9U: _LL9: _tmp2DB=((struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct*)_tmp2D7)->f1;_tmp2DC=((struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct*)_tmp2D7)->f2;_LLA: {struct Cyc_Absyn_Exp*e1=_tmp2DB;struct Cyc_Absyn_Exp*e2=_tmp2DC;
# 2117
struct _tuple17 _stmttmp7C=Cyc_NewControlFlow_anal_Rexp(env,0,inflow,e1);void*_tmp2F7;union Cyc_CfFlowInfo_FlowInfo _tmp2F6;_LL2A: _tmp2F6=_stmttmp7C.f1;_tmp2F7=_stmttmp7C.f2;_LL2B: {union Cyc_CfFlowInfo_FlowInfo f=_tmp2F6;void*r=_tmp2F7;
return Cyc_NewControlFlow_anal_test(env,f,e2);}}case 3U: if(((struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct*)_tmp2D7)->f1 == Cyc_Absyn_Not){if(((struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct*)_tmp2D7)->f2 != 0){if(((struct Cyc_List_List*)((struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct*)_tmp2D7)->f2)->tl == 0){_LLB: _tmp2DA=(struct Cyc_Absyn_Exp*)(((struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct*)_tmp2D7)->f2)->hd;_LLC: {struct Cyc_Absyn_Exp*e1=_tmp2DA;
# 2120
struct _tuple20 _stmttmp7D=Cyc_NewControlFlow_anal_test(env,inflow,e1);union Cyc_CfFlowInfo_FlowInfo _tmp2F9;union Cyc_CfFlowInfo_FlowInfo _tmp2F8;_LL2D: _tmp2F8=_stmttmp7D.f1;_tmp2F9=_stmttmp7D.f2;_LL2E: {union Cyc_CfFlowInfo_FlowInfo f1=_tmp2F8;union Cyc_CfFlowInfo_FlowInfo f2=_tmp2F9;
return({struct _tuple20 _tmp4B6;_tmp4B6.f1=f2,_tmp4B6.f2=f1;_tmp4B6;});}}}else{goto _LLD;}}else{goto _LLD;}}else{_LLD: _tmp2D8=((struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct*)_tmp2D7)->f1;_tmp2D9=((struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct*)_tmp2D7)->f2;_LLE: {enum Cyc_Absyn_Primop p=_tmp2D8;struct Cyc_List_List*es=_tmp2D9;
# 2123
return Cyc_NewControlFlow_anal_primop_test(env,inflow,p,es);}}default: _LLF: _LL10: {
# 2127
struct _tuple17 _stmttmp7E=Cyc_NewControlFlow_anal_Rexp(env,0,inflow,e);void*_tmp2FB;union Cyc_CfFlowInfo_FlowInfo _tmp2FA;_LL30: _tmp2FA=_stmttmp7E.f1;_tmp2FB=_stmttmp7E.f2;_LL31: {union Cyc_CfFlowInfo_FlowInfo f=_tmp2FA;void*r=_tmp2FB;
union Cyc_CfFlowInfo_FlowInfo _tmp2FC=f;struct Cyc_Dict_Dict _tmp2FD;if((_tmp2FC.BottomFL).tag == 1){_LL33: _LL34:
 return({struct _tuple20 _tmp4B7;_tmp4B7.f1=f,_tmp4B7.f2=f;_tmp4B7;});}else{_LL35: _tmp2FD=((_tmp2FC.ReachableFL).val).f1;_LL36: {struct Cyc_Dict_Dict d=_tmp2FD;
# 2131
struct _tuple16 _stmttmp7F=Cyc_CfFlowInfo_unname_rval(r);struct Cyc_List_List*_tmp2FF;void*_tmp2FE;_LL38: _tmp2FE=_stmttmp7F.f1;_tmp2FF=_stmttmp7F.f2;_LL39: {void*r=_tmp2FE;struct Cyc_List_List*names=_tmp2FF;
void*_tmp300=r;enum Cyc_CfFlowInfo_InitLevel _tmp301;void*_tmp303;struct Cyc_Absyn_Vardecl*_tmp302;switch(*((int*)_tmp300)){case 8U: _LL3B: _tmp302=((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmp300)->f1;_tmp303=(void*)((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmp300)->f2;_LL3C: {struct Cyc_Absyn_Vardecl*n=_tmp302;void*r2=_tmp303;
# 2134
({void*_tmp304=0U;({int(*_tmp60C)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp306)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmp306;});struct _fat_ptr _tmp60B=({const char*_tmp305="anal_test: bad namedlocation";_tag_fat(_tmp305,sizeof(char),29U);});_tmp60C(_tmp60B,_tag_fat(_tmp304,sizeof(void*),0U));});});}case 0U: _LL3D: _LL3E:
 return({struct _tuple20 _tmp4B8;({union Cyc_CfFlowInfo_FlowInfo _tmp60D=Cyc_CfFlowInfo_BottomFL();_tmp4B8.f1=_tmp60D;}),_tmp4B8.f2=f;_tmp4B8;});case 1U: _LL3F: _LL40:
 goto _LL42;case 5U: _LL41: _LL42:
 goto _LL44;case 4U: _LL43: _LL44:
 return({struct _tuple20 _tmp4B9;_tmp4B9.f1=f,({union Cyc_CfFlowInfo_FlowInfo _tmp60E=Cyc_CfFlowInfo_BottomFL();_tmp4B9.f2=_tmp60E;});_tmp4B9;});case 2U: if(((struct Cyc_CfFlowInfo_UnknownR_CfFlowInfo_AbsRVal_struct*)_tmp300)->f1 == Cyc_CfFlowInfo_NoneIL){_LL45: _LL46:
 goto _LL48;}else{_LL4B: _tmp301=((struct Cyc_CfFlowInfo_UnknownR_CfFlowInfo_AbsRVal_struct*)_tmp300)->f1;_LL4C: {enum Cyc_CfFlowInfo_InitLevel il=_tmp301;
# 2144
return Cyc_NewControlFlow_splitzero(env,inflow,f,e,il,names);}}case 3U: if(((struct Cyc_CfFlowInfo_Esc_CfFlowInfo_AbsRVal_struct*)_tmp300)->f1 == Cyc_CfFlowInfo_NoneIL){_LL47: _LL48:
# 2140
 goto _LL4A;}else{_LL4D: _LL4E:
# 2145
 return({struct _tuple20 _tmp4BA;_tmp4BA.f1=f,_tmp4BA.f2=f;_tmp4BA;});}case 7U: _LL49: _LL4A:
# 2142
({void*_tmp307=0U;({unsigned _tmp610=e->loc;struct _fat_ptr _tmp60F=({const char*_tmp308="expression may not be initialized";_tag_fat(_tmp308,sizeof(char),34U);});Cyc_CfFlowInfo_aerr(_tmp610,_tmp60F,_tag_fat(_tmp307,sizeof(void*),0U));});});
return({struct _tuple20 _tmp4BB;({union Cyc_CfFlowInfo_FlowInfo _tmp612=Cyc_CfFlowInfo_BottomFL();_tmp4BB.f1=_tmp612;}),({union Cyc_CfFlowInfo_FlowInfo _tmp611=Cyc_CfFlowInfo_BottomFL();_tmp4BB.f2=_tmp611;});_tmp4BB;});default: _LL4F: _LL50:
# 2146
({void*_tmp309=0U;({int(*_tmp614)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp30B)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmp30B;});struct _fat_ptr _tmp613=({const char*_tmp30A="anal_test";_tag_fat(_tmp30A,sizeof(char),10U);});_tmp614(_tmp613,_tag_fat(_tmp309,sizeof(void*),0U));});});}_LL3A:;}}}_LL32:;}}}_LL0:;}struct _tuple29{unsigned f1;struct Cyc_NewControlFlow_AnalEnv*f2;struct Cyc_Dict_Dict f3;};
# 548 "new_control_flow.cyc"
static void Cyc_NewControlFlow_check_for_unused_unique(struct _tuple29*ckenv,void*root,void*rval){
# 2154 "new_control_flow.cyc"
struct Cyc_Dict_Dict _tmp30E;struct Cyc_NewControlFlow_AnalEnv*_tmp30D;unsigned _tmp30C;_LL1: _tmp30C=ckenv->f1;_tmp30D=ckenv->f2;_tmp30E=ckenv->f3;_LL2: {unsigned loc=_tmp30C;struct Cyc_NewControlFlow_AnalEnv*env=_tmp30D;struct Cyc_Dict_Dict fd=_tmp30E;
void*_tmp30F=root;struct Cyc_Absyn_Vardecl*_tmp310;if(((struct Cyc_CfFlowInfo_VarRoot_CfFlowInfo_Root_struct*)_tmp30F)->tag == 0U){_LL4: _tmp310=((struct Cyc_CfFlowInfo_VarRoot_CfFlowInfo_Root_struct*)_tmp30F)->f1;_LL5: {struct Cyc_Absyn_Vardecl*vd=_tmp310;
# 2157
if(Cyc_Tcutil_is_noalias_pointer_or_aggr(vd->type)){
struct _tuple16 _stmttmp80=Cyc_CfFlowInfo_unname_rval(rval);void*_tmp311;_LL9: _tmp311=_stmttmp80.f1;_LLA: {void*rval=_tmp311;
void*_tmp312=rval;switch(*((int*)_tmp312)){case 7U: _LLC: _LLD:
 goto _LLF;case 0U: _LLE: _LLF:
 goto _LL11;case 2U: if(((struct Cyc_CfFlowInfo_UnknownR_CfFlowInfo_AbsRVal_struct*)_tmp312)->f1 == Cyc_CfFlowInfo_NoneIL){_LL10: _LL11:
 goto _LLB;}else{goto _LL12;}default: _LL12: _LL13:
# 2164
({struct Cyc_String_pa_PrintArg_struct _tmp315=({struct Cyc_String_pa_PrintArg_struct _tmp4BC;_tmp4BC.tag=0U,({struct _fat_ptr _tmp615=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(vd->name));_tmp4BC.f1=_tmp615;});_tmp4BC;});void*_tmp313[1U];_tmp313[0]=& _tmp315;({unsigned _tmp617=loc;struct _fat_ptr _tmp616=({const char*_tmp314="unique pointers reachable from %s may become unreachable";_tag_fat(_tmp314,sizeof(char),57U);});Cyc_Warn_warn(_tmp617,_tmp616,_tag_fat(_tmp313,sizeof(void*),1U));});});
goto _LLB;}_LLB:;}}
# 2157
goto _LL3;}}else{_LL6: _LL7:
# 2169
 goto _LL3;}_LL3:;}}
# 548 "new_control_flow.cyc"
static void Cyc_NewControlFlow_check_init_params(unsigned loc,struct Cyc_NewControlFlow_AnalEnv*env,union Cyc_CfFlowInfo_FlowInfo flow){
# 2174 "new_control_flow.cyc"
union Cyc_CfFlowInfo_FlowInfo _tmp316=flow;struct Cyc_Dict_Dict _tmp317;if((_tmp316.BottomFL).tag == 1){_LL1: _LL2:
 return;}else{_LL3: _tmp317=((_tmp316.ReachableFL).val).f1;_LL4: {struct Cyc_Dict_Dict d=_tmp317;
# 2177
{struct Cyc_List_List*inits=env->param_roots;for(0;inits != 0;inits=inits->tl){
if((int)({struct Cyc_CfFlowInfo_FlowEnv*_tmp619=env->fenv;struct Cyc_Dict_Dict _tmp618=d;Cyc_CfFlowInfo_initlevel(_tmp619,_tmp618,Cyc_CfFlowInfo_lookup_place(d,(struct Cyc_CfFlowInfo_Place*)inits->hd));})!= (int)Cyc_CfFlowInfo_AllIL)
({void*_tmp318=0U;({unsigned _tmp61B=loc;struct _fat_ptr _tmp61A=({const char*_tmp319="function may not initialize all the parameters with attribute 'initializes'";_tag_fat(_tmp319,sizeof(char),76U);});Cyc_CfFlowInfo_aerr(_tmp61B,_tmp61A,_tag_fat(_tmp318,sizeof(void*),0U));});});}}
# 2177
if(Cyc_NewControlFlow_warn_lose_unique){
# 2182
struct _tuple29 check_env=({struct _tuple29 _tmp4BD;_tmp4BD.f1=loc,_tmp4BD.f2=env,_tmp4BD.f3=d;_tmp4BD;});
({void(*_tmp61C)(void(*f)(struct _tuple29*,void*,void*),struct _tuple29*env,struct Cyc_Dict_Dict d)=({void(*_tmp31A)(void(*f)(struct _tuple29*,void*,void*),struct _tuple29*env,struct Cyc_Dict_Dict d)=(void(*)(void(*f)(struct _tuple29*,void*,void*),struct _tuple29*env,struct Cyc_Dict_Dict d))Cyc_Dict_iter_c;_tmp31A;});_tmp61C(Cyc_NewControlFlow_check_for_unused_unique,& check_env,d);});}
# 2177
return;}}_LL0:;}
# 548 "new_control_flow.cyc"
static struct _tuple1 Cyc_NewControlFlow_get_unconsume_pat_vars(struct Cyc_List_List*vds){
# 2196 "new_control_flow.cyc"
struct Cyc_List_List*roots=0;
struct Cyc_List_List*es=0;
{struct Cyc_List_List*x=vds;for(0;x != 0;x=x->tl){
struct _tuple25*_stmttmp81=(struct _tuple25*)x->hd;struct Cyc_Absyn_Exp*_tmp31C;struct Cyc_Absyn_Vardecl**_tmp31B;_LL1: _tmp31B=_stmttmp81->f1;_tmp31C=_stmttmp81->f2;_LL2: {struct Cyc_Absyn_Vardecl**vopt=_tmp31B;struct Cyc_Absyn_Exp*eopt=_tmp31C;
if((vopt != 0 && eopt != 0)&&
 Cyc_Tcutil_is_noalias_pointer((void*)_check_null(eopt->topt),0)){
struct Cyc_CfFlowInfo_VarRoot_CfFlowInfo_Root_struct*root=({struct Cyc_CfFlowInfo_VarRoot_CfFlowInfo_Root_struct*_tmp320=_cycalloc(sizeof(*_tmp320));_tmp320->tag=0U,_tmp320->f1=*vopt;_tmp320;});
struct Cyc_CfFlowInfo_Place*rp=({struct Cyc_CfFlowInfo_Place*_tmp31F=_cycalloc(sizeof(*_tmp31F));_tmp31F->root=(void*)root,_tmp31F->path=0;_tmp31F;});
roots=({struct Cyc_List_List*_tmp31D=_cycalloc(sizeof(*_tmp31D));_tmp31D->hd=rp,_tmp31D->tl=roots;_tmp31D;});
es=({struct Cyc_List_List*_tmp31E=_cycalloc(sizeof(*_tmp31E));_tmp31E->hd=eopt,_tmp31E->tl=es;_tmp31E;});}}}}
# 2208
return({struct _tuple1 _tmp4BE;_tmp4BE.f1=roots,_tmp4BE.f2=es;_tmp4BE;});}struct _tuple30{int f1;void*f2;};
# 548 "new_control_flow.cyc"
static struct _tuple30 Cyc_NewControlFlow_noconsume_place_ok(struct Cyc_NewControlFlow_AnalEnv*env,struct Cyc_CfFlowInfo_Place*place,int do_unconsume,struct Cyc_Absyn_Vardecl*vd,union Cyc_CfFlowInfo_FlowInfo flow,unsigned loc){
# 2221 "new_control_flow.cyc"
union Cyc_CfFlowInfo_FlowInfo _tmp321=flow;struct Cyc_Dict_Dict _tmp322;if((_tmp321.BottomFL).tag == 1){_LL1: _LL2:
({void*_tmp323=0U;({int(*_tmp61E)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp325)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmp325;});struct _fat_ptr _tmp61D=({const char*_tmp324="noconsume_place_ok: flow became Bottom!";_tag_fat(_tmp324,sizeof(char),40U);});_tmp61E(_tmp61D,_tag_fat(_tmp323,sizeof(void*),0U));});});}else{_LL3: _tmp322=((_tmp321.ReachableFL).val).f1;_LL4: {struct Cyc_Dict_Dict d=_tmp322;
# 2229
struct Cyc_Absyn_Exp*bogus_exp=Cyc_Absyn_uint_exp(1U,0U);
int bogus_bool=0;
int bogus_int=1;
void*curr_rval=Cyc_CfFlowInfo_lookup_place(d,place);
void*rval=curr_rval;
# 2241
int varok=0;
{void*_tmp326=curr_rval;void*_tmp328;struct Cyc_Absyn_Vardecl*_tmp327;if(((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmp326)->tag == 8U){_LL6: _tmp327=((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmp326)->f1;_tmp328=(void*)((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmp326)->f2;_LL7: {struct Cyc_Absyn_Vardecl*n=_tmp327;void*r=_tmp328;
# 2244
if(vd == n){
rval=r;
# 2247
if(Cyc_Tcutil_is_noalias_pointer_or_aggr(vd->type)){
# 2249
if(Cyc_CfFlowInfo_is_unique_consumed(bogus_exp,bogus_int,rval,& bogus_bool)){
if(!do_unconsume)
({struct Cyc_String_pa_PrintArg_struct _tmp32B=({struct Cyc_String_pa_PrintArg_struct _tmp4BF;_tmp4BF.tag=0U,({
# 2253
struct _fat_ptr _tmp61F=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(vd->name));_tmp4BF.f1=_tmp61F;});_tmp4BF;});void*_tmp329[1U];_tmp329[0]=& _tmp32B;({unsigned _tmp621=loc;struct _fat_ptr _tmp620=({const char*_tmp32A="function consumes parameter %s which is does not have the 'consume' attribute";_tag_fat(_tmp32A,sizeof(char),78U);});Cyc_CfFlowInfo_aerr(_tmp621,_tmp620,_tag_fat(_tmp329,sizeof(void*),1U));});});}else{
# 2256
if((int)Cyc_CfFlowInfo_initlevel(env->fenv,d,rval)!= (int)Cyc_CfFlowInfo_AllIL && !do_unconsume)
({struct Cyc_String_pa_PrintArg_struct _tmp32E=({struct Cyc_String_pa_PrintArg_struct _tmp4C0;_tmp4C0.tag=0U,({
# 2259
struct _fat_ptr _tmp622=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(vd->name));_tmp4C0.f1=_tmp622;});_tmp4C0;});void*_tmp32C[1U];_tmp32C[0]=& _tmp32E;({unsigned _tmp624=loc;struct _fat_ptr _tmp623=({const char*_tmp32D="function consumes value pointed to by parameter %s, which does not have the 'consume' attribute";_tag_fat(_tmp32D,sizeof(char),96U);});Cyc_CfFlowInfo_aerr(_tmp624,_tmp623,_tag_fat(_tmp32C,sizeof(void*),1U));});});else{
# 2261
varok=1;}}}else{
# 2264
varok=1;}}else{
# 2267
goto _LL9;}
goto _LL5;}}else{_LL8: _LL9:
# 2271
 if(!Cyc_Tcutil_is_noalias_pointer_or_aggr(vd->type))
varok=1;else{
if(!do_unconsume)
({struct Cyc_String_pa_PrintArg_struct _tmp331=({struct Cyc_String_pa_PrintArg_struct _tmp4C1;_tmp4C1.tag=0U,({
# 2276
struct _fat_ptr _tmp625=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(vd->name));_tmp4C1.f1=_tmp625;});_tmp4C1;});void*_tmp32F[1U];_tmp32F[0]=& _tmp331;({unsigned _tmp627=loc;struct _fat_ptr _tmp626=({const char*_tmp330="function parameter %s without 'consume' attribute no longer set to its original value";_tag_fat(_tmp330,sizeof(char),86U);});Cyc_CfFlowInfo_aerr(_tmp627,_tmp626,_tag_fat(_tmp32F,sizeof(void*),1U));});});}
# 2271
goto _LL5;}_LL5:;}
# 2283
return({struct _tuple30 _tmp4C2;_tmp4C2.f1=varok,_tmp4C2.f2=rval;_tmp4C2;});}}_LL0:;}
# 548 "new_control_flow.cyc"
static struct Cyc_Absyn_Vardecl*Cyc_NewControlFlow_get_vd_from_place(struct Cyc_CfFlowInfo_Place*p){
# 2290 "new_control_flow.cyc"
struct Cyc_List_List*_tmp333;void*_tmp332;_LL1: _tmp332=p->root;_tmp333=p->path;_LL2: {void*root=_tmp332;struct Cyc_List_List*fs=_tmp333;
if(fs != 0)
({void*_tmp334=0U;({int(*_tmp629)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp336)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmp336;});struct _fat_ptr _tmp628=({const char*_tmp335="unconsume_params: param to unconsume is non-variable";_tag_fat(_tmp335,sizeof(char),53U);});_tmp629(_tmp628,_tag_fat(_tmp334,sizeof(void*),0U));});});{
# 2291
struct Cyc_Absyn_Vardecl*vd;
# 2294
void*_tmp337=root;struct Cyc_Absyn_Vardecl*_tmp338;if(((struct Cyc_CfFlowInfo_VarRoot_CfFlowInfo_Root_struct*)_tmp337)->tag == 0U){_LL4: _tmp338=((struct Cyc_CfFlowInfo_VarRoot_CfFlowInfo_Root_struct*)_tmp337)->f1;_LL5: {struct Cyc_Absyn_Vardecl*vd=_tmp338;
return vd;}}else{_LL6: _LL7:
({void*_tmp339=0U;({int(*_tmp62B)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp33B)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmp33B;});struct _fat_ptr _tmp62A=({const char*_tmp33A="unconsume_params: root is not a varroot";_tag_fat(_tmp33A,sizeof(char),40U);});_tmp62B(_tmp62A,_tag_fat(_tmp339,sizeof(void*),0U));});});}_LL3:;}}}
# 548 "new_control_flow.cyc"
static union Cyc_CfFlowInfo_FlowInfo Cyc_NewControlFlow_unconsume_exp(struct Cyc_NewControlFlow_AnalEnv*env,struct Cyc_Absyn_Exp*unconsume_exp,struct Cyc_Absyn_Vardecl*vd,int varok,void*ropt,union Cyc_CfFlowInfo_FlowInfo flow,unsigned loc){
# 2316 "new_control_flow.cyc"
{union Cyc_CfFlowInfo_FlowInfo _tmp33C=flow;struct Cyc_Dict_Dict _tmp33D;if((_tmp33C.BottomFL).tag == 1){_LL1: _LL2:
 return flow;}else{_LL3: _tmp33D=((_tmp33C.ReachableFL).val).f1;_LL4: {struct Cyc_Dict_Dict d=_tmp33D;
# 2323
struct _tuple19 _stmttmp82=Cyc_NewControlFlow_anal_Lexp(env,flow,0,1,unconsume_exp);union Cyc_CfFlowInfo_AbsLVal _tmp33F;union Cyc_CfFlowInfo_FlowInfo _tmp33E;_LL6: _tmp33E=_stmttmp82.f1;_tmp33F=_stmttmp82.f2;_LL7: {union Cyc_CfFlowInfo_FlowInfo f=_tmp33E;union Cyc_CfFlowInfo_AbsLVal lval=_tmp33F;
# 2326
{union Cyc_CfFlowInfo_AbsLVal _tmp340=lval;struct Cyc_CfFlowInfo_Place*_tmp341;if((_tmp340.PlaceL).tag == 1){_LL9: _tmp341=(_tmp340.PlaceL).val;_LLA: {struct Cyc_CfFlowInfo_Place*p=_tmp341;
# 2330
void*old_rval=Cyc_CfFlowInfo_lookup_place(d,p);
{void*_tmp342=old_rval;void*_tmp344;struct Cyc_Absyn_Vardecl*_tmp343;if(((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmp342)->tag == 8U){_LLE: _tmp343=((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmp342)->f1;_tmp344=(void*)((struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*)_tmp342)->f2;_LLF: {struct Cyc_Absyn_Vardecl*old_vd=_tmp343;void*r=_tmp344;
# 2336
void*new_rval;
if(old_vd == vd){
# 2340
if(varok){
# 2342
old_rval=Cyc_CfFlowInfo_make_unique_unconsumed(env->fenv,r);
# 2347
if(ropt != 0){
# 2353
struct _tuple17 _stmttmp83=({
struct Cyc_CfFlowInfo_FlowEnv*_tmp62D=env->fenv;struct _tuple17 _tmp62C=({struct _tuple17 _tmp4C3;_tmp4C3.f1=f,_tmp4C3.f2=old_rval;_tmp4C3;});Cyc_CfFlowInfo_join_flow_and_rval(_tmp62D,_tmp62C,({struct _tuple17 _tmp4C4;_tmp4C4.f1=f,_tmp4C4.f2=ropt;_tmp4C4;}));});
# 2353
void*_tmp346;union Cyc_CfFlowInfo_FlowInfo _tmp345;_LL13: _tmp345=_stmttmp83.f1;_tmp346=_stmttmp83.f2;_LL14: {union Cyc_CfFlowInfo_FlowInfo f2=_tmp345;void*new_rval2=_tmp346;
# 2357
f=f2;new_rval=new_rval2;}}else{
# 2362
new_rval=old_rval;}}else{
# 2365
new_rval=r;}
# 2367
{union Cyc_CfFlowInfo_FlowInfo _tmp347=f;struct Cyc_List_List*_tmp349;struct Cyc_Dict_Dict _tmp348;if((_tmp347.ReachableFL).tag == 2){_LL16: _tmp348=((_tmp347.ReachableFL).val).f1;_tmp349=((_tmp347.ReachableFL).val).f2;_LL17: {struct Cyc_Dict_Dict d2=_tmp348;struct Cyc_List_List*relns=_tmp349;
# 2369
flow=({struct Cyc_Dict_Dict _tmp62E=Cyc_CfFlowInfo_assign_place(env->fenv,loc,d2,p,new_rval);Cyc_CfFlowInfo_ReachableFL(_tmp62E,relns);});
# 2373
goto _LL15;}}else{_LL18: _LL19:
# 2375
({void*_tmp34A=0U;({int(*_tmp630)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp34C)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmp34C;});struct _fat_ptr _tmp62F=({const char*_tmp34B="unconsume_params: joined flow became bot!";_tag_fat(_tmp34B,sizeof(char),42U);});_tmp630(_tmp62F,_tag_fat(_tmp34A,sizeof(void*),0U));});});}_LL15:;}
# 2377
goto _LLD;}else{
# 2379
goto _LL11;}
goto _LLD;}}else{_LL10: _LL11:
# 2386
 if(ropt != 0 && !
Cyc_Tcutil_is_noalias_pointer_or_aggr(vd->type))
({struct Cyc_String_pa_PrintArg_struct _tmp34F=({struct Cyc_String_pa_PrintArg_struct _tmp4C5;_tmp4C5.tag=0U,({
struct _fat_ptr _tmp631=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_exp2string(unconsume_exp));_tmp4C5.f1=_tmp631;});_tmp4C5;});void*_tmp34D[1U];_tmp34D[0]=& _tmp34F;({unsigned _tmp633=loc;struct _fat_ptr _tmp632=({const char*_tmp34E="aliased expression %s was overwritten";_tag_fat(_tmp34E,sizeof(char),38U);});Cyc_CfFlowInfo_aerr(_tmp633,_tmp632,_tag_fat(_tmp34D,sizeof(void*),1U));});});
# 2386
goto _LLD;}_LLD:;}
# 2405 "new_control_flow.cyc"
goto _LL8;}}else{_LLB: _LLC:
# 2411
 goto _LL8;}_LL8:;}
# 2413
goto _LL0;}}}_LL0:;}
# 2415
return flow;}
# 548 "new_control_flow.cyc"
static union Cyc_CfFlowInfo_FlowInfo Cyc_NewControlFlow_unconsume_params(struct Cyc_NewControlFlow_AnalEnv*env,struct Cyc_List_List*consumed_vals,struct Cyc_List_List*unconsume_exps,int is_region_open,union Cyc_CfFlowInfo_FlowInfo flow,unsigned loc){
# 2434 "new_control_flow.cyc"
{union Cyc_CfFlowInfo_FlowInfo _tmp350=flow;if((_tmp350.BottomFL).tag == 1){_LL1: _LL2:
 return flow;}else{_LL3: _LL4:
 goto _LL0;}_LL0:;}{
# 2438
int do_unconsume=unconsume_exps != 0;
struct Cyc_List_List*_tmp351=Cyc_List_rev(consumed_vals);struct Cyc_List_List*consumed_vals=_tmp351;
struct Cyc_List_List*_tmp352=Cyc_List_rev(unconsume_exps);struct Cyc_List_List*unconsume_exps=_tmp352;
{struct Cyc_List_List*params=consumed_vals;for(0;params != 0;(
params=params->tl,
unconsume_exps != 0?unconsume_exps=unconsume_exps->tl: 0)){
# 2447
struct Cyc_Absyn_Vardecl*vd=Cyc_NewControlFlow_get_vd_from_place((struct Cyc_CfFlowInfo_Place*)params->hd);
struct _tuple30 _stmttmp84=
is_region_open?({struct _tuple30 _tmp4C6;_tmp4C6.f1=1,_tmp4C6.f2=0;_tmp4C6;}):
 Cyc_NewControlFlow_noconsume_place_ok(env,(struct Cyc_CfFlowInfo_Place*)params->hd,do_unconsume,vd,flow,loc);
# 2448
void*_tmp354;int _tmp353;_LL6: _tmp353=_stmttmp84.f1;_tmp354=_stmttmp84.f2;_LL7: {int varok=_tmp353;void*rval=_tmp354;
# 2454
if(do_unconsume)
flow=Cyc_NewControlFlow_unconsume_exp(env,(struct Cyc_Absyn_Exp*)((struct Cyc_List_List*)_check_null(unconsume_exps))->hd,vd,varok,rval,flow,loc);}}}
# 2457
Cyc_NewControlFlow_update_tryflow(env,flow);
return flow;}}struct _tuple31{int f1;struct Cyc_Absyn_Stmt*f2;struct Cyc_List_List*f3;struct Cyc_List_List*f4;};
# 548 "new_control_flow.cyc"
static union Cyc_CfFlowInfo_FlowInfo Cyc_NewControlFlow_anal_scs(struct Cyc_NewControlFlow_AnalEnv*env,union Cyc_CfFlowInfo_FlowInfo inflow,struct Cyc_List_List*scs,unsigned exp_loc){
# 2463 "new_control_flow.cyc"
struct Cyc_CfFlowInfo_FlowEnv*fenv=env->fenv;
for(0;scs != 0;scs=scs->tl){
struct Cyc_Absyn_Switch_clause*_stmttmp85=(struct Cyc_Absyn_Switch_clause*)scs->hd;unsigned _tmp358;struct Cyc_Absyn_Stmt*_tmp357;struct Cyc_Absyn_Exp*_tmp356;struct Cyc_Core_Opt*_tmp355;_LL1: _tmp355=_stmttmp85->pat_vars;_tmp356=_stmttmp85->where_clause;_tmp357=_stmttmp85->body;_tmp358=_stmttmp85->loc;_LL2: {struct Cyc_Core_Opt*vds_opt=_tmp355;struct Cyc_Absyn_Exp*where_opt=_tmp356;struct Cyc_Absyn_Stmt*body=_tmp357;unsigned loc=_tmp358;
struct _tuple1 _stmttmp86=Cyc_NewControlFlow_get_unconsume_pat_vars((struct Cyc_List_List*)((struct Cyc_Core_Opt*)_check_null(vds_opt))->v);struct Cyc_List_List*_tmp35A;struct Cyc_List_List*_tmp359;_LL4: _tmp359=_stmttmp86.f1;_tmp35A=_stmttmp86.f2;_LL5: {struct Cyc_List_List*roots=_tmp359;struct Cyc_List_List*es=_tmp35A;
union Cyc_CfFlowInfo_FlowInfo clause_inflow=Cyc_NewControlFlow_initialize_pat_vars(env->fenv,env,inflow,(struct Cyc_List_List*)vds_opt->v,roots != 0,loc,exp_loc);
# 2471
union Cyc_CfFlowInfo_FlowInfo clause_outflow;
struct Cyc_List_List*old_unique_pat_vars=env->unique_pat_vars;
# 2474
if(Cyc_Tcpat_has_vars(vds_opt))
({struct Cyc_List_List*_tmp635=({struct Cyc_List_List*_tmp35C=_cycalloc(sizeof(*_tmp35C));
({struct _tuple31*_tmp634=({struct _tuple31*_tmp35B=_cycalloc(sizeof(*_tmp35B));_tmp35B->f1=0,_tmp35B->f2=body,_tmp35B->f3=roots,_tmp35B->f4=es;_tmp35B;});_tmp35C->hd=_tmp634;}),_tmp35C->tl=old_unique_pat_vars;_tmp35C;});
# 2475
env->unique_pat_vars=_tmp635;});
# 2474
if(where_opt != 0){
# 2479
struct Cyc_Absyn_Exp*wexp=where_opt;
struct _tuple20 _stmttmp87=Cyc_NewControlFlow_anal_test(env,clause_inflow,wexp);union Cyc_CfFlowInfo_FlowInfo _tmp35E;union Cyc_CfFlowInfo_FlowInfo _tmp35D;_LL7: _tmp35D=_stmttmp87.f1;_tmp35E=_stmttmp87.f2;_LL8: {union Cyc_CfFlowInfo_FlowInfo ft=_tmp35D;union Cyc_CfFlowInfo_FlowInfo ff=_tmp35E;
inflow=ff;
clause_outflow=Cyc_NewControlFlow_anal_stmt(env,ft,body,0);}}else{
# 2484
clause_outflow=Cyc_NewControlFlow_anal_stmt(env,clause_inflow,body,0);}
# 2486
env->unique_pat_vars=old_unique_pat_vars;{
union Cyc_CfFlowInfo_FlowInfo _tmp35F=clause_outflow;if((_tmp35F.BottomFL).tag == 1){_LLA: _LLB:
 goto _LL9;}else{_LLC: _LLD:
# 2491
 clause_outflow=Cyc_NewControlFlow_unconsume_params(env,roots,es,0,clause_outflow,loc);
# 2493
if(scs->tl == 0)
return clause_outflow;else{
# 2498
if((struct Cyc_List_List*)((struct Cyc_Core_Opt*)_check_null(((struct Cyc_Absyn_Switch_clause*)((struct Cyc_List_List*)_check_null(scs->tl))->hd)->pat_vars))->v != 0)
({void*_tmp360=0U;({unsigned _tmp637=body->loc;struct _fat_ptr _tmp636=({const char*_tmp361="switch clause may implicitly fallthru";_tag_fat(_tmp361,sizeof(char),38U);});Cyc_CfFlowInfo_aerr(_tmp637,_tmp636,_tag_fat(_tmp360,sizeof(void*),0U));});});else{
# 2501
({void*_tmp362=0U;({unsigned _tmp639=body->loc;struct _fat_ptr _tmp638=({const char*_tmp363="switch clause may implicitly fallthru";_tag_fat(_tmp363,sizeof(char),38U);});Cyc_Warn_warn(_tmp639,_tmp638,_tag_fat(_tmp362,sizeof(void*),0U));});});}
# 2503
Cyc_NewControlFlow_update_flow(env,((struct Cyc_Absyn_Switch_clause*)((struct Cyc_List_List*)_check_null(scs->tl))->hd)->body,clause_outflow);}
# 2505
goto _LL9;}_LL9:;}}}}
# 2508
return Cyc_CfFlowInfo_BottomFL();}struct _tuple32{struct Cyc_NewControlFlow_AnalEnv*f1;struct Cyc_Dict_Dict f2;unsigned f3;};
# 548 "new_control_flow.cyc"
static void Cyc_NewControlFlow_check_dropped_unique_vd(struct _tuple32*vdenv,struct Cyc_Absyn_Vardecl*vd){
# 2515 "new_control_flow.cyc"
unsigned _tmp366;struct Cyc_Dict_Dict _tmp365;struct Cyc_NewControlFlow_AnalEnv*_tmp364;_LL1: _tmp364=vdenv->f1;_tmp365=vdenv->f2;_tmp366=vdenv->f3;_LL2: {struct Cyc_NewControlFlow_AnalEnv*env=_tmp364;struct Cyc_Dict_Dict fd=_tmp365;unsigned loc=_tmp366;
if(Cyc_Tcutil_is_noalias_pointer_or_aggr(vd->type)){
# 2518
struct Cyc_CfFlowInfo_VarRoot_CfFlowInfo_Root_struct vdroot=({struct Cyc_CfFlowInfo_VarRoot_CfFlowInfo_Root_struct _tmp4C8;_tmp4C8.tag=0U,_tmp4C8.f1=vd;_tmp4C8;});
struct _tuple16 _stmttmp88=Cyc_CfFlowInfo_unname_rval(({struct Cyc_Dict_Dict _tmp63A=fd;Cyc_Dict_lookup_other(_tmp63A,Cyc_CfFlowInfo_root_cmp,(void*)({struct Cyc_CfFlowInfo_VarRoot_CfFlowInfo_Root_struct*_tmp36C=_cycalloc(sizeof(*_tmp36C));*_tmp36C=vdroot;_tmp36C;}));}));void*_tmp367;_LL4: _tmp367=_stmttmp88.f1;_LL5: {void*rval=_tmp367;
void*_tmp368=rval;switch(*((int*)_tmp368)){case 7U: _LL7: _LL8:
 goto _LLA;case 0U: _LL9: _LLA:
 goto _LLC;case 2U: if(((struct Cyc_CfFlowInfo_UnknownR_CfFlowInfo_AbsRVal_struct*)_tmp368)->f1 == Cyc_CfFlowInfo_NoneIL){_LLB: _LLC:
 goto _LL6;}else{goto _LLD;}default: _LLD: _LLE:
# 2525
({struct Cyc_String_pa_PrintArg_struct _tmp36B=({struct Cyc_String_pa_PrintArg_struct _tmp4C7;_tmp4C7.tag=0U,({struct _fat_ptr _tmp63B=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(vd->name));_tmp4C7.f1=_tmp63B;});_tmp4C7;});void*_tmp369[1U];_tmp369[0]=& _tmp36B;({unsigned _tmp63D=loc;struct _fat_ptr _tmp63C=({const char*_tmp36A="unique pointers may still exist after variable %s goes out of scope";_tag_fat(_tmp36A,sizeof(char),68U);});Cyc_Warn_warn(_tmp63D,_tmp63C,_tag_fat(_tmp369,sizeof(void*),1U));});});
# 2527
goto _LL6;}_LL6:;}}}}
# 548 "new_control_flow.cyc"
static void Cyc_NewControlFlow_check_dropped_unique(struct Cyc_NewControlFlow_AnalEnv*env,union Cyc_CfFlowInfo_FlowInfo inflow,struct Cyc_Absyn_Decl*decl){
# 2533 "new_control_flow.cyc"
{union Cyc_CfFlowInfo_FlowInfo _tmp36D=inflow;struct Cyc_Dict_Dict _tmp36E;if((_tmp36D.ReachableFL).tag == 2){_LL1: _tmp36E=((_tmp36D.ReachableFL).val).f1;_LL2: {struct Cyc_Dict_Dict fd=_tmp36E;
# 2535
struct _tuple32 vdenv=({struct _tuple32 _tmp4C9;_tmp4C9.f1=env,_tmp4C9.f2=fd,_tmp4C9.f3=decl->loc;_tmp4C9;});
struct Cyc_CfFlowInfo_FlowEnv*fenv=env->fenv;
{void*_stmttmp89=decl->r;void*_tmp36F=_stmttmp89;struct Cyc_List_List*_tmp370;struct Cyc_List_List*_tmp371;struct Cyc_Absyn_Vardecl*_tmp372;switch(*((int*)_tmp36F)){case 0U: _LL6: _tmp372=((struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct*)_tmp36F)->f1;_LL7: {struct Cyc_Absyn_Vardecl*vd=_tmp372;
# 2539
Cyc_NewControlFlow_check_dropped_unique_vd(& vdenv,vd);
goto _LL5;}case 2U: if(((struct Cyc_Absyn_Let_d_Absyn_Raw_decl_struct*)_tmp36F)->f2 != 0){_LL8: _tmp371=(struct Cyc_List_List*)(((struct Cyc_Absyn_Let_d_Absyn_Raw_decl_struct*)_tmp36F)->f2)->v;_LL9: {struct Cyc_List_List*vds=_tmp371;
# 2542
struct _tuple1 _stmttmp8A=Cyc_List_split(vds);struct Cyc_List_List*_tmp373;_LLF: _tmp373=_stmttmp8A.f1;_LL10: {struct Cyc_List_List*vs=_tmp373;
struct Cyc_List_List*_tmp374=Cyc_Tcutil_filter_nulls(vs);{struct Cyc_List_List*vs=_tmp374;
_tmp370=vs;goto _LLB;}}}}else{goto _LLC;}case 3U: _LLA: _tmp370=((struct Cyc_Absyn_Letv_d_Absyn_Raw_decl_struct*)_tmp36F)->f1;_LLB: {struct Cyc_List_List*vds=_tmp370;
# 2546
({void(*_tmp63E)(void(*f)(struct _tuple32*,struct Cyc_Absyn_Vardecl*),struct _tuple32*env,struct Cyc_List_List*x)=({void(*_tmp375)(void(*f)(struct _tuple32*,struct Cyc_Absyn_Vardecl*),struct _tuple32*env,struct Cyc_List_List*x)=(void(*)(void(*f)(struct _tuple32*,struct Cyc_Absyn_Vardecl*),struct _tuple32*env,struct Cyc_List_List*x))Cyc_List_iter_c;_tmp375;});_tmp63E(Cyc_NewControlFlow_check_dropped_unique_vd,& vdenv,vds);});
goto _LL5;}default: _LLC: _LLD:
 goto _LL5;}_LL5:;}
# 2550
goto _LL0;}}else{_LL3: _LL4:
 goto _LL0;}_LL0:;}
# 2553
return;}
# 548 "new_control_flow.cyc"
static union Cyc_CfFlowInfo_FlowInfo Cyc_NewControlFlow_unconsume_pat_vars(struct Cyc_NewControlFlow_AnalEnv*env,struct Cyc_Absyn_Stmt*src,union Cyc_CfFlowInfo_FlowInfo inflow,struct Cyc_Absyn_Stmt*dest){
# 2563 "new_control_flow.cyc"
int num_pop=({int(*_tmp640)(struct Cyc_Hashtable_Table*t,struct Cyc_Absyn_Stmt*key)=({int(*_tmp37A)(struct Cyc_Hashtable_Table*t,struct Cyc_Absyn_Stmt*key)=(int(*)(struct Cyc_Hashtable_Table*t,struct Cyc_Absyn_Stmt*key))Cyc_Hashtable_lookup;_tmp37A;});struct Cyc_Hashtable_Table*_tmp63F=env->pat_pop_table;_tmp640(_tmp63F,src);});
{struct Cyc_List_List*x=env->unique_pat_vars;for(0;num_pop > 0;(x=x->tl,-- num_pop)){
struct _tuple31*_stmttmp8B=(struct _tuple31*)((struct Cyc_List_List*)_check_null(x))->hd;struct Cyc_List_List*_tmp379;struct Cyc_List_List*_tmp378;struct Cyc_Absyn_Stmt*_tmp377;int _tmp376;_LL1: _tmp376=_stmttmp8B->f1;_tmp377=_stmttmp8B->f2;_tmp378=_stmttmp8B->f3;_tmp379=_stmttmp8B->f4;_LL2: {int is_open=_tmp376;struct Cyc_Absyn_Stmt*pat_stmt=_tmp377;struct Cyc_List_List*roots=_tmp378;struct Cyc_List_List*es=_tmp379;
inflow=Cyc_NewControlFlow_unconsume_params(env,roots,es,is_open,inflow,dest->loc);}}}
# 2568
return inflow;}
# 548 "new_control_flow.cyc"
static union Cyc_CfFlowInfo_FlowInfo Cyc_NewControlFlow_anal_stmt(struct Cyc_NewControlFlow_AnalEnv*env,union Cyc_CfFlowInfo_FlowInfo inflow,struct Cyc_Absyn_Stmt*s,struct _tuple18*rval_opt){
# 2576 "new_control_flow.cyc"
union Cyc_CfFlowInfo_FlowInfo outflow;
struct _tuple21 _stmttmp8C=Cyc_NewControlFlow_pre_stmt_check(env,inflow,s);union Cyc_CfFlowInfo_FlowInfo*_tmp37C;struct Cyc_NewControlFlow_CFStmtAnnot*_tmp37B;_LL1: _tmp37B=_stmttmp8C.f1;_tmp37C=_stmttmp8C.f2;_LL2: {struct Cyc_NewControlFlow_CFStmtAnnot*annot=_tmp37B;union Cyc_CfFlowInfo_FlowInfo*sflow=_tmp37C;
inflow=*sflow;{
struct Cyc_CfFlowInfo_FlowEnv*fenv=env->fenv;
# 2583
void*_stmttmp8D=s->r;void*_tmp37D=_stmttmp8D;struct Cyc_Absyn_Stmt*_tmp37E;struct Cyc_Absyn_Stmt*_tmp380;struct Cyc_Absyn_Decl*_tmp37F;struct Cyc_Absyn_Stmt*_tmp384;unsigned _tmp383;struct Cyc_Absyn_Exp*_tmp382;struct Cyc_Absyn_Vardecl*_tmp381;struct Cyc_Absyn_Stmt*_tmp388;unsigned _tmp387;struct Cyc_Absyn_Exp*_tmp386;struct Cyc_List_List*_tmp385;void*_tmp38B;struct Cyc_List_List*_tmp38A;struct Cyc_Absyn_Stmt*_tmp389;void*_tmp38E;struct Cyc_List_List*_tmp38D;struct Cyc_Absyn_Exp*_tmp38C;struct Cyc_Absyn_Switch_clause*_tmp390;struct Cyc_List_List*_tmp38F;struct Cyc_Absyn_Stmt*_tmp396;struct Cyc_Absyn_Stmt*_tmp395;struct Cyc_Absyn_Exp*_tmp394;struct Cyc_Absyn_Stmt*_tmp393;struct Cyc_Absyn_Exp*_tmp392;struct Cyc_Absyn_Exp*_tmp391;struct Cyc_Absyn_Stmt*_tmp399;struct Cyc_Absyn_Exp*_tmp398;struct Cyc_Absyn_Stmt*_tmp397;struct Cyc_Absyn_Stmt*_tmp39C;struct Cyc_Absyn_Stmt*_tmp39B;struct Cyc_Absyn_Exp*_tmp39A;struct Cyc_Absyn_Stmt*_tmp39F;struct Cyc_Absyn_Stmt*_tmp39E;struct Cyc_Absyn_Exp*_tmp39D;struct Cyc_Absyn_Stmt*_tmp3A1;struct Cyc_Absyn_Stmt*_tmp3A0;struct Cyc_Absyn_Exp*_tmp3A2;struct Cyc_Absyn_Exp*_tmp3A3;switch(*((int*)_tmp37D)){case 0U: _LL4: _LL5:
 return inflow;case 3U: if(((struct Cyc_Absyn_Return_s_Absyn_Raw_stmt_struct*)_tmp37D)->f1 == 0){_LL6: _LL7:
# 2587
 if(env->noreturn)
({void*_tmp3A4=0U;({unsigned _tmp642=s->loc;struct _fat_ptr _tmp641=({const char*_tmp3A5="`noreturn' function might return";_tag_fat(_tmp3A5,sizeof(char),33U);});Cyc_CfFlowInfo_aerr(_tmp642,_tmp641,_tag_fat(_tmp3A4,sizeof(void*),0U));});});
# 2587
Cyc_NewControlFlow_check_init_params(s->loc,env,inflow);
# 2590
Cyc_NewControlFlow_unconsume_params(env,env->noconsume_params,0,0,inflow,s->loc);
return Cyc_CfFlowInfo_BottomFL();}else{_LL8: _tmp3A3=((struct Cyc_Absyn_Return_s_Absyn_Raw_stmt_struct*)_tmp37D)->f1;_LL9: {struct Cyc_Absyn_Exp*e=_tmp3A3;
# 2593
if(env->noreturn)
({void*_tmp3A6=0U;({unsigned _tmp644=s->loc;struct _fat_ptr _tmp643=({const char*_tmp3A7="`noreturn' function might return";_tag_fat(_tmp3A7,sizeof(char),33U);});Cyc_CfFlowInfo_aerr(_tmp644,_tmp643,_tag_fat(_tmp3A6,sizeof(void*),0U));});});{
# 2593
struct _tuple17 _stmttmp8E=
# 2595
Cyc_NewControlFlow_anal_Rexp(env,1,inflow,(struct Cyc_Absyn_Exp*)_check_null(e));
# 2593
void*_tmp3A9;union Cyc_CfFlowInfo_FlowInfo _tmp3A8;_LL2D: _tmp3A8=_stmttmp8E.f1;_tmp3A9=_stmttmp8E.f2;_LL2E: {union Cyc_CfFlowInfo_FlowInfo f=_tmp3A8;void*r=_tmp3A9;
# 2596
f=Cyc_NewControlFlow_use_Rval(env,e->loc,f,r);
Cyc_NewControlFlow_check_init_params(s->loc,env,f);
Cyc_NewControlFlow_unconsume_params(env,env->noconsume_params,0,0,f,s->loc);
return Cyc_CfFlowInfo_BottomFL();}}}}case 1U: _LLA: _tmp3A2=((struct Cyc_Absyn_Exp_s_Absyn_Raw_stmt_struct*)_tmp37D)->f1;_LLB: {struct Cyc_Absyn_Exp*e=_tmp3A2;
# 2602
struct _tuple18*_tmp3AA=rval_opt;int _tmp3AC;void**_tmp3AB;if(_tmp3AA != 0){_LL30: _tmp3AB=(void**)& _tmp3AA->f1;_tmp3AC=_tmp3AA->f2;_LL31: {void**rval=_tmp3AB;int copy_ctxt=_tmp3AC;
# 2604
struct _tuple17 _stmttmp8F=Cyc_NewControlFlow_anal_Rexp(env,copy_ctxt,inflow,e);void*_tmp3AE;union Cyc_CfFlowInfo_FlowInfo _tmp3AD;_LL35: _tmp3AD=_stmttmp8F.f1;_tmp3AE=_stmttmp8F.f2;_LL36: {union Cyc_CfFlowInfo_FlowInfo f=_tmp3AD;void*r=_tmp3AE;
*rval=r;
return f;}}}else{_LL32: _LL33:
# 2608
 return(Cyc_NewControlFlow_anal_Rexp(env,0,inflow,e)).f1;}_LL2F:;}case 2U: _LLC: _tmp3A0=((struct Cyc_Absyn_Seq_s_Absyn_Raw_stmt_struct*)_tmp37D)->f1;_tmp3A1=((struct Cyc_Absyn_Seq_s_Absyn_Raw_stmt_struct*)_tmp37D)->f2;_LLD: {struct Cyc_Absyn_Stmt*s1=_tmp3A0;struct Cyc_Absyn_Stmt*s2=_tmp3A1;
# 2612
return({struct Cyc_NewControlFlow_AnalEnv*_tmp647=env;union Cyc_CfFlowInfo_FlowInfo _tmp646=Cyc_NewControlFlow_anal_stmt(env,inflow,s1,0);struct Cyc_Absyn_Stmt*_tmp645=s2;Cyc_NewControlFlow_anal_stmt(_tmp647,_tmp646,_tmp645,rval_opt);});}case 4U: _LLE: _tmp39D=((struct Cyc_Absyn_IfThenElse_s_Absyn_Raw_stmt_struct*)_tmp37D)->f1;_tmp39E=((struct Cyc_Absyn_IfThenElse_s_Absyn_Raw_stmt_struct*)_tmp37D)->f2;_tmp39F=((struct Cyc_Absyn_IfThenElse_s_Absyn_Raw_stmt_struct*)_tmp37D)->f3;_LLF: {struct Cyc_Absyn_Exp*e=_tmp39D;struct Cyc_Absyn_Stmt*s1=_tmp39E;struct Cyc_Absyn_Stmt*s2=_tmp39F;
# 2615
struct _tuple20 _stmttmp90=Cyc_NewControlFlow_anal_test(env,inflow,e);union Cyc_CfFlowInfo_FlowInfo _tmp3B0;union Cyc_CfFlowInfo_FlowInfo _tmp3AF;_LL38: _tmp3AF=_stmttmp90.f1;_tmp3B0=_stmttmp90.f2;_LL39: {union Cyc_CfFlowInfo_FlowInfo f1t=_tmp3AF;union Cyc_CfFlowInfo_FlowInfo f1f=_tmp3B0;
# 2622
union Cyc_CfFlowInfo_FlowInfo ff=Cyc_NewControlFlow_anal_stmt(env,f1f,s2,0);
union Cyc_CfFlowInfo_FlowInfo ft=Cyc_NewControlFlow_anal_stmt(env,f1t,s1,0);
return Cyc_CfFlowInfo_join_flow(fenv,ft,ff);}}case 5U: _LL10: _tmp39A=(((struct Cyc_Absyn_While_s_Absyn_Raw_stmt_struct*)_tmp37D)->f1).f1;_tmp39B=(((struct Cyc_Absyn_While_s_Absyn_Raw_stmt_struct*)_tmp37D)->f1).f2;_tmp39C=((struct Cyc_Absyn_While_s_Absyn_Raw_stmt_struct*)_tmp37D)->f2;_LL11: {struct Cyc_Absyn_Exp*e=_tmp39A;struct Cyc_Absyn_Stmt*cont=_tmp39B;struct Cyc_Absyn_Stmt*body=_tmp39C;
# 2630
struct _tuple21 _stmttmp91=Cyc_NewControlFlow_pre_stmt_check(env,inflow,cont);union Cyc_CfFlowInfo_FlowInfo*_tmp3B1;_LL3B: _tmp3B1=_stmttmp91.f2;_LL3C: {union Cyc_CfFlowInfo_FlowInfo*eflow_ptr=_tmp3B1;
union Cyc_CfFlowInfo_FlowInfo e_inflow=*eflow_ptr;
struct _tuple20 _stmttmp92=Cyc_NewControlFlow_anal_test(env,e_inflow,e);union Cyc_CfFlowInfo_FlowInfo _tmp3B3;union Cyc_CfFlowInfo_FlowInfo _tmp3B2;_LL3E: _tmp3B2=_stmttmp92.f1;_tmp3B3=_stmttmp92.f2;_LL3F: {union Cyc_CfFlowInfo_FlowInfo f1t=_tmp3B2;union Cyc_CfFlowInfo_FlowInfo f1f=_tmp3B3;
union Cyc_CfFlowInfo_FlowInfo body_outflow=Cyc_NewControlFlow_anal_stmt(env,f1t,body,0);
Cyc_NewControlFlow_update_flow(env,cont,body_outflow);
return f1f;}}}case 14U: _LL12: _tmp397=((struct Cyc_Absyn_Do_s_Absyn_Raw_stmt_struct*)_tmp37D)->f1;_tmp398=(((struct Cyc_Absyn_Do_s_Absyn_Raw_stmt_struct*)_tmp37D)->f2).f1;_tmp399=(((struct Cyc_Absyn_Do_s_Absyn_Raw_stmt_struct*)_tmp37D)->f2).f2;_LL13: {struct Cyc_Absyn_Stmt*body=_tmp397;struct Cyc_Absyn_Exp*e=_tmp398;struct Cyc_Absyn_Stmt*cont=_tmp399;
# 2640
union Cyc_CfFlowInfo_FlowInfo body_outflow=Cyc_NewControlFlow_anal_stmt(env,inflow,body,0);
struct _tuple21 _stmttmp93=Cyc_NewControlFlow_pre_stmt_check(env,body_outflow,cont);union Cyc_CfFlowInfo_FlowInfo*_tmp3B4;_LL41: _tmp3B4=_stmttmp93.f2;_LL42: {union Cyc_CfFlowInfo_FlowInfo*eflow_ptr=_tmp3B4;
union Cyc_CfFlowInfo_FlowInfo e_inflow=*eflow_ptr;
struct _tuple20 _stmttmp94=Cyc_NewControlFlow_anal_test(env,e_inflow,e);union Cyc_CfFlowInfo_FlowInfo _tmp3B6;union Cyc_CfFlowInfo_FlowInfo _tmp3B5;_LL44: _tmp3B5=_stmttmp94.f1;_tmp3B6=_stmttmp94.f2;_LL45: {union Cyc_CfFlowInfo_FlowInfo f1t=_tmp3B5;union Cyc_CfFlowInfo_FlowInfo f1f=_tmp3B6;
Cyc_NewControlFlow_update_flow(env,body,f1t);
return f1f;}}}case 9U: _LL14: _tmp391=((struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct*)_tmp37D)->f1;_tmp392=(((struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct*)_tmp37D)->f2).f1;_tmp393=(((struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct*)_tmp37D)->f2).f2;_tmp394=(((struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct*)_tmp37D)->f3).f1;_tmp395=(((struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct*)_tmp37D)->f3).f2;_tmp396=((struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct*)_tmp37D)->f4;_LL15: {struct Cyc_Absyn_Exp*e1=_tmp391;struct Cyc_Absyn_Exp*e2=_tmp392;struct Cyc_Absyn_Stmt*guard=_tmp393;struct Cyc_Absyn_Exp*e3=_tmp394;struct Cyc_Absyn_Stmt*cont=_tmp395;struct Cyc_Absyn_Stmt*body=_tmp396;
# 2649
union Cyc_CfFlowInfo_FlowInfo e1_outflow=(Cyc_NewControlFlow_anal_Rexp(env,0,inflow,e1)).f1;
struct _tuple21 _stmttmp95=Cyc_NewControlFlow_pre_stmt_check(env,e1_outflow,guard);union Cyc_CfFlowInfo_FlowInfo*_tmp3B7;_LL47: _tmp3B7=_stmttmp95.f2;_LL48: {union Cyc_CfFlowInfo_FlowInfo*e2flow_ptr=_tmp3B7;
union Cyc_CfFlowInfo_FlowInfo e2_inflow=*e2flow_ptr;
struct _tuple20 _stmttmp96=Cyc_NewControlFlow_anal_test(env,e2_inflow,e2);union Cyc_CfFlowInfo_FlowInfo _tmp3B9;union Cyc_CfFlowInfo_FlowInfo _tmp3B8;_LL4A: _tmp3B8=_stmttmp96.f1;_tmp3B9=_stmttmp96.f2;_LL4B: {union Cyc_CfFlowInfo_FlowInfo f2t=_tmp3B8;union Cyc_CfFlowInfo_FlowInfo f2f=_tmp3B9;
union Cyc_CfFlowInfo_FlowInfo body_outflow=Cyc_NewControlFlow_anal_stmt(env,f2t,body,0);
struct _tuple21 _stmttmp97=Cyc_NewControlFlow_pre_stmt_check(env,body_outflow,cont);union Cyc_CfFlowInfo_FlowInfo*_tmp3BA;_LL4D: _tmp3BA=_stmttmp97.f2;_LL4E: {union Cyc_CfFlowInfo_FlowInfo*e3flow_ptr=_tmp3BA;
union Cyc_CfFlowInfo_FlowInfo e3_inflow=*e3flow_ptr;
union Cyc_CfFlowInfo_FlowInfo e3_outflow=(Cyc_NewControlFlow_anal_Rexp(env,0,e3_inflow,e3)).f1;
Cyc_NewControlFlow_update_flow(env,guard,e3_outflow);
return f2f;}}}}case 11U: if(((struct Cyc_Absyn_Fallthru_s_Absyn_Raw_stmt_struct*)_tmp37D)->f2 != 0){_LL16: _tmp38F=((struct Cyc_Absyn_Fallthru_s_Absyn_Raw_stmt_struct*)_tmp37D)->f1;_tmp390=*((struct Cyc_Absyn_Fallthru_s_Absyn_Raw_stmt_struct*)_tmp37D)->f2;_LL17: {struct Cyc_List_List*es=_tmp38F;struct Cyc_Absyn_Switch_clause*destclause=_tmp390;
# 2661
struct _tuple24 _stmttmp98=Cyc_NewControlFlow_anal_Rexps(env,inflow,es,1,1);struct Cyc_List_List*_tmp3BC;union Cyc_CfFlowInfo_FlowInfo _tmp3BB;_LL50: _tmp3BB=_stmttmp98.f1;_tmp3BC=_stmttmp98.f2;_LL51: {union Cyc_CfFlowInfo_FlowInfo f=_tmp3BB;struct Cyc_List_List*rvals=_tmp3BC;
# 2663
inflow=Cyc_NewControlFlow_unconsume_pat_vars(env,s,inflow,destclause->body);{
# 2665
struct Cyc_List_List*vds=Cyc_Tcutil_filter_nulls((Cyc_List_split((struct Cyc_List_List*)((struct Cyc_Core_Opt*)_check_null(destclause->pat_vars))->v)).f1);
f=Cyc_NewControlFlow_add_vars(fenv,f,vds,fenv->unknown_all,s->loc,0);
# 2668
{struct Cyc_List_List*x=(struct Cyc_List_List*)((struct Cyc_Core_Opt*)_check_null(destclause->pat_vars))->v;for(0;x != 0;x=x->tl){
struct _tuple25*_stmttmp99=(struct _tuple25*)x->hd;struct Cyc_Absyn_Exp*_tmp3BE;struct Cyc_Absyn_Vardecl**_tmp3BD;_LL53: _tmp3BD=_stmttmp99->f1;_tmp3BE=_stmttmp99->f2;_LL54: {struct Cyc_Absyn_Vardecl**vd=_tmp3BD;struct Cyc_Absyn_Exp*ve=_tmp3BE;
if(vd != 0){
f=({struct Cyc_CfFlowInfo_FlowEnv*_tmp64D=fenv;struct Cyc_NewControlFlow_AnalEnv*_tmp64C=env;union Cyc_CfFlowInfo_FlowInfo _tmp64B=f;struct Cyc_Absyn_Vardecl*_tmp64A=*vd;struct Cyc_Absyn_Exp*_tmp649=(struct Cyc_Absyn_Exp*)((struct Cyc_List_List*)_check_null(es))->hd;void*_tmp648=(void*)((struct Cyc_List_List*)_check_null(rvals))->hd;Cyc_NewControlFlow_do_initialize_var(_tmp64D,_tmp64C,_tmp64B,_tmp64A,_tmp649,_tmp648,s->loc);});
rvals=rvals->tl;
es=es->tl;}}}}
# 2676
Cyc_NewControlFlow_update_flow(env,destclause->body,f);
return Cyc_CfFlowInfo_BottomFL();}}}}else{_LL2A: _LL2B:
# 2840
({void*_tmp3DF=0U;({int(*_tmp64F)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp3E1)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmp3E1;});struct _fat_ptr _tmp64E=({const char*_tmp3E0="anal_stmt: bad stmt syntax or unimplemented stmt form";_tag_fat(_tmp3E0,sizeof(char),54U);});_tmp64F(_tmp64E,_tag_fat(_tmp3DF,sizeof(void*),0U));});});}case 6U: _LL18: _LL19:
# 2682
 if(({struct Cyc_Absyn_Stmt*(*_tmp651)(struct Cyc_Hashtable_Table*t,struct Cyc_Absyn_Stmt*key)=({struct Cyc_Absyn_Stmt*(*_tmp3BF)(struct Cyc_Hashtable_Table*t,struct Cyc_Absyn_Stmt*key)=(struct Cyc_Absyn_Stmt*(*)(struct Cyc_Hashtable_Table*t,struct Cyc_Absyn_Stmt*key))Cyc_Hashtable_lookup;_tmp3BF;});struct Cyc_Hashtable_Table*_tmp650=env->succ_table;_tmp651(_tmp650,s);})== 0){
{union Cyc_CfFlowInfo_FlowInfo _tmp3C0=inflow;if((_tmp3C0.ReachableFL).tag == 2){_LL56: _LL57:
# 2685
 if(!Cyc_Tcutil_is_void_type(env->return_type)){
if(Cyc_Tcutil_is_any_float_type(env->return_type)||
 Cyc_Tcutil_is_any_int_type(env->return_type))
({void*_tmp3C1=0U;({unsigned _tmp653=s->loc;struct _fat_ptr _tmp652=({const char*_tmp3C2="break may cause function not to return a value";_tag_fat(_tmp3C2,sizeof(char),47U);});Cyc_Warn_warn(_tmp653,_tmp652,_tag_fat(_tmp3C1,sizeof(void*),0U));});});else{
# 2690
({void*_tmp3C3=0U;({unsigned _tmp655=s->loc;struct _fat_ptr _tmp654=({const char*_tmp3C4="break may cause function not to return a value";_tag_fat(_tmp3C4,sizeof(char),47U);});Cyc_CfFlowInfo_aerr(_tmp655,_tmp654,_tag_fat(_tmp3C3,sizeof(void*),0U));});});}}
# 2685
goto _LL55;}else{_LL58: _LL59:
# 2693
 goto _LL55;}_LL55:;}
# 2695
if(env->noreturn)
({void*_tmp3C5=0U;({unsigned _tmp657=s->loc;struct _fat_ptr _tmp656=({const char*_tmp3C6="`noreturn' function might return";_tag_fat(_tmp3C6,sizeof(char),33U);});Cyc_CfFlowInfo_aerr(_tmp657,_tmp656,_tag_fat(_tmp3C5,sizeof(void*),0U));});});
# 2695
Cyc_NewControlFlow_check_init_params(s->loc,env,inflow);
# 2698
Cyc_NewControlFlow_unconsume_params(env,env->noconsume_params,0,0,inflow,s->loc);
return Cyc_CfFlowInfo_BottomFL();}
# 2682
goto _LL1B;case 7U: _LL1A: _LL1B:
# 2702
 goto _LL1D;case 8U: _LL1C: _LL1D: {
# 2705
struct Cyc_Absyn_Stmt*dest=({struct Cyc_Absyn_Stmt*(*_tmp659)(struct Cyc_Hashtable_Table*t,struct Cyc_Absyn_Stmt*key)=({struct Cyc_Absyn_Stmt*(*_tmp3C9)(struct Cyc_Hashtable_Table*t,struct Cyc_Absyn_Stmt*key)=(struct Cyc_Absyn_Stmt*(*)(struct Cyc_Hashtable_Table*t,struct Cyc_Absyn_Stmt*key))Cyc_Hashtable_lookup;_tmp3C9;});struct Cyc_Hashtable_Table*_tmp658=env->succ_table;_tmp659(_tmp658,s);});
if(dest == 0)
({void*_tmp3C7=0U;({unsigned _tmp65B=s->loc;struct _fat_ptr _tmp65A=({const char*_tmp3C8="jump has no target (should be impossible)";_tag_fat(_tmp3C8,sizeof(char),42U);});Cyc_CfFlowInfo_aerr(_tmp65B,_tmp65A,_tag_fat(_tmp3C7,sizeof(void*),0U));});});
# 2706
inflow=
# 2708
Cyc_NewControlFlow_unconsume_pat_vars(env,s,inflow,(struct Cyc_Absyn_Stmt*)_check_null(dest));
# 2710
Cyc_NewControlFlow_update_flow(env,dest,inflow);
return Cyc_CfFlowInfo_BottomFL();}case 10U: _LL1E: _tmp38C=((struct Cyc_Absyn_Switch_s_Absyn_Raw_stmt_struct*)_tmp37D)->f1;_tmp38D=((struct Cyc_Absyn_Switch_s_Absyn_Raw_stmt_struct*)_tmp37D)->f2;_tmp38E=(void*)((struct Cyc_Absyn_Switch_s_Absyn_Raw_stmt_struct*)_tmp37D)->f3;_LL1F: {struct Cyc_Absyn_Exp*e=_tmp38C;struct Cyc_List_List*scs=_tmp38D;void*dec_tree=_tmp38E;
# 2716
return Cyc_NewControlFlow_anal_scs(env,inflow,scs,e->loc);}case 15U: _LL20: _tmp389=((struct Cyc_Absyn_TryCatch_s_Absyn_Raw_stmt_struct*)_tmp37D)->f1;_tmp38A=((struct Cyc_Absyn_TryCatch_s_Absyn_Raw_stmt_struct*)_tmp37D)->f2;_tmp38B=(void*)((struct Cyc_Absyn_TryCatch_s_Absyn_Raw_stmt_struct*)_tmp37D)->f3;_LL21: {struct Cyc_Absyn_Stmt*s1=_tmp389;struct Cyc_List_List*scs=_tmp38A;void*dec_tree=_tmp38B;
# 2721
int old_in_try=env->in_try;
union Cyc_CfFlowInfo_FlowInfo old_tryflow=env->tryflow;
env->in_try=1;
env->tryflow=inflow;{
union Cyc_CfFlowInfo_FlowInfo s1_outflow=Cyc_NewControlFlow_anal_stmt(env,inflow,s1,0);
union Cyc_CfFlowInfo_FlowInfo scs_inflow=env->tryflow;
# 2729
env->in_try=old_in_try;
env->tryflow=old_tryflow;
# 2732
Cyc_NewControlFlow_update_tryflow(env,scs_inflow);{
union Cyc_CfFlowInfo_FlowInfo scs_outflow=Cyc_NewControlFlow_anal_scs(env,scs_inflow,scs,0U);
{union Cyc_CfFlowInfo_FlowInfo _tmp3CA=scs_outflow;if((_tmp3CA.BottomFL).tag == 1){_LL5B: _LL5C:
 goto _LL5A;}else{_LL5D: _LL5E:
({void*_tmp3CB=0U;({unsigned _tmp65D=s->loc;struct _fat_ptr _tmp65C=({const char*_tmp3CC="last catch clause may implicitly fallthru";_tag_fat(_tmp3CC,sizeof(char),42U);});Cyc_CfFlowInfo_aerr(_tmp65D,_tmp65C,_tag_fat(_tmp3CB,sizeof(void*),0U));});});}_LL5A:;}
# 2738
outflow=s1_outflow;
# 2740
return outflow;}}}case 12U: switch(*((int*)((struct Cyc_Absyn_Decl*)((struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct*)_tmp37D)->f1)->r)){case 2U: if(((struct Cyc_Absyn_Let_d_Absyn_Raw_decl_struct*)((struct Cyc_Absyn_Decl*)((struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct*)_tmp37D)->f1)->r)->f2 != 0){_LL22: _tmp385=(struct Cyc_List_List*)(((struct Cyc_Absyn_Let_d_Absyn_Raw_decl_struct*)(((struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct*)_tmp37D)->f1)->r)->f2)->v;_tmp386=((struct Cyc_Absyn_Let_d_Absyn_Raw_decl_struct*)(((struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct*)_tmp37D)->f1)->r)->f3;_tmp387=(((struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct*)_tmp37D)->f1)->loc;_tmp388=((struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct*)_tmp37D)->f2;_LL23: {struct Cyc_List_List*vds=_tmp385;struct Cyc_Absyn_Exp*e=_tmp386;unsigned loc=_tmp387;struct Cyc_Absyn_Stmt*s1=_tmp388;
# 2750
struct _tuple1 _stmttmp9A=Cyc_NewControlFlow_get_unconsume_pat_vars(vds);struct Cyc_List_List*_tmp3CE;struct Cyc_List_List*_tmp3CD;_LL60: _tmp3CD=_stmttmp9A.f1;_tmp3CE=_stmttmp9A.f2;_LL61: {struct Cyc_List_List*roots=_tmp3CD;struct Cyc_List_List*es=_tmp3CE;
inflow=Cyc_NewControlFlow_initialize_pat_vars(fenv,env,inflow,vds,roots != 0,loc,e->loc);{
struct Cyc_List_List*old_unique_pat_vars=env->unique_pat_vars;
# 2754
({struct Cyc_List_List*_tmp65F=({struct Cyc_List_List*_tmp3D0=_cycalloc(sizeof(*_tmp3D0));
({struct _tuple31*_tmp65E=({struct _tuple31*_tmp3CF=_cycalloc(sizeof(*_tmp3CF));_tmp3CF->f1=0,_tmp3CF->f2=s,_tmp3CF->f3=roots,_tmp3CF->f4=es;_tmp3CF;});_tmp3D0->hd=_tmp65E;}),_tmp3D0->tl=old_unique_pat_vars;_tmp3D0;});
# 2754
env->unique_pat_vars=_tmp65F;});
# 2759
inflow=Cyc_NewControlFlow_anal_stmt(env,inflow,s1,rval_opt);
env->unique_pat_vars=old_unique_pat_vars;
# 2763
inflow=Cyc_NewControlFlow_unconsume_params(env,roots,es,0,inflow,loc);
# 2767
return inflow;}}}}else{goto _LL26;}case 4U: _LL24: _tmp381=((struct Cyc_Absyn_Region_d_Absyn_Raw_decl_struct*)(((struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct*)_tmp37D)->f1)->r)->f2;_tmp382=((struct Cyc_Absyn_Region_d_Absyn_Raw_decl_struct*)(((struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct*)_tmp37D)->f1)->r)->f3;_tmp383=(((struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct*)_tmp37D)->f1)->loc;_tmp384=((struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct*)_tmp37D)->f2;if(_tmp382 != 0){_LL25: {struct Cyc_Absyn_Vardecl*vd=_tmp381;struct Cyc_Absyn_Exp*open_exp_opt=_tmp382;unsigned loc=_tmp383;struct Cyc_Absyn_Stmt*s1=_tmp384;
# 2779
struct Cyc_List_List l=({struct Cyc_List_List _tmp4CB;_tmp4CB.hd=open_exp_opt,_tmp4CB.tl=0;_tmp4CB;});
union Cyc_CfFlowInfo_FlowInfo f=Cyc_NewControlFlow_expand_unique_places(env,inflow,& l);
struct _tuple19 _stmttmp9B=Cyc_NewControlFlow_anal_Lexp(env,f,0,0,open_exp_opt);union Cyc_CfFlowInfo_AbsLVal _tmp3D1;_LL63: _tmp3D1=_stmttmp9B.f2;_LL64: {union Cyc_CfFlowInfo_AbsLVal lval=_tmp3D1;
struct _tuple17 _stmttmp9C=Cyc_NewControlFlow_anal_Rexp(env,1,f,open_exp_opt);union Cyc_CfFlowInfo_FlowInfo _tmp3D2;_LL66: _tmp3D2=_stmttmp9C.f1;_LL67: {union Cyc_CfFlowInfo_FlowInfo f=_tmp3D2;
struct Cyc_List_List*roots=0;
struct Cyc_List_List*es=0;
{union Cyc_CfFlowInfo_FlowInfo _tmp3D3=f;struct Cyc_List_List*_tmp3D5;struct Cyc_Dict_Dict _tmp3D4;if((_tmp3D3.ReachableFL).tag == 2){_LL69: _tmp3D4=((_tmp3D3.ReachableFL).val).f1;_tmp3D5=((_tmp3D3.ReachableFL).val).f2;_LL6A: {struct Cyc_Dict_Dict fd=_tmp3D4;struct Cyc_List_List*relns=_tmp3D5;
# 2787
{union Cyc_CfFlowInfo_AbsLVal _tmp3D6=lval;struct Cyc_CfFlowInfo_Place*_tmp3D7;if((_tmp3D6.PlaceL).tag == 1){_LL6E: _tmp3D7=(_tmp3D6.PlaceL).val;_LL6F: {struct Cyc_CfFlowInfo_Place*p=_tmp3D7;
# 2791
void*new_rval=Cyc_CfFlowInfo_lookup_place(fd,p);
new_rval=(void*)({struct Cyc_CfFlowInfo_NamedLocation_CfFlowInfo_AbsRVal_struct*_tmp3D8=_cycalloc(sizeof(*_tmp3D8));_tmp3D8->tag=8U,_tmp3D8->f1=vd,_tmp3D8->f2=new_rval;_tmp3D8;});
fd=Cyc_CfFlowInfo_assign_place(fenv,open_exp_opt->loc,fd,p,new_rval);
f=Cyc_CfFlowInfo_ReachableFL(fd,relns);{
struct Cyc_CfFlowInfo_VarRoot_CfFlowInfo_Root_struct*root=({struct Cyc_CfFlowInfo_VarRoot_CfFlowInfo_Root_struct*_tmp3DC=_cycalloc(sizeof(*_tmp3DC));_tmp3DC->tag=0U,_tmp3DC->f1=vd;_tmp3DC;});
struct Cyc_CfFlowInfo_Place*rp=({struct Cyc_CfFlowInfo_Place*_tmp3DB=_cycalloc(sizeof(*_tmp3DB));_tmp3DB->root=(void*)root,_tmp3DB->path=0;_tmp3DB;});
roots=({struct Cyc_List_List*_tmp3D9=_cycalloc(sizeof(*_tmp3D9));_tmp3D9->hd=rp,_tmp3D9->tl=roots;_tmp3D9;});
es=({struct Cyc_List_List*_tmp3DA=_cycalloc(sizeof(*_tmp3DA));_tmp3DA->hd=open_exp_opt,_tmp3DA->tl=es;_tmp3DA;});
goto _LL6D;}}}else{_LL70: _LL71:
# 2806
 goto _LL6D;}_LL6D:;}
# 2808
goto _LL68;}}else{_LL6B: _LL6C:
# 2810
 goto _LL68;}_LL68:;}{
# 2813
struct Cyc_List_List vds=({struct Cyc_List_List _tmp4CA;_tmp4CA.hd=vd,_tmp4CA.tl=0;_tmp4CA;});
f=Cyc_NewControlFlow_add_vars(fenv,f,& vds,fenv->unknown_all,loc,0);{
# 2817
struct Cyc_List_List*old_unique_pat_vars=env->unique_pat_vars;
({struct Cyc_List_List*_tmp661=({struct Cyc_List_List*_tmp3DE=_cycalloc(sizeof(*_tmp3DE));({struct _tuple31*_tmp660=({struct _tuple31*_tmp3DD=_cycalloc(sizeof(*_tmp3DD));_tmp3DD->f1=1,_tmp3DD->f2=s,_tmp3DD->f3=roots,_tmp3DD->f4=es;_tmp3DD;});_tmp3DE->hd=_tmp660;}),_tmp3DE->tl=old_unique_pat_vars;_tmp3DE;});env->unique_pat_vars=_tmp661;});
# 2822
f=Cyc_NewControlFlow_anal_stmt(env,f,s1,rval_opt);
env->unique_pat_vars=old_unique_pat_vars;
# 2826
f=Cyc_NewControlFlow_unconsume_params(env,roots,es,1,f,loc);
# 2830
return f;}}}}}}else{goto _LL26;}default: _LL26: _tmp37F=((struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct*)_tmp37D)->f1;_tmp380=((struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct*)_tmp37D)->f2;_LL27: {struct Cyc_Absyn_Decl*d=_tmp37F;struct Cyc_Absyn_Stmt*s=_tmp380;
# 2833
outflow=({struct Cyc_NewControlFlow_AnalEnv*_tmp664=env;union Cyc_CfFlowInfo_FlowInfo _tmp663=Cyc_NewControlFlow_anal_decl(env,inflow,d);struct Cyc_Absyn_Stmt*_tmp662=s;Cyc_NewControlFlow_anal_stmt(_tmp664,_tmp663,_tmp662,rval_opt);});
if(Cyc_NewControlFlow_warn_lose_unique)
Cyc_NewControlFlow_check_dropped_unique(env,outflow,d);
# 2834
return outflow;}}default: _LL28: _tmp37E=((struct Cyc_Absyn_Label_s_Absyn_Raw_stmt_struct*)_tmp37D)->f2;_LL29: {struct Cyc_Absyn_Stmt*s=_tmp37E;
# 2838
return Cyc_NewControlFlow_anal_stmt(env,inflow,s,rval_opt);}}_LL3:;}}}
# 548 "new_control_flow.cyc"
static void Cyc_NewControlFlow_check_nested_fun(struct Cyc_JumpAnalysis_Jump_Anal_Result*tables,struct Cyc_CfFlowInfo_FlowEnv*,union Cyc_CfFlowInfo_FlowInfo inflow,struct Cyc_Absyn_Fndecl*fd);
# 2848 "new_control_flow.cyc"
static union Cyc_CfFlowInfo_FlowInfo Cyc_NewControlFlow_anal_decl(struct Cyc_NewControlFlow_AnalEnv*env,union Cyc_CfFlowInfo_FlowInfo inflow,struct Cyc_Absyn_Decl*decl){
struct Cyc_CfFlowInfo_FlowEnv*fenv=env->fenv;
void*_stmttmp9D=decl->r;void*_tmp3E2=_stmttmp9D;struct Cyc_Absyn_Exp*_tmp3E5;struct Cyc_Absyn_Vardecl*_tmp3E4;struct Cyc_Absyn_Tvar*_tmp3E3;struct Cyc_Absyn_Fndecl*_tmp3E6;struct Cyc_List_List*_tmp3E7;struct Cyc_Absyn_Vardecl*_tmp3E8;switch(*((int*)_tmp3E2)){case 0U: _LL1: _tmp3E8=((struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct*)_tmp3E2)->f1;_LL2: {struct Cyc_Absyn_Vardecl*vd=_tmp3E8;
# 2856
if((int)vd->sc == (int)Cyc_Absyn_Static)
return inflow;{
# 2861
struct Cyc_List_List vds=({struct Cyc_List_List _tmp4CC;_tmp4CC.hd=vd,_tmp4CC.tl=0;_tmp4CC;});
inflow=Cyc_NewControlFlow_add_vars(fenv,inflow,& vds,fenv->unknown_none,decl->loc,0);{
struct Cyc_Absyn_Exp*e=vd->initializer;
if(e == 0)
return inflow;{
# 2864
struct _tuple17 _stmttmp9E=
# 2866
Cyc_NewControlFlow_anal_Rexp(env,1,inflow,e);
# 2864
void*_tmp3EA;union Cyc_CfFlowInfo_FlowInfo _tmp3E9;_LLC: _tmp3E9=_stmttmp9E.f1;_tmp3EA=_stmttmp9E.f2;_LLD: {union Cyc_CfFlowInfo_FlowInfo f=_tmp3E9;void*r=_tmp3EA;
# 2867
return Cyc_NewControlFlow_do_initialize_var(fenv,env,f,vd,e,r,decl->loc);}}}}}case 3U: _LL3: _tmp3E7=((struct Cyc_Absyn_Letv_d_Absyn_Raw_decl_struct*)_tmp3E2)->f1;_LL4: {struct Cyc_List_List*vds=_tmp3E7;
# 2870
return Cyc_NewControlFlow_add_vars(fenv,inflow,vds,fenv->unknown_none,decl->loc,0);}case 1U: _LL5: _tmp3E6=((struct Cyc_Absyn_Fn_d_Absyn_Raw_decl_struct*)_tmp3E2)->f1;_LL6: {struct Cyc_Absyn_Fndecl*fd=_tmp3E6;
# 2873
Cyc_NewControlFlow_check_nested_fun(env->all_tables,fenv,inflow,fd);{
struct Cyc_Absyn_Vardecl*vd=(struct Cyc_Absyn_Vardecl*)_check_null(fd->fn_vardecl);
# 2878
return({struct Cyc_CfFlowInfo_FlowEnv*_tmp668=fenv;union Cyc_CfFlowInfo_FlowInfo _tmp667=inflow;struct Cyc_List_List*_tmp666=({struct Cyc_List_List*_tmp3EB=_cycalloc(sizeof(*_tmp3EB));_tmp3EB->hd=vd,_tmp3EB->tl=0;_tmp3EB;});void*_tmp665=fenv->unknown_all;Cyc_NewControlFlow_add_vars(_tmp668,_tmp667,_tmp666,_tmp665,decl->loc,0);});}}case 4U: _LL7: _tmp3E3=((struct Cyc_Absyn_Region_d_Absyn_Raw_decl_struct*)_tmp3E2)->f1;_tmp3E4=((struct Cyc_Absyn_Region_d_Absyn_Raw_decl_struct*)_tmp3E2)->f2;_tmp3E5=((struct Cyc_Absyn_Region_d_Absyn_Raw_decl_struct*)_tmp3E2)->f3;_LL8: {struct Cyc_Absyn_Tvar*tv=_tmp3E3;struct Cyc_Absyn_Vardecl*vd=_tmp3E4;struct Cyc_Absyn_Exp*open_exp_opt=_tmp3E5;
# 2881
if(open_exp_opt != 0)
({void*_tmp3EC=0U;({int(*_tmp66A)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp3EE)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmp3EE;});struct _fat_ptr _tmp669=({const char*_tmp3ED="found open expression in declaration!";_tag_fat(_tmp3ED,sizeof(char),38U);});_tmp66A(_tmp669,_tag_fat(_tmp3EC,sizeof(void*),0U));});});{
# 2881
struct Cyc_List_List vds=({struct Cyc_List_List _tmp4CD;_tmp4CD.hd=vd,_tmp4CD.tl=0;_tmp4CD;});
# 2884
return Cyc_NewControlFlow_add_vars(fenv,inflow,& vds,fenv->unknown_all,decl->loc,0);}}default: _LL9: _LLA:
# 2887
({void*_tmp3EF=0U;({int(*_tmp66C)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp3F1)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos;_tmp3F1;});struct _fat_ptr _tmp66B=({const char*_tmp3F0="anal_decl: unexpected decl variant";_tag_fat(_tmp3F0,sizeof(char),35U);});_tmp66C(_tmp66B,_tag_fat(_tmp3EF,sizeof(void*),0U));});});}_LL0:;}
# 2848
static void Cyc_NewControlFlow_check_fun(struct Cyc_JumpAnalysis_Jump_Anal_Result*tables,struct Cyc_Absyn_Fndecl*fd){
# 2896
struct _handler_cons _tmp3F2;_push_handler(& _tmp3F2);{int _tmp3F4=0;if(setjmp(_tmp3F2.handler))_tmp3F4=1;if(!_tmp3F4){
{struct Cyc_CfFlowInfo_FlowEnv*fenv=Cyc_CfFlowInfo_new_flow_env();
({struct Cyc_JumpAnalysis_Jump_Anal_Result*_tmp66F=tables;struct Cyc_CfFlowInfo_FlowEnv*_tmp66E=fenv;union Cyc_CfFlowInfo_FlowInfo _tmp66D=Cyc_CfFlowInfo_ReachableFL(fenv->mt_flowdict,0);Cyc_NewControlFlow_check_nested_fun(_tmp66F,_tmp66E,_tmp66D,fd);});}
# 2897
;_pop_handler();}else{void*_tmp3F3=(void*)Cyc_Core_get_exn_thrown();void*_tmp3F5=_tmp3F3;void*_tmp3F6;if(((struct Cyc_Dict_Absent_exn_struct*)_tmp3F5)->tag == Cyc_Dict_Absent){_LL1: _LL2:
# 2903
 if(Cyc_Position_num_errors > 0)
goto _LL0;else{
Cyc_Core_rethrow((void*)({struct Cyc_Dict_Absent_exn_struct*_tmp3F7=_cycalloc(sizeof(*_tmp3F7));_tmp3F7->tag=Cyc_Dict_Absent;_tmp3F7;}));}}else{_LL3: _tmp3F6=_tmp3F5;_LL4: {void*exn=_tmp3F6;(int)_rethrow(exn);}}_LL0:;}}}
# 2848
static int Cyc_NewControlFlow_hash_ptr(void*s){
# 2910
return(int)s;}
# 2848
static union Cyc_Relations_RelnOp Cyc_NewControlFlow_translate_rop(struct Cyc_List_List*vds,union Cyc_Relations_RelnOp r){
# 2915
union Cyc_Relations_RelnOp _tmp3F8=r;unsigned _tmp3F9;if((_tmp3F8.RParam).tag == 5){_LL1: _tmp3F9=(_tmp3F8.RParam).val;_LL2: {unsigned i=_tmp3F9;
# 2917
struct Cyc_Absyn_Vardecl*vd=({struct Cyc_Absyn_Vardecl*(*_tmp671)(struct Cyc_List_List*x,int n)=({struct Cyc_Absyn_Vardecl*(*_tmp3FA)(struct Cyc_List_List*x,int n)=(struct Cyc_Absyn_Vardecl*(*)(struct Cyc_List_List*x,int n))Cyc_List_nth;_tmp3FA;});struct Cyc_List_List*_tmp670=vds;_tmp671(_tmp670,(int)i);});
if(!vd->escapes)
return Cyc_Relations_RVar(vd);
# 2918
return r;}}else{_LL3: _LL4:
# 2921
 return r;}_LL0:;}
# 2848
static struct Cyc_List_List*Cyc_NewControlFlow_get_noconsume_params(struct Cyc_List_List*param_vardecls,struct Cyc_List_List*atts){
# 2930
struct _RegionHandle _tmp3FB=_new_region("r");struct _RegionHandle*r=& _tmp3FB;_push_region(r);
{int len=Cyc_List_length(param_vardecls);
struct _fat_ptr cons=({unsigned _tmp404=(unsigned)len;int*_tmp403=({struct _RegionHandle*_tmp672=r;_region_malloc(_tmp672,_check_times(_tmp404,sizeof(int)));});({{unsigned _tmp4CE=(unsigned)len;unsigned i;for(i=0;i < _tmp4CE;++ i){_tmp403[i]=0;}}0;});_tag_fat(_tmp403,sizeof(int),_tmp404);});
for(0;atts != 0;atts=atts->tl){
void*_stmttmp9F=(void*)atts->hd;void*_tmp3FC=_stmttmp9F;int _tmp3FD;int _tmp3FE;switch(*((int*)_tmp3FC)){case 22U: _LL1: _tmp3FE=((struct Cyc_Absyn_Consume_att_Absyn_Attribute_struct*)_tmp3FC)->f1;_LL2: {int i=_tmp3FE;
_tmp3FD=i;goto _LL4;}case 21U: _LL3: _tmp3FD=((struct Cyc_Absyn_Noliveunique_att_Absyn_Attribute_struct*)_tmp3FC)->f1;_LL4: {int i=_tmp3FD;
*((int*)_check_fat_subscript(cons,sizeof(int),i - 1))=1;goto _LL0;}default: _LL5: _LL6:
 goto _LL0;}_LL0:;}{
# 2940
struct Cyc_List_List*noconsume_roots=0;
{int i=0;for(0;param_vardecls != 0;(param_vardecls=param_vardecls->tl,i ++)){
struct Cyc_Absyn_Vardecl*vd=(struct Cyc_Absyn_Vardecl*)param_vardecls->hd;
if(Cyc_Tcutil_is_noalias_pointer(vd->type,0)&& !(*((int*)_check_fat_subscript(cons,sizeof(int),i)))){
struct Cyc_CfFlowInfo_VarRoot_CfFlowInfo_Root_struct*root=({struct Cyc_CfFlowInfo_VarRoot_CfFlowInfo_Root_struct*_tmp401=_cycalloc(sizeof(*_tmp401));_tmp401->tag=0U,_tmp401->f1=vd;_tmp401;});
struct Cyc_CfFlowInfo_Place*rp=({struct Cyc_CfFlowInfo_Place*_tmp400=_cycalloc(sizeof(*_tmp400));_tmp400->root=(void*)root,_tmp400->path=0;_tmp400;});
noconsume_roots=({struct Cyc_List_List*_tmp3FF=_cycalloc(sizeof(*_tmp3FF));_tmp3FF->hd=rp,_tmp3FF->tl=noconsume_roots;_tmp3FF;});}}}{
# 2949
struct Cyc_List_List*_tmp402=noconsume_roots;_npop_handler(0U);return _tmp402;}}}
# 2931
;_pop_region(r);}
# 2848
static void Cyc_NewControlFlow_check_nested_fun(struct Cyc_JumpAnalysis_Jump_Anal_Result*tables,struct Cyc_CfFlowInfo_FlowEnv*fenv,union Cyc_CfFlowInfo_FlowInfo inflow,struct Cyc_Absyn_Fndecl*fd){
# 2956
unsigned loc=(fd->body)->loc;
inflow=Cyc_NewControlFlow_add_vars(fenv,inflow,(struct Cyc_List_List*)((struct Cyc_Core_Opt*)_check_null(fd->param_vardecls))->v,fenv->unknown_all,loc,1);{
# 2961
struct Cyc_List_List*param_roots=0;
struct _tuple15 _stmttmpA0=({union Cyc_CfFlowInfo_FlowInfo _tmp427=inflow;if((_tmp427.ReachableFL).tag != 2)_throw_match();(_tmp427.ReachableFL).val;});struct Cyc_List_List*_tmp406;struct Cyc_Dict_Dict _tmp405;_LL1: _tmp405=_stmttmpA0.f1;_tmp406=_stmttmpA0.f2;_LL2: {struct Cyc_Dict_Dict d=_tmp405;struct Cyc_List_List*relns=_tmp406;
# 2965
struct Cyc_List_List*vardecls=(struct Cyc_List_List*)((struct Cyc_Core_Opt*)_check_null(fd->param_vardecls))->v;
{struct Cyc_List_List*reqs=(fd->i).requires_relns;for(0;reqs != 0;reqs=reqs->tl){
struct Cyc_Relations_Reln*req=(struct Cyc_Relations_Reln*)reqs->hd;
relns=({struct _RegionHandle*_tmp676=Cyc_Core_heap_region;union Cyc_Relations_RelnOp _tmp675=Cyc_NewControlFlow_translate_rop(vardecls,req->rop1);enum Cyc_Relations_Relation _tmp674=req->relation;union Cyc_Relations_RelnOp _tmp673=
Cyc_NewControlFlow_translate_rop(vardecls,req->rop2);
# 2968
Cyc_Relations_add_relation(_tmp676,_tmp675,_tmp674,_tmp673,relns);});}}{
# 2974
struct Cyc_List_List*atts;
{void*_stmttmpA1=Cyc_Tcutil_compress((void*)_check_null(fd->cached_type));void*_tmp407=_stmttmpA1;struct Cyc_List_List*_tmp408;if(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp407)->tag == 5U){_LL4: _tmp408=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp407)->f1).attributes;_LL5: {struct Cyc_List_List*as=_tmp408;
atts=as;goto _LL3;}}else{_LL6: _LL7:
({void*_tmp409=0U;({int(*_tmp678)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp40B)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp40B;});struct _fat_ptr _tmp677=({const char*_tmp40A="check_fun: non-function type cached with fndecl_t";_tag_fat(_tmp40A,sizeof(char),50U);});_tmp678(_tmp677,_tag_fat(_tmp409,sizeof(void*),0U));});});}_LL3:;}{
# 2979
struct Cyc_List_List*noconsume_roots=Cyc_NewControlFlow_get_noconsume_params((struct Cyc_List_List*)((struct Cyc_Core_Opt*)_check_null(fd->param_vardecls))->v,atts);
# 2982
for(0;atts != 0;atts=atts->tl){
void*_stmttmpA2=(void*)atts->hd;void*_tmp40C=_stmttmpA2;int _tmp40D;int _tmp40E;switch(*((int*)_tmp40C)){case 21U: _LL9: _tmp40E=((struct Cyc_Absyn_Noliveunique_att_Absyn_Attribute_struct*)_tmp40C)->f1;_LLA: {int i=_tmp40E;
# 2985
struct Cyc_Absyn_Exp*bogus_exp=Cyc_Absyn_signed_int_exp(- 1,0U);
struct Cyc_Absyn_Vardecl*vd=({struct Cyc_Absyn_Vardecl*(*_tmp67A)(struct Cyc_List_List*x,int n)=({struct Cyc_Absyn_Vardecl*(*_tmp413)(struct Cyc_List_List*x,int n)=(struct Cyc_Absyn_Vardecl*(*)(struct Cyc_List_List*x,int n))Cyc_List_nth;_tmp413;});struct Cyc_List_List*_tmp679=(struct Cyc_List_List*)((struct Cyc_Core_Opt*)_check_null(fd->param_vardecls))->v;_tmp67A(_tmp679,i - 1);});
void*t=Cyc_Tcutil_compress(vd->type);
void*elttype=Cyc_Tcutil_pointer_elt_type(t);
void*rval=({
struct Cyc_CfFlowInfo_FlowEnv*_tmp67D=fenv;void*_tmp67C=elttype;struct Cyc_Absyn_Exp*_tmp67B=bogus_exp;Cyc_CfFlowInfo_make_unique_consumed(_tmp67D,_tmp67C,_tmp67B,- 1,
Cyc_CfFlowInfo_typ_to_absrval(fenv,elttype,0,fenv->unknown_all));});
struct Cyc_CfFlowInfo_InitParam_CfFlowInfo_Root_struct*r=({struct Cyc_CfFlowInfo_InitParam_CfFlowInfo_Root_struct*_tmp412=_cycalloc(sizeof(*_tmp412));_tmp412->tag=2U,_tmp412->f1=i,_tmp412->f2=t;_tmp412;});
struct Cyc_CfFlowInfo_Place*rp=({struct Cyc_CfFlowInfo_Place*_tmp411=_cycalloc(sizeof(*_tmp411));_tmp411->root=(void*)r,_tmp411->path=0;_tmp411;});
d=Cyc_Dict_insert(d,(void*)r,rval);
d=({struct Cyc_Dict_Dict _tmp67F=d;void*_tmp67E=(void*)({struct Cyc_CfFlowInfo_VarRoot_CfFlowInfo_Root_struct*_tmp40F=_cycalloc(sizeof(*_tmp40F));_tmp40F->tag=0U,_tmp40F->f1=vd;_tmp40F;});Cyc_Dict_insert(_tmp67F,_tmp67E,(void*)({struct Cyc_CfFlowInfo_AddressOf_CfFlowInfo_AbsRVal_struct*_tmp410=_cycalloc(sizeof(*_tmp410));_tmp410->tag=4U,_tmp410->f1=rp;_tmp410;}));});
goto _LL8;}case 20U: _LLB: _tmp40D=((struct Cyc_Absyn_Initializes_att_Absyn_Attribute_struct*)_tmp40C)->f1;_LLC: {int i=_tmp40D;
# 2998
struct Cyc_Absyn_Vardecl*vd=({struct Cyc_Absyn_Vardecl*(*_tmp681)(struct Cyc_List_List*x,int n)=({struct Cyc_Absyn_Vardecl*(*_tmp419)(struct Cyc_List_List*x,int n)=(struct Cyc_Absyn_Vardecl*(*)(struct Cyc_List_List*x,int n))Cyc_List_nth;_tmp419;});struct Cyc_List_List*_tmp680=(struct Cyc_List_List*)((struct Cyc_Core_Opt*)_check_null(fd->param_vardecls))->v;_tmp681(_tmp680,i - 1);});
void*elttype=Cyc_Tcutil_pointer_elt_type(vd->type);
struct Cyc_CfFlowInfo_InitParam_CfFlowInfo_Root_struct*r=({struct Cyc_CfFlowInfo_InitParam_CfFlowInfo_Root_struct*_tmp418=_cycalloc(sizeof(*_tmp418));_tmp418->tag=2U,_tmp418->f1=i,_tmp418->f2=elttype;_tmp418;});
struct Cyc_CfFlowInfo_Place*rp=({struct Cyc_CfFlowInfo_Place*_tmp417=_cycalloc(sizeof(*_tmp417));_tmp417->root=(void*)r,_tmp417->path=0;_tmp417;});
d=({struct Cyc_Dict_Dict _tmp683=d;void*_tmp682=(void*)r;Cyc_Dict_insert(_tmp683,_tmp682,Cyc_CfFlowInfo_typ_to_absrval(fenv,elttype,0,fenv->esc_none));});
d=({struct Cyc_Dict_Dict _tmp685=d;void*_tmp684=(void*)({struct Cyc_CfFlowInfo_VarRoot_CfFlowInfo_Root_struct*_tmp414=_cycalloc(sizeof(*_tmp414));_tmp414->tag=0U,_tmp414->f1=vd;_tmp414;});Cyc_Dict_insert(_tmp685,_tmp684,(void*)({struct Cyc_CfFlowInfo_AddressOf_CfFlowInfo_AbsRVal_struct*_tmp415=_cycalloc(sizeof(*_tmp415));_tmp415->tag=4U,_tmp415->f1=rp;_tmp415;}));});
param_roots=({struct Cyc_List_List*_tmp416=_cycalloc(sizeof(*_tmp416));_tmp416->hd=rp,_tmp416->tl=param_roots;_tmp416;});
goto _LL8;}default: _LLD: _LLE:
 goto _LL8;}_LL8:;}
# 3009
inflow=Cyc_CfFlowInfo_ReachableFL(d,relns);{
# 3011
int noreturn=Cyc_Tcutil_is_noreturn_fn_type(Cyc_Tcutil_fndecl2type(fd));
struct Cyc_Hashtable_Table*flow_table=({
struct Cyc_Hashtable_Table*(*_tmp687)(int sz,int(*cmp)(struct Cyc_Absyn_Stmt*,struct Cyc_Absyn_Stmt*),int(*hash)(struct Cyc_Absyn_Stmt*))=({struct Cyc_Hashtable_Table*(*_tmp424)(int sz,int(*cmp)(struct Cyc_Absyn_Stmt*,struct Cyc_Absyn_Stmt*),int(*hash)(struct Cyc_Absyn_Stmt*))=(struct Cyc_Hashtable_Table*(*)(int sz,int(*cmp)(struct Cyc_Absyn_Stmt*,struct Cyc_Absyn_Stmt*),int(*hash)(struct Cyc_Absyn_Stmt*)))Cyc_Hashtable_create;_tmp424;});int(*_tmp686)(struct Cyc_Absyn_Stmt*,struct Cyc_Absyn_Stmt*)=({int(*_tmp425)(struct Cyc_Absyn_Stmt*,struct Cyc_Absyn_Stmt*)=(int(*)(struct Cyc_Absyn_Stmt*,struct Cyc_Absyn_Stmt*))Cyc_Core_ptrcmp;_tmp425;});_tmp687(33,_tmp686,({int(*_tmp426)(struct Cyc_Absyn_Stmt*s)=(int(*)(struct Cyc_Absyn_Stmt*s))Cyc_NewControlFlow_hash_ptr;_tmp426;}));});
struct Cyc_NewControlFlow_AnalEnv*env=({struct Cyc_NewControlFlow_AnalEnv*_tmp423=_cycalloc(sizeof(*_tmp423));
_tmp423->all_tables=tables,({
struct Cyc_Hashtable_Table*_tmp68D=({struct Cyc_Hashtable_Table*(*_tmp68C)(struct Cyc_Hashtable_Table*t,struct Cyc_Absyn_Fndecl*key)=({struct Cyc_Hashtable_Table*(*_tmp421)(struct Cyc_Hashtable_Table*t,struct Cyc_Absyn_Fndecl*key)=(struct Cyc_Hashtable_Table*(*)(struct Cyc_Hashtable_Table*t,struct Cyc_Absyn_Fndecl*key))Cyc_Hashtable_lookup;_tmp421;});struct Cyc_Hashtable_Table*_tmp68B=tables->succ_tables;_tmp68C(_tmp68B,fd);});_tmp423->succ_table=_tmp68D;}),({
struct Cyc_Hashtable_Table*_tmp68A=({struct Cyc_Hashtable_Table*(*_tmp689)(struct Cyc_Hashtable_Table*t,struct Cyc_Absyn_Fndecl*key)=({struct Cyc_Hashtable_Table*(*_tmp422)(struct Cyc_Hashtable_Table*t,struct Cyc_Absyn_Fndecl*key)=(struct Cyc_Hashtable_Table*(*)(struct Cyc_Hashtable_Table*t,struct Cyc_Absyn_Fndecl*key))Cyc_Hashtable_lookup;_tmp422;});struct Cyc_Hashtable_Table*_tmp688=tables->pat_pop_tables;_tmp689(_tmp688,fd);});_tmp423->pat_pop_table=_tmp68A;}),_tmp423->fenv=fenv,_tmp423->iterate_again=1,_tmp423->iteration_num=0,_tmp423->in_try=0,_tmp423->tryflow=inflow,_tmp423->noreturn=noreturn,_tmp423->return_type=(fd->i).ret_type,_tmp423->unique_pat_vars=0,_tmp423->param_roots=param_roots,_tmp423->noconsume_params=noconsume_roots,_tmp423->flow_table=flow_table,_tmp423->return_relns=(fd->i).ensures_relns;_tmp423;});
# 3021
union Cyc_CfFlowInfo_FlowInfo outflow=inflow;
while(env->iterate_again && !Cyc_CfFlowInfo_anal_error){
++ env->iteration_num;
# 3027
env->iterate_again=0;
outflow=Cyc_NewControlFlow_anal_stmt(env,inflow,fd->body,0);}{
# 3030
union Cyc_CfFlowInfo_FlowInfo _tmp41A=outflow;if((_tmp41A.BottomFL).tag == 1){_LL10: _LL11:
 goto _LLF;}else{_LL12: _LL13:
# 3033
 Cyc_NewControlFlow_check_init_params(loc,env,outflow);
Cyc_NewControlFlow_unconsume_params(env,env->noconsume_params,0,0,outflow,loc);
# 3037
if(noreturn)
({void*_tmp41B=0U;({unsigned _tmp68F=loc;struct _fat_ptr _tmp68E=({const char*_tmp41C="`noreturn' function might (implicitly) return";_tag_fat(_tmp41C,sizeof(char),46U);});Cyc_CfFlowInfo_aerr(_tmp68F,_tmp68E,_tag_fat(_tmp41B,sizeof(void*),0U));});});else{
if(!Cyc_Tcutil_is_void_type((fd->i).ret_type)){
if(Cyc_Tcutil_is_any_float_type((fd->i).ret_type)||
 Cyc_Tcutil_is_any_int_type((fd->i).ret_type))
({void*_tmp41D=0U;({unsigned _tmp691=loc;struct _fat_ptr _tmp690=({const char*_tmp41E="function may not return a value";_tag_fat(_tmp41E,sizeof(char),32U);});Cyc_Warn_warn(_tmp691,_tmp690,_tag_fat(_tmp41D,sizeof(void*),0U));});});else{
# 3044
({void*_tmp41F=0U;({unsigned _tmp693=loc;struct _fat_ptr _tmp692=({const char*_tmp420="function may not return a value";_tag_fat(_tmp420,sizeof(char),32U);});Cyc_CfFlowInfo_aerr(_tmp693,_tmp692,_tag_fat(_tmp41F,sizeof(void*),0U));});});}goto _LLF;}}
# 3037
goto _LLF;}_LLF:;}}}}}}}
# 2848
void Cyc_NewControlFlow_cf_check(struct Cyc_JumpAnalysis_Jump_Anal_Result*tables,struct Cyc_List_List*ds){
# 3051
for(0;ds != 0;ds=ds->tl){
Cyc_CfFlowInfo_anal_error=0;{
void*_stmttmpA3=((struct Cyc_Absyn_Decl*)ds->hd)->r;void*_tmp428=_stmttmpA3;struct Cyc_List_List*_tmp429;struct Cyc_List_List*_tmp42A;struct Cyc_List_List*_tmp42B;struct Cyc_Absyn_Fndecl*_tmp42C;switch(*((int*)_tmp428)){case 1U: _LL1: _tmp42C=((struct Cyc_Absyn_Fn_d_Absyn_Raw_decl_struct*)_tmp428)->f1;_LL2: {struct Cyc_Absyn_Fndecl*fd=_tmp42C;
Cyc_NewControlFlow_check_fun(tables,fd);goto _LL0;}case 11U: _LL3: _tmp42B=((struct Cyc_Absyn_ExternC_d_Absyn_Raw_decl_struct*)_tmp428)->f1;_LL4: {struct Cyc_List_List*ds2=_tmp42B;
# 3056
_tmp42A=ds2;goto _LL6;}case 10U: _LL5: _tmp42A=((struct Cyc_Absyn_Using_d_Absyn_Raw_decl_struct*)_tmp428)->f2;_LL6: {struct Cyc_List_List*ds2=_tmp42A;
_tmp429=ds2;goto _LL8;}case 9U: _LL7: _tmp429=((struct Cyc_Absyn_Namespace_d_Absyn_Raw_decl_struct*)_tmp428)->f2;_LL8: {struct Cyc_List_List*ds2=_tmp429;
Cyc_NewControlFlow_cf_check(tables,ds2);goto _LL0;}case 12U: _LL9: _LLA:
 goto _LL0;default: _LLB: _LLC:
 goto _LL0;}_LL0:;}}}
