#include <setjmp.h>
/* This is a C header file to be used by the output of the Cyclone to
   C translator.  The corresponding definitions are in file lib/runtime_*.c */
#ifndef _CYC_INCLUDE_H_
#define _CYC_INCLUDE_H_

/* Need one of these per thread (see runtime_stack.c). The runtime maintains 
   a stack that contains either _handler_cons structs or _RegionHandle structs.
   The tag is 0 for a handler_cons and 1 for a region handle.  */
struct _RuntimeStack {
  int tag; 
  struct _RuntimeStack *next;
  void (*cleanup)(struct _RuntimeStack *frame);
};

#ifndef offsetof
/* should be size_t, but int is fine. */
#define offsetof(t,n) ((int)(&(((t *)0)->n)))
#endif

/* Fat pointers */
struct _fat_ptr {
  unsigned char *curr; 
  unsigned char *base; 
  unsigned char *last_plus_one; 
};  

/* Discriminated Unions */
struct _xtunion_struct { char *tag; };

/* Regions */
struct _RegionPage
#ifdef CYC_REGION_PROFILE
{ unsigned total_bytes;
  unsigned free_bytes;
  /* MWH: wish we didn't have to include the stuff below ... */
  struct _RegionPage *next;
  char data[1];
}
#endif
; // abstract -- defined in runtime_memory.c
struct _RegionHandle {
  struct _RuntimeStack s;
  struct _RegionPage *curr;
  char               *offset;
  char               *last_plus_one;
  struct _DynRegionHandle *sub_regions;
#ifdef CYC_REGION_PROFILE
  const char         *name;
#else
  unsigned used_bytes;
  unsigned wasted_bytes;
#endif
};
struct _DynRegionFrame {
  struct _RuntimeStack s;
  struct _DynRegionHandle *x;
};
// A dynamic region is just a region handle.  The wrapper struct is for type
// abstraction.
struct Cyc_Core_DynamicRegion {
  struct _RegionHandle h;
};

struct _RegionHandle _new_region(const char*);
void* _region_malloc(struct _RegionHandle*, unsigned);
void* _region_calloc(struct _RegionHandle*, unsigned t, unsigned n);
void   _free_region(struct _RegionHandle*);
struct _RegionHandle*_open_dynregion(struct _DynRegionFrame*,struct _DynRegionHandle*);
void   _pop_dynregion();

/* Exceptions */
struct _handler_cons {
  struct _RuntimeStack s;
  jmp_buf handler;
};
void _push_handler(struct _handler_cons *);
void _push_region(struct _RegionHandle *);
void _npop_handler(int);
void _pop_handler();
void _pop_region();

#ifndef _throw
void* _throw_null_fn(const char*,unsigned);
void* _throw_arraybounds_fn(const char*,unsigned);
void* _throw_badalloc_fn(const char*,unsigned);
void* _throw_match_fn(const char*,unsigned);
void* _throw_fn(void*,const char*,unsigned);
void* _rethrow(void*);
#define _throw_null() (_throw_null_fn(__FILE__,__LINE__))
#define _throw_arraybounds() (_throw_arraybounds_fn(__FILE__,__LINE__))
#define _throw_badalloc() (_throw_badalloc_fn(__FILE__,__LINE__))
#define _throw_match() (_throw_match_fn(__FILE__,__LINE__))
#define _throw(e) (_throw_fn((e),__FILE__,__LINE__))
#endif

struct _xtunion_struct* Cyc_Core_get_exn_thrown();
/* Built-in Exceptions */
struct Cyc_Null_Exception_exn_struct { char *tag; };
struct Cyc_Array_bounds_exn_struct { char *tag; };
struct Cyc_Match_Exception_exn_struct { char *tag; };
struct Cyc_Bad_alloc_exn_struct { char *tag; };
extern char Cyc_Null_Exception[];
extern char Cyc_Array_bounds[];
extern char Cyc_Match_Exception[];
extern char Cyc_Bad_alloc[];

/* Built-in Run-time Checks and company */
#ifdef CYC_ANSI_OUTPUT
#define _INLINE  
#else
#define _INLINE inline
#endif

#ifdef NO_CYC_NULL_CHECKS
#define _check_null(ptr) (ptr)
#else
#define _check_null(ptr) \
  ({ void*_cks_null = (void*)(ptr); \
     if (!_cks_null) _throw_null(); \
     _cks_null; })
#endif

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index)\
   (((char*)ptr) + (elt_sz)*(index))
#ifdef NO_CYC_NULL_CHECKS
#define _check_known_subscript_null _check_known_subscript_notnull
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr);\
  int _index = (index);\
  if (!_cks_ptr) _throw_null(); \
  _cks_ptr + (elt_sz)*_index; })
#endif
#define _zero_arr_plus_fn(orig_x,orig_sz,orig_i,f,l) ((orig_x)+(orig_i))
#define _zero_arr_plus_char_fn _zero_arr_plus_fn
#define _zero_arr_plus_short_fn _zero_arr_plus_fn
#define _zero_arr_plus_int_fn _zero_arr_plus_fn
#define _zero_arr_plus_float_fn _zero_arr_plus_fn
#define _zero_arr_plus_double_fn _zero_arr_plus_fn
#define _zero_arr_plus_longdouble_fn _zero_arr_plus_fn
#define _zero_arr_plus_voidstar_fn _zero_arr_plus_fn
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (!_cks_ptr) _throw_null(); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })

/* _zero_arr_plus_*_fn(x,sz,i,filename,lineno) adds i to zero-terminated ptr
   x that has at least sz elements */
char* _zero_arr_plus_char_fn(char*,unsigned,int,const char*,unsigned);
short* _zero_arr_plus_short_fn(short*,unsigned,int,const char*,unsigned);
int* _zero_arr_plus_int_fn(int*,unsigned,int,const char*,unsigned);
float* _zero_arr_plus_float_fn(float*,unsigned,int,const char*,unsigned);
double* _zero_arr_plus_double_fn(double*,unsigned,int,const char*,unsigned);
long double* _zero_arr_plus_longdouble_fn(long double*,unsigned,int,const char*, unsigned);
void** _zero_arr_plus_voidstar_fn(void**,unsigned,int,const char*,unsigned);
#endif

/* _get_zero_arr_size_*(x,sz) returns the number of elements in a
   zero-terminated array that is NULL or has at least sz elements */
int _get_zero_arr_size_char(const char*,unsigned);
int _get_zero_arr_size_short(const short*,unsigned);
int _get_zero_arr_size_int(const int*,unsigned);
int _get_zero_arr_size_float(const float*,unsigned);
int _get_zero_arr_size_double(const double*,unsigned);
int _get_zero_arr_size_longdouble(const long double*,unsigned);
int _get_zero_arr_size_voidstar(const void**,unsigned);

/* _zero_arr_inplace_plus_*_fn(x,i,filename,lineno) sets
   zero-terminated pointer *x to *x + i */
char* _zero_arr_inplace_plus_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_short_fn(short**,int,const char*,unsigned);
int* _zero_arr_inplace_plus_int(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_longdouble_fn(long double**,int,const char*,unsigned);
void** _zero_arr_inplace_plus_voidstar_fn(void***,int,const char*,unsigned);
/* like the previous functions, but does post-addition (as in e++) */
char* _zero_arr_inplace_plus_post_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_post_short_fn(short**x,int,const char*,unsigned);
int* _zero_arr_inplace_plus_post_int_fn(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_post_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_post_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_post_longdouble_fn(long double**,int,const char *,unsigned);
void** _zero_arr_inplace_plus_post_voidstar_fn(void***,int,const char*,unsigned);
#define _zero_arr_plus_char(x,s,i) \
  (_zero_arr_plus_char_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_short(x,s,i) \
  (_zero_arr_plus_short_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_int(x,s,i) \
  (_zero_arr_plus_int_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_float(x,s,i) \
  (_zero_arr_plus_float_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_double(x,s,i) \
  (_zero_arr_plus_double_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_longdouble(x,s,i) \
  (_zero_arr_plus_longdouble_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_voidstar(x,s,i) \
  (_zero_arr_plus_voidstar_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_inplace_plus_char(x,i) \
  _zero_arr_inplace_plus_char_fn((char **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_short(x,i) \
  _zero_arr_inplace_plus_short_fn((short **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_int(x,i) \
  _zero_arr_inplace_plus_int_fn((int **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_float(x,i) \
  _zero_arr_inplace_plus_float_fn((float **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_double(x,i) \
  _zero_arr_inplace_plus_double_fn((double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_longdouble(x,i) \
  _zero_arr_inplace_plus_longdouble_fn((long double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_voidstar(x,i) \
  _zero_arr_inplace_plus_voidstar_fn((void ***)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_char(x,i) \
  _zero_arr_inplace_plus_post_char_fn((char **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_short(x,i) \
  _zero_arr_inplace_plus_post_short_fn((short **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_int(x,i) \
  _zero_arr_inplace_plus_post_int_fn((int **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_float(x,i) \
  _zero_arr_inplace_plus_post_float_fn((float **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_double(x,i) \
  _zero_arr_inplace_plus_post_double_fn((double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_longdouble(x,i) \
  _zero_arr_inplace_plus_post_longdouble_fn((long double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_voidstar(x,i) \
  _zero_arr_inplace_plus_post_voidstar_fn((void***)(x),(i),__FILE__,__LINE__)

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_fat_subscript(arr,elt_sz,index) ((arr).curr + (elt_sz) * (index))
#define _untag_fat_ptr(arr,elt_sz,num_elts) ((arr).curr)
#else
#define _check_fat_subscript(arr,elt_sz,index) ({ \
  struct _fat_ptr _cus_arr = (arr); \
  unsigned char *_cus_ans = _cus_arr.curr + (elt_sz) * (index); \
  /* JGM: not needed! if (!_cus_arr.base) _throw_null();*/ \
  if (_cus_ans < _cus_arr.base || _cus_ans >= _cus_arr.last_plus_one) \
    _throw_arraybounds(); \
  _cus_ans; })
#define _untag_fat_ptr(arr,elt_sz,num_elts) ({ \
  struct _fat_ptr _arr = (arr); \
  unsigned char *_curr = _arr.curr; \
  if ((_curr < _arr.base || _curr + (elt_sz) * (num_elts) > _arr.last_plus_one) &&\
      _curr != (unsigned char *)0) \
    _throw_arraybounds(); \
  _curr; })
#endif

#define _tag_fat(tcurr,elt_sz,num_elts) ({ \
  struct _fat_ptr _ans; \
  unsigned _num_elts = (num_elts);\
  _ans.base = _ans.curr = (void*)(tcurr); \
  /* JGM: if we're tagging NULL, ignore num_elts */ \
  _ans.last_plus_one = _ans.base ? (_ans.base + (elt_sz) * _num_elts) : 0; \
  _ans; })

#define _get_fat_size(arr,elt_sz) \
  ({struct _fat_ptr _arr = (arr); \
    unsigned char *_arr_curr=_arr.curr; \
    unsigned char *_arr_last=_arr.last_plus_one; \
    (_arr_curr < _arr.base || _arr_curr >= _arr_last) ? 0 : \
    ((_arr_last - _arr_curr) / (elt_sz));})

#define _fat_ptr_plus(arr,elt_sz,change) ({ \
  struct _fat_ptr _ans = (arr); \
  int _change = (change);\
  _ans.curr += (elt_sz) * _change;\
  _ans; })
#define _fat_ptr_inplace_plus(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  _arr_ptr->curr += (elt_sz) * (change);\
  *_arr_ptr; })
#define _fat_ptr_inplace_plus_post(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  struct _fat_ptr _ans = *_arr_ptr; \
  _arr_ptr->curr += (elt_sz) * (change);\
  _ans; })

//Not a macro since initialization order matters. Defined in runtime_zeroterm.c.
struct _fat_ptr _fat_ptr_decrease_size(struct _fat_ptr,unsigned sz,unsigned numelts);

/* Allocation */
void* GC_malloc(int);
void* GC_malloc_atomic(int);
void* GC_calloc(unsigned,unsigned);
void* GC_calloc_atomic(unsigned,unsigned);
// bound the allocation size to be < MAX_ALLOC_SIZE. See macros below for usage.
#define MAX_MALLOC_SIZE (1 << 28)
void* _bounded_GC_malloc(int,const char*,int);
void* _bounded_GC_malloc_atomic(int,const char*,int);
void* _bounded_GC_calloc(unsigned,unsigned,const char*,int);
void* _bounded_GC_calloc_atomic(unsigned,unsigned,const char*,int);
/* these macros are overridden below ifdef CYC_REGION_PROFILE */
#ifndef CYC_REGION_PROFILE
#define _cycalloc(n) _bounded_GC_malloc(n,__FILE__,__LINE__)
#define _cycalloc_atomic(n) _bounded_GC_malloc_atomic(n,__FILE__,__LINE__)
#define _cyccalloc(n,s) _bounded_GC_calloc(n,s,__FILE__,__LINE__)
#define _cyccalloc_atomic(n,s) _bounded_GC_calloc_atomic(n,s,__FILE__,__LINE__)
#endif

static _INLINE unsigned int _check_times(unsigned x, unsigned y) {
  unsigned long long whole_ans = 
    ((unsigned long long) x)*((unsigned long long)y);
  unsigned word_ans = (unsigned)whole_ans;
  if(word_ans < whole_ans || word_ans > MAX_MALLOC_SIZE)
    _throw_badalloc();
  return word_ans;
}

#define _CYC_MAX_REGION_CONST 2
#define _CYC_MIN_ALIGNMENT (sizeof(double))

#ifdef CYC_REGION_PROFILE
extern int rgn_total_bytes;
#endif

static _INLINE void *_fast_region_malloc(struct _RegionHandle *r, unsigned orig_s) {  
  if (r > (struct _RegionHandle *)_CYC_MAX_REGION_CONST && r->curr != 0) { 
#ifdef CYC_NOALIGN
    unsigned s =  orig_s;
#else
    unsigned s =  (orig_s + _CYC_MIN_ALIGNMENT - 1) & (~(_CYC_MIN_ALIGNMENT -1)); 
#endif
    char *result; 
    result = r->offset; 
    if (s <= (r->last_plus_one - result)) {
      r->offset = result + s; 
#ifdef CYC_REGION_PROFILE
    r->curr->free_bytes = r->curr->free_bytes - s;
    rgn_total_bytes += s;
#endif
      return result;
    }
  } 
  return _region_malloc(r,orig_s); 
}

#ifdef CYC_REGION_PROFILE
/* see macros below for usage. defined in runtime_memory.c */
void* _profile_GC_malloc(int,const char*,const char*,int);
void* _profile_GC_malloc_atomic(int,const char*,const char*,int);
void* _profile_GC_calloc(unsigned,unsigned,const char*,const char*,int);
void* _profile_GC_calloc_atomic(unsigned,unsigned,const char*,const char*,int);
void* _profile_region_malloc(struct _RegionHandle*,unsigned,const char*,const char*,int);
void* _profile_region_calloc(struct _RegionHandle*,unsigned,unsigned,const char *,const char*,int);
struct _RegionHandle _profile_new_region(const char*,const char*,const char*,int);
void _profile_free_region(struct _RegionHandle*,const char*,const char*,int);
#ifndef RUNTIME_CYC
#define _new_region(n) _profile_new_region(n,__FILE__,__FUNCTION__,__LINE__)
#define _free_region(r) _profile_free_region(r,__FILE__,__FUNCTION__,__LINE__)
#define _region_malloc(rh,n) _profile_region_malloc(rh,n,__FILE__,__FUNCTION__,__LINE__)
#define _region_calloc(rh,n,t) _profile_region_calloc(rh,n,t,__FILE__,__FUNCTION__,__LINE__)
#  endif
#define _cycalloc(n) _profile_GC_malloc(n,__FILE__,__FUNCTION__,__LINE__)
#define _cycalloc_atomic(n) _profile_GC_malloc_atomic(n,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc(n,s) _profile_GC_calloc(n,s,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc_atomic(n,s) _profile_GC_calloc_atomic(n,s,__FILE__,__FUNCTION__,__LINE__)
#endif
#endif

# 6 "parse_tab.cyc"
 void(*Cyc_beforedie)()=0;struct Cyc_Core_Opt{void*v;};extern char Cyc_Core_Invalid_argument[17U];struct Cyc_Core_Invalid_argument_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Failure[8U];struct Cyc_Core_Failure_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Impossible[11U];struct Cyc_Core_Impossible_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Not_found[10U];struct Cyc_Core_Not_found_exn_struct{char*tag;};extern char Cyc_Core_Unreachable[12U];struct Cyc_Core_Unreachable_exn_struct{char*tag;struct _fat_ptr f1;};
# 168 "core.h"
extern struct _RegionHandle*Cyc_Core_unique_region;struct Cyc_Core_DynamicRegion;struct Cyc_Core_NewDynamicRegion{struct Cyc_Core_DynamicRegion*key;};struct Cyc_Core_ThinRes{void*arr;unsigned nelts;};struct Cyc___cycFILE;
# 53 "cycboot.h"
extern struct Cyc___cycFILE*Cyc_stderr;struct Cyc_String_pa_PrintArg_struct{int tag;struct _fat_ptr f1;};struct Cyc_Int_pa_PrintArg_struct{int tag;unsigned long f1;};struct Cyc_Double_pa_PrintArg_struct{int tag;double f1;};struct Cyc_LongDouble_pa_PrintArg_struct{int tag;long double f1;};struct Cyc_ShortPtr_pa_PrintArg_struct{int tag;short*f1;};struct Cyc_IntPtr_pa_PrintArg_struct{int tag;unsigned long*f1;};
# 73
struct _fat_ptr Cyc_aprintf(struct _fat_ptr,struct _fat_ptr);
# 100
int Cyc_fprintf(struct Cyc___cycFILE*,struct _fat_ptr,struct _fat_ptr);struct Cyc_ShortPtr_sa_ScanfArg_struct{int tag;short*f1;};struct Cyc_UShortPtr_sa_ScanfArg_struct{int tag;unsigned short*f1;};struct Cyc_IntPtr_sa_ScanfArg_struct{int tag;int*f1;};struct Cyc_UIntPtr_sa_ScanfArg_struct{int tag;unsigned*f1;};struct Cyc_StringPtr_sa_ScanfArg_struct{int tag;struct _fat_ptr f1;};struct Cyc_DoublePtr_sa_ScanfArg_struct{int tag;double*f1;};struct Cyc_FloatPtr_sa_ScanfArg_struct{int tag;float*f1;};struct Cyc_CharPtr_sa_ScanfArg_struct{int tag;struct _fat_ptr f1;};extern char Cyc_FileCloseError[15U];struct Cyc_FileCloseError_exn_struct{char*tag;};extern char Cyc_FileOpenError[14U];struct Cyc_FileOpenError_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Lexing_Error[6U];struct Cyc_Lexing_Error_exn_struct{char*tag;struct _fat_ptr f1;};struct Cyc_Lexing_lexbuf{void(*refill_buff)(struct Cyc_Lexing_lexbuf*);void*refill_state;struct _fat_ptr lex_buffer;int lex_buffer_len;int lex_abs_pos;int lex_start_pos;int lex_curr_pos;int lex_last_pos;int lex_last_action;int lex_eof_reached;};struct Cyc_Lexing_function_lexbuf_state{int(*read_fun)(struct _fat_ptr,int,void*);void*read_fun_state;};struct Cyc_Lexing_lex_tables{struct _fat_ptr lex_base;struct _fat_ptr lex_backtrk;struct _fat_ptr lex_default;struct _fat_ptr lex_trans;struct _fat_ptr lex_check;};
# 78 "lexing.h"
struct Cyc_Lexing_lexbuf*Cyc_Lexing_from_file(struct Cyc___cycFILE*);struct Cyc_List_List{void*hd;struct Cyc_List_List*tl;};
# 54 "list.h"
struct Cyc_List_List*Cyc_List_list(struct _fat_ptr);
# 61
int Cyc_List_length(struct Cyc_List_List*x);
# 76
struct Cyc_List_List*Cyc_List_map(void*(*f)(void*),struct Cyc_List_List*x);
# 83
struct Cyc_List_List*Cyc_List_map_c(void*(*f)(void*,void*),void*env,struct Cyc_List_List*x);extern char Cyc_List_List_mismatch[14U];struct Cyc_List_List_mismatch_exn_struct{char*tag;};
# 135
void Cyc_List_iter_c(void(*f)(void*,void*),void*env,struct Cyc_List_List*x);
# 153
void*Cyc_List_fold_right(void*(*f)(void*,void*),struct Cyc_List_List*x,void*accum);
# 172
struct Cyc_List_List*Cyc_List_rev(struct Cyc_List_List*x);
# 178
struct Cyc_List_List*Cyc_List_imp_rev(struct Cyc_List_List*x);
# 184
struct Cyc_List_List*Cyc_List_append(struct Cyc_List_List*x,struct Cyc_List_List*y);
# 195
struct Cyc_List_List*Cyc_List_imp_append(struct Cyc_List_List*x,struct Cyc_List_List*y);extern char Cyc_List_Nth[4U];struct Cyc_List_Nth_exn_struct{char*tag;};
# 276
struct Cyc_List_List*Cyc_List_rzip(struct _RegionHandle*r1,struct _RegionHandle*r2,struct Cyc_List_List*x,struct Cyc_List_List*y);
# 38 "string.h"
unsigned long Cyc_strlen(struct _fat_ptr s);
# 49 "string.h"
int Cyc_strcmp(struct _fat_ptr s1,struct _fat_ptr s2);
int Cyc_strptrcmp(struct _fat_ptr*s1,struct _fat_ptr*s2);
# 52
int Cyc_zstrcmp(struct _fat_ptr,struct _fat_ptr);
# 54
int Cyc_zstrptrcmp(struct _fat_ptr*,struct _fat_ptr*);
# 60
struct _fat_ptr Cyc_strcat(struct _fat_ptr dest,struct _fat_ptr src);
# 71
struct _fat_ptr Cyc_strcpy(struct _fat_ptr dest,struct _fat_ptr src);struct Cyc_Position_Error;struct Cyc_Relations_Reln;struct _union_Nmspace_Rel_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_Abs_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_C_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_Loc_n{int tag;int val;};union Cyc_Absyn_Nmspace{struct _union_Nmspace_Rel_n Rel_n;struct _union_Nmspace_Abs_n Abs_n;struct _union_Nmspace_C_n C_n;struct _union_Nmspace_Loc_n Loc_n;};
# 95 "absyn.h"
extern union Cyc_Absyn_Nmspace Cyc_Absyn_Loc_n;
union Cyc_Absyn_Nmspace Cyc_Absyn_Rel_n(struct Cyc_List_List*);struct _tuple0{union Cyc_Absyn_Nmspace f1;struct _fat_ptr*f2;};
# 161
enum Cyc_Absyn_Scope{Cyc_Absyn_Static =0U,Cyc_Absyn_Abstract =1U,Cyc_Absyn_Public =2U,Cyc_Absyn_Extern =3U,Cyc_Absyn_ExternC =4U,Cyc_Absyn_Register =5U};struct Cyc_Absyn_Tqual{int print_const: 1;int q_volatile: 1;int q_restrict: 1;int real_const: 1;unsigned loc;};
# 182
enum Cyc_Absyn_Size_of{Cyc_Absyn_Char_sz =0U,Cyc_Absyn_Short_sz =1U,Cyc_Absyn_Int_sz =2U,Cyc_Absyn_Long_sz =3U,Cyc_Absyn_LongLong_sz =4U};
enum Cyc_Absyn_Sign{Cyc_Absyn_Signed =0U,Cyc_Absyn_Unsigned =1U,Cyc_Absyn_None =2U};
enum Cyc_Absyn_AggrKind{Cyc_Absyn_StructA =0U,Cyc_Absyn_UnionA =1U};
# 187
enum Cyc_Absyn_AliasQual{Cyc_Absyn_Aliasable =0U,Cyc_Absyn_Unique =1U,Cyc_Absyn_Top =2U};
# 192
enum Cyc_Absyn_KindQual{Cyc_Absyn_AnyKind =0U,Cyc_Absyn_MemKind =1U,Cyc_Absyn_BoxKind =2U,Cyc_Absyn_RgnKind =3U,Cyc_Absyn_EffKind =4U,Cyc_Absyn_IntKind =5U,Cyc_Absyn_BoolKind =6U,Cyc_Absyn_PtrBndKind =7U};struct Cyc_Absyn_Kind{enum Cyc_Absyn_KindQual kind;enum Cyc_Absyn_AliasQual aliasqual;};struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct{int tag;struct Cyc_Absyn_Kind*f1;};struct Cyc_Absyn_Unknown_kb_Absyn_KindBound_struct{int tag;struct Cyc_Core_Opt*f1;};struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct{int tag;struct Cyc_Core_Opt*f1;struct Cyc_Absyn_Kind*f2;};struct Cyc_Absyn_Tvar{struct _fat_ptr*name;int identity;void*kind;};struct Cyc_Absyn_PtrLoc{unsigned ptr_loc;unsigned rgn_loc;unsigned zt_loc;};struct Cyc_Absyn_PtrAtts{void*rgn;void*nullable;void*bounds;void*zero_term;struct Cyc_Absyn_PtrLoc*ptrloc;};struct Cyc_Absyn_PtrInfo{void*elt_type;struct Cyc_Absyn_Tqual elt_tq;struct Cyc_Absyn_PtrAtts ptr_atts;};struct Cyc_Absyn_VarargInfo{struct _fat_ptr*name;struct Cyc_Absyn_Tqual tq;void*type;int inject;};struct Cyc_Absyn_Star_Absyn_Cap_struct{int tag;};struct Cyc_Absyn_Bot_Absyn_Cap_struct{int tag;};struct Cyc_Absyn_Nat_Absyn_Cap_struct{int tag;struct Cyc_Absyn_Exp*f1;int f2;};struct Cyc_Absyn_RgnEffect{void*name;void*rgn;void*lock;void*unadopt;void*parent;};struct Cyc_Absyn_FnInfo{struct Cyc_List_List*tvars;void*effect;struct Cyc_Absyn_Tqual ret_tqual;void*ret_type;struct Cyc_List_List*args;int c_varargs;struct Cyc_Absyn_VarargInfo*cyc_varargs;struct Cyc_List_List*rgn_po;struct Cyc_List_List*attributes;struct Cyc_Absyn_Exp*requires_clause;struct Cyc_List_List*requires_relns;struct Cyc_Absyn_Exp*ensures_clause;struct Cyc_List_List*ensures_relns;struct Cyc_List_List*ieffect;struct Cyc_List_List*oeffect;};struct Cyc_Absyn_UnknownDatatypeInfo{struct _tuple0*name;int is_extensible;};struct _union_DatatypeInfo_UnknownDatatype{int tag;struct Cyc_Absyn_UnknownDatatypeInfo val;};struct _union_DatatypeInfo_KnownDatatype{int tag;struct Cyc_Absyn_Datatypedecl**val;};union Cyc_Absyn_DatatypeInfo{struct _union_DatatypeInfo_UnknownDatatype UnknownDatatype;struct _union_DatatypeInfo_KnownDatatype KnownDatatype;};
# 311
union Cyc_Absyn_DatatypeInfo Cyc_Absyn_UnknownDatatype(struct Cyc_Absyn_UnknownDatatypeInfo);struct Cyc_Absyn_UnknownDatatypeFieldInfo{struct _tuple0*datatype_name;struct _tuple0*field_name;int is_extensible;};struct _union_DatatypeFieldInfo_UnknownDatatypefield{int tag;struct Cyc_Absyn_UnknownDatatypeFieldInfo val;};struct _tuple1{struct Cyc_Absyn_Datatypedecl*f1;struct Cyc_Absyn_Datatypefield*f2;};struct _union_DatatypeFieldInfo_KnownDatatypefield{int tag;struct _tuple1 val;};union Cyc_Absyn_DatatypeFieldInfo{struct _union_DatatypeFieldInfo_UnknownDatatypefield UnknownDatatypefield;struct _union_DatatypeFieldInfo_KnownDatatypefield KnownDatatypefield;};
# 324
union Cyc_Absyn_DatatypeFieldInfo Cyc_Absyn_UnknownDatatypefield(struct Cyc_Absyn_UnknownDatatypeFieldInfo);struct _tuple2{enum Cyc_Absyn_AggrKind f1;struct _tuple0*f2;struct Cyc_Core_Opt*f3;};struct _union_AggrInfo_UnknownAggr{int tag;struct _tuple2 val;};struct _union_AggrInfo_KnownAggr{int tag;struct Cyc_Absyn_Aggrdecl**val;};union Cyc_Absyn_AggrInfo{struct _union_AggrInfo_UnknownAggr UnknownAggr;struct _union_AggrInfo_KnownAggr KnownAggr;};
# 331
union Cyc_Absyn_AggrInfo Cyc_Absyn_UnknownAggr(enum Cyc_Absyn_AggrKind,struct _tuple0*,struct Cyc_Core_Opt*);struct Cyc_Absyn_ArrayInfo{void*elt_type;struct Cyc_Absyn_Tqual tq;struct Cyc_Absyn_Exp*num_elts;void*zero_term;unsigned zt_loc;};struct Cyc_Absyn_Aggr_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Aggrdecl*f1;};struct Cyc_Absyn_Enum_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;};struct Cyc_Absyn_Datatype_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;};struct Cyc_Absyn_TypeDecl{void*r;unsigned loc;};struct Cyc_Absyn_VoidCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_IntCon_Absyn_TyCon_struct{int tag;enum Cyc_Absyn_Sign f1;enum Cyc_Absyn_Size_of f2;};struct Cyc_Absyn_FloatCon_Absyn_TyCon_struct{int tag;int f1;};struct Cyc_Absyn_RgnHandleCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_TagCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_HeapCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_UniqueCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_RefCntCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_AccessCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_JoinCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_RgnsCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_TrueCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_FalseCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_ThinCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_FatCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_EnumCon_Absyn_TyCon_struct{int tag;struct _tuple0*f1;struct Cyc_Absyn_Enumdecl*f2;};struct Cyc_Absyn_AnonEnumCon_Absyn_TyCon_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_BuiltinCon_Absyn_TyCon_struct{int tag;struct _fat_ptr f1;struct Cyc_Absyn_Kind*f2;};struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_DatatypeInfo f1;};struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_DatatypeFieldInfo f1;};struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_AggrInfo f1;};struct Cyc_Absyn_AppType_Absyn_Type_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Evar_Absyn_Type_struct{int tag;struct Cyc_Core_Opt*f1;void*f2;int f3;struct Cyc_Core_Opt*f4;};struct Cyc_Absyn_VarType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Tvar*f1;};struct Cyc_Absyn_PointerType_Absyn_Type_struct{int tag;struct Cyc_Absyn_PtrInfo f1;};struct Cyc_Absyn_ArrayType_Absyn_Type_struct{int tag;struct Cyc_Absyn_ArrayInfo f1;};struct Cyc_Absyn_FnType_Absyn_Type_struct{int tag;struct Cyc_Absyn_FnInfo f1;};struct Cyc_Absyn_TupleType_Absyn_Type_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct{int tag;enum Cyc_Absyn_AggrKind f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_TypedefType_Absyn_Type_struct{int tag;struct _tuple0*f1;struct Cyc_List_List*f2;struct Cyc_Absyn_Typedefdecl*f3;void*f4;};struct Cyc_Absyn_ValueofType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct{int tag;struct Cyc_Absyn_TypeDecl*f1;void**f2;};struct Cyc_Absyn_TypeofType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_NoTypes_Absyn_Funcparams_struct{int tag;struct Cyc_List_List*f1;unsigned f2;};struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct{int tag;struct Cyc_List_List*f1;int f2;struct Cyc_Absyn_VarargInfo*f3;void*f4;struct Cyc_List_List*f5;struct Cyc_Absyn_Exp*f6;struct Cyc_Absyn_Exp*f7;struct Cyc_List_List*f8;struct Cyc_List_List*f9;};
# 437 "absyn.h"
enum Cyc_Absyn_Format_Type{Cyc_Absyn_Printf_ft =0U,Cyc_Absyn_Scanf_ft =1U};struct Cyc_Absyn_Regparm_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Stdcall_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Cdecl_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Fastcall_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Noreturn_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Const_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Aligned_att_Absyn_Attribute_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Packed_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Section_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Nocommon_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Shared_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Unused_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Weak_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Dllimport_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Dllexport_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_instrument_function_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Constructor_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Destructor_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_check_memory_usage_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Format_att_Absyn_Attribute_struct{int tag;enum Cyc_Absyn_Format_Type f1;int f2;int f3;};struct Cyc_Absyn_Initializes_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Noliveunique_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Consume_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Pure_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Mode_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Alias_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Always_inline_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_throw_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Non_null_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Deprecated_att_Absyn_Attribute_struct{int tag;};
# 480
extern struct Cyc_Absyn_Const_att_Absyn_Attribute_struct Cyc_Absyn_Const_att_val;struct Cyc_Absyn_Carray_mod_Absyn_Type_modifier_struct{int tag;void*f1;unsigned f2;};struct Cyc_Absyn_ConstArray_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_Absyn_Exp*f1;void*f2;unsigned f3;};struct Cyc_Absyn_Pointer_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_Absyn_PtrAtts f1;struct Cyc_Absyn_Tqual f2;};struct Cyc_Absyn_Function_mod_Absyn_Type_modifier_struct{int tag;void*f1;};struct Cyc_Absyn_TypeParams_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_List_List*f1;unsigned f2;int f3;};struct Cyc_Absyn_Attributes_mod_Absyn_Type_modifier_struct{int tag;unsigned f1;struct Cyc_List_List*f2;};struct _union_Cnst_Null_c{int tag;int val;};struct _tuple3{enum Cyc_Absyn_Sign f1;char f2;};struct _union_Cnst_Char_c{int tag;struct _tuple3 val;};struct _union_Cnst_Wchar_c{int tag;struct _fat_ptr val;};struct _tuple4{enum Cyc_Absyn_Sign f1;short f2;};struct _union_Cnst_Short_c{int tag;struct _tuple4 val;};struct _tuple5{enum Cyc_Absyn_Sign f1;int f2;};struct _union_Cnst_Int_c{int tag;struct _tuple5 val;};struct _tuple6{enum Cyc_Absyn_Sign f1;long long f2;};struct _union_Cnst_LongLong_c{int tag;struct _tuple6 val;};struct _tuple7{struct _fat_ptr f1;int f2;};struct _union_Cnst_Float_c{int tag;struct _tuple7 val;};struct _union_Cnst_String_c{int tag;struct _fat_ptr val;};struct _union_Cnst_Wstring_c{int tag;struct _fat_ptr val;};union Cyc_Absyn_Cnst{struct _union_Cnst_Null_c Null_c;struct _union_Cnst_Char_c Char_c;struct _union_Cnst_Wchar_c Wchar_c;struct _union_Cnst_Short_c Short_c;struct _union_Cnst_Int_c Int_c;struct _union_Cnst_LongLong_c LongLong_c;struct _union_Cnst_Float_c Float_c;struct _union_Cnst_String_c String_c;struct _union_Cnst_Wstring_c Wstring_c;};
# 532
enum Cyc_Absyn_Primop{Cyc_Absyn_Plus =0U,Cyc_Absyn_Times =1U,Cyc_Absyn_Minus =2U,Cyc_Absyn_Div =3U,Cyc_Absyn_Mod =4U,Cyc_Absyn_Eq =5U,Cyc_Absyn_Neq =6U,Cyc_Absyn_Gt =7U,Cyc_Absyn_Lt =8U,Cyc_Absyn_Gte =9U,Cyc_Absyn_Lte =10U,Cyc_Absyn_Not =11U,Cyc_Absyn_Bitnot =12U,Cyc_Absyn_Bitand =13U,Cyc_Absyn_Bitor =14U,Cyc_Absyn_Bitxor =15U,Cyc_Absyn_Bitlshift =16U,Cyc_Absyn_Bitlrshift =17U,Cyc_Absyn_Numelts =18U};
# 539
enum Cyc_Absyn_Incrementor{Cyc_Absyn_PreInc =0U,Cyc_Absyn_PostInc =1U,Cyc_Absyn_PreDec =2U,Cyc_Absyn_PostDec =3U};struct Cyc_Absyn_VarargCallInfo{int num_varargs;struct Cyc_List_List*injectors;struct Cyc_Absyn_VarargInfo*vai;};struct Cyc_Absyn_StructField_Absyn_OffsetofField_struct{int tag;struct _fat_ptr*f1;};struct Cyc_Absyn_TupleIndex_Absyn_OffsetofField_struct{int tag;unsigned f1;};
# 557
enum Cyc_Absyn_Coercion{Cyc_Absyn_Unknown_coercion =0U,Cyc_Absyn_No_coercion =1U,Cyc_Absyn_Null_to_NonNull =2U,Cyc_Absyn_Other_coercion =3U};struct Cyc_Absyn_MallocInfo{int is_calloc;struct Cyc_Absyn_Exp*rgn;void**elt_type;struct Cyc_Absyn_Exp*num_elts;int fat_result;int inline_call;};struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct{int tag;union Cyc_Absyn_Cnst f1;};struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Pragma_e_Absyn_Raw_exp_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct{int tag;enum Cyc_Absyn_Primop f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_AssignOp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Core_Opt*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_Increment_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;enum Cyc_Absyn_Incrementor f2;};struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;struct Cyc_Absyn_VarargCallInfo*f3;int f4;};struct Cyc_Absyn_Throw_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;int f2;};struct Cyc_Absyn_NoInstantiate_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Instantiate_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_Absyn_Exp*f2;int f3;enum Cyc_Absyn_Coercion f4;};struct Cyc_Absyn_Address_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_New_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Sizeoftype_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Sizeofexp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Offsetof_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Deref_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;int f3;int f4;};struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;int f3;int f4;};struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Tuple_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;};struct _tuple8{struct _fat_ptr*f1;struct Cyc_Absyn_Tqual f2;void*f3;};struct Cyc_Absyn_CompoundLit_e_Absyn_Raw_exp_struct{int tag;struct _tuple8*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Array_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Exp*f2;struct Cyc_Absyn_Exp*f3;int f4;};struct Cyc_Absyn_ComprehensionNoinit_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;void*f2;int f3;};struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct{int tag;struct _tuple0*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;struct Cyc_Absyn_Aggrdecl*f4;};struct Cyc_Absyn_AnonStruct_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Datatype_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;struct Cyc_Absyn_Datatypedecl*f2;struct Cyc_Absyn_Datatypefield*f3;};struct Cyc_Absyn_Enum_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_AnonEnum_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_Spawn_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_MallocInfo f1;};struct Cyc_Absyn_Swap_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_UnresolvedMem_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Core_Opt*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_StmtExp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Stmt*f1;};struct Cyc_Absyn_Tagcheck_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;};struct Cyc_Absyn_Valueof_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Asm_e_Absyn_Raw_exp_struct{int tag;int f1;struct _fat_ptr f2;struct Cyc_List_List*f3;struct Cyc_List_List*f4;struct Cyc_List_List*f5;};struct Cyc_Absyn_Extension_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Exp{void*topt;void*r;unsigned loc;void*annot;};struct Cyc_Absyn_Skip_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Exp_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Seq_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Return_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_IfThenElse_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Stmt*f2;struct Cyc_Absyn_Stmt*f3;};struct _tuple9{struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_While_s_Absyn_Raw_stmt_struct{int tag;struct _tuple9 f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Break_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Continue_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Goto_s_Absyn_Raw_stmt_struct{int tag;struct _fat_ptr*f1;};struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _tuple9 f2;struct _tuple9 f3;struct Cyc_Absyn_Stmt*f4;};struct Cyc_Absyn_Switch_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;void*f3;};struct Cyc_Absyn_Fallthru_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_List_List*f1;struct Cyc_Absyn_Switch_clause**f2;};struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Decl*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Label_s_Absyn_Raw_stmt_struct{int tag;struct _fat_ptr*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Do_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct _tuple9 f2;};struct Cyc_Absyn_TryCatch_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct Cyc_List_List*f2;void*f3;};struct Cyc_Absyn_Stmt{void*r;unsigned loc;void*annot;};struct Cyc_Absyn_Wild_p_Absyn_Raw_pat_struct{int tag;};struct Cyc_Absyn_Var_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Pat*f2;};struct Cyc_Absyn_AliasVar_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;};struct Cyc_Absyn_Reference_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Pat*f2;};struct Cyc_Absyn_TagInt_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;};struct Cyc_Absyn_Tuple_p_Absyn_Raw_pat_struct{int tag;struct Cyc_List_List*f1;int f2;};struct Cyc_Absyn_Pointer_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Pat*f1;};struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct{int tag;union Cyc_Absyn_AggrInfo*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;int f4;};struct Cyc_Absyn_Datatype_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;struct Cyc_Absyn_Datatypefield*f2;struct Cyc_List_List*f3;int f4;};struct Cyc_Absyn_Null_p_Absyn_Raw_pat_struct{int tag;};struct Cyc_Absyn_Int_p_Absyn_Raw_pat_struct{int tag;enum Cyc_Absyn_Sign f1;int f2;};struct Cyc_Absyn_Char_p_Absyn_Raw_pat_struct{int tag;char f1;};struct Cyc_Absyn_Float_p_Absyn_Raw_pat_struct{int tag;struct _fat_ptr f1;int f2;};struct Cyc_Absyn_Enum_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_AnonEnum_p_Absyn_Raw_pat_struct{int tag;void*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_UnknownId_p_Absyn_Raw_pat_struct{int tag;struct _tuple0*f1;};struct Cyc_Absyn_UnknownCall_p_Absyn_Raw_pat_struct{int tag;struct _tuple0*f1;struct Cyc_List_List*f2;int f3;};struct Cyc_Absyn_Exp_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Exp*f1;};
# 725 "absyn.h"
extern struct Cyc_Absyn_Wild_p_Absyn_Raw_pat_struct Cyc_Absyn_Wild_p_val;
extern struct Cyc_Absyn_Null_p_Absyn_Raw_pat_struct Cyc_Absyn_Null_p_val;struct Cyc_Absyn_Pat{void*r;void*topt;unsigned loc;};struct Cyc_Absyn_Switch_clause{struct Cyc_Absyn_Pat*pattern;struct Cyc_Core_Opt*pat_vars;struct Cyc_Absyn_Exp*where_clause;struct Cyc_Absyn_Stmt*body;unsigned loc;};struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct{int tag;struct _tuple0*f1;};struct Cyc_Absyn_Global_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Funname_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Fndecl*f1;};struct Cyc_Absyn_Param_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Local_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Pat_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Vardecl{enum Cyc_Absyn_Scope sc;struct _tuple0*name;unsigned varloc;struct Cyc_Absyn_Tqual tq;void*type;struct Cyc_Absyn_Exp*initializer;void*rgn;struct Cyc_List_List*attributes;int escapes;};struct Cyc_Absyn_Fndecl{enum Cyc_Absyn_Scope sc;int is_inline;struct _tuple0*name;struct Cyc_Absyn_Stmt*body;struct Cyc_Absyn_FnInfo i;void*cached_type;struct Cyc_Core_Opt*param_vardecls;struct Cyc_Absyn_Vardecl*fn_vardecl;};struct Cyc_Absyn_Aggrfield{struct _fat_ptr*name;struct Cyc_Absyn_Tqual tq;void*type;struct Cyc_Absyn_Exp*width;struct Cyc_List_List*attributes;struct Cyc_Absyn_Exp*requires_clause;};struct Cyc_Absyn_AggrdeclImpl{struct Cyc_List_List*exist_vars;struct Cyc_List_List*rgn_po;struct Cyc_List_List*fields;int tagged;};struct Cyc_Absyn_Aggrdecl{enum Cyc_Absyn_AggrKind kind;enum Cyc_Absyn_Scope sc;struct _tuple0*name;struct Cyc_List_List*tvs;struct Cyc_Absyn_AggrdeclImpl*impl;struct Cyc_List_List*attributes;int expected_mem_kind;};struct Cyc_Absyn_Datatypefield{struct _tuple0*name;struct Cyc_List_List*typs;unsigned loc;enum Cyc_Absyn_Scope sc;};struct Cyc_Absyn_Datatypedecl{enum Cyc_Absyn_Scope sc;struct _tuple0*name;struct Cyc_List_List*tvs;struct Cyc_Core_Opt*fields;int is_extensible;};struct Cyc_Absyn_Enumfield{struct _tuple0*name;struct Cyc_Absyn_Exp*tag;unsigned loc;};struct Cyc_Absyn_Enumdecl{enum Cyc_Absyn_Scope sc;struct _tuple0*name;struct Cyc_Core_Opt*fields;};struct Cyc_Absyn_Typedefdecl{struct _tuple0*name;struct Cyc_Absyn_Tqual tq;struct Cyc_List_List*tvs;struct Cyc_Core_Opt*kind;void*defn;struct Cyc_List_List*atts;int extern_c;};struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Fn_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Fndecl*f1;};struct Cyc_Absyn_Let_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Pat*f1;struct Cyc_Core_Opt*f2;struct Cyc_Absyn_Exp*f3;void*f4;};struct Cyc_Absyn_Letv_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_Region_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_Aggr_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Aggrdecl*f1;};struct Cyc_Absyn_Datatype_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;};struct Cyc_Absyn_Enum_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;};struct Cyc_Absyn_Typedef_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Typedefdecl*f1;};struct Cyc_Absyn_Namespace_d_Absyn_Raw_decl_struct{int tag;struct _fat_ptr*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Using_d_Absyn_Raw_decl_struct{int tag;struct _tuple0*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_ExternC_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;};struct _tuple10{unsigned f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_ExternCinclude_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;struct _tuple10*f4;};struct Cyc_Absyn_Porton_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Portoff_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Tempeston_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Tempestoff_d_Absyn_Raw_decl_struct{int tag;};
# 884
extern struct Cyc_Absyn_Porton_d_Absyn_Raw_decl_struct Cyc_Absyn_Porton_d_val;
extern struct Cyc_Absyn_Portoff_d_Absyn_Raw_decl_struct Cyc_Absyn_Portoff_d_val;
extern struct Cyc_Absyn_Tempeston_d_Absyn_Raw_decl_struct Cyc_Absyn_Tempeston_d_val;
extern struct Cyc_Absyn_Tempestoff_d_Absyn_Raw_decl_struct Cyc_Absyn_Tempestoff_d_val;struct Cyc_Absyn_Decl{void*r;unsigned loc;};struct Cyc_Absyn_ArrayElement_Absyn_Designator_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_FieldName_Absyn_Designator_struct{int tag;struct _fat_ptr*f1;};extern char Cyc_Absyn_EmptyAnnot[11U];struct Cyc_Absyn_EmptyAnnot_Absyn_AbsynAnnot_struct{char*tag;};
# 913
int Cyc_Absyn_is_qvar_qualified(struct _tuple0*);
# 917
struct Cyc_Absyn_Tqual Cyc_Absyn_empty_tqual(unsigned);
struct Cyc_Absyn_Tqual Cyc_Absyn_combine_tqual(struct Cyc_Absyn_Tqual,struct Cyc_Absyn_Tqual);
# 922
void*Cyc_Absyn_compress_kb(void*);
# 933
void*Cyc_Absyn_new_evar(struct Cyc_Core_Opt*k,struct Cyc_Core_Opt*tenv);
# 935
void*Cyc_Absyn_wildtyp(struct Cyc_Core_Opt*);
void*Cyc_Absyn_int_type(enum Cyc_Absyn_Sign,enum Cyc_Absyn_Size_of);
# 938
extern void*Cyc_Absyn_char_type;extern void*Cyc_Absyn_uint_type;
# 940
extern void*Cyc_Absyn_sint_type;
# 942
extern void*Cyc_Absyn_float_type;extern void*Cyc_Absyn_double_type;extern void*Cyc_Absyn_long_double_type;
# 945
extern void*Cyc_Absyn_heap_rgn_type;extern void*Cyc_Absyn_unique_rgn_type;extern void*Cyc_Absyn_refcnt_rgn_type;
# 949
extern void*Cyc_Absyn_true_type;extern void*Cyc_Absyn_false_type;
# 951
extern void*Cyc_Absyn_void_type;void*Cyc_Absyn_var_type(struct Cyc_Absyn_Tvar*);void*Cyc_Absyn_tag_type(void*);void*Cyc_Absyn_rgn_handle_type(void*);void*Cyc_Absyn_valueof_type(struct Cyc_Absyn_Exp*);void*Cyc_Absyn_typeof_type(struct Cyc_Absyn_Exp*);void*Cyc_Absyn_access_eff(void*);void*Cyc_Absyn_join_eff(struct Cyc_List_List*);void*Cyc_Absyn_regionsof_eff(void*);void*Cyc_Absyn_enum_type(struct _tuple0*n,struct Cyc_Absyn_Enumdecl*d);void*Cyc_Absyn_anon_enum_type(struct Cyc_List_List*);void*Cyc_Absyn_builtin_type(struct _fat_ptr s,struct Cyc_Absyn_Kind*k);void*Cyc_Absyn_typedef_type(struct _tuple0*,struct Cyc_List_List*,struct Cyc_Absyn_Typedefdecl*,void*);
# 976
extern void*Cyc_Absyn_fat_bound_type;
void*Cyc_Absyn_thin_bounds_type(void*);
void*Cyc_Absyn_thin_bounds_exp(struct Cyc_Absyn_Exp*);
# 980
void*Cyc_Absyn_bounds_one();
# 982
void*Cyc_Absyn_pointer_type(struct Cyc_Absyn_PtrInfo);
# 1004
void*Cyc_Absyn_array_type(void*elt_type,struct Cyc_Absyn_Tqual tq,struct Cyc_Absyn_Exp*num_elts,void*zero_term,unsigned ztloc);
# 1007
void*Cyc_Absyn_datatype_type(union Cyc_Absyn_DatatypeInfo,struct Cyc_List_List*args);
void*Cyc_Absyn_datatype_field_type(union Cyc_Absyn_DatatypeFieldInfo,struct Cyc_List_List*args);
void*Cyc_Absyn_aggr_type(union Cyc_Absyn_AggrInfo,struct Cyc_List_List*args);
# 1012
struct Cyc_Absyn_Exp*Cyc_Absyn_new_exp(void*,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_New_exp(struct Cyc_Absyn_Exp*rgn_handle,struct Cyc_Absyn_Exp*,unsigned);
# 1015
struct Cyc_Absyn_Exp*Cyc_Absyn_const_exp(union Cyc_Absyn_Cnst,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_null_exp(unsigned);
# 1018
struct Cyc_Absyn_Exp*Cyc_Absyn_true_exp(unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_false_exp(unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_int_exp(enum Cyc_Absyn_Sign,int,unsigned);
# 1022
struct Cyc_Absyn_Exp*Cyc_Absyn_uint_exp(unsigned,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_char_exp(char,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_wchar_exp(struct _fat_ptr,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_float_exp(struct _fat_ptr,int,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_string_exp(struct _fat_ptr,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_wstring_exp(struct _fat_ptr,unsigned);
# 1030
struct Cyc_Absyn_Exp*Cyc_Absyn_unknownid_exp(struct _tuple0*,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_pragma_exp(struct _fat_ptr,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_primop_exp(enum Cyc_Absyn_Primop,struct Cyc_List_List*,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_prim1_exp(enum Cyc_Absyn_Primop,struct Cyc_Absyn_Exp*,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_prim2_exp(enum Cyc_Absyn_Primop,struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_swap_exp(struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*,unsigned);
# 1039
struct Cyc_Absyn_Exp*Cyc_Absyn_eq_exp(struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_neq_exp(struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_gt_exp(struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_lt_exp(struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_gte_exp(struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_lte_exp(struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_assignop_exp(struct Cyc_Absyn_Exp*,struct Cyc_Core_Opt*,struct Cyc_Absyn_Exp*,unsigned);
# 1047
struct Cyc_Absyn_Exp*Cyc_Absyn_increment_exp(struct Cyc_Absyn_Exp*,enum Cyc_Absyn_Incrementor,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_conditional_exp(struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_and_exp(struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_or_exp(struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_seq_exp(struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_unknowncall_exp(struct Cyc_Absyn_Exp*,struct Cyc_List_List*,unsigned);
# 1054
struct Cyc_Absyn_Exp*Cyc_Absyn_throw_exp(struct Cyc_Absyn_Exp*,unsigned);
# 1056
struct Cyc_Absyn_Exp*Cyc_Absyn_noinstantiate_exp(struct Cyc_Absyn_Exp*,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_instantiate_exp(struct Cyc_Absyn_Exp*,struct Cyc_List_List*,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_cast_exp(void*,struct Cyc_Absyn_Exp*,int user_cast,enum Cyc_Absyn_Coercion,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_address_exp(struct Cyc_Absyn_Exp*,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_sizeoftype_exp(void*t,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_sizeofexp_exp(struct Cyc_Absyn_Exp*e,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_offsetof_exp(void*,struct Cyc_List_List*,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_deref_exp(struct Cyc_Absyn_Exp*,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_aggrmember_exp(struct Cyc_Absyn_Exp*,struct _fat_ptr*,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_aggrarrow_exp(struct Cyc_Absyn_Exp*,struct _fat_ptr*,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_subscript_exp(struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_tuple_exp(struct Cyc_List_List*,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_stmt_exp(struct Cyc_Absyn_Stmt*,unsigned);
# 1071
struct Cyc_Absyn_Exp*Cyc_Absyn_valueof_exp(void*,unsigned);
# 1074
struct Cyc_Absyn_Exp*Cyc_Absyn_extension_exp(struct Cyc_Absyn_Exp*,unsigned);
# 1081
struct Cyc_Absyn_Stmt*Cyc_Absyn_new_stmt(void*,unsigned);
struct Cyc_Absyn_Stmt*Cyc_Absyn_skip_stmt(unsigned);
struct Cyc_Absyn_Stmt*Cyc_Absyn_exp_stmt(struct Cyc_Absyn_Exp*,unsigned);
struct Cyc_Absyn_Stmt*Cyc_Absyn_seq_stmt(struct Cyc_Absyn_Stmt*,struct Cyc_Absyn_Stmt*,unsigned);
# 1086
struct Cyc_Absyn_Stmt*Cyc_Absyn_return_stmt(struct Cyc_Absyn_Exp*,unsigned);
struct Cyc_Absyn_Stmt*Cyc_Absyn_ifthenelse_stmt(struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Stmt*,struct Cyc_Absyn_Stmt*,unsigned);
struct Cyc_Absyn_Stmt*Cyc_Absyn_while_stmt(struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Stmt*,unsigned);
struct Cyc_Absyn_Stmt*Cyc_Absyn_break_stmt(unsigned);
struct Cyc_Absyn_Stmt*Cyc_Absyn_continue_stmt(unsigned);
struct Cyc_Absyn_Stmt*Cyc_Absyn_for_stmt(struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Stmt*,unsigned);
struct Cyc_Absyn_Stmt*Cyc_Absyn_switch_stmt(struct Cyc_Absyn_Exp*,struct Cyc_List_List*,unsigned);
struct Cyc_Absyn_Stmt*Cyc_Absyn_fallthru_stmt(struct Cyc_List_List*,unsigned);
# 1097
struct Cyc_Absyn_Stmt*Cyc_Absyn_do_stmt(struct Cyc_Absyn_Stmt*,struct Cyc_Absyn_Exp*,unsigned);
struct Cyc_Absyn_Stmt*Cyc_Absyn_goto_stmt(struct _fat_ptr*,unsigned);
# 1100
struct Cyc_Absyn_Stmt*Cyc_Absyn_trycatch_stmt(struct Cyc_Absyn_Stmt*,struct Cyc_List_List*,unsigned);
# 1103
struct Cyc_Absyn_Pat*Cyc_Absyn_new_pat(void*,unsigned);
struct Cyc_Absyn_Pat*Cyc_Absyn_exp_pat(struct Cyc_Absyn_Exp*);
# 1107
struct Cyc_Absyn_Decl*Cyc_Absyn_new_decl(void*,unsigned);
struct Cyc_Absyn_Decl*Cyc_Absyn_let_decl(struct Cyc_Absyn_Pat*,struct Cyc_Absyn_Exp*,unsigned);
struct Cyc_Absyn_Decl*Cyc_Absyn_letv_decl(struct Cyc_List_List*,unsigned);
struct Cyc_Absyn_Decl*Cyc_Absyn_region_decl(struct Cyc_Absyn_Tvar*,struct Cyc_Absyn_Vardecl*,struct Cyc_Absyn_Exp*open_exp,unsigned);
# 1112
struct Cyc_Absyn_Vardecl*Cyc_Absyn_new_vardecl(unsigned varloc,struct _tuple0*,void*,struct Cyc_Absyn_Exp*init);
# 1114
struct Cyc_Absyn_AggrdeclImpl*Cyc_Absyn_aggrdecl_impl(struct Cyc_List_List*exists,struct Cyc_List_List*po,struct Cyc_List_List*fs,int tagged);
# 1121
struct Cyc_Absyn_TypeDecl*Cyc_Absyn_aggr_tdecl(enum Cyc_Absyn_AggrKind,enum Cyc_Absyn_Scope,struct _tuple0*,struct Cyc_List_List*ts,struct Cyc_Absyn_AggrdeclImpl*,struct Cyc_List_List*,unsigned);
# 1128
struct Cyc_Absyn_Decl*Cyc_Absyn_datatype_decl(enum Cyc_Absyn_Scope,struct _tuple0*,struct Cyc_List_List*ts,struct Cyc_Core_Opt*fs,int is_extensible,unsigned);
# 1131
struct Cyc_Absyn_TypeDecl*Cyc_Absyn_datatype_tdecl(enum Cyc_Absyn_Scope,struct _tuple0*,struct Cyc_List_List*ts,struct Cyc_Core_Opt*fs,int is_extensible,unsigned);
# 1135
void*Cyc_Absyn_function_type(struct Cyc_List_List*tvs,void*eff_typ,struct Cyc_Absyn_Tqual ret_tqual,void*ret_type,struct Cyc_List_List*args,int c_varargs,struct Cyc_Absyn_VarargInfo*cyc_varargs,struct Cyc_List_List*rgn_po,struct Cyc_List_List*,struct Cyc_Absyn_Exp*requires_clause,struct Cyc_Absyn_Exp*ensures_clause,struct Cyc_List_List*,struct Cyc_List_List*);
# 1166
int Cyc_Absyn_fntype_att(void*);
# 1188
extern int Cyc_Absyn_porting_c_code;
# 1203
void*Cyc_Absyn_parse_nullary_att(unsigned loc,struct _fat_ptr s);
void*Cyc_Absyn_parse_unary_att(unsigned sloc,struct _fat_ptr s,unsigned eloc,struct Cyc_Absyn_Exp*e);
void*Cyc_Absyn_parse_format_att(unsigned loc,unsigned s2loc,struct _fat_ptr s1,struct _fat_ptr s2,unsigned u1,unsigned u2);
# 1209
struct Cyc_List_List*Cyc_Absyn_parse_rgneffects(unsigned loc,void*t);
# 29 "warn.h"
void Cyc_Warn_warn(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap);
# 33
void Cyc_Warn_verr(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap);
# 35
void Cyc_Warn_err(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap);struct Cyc_Warn_String_Warn_Warg_struct{int tag;struct _fat_ptr f1;};struct Cyc_Warn_Exp_Warn_Warg_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Warn_Typ_Warn_Warg_struct{int tag;void*f1;};struct Cyc_Warn_Qvar_Warn_Warg_struct{int tag;struct _tuple0*f1;};struct _union_RelnOp_RConst{int tag;unsigned val;};struct _union_RelnOp_RVar{int tag;struct Cyc_Absyn_Vardecl*val;};struct _union_RelnOp_RNumelts{int tag;struct Cyc_Absyn_Vardecl*val;};struct _union_RelnOp_RType{int tag;void*val;};struct _union_RelnOp_RParam{int tag;unsigned val;};struct _union_RelnOp_RParamNumelts{int tag;unsigned val;};struct _union_RelnOp_RReturn{int tag;unsigned val;};union Cyc_Relations_RelnOp{struct _union_RelnOp_RConst RConst;struct _union_RelnOp_RVar RVar;struct _union_RelnOp_RNumelts RNumelts;struct _union_RelnOp_RType RType;struct _union_RelnOp_RParam RParam;struct _union_RelnOp_RParamNumelts RParamNumelts;struct _union_RelnOp_RReturn RReturn;};
# 50 "relations-ap.h"
enum Cyc_Relations_Relation{Cyc_Relations_Req =0U,Cyc_Relations_Rneq =1U,Cyc_Relations_Rlte =2U,Cyc_Relations_Rlt =3U};struct Cyc_Relations_Reln{union Cyc_Relations_RelnOp rop1;enum Cyc_Relations_Relation relation;union Cyc_Relations_RelnOp rop2;};struct Cyc_Iter_Iter{void*env;int(*next)(void*env,void*dest);};struct Cyc_Dict_T;struct Cyc_Dict_Dict{int(*rel)(void*,void*);struct _RegionHandle*r;const struct Cyc_Dict_T*t;};extern char Cyc_Dict_Present[8U];struct Cyc_Dict_Present_exn_struct{char*tag;};extern char Cyc_Dict_Absent[7U];struct Cyc_Dict_Absent_exn_struct{char*tag;};struct Cyc_RgnOrder_RgnPO;extern char Cyc_Tcenv_Env_error[10U];struct Cyc_Tcenv_Env_error_exn_struct{char*tag;};struct Cyc_Tcenv_Genv{struct Cyc_Dict_Dict aggrdecls;struct Cyc_Dict_Dict datatypedecls;struct Cyc_Dict_Dict enumdecls;struct Cyc_Dict_Dict typedefs;struct Cyc_Dict_Dict ordinaries;};struct Cyc_Tcenv_Fenv;struct Cyc_Tcenv_Tenv{struct Cyc_List_List*ns;struct Cyc_Tcenv_Genv*ae;struct Cyc_Tcenv_Fenv*le;int allow_valueof: 1;int in_extern_c_include: 1;int in_tempest: 1;int tempest_generalize: 1;};
# 88 "tcenv.h"
enum Cyc_Tcenv_NewStatus{Cyc_Tcenv_NoneNew =0U,Cyc_Tcenv_InNew =1U,Cyc_Tcenv_InNewAggr =2U};
# 45 "tcutil.h"
int Cyc_Tcutil_is_array_type(void*);
# 94
void*Cyc_Tcutil_copy_type(void*);
# 107
void*Cyc_Tcutil_compress(void*);
# 135
extern struct Cyc_Absyn_Kind Cyc_Tcutil_rk;
extern struct Cyc_Absyn_Kind Cyc_Tcutil_ak;
extern struct Cyc_Absyn_Kind Cyc_Tcutil_bk;
extern struct Cyc_Absyn_Kind Cyc_Tcutil_mk;
extern struct Cyc_Absyn_Kind Cyc_Tcutil_ek;
extern struct Cyc_Absyn_Kind Cyc_Tcutil_ik;
# 144
extern struct Cyc_Absyn_Kind Cyc_Tcutil_trk;
extern struct Cyc_Absyn_Kind Cyc_Tcutil_tak;
extern struct Cyc_Absyn_Kind Cyc_Tcutil_tbk;
extern struct Cyc_Absyn_Kind Cyc_Tcutil_tmk;
# 149
extern struct Cyc_Absyn_Kind Cyc_Tcutil_urk;
extern struct Cyc_Absyn_Kind Cyc_Tcutil_uak;
extern struct Cyc_Absyn_Kind Cyc_Tcutil_ubk;
extern struct Cyc_Absyn_Kind Cyc_Tcutil_umk;
# 154
extern struct Cyc_Core_Opt Cyc_Tcutil_rko;
# 156
extern struct Cyc_Core_Opt Cyc_Tcutil_bko;
# 158
extern struct Cyc_Core_Opt Cyc_Tcutil_iko;
# 163
extern struct Cyc_Core_Opt Cyc_Tcutil_trko;
# 173
struct Cyc_Core_Opt*Cyc_Tcutil_kind_to_opt(struct Cyc_Absyn_Kind*k);
void*Cyc_Tcutil_kind_to_bound(struct Cyc_Absyn_Kind*k);
# 256
struct Cyc_Absyn_Tvar*Cyc_Tcutil_new_tvar(void*);
# 298
void*Cyc_Tcutil_promote_array(void*t,void*rgn,int convert_tag);
# 308
void*Cyc_Tcutil_any_bool(struct Cyc_List_List*);struct Cyc_PP_Ppstate;struct Cyc_PP_Out;struct Cyc_PP_Doc;struct Cyc_Absynpp_Params{int expand_typedefs;int qvar_to_Cids;int add_cyc_prefix;int to_VC;int decls_first;int rewrite_temp_tvars;int print_all_tvars;int print_all_kinds;int print_all_effects;int print_using_stmts;int print_externC_stmts;int print_full_evars;int print_zeroterm;int generate_line_directives;int use_curr_namespace;struct Cyc_List_List*curr_namespace;};
# 66 "absynpp.h"
struct _fat_ptr Cyc_Absynpp_cnst2string(union Cyc_Absyn_Cnst);
struct _fat_ptr Cyc_Absynpp_exp2string(struct Cyc_Absyn_Exp*);
struct _fat_ptr Cyc_Absynpp_stmt2string(struct Cyc_Absyn_Stmt*);
struct _fat_ptr Cyc_Absynpp_qvar2string(struct _tuple0*);
# 64 "parse.y"
extern void(*Cyc_beforedie)();
# 70
void Cyc_Lex_register_typedef(struct _tuple0*s);
void Cyc_Lex_enter_namespace(struct _fat_ptr*);
void Cyc_Lex_leave_namespace();
void Cyc_Lex_enter_using(struct _tuple0*);
void Cyc_Lex_leave_using();
void Cyc_Lex_enter_extern_c();
void Cyc_Lex_leave_extern_c();
extern struct _tuple0*Cyc_Lex_token_qvar;
extern struct _fat_ptr Cyc_Lex_token_string;
# 96 "parse.y"
int Cyc_Parse_parsing_tempest=0;struct Cyc_Parse_FlatList{struct Cyc_Parse_FlatList*tl;void*hd[0U] __attribute__((aligned )) ;};
# 102
struct Cyc_Parse_FlatList*Cyc_Parse_flat_imp_rev(struct Cyc_Parse_FlatList*x){
if(x == 0)return x;else{
# 105
struct Cyc_Parse_FlatList*first=x;
struct Cyc_Parse_FlatList*second=x->tl;
x->tl=0;
while(second != 0){
struct Cyc_Parse_FlatList*temp=second->tl;
second->tl=first;
first=second;
second=temp;}
# 114
return first;}}
# 102
int Cyc_Parse_no_register=0;char Cyc_Parse_Exit[5U]="Exit";struct Cyc_Parse_Exit_exn_struct{char*tag;};struct Cyc_Parse_Type_specifier{int Signed_spec: 1;int Unsigned_spec: 1;int Short_spec: 1;int Long_spec: 1;int Long_Long_spec: 1;int Valid_type_spec: 1;void*Type_spec;unsigned loc;};
# 136
enum Cyc_Parse_Storage_class{Cyc_Parse_Typedef_sc =0U,Cyc_Parse_Extern_sc =1U,Cyc_Parse_ExternC_sc =2U,Cyc_Parse_Static_sc =3U,Cyc_Parse_Auto_sc =4U,Cyc_Parse_Register_sc =5U,Cyc_Parse_Abstract_sc =6U};struct Cyc_Parse_Declaration_spec{enum Cyc_Parse_Storage_class*sc;struct Cyc_Absyn_Tqual tq;struct Cyc_Parse_Type_specifier type_specs;int is_inline;struct Cyc_List_List*attributes;};struct Cyc_Parse_Declarator{struct _tuple0*id;unsigned varloc;struct Cyc_List_List*tms;};struct _tuple11{struct _tuple11*tl;struct Cyc_Parse_Declarator hd  __attribute__((aligned )) ;};struct _tuple12{struct Cyc_Parse_Declarator f1;struct Cyc_Absyn_Exp*f2;};struct _tuple13{struct _tuple13*tl;struct _tuple12 hd  __attribute__((aligned )) ;};struct Cyc_Parse_Numelts_ptrqual_Parse_Pointer_qual_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Parse_Region_ptrqual_Parse_Pointer_qual_struct{int tag;void*f1;};struct Cyc_Parse_Thin_ptrqual_Parse_Pointer_qual_struct{int tag;};struct Cyc_Parse_Fat_ptrqual_Parse_Pointer_qual_struct{int tag;};struct Cyc_Parse_Zeroterm_ptrqual_Parse_Pointer_qual_struct{int tag;};struct Cyc_Parse_Nozeroterm_ptrqual_Parse_Pointer_qual_struct{int tag;};struct Cyc_Parse_Notnull_ptrqual_Parse_Pointer_qual_struct{int tag;};struct Cyc_Parse_Nullable_ptrqual_Parse_Pointer_qual_struct{int tag;};
# 172
static void Cyc_Parse_decl_split(struct _RegionHandle*r,struct _tuple13*ds,struct _tuple11**decls,struct Cyc_List_List**es){
# 176
struct _tuple11*declarators=0;
struct Cyc_List_List*exprs=0;
for(0;ds != 0;ds=ds->tl){
struct _tuple12 _stmttmp0=ds->hd;struct Cyc_Absyn_Exp*_tmp1;struct Cyc_Parse_Declarator _tmp0;_LL1: _tmp0=_stmttmp0.f1;_tmp1=_stmttmp0.f2;_LL2: {struct Cyc_Parse_Declarator d=_tmp0;struct Cyc_Absyn_Exp*e=_tmp1;
declarators=({struct _tuple11*_tmp2=_region_malloc(r,sizeof(*_tmp2));_tmp2->tl=declarators,_tmp2->hd=d;_tmp2;});
exprs=({struct Cyc_List_List*_tmp3=_region_malloc(r,sizeof(*_tmp3));_tmp3->hd=e,_tmp3->tl=exprs;_tmp3;});}}
# 183
({struct Cyc_List_List*_tmp687=Cyc_List_imp_rev(exprs);*es=_tmp687;});
({struct _tuple11*_tmp689=({struct _tuple11*(*_tmp688)(struct _tuple11*x)=({struct _tuple11*(*_tmp4)(struct _tuple11*x)=(struct _tuple11*(*)(struct _tuple11*x))Cyc_Parse_flat_imp_rev;_tmp4;});_tmp688(declarators);});*decls=_tmp689;});}struct Cyc_Parse_Abstractdeclarator{struct Cyc_List_List*tms;};
# 193
static void*Cyc_Parse_collapse_type_specifiers(struct Cyc_Parse_Type_specifier ts,unsigned loc);struct _tuple14{struct Cyc_Absyn_Tqual f1;void*f2;struct Cyc_List_List*f3;struct Cyc_List_List*f4;};
static struct _tuple14 Cyc_Parse_apply_tms(struct Cyc_Absyn_Tqual,void*,struct Cyc_List_List*,struct Cyc_List_List*);
# 199
static struct Cyc_List_List*Cyc_Parse_parse_result=0;
# 201
static void*Cyc_Parse_parse_abort(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap){
# 203
Cyc_Warn_verr(loc,fmt,ap);
(int)_throw((void*)({struct Cyc_Parse_Exit_exn_struct*_tmp5=_cycalloc(sizeof(*_tmp5));_tmp5->tag=Cyc_Parse_Exit;_tmp5;}));}
# 201
static void*Cyc_Parse_type_name_to_type(struct _tuple8*tqt,unsigned loc){
# 209
void*_tmp7;struct Cyc_Absyn_Tqual _tmp6;_LL1: _tmp6=tqt->f2;_tmp7=tqt->f3;_LL2: {struct Cyc_Absyn_Tqual tq=_tmp6;void*t=_tmp7;
if((tq.print_const || tq.q_volatile)|| tq.q_restrict){
if(tq.loc != (unsigned)0)loc=tq.loc;({void*_tmp8=0U;({unsigned _tmp68B=loc;struct _fat_ptr _tmp68A=({const char*_tmp9="qualifier on type is ignored";_tag_fat(_tmp9,sizeof(char),29U);});Cyc_Warn_warn(_tmp68B,_tmp68A,_tag_fat(_tmp8,sizeof(void*),0U));});});}
# 210
return t;}}struct _tuple15{void*f1;void*f2;void*f3;void*f4;};
# 201
static struct _tuple15 Cyc_Parse_collapse_pointer_quals(unsigned loc,void*nullable,void*bound,void*rgn,struct Cyc_List_List*pqs){
# 223
void*zeroterm=Cyc_Tcutil_any_bool(0);
for(0;pqs != 0;pqs=pqs->tl){
void*_stmttmp1=(void*)pqs->hd;void*_tmpA=_stmttmp1;void*_tmpB;struct Cyc_Absyn_Exp*_tmpC;switch(*((int*)_tmpA)){case 4U: _LL1: _LL2:
 zeroterm=Cyc_Absyn_true_type;goto _LL0;case 5U: _LL3: _LL4:
 zeroterm=Cyc_Absyn_false_type;goto _LL0;case 7U: _LL5: _LL6:
 nullable=Cyc_Absyn_true_type;goto _LL0;case 6U: _LL7: _LL8:
 nullable=Cyc_Absyn_false_type;goto _LL0;case 3U: _LL9: _LLA:
 bound=Cyc_Absyn_fat_bound_type;goto _LL0;case 2U: _LLB: _LLC:
 bound=Cyc_Absyn_bounds_one();goto _LL0;case 0U: _LLD: _tmpC=((struct Cyc_Parse_Numelts_ptrqual_Parse_Pointer_qual_struct*)_tmpA)->f1;_LLE: {struct Cyc_Absyn_Exp*e=_tmpC;
bound=Cyc_Absyn_thin_bounds_exp(e);goto _LL0;}default: _LLF: _tmpB=(void*)((struct Cyc_Parse_Region_ptrqual_Parse_Pointer_qual_struct*)_tmpA)->f1;_LL10: {void*t=_tmpB;
rgn=t;goto _LL0;}}_LL0:;}
# 235
return({struct _tuple15 _tmp5E5;_tmp5E5.f1=nullable,_tmp5E5.f2=bound,_tmp5E5.f3=zeroterm,_tmp5E5.f4=rgn;_tmp5E5;});}
# 201
struct _tuple0*Cyc_Parse_gensym_enum(){
# 243
static int enum_counter=0;
return({struct _tuple0*_tmp11=_cycalloc(sizeof(*_tmp11));({union Cyc_Absyn_Nmspace _tmp68F=Cyc_Absyn_Rel_n(0);_tmp11->f1=_tmp68F;}),({
struct _fat_ptr*_tmp68E=({struct _fat_ptr*_tmp10=_cycalloc(sizeof(*_tmp10));({struct _fat_ptr _tmp68D=(struct _fat_ptr)({struct Cyc_Int_pa_PrintArg_struct _tmpF=({struct Cyc_Int_pa_PrintArg_struct _tmp5E6;_tmp5E6.tag=1U,_tmp5E6.f1=(unsigned long)enum_counter ++;_tmp5E6;});void*_tmpD[1U];_tmpD[0]=& _tmpF;({struct _fat_ptr _tmp68C=({const char*_tmpE="__anonymous_enum_%d__";_tag_fat(_tmpE,sizeof(char),22U);});Cyc_aprintf(_tmp68C,_tag_fat(_tmpD,sizeof(void*),1U));});});*_tmp10=_tmp68D;});_tmp10;});_tmp11->f2=_tmp68E;});_tmp11;});}struct _tuple16{unsigned f1;struct _tuple0*f2;struct Cyc_Absyn_Tqual f3;void*f4;struct Cyc_List_List*f5;struct Cyc_List_List*f6;};struct _tuple17{struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct _tuple18{struct _tuple16*f1;struct _tuple17*f2;};
# 201
static struct Cyc_Absyn_Aggrfield*Cyc_Parse_make_aggr_field(unsigned loc,struct _tuple18*field_info){
# 253
struct Cyc_Absyn_Exp*_tmp19;struct Cyc_Absyn_Exp*_tmp18;struct Cyc_List_List*_tmp17;struct Cyc_List_List*_tmp16;void*_tmp15;struct Cyc_Absyn_Tqual _tmp14;struct _tuple0*_tmp13;unsigned _tmp12;_LL1: _tmp12=(field_info->f1)->f1;_tmp13=(field_info->f1)->f2;_tmp14=(field_info->f1)->f3;_tmp15=(field_info->f1)->f4;_tmp16=(field_info->f1)->f5;_tmp17=(field_info->f1)->f6;_tmp18=(field_info->f2)->f1;_tmp19=(field_info->f2)->f2;_LL2: {unsigned varloc=_tmp12;struct _tuple0*qid=_tmp13;struct Cyc_Absyn_Tqual tq=_tmp14;void*t=_tmp15;struct Cyc_List_List*tvs=_tmp16;struct Cyc_List_List*atts=_tmp17;struct Cyc_Absyn_Exp*widthopt=_tmp18;struct Cyc_Absyn_Exp*reqopt=_tmp19;
if(tvs != 0)
({void*_tmp1A=0U;({unsigned _tmp691=loc;struct _fat_ptr _tmp690=({const char*_tmp1B="bad type params in struct field";_tag_fat(_tmp1B,sizeof(char),32U);});Cyc_Warn_err(_tmp691,_tmp690,_tag_fat(_tmp1A,sizeof(void*),0U));});});
# 254
if(
# 256
Cyc_Absyn_is_qvar_qualified(qid))
({void*_tmp1C=0U;({unsigned _tmp693=loc;struct _fat_ptr _tmp692=({const char*_tmp1D="struct or union field cannot be qualified with a namespace";_tag_fat(_tmp1D,sizeof(char),59U);});Cyc_Warn_err(_tmp693,_tmp692,_tag_fat(_tmp1C,sizeof(void*),0U));});});
# 254
return({struct Cyc_Absyn_Aggrfield*_tmp1E=_cycalloc(sizeof(*_tmp1E));
# 258
_tmp1E->name=(*qid).f2,_tmp1E->tq=tq,_tmp1E->type=t,_tmp1E->width=widthopt,_tmp1E->attributes=atts,_tmp1E->requires_clause=reqopt;_tmp1E;});}}
# 201
static struct Cyc_Parse_Type_specifier Cyc_Parse_empty_spec(unsigned loc){
# 264
return({struct Cyc_Parse_Type_specifier _tmp5E7;_tmp5E7.Signed_spec=0,_tmp5E7.Unsigned_spec=0,_tmp5E7.Short_spec=0,_tmp5E7.Long_spec=0,_tmp5E7.Long_Long_spec=0,_tmp5E7.Valid_type_spec=0,_tmp5E7.Type_spec=Cyc_Absyn_sint_type,_tmp5E7.loc=loc;_tmp5E7;});}
# 201
static struct Cyc_Parse_Type_specifier Cyc_Parse_type_spec(void*t,unsigned loc){
# 275
struct Cyc_Parse_Type_specifier s=Cyc_Parse_empty_spec(loc);
s.Type_spec=t;
s.Valid_type_spec=1;
return s;}
# 201
static struct Cyc_Parse_Type_specifier Cyc_Parse_signed_spec(unsigned loc){
# 281
struct Cyc_Parse_Type_specifier s=Cyc_Parse_empty_spec(loc);
s.Signed_spec=1;
return s;}
# 201
static struct Cyc_Parse_Type_specifier Cyc_Parse_unsigned_spec(unsigned loc){
# 286
struct Cyc_Parse_Type_specifier s=Cyc_Parse_empty_spec(loc);
s.Unsigned_spec=1;
return s;}
# 201
static struct Cyc_Parse_Type_specifier Cyc_Parse_short_spec(unsigned loc){
# 291
struct Cyc_Parse_Type_specifier s=Cyc_Parse_empty_spec(loc);
s.Short_spec=1;
return s;}
# 201
static struct Cyc_Parse_Type_specifier Cyc_Parse_long_spec(unsigned loc){
# 296
struct Cyc_Parse_Type_specifier s=Cyc_Parse_empty_spec(loc);
s.Long_spec=1;
return s;}
# 201
static void*Cyc_Parse_array2ptr(void*t,int argposn){
# 304
return Cyc_Tcutil_is_array_type(t)?({
void*_tmp694=t;Cyc_Tcutil_promote_array(_tmp694,argposn?Cyc_Absyn_new_evar(& Cyc_Tcutil_rko,0): Cyc_Absyn_heap_rgn_type,0);}): t;}struct _tuple19{struct _fat_ptr*f1;void*f2;};
# 201
static struct Cyc_List_List*Cyc_Parse_get_arg_tags(struct Cyc_List_List*x){
# 318 "parse.y"
struct Cyc_List_List*res=0;
for(0;x != 0;x=x->tl){
struct _tuple8*_stmttmp2=(struct _tuple8*)x->hd;struct _tuple8*_tmp1F=_stmttmp2;void**_tmp21;struct _fat_ptr _tmp20;void*_tmp23;struct _fat_ptr*_tmp22;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)((struct _tuple8*)_tmp1F)->f3)->tag == 0U){if(((struct Cyc_Absyn_TagCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)((struct _tuple8*)_tmp1F)->f3)->f1)->tag == 4U){if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)((struct _tuple8*)_tmp1F)->f3)->f2 != 0){if(((struct Cyc_List_List*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)((struct _tuple8*)_tmp1F)->f3)->f2)->tl == 0){_LL1: _tmp22=_tmp1F->f1;_tmp23=(void*)(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp1F->f3)->f2)->hd;if(_tmp22 != 0){_LL2: {struct _fat_ptr*v=_tmp22;void*i=_tmp23;
# 322
{void*_tmp24=i;void**_tmp25;if(((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp24)->tag == 1U){_LL8: _tmp25=(void**)&((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp24)->f2;_LL9: {void**z=_tmp25;
# 326
struct _fat_ptr*nm=({struct _fat_ptr*_tmp2B=_cycalloc(sizeof(*_tmp2B));({struct _fat_ptr _tmp696=(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp2A=({struct Cyc_String_pa_PrintArg_struct _tmp5E8;_tmp5E8.tag=0U,_tmp5E8.f1=(struct _fat_ptr)((struct _fat_ptr)*v);_tmp5E8;});void*_tmp28[1U];_tmp28[0]=& _tmp2A;({struct _fat_ptr _tmp695=({const char*_tmp29="`%s";_tag_fat(_tmp29,sizeof(char),4U);});Cyc_aprintf(_tmp695,_tag_fat(_tmp28,sizeof(void*),1U));});});*_tmp2B=_tmp696;});_tmp2B;});
({void*_tmp698=Cyc_Absyn_var_type(({struct Cyc_Absyn_Tvar*_tmp27=_cycalloc(sizeof(*_tmp27));_tmp27->name=nm,_tmp27->identity=- 1,({void*_tmp697=(void*)({struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct*_tmp26=_cycalloc(sizeof(*_tmp26));_tmp26->tag=0U,_tmp26->f1=& Cyc_Tcutil_ik;_tmp26;});_tmp27->kind=_tmp697;});_tmp27;}));*z=_tmp698;});
goto _LL7;}}else{_LLA: _LLB:
 goto _LL7;}_LL7:;}
# 331
res=({struct Cyc_List_List*_tmp2D=_cycalloc(sizeof(*_tmp2D));({struct _tuple19*_tmp699=({struct _tuple19*_tmp2C=_cycalloc(sizeof(*_tmp2C));_tmp2C->f1=v,_tmp2C->f2=i;_tmp2C;});_tmp2D->hd=_tmp699;}),_tmp2D->tl=res;_tmp2D;});goto _LL0;}}else{if(((struct _tuple8*)_tmp1F)->f1 != 0)goto _LL5;else{goto _LL5;}}}else{if(((struct _tuple8*)_tmp1F)->f1 != 0)goto _LL5;else{goto _LL5;}}}else{if(((struct _tuple8*)_tmp1F)->f1 != 0)goto _LL5;else{goto _LL5;}}}else{if(((struct _tuple8*)_tmp1F)->f1 != 0){if(((struct Cyc_Absyn_RgnHandleCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)((struct _tuple8*)_tmp1F)->f3)->f1)->tag == 3U){if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)((struct _tuple8*)_tmp1F)->f3)->f2 != 0){if(((struct Cyc_Absyn_Evar_Absyn_Type_struct*)((struct Cyc_List_List*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)((struct _tuple8*)_tmp1F)->f3)->f2)->hd)->tag == 1U){if(((struct Cyc_List_List*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)((struct _tuple8*)_tmp1F)->f3)->f2)->tl == 0){_LL3: _tmp20=*_tmp1F->f1;_tmp21=(void**)&((struct Cyc_Absyn_Evar_Absyn_Type_struct*)(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp1F->f3)->f2)->hd)->f2;_LL4: {struct _fat_ptr v=_tmp20;void**z=_tmp21;
# 335
struct _fat_ptr*nm=({struct _fat_ptr*_tmp33=_cycalloc(sizeof(*_tmp33));({struct _fat_ptr _tmp69B=(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp32=({struct Cyc_String_pa_PrintArg_struct _tmp5E9;_tmp5E9.tag=0U,_tmp5E9.f1=(struct _fat_ptr)((struct _fat_ptr)v);_tmp5E9;});void*_tmp30[1U];_tmp30[0]=& _tmp32;({struct _fat_ptr _tmp69A=({const char*_tmp31="`%s";_tag_fat(_tmp31,sizeof(char),4U);});Cyc_aprintf(_tmp69A,_tag_fat(_tmp30,sizeof(void*),1U));});});*_tmp33=_tmp69B;});_tmp33;});
({void*_tmp69D=Cyc_Absyn_var_type(({struct Cyc_Absyn_Tvar*_tmp2F=_cycalloc(sizeof(*_tmp2F));_tmp2F->name=nm,_tmp2F->identity=- 1,({void*_tmp69C=(void*)({struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct*_tmp2E=_cycalloc(sizeof(*_tmp2E));_tmp2E->tag=0U,_tmp2E->f1=& Cyc_Tcutil_rk;_tmp2E;});_tmp2F->kind=_tmp69C;});_tmp2F;}));*z=_tmp69D;});
goto _LL0;}}else{goto _LL5;}}else{goto _LL5;}}else{goto _LL5;}}else{goto _LL5;}}else{goto _LL5;}}}else{if(((struct _tuple8*)_tmp1F)->f1 != 0)goto _LL5;else{_LL5: _LL6:
 goto _LL0;}}_LL0:;}
# 341
return res;}
# 201 "parse.y"
static struct Cyc_List_List*Cyc_Parse_get_aggrfield_tags(struct Cyc_List_List*x){
# 346 "parse.y"
struct Cyc_List_List*res=0;
for(0;x != 0;x=x->tl){
void*_stmttmp3=((struct Cyc_Absyn_Aggrfield*)x->hd)->type;void*_tmp34=_stmttmp3;void*_tmp35;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp34)->tag == 0U){if(((struct Cyc_Absyn_TagCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp34)->f1)->tag == 4U){if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp34)->f2 != 0){if(((struct Cyc_List_List*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp34)->f2)->tl == 0){_LL1: _tmp35=(void*)(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp34)->f2)->hd;_LL2: {void*i=_tmp35;
# 350
res=({struct Cyc_List_List*_tmp37=_cycalloc(sizeof(*_tmp37));({struct _tuple19*_tmp69E=({struct _tuple19*_tmp36=_cycalloc(sizeof(*_tmp36));_tmp36->f1=((struct Cyc_Absyn_Aggrfield*)x->hd)->name,_tmp36->f2=i;_tmp36;});_tmp37->hd=_tmp69E;}),_tmp37->tl=res;_tmp37;});goto _LL0;}}else{goto _LL3;}}else{goto _LL3;}}else{goto _LL3;}}else{_LL3: _LL4:
 goto _LL0;}_LL0:;}
# 354
return res;}
# 201 "parse.y"
static struct Cyc_Absyn_Exp*Cyc_Parse_substitute_tags_exp(struct Cyc_List_List*tags,struct Cyc_Absyn_Exp*e){
# 359 "parse.y"
{void*_stmttmp4=e->r;void*_tmp38=_stmttmp4;struct _fat_ptr*_tmp39;if(((struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct*)_tmp38)->tag == 1U){if(((struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct*)((struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct*)_tmp38)->f1)->tag == 0U){if(((((struct _tuple0*)((struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct*)((struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct*)_tmp38)->f1)->f1)->f1).Rel_n).tag == 1){if(((((struct _tuple0*)((struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct*)((struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct*)_tmp38)->f1)->f1)->f1).Rel_n).val == 0){_LL1: _tmp39=(((struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct*)((struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct*)_tmp38)->f1)->f1)->f2;_LL2: {struct _fat_ptr*y=_tmp39;
# 361
{struct Cyc_List_List*ts=tags;for(0;ts != 0;ts=ts->tl){
struct _tuple19*_stmttmp5=(struct _tuple19*)ts->hd;void*_tmp3B;struct _fat_ptr*_tmp3A;_LL6: _tmp3A=_stmttmp5->f1;_tmp3B=_stmttmp5->f2;_LL7: {struct _fat_ptr*x=_tmp3A;void*i=_tmp3B;
if(Cyc_strptrcmp(x,y)== 0)
return({void*_tmp6A0=(void*)({struct Cyc_Absyn_Valueof_e_Absyn_Raw_exp_struct*_tmp3C=_cycalloc(sizeof(*_tmp3C));_tmp3C->tag=40U,({void*_tmp69F=Cyc_Tcutil_copy_type(i);_tmp3C->f1=_tmp69F;});_tmp3C;});Cyc_Absyn_new_exp(_tmp6A0,e->loc);});}}}
# 366
goto _LL0;}}else{goto _LL3;}}else{goto _LL3;}}else{goto _LL3;}}else{_LL3: _LL4:
 goto _LL0;}_LL0:;}
# 369
return e;}
# 201 "parse.y"
static void*Cyc_Parse_substitute_tags(struct Cyc_List_List*tags,void*t){
# 375 "parse.y"
{void*_tmp3D=t;struct Cyc_Absyn_Exp*_tmp3E;void*_tmp3F;struct Cyc_Absyn_PtrLoc*_tmp46;void*_tmp45;void*_tmp44;void*_tmp43;void*_tmp42;struct Cyc_Absyn_Tqual _tmp41;void*_tmp40;unsigned _tmp4B;void*_tmp4A;struct Cyc_Absyn_Exp*_tmp49;struct Cyc_Absyn_Tqual _tmp48;void*_tmp47;switch(*((int*)_tmp3D)){case 4U: _LL1: _tmp47=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp3D)->f1).elt_type;_tmp48=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp3D)->f1).tq;_tmp49=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp3D)->f1).num_elts;_tmp4A=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp3D)->f1).zero_term;_tmp4B=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp3D)->f1).zt_loc;_LL2: {void*et=_tmp47;struct Cyc_Absyn_Tqual tq=_tmp48;struct Cyc_Absyn_Exp*nelts=_tmp49;void*zt=_tmp4A;unsigned ztloc=_tmp4B;
# 377
struct Cyc_Absyn_Exp*nelts2=nelts;
if(nelts != 0)
nelts2=Cyc_Parse_substitute_tags_exp(tags,nelts);{
# 378
void*et2=
# 381
Cyc_Parse_substitute_tags(tags,et);
if(nelts != nelts2 || et != et2)
return Cyc_Absyn_array_type(et2,tq,nelts2,zt,ztloc);
# 382
goto _LL0;}}case 3U: _LL3: _tmp40=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp3D)->f1).elt_type;_tmp41=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp3D)->f1).elt_tq;_tmp42=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp3D)->f1).ptr_atts).rgn;_tmp43=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp3D)->f1).ptr_atts).nullable;_tmp44=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp3D)->f1).ptr_atts).bounds;_tmp45=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp3D)->f1).ptr_atts).zero_term;_tmp46=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp3D)->f1).ptr_atts).ptrloc;_LL4: {void*et=_tmp40;struct Cyc_Absyn_Tqual tq=_tmp41;void*r=_tmp42;void*n=_tmp43;void*b=_tmp44;void*zt=_tmp45;struct Cyc_Absyn_PtrLoc*pl=_tmp46;
# 386
void*et2=Cyc_Parse_substitute_tags(tags,et);
void*b2=Cyc_Parse_substitute_tags(tags,b);
if(et2 != et || b2 != b)
return Cyc_Absyn_pointer_type(({struct Cyc_Absyn_PtrInfo _tmp5EA;_tmp5EA.elt_type=et2,_tmp5EA.elt_tq=tq,(_tmp5EA.ptr_atts).rgn=r,(_tmp5EA.ptr_atts).nullable=n,(_tmp5EA.ptr_atts).bounds=b2,(_tmp5EA.ptr_atts).zero_term=zt,(_tmp5EA.ptr_atts).ptrloc=pl;_tmp5EA;}));
# 388
goto _LL0;}case 0U: if(((struct Cyc_Absyn_ThinCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp3D)->f1)->tag == 13U){if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp3D)->f2 != 0){if(((struct Cyc_List_List*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp3D)->f2)->tl == 0){_LL5: _tmp3F=(void*)(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp3D)->f2)->hd;_LL6: {void*t=_tmp3F;
# 392
void*t2=Cyc_Parse_substitute_tags(tags,t);
if(t != t2)return Cyc_Absyn_thin_bounds_type(t2);goto _LL0;}}else{goto _LL9;}}else{goto _LL9;}}else{goto _LL9;}case 9U: _LL7: _tmp3E=((struct Cyc_Absyn_ValueofType_Absyn_Type_struct*)_tmp3D)->f1;_LL8: {struct Cyc_Absyn_Exp*e=_tmp3E;
# 396
struct Cyc_Absyn_Exp*e2=Cyc_Parse_substitute_tags_exp(tags,e);
if(e2 != e)return Cyc_Absyn_valueof_type(e2);goto _LL0;}default: _LL9: _LLA:
# 402
 goto _LL0;}_LL0:;}
# 404
return t;}
# 201 "parse.y"
static void Cyc_Parse_substitute_aggrfield_tags(struct Cyc_List_List*tags,struct Cyc_Absyn_Aggrfield*x){
# 410 "parse.y"
({void*_tmp6A1=Cyc_Parse_substitute_tags(tags,x->type);x->type=_tmp6A1;});}struct _tuple20{struct Cyc_Absyn_Tqual f1;void*f2;};
# 201 "parse.y"
static struct _tuple20*Cyc_Parse_get_tqual_typ(unsigned loc,struct _tuple8*t){
# 418 "parse.y"
return({struct _tuple20*_tmp4C=_cycalloc(sizeof(*_tmp4C));_tmp4C->f1=(*t).f2,_tmp4C->f2=(*t).f3;_tmp4C;});}
# 201 "parse.y"
static int Cyc_Parse_is_typeparam(void*tm){
# 422 "parse.y"
void*_tmp4D=tm;if(((struct Cyc_Absyn_TypeParams_mod_Absyn_Type_modifier_struct*)_tmp4D)->tag == 4U){_LL1: _LL2:
 return 1;}else{_LL3: _LL4:
 return 0;}_LL0:;}
# 201 "parse.y"
static void*Cyc_Parse_id2type(struct _fat_ptr s,void*k){
# 431 "parse.y"
if(({struct _fat_ptr _tmp6A2=(struct _fat_ptr)s;Cyc_zstrcmp(_tmp6A2,({const char*_tmp4E="`H";_tag_fat(_tmp4E,sizeof(char),3U);}));})== 0)
return Cyc_Absyn_heap_rgn_type;else{
if(({struct _fat_ptr _tmp6A3=(struct _fat_ptr)s;Cyc_zstrcmp(_tmp6A3,({const char*_tmp4F="`U";_tag_fat(_tmp4F,sizeof(char),3U);}));})== 0)
return Cyc_Absyn_unique_rgn_type;else{
if(({struct _fat_ptr _tmp6A4=(struct _fat_ptr)s;Cyc_zstrcmp(_tmp6A4,({const char*_tmp50="`RC";_tag_fat(_tmp50,sizeof(char),4U);}));})== 0)
return Cyc_Absyn_refcnt_rgn_type;else{
# 438
return Cyc_Absyn_var_type(({struct Cyc_Absyn_Tvar*_tmp52=_cycalloc(sizeof(*_tmp52));({struct _fat_ptr*_tmp6A5=({struct _fat_ptr*_tmp51=_cycalloc(sizeof(*_tmp51));*_tmp51=s;_tmp51;});_tmp52->name=_tmp6A5;}),_tmp52->identity=- 1,_tmp52->kind=k;_tmp52;}));}}}}
# 201 "parse.y"
static struct Cyc_Absyn_Tvar*Cyc_Parse_typ2tvar(unsigned loc,void*t){
# 446 "parse.y"
void*_tmp53=t;struct Cyc_Absyn_Tvar*_tmp54;if(((struct Cyc_Absyn_VarType_Absyn_Type_struct*)_tmp53)->tag == 2U){_LL1: _tmp54=((struct Cyc_Absyn_VarType_Absyn_Type_struct*)_tmp53)->f1;_LL2: {struct Cyc_Absyn_Tvar*pr=_tmp54;
return pr;}}else{_LL3: _LL4:
({void*_tmp55=0U;({int(*_tmp6A8)(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp57)(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Parse_parse_abort;_tmp57;});unsigned _tmp6A7=loc;struct _fat_ptr _tmp6A6=({const char*_tmp56="expecting a list of type variables, not types";_tag_fat(_tmp56,sizeof(char),46U);});_tmp6A8(_tmp6A7,_tmp6A6,_tag_fat(_tmp55,sizeof(void*),0U));});});}_LL0:;}
# 201 "parse.y"
static void Cyc_Parse_set_vartyp_kind(void*t,struct Cyc_Absyn_Kind*k,int leq){
# 454 "parse.y"
void*_stmttmp6=Cyc_Tcutil_compress(t);void*_tmp58=_stmttmp6;void**_tmp59;if(((struct Cyc_Absyn_VarType_Absyn_Type_struct*)_tmp58)->tag == 2U){_LL1: _tmp59=(void**)&(((struct Cyc_Absyn_VarType_Absyn_Type_struct*)_tmp58)->f1)->kind;_LL2: {void**cptr=_tmp59;
# 456
void*_stmttmp7=Cyc_Absyn_compress_kb(*cptr);void*_tmp5A=_stmttmp7;if(((struct Cyc_Absyn_Unknown_kb_Absyn_KindBound_struct*)_tmp5A)->tag == 1U){_LL6: _LL7:
# 458
 if(!leq)({void*_tmp6A9=Cyc_Tcutil_kind_to_bound(k);*cptr=_tmp6A9;});else{
({void*_tmp6AA=(void*)({struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*_tmp5B=_cycalloc(sizeof(*_tmp5B));_tmp5B->tag=2U,_tmp5B->f1=0,_tmp5B->f2=k;_tmp5B;});*cptr=_tmp6AA;});}
return;}else{_LL8: _LL9:
 return;}_LL5:;}}else{_LL3: _LL4:
# 463
 return;}_LL0:;}
# 201 "parse.y"
static struct Cyc_List_List*Cyc_Parse_oldstyle2newstyle(struct _RegionHandle*yy,struct Cyc_List_List*tms,struct Cyc_List_List*tds,unsigned loc){
# 474 "parse.y"
if(tds == 0)return tms;if(tms == 0)
# 479
return 0;{
# 474
void*_stmttmp8=(void*)tms->hd;void*_tmp5C=_stmttmp8;void*_tmp5D;if(((struct Cyc_Absyn_Function_mod_Absyn_Type_modifier_struct*)_tmp5C)->tag == 3U){_LL1: _tmp5D=(void*)((struct Cyc_Absyn_Function_mod_Absyn_Type_modifier_struct*)_tmp5C)->f1;_LL2: {void*args=_tmp5D;
# 484
if(tms->tl == 0 ||
 Cyc_Parse_is_typeparam((void*)((struct Cyc_List_List*)_check_null(tms->tl))->hd)&&((struct Cyc_List_List*)_check_null(tms->tl))->tl == 0){
# 487
void*_tmp5E=args;struct Cyc_List_List*_tmp5F;if(((struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct*)_tmp5E)->tag == 1U){_LL6: _LL7:
# 489
({void*_tmp60=0U;({unsigned _tmp6AC=loc;struct _fat_ptr _tmp6AB=({const char*_tmp61="function declaration with both new- and old-style parameter declarations; ignoring old-style";_tag_fat(_tmp61,sizeof(char),93U);});Cyc_Warn_warn(_tmp6AC,_tmp6AB,_tag_fat(_tmp60,sizeof(void*),0U));});});
# 491
return tms;}else{_LL8: _tmp5F=((struct Cyc_Absyn_NoTypes_Absyn_Funcparams_struct*)_tmp5E)->f1;_LL9: {struct Cyc_List_List*ids=_tmp5F;
# 493
if(({int _tmp6AD=Cyc_List_length(ids);_tmp6AD != Cyc_List_length(tds);}))
({void*_tmp62=0U;({int(*_tmp6B0)(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp64)(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Parse_parse_abort;_tmp64;});unsigned _tmp6AF=loc;struct _fat_ptr _tmp6AE=({const char*_tmp63="wrong number of parameter declarations in old-style function declaration";_tag_fat(_tmp63,sizeof(char),73U);});_tmp6B0(_tmp6AF,_tmp6AE,_tag_fat(_tmp62,sizeof(void*),0U));});});{
# 493
struct Cyc_List_List*rev_new_params=0;
# 498
for(0;ids != 0;ids=ids->tl){
struct Cyc_List_List*tds2=tds;
for(0;tds2 != 0;tds2=tds2->tl){
struct Cyc_Absyn_Decl*x=(struct Cyc_Absyn_Decl*)tds2->hd;
void*_stmttmp9=x->r;void*_tmp65=_stmttmp9;struct Cyc_Absyn_Vardecl*_tmp66;if(((struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct*)_tmp65)->tag == 0U){_LLB: _tmp66=((struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct*)_tmp65)->f1;_LLC: {struct Cyc_Absyn_Vardecl*vd=_tmp66;
# 504
if(Cyc_zstrptrcmp((*vd->name).f2,(struct _fat_ptr*)ids->hd)!= 0)
continue;
# 504
if(vd->initializer != 0)
# 507
({void*_tmp67=0U;({int(*_tmp6B3)(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp69)(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Parse_parse_abort;_tmp69;});unsigned _tmp6B2=x->loc;struct _fat_ptr _tmp6B1=({const char*_tmp68="initializer found in parameter declaration";_tag_fat(_tmp68,sizeof(char),43U);});_tmp6B3(_tmp6B2,_tmp6B1,_tag_fat(_tmp67,sizeof(void*),0U));});});
# 504
if(
# 508
Cyc_Absyn_is_qvar_qualified(vd->name))
({void*_tmp6A=0U;({int(*_tmp6B6)(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp6C)(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Parse_parse_abort;_tmp6C;});unsigned _tmp6B5=x->loc;struct _fat_ptr _tmp6B4=({const char*_tmp6B="namespaces forbidden in parameter declarations";_tag_fat(_tmp6B,sizeof(char),47U);});_tmp6B6(_tmp6B5,_tmp6B4,_tag_fat(_tmp6A,sizeof(void*),0U));});});
# 504
rev_new_params=({struct Cyc_List_List*_tmp6E=_cycalloc(sizeof(*_tmp6E));
# 511
({struct _tuple8*_tmp6B7=({struct _tuple8*_tmp6D=_cycalloc(sizeof(*_tmp6D));_tmp6D->f1=(*vd->name).f2,_tmp6D->f2=vd->tq,_tmp6D->f3=vd->type;_tmp6D;});_tmp6E->hd=_tmp6B7;}),_tmp6E->tl=rev_new_params;_tmp6E;});
# 513
goto L;}}else{_LLD: _LLE:
({void*_tmp6F=0U;({int(*_tmp6BA)(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp71)(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Parse_parse_abort;_tmp71;});unsigned _tmp6B9=x->loc;struct _fat_ptr _tmp6B8=({const char*_tmp70="nonvariable declaration in parameter type";_tag_fat(_tmp70,sizeof(char),42U);});_tmp6BA(_tmp6B9,_tmp6B8,_tag_fat(_tmp6F,sizeof(void*),0U));});});}_LLA:;}
# 517
L: if(tds2 == 0)
({struct Cyc_String_pa_PrintArg_struct _tmp75=({struct Cyc_String_pa_PrintArg_struct _tmp5EB;_tmp5EB.tag=0U,_tmp5EB.f1=(struct _fat_ptr)((struct _fat_ptr)*((struct _fat_ptr*)ids->hd));_tmp5EB;});void*_tmp72[1U];_tmp72[0]=& _tmp75;({int(*_tmp6BD)(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp74)(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Parse_parse_abort;_tmp74;});unsigned _tmp6BC=loc;struct _fat_ptr _tmp6BB=({const char*_tmp73="%s is not given a type";_tag_fat(_tmp73,sizeof(char),23U);});_tmp6BD(_tmp6BC,_tmp6BB,_tag_fat(_tmp72,sizeof(void*),1U));});});}
# 520
return({struct Cyc_List_List*_tmp78=_region_malloc(yy,sizeof(*_tmp78));
({void*_tmp6C0=(void*)({struct Cyc_Absyn_Function_mod_Absyn_Type_modifier_struct*_tmp77=_region_malloc(yy,sizeof(*_tmp77));_tmp77->tag=3U,({void*_tmp6BF=(void*)({struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct*_tmp76=_region_malloc(yy,sizeof(*_tmp76));_tmp76->tag=1U,({struct Cyc_List_List*_tmp6BE=Cyc_List_imp_rev(rev_new_params);_tmp76->f1=_tmp6BE;}),_tmp76->f2=0,_tmp76->f3=0,_tmp76->f4=0,_tmp76->f5=0,_tmp76->f6=0,_tmp76->f7=0,_tmp76->f8=0,_tmp76->f9=0;_tmp76;});_tmp77->f1=_tmp6BF;});_tmp77;});_tmp78->hd=_tmp6C0;}),_tmp78->tl=0;_tmp78;});}}}_LL5:;}
# 484
goto _LL4;}}else{_LL3: _LL4:
# 528
 return({struct Cyc_List_List*_tmp79=_region_malloc(yy,sizeof(*_tmp79));_tmp79->hd=(void*)tms->hd,({struct Cyc_List_List*_tmp6C1=Cyc_Parse_oldstyle2newstyle(yy,tms->tl,tds,loc);_tmp79->tl=_tmp6C1;});_tmp79;});}_LL0:;}}
# 201 "parse.y"
static struct Cyc_Absyn_Fndecl*Cyc_Parse_make_function(struct _RegionHandle*yy,struct Cyc_Parse_Declaration_spec*dso,struct Cyc_Parse_Declarator d,struct Cyc_List_List*tds,struct Cyc_Absyn_Stmt*body,unsigned loc){
# 539 "parse.y"
if(tds != 0)
d=({struct Cyc_Parse_Declarator _tmp5EC;_tmp5EC.id=d.id,_tmp5EC.varloc=d.varloc,({struct Cyc_List_List*_tmp6C2=Cyc_Parse_oldstyle2newstyle(yy,d.tms,tds,loc);_tmp5EC.tms=_tmp6C2;});_tmp5EC;});{
# 539
enum Cyc_Absyn_Scope sc=2U;
# 543
struct Cyc_Parse_Type_specifier tss=Cyc_Parse_empty_spec(loc);
struct Cyc_Absyn_Tqual tq=Cyc_Absyn_empty_tqual(0U);
int is_inline=0;
struct Cyc_List_List*atts=0;
# 548
if(dso != 0){
tss=dso->type_specs;
tq=dso->tq;
is_inline=dso->is_inline;
atts=dso->attributes;
# 554
if(dso->sc != 0){
enum Cyc_Parse_Storage_class _stmttmpA=*((enum Cyc_Parse_Storage_class*)_check_null(dso->sc));enum Cyc_Parse_Storage_class _tmp7A=_stmttmpA;switch(_tmp7A){case Cyc_Parse_Extern_sc: _LL1: _LL2:
 sc=3U;goto _LL0;case Cyc_Parse_Static_sc: _LL3: _LL4:
 sc=0U;goto _LL0;default: _LL5: _LL6:
({void*_tmp7B=0U;({unsigned _tmp6C4=loc;struct _fat_ptr _tmp6C3=({const char*_tmp7C="bad storage class on function";_tag_fat(_tmp7C,sizeof(char),30U);});Cyc_Warn_err(_tmp6C4,_tmp6C3,_tag_fat(_tmp7B,sizeof(void*),0U));});});goto _LL0;}_LL0:;}}{
# 548
void*t=
# 561
Cyc_Parse_collapse_type_specifiers(tss,loc);
struct _tuple14 _stmttmpB=Cyc_Parse_apply_tms(tq,t,atts,d.tms);struct Cyc_List_List*_tmp80;struct Cyc_List_List*_tmp7F;void*_tmp7E;struct Cyc_Absyn_Tqual _tmp7D;_LL8: _tmp7D=_stmttmpB.f1;_tmp7E=_stmttmpB.f2;_tmp7F=_stmttmpB.f3;_tmp80=_stmttmpB.f4;_LL9: {struct Cyc_Absyn_Tqual fn_tqual=_tmp7D;void*fn_type=_tmp7E;struct Cyc_List_List*x=_tmp7F;struct Cyc_List_List*out_atts=_tmp80;
# 566
if(x != 0)
# 569
({void*_tmp81=0U;({unsigned _tmp6C6=loc;struct _fat_ptr _tmp6C5=({const char*_tmp82="bad type params, ignoring";_tag_fat(_tmp82,sizeof(char),26U);});Cyc_Warn_warn(_tmp6C6,_tmp6C5,_tag_fat(_tmp81,sizeof(void*),0U));});});{
# 566
void*_tmp83=fn_type;struct Cyc_Absyn_FnInfo _tmp84;if(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp83)->tag == 5U){_LLB: _tmp84=((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp83)->f1;_LLC: {struct Cyc_Absyn_FnInfo i=_tmp84;
# 573
{struct Cyc_List_List*args2=i.args;for(0;args2 != 0;args2=args2->tl){
if((*((struct _tuple8*)args2->hd)).f1 == 0){
({void*_tmp85=0U;({unsigned _tmp6C8=loc;struct _fat_ptr _tmp6C7=({const char*_tmp86="missing argument variable in function prototype";_tag_fat(_tmp86,sizeof(char),48U);});Cyc_Warn_err(_tmp6C8,_tmp6C7,_tag_fat(_tmp85,sizeof(void*),0U));});});
({struct _fat_ptr*_tmp6CA=({struct _fat_ptr*_tmp88=_cycalloc(sizeof(*_tmp88));({struct _fat_ptr _tmp6C9=({const char*_tmp87="?";_tag_fat(_tmp87,sizeof(char),2U);});*_tmp88=_tmp6C9;});_tmp88;});(*((struct _tuple8*)args2->hd)).f1=_tmp6CA;});}}}
# 573
({struct Cyc_List_List*_tmp6CB=
# 580
Cyc_List_append(i.attributes,out_atts);
# 573
i.attributes=_tmp6CB;});
# 581
return({struct Cyc_Absyn_Fndecl*_tmp89=_cycalloc(sizeof(*_tmp89));_tmp89->sc=sc,_tmp89->is_inline=is_inline,_tmp89->name=d.id,_tmp89->body=body,_tmp89->i=i,_tmp89->cached_type=0,_tmp89->param_vardecls=0,_tmp89->fn_vardecl=0;_tmp89;});}}else{_LLD: _LLE:
# 584
({void*_tmp8A=0U;({int(*_tmp6CE)(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp8C)(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Parse_parse_abort;_tmp8C;});unsigned _tmp6CD=loc;struct _fat_ptr _tmp6CC=({const char*_tmp8B="declarator is not a function prototype";_tag_fat(_tmp8B,sizeof(char),39U);});_tmp6CE(_tmp6CD,_tmp6CC,_tag_fat(_tmp8A,sizeof(void*),0U));});});}_LLA:;}}}}}static char _tmp8D[76U]="at most one type may appear within a type specifier \n\t(missing ';' or ','?)";
# 201 "parse.y"
static struct _fat_ptr Cyc_Parse_msg1={_tmp8D,_tmp8D,_tmp8D + 76U};static char _tmp8E[87U]="const or volatile may appear only once within a type specifier \n\t(missing ';' or ','?)";
# 590 "parse.y"
static struct _fat_ptr Cyc_Parse_msg2={_tmp8E,_tmp8E,_tmp8E + 87U};static char _tmp8F[74U]="type specifier includes more than one declaration \n\t(missing ';' or ','?)";
# 592
static struct _fat_ptr Cyc_Parse_msg3={_tmp8F,_tmp8F,_tmp8F + 74U};static char _tmp90[84U]="sign specifier may appear only once within a type specifier \n\t(missing ';' or ','?)";
# 594
static struct _fat_ptr Cyc_Parse_msg4={_tmp90,_tmp90,_tmp90 + 84U};
# 601
static struct Cyc_Parse_Type_specifier Cyc_Parse_combine_specifiers(unsigned loc,struct Cyc_Parse_Type_specifier s1,struct Cyc_Parse_Type_specifier s2){
# 604
if(s1.Signed_spec && s2.Signed_spec)
({void*_tmp91=0U;({unsigned _tmp6D0=loc;struct _fat_ptr _tmp6CF=Cyc_Parse_msg4;Cyc_Warn_warn(_tmp6D0,_tmp6CF,_tag_fat(_tmp91,sizeof(void*),0U));});});
# 604
s1.Signed_spec |=s2.Signed_spec;
# 607
if(s1.Unsigned_spec && s2.Unsigned_spec)
({void*_tmp92=0U;({unsigned _tmp6D2=loc;struct _fat_ptr _tmp6D1=Cyc_Parse_msg4;Cyc_Warn_warn(_tmp6D2,_tmp6D1,_tag_fat(_tmp92,sizeof(void*),0U));});});
# 607
s1.Unsigned_spec |=s2.Unsigned_spec;
# 610
if(s1.Short_spec && s2.Short_spec)
({void*_tmp93=0U;({unsigned _tmp6D4=loc;struct _fat_ptr _tmp6D3=Cyc_Parse_msg4;Cyc_Warn_warn(_tmp6D4,_tmp6D3,_tag_fat(_tmp93,sizeof(void*),0U));});});
# 610
s1.Short_spec |=s2.Short_spec;
# 613
if((s1.Long_Long_spec && s2.Long_Long_spec ||
 s1.Long_Long_spec && s2.Long_spec)||
 s2.Long_Long_spec && s1.Long_spec)
({void*_tmp94=0U;({unsigned _tmp6D6=loc;struct _fat_ptr _tmp6D5=Cyc_Parse_msg4;Cyc_Warn_warn(_tmp6D6,_tmp6D5,_tag_fat(_tmp94,sizeof(void*),0U));});});
# 613
s1.Long_Long_spec=
# 618
(s1.Long_Long_spec || s2.Long_Long_spec)|| s1.Long_spec && s2.Long_spec;
s1.Long_spec=!s1.Long_Long_spec &&(s1.Long_spec || s2.Long_spec);
if(s1.Valid_type_spec && s2.Valid_type_spec)
({void*_tmp95=0U;({unsigned _tmp6D8=loc;struct _fat_ptr _tmp6D7=Cyc_Parse_msg1;Cyc_Warn_err(_tmp6D8,_tmp6D7,_tag_fat(_tmp95,sizeof(void*),0U));});});else{
if(s2.Valid_type_spec){
s1.Type_spec=s2.Type_spec;
s1.Valid_type_spec=1;}}
# 620
return s1;}
# 601
static void*Cyc_Parse_collapse_type_specifiers(struct Cyc_Parse_Type_specifier ts,unsigned loc){
# 635
int seen_type=ts.Valid_type_spec;
int seen_sign=ts.Signed_spec || ts.Unsigned_spec;
int seen_size=(ts.Short_spec || ts.Long_spec)|| ts.Long_Long_spec;
void*t=seen_type?ts.Type_spec: Cyc_Absyn_void_type;
enum Cyc_Absyn_Size_of sz=2U;
enum Cyc_Absyn_Sign sgn=0U;
# 642
if(ts.Signed_spec && ts.Unsigned_spec)
({void*_tmp96=0U;({unsigned _tmp6DA=loc;struct _fat_ptr _tmp6D9=Cyc_Parse_msg4;Cyc_Warn_err(_tmp6DA,_tmp6D9,_tag_fat(_tmp96,sizeof(void*),0U));});});
# 642
if(ts.Unsigned_spec)
# 644
sgn=1U;
# 642
if(
# 645
ts.Short_spec &&(ts.Long_spec || ts.Long_Long_spec)||
 ts.Long_spec && ts.Long_Long_spec)
({void*_tmp97=0U;({unsigned _tmp6DC=loc;struct _fat_ptr _tmp6DB=Cyc_Parse_msg4;Cyc_Warn_err(_tmp6DC,_tmp6DB,_tag_fat(_tmp97,sizeof(void*),0U));});});
# 642
if(ts.Short_spec)
# 648
sz=1U;
# 642
if(ts.Long_spec)
# 649
sz=3U;
# 642
if(ts.Long_Long_spec)
# 650
sz=4U;
# 642
if(!seen_type){
# 655
if(!seen_sign && !seen_size)
({void*_tmp98=0U;({unsigned _tmp6DE=loc;struct _fat_ptr _tmp6DD=({const char*_tmp99="missing type within specifier";_tag_fat(_tmp99,sizeof(char),30U);});Cyc_Warn_warn(_tmp6DE,_tmp6DD,_tag_fat(_tmp98,sizeof(void*),0U));});});
# 655
t=
# 657
Cyc_Absyn_int_type(sgn,sz);}else{
# 659
if(seen_sign){
void*_tmp9A=t;enum Cyc_Absyn_Size_of _tmp9C;enum Cyc_Absyn_Sign _tmp9B;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp9A)->tag == 0U){if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp9A)->f1)->tag == 1U){_LL1: _tmp9B=((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp9A)->f1)->f1;_tmp9C=((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp9A)->f1)->f2;_LL2: {enum Cyc_Absyn_Sign sgn2=_tmp9B;enum Cyc_Absyn_Size_of sz2=_tmp9C;
# 662
if((int)sgn2 != (int)sgn)
t=Cyc_Absyn_int_type(sgn,sz2);
# 662
goto _LL0;}}else{goto _LL3;}}else{_LL3: _LL4:
# 665
({void*_tmp9D=0U;({unsigned _tmp6E0=loc;struct _fat_ptr _tmp6DF=({const char*_tmp9E="sign specification on non-integral type";_tag_fat(_tmp9E,sizeof(char),40U);});Cyc_Warn_err(_tmp6E0,_tmp6DF,_tag_fat(_tmp9D,sizeof(void*),0U));});});goto _LL0;}_LL0:;}
# 659
if(seen_size){
# 668
void*_tmp9F=t;enum Cyc_Absyn_Size_of _tmpA1;enum Cyc_Absyn_Sign _tmpA0;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp9F)->tag == 0U)switch(*((int*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp9F)->f1)){case 1U: _LL6: _tmpA0=((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp9F)->f1)->f1;_tmpA1=((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp9F)->f1)->f2;_LL7: {enum Cyc_Absyn_Sign sgn2=_tmpA0;enum Cyc_Absyn_Size_of sz2=_tmpA1;
# 670
if((int)sz2 != (int)sz)
t=Cyc_Absyn_int_type(sgn2,sz);
# 670
goto _LL5;}case 2U: _LL8: _LL9:
# 674
 t=Cyc_Absyn_long_double_type;goto _LL5;default: goto _LLA;}else{_LLA: _LLB:
({void*_tmpA2=0U;({unsigned _tmp6E2=loc;struct _fat_ptr _tmp6E1=({const char*_tmpA3="size qualifier on non-integral type";_tag_fat(_tmpA3,sizeof(char),36U);});Cyc_Warn_err(_tmp6E2,_tmp6E1,_tag_fat(_tmpA2,sizeof(void*),0U));});});goto _LL5;}_LL5:;}}
# 678
return t;}
# 601
static struct Cyc_List_List*Cyc_Parse_apply_tmss(struct _RegionHandle*r,struct Cyc_Absyn_Tqual tq,void*t,struct _tuple11*ds,struct Cyc_List_List*shared_atts){
# 685
if(ds == 0)return 0;{struct Cyc_Parse_Declarator d=ds->hd;
# 687
struct _tuple0*q=d.id;
unsigned varloc=d.varloc;
struct _tuple14 _stmttmpC=Cyc_Parse_apply_tms(tq,t,shared_atts,d.tms);struct Cyc_List_List*_tmpA7;struct Cyc_List_List*_tmpA6;void*_tmpA5;struct Cyc_Absyn_Tqual _tmpA4;_LL1: _tmpA4=_stmttmpC.f1;_tmpA5=_stmttmpC.f2;_tmpA6=_stmttmpC.f3;_tmpA7=_stmttmpC.f4;_LL2: {struct Cyc_Absyn_Tqual tq2=_tmpA4;void*new_typ=_tmpA5;struct Cyc_List_List*tvs=_tmpA6;struct Cyc_List_List*atts=_tmpA7;
# 692
if(ds->tl == 0)
return({struct Cyc_List_List*_tmpA9=_region_malloc(r,sizeof(*_tmpA9));({struct _tuple16*_tmp6E3=({struct _tuple16*_tmpA8=_region_malloc(r,sizeof(*_tmpA8));_tmpA8->f1=varloc,_tmpA8->f2=q,_tmpA8->f3=tq2,_tmpA8->f4=new_typ,_tmpA8->f5=tvs,_tmpA8->f6=atts;_tmpA8;});_tmpA9->hd=_tmp6E3;}),_tmpA9->tl=0;_tmpA9;});else{
# 695
return({struct Cyc_List_List*_tmpAB=_region_malloc(r,sizeof(*_tmpAB));({struct _tuple16*_tmp6E9=({struct _tuple16*_tmpAA=_region_malloc(r,sizeof(*_tmpAA));_tmpAA->f1=varloc,_tmpAA->f2=q,_tmpAA->f3=tq2,_tmpAA->f4=new_typ,_tmpAA->f5=tvs,_tmpAA->f6=atts;_tmpAA;});_tmpAB->hd=_tmp6E9;}),({
struct Cyc_List_List*_tmp6E8=({struct _RegionHandle*_tmp6E7=r;struct Cyc_Absyn_Tqual _tmp6E6=tq;void*_tmp6E5=Cyc_Tcutil_copy_type(t);struct _tuple11*_tmp6E4=ds->tl;Cyc_Parse_apply_tmss(_tmp6E7,_tmp6E6,_tmp6E5,_tmp6E4,shared_atts);});_tmpAB->tl=_tmp6E8;});_tmpAB;});}}}}
# 601
static struct _tuple14 Cyc_Parse_apply_tms(struct Cyc_Absyn_Tqual tq,void*t,struct Cyc_List_List*atts,struct Cyc_List_List*tms){
# 702
if(tms == 0)return({struct _tuple14 _tmp5ED;_tmp5ED.f1=tq,_tmp5ED.f2=t,_tmp5ED.f3=0,_tmp5ED.f4=atts;_tmp5ED;});{void*_stmttmpD=(void*)tms->hd;void*_tmpAC=_stmttmpD;struct Cyc_List_List*_tmpAE;unsigned _tmpAD;struct Cyc_Absyn_Tqual _tmpB0;struct Cyc_Absyn_PtrAtts _tmpAF;unsigned _tmpB2;struct Cyc_List_List*_tmpB1;void*_tmpB3;unsigned _tmpB6;void*_tmpB5;struct Cyc_Absyn_Exp*_tmpB4;unsigned _tmpB8;void*_tmpB7;switch(*((int*)_tmpAC)){case 0U: _LL1: _tmpB7=(void*)((struct Cyc_Absyn_Carray_mod_Absyn_Type_modifier_struct*)_tmpAC)->f1;_tmpB8=((struct Cyc_Absyn_Carray_mod_Absyn_Type_modifier_struct*)_tmpAC)->f2;_LL2: {void*zeroterm=_tmpB7;unsigned ztloc=_tmpB8;
# 705
return({struct Cyc_Absyn_Tqual _tmp6EC=Cyc_Absyn_empty_tqual(0U);void*_tmp6EB=
Cyc_Absyn_array_type(t,tq,0,zeroterm,ztloc);
# 705
struct Cyc_List_List*_tmp6EA=atts;Cyc_Parse_apply_tms(_tmp6EC,_tmp6EB,_tmp6EA,tms->tl);});}case 1U: _LL3: _tmpB4=((struct Cyc_Absyn_ConstArray_mod_Absyn_Type_modifier_struct*)_tmpAC)->f1;_tmpB5=(void*)((struct Cyc_Absyn_ConstArray_mod_Absyn_Type_modifier_struct*)_tmpAC)->f2;_tmpB6=((struct Cyc_Absyn_ConstArray_mod_Absyn_Type_modifier_struct*)_tmpAC)->f3;_LL4: {struct Cyc_Absyn_Exp*e=_tmpB4;void*zeroterm=_tmpB5;unsigned ztloc=_tmpB6;
# 708
return({struct Cyc_Absyn_Tqual _tmp6EF=Cyc_Absyn_empty_tqual(0U);void*_tmp6EE=
Cyc_Absyn_array_type(t,tq,e,zeroterm,ztloc);
# 708
struct Cyc_List_List*_tmp6ED=atts;Cyc_Parse_apply_tms(_tmp6EF,_tmp6EE,_tmp6ED,tms->tl);});}case 3U: _LL5: _tmpB3=(void*)((struct Cyc_Absyn_Function_mod_Absyn_Type_modifier_struct*)_tmpAC)->f1;_LL6: {void*args=_tmpB3;
# 711
void*_tmpB9=args;unsigned _tmpBA;struct Cyc_List_List*_tmpC3;struct Cyc_List_List*_tmpC2;struct Cyc_Absyn_Exp*_tmpC1;struct Cyc_Absyn_Exp*_tmpC0;struct Cyc_List_List*_tmpBF;void*_tmpBE;struct Cyc_Absyn_VarargInfo*_tmpBD;int _tmpBC;struct Cyc_List_List*_tmpBB;if(((struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct*)_tmpB9)->tag == 1U){_LLE: _tmpBB=((struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct*)_tmpB9)->f1;_tmpBC=((struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct*)_tmpB9)->f2;_tmpBD=((struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct*)_tmpB9)->f3;_tmpBE=(void*)((struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct*)_tmpB9)->f4;_tmpBF=((struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct*)_tmpB9)->f5;_tmpC0=((struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct*)_tmpB9)->f6;_tmpC1=((struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct*)_tmpB9)->f7;_tmpC2=((struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct*)_tmpB9)->f8;_tmpC3=((struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct*)_tmpB9)->f9;_LLF: {struct Cyc_List_List*args2=_tmpBB;int c_vararg=_tmpBC;struct Cyc_Absyn_VarargInfo*cyc_vararg=_tmpBD;void*eff=_tmpBE;struct Cyc_List_List*rgn_po=_tmpBF;struct Cyc_Absyn_Exp*req=_tmpC0;struct Cyc_Absyn_Exp*ens=_tmpC1;struct Cyc_List_List*ieff=_tmpC2;struct Cyc_List_List*oeff=_tmpC3;
# 713
struct Cyc_List_List*typvars=0;
# 715
struct Cyc_List_List*fn_atts=0;struct Cyc_List_List*new_atts=0;
{struct Cyc_List_List*as=atts;for(0;as != 0;as=as->tl){
if(Cyc_Absyn_fntype_att((void*)as->hd))
fn_atts=({struct Cyc_List_List*_tmpC4=_cycalloc(sizeof(*_tmpC4));_tmpC4->hd=(void*)as->hd,_tmpC4->tl=fn_atts;_tmpC4;});else{
# 720
new_atts=({struct Cyc_List_List*_tmpC5=_cycalloc(sizeof(*_tmpC5));_tmpC5->hd=(void*)as->hd,_tmpC5->tl=new_atts;_tmpC5;});}}}
# 723
if(tms->tl != 0){
void*_stmttmpE=(void*)((struct Cyc_List_List*)_check_null(tms->tl))->hd;void*_tmpC6=_stmttmpE;struct Cyc_List_List*_tmpC7;if(((struct Cyc_Absyn_TypeParams_mod_Absyn_Type_modifier_struct*)_tmpC6)->tag == 4U){_LL13: _tmpC7=((struct Cyc_Absyn_TypeParams_mod_Absyn_Type_modifier_struct*)_tmpC6)->f1;_LL14: {struct Cyc_List_List*ts=_tmpC7;
# 726
typvars=ts;
tms=tms->tl;
goto _LL12;}}else{_LL15: _LL16:
# 730
 goto _LL12;}_LL12:;}
# 723
if(
# 734
((((!c_vararg && cyc_vararg == 0)&& args2 != 0)&& args2->tl == 0)&&(*((struct _tuple8*)args2->hd)).f1 == 0)&&(*((struct _tuple8*)args2->hd)).f3 == Cyc_Absyn_void_type)
# 739
args2=0;{
# 723
struct Cyc_List_List*tags=
# 742
Cyc_Parse_get_arg_tags(args2);
# 744
if(tags != 0)
t=Cyc_Parse_substitute_tags(tags,t);
# 744
t=
# 746
Cyc_Parse_array2ptr(t,0);
# 749
{struct Cyc_List_List*a=args2;for(0;a != 0;a=a->tl){
struct _tuple8*_stmttmpF=(struct _tuple8*)a->hd;void**_tmpCA;struct Cyc_Absyn_Tqual _tmpC9;struct _fat_ptr*_tmpC8;_LL18: _tmpC8=_stmttmpF->f1;_tmpC9=_stmttmpF->f2;_tmpCA=(void**)& _stmttmpF->f3;_LL19: {struct _fat_ptr*vopt=_tmpC8;struct Cyc_Absyn_Tqual tq=_tmpC9;void**t=_tmpCA;
if(tags != 0)
({void*_tmp6F0=Cyc_Parse_substitute_tags(tags,*t);*t=_tmp6F0;});
# 751
({void*_tmp6F1=
# 753
Cyc_Parse_array2ptr(*t,1);
# 751
*t=_tmp6F1;});}}}
# 761
return({struct Cyc_Absyn_Tqual _tmp6F4=Cyc_Absyn_empty_tqual(tq.loc);void*_tmp6F3=
Cyc_Absyn_function_type(typvars,eff,tq,t,args2,c_vararg,cyc_vararg,rgn_po,fn_atts,req,ens,ieff,oeff);
# 761
struct Cyc_List_List*_tmp6F2=new_atts;Cyc_Parse_apply_tms(_tmp6F4,_tmp6F3,_tmp6F2,((struct Cyc_List_List*)_check_null(tms))->tl);});}}}else{_LL10: _tmpBA=((struct Cyc_Absyn_NoTypes_Absyn_Funcparams_struct*)_tmpB9)->f2;_LL11: {unsigned loc=_tmpBA;
# 770
({void*_tmpCB=0U;({int(*_tmp6F7)(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmpCD)(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Parse_parse_abort;_tmpCD;});unsigned _tmp6F6=loc;struct _fat_ptr _tmp6F5=({const char*_tmpCC="function declaration without parameter types";_tag_fat(_tmpCC,sizeof(char),45U);});_tmp6F7(_tmp6F6,_tmp6F5,_tag_fat(_tmpCB,sizeof(void*),0U));});});}}_LLD:;}case 4U: _LL7: _tmpB1=((struct Cyc_Absyn_TypeParams_mod_Absyn_Type_modifier_struct*)_tmpAC)->f1;_tmpB2=((struct Cyc_Absyn_TypeParams_mod_Absyn_Type_modifier_struct*)_tmpAC)->f2;_LL8: {struct Cyc_List_List*ts=_tmpB1;unsigned loc=_tmpB2;
# 777
if(tms->tl == 0)
return({struct _tuple14 _tmp5EE;_tmp5EE.f1=tq,_tmp5EE.f2=t,_tmp5EE.f3=ts,_tmp5EE.f4=atts;_tmp5EE;});
# 777
({void*_tmpCE=0U;({int(*_tmp6FA)(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmpD0)(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Parse_parse_abort;_tmpD0;});unsigned _tmp6F9=loc;struct _fat_ptr _tmp6F8=({const char*_tmpCF="type parameters must appear before function arguments in declarator";_tag_fat(_tmpCF,sizeof(char),68U);});_tmp6FA(_tmp6F9,_tmp6F8,_tag_fat(_tmpCE,sizeof(void*),0U));});});}case 2U: _LL9: _tmpAF=((struct Cyc_Absyn_Pointer_mod_Absyn_Type_modifier_struct*)_tmpAC)->f1;_tmpB0=((struct Cyc_Absyn_Pointer_mod_Absyn_Type_modifier_struct*)_tmpAC)->f2;_LLA: {struct Cyc_Absyn_PtrAtts ptratts=_tmpAF;struct Cyc_Absyn_Tqual tq2=_tmpB0;
# 785
return({struct Cyc_Absyn_Tqual _tmp6FD=tq2;void*_tmp6FC=Cyc_Absyn_pointer_type(({struct Cyc_Absyn_PtrInfo _tmp5EF;_tmp5EF.elt_type=t,_tmp5EF.elt_tq=tq,_tmp5EF.ptr_atts=ptratts;_tmp5EF;}));struct Cyc_List_List*_tmp6FB=atts;Cyc_Parse_apply_tms(_tmp6FD,_tmp6FC,_tmp6FB,tms->tl);});}default: _LLB: _tmpAD=((struct Cyc_Absyn_Attributes_mod_Absyn_Type_modifier_struct*)_tmpAC)->f1;_tmpAE=((struct Cyc_Absyn_Attributes_mod_Absyn_Type_modifier_struct*)_tmpAC)->f2;_LLC: {unsigned loc=_tmpAD;struct Cyc_List_List*atts2=_tmpAE;
# 790
return({struct Cyc_Absyn_Tqual _tmp700=tq;void*_tmp6FF=t;struct Cyc_List_List*_tmp6FE=Cyc_List_append(atts,atts2);Cyc_Parse_apply_tms(_tmp700,_tmp6FF,_tmp6FE,tms->tl);});}}_LL0:;}}
# 601
void*Cyc_Parse_speclist2typ(struct Cyc_Parse_Type_specifier tss,unsigned loc){
# 797
return Cyc_Parse_collapse_type_specifiers(tss,loc);}
# 601
static struct Cyc_Absyn_Decl*Cyc_Parse_v_typ_to_typedef(unsigned loc,struct _tuple16*t){
# 806
struct Cyc_List_List*_tmpD6;struct Cyc_List_List*_tmpD5;void*_tmpD4;struct Cyc_Absyn_Tqual _tmpD3;struct _tuple0*_tmpD2;unsigned _tmpD1;_LL1: _tmpD1=t->f1;_tmpD2=t->f2;_tmpD3=t->f3;_tmpD4=t->f4;_tmpD5=t->f5;_tmpD6=t->f6;_LL2: {unsigned varloc=_tmpD1;struct _tuple0*x=_tmpD2;struct Cyc_Absyn_Tqual tq=_tmpD3;void*typ=_tmpD4;struct Cyc_List_List*tvs=_tmpD5;struct Cyc_List_List*atts=_tmpD6;
# 808
Cyc_Lex_register_typedef(x);{
# 810
struct Cyc_Core_Opt*kind;
void*type;
{void*_tmpD7=typ;struct Cyc_Core_Opt*_tmpD8;if(((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmpD7)->tag == 1U){_LL4: _tmpD8=((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmpD7)->f1;_LL5: {struct Cyc_Core_Opt*kopt=_tmpD8;
# 814
type=0;
if(kopt == 0)kind=& Cyc_Tcutil_bko;else{
kind=kopt;}
goto _LL3;}}else{_LL6: _LL7:
 kind=0;type=typ;goto _LL3;}_LL3:;}
# 820
return({void*_tmp702=(void*)({struct Cyc_Absyn_Typedef_d_Absyn_Raw_decl_struct*_tmpDA=_cycalloc(sizeof(*_tmpDA));_tmpDA->tag=8U,({struct Cyc_Absyn_Typedefdecl*_tmp701=({struct Cyc_Absyn_Typedefdecl*_tmpD9=_cycalloc(sizeof(*_tmpD9));_tmpD9->name=x,_tmpD9->tvs=tvs,_tmpD9->kind=kind,_tmpD9->defn=type,_tmpD9->atts=atts,_tmpD9->tq=tq,_tmpD9->extern_c=0;_tmpD9;});_tmpDA->f1=_tmp701;});_tmpDA;});Cyc_Absyn_new_decl(_tmp702,loc);});}}}
# 601
static struct Cyc_Absyn_Stmt*Cyc_Parse_flatten_decl(struct Cyc_Absyn_Decl*d,struct Cyc_Absyn_Stmt*s){
# 828
return({void*_tmp703=(void*)({struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct*_tmpDB=_cycalloc(sizeof(*_tmpDB));_tmpDB->tag=12U,_tmpDB->f1=d,_tmpDB->f2=s;_tmpDB;});Cyc_Absyn_new_stmt(_tmp703,d->loc);});}
# 601
static struct Cyc_Absyn_Stmt*Cyc_Parse_flatten_declarations(struct Cyc_List_List*ds,struct Cyc_Absyn_Stmt*s){
# 834
return({struct Cyc_Absyn_Stmt*(*_tmp705)(struct Cyc_Absyn_Stmt*(*f)(struct Cyc_Absyn_Decl*,struct Cyc_Absyn_Stmt*),struct Cyc_List_List*x,struct Cyc_Absyn_Stmt*accum)=({struct Cyc_Absyn_Stmt*(*_tmpDC)(struct Cyc_Absyn_Stmt*(*f)(struct Cyc_Absyn_Decl*,struct Cyc_Absyn_Stmt*),struct Cyc_List_List*x,struct Cyc_Absyn_Stmt*accum)=(struct Cyc_Absyn_Stmt*(*)(struct Cyc_Absyn_Stmt*(*f)(struct Cyc_Absyn_Decl*,struct Cyc_Absyn_Stmt*),struct Cyc_List_List*x,struct Cyc_Absyn_Stmt*accum))Cyc_List_fold_right;_tmpDC;});struct Cyc_List_List*_tmp704=ds;_tmp705(Cyc_Parse_flatten_decl,_tmp704,s);});}
# 601
static struct Cyc_List_List*Cyc_Parse_make_declarations(struct Cyc_Parse_Declaration_spec ds,struct _tuple13*ids,unsigned tqual_loc,unsigned loc){
# 846
struct _RegionHandle _tmpDD=_new_region("mkrgn");struct _RegionHandle*mkrgn=& _tmpDD;_push_region(mkrgn);
{struct Cyc_List_List*_tmpE0;struct Cyc_Parse_Type_specifier _tmpDF;struct Cyc_Absyn_Tqual _tmpDE;_LL1: _tmpDE=ds.tq;_tmpDF=ds.type_specs;_tmpE0=ds.attributes;_LL2: {struct Cyc_Absyn_Tqual tq=_tmpDE;struct Cyc_Parse_Type_specifier tss=_tmpDF;struct Cyc_List_List*atts=_tmpE0;
if(tq.loc == (unsigned)0)tq.loc=tqual_loc;if(ds.is_inline)
# 850
({void*_tmpE1=0U;({unsigned _tmp707=loc;struct _fat_ptr _tmp706=({const char*_tmpE2="inline qualifier on non-function definition";_tag_fat(_tmpE2,sizeof(char),44U);});Cyc_Warn_warn(_tmp707,_tmp706,_tag_fat(_tmpE1,sizeof(void*),0U));});});{
# 848
enum Cyc_Absyn_Scope s=2U;
# 853
int istypedef=0;
if(ds.sc != 0){
enum Cyc_Parse_Storage_class _stmttmp10=*ds.sc;enum Cyc_Parse_Storage_class _tmpE3=_stmttmp10;switch(_tmpE3){case Cyc_Parse_Typedef_sc: _LL4: _LL5:
 istypedef=1;goto _LL3;case Cyc_Parse_Extern_sc: _LL6: _LL7:
 s=3U;goto _LL3;case Cyc_Parse_ExternC_sc: _LL8: _LL9:
 s=4U;goto _LL3;case Cyc_Parse_Static_sc: _LLA: _LLB:
 s=0U;goto _LL3;case Cyc_Parse_Auto_sc: _LLC: _LLD:
 s=2U;goto _LL3;case Cyc_Parse_Register_sc: _LLE: _LLF:
 if(Cyc_Parse_no_register)s=2U;else{s=5U;}goto _LL3;case Cyc_Parse_Abstract_sc: _LL10: _LL11:
 goto _LL13;default: _LL12: _LL13:
 s=1U;goto _LL3;}_LL3:;}{
# 854
struct _tuple11*declarators=0;
# 870
struct Cyc_List_List*exprs=0;
Cyc_Parse_decl_split(mkrgn,ids,& declarators,& exprs);{
# 873
int exps_empty=1;
{struct Cyc_List_List*es=exprs;for(0;es != 0;es=es->tl){
if((struct Cyc_Absyn_Exp*)es->hd != 0){
exps_empty=0;
break;}}}{
# 874
void*base_type=
# 881
Cyc_Parse_collapse_type_specifiers(tss,loc);
if(declarators == 0){
# 885
void*_tmpE4=base_type;struct Cyc_List_List*_tmpE5;struct _tuple0*_tmpE6;struct Cyc_List_List*_tmpE9;int _tmpE8;struct _tuple0*_tmpE7;struct Cyc_Absyn_Datatypedecl**_tmpEA;struct Cyc_List_List*_tmpED;struct _tuple0*_tmpEC;enum Cyc_Absyn_AggrKind _tmpEB;struct Cyc_Absyn_Datatypedecl*_tmpEE;struct Cyc_Absyn_Enumdecl*_tmpEF;struct Cyc_Absyn_Aggrdecl*_tmpF0;switch(*((int*)_tmpE4)){case 10U: switch(*((int*)((struct Cyc_Absyn_TypeDecl*)((struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct*)_tmpE4)->f1)->r)){case 0U: _LL15: _tmpF0=((struct Cyc_Absyn_Aggr_td_Absyn_Raw_typedecl_struct*)(((struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct*)_tmpE4)->f1)->r)->f1;_LL16: {struct Cyc_Absyn_Aggrdecl*ad=_tmpF0;
# 887
({struct Cyc_List_List*_tmp708=Cyc_List_append(ad->attributes,atts);ad->attributes=_tmp708;});
ad->sc=s;{
struct Cyc_List_List*_tmpF3=({struct Cyc_List_List*_tmpF2=_cycalloc(sizeof(*_tmpF2));({struct Cyc_Absyn_Decl*_tmp70A=({void*_tmp709=(void*)({struct Cyc_Absyn_Aggr_d_Absyn_Raw_decl_struct*_tmpF1=_cycalloc(sizeof(*_tmpF1));_tmpF1->tag=5U,_tmpF1->f1=ad;_tmpF1;});Cyc_Absyn_new_decl(_tmp709,loc);});_tmpF2->hd=_tmp70A;}),_tmpF2->tl=0;_tmpF2;});_npop_handler(0U);return _tmpF3;}}case 1U: _LL17: _tmpEF=((struct Cyc_Absyn_Enum_td_Absyn_Raw_typedecl_struct*)(((struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct*)_tmpE4)->f1)->r)->f1;_LL18: {struct Cyc_Absyn_Enumdecl*ed=_tmpEF;
# 891
if(atts != 0)({void*_tmpF4=0U;({unsigned _tmp70C=loc;struct _fat_ptr _tmp70B=({const char*_tmpF5="attributes on enum not supported";_tag_fat(_tmpF5,sizeof(char),33U);});Cyc_Warn_err(_tmp70C,_tmp70B,_tag_fat(_tmpF4,sizeof(void*),0U));});});ed->sc=s;{
# 893
struct Cyc_List_List*_tmpF8=({struct Cyc_List_List*_tmpF7=_cycalloc(sizeof(*_tmpF7));({struct Cyc_Absyn_Decl*_tmp70E=({void*_tmp70D=(void*)({struct Cyc_Absyn_Enum_d_Absyn_Raw_decl_struct*_tmpF6=_cycalloc(sizeof(*_tmpF6));_tmpF6->tag=7U,_tmpF6->f1=ed;_tmpF6;});Cyc_Absyn_new_decl(_tmp70D,loc);});_tmpF7->hd=_tmp70E;}),_tmpF7->tl=0;_tmpF7;});_npop_handler(0U);return _tmpF8;}}default: _LL19: _tmpEE=((struct Cyc_Absyn_Datatype_td_Absyn_Raw_typedecl_struct*)(((struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct*)_tmpE4)->f1)->r)->f1;_LL1A: {struct Cyc_Absyn_Datatypedecl*dd=_tmpEE;
# 895
if(atts != 0)({void*_tmpF9=0U;({unsigned _tmp710=loc;struct _fat_ptr _tmp70F=({const char*_tmpFA="attributes on datatypes not supported";_tag_fat(_tmpFA,sizeof(char),38U);});Cyc_Warn_err(_tmp710,_tmp70F,_tag_fat(_tmpF9,sizeof(void*),0U));});});dd->sc=s;{
# 897
struct Cyc_List_List*_tmpFD=({struct Cyc_List_List*_tmpFC=_cycalloc(sizeof(*_tmpFC));({struct Cyc_Absyn_Decl*_tmp712=({void*_tmp711=(void*)({struct Cyc_Absyn_Datatype_d_Absyn_Raw_decl_struct*_tmpFB=_cycalloc(sizeof(*_tmpFB));_tmpFB->tag=6U,_tmpFB->f1=dd;_tmpFB;});Cyc_Absyn_new_decl(_tmp711,loc);});_tmpFC->hd=_tmp712;}),_tmpFC->tl=0;_tmpFC;});_npop_handler(0U);return _tmpFD;}}}case 0U: switch(*((int*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmpE4)->f1)){case 20U: if(((((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmpE4)->f1)->f1).UnknownAggr).tag == 1){_LL1B: _tmpEB=(((((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmpE4)->f1)->f1).UnknownAggr).val).f1;_tmpEC=(((((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmpE4)->f1)->f1).UnknownAggr).val).f2;_tmpED=((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmpE4)->f2;_LL1C: {enum Cyc_Absyn_AggrKind k=_tmpEB;struct _tuple0*n=_tmpEC;struct Cyc_List_List*ts=_tmpED;
# 899
struct Cyc_List_List*ts2=({struct Cyc_List_List*(*_tmp714)(struct Cyc_Absyn_Tvar*(*f)(unsigned,void*),unsigned env,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp104)(struct Cyc_Absyn_Tvar*(*f)(unsigned,void*),unsigned env,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_Absyn_Tvar*(*f)(unsigned,void*),unsigned env,struct Cyc_List_List*x))Cyc_List_map_c;_tmp104;});unsigned _tmp713=loc;_tmp714(Cyc_Parse_typ2tvar,_tmp713,ts);});
struct Cyc_Absyn_Aggrdecl*ad=({struct Cyc_Absyn_Aggrdecl*_tmp103=_cycalloc(sizeof(*_tmp103));_tmp103->kind=k,_tmp103->sc=s,_tmp103->name=n,_tmp103->tvs=ts2,_tmp103->impl=0,_tmp103->attributes=0,_tmp103->expected_mem_kind=0;_tmp103;});
if(atts != 0)({void*_tmpFE=0U;({unsigned _tmp716=loc;struct _fat_ptr _tmp715=({const char*_tmpFF="bad attributes on type declaration";_tag_fat(_tmpFF,sizeof(char),35U);});Cyc_Warn_err(_tmp716,_tmp715,_tag_fat(_tmpFE,sizeof(void*),0U));});});{struct Cyc_List_List*_tmp102=({struct Cyc_List_List*_tmp101=_cycalloc(sizeof(*_tmp101));
({struct Cyc_Absyn_Decl*_tmp718=({void*_tmp717=(void*)({struct Cyc_Absyn_Aggr_d_Absyn_Raw_decl_struct*_tmp100=_cycalloc(sizeof(*_tmp100));_tmp100->tag=5U,_tmp100->f1=ad;_tmp100;});Cyc_Absyn_new_decl(_tmp717,loc);});_tmp101->hd=_tmp718;}),_tmp101->tl=0;_tmp101;});_npop_handler(0U);return _tmp102;}}}else{goto _LL25;}case 18U: if(((((struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmpE4)->f1)->f1).KnownDatatype).tag == 2){_LL1D: _tmpEA=((((struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmpE4)->f1)->f1).KnownDatatype).val;_LL1E: {struct Cyc_Absyn_Datatypedecl**tudp=_tmpEA;
# 904
if(atts != 0)({void*_tmp105=0U;({unsigned _tmp71A=loc;struct _fat_ptr _tmp719=({const char*_tmp106="bad attributes on datatype";_tag_fat(_tmp106,sizeof(char),27U);});Cyc_Warn_err(_tmp71A,_tmp719,_tag_fat(_tmp105,sizeof(void*),0U));});});{struct Cyc_List_List*_tmp109=({struct Cyc_List_List*_tmp108=_cycalloc(sizeof(*_tmp108));
({struct Cyc_Absyn_Decl*_tmp71C=({void*_tmp71B=(void*)({struct Cyc_Absyn_Datatype_d_Absyn_Raw_decl_struct*_tmp107=_cycalloc(sizeof(*_tmp107));_tmp107->tag=6U,_tmp107->f1=*tudp;_tmp107;});Cyc_Absyn_new_decl(_tmp71B,loc);});_tmp108->hd=_tmp71C;}),_tmp108->tl=0;_tmp108;});_npop_handler(0U);return _tmp109;}}}else{_LL1F: _tmpE7=(((((struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmpE4)->f1)->f1).UnknownDatatype).val).name;_tmpE8=(((((struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmpE4)->f1)->f1).UnknownDatatype).val).is_extensible;_tmpE9=((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmpE4)->f2;_LL20: {struct _tuple0*n=_tmpE7;int isx=_tmpE8;struct Cyc_List_List*ts=_tmpE9;
# 907
struct Cyc_List_List*ts2=({struct Cyc_List_List*(*_tmp71E)(struct Cyc_Absyn_Tvar*(*f)(unsigned,void*),unsigned env,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp10E)(struct Cyc_Absyn_Tvar*(*f)(unsigned,void*),unsigned env,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_Absyn_Tvar*(*f)(unsigned,void*),unsigned env,struct Cyc_List_List*x))Cyc_List_map_c;_tmp10E;});unsigned _tmp71D=loc;_tmp71E(Cyc_Parse_typ2tvar,_tmp71D,ts);});
struct Cyc_Absyn_Decl*tud=Cyc_Absyn_datatype_decl(s,n,ts2,0,isx,loc);
if(atts != 0)({void*_tmp10A=0U;({unsigned _tmp720=loc;struct _fat_ptr _tmp71F=({const char*_tmp10B="bad attributes on datatype";_tag_fat(_tmp10B,sizeof(char),27U);});Cyc_Warn_err(_tmp720,_tmp71F,_tag_fat(_tmp10A,sizeof(void*),0U));});});{struct Cyc_List_List*_tmp10D=({struct Cyc_List_List*_tmp10C=_cycalloc(sizeof(*_tmp10C));
_tmp10C->hd=tud,_tmp10C->tl=0;_tmp10C;});_npop_handler(0U);return _tmp10D;}}}case 15U: _LL21: _tmpE6=((struct Cyc_Absyn_EnumCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmpE4)->f1)->f1;_LL22: {struct _tuple0*n=_tmpE6;
# 912
struct Cyc_Absyn_Enumdecl*ed=({struct Cyc_Absyn_Enumdecl*_tmp115=_cycalloc(sizeof(*_tmp115));_tmp115->sc=s,_tmp115->name=n,_tmp115->fields=0;_tmp115;});
if(atts != 0)({void*_tmp10F=0U;({unsigned _tmp722=loc;struct _fat_ptr _tmp721=({const char*_tmp110="bad attributes on enum";_tag_fat(_tmp110,sizeof(char),23U);});Cyc_Warn_err(_tmp722,_tmp721,_tag_fat(_tmp10F,sizeof(void*),0U));});});{struct Cyc_List_List*_tmp114=({struct Cyc_List_List*_tmp113=_cycalloc(sizeof(*_tmp113));
({struct Cyc_Absyn_Decl*_tmp724=({struct Cyc_Absyn_Decl*_tmp112=_cycalloc(sizeof(*_tmp112));({void*_tmp723=(void*)({struct Cyc_Absyn_Enum_d_Absyn_Raw_decl_struct*_tmp111=_cycalloc(sizeof(*_tmp111));_tmp111->tag=7U,_tmp111->f1=ed;_tmp111;});_tmp112->r=_tmp723;}),_tmp112->loc=loc;_tmp112;});_tmp113->hd=_tmp724;}),_tmp113->tl=0;_tmp113;});_npop_handler(0U);return _tmp114;}}case 16U: _LL23: _tmpE5=((struct Cyc_Absyn_AnonEnumCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmpE4)->f1)->f1;_LL24: {struct Cyc_List_List*fs=_tmpE5;
# 918
struct Cyc_Absyn_Enumdecl*ed=({struct Cyc_Absyn_Enumdecl*_tmp11D=_cycalloc(sizeof(*_tmp11D));_tmp11D->sc=s,({struct _tuple0*_tmp726=Cyc_Parse_gensym_enum();_tmp11D->name=_tmp726;}),({struct Cyc_Core_Opt*_tmp725=({struct Cyc_Core_Opt*_tmp11C=_cycalloc(sizeof(*_tmp11C));_tmp11C->v=fs;_tmp11C;});_tmp11D->fields=_tmp725;});_tmp11D;});
if(atts != 0)({void*_tmp116=0U;({unsigned _tmp728=loc;struct _fat_ptr _tmp727=({const char*_tmp117="bad attributes on enum";_tag_fat(_tmp117,sizeof(char),23U);});Cyc_Warn_err(_tmp728,_tmp727,_tag_fat(_tmp116,sizeof(void*),0U));});});{struct Cyc_List_List*_tmp11B=({struct Cyc_List_List*_tmp11A=_cycalloc(sizeof(*_tmp11A));
({struct Cyc_Absyn_Decl*_tmp72A=({struct Cyc_Absyn_Decl*_tmp119=_cycalloc(sizeof(*_tmp119));({void*_tmp729=(void*)({struct Cyc_Absyn_Enum_d_Absyn_Raw_decl_struct*_tmp118=_cycalloc(sizeof(*_tmp118));_tmp118->tag=7U,_tmp118->f1=ed;_tmp118;});_tmp119->r=_tmp729;}),_tmp119->loc=loc;_tmp119;});_tmp11A->hd=_tmp72A;}),_tmp11A->tl=0;_tmp11A;});_npop_handler(0U);return _tmp11B;}}default: goto _LL25;}default: _LL25: _LL26:
({void*_tmp11E=0U;({unsigned _tmp72C=loc;struct _fat_ptr _tmp72B=({const char*_tmp11F="missing declarator";_tag_fat(_tmp11F,sizeof(char),19U);});Cyc_Warn_err(_tmp72C,_tmp72B,_tag_fat(_tmp11E,sizeof(void*),0U));});});{struct Cyc_List_List*_tmp120=0;_npop_handler(0U);return _tmp120;}}_LL14:;}else{
# 925
struct Cyc_List_List*fields=Cyc_Parse_apply_tmss(mkrgn,tq,base_type,declarators,atts);
if(istypedef){
# 930
if(!exps_empty)
({void*_tmp121=0U;({unsigned _tmp72E=loc;struct _fat_ptr _tmp72D=({const char*_tmp122="initializer in typedef declaration";_tag_fat(_tmp122,sizeof(char),35U);});Cyc_Warn_err(_tmp72E,_tmp72D,_tag_fat(_tmp121,sizeof(void*),0U));});});{
# 930
struct Cyc_List_List*decls=({
# 932
struct Cyc_List_List*(*_tmp730)(struct Cyc_Absyn_Decl*(*f)(unsigned,struct _tuple16*),unsigned env,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp124)(struct Cyc_Absyn_Decl*(*f)(unsigned,struct _tuple16*),unsigned env,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_Absyn_Decl*(*f)(unsigned,struct _tuple16*),unsigned env,struct Cyc_List_List*x))Cyc_List_map_c;_tmp124;});unsigned _tmp72F=loc;_tmp730(Cyc_Parse_v_typ_to_typedef,_tmp72F,fields);});
struct Cyc_List_List*_tmp123=decls;_npop_handler(0U);return _tmp123;}}else{
# 936
struct Cyc_List_List*decls=0;
{struct Cyc_List_List*ds=fields;for(0;ds != 0;(ds=ds->tl,exprs=((struct Cyc_List_List*)_check_null(exprs))->tl)){
struct _tuple16*_stmttmp11=(struct _tuple16*)ds->hd;struct Cyc_List_List*_tmp12A;struct Cyc_List_List*_tmp129;void*_tmp128;struct Cyc_Absyn_Tqual _tmp127;struct _tuple0*_tmp126;unsigned _tmp125;_LL28: _tmp125=_stmttmp11->f1;_tmp126=_stmttmp11->f2;_tmp127=_stmttmp11->f3;_tmp128=_stmttmp11->f4;_tmp129=_stmttmp11->f5;_tmp12A=_stmttmp11->f6;_LL29: {unsigned varloc=_tmp125;struct _tuple0*x=_tmp126;struct Cyc_Absyn_Tqual tq2=_tmp127;void*t2=_tmp128;struct Cyc_List_List*tvs2=_tmp129;struct Cyc_List_List*atts2=_tmp12A;
if(tvs2 != 0)
({void*_tmp12B=0U;({unsigned _tmp732=loc;struct _fat_ptr _tmp731=({const char*_tmp12C="bad type params, ignoring";_tag_fat(_tmp12C,sizeof(char),26U);});Cyc_Warn_warn(_tmp732,_tmp731,_tag_fat(_tmp12B,sizeof(void*),0U));});});
# 939
if(exprs == 0)
# 942
({void*_tmp12D=0U;({int(*_tmp735)(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp12F)(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Parse_parse_abort;_tmp12F;});unsigned _tmp734=loc;struct _fat_ptr _tmp733=({const char*_tmp12E="unexpected NULL in parse!";_tag_fat(_tmp12E,sizeof(char),26U);});_tmp735(_tmp734,_tmp733,_tag_fat(_tmp12D,sizeof(void*),0U));});});{
# 939
struct Cyc_Absyn_Exp*eopt=(struct Cyc_Absyn_Exp*)((struct Cyc_List_List*)_check_null(exprs))->hd;
# 944
struct Cyc_Absyn_Vardecl*vd=Cyc_Absyn_new_vardecl(varloc,x,t2,eopt);
vd->tq=tq2;
vd->sc=s;
vd->attributes=atts2;{
struct Cyc_Absyn_Decl*d=({struct Cyc_Absyn_Decl*_tmp132=_cycalloc(sizeof(*_tmp132));({void*_tmp736=(void*)({struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct*_tmp131=_cycalloc(sizeof(*_tmp131));_tmp131->tag=0U,_tmp131->f1=vd;_tmp131;});_tmp132->r=_tmp736;}),_tmp132->loc=loc;_tmp132;});
decls=({struct Cyc_List_List*_tmp130=_cycalloc(sizeof(*_tmp130));_tmp130->hd=d,_tmp130->tl=decls;_tmp130;});}}}}}{
# 951
struct Cyc_List_List*_tmp133=Cyc_List_imp_rev(decls);_npop_handler(0U);return _tmp133;}}}}}}}}}
# 847
;_pop_region(mkrgn);}
# 601
static struct Cyc_Absyn_Kind*Cyc_Parse_id_to_kind(struct _fat_ptr s,unsigned loc){
# 958
if(Cyc_strlen((struct _fat_ptr)s)== (unsigned long)1 || Cyc_strlen((struct _fat_ptr)s)== (unsigned long)2){
char _stmttmp12=*((const char*)_check_fat_subscript(s,sizeof(char),0));char _tmp134=_stmttmp12;switch(_tmp134){case 65U: _LL1: _LL2:
 return& Cyc_Tcutil_ak;case 77U: _LL3: _LL4:
 return& Cyc_Tcutil_mk;case 66U: _LL5: _LL6:
 return& Cyc_Tcutil_bk;case 82U: _LL7: _LL8:
 return& Cyc_Tcutil_rk;case 69U: _LL9: _LLA:
 return& Cyc_Tcutil_ek;case 73U: _LLB: _LLC:
 return& Cyc_Tcutil_ik;case 85U: _LLD: _LLE:
# 967
{char _stmttmp13=*((const char*)_check_fat_subscript(s,sizeof(char),1));char _tmp135=_stmttmp13;switch(_tmp135){case 82U: _LL14: _LL15:
 return& Cyc_Tcutil_urk;case 65U: _LL16: _LL17:
 return& Cyc_Tcutil_uak;case 77U: _LL18: _LL19:
 return& Cyc_Tcutil_umk;case 66U: _LL1A: _LL1B:
 return& Cyc_Tcutil_ubk;default: _LL1C: _LL1D:
 goto _LL13;}_LL13:;}
# 974
goto _LL0;case 84U: _LLF: _LL10:
# 976
{char _stmttmp14=*((const char*)_check_fat_subscript(s,sizeof(char),1));char _tmp136=_stmttmp14;switch(_tmp136){case 82U: _LL1F: _LL20:
 return& Cyc_Tcutil_trk;case 65U: _LL21: _LL22:
 return& Cyc_Tcutil_tak;case 77U: _LL23: _LL24:
 return& Cyc_Tcutil_tmk;case 66U: _LL25: _LL26:
 return& Cyc_Tcutil_tbk;default: _LL27: _LL28:
 goto _LL1E;}_LL1E:;}
# 983
goto _LL0;default: _LL11: _LL12:
 goto _LL0;}_LL0:;}
# 958
({struct Cyc_String_pa_PrintArg_struct _tmp139=({struct Cyc_String_pa_PrintArg_struct _tmp5F1;_tmp5F1.tag=0U,_tmp5F1.f1=(struct _fat_ptr)((struct _fat_ptr)s);_tmp5F1;});struct Cyc_Int_pa_PrintArg_struct _tmp13A=({struct Cyc_Int_pa_PrintArg_struct _tmp5F0;_tmp5F0.tag=1U,({
# 986
unsigned long _tmp737=(unsigned long)((int)Cyc_strlen((struct _fat_ptr)s));_tmp5F0.f1=_tmp737;});_tmp5F0;});void*_tmp137[2U];_tmp137[0]=& _tmp139,_tmp137[1]=& _tmp13A;({unsigned _tmp739=loc;struct _fat_ptr _tmp738=({const char*_tmp138="bad kind: %s; strlen=%d";_tag_fat(_tmp138,sizeof(char),24U);});Cyc_Warn_err(_tmp739,_tmp738,_tag_fat(_tmp137,sizeof(void*),2U));});});
return& Cyc_Tcutil_bk;}
# 601
static struct _fat_ptr Cyc_Parse_exp2string(unsigned loc,struct Cyc_Absyn_Exp*e){
# 992
void*_stmttmp15=e->r;void*_tmp13B=_stmttmp15;struct _fat_ptr _tmp13C;if(((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmp13B)->tag == 0U){if(((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmp13B)->f1).String_c).tag == 8){_LL1: _tmp13C=((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmp13B)->f1).String_c).val;_LL2: {struct _fat_ptr s=_tmp13C;
return s;}}else{goto _LL3;}}else{_LL3: _LL4:
# 995
({void*_tmp13D=0U;({unsigned _tmp73B=loc;struct _fat_ptr _tmp73A=({const char*_tmp13E="expecting string constant";_tag_fat(_tmp13E,sizeof(char),26U);});Cyc_Warn_err(_tmp73B,_tmp73A,_tag_fat(_tmp13D,sizeof(void*),0U));});});
return _tag_fat(0,0,0);}_LL0:;}
# 601
static unsigned Cyc_Parse_cnst2uint(unsigned loc,union Cyc_Absyn_Cnst x){
# 1002
union Cyc_Absyn_Cnst _tmp13F=x;long long _tmp140;char _tmp141;int _tmp142;switch((_tmp13F.LongLong_c).tag){case 5U: _LL1: _tmp142=((_tmp13F.Int_c).val).f2;_LL2: {int i=_tmp142;
return(unsigned)i;}case 2U: _LL3: _tmp141=((_tmp13F.Char_c).val).f2;_LL4: {char c=_tmp141;
return(unsigned)c;}case 6U: _LL5: _tmp140=((_tmp13F.LongLong_c).val).f2;_LL6: {long long x=_tmp140;
# 1006
unsigned long long y=(unsigned long long)x;
if(y > (unsigned long long)-1)
({void*_tmp143=0U;({unsigned _tmp73D=loc;struct _fat_ptr _tmp73C=({const char*_tmp144="integer constant too large";_tag_fat(_tmp144,sizeof(char),27U);});Cyc_Warn_err(_tmp73D,_tmp73C,_tag_fat(_tmp143,sizeof(void*),0U));});});
# 1007
return(unsigned)x;}default: _LL7: _LL8:
# 1011
({struct Cyc_String_pa_PrintArg_struct _tmp147=({struct Cyc_String_pa_PrintArg_struct _tmp5F2;_tmp5F2.tag=0U,({struct _fat_ptr _tmp73E=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_cnst2string(x));_tmp5F2.f1=_tmp73E;});_tmp5F2;});void*_tmp145[1U];_tmp145[0]=& _tmp147;({unsigned _tmp740=loc;struct _fat_ptr _tmp73F=({const char*_tmp146="expected integer constant but found %s";_tag_fat(_tmp146,sizeof(char),39U);});Cyc_Warn_err(_tmp740,_tmp73F,_tag_fat(_tmp145,sizeof(void*),1U));});});
return 0U;}_LL0:;}
# 601
static struct Cyc_Absyn_Exp*Cyc_Parse_pat2exp(struct Cyc_Absyn_Pat*p){
# 1018
void*_stmttmp16=p->r;void*_tmp148=_stmttmp16;struct Cyc_Absyn_Exp*_tmp149;struct Cyc_List_List*_tmp14B;struct _tuple0*_tmp14A;int _tmp14D;struct _fat_ptr _tmp14C;char _tmp14E;int _tmp150;enum Cyc_Absyn_Sign _tmp14F;struct Cyc_Absyn_Pat*_tmp151;struct Cyc_Absyn_Vardecl*_tmp152;struct _tuple0*_tmp153;switch(*((int*)_tmp148)){case 15U: _LL1: _tmp153=((struct Cyc_Absyn_UnknownId_p_Absyn_Raw_pat_struct*)_tmp148)->f1;_LL2: {struct _tuple0*x=_tmp153;
return Cyc_Absyn_unknownid_exp(x,p->loc);}case 3U: if(((struct Cyc_Absyn_Wild_p_Absyn_Raw_pat_struct*)((struct Cyc_Absyn_Pat*)((struct Cyc_Absyn_Reference_p_Absyn_Raw_pat_struct*)_tmp148)->f2)->r)->tag == 0U){_LL3: _tmp152=((struct Cyc_Absyn_Reference_p_Absyn_Raw_pat_struct*)_tmp148)->f1;_LL4: {struct Cyc_Absyn_Vardecl*vd=_tmp152;
# 1021
return({struct Cyc_Absyn_Exp*_tmp741=Cyc_Absyn_unknownid_exp(vd->name,p->loc);Cyc_Absyn_deref_exp(_tmp741,p->loc);});}}else{goto _LL13;}case 6U: _LL5: _tmp151=((struct Cyc_Absyn_Pointer_p_Absyn_Raw_pat_struct*)_tmp148)->f1;_LL6: {struct Cyc_Absyn_Pat*p2=_tmp151;
return({struct Cyc_Absyn_Exp*_tmp742=Cyc_Parse_pat2exp(p2);Cyc_Absyn_address_exp(_tmp742,p->loc);});}case 9U: _LL7: _LL8:
 return Cyc_Absyn_null_exp(p->loc);case 10U: _LL9: _tmp14F=((struct Cyc_Absyn_Int_p_Absyn_Raw_pat_struct*)_tmp148)->f1;_tmp150=((struct Cyc_Absyn_Int_p_Absyn_Raw_pat_struct*)_tmp148)->f2;_LLA: {enum Cyc_Absyn_Sign s=_tmp14F;int i=_tmp150;
return Cyc_Absyn_int_exp(s,i,p->loc);}case 11U: _LLB: _tmp14E=((struct Cyc_Absyn_Char_p_Absyn_Raw_pat_struct*)_tmp148)->f1;_LLC: {char c=_tmp14E;
return Cyc_Absyn_char_exp(c,p->loc);}case 12U: _LLD: _tmp14C=((struct Cyc_Absyn_Float_p_Absyn_Raw_pat_struct*)_tmp148)->f1;_tmp14D=((struct Cyc_Absyn_Float_p_Absyn_Raw_pat_struct*)_tmp148)->f2;_LLE: {struct _fat_ptr s=_tmp14C;int i=_tmp14D;
return Cyc_Absyn_float_exp(s,i,p->loc);}case 16U: if(((struct Cyc_Absyn_UnknownCall_p_Absyn_Raw_pat_struct*)_tmp148)->f3 == 0){_LLF: _tmp14A=((struct Cyc_Absyn_UnknownCall_p_Absyn_Raw_pat_struct*)_tmp148)->f1;_tmp14B=((struct Cyc_Absyn_UnknownCall_p_Absyn_Raw_pat_struct*)_tmp148)->f2;_LL10: {struct _tuple0*x=_tmp14A;struct Cyc_List_List*ps=_tmp14B;
# 1028
struct Cyc_Absyn_Exp*e1=Cyc_Absyn_unknownid_exp(x,p->loc);
struct Cyc_List_List*es=({struct Cyc_List_List*(*_tmp743)(struct Cyc_Absyn_Exp*(*f)(struct Cyc_Absyn_Pat*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp154)(struct Cyc_Absyn_Exp*(*f)(struct Cyc_Absyn_Pat*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_Absyn_Exp*(*f)(struct Cyc_Absyn_Pat*),struct Cyc_List_List*x))Cyc_List_map;_tmp154;});_tmp743(Cyc_Parse_pat2exp,ps);});
return Cyc_Absyn_unknowncall_exp(e1,es,p->loc);}}else{goto _LL13;}case 17U: _LL11: _tmp149=((struct Cyc_Absyn_Exp_p_Absyn_Raw_pat_struct*)_tmp148)->f1;_LL12: {struct Cyc_Absyn_Exp*e=_tmp149;
return e;}default: _LL13: _LL14:
# 1033
({void*_tmp155=0U;({unsigned _tmp745=p->loc;struct _fat_ptr _tmp744=({const char*_tmp156="cannot mix patterns and expressions in case";_tag_fat(_tmp156,sizeof(char),44U);});Cyc_Warn_err(_tmp745,_tmp744,_tag_fat(_tmp155,sizeof(void*),0U));});});
return Cyc_Absyn_null_exp(p->loc);}_LL0:;}struct _union_YYSTYPE_Int_tok{int tag;union Cyc_Absyn_Cnst val;};struct _union_YYSTYPE_Char_tok{int tag;char val;};struct _union_YYSTYPE_String_tok{int tag;struct _fat_ptr val;};struct _union_YYSTYPE_Stringopt_tok{int tag;struct Cyc_Core_Opt*val;};struct _union_YYSTYPE_QualId_tok{int tag;struct _tuple0*val;};struct _tuple21{int f1;struct _fat_ptr f2;};struct _union_YYSTYPE_Asm_tok{int tag;struct _tuple21 val;};struct _union_YYSTYPE_Exp_tok{int tag;struct Cyc_Absyn_Exp*val;};struct _union_YYSTYPE_Stmt_tok{int tag;struct Cyc_Absyn_Stmt*val;};struct _tuple22{unsigned f1;void*f2;void*f3;};struct _union_YYSTYPE_YY1{int tag;struct _tuple22*val;};struct _union_YYSTYPE_YY2{int tag;void*val;};struct _union_YYSTYPE_YY3{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY4{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY5{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY6{int tag;enum Cyc_Absyn_Primop val;};struct _union_YYSTYPE_YY7{int tag;struct Cyc_Core_Opt*val;};struct _union_YYSTYPE_YY8{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY9{int tag;struct Cyc_Absyn_Pat*val;};struct _tuple23{struct Cyc_List_List*f1;int f2;};struct _union_YYSTYPE_YY10{int tag;struct _tuple23*val;};struct _union_YYSTYPE_YY11{int tag;struct Cyc_List_List*val;};struct _tuple24{struct Cyc_List_List*f1;struct Cyc_Absyn_Pat*f2;};struct _union_YYSTYPE_YY12{int tag;struct _tuple24*val;};struct _union_YYSTYPE_YY13{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY14{int tag;struct _tuple23*val;};struct _union_YYSTYPE_YY15{int tag;struct Cyc_Absyn_Fndecl*val;};struct _union_YYSTYPE_YY16{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY17{int tag;struct Cyc_Parse_Declaration_spec val;};struct _union_YYSTYPE_YY18{int tag;struct _tuple12 val;};struct _union_YYSTYPE_YY19{int tag;struct _tuple13*val;};struct _union_YYSTYPE_YY20{int tag;enum Cyc_Parse_Storage_class*val;};struct _union_YYSTYPE_YY21{int tag;struct Cyc_Parse_Type_specifier val;};struct _union_YYSTYPE_YY22{int tag;enum Cyc_Absyn_AggrKind val;};struct _union_YYSTYPE_YY23{int tag;struct Cyc_Absyn_Tqual val;};struct _union_YYSTYPE_YY24{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY25{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY26{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY27{int tag;struct Cyc_Parse_Declarator val;};struct _tuple25{struct Cyc_Parse_Declarator f1;struct Cyc_Absyn_Exp*f2;struct Cyc_Absyn_Exp*f3;};struct _union_YYSTYPE_YY28{int tag;struct _tuple25*val;};struct _union_YYSTYPE_YY29{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY30{int tag;struct Cyc_Parse_Abstractdeclarator val;};struct _union_YYSTYPE_YY31{int tag;int val;};struct _union_YYSTYPE_YY32{int tag;enum Cyc_Absyn_Scope val;};struct _union_YYSTYPE_YY33{int tag;struct Cyc_Absyn_Datatypefield*val;};struct _union_YYSTYPE_YY34{int tag;struct Cyc_List_List*val;};struct _tuple26{struct Cyc_Absyn_Tqual f1;struct Cyc_Parse_Type_specifier f2;struct Cyc_List_List*f3;};struct _union_YYSTYPE_YY35{int tag;struct _tuple26 val;};struct _union_YYSTYPE_YY36{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY37{int tag;struct _tuple8*val;};struct _union_YYSTYPE_YY38{int tag;struct Cyc_List_List*val;};struct _tuple27{struct Cyc_List_List*f1;int f2;struct Cyc_Absyn_VarargInfo*f3;void*f4;struct Cyc_List_List*f5;};struct _union_YYSTYPE_YY39{int tag;struct _tuple27*val;};struct _union_YYSTYPE_YY40{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY41{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY42{int tag;void*val;};struct _union_YYSTYPE_YY43{int tag;struct Cyc_Absyn_Kind*val;};struct _union_YYSTYPE_YY44{int tag;void*val;};struct _union_YYSTYPE_YY45{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY46{int tag;void*val;};struct _union_YYSTYPE_YY47{int tag;struct Cyc_Absyn_Enumfield*val;};struct _union_YYSTYPE_YY48{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY49{int tag;void*val;};struct _union_YYSTYPE_YY50{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY51{int tag;void*val;};struct _union_YYSTYPE_YY52{int tag;struct Cyc_List_List*val;};struct _tuple28{struct Cyc_List_List*f1;unsigned f2;};struct _union_YYSTYPE_YY53{int tag;struct _tuple28*val;};struct _union_YYSTYPE_YY54{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY55{int tag;void*val;};struct _union_YYSTYPE_YY56{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY57{int tag;struct Cyc_Absyn_Exp*val;};struct _union_YYSTYPE_YY58{int tag;void*val;};struct _tuple29{struct Cyc_List_List*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;};struct _union_YYSTYPE_YY59{int tag;struct _tuple29*val;};struct _tuple30{struct Cyc_List_List*f1;struct Cyc_List_List*f2;};struct _union_YYSTYPE_YY60{int tag;struct _tuple30*val;};struct _union_YYSTYPE_YY61{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YY62{int tag;struct Cyc_List_List*val;};struct _tuple31{struct _fat_ptr f1;struct Cyc_Absyn_Exp*f2;};struct _union_YYSTYPE_YY63{int tag;struct _tuple31*val;};struct _union_YYSTYPE_YY64{int tag;struct Cyc_List_List*val;};struct _union_YYSTYPE_YYINITIALSVAL{int tag;int val;};union Cyc_YYSTYPE{struct _union_YYSTYPE_Int_tok Int_tok;struct _union_YYSTYPE_Char_tok Char_tok;struct _union_YYSTYPE_String_tok String_tok;struct _union_YYSTYPE_Stringopt_tok Stringopt_tok;struct _union_YYSTYPE_QualId_tok QualId_tok;struct _union_YYSTYPE_Asm_tok Asm_tok;struct _union_YYSTYPE_Exp_tok Exp_tok;struct _union_YYSTYPE_Stmt_tok Stmt_tok;struct _union_YYSTYPE_YY1 YY1;struct _union_YYSTYPE_YY2 YY2;struct _union_YYSTYPE_YY3 YY3;struct _union_YYSTYPE_YY4 YY4;struct _union_YYSTYPE_YY5 YY5;struct _union_YYSTYPE_YY6 YY6;struct _union_YYSTYPE_YY7 YY7;struct _union_YYSTYPE_YY8 YY8;struct _union_YYSTYPE_YY9 YY9;struct _union_YYSTYPE_YY10 YY10;struct _union_YYSTYPE_YY11 YY11;struct _union_YYSTYPE_YY12 YY12;struct _union_YYSTYPE_YY13 YY13;struct _union_YYSTYPE_YY14 YY14;struct _union_YYSTYPE_YY15 YY15;struct _union_YYSTYPE_YY16 YY16;struct _union_YYSTYPE_YY17 YY17;struct _union_YYSTYPE_YY18 YY18;struct _union_YYSTYPE_YY19 YY19;struct _union_YYSTYPE_YY20 YY20;struct _union_YYSTYPE_YY21 YY21;struct _union_YYSTYPE_YY22 YY22;struct _union_YYSTYPE_YY23 YY23;struct _union_YYSTYPE_YY24 YY24;struct _union_YYSTYPE_YY25 YY25;struct _union_YYSTYPE_YY26 YY26;struct _union_YYSTYPE_YY27 YY27;struct _union_YYSTYPE_YY28 YY28;struct _union_YYSTYPE_YY29 YY29;struct _union_YYSTYPE_YY30 YY30;struct _union_YYSTYPE_YY31 YY31;struct _union_YYSTYPE_YY32 YY32;struct _union_YYSTYPE_YY33 YY33;struct _union_YYSTYPE_YY34 YY34;struct _union_YYSTYPE_YY35 YY35;struct _union_YYSTYPE_YY36 YY36;struct _union_YYSTYPE_YY37 YY37;struct _union_YYSTYPE_YY38 YY38;struct _union_YYSTYPE_YY39 YY39;struct _union_YYSTYPE_YY40 YY40;struct _union_YYSTYPE_YY41 YY41;struct _union_YYSTYPE_YY42 YY42;struct _union_YYSTYPE_YY43 YY43;struct _union_YYSTYPE_YY44 YY44;struct _union_YYSTYPE_YY45 YY45;struct _union_YYSTYPE_YY46 YY46;struct _union_YYSTYPE_YY47 YY47;struct _union_YYSTYPE_YY48 YY48;struct _union_YYSTYPE_YY49 YY49;struct _union_YYSTYPE_YY50 YY50;struct _union_YYSTYPE_YY51 YY51;struct _union_YYSTYPE_YY52 YY52;struct _union_YYSTYPE_YY53 YY53;struct _union_YYSTYPE_YY54 YY54;struct _union_YYSTYPE_YY55 YY55;struct _union_YYSTYPE_YY56 YY56;struct _union_YYSTYPE_YY57 YY57;struct _union_YYSTYPE_YY58 YY58;struct _union_YYSTYPE_YY59 YY59;struct _union_YYSTYPE_YY60 YY60;struct _union_YYSTYPE_YY61 YY61;struct _union_YYSTYPE_YY62 YY62;struct _union_YYSTYPE_YY63 YY63;struct _union_YYSTYPE_YY64 YY64;struct _union_YYSTYPE_YYINITIALSVAL YYINITIALSVAL;};
# 1119
static void Cyc_yythrowfail(struct _fat_ptr s){
(int)_throw((void*)({struct Cyc_Core_Failure_exn_struct*_tmp157=_cycalloc(sizeof(*_tmp157));_tmp157->tag=Cyc_Core_Failure,_tmp157->f1=s;_tmp157;}));}static char _tmp15A[7U]="cnst_t";
# 1119
static union Cyc_Absyn_Cnst Cyc_yyget_Int_tok(union Cyc_YYSTYPE*yy1){
# 1091 "parse.y"
static struct _fat_ptr s={_tmp15A,_tmp15A,_tmp15A + 7U};
union Cyc_YYSTYPE*_tmp158=yy1;union Cyc_Absyn_Cnst _tmp159;if((((union Cyc_YYSTYPE*)_tmp158)->Int_tok).tag == 1){_LL1: _tmp159=(_tmp158->Int_tok).val;_LL2: {union Cyc_Absyn_Cnst yy=_tmp159;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_Int_tok(union Cyc_Absyn_Cnst yy1){
# 1097 "parse.y"
return({union Cyc_YYSTYPE _tmp5F3;(_tmp5F3.Int_tok).tag=1U,(_tmp5F3.Int_tok).val=yy1;_tmp5F3;});}static char _tmp15D[5U]="char";
# 1119 "parse.y"
static char Cyc_yyget_Char_tok(union Cyc_YYSTYPE*yy1){
# 1092 "parse.y"
static struct _fat_ptr s={_tmp15D,_tmp15D,_tmp15D + 5U};
union Cyc_YYSTYPE*_tmp15B=yy1;char _tmp15C;if((((union Cyc_YYSTYPE*)_tmp15B)->Char_tok).tag == 2){_LL1: _tmp15C=(_tmp15B->Char_tok).val;_LL2: {char yy=_tmp15C;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_Char_tok(char yy1){
# 1098 "parse.y"
return({union Cyc_YYSTYPE _tmp5F4;(_tmp5F4.Char_tok).tag=2U,(_tmp5F4.Char_tok).val=yy1;_tmp5F4;});}static char _tmp160[13U]="string_t<`H>";
# 1119 "parse.y"
static struct _fat_ptr Cyc_yyget_String_tok(union Cyc_YYSTYPE*yy1){
# 1093 "parse.y"
static struct _fat_ptr s={_tmp160,_tmp160,_tmp160 + 13U};
union Cyc_YYSTYPE*_tmp15E=yy1;struct _fat_ptr _tmp15F;if((((union Cyc_YYSTYPE*)_tmp15E)->String_tok).tag == 3){_LL1: _tmp15F=(_tmp15E->String_tok).val;_LL2: {struct _fat_ptr yy=_tmp15F;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_String_tok(struct _fat_ptr yy1){
# 1099 "parse.y"
return({union Cyc_YYSTYPE _tmp5F5;(_tmp5F5.String_tok).tag=3U,(_tmp5F5.String_tok).val=yy1;_tmp5F5;});}static char _tmp163[45U]="$(Position::seg_t,booltype_t, ptrbound_t)@`H";
# 1119 "parse.y"
static struct _tuple22*Cyc_yyget_YY1(union Cyc_YYSTYPE*yy1){
# 1096 "parse.y"
static struct _fat_ptr s={_tmp163,_tmp163,_tmp163 + 45U};
union Cyc_YYSTYPE*_tmp161=yy1;struct _tuple22*_tmp162;if((((union Cyc_YYSTYPE*)_tmp161)->YY1).tag == 9){_LL1: _tmp162=(_tmp161->YY1).val;_LL2: {struct _tuple22*yy=_tmp162;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY1(struct _tuple22*yy1){
# 1102 "parse.y"
return({union Cyc_YYSTYPE _tmp5F6;(_tmp5F6.YY1).tag=9U,(_tmp5F6.YY1).val=yy1;_tmp5F6;});}static char _tmp166[11U]="ptrbound_t";
# 1119 "parse.y"
static void*Cyc_yyget_YY2(union Cyc_YYSTYPE*yy1){
# 1097 "parse.y"
static struct _fat_ptr s={_tmp166,_tmp166,_tmp166 + 11U};
union Cyc_YYSTYPE*_tmp164=yy1;void*_tmp165;if((((union Cyc_YYSTYPE*)_tmp164)->YY2).tag == 10){_LL1: _tmp165=(_tmp164->YY2).val;_LL2: {void*yy=_tmp165;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY2(void*yy1){
# 1103 "parse.y"
return({union Cyc_YYSTYPE _tmp5F7;(_tmp5F7.YY2).tag=10U,(_tmp5F7.YY2).val=yy1;_tmp5F7;});}static char _tmp169[28U]="list_t<offsetof_field_t,`H>";
# 1119 "parse.y"
static struct Cyc_List_List*Cyc_yyget_YY3(union Cyc_YYSTYPE*yy1){
# 1098 "parse.y"
static struct _fat_ptr s={_tmp169,_tmp169,_tmp169 + 28U};
union Cyc_YYSTYPE*_tmp167=yy1;struct Cyc_List_List*_tmp168;if((((union Cyc_YYSTYPE*)_tmp167)->YY3).tag == 11){_LL1: _tmp168=(_tmp167->YY3).val;_LL2: {struct Cyc_List_List*yy=_tmp168;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY3(struct Cyc_List_List*yy1){
# 1104 "parse.y"
return({union Cyc_YYSTYPE _tmp5F8;(_tmp5F8.YY3).tag=11U,(_tmp5F8.YY3).val=yy1;_tmp5F8;});}static char _tmp16C[6U]="exp_t";
# 1119 "parse.y"
static struct Cyc_Absyn_Exp*Cyc_yyget_Exp_tok(union Cyc_YYSTYPE*yy1){
# 1099 "parse.y"
static struct _fat_ptr s={_tmp16C,_tmp16C,_tmp16C + 6U};
union Cyc_YYSTYPE*_tmp16A=yy1;struct Cyc_Absyn_Exp*_tmp16B;if((((union Cyc_YYSTYPE*)_tmp16A)->Exp_tok).tag == 7){_LL1: _tmp16B=(_tmp16A->Exp_tok).val;_LL2: {struct Cyc_Absyn_Exp*yy=_tmp16B;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_Exp_tok(struct Cyc_Absyn_Exp*yy1){
# 1105 "parse.y"
return({union Cyc_YYSTYPE _tmp5F9;(_tmp5F9.Exp_tok).tag=7U,(_tmp5F9.Exp_tok).val=yy1;_tmp5F9;});}static char _tmp16F[17U]="list_t<exp_t,`H>";
# 1119 "parse.y"
static struct Cyc_List_List*Cyc_yyget_YY4(union Cyc_YYSTYPE*yy1){
# 1107 "parse.y"
static struct _fat_ptr s={_tmp16F,_tmp16F,_tmp16F + 17U};
union Cyc_YYSTYPE*_tmp16D=yy1;struct Cyc_List_List*_tmp16E;if((((union Cyc_YYSTYPE*)_tmp16D)->YY4).tag == 12){_LL1: _tmp16E=(_tmp16D->YY4).val;_LL2: {struct Cyc_List_List*yy=_tmp16E;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY4(struct Cyc_List_List*yy1){
# 1113 "parse.y"
return({union Cyc_YYSTYPE _tmp5FA;(_tmp5FA.YY4).tag=12U,(_tmp5FA.YY4).val=yy1;_tmp5FA;});}static char _tmp172[47U]="list_t<$(list_t<designator_t,`H>,exp_t)@`H,`H>";
# 1119 "parse.y"
static struct Cyc_List_List*Cyc_yyget_YY5(union Cyc_YYSTYPE*yy1){
# 1108 "parse.y"
static struct _fat_ptr s={_tmp172,_tmp172,_tmp172 + 47U};
union Cyc_YYSTYPE*_tmp170=yy1;struct Cyc_List_List*_tmp171;if((((union Cyc_YYSTYPE*)_tmp170)->YY5).tag == 13){_LL1: _tmp171=(_tmp170->YY5).val;_LL2: {struct Cyc_List_List*yy=_tmp171;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY5(struct Cyc_List_List*yy1){
# 1114 "parse.y"
return({union Cyc_YYSTYPE _tmp5FB;(_tmp5FB.YY5).tag=13U,(_tmp5FB.YY5).val=yy1;_tmp5FB;});}static char _tmp175[9U]="primop_t";
# 1119 "parse.y"
static enum Cyc_Absyn_Primop Cyc_yyget_YY6(union Cyc_YYSTYPE*yy1){
# 1109 "parse.y"
static struct _fat_ptr s={_tmp175,_tmp175,_tmp175 + 9U};
union Cyc_YYSTYPE*_tmp173=yy1;enum Cyc_Absyn_Primop _tmp174;if((((union Cyc_YYSTYPE*)_tmp173)->YY6).tag == 14){_LL1: _tmp174=(_tmp173->YY6).val;_LL2: {enum Cyc_Absyn_Primop yy=_tmp174;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY6(enum Cyc_Absyn_Primop yy1){
# 1115 "parse.y"
return({union Cyc_YYSTYPE _tmp5FC;(_tmp5FC.YY6).tag=14U,(_tmp5FC.YY6).val=yy1;_tmp5FC;});}static char _tmp178[19U]="opt_t<primop_t,`H>";
# 1119 "parse.y"
static struct Cyc_Core_Opt*Cyc_yyget_YY7(union Cyc_YYSTYPE*yy1){
# 1110 "parse.y"
static struct _fat_ptr s={_tmp178,_tmp178,_tmp178 + 19U};
union Cyc_YYSTYPE*_tmp176=yy1;struct Cyc_Core_Opt*_tmp177;if((((union Cyc_YYSTYPE*)_tmp176)->YY7).tag == 15){_LL1: _tmp177=(_tmp176->YY7).val;_LL2: {struct Cyc_Core_Opt*yy=_tmp177;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY7(struct Cyc_Core_Opt*yy1){
# 1116 "parse.y"
return({union Cyc_YYSTYPE _tmp5FD;(_tmp5FD.YY7).tag=15U,(_tmp5FD.YY7).val=yy1;_tmp5FD;});}static char _tmp17B[7U]="qvar_t";
# 1119 "parse.y"
static struct _tuple0*Cyc_yyget_QualId_tok(union Cyc_YYSTYPE*yy1){
# 1111 "parse.y"
static struct _fat_ptr s={_tmp17B,_tmp17B,_tmp17B + 7U};
union Cyc_YYSTYPE*_tmp179=yy1;struct _tuple0*_tmp17A;if((((union Cyc_YYSTYPE*)_tmp179)->QualId_tok).tag == 5){_LL1: _tmp17A=(_tmp179->QualId_tok).val;_LL2: {struct _tuple0*yy=_tmp17A;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_QualId_tok(struct _tuple0*yy1){
# 1117 "parse.y"
return({union Cyc_YYSTYPE _tmp5FE;(_tmp5FE.QualId_tok).tag=5U,(_tmp5FE.QualId_tok).val=yy1;_tmp5FE;});}static char _tmp17E[7U]="stmt_t";
# 1119 "parse.y"
static struct Cyc_Absyn_Stmt*Cyc_yyget_Stmt_tok(union Cyc_YYSTYPE*yy1){
# 1114 "parse.y"
static struct _fat_ptr s={_tmp17E,_tmp17E,_tmp17E + 7U};
union Cyc_YYSTYPE*_tmp17C=yy1;struct Cyc_Absyn_Stmt*_tmp17D;if((((union Cyc_YYSTYPE*)_tmp17C)->Stmt_tok).tag == 8){_LL1: _tmp17D=(_tmp17C->Stmt_tok).val;_LL2: {struct Cyc_Absyn_Stmt*yy=_tmp17D;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_Stmt_tok(struct Cyc_Absyn_Stmt*yy1){
# 1120 "parse.y"
return({union Cyc_YYSTYPE _tmp5FF;(_tmp5FF.Stmt_tok).tag=8U,(_tmp5FF.Stmt_tok).val=yy1;_tmp5FF;});}static char _tmp181[27U]="list_t<switch_clause_t,`H>";
# 1119 "parse.y"
static struct Cyc_List_List*Cyc_yyget_YY8(union Cyc_YYSTYPE*yy1){
# 1118 "parse.y"
static struct _fat_ptr s={_tmp181,_tmp181,_tmp181 + 27U};
union Cyc_YYSTYPE*_tmp17F=yy1;struct Cyc_List_List*_tmp180;if((((union Cyc_YYSTYPE*)_tmp17F)->YY8).tag == 16){_LL1: _tmp180=(_tmp17F->YY8).val;_LL2: {struct Cyc_List_List*yy=_tmp180;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY8(struct Cyc_List_List*yy1){
# 1124 "parse.y"
return({union Cyc_YYSTYPE _tmp600;(_tmp600.YY8).tag=16U,(_tmp600.YY8).val=yy1;_tmp600;});}static char _tmp184[6U]="pat_t";
# 1119 "parse.y"
static struct Cyc_Absyn_Pat*Cyc_yyget_YY9(union Cyc_YYSTYPE*yy1){
# 1119 "parse.y"
static struct _fat_ptr s={_tmp184,_tmp184,_tmp184 + 6U};
union Cyc_YYSTYPE*_tmp182=yy1;struct Cyc_Absyn_Pat*_tmp183;if((((union Cyc_YYSTYPE*)_tmp182)->YY9).tag == 17){_LL1: _tmp183=(_tmp182->YY9).val;_LL2: {struct Cyc_Absyn_Pat*yy=_tmp183;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY9(struct Cyc_Absyn_Pat*yy1){
# 1125 "parse.y"
return({union Cyc_YYSTYPE _tmp601;(_tmp601.YY9).tag=17U,(_tmp601.YY9).val=yy1;_tmp601;});}static char _tmp187[28U]="$(list_t<pat_t,`H>,bool)@`H";
# 1119 "parse.y"
static struct _tuple23*Cyc_yyget_YY10(union Cyc_YYSTYPE*yy1){
# 1124 "parse.y"
static struct _fat_ptr s={_tmp187,_tmp187,_tmp187 + 28U};
union Cyc_YYSTYPE*_tmp185=yy1;struct _tuple23*_tmp186;if((((union Cyc_YYSTYPE*)_tmp185)->YY10).tag == 18){_LL1: _tmp186=(_tmp185->YY10).val;_LL2: {struct _tuple23*yy=_tmp186;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY10(struct _tuple23*yy1){
# 1130 "parse.y"
return({union Cyc_YYSTYPE _tmp602;(_tmp602.YY10).tag=18U,(_tmp602.YY10).val=yy1;_tmp602;});}static char _tmp18A[17U]="list_t<pat_t,`H>";
# 1119 "parse.y"
static struct Cyc_List_List*Cyc_yyget_YY11(union Cyc_YYSTYPE*yy1){
# 1125 "parse.y"
static struct _fat_ptr s={_tmp18A,_tmp18A,_tmp18A + 17U};
union Cyc_YYSTYPE*_tmp188=yy1;struct Cyc_List_List*_tmp189;if((((union Cyc_YYSTYPE*)_tmp188)->YY11).tag == 19){_LL1: _tmp189=(_tmp188->YY11).val;_LL2: {struct Cyc_List_List*yy=_tmp189;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY11(struct Cyc_List_List*yy1){
# 1131 "parse.y"
return({union Cyc_YYSTYPE _tmp603;(_tmp603.YY11).tag=19U,(_tmp603.YY11).val=yy1;_tmp603;});}static char _tmp18D[36U]="$(list_t<designator_t,`H>,pat_t)@`H";
# 1119 "parse.y"
static struct _tuple24*Cyc_yyget_YY12(union Cyc_YYSTYPE*yy1){
# 1126 "parse.y"
static struct _fat_ptr s={_tmp18D,_tmp18D,_tmp18D + 36U};
union Cyc_YYSTYPE*_tmp18B=yy1;struct _tuple24*_tmp18C;if((((union Cyc_YYSTYPE*)_tmp18B)->YY12).tag == 20){_LL1: _tmp18C=(_tmp18B->YY12).val;_LL2: {struct _tuple24*yy=_tmp18C;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY12(struct _tuple24*yy1){
# 1132 "parse.y"
return({union Cyc_YYSTYPE _tmp604;(_tmp604.YY12).tag=20U,(_tmp604.YY12).val=yy1;_tmp604;});}static char _tmp190[47U]="list_t<$(list_t<designator_t,`H>,pat_t)@`H,`H>";
# 1119 "parse.y"
static struct Cyc_List_List*Cyc_yyget_YY13(union Cyc_YYSTYPE*yy1){
# 1127 "parse.y"
static struct _fat_ptr s={_tmp190,_tmp190,_tmp190 + 47U};
union Cyc_YYSTYPE*_tmp18E=yy1;struct Cyc_List_List*_tmp18F;if((((union Cyc_YYSTYPE*)_tmp18E)->YY13).tag == 21){_LL1: _tmp18F=(_tmp18E->YY13).val;_LL2: {struct Cyc_List_List*yy=_tmp18F;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY13(struct Cyc_List_List*yy1){
# 1133 "parse.y"
return({union Cyc_YYSTYPE _tmp605;(_tmp605.YY13).tag=21U,(_tmp605.YY13).val=yy1;_tmp605;});}static char _tmp193[58U]="$(list_t<$(list_t<designator_t,`H>,pat_t)@`H,`H>,bool)@`H";
# 1119 "parse.y"
static struct _tuple23*Cyc_yyget_YY14(union Cyc_YYSTYPE*yy1){
# 1128 "parse.y"
static struct _fat_ptr s={_tmp193,_tmp193,_tmp193 + 58U};
union Cyc_YYSTYPE*_tmp191=yy1;struct _tuple23*_tmp192;if((((union Cyc_YYSTYPE*)_tmp191)->YY14).tag == 22){_LL1: _tmp192=(_tmp191->YY14).val;_LL2: {struct _tuple23*yy=_tmp192;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY14(struct _tuple23*yy1){
# 1134 "parse.y"
return({union Cyc_YYSTYPE _tmp606;(_tmp606.YY14).tag=22U,(_tmp606.YY14).val=yy1;_tmp606;});}static char _tmp196[9U]="fndecl_t";
# 1119 "parse.y"
static struct Cyc_Absyn_Fndecl*Cyc_yyget_YY15(union Cyc_YYSTYPE*yy1){
# 1129 "parse.y"
static struct _fat_ptr s={_tmp196,_tmp196,_tmp196 + 9U};
union Cyc_YYSTYPE*_tmp194=yy1;struct Cyc_Absyn_Fndecl*_tmp195;if((((union Cyc_YYSTYPE*)_tmp194)->YY15).tag == 23){_LL1: _tmp195=(_tmp194->YY15).val;_LL2: {struct Cyc_Absyn_Fndecl*yy=_tmp195;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY15(struct Cyc_Absyn_Fndecl*yy1){
# 1135 "parse.y"
return({union Cyc_YYSTYPE _tmp607;(_tmp607.YY15).tag=23U,(_tmp607.YY15).val=yy1;_tmp607;});}static char _tmp199[18U]="list_t<decl_t,`H>";
# 1119 "parse.y"
static struct Cyc_List_List*Cyc_yyget_YY16(union Cyc_YYSTYPE*yy1){
# 1130 "parse.y"
static struct _fat_ptr s={_tmp199,_tmp199,_tmp199 + 18U};
union Cyc_YYSTYPE*_tmp197=yy1;struct Cyc_List_List*_tmp198;if((((union Cyc_YYSTYPE*)_tmp197)->YY16).tag == 24){_LL1: _tmp198=(_tmp197->YY16).val;_LL2: {struct Cyc_List_List*yy=_tmp198;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY16(struct Cyc_List_List*yy1){
# 1136 "parse.y"
return({union Cyc_YYSTYPE _tmp608;(_tmp608.YY16).tag=24U,(_tmp608.YY16).val=yy1;_tmp608;});}static char _tmp19C[12U]="decl_spec_t";
# 1119 "parse.y"
static struct Cyc_Parse_Declaration_spec Cyc_yyget_YY17(union Cyc_YYSTYPE*yy1){
# 1133 "parse.y"
static struct _fat_ptr s={_tmp19C,_tmp19C,_tmp19C + 12U};
union Cyc_YYSTYPE*_tmp19A=yy1;struct Cyc_Parse_Declaration_spec _tmp19B;if((((union Cyc_YYSTYPE*)_tmp19A)->YY17).tag == 25){_LL1: _tmp19B=(_tmp19A->YY17).val;_LL2: {struct Cyc_Parse_Declaration_spec yy=_tmp19B;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY17(struct Cyc_Parse_Declaration_spec yy1){
# 1139 "parse.y"
return({union Cyc_YYSTYPE _tmp609;(_tmp609.YY17).tag=25U,(_tmp609.YY17).val=yy1;_tmp609;});}static char _tmp19F[31U]="$(declarator_t<`yy>,exp_opt_t)";
# 1119 "parse.y"
static struct _tuple12 Cyc_yyget_YY18(union Cyc_YYSTYPE*yy1){
# 1134 "parse.y"
static struct _fat_ptr s={_tmp19F,_tmp19F,_tmp19F + 31U};
union Cyc_YYSTYPE*_tmp19D=yy1;struct _tuple12 _tmp19E;if((((union Cyc_YYSTYPE*)_tmp19D)->YY18).tag == 26){_LL1: _tmp19E=(_tmp19D->YY18).val;_LL2: {struct _tuple12 yy=_tmp19E;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY18(struct _tuple12 yy1){
# 1140 "parse.y"
return({union Cyc_YYSTYPE _tmp60A;(_tmp60A.YY18).tag=26U,(_tmp60A.YY18).val=yy1;_tmp60A;});}static char _tmp1A2[23U]="declarator_list_t<`yy>";
# 1119 "parse.y"
static struct _tuple13*Cyc_yyget_YY19(union Cyc_YYSTYPE*yy1){
# 1135 "parse.y"
static struct _fat_ptr s={_tmp1A2,_tmp1A2,_tmp1A2 + 23U};
union Cyc_YYSTYPE*_tmp1A0=yy1;struct _tuple13*_tmp1A1;if((((union Cyc_YYSTYPE*)_tmp1A0)->YY19).tag == 27){_LL1: _tmp1A1=(_tmp1A0->YY19).val;_LL2: {struct _tuple13*yy=_tmp1A1;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY19(struct _tuple13*yy1){
# 1141 "parse.y"
return({union Cyc_YYSTYPE _tmp60B;(_tmp60B.YY19).tag=27U,(_tmp60B.YY19).val=yy1;_tmp60B;});}static char _tmp1A5[19U]="storage_class_t@`H";
# 1119 "parse.y"
static enum Cyc_Parse_Storage_class*Cyc_yyget_YY20(union Cyc_YYSTYPE*yy1){
# 1136 "parse.y"
static struct _fat_ptr s={_tmp1A5,_tmp1A5,_tmp1A5 + 19U};
union Cyc_YYSTYPE*_tmp1A3=yy1;enum Cyc_Parse_Storage_class*_tmp1A4;if((((union Cyc_YYSTYPE*)_tmp1A3)->YY20).tag == 28){_LL1: _tmp1A4=(_tmp1A3->YY20).val;_LL2: {enum Cyc_Parse_Storage_class*yy=_tmp1A4;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY20(enum Cyc_Parse_Storage_class*yy1){
# 1142 "parse.y"
return({union Cyc_YYSTYPE _tmp60C;(_tmp60C.YY20).tag=28U,(_tmp60C.YY20).val=yy1;_tmp60C;});}static char _tmp1A8[17U]="type_specifier_t";
# 1119 "parse.y"
static struct Cyc_Parse_Type_specifier Cyc_yyget_YY21(union Cyc_YYSTYPE*yy1){
# 1137 "parse.y"
static struct _fat_ptr s={_tmp1A8,_tmp1A8,_tmp1A8 + 17U};
union Cyc_YYSTYPE*_tmp1A6=yy1;struct Cyc_Parse_Type_specifier _tmp1A7;if((((union Cyc_YYSTYPE*)_tmp1A6)->YY21).tag == 29){_LL1: _tmp1A7=(_tmp1A6->YY21).val;_LL2: {struct Cyc_Parse_Type_specifier yy=_tmp1A7;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY21(struct Cyc_Parse_Type_specifier yy1){
# 1143 "parse.y"
return({union Cyc_YYSTYPE _tmp60D;(_tmp60D.YY21).tag=29U,(_tmp60D.YY21).val=yy1;_tmp60D;});}static char _tmp1AB[12U]="aggr_kind_t";
# 1119 "parse.y"
static enum Cyc_Absyn_AggrKind Cyc_yyget_YY22(union Cyc_YYSTYPE*yy1){
# 1139 "parse.y"
static struct _fat_ptr s={_tmp1AB,_tmp1AB,_tmp1AB + 12U};
union Cyc_YYSTYPE*_tmp1A9=yy1;enum Cyc_Absyn_AggrKind _tmp1AA;if((((union Cyc_YYSTYPE*)_tmp1A9)->YY22).tag == 30){_LL1: _tmp1AA=(_tmp1A9->YY22).val;_LL2: {enum Cyc_Absyn_AggrKind yy=_tmp1AA;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY22(enum Cyc_Absyn_AggrKind yy1){
# 1145 "parse.y"
return({union Cyc_YYSTYPE _tmp60E;(_tmp60E.YY22).tag=30U,(_tmp60E.YY22).val=yy1;_tmp60E;});}static char _tmp1AE[8U]="tqual_t";
# 1119 "parse.y"
static struct Cyc_Absyn_Tqual Cyc_yyget_YY23(union Cyc_YYSTYPE*yy1){
# 1140 "parse.y"
static struct _fat_ptr s={_tmp1AE,_tmp1AE,_tmp1AE + 8U};
union Cyc_YYSTYPE*_tmp1AC=yy1;struct Cyc_Absyn_Tqual _tmp1AD;if((((union Cyc_YYSTYPE*)_tmp1AC)->YY23).tag == 31){_LL1: _tmp1AD=(_tmp1AC->YY23).val;_LL2: {struct Cyc_Absyn_Tqual yy=_tmp1AD;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY23(struct Cyc_Absyn_Tqual yy1){
# 1146 "parse.y"
return({union Cyc_YYSTYPE _tmp60F;(_tmp60F.YY23).tag=31U,(_tmp60F.YY23).val=yy1;_tmp60F;});}static char _tmp1B1[23U]="list_t<aggrfield_t,`H>";
# 1119 "parse.y"
static struct Cyc_List_List*Cyc_yyget_YY24(union Cyc_YYSTYPE*yy1){
# 1141 "parse.y"
static struct _fat_ptr s={_tmp1B1,_tmp1B1,_tmp1B1 + 23U};
union Cyc_YYSTYPE*_tmp1AF=yy1;struct Cyc_List_List*_tmp1B0;if((((union Cyc_YYSTYPE*)_tmp1AF)->YY24).tag == 32){_LL1: _tmp1B0=(_tmp1AF->YY24).val;_LL2: {struct Cyc_List_List*yy=_tmp1B0;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY24(struct Cyc_List_List*yy1){
# 1147 "parse.y"
return({union Cyc_YYSTYPE _tmp610;(_tmp610.YY24).tag=32U,(_tmp610.YY24).val=yy1;_tmp610;});}static char _tmp1B4[34U]="list_t<list_t<aggrfield_t,`H>,`H>";
# 1119 "parse.y"
static struct Cyc_List_List*Cyc_yyget_YY25(union Cyc_YYSTYPE*yy1){
# 1142 "parse.y"
static struct _fat_ptr s={_tmp1B4,_tmp1B4,_tmp1B4 + 34U};
union Cyc_YYSTYPE*_tmp1B2=yy1;struct Cyc_List_List*_tmp1B3;if((((union Cyc_YYSTYPE*)_tmp1B2)->YY25).tag == 33){_LL1: _tmp1B3=(_tmp1B2->YY25).val;_LL2: {struct Cyc_List_List*yy=_tmp1B3;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY25(struct Cyc_List_List*yy1){
# 1148 "parse.y"
return({union Cyc_YYSTYPE _tmp611;(_tmp611.YY25).tag=33U,(_tmp611.YY25).val=yy1;_tmp611;});}static char _tmp1B7[33U]="list_t<type_modifier_t<`yy>,`yy>";
# 1119 "parse.y"
static struct Cyc_List_List*Cyc_yyget_YY26(union Cyc_YYSTYPE*yy1){
# 1143 "parse.y"
static struct _fat_ptr s={_tmp1B7,_tmp1B7,_tmp1B7 + 33U};
union Cyc_YYSTYPE*_tmp1B5=yy1;struct Cyc_List_List*_tmp1B6;if((((union Cyc_YYSTYPE*)_tmp1B5)->YY26).tag == 34){_LL1: _tmp1B6=(_tmp1B5->YY26).val;_LL2: {struct Cyc_List_List*yy=_tmp1B6;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY26(struct Cyc_List_List*yy1){
# 1149 "parse.y"
return({union Cyc_YYSTYPE _tmp612;(_tmp612.YY26).tag=34U,(_tmp612.YY26).val=yy1;_tmp612;});}static char _tmp1BA[18U]="declarator_t<`yy>";
# 1119 "parse.y"
static struct Cyc_Parse_Declarator Cyc_yyget_YY27(union Cyc_YYSTYPE*yy1){
# 1144 "parse.y"
static struct _fat_ptr s={_tmp1BA,_tmp1BA,_tmp1BA + 18U};
union Cyc_YYSTYPE*_tmp1B8=yy1;struct Cyc_Parse_Declarator _tmp1B9;if((((union Cyc_YYSTYPE*)_tmp1B8)->YY27).tag == 35){_LL1: _tmp1B9=(_tmp1B8->YY27).val;_LL2: {struct Cyc_Parse_Declarator yy=_tmp1B9;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY27(struct Cyc_Parse_Declarator yy1){
# 1150 "parse.y"
return({union Cyc_YYSTYPE _tmp613;(_tmp613.YY27).tag=35U,(_tmp613.YY27).val=yy1;_tmp613;});}static char _tmp1BD[45U]="$(declarator_t<`yy>,exp_opt_t,exp_opt_t)@`yy";
# 1119 "parse.y"
static struct _tuple25*Cyc_yyget_YY28(union Cyc_YYSTYPE*yy1){
# 1145 "parse.y"
static struct _fat_ptr s={_tmp1BD,_tmp1BD,_tmp1BD + 45U};
union Cyc_YYSTYPE*_tmp1BB=yy1;struct _tuple25*_tmp1BC;if((((union Cyc_YYSTYPE*)_tmp1BB)->YY28).tag == 36){_LL1: _tmp1BC=(_tmp1BB->YY28).val;_LL2: {struct _tuple25*yy=_tmp1BC;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY28(struct _tuple25*yy1){
# 1151 "parse.y"
return({union Cyc_YYSTYPE _tmp614;(_tmp614.YY28).tag=36U,(_tmp614.YY28).val=yy1;_tmp614;});}static char _tmp1C0[57U]="list_t<$(declarator_t<`yy>,exp_opt_t,exp_opt_t)@`yy,`yy>";
# 1119 "parse.y"
static struct Cyc_List_List*Cyc_yyget_YY29(union Cyc_YYSTYPE*yy1){
# 1146 "parse.y"
static struct _fat_ptr s={_tmp1C0,_tmp1C0,_tmp1C0 + 57U};
union Cyc_YYSTYPE*_tmp1BE=yy1;struct Cyc_List_List*_tmp1BF;if((((union Cyc_YYSTYPE*)_tmp1BE)->YY29).tag == 37){_LL1: _tmp1BF=(_tmp1BE->YY29).val;_LL2: {struct Cyc_List_List*yy=_tmp1BF;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY29(struct Cyc_List_List*yy1){
# 1152 "parse.y"
return({union Cyc_YYSTYPE _tmp615;(_tmp615.YY29).tag=37U,(_tmp615.YY29).val=yy1;_tmp615;});}static char _tmp1C3[26U]="abstractdeclarator_t<`yy>";
# 1119 "parse.y"
static struct Cyc_Parse_Abstractdeclarator Cyc_yyget_YY30(union Cyc_YYSTYPE*yy1){
# 1147 "parse.y"
static struct _fat_ptr s={_tmp1C3,_tmp1C3,_tmp1C3 + 26U};
union Cyc_YYSTYPE*_tmp1C1=yy1;struct Cyc_Parse_Abstractdeclarator _tmp1C2;if((((union Cyc_YYSTYPE*)_tmp1C1)->YY30).tag == 38){_LL1: _tmp1C2=(_tmp1C1->YY30).val;_LL2: {struct Cyc_Parse_Abstractdeclarator yy=_tmp1C2;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY30(struct Cyc_Parse_Abstractdeclarator yy1){
# 1153 "parse.y"
return({union Cyc_YYSTYPE _tmp616;(_tmp616.YY30).tag=38U,(_tmp616.YY30).val=yy1;_tmp616;});}static char _tmp1C6[5U]="bool";
# 1119 "parse.y"
static int Cyc_yyget_YY31(union Cyc_YYSTYPE*yy1){
# 1148 "parse.y"
static struct _fat_ptr s={_tmp1C6,_tmp1C6,_tmp1C6 + 5U};
union Cyc_YYSTYPE*_tmp1C4=yy1;int _tmp1C5;if((((union Cyc_YYSTYPE*)_tmp1C4)->YY31).tag == 39){_LL1: _tmp1C5=(_tmp1C4->YY31).val;_LL2: {int yy=_tmp1C5;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY31(int yy1){
# 1154 "parse.y"
return({union Cyc_YYSTYPE _tmp617;(_tmp617.YY31).tag=39U,(_tmp617.YY31).val=yy1;_tmp617;});}static char _tmp1C9[8U]="scope_t";
# 1119 "parse.y"
static enum Cyc_Absyn_Scope Cyc_yyget_YY32(union Cyc_YYSTYPE*yy1){
# 1149 "parse.y"
static struct _fat_ptr s={_tmp1C9,_tmp1C9,_tmp1C9 + 8U};
union Cyc_YYSTYPE*_tmp1C7=yy1;enum Cyc_Absyn_Scope _tmp1C8;if((((union Cyc_YYSTYPE*)_tmp1C7)->YY32).tag == 40){_LL1: _tmp1C8=(_tmp1C7->YY32).val;_LL2: {enum Cyc_Absyn_Scope yy=_tmp1C8;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY32(enum Cyc_Absyn_Scope yy1){
# 1155 "parse.y"
return({union Cyc_YYSTYPE _tmp618;(_tmp618.YY32).tag=40U,(_tmp618.YY32).val=yy1;_tmp618;});}static char _tmp1CC[16U]="datatypefield_t";
# 1119 "parse.y"
static struct Cyc_Absyn_Datatypefield*Cyc_yyget_YY33(union Cyc_YYSTYPE*yy1){
# 1150 "parse.y"
static struct _fat_ptr s={_tmp1CC,_tmp1CC,_tmp1CC + 16U};
union Cyc_YYSTYPE*_tmp1CA=yy1;struct Cyc_Absyn_Datatypefield*_tmp1CB;if((((union Cyc_YYSTYPE*)_tmp1CA)->YY33).tag == 41){_LL1: _tmp1CB=(_tmp1CA->YY33).val;_LL2: {struct Cyc_Absyn_Datatypefield*yy=_tmp1CB;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY33(struct Cyc_Absyn_Datatypefield*yy1){
# 1156 "parse.y"
return({union Cyc_YYSTYPE _tmp619;(_tmp619.YY33).tag=41U,(_tmp619.YY33).val=yy1;_tmp619;});}static char _tmp1CF[27U]="list_t<datatypefield_t,`H>";
# 1119 "parse.y"
static struct Cyc_List_List*Cyc_yyget_YY34(union Cyc_YYSTYPE*yy1){
# 1151 "parse.y"
static struct _fat_ptr s={_tmp1CF,_tmp1CF,_tmp1CF + 27U};
union Cyc_YYSTYPE*_tmp1CD=yy1;struct Cyc_List_List*_tmp1CE;if((((union Cyc_YYSTYPE*)_tmp1CD)->YY34).tag == 42){_LL1: _tmp1CE=(_tmp1CD->YY34).val;_LL2: {struct Cyc_List_List*yy=_tmp1CE;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY34(struct Cyc_List_List*yy1){
# 1157 "parse.y"
return({union Cyc_YYSTYPE _tmp61A;(_tmp61A.YY34).tag=42U,(_tmp61A.YY34).val=yy1;_tmp61A;});}static char _tmp1D2[41U]="$(tqual_t,type_specifier_t,attributes_t)";
# 1119 "parse.y"
static struct _tuple26 Cyc_yyget_YY35(union Cyc_YYSTYPE*yy1){
# 1152 "parse.y"
static struct _fat_ptr s={_tmp1D2,_tmp1D2,_tmp1D2 + 41U};
union Cyc_YYSTYPE*_tmp1D0=yy1;struct _tuple26 _tmp1D1;if((((union Cyc_YYSTYPE*)_tmp1D0)->YY35).tag == 43){_LL1: _tmp1D1=(_tmp1D0->YY35).val;_LL2: {struct _tuple26 yy=_tmp1D1;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY35(struct _tuple26 yy1){
# 1158 "parse.y"
return({union Cyc_YYSTYPE _tmp61B;(_tmp61B.YY35).tag=43U,(_tmp61B.YY35).val=yy1;_tmp61B;});}static char _tmp1D5[17U]="list_t<var_t,`H>";
# 1119 "parse.y"
static struct Cyc_List_List*Cyc_yyget_YY36(union Cyc_YYSTYPE*yy1){
# 1153 "parse.y"
static struct _fat_ptr s={_tmp1D5,_tmp1D5,_tmp1D5 + 17U};
union Cyc_YYSTYPE*_tmp1D3=yy1;struct Cyc_List_List*_tmp1D4;if((((union Cyc_YYSTYPE*)_tmp1D3)->YY36).tag == 44){_LL1: _tmp1D4=(_tmp1D3->YY36).val;_LL2: {struct Cyc_List_List*yy=_tmp1D4;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY36(struct Cyc_List_List*yy1){
# 1159 "parse.y"
return({union Cyc_YYSTYPE _tmp61C;(_tmp61C.YY36).tag=44U,(_tmp61C.YY36).val=yy1;_tmp61C;});}static char _tmp1D8[31U]="$(var_opt_t,tqual_t,type_t)@`H";
# 1119 "parse.y"
static struct _tuple8*Cyc_yyget_YY37(union Cyc_YYSTYPE*yy1){
# 1154 "parse.y"
static struct _fat_ptr s={_tmp1D8,_tmp1D8,_tmp1D8 + 31U};
union Cyc_YYSTYPE*_tmp1D6=yy1;struct _tuple8*_tmp1D7;if((((union Cyc_YYSTYPE*)_tmp1D6)->YY37).tag == 45){_LL1: _tmp1D7=(_tmp1D6->YY37).val;_LL2: {struct _tuple8*yy=_tmp1D7;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY37(struct _tuple8*yy1){
# 1160 "parse.y"
return({union Cyc_YYSTYPE _tmp61D;(_tmp61D.YY37).tag=45U,(_tmp61D.YY37).val=yy1;_tmp61D;});}static char _tmp1DB[42U]="list_t<$(var_opt_t,tqual_t,type_t)@`H,`H>";
# 1119 "parse.y"
static struct Cyc_List_List*Cyc_yyget_YY38(union Cyc_YYSTYPE*yy1){
# 1155 "parse.y"
static struct _fat_ptr s={_tmp1DB,_tmp1DB,_tmp1DB + 42U};
union Cyc_YYSTYPE*_tmp1D9=yy1;struct Cyc_List_List*_tmp1DA;if((((union Cyc_YYSTYPE*)_tmp1D9)->YY38).tag == 46){_LL1: _tmp1DA=(_tmp1D9->YY38).val;_LL2: {struct Cyc_List_List*yy=_tmp1DA;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY38(struct Cyc_List_List*yy1){
# 1161 "parse.y"
return({union Cyc_YYSTYPE _tmp61E;(_tmp61E.YY38).tag=46U,(_tmp61E.YY38).val=yy1;_tmp61E;});}static char _tmp1DE[115U]="$(list_t<$(var_opt_t,tqual_t,type_t)@`H,`H>, bool,vararg_info_t *`H,type_opt_t, list_t<$(type_t,type_t)@`H,`H>)@`H";
# 1119 "parse.y"
static struct _tuple27*Cyc_yyget_YY39(union Cyc_YYSTYPE*yy1){
# 1156 "parse.y"
static struct _fat_ptr s={_tmp1DE,_tmp1DE,_tmp1DE + 115U};
union Cyc_YYSTYPE*_tmp1DC=yy1;struct _tuple27*_tmp1DD;if((((union Cyc_YYSTYPE*)_tmp1DC)->YY39).tag == 47){_LL1: _tmp1DD=(_tmp1DC->YY39).val;_LL2: {struct _tuple27*yy=_tmp1DD;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY39(struct _tuple27*yy1){
# 1162 "parse.y"
return({union Cyc_YYSTYPE _tmp61F;(_tmp61F.YY39).tag=47U,(_tmp61F.YY39).val=yy1;_tmp61F;});}static char _tmp1E1[8U]="types_t";
# 1119 "parse.y"
static struct Cyc_List_List*Cyc_yyget_YY40(union Cyc_YYSTYPE*yy1){
# 1157 "parse.y"
static struct _fat_ptr s={_tmp1E1,_tmp1E1,_tmp1E1 + 8U};
union Cyc_YYSTYPE*_tmp1DF=yy1;struct Cyc_List_List*_tmp1E0;if((((union Cyc_YYSTYPE*)_tmp1DF)->YY40).tag == 48){_LL1: _tmp1E0=(_tmp1DF->YY40).val;_LL2: {struct Cyc_List_List*yy=_tmp1E0;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY40(struct Cyc_List_List*yy1){
# 1163 "parse.y"
return({union Cyc_YYSTYPE _tmp620;(_tmp620.YY40).tag=48U,(_tmp620.YY40).val=yy1;_tmp620;});}static char _tmp1E4[24U]="list_t<designator_t,`H>";
# 1119 "parse.y"
static struct Cyc_List_List*Cyc_yyget_YY41(union Cyc_YYSTYPE*yy1){
# 1159 "parse.y"
static struct _fat_ptr s={_tmp1E4,_tmp1E4,_tmp1E4 + 24U};
union Cyc_YYSTYPE*_tmp1E2=yy1;struct Cyc_List_List*_tmp1E3;if((((union Cyc_YYSTYPE*)_tmp1E2)->YY41).tag == 49){_LL1: _tmp1E3=(_tmp1E2->YY41).val;_LL2: {struct Cyc_List_List*yy=_tmp1E3;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY41(struct Cyc_List_List*yy1){
# 1165 "parse.y"
return({union Cyc_YYSTYPE _tmp621;(_tmp621.YY41).tag=49U,(_tmp621.YY41).val=yy1;_tmp621;});}static char _tmp1E7[13U]="designator_t";
# 1119 "parse.y"
static void*Cyc_yyget_YY42(union Cyc_YYSTYPE*yy1){
# 1160 "parse.y"
static struct _fat_ptr s={_tmp1E7,_tmp1E7,_tmp1E7 + 13U};
union Cyc_YYSTYPE*_tmp1E5=yy1;void*_tmp1E6;if((((union Cyc_YYSTYPE*)_tmp1E5)->YY42).tag == 50){_LL1: _tmp1E6=(_tmp1E5->YY42).val;_LL2: {void*yy=_tmp1E6;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY42(void*yy1){
# 1166 "parse.y"
return({union Cyc_YYSTYPE _tmp622;(_tmp622.YY42).tag=50U,(_tmp622.YY42).val=yy1;_tmp622;});}static char _tmp1EA[7U]="kind_t";
# 1119 "parse.y"
static struct Cyc_Absyn_Kind*Cyc_yyget_YY43(union Cyc_YYSTYPE*yy1){
# 1161 "parse.y"
static struct _fat_ptr s={_tmp1EA,_tmp1EA,_tmp1EA + 7U};
union Cyc_YYSTYPE*_tmp1E8=yy1;struct Cyc_Absyn_Kind*_tmp1E9;if((((union Cyc_YYSTYPE*)_tmp1E8)->YY43).tag == 51){_LL1: _tmp1E9=(_tmp1E8->YY43).val;_LL2: {struct Cyc_Absyn_Kind*yy=_tmp1E9;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY43(struct Cyc_Absyn_Kind*yy1){
# 1167 "parse.y"
return({union Cyc_YYSTYPE _tmp623;(_tmp623.YY43).tag=51U,(_tmp623.YY43).val=yy1;_tmp623;});}static char _tmp1ED[7U]="type_t";
# 1119 "parse.y"
static void*Cyc_yyget_YY44(union Cyc_YYSTYPE*yy1){
# 1162 "parse.y"
static struct _fat_ptr s={_tmp1ED,_tmp1ED,_tmp1ED + 7U};
union Cyc_YYSTYPE*_tmp1EB=yy1;void*_tmp1EC;if((((union Cyc_YYSTYPE*)_tmp1EB)->YY44).tag == 52){_LL1: _tmp1EC=(_tmp1EB->YY44).val;_LL2: {void*yy=_tmp1EC;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY44(void*yy1){
# 1168 "parse.y"
return({union Cyc_YYSTYPE _tmp624;(_tmp624.YY44).tag=52U,(_tmp624.YY44).val=yy1;_tmp624;});}static char _tmp1F0[23U]="list_t<attribute_t,`H>";
# 1119 "parse.y"
static struct Cyc_List_List*Cyc_yyget_YY45(union Cyc_YYSTYPE*yy1){
# 1163 "parse.y"
static struct _fat_ptr s={_tmp1F0,_tmp1F0,_tmp1F0 + 23U};
union Cyc_YYSTYPE*_tmp1EE=yy1;struct Cyc_List_List*_tmp1EF;if((((union Cyc_YYSTYPE*)_tmp1EE)->YY45).tag == 53){_LL1: _tmp1EF=(_tmp1EE->YY45).val;_LL2: {struct Cyc_List_List*yy=_tmp1EF;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY45(struct Cyc_List_List*yy1){
# 1169 "parse.y"
return({union Cyc_YYSTYPE _tmp625;(_tmp625.YY45).tag=53U,(_tmp625.YY45).val=yy1;_tmp625;});}static char _tmp1F3[12U]="attribute_t";
# 1119 "parse.y"
static void*Cyc_yyget_YY46(union Cyc_YYSTYPE*yy1){
# 1164 "parse.y"
static struct _fat_ptr s={_tmp1F3,_tmp1F3,_tmp1F3 + 12U};
union Cyc_YYSTYPE*_tmp1F1=yy1;void*_tmp1F2;if((((union Cyc_YYSTYPE*)_tmp1F1)->YY46).tag == 54){_LL1: _tmp1F2=(_tmp1F1->YY46).val;_LL2: {void*yy=_tmp1F2;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY46(void*yy1){
# 1170 "parse.y"
return({union Cyc_YYSTYPE _tmp626;(_tmp626.YY46).tag=54U,(_tmp626.YY46).val=yy1;_tmp626;});}static char _tmp1F6[12U]="enumfield_t";
# 1119 "parse.y"
static struct Cyc_Absyn_Enumfield*Cyc_yyget_YY47(union Cyc_YYSTYPE*yy1){
# 1165 "parse.y"
static struct _fat_ptr s={_tmp1F6,_tmp1F6,_tmp1F6 + 12U};
union Cyc_YYSTYPE*_tmp1F4=yy1;struct Cyc_Absyn_Enumfield*_tmp1F5;if((((union Cyc_YYSTYPE*)_tmp1F4)->YY47).tag == 55){_LL1: _tmp1F5=(_tmp1F4->YY47).val;_LL2: {struct Cyc_Absyn_Enumfield*yy=_tmp1F5;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY47(struct Cyc_Absyn_Enumfield*yy1){
# 1171 "parse.y"
return({union Cyc_YYSTYPE _tmp627;(_tmp627.YY47).tag=55U,(_tmp627.YY47).val=yy1;_tmp627;});}static char _tmp1F9[23U]="list_t<enumfield_t,`H>";
# 1119 "parse.y"
static struct Cyc_List_List*Cyc_yyget_YY48(union Cyc_YYSTYPE*yy1){
# 1166 "parse.y"
static struct _fat_ptr s={_tmp1F9,_tmp1F9,_tmp1F9 + 23U};
union Cyc_YYSTYPE*_tmp1F7=yy1;struct Cyc_List_List*_tmp1F8;if((((union Cyc_YYSTYPE*)_tmp1F7)->YY48).tag == 56){_LL1: _tmp1F8=(_tmp1F7->YY48).val;_LL2: {struct Cyc_List_List*yy=_tmp1F8;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY48(struct Cyc_List_List*yy1){
# 1172 "parse.y"
return({union Cyc_YYSTYPE _tmp628;(_tmp628.YY48).tag=56U,(_tmp628.YY48).val=yy1;_tmp628;});}static char _tmp1FC[11U]="type_opt_t";
# 1119 "parse.y"
static void*Cyc_yyget_YY49(union Cyc_YYSTYPE*yy1){
# 1167 "parse.y"
static struct _fat_ptr s={_tmp1FC,_tmp1FC,_tmp1FC + 11U};
union Cyc_YYSTYPE*_tmp1FA=yy1;void*_tmp1FB;if((((union Cyc_YYSTYPE*)_tmp1FA)->YY49).tag == 57){_LL1: _tmp1FB=(_tmp1FA->YY49).val;_LL2: {void*yy=_tmp1FB;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY49(void*yy1){
# 1173 "parse.y"
return({union Cyc_YYSTYPE _tmp629;(_tmp629.YY49).tag=57U,(_tmp629.YY49).val=yy1;_tmp629;});}static char _tmp1FF[31U]="list_t<$(type_t,type_t)@`H,`H>";
# 1119 "parse.y"
static struct Cyc_List_List*Cyc_yyget_YY50(union Cyc_YYSTYPE*yy1){
# 1168 "parse.y"
static struct _fat_ptr s={_tmp1FF,_tmp1FF,_tmp1FF + 31U};
union Cyc_YYSTYPE*_tmp1FD=yy1;struct Cyc_List_List*_tmp1FE;if((((union Cyc_YYSTYPE*)_tmp1FD)->YY50).tag == 58){_LL1: _tmp1FE=(_tmp1FD->YY50).val;_LL2: {struct Cyc_List_List*yy=_tmp1FE;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY50(struct Cyc_List_List*yy1){
# 1174 "parse.y"
return({union Cyc_YYSTYPE _tmp62A;(_tmp62A.YY50).tag=58U,(_tmp62A.YY50).val=yy1;_tmp62A;});}static char _tmp202[11U]="booltype_t";
# 1119 "parse.y"
static void*Cyc_yyget_YY51(union Cyc_YYSTYPE*yy1){
# 1169 "parse.y"
static struct _fat_ptr s={_tmp202,_tmp202,_tmp202 + 11U};
union Cyc_YYSTYPE*_tmp200=yy1;void*_tmp201;if((((union Cyc_YYSTYPE*)_tmp200)->YY51).tag == 59){_LL1: _tmp201=(_tmp200->YY51).val;_LL2: {void*yy=_tmp201;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY51(void*yy1){
# 1175 "parse.y"
return({union Cyc_YYSTYPE _tmp62B;(_tmp62B.YY51).tag=59U,(_tmp62B.YY51).val=yy1;_tmp62B;});}static char _tmp205[45U]="list_t<$(Position::seg_t,qvar_t,bool)@`H,`H>";
# 1119 "parse.y"
static struct Cyc_List_List*Cyc_yyget_YY52(union Cyc_YYSTYPE*yy1){
# 1170 "parse.y"
static struct _fat_ptr s={_tmp205,_tmp205,_tmp205 + 45U};
union Cyc_YYSTYPE*_tmp203=yy1;struct Cyc_List_List*_tmp204;if((((union Cyc_YYSTYPE*)_tmp203)->YY52).tag == 60){_LL1: _tmp204=(_tmp203->YY52).val;_LL2: {struct Cyc_List_List*yy=_tmp204;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY52(struct Cyc_List_List*yy1){
# 1176 "parse.y"
return({union Cyc_YYSTYPE _tmp62C;(_tmp62C.YY52).tag=60U,(_tmp62C.YY52).val=yy1;_tmp62C;});}static char _tmp208[58U]="$(list_t<$(Position::seg_t,qvar_t,bool)@`H,`H>, seg_t)@`H";
# 1119 "parse.y"
static struct _tuple28*Cyc_yyget_YY53(union Cyc_YYSTYPE*yy1){
# 1171 "parse.y"
static struct _fat_ptr s={_tmp208,_tmp208,_tmp208 + 58U};
union Cyc_YYSTYPE*_tmp206=yy1;struct _tuple28*_tmp207;if((((union Cyc_YYSTYPE*)_tmp206)->YY53).tag == 61){_LL1: _tmp207=(_tmp206->YY53).val;_LL2: {struct _tuple28*yy=_tmp207;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY53(struct _tuple28*yy1){
# 1177 "parse.y"
return({union Cyc_YYSTYPE _tmp62D;(_tmp62D.YY53).tag=61U,(_tmp62D.YY53).val=yy1;_tmp62D;});}static char _tmp20B[18U]="list_t<qvar_t,`H>";
# 1119 "parse.y"
static struct Cyc_List_List*Cyc_yyget_YY54(union Cyc_YYSTYPE*yy1){
# 1172 "parse.y"
static struct _fat_ptr s={_tmp20B,_tmp20B,_tmp20B + 18U};
union Cyc_YYSTYPE*_tmp209=yy1;struct Cyc_List_List*_tmp20A;if((((union Cyc_YYSTYPE*)_tmp209)->YY54).tag == 62){_LL1: _tmp20A=(_tmp209->YY54).val;_LL2: {struct Cyc_List_List*yy=_tmp20A;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY54(struct Cyc_List_List*yy1){
# 1178 "parse.y"
return({union Cyc_YYSTYPE _tmp62E;(_tmp62E.YY54).tag=62U,(_tmp62E.YY54).val=yy1;_tmp62E;});}static char _tmp20E[20U]="pointer_qual_t<`yy>";
# 1119 "parse.y"
static void*Cyc_yyget_YY55(union Cyc_YYSTYPE*yy1){
# 1173 "parse.y"
static struct _fat_ptr s={_tmp20E,_tmp20E,_tmp20E + 20U};
union Cyc_YYSTYPE*_tmp20C=yy1;void*_tmp20D;if((((union Cyc_YYSTYPE*)_tmp20C)->YY55).tag == 63){_LL1: _tmp20D=(_tmp20C->YY55).val;_LL2: {void*yy=_tmp20D;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY55(void*yy1){
# 1179 "parse.y"
return({union Cyc_YYSTYPE _tmp62F;(_tmp62F.YY55).tag=63U,(_tmp62F.YY55).val=yy1;_tmp62F;});}static char _tmp211[21U]="pointer_quals_t<`yy>";
# 1119 "parse.y"
static struct Cyc_List_List*Cyc_yyget_YY56(union Cyc_YYSTYPE*yy1){
# 1174 "parse.y"
static struct _fat_ptr s={_tmp211,_tmp211,_tmp211 + 21U};
union Cyc_YYSTYPE*_tmp20F=yy1;struct Cyc_List_List*_tmp210;if((((union Cyc_YYSTYPE*)_tmp20F)->YY56).tag == 64){_LL1: _tmp210=(_tmp20F->YY56).val;_LL2: {struct Cyc_List_List*yy=_tmp210;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY56(struct Cyc_List_List*yy1){
# 1180 "parse.y"
return({union Cyc_YYSTYPE _tmp630;(_tmp630.YY56).tag=64U,(_tmp630.YY56).val=yy1;_tmp630;});}static char _tmp214[10U]="exp_opt_t";
# 1119 "parse.y"
static struct Cyc_Absyn_Exp*Cyc_yyget_YY57(union Cyc_YYSTYPE*yy1){
# 1175 "parse.y"
static struct _fat_ptr s={_tmp214,_tmp214,_tmp214 + 10U};
union Cyc_YYSTYPE*_tmp212=yy1;struct Cyc_Absyn_Exp*_tmp213;if((((union Cyc_YYSTYPE*)_tmp212)->YY57).tag == 65){_LL1: _tmp213=(_tmp212->YY57).val;_LL2: {struct Cyc_Absyn_Exp*yy=_tmp213;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY57(struct Cyc_Absyn_Exp*yy1){
# 1181 "parse.y"
return({union Cyc_YYSTYPE _tmp631;(_tmp631.YY57).tag=65U,(_tmp631.YY57).val=yy1;_tmp631;});}static char _tmp217[10U]="raw_exp_t";
# 1119 "parse.y"
static void*Cyc_yyget_YY58(union Cyc_YYSTYPE*yy1){
# 1176 "parse.y"
static struct _fat_ptr s={_tmp217,_tmp217,_tmp217 + 10U};
union Cyc_YYSTYPE*_tmp215=yy1;void*_tmp216;if((((union Cyc_YYSTYPE*)_tmp215)->YY58).tag == 66){_LL1: _tmp216=(_tmp215->YY58).val;_LL2: {void*yy=_tmp216;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY58(void*yy1){
# 1182 "parse.y"
return({union Cyc_YYSTYPE _tmp632;(_tmp632.YY58).tag=66U,(_tmp632.YY58).val=yy1;_tmp632;});}static char _tmp21A[112U]="$(list_t<$(string_t<`H>, exp_t)@`H, `H>, list_t<$(string_t<`H>, exp_t)@`H, `H>, list_t<string_t<`H>@`H, `H>)@`H";
# 1119 "parse.y"
static struct _tuple29*Cyc_yyget_YY59(union Cyc_YYSTYPE*yy1){
# 1178 "parse.y"
static struct _fat_ptr s={_tmp21A,_tmp21A,_tmp21A + 112U};
union Cyc_YYSTYPE*_tmp218=yy1;struct _tuple29*_tmp219;if((((union Cyc_YYSTYPE*)_tmp218)->YY59).tag == 67){_LL1: _tmp219=(_tmp218->YY59).val;_LL2: {struct _tuple29*yy=_tmp219;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY59(struct _tuple29*yy1){
# 1184 "parse.y"
return({union Cyc_YYSTYPE _tmp633;(_tmp633.YY59).tag=67U,(_tmp633.YY59).val=yy1;_tmp633;});}static char _tmp21D[73U]="$(list_t<$(string_t<`H>, exp_t)@`H, `H>, list_t<string_t<`H>@`H, `H>)@`H";
# 1119 "parse.y"
static struct _tuple30*Cyc_yyget_YY60(union Cyc_YYSTYPE*yy1){
# 1179 "parse.y"
static struct _fat_ptr s={_tmp21D,_tmp21D,_tmp21D + 73U};
union Cyc_YYSTYPE*_tmp21B=yy1;struct _tuple30*_tmp21C;if((((union Cyc_YYSTYPE*)_tmp21B)->YY60).tag == 68){_LL1: _tmp21C=(_tmp21B->YY60).val;_LL2: {struct _tuple30*yy=_tmp21C;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY60(struct _tuple30*yy1){
# 1185 "parse.y"
return({union Cyc_YYSTYPE _tmp634;(_tmp634.YY60).tag=68U,(_tmp634.YY60).val=yy1;_tmp634;});}static char _tmp220[28U]="list_t<string_t<`H>@`H, `H>";
# 1119 "parse.y"
static struct Cyc_List_List*Cyc_yyget_YY61(union Cyc_YYSTYPE*yy1){
# 1180 "parse.y"
static struct _fat_ptr s={_tmp220,_tmp220,_tmp220 + 28U};
union Cyc_YYSTYPE*_tmp21E=yy1;struct Cyc_List_List*_tmp21F;if((((union Cyc_YYSTYPE*)_tmp21E)->YY61).tag == 69){_LL1: _tmp21F=(_tmp21E->YY61).val;_LL2: {struct Cyc_List_List*yy=_tmp21F;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY61(struct Cyc_List_List*yy1){
# 1186 "parse.y"
return({union Cyc_YYSTYPE _tmp635;(_tmp635.YY61).tag=69U,(_tmp635.YY61).val=yy1;_tmp635;});}static char _tmp223[38U]="list_t<$(string_t<`H>, exp_t)@`H, `H>";
# 1119 "parse.y"
static struct Cyc_List_List*Cyc_yyget_YY62(union Cyc_YYSTYPE*yy1){
# 1181 "parse.y"
static struct _fat_ptr s={_tmp223,_tmp223,_tmp223 + 38U};
union Cyc_YYSTYPE*_tmp221=yy1;struct Cyc_List_List*_tmp222;if((((union Cyc_YYSTYPE*)_tmp221)->YY62).tag == 70){_LL1: _tmp222=(_tmp221->YY62).val;_LL2: {struct Cyc_List_List*yy=_tmp222;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY62(struct Cyc_List_List*yy1){
# 1187 "parse.y"
return({union Cyc_YYSTYPE _tmp636;(_tmp636.YY62).tag=70U,(_tmp636.YY62).val=yy1;_tmp636;});}static char _tmp226[26U]="$(string_t<`H>, exp_t)@`H";
# 1119 "parse.y"
static struct _tuple31*Cyc_yyget_YY63(union Cyc_YYSTYPE*yy1){
# 1182 "parse.y"
static struct _fat_ptr s={_tmp226,_tmp226,_tmp226 + 26U};
union Cyc_YYSTYPE*_tmp224=yy1;struct _tuple31*_tmp225;if((((union Cyc_YYSTYPE*)_tmp224)->YY63).tag == 71){_LL1: _tmp225=(_tmp224->YY63).val;_LL2: {struct _tuple31*yy=_tmp225;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY63(struct _tuple31*yy1){
# 1188 "parse.y"
return({union Cyc_YYSTYPE _tmp637;(_tmp637.YY63).tag=71U,(_tmp637.YY63).val=yy1;_tmp637;});}static char _tmp229[18U]="list_t<type_t,`H>";
# 1119 "parse.y"
static struct Cyc_List_List*Cyc_yyget_YY64(union Cyc_YYSTYPE*yy1){
# 1184 "parse.y"
static struct _fat_ptr s={_tmp229,_tmp229,_tmp229 + 18U};
union Cyc_YYSTYPE*_tmp227=yy1;struct Cyc_List_List*_tmp228;if((((union Cyc_YYSTYPE*)_tmp227)->YY64).tag == 72){_LL1: _tmp228=(_tmp227->YY64).val;_LL2: {struct Cyc_List_List*yy=_tmp228;
return yy;}}else{_LL3: _LL4:
 if(Cyc_beforedie != 0)((void(*)())_check_null(Cyc_beforedie))();Cyc_yythrowfail(s);}_LL0:;}
# 1119 "parse.y"
static union Cyc_YYSTYPE Cyc_YY64(struct Cyc_List_List*yy1){
# 1190 "parse.y"
return({union Cyc_YYSTYPE _tmp638;(_tmp638.YY64).tag=72U,(_tmp638.YY64).val=yy1;_tmp638;});}struct Cyc_Yyltype{int timestamp;int first_line;int first_column;int last_line;int last_column;};
# 1206
struct Cyc_Yyltype Cyc_yynewloc(){
return({struct Cyc_Yyltype _tmp639;_tmp639.timestamp=0,_tmp639.first_line=0,_tmp639.first_column=0,_tmp639.last_line=0,_tmp639.last_column=0;_tmp639;});}
# 1206
struct Cyc_Yyltype Cyc_yylloc={0,0,0,0,0};
# 1220 "parse.y"
static short Cyc_yytranslate[382U]={0,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,153,2,2,138,151,148,2,135,136,131,145,130,149,140,150,2,2,2,2,2,2,2,2,2,2,139,127,133,132,134,144,143,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,141,2,142,147,137,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,128,146,129,152,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63,64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,81,82,83,84,85,86,87,88,89,90,91,92,93,94,95,96,97,98,99,100,101,102,103,104,105,106,107,108,109,110,111,112,113,114,115,116,117,118,119,120,121,122,123,124,125,126};static char _tmp22A[2U]="$";static char _tmp22B[6U]="error";static char _tmp22C[12U]="$undefined.";static char _tmp22D[5U]="AUTO";static char _tmp22E[9U]="REGISTER";static char _tmp22F[7U]="STATIC";static char _tmp230[7U]="EXTERN";static char _tmp231[8U]="TYPEDEF";static char _tmp232[5U]="VOID";static char _tmp233[5U]="CHAR";static char _tmp234[6U]="SHORT";static char _tmp235[4U]="INT";static char _tmp236[5U]="LONG";static char _tmp237[6U]="FLOAT";static char _tmp238[7U]="DOUBLE";static char _tmp239[7U]="SIGNED";static char _tmp23A[9U]="UNSIGNED";static char _tmp23B[6U]="CONST";static char _tmp23C[9U]="VOLATILE";static char _tmp23D[9U]="RESTRICT";static char _tmp23E[7U]="STRUCT";static char _tmp23F[6U]="UNION";static char _tmp240[5U]="CASE";static char _tmp241[8U]="DEFAULT";static char _tmp242[7U]="INLINE";static char _tmp243[7U]="SIZEOF";static char _tmp244[9U]="OFFSETOF";static char _tmp245[3U]="IF";static char _tmp246[5U]="ELSE";static char _tmp247[7U]="SWITCH";static char _tmp248[6U]="WHILE";static char _tmp249[3U]="DO";static char _tmp24A[4U]="FOR";static char _tmp24B[5U]="GOTO";static char _tmp24C[9U]="CONTINUE";static char _tmp24D[6U]="BREAK";static char _tmp24E[7U]="RETURN";static char _tmp24F[5U]="ENUM";static char _tmp250[7U]="TYPEOF";static char _tmp251[16U]="BUILTIN_VA_LIST";static char _tmp252[10U]="EXTENSION";static char _tmp253[8U]="NULL_kw";static char _tmp254[4U]="LET";static char _tmp255[6U]="THROW";static char _tmp256[4U]="TRY";static char _tmp257[6U]="CATCH";static char _tmp258[7U]="EXPORT";static char _tmp259[9U]="OVERRIDE";static char _tmp25A[5U]="HIDE";static char _tmp25B[4U]="NEW";static char _tmp25C[9U]="ABSTRACT";static char _tmp25D[9U]="FALLTHRU";static char _tmp25E[6U]="USING";static char _tmp25F[10U]="NAMESPACE";static char _tmp260[9U]="DATATYPE";static char _tmp261[6U]="SPAWN";static char _tmp262[7U]="MALLOC";static char _tmp263[8U]="RMALLOC";static char _tmp264[15U]="RMALLOC_INLINE";static char _tmp265[7U]="CALLOC";static char _tmp266[8U]="RCALLOC";static char _tmp267[5U]="SWAP";static char _tmp268[9U]="REGION_T";static char _tmp269[6U]="TAG_T";static char _tmp26A[7U]="REGION";static char _tmp26B[5U]="RNEW";static char _tmp26C[8U]="REGIONS";static char _tmp26D[7U]="PORTON";static char _tmp26E[8U]="PORTOFF";static char _tmp26F[7U]="PRAGMA";static char _tmp270[10U]="TEMPESTON";static char _tmp271[11U]="TEMPESTOFF";static char _tmp272[8U]="NUMELTS";static char _tmp273[8U]="VALUEOF";static char _tmp274[10U]="VALUEOF_T";static char _tmp275[9U]="TAGCHECK";static char _tmp276[13U]="NUMELTS_QUAL";static char _tmp277[10U]="THIN_QUAL";static char _tmp278[9U]="FAT_QUAL";static char _tmp279[13U]="NOTNULL_QUAL";static char _tmp27A[14U]="NULLABLE_QUAL";static char _tmp27B[14U]="REQUIRES_QUAL";static char _tmp27C[13U]="ENSURES_QUAL";static char _tmp27D[13U]="IEFFECT_QUAL";static char _tmp27E[13U]="OEFFECT_QUAL";static char _tmp27F[12U]="REGION_QUAL";static char _tmp280[16U]="NOZEROTERM_QUAL";static char _tmp281[14U]="ZEROTERM_QUAL";static char _tmp282[12U]="TAGGED_QUAL";static char _tmp283[16U]="EXTENSIBLE_QUAL";static char _tmp284[7U]="PTR_OP";static char _tmp285[7U]="INC_OP";static char _tmp286[7U]="DEC_OP";static char _tmp287[8U]="LEFT_OP";static char _tmp288[9U]="RIGHT_OP";static char _tmp289[6U]="LE_OP";static char _tmp28A[6U]="GE_OP";static char _tmp28B[6U]="EQ_OP";static char _tmp28C[6U]="NE_OP";static char _tmp28D[7U]="AND_OP";static char _tmp28E[6U]="OR_OP";static char _tmp28F[11U]="MUL_ASSIGN";static char _tmp290[11U]="DIV_ASSIGN";static char _tmp291[11U]="MOD_ASSIGN";static char _tmp292[11U]="ADD_ASSIGN";static char _tmp293[11U]="SUB_ASSIGN";static char _tmp294[12U]="LEFT_ASSIGN";static char _tmp295[13U]="RIGHT_ASSIGN";static char _tmp296[11U]="AND_ASSIGN";static char _tmp297[11U]="XOR_ASSIGN";static char _tmp298[10U]="OR_ASSIGN";static char _tmp299[9U]="ELLIPSIS";static char _tmp29A[11U]="LEFT_RIGHT";static char _tmp29B[12U]="COLON_COLON";static char _tmp29C[11U]="IDENTIFIER";static char _tmp29D[17U]="INTEGER_CONSTANT";static char _tmp29E[7U]="STRING";static char _tmp29F[8U]="WSTRING";static char _tmp2A0[19U]="CHARACTER_CONSTANT";static char _tmp2A1[20U]="WCHARACTER_CONSTANT";static char _tmp2A2[18U]="FLOATING_CONSTANT";static char _tmp2A3[9U]="TYPE_VAR";static char _tmp2A4[13U]="TYPEDEF_NAME";static char _tmp2A5[16U]="QUAL_IDENTIFIER";static char _tmp2A6[18U]="QUAL_TYPEDEF_NAME";static char _tmp2A7[10U]="ATTRIBUTE";static char _tmp2A8[8U]="ASM_TOK";static char _tmp2A9[4U]="';'";static char _tmp2AA[4U]="'{'";static char _tmp2AB[4U]="'}'";static char _tmp2AC[4U]="','";static char _tmp2AD[4U]="'*'";static char _tmp2AE[4U]="'='";static char _tmp2AF[4U]="'<'";static char _tmp2B0[4U]="'>'";static char _tmp2B1[4U]="'('";static char _tmp2B2[4U]="')'";static char _tmp2B3[4U]="'_'";static char _tmp2B4[4U]="'$'";static char _tmp2B5[4U]="':'";static char _tmp2B6[4U]="'.'";static char _tmp2B7[4U]="'['";static char _tmp2B8[4U]="']'";static char _tmp2B9[4U]="'@'";static char _tmp2BA[4U]="'?'";static char _tmp2BB[4U]="'+'";static char _tmp2BC[4U]="'|'";static char _tmp2BD[4U]="'^'";static char _tmp2BE[4U]="'&'";static char _tmp2BF[4U]="'-'";static char _tmp2C0[4U]="'/'";static char _tmp2C1[4U]="'%'";static char _tmp2C2[4U]="'~'";static char _tmp2C3[4U]="'!'";static char _tmp2C4[5U]="prog";static char _tmp2C5[17U]="translation_unit";static char _tmp2C6[18U]="tempest_on_action";static char _tmp2C7[19U]="tempest_off_action";static char _tmp2C8[16U]="extern_c_action";static char _tmp2C9[13U]="end_extern_c";static char _tmp2CA[14U]="hide_list_opt";static char _tmp2CB[17U]="hide_list_values";static char _tmp2CC[16U]="export_list_opt";static char _tmp2CD[12U]="export_list";static char _tmp2CE[19U]="export_list_values";static char _tmp2CF[13U]="override_opt";static char _tmp2D0[21U]="external_declaration";static char _tmp2D1[15U]="optional_comma";static char _tmp2D2[20U]="function_definition";static char _tmp2D3[21U]="function_definition2";static char _tmp2D4[13U]="using_action";static char _tmp2D5[15U]="unusing_action";static char _tmp2D6[17U]="namespace_action";static char _tmp2D7[19U]="unnamespace_action";static char _tmp2D8[12U]="declaration";static char _tmp2D9[17U]="declaration_list";static char _tmp2DA[23U]="declaration_specifiers";static char _tmp2DB[24U]="storage_class_specifier";static char _tmp2DC[15U]="attributes_opt";static char _tmp2DD[11U]="attributes";static char _tmp2DE[15U]="attribute_list";static char _tmp2DF[10U]="attribute";static char _tmp2E0[15U]="type_specifier";static char _tmp2E1[25U]="type_specifier_notypedef";static char _tmp2E2[5U]="kind";static char _tmp2E3[15U]="type_qualifier";static char _tmp2E4[15U]="enum_specifier";static char _tmp2E5[11U]="enum_field";static char _tmp2E6[22U]="enum_declaration_list";static char _tmp2E7[26U]="struct_or_union_specifier";static char _tmp2E8[16U]="type_params_opt";static char _tmp2E9[16U]="struct_or_union";static char _tmp2EA[24U]="struct_declaration_list";static char _tmp2EB[25U]="struct_declaration_list0";static char _tmp2EC[21U]="init_declarator_list";static char _tmp2ED[22U]="init_declarator_list0";static char _tmp2EE[16U]="init_declarator";static char _tmp2EF[19U]="struct_declaration";static char _tmp2F0[25U]="specifier_qualifier_list";static char _tmp2F1[35U]="notypedef_specifier_qualifier_list";static char _tmp2F2[23U]="struct_declarator_list";static char _tmp2F3[24U]="struct_declarator_list0";static char _tmp2F4[18U]="struct_declarator";static char _tmp2F5[20U]="requires_clause_opt";static char _tmp2F6[9U]="eff_list";static char _tmp2F7[19U]="ieffect_clause_opt";static char _tmp2F8[19U]="oeffect_clause_opt";static char _tmp2F9[19U]="ensures_clause_opt";static char _tmp2FA[19U]="datatype_specifier";static char _tmp2FB[14U]="qual_datatype";static char _tmp2FC[19U]="datatypefield_list";static char _tmp2FD[20U]="datatypefield_scope";static char _tmp2FE[14U]="datatypefield";static char _tmp2FF[11U]="declarator";static char _tmp300[23U]="declarator_withtypedef";static char _tmp301[18U]="direct_declarator";static char _tmp302[30U]="direct_declarator_withtypedef";static char _tmp303[8U]="pointer";static char _tmp304[12U]="one_pointer";static char _tmp305[14U]="pointer_quals";static char _tmp306[13U]="pointer_qual";static char _tmp307[23U]="pointer_null_and_bound";static char _tmp308[14U]="pointer_bound";static char _tmp309[18U]="zeroterm_qual_opt";static char _tmp30A[8U]="rgn_opt";static char _tmp30B[11U]="tqual_list";static char _tmp30C[20U]="parameter_type_list";static char _tmp30D[9U]="type_var";static char _tmp30E[16U]="optional_effect";static char _tmp30F[19U]="optional_rgn_order";static char _tmp310[10U]="rgn_order";static char _tmp311[16U]="optional_inject";static char _tmp312[11U]="effect_set";static char _tmp313[14U]="atomic_effect";static char _tmp314[11U]="region_set";static char _tmp315[15U]="parameter_list";static char _tmp316[22U]="parameter_declaration";static char _tmp317[16U]="identifier_list";static char _tmp318[17U]="identifier_list0";static char _tmp319[12U]="initializer";static char _tmp31A[18U]="array_initializer";static char _tmp31B[17U]="initializer_list";static char _tmp31C[12U]="designation";static char _tmp31D[16U]="designator_list";static char _tmp31E[11U]="designator";static char _tmp31F[10U]="type_name";static char _tmp320[14U]="any_type_name";static char _tmp321[15U]="type_name_list";static char _tmp322[20U]="abstract_declarator";static char _tmp323[27U]="direct_abstract_declarator";static char _tmp324[10U]="statement";static char _tmp325[18U]="labeled_statement";static char _tmp326[21U]="expression_statement";static char _tmp327[19U]="compound_statement";static char _tmp328[16U]="block_item_list";static char _tmp329[20U]="selection_statement";static char _tmp32A[15U]="switch_clauses";static char _tmp32B[20U]="iteration_statement";static char _tmp32C[15U]="jump_statement";static char _tmp32D[12U]="exp_pattern";static char _tmp32E[20U]="conditional_pattern";static char _tmp32F[19U]="logical_or_pattern";static char _tmp330[20U]="logical_and_pattern";static char _tmp331[21U]="inclusive_or_pattern";static char _tmp332[21U]="exclusive_or_pattern";static char _tmp333[12U]="and_pattern";static char _tmp334[17U]="equality_pattern";static char _tmp335[19U]="relational_pattern";static char _tmp336[14U]="shift_pattern";static char _tmp337[17U]="additive_pattern";static char _tmp338[23U]="multiplicative_pattern";static char _tmp339[13U]="cast_pattern";static char _tmp33A[14U]="unary_pattern";static char _tmp33B[16U]="postfix_pattern";static char _tmp33C[16U]="primary_pattern";static char _tmp33D[8U]="pattern";static char _tmp33E[19U]="tuple_pattern_list";static char _tmp33F[20U]="tuple_pattern_list0";static char _tmp340[14U]="field_pattern";static char _tmp341[19U]="field_pattern_list";static char _tmp342[20U]="field_pattern_list0";static char _tmp343[11U]="expression";static char _tmp344[22U]="assignment_expression";static char _tmp345[20U]="assignment_operator";static char _tmp346[23U]="conditional_expression";static char _tmp347[20U]="constant_expression";static char _tmp348[22U]="logical_or_expression";static char _tmp349[23U]="logical_and_expression";static char _tmp34A[24U]="inclusive_or_expression";static char _tmp34B[24U]="exclusive_or_expression";static char _tmp34C[15U]="and_expression";static char _tmp34D[20U]="equality_expression";static char _tmp34E[22U]="relational_expression";static char _tmp34F[17U]="shift_expression";static char _tmp350[20U]="additive_expression";static char _tmp351[26U]="multiplicative_expression";static char _tmp352[16U]="cast_expression";static char _tmp353[17U]="unary_expression";static char _tmp354[9U]="asm_expr";static char _tmp355[13U]="volatile_opt";static char _tmp356[12U]="asm_out_opt";static char _tmp357[12U]="asm_outlist";static char _tmp358[11U]="asm_in_opt";static char _tmp359[11U]="asm_inlist";static char _tmp35A[11U]="asm_io_elt";static char _tmp35B[16U]="asm_clobber_opt";static char _tmp35C[17U]="asm_clobber_list";static char _tmp35D[15U]="unary_operator";static char _tmp35E[19U]="postfix_expression";static char _tmp35F[17U]="field_expression";static char _tmp360[19U]="primary_expression";static char _tmp361[25U]="argument_expression_list";static char _tmp362[26U]="argument_expression_list0";static char _tmp363[9U]="constant";static char _tmp364[20U]="qual_opt_identifier";static char _tmp365[17U]="qual_opt_typedef";static char _tmp366[18U]="struct_union_name";static char _tmp367[11U]="field_name";static char _tmp368[12U]="right_angle";
# 1597 "parse.y"
static struct _fat_ptr Cyc_yytname[319U]={{_tmp22A,_tmp22A,_tmp22A + 2U},{_tmp22B,_tmp22B,_tmp22B + 6U},{_tmp22C,_tmp22C,_tmp22C + 12U},{_tmp22D,_tmp22D,_tmp22D + 5U},{_tmp22E,_tmp22E,_tmp22E + 9U},{_tmp22F,_tmp22F,_tmp22F + 7U},{_tmp230,_tmp230,_tmp230 + 7U},{_tmp231,_tmp231,_tmp231 + 8U},{_tmp232,_tmp232,_tmp232 + 5U},{_tmp233,_tmp233,_tmp233 + 5U},{_tmp234,_tmp234,_tmp234 + 6U},{_tmp235,_tmp235,_tmp235 + 4U},{_tmp236,_tmp236,_tmp236 + 5U},{_tmp237,_tmp237,_tmp237 + 6U},{_tmp238,_tmp238,_tmp238 + 7U},{_tmp239,_tmp239,_tmp239 + 7U},{_tmp23A,_tmp23A,_tmp23A + 9U},{_tmp23B,_tmp23B,_tmp23B + 6U},{_tmp23C,_tmp23C,_tmp23C + 9U},{_tmp23D,_tmp23D,_tmp23D + 9U},{_tmp23E,_tmp23E,_tmp23E + 7U},{_tmp23F,_tmp23F,_tmp23F + 6U},{_tmp240,_tmp240,_tmp240 + 5U},{_tmp241,_tmp241,_tmp241 + 8U},{_tmp242,_tmp242,_tmp242 + 7U},{_tmp243,_tmp243,_tmp243 + 7U},{_tmp244,_tmp244,_tmp244 + 9U},{_tmp245,_tmp245,_tmp245 + 3U},{_tmp246,_tmp246,_tmp246 + 5U},{_tmp247,_tmp247,_tmp247 + 7U},{_tmp248,_tmp248,_tmp248 + 6U},{_tmp249,_tmp249,_tmp249 + 3U},{_tmp24A,_tmp24A,_tmp24A + 4U},{_tmp24B,_tmp24B,_tmp24B + 5U},{_tmp24C,_tmp24C,_tmp24C + 9U},{_tmp24D,_tmp24D,_tmp24D + 6U},{_tmp24E,_tmp24E,_tmp24E + 7U},{_tmp24F,_tmp24F,_tmp24F + 5U},{_tmp250,_tmp250,_tmp250 + 7U},{_tmp251,_tmp251,_tmp251 + 16U},{_tmp252,_tmp252,_tmp252 + 10U},{_tmp253,_tmp253,_tmp253 + 8U},{_tmp254,_tmp254,_tmp254 + 4U},{_tmp255,_tmp255,_tmp255 + 6U},{_tmp256,_tmp256,_tmp256 + 4U},{_tmp257,_tmp257,_tmp257 + 6U},{_tmp258,_tmp258,_tmp258 + 7U},{_tmp259,_tmp259,_tmp259 + 9U},{_tmp25A,_tmp25A,_tmp25A + 5U},{_tmp25B,_tmp25B,_tmp25B + 4U},{_tmp25C,_tmp25C,_tmp25C + 9U},{_tmp25D,_tmp25D,_tmp25D + 9U},{_tmp25E,_tmp25E,_tmp25E + 6U},{_tmp25F,_tmp25F,_tmp25F + 10U},{_tmp260,_tmp260,_tmp260 + 9U},{_tmp261,_tmp261,_tmp261 + 6U},{_tmp262,_tmp262,_tmp262 + 7U},{_tmp263,_tmp263,_tmp263 + 8U},{_tmp264,_tmp264,_tmp264 + 15U},{_tmp265,_tmp265,_tmp265 + 7U},{_tmp266,_tmp266,_tmp266 + 8U},{_tmp267,_tmp267,_tmp267 + 5U},{_tmp268,_tmp268,_tmp268 + 9U},{_tmp269,_tmp269,_tmp269 + 6U},{_tmp26A,_tmp26A,_tmp26A + 7U},{_tmp26B,_tmp26B,_tmp26B + 5U},{_tmp26C,_tmp26C,_tmp26C + 8U},{_tmp26D,_tmp26D,_tmp26D + 7U},{_tmp26E,_tmp26E,_tmp26E + 8U},{_tmp26F,_tmp26F,_tmp26F + 7U},{_tmp270,_tmp270,_tmp270 + 10U},{_tmp271,_tmp271,_tmp271 + 11U},{_tmp272,_tmp272,_tmp272 + 8U},{_tmp273,_tmp273,_tmp273 + 8U},{_tmp274,_tmp274,_tmp274 + 10U},{_tmp275,_tmp275,_tmp275 + 9U},{_tmp276,_tmp276,_tmp276 + 13U},{_tmp277,_tmp277,_tmp277 + 10U},{_tmp278,_tmp278,_tmp278 + 9U},{_tmp279,_tmp279,_tmp279 + 13U},{_tmp27A,_tmp27A,_tmp27A + 14U},{_tmp27B,_tmp27B,_tmp27B + 14U},{_tmp27C,_tmp27C,_tmp27C + 13U},{_tmp27D,_tmp27D,_tmp27D + 13U},{_tmp27E,_tmp27E,_tmp27E + 13U},{_tmp27F,_tmp27F,_tmp27F + 12U},{_tmp280,_tmp280,_tmp280 + 16U},{_tmp281,_tmp281,_tmp281 + 14U},{_tmp282,_tmp282,_tmp282 + 12U},{_tmp283,_tmp283,_tmp283 + 16U},{_tmp284,_tmp284,_tmp284 + 7U},{_tmp285,_tmp285,_tmp285 + 7U},{_tmp286,_tmp286,_tmp286 + 7U},{_tmp287,_tmp287,_tmp287 + 8U},{_tmp288,_tmp288,_tmp288 + 9U},{_tmp289,_tmp289,_tmp289 + 6U},{_tmp28A,_tmp28A,_tmp28A + 6U},{_tmp28B,_tmp28B,_tmp28B + 6U},{_tmp28C,_tmp28C,_tmp28C + 6U},{_tmp28D,_tmp28D,_tmp28D + 7U},{_tmp28E,_tmp28E,_tmp28E + 6U},{_tmp28F,_tmp28F,_tmp28F + 11U},{_tmp290,_tmp290,_tmp290 + 11U},{_tmp291,_tmp291,_tmp291 + 11U},{_tmp292,_tmp292,_tmp292 + 11U},{_tmp293,_tmp293,_tmp293 + 11U},{_tmp294,_tmp294,_tmp294 + 12U},{_tmp295,_tmp295,_tmp295 + 13U},{_tmp296,_tmp296,_tmp296 + 11U},{_tmp297,_tmp297,_tmp297 + 11U},{_tmp298,_tmp298,_tmp298 + 10U},{_tmp299,_tmp299,_tmp299 + 9U},{_tmp29A,_tmp29A,_tmp29A + 11U},{_tmp29B,_tmp29B,_tmp29B + 12U},{_tmp29C,_tmp29C,_tmp29C + 11U},{_tmp29D,_tmp29D,_tmp29D + 17U},{_tmp29E,_tmp29E,_tmp29E + 7U},{_tmp29F,_tmp29F,_tmp29F + 8U},{_tmp2A0,_tmp2A0,_tmp2A0 + 19U},{_tmp2A1,_tmp2A1,_tmp2A1 + 20U},{_tmp2A2,_tmp2A2,_tmp2A2 + 18U},{_tmp2A3,_tmp2A3,_tmp2A3 + 9U},{_tmp2A4,_tmp2A4,_tmp2A4 + 13U},{_tmp2A5,_tmp2A5,_tmp2A5 + 16U},{_tmp2A6,_tmp2A6,_tmp2A6 + 18U},{_tmp2A7,_tmp2A7,_tmp2A7 + 10U},{_tmp2A8,_tmp2A8,_tmp2A8 + 8U},{_tmp2A9,_tmp2A9,_tmp2A9 + 4U},{_tmp2AA,_tmp2AA,_tmp2AA + 4U},{_tmp2AB,_tmp2AB,_tmp2AB + 4U},{_tmp2AC,_tmp2AC,_tmp2AC + 4U},{_tmp2AD,_tmp2AD,_tmp2AD + 4U},{_tmp2AE,_tmp2AE,_tmp2AE + 4U},{_tmp2AF,_tmp2AF,_tmp2AF + 4U},{_tmp2B0,_tmp2B0,_tmp2B0 + 4U},{_tmp2B1,_tmp2B1,_tmp2B1 + 4U},{_tmp2B2,_tmp2B2,_tmp2B2 + 4U},{_tmp2B3,_tmp2B3,_tmp2B3 + 4U},{_tmp2B4,_tmp2B4,_tmp2B4 + 4U},{_tmp2B5,_tmp2B5,_tmp2B5 + 4U},{_tmp2B6,_tmp2B6,_tmp2B6 + 4U},{_tmp2B7,_tmp2B7,_tmp2B7 + 4U},{_tmp2B8,_tmp2B8,_tmp2B8 + 4U},{_tmp2B9,_tmp2B9,_tmp2B9 + 4U},{_tmp2BA,_tmp2BA,_tmp2BA + 4U},{_tmp2BB,_tmp2BB,_tmp2BB + 4U},{_tmp2BC,_tmp2BC,_tmp2BC + 4U},{_tmp2BD,_tmp2BD,_tmp2BD + 4U},{_tmp2BE,_tmp2BE,_tmp2BE + 4U},{_tmp2BF,_tmp2BF,_tmp2BF + 4U},{_tmp2C0,_tmp2C0,_tmp2C0 + 4U},{_tmp2C1,_tmp2C1,_tmp2C1 + 4U},{_tmp2C2,_tmp2C2,_tmp2C2 + 4U},{_tmp2C3,_tmp2C3,_tmp2C3 + 4U},{_tmp2C4,_tmp2C4,_tmp2C4 + 5U},{_tmp2C5,_tmp2C5,_tmp2C5 + 17U},{_tmp2C6,_tmp2C6,_tmp2C6 + 18U},{_tmp2C7,_tmp2C7,_tmp2C7 + 19U},{_tmp2C8,_tmp2C8,_tmp2C8 + 16U},{_tmp2C9,_tmp2C9,_tmp2C9 + 13U},{_tmp2CA,_tmp2CA,_tmp2CA + 14U},{_tmp2CB,_tmp2CB,_tmp2CB + 17U},{_tmp2CC,_tmp2CC,_tmp2CC + 16U},{_tmp2CD,_tmp2CD,_tmp2CD + 12U},{_tmp2CE,_tmp2CE,_tmp2CE + 19U},{_tmp2CF,_tmp2CF,_tmp2CF + 13U},{_tmp2D0,_tmp2D0,_tmp2D0 + 21U},{_tmp2D1,_tmp2D1,_tmp2D1 + 15U},{_tmp2D2,_tmp2D2,_tmp2D2 + 20U},{_tmp2D3,_tmp2D3,_tmp2D3 + 21U},{_tmp2D4,_tmp2D4,_tmp2D4 + 13U},{_tmp2D5,_tmp2D5,_tmp2D5 + 15U},{_tmp2D6,_tmp2D6,_tmp2D6 + 17U},{_tmp2D7,_tmp2D7,_tmp2D7 + 19U},{_tmp2D8,_tmp2D8,_tmp2D8 + 12U},{_tmp2D9,_tmp2D9,_tmp2D9 + 17U},{_tmp2DA,_tmp2DA,_tmp2DA + 23U},{_tmp2DB,_tmp2DB,_tmp2DB + 24U},{_tmp2DC,_tmp2DC,_tmp2DC + 15U},{_tmp2DD,_tmp2DD,_tmp2DD + 11U},{_tmp2DE,_tmp2DE,_tmp2DE + 15U},{_tmp2DF,_tmp2DF,_tmp2DF + 10U},{_tmp2E0,_tmp2E0,_tmp2E0 + 15U},{_tmp2E1,_tmp2E1,_tmp2E1 + 25U},{_tmp2E2,_tmp2E2,_tmp2E2 + 5U},{_tmp2E3,_tmp2E3,_tmp2E3 + 15U},{_tmp2E4,_tmp2E4,_tmp2E4 + 15U},{_tmp2E5,_tmp2E5,_tmp2E5 + 11U},{_tmp2E6,_tmp2E6,_tmp2E6 + 22U},{_tmp2E7,_tmp2E7,_tmp2E7 + 26U},{_tmp2E8,_tmp2E8,_tmp2E8 + 16U},{_tmp2E9,_tmp2E9,_tmp2E9 + 16U},{_tmp2EA,_tmp2EA,_tmp2EA + 24U},{_tmp2EB,_tmp2EB,_tmp2EB + 25U},{_tmp2EC,_tmp2EC,_tmp2EC + 21U},{_tmp2ED,_tmp2ED,_tmp2ED + 22U},{_tmp2EE,_tmp2EE,_tmp2EE + 16U},{_tmp2EF,_tmp2EF,_tmp2EF + 19U},{_tmp2F0,_tmp2F0,_tmp2F0 + 25U},{_tmp2F1,_tmp2F1,_tmp2F1 + 35U},{_tmp2F2,_tmp2F2,_tmp2F2 + 23U},{_tmp2F3,_tmp2F3,_tmp2F3 + 24U},{_tmp2F4,_tmp2F4,_tmp2F4 + 18U},{_tmp2F5,_tmp2F5,_tmp2F5 + 20U},{_tmp2F6,_tmp2F6,_tmp2F6 + 9U},{_tmp2F7,_tmp2F7,_tmp2F7 + 19U},{_tmp2F8,_tmp2F8,_tmp2F8 + 19U},{_tmp2F9,_tmp2F9,_tmp2F9 + 19U},{_tmp2FA,_tmp2FA,_tmp2FA + 19U},{_tmp2FB,_tmp2FB,_tmp2FB + 14U},{_tmp2FC,_tmp2FC,_tmp2FC + 19U},{_tmp2FD,_tmp2FD,_tmp2FD + 20U},{_tmp2FE,_tmp2FE,_tmp2FE + 14U},{_tmp2FF,_tmp2FF,_tmp2FF + 11U},{_tmp300,_tmp300,_tmp300 + 23U},{_tmp301,_tmp301,_tmp301 + 18U},{_tmp302,_tmp302,_tmp302 + 30U},{_tmp303,_tmp303,_tmp303 + 8U},{_tmp304,_tmp304,_tmp304 + 12U},{_tmp305,_tmp305,_tmp305 + 14U},{_tmp306,_tmp306,_tmp306 + 13U},{_tmp307,_tmp307,_tmp307 + 23U},{_tmp308,_tmp308,_tmp308 + 14U},{_tmp309,_tmp309,_tmp309 + 18U},{_tmp30A,_tmp30A,_tmp30A + 8U},{_tmp30B,_tmp30B,_tmp30B + 11U},{_tmp30C,_tmp30C,_tmp30C + 20U},{_tmp30D,_tmp30D,_tmp30D + 9U},{_tmp30E,_tmp30E,_tmp30E + 16U},{_tmp30F,_tmp30F,_tmp30F + 19U},{_tmp310,_tmp310,_tmp310 + 10U},{_tmp311,_tmp311,_tmp311 + 16U},{_tmp312,_tmp312,_tmp312 + 11U},{_tmp313,_tmp313,_tmp313 + 14U},{_tmp314,_tmp314,_tmp314 + 11U},{_tmp315,_tmp315,_tmp315 + 15U},{_tmp316,_tmp316,_tmp316 + 22U},{_tmp317,_tmp317,_tmp317 + 16U},{_tmp318,_tmp318,_tmp318 + 17U},{_tmp319,_tmp319,_tmp319 + 12U},{_tmp31A,_tmp31A,_tmp31A + 18U},{_tmp31B,_tmp31B,_tmp31B + 17U},{_tmp31C,_tmp31C,_tmp31C + 12U},{_tmp31D,_tmp31D,_tmp31D + 16U},{_tmp31E,_tmp31E,_tmp31E + 11U},{_tmp31F,_tmp31F,_tmp31F + 10U},{_tmp320,_tmp320,_tmp320 + 14U},{_tmp321,_tmp321,_tmp321 + 15U},{_tmp322,_tmp322,_tmp322 + 20U},{_tmp323,_tmp323,_tmp323 + 27U},{_tmp324,_tmp324,_tmp324 + 10U},{_tmp325,_tmp325,_tmp325 + 18U},{_tmp326,_tmp326,_tmp326 + 21U},{_tmp327,_tmp327,_tmp327 + 19U},{_tmp328,_tmp328,_tmp328 + 16U},{_tmp329,_tmp329,_tmp329 + 20U},{_tmp32A,_tmp32A,_tmp32A + 15U},{_tmp32B,_tmp32B,_tmp32B + 20U},{_tmp32C,_tmp32C,_tmp32C + 15U},{_tmp32D,_tmp32D,_tmp32D + 12U},{_tmp32E,_tmp32E,_tmp32E + 20U},{_tmp32F,_tmp32F,_tmp32F + 19U},{_tmp330,_tmp330,_tmp330 + 20U},{_tmp331,_tmp331,_tmp331 + 21U},{_tmp332,_tmp332,_tmp332 + 21U},{_tmp333,_tmp333,_tmp333 + 12U},{_tmp334,_tmp334,_tmp334 + 17U},{_tmp335,_tmp335,_tmp335 + 19U},{_tmp336,_tmp336,_tmp336 + 14U},{_tmp337,_tmp337,_tmp337 + 17U},{_tmp338,_tmp338,_tmp338 + 23U},{_tmp339,_tmp339,_tmp339 + 13U},{_tmp33A,_tmp33A,_tmp33A + 14U},{_tmp33B,_tmp33B,_tmp33B + 16U},{_tmp33C,_tmp33C,_tmp33C + 16U},{_tmp33D,_tmp33D,_tmp33D + 8U},{_tmp33E,_tmp33E,_tmp33E + 19U},{_tmp33F,_tmp33F,_tmp33F + 20U},{_tmp340,_tmp340,_tmp340 + 14U},{_tmp341,_tmp341,_tmp341 + 19U},{_tmp342,_tmp342,_tmp342 + 20U},{_tmp343,_tmp343,_tmp343 + 11U},{_tmp344,_tmp344,_tmp344 + 22U},{_tmp345,_tmp345,_tmp345 + 20U},{_tmp346,_tmp346,_tmp346 + 23U},{_tmp347,_tmp347,_tmp347 + 20U},{_tmp348,_tmp348,_tmp348 + 22U},{_tmp349,_tmp349,_tmp349 + 23U},{_tmp34A,_tmp34A,_tmp34A + 24U},{_tmp34B,_tmp34B,_tmp34B + 24U},{_tmp34C,_tmp34C,_tmp34C + 15U},{_tmp34D,_tmp34D,_tmp34D + 20U},{_tmp34E,_tmp34E,_tmp34E + 22U},{_tmp34F,_tmp34F,_tmp34F + 17U},{_tmp350,_tmp350,_tmp350 + 20U},{_tmp351,_tmp351,_tmp351 + 26U},{_tmp352,_tmp352,_tmp352 + 16U},{_tmp353,_tmp353,_tmp353 + 17U},{_tmp354,_tmp354,_tmp354 + 9U},{_tmp355,_tmp355,_tmp355 + 13U},{_tmp356,_tmp356,_tmp356 + 12U},{_tmp357,_tmp357,_tmp357 + 12U},{_tmp358,_tmp358,_tmp358 + 11U},{_tmp359,_tmp359,_tmp359 + 11U},{_tmp35A,_tmp35A,_tmp35A + 11U},{_tmp35B,_tmp35B,_tmp35B + 16U},{_tmp35C,_tmp35C,_tmp35C + 17U},{_tmp35D,_tmp35D,_tmp35D + 15U},{_tmp35E,_tmp35E,_tmp35E + 19U},{_tmp35F,_tmp35F,_tmp35F + 17U},{_tmp360,_tmp360,_tmp360 + 19U},{_tmp361,_tmp361,_tmp361 + 25U},{_tmp362,_tmp362,_tmp362 + 26U},{_tmp363,_tmp363,_tmp363 + 9U},{_tmp364,_tmp364,_tmp364 + 20U},{_tmp365,_tmp365,_tmp365 + 17U},{_tmp366,_tmp366,_tmp366 + 18U},{_tmp367,_tmp367,_tmp367 + 11U},{_tmp368,_tmp368,_tmp368 + 12U}};
# 1656
static short Cyc_yyr1[568U]={0,154,155,155,155,155,155,155,155,155,155,155,155,156,157,158,159,160,160,161,161,161,162,162,163,163,163,164,164,164,165,165,166,166,166,167,167,168,168,168,168,169,169,170,171,172,173,174,174,174,174,174,174,174,175,175,176,176,176,176,176,176,176,176,176,176,176,177,177,177,177,177,177,177,178,178,179,180,180,181,181,181,181,182,182,183,183,183,183,183,183,183,183,183,183,183,183,183,183,183,183,183,183,183,183,183,183,183,184,185,185,185,186,186,186,187,187,188,188,188,189,189,189,189,189,190,190,191,191,192,192,193,193,194,195,195,196,196,197,198,198,198,198,198,198,199,199,199,199,199,199,200,201,201,202,202,202,202,203,203,204,204,205,205,206,206,207,207,208,208,208,209,209,210,210,210,210,211,211,211,212,212,213,213,214,214,215,215,215,215,215,215,215,215,215,215,216,216,216,216,216,216,216,216,216,216,216,217,217,218,219,219,220,220,220,220,220,220,220,220,221,221,221,222,222,223,223,223,224,224,224,225,225,226,226,226,226,227,227,228,228,229,229,230,230,231,231,232,232,233,233,233,233,234,234,235,235,236,236,236,237,238,238,239,239,240,240,240,240,240,241,241,241,241,242,242,243,243,244,244,245,245,246,246,246,246,246,247,247,248,248,248,249,249,249,249,249,249,249,249,249,249,249,250,250,250,250,250,250,251,252,252,253,253,254,254,254,254,254,254,254,254,255,255,255,255,255,255,256,256,256,256,256,256,257,257,257,257,257,257,257,257,257,257,257,257,257,257,258,258,258,258,258,258,258,258,259,260,260,261,261,262,262,263,263,264,264,265,265,266,266,266,267,267,267,267,267,268,268,268,269,269,269,270,270,270,270,271,271,272,272,272,272,272,272,273,274,275,275,275,275,275,275,275,275,275,275,275,275,275,275,275,275,275,276,276,276,277,277,278,278,279,279,279,280,280,281,281,282,282,282,283,283,283,283,283,283,283,283,283,283,283,284,284,284,284,284,284,284,285,286,286,287,287,288,288,289,289,290,290,291,291,291,292,292,292,292,292,293,293,293,294,294,294,295,295,295,295,296,296,297,297,297,297,297,297,297,297,297,297,297,297,297,297,297,297,297,297,297,297,297,297,298,299,299,300,300,300,301,301,302,302,302,303,303,304,305,305,305,306,306,307,307,307,308,308,308,308,308,308,308,308,308,308,308,309,309,309,309,310,310,310,310,310,310,310,310,310,310,310,311,312,312,313,313,313,313,313,314,314,315,315,316,316,317,317,318,318};
# 1716
static short Cyc_yyr2[568U]={0,1,2,3,5,3,5,8,3,3,3,3,0,1,1,2,1,0,4,1,2,3,0,1,4,3,4,1,2,3,0,4,1,1,1,1,0,3,4,4,5,3,4,2,1,2,1,2,3,5,3,6,3,8,1,2,1,2,2,1,2,1,2,1,2,1,2,1,1,1,1,2,1,1,0,1,6,1,3,1,1,4,8,1,2,1,1,1,1,1,1,1,1,1,1,1,4,1,1,1,1,3,4,4,1,4,1,4,1,1,1,1,5,2,4,1,3,1,2,3,4,9,8,4,3,0,3,1,1,0,1,1,2,1,1,3,1,3,3,1,2,1,2,1,2,1,2,1,2,1,2,1,1,3,2,2,0,3,4,0,3,5,4,0,4,0,4,0,6,3,5,1,2,1,2,3,3,0,1,1,2,5,1,2,1,2,1,3,4,4,5,8,9,4,4,2,1,1,3,4,4,5,8,9,4,4,2,1,2,5,0,2,4,4,1,1,1,1,1,1,2,2,1,0,3,0,1,1,0,1,1,0,2,3,5,5,7,1,3,0,2,0,2,3,5,0,1,1,3,2,3,4,1,1,3,1,3,2,1,2,1,1,3,1,1,2,3,4,8,8,1,2,3,4,2,2,1,2,3,2,1,2,1,2,3,4,3,1,3,1,1,2,3,3,4,4,5,8,7,9,8,4,2,1,1,1,1,1,1,3,1,2,2,3,1,2,3,4,1,2,1,2,5,7,7,5,8,6,0,4,4,5,6,7,5,7,6,7,7,8,7,8,8,9,6,7,7,8,3,2,2,2,3,2,4,5,1,1,5,1,3,1,3,1,3,1,3,1,3,1,3,3,1,3,3,3,3,1,3,3,1,3,3,1,3,3,3,1,4,1,2,2,4,2,6,1,1,1,3,1,1,3,6,6,4,4,5,4,2,2,2,4,4,4,1,3,1,1,3,1,2,1,3,1,1,3,1,3,1,3,3,1,1,1,1,1,1,1,1,1,1,1,1,5,2,2,2,5,5,1,1,3,1,3,1,3,1,3,1,3,1,3,3,1,3,3,3,3,1,3,3,1,3,3,1,3,3,3,1,4,1,2,2,2,2,2,2,4,2,6,6,4,6,6,9,11,4,6,6,4,2,2,5,0,1,0,2,3,1,3,0,2,3,1,3,4,0,1,2,1,3,1,1,1,1,4,3,4,3,3,2,2,5,6,7,1,1,3,3,1,4,1,1,1,3,2,5,4,5,5,1,1,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1};
# 1776
static short Cyc_yydefact[1170U]={0,34,67,68,69,70,72,85,86,87,88,89,90,91,92,93,109,110,111,127,128,63,0,0,97,0,0,73,0,0,171,104,106,0,0,0,13,14,0,0,0,558,237,560,559,561,0,223,0,100,0,223,222,1,0,0,0,0,32,0,0,33,0,56,65,59,83,61,94,95,0,98,0,0,182,0,207,210,99,186,125,71,70,64,0,113,0,58,557,0,558,553,554,555,556,0,125,0,0,397,0,0,0,260,0,399,400,43,45,0,0,0,0,0,0,0,0,172,0,0,0,220,0,0,0,0,221,0,0,0,2,0,0,0,0,47,0,133,134,136,57,66,60,62,129,562,563,125,125,0,54,0,0,36,0,239,0,195,183,208,0,214,215,218,219,0,217,216,228,210,0,84,71,117,0,115,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,542,543,503,0,0,0,0,0,523,521,522,0,426,428,442,450,452,454,456,458,460,463,468,471,474,478,0,480,524,541,539,558,409,0,0,0,0,410,0,0,408,50,0,0,125,0,0,0,143,139,141,280,282,0,0,52,0,0,8,9,0,125,564,565,238,108,0,0,0,187,101,258,0,255,10,11,0,3,0,5,0,48,0,0,0,36,0,130,131,156,124,0,169,0,0,0,0,0,0,0,0,0,0,0,0,558,310,312,0,320,314,0,318,303,304,305,0,306,307,308,0,55,36,136,35,37,287,0,245,261,0,0,241,239,0,225,0,0,0,230,74,229,211,0,118,114,0,0,0,488,0,0,501,444,478,0,445,446,0,0,0,0,0,0,0,0,0,0,0,481,482,504,500,0,484,0,0,0,0,485,483,0,96,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,432,433,434,435,436,437,438,439,440,441,431,0,486,0,530,531,0,0,0,545,0,125,401,0,0,0,423,558,565,0,0,0,0,276,419,424,0,421,0,0,398,416,417,0,414,262,0,0,0,0,283,0,253,144,149,145,147,140,142,239,0,289,281,290,567,566,0,103,105,0,0,107,123,80,79,0,77,224,188,239,257,184,289,259,196,197,0,102,16,30,44,0,46,0,135,137,264,263,36,38,120,132,0,0,0,151,152,159,0,125,125,177,0,0,0,0,0,558,0,0,0,349,350,351,0,0,353,0,0,0,321,315,136,319,313,311,39,0,194,246,0,0,0,252,240,247,159,0,0,0,241,193,227,226,189,225,0,0,231,75,126,119,449,116,112,0,0,0,0,558,265,270,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,544,551,0,550,427,451,0,453,455,457,459,461,462,466,467,464,465,469,470,472,473,475,476,477,430,429,529,526,0,528,0,0,0,412,413,0,279,0,420,274,277,407,0,275,411,404,0,49,0,405,0,284,0,150,146,148,0,241,0,225,0,291,0,239,0,302,286,0,0,125,0,0,0,143,0,125,0,239,0,206,185,256,0,22,4,6,40,0,155,138,156,0,0,154,241,170,179,178,0,0,173,0,0,0,328,0,0,0,0,0,0,348,352,0,0,0,316,309,0,41,288,239,0,249,0,0,167,242,0,159,245,233,190,212,213,231,209,487,0,0,0,266,0,271,0,491,0,0,0,0,0,540,496,499,0,0,505,0,0,479,547,0,0,527,525,0,0,0,0,278,422,425,415,418,406,285,254,159,0,292,293,225,0,0,241,225,0,0,51,241,558,0,76,78,0,198,0,0,241,0,225,0,0,0,17,23,153,0,157,129,168,180,177,177,0,0,0,0,0,0,0,0,0,0,0,0,0,328,354,0,317,42,241,0,250,248,0,163,0,167,241,0,232,536,0,535,0,267,272,0,0,0,0,0,0,447,448,529,528,510,0,549,532,0,552,443,546,548,0,402,403,167,159,295,301,159,0,294,225,0,129,0,81,199,205,159,0,204,200,225,0,0,0,0,0,0,0,176,175,322,328,0,0,0,0,0,0,356,357,359,361,363,365,367,369,372,377,380,383,387,389,395,396,0,0,325,334,0,0,0,0,0,0,0,0,0,0,355,235,251,0,0,165,243,163,234,239,489,0,0,273,490,492,493,0,0,498,497,0,516,510,506,508,502,533,0,163,167,167,159,296,53,0,0,167,159,201,31,25,0,0,27,0,7,158,122,0,0,0,328,0,393,0,0,390,328,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,391,328,0,336,0,0,0,344,0,0,0,0,0,0,327,0,0,0,191,0,165,241,538,537,0,0,0,0,517,516,513,511,0,507,534,165,163,163,167,121,0,163,167,26,24,28,0,0,19,181,323,324,0,0,0,0,328,330,360,0,362,364,366,368,370,371,375,376,373,374,378,379,381,382,384,385,386,0,329,335,337,338,0,346,345,0,340,0,0,0,166,0,0,0,244,192,236,0,0,0,0,0,519,518,0,512,509,298,165,165,163,0,165,163,29,18,20,0,326,392,0,388,331,0,328,339,347,341,342,0,0,0,162,0,269,268,494,0,515,0,514,297,300,165,82,202,165,21,0,358,328,332,343,160,0,164,0,520,299,203,394,333,0,495,161,0,0,0};
# 1896
static short Cyc_yydefgoto[165U]={1167,53,54,55,56,487,879,1049,790,791,966,669,57,320,58,304,59,489,60,491,61,151,62,63,556,243,473,474,244,66,259,245,68,173,174,69,171,70,281,282,136,137,138,283,246,455,502,503,504,679,1094,928,1020,823,71,72,684,685,686,73,505,74,479,75,76,168,169,77,121,552,335,722,642,78,643,546,713,538,542,543,449,328,268,102,103,569,494,570,429,430,431,247,321,322,644,461,307,308,309,310,311,312,805,313,314,892,893,894,895,896,897,898,899,900,901,902,903,904,905,906,907,432,441,442,433,434,435,315,207,409,208,561,209,210,211,212,213,214,215,216,217,218,219,220,367,368,846,946,947,1031,948,1033,1105,221,222,830,223,588,589,224,225,80,967,436,465};
# 1916
static short Cyc_yypact[1170U]={3062,- -32768,- -32768,- -32768,- -32768,- 60,- -32768,- -32768,- -32768,- -32768,- -32768,- -32768,- -32768,- -32768,- -32768,- -32768,- -32768,- -32768,- -32768,- -32768,- -32768,3987,94,- 10,- -32768,3987,3188,- -32768,213,56,- -32768,17,85,43,- 15,96,- -32768,- -32768,113,447,228,- -32768,140,- -32768,- -32768,- -32768,173,178,633,216,224,178,- -32768,- -32768,207,225,254,2917,- -32768,508,625,- -32768,865,3987,3987,3987,- -32768,3987,- -32768,- -32768,617,- -32768,213,3895,380,40,127,1166,- -32768,- -32768,264,279,303,- -32768,213,298,7334,- -32768,- -32768,3272,217,- -32768,- -32768,- -32768,- -32768,309,264,253,7334,- -32768,320,3272,372,399,378,- -32768,57,- -32768,- -32768,4207,4207,- 38,426,2917,2917,7334,478,- -32768,- 31,470,7334,- -32768,548,462,- 31,4697,- -32768,2917,2917,3206,- -32768,2917,3206,2917,3206,- -32768,480,489,- -32768,3759,- -32768,- -32768,- -32768,- -32768,4697,- -32768,- -32768,264,199,1867,- -32768,3895,865,499,4207,4076,1518,- -32768,380,- -32768,513,- -32768,- -32768,- -32768,- -32768,530,- -32768,- -32768,- 2,1166,4207,- -32768,- -32768,540,597,549,213,7745,559,7844,7334,7547,587,599,601,609,627,658,668,675,684,689,695,7844,7844,- -32768,- -32768,817,7943,2617,709,7943,7943,- -32768,- -32768,- -32768,38,- -32768,- -32768,- 65,747,702,705,706,717,7,789,- 19,88,- -32768,978,7943,343,- 35,- -32768,744,155,- -32768,3272,15,753,1026,766,129,2899,- -32768,- -32768,774,7334,264,2899,761,4430,4697,1288,4697,698,- -32768,- 25,- 25,- -32768,784,768,- -32768,- -32768,275,264,- -32768,- -32768,- -32768,- -32768,42,777,773,- -32768,- -32768,839,285,- -32768,- -32768,- -32768,788,- -32768,791,- -32768,794,- -32768,548,5510,3895,499,796,4697,- -32768,918,814,213,821,797,359,829,4811,831,854,842,845,5624,2471,4811,86,834,- -32768,- -32768,840,2018,2018,865,2018,- -32768,- -32768,- -32768,848,- -32768,- -32768,- -32768,239,- -32768,499,853,- -32768,- -32768,849,234,872,- -32768,50,851,856,313,862,804,857,7334,4207,- -32768,876,- -32768,- -32768,234,213,- -32768,7334,861,2617,- -32768,4697,2617,- -32768,- -32768,- -32768,4925,- -32768,905,7334,7334,7334,7334,7334,7334,7334,896,7334,4697,257,- -32768,- -32768,- -32768,- -32768,878,- -32768,2018,883,333,7334,- -32768,- -32768,7334,- -32768,7943,7334,7943,7943,7943,7943,7943,7943,7943,7943,7943,7943,7943,7943,7943,7943,7943,7943,7943,7334,- -32768,- -32768,- -32768,- -32768,- -32768,- -32768,- -32768,- -32768,- -32768,- -32768,- -32768,7334,- -32768,- 31,- -32768,- -32768,5738,- 31,7334,- -32768,888,264,- -32768,889,890,892,- -32768,102,309,- 31,7334,3272,135,- -32768,- -32768,- -32768,893,897,891,3272,- -32768,- -32768,- -32768,895,898,- -32768,425,1026,899,4207,- -32768,900,904,- -32768,1288,1288,1288,- -32768,- -32768,3480,5852,339,- -32768,413,- -32768,- -32768,50,- -32768,- -32768,902,929,- -32768,917,- -32768,911,912,921,- -32768,- -32768,3343,- -32768,638,576,- -32768,- -32768,- -32768,4697,- -32768,- -32768,1000,- -32768,2917,- -32768,2917,- -32768,- -32768,- -32768,- -32768,499,- -32768,- -32768,- -32768,1057,7334,925,924,- -32768,- 21,592,264,264,887,7334,7334,920,928,7334,919,1029,2320,936,- -32768,- -32768,- -32768,452,1020,- -32768,5966,2169,2763,- -32768,- -32768,3759,- -32768,- -32768,- -32768,- -32768,4207,- -32768,- -32768,4697,931,4519,- -32768,- -32768,923,988,50,934,4608,856,- -32768,- -32768,- -32768,- -32768,804,935,55,859,- -32768,- -32768,- -32768,- -32768,- -32768,- -32768,937,944,941,964,951,- -32768,- -32768,771,5510,961,957,966,967,970,971,463,973,975,976,368,990,984,7646,- -32768,- -32768,980,977,- -32768,747,259,702,705,706,717,7,7,789,789,789,789,- 19,- 19,88,88,- -32768,- -32768,- -32768,- -32768,- -32768,- -32768,- -32768,983,- -32768,183,4207,5393,4697,- -32768,4697,- -32768,972,- -32768,- -32768,- -32768,- -32768,1563,- -32768,- -32768,- -32768,3043,- -32768,992,- -32768,287,- -32768,4697,- -32768,- -32768,- -32768,986,856,987,804,982,413,4207,4299,6080,- -32768,- -32768,7334,1001,264,7448,991,42,3617,994,264,4207,4076,6194,- -32768,638,- -32768,1005,1088,- -32768,- -32768,- -32768,515,- -32768,- -32768,918,1003,7334,- -32768,856,- -32768,- -32768,- -32768,1010,213,467,527,531,7334,882,552,4811,1008,6308,6422,582,- -32768,- -32768,1019,1023,1015,2018,- -32768,3895,- -32768,849,1025,4207,- -32768,1027,50,1073,- -32768,1024,988,251,- -32768,- -32768,- -32768,- -32768,859,- -32768,1032,231,1032,1035,- -32768,5042,- -32768,7334,- -32768,7334,7334,1137,7334,7547,- -32768,- -32768,- -32768,- 31,- 31,1030,1036,5159,- -32768,- -32768,7334,7334,- -32768,- -32768,234,785,1051,1059,- -32768,- -32768,- -32768,- -32768,- -32768,- -32768,- -32768,- -32768,988,1039,- -32768,- -32768,804,234,1040,856,804,1041,565,- -32768,856,1047,1048,- -32768,- -32768,1049,- -32768,234,1050,856,1053,804,1045,3206,1062,1143,- -32768,- -32768,7334,- -32768,4697,- -32768,1058,65,887,4811,1066,1061,1403,1056,1069,4811,7334,6536,640,6650,655,6764,882,- -32768,1072,- -32768,- -32768,856,301,- -32768,- -32768,1067,1120,1083,1073,856,4697,- -32768,- -32768,243,- -32768,7334,- -32768,- -32768,5510,1070,1071,1074,1077,1075,- -32768,905,1079,1080,- 54,1090,- -32768,- -32768,798,- -32768,- -32768,- -32768,- -32768,5393,- -32768,- -32768,1073,988,- -32768,- -32768,988,1091,- -32768,804,1081,4697,1094,- -32768,- -32768,- -32768,988,1092,- -32768,- -32768,804,1084,667,1096,2917,1093,1101,4697,- -32768,- -32768,1203,882,1105,8042,1099,2763,7943,1097,- -32768,- 34,- -32768,1136,1102,782,841,147,847,255,171,- -32768,- -32768,- -32768,- -32768,1139,7943,2018,- -32768,- -32768,583,4811,593,6878,4811,619,6992,7106,701,1110,- -32768,- -32768,- -32768,7334,1119,1171,1126,1120,- -32768,1025,- -32768,271,326,- -32768,- -32768,- -32768,- -32768,4697,1232,- -32768,- -32768,1123,- 32,397,- -32768,- -32768,- -32768,- -32768,5276,1120,1073,1073,988,- -32768,- -32768,1130,1132,1073,988,- -32768,- -32768,- -32768,1134,1135,707,478,- -32768,- -32768,- -32768,639,4811,1138,882,2617,- -32768,4697,1129,- -32768,1716,7943,7334,7943,7943,7943,7943,7943,7943,7943,7943,7943,7943,7943,7943,7943,7943,7943,7943,7943,7334,- -32768,882,1142,- -32768,4811,4811,644,- -32768,4811,4811,648,4811,664,7220,- -32768,1140,1144,1131,- -32768,50,1171,856,- -32768,- -32768,2763,1145,1149,7334,1155,402,- -32768,- -32768,1158,- -32768,- -32768,1171,1120,1120,1073,- -32768,1160,1120,1073,- -32768,- -32768,- -32768,478,1150,713,- -32768,- -32768,- -32768,1151,1152,1156,7943,882,- -32768,747,404,702,705,705,717,7,7,789,789,789,789,- 19,- 19,88,88,- -32768,- -32768,- -32768,412,- -32768,- -32768,- -32768,- -32768,4811,- -32768,- -32768,4811,- -32768,4811,4811,665,- -32768,4207,672,1144,- -32768,- -32768,- -32768,1161,816,1153,4697,677,- -32768,1157,1158,- -32768,- -32768,- -32768,1171,1171,1120,1159,1171,1120,- -32768,- -32768,- -32768,478,- -32768,1032,231,- -32768,- -32768,7334,1716,- -32768,- -32768,- -32768,- -32768,4811,822,1163,- -32768,687,- -32768,- -32768,- -32768,1174,- -32768,1169,- -32768,- -32768,- -32768,1171,- -32768,- -32768,1171,- -32768,362,- -32768,882,- -32768,- -32768,- -32768,4207,- -32768,1175,- -32768,- -32768,- -32768,- -32768,- -32768,826,- -32768,- -32768,1292,1293,- -32768};
# 2036
static short Cyc_yypgoto[165U]={- -32768,229,- -32768,- -32768,- -32768,- -32768,- -32768,175,- -32768,- -32768,265,- -32768,- -32768,- 236,- -32768,- -32768,- -32768,- -32768,- -32768,- -32768,44,- 134,134,- -32768,- -32768,0,654,- -32768,123,- 162,1190,33,- -32768,- -32768,- 130,- -32768,77,1276,- 745,- -32768,- -32768,- -32768,1042,1034,793,433,- -32768,- -32768,641,36,223,- 724,- 884,- 720,- -32768,- -32768,159,- -32768,- -32768,141,- 211,1249,- 405,459,- -32768,1164,- -32768,- -32768,1277,- 78,- -32768,608,- 138,- 146,- 126,- 231,311,614,623,- 452,- 495,- 121,- 443,- 123,- -32768,- 271,- 150,- 575,- 334,- -32768,906,- 131,- 23,- 167,- 124,- 381,5,- -32768,- -32768,- 59,- 294,- -32768,- 771,- -32768,- -32768,- -32768,- -32768,- -32768,- -32768,- -32768,- -32768,- -32768,- -32768,- -32768,- -32768,- -32768,- -32768,- -32768,- -32768,- -32768,- -32768,328,1095,- -32768,710,894,- -32768,588,864,- -32768,- 160,- 377,- 145,- 369,- 354,- 375,955,- 353,- 366,- 349,- 367,- 365,- 149,475,- -32768,- -32768,- -32768,- -32768,395,- -32768,- 857,312,- -32768,541,985,227,- -32768,- 380,- -32768,375,444,- 55,- 69,- 116,- 44};
# 2060
static short Cyc_yytable[8196U]={64,147,260,338,267,279,594,493,260,591,528,529,652,531,153,146,571,326,597,598,348,64,336,603,604,64,593,605,606,327,596,351,329,67,614,378,352,599,600,601,602,667,922,752,497,710,342,256,122,369,881,623,374,375,67,478,81,64,67,471,677,146,944,64,64,64,982,64,371,462,682,683,410,64,157,666,584,417,647,379,280,534,453,257,944,945,248,249,1032,250,67,258,317,714,251,707,67,67,67,647,67,666,386,387,667,930,67,1030,418,463,983,450,113,64,64,974,539,150,678,42,464,958,460,65,674,86,392,64,64,64,393,64,64,64,64,334,421,952,1097,64,388,389,481,762,65,701,67,67,65,64,109,64,422,1109,41,83,472,111,157,87,67,67,67,44,67,67,67,67,376,849,108,42,67,231,377,48,112,1108,540,541,65,560,67,150,67,239,65,65,65,123,65,720,240,305,- 174,316,65,140,141,142,464,143,548,139,1054,466,1022,152,41,559,1059,483,563,524,564,565,228,44,110,394,496,525,84,114,285,287,1143,1144,1037,483,1147,581,1080,1038,1039,229,65,65,395,396,1043,- 564,990,991,452,607,608,609,115,1142,65,65,65,118,65,65,65,65,47,376,672,1160,65,263,1161,438,660,625,560,228,51,52,65,152,65,427,428,454,536,64,992,993,117,306,571,152,130,1124,229,660,453,453,453,318,558,612,516,64,88,615,729,794,998,523,64,64,120,64,119,802,555,622,67,376,1110,1111,445,717,541,1114,1112,999,1000,316,1115,750,189,41,462,124,67,228,170,470,127,557,44,67,67,286,67,560,253,254,- 261,257,829,- 261,305,305,229,305,128,258,104,1153,269,270,271,125,272,273,274,275,535,537,533,232,463,376,64,41,91,195,196,92,93,94,325,933,44,1163,129,934,932,257,1024,- 239,1145,376,- 239,1148,346,258,835,200,704,170,748,541,996,105,65,67,997,376,267,- 15,816,450,571,469,764,152,305,484,880,227,318,172,65,485,661,761,636,483,176,65,65,235,65,87,464,411,412,413,745,925,306,306,325,306,230,547,79,483,464,530,780,795,751,483,452,452,452,234,376,834,740,412,413,651,780,376,105,1026,85,19,20,586,106,705,107,41,457,718,105,659,414,665,458,768,44,415,416,454,454,454,753,64,754,64,79,65,511,782,618,512,1162,236,673,79,934,414,306,46,79,450,741,416,238,769,706,154,145,155,148,64,560,79,835,156,67,770,67,783,64,1034,175,237,64,703,1106,106,376,159,945,784,46,862,785,1030,376,1125,866,106,648,252,649,1017,67,1126,633,872,650,376,420,79,79,67,145,695,440,67,936,541,79,766,440,714,702,79,79,79,150,79,79,79,79,698,712,818,376,834,680,681,841,924,851,721,826,842,41,376,798,931,79,799,264,736,43,44,45,105,661,261,105,276,831,105,1063,1064,65,1060,65,105,1003,835,483,277,175,483,1066,1067,843,844,319,1072,1073,41,1062,1074,1075,560,1065,131,132,43,44,45,65,1068,1069,1070,1071,817,47,651,332,65,500,152,344,65,347,349,349,376,51,52,152,376,41,800,152,333,665,801,364,365,339,44,106,349,206,106,349,349,106,47,834,341,376,48,106,819,233,1058,806,859,41,51,52,863,345,376,349,703,43,44,45,865,64,255,64,459,41,852,874,812,482,477,376,376,43,44,45,458,670,1004,671,79,353,376,860,480,340,500,482,1006,508,41,775,513,354,67,355,67,870,43,44,45,980,506,356,144,305,41,316,376,79,825,133,134,721,1010,44,624,46,979,1002,972,357,46,47,630,560,916,48,484,376,662,347,663,376,1051,51,52,376,664,1084,41,919,175,1087,376,956,372,64,43,44,45,1098,358,376,376,964,962,965,857,1089,1131,1133,359,105,885,1023,376,1134,1027,360,912,105,1140,384,385,349,1133,1025,361,105,67,146,1157,362,65,444,65,1015,47,363,376,1152,457,1047,366,306,1048,152,458,1118,51,52,1119,373,1055,380,1056,381,1076,1077,1078,382,349,383,349,349,349,349,349,349,349,349,349,349,349,349,349,349,349,349,349,419,106,423,541,16,17,18,64,437,106,390,391,522,639,640,641,443,106,550,551,682,683,953,1099,447,954,467,1050,727,728,468,349,803,804,475,960,1123,476,64,65,67,146,853,854,459,486,266,1005,488,482,1009,490,482,498,1132,950,951,986,987,372,510,79,372,79,480,284,988,989,994,995,507,67,482,1137,376,578,266,509,482,1155,535,41,305,1166,535,883,884,506,759,43,44,45,514,1151,517,592,518,519,47,1139,520,526,477,527,349,532,1052,41,458,64,51,52,262,278,537,544,44,1164,562,1040,135,146,464,545,47,1044,549,553,48,46,65,105,616,378,831,105,51,52,579,1082,1083,583,67,1085,1086,876,1088,585,331,617,627,619,620,305,621,628,632,637,629,631,41,65,638,635,451,653,456,397,43,44,45,654,306,655,656,668,657,47,1050,658,675,500,676,689,690,501,692,693,349,51,52,697,146,699,708,88,711,677,715,719,106,723,724,284,106,725,726,398,399,400,401,402,403,404,405,406,407,1127,- 564,730,1128,731,1129,1130,732,733,687,688,734,735,691,482,65,696,742,747,969,737,408,738,739,743,755,306,746,482,506,749,482,760,763,765,767,89,64,778,774,797,781,908,506,788,789,506,1154,424,793,796,425,91,495,807,92,93,94,813,426,44,814,815,325,349,96,822,820,97,824,67,744,98,839,99,100,855,427,428,832,845,305,41,847,856,101,858,861,867,105,43,44,45,46,864,868,869,871,875,47,873,877,878,500,882,886,910,554,887,911,923,51,52,926,927,929,941,937,938,957,959,939,349,940,963,495,942,943,572,573,574,575,576,577,349,968,580,949,955,961,970,971,973,79,975,978,984,981,587,1001,1016,590,773,160,161,162,163,164,106,985,65,266,165,166,167,1018,1019,1021,1028,1029,1041,306,610,1042,1045,1046,1057,1095,1053,349,1081,266,1104,1093,611,944,1113,1092,266,587,1117,1120,1101,809,811,1102,1159,1122,1141,1121,1138,1136,1156,1168,1169,1149,1146,7,8,9,10,11,12,13,14,15,16,17,18,19,20,1158,1165,779,1116,265,116,499,792,1135,492,266,145,646,79,158,22,23,24,126,828,827,266,1096,337,821,446,626,595,757,634,266,1035,30,1107,909,0,0,0,582,1150,31,32,0,0,0,0,0,0,0,0,0,0,38,977,0,0,349,0,0,0,0,0,0,0,0,0,39,40,0,0,0,0,0,0,349,0,0,0,0,587,0,587,0,0,0,913,915,0,918,0,921,349,0,0,0,0,0,0,0,42,0,0,145,46,0,0,0,0,0,0,935,0,0,0,0,49,50,0,888,889,0,0,0,0,0,495,0,0,0,0,0,0,266,0,88,0,0,0,0,0,0,0,451,0,0,0,266,349,0,349,349,349,349,349,349,349,349,349,349,349,349,349,349,349,349,349,0,0,233,0,0,0,495,0,0,0,0,0,0,0,0,0,145,0,0,0,0,0,0,0,0,0,89,0,1008,0,0,1012,1014,0,0,0,0,0,772,0,0,226,91,0,777,92,93,94,0,95,44,0,787,0,0,96,349,0,97,0,0,0,890,0,99,100,0,177,178,0,0,0,891,0,0,101,203,587,0,204,205,0,179,88,0,180,0,145,372,0,0,181,0,0,0,1061,0,182,183,184,185,186,187,0,0,0,0,188,0,0,0,189,284,1079,190,191,495,192,836,0,837,838,0,840,349,0,0,1091,88,0,0,0,495,193,194,850,0,0,1100,0,0,1103,0,0,266,0,0,0,0,0,0,0,0,0,0,0,41,91,195,196,92,93,94,0,0,44,0,0,197,0,0,0,0,198,0,0,0,199,0,0,200,0,0,284,330,0,89,201,0,0,202,203,0,0,204,205,0,0,756,266,0,425,91,0,0,92,93,94,0,426,44,0,0,0,0,96,0,0,97,0,0,0,98,495,99,100,0,427,428,0,0,0,0,0,0,101,0,0,0,0,0,0,495,2,3,4,82,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,803,804,21,177,178,288,0,289,290,291,292,293,294,295,296,22,23,24,297,88,26,180,298,0,0,0,0,181,27,299,0,0,30,182,183,184,185,186,187,0,31,32,33,188,0,0,0,189,0,0,190,191,38,192,0,0,0,0,0,0,0,0,0,0,0,0,39,40,0,193,194,0,0,0,0,0,0,495,0,0,0,0,0,0,0,0,0,0,0,0,0,0,300,91,195,196,92,93,94,42,43,44,45,46,197,301,149,0,0,198,0,0,0,199,0,49,303,0,0,0,0,0,0,201,0,0,202,203,0,0,204,205,2,3,4,82,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,0,0,21,177,178,288,0,289,290,291,292,293,294,295,296,22,23,24,297,88,26,180,298,0,0,0,0,181,27,299,0,0,30,182,183,184,185,186,187,0,31,32,33,188,0,0,0,189,0,0,190,191,38,192,0,0,0,0,0,0,0,0,0,0,0,0,39,40,0,193,194,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,300,91,195,196,92,93,94,42,43,44,45,46,197,301,149,302,0,198,0,0,0,199,0,49,303,0,0,0,0,0,0,201,0,0,202,203,0,0,204,205,2,3,4,82,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,0,0,21,177,178,288,0,289,290,291,292,293,294,295,296,22,23,24,297,88,26,180,298,0,0,0,0,181,27,299,0,0,30,182,183,184,185,186,187,0,31,32,33,188,0,0,0,189,0,0,190,191,38,192,0,0,0,0,0,0,0,0,0,0,0,0,39,40,0,193,194,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,300,91,195,196,92,93,94,42,43,44,45,46,197,301,149,0,0,198,0,0,0,199,0,49,303,0,0,0,0,0,0,201,0,0,202,203,0,0,204,205,2,3,4,82,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,0,0,21,177,178,288,0,289,290,291,292,293,294,295,296,22,23,24,297,88,26,180,298,0,0,0,0,181,27,299,0,0,30,182,183,184,185,186,187,0,31,32,33,188,0,0,0,189,0,0,190,191,38,192,0,0,0,0,0,0,0,0,0,0,0,0,39,40,0,193,194,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,515,91,195,196,92,93,94,42,43,44,45,46,197,301,149,0,0,198,0,0,0,199,0,49,303,0,0,0,0,0,0,201,0,0,202,203,0,0,204,205,2,3,4,82,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,0,0,21,177,178,0,0,0,0,0,0,0,0,0,0,22,23,24,297,88,26,180,0,0,0,0,0,181,27,0,0,0,30,182,183,184,185,186,187,0,31,32,33,188,0,0,0,189,0,0,190,191,38,192,0,0,0,0,0,0,0,0,0,0,0,0,39,40,0,193,194,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,41,91,195,196,92,93,94,42,43,44,45,46,197,694,0,0,0,198,0,0,0,199,0,49,303,0,0,0,0,0,0,201,0,0,202,203,0,0,204,205,2,3,4,82,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,0,0,21,177,178,0,0,0,0,0,0,0,0,0,0,22,23,24,297,88,0,0,0,0,0,0,0,0,27,0,0,0,30,182,183,184,185,186,187,0,31,32,0,0,0,0,0,189,0,0,190,191,38,192,0,0,0,0,0,0,0,0,0,0,0,0,39,40,0,193,194,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,41,91,195,196,92,93,94,42,43,44,45,46,197,0,0,0,0,198,0,0,0,346,0,49,303,0,0,0,0,0,0,201,0,0,202,203,0,0,204,205,7,8,9,10,11,12,13,14,15,16,17,18,19,20,0,0,0,177,178,0,0,0,0,0,0,0,0,0,0,22,23,24,179,88,0,180,0,0,0,0,0,181,0,0,0,0,30,182,183,184,185,186,187,0,31,32,0,188,0,0,0,189,0,0,190,191,38,192,0,0,0,0,0,0,0,0,0,0,0,0,39,40,0,193,194,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,41,91,195,196,92,93,94,42,43,44,45,46,197,0,370,0,0,198,0,0,0,199,0,49,303,0,0,0,0,0,0,201,0,0,202,203,0,0,204,205,7,8,9,10,11,12,13,14,15,16,17,18,19,20,0,0,0,177,178,0,0,0,0,0,0,0,0,0,0,22,23,24,179,88,0,180,0,0,0,0,0,181,0,0,0,0,30,182,183,184,185,186,187,0,31,32,0,188,0,0,0,189,0,0,190,191,38,192,0,0,0,0,0,0,0,0,0,0,0,0,39,40,0,193,194,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,41,91,195,196,92,93,94,42,43,44,45,46,197,0,0,0,0,198,0,0,0,199,0,49,303,0,0,0,0,0,0,201,0,0,202,203,0,0,204,205,- 12,1,0,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,0,88,21,0,0,0,0,0,0,0,0,0,0,0,0,22,23,24,25,0,26,0,0,0,0,0,0,0,27,0,28,29,30,0,0,0,0,0,0,0,31,32,33,0,0,34,35,0,36,37,0,0,38,0,0,0,0,0,0,89,0,0,0,0,0,0,39,40,0,0,0,439,0,0,226,91,0,0,92,93,94,0,95,44,0,0,0,0,96,0,0,97,41,0,0,98,0,99,100,42,43,44,45,46,0,0,0,- 12,101,47,0,0,0,48,0,49,50,0,0,0,0,51,52,- 12,1,0,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,88,0,21,0,0,0,0,0,0,0,0,0,0,0,0,22,23,24,25,0,26,0,0,0,0,0,0,0,27,0,28,29,30,0,0,0,0,0,0,0,31,32,33,0,0,34,35,0,36,37,0,0,38,0,0,0,0,0,89,0,0,0,0,0,0,0,39,40,0,0,758,0,0,226,91,0,0,92,93,94,0,95,44,0,0,0,0,96,0,0,97,0,41,0,98,0,99,100,0,42,43,44,45,46,0,0,0,101,0,47,0,0,0,48,0,49,50,0,0,0,0,51,52,1,0,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,0,88,21,0,0,0,0,0,0,0,0,0,0,0,0,22,23,24,25,0,26,0,0,0,0,0,0,0,27,0,28,29,30,0,0,0,0,0,0,0,31,32,33,0,0,34,35,0,36,37,0,0,38,0,0,0,0,0,0,89,0,0,0,0,0,0,39,40,0,0,0,0,0,0,90,91,0,0,92,93,94,0,95,44,0,88,0,0,96,0,0,97,41,0,0,98,0,99,100,42,43,44,45,46,0,0,0,- 12,101,47,0,0,0,48,0,49,50,0,0,0,0,51,52,7,8,9,10,11,12,13,14,15,16,17,18,19,20,0,0,0,0,0,0,89,0,0,0,0,0,0,0,0,22,23,24,0,0,0,226,91,0,0,92,93,94,0,95,44,0,30,0,0,96,0,0,97,0,31,32,98,0,99,100,0,0,0,0,0,0,38,0,0,101,0,0,0,0,0,0,0,0,0,0,39,40,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,323,0,0,41,0,0,0,0,0,0,42,43,44,45,46,0,325,0,0,0,47,0,0,0,477,0,49,50,0,0,458,0,51,52,7,8,9,10,11,12,13,14,15,16,17,18,19,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,22,23,24,0,0,0,0,0,0,0,0,0,0,0,0,0,0,30,0,0,0,0,0,0,0,31,32,0,0,0,0,0,0,0,0,0,0,38,0,0,0,0,0,0,0,0,0,0,0,0,0,39,40,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,323,0,0,0,0,0,0,0,0,0,42,43,0,45,46,0,325,0,0,0,47,0,0,0,457,0,49,50,0,0,458,0,51,52,7,8,9,10,11,12,13,14,15,16,17,18,19,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,22,23,24,0,0,0,0,0,0,0,0,0,0,0,0,0,0,30,0,0,0,0,0,0,0,31,32,0,0,0,0,0,0,0,0,0,0,38,0,0,0,0,0,0,0,0,0,0,0,0,0,39,40,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,41,0,0,0,0,0,0,42,43,44,45,46,0,0,0,0,0,47,0,0,0,500,0,49,50,0,0,0,0,51,52,2,3,4,82,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,22,23,24,25,0,26,0,0,0,0,0,0,0,27,0,0,0,30,0,0,0,0,0,0,0,31,32,33,0,0,0,0,0,0,0,0,0,38,0,0,0,0,0,0,0,0,0,0,0,0,0,39,40,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,42,43,0,45,46,0,0,149,0,0,0,278,0,0,0,0,49,50,2,3,4,82,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,0,0,21,0,0,0,0,0,0,0,0,0,0,0,0,22,23,24,25,0,26,0,0,0,0,0,0,0,27,0,0,0,30,0,0,0,0,0,0,0,31,32,33,0,0,0,0,0,0,0,0,0,38,0,0,0,0,0,0,0,0,0,0,0,0,0,39,40,0,0,0,0,0,2,3,4,82,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,0,0,21,0,0,0,0,42,43,0,45,46,0,0,149,22,23,24,25,0,0,0,0,49,50,0,0,0,27,0,0,0,30,0,0,0,0,0,0,0,31,32,0,0,0,0,0,0,0,0,0,0,38,0,0,0,0,0,0,0,0,0,0,0,0,0,39,40,0,0,0,0,0,0,0,7,8,9,10,11,12,13,14,15,16,17,18,19,20,0,0,0,0,0,0,0,0,0,0,42,43,0,45,46,22,23,24,0,0,0,0,0,0,0,0,49,50,0,0,0,0,30,0,0,0,0,0,0,0,31,32,0,0,0,0,0,0,0,0,0,0,38,0,0,0,0,0,0,0,0,0,0,0,0,0,39,40,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,323,0,0,324,0,0,0,0,0,0,42,43,0,45,46,0,325,0,0,0,0,0,0,0,0,0,49,50,7,8,9,10,11,12,13,14,15,16,17,18,19,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,22,23,24,0,0,0,0,0,0,0,0,0,0,0,0,0,0,30,0,0,0,0,0,0,0,31,32,0,0,241,0,0,0,0,0,0,0,38,0,0,0,0,0,0,0,0,0,0,0,0,0,39,40,0,0,0,0,0,0,0,0,0,0,7,8,9,10,11,12,13,14,15,16,17,18,19,20,0,0,0,0,0,0,0,42,43,0,45,46,0,0,242,22,23,24,0,0,0,0,0,49,50,0,0,0,0,0,0,0,30,0,0,0,0,0,0,0,31,32,0,0,0,0,0,0,0,0,0,0,38,0,0,0,0,0,0,0,0,0,0,0,0,0,39,40,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,323,0,0,0,0,0,0,0,0,0,42,43,0,45,46,0,325,0,0,0,0,0,0,0,0,0,49,50,7,8,9,10,11,12,13,14,15,16,17,18,19,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,22,23,24,0,0,0,0,0,0,0,0,0,0,0,0,0,0,30,0,0,0,0,0,0,0,31,32,0,0,0,0,0,0,0,0,0,0,38,0,0,0,0,0,0,0,0,0,0,0,0,0,39,40,0,0,0,0,0,0,0,7,8,9,10,11,12,13,14,15,16,17,18,19,20,0,0,0,0,0,0,0,0,0,0,42,43,0,45,46,22,23,24,448,0,0,0,0,0,0,0,49,50,0,0,0,0,30,0,0,0,0,0,0,0,31,32,0,0,0,0,0,0,0,0,0,0,38,0,0,0,0,0,0,0,0,0,0,0,0,0,39,40,0,0,0,0,0,0,0,7,8,9,10,11,12,13,14,15,16,17,18,19,20,0,0,0,0,0,0,0,0,0,0,42,43,0,45,46,22,23,24,709,0,0,0,0,0,0,0,49,50,0,0,0,0,30,0,0,0,0,0,0,0,31,32,0,0,0,0,0,0,0,0,0,0,38,0,0,0,0,0,0,0,0,0,0,0,0,0,39,40,0,0,0,0,0,0,0,7,8,9,10,11,12,13,14,15,16,17,18,19,20,716,0,0,0,0,0,0,0,0,0,42,43,0,45,46,22,23,24,0,0,0,0,0,0,0,0,49,50,0,0,0,0,30,0,0,0,0,0,0,0,31,32,0,0,0,0,0,0,0,0,0,0,38,0,0,0,0,0,0,0,0,0,0,0,0,0,39,40,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,42,43,0,45,46,0,0,0,0,0,0,0,0,0,0,0,49,50,177,178,288,0,289,290,291,292,293,294,295,296,0,0,0,179,88,0,180,298,0,0,0,0,181,0,299,0,0,0,182,183,184,185,186,187,0,0,0,0,188,0,0,0,189,0,0,190,191,0,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,193,194,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,515,91,195,196,92,93,94,0,0,44,0,0,197,301,149,0,0,198,0,0,0,199,0,0,200,177,178,0,0,0,0,201,566,0,202,203,0,0,204,205,179,88,0,180,0,0,0,0,0,181,0,0,0,0,0,182,183,184,185,186,187,0,0,0,0,188,0,0,0,189,0,0,190,191,0,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,193,194,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,567,91,195,196,92,93,94,0,258,44,0,0,197,0,350,568,0,198,0,0,0,199,0,0,200,0,427,428,177,178,0,201,0,0,202,203,0,0,204,205,0,0,0,179,88,0,180,0,0,0,0,0,181,0,0,0,0,0,182,183,184,185,186,187,0,0,0,0,188,0,0,0,189,0,0,190,191,0,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,193,194,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,567,91,195,196,92,93,94,0,258,44,0,0,197,0,350,833,0,198,0,0,0,199,0,0,200,0,427,428,177,178,0,201,0,0,202,203,0,0,204,205,0,0,0,179,88,0,180,0,0,0,0,0,181,0,0,0,0,0,182,183,184,185,186,187,0,0,0,0,188,0,0,0,189,0,0,190,191,0,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,193,194,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,567,91,195,196,92,93,94,0,258,44,0,0,197,0,350,848,0,198,0,0,0,199,0,0,200,0,427,428,177,178,0,201,0,0,202,203,0,0,204,205,0,0,0,179,88,0,180,0,0,0,0,0,181,0,0,0,0,0,182,183,184,185,186,187,0,0,0,0,188,0,0,0,189,0,0,190,191,0,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,193,194,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,567,91,195,196,92,93,94,0,258,44,0,0,197,0,350,1036,0,198,0,0,0,199,0,0,200,0,427,428,177,178,0,201,0,0,202,203,0,0,204,205,0,0,0,179,88,0,180,0,0,0,0,0,181,0,0,0,0,0,182,183,184,185,186,187,0,0,0,0,188,0,0,0,189,0,0,190,191,0,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,193,194,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,567,91,195,196,92,93,94,0,258,44,0,0,197,0,350,0,0,198,0,0,0,199,0,0,200,0,427,428,177,178,0,201,0,0,202,203,0,0,204,205,0,0,0,179,88,0,180,0,0,0,0,0,181,0,0,0,0,0,182,183,184,185,186,187,0,0,0,0,188,0,0,0,189,0,0,190,191,0,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,193,194,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,41,91,195,196,92,93,94,0,0,44,0,0,197,0,350,0,0,198,0,0,0,199,0,0,200,177,178,0,0,0,0,201,0,0,202,203,0,0,204,205,179,88,0,180,0,0,0,0,0,181,0,0,0,0,0,182,183,184,185,186,187,0,0,0,0,188,0,0,0,189,0,0,190,191,0,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,193,194,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,41,91,195,196,92,93,94,0,0,44,0,0,197,521,0,0,0,198,0,0,0,199,0,0,200,177,178,0,0,0,0,201,0,0,202,203,0,0,204,205,179,88,0,180,0,0,0,0,0,181,0,0,0,0,0,182,183,184,185,186,187,0,0,0,0,188,0,0,0,189,0,0,190,191,0,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,193,194,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,41,91,195,196,92,93,94,0,0,44,0,0,197,0,0,0,0,198,0,0,0,199,613,0,200,177,178,0,0,0,0,201,0,0,202,203,0,0,204,205,179,88,0,180,0,0,0,0,0,181,0,0,0,0,0,182,183,184,185,186,187,0,0,0,0,188,0,0,0,189,0,0,190,191,0,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,193,194,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,41,91,195,196,92,93,94,0,0,44,0,0,197,0,0,0,0,198,0,0,0,199,0,0,200,177,178,0,645,0,0,201,0,0,202,203,0,0,204,205,179,88,0,180,0,0,0,0,0,181,0,0,0,0,0,182,183,184,185,186,187,0,0,0,0,188,0,0,0,189,0,0,190,191,0,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,193,194,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,41,91,195,196,92,93,94,0,0,44,0,0,197,0,0,0,0,198,0,0,0,199,700,0,200,177,178,0,0,0,0,201,0,0,202,203,0,0,204,205,179,88,0,180,0,0,0,0,0,181,0,0,0,0,0,182,183,184,185,186,187,0,0,0,0,188,0,0,0,189,0,0,190,191,0,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,193,194,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,41,91,195,196,92,93,94,0,0,44,0,0,197,0,0,0,0,198,0,0,0,199,0,0,200,177,178,0,771,0,0,201,0,0,202,203,0,0,204,205,179,88,0,180,0,0,0,0,0,181,0,0,0,0,0,182,183,184,185,186,187,0,0,0,0,188,0,0,0,189,0,0,190,191,0,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,193,194,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,41,91,195,196,92,93,94,0,0,44,0,0,197,0,0,0,0,198,0,0,0,199,0,0,200,177,178,0,786,0,0,201,0,0,202,203,0,0,204,205,179,88,0,180,0,0,0,0,0,181,0,0,0,0,0,182,183,184,185,186,187,0,0,0,0,188,0,0,0,189,0,0,190,191,0,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,193,194,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,41,91,195,196,92,93,94,0,0,44,0,0,197,808,0,0,0,198,0,0,0,199,0,0,200,177,178,0,0,0,0,201,0,0,202,203,0,0,204,205,179,88,0,180,0,0,0,0,0,181,0,0,0,0,0,182,183,184,185,186,187,0,0,0,0,188,0,0,0,189,0,0,190,191,0,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,193,194,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,41,91,195,196,92,93,94,0,0,44,0,0,197,810,0,0,0,198,0,0,0,199,0,0,200,177,178,0,0,0,0,201,0,0,202,203,0,0,204,205,179,88,0,180,0,0,0,0,0,181,0,0,0,0,0,182,183,184,185,186,187,0,0,0,0,188,0,0,0,189,0,0,190,191,0,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,193,194,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,41,91,195,196,92,93,94,0,0,44,0,0,197,0,0,0,0,198,0,0,0,199,914,0,200,177,178,0,0,0,0,201,0,0,202,203,0,0,204,205,179,88,0,180,0,0,0,0,0,181,0,0,0,0,0,182,183,184,185,186,187,0,0,0,0,188,0,0,0,189,0,0,190,191,0,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,193,194,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,41,91,195,196,92,93,94,0,0,44,0,0,197,0,0,0,0,198,0,0,0,199,917,0,200,177,178,0,0,0,0,201,0,0,202,203,0,0,204,205,179,88,0,180,0,0,0,0,0,181,0,0,0,0,0,182,183,184,185,186,187,0,0,0,0,188,0,0,0,189,0,0,190,191,0,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,193,194,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,41,91,195,196,92,93,94,0,0,44,0,0,197,920,0,0,0,198,0,0,0,199,0,0,200,177,178,0,0,0,0,201,0,0,202,203,0,0,204,205,179,88,0,180,0,0,0,0,0,181,0,0,0,0,0,182,183,184,185,186,187,0,0,0,0,188,0,0,0,189,0,0,190,191,0,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,193,194,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,41,91,195,196,92,93,94,0,0,44,0,0,197,0,0,0,0,198,0,0,0,199,1007,0,200,177,178,0,0,0,0,201,0,0,202,203,0,0,204,205,179,88,0,180,0,0,0,0,0,181,0,0,0,0,0,182,183,184,185,186,187,0,0,0,0,188,0,0,0,189,0,0,190,191,0,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,193,194,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,41,91,195,196,92,93,94,0,0,44,0,0,197,0,0,0,0,198,0,0,0,199,1011,0,200,177,178,0,0,0,0,201,0,0,202,203,0,0,204,205,179,88,0,180,0,0,0,0,0,181,0,0,0,0,0,182,183,184,185,186,187,0,0,0,0,188,0,0,0,189,0,0,190,191,0,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,193,194,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,41,91,195,196,92,93,94,0,0,44,0,0,197,0,0,0,0,198,0,0,0,199,1013,0,200,177,178,0,0,0,0,201,0,0,202,203,0,0,204,205,179,88,0,180,0,0,0,0,0,181,0,0,0,0,0,182,183,184,185,186,187,0,0,0,0,188,0,0,0,189,0,0,190,191,0,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,193,194,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,41,91,195,196,92,93,94,0,0,44,0,0,197,0,0,0,0,198,0,0,0,199,1090,0,200,177,178,0,0,0,0,201,0,0,202,203,0,0,204,205,179,88,0,180,0,0,0,0,0,181,0,0,0,0,0,182,183,184,185,186,187,0,0,0,0,188,0,0,0,189,0,0,190,191,0,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,193,194,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,41,91,195,196,92,93,94,0,0,44,0,0,197,0,0,0,0,198,0,0,0,199,0,0,200,177,178,0,0,0,0,201,0,0,202,203,0,0,204,205,179,88,0,180,0,0,0,0,0,181,0,0,0,0,0,182,183,184,185,186,187,0,0,0,0,188,0,0,0,189,0,0,190,191,0,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,193,194,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,776,91,195,196,92,93,94,0,0,44,177,178,197,0,0,0,0,198,0,0,0,199,0,0,200,179,88,0,0,0,0,201,0,0,202,203,0,0,204,205,182,183,184,185,186,187,0,0,0,0,0,0,0,0,189,0,0,190,191,0,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,193,194,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,41,91,195,196,92,93,94,0,0,44,177,178,197,0,350,0,0,198,0,0,0,199,0,0,200,179,88,0,0,0,0,201,0,0,202,203,0,0,204,205,182,183,184,185,186,187,0,0,0,0,0,0,0,0,189,0,0,190,191,0,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,193,194,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,41,91,195,196,92,93,94,0,0,44,177,178,197,0,744,0,0,198,0,0,0,199,0,0,200,179,88,0,0,0,0,201,0,0,202,203,0,0,204,205,182,183,184,185,186,187,0,0,0,0,0,0,0,0,189,0,0,190,191,0,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,193,194,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,41,91,195,196,92,93,94,0,0,44,177,178,197,0,0,0,0,198,0,0,0,343,0,0,200,179,88,0,0,0,0,201,0,0,202,203,0,0,204,205,182,183,184,185,186,187,0,0,0,0,0,0,0,0,189,0,0,190,191,0,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,193,194,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,41,91,195,196,92,93,94,0,0,44,177,178,197,0,0,0,0,198,0,0,0,346,0,0,200,179,88,0,0,0,0,201,0,0,202,203,0,0,204,205,182,183,184,185,186,187,0,0,0,0,0,0,0,0,189,0,0,190,191,0,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,193,194,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,41,91,195,196,92,93,94,0,0,44,177,178,197,0,0,0,0,198,0,0,0,199,0,0,200,179,88,0,0,0,0,201,0,0,202,203,0,0,204,205,182,183,184,185,186,187,0,0,0,0,0,0,0,0,189,0,0,190,191,0,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,193,194,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,41,91,195,196,92,93,94,0,0,44,0,0,197,0,0,0,0,198,0,0,0,976,0,0,200,0,0,0,0,0,0,201,0,0,202,203,0,0,204,205};
# 2883
static short Cyc_yycheck[8196U]={0,70,118,170,125,139,381,278,124,378,304,305,464,307,73,70,350,155,384,385,180,21,168,390,391,25,380,392,393,155,383,181,155,0,414,100,181,386,387,388,389,484,813,618,280,540,176,116,48,198,795,428,201,202,21,266,116,57,25,17,81,116,116,63,64,65,100,67,199,94,5,6,221,73,74,480,370,112,459,144,139,317,244,114,116,139,109,110,945,127,57,122,151,545,132,538,63,64,65,480,67,506,95,96,547,825,73,139,143,134,144,242,127,113,114,886,66,73,139,121,145,866,246,0,501,135,145,127,128,129,149,131,132,133,134,137,121,857,1022,139,133,134,266,638,21,525,113,114,25,149,133,151,137,1037,114,21,114,114,158,25,127,128,129,123,131,132,133,134,130,744,114,121,139,96,136,135,133,1034,128,325,57,341,149,139,151,128,63,64,65,48,67,136,135,149,129,151,73,63,64,65,145,67,328,62,975,249,930,73,114,339,981,266,343,127,345,346,114,123,133,131,279,135,128,127,147,148,1110,1111,952,284,1114,362,1003,953,954,133,113,114,150,151,960,139,95,96,244,394,395,396,135,1106,127,128,129,113,131,132,133,134,131,130,496,1145,139,122,1148,136,477,132,428,114,143,144,149,139,151,140,141,244,322,279,133,134,54,149,618,151,57,1058,133,500,452,453,454,152,338,411,291,297,41,415,571,678,131,298,304,305,128,307,135,689,333,427,279,130,1038,1039,239,548,464,1043,1040,150,151,279,1044,142,69,114,94,113,297,114,133,256,127,335,123,304,305,140,307,501,113,114,127,114,115,130,304,305,133,307,127,122,26,1126,127,128,129,135,131,132,133,134,130,114,127,114,134,130,370,114,115,116,117,118,119,120,127,136,123,1152,128,140,827,114,115,136,1112,130,139,1115,135,122,728,138,530,133,139,545,145,26,279,370,149,130,527,128,702,540,744,136,643,279,370,130,793,89,277,116,297,136,477,136,447,480,128,304,305,101,307,297,145,90,91,92,585,136,304,305,127,307,133,130,0,500,145,306,659,680,617,506,452,453,454,135,130,728,90,91,92,461,673,130,89,139,22,20,21,136,26,530,28,114,135,553,101,477,135,479,141,648,123,140,141,452,453,454,619,489,621,491,48,370,135,662,419,138,136,127,500,57,140,135,370,125,62,638,140,141,132,649,535,133,70,135,72,517,678,75,854,141,489,649,491,663,526,130,84,130,530,526,130,89,130,76,139,663,125,770,663,139,130,139,775,101,133,121,135,926,517,139,127,784,141,130,228,113,114,526,116,517,234,530,835,711,122,645,240,1021,526,127,128,129,530,131,132,133,134,127,544,707,130,854,507,508,736,818,748,556,716,736,114,130,127,826,152,130,136,136,122,123,124,228,659,135,231,127,724,234,985,986,489,982,491,240,910,951,673,130,176,676,988,989,740,741,127,994,995,114,984,996,997,793,987,127,128,122,123,124,517,990,991,992,993,704,131,647,135,526,135,517,177,530,179,180,181,130,143,144,526,130,114,136,530,135,666,136,193,194,130,123,228,198,86,231,201,202,234,131,951,132,130,135,240,708,98,981,136,767,114,143,144,771,135,130,221,692,122,123,124,136,702,115,704,246,114,751,786,127,266,135,130,130,122,123,124,141,489,136,491,277,135,130,768,266,129,135,284,136,286,114,655,289,135,702,135,704,782,122,123,124,891,284,135,128,702,114,704,130,306,715,127,128,721,136,123,429,125,890,909,882,135,125,131,437,926,127,135,130,130,133,297,135,130,136,143,144,130,141,136,114,127,339,136,130,864,199,788,122,123,124,1023,135,130,130,129,875,131,763,136,136,130,135,429,800,932,130,136,940,135,806,437,136,97,98,341,130,934,135,445,788,877,136,135,702,238,704,127,131,135,130,1126,135,127,18,702,130,704,141,127,143,144,130,135,976,99,978,146,998,999,1000,147,378,148,380,381,382,383,384,385,386,387,388,389,390,391,392,393,394,395,396,128,429,121,1021,17,18,19,879,114,437,93,94,296,452,453,454,114,445,86,87,5,6,858,1026,135,861,114,968,129,130,134,428,22,23,129,871,1057,136,910,788,879,968,129,130,457,129,125,914,129,477,917,129,480,129,1093,129,130,147,148,343,135,489,346,491,477,144,97,98,93,94,128,910,500,129,130,359,155,128,506,129,130,114,910,129,130,798,799,500,632,122,123,124,135,1125,135,379,114,127,131,1102,127,139,135,135,501,129,973,114,141,981,143,144,120,132,114,136,123,1156,129,955,127,1048,145,139,131,961,136,142,135,125,879,628,416,100,1122,632,143,144,114,1006,1007,135,981,1010,1011,788,1013,136,156,133,129,134,134,981,134,130,130,129,139,136,114,910,130,136,243,135,245,61,122,123,124,114,910,128,135,47,136,131,1119,130,127,135,130,135,128,139,139,30,585,143,144,127,1119,45,135,41,145,81,136,136,628,136,130,282,632,136,114,101,102,103,104,105,106,107,108,109,110,1084,139,130,1087,136,1089,1090,130,130,510,511,130,130,514,659,981,517,116,130,879,136,132,136,136,129,142,981,136,673,659,136,676,129,136,136,142,99,1126,136,127,685,136,803,673,128,46,676,1131,111,135,129,114,115,278,135,118,119,120,128,122,123,127,136,127,678,128,82,129,131,134,1126,128,135,25,137,138,114,140,141,133,139,1126,114,136,114,148,136,136,130,803,122,123,124,125,142,136,136,136,142,131,136,128,48,135,135,128,139,332,136,129,127,143,144,135,83,121,130,136,136,127,115,136,736,135,129,350,136,136,353,354,355,356,357,358,748,128,361,136,136,136,136,129,28,788,128,135,99,139,373,99,129,376,653,76,77,78,79,80,803,146,1126,457,85,86,87,135,84,130,25,135,129,1126,397,130,129,129,136,135,129,793,127,477,116,128,409,116,115,136,484,414,129,129,136,694,695,135,116,130,130,136,136,129,128,0,0,1119,136,8,9,10,11,12,13,14,15,16,17,18,19,20,21,136,136,658,1048,124,39,282,676,1095,277,527,877,458,879,75,37,38,39,51,721,716,538,1021,169,711,240,430,382,628,445,547,946,54,1031,803,- 1,- 1,- 1,363,1122,62,63,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,74,888,- 1,- 1,891,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,88,89,- 1,- 1,- 1,- 1,- 1,- 1,909,- 1,- 1,- 1,- 1,525,- 1,527,- 1,- 1,- 1,807,808,- 1,810,- 1,812,926,- 1,- 1,- 1,- 1,- 1,- 1,- 1,121,- 1,- 1,968,125,- 1,- 1,- 1,- 1,- 1,- 1,832,- 1,- 1,- 1,- 1,137,138,- 1,25,26,- 1,- 1,- 1,- 1,- 1,571,- 1,- 1,- 1,- 1,- 1,- 1,649,- 1,41,- 1,- 1,- 1,- 1,- 1,- 1,- 1,659,- 1,- 1,- 1,663,982,- 1,984,985,986,987,988,989,990,991,992,993,994,995,996,997,998,999,1000,- 1,- 1,890,- 1,- 1,- 1,618,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,1048,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,99,- 1,916,- 1,- 1,919,920,- 1,- 1,- 1,- 1,- 1,650,- 1,- 1,114,115,- 1,656,118,119,120,- 1,122,123,- 1,664,- 1,- 1,128,1057,- 1,131,- 1,- 1,- 1,135,- 1,137,138,- 1,25,26,- 1,- 1,- 1,145,- 1,- 1,148,149,689,- 1,152,153,- 1,40,41,- 1,43,- 1,1119,976,- 1,- 1,49,- 1,- 1,- 1,983,- 1,55,56,57,58,59,60,- 1,- 1,- 1,- 1,65,- 1,- 1,- 1,69,795,1001,72,73,728,75,730,- 1,732,733,- 1,735,1125,- 1,- 1,1015,41,- 1,- 1,- 1,744,91,92,747,- 1,- 1,1026,- 1,- 1,1029,- 1,- 1,827,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,114,115,116,117,118,119,120,- 1,- 1,123,- 1,- 1,126,- 1,- 1,- 1,- 1,131,- 1,- 1,- 1,135,- 1,- 1,138,- 1,- 1,866,142,- 1,99,145,- 1,- 1,148,149,- 1,- 1,152,153,- 1,- 1,111,882,- 1,114,115,- 1,- 1,118,119,120,- 1,122,123,- 1,- 1,- 1,- 1,128,- 1,- 1,131,- 1,- 1,- 1,135,835,137,138,- 1,140,141,- 1,- 1,- 1,- 1,- 1,- 1,148,- 1,- 1,- 1,- 1,- 1,- 1,854,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,- 1,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,- 1,- 1,- 1,- 1,49,50,51,- 1,- 1,54,55,56,57,58,59,60,- 1,62,63,64,65,- 1,- 1,- 1,69,- 1,- 1,72,73,74,75,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,88,89,- 1,91,92,- 1,- 1,- 1,- 1,- 1,- 1,951,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,114,115,116,117,118,119,120,121,122,123,124,125,126,127,128,- 1,- 1,131,- 1,- 1,- 1,135,- 1,137,138,- 1,- 1,- 1,- 1,- 1,- 1,145,- 1,- 1,148,149,- 1,- 1,152,153,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,- 1,- 1,24,25,26,27,- 1,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,- 1,- 1,- 1,- 1,49,50,51,- 1,- 1,54,55,56,57,58,59,60,- 1,62,63,64,65,- 1,- 1,- 1,69,- 1,- 1,72,73,74,75,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,88,89,- 1,91,92,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,114,115,116,117,118,119,120,121,122,123,124,125,126,127,128,129,- 1,131,- 1,- 1,- 1,135,- 1,137,138,- 1,- 1,- 1,- 1,- 1,- 1,145,- 1,- 1,148,149,- 1,- 1,152,153,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,- 1,- 1,24,25,26,27,- 1,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,- 1,- 1,- 1,- 1,49,50,51,- 1,- 1,54,55,56,57,58,59,60,- 1,62,63,64,65,- 1,- 1,- 1,69,- 1,- 1,72,73,74,75,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,88,89,- 1,91,92,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,114,115,116,117,118,119,120,121,122,123,124,125,126,127,128,- 1,- 1,131,- 1,- 1,- 1,135,- 1,137,138,- 1,- 1,- 1,- 1,- 1,- 1,145,- 1,- 1,148,149,- 1,- 1,152,153,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,- 1,- 1,24,25,26,27,- 1,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,- 1,- 1,- 1,- 1,49,50,51,- 1,- 1,54,55,56,57,58,59,60,- 1,62,63,64,65,- 1,- 1,- 1,69,- 1,- 1,72,73,74,75,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,88,89,- 1,91,92,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,114,115,116,117,118,119,120,121,122,123,124,125,126,127,128,- 1,- 1,131,- 1,- 1,- 1,135,- 1,137,138,- 1,- 1,- 1,- 1,- 1,- 1,145,- 1,- 1,148,149,- 1,- 1,152,153,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,- 1,- 1,24,25,26,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,37,38,39,40,41,42,43,- 1,- 1,- 1,- 1,- 1,49,50,- 1,- 1,- 1,54,55,56,57,58,59,60,- 1,62,63,64,65,- 1,- 1,- 1,69,- 1,- 1,72,73,74,75,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,88,89,- 1,91,92,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,114,115,116,117,118,119,120,121,122,123,124,125,126,127,- 1,- 1,- 1,131,- 1,- 1,- 1,135,- 1,137,138,- 1,- 1,- 1,- 1,- 1,- 1,145,- 1,- 1,148,149,- 1,- 1,152,153,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,- 1,- 1,24,25,26,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,37,38,39,40,41,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,50,- 1,- 1,- 1,54,55,56,57,58,59,60,- 1,62,63,- 1,- 1,- 1,- 1,- 1,69,- 1,- 1,72,73,74,75,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,88,89,- 1,91,92,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,114,115,116,117,118,119,120,121,122,123,124,125,126,- 1,- 1,- 1,- 1,131,- 1,- 1,- 1,135,- 1,137,138,- 1,- 1,- 1,- 1,- 1,- 1,145,- 1,- 1,148,149,- 1,- 1,152,153,8,9,10,11,12,13,14,15,16,17,18,19,20,21,- 1,- 1,- 1,25,26,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,37,38,39,40,41,- 1,43,- 1,- 1,- 1,- 1,- 1,49,- 1,- 1,- 1,- 1,54,55,56,57,58,59,60,- 1,62,63,- 1,65,- 1,- 1,- 1,69,- 1,- 1,72,73,74,75,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,88,89,- 1,91,92,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,114,115,116,117,118,119,120,121,122,123,124,125,126,- 1,128,- 1,- 1,131,- 1,- 1,- 1,135,- 1,137,138,- 1,- 1,- 1,- 1,- 1,- 1,145,- 1,- 1,148,149,- 1,- 1,152,153,8,9,10,11,12,13,14,15,16,17,18,19,20,21,- 1,- 1,- 1,25,26,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,37,38,39,40,41,- 1,43,- 1,- 1,- 1,- 1,- 1,49,- 1,- 1,- 1,- 1,54,55,56,57,58,59,60,- 1,62,63,- 1,65,- 1,- 1,- 1,69,- 1,- 1,72,73,74,75,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,88,89,- 1,91,92,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,114,115,116,117,118,119,120,121,122,123,124,125,126,- 1,- 1,- 1,- 1,131,- 1,- 1,- 1,135,- 1,137,138,- 1,- 1,- 1,- 1,- 1,- 1,145,- 1,- 1,148,149,- 1,- 1,152,153,0,1,- 1,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,- 1,41,24,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,37,38,39,40,- 1,42,- 1,- 1,- 1,- 1,- 1,- 1,- 1,50,- 1,52,53,54,- 1,- 1,- 1,- 1,- 1,- 1,- 1,62,63,64,- 1,- 1,67,68,- 1,70,71,- 1,- 1,74,- 1,- 1,- 1,- 1,- 1,- 1,99,- 1,- 1,- 1,- 1,- 1,- 1,88,89,- 1,- 1,- 1,111,- 1,- 1,114,115,- 1,- 1,118,119,120,- 1,122,123,- 1,- 1,- 1,- 1,128,- 1,- 1,131,114,- 1,- 1,135,- 1,137,138,121,122,123,124,125,- 1,- 1,- 1,129,148,131,- 1,- 1,- 1,135,- 1,137,138,- 1,- 1,- 1,- 1,143,144,0,1,- 1,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,41,- 1,24,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,37,38,39,40,- 1,42,- 1,- 1,- 1,- 1,- 1,- 1,- 1,50,- 1,52,53,54,- 1,- 1,- 1,- 1,- 1,- 1,- 1,62,63,64,- 1,- 1,67,68,- 1,70,71,- 1,- 1,74,- 1,- 1,- 1,- 1,- 1,99,- 1,- 1,- 1,- 1,- 1,- 1,- 1,88,89,- 1,- 1,111,- 1,- 1,114,115,- 1,- 1,118,119,120,- 1,122,123,- 1,- 1,- 1,- 1,128,- 1,- 1,131,- 1,114,- 1,135,- 1,137,138,- 1,121,122,123,124,125,- 1,- 1,- 1,148,- 1,131,- 1,- 1,- 1,135,- 1,137,138,- 1,- 1,- 1,- 1,143,144,1,- 1,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,- 1,41,24,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,37,38,39,40,- 1,42,- 1,- 1,- 1,- 1,- 1,- 1,- 1,50,- 1,52,53,54,- 1,- 1,- 1,- 1,- 1,- 1,- 1,62,63,64,- 1,- 1,67,68,- 1,70,71,- 1,- 1,74,- 1,- 1,- 1,- 1,- 1,- 1,99,- 1,- 1,- 1,- 1,- 1,- 1,88,89,- 1,- 1,- 1,- 1,- 1,- 1,114,115,- 1,- 1,118,119,120,- 1,122,123,- 1,41,- 1,- 1,128,- 1,- 1,131,114,- 1,- 1,135,- 1,137,138,121,122,123,124,125,- 1,- 1,- 1,129,148,131,- 1,- 1,- 1,135,- 1,137,138,- 1,- 1,- 1,- 1,143,144,8,9,10,11,12,13,14,15,16,17,18,19,20,21,- 1,- 1,- 1,- 1,- 1,- 1,99,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,37,38,39,- 1,- 1,- 1,114,115,- 1,- 1,118,119,120,- 1,122,123,- 1,54,- 1,- 1,128,- 1,- 1,131,- 1,62,63,135,- 1,137,138,- 1,- 1,- 1,- 1,- 1,- 1,74,- 1,- 1,148,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,88,89,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,111,- 1,- 1,114,- 1,- 1,- 1,- 1,- 1,- 1,121,122,123,124,125,- 1,127,- 1,- 1,- 1,131,- 1,- 1,- 1,135,- 1,137,138,- 1,- 1,141,- 1,143,144,8,9,10,11,12,13,14,15,16,17,18,19,20,21,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,37,38,39,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,54,- 1,- 1,- 1,- 1,- 1,- 1,- 1,62,63,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,74,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,88,89,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,111,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,121,122,- 1,124,125,- 1,127,- 1,- 1,- 1,131,- 1,- 1,- 1,135,- 1,137,138,- 1,- 1,141,- 1,143,144,8,9,10,11,12,13,14,15,16,17,18,19,20,21,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,37,38,39,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,54,- 1,- 1,- 1,- 1,- 1,- 1,- 1,62,63,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,74,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,88,89,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,114,- 1,- 1,- 1,- 1,- 1,- 1,121,122,123,124,125,- 1,- 1,- 1,- 1,- 1,131,- 1,- 1,- 1,135,- 1,137,138,- 1,- 1,- 1,- 1,143,144,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,- 1,- 1,24,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,37,38,39,40,- 1,42,- 1,- 1,- 1,- 1,- 1,- 1,- 1,50,- 1,- 1,- 1,54,- 1,- 1,- 1,- 1,- 1,- 1,- 1,62,63,64,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,74,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,88,89,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,121,122,- 1,124,125,- 1,- 1,128,- 1,- 1,- 1,132,- 1,- 1,- 1,- 1,137,138,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,- 1,- 1,24,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,37,38,39,40,- 1,42,- 1,- 1,- 1,- 1,- 1,- 1,- 1,50,- 1,- 1,- 1,54,- 1,- 1,- 1,- 1,- 1,- 1,- 1,62,63,64,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,74,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,88,89,- 1,- 1,- 1,- 1,- 1,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,- 1,- 1,24,- 1,- 1,- 1,- 1,121,122,- 1,124,125,- 1,- 1,128,37,38,39,40,- 1,- 1,- 1,- 1,137,138,- 1,- 1,- 1,50,- 1,- 1,- 1,54,- 1,- 1,- 1,- 1,- 1,- 1,- 1,62,63,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,74,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,88,89,- 1,- 1,- 1,- 1,- 1,- 1,- 1,8,9,10,11,12,13,14,15,16,17,18,19,20,21,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,121,122,- 1,124,125,37,38,39,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,137,138,- 1,- 1,- 1,- 1,54,- 1,- 1,- 1,- 1,- 1,- 1,- 1,62,63,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,74,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,88,89,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,111,- 1,- 1,114,- 1,- 1,- 1,- 1,- 1,- 1,121,122,- 1,124,125,- 1,127,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,137,138,8,9,10,11,12,13,14,15,16,17,18,19,20,21,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,37,38,39,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,54,- 1,- 1,- 1,- 1,- 1,- 1,- 1,62,63,- 1,- 1,66,- 1,- 1,- 1,- 1,- 1,- 1,- 1,74,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,88,89,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,8,9,10,11,12,13,14,15,16,17,18,19,20,21,- 1,- 1,- 1,- 1,- 1,- 1,- 1,121,122,- 1,124,125,- 1,- 1,128,37,38,39,- 1,- 1,- 1,- 1,- 1,137,138,- 1,- 1,- 1,- 1,- 1,- 1,- 1,54,- 1,- 1,- 1,- 1,- 1,- 1,- 1,62,63,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,74,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,88,89,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,111,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,121,122,- 1,124,125,- 1,127,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,137,138,8,9,10,11,12,13,14,15,16,17,18,19,20,21,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,37,38,39,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,54,- 1,- 1,- 1,- 1,- 1,- 1,- 1,62,63,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,74,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,88,89,- 1,- 1,- 1,- 1,- 1,- 1,- 1,8,9,10,11,12,13,14,15,16,17,18,19,20,21,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,121,122,- 1,124,125,37,38,39,129,- 1,- 1,- 1,- 1,- 1,- 1,- 1,137,138,- 1,- 1,- 1,- 1,54,- 1,- 1,- 1,- 1,- 1,- 1,- 1,62,63,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,74,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,88,89,- 1,- 1,- 1,- 1,- 1,- 1,- 1,8,9,10,11,12,13,14,15,16,17,18,19,20,21,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,121,122,- 1,124,125,37,38,39,129,- 1,- 1,- 1,- 1,- 1,- 1,- 1,137,138,- 1,- 1,- 1,- 1,54,- 1,- 1,- 1,- 1,- 1,- 1,- 1,62,63,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,74,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,88,89,- 1,- 1,- 1,- 1,- 1,- 1,- 1,8,9,10,11,12,13,14,15,16,17,18,19,20,21,111,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,121,122,- 1,124,125,37,38,39,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,137,138,- 1,- 1,- 1,- 1,54,- 1,- 1,- 1,- 1,- 1,- 1,- 1,62,63,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,74,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,88,89,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,121,122,- 1,124,125,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,137,138,25,26,27,- 1,29,30,31,32,33,34,35,36,- 1,- 1,- 1,40,41,- 1,43,44,- 1,- 1,- 1,- 1,49,- 1,51,- 1,- 1,- 1,55,56,57,58,59,60,- 1,- 1,- 1,- 1,65,- 1,- 1,- 1,69,- 1,- 1,72,73,- 1,75,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,91,92,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,114,115,116,117,118,119,120,- 1,- 1,123,- 1,- 1,126,127,128,- 1,- 1,131,- 1,- 1,- 1,135,- 1,- 1,138,25,26,- 1,- 1,- 1,- 1,145,32,- 1,148,149,- 1,- 1,152,153,40,41,- 1,43,- 1,- 1,- 1,- 1,- 1,49,- 1,- 1,- 1,- 1,- 1,55,56,57,58,59,60,- 1,- 1,- 1,- 1,65,- 1,- 1,- 1,69,- 1,- 1,72,73,- 1,75,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,91,92,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,114,115,116,117,118,119,120,- 1,122,123,- 1,- 1,126,- 1,128,129,- 1,131,- 1,- 1,- 1,135,- 1,- 1,138,- 1,140,141,25,26,- 1,145,- 1,- 1,148,149,- 1,- 1,152,153,- 1,- 1,- 1,40,41,- 1,43,- 1,- 1,- 1,- 1,- 1,49,- 1,- 1,- 1,- 1,- 1,55,56,57,58,59,60,- 1,- 1,- 1,- 1,65,- 1,- 1,- 1,69,- 1,- 1,72,73,- 1,75,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,91,92,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,114,115,116,117,118,119,120,- 1,122,123,- 1,- 1,126,- 1,128,129,- 1,131,- 1,- 1,- 1,135,- 1,- 1,138,- 1,140,141,25,26,- 1,145,- 1,- 1,148,149,- 1,- 1,152,153,- 1,- 1,- 1,40,41,- 1,43,- 1,- 1,- 1,- 1,- 1,49,- 1,- 1,- 1,- 1,- 1,55,56,57,58,59,60,- 1,- 1,- 1,- 1,65,- 1,- 1,- 1,69,- 1,- 1,72,73,- 1,75,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,91,92,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,114,115,116,117,118,119,120,- 1,122,123,- 1,- 1,126,- 1,128,129,- 1,131,- 1,- 1,- 1,135,- 1,- 1,138,- 1,140,141,25,26,- 1,145,- 1,- 1,148,149,- 1,- 1,152,153,- 1,- 1,- 1,40,41,- 1,43,- 1,- 1,- 1,- 1,- 1,49,- 1,- 1,- 1,- 1,- 1,55,56,57,58,59,60,- 1,- 1,- 1,- 1,65,- 1,- 1,- 1,69,- 1,- 1,72,73,- 1,75,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,91,92,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,114,115,116,117,118,119,120,- 1,122,123,- 1,- 1,126,- 1,128,129,- 1,131,- 1,- 1,- 1,135,- 1,- 1,138,- 1,140,141,25,26,- 1,145,- 1,- 1,148,149,- 1,- 1,152,153,- 1,- 1,- 1,40,41,- 1,43,- 1,- 1,- 1,- 1,- 1,49,- 1,- 1,- 1,- 1,- 1,55,56,57,58,59,60,- 1,- 1,- 1,- 1,65,- 1,- 1,- 1,69,- 1,- 1,72,73,- 1,75,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,91,92,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,114,115,116,117,118,119,120,- 1,122,123,- 1,- 1,126,- 1,128,- 1,- 1,131,- 1,- 1,- 1,135,- 1,- 1,138,- 1,140,141,25,26,- 1,145,- 1,- 1,148,149,- 1,- 1,152,153,- 1,- 1,- 1,40,41,- 1,43,- 1,- 1,- 1,- 1,- 1,49,- 1,- 1,- 1,- 1,- 1,55,56,57,58,59,60,- 1,- 1,- 1,- 1,65,- 1,- 1,- 1,69,- 1,- 1,72,73,- 1,75,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,91,92,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,114,115,116,117,118,119,120,- 1,- 1,123,- 1,- 1,126,- 1,128,- 1,- 1,131,- 1,- 1,- 1,135,- 1,- 1,138,25,26,- 1,- 1,- 1,- 1,145,- 1,- 1,148,149,- 1,- 1,152,153,40,41,- 1,43,- 1,- 1,- 1,- 1,- 1,49,- 1,- 1,- 1,- 1,- 1,55,56,57,58,59,60,- 1,- 1,- 1,- 1,65,- 1,- 1,- 1,69,- 1,- 1,72,73,- 1,75,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,91,92,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,114,115,116,117,118,119,120,- 1,- 1,123,- 1,- 1,126,127,- 1,- 1,- 1,131,- 1,- 1,- 1,135,- 1,- 1,138,25,26,- 1,- 1,- 1,- 1,145,- 1,- 1,148,149,- 1,- 1,152,153,40,41,- 1,43,- 1,- 1,- 1,- 1,- 1,49,- 1,- 1,- 1,- 1,- 1,55,56,57,58,59,60,- 1,- 1,- 1,- 1,65,- 1,- 1,- 1,69,- 1,- 1,72,73,- 1,75,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,91,92,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,114,115,116,117,118,119,120,- 1,- 1,123,- 1,- 1,126,- 1,- 1,- 1,- 1,131,- 1,- 1,- 1,135,136,- 1,138,25,26,- 1,- 1,- 1,- 1,145,- 1,- 1,148,149,- 1,- 1,152,153,40,41,- 1,43,- 1,- 1,- 1,- 1,- 1,49,- 1,- 1,- 1,- 1,- 1,55,56,57,58,59,60,- 1,- 1,- 1,- 1,65,- 1,- 1,- 1,69,- 1,- 1,72,73,- 1,75,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,91,92,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,114,115,116,117,118,119,120,- 1,- 1,123,- 1,- 1,126,- 1,- 1,- 1,- 1,131,- 1,- 1,- 1,135,- 1,- 1,138,25,26,- 1,142,- 1,- 1,145,- 1,- 1,148,149,- 1,- 1,152,153,40,41,- 1,43,- 1,- 1,- 1,- 1,- 1,49,- 1,- 1,- 1,- 1,- 1,55,56,57,58,59,60,- 1,- 1,- 1,- 1,65,- 1,- 1,- 1,69,- 1,- 1,72,73,- 1,75,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,91,92,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,114,115,116,117,118,119,120,- 1,- 1,123,- 1,- 1,126,- 1,- 1,- 1,- 1,131,- 1,- 1,- 1,135,136,- 1,138,25,26,- 1,- 1,- 1,- 1,145,- 1,- 1,148,149,- 1,- 1,152,153,40,41,- 1,43,- 1,- 1,- 1,- 1,- 1,49,- 1,- 1,- 1,- 1,- 1,55,56,57,58,59,60,- 1,- 1,- 1,- 1,65,- 1,- 1,- 1,69,- 1,- 1,72,73,- 1,75,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,91,92,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,114,115,116,117,118,119,120,- 1,- 1,123,- 1,- 1,126,- 1,- 1,- 1,- 1,131,- 1,- 1,- 1,135,- 1,- 1,138,25,26,- 1,142,- 1,- 1,145,- 1,- 1,148,149,- 1,- 1,152,153,40,41,- 1,43,- 1,- 1,- 1,- 1,- 1,49,- 1,- 1,- 1,- 1,- 1,55,56,57,58,59,60,- 1,- 1,- 1,- 1,65,- 1,- 1,- 1,69,- 1,- 1,72,73,- 1,75,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,91,92,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,114,115,116,117,118,119,120,- 1,- 1,123,- 1,- 1,126,- 1,- 1,- 1,- 1,131,- 1,- 1,- 1,135,- 1,- 1,138,25,26,- 1,142,- 1,- 1,145,- 1,- 1,148,149,- 1,- 1,152,153,40,41,- 1,43,- 1,- 1,- 1,- 1,- 1,49,- 1,- 1,- 1,- 1,- 1,55,56,57,58,59,60,- 1,- 1,- 1,- 1,65,- 1,- 1,- 1,69,- 1,- 1,72,73,- 1,75,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,91,92,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,114,115,116,117,118,119,120,- 1,- 1,123,- 1,- 1,126,127,- 1,- 1,- 1,131,- 1,- 1,- 1,135,- 1,- 1,138,25,26,- 1,- 1,- 1,- 1,145,- 1,- 1,148,149,- 1,- 1,152,153,40,41,- 1,43,- 1,- 1,- 1,- 1,- 1,49,- 1,- 1,- 1,- 1,- 1,55,56,57,58,59,60,- 1,- 1,- 1,- 1,65,- 1,- 1,- 1,69,- 1,- 1,72,73,- 1,75,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,91,92,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,114,115,116,117,118,119,120,- 1,- 1,123,- 1,- 1,126,127,- 1,- 1,- 1,131,- 1,- 1,- 1,135,- 1,- 1,138,25,26,- 1,- 1,- 1,- 1,145,- 1,- 1,148,149,- 1,- 1,152,153,40,41,- 1,43,- 1,- 1,- 1,- 1,- 1,49,- 1,- 1,- 1,- 1,- 1,55,56,57,58,59,60,- 1,- 1,- 1,- 1,65,- 1,- 1,- 1,69,- 1,- 1,72,73,- 1,75,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,91,92,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,114,115,116,117,118,119,120,- 1,- 1,123,- 1,- 1,126,- 1,- 1,- 1,- 1,131,- 1,- 1,- 1,135,136,- 1,138,25,26,- 1,- 1,- 1,- 1,145,- 1,- 1,148,149,- 1,- 1,152,153,40,41,- 1,43,- 1,- 1,- 1,- 1,- 1,49,- 1,- 1,- 1,- 1,- 1,55,56,57,58,59,60,- 1,- 1,- 1,- 1,65,- 1,- 1,- 1,69,- 1,- 1,72,73,- 1,75,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,91,92,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,114,115,116,117,118,119,120,- 1,- 1,123,- 1,- 1,126,- 1,- 1,- 1,- 1,131,- 1,- 1,- 1,135,136,- 1,138,25,26,- 1,- 1,- 1,- 1,145,- 1,- 1,148,149,- 1,- 1,152,153,40,41,- 1,43,- 1,- 1,- 1,- 1,- 1,49,- 1,- 1,- 1,- 1,- 1,55,56,57,58,59,60,- 1,- 1,- 1,- 1,65,- 1,- 1,- 1,69,- 1,- 1,72,73,- 1,75,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,91,92,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,114,115,116,117,118,119,120,- 1,- 1,123,- 1,- 1,126,127,- 1,- 1,- 1,131,- 1,- 1,- 1,135,- 1,- 1,138,25,26,- 1,- 1,- 1,- 1,145,- 1,- 1,148,149,- 1,- 1,152,153,40,41,- 1,43,- 1,- 1,- 1,- 1,- 1,49,- 1,- 1,- 1,- 1,- 1,55,56,57,58,59,60,- 1,- 1,- 1,- 1,65,- 1,- 1,- 1,69,- 1,- 1,72,73,- 1,75,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,91,92,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,114,115,116,117,118,119,120,- 1,- 1,123,- 1,- 1,126,- 1,- 1,- 1,- 1,131,- 1,- 1,- 1,135,136,- 1,138,25,26,- 1,- 1,- 1,- 1,145,- 1,- 1,148,149,- 1,- 1,152,153,40,41,- 1,43,- 1,- 1,- 1,- 1,- 1,49,- 1,- 1,- 1,- 1,- 1,55,56,57,58,59,60,- 1,- 1,- 1,- 1,65,- 1,- 1,- 1,69,- 1,- 1,72,73,- 1,75,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,91,92,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,114,115,116,117,118,119,120,- 1,- 1,123,- 1,- 1,126,- 1,- 1,- 1,- 1,131,- 1,- 1,- 1,135,136,- 1,138,25,26,- 1,- 1,- 1,- 1,145,- 1,- 1,148,149,- 1,- 1,152,153,40,41,- 1,43,- 1,- 1,- 1,- 1,- 1,49,- 1,- 1,- 1,- 1,- 1,55,56,57,58,59,60,- 1,- 1,- 1,- 1,65,- 1,- 1,- 1,69,- 1,- 1,72,73,- 1,75,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,91,92,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,114,115,116,117,118,119,120,- 1,- 1,123,- 1,- 1,126,- 1,- 1,- 1,- 1,131,- 1,- 1,- 1,135,136,- 1,138,25,26,- 1,- 1,- 1,- 1,145,- 1,- 1,148,149,- 1,- 1,152,153,40,41,- 1,43,- 1,- 1,- 1,- 1,- 1,49,- 1,- 1,- 1,- 1,- 1,55,56,57,58,59,60,- 1,- 1,- 1,- 1,65,- 1,- 1,- 1,69,- 1,- 1,72,73,- 1,75,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,91,92,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,114,115,116,117,118,119,120,- 1,- 1,123,- 1,- 1,126,- 1,- 1,- 1,- 1,131,- 1,- 1,- 1,135,136,- 1,138,25,26,- 1,- 1,- 1,- 1,145,- 1,- 1,148,149,- 1,- 1,152,153,40,41,- 1,43,- 1,- 1,- 1,- 1,- 1,49,- 1,- 1,- 1,- 1,- 1,55,56,57,58,59,60,- 1,- 1,- 1,- 1,65,- 1,- 1,- 1,69,- 1,- 1,72,73,- 1,75,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,91,92,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,114,115,116,117,118,119,120,- 1,- 1,123,- 1,- 1,126,- 1,- 1,- 1,- 1,131,- 1,- 1,- 1,135,- 1,- 1,138,25,26,- 1,- 1,- 1,- 1,145,- 1,- 1,148,149,- 1,- 1,152,153,40,41,- 1,43,- 1,- 1,- 1,- 1,- 1,49,- 1,- 1,- 1,- 1,- 1,55,56,57,58,59,60,- 1,- 1,- 1,- 1,65,- 1,- 1,- 1,69,- 1,- 1,72,73,- 1,75,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,91,92,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,114,115,116,117,118,119,120,- 1,- 1,123,25,26,126,- 1,- 1,- 1,- 1,131,- 1,- 1,- 1,135,- 1,- 1,138,40,41,- 1,- 1,- 1,- 1,145,- 1,- 1,148,149,- 1,- 1,152,153,55,56,57,58,59,60,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,69,- 1,- 1,72,73,- 1,75,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,91,92,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,114,115,116,117,118,119,120,- 1,- 1,123,25,26,126,- 1,128,- 1,- 1,131,- 1,- 1,- 1,135,- 1,- 1,138,40,41,- 1,- 1,- 1,- 1,145,- 1,- 1,148,149,- 1,- 1,152,153,55,56,57,58,59,60,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,69,- 1,- 1,72,73,- 1,75,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,91,92,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,114,115,116,117,118,119,120,- 1,- 1,123,25,26,126,- 1,128,- 1,- 1,131,- 1,- 1,- 1,135,- 1,- 1,138,40,41,- 1,- 1,- 1,- 1,145,- 1,- 1,148,149,- 1,- 1,152,153,55,56,57,58,59,60,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,69,- 1,- 1,72,73,- 1,75,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,91,92,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,114,115,116,117,118,119,120,- 1,- 1,123,25,26,126,- 1,- 1,- 1,- 1,131,- 1,- 1,- 1,135,- 1,- 1,138,40,41,- 1,- 1,- 1,- 1,145,- 1,- 1,148,149,- 1,- 1,152,153,55,56,57,58,59,60,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,69,- 1,- 1,72,73,- 1,75,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,91,92,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,114,115,116,117,118,119,120,- 1,- 1,123,25,26,126,- 1,- 1,- 1,- 1,131,- 1,- 1,- 1,135,- 1,- 1,138,40,41,- 1,- 1,- 1,- 1,145,- 1,- 1,148,149,- 1,- 1,152,153,55,56,57,58,59,60,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,69,- 1,- 1,72,73,- 1,75,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,91,92,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,114,115,116,117,118,119,120,- 1,- 1,123,25,26,126,- 1,- 1,- 1,- 1,131,- 1,- 1,- 1,135,- 1,- 1,138,40,41,- 1,- 1,- 1,- 1,145,- 1,- 1,148,149,- 1,- 1,152,153,55,56,57,58,59,60,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,69,- 1,- 1,72,73,- 1,75,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,91,92,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,- 1,114,115,116,117,118,119,120,- 1,- 1,123,- 1,- 1,126,- 1,- 1,- 1,- 1,131,- 1,- 1,- 1,135,- 1,- 1,138,- 1,- 1,- 1,- 1,- 1,- 1,145,- 1,- 1,148,149,- 1,- 1,152,153};char Cyc_Yystack_overflow[17U]="Yystack_overflow";struct Cyc_Yystack_overflow_exn_struct{char*tag;int f1;};
# 45 "cycbison.simple"
struct Cyc_Yystack_overflow_exn_struct Cyc_Yystack_overflow_val={Cyc_Yystack_overflow,0};
# 72 "cycbison.simple"
void Cyc_yyerror(struct _fat_ptr,int state,int token);
# 82 "cycbison.simple"
int Cyc_yylex(struct Cyc_Lexing_lexbuf*,union Cyc_YYSTYPE*yylval_ptr,struct Cyc_Yyltype*yylloc);struct Cyc_Yystacktype{union Cyc_YYSTYPE v;struct Cyc_Yyltype l;};struct _tuple32{unsigned f1;struct _tuple0*f2;int f3;};struct _tuple33{void*f1;void*f2;};struct _tuple34{struct Cyc_List_List*f1;struct Cyc_Absyn_Exp*f2;};
# 145 "cycbison.simple"
int Cyc_yyparse(struct _RegionHandle*yyr,struct Cyc_Lexing_lexbuf*yylex_buf){
# 148
struct _RegionHandle _tmp369=_new_region("yyregion");struct _RegionHandle*yyregion=& _tmp369;_push_region(yyregion);
{int yystate;
int yyn=0;
int yyerrstatus;
int yychar1=0;
# 154
int yychar;
union Cyc_YYSTYPE yylval=({union Cyc_YYSTYPE _tmp67E;(_tmp67E.YYINITIALSVAL).tag=73U,(_tmp67E.YYINITIALSVAL).val=0;_tmp67E;});
int yynerrs;
# 158
struct Cyc_Yyltype yylloc;
# 162
int yyssp_offset;
# 164
struct _fat_ptr yyss=({unsigned _tmp5C2=200U;_tag_fat(_region_calloc(yyregion,sizeof(short),_tmp5C2),sizeof(short),_tmp5C2);});
# 166
int yyvsp_offset;
# 168
struct _fat_ptr yyvs=
_tag_fat(({unsigned _tmp5C1=200U;struct Cyc_Yystacktype*_tmp5C0=({struct _RegionHandle*_tmp746=yyregion;_region_malloc(_tmp746,_check_times(_tmp5C1,sizeof(struct Cyc_Yystacktype)));});({{unsigned _tmp67D=200U;unsigned i;for(i=0;i < _tmp67D;++ i){(_tmp5C0[i]).v=yylval,({struct Cyc_Yyltype _tmp747=Cyc_yynewloc();(_tmp5C0[i]).l=_tmp747;});}}0;});_tmp5C0;}),sizeof(struct Cyc_Yystacktype),200U);
# 174
struct Cyc_Yystacktype*yyyvsp;
# 177
int yystacksize=200;
# 179
union Cyc_YYSTYPE yyval=yylval;
# 183
int yylen;
# 190
yystate=0;
yyerrstatus=0;
yynerrs=0;
yychar=-2;
# 200
yyssp_offset=-1;
yyvsp_offset=0;
# 206
yynewstate:
# 208
*((short*)_check_fat_subscript(yyss,sizeof(short),++ yyssp_offset))=(short)yystate;
# 210
if(yyssp_offset >= (yystacksize - 1)- 12){
# 212
if(yystacksize >= 10000){
({struct _fat_ptr _tmp749=({const char*_tmp36A="parser stack overflow";_tag_fat(_tmp36A,sizeof(char),22U);});int _tmp748=yystate;Cyc_yyerror(_tmp749,_tmp748,yychar);});
(int)_throw((void*)& Cyc_Yystack_overflow_val);}
# 212
yystacksize *=2;
# 217
if(yystacksize > 10000)
yystacksize=10000;{
# 217
struct _fat_ptr yyss1=({unsigned _tmp36E=(unsigned)yystacksize;short*_tmp36D=({struct _RegionHandle*_tmp74A=yyregion;_region_malloc(_tmp74A,_check_times(_tmp36E,sizeof(short)));});({{unsigned _tmp63B=(unsigned)yystacksize;unsigned i;for(i=0;i < _tmp63B;++ i){
# 220
i <= (unsigned)yyssp_offset?_tmp36D[i]=*((short*)_check_fat_subscript(yyss,sizeof(short),(int)i)):(_tmp36D[i]=0);}}0;});_tag_fat(_tmp36D,sizeof(short),_tmp36E);});
# 222
struct _fat_ptr yyvs1=({unsigned _tmp36C=(unsigned)yystacksize;struct Cyc_Yystacktype*_tmp36B=({struct _RegionHandle*_tmp74B=yyregion;_region_malloc(_tmp74B,_check_times(_tmp36C,sizeof(struct Cyc_Yystacktype)));});({{unsigned _tmp63A=(unsigned)yystacksize;unsigned i;for(i=0;i < _tmp63A;++ i){
# 224
i <= (unsigned)yyssp_offset?_tmp36B[i]=*((struct Cyc_Yystacktype*)_check_fat_subscript(yyvs,sizeof(struct Cyc_Yystacktype),(int)i)):(_tmp36B[i]=*((struct Cyc_Yystacktype*)_check_fat_subscript(yyvs,sizeof(struct Cyc_Yystacktype),0)));}}0;});_tag_fat(_tmp36B,sizeof(struct Cyc_Yystacktype),_tmp36C);});
# 230
yyss=yyss1;
yyvs=yyvs1;}}
# 210
goto yybackup;
# 242
yybackup:
# 254 "cycbison.simple"
 yyn=(int)*((short*)_check_known_subscript_notnull(Cyc_yypact,1170U,sizeof(short),yystate));
if(yyn == -32768)goto yydefault;if(yychar == -2)
# 267
yychar=Cyc_yylex(yylex_buf,& yylval,& yylloc);
# 255
if(yychar <= 0){
# 273
yychar1=0;
yychar=0;}else{
# 282
yychar1=yychar > 0 && yychar <= 381?(int)*((short*)_check_known_subscript_notnull(Cyc_yytranslate,382U,sizeof(short),yychar)): 319;}
# 299 "cycbison.simple"
yyn +=yychar1;
if((yyn < 0 || yyn > 8195)||(int)*((short*)_check_known_subscript_notnull(Cyc_yycheck,8196U,sizeof(short),yyn))!= yychar1)goto yydefault;yyn=(int)Cyc_yytable[yyn];
# 309
if(yyn < 0){
# 311
if(yyn == -32768)goto yyerrlab;yyn=- yyn;
# 313
goto yyreduce;}else{
# 315
if(yyn == 0)goto yyerrlab;}
# 309
if(yyn == 1169){
# 318
int _tmp36F=0;_npop_handler(0U);return _tmp36F;}
# 309
if(yychar != 0)
# 329 "cycbison.simple"
yychar=-2;
# 309 "cycbison.simple"
({struct Cyc_Yystacktype _tmp74C=({struct Cyc_Yystacktype _tmp63C;_tmp63C.v=yylval,_tmp63C.l=yylloc;_tmp63C;});*((struct Cyc_Yystacktype*)_check_fat_subscript(yyvs,sizeof(struct Cyc_Yystacktype),++ yyvsp_offset))=_tmp74C;});
# 338 "cycbison.simple"
if(yyerrstatus != 0)-- yyerrstatus;yystate=yyn;
# 341
goto yynewstate;
# 344
yydefault:
# 346
 yyn=(int)Cyc_yydefact[yystate];
if(yyn == 0)goto yyerrlab;yyreduce:
# 353
 yylen=(int)*((short*)_check_known_subscript_notnull(Cyc_yyr2,568U,sizeof(short),yyn));
yyyvsp=(struct Cyc_Yystacktype*)_check_null(_untag_fat_ptr(_fat_ptr_plus(yyvs,sizeof(struct Cyc_Yystacktype),(yyvsp_offset + 1)- yylen),sizeof(struct Cyc_Yystacktype),12U));
if(yylen > 0)
yyval=(yyyvsp[0]).v;
# 355
{int _tmp370=yyn;switch(_tmp370){case 1U: _LL1: _LL2:
# 1190 "parse.y"
 yyval=(yyyvsp[0]).v;
Cyc_Parse_parse_result=Cyc_yyget_YY16(&(yyyvsp[0]).v);
# 1193
goto _LL0;case 2U: _LL3: _LL4:
# 1196 "parse.y"
 yyval=Cyc_YY16(({struct Cyc_List_List*_tmp74D=Cyc_yyget_YY16(&(yyyvsp[0]).v);Cyc_List_imp_append(_tmp74D,Cyc_yyget_YY16(&(yyyvsp[1]).v));}));
goto _LL0;case 3U: _LL5: _LL6:
# 1200 "parse.y"
 yyval=Cyc_YY16(({struct Cyc_List_List*_tmp373=_cycalloc(sizeof(*_tmp373));({struct Cyc_Absyn_Decl*_tmp752=({struct Cyc_Absyn_Decl*_tmp372=_cycalloc(sizeof(*_tmp372));({void*_tmp751=(void*)({struct Cyc_Absyn_Using_d_Absyn_Raw_decl_struct*_tmp371=_cycalloc(sizeof(*_tmp371));_tmp371->tag=10U,({struct _tuple0*_tmp750=Cyc_yyget_QualId_tok(&(yyyvsp[0]).v);_tmp371->f1=_tmp750;}),({struct Cyc_List_List*_tmp74F=Cyc_yyget_YY16(&(yyyvsp[2]).v);_tmp371->f2=_tmp74F;});_tmp371;});_tmp372->r=_tmp751;}),({unsigned _tmp74E=(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;});_tmp372->loc=_tmp74E;});_tmp372;});_tmp373->hd=_tmp752;}),_tmp373->tl=0;_tmp373;}));
Cyc_Lex_leave_using();
# 1203
goto _LL0;case 4U: _LL7: _LL8:
# 1204 "parse.y"
 yyval=Cyc_YY16(({struct Cyc_List_List*_tmp376=_cycalloc(sizeof(*_tmp376));({struct Cyc_Absyn_Decl*_tmp758=({struct Cyc_Absyn_Decl*_tmp375=_cycalloc(sizeof(*_tmp375));({void*_tmp757=(void*)({struct Cyc_Absyn_Using_d_Absyn_Raw_decl_struct*_tmp374=_cycalloc(sizeof(*_tmp374));_tmp374->tag=10U,({struct _tuple0*_tmp756=Cyc_yyget_QualId_tok(&(yyyvsp[0]).v);_tmp374->f1=_tmp756;}),({struct Cyc_List_List*_tmp755=Cyc_yyget_YY16(&(yyyvsp[2]).v);_tmp374->f2=_tmp755;});_tmp374;});_tmp375->r=_tmp757;}),({unsigned _tmp754=(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;});_tmp375->loc=_tmp754;});_tmp375;});_tmp376->hd=_tmp758;}),({struct Cyc_List_List*_tmp753=Cyc_yyget_YY16(&(yyyvsp[4]).v);_tmp376->tl=_tmp753;});_tmp376;}));
goto _LL0;case 5U: _LL9: _LLA:
# 1207
 yyval=Cyc_YY16(({struct Cyc_List_List*_tmp37A=_cycalloc(sizeof(*_tmp37A));({struct Cyc_Absyn_Decl*_tmp75E=({struct Cyc_Absyn_Decl*_tmp379=_cycalloc(sizeof(*_tmp379));({void*_tmp75D=(void*)({struct Cyc_Absyn_Namespace_d_Absyn_Raw_decl_struct*_tmp378=_cycalloc(sizeof(*_tmp378));_tmp378->tag=9U,({struct _fat_ptr*_tmp75C=({struct _fat_ptr*_tmp377=_cycalloc(sizeof(*_tmp377));({struct _fat_ptr _tmp75B=Cyc_yyget_String_tok(&(yyyvsp[0]).v);*_tmp377=_tmp75B;});_tmp377;});_tmp378->f1=_tmp75C;}),({struct Cyc_List_List*_tmp75A=Cyc_yyget_YY16(&(yyyvsp[2]).v);_tmp378->f2=_tmp75A;});_tmp378;});_tmp379->r=_tmp75D;}),({unsigned _tmp759=(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;});_tmp379->loc=_tmp759;});_tmp379;});_tmp37A->hd=_tmp75E;}),_tmp37A->tl=0;_tmp37A;}));
Cyc_Lex_leave_namespace();
# 1210
goto _LL0;case 6U: _LLB: _LLC:
# 1211 "parse.y"
 yyval=Cyc_YY16(({struct Cyc_List_List*_tmp37E=_cycalloc(sizeof(*_tmp37E));({struct Cyc_Absyn_Decl*_tmp765=({struct Cyc_Absyn_Decl*_tmp37D=_cycalloc(sizeof(*_tmp37D));({void*_tmp764=(void*)({struct Cyc_Absyn_Namespace_d_Absyn_Raw_decl_struct*_tmp37C=_cycalloc(sizeof(*_tmp37C));_tmp37C->tag=9U,({struct _fat_ptr*_tmp763=({struct _fat_ptr*_tmp37B=_cycalloc(sizeof(*_tmp37B));({struct _fat_ptr _tmp762=Cyc_yyget_String_tok(&(yyyvsp[0]).v);*_tmp37B=_tmp762;});_tmp37B;});_tmp37C->f1=_tmp763;}),({struct Cyc_List_List*_tmp761=Cyc_yyget_YY16(&(yyyvsp[2]).v);_tmp37C->f2=_tmp761;});_tmp37C;});_tmp37D->r=_tmp764;}),({unsigned _tmp760=(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;});_tmp37D->loc=_tmp760;});_tmp37D;});_tmp37E->hd=_tmp765;}),({struct Cyc_List_List*_tmp75F=Cyc_yyget_YY16(&(yyyvsp[4]).v);_tmp37E->tl=_tmp75F;});_tmp37E;}));
goto _LL0;case 7U: _LLD: _LLE: {
# 1213 "parse.y"
int is_c_include=Cyc_yyget_YY31(&(yyyvsp[0]).v);
struct Cyc_List_List*cycdecls=Cyc_yyget_YY16(&(yyyvsp[4]).v);
struct _tuple28*_stmttmp17=Cyc_yyget_YY53(&(yyyvsp[5]).v);unsigned _tmp380;struct Cyc_List_List*_tmp37F;_LL46E: _tmp37F=_stmttmp17->f1;_tmp380=_stmttmp17->f2;_LL46F: {struct Cyc_List_List*exs=_tmp37F;unsigned wc=_tmp380;
struct Cyc_List_List*hides=Cyc_yyget_YY54(&(yyyvsp[6]).v);
if(exs != 0 && hides != 0)
({void*_tmp381=0U;({unsigned _tmp767=(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;});struct _fat_ptr _tmp766=({const char*_tmp382="hide list can only be used with export { * }, or no export block";_tag_fat(_tmp382,sizeof(char),65U);});Cyc_Warn_err(_tmp767,_tmp766,_tag_fat(_tmp381,sizeof(void*),0U));});});
# 1217
if(
# 1220
(unsigned)hides && !((int)wc))
wc=(unsigned)((yyyvsp[6]).l).first_line;
# 1217
if(!is_c_include){
# 1224
if(exs != 0 || cycdecls != 0){
({void*_tmp383=0U;({unsigned _tmp769=(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;});struct _fat_ptr _tmp768=({const char*_tmp384="expecting \"C include\"";_tag_fat(_tmp384,sizeof(char),22U);});Cyc_Warn_err(_tmp769,_tmp768,_tag_fat(_tmp383,sizeof(void*),0U));});});
yyval=Cyc_YY16(({struct Cyc_List_List*_tmp388=_cycalloc(sizeof(*_tmp388));({struct Cyc_Absyn_Decl*_tmp76F=({struct Cyc_Absyn_Decl*_tmp387=_cycalloc(sizeof(*_tmp387));({void*_tmp76E=(void*)({struct Cyc_Absyn_ExternCinclude_d_Absyn_Raw_decl_struct*_tmp386=_cycalloc(sizeof(*_tmp386));_tmp386->tag=12U,({struct Cyc_List_List*_tmp76D=Cyc_yyget_YY16(&(yyyvsp[2]).v);_tmp386->f1=_tmp76D;}),_tmp386->f2=cycdecls,_tmp386->f3=exs,({struct _tuple10*_tmp76C=({struct _tuple10*_tmp385=_cycalloc(sizeof(*_tmp385));_tmp385->f1=wc,_tmp385->f2=hides;_tmp385;});_tmp386->f4=_tmp76C;});_tmp386;});_tmp387->r=_tmp76E;}),({unsigned _tmp76B=(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;});_tmp387->loc=_tmp76B;});_tmp387;});_tmp388->hd=_tmp76F;}),({struct Cyc_List_List*_tmp76A=Cyc_yyget_YY16(&(yyyvsp[7]).v);_tmp388->tl=_tmp76A;});_tmp388;}));}else{
# 1229
yyval=Cyc_YY16(({struct Cyc_List_List*_tmp38B=_cycalloc(sizeof(*_tmp38B));({struct Cyc_Absyn_Decl*_tmp774=({struct Cyc_Absyn_Decl*_tmp38A=_cycalloc(sizeof(*_tmp38A));({void*_tmp773=(void*)({struct Cyc_Absyn_ExternC_d_Absyn_Raw_decl_struct*_tmp389=_cycalloc(sizeof(*_tmp389));_tmp389->tag=11U,({struct Cyc_List_List*_tmp772=Cyc_yyget_YY16(&(yyyvsp[2]).v);_tmp389->f1=_tmp772;});_tmp389;});_tmp38A->r=_tmp773;}),({unsigned _tmp771=(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;});_tmp38A->loc=_tmp771;});_tmp38A;});_tmp38B->hd=_tmp774;}),({struct Cyc_List_List*_tmp770=Cyc_yyget_YY16(&(yyyvsp[7]).v);_tmp38B->tl=_tmp770;});_tmp38B;}));}}else{
# 1233
yyval=Cyc_YY16(({struct Cyc_List_List*_tmp38F=_cycalloc(sizeof(*_tmp38F));({struct Cyc_Absyn_Decl*_tmp77A=({struct Cyc_Absyn_Decl*_tmp38E=_cycalloc(sizeof(*_tmp38E));({void*_tmp779=(void*)({struct Cyc_Absyn_ExternCinclude_d_Absyn_Raw_decl_struct*_tmp38D=_cycalloc(sizeof(*_tmp38D));_tmp38D->tag=12U,({struct Cyc_List_List*_tmp778=Cyc_yyget_YY16(&(yyyvsp[2]).v);_tmp38D->f1=_tmp778;}),_tmp38D->f2=cycdecls,_tmp38D->f3=exs,({struct _tuple10*_tmp777=({struct _tuple10*_tmp38C=_cycalloc(sizeof(*_tmp38C));_tmp38C->f1=wc,_tmp38C->f2=hides;_tmp38C;});_tmp38D->f4=_tmp777;});_tmp38D;});_tmp38E->r=_tmp779;}),({unsigned _tmp776=(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;});_tmp38E->loc=_tmp776;});_tmp38E;});_tmp38F->hd=_tmp77A;}),({struct Cyc_List_List*_tmp775=Cyc_yyget_YY16(&(yyyvsp[7]).v);_tmp38F->tl=_tmp775;});_tmp38F;}));}
# 1236
goto _LL0;}}case 8U: _LLF: _LL10:
# 1237 "parse.y"
 yyval=Cyc_YY16(({struct Cyc_List_List*_tmp391=_cycalloc(sizeof(*_tmp391));({struct Cyc_Absyn_Decl*_tmp77C=({struct Cyc_Absyn_Decl*_tmp390=_cycalloc(sizeof(*_tmp390));_tmp390->r=(void*)& Cyc_Absyn_Porton_d_val,_tmp390->loc=(unsigned)((yyyvsp[0]).l).first_line;_tmp390;});_tmp391->hd=_tmp77C;}),({struct Cyc_List_List*_tmp77B=Cyc_yyget_YY16(&(yyyvsp[2]).v);_tmp391->tl=_tmp77B;});_tmp391;}));
goto _LL0;case 9U: _LL11: _LL12:
# 1239 "parse.y"
 yyval=Cyc_YY16(({struct Cyc_List_List*_tmp393=_cycalloc(sizeof(*_tmp393));({struct Cyc_Absyn_Decl*_tmp77E=({struct Cyc_Absyn_Decl*_tmp392=_cycalloc(sizeof(*_tmp392));_tmp392->r=(void*)& Cyc_Absyn_Portoff_d_val,_tmp392->loc=(unsigned)((yyyvsp[0]).l).first_line;_tmp392;});_tmp393->hd=_tmp77E;}),({struct Cyc_List_List*_tmp77D=Cyc_yyget_YY16(&(yyyvsp[2]).v);_tmp393->tl=_tmp77D;});_tmp393;}));
goto _LL0;case 10U: _LL13: _LL14:
# 1241 "parse.y"
 yyval=Cyc_YY16(({struct Cyc_List_List*_tmp395=_cycalloc(sizeof(*_tmp395));({struct Cyc_Absyn_Decl*_tmp780=({struct Cyc_Absyn_Decl*_tmp394=_cycalloc(sizeof(*_tmp394));_tmp394->r=(void*)& Cyc_Absyn_Tempeston_d_val,_tmp394->loc=(unsigned)((yyyvsp[0]).l).first_line;_tmp394;});_tmp395->hd=_tmp780;}),({struct Cyc_List_List*_tmp77F=Cyc_yyget_YY16(&(yyyvsp[2]).v);_tmp395->tl=_tmp77F;});_tmp395;}));
goto _LL0;case 11U: _LL15: _LL16:
# 1243 "parse.y"
 yyval=Cyc_YY16(({struct Cyc_List_List*_tmp397=_cycalloc(sizeof(*_tmp397));({struct Cyc_Absyn_Decl*_tmp782=({struct Cyc_Absyn_Decl*_tmp396=_cycalloc(sizeof(*_tmp396));_tmp396->r=(void*)& Cyc_Absyn_Tempestoff_d_val,_tmp396->loc=(unsigned)((yyyvsp[0]).l).first_line;_tmp396;});_tmp397->hd=_tmp782;}),({struct Cyc_List_List*_tmp781=Cyc_yyget_YY16(&(yyyvsp[2]).v);_tmp397->tl=_tmp781;});_tmp397;}));
goto _LL0;case 12U: _LL17: _LL18:
# 1244 "parse.y"
 yyval=Cyc_YY16(0);
goto _LL0;case 13U: _LL19: _LL1A:
# 1249 "parse.y"
 Cyc_Parse_parsing_tempest=1;
goto _LL0;case 14U: _LL1B: _LL1C:
# 1254 "parse.y"
 Cyc_Parse_parsing_tempest=0;
goto _LL0;case 15U: _LL1D: _LL1E: {
# 1259 "parse.y"
struct _fat_ptr two=Cyc_yyget_String_tok(&(yyyvsp[1]).v);
Cyc_Lex_enter_extern_c();
if(({struct _fat_ptr _tmp783=(struct _fat_ptr)two;Cyc_strcmp(_tmp783,({const char*_tmp398="C";_tag_fat(_tmp398,sizeof(char),2U);}));})== 0)
yyval=Cyc_YY31(0);else{
if(({struct _fat_ptr _tmp784=(struct _fat_ptr)two;Cyc_strcmp(_tmp784,({const char*_tmp399="C include";_tag_fat(_tmp399,sizeof(char),10U);}));})== 0)
yyval=Cyc_YY31(1);else{
# 1266
({void*_tmp39A=0U;({unsigned _tmp786=(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;});struct _fat_ptr _tmp785=({const char*_tmp39B="expecting \"C\" or \"C include\"";_tag_fat(_tmp39B,sizeof(char),29U);});Cyc_Warn_err(_tmp786,_tmp785,_tag_fat(_tmp39A,sizeof(void*),0U));});});
yyval=Cyc_YY31(1);}}
# 1270
goto _LL0;}case 16U: _LL1F: _LL20:
# 1273 "parse.y"
 Cyc_Lex_leave_extern_c();
goto _LL0;case 17U: _LL21: _LL22:
# 1277 "parse.y"
 yyval=Cyc_YY54(0);
goto _LL0;case 18U: _LL23: _LL24:
# 1278 "parse.y"
 yyval=(yyyvsp[2]).v;
goto _LL0;case 19U: _LL25: _LL26:
# 1282 "parse.y"
 yyval=Cyc_YY54(({struct Cyc_List_List*_tmp39C=_cycalloc(sizeof(*_tmp39C));({struct _tuple0*_tmp787=Cyc_yyget_QualId_tok(&(yyyvsp[0]).v);_tmp39C->hd=_tmp787;}),_tmp39C->tl=0;_tmp39C;}));
goto _LL0;case 20U: _LL27: _LL28:
# 1283 "parse.y"
 yyval=Cyc_YY54(({struct Cyc_List_List*_tmp39D=_cycalloc(sizeof(*_tmp39D));({struct _tuple0*_tmp788=Cyc_yyget_QualId_tok(&(yyyvsp[0]).v);_tmp39D->hd=_tmp788;}),_tmp39D->tl=0;_tmp39D;}));
goto _LL0;case 21U: _LL29: _LL2A:
# 1285 "parse.y"
 yyval=Cyc_YY54(({struct Cyc_List_List*_tmp39E=_cycalloc(sizeof(*_tmp39E));({struct _tuple0*_tmp78A=Cyc_yyget_QualId_tok(&(yyyvsp[0]).v);_tmp39E->hd=_tmp78A;}),({struct Cyc_List_List*_tmp789=Cyc_yyget_YY54(&(yyyvsp[2]).v);_tmp39E->tl=_tmp789;});_tmp39E;}));
goto _LL0;case 22U: _LL2B: _LL2C:
# 1289 "parse.y"
 yyval=Cyc_YY53(({struct _tuple28*_tmp39F=_cycalloc(sizeof(*_tmp39F));_tmp39F->f1=0,_tmp39F->f2=0U;_tmp39F;}));
goto _LL0;case 23U: _LL2D: _LL2E:
# 1290 "parse.y"
 yyval=(yyyvsp[0]).v;
goto _LL0;case 24U: _LL2F: _LL30:
# 1294 "parse.y"
 yyval=Cyc_YY53(({struct _tuple28*_tmp3A0=_cycalloc(sizeof(*_tmp3A0));({struct Cyc_List_List*_tmp78B=Cyc_yyget_YY52(&(yyyvsp[2]).v);_tmp3A0->f1=_tmp78B;}),_tmp3A0->f2=0U;_tmp3A0;}));
goto _LL0;case 25U: _LL31: _LL32:
# 1295 "parse.y"
 yyval=Cyc_YY53(({struct _tuple28*_tmp3A1=_cycalloc(sizeof(*_tmp3A1));_tmp3A1->f1=0,_tmp3A1->f2=0U;_tmp3A1;}));
goto _LL0;case 26U: _LL33: _LL34:
# 1296 "parse.y"
 yyval=Cyc_YY53(({struct _tuple28*_tmp3A2=_cycalloc(sizeof(*_tmp3A2));_tmp3A2->f1=0,_tmp3A2->f2=(unsigned)((yyyvsp[0]).l).first_line;_tmp3A2;}));
goto _LL0;case 27U: _LL35: _LL36:
# 1300 "parse.y"
 yyval=Cyc_YY52(({struct Cyc_List_List*_tmp3A4=_cycalloc(sizeof(*_tmp3A4));({struct _tuple32*_tmp78D=({struct _tuple32*_tmp3A3=_cycalloc(sizeof(*_tmp3A3));_tmp3A3->f1=(unsigned)((yyyvsp[0]).l).first_line,({struct _tuple0*_tmp78C=Cyc_yyget_QualId_tok(&(yyyvsp[0]).v);_tmp3A3->f2=_tmp78C;}),_tmp3A3->f3=0;_tmp3A3;});_tmp3A4->hd=_tmp78D;}),_tmp3A4->tl=0;_tmp3A4;}));
goto _LL0;case 28U: _LL37: _LL38:
# 1301 "parse.y"
 yyval=Cyc_YY52(({struct Cyc_List_List*_tmp3A6=_cycalloc(sizeof(*_tmp3A6));({struct _tuple32*_tmp78F=({struct _tuple32*_tmp3A5=_cycalloc(sizeof(*_tmp3A5));_tmp3A5->f1=(unsigned)((yyyvsp[0]).l).first_line,({struct _tuple0*_tmp78E=Cyc_yyget_QualId_tok(&(yyyvsp[0]).v);_tmp3A5->f2=_tmp78E;}),_tmp3A5->f3=0;_tmp3A5;});_tmp3A6->hd=_tmp78F;}),_tmp3A6->tl=0;_tmp3A6;}));
goto _LL0;case 29U: _LL39: _LL3A:
# 1303 "parse.y"
 yyval=Cyc_YY52(({struct Cyc_List_List*_tmp3A8=_cycalloc(sizeof(*_tmp3A8));({struct _tuple32*_tmp792=({struct _tuple32*_tmp3A7=_cycalloc(sizeof(*_tmp3A7));_tmp3A7->f1=(unsigned)((yyyvsp[0]).l).first_line,({struct _tuple0*_tmp791=Cyc_yyget_QualId_tok(&(yyyvsp[0]).v);_tmp3A7->f2=_tmp791;}),_tmp3A7->f3=0;_tmp3A7;});_tmp3A8->hd=_tmp792;}),({struct Cyc_List_List*_tmp790=Cyc_yyget_YY52(&(yyyvsp[2]).v);_tmp3A8->tl=_tmp790;});_tmp3A8;}));
goto _LL0;case 30U: _LL3B: _LL3C:
# 1307 "parse.y"
 yyval=Cyc_YY16(0);
goto _LL0;case 31U: _LL3D: _LL3E:
# 1308 "parse.y"
 yyval=(yyyvsp[2]).v;
goto _LL0;case 32U: _LL3F: _LL40:
# 1312 "parse.y"
 yyval=Cyc_YY16(({struct Cyc_List_List*_tmp3AA=_cycalloc(sizeof(*_tmp3AA));({struct Cyc_Absyn_Decl*_tmp795=({void*_tmp794=(void*)({struct Cyc_Absyn_Fn_d_Absyn_Raw_decl_struct*_tmp3A9=_cycalloc(sizeof(*_tmp3A9));_tmp3A9->tag=1U,({struct Cyc_Absyn_Fndecl*_tmp793=Cyc_yyget_YY15(&(yyyvsp[0]).v);_tmp3A9->f1=_tmp793;});_tmp3A9;});Cyc_Absyn_new_decl(_tmp794,(unsigned)((yyyvsp[0]).l).first_line);});_tmp3AA->hd=_tmp795;}),_tmp3AA->tl=0;_tmp3AA;}));
goto _LL0;case 33U: _LL41: _LL42:
# 1313 "parse.y"
 yyval=(yyyvsp[0]).v;
goto _LL0;case 34U: _LL43: _LL44:
# 1314 "parse.y"
 yyval=Cyc_YY16(0);
goto _LL0;case 37U: _LL45: _LL46:
# 1323 "parse.y"
 yyval=Cyc_YY15(({struct _RegionHandle*_tmp798=yyr;struct Cyc_Parse_Declarator _tmp797=Cyc_yyget_YY27(&(yyyvsp[0]).v);struct Cyc_Absyn_Stmt*_tmp796=Cyc_yyget_Stmt_tok(&(yyyvsp[1]).v);Cyc_Parse_make_function(_tmp798,0,_tmp797,0,_tmp796,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 38U: _LL47: _LL48: {
# 1325 "parse.y"
struct Cyc_Parse_Declaration_spec d=Cyc_yyget_YY17(&(yyyvsp[0]).v);
yyval=Cyc_YY15(({struct _RegionHandle*_tmp79B=yyr;struct Cyc_Parse_Declarator _tmp79A=Cyc_yyget_YY27(&(yyyvsp[1]).v);struct Cyc_Absyn_Stmt*_tmp799=Cyc_yyget_Stmt_tok(&(yyyvsp[2]).v);Cyc_Parse_make_function(_tmp79B,& d,_tmp79A,0,_tmp799,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;}case 39U: _LL49: _LL4A:
# 1338 "parse.y"
 yyval=Cyc_YY15(({struct _RegionHandle*_tmp79F=yyr;struct Cyc_Parse_Declarator _tmp79E=Cyc_yyget_YY27(&(yyyvsp[0]).v);struct Cyc_List_List*_tmp79D=Cyc_yyget_YY16(&(yyyvsp[1]).v);struct Cyc_Absyn_Stmt*_tmp79C=Cyc_yyget_Stmt_tok(&(yyyvsp[2]).v);Cyc_Parse_make_function(_tmp79F,0,_tmp79E,_tmp79D,_tmp79C,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 40U: _LL4B: _LL4C: {
# 1340 "parse.y"
struct Cyc_Parse_Declaration_spec d=Cyc_yyget_YY17(&(yyyvsp[0]).v);
yyval=Cyc_YY15(({struct _RegionHandle*_tmp7A3=yyr;struct Cyc_Parse_Declarator _tmp7A2=Cyc_yyget_YY27(&(yyyvsp[1]).v);struct Cyc_List_List*_tmp7A1=Cyc_yyget_YY16(&(yyyvsp[2]).v);struct Cyc_Absyn_Stmt*_tmp7A0=Cyc_yyget_Stmt_tok(&(yyyvsp[3]).v);Cyc_Parse_make_function(_tmp7A3,& d,_tmp7A2,_tmp7A1,_tmp7A0,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;}case 41U: _LL4D: _LL4E: {
# 1348 "parse.y"
struct Cyc_Parse_Declaration_spec d=Cyc_yyget_YY17(&(yyyvsp[0]).v);
yyval=Cyc_YY15(({struct _RegionHandle*_tmp7A6=yyr;struct Cyc_Parse_Declarator _tmp7A5=Cyc_yyget_YY27(&(yyyvsp[1]).v);struct Cyc_Absyn_Stmt*_tmp7A4=Cyc_yyget_Stmt_tok(&(yyyvsp[2]).v);Cyc_Parse_make_function(_tmp7A6,& d,_tmp7A5,0,_tmp7A4,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;}case 42U: _LL4F: _LL50: {
# 1351 "parse.y"
struct Cyc_Parse_Declaration_spec d=Cyc_yyget_YY17(&(yyyvsp[0]).v);
yyval=Cyc_YY15(({struct _RegionHandle*_tmp7AA=yyr;struct Cyc_Parse_Declarator _tmp7A9=Cyc_yyget_YY27(&(yyyvsp[1]).v);struct Cyc_List_List*_tmp7A8=Cyc_yyget_YY16(&(yyyvsp[2]).v);struct Cyc_Absyn_Stmt*_tmp7A7=Cyc_yyget_Stmt_tok(&(yyyvsp[3]).v);Cyc_Parse_make_function(_tmp7AA,& d,_tmp7A9,_tmp7A8,_tmp7A7,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;}case 43U: _LL51: _LL52:
# 1356 "parse.y"
 Cyc_Lex_enter_using(Cyc_yyget_QualId_tok(&(yyyvsp[1]).v));yyval=(yyyvsp[1]).v;
goto _LL0;case 44U: _LL53: _LL54:
# 1359
 Cyc_Lex_leave_using();
goto _LL0;case 45U: _LL55: _LL56:
# 1362
 Cyc_Lex_enter_namespace(({struct _fat_ptr*_tmp3AB=_cycalloc(sizeof(*_tmp3AB));({struct _fat_ptr _tmp7AB=Cyc_yyget_String_tok(&(yyyvsp[1]).v);*_tmp3AB=_tmp7AB;});_tmp3AB;}));yyval=(yyyvsp[1]).v;
goto _LL0;case 46U: _LL57: _LL58:
# 1365
 Cyc_Lex_leave_namespace();
goto _LL0;case 47U: _LL59: _LL5A: {
# 1371 "parse.y"
int location=((yyyvsp[0]).l).first_line;
yyval=Cyc_YY16(({struct Cyc_Parse_Declaration_spec _tmp7AD=Cyc_yyget_YY17(&(yyyvsp[0]).v);unsigned _tmp7AC=(unsigned)location;Cyc_Parse_make_declarations(_tmp7AD,0,_tmp7AC,(unsigned)location);}));
goto _LL0;}case 48U: _LL5B: _LL5C: {
# 1374 "parse.y"
int _tmp3AC=({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;});int location=_tmp3AC;
yyval=Cyc_YY16(({struct Cyc_Parse_Declaration_spec _tmp7B0=Cyc_yyget_YY17(&(yyyvsp[0]).v);struct _tuple13*_tmp7AF=Cyc_yyget_YY19(&(yyyvsp[1]).v);unsigned _tmp7AE=(unsigned)((yyyvsp[0]).l).first_line;Cyc_Parse_make_declarations(_tmp7B0,_tmp7AF,_tmp7AE,(unsigned)location);}));
goto _LL0;}case 49U: _LL5D: _LL5E:
# 1378
 yyval=Cyc_YY16(({struct Cyc_List_List*_tmp3AD=_cycalloc(sizeof(*_tmp3AD));({struct Cyc_Absyn_Decl*_tmp7B3=({struct Cyc_Absyn_Pat*_tmp7B2=Cyc_yyget_YY9(&(yyyvsp[1]).v);struct Cyc_Absyn_Exp*_tmp7B1=Cyc_yyget_Exp_tok(&(yyyvsp[3]).v);Cyc_Absyn_let_decl(_tmp7B2,_tmp7B1,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));});_tmp3AD->hd=_tmp7B3;}),_tmp3AD->tl=0;_tmp3AD;}));
goto _LL0;case 50U: _LL5F: _LL60: {
# 1380 "parse.y"
struct Cyc_List_List*vds=0;
{struct Cyc_List_List*ids=Cyc_yyget_YY36(&(yyyvsp[1]).v);for(0;ids != 0;ids=ids->tl){
struct _fat_ptr*id=(struct _fat_ptr*)ids->hd;
struct _tuple0*qv=({struct _tuple0*_tmp3AF=_cycalloc(sizeof(*_tmp3AF));({union Cyc_Absyn_Nmspace _tmp7B4=Cyc_Absyn_Rel_n(0);_tmp3AF->f1=_tmp7B4;}),_tmp3AF->f2=id;_tmp3AF;});
struct Cyc_Absyn_Vardecl*vd=({struct _tuple0*_tmp7B5=qv;Cyc_Absyn_new_vardecl(0U,_tmp7B5,Cyc_Absyn_wildtyp(0),0);});
vds=({struct Cyc_List_List*_tmp3AE=_cycalloc(sizeof(*_tmp3AE));_tmp3AE->hd=vd,_tmp3AE->tl=vds;_tmp3AE;});}}
# 1387
vds=Cyc_List_imp_rev(vds);
yyval=Cyc_YY16(({struct Cyc_List_List*_tmp3B0=_cycalloc(sizeof(*_tmp3B0));({struct Cyc_Absyn_Decl*_tmp7B7=({struct Cyc_List_List*_tmp7B6=vds;Cyc_Absyn_letv_decl(_tmp7B6,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));});_tmp3B0->hd=_tmp7B7;}),_tmp3B0->tl=0;_tmp3B0;}));
# 1390
goto _LL0;}case 51U: _LL61: _LL62: {
# 1393 "parse.y"
struct _fat_ptr three=Cyc_yyget_String_tok(&(yyyvsp[2]).v);
# 1395
if(({struct _fat_ptr _tmp7B8=(struct _fat_ptr)three;Cyc_zstrcmp(_tmp7B8,({const char*_tmp3B1="`H";_tag_fat(_tmp3B1,sizeof(char),3U);}));})== 0)
({void*_tmp3B2=0U;({unsigned _tmp7BA=(unsigned)((yyyvsp[2]).l).first_line;struct _fat_ptr _tmp7B9=({const char*_tmp3B3="bad occurrence of heap region";_tag_fat(_tmp3B3,sizeof(char),30U);});Cyc_Warn_err(_tmp7BA,_tmp7B9,_tag_fat(_tmp3B2,sizeof(void*),0U));});});
# 1395
if(({
# 1397
struct _fat_ptr _tmp7BB=(struct _fat_ptr)three;Cyc_zstrcmp(_tmp7BB,({const char*_tmp3B4="`U";_tag_fat(_tmp3B4,sizeof(char),3U);}));})== 0)
({void*_tmp3B5=0U;({unsigned _tmp7BD=(unsigned)((yyyvsp[2]).l).first_line;struct _fat_ptr _tmp7BC=({const char*_tmp3B6="bad occurrence of unique region";_tag_fat(_tmp3B6,sizeof(char),32U);});Cyc_Warn_err(_tmp7BD,_tmp7BC,_tag_fat(_tmp3B5,sizeof(void*),0U));});});{
# 1395
struct Cyc_Absyn_Tvar*tv=({struct Cyc_Absyn_Tvar*_tmp3BB=_cycalloc(sizeof(*_tmp3BB));
# 1399
({struct _fat_ptr*_tmp7BF=({struct _fat_ptr*_tmp3BA=_cycalloc(sizeof(*_tmp3BA));*_tmp3BA=three;_tmp3BA;});_tmp3BB->name=_tmp7BF;}),_tmp3BB->identity=- 1,({void*_tmp7BE=Cyc_Tcutil_kind_to_bound(& Cyc_Tcutil_rk);_tmp3BB->kind=_tmp7BE;});_tmp3BB;});
void*t=Cyc_Absyn_var_type(tv);
struct Cyc_Absyn_Vardecl*vd=({unsigned _tmp7C3=(unsigned)((yyyvsp[4]).l).first_line;struct _tuple0*_tmp7C2=({struct _tuple0*_tmp3B9=_cycalloc(sizeof(*_tmp3B9));_tmp3B9->f1=Cyc_Absyn_Loc_n,({struct _fat_ptr*_tmp7C1=({struct _fat_ptr*_tmp3B8=_cycalloc(sizeof(*_tmp3B8));({struct _fat_ptr _tmp7C0=Cyc_yyget_String_tok(&(yyyvsp[4]).v);*_tmp3B8=_tmp7C0;});_tmp3B8;});_tmp3B9->f2=_tmp7C1;});_tmp3B9;});Cyc_Absyn_new_vardecl(_tmp7C3,_tmp7C2,Cyc_Absyn_rgn_handle_type(t),0);});
yyval=Cyc_YY16(({struct Cyc_List_List*_tmp3B7=_cycalloc(sizeof(*_tmp3B7));({struct Cyc_Absyn_Decl*_tmp7C6=({struct Cyc_Absyn_Tvar*_tmp7C5=tv;struct Cyc_Absyn_Vardecl*_tmp7C4=vd;Cyc_Absyn_region_decl(_tmp7C5,_tmp7C4,0,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));});_tmp3B7->hd=_tmp7C6;}),_tmp3B7->tl=0;_tmp3B7;}));
# 1404
goto _LL0;}}case 52U: _LL63: _LL64: {
# 1406
struct _fat_ptr two=Cyc_yyget_String_tok(&(yyyvsp[1]).v);
if(({struct _fat_ptr _tmp7C7=(struct _fat_ptr)two;Cyc_zstrcmp(_tmp7C7,({const char*_tmp3BC="H";_tag_fat(_tmp3BC,sizeof(char),2U);}));})== 0)
({void*_tmp3BD=0U;({unsigned _tmp7C9=(unsigned)((yyyvsp[1]).l).first_line;struct _fat_ptr _tmp7C8=({const char*_tmp3BE="bad occurrence of heap region `H";_tag_fat(_tmp3BE,sizeof(char),33U);});Cyc_Warn_err(_tmp7C9,_tmp7C8,_tag_fat(_tmp3BD,sizeof(void*),0U));});});
# 1407
if(({
# 1409
struct _fat_ptr _tmp7CA=(struct _fat_ptr)two;Cyc_zstrcmp(_tmp7CA,({const char*_tmp3BF="U";_tag_fat(_tmp3BF,sizeof(char),2U);}));})== 0)
({void*_tmp3C0=0U;({unsigned _tmp7CC=(unsigned)((yyyvsp[1]).l).first_line;struct _fat_ptr _tmp7CB=({const char*_tmp3C1="bad occurrence of unique region `U";_tag_fat(_tmp3C1,sizeof(char),35U);});Cyc_Warn_err(_tmp7CC,_tmp7CB,_tag_fat(_tmp3C0,sizeof(void*),0U));});});{
# 1407
struct Cyc_Absyn_Tvar*tv=({struct Cyc_Absyn_Tvar*_tmp3C9=_cycalloc(sizeof(*_tmp3C9));
# 1411
({struct _fat_ptr*_tmp7D0=({struct _fat_ptr*_tmp3C8=_cycalloc(sizeof(*_tmp3C8));({struct _fat_ptr _tmp7CF=(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp3C7=({struct Cyc_String_pa_PrintArg_struct _tmp63D;_tmp63D.tag=0U,_tmp63D.f1=(struct _fat_ptr)((struct _fat_ptr)two);_tmp63D;});void*_tmp3C5[1U];_tmp3C5[0]=& _tmp3C7;({struct _fat_ptr _tmp7CE=({const char*_tmp3C6="`%s";_tag_fat(_tmp3C6,sizeof(char),4U);});Cyc_aprintf(_tmp7CE,_tag_fat(_tmp3C5,sizeof(void*),1U));});});*_tmp3C8=_tmp7CF;});_tmp3C8;});_tmp3C9->name=_tmp7D0;}),_tmp3C9->identity=- 1,({
void*_tmp7CD=Cyc_Tcutil_kind_to_bound(& Cyc_Tcutil_rk);_tmp3C9->kind=_tmp7CD;});_tmp3C9;});
void*t=Cyc_Absyn_var_type(tv);
struct Cyc_Absyn_Vardecl*vd=({unsigned _tmp7D3=(unsigned)((yyyvsp[1]).l).first_line;struct _tuple0*_tmp7D2=({struct _tuple0*_tmp3C4=_cycalloc(sizeof(*_tmp3C4));_tmp3C4->f1=Cyc_Absyn_Loc_n,({struct _fat_ptr*_tmp7D1=({struct _fat_ptr*_tmp3C3=_cycalloc(sizeof(*_tmp3C3));*_tmp3C3=two;_tmp3C3;});_tmp3C4->f2=_tmp7D1;});_tmp3C4;});Cyc_Absyn_new_vardecl(_tmp7D3,_tmp7D2,Cyc_Absyn_rgn_handle_type(t),0);});
yyval=Cyc_YY16(({struct Cyc_List_List*_tmp3C2=_cycalloc(sizeof(*_tmp3C2));({struct Cyc_Absyn_Decl*_tmp7D6=({struct Cyc_Absyn_Tvar*_tmp7D5=tv;struct Cyc_Absyn_Vardecl*_tmp7D4=vd;Cyc_Absyn_region_decl(_tmp7D5,_tmp7D4,0,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));});_tmp3C2->hd=_tmp7D6;}),_tmp3C2->tl=0;_tmp3C2;}));
# 1417
goto _LL0;}}case 53U: _LL65: _LL66: {
# 1419
struct _fat_ptr two=Cyc_yyget_String_tok(&(yyyvsp[1]).v);
struct _fat_ptr four=Cyc_yyget_String_tok(&(yyyvsp[3]).v);
struct Cyc_Absyn_Exp*six=Cyc_yyget_Exp_tok(&(yyyvsp[5]).v);
if(({struct _fat_ptr _tmp7D7=(struct _fat_ptr)four;Cyc_strcmp(_tmp7D7,({const char*_tmp3CA="open";_tag_fat(_tmp3CA,sizeof(char),5U);}));})!= 0)({void*_tmp3CB=0U;({unsigned _tmp7D9=(unsigned)((yyyvsp[3]).l).first_line;struct _fat_ptr _tmp7D8=({const char*_tmp3CC="expecting `open'";_tag_fat(_tmp3CC,sizeof(char),17U);});Cyc_Warn_err(_tmp7D9,_tmp7D8,_tag_fat(_tmp3CB,sizeof(void*),0U));});});{struct Cyc_Absyn_Tvar*tv=({struct Cyc_Absyn_Tvar*_tmp3D4=_cycalloc(sizeof(*_tmp3D4));
({struct _fat_ptr*_tmp7DD=({struct _fat_ptr*_tmp3D3=_cycalloc(sizeof(*_tmp3D3));({struct _fat_ptr _tmp7DC=(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp3D2=({struct Cyc_String_pa_PrintArg_struct _tmp63E;_tmp63E.tag=0U,_tmp63E.f1=(struct _fat_ptr)((struct _fat_ptr)two);_tmp63E;});void*_tmp3D0[1U];_tmp3D0[0]=& _tmp3D2;({struct _fat_ptr _tmp7DB=({const char*_tmp3D1="`%s";_tag_fat(_tmp3D1,sizeof(char),4U);});Cyc_aprintf(_tmp7DB,_tag_fat(_tmp3D0,sizeof(void*),1U));});});*_tmp3D3=_tmp7DC;});_tmp3D3;});_tmp3D4->name=_tmp7DD;}),_tmp3D4->identity=- 1,({
void*_tmp7DA=Cyc_Tcutil_kind_to_bound(& Cyc_Tcutil_rk);_tmp3D4->kind=_tmp7DA;});_tmp3D4;});
void*t=Cyc_Absyn_var_type(tv);
struct Cyc_Absyn_Vardecl*vd=({unsigned _tmp7E0=(unsigned)((yyyvsp[2]).l).first_line;struct _tuple0*_tmp7DF=({struct _tuple0*_tmp3CF=_cycalloc(sizeof(*_tmp3CF));_tmp3CF->f1=Cyc_Absyn_Loc_n,({struct _fat_ptr*_tmp7DE=({struct _fat_ptr*_tmp3CE=_cycalloc(sizeof(*_tmp3CE));*_tmp3CE=two;_tmp3CE;});_tmp3CF->f2=_tmp7DE;});_tmp3CF;});Cyc_Absyn_new_vardecl(_tmp7E0,_tmp7DF,Cyc_Absyn_rgn_handle_type(t),0);});
yyval=Cyc_YY16(({struct Cyc_List_List*_tmp3CD=_cycalloc(sizeof(*_tmp3CD));({struct Cyc_Absyn_Decl*_tmp7E4=({struct Cyc_Absyn_Tvar*_tmp7E3=tv;struct Cyc_Absyn_Vardecl*_tmp7E2=vd;struct Cyc_Absyn_Exp*_tmp7E1=six;Cyc_Absyn_region_decl(_tmp7E3,_tmp7E2,_tmp7E1,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));});_tmp3CD->hd=_tmp7E4;}),_tmp3CD->tl=0;_tmp3CD;}));
# 1429
goto _LL0;}}case 54U: _LL67: _LL68:
# 1433 "parse.y"
 yyval=(yyyvsp[0]).v;
goto _LL0;case 55U: _LL69: _LL6A:
# 1435 "parse.y"
 yyval=Cyc_YY16(({struct Cyc_List_List*_tmp7E5=Cyc_yyget_YY16(&(yyyvsp[0]).v);Cyc_List_imp_append(_tmp7E5,Cyc_yyget_YY16(&(yyyvsp[1]).v));}));
goto _LL0;case 56U: _LL6B: _LL6C:
# 1441 "parse.y"
 yyval=Cyc_YY17(({struct Cyc_Parse_Declaration_spec _tmp63F;({enum Cyc_Parse_Storage_class*_tmp7E8=Cyc_yyget_YY20(&(yyyvsp[0]).v);_tmp63F.sc=_tmp7E8;}),({struct Cyc_Absyn_Tqual _tmp7E7=Cyc_Absyn_empty_tqual((unsigned)((yyyvsp[0]).l).first_line);_tmp63F.tq=_tmp7E7;}),({
struct Cyc_Parse_Type_specifier _tmp7E6=Cyc_Parse_empty_spec(0U);_tmp63F.type_specs=_tmp7E6;}),_tmp63F.is_inline=0,_tmp63F.attributes=0;_tmp63F;}));
goto _LL0;case 57U: _LL6D: _LL6E: {
# 1444 "parse.y"
struct Cyc_Parse_Declaration_spec two=Cyc_yyget_YY17(&(yyyvsp[1]).v);
if(two.sc != 0)
({void*_tmp3D5=0U;({unsigned _tmp7EA=(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;});struct _fat_ptr _tmp7E9=({const char*_tmp3D6="Only one storage class is allowed in a declaration (missing ';' or ','?)";_tag_fat(_tmp3D6,sizeof(char),73U);});Cyc_Warn_warn(_tmp7EA,_tmp7E9,_tag_fat(_tmp3D5,sizeof(void*),0U));});});
# 1445
yyval=
# 1448
Cyc_YY17(({struct Cyc_Parse_Declaration_spec _tmp640;({enum Cyc_Parse_Storage_class*_tmp7EB=Cyc_yyget_YY20(&(yyyvsp[0]).v);_tmp640.sc=_tmp7EB;}),_tmp640.tq=two.tq,_tmp640.type_specs=two.type_specs,_tmp640.is_inline=two.is_inline,_tmp640.attributes=two.attributes;_tmp640;}));
# 1452
goto _LL0;}case 58U: _LL6F: _LL70:
# 1453 "parse.y"
({void*_tmp3D7=0U;({unsigned _tmp7ED=(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;});struct _fat_ptr _tmp7EC=({const char*_tmp3D8="__extension__ keyword ignored in declaration";_tag_fat(_tmp3D8,sizeof(char),45U);});Cyc_Warn_warn(_tmp7ED,_tmp7EC,_tag_fat(_tmp3D7,sizeof(void*),0U));});});
yyval=(yyyvsp[1]).v;
# 1456
goto _LL0;case 59U: _LL71: _LL72:
# 1457 "parse.y"
 yyval=Cyc_YY17(({struct Cyc_Parse_Declaration_spec _tmp641;_tmp641.sc=0,({struct Cyc_Absyn_Tqual _tmp7EF=Cyc_Absyn_empty_tqual((unsigned)((yyyvsp[0]).l).first_line);_tmp641.tq=_tmp7EF;}),({
struct Cyc_Parse_Type_specifier _tmp7EE=Cyc_yyget_YY21(&(yyyvsp[0]).v);_tmp641.type_specs=_tmp7EE;}),_tmp641.is_inline=0,_tmp641.attributes=0;_tmp641;}));
goto _LL0;case 60U: _LL73: _LL74: {
# 1460 "parse.y"
struct Cyc_Parse_Declaration_spec two=Cyc_yyget_YY17(&(yyyvsp[1]).v);
yyval=Cyc_YY17(({struct Cyc_Parse_Declaration_spec _tmp642;_tmp642.sc=two.sc,_tmp642.tq=two.tq,({
struct Cyc_Parse_Type_specifier _tmp7F2=({unsigned _tmp7F1=(unsigned)((yyyvsp[0]).l).first_line;struct Cyc_Parse_Type_specifier _tmp7F0=two.type_specs;Cyc_Parse_combine_specifiers(_tmp7F1,_tmp7F0,Cyc_yyget_YY21(&(yyyvsp[0]).v));});_tmp642.type_specs=_tmp7F2;}),_tmp642.is_inline=two.is_inline,_tmp642.attributes=two.attributes;_tmp642;}));
# 1466
goto _LL0;}case 61U: _LL75: _LL76:
# 1467 "parse.y"
 yyval=Cyc_YY17(({struct Cyc_Parse_Declaration_spec _tmp643;_tmp643.sc=0,({struct Cyc_Absyn_Tqual _tmp7F4=Cyc_yyget_YY23(&(yyyvsp[0]).v);_tmp643.tq=_tmp7F4;}),({struct Cyc_Parse_Type_specifier _tmp7F3=Cyc_Parse_empty_spec(0U);_tmp643.type_specs=_tmp7F3;}),_tmp643.is_inline=0,_tmp643.attributes=0;_tmp643;}));
goto _LL0;case 62U: _LL77: _LL78: {
# 1469 "parse.y"
struct Cyc_Parse_Declaration_spec two=Cyc_yyget_YY17(&(yyyvsp[1]).v);
yyval=Cyc_YY17(({struct Cyc_Parse_Declaration_spec _tmp644;_tmp644.sc=two.sc,({struct Cyc_Absyn_Tqual _tmp7F6=({struct Cyc_Absyn_Tqual _tmp7F5=Cyc_yyget_YY23(&(yyyvsp[0]).v);Cyc_Absyn_combine_tqual(_tmp7F5,two.tq);});_tmp644.tq=_tmp7F6;}),_tmp644.type_specs=two.type_specs,_tmp644.is_inline=two.is_inline,_tmp644.attributes=two.attributes;_tmp644;}));
# 1474
goto _LL0;}case 63U: _LL79: _LL7A:
# 1475 "parse.y"
 yyval=Cyc_YY17(({struct Cyc_Parse_Declaration_spec _tmp645;_tmp645.sc=0,({struct Cyc_Absyn_Tqual _tmp7F8=Cyc_Absyn_empty_tqual((unsigned)((yyyvsp[0]).l).first_line);_tmp645.tq=_tmp7F8;}),({
struct Cyc_Parse_Type_specifier _tmp7F7=Cyc_Parse_empty_spec(0U);_tmp645.type_specs=_tmp7F7;}),_tmp645.is_inline=1,_tmp645.attributes=0;_tmp645;}));
goto _LL0;case 64U: _LL7B: _LL7C: {
# 1478 "parse.y"
struct Cyc_Parse_Declaration_spec two=Cyc_yyget_YY17(&(yyyvsp[1]).v);
yyval=Cyc_YY17(({struct Cyc_Parse_Declaration_spec _tmp646;_tmp646.sc=two.sc,_tmp646.tq=two.tq,_tmp646.type_specs=two.type_specs,_tmp646.is_inline=1,_tmp646.attributes=two.attributes;_tmp646;}));
# 1482
goto _LL0;}case 65U: _LL7D: _LL7E:
# 1483 "parse.y"
 yyval=Cyc_YY17(({struct Cyc_Parse_Declaration_spec _tmp647;_tmp647.sc=0,({struct Cyc_Absyn_Tqual _tmp7FB=Cyc_Absyn_empty_tqual((unsigned)((yyyvsp[0]).l).first_line);_tmp647.tq=_tmp7FB;}),({
struct Cyc_Parse_Type_specifier _tmp7FA=Cyc_Parse_empty_spec(0U);_tmp647.type_specs=_tmp7FA;}),_tmp647.is_inline=0,({struct Cyc_List_List*_tmp7F9=Cyc_yyget_YY45(&(yyyvsp[0]).v);_tmp647.attributes=_tmp7F9;});_tmp647;}));
goto _LL0;case 66U: _LL7F: _LL80: {
# 1486 "parse.y"
struct Cyc_Parse_Declaration_spec two=Cyc_yyget_YY17(&(yyyvsp[1]).v);
yyval=Cyc_YY17(({struct Cyc_Parse_Declaration_spec _tmp648;_tmp648.sc=two.sc,_tmp648.tq=two.tq,_tmp648.type_specs=two.type_specs,_tmp648.is_inline=two.is_inline,({
# 1489
struct Cyc_List_List*_tmp7FD=({struct Cyc_List_List*_tmp7FC=Cyc_yyget_YY45(&(yyyvsp[0]).v);Cyc_List_imp_append(_tmp7FC,two.attributes);});_tmp648.attributes=_tmp7FD;});_tmp648;}));
goto _LL0;}case 67U: _LL81: _LL82: {
# 1493 "parse.y"
static enum Cyc_Parse_Storage_class auto_sc=Cyc_Parse_Auto_sc;
yyval=Cyc_YY20(& auto_sc);
goto _LL0;}case 68U: _LL83: _LL84: {
# 1495 "parse.y"
static enum Cyc_Parse_Storage_class register_sc=Cyc_Parse_Register_sc;
yyval=Cyc_YY20(& register_sc);
goto _LL0;}case 69U: _LL85: _LL86: {
# 1497 "parse.y"
static enum Cyc_Parse_Storage_class static_sc=Cyc_Parse_Static_sc;
yyval=Cyc_YY20(& static_sc);
goto _LL0;}case 70U: _LL87: _LL88: {
# 1499 "parse.y"
static enum Cyc_Parse_Storage_class extern_sc=Cyc_Parse_Extern_sc;
yyval=Cyc_YY20(& extern_sc);
goto _LL0;}case 71U: _LL89: _LL8A: {
# 1502 "parse.y"
static enum Cyc_Parse_Storage_class externC_sc=Cyc_Parse_ExternC_sc;
if(({struct _fat_ptr _tmp7FE=(struct _fat_ptr)Cyc_yyget_String_tok(&(yyyvsp[1]).v);Cyc_strcmp(_tmp7FE,({const char*_tmp3D9="C";_tag_fat(_tmp3D9,sizeof(char),2U);}));})!= 0)
({void*_tmp3DA=0U;({unsigned _tmp800=(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;});struct _fat_ptr _tmp7FF=({const char*_tmp3DB="only extern or extern \"C\" is allowed";_tag_fat(_tmp3DB,sizeof(char),37U);});Cyc_Warn_err(_tmp800,_tmp7FF,_tag_fat(_tmp3DA,sizeof(void*),0U));});});
# 1503
yyval=
# 1505
Cyc_YY20(& externC_sc);
# 1507
goto _LL0;}case 72U: _LL8B: _LL8C: {
# 1507 "parse.y"
static enum Cyc_Parse_Storage_class typedef_sc=Cyc_Parse_Typedef_sc;
yyval=Cyc_YY20(& typedef_sc);
goto _LL0;}case 73U: _LL8D: _LL8E: {
# 1510 "parse.y"
static enum Cyc_Parse_Storage_class abstract_sc=Cyc_Parse_Abstract_sc;
yyval=Cyc_YY20(& abstract_sc);
goto _LL0;}case 74U: _LL8F: _LL90:
# 1516 "parse.y"
 yyval=Cyc_YY45(0);
goto _LL0;case 75U: _LL91: _LL92:
# 1517 "parse.y"
 yyval=(yyyvsp[0]).v;
goto _LL0;case 76U: _LL93: _LL94:
# 1522 "parse.y"
 yyval=(yyyvsp[3]).v;
goto _LL0;case 77U: _LL95: _LL96:
# 1526 "parse.y"
 yyval=Cyc_YY45(({struct Cyc_List_List*_tmp3DC=_cycalloc(sizeof(*_tmp3DC));({void*_tmp801=Cyc_yyget_YY46(&(yyyvsp[0]).v);_tmp3DC->hd=_tmp801;}),_tmp3DC->tl=0;_tmp3DC;}));
goto _LL0;case 78U: _LL97: _LL98:
# 1527 "parse.y"
 yyval=Cyc_YY45(({struct Cyc_List_List*_tmp3DD=_cycalloc(sizeof(*_tmp3DD));({void*_tmp803=Cyc_yyget_YY46(&(yyyvsp[0]).v);_tmp3DD->hd=_tmp803;}),({struct Cyc_List_List*_tmp802=Cyc_yyget_YY45(&(yyyvsp[2]).v);_tmp3DD->tl=_tmp802;});_tmp3DD;}));
goto _LL0;case 79U: _LL99: _LL9A:
# 1531 "parse.y"
 yyval=Cyc_YY46(({unsigned _tmp804=(unsigned)((yyyvsp[0]).l).first_line;Cyc_Absyn_parse_nullary_att(_tmp804,Cyc_yyget_String_tok(&(yyyvsp[0]).v));}));
goto _LL0;case 80U: _LL9B: _LL9C:
# 1532 "parse.y"
 yyval=Cyc_YY46((void*)& Cyc_Absyn_Const_att_val);
goto _LL0;case 81U: _LL9D: _LL9E:
# 1534 "parse.y"
 yyval=Cyc_YY46(({unsigned _tmp807=(unsigned)((yyyvsp[0]).l).first_line;struct _fat_ptr _tmp806=Cyc_yyget_String_tok(&(yyyvsp[0]).v);unsigned _tmp805=(unsigned)((yyyvsp[2]).l).first_line;Cyc_Absyn_parse_unary_att(_tmp807,_tmp806,_tmp805,Cyc_yyget_Exp_tok(&(yyyvsp[2]).v));}));
goto _LL0;case 82U: _LL9F: _LLA0:
# 1536 "parse.y"
 yyval=Cyc_YY46(({unsigned _tmp80E=(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;});unsigned _tmp80D=(unsigned)((yyyvsp[2]).l).first_line;struct _fat_ptr _tmp80C=Cyc_yyget_String_tok(&(yyyvsp[0]).v);struct _fat_ptr _tmp80B=Cyc_yyget_String_tok(&(yyyvsp[2]).v);unsigned _tmp80A=({
unsigned _tmp808=(unsigned)((yyyvsp[4]).l).first_line;Cyc_Parse_cnst2uint(_tmp808,Cyc_yyget_Int_tok(&(yyyvsp[4]).v));});
# 1536
Cyc_Absyn_parse_format_att(_tmp80E,_tmp80D,_tmp80C,_tmp80B,_tmp80A,({
# 1538
unsigned _tmp809=(unsigned)((yyyvsp[6]).l).first_line;Cyc_Parse_cnst2uint(_tmp809,Cyc_yyget_Int_tok(&(yyyvsp[6]).v));}));}));
goto _LL0;case 83U: _LLA1: _LLA2:
# 1616 "parse.y"
 yyval=(yyyvsp[0]).v;
goto _LL0;case 84U: _LLA3: _LLA4:
# 1618 "parse.y"
 yyval=Cyc_YY21(({void*_tmp810=({struct _tuple0*_tmp80F=Cyc_yyget_QualId_tok(&(yyyvsp[0]).v);Cyc_Absyn_typedef_type(_tmp80F,Cyc_yyget_YY40(&(yyyvsp[1]).v),0,0);});Cyc_Parse_type_spec(_tmp810,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 85U: _LLA5: _LLA6:
# 1622 "parse.y"
 yyval=Cyc_YY21(Cyc_Parse_type_spec(Cyc_Absyn_void_type,(unsigned)((yyyvsp[0]).l).first_line));
goto _LL0;case 86U: _LLA7: _LLA8:
# 1623 "parse.y"
 yyval=Cyc_YY21(Cyc_Parse_type_spec(Cyc_Absyn_char_type,(unsigned)((yyyvsp[0]).l).first_line));
goto _LL0;case 87U: _LLA9: _LLAA:
# 1624 "parse.y"
 yyval=Cyc_YY21(Cyc_Parse_short_spec((unsigned)((yyyvsp[0]).l).first_line));
goto _LL0;case 88U: _LLAB: _LLAC:
# 1625 "parse.y"
 yyval=Cyc_YY21(Cyc_Parse_type_spec(Cyc_Absyn_sint_type,(unsigned)((yyyvsp[0]).l).first_line));
goto _LL0;case 89U: _LLAD: _LLAE:
# 1626 "parse.y"
 yyval=Cyc_YY21(Cyc_Parse_long_spec((unsigned)((yyyvsp[0]).l).first_line));
goto _LL0;case 90U: _LLAF: _LLB0:
# 1627 "parse.y"
 yyval=Cyc_YY21(Cyc_Parse_type_spec(Cyc_Absyn_float_type,(unsigned)((yyyvsp[0]).l).first_line));
goto _LL0;case 91U: _LLB1: _LLB2:
# 1628 "parse.y"
 yyval=Cyc_YY21(Cyc_Parse_type_spec(Cyc_Absyn_double_type,(unsigned)((yyyvsp[0]).l).first_line));
goto _LL0;case 92U: _LLB3: _LLB4:
# 1629 "parse.y"
 yyval=Cyc_YY21(Cyc_Parse_signed_spec((unsigned)((yyyvsp[0]).l).first_line));
goto _LL0;case 93U: _LLB5: _LLB6:
# 1630 "parse.y"
 yyval=Cyc_YY21(Cyc_Parse_unsigned_spec((unsigned)((yyyvsp[0]).l).first_line));
goto _LL0;case 94U: _LLB7: _LLB8:
# 1631 "parse.y"
 yyval=(yyyvsp[0]).v;
goto _LL0;case 95U: _LLB9: _LLBA:
# 1632 "parse.y"
 yyval=(yyyvsp[0]).v;
goto _LL0;case 96U: _LLBB: _LLBC:
# 1635
 yyval=Cyc_YY21(({void*_tmp811=Cyc_Absyn_typeof_type(Cyc_yyget_Exp_tok(&(yyyvsp[2]).v));Cyc_Parse_type_spec(_tmp811,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 97U: _LLBD: _LLBE:
# 1637 "parse.y"
 yyval=Cyc_YY21(({void*_tmp812=Cyc_Absyn_builtin_type(({const char*_tmp3DE="__builtin_va_list";_tag_fat(_tmp3DE,sizeof(char),18U);}),& Cyc_Tcutil_bk);Cyc_Parse_type_spec(_tmp812,(unsigned)((yyyvsp[0]).l).first_line);}));
goto _LL0;case 98U: _LLBF: _LLC0:
# 1639 "parse.y"
 yyval=(yyyvsp[0]).v;
goto _LL0;case 99U: _LLC1: _LLC2:
# 1641 "parse.y"
 yyval=Cyc_YY21(({void*_tmp813=Cyc_yyget_YY44(&(yyyvsp[0]).v);Cyc_Parse_type_spec(_tmp813,(unsigned)((yyyvsp[0]).l).first_line);}));
goto _LL0;case 100U: _LLC3: _LLC4:
# 1643 "parse.y"
 yyval=Cyc_YY21(({void*_tmp814=Cyc_Absyn_new_evar(0,0);Cyc_Parse_type_spec(_tmp814,(unsigned)((yyyvsp[0]).l).first_line);}));
goto _LL0;case 101U: _LLC5: _LLC6:
# 1645 "parse.y"
 yyval=Cyc_YY21(({void*_tmp815=Cyc_Absyn_new_evar(Cyc_Tcutil_kind_to_opt(Cyc_yyget_YY43(&(yyyvsp[2]).v)),0);Cyc_Parse_type_spec(_tmp815,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 102U: _LLC7: _LLC8:
# 1647 "parse.y"
 yyval=Cyc_YY21(({void*_tmp819=(void*)({struct Cyc_Absyn_TupleType_Absyn_Type_struct*_tmp3E0=_cycalloc(sizeof(*_tmp3E0));_tmp3E0->tag=6U,({struct Cyc_List_List*_tmp818=({struct Cyc_List_List*(*_tmp817)(struct _tuple20*(*f)(unsigned,struct _tuple8*),unsigned env,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp3DF)(struct _tuple20*(*f)(unsigned,struct _tuple8*),unsigned env,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct _tuple20*(*f)(unsigned,struct _tuple8*),unsigned env,struct Cyc_List_List*x))Cyc_List_map_c;_tmp3DF;});unsigned _tmp816=(unsigned)((yyyvsp[2]).l).first_line;_tmp817(Cyc_Parse_get_tqual_typ,_tmp816,
Cyc_List_imp_rev(Cyc_yyget_YY38(&(yyyvsp[2]).v)));});
# 1647
_tmp3E0->f1=_tmp818;});_tmp3E0;});Cyc_Parse_type_spec(_tmp819,(unsigned)({
# 1649
yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 103U: _LLC9: _LLCA:
# 1651 "parse.y"
 yyval=Cyc_YY21(({void*_tmp81A=Cyc_Absyn_rgn_handle_type(Cyc_yyget_YY44(&(yyyvsp[2]).v));Cyc_Parse_type_spec(_tmp81A,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 104U: _LLCB: _LLCC:
# 1653 "parse.y"
 yyval=Cyc_YY21(({void*_tmp81B=Cyc_Absyn_rgn_handle_type(Cyc_Absyn_new_evar(& Cyc_Tcutil_rko,0));Cyc_Parse_type_spec(_tmp81B,(unsigned)((yyyvsp[0]).l).first_line);}));
# 1655
goto _LL0;case 105U: _LLCD: _LLCE:
# 1656 "parse.y"
 yyval=Cyc_YY21(({void*_tmp81C=Cyc_Absyn_tag_type(Cyc_yyget_YY44(&(yyyvsp[2]).v));Cyc_Parse_type_spec(_tmp81C,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 106U: _LLCF: _LLD0:
# 1658 "parse.y"
 yyval=Cyc_YY21(({void*_tmp81D=Cyc_Absyn_tag_type(Cyc_Absyn_new_evar(& Cyc_Tcutil_iko,0));Cyc_Parse_type_spec(_tmp81D,(unsigned)((yyyvsp[0]).l).first_line);}));
goto _LL0;case 107U: _LLD1: _LLD2:
# 1660 "parse.y"
 yyval=Cyc_YY21(({void*_tmp81E=Cyc_Absyn_valueof_type(Cyc_yyget_Exp_tok(&(yyyvsp[2]).v));Cyc_Parse_type_spec(_tmp81E,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 108U: _LLD3: _LLD4:
# 1666 "parse.y"
 yyval=Cyc_YY43(({struct _fat_ptr _tmp81F=Cyc_yyget_String_tok(&(yyyvsp[0]).v);Cyc_Parse_id_to_kind(_tmp81F,(unsigned)((yyyvsp[0]).l).first_line);}));
goto _LL0;case 109U: _LLD5: _LLD6: {
# 1670 "parse.y"
unsigned loc=(unsigned)(Cyc_Absyn_porting_c_code?((yyyvsp[0]).l).first_line:(int)0U);
yyval=Cyc_YY23(({struct Cyc_Absyn_Tqual _tmp649;_tmp649.print_const=1,_tmp649.q_volatile=0,_tmp649.q_restrict=0,_tmp649.real_const=1,_tmp649.loc=loc;_tmp649;}));
goto _LL0;}case 110U: _LLD7: _LLD8:
# 1672 "parse.y"
 yyval=Cyc_YY23(({struct Cyc_Absyn_Tqual _tmp64A;_tmp64A.print_const=0,_tmp64A.q_volatile=1,_tmp64A.q_restrict=0,_tmp64A.real_const=0,_tmp64A.loc=0U;_tmp64A;}));
goto _LL0;case 111U: _LLD9: _LLDA:
# 1673 "parse.y"
 yyval=Cyc_YY23(({struct Cyc_Absyn_Tqual _tmp64B;_tmp64B.print_const=0,_tmp64B.q_volatile=0,_tmp64B.q_restrict=1,_tmp64B.real_const=0,_tmp64B.loc=0U;_tmp64B;}));
goto _LL0;case 112U: _LLDB: _LLDC: {
# 1679 "parse.y"
struct Cyc_Absyn_TypeDecl*_tmp3E1=({struct Cyc_Absyn_TypeDecl*_tmp3E6=_cycalloc(sizeof(*_tmp3E6));({void*_tmp825=(void*)({struct Cyc_Absyn_Enum_td_Absyn_Raw_typedecl_struct*_tmp3E5=_cycalloc(sizeof(*_tmp3E5));_tmp3E5->tag=1U,({struct Cyc_Absyn_Enumdecl*_tmp824=({struct Cyc_Absyn_Enumdecl*_tmp3E4=_cycalloc(sizeof(*_tmp3E4));_tmp3E4->sc=Cyc_Absyn_Public,({struct _tuple0*_tmp823=Cyc_yyget_QualId_tok(&(yyyvsp[1]).v);_tmp3E4->name=_tmp823;}),({struct Cyc_Core_Opt*_tmp822=({struct Cyc_Core_Opt*_tmp3E3=_cycalloc(sizeof(*_tmp3E3));({struct Cyc_List_List*_tmp821=Cyc_yyget_YY48(&(yyyvsp[3]).v);_tmp3E3->v=_tmp821;});_tmp3E3;});_tmp3E4->fields=_tmp822;});_tmp3E4;});_tmp3E5->f1=_tmp824;});_tmp3E5;});_tmp3E6->r=_tmp825;}),({
unsigned _tmp820=(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;});_tmp3E6->loc=_tmp820;});_tmp3E6;});
# 1679
struct Cyc_Absyn_TypeDecl*ed=_tmp3E1;
# 1681
yyval=Cyc_YY21(({void*_tmp826=(void*)({struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct*_tmp3E2=_cycalloc(sizeof(*_tmp3E2));_tmp3E2->tag=10U,_tmp3E2->f1=ed,_tmp3E2->f2=0;_tmp3E2;});Cyc_Parse_type_spec(_tmp826,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
# 1683
goto _LL0;}case 113U: _LLDD: _LLDE:
# 1684 "parse.y"
 yyval=Cyc_YY21(({void*_tmp827=Cyc_Absyn_enum_type(Cyc_yyget_QualId_tok(&(yyyvsp[1]).v),0);Cyc_Parse_type_spec(_tmp827,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 114U: _LLDF: _LLE0:
# 1686 "parse.y"
 yyval=Cyc_YY21(({void*_tmp828=Cyc_Absyn_anon_enum_type(Cyc_yyget_YY48(&(yyyvsp[2]).v));Cyc_Parse_type_spec(_tmp828,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 115U: _LLE1: _LLE2:
# 1692 "parse.y"
 yyval=Cyc_YY47(({struct Cyc_Absyn_Enumfield*_tmp3E7=_cycalloc(sizeof(*_tmp3E7));({struct _tuple0*_tmp829=Cyc_yyget_QualId_tok(&(yyyvsp[0]).v);_tmp3E7->name=_tmp829;}),_tmp3E7->tag=0,_tmp3E7->loc=(unsigned)((yyyvsp[0]).l).first_line;_tmp3E7;}));
goto _LL0;case 116U: _LLE3: _LLE4:
# 1694 "parse.y"
 yyval=Cyc_YY47(({struct Cyc_Absyn_Enumfield*_tmp3E8=_cycalloc(sizeof(*_tmp3E8));({struct _tuple0*_tmp82C=Cyc_yyget_QualId_tok(&(yyyvsp[0]).v);_tmp3E8->name=_tmp82C;}),({struct Cyc_Absyn_Exp*_tmp82B=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);_tmp3E8->tag=_tmp82B;}),({unsigned _tmp82A=(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;});_tmp3E8->loc=_tmp82A;});_tmp3E8;}));
goto _LL0;case 117U: _LLE5: _LLE6:
# 1698 "parse.y"
 yyval=Cyc_YY48(({struct Cyc_List_List*_tmp3E9=_cycalloc(sizeof(*_tmp3E9));({struct Cyc_Absyn_Enumfield*_tmp82D=Cyc_yyget_YY47(&(yyyvsp[0]).v);_tmp3E9->hd=_tmp82D;}),_tmp3E9->tl=0;_tmp3E9;}));
goto _LL0;case 118U: _LLE7: _LLE8:
# 1699 "parse.y"
 yyval=Cyc_YY48(({struct Cyc_List_List*_tmp3EA=_cycalloc(sizeof(*_tmp3EA));({struct Cyc_Absyn_Enumfield*_tmp82E=Cyc_yyget_YY47(&(yyyvsp[0]).v);_tmp3EA->hd=_tmp82E;}),_tmp3EA->tl=0;_tmp3EA;}));
goto _LL0;case 119U: _LLE9: _LLEA:
# 1700 "parse.y"
 yyval=Cyc_YY48(({struct Cyc_List_List*_tmp3EB=_cycalloc(sizeof(*_tmp3EB));({struct Cyc_Absyn_Enumfield*_tmp830=Cyc_yyget_YY47(&(yyyvsp[0]).v);_tmp3EB->hd=_tmp830;}),({struct Cyc_List_List*_tmp82F=Cyc_yyget_YY48(&(yyyvsp[2]).v);_tmp3EB->tl=_tmp82F;});_tmp3EB;}));
goto _LL0;case 120U: _LLEB: _LLEC:
# 1706 "parse.y"
 yyval=Cyc_YY21(({void*_tmp833=(void*)({struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*_tmp3EC=_cycalloc(sizeof(*_tmp3EC));_tmp3EC->tag=7U,({enum Cyc_Absyn_AggrKind _tmp832=Cyc_yyget_YY22(&(yyyvsp[0]).v);_tmp3EC->f1=_tmp832;}),({struct Cyc_List_List*_tmp831=Cyc_yyget_YY24(&(yyyvsp[2]).v);_tmp3EC->f2=_tmp831;});_tmp3EC;});Cyc_Parse_type_spec(_tmp833,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 121U: _LLED: _LLEE: {
# 1712 "parse.y"
struct Cyc_List_List*ts=({struct Cyc_List_List*(*_tmp835)(struct Cyc_Absyn_Tvar*(*f)(unsigned,void*),unsigned env,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp3F0)(struct Cyc_Absyn_Tvar*(*f)(unsigned,void*),unsigned env,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_Absyn_Tvar*(*f)(unsigned,void*),unsigned env,struct Cyc_List_List*x))Cyc_List_map_c;_tmp3F0;});unsigned _tmp834=(unsigned)((yyyvsp[3]).l).first_line;_tmp835(Cyc_Parse_typ2tvar,_tmp834,Cyc_yyget_YY40(&(yyyvsp[3]).v));});
struct Cyc_List_List*exist_ts=({struct Cyc_List_List*(*_tmp837)(struct Cyc_Absyn_Tvar*(*f)(unsigned,void*),unsigned env,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp3EF)(struct Cyc_Absyn_Tvar*(*f)(unsigned,void*),unsigned env,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_Absyn_Tvar*(*f)(unsigned,void*),unsigned env,struct Cyc_List_List*x))Cyc_List_map_c;_tmp3EF;});unsigned _tmp836=(unsigned)((yyyvsp[5]).l).first_line;_tmp837(Cyc_Parse_typ2tvar,_tmp836,Cyc_yyget_YY40(&(yyyvsp[5]).v));});
struct Cyc_Absyn_TypeDecl*_tmp3ED=({enum Cyc_Absyn_AggrKind _tmp83D=Cyc_yyget_YY22(&(yyyvsp[1]).v);struct _tuple0*_tmp83C=Cyc_yyget_QualId_tok(&(yyyvsp[2]).v);struct Cyc_List_List*_tmp83B=ts;struct Cyc_Absyn_AggrdeclImpl*_tmp83A=({
struct Cyc_List_List*_tmp839=exist_ts;struct Cyc_List_List*_tmp838=Cyc_yyget_YY50(&(yyyvsp[6]).v);Cyc_Absyn_aggrdecl_impl(_tmp839,_tmp838,Cyc_yyget_YY24(&(yyyvsp[7]).v),1);});
# 1714
Cyc_Absyn_aggr_tdecl(_tmp83D,Cyc_Absyn_Public,_tmp83C,_tmp83B,_tmp83A,0,(unsigned)({
# 1716
yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));});
# 1714
struct Cyc_Absyn_TypeDecl*td=_tmp3ED;
# 1717
yyval=Cyc_YY21(({void*_tmp83E=(void*)({struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct*_tmp3EE=_cycalloc(sizeof(*_tmp3EE));_tmp3EE->tag=10U,_tmp3EE->f1=td,_tmp3EE->f2=0;_tmp3EE;});Cyc_Parse_type_spec(_tmp83E,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
# 1719
goto _LL0;}case 122U: _LLEF: _LLF0: {
# 1723 "parse.y"
struct Cyc_List_List*ts=({struct Cyc_List_List*(*_tmp840)(struct Cyc_Absyn_Tvar*(*f)(unsigned,void*),unsigned env,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp3F4)(struct Cyc_Absyn_Tvar*(*f)(unsigned,void*),unsigned env,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_Absyn_Tvar*(*f)(unsigned,void*),unsigned env,struct Cyc_List_List*x))Cyc_List_map_c;_tmp3F4;});unsigned _tmp83F=(unsigned)((yyyvsp[2]).l).first_line;_tmp840(Cyc_Parse_typ2tvar,_tmp83F,Cyc_yyget_YY40(&(yyyvsp[2]).v));});
struct Cyc_List_List*exist_ts=({struct Cyc_List_List*(*_tmp842)(struct Cyc_Absyn_Tvar*(*f)(unsigned,void*),unsigned env,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp3F3)(struct Cyc_Absyn_Tvar*(*f)(unsigned,void*),unsigned env,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_Absyn_Tvar*(*f)(unsigned,void*),unsigned env,struct Cyc_List_List*x))Cyc_List_map_c;_tmp3F3;});unsigned _tmp841=(unsigned)((yyyvsp[4]).l).first_line;_tmp842(Cyc_Parse_typ2tvar,_tmp841,Cyc_yyget_YY40(&(yyyvsp[4]).v));});
struct Cyc_Absyn_TypeDecl*_tmp3F1=({enum Cyc_Absyn_AggrKind _tmp848=Cyc_yyget_YY22(&(yyyvsp[0]).v);struct _tuple0*_tmp847=Cyc_yyget_QualId_tok(&(yyyvsp[1]).v);struct Cyc_List_List*_tmp846=ts;struct Cyc_Absyn_AggrdeclImpl*_tmp845=({
struct Cyc_List_List*_tmp844=exist_ts;struct Cyc_List_List*_tmp843=Cyc_yyget_YY50(&(yyyvsp[5]).v);Cyc_Absyn_aggrdecl_impl(_tmp844,_tmp843,Cyc_yyget_YY24(&(yyyvsp[6]).v),0);});
# 1725
Cyc_Absyn_aggr_tdecl(_tmp848,Cyc_Absyn_Public,_tmp847,_tmp846,_tmp845,0,(unsigned)({
# 1727
yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));});
# 1725
struct Cyc_Absyn_TypeDecl*td=_tmp3F1;
# 1728
yyval=Cyc_YY21(({void*_tmp849=(void*)({struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct*_tmp3F2=_cycalloc(sizeof(*_tmp3F2));_tmp3F2->tag=10U,_tmp3F2->f1=td,_tmp3F2->f2=0;_tmp3F2;});Cyc_Parse_type_spec(_tmp849,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
# 1730
goto _LL0;}case 123U: _LLF1: _LLF2:
# 1731 "parse.y"
 yyval=Cyc_YY21(({void*_tmp84D=({union Cyc_Absyn_AggrInfo _tmp84C=({enum Cyc_Absyn_AggrKind _tmp84B=Cyc_yyget_YY22(&(yyyvsp[1]).v);struct _tuple0*_tmp84A=Cyc_yyget_QualId_tok(&(yyyvsp[2]).v);Cyc_Absyn_UnknownAggr(_tmp84B,_tmp84A,({struct Cyc_Core_Opt*_tmp3F5=_cycalloc(sizeof(*_tmp3F5));_tmp3F5->v=(void*)1;_tmp3F5;}));});Cyc_Absyn_aggr_type(_tmp84C,Cyc_yyget_YY40(&(yyyvsp[3]).v));});Cyc_Parse_type_spec(_tmp84D,(unsigned)({
yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
# 1734
goto _LL0;case 124U: _LLF3: _LLF4:
# 1735 "parse.y"
 yyval=Cyc_YY21(({void*_tmp850=({union Cyc_Absyn_AggrInfo _tmp84F=({enum Cyc_Absyn_AggrKind _tmp84E=Cyc_yyget_YY22(&(yyyvsp[0]).v);Cyc_Absyn_UnknownAggr(_tmp84E,Cyc_yyget_QualId_tok(&(yyyvsp[1]).v),0);});Cyc_Absyn_aggr_type(_tmp84F,Cyc_yyget_YY40(&(yyyvsp[2]).v));});Cyc_Parse_type_spec(_tmp850,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 125U: _LLF5: _LLF6:
# 1740 "parse.y"
 yyval=Cyc_YY40(0);
goto _LL0;case 126U: _LLF7: _LLF8:
# 1742 "parse.y"
 yyval=Cyc_YY40(Cyc_List_imp_rev(Cyc_yyget_YY40(&(yyyvsp[1]).v)));
goto _LL0;case 127U: _LLF9: _LLFA:
# 1746 "parse.y"
 yyval=Cyc_YY22(Cyc_Absyn_StructA);
goto _LL0;case 128U: _LLFB: _LLFC:
# 1747 "parse.y"
 yyval=Cyc_YY22(Cyc_Absyn_UnionA);
goto _LL0;case 129U: _LLFD: _LLFE:
# 1752 "parse.y"
 yyval=Cyc_YY24(0);
goto _LL0;case 130U: _LLFF: _LL100: {
# 1754 "parse.y"
struct Cyc_List_List*decls=0;
{struct Cyc_List_List*x=Cyc_yyget_YY25(&(yyyvsp[0]).v);for(0;x != 0;x=x->tl){
decls=Cyc_List_imp_append((struct Cyc_List_List*)x->hd,decls);}}{
# 1758
struct Cyc_List_List*tags=Cyc_Parse_get_aggrfield_tags(decls);
if(tags != 0)
({void(*_tmp852)(void(*f)(struct Cyc_List_List*,struct Cyc_Absyn_Aggrfield*),struct Cyc_List_List*env,struct Cyc_List_List*x)=({void(*_tmp3F6)(void(*f)(struct Cyc_List_List*,struct Cyc_Absyn_Aggrfield*),struct Cyc_List_List*env,struct Cyc_List_List*x)=(void(*)(void(*f)(struct Cyc_List_List*,struct Cyc_Absyn_Aggrfield*),struct Cyc_List_List*env,struct Cyc_List_List*x))Cyc_List_iter_c;_tmp3F6;});struct Cyc_List_List*_tmp851=tags;_tmp852(Cyc_Parse_substitute_aggrfield_tags,_tmp851,decls);});
# 1759
yyval=
# 1761
Cyc_YY24(decls);
# 1763
goto _LL0;}}case 131U: _LL101: _LL102:
# 1768 "parse.y"
 yyval=Cyc_YY25(({struct Cyc_List_List*_tmp3F7=_cycalloc(sizeof(*_tmp3F7));({struct Cyc_List_List*_tmp853=Cyc_yyget_YY24(&(yyyvsp[0]).v);_tmp3F7->hd=_tmp853;}),_tmp3F7->tl=0;_tmp3F7;}));
goto _LL0;case 132U: _LL103: _LL104:
# 1770 "parse.y"
 yyval=Cyc_YY25(({struct Cyc_List_List*_tmp3F8=_cycalloc(sizeof(*_tmp3F8));({struct Cyc_List_List*_tmp855=Cyc_yyget_YY24(&(yyyvsp[1]).v);_tmp3F8->hd=_tmp855;}),({struct Cyc_List_List*_tmp854=Cyc_yyget_YY25(&(yyyvsp[0]).v);_tmp3F8->tl=_tmp854;});_tmp3F8;}));
goto _LL0;case 133U: _LL105: _LL106:
# 1774 "parse.y"
 yyval=Cyc_YY19(({struct _tuple13*(*_tmp856)(struct _tuple13*x)=({struct _tuple13*(*_tmp3F9)(struct _tuple13*x)=(struct _tuple13*(*)(struct _tuple13*x))Cyc_Parse_flat_imp_rev;_tmp3F9;});_tmp856(Cyc_yyget_YY19(&(yyyvsp[0]).v));}));
goto _LL0;case 134U: _LL107: _LL108:
# 1780 "parse.y"
 yyval=Cyc_YY19(({struct _tuple13*_tmp3FA=_region_malloc(yyr,sizeof(*_tmp3FA));_tmp3FA->tl=0,({struct _tuple12 _tmp857=Cyc_yyget_YY18(&(yyyvsp[0]).v);_tmp3FA->hd=_tmp857;});_tmp3FA;}));
goto _LL0;case 135U: _LL109: _LL10A:
# 1782 "parse.y"
 yyval=Cyc_YY19(({struct _tuple13*_tmp3FB=_region_malloc(yyr,sizeof(*_tmp3FB));({struct _tuple13*_tmp859=Cyc_yyget_YY19(&(yyyvsp[0]).v);_tmp3FB->tl=_tmp859;}),({struct _tuple12 _tmp858=Cyc_yyget_YY18(&(yyyvsp[2]).v);_tmp3FB->hd=_tmp858;});_tmp3FB;}));
goto _LL0;case 136U: _LL10B: _LL10C:
# 1787 "parse.y"
 yyval=Cyc_YY18(({struct _tuple12 _tmp64C;({struct Cyc_Parse_Declarator _tmp85A=Cyc_yyget_YY27(&(yyyvsp[0]).v);_tmp64C.f1=_tmp85A;}),_tmp64C.f2=0;_tmp64C;}));
goto _LL0;case 137U: _LL10D: _LL10E:
# 1789 "parse.y"
 yyval=Cyc_YY18(({struct _tuple12 _tmp64D;({struct Cyc_Parse_Declarator _tmp85C=Cyc_yyget_YY27(&(yyyvsp[0]).v);_tmp64D.f1=_tmp85C;}),({struct Cyc_Absyn_Exp*_tmp85B=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);_tmp64D.f2=_tmp85B;});_tmp64D;}));
goto _LL0;case 138U: _LL10F: _LL110: {
# 1795 "parse.y"
struct _RegionHandle _tmp3FC=_new_region("temp");struct _RegionHandle*temp=& _tmp3FC;_push_region(temp);
{struct _tuple26 _stmttmp18=Cyc_yyget_YY35(&(yyyvsp[0]).v);struct Cyc_List_List*_tmp3FF;struct Cyc_Parse_Type_specifier _tmp3FE;struct Cyc_Absyn_Tqual _tmp3FD;_LL471: _tmp3FD=_stmttmp18.f1;_tmp3FE=_stmttmp18.f2;_tmp3FF=_stmttmp18.f3;_LL472: {struct Cyc_Absyn_Tqual tq=_tmp3FD;struct Cyc_Parse_Type_specifier tspecs=_tmp3FE;struct Cyc_List_List*atts=_tmp3FF;
if(tq.loc == (unsigned)0)tq.loc=(unsigned)((yyyvsp[0]).l).first_line;{struct _tuple11*decls=0;
# 1799
struct Cyc_List_List*widths_and_reqs=0;
{struct Cyc_List_List*x=Cyc_yyget_YY29(&(yyyvsp[1]).v);for(0;x != 0;x=x->tl){
struct _tuple25*_stmttmp19=(struct _tuple25*)x->hd;struct Cyc_Absyn_Exp*_tmp402;struct Cyc_Absyn_Exp*_tmp401;struct Cyc_Parse_Declarator _tmp400;_LL474: _tmp400=_stmttmp19->f1;_tmp401=_stmttmp19->f2;_tmp402=_stmttmp19->f3;_LL475: {struct Cyc_Parse_Declarator d=_tmp400;struct Cyc_Absyn_Exp*wd=_tmp401;struct Cyc_Absyn_Exp*wh=_tmp402;
decls=({struct _tuple11*_tmp403=_region_malloc(temp,sizeof(*_tmp403));_tmp403->tl=decls,_tmp403->hd=d;_tmp403;});
widths_and_reqs=({struct Cyc_List_List*_tmp405=_region_malloc(temp,sizeof(*_tmp405));
({struct _tuple17*_tmp85D=({struct _tuple17*_tmp404=_region_malloc(temp,sizeof(*_tmp404));_tmp404->f1=wd,_tmp404->f2=wh;_tmp404;});_tmp405->hd=_tmp85D;}),_tmp405->tl=widths_and_reqs;_tmp405;});}}}
# 1806
decls=({struct _tuple11*(*_tmp85E)(struct _tuple11*x)=({struct _tuple11*(*_tmp406)(struct _tuple11*x)=(struct _tuple11*(*)(struct _tuple11*x))Cyc_Parse_flat_imp_rev;_tmp406;});_tmp85E(decls);});
widths_and_reqs=Cyc_List_imp_rev(widths_and_reqs);{
void*t=Cyc_Parse_speclist2typ(tspecs,(unsigned)((yyyvsp[0]).l).first_line);
struct Cyc_List_List*info=({struct _RegionHandle*_tmp861=temp;struct _RegionHandle*_tmp860=temp;struct Cyc_List_List*_tmp85F=
Cyc_Parse_apply_tmss(temp,tq,t,decls,atts);
# 1809
Cyc_List_rzip(_tmp861,_tmp860,_tmp85F,widths_and_reqs);});
# 1812
yyval=Cyc_YY24(({struct Cyc_List_List*(*_tmp863)(struct Cyc_Absyn_Aggrfield*(*f)(unsigned,struct _tuple18*),unsigned env,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp407)(struct Cyc_Absyn_Aggrfield*(*f)(unsigned,struct _tuple18*),unsigned env,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_Absyn_Aggrfield*(*f)(unsigned,struct _tuple18*),unsigned env,struct Cyc_List_List*x))Cyc_List_map_c;_tmp407;});unsigned _tmp862=(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;});_tmp863(Cyc_Parse_make_aggr_field,_tmp862,info);}));
# 1814
_npop_handler(0U);goto _LL0;}}}}
# 1796
;_pop_region(temp);}case 139U: _LL111: _LL112:
# 1822 "parse.y"
 yyval=Cyc_YY35(({struct _tuple26 _tmp64E;({struct Cyc_Absyn_Tqual _tmp865=Cyc_Absyn_empty_tqual((unsigned)((yyyvsp[0]).l).first_line);_tmp64E.f1=_tmp865;}),({struct Cyc_Parse_Type_specifier _tmp864=Cyc_yyget_YY21(&(yyyvsp[0]).v);_tmp64E.f2=_tmp864;}),_tmp64E.f3=0;_tmp64E;}));
goto _LL0;case 140U: _LL113: _LL114: {
# 1824 "parse.y"
struct _tuple26 two=Cyc_yyget_YY35(&(yyyvsp[1]).v);yyval=Cyc_YY35(({struct _tuple26 _tmp64F;_tmp64F.f1=two.f1,({struct Cyc_Parse_Type_specifier _tmp868=({unsigned _tmp867=(unsigned)((yyyvsp[0]).l).first_line;struct Cyc_Parse_Type_specifier _tmp866=Cyc_yyget_YY21(&(yyyvsp[0]).v);Cyc_Parse_combine_specifiers(_tmp867,_tmp866,two.f2);});_tmp64F.f2=_tmp868;}),_tmp64F.f3=two.f3;_tmp64F;}));
goto _LL0;}case 141U: _LL115: _LL116:
# 1826 "parse.y"
 yyval=Cyc_YY35(({struct _tuple26 _tmp650;({struct Cyc_Absyn_Tqual _tmp86A=Cyc_yyget_YY23(&(yyyvsp[0]).v);_tmp650.f1=_tmp86A;}),({struct Cyc_Parse_Type_specifier _tmp869=Cyc_Parse_empty_spec(0U);_tmp650.f2=_tmp869;}),_tmp650.f3=0;_tmp650;}));
goto _LL0;case 142U: _LL117: _LL118: {
# 1828 "parse.y"
struct _tuple26 two=Cyc_yyget_YY35(&(yyyvsp[1]).v);
yyval=Cyc_YY35(({struct _tuple26 _tmp651;({struct Cyc_Absyn_Tqual _tmp86C=({struct Cyc_Absyn_Tqual _tmp86B=Cyc_yyget_YY23(&(yyyvsp[0]).v);Cyc_Absyn_combine_tqual(_tmp86B,two.f1);});_tmp651.f1=_tmp86C;}),_tmp651.f2=two.f2,_tmp651.f3=two.f3;_tmp651;}));
goto _LL0;}case 143U: _LL119: _LL11A:
# 1831 "parse.y"
 yyval=Cyc_YY35(({struct _tuple26 _tmp652;({struct Cyc_Absyn_Tqual _tmp86F=Cyc_Absyn_empty_tqual((unsigned)((yyyvsp[0]).l).first_line);_tmp652.f1=_tmp86F;}),({struct Cyc_Parse_Type_specifier _tmp86E=Cyc_Parse_empty_spec(0U);_tmp652.f2=_tmp86E;}),({struct Cyc_List_List*_tmp86D=Cyc_yyget_YY45(&(yyyvsp[0]).v);_tmp652.f3=_tmp86D;});_tmp652;}));
goto _LL0;case 144U: _LL11B: _LL11C: {
# 1833 "parse.y"
struct _tuple26 two=Cyc_yyget_YY35(&(yyyvsp[1]).v);yyval=Cyc_YY35(({struct _tuple26 _tmp653;_tmp653.f1=two.f1,_tmp653.f2=two.f2,({struct Cyc_List_List*_tmp871=({struct Cyc_List_List*_tmp870=Cyc_yyget_YY45(&(yyyvsp[0]).v);Cyc_List_append(_tmp870,two.f3);});_tmp653.f3=_tmp871;});_tmp653;}));
goto _LL0;}case 145U: _LL11D: _LL11E:
# 1839 "parse.y"
 yyval=Cyc_YY35(({struct _tuple26 _tmp654;({struct Cyc_Absyn_Tqual _tmp873=Cyc_Absyn_empty_tqual((unsigned)((yyyvsp[0]).l).first_line);_tmp654.f1=_tmp873;}),({struct Cyc_Parse_Type_specifier _tmp872=Cyc_yyget_YY21(&(yyyvsp[0]).v);_tmp654.f2=_tmp872;}),_tmp654.f3=0;_tmp654;}));
goto _LL0;case 146U: _LL11F: _LL120: {
# 1841 "parse.y"
struct _tuple26 two=Cyc_yyget_YY35(&(yyyvsp[1]).v);yyval=Cyc_YY35(({struct _tuple26 _tmp655;_tmp655.f1=two.f1,({struct Cyc_Parse_Type_specifier _tmp876=({unsigned _tmp875=(unsigned)((yyyvsp[0]).l).first_line;struct Cyc_Parse_Type_specifier _tmp874=Cyc_yyget_YY21(&(yyyvsp[0]).v);Cyc_Parse_combine_specifiers(_tmp875,_tmp874,two.f2);});_tmp655.f2=_tmp876;}),_tmp655.f3=two.f3;_tmp655;}));
goto _LL0;}case 147U: _LL121: _LL122:
# 1843 "parse.y"
 yyval=Cyc_YY35(({struct _tuple26 _tmp656;({struct Cyc_Absyn_Tqual _tmp878=Cyc_yyget_YY23(&(yyyvsp[0]).v);_tmp656.f1=_tmp878;}),({struct Cyc_Parse_Type_specifier _tmp877=Cyc_Parse_empty_spec(0U);_tmp656.f2=_tmp877;}),_tmp656.f3=0;_tmp656;}));
goto _LL0;case 148U: _LL123: _LL124: {
# 1845 "parse.y"
struct _tuple26 two=Cyc_yyget_YY35(&(yyyvsp[1]).v);
yyval=Cyc_YY35(({struct _tuple26 _tmp657;({struct Cyc_Absyn_Tqual _tmp87A=({struct Cyc_Absyn_Tqual _tmp879=Cyc_yyget_YY23(&(yyyvsp[0]).v);Cyc_Absyn_combine_tqual(_tmp879,two.f1);});_tmp657.f1=_tmp87A;}),_tmp657.f2=two.f2,_tmp657.f3=two.f3;_tmp657;}));
goto _LL0;}case 149U: _LL125: _LL126:
# 1848 "parse.y"
 yyval=Cyc_YY35(({struct _tuple26 _tmp658;({struct Cyc_Absyn_Tqual _tmp87D=Cyc_Absyn_empty_tqual((unsigned)((yyyvsp[0]).l).first_line);_tmp658.f1=_tmp87D;}),({struct Cyc_Parse_Type_specifier _tmp87C=Cyc_Parse_empty_spec(0U);_tmp658.f2=_tmp87C;}),({struct Cyc_List_List*_tmp87B=Cyc_yyget_YY45(&(yyyvsp[0]).v);_tmp658.f3=_tmp87B;});_tmp658;}));
goto _LL0;case 150U: _LL127: _LL128: {
# 1850 "parse.y"
struct _tuple26 two=Cyc_yyget_YY35(&(yyyvsp[1]).v);yyval=Cyc_YY35(({struct _tuple26 _tmp659;_tmp659.f1=two.f1,_tmp659.f2=two.f2,({struct Cyc_List_List*_tmp87F=({struct Cyc_List_List*_tmp87E=Cyc_yyget_YY45(&(yyyvsp[0]).v);Cyc_List_append(_tmp87E,two.f3);});_tmp659.f3=_tmp87F;});_tmp659;}));
goto _LL0;}case 151U: _LL129: _LL12A:
# 1854 "parse.y"
 yyval=Cyc_YY29(Cyc_List_imp_rev(Cyc_yyget_YY29(&(yyyvsp[0]).v)));
goto _LL0;case 152U: _LL12B: _LL12C:
# 1860 "parse.y"
 yyval=Cyc_YY29(({struct Cyc_List_List*_tmp408=_region_malloc(yyr,sizeof(*_tmp408));({struct _tuple25*_tmp880=Cyc_yyget_YY28(&(yyyvsp[0]).v);_tmp408->hd=_tmp880;}),_tmp408->tl=0;_tmp408;}));
goto _LL0;case 153U: _LL12D: _LL12E:
# 1862 "parse.y"
 yyval=Cyc_YY29(({struct Cyc_List_List*_tmp409=_region_malloc(yyr,sizeof(*_tmp409));({struct _tuple25*_tmp882=Cyc_yyget_YY28(&(yyyvsp[2]).v);_tmp409->hd=_tmp882;}),({struct Cyc_List_List*_tmp881=Cyc_yyget_YY29(&(yyyvsp[0]).v);_tmp409->tl=_tmp881;});_tmp409;}));
goto _LL0;case 154U: _LL12F: _LL130:
# 1867 "parse.y"
 yyval=Cyc_YY28(({struct _tuple25*_tmp40A=_region_malloc(yyr,sizeof(*_tmp40A));({struct Cyc_Parse_Declarator _tmp884=Cyc_yyget_YY27(&(yyyvsp[0]).v);_tmp40A->f1=_tmp884;}),_tmp40A->f2=0,({struct Cyc_Absyn_Exp*_tmp883=Cyc_yyget_YY57(&(yyyvsp[1]).v);_tmp40A->f3=_tmp883;});_tmp40A;}));
goto _LL0;case 155U: _LL131: _LL132:
# 1871 "parse.y"
 yyval=Cyc_YY28(({struct _tuple25*_tmp40E=_region_malloc(yyr,sizeof(*_tmp40E));({struct _tuple0*_tmp889=({struct _tuple0*_tmp40D=_cycalloc(sizeof(*_tmp40D));({union Cyc_Absyn_Nmspace _tmp888=Cyc_Absyn_Rel_n(0);_tmp40D->f1=_tmp888;}),({struct _fat_ptr*_tmp887=({struct _fat_ptr*_tmp40C=_cycalloc(sizeof(*_tmp40C));({struct _fat_ptr _tmp886=({const char*_tmp40B="";_tag_fat(_tmp40B,sizeof(char),1U);});*_tmp40C=_tmp886;});_tmp40C;});_tmp40D->f2=_tmp887;});_tmp40D;});(_tmp40E->f1).id=_tmp889;}),(_tmp40E->f1).varloc=0U,(_tmp40E->f1).tms=0,({struct Cyc_Absyn_Exp*_tmp885=Cyc_yyget_Exp_tok(&(yyyvsp[1]).v);_tmp40E->f2=_tmp885;}),_tmp40E->f3=0;_tmp40E;}));
# 1873
goto _LL0;case 156U: _LL133: _LL134:
# 1876 "parse.y"
 yyval=Cyc_YY28(({struct _tuple25*_tmp412=_region_malloc(yyr,sizeof(*_tmp412));({struct _tuple0*_tmp88D=({struct _tuple0*_tmp411=_cycalloc(sizeof(*_tmp411));({union Cyc_Absyn_Nmspace _tmp88C=Cyc_Absyn_Rel_n(0);_tmp411->f1=_tmp88C;}),({struct _fat_ptr*_tmp88B=({struct _fat_ptr*_tmp410=_cycalloc(sizeof(*_tmp410));({struct _fat_ptr _tmp88A=({const char*_tmp40F="";_tag_fat(_tmp40F,sizeof(char),1U);});*_tmp410=_tmp88A;});_tmp410;});_tmp411->f2=_tmp88B;});_tmp411;});(_tmp412->f1).id=_tmp88D;}),(_tmp412->f1).varloc=0U,(_tmp412->f1).tms=0,_tmp412->f2=0,_tmp412->f3=0;_tmp412;}));
# 1878
goto _LL0;case 157U: _LL135: _LL136:
# 1879 "parse.y"
 yyval=Cyc_YY28(({struct _tuple25*_tmp413=_region_malloc(yyr,sizeof(*_tmp413));({struct Cyc_Parse_Declarator _tmp88F=Cyc_yyget_YY27(&(yyyvsp[0]).v);_tmp413->f1=_tmp88F;}),({struct Cyc_Absyn_Exp*_tmp88E=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);_tmp413->f2=_tmp88E;}),_tmp413->f3=0;_tmp413;}));
goto _LL0;case 158U: _LL137: _LL138:
# 1883 "parse.y"
 yyval=Cyc_YY57(Cyc_yyget_Exp_tok(&(yyyvsp[2]).v));
goto _LL0;case 159U: _LL139: _LL13A:
# 1884 "parse.y"
 yyval=Cyc_YY57(0);
goto _LL0;case 160U: _LL13B: _LL13C:
# 1891 "parse.y"
 yyval=Cyc_YY64(({struct Cyc_List_List*_tmp414=_cycalloc(sizeof(*_tmp414));({void*_tmp890=Cyc_Absyn_join_eff(Cyc_yyget_YY40(&(yyyvsp[1]).v));_tmp414->hd=_tmp890;}),_tmp414->tl=0;_tmp414;}));
# 1893
goto _LL0;case 161U: _LL13D: _LL13E:
# 1896
 yyval=Cyc_YY64(({struct Cyc_List_List*_tmp415=_cycalloc(sizeof(*_tmp415));({void*_tmp892=Cyc_Absyn_join_eff(Cyc_yyget_YY40(&(yyyvsp[3]).v));_tmp415->hd=_tmp892;}),({struct Cyc_List_List*_tmp891=Cyc_yyget_YY64(&(yyyvsp[0]).v);_tmp415->tl=_tmp891;});_tmp415;}));
# 1898
goto _LL0;case 162U: _LL13F: _LL140:
# 1903 "parse.y"
 yyval=Cyc_YY49(Cyc_Absyn_join_eff(Cyc_yyget_YY64(&(yyyvsp[2]).v)));
# 1905
goto _LL0;case 163U: _LL141: _LL142:
# 1905 "parse.y"
 yyval=Cyc_YY49(0);
goto _LL0;case 164U: _LL143: _LL144:
# 1911 "parse.y"
 yyval=Cyc_YY49(Cyc_Absyn_join_eff(Cyc_yyget_YY64(&(yyyvsp[2]).v)));
# 1913
goto _LL0;case 165U: _LL145: _LL146:
# 1913 "parse.y"
 yyval=Cyc_YY49(0);
goto _LL0;case 166U: _LL147: _LL148:
# 1918 "parse.y"
 yyval=Cyc_YY57(Cyc_yyget_Exp_tok(&(yyyvsp[2]).v));
goto _LL0;case 167U: _LL149: _LL14A:
# 1919 "parse.y"
 yyval=Cyc_YY57(0);
goto _LL0;case 168U: _LL14B: _LL14C: {
# 1925 "parse.y"
int is_extensible=Cyc_yyget_YY31(&(yyyvsp[0]).v);
struct Cyc_List_List*ts=({struct Cyc_List_List*(*_tmp894)(struct Cyc_Absyn_Tvar*(*f)(unsigned,void*),unsigned env,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp419)(struct Cyc_Absyn_Tvar*(*f)(unsigned,void*),unsigned env,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_Absyn_Tvar*(*f)(unsigned,void*),unsigned env,struct Cyc_List_List*x))Cyc_List_map_c;_tmp419;});unsigned _tmp893=(unsigned)((yyyvsp[2]).l).first_line;_tmp894(Cyc_Parse_typ2tvar,_tmp893,Cyc_yyget_YY40(&(yyyvsp[2]).v));});
struct Cyc_Absyn_TypeDecl*_tmp416=({struct _tuple0*_tmp899=Cyc_yyget_QualId_tok(&(yyyvsp[1]).v);struct Cyc_List_List*_tmp898=ts;struct Cyc_Core_Opt*_tmp897=({struct Cyc_Core_Opt*_tmp418=_cycalloc(sizeof(*_tmp418));({struct Cyc_List_List*_tmp895=Cyc_yyget_YY34(&(yyyvsp[4]).v);_tmp418->v=_tmp895;});_tmp418;});int _tmp896=is_extensible;Cyc_Absyn_datatype_tdecl(Cyc_Absyn_Public,_tmp899,_tmp898,_tmp897,_tmp896,(unsigned)({
yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));});
# 1927
struct Cyc_Absyn_TypeDecl*dd=_tmp416;
# 1929
yyval=Cyc_YY21(({void*_tmp89A=(void*)({struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct*_tmp417=_cycalloc(sizeof(*_tmp417));_tmp417->tag=10U,_tmp417->f1=dd,_tmp417->f2=0;_tmp417;});Cyc_Parse_type_spec(_tmp89A,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
# 1931
goto _LL0;}case 169U: _LL14D: _LL14E: {
# 1932 "parse.y"
int is_extensible=Cyc_yyget_YY31(&(yyyvsp[0]).v);
yyval=Cyc_YY21(({void*_tmp89D=({union Cyc_Absyn_DatatypeInfo _tmp89C=Cyc_Absyn_UnknownDatatype(({struct Cyc_Absyn_UnknownDatatypeInfo _tmp65A;({struct _tuple0*_tmp89B=Cyc_yyget_QualId_tok(&(yyyvsp[1]).v);_tmp65A.name=_tmp89B;}),_tmp65A.is_extensible=is_extensible;_tmp65A;}));Cyc_Absyn_datatype_type(_tmp89C,Cyc_yyget_YY40(&(yyyvsp[2]).v));});Cyc_Parse_type_spec(_tmp89D,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
# 1935
goto _LL0;}case 170U: _LL14F: _LL150: {
# 1936 "parse.y"
int is_extensible=Cyc_yyget_YY31(&(yyyvsp[0]).v);
yyval=Cyc_YY21(({void*_tmp8A1=({union Cyc_Absyn_DatatypeFieldInfo _tmp8A0=Cyc_Absyn_UnknownDatatypefield(({struct Cyc_Absyn_UnknownDatatypeFieldInfo _tmp65B;({struct _tuple0*_tmp89F=Cyc_yyget_QualId_tok(&(yyyvsp[1]).v);_tmp65B.datatype_name=_tmp89F;}),({struct _tuple0*_tmp89E=Cyc_yyget_QualId_tok(&(yyyvsp[3]).v);_tmp65B.field_name=_tmp89E;}),_tmp65B.is_extensible=is_extensible;_tmp65B;}));Cyc_Absyn_datatype_field_type(_tmp8A0,Cyc_yyget_YY40(&(yyyvsp[4]).v));});Cyc_Parse_type_spec(_tmp8A1,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
# 1939
goto _LL0;}case 171U: _LL151: _LL152:
# 1942 "parse.y"
 yyval=Cyc_YY31(0);
goto _LL0;case 172U: _LL153: _LL154:
# 1943 "parse.y"
 yyval=Cyc_YY31(1);
goto _LL0;case 173U: _LL155: _LL156:
# 1947 "parse.y"
 yyval=Cyc_YY34(({struct Cyc_List_List*_tmp41A=_cycalloc(sizeof(*_tmp41A));({struct Cyc_Absyn_Datatypefield*_tmp8A2=Cyc_yyget_YY33(&(yyyvsp[0]).v);_tmp41A->hd=_tmp8A2;}),_tmp41A->tl=0;_tmp41A;}));
goto _LL0;case 174U: _LL157: _LL158:
# 1948 "parse.y"
 yyval=Cyc_YY34(({struct Cyc_List_List*_tmp41B=_cycalloc(sizeof(*_tmp41B));({struct Cyc_Absyn_Datatypefield*_tmp8A3=Cyc_yyget_YY33(&(yyyvsp[0]).v);_tmp41B->hd=_tmp8A3;}),_tmp41B->tl=0;_tmp41B;}));
goto _LL0;case 175U: _LL159: _LL15A:
# 1949 "parse.y"
 yyval=Cyc_YY34(({struct Cyc_List_List*_tmp41C=_cycalloc(sizeof(*_tmp41C));({struct Cyc_Absyn_Datatypefield*_tmp8A5=Cyc_yyget_YY33(&(yyyvsp[0]).v);_tmp41C->hd=_tmp8A5;}),({struct Cyc_List_List*_tmp8A4=Cyc_yyget_YY34(&(yyyvsp[2]).v);_tmp41C->tl=_tmp8A4;});_tmp41C;}));
goto _LL0;case 176U: _LL15B: _LL15C:
# 1950 "parse.y"
 yyval=Cyc_YY34(({struct Cyc_List_List*_tmp41D=_cycalloc(sizeof(*_tmp41D));({struct Cyc_Absyn_Datatypefield*_tmp8A7=Cyc_yyget_YY33(&(yyyvsp[0]).v);_tmp41D->hd=_tmp8A7;}),({struct Cyc_List_List*_tmp8A6=Cyc_yyget_YY34(&(yyyvsp[2]).v);_tmp41D->tl=_tmp8A6;});_tmp41D;}));
goto _LL0;case 177U: _LL15D: _LL15E:
# 1954 "parse.y"
 yyval=Cyc_YY32(Cyc_Absyn_Public);
goto _LL0;case 178U: _LL15F: _LL160:
# 1955 "parse.y"
 yyval=Cyc_YY32(Cyc_Absyn_Extern);
goto _LL0;case 179U: _LL161: _LL162:
# 1956 "parse.y"
 yyval=Cyc_YY32(Cyc_Absyn_Static);
goto _LL0;case 180U: _LL163: _LL164:
# 1960 "parse.y"
 yyval=Cyc_YY33(({struct Cyc_Absyn_Datatypefield*_tmp41E=_cycalloc(sizeof(*_tmp41E));({struct _tuple0*_tmp8AA=Cyc_yyget_QualId_tok(&(yyyvsp[1]).v);_tmp41E->name=_tmp8AA;}),_tmp41E->typs=0,({unsigned _tmp8A9=(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;});_tmp41E->loc=_tmp8A9;}),({enum Cyc_Absyn_Scope _tmp8A8=Cyc_yyget_YY32(&(yyyvsp[0]).v);_tmp41E->sc=_tmp8A8;});_tmp41E;}));
goto _LL0;case 181U: _LL165: _LL166: {
# 1962 "parse.y"
struct Cyc_List_List*typs=({struct Cyc_List_List*(*_tmp8AC)(struct _tuple20*(*f)(unsigned,struct _tuple8*),unsigned env,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp420)(struct _tuple20*(*f)(unsigned,struct _tuple8*),unsigned env,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct _tuple20*(*f)(unsigned,struct _tuple8*),unsigned env,struct Cyc_List_List*x))Cyc_List_map_c;_tmp420;});unsigned _tmp8AB=(unsigned)((yyyvsp[3]).l).first_line;_tmp8AC(Cyc_Parse_get_tqual_typ,_tmp8AB,Cyc_List_imp_rev(Cyc_yyget_YY38(&(yyyvsp[3]).v)));});
yyval=Cyc_YY33(({struct Cyc_Absyn_Datatypefield*_tmp41F=_cycalloc(sizeof(*_tmp41F));({struct _tuple0*_tmp8AF=Cyc_yyget_QualId_tok(&(yyyvsp[1]).v);_tmp41F->name=_tmp8AF;}),_tmp41F->typs=typs,({unsigned _tmp8AE=(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;});_tmp41F->loc=_tmp8AE;}),({enum Cyc_Absyn_Scope _tmp8AD=Cyc_yyget_YY32(&(yyyvsp[0]).v);_tmp41F->sc=_tmp8AD;});_tmp41F;}));
goto _LL0;}case 182U: _LL167: _LL168:
# 1968 "parse.y"
 yyval=(yyyvsp[0]).v;
goto _LL0;case 183U: _LL169: _LL16A: {
# 1970 "parse.y"
struct Cyc_Parse_Declarator two=Cyc_yyget_YY27(&(yyyvsp[1]).v);
yyval=Cyc_YY27(({struct Cyc_Parse_Declarator _tmp65C;_tmp65C.id=two.id,_tmp65C.varloc=two.varloc,({struct Cyc_List_List*_tmp8B1=({struct Cyc_List_List*_tmp8B0=Cyc_yyget_YY26(&(yyyvsp[0]).v);Cyc_List_imp_append(_tmp8B0,two.tms);});_tmp65C.tms=_tmp8B1;});_tmp65C;}));
goto _LL0;}case 184U: _LL16B: _LL16C:
# 1977 "parse.y"
 yyval=(yyyvsp[0]).v;
goto _LL0;case 185U: _LL16D: _LL16E: {
# 1979 "parse.y"
struct Cyc_Parse_Declarator two=Cyc_yyget_YY27(&(yyyvsp[1]).v);
yyval=Cyc_YY27(({struct Cyc_Parse_Declarator _tmp65D;_tmp65D.id=two.id,_tmp65D.varloc=two.varloc,({struct Cyc_List_List*_tmp8B3=({struct Cyc_List_List*_tmp8B2=Cyc_yyget_YY26(&(yyyvsp[0]).v);Cyc_List_imp_append(_tmp8B2,two.tms);});_tmp65D.tms=_tmp8B3;});_tmp65D;}));
goto _LL0;}case 186U: _LL16F: _LL170:
# 1985 "parse.y"
 yyval=Cyc_YY27(({struct Cyc_Parse_Declarator _tmp65E;({struct _tuple0*_tmp8B4=Cyc_yyget_QualId_tok(&(yyyvsp[0]).v);_tmp65E.id=_tmp8B4;}),_tmp65E.varloc=(unsigned)((yyyvsp[0]).l).first_line,_tmp65E.tms=0;_tmp65E;}));
goto _LL0;case 187U: _LL171: _LL172:
# 1987 "parse.y"
 yyval=(yyyvsp[1]).v;
goto _LL0;case 188U: _LL173: _LL174: {
# 1991 "parse.y"
struct Cyc_Parse_Declarator d=Cyc_yyget_YY27(&(yyyvsp[2]).v);
({struct Cyc_List_List*_tmp8B7=({struct Cyc_List_List*_tmp422=_region_malloc(yyr,sizeof(*_tmp422));({void*_tmp8B6=(void*)({struct Cyc_Absyn_Attributes_mod_Absyn_Type_modifier_struct*_tmp421=_region_malloc(yyr,sizeof(*_tmp421));_tmp421->tag=5U,_tmp421->f1=(unsigned)((yyyvsp[1]).l).first_line,({struct Cyc_List_List*_tmp8B5=Cyc_yyget_YY45(&(yyyvsp[1]).v);_tmp421->f2=_tmp8B5;});_tmp421;});_tmp422->hd=_tmp8B6;}),_tmp422->tl=d.tms;_tmp422;});d.tms=_tmp8B7;});
yyval=(yyyvsp[2]).v;
# 1995
goto _LL0;}case 189U: _LL175: _LL176:
# 1996 "parse.y"
 yyval=Cyc_YY27(({struct Cyc_Parse_Declarator _tmp65F;({struct _tuple0*_tmp8BD=(Cyc_yyget_YY27(&(yyyvsp[0]).v)).id;_tmp65F.id=_tmp8BD;}),({unsigned _tmp8BC=(Cyc_yyget_YY27(&(yyyvsp[0]).v)).varloc;_tmp65F.varloc=_tmp8BC;}),({struct Cyc_List_List*_tmp8BB=({struct Cyc_List_List*_tmp424=_region_malloc(yyr,sizeof(*_tmp424));({void*_tmp8BA=(void*)({struct Cyc_Absyn_Carray_mod_Absyn_Type_modifier_struct*_tmp423=_region_malloc(yyr,sizeof(*_tmp423));_tmp423->tag=0U,({void*_tmp8B9=Cyc_yyget_YY51(&(yyyvsp[3]).v);_tmp423->f1=_tmp8B9;}),_tmp423->f2=(unsigned)((yyyvsp[3]).l).first_line;_tmp423;});_tmp424->hd=_tmp8BA;}),({struct Cyc_List_List*_tmp8B8=(Cyc_yyget_YY27(&(yyyvsp[0]).v)).tms;_tmp424->tl=_tmp8B8;});_tmp424;});_tmp65F.tms=_tmp8BB;});_tmp65F;}));
goto _LL0;case 190U: _LL177: _LL178:
# 1998 "parse.y"
 yyval=Cyc_YY27(({struct Cyc_Parse_Declarator _tmp660;({struct _tuple0*_tmp8C4=(Cyc_yyget_YY27(&(yyyvsp[0]).v)).id;_tmp660.id=_tmp8C4;}),({unsigned _tmp8C3=(Cyc_yyget_YY27(&(yyyvsp[0]).v)).varloc;_tmp660.varloc=_tmp8C3;}),({
struct Cyc_List_List*_tmp8C2=({struct Cyc_List_List*_tmp426=_region_malloc(yyr,sizeof(*_tmp426));({void*_tmp8C1=(void*)({struct Cyc_Absyn_ConstArray_mod_Absyn_Type_modifier_struct*_tmp425=_region_malloc(yyr,sizeof(*_tmp425));_tmp425->tag=1U,({struct Cyc_Absyn_Exp*_tmp8C0=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);_tmp425->f1=_tmp8C0;}),({void*_tmp8BF=Cyc_yyget_YY51(&(yyyvsp[4]).v);_tmp425->f2=_tmp8BF;}),_tmp425->f3=(unsigned)((yyyvsp[4]).l).first_line;_tmp425;});_tmp426->hd=_tmp8C1;}),({struct Cyc_List_List*_tmp8BE=(Cyc_yyget_YY27(&(yyyvsp[0]).v)).tms;_tmp426->tl=_tmp8BE;});_tmp426;});_tmp660.tms=_tmp8C2;});_tmp660;}));
goto _LL0;case 191U: _LL179: _LL17A: {
# 2001 "parse.y"
struct _tuple27*_stmttmp1A=Cyc_yyget_YY39(&(yyyvsp[2]).v);struct Cyc_List_List*_tmp42B;void*_tmp42A;struct Cyc_Absyn_VarargInfo*_tmp429;int _tmp428;struct Cyc_List_List*_tmp427;_LL477: _tmp427=_stmttmp1A->f1;_tmp428=_stmttmp1A->f2;_tmp429=_stmttmp1A->f3;_tmp42A=_stmttmp1A->f4;_tmp42B=_stmttmp1A->f5;_LL478: {struct Cyc_List_List*lis=_tmp427;int b=_tmp428;struct Cyc_Absyn_VarargInfo*c=_tmp429;void*eff=_tmp42A;struct Cyc_List_List*po=_tmp42B;
struct Cyc_Absyn_Exp*req=Cyc_yyget_YY57(&(yyyvsp[4]).v);
struct Cyc_Absyn_Exp*ens=Cyc_yyget_YY57(&(yyyvsp[5]).v);
struct Cyc_List_List*ieff=({unsigned _tmp8C5=(unsigned)((yyyvsp[0]).l).first_line;Cyc_Absyn_parse_rgneffects(_tmp8C5,Cyc_yyget_YY49(&(yyyvsp[6]).v));});
struct Cyc_List_List*oeff=({unsigned _tmp8C6=(unsigned)((yyyvsp[0]).l).first_line;Cyc_Absyn_parse_rgneffects(_tmp8C6,Cyc_yyget_YY49(&(yyyvsp[7]).v));});
yyval=Cyc_YY27(({struct Cyc_Parse_Declarator _tmp661;({struct _tuple0*_tmp8CC=(Cyc_yyget_YY27(&(yyyvsp[0]).v)).id;_tmp661.id=_tmp8CC;}),({unsigned _tmp8CB=(Cyc_yyget_YY27(&(yyyvsp[0]).v)).varloc;_tmp661.varloc=_tmp8CB;}),({struct Cyc_List_List*_tmp8CA=({struct Cyc_List_List*_tmp42E=_region_malloc(yyr,sizeof(*_tmp42E));({void*_tmp8C9=(void*)({struct Cyc_Absyn_Function_mod_Absyn_Type_modifier_struct*_tmp42D=_region_malloc(yyr,sizeof(*_tmp42D));_tmp42D->tag=3U,({void*_tmp8C8=(void*)({struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct*_tmp42C=_region_malloc(yyr,sizeof(*_tmp42C));_tmp42C->tag=1U,_tmp42C->f1=lis,_tmp42C->f2=b,_tmp42C->f3=c,_tmp42C->f4=eff,_tmp42C->f5=po,_tmp42C->f6=req,_tmp42C->f7=ens,_tmp42C->f8=ieff,_tmp42C->f9=oeff;_tmp42C;});_tmp42D->f1=_tmp8C8;});_tmp42D;});_tmp42E->hd=_tmp8C9;}),({struct Cyc_List_List*_tmp8C7=(Cyc_yyget_YY27(&(yyyvsp[0]).v)).tms;_tmp42E->tl=_tmp8C7;});_tmp42E;});_tmp661.tms=_tmp8CA;});_tmp661;}));
# 2008
goto _LL0;}}case 192U: _LL17B: _LL17C: {
# 2011
struct Cyc_List_List*ieff=({unsigned _tmp8CD=(unsigned)((yyyvsp[0]).l).first_line;Cyc_Absyn_parse_rgneffects(_tmp8CD,Cyc_yyget_YY49(&(yyyvsp[7]).v));});
struct Cyc_List_List*oeff=({unsigned _tmp8CE=(unsigned)((yyyvsp[0]).l).first_line;Cyc_Absyn_parse_rgneffects(_tmp8CE,Cyc_yyget_YY49(&(yyyvsp[8]).v));});
yyval=Cyc_YY27(({struct Cyc_Parse_Declarator _tmp662;({struct _tuple0*_tmp8D8=(Cyc_yyget_YY27(&(yyyvsp[0]).v)).id;_tmp662.id=_tmp8D8;}),({unsigned _tmp8D7=(Cyc_yyget_YY27(&(yyyvsp[0]).v)).varloc;_tmp662.varloc=_tmp8D7;}),({
struct Cyc_List_List*_tmp8D6=({struct Cyc_List_List*_tmp431=_region_malloc(yyr,sizeof(*_tmp431));({void*_tmp8D5=(void*)({struct Cyc_Absyn_Function_mod_Absyn_Type_modifier_struct*_tmp430=_region_malloc(yyr,sizeof(*_tmp430));_tmp430->tag=3U,({void*_tmp8D4=(void*)({struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct*_tmp42F=_region_malloc(yyr,sizeof(*_tmp42F));_tmp42F->tag=1U,_tmp42F->f1=0,_tmp42F->f2=0,_tmp42F->f3=0,({
# 2016
void*_tmp8D3=Cyc_yyget_YY49(&(yyyvsp[2]).v);_tmp42F->f4=_tmp8D3;}),({struct Cyc_List_List*_tmp8D2=Cyc_yyget_YY50(&(yyyvsp[3]).v);_tmp42F->f5=_tmp8D2;}),({struct Cyc_Absyn_Exp*_tmp8D1=Cyc_yyget_YY57(&(yyyvsp[5]).v);_tmp42F->f6=_tmp8D1;}),({struct Cyc_Absyn_Exp*_tmp8D0=Cyc_yyget_YY57(&(yyyvsp[6]).v);_tmp42F->f7=_tmp8D0;}),_tmp42F->f8=ieff,_tmp42F->f9=oeff;_tmp42F;});
# 2014
_tmp430->f1=_tmp8D4;});_tmp430;});_tmp431->hd=_tmp8D5;}),({
# 2017
struct Cyc_List_List*_tmp8CF=(Cyc_yyget_YY27(&(yyyvsp[0]).v)).tms;_tmp431->tl=_tmp8CF;});_tmp431;});
# 2014
_tmp662.tms=_tmp8D6;});_tmp662;}));
# 2019
goto _LL0;}case 193U: _LL17D: _LL17E:
# 2020 "parse.y"
 yyval=Cyc_YY27(({struct Cyc_Parse_Declarator _tmp663;({struct _tuple0*_tmp8E0=(Cyc_yyget_YY27(&(yyyvsp[0]).v)).id;_tmp663.id=_tmp8E0;}),({unsigned _tmp8DF=(Cyc_yyget_YY27(&(yyyvsp[0]).v)).varloc;_tmp663.varloc=_tmp8DF;}),({struct Cyc_List_List*_tmp8DE=({struct Cyc_List_List*_tmp434=_region_malloc(yyr,sizeof(*_tmp434));({void*_tmp8DD=(void*)({struct Cyc_Absyn_Function_mod_Absyn_Type_modifier_struct*_tmp433=_region_malloc(yyr,sizeof(*_tmp433));_tmp433->tag=3U,({void*_tmp8DC=(void*)({struct Cyc_Absyn_NoTypes_Absyn_Funcparams_struct*_tmp432=_region_malloc(yyr,sizeof(*_tmp432));_tmp432->tag=0U,({struct Cyc_List_List*_tmp8DB=Cyc_yyget_YY36(&(yyyvsp[2]).v);_tmp432->f1=_tmp8DB;}),({unsigned _tmp8DA=(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;});_tmp432->f2=_tmp8DA;});_tmp432;});_tmp433->f1=_tmp8DC;});_tmp433;});_tmp434->hd=_tmp8DD;}),({struct Cyc_List_List*_tmp8D9=(Cyc_yyget_YY27(&(yyyvsp[0]).v)).tms;_tmp434->tl=_tmp8D9;});_tmp434;});_tmp663.tms=_tmp8DE;});_tmp663;}));
goto _LL0;case 194U: _LL17F: _LL180: {
# 2023
struct Cyc_List_List*_tmp435=({struct Cyc_List_List*(*_tmp8E2)(struct Cyc_Absyn_Tvar*(*f)(unsigned,void*),unsigned env,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp438)(struct Cyc_Absyn_Tvar*(*f)(unsigned,void*),unsigned env,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_Absyn_Tvar*(*f)(unsigned,void*),unsigned env,struct Cyc_List_List*x))Cyc_List_map_c;_tmp438;});unsigned _tmp8E1=(unsigned)({yylloc.first_line=((yyyvsp[1]).l).first_line;((yyyvsp[1]).l).first_line;});_tmp8E2(Cyc_Parse_typ2tvar,_tmp8E1,Cyc_List_imp_rev(Cyc_yyget_YY40(&(yyyvsp[2]).v)));});struct Cyc_List_List*ts=_tmp435;
yyval=Cyc_YY27(({struct Cyc_Parse_Declarator _tmp664;({struct _tuple0*_tmp8E8=(Cyc_yyget_YY27(&(yyyvsp[0]).v)).id;_tmp664.id=_tmp8E8;}),({unsigned _tmp8E7=(Cyc_yyget_YY27(&(yyyvsp[0]).v)).varloc;_tmp664.varloc=_tmp8E7;}),({struct Cyc_List_List*_tmp8E6=({struct Cyc_List_List*_tmp437=_region_malloc(yyr,sizeof(*_tmp437));({void*_tmp8E5=(void*)({struct Cyc_Absyn_TypeParams_mod_Absyn_Type_modifier_struct*_tmp436=_region_malloc(yyr,sizeof(*_tmp436));_tmp436->tag=4U,_tmp436->f1=ts,({unsigned _tmp8E4=(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;});_tmp436->f2=_tmp8E4;}),_tmp436->f3=0;_tmp436;});_tmp437->hd=_tmp8E5;}),({struct Cyc_List_List*_tmp8E3=(Cyc_yyget_YY27(&(yyyvsp[0]).v)).tms;_tmp437->tl=_tmp8E3;});_tmp437;});_tmp664.tms=_tmp8E6;});_tmp664;}));
# 2026
goto _LL0;}case 195U: _LL181: _LL182:
# 2027 "parse.y"
 yyval=Cyc_YY27(({struct Cyc_Parse_Declarator _tmp665;({struct _tuple0*_tmp8EE=(Cyc_yyget_YY27(&(yyyvsp[0]).v)).id;_tmp665.id=_tmp8EE;}),({unsigned _tmp8ED=(Cyc_yyget_YY27(&(yyyvsp[0]).v)).varloc;_tmp665.varloc=_tmp8ED;}),({struct Cyc_List_List*_tmp8EC=({struct Cyc_List_List*_tmp43A=_region_malloc(yyr,sizeof(*_tmp43A));({void*_tmp8EB=(void*)({struct Cyc_Absyn_Attributes_mod_Absyn_Type_modifier_struct*_tmp439=_region_malloc(yyr,sizeof(*_tmp439));_tmp439->tag=5U,_tmp439->f1=(unsigned)((yyyvsp[1]).l).first_line,({struct Cyc_List_List*_tmp8EA=Cyc_yyget_YY45(&(yyyvsp[1]).v);_tmp439->f2=_tmp8EA;});_tmp439;});_tmp43A->hd=_tmp8EB;}),({
struct Cyc_List_List*_tmp8E9=(Cyc_yyget_YY27(&(yyyvsp[0]).v)).tms;_tmp43A->tl=_tmp8E9;});_tmp43A;});
# 2027
_tmp665.tms=_tmp8EC;});_tmp665;}));
# 2030
goto _LL0;case 196U: _LL183: _LL184:
# 2035 "parse.y"
 yyval=Cyc_YY27(({struct Cyc_Parse_Declarator _tmp666;({struct _tuple0*_tmp8EF=Cyc_yyget_QualId_tok(&(yyyvsp[0]).v);_tmp666.id=_tmp8EF;}),_tmp666.varloc=(unsigned)((yyyvsp[0]).l).first_line,_tmp666.tms=0;_tmp666;}));
goto _LL0;case 197U: _LL185: _LL186:
# 2037 "parse.y"
 yyval=Cyc_YY27(({struct Cyc_Parse_Declarator _tmp667;({struct _tuple0*_tmp8F0=Cyc_yyget_QualId_tok(&(yyyvsp[0]).v);_tmp667.id=_tmp8F0;}),_tmp667.varloc=(unsigned)((yyyvsp[0]).l).first_line,_tmp667.tms=0;_tmp667;}));
goto _LL0;case 198U: _LL187: _LL188:
# 2039 "parse.y"
 yyval=(yyyvsp[1]).v;
goto _LL0;case 199U: _LL189: _LL18A: {
# 2043 "parse.y"
struct Cyc_Parse_Declarator d=Cyc_yyget_YY27(&(yyyvsp[2]).v);
({struct Cyc_List_List*_tmp8F3=({struct Cyc_List_List*_tmp43C=_region_malloc(yyr,sizeof(*_tmp43C));({void*_tmp8F2=(void*)({struct Cyc_Absyn_Attributes_mod_Absyn_Type_modifier_struct*_tmp43B=_region_malloc(yyr,sizeof(*_tmp43B));_tmp43B->tag=5U,_tmp43B->f1=(unsigned)((yyyvsp[1]).l).first_line,({struct Cyc_List_List*_tmp8F1=Cyc_yyget_YY45(&(yyyvsp[1]).v);_tmp43B->f2=_tmp8F1;});_tmp43B;});_tmp43C->hd=_tmp8F2;}),_tmp43C->tl=d.tms;_tmp43C;});d.tms=_tmp8F3;});
yyval=(yyyvsp[2]).v;
# 2047
goto _LL0;}case 200U: _LL18B: _LL18C: {
# 2048 "parse.y"
struct Cyc_Parse_Declarator one=Cyc_yyget_YY27(&(yyyvsp[0]).v);
yyval=Cyc_YY27(({struct Cyc_Parse_Declarator _tmp668;_tmp668.id=one.id,_tmp668.varloc=one.varloc,({
struct Cyc_List_List*_tmp8F6=({struct Cyc_List_List*_tmp43E=_region_malloc(yyr,sizeof(*_tmp43E));({void*_tmp8F5=(void*)({struct Cyc_Absyn_Carray_mod_Absyn_Type_modifier_struct*_tmp43D=_region_malloc(yyr,sizeof(*_tmp43D));_tmp43D->tag=0U,({void*_tmp8F4=Cyc_yyget_YY51(&(yyyvsp[3]).v);_tmp43D->f1=_tmp8F4;}),_tmp43D->f2=(unsigned)((yyyvsp[3]).l).first_line;_tmp43D;});_tmp43E->hd=_tmp8F5;}),_tmp43E->tl=one.tms;_tmp43E;});_tmp668.tms=_tmp8F6;});_tmp668;}));
goto _LL0;}case 201U: _LL18D: _LL18E: {
# 2052 "parse.y"
struct Cyc_Parse_Declarator one=Cyc_yyget_YY27(&(yyyvsp[0]).v);
yyval=Cyc_YY27(({struct Cyc_Parse_Declarator _tmp669;_tmp669.id=one.id,_tmp669.varloc=one.varloc,({
struct Cyc_List_List*_tmp8FA=({struct Cyc_List_List*_tmp440=_region_malloc(yyr,sizeof(*_tmp440));({void*_tmp8F9=(void*)({struct Cyc_Absyn_ConstArray_mod_Absyn_Type_modifier_struct*_tmp43F=_region_malloc(yyr,sizeof(*_tmp43F));_tmp43F->tag=1U,({struct Cyc_Absyn_Exp*_tmp8F8=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);_tmp43F->f1=_tmp8F8;}),({void*_tmp8F7=Cyc_yyget_YY51(&(yyyvsp[4]).v);_tmp43F->f2=_tmp8F7;}),_tmp43F->f3=(unsigned)((yyyvsp[4]).l).first_line;_tmp43F;});_tmp440->hd=_tmp8F9;}),_tmp440->tl=one.tms;_tmp440;});_tmp669.tms=_tmp8FA;});_tmp669;}));
# 2056
goto _LL0;}case 202U: _LL18F: _LL190: {
# 2058
struct _tuple27*_stmttmp1B=Cyc_yyget_YY39(&(yyyvsp[2]).v);struct Cyc_List_List*_tmp445;void*_tmp444;struct Cyc_Absyn_VarargInfo*_tmp443;int _tmp442;struct Cyc_List_List*_tmp441;_LL47A: _tmp441=_stmttmp1B->f1;_tmp442=_stmttmp1B->f2;_tmp443=_stmttmp1B->f3;_tmp444=_stmttmp1B->f4;_tmp445=_stmttmp1B->f5;_LL47B: {struct Cyc_List_List*lis=_tmp441;int b=_tmp442;struct Cyc_Absyn_VarargInfo*c=_tmp443;void*eff=_tmp444;struct Cyc_List_List*po=_tmp445;
struct Cyc_Absyn_Exp*req=Cyc_yyget_YY57(&(yyyvsp[4]).v);
struct Cyc_Absyn_Exp*ens=Cyc_yyget_YY57(&(yyyvsp[5]).v);
struct Cyc_Parse_Declarator one=Cyc_yyget_YY27(&(yyyvsp[0]).v);
struct Cyc_List_List*ieff=({unsigned _tmp8FB=(unsigned)((yyyvsp[0]).l).first_line;Cyc_Absyn_parse_rgneffects(_tmp8FB,Cyc_yyget_YY49(&(yyyvsp[6]).v));});
struct Cyc_List_List*oeff=({unsigned _tmp8FC=(unsigned)((yyyvsp[0]).l).first_line;Cyc_Absyn_parse_rgneffects(_tmp8FC,Cyc_yyget_YY49(&(yyyvsp[7]).v));});
yyval=Cyc_YY27(({struct Cyc_Parse_Declarator _tmp66A;_tmp66A.id=one.id,_tmp66A.varloc=one.varloc,({struct Cyc_List_List*_tmp8FF=({struct Cyc_List_List*_tmp448=_region_malloc(yyr,sizeof(*_tmp448));({void*_tmp8FE=(void*)({struct Cyc_Absyn_Function_mod_Absyn_Type_modifier_struct*_tmp447=_region_malloc(yyr,sizeof(*_tmp447));_tmp447->tag=3U,({void*_tmp8FD=(void*)({struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct*_tmp446=_region_malloc(yyr,sizeof(*_tmp446));_tmp446->tag=1U,_tmp446->f1=lis,_tmp446->f2=b,_tmp446->f3=c,_tmp446->f4=eff,_tmp446->f5=po,_tmp446->f6=req,_tmp446->f7=ens,_tmp446->f8=ieff,_tmp446->f9=oeff;_tmp446;});_tmp447->f1=_tmp8FD;});_tmp447;});_tmp448->hd=_tmp8FE;}),_tmp448->tl=one.tms;_tmp448;});_tmp66A.tms=_tmp8FF;});_tmp66A;}));
# 2066
goto _LL0;}}case 203U: _LL191: _LL192: {
# 2068
struct Cyc_Parse_Declarator one=Cyc_yyget_YY27(&(yyyvsp[0]).v);
struct Cyc_List_List*ieff=({unsigned _tmp900=(unsigned)((yyyvsp[0]).l).first_line;Cyc_Absyn_parse_rgneffects(_tmp900,Cyc_yyget_YY49(&(yyyvsp[7]).v));});
struct Cyc_List_List*oeff=({unsigned _tmp901=(unsigned)((yyyvsp[0]).l).first_line;Cyc_Absyn_parse_rgneffects(_tmp901,Cyc_yyget_YY49(&(yyyvsp[8]).v));});
# 2072
yyval=Cyc_YY27(({struct Cyc_Parse_Declarator _tmp66B;_tmp66B.id=one.id,_tmp66B.varloc=one.varloc,({
struct Cyc_List_List*_tmp908=({struct Cyc_List_List*_tmp44B=_region_malloc(yyr,sizeof(*_tmp44B));({void*_tmp907=(void*)({struct Cyc_Absyn_Function_mod_Absyn_Type_modifier_struct*_tmp44A=_region_malloc(yyr,sizeof(*_tmp44A));_tmp44A->tag=3U,({void*_tmp906=(void*)({struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct*_tmp449=_region_malloc(yyr,sizeof(*_tmp449));_tmp449->tag=1U,_tmp449->f1=0,_tmp449->f2=0,_tmp449->f3=0,({
# 2075
void*_tmp905=Cyc_yyget_YY49(&(yyyvsp[2]).v);_tmp449->f4=_tmp905;}),({struct Cyc_List_List*_tmp904=Cyc_yyget_YY50(&(yyyvsp[3]).v);_tmp449->f5=_tmp904;}),({struct Cyc_Absyn_Exp*_tmp903=Cyc_yyget_YY57(&(yyyvsp[5]).v);_tmp449->f6=_tmp903;}),({struct Cyc_Absyn_Exp*_tmp902=Cyc_yyget_YY57(&(yyyvsp[6]).v);_tmp449->f7=_tmp902;}),_tmp449->f8=ieff,_tmp449->f9=oeff;_tmp449;});
# 2073
_tmp44A->f1=_tmp906;});_tmp44A;});_tmp44B->hd=_tmp907;}),_tmp44B->tl=one.tms;_tmp44B;});_tmp66B.tms=_tmp908;});_tmp66B;}));
# 2078
goto _LL0;}case 204U: _LL193: _LL194: {
# 2079 "parse.y"
struct Cyc_Parse_Declarator one=Cyc_yyget_YY27(&(yyyvsp[0]).v);
yyval=Cyc_YY27(({struct Cyc_Parse_Declarator _tmp66C;_tmp66C.id=one.id,_tmp66C.varloc=one.varloc,({struct Cyc_List_List*_tmp90D=({struct Cyc_List_List*_tmp44E=_region_malloc(yyr,sizeof(*_tmp44E));({void*_tmp90C=(void*)({struct Cyc_Absyn_Function_mod_Absyn_Type_modifier_struct*_tmp44D=_region_malloc(yyr,sizeof(*_tmp44D));_tmp44D->tag=3U,({void*_tmp90B=(void*)({struct Cyc_Absyn_NoTypes_Absyn_Funcparams_struct*_tmp44C=_region_malloc(yyr,sizeof(*_tmp44C));_tmp44C->tag=0U,({struct Cyc_List_List*_tmp90A=Cyc_yyget_YY36(&(yyyvsp[2]).v);_tmp44C->f1=_tmp90A;}),({unsigned _tmp909=(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;});_tmp44C->f2=_tmp909;});_tmp44C;});_tmp44D->f1=_tmp90B;});_tmp44D;});_tmp44E->hd=_tmp90C;}),_tmp44E->tl=one.tms;_tmp44E;});_tmp66C.tms=_tmp90D;});_tmp66C;}));
goto _LL0;}case 205U: _LL195: _LL196: {
# 2083
struct Cyc_List_List*_tmp44F=({struct Cyc_List_List*(*_tmp90F)(struct Cyc_Absyn_Tvar*(*f)(unsigned,void*),unsigned env,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp452)(struct Cyc_Absyn_Tvar*(*f)(unsigned,void*),unsigned env,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_Absyn_Tvar*(*f)(unsigned,void*),unsigned env,struct Cyc_List_List*x))Cyc_List_map_c;_tmp452;});unsigned _tmp90E=(unsigned)({yylloc.first_line=((yyyvsp[1]).l).first_line;((yyyvsp[1]).l).first_line;});_tmp90F(Cyc_Parse_typ2tvar,_tmp90E,Cyc_List_imp_rev(Cyc_yyget_YY40(&(yyyvsp[2]).v)));});struct Cyc_List_List*ts=_tmp44F;
struct Cyc_Parse_Declarator one=Cyc_yyget_YY27(&(yyyvsp[0]).v);
yyval=Cyc_YY27(({struct Cyc_Parse_Declarator _tmp66D;_tmp66D.id=one.id,_tmp66D.varloc=one.varloc,({struct Cyc_List_List*_tmp912=({struct Cyc_List_List*_tmp451=_region_malloc(yyr,sizeof(*_tmp451));({void*_tmp911=(void*)({struct Cyc_Absyn_TypeParams_mod_Absyn_Type_modifier_struct*_tmp450=_region_malloc(yyr,sizeof(*_tmp450));_tmp450->tag=4U,_tmp450->f1=ts,({unsigned _tmp910=(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;});_tmp450->f2=_tmp910;}),_tmp450->f3=0;_tmp450;});_tmp451->hd=_tmp911;}),_tmp451->tl=one.tms;_tmp451;});_tmp66D.tms=_tmp912;});_tmp66D;}));
# 2087
goto _LL0;}case 206U: _LL197: _LL198: {
# 2088 "parse.y"
struct Cyc_Parse_Declarator one=Cyc_yyget_YY27(&(yyyvsp[0]).v);
yyval=Cyc_YY27(({struct Cyc_Parse_Declarator _tmp66E;_tmp66E.id=one.id,_tmp66E.varloc=one.varloc,({struct Cyc_List_List*_tmp915=({struct Cyc_List_List*_tmp454=_region_malloc(yyr,sizeof(*_tmp454));({void*_tmp914=(void*)({struct Cyc_Absyn_Attributes_mod_Absyn_Type_modifier_struct*_tmp453=_region_malloc(yyr,sizeof(*_tmp453));_tmp453->tag=5U,_tmp453->f1=(unsigned)((yyyvsp[1]).l).first_line,({struct Cyc_List_List*_tmp913=Cyc_yyget_YY45(&(yyyvsp[1]).v);_tmp453->f2=_tmp913;});_tmp453;});_tmp454->hd=_tmp914;}),_tmp454->tl=one.tms;_tmp454;});_tmp66E.tms=_tmp915;});_tmp66E;}));
# 2091
goto _LL0;}case 207U: _LL199: _LL19A:
# 2095 "parse.y"
 yyval=(yyyvsp[0]).v;
goto _LL0;case 208U: _LL19B: _LL19C:
# 2096 "parse.y"
 yyval=Cyc_YY26(({struct Cyc_List_List*_tmp916=Cyc_yyget_YY26(&(yyyvsp[0]).v);Cyc_List_imp_append(_tmp916,Cyc_yyget_YY26(&(yyyvsp[1]).v));}));
goto _LL0;case 209U: _LL19D: _LL19E: {
# 2100 "parse.y"
struct Cyc_List_List*ans=0;
if(Cyc_yyget_YY45(&(yyyvsp[3]).v)!= 0)
ans=({struct Cyc_List_List*_tmp456=_region_malloc(yyr,sizeof(*_tmp456));({void*_tmp918=(void*)({struct Cyc_Absyn_Attributes_mod_Absyn_Type_modifier_struct*_tmp455=_region_malloc(yyr,sizeof(*_tmp455));_tmp455->tag=5U,_tmp455->f1=(unsigned)((yyyvsp[3]).l).first_line,({struct Cyc_List_List*_tmp917=Cyc_yyget_YY45(&(yyyvsp[3]).v);_tmp455->f2=_tmp917;});_tmp455;});_tmp456->hd=_tmp918;}),_tmp456->tl=ans;_tmp456;});{
# 2101
struct Cyc_Absyn_PtrLoc*ptrloc=0;
# 2105
struct _tuple22 _stmttmp1C=*Cyc_yyget_YY1(&(yyyvsp[0]).v);void*_tmp459;void*_tmp458;unsigned _tmp457;_LL47D: _tmp457=_stmttmp1C.f1;_tmp458=_stmttmp1C.f2;_tmp459=_stmttmp1C.f3;_LL47E: {unsigned ploc=_tmp457;void*nullable=_tmp458;void*bound=_tmp459;
if(Cyc_Absyn_porting_c_code)
ptrloc=({struct Cyc_Absyn_PtrLoc*_tmp45A=_cycalloc(sizeof(*_tmp45A));_tmp45A->ptr_loc=ploc,_tmp45A->rgn_loc=(unsigned)((yyyvsp[2]).l).first_line,_tmp45A->zt_loc=(unsigned)((yyyvsp[1]).l).first_line;_tmp45A;});{
# 2106
struct _tuple15 _stmttmp1D=({
# 2109
unsigned _tmp91C=ploc;void*_tmp91B=nullable;void*_tmp91A=bound;void*_tmp919=Cyc_yyget_YY44(&(yyyvsp[2]).v);Cyc_Parse_collapse_pointer_quals(_tmp91C,_tmp91B,_tmp91A,_tmp919,Cyc_yyget_YY56(&(yyyvsp[1]).v));});
# 2106
void*_tmp45E;void*_tmp45D;void*_tmp45C;void*_tmp45B;_LL480: _tmp45B=_stmttmp1D.f1;_tmp45C=_stmttmp1D.f2;_tmp45D=_stmttmp1D.f3;_tmp45E=_stmttmp1D.f4;_LL481: {void*nullable=_tmp45B;void*bound=_tmp45C;void*zeroterm=_tmp45D;void*rgn_opt=_tmp45E;
# 2110
ans=({struct Cyc_List_List*_tmp460=_region_malloc(yyr,sizeof(*_tmp460));({void*_tmp91E=(void*)({struct Cyc_Absyn_Pointer_mod_Absyn_Type_modifier_struct*_tmp45F=_region_malloc(yyr,sizeof(*_tmp45F));_tmp45F->tag=2U,(_tmp45F->f1).rgn=rgn_opt,(_tmp45F->f1).nullable=nullable,(_tmp45F->f1).bounds=bound,(_tmp45F->f1).zero_term=zeroterm,(_tmp45F->f1).ptrloc=ptrloc,({struct Cyc_Absyn_Tqual _tmp91D=Cyc_yyget_YY23(&(yyyvsp[4]).v);_tmp45F->f2=_tmp91D;});_tmp45F;});_tmp460->hd=_tmp91E;}),_tmp460->tl=ans;_tmp460;});
yyval=Cyc_YY26(ans);
# 2113
goto _LL0;}}}}}case 210U: _LL19F: _LL1A0:
# 2115
 yyval=Cyc_YY56(0);
goto _LL0;case 211U: _LL1A1: _LL1A2:
# 2116 "parse.y"
 yyval=Cyc_YY56(({struct Cyc_List_List*_tmp461=_region_malloc(yyr,sizeof(*_tmp461));({void*_tmp920=Cyc_yyget_YY55(&(yyyvsp[0]).v);_tmp461->hd=_tmp920;}),({struct Cyc_List_List*_tmp91F=Cyc_yyget_YY56(&(yyyvsp[1]).v);_tmp461->tl=_tmp91F;});_tmp461;}));
goto _LL0;case 212U: _LL1A3: _LL1A4:
# 2121 "parse.y"
 yyval=Cyc_YY55((void*)({struct Cyc_Parse_Numelts_ptrqual_Parse_Pointer_qual_struct*_tmp462=_region_malloc(yyr,sizeof(*_tmp462));_tmp462->tag=0U,({struct Cyc_Absyn_Exp*_tmp921=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);_tmp462->f1=_tmp921;});_tmp462;}));
goto _LL0;case 213U: _LL1A5: _LL1A6:
# 2123 "parse.y"
 yyval=Cyc_YY55((void*)({struct Cyc_Parse_Region_ptrqual_Parse_Pointer_qual_struct*_tmp463=_region_malloc(yyr,sizeof(*_tmp463));_tmp463->tag=1U,({void*_tmp922=Cyc_yyget_YY44(&(yyyvsp[2]).v);_tmp463->f1=_tmp922;});_tmp463;}));
goto _LL0;case 214U: _LL1A7: _LL1A8:
# 2125 "parse.y"
 yyval=Cyc_YY55((void*)({struct Cyc_Parse_Thin_ptrqual_Parse_Pointer_qual_struct*_tmp464=_region_malloc(yyr,sizeof(*_tmp464));_tmp464->tag=2U;_tmp464;}));
goto _LL0;case 215U: _LL1A9: _LL1AA:
# 2127 "parse.y"
 yyval=Cyc_YY55((void*)({struct Cyc_Parse_Fat_ptrqual_Parse_Pointer_qual_struct*_tmp465=_region_malloc(yyr,sizeof(*_tmp465));_tmp465->tag=3U;_tmp465;}));
goto _LL0;case 216U: _LL1AB: _LL1AC:
# 2129 "parse.y"
 yyval=Cyc_YY55((void*)({struct Cyc_Parse_Zeroterm_ptrqual_Parse_Pointer_qual_struct*_tmp466=_region_malloc(yyr,sizeof(*_tmp466));_tmp466->tag=4U;_tmp466;}));
goto _LL0;case 217U: _LL1AD: _LL1AE:
# 2131 "parse.y"
 yyval=Cyc_YY55((void*)({struct Cyc_Parse_Nozeroterm_ptrqual_Parse_Pointer_qual_struct*_tmp467=_region_malloc(yyr,sizeof(*_tmp467));_tmp467->tag=5U;_tmp467;}));
goto _LL0;case 218U: _LL1AF: _LL1B0:
# 2133 "parse.y"
 yyval=Cyc_YY55((void*)({struct Cyc_Parse_Notnull_ptrqual_Parse_Pointer_qual_struct*_tmp468=_region_malloc(yyr,sizeof(*_tmp468));_tmp468->tag=6U;_tmp468;}));
goto _LL0;case 219U: _LL1B1: _LL1B2:
# 2135 "parse.y"
 yyval=Cyc_YY55((void*)({struct Cyc_Parse_Nullable_ptrqual_Parse_Pointer_qual_struct*_tmp469=_region_malloc(yyr,sizeof(*_tmp469));_tmp469->tag=7U;_tmp469;}));
goto _LL0;case 220U: _LL1B3: _LL1B4: {
# 2141 "parse.y"
unsigned loc=(unsigned)((yyyvsp[0]).l).first_line;
if(!Cyc_Parse_parsing_tempest)
yyval=Cyc_YY1(({struct _tuple22*_tmp46A=_cycalloc(sizeof(*_tmp46A));_tmp46A->f1=loc,_tmp46A->f2=Cyc_Absyn_true_type,({void*_tmp923=Cyc_yyget_YY2(&(yyyvsp[1]).v);_tmp46A->f3=_tmp923;});_tmp46A;}));else{
# 2145
yyval=Cyc_YY1(({struct _tuple22*_tmp46B=_cycalloc(sizeof(*_tmp46B));_tmp46B->f1=loc,_tmp46B->f2=Cyc_Absyn_true_type,_tmp46B->f3=Cyc_Absyn_fat_bound_type;_tmp46B;}));}
# 2147
goto _LL0;}case 221U: _LL1B5: _LL1B6: {
# 2148 "parse.y"
unsigned loc=(unsigned)((yyyvsp[0]).l).first_line;
yyval=Cyc_YY1(({struct _tuple22*_tmp46C=_cycalloc(sizeof(*_tmp46C));_tmp46C->f1=loc,_tmp46C->f2=Cyc_Absyn_false_type,({void*_tmp924=Cyc_yyget_YY2(&(yyyvsp[1]).v);_tmp46C->f3=_tmp924;});_tmp46C;}));
# 2151
goto _LL0;}case 222U: _LL1B7: _LL1B8: {
# 2152 "parse.y"
unsigned loc=(unsigned)((yyyvsp[0]).l).first_line;
yyval=Cyc_YY1(({struct _tuple22*_tmp46D=_cycalloc(sizeof(*_tmp46D));_tmp46D->f1=loc,_tmp46D->f2=Cyc_Absyn_true_type,_tmp46D->f3=Cyc_Absyn_fat_bound_type;_tmp46D;}));
# 2155
goto _LL0;}case 223U: _LL1B9: _LL1BA:
# 2157
 yyval=Cyc_YY2(Cyc_Absyn_bounds_one());
goto _LL0;case 224U: _LL1BB: _LL1BC:
# 2158 "parse.y"
 yyval=Cyc_YY2(Cyc_Absyn_thin_bounds_exp(Cyc_yyget_Exp_tok(&(yyyvsp[1]).v)));
goto _LL0;case 225U: _LL1BD: _LL1BE:
# 2161
 yyval=Cyc_YY51(Cyc_Tcutil_any_bool(0));
goto _LL0;case 226U: _LL1BF: _LL1C0:
# 2162 "parse.y"
 yyval=Cyc_YY51(Cyc_Absyn_true_type);
goto _LL0;case 227U: _LL1C1: _LL1C2:
# 2163 "parse.y"
 yyval=Cyc_YY51(Cyc_Absyn_false_type);
goto _LL0;case 228U: _LL1C3: _LL1C4:
# 2168 "parse.y"
 yyval=Cyc_YY44(Cyc_Absyn_new_evar(& Cyc_Tcutil_trko,0));
goto _LL0;case 229U: _LL1C5: _LL1C6:
# 2169 "parse.y"
 Cyc_Parse_set_vartyp_kind(Cyc_yyget_YY44(&(yyyvsp[0]).v),& Cyc_Tcutil_trk,1);yyval=(yyyvsp[0]).v;
goto _LL0;case 230U: _LL1C7: _LL1C8:
# 2170 "parse.y"
 yyval=Cyc_YY44(Cyc_Absyn_new_evar(& Cyc_Tcutil_trko,0));
goto _LL0;case 231U: _LL1C9: _LL1CA:
# 2174 "parse.y"
 yyval=Cyc_YY23(Cyc_Absyn_empty_tqual((unsigned)((*((struct Cyc_Yystacktype*)_check_fat_subscript(yyvs,sizeof(struct Cyc_Yystacktype),yyvsp_offset + 1))).l).first_line));
goto _LL0;case 232U: _LL1CB: _LL1CC:
# 2175 "parse.y"
 yyval=Cyc_YY23(({struct Cyc_Absyn_Tqual _tmp925=Cyc_yyget_YY23(&(yyyvsp[0]).v);Cyc_Absyn_combine_tqual(_tmp925,Cyc_yyget_YY23(&(yyyvsp[1]).v));}));
goto _LL0;case 233U: _LL1CD: _LL1CE:
# 2180 "parse.y"
 yyval=Cyc_YY39(({struct _tuple27*_tmp46E=_cycalloc(sizeof(*_tmp46E));({struct Cyc_List_List*_tmp928=Cyc_List_imp_rev(Cyc_yyget_YY38(&(yyyvsp[0]).v));_tmp46E->f1=_tmp928;}),_tmp46E->f2=0,_tmp46E->f3=0,({void*_tmp927=Cyc_yyget_YY49(&(yyyvsp[1]).v);_tmp46E->f4=_tmp927;}),({struct Cyc_List_List*_tmp926=Cyc_yyget_YY50(&(yyyvsp[2]).v);_tmp46E->f5=_tmp926;});_tmp46E;}));
goto _LL0;case 234U: _LL1CF: _LL1D0:
# 2182 "parse.y"
 yyval=Cyc_YY39(({struct _tuple27*_tmp46F=_cycalloc(sizeof(*_tmp46F));({struct Cyc_List_List*_tmp92B=Cyc_List_imp_rev(Cyc_yyget_YY38(&(yyyvsp[0]).v));_tmp46F->f1=_tmp92B;}),_tmp46F->f2=1,_tmp46F->f3=0,({void*_tmp92A=Cyc_yyget_YY49(&(yyyvsp[3]).v);_tmp46F->f4=_tmp92A;}),({struct Cyc_List_List*_tmp929=Cyc_yyget_YY50(&(yyyvsp[4]).v);_tmp46F->f5=_tmp929;});_tmp46F;}));
goto _LL0;case 235U: _LL1D1: _LL1D2: {
# 2185
struct _tuple8*_stmttmp1E=Cyc_yyget_YY37(&(yyyvsp[2]).v);void*_tmp472;struct Cyc_Absyn_Tqual _tmp471;struct _fat_ptr*_tmp470;_LL483: _tmp470=_stmttmp1E->f1;_tmp471=_stmttmp1E->f2;_tmp472=_stmttmp1E->f3;_LL484: {struct _fat_ptr*n=_tmp470;struct Cyc_Absyn_Tqual tq=_tmp471;void*t=_tmp472;
struct Cyc_Absyn_VarargInfo*v=({struct Cyc_Absyn_VarargInfo*_tmp474=_cycalloc(sizeof(*_tmp474));_tmp474->name=n,_tmp474->tq=tq,_tmp474->type=t,({int _tmp92C=Cyc_yyget_YY31(&(yyyvsp[1]).v);_tmp474->inject=_tmp92C;});_tmp474;});
yyval=Cyc_YY39(({struct _tuple27*_tmp473=_cycalloc(sizeof(*_tmp473));_tmp473->f1=0,_tmp473->f2=0,_tmp473->f3=v,({void*_tmp92E=Cyc_yyget_YY49(&(yyyvsp[3]).v);_tmp473->f4=_tmp92E;}),({struct Cyc_List_List*_tmp92D=Cyc_yyget_YY50(&(yyyvsp[4]).v);_tmp473->f5=_tmp92D;});_tmp473;}));
# 2189
goto _LL0;}}case 236U: _LL1D3: _LL1D4: {
# 2191
struct _tuple8*_stmttmp1F=Cyc_yyget_YY37(&(yyyvsp[4]).v);void*_tmp477;struct Cyc_Absyn_Tqual _tmp476;struct _fat_ptr*_tmp475;_LL486: _tmp475=_stmttmp1F->f1;_tmp476=_stmttmp1F->f2;_tmp477=_stmttmp1F->f3;_LL487: {struct _fat_ptr*n=_tmp475;struct Cyc_Absyn_Tqual tq=_tmp476;void*t=_tmp477;
struct Cyc_Absyn_VarargInfo*v=({struct Cyc_Absyn_VarargInfo*_tmp479=_cycalloc(sizeof(*_tmp479));_tmp479->name=n,_tmp479->tq=tq,_tmp479->type=t,({int _tmp92F=Cyc_yyget_YY31(&(yyyvsp[3]).v);_tmp479->inject=_tmp92F;});_tmp479;});
yyval=Cyc_YY39(({struct _tuple27*_tmp478=_cycalloc(sizeof(*_tmp478));({struct Cyc_List_List*_tmp932=Cyc_List_imp_rev(Cyc_yyget_YY38(&(yyyvsp[0]).v));_tmp478->f1=_tmp932;}),_tmp478->f2=0,_tmp478->f3=v,({void*_tmp931=Cyc_yyget_YY49(&(yyyvsp[5]).v);_tmp478->f4=_tmp931;}),({struct Cyc_List_List*_tmp930=Cyc_yyget_YY50(&(yyyvsp[6]).v);_tmp478->f5=_tmp930;});_tmp478;}));
# 2195
goto _LL0;}}case 237U: _LL1D5: _LL1D6:
# 2199 "parse.y"
 yyval=Cyc_YY44(({struct _fat_ptr _tmp933=Cyc_yyget_String_tok(&(yyyvsp[0]).v);Cyc_Parse_id2type(_tmp933,(void*)({struct Cyc_Absyn_Unknown_kb_Absyn_KindBound_struct*_tmp47A=_cycalloc(sizeof(*_tmp47A));_tmp47A->tag=1U,_tmp47A->f1=0;_tmp47A;}));}));
goto _LL0;case 238U: _LL1D7: _LL1D8:
# 2200 "parse.y"
 yyval=Cyc_YY44(({struct _fat_ptr _tmp934=Cyc_yyget_String_tok(&(yyyvsp[0]).v);Cyc_Parse_id2type(_tmp934,Cyc_Tcutil_kind_to_bound(Cyc_yyget_YY43(&(yyyvsp[2]).v)));}));
goto _LL0;case 239U: _LL1D9: _LL1DA:
# 2203
 yyval=Cyc_YY49(0);
goto _LL0;case 240U: _LL1DB: _LL1DC:
# 2204 "parse.y"
 yyval=Cyc_YY49(Cyc_Absyn_join_eff(Cyc_yyget_YY40(&(yyyvsp[1]).v)));
goto _LL0;case 241U: _LL1DD: _LL1DE:
# 2208 "parse.y"
 yyval=Cyc_YY50(0);
goto _LL0;case 242U: _LL1DF: _LL1E0:
# 2209 "parse.y"
 yyval=(yyyvsp[1]).v;
goto _LL0;case 243U: _LL1E1: _LL1E2: {
# 2217 "parse.y"
struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*kb=({struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*_tmp47D=_cycalloc(sizeof(*_tmp47D));_tmp47D->tag=2U,_tmp47D->f1=0,_tmp47D->f2=& Cyc_Tcutil_trk;_tmp47D;});
struct _fat_ptr id=Cyc_yyget_String_tok(&(yyyvsp[2]).v);
void*t=Cyc_Parse_id2type(id,(void*)kb);
yyval=Cyc_YY50(({struct Cyc_List_List*_tmp47C=_cycalloc(sizeof(*_tmp47C));({struct _tuple33*_tmp936=({struct _tuple33*_tmp47B=_cycalloc(sizeof(*_tmp47B));({void*_tmp935=Cyc_Absyn_join_eff(Cyc_yyget_YY40(&(yyyvsp[0]).v));_tmp47B->f1=_tmp935;}),_tmp47B->f2=t;_tmp47B;});_tmp47C->hd=_tmp936;}),_tmp47C->tl=0;_tmp47C;}));
# 2222
goto _LL0;}case 244U: _LL1E3: _LL1E4: {
# 2224 "parse.y"
struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*kb=({struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*_tmp480=_cycalloc(sizeof(*_tmp480));_tmp480->tag=2U,_tmp480->f1=0,_tmp480->f2=& Cyc_Tcutil_trk;_tmp480;});
struct _fat_ptr id=Cyc_yyget_String_tok(&(yyyvsp[2]).v);
void*t=Cyc_Parse_id2type(id,(void*)kb);
yyval=Cyc_YY50(({struct Cyc_List_List*_tmp47F=_cycalloc(sizeof(*_tmp47F));({struct _tuple33*_tmp939=({struct _tuple33*_tmp47E=_cycalloc(sizeof(*_tmp47E));({void*_tmp938=Cyc_Absyn_join_eff(Cyc_yyget_YY40(&(yyyvsp[0]).v));_tmp47E->f1=_tmp938;}),_tmp47E->f2=t;_tmp47E;});_tmp47F->hd=_tmp939;}),({struct Cyc_List_List*_tmp937=Cyc_yyget_YY50(&(yyyvsp[4]).v);_tmp47F->tl=_tmp937;});_tmp47F;}));
# 2229
goto _LL0;}case 245U: _LL1E5: _LL1E6:
# 2233 "parse.y"
 yyval=Cyc_YY31(0);
goto _LL0;case 246U: _LL1E7: _LL1E8:
# 2235 "parse.y"
 if(({struct _fat_ptr _tmp93A=(struct _fat_ptr)Cyc_yyget_String_tok(&(yyyvsp[0]).v);Cyc_zstrcmp(_tmp93A,({const char*_tmp481="inject";_tag_fat(_tmp481,sizeof(char),7U);}));})!= 0)
({void*_tmp482=0U;({unsigned _tmp93C=(unsigned)((yyyvsp[0]).l).first_line;struct _fat_ptr _tmp93B=({const char*_tmp483="missing type in function declaration";_tag_fat(_tmp483,sizeof(char),37U);});Cyc_Warn_err(_tmp93C,_tmp93B,_tag_fat(_tmp482,sizeof(void*),0U));});});
# 2235
yyval=
# 2237
Cyc_YY31(1);
# 2239
goto _LL0;case 247U: _LL1E9: _LL1EA:
# 2242 "parse.y"
 yyval=(yyyvsp[0]).v;
goto _LL0;case 248U: _LL1EB: _LL1EC:
# 2243 "parse.y"
 yyval=Cyc_YY40(({struct Cyc_List_List*_tmp93D=Cyc_yyget_YY40(&(yyyvsp[0]).v);Cyc_List_imp_append(_tmp93D,Cyc_yyget_YY40(&(yyyvsp[2]).v));}));
goto _LL0;case 249U: _LL1ED: _LL1EE:
# 2247 "parse.y"
 yyval=Cyc_YY40(0);
goto _LL0;case 250U: _LL1EF: _LL1F0:
# 2248 "parse.y"
 yyval=(yyyvsp[1]).v;
goto _LL0;case 251U: _LL1F1: _LL1F2:
# 2250 "parse.y"
 yyval=Cyc_YY40(({struct Cyc_List_List*_tmp484=_cycalloc(sizeof(*_tmp484));({void*_tmp93E=Cyc_Absyn_regionsof_eff(Cyc_yyget_YY44(&(yyyvsp[2]).v));_tmp484->hd=_tmp93E;}),_tmp484->tl=0;_tmp484;}));
goto _LL0;case 252U: _LL1F3: _LL1F4:
# 2252 "parse.y"
 Cyc_Parse_set_vartyp_kind(Cyc_yyget_YY44(&(yyyvsp[0]).v),& Cyc_Tcutil_ek,0);
yyval=Cyc_YY40(({struct Cyc_List_List*_tmp485=_cycalloc(sizeof(*_tmp485));({void*_tmp93F=Cyc_yyget_YY44(&(yyyvsp[0]).v);_tmp485->hd=_tmp93F;}),_tmp485->tl=0;_tmp485;}));
# 2255
goto _LL0;case 253U: _LL1F5: _LL1F6:
# 2272 "parse.y"
 yyval=Cyc_YY40(({struct Cyc_List_List*_tmp486=_cycalloc(sizeof(*_tmp486));({void*_tmp941=Cyc_Absyn_access_eff(({struct _tuple8*_tmp940=Cyc_yyget_YY37(&(yyyvsp[0]).v);Cyc_Parse_type_name_to_type(_tmp940,(unsigned)((yyyvsp[0]).l).first_line);}));_tmp486->hd=_tmp941;}),_tmp486->tl=0;_tmp486;}));
goto _LL0;case 254U: _LL1F7: _LL1F8:
# 2274 "parse.y"
 yyval=Cyc_YY40(({struct Cyc_List_List*_tmp487=_cycalloc(sizeof(*_tmp487));({void*_tmp944=Cyc_Absyn_access_eff(({struct _tuple8*_tmp943=Cyc_yyget_YY37(&(yyyvsp[0]).v);Cyc_Parse_type_name_to_type(_tmp943,(unsigned)((yyyvsp[0]).l).first_line);}));_tmp487->hd=_tmp944;}),({struct Cyc_List_List*_tmp942=Cyc_yyget_YY40(&(yyyvsp[2]).v);_tmp487->tl=_tmp942;});_tmp487;}));
goto _LL0;case 255U: _LL1F9: _LL1FA:
# 2280 "parse.y"
 yyval=Cyc_YY38(({struct Cyc_List_List*_tmp488=_cycalloc(sizeof(*_tmp488));({struct _tuple8*_tmp945=Cyc_yyget_YY37(&(yyyvsp[0]).v);_tmp488->hd=_tmp945;}),_tmp488->tl=0;_tmp488;}));
goto _LL0;case 256U: _LL1FB: _LL1FC:
# 2282 "parse.y"
 yyval=Cyc_YY38(({struct Cyc_List_List*_tmp489=_cycalloc(sizeof(*_tmp489));({struct _tuple8*_tmp947=Cyc_yyget_YY37(&(yyyvsp[2]).v);_tmp489->hd=_tmp947;}),({struct Cyc_List_List*_tmp946=Cyc_yyget_YY38(&(yyyvsp[0]).v);_tmp489->tl=_tmp946;});_tmp489;}));
goto _LL0;case 257U: _LL1FD: _LL1FE: {
# 2288 "parse.y"
struct _tuple26 _stmttmp20=Cyc_yyget_YY35(&(yyyvsp[0]).v);struct Cyc_List_List*_tmp48C;struct Cyc_Parse_Type_specifier _tmp48B;struct Cyc_Absyn_Tqual _tmp48A;_LL489: _tmp48A=_stmttmp20.f1;_tmp48B=_stmttmp20.f2;_tmp48C=_stmttmp20.f3;_LL48A: {struct Cyc_Absyn_Tqual tq=_tmp48A;struct Cyc_Parse_Type_specifier tspecs=_tmp48B;struct Cyc_List_List*atts=_tmp48C;
if(tq.loc == (unsigned)0)tq.loc=(unsigned)((yyyvsp[0]).l).first_line;{struct Cyc_Parse_Declarator _stmttmp21=
Cyc_yyget_YY27(&(yyyvsp[1]).v);
# 2289
struct Cyc_List_List*_tmp48F;unsigned _tmp48E;struct _tuple0*_tmp48D;_LL48C: _tmp48D=_stmttmp21.id;_tmp48E=_stmttmp21.varloc;_tmp48F=_stmttmp21.tms;_LL48D: {struct _tuple0*qv=_tmp48D;unsigned varloc=_tmp48E;struct Cyc_List_List*tms=_tmp48F;
# 2291
void*t=Cyc_Parse_speclist2typ(tspecs,(unsigned)((yyyvsp[0]).l).first_line);
struct _tuple14 _stmttmp22=Cyc_Parse_apply_tms(tq,t,atts,tms);struct Cyc_List_List*_tmp493;struct Cyc_List_List*_tmp492;void*_tmp491;struct Cyc_Absyn_Tqual _tmp490;_LL48F: _tmp490=_stmttmp22.f1;_tmp491=_stmttmp22.f2;_tmp492=_stmttmp22.f3;_tmp493=_stmttmp22.f4;_LL490: {struct Cyc_Absyn_Tqual tq2=_tmp490;void*t2=_tmp491;struct Cyc_List_List*tvs=_tmp492;struct Cyc_List_List*atts2=_tmp493;
if(tvs != 0)
({void*_tmp494=0U;({unsigned _tmp949=(unsigned)((yyyvsp[1]).l).first_line;struct _fat_ptr _tmp948=({const char*_tmp495="parameter with bad type params";_tag_fat(_tmp495,sizeof(char),31U);});Cyc_Warn_err(_tmp949,_tmp948,_tag_fat(_tmp494,sizeof(void*),0U));});});
# 2293
if(
# 2295
Cyc_Absyn_is_qvar_qualified(qv))
({void*_tmp496=0U;({unsigned _tmp94B=(unsigned)((yyyvsp[0]).l).first_line;struct _fat_ptr _tmp94A=({const char*_tmp497="parameter cannot be qualified with a namespace";_tag_fat(_tmp497,sizeof(char),47U);});Cyc_Warn_err(_tmp94B,_tmp94A,_tag_fat(_tmp496,sizeof(void*),0U));});});{
# 2293
struct _fat_ptr*idopt=(*qv).f2;
# 2298
if(atts2 != 0)
({void*_tmp498=0U;({unsigned _tmp94D=(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;});struct _fat_ptr _tmp94C=({const char*_tmp499="extra attributes on parameter, ignoring";_tag_fat(_tmp499,sizeof(char),40U);});Cyc_Warn_warn(_tmp94D,_tmp94C,_tag_fat(_tmp498,sizeof(void*),0U));});});
# 2298
yyval=
# 2300
Cyc_YY37(({struct _tuple8*_tmp49A=_cycalloc(sizeof(*_tmp49A));_tmp49A->f1=idopt,_tmp49A->f2=tq2,_tmp49A->f3=t2;_tmp49A;}));
# 2302
goto _LL0;}}}}}}case 258U: _LL1FF: _LL200: {
# 2303 "parse.y"
struct _tuple26 _stmttmp23=Cyc_yyget_YY35(&(yyyvsp[0]).v);struct Cyc_List_List*_tmp49D;struct Cyc_Parse_Type_specifier _tmp49C;struct Cyc_Absyn_Tqual _tmp49B;_LL492: _tmp49B=_stmttmp23.f1;_tmp49C=_stmttmp23.f2;_tmp49D=_stmttmp23.f3;_LL493: {struct Cyc_Absyn_Tqual tq=_tmp49B;struct Cyc_Parse_Type_specifier tspecs=_tmp49C;struct Cyc_List_List*atts=_tmp49D;
if(tq.loc == (unsigned)0)tq.loc=(unsigned)((yyyvsp[0]).l).first_line;{void*t=
Cyc_Parse_speclist2typ(tspecs,(unsigned)((yyyvsp[0]).l).first_line);
if(atts != 0)
({void*_tmp49E=0U;({unsigned _tmp94F=(unsigned)((yyyvsp[0]).l).first_line;struct _fat_ptr _tmp94E=({const char*_tmp49F="bad attributes on parameter, ignoring";_tag_fat(_tmp49F,sizeof(char),38U);});Cyc_Warn_warn(_tmp94F,_tmp94E,_tag_fat(_tmp49E,sizeof(void*),0U));});});
# 2306
yyval=
# 2308
Cyc_YY37(({struct _tuple8*_tmp4A0=_cycalloc(sizeof(*_tmp4A0));_tmp4A0->f1=0,_tmp4A0->f2=tq,_tmp4A0->f3=t;_tmp4A0;}));
# 2310
goto _LL0;}}}case 259U: _LL201: _LL202: {
# 2311 "parse.y"
struct _tuple26 _stmttmp24=Cyc_yyget_YY35(&(yyyvsp[0]).v);struct Cyc_List_List*_tmp4A3;struct Cyc_Parse_Type_specifier _tmp4A2;struct Cyc_Absyn_Tqual _tmp4A1;_LL495: _tmp4A1=_stmttmp24.f1;_tmp4A2=_stmttmp24.f2;_tmp4A3=_stmttmp24.f3;_LL496: {struct Cyc_Absyn_Tqual tq=_tmp4A1;struct Cyc_Parse_Type_specifier tspecs=_tmp4A2;struct Cyc_List_List*atts=_tmp4A3;
if(tq.loc == (unsigned)0)tq.loc=(unsigned)((yyyvsp[0]).l).first_line;{void*t=
Cyc_Parse_speclist2typ(tspecs,(unsigned)((yyyvsp[0]).l).first_line);
struct Cyc_List_List*tms=(Cyc_yyget_YY30(&(yyyvsp[1]).v)).tms;
struct _tuple14 _stmttmp25=Cyc_Parse_apply_tms(tq,t,atts,tms);struct Cyc_List_List*_tmp4A7;struct Cyc_List_List*_tmp4A6;void*_tmp4A5;struct Cyc_Absyn_Tqual _tmp4A4;_LL498: _tmp4A4=_stmttmp25.f1;_tmp4A5=_stmttmp25.f2;_tmp4A6=_stmttmp25.f3;_tmp4A7=_stmttmp25.f4;_LL499: {struct Cyc_Absyn_Tqual tq2=_tmp4A4;void*t2=_tmp4A5;struct Cyc_List_List*tvs=_tmp4A6;struct Cyc_List_List*atts2=_tmp4A7;
if(tvs != 0)
({void*_tmp4A8=0U;({unsigned _tmp951=(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;});struct _fat_ptr _tmp950=({const char*_tmp4A9="bad type parameters on formal argument, ignoring";_tag_fat(_tmp4A9,sizeof(char),49U);});Cyc_Warn_warn(_tmp951,_tmp950,_tag_fat(_tmp4A8,sizeof(void*),0U));});});
# 2316
if(atts2 != 0)
# 2320
({void*_tmp4AA=0U;({unsigned _tmp953=(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;});struct _fat_ptr _tmp952=({const char*_tmp4AB="bad attributes on parameter, ignoring";_tag_fat(_tmp4AB,sizeof(char),38U);});Cyc_Warn_warn(_tmp953,_tmp952,_tag_fat(_tmp4AA,sizeof(void*),0U));});});
# 2316
yyval=
# 2321
Cyc_YY37(({struct _tuple8*_tmp4AC=_cycalloc(sizeof(*_tmp4AC));_tmp4AC->f1=0,_tmp4AC->f2=tq2,_tmp4AC->f3=t2;_tmp4AC;}));
# 2323
goto _LL0;}}}}case 260U: _LL203: _LL204:
# 2327 "parse.y"
 yyval=Cyc_YY36(Cyc_List_imp_rev(Cyc_yyget_YY36(&(yyyvsp[0]).v)));
goto _LL0;case 261U: _LL205: _LL206:
# 2331 "parse.y"
 yyval=Cyc_YY36(({struct Cyc_List_List*_tmp4AE=_cycalloc(sizeof(*_tmp4AE));({struct _fat_ptr*_tmp955=({struct _fat_ptr*_tmp4AD=_cycalloc(sizeof(*_tmp4AD));({struct _fat_ptr _tmp954=Cyc_yyget_String_tok(&(yyyvsp[0]).v);*_tmp4AD=_tmp954;});_tmp4AD;});_tmp4AE->hd=_tmp955;}),_tmp4AE->tl=0;_tmp4AE;}));
goto _LL0;case 262U: _LL207: _LL208:
# 2333 "parse.y"
 yyval=Cyc_YY36(({struct Cyc_List_List*_tmp4B0=_cycalloc(sizeof(*_tmp4B0));({struct _fat_ptr*_tmp958=({struct _fat_ptr*_tmp4AF=_cycalloc(sizeof(*_tmp4AF));({struct _fat_ptr _tmp957=Cyc_yyget_String_tok(&(yyyvsp[2]).v);*_tmp4AF=_tmp957;});_tmp4AF;});_tmp4B0->hd=_tmp958;}),({struct Cyc_List_List*_tmp956=Cyc_yyget_YY36(&(yyyvsp[0]).v);_tmp4B0->tl=_tmp956;});_tmp4B0;}));
goto _LL0;case 263U: _LL209: _LL20A:
# 2337 "parse.y"
 yyval=(yyyvsp[0]).v;
goto _LL0;case 264U: _LL20B: _LL20C:
# 2338 "parse.y"
 yyval=(yyyvsp[0]).v;
goto _LL0;case 265U: _LL20D: _LL20E:
# 2343 "parse.y"
 yyval=Cyc_Exp_tok(({void*_tmp959=(void*)({struct Cyc_Absyn_UnresolvedMem_e_Absyn_Raw_exp_struct*_tmp4B1=_cycalloc(sizeof(*_tmp4B1));_tmp4B1->tag=37U,_tmp4B1->f1=0,_tmp4B1->f2=0;_tmp4B1;});Cyc_Absyn_new_exp(_tmp959,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 266U: _LL20F: _LL210:
# 2345 "parse.y"
 yyval=Cyc_Exp_tok(({void*_tmp95B=(void*)({struct Cyc_Absyn_UnresolvedMem_e_Absyn_Raw_exp_struct*_tmp4B2=_cycalloc(sizeof(*_tmp4B2));_tmp4B2->tag=37U,_tmp4B2->f1=0,({struct Cyc_List_List*_tmp95A=Cyc_List_imp_rev(Cyc_yyget_YY5(&(yyyvsp[1]).v));_tmp4B2->f2=_tmp95A;});_tmp4B2;});Cyc_Absyn_new_exp(_tmp95B,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 267U: _LL211: _LL212:
# 2347 "parse.y"
 yyval=Cyc_Exp_tok(({void*_tmp95D=(void*)({struct Cyc_Absyn_UnresolvedMem_e_Absyn_Raw_exp_struct*_tmp4B3=_cycalloc(sizeof(*_tmp4B3));_tmp4B3->tag=37U,_tmp4B3->f1=0,({struct Cyc_List_List*_tmp95C=Cyc_List_imp_rev(Cyc_yyget_YY5(&(yyyvsp[1]).v));_tmp4B3->f2=_tmp95C;});_tmp4B3;});Cyc_Absyn_new_exp(_tmp95D,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 268U: _LL213: _LL214: {
# 2349 "parse.y"
struct Cyc_Absyn_Vardecl*vd=({unsigned _tmp962=(unsigned)((yyyvsp[2]).l).first_line;struct _tuple0*_tmp961=({struct _tuple0*_tmp4B6=_cycalloc(sizeof(*_tmp4B6));_tmp4B6->f1=Cyc_Absyn_Loc_n,({struct _fat_ptr*_tmp95F=({struct _fat_ptr*_tmp4B5=_cycalloc(sizeof(*_tmp4B5));({struct _fat_ptr _tmp95E=Cyc_yyget_String_tok(&(yyyvsp[2]).v);*_tmp4B5=_tmp95E;});_tmp4B5;});_tmp4B6->f2=_tmp95F;});_tmp4B6;});void*_tmp960=Cyc_Absyn_uint_type;Cyc_Absyn_new_vardecl(_tmp962,_tmp961,_tmp960,
Cyc_Absyn_uint_exp(0U,(unsigned)((yyyvsp[2]).l).first_line));});
# 2352
(vd->tq).real_const=1;
yyval=Cyc_Exp_tok(({void*_tmp965=(void*)({struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct*_tmp4B4=_cycalloc(sizeof(*_tmp4B4));_tmp4B4->tag=27U,_tmp4B4->f1=vd,({struct Cyc_Absyn_Exp*_tmp964=Cyc_yyget_Exp_tok(&(yyyvsp[4]).v);_tmp4B4->f2=_tmp964;}),({struct Cyc_Absyn_Exp*_tmp963=Cyc_yyget_Exp_tok(&(yyyvsp[6]).v);_tmp4B4->f3=_tmp963;}),_tmp4B4->f4=0;_tmp4B4;});Cyc_Absyn_new_exp(_tmp965,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
# 2355
goto _LL0;}case 269U: _LL215: _LL216: {
# 2357 "parse.y"
void*t=({struct _tuple8*_tmp966=Cyc_yyget_YY37(&(yyyvsp[6]).v);Cyc_Parse_type_name_to_type(_tmp966,(unsigned)((yyyvsp[6]).l).first_line);});
yyval=Cyc_Exp_tok(({void*_tmp968=(void*)({struct Cyc_Absyn_ComprehensionNoinit_e_Absyn_Raw_exp_struct*_tmp4B7=_cycalloc(sizeof(*_tmp4B7));_tmp4B7->tag=28U,({struct Cyc_Absyn_Exp*_tmp967=Cyc_yyget_Exp_tok(&(yyyvsp[4]).v);_tmp4B7->f1=_tmp967;}),_tmp4B7->f2=t,_tmp4B7->f3=0;_tmp4B7;});Cyc_Absyn_new_exp(_tmp968,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
# 2360
goto _LL0;}case 270U: _LL217: _LL218:
# 2365 "parse.y"
 yyval=Cyc_YY5(({struct Cyc_List_List*_tmp4B9=_cycalloc(sizeof(*_tmp4B9));({struct _tuple34*_tmp96A=({struct _tuple34*_tmp4B8=_cycalloc(sizeof(*_tmp4B8));_tmp4B8->f1=0,({struct Cyc_Absyn_Exp*_tmp969=Cyc_yyget_Exp_tok(&(yyyvsp[0]).v);_tmp4B8->f2=_tmp969;});_tmp4B8;});_tmp4B9->hd=_tmp96A;}),_tmp4B9->tl=0;_tmp4B9;}));
goto _LL0;case 271U: _LL219: _LL21A:
# 2367 "parse.y"
 yyval=Cyc_YY5(({struct Cyc_List_List*_tmp4BB=_cycalloc(sizeof(*_tmp4BB));({struct _tuple34*_tmp96D=({struct _tuple34*_tmp4BA=_cycalloc(sizeof(*_tmp4BA));({struct Cyc_List_List*_tmp96C=Cyc_yyget_YY41(&(yyyvsp[0]).v);_tmp4BA->f1=_tmp96C;}),({struct Cyc_Absyn_Exp*_tmp96B=Cyc_yyget_Exp_tok(&(yyyvsp[1]).v);_tmp4BA->f2=_tmp96B;});_tmp4BA;});_tmp4BB->hd=_tmp96D;}),_tmp4BB->tl=0;_tmp4BB;}));
goto _LL0;case 272U: _LL21B: _LL21C:
# 2369 "parse.y"
 yyval=Cyc_YY5(({struct Cyc_List_List*_tmp4BD=_cycalloc(sizeof(*_tmp4BD));({struct _tuple34*_tmp970=({struct _tuple34*_tmp4BC=_cycalloc(sizeof(*_tmp4BC));_tmp4BC->f1=0,({struct Cyc_Absyn_Exp*_tmp96F=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);_tmp4BC->f2=_tmp96F;});_tmp4BC;});_tmp4BD->hd=_tmp970;}),({struct Cyc_List_List*_tmp96E=Cyc_yyget_YY5(&(yyyvsp[0]).v);_tmp4BD->tl=_tmp96E;});_tmp4BD;}));
goto _LL0;case 273U: _LL21D: _LL21E:
# 2371 "parse.y"
 yyval=Cyc_YY5(({struct Cyc_List_List*_tmp4BF=_cycalloc(sizeof(*_tmp4BF));({struct _tuple34*_tmp974=({struct _tuple34*_tmp4BE=_cycalloc(sizeof(*_tmp4BE));({struct Cyc_List_List*_tmp973=Cyc_yyget_YY41(&(yyyvsp[2]).v);_tmp4BE->f1=_tmp973;}),({struct Cyc_Absyn_Exp*_tmp972=Cyc_yyget_Exp_tok(&(yyyvsp[3]).v);_tmp4BE->f2=_tmp972;});_tmp4BE;});_tmp4BF->hd=_tmp974;}),({struct Cyc_List_List*_tmp971=Cyc_yyget_YY5(&(yyyvsp[0]).v);_tmp4BF->tl=_tmp971;});_tmp4BF;}));
goto _LL0;case 274U: _LL21F: _LL220:
# 2375 "parse.y"
 yyval=Cyc_YY41(Cyc_List_imp_rev(Cyc_yyget_YY41(&(yyyvsp[0]).v)));
goto _LL0;case 275U: _LL221: _LL222:
# 2376 "parse.y"
 yyval=Cyc_YY41(({struct Cyc_List_List*_tmp4C2=_cycalloc(sizeof(*_tmp4C2));({void*_tmp977=(void*)({struct Cyc_Absyn_FieldName_Absyn_Designator_struct*_tmp4C1=_cycalloc(sizeof(*_tmp4C1));_tmp4C1->tag=1U,({struct _fat_ptr*_tmp976=({struct _fat_ptr*_tmp4C0=_cycalloc(sizeof(*_tmp4C0));({struct _fat_ptr _tmp975=Cyc_yyget_String_tok(&(yyyvsp[0]).v);*_tmp4C0=_tmp975;});_tmp4C0;});_tmp4C1->f1=_tmp976;});_tmp4C1;});_tmp4C2->hd=_tmp977;}),_tmp4C2->tl=0;_tmp4C2;}));
goto _LL0;case 276U: _LL223: _LL224:
# 2381 "parse.y"
 yyval=Cyc_YY41(({struct Cyc_List_List*_tmp4C3=_cycalloc(sizeof(*_tmp4C3));({void*_tmp978=Cyc_yyget_YY42(&(yyyvsp[0]).v);_tmp4C3->hd=_tmp978;}),_tmp4C3->tl=0;_tmp4C3;}));
goto _LL0;case 277U: _LL225: _LL226:
# 2382 "parse.y"
 yyval=Cyc_YY41(({struct Cyc_List_List*_tmp4C4=_cycalloc(sizeof(*_tmp4C4));({void*_tmp97A=Cyc_yyget_YY42(&(yyyvsp[1]).v);_tmp4C4->hd=_tmp97A;}),({struct Cyc_List_List*_tmp979=Cyc_yyget_YY41(&(yyyvsp[0]).v);_tmp4C4->tl=_tmp979;});_tmp4C4;}));
goto _LL0;case 278U: _LL227: _LL228:
# 2386 "parse.y"
 yyval=Cyc_YY42((void*)({struct Cyc_Absyn_ArrayElement_Absyn_Designator_struct*_tmp4C5=_cycalloc(sizeof(*_tmp4C5));_tmp4C5->tag=0U,({struct Cyc_Absyn_Exp*_tmp97B=Cyc_yyget_Exp_tok(&(yyyvsp[1]).v);_tmp4C5->f1=_tmp97B;});_tmp4C5;}));
goto _LL0;case 279U: _LL229: _LL22A:
# 2387 "parse.y"
 yyval=Cyc_YY42((void*)({struct Cyc_Absyn_FieldName_Absyn_Designator_struct*_tmp4C7=_cycalloc(sizeof(*_tmp4C7));_tmp4C7->tag=1U,({struct _fat_ptr*_tmp97D=({struct _fat_ptr*_tmp4C6=_cycalloc(sizeof(*_tmp4C6));({struct _fat_ptr _tmp97C=Cyc_yyget_String_tok(&(yyyvsp[1]).v);*_tmp4C6=_tmp97C;});_tmp4C6;});_tmp4C7->f1=_tmp97D;});_tmp4C7;}));
goto _LL0;case 280U: _LL22B: _LL22C: {
# 2392 "parse.y"
struct _tuple26 _stmttmp26=Cyc_yyget_YY35(&(yyyvsp[0]).v);struct Cyc_List_List*_tmp4CA;struct Cyc_Parse_Type_specifier _tmp4C9;struct Cyc_Absyn_Tqual _tmp4C8;_LL49B: _tmp4C8=_stmttmp26.f1;_tmp4C9=_stmttmp26.f2;_tmp4CA=_stmttmp26.f3;_LL49C: {struct Cyc_Absyn_Tqual tq=_tmp4C8;struct Cyc_Parse_Type_specifier tss=_tmp4C9;struct Cyc_List_List*atts=_tmp4CA;
void*t=Cyc_Parse_speclist2typ(tss,(unsigned)((yyyvsp[0]).l).first_line);
if(atts != 0)
({void*_tmp4CB=0U;({unsigned _tmp97F=(unsigned)((yyyvsp[0]).l).first_line;struct _fat_ptr _tmp97E=({const char*_tmp4CC="ignoring attributes in type";_tag_fat(_tmp4CC,sizeof(char),28U);});Cyc_Warn_warn(_tmp97F,_tmp97E,_tag_fat(_tmp4CB,sizeof(void*),0U));});});
# 2394
yyval=
# 2396
Cyc_YY37(({struct _tuple8*_tmp4CD=_cycalloc(sizeof(*_tmp4CD));_tmp4CD->f1=0,_tmp4CD->f2=tq,_tmp4CD->f3=t;_tmp4CD;}));
# 2398
goto _LL0;}}case 281U: _LL22D: _LL22E: {
# 2399 "parse.y"
struct _tuple26 _stmttmp27=Cyc_yyget_YY35(&(yyyvsp[0]).v);struct Cyc_List_List*_tmp4D0;struct Cyc_Parse_Type_specifier _tmp4CF;struct Cyc_Absyn_Tqual _tmp4CE;_LL49E: _tmp4CE=_stmttmp27.f1;_tmp4CF=_stmttmp27.f2;_tmp4D0=_stmttmp27.f3;_LL49F: {struct Cyc_Absyn_Tqual tq=_tmp4CE;struct Cyc_Parse_Type_specifier tss=_tmp4CF;struct Cyc_List_List*atts=_tmp4D0;
void*t=Cyc_Parse_speclist2typ(tss,(unsigned)((yyyvsp[0]).l).first_line);
struct Cyc_List_List*tms=(Cyc_yyget_YY30(&(yyyvsp[1]).v)).tms;
struct _tuple14 t_info=Cyc_Parse_apply_tms(tq,t,atts,tms);
if(t_info.f3 != 0)
# 2405
({void*_tmp4D1=0U;({unsigned _tmp981=(unsigned)((yyyvsp[1]).l).first_line;struct _fat_ptr _tmp980=({const char*_tmp4D2="bad type params, ignoring";_tag_fat(_tmp4D2,sizeof(char),26U);});Cyc_Warn_warn(_tmp981,_tmp980,_tag_fat(_tmp4D1,sizeof(void*),0U));});});
# 2403
if(t_info.f4 != 0)
# 2407
({void*_tmp4D3=0U;({unsigned _tmp983=(unsigned)((yyyvsp[1]).l).first_line;struct _fat_ptr _tmp982=({const char*_tmp4D4="bad specifiers, ignoring";_tag_fat(_tmp4D4,sizeof(char),25U);});Cyc_Warn_warn(_tmp983,_tmp982,_tag_fat(_tmp4D3,sizeof(void*),0U));});});
# 2403
yyval=
# 2408
Cyc_YY37(({struct _tuple8*_tmp4D5=_cycalloc(sizeof(*_tmp4D5));_tmp4D5->f1=0,_tmp4D5->f2=t_info.f1,_tmp4D5->f3=t_info.f2;_tmp4D5;}));
# 2410
goto _LL0;}}case 282U: _LL22F: _LL230:
# 2413 "parse.y"
 yyval=Cyc_YY44(({struct _tuple8*_tmp984=Cyc_yyget_YY37(&(yyyvsp[0]).v);Cyc_Parse_type_name_to_type(_tmp984,(unsigned)((yyyvsp[0]).l).first_line);}));
goto _LL0;case 283U: _LL231: _LL232:
# 2414 "parse.y"
 yyval=Cyc_YY44(Cyc_Absyn_join_eff(0));
goto _LL0;case 284U: _LL233: _LL234:
# 2415 "parse.y"
 yyval=Cyc_YY44(Cyc_Absyn_join_eff(Cyc_yyget_YY40(&(yyyvsp[1]).v)));
goto _LL0;case 285U: _LL235: _LL236:
# 2416 "parse.y"
 yyval=Cyc_YY44(Cyc_Absyn_regionsof_eff(Cyc_yyget_YY44(&(yyyvsp[2]).v)));
goto _LL0;case 286U: _LL237: _LL238:
# 2417 "parse.y"
 yyval=Cyc_YY44(Cyc_Absyn_join_eff(({struct Cyc_List_List*_tmp4D6=_cycalloc(sizeof(*_tmp4D6));({void*_tmp986=Cyc_yyget_YY44(&(yyyvsp[0]).v);_tmp4D6->hd=_tmp986;}),({struct Cyc_List_List*_tmp985=Cyc_yyget_YY40(&(yyyvsp[2]).v);_tmp4D6->tl=_tmp985;});_tmp4D6;})));
goto _LL0;case 287U: _LL239: _LL23A:
# 2423 "parse.y"
 yyval=Cyc_YY40(({struct Cyc_List_List*_tmp4D7=_cycalloc(sizeof(*_tmp4D7));({void*_tmp987=Cyc_yyget_YY44(&(yyyvsp[0]).v);_tmp4D7->hd=_tmp987;}),_tmp4D7->tl=0;_tmp4D7;}));
goto _LL0;case 288U: _LL23B: _LL23C:
# 2424 "parse.y"
 yyval=Cyc_YY40(({struct Cyc_List_List*_tmp4D8=_cycalloc(sizeof(*_tmp4D8));({void*_tmp989=Cyc_yyget_YY44(&(yyyvsp[2]).v);_tmp4D8->hd=_tmp989;}),({struct Cyc_List_List*_tmp988=Cyc_yyget_YY40(&(yyyvsp[0]).v);_tmp4D8->tl=_tmp988;});_tmp4D8;}));
goto _LL0;case 289U: _LL23D: _LL23E:
# 2429 "parse.y"
 yyval=Cyc_YY30(({struct Cyc_Parse_Abstractdeclarator _tmp66F;({struct Cyc_List_List*_tmp98A=Cyc_yyget_YY26(&(yyyvsp[0]).v);_tmp66F.tms=_tmp98A;});_tmp66F;}));
goto _LL0;case 290U: _LL23F: _LL240:
# 2431 "parse.y"
 yyval=(yyyvsp[0]).v;
goto _LL0;case 291U: _LL241: _LL242:
# 2433 "parse.y"
 yyval=Cyc_YY30(({struct Cyc_Parse_Abstractdeclarator _tmp670;({struct Cyc_List_List*_tmp98C=({struct Cyc_List_List*_tmp98B=Cyc_yyget_YY26(&(yyyvsp[0]).v);Cyc_List_imp_append(_tmp98B,(Cyc_yyget_YY30(&(yyyvsp[1]).v)).tms);});_tmp670.tms=_tmp98C;});_tmp670;}));
goto _LL0;case 292U: _LL243: _LL244:
# 2438 "parse.y"
 yyval=(yyyvsp[1]).v;
goto _LL0;case 293U: _LL245: _LL246:
# 2440 "parse.y"
 yyval=Cyc_YY30(({struct Cyc_Parse_Abstractdeclarator _tmp671;({struct Cyc_List_List*_tmp98F=({struct Cyc_List_List*_tmp4DA=_region_malloc(yyr,sizeof(*_tmp4DA));({void*_tmp98E=(void*)({struct Cyc_Absyn_Carray_mod_Absyn_Type_modifier_struct*_tmp4D9=_region_malloc(yyr,sizeof(*_tmp4D9));_tmp4D9->tag=0U,({void*_tmp98D=Cyc_yyget_YY51(&(yyyvsp[2]).v);_tmp4D9->f1=_tmp98D;}),_tmp4D9->f2=(unsigned)((yyyvsp[2]).l).first_line;_tmp4D9;});_tmp4DA->hd=_tmp98E;}),_tmp4DA->tl=0;_tmp4DA;});_tmp671.tms=_tmp98F;});_tmp671;}));
goto _LL0;case 294U: _LL247: _LL248:
# 2442 "parse.y"
 yyval=Cyc_YY30(({struct Cyc_Parse_Abstractdeclarator _tmp672;({struct Cyc_List_List*_tmp993=({struct Cyc_List_List*_tmp4DC=_region_malloc(yyr,sizeof(*_tmp4DC));({void*_tmp992=(void*)({struct Cyc_Absyn_Carray_mod_Absyn_Type_modifier_struct*_tmp4DB=_region_malloc(yyr,sizeof(*_tmp4DB));_tmp4DB->tag=0U,({void*_tmp991=Cyc_yyget_YY51(&(yyyvsp[3]).v);_tmp4DB->f1=_tmp991;}),_tmp4DB->f2=(unsigned)((yyyvsp[3]).l).first_line;_tmp4DB;});_tmp4DC->hd=_tmp992;}),({struct Cyc_List_List*_tmp990=(Cyc_yyget_YY30(&(yyyvsp[0]).v)).tms;_tmp4DC->tl=_tmp990;});_tmp4DC;});_tmp672.tms=_tmp993;});_tmp672;}));
goto _LL0;case 295U: _LL249: _LL24A:
# 2444 "parse.y"
 yyval=Cyc_YY30(({struct Cyc_Parse_Abstractdeclarator _tmp673;({struct Cyc_List_List*_tmp997=({struct Cyc_List_List*_tmp4DE=_region_malloc(yyr,sizeof(*_tmp4DE));({void*_tmp996=(void*)({struct Cyc_Absyn_ConstArray_mod_Absyn_Type_modifier_struct*_tmp4DD=_region_malloc(yyr,sizeof(*_tmp4DD));_tmp4DD->tag=1U,({struct Cyc_Absyn_Exp*_tmp995=Cyc_yyget_Exp_tok(&(yyyvsp[1]).v);_tmp4DD->f1=_tmp995;}),({void*_tmp994=Cyc_yyget_YY51(&(yyyvsp[3]).v);_tmp4DD->f2=_tmp994;}),_tmp4DD->f3=(unsigned)((yyyvsp[3]).l).first_line;_tmp4DD;});_tmp4DE->hd=_tmp996;}),_tmp4DE->tl=0;_tmp4DE;});_tmp673.tms=_tmp997;});_tmp673;}));
goto _LL0;case 296U: _LL24B: _LL24C:
# 2446 "parse.y"
 yyval=Cyc_YY30(({struct Cyc_Parse_Abstractdeclarator _tmp674;({struct Cyc_List_List*_tmp99C=({struct Cyc_List_List*_tmp4E0=_region_malloc(yyr,sizeof(*_tmp4E0));({void*_tmp99B=(void*)({struct Cyc_Absyn_ConstArray_mod_Absyn_Type_modifier_struct*_tmp4DF=_region_malloc(yyr,sizeof(*_tmp4DF));_tmp4DF->tag=1U,({struct Cyc_Absyn_Exp*_tmp99A=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);_tmp4DF->f1=_tmp99A;}),({void*_tmp999=Cyc_yyget_YY51(&(yyyvsp[4]).v);_tmp4DF->f2=_tmp999;}),_tmp4DF->f3=(unsigned)((yyyvsp[4]).l).first_line;_tmp4DF;});_tmp4E0->hd=_tmp99B;}),({
struct Cyc_List_List*_tmp998=(Cyc_yyget_YY30(&(yyyvsp[0]).v)).tms;_tmp4E0->tl=_tmp998;});_tmp4E0;});
# 2446
_tmp674.tms=_tmp99C;});_tmp674;}));
# 2449
goto _LL0;case 297U: _LL24D: _LL24E: {
# 2452
struct Cyc_List_List*ieff=({unsigned _tmp99D=(unsigned)((yyyvsp[0]).l).first_line;Cyc_Absyn_parse_rgneffects(_tmp99D,Cyc_yyget_YY49(&(yyyvsp[6]).v));});
struct Cyc_List_List*oeff=({unsigned _tmp99E=(unsigned)((yyyvsp[0]).l).first_line;Cyc_Absyn_parse_rgneffects(_tmp99E,Cyc_yyget_YY49(&(yyyvsp[7]).v));});
# 2455
yyval=Cyc_YY30(({struct Cyc_Parse_Abstractdeclarator _tmp675;({struct Cyc_List_List*_tmp9A5=({struct Cyc_List_List*_tmp4E3=_region_malloc(yyr,sizeof(*_tmp4E3));({void*_tmp9A4=(void*)({struct Cyc_Absyn_Function_mod_Absyn_Type_modifier_struct*_tmp4E2=_region_malloc(yyr,sizeof(*_tmp4E2));_tmp4E2->tag=3U,({void*_tmp9A3=(void*)({struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct*_tmp4E1=_region_malloc(yyr,sizeof(*_tmp4E1));_tmp4E1->tag=1U,_tmp4E1->f1=0,_tmp4E1->f2=0,_tmp4E1->f3=0,({void*_tmp9A2=Cyc_yyget_YY49(&(yyyvsp[1]).v);_tmp4E1->f4=_tmp9A2;}),({struct Cyc_List_List*_tmp9A1=Cyc_yyget_YY50(&(yyyvsp[2]).v);_tmp4E1->f5=_tmp9A1;}),({struct Cyc_Absyn_Exp*_tmp9A0=Cyc_yyget_YY57(&(yyyvsp[4]).v);_tmp4E1->f6=_tmp9A0;}),({struct Cyc_Absyn_Exp*_tmp99F=Cyc_yyget_YY57(&(yyyvsp[5]).v);_tmp4E1->f7=_tmp99F;}),_tmp4E1->f8=ieff,_tmp4E1->f9=oeff;_tmp4E1;});_tmp4E2->f1=_tmp9A3;});_tmp4E2;});_tmp4E3->hd=_tmp9A4;}),_tmp4E3->tl=0;_tmp4E3;});_tmp675.tms=_tmp9A5;});_tmp675;}));
# 2457
goto _LL0;}case 298U: _LL24F: _LL250: {
# 2459
struct _tuple27*_stmttmp28=Cyc_yyget_YY39(&(yyyvsp[1]).v);struct Cyc_List_List*_tmp4E8;void*_tmp4E7;struct Cyc_Absyn_VarargInfo*_tmp4E6;int _tmp4E5;struct Cyc_List_List*_tmp4E4;_LL4A1: _tmp4E4=_stmttmp28->f1;_tmp4E5=_stmttmp28->f2;_tmp4E6=_stmttmp28->f3;_tmp4E7=_stmttmp28->f4;_tmp4E8=_stmttmp28->f5;_LL4A2: {struct Cyc_List_List*lis=_tmp4E4;int b=_tmp4E5;struct Cyc_Absyn_VarargInfo*c=_tmp4E6;void*eff=_tmp4E7;struct Cyc_List_List*po=_tmp4E8;
struct Cyc_List_List*ieff=({unsigned _tmp9A6=(unsigned)((yyyvsp[0]).l).first_line;Cyc_Absyn_parse_rgneffects(_tmp9A6,Cyc_yyget_YY49(&(yyyvsp[5]).v));});
struct Cyc_List_List*oeff=({unsigned _tmp9A7=(unsigned)((yyyvsp[0]).l).first_line;Cyc_Absyn_parse_rgneffects(_tmp9A7,Cyc_yyget_YY49(&(yyyvsp[6]).v));});
# 2463
yyval=Cyc_YY30(({struct Cyc_Parse_Abstractdeclarator _tmp676;({struct Cyc_List_List*_tmp9AC=({struct Cyc_List_List*_tmp4EB=_region_malloc(yyr,sizeof(*_tmp4EB));({void*_tmp9AB=(void*)({struct Cyc_Absyn_Function_mod_Absyn_Type_modifier_struct*_tmp4EA=_region_malloc(yyr,sizeof(*_tmp4EA));_tmp4EA->tag=3U,({void*_tmp9AA=(void*)({struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct*_tmp4E9=_region_malloc(yyr,sizeof(*_tmp4E9));_tmp4E9->tag=1U,_tmp4E9->f1=lis,_tmp4E9->f2=b,_tmp4E9->f3=c,_tmp4E9->f4=eff,_tmp4E9->f5=po,({struct Cyc_Absyn_Exp*_tmp9A9=Cyc_yyget_YY57(&(yyyvsp[3]).v);_tmp4E9->f6=_tmp9A9;}),({struct Cyc_Absyn_Exp*_tmp9A8=Cyc_yyget_YY57(&(yyyvsp[4]).v);_tmp4E9->f7=_tmp9A8;}),_tmp4E9->f8=ieff,_tmp4E9->f9=oeff;_tmp4E9;});_tmp4EA->f1=_tmp9AA;});_tmp4EA;});_tmp4EB->hd=_tmp9AB;}),_tmp4EB->tl=0;_tmp4EB;});_tmp676.tms=_tmp9AC;});_tmp676;}));
# 2465
goto _LL0;}}case 299U: _LL251: _LL252: {
# 2468
struct Cyc_List_List*ieff=({unsigned _tmp9AD=(unsigned)((yyyvsp[0]).l).first_line;Cyc_Absyn_parse_rgneffects(_tmp9AD,Cyc_yyget_YY49(&(yyyvsp[7]).v));});
struct Cyc_List_List*oeff=({unsigned _tmp9AE=(unsigned)((yyyvsp[0]).l).first_line;Cyc_Absyn_parse_rgneffects(_tmp9AE,Cyc_yyget_YY49(&(yyyvsp[8]).v));});
# 2471
yyval=Cyc_YY30(({struct Cyc_Parse_Abstractdeclarator _tmp677;({struct Cyc_List_List*_tmp9B6=({struct Cyc_List_List*_tmp4EE=_region_malloc(yyr,sizeof(*_tmp4EE));({void*_tmp9B5=(void*)({struct Cyc_Absyn_Function_mod_Absyn_Type_modifier_struct*_tmp4ED=_region_malloc(yyr,sizeof(*_tmp4ED));_tmp4ED->tag=3U,({void*_tmp9B4=(void*)({struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct*_tmp4EC=_region_malloc(yyr,sizeof(*_tmp4EC));_tmp4EC->tag=1U,_tmp4EC->f1=0,_tmp4EC->f2=0,_tmp4EC->f3=0,({void*_tmp9B3=Cyc_yyget_YY49(&(yyyvsp[2]).v);_tmp4EC->f4=_tmp9B3;}),({struct Cyc_List_List*_tmp9B2=Cyc_yyget_YY50(&(yyyvsp[3]).v);_tmp4EC->f5=_tmp9B2;}),({struct Cyc_Absyn_Exp*_tmp9B1=Cyc_yyget_YY57(&(yyyvsp[5]).v);_tmp4EC->f6=_tmp9B1;}),({struct Cyc_Absyn_Exp*_tmp9B0=Cyc_yyget_YY57(&(yyyvsp[6]).v);_tmp4EC->f7=_tmp9B0;}),_tmp4EC->f8=ieff,_tmp4EC->f9=oeff;_tmp4EC;});_tmp4ED->f1=_tmp9B4;});_tmp4ED;});_tmp4EE->hd=_tmp9B5;}),({
struct Cyc_List_List*_tmp9AF=(Cyc_yyget_YY30(&(yyyvsp[0]).v)).tms;_tmp4EE->tl=_tmp9AF;});_tmp4EE;});
# 2471
_tmp677.tms=_tmp9B6;});_tmp677;}));
# 2474
goto _LL0;}case 300U: _LL253: _LL254: {
# 2476
struct _tuple27*_stmttmp29=Cyc_yyget_YY39(&(yyyvsp[2]).v);struct Cyc_List_List*_tmp4F3;void*_tmp4F2;struct Cyc_Absyn_VarargInfo*_tmp4F1;int _tmp4F0;struct Cyc_List_List*_tmp4EF;_LL4A4: _tmp4EF=_stmttmp29->f1;_tmp4F0=_stmttmp29->f2;_tmp4F1=_stmttmp29->f3;_tmp4F2=_stmttmp29->f4;_tmp4F3=_stmttmp29->f5;_LL4A5: {struct Cyc_List_List*lis=_tmp4EF;int b=_tmp4F0;struct Cyc_Absyn_VarargInfo*c=_tmp4F1;void*eff=_tmp4F2;struct Cyc_List_List*po=_tmp4F3;
struct Cyc_List_List*ieff=({unsigned _tmp9B7=(unsigned)((yyyvsp[0]).l).first_line;Cyc_Absyn_parse_rgneffects(_tmp9B7,Cyc_yyget_YY49(&(yyyvsp[6]).v));});
struct Cyc_List_List*oeff=({unsigned _tmp9B8=(unsigned)((yyyvsp[0]).l).first_line;Cyc_Absyn_parse_rgneffects(_tmp9B8,Cyc_yyget_YY49(&(yyyvsp[7]).v));});
# 2480
yyval=Cyc_YY30(({struct Cyc_Parse_Abstractdeclarator _tmp678;({struct Cyc_List_List*_tmp9BE=({struct Cyc_List_List*_tmp4F6=_region_malloc(yyr,sizeof(*_tmp4F6));({void*_tmp9BD=(void*)({struct Cyc_Absyn_Function_mod_Absyn_Type_modifier_struct*_tmp4F5=_region_malloc(yyr,sizeof(*_tmp4F5));_tmp4F5->tag=3U,({void*_tmp9BC=(void*)({struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct*_tmp4F4=_region_malloc(yyr,sizeof(*_tmp4F4));_tmp4F4->tag=1U,_tmp4F4->f1=lis,_tmp4F4->f2=b,_tmp4F4->f3=c,_tmp4F4->f4=eff,_tmp4F4->f5=po,({
struct Cyc_Absyn_Exp*_tmp9BB=Cyc_yyget_YY57(&(yyyvsp[4]).v);_tmp4F4->f6=_tmp9BB;}),({struct Cyc_Absyn_Exp*_tmp9BA=Cyc_yyget_YY57(&(yyyvsp[5]).v);_tmp4F4->f7=_tmp9BA;}),_tmp4F4->f8=ieff,_tmp4F4->f9=oeff;_tmp4F4;});
# 2480
_tmp4F5->f1=_tmp9BC;});_tmp4F5;});_tmp4F6->hd=_tmp9BD;}),({
struct Cyc_List_List*_tmp9B9=(Cyc_yyget_YY30(&(yyyvsp[0]).v)).tms;_tmp4F6->tl=_tmp9B9;});_tmp4F6;});
# 2480
_tmp678.tms=_tmp9BE;});_tmp678;}));
# 2483
goto _LL0;}}case 301U: _LL255: _LL256: {
# 2485
struct Cyc_List_List*_tmp4F7=({struct Cyc_List_List*(*_tmp9C0)(struct Cyc_Absyn_Tvar*(*f)(unsigned,void*),unsigned env,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp4FA)(struct Cyc_Absyn_Tvar*(*f)(unsigned,void*),unsigned env,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_Absyn_Tvar*(*f)(unsigned,void*),unsigned env,struct Cyc_List_List*x))Cyc_List_map_c;_tmp4FA;});unsigned _tmp9BF=(unsigned)({yylloc.first_line=((yyyvsp[1]).l).first_line;((yyyvsp[1]).l).first_line;});_tmp9C0(Cyc_Parse_typ2tvar,_tmp9BF,Cyc_List_imp_rev(Cyc_yyget_YY40(&(yyyvsp[2]).v)));});struct Cyc_List_List*ts=_tmp4F7;
yyval=Cyc_YY30(({struct Cyc_Parse_Abstractdeclarator _tmp679;({struct Cyc_List_List*_tmp9C4=({struct Cyc_List_List*_tmp4F9=_region_malloc(yyr,sizeof(*_tmp4F9));({void*_tmp9C3=(void*)({struct Cyc_Absyn_TypeParams_mod_Absyn_Type_modifier_struct*_tmp4F8=_region_malloc(yyr,sizeof(*_tmp4F8));_tmp4F8->tag=4U,_tmp4F8->f1=ts,({unsigned _tmp9C2=(unsigned)({yylloc.first_line=((yyyvsp[1]).l).first_line;((yyyvsp[1]).l).first_line;});_tmp4F8->f2=_tmp9C2;}),_tmp4F8->f3=0;_tmp4F8;});_tmp4F9->hd=_tmp9C3;}),({
struct Cyc_List_List*_tmp9C1=(Cyc_yyget_YY30(&(yyyvsp[0]).v)).tms;_tmp4F9->tl=_tmp9C1;});_tmp4F9;});
# 2486
_tmp679.tms=_tmp9C4;});_tmp679;}));
# 2489
goto _LL0;}case 302U: _LL257: _LL258:
# 2490 "parse.y"
 yyval=Cyc_YY30(({struct Cyc_Parse_Abstractdeclarator _tmp67A;({struct Cyc_List_List*_tmp9C8=({struct Cyc_List_List*_tmp4FC=_region_malloc(yyr,sizeof(*_tmp4FC));({void*_tmp9C7=(void*)({struct Cyc_Absyn_Attributes_mod_Absyn_Type_modifier_struct*_tmp4FB=_region_malloc(yyr,sizeof(*_tmp4FB));_tmp4FB->tag=5U,_tmp4FB->f1=(unsigned)((yyyvsp[1]).l).first_line,({struct Cyc_List_List*_tmp9C6=Cyc_yyget_YY45(&(yyyvsp[1]).v);_tmp4FB->f2=_tmp9C6;});_tmp4FB;});_tmp4FC->hd=_tmp9C7;}),({struct Cyc_List_List*_tmp9C5=(Cyc_yyget_YY30(&(yyyvsp[0]).v)).tms;_tmp4FC->tl=_tmp9C5;});_tmp4FC;});_tmp67A.tms=_tmp9C8;});_tmp67A;}));
# 2492
goto _LL0;case 303U: _LL259: _LL25A:
# 2496 "parse.y"
 yyval=(yyyvsp[0]).v;
goto _LL0;case 304U: _LL25B: _LL25C:
# 2497 "parse.y"
 yyval=(yyyvsp[0]).v;
goto _LL0;case 305U: _LL25D: _LL25E:
# 2498 "parse.y"
 yyval=(yyyvsp[0]).v;
goto _LL0;case 306U: _LL25F: _LL260:
# 2499 "parse.y"
 yyval=(yyyvsp[0]).v;
goto _LL0;case 307U: _LL261: _LL262:
# 2500 "parse.y"
 yyval=(yyyvsp[0]).v;
goto _LL0;case 308U: _LL263: _LL264:
# 2501 "parse.y"
 yyval=(yyyvsp[0]).v;
goto _LL0;case 309U: _LL265: _LL266:
# 2507 "parse.y"
 yyval=Cyc_Stmt_tok(({void*_tmp9CC=(void*)({struct Cyc_Absyn_Label_s_Absyn_Raw_stmt_struct*_tmp4FE=_cycalloc(sizeof(*_tmp4FE));_tmp4FE->tag=13U,({struct _fat_ptr*_tmp9CB=({struct _fat_ptr*_tmp4FD=_cycalloc(sizeof(*_tmp4FD));({struct _fat_ptr _tmp9CA=Cyc_yyget_String_tok(&(yyyvsp[0]).v);*_tmp4FD=_tmp9CA;});_tmp4FD;});_tmp4FE->f1=_tmp9CB;}),({struct Cyc_Absyn_Stmt*_tmp9C9=Cyc_yyget_Stmt_tok(&(yyyvsp[2]).v);_tmp4FE->f2=_tmp9C9;});_tmp4FE;});Cyc_Absyn_new_stmt(_tmp9CC,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 310U: _LL267: _LL268:
# 2511 "parse.y"
 yyval=Cyc_Stmt_tok(Cyc_Absyn_skip_stmt((unsigned)((yyyvsp[0]).l).first_line));
goto _LL0;case 311U: _LL269: _LL26A:
# 2512 "parse.y"
 yyval=Cyc_Stmt_tok(({struct Cyc_Absyn_Exp*_tmp9CD=Cyc_yyget_Exp_tok(&(yyyvsp[0]).v);Cyc_Absyn_exp_stmt(_tmp9CD,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 312U: _LL26B: _LL26C:
# 2517 "parse.y"
 yyval=Cyc_Stmt_tok(Cyc_Absyn_skip_stmt((unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;})));
goto _LL0;case 313U: _LL26D: _LL26E:
# 2518 "parse.y"
 yyval=(yyyvsp[1]).v;
goto _LL0;case 314U: _LL26F: _LL270:
# 2523 "parse.y"
 yyval=Cyc_Stmt_tok(({struct Cyc_List_List*_tmp9CE=Cyc_yyget_YY16(&(yyyvsp[0]).v);Cyc_Parse_flatten_declarations(_tmp9CE,Cyc_Absyn_skip_stmt((unsigned)((yyyvsp[0]).l).first_line));}));
goto _LL0;case 315U: _LL271: _LL272:
# 2524 "parse.y"
 yyval=Cyc_Stmt_tok(({struct Cyc_List_List*_tmp9CF=Cyc_yyget_YY16(&(yyyvsp[0]).v);Cyc_Parse_flatten_declarations(_tmp9CF,Cyc_yyget_Stmt_tok(&(yyyvsp[1]).v));}));
goto _LL0;case 316U: _LL273: _LL274:
# 2525 "parse.y"
 yyval=Cyc_Stmt_tok(({void*_tmp9D4=(void*)({struct Cyc_Absyn_Label_s_Absyn_Raw_stmt_struct*_tmp500=_cycalloc(sizeof(*_tmp500));_tmp500->tag=13U,({struct _fat_ptr*_tmp9D3=({struct _fat_ptr*_tmp4FF=_cycalloc(sizeof(*_tmp4FF));({struct _fat_ptr _tmp9D2=Cyc_yyget_String_tok(&(yyyvsp[0]).v);*_tmp4FF=_tmp9D2;});_tmp4FF;});_tmp500->f1=_tmp9D3;}),({struct Cyc_Absyn_Stmt*_tmp9D1=({struct Cyc_List_List*_tmp9D0=Cyc_yyget_YY16(&(yyyvsp[2]).v);Cyc_Parse_flatten_declarations(_tmp9D0,Cyc_Absyn_skip_stmt(0U));});_tmp500->f2=_tmp9D1;});_tmp500;});Cyc_Absyn_new_stmt(_tmp9D4,(unsigned)((yyyvsp[0]).l).first_line);}));
# 2527
goto _LL0;case 317U: _LL275: _LL276:
# 2527 "parse.y"
 yyval=Cyc_Stmt_tok(({void*_tmp9D9=(void*)({struct Cyc_Absyn_Label_s_Absyn_Raw_stmt_struct*_tmp502=_cycalloc(sizeof(*_tmp502));_tmp502->tag=13U,({struct _fat_ptr*_tmp9D8=({struct _fat_ptr*_tmp501=_cycalloc(sizeof(*_tmp501));({struct _fat_ptr _tmp9D7=Cyc_yyget_String_tok(&(yyyvsp[0]).v);*_tmp501=_tmp9D7;});_tmp501;});_tmp502->f1=_tmp9D8;}),({struct Cyc_Absyn_Stmt*_tmp9D6=({struct Cyc_List_List*_tmp9D5=Cyc_yyget_YY16(&(yyyvsp[2]).v);Cyc_Parse_flatten_declarations(_tmp9D5,Cyc_yyget_Stmt_tok(&(yyyvsp[3]).v));});_tmp502->f2=_tmp9D6;});_tmp502;});Cyc_Absyn_new_stmt(_tmp9D9,(unsigned)((yyyvsp[0]).l).first_line);}));
# 2529
goto _LL0;case 318U: _LL277: _LL278:
# 2529 "parse.y"
 yyval=(yyyvsp[0]).v;
goto _LL0;case 319U: _LL279: _LL27A:
# 2530 "parse.y"
 yyval=Cyc_Stmt_tok(({struct Cyc_Absyn_Stmt*_tmp9DB=Cyc_yyget_Stmt_tok(&(yyyvsp[0]).v);struct Cyc_Absyn_Stmt*_tmp9DA=Cyc_yyget_Stmt_tok(&(yyyvsp[1]).v);Cyc_Absyn_seq_stmt(_tmp9DB,_tmp9DA,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 320U: _LL27B: _LL27C:
# 2531 "parse.y"
 yyval=Cyc_Stmt_tok(({struct Cyc_Absyn_Decl*_tmp9DE=({void*_tmp9DD=(void*)({struct Cyc_Absyn_Fn_d_Absyn_Raw_decl_struct*_tmp503=_cycalloc(sizeof(*_tmp503));_tmp503->tag=1U,({struct Cyc_Absyn_Fndecl*_tmp9DC=Cyc_yyget_YY15(&(yyyvsp[0]).v);_tmp503->f1=_tmp9DC;});_tmp503;});Cyc_Absyn_new_decl(_tmp9DD,(unsigned)((yyyvsp[0]).l).first_line);});Cyc_Parse_flatten_decl(_tmp9DE,
Cyc_Absyn_skip_stmt(0U));}));
goto _LL0;case 321U: _LL27D: _LL27E:
# 2534 "parse.y"
 yyval=Cyc_Stmt_tok(({struct Cyc_Absyn_Decl*_tmp9E1=({void*_tmp9E0=(void*)({struct Cyc_Absyn_Fn_d_Absyn_Raw_decl_struct*_tmp504=_cycalloc(sizeof(*_tmp504));_tmp504->tag=1U,({struct Cyc_Absyn_Fndecl*_tmp9DF=Cyc_yyget_YY15(&(yyyvsp[0]).v);_tmp504->f1=_tmp9DF;});_tmp504;});Cyc_Absyn_new_decl(_tmp9E0,(unsigned)((yyyvsp[0]).l).first_line);});Cyc_Parse_flatten_decl(_tmp9E1,Cyc_yyget_Stmt_tok(&(yyyvsp[1]).v));}));
goto _LL0;case 322U: _LL27F: _LL280:
# 2539 "parse.y"
 yyval=Cyc_Stmt_tok(({struct Cyc_Absyn_Exp*_tmp9E4=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);struct Cyc_Absyn_Stmt*_tmp9E3=Cyc_yyget_Stmt_tok(&(yyyvsp[4]).v);struct Cyc_Absyn_Stmt*_tmp9E2=Cyc_Absyn_skip_stmt(0U);Cyc_Absyn_ifthenelse_stmt(_tmp9E4,_tmp9E3,_tmp9E2,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 323U: _LL281: _LL282:
# 2541 "parse.y"
 yyval=Cyc_Stmt_tok(({struct Cyc_Absyn_Exp*_tmp9E7=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);struct Cyc_Absyn_Stmt*_tmp9E6=Cyc_yyget_Stmt_tok(&(yyyvsp[4]).v);struct Cyc_Absyn_Stmt*_tmp9E5=Cyc_yyget_Stmt_tok(&(yyyvsp[6]).v);Cyc_Absyn_ifthenelse_stmt(_tmp9E7,_tmp9E6,_tmp9E5,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 324U: _LL283: _LL284:
# 2547 "parse.y"
 yyval=Cyc_Stmt_tok(({struct Cyc_Absyn_Exp*_tmp9E9=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);struct Cyc_List_List*_tmp9E8=Cyc_yyget_YY8(&(yyyvsp[5]).v);Cyc_Absyn_switch_stmt(_tmp9E9,_tmp9E8,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 325U: _LL285: _LL286: {
# 2550
struct Cyc_Absyn_Exp*e=({struct _tuple0*_tmp9EA=Cyc_yyget_QualId_tok(&(yyyvsp[1]).v);Cyc_Absyn_unknownid_exp(_tmp9EA,(unsigned)((yyyvsp[1]).l).first_line);});
yyval=Cyc_Stmt_tok(({struct Cyc_Absyn_Exp*_tmp9EC=e;struct Cyc_List_List*_tmp9EB=Cyc_yyget_YY8(&(yyyvsp[3]).v);Cyc_Absyn_switch_stmt(_tmp9EC,_tmp9EB,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;}case 326U: _LL287: _LL288: {
# 2554
struct Cyc_Absyn_Exp*_tmp505=({struct Cyc_List_List*_tmp9ED=Cyc_yyget_YY4(&(yyyvsp[3]).v);Cyc_Absyn_tuple_exp(_tmp9ED,(unsigned)({yylloc.first_line=((yyyvsp[1]).l).first_line;((yyyvsp[1]).l).first_line;}));});struct Cyc_Absyn_Exp*e=_tmp505;
yyval=Cyc_Stmt_tok(({struct Cyc_Absyn_Exp*_tmp9EF=e;struct Cyc_List_List*_tmp9EE=Cyc_yyget_YY8(&(yyyvsp[6]).v);Cyc_Absyn_switch_stmt(_tmp9EF,_tmp9EE,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
# 2557
goto _LL0;}case 327U: _LL289: _LL28A:
# 2560 "parse.y"
 yyval=Cyc_Stmt_tok(({struct Cyc_Absyn_Stmt*_tmp9F1=Cyc_yyget_Stmt_tok(&(yyyvsp[1]).v);struct Cyc_List_List*_tmp9F0=Cyc_yyget_YY8(&(yyyvsp[4]).v);Cyc_Absyn_trycatch_stmt(_tmp9F1,_tmp9F0,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 328U: _LL28B: _LL28C:
# 2574 "parse.y"
 yyval=Cyc_YY8(0);
goto _LL0;case 329U: _LL28D: _LL28E:
# 2578 "parse.y"
 yyval=Cyc_YY8(({struct Cyc_List_List*_tmp507=_cycalloc(sizeof(*_tmp507));({struct Cyc_Absyn_Switch_clause*_tmp9F6=({struct Cyc_Absyn_Switch_clause*_tmp506=_cycalloc(sizeof(*_tmp506));({struct Cyc_Absyn_Pat*_tmp9F5=Cyc_Absyn_new_pat((void*)& Cyc_Absyn_Wild_p_val,(unsigned)((yyyvsp[0]).l).first_line);_tmp506->pattern=_tmp9F5;}),_tmp506->pat_vars=0,_tmp506->where_clause=0,({
struct Cyc_Absyn_Stmt*_tmp9F4=Cyc_yyget_Stmt_tok(&(yyyvsp[2]).v);_tmp506->body=_tmp9F4;}),({unsigned _tmp9F3=(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;});_tmp506->loc=_tmp9F3;});_tmp506;});
# 2578
_tmp507->hd=_tmp9F6;}),({
# 2580
struct Cyc_List_List*_tmp9F2=Cyc_yyget_YY8(&(yyyvsp[3]).v);_tmp507->tl=_tmp9F2;});_tmp507;}));
goto _LL0;case 330U: _LL28F: _LL290:
# 2582 "parse.y"
 yyval=Cyc_YY8(({struct Cyc_List_List*_tmp509=_cycalloc(sizeof(*_tmp509));({struct Cyc_Absyn_Switch_clause*_tmp9FB=({struct Cyc_Absyn_Switch_clause*_tmp508=_cycalloc(sizeof(*_tmp508));({struct Cyc_Absyn_Pat*_tmp9FA=Cyc_yyget_YY9(&(yyyvsp[1]).v);_tmp508->pattern=_tmp9FA;}),_tmp508->pat_vars=0,_tmp508->where_clause=0,({
struct Cyc_Absyn_Stmt*_tmp9F9=Cyc_Absyn_fallthru_stmt(0,(unsigned)((yyyvsp[2]).l).first_line);_tmp508->body=_tmp9F9;}),({
unsigned _tmp9F8=(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;});_tmp508->loc=_tmp9F8;});_tmp508;});
# 2582
_tmp509->hd=_tmp9FB;}),({
# 2584
struct Cyc_List_List*_tmp9F7=Cyc_yyget_YY8(&(yyyvsp[3]).v);_tmp509->tl=_tmp9F7;});_tmp509;}));
goto _LL0;case 331U: _LL291: _LL292:
# 2586 "parse.y"
 yyval=Cyc_YY8(({struct Cyc_List_List*_tmp50B=_cycalloc(sizeof(*_tmp50B));({struct Cyc_Absyn_Switch_clause*_tmpA00=({struct Cyc_Absyn_Switch_clause*_tmp50A=_cycalloc(sizeof(*_tmp50A));({struct Cyc_Absyn_Pat*_tmp9FF=Cyc_yyget_YY9(&(yyyvsp[1]).v);_tmp50A->pattern=_tmp9FF;}),_tmp50A->pat_vars=0,_tmp50A->where_clause=0,({struct Cyc_Absyn_Stmt*_tmp9FE=Cyc_yyget_Stmt_tok(&(yyyvsp[3]).v);_tmp50A->body=_tmp9FE;}),({unsigned _tmp9FD=(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;});_tmp50A->loc=_tmp9FD;});_tmp50A;});_tmp50B->hd=_tmpA00;}),({struct Cyc_List_List*_tmp9FC=Cyc_yyget_YY8(&(yyyvsp[4]).v);_tmp50B->tl=_tmp9FC;});_tmp50B;}));
goto _LL0;case 332U: _LL293: _LL294:
# 2588 "parse.y"
 yyval=Cyc_YY8(({struct Cyc_List_List*_tmp50D=_cycalloc(sizeof(*_tmp50D));({struct Cyc_Absyn_Switch_clause*_tmpA06=({struct Cyc_Absyn_Switch_clause*_tmp50C=_cycalloc(sizeof(*_tmp50C));({struct Cyc_Absyn_Pat*_tmpA05=Cyc_yyget_YY9(&(yyyvsp[1]).v);_tmp50C->pattern=_tmpA05;}),_tmp50C->pat_vars=0,({struct Cyc_Absyn_Exp*_tmpA04=Cyc_yyget_Exp_tok(&(yyyvsp[3]).v);_tmp50C->where_clause=_tmpA04;}),({
struct Cyc_Absyn_Stmt*_tmpA03=Cyc_Absyn_fallthru_stmt(0,(unsigned)((yyyvsp[4]).l).first_line);_tmp50C->body=_tmpA03;}),({
unsigned _tmpA02=(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;});_tmp50C->loc=_tmpA02;});_tmp50C;});
# 2588
_tmp50D->hd=_tmpA06;}),({
# 2590
struct Cyc_List_List*_tmpA01=Cyc_yyget_YY8(&(yyyvsp[5]).v);_tmp50D->tl=_tmpA01;});_tmp50D;}));
goto _LL0;case 333U: _LL295: _LL296:
# 2592 "parse.y"
 yyval=Cyc_YY8(({struct Cyc_List_List*_tmp50F=_cycalloc(sizeof(*_tmp50F));({struct Cyc_Absyn_Switch_clause*_tmpA0C=({struct Cyc_Absyn_Switch_clause*_tmp50E=_cycalloc(sizeof(*_tmp50E));({struct Cyc_Absyn_Pat*_tmpA0B=Cyc_yyget_YY9(&(yyyvsp[1]).v);_tmp50E->pattern=_tmpA0B;}),_tmp50E->pat_vars=0,({struct Cyc_Absyn_Exp*_tmpA0A=Cyc_yyget_Exp_tok(&(yyyvsp[3]).v);_tmp50E->where_clause=_tmpA0A;}),({struct Cyc_Absyn_Stmt*_tmpA09=Cyc_yyget_Stmt_tok(&(yyyvsp[5]).v);_tmp50E->body=_tmpA09;}),({unsigned _tmpA08=(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;});_tmp50E->loc=_tmpA08;});_tmp50E;});_tmp50F->hd=_tmpA0C;}),({struct Cyc_List_List*_tmpA07=Cyc_yyget_YY8(&(yyyvsp[6]).v);_tmp50F->tl=_tmpA07;});_tmp50F;}));
goto _LL0;case 334U: _LL297: _LL298:
# 2599 "parse.y"
 yyval=Cyc_Stmt_tok(({struct Cyc_Absyn_Exp*_tmpA0E=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);struct Cyc_Absyn_Stmt*_tmpA0D=Cyc_yyget_Stmt_tok(&(yyyvsp[4]).v);Cyc_Absyn_while_stmt(_tmpA0E,_tmpA0D,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 335U: _LL299: _LL29A:
# 2603 "parse.y"
 yyval=Cyc_Stmt_tok(({struct Cyc_Absyn_Stmt*_tmpA10=Cyc_yyget_Stmt_tok(&(yyyvsp[1]).v);struct Cyc_Absyn_Exp*_tmpA0F=Cyc_yyget_Exp_tok(&(yyyvsp[4]).v);Cyc_Absyn_do_stmt(_tmpA10,_tmpA0F,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 336U: _LL29B: _LL29C:
# 2607 "parse.y"
 yyval=Cyc_Stmt_tok(({struct Cyc_Absyn_Exp*_tmpA14=Cyc_Absyn_false_exp(0U);struct Cyc_Absyn_Exp*_tmpA13=Cyc_Absyn_true_exp(0U);struct Cyc_Absyn_Exp*_tmpA12=Cyc_Absyn_false_exp(0U);struct Cyc_Absyn_Stmt*_tmpA11=
Cyc_yyget_Stmt_tok(&(yyyvsp[5]).v);
# 2607
Cyc_Absyn_for_stmt(_tmpA14,_tmpA13,_tmpA12,_tmpA11,(unsigned)({
yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 337U: _LL29D: _LL29E:
# 2610 "parse.y"
 yyval=Cyc_Stmt_tok(({struct Cyc_Absyn_Exp*_tmpA18=Cyc_Absyn_false_exp(0U);struct Cyc_Absyn_Exp*_tmpA17=Cyc_Absyn_true_exp(0U);struct Cyc_Absyn_Exp*_tmpA16=Cyc_yyget_Exp_tok(&(yyyvsp[4]).v);struct Cyc_Absyn_Stmt*_tmpA15=
Cyc_yyget_Stmt_tok(&(yyyvsp[6]).v);
# 2610
Cyc_Absyn_for_stmt(_tmpA18,_tmpA17,_tmpA16,_tmpA15,(unsigned)({
yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 338U: _LL29F: _LL2A0:
# 2613 "parse.y"
 yyval=Cyc_Stmt_tok(({struct Cyc_Absyn_Exp*_tmpA1C=Cyc_Absyn_false_exp(0U);struct Cyc_Absyn_Exp*_tmpA1B=Cyc_yyget_Exp_tok(&(yyyvsp[3]).v);struct Cyc_Absyn_Exp*_tmpA1A=Cyc_Absyn_false_exp(0U);struct Cyc_Absyn_Stmt*_tmpA19=
Cyc_yyget_Stmt_tok(&(yyyvsp[6]).v);
# 2613
Cyc_Absyn_for_stmt(_tmpA1C,_tmpA1B,_tmpA1A,_tmpA19,(unsigned)({
yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 339U: _LL2A1: _LL2A2:
# 2616 "parse.y"
 yyval=Cyc_Stmt_tok(({struct Cyc_Absyn_Exp*_tmpA20=Cyc_Absyn_false_exp(0U);struct Cyc_Absyn_Exp*_tmpA1F=Cyc_yyget_Exp_tok(&(yyyvsp[3]).v);struct Cyc_Absyn_Exp*_tmpA1E=Cyc_yyget_Exp_tok(&(yyyvsp[5]).v);struct Cyc_Absyn_Stmt*_tmpA1D=
Cyc_yyget_Stmt_tok(&(yyyvsp[7]).v);
# 2616
Cyc_Absyn_for_stmt(_tmpA20,_tmpA1F,_tmpA1E,_tmpA1D,(unsigned)({
yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 340U: _LL2A3: _LL2A4:
# 2619 "parse.y"
 yyval=Cyc_Stmt_tok(({struct Cyc_Absyn_Exp*_tmpA24=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);struct Cyc_Absyn_Exp*_tmpA23=Cyc_Absyn_true_exp(0U);struct Cyc_Absyn_Exp*_tmpA22=Cyc_Absyn_false_exp(0U);struct Cyc_Absyn_Stmt*_tmpA21=
Cyc_yyget_Stmt_tok(&(yyyvsp[6]).v);
# 2619
Cyc_Absyn_for_stmt(_tmpA24,_tmpA23,_tmpA22,_tmpA21,(unsigned)({
yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 341U: _LL2A5: _LL2A6:
# 2622 "parse.y"
 yyval=Cyc_Stmt_tok(({struct Cyc_Absyn_Exp*_tmpA28=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);struct Cyc_Absyn_Exp*_tmpA27=Cyc_Absyn_true_exp(0U);struct Cyc_Absyn_Exp*_tmpA26=Cyc_yyget_Exp_tok(&(yyyvsp[5]).v);struct Cyc_Absyn_Stmt*_tmpA25=
Cyc_yyget_Stmt_tok(&(yyyvsp[7]).v);
# 2622
Cyc_Absyn_for_stmt(_tmpA28,_tmpA27,_tmpA26,_tmpA25,(unsigned)({
yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 342U: _LL2A7: _LL2A8:
# 2625 "parse.y"
 yyval=Cyc_Stmt_tok(({struct Cyc_Absyn_Exp*_tmpA2C=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);struct Cyc_Absyn_Exp*_tmpA2B=Cyc_yyget_Exp_tok(&(yyyvsp[4]).v);struct Cyc_Absyn_Exp*_tmpA2A=Cyc_Absyn_false_exp(0U);struct Cyc_Absyn_Stmt*_tmpA29=
Cyc_yyget_Stmt_tok(&(yyyvsp[7]).v);
# 2625
Cyc_Absyn_for_stmt(_tmpA2C,_tmpA2B,_tmpA2A,_tmpA29,(unsigned)({
yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 343U: _LL2A9: _LL2AA:
# 2628 "parse.y"
 yyval=Cyc_Stmt_tok(({struct Cyc_Absyn_Exp*_tmpA30=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);struct Cyc_Absyn_Exp*_tmpA2F=Cyc_yyget_Exp_tok(&(yyyvsp[4]).v);struct Cyc_Absyn_Exp*_tmpA2E=Cyc_yyget_Exp_tok(&(yyyvsp[6]).v);struct Cyc_Absyn_Stmt*_tmpA2D=
Cyc_yyget_Stmt_tok(&(yyyvsp[8]).v);
# 2628
Cyc_Absyn_for_stmt(_tmpA30,_tmpA2F,_tmpA2E,_tmpA2D,(unsigned)({
yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 344U: _LL2AB: _LL2AC: {
# 2631 "parse.y"
struct Cyc_List_List*decls=Cyc_yyget_YY16(&(yyyvsp[2]).v);
struct Cyc_Absyn_Stmt*_tmp510=({struct Cyc_Absyn_Exp*_tmpA34=Cyc_Absyn_false_exp(0U);struct Cyc_Absyn_Exp*_tmpA33=Cyc_Absyn_true_exp(0U);struct Cyc_Absyn_Exp*_tmpA32=Cyc_Absyn_false_exp(0U);struct Cyc_Absyn_Stmt*_tmpA31=
Cyc_yyget_Stmt_tok(&(yyyvsp[5]).v);
# 2632
Cyc_Absyn_for_stmt(_tmpA34,_tmpA33,_tmpA32,_tmpA31,(unsigned)({
yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));});
# 2632
struct Cyc_Absyn_Stmt*s=_tmp510;
# 2634
yyval=Cyc_Stmt_tok(Cyc_Parse_flatten_declarations(decls,s));
# 2636
goto _LL0;}case 345U: _LL2AD: _LL2AE: {
# 2637 "parse.y"
struct Cyc_List_List*decls=Cyc_yyget_YY16(&(yyyvsp[2]).v);
struct Cyc_Absyn_Stmt*_tmp511=({struct Cyc_Absyn_Exp*_tmpA38=Cyc_Absyn_false_exp(0U);struct Cyc_Absyn_Exp*_tmpA37=Cyc_yyget_Exp_tok(&(yyyvsp[3]).v);struct Cyc_Absyn_Exp*_tmpA36=Cyc_Absyn_false_exp(0U);struct Cyc_Absyn_Stmt*_tmpA35=
Cyc_yyget_Stmt_tok(&(yyyvsp[6]).v);
# 2638
Cyc_Absyn_for_stmt(_tmpA38,_tmpA37,_tmpA36,_tmpA35,(unsigned)({
yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));});
# 2638
struct Cyc_Absyn_Stmt*s=_tmp511;
# 2640
yyval=Cyc_Stmt_tok(Cyc_Parse_flatten_declarations(decls,s));
# 2642
goto _LL0;}case 346U: _LL2AF: _LL2B0: {
# 2643 "parse.y"
struct Cyc_List_List*decls=Cyc_yyget_YY16(&(yyyvsp[2]).v);
struct Cyc_Absyn_Stmt*_tmp512=({struct Cyc_Absyn_Exp*_tmpA3C=Cyc_Absyn_false_exp(0U);struct Cyc_Absyn_Exp*_tmpA3B=Cyc_Absyn_true_exp(0U);struct Cyc_Absyn_Exp*_tmpA3A=Cyc_yyget_Exp_tok(&(yyyvsp[4]).v);struct Cyc_Absyn_Stmt*_tmpA39=
Cyc_yyget_Stmt_tok(&(yyyvsp[6]).v);
# 2644
Cyc_Absyn_for_stmt(_tmpA3C,_tmpA3B,_tmpA3A,_tmpA39,(unsigned)({
yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));});
# 2644
struct Cyc_Absyn_Stmt*s=_tmp512;
# 2646
yyval=Cyc_Stmt_tok(Cyc_Parse_flatten_declarations(decls,s));
# 2648
goto _LL0;}case 347U: _LL2B1: _LL2B2: {
# 2649 "parse.y"
struct Cyc_List_List*decls=Cyc_yyget_YY16(&(yyyvsp[2]).v);
struct Cyc_Absyn_Stmt*_tmp513=({struct Cyc_Absyn_Exp*_tmpA40=Cyc_Absyn_false_exp(0U);struct Cyc_Absyn_Exp*_tmpA3F=Cyc_yyget_Exp_tok(&(yyyvsp[3]).v);struct Cyc_Absyn_Exp*_tmpA3E=Cyc_yyget_Exp_tok(&(yyyvsp[5]).v);struct Cyc_Absyn_Stmt*_tmpA3D=
Cyc_yyget_Stmt_tok(&(yyyvsp[7]).v);
# 2650
Cyc_Absyn_for_stmt(_tmpA40,_tmpA3F,_tmpA3E,_tmpA3D,(unsigned)({
yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));});
# 2650
struct Cyc_Absyn_Stmt*s=_tmp513;
# 2652
yyval=Cyc_Stmt_tok(Cyc_Parse_flatten_declarations(decls,s));
# 2654
goto _LL0;}case 348U: _LL2B3: _LL2B4:
# 2659 "parse.y"
 yyval=Cyc_Stmt_tok(({struct _fat_ptr*_tmpA42=({struct _fat_ptr*_tmp514=_cycalloc(sizeof(*_tmp514));({struct _fat_ptr _tmpA41=Cyc_yyget_String_tok(&(yyyvsp[1]).v);*_tmp514=_tmpA41;});_tmp514;});Cyc_Absyn_goto_stmt(_tmpA42,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 349U: _LL2B5: _LL2B6:
# 2660 "parse.y"
 yyval=Cyc_Stmt_tok(Cyc_Absyn_continue_stmt((unsigned)((yyyvsp[0]).l).first_line));
goto _LL0;case 350U: _LL2B7: _LL2B8:
# 2661 "parse.y"
 yyval=Cyc_Stmt_tok(Cyc_Absyn_break_stmt((unsigned)((yyyvsp[0]).l).first_line));
goto _LL0;case 351U: _LL2B9: _LL2BA:
# 2662 "parse.y"
 yyval=Cyc_Stmt_tok(Cyc_Absyn_return_stmt(0,(unsigned)((yyyvsp[0]).l).first_line));
goto _LL0;case 352U: _LL2BB: _LL2BC:
# 2663 "parse.y"
 yyval=Cyc_Stmt_tok(({struct Cyc_Absyn_Exp*_tmpA43=Cyc_yyget_Exp_tok(&(yyyvsp[1]).v);Cyc_Absyn_return_stmt(_tmpA43,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 353U: _LL2BD: _LL2BE:
# 2665 "parse.y"
 yyval=Cyc_Stmt_tok(Cyc_Absyn_fallthru_stmt(0,(unsigned)((yyyvsp[0]).l).first_line));
goto _LL0;case 354U: _LL2BF: _LL2C0:
# 2666 "parse.y"
 yyval=Cyc_Stmt_tok(Cyc_Absyn_fallthru_stmt(0,(unsigned)((yyyvsp[0]).l).first_line));
goto _LL0;case 355U: _LL2C1: _LL2C2:
# 2668 "parse.y"
 yyval=Cyc_Stmt_tok(({struct Cyc_List_List*_tmpA44=Cyc_yyget_YY4(&(yyyvsp[2]).v);Cyc_Absyn_fallthru_stmt(_tmpA44,(unsigned)((yyyvsp[0]).l).first_line);}));
goto _LL0;case 356U: _LL2C3: _LL2C4:
# 2677 "parse.y"
 yyval=(yyyvsp[0]).v;
goto _LL0;case 357U: _LL2C5: _LL2C6:
# 2680
 yyval=(yyyvsp[0]).v;
goto _LL0;case 358U: _LL2C7: _LL2C8:
# 2682 "parse.y"
 yyval=Cyc_YY9(Cyc_Absyn_exp_pat(({struct Cyc_Absyn_Exp*_tmpA47=Cyc_Parse_pat2exp(Cyc_yyget_YY9(&(yyyvsp[0]).v));struct Cyc_Absyn_Exp*_tmpA46=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);struct Cyc_Absyn_Exp*_tmpA45=Cyc_yyget_Exp_tok(&(yyyvsp[4]).v);Cyc_Absyn_conditional_exp(_tmpA47,_tmpA46,_tmpA45,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));})));
goto _LL0;case 359U: _LL2C9: _LL2CA:
# 2685
 yyval=(yyyvsp[0]).v;
goto _LL0;case 360U: _LL2CB: _LL2CC:
# 2687 "parse.y"
 yyval=Cyc_YY9(Cyc_Absyn_exp_pat(({struct Cyc_Absyn_Exp*_tmpA49=Cyc_Parse_pat2exp(Cyc_yyget_YY9(&(yyyvsp[0]).v));struct Cyc_Absyn_Exp*_tmpA48=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);Cyc_Absyn_or_exp(_tmpA49,_tmpA48,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));})));
goto _LL0;case 361U: _LL2CD: _LL2CE:
# 2690
 yyval=(yyyvsp[0]).v;
goto _LL0;case 362U: _LL2CF: _LL2D0:
# 2692 "parse.y"
 yyval=Cyc_YY9(Cyc_Absyn_exp_pat(({struct Cyc_Absyn_Exp*_tmpA4B=Cyc_Parse_pat2exp(Cyc_yyget_YY9(&(yyyvsp[0]).v));struct Cyc_Absyn_Exp*_tmpA4A=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);Cyc_Absyn_and_exp(_tmpA4B,_tmpA4A,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));})));
goto _LL0;case 363U: _LL2D1: _LL2D2:
# 2695
 yyval=(yyyvsp[0]).v;
goto _LL0;case 364U: _LL2D3: _LL2D4:
# 2697 "parse.y"
 yyval=Cyc_YY9(Cyc_Absyn_exp_pat(({struct Cyc_Absyn_Exp*_tmpA4D=Cyc_Parse_pat2exp(Cyc_yyget_YY9(&(yyyvsp[0]).v));struct Cyc_Absyn_Exp*_tmpA4C=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);Cyc_Absyn_prim2_exp(Cyc_Absyn_Bitor,_tmpA4D,_tmpA4C,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));})));
goto _LL0;case 365U: _LL2D5: _LL2D6:
# 2700
 yyval=(yyyvsp[0]).v;
goto _LL0;case 366U: _LL2D7: _LL2D8:
# 2702 "parse.y"
 yyval=Cyc_YY9(Cyc_Absyn_exp_pat(({struct Cyc_Absyn_Exp*_tmpA4F=Cyc_Parse_pat2exp(Cyc_yyget_YY9(&(yyyvsp[0]).v));struct Cyc_Absyn_Exp*_tmpA4E=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);Cyc_Absyn_prim2_exp(Cyc_Absyn_Bitxor,_tmpA4F,_tmpA4E,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));})));
goto _LL0;case 367U: _LL2D9: _LL2DA:
# 2705
 yyval=(yyyvsp[0]).v;
goto _LL0;case 368U: _LL2DB: _LL2DC:
# 2707 "parse.y"
 yyval=Cyc_YY9(Cyc_Absyn_exp_pat(({struct Cyc_Absyn_Exp*_tmpA51=Cyc_Parse_pat2exp(Cyc_yyget_YY9(&(yyyvsp[0]).v));struct Cyc_Absyn_Exp*_tmpA50=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);Cyc_Absyn_prim2_exp(Cyc_Absyn_Bitand,_tmpA51,_tmpA50,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));})));
goto _LL0;case 369U: _LL2DD: _LL2DE:
# 2710
 yyval=(yyyvsp[0]).v;
goto _LL0;case 370U: _LL2DF: _LL2E0:
# 2712 "parse.y"
 yyval=Cyc_YY9(Cyc_Absyn_exp_pat(({struct Cyc_Absyn_Exp*_tmpA53=Cyc_Parse_pat2exp(Cyc_yyget_YY9(&(yyyvsp[0]).v));struct Cyc_Absyn_Exp*_tmpA52=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);Cyc_Absyn_eq_exp(_tmpA53,_tmpA52,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));})));
goto _LL0;case 371U: _LL2E1: _LL2E2:
# 2714 "parse.y"
 yyval=Cyc_YY9(Cyc_Absyn_exp_pat(({struct Cyc_Absyn_Exp*_tmpA55=Cyc_Parse_pat2exp(Cyc_yyget_YY9(&(yyyvsp[0]).v));struct Cyc_Absyn_Exp*_tmpA54=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);Cyc_Absyn_neq_exp(_tmpA55,_tmpA54,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));})));
goto _LL0;case 372U: _LL2E3: _LL2E4:
# 2717
 yyval=(yyyvsp[0]).v;
goto _LL0;case 373U: _LL2E5: _LL2E6:
# 2719 "parse.y"
 yyval=Cyc_YY9(Cyc_Absyn_exp_pat(({struct Cyc_Absyn_Exp*_tmpA57=Cyc_Parse_pat2exp(Cyc_yyget_YY9(&(yyyvsp[0]).v));struct Cyc_Absyn_Exp*_tmpA56=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);Cyc_Absyn_lt_exp(_tmpA57,_tmpA56,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));})));
goto _LL0;case 374U: _LL2E7: _LL2E8:
# 2721 "parse.y"
 yyval=Cyc_YY9(Cyc_Absyn_exp_pat(({struct Cyc_Absyn_Exp*_tmpA59=Cyc_Parse_pat2exp(Cyc_yyget_YY9(&(yyyvsp[0]).v));struct Cyc_Absyn_Exp*_tmpA58=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);Cyc_Absyn_gt_exp(_tmpA59,_tmpA58,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));})));
goto _LL0;case 375U: _LL2E9: _LL2EA:
# 2723 "parse.y"
 yyval=Cyc_YY9(Cyc_Absyn_exp_pat(({struct Cyc_Absyn_Exp*_tmpA5B=Cyc_Parse_pat2exp(Cyc_yyget_YY9(&(yyyvsp[0]).v));struct Cyc_Absyn_Exp*_tmpA5A=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);Cyc_Absyn_lte_exp(_tmpA5B,_tmpA5A,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));})));
goto _LL0;case 376U: _LL2EB: _LL2EC:
# 2725 "parse.y"
 yyval=Cyc_YY9(Cyc_Absyn_exp_pat(({struct Cyc_Absyn_Exp*_tmpA5D=Cyc_Parse_pat2exp(Cyc_yyget_YY9(&(yyyvsp[0]).v));struct Cyc_Absyn_Exp*_tmpA5C=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);Cyc_Absyn_gte_exp(_tmpA5D,_tmpA5C,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));})));
goto _LL0;case 377U: _LL2ED: _LL2EE:
# 2728
 yyval=(yyyvsp[0]).v;
goto _LL0;case 378U: _LL2EF: _LL2F0:
# 2730 "parse.y"
 yyval=Cyc_YY9(Cyc_Absyn_exp_pat(({struct Cyc_Absyn_Exp*_tmpA5F=Cyc_Parse_pat2exp(Cyc_yyget_YY9(&(yyyvsp[0]).v));struct Cyc_Absyn_Exp*_tmpA5E=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);Cyc_Absyn_prim2_exp(Cyc_Absyn_Bitlshift,_tmpA5F,_tmpA5E,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));})));
goto _LL0;case 379U: _LL2F1: _LL2F2:
# 2732 "parse.y"
 yyval=Cyc_YY9(Cyc_Absyn_exp_pat(({struct Cyc_Absyn_Exp*_tmpA61=Cyc_Parse_pat2exp(Cyc_yyget_YY9(&(yyyvsp[0]).v));struct Cyc_Absyn_Exp*_tmpA60=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);Cyc_Absyn_prim2_exp(Cyc_Absyn_Bitlrshift,_tmpA61,_tmpA60,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));})));
goto _LL0;case 380U: _LL2F3: _LL2F4:
# 2735
 yyval=(yyyvsp[0]).v;
goto _LL0;case 381U: _LL2F5: _LL2F6:
# 2737 "parse.y"
 yyval=Cyc_YY9(Cyc_Absyn_exp_pat(({struct Cyc_Absyn_Exp*_tmpA63=Cyc_Parse_pat2exp(Cyc_yyget_YY9(&(yyyvsp[0]).v));struct Cyc_Absyn_Exp*_tmpA62=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);Cyc_Absyn_prim2_exp(Cyc_Absyn_Plus,_tmpA63,_tmpA62,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));})));
goto _LL0;case 382U: _LL2F7: _LL2F8:
# 2739 "parse.y"
 yyval=Cyc_YY9(Cyc_Absyn_exp_pat(({struct Cyc_Absyn_Exp*_tmpA65=Cyc_Parse_pat2exp(Cyc_yyget_YY9(&(yyyvsp[0]).v));struct Cyc_Absyn_Exp*_tmpA64=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);Cyc_Absyn_prim2_exp(Cyc_Absyn_Minus,_tmpA65,_tmpA64,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));})));
goto _LL0;case 383U: _LL2F9: _LL2FA:
# 2742
 yyval=(yyyvsp[0]).v;
goto _LL0;case 384U: _LL2FB: _LL2FC:
# 2744 "parse.y"
 yyval=Cyc_YY9(Cyc_Absyn_exp_pat(({struct Cyc_Absyn_Exp*_tmpA67=Cyc_Parse_pat2exp(Cyc_yyget_YY9(&(yyyvsp[0]).v));struct Cyc_Absyn_Exp*_tmpA66=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);Cyc_Absyn_prim2_exp(Cyc_Absyn_Times,_tmpA67,_tmpA66,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));})));
goto _LL0;case 385U: _LL2FD: _LL2FE:
# 2746 "parse.y"
 yyval=Cyc_YY9(Cyc_Absyn_exp_pat(({struct Cyc_Absyn_Exp*_tmpA69=Cyc_Parse_pat2exp(Cyc_yyget_YY9(&(yyyvsp[0]).v));struct Cyc_Absyn_Exp*_tmpA68=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);Cyc_Absyn_prim2_exp(Cyc_Absyn_Div,_tmpA69,_tmpA68,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));})));
goto _LL0;case 386U: _LL2FF: _LL300:
# 2748 "parse.y"
 yyval=Cyc_YY9(Cyc_Absyn_exp_pat(({struct Cyc_Absyn_Exp*_tmpA6B=Cyc_Parse_pat2exp(Cyc_yyget_YY9(&(yyyvsp[0]).v));struct Cyc_Absyn_Exp*_tmpA6A=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);Cyc_Absyn_prim2_exp(Cyc_Absyn_Mod,_tmpA6B,_tmpA6A,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));})));
goto _LL0;case 387U: _LL301: _LL302:
# 2751
 yyval=(yyyvsp[0]).v;
goto _LL0;case 388U: _LL303: _LL304: {
# 2753 "parse.y"
void*t=({struct _tuple8*_tmpA6C=Cyc_yyget_YY37(&(yyyvsp[1]).v);Cyc_Parse_type_name_to_type(_tmpA6C,(unsigned)((yyyvsp[1]).l).first_line);});
yyval=Cyc_YY9(Cyc_Absyn_exp_pat(({void*_tmpA6E=t;struct Cyc_Absyn_Exp*_tmpA6D=Cyc_yyget_Exp_tok(&(yyyvsp[3]).v);Cyc_Absyn_cast_exp(_tmpA6E,_tmpA6D,1,Cyc_Absyn_Unknown_coercion,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));})));
# 2756
goto _LL0;}case 389U: _LL305: _LL306:
# 2759 "parse.y"
 yyval=(yyyvsp[0]).v;
goto _LL0;case 390U: _LL307: _LL308:
# 2762
 yyval=Cyc_YY9(Cyc_Absyn_exp_pat(Cyc_yyget_Exp_tok(&(yyyvsp[1]).v)));
goto _LL0;case 391U: _LL309: _LL30A:
# 2764 "parse.y"
 yyval=Cyc_YY9(Cyc_Absyn_exp_pat(({enum Cyc_Absyn_Primop _tmpA70=Cyc_yyget_YY6(&(yyyvsp[0]).v);struct Cyc_Absyn_Exp*_tmpA6F=Cyc_yyget_Exp_tok(&(yyyvsp[1]).v);Cyc_Absyn_prim1_exp(_tmpA70,_tmpA6F,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));})));
goto _LL0;case 392U: _LL30B: _LL30C: {
# 2766 "parse.y"
void*t=({struct _tuple8*_tmpA71=Cyc_yyget_YY37(&(yyyvsp[2]).v);Cyc_Parse_type_name_to_type(_tmpA71,(unsigned)((yyyvsp[2]).l).first_line);});
yyval=Cyc_YY9(Cyc_Absyn_exp_pat(({void*_tmpA72=t;Cyc_Absyn_sizeoftype_exp(_tmpA72,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));})));
# 2769
goto _LL0;}case 393U: _LL30D: _LL30E:
# 2770 "parse.y"
 yyval=Cyc_YY9(Cyc_Absyn_exp_pat(({struct Cyc_Absyn_Exp*_tmpA73=Cyc_yyget_Exp_tok(&(yyyvsp[1]).v);Cyc_Absyn_sizeofexp_exp(_tmpA73,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));})));
goto _LL0;case 394U: _LL30F: _LL310:
# 2772 "parse.y"
 yyval=Cyc_YY9(Cyc_Absyn_exp_pat(({void*_tmpA75=(*Cyc_yyget_YY37(&(yyyvsp[2]).v)).f3;struct Cyc_List_List*_tmpA74=Cyc_List_imp_rev(Cyc_yyget_YY3(&(yyyvsp[4]).v));Cyc_Absyn_offsetof_exp(_tmpA75,_tmpA74,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));})));
goto _LL0;case 395U: _LL311: _LL312:
# 2777 "parse.y"
 yyval=(yyyvsp[0]).v;
goto _LL0;case 396U: _LL313: _LL314:
# 2785 "parse.y"
 yyval=(yyyvsp[0]).v;
goto _LL0;case 397U: _LL315: _LL316:
# 2790 "parse.y"
 yyval=Cyc_YY9(Cyc_Absyn_new_pat((void*)& Cyc_Absyn_Wild_p_val,(unsigned)((yyyvsp[0]).l).first_line));
goto _LL0;case 398U: _LL317: _LL318:
# 2792 "parse.y"
 yyval=Cyc_YY9(Cyc_Absyn_exp_pat(Cyc_yyget_Exp_tok(&(yyyvsp[1]).v)));
goto _LL0;case 399U: _LL319: _LL31A: {
# 2794 "parse.y"
struct Cyc_Absyn_Exp*e=Cyc_yyget_Exp_tok(&(yyyvsp[0]).v);
{void*_stmttmp2A=e->r;void*_tmp515=_stmttmp2A;int _tmp517;struct _fat_ptr _tmp516;int _tmp519;enum Cyc_Absyn_Sign _tmp518;short _tmp51B;enum Cyc_Absyn_Sign _tmp51A;char _tmp51D;enum Cyc_Absyn_Sign _tmp51C;if(((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmp515)->tag == 0U)switch(((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmp515)->f1).LongLong_c).tag){case 2U: _LL4A7: _tmp51C=(((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmp515)->f1).Char_c).val).f1;_tmp51D=(((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmp515)->f1).Char_c).val).f2;_LL4A8: {enum Cyc_Absyn_Sign s=_tmp51C;char i=_tmp51D;
# 2798
yyval=Cyc_YY9(({void*_tmpA76=(void*)({struct Cyc_Absyn_Char_p_Absyn_Raw_pat_struct*_tmp51E=_cycalloc(sizeof(*_tmp51E));_tmp51E->tag=11U,_tmp51E->f1=i;_tmp51E;});Cyc_Absyn_new_pat(_tmpA76,e->loc);}));goto _LL4A6;}case 4U: _LL4A9: _tmp51A=(((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmp515)->f1).Short_c).val).f1;_tmp51B=(((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmp515)->f1).Short_c).val).f2;_LL4AA: {enum Cyc_Absyn_Sign s=_tmp51A;short i=_tmp51B;
# 2800
yyval=Cyc_YY9(({void*_tmpA77=(void*)({struct Cyc_Absyn_Int_p_Absyn_Raw_pat_struct*_tmp51F=_cycalloc(sizeof(*_tmp51F));_tmp51F->tag=10U,_tmp51F->f1=s,_tmp51F->f2=(int)i;_tmp51F;});Cyc_Absyn_new_pat(_tmpA77,e->loc);}));goto _LL4A6;}case 5U: _LL4AB: _tmp518=(((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmp515)->f1).Int_c).val).f1;_tmp519=(((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmp515)->f1).Int_c).val).f2;_LL4AC: {enum Cyc_Absyn_Sign s=_tmp518;int i=_tmp519;
# 2802
yyval=Cyc_YY9(({void*_tmpA78=(void*)({struct Cyc_Absyn_Int_p_Absyn_Raw_pat_struct*_tmp520=_cycalloc(sizeof(*_tmp520));_tmp520->tag=10U,_tmp520->f1=s,_tmp520->f2=i;_tmp520;});Cyc_Absyn_new_pat(_tmpA78,e->loc);}));goto _LL4A6;}case 7U: _LL4AD: _tmp516=(((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmp515)->f1).Float_c).val).f1;_tmp517=(((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmp515)->f1).Float_c).val).f2;_LL4AE: {struct _fat_ptr s=_tmp516;int i=_tmp517;
# 2804
yyval=Cyc_YY9(({void*_tmpA79=(void*)({struct Cyc_Absyn_Float_p_Absyn_Raw_pat_struct*_tmp521=_cycalloc(sizeof(*_tmp521));_tmp521->tag=12U,_tmp521->f1=s,_tmp521->f2=i;_tmp521;});Cyc_Absyn_new_pat(_tmpA79,e->loc);}));goto _LL4A6;}case 1U: _LL4AF: _LL4B0:
# 2806
 yyval=Cyc_YY9(Cyc_Absyn_new_pat((void*)& Cyc_Absyn_Null_p_val,e->loc));goto _LL4A6;case 8U: _LL4B1: _LL4B2:
# 2808
({void*_tmp522=0U;({unsigned _tmpA7B=(unsigned)((yyyvsp[0]).l).first_line;struct _fat_ptr _tmpA7A=({const char*_tmp523="strings cannot occur within patterns";_tag_fat(_tmp523,sizeof(char),37U);});Cyc_Warn_err(_tmpA7B,_tmpA7A,_tag_fat(_tmp522,sizeof(void*),0U));});});goto _LL4A6;case 9U: _LL4B3: _LL4B4:
# 2810
({void*_tmp524=0U;({unsigned _tmpA7D=(unsigned)((yyyvsp[0]).l).first_line;struct _fat_ptr _tmpA7C=({const char*_tmp525="strings cannot occur within patterns";_tag_fat(_tmp525,sizeof(char),37U);});Cyc_Warn_err(_tmpA7D,_tmpA7C,_tag_fat(_tmp524,sizeof(void*),0U));});});goto _LL4A6;case 6U: _LL4B5: _LL4B6:
# 2812
({void*_tmp526=0U;({unsigned _tmpA7F=(unsigned)((yyyvsp[0]).l).first_line;struct _fat_ptr _tmpA7E=({const char*_tmp527="long long's in patterns not yet implemented";_tag_fat(_tmp527,sizeof(char),44U);});Cyc_Warn_err(_tmpA7F,_tmpA7E,_tag_fat(_tmp526,sizeof(void*),0U));});});goto _LL4A6;default: goto _LL4B7;}else{_LL4B7: _LL4B8:
# 2814
({void*_tmp528=0U;({unsigned _tmpA81=(unsigned)((yyyvsp[0]).l).first_line;struct _fat_ptr _tmpA80=({const char*_tmp529="bad constant in case";_tag_fat(_tmp529,sizeof(char),21U);});Cyc_Warn_err(_tmpA81,_tmpA80,_tag_fat(_tmp528,sizeof(void*),0U));});});}_LL4A6:;}
# 2817
goto _LL0;}case 400U: _LL31B: _LL31C:
# 2818 "parse.y"
 yyval=Cyc_YY9(({void*_tmpA83=(void*)({struct Cyc_Absyn_UnknownId_p_Absyn_Raw_pat_struct*_tmp52A=_cycalloc(sizeof(*_tmp52A));_tmp52A->tag=15U,({struct _tuple0*_tmpA82=Cyc_yyget_QualId_tok(&(yyyvsp[0]).v);_tmp52A->f1=_tmpA82;});_tmp52A;});Cyc_Absyn_new_pat(_tmpA83,(unsigned)((yyyvsp[0]).l).first_line);}));
goto _LL0;case 401U: _LL31D: _LL31E:
# 2820 "parse.y"
 if(({struct _fat_ptr _tmpA84=(struct _fat_ptr)Cyc_yyget_String_tok(&(yyyvsp[1]).v);Cyc_strcmp(_tmpA84,({const char*_tmp52B="as";_tag_fat(_tmp52B,sizeof(char),3U);}));})!= 0)
({void*_tmp52C=0U;({unsigned _tmpA86=(unsigned)((yyyvsp[1]).l).first_line;struct _fat_ptr _tmpA85=({const char*_tmp52D="expecting `as'";_tag_fat(_tmp52D,sizeof(char),15U);});Cyc_Warn_err(_tmpA86,_tmpA85,_tag_fat(_tmp52C,sizeof(void*),0U));});});
# 2820
yyval=
# 2822
Cyc_YY9(({void*_tmpA8D=(void*)({struct Cyc_Absyn_Var_p_Absyn_Raw_pat_struct*_tmp530=_cycalloc(sizeof(*_tmp530));_tmp530->tag=1U,({struct Cyc_Absyn_Vardecl*_tmpA8C=({unsigned _tmpA8B=(unsigned)((yyyvsp[0]).l).first_line;struct _tuple0*_tmpA8A=({struct _tuple0*_tmp52F=_cycalloc(sizeof(*_tmp52F));_tmp52F->f1=Cyc_Absyn_Loc_n,({struct _fat_ptr*_tmpA89=({struct _fat_ptr*_tmp52E=_cycalloc(sizeof(*_tmp52E));({struct _fat_ptr _tmpA88=Cyc_yyget_String_tok(&(yyyvsp[0]).v);*_tmp52E=_tmpA88;});_tmp52E;});_tmp52F->f2=_tmpA89;});_tmp52F;});Cyc_Absyn_new_vardecl(_tmpA8B,_tmpA8A,Cyc_Absyn_void_type,0);});_tmp530->f1=_tmpA8C;}),({
struct Cyc_Absyn_Pat*_tmpA87=Cyc_yyget_YY9(&(yyyvsp[2]).v);_tmp530->f2=_tmpA87;});_tmp530;});
# 2822
Cyc_Absyn_new_pat(_tmpA8D,(unsigned)((yyyvsp[0]).l).first_line);}));
# 2825
goto _LL0;case 402U: _LL31F: _LL320:
# 2826 "parse.y"
 if(({struct _fat_ptr _tmpA8E=(struct _fat_ptr)Cyc_yyget_String_tok(&(yyyvsp[0]).v);Cyc_strcmp(_tmpA8E,({const char*_tmp531="alias";_tag_fat(_tmp531,sizeof(char),6U);}));})!= 0)
({void*_tmp532=0U;({unsigned _tmpA90=(unsigned)((yyyvsp[1]).l).first_line;struct _fat_ptr _tmpA8F=({const char*_tmp533="expecting `alias'";_tag_fat(_tmp533,sizeof(char),18U);});Cyc_Warn_err(_tmpA90,_tmpA8F,_tag_fat(_tmp532,sizeof(void*),0U));});});{
# 2826
int _tmp534=({
# 2828
yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;});
# 2826
int location=_tmp534;
# 2829
struct Cyc_Absyn_Tvar*tv=({struct Cyc_Absyn_Tvar*_tmp53A=_cycalloc(sizeof(*_tmp53A));({struct _fat_ptr*_tmpA93=({struct _fat_ptr*_tmp538=_cycalloc(sizeof(*_tmp538));({struct _fat_ptr _tmpA92=Cyc_yyget_String_tok(&(yyyvsp[2]).v);*_tmp538=_tmpA92;});_tmp538;});_tmp53A->name=_tmpA93;}),_tmp53A->identity=- 1,({void*_tmpA91=(void*)({struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct*_tmp539=_cycalloc(sizeof(*_tmp539));_tmp539->tag=0U,_tmp539->f1=& Cyc_Tcutil_rk;_tmp539;});_tmp53A->kind=_tmpA91;});_tmp53A;});
struct Cyc_Absyn_Vardecl*vd=({unsigned _tmpA98=(unsigned)((yyyvsp[0]).l).first_line;struct _tuple0*_tmpA97=({struct _tuple0*_tmp537=_cycalloc(sizeof(*_tmp537));_tmp537->f1=Cyc_Absyn_Loc_n,({struct _fat_ptr*_tmpA95=({struct _fat_ptr*_tmp536=_cycalloc(sizeof(*_tmp536));({struct _fat_ptr _tmpA94=Cyc_yyget_String_tok(&(yyyvsp[5]).v);*_tmp536=_tmpA94;});_tmp536;});_tmp537->f2=_tmpA95;});_tmp537;});Cyc_Absyn_new_vardecl(_tmpA98,_tmpA97,({
struct _tuple8*_tmpA96=Cyc_yyget_YY37(&(yyyvsp[4]).v);Cyc_Parse_type_name_to_type(_tmpA96,(unsigned)((yyyvsp[4]).l).first_line);}),0);});
yyval=Cyc_YY9(({void*_tmpA99=(void*)({struct Cyc_Absyn_AliasVar_p_Absyn_Raw_pat_struct*_tmp535=_cycalloc(sizeof(*_tmp535));_tmp535->tag=2U,_tmp535->f1=tv,_tmp535->f2=vd;_tmp535;});Cyc_Absyn_new_pat(_tmpA99,(unsigned)location);}));
# 2834
goto _LL0;}case 403U: _LL321: _LL322:
# 2835 "parse.y"
 if(({struct _fat_ptr _tmpA9A=(struct _fat_ptr)Cyc_yyget_String_tok(&(yyyvsp[0]).v);Cyc_strcmp(_tmpA9A,({const char*_tmp53B="alias";_tag_fat(_tmp53B,sizeof(char),6U);}));})!= 0)
({void*_tmp53C=0U;({unsigned _tmpA9C=(unsigned)((yyyvsp[1]).l).first_line;struct _fat_ptr _tmpA9B=({const char*_tmp53D="expecting `alias'";_tag_fat(_tmp53D,sizeof(char),18U);});Cyc_Warn_err(_tmpA9C,_tmpA9B,_tag_fat(_tmp53C,sizeof(void*),0U));});});{
# 2835
int _tmp53E=({
# 2837
yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;});
# 2835
int location=_tmp53E;
# 2838
struct Cyc_Absyn_Tvar*tv=({struct Cyc_Absyn_Tvar*_tmp544=_cycalloc(sizeof(*_tmp544));({struct _fat_ptr*_tmpA9F=({struct _fat_ptr*_tmp542=_cycalloc(sizeof(*_tmp542));({struct _fat_ptr _tmpA9E=Cyc_yyget_String_tok(&(yyyvsp[2]).v);*_tmp542=_tmpA9E;});_tmp542;});_tmp544->name=_tmpA9F;}),_tmp544->identity=- 1,({void*_tmpA9D=(void*)({struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct*_tmp543=_cycalloc(sizeof(*_tmp543));_tmp543->tag=0U,_tmp543->f1=& Cyc_Tcutil_rk;_tmp543;});_tmp544->kind=_tmpA9D;});_tmp544;});
struct Cyc_Absyn_Vardecl*vd=({unsigned _tmpAA4=(unsigned)((yyyvsp[0]).l).first_line;struct _tuple0*_tmpAA3=({struct _tuple0*_tmp541=_cycalloc(sizeof(*_tmp541));_tmp541->f1=Cyc_Absyn_Loc_n,({struct _fat_ptr*_tmpAA1=({struct _fat_ptr*_tmp540=_cycalloc(sizeof(*_tmp540));({struct _fat_ptr _tmpAA0=Cyc_yyget_String_tok(&(yyyvsp[5]).v);*_tmp540=_tmpAA0;});_tmp540;});_tmp541->f2=_tmpAA1;});_tmp541;});Cyc_Absyn_new_vardecl(_tmpAA4,_tmpAA3,({
struct _tuple8*_tmpAA2=Cyc_yyget_YY37(&(yyyvsp[4]).v);Cyc_Parse_type_name_to_type(_tmpAA2,(unsigned)((yyyvsp[4]).l).first_line);}),0);});
yyval=Cyc_YY9(({void*_tmpAA5=(void*)({struct Cyc_Absyn_AliasVar_p_Absyn_Raw_pat_struct*_tmp53F=_cycalloc(sizeof(*_tmp53F));_tmp53F->tag=2U,_tmp53F->f1=tv,_tmp53F->f2=vd;_tmp53F;});Cyc_Absyn_new_pat(_tmpAA5,(unsigned)location);}));
# 2843
goto _LL0;}case 404U: _LL323: _LL324: {
# 2844 "parse.y"
struct _tuple23 _stmttmp2B=*Cyc_yyget_YY10(&(yyyvsp[2]).v);int _tmp546;struct Cyc_List_List*_tmp545;_LL4BA: _tmp545=_stmttmp2B.f1;_tmp546=_stmttmp2B.f2;_LL4BB: {struct Cyc_List_List*ps=_tmp545;int dots=_tmp546;
yyval=Cyc_YY9(({void*_tmpAA6=(void*)({struct Cyc_Absyn_Tuple_p_Absyn_Raw_pat_struct*_tmp547=_cycalloc(sizeof(*_tmp547));_tmp547->tag=5U,_tmp547->f1=ps,_tmp547->f2=dots;_tmp547;});Cyc_Absyn_new_pat(_tmpAA6,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
# 2847
goto _LL0;}}case 405U: _LL325: _LL326: {
# 2848 "parse.y"
struct _tuple23 _stmttmp2C=*Cyc_yyget_YY10(&(yyyvsp[2]).v);int _tmp549;struct Cyc_List_List*_tmp548;_LL4BD: _tmp548=_stmttmp2C.f1;_tmp549=_stmttmp2C.f2;_LL4BE: {struct Cyc_List_List*ps=_tmp548;int dots=_tmp549;
yyval=Cyc_YY9(({void*_tmpAA8=(void*)({struct Cyc_Absyn_UnknownCall_p_Absyn_Raw_pat_struct*_tmp54A=_cycalloc(sizeof(*_tmp54A));_tmp54A->tag=16U,({struct _tuple0*_tmpAA7=Cyc_yyget_QualId_tok(&(yyyvsp[0]).v);_tmp54A->f1=_tmpAA7;}),_tmp54A->f2=ps,_tmp54A->f3=dots;_tmp54A;});Cyc_Absyn_new_pat(_tmpAA8,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
# 2851
goto _LL0;}}case 406U: _LL327: _LL328: {
# 2853 "parse.y"
struct _tuple23 _stmttmp2D=*Cyc_yyget_YY14(&(yyyvsp[3]).v);int _tmp54C;struct Cyc_List_List*_tmp54B;_LL4C0: _tmp54B=_stmttmp2D.f1;_tmp54C=_stmttmp2D.f2;_LL4C1: {struct Cyc_List_List*fps=_tmp54B;int dots=_tmp54C;
struct Cyc_List_List*exist_ts=({struct Cyc_List_List*(*_tmpAAA)(struct Cyc_Absyn_Tvar*(*f)(unsigned,void*),unsigned env,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp54F)(struct Cyc_Absyn_Tvar*(*f)(unsigned,void*),unsigned env,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_Absyn_Tvar*(*f)(unsigned,void*),unsigned env,struct Cyc_List_List*x))Cyc_List_map_c;_tmp54F;});unsigned _tmpAA9=(unsigned)((yyyvsp[2]).l).first_line;_tmpAAA(Cyc_Parse_typ2tvar,_tmpAA9,Cyc_yyget_YY40(&(yyyvsp[2]).v));});
yyval=Cyc_YY9(({void*_tmpAAD=(void*)({struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*_tmp54E=_cycalloc(sizeof(*_tmp54E));_tmp54E->tag=7U,({union Cyc_Absyn_AggrInfo*_tmpAAC=({union Cyc_Absyn_AggrInfo*_tmp54D=_cycalloc(sizeof(*_tmp54D));({union Cyc_Absyn_AggrInfo _tmpAAB=Cyc_Absyn_UnknownAggr(Cyc_Absyn_StructA,Cyc_yyget_QualId_tok(&(yyyvsp[0]).v),0);*_tmp54D=_tmpAAB;});_tmp54D;});_tmp54E->f1=_tmpAAC;}),_tmp54E->f2=exist_ts,_tmp54E->f3=fps,_tmp54E->f4=dots;_tmp54E;});Cyc_Absyn_new_pat(_tmpAAD,(unsigned)({
yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
# 2858
goto _LL0;}}case 407U: _LL329: _LL32A: {
# 2860 "parse.y"
struct _tuple23 _stmttmp2E=*Cyc_yyget_YY14(&(yyyvsp[2]).v);int _tmp551;struct Cyc_List_List*_tmp550;_LL4C3: _tmp550=_stmttmp2E.f1;_tmp551=_stmttmp2E.f2;_LL4C4: {struct Cyc_List_List*fps=_tmp550;int dots=_tmp551;
struct Cyc_List_List*exist_ts=({struct Cyc_List_List*(*_tmpAAF)(struct Cyc_Absyn_Tvar*(*f)(unsigned,void*),unsigned env,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp553)(struct Cyc_Absyn_Tvar*(*f)(unsigned,void*),unsigned env,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_Absyn_Tvar*(*f)(unsigned,void*),unsigned env,struct Cyc_List_List*x))Cyc_List_map_c;_tmp553;});unsigned _tmpAAE=(unsigned)((yyyvsp[1]).l).first_line;_tmpAAF(Cyc_Parse_typ2tvar,_tmpAAE,Cyc_yyget_YY40(&(yyyvsp[1]).v));});
yyval=Cyc_YY9(({void*_tmpAB0=(void*)({struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*_tmp552=_cycalloc(sizeof(*_tmp552));_tmp552->tag=7U,_tmp552->f1=0,_tmp552->f2=exist_ts,_tmp552->f3=fps,_tmp552->f4=dots;_tmp552;});Cyc_Absyn_new_pat(_tmpAB0,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
# 2864
goto _LL0;}}case 408U: _LL32B: _LL32C:
# 2865 "parse.y"
 yyval=Cyc_YY9(({void*_tmpAB2=(void*)({struct Cyc_Absyn_Pointer_p_Absyn_Raw_pat_struct*_tmp554=_cycalloc(sizeof(*_tmp554));_tmp554->tag=6U,({struct Cyc_Absyn_Pat*_tmpAB1=Cyc_yyget_YY9(&(yyyvsp[1]).v);_tmp554->f1=_tmpAB1;});_tmp554;});Cyc_Absyn_new_pat(_tmpAB2,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 409U: _LL32D: _LL32E:
# 2867 "parse.y"
 yyval=Cyc_YY9(({void*_tmpAB6=(void*)({struct Cyc_Absyn_Pointer_p_Absyn_Raw_pat_struct*_tmp556=_cycalloc(sizeof(*_tmp556));_tmp556->tag=6U,({struct Cyc_Absyn_Pat*_tmpAB5=({void*_tmpAB4=(void*)({struct Cyc_Absyn_Pointer_p_Absyn_Raw_pat_struct*_tmp555=_cycalloc(sizeof(*_tmp555));_tmp555->tag=6U,({struct Cyc_Absyn_Pat*_tmpAB3=Cyc_yyget_YY9(&(yyyvsp[1]).v);_tmp555->f1=_tmpAB3;});_tmp555;});Cyc_Absyn_new_pat(_tmpAB4,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));});_tmp556->f1=_tmpAB5;});_tmp556;});Cyc_Absyn_new_pat(_tmpAB6,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 410U: _LL32F: _LL330:
# 2869 "parse.y"
 yyval=Cyc_YY9(({void*_tmpABD=(void*)({struct Cyc_Absyn_Reference_p_Absyn_Raw_pat_struct*_tmp559=_cycalloc(sizeof(*_tmp559));_tmp559->tag=3U,({struct Cyc_Absyn_Vardecl*_tmpABC=({unsigned _tmpABB=(unsigned)((yyyvsp[0]).l).first_line;struct _tuple0*_tmpABA=({struct _tuple0*_tmp558=_cycalloc(sizeof(*_tmp558));_tmp558->f1=Cyc_Absyn_Loc_n,({struct _fat_ptr*_tmpAB9=({struct _fat_ptr*_tmp557=_cycalloc(sizeof(*_tmp557));({struct _fat_ptr _tmpAB8=Cyc_yyget_String_tok(&(yyyvsp[1]).v);*_tmp557=_tmpAB8;});_tmp557;});_tmp558->f2=_tmpAB9;});_tmp558;});Cyc_Absyn_new_vardecl(_tmpABB,_tmpABA,Cyc_Absyn_void_type,0);});_tmp559->f1=_tmpABC;}),({
# 2871
struct Cyc_Absyn_Pat*_tmpAB7=Cyc_Absyn_new_pat((void*)& Cyc_Absyn_Wild_p_val,(unsigned)((yyyvsp[1]).l).first_line);_tmp559->f2=_tmpAB7;});_tmp559;});
# 2869
Cyc_Absyn_new_pat(_tmpABD,(unsigned)({
# 2872
yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 411U: _LL331: _LL332:
# 2874 "parse.y"
 if(({struct _fat_ptr _tmpABE=(struct _fat_ptr)Cyc_yyget_String_tok(&(yyyvsp[2]).v);Cyc_strcmp(_tmpABE,({const char*_tmp55A="as";_tag_fat(_tmp55A,sizeof(char),3U);}));})!= 0)
({void*_tmp55B=0U;({unsigned _tmpAC0=(unsigned)((yyyvsp[2]).l).first_line;struct _fat_ptr _tmpABF=({const char*_tmp55C="expecting `as'";_tag_fat(_tmp55C,sizeof(char),15U);});Cyc_Warn_err(_tmpAC0,_tmpABF,_tag_fat(_tmp55B,sizeof(void*),0U));});});
# 2874
yyval=
# 2876
Cyc_YY9(({void*_tmpAC7=(void*)({struct Cyc_Absyn_Reference_p_Absyn_Raw_pat_struct*_tmp55F=_cycalloc(sizeof(*_tmp55F));_tmp55F->tag=3U,({struct Cyc_Absyn_Vardecl*_tmpAC6=({unsigned _tmpAC5=(unsigned)((yyyvsp[0]).l).first_line;struct _tuple0*_tmpAC4=({struct _tuple0*_tmp55E=_cycalloc(sizeof(*_tmp55E));_tmp55E->f1=Cyc_Absyn_Loc_n,({struct _fat_ptr*_tmpAC3=({struct _fat_ptr*_tmp55D=_cycalloc(sizeof(*_tmp55D));({struct _fat_ptr _tmpAC2=Cyc_yyget_String_tok(&(yyyvsp[1]).v);*_tmp55D=_tmpAC2;});_tmp55D;});_tmp55E->f2=_tmpAC3;});_tmp55E;});Cyc_Absyn_new_vardecl(_tmpAC5,_tmpAC4,Cyc_Absyn_void_type,0);});_tmp55F->f1=_tmpAC6;}),({
# 2878
struct Cyc_Absyn_Pat*_tmpAC1=Cyc_yyget_YY9(&(yyyvsp[3]).v);_tmp55F->f2=_tmpAC1;});_tmp55F;});
# 2876
Cyc_Absyn_new_pat(_tmpAC7,(unsigned)({
# 2878
yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
# 2880
goto _LL0;case 412U: _LL333: _LL334: {
# 2881 "parse.y"
void*tag=({struct _fat_ptr _tmpAC8=Cyc_yyget_String_tok(&(yyyvsp[2]).v);Cyc_Parse_id2type(_tmpAC8,Cyc_Tcutil_kind_to_bound(& Cyc_Tcutil_ik));});
yyval=Cyc_YY9(({void*_tmpACF=(void*)({struct Cyc_Absyn_TagInt_p_Absyn_Raw_pat_struct*_tmp562=_cycalloc(sizeof(*_tmp562));_tmp562->tag=4U,({struct Cyc_Absyn_Tvar*_tmpACE=Cyc_Parse_typ2tvar((unsigned)((yyyvsp[2]).l).first_line,tag);_tmp562->f1=_tmpACE;}),({
struct Cyc_Absyn_Vardecl*_tmpACD=({unsigned _tmpACC=(unsigned)((yyyvsp[0]).l).first_line;struct _tuple0*_tmpACB=({struct _tuple0*_tmp561=_cycalloc(sizeof(*_tmp561));_tmp561->f1=Cyc_Absyn_Loc_n,({struct _fat_ptr*_tmpACA=({struct _fat_ptr*_tmp560=_cycalloc(sizeof(*_tmp560));({struct _fat_ptr _tmpAC9=Cyc_yyget_String_tok(&(yyyvsp[0]).v);*_tmp560=_tmpAC9;});_tmp560;});_tmp561->f2=_tmpACA;});_tmp561;});Cyc_Absyn_new_vardecl(_tmpACC,_tmpACB,
Cyc_Absyn_tag_type(tag),0);});
# 2883
_tmp562->f2=_tmpACD;});_tmp562;});
# 2882
Cyc_Absyn_new_pat(_tmpACF,(unsigned)({
# 2885
yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;}case 413U: _LL335: _LL336: {
# 2887 "parse.y"
struct Cyc_Absyn_Tvar*tv=Cyc_Tcutil_new_tvar(Cyc_Tcutil_kind_to_bound(& Cyc_Tcutil_ik));
yyval=Cyc_YY9(({void*_tmpAD5=(void*)({struct Cyc_Absyn_TagInt_p_Absyn_Raw_pat_struct*_tmp565=_cycalloc(sizeof(*_tmp565));_tmp565->tag=4U,_tmp565->f1=tv,({
struct Cyc_Absyn_Vardecl*_tmpAD4=({unsigned _tmpAD3=(unsigned)((yyyvsp[0]).l).first_line;struct _tuple0*_tmpAD2=({struct _tuple0*_tmp564=_cycalloc(sizeof(*_tmp564));_tmp564->f1=Cyc_Absyn_Loc_n,({struct _fat_ptr*_tmpAD1=({struct _fat_ptr*_tmp563=_cycalloc(sizeof(*_tmp563));({struct _fat_ptr _tmpAD0=Cyc_yyget_String_tok(&(yyyvsp[0]).v);*_tmp563=_tmpAD0;});_tmp563;});_tmp564->f2=_tmpAD1;});_tmp564;});Cyc_Absyn_new_vardecl(_tmpAD3,_tmpAD2,
Cyc_Absyn_tag_type(Cyc_Absyn_var_type(tv)),0);});
# 2889
_tmp565->f2=_tmpAD4;});_tmp565;});
# 2888
Cyc_Absyn_new_pat(_tmpAD5,(unsigned)({
# 2891
yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;}case 414U: _LL337: _LL338:
# 2895 "parse.y"
 yyval=Cyc_YY10(({struct _tuple23*_tmp566=_cycalloc(sizeof(*_tmp566));({struct Cyc_List_List*_tmpAD6=Cyc_List_rev(Cyc_yyget_YY11(&(yyyvsp[0]).v));_tmp566->f1=_tmpAD6;}),_tmp566->f2=0;_tmp566;}));
goto _LL0;case 415U: _LL339: _LL33A:
# 2896 "parse.y"
 yyval=Cyc_YY10(({struct _tuple23*_tmp567=_cycalloc(sizeof(*_tmp567));({struct Cyc_List_List*_tmpAD7=Cyc_List_rev(Cyc_yyget_YY11(&(yyyvsp[0]).v));_tmp567->f1=_tmpAD7;}),_tmp567->f2=1;_tmp567;}));
goto _LL0;case 416U: _LL33B: _LL33C:
# 2897 "parse.y"
 yyval=Cyc_YY10(({struct _tuple23*_tmp568=_cycalloc(sizeof(*_tmp568));_tmp568->f1=0,_tmp568->f2=1;_tmp568;}));
goto _LL0;case 417U: _LL33D: _LL33E:
# 2902 "parse.y"
 yyval=Cyc_YY11(({struct Cyc_List_List*_tmp569=_cycalloc(sizeof(*_tmp569));({struct Cyc_Absyn_Pat*_tmpAD8=Cyc_yyget_YY9(&(yyyvsp[0]).v);_tmp569->hd=_tmpAD8;}),_tmp569->tl=0;_tmp569;}));
goto _LL0;case 418U: _LL33F: _LL340:
# 2904 "parse.y"
 yyval=Cyc_YY11(({struct Cyc_List_List*_tmp56A=_cycalloc(sizeof(*_tmp56A));({struct Cyc_Absyn_Pat*_tmpADA=Cyc_yyget_YY9(&(yyyvsp[2]).v);_tmp56A->hd=_tmpADA;}),({struct Cyc_List_List*_tmpAD9=Cyc_yyget_YY11(&(yyyvsp[0]).v);_tmp56A->tl=_tmpAD9;});_tmp56A;}));
goto _LL0;case 419U: _LL341: _LL342:
# 2909 "parse.y"
 yyval=Cyc_YY12(({struct _tuple24*_tmp56B=_cycalloc(sizeof(*_tmp56B));_tmp56B->f1=0,({struct Cyc_Absyn_Pat*_tmpADB=Cyc_yyget_YY9(&(yyyvsp[0]).v);_tmp56B->f2=_tmpADB;});_tmp56B;}));
goto _LL0;case 420U: _LL343: _LL344:
# 2911 "parse.y"
 yyval=Cyc_YY12(({struct _tuple24*_tmp56C=_cycalloc(sizeof(*_tmp56C));({struct Cyc_List_List*_tmpADD=Cyc_yyget_YY41(&(yyyvsp[0]).v);_tmp56C->f1=_tmpADD;}),({struct Cyc_Absyn_Pat*_tmpADC=Cyc_yyget_YY9(&(yyyvsp[1]).v);_tmp56C->f2=_tmpADC;});_tmp56C;}));
goto _LL0;case 421U: _LL345: _LL346:
# 2914
 yyval=Cyc_YY14(({struct _tuple23*_tmp56D=_cycalloc(sizeof(*_tmp56D));({struct Cyc_List_List*_tmpADE=Cyc_List_rev(Cyc_yyget_YY13(&(yyyvsp[0]).v));_tmp56D->f1=_tmpADE;}),_tmp56D->f2=0;_tmp56D;}));
goto _LL0;case 422U: _LL347: _LL348:
# 2915 "parse.y"
 yyval=Cyc_YY14(({struct _tuple23*_tmp56E=_cycalloc(sizeof(*_tmp56E));({struct Cyc_List_List*_tmpADF=Cyc_List_rev(Cyc_yyget_YY13(&(yyyvsp[0]).v));_tmp56E->f1=_tmpADF;}),_tmp56E->f2=1;_tmp56E;}));
goto _LL0;case 423U: _LL349: _LL34A:
# 2916 "parse.y"
 yyval=Cyc_YY14(({struct _tuple23*_tmp56F=_cycalloc(sizeof(*_tmp56F));_tmp56F->f1=0,_tmp56F->f2=1;_tmp56F;}));
goto _LL0;case 424U: _LL34B: _LL34C:
# 2921 "parse.y"
 yyval=Cyc_YY13(({struct Cyc_List_List*_tmp570=_cycalloc(sizeof(*_tmp570));({struct _tuple24*_tmpAE0=Cyc_yyget_YY12(&(yyyvsp[0]).v);_tmp570->hd=_tmpAE0;}),_tmp570->tl=0;_tmp570;}));
goto _LL0;case 425U: _LL34D: _LL34E:
# 2923 "parse.y"
 yyval=Cyc_YY13(({struct Cyc_List_List*_tmp571=_cycalloc(sizeof(*_tmp571));({struct _tuple24*_tmpAE2=Cyc_yyget_YY12(&(yyyvsp[2]).v);_tmp571->hd=_tmpAE2;}),({struct Cyc_List_List*_tmpAE1=Cyc_yyget_YY13(&(yyyvsp[0]).v);_tmp571->tl=_tmpAE1;});_tmp571;}));
goto _LL0;case 426U: _LL34F: _LL350:
# 2929 "parse.y"
 yyval=(yyyvsp[0]).v;
goto _LL0;case 427U: _LL351: _LL352:
# 2931 "parse.y"
 yyval=Cyc_Exp_tok(({struct Cyc_Absyn_Exp*_tmpAE4=Cyc_yyget_Exp_tok(&(yyyvsp[0]).v);struct Cyc_Absyn_Exp*_tmpAE3=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);Cyc_Absyn_seq_exp(_tmpAE4,_tmpAE3,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 428U: _LL353: _LL354:
# 2936 "parse.y"
 yyval=(yyyvsp[0]).v;
goto _LL0;case 429U: _LL355: _LL356:
# 2938 "parse.y"
 yyval=Cyc_Exp_tok(({struct Cyc_Absyn_Exp*_tmpAE7=Cyc_yyget_Exp_tok(&(yyyvsp[0]).v);struct Cyc_Core_Opt*_tmpAE6=Cyc_yyget_YY7(&(yyyvsp[1]).v);struct Cyc_Absyn_Exp*_tmpAE5=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);Cyc_Absyn_assignop_exp(_tmpAE7,_tmpAE6,_tmpAE5,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 430U: _LL357: _LL358:
# 2940 "parse.y"
 yyval=Cyc_Exp_tok(({struct Cyc_Absyn_Exp*_tmpAE9=Cyc_yyget_Exp_tok(&(yyyvsp[0]).v);struct Cyc_Absyn_Exp*_tmpAE8=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);Cyc_Absyn_swap_exp(_tmpAE9,_tmpAE8,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 431U: _LL359: _LL35A:
# 2944 "parse.y"
 yyval=Cyc_YY7(0);
goto _LL0;case 432U: _LL35B: _LL35C:
# 2945 "parse.y"
 yyval=Cyc_YY7(({struct Cyc_Core_Opt*_tmp572=_cycalloc(sizeof(*_tmp572));_tmp572->v=(void*)Cyc_Absyn_Times;_tmp572;}));
goto _LL0;case 433U: _LL35D: _LL35E:
# 2946 "parse.y"
 yyval=Cyc_YY7(({struct Cyc_Core_Opt*_tmp573=_cycalloc(sizeof(*_tmp573));_tmp573->v=(void*)Cyc_Absyn_Div;_tmp573;}));
goto _LL0;case 434U: _LL35F: _LL360:
# 2947 "parse.y"
 yyval=Cyc_YY7(({struct Cyc_Core_Opt*_tmp574=_cycalloc(sizeof(*_tmp574));_tmp574->v=(void*)Cyc_Absyn_Mod;_tmp574;}));
goto _LL0;case 435U: _LL361: _LL362:
# 2948 "parse.y"
 yyval=Cyc_YY7(({struct Cyc_Core_Opt*_tmp575=_cycalloc(sizeof(*_tmp575));_tmp575->v=(void*)Cyc_Absyn_Plus;_tmp575;}));
goto _LL0;case 436U: _LL363: _LL364:
# 2949 "parse.y"
 yyval=Cyc_YY7(({struct Cyc_Core_Opt*_tmp576=_cycalloc(sizeof(*_tmp576));_tmp576->v=(void*)Cyc_Absyn_Minus;_tmp576;}));
goto _LL0;case 437U: _LL365: _LL366:
# 2950 "parse.y"
 yyval=Cyc_YY7(({struct Cyc_Core_Opt*_tmp577=_cycalloc(sizeof(*_tmp577));_tmp577->v=(void*)Cyc_Absyn_Bitlshift;_tmp577;}));
goto _LL0;case 438U: _LL367: _LL368:
# 2951 "parse.y"
 yyval=Cyc_YY7(({struct Cyc_Core_Opt*_tmp578=_cycalloc(sizeof(*_tmp578));_tmp578->v=(void*)Cyc_Absyn_Bitlrshift;_tmp578;}));
goto _LL0;case 439U: _LL369: _LL36A:
# 2952 "parse.y"
 yyval=Cyc_YY7(({struct Cyc_Core_Opt*_tmp579=_cycalloc(sizeof(*_tmp579));_tmp579->v=(void*)Cyc_Absyn_Bitand;_tmp579;}));
goto _LL0;case 440U: _LL36B: _LL36C:
# 2953 "parse.y"
 yyval=Cyc_YY7(({struct Cyc_Core_Opt*_tmp57A=_cycalloc(sizeof(*_tmp57A));_tmp57A->v=(void*)Cyc_Absyn_Bitxor;_tmp57A;}));
goto _LL0;case 441U: _LL36D: _LL36E:
# 2954 "parse.y"
 yyval=Cyc_YY7(({struct Cyc_Core_Opt*_tmp57B=_cycalloc(sizeof(*_tmp57B));_tmp57B->v=(void*)Cyc_Absyn_Bitor;_tmp57B;}));
goto _LL0;case 442U: _LL36F: _LL370:
# 2959 "parse.y"
 yyval=(yyyvsp[0]).v;
goto _LL0;case 443U: _LL371: _LL372:
# 2961 "parse.y"
 yyval=Cyc_Exp_tok(({struct Cyc_Absyn_Exp*_tmpAEC=Cyc_yyget_Exp_tok(&(yyyvsp[0]).v);struct Cyc_Absyn_Exp*_tmpAEB=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);struct Cyc_Absyn_Exp*_tmpAEA=Cyc_yyget_Exp_tok(&(yyyvsp[4]).v);Cyc_Absyn_conditional_exp(_tmpAEC,_tmpAEB,_tmpAEA,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 444U: _LL373: _LL374:
# 2964
 yyval=Cyc_Exp_tok(({struct Cyc_Absyn_Exp*_tmpAED=Cyc_yyget_Exp_tok(&(yyyvsp[1]).v);Cyc_Absyn_throw_exp(_tmpAED,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 445U: _LL375: _LL376:
# 2967
 yyval=Cyc_Exp_tok(({struct Cyc_Absyn_Exp*_tmpAEE=Cyc_yyget_Exp_tok(&(yyyvsp[1]).v);Cyc_Absyn_New_exp(0,_tmpAEE,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 446U: _LL377: _LL378:
# 2969 "parse.y"
 yyval=Cyc_Exp_tok(({struct Cyc_Absyn_Exp*_tmpAEF=Cyc_yyget_Exp_tok(&(yyyvsp[1]).v);Cyc_Absyn_New_exp(0,_tmpAEF,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 447U: _LL379: _LL37A:
# 2971 "parse.y"
 yyval=Cyc_Exp_tok(({struct Cyc_Absyn_Exp*_tmpAF1=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);struct Cyc_Absyn_Exp*_tmpAF0=Cyc_yyget_Exp_tok(&(yyyvsp[4]).v);Cyc_Absyn_New_exp(_tmpAF1,_tmpAF0,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 448U: _LL37B: _LL37C:
# 2973 "parse.y"
 yyval=Cyc_Exp_tok(({struct Cyc_Absyn_Exp*_tmpAF3=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);struct Cyc_Absyn_Exp*_tmpAF2=Cyc_yyget_Exp_tok(&(yyyvsp[4]).v);Cyc_Absyn_New_exp(_tmpAF3,_tmpAF2,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 449U: _LL37D: _LL37E:
# 2977 "parse.y"
 yyval=(yyyvsp[0]).v;
goto _LL0;case 450U: _LL37F: _LL380:
# 2981 "parse.y"
 yyval=(yyyvsp[0]).v;
goto _LL0;case 451U: _LL381: _LL382:
# 2983 "parse.y"
 yyval=Cyc_Exp_tok(({struct Cyc_Absyn_Exp*_tmpAF5=Cyc_yyget_Exp_tok(&(yyyvsp[0]).v);struct Cyc_Absyn_Exp*_tmpAF4=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);Cyc_Absyn_or_exp(_tmpAF5,_tmpAF4,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 452U: _LL383: _LL384:
# 2987 "parse.y"
 yyval=(yyyvsp[0]).v;
goto _LL0;case 453U: _LL385: _LL386:
# 2989 "parse.y"
 yyval=Cyc_Exp_tok(({struct Cyc_Absyn_Exp*_tmpAF7=Cyc_yyget_Exp_tok(&(yyyvsp[0]).v);struct Cyc_Absyn_Exp*_tmpAF6=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);Cyc_Absyn_and_exp(_tmpAF7,_tmpAF6,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 454U: _LL387: _LL388:
# 2993 "parse.y"
 yyval=(yyyvsp[0]).v;
goto _LL0;case 455U: _LL389: _LL38A:
# 2995 "parse.y"
 yyval=Cyc_Exp_tok(({struct Cyc_Absyn_Exp*_tmpAF9=Cyc_yyget_Exp_tok(&(yyyvsp[0]).v);struct Cyc_Absyn_Exp*_tmpAF8=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);Cyc_Absyn_prim2_exp(Cyc_Absyn_Bitor,_tmpAF9,_tmpAF8,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 456U: _LL38B: _LL38C:
# 2999 "parse.y"
 yyval=(yyyvsp[0]).v;
goto _LL0;case 457U: _LL38D: _LL38E:
# 3001 "parse.y"
 yyval=Cyc_Exp_tok(({struct Cyc_Absyn_Exp*_tmpAFB=Cyc_yyget_Exp_tok(&(yyyvsp[0]).v);struct Cyc_Absyn_Exp*_tmpAFA=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);Cyc_Absyn_prim2_exp(Cyc_Absyn_Bitxor,_tmpAFB,_tmpAFA,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 458U: _LL38F: _LL390:
# 3005 "parse.y"
 yyval=(yyyvsp[0]).v;
goto _LL0;case 459U: _LL391: _LL392:
# 3007 "parse.y"
 yyval=Cyc_Exp_tok(({struct Cyc_Absyn_Exp*_tmpAFD=Cyc_yyget_Exp_tok(&(yyyvsp[0]).v);struct Cyc_Absyn_Exp*_tmpAFC=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);Cyc_Absyn_prim2_exp(Cyc_Absyn_Bitand,_tmpAFD,_tmpAFC,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 460U: _LL393: _LL394:
# 3011 "parse.y"
 yyval=(yyyvsp[0]).v;
goto _LL0;case 461U: _LL395: _LL396:
# 3013 "parse.y"
 yyval=Cyc_Exp_tok(({struct Cyc_Absyn_Exp*_tmpAFF=Cyc_yyget_Exp_tok(&(yyyvsp[0]).v);struct Cyc_Absyn_Exp*_tmpAFE=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);Cyc_Absyn_eq_exp(_tmpAFF,_tmpAFE,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 462U: _LL397: _LL398:
# 3015 "parse.y"
 yyval=Cyc_Exp_tok(({struct Cyc_Absyn_Exp*_tmpB01=Cyc_yyget_Exp_tok(&(yyyvsp[0]).v);struct Cyc_Absyn_Exp*_tmpB00=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);Cyc_Absyn_neq_exp(_tmpB01,_tmpB00,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 463U: _LL399: _LL39A:
# 3019 "parse.y"
 yyval=(yyyvsp[0]).v;
goto _LL0;case 464U: _LL39B: _LL39C:
# 3021 "parse.y"
 yyval=Cyc_Exp_tok(({struct Cyc_Absyn_Exp*_tmpB03=Cyc_yyget_Exp_tok(&(yyyvsp[0]).v);struct Cyc_Absyn_Exp*_tmpB02=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);Cyc_Absyn_lt_exp(_tmpB03,_tmpB02,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 465U: _LL39D: _LL39E:
# 3023 "parse.y"
 yyval=Cyc_Exp_tok(({struct Cyc_Absyn_Exp*_tmpB05=Cyc_yyget_Exp_tok(&(yyyvsp[0]).v);struct Cyc_Absyn_Exp*_tmpB04=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);Cyc_Absyn_gt_exp(_tmpB05,_tmpB04,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 466U: _LL39F: _LL3A0:
# 3025 "parse.y"
 yyval=Cyc_Exp_tok(({struct Cyc_Absyn_Exp*_tmpB07=Cyc_yyget_Exp_tok(&(yyyvsp[0]).v);struct Cyc_Absyn_Exp*_tmpB06=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);Cyc_Absyn_lte_exp(_tmpB07,_tmpB06,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 467U: _LL3A1: _LL3A2:
# 3027 "parse.y"
 yyval=Cyc_Exp_tok(({struct Cyc_Absyn_Exp*_tmpB09=Cyc_yyget_Exp_tok(&(yyyvsp[0]).v);struct Cyc_Absyn_Exp*_tmpB08=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);Cyc_Absyn_gte_exp(_tmpB09,_tmpB08,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 468U: _LL3A3: _LL3A4:
# 3031 "parse.y"
 yyval=(yyyvsp[0]).v;
goto _LL0;case 469U: _LL3A5: _LL3A6:
# 3033 "parse.y"
 yyval=Cyc_Exp_tok(({struct Cyc_Absyn_Exp*_tmpB0B=Cyc_yyget_Exp_tok(&(yyyvsp[0]).v);struct Cyc_Absyn_Exp*_tmpB0A=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);Cyc_Absyn_prim2_exp(Cyc_Absyn_Bitlshift,_tmpB0B,_tmpB0A,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 470U: _LL3A7: _LL3A8:
# 3035 "parse.y"
 yyval=Cyc_Exp_tok(({struct Cyc_Absyn_Exp*_tmpB0D=Cyc_yyget_Exp_tok(&(yyyvsp[0]).v);struct Cyc_Absyn_Exp*_tmpB0C=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);Cyc_Absyn_prim2_exp(Cyc_Absyn_Bitlrshift,_tmpB0D,_tmpB0C,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 471U: _LL3A9: _LL3AA:
# 3039 "parse.y"
 yyval=(yyyvsp[0]).v;
goto _LL0;case 472U: _LL3AB: _LL3AC:
# 3041 "parse.y"
 yyval=Cyc_Exp_tok(({struct Cyc_Absyn_Exp*_tmpB0F=Cyc_yyget_Exp_tok(&(yyyvsp[0]).v);struct Cyc_Absyn_Exp*_tmpB0E=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);Cyc_Absyn_prim2_exp(Cyc_Absyn_Plus,_tmpB0F,_tmpB0E,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 473U: _LL3AD: _LL3AE:
# 3043 "parse.y"
 yyval=Cyc_Exp_tok(({struct Cyc_Absyn_Exp*_tmpB11=Cyc_yyget_Exp_tok(&(yyyvsp[0]).v);struct Cyc_Absyn_Exp*_tmpB10=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);Cyc_Absyn_prim2_exp(Cyc_Absyn_Minus,_tmpB11,_tmpB10,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 474U: _LL3AF: _LL3B0:
# 3047 "parse.y"
 yyval=(yyyvsp[0]).v;
goto _LL0;case 475U: _LL3B1: _LL3B2:
# 3049 "parse.y"
 yyval=Cyc_Exp_tok(({struct Cyc_Absyn_Exp*_tmpB13=Cyc_yyget_Exp_tok(&(yyyvsp[0]).v);struct Cyc_Absyn_Exp*_tmpB12=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);Cyc_Absyn_prim2_exp(Cyc_Absyn_Times,_tmpB13,_tmpB12,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 476U: _LL3B3: _LL3B4:
# 3051 "parse.y"
 yyval=Cyc_Exp_tok(({struct Cyc_Absyn_Exp*_tmpB15=Cyc_yyget_Exp_tok(&(yyyvsp[0]).v);struct Cyc_Absyn_Exp*_tmpB14=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);Cyc_Absyn_prim2_exp(Cyc_Absyn_Div,_tmpB15,_tmpB14,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 477U: _LL3B5: _LL3B6:
# 3053 "parse.y"
 yyval=Cyc_Exp_tok(({struct Cyc_Absyn_Exp*_tmpB17=Cyc_yyget_Exp_tok(&(yyyvsp[0]).v);struct Cyc_Absyn_Exp*_tmpB16=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);Cyc_Absyn_prim2_exp(Cyc_Absyn_Mod,_tmpB17,_tmpB16,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 478U: _LL3B7: _LL3B8:
# 3057 "parse.y"
 yyval=(yyyvsp[0]).v;
goto _LL0;case 479U: _LL3B9: _LL3BA: {
# 3059 "parse.y"
void*t=({struct _tuple8*_tmpB18=Cyc_yyget_YY37(&(yyyvsp[1]).v);Cyc_Parse_type_name_to_type(_tmpB18,(unsigned)((yyyvsp[1]).l).first_line);});
yyval=Cyc_Exp_tok(({void*_tmpB1A=t;struct Cyc_Absyn_Exp*_tmpB19=Cyc_yyget_Exp_tok(&(yyyvsp[3]).v);Cyc_Absyn_cast_exp(_tmpB1A,_tmpB19,1,Cyc_Absyn_Unknown_coercion,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
# 3062
goto _LL0;}case 480U: _LL3BB: _LL3BC:
# 3065 "parse.y"
 yyval=(yyyvsp[0]).v;
goto _LL0;case 481U: _LL3BD: _LL3BE:
# 3066 "parse.y"
 yyval=Cyc_Exp_tok(({struct Cyc_Absyn_Exp*_tmpB1B=Cyc_yyget_Exp_tok(&(yyyvsp[1]).v);Cyc_Absyn_increment_exp(_tmpB1B,Cyc_Absyn_PreInc,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 482U: _LL3BF: _LL3C0:
# 3067 "parse.y"
 yyval=Cyc_Exp_tok(({struct Cyc_Absyn_Exp*_tmpB1C=Cyc_yyget_Exp_tok(&(yyyvsp[1]).v);Cyc_Absyn_increment_exp(_tmpB1C,Cyc_Absyn_PreDec,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 483U: _LL3C1: _LL3C2:
# 3068 "parse.y"
 yyval=Cyc_Exp_tok(({struct Cyc_Absyn_Exp*_tmpB1D=Cyc_yyget_Exp_tok(&(yyyvsp[1]).v);Cyc_Absyn_address_exp(_tmpB1D,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 484U: _LL3C3: _LL3C4:
# 3069 "parse.y"
 yyval=Cyc_Exp_tok(({struct Cyc_Absyn_Exp*_tmpB1E=Cyc_yyget_Exp_tok(&(yyyvsp[1]).v);Cyc_Absyn_deref_exp(_tmpB1E,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 485U: _LL3C5: _LL3C6:
# 3070 "parse.y"
 yyval=Cyc_Exp_tok(({struct Cyc_Absyn_Exp*_tmpB1F=Cyc_yyget_Exp_tok(&(yyyvsp[1]).v);Cyc_Absyn_prim1_exp(Cyc_Absyn_Plus,_tmpB1F,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 486U: _LL3C7: _LL3C8:
# 3071 "parse.y"
 yyval=Cyc_Exp_tok(({enum Cyc_Absyn_Primop _tmpB21=Cyc_yyget_YY6(&(yyyvsp[0]).v);struct Cyc_Absyn_Exp*_tmpB20=Cyc_yyget_Exp_tok(&(yyyvsp[1]).v);Cyc_Absyn_prim1_exp(_tmpB21,_tmpB20,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 487U: _LL3C9: _LL3CA: {
# 3073 "parse.y"
void*t=({struct _tuple8*_tmpB22=Cyc_yyget_YY37(&(yyyvsp[2]).v);Cyc_Parse_type_name_to_type(_tmpB22,(unsigned)((yyyvsp[2]).l).first_line);});
yyval=Cyc_Exp_tok(({void*_tmpB23=t;Cyc_Absyn_sizeoftype_exp(_tmpB23,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
# 3076
goto _LL0;}case 488U: _LL3CB: _LL3CC:
# 3076 "parse.y"
 yyval=Cyc_Exp_tok(({struct Cyc_Absyn_Exp*_tmpB24=Cyc_yyget_Exp_tok(&(yyyvsp[1]).v);Cyc_Absyn_sizeofexp_exp(_tmpB24,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 489U: _LL3CD: _LL3CE: {
# 3078 "parse.y"
void*t=({struct _tuple8*_tmpB25=Cyc_yyget_YY37(&(yyyvsp[2]).v);Cyc_Parse_type_name_to_type(_tmpB25,(unsigned)((yyyvsp[2]).l).first_line);});
yyval=Cyc_Exp_tok(({void*_tmpB27=t;struct Cyc_List_List*_tmpB26=Cyc_List_imp_rev(Cyc_yyget_YY3(&(yyyvsp[4]).v));Cyc_Absyn_offsetof_exp(_tmpB27,_tmpB26,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
# 3081
goto _LL0;}case 490U: _LL3CF: _LL3D0:
# 3083
 yyval=Cyc_Exp_tok(({void*_tmpB2A=(void*)({struct Cyc_Absyn_Spawn_e_Absyn_Raw_exp_struct*_tmp57C=_cycalloc(sizeof(*_tmp57C));_tmp57C->tag=34U,({struct Cyc_Absyn_Exp*_tmpB29=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);_tmp57C->f1=_tmpB29;}),({struct Cyc_Absyn_Exp*_tmpB28=Cyc_yyget_Exp_tok(&(yyyvsp[4]).v);_tmp57C->f2=_tmpB28;});_tmp57C;});Cyc_Absyn_new_exp(_tmpB2A,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 491U: _LL3D1: _LL3D2:
# 3086
 yyval=Cyc_Exp_tok(({void*_tmpB2C=(void*)({struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct*_tmp57D=_cycalloc(sizeof(*_tmp57D));_tmp57D->tag=35U,(_tmp57D->f1).is_calloc=0,(_tmp57D->f1).rgn=0,(_tmp57D->f1).elt_type=0,({struct Cyc_Absyn_Exp*_tmpB2B=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);(_tmp57D->f1).num_elts=_tmpB2B;}),(_tmp57D->f1).fat_result=0,(_tmp57D->f1).inline_call=0;_tmp57D;});Cyc_Absyn_new_exp(_tmpB2C,(unsigned)({
yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 492U: _LL3D3: _LL3D4:
# 3089 "parse.y"
 yyval=Cyc_Exp_tok(({void*_tmpB2F=(void*)({struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct*_tmp57E=_cycalloc(sizeof(*_tmp57E));_tmp57E->tag=35U,(_tmp57E->f1).is_calloc=0,({struct Cyc_Absyn_Exp*_tmpB2E=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);(_tmp57E->f1).rgn=_tmpB2E;}),(_tmp57E->f1).elt_type=0,({struct Cyc_Absyn_Exp*_tmpB2D=Cyc_yyget_Exp_tok(&(yyyvsp[4]).v);(_tmp57E->f1).num_elts=_tmpB2D;}),(_tmp57E->f1).fat_result=0,(_tmp57E->f1).inline_call=0;_tmp57E;});Cyc_Absyn_new_exp(_tmpB2F,(unsigned)({
yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 493U: _LL3D5: _LL3D6:
# 3092 "parse.y"
 yyval=Cyc_Exp_tok(({void*_tmpB32=(void*)({struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct*_tmp57F=_cycalloc(sizeof(*_tmp57F));_tmp57F->tag=35U,(_tmp57F->f1).is_calloc=0,({struct Cyc_Absyn_Exp*_tmpB31=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);(_tmp57F->f1).rgn=_tmpB31;}),(_tmp57F->f1).elt_type=0,({struct Cyc_Absyn_Exp*_tmpB30=Cyc_yyget_Exp_tok(&(yyyvsp[4]).v);(_tmp57F->f1).num_elts=_tmpB30;}),(_tmp57F->f1).fat_result=0,(_tmp57F->f1).inline_call=1;_tmp57F;});Cyc_Absyn_new_exp(_tmpB32,(unsigned)({
yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 494U: _LL3D7: _LL3D8: {
# 3095 "parse.y"
void*t=({struct _tuple8*_tmpB33=Cyc_yyget_YY37(&(yyyvsp[6]).v);Cyc_Parse_type_name_to_type(_tmpB33,(unsigned)((yyyvsp[6]).l).first_line);});
yyval=Cyc_Exp_tok(({void*_tmpB36=(void*)({struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct*_tmp581=_cycalloc(sizeof(*_tmp581));_tmp581->tag=35U,(_tmp581->f1).is_calloc=1,(_tmp581->f1).rgn=0,({void**_tmpB35=({void**_tmp580=_cycalloc(sizeof(*_tmp580));*_tmp580=t;_tmp580;});(_tmp581->f1).elt_type=_tmpB35;}),({struct Cyc_Absyn_Exp*_tmpB34=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);(_tmp581->f1).num_elts=_tmpB34;}),(_tmp581->f1).fat_result=0,(_tmp581->f1).inline_call=0;_tmp581;});Cyc_Absyn_new_exp(_tmpB36,(unsigned)({
yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;}case 495U: _LL3D9: _LL3DA: {
# 3100
void*t=({struct _tuple8*_tmpB37=Cyc_yyget_YY37(&(yyyvsp[8]).v);Cyc_Parse_type_name_to_type(_tmpB37,(unsigned)((yyyvsp[8]).l).first_line);});
yyval=Cyc_Exp_tok(({void*_tmpB3B=(void*)({struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct*_tmp583=_cycalloc(sizeof(*_tmp583));_tmp583->tag=35U,(_tmp583->f1).is_calloc=1,({struct Cyc_Absyn_Exp*_tmpB3A=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);(_tmp583->f1).rgn=_tmpB3A;}),({void**_tmpB39=({void**_tmp582=_cycalloc(sizeof(*_tmp582));*_tmp582=t;_tmp582;});(_tmp583->f1).elt_type=_tmpB39;}),({struct Cyc_Absyn_Exp*_tmpB38=Cyc_yyget_Exp_tok(&(yyyvsp[4]).v);(_tmp583->f1).num_elts=_tmpB38;}),(_tmp583->f1).fat_result=0,(_tmp583->f1).inline_call=0;_tmp583;});Cyc_Absyn_new_exp(_tmpB3B,(unsigned)({
yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;}case 496U: _LL3DB: _LL3DC:
# 3104 "parse.y"
 yyval=Cyc_Exp_tok(({struct Cyc_List_List*_tmpB3D=({struct Cyc_Absyn_Exp*_tmp584[1U];({struct Cyc_Absyn_Exp*_tmpB3C=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);_tmp584[0]=_tmpB3C;});Cyc_List_list(_tag_fat(_tmp584,sizeof(struct Cyc_Absyn_Exp*),1U));});Cyc_Absyn_primop_exp(Cyc_Absyn_Numelts,_tmpB3D,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 497U: _LL3DD: _LL3DE:
# 3106 "parse.y"
 yyval=Cyc_Exp_tok(({void*_tmpB41=(void*)({struct Cyc_Absyn_Tagcheck_e_Absyn_Raw_exp_struct*_tmp586=_cycalloc(sizeof(*_tmp586));_tmp586->tag=39U,({struct Cyc_Absyn_Exp*_tmpB40=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);_tmp586->f1=_tmpB40;}),({struct _fat_ptr*_tmpB3F=({struct _fat_ptr*_tmp585=_cycalloc(sizeof(*_tmp585));({struct _fat_ptr _tmpB3E=Cyc_yyget_String_tok(&(yyyvsp[4]).v);*_tmp585=_tmpB3E;});_tmp585;});_tmp586->f2=_tmpB3F;});_tmp586;});Cyc_Absyn_new_exp(_tmpB41,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 498U: _LL3DF: _LL3E0:
# 3108 "parse.y"
 yyval=Cyc_Exp_tok(({void*_tmpB46=(void*)({struct Cyc_Absyn_Tagcheck_e_Absyn_Raw_exp_struct*_tmp588=_cycalloc(sizeof(*_tmp588));_tmp588->tag=39U,({struct Cyc_Absyn_Exp*_tmpB45=({struct Cyc_Absyn_Exp*_tmpB44=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);Cyc_Absyn_deref_exp(_tmpB44,(unsigned)((yyyvsp[2]).l).first_line);});_tmp588->f1=_tmpB45;}),({struct _fat_ptr*_tmpB43=({struct _fat_ptr*_tmp587=_cycalloc(sizeof(*_tmp587));({struct _fat_ptr _tmpB42=Cyc_yyget_String_tok(&(yyyvsp[4]).v);*_tmp587=_tmpB42;});_tmp587;});_tmp588->f2=_tmpB43;});_tmp588;});Cyc_Absyn_new_exp(_tmpB46,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 499U: _LL3E1: _LL3E2: {
# 3110 "parse.y"
void*t=({struct _tuple8*_tmpB47=Cyc_yyget_YY37(&(yyyvsp[2]).v);Cyc_Parse_type_name_to_type(_tmpB47,(unsigned)((yyyvsp[2]).l).first_line);});
yyval=Cyc_Exp_tok(({void*_tmpB48=t;Cyc_Absyn_valueof_exp(_tmpB48,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;}case 500U: _LL3E3: _LL3E4:
# 3113 "parse.y"
 yyval=Cyc_Exp_tok(({void*_tmpB49=Cyc_yyget_YY58(&(yyyvsp[1]).v);Cyc_Absyn_new_exp(_tmpB49,(unsigned)((yyyvsp[0]).l).first_line);}));
goto _LL0;case 501U: _LL3E5: _LL3E6:
# 3114 "parse.y"
 yyval=Cyc_Exp_tok(({struct Cyc_Absyn_Exp*_tmpB4A=Cyc_yyget_Exp_tok(&(yyyvsp[1]).v);Cyc_Absyn_extension_exp(_tmpB4A,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 502U: _LL3E7: _LL3E8: {
# 3119 "parse.y"
struct _tuple29*_stmttmp2F=Cyc_yyget_YY59(&(yyyvsp[3]).v);struct Cyc_List_List*_tmp58B;struct Cyc_List_List*_tmp58A;struct Cyc_List_List*_tmp589;_LL4C6: _tmp589=_stmttmp2F->f1;_tmp58A=_stmttmp2F->f2;_tmp58B=_stmttmp2F->f3;_LL4C7: {struct Cyc_List_List*outlist=_tmp589;struct Cyc_List_List*inlist=_tmp58A;struct Cyc_List_List*clobbers=_tmp58B;
yyval=Cyc_YY58((void*)({struct Cyc_Absyn_Asm_e_Absyn_Raw_exp_struct*_tmp58C=_cycalloc(sizeof(*_tmp58C));_tmp58C->tag=41U,({int _tmpB4C=Cyc_yyget_YY31(&(yyyvsp[0]).v);_tmp58C->f1=_tmpB4C;}),({struct _fat_ptr _tmpB4B=Cyc_yyget_String_tok(&(yyyvsp[2]).v);_tmp58C->f2=_tmpB4B;}),_tmp58C->f3=outlist,_tmp58C->f4=inlist,_tmp58C->f5=clobbers;_tmp58C;}));
goto _LL0;}}case 503U: _LL3E9: _LL3EA:
# 3124 "parse.y"
 yyval=Cyc_YY31(0);
goto _LL0;case 504U: _LL3EB: _LL3EC:
# 3125 "parse.y"
 yyval=Cyc_YY31(1);
goto _LL0;case 505U: _LL3ED: _LL3EE:
# 3129 "parse.y"
 yyval=Cyc_YY59(({struct _tuple29*_tmp58D=_cycalloc(sizeof(*_tmp58D));_tmp58D->f1=0,_tmp58D->f2=0,_tmp58D->f3=0;_tmp58D;}));
goto _LL0;case 506U: _LL3EF: _LL3F0: {
# 3131 "parse.y"
struct _tuple30*_stmttmp30=Cyc_yyget_YY60(&(yyyvsp[1]).v);struct Cyc_List_List*_tmp58F;struct Cyc_List_List*_tmp58E;_LL4C9: _tmp58E=_stmttmp30->f1;_tmp58F=_stmttmp30->f2;_LL4CA: {struct Cyc_List_List*inlist=_tmp58E;struct Cyc_List_List*clobbers=_tmp58F;
yyval=Cyc_YY59(({struct _tuple29*_tmp590=_cycalloc(sizeof(*_tmp590));_tmp590->f1=0,_tmp590->f2=inlist,_tmp590->f3=clobbers;_tmp590;}));
goto _LL0;}}case 507U: _LL3F1: _LL3F2: {
# 3134 "parse.y"
struct _tuple30*_stmttmp31=Cyc_yyget_YY60(&(yyyvsp[2]).v);struct Cyc_List_List*_tmp592;struct Cyc_List_List*_tmp591;_LL4CC: _tmp591=_stmttmp31->f1;_tmp592=_stmttmp31->f2;_LL4CD: {struct Cyc_List_List*inlist=_tmp591;struct Cyc_List_List*clobbers=_tmp592;
yyval=Cyc_YY59(({struct _tuple29*_tmp593=_cycalloc(sizeof(*_tmp593));({struct Cyc_List_List*_tmpB4D=Cyc_List_imp_rev(Cyc_yyget_YY62(&(yyyvsp[1]).v));_tmp593->f1=_tmpB4D;}),_tmp593->f2=inlist,_tmp593->f3=clobbers;_tmp593;}));
goto _LL0;}}case 508U: _LL3F3: _LL3F4:
# 3139 "parse.y"
 yyval=Cyc_YY62(({struct Cyc_List_List*_tmp594=_cycalloc(sizeof(*_tmp594));({struct _tuple31*_tmpB4E=Cyc_yyget_YY63(&(yyyvsp[0]).v);_tmp594->hd=_tmpB4E;}),_tmp594->tl=0;_tmp594;}));
goto _LL0;case 509U: _LL3F5: _LL3F6:
# 3140 "parse.y"
 yyval=Cyc_YY62(({struct Cyc_List_List*_tmp595=_cycalloc(sizeof(*_tmp595));({struct _tuple31*_tmpB50=Cyc_yyget_YY63(&(yyyvsp[2]).v);_tmp595->hd=_tmpB50;}),({struct Cyc_List_List*_tmpB4F=Cyc_yyget_YY62(&(yyyvsp[0]).v);_tmp595->tl=_tmpB4F;});_tmp595;}));
goto _LL0;case 510U: _LL3F7: _LL3F8:
# 3144 "parse.y"
 yyval=Cyc_YY60(({struct _tuple30*_tmp596=_cycalloc(sizeof(*_tmp596));_tmp596->f1=0,_tmp596->f2=0;_tmp596;}));
goto _LL0;case 511U: _LL3F9: _LL3FA:
# 3146 "parse.y"
 yyval=Cyc_YY60(({struct _tuple30*_tmp597=_cycalloc(sizeof(*_tmp597));_tmp597->f1=0,({struct Cyc_List_List*_tmpB51=Cyc_yyget_YY61(&(yyyvsp[1]).v);_tmp597->f2=_tmpB51;});_tmp597;}));
goto _LL0;case 512U: _LL3FB: _LL3FC:
# 3148 "parse.y"
 yyval=Cyc_YY60(({struct _tuple30*_tmp598=_cycalloc(sizeof(*_tmp598));({struct Cyc_List_List*_tmpB53=Cyc_List_imp_rev(Cyc_yyget_YY62(&(yyyvsp[1]).v));_tmp598->f1=_tmpB53;}),({struct Cyc_List_List*_tmpB52=Cyc_yyget_YY61(&(yyyvsp[2]).v);_tmp598->f2=_tmpB52;});_tmp598;}));
goto _LL0;case 513U: _LL3FD: _LL3FE:
# 3152 "parse.y"
 yyval=Cyc_YY62(({struct Cyc_List_List*_tmp599=_cycalloc(sizeof(*_tmp599));({struct _tuple31*_tmpB54=Cyc_yyget_YY63(&(yyyvsp[0]).v);_tmp599->hd=_tmpB54;}),_tmp599->tl=0;_tmp599;}));
goto _LL0;case 514U: _LL3FF: _LL400:
# 3153 "parse.y"
 yyval=Cyc_YY62(({struct Cyc_List_List*_tmp59A=_cycalloc(sizeof(*_tmp59A));({struct _tuple31*_tmpB56=Cyc_yyget_YY63(&(yyyvsp[2]).v);_tmp59A->hd=_tmpB56;}),({struct Cyc_List_List*_tmpB55=Cyc_yyget_YY62(&(yyyvsp[0]).v);_tmp59A->tl=_tmpB55;});_tmp59A;}));
goto _LL0;case 515U: _LL401: _LL402: {
# 3158 "parse.y"
struct Cyc_Absyn_Exp*pf_exp=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);
yyval=Cyc_YY63(({struct _tuple31*_tmp59B=_cycalloc(sizeof(*_tmp59B));({struct _fat_ptr _tmpB58=Cyc_yyget_String_tok(&(yyyvsp[0]).v);_tmp59B->f1=_tmpB58;}),({struct Cyc_Absyn_Exp*_tmpB57=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);_tmp59B->f2=_tmpB57;});_tmp59B;}));
goto _LL0;}case 516U: _LL403: _LL404:
# 3164 "parse.y"
 yyval=Cyc_YY61(0);
goto _LL0;case 517U: _LL405: _LL406:
# 3165 "parse.y"
 yyval=Cyc_YY61(0);
goto _LL0;case 518U: _LL407: _LL408:
# 3166 "parse.y"
 yyval=Cyc_YY61(Cyc_List_imp_rev(Cyc_yyget_YY61(&(yyyvsp[1]).v)));
goto _LL0;case 519U: _LL409: _LL40A:
# 3170 "parse.y"
 yyval=Cyc_YY61(({struct Cyc_List_List*_tmp59D=_cycalloc(sizeof(*_tmp59D));({struct _fat_ptr*_tmpB5A=({struct _fat_ptr*_tmp59C=_cycalloc(sizeof(*_tmp59C));({struct _fat_ptr _tmpB59=Cyc_yyget_String_tok(&(yyyvsp[0]).v);*_tmp59C=_tmpB59;});_tmp59C;});_tmp59D->hd=_tmpB5A;}),_tmp59D->tl=0;_tmp59D;}));
goto _LL0;case 520U: _LL40B: _LL40C:
# 3171 "parse.y"
 yyval=Cyc_YY61(({struct Cyc_List_List*_tmp59F=_cycalloc(sizeof(*_tmp59F));({struct _fat_ptr*_tmpB5D=({struct _fat_ptr*_tmp59E=_cycalloc(sizeof(*_tmp59E));({struct _fat_ptr _tmpB5C=Cyc_yyget_String_tok(&(yyyvsp[2]).v);*_tmp59E=_tmpB5C;});_tmp59E;});_tmp59F->hd=_tmpB5D;}),({struct Cyc_List_List*_tmpB5B=Cyc_yyget_YY61(&(yyyvsp[0]).v);_tmp59F->tl=_tmpB5B;});_tmp59F;}));
goto _LL0;case 521U: _LL40D: _LL40E:
# 3175 "parse.y"
 yyval=Cyc_YY6(Cyc_Absyn_Bitnot);
goto _LL0;case 522U: _LL40F: _LL410:
# 3176 "parse.y"
 yyval=Cyc_YY6(Cyc_Absyn_Not);
goto _LL0;case 523U: _LL411: _LL412:
# 3177 "parse.y"
 yyval=Cyc_YY6(Cyc_Absyn_Minus);
goto _LL0;case 524U: _LL413: _LL414:
# 3182 "parse.y"
 yyval=(yyyvsp[0]).v;
goto _LL0;case 525U: _LL415: _LL416:
# 3184 "parse.y"
 yyval=Cyc_Exp_tok(({struct Cyc_Absyn_Exp*_tmpB5F=Cyc_yyget_Exp_tok(&(yyyvsp[0]).v);struct Cyc_Absyn_Exp*_tmpB5E=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);Cyc_Absyn_subscript_exp(_tmpB5F,_tmpB5E,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 526U: _LL417: _LL418:
# 3186 "parse.y"
 yyval=Cyc_Exp_tok(({struct Cyc_Absyn_Exp*_tmpB60=Cyc_yyget_Exp_tok(&(yyyvsp[0]).v);Cyc_Absyn_unknowncall_exp(_tmpB60,0,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 527U: _LL419: _LL41A:
# 3188 "parse.y"
 yyval=Cyc_Exp_tok(({struct Cyc_Absyn_Exp*_tmpB62=Cyc_yyget_Exp_tok(&(yyyvsp[0]).v);struct Cyc_List_List*_tmpB61=Cyc_yyget_YY4(&(yyyvsp[2]).v);Cyc_Absyn_unknowncall_exp(_tmpB62,_tmpB61,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 528U: _LL41B: _LL41C:
# 3190 "parse.y"
 yyval=Cyc_Exp_tok(({struct Cyc_Absyn_Exp*_tmpB65=Cyc_yyget_Exp_tok(&(yyyvsp[0]).v);struct _fat_ptr*_tmpB64=({struct _fat_ptr*_tmp5A0=_cycalloc(sizeof(*_tmp5A0));({struct _fat_ptr _tmpB63=Cyc_yyget_String_tok(&(yyyvsp[2]).v);*_tmp5A0=_tmpB63;});_tmp5A0;});Cyc_Absyn_aggrmember_exp(_tmpB65,_tmpB64,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 529U: _LL41D: _LL41E:
# 3192 "parse.y"
 yyval=Cyc_Exp_tok(({struct Cyc_Absyn_Exp*_tmpB68=Cyc_yyget_Exp_tok(&(yyyvsp[0]).v);struct _fat_ptr*_tmpB67=({struct _fat_ptr*_tmp5A1=_cycalloc(sizeof(*_tmp5A1));({struct _fat_ptr _tmpB66=Cyc_yyget_String_tok(&(yyyvsp[2]).v);*_tmp5A1=_tmpB66;});_tmp5A1;});Cyc_Absyn_aggrarrow_exp(_tmpB68,_tmpB67,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 530U: _LL41F: _LL420:
# 3194 "parse.y"
 yyval=Cyc_Exp_tok(({struct Cyc_Absyn_Exp*_tmpB69=Cyc_yyget_Exp_tok(&(yyyvsp[0]).v);Cyc_Absyn_increment_exp(_tmpB69,Cyc_Absyn_PostInc,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 531U: _LL421: _LL422:
# 3196 "parse.y"
 yyval=Cyc_Exp_tok(({struct Cyc_Absyn_Exp*_tmpB6A=Cyc_yyget_Exp_tok(&(yyyvsp[0]).v);Cyc_Absyn_increment_exp(_tmpB6A,Cyc_Absyn_PostDec,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 532U: _LL423: _LL424:
# 3198 "parse.y"
 yyval=Cyc_Exp_tok(({void*_tmpB6C=(void*)({struct Cyc_Absyn_CompoundLit_e_Absyn_Raw_exp_struct*_tmp5A2=_cycalloc(sizeof(*_tmp5A2));_tmp5A2->tag=25U,({struct _tuple8*_tmpB6B=Cyc_yyget_YY37(&(yyyvsp[1]).v);_tmp5A2->f1=_tmpB6B;}),_tmp5A2->f2=0;_tmp5A2;});Cyc_Absyn_new_exp(_tmpB6C,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 533U: _LL425: _LL426:
# 3200 "parse.y"
 yyval=Cyc_Exp_tok(({void*_tmpB6F=(void*)({struct Cyc_Absyn_CompoundLit_e_Absyn_Raw_exp_struct*_tmp5A3=_cycalloc(sizeof(*_tmp5A3));_tmp5A3->tag=25U,({struct _tuple8*_tmpB6E=Cyc_yyget_YY37(&(yyyvsp[1]).v);_tmp5A3->f1=_tmpB6E;}),({struct Cyc_List_List*_tmpB6D=Cyc_List_imp_rev(Cyc_yyget_YY5(&(yyyvsp[4]).v));_tmp5A3->f2=_tmpB6D;});_tmp5A3;});Cyc_Absyn_new_exp(_tmpB6F,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 534U: _LL427: _LL428:
# 3202 "parse.y"
 yyval=Cyc_Exp_tok(({void*_tmpB72=(void*)({struct Cyc_Absyn_CompoundLit_e_Absyn_Raw_exp_struct*_tmp5A4=_cycalloc(sizeof(*_tmp5A4));_tmp5A4->tag=25U,({struct _tuple8*_tmpB71=Cyc_yyget_YY37(&(yyyvsp[1]).v);_tmp5A4->f1=_tmpB71;}),({struct Cyc_List_List*_tmpB70=Cyc_List_imp_rev(Cyc_yyget_YY5(&(yyyvsp[4]).v));_tmp5A4->f2=_tmpB70;});_tmp5A4;});Cyc_Absyn_new_exp(_tmpB72,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 535U: _LL429: _LL42A:
# 3207 "parse.y"
 yyval=Cyc_YY3(({struct Cyc_List_List*_tmp5A7=_cycalloc(sizeof(*_tmp5A7));({void*_tmpB75=(void*)({struct Cyc_Absyn_StructField_Absyn_OffsetofField_struct*_tmp5A6=_cycalloc(sizeof(*_tmp5A6));_tmp5A6->tag=0U,({struct _fat_ptr*_tmpB74=({struct _fat_ptr*_tmp5A5=_cycalloc(sizeof(*_tmp5A5));({struct _fat_ptr _tmpB73=Cyc_yyget_String_tok(&(yyyvsp[0]).v);*_tmp5A5=_tmpB73;});_tmp5A5;});_tmp5A6->f1=_tmpB74;});_tmp5A6;});_tmp5A7->hd=_tmpB75;}),_tmp5A7->tl=0;_tmp5A7;}));
goto _LL0;case 536U: _LL42B: _LL42C:
# 3210
 yyval=Cyc_YY3(({struct Cyc_List_List*_tmp5A9=_cycalloc(sizeof(*_tmp5A9));({void*_tmpB78=(void*)({struct Cyc_Absyn_TupleIndex_Absyn_OffsetofField_struct*_tmp5A8=_cycalloc(sizeof(*_tmp5A8));_tmp5A8->tag=1U,({unsigned _tmpB77=({unsigned _tmpB76=(unsigned)((yyyvsp[0]).l).first_line;Cyc_Parse_cnst2uint(_tmpB76,Cyc_yyget_Int_tok(&(yyyvsp[0]).v));});_tmp5A8->f1=_tmpB77;});_tmp5A8;});_tmp5A9->hd=_tmpB78;}),_tmp5A9->tl=0;_tmp5A9;}));
goto _LL0;case 537U: _LL42D: _LL42E:
# 3212 "parse.y"
 yyval=Cyc_YY3(({struct Cyc_List_List*_tmp5AC=_cycalloc(sizeof(*_tmp5AC));({void*_tmpB7C=(void*)({struct Cyc_Absyn_StructField_Absyn_OffsetofField_struct*_tmp5AB=_cycalloc(sizeof(*_tmp5AB));_tmp5AB->tag=0U,({struct _fat_ptr*_tmpB7B=({struct _fat_ptr*_tmp5AA=_cycalloc(sizeof(*_tmp5AA));({struct _fat_ptr _tmpB7A=Cyc_yyget_String_tok(&(yyyvsp[2]).v);*_tmp5AA=_tmpB7A;});_tmp5AA;});_tmp5AB->f1=_tmpB7B;});_tmp5AB;});_tmp5AC->hd=_tmpB7C;}),({struct Cyc_List_List*_tmpB79=Cyc_yyget_YY3(&(yyyvsp[0]).v);_tmp5AC->tl=_tmpB79;});_tmp5AC;}));
goto _LL0;case 538U: _LL42F: _LL430:
# 3215
 yyval=Cyc_YY3(({struct Cyc_List_List*_tmp5AE=_cycalloc(sizeof(*_tmp5AE));({void*_tmpB80=(void*)({struct Cyc_Absyn_TupleIndex_Absyn_OffsetofField_struct*_tmp5AD=_cycalloc(sizeof(*_tmp5AD));_tmp5AD->tag=1U,({unsigned _tmpB7F=({unsigned _tmpB7E=(unsigned)((yyyvsp[2]).l).first_line;Cyc_Parse_cnst2uint(_tmpB7E,Cyc_yyget_Int_tok(&(yyyvsp[2]).v));});_tmp5AD->f1=_tmpB7F;});_tmp5AD;});_tmp5AE->hd=_tmpB80;}),({struct Cyc_List_List*_tmpB7D=Cyc_yyget_YY3(&(yyyvsp[0]).v);_tmp5AE->tl=_tmpB7D;});_tmp5AE;}));
goto _LL0;case 539U: _LL431: _LL432:
# 3221 "parse.y"
 yyval=Cyc_Exp_tok(({struct _tuple0*_tmpB81=Cyc_yyget_QualId_tok(&(yyyvsp[0]).v);Cyc_Absyn_unknownid_exp(_tmpB81,(unsigned)((yyyvsp[0]).l).first_line);}));
goto _LL0;case 540U: _LL433: _LL434:
# 3223 "parse.y"
 yyval=Cyc_Exp_tok(({struct _fat_ptr _tmpB82=Cyc_yyget_String_tok(&(yyyvsp[2]).v);Cyc_Absyn_pragma_exp(_tmpB82,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 541U: _LL435: _LL436:
# 3225 "parse.y"
 yyval=(yyyvsp[0]).v;
goto _LL0;case 542U: _LL437: _LL438:
# 3227 "parse.y"
 yyval=Cyc_Exp_tok(({struct _fat_ptr _tmpB83=Cyc_yyget_String_tok(&(yyyvsp[0]).v);Cyc_Absyn_string_exp(_tmpB83,(unsigned)((yyyvsp[0]).l).first_line);}));
goto _LL0;case 543U: _LL439: _LL43A:
# 3229 "parse.y"
 yyval=Cyc_Exp_tok(({struct _fat_ptr _tmpB84=Cyc_yyget_String_tok(&(yyyvsp[0]).v);Cyc_Absyn_wstring_exp(_tmpB84,(unsigned)((yyyvsp[0]).l).first_line);}));
goto _LL0;case 544U: _LL43B: _LL43C:
# 3231 "parse.y"
 yyval=(yyyvsp[1]).v;
goto _LL0;case 545U: _LL43D: _LL43E:
# 3236 "parse.y"
 yyval=Cyc_Exp_tok(({struct Cyc_Absyn_Exp*_tmpB85=Cyc_yyget_Exp_tok(&(yyyvsp[0]).v);Cyc_Absyn_noinstantiate_exp(_tmpB85,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 546U: _LL43F: _LL440:
# 3238 "parse.y"
 yyval=Cyc_Exp_tok(({struct Cyc_Absyn_Exp*_tmpB87=Cyc_yyget_Exp_tok(&(yyyvsp[0]).v);struct Cyc_List_List*_tmpB86=Cyc_List_imp_rev(Cyc_yyget_YY40(&(yyyvsp[3]).v));Cyc_Absyn_instantiate_exp(_tmpB87,_tmpB86,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 547U: _LL441: _LL442:
# 3241
 yyval=Cyc_Exp_tok(({struct Cyc_List_List*_tmpB88=Cyc_yyget_YY4(&(yyyvsp[2]).v);Cyc_Absyn_tuple_exp(_tmpB88,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 548U: _LL443: _LL444:
# 3244
 yyval=Cyc_Exp_tok(({void*_tmpB8C=(void*)({struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct*_tmp5AF=_cycalloc(sizeof(*_tmp5AF));_tmp5AF->tag=29U,({struct _tuple0*_tmpB8B=Cyc_yyget_QualId_tok(&(yyyvsp[0]).v);_tmp5AF->f1=_tmpB8B;}),({struct Cyc_List_List*_tmpB8A=Cyc_yyget_YY40(&(yyyvsp[2]).v);_tmp5AF->f2=_tmpB8A;}),({struct Cyc_List_List*_tmpB89=Cyc_List_imp_rev(Cyc_yyget_YY5(&(yyyvsp[3]).v));_tmp5AF->f3=_tmpB89;}),_tmp5AF->f4=0;_tmp5AF;});Cyc_Absyn_new_exp(_tmpB8C,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 549U: _LL445: _LL446:
# 3247
 yyval=Cyc_Exp_tok(({struct Cyc_Absyn_Stmt*_tmpB8D=Cyc_yyget_Stmt_tok(&(yyyvsp[2]).v);Cyc_Absyn_stmt_exp(_tmpB8D,(unsigned)({yylloc.first_line=((yyyvsp[0]).l).first_line;((yyyvsp[0]).l).first_line;}));}));
goto _LL0;case 550U: _LL447: _LL448:
# 3251 "parse.y"
 yyval=Cyc_YY4(Cyc_List_imp_rev(Cyc_yyget_YY4(&(yyyvsp[0]).v)));
goto _LL0;case 551U: _LL449: _LL44A:
# 3257 "parse.y"
 yyval=Cyc_YY4(({struct Cyc_List_List*_tmp5B0=_cycalloc(sizeof(*_tmp5B0));({struct Cyc_Absyn_Exp*_tmpB8E=Cyc_yyget_Exp_tok(&(yyyvsp[0]).v);_tmp5B0->hd=_tmpB8E;}),_tmp5B0->tl=0;_tmp5B0;}));
goto _LL0;case 552U: _LL44B: _LL44C:
# 3259 "parse.y"
 yyval=Cyc_YY4(({struct Cyc_List_List*_tmp5B1=_cycalloc(sizeof(*_tmp5B1));({struct Cyc_Absyn_Exp*_tmpB90=Cyc_yyget_Exp_tok(&(yyyvsp[2]).v);_tmp5B1->hd=_tmpB90;}),({struct Cyc_List_List*_tmpB8F=Cyc_yyget_YY4(&(yyyvsp[0]).v);_tmp5B1->tl=_tmpB8F;});_tmp5B1;}));
goto _LL0;case 553U: _LL44D: _LL44E:
# 3265 "parse.y"
 yyval=Cyc_Exp_tok(({union Cyc_Absyn_Cnst _tmpB91=Cyc_yyget_Int_tok(&(yyyvsp[0]).v);Cyc_Absyn_const_exp(_tmpB91,(unsigned)((yyyvsp[0]).l).first_line);}));
goto _LL0;case 554U: _LL44F: _LL450:
# 3266 "parse.y"
 yyval=Cyc_Exp_tok(({char _tmpB92=Cyc_yyget_Char_tok(&(yyyvsp[0]).v);Cyc_Absyn_char_exp(_tmpB92,(unsigned)((yyyvsp[0]).l).first_line);}));
goto _LL0;case 555U: _LL451: _LL452:
# 3267 "parse.y"
 yyval=Cyc_Exp_tok(({struct _fat_ptr _tmpB93=Cyc_yyget_String_tok(&(yyyvsp[0]).v);Cyc_Absyn_wchar_exp(_tmpB93,(unsigned)((yyyvsp[0]).l).first_line);}));
goto _LL0;case 556U: _LL453: _LL454: {
# 3269 "parse.y"
struct _fat_ptr f=Cyc_yyget_String_tok(&(yyyvsp[0]).v);
int l=(int)Cyc_strlen((struct _fat_ptr)f);
int i=1;
if(l > 0){
char c=*((const char*)_check_fat_subscript(f,sizeof(char),l - 1));
if((int)c == (int)'f' ||(int)c == (int)'F')i=0;else{
if((int)c == (int)'l' ||(int)c == (int)'L')i=2;}}
# 3272
yyval=
# 3277
Cyc_Exp_tok(Cyc_Absyn_float_exp(f,i,(unsigned)((yyyvsp[0]).l).first_line));
# 3279
goto _LL0;}case 557U: _LL455: _LL456:
# 3280 "parse.y"
 yyval=Cyc_Exp_tok(Cyc_Absyn_null_exp((unsigned)((yyyvsp[0]).l).first_line));
goto _LL0;case 558U: _LL457: _LL458:
# 3284 "parse.y"
 yyval=Cyc_QualId_tok(({struct _tuple0*_tmp5B3=_cycalloc(sizeof(*_tmp5B3));({union Cyc_Absyn_Nmspace _tmpB96=Cyc_Absyn_Rel_n(0);_tmp5B3->f1=_tmpB96;}),({struct _fat_ptr*_tmpB95=({struct _fat_ptr*_tmp5B2=_cycalloc(sizeof(*_tmp5B2));({struct _fat_ptr _tmpB94=Cyc_yyget_String_tok(&(yyyvsp[0]).v);*_tmp5B2=_tmpB94;});_tmp5B2;});_tmp5B3->f2=_tmpB95;});_tmp5B3;}));
goto _LL0;case 559U: _LL459: _LL45A:
# 3285 "parse.y"
 yyval=(yyyvsp[0]).v;
goto _LL0;case 560U: _LL45B: _LL45C:
# 3288
 yyval=Cyc_QualId_tok(({struct _tuple0*_tmp5B5=_cycalloc(sizeof(*_tmp5B5));({union Cyc_Absyn_Nmspace _tmpB99=Cyc_Absyn_Rel_n(0);_tmp5B5->f1=_tmpB99;}),({struct _fat_ptr*_tmpB98=({struct _fat_ptr*_tmp5B4=_cycalloc(sizeof(*_tmp5B4));({struct _fat_ptr _tmpB97=Cyc_yyget_String_tok(&(yyyvsp[0]).v);*_tmp5B4=_tmpB97;});_tmp5B4;});_tmp5B5->f2=_tmpB98;});_tmp5B5;}));
goto _LL0;case 561U: _LL45D: _LL45E:
# 3289 "parse.y"
 yyval=(yyyvsp[0]).v;
goto _LL0;case 562U: _LL45F: _LL460:
# 3294 "parse.y"
 yyval=(yyyvsp[0]).v;
goto _LL0;case 563U: _LL461: _LL462:
# 3295 "parse.y"
 yyval=(yyyvsp[0]).v;
goto _LL0;case 564U: _LL463: _LL464:
# 3298
 yyval=(yyyvsp[0]).v;
goto _LL0;case 565U: _LL465: _LL466:
# 3299 "parse.y"
 yyval=(yyyvsp[0]).v;
goto _LL0;case 566U: _LL467: _LL468:
# 3304 "parse.y"
 goto _LL0;case 567U: _LL469: _LL46A:
# 3304 "parse.y"
 yylex_buf->lex_curr_pos -=1;
goto _LL0;default: _LL46B: _LL46C:
# 3308
 goto _LL0;}_LL0:;}
# 375 "cycbison.simple"
yyvsp_offset -=yylen;
yyssp_offset -=yylen;
# 389 "cycbison.simple"
(*((struct Cyc_Yystacktype*)_check_fat_subscript(yyvs,sizeof(struct Cyc_Yystacktype),++ yyvsp_offset))).v=yyval;
# 392
if(yylen == 0){
struct Cyc_Yystacktype*p=(struct Cyc_Yystacktype*)_check_null(_untag_fat_ptr(_fat_ptr_plus(yyvs,sizeof(struct Cyc_Yystacktype),yyvsp_offset - 1),sizeof(struct Cyc_Yystacktype),2U));
((p[1]).l).first_line=yylloc.first_line;
((p[1]).l).first_column=yylloc.first_column;
((p[1]).l).last_line=((p[0]).l).last_line;
((p[1]).l).last_column=((p[0]).l).last_column;}else{
# 399
({int _tmpB9A=((*((struct Cyc_Yystacktype*)_check_fat_subscript(yyvs,sizeof(struct Cyc_Yystacktype),(yyvsp_offset + yylen)- 1))).l).last_line;((*((struct Cyc_Yystacktype*)_check_fat_subscript(yyvs,sizeof(struct Cyc_Yystacktype),yyvsp_offset))).l).last_line=_tmpB9A;});
((((struct Cyc_Yystacktype*)yyvs.curr)[yyvsp_offset]).l).last_column=((*((struct Cyc_Yystacktype*)_check_fat_subscript(yyvs,sizeof(struct Cyc_Yystacktype),(yyvsp_offset + yylen)- 1))).l).last_column;}
# 409
yyn=(int)Cyc_yyr1[yyn];
# 411
yystate=({int _tmpB9B=(int)*((short*)_check_known_subscript_notnull(Cyc_yypgoto,165U,sizeof(short),yyn - 154));_tmpB9B + (int)*((short*)_check_fat_subscript(yyss,sizeof(short),yyssp_offset));});
if((yystate >= 0 && yystate <= 8195)&&(int)*((short*)_check_known_subscript_notnull(Cyc_yycheck,8196U,sizeof(short),yystate))== (int)((short*)yyss.curr)[yyssp_offset])
yystate=(int)Cyc_yytable[yystate];else{
# 415
yystate=(int)*((short*)_check_known_subscript_notnull(Cyc_yydefgoto,165U,sizeof(short),yyn - 154));}
# 417
goto yynewstate;
# 419
yyerrlab:
# 421
 if(yyerrstatus == 0){
# 424
++ yynerrs;
# 427
yyn=(int)Cyc_yypact[yystate];
# 429
if(yyn > - 32768 && yyn < 8195){
# 431
int sze=0;
struct _fat_ptr msg;
int x;int count;
# 435
count=0;
# 437
for(x=yyn < 0?- yyn: 0;(unsigned)x < 319U / sizeof(char*);++ x){
# 439
if((int)*((short*)_check_known_subscript_notnull(Cyc_yycheck,8196U,sizeof(short),x + yyn))== x)
({unsigned long _tmpB9C=Cyc_strlen((struct _fat_ptr)*((struct _fat_ptr*)_check_known_subscript_notnull(Cyc_yytname,319U,sizeof(struct _fat_ptr),x)))+ (unsigned long)15;sze +=_tmpB9C;}),count ++;}
# 437
msg=({unsigned _tmp5B7=(unsigned)(sze + 15)+ 1U;char*_tmp5B6=({struct _RegionHandle*_tmpB9D=yyregion;_region_malloc(_tmpB9D,_check_times(_tmp5B7,sizeof(char)));});({{unsigned _tmp67B=(unsigned)(sze + 15);unsigned i;for(i=0;i < _tmp67B;++ i){_tmp5B6[i]='\000';}_tmp5B6[_tmp67B]=0;}0;});_tag_fat(_tmp5B6,sizeof(char),_tmp5B7);});
# 442
({struct _fat_ptr _tmpB9E=msg;Cyc_strcpy(_tmpB9E,({const char*_tmp5B8="parse error";_tag_fat(_tmp5B8,sizeof(char),12U);}));});
# 444
if(count < 5){
# 446
count=0;
for(x=yyn < 0?- yyn: 0;(unsigned)x < 319U / sizeof(char*);++ x){
# 449
if((int)*((short*)_check_known_subscript_notnull(Cyc_yycheck,8196U,sizeof(short),x + yyn))== x){
# 451
({struct _fat_ptr _tmpB9F=msg;Cyc_strcat(_tmpB9F,(struct _fat_ptr)(count == 0?(struct _fat_ptr)({const char*_tmp5B9=", expecting `";_tag_fat(_tmp5B9,sizeof(char),14U);}):(struct _fat_ptr)({const char*_tmp5BA=" or `";_tag_fat(_tmp5BA,sizeof(char),6U);})));});
# 454
Cyc_strcat(msg,(struct _fat_ptr)*((struct _fat_ptr*)_check_known_subscript_notnull(Cyc_yytname,319U,sizeof(struct _fat_ptr),x)));
({struct _fat_ptr _tmpBA0=msg;Cyc_strcat(_tmpBA0,({const char*_tmp5BB="'";_tag_fat(_tmp5BB,sizeof(char),2U);}));});
++ count;}}}
# 444
Cyc_yyerror((struct _fat_ptr)msg,yystate,yychar);}else{
# 463
({struct _fat_ptr _tmpBA2=({const char*_tmp5BC="parse error";_tag_fat(_tmp5BC,sizeof(char),12U);});int _tmpBA1=yystate;Cyc_yyerror(_tmpBA2,_tmpBA1,yychar);});}}
# 419
goto yyerrlab1;
# 467
yyerrlab1:
# 469
 if(yyerrstatus == 3){
# 474
if(yychar == 0){
int _tmp5BD=1;_npop_handler(0U);return _tmp5BD;}
# 474
yychar=-2;}
# 467
yyerrstatus=3;
# 491
goto yyerrhandle;
# 493
yyerrdefault:
# 503 "cycbison.simple"
 yyerrpop:
# 505
 if(yyssp_offset == 0){int _tmp5BE=1;_npop_handler(0U);return _tmp5BE;}
# 493 "cycbison.simple"
-- yyvsp_offset;
# 507 "cycbison.simple"
yystate=(int)*((short*)_check_fat_subscript(yyss,sizeof(short),-- yyssp_offset));
# 521 "cycbison.simple"
yyerrhandle:
 yyn=(int)*((short*)_check_known_subscript_notnull(Cyc_yypact,1170U,sizeof(short),yystate));
if(yyn == -32768)goto yyerrdefault;yyn +=1;
# 526
if((yyn < 0 || yyn > 8195)||(int)*((short*)_check_known_subscript_notnull(Cyc_yycheck,8196U,sizeof(short),yyn))!= 1)goto yyerrdefault;yyn=(int)Cyc_yytable[yyn];
# 529
if(yyn < 0){
# 531
if(yyn == -32768)goto yyerrpop;yyn=- yyn;
# 533
goto yyreduce;}else{
# 535
if(yyn == 0)goto yyerrpop;}
# 529
if(yyn == 1169){
# 538
int _tmp5BF=0;_npop_handler(0U);return _tmp5BF;}
# 529
({struct Cyc_Yystacktype _tmpBA3=({struct Cyc_Yystacktype _tmp67C;_tmp67C.v=yylval,_tmp67C.l=yylloc;_tmp67C;});*((struct Cyc_Yystacktype*)_check_fat_subscript(yyvs,sizeof(struct Cyc_Yystacktype),++ yyvsp_offset))=_tmpBA3;});
# 551
goto yynewstate;}
# 149 "cycbison.simple"
;_pop_region(yyregion);}
# 145
void Cyc_yyprint(int i,union Cyc_YYSTYPE v){
# 3308 "parse.y"
union Cyc_YYSTYPE _tmp5C3=v;struct Cyc_Absyn_Stmt*_tmp5C4;struct Cyc_Absyn_Exp*_tmp5C5;struct _tuple0*_tmp5C6;struct _fat_ptr _tmp5C7;char _tmp5C8;union Cyc_Absyn_Cnst _tmp5C9;switch((_tmp5C3.Stmt_tok).tag){case 1U: _LL1: _tmp5C9=(_tmp5C3.Int_tok).val;_LL2: {union Cyc_Absyn_Cnst c=_tmp5C9;
({struct Cyc_String_pa_PrintArg_struct _tmp5CC=({struct Cyc_String_pa_PrintArg_struct _tmp67F;_tmp67F.tag=0U,({struct _fat_ptr _tmpBA4=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_cnst2string(c));_tmp67F.f1=_tmpBA4;});_tmp67F;});void*_tmp5CA[1U];_tmp5CA[0]=& _tmp5CC;({struct Cyc___cycFILE*_tmpBA6=Cyc_stderr;struct _fat_ptr _tmpBA5=({const char*_tmp5CB="%s";_tag_fat(_tmp5CB,sizeof(char),3U);});Cyc_fprintf(_tmpBA6,_tmpBA5,_tag_fat(_tmp5CA,sizeof(void*),1U));});});goto _LL0;}case 2U: _LL3: _tmp5C8=(_tmp5C3.Char_tok).val;_LL4: {char c=_tmp5C8;
({struct Cyc_Int_pa_PrintArg_struct _tmp5CF=({struct Cyc_Int_pa_PrintArg_struct _tmp680;_tmp680.tag=1U,_tmp680.f1=(unsigned long)((int)c);_tmp680;});void*_tmp5CD[1U];_tmp5CD[0]=& _tmp5CF;({struct Cyc___cycFILE*_tmpBA8=Cyc_stderr;struct _fat_ptr _tmpBA7=({const char*_tmp5CE="%c";_tag_fat(_tmp5CE,sizeof(char),3U);});Cyc_fprintf(_tmpBA8,_tmpBA7,_tag_fat(_tmp5CD,sizeof(void*),1U));});});goto _LL0;}case 3U: _LL5: _tmp5C7=(_tmp5C3.String_tok).val;_LL6: {struct _fat_ptr s=_tmp5C7;
({struct Cyc_String_pa_PrintArg_struct _tmp5D2=({struct Cyc_String_pa_PrintArg_struct _tmp681;_tmp681.tag=0U,_tmp681.f1=(struct _fat_ptr)((struct _fat_ptr)s);_tmp681;});void*_tmp5D0[1U];_tmp5D0[0]=& _tmp5D2;({struct Cyc___cycFILE*_tmpBAA=Cyc_stderr;struct _fat_ptr _tmpBA9=({const char*_tmp5D1="\"%s\"";_tag_fat(_tmp5D1,sizeof(char),5U);});Cyc_fprintf(_tmpBAA,_tmpBA9,_tag_fat(_tmp5D0,sizeof(void*),1U));});});goto _LL0;}case 5U: _LL7: _tmp5C6=(_tmp5C3.QualId_tok).val;_LL8: {struct _tuple0*q=_tmp5C6;
({struct Cyc_String_pa_PrintArg_struct _tmp5D5=({struct Cyc_String_pa_PrintArg_struct _tmp682;_tmp682.tag=0U,({struct _fat_ptr _tmpBAB=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(q));_tmp682.f1=_tmpBAB;});_tmp682;});void*_tmp5D3[1U];_tmp5D3[0]=& _tmp5D5;({struct Cyc___cycFILE*_tmpBAD=Cyc_stderr;struct _fat_ptr _tmpBAC=({const char*_tmp5D4="%s";_tag_fat(_tmp5D4,sizeof(char),3U);});Cyc_fprintf(_tmpBAD,_tmpBAC,_tag_fat(_tmp5D3,sizeof(void*),1U));});});goto _LL0;}case 7U: _LL9: _tmp5C5=(_tmp5C3.Exp_tok).val;_LLA: {struct Cyc_Absyn_Exp*e=_tmp5C5;
({struct Cyc_String_pa_PrintArg_struct _tmp5D8=({struct Cyc_String_pa_PrintArg_struct _tmp683;_tmp683.tag=0U,({struct _fat_ptr _tmpBAE=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_exp2string(e));_tmp683.f1=_tmpBAE;});_tmp683;});void*_tmp5D6[1U];_tmp5D6[0]=& _tmp5D8;({struct Cyc___cycFILE*_tmpBB0=Cyc_stderr;struct _fat_ptr _tmpBAF=({const char*_tmp5D7="%s";_tag_fat(_tmp5D7,sizeof(char),3U);});Cyc_fprintf(_tmpBB0,_tmpBAF,_tag_fat(_tmp5D6,sizeof(void*),1U));});});goto _LL0;}case 8U: _LLB: _tmp5C4=(_tmp5C3.Stmt_tok).val;_LLC: {struct Cyc_Absyn_Stmt*s=_tmp5C4;
({struct Cyc_String_pa_PrintArg_struct _tmp5DB=({struct Cyc_String_pa_PrintArg_struct _tmp684;_tmp684.tag=0U,({struct _fat_ptr _tmpBB1=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_stmt2string(s));_tmp684.f1=_tmpBB1;});_tmp684;});void*_tmp5D9[1U];_tmp5D9[0]=& _tmp5DB;({struct Cyc___cycFILE*_tmpBB3=Cyc_stderr;struct _fat_ptr _tmpBB2=({const char*_tmp5DA="%s";_tag_fat(_tmp5DA,sizeof(char),3U);});Cyc_fprintf(_tmpBB3,_tmpBB2,_tag_fat(_tmp5D9,sizeof(void*),1U));});});goto _LL0;}default: _LLD: _LLE:
({void*_tmp5DC=0U;({struct Cyc___cycFILE*_tmpBB5=Cyc_stderr;struct _fat_ptr _tmpBB4=({const char*_tmp5DD="?";_tag_fat(_tmp5DD,sizeof(char),2U);});Cyc_fprintf(_tmpBB5,_tmpBB4,_tag_fat(_tmp5DC,sizeof(void*),0U));});});goto _LL0;}_LL0:;}
# 145 "cycbison.simple"
struct _fat_ptr Cyc_token2string(int token){
# 3320 "parse.y"
if(token <= 0)
return({const char*_tmp5DE="end-of-file";_tag_fat(_tmp5DE,sizeof(char),12U);});
# 3320
if(token == 369)
# 3323
return Cyc_Lex_token_string;else{
if(token == 378)
return Cyc_Absynpp_qvar2string(Cyc_Lex_token_qvar);}{
# 3320
int z=
# 3326
token > 0 && token <= 381?(int)*((short*)_check_known_subscript_notnull(Cyc_yytranslate,382U,sizeof(short),token)): 319;
if((unsigned)z < 319U)
return Cyc_yytname[z];else{
return _tag_fat(0,0,0);}}}
# 3334
struct _fat_ptr Cyc_Parse_filename={(void*)0,(void*)0,(void*)(0 + 0)};
# 3336
void Cyc_Parse_die(){
# 3338
({struct Cyc_String_pa_PrintArg_struct _tmp5E1=({struct Cyc_String_pa_PrintArg_struct _tmp686;_tmp686.tag=0U,_tmp686.f1=(struct _fat_ptr)((struct _fat_ptr)Cyc_Parse_filename);_tmp686;});struct Cyc_Int_pa_PrintArg_struct _tmp5E2=({struct Cyc_Int_pa_PrintArg_struct _tmp685;_tmp685.tag=1U,_tmp685.f1=(unsigned long)Cyc_yylloc.first_line;_tmp685;});void*_tmp5DF[2U];_tmp5DF[0]=& _tmp5E1,_tmp5DF[1]=& _tmp5E2;({struct Cyc___cycFILE*_tmpBB7=Cyc_stderr;struct _fat_ptr _tmpBB6=({const char*_tmp5E0="\nBefore die! File name : %s Line : %d\n";_tag_fat(_tmp5E0,sizeof(char),39U);});Cyc_fprintf(_tmpBB7,_tmpBB6,_tag_fat(_tmp5DF,sizeof(void*),2U));});});}
# 3336
struct Cyc_List_List*Cyc_Parse_parse_file(struct Cyc___cycFILE*f,struct _fat_ptr name){
# 3343
Cyc_Parse_parse_result=0;{
struct _RegionHandle _tmp5E3=_new_region("yyr");struct _RegionHandle*yyr=& _tmp5E3;_push_region(yyr);
Cyc_Parse_filename=name;
# 3347
({struct _RegionHandle*_tmpBB8=yyr;Cyc_yyparse(_tmpBB8,Cyc_Lexing_from_file(f));});{
struct Cyc_List_List*_tmp5E4=Cyc_Parse_parse_result;_npop_handler(0U);return _tmp5E4;}
# 3345
;_pop_region(yyr);}}
