#include <setjmp.h>
/* This is a C header file to be used by the output of the Cyclone to
   C translator.  The corresponding definitions are in file lib/runtime_*.c */
#ifndef _CYC_INCLUDE_H_
#define _CYC_INCLUDE_H_

/* Need one of these per thread (see runtime_stack.c). The runtime maintains 
   a stack that contains either _handler_cons structs or _RegionHandle structs.
   The tag is 0 for a handler_cons and 1 for a region handle.  */
struct _RuntimeStack {
  int tag; 
  struct _RuntimeStack *next;
  void (*cleanup)(struct _RuntimeStack *frame);
};

#ifndef offsetof
/* should be size_t, but int is fine. */
#define offsetof(t,n) ((int)(&(((t *)0)->n)))
#endif

/* Fat pointers */
struct _fat_ptr {
  unsigned char *curr; 
  unsigned char *base; 
  unsigned char *last_plus_one; 
};  

/* Discriminated Unions */
struct _xtunion_struct { char *tag; };

/* Regions */
struct _RegionPage
#ifdef CYC_REGION_PROFILE
{ unsigned total_bytes;
  unsigned free_bytes;
  /* MWH: wish we didn't have to include the stuff below ... */
  struct _RegionPage *next;
  char data[1];
}
#endif
; // abstract -- defined in runtime_memory.c
struct _RegionHandle {
  struct _RuntimeStack s;
  struct _RegionPage *curr;
  char               *offset;
  char               *last_plus_one;
  struct _DynRegionHandle *sub_regions;
#ifdef CYC_REGION_PROFILE
  const char         *name;
#else
  unsigned used_bytes;
  unsigned wasted_bytes;
#endif
};
struct _DynRegionFrame {
  struct _RuntimeStack s;
  struct _DynRegionHandle *x;
};
// A dynamic region is just a region handle.  The wrapper struct is for type
// abstraction.
struct Cyc_Core_DynamicRegion {
  struct _RegionHandle h;
};

struct _RegionHandle _new_region(const char*);
void* _region_malloc(struct _RegionHandle*, unsigned);
void* _region_calloc(struct _RegionHandle*, unsigned t, unsigned n);
void   _free_region(struct _RegionHandle*);
struct _RegionHandle*_open_dynregion(struct _DynRegionFrame*,struct _DynRegionHandle*);
void   _pop_dynregion();

/* Exceptions */
struct _handler_cons {
  struct _RuntimeStack s;
  jmp_buf handler;
};
void _push_handler(struct _handler_cons *);
void _push_region(struct _RegionHandle *);
void _npop_handler(int);
void _pop_handler();
void _pop_region();

#ifndef _throw
void* _throw_null_fn(const char*,unsigned);
void* _throw_arraybounds_fn(const char*,unsigned);
void* _throw_badalloc_fn(const char*,unsigned);
void* _throw_match_fn(const char*,unsigned);
void* _throw_fn(void*,const char*,unsigned);
void* _rethrow(void*);
#define _throw_null() (_throw_null_fn(__FILE__,__LINE__))
#define _throw_arraybounds() (_throw_arraybounds_fn(__FILE__,__LINE__))
#define _throw_badalloc() (_throw_badalloc_fn(__FILE__,__LINE__))
#define _throw_match() (_throw_match_fn(__FILE__,__LINE__))
#define _throw(e) (_throw_fn((e),__FILE__,__LINE__))
#endif

struct _xtunion_struct* Cyc_Core_get_exn_thrown();
/* Built-in Exceptions */
struct Cyc_Null_Exception_exn_struct { char *tag; };
struct Cyc_Array_bounds_exn_struct { char *tag; };
struct Cyc_Match_Exception_exn_struct { char *tag; };
struct Cyc_Bad_alloc_exn_struct { char *tag; };
extern char Cyc_Null_Exception[];
extern char Cyc_Array_bounds[];
extern char Cyc_Match_Exception[];
extern char Cyc_Bad_alloc[];

/* Built-in Run-time Checks and company */
#ifdef CYC_ANSI_OUTPUT
#define _INLINE  
#else
#define _INLINE inline
#endif

#ifdef NO_CYC_NULL_CHECKS
#define _check_null(ptr) (ptr)
#else
#define _check_null(ptr) \
  ({ void*_cks_null = (void*)(ptr); \
     if (!_cks_null) _throw_null(); \
     _cks_null; })
#endif

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index)\
   (((char*)ptr) + (elt_sz)*(index))
#ifdef NO_CYC_NULL_CHECKS
#define _check_known_subscript_null _check_known_subscript_notnull
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr);\
  int _index = (index);\
  if (!_cks_ptr) _throw_null(); \
  _cks_ptr + (elt_sz)*_index; })
#endif
#define _zero_arr_plus_fn(orig_x,orig_sz,orig_i,f,l) ((orig_x)+(orig_i))
#define _zero_arr_plus_char_fn _zero_arr_plus_fn
#define _zero_arr_plus_short_fn _zero_arr_plus_fn
#define _zero_arr_plus_int_fn _zero_arr_plus_fn
#define _zero_arr_plus_float_fn _zero_arr_plus_fn
#define _zero_arr_plus_double_fn _zero_arr_plus_fn
#define _zero_arr_plus_longdouble_fn _zero_arr_plus_fn
#define _zero_arr_plus_voidstar_fn _zero_arr_plus_fn
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (!_cks_ptr) _throw_null(); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })

/* _zero_arr_plus_*_fn(x,sz,i,filename,lineno) adds i to zero-terminated ptr
   x that has at least sz elements */
char* _zero_arr_plus_char_fn(char*,unsigned,int,const char*,unsigned);
short* _zero_arr_plus_short_fn(short*,unsigned,int,const char*,unsigned);
int* _zero_arr_plus_int_fn(int*,unsigned,int,const char*,unsigned);
float* _zero_arr_plus_float_fn(float*,unsigned,int,const char*,unsigned);
double* _zero_arr_plus_double_fn(double*,unsigned,int,const char*,unsigned);
long double* _zero_arr_plus_longdouble_fn(long double*,unsigned,int,const char*, unsigned);
void** _zero_arr_plus_voidstar_fn(void**,unsigned,int,const char*,unsigned);
#endif

/* _get_zero_arr_size_*(x,sz) returns the number of elements in a
   zero-terminated array that is NULL or has at least sz elements */
int _get_zero_arr_size_char(const char*,unsigned);
int _get_zero_arr_size_short(const short*,unsigned);
int _get_zero_arr_size_int(const int*,unsigned);
int _get_zero_arr_size_float(const float*,unsigned);
int _get_zero_arr_size_double(const double*,unsigned);
int _get_zero_arr_size_longdouble(const long double*,unsigned);
int _get_zero_arr_size_voidstar(const void**,unsigned);

/* _zero_arr_inplace_plus_*_fn(x,i,filename,lineno) sets
   zero-terminated pointer *x to *x + i */
char* _zero_arr_inplace_plus_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_short_fn(short**,int,const char*,unsigned);
int* _zero_arr_inplace_plus_int(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_longdouble_fn(long double**,int,const char*,unsigned);
void** _zero_arr_inplace_plus_voidstar_fn(void***,int,const char*,unsigned);
/* like the previous functions, but does post-addition (as in e++) */
char* _zero_arr_inplace_plus_post_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_post_short_fn(short**x,int,const char*,unsigned);
int* _zero_arr_inplace_plus_post_int_fn(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_post_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_post_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_post_longdouble_fn(long double**,int,const char *,unsigned);
void** _zero_arr_inplace_plus_post_voidstar_fn(void***,int,const char*,unsigned);
#define _zero_arr_plus_char(x,s,i) \
  (_zero_arr_plus_char_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_short(x,s,i) \
  (_zero_arr_plus_short_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_int(x,s,i) \
  (_zero_arr_plus_int_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_float(x,s,i) \
  (_zero_arr_plus_float_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_double(x,s,i) \
  (_zero_arr_plus_double_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_longdouble(x,s,i) \
  (_zero_arr_plus_longdouble_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_voidstar(x,s,i) \
  (_zero_arr_plus_voidstar_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_inplace_plus_char(x,i) \
  _zero_arr_inplace_plus_char_fn((char **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_short(x,i) \
  _zero_arr_inplace_plus_short_fn((short **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_int(x,i) \
  _zero_arr_inplace_plus_int_fn((int **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_float(x,i) \
  _zero_arr_inplace_plus_float_fn((float **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_double(x,i) \
  _zero_arr_inplace_plus_double_fn((double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_longdouble(x,i) \
  _zero_arr_inplace_plus_longdouble_fn((long double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_voidstar(x,i) \
  _zero_arr_inplace_plus_voidstar_fn((void ***)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_char(x,i) \
  _zero_arr_inplace_plus_post_char_fn((char **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_short(x,i) \
  _zero_arr_inplace_plus_post_short_fn((short **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_int(x,i) \
  _zero_arr_inplace_plus_post_int_fn((int **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_float(x,i) \
  _zero_arr_inplace_plus_post_float_fn((float **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_double(x,i) \
  _zero_arr_inplace_plus_post_double_fn((double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_longdouble(x,i) \
  _zero_arr_inplace_plus_post_longdouble_fn((long double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_voidstar(x,i) \
  _zero_arr_inplace_plus_post_voidstar_fn((void***)(x),(i),__FILE__,__LINE__)

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_fat_subscript(arr,elt_sz,index) ((arr).curr + (elt_sz) * (index))
#define _untag_fat_ptr(arr,elt_sz,num_elts) ((arr).curr)
#else
#define _check_fat_subscript(arr,elt_sz,index) ({ \
  struct _fat_ptr _cus_arr = (arr); \
  unsigned char *_cus_ans = _cus_arr.curr + (elt_sz) * (index); \
  /* JGM: not needed! if (!_cus_arr.base) _throw_null();*/ \
  if (_cus_ans < _cus_arr.base || _cus_ans >= _cus_arr.last_plus_one) \
    _throw_arraybounds(); \
  _cus_ans; })
#define _untag_fat_ptr(arr,elt_sz,num_elts) ({ \
  struct _fat_ptr _arr = (arr); \
  unsigned char *_curr = _arr.curr; \
  if ((_curr < _arr.base || _curr + (elt_sz) * (num_elts) > _arr.last_plus_one) &&\
      _curr != (unsigned char *)0) \
    _throw_arraybounds(); \
  _curr; })
#endif

#define _tag_fat(tcurr,elt_sz,num_elts) ({ \
  struct _fat_ptr _ans; \
  unsigned _num_elts = (num_elts);\
  _ans.base = _ans.curr = (void*)(tcurr); \
  /* JGM: if we're tagging NULL, ignore num_elts */ \
  _ans.last_plus_one = _ans.base ? (_ans.base + (elt_sz) * _num_elts) : 0; \
  _ans; })

#define _get_fat_size(arr,elt_sz) \
  ({struct _fat_ptr _arr = (arr); \
    unsigned char *_arr_curr=_arr.curr; \
    unsigned char *_arr_last=_arr.last_plus_one; \
    (_arr_curr < _arr.base || _arr_curr >= _arr_last) ? 0 : \
    ((_arr_last - _arr_curr) / (elt_sz));})

#define _fat_ptr_plus(arr,elt_sz,change) ({ \
  struct _fat_ptr _ans = (arr); \
  int _change = (change);\
  _ans.curr += (elt_sz) * _change;\
  _ans; })
#define _fat_ptr_inplace_plus(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  _arr_ptr->curr += (elt_sz) * (change);\
  *_arr_ptr; })
#define _fat_ptr_inplace_plus_post(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  struct _fat_ptr _ans = *_arr_ptr; \
  _arr_ptr->curr += (elt_sz) * (change);\
  _ans; })

//Not a macro since initialization order matters. Defined in runtime_zeroterm.c.
struct _fat_ptr _fat_ptr_decrease_size(struct _fat_ptr,unsigned sz,unsigned numelts);

/* Allocation */
void* GC_malloc(int);
void* GC_malloc_atomic(int);
void* GC_calloc(unsigned,unsigned);
void* GC_calloc_atomic(unsigned,unsigned);
// bound the allocation size to be < MAX_ALLOC_SIZE. See macros below for usage.
#define MAX_MALLOC_SIZE (1 << 28)
void* _bounded_GC_malloc(int,const char*,int);
void* _bounded_GC_malloc_atomic(int,const char*,int);
void* _bounded_GC_calloc(unsigned,unsigned,const char*,int);
void* _bounded_GC_calloc_atomic(unsigned,unsigned,const char*,int);
/* these macros are overridden below ifdef CYC_REGION_PROFILE */
#ifndef CYC_REGION_PROFILE
#define _cycalloc(n) _bounded_GC_malloc(n,__FILE__,__LINE__)
#define _cycalloc_atomic(n) _bounded_GC_malloc_atomic(n,__FILE__,__LINE__)
#define _cyccalloc(n,s) _bounded_GC_calloc(n,s,__FILE__,__LINE__)
#define _cyccalloc_atomic(n,s) _bounded_GC_calloc_atomic(n,s,__FILE__,__LINE__)
#endif

static _INLINE unsigned int _check_times(unsigned x, unsigned y) {
  unsigned long long whole_ans = 
    ((unsigned long long) x)*((unsigned long long)y);
  unsigned word_ans = (unsigned)whole_ans;
  if(word_ans < whole_ans || word_ans > MAX_MALLOC_SIZE)
    _throw_badalloc();
  return word_ans;
}

#define _CYC_MAX_REGION_CONST 2
#define _CYC_MIN_ALIGNMENT (sizeof(double))

#ifdef CYC_REGION_PROFILE
extern int rgn_total_bytes;
#endif

static _INLINE void *_fast_region_malloc(struct _RegionHandle *r, unsigned orig_s) {  
  if (r > (struct _RegionHandle *)_CYC_MAX_REGION_CONST && r->curr != 0) { 
#ifdef CYC_NOALIGN
    unsigned s =  orig_s;
#else
    unsigned s =  (orig_s + _CYC_MIN_ALIGNMENT - 1) & (~(_CYC_MIN_ALIGNMENT -1)); 
#endif
    char *result; 
    result = r->offset; 
    if (s <= (r->last_plus_one - result)) {
      r->offset = result + s; 
#ifdef CYC_REGION_PROFILE
    r->curr->free_bytes = r->curr->free_bytes - s;
    rgn_total_bytes += s;
#endif
      return result;
    }
  } 
  return _region_malloc(r,orig_s); 
}

#ifdef CYC_REGION_PROFILE
/* see macros below for usage. defined in runtime_memory.c */
void* _profile_GC_malloc(int,const char*,const char*,int);
void* _profile_GC_malloc_atomic(int,const char*,const char*,int);
void* _profile_GC_calloc(unsigned,unsigned,const char*,const char*,int);
void* _profile_GC_calloc_atomic(unsigned,unsigned,const char*,const char*,int);
void* _profile_region_malloc(struct _RegionHandle*,unsigned,const char*,const char*,int);
void* _profile_region_calloc(struct _RegionHandle*,unsigned,unsigned,const char *,const char*,int);
struct _RegionHandle _profile_new_region(const char*,const char*,const char*,int);
void _profile_free_region(struct _RegionHandle*,const char*,const char*,int);
#ifndef RUNTIME_CYC
#define _new_region(n) _profile_new_region(n,__FILE__,__FUNCTION__,__LINE__)
#define _free_region(r) _profile_free_region(r,__FILE__,__FUNCTION__,__LINE__)
#define _region_malloc(rh,n) _profile_region_malloc(rh,n,__FILE__,__FUNCTION__,__LINE__)
#define _region_calloc(rh,n,t) _profile_region_calloc(rh,n,t,__FILE__,__FUNCTION__,__LINE__)
#  endif
#define _cycalloc(n) _profile_GC_malloc(n,__FILE__,__FUNCTION__,__LINE__)
#define _cycalloc_atomic(n) _profile_GC_malloc_atomic(n,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc(n,s) _profile_GC_calloc(n,s,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc_atomic(n,s) _profile_GC_calloc_atomic(n,s,__FILE__,__FUNCTION__,__LINE__)
#endif
#endif
 struct Cyc_Core_Opt{void*v;};extern char Cyc_Core_Invalid_argument[17U];struct Cyc_Core_Invalid_argument_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Failure[8U];struct Cyc_Core_Failure_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Impossible[11U];struct Cyc_Core_Impossible_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Not_found[10U];struct Cyc_Core_Not_found_exn_struct{char*tag;};extern char Cyc_Core_Unreachable[12U];struct Cyc_Core_Unreachable_exn_struct{char*tag;struct _fat_ptr f1;};
# 168 "core.h"
extern struct _RegionHandle*Cyc_Core_unique_region;struct Cyc_Core_DynamicRegion;struct Cyc_Core_NewDynamicRegion{struct Cyc_Core_DynamicRegion*key;};struct Cyc_Core_ThinRes{void*arr;unsigned nelts;};struct Cyc_List_List{void*hd;struct Cyc_List_List*tl;};
# 54 "list.h"
struct Cyc_List_List*Cyc_List_list(struct _fat_ptr);
# 76
struct Cyc_List_List*Cyc_List_map(void*(*f)(void*),struct Cyc_List_List*x);
# 83
struct Cyc_List_List*Cyc_List_map_c(void*(*f)(void*,void*),void*env,struct Cyc_List_List*x);extern char Cyc_List_List_mismatch[14U];struct Cyc_List_List_mismatch_exn_struct{char*tag;};
# 135
void Cyc_List_iter_c(void(*f)(void*,void*),void*env,struct Cyc_List_List*x);
# 178
struct Cyc_List_List*Cyc_List_imp_rev(struct Cyc_List_List*x);
# 210
struct Cyc_List_List*Cyc_List_merge_sort(int(*cmp)(void*,void*),struct Cyc_List_List*x);extern char Cyc_List_Nth[4U];struct Cyc_List_Nth_exn_struct{char*tag;};struct Cyc___cycFILE;
# 53 "cycboot.h"
extern struct Cyc___cycFILE*Cyc_stderr;struct Cyc_String_pa_PrintArg_struct{int tag;struct _fat_ptr f1;};struct Cyc_Int_pa_PrintArg_struct{int tag;unsigned long f1;};struct Cyc_Double_pa_PrintArg_struct{int tag;double f1;};struct Cyc_LongDouble_pa_PrintArg_struct{int tag;long double f1;};struct Cyc_ShortPtr_pa_PrintArg_struct{int tag;short*f1;};struct Cyc_IntPtr_pa_PrintArg_struct{int tag;unsigned long*f1;};
# 73
struct _fat_ptr Cyc_aprintf(struct _fat_ptr,struct _fat_ptr);
# 100
int Cyc_fprintf(struct Cyc___cycFILE*,struct _fat_ptr,struct _fat_ptr);struct Cyc_ShortPtr_sa_ScanfArg_struct{int tag;short*f1;};struct Cyc_UShortPtr_sa_ScanfArg_struct{int tag;unsigned short*f1;};struct Cyc_IntPtr_sa_ScanfArg_struct{int tag;int*f1;};struct Cyc_UIntPtr_sa_ScanfArg_struct{int tag;unsigned*f1;};struct Cyc_StringPtr_sa_ScanfArg_struct{int tag;struct _fat_ptr f1;};struct Cyc_DoublePtr_sa_ScanfArg_struct{int tag;double*f1;};struct Cyc_FloatPtr_sa_ScanfArg_struct{int tag;float*f1;};struct Cyc_CharPtr_sa_ScanfArg_struct{int tag;struct _fat_ptr f1;};
# 157 "cycboot.h"
int Cyc_printf(struct _fat_ptr,struct _fat_ptr);extern char Cyc_FileCloseError[15U];struct Cyc_FileCloseError_exn_struct{char*tag;};extern char Cyc_FileOpenError[14U];struct Cyc_FileOpenError_exn_struct{char*tag;struct _fat_ptr f1;};
# 49 "string.h"
int Cyc_strcmp(struct _fat_ptr s1,struct _fat_ptr s2);
int Cyc_strptrcmp(struct _fat_ptr*s1,struct _fat_ptr*s2);
# 64
struct _fat_ptr Cyc_strconcat_l(struct Cyc_List_List*);struct Cyc_PP_Ppstate;struct Cyc_PP_Out;struct Cyc_PP_Doc;
# 38 "position.h"
struct Cyc_List_List*Cyc_Position_strings_of_segments(struct Cyc_List_List*);struct Cyc_Position_Error;
# 46
extern int Cyc_Position_use_gcc_style_location;struct Cyc_Relations_Reln;struct _union_Nmspace_Rel_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_Abs_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_C_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_Loc_n{int tag;int val;};union Cyc_Absyn_Nmspace{struct _union_Nmspace_Rel_n Rel_n;struct _union_Nmspace_Abs_n Abs_n;struct _union_Nmspace_C_n C_n;struct _union_Nmspace_Loc_n Loc_n;};
# 95 "absyn.h"
extern union Cyc_Absyn_Nmspace Cyc_Absyn_Loc_n;struct _tuple0{union Cyc_Absyn_Nmspace f1;struct _fat_ptr*f2;};
# 161
enum Cyc_Absyn_Scope{Cyc_Absyn_Static =0U,Cyc_Absyn_Abstract =1U,Cyc_Absyn_Public =2U,Cyc_Absyn_Extern =3U,Cyc_Absyn_ExternC =4U,Cyc_Absyn_Register =5U};struct Cyc_Absyn_Tqual{int print_const: 1;int q_volatile: 1;int q_restrict: 1;int real_const: 1;unsigned loc;};
# 182
enum Cyc_Absyn_Size_of{Cyc_Absyn_Char_sz =0U,Cyc_Absyn_Short_sz =1U,Cyc_Absyn_Int_sz =2U,Cyc_Absyn_Long_sz =3U,Cyc_Absyn_LongLong_sz =4U};
enum Cyc_Absyn_Sign{Cyc_Absyn_Signed =0U,Cyc_Absyn_Unsigned =1U,Cyc_Absyn_None =2U};
enum Cyc_Absyn_AggrKind{Cyc_Absyn_StructA =0U,Cyc_Absyn_UnionA =1U};
# 187
enum Cyc_Absyn_AliasQual{Cyc_Absyn_Aliasable =0U,Cyc_Absyn_Unique =1U,Cyc_Absyn_Top =2U};
# 192
enum Cyc_Absyn_KindQual{Cyc_Absyn_AnyKind =0U,Cyc_Absyn_MemKind =1U,Cyc_Absyn_BoxKind =2U,Cyc_Absyn_RgnKind =3U,Cyc_Absyn_EffKind =4U,Cyc_Absyn_IntKind =5U,Cyc_Absyn_BoolKind =6U,Cyc_Absyn_PtrBndKind =7U};struct Cyc_Absyn_Kind{enum Cyc_Absyn_KindQual kind;enum Cyc_Absyn_AliasQual aliasqual;};struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct{int tag;struct Cyc_Absyn_Kind*f1;};struct Cyc_Absyn_Unknown_kb_Absyn_KindBound_struct{int tag;struct Cyc_Core_Opt*f1;};struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct{int tag;struct Cyc_Core_Opt*f1;struct Cyc_Absyn_Kind*f2;};struct Cyc_Absyn_Tvar{struct _fat_ptr*name;int identity;void*kind;};struct Cyc_Absyn_PtrLoc{unsigned ptr_loc;unsigned rgn_loc;unsigned zt_loc;};struct Cyc_Absyn_PtrAtts{void*rgn;void*nullable;void*bounds;void*zero_term;struct Cyc_Absyn_PtrLoc*ptrloc;};struct Cyc_Absyn_PtrInfo{void*elt_type;struct Cyc_Absyn_Tqual elt_tq;struct Cyc_Absyn_PtrAtts ptr_atts;};struct Cyc_Absyn_VarargInfo{struct _fat_ptr*name;struct Cyc_Absyn_Tqual tq;void*type;int inject;};struct Cyc_Absyn_Star_Absyn_Cap_struct{int tag;};struct Cyc_Absyn_Bot_Absyn_Cap_struct{int tag;};struct Cyc_Absyn_Nat_Absyn_Cap_struct{int tag;struct Cyc_Absyn_Exp*f1;int f2;};struct Cyc_Absyn_RgnEffect{void*name;void*rgn;void*lock;void*unadopt;void*parent;};struct Cyc_Absyn_FnInfo{struct Cyc_List_List*tvars;void*effect;struct Cyc_Absyn_Tqual ret_tqual;void*ret_type;struct Cyc_List_List*args;int c_varargs;struct Cyc_Absyn_VarargInfo*cyc_varargs;struct Cyc_List_List*rgn_po;struct Cyc_List_List*attributes;struct Cyc_Absyn_Exp*requires_clause;struct Cyc_List_List*requires_relns;struct Cyc_Absyn_Exp*ensures_clause;struct Cyc_List_List*ensures_relns;struct Cyc_List_List*ieffect;struct Cyc_List_List*oeffect;};struct Cyc_Absyn_UnknownDatatypeInfo{struct _tuple0*name;int is_extensible;};struct _union_DatatypeInfo_UnknownDatatype{int tag;struct Cyc_Absyn_UnknownDatatypeInfo val;};struct _union_DatatypeInfo_KnownDatatype{int tag;struct Cyc_Absyn_Datatypedecl**val;};union Cyc_Absyn_DatatypeInfo{struct _union_DatatypeInfo_UnknownDatatype UnknownDatatype;struct _union_DatatypeInfo_KnownDatatype KnownDatatype;};struct Cyc_Absyn_UnknownDatatypeFieldInfo{struct _tuple0*datatype_name;struct _tuple0*field_name;int is_extensible;};struct _union_DatatypeFieldInfo_UnknownDatatypefield{int tag;struct Cyc_Absyn_UnknownDatatypeFieldInfo val;};struct _tuple1{struct Cyc_Absyn_Datatypedecl*f1;struct Cyc_Absyn_Datatypefield*f2;};struct _union_DatatypeFieldInfo_KnownDatatypefield{int tag;struct _tuple1 val;};union Cyc_Absyn_DatatypeFieldInfo{struct _union_DatatypeFieldInfo_UnknownDatatypefield UnknownDatatypefield;struct _union_DatatypeFieldInfo_KnownDatatypefield KnownDatatypefield;};struct _tuple2{enum Cyc_Absyn_AggrKind f1;struct _tuple0*f2;struct Cyc_Core_Opt*f3;};struct _union_AggrInfo_UnknownAggr{int tag;struct _tuple2 val;};struct _union_AggrInfo_KnownAggr{int tag;struct Cyc_Absyn_Aggrdecl**val;};union Cyc_Absyn_AggrInfo{struct _union_AggrInfo_UnknownAggr UnknownAggr;struct _union_AggrInfo_KnownAggr KnownAggr;};struct Cyc_Absyn_ArrayInfo{void*elt_type;struct Cyc_Absyn_Tqual tq;struct Cyc_Absyn_Exp*num_elts;void*zero_term;unsigned zt_loc;};struct Cyc_Absyn_Aggr_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Aggrdecl*f1;};struct Cyc_Absyn_Enum_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;};struct Cyc_Absyn_Datatype_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;};struct Cyc_Absyn_TypeDecl{void*r;unsigned loc;};struct Cyc_Absyn_VoidCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_IntCon_Absyn_TyCon_struct{int tag;enum Cyc_Absyn_Sign f1;enum Cyc_Absyn_Size_of f2;};struct Cyc_Absyn_FloatCon_Absyn_TyCon_struct{int tag;int f1;};struct Cyc_Absyn_RgnHandleCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_TagCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_HeapCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_UniqueCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_RefCntCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_AccessCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_JoinCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_RgnsCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_TrueCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_FalseCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_ThinCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_FatCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_EnumCon_Absyn_TyCon_struct{int tag;struct _tuple0*f1;struct Cyc_Absyn_Enumdecl*f2;};struct Cyc_Absyn_AnonEnumCon_Absyn_TyCon_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_BuiltinCon_Absyn_TyCon_struct{int tag;struct _fat_ptr f1;struct Cyc_Absyn_Kind*f2;};struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_DatatypeInfo f1;};struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_DatatypeFieldInfo f1;};struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_AggrInfo f1;};struct Cyc_Absyn_AppType_Absyn_Type_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Evar_Absyn_Type_struct{int tag;struct Cyc_Core_Opt*f1;void*f2;int f3;struct Cyc_Core_Opt*f4;};struct Cyc_Absyn_VarType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Tvar*f1;};struct Cyc_Absyn_PointerType_Absyn_Type_struct{int tag;struct Cyc_Absyn_PtrInfo f1;};struct Cyc_Absyn_ArrayType_Absyn_Type_struct{int tag;struct Cyc_Absyn_ArrayInfo f1;};struct Cyc_Absyn_FnType_Absyn_Type_struct{int tag;struct Cyc_Absyn_FnInfo f1;};struct Cyc_Absyn_TupleType_Absyn_Type_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct{int tag;enum Cyc_Absyn_AggrKind f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_TypedefType_Absyn_Type_struct{int tag;struct _tuple0*f1;struct Cyc_List_List*f2;struct Cyc_Absyn_Typedefdecl*f3;void*f4;};struct Cyc_Absyn_ValueofType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct{int tag;struct Cyc_Absyn_TypeDecl*f1;void**f2;};struct Cyc_Absyn_TypeofType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_NoTypes_Absyn_Funcparams_struct{int tag;struct Cyc_List_List*f1;unsigned f2;};struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct{int tag;struct Cyc_List_List*f1;int f2;struct Cyc_Absyn_VarargInfo*f3;void*f4;struct Cyc_List_List*f5;struct Cyc_Absyn_Exp*f6;struct Cyc_Absyn_Exp*f7;struct Cyc_List_List*f8;struct Cyc_List_List*f9;};
# 437 "absyn.h"
enum Cyc_Absyn_Format_Type{Cyc_Absyn_Printf_ft =0U,Cyc_Absyn_Scanf_ft =1U};struct Cyc_Absyn_Regparm_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Stdcall_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Cdecl_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Fastcall_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Noreturn_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Const_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Aligned_att_Absyn_Attribute_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Packed_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Section_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Nocommon_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Shared_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Unused_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Weak_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Dllimport_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Dllexport_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_instrument_function_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Constructor_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Destructor_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_check_memory_usage_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Format_att_Absyn_Attribute_struct{int tag;enum Cyc_Absyn_Format_Type f1;int f2;int f3;};struct Cyc_Absyn_Initializes_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Noliveunique_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Consume_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Pure_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Mode_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Alias_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Always_inline_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_throw_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Non_null_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Deprecated_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Carray_mod_Absyn_Type_modifier_struct{int tag;void*f1;unsigned f2;};struct Cyc_Absyn_ConstArray_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_Absyn_Exp*f1;void*f2;unsigned f3;};struct Cyc_Absyn_Pointer_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_Absyn_PtrAtts f1;struct Cyc_Absyn_Tqual f2;};struct Cyc_Absyn_Function_mod_Absyn_Type_modifier_struct{int tag;void*f1;};struct Cyc_Absyn_TypeParams_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_List_List*f1;unsigned f2;int f3;};struct Cyc_Absyn_Attributes_mod_Absyn_Type_modifier_struct{int tag;unsigned f1;struct Cyc_List_List*f2;};struct _union_Cnst_Null_c{int tag;int val;};struct _tuple3{enum Cyc_Absyn_Sign f1;char f2;};struct _union_Cnst_Char_c{int tag;struct _tuple3 val;};struct _union_Cnst_Wchar_c{int tag;struct _fat_ptr val;};struct _tuple4{enum Cyc_Absyn_Sign f1;short f2;};struct _union_Cnst_Short_c{int tag;struct _tuple4 val;};struct _tuple5{enum Cyc_Absyn_Sign f1;int f2;};struct _union_Cnst_Int_c{int tag;struct _tuple5 val;};struct _tuple6{enum Cyc_Absyn_Sign f1;long long f2;};struct _union_Cnst_LongLong_c{int tag;struct _tuple6 val;};struct _tuple7{struct _fat_ptr f1;int f2;};struct _union_Cnst_Float_c{int tag;struct _tuple7 val;};struct _union_Cnst_String_c{int tag;struct _fat_ptr val;};struct _union_Cnst_Wstring_c{int tag;struct _fat_ptr val;};union Cyc_Absyn_Cnst{struct _union_Cnst_Null_c Null_c;struct _union_Cnst_Char_c Char_c;struct _union_Cnst_Wchar_c Wchar_c;struct _union_Cnst_Short_c Short_c;struct _union_Cnst_Int_c Int_c;struct _union_Cnst_LongLong_c LongLong_c;struct _union_Cnst_Float_c Float_c;struct _union_Cnst_String_c String_c;struct _union_Cnst_Wstring_c Wstring_c;};
# 532
enum Cyc_Absyn_Primop{Cyc_Absyn_Plus =0U,Cyc_Absyn_Times =1U,Cyc_Absyn_Minus =2U,Cyc_Absyn_Div =3U,Cyc_Absyn_Mod =4U,Cyc_Absyn_Eq =5U,Cyc_Absyn_Neq =6U,Cyc_Absyn_Gt =7U,Cyc_Absyn_Lt =8U,Cyc_Absyn_Gte =9U,Cyc_Absyn_Lte =10U,Cyc_Absyn_Not =11U,Cyc_Absyn_Bitnot =12U,Cyc_Absyn_Bitand =13U,Cyc_Absyn_Bitor =14U,Cyc_Absyn_Bitxor =15U,Cyc_Absyn_Bitlshift =16U,Cyc_Absyn_Bitlrshift =17U,Cyc_Absyn_Numelts =18U};
# 539
enum Cyc_Absyn_Incrementor{Cyc_Absyn_PreInc =0U,Cyc_Absyn_PostInc =1U,Cyc_Absyn_PreDec =2U,Cyc_Absyn_PostDec =3U};struct Cyc_Absyn_VarargCallInfo{int num_varargs;struct Cyc_List_List*injectors;struct Cyc_Absyn_VarargInfo*vai;};struct Cyc_Absyn_StructField_Absyn_OffsetofField_struct{int tag;struct _fat_ptr*f1;};struct Cyc_Absyn_TupleIndex_Absyn_OffsetofField_struct{int tag;unsigned f1;};
# 557
enum Cyc_Absyn_Coercion{Cyc_Absyn_Unknown_coercion =0U,Cyc_Absyn_No_coercion =1U,Cyc_Absyn_Null_to_NonNull =2U,Cyc_Absyn_Other_coercion =3U};struct Cyc_Absyn_MallocInfo{int is_calloc;struct Cyc_Absyn_Exp*rgn;void**elt_type;struct Cyc_Absyn_Exp*num_elts;int fat_result;int inline_call;};struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct{int tag;union Cyc_Absyn_Cnst f1;};struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Pragma_e_Absyn_Raw_exp_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct{int tag;enum Cyc_Absyn_Primop f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_AssignOp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Core_Opt*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_Increment_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;enum Cyc_Absyn_Incrementor f2;};struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;struct Cyc_Absyn_VarargCallInfo*f3;int f4;};struct Cyc_Absyn_Throw_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;int f2;};struct Cyc_Absyn_NoInstantiate_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Instantiate_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_Absyn_Exp*f2;int f3;enum Cyc_Absyn_Coercion f4;};struct Cyc_Absyn_Address_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_New_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Sizeoftype_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Sizeofexp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Offsetof_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Deref_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;int f3;int f4;};struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;int f3;int f4;};struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Tuple_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;};struct _tuple8{struct _fat_ptr*f1;struct Cyc_Absyn_Tqual f2;void*f3;};struct Cyc_Absyn_CompoundLit_e_Absyn_Raw_exp_struct{int tag;struct _tuple8*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Array_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Exp*f2;struct Cyc_Absyn_Exp*f3;int f4;};struct Cyc_Absyn_ComprehensionNoinit_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;void*f2;int f3;};struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct{int tag;struct _tuple0*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;struct Cyc_Absyn_Aggrdecl*f4;};struct Cyc_Absyn_AnonStruct_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Datatype_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;struct Cyc_Absyn_Datatypedecl*f2;struct Cyc_Absyn_Datatypefield*f3;};struct Cyc_Absyn_Enum_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_AnonEnum_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_Spawn_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_MallocInfo f1;};struct Cyc_Absyn_Swap_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_UnresolvedMem_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Core_Opt*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_StmtExp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Stmt*f1;};struct Cyc_Absyn_Tagcheck_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;};struct Cyc_Absyn_Valueof_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Asm_e_Absyn_Raw_exp_struct{int tag;int f1;struct _fat_ptr f2;struct Cyc_List_List*f3;struct Cyc_List_List*f4;struct Cyc_List_List*f5;};struct Cyc_Absyn_Extension_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Exp{void*topt;void*r;unsigned loc;void*annot;};struct Cyc_Absyn_Skip_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Exp_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Seq_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Return_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_IfThenElse_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Stmt*f2;struct Cyc_Absyn_Stmt*f3;};struct _tuple9{struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_While_s_Absyn_Raw_stmt_struct{int tag;struct _tuple9 f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Break_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Continue_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Goto_s_Absyn_Raw_stmt_struct{int tag;struct _fat_ptr*f1;};struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _tuple9 f2;struct _tuple9 f3;struct Cyc_Absyn_Stmt*f4;};struct Cyc_Absyn_Switch_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;void*f3;};struct Cyc_Absyn_Fallthru_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_List_List*f1;struct Cyc_Absyn_Switch_clause**f2;};struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Decl*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Label_s_Absyn_Raw_stmt_struct{int tag;struct _fat_ptr*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Do_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct _tuple9 f2;};struct Cyc_Absyn_TryCatch_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct Cyc_List_List*f2;void*f3;};struct Cyc_Absyn_Stmt{void*r;unsigned loc;void*annot;};struct Cyc_Absyn_Wild_p_Absyn_Raw_pat_struct{int tag;};struct Cyc_Absyn_Var_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Pat*f2;};struct Cyc_Absyn_AliasVar_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;};struct Cyc_Absyn_Reference_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Pat*f2;};struct Cyc_Absyn_TagInt_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;};struct Cyc_Absyn_Tuple_p_Absyn_Raw_pat_struct{int tag;struct Cyc_List_List*f1;int f2;};struct Cyc_Absyn_Pointer_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Pat*f1;};struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct{int tag;union Cyc_Absyn_AggrInfo*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;int f4;};struct Cyc_Absyn_Datatype_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;struct Cyc_Absyn_Datatypefield*f2;struct Cyc_List_List*f3;int f4;};struct Cyc_Absyn_Null_p_Absyn_Raw_pat_struct{int tag;};struct Cyc_Absyn_Int_p_Absyn_Raw_pat_struct{int tag;enum Cyc_Absyn_Sign f1;int f2;};struct Cyc_Absyn_Char_p_Absyn_Raw_pat_struct{int tag;char f1;};struct Cyc_Absyn_Float_p_Absyn_Raw_pat_struct{int tag;struct _fat_ptr f1;int f2;};struct Cyc_Absyn_Enum_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_AnonEnum_p_Absyn_Raw_pat_struct{int tag;void*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_UnknownId_p_Absyn_Raw_pat_struct{int tag;struct _tuple0*f1;};struct Cyc_Absyn_UnknownCall_p_Absyn_Raw_pat_struct{int tag;struct _tuple0*f1;struct Cyc_List_List*f2;int f3;};struct Cyc_Absyn_Exp_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Pat{void*r;void*topt;unsigned loc;};struct Cyc_Absyn_Switch_clause{struct Cyc_Absyn_Pat*pattern;struct Cyc_Core_Opt*pat_vars;struct Cyc_Absyn_Exp*where_clause;struct Cyc_Absyn_Stmt*body;unsigned loc;};struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct{int tag;struct _tuple0*f1;};struct Cyc_Absyn_Global_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Funname_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Fndecl*f1;};struct Cyc_Absyn_Param_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Local_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Pat_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Vardecl{enum Cyc_Absyn_Scope sc;struct _tuple0*name;unsigned varloc;struct Cyc_Absyn_Tqual tq;void*type;struct Cyc_Absyn_Exp*initializer;void*rgn;struct Cyc_List_List*attributes;int escapes;};struct Cyc_Absyn_Fndecl{enum Cyc_Absyn_Scope sc;int is_inline;struct _tuple0*name;struct Cyc_Absyn_Stmt*body;struct Cyc_Absyn_FnInfo i;void*cached_type;struct Cyc_Core_Opt*param_vardecls;struct Cyc_Absyn_Vardecl*fn_vardecl;};struct Cyc_Absyn_Aggrfield{struct _fat_ptr*name;struct Cyc_Absyn_Tqual tq;void*type;struct Cyc_Absyn_Exp*width;struct Cyc_List_List*attributes;struct Cyc_Absyn_Exp*requires_clause;};struct Cyc_Absyn_AggrdeclImpl{struct Cyc_List_List*exist_vars;struct Cyc_List_List*rgn_po;struct Cyc_List_List*fields;int tagged;};struct Cyc_Absyn_Aggrdecl{enum Cyc_Absyn_AggrKind kind;enum Cyc_Absyn_Scope sc;struct _tuple0*name;struct Cyc_List_List*tvs;struct Cyc_Absyn_AggrdeclImpl*impl;struct Cyc_List_List*attributes;int expected_mem_kind;};struct Cyc_Absyn_Datatypefield{struct _tuple0*name;struct Cyc_List_List*typs;unsigned loc;enum Cyc_Absyn_Scope sc;};struct Cyc_Absyn_Datatypedecl{enum Cyc_Absyn_Scope sc;struct _tuple0*name;struct Cyc_List_List*tvs;struct Cyc_Core_Opt*fields;int is_extensible;};struct Cyc_Absyn_Enumfield{struct _tuple0*name;struct Cyc_Absyn_Exp*tag;unsigned loc;};struct Cyc_Absyn_Enumdecl{enum Cyc_Absyn_Scope sc;struct _tuple0*name;struct Cyc_Core_Opt*fields;};struct Cyc_Absyn_Typedefdecl{struct _tuple0*name;struct Cyc_Absyn_Tqual tq;struct Cyc_List_List*tvs;struct Cyc_Core_Opt*kind;void*defn;struct Cyc_List_List*atts;int extern_c;};struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Fn_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Fndecl*f1;};struct Cyc_Absyn_Let_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Pat*f1;struct Cyc_Core_Opt*f2;struct Cyc_Absyn_Exp*f3;void*f4;};struct Cyc_Absyn_Letv_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_Region_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_Aggr_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Aggrdecl*f1;};struct Cyc_Absyn_Datatype_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;};struct Cyc_Absyn_Enum_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;};struct Cyc_Absyn_Typedef_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Typedefdecl*f1;};struct Cyc_Absyn_Namespace_d_Absyn_Raw_decl_struct{int tag;struct _fat_ptr*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Using_d_Absyn_Raw_decl_struct{int tag;struct _tuple0*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_ExternC_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;};struct _tuple10{unsigned f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_ExternCinclude_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;struct _tuple10*f4;};struct Cyc_Absyn_Porton_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Portoff_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Tempeston_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Tempestoff_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Decl{void*r;unsigned loc;};struct Cyc_Absyn_ArrayElement_Absyn_Designator_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_FieldName_Absyn_Designator_struct{int tag;struct _fat_ptr*f1;};extern char Cyc_Absyn_EmptyAnnot[11U];struct Cyc_Absyn_EmptyAnnot_Absyn_AbsynAnnot_struct{char*tag;};
# 908 "absyn.h"
int Cyc_Absyn_qvar_cmp(struct _tuple0*,struct _tuple0*);
# 917
struct Cyc_Absyn_Tqual Cyc_Absyn_empty_tqual(unsigned);
# 935
void*Cyc_Absyn_wildtyp(struct Cyc_Core_Opt*);
# 940
extern void*Cyc_Absyn_sint_type;
# 951
void*Cyc_Absyn_enum_type(struct _tuple0*n,struct Cyc_Absyn_Enumdecl*d);
# 973
void*Cyc_Absyn_string_type(void*rgn);
# 976
extern void*Cyc_Absyn_fat_bound_type;
# 996
void*Cyc_Absyn_fatptr_type(void*t,void*rgn,struct Cyc_Absyn_Tqual,void*zeroterm);
# 1182
struct _tuple0*Cyc_Absyn_binding2qvar(void*);struct Cyc_Absynpp_Params{int expand_typedefs;int qvar_to_Cids;int add_cyc_prefix;int to_VC;int decls_first;int rewrite_temp_tvars;int print_all_tvars;int print_all_kinds;int print_all_effects;int print_using_stmts;int print_externC_stmts;int print_full_evars;int print_zeroterm;int generate_line_directives;int use_curr_namespace;struct Cyc_List_List*curr_namespace;};
# 62 "absynpp.h"
struct _fat_ptr Cyc_Absynpp_typ2string(void*);
# 67
struct _fat_ptr Cyc_Absynpp_exp2string(struct Cyc_Absyn_Exp*);
# 69
struct _fat_ptr Cyc_Absynpp_qvar2string(struct _tuple0*);struct _union_RelnOp_RConst{int tag;unsigned val;};struct _union_RelnOp_RVar{int tag;struct Cyc_Absyn_Vardecl*val;};struct _union_RelnOp_RNumelts{int tag;struct Cyc_Absyn_Vardecl*val;};struct _union_RelnOp_RType{int tag;void*val;};struct _union_RelnOp_RParam{int tag;unsigned val;};struct _union_RelnOp_RParamNumelts{int tag;unsigned val;};struct _union_RelnOp_RReturn{int tag;unsigned val;};union Cyc_Relations_RelnOp{struct _union_RelnOp_RConst RConst;struct _union_RelnOp_RVar RVar;struct _union_RelnOp_RNumelts RNumelts;struct _union_RelnOp_RType RType;struct _union_RelnOp_RParam RParam;struct _union_RelnOp_RParamNumelts RParamNumelts;struct _union_RelnOp_RReturn RReturn;};
# 50 "relations-ap.h"
enum Cyc_Relations_Relation{Cyc_Relations_Req =0U,Cyc_Relations_Rneq =1U,Cyc_Relations_Rlte =2U,Cyc_Relations_Rlt =3U};struct Cyc_Relations_Reln{union Cyc_Relations_RelnOp rop1;enum Cyc_Relations_Relation relation;union Cyc_Relations_RelnOp rop2;};struct Cyc_Iter_Iter{void*env;int(*next)(void*env,void*dest);};struct Cyc_Dict_T;struct Cyc_Dict_Dict{int(*rel)(void*,void*);struct _RegionHandle*r;const struct Cyc_Dict_T*t;};extern char Cyc_Dict_Present[8U];struct Cyc_Dict_Present_exn_struct{char*tag;};extern char Cyc_Dict_Absent[7U];struct Cyc_Dict_Absent_exn_struct{char*tag;};
# 62 "dict.h"
struct Cyc_Dict_Dict Cyc_Dict_empty(int(*cmp)(void*,void*));
# 83
int Cyc_Dict_member(struct Cyc_Dict_Dict d,void*k);
# 87
struct Cyc_Dict_Dict Cyc_Dict_insert(struct Cyc_Dict_Dict d,void*k,void*v);
# 110
void*Cyc_Dict_lookup(struct Cyc_Dict_Dict d,void*k);
# 122 "dict.h"
void**Cyc_Dict_lookup_opt(struct Cyc_Dict_Dict d,void*k);struct Cyc_RgnOrder_RgnPO;extern char Cyc_Tcenv_Env_error[10U];struct Cyc_Tcenv_Env_error_exn_struct{char*tag;};struct Cyc_Tcenv_Genv{struct Cyc_Dict_Dict aggrdecls;struct Cyc_Dict_Dict datatypedecls;struct Cyc_Dict_Dict enumdecls;struct Cyc_Dict_Dict typedefs;struct Cyc_Dict_Dict ordinaries;};struct Cyc_Tcenv_Fenv;struct Cyc_Tcenv_Tenv{struct Cyc_List_List*ns;struct Cyc_Tcenv_Genv*ae;struct Cyc_Tcenv_Fenv*le;int allow_valueof: 1;int in_extern_c_include: 1;int in_tempest: 1;int tempest_generalize: 1;};
# 88 "tcenv.h"
enum Cyc_Tcenv_NewStatus{Cyc_Tcenv_NoneNew =0U,Cyc_Tcenv_InNew =1U,Cyc_Tcenv_InNewAggr =2U};
# 30 "tcutil.h"
void*Cyc_Tcutil_impos(struct _fat_ptr,struct _fat_ptr);
# 72
struct Cyc_Absyn_Exp*Cyc_Tcutil_get_bounds_exp(void*def,void*b);
# 84
struct Cyc_Absyn_Exp*Cyc_Tcutil_get_bounds_exp(void*def,void*b);
# 107
void*Cyc_Tcutil_compress(void*);
# 305
int Cyc_Tcutil_force_type2bool(int desired,void*);
# 308
void*Cyc_Tcutil_any_bool(struct Cyc_List_List*);extern char Cyc_Tcdecl_Incompatible[13U];struct Cyc_Tcdecl_Incompatible_exn_struct{char*tag;};struct Cyc_Tcdecl_Xdatatypefielddecl{struct Cyc_Absyn_Datatypedecl*base;struct Cyc_Absyn_Datatypefield*field;};struct Cyc_Set_Set;extern char Cyc_Set_Absent[7U];struct Cyc_Set_Absent_exn_struct{char*tag;};struct Cyc_Hashtable_Table;
# 39 "hashtable.h"
struct Cyc_Hashtable_Table*Cyc_Hashtable_create(int sz,int(*cmp)(void*,void*),int(*hash)(void*));
# 50
void Cyc_Hashtable_insert(struct Cyc_Hashtable_Table*t,void*key,void*val);
# 52
void*Cyc_Hashtable_lookup(struct Cyc_Hashtable_Table*t,void*key);
# 59
int Cyc_Hashtable_try_lookup(struct Cyc_Hashtable_Table*t,void*key,void**data);struct Cyc_Port_Edit{unsigned loc;struct _fat_ptr old_string;struct _fat_ptr new_string;};
# 88 "port.cyc"
int Cyc_Port_cmp_edit(struct Cyc_Port_Edit*e1,struct Cyc_Port_Edit*e2){
return(int)e1 - (int)e2;}
# 88
static unsigned Cyc_Port_get_seg(struct Cyc_Port_Edit*e){
# 92
return e->loc;}
# 88
int Cyc_Port_cmp_seg_t(unsigned loc1,unsigned loc2){
# 95
return(int)(loc1 - loc2);}
# 88
int Cyc_Port_hash_seg_t(unsigned loc){
# 98
return(int)loc;}struct Cyc_Port_VarUsage{int address_taken;struct Cyc_Absyn_Exp*init;struct Cyc_List_List*usage_locs;};struct Cyc_Port_Cvar{int id;void**eq;struct Cyc_List_List*uppers;struct Cyc_List_List*lowers;};struct Cyc_Port_Cfield{void*qual;struct _fat_ptr*f;void*type;};struct Cyc_Port_Const_ct_Port_Ctype_struct{int tag;};struct Cyc_Port_Notconst_ct_Port_Ctype_struct{int tag;};struct Cyc_Port_Thin_ct_Port_Ctype_struct{int tag;};struct Cyc_Port_Fat_ct_Port_Ctype_struct{int tag;};struct Cyc_Port_Void_ct_Port_Ctype_struct{int tag;};struct Cyc_Port_Zero_ct_Port_Ctype_struct{int tag;};struct Cyc_Port_Arith_ct_Port_Ctype_struct{int tag;};struct Cyc_Port_Heap_ct_Port_Ctype_struct{int tag;};struct Cyc_Port_Zterm_ct_Port_Ctype_struct{int tag;};struct Cyc_Port_Nozterm_ct_Port_Ctype_struct{int tag;};struct Cyc_Port_RgnVar_ct_Port_Ctype_struct{int tag;struct _fat_ptr*f1;};struct Cyc_Port_Ptr_ct_Port_Ctype_struct{int tag;void*f1;void*f2;void*f3;void*f4;void*f5;};struct Cyc_Port_Array_ct_Port_Ctype_struct{int tag;void*f1;void*f2;void*f3;};struct _tuple11{struct Cyc_Absyn_Aggrdecl*f1;struct Cyc_List_List*f2;};struct Cyc_Port_KnownAggr_ct_Port_Ctype_struct{int tag;struct _tuple11*f1;};struct Cyc_Port_UnknownAggr_ct_Port_Ctype_struct{int tag;struct Cyc_List_List*f1;void**f2;};struct Cyc_Port_Fn_ct_Port_Ctype_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Port_Var_ct_Port_Ctype_struct{int tag;struct Cyc_Port_Cvar*f1;};
# 153
struct Cyc_Port_Const_ct_Port_Ctype_struct Cyc_Port_Const_ct_val={0U};
struct Cyc_Port_Notconst_ct_Port_Ctype_struct Cyc_Port_Notconst_ct_val={1U};
struct Cyc_Port_Thin_ct_Port_Ctype_struct Cyc_Port_Thin_ct_val={2U};
struct Cyc_Port_Fat_ct_Port_Ctype_struct Cyc_Port_Fat_ct_val={3U};
struct Cyc_Port_Void_ct_Port_Ctype_struct Cyc_Port_Void_ct_val={4U};
struct Cyc_Port_Zero_ct_Port_Ctype_struct Cyc_Port_Zero_ct_val={5U};
struct Cyc_Port_Arith_ct_Port_Ctype_struct Cyc_Port_Arith_ct_val={6U};
struct Cyc_Port_Heap_ct_Port_Ctype_struct Cyc_Port_Heap_ct_val={7U};
struct Cyc_Port_Zterm_ct_Port_Ctype_struct Cyc_Port_Zterm_ct_val={8U};
struct Cyc_Port_Nozterm_ct_Port_Ctype_struct Cyc_Port_Nozterm_ct_val={9U};
# 166
static struct _fat_ptr Cyc_Port_ctypes2string(int deep,struct Cyc_List_List*ts);
static struct _fat_ptr Cyc_Port_cfields2string(int deep,struct Cyc_List_List*ts);
static struct _fat_ptr Cyc_Port_ctype2string(int deep,void*t){
void*_tmp0=t;struct Cyc_Port_Cvar*_tmp1;struct Cyc_List_List*_tmp3;void*_tmp2;struct Cyc_List_List*_tmp4;void*_tmp6;struct Cyc_List_List*_tmp5;struct Cyc_List_List*_tmp8;struct Cyc_Absyn_Aggrdecl*_tmp7;void*_tmpB;void*_tmpA;void*_tmp9;void*_tmp10;void*_tmpF;void*_tmpE;void*_tmpD;void*_tmpC;struct _fat_ptr*_tmp11;switch(*((int*)_tmp0)){case 0U: _LL1: _LL2:
 return({const char*_tmp12="const";_tag_fat(_tmp12,sizeof(char),6U);});case 1U: _LL3: _LL4:
 return({const char*_tmp13="notconst";_tag_fat(_tmp13,sizeof(char),9U);});case 2U: _LL5: _LL6:
 return({const char*_tmp14="thin";_tag_fat(_tmp14,sizeof(char),5U);});case 3U: _LL7: _LL8:
 return({const char*_tmp15="fat";_tag_fat(_tmp15,sizeof(char),4U);});case 4U: _LL9: _LLA:
 return({const char*_tmp16="void";_tag_fat(_tmp16,sizeof(char),5U);});case 5U: _LLB: _LLC:
 return({const char*_tmp17="zero";_tag_fat(_tmp17,sizeof(char),5U);});case 6U: _LLD: _LLE:
 return({const char*_tmp18="arith";_tag_fat(_tmp18,sizeof(char),6U);});case 7U: _LLF: _LL10:
 return({const char*_tmp19="heap";_tag_fat(_tmp19,sizeof(char),5U);});case 8U: _LL11: _LL12:
 return({const char*_tmp1A="ZT";_tag_fat(_tmp1A,sizeof(char),3U);});case 9U: _LL13: _LL14:
 return({const char*_tmp1B="NZT";_tag_fat(_tmp1B,sizeof(char),4U);});case 10U: _LL15: _tmp11=((struct Cyc_Port_RgnVar_ct_Port_Ctype_struct*)_tmp0)->f1;_LL16: {struct _fat_ptr*n=_tmp11;
return(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp1E=({struct Cyc_String_pa_PrintArg_struct _tmp2E6;_tmp2E6.tag=0U,_tmp2E6.f1=(struct _fat_ptr)((struct _fat_ptr)*n);_tmp2E6;});void*_tmp1C[1U];_tmp1C[0]=& _tmp1E;({struct _fat_ptr _tmp316=({const char*_tmp1D="%s";_tag_fat(_tmp1D,sizeof(char),3U);});Cyc_aprintf(_tmp316,_tag_fat(_tmp1C,sizeof(void*),1U));});});}case 11U: _LL17: _tmpC=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmp0)->f1;_tmpD=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmp0)->f2;_tmpE=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmp0)->f3;_tmpF=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmp0)->f4;_tmp10=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmp0)->f5;_LL18: {void*elt=_tmpC;void*qual=_tmpD;void*k=_tmpE;void*rgn=_tmpF;void*zt=_tmp10;
# 182
return(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp21=({struct Cyc_String_pa_PrintArg_struct _tmp2EB;_tmp2EB.tag=0U,({struct _fat_ptr _tmp317=(struct _fat_ptr)((struct _fat_ptr)Cyc_Port_ctype2string(deep,elt));_tmp2EB.f1=_tmp317;});_tmp2EB;});struct Cyc_String_pa_PrintArg_struct _tmp22=({struct Cyc_String_pa_PrintArg_struct _tmp2EA;_tmp2EA.tag=0U,({
struct _fat_ptr _tmp318=(struct _fat_ptr)((struct _fat_ptr)Cyc_Port_ctype2string(deep,qual));_tmp2EA.f1=_tmp318;});_tmp2EA;});struct Cyc_String_pa_PrintArg_struct _tmp23=({struct Cyc_String_pa_PrintArg_struct _tmp2E9;_tmp2E9.tag=0U,({struct _fat_ptr _tmp319=(struct _fat_ptr)((struct _fat_ptr)Cyc_Port_ctype2string(deep,k));_tmp2E9.f1=_tmp319;});_tmp2E9;});struct Cyc_String_pa_PrintArg_struct _tmp24=({struct Cyc_String_pa_PrintArg_struct _tmp2E8;_tmp2E8.tag=0U,({
struct _fat_ptr _tmp31A=(struct _fat_ptr)((struct _fat_ptr)Cyc_Port_ctype2string(deep,rgn));_tmp2E8.f1=_tmp31A;});_tmp2E8;});struct Cyc_String_pa_PrintArg_struct _tmp25=({struct Cyc_String_pa_PrintArg_struct _tmp2E7;_tmp2E7.tag=0U,({struct _fat_ptr _tmp31B=(struct _fat_ptr)((struct _fat_ptr)Cyc_Port_ctype2string(deep,zt));_tmp2E7.f1=_tmp31B;});_tmp2E7;});void*_tmp1F[5U];_tmp1F[0]=& _tmp21,_tmp1F[1]=& _tmp22,_tmp1F[2]=& _tmp23,_tmp1F[3]=& _tmp24,_tmp1F[4]=& _tmp25;({struct _fat_ptr _tmp31C=({const char*_tmp20="ptr(%s,%s,%s,%s,%s)";_tag_fat(_tmp20,sizeof(char),20U);});Cyc_aprintf(_tmp31C,_tag_fat(_tmp1F,sizeof(void*),5U));});});}case 12U: _LL19: _tmp9=(void*)((struct Cyc_Port_Array_ct_Port_Ctype_struct*)_tmp0)->f1;_tmpA=(void*)((struct Cyc_Port_Array_ct_Port_Ctype_struct*)_tmp0)->f2;_tmpB=(void*)((struct Cyc_Port_Array_ct_Port_Ctype_struct*)_tmp0)->f3;_LL1A: {void*elt=_tmp9;void*qual=_tmpA;void*zt=_tmpB;
# 186
return(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp28=({struct Cyc_String_pa_PrintArg_struct _tmp2EE;_tmp2EE.tag=0U,({struct _fat_ptr _tmp31D=(struct _fat_ptr)((struct _fat_ptr)Cyc_Port_ctype2string(deep,elt));_tmp2EE.f1=_tmp31D;});_tmp2EE;});struct Cyc_String_pa_PrintArg_struct _tmp29=({struct Cyc_String_pa_PrintArg_struct _tmp2ED;_tmp2ED.tag=0U,({
struct _fat_ptr _tmp31E=(struct _fat_ptr)((struct _fat_ptr)Cyc_Port_ctype2string(deep,qual));_tmp2ED.f1=_tmp31E;});_tmp2ED;});struct Cyc_String_pa_PrintArg_struct _tmp2A=({struct Cyc_String_pa_PrintArg_struct _tmp2EC;_tmp2EC.tag=0U,({struct _fat_ptr _tmp31F=(struct _fat_ptr)((struct _fat_ptr)Cyc_Port_ctype2string(deep,zt));_tmp2EC.f1=_tmp31F;});_tmp2EC;});void*_tmp26[3U];_tmp26[0]=& _tmp28,_tmp26[1]=& _tmp29,_tmp26[2]=& _tmp2A;({struct _fat_ptr _tmp320=({const char*_tmp27="array(%s,%s,%s)";_tag_fat(_tmp27,sizeof(char),16U);});Cyc_aprintf(_tmp320,_tag_fat(_tmp26,sizeof(void*),3U));});});}case 13U: _LL1B: _tmp7=(((struct Cyc_Port_KnownAggr_ct_Port_Ctype_struct*)_tmp0)->f1)->f1;_tmp8=(((struct Cyc_Port_KnownAggr_ct_Port_Ctype_struct*)_tmp0)->f1)->f2;_LL1C: {struct Cyc_Absyn_Aggrdecl*ad=_tmp7;struct Cyc_List_List*cfs=_tmp8;
# 189
struct _fat_ptr s=(int)ad->kind == (int)Cyc_Absyn_StructA?({const char*_tmp34="struct";_tag_fat(_tmp34,sizeof(char),7U);}):({const char*_tmp35="union";_tag_fat(_tmp35,sizeof(char),6U);});
if(!deep)
return(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp2D=({struct Cyc_String_pa_PrintArg_struct _tmp2F0;_tmp2F0.tag=0U,_tmp2F0.f1=(struct _fat_ptr)((struct _fat_ptr)s);_tmp2F0;});struct Cyc_String_pa_PrintArg_struct _tmp2E=({struct Cyc_String_pa_PrintArg_struct _tmp2EF;_tmp2EF.tag=0U,({struct _fat_ptr _tmp321=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(ad->name));_tmp2EF.f1=_tmp321;});_tmp2EF;});void*_tmp2B[2U];_tmp2B[0]=& _tmp2D,_tmp2B[1]=& _tmp2E;({struct _fat_ptr _tmp322=({const char*_tmp2C="%s %s";_tag_fat(_tmp2C,sizeof(char),6U);});Cyc_aprintf(_tmp322,_tag_fat(_tmp2B,sizeof(void*),2U));});});else{
# 193
return(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp31=({struct Cyc_String_pa_PrintArg_struct _tmp2F3;_tmp2F3.tag=0U,_tmp2F3.f1=(struct _fat_ptr)((struct _fat_ptr)s);_tmp2F3;});struct Cyc_String_pa_PrintArg_struct _tmp32=({struct Cyc_String_pa_PrintArg_struct _tmp2F2;_tmp2F2.tag=0U,({struct _fat_ptr _tmp323=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(ad->name));_tmp2F2.f1=_tmp323;});_tmp2F2;});struct Cyc_String_pa_PrintArg_struct _tmp33=({struct Cyc_String_pa_PrintArg_struct _tmp2F1;_tmp2F1.tag=0U,({
struct _fat_ptr _tmp324=(struct _fat_ptr)((struct _fat_ptr)Cyc_Port_cfields2string(0,cfs));_tmp2F1.f1=_tmp324;});_tmp2F1;});void*_tmp2F[3U];_tmp2F[0]=& _tmp31,_tmp2F[1]=& _tmp32,_tmp2F[2]=& _tmp33;({struct _fat_ptr _tmp325=({const char*_tmp30="%s %s {%s}";_tag_fat(_tmp30,sizeof(char),11U);});Cyc_aprintf(_tmp325,_tag_fat(_tmp2F,sizeof(void*),3U));});});}}case 14U: if(((struct Cyc_Port_UnknownAggr_ct_Port_Ctype_struct*)_tmp0)->f2 != 0){_LL1D: _tmp5=((struct Cyc_Port_UnknownAggr_ct_Port_Ctype_struct*)_tmp0)->f1;_tmp6=*((struct Cyc_Port_UnknownAggr_ct_Port_Ctype_struct*)_tmp0)->f2;_LL1E: {struct Cyc_List_List*cfs=_tmp5;void*eq=_tmp6;
return Cyc_Port_ctype2string(deep,eq);}}else{_LL1F: _tmp4=((struct Cyc_Port_UnknownAggr_ct_Port_Ctype_struct*)_tmp0)->f1;_LL20: {struct Cyc_List_List*cfs=_tmp4;
# 197
return(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp38=({struct Cyc_String_pa_PrintArg_struct _tmp2F4;_tmp2F4.tag=0U,({struct _fat_ptr _tmp326=(struct _fat_ptr)((struct _fat_ptr)Cyc_Port_cfields2string(deep,cfs));_tmp2F4.f1=_tmp326;});_tmp2F4;});void*_tmp36[1U];_tmp36[0]=& _tmp38;({struct _fat_ptr _tmp327=({const char*_tmp37="aggr {%s}";_tag_fat(_tmp37,sizeof(char),10U);});Cyc_aprintf(_tmp327,_tag_fat(_tmp36,sizeof(void*),1U));});});}}case 15U: _LL21: _tmp2=(void*)((struct Cyc_Port_Fn_ct_Port_Ctype_struct*)_tmp0)->f1;_tmp3=((struct Cyc_Port_Fn_ct_Port_Ctype_struct*)_tmp0)->f2;_LL22: {void*t=_tmp2;struct Cyc_List_List*ts=_tmp3;
# 199
return(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp3B=({struct Cyc_String_pa_PrintArg_struct _tmp2F6;_tmp2F6.tag=0U,({struct _fat_ptr _tmp328=(struct _fat_ptr)((struct _fat_ptr)Cyc_Port_ctypes2string(deep,ts));_tmp2F6.f1=_tmp328;});_tmp2F6;});struct Cyc_String_pa_PrintArg_struct _tmp3C=({struct Cyc_String_pa_PrintArg_struct _tmp2F5;_tmp2F5.tag=0U,({struct _fat_ptr _tmp329=(struct _fat_ptr)((struct _fat_ptr)Cyc_Port_ctype2string(deep,t));_tmp2F5.f1=_tmp329;});_tmp2F5;});void*_tmp39[2U];_tmp39[0]=& _tmp3B,_tmp39[1]=& _tmp3C;({struct _fat_ptr _tmp32A=({const char*_tmp3A="fn(%s)->%s";_tag_fat(_tmp3A,sizeof(char),11U);});Cyc_aprintf(_tmp32A,_tag_fat(_tmp39,sizeof(void*),2U));});});}default: _LL23: _tmp1=((struct Cyc_Port_Var_ct_Port_Ctype_struct*)_tmp0)->f1;_LL24: {struct Cyc_Port_Cvar*cv=_tmp1;
# 201
if((unsigned)cv->eq)
return Cyc_Port_ctype2string(deep,*((void**)_check_null(cv->eq)));else{
if(!deep || cv->uppers == 0 && cv->lowers == 0)
return(struct _fat_ptr)({struct Cyc_Int_pa_PrintArg_struct _tmp3F=({struct Cyc_Int_pa_PrintArg_struct _tmp2F7;_tmp2F7.tag=1U,_tmp2F7.f1=(unsigned long)cv->id;_tmp2F7;});void*_tmp3D[1U];_tmp3D[0]=& _tmp3F;({struct _fat_ptr _tmp32B=({const char*_tmp3E="var(%d)";_tag_fat(_tmp3E,sizeof(char),8U);});Cyc_aprintf(_tmp32B,_tag_fat(_tmp3D,sizeof(void*),1U));});});else{
return(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp42=({struct Cyc_String_pa_PrintArg_struct _tmp2FA;_tmp2FA.tag=0U,({
struct _fat_ptr _tmp32C=(struct _fat_ptr)((struct _fat_ptr)Cyc_Port_ctypes2string(0,cv->lowers));_tmp2FA.f1=_tmp32C;});_tmp2FA;});struct Cyc_Int_pa_PrintArg_struct _tmp43=({struct Cyc_Int_pa_PrintArg_struct _tmp2F9;_tmp2F9.tag=1U,_tmp2F9.f1=(unsigned long)cv->id;_tmp2F9;});struct Cyc_String_pa_PrintArg_struct _tmp44=({struct Cyc_String_pa_PrintArg_struct _tmp2F8;_tmp2F8.tag=0U,({
# 208
struct _fat_ptr _tmp32D=(struct _fat_ptr)((struct _fat_ptr)Cyc_Port_ctypes2string(0,cv->uppers));_tmp2F8.f1=_tmp32D;});_tmp2F8;});void*_tmp40[3U];_tmp40[0]=& _tmp42,_tmp40[1]=& _tmp43,_tmp40[2]=& _tmp44;({struct _fat_ptr _tmp32E=({const char*_tmp41="var([%s]<=%d<=[%s])";_tag_fat(_tmp41,sizeof(char),20U);});Cyc_aprintf(_tmp32E,_tag_fat(_tmp40,sizeof(void*),3U));});});}}}}_LL0:;}
# 168
static struct _fat_ptr*Cyc_Port_ctype2stringptr(int deep,void*t){
# 211
return({struct _fat_ptr*_tmp45=_cycalloc(sizeof(*_tmp45));({struct _fat_ptr _tmp32F=Cyc_Port_ctype2string(deep,t);*_tmp45=_tmp32F;});_tmp45;});}
# 168
struct Cyc_List_List*Cyc_Port_sep(struct _fat_ptr s,struct Cyc_List_List*xs){
# 213
struct _fat_ptr*sptr=({struct _fat_ptr*_tmp47=_cycalloc(sizeof(*_tmp47));*_tmp47=s;_tmp47;});
if(xs == 0)return xs;{struct Cyc_List_List*prev=xs;
# 216
struct Cyc_List_List*curr=xs->tl;
for(0;curr != 0;(prev=curr,curr=curr->tl)){
({struct Cyc_List_List*_tmp330=({struct Cyc_List_List*_tmp46=_cycalloc(sizeof(*_tmp46));_tmp46->hd=sptr,_tmp46->tl=curr;_tmp46;});prev->tl=_tmp330;});}
# 220
return xs;}}
# 168
static struct _fat_ptr*Cyc_Port_cfield2stringptr(int deep,struct Cyc_Port_Cfield*f){
# 223
struct _fat_ptr s=(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp4B=({struct Cyc_String_pa_PrintArg_struct _tmp2FD;_tmp2FD.tag=0U,({
struct _fat_ptr _tmp331=(struct _fat_ptr)((struct _fat_ptr)Cyc_Port_ctype2string(deep,f->qual));_tmp2FD.f1=_tmp331;});_tmp2FD;});struct Cyc_String_pa_PrintArg_struct _tmp4C=({struct Cyc_String_pa_PrintArg_struct _tmp2FC;_tmp2FC.tag=0U,_tmp2FC.f1=(struct _fat_ptr)((struct _fat_ptr)*f->f);_tmp2FC;});struct Cyc_String_pa_PrintArg_struct _tmp4D=({struct Cyc_String_pa_PrintArg_struct _tmp2FB;_tmp2FB.tag=0U,({struct _fat_ptr _tmp332=(struct _fat_ptr)((struct _fat_ptr)Cyc_Port_ctype2string(deep,f->type));_tmp2FB.f1=_tmp332;});_tmp2FB;});void*_tmp49[3U];_tmp49[0]=& _tmp4B,_tmp49[1]=& _tmp4C,_tmp49[2]=& _tmp4D;({struct _fat_ptr _tmp333=({const char*_tmp4A="%s %s: %s";_tag_fat(_tmp4A,sizeof(char),10U);});Cyc_aprintf(_tmp333,_tag_fat(_tmp49,sizeof(void*),3U));});});
return({struct _fat_ptr*_tmp48=_cycalloc(sizeof(*_tmp48));*_tmp48=s;_tmp48;});}
# 168
static struct _fat_ptr Cyc_Port_ctypes2string(int deep,struct Cyc_List_List*ts){
# 228
return(struct _fat_ptr)Cyc_strconcat_l(({struct _fat_ptr _tmp336=({const char*_tmp4E=",";_tag_fat(_tmp4E,sizeof(char),2U);});Cyc_Port_sep(_tmp336,({struct Cyc_List_List*(*_tmp335)(struct _fat_ptr*(*f)(int,void*),int env,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp4F)(struct _fat_ptr*(*f)(int,void*),int env,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct _fat_ptr*(*f)(int,void*),int env,struct Cyc_List_List*x))Cyc_List_map_c;_tmp4F;});int _tmp334=deep;_tmp335(Cyc_Port_ctype2stringptr,_tmp334,ts);}));}));}
# 168
static struct _fat_ptr Cyc_Port_cfields2string(int deep,struct Cyc_List_List*fs){
# 231
return(struct _fat_ptr)Cyc_strconcat_l(({struct _fat_ptr _tmp339=({const char*_tmp50=";";_tag_fat(_tmp50,sizeof(char),2U);});Cyc_Port_sep(_tmp339,({struct Cyc_List_List*(*_tmp338)(struct _fat_ptr*(*f)(int,struct Cyc_Port_Cfield*),int env,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp51)(struct _fat_ptr*(*f)(int,struct Cyc_Port_Cfield*),int env,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct _fat_ptr*(*f)(int,struct Cyc_Port_Cfield*),int env,struct Cyc_List_List*x))Cyc_List_map_c;_tmp51;});int _tmp337=deep;_tmp338(Cyc_Port_cfield2stringptr,_tmp337,fs);}));}));}
# 168
static void*Cyc_Port_notconst_ct(){
# 236
return(void*)& Cyc_Port_Notconst_ct_val;}
# 168
static void*Cyc_Port_const_ct(){
# 237
return(void*)& Cyc_Port_Const_ct_val;}
# 168
static void*Cyc_Port_thin_ct(){
# 238
return(void*)& Cyc_Port_Thin_ct_val;}
# 168
static void*Cyc_Port_fat_ct(){
# 239
return(void*)& Cyc_Port_Fat_ct_val;}
# 168
static void*Cyc_Port_void_ct(){
# 240
return(void*)& Cyc_Port_Void_ct_val;}
# 168
static void*Cyc_Port_zero_ct(){
# 241
return(void*)& Cyc_Port_Zero_ct_val;}
# 168
static void*Cyc_Port_arith_ct(){
# 242
return(void*)& Cyc_Port_Arith_ct_val;}
# 168
static void*Cyc_Port_heap_ct(){
# 243
return(void*)& Cyc_Port_Heap_ct_val;}
# 168
static void*Cyc_Port_zterm_ct(){
# 244
return(void*)& Cyc_Port_Zterm_ct_val;}
# 168
static void*Cyc_Port_nozterm_ct(){
# 245
return(void*)& Cyc_Port_Nozterm_ct_val;}
# 168
static void*Cyc_Port_rgnvar_ct(struct _fat_ptr*n){
# 246
return(void*)({struct Cyc_Port_RgnVar_ct_Port_Ctype_struct*_tmp52=_cycalloc(sizeof(*_tmp52));_tmp52->tag=10U,_tmp52->f1=n;_tmp52;});}
# 168
static void*Cyc_Port_unknown_aggr_ct(struct Cyc_List_List*fs){
# 248
return(void*)({struct Cyc_Port_UnknownAggr_ct_Port_Ctype_struct*_tmp53=_cycalloc(sizeof(*_tmp53));_tmp53->tag=14U,_tmp53->f1=fs,_tmp53->f2=0;_tmp53;});}
# 168
static void*Cyc_Port_known_aggr_ct(struct _tuple11*p){
# 251
return(void*)({struct Cyc_Port_KnownAggr_ct_Port_Ctype_struct*_tmp54=_cycalloc(sizeof(*_tmp54));_tmp54->tag=13U,_tmp54->f1=p;_tmp54;});}
# 168
static void*Cyc_Port_ptr_ct(void*elt,void*qual,void*ptr_kind,void*r,void*zt){
# 255
return(void*)({struct Cyc_Port_Ptr_ct_Port_Ctype_struct*_tmp55=_cycalloc(sizeof(*_tmp55));_tmp55->tag=11U,_tmp55->f1=elt,_tmp55->f2=qual,_tmp55->f3=ptr_kind,_tmp55->f4=r,_tmp55->f5=zt;_tmp55;});}
# 168
static void*Cyc_Port_array_ct(void*elt,void*qual,void*zt){
# 258
return(void*)({struct Cyc_Port_Array_ct_Port_Ctype_struct*_tmp56=_cycalloc(sizeof(*_tmp56));_tmp56->tag=12U,_tmp56->f1=elt,_tmp56->f2=qual,_tmp56->f3=zt;_tmp56;});}
# 168
static void*Cyc_Port_fn_ct(void*return_type,struct Cyc_List_List*args){
# 261
return(void*)({struct Cyc_Port_Fn_ct_Port_Ctype_struct*_tmp57=_cycalloc(sizeof(*_tmp57));_tmp57->tag=15U,_tmp57->f1=return_type,_tmp57->f2=args;_tmp57;});}
# 168
static void*Cyc_Port_var(){
# 264
static int counter=0;
return(void*)({struct Cyc_Port_Var_ct_Port_Ctype_struct*_tmp59=_cycalloc(sizeof(*_tmp59));_tmp59->tag=16U,({struct Cyc_Port_Cvar*_tmp33A=({struct Cyc_Port_Cvar*_tmp58=_cycalloc(sizeof(*_tmp58));_tmp58->id=counter ++,_tmp58->eq=0,_tmp58->uppers=0,_tmp58->lowers=0;_tmp58;});_tmp59->f1=_tmp33A;});_tmp59;});}
# 168
static void*Cyc_Port_new_var(void*x){
# 268
return Cyc_Port_var();}
# 168
static struct _fat_ptr*Cyc_Port_new_region_var(){
# 271
static int counter=0;
struct _fat_ptr s=(struct _fat_ptr)({struct Cyc_Int_pa_PrintArg_struct _tmp5D=({struct Cyc_Int_pa_PrintArg_struct _tmp2FE;_tmp2FE.tag=1U,_tmp2FE.f1=(unsigned long)counter ++;_tmp2FE;});void*_tmp5B[1U];_tmp5B[0]=& _tmp5D;({struct _fat_ptr _tmp33B=({const char*_tmp5C="`R%d";_tag_fat(_tmp5C,sizeof(char),5U);});Cyc_aprintf(_tmp33B,_tag_fat(_tmp5B,sizeof(void*),1U));});});
return({struct _fat_ptr*_tmp5A=_cycalloc(sizeof(*_tmp5A));*_tmp5A=s;_tmp5A;});}
# 168
static int Cyc_Port_unifies(void*t1,void*t2);
# 280
static void*Cyc_Port_compress_ct(void*t){
void*_tmp5E=t;void**_tmp5F;struct Cyc_List_List*_tmp62;struct Cyc_List_List*_tmp61;void***_tmp60;switch(*((int*)_tmp5E)){case 16U: _LL1: _tmp60=(void***)&(((struct Cyc_Port_Var_ct_Port_Ctype_struct*)_tmp5E)->f1)->eq;_tmp61=(((struct Cyc_Port_Var_ct_Port_Ctype_struct*)_tmp5E)->f1)->uppers;_tmp62=(((struct Cyc_Port_Var_ct_Port_Ctype_struct*)_tmp5E)->f1)->lowers;_LL2: {void***eqp=_tmp60;struct Cyc_List_List*us=_tmp61;struct Cyc_List_List*ls=_tmp62;
# 283
void**eq=*eqp;
if((unsigned)eq){
# 287
void*r=Cyc_Port_compress_ct(*eq);
if(*eq != r)({void**_tmp33C=({void**_tmp63=_cycalloc(sizeof(*_tmp63));*_tmp63=r;_tmp63;});*eqp=_tmp33C;});return r;}
# 284
for(0;ls != 0;ls=ls->tl){
# 295
void*_stmttmp0=(void*)ls->hd;void*_tmp64=_stmttmp0;switch(*((int*)_tmp64)){case 0U: _LL8: _LL9:
 goto _LLB;case 9U: _LLA: _LLB:
 goto _LLD;case 4U: _LLC: _LLD:
 goto _LLF;case 13U: _LLE: _LLF:
 goto _LL11;case 15U: _LL10: _LL11:
# 301
({void**_tmp33D=({void**_tmp65=_cycalloc(sizeof(*_tmp65));*_tmp65=(void*)ls->hd;_tmp65;});*eqp=_tmp33D;});
return(void*)ls->hd;default: _LL12: _LL13:
# 304
 goto _LL7;}_LL7:;}
# 307
for(0;us != 0;us=us->tl){
void*_stmttmp1=(void*)us->hd;void*_tmp66=_stmttmp1;switch(*((int*)_tmp66)){case 1U: _LL15: _LL16:
 goto _LL18;case 8U: _LL17: _LL18:
 goto _LL1A;case 5U: _LL19: _LL1A:
 goto _LL1C;case 13U: _LL1B: _LL1C:
 goto _LL1E;case 15U: _LL1D: _LL1E:
# 314
({void**_tmp33E=({void**_tmp67=_cycalloc(sizeof(*_tmp67));*_tmp67=(void*)us->hd;_tmp67;});*eqp=_tmp33E;});
return(void*)us->hd;default: _LL1F: _LL20:
# 317
 goto _LL14;}_LL14:;}
# 320
return t;}case 14U: _LL3: _tmp5F=((struct Cyc_Port_UnknownAggr_ct_Port_Ctype_struct*)_tmp5E)->f2;_LL4: {void**eq=_tmp5F;
# 323
if((unsigned)eq)return Cyc_Port_compress_ct(*eq);else{
return t;}}default: _LL5: _LL6:
# 327
 return t;}_LL0:;}struct _tuple12{void*f1;void*f2;};
# 280
static void*Cyc_Port_inst(struct Cyc_Dict_Dict*instenv,void*t){
# 334
void*_stmttmp2=Cyc_Port_compress_ct(t);void*_tmp68=_stmttmp2;struct Cyc_List_List*_tmp6A;void*_tmp69;void*_tmp6D;void*_tmp6C;void*_tmp6B;void*_tmp72;void*_tmp71;void*_tmp70;void*_tmp6F;void*_tmp6E;struct _fat_ptr*_tmp73;switch(*((int*)_tmp68)){case 0U: _LL1: _LL2:
 goto _LL4;case 1U: _LL3: _LL4:
 goto _LL6;case 2U: _LL5: _LL6:
 goto _LL8;case 3U: _LL7: _LL8:
 goto _LLA;case 4U: _LL9: _LLA:
 goto _LLC;case 5U: _LLB: _LLC:
 goto _LLE;case 6U: _LLD: _LLE:
 goto _LL10;case 8U: _LLF: _LL10:
 goto _LL12;case 9U: _LL11: _LL12:
 goto _LL14;case 14U: _LL13: _LL14:
 goto _LL16;case 13U: _LL15: _LL16:
 goto _LL18;case 16U: _LL17: _LL18:
 goto _LL1A;case 7U: _LL19: _LL1A:
 return t;case 10U: _LL1B: _tmp73=((struct Cyc_Port_RgnVar_ct_Port_Ctype_struct*)_tmp68)->f1;_LL1C: {struct _fat_ptr*n=_tmp73;
# 349
if(!({int(*_tmp340)(struct Cyc_Dict_Dict d,struct _fat_ptr*k)=({int(*_tmp74)(struct Cyc_Dict_Dict d,struct _fat_ptr*k)=(int(*)(struct Cyc_Dict_Dict d,struct _fat_ptr*k))Cyc_Dict_member;_tmp74;});struct Cyc_Dict_Dict _tmp33F=*instenv;_tmp340(_tmp33F,n);}))
({struct Cyc_Dict_Dict _tmp344=({struct Cyc_Dict_Dict(*_tmp343)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,void*v)=({struct Cyc_Dict_Dict(*_tmp75)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,void*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,void*v))Cyc_Dict_insert;_tmp75;});struct Cyc_Dict_Dict _tmp342=*instenv;struct _fat_ptr*_tmp341=n;_tmp343(_tmp342,_tmp341,Cyc_Port_var());});*instenv=_tmp344;});
# 349
return({
# 351
void*(*_tmp346)(struct Cyc_Dict_Dict d,struct _fat_ptr*k)=({void*(*_tmp76)(struct Cyc_Dict_Dict d,struct _fat_ptr*k)=(void*(*)(struct Cyc_Dict_Dict d,struct _fat_ptr*k))Cyc_Dict_lookup;_tmp76;});struct Cyc_Dict_Dict _tmp345=*instenv;_tmp346(_tmp345,n);});}case 11U: _LL1D: _tmp6E=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmp68)->f1;_tmp6F=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmp68)->f2;_tmp70=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmp68)->f3;_tmp71=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmp68)->f4;_tmp72=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmp68)->f5;_LL1E: {void*t1=_tmp6E;void*t2=_tmp6F;void*t3=_tmp70;void*t4=_tmp71;void*zt=_tmp72;
# 353
struct _tuple12 _stmttmp3=({struct _tuple12 _tmp2FF;({void*_tmp348=Cyc_Port_inst(instenv,t1);_tmp2FF.f1=_tmp348;}),({void*_tmp347=Cyc_Port_inst(instenv,t4);_tmp2FF.f2=_tmp347;});_tmp2FF;});void*_tmp78;void*_tmp77;_LL24: _tmp77=_stmttmp3.f1;_tmp78=_stmttmp3.f2;_LL25: {void*nt1=_tmp77;void*nt4=_tmp78;
if(nt1 == t1 && nt4 == t4)return t;return(void*)({struct Cyc_Port_Ptr_ct_Port_Ctype_struct*_tmp79=_cycalloc(sizeof(*_tmp79));
_tmp79->tag=11U,_tmp79->f1=nt1,_tmp79->f2=t2,_tmp79->f3=t3,_tmp79->f4=nt4,_tmp79->f5=zt;_tmp79;});}}case 12U: _LL1F: _tmp6B=(void*)((struct Cyc_Port_Array_ct_Port_Ctype_struct*)_tmp68)->f1;_tmp6C=(void*)((struct Cyc_Port_Array_ct_Port_Ctype_struct*)_tmp68)->f2;_tmp6D=(void*)((struct Cyc_Port_Array_ct_Port_Ctype_struct*)_tmp68)->f3;_LL20: {void*t1=_tmp6B;void*t2=_tmp6C;void*zt=_tmp6D;
# 357
void*nt1=Cyc_Port_inst(instenv,t1);
if(nt1 == t1)return t;return(void*)({struct Cyc_Port_Array_ct_Port_Ctype_struct*_tmp7A=_cycalloc(sizeof(*_tmp7A));
_tmp7A->tag=12U,_tmp7A->f1=nt1,_tmp7A->f2=t2,_tmp7A->f3=zt;_tmp7A;});}default: _LL21: _tmp69=(void*)((struct Cyc_Port_Fn_ct_Port_Ctype_struct*)_tmp68)->f1;_tmp6A=((struct Cyc_Port_Fn_ct_Port_Ctype_struct*)_tmp68)->f2;_LL22: {void*t1=_tmp69;struct Cyc_List_List*ts=_tmp6A;
# 361
return(void*)({struct Cyc_Port_Fn_ct_Port_Ctype_struct*_tmp7C=_cycalloc(sizeof(*_tmp7C));_tmp7C->tag=15U,({void*_tmp34C=Cyc_Port_inst(instenv,t1);_tmp7C->f1=_tmp34C;}),({struct Cyc_List_List*_tmp34B=({struct Cyc_List_List*(*_tmp34A)(void*(*f)(struct Cyc_Dict_Dict*,void*),struct Cyc_Dict_Dict*env,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp7B)(void*(*f)(struct Cyc_Dict_Dict*,void*),struct Cyc_Dict_Dict*env,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(void*(*f)(struct Cyc_Dict_Dict*,void*),struct Cyc_Dict_Dict*env,struct Cyc_List_List*x))Cyc_List_map_c;_tmp7B;});struct Cyc_Dict_Dict*_tmp349=instenv;_tmp34A(Cyc_Port_inst,_tmp349,ts);});_tmp7C->f2=_tmp34B;});_tmp7C;});}}_LL0:;}
# 280
void*Cyc_Port_instantiate(void*t){
# 366
return({struct Cyc_Dict_Dict*_tmp34E=({struct Cyc_Dict_Dict*_tmp7E=_cycalloc(sizeof(*_tmp7E));({struct Cyc_Dict_Dict _tmp34D=({struct Cyc_Dict_Dict(*_tmp7D)(int(*cmp)(struct _fat_ptr*,struct _fat_ptr*))=(struct Cyc_Dict_Dict(*)(int(*cmp)(struct _fat_ptr*,struct _fat_ptr*)))Cyc_Dict_empty;_tmp7D;})(Cyc_strptrcmp);*_tmp7E=_tmp34D;});_tmp7E;});Cyc_Port_inst(_tmp34E,t);});}
# 280
struct Cyc_List_List*Cyc_Port_filter_self(void*t,struct Cyc_List_List*ts){
# 373
int found=0;
{struct Cyc_List_List*xs=ts;for(0;(unsigned)xs;xs=xs->tl){
void*t2=Cyc_Port_compress_ct((void*)xs->hd);
if(t == t2)found=1;}}
# 378
if(!found)return ts;{struct Cyc_List_List*res=0;
# 380
for(0;ts != 0;ts=ts->tl){
if(({void*_tmp34F=t;_tmp34F == Cyc_Port_compress_ct((void*)ts->hd);}))continue;res=({struct Cyc_List_List*_tmp7F=_cycalloc(sizeof(*_tmp7F));
_tmp7F->hd=(void*)ts->hd,_tmp7F->tl=res;_tmp7F;});}
# 384
return res;}}
# 280
void Cyc_Port_generalize(int is_rgn,void*t){
# 390
t=Cyc_Port_compress_ct(t);{
void*_tmp80=t;struct Cyc_List_List*_tmp82;void*_tmp81;void*_tmp85;void*_tmp84;void*_tmp83;void*_tmp8A;void*_tmp89;void*_tmp88;void*_tmp87;void*_tmp86;struct Cyc_Port_Cvar*_tmp8B;switch(*((int*)_tmp80)){case 16U: _LL1: _tmp8B=((struct Cyc_Port_Var_ct_Port_Ctype_struct*)_tmp80)->f1;_LL2: {struct Cyc_Port_Cvar*cv=_tmp8B;
# 394
({struct Cyc_List_List*_tmp350=Cyc_Port_filter_self(t,cv->uppers);cv->uppers=_tmp350;});
({struct Cyc_List_List*_tmp351=Cyc_Port_filter_self(t,cv->lowers);cv->lowers=_tmp351;});
if(is_rgn){
# 399
if(cv->uppers == 0 && cv->lowers == 0){
({void*_tmp352=t;Cyc_Port_unifies(_tmp352,Cyc_Port_rgnvar_ct(Cyc_Port_new_region_var()));});
return;}
# 399
if((unsigned)cv->uppers){
# 405
Cyc_Port_unifies(t,(void*)((struct Cyc_List_List*)_check_null(cv->uppers))->hd);
Cyc_Port_generalize(1,t);}else{
# 408
Cyc_Port_unifies(t,(void*)((struct Cyc_List_List*)_check_null(cv->lowers))->hd);
Cyc_Port_generalize(1,t);}}
# 396
return;}case 0U: _LL3: _LL4:
# 413
 goto _LL6;case 1U: _LL5: _LL6:
 goto _LL8;case 2U: _LL7: _LL8:
 goto _LLA;case 3U: _LL9: _LLA:
 goto _LLC;case 4U: _LLB: _LLC:
 goto _LLE;case 5U: _LLD: _LLE:
 goto _LL10;case 6U: _LLF: _LL10:
 goto _LL12;case 14U: _LL11: _LL12:
 goto _LL14;case 13U: _LL13: _LL14:
 goto _LL16;case 10U: _LL15: _LL16:
 goto _LL18;case 9U: _LL17: _LL18:
 goto _LL1A;case 8U: _LL19: _LL1A:
 goto _LL1C;case 7U: _LL1B: _LL1C:
 return;case 11U: _LL1D: _tmp86=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmp80)->f1;_tmp87=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmp80)->f2;_tmp88=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmp80)->f3;_tmp89=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmp80)->f4;_tmp8A=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmp80)->f5;_LL1E: {void*t1=_tmp86;void*t2=_tmp87;void*t3=_tmp88;void*t4=_tmp89;void*t5=_tmp8A;
# 429
Cyc_Port_generalize(0,t1);Cyc_Port_generalize(1,t4);goto _LL0;}case 12U: _LL1F: _tmp83=(void*)((struct Cyc_Port_Array_ct_Port_Ctype_struct*)_tmp80)->f1;_tmp84=(void*)((struct Cyc_Port_Array_ct_Port_Ctype_struct*)_tmp80)->f2;_tmp85=(void*)((struct Cyc_Port_Array_ct_Port_Ctype_struct*)_tmp80)->f3;_LL20: {void*t1=_tmp83;void*t2=_tmp84;void*t3=_tmp85;
# 431
Cyc_Port_generalize(0,t1);Cyc_Port_generalize(0,t2);goto _LL0;}default: _LL21: _tmp81=(void*)((struct Cyc_Port_Fn_ct_Port_Ctype_struct*)_tmp80)->f1;_tmp82=((struct Cyc_Port_Fn_ct_Port_Ctype_struct*)_tmp80)->f2;_LL22: {void*t1=_tmp81;struct Cyc_List_List*ts=_tmp82;
# 433
Cyc_Port_generalize(0,t1);({void(*_tmp353)(void(*f)(int,void*),int env,struct Cyc_List_List*x)=({void(*_tmp8C)(void(*f)(int,void*),int env,struct Cyc_List_List*x)=(void(*)(void(*f)(int,void*),int env,struct Cyc_List_List*x))Cyc_List_iter_c;_tmp8C;});_tmp353(Cyc_Port_generalize,0,ts);});goto _LL0;}}_LL0:;}}
# 280
static int Cyc_Port_occurs(void*v,void*t){
# 440
t=Cyc_Port_compress_ct(t);
if(t == v)return 1;{void*_tmp8D=t;struct Cyc_List_List*_tmp8E;struct Cyc_List_List*_tmp8F;struct Cyc_List_List*_tmp91;void*_tmp90;void*_tmp94;void*_tmp93;void*_tmp92;void*_tmp99;void*_tmp98;void*_tmp97;void*_tmp96;void*_tmp95;switch(*((int*)_tmp8D)){case 11U: _LL1: _tmp95=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmp8D)->f1;_tmp96=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmp8D)->f2;_tmp97=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmp8D)->f3;_tmp98=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmp8D)->f4;_tmp99=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmp8D)->f5;_LL2: {void*t1=_tmp95;void*t2=_tmp96;void*t3=_tmp97;void*t4=_tmp98;void*t5=_tmp99;
# 444
return(((Cyc_Port_occurs(v,t1)|| Cyc_Port_occurs(v,t2))|| Cyc_Port_occurs(v,t3))|| Cyc_Port_occurs(v,t4))||
 Cyc_Port_occurs(v,t5);}case 12U: _LL3: _tmp92=(void*)((struct Cyc_Port_Array_ct_Port_Ctype_struct*)_tmp8D)->f1;_tmp93=(void*)((struct Cyc_Port_Array_ct_Port_Ctype_struct*)_tmp8D)->f2;_tmp94=(void*)((struct Cyc_Port_Array_ct_Port_Ctype_struct*)_tmp8D)->f3;_LL4: {void*t1=_tmp92;void*t2=_tmp93;void*t3=_tmp94;
# 447
return(Cyc_Port_occurs(v,t1)|| Cyc_Port_occurs(v,t2))|| Cyc_Port_occurs(v,t3);}case 15U: _LL5: _tmp90=(void*)((struct Cyc_Port_Fn_ct_Port_Ctype_struct*)_tmp8D)->f1;_tmp91=((struct Cyc_Port_Fn_ct_Port_Ctype_struct*)_tmp8D)->f2;_LL6: {void*t=_tmp90;struct Cyc_List_List*ts=_tmp91;
# 449
if(Cyc_Port_occurs(v,t))return 1;for(0;(unsigned)ts;ts=ts->tl){
# 451
if(Cyc_Port_occurs(v,(void*)ts->hd))return 1;}
# 449
return 0;}case 13U: _LL7: _tmp8F=(((struct Cyc_Port_KnownAggr_ct_Port_Ctype_struct*)_tmp8D)->f1)->f2;_LL8: {struct Cyc_List_List*fs=_tmp8F;
# 453
return 0;}case 14U: _LL9: _tmp8E=((struct Cyc_Port_UnknownAggr_ct_Port_Ctype_struct*)_tmp8D)->f1;_LLA: {struct Cyc_List_List*fs=_tmp8E;
# 455
for(0;(unsigned)fs;fs=fs->tl){
if(Cyc_Port_occurs(v,((struct Cyc_Port_Cfield*)fs->hd)->qual)|| Cyc_Port_occurs(v,((struct Cyc_Port_Cfield*)fs->hd)->type))return 1;}
# 455
return 0;}default: _LLB: _LLC:
# 458
 return 0;}_LL0:;}}char Cyc_Port_Unify_ct[9U]="Unify_ct";struct Cyc_Port_Unify_ct_exn_struct{char*tag;};
# 467
struct Cyc_Port_Unify_ct_exn_struct Cyc_Port_Unify_ct_val={Cyc_Port_Unify_ct};
# 469
static int Cyc_Port_leq(void*t1,void*t2);
static void Cyc_Port_unify_cts(struct Cyc_List_List*t1,struct Cyc_List_List*t2);
static struct Cyc_List_List*Cyc_Port_merge_fields(struct Cyc_List_List*fs1,struct Cyc_List_List*fs2,int allow_subset);
# 473
static void Cyc_Port_unify_ct(void*t1,void*t2){
t1=Cyc_Port_compress_ct(t1);
t2=Cyc_Port_compress_ct(t2);
if(t1 == t2)return;{struct _tuple12 _stmttmp4=({struct _tuple12 _tmp300;_tmp300.f1=t1,_tmp300.f2=t2;_tmp300;});struct _tuple12 _tmp9A=_stmttmp4;struct Cyc_List_List*_tmp9E;struct Cyc_Absyn_Aggrdecl*_tmp9D;void***_tmp9C;struct Cyc_List_List*_tmp9B;void***_tmpA2;struct Cyc_List_List*_tmpA1;void***_tmpA0;struct Cyc_List_List*_tmp9F;void***_tmpA6;struct Cyc_List_List*_tmpA5;struct Cyc_List_List*_tmpA4;struct Cyc_Absyn_Aggrdecl*_tmpA3;struct _tuple11*_tmpA8;struct _tuple11*_tmpA7;struct Cyc_List_List*_tmpAC;void*_tmpAB;struct Cyc_List_List*_tmpAA;void*_tmpA9;void*_tmpB2;void*_tmpB1;void*_tmpB0;void*_tmpAF;void*_tmpAE;void*_tmpAD;struct _fat_ptr _tmpB4;struct _fat_ptr _tmpB3;void*_tmpBE;void*_tmpBD;void*_tmpBC;void*_tmpBB;void*_tmpBA;void*_tmpB9;void*_tmpB8;void*_tmpB7;void*_tmpB6;void*_tmpB5;struct Cyc_Port_Cvar*_tmpBF;struct Cyc_Port_Cvar*_tmpC0;if(((struct Cyc_Port_Var_ct_Port_Ctype_struct*)_tmp9A.f1)->tag == 16U){_LL1: _tmpC0=((struct Cyc_Port_Var_ct_Port_Ctype_struct*)_tmp9A.f1)->f1;_LL2: {struct Cyc_Port_Cvar*v1=_tmpC0;
# 479
if(!Cyc_Port_occurs(t1,t2)){
# 482
{struct Cyc_List_List*us=Cyc_Port_filter_self(t1,v1->uppers);for(0;(unsigned)us;us=us->tl){
if(!Cyc_Port_leq(t2,(void*)us->hd))(int)_throw((void*)& Cyc_Port_Unify_ct_val);}}
# 482
{
# 484
struct Cyc_List_List*ls=Cyc_Port_filter_self(t1,v1->lowers);
# 482
for(0;(unsigned)ls;ls=ls->tl){
# 485
if(!Cyc_Port_leq((void*)ls->hd,t2))(int)_throw((void*)& Cyc_Port_Unify_ct_val);}}
# 482
({void**_tmp354=({void**_tmpC1=_cycalloc(sizeof(*_tmpC1));*_tmpC1=t2;_tmpC1;});v1->eq=_tmp354;});
# 487
return;}else{
(int)_throw((void*)& Cyc_Port_Unify_ct_val);}}}else{if(((struct Cyc_Port_Var_ct_Port_Ctype_struct*)_tmp9A.f2)->tag == 16U){_LL3: _tmpBF=((struct Cyc_Port_Var_ct_Port_Ctype_struct*)_tmp9A.f2)->f1;_LL4: {struct Cyc_Port_Cvar*v1=_tmpBF;
Cyc_Port_unify_ct(t2,t1);return;}}else{switch(*((int*)_tmp9A.f1)){case 11U: if(((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmp9A.f2)->tag == 11U){_LL5: _tmpB5=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmp9A.f1)->f1;_tmpB6=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmp9A.f1)->f2;_tmpB7=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmp9A.f1)->f3;_tmpB8=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmp9A.f1)->f4;_tmpB9=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmp9A.f1)->f5;_tmpBA=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmp9A.f2)->f1;_tmpBB=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmp9A.f2)->f2;_tmpBC=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmp9A.f2)->f3;_tmpBD=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmp9A.f2)->f4;_tmpBE=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmp9A.f2)->f5;_LL6: {void*e1=_tmpB5;void*q1=_tmpB6;void*k1=_tmpB7;void*r1=_tmpB8;void*zt1=_tmpB9;void*e2=_tmpBA;void*q2=_tmpBB;void*k2=_tmpBC;void*r2=_tmpBD;void*zt2=_tmpBE;
# 491
Cyc_Port_unify_ct(e1,e2);Cyc_Port_unify_ct(q1,q2);Cyc_Port_unify_ct(k1,k2);Cyc_Port_unify_ct(r1,r2);
Cyc_Port_unify_ct(zt1,zt2);
return;}}else{goto _LL15;}case 10U: if(((struct Cyc_Port_RgnVar_ct_Port_Ctype_struct*)_tmp9A.f2)->tag == 10U){_LL7: _tmpB3=*((struct Cyc_Port_RgnVar_ct_Port_Ctype_struct*)_tmp9A.f1)->f1;_tmpB4=*((struct Cyc_Port_RgnVar_ct_Port_Ctype_struct*)_tmp9A.f2)->f1;_LL8: {struct _fat_ptr n1=_tmpB3;struct _fat_ptr n2=_tmpB4;
# 495
if(Cyc_strcmp((struct _fat_ptr)n1,(struct _fat_ptr)n2)!= 0)(int)_throw((void*)& Cyc_Port_Unify_ct_val);return;}}else{goto _LL15;}case 12U: if(((struct Cyc_Port_Array_ct_Port_Ctype_struct*)_tmp9A.f2)->tag == 12U){_LL9: _tmpAD=(void*)((struct Cyc_Port_Array_ct_Port_Ctype_struct*)_tmp9A.f1)->f1;_tmpAE=(void*)((struct Cyc_Port_Array_ct_Port_Ctype_struct*)_tmp9A.f1)->f2;_tmpAF=(void*)((struct Cyc_Port_Array_ct_Port_Ctype_struct*)_tmp9A.f1)->f3;_tmpB0=(void*)((struct Cyc_Port_Array_ct_Port_Ctype_struct*)_tmp9A.f2)->f1;_tmpB1=(void*)((struct Cyc_Port_Array_ct_Port_Ctype_struct*)_tmp9A.f2)->f2;_tmpB2=(void*)((struct Cyc_Port_Array_ct_Port_Ctype_struct*)_tmp9A.f2)->f3;_LLA: {void*e1=_tmpAD;void*q1=_tmpAE;void*zt1=_tmpAF;void*e2=_tmpB0;void*q2=_tmpB1;void*zt2=_tmpB2;
# 498
Cyc_Port_unify_ct(e1,e2);Cyc_Port_unify_ct(q1,q2);Cyc_Port_unify_ct(zt1,zt2);return;}}else{goto _LL15;}case 15U: if(((struct Cyc_Port_Fn_ct_Port_Ctype_struct*)_tmp9A.f2)->tag == 15U){_LLB: _tmpA9=(void*)((struct Cyc_Port_Fn_ct_Port_Ctype_struct*)_tmp9A.f1)->f1;_tmpAA=((struct Cyc_Port_Fn_ct_Port_Ctype_struct*)_tmp9A.f1)->f2;_tmpAB=(void*)((struct Cyc_Port_Fn_ct_Port_Ctype_struct*)_tmp9A.f2)->f1;_tmpAC=((struct Cyc_Port_Fn_ct_Port_Ctype_struct*)_tmp9A.f2)->f2;_LLC: {void*t1=_tmpA9;struct Cyc_List_List*ts1=_tmpAA;void*t2=_tmpAB;struct Cyc_List_List*ts2=_tmpAC;
# 500
Cyc_Port_unify_ct(t1,t2);Cyc_Port_unify_cts(ts1,ts2);return;}}else{goto _LL15;}case 13U: switch(*((int*)_tmp9A.f2)){case 13U: _LLD: _tmpA7=((struct Cyc_Port_KnownAggr_ct_Port_Ctype_struct*)_tmp9A.f1)->f1;_tmpA8=((struct Cyc_Port_KnownAggr_ct_Port_Ctype_struct*)_tmp9A.f2)->f1;_LLE: {struct _tuple11*p1=_tmpA7;struct _tuple11*p2=_tmpA8;
# 502
if(p1 == p2)return;else{(int)_throw((void*)& Cyc_Port_Unify_ct_val);}}case 14U: _LL13: _tmpA3=(((struct Cyc_Port_KnownAggr_ct_Port_Ctype_struct*)_tmp9A.f1)->f1)->f1;_tmpA4=(((struct Cyc_Port_KnownAggr_ct_Port_Ctype_struct*)_tmp9A.f1)->f1)->f2;_tmpA5=((struct Cyc_Port_UnknownAggr_ct_Port_Ctype_struct*)_tmp9A.f2)->f1;_tmpA6=(void***)&((struct Cyc_Port_UnknownAggr_ct_Port_Ctype_struct*)_tmp9A.f2)->f2;_LL14: {struct Cyc_Absyn_Aggrdecl*ad=_tmpA3;struct Cyc_List_List*fs2=_tmpA4;struct Cyc_List_List*fs1=_tmpA5;void***eq=_tmpA6;
# 512
Cyc_Port_merge_fields(fs2,fs1,0);
({void**_tmp355=({void**_tmpC4=_cycalloc(sizeof(*_tmpC4));*_tmpC4=t1;_tmpC4;});*eq=_tmp355;});
return;}default: goto _LL15;}case 14U: switch(*((int*)_tmp9A.f2)){case 14U: _LLF: _tmp9F=((struct Cyc_Port_UnknownAggr_ct_Port_Ctype_struct*)_tmp9A.f1)->f1;_tmpA0=(void***)&((struct Cyc_Port_UnknownAggr_ct_Port_Ctype_struct*)_tmp9A.f1)->f2;_tmpA1=((struct Cyc_Port_UnknownAggr_ct_Port_Ctype_struct*)_tmp9A.f2)->f1;_tmpA2=(void***)&((struct Cyc_Port_UnknownAggr_ct_Port_Ctype_struct*)_tmp9A.f2)->f2;_LL10: {struct Cyc_List_List*fs1=_tmp9F;void***eq1=_tmpA0;struct Cyc_List_List*fs2=_tmpA1;void***eq2=_tmpA2;
# 504
void*t=Cyc_Port_unknown_aggr_ct(Cyc_Port_merge_fields(fs1,fs2,1));
({void**_tmp357=({void**_tmp356=({void**_tmpC2=_cycalloc(sizeof(*_tmpC2));*_tmpC2=t;_tmpC2;});*eq2=_tmp356;});*eq1=_tmp357;});
return;}case 13U: _LL11: _tmp9B=((struct Cyc_Port_UnknownAggr_ct_Port_Ctype_struct*)_tmp9A.f1)->f1;_tmp9C=(void***)&((struct Cyc_Port_UnknownAggr_ct_Port_Ctype_struct*)_tmp9A.f1)->f2;_tmp9D=(((struct Cyc_Port_KnownAggr_ct_Port_Ctype_struct*)_tmp9A.f2)->f1)->f1;_tmp9E=(((struct Cyc_Port_KnownAggr_ct_Port_Ctype_struct*)_tmp9A.f2)->f1)->f2;_LL12: {struct Cyc_List_List*fs1=_tmp9B;void***eq=_tmp9C;struct Cyc_Absyn_Aggrdecl*ad=_tmp9D;struct Cyc_List_List*fs2=_tmp9E;
# 508
Cyc_Port_merge_fields(fs2,fs1,0);
({void**_tmp358=({void**_tmpC3=_cycalloc(sizeof(*_tmpC3));*_tmpC3=t2;_tmpC3;});*eq=_tmp358;});
return;}default: goto _LL15;}default: _LL15: _LL16:
# 515
(int)_throw((void*)& Cyc_Port_Unify_ct_val);}}}_LL0:;}}
# 473
static void Cyc_Port_unify_cts(struct Cyc_List_List*t1,struct Cyc_List_List*t2){
# 520
for(0;t1 != 0 && t2 != 0;(t1=t1->tl,t2=t2->tl)){
Cyc_Port_unify_ct((void*)t1->hd,(void*)t2->hd);}
# 523
if(t1 != 0 || t2 != 0)
(int)_throw((void*)& Cyc_Port_Unify_ct_val);}
# 473
static struct Cyc_List_List*Cyc_Port_merge_fields(struct Cyc_List_List*fs1,struct Cyc_List_List*fs2,int allow_f1_subset_f2){
# 531
struct Cyc_List_List*common=0;
{struct Cyc_List_List*xs2=fs2;for(0;(unsigned)xs2;xs2=xs2->tl){
struct Cyc_Port_Cfield*f2=(struct Cyc_Port_Cfield*)xs2->hd;
int found=0;
{struct Cyc_List_List*xs1=fs1;for(0;(unsigned)xs1;xs1=xs1->tl){
struct Cyc_Port_Cfield*f1=(struct Cyc_Port_Cfield*)xs1->hd;
if(Cyc_strptrcmp(f1->f,f2->f)== 0){
common=({struct Cyc_List_List*_tmpC5=_cycalloc(sizeof(*_tmpC5));_tmpC5->hd=f1,_tmpC5->tl=common;_tmpC5;});
Cyc_Port_unify_ct(f1->qual,f2->qual);
Cyc_Port_unify_ct(f1->type,f2->type);
found=1;
break;}}}
# 545
if(!found){
if(allow_f1_subset_f2)
common=({struct Cyc_List_List*_tmpC6=_cycalloc(sizeof(*_tmpC6));_tmpC6->hd=f2,_tmpC6->tl=common;_tmpC6;});else{
(int)_throw((void*)& Cyc_Port_Unify_ct_val);}}}}
# 551
{struct Cyc_List_List*xs1=fs1;for(0;(unsigned)xs1;xs1=xs1->tl){
struct Cyc_Port_Cfield*f1=(struct Cyc_Port_Cfield*)xs1->hd;
int found=0;
{struct Cyc_List_List*xs2=fs2;for(0;(unsigned)xs2;xs2=xs2->tl){
struct Cyc_Port_Cfield*f2=(struct Cyc_Port_Cfield*)xs2->hd;
if(Cyc_strptrcmp(f1->f,f2->f))found=1;}}
# 558
if(!found)
common=({struct Cyc_List_List*_tmpC7=_cycalloc(sizeof(*_tmpC7));_tmpC7->hd=f1,_tmpC7->tl=common;_tmpC7;});}}
# 561
return common;}
# 473
static int Cyc_Port_unifies(void*t1,void*t2){
# 565
{struct _handler_cons _tmpC8;_push_handler(& _tmpC8);{int _tmpCA=0;if(setjmp(_tmpC8.handler))_tmpCA=1;if(!_tmpCA){
# 571
Cyc_Port_unify_ct(t1,t2);;_pop_handler();}else{void*_tmpC9=(void*)Cyc_Core_get_exn_thrown();void*_tmpCB=_tmpC9;void*_tmpCC;if(((struct Cyc_Port_Unify_ct_exn_struct*)_tmpCB)->tag == Cyc_Port_Unify_ct){_LL1: _LL2:
# 578
 return 0;}else{_LL3: _tmpCC=_tmpCB;_LL4: {void*exn=_tmpCC;(int)_rethrow(exn);}}_LL0:;}}}
# 580
return 1;}struct _tuple13{void*f1;void*f2;void*f3;void*f4;void*f5;};
# 473
static struct Cyc_List_List*Cyc_Port_insert_upper(void*v,void*t,struct Cyc_List_List**uppers){
# 588
t=Cyc_Port_compress_ct(t);
{void*_tmpCD=t;switch(*((int*)_tmpCD)){case 1U: _LL1: _LL2:
# 592
 goto _LL4;case 8U: _LL3: _LL4:
 goto _LL6;case 5U: _LL5: _LL6:
 goto _LL8;case 2U: _LL7: _LL8:
 goto _LLA;case 3U: _LL9: _LLA:
 goto _LLC;case 12U: _LLB: _LLC:
 goto _LLE;case 13U: _LLD: _LLE:
 goto _LL10;case 15U: _LLF: _LL10:
 goto _LL12;case 7U: _LL11: _LL12:
# 603
*uppers=0;
Cyc_Port_unifies(v,t);
return*uppers;case 4U: _LL13: _LL14:
# 608
 goto _LL16;case 0U: _LL15: _LL16:
 goto _LL18;case 9U: _LL17: _LL18:
# 611
 return*uppers;default: _LL19: _LL1A:
 goto _LL0;}_LL0:;}
# 615
{struct Cyc_List_List*us=*uppers;for(0;(unsigned)us;us=us->tl){
void*t2=Cyc_Port_compress_ct((void*)us->hd);
# 618
if(t == t2)return*uppers;{struct _tuple12 _stmttmp5=({struct _tuple12 _tmp302;_tmp302.f1=t,_tmp302.f2=t2;_tmp302;});struct _tuple12 _tmpCE=_stmttmp5;void*_tmpD8;void*_tmpD7;void*_tmpD6;void*_tmpD5;void*_tmpD4;void*_tmpD3;void*_tmpD2;void*_tmpD1;void*_tmpD0;void*_tmpCF;switch(*((int*)_tmpCE.f1)){case 6U: switch(*((int*)_tmpCE.f2)){case 11U: _LL1C: _LL1D:
# 623
 goto _LL1F;case 5U: _LL1E: _LL1F:
 goto _LL21;case 12U: _LL20: _LL21:
# 626
 return*uppers;default: goto _LL24;}case 11U: if(((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmpCE.f2)->tag == 11U){_LL22: _tmpCF=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmpCE.f1)->f1;_tmpD0=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmpCE.f1)->f2;_tmpD1=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmpCE.f1)->f3;_tmpD2=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmpCE.f1)->f4;_tmpD3=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmpCE.f1)->f5;_tmpD4=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmpCE.f2)->f1;_tmpD5=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmpCE.f2)->f2;_tmpD6=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmpCE.f2)->f3;_tmpD7=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmpCE.f2)->f4;_tmpD8=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmpCE.f2)->f5;_LL23: {void*ta=_tmpCF;void*qa=_tmpD0;void*ka=_tmpD1;void*ra=_tmpD2;void*za=_tmpD3;void*tb=_tmpD4;void*qb=_tmpD5;void*kb=_tmpD6;void*rb=_tmpD7;void*zb=_tmpD8;
# 631
struct _tuple13 _stmttmp6=({struct _tuple13 _tmp301;({void*_tmp35D=Cyc_Port_var();_tmp301.f1=_tmp35D;}),({void*_tmp35C=Cyc_Port_var();_tmp301.f2=_tmp35C;}),({void*_tmp35B=Cyc_Port_var();_tmp301.f3=_tmp35B;}),({void*_tmp35A=Cyc_Port_var();_tmp301.f4=_tmp35A;}),({void*_tmp359=Cyc_Port_var();_tmp301.f5=_tmp359;});_tmp301;});void*_tmpDD;void*_tmpDC;void*_tmpDB;void*_tmpDA;void*_tmpD9;_LL27: _tmpD9=_stmttmp6.f1;_tmpDA=_stmttmp6.f2;_tmpDB=_stmttmp6.f3;_tmpDC=_stmttmp6.f4;_tmpDD=_stmttmp6.f5;_LL28: {void*tc=_tmpD9;void*qc=_tmpDA;void*kc=_tmpDB;void*rc=_tmpDC;void*zc=_tmpDD;
struct Cyc_Port_Ptr_ct_Port_Ctype_struct*p=({struct Cyc_Port_Ptr_ct_Port_Ctype_struct*_tmpDE=_cycalloc(sizeof(*_tmpDE));_tmpDE->tag=11U,_tmpDE->f1=tc,_tmpDE->f2=qc,_tmpDE->f3=kc,_tmpDE->f4=rc,_tmpDE->f5=zc;_tmpDE;});
Cyc_Port_leq(tc,ta);Cyc_Port_leq(tc,tb);
Cyc_Port_leq(qc,qa);Cyc_Port_leq(qc,qb);
Cyc_Port_leq(kc,ka);Cyc_Port_leq(kc,qb);
Cyc_Port_leq(rc,ra);Cyc_Port_leq(rc,rb);
Cyc_Port_leq(zc,za);Cyc_Port_leq(zc,zb);
us->hd=(void*)((void*)p);
return*uppers;}}}else{goto _LL24;}default: _LL24: _LL25:
 goto _LL1B;}_LL1B:;}}}
# 643
return({struct Cyc_List_List*_tmpDF=_cycalloc(sizeof(*_tmpDF));_tmpDF->hd=t,_tmpDF->tl=*uppers;_tmpDF;});}
# 473
static struct Cyc_List_List*Cyc_Port_insert_lower(void*v,void*t,struct Cyc_List_List**lowers){
# 650
t=Cyc_Port_compress_ct(t);
{void*_tmpE0=t;switch(*((int*)_tmpE0)){case 0U: _LL1: _LL2:
 goto _LL4;case 8U: _LL3: _LL4:
 goto _LL6;case 2U: _LL5: _LL6:
 goto _LL8;case 3U: _LL7: _LL8:
 goto _LLA;case 4U: _LL9: _LLA:
 goto _LLC;case 13U: _LLB: _LLC:
 goto _LLE;case 15U: _LLD: _LLE:
 goto _LL10;case 10U: _LLF: _LL10:
# 660
*lowers=0;
Cyc_Port_unifies(v,t);
return*lowers;case 7U: _LL11: _LL12:
 goto _LL14;case 1U: _LL13: _LL14:
 goto _LL16;case 9U: _LL15: _LL16:
# 666
 return*lowers;default: _LL17: _LL18:
# 668
 goto _LL0;}_LL0:;}
# 670
{struct Cyc_List_List*ls=*lowers;for(0;(unsigned)ls;ls=ls->tl){
void*t2=Cyc_Port_compress_ct((void*)ls->hd);
if(t == t2)return*lowers;{struct _tuple12 _stmttmp7=({struct _tuple12 _tmp304;_tmp304.f1=t,_tmp304.f2=t2;_tmp304;});struct _tuple12 _tmpE1=_stmttmp7;void*_tmpEB;void*_tmpEA;void*_tmpE9;void*_tmpE8;void*_tmpE7;void*_tmpE6;void*_tmpE5;void*_tmpE4;void*_tmpE3;void*_tmpE2;if(((struct Cyc_Port_Void_ct_Port_Ctype_struct*)_tmpE1.f2)->tag == 4U){_LL1A: _LL1B:
# 674
 goto _LL1D;}else{switch(*((int*)_tmpE1.f1)){case 5U: switch(*((int*)_tmpE1.f2)){case 6U: _LL1C: _LL1D:
 goto _LL1F;case 11U: _LL1E: _LL1F:
 goto _LL21;default: goto _LL26;}case 11U: switch(*((int*)_tmpE1.f2)){case 6U: _LL20: _LL21:
 goto _LL23;case 11U: _LL24: _tmpE2=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmpE1.f1)->f1;_tmpE3=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmpE1.f1)->f2;_tmpE4=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmpE1.f1)->f3;_tmpE5=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmpE1.f1)->f4;_tmpE6=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmpE1.f1)->f5;_tmpE7=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmpE1.f2)->f1;_tmpE8=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmpE1.f2)->f2;_tmpE9=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmpE1.f2)->f3;_tmpEA=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmpE1.f2)->f4;_tmpEB=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmpE1.f2)->f5;_LL25: {void*ta=_tmpE2;void*qa=_tmpE3;void*ka=_tmpE4;void*ra=_tmpE5;void*za=_tmpE6;void*tb=_tmpE7;void*qb=_tmpE8;void*kb=_tmpE9;void*rb=_tmpEA;void*zb=_tmpEB;
# 684
struct _tuple13 _stmttmp8=({struct _tuple13 _tmp303;({void*_tmp362=Cyc_Port_var();_tmp303.f1=_tmp362;}),({void*_tmp361=Cyc_Port_var();_tmp303.f2=_tmp361;}),({void*_tmp360=Cyc_Port_var();_tmp303.f3=_tmp360;}),({void*_tmp35F=Cyc_Port_var();_tmp303.f4=_tmp35F;}),({void*_tmp35E=Cyc_Port_var();_tmp303.f5=_tmp35E;});_tmp303;});void*_tmpF0;void*_tmpEF;void*_tmpEE;void*_tmpED;void*_tmpEC;_LL29: _tmpEC=_stmttmp8.f1;_tmpED=_stmttmp8.f2;_tmpEE=_stmttmp8.f3;_tmpEF=_stmttmp8.f4;_tmpF0=_stmttmp8.f5;_LL2A: {void*tc=_tmpEC;void*qc=_tmpED;void*kc=_tmpEE;void*rc=_tmpEF;void*zc=_tmpF0;
struct Cyc_Port_Ptr_ct_Port_Ctype_struct*p=({struct Cyc_Port_Ptr_ct_Port_Ctype_struct*_tmpF1=_cycalloc(sizeof(*_tmpF1));_tmpF1->tag=11U,_tmpF1->f1=tc,_tmpF1->f2=qc,_tmpF1->f3=kc,_tmpF1->f4=rc,_tmpF1->f5=zc;_tmpF1;});
Cyc_Port_leq(ta,tc);Cyc_Port_leq(tb,tc);
Cyc_Port_leq(qa,qc);Cyc_Port_leq(qb,qc);
Cyc_Port_leq(ka,kc);Cyc_Port_leq(qb,kc);
Cyc_Port_leq(ra,rc);Cyc_Port_leq(rb,rc);
Cyc_Port_leq(za,zc);Cyc_Port_leq(zb,zc);
ls->hd=(void*)((void*)p);
return*lowers;}}default: goto _LL26;}case 12U: if(((struct Cyc_Port_Arith_ct_Port_Ctype_struct*)_tmpE1.f2)->tag == 6U){_LL22: _LL23:
# 679
 return*lowers;}else{goto _LL26;}default: _LL26: _LL27:
# 693
 goto _LL19;}}_LL19:;}}}
# 696
return({struct Cyc_List_List*_tmpF2=_cycalloc(sizeof(*_tmpF2));_tmpF2->hd=t,_tmpF2->tl=*lowers;_tmpF2;});}
# 473
static int Cyc_Port_leq(void*t1,void*t2){
# 709
if(t1 == t2)return 1;t1=
Cyc_Port_compress_ct(t1);
t2=Cyc_Port_compress_ct(t2);{
struct _tuple12 _stmttmp9=({struct _tuple12 _tmp305;_tmp305.f1=t1,_tmp305.f2=t2;_tmp305;});struct _tuple12 _tmpF3=_stmttmp9;struct Cyc_Port_Cvar*_tmpF4;void*_tmpFB;void*_tmpFA;void*_tmpF9;void*_tmpF8;void*_tmpF7;void*_tmpF6;void*_tmpF5;void*_tmp101;void*_tmp100;void*_tmpFF;void*_tmpFE;void*_tmpFD;void*_tmpFC;void*_tmp10B;void*_tmp10A;void*_tmp109;void*_tmp108;void*_tmp107;void*_tmp106;void*_tmp105;void*_tmp104;void*_tmp103;void*_tmp102;struct Cyc_Port_Cvar*_tmp10C;struct Cyc_Port_Cvar*_tmp10E;struct Cyc_Port_Cvar*_tmp10D;struct _fat_ptr _tmp10F;struct _fat_ptr _tmp111;struct _fat_ptr _tmp110;switch(*((int*)_tmpF3.f1)){case 7U: _LL1: _LL2:
 return 1;case 10U: switch(*((int*)_tmpF3.f2)){case 10U: _LL3: _tmp110=*((struct Cyc_Port_RgnVar_ct_Port_Ctype_struct*)_tmpF3.f1)->f1;_tmp111=*((struct Cyc_Port_RgnVar_ct_Port_Ctype_struct*)_tmpF3.f2)->f1;_LL4: {struct _fat_ptr n1=_tmp110;struct _fat_ptr n2=_tmp111;
return Cyc_strcmp((struct _fat_ptr)n1,(struct _fat_ptr)n2)== 0;}case 7U: _LL5: _tmp10F=*((struct Cyc_Port_RgnVar_ct_Port_Ctype_struct*)_tmpF3.f1)->f1;_LL6: {struct _fat_ptr n1=_tmp10F;
return 0;}case 16U: goto _LL2D;default: goto _LL2F;}case 1U: switch(*((int*)_tmpF3.f2)){case 0U: _LL7: _LL8:
 return 1;case 16U: goto _LL2D;default: goto _LL2F;}case 0U: switch(*((int*)_tmpF3.f2)){case 1U: _LL9: _LLA:
 return 0;case 16U: goto _LL2D;default: goto _LL2F;}case 9U: switch(*((int*)_tmpF3.f2)){case 8U: _LLB: _LLC:
 return 0;case 16U: goto _LL2D;default: goto _LL2F;}case 8U: switch(*((int*)_tmpF3.f2)){case 9U: _LLD: _LLE:
 return 1;case 16U: goto _LL2D;default: goto _LL2F;}case 16U: switch(*((int*)_tmpF3.f2)){case 0U: _LLF: _LL10:
 return 1;case 4U: _LL11: _LL12:
 return 1;case 16U: _LL29: _tmp10D=((struct Cyc_Port_Var_ct_Port_Ctype_struct*)_tmpF3.f1)->f1;_tmp10E=((struct Cyc_Port_Var_ct_Port_Ctype_struct*)_tmpF3.f2)->f1;_LL2A: {struct Cyc_Port_Cvar*v1=_tmp10D;struct Cyc_Port_Cvar*v2=_tmp10E;
# 739
({struct Cyc_List_List*_tmp363=Cyc_Port_filter_self(t1,v1->uppers);v1->uppers=_tmp363;});
({struct Cyc_List_List*_tmp364=Cyc_Port_filter_self(t2,v2->lowers);v2->lowers=_tmp364;});
({struct Cyc_List_List*_tmp365=Cyc_Port_insert_upper(t1,t2,& v1->uppers);v1->uppers=_tmp365;});
({struct Cyc_List_List*_tmp366=Cyc_Port_insert_lower(t2,t1,& v2->lowers);v2->lowers=_tmp366;});
return 1;}default: _LL2B: _tmp10C=((struct Cyc_Port_Var_ct_Port_Ctype_struct*)_tmpF3.f1)->f1;_LL2C: {struct Cyc_Port_Cvar*v1=_tmp10C;
# 745
({struct Cyc_List_List*_tmp367=Cyc_Port_filter_self(t1,v1->uppers);v1->uppers=_tmp367;});
({struct Cyc_List_List*_tmp368=Cyc_Port_insert_upper(t1,t2,& v1->uppers);v1->uppers=_tmp368;});
return 1;}}case 4U: _LL13: _LL14:
# 722
 return 0;case 5U: switch(*((int*)_tmpF3.f2)){case 6U: _LL15: _LL16:
 return 1;case 11U: _LL17: _LL18:
 return 1;case 4U: _LL19: _LL1A:
 return 1;case 16U: goto _LL2D;default: goto _LL2F;}case 11U: switch(*((int*)_tmpF3.f2)){case 6U: _LL1B: _LL1C:
 return 1;case 4U: _LL1D: _LL1E:
 return 1;case 11U: _LL23: _tmp102=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmpF3.f1)->f1;_tmp103=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmpF3.f1)->f2;_tmp104=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmpF3.f1)->f3;_tmp105=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmpF3.f1)->f4;_tmp106=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmpF3.f1)->f5;_tmp107=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmpF3.f2)->f1;_tmp108=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmpF3.f2)->f2;_tmp109=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmpF3.f2)->f3;_tmp10A=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmpF3.f2)->f4;_tmp10B=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmpF3.f2)->f5;_LL24: {void*t1=_tmp102;void*q1=_tmp103;void*k1=_tmp104;void*r1=_tmp105;void*z1=_tmp106;void*t2=_tmp107;void*q2=_tmp108;void*k2=_tmp109;void*r2=_tmp10A;void*z2=_tmp10B;
# 731
return(((Cyc_Port_leq(t1,t2)&& Cyc_Port_leq(q1,q2))&& Cyc_Port_unifies(k1,k2))&& Cyc_Port_leq(r1,r2))&&
 Cyc_Port_leq(z1,z2);}case 16U: goto _LL2D;default: goto _LL2F;}case 12U: switch(*((int*)_tmpF3.f2)){case 6U: _LL1F: _LL20:
# 728
 return 1;case 4U: _LL21: _LL22:
 return 1;case 12U: _LL25: _tmpFC=(void*)((struct Cyc_Port_Array_ct_Port_Ctype_struct*)_tmpF3.f1)->f1;_tmpFD=(void*)((struct Cyc_Port_Array_ct_Port_Ctype_struct*)_tmpF3.f1)->f2;_tmpFE=(void*)((struct Cyc_Port_Array_ct_Port_Ctype_struct*)_tmpF3.f1)->f3;_tmpFF=(void*)((struct Cyc_Port_Array_ct_Port_Ctype_struct*)_tmpF3.f2)->f1;_tmp100=(void*)((struct Cyc_Port_Array_ct_Port_Ctype_struct*)_tmpF3.f2)->f2;_tmp101=(void*)((struct Cyc_Port_Array_ct_Port_Ctype_struct*)_tmpF3.f2)->f3;_LL26: {void*t1=_tmpFC;void*q1=_tmpFD;void*z1=_tmpFE;void*t2=_tmpFF;void*q2=_tmp100;void*z2=_tmp101;
# 734
return(Cyc_Port_leq(t1,t2)&& Cyc_Port_leq(q1,q2))&& Cyc_Port_leq(z1,z2);}case 11U: _LL27: _tmpF5=(void*)((struct Cyc_Port_Array_ct_Port_Ctype_struct*)_tmpF3.f1)->f1;_tmpF6=(void*)((struct Cyc_Port_Array_ct_Port_Ctype_struct*)_tmpF3.f1)->f2;_tmpF7=(void*)((struct Cyc_Port_Array_ct_Port_Ctype_struct*)_tmpF3.f1)->f3;_tmpF8=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmpF3.f2)->f1;_tmpF9=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmpF3.f2)->f2;_tmpFA=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmpF3.f2)->f3;_tmpFB=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmpF3.f2)->f5;_LL28: {void*t1=_tmpF5;void*q1=_tmpF6;void*z1=_tmpF7;void*t2=_tmpF8;void*q2=_tmpF9;void*k2=_tmpFA;void*z2=_tmpFB;
# 736
return((Cyc_Port_leq(t1,t2)&& Cyc_Port_leq(q1,q2))&& Cyc_Port_unifies((void*)& Cyc_Port_Fat_ct_val,k2))&&
 Cyc_Port_leq(z1,z2);}case 16U: goto _LL2D;default: goto _LL2F;}default: if(((struct Cyc_Port_Var_ct_Port_Ctype_struct*)_tmpF3.f2)->tag == 16U){_LL2D: _tmpF4=((struct Cyc_Port_Var_ct_Port_Ctype_struct*)_tmpF3.f2)->f1;_LL2E: {struct Cyc_Port_Cvar*v2=_tmpF4;
# 749
({struct Cyc_List_List*_tmp369=Cyc_Port_filter_self(t2,v2->lowers);v2->lowers=_tmp369;});
({struct Cyc_List_List*_tmp36A=Cyc_Port_insert_lower(t2,t1,& v2->lowers);v2->lowers=_tmp36A;});
return 1;}}else{_LL2F: _LL30:
 return Cyc_Port_unifies(t1,t2);}}_LL0:;}}struct Cyc_Port_GlobalCenv{struct Cyc_Dict_Dict typedef_dict;struct Cyc_Dict_Dict struct_dict;struct Cyc_Dict_Dict union_dict;void*return_type;struct Cyc_List_List*qualifier_edits;struct Cyc_List_List*pointer_edits;struct Cyc_List_List*zeroterm_edits;struct Cyc_List_List*vardecl_locs;struct Cyc_Hashtable_Table*varusage_tab;struct Cyc_List_List*edits;int porting;};
# 783
enum Cyc_Port_CPos{Cyc_Port_FnRes_pos =0U,Cyc_Port_FnArg_pos =1U,Cyc_Port_FnBody_pos =2U,Cyc_Port_Toplevel_pos =3U};struct Cyc_Port_Cenv{struct Cyc_Port_GlobalCenv*gcenv;struct Cyc_Dict_Dict var_dict;enum Cyc_Port_CPos cpos;};
# 796
static struct Cyc_Port_Cenv*Cyc_Port_empty_cenv(){
struct Cyc_Port_GlobalCenv*g=({struct Cyc_Port_GlobalCenv*_tmp118=_cycalloc(sizeof(*_tmp118));({struct Cyc_Dict_Dict _tmp36F=({struct Cyc_Dict_Dict(*_tmp114)(int(*cmp)(struct _tuple0*,struct _tuple0*))=(struct Cyc_Dict_Dict(*)(int(*cmp)(struct _tuple0*,struct _tuple0*)))Cyc_Dict_empty;_tmp114;})(Cyc_Absyn_qvar_cmp);_tmp118->typedef_dict=_tmp36F;}),({
struct Cyc_Dict_Dict _tmp36E=({struct Cyc_Dict_Dict(*_tmp115)(int(*cmp)(struct _tuple0*,struct _tuple0*))=(struct Cyc_Dict_Dict(*)(int(*cmp)(struct _tuple0*,struct _tuple0*)))Cyc_Dict_empty;_tmp115;})(Cyc_Absyn_qvar_cmp);_tmp118->struct_dict=_tmp36E;}),({
struct Cyc_Dict_Dict _tmp36D=({struct Cyc_Dict_Dict(*_tmp116)(int(*cmp)(struct _tuple0*,struct _tuple0*))=(struct Cyc_Dict_Dict(*)(int(*cmp)(struct _tuple0*,struct _tuple0*)))Cyc_Dict_empty;_tmp116;})(Cyc_Absyn_qvar_cmp);_tmp118->union_dict=_tmp36D;}),_tmp118->qualifier_edits=0,_tmp118->pointer_edits=0,_tmp118->zeroterm_edits=0,_tmp118->vardecl_locs=0,({
# 804
struct Cyc_Hashtable_Table*_tmp36C=({struct Cyc_Hashtable_Table*(*_tmp117)(int sz,int(*cmp)(unsigned,unsigned),int(*hash)(unsigned))=(struct Cyc_Hashtable_Table*(*)(int sz,int(*cmp)(unsigned,unsigned),int(*hash)(unsigned)))Cyc_Hashtable_create;_tmp117;})(128,Cyc_Port_cmp_seg_t,Cyc_Port_hash_seg_t);_tmp118->varusage_tab=_tmp36C;}),_tmp118->edits=0,_tmp118->porting=0,({
# 807
void*_tmp36B=Cyc_Port_void_ct();_tmp118->return_type=_tmp36B;});_tmp118;});
return({struct Cyc_Port_Cenv*_tmp113=_cycalloc(sizeof(*_tmp113));_tmp113->gcenv=g,_tmp113->cpos=Cyc_Port_Toplevel_pos,({
# 810
struct Cyc_Dict_Dict _tmp370=({struct Cyc_Dict_Dict(*_tmp112)(int(*cmp)(struct _tuple0*,struct _tuple0*))=(struct Cyc_Dict_Dict(*)(int(*cmp)(struct _tuple0*,struct _tuple0*)))Cyc_Dict_empty;_tmp112;})(Cyc_Absyn_qvar_cmp);_tmp113->var_dict=_tmp370;});_tmp113;});}
# 796
static int Cyc_Port_in_fn_arg(struct Cyc_Port_Cenv*env){
# 817
return(int)env->cpos == (int)Cyc_Port_FnArg_pos;}
# 796
static int Cyc_Port_in_fn_res(struct Cyc_Port_Cenv*env){
# 820
return(int)env->cpos == (int)Cyc_Port_FnRes_pos;}
# 796
static int Cyc_Port_in_toplevel(struct Cyc_Port_Cenv*env){
# 823
return(int)env->cpos == (int)Cyc_Port_Toplevel_pos;}
# 796
static void*Cyc_Port_lookup_return_type(struct Cyc_Port_Cenv*env){
# 826
return(env->gcenv)->return_type;}
# 796
static void*Cyc_Port_lookup_typedef(struct Cyc_Port_Cenv*env,struct _tuple0*n){
# 829
struct _handler_cons _tmp119;_push_handler(& _tmp119);{int _tmp11B=0;if(setjmp(_tmp119.handler))_tmp11B=1;if(!_tmp11B){
{struct _tuple12 _stmttmpA=*({struct _tuple12*(*_tmp372)(struct Cyc_Dict_Dict d,struct _tuple0*k)=({struct _tuple12*(*_tmp11E)(struct Cyc_Dict_Dict d,struct _tuple0*k)=(struct _tuple12*(*)(struct Cyc_Dict_Dict d,struct _tuple0*k))Cyc_Dict_lookup;_tmp11E;});struct Cyc_Dict_Dict _tmp371=(env->gcenv)->typedef_dict;_tmp372(_tmp371,n);});void*_tmp11C;_LL1: _tmp11C=_stmttmpA.f1;_LL2: {void*t=_tmp11C;
void*_tmp11D=t;_npop_handler(0U);return _tmp11D;}}
# 830
;_pop_handler();}else{void*_tmp11A=(void*)Cyc_Core_get_exn_thrown();void*_tmp11F=_tmp11A;void*_tmp120;if(((struct Cyc_Dict_Absent_exn_struct*)_tmp11F)->tag == Cyc_Dict_Absent){_LL4: _LL5:
# 837
 return Cyc_Absyn_sint_type;}else{_LL6: _tmp120=_tmp11F;_LL7: {void*exn=_tmp120;(int)_rethrow(exn);}}_LL3:;}}}
# 796
static void*Cyc_Port_lookup_typedef_ctype(struct Cyc_Port_Cenv*env,struct _tuple0*n){
# 843
struct _handler_cons _tmp121;_push_handler(& _tmp121);{int _tmp123=0;if(setjmp(_tmp121.handler))_tmp123=1;if(!_tmp123){
{struct _tuple12 _stmttmpB=*({struct _tuple12*(*_tmp374)(struct Cyc_Dict_Dict d,struct _tuple0*k)=({struct _tuple12*(*_tmp126)(struct Cyc_Dict_Dict d,struct _tuple0*k)=(struct _tuple12*(*)(struct Cyc_Dict_Dict d,struct _tuple0*k))Cyc_Dict_lookup;_tmp126;});struct Cyc_Dict_Dict _tmp373=(env->gcenv)->typedef_dict;_tmp374(_tmp373,n);});void*_tmp124;_LL1: _tmp124=_stmttmpB.f2;_LL2: {void*ct=_tmp124;
void*_tmp125=ct;_npop_handler(0U);return _tmp125;}}
# 844
;_pop_handler();}else{void*_tmp122=(void*)Cyc_Core_get_exn_thrown();void*_tmp127=_tmp122;void*_tmp128;if(((struct Cyc_Dict_Absent_exn_struct*)_tmp127)->tag == Cyc_Dict_Absent){_LL4: _LL5:
# 851
 return Cyc_Port_var();}else{_LL6: _tmp128=_tmp127;_LL7: {void*exn=_tmp128;(int)_rethrow(exn);}}_LL3:;}}}
# 796
static struct _tuple11*Cyc_Port_lookup_struct_decl(struct Cyc_Port_Cenv*env,struct _tuple0*n){
# 859
struct _tuple11**popt=({struct _tuple11**(*_tmp376)(struct Cyc_Dict_Dict d,struct _tuple0*k)=({struct _tuple11**(*_tmp12C)(struct Cyc_Dict_Dict d,struct _tuple0*k)=(struct _tuple11**(*)(struct Cyc_Dict_Dict d,struct _tuple0*k))Cyc_Dict_lookup_opt;_tmp12C;});struct Cyc_Dict_Dict _tmp375=(env->gcenv)->struct_dict;_tmp376(_tmp375,n);});
if((unsigned)popt)
return*popt;else{
# 863
struct Cyc_Absyn_Aggrdecl*ad=({struct Cyc_Absyn_Aggrdecl*_tmp12B=_cycalloc(sizeof(*_tmp12B));_tmp12B->kind=Cyc_Absyn_StructA,_tmp12B->sc=Cyc_Absyn_Public,_tmp12B->name=n,_tmp12B->tvs=0,_tmp12B->impl=0,_tmp12B->attributes=0,_tmp12B->expected_mem_kind=0;_tmp12B;});
# 866
struct _tuple11*p=({struct _tuple11*_tmp12A=_cycalloc(sizeof(*_tmp12A));_tmp12A->f1=ad,_tmp12A->f2=0;_tmp12A;});
({struct Cyc_Dict_Dict _tmp37A=({struct Cyc_Dict_Dict(*_tmp379)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct _tuple11*v)=({struct Cyc_Dict_Dict(*_tmp129)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct _tuple11*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct _tuple11*v))Cyc_Dict_insert;_tmp129;});struct Cyc_Dict_Dict _tmp378=(env->gcenv)->struct_dict;struct _tuple0*_tmp377=n;_tmp379(_tmp378,_tmp377,p);});(env->gcenv)->struct_dict=_tmp37A;});
return p;}}
# 796
static struct _tuple11*Cyc_Port_lookup_union_decl(struct Cyc_Port_Cenv*env,struct _tuple0*n){
# 876
struct _tuple11**popt=({struct _tuple11**(*_tmp37C)(struct Cyc_Dict_Dict d,struct _tuple0*k)=({struct _tuple11**(*_tmp130)(struct Cyc_Dict_Dict d,struct _tuple0*k)=(struct _tuple11**(*)(struct Cyc_Dict_Dict d,struct _tuple0*k))Cyc_Dict_lookup_opt;_tmp130;});struct Cyc_Dict_Dict _tmp37B=(env->gcenv)->union_dict;_tmp37C(_tmp37B,n);});
if((unsigned)popt)
return*popt;else{
# 880
struct Cyc_Absyn_Aggrdecl*ad=({struct Cyc_Absyn_Aggrdecl*_tmp12F=_cycalloc(sizeof(*_tmp12F));_tmp12F->kind=Cyc_Absyn_UnionA,_tmp12F->sc=Cyc_Absyn_Public,_tmp12F->name=n,_tmp12F->tvs=0,_tmp12F->impl=0,_tmp12F->attributes=0,_tmp12F->expected_mem_kind=0;_tmp12F;});
# 883
struct _tuple11*p=({struct _tuple11*_tmp12E=_cycalloc(sizeof(*_tmp12E));_tmp12E->f1=ad,_tmp12E->f2=0;_tmp12E;});
({struct Cyc_Dict_Dict _tmp380=({struct Cyc_Dict_Dict(*_tmp37F)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct _tuple11*v)=({struct Cyc_Dict_Dict(*_tmp12D)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct _tuple11*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct _tuple11*v))Cyc_Dict_insert;_tmp12D;});struct Cyc_Dict_Dict _tmp37E=(env->gcenv)->union_dict;struct _tuple0*_tmp37D=n;_tmp37F(_tmp37E,_tmp37D,p);});(env->gcenv)->union_dict=_tmp380;});
return p;}}struct _tuple14{struct _tuple12 f1;unsigned f2;};struct _tuple15{void*f1;struct _tuple12*f2;unsigned f3;};
# 796
static struct _tuple14 Cyc_Port_lookup_var(struct Cyc_Port_Cenv*env,struct _tuple0*x){
# 891
struct _handler_cons _tmp131;_push_handler(& _tmp131);{int _tmp133=0;if(setjmp(_tmp131.handler))_tmp133=1;if(!_tmp133){
{struct _tuple15 _stmttmpC=*({struct _tuple15*(*_tmp382)(struct Cyc_Dict_Dict d,struct _tuple0*k)=({struct _tuple15*(*_tmp137)(struct Cyc_Dict_Dict d,struct _tuple0*k)=(struct _tuple15*(*)(struct Cyc_Dict_Dict d,struct _tuple0*k))Cyc_Dict_lookup;_tmp137;});struct Cyc_Dict_Dict _tmp381=env->var_dict;_tmp382(_tmp381,x);});unsigned _tmp135;struct _tuple12*_tmp134;_LL1: _tmp134=_stmttmpC.f2;_tmp135=_stmttmpC.f3;_LL2: {struct _tuple12*p=_tmp134;unsigned loc=_tmp135;
struct _tuple14 _tmp136=({struct _tuple14 _tmp306;_tmp306.f1=*p,_tmp306.f2=loc;_tmp306;});_npop_handler(0U);return _tmp136;}}
# 892
;_pop_handler();}else{void*_tmp132=(void*)Cyc_Core_get_exn_thrown();void*_tmp138=_tmp132;void*_tmp139;if(((struct Cyc_Dict_Absent_exn_struct*)_tmp138)->tag == Cyc_Dict_Absent){_LL4: _LL5:
# 899
 return({struct _tuple14 _tmp307;({void*_tmp384=Cyc_Port_var();(_tmp307.f1).f1=_tmp384;}),({void*_tmp383=Cyc_Port_var();(_tmp307.f1).f2=_tmp383;}),_tmp307.f2=0U;_tmp307;});}else{_LL6: _tmp139=_tmp138;_LL7: {void*exn=_tmp139;(int)_rethrow(exn);}}_LL3:;}}}
# 796
static struct _tuple15*Cyc_Port_lookup_full_var(struct Cyc_Port_Cenv*env,struct _tuple0*x){
# 903
return({struct _tuple15*(*_tmp386)(struct Cyc_Dict_Dict d,struct _tuple0*k)=({struct _tuple15*(*_tmp13A)(struct Cyc_Dict_Dict d,struct _tuple0*k)=(struct _tuple15*(*)(struct Cyc_Dict_Dict d,struct _tuple0*k))Cyc_Dict_lookup;_tmp13A;});struct Cyc_Dict_Dict _tmp385=env->var_dict;_tmp386(_tmp385,x);});}
# 796
static int Cyc_Port_declared_var(struct Cyc_Port_Cenv*env,struct _tuple0*x){
# 907
return({int(*_tmp388)(struct Cyc_Dict_Dict d,struct _tuple0*k)=({int(*_tmp13B)(struct Cyc_Dict_Dict d,struct _tuple0*k)=(int(*)(struct Cyc_Dict_Dict d,struct _tuple0*k))Cyc_Dict_member;_tmp13B;});struct Cyc_Dict_Dict _tmp387=env->var_dict;_tmp388(_tmp387,x);});}
# 796
static int Cyc_Port_isfn(struct Cyc_Port_Cenv*env,struct _tuple0*x){
# 911
struct _handler_cons _tmp13C;_push_handler(& _tmp13C);{int _tmp13E=0;if(setjmp(_tmp13C.handler))_tmp13E=1;if(!_tmp13E){
{struct _tuple15 _stmttmpD=*({struct _tuple15*(*_tmp38A)(struct Cyc_Dict_Dict d,struct _tuple0*k)=({struct _tuple15*(*_tmp144)(struct Cyc_Dict_Dict d,struct _tuple0*k)=(struct _tuple15*(*)(struct Cyc_Dict_Dict d,struct _tuple0*k))Cyc_Dict_lookup;_tmp144;});struct Cyc_Dict_Dict _tmp389=env->var_dict;_tmp38A(_tmp389,x);});void*_tmp13F;_LL1: _tmp13F=_stmttmpD.f1;_LL2: {void*t=_tmp13F;
LOOP: {void*_tmp140=t;struct _tuple0*_tmp141;switch(*((int*)_tmp140)){case 8U: _LL4: _tmp141=((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmp140)->f1;_LL5: {struct _tuple0*n=_tmp141;
t=Cyc_Port_lookup_typedef(env,n);goto LOOP;}case 5U: _LL6: _LL7: {
int _tmp142=1;_npop_handler(0U);return _tmp142;}default: _LL8: _LL9: {
int _tmp143=0;_npop_handler(0U);return _tmp143;}}_LL3:;}}}
# 912
;_pop_handler();}else{void*_tmp13D=(void*)Cyc_Core_get_exn_thrown();void*_tmp145=_tmp13D;void*_tmp146;if(((struct Cyc_Dict_Absent_exn_struct*)_tmp145)->tag == Cyc_Dict_Absent){_LLB: _LLC:
# 923
 return 0;}else{_LLD: _tmp146=_tmp145;_LLE: {void*exn=_tmp146;(int)_rethrow(exn);}}_LLA:;}}}
# 796
static int Cyc_Port_isarray(struct Cyc_Port_Cenv*env,struct _tuple0*x){
# 928
struct _handler_cons _tmp147;_push_handler(& _tmp147);{int _tmp149=0;if(setjmp(_tmp147.handler))_tmp149=1;if(!_tmp149){
{struct _tuple15 _stmttmpE=*({struct _tuple15*(*_tmp38C)(struct Cyc_Dict_Dict d,struct _tuple0*k)=({struct _tuple15*(*_tmp14F)(struct Cyc_Dict_Dict d,struct _tuple0*k)=(struct _tuple15*(*)(struct Cyc_Dict_Dict d,struct _tuple0*k))Cyc_Dict_lookup;_tmp14F;});struct Cyc_Dict_Dict _tmp38B=env->var_dict;_tmp38C(_tmp38B,x);});void*_tmp14A;_LL1: _tmp14A=_stmttmpE.f1;_LL2: {void*t=_tmp14A;
LOOP: {void*_tmp14B=t;struct _tuple0*_tmp14C;switch(*((int*)_tmp14B)){case 8U: _LL4: _tmp14C=((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmp14B)->f1;_LL5: {struct _tuple0*n=_tmp14C;
t=Cyc_Port_lookup_typedef(env,n);goto LOOP;}case 4U: _LL6: _LL7: {
int _tmp14D=1;_npop_handler(0U);return _tmp14D;}default: _LL8: _LL9: {
int _tmp14E=0;_npop_handler(0U);return _tmp14E;}}_LL3:;}}}
# 929
;_pop_handler();}else{void*_tmp148=(void*)Cyc_Core_get_exn_thrown();void*_tmp150=_tmp148;void*_tmp151;if(((struct Cyc_Dict_Absent_exn_struct*)_tmp150)->tag == Cyc_Dict_Absent){_LLB: _LLC:
# 940
 return 0;}else{_LLD: _tmp151=_tmp150;_LLE: {void*exn=_tmp151;(int)_rethrow(exn);}}_LLA:;}}}
# 796
static struct Cyc_Port_Cenv*Cyc_Port_set_cpos(struct Cyc_Port_Cenv*env,enum Cyc_Port_CPos cpos){
# 947
return({struct Cyc_Port_Cenv*_tmp152=_cycalloc(sizeof(*_tmp152));_tmp152->gcenv=env->gcenv,_tmp152->var_dict=env->var_dict,_tmp152->cpos=cpos;_tmp152;});}
# 796
static void Cyc_Port_add_return_type(struct Cyc_Port_Cenv*env,void*t){
# 952
(env->gcenv)->return_type=t;}
# 796
static struct Cyc_Port_Cenv*Cyc_Port_add_var(struct Cyc_Port_Cenv*env,struct _tuple0*x,void*t,void*qual,void*ctype,unsigned loc){
# 958
return({struct Cyc_Port_Cenv*_tmp156=_cycalloc(sizeof(*_tmp156));_tmp156->gcenv=env->gcenv,({
struct Cyc_Dict_Dict _tmp391=({struct Cyc_Dict_Dict(*_tmp390)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct _tuple15*v)=({struct Cyc_Dict_Dict(*_tmp153)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct _tuple15*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct _tuple15*v))Cyc_Dict_insert;_tmp153;});struct Cyc_Dict_Dict _tmp38F=env->var_dict;struct _tuple0*_tmp38E=x;_tmp390(_tmp38F,_tmp38E,({struct _tuple15*_tmp155=_cycalloc(sizeof(*_tmp155));_tmp155->f1=t,({struct _tuple12*_tmp38D=({struct _tuple12*_tmp154=_cycalloc(sizeof(*_tmp154));_tmp154->f1=qual,_tmp154->f2=ctype;_tmp154;});_tmp155->f2=_tmp38D;}),_tmp155->f3=loc;_tmp155;}));});_tmp156->var_dict=_tmp391;}),_tmp156->cpos=env->cpos;_tmp156;});}
# 796
static void Cyc_Port_add_typedef(struct Cyc_Port_Cenv*env,struct _tuple0*n,void*t,void*ct){
# 965
({struct Cyc_Dict_Dict _tmp395=({struct Cyc_Dict_Dict(*_tmp394)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct _tuple12*v)=({struct Cyc_Dict_Dict(*_tmp157)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct _tuple12*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct _tuple12*v))Cyc_Dict_insert;_tmp157;});struct Cyc_Dict_Dict _tmp393=(env->gcenv)->typedef_dict;struct _tuple0*_tmp392=n;_tmp394(_tmp393,_tmp392,({struct _tuple12*_tmp158=_cycalloc(sizeof(*_tmp158));
_tmp158->f1=t,_tmp158->f2=ct;_tmp158;}));});
# 965
(env->gcenv)->typedef_dict=_tmp395;});}struct _tuple16{struct _tuple0*f1;unsigned f2;void*f3;};
# 796
static void Cyc_Port_register_localvar_decl(struct Cyc_Port_Cenv*env,struct _tuple0*x,unsigned loc,void*type,struct Cyc_Absyn_Exp*init){
# 969
({struct Cyc_List_List*_tmp397=({struct Cyc_List_List*_tmp15A=_cycalloc(sizeof(*_tmp15A));({struct _tuple16*_tmp396=({struct _tuple16*_tmp159=_cycalloc(sizeof(*_tmp159));_tmp159->f1=x,_tmp159->f2=loc,_tmp159->f3=type;_tmp159;});_tmp15A->hd=_tmp396;}),_tmp15A->tl=(env->gcenv)->vardecl_locs;_tmp15A;});(env->gcenv)->vardecl_locs=_tmp397;});
({void(*_tmp39A)(struct Cyc_Hashtable_Table*t,unsigned key,struct Cyc_Port_VarUsage*val)=({void(*_tmp15B)(struct Cyc_Hashtable_Table*t,unsigned key,struct Cyc_Port_VarUsage*val)=(void(*)(struct Cyc_Hashtable_Table*t,unsigned key,struct Cyc_Port_VarUsage*val))Cyc_Hashtable_insert;_tmp15B;});struct Cyc_Hashtable_Table*_tmp399=(env->gcenv)->varusage_tab;unsigned _tmp398=loc;_tmp39A(_tmp399,_tmp398,({struct Cyc_Port_VarUsage*_tmp15C=_cycalloc(sizeof(*_tmp15C));_tmp15C->address_taken=0,_tmp15C->init=init,_tmp15C->usage_locs=0;_tmp15C;}));});}
# 796
static void Cyc_Port_register_localvar_usage(struct Cyc_Port_Cenv*env,unsigned declloc,unsigned useloc,int takeaddress){
# 973
struct Cyc_Port_VarUsage*varusage=0;
if(({int(*_tmp39C)(struct Cyc_Hashtable_Table*t,unsigned key,struct Cyc_Port_VarUsage**data)=({int(*_tmp15D)(struct Cyc_Hashtable_Table*t,unsigned key,struct Cyc_Port_VarUsage**data)=(int(*)(struct Cyc_Hashtable_Table*t,unsigned key,struct Cyc_Port_VarUsage**data))Cyc_Hashtable_try_lookup;_tmp15D;});struct Cyc_Hashtable_Table*_tmp39B=(env->gcenv)->varusage_tab;_tmp39C(_tmp39B,declloc,& varusage);})){
if(takeaddress)((struct Cyc_Port_VarUsage*)_check_null(varusage))->address_taken=1;{struct Cyc_List_List*l=((struct Cyc_Port_VarUsage*)_check_null(varusage))->usage_locs;
# 977
({struct Cyc_List_List*_tmp39D=({struct Cyc_List_List*_tmp15E=_cycalloc(sizeof(*_tmp15E));_tmp15E->hd=(void*)useloc,_tmp15E->tl=l;_tmp15E;});((struct Cyc_Port_VarUsage*)_check_null(varusage))->usage_locs=_tmp39D;});}}}struct _tuple17{void*f1;void*f2;unsigned f3;};
# 796
static void Cyc_Port_register_const_cvar(struct Cyc_Port_Cenv*env,void*new_qual,void*orig_qual,unsigned loc){
# 985
({struct Cyc_List_List*_tmp39F=({struct Cyc_List_List*_tmp160=_cycalloc(sizeof(*_tmp160));({struct _tuple17*_tmp39E=({struct _tuple17*_tmp15F=_cycalloc(sizeof(*_tmp15F));_tmp15F->f1=new_qual,_tmp15F->f2=orig_qual,_tmp15F->f3=loc;_tmp15F;});_tmp160->hd=_tmp39E;}),_tmp160->tl=(env->gcenv)->qualifier_edits;_tmp160;});(env->gcenv)->qualifier_edits=_tmp39F;});}
# 796
static void Cyc_Port_register_ptr_cvars(struct Cyc_Port_Cenv*env,void*new_ptr,void*orig_ptr,unsigned loc){
# 993
({struct Cyc_List_List*_tmp3A1=({struct Cyc_List_List*_tmp162=_cycalloc(sizeof(*_tmp162));({struct _tuple17*_tmp3A0=({struct _tuple17*_tmp161=_cycalloc(sizeof(*_tmp161));_tmp161->f1=new_ptr,_tmp161->f2=orig_ptr,_tmp161->f3=loc;_tmp161;});_tmp162->hd=_tmp3A0;}),_tmp162->tl=(env->gcenv)->pointer_edits;_tmp162;});(env->gcenv)->pointer_edits=_tmp3A1;});}
# 796
static void Cyc_Port_register_zeroterm_cvars(struct Cyc_Port_Cenv*env,void*new_zt,void*orig_zt,unsigned loc){
# 999
({struct Cyc_List_List*_tmp3A3=({struct Cyc_List_List*_tmp164=_cycalloc(sizeof(*_tmp164));({struct _tuple17*_tmp3A2=({struct _tuple17*_tmp163=_cycalloc(sizeof(*_tmp163));_tmp163->f1=new_zt,_tmp163->f2=orig_zt,_tmp163->f3=loc;_tmp163;});_tmp164->hd=_tmp3A2;}),_tmp164->tl=(env->gcenv)->zeroterm_edits;_tmp164;});(env->gcenv)->zeroterm_edits=_tmp3A3;});}
# 796
static void*Cyc_Port_main_type(){
# 1006
struct _tuple8*arg1=({struct _tuple8*_tmp168=_cycalloc(sizeof(*_tmp168));
_tmp168->f1=0,({struct Cyc_Absyn_Tqual _tmp3A4=Cyc_Absyn_empty_tqual(0U);_tmp168->f2=_tmp3A4;}),_tmp168->f3=Cyc_Absyn_sint_type;_tmp168;});
struct _tuple8*arg2=({struct _tuple8*_tmp167=_cycalloc(sizeof(*_tmp167));
_tmp167->f1=0,({struct Cyc_Absyn_Tqual _tmp3A9=Cyc_Absyn_empty_tqual(0U);_tmp167->f2=_tmp3A9;}),({
void*_tmp3A8=({void*_tmp3A7=Cyc_Absyn_string_type(Cyc_Absyn_wildtyp(0));void*_tmp3A6=Cyc_Absyn_wildtyp(0);struct Cyc_Absyn_Tqual _tmp3A5=
Cyc_Absyn_empty_tqual(0U);
# 1010
Cyc_Absyn_fatptr_type(_tmp3A7,_tmp3A6,_tmp3A5,
Cyc_Tcutil_any_bool(0));});
# 1010
_tmp167->f3=_tmp3A8;});_tmp167;});
# 1012
return(void*)({struct Cyc_Absyn_FnType_Absyn_Type_struct*_tmp166=_cycalloc(sizeof(*_tmp166));_tmp166->tag=5U,(_tmp166->f1).tvars=0,(_tmp166->f1).effect=0,({
struct Cyc_Absyn_Tqual _tmp3AB=Cyc_Absyn_empty_tqual(0U);(_tmp166->f1).ret_tqual=_tmp3AB;}),(_tmp166->f1).ret_type=Cyc_Absyn_sint_type,({
# 1015
struct Cyc_List_List*_tmp3AA=({struct _tuple8*_tmp165[2U];_tmp165[0]=arg1,_tmp165[1]=arg2;Cyc_List_list(_tag_fat(_tmp165,sizeof(struct _tuple8*),2U));});(_tmp166->f1).args=_tmp3AA;}),(_tmp166->f1).c_varargs=0,(_tmp166->f1).cyc_varargs=0,(_tmp166->f1).rgn_po=0,(_tmp166->f1).attributes=0,(_tmp166->f1).requires_clause=0,(_tmp166->f1).requires_relns=0,(_tmp166->f1).ensures_clause=0,(_tmp166->f1).ensures_relns=0,(_tmp166->f1).ieffect=0,(_tmp166->f1).oeffect=0;_tmp166;});}
# 796
static void*Cyc_Port_type_to_ctype(struct Cyc_Port_Cenv*env,void*t);
# 1029
static struct Cyc_Port_Cenv*Cyc_Port_initial_cenv(){
struct Cyc_Port_Cenv*e=Cyc_Port_empty_cenv();
# 1032
e=({struct Cyc_Port_Cenv*_tmp3B2=e;struct _tuple0*_tmp3B1=({struct _tuple0*_tmp16B=_cycalloc(sizeof(*_tmp16B));_tmp16B->f1=Cyc_Absyn_Loc_n,({struct _fat_ptr*_tmp3AD=({struct _fat_ptr*_tmp16A=_cycalloc(sizeof(*_tmp16A));({struct _fat_ptr _tmp3AC=({const char*_tmp169="main";_tag_fat(_tmp169,sizeof(char),5U);});*_tmp16A=_tmp3AC;});_tmp16A;});_tmp16B->f2=_tmp3AD;});_tmp16B;});void*_tmp3B0=Cyc_Port_main_type();void*_tmp3AF=Cyc_Port_const_ct();Cyc_Port_add_var(_tmp3B2,_tmp3B1,_tmp3B0,_tmp3AF,({
struct Cyc_Port_Cenv*_tmp3AE=e;Cyc_Port_type_to_ctype(_tmp3AE,Cyc_Port_main_type());}),0U);});
return e;}
# 1029
static void Cyc_Port_region_counts(struct Cyc_Dict_Dict*cts,void*t){
# 1041
void*_stmttmpF=Cyc_Port_compress_ct(t);void*_tmp16C=_stmttmpF;struct Cyc_List_List*_tmp16E;void*_tmp16D;void*_tmp171;void*_tmp170;void*_tmp16F;void*_tmp176;void*_tmp175;void*_tmp174;void*_tmp173;void*_tmp172;struct _fat_ptr*_tmp177;switch(*((int*)_tmp16C)){case 0U: _LL1: _LL2:
 goto _LL4;case 1U: _LL3: _LL4:
 goto _LL6;case 2U: _LL5: _LL6:
 goto _LL8;case 3U: _LL7: _LL8:
 goto _LLA;case 4U: _LL9: _LLA:
 goto _LLC;case 5U: _LLB: _LLC:
 goto _LLE;case 6U: _LLD: _LLE:
 goto _LL10;case 14U: _LLF: _LL10:
 goto _LL12;case 13U: _LL11: _LL12:
 goto _LL14;case 16U: _LL13: _LL14:
 goto _LL16;case 8U: _LL15: _LL16:
 goto _LL18;case 9U: _LL17: _LL18:
 goto _LL1A;case 7U: _LL19: _LL1A:
 return;case 10U: _LL1B: _tmp177=((struct Cyc_Port_RgnVar_ct_Port_Ctype_struct*)_tmp16C)->f1;_LL1C: {struct _fat_ptr*n=_tmp177;
# 1056
if(!({int(*_tmp3B4)(struct Cyc_Dict_Dict d,struct _fat_ptr*k)=({int(*_tmp178)(struct Cyc_Dict_Dict d,struct _fat_ptr*k)=(int(*)(struct Cyc_Dict_Dict d,struct _fat_ptr*k))Cyc_Dict_member;_tmp178;});struct Cyc_Dict_Dict _tmp3B3=*cts;_tmp3B4(_tmp3B3,n);}))
({struct Cyc_Dict_Dict _tmp3B8=({struct Cyc_Dict_Dict(*_tmp3B7)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,int*v)=({struct Cyc_Dict_Dict(*_tmp179)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,int*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _fat_ptr*k,int*v))Cyc_Dict_insert;_tmp179;});struct Cyc_Dict_Dict _tmp3B6=*cts;struct _fat_ptr*_tmp3B5=n;_tmp3B7(_tmp3B6,_tmp3B5,({int*_tmp17A=_cycalloc_atomic(sizeof(*_tmp17A));*_tmp17A=0;_tmp17A;}));});*cts=_tmp3B8;});{
# 1056
int*p=({
# 1058
int*(*_tmp3BA)(struct Cyc_Dict_Dict d,struct _fat_ptr*k)=({int*(*_tmp17B)(struct Cyc_Dict_Dict d,struct _fat_ptr*k)=(int*(*)(struct Cyc_Dict_Dict d,struct _fat_ptr*k))Cyc_Dict_lookup;_tmp17B;});struct Cyc_Dict_Dict _tmp3B9=*cts;_tmp3BA(_tmp3B9,n);});
*p=*p + 1;
return;}}case 11U: _LL1D: _tmp172=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmp16C)->f1;_tmp173=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmp16C)->f2;_tmp174=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmp16C)->f3;_tmp175=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmp16C)->f4;_tmp176=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmp16C)->f5;_LL1E: {void*t1=_tmp172;void*t2=_tmp173;void*t3=_tmp174;void*t4=_tmp175;void*zt=_tmp176;
# 1062
Cyc_Port_region_counts(cts,t1);
Cyc_Port_region_counts(cts,t4);
return;}case 12U: _LL1F: _tmp16F=(void*)((struct Cyc_Port_Array_ct_Port_Ctype_struct*)_tmp16C)->f1;_tmp170=(void*)((struct Cyc_Port_Array_ct_Port_Ctype_struct*)_tmp16C)->f2;_tmp171=(void*)((struct Cyc_Port_Array_ct_Port_Ctype_struct*)_tmp16C)->f3;_LL20: {void*t1=_tmp16F;void*t2=_tmp170;void*zt=_tmp171;
# 1066
Cyc_Port_region_counts(cts,t1);
return;}default: _LL21: _tmp16D=(void*)((struct Cyc_Port_Fn_ct_Port_Ctype_struct*)_tmp16C)->f1;_tmp16E=((struct Cyc_Port_Fn_ct_Port_Ctype_struct*)_tmp16C)->f2;_LL22: {void*t1=_tmp16D;struct Cyc_List_List*ts=_tmp16E;
# 1069
Cyc_Port_region_counts(cts,t1);
for(0;(unsigned)ts;ts=ts->tl){Cyc_Port_region_counts(cts,(void*)ts->hd);}
return;}}_LL0:;}
# 1029
static void Cyc_Port_register_rgns(struct Cyc_Port_Cenv*env,struct Cyc_Dict_Dict counts,int fn_res,void*t,void*c){
# 1082
c=Cyc_Port_compress_ct(c);{
struct _tuple12 _stmttmp10=({struct _tuple12 _tmp308;_tmp308.f1=t,_tmp308.f2=c;_tmp308;});struct _tuple12 _tmp17C=_stmttmp10;struct Cyc_List_List*_tmp180;void*_tmp17F;struct Cyc_List_List*_tmp17E;void*_tmp17D;void*_tmp182;void*_tmp181;void*_tmp187;void*_tmp186;struct Cyc_Absyn_PtrLoc*_tmp185;void*_tmp184;void*_tmp183;switch(*((int*)_tmp17C.f1)){case 3U: if(((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmp17C.f2)->tag == 11U){_LL1: _tmp183=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp17C.f1)->f1).elt_type;_tmp184=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp17C.f1)->f1).ptr_atts).rgn;_tmp185=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp17C.f1)->f1).ptr_atts).ptrloc;_tmp186=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmp17C.f2)->f1;_tmp187=(void*)((struct Cyc_Port_Ptr_ct_Port_Ctype_struct*)_tmp17C.f2)->f4;_LL2: {void*et=_tmp183;void*rt=_tmp184;struct Cyc_Absyn_PtrLoc*ploc=_tmp185;void*ec=_tmp186;void*rc=_tmp187;
# 1085
Cyc_Port_register_rgns(env,counts,fn_res,et,ec);{
unsigned loc=(unsigned)ploc?ploc->rgn_loc:(unsigned)0;
rc=Cyc_Port_compress_ct(rc);
# 1102 "port.cyc"
({struct Cyc_List_List*_tmp3BE=({struct Cyc_List_List*_tmp18B=_cycalloc(sizeof(*_tmp18B));
({struct Cyc_Port_Edit*_tmp3BD=({struct Cyc_Port_Edit*_tmp18A=_cycalloc(sizeof(*_tmp18A));_tmp18A->loc=loc,({struct _fat_ptr _tmp3BC=({const char*_tmp188="`H ";_tag_fat(_tmp188,sizeof(char),4U);});_tmp18A->old_string=_tmp3BC;}),({struct _fat_ptr _tmp3BB=({const char*_tmp189="";_tag_fat(_tmp189,sizeof(char),1U);});_tmp18A->new_string=_tmp3BB;});_tmp18A;});_tmp18B->hd=_tmp3BD;}),_tmp18B->tl=(env->gcenv)->edits;_tmp18B;});
# 1102
(env->gcenv)->edits=_tmp3BE;});
# 1105
goto _LL0;}}}else{goto _LL7;}case 4U: if(((struct Cyc_Port_Array_ct_Port_Ctype_struct*)_tmp17C.f2)->tag == 12U){_LL3: _tmp181=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp17C.f1)->f1).elt_type;_tmp182=(void*)((struct Cyc_Port_Array_ct_Port_Ctype_struct*)_tmp17C.f2)->f1;_LL4: {void*et=_tmp181;void*ec=_tmp182;
# 1107
Cyc_Port_register_rgns(env,counts,fn_res,et,ec);
goto _LL0;}}else{goto _LL7;}case 5U: if(((struct Cyc_Port_Fn_ct_Port_Ctype_struct*)_tmp17C.f2)->tag == 15U){_LL5: _tmp17D=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp17C.f1)->f1).ret_type;_tmp17E=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp17C.f1)->f1).args;_tmp17F=(void*)((struct Cyc_Port_Fn_ct_Port_Ctype_struct*)_tmp17C.f2)->f1;_tmp180=((struct Cyc_Port_Fn_ct_Port_Ctype_struct*)_tmp17C.f2)->f2;_LL6: {void*rt=_tmp17D;struct Cyc_List_List*argst=_tmp17E;void*rc=_tmp17F;struct Cyc_List_List*argsc=_tmp180;
# 1110
Cyc_Port_register_rgns(env,counts,1,rt,rc);
for(0;argst != 0 && argsc != 0;(argst=argst->tl,argsc=argsc->tl)){
struct _tuple8 _stmttmp11=*((struct _tuple8*)argst->hd);void*_tmp18C;_LLA: _tmp18C=_stmttmp11.f3;_LLB: {void*at=_tmp18C;
void*ac=(void*)argsc->hd;
Cyc_Port_register_rgns(env,counts,0,at,ac);}}
# 1116
goto _LL0;}}else{goto _LL7;}default: _LL7: _LL8:
 goto _LL0;}_LL0:;}}
# 1029 "port.cyc"
static void*Cyc_Port_gen_exp(int takeaddress,struct Cyc_Port_Cenv*env,struct Cyc_Absyn_Exp*e);
# 1124 "port.cyc"
static void Cyc_Port_gen_stmt(struct Cyc_Port_Cenv*env,struct Cyc_Absyn_Stmt*s);
static struct Cyc_Port_Cenv*Cyc_Port_gen_localdecl(struct Cyc_Port_Cenv*env,struct Cyc_Absyn_Decl*d,int takeaddress);
# 1128
static int Cyc_Port_is_char(struct Cyc_Port_Cenv*env,void*t){
void*_tmp18D=t;struct _tuple0*_tmp18E;switch(*((int*)_tmp18D)){case 8U: _LL1: _tmp18E=((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmp18D)->f1;_LL2: {struct _tuple0*n=_tmp18E;
# 1131
(*n).f1=Cyc_Absyn_Loc_n;
return({struct Cyc_Port_Cenv*_tmp3BF=env;Cyc_Port_is_char(_tmp3BF,Cyc_Port_lookup_typedef(env,n));});}case 0U: if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp18D)->f1)->tag == 1U){if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp18D)->f1)->f2 == Cyc_Absyn_Char_sz){_LL3: _LL4:
 return 1;}else{goto _LL5;}}else{goto _LL5;}default: _LL5: _LL6:
 return 0;}_LL0:;}
# 1128
static void*Cyc_Port_type_to_ctype(struct Cyc_Port_Cenv*env,void*t){
# 1140
void*_tmp18F=t;struct Cyc_List_List*_tmp191;void*_tmp190;unsigned _tmp195;void*_tmp194;struct Cyc_Absyn_Tqual _tmp193;void*_tmp192;struct Cyc_Absyn_PtrLoc*_tmp19C;void*_tmp19B;void*_tmp19A;void*_tmp199;void*_tmp198;struct Cyc_Absyn_Tqual _tmp197;void*_tmp196;struct Cyc_List_List*_tmp19D;union Cyc_Absyn_AggrInfo _tmp19E;struct _tuple0*_tmp19F;switch(*((int*)_tmp18F)){case 8U: _LL1: _tmp19F=((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmp18F)->f1;_LL2: {struct _tuple0*n=_tmp19F;
# 1142
(*n).f1=Cyc_Absyn_Loc_n;
return Cyc_Port_lookup_typedef_ctype(env,n);}case 0U: switch(*((int*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp18F)->f1)){case 0U: _LL3: _LL4:
 return Cyc_Port_void_ct();case 1U: _LL7: _LL8:
# 1221
 goto _LLA;case 2U: _LL9: _LLA:
 return Cyc_Port_arith_ct();case 20U: _LLF: _tmp19E=((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp18F)->f1)->f1;_LL10: {union Cyc_Absyn_AggrInfo ai=_tmp19E;
# 1264
union Cyc_Absyn_AggrInfo _tmp1A9=ai;struct Cyc_Absyn_Aggrdecl**_tmp1AA;struct _tuple0*_tmp1AB;struct _tuple0*_tmp1AC;if((_tmp1A9.UnknownAggr).tag == 1){if(((_tmp1A9.UnknownAggr).val).f1 == Cyc_Absyn_StructA){_LL38: _tmp1AC=((_tmp1A9.UnknownAggr).val).f2;_LL39: {struct _tuple0*n=_tmp1AC;
# 1266
(*n).f1=Cyc_Absyn_Loc_n;{
struct _tuple11*p=Cyc_Port_lookup_struct_decl(env,n);
return Cyc_Port_known_aggr_ct(p);}}}else{_LL3A: _tmp1AB=((_tmp1A9.UnknownAggr).val).f2;_LL3B: {struct _tuple0*n=_tmp1AB;
# 1270
(*n).f1=Cyc_Absyn_Loc_n;{
struct _tuple11*p=Cyc_Port_lookup_union_decl(env,n);
return Cyc_Port_known_aggr_ct(p);}}}}else{_LL3C: _tmp1AA=(_tmp1A9.KnownAggr).val;_LL3D: {struct Cyc_Absyn_Aggrdecl**adp=_tmp1AA;
# 1274
struct Cyc_Absyn_Aggrdecl*ad=*adp;
enum Cyc_Absyn_AggrKind _stmttmp17=ad->kind;enum Cyc_Absyn_AggrKind _tmp1AD=_stmttmp17;if(_tmp1AD == Cyc_Absyn_StructA){_LL3F: _LL40: {
# 1277
struct _tuple11*p=Cyc_Port_lookup_struct_decl(env,ad->name);
return Cyc_Port_known_aggr_ct(p);}}else{_LL41: _LL42: {
# 1280
struct _tuple11*p=Cyc_Port_lookup_union_decl(env,ad->name);
return Cyc_Port_known_aggr_ct(p);}}_LL3E:;}}_LL37:;}case 15U: _LL11: _LL12:
# 1284
 return Cyc_Port_arith_ct();case 16U: _LL13: _tmp19D=((struct Cyc_Absyn_AnonEnumCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp18F)->f1)->f1;_LL14: {struct Cyc_List_List*fs=_tmp19D;
# 1287
for(0;(unsigned)fs;fs=fs->tl){
(*((struct Cyc_Absyn_Enumfield*)fs->hd)->name).f1=Cyc_Absyn_Loc_n;
env=({struct Cyc_Port_Cenv*_tmp3C3=env;struct _tuple0*_tmp3C2=((struct Cyc_Absyn_Enumfield*)fs->hd)->name;void*_tmp3C1=Cyc_Absyn_sint_type;void*_tmp3C0=Cyc_Port_const_ct();Cyc_Port_add_var(_tmp3C3,_tmp3C2,_tmp3C1,_tmp3C0,Cyc_Port_arith_ct(),0U);});}
# 1291
return Cyc_Port_arith_ct();}default: goto _LL15;}case 3U: _LL5: _tmp196=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp18F)->f1).elt_type;_tmp197=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp18F)->f1).elt_tq;_tmp198=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp18F)->f1).ptr_atts).rgn;_tmp199=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp18F)->f1).ptr_atts).nullable;_tmp19A=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp18F)->f1).ptr_atts).bounds;_tmp19B=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp18F)->f1).ptr_atts).zero_term;_tmp19C=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp18F)->f1).ptr_atts).ptrloc;_LL6: {void*et=_tmp196;struct Cyc_Absyn_Tqual tq=_tmp197;void*r=_tmp198;void*n=_tmp199;void*b=_tmp19A;void*zt=_tmp19B;struct Cyc_Absyn_PtrLoc*loc=_tmp19C;
# 1146
unsigned ptr_loc=(unsigned)loc?loc->ptr_loc:(unsigned)0;
unsigned rgn_loc=(unsigned)loc?loc->rgn_loc:(unsigned)0;
unsigned zt_loc=(unsigned)loc?loc->zt_loc:(unsigned)0;
# 1152
void*cet=Cyc_Port_type_to_ctype(env,et);
void*new_rgn;
# 1155
{void*_tmp1A0=r;struct _fat_ptr*_tmp1A1;switch(*((int*)_tmp1A0)){case 0U: if(((struct Cyc_Absyn_HeapCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp1A0)->f1)->tag == 5U){_LL18: _LL19:
# 1157
 new_rgn=Cyc_Port_heap_ct();goto _LL17;}else{goto _LL1E;}case 2U: _LL1A: _tmp1A1=(((struct Cyc_Absyn_VarType_Absyn_Type_struct*)_tmp1A0)->f1)->name;_LL1B: {struct _fat_ptr*name=_tmp1A1;
# 1159
new_rgn=Cyc_Port_rgnvar_ct(name);goto _LL17;}case 1U: _LL1C: _LL1D:
# 1163
 if(Cyc_Port_in_fn_arg(env))
new_rgn=Cyc_Port_rgnvar_ct(Cyc_Port_new_region_var());else{
# 1166
if(Cyc_Port_in_fn_res(env)|| Cyc_Port_in_toplevel(env))
new_rgn=Cyc_Port_heap_ct();else{
new_rgn=Cyc_Port_var();}}
goto _LL17;default: _LL1E: _LL1F:
# 1171
 new_rgn=Cyc_Port_heap_ct();goto _LL17;}_LL17:;}{
# 1173
int orig_fat=Cyc_Tcutil_get_bounds_exp(Cyc_Absyn_fat_bound_type,b)== 0;
int orig_zt;
{void*_stmttmp12=Cyc_Tcutil_compress(zt);void*_tmp1A2=_stmttmp12;if(((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp1A2)->tag == 1U){_LL21: _LL22:
 orig_zt=Cyc_Port_is_char(env,et);goto _LL20;}else{_LL23: _LL24:
 orig_zt=Cyc_Tcutil_force_type2bool(0,zt);goto _LL20;}_LL20:;}
# 1179
if((env->gcenv)->porting){
void*cqv=Cyc_Port_var();
# 1184
({struct Cyc_Port_Cenv*_tmp3C6=env;void*_tmp3C5=cqv;void*_tmp3C4=tq.print_const?Cyc_Port_const_ct(): Cyc_Port_notconst_ct();Cyc_Port_register_const_cvar(_tmp3C6,_tmp3C5,_tmp3C4,tq.loc);});
# 1187
if(tq.print_const)({void*_tmp3C7=cqv;Cyc_Port_unify_ct(_tmp3C7,Cyc_Port_const_ct());});{void*cbv=
# 1193
Cyc_Port_var();
# 1196
({struct Cyc_Port_Cenv*_tmp3CA=env;void*_tmp3C9=cbv;void*_tmp3C8=orig_fat?Cyc_Port_fat_ct(): Cyc_Port_thin_ct();Cyc_Port_register_ptr_cvars(_tmp3CA,_tmp3C9,_tmp3C8,ptr_loc);});
# 1198
if(orig_fat)({void*_tmp3CB=cbv;Cyc_Port_unify_ct(_tmp3CB,Cyc_Port_fat_ct());});{void*czv=
Cyc_Port_var();
# 1202
({struct Cyc_Port_Cenv*_tmp3CE=env;void*_tmp3CD=czv;void*_tmp3CC=orig_zt?Cyc_Port_zterm_ct(): Cyc_Port_nozterm_ct();Cyc_Port_register_zeroterm_cvars(_tmp3CE,_tmp3CD,_tmp3CC,zt_loc);});
# 1204
{void*_stmttmp13=Cyc_Tcutil_compress(zt);void*_tmp1A3=_stmttmp13;if(((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp1A3)->tag == 1U){_LL26: _LL27:
 goto _LL25;}else{_LL28: _LL29:
# 1207
({void*_tmp3CF=czv;Cyc_Port_unify_ct(_tmp3CF,Cyc_Tcutil_force_type2bool(0,zt)?Cyc_Port_zterm_ct(): Cyc_Port_nozterm_ct());});
goto _LL25;}_LL25:;}
# 1213
return Cyc_Port_ptr_ct(cet,cqv,cbv,new_rgn,czv);}}}else{
# 1217
return({void*_tmp3D3=cet;void*_tmp3D2=tq.print_const?Cyc_Port_const_ct(): Cyc_Port_notconst_ct();void*_tmp3D1=
orig_fat?Cyc_Port_fat_ct(): Cyc_Port_thin_ct();
# 1217
void*_tmp3D0=new_rgn;Cyc_Port_ptr_ct(_tmp3D3,_tmp3D2,_tmp3D1,_tmp3D0,
# 1219
orig_zt?Cyc_Port_zterm_ct(): Cyc_Port_nozterm_ct());});}}}case 4U: _LLB: _tmp192=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp18F)->f1).elt_type;_tmp193=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp18F)->f1).tq;_tmp194=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp18F)->f1).zero_term;_tmp195=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp18F)->f1).zt_loc;_LLC: {void*et=_tmp192;struct Cyc_Absyn_Tqual tq=_tmp193;void*zt=_tmp194;unsigned ztloc=_tmp195;
# 1225
void*cet=Cyc_Port_type_to_ctype(env,et);
int orig_zt;
{void*_stmttmp14=Cyc_Tcutil_compress(zt);void*_tmp1A4=_stmttmp14;if(((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp1A4)->tag == 1U){_LL2B: _LL2C:
 orig_zt=0;goto _LL2A;}else{_LL2D: _LL2E:
 orig_zt=Cyc_Tcutil_force_type2bool(0,zt);goto _LL2A;}_LL2A:;}
# 1231
if((env->gcenv)->porting){
void*cqv=Cyc_Port_var();
({struct Cyc_Port_Cenv*_tmp3D6=env;void*_tmp3D5=cqv;void*_tmp3D4=tq.print_const?Cyc_Port_const_ct(): Cyc_Port_notconst_ct();Cyc_Port_register_const_cvar(_tmp3D6,_tmp3D5,_tmp3D4,tq.loc);});{
# 1240
void*czv=Cyc_Port_var();
({struct Cyc_Port_Cenv*_tmp3D9=env;void*_tmp3D8=czv;void*_tmp3D7=orig_zt?Cyc_Port_zterm_ct(): Cyc_Port_nozterm_ct();Cyc_Port_register_zeroterm_cvars(_tmp3D9,_tmp3D8,_tmp3D7,ztloc);});
# 1243
{void*_stmttmp15=Cyc_Tcutil_compress(zt);void*_tmp1A5=_stmttmp15;if(((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp1A5)->tag == 1U){_LL30: _LL31:
 goto _LL2F;}else{_LL32: _LL33:
# 1246
({void*_tmp3DA=czv;Cyc_Port_unify_ct(_tmp3DA,Cyc_Tcutil_force_type2bool(0,zt)?Cyc_Port_zterm_ct(): Cyc_Port_nozterm_ct());});
goto _LL2F;}_LL2F:;}
# 1249
return Cyc_Port_array_ct(cet,cqv,czv);}}else{
# 1251
return({void*_tmp3DC=cet;void*_tmp3DB=tq.print_const?Cyc_Port_const_ct(): Cyc_Port_notconst_ct();Cyc_Port_array_ct(_tmp3DC,_tmp3DB,
orig_zt?Cyc_Port_zterm_ct(): Cyc_Port_nozterm_ct());});}}case 5U: _LLD: _tmp190=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp18F)->f1).ret_type;_tmp191=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp18F)->f1).args;_LLE: {void*rt=_tmp190;struct Cyc_List_List*args=_tmp191;
# 1255
void*crt=({struct Cyc_Port_Cenv*_tmp3DD=Cyc_Port_set_cpos(env,Cyc_Port_FnRes_pos);Cyc_Port_type_to_ctype(_tmp3DD,rt);});
struct Cyc_Port_Cenv*env_arg=Cyc_Port_set_cpos(env,Cyc_Port_FnArg_pos);
struct Cyc_List_List*cargs=0;
for(0;args != 0;args=args->tl){
struct _tuple8 _stmttmp16=*((struct _tuple8*)args->hd);void*_tmp1A7;struct Cyc_Absyn_Tqual _tmp1A6;_LL35: _tmp1A6=_stmttmp16.f2;_tmp1A7=_stmttmp16.f3;_LL36: {struct Cyc_Absyn_Tqual tq=_tmp1A6;void*t=_tmp1A7;
cargs=({struct Cyc_List_List*_tmp1A8=_cycalloc(sizeof(*_tmp1A8));({void*_tmp3DE=Cyc_Port_type_to_ctype(env_arg,t);_tmp1A8->hd=_tmp3DE;}),_tmp1A8->tl=cargs;_tmp1A8;});}}
# 1262
return({void*_tmp3DF=crt;Cyc_Port_fn_ct(_tmp3DF,Cyc_List_imp_rev(cargs));});}default: _LL15: _LL16:
# 1292
 return Cyc_Port_arith_ct();}_LL0:;}
# 1128
static void*Cyc_Port_gen_primop(struct Cyc_Port_Cenv*env,enum Cyc_Absyn_Primop p,struct Cyc_List_List*args){
# 1298
void*arg1=Cyc_Port_compress_ct((void*)((struct Cyc_List_List*)_check_null(args))->hd);
struct Cyc_List_List*arg2s=args->tl;
enum Cyc_Absyn_Primop _tmp1AE=p;switch(_tmp1AE){case Cyc_Absyn_Plus: _LL1: _LL2: {
# 1305
void*arg2=Cyc_Port_compress_ct((void*)((struct Cyc_List_List*)_check_null(arg2s))->hd);
if(({void*_tmp3E4=arg1;Cyc_Port_leq(_tmp3E4,({void*_tmp3E3=Cyc_Port_var();void*_tmp3E2=Cyc_Port_var();void*_tmp3E1=Cyc_Port_fat_ct();void*_tmp3E0=Cyc_Port_var();Cyc_Port_ptr_ct(_tmp3E3,_tmp3E2,_tmp3E1,_tmp3E0,Cyc_Port_var());}));})){
({void*_tmp3E5=arg2;Cyc_Port_leq(_tmp3E5,Cyc_Port_arith_ct());});
return arg1;}else{
if(({void*_tmp3EA=arg2;Cyc_Port_leq(_tmp3EA,({void*_tmp3E9=Cyc_Port_var();void*_tmp3E8=Cyc_Port_var();void*_tmp3E7=Cyc_Port_fat_ct();void*_tmp3E6=Cyc_Port_var();Cyc_Port_ptr_ct(_tmp3E9,_tmp3E8,_tmp3E7,_tmp3E6,Cyc_Port_var());}));})){
({void*_tmp3EB=arg1;Cyc_Port_leq(_tmp3EB,Cyc_Port_arith_ct());});
return arg2;}else{
# 1313
({void*_tmp3EC=arg1;Cyc_Port_leq(_tmp3EC,Cyc_Port_arith_ct());});
({void*_tmp3ED=arg2;Cyc_Port_leq(_tmp3ED,Cyc_Port_arith_ct());});
return Cyc_Port_arith_ct();}}}case Cyc_Absyn_Minus: _LL3: _LL4:
# 1322
 if(arg2s == 0){
# 1324
({void*_tmp3EE=arg1;Cyc_Port_leq(_tmp3EE,Cyc_Port_arith_ct());});
return Cyc_Port_arith_ct();}else{
# 1327
void*arg2=Cyc_Port_compress_ct((void*)arg2s->hd);
if(({void*_tmp3F3=arg1;Cyc_Port_leq(_tmp3F3,({void*_tmp3F2=Cyc_Port_var();void*_tmp3F1=Cyc_Port_var();void*_tmp3F0=Cyc_Port_fat_ct();void*_tmp3EF=Cyc_Port_var();Cyc_Port_ptr_ct(_tmp3F2,_tmp3F1,_tmp3F0,_tmp3EF,Cyc_Port_var());}));})){
if(({void*_tmp3F8=arg2;Cyc_Port_leq(_tmp3F8,({void*_tmp3F7=Cyc_Port_var();void*_tmp3F6=Cyc_Port_var();void*_tmp3F5=Cyc_Port_fat_ct();void*_tmp3F4=Cyc_Port_var();Cyc_Port_ptr_ct(_tmp3F7,_tmp3F6,_tmp3F5,_tmp3F4,Cyc_Port_var());}));}))
return Cyc_Port_arith_ct();else{
# 1332
({void*_tmp3F9=arg2;Cyc_Port_leq(_tmp3F9,Cyc_Port_arith_ct());});
return arg1;}}else{
# 1336
({void*_tmp3FA=arg1;Cyc_Port_leq(_tmp3FA,Cyc_Port_arith_ct());});
({void*_tmp3FB=arg1;Cyc_Port_leq(_tmp3FB,Cyc_Port_arith_ct());});
return Cyc_Port_arith_ct();}}case Cyc_Absyn_Times: _LL5: _LL6:
# 1341
 goto _LL8;case Cyc_Absyn_Div: _LL7: _LL8: goto _LLA;case Cyc_Absyn_Mod: _LL9: _LLA: goto _LLC;case Cyc_Absyn_Eq: _LLB: _LLC: goto _LLE;case Cyc_Absyn_Neq: _LLD: _LLE: goto _LL10;case Cyc_Absyn_Lt: _LLF: _LL10: goto _LL12;case Cyc_Absyn_Gt: _LL11: _LL12: goto _LL14;case Cyc_Absyn_Lte: _LL13: _LL14:
 goto _LL16;case Cyc_Absyn_Gte: _LL15: _LL16: goto _LL18;case Cyc_Absyn_Bitand: _LL17: _LL18: goto _LL1A;case Cyc_Absyn_Bitor: _LL19: _LL1A: goto _LL1C;case Cyc_Absyn_Bitxor: _LL1B: _LL1C: goto _LL1E;case Cyc_Absyn_Bitlshift: _LL1D: _LL1E: goto _LL20;case Cyc_Absyn_Bitlrshift: _LL1F: _LL20:
# 1344
({void*_tmp3FC=arg1;Cyc_Port_leq(_tmp3FC,Cyc_Port_arith_ct());});
({void*_tmp3FD=(void*)((struct Cyc_List_List*)_check_null(arg2s))->hd;Cyc_Port_leq(_tmp3FD,Cyc_Port_arith_ct());});
return Cyc_Port_arith_ct();case Cyc_Absyn_Not: _LL21: _LL22:
 goto _LL24;case Cyc_Absyn_Bitnot: _LL23: _LL24:
# 1349
({void*_tmp3FE=arg1;Cyc_Port_leq(_tmp3FE,Cyc_Port_arith_ct());});
return Cyc_Port_arith_ct();default: _LL25: _LL26:
({void*_tmp1AF=0U;({int(*_tmp400)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp1B1)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp1B1;});struct _fat_ptr _tmp3FF=({const char*_tmp1B0="Port: unknown primop";_tag_fat(_tmp1B0,sizeof(char),21U);});_tmp400(_tmp3FF,_tag_fat(_tmp1AF,sizeof(void*),0U));});});}_LL0:;}
# 1128
static struct _tuple12 Cyc_Port_gen_lhs(struct Cyc_Port_Cenv*env,struct Cyc_Absyn_Exp*e,int takeaddress){
# 1357
void*_stmttmp18=e->r;void*_tmp1B2=_stmttmp18;struct _fat_ptr*_tmp1B4;struct Cyc_Absyn_Exp*_tmp1B3;struct _fat_ptr*_tmp1B6;struct Cyc_Absyn_Exp*_tmp1B5;struct Cyc_Absyn_Exp*_tmp1B7;struct Cyc_Absyn_Exp*_tmp1B9;struct Cyc_Absyn_Exp*_tmp1B8;void*_tmp1BA;switch(*((int*)_tmp1B2)){case 1U: _LL1: _tmp1BA=(void*)((struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct*)_tmp1B2)->f1;_LL2: {void*b=_tmp1BA;
# 1359
struct _tuple0*x=Cyc_Absyn_binding2qvar(b);
(*x).f1=Cyc_Absyn_Loc_n;{
struct _tuple14 _stmttmp19=Cyc_Port_lookup_var(env,x);unsigned _tmp1BC;struct _tuple12 _tmp1BB;_LLE: _tmp1BB=_stmttmp19.f1;_tmp1BC=_stmttmp19.f2;_LLF: {struct _tuple12 p=_tmp1BB;unsigned loc=_tmp1BC;
Cyc_Port_register_localvar_usage(env,loc,e->loc,takeaddress);
return p;}}}case 23U: _LL3: _tmp1B8=((struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct*)_tmp1B2)->f1;_tmp1B9=((struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct*)_tmp1B2)->f2;_LL4: {struct Cyc_Absyn_Exp*e1=_tmp1B8;struct Cyc_Absyn_Exp*e2=_tmp1B9;
# 1365
void*v=Cyc_Port_var();
void*q=Cyc_Port_var();
void*t1=Cyc_Port_gen_exp(0,env,e1);
({void*_tmp401=Cyc_Port_gen_exp(0,env,e2);Cyc_Port_leq(_tmp401,Cyc_Port_arith_ct());});{
void*p=({void*_tmp405=v;void*_tmp404=q;void*_tmp403=Cyc_Port_fat_ct();void*_tmp402=Cyc_Port_var();Cyc_Port_ptr_ct(_tmp405,_tmp404,_tmp403,_tmp402,Cyc_Port_var());});
Cyc_Port_leq(t1,p);
return({struct _tuple12 _tmp309;_tmp309.f1=q,_tmp309.f2=v;_tmp309;});}}case 20U: _LL5: _tmp1B7=((struct Cyc_Absyn_Deref_e_Absyn_Raw_exp_struct*)_tmp1B2)->f1;_LL6: {struct Cyc_Absyn_Exp*e1=_tmp1B7;
# 1373
void*v=Cyc_Port_var();
void*q=Cyc_Port_var();
void*p=({void*_tmp409=v;void*_tmp408=q;void*_tmp407=Cyc_Port_var();void*_tmp406=Cyc_Port_var();Cyc_Port_ptr_ct(_tmp409,_tmp408,_tmp407,_tmp406,Cyc_Port_var());});
({void*_tmp40A=Cyc_Port_gen_exp(0,env,e1);Cyc_Port_leq(_tmp40A,p);});
return({struct _tuple12 _tmp30A;_tmp30A.f1=q,_tmp30A.f2=v;_tmp30A;});}case 21U: _LL7: _tmp1B5=((struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct*)_tmp1B2)->f1;_tmp1B6=((struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct*)_tmp1B2)->f2;_LL8: {struct Cyc_Absyn_Exp*e1=_tmp1B5;struct _fat_ptr*f=_tmp1B6;
# 1379
void*v=Cyc_Port_var();
void*q=Cyc_Port_var();
void*t1=Cyc_Port_gen_exp(takeaddress,env,e1);
({void*_tmp40C=t1;Cyc_Port_leq(_tmp40C,Cyc_Port_unknown_aggr_ct(({struct Cyc_Port_Cfield*_tmp1BD[1U];({struct Cyc_Port_Cfield*_tmp40B=({struct Cyc_Port_Cfield*_tmp1BE=_cycalloc(sizeof(*_tmp1BE));_tmp1BE->qual=q,_tmp1BE->f=f,_tmp1BE->type=v;_tmp1BE;});_tmp1BD[0]=_tmp40B;});Cyc_List_list(_tag_fat(_tmp1BD,sizeof(struct Cyc_Port_Cfield*),1U));})));});
return({struct _tuple12 _tmp30B;_tmp30B.f1=q,_tmp30B.f2=v;_tmp30B;});}case 22U: _LL9: _tmp1B3=((struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct*)_tmp1B2)->f1;_tmp1B4=((struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct*)_tmp1B2)->f2;_LLA: {struct Cyc_Absyn_Exp*e1=_tmp1B3;struct _fat_ptr*f=_tmp1B4;
# 1385
void*v=Cyc_Port_var();
void*q=Cyc_Port_var();
void*t1=Cyc_Port_gen_exp(0,env,e1);
({void*_tmp412=t1;Cyc_Port_leq(_tmp412,({void*_tmp411=Cyc_Port_unknown_aggr_ct(({struct Cyc_Port_Cfield*_tmp1BF[1U];({struct Cyc_Port_Cfield*_tmp40D=({struct Cyc_Port_Cfield*_tmp1C0=_cycalloc(sizeof(*_tmp1C0));_tmp1C0->qual=q,_tmp1C0->f=f,_tmp1C0->type=v;_tmp1C0;});_tmp1BF[0]=_tmp40D;});Cyc_List_list(_tag_fat(_tmp1BF,sizeof(struct Cyc_Port_Cfield*),1U));}));void*_tmp410=
Cyc_Port_var();
# 1388
void*_tmp40F=
Cyc_Port_var();
# 1388
void*_tmp40E=
Cyc_Port_var();
# 1388
Cyc_Port_ptr_ct(_tmp411,_tmp410,_tmp40F,_tmp40E,
Cyc_Port_var());}));});
return({struct _tuple12 _tmp30C;_tmp30C.f1=q,_tmp30C.f2=v;_tmp30C;});}default: _LLB: _LLC:
({void*_tmp1C1=0U;({int(*_tmp416)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp1C5)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp1C5;});struct _fat_ptr _tmp415=(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp1C4=({struct Cyc_String_pa_PrintArg_struct _tmp30D;_tmp30D.tag=0U,({struct _fat_ptr _tmp413=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_exp2string(e));_tmp30D.f1=_tmp413;});_tmp30D;});void*_tmp1C2[1U];_tmp1C2[0]=& _tmp1C4;({struct _fat_ptr _tmp414=({const char*_tmp1C3="gen_lhs: %s";_tag_fat(_tmp1C3,sizeof(char),12U);});Cyc_aprintf(_tmp414,_tag_fat(_tmp1C2,sizeof(void*),1U));});});_tmp416(_tmp415,_tag_fat(_tmp1C1,sizeof(void*),0U));});});}_LL0:;}
# 1128
static void*Cyc_Port_gen_exp_false(struct Cyc_Port_Cenv*env,struct Cyc_Absyn_Exp*e){
# 1397
return Cyc_Port_gen_exp(0,env,e);}
# 1128
static void*Cyc_Port_gen_exp(int takeaddress,struct Cyc_Port_Cenv*env,struct Cyc_Absyn_Exp*e){
# 1400
void*_stmttmp1A=e->r;void*_tmp1C6=_stmttmp1A;struct Cyc_Absyn_Stmt*_tmp1C7;struct Cyc_Absyn_Exp*_tmp1C9;struct Cyc_Absyn_Exp*_tmp1C8;struct Cyc_Absyn_Exp*_tmp1CB;void**_tmp1CA;struct _fat_ptr*_tmp1CD;struct Cyc_Absyn_Exp*_tmp1CC;struct _fat_ptr*_tmp1CF;struct Cyc_Absyn_Exp*_tmp1CE;struct Cyc_Absyn_Exp*_tmp1D0;struct Cyc_Absyn_Exp*_tmp1D2;struct Cyc_Absyn_Exp*_tmp1D1;struct Cyc_Absyn_Exp*_tmp1D3;struct Cyc_Absyn_Exp*_tmp1D5;void*_tmp1D4;struct Cyc_Absyn_Exp*_tmp1D6;struct Cyc_Absyn_Exp*_tmp1D7;struct Cyc_List_List*_tmp1D9;struct Cyc_Absyn_Exp*_tmp1D8;struct Cyc_Absyn_Exp*_tmp1DB;struct Cyc_Absyn_Exp*_tmp1DA;struct Cyc_Absyn_Exp*_tmp1DD;struct Cyc_Absyn_Exp*_tmp1DC;struct Cyc_Absyn_Exp*_tmp1DF;struct Cyc_Absyn_Exp*_tmp1DE;struct Cyc_Absyn_Exp*_tmp1E2;struct Cyc_Absyn_Exp*_tmp1E1;struct Cyc_Absyn_Exp*_tmp1E0;enum Cyc_Absyn_Incrementor _tmp1E4;struct Cyc_Absyn_Exp*_tmp1E3;struct Cyc_Absyn_Exp*_tmp1E7;struct Cyc_Core_Opt*_tmp1E6;struct Cyc_Absyn_Exp*_tmp1E5;struct Cyc_List_List*_tmp1E9;enum Cyc_Absyn_Primop _tmp1E8;void*_tmp1EA;switch(*((int*)_tmp1C6)){case 0U: switch(((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmp1C6)->f1).Wstring_c).tag){case 2U: _LL1: _LL2:
 goto _LL4;case 3U: _LL3: _LL4:
 goto _LL6;case 4U: _LL5: _LL6:
 goto _LL8;case 6U: _LL7: _LL8:
 goto _LLA;case 7U: _LL13: _LL14:
# 1410
 return Cyc_Port_arith_ct();case 5U: if((((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmp1C6)->f1).Int_c).val).f2 == 0){_LL15: _LL16:
 return Cyc_Port_zero_ct();}else{_LL17: _LL18:
 return Cyc_Port_arith_ct();}case 8U: _LL19: _LL1A:
# 1414
 return({void*_tmp41A=Cyc_Port_arith_ct();void*_tmp419=Cyc_Port_const_ct();void*_tmp418=Cyc_Port_fat_ct();void*_tmp417=Cyc_Port_heap_ct();Cyc_Port_ptr_ct(_tmp41A,_tmp419,_tmp418,_tmp417,Cyc_Port_zterm_ct());});case 9U: _LL1B: _LL1C:
# 1416
 return({void*_tmp41E=Cyc_Port_arith_ct();void*_tmp41D=Cyc_Port_const_ct();void*_tmp41C=Cyc_Port_fat_ct();void*_tmp41B=Cyc_Port_heap_ct();Cyc_Port_ptr_ct(_tmp41E,_tmp41D,_tmp41C,_tmp41B,Cyc_Port_zterm_ct());});default: _LL1D: _LL1E:
 return Cyc_Port_zero_ct();}case 17U: _LL9: _LLA:
# 1405
 goto _LLC;case 18U: _LLB: _LLC:
 goto _LLE;case 19U: _LLD: _LLE:
 goto _LL10;case 32U: _LLF: _LL10:
 goto _LL12;case 33U: _LL11: _LL12:
 goto _LL14;case 1U: _LL1F: _tmp1EA=(void*)((struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct*)_tmp1C6)->f1;_LL20: {void*b=_tmp1EA;
# 1419
struct _tuple0*x=Cyc_Absyn_binding2qvar(b);
(*x).f1=Cyc_Absyn_Loc_n;{
struct _tuple14 _stmttmp1B=Cyc_Port_lookup_var(env,x);unsigned _tmp1EC;struct _tuple12 _tmp1EB;_LL6A: _tmp1EB=_stmttmp1B.f1;_tmp1EC=_stmttmp1B.f2;_LL6B: {struct _tuple12 p=_tmp1EB;unsigned loc=_tmp1EC;
void*_tmp1ED;_LL6D: _tmp1ED=p.f2;_LL6E: {void*t=_tmp1ED;
if(Cyc_Port_isfn(env,x)){
t=Cyc_Port_instantiate(t);
return({void*_tmp422=t;void*_tmp421=Cyc_Port_var();void*_tmp420=Cyc_Port_var();void*_tmp41F=Cyc_Port_heap_ct();Cyc_Port_ptr_ct(_tmp422,_tmp421,_tmp420,_tmp41F,Cyc_Port_nozterm_ct());});}else{
# 1427
if(Cyc_Port_isarray(env,x)){
void*elt_type=Cyc_Port_var();
void*qual=Cyc_Port_var();
void*zt=Cyc_Port_var();
void*array_type=Cyc_Port_array_ct(elt_type,qual,zt);
Cyc_Port_unifies(t,array_type);{
void*ptr_type=({void*_tmp426=elt_type;void*_tmp425=qual;void*_tmp424=Cyc_Port_fat_ct();void*_tmp423=Cyc_Port_var();Cyc_Port_ptr_ct(_tmp426,_tmp425,_tmp424,_tmp423,zt);});
return ptr_type;}}else{
# 1436
Cyc_Port_register_localvar_usage(env,loc,e->loc,takeaddress);
return t;}}}}}}case 3U: _LL21: _tmp1E8=((struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct*)_tmp1C6)->f1;_tmp1E9=((struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct*)_tmp1C6)->f2;_LL22: {enum Cyc_Absyn_Primop p=_tmp1E8;struct Cyc_List_List*es=_tmp1E9;
# 1439
return({struct Cyc_Port_Cenv*_tmp42A=env;enum Cyc_Absyn_Primop _tmp429=p;Cyc_Port_gen_primop(_tmp42A,_tmp429,({struct Cyc_List_List*(*_tmp428)(void*(*f)(struct Cyc_Port_Cenv*,struct Cyc_Absyn_Exp*),struct Cyc_Port_Cenv*env,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp1EE)(void*(*f)(struct Cyc_Port_Cenv*,struct Cyc_Absyn_Exp*),struct Cyc_Port_Cenv*env,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(void*(*f)(struct Cyc_Port_Cenv*,struct Cyc_Absyn_Exp*),struct Cyc_Port_Cenv*env,struct Cyc_List_List*x))Cyc_List_map_c;_tmp1EE;});struct Cyc_Port_Cenv*_tmp427=env;_tmp428(Cyc_Port_gen_exp_false,_tmp427,es);}));});}case 4U: _LL23: _tmp1E5=((struct Cyc_Absyn_AssignOp_e_Absyn_Raw_exp_struct*)_tmp1C6)->f1;_tmp1E6=((struct Cyc_Absyn_AssignOp_e_Absyn_Raw_exp_struct*)_tmp1C6)->f2;_tmp1E7=((struct Cyc_Absyn_AssignOp_e_Absyn_Raw_exp_struct*)_tmp1C6)->f3;_LL24: {struct Cyc_Absyn_Exp*e1=_tmp1E5;struct Cyc_Core_Opt*popt=_tmp1E6;struct Cyc_Absyn_Exp*e2=_tmp1E7;
# 1441
struct _tuple12 _stmttmp1C=Cyc_Port_gen_lhs(env,e1,0);void*_tmp1F0;void*_tmp1EF;_LL70: _tmp1EF=_stmttmp1C.f1;_tmp1F0=_stmttmp1C.f2;_LL71: {void*q=_tmp1EF;void*t1=_tmp1F0;
({void*_tmp42B=q;Cyc_Port_unifies(_tmp42B,Cyc_Port_notconst_ct());});{
void*t2=Cyc_Port_gen_exp(0,env,e2);
if((unsigned)popt){
struct Cyc_List_List x2=({struct Cyc_List_List _tmp30F;_tmp30F.hd=t2,_tmp30F.tl=0;_tmp30F;});
struct Cyc_List_List x1=({struct Cyc_List_List _tmp30E;_tmp30E.hd=t1,_tmp30E.tl=& x2;_tmp30E;});
void*t3=Cyc_Port_gen_primop(env,(enum Cyc_Absyn_Primop)popt->v,& x1);
Cyc_Port_leq(t3,t1);
return t1;}else{
# 1451
Cyc_Port_leq(t2,t1);
return t1;}}}}case 5U: _LL25: _tmp1E3=((struct Cyc_Absyn_Increment_e_Absyn_Raw_exp_struct*)_tmp1C6)->f1;_tmp1E4=((struct Cyc_Absyn_Increment_e_Absyn_Raw_exp_struct*)_tmp1C6)->f2;_LL26: {struct Cyc_Absyn_Exp*e1=_tmp1E3;enum Cyc_Absyn_Incrementor inc=_tmp1E4;
# 1455
struct _tuple12 _stmttmp1D=Cyc_Port_gen_lhs(env,e1,0);void*_tmp1F2;void*_tmp1F1;_LL73: _tmp1F1=_stmttmp1D.f1;_tmp1F2=_stmttmp1D.f2;_LL74: {void*q=_tmp1F1;void*t=_tmp1F2;
({void*_tmp42C=q;Cyc_Port_unifies(_tmp42C,Cyc_Port_notconst_ct());});
# 1458
({void*_tmp431=t;Cyc_Port_leq(_tmp431,({void*_tmp430=Cyc_Port_var();void*_tmp42F=Cyc_Port_var();void*_tmp42E=Cyc_Port_fat_ct();void*_tmp42D=Cyc_Port_var();Cyc_Port_ptr_ct(_tmp430,_tmp42F,_tmp42E,_tmp42D,Cyc_Port_var());}));});
({void*_tmp432=t;Cyc_Port_leq(_tmp432,Cyc_Port_arith_ct());});
return t;}}case 6U: _LL27: _tmp1E0=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmp1C6)->f1;_tmp1E1=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmp1C6)->f2;_tmp1E2=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmp1C6)->f3;_LL28: {struct Cyc_Absyn_Exp*e1=_tmp1E0;struct Cyc_Absyn_Exp*e2=_tmp1E1;struct Cyc_Absyn_Exp*e3=_tmp1E2;
# 1462
void*v=Cyc_Port_var();
({void*_tmp433=Cyc_Port_gen_exp(0,env,e1);Cyc_Port_leq(_tmp433,Cyc_Port_arith_ct());});
({void*_tmp434=Cyc_Port_gen_exp(0,env,e2);Cyc_Port_leq(_tmp434,v);});
({void*_tmp435=Cyc_Port_gen_exp(0,env,e3);Cyc_Port_leq(_tmp435,v);});
return v;}case 7U: _LL29: _tmp1DE=((struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct*)_tmp1C6)->f1;_tmp1DF=((struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct*)_tmp1C6)->f2;_LL2A: {struct Cyc_Absyn_Exp*e1=_tmp1DE;struct Cyc_Absyn_Exp*e2=_tmp1DF;
_tmp1DC=e1;_tmp1DD=e2;goto _LL2C;}case 8U: _LL2B: _tmp1DC=((struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct*)_tmp1C6)->f1;_tmp1DD=((struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct*)_tmp1C6)->f2;_LL2C: {struct Cyc_Absyn_Exp*e1=_tmp1DC;struct Cyc_Absyn_Exp*e2=_tmp1DD;
# 1469
({void*_tmp436=Cyc_Port_gen_exp(0,env,e1);Cyc_Port_leq(_tmp436,Cyc_Port_arith_ct());});
({void*_tmp437=Cyc_Port_gen_exp(0,env,e2);Cyc_Port_leq(_tmp437,Cyc_Port_arith_ct());});
return Cyc_Port_arith_ct();}case 9U: _LL2D: _tmp1DA=((struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct*)_tmp1C6)->f1;_tmp1DB=((struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct*)_tmp1C6)->f2;_LL2E: {struct Cyc_Absyn_Exp*e1=_tmp1DA;struct Cyc_Absyn_Exp*e2=_tmp1DB;
# 1473
Cyc_Port_gen_exp(0,env,e1);
return Cyc_Port_gen_exp(0,env,e2);}case 10U: _LL2F: _tmp1D8=((struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct*)_tmp1C6)->f1;_tmp1D9=((struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct*)_tmp1C6)->f2;_LL30: {struct Cyc_Absyn_Exp*e1=_tmp1D8;struct Cyc_List_List*es=_tmp1D9;
# 1476
void*v=Cyc_Port_var();
void*t1=Cyc_Port_gen_exp(0,env,e1);
struct Cyc_List_List*ts=({struct Cyc_List_List*(*_tmp439)(void*(*f)(struct Cyc_Port_Cenv*,struct Cyc_Absyn_Exp*),struct Cyc_Port_Cenv*env,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp1F3)(void*(*f)(struct Cyc_Port_Cenv*,struct Cyc_Absyn_Exp*),struct Cyc_Port_Cenv*env,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(void*(*f)(struct Cyc_Port_Cenv*,struct Cyc_Absyn_Exp*),struct Cyc_Port_Cenv*env,struct Cyc_List_List*x))Cyc_List_map_c;_tmp1F3;});struct Cyc_Port_Cenv*_tmp438=env;_tmp439(Cyc_Port_gen_exp_false,_tmp438,es);});
struct Cyc_List_List*vs=Cyc_List_map(Cyc_Port_new_var,ts);
({void*_tmp43E=t1;Cyc_Port_unifies(_tmp43E,({void*_tmp43D=Cyc_Port_fn_ct(v,vs);void*_tmp43C=Cyc_Port_var();void*_tmp43B=Cyc_Port_var();void*_tmp43A=Cyc_Port_var();Cyc_Port_ptr_ct(_tmp43D,_tmp43C,_tmp43B,_tmp43A,Cyc_Port_nozterm_ct());}));});
for(0;ts != 0;(ts=ts->tl,vs=vs->tl)){
Cyc_Port_leq((void*)ts->hd,(void*)((struct Cyc_List_List*)_check_null(vs))->hd);}
# 1484
return v;}case 34U: _LL31: _LL32:
({void*_tmp1F4=0U;({int(*_tmp440)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp1F6)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp1F6;});struct _fat_ptr _tmp43F=({const char*_tmp1F5="throw";_tag_fat(_tmp1F5,sizeof(char),6U);});_tmp440(_tmp43F,_tag_fat(_tmp1F4,sizeof(void*),0U));});});case 11U: _LL33: _LL34:
({void*_tmp1F7=0U;({int(*_tmp442)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp1F9)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp1F9;});struct _fat_ptr _tmp441=({const char*_tmp1F8="throw";_tag_fat(_tmp1F8,sizeof(char),6U);});_tmp442(_tmp441,_tag_fat(_tmp1F7,sizeof(void*),0U));});});case 42U: _LL35: _tmp1D7=((struct Cyc_Absyn_Extension_e_Absyn_Raw_exp_struct*)_tmp1C6)->f1;_LL36: {struct Cyc_Absyn_Exp*e1=_tmp1D7;
return Cyc_Port_gen_exp(0,env,e1);}case 12U: _LL37: _tmp1D6=((struct Cyc_Absyn_NoInstantiate_e_Absyn_Raw_exp_struct*)_tmp1C6)->f1;_LL38: {struct Cyc_Absyn_Exp*e1=_tmp1D6;
return Cyc_Port_gen_exp(0,env,e1);}case 13U: _LL39: _LL3A:
({void*_tmp1FA=0U;({int(*_tmp444)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp1FC)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp1FC;});struct _fat_ptr _tmp443=({const char*_tmp1FB="instantiate";_tag_fat(_tmp1FB,sizeof(char),12U);});_tmp444(_tmp443,_tag_fat(_tmp1FA,sizeof(void*),0U));});});case 14U: _LL3B: _tmp1D4=(void*)((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp1C6)->f1;_tmp1D5=((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp1C6)->f2;_LL3C: {void*t=_tmp1D4;struct Cyc_Absyn_Exp*e1=_tmp1D5;
# 1491
void*t1=Cyc_Port_type_to_ctype(env,t);
void*t2=Cyc_Port_gen_exp(0,env,e1);
Cyc_Port_leq(t1,t2);
return t2;}case 15U: _LL3D: _tmp1D3=((struct Cyc_Absyn_Address_e_Absyn_Raw_exp_struct*)_tmp1C6)->f1;_LL3E: {struct Cyc_Absyn_Exp*e1=_tmp1D3;
# 1496
struct _tuple12 _stmttmp1E=Cyc_Port_gen_lhs(env,e1,1);void*_tmp1FE;void*_tmp1FD;_LL76: _tmp1FD=_stmttmp1E.f1;_tmp1FE=_stmttmp1E.f2;_LL77: {void*q=_tmp1FD;void*t=_tmp1FE;
return({void*_tmp448=t;void*_tmp447=q;void*_tmp446=Cyc_Port_var();void*_tmp445=Cyc_Port_var();Cyc_Port_ptr_ct(_tmp448,_tmp447,_tmp446,_tmp445,Cyc_Port_var());});}}case 23U: _LL3F: _tmp1D1=((struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct*)_tmp1C6)->f1;_tmp1D2=((struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct*)_tmp1C6)->f2;_LL40: {struct Cyc_Absyn_Exp*e1=_tmp1D1;struct Cyc_Absyn_Exp*e2=_tmp1D2;
# 1499
void*v=Cyc_Port_var();
({void*_tmp449=Cyc_Port_gen_exp(0,env,e2);Cyc_Port_leq(_tmp449,Cyc_Port_arith_ct());});{
void*t=Cyc_Port_gen_exp(0,env,e1);
({void*_tmp44E=t;Cyc_Port_leq(_tmp44E,({void*_tmp44D=v;void*_tmp44C=Cyc_Port_var();void*_tmp44B=Cyc_Port_fat_ct();void*_tmp44A=Cyc_Port_var();Cyc_Port_ptr_ct(_tmp44D,_tmp44C,_tmp44B,_tmp44A,Cyc_Port_var());}));});
return v;}}case 20U: _LL41: _tmp1D0=((struct Cyc_Absyn_Deref_e_Absyn_Raw_exp_struct*)_tmp1C6)->f1;_LL42: {struct Cyc_Absyn_Exp*e1=_tmp1D0;
# 1505
void*v=Cyc_Port_var();
({void*_tmp453=Cyc_Port_gen_exp(0,env,e1);Cyc_Port_leq(_tmp453,({void*_tmp452=v;void*_tmp451=Cyc_Port_var();void*_tmp450=Cyc_Port_var();void*_tmp44F=Cyc_Port_var();Cyc_Port_ptr_ct(_tmp452,_tmp451,_tmp450,_tmp44F,Cyc_Port_var());}));});
return v;}case 21U: _LL43: _tmp1CE=((struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct*)_tmp1C6)->f1;_tmp1CF=((struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct*)_tmp1C6)->f2;_LL44: {struct Cyc_Absyn_Exp*e1=_tmp1CE;struct _fat_ptr*f=_tmp1CF;
# 1509
void*v=Cyc_Port_var();
void*t=Cyc_Port_gen_exp(takeaddress,env,e1);
({void*_tmp456=t;Cyc_Port_leq(_tmp456,Cyc_Port_unknown_aggr_ct(({struct Cyc_Port_Cfield*_tmp1FF[1U];({struct Cyc_Port_Cfield*_tmp455=({struct Cyc_Port_Cfield*_tmp200=_cycalloc(sizeof(*_tmp200));({void*_tmp454=Cyc_Port_var();_tmp200->qual=_tmp454;}),_tmp200->f=f,_tmp200->type=v;_tmp200;});_tmp1FF[0]=_tmp455;});Cyc_List_list(_tag_fat(_tmp1FF,sizeof(struct Cyc_Port_Cfield*),1U));})));});
return v;}case 22U: _LL45: _tmp1CC=((struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct*)_tmp1C6)->f1;_tmp1CD=((struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct*)_tmp1C6)->f2;_LL46: {struct Cyc_Absyn_Exp*e1=_tmp1CC;struct _fat_ptr*f=_tmp1CD;
# 1514
void*v=Cyc_Port_var();
void*t=Cyc_Port_gen_exp(0,env,e1);
({void*_tmp45D=Cyc_Port_gen_exp(0,env,e1);Cyc_Port_leq(_tmp45D,({void*_tmp45C=Cyc_Port_unknown_aggr_ct(({struct Cyc_Port_Cfield*_tmp201[1U];({struct Cyc_Port_Cfield*_tmp458=({struct Cyc_Port_Cfield*_tmp202=_cycalloc(sizeof(*_tmp202));({void*_tmp457=Cyc_Port_var();_tmp202->qual=_tmp457;}),_tmp202->f=f,_tmp202->type=v;_tmp202;});_tmp201[0]=_tmp458;});Cyc_List_list(_tag_fat(_tmp201,sizeof(struct Cyc_Port_Cfield*),1U));}));void*_tmp45B=
Cyc_Port_var();
# 1516
void*_tmp45A=
Cyc_Port_var();
# 1516
void*_tmp459=
Cyc_Port_var();
# 1516
Cyc_Port_ptr_ct(_tmp45C,_tmp45B,_tmp45A,_tmp459,
Cyc_Port_var());}));});
return v;}case 35U: _LL47: _tmp1CA=(((struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct*)_tmp1C6)->f1).elt_type;_tmp1CB=(((struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct*)_tmp1C6)->f1).num_elts;_LL48: {void**topt=_tmp1CA;struct Cyc_Absyn_Exp*e1=_tmp1CB;
# 1520
({void*_tmp45E=Cyc_Port_gen_exp(0,env,e1);Cyc_Port_leq(_tmp45E,Cyc_Port_arith_ct());});{
void*t=(unsigned)topt?Cyc_Port_type_to_ctype(env,*topt): Cyc_Port_var();
return({void*_tmp462=t;void*_tmp461=Cyc_Port_var();void*_tmp460=Cyc_Port_fat_ct();void*_tmp45F=Cyc_Port_heap_ct();Cyc_Port_ptr_ct(_tmp462,_tmp461,_tmp460,_tmp45F,Cyc_Port_var());});}}case 2U: _LL49: _LL4A:
({void*_tmp203=0U;({int(*_tmp464)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp205)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp205;});struct _fat_ptr _tmp463=({const char*_tmp204="Pragma_e";_tag_fat(_tmp204,sizeof(char),9U);});_tmp464(_tmp463,_tag_fat(_tmp203,sizeof(void*),0U));});});case 36U: _LL4B: _tmp1C8=((struct Cyc_Absyn_Swap_e_Absyn_Raw_exp_struct*)_tmp1C6)->f1;_tmp1C9=((struct Cyc_Absyn_Swap_e_Absyn_Raw_exp_struct*)_tmp1C6)->f2;_LL4C: {struct Cyc_Absyn_Exp*e1=_tmp1C8;struct Cyc_Absyn_Exp*e2=_tmp1C9;
({void*_tmp206=0U;({int(*_tmp466)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp208)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp208;});struct _fat_ptr _tmp465=({const char*_tmp207="Swap_e";_tag_fat(_tmp207,sizeof(char),7U);});_tmp466(_tmp465,_tag_fat(_tmp206,sizeof(void*),0U));});});}case 16U: _LL4D: _LL4E:
({void*_tmp209=0U;({int(*_tmp468)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp20B)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp20B;});struct _fat_ptr _tmp467=({const char*_tmp20A="New_e";_tag_fat(_tmp20A,sizeof(char),6U);});_tmp468(_tmp467,_tag_fat(_tmp209,sizeof(void*),0U));});});case 24U: _LL4F: _LL50:
({void*_tmp20C=0U;({int(*_tmp46A)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp20E)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp20E;});struct _fat_ptr _tmp469=({const char*_tmp20D="Tuple_e";_tag_fat(_tmp20D,sizeof(char),8U);});_tmp46A(_tmp469,_tag_fat(_tmp20C,sizeof(void*),0U));});});case 25U: _LL51: _LL52:
({void*_tmp20F=0U;({int(*_tmp46C)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp211)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp211;});struct _fat_ptr _tmp46B=({const char*_tmp210="CompoundLit_e";_tag_fat(_tmp210,sizeof(char),14U);});_tmp46C(_tmp46B,_tag_fat(_tmp20F,sizeof(void*),0U));});});case 26U: _LL53: _LL54:
({void*_tmp212=0U;({int(*_tmp46E)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp214)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp214;});struct _fat_ptr _tmp46D=({const char*_tmp213="Array_e";_tag_fat(_tmp213,sizeof(char),8U);});_tmp46E(_tmp46D,_tag_fat(_tmp212,sizeof(void*),0U));});});case 27U: _LL55: _LL56:
({void*_tmp215=0U;({int(*_tmp470)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp217)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp217;});struct _fat_ptr _tmp46F=({const char*_tmp216="Comprehension_e";_tag_fat(_tmp216,sizeof(char),16U);});_tmp470(_tmp46F,_tag_fat(_tmp215,sizeof(void*),0U));});});case 28U: _LL57: _LL58:
({void*_tmp218=0U;({int(*_tmp472)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp21A)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp21A;});struct _fat_ptr _tmp471=({const char*_tmp219="ComprehensionNoinit_e";_tag_fat(_tmp219,sizeof(char),22U);});_tmp472(_tmp471,_tag_fat(_tmp218,sizeof(void*),0U));});});case 29U: _LL59: _LL5A:
({void*_tmp21B=0U;({int(*_tmp474)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp21D)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp21D;});struct _fat_ptr _tmp473=({const char*_tmp21C="Aggregate_e";_tag_fat(_tmp21C,sizeof(char),12U);});_tmp474(_tmp473,_tag_fat(_tmp21B,sizeof(void*),0U));});});case 30U: _LL5B: _LL5C:
({void*_tmp21E=0U;({int(*_tmp476)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp220)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp220;});struct _fat_ptr _tmp475=({const char*_tmp21F="AnonStruct_e";_tag_fat(_tmp21F,sizeof(char),13U);});_tmp476(_tmp475,_tag_fat(_tmp21E,sizeof(void*),0U));});});case 31U: _LL5D: _LL5E:
({void*_tmp221=0U;({int(*_tmp478)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp223)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp223;});struct _fat_ptr _tmp477=({const char*_tmp222="Datatype_e";_tag_fat(_tmp222,sizeof(char),11U);});_tmp478(_tmp477,_tag_fat(_tmp221,sizeof(void*),0U));});});case 37U: _LL5F: _LL60:
({void*_tmp224=0U;({int(*_tmp47A)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp226)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp226;});struct _fat_ptr _tmp479=({const char*_tmp225="UnresolvedMem_e";_tag_fat(_tmp225,sizeof(char),16U);});_tmp47A(_tmp479,_tag_fat(_tmp224,sizeof(void*),0U));});});case 38U: _LL61: _tmp1C7=((struct Cyc_Absyn_StmtExp_e_Absyn_Raw_exp_struct*)_tmp1C6)->f1;_LL62: {struct Cyc_Absyn_Stmt*s=_tmp1C7;
({void*_tmp227=0U;({int(*_tmp47C)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp229)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp229;});struct _fat_ptr _tmp47B=({const char*_tmp228="StmtExp_e";_tag_fat(_tmp228,sizeof(char),10U);});_tmp47C(_tmp47B,_tag_fat(_tmp227,sizeof(void*),0U));});});}case 40U: _LL63: _LL64:
({void*_tmp22A=0U;({int(*_tmp47E)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp22C)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp22C;});struct _fat_ptr _tmp47D=({const char*_tmp22B="Valueof_e";_tag_fat(_tmp22B,sizeof(char),10U);});_tmp47E(_tmp47D,_tag_fat(_tmp22A,sizeof(void*),0U));});});case 41U: _LL65: _LL66:
({void*_tmp22D=0U;({int(*_tmp480)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp22F)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp22F;});struct _fat_ptr _tmp47F=({const char*_tmp22E="Asm_e";_tag_fat(_tmp22E,sizeof(char),6U);});_tmp480(_tmp47F,_tag_fat(_tmp22D,sizeof(void*),0U));});});default: _LL67: _LL68:
({void*_tmp230=0U;({int(*_tmp482)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp232)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp232;});struct _fat_ptr _tmp481=({const char*_tmp231="Tagcheck_e";_tag_fat(_tmp231,sizeof(char),11U);});_tmp482(_tmp481,_tag_fat(_tmp230,sizeof(void*),0U));});});}_LL0:;}
# 1128
static void Cyc_Port_gen_stmt(struct Cyc_Port_Cenv*env,struct Cyc_Absyn_Stmt*s){
# 1544
void*_stmttmp1F=s->r;void*_tmp233=_stmttmp1F;struct Cyc_Absyn_Exp*_tmp235;struct Cyc_Absyn_Stmt*_tmp234;struct Cyc_Absyn_Stmt*_tmp236;struct Cyc_Absyn_Stmt*_tmp238;struct Cyc_Absyn_Decl*_tmp237;struct Cyc_List_List*_tmp23A;struct Cyc_Absyn_Exp*_tmp239;struct Cyc_Absyn_Stmt*_tmp23E;struct Cyc_Absyn_Exp*_tmp23D;struct Cyc_Absyn_Exp*_tmp23C;struct Cyc_Absyn_Exp*_tmp23B;struct Cyc_Absyn_Stmt*_tmp240;struct Cyc_Absyn_Exp*_tmp23F;struct Cyc_Absyn_Stmt*_tmp243;struct Cyc_Absyn_Stmt*_tmp242;struct Cyc_Absyn_Exp*_tmp241;struct Cyc_Absyn_Exp*_tmp244;struct Cyc_Absyn_Stmt*_tmp246;struct Cyc_Absyn_Stmt*_tmp245;struct Cyc_Absyn_Exp*_tmp247;switch(*((int*)_tmp233)){case 0U: _LL1: _LL2:
 return;case 1U: _LL3: _tmp247=((struct Cyc_Absyn_Exp_s_Absyn_Raw_stmt_struct*)_tmp233)->f1;_LL4: {struct Cyc_Absyn_Exp*e=_tmp247;
Cyc_Port_gen_exp(0,env,e);return;}case 2U: _LL5: _tmp245=((struct Cyc_Absyn_Seq_s_Absyn_Raw_stmt_struct*)_tmp233)->f1;_tmp246=((struct Cyc_Absyn_Seq_s_Absyn_Raw_stmt_struct*)_tmp233)->f2;_LL6: {struct Cyc_Absyn_Stmt*s1=_tmp245;struct Cyc_Absyn_Stmt*s2=_tmp246;
Cyc_Port_gen_stmt(env,s1);Cyc_Port_gen_stmt(env,s2);return;}case 3U: _LL7: _tmp244=((struct Cyc_Absyn_Return_s_Absyn_Raw_stmt_struct*)_tmp233)->f1;_LL8: {struct Cyc_Absyn_Exp*eopt=_tmp244;
# 1549
void*v=Cyc_Port_lookup_return_type(env);
void*t=(unsigned)eopt?Cyc_Port_gen_exp(0,env,eopt): Cyc_Port_void_ct();
Cyc_Port_leq(t,v);
return;}case 4U: _LL9: _tmp241=((struct Cyc_Absyn_IfThenElse_s_Absyn_Raw_stmt_struct*)_tmp233)->f1;_tmp242=((struct Cyc_Absyn_IfThenElse_s_Absyn_Raw_stmt_struct*)_tmp233)->f2;_tmp243=((struct Cyc_Absyn_IfThenElse_s_Absyn_Raw_stmt_struct*)_tmp233)->f3;_LLA: {struct Cyc_Absyn_Exp*e=_tmp241;struct Cyc_Absyn_Stmt*s1=_tmp242;struct Cyc_Absyn_Stmt*s2=_tmp243;
# 1554
({void*_tmp483=Cyc_Port_gen_exp(0,env,e);Cyc_Port_leq(_tmp483,Cyc_Port_arith_ct());});
Cyc_Port_gen_stmt(env,s1);
Cyc_Port_gen_stmt(env,s2);
return;}case 5U: _LLB: _tmp23F=(((struct Cyc_Absyn_While_s_Absyn_Raw_stmt_struct*)_tmp233)->f1).f1;_tmp240=((struct Cyc_Absyn_While_s_Absyn_Raw_stmt_struct*)_tmp233)->f2;_LLC: {struct Cyc_Absyn_Exp*e=_tmp23F;struct Cyc_Absyn_Stmt*s=_tmp240;
# 1559
({void*_tmp484=Cyc_Port_gen_exp(0,env,e);Cyc_Port_leq(_tmp484,Cyc_Port_arith_ct());});
Cyc_Port_gen_stmt(env,s);
return;}case 6U: _LLD: _LLE:
 goto _LL10;case 7U: _LLF: _LL10:
 goto _LL12;case 8U: _LL11: _LL12:
 return;case 9U: _LL13: _tmp23B=((struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct*)_tmp233)->f1;_tmp23C=(((struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct*)_tmp233)->f2).f1;_tmp23D=(((struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct*)_tmp233)->f3).f1;_tmp23E=((struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct*)_tmp233)->f4;_LL14: {struct Cyc_Absyn_Exp*e1=_tmp23B;struct Cyc_Absyn_Exp*e2=_tmp23C;struct Cyc_Absyn_Exp*e3=_tmp23D;struct Cyc_Absyn_Stmt*s=_tmp23E;
# 1566
Cyc_Port_gen_exp(0,env,e1);
({void*_tmp485=Cyc_Port_gen_exp(0,env,e2);Cyc_Port_leq(_tmp485,Cyc_Port_arith_ct());});
Cyc_Port_gen_exp(0,env,e3);
Cyc_Port_gen_stmt(env,s);
return;}case 10U: _LL15: _tmp239=((struct Cyc_Absyn_Switch_s_Absyn_Raw_stmt_struct*)_tmp233)->f1;_tmp23A=((struct Cyc_Absyn_Switch_s_Absyn_Raw_stmt_struct*)_tmp233)->f2;_LL16: {struct Cyc_Absyn_Exp*e=_tmp239;struct Cyc_List_List*scs=_tmp23A;
# 1572
({void*_tmp486=Cyc_Port_gen_exp(0,env,e);Cyc_Port_leq(_tmp486,Cyc_Port_arith_ct());});
for(0;(unsigned)scs;scs=scs->tl){
# 1575
Cyc_Port_gen_stmt(env,((struct Cyc_Absyn_Switch_clause*)scs->hd)->body);}
# 1577
return;}case 11U: _LL17: _LL18:
({void*_tmp248=0U;({int(*_tmp488)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp24A)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp24A;});struct _fat_ptr _tmp487=({const char*_tmp249="fallthru";_tag_fat(_tmp249,sizeof(char),9U);});_tmp488(_tmp487,_tag_fat(_tmp248,sizeof(void*),0U));});});case 12U: _LL19: _tmp237=((struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct*)_tmp233)->f1;_tmp238=((struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct*)_tmp233)->f2;_LL1A: {struct Cyc_Absyn_Decl*d=_tmp237;struct Cyc_Absyn_Stmt*s=_tmp238;
# 1580
env=Cyc_Port_gen_localdecl(env,d,0);Cyc_Port_gen_stmt(env,s);return;}case 13U: _LL1B: _tmp236=((struct Cyc_Absyn_Label_s_Absyn_Raw_stmt_struct*)_tmp233)->f2;_LL1C: {struct Cyc_Absyn_Stmt*s=_tmp236;
Cyc_Port_gen_stmt(env,s);return;}case 14U: _LL1D: _tmp234=((struct Cyc_Absyn_Do_s_Absyn_Raw_stmt_struct*)_tmp233)->f1;_tmp235=(((struct Cyc_Absyn_Do_s_Absyn_Raw_stmt_struct*)_tmp233)->f2).f1;_LL1E: {struct Cyc_Absyn_Stmt*s=_tmp234;struct Cyc_Absyn_Exp*e=_tmp235;
# 1583
Cyc_Port_gen_stmt(env,s);
({void*_tmp489=Cyc_Port_gen_exp(0,env,e);Cyc_Port_leq(_tmp489,Cyc_Port_arith_ct());});
return;}default: _LL1F: _LL20:
({void*_tmp24B=0U;({int(*_tmp48B)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp24D)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp24D;});struct _fat_ptr _tmp48A=({const char*_tmp24C="try/catch";_tag_fat(_tmp24C,sizeof(char),10U);});_tmp48B(_tmp48A,_tag_fat(_tmp24B,sizeof(void*),0U));});});}_LL0:;}struct _tuple18{struct Cyc_List_List*f1;struct Cyc_Absyn_Exp*f2;};
# 1128
static void*Cyc_Port_gen_initializer(struct Cyc_Port_Cenv*env,void*t,struct Cyc_Absyn_Exp*e){
# 1592
void*_stmttmp20=e->r;void*_tmp24E=_stmttmp20;struct Cyc_List_List*_tmp24F;struct Cyc_List_List*_tmp250;struct Cyc_List_List*_tmp251;switch(*((int*)_tmp24E)){case 37U: _LL1: _tmp251=((struct Cyc_Absyn_UnresolvedMem_e_Absyn_Raw_exp_struct*)_tmp24E)->f2;_LL2: {struct Cyc_List_List*dles=_tmp251;
_tmp250=dles;goto _LL4;}case 26U: _LL3: _tmp250=((struct Cyc_Absyn_Array_e_Absyn_Raw_exp_struct*)_tmp24E)->f1;_LL4: {struct Cyc_List_List*dles=_tmp250;
_tmp24F=dles;goto _LL6;}case 25U: _LL5: _tmp24F=((struct Cyc_Absyn_CompoundLit_e_Absyn_Raw_exp_struct*)_tmp24E)->f2;_LL6: {struct Cyc_List_List*dles=_tmp24F;
# 1596
LOOP: {
void*_tmp252=t;struct Cyc_Absyn_Aggrdecl*_tmp253;struct _tuple0*_tmp254;unsigned _tmp257;void*_tmp256;void*_tmp255;struct _tuple0*_tmp258;switch(*((int*)_tmp252)){case 8U: _LLE: _tmp258=((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmp252)->f1;_LLF: {struct _tuple0*n=_tmp258;
# 1599
(*n).f1=Cyc_Absyn_Loc_n;
t=Cyc_Port_lookup_typedef(env,n);goto LOOP;}case 4U: _LL10: _tmp255=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp252)->f1).elt_type;_tmp256=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp252)->f1).zero_term;_tmp257=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp252)->f1).zt_loc;_LL11: {void*et=_tmp255;void*zt=_tmp256;unsigned ztl=_tmp257;
# 1602
void*v=Cyc_Port_var();
void*q=Cyc_Port_var();
void*z=Cyc_Port_var();
void*t=Cyc_Port_type_to_ctype(env,et);
for(0;dles != 0;dles=dles->tl){
struct _tuple18 _stmttmp21=*((struct _tuple18*)dles->hd);struct Cyc_Absyn_Exp*_tmp25A;struct Cyc_List_List*_tmp259;_LL19: _tmp259=_stmttmp21.f1;_tmp25A=_stmttmp21.f2;_LL1A: {struct Cyc_List_List*d=_tmp259;struct Cyc_Absyn_Exp*e=_tmp25A;
if((unsigned)d)({void*_tmp25B=0U;({int(*_tmp48D)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp25D)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp25D;});struct _fat_ptr _tmp48C=({const char*_tmp25C="designators in initializers";_tag_fat(_tmp25C,sizeof(char),28U);});_tmp48D(_tmp48C,_tag_fat(_tmp25B,sizeof(void*),0U));});});{void*te=
Cyc_Port_gen_initializer(env,et,e);
Cyc_Port_leq(te,v);}}}
# 1612
return Cyc_Port_array_ct(v,q,z);}case 0U: if(((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp252)->f1)->tag == 20U){if(((((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp252)->f1)->f1).UnknownAggr).tag == 1){if((((((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp252)->f1)->f1).UnknownAggr).val).f1 == Cyc_Absyn_StructA){_LL12: _tmp254=(((((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp252)->f1)->f1).UnknownAggr).val).f2;_LL13: {struct _tuple0*n=_tmp254;
# 1614
(*n).f1=Cyc_Absyn_Loc_n;{
struct _tuple11 _stmttmp22=*Cyc_Port_lookup_struct_decl(env,n);struct Cyc_Absyn_Aggrdecl*_tmp25E;_LL1C: _tmp25E=_stmttmp22.f1;_LL1D: {struct Cyc_Absyn_Aggrdecl*ad=_tmp25E;
if(ad == 0)({void*_tmp25F=0U;({int(*_tmp48F)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp261)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp261;});struct _fat_ptr _tmp48E=({const char*_tmp260="struct is not yet defined";_tag_fat(_tmp260,sizeof(char),26U);});_tmp48F(_tmp48E,_tag_fat(_tmp25F,sizeof(void*),0U));});});_tmp253=ad;goto _LL15;}}}}else{goto _LL16;}}else{_LL14: _tmp253=*((((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp252)->f1)->f1).KnownAggr).val;_LL15: {struct Cyc_Absyn_Aggrdecl*ad=_tmp253;
# 1619
struct Cyc_List_List*fields=((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->fields;
for(0;dles != 0;dles=dles->tl){
struct _tuple18 _stmttmp23=*((struct _tuple18*)dles->hd);struct Cyc_Absyn_Exp*_tmp263;struct Cyc_List_List*_tmp262;_LL1F: _tmp262=_stmttmp23.f1;_tmp263=_stmttmp23.f2;_LL20: {struct Cyc_List_List*d=_tmp262;struct Cyc_Absyn_Exp*e=_tmp263;
if((unsigned)d)({void*_tmp264=0U;({int(*_tmp491)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp266)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp266;});struct _fat_ptr _tmp490=({const char*_tmp265="designators in initializers";_tag_fat(_tmp265,sizeof(char),28U);});_tmp491(_tmp490,_tag_fat(_tmp264,sizeof(void*),0U));});});{struct Cyc_Absyn_Aggrfield*f=(struct Cyc_Absyn_Aggrfield*)((struct Cyc_List_List*)_check_null(fields))->hd;
# 1624
fields=fields->tl;{
void*te=Cyc_Port_gen_initializer(env,f->type,e);;}}}}
# 1627
return Cyc_Port_type_to_ctype(env,t);}}}else{goto _LL16;}default: _LL16: _LL17:
({void*_tmp267=0U;({int(*_tmp493)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp269)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp269;});struct _fat_ptr _tmp492=({const char*_tmp268="bad type for aggregate initializer";_tag_fat(_tmp268,sizeof(char),35U);});_tmp493(_tmp492,_tag_fat(_tmp267,sizeof(void*),0U));});});}_LLD:;}}case 0U: switch(((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmp24E)->f1).Wstring_c).tag){case 8U: _LL7: _LL8:
# 1630
 goto _LLA;case 9U: _LL9: _LLA:
# 1632
 LOOP2: {
void*_tmp26A=t;struct _tuple0*_tmp26B;switch(*((int*)_tmp26A)){case 8U: _LL22: _tmp26B=((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmp26A)->f1;_LL23: {struct _tuple0*n=_tmp26B;
# 1635
(*n).f1=Cyc_Absyn_Loc_n;
t=Cyc_Port_lookup_typedef(env,n);goto LOOP2;}case 4U: _LL24: _LL25:
 return({void*_tmp495=Cyc_Port_arith_ct();void*_tmp494=Cyc_Port_const_ct();Cyc_Port_array_ct(_tmp495,_tmp494,Cyc_Port_zterm_ct());});default: _LL26: _LL27:
 return Cyc_Port_gen_exp(0,env,e);}_LL21:;}default: goto _LLB;}default: _LLB: _LLC:
# 1640
 return Cyc_Port_gen_exp(0,env,e);}_LL0:;}
# 1128
static struct Cyc_Port_Cenv*Cyc_Port_gen_localdecl(struct Cyc_Port_Cenv*env,struct Cyc_Absyn_Decl*d,int fromglobal){
# 1646
void*_stmttmp24=d->r;void*_tmp26C=_stmttmp24;struct Cyc_Absyn_Vardecl*_tmp26D;if(((struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct*)_tmp26C)->tag == 0U){_LL1: _tmp26D=((struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct*)_tmp26C)->f1;_LL2: {struct Cyc_Absyn_Vardecl*vd=_tmp26D;
# 1648
if(!fromglobal)Cyc_Port_register_localvar_decl(env,vd->name,vd->varloc,vd->type,vd->initializer);{void*t=
Cyc_Port_var();
void*q;
if((env->gcenv)->porting){
q=Cyc_Port_var();
({struct Cyc_Port_Cenv*_tmp498=env;void*_tmp497=q;void*_tmp496=
(vd->tq).print_const?Cyc_Port_const_ct(): Cyc_Port_notconst_ct();
# 1653
Cyc_Port_register_const_cvar(_tmp498,_tmp497,_tmp496,(vd->tq).loc);});}else{
# 1662
q=(vd->tq).print_const?Cyc_Port_const_ct(): Cyc_Port_notconst_ct();}
# 1664
(*vd->name).f1=Cyc_Absyn_Loc_n;
env=Cyc_Port_add_var(env,vd->name,vd->type,q,t,vd->varloc);
({void*_tmp499=t;Cyc_Port_unifies(_tmp499,Cyc_Port_type_to_ctype(env,vd->type));});
if((unsigned)vd->initializer){
struct Cyc_Absyn_Exp*e=(struct Cyc_Absyn_Exp*)_check_null(vd->initializer);
void*t2=Cyc_Port_gen_initializer(env,vd->type,e);
Cyc_Port_leq(t2,t);}
# 1667
return env;}}}else{_LL3: _LL4:
# 1673
({void*_tmp26E=0U;({int(*_tmp49B)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp270)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp270;});struct _fat_ptr _tmp49A=({const char*_tmp26F="non-local decl that isn't a variable";_tag_fat(_tmp26F,sizeof(char),37U);});_tmp49B(_tmp49A,_tag_fat(_tmp26E,sizeof(void*),0U));});});}_LL0:;}
# 1128
static struct _tuple8*Cyc_Port_make_targ(struct _tuple8*arg){
# 1678
struct _tuple8 _stmttmp25=*arg;void*_tmp273;struct Cyc_Absyn_Tqual _tmp272;struct _fat_ptr*_tmp271;_LL1: _tmp271=_stmttmp25.f1;_tmp272=_stmttmp25.f2;_tmp273=_stmttmp25.f3;_LL2: {struct _fat_ptr*x=_tmp271;struct Cyc_Absyn_Tqual q=_tmp272;void*t=_tmp273;
return({struct _tuple8*_tmp274=_cycalloc(sizeof(*_tmp274));_tmp274->f1=0,_tmp274->f2=q,_tmp274->f3=t;_tmp274;});}}
# 1128
static struct Cyc_Port_Cenv*Cyc_Port_gen_topdecls(struct Cyc_Port_Cenv*env,struct Cyc_List_List*ds);struct _tuple19{struct _fat_ptr*f1;void*f2;void*f3;void*f4;};
# 1684
static struct Cyc_Port_Cenv*Cyc_Port_gen_topdecl(struct Cyc_Port_Cenv*env,struct Cyc_Absyn_Decl*d){
void*_stmttmp26=d->r;void*_tmp275=_stmttmp26;struct Cyc_Absyn_Enumdecl*_tmp276;struct Cyc_Absyn_Aggrdecl*_tmp277;struct Cyc_Absyn_Typedefdecl*_tmp278;struct Cyc_Absyn_Fndecl*_tmp279;struct Cyc_Absyn_Vardecl*_tmp27A;switch(*((int*)_tmp275)){case 13U: _LL1: _LL2:
# 1687
(env->gcenv)->porting=1;
return env;case 14U: _LL3: _LL4:
# 1690
(env->gcenv)->porting=0;
return env;case 0U: _LL5: _tmp27A=((struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct*)_tmp275)->f1;_LL6: {struct Cyc_Absyn_Vardecl*vd=_tmp27A;
# 1693
(*vd->name).f1=Cyc_Absyn_Loc_n;
# 1697
if(Cyc_Port_declared_var(env,vd->name)){
struct _tuple14 _stmttmp27=Cyc_Port_lookup_var(env,vd->name);unsigned _tmp27C;struct _tuple12 _tmp27B;_LL12: _tmp27B=_stmttmp27.f1;_tmp27C=_stmttmp27.f2;_LL13: {struct _tuple12 p=_tmp27B;unsigned loc=_tmp27C;
void*_tmp27E;void*_tmp27D;_LL15: _tmp27D=p.f1;_tmp27E=p.f2;_LL16: {void*q=_tmp27D;void*t=_tmp27E;
void*q2;
if((env->gcenv)->porting && !Cyc_Port_isfn(env,vd->name)){
q2=Cyc_Port_var();
({struct Cyc_Port_Cenv*_tmp49E=env;void*_tmp49D=q2;void*_tmp49C=
(vd->tq).print_const?Cyc_Port_const_ct(): Cyc_Port_notconst_ct();
# 1703
Cyc_Port_register_const_cvar(_tmp49E,_tmp49D,_tmp49C,(vd->tq).loc);});}else{
# 1712
q2=(vd->tq).print_const?Cyc_Port_const_ct(): Cyc_Port_notconst_ct();}
# 1714
Cyc_Port_unifies(q,q2);
({void*_tmp49F=t;Cyc_Port_unifies(_tmp49F,Cyc_Port_type_to_ctype(env,vd->type));});
if((unsigned)vd->initializer){
struct Cyc_Absyn_Exp*e=(struct Cyc_Absyn_Exp*)_check_null(vd->initializer);
({void*_tmp4A0=Cyc_Port_gen_initializer(env,vd->type,e);Cyc_Port_leq(_tmp4A0,t);});}
# 1716
return env;}}}else{
# 1722
return Cyc_Port_gen_localdecl(env,d,1);}}case 1U: _LL7: _tmp279=((struct Cyc_Absyn_Fn_d_Absyn_Raw_decl_struct*)_tmp275)->f1;_LL8: {struct Cyc_Absyn_Fndecl*fd=_tmp279;
# 1728
(*fd->name).f1=Cyc_Absyn_Loc_n;{
struct _tuple15*predeclared=0;
# 1731
if(Cyc_Port_declared_var(env,fd->name))
predeclared=Cyc_Port_lookup_full_var(env,fd->name);{
# 1731
void*rettype=(fd->i).ret_type;
# 1735
struct Cyc_List_List*args=(fd->i).args;
struct Cyc_List_List*targs=({struct Cyc_List_List*(*_tmp4A1)(struct _tuple8*(*f)(struct _tuple8*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp28F)(struct _tuple8*(*f)(struct _tuple8*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct _tuple8*(*f)(struct _tuple8*),struct Cyc_List_List*x))Cyc_List_map;_tmp28F;});_tmp4A1(Cyc_Port_make_targ,args);});
struct Cyc_Absyn_FnType_Absyn_Type_struct*fntype=({struct Cyc_Absyn_FnType_Absyn_Type_struct*_tmp28E=_cycalloc(sizeof(*_tmp28E));
_tmp28E->tag=5U,(_tmp28E->f1).tvars=0,(_tmp28E->f1).effect=0,({struct Cyc_Absyn_Tqual _tmp4A2=Cyc_Absyn_empty_tqual(0U);(_tmp28E->f1).ret_tqual=_tmp4A2;}),(_tmp28E->f1).ret_type=rettype,(_tmp28E->f1).args=targs,(_tmp28E->f1).c_varargs=0,(_tmp28E->f1).cyc_varargs=0,(_tmp28E->f1).rgn_po=0,(_tmp28E->f1).attributes=0,(_tmp28E->f1).requires_clause=0,(_tmp28E->f1).requires_relns=0,(_tmp28E->f1).ensures_clause=0,(_tmp28E->f1).ensures_relns=0,(_tmp28E->f1).ieffect=0,(_tmp28E->f1).oeffect=0;_tmp28E;});
# 1741
struct Cyc_Port_Cenv*fn_env=Cyc_Port_set_cpos(env,Cyc_Port_FnBody_pos);
void*c_rettype=Cyc_Port_type_to_ctype(fn_env,rettype);
struct Cyc_List_List*c_args=0;
struct Cyc_List_List*c_arg_types=0;
{struct Cyc_List_List*xs=args;for(0;(unsigned)xs;xs=xs->tl){
struct _tuple8 _stmttmp28=*((struct _tuple8*)xs->hd);void*_tmp281;struct Cyc_Absyn_Tqual _tmp280;struct _fat_ptr*_tmp27F;_LL18: _tmp27F=_stmttmp28.f1;_tmp280=_stmttmp28.f2;_tmp281=_stmttmp28.f3;_LL19: {struct _fat_ptr*x=_tmp27F;struct Cyc_Absyn_Tqual tq=_tmp280;void*t=_tmp281;
# 1749
void*ct=Cyc_Port_type_to_ctype(fn_env,t);
void*tqv;
if((env->gcenv)->porting){
tqv=Cyc_Port_var();
({struct Cyc_Port_Cenv*_tmp4A5=env;void*_tmp4A4=tqv;void*_tmp4A3=tq.print_const?Cyc_Port_const_ct(): Cyc_Port_notconst_ct();Cyc_Port_register_const_cvar(_tmp4A5,_tmp4A4,_tmp4A3,tq.loc);});}else{
# 1761
tqv=tq.print_const?Cyc_Port_const_ct(): Cyc_Port_notconst_ct();}
# 1763
c_args=({struct Cyc_List_List*_tmp283=_cycalloc(sizeof(*_tmp283));({struct _tuple19*_tmp4A6=({struct _tuple19*_tmp282=_cycalloc(sizeof(*_tmp282));_tmp282->f1=x,_tmp282->f2=t,_tmp282->f3=tqv,_tmp282->f4=ct;_tmp282;});_tmp283->hd=_tmp4A6;}),_tmp283->tl=c_args;_tmp283;});
c_arg_types=({struct Cyc_List_List*_tmp284=_cycalloc(sizeof(*_tmp284));_tmp284->hd=ct,_tmp284->tl=c_arg_types;_tmp284;});}}}
# 1766
c_args=Cyc_List_imp_rev(c_args);
c_arg_types=Cyc_List_imp_rev(c_arg_types);{
void*ctype=Cyc_Port_fn_ct(c_rettype,c_arg_types);
# 1772
(*fd->name).f1=Cyc_Absyn_Loc_n;
fn_env=({struct Cyc_Port_Cenv*_tmp4AA=fn_env;struct _tuple0*_tmp4A9=fd->name;void*_tmp4A8=(void*)fntype;void*_tmp4A7=Cyc_Port_const_ct();Cyc_Port_add_var(_tmp4AA,_tmp4A9,_tmp4A8,_tmp4A7,ctype,0U);});
Cyc_Port_add_return_type(fn_env,c_rettype);
{struct Cyc_List_List*xs=c_args;for(0;(unsigned)xs;xs=xs->tl){
struct _tuple19 _stmttmp29=*((struct _tuple19*)xs->hd);void*_tmp288;void*_tmp287;void*_tmp286;struct _fat_ptr*_tmp285;_LL1B: _tmp285=_stmttmp29.f1;_tmp286=_stmttmp29.f2;_tmp287=_stmttmp29.f3;_tmp288=_stmttmp29.f4;_LL1C: {struct _fat_ptr*x=_tmp285;void*t=_tmp286;void*q=_tmp287;void*ct=_tmp288;
fn_env=({struct Cyc_Port_Cenv*_tmp4AE=fn_env;struct _tuple0*_tmp4AD=({struct _tuple0*_tmp289=_cycalloc(sizeof(*_tmp289));_tmp289->f1=Cyc_Absyn_Loc_n,_tmp289->f2=(struct _fat_ptr*)_check_null(x);_tmp289;});void*_tmp4AC=t;void*_tmp4AB=q;Cyc_Port_add_var(_tmp4AE,_tmp4AD,_tmp4AC,_tmp4AB,ct,0U);});}}}
# 1779
Cyc_Port_gen_stmt(fn_env,fd->body);
# 1784
Cyc_Port_generalize(0,ctype);{
# 1791
struct Cyc_Dict_Dict counts=({struct Cyc_Dict_Dict(*_tmp28D)(int(*cmp)(struct _fat_ptr*,struct _fat_ptr*))=(struct Cyc_Dict_Dict(*)(int(*cmp)(struct _fat_ptr*,struct _fat_ptr*)))Cyc_Dict_empty;_tmp28D;})(Cyc_strptrcmp);
Cyc_Port_region_counts(& counts,ctype);
# 1795
Cyc_Port_register_rgns(env,counts,1,(void*)fntype,ctype);
env=({struct Cyc_Port_Cenv*_tmp4B2=fn_env;struct _tuple0*_tmp4B1=fd->name;void*_tmp4B0=(void*)fntype;void*_tmp4AF=Cyc_Port_const_ct();Cyc_Port_add_var(_tmp4B2,_tmp4B1,_tmp4B0,_tmp4AF,ctype,0U);});
if((unsigned)predeclared){
# 1804
struct _tuple15 _stmttmp2A=*predeclared;void*_tmp28C;void*_tmp28B;void*_tmp28A;_LL1E: _tmp28A=_stmttmp2A.f1;_tmp28B=(_stmttmp2A.f2)->f1;_tmp28C=(_stmttmp2A.f2)->f2;_LL1F: {void*orig_type=_tmp28A;void*q2=_tmp28B;void*t2=_tmp28C;
({void*_tmp4B3=q2;Cyc_Port_unifies(_tmp4B3,Cyc_Port_const_ct());});
({void*_tmp4B4=t2;Cyc_Port_unifies(_tmp4B4,Cyc_Port_instantiate(ctype));});
# 1808
Cyc_Port_register_rgns(env,counts,1,orig_type,ctype);}}
# 1797
return env;}}}}}case 8U: _LL9: _tmp278=((struct Cyc_Absyn_Typedef_d_Absyn_Raw_decl_struct*)_tmp275)->f1;_LLA: {struct Cyc_Absyn_Typedefdecl*td=_tmp278;
# 1816
void*t=(void*)_check_null(td->defn);
void*ct=Cyc_Port_type_to_ctype(env,t);
(*td->name).f1=Cyc_Absyn_Loc_n;
Cyc_Port_add_typedef(env,td->name,t,ct);
return env;}case 5U: _LLB: _tmp277=((struct Cyc_Absyn_Aggr_d_Absyn_Raw_decl_struct*)_tmp275)->f1;_LLC: {struct Cyc_Absyn_Aggrdecl*ad=_tmp277;
# 1826
struct _tuple0*name=ad->name;
(*ad->name).f1=Cyc_Absyn_Loc_n;{
struct _tuple11*previous;
struct Cyc_List_List*curr=0;
{enum Cyc_Absyn_AggrKind _stmttmp2B=ad->kind;enum Cyc_Absyn_AggrKind _tmp290=_stmttmp2B;if(_tmp290 == Cyc_Absyn_StructA){_LL21: _LL22:
# 1832
 previous=Cyc_Port_lookup_struct_decl(env,name);
goto _LL20;}else{_LL23: _LL24:
# 1835
 previous=Cyc_Port_lookup_union_decl(env,name);
goto _LL20;}_LL20:;}
# 1838
if((unsigned)ad->impl){
struct Cyc_List_List*cf=(*previous).f2;
{struct Cyc_List_List*fields=((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->fields;for(0;(unsigned)fields;fields=fields->tl){
struct Cyc_Absyn_Aggrfield*f=(struct Cyc_Absyn_Aggrfield*)fields->hd;
void*qv;
if((env->gcenv)->porting){
qv=Cyc_Port_var();
({struct Cyc_Port_Cenv*_tmp4B7=env;void*_tmp4B6=qv;void*_tmp4B5=
(f->tq).print_const?Cyc_Port_const_ct(): Cyc_Port_notconst_ct();
# 1845
Cyc_Port_register_const_cvar(_tmp4B7,_tmp4B6,_tmp4B5,(f->tq).loc);});}else{
# 1854
qv=(f->tq).print_const?Cyc_Port_const_ct(): Cyc_Port_notconst_ct();}{
# 1856
void*ct=Cyc_Port_type_to_ctype(env,f->type);
if(cf != 0){
struct Cyc_Port_Cfield _stmttmp2C=*((struct Cyc_Port_Cfield*)cf->hd);void*_tmp292;void*_tmp291;_LL26: _tmp291=_stmttmp2C.qual;_tmp292=_stmttmp2C.type;_LL27: {void*qv2=_tmp291;void*ct2=_tmp292;
cf=cf->tl;
Cyc_Port_unifies(qv,qv2);
Cyc_Port_unifies(ct,ct2);}}
# 1857
curr=({struct Cyc_List_List*_tmp294=_cycalloc(sizeof(*_tmp294));
# 1863
({struct Cyc_Port_Cfield*_tmp4B8=({struct Cyc_Port_Cfield*_tmp293=_cycalloc(sizeof(*_tmp293));_tmp293->qual=qv,_tmp293->f=f->name,_tmp293->type=ct;_tmp293;});_tmp294->hd=_tmp4B8;}),_tmp294->tl=curr;_tmp294;});}}}
# 1865
curr=Cyc_List_imp_rev(curr);
if((*previous).f2 == 0)
(*previous).f2=curr;}
# 1838
return env;}}case 7U: _LLD: _tmp276=((struct Cyc_Absyn_Enum_d_Absyn_Raw_decl_struct*)_tmp275)->f1;_LLE: {struct Cyc_Absyn_Enumdecl*ed=_tmp276;
# 1875
(*ed->name).f1=Cyc_Absyn_Loc_n;
# 1877
if((unsigned)ed->fields){
struct Cyc_List_List*fs=(struct Cyc_List_List*)((struct Cyc_Core_Opt*)_check_null(ed->fields))->v;for(0;(unsigned)fs;fs=fs->tl){
(*((struct Cyc_Absyn_Enumfield*)fs->hd)->name).f1=Cyc_Absyn_Loc_n;
env=({struct Cyc_Port_Cenv*_tmp4BC=env;struct _tuple0*_tmp4BB=((struct Cyc_Absyn_Enumfield*)fs->hd)->name;void*_tmp4BA=Cyc_Absyn_enum_type(ed->name,ed);void*_tmp4B9=
Cyc_Port_const_ct();
# 1880
Cyc_Port_add_var(_tmp4BC,_tmp4BB,_tmp4BA,_tmp4B9,
Cyc_Port_arith_ct(),0U);});}}
# 1877
return env;}default: _LLF: _LL10:
# 1885
 if((env->gcenv)->porting)
({void*_tmp295=0U;({struct Cyc___cycFILE*_tmp4BE=Cyc_stderr;struct _fat_ptr _tmp4BD=({const char*_tmp296="Warning: Cyclone declaration found in code to be ported -- skipping.";_tag_fat(_tmp296,sizeof(char),69U);});Cyc_fprintf(_tmp4BE,_tmp4BD,_tag_fat(_tmp295,sizeof(void*),0U));});});
# 1885
return env;}_LL0:;}
# 1684
static struct Cyc_Port_Cenv*Cyc_Port_gen_topdecls(struct Cyc_Port_Cenv*env,struct Cyc_List_List*ds){
# 1893
for(0;(unsigned)ds;ds=ds->tl){
env=Cyc_Port_gen_topdecl(env,(struct Cyc_Absyn_Decl*)ds->hd);}
return env;}
# 1684
static struct Cyc_List_List*Cyc_Port_gen_edits(struct Cyc_List_List*ds){
# 1901
struct Cyc_Port_Cenv*env=({struct Cyc_Port_Cenv*_tmp4BF=Cyc_Port_initial_cenv();Cyc_Port_gen_topdecls(_tmp4BF,ds);});
# 1903
struct Cyc_List_List*ptrs=(env->gcenv)->pointer_edits;
struct Cyc_List_List*consts=(env->gcenv)->qualifier_edits;
struct Cyc_List_List*zts=(env->gcenv)->zeroterm_edits;
struct Cyc_List_List*edits=(env->gcenv)->edits;
struct Cyc_List_List*localvars=(env->gcenv)->vardecl_locs;
# 1909
for(0;(unsigned)localvars;localvars=localvars->tl){
struct _tuple16 _stmttmp2D=*((struct _tuple16*)localvars->hd);void*_tmp299;unsigned _tmp298;struct _tuple0*_tmp297;_LL1: _tmp297=_stmttmp2D.f1;_tmp298=_stmttmp2D.f2;_tmp299=_stmttmp2D.f3;_LL2: {struct _tuple0*var=_tmp297;unsigned loc=_tmp298;void*vartype=_tmp299;
struct _tuple0 _stmttmp2E=*var;struct _fat_ptr*_tmp29A;_LL4: _tmp29A=_stmttmp2E.f2;_LL5: {struct _fat_ptr*x=_tmp29A;
struct Cyc_Port_VarUsage*varusage=({struct Cyc_Port_VarUsage*(*_tmp4C1)(struct Cyc_Hashtable_Table*t,unsigned key)=({struct Cyc_Port_VarUsage*(*_tmp2AE)(struct Cyc_Hashtable_Table*t,unsigned key)=(struct Cyc_Port_VarUsage*(*)(struct Cyc_Hashtable_Table*t,unsigned key))Cyc_Hashtable_lookup;_tmp2AE;});struct Cyc_Hashtable_Table*_tmp4C0=(env->gcenv)->varusage_tab;_tmp4C1(_tmp4C0,loc);});
if(((struct Cyc_Port_VarUsage*)_check_null(varusage))->address_taken){
if((unsigned)varusage->init){
# 1916
edits=({struct Cyc_List_List*_tmp29E=_cycalloc(sizeof(*_tmp29E));({struct Cyc_Port_Edit*_tmp4C4=({struct Cyc_Port_Edit*_tmp29D=_cycalloc(sizeof(*_tmp29D));_tmp29D->loc=loc,({struct _fat_ptr _tmp4C3=({const char*_tmp29B="@";_tag_fat(_tmp29B,sizeof(char),2U);});_tmp29D->old_string=_tmp4C3;}),({struct _fat_ptr _tmp4C2=({const char*_tmp29C="";_tag_fat(_tmp29C,sizeof(char),1U);});_tmp29D->new_string=_tmp4C2;});_tmp29D;});_tmp29E->hd=_tmp4C4;}),_tmp29E->tl=edits;_tmp29E;});
edits=({struct Cyc_List_List*_tmp2A2=_cycalloc(sizeof(*_tmp2A2));({struct Cyc_Port_Edit*_tmp4C7=({struct Cyc_Port_Edit*_tmp2A1=_cycalloc(sizeof(*_tmp2A1));_tmp2A1->loc=((struct Cyc_Absyn_Exp*)_check_null(varusage->init))->loc,({struct _fat_ptr _tmp4C6=({const char*_tmp29F="new ";_tag_fat(_tmp29F,sizeof(char),5U);});_tmp2A1->old_string=_tmp4C6;}),({struct _fat_ptr _tmp4C5=({const char*_tmp2A0="";_tag_fat(_tmp2A0,sizeof(char),1U);});_tmp2A1->new_string=_tmp4C5;});_tmp2A1;});_tmp2A2->hd=_tmp4C7;}),_tmp2A2->tl=edits;_tmp2A2;});}else{
# 1921
edits=({struct Cyc_List_List*_tmp2A8=_cycalloc(sizeof(*_tmp2A8));({struct Cyc_Port_Edit*_tmp4CB=({struct Cyc_Port_Edit*_tmp2A7=_cycalloc(sizeof(*_tmp2A7));_tmp2A7->loc=loc,({struct _fat_ptr _tmp4CA=(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp2A5=({struct Cyc_String_pa_PrintArg_struct _tmp311;_tmp311.tag=0U,_tmp311.f1=(struct _fat_ptr)((struct _fat_ptr)*x);_tmp311;});struct Cyc_String_pa_PrintArg_struct _tmp2A6=({struct Cyc_String_pa_PrintArg_struct _tmp310;_tmp310.tag=0U,({struct _fat_ptr _tmp4C8=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(vartype));_tmp310.f1=_tmp4C8;});_tmp310;});void*_tmp2A3[2U];_tmp2A3[0]=& _tmp2A5,_tmp2A3[1]=& _tmp2A6;({struct _fat_ptr _tmp4C9=({const char*_tmp2A4="?%s = malloc(sizeof(%s))";_tag_fat(_tmp2A4,sizeof(char),25U);});Cyc_aprintf(_tmp4C9,_tag_fat(_tmp2A3,sizeof(void*),2U));});});_tmp2A7->old_string=_tmp4CA;}),_tmp2A7->new_string=*x;_tmp2A7;});_tmp2A8->hd=_tmp4CB;}),_tmp2A8->tl=edits;_tmp2A8;});}{
# 1923
struct Cyc_List_List*loclist=varusage->usage_locs;
for(0;(unsigned)loclist;loclist=loclist->tl){
edits=({struct Cyc_List_List*_tmp2AD=_cycalloc(sizeof(*_tmp2AD));({struct Cyc_Port_Edit*_tmp4CE=({struct Cyc_Port_Edit*_tmp2AC=_cycalloc(sizeof(*_tmp2AC));_tmp2AC->loc=(unsigned)loclist->hd,({struct _fat_ptr _tmp4CD=(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp2AB=({struct Cyc_String_pa_PrintArg_struct _tmp312;_tmp312.tag=0U,_tmp312.f1=(struct _fat_ptr)((struct _fat_ptr)*x);_tmp312;});void*_tmp2A9[1U];_tmp2A9[0]=& _tmp2AB;({struct _fat_ptr _tmp4CC=({const char*_tmp2AA="(*%s)";_tag_fat(_tmp2AA,sizeof(char),6U);});Cyc_aprintf(_tmp4CC,_tag_fat(_tmp2A9,sizeof(void*),1U));});});_tmp2AC->old_string=_tmp4CD;}),_tmp2AC->new_string=*x;_tmp2AC;});_tmp2AD->hd=_tmp4CE;}),_tmp2AD->tl=edits;_tmp2AD;});}}}}}}
# 1931
{struct Cyc_List_List*ps=ptrs;for(0;(unsigned)ps;ps=ps->tl){
struct _tuple17 _stmttmp2F=*((struct _tuple17*)ps->hd);unsigned _tmp2B1;void*_tmp2B0;void*_tmp2AF;_LL7: _tmp2AF=_stmttmp2F.f1;_tmp2B0=_stmttmp2F.f2;_tmp2B1=_stmttmp2F.f3;_LL8: {void*new_ctype=_tmp2AF;void*orig_ctype=_tmp2B0;unsigned loc=_tmp2B1;
Cyc_Port_unifies(new_ctype,orig_ctype);}}}
# 1935
{struct Cyc_List_List*cs=consts;for(0;(unsigned)cs;cs=cs->tl){
struct _tuple17 _stmttmp30=*((struct _tuple17*)cs->hd);unsigned _tmp2B4;void*_tmp2B3;void*_tmp2B2;_LLA: _tmp2B2=_stmttmp30.f1;_tmp2B3=_stmttmp30.f2;_tmp2B4=_stmttmp30.f3;_LLB: {void*new_qual=_tmp2B2;void*old_qual=_tmp2B3;unsigned loc=_tmp2B4;
Cyc_Port_unifies(new_qual,old_qual);}}}
# 1939
{struct Cyc_List_List*zs=zts;for(0;(unsigned)zs;zs=zs->tl){
struct _tuple17 _stmttmp31=*((struct _tuple17*)zs->hd);unsigned _tmp2B7;void*_tmp2B6;void*_tmp2B5;_LLD: _tmp2B5=_stmttmp31.f1;_tmp2B6=_stmttmp31.f2;_tmp2B7=_stmttmp31.f3;_LLE: {void*new_zt=_tmp2B5;void*old_zt=_tmp2B6;unsigned loc=_tmp2B7;
Cyc_Port_unifies(new_zt,old_zt);}}}
# 1945
for(0;(unsigned)ptrs;ptrs=ptrs->tl){
struct _tuple17 _stmttmp32=*((struct _tuple17*)ptrs->hd);unsigned _tmp2BA;void*_tmp2B9;void*_tmp2B8;_LL10: _tmp2B8=_stmttmp32.f1;_tmp2B9=_stmttmp32.f2;_tmp2BA=_stmttmp32.f3;_LL11: {void*new_ctype=_tmp2B8;void*orig_ctype=_tmp2B9;unsigned loc=_tmp2BA;
if(!Cyc_Port_unifies(new_ctype,orig_ctype)&&(int)loc){
void*_stmttmp33=Cyc_Port_compress_ct(orig_ctype);void*_tmp2BB=_stmttmp33;switch(*((int*)_tmp2BB)){case 2U: _LL13: _LL14:
# 1950
 edits=({struct Cyc_List_List*_tmp2BF=_cycalloc(sizeof(*_tmp2BF));({struct Cyc_Port_Edit*_tmp4D1=({struct Cyc_Port_Edit*_tmp2BE=_cycalloc(sizeof(*_tmp2BE));_tmp2BE->loc=loc,({struct _fat_ptr _tmp4D0=({const char*_tmp2BC="?";_tag_fat(_tmp2BC,sizeof(char),2U);});_tmp2BE->old_string=_tmp4D0;}),({struct _fat_ptr _tmp4CF=({const char*_tmp2BD="*";_tag_fat(_tmp2BD,sizeof(char),2U);});_tmp2BE->new_string=_tmp4CF;});_tmp2BE;});_tmp2BF->hd=_tmp4D1;}),_tmp2BF->tl=edits;_tmp2BF;});
goto _LL12;case 3U: _LL15: _LL16:
# 1953
 edits=({struct Cyc_List_List*_tmp2C3=_cycalloc(sizeof(*_tmp2C3));({struct Cyc_Port_Edit*_tmp4D4=({struct Cyc_Port_Edit*_tmp2C2=_cycalloc(sizeof(*_tmp2C2));_tmp2C2->loc=loc,({struct _fat_ptr _tmp4D3=({const char*_tmp2C0="*";_tag_fat(_tmp2C0,sizeof(char),2U);});_tmp2C2->old_string=_tmp4D3;}),({struct _fat_ptr _tmp4D2=({const char*_tmp2C1="?";_tag_fat(_tmp2C1,sizeof(char),2U);});_tmp2C2->new_string=_tmp4D2;});_tmp2C2;});_tmp2C3->hd=_tmp4D4;}),_tmp2C3->tl=edits;_tmp2C3;});
goto _LL12;default: _LL17: _LL18:
 goto _LL12;}_LL12:;}}}
# 1960
for(0;(unsigned)consts;consts=consts->tl){
struct _tuple17 _stmttmp34=*((struct _tuple17*)consts->hd);unsigned _tmp2C6;void*_tmp2C5;void*_tmp2C4;_LL1A: _tmp2C4=_stmttmp34.f1;_tmp2C5=_stmttmp34.f2;_tmp2C6=_stmttmp34.f3;_LL1B: {void*new_qual=_tmp2C4;void*old_qual=_tmp2C5;unsigned loc=_tmp2C6;
if(!Cyc_Port_unifies(new_qual,old_qual)&&(int)loc){
void*_stmttmp35=Cyc_Port_compress_ct(old_qual);void*_tmp2C7=_stmttmp35;switch(*((int*)_tmp2C7)){case 1U: _LL1D: _LL1E:
# 1965
 edits=({struct Cyc_List_List*_tmp2CB=_cycalloc(sizeof(*_tmp2CB));({struct Cyc_Port_Edit*_tmp4D7=({struct Cyc_Port_Edit*_tmp2CA=_cycalloc(sizeof(*_tmp2CA));_tmp2CA->loc=loc,({struct _fat_ptr _tmp4D6=({const char*_tmp2C8="const ";_tag_fat(_tmp2C8,sizeof(char),7U);});_tmp2CA->old_string=_tmp4D6;}),({struct _fat_ptr _tmp4D5=({const char*_tmp2C9="";_tag_fat(_tmp2C9,sizeof(char),1U);});_tmp2CA->new_string=_tmp4D5;});_tmp2CA;});_tmp2CB->hd=_tmp4D7;}),_tmp2CB->tl=edits;_tmp2CB;});goto _LL1C;case 0U: _LL1F: _LL20:
# 1967
 edits=({struct Cyc_List_List*_tmp2CF=_cycalloc(sizeof(*_tmp2CF));({struct Cyc_Port_Edit*_tmp4DA=({struct Cyc_Port_Edit*_tmp2CE=_cycalloc(sizeof(*_tmp2CE));_tmp2CE->loc=loc,({struct _fat_ptr _tmp4D9=({const char*_tmp2CC="";_tag_fat(_tmp2CC,sizeof(char),1U);});_tmp2CE->old_string=_tmp4D9;}),({struct _fat_ptr _tmp4D8=({const char*_tmp2CD="const ";_tag_fat(_tmp2CD,sizeof(char),7U);});_tmp2CE->new_string=_tmp4D8;});_tmp2CE;});_tmp2CF->hd=_tmp4DA;}),_tmp2CF->tl=edits;_tmp2CF;});goto _LL1C;default: _LL21: _LL22:
 goto _LL1C;}_LL1C:;}}}
# 1973
for(0;(unsigned)zts;zts=zts->tl){
struct _tuple17 _stmttmp36=*((struct _tuple17*)zts->hd);unsigned _tmp2D2;void*_tmp2D1;void*_tmp2D0;_LL24: _tmp2D0=_stmttmp36.f1;_tmp2D1=_stmttmp36.f2;_tmp2D2=_stmttmp36.f3;_LL25: {void*new_zt=_tmp2D0;void*old_zt=_tmp2D1;unsigned loc=_tmp2D2;
if(!Cyc_Port_unifies(new_zt,old_zt)&&(int)loc){
void*_stmttmp37=Cyc_Port_compress_ct(old_zt);void*_tmp2D3=_stmttmp37;switch(*((int*)_tmp2D3)){case 8U: _LL27: _LL28:
# 1978
 edits=({struct Cyc_List_List*_tmp2D7=_cycalloc(sizeof(*_tmp2D7));({struct Cyc_Port_Edit*_tmp4DD=({struct Cyc_Port_Edit*_tmp2D6=_cycalloc(sizeof(*_tmp2D6));_tmp2D6->loc=loc,({struct _fat_ptr _tmp4DC=({const char*_tmp2D4="@nozeroterm ";_tag_fat(_tmp2D4,sizeof(char),13U);});_tmp2D6->old_string=_tmp4DC;}),({struct _fat_ptr _tmp4DB=({const char*_tmp2D5="";_tag_fat(_tmp2D5,sizeof(char),1U);});_tmp2D6->new_string=_tmp4DB;});_tmp2D6;});_tmp2D7->hd=_tmp4DD;}),_tmp2D7->tl=edits;_tmp2D7;});goto _LL26;case 9U: _LL29: _LL2A:
# 1980
 edits=({struct Cyc_List_List*_tmp2DB=_cycalloc(sizeof(*_tmp2DB));({struct Cyc_Port_Edit*_tmp4E0=({struct Cyc_Port_Edit*_tmp2DA=_cycalloc(sizeof(*_tmp2DA));_tmp2DA->loc=loc,({struct _fat_ptr _tmp4DF=({const char*_tmp2D8="@zeroterm ";_tag_fat(_tmp2D8,sizeof(char),11U);});_tmp2DA->old_string=_tmp4DF;}),({struct _fat_ptr _tmp4DE=({const char*_tmp2D9="";_tag_fat(_tmp2D9,sizeof(char),1U);});_tmp2DA->new_string=_tmp4DE;});_tmp2DA;});_tmp2DB->hd=_tmp4E0;}),_tmp2DB->tl=edits;_tmp2DB;});goto _LL26;default: _LL2B: _LL2C:
 goto _LL26;}_LL26:;}}}
# 1987
edits=({struct Cyc_List_List*(*_tmp4E1)(int(*cmp)(struct Cyc_Port_Edit*,struct Cyc_Port_Edit*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp2DC)(int(*cmp)(struct Cyc_Port_Edit*,struct Cyc_Port_Edit*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(int(*cmp)(struct Cyc_Port_Edit*,struct Cyc_Port_Edit*),struct Cyc_List_List*x))Cyc_List_merge_sort;_tmp2DC;});_tmp4E1(Cyc_Port_cmp_edit,edits);});
# 1989
while((unsigned)edits &&((struct Cyc_Port_Edit*)edits->hd)->loc == (unsigned)0){
# 1993
edits=edits->tl;}
# 1995
return edits;}
# 1684
void Cyc_Port_port(struct Cyc_List_List*ds){
# 2001
struct Cyc_List_List*edits=Cyc_Port_gen_edits(ds);
struct Cyc_List_List*locs=({struct Cyc_List_List*(*_tmp4E2)(unsigned(*f)(struct Cyc_Port_Edit*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp2E5)(unsigned(*f)(struct Cyc_Port_Edit*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(unsigned(*f)(struct Cyc_Port_Edit*),struct Cyc_List_List*x))Cyc_List_map;_tmp2E5;});_tmp4E2(Cyc_Port_get_seg,edits);});
Cyc_Position_use_gcc_style_location=0;{
struct Cyc_List_List*slocs=Cyc_List_imp_rev(Cyc_Position_strings_of_segments(locs));
for(0;(unsigned)edits;(edits=edits->tl,slocs=slocs->tl)){
struct Cyc_Port_Edit _stmttmp38=*((struct Cyc_Port_Edit*)edits->hd);struct _fat_ptr _tmp2DF;struct _fat_ptr _tmp2DE;unsigned _tmp2DD;_LL1: _tmp2DD=_stmttmp38.loc;_tmp2DE=_stmttmp38.old_string;_tmp2DF=_stmttmp38.new_string;_LL2: {unsigned loc=_tmp2DD;struct _fat_ptr s1=_tmp2DE;struct _fat_ptr s2=_tmp2DF;
struct _fat_ptr sloc=(struct _fat_ptr)*((struct _fat_ptr*)((struct Cyc_List_List*)_check_null(slocs))->hd);
({struct Cyc_String_pa_PrintArg_struct _tmp2E2=({struct Cyc_String_pa_PrintArg_struct _tmp315;_tmp315.tag=0U,_tmp315.f1=(struct _fat_ptr)((struct _fat_ptr)sloc);_tmp315;});struct Cyc_String_pa_PrintArg_struct _tmp2E3=({struct Cyc_String_pa_PrintArg_struct _tmp314;_tmp314.tag=0U,_tmp314.f1=(struct _fat_ptr)((struct _fat_ptr)s1);_tmp314;});struct Cyc_String_pa_PrintArg_struct _tmp2E4=({struct Cyc_String_pa_PrintArg_struct _tmp313;_tmp313.tag=0U,_tmp313.f1=(struct _fat_ptr)((struct _fat_ptr)s2);_tmp313;});void*_tmp2E0[3U];_tmp2E0[0]=& _tmp2E2,_tmp2E0[1]=& _tmp2E3,_tmp2E0[2]=& _tmp2E4;({struct _fat_ptr _tmp4E3=({const char*_tmp2E1="%s: insert `%s' for `%s'\n";_tag_fat(_tmp2E1,sizeof(char),26U);});Cyc_printf(_tmp4E3,_tag_fat(_tmp2E0,sizeof(void*),3U));});});}}}}
