#include <setjmp.h>
/* This is a C header file to be used by the output of the Cyclone to
   C translator.  The corresponding definitions are in file lib/runtime_*.c */
#ifndef _CYC_INCLUDE_H_
#define _CYC_INCLUDE_H_

/* Need one of these per thread (see runtime_stack.c). The runtime maintains 
   a stack that contains either _handler_cons structs or _RegionHandle structs.
   The tag is 0 for a handler_cons and 1 for a region handle.  */
struct _RuntimeStack {
  int tag; 
  struct _RuntimeStack *next;
  void (*cleanup)(struct _RuntimeStack *frame);
};

#ifndef offsetof
/* should be size_t, but int is fine. */
#define offsetof(t,n) ((int)(&(((t *)0)->n)))
#endif

/* Fat pointers */
struct _fat_ptr {
  unsigned char *curr; 
  unsigned char *base; 
  unsigned char *last_plus_one; 
};  

/* Discriminated Unions */
struct _xtunion_struct { char *tag; };

/* Regions */
struct _RegionPage
#ifdef CYC_REGION_PROFILE
{ unsigned total_bytes;
  unsigned free_bytes;
  /* MWH: wish we didn't have to include the stuff below ... */
  struct _RegionPage *next;
  char data[1];
}
#endif
; // abstract -- defined in runtime_memory.c
struct _RegionHandle {
  struct _RuntimeStack s;
  struct _RegionPage *curr;
  char               *offset;
  char               *last_plus_one;
  struct _DynRegionHandle *sub_regions;
#ifdef CYC_REGION_PROFILE
  const char         *name;
#else
  unsigned used_bytes;
  unsigned wasted_bytes;
#endif
};
struct _DynRegionFrame {
  struct _RuntimeStack s;
  struct _DynRegionHandle *x;
};
// A dynamic region is just a region handle.  The wrapper struct is for type
// abstraction.
struct Cyc_Core_DynamicRegion {
  struct _RegionHandle h;
};

struct _RegionHandle _new_region(const char*);
void* _region_malloc(struct _RegionHandle*, unsigned);
void* _region_calloc(struct _RegionHandle*, unsigned t, unsigned n);
void   _free_region(struct _RegionHandle*);
struct _RegionHandle*_open_dynregion(struct _DynRegionFrame*,struct _DynRegionHandle*);
void   _pop_dynregion();

/* Exceptions */
struct _handler_cons {
  struct _RuntimeStack s;
  jmp_buf handler;
};
void _push_handler(struct _handler_cons *);
void _push_region(struct _RegionHandle *);
void _npop_handler(int);
void _pop_handler();
void _pop_region();

#ifndef _throw
void* _throw_null_fn(const char*,unsigned);
void* _throw_arraybounds_fn(const char*,unsigned);
void* _throw_badalloc_fn(const char*,unsigned);
void* _throw_match_fn(const char*,unsigned);
void* _throw_fn(void*,const char*,unsigned);
void* _rethrow(void*);
#define _throw_null() (_throw_null_fn(__FILE__,__LINE__))
#define _throw_arraybounds() (_throw_arraybounds_fn(__FILE__,__LINE__))
#define _throw_badalloc() (_throw_badalloc_fn(__FILE__,__LINE__))
#define _throw_match() (_throw_match_fn(__FILE__,__LINE__))
#define _throw(e) (_throw_fn((e),__FILE__,__LINE__))
#endif

struct _xtunion_struct* Cyc_Core_get_exn_thrown();
/* Built-in Exceptions */
struct Cyc_Null_Exception_exn_struct { char *tag; };
struct Cyc_Array_bounds_exn_struct { char *tag; };
struct Cyc_Match_Exception_exn_struct { char *tag; };
struct Cyc_Bad_alloc_exn_struct { char *tag; };
extern char Cyc_Null_Exception[];
extern char Cyc_Array_bounds[];
extern char Cyc_Match_Exception[];
extern char Cyc_Bad_alloc[];

/* Built-in Run-time Checks and company */
#ifdef CYC_ANSI_OUTPUT
#define _INLINE  
#else
#define _INLINE inline
#endif

#ifdef NO_CYC_NULL_CHECKS
#define _check_null(ptr) (ptr)
#else
#define _check_null(ptr) \
  ({ void*_cks_null = (void*)(ptr); \
     if (!_cks_null) _throw_null(); \
     _cks_null; })
#endif

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index)\
   (((char*)ptr) + (elt_sz)*(index))
#ifdef NO_CYC_NULL_CHECKS
#define _check_known_subscript_null _check_known_subscript_notnull
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr);\
  int _index = (index);\
  if (!_cks_ptr) _throw_null(); \
  _cks_ptr + (elt_sz)*_index; })
#endif
#define _zero_arr_plus_fn(orig_x,orig_sz,orig_i,f,l) ((orig_x)+(orig_i))
#define _zero_arr_plus_char_fn _zero_arr_plus_fn
#define _zero_arr_plus_short_fn _zero_arr_plus_fn
#define _zero_arr_plus_int_fn _zero_arr_plus_fn
#define _zero_arr_plus_float_fn _zero_arr_plus_fn
#define _zero_arr_plus_double_fn _zero_arr_plus_fn
#define _zero_arr_plus_longdouble_fn _zero_arr_plus_fn
#define _zero_arr_plus_voidstar_fn _zero_arr_plus_fn
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (!_cks_ptr) _throw_null(); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })

/* _zero_arr_plus_*_fn(x,sz,i,filename,lineno) adds i to zero-terminated ptr
   x that has at least sz elements */
char* _zero_arr_plus_char_fn(char*,unsigned,int,const char*,unsigned);
short* _zero_arr_plus_short_fn(short*,unsigned,int,const char*,unsigned);
int* _zero_arr_plus_int_fn(int*,unsigned,int,const char*,unsigned);
float* _zero_arr_plus_float_fn(float*,unsigned,int,const char*,unsigned);
double* _zero_arr_plus_double_fn(double*,unsigned,int,const char*,unsigned);
long double* _zero_arr_plus_longdouble_fn(long double*,unsigned,int,const char*, unsigned);
void** _zero_arr_plus_voidstar_fn(void**,unsigned,int,const char*,unsigned);
#endif

/* _get_zero_arr_size_*(x,sz) returns the number of elements in a
   zero-terminated array that is NULL or has at least sz elements */
int _get_zero_arr_size_char(const char*,unsigned);
int _get_zero_arr_size_short(const short*,unsigned);
int _get_zero_arr_size_int(const int*,unsigned);
int _get_zero_arr_size_float(const float*,unsigned);
int _get_zero_arr_size_double(const double*,unsigned);
int _get_zero_arr_size_longdouble(const long double*,unsigned);
int _get_zero_arr_size_voidstar(const void**,unsigned);

/* _zero_arr_inplace_plus_*_fn(x,i,filename,lineno) sets
   zero-terminated pointer *x to *x + i */
char* _zero_arr_inplace_plus_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_short_fn(short**,int,const char*,unsigned);
int* _zero_arr_inplace_plus_int(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_longdouble_fn(long double**,int,const char*,unsigned);
void** _zero_arr_inplace_plus_voidstar_fn(void***,int,const char*,unsigned);
/* like the previous functions, but does post-addition (as in e++) */
char* _zero_arr_inplace_plus_post_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_post_short_fn(short**x,int,const char*,unsigned);
int* _zero_arr_inplace_plus_post_int_fn(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_post_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_post_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_post_longdouble_fn(long double**,int,const char *,unsigned);
void** _zero_arr_inplace_plus_post_voidstar_fn(void***,int,const char*,unsigned);
#define _zero_arr_plus_char(x,s,i) \
  (_zero_arr_plus_char_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_short(x,s,i) \
  (_zero_arr_plus_short_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_int(x,s,i) \
  (_zero_arr_plus_int_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_float(x,s,i) \
  (_zero_arr_plus_float_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_double(x,s,i) \
  (_zero_arr_plus_double_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_longdouble(x,s,i) \
  (_zero_arr_plus_longdouble_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_voidstar(x,s,i) \
  (_zero_arr_plus_voidstar_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_inplace_plus_char(x,i) \
  _zero_arr_inplace_plus_char_fn((char **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_short(x,i) \
  _zero_arr_inplace_plus_short_fn((short **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_int(x,i) \
  _zero_arr_inplace_plus_int_fn((int **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_float(x,i) \
  _zero_arr_inplace_plus_float_fn((float **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_double(x,i) \
  _zero_arr_inplace_plus_double_fn((double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_longdouble(x,i) \
  _zero_arr_inplace_plus_longdouble_fn((long double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_voidstar(x,i) \
  _zero_arr_inplace_plus_voidstar_fn((void ***)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_char(x,i) \
  _zero_arr_inplace_plus_post_char_fn((char **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_short(x,i) \
  _zero_arr_inplace_plus_post_short_fn((short **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_int(x,i) \
  _zero_arr_inplace_plus_post_int_fn((int **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_float(x,i) \
  _zero_arr_inplace_plus_post_float_fn((float **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_double(x,i) \
  _zero_arr_inplace_plus_post_double_fn((double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_longdouble(x,i) \
  _zero_arr_inplace_plus_post_longdouble_fn((long double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_voidstar(x,i) \
  _zero_arr_inplace_plus_post_voidstar_fn((void***)(x),(i),__FILE__,__LINE__)

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_fat_subscript(arr,elt_sz,index) ((arr).curr + (elt_sz) * (index))
#define _untag_fat_ptr(arr,elt_sz,num_elts) ((arr).curr)
#else
#define _check_fat_subscript(arr,elt_sz,index) ({ \
  struct _fat_ptr _cus_arr = (arr); \
  unsigned char *_cus_ans = _cus_arr.curr + (elt_sz) * (index); \
  /* JGM: not needed! if (!_cus_arr.base) _throw_null();*/ \
  if (_cus_ans < _cus_arr.base || _cus_ans >= _cus_arr.last_plus_one) \
    _throw_arraybounds(); \
  _cus_ans; })
#define _untag_fat_ptr(arr,elt_sz,num_elts) ({ \
  struct _fat_ptr _arr = (arr); \
  unsigned char *_curr = _arr.curr; \
  if ((_curr < _arr.base || _curr + (elt_sz) * (num_elts) > _arr.last_plus_one) &&\
      _curr != (unsigned char *)0) \
    _throw_arraybounds(); \
  _curr; })
#endif

#define _tag_fat(tcurr,elt_sz,num_elts) ({ \
  struct _fat_ptr _ans; \
  unsigned _num_elts = (num_elts);\
  _ans.base = _ans.curr = (void*)(tcurr); \
  /* JGM: if we're tagging NULL, ignore num_elts */ \
  _ans.last_plus_one = _ans.base ? (_ans.base + (elt_sz) * _num_elts) : 0; \
  _ans; })

#define _get_fat_size(arr,elt_sz) \
  ({struct _fat_ptr _arr = (arr); \
    unsigned char *_arr_curr=_arr.curr; \
    unsigned char *_arr_last=_arr.last_plus_one; \
    (_arr_curr < _arr.base || _arr_curr >= _arr_last) ? 0 : \
    ((_arr_last - _arr_curr) / (elt_sz));})

#define _fat_ptr_plus(arr,elt_sz,change) ({ \
  struct _fat_ptr _ans = (arr); \
  int _change = (change);\
  _ans.curr += (elt_sz) * _change;\
  _ans; })
#define _fat_ptr_inplace_plus(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  _arr_ptr->curr += (elt_sz) * (change);\
  *_arr_ptr; })
#define _fat_ptr_inplace_plus_post(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  struct _fat_ptr _ans = *_arr_ptr; \
  _arr_ptr->curr += (elt_sz) * (change);\
  _ans; })

//Not a macro since initialization order matters. Defined in runtime_zeroterm.c.
struct _fat_ptr _fat_ptr_decrease_size(struct _fat_ptr,unsigned sz,unsigned numelts);

/* Allocation */
void* GC_malloc(int);
void* GC_malloc_atomic(int);
void* GC_calloc(unsigned,unsigned);
void* GC_calloc_atomic(unsigned,unsigned);
// bound the allocation size to be < MAX_ALLOC_SIZE. See macros below for usage.
#define MAX_MALLOC_SIZE (1 << 28)
void* _bounded_GC_malloc(int,const char*,int);
void* _bounded_GC_malloc_atomic(int,const char*,int);
void* _bounded_GC_calloc(unsigned,unsigned,const char*,int);
void* _bounded_GC_calloc_atomic(unsigned,unsigned,const char*,int);
/* these macros are overridden below ifdef CYC_REGION_PROFILE */
#ifndef CYC_REGION_PROFILE
#define _cycalloc(n) _bounded_GC_malloc(n,__FILE__,__LINE__)
#define _cycalloc_atomic(n) _bounded_GC_malloc_atomic(n,__FILE__,__LINE__)
#define _cyccalloc(n,s) _bounded_GC_calloc(n,s,__FILE__,__LINE__)
#define _cyccalloc_atomic(n,s) _bounded_GC_calloc_atomic(n,s,__FILE__,__LINE__)
#endif

static _INLINE unsigned int _check_times(unsigned x, unsigned y) {
  unsigned long long whole_ans = 
    ((unsigned long long) x)*((unsigned long long)y);
  unsigned word_ans = (unsigned)whole_ans;
  if(word_ans < whole_ans || word_ans > MAX_MALLOC_SIZE)
    _throw_badalloc();
  return word_ans;
}

#define _CYC_MAX_REGION_CONST 2
#define _CYC_MIN_ALIGNMENT (sizeof(double))

#ifdef CYC_REGION_PROFILE
extern int rgn_total_bytes;
#endif

static _INLINE void *_fast_region_malloc(struct _RegionHandle *r, unsigned orig_s) {  
  if (r > (struct _RegionHandle *)_CYC_MAX_REGION_CONST && r->curr != 0) { 
#ifdef CYC_NOALIGN
    unsigned s =  orig_s;
#else
    unsigned s =  (orig_s + _CYC_MIN_ALIGNMENT - 1) & (~(_CYC_MIN_ALIGNMENT -1)); 
#endif
    char *result; 
    result = r->offset; 
    if (s <= (r->last_plus_one - result)) {
      r->offset = result + s; 
#ifdef CYC_REGION_PROFILE
    r->curr->free_bytes = r->curr->free_bytes - s;
    rgn_total_bytes += s;
#endif
      return result;
    }
  } 
  return _region_malloc(r,orig_s); 
}

#ifdef CYC_REGION_PROFILE
/* see macros below for usage. defined in runtime_memory.c */
void* _profile_GC_malloc(int,const char*,const char*,int);
void* _profile_GC_malloc_atomic(int,const char*,const char*,int);
void* _profile_GC_calloc(unsigned,unsigned,const char*,const char*,int);
void* _profile_GC_calloc_atomic(unsigned,unsigned,const char*,const char*,int);
void* _profile_region_malloc(struct _RegionHandle*,unsigned,const char*,const char*,int);
void* _profile_region_calloc(struct _RegionHandle*,unsigned,unsigned,const char *,const char*,int);
struct _RegionHandle _profile_new_region(const char*,const char*,const char*,int);
void _profile_free_region(struct _RegionHandle*,const char*,const char*,int);
#ifndef RUNTIME_CYC
#define _new_region(n) _profile_new_region(n,__FILE__,__FUNCTION__,__LINE__)
#define _free_region(r) _profile_free_region(r,__FILE__,__FUNCTION__,__LINE__)
#define _region_malloc(rh,n) _profile_region_malloc(rh,n,__FILE__,__FUNCTION__,__LINE__)
#define _region_calloc(rh,n,t) _profile_region_calloc(rh,n,t,__FILE__,__FUNCTION__,__LINE__)
#  endif
#define _cycalloc(n) _profile_GC_malloc(n,__FILE__,__FUNCTION__,__LINE__)
#define _cycalloc_atomic(n) _profile_GC_malloc_atomic(n,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc(n,s) _profile_GC_calloc(n,s,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc_atomic(n,s) _profile_GC_calloc_atomic(n,s,__FILE__,__FUNCTION__,__LINE__)
#endif
#endif
 struct Cyc_Core_Opt{void*v;};extern char Cyc_Core_Invalid_argument[17U];struct Cyc_Core_Invalid_argument_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Failure[8U];struct Cyc_Core_Failure_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Impossible[11U];struct Cyc_Core_Impossible_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Not_found[10U];struct Cyc_Core_Not_found_exn_struct{char*tag;};extern char Cyc_Core_Unreachable[12U];struct Cyc_Core_Unreachable_exn_struct{char*tag;struct _fat_ptr f1;};
# 165 "core.h"
extern struct _RegionHandle*Cyc_Core_heap_region;
# 168
extern struct _RegionHandle*Cyc_Core_unique_region;struct Cyc_Core_DynamicRegion;struct Cyc_Core_NewDynamicRegion{struct Cyc_Core_DynamicRegion*key;};struct Cyc_Core_ThinRes{void*arr;unsigned nelts;};struct Cyc_List_List{void*hd;struct Cyc_List_List*tl;};
# 61 "list.h"
int Cyc_List_length(struct Cyc_List_List*x);extern char Cyc_List_List_mismatch[14U];struct Cyc_List_List_mismatch_exn_struct{char*tag;};
# 184
struct Cyc_List_List*Cyc_List_append(struct Cyc_List_List*x,struct Cyc_List_List*y);
# 190
struct Cyc_List_List*Cyc_List_rappend(struct _RegionHandle*,struct Cyc_List_List*x,struct Cyc_List_List*y);extern char Cyc_List_Nth[4U];struct Cyc_List_Nth_exn_struct{char*tag;};
# 242
void*Cyc_List_nth(struct Cyc_List_List*x,int n);
# 319
int Cyc_List_memq(struct Cyc_List_List*l,void*x);
# 322
int Cyc_List_mem(int(*compare)(void*,void*),struct Cyc_List_List*l,void*x);
# 394
struct Cyc_List_List*Cyc_List_filter_c(int(*f)(void*,void*),void*env,struct Cyc_List_List*x);struct Cyc___cycFILE;struct Cyc_String_pa_PrintArg_struct{int tag;struct _fat_ptr f1;};struct Cyc_Int_pa_PrintArg_struct{int tag;unsigned long f1;};struct Cyc_Double_pa_PrintArg_struct{int tag;double f1;};struct Cyc_LongDouble_pa_PrintArg_struct{int tag;long double f1;};struct Cyc_ShortPtr_pa_PrintArg_struct{int tag;short*f1;};struct Cyc_IntPtr_pa_PrintArg_struct{int tag;unsigned long*f1;};
# 98 "cycboot.h"
struct Cyc___cycFILE*Cyc_fopen(const char*,const char*);struct Cyc_ShortPtr_sa_ScanfArg_struct{int tag;short*f1;};struct Cyc_UShortPtr_sa_ScanfArg_struct{int tag;unsigned short*f1;};struct Cyc_IntPtr_sa_ScanfArg_struct{int tag;int*f1;};struct Cyc_UIntPtr_sa_ScanfArg_struct{int tag;unsigned*f1;};struct Cyc_StringPtr_sa_ScanfArg_struct{int tag;struct _fat_ptr f1;};struct Cyc_DoublePtr_sa_ScanfArg_struct{int tag;double*f1;};struct Cyc_FloatPtr_sa_ScanfArg_struct{int tag;float*f1;};struct Cyc_CharPtr_sa_ScanfArg_struct{int tag;struct _fat_ptr f1;};extern char Cyc_FileCloseError[15U];struct Cyc_FileCloseError_exn_struct{char*tag;};extern char Cyc_FileOpenError[14U];struct Cyc_FileOpenError_exn_struct{char*tag;struct _fat_ptr f1;};
# 38 "string.h"
unsigned long Cyc_strlen(struct _fat_ptr s);
# 49 "string.h"
int Cyc_strcmp(struct _fat_ptr s1,struct _fat_ptr s2);
int Cyc_strptrcmp(struct _fat_ptr*s1,struct _fat_ptr*s2);struct Cyc_PP_Ppstate;struct Cyc_PP_Out;struct Cyc_PP_Doc;struct Cyc_Position_Error;struct Cyc_Relations_Reln;struct _union_Nmspace_Rel_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_Abs_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_C_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_Loc_n{int tag;int val;};union Cyc_Absyn_Nmspace{struct _union_Nmspace_Rel_n Rel_n;struct _union_Nmspace_Abs_n Abs_n;struct _union_Nmspace_C_n C_n;struct _union_Nmspace_Loc_n Loc_n;};
# 98 "absyn.h"
union Cyc_Absyn_Nmspace Cyc_Absyn_Abs_n(struct Cyc_List_List*ns,int C_scope);struct _tuple0{union Cyc_Absyn_Nmspace f1;struct _fat_ptr*f2;};
# 161
enum Cyc_Absyn_Scope{Cyc_Absyn_Static =0U,Cyc_Absyn_Abstract =1U,Cyc_Absyn_Public =2U,Cyc_Absyn_Extern =3U,Cyc_Absyn_ExternC =4U,Cyc_Absyn_Register =5U};struct Cyc_Absyn_Tqual{int print_const: 1;int q_volatile: 1;int q_restrict: 1;int real_const: 1;unsigned loc;};
# 182
enum Cyc_Absyn_Size_of{Cyc_Absyn_Char_sz =0U,Cyc_Absyn_Short_sz =1U,Cyc_Absyn_Int_sz =2U,Cyc_Absyn_Long_sz =3U,Cyc_Absyn_LongLong_sz =4U};
enum Cyc_Absyn_Sign{Cyc_Absyn_Signed =0U,Cyc_Absyn_Unsigned =1U,Cyc_Absyn_None =2U};
enum Cyc_Absyn_AggrKind{Cyc_Absyn_StructA =0U,Cyc_Absyn_UnionA =1U};
# 187
enum Cyc_Absyn_AliasQual{Cyc_Absyn_Aliasable =0U,Cyc_Absyn_Unique =1U,Cyc_Absyn_Top =2U};
# 192
enum Cyc_Absyn_KindQual{Cyc_Absyn_AnyKind =0U,Cyc_Absyn_MemKind =1U,Cyc_Absyn_BoxKind =2U,Cyc_Absyn_RgnKind =3U,Cyc_Absyn_EffKind =4U,Cyc_Absyn_IntKind =5U,Cyc_Absyn_BoolKind =6U,Cyc_Absyn_PtrBndKind =7U};struct Cyc_Absyn_Kind{enum Cyc_Absyn_KindQual kind;enum Cyc_Absyn_AliasQual aliasqual;};struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct{int tag;struct Cyc_Absyn_Kind*f1;};struct Cyc_Absyn_Unknown_kb_Absyn_KindBound_struct{int tag;struct Cyc_Core_Opt*f1;};struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct{int tag;struct Cyc_Core_Opt*f1;struct Cyc_Absyn_Kind*f2;};struct Cyc_Absyn_Tvar{struct _fat_ptr*name;int identity;void*kind;};struct Cyc_Absyn_PtrLoc{unsigned ptr_loc;unsigned rgn_loc;unsigned zt_loc;};struct Cyc_Absyn_PtrAtts{void*rgn;void*nullable;void*bounds;void*zero_term;struct Cyc_Absyn_PtrLoc*ptrloc;};struct Cyc_Absyn_PtrInfo{void*elt_type;struct Cyc_Absyn_Tqual elt_tq;struct Cyc_Absyn_PtrAtts ptr_atts;};struct Cyc_Absyn_VarargInfo{struct _fat_ptr*name;struct Cyc_Absyn_Tqual tq;void*type;int inject;};struct Cyc_Absyn_Star_Absyn_Cap_struct{int tag;};struct Cyc_Absyn_Bot_Absyn_Cap_struct{int tag;};struct Cyc_Absyn_Nat_Absyn_Cap_struct{int tag;struct Cyc_Absyn_Exp*f1;int f2;};struct Cyc_Absyn_RgnEffect{void*name;void*rgn;void*lock;void*unadopt;void*parent;};struct Cyc_Absyn_FnInfo{struct Cyc_List_List*tvars;void*effect;struct Cyc_Absyn_Tqual ret_tqual;void*ret_type;struct Cyc_List_List*args;int c_varargs;struct Cyc_Absyn_VarargInfo*cyc_varargs;struct Cyc_List_List*rgn_po;struct Cyc_List_List*attributes;struct Cyc_Absyn_Exp*requires_clause;struct Cyc_List_List*requires_relns;struct Cyc_Absyn_Exp*ensures_clause;struct Cyc_List_List*ensures_relns;struct Cyc_List_List*ieffect;struct Cyc_List_List*oeffect;};struct Cyc_Absyn_UnknownDatatypeInfo{struct _tuple0*name;int is_extensible;};struct _union_DatatypeInfo_UnknownDatatype{int tag;struct Cyc_Absyn_UnknownDatatypeInfo val;};struct _union_DatatypeInfo_KnownDatatype{int tag;struct Cyc_Absyn_Datatypedecl**val;};union Cyc_Absyn_DatatypeInfo{struct _union_DatatypeInfo_UnknownDatatype UnknownDatatype;struct _union_DatatypeInfo_KnownDatatype KnownDatatype;};struct Cyc_Absyn_UnknownDatatypeFieldInfo{struct _tuple0*datatype_name;struct _tuple0*field_name;int is_extensible;};struct _union_DatatypeFieldInfo_UnknownDatatypefield{int tag;struct Cyc_Absyn_UnknownDatatypeFieldInfo val;};struct _tuple1{struct Cyc_Absyn_Datatypedecl*f1;struct Cyc_Absyn_Datatypefield*f2;};struct _union_DatatypeFieldInfo_KnownDatatypefield{int tag;struct _tuple1 val;};union Cyc_Absyn_DatatypeFieldInfo{struct _union_DatatypeFieldInfo_UnknownDatatypefield UnknownDatatypefield;struct _union_DatatypeFieldInfo_KnownDatatypefield KnownDatatypefield;};struct _tuple2{enum Cyc_Absyn_AggrKind f1;struct _tuple0*f2;struct Cyc_Core_Opt*f3;};struct _union_AggrInfo_UnknownAggr{int tag;struct _tuple2 val;};struct _union_AggrInfo_KnownAggr{int tag;struct Cyc_Absyn_Aggrdecl**val;};union Cyc_Absyn_AggrInfo{struct _union_AggrInfo_UnknownAggr UnknownAggr;struct _union_AggrInfo_KnownAggr KnownAggr;};struct Cyc_Absyn_ArrayInfo{void*elt_type;struct Cyc_Absyn_Tqual tq;struct Cyc_Absyn_Exp*num_elts;void*zero_term;unsigned zt_loc;};struct Cyc_Absyn_Aggr_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Aggrdecl*f1;};struct Cyc_Absyn_Enum_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;};struct Cyc_Absyn_Datatype_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;};struct Cyc_Absyn_TypeDecl{void*r;unsigned loc;};struct Cyc_Absyn_VoidCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_IntCon_Absyn_TyCon_struct{int tag;enum Cyc_Absyn_Sign f1;enum Cyc_Absyn_Size_of f2;};struct Cyc_Absyn_FloatCon_Absyn_TyCon_struct{int tag;int f1;};struct Cyc_Absyn_RgnHandleCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_TagCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_HeapCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_UniqueCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_RefCntCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_AccessCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_JoinCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_RgnsCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_TrueCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_FalseCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_ThinCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_FatCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_EnumCon_Absyn_TyCon_struct{int tag;struct _tuple0*f1;struct Cyc_Absyn_Enumdecl*f2;};struct Cyc_Absyn_AnonEnumCon_Absyn_TyCon_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_BuiltinCon_Absyn_TyCon_struct{int tag;struct _fat_ptr f1;struct Cyc_Absyn_Kind*f2;};struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_DatatypeInfo f1;};struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_DatatypeFieldInfo f1;};struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_AggrInfo f1;};struct Cyc_Absyn_AppType_Absyn_Type_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Evar_Absyn_Type_struct{int tag;struct Cyc_Core_Opt*f1;void*f2;int f3;struct Cyc_Core_Opt*f4;};struct Cyc_Absyn_VarType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Tvar*f1;};struct Cyc_Absyn_PointerType_Absyn_Type_struct{int tag;struct Cyc_Absyn_PtrInfo f1;};struct Cyc_Absyn_ArrayType_Absyn_Type_struct{int tag;struct Cyc_Absyn_ArrayInfo f1;};struct Cyc_Absyn_FnType_Absyn_Type_struct{int tag;struct Cyc_Absyn_FnInfo f1;};struct Cyc_Absyn_TupleType_Absyn_Type_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct{int tag;enum Cyc_Absyn_AggrKind f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_TypedefType_Absyn_Type_struct{int tag;struct _tuple0*f1;struct Cyc_List_List*f2;struct Cyc_Absyn_Typedefdecl*f3;void*f4;};struct Cyc_Absyn_ValueofType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct{int tag;struct Cyc_Absyn_TypeDecl*f1;void**f2;};struct Cyc_Absyn_TypeofType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_NoTypes_Absyn_Funcparams_struct{int tag;struct Cyc_List_List*f1;unsigned f2;};struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct{int tag;struct Cyc_List_List*f1;int f2;struct Cyc_Absyn_VarargInfo*f3;void*f4;struct Cyc_List_List*f5;struct Cyc_Absyn_Exp*f6;struct Cyc_Absyn_Exp*f7;struct Cyc_List_List*f8;struct Cyc_List_List*f9;};
# 437 "absyn.h"
enum Cyc_Absyn_Format_Type{Cyc_Absyn_Printf_ft =0U,Cyc_Absyn_Scanf_ft =1U};struct Cyc_Absyn_Regparm_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Stdcall_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Cdecl_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Fastcall_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Noreturn_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Const_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Aligned_att_Absyn_Attribute_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Packed_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Section_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Nocommon_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Shared_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Unused_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Weak_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Dllimport_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Dllexport_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_instrument_function_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Constructor_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Destructor_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_check_memory_usage_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Format_att_Absyn_Attribute_struct{int tag;enum Cyc_Absyn_Format_Type f1;int f2;int f3;};struct Cyc_Absyn_Initializes_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Noliveunique_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Consume_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Pure_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Mode_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Alias_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Always_inline_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_throw_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Non_null_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Deprecated_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Carray_mod_Absyn_Type_modifier_struct{int tag;void*f1;unsigned f2;};struct Cyc_Absyn_ConstArray_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_Absyn_Exp*f1;void*f2;unsigned f3;};struct Cyc_Absyn_Pointer_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_Absyn_PtrAtts f1;struct Cyc_Absyn_Tqual f2;};struct Cyc_Absyn_Function_mod_Absyn_Type_modifier_struct{int tag;void*f1;};struct Cyc_Absyn_TypeParams_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_List_List*f1;unsigned f2;int f3;};struct Cyc_Absyn_Attributes_mod_Absyn_Type_modifier_struct{int tag;unsigned f1;struct Cyc_List_List*f2;};struct _union_Cnst_Null_c{int tag;int val;};struct _tuple3{enum Cyc_Absyn_Sign f1;char f2;};struct _union_Cnst_Char_c{int tag;struct _tuple3 val;};struct _union_Cnst_Wchar_c{int tag;struct _fat_ptr val;};struct _tuple4{enum Cyc_Absyn_Sign f1;short f2;};struct _union_Cnst_Short_c{int tag;struct _tuple4 val;};struct _tuple5{enum Cyc_Absyn_Sign f1;int f2;};struct _union_Cnst_Int_c{int tag;struct _tuple5 val;};struct _tuple6{enum Cyc_Absyn_Sign f1;long long f2;};struct _union_Cnst_LongLong_c{int tag;struct _tuple6 val;};struct _tuple7{struct _fat_ptr f1;int f2;};struct _union_Cnst_Float_c{int tag;struct _tuple7 val;};struct _union_Cnst_String_c{int tag;struct _fat_ptr val;};struct _union_Cnst_Wstring_c{int tag;struct _fat_ptr val;};union Cyc_Absyn_Cnst{struct _union_Cnst_Null_c Null_c;struct _union_Cnst_Char_c Char_c;struct _union_Cnst_Wchar_c Wchar_c;struct _union_Cnst_Short_c Short_c;struct _union_Cnst_Int_c Int_c;struct _union_Cnst_LongLong_c LongLong_c;struct _union_Cnst_Float_c Float_c;struct _union_Cnst_String_c String_c;struct _union_Cnst_Wstring_c Wstring_c;};
# 532
enum Cyc_Absyn_Primop{Cyc_Absyn_Plus =0U,Cyc_Absyn_Times =1U,Cyc_Absyn_Minus =2U,Cyc_Absyn_Div =3U,Cyc_Absyn_Mod =4U,Cyc_Absyn_Eq =5U,Cyc_Absyn_Neq =6U,Cyc_Absyn_Gt =7U,Cyc_Absyn_Lt =8U,Cyc_Absyn_Gte =9U,Cyc_Absyn_Lte =10U,Cyc_Absyn_Not =11U,Cyc_Absyn_Bitnot =12U,Cyc_Absyn_Bitand =13U,Cyc_Absyn_Bitor =14U,Cyc_Absyn_Bitxor =15U,Cyc_Absyn_Bitlshift =16U,Cyc_Absyn_Bitlrshift =17U,Cyc_Absyn_Numelts =18U};
# 539
enum Cyc_Absyn_Incrementor{Cyc_Absyn_PreInc =0U,Cyc_Absyn_PostInc =1U,Cyc_Absyn_PreDec =2U,Cyc_Absyn_PostDec =3U};struct Cyc_Absyn_VarargCallInfo{int num_varargs;struct Cyc_List_List*injectors;struct Cyc_Absyn_VarargInfo*vai;};struct Cyc_Absyn_StructField_Absyn_OffsetofField_struct{int tag;struct _fat_ptr*f1;};struct Cyc_Absyn_TupleIndex_Absyn_OffsetofField_struct{int tag;unsigned f1;};
# 557
enum Cyc_Absyn_Coercion{Cyc_Absyn_Unknown_coercion =0U,Cyc_Absyn_No_coercion =1U,Cyc_Absyn_Null_to_NonNull =2U,Cyc_Absyn_Other_coercion =3U};struct Cyc_Absyn_MallocInfo{int is_calloc;struct Cyc_Absyn_Exp*rgn;void**elt_type;struct Cyc_Absyn_Exp*num_elts;int fat_result;int inline_call;};struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct{int tag;union Cyc_Absyn_Cnst f1;};struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Pragma_e_Absyn_Raw_exp_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct{int tag;enum Cyc_Absyn_Primop f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_AssignOp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Core_Opt*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_Increment_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;enum Cyc_Absyn_Incrementor f2;};struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;struct Cyc_Absyn_VarargCallInfo*f3;int f4;};struct Cyc_Absyn_Throw_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;int f2;};struct Cyc_Absyn_NoInstantiate_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Instantiate_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_Absyn_Exp*f2;int f3;enum Cyc_Absyn_Coercion f4;};struct Cyc_Absyn_Address_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_New_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Sizeoftype_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Sizeofexp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Offsetof_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Deref_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;int f3;int f4;};struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;int f3;int f4;};struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Tuple_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;};struct _tuple8{struct _fat_ptr*f1;struct Cyc_Absyn_Tqual f2;void*f3;};struct Cyc_Absyn_CompoundLit_e_Absyn_Raw_exp_struct{int tag;struct _tuple8*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Array_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Exp*f2;struct Cyc_Absyn_Exp*f3;int f4;};struct Cyc_Absyn_ComprehensionNoinit_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;void*f2;int f3;};struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct{int tag;struct _tuple0*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;struct Cyc_Absyn_Aggrdecl*f4;};struct Cyc_Absyn_AnonStruct_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Datatype_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;struct Cyc_Absyn_Datatypedecl*f2;struct Cyc_Absyn_Datatypefield*f3;};struct Cyc_Absyn_Enum_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_AnonEnum_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_Spawn_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_MallocInfo f1;};struct Cyc_Absyn_Swap_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_UnresolvedMem_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Core_Opt*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_StmtExp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Stmt*f1;};struct Cyc_Absyn_Tagcheck_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;};struct Cyc_Absyn_Valueof_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Asm_e_Absyn_Raw_exp_struct{int tag;int f1;struct _fat_ptr f2;struct Cyc_List_List*f3;struct Cyc_List_List*f4;struct Cyc_List_List*f5;};struct Cyc_Absyn_Extension_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Exp{void*topt;void*r;unsigned loc;void*annot;};struct Cyc_Absyn_Skip_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Exp_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Seq_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Return_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_IfThenElse_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Stmt*f2;struct Cyc_Absyn_Stmt*f3;};struct _tuple9{struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_While_s_Absyn_Raw_stmt_struct{int tag;struct _tuple9 f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Break_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Continue_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Goto_s_Absyn_Raw_stmt_struct{int tag;struct _fat_ptr*f1;};struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _tuple9 f2;struct _tuple9 f3;struct Cyc_Absyn_Stmt*f4;};struct Cyc_Absyn_Switch_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;void*f3;};struct Cyc_Absyn_Fallthru_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_List_List*f1;struct Cyc_Absyn_Switch_clause**f2;};struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Decl*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Label_s_Absyn_Raw_stmt_struct{int tag;struct _fat_ptr*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Do_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct _tuple9 f2;};struct Cyc_Absyn_TryCatch_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct Cyc_List_List*f2;void*f3;};struct Cyc_Absyn_Stmt{void*r;unsigned loc;void*annot;};struct Cyc_Absyn_Wild_p_Absyn_Raw_pat_struct{int tag;};struct Cyc_Absyn_Var_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Pat*f2;};struct Cyc_Absyn_AliasVar_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;};struct Cyc_Absyn_Reference_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Pat*f2;};struct Cyc_Absyn_TagInt_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;};struct Cyc_Absyn_Tuple_p_Absyn_Raw_pat_struct{int tag;struct Cyc_List_List*f1;int f2;};struct Cyc_Absyn_Pointer_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Pat*f1;};struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct{int tag;union Cyc_Absyn_AggrInfo*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;int f4;};struct Cyc_Absyn_Datatype_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;struct Cyc_Absyn_Datatypefield*f2;struct Cyc_List_List*f3;int f4;};struct Cyc_Absyn_Null_p_Absyn_Raw_pat_struct{int tag;};struct Cyc_Absyn_Int_p_Absyn_Raw_pat_struct{int tag;enum Cyc_Absyn_Sign f1;int f2;};struct Cyc_Absyn_Char_p_Absyn_Raw_pat_struct{int tag;char f1;};struct Cyc_Absyn_Float_p_Absyn_Raw_pat_struct{int tag;struct _fat_ptr f1;int f2;};struct Cyc_Absyn_Enum_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_AnonEnum_p_Absyn_Raw_pat_struct{int tag;void*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_UnknownId_p_Absyn_Raw_pat_struct{int tag;struct _tuple0*f1;};struct Cyc_Absyn_UnknownCall_p_Absyn_Raw_pat_struct{int tag;struct _tuple0*f1;struct Cyc_List_List*f2;int f3;};struct Cyc_Absyn_Exp_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Pat{void*r;void*topt;unsigned loc;};struct Cyc_Absyn_Switch_clause{struct Cyc_Absyn_Pat*pattern;struct Cyc_Core_Opt*pat_vars;struct Cyc_Absyn_Exp*where_clause;struct Cyc_Absyn_Stmt*body;unsigned loc;};struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct{int tag;struct _tuple0*f1;};struct Cyc_Absyn_Global_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Funname_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Fndecl*f1;};struct Cyc_Absyn_Param_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Local_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Pat_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Vardecl{enum Cyc_Absyn_Scope sc;struct _tuple0*name;unsigned varloc;struct Cyc_Absyn_Tqual tq;void*type;struct Cyc_Absyn_Exp*initializer;void*rgn;struct Cyc_List_List*attributes;int escapes;};struct Cyc_Absyn_Fndecl{enum Cyc_Absyn_Scope sc;int is_inline;struct _tuple0*name;struct Cyc_Absyn_Stmt*body;struct Cyc_Absyn_FnInfo i;void*cached_type;struct Cyc_Core_Opt*param_vardecls;struct Cyc_Absyn_Vardecl*fn_vardecl;};struct Cyc_Absyn_Aggrfield{struct _fat_ptr*name;struct Cyc_Absyn_Tqual tq;void*type;struct Cyc_Absyn_Exp*width;struct Cyc_List_List*attributes;struct Cyc_Absyn_Exp*requires_clause;};struct Cyc_Absyn_AggrdeclImpl{struct Cyc_List_List*exist_vars;struct Cyc_List_List*rgn_po;struct Cyc_List_List*fields;int tagged;};struct Cyc_Absyn_Aggrdecl{enum Cyc_Absyn_AggrKind kind;enum Cyc_Absyn_Scope sc;struct _tuple0*name;struct Cyc_List_List*tvs;struct Cyc_Absyn_AggrdeclImpl*impl;struct Cyc_List_List*attributes;int expected_mem_kind;};struct Cyc_Absyn_Datatypefield{struct _tuple0*name;struct Cyc_List_List*typs;unsigned loc;enum Cyc_Absyn_Scope sc;};struct Cyc_Absyn_Datatypedecl{enum Cyc_Absyn_Scope sc;struct _tuple0*name;struct Cyc_List_List*tvs;struct Cyc_Core_Opt*fields;int is_extensible;};struct Cyc_Absyn_Enumfield{struct _tuple0*name;struct Cyc_Absyn_Exp*tag;unsigned loc;};struct Cyc_Absyn_Enumdecl{enum Cyc_Absyn_Scope sc;struct _tuple0*name;struct Cyc_Core_Opt*fields;};struct Cyc_Absyn_Typedefdecl{struct _tuple0*name;struct Cyc_Absyn_Tqual tq;struct Cyc_List_List*tvs;struct Cyc_Core_Opt*kind;void*defn;struct Cyc_List_List*atts;int extern_c;};struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Fn_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Fndecl*f1;};struct Cyc_Absyn_Let_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Pat*f1;struct Cyc_Core_Opt*f2;struct Cyc_Absyn_Exp*f3;void*f4;};struct Cyc_Absyn_Letv_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_Region_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_Aggr_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Aggrdecl*f1;};struct Cyc_Absyn_Datatype_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;};struct Cyc_Absyn_Enum_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;};struct Cyc_Absyn_Typedef_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Typedefdecl*f1;};struct Cyc_Absyn_Namespace_d_Absyn_Raw_decl_struct{int tag;struct _fat_ptr*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Using_d_Absyn_Raw_decl_struct{int tag;struct _tuple0*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_ExternC_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;};struct _tuple10{unsigned f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_ExternCinclude_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;struct _tuple10*f4;};struct Cyc_Absyn_Porton_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Portoff_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Tempeston_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Tempestoff_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Decl{void*r;unsigned loc;};struct Cyc_Absyn_ArrayElement_Absyn_Designator_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_FieldName_Absyn_Designator_struct{int tag;struct _fat_ptr*f1;};extern char Cyc_Absyn_EmptyAnnot[11U];struct Cyc_Absyn_EmptyAnnot_Absyn_AbsynAnnot_struct{char*tag;};
# 908 "absyn.h"
int Cyc_Absyn_qvar_cmp(struct _tuple0*,struct _tuple0*);
# 916
struct Cyc_Absyn_Tqual Cyc_Absyn_const_tqual(unsigned);
struct Cyc_Absyn_Tqual Cyc_Absyn_empty_tqual(unsigned);
# 922
void*Cyc_Absyn_compress_kb(void*);
# 933
void*Cyc_Absyn_new_evar(struct Cyc_Core_Opt*k,struct Cyc_Core_Opt*tenv);
# 973
void*Cyc_Absyn_string_type(void*rgn);
void*Cyc_Absyn_const_string_type(void*rgn);
# 996
void*Cyc_Absyn_fatptr_type(void*t,void*rgn,struct Cyc_Absyn_Tqual,void*zeroterm);
# 1004
void*Cyc_Absyn_array_type(void*elt_type,struct Cyc_Absyn_Tqual tq,struct Cyc_Absyn_Exp*num_elts,void*zero_term,unsigned ztloc);
# 1022
struct Cyc_Absyn_Exp*Cyc_Absyn_uint_exp(unsigned,unsigned);
# 1078
struct _tuple0*Cyc_Absyn_uniquergn_qvar();
# 1164
struct _fat_ptr Cyc_Absyn_attribute2string(void*);struct Cyc_Absynpp_Params{int expand_typedefs;int qvar_to_Cids;int add_cyc_prefix;int to_VC;int decls_first;int rewrite_temp_tvars;int print_all_tvars;int print_all_kinds;int print_all_effects;int print_using_stmts;int print_externC_stmts;int print_full_evars;int print_zeroterm;int generate_line_directives;int use_curr_namespace;struct Cyc_List_List*curr_namespace;};
# 53 "absynpp.h"
void Cyc_Absynpp_set_params(struct Cyc_Absynpp_Params*fs);
# 55
extern struct Cyc_Absynpp_Params Cyc_Absynpp_tc_params_r;
# 62
struct _fat_ptr Cyc_Absynpp_typ2string(void*);
# 64
struct _fat_ptr Cyc_Absynpp_kind2string(struct Cyc_Absyn_Kind*);
# 69
struct _fat_ptr Cyc_Absynpp_qvar2string(struct _tuple0*);
# 27 "unify.h"
void Cyc_Unify_explain_failure();
# 29
int Cyc_Unify_unify(void*,void*);struct _union_RelnOp_RConst{int tag;unsigned val;};struct _union_RelnOp_RVar{int tag;struct Cyc_Absyn_Vardecl*val;};struct _union_RelnOp_RNumelts{int tag;struct Cyc_Absyn_Vardecl*val;};struct _union_RelnOp_RType{int tag;void*val;};struct _union_RelnOp_RParam{int tag;unsigned val;};struct _union_RelnOp_RParamNumelts{int tag;unsigned val;};struct _union_RelnOp_RReturn{int tag;unsigned val;};union Cyc_Relations_RelnOp{struct _union_RelnOp_RConst RConst;struct _union_RelnOp_RVar RVar;struct _union_RelnOp_RNumelts RNumelts;struct _union_RelnOp_RType RType;struct _union_RelnOp_RParam RParam;struct _union_RelnOp_RParamNumelts RParamNumelts;struct _union_RelnOp_RReturn RReturn;};
# 50 "relations-ap.h"
enum Cyc_Relations_Relation{Cyc_Relations_Req =0U,Cyc_Relations_Rneq =1U,Cyc_Relations_Rlte =2U,Cyc_Relations_Rlt =3U};struct Cyc_Relations_Reln{union Cyc_Relations_RelnOp rop1;enum Cyc_Relations_Relation relation;union Cyc_Relations_RelnOp rop2;};
# 84
struct Cyc_List_List*Cyc_Relations_exp2relns(struct _RegionHandle*r,struct Cyc_Absyn_Exp*e);
# 129
int Cyc_Relations_consistent_relations(struct Cyc_List_List*rlns);struct Cyc_Iter_Iter{void*env;int(*next)(void*env,void*dest);};struct Cyc_Dict_T;struct Cyc_Dict_Dict{int(*rel)(void*,void*);struct _RegionHandle*r;const struct Cyc_Dict_T*t;};extern char Cyc_Dict_Present[8U];struct Cyc_Dict_Present_exn_struct{char*tag;};extern char Cyc_Dict_Absent[7U];struct Cyc_Dict_Absent_exn_struct{char*tag;};
# 83 "dict.h"
int Cyc_Dict_member(struct Cyc_Dict_Dict d,void*k);
# 87
struct Cyc_Dict_Dict Cyc_Dict_insert(struct Cyc_Dict_Dict d,void*k,void*v);
# 110
void*Cyc_Dict_lookup(struct Cyc_Dict_Dict d,void*k);
# 122 "dict.h"
void**Cyc_Dict_lookup_opt(struct Cyc_Dict_Dict d,void*k);struct Cyc_RgnOrder_RgnPO;extern char Cyc_Tcenv_Env_error[10U];struct Cyc_Tcenv_Env_error_exn_struct{char*tag;};struct Cyc_Tcenv_Genv{struct Cyc_Dict_Dict aggrdecls;struct Cyc_Dict_Dict datatypedecls;struct Cyc_Dict_Dict enumdecls;struct Cyc_Dict_Dict typedefs;struct Cyc_Dict_Dict ordinaries;};struct Cyc_Tcenv_Fenv;struct Cyc_Tcenv_Tenv{struct Cyc_List_List*ns;struct Cyc_Tcenv_Genv*ae;struct Cyc_Tcenv_Fenv*le;int allow_valueof: 1;int in_extern_c_include: 1;int in_tempest: 1;int tempest_generalize: 1;};
# 69 "tcenv.h"
struct Cyc_Tcenv_Fenv*Cyc_Tcenv_new_fenv(unsigned,struct Cyc_Absyn_Fndecl*);
# 78
struct Cyc_Absyn_Datatypedecl***Cyc_Tcenv_lookup_xdatatypedecl(struct _RegionHandle*,struct Cyc_Tcenv_Tenv*,unsigned,struct _tuple0*);
# 82
struct Cyc_Tcenv_Tenv*Cyc_Tcenv_allow_valueof(struct Cyc_Tcenv_Tenv*);
struct Cyc_Tcenv_Tenv*Cyc_Tcenv_enter_extern_c_include(struct Cyc_Tcenv_Tenv*);
struct Cyc_Tcenv_Tenv*Cyc_Tcenv_enter_tempest(struct Cyc_Tcenv_Tenv*);
struct Cyc_Tcenv_Tenv*Cyc_Tcenv_clear_tempest(struct Cyc_Tcenv_Tenv*);
# 88
enum Cyc_Tcenv_NewStatus{Cyc_Tcenv_NoneNew =0U,Cyc_Tcenv_InNew =1U,Cyc_Tcenv_InNewAggr =2U};
# 148
void Cyc_Tcenv_check_delayed_effects(struct Cyc_Tcenv_Tenv*);
void Cyc_Tcenv_check_delayed_constraints(struct Cyc_Tcenv_Tenv*);
# 30 "tcutil.h"
void*Cyc_Tcutil_impos(struct _fat_ptr,struct _fat_ptr);
void Cyc_Tcutil_terr(unsigned,struct _fat_ptr,struct _fat_ptr);
void Cyc_Tcutil_warn(unsigned,struct _fat_ptr,struct _fat_ptr);
# 43
int Cyc_Tcutil_is_function_type(void*);
# 45
int Cyc_Tcutil_is_array_type(void*);
# 52
int Cyc_Tcutil_is_bound_one(void*);
# 57
int Cyc_Tcutil_is_bits_only_type(void*);
# 87
int Cyc_Tcutil_is_integral(struct Cyc_Absyn_Exp*);
# 107
void*Cyc_Tcutil_compress(void*);
# 114
int Cyc_Tcutil_coerce_assign(struct Cyc_Tcenv_Tenv*te,struct Cyc_Absyn_Exp*,void*);
# 135
extern struct Cyc_Absyn_Kind Cyc_Tcutil_rk;
extern struct Cyc_Absyn_Kind Cyc_Tcutil_ak;
extern struct Cyc_Absyn_Kind Cyc_Tcutil_bk;
# 139
extern struct Cyc_Absyn_Kind Cyc_Tcutil_ek;
extern struct Cyc_Absyn_Kind Cyc_Tcutil_ik;
# 144
extern struct Cyc_Absyn_Kind Cyc_Tcutil_trk;
extern struct Cyc_Absyn_Kind Cyc_Tcutil_tak;
extern struct Cyc_Absyn_Kind Cyc_Tcutil_tbk;
extern struct Cyc_Absyn_Kind Cyc_Tcutil_tmk;
# 174
void*Cyc_Tcutil_kind_to_bound(struct Cyc_Absyn_Kind*k);
# 202
void*Cyc_Tcutil_fndecl2type(struct Cyc_Absyn_Fndecl*);
# 212
void Cyc_Tcutil_check_bitfield(unsigned,void*field_typ,struct Cyc_Absyn_Exp*width,struct _fat_ptr*fn);
# 215
void Cyc_Tcutil_check_unique_tvars(unsigned,struct Cyc_List_List*);
# 236
int Cyc_Tcutil_is_noalias_pointer(void*t,int must_be_unique);
# 246
int Cyc_Tcutil_is_noalias_pointer_or_aggr(void*);
# 261
void Cyc_Tcutil_add_tvar_identities(struct Cyc_List_List*);
# 273
int Cyc_Tcutil_is_const_exp(struct Cyc_Absyn_Exp*);
# 280
int Cyc_Tcutil_extract_const_from_typedef(unsigned,int declared_const,void*);
# 284
struct Cyc_List_List*Cyc_Tcutil_transfer_fn_type_atts(void*,struct Cyc_List_List*);
# 301
int Cyc_Tcutil_zeroable_type(void*);
# 305
int Cyc_Tcutil_force_type2bool(int desired,void*);
# 308
void*Cyc_Tcutil_any_bool(struct Cyc_List_List*);
# 34 "tctyp.h"
void Cyc_Tctyp_check_valid_toplevel_type(unsigned,struct Cyc_Tcenv_Tenv*,void*);
void Cyc_Tctyp_check_fndecl_valid_type(unsigned,struct Cyc_Tcenv_Tenv*,struct Cyc_Absyn_Fndecl*);
# 44 "tctyp.h"
void Cyc_Tctyp_check_type(unsigned,struct Cyc_Tcenv_Tenv*,struct Cyc_List_List*bound_tvars,struct Cyc_Absyn_Kind*k,int allow_evars,int allow_abs_aggr,void*);
# 28 "tcexp.h"
void*Cyc_Tcexp_tcExp(struct Cyc_Tcenv_Tenv*,void**,struct Cyc_Absyn_Exp*);
void*Cyc_Tcexp_tcExpInitializer(struct Cyc_Tcenv_Tenv*,void**,struct Cyc_Absyn_Exp*);
# 26 "tcstmt.h"
void Cyc_Tcstmt_tcStmt(struct Cyc_Tcenv_Tenv*,struct Cyc_Absyn_Stmt*,int new_block);struct _tuple11{unsigned f1;int f2;};
# 28 "evexp.h"
struct _tuple11 Cyc_Evexp_eval_const_uint_exp(struct Cyc_Absyn_Exp*e);extern char Cyc_Tcdecl_Incompatible[13U];struct Cyc_Tcdecl_Incompatible_exn_struct{char*tag;};struct Cyc_Tcdecl_Xdatatypefielddecl{struct Cyc_Absyn_Datatypedecl*base;struct Cyc_Absyn_Datatypefield*field;};
# 54 "tcdecl.h"
struct Cyc_Absyn_Aggrdecl*Cyc_Tcdecl_merge_aggrdecl(struct Cyc_Absyn_Aggrdecl*,struct Cyc_Absyn_Aggrdecl*,unsigned,struct _fat_ptr*msg);
# 57
struct Cyc_Absyn_Datatypedecl*Cyc_Tcdecl_merge_datatypedecl(struct Cyc_Absyn_Datatypedecl*,struct Cyc_Absyn_Datatypedecl*,unsigned,struct _fat_ptr*msg);
# 59
struct Cyc_Absyn_Enumdecl*Cyc_Tcdecl_merge_enumdecl(struct Cyc_Absyn_Enumdecl*,struct Cyc_Absyn_Enumdecl*,unsigned,struct _fat_ptr*msg);
# 66
void*Cyc_Tcdecl_merge_binding(void*,void*,unsigned,struct _fat_ptr*msg);
# 74
struct Cyc_List_List*Cyc_Tcdecl_sort_xdatatype_fields(struct Cyc_List_List*f,int*res,struct _fat_ptr*v,unsigned,struct _fat_ptr*msg);struct Cyc_Set_Set;extern char Cyc_Set_Absent[7U];struct Cyc_Set_Absent_exn_struct{char*tag;};
# 92 "graph.h"
struct Cyc_Dict_Dict Cyc_Graph_scc(struct Cyc_Dict_Dict g);
# 30 "callgraph.h"
struct Cyc_Dict_Dict Cyc_Callgraph_compute_callgraph(struct Cyc_List_List*);
void Cyc_Callgraph_print_callgraph(struct Cyc___cycFILE*,struct Cyc_Dict_Dict);
# 31 "tc.h"
extern int Cyc_Tc_aggressive_warn;
# 40
void Cyc_Tc_tcAggrdecl(struct Cyc_Tcenv_Tenv*,unsigned,struct Cyc_Absyn_Aggrdecl*);
void Cyc_Tc_tcDatatypedecl(struct Cyc_Tcenv_Tenv*,unsigned,struct Cyc_Absyn_Datatypedecl*);
void Cyc_Tc_tcEnumdecl(struct Cyc_Tcenv_Tenv*,unsigned,struct Cyc_Absyn_Enumdecl*);static char _tmp0[1U]="";
# 41 "tc.cyc"
static struct _fat_ptr Cyc_Tc_tc_msg_c={_tmp0,_tmp0,_tmp0 + 1U};
static struct _fat_ptr*Cyc_Tc_tc_msg=& Cyc_Tc_tc_msg_c;
# 45
int Cyc_Tc_aggressive_warn=0;struct _tuple12{unsigned f1;struct _tuple0*f2;int f3;};
# 47
static int Cyc_Tc_export_member(struct _tuple0*x,struct Cyc_List_List*exports){
for(0;exports != 0;exports=exports->tl){
struct _tuple12*p=(struct _tuple12*)exports->hd;
if(Cyc_Absyn_qvar_cmp(x,(*p).f2)== 0){
# 53
(*p).f3=1;
return 1;}}
# 57
return 0;}
# 47
static int Cyc_Tc_fnTypeAttsRangeOK(unsigned loc,int i,int nargs,void*att){
# 61
if(i < 1 || i > nargs){
({struct Cyc_String_pa_PrintArg_struct _tmp3=({struct Cyc_String_pa_PrintArg_struct _tmp19A;_tmp19A.tag=0U,({struct _fat_ptr _tmp1D8=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absyn_attribute2string(att));_tmp19A.f1=_tmp1D8;});_tmp19A;});void*_tmp1[1U];_tmp1[0]=& _tmp3;({unsigned _tmp1DA=loc;struct _fat_ptr _tmp1D9=({const char*_tmp2="%s has an out-of-range index";_tag_fat(_tmp2,sizeof(char),29U);});Cyc_Tcutil_terr(_tmp1DA,_tmp1D9,_tag_fat(_tmp1,sizeof(void*),1U));});});
return 0;}
# 61
return 1;}struct _tuple13{struct Cyc_List_List*f1;struct _fat_ptr f2;};
# 47
static void Cyc_Tc_fnTypeAttsOverlap(unsigned loc,int i,struct _tuple13 lst1,struct _tuple13 lst2){
# 70
if(({int(*_tmp1DC)(struct Cyc_List_List*l,int x)=({int(*_tmp4)(struct Cyc_List_List*l,int x)=(int(*)(struct Cyc_List_List*l,int x))Cyc_List_memq;_tmp4;});struct Cyc_List_List*_tmp1DB=lst2.f1;_tmp1DC(_tmp1DB,i);}))
({struct Cyc_String_pa_PrintArg_struct _tmp7=({struct Cyc_String_pa_PrintArg_struct _tmp19D;_tmp19D.tag=0U,_tmp19D.f1=(struct _fat_ptr)((struct _fat_ptr)lst1.f2);_tmp19D;});struct Cyc_String_pa_PrintArg_struct _tmp8=({struct Cyc_String_pa_PrintArg_struct _tmp19C;_tmp19C.tag=0U,_tmp19C.f1=(struct _fat_ptr)((struct _fat_ptr)lst2.f2);_tmp19C;});struct Cyc_Int_pa_PrintArg_struct _tmp9=({struct Cyc_Int_pa_PrintArg_struct _tmp19B;_tmp19B.tag=1U,_tmp19B.f1=(unsigned long)i;_tmp19B;});void*_tmp5[3U];_tmp5[0]=& _tmp7,_tmp5[1]=& _tmp8,_tmp5[2]=& _tmp9;({unsigned _tmp1DE=loc;struct _fat_ptr _tmp1DD=({const char*_tmp6="incompatible %s() and %s() attributes on parameter %d";_tag_fat(_tmp6,sizeof(char),54U);});Cyc_Tcutil_terr(_tmp1DE,_tmp1DD,_tag_fat(_tmp5,sizeof(void*),3U));});});}
# 47
static void Cyc_Tc_fnTypeAttsOK(struct Cyc_Tcenv_Tenv*te,void*t,unsigned loc){
# 75
struct _tuple13 init_params=({struct _tuple13 _tmp1A4;_tmp1A4.f1=0,({struct _fat_ptr _tmp1DF=({const char*_tmp35="initializes";_tag_fat(_tmp35,sizeof(char),12U);});_tmp1A4.f2=_tmp1DF;});_tmp1A4;});
struct _tuple13 nolive_unique_params=({struct _tuple13 _tmp1A3;_tmp1A3.f1=0,({struct _fat_ptr _tmp1E0=({const char*_tmp34="noliveunique";_tag_fat(_tmp34,sizeof(char),13U);});_tmp1A3.f2=_tmp1E0;});_tmp1A3;});
struct _tuple13 consume_params=({struct _tuple13 _tmp1A2;_tmp1A2.f1=0,({struct _fat_ptr _tmp1E1=({const char*_tmp33="consume";_tag_fat(_tmp33,sizeof(char),8U);});_tmp1A2.f2=_tmp1E1;});_tmp1A2;});
void*_stmttmp0=Cyc_Tcutil_compress(t);void*_tmpA=_stmttmp0;struct Cyc_List_List*_tmpC;struct Cyc_List_List*_tmpB;if(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmpA)->tag == 5U){_LL1: _tmpB=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmpA)->f1).attributes;_tmpC=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmpA)->f1).args;_LL2: {struct Cyc_List_List*atts=_tmpB;struct Cyc_List_List*args=_tmpC;
# 80
int nargs=Cyc_List_length(args);
for(0;atts != 0;atts=atts->tl){
void*_stmttmp1=(void*)atts->hd;void*_tmpD=_stmttmp1;int _tmpE;int _tmpF;int _tmp10;switch(*((int*)_tmpD)){case 20U: _LL6: _tmp10=((struct Cyc_Absyn_Initializes_att_Absyn_Attribute_struct*)_tmpD)->f1;_LL7: {int i=_tmp10;
# 84
if(!Cyc_Tc_fnTypeAttsRangeOK(loc,i,nargs,(void*)atts->hd))goto _LL5;Cyc_Tc_fnTypeAttsOverlap(loc,i,init_params,nolive_unique_params);
# 86
Cyc_Tc_fnTypeAttsOverlap(loc,i,init_params,consume_params);{
struct _tuple8*_stmttmp2=({struct _tuple8*(*_tmp1E3)(struct Cyc_List_List*x,int n)=({struct _tuple8*(*_tmp25)(struct Cyc_List_List*x,int n)=(struct _tuple8*(*)(struct Cyc_List_List*x,int n))Cyc_List_nth;_tmp25;});struct Cyc_List_List*_tmp1E2=args;_tmp1E3(_tmp1E2,i - 1);});void*_tmp11;_LLF: _tmp11=_stmttmp2->f3;_LL10: {void*t=_tmp11;
struct _fat_ptr s=({const char*_tmp24="initializes attribute allowed only on";_tag_fat(_tmp24,sizeof(char),38U);});
{void*_stmttmp3=Cyc_Tcutil_compress(t);void*_tmp12=_stmttmp3;void*_tmp16;void*_tmp15;void*_tmp14;void*_tmp13;if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp12)->tag == 3U){_LL12: _tmp13=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp12)->f1).elt_type;_tmp14=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp12)->f1).ptr_atts).nullable;_tmp15=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp12)->f1).ptr_atts).bounds;_tmp16=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp12)->f1).ptr_atts).zero_term;_LL13: {void*t=_tmp13;void*nullable=_tmp14;void*bd=_tmp15;void*zt=_tmp16;
# 91
if(Cyc_Tcutil_force_type2bool(0,nullable))
({struct Cyc_String_pa_PrintArg_struct _tmp19=({struct Cyc_String_pa_PrintArg_struct _tmp19E;_tmp19E.tag=0U,_tmp19E.f1=(struct _fat_ptr)((struct _fat_ptr)s);_tmp19E;});void*_tmp17[1U];_tmp17[0]=& _tmp19;({unsigned _tmp1E5=loc;struct _fat_ptr _tmp1E4=({const char*_tmp18="%s non-null pointers";_tag_fat(_tmp18,sizeof(char),21U);});Cyc_Tcutil_terr(_tmp1E5,_tmp1E4,_tag_fat(_tmp17,sizeof(void*),1U));});});
# 91
if(!
# 93
Cyc_Tcutil_is_bound_one(bd))
({struct Cyc_String_pa_PrintArg_struct _tmp1C=({struct Cyc_String_pa_PrintArg_struct _tmp19F;_tmp19F.tag=0U,_tmp19F.f1=(struct _fat_ptr)((struct _fat_ptr)s);_tmp19F;});void*_tmp1A[1U];_tmp1A[0]=& _tmp1C;({unsigned _tmp1E7=loc;struct _fat_ptr _tmp1E6=({const char*_tmp1B="%s pointers of size 1";_tag_fat(_tmp1B,sizeof(char),22U);});Cyc_Tcutil_terr(_tmp1E7,_tmp1E6,_tag_fat(_tmp1A,sizeof(void*),1U));});});
# 91
if(
# 95
Cyc_Tcutil_force_type2bool(0,zt))
({struct Cyc_String_pa_PrintArg_struct _tmp1F=({struct Cyc_String_pa_PrintArg_struct _tmp1A0;_tmp1A0.tag=0U,_tmp1A0.f1=(struct _fat_ptr)((struct _fat_ptr)s);_tmp1A0;});void*_tmp1D[1U];_tmp1D[0]=& _tmp1F;({unsigned _tmp1E9=loc;struct _fat_ptr _tmp1E8=({const char*_tmp1E="%s pointers to non-zero-terminated arrays";_tag_fat(_tmp1E,sizeof(char),42U);});Cyc_Tcutil_terr(_tmp1E9,_tmp1E8,_tag_fat(_tmp1D,sizeof(void*),1U));});});
# 91
goto _LL11;}}else{_LL14: _LL15:
# 99
({struct Cyc_String_pa_PrintArg_struct _tmp22=({struct Cyc_String_pa_PrintArg_struct _tmp1A1;_tmp1A1.tag=0U,_tmp1A1.f1=(struct _fat_ptr)((struct _fat_ptr)s);_tmp1A1;});void*_tmp20[1U];_tmp20[0]=& _tmp22;({unsigned _tmp1EB=loc;struct _fat_ptr _tmp1EA=({const char*_tmp21="%s pointers";_tag_fat(_tmp21,sizeof(char),12U);});Cyc_Tcutil_terr(_tmp1EB,_tmp1EA,_tag_fat(_tmp20,sizeof(void*),1U));});});}_LL11:;}
# 101
({struct Cyc_List_List*_tmp1EC=({struct Cyc_List_List*_tmp23=_cycalloc(sizeof(*_tmp23));_tmp23->hd=(void*)i,_tmp23->tl=init_params.f1;_tmp23;});init_params.f1=_tmp1EC;});
goto _LL5;}}}case 21U: _LL8: _tmpF=((struct Cyc_Absyn_Noliveunique_att_Absyn_Attribute_struct*)_tmpD)->f1;_LL9: {int i=_tmpF;
# 104
if(!Cyc_Tc_fnTypeAttsRangeOK(loc,i,nargs,(void*)atts->hd))goto _LL5;Cyc_Tc_fnTypeAttsOverlap(loc,i,nolive_unique_params,init_params);{
# 107
struct _tuple8*_stmttmp4=({struct _tuple8*(*_tmp1EE)(struct Cyc_List_List*x,int n)=({struct _tuple8*(*_tmp2A)(struct Cyc_List_List*x,int n)=(struct _tuple8*(*)(struct Cyc_List_List*x,int n))Cyc_List_nth;_tmp2A;});struct Cyc_List_List*_tmp1ED=args;_tmp1EE(_tmp1ED,i - 1);});void*_tmp26;_LL17: _tmp26=_stmttmp4->f3;_LL18: {void*t=_tmp26;
if(!Cyc_Tcutil_is_noalias_pointer(t,0))
({void*_tmp27=0U;({unsigned _tmp1F0=loc;struct _fat_ptr _tmp1EF=({const char*_tmp28="noliveunique attribute allowed only on unique pointers";_tag_fat(_tmp28,sizeof(char),55U);});Cyc_Tcutil_terr(_tmp1F0,_tmp1EF,_tag_fat(_tmp27,sizeof(void*),0U));});});
# 108
({struct Cyc_List_List*_tmp1F1=({struct Cyc_List_List*_tmp29=_cycalloc(sizeof(*_tmp29));
# 110
_tmp29->hd=(void*)i,_tmp29->tl=nolive_unique_params.f1;_tmp29;});
# 108
nolive_unique_params.f1=_tmp1F1;});
# 111
goto _LL5;}}}case 22U: _LLA: _tmpE=((struct Cyc_Absyn_Consume_att_Absyn_Attribute_struct*)_tmpD)->f1;_LLB: {int i=_tmpE;
# 113
if(!Cyc_Tc_fnTypeAttsRangeOK(loc,i,nargs,(void*)atts->hd))goto _LL5;Cyc_Tc_fnTypeAttsOverlap(loc,i,consume_params,init_params);{
# 116
struct _tuple8*_stmttmp5=({struct _tuple8*(*_tmp1F3)(struct Cyc_List_List*x,int n)=({struct _tuple8*(*_tmp2F)(struct Cyc_List_List*x,int n)=(struct _tuple8*(*)(struct Cyc_List_List*x,int n))Cyc_List_nth;_tmp2F;});struct Cyc_List_List*_tmp1F2=args;_tmp1F3(_tmp1F2,i - 1);});void*_tmp2B;_LL1A: _tmp2B=_stmttmp5->f3;_LL1B: {void*t=_tmp2B;
if(!Cyc_Tcutil_is_noalias_pointer(t,0))
({void*_tmp2C=0U;({unsigned _tmp1F5=loc;struct _fat_ptr _tmp1F4=({const char*_tmp2D="consume attribute allowed only on unique pointers";_tag_fat(_tmp2D,sizeof(char),50U);});Cyc_Tcutil_terr(_tmp1F5,_tmp1F4,_tag_fat(_tmp2C,sizeof(void*),0U));});});
# 117
({struct Cyc_List_List*_tmp1F6=({struct Cyc_List_List*_tmp2E=_cycalloc(sizeof(*_tmp2E));
# 119
_tmp2E->hd=(void*)i,_tmp2E->tl=consume_params.f1;_tmp2E;});
# 117
consume_params.f1=_tmp1F6;});
# 120
goto _LL5;}}}default: _LLC: _LLD:
 goto _LL5;}_LL5:;}
# 124
goto _LL0;}}else{_LL3: _LL4:
({void*_tmp30=0U;({int(*_tmp1F8)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp32)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp32;});struct _fat_ptr _tmp1F7=({const char*_tmp31="fnTypeAttsOK: not a function type";_tag_fat(_tmp31,sizeof(char),34U);});_tmp1F8(_tmp1F7,_tag_fat(_tmp30,sizeof(void*),0U));});});}_LL0:;}struct _tuple14{void*f1;int f2;};
# 47
static void Cyc_Tc_tcVardecl(struct Cyc_Tcenv_Tenv*te,unsigned loc,struct Cyc_Absyn_Vardecl*vd,int check_var_init,int in_cinclude,struct Cyc_List_List**exports){
# 131
struct Cyc_List_List*_tmp3A;struct Cyc_Absyn_Exp*_tmp39;void*_tmp38;struct _tuple0*_tmp37;enum Cyc_Absyn_Scope _tmp36;_LL1: _tmp36=vd->sc;_tmp37=vd->name;_tmp38=vd->type;_tmp39=vd->initializer;_tmp3A=vd->attributes;_LL2: {enum Cyc_Absyn_Scope sc=_tmp36;struct _tuple0*q=_tmp37;void*t=_tmp38;struct Cyc_Absyn_Exp*initializer=_tmp39;struct Cyc_List_List*atts=_tmp3A;
# 138
{void*_stmttmp6=Cyc_Tcutil_compress(t);void*_tmp3B=_stmttmp6;unsigned _tmp3F;void*_tmp3E;struct Cyc_Absyn_Tqual _tmp3D;void*_tmp3C;if(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp3B)->tag == 4U){if((((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp3B)->f1).num_elts == 0){_LL4: _tmp3C=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp3B)->f1).elt_type;_tmp3D=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp3B)->f1).tq;_tmp3E=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp3B)->f1).zero_term;_tmp3F=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp3B)->f1).zt_loc;if(initializer != 0){_LL5: {void*telt=_tmp3C;struct Cyc_Absyn_Tqual tq=_tmp3D;void*zt=_tmp3E;unsigned ztl=_tmp3F;
# 140
{void*_stmttmp7=initializer->r;void*_tmp40=_stmttmp7;struct Cyc_List_List*_tmp41;struct Cyc_List_List*_tmp42;struct Cyc_Absyn_Exp*_tmp43;struct Cyc_Absyn_Exp*_tmp44;struct _fat_ptr _tmp45;struct _fat_ptr _tmp46;switch(*((int*)_tmp40)){case 0U: switch(((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmp40)->f1).Wstring_c).tag){case 8U: _LL9: _tmp46=((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmp40)->f1).String_c).val;_LLA: {struct _fat_ptr s=_tmp46;
# 142
t=({void*_tmp1FD=({void*_tmp1FC=telt;struct Cyc_Absyn_Tqual _tmp1FB=tq;struct Cyc_Absyn_Exp*_tmp1FA=Cyc_Absyn_uint_exp(_get_fat_size(s,sizeof(char)),0U);void*_tmp1F9=zt;Cyc_Absyn_array_type(_tmp1FC,_tmp1FB,_tmp1FA,_tmp1F9,ztl);});vd->type=_tmp1FD;});
goto _LL8;}case 9U: _LLB: _tmp45=((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmp40)->f1).Wstring_c).val;_LLC: {struct _fat_ptr s=_tmp45;
# 145
t=({void*_tmp202=({void*_tmp201=telt;struct Cyc_Absyn_Tqual _tmp200=tq;struct Cyc_Absyn_Exp*_tmp1FF=Cyc_Absyn_uint_exp(1U,0U);void*_tmp1FE=zt;Cyc_Absyn_array_type(_tmp201,_tmp200,_tmp1FF,_tmp1FE,ztl);});vd->type=_tmp202;});
goto _LL8;}default: goto _LL15;}case 27U: _LLD: _tmp44=((struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct*)_tmp40)->f2;_LLE: {struct Cyc_Absyn_Exp*e=_tmp44;
_tmp43=e;goto _LL10;}case 28U: _LLF: _tmp43=((struct Cyc_Absyn_ComprehensionNoinit_e_Absyn_Raw_exp_struct*)_tmp40)->f1;_LL10: {struct Cyc_Absyn_Exp*e=_tmp43;
# 150
t=({void*_tmp203=Cyc_Absyn_array_type(telt,tq,e,zt,ztl);vd->type=_tmp203;});
goto _LL8;}case 37U: _LL11: _tmp42=((struct Cyc_Absyn_UnresolvedMem_e_Absyn_Raw_exp_struct*)_tmp40)->f2;_LL12: {struct Cyc_List_List*es=_tmp42;
_tmp41=es;goto _LL14;}case 26U: _LL13: _tmp41=((struct Cyc_Absyn_Array_e_Absyn_Raw_exp_struct*)_tmp40)->f1;_LL14: {struct Cyc_List_List*es=_tmp41;
# 154
t=({void*_tmp208=({void*_tmp207=telt;struct Cyc_Absyn_Tqual _tmp206=tq;struct Cyc_Absyn_Exp*_tmp205=Cyc_Absyn_uint_exp((unsigned)Cyc_List_length(es),0U);void*_tmp204=zt;Cyc_Absyn_array_type(_tmp207,_tmp206,_tmp205,_tmp204,ztl);});vd->type=_tmp208;});
goto _LL8;}default: _LL15: _LL16:
 goto _LL8;}_LL8:;}
# 158
goto _LL3;}}else{goto _LL6;}}else{goto _LL6;}}else{_LL6: _LL7:
 goto _LL3;}_LL3:;}
# 162
Cyc_Tctyp_check_valid_toplevel_type(loc,te,t);
# 164
({int _tmp209=Cyc_Tcutil_extract_const_from_typedef(loc,(vd->tq).print_const,t);(vd->tq).real_const=_tmp209;});
# 167
({int _tmp20A=!Cyc_Tcutil_is_array_type(t);vd->escapes=_tmp20A;});
# 169
if(Cyc_Tcutil_is_function_type(t)){
atts=Cyc_Tcutil_transfer_fn_type_atts(t,atts);
Cyc_Tc_fnTypeAttsOK(te,t,loc);}
# 169
if(
# 174
(int)sc == (int)3U ||(int)sc == (int)4U){
if(initializer != 0 && !in_cinclude)
({void*_tmp47=0U;({unsigned _tmp20C=loc;struct _fat_ptr _tmp20B=({const char*_tmp48="extern declaration should not have initializer";_tag_fat(_tmp48,sizeof(char),47U);});Cyc_Tcutil_terr(_tmp20C,_tmp20B,_tag_fat(_tmp47,sizeof(void*),0U));});});}else{
if(!Cyc_Tcutil_is_function_type(t)){
# 181
for(0;atts != 0;atts=atts->tl){
void*_stmttmp8=(void*)atts->hd;void*_tmp49=_stmttmp8;switch(*((int*)_tmp49)){case 6U: _LL18: _LL19:
 goto _LL1B;case 8U: _LL1A: _LL1B:
# 186
 goto _LL1D;case 9U: _LL1C: _LL1D:
 goto _LL1F;case 10U: _LL1E: _LL1F:
 goto _LL21;case 11U: _LL20: _LL21:
 goto _LL23;case 12U: _LL22: _LL23:
 goto _LL25;case 13U: _LL24: _LL25:
 goto _LL27;case 14U: _LL26: _LL27:
 continue;default: _LL28: _LL29:
# 194
({struct Cyc_String_pa_PrintArg_struct _tmp4C=({struct Cyc_String_pa_PrintArg_struct _tmp1A6;_tmp1A6.tag=0U,({
struct _fat_ptr _tmp20D=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absyn_attribute2string((void*)atts->hd));_tmp1A6.f1=_tmp20D;});_tmp1A6;});struct Cyc_String_pa_PrintArg_struct _tmp4D=({struct Cyc_String_pa_PrintArg_struct _tmp1A5;_tmp1A5.tag=0U,({struct _fat_ptr _tmp20E=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(vd->name));_tmp1A5.f1=_tmp20E;});_tmp1A5;});void*_tmp4A[2U];_tmp4A[0]=& _tmp4C,_tmp4A[1]=& _tmp4D;({unsigned _tmp210=loc;struct _fat_ptr _tmp20F=({const char*_tmp4B="bad attribute %s for variable %s";_tag_fat(_tmp4B,sizeof(char),33U);});Cyc_Tcutil_terr(_tmp210,_tmp20F,_tag_fat(_tmp4A,sizeof(void*),2U));});});
goto _LL17;}_LL17:;}
# 199
if(initializer == 0 || in_cinclude){
if((check_var_init && !in_cinclude)&& !Cyc_Tcutil_zeroable_type(t))
({struct Cyc_String_pa_PrintArg_struct _tmp50=({struct Cyc_String_pa_PrintArg_struct _tmp1A8;_tmp1A8.tag=0U,({
struct _fat_ptr _tmp211=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(vd->name));_tmp1A8.f1=_tmp211;});_tmp1A8;});struct Cyc_String_pa_PrintArg_struct _tmp51=({struct Cyc_String_pa_PrintArg_struct _tmp1A7;_tmp1A7.tag=0U,({struct _fat_ptr _tmp212=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t));_tmp1A7.f1=_tmp212;});_tmp1A7;});void*_tmp4E[2U];_tmp4E[0]=& _tmp50,_tmp4E[1]=& _tmp51;({unsigned _tmp214=loc;struct _fat_ptr _tmp213=({const char*_tmp4F="initializer required for variable %s of type %s";_tag_fat(_tmp4F,sizeof(char),48U);});Cyc_Tcutil_terr(_tmp214,_tmp213,_tag_fat(_tmp4E,sizeof(void*),2U));});});}else{
# 204
struct Cyc_Absyn_Exp*e=initializer;
# 208
struct _handler_cons _tmp52;_push_handler(& _tmp52);{int _tmp54=0;if(setjmp(_tmp52.handler))_tmp54=1;if(!_tmp54){
{void*t2=Cyc_Tcexp_tcExpInitializer(te,& t,e);
if(!Cyc_Tcutil_coerce_assign(te,e,t)){
struct _fat_ptr s0=Cyc_Absynpp_qvar2string(vd->name);
const char*s1=" declared with type ";
struct _fat_ptr s2=Cyc_Absynpp_typ2string(t);
const char*s3=" but initialized with type ";
struct _fat_ptr s4=Cyc_Absynpp_typ2string(t2);
if(({unsigned long _tmp218=({unsigned long _tmp217=({unsigned long _tmp216=({unsigned long _tmp215=Cyc_strlen((struct _fat_ptr)s0);_tmp215 + Cyc_strlen(({const char*_tmp55=s1;_tag_fat(_tmp55,sizeof(char),21U);}));});_tmp216 + Cyc_strlen((struct _fat_ptr)s2);});_tmp217 + Cyc_strlen(({const char*_tmp56=s3;_tag_fat(_tmp56,sizeof(char),28U);}));});_tmp218 + Cyc_strlen((struct _fat_ptr)s4);})> (unsigned long)70)
({struct Cyc_String_pa_PrintArg_struct _tmp59=({struct Cyc_String_pa_PrintArg_struct _tmp1AD;_tmp1AD.tag=0U,_tmp1AD.f1=(struct _fat_ptr)((struct _fat_ptr)s0);_tmp1AD;});struct Cyc_String_pa_PrintArg_struct _tmp5A=({struct Cyc_String_pa_PrintArg_struct _tmp1AC;_tmp1AC.tag=0U,({struct _fat_ptr _tmp219=(struct _fat_ptr)({const char*_tmp5F=s1;_tag_fat(_tmp5F,sizeof(char),21U);});_tmp1AC.f1=_tmp219;});_tmp1AC;});struct Cyc_String_pa_PrintArg_struct _tmp5B=({struct Cyc_String_pa_PrintArg_struct _tmp1AB;_tmp1AB.tag=0U,_tmp1AB.f1=(struct _fat_ptr)((struct _fat_ptr)s2);_tmp1AB;});struct Cyc_String_pa_PrintArg_struct _tmp5C=({struct Cyc_String_pa_PrintArg_struct _tmp1AA;_tmp1AA.tag=0U,({struct _fat_ptr _tmp21A=(struct _fat_ptr)({const char*_tmp5E=s3;_tag_fat(_tmp5E,sizeof(char),28U);});_tmp1AA.f1=_tmp21A;});_tmp1AA;});struct Cyc_String_pa_PrintArg_struct _tmp5D=({struct Cyc_String_pa_PrintArg_struct _tmp1A9;_tmp1A9.tag=0U,_tmp1A9.f1=(struct _fat_ptr)((struct _fat_ptr)s4);_tmp1A9;});void*_tmp57[5U];_tmp57[0]=& _tmp59,_tmp57[1]=& _tmp5A,_tmp57[2]=& _tmp5B,_tmp57[3]=& _tmp5C,_tmp57[4]=& _tmp5D;({unsigned _tmp21C=loc;struct _fat_ptr _tmp21B=({const char*_tmp58="%s%s\n\t%s\n%s\n\t%s";_tag_fat(_tmp58,sizeof(char),16U);});Cyc_Tcutil_terr(_tmp21C,_tmp21B,_tag_fat(_tmp57,sizeof(void*),5U));});});else{
# 219
({struct Cyc_String_pa_PrintArg_struct _tmp62=({struct Cyc_String_pa_PrintArg_struct _tmp1B2;_tmp1B2.tag=0U,_tmp1B2.f1=(struct _fat_ptr)((struct _fat_ptr)s0);_tmp1B2;});struct Cyc_String_pa_PrintArg_struct _tmp63=({struct Cyc_String_pa_PrintArg_struct _tmp1B1;_tmp1B1.tag=0U,({struct _fat_ptr _tmp21D=(struct _fat_ptr)({const char*_tmp68=s1;_tag_fat(_tmp68,sizeof(char),21U);});_tmp1B1.f1=_tmp21D;});_tmp1B1;});struct Cyc_String_pa_PrintArg_struct _tmp64=({struct Cyc_String_pa_PrintArg_struct _tmp1B0;_tmp1B0.tag=0U,_tmp1B0.f1=(struct _fat_ptr)((struct _fat_ptr)s2);_tmp1B0;});struct Cyc_String_pa_PrintArg_struct _tmp65=({struct Cyc_String_pa_PrintArg_struct _tmp1AF;_tmp1AF.tag=0U,({struct _fat_ptr _tmp21E=(struct _fat_ptr)({const char*_tmp67=s3;_tag_fat(_tmp67,sizeof(char),28U);});_tmp1AF.f1=_tmp21E;});_tmp1AF;});struct Cyc_String_pa_PrintArg_struct _tmp66=({struct Cyc_String_pa_PrintArg_struct _tmp1AE;_tmp1AE.tag=0U,_tmp1AE.f1=(struct _fat_ptr)((struct _fat_ptr)s4);_tmp1AE;});void*_tmp60[5U];_tmp60[0]=& _tmp62,_tmp60[1]=& _tmp63,_tmp60[2]=& _tmp64,_tmp60[3]=& _tmp65,_tmp60[4]=& _tmp66;({unsigned _tmp220=loc;struct _fat_ptr _tmp21F=({const char*_tmp61="%s%s%s%s%s";_tag_fat(_tmp61,sizeof(char),11U);});Cyc_Tcutil_terr(_tmp220,_tmp21F,_tag_fat(_tmp60,sizeof(void*),5U));});});}
Cyc_Unify_explain_failure();}
# 210
if(!
# 223
Cyc_Tcutil_is_const_exp(e))
({void*_tmp69=0U;({unsigned _tmp222=loc;struct _fat_ptr _tmp221=({const char*_tmp6A="initializer is not a constant expression";_tag_fat(_tmp6A,sizeof(char),41U);});Cyc_Tcutil_terr(_tmp222,_tmp221,_tag_fat(_tmp69,sizeof(void*),0U));});});}
# 209
;_pop_handler();}else{void*_tmp53=(void*)Cyc_Core_get_exn_thrown();void*_tmp6B=_tmp53;void*_tmp6C;if(((struct Cyc_Tcenv_Env_error_exn_struct*)_tmp6B)->tag == Cyc_Tcenv_Env_error){_LL2B: _LL2C:
# 227
({void*_tmp6D=0U;({unsigned _tmp224=loc;struct _fat_ptr _tmp223=({const char*_tmp6E="initializer is not a constant expression";_tag_fat(_tmp6E,sizeof(char),41U);});Cyc_Tcutil_terr(_tmp224,_tmp223,_tag_fat(_tmp6D,sizeof(void*),0U));});});
goto _LL2A;}else{_LL2D: _tmp6C=_tmp6B;_LL2E: {void*exn=_tmp6C;(int)_rethrow(exn);}}_LL2A:;}}}}else{
# 233
for(0;atts != 0;atts=atts->tl){
void*_stmttmp9=(void*)atts->hd;void*_tmp6F=_stmttmp9;switch(*((int*)_tmp6F)){case 0U: _LL30: _LL31:
# 236
 goto _LL33;case 1U: _LL32: _LL33:
 goto _LL35;case 2U: _LL34: _LL35:
 goto _LL37;case 3U: _LL36: _LL37:
 goto _LL39;case 4U: _LL38: _LL39:
 goto _LL3B;case 19U: _LL3A: _LL3B:
 goto _LL3D;case 20U: _LL3C: _LL3D:
 goto _LL3F;case 23U: _LL3E: _LL3F:
 goto _LL41;case 26U: _LL40: _LL41:
 goto _LL43;case 5U: _LL42: _LL43:
({void*_tmp70=0U;({int(*_tmp226)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp72)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp72;});struct _fat_ptr _tmp225=({const char*_tmp71="tcVardecl: fn type atts in function var decl";_tag_fat(_tmp71,sizeof(char),45U);});_tmp226(_tmp225,_tag_fat(_tmp70,sizeof(void*),0U));});});case 6U: _LL44: _LL45:
# 247
 goto _LL47;case 7U: _LL46: _LL47:
# 249
({struct Cyc_String_pa_PrintArg_struct _tmp75=({struct Cyc_String_pa_PrintArg_struct _tmp1B3;_tmp1B3.tag=0U,({
struct _fat_ptr _tmp227=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absyn_attribute2string((void*)atts->hd));_tmp1B3.f1=_tmp227;});_tmp1B3;});void*_tmp73[1U];_tmp73[0]=& _tmp75;({unsigned _tmp229=loc;struct _fat_ptr _tmp228=({const char*_tmp74="bad attribute %s in function declaration";_tag_fat(_tmp74,sizeof(char),41U);});Cyc_Tcutil_terr(_tmp229,_tmp228,_tag_fat(_tmp73,sizeof(void*),1U));});});
goto _LL2F;default: _LL48: _LL49:
 continue;}_LL2F:;}}}
# 257
{struct _handler_cons _tmp76;_push_handler(& _tmp76);{int _tmp78=0;if(setjmp(_tmp76.handler))_tmp78=1;if(!_tmp78){
{struct _tuple14*ans=({struct _tuple14*(*_tmp22B)(struct Cyc_Dict_Dict d,struct _tuple0*k)=({struct _tuple14*(*_tmp7C)(struct Cyc_Dict_Dict d,struct _tuple0*k)=(struct _tuple14*(*)(struct Cyc_Dict_Dict d,struct _tuple0*k))Cyc_Dict_lookup;_tmp7C;});struct Cyc_Dict_Dict _tmp22A=(te->ae)->ordinaries;_tmp22B(_tmp22A,q);});
void*b0=(*ans).f1;
struct Cyc_Absyn_Global_b_Absyn_Binding_struct*b1=({struct Cyc_Absyn_Global_b_Absyn_Binding_struct*_tmp7B=_cycalloc(sizeof(*_tmp7B));_tmp7B->tag=1U,_tmp7B->f1=vd;_tmp7B;});
void*b=Cyc_Tcdecl_merge_binding(b0,(void*)b1,loc,Cyc_Tc_tc_msg);
if(b == 0){_npop_handler(0U);return;}if(
# 264
exports == 0 || Cyc_Tc_export_member(vd->name,*exports)){
if(b != b0 ||(*ans).f2)
# 267
({struct Cyc_Dict_Dict _tmp22F=({struct Cyc_Dict_Dict(*_tmp22E)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct _tuple14*v)=({struct Cyc_Dict_Dict(*_tmp79)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct _tuple14*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct _tuple14*v))Cyc_Dict_insert;_tmp79;});struct Cyc_Dict_Dict _tmp22D=(te->ae)->ordinaries;struct _tuple0*_tmp22C=q;_tmp22E(_tmp22D,_tmp22C,({struct _tuple14*_tmp7A=_cycalloc(sizeof(*_tmp7A));
_tmp7A->f1=b,_tmp7A->f2=(*ans).f2;_tmp7A;}));});
# 267
(te->ae)->ordinaries=_tmp22F;});}
# 262
_npop_handler(0U);return;}
# 258
;_pop_handler();}else{void*_tmp77=(void*)Cyc_Core_get_exn_thrown();void*_tmp7D=_tmp77;void*_tmp7E;if(((struct Cyc_Dict_Absent_exn_struct*)_tmp7D)->tag == Cyc_Dict_Absent){_LL4B: _LL4C:
# 270
 goto _LL4A;}else{_LL4D: _tmp7E=_tmp7D;_LL4E: {void*exn=_tmp7E;(int)_rethrow(exn);}}_LL4A:;}}}
if(exports == 0 || Cyc_Tc_export_member(vd->name,*exports))
({struct Cyc_Dict_Dict _tmp234=({struct Cyc_Dict_Dict(*_tmp233)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct _tuple14*v)=({struct Cyc_Dict_Dict(*_tmp7F)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct _tuple14*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct _tuple14*v))Cyc_Dict_insert;_tmp7F;});struct Cyc_Dict_Dict _tmp232=(te->ae)->ordinaries;struct _tuple0*_tmp231=q;_tmp233(_tmp232,_tmp231,({struct _tuple14*_tmp81=_cycalloc(sizeof(*_tmp81));
({void*_tmp230=(void*)({struct Cyc_Absyn_Global_b_Absyn_Binding_struct*_tmp80=_cycalloc(sizeof(*_tmp80));_tmp80->tag=1U,_tmp80->f1=vd;_tmp80;});_tmp81->f1=_tmp230;}),_tmp81->f2=0;_tmp81;}));});
# 272
(te->ae)->ordinaries=_tmp234;});}}
# 47
static int Cyc_Tc_is_main(struct _tuple0*n){
# 277
struct _fat_ptr*_tmp83;union Cyc_Absyn_Nmspace _tmp82;_LL1: _tmp82=n->f1;_tmp83=n->f2;_LL2: {union Cyc_Absyn_Nmspace nms=_tmp82;struct _fat_ptr*v=_tmp83;
union Cyc_Absyn_Nmspace _tmp84=nms;if((_tmp84.Abs_n).tag == 2){if((_tmp84.Abs_n).val == 0){_LL4: _LL5:
# 280
 return({struct _fat_ptr _tmp235=(struct _fat_ptr)*v;Cyc_strcmp(_tmp235,({const char*_tmp85="main";_tag_fat(_tmp85,sizeof(char),5U);}));})== 0;}else{goto _LL6;}}else{_LL6: _LL7:
 return 0;}_LL3:;}}
# 47
static void Cyc_Tc_tcFndecl(struct Cyc_Tcenv_Tenv*te,unsigned loc,struct Cyc_Absyn_Fndecl*fd,struct Cyc_List_List**exports){
# 288
struct _tuple0*q=fd->name;
# 292
if((int)fd->sc == (int)Cyc_Absyn_ExternC && !te->in_extern_c_include)
({void*_tmp86=0U;({unsigned _tmp237=loc;struct _fat_ptr _tmp236=({const char*_tmp87="extern \"C\" functions cannot be implemented in Cyclone";_tag_fat(_tmp87,sizeof(char),54U);});Cyc_Tcutil_terr(_tmp237,_tmp236,_tag_fat(_tmp86,sizeof(void*),0U));});});
# 292
{
# 296
struct Cyc_List_List*atts=(fd->i).attributes;
# 292
for(0;atts != 0;atts=atts->tl){
# 297
void*_stmttmpA=(void*)atts->hd;void*_tmp88=_stmttmpA;switch(*((int*)_tmp88)){case 7U: _LL1: _LL2:
 goto _LL4;case 6U: _LL3: _LL4:
# 300
({struct Cyc_String_pa_PrintArg_struct _tmp8B=({struct Cyc_String_pa_PrintArg_struct _tmp1B4;_tmp1B4.tag=0U,({struct _fat_ptr _tmp238=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absyn_attribute2string((void*)atts->hd));_tmp1B4.f1=_tmp238;});_tmp1B4;});void*_tmp89[1U];_tmp89[0]=& _tmp8B;({unsigned _tmp23A=loc;struct _fat_ptr _tmp239=({const char*_tmp8A="bad attribute %s for function";_tag_fat(_tmp8A,sizeof(char),30U);});Cyc_Tcutil_terr(_tmp23A,_tmp239,_tag_fat(_tmp89,sizeof(void*),1U));});});
goto _LL0;default: _LL5: _LL6:
 goto _LL0;}_LL0:;}}
# 306
Cyc_Tctyp_check_fndecl_valid_type(loc,te,fd);{
void*t=Cyc_Tcutil_fndecl2type(fd);
# 309
({struct Cyc_List_List*_tmp23B=Cyc_Tcutil_transfer_fn_type_atts(t,(fd->i).attributes);(fd->i).attributes=_tmp23B;});
Cyc_Tc_fnTypeAttsOK(te,t,loc);
# 313
{struct _handler_cons _tmp8C;_push_handler(& _tmp8C);{int _tmp8E=0;if(setjmp(_tmp8C.handler))_tmp8E=1;if(!_tmp8E){
{struct _tuple14*ans=({struct _tuple14*(*_tmp23D)(struct Cyc_Dict_Dict d,struct _tuple0*k)=({struct _tuple14*(*_tmp92)(struct Cyc_Dict_Dict d,struct _tuple0*k)=(struct _tuple14*(*)(struct Cyc_Dict_Dict d,struct _tuple0*k))Cyc_Dict_lookup;_tmp92;});struct Cyc_Dict_Dict _tmp23C=(te->ae)->ordinaries;_tmp23D(_tmp23C,q);});
void*b0=(*ans).f1;
struct Cyc_Absyn_Funname_b_Absyn_Binding_struct*b1=({struct Cyc_Absyn_Funname_b_Absyn_Binding_struct*_tmp91=_cycalloc(sizeof(*_tmp91));_tmp91->tag=2U,_tmp91->f1=fd;_tmp91;});
void*b=Cyc_Tcdecl_merge_binding(b0,(void*)b1,loc,Cyc_Tc_tc_msg);
if(b != 0){
# 320
if(exports == 0 || Cyc_Tc_export_member(q,*exports)){
if(!(b == b0 &&(*ans).f2))
({struct Cyc_Dict_Dict _tmp241=({struct Cyc_Dict_Dict(*_tmp240)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct _tuple14*v)=({struct Cyc_Dict_Dict(*_tmp8F)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct _tuple14*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct _tuple14*v))Cyc_Dict_insert;_tmp8F;});struct Cyc_Dict_Dict _tmp23F=(te->ae)->ordinaries;struct _tuple0*_tmp23E=q;_tmp240(_tmp23F,_tmp23E,({struct _tuple14*_tmp90=_cycalloc(sizeof(*_tmp90));
_tmp90->f1=b,_tmp90->f2=(*ans).f2;_tmp90;}));});
# 322
(te->ae)->ordinaries=_tmp241;});}}}
# 314
;_pop_handler();}else{void*_tmp8D=(void*)Cyc_Core_get_exn_thrown();void*_tmp93=_tmp8D;void*_tmp94;if(((struct Cyc_Dict_Absent_exn_struct*)_tmp93)->tag == Cyc_Dict_Absent){_LL8: _LL9:
# 326
 if(exports == 0 || Cyc_Tc_export_member(fd->name,*exports)){
struct Cyc_Absyn_Funname_b_Absyn_Binding_struct*b=({struct Cyc_Absyn_Funname_b_Absyn_Binding_struct*_tmp98=_cycalloc(sizeof(*_tmp98));_tmp98->tag=2U,_tmp98->f1=fd;_tmp98;});
({struct Cyc_Dict_Dict _tmp246=({struct Cyc_Dict_Dict(*_tmp245)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct _tuple14*v)=({struct Cyc_Dict_Dict(*_tmp95)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct _tuple14*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct _tuple14*v))Cyc_Dict_insert;_tmp95;});struct Cyc_Dict_Dict _tmp244=(te->ae)->ordinaries;struct _tuple0*_tmp243=q;_tmp245(_tmp244,_tmp243,({struct _tuple14*_tmp97=_cycalloc(sizeof(*_tmp97));
({void*_tmp242=(void*)({struct Cyc_Absyn_Funname_b_Absyn_Binding_struct*_tmp96=_cycalloc(sizeof(*_tmp96));_tmp96->tag=2U,_tmp96->f1=fd;_tmp96;});_tmp97->f1=_tmp242;}),_tmp97->f2=0;_tmp97;}));});
# 328
(te->ae)->ordinaries=_tmp246;});}
# 326
goto _LL7;}else{_LLA: _tmp94=_tmp93;_LLB: {void*exn=_tmp94;(int)_rethrow(exn);}}_LL7:;}}}
# 335
if(te->in_extern_c_include)return;{struct Cyc_Tcenv_Fenv*fenv=
# 340
Cyc_Tcenv_new_fenv(loc,fd);
struct Cyc_Tcenv_Tenv*_tmp99=({struct Cyc_Tcenv_Tenv*_tmpB6=_cycalloc(sizeof(*_tmpB6));_tmpB6->ns=te->ns,_tmpB6->ae=te->ae,_tmpB6->le=fenv,_tmpB6->allow_valueof=0,_tmpB6->in_extern_c_include=0,_tmpB6->in_tempest=te->in_tempest,_tmpB6->tempest_generalize=te->tempest_generalize;_tmpB6;});struct Cyc_Tcenv_Tenv*te=_tmp99;
# 343
Cyc_Tcstmt_tcStmt(te,fd->body,0);
# 346
Cyc_Tcenv_check_delayed_effects(te);
Cyc_Tcenv_check_delayed_constraints(te);
# 349
if(te->in_tempest){
te->tempest_generalize=1;
Cyc_Tctyp_check_fndecl_valid_type(loc,te,fd);
te->tempest_generalize=0;}
# 349
if(
# 355
Cyc_Tc_is_main(q)){
# 357
{void*_stmttmpB=Cyc_Tcutil_compress((fd->i).ret_type);void*_tmp9A=_stmttmpB;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp9A)->tag == 0U)switch(*((int*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp9A)->f1)){case 0U: _LLD: _LLE:
({void*_tmp9B=0U;({unsigned _tmp248=loc;struct _fat_ptr _tmp247=({const char*_tmp9C="main declared with return type void";_tag_fat(_tmp9C,sizeof(char),36U);});Cyc_Tcutil_warn(_tmp248,_tmp247,_tag_fat(_tmp9B,sizeof(void*),0U));});});goto _LLC;case 1U: switch(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp9A)->f1)->f2){case Cyc_Absyn_Int_sz: _LLF: _LL10:
 goto _LLC;case Cyc_Absyn_Long_sz: _LL11: _LL12:
 goto _LLC;default: goto _LL13;}default: goto _LL13;}else{_LL13: _LL14:
# 362
({struct Cyc_String_pa_PrintArg_struct _tmp9F=({struct Cyc_String_pa_PrintArg_struct _tmp1B5;_tmp1B5.tag=0U,({
struct _fat_ptr _tmp249=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string((fd->i).ret_type));_tmp1B5.f1=_tmp249;});_tmp1B5;});void*_tmp9D[1U];_tmp9D[0]=& _tmp9F;({unsigned _tmp24B=loc;struct _fat_ptr _tmp24A=({const char*_tmp9E="main declared with return type %s instead of int or void";_tag_fat(_tmp9E,sizeof(char),57U);});Cyc_Tcutil_terr(_tmp24B,_tmp24A,_tag_fat(_tmp9D,sizeof(void*),1U));});});
goto _LLC;}_LLC:;}
# 366
if((fd->i).c_varargs ||(fd->i).cyc_varargs != 0)
({void*_tmpA0=0U;({unsigned _tmp24D=loc;struct _fat_ptr _tmp24C=({const char*_tmpA1="main declared with varargs";_tag_fat(_tmpA1,sizeof(char),27U);});Cyc_Tcutil_terr(_tmp24D,_tmp24C,_tag_fat(_tmpA0,sizeof(void*),0U));});});{
# 366
struct Cyc_List_List*args=(fd->i).args;
# 369
if(args != 0){
struct _tuple8*_stmttmpC=(struct _tuple8*)args->hd;void*_tmpA2;_LL16: _tmpA2=_stmttmpC->f3;_LL17: {void*t1=_tmpA2;
{void*_stmttmpD=Cyc_Tcutil_compress(t1);void*_tmpA3=_stmttmpD;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmpA3)->tag == 0U){if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmpA3)->f1)->tag == 1U)switch(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmpA3)->f1)->f2){case Cyc_Absyn_Int_sz: _LL19: _LL1A:
 goto _LL18;case Cyc_Absyn_Long_sz: _LL1B: _LL1C:
 goto _LL18;default: goto _LL1D;}else{goto _LL1D;}}else{_LL1D: _LL1E:
# 375
({struct Cyc_String_pa_PrintArg_struct _tmpA6=({struct Cyc_String_pa_PrintArg_struct _tmp1B6;_tmp1B6.tag=0U,({
struct _fat_ptr _tmp24E=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t1));_tmp1B6.f1=_tmp24E;});_tmp1B6;});void*_tmpA4[1U];_tmpA4[0]=& _tmpA6;({unsigned _tmp250=loc;struct _fat_ptr _tmp24F=({const char*_tmpA5="main declared with first argument of type %s instead of int";_tag_fat(_tmpA5,sizeof(char),60U);});Cyc_Tcutil_terr(_tmp250,_tmp24F,_tag_fat(_tmpA4,sizeof(void*),1U));});});
goto _LL18;}_LL18:;}
# 379
args=args->tl;
if(args != 0){
struct _tuple8*_stmttmpE=(struct _tuple8*)args->hd;void*_tmpA7;_LL20: _tmpA7=_stmttmpE->f3;_LL21: {void*t2=_tmpA7;
args=args->tl;
if(args != 0)
({void*_tmpA8=0U;({unsigned _tmp252=loc;struct _fat_ptr _tmp251=({const char*_tmpA9="main declared with too many arguments";_tag_fat(_tmpA9,sizeof(char),38U);});Cyc_Tcutil_terr(_tmp252,_tmp251,_tag_fat(_tmpA8,sizeof(void*),0U));});});{
# 383
struct Cyc_Core_Opt*tvs=({struct Cyc_Core_Opt*_tmpB5=_cycalloc(sizeof(*_tmpB5));_tmpB5->v=(fd->i).tvars;_tmpB5;});
# 386
if(((!({void*_tmp26A=t2;Cyc_Unify_unify(_tmp26A,({void*_tmp269=Cyc_Absyn_string_type(({struct Cyc_Core_Opt*_tmp265=({struct Cyc_Core_Opt*_tmpAA=_cycalloc(sizeof(*_tmpAA));_tmpAA->v=& Cyc_Tcutil_rk;_tmpAA;});Cyc_Absyn_new_evar(_tmp265,tvs);}));void*_tmp268=({
struct Cyc_Core_Opt*_tmp266=({struct Cyc_Core_Opt*_tmpAB=_cycalloc(sizeof(*_tmpAB));_tmpAB->v=& Cyc_Tcutil_rk;_tmpAB;});Cyc_Absyn_new_evar(_tmp266,tvs);});
# 386
struct Cyc_Absyn_Tqual _tmp267=
# 388
Cyc_Absyn_empty_tqual(0U);
# 386
Cyc_Absyn_fatptr_type(_tmp269,_tmp268,_tmp267,
# 388
Cyc_Tcutil_any_bool((struct Cyc_List_List*)tvs->v));}));})&& !({
void*_tmp264=t2;Cyc_Unify_unify(_tmp264,({void*_tmp263=Cyc_Absyn_const_string_type(({struct Cyc_Core_Opt*_tmp25F=({struct Cyc_Core_Opt*_tmpAC=_cycalloc(sizeof(*_tmpAC));_tmpAC->v=& Cyc_Tcutil_rk;_tmpAC;});Cyc_Absyn_new_evar(_tmp25F,tvs);}));void*_tmp262=({
# 391
struct Cyc_Core_Opt*_tmp260=({struct Cyc_Core_Opt*_tmpAD=_cycalloc(sizeof(*_tmpAD));_tmpAD->v=& Cyc_Tcutil_rk;_tmpAD;});Cyc_Absyn_new_evar(_tmp260,tvs);});
# 389
struct Cyc_Absyn_Tqual _tmp261=
# 392
Cyc_Absyn_empty_tqual(0U);
# 389
Cyc_Absyn_fatptr_type(_tmp263,_tmp262,_tmp261,
# 392
Cyc_Tcutil_any_bool((struct Cyc_List_List*)tvs->v));}));}))&& !({
void*_tmp25E=t2;Cyc_Unify_unify(_tmp25E,({void*_tmp25D=Cyc_Absyn_string_type(({struct Cyc_Core_Opt*_tmp259=({struct Cyc_Core_Opt*_tmpAE=_cycalloc(sizeof(*_tmpAE));_tmpAE->v=& Cyc_Tcutil_rk;_tmpAE;});Cyc_Absyn_new_evar(_tmp259,tvs);}));void*_tmp25C=({
struct Cyc_Core_Opt*_tmp25A=({struct Cyc_Core_Opt*_tmpAF=_cycalloc(sizeof(*_tmpAF));_tmpAF->v=& Cyc_Tcutil_rk;_tmpAF;});Cyc_Absyn_new_evar(_tmp25A,tvs);});
# 393
struct Cyc_Absyn_Tqual _tmp25B=
# 395
Cyc_Absyn_const_tqual(0U);
# 393
Cyc_Absyn_fatptr_type(_tmp25D,_tmp25C,_tmp25B,
# 395
Cyc_Tcutil_any_bool((struct Cyc_List_List*)tvs->v));}));}))&& !({
void*_tmp258=t2;Cyc_Unify_unify(_tmp258,({void*_tmp257=Cyc_Absyn_const_string_type(({struct Cyc_Core_Opt*_tmp253=({struct Cyc_Core_Opt*_tmpB0=_cycalloc(sizeof(*_tmpB0));_tmpB0->v=& Cyc_Tcutil_rk;_tmpB0;});Cyc_Absyn_new_evar(_tmp253,tvs);}));void*_tmp256=({
# 398
struct Cyc_Core_Opt*_tmp254=({struct Cyc_Core_Opt*_tmpB1=_cycalloc(sizeof(*_tmpB1));_tmpB1->v=& Cyc_Tcutil_rk;_tmpB1;});Cyc_Absyn_new_evar(_tmp254,tvs);});
# 396
struct Cyc_Absyn_Tqual _tmp255=
# 399
Cyc_Absyn_const_tqual(0U);
# 396
Cyc_Absyn_fatptr_type(_tmp257,_tmp256,_tmp255,
# 399
Cyc_Tcutil_any_bool((struct Cyc_List_List*)tvs->v));}));}))
({struct Cyc_String_pa_PrintArg_struct _tmpB4=({struct Cyc_String_pa_PrintArg_struct _tmp1B7;_tmp1B7.tag=0U,({
struct _fat_ptr _tmp26B=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t2));_tmp1B7.f1=_tmp26B;});_tmp1B7;});void*_tmpB2[1U];_tmpB2[0]=& _tmpB4;({unsigned _tmp26D=loc;struct _fat_ptr _tmp26C=({const char*_tmpB3="second argument of main has type %s instead of char??";_tag_fat(_tmpB3,sizeof(char),54U);});Cyc_Tcutil_terr(_tmp26D,_tmp26C,_tag_fat(_tmpB2,sizeof(void*),1U));});});}}}}}}}}}}
# 47
static void Cyc_Tc_tcTypedefdecl(struct Cyc_Tcenv_Tenv*te,unsigned loc,struct Cyc_Absyn_Typedefdecl*td){
# 409
struct _tuple0*q=td->name;
# 415
if(({int(*_tmp26F)(struct Cyc_Dict_Dict d,struct _tuple0*k)=({int(*_tmpB7)(struct Cyc_Dict_Dict d,struct _tuple0*k)=(int(*)(struct Cyc_Dict_Dict d,struct _tuple0*k))Cyc_Dict_member;_tmpB7;});struct Cyc_Dict_Dict _tmp26E=(te->ae)->typedefs;_tmp26F(_tmp26E,q);})){
({struct Cyc_String_pa_PrintArg_struct _tmpBA=({struct Cyc_String_pa_PrintArg_struct _tmp1B8;_tmp1B8.tag=0U,_tmp1B8.f1=(struct _fat_ptr)((struct _fat_ptr)*(*q).f2);_tmp1B8;});void*_tmpB8[1U];_tmpB8[0]=& _tmpBA;({unsigned _tmp271=loc;struct _fat_ptr _tmp270=({const char*_tmpB9="redeclaration of typedef %s";_tag_fat(_tmpB9,sizeof(char),28U);});Cyc_Tcutil_terr(_tmp271,_tmp270,_tag_fat(_tmpB8,sizeof(void*),1U));});});
return;}
# 415
Cyc_Tcutil_check_unique_tvars(loc,td->tvs);
# 421
Cyc_Tcutil_add_tvar_identities(td->tvs);
if(td->defn != 0){
Cyc_Tctyp_check_type(loc,te,td->tvs,& Cyc_Tcutil_tak,0,1,(void*)_check_null(td->defn));
({int _tmp272=
Cyc_Tcutil_extract_const_from_typedef(loc,(td->tq).print_const,(void*)_check_null(td->defn));
# 424
(td->tq).real_const=_tmp272;});}
# 422
{
# 429
struct Cyc_List_List*tvs=td->tvs;
# 422
for(0;tvs != 0;tvs=tvs->tl){
# 430
void*_stmttmpF=Cyc_Absyn_compress_kb(((struct Cyc_Absyn_Tvar*)tvs->hd)->kind);void*_tmpBB=_stmttmpF;struct Cyc_Absyn_Kind*_tmpBD;struct Cyc_Core_Opt**_tmpBC;struct Cyc_Core_Opt**_tmpBE;switch(*((int*)_tmpBB)){case 1U: _LL1: _tmpBE=(struct Cyc_Core_Opt**)&((struct Cyc_Absyn_Unknown_kb_Absyn_KindBound_struct*)_tmpBB)->f1;_LL2: {struct Cyc_Core_Opt**f=_tmpBE;
# 432
if(td->defn != 0)
({struct Cyc_String_pa_PrintArg_struct _tmpC1=({struct Cyc_String_pa_PrintArg_struct _tmp1B9;_tmp1B9.tag=0U,_tmp1B9.f1=(struct _fat_ptr)((struct _fat_ptr)*((struct Cyc_Absyn_Tvar*)tvs->hd)->name);_tmp1B9;});void*_tmpBF[1U];_tmpBF[0]=& _tmpC1;({unsigned _tmp274=loc;struct _fat_ptr _tmp273=({const char*_tmpC0="type variable %s is not used in typedef definition";_tag_fat(_tmpC0,sizeof(char),51U);});Cyc_Tcutil_warn(_tmp274,_tmp273,_tag_fat(_tmpBF,sizeof(void*),1U));});});
# 432
({struct Cyc_Core_Opt*_tmp276=({struct Cyc_Core_Opt*_tmpC2=_cycalloc(sizeof(*_tmpC2));({
# 435
void*_tmp275=Cyc_Tcutil_kind_to_bound(& Cyc_Tcutil_tbk);_tmpC2->v=_tmp275;});_tmpC2;});
# 432
*f=_tmp276;});
# 435
goto _LL0;}case 2U: _LL3: _tmpBC=(struct Cyc_Core_Opt**)&((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmpBB)->f1;_tmpBD=((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmpBB)->f2;_LL4: {struct Cyc_Core_Opt**f=_tmpBC;struct Cyc_Absyn_Kind*k=_tmpBD;
# 442
({struct Cyc_Core_Opt*_tmp278=({struct Cyc_Core_Opt*_tmpC3=_cycalloc(sizeof(*_tmpC3));({void*_tmp277=Cyc_Tcutil_kind_to_bound(k);_tmpC3->v=_tmp277;});_tmpC3;});*f=_tmp278;});
goto _LL0;}default: _LL5: _LL6:
 continue;}_LL0:;}}
# 448
({struct Cyc_Dict_Dict _tmp27C=({struct Cyc_Dict_Dict(*_tmp27B)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Typedefdecl*v)=({struct Cyc_Dict_Dict(*_tmpC4)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Typedefdecl*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Typedefdecl*v))Cyc_Dict_insert;_tmpC4;});struct Cyc_Dict_Dict _tmp27A=(te->ae)->typedefs;struct _tuple0*_tmp279=q;_tmp27B(_tmp27A,_tmp279,td);});(te->ae)->typedefs=_tmp27C;});}struct _tuple15{void*f1;void*f2;};
# 47
static void Cyc_Tc_tcAggrImpl(struct Cyc_Tcenv_Tenv*te,unsigned loc,enum Cyc_Absyn_AggrKind str_or_union,struct Cyc_List_List*tvs,struct Cyc_List_List*rpo,struct Cyc_List_List*fields){
# 458
struct _RegionHandle _tmpC5=_new_region("uprev_rgn");struct _RegionHandle*uprev_rgn=& _tmpC5;_push_region(uprev_rgn);
# 460
for(0;rpo != 0;rpo=rpo->tl){
struct _tuple15*_stmttmp10=(struct _tuple15*)rpo->hd;void*_tmpC7;void*_tmpC6;_LL1: _tmpC6=_stmttmp10->f1;_tmpC7=_stmttmp10->f2;_LL2: {void*e=_tmpC6;void*r=_tmpC7;
Cyc_Tctyp_check_type(loc,te,tvs,& Cyc_Tcutil_ek,0,0,e);
Cyc_Tctyp_check_type(loc,te,tvs,& Cyc_Tcutil_trk,0,0,r);}}{
# 466
struct Cyc_List_List*prev_fields=0;
struct Cyc_List_List*prev_relations=0;
# 469
struct Cyc_List_List*fs=fields;for(0;fs != 0;fs=fs->tl){
struct Cyc_Absyn_Aggrfield*_stmttmp11=(struct Cyc_Absyn_Aggrfield*)fs->hd;struct Cyc_Absyn_Exp*_tmpCD;struct Cyc_List_List*_tmpCC;struct Cyc_Absyn_Exp*_tmpCB;void*_tmpCA;struct Cyc_Absyn_Tqual _tmpC9;struct _fat_ptr*_tmpC8;_LL4: _tmpC8=_stmttmp11->name;_tmpC9=_stmttmp11->tq;_tmpCA=_stmttmp11->type;_tmpCB=_stmttmp11->width;_tmpCC=_stmttmp11->attributes;_tmpCD=_stmttmp11->requires_clause;_LL5: {struct _fat_ptr*fn=_tmpC8;struct Cyc_Absyn_Tqual tq=_tmpC9;void*t=_tmpCA;struct Cyc_Absyn_Exp*width=_tmpCB;struct Cyc_List_List*atts=_tmpCC;struct Cyc_Absyn_Exp*requires_clause=_tmpCD;
# 472
if(({int(*_tmp27E)(int(*compare)(struct _fat_ptr*,struct _fat_ptr*),struct Cyc_List_List*l,struct _fat_ptr*x)=({int(*_tmpCE)(int(*compare)(struct _fat_ptr*,struct _fat_ptr*),struct Cyc_List_List*l,struct _fat_ptr*x)=(int(*)(int(*compare)(struct _fat_ptr*,struct _fat_ptr*),struct Cyc_List_List*l,struct _fat_ptr*x))Cyc_List_mem;_tmpCE;});struct Cyc_List_List*_tmp27D=prev_fields;_tmp27E(Cyc_strptrcmp,_tmp27D,fn);}))
({struct Cyc_String_pa_PrintArg_struct _tmpD1=({struct Cyc_String_pa_PrintArg_struct _tmp1BA;_tmp1BA.tag=0U,_tmp1BA.f1=(struct _fat_ptr)((struct _fat_ptr)*fn);_tmp1BA;});void*_tmpCF[1U];_tmpCF[0]=& _tmpD1;({unsigned _tmp280=loc;struct _fat_ptr _tmp27F=({const char*_tmpD0="duplicate member %s";_tag_fat(_tmpD0,sizeof(char),20U);});Cyc_Tcutil_terr(_tmp280,_tmp27F,_tag_fat(_tmpCF,sizeof(void*),1U));});});
# 472
if(({
# 476
struct _fat_ptr _tmp281=(struct _fat_ptr)*fn;Cyc_strcmp(_tmp281,({const char*_tmpD2="";_tag_fat(_tmpD2,sizeof(char),1U);}));})!= 0)
prev_fields=({struct Cyc_List_List*_tmpD3=_region_malloc(uprev_rgn,sizeof(*_tmpD3));_tmpD3->hd=fn,_tmpD3->tl=prev_fields;_tmpD3;});{
# 472
struct Cyc_Absyn_Kind*field_kind=& Cyc_Tcutil_tmk;
# 483
if((int)str_or_union == (int)1U ||
 fs->tl == 0 &&(int)str_or_union == (int)0U)
field_kind=& Cyc_Tcutil_tak;
# 483
Cyc_Tctyp_check_type(loc,te,tvs,field_kind,0,0,t);
# 488
({int _tmp282=Cyc_Tcutil_extract_const_from_typedef(loc,(((struct Cyc_Absyn_Aggrfield*)fs->hd)->tq).print_const,t);(((struct Cyc_Absyn_Aggrfield*)fs->hd)->tq).real_const=_tmp282;});
# 491
Cyc_Tcutil_check_bitfield(loc,t,width,fn);
# 493
if((unsigned)requires_clause){
if((int)str_or_union != (int)1U)
({void*_tmpD4=0U;({unsigned _tmp284=loc;struct _fat_ptr _tmp283=({const char*_tmpD5="@requires clauses are only allowed on union members";_tag_fat(_tmpD5,sizeof(char),52U);});Cyc_Tcutil_terr(_tmp284,_tmp283,_tag_fat(_tmpD4,sizeof(void*),0U));});});{
# 494
struct Cyc_Tcenv_Tenv*te2=
# 496
Cyc_Tcenv_allow_valueof(te);
Cyc_Tcexp_tcExp(te2,0,requires_clause);
if(!Cyc_Tcutil_is_integral(requires_clause))
({struct Cyc_String_pa_PrintArg_struct _tmpD8=({struct Cyc_String_pa_PrintArg_struct _tmp1BB;_tmp1BB.tag=0U,({
# 501
struct _fat_ptr _tmp285=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string((void*)_check_null(requires_clause->topt)));_tmp1BB.f1=_tmp285;});_tmp1BB;});void*_tmpD6[1U];_tmpD6[0]=& _tmpD8;({unsigned _tmp287=requires_clause->loc;struct _fat_ptr _tmp286=({const char*_tmpD7="@requires clause has type %s instead of integral type";_tag_fat(_tmpD7,sizeof(char),54U);});Cyc_Tcutil_terr(_tmp287,_tmp286,_tag_fat(_tmpD6,sizeof(void*),1U));});});else{
# 503
({unsigned _tmp28A=requires_clause->loc;struct Cyc_Tcenv_Tenv*_tmp289=te;struct Cyc_List_List*_tmp288=tvs;Cyc_Tctyp_check_type(_tmp28A,_tmp289,_tmp288,& Cyc_Tcutil_ik,0,0,(void*)({struct Cyc_Absyn_ValueofType_Absyn_Type_struct*_tmpD9=_cycalloc(sizeof(*_tmpD9));_tmpD9->tag=9U,_tmpD9->f1=requires_clause;_tmpD9;}));});{
# 505
struct Cyc_List_List*relns=Cyc_Relations_exp2relns(uprev_rgn,requires_clause);
# 512
if(!Cyc_Relations_consistent_relations(relns))
({void*_tmpDA=0U;({unsigned _tmp28C=requires_clause->loc;struct _fat_ptr _tmp28B=({const char*_tmpDB="@requires clause may be unsatisfiable";_tag_fat(_tmpDB,sizeof(char),38U);});Cyc_Tcutil_terr(_tmp28C,_tmp28B,_tag_fat(_tmpDA,sizeof(void*),0U));});});
# 512
{
# 518
struct Cyc_List_List*p=prev_relations;
# 512
for(0;p != 0;p=p->tl){
# 519
if(Cyc_Relations_consistent_relations(Cyc_List_rappend(uprev_rgn,relns,(struct Cyc_List_List*)p->hd)))
({void*_tmpDC=0U;({unsigned _tmp28E=requires_clause->loc;struct _fat_ptr _tmp28D=({const char*_tmpDD="@requires clause may overlap with previous clauses";_tag_fat(_tmpDD,sizeof(char),51U);});Cyc_Tcutil_terr(_tmp28E,_tmp28D,_tag_fat(_tmpDC,sizeof(void*),0U));});});}}
# 523
prev_relations=({struct Cyc_List_List*_tmpDE=_region_malloc(uprev_rgn,sizeof(*_tmpDE));_tmpDE->hd=relns,_tmpDE->tl=prev_relations;_tmpDE;});}}}}else{
# 526
if(prev_relations != 0)
({void*_tmpDF=0U;({unsigned _tmp290=loc;struct _fat_ptr _tmp28F=({const char*_tmpE0="if one field has a @requires clause, they all must";_tag_fat(_tmpE0,sizeof(char),51U);});Cyc_Tcutil_terr(_tmp290,_tmp28F,_tag_fat(_tmpDF,sizeof(void*),0U));});});}}}}}
# 460
;_pop_region(uprev_rgn);}
# 47
static void Cyc_Tc_rule_out_memkind(unsigned loc,struct _tuple0*n,struct Cyc_List_List*tvs,int constrain_top_kind){
# 534
struct Cyc_List_List*tvs2=tvs;for(0;tvs2 != 0;tvs2=tvs2->tl){
void*_stmttmp12=Cyc_Absyn_compress_kb(((struct Cyc_Absyn_Tvar*)tvs2->hd)->kind);void*_tmpE1=_stmttmp12;enum Cyc_Absyn_AliasQual _tmpE2;struct Cyc_Absyn_Kind*_tmpE4;struct Cyc_Core_Opt**_tmpE3;struct Cyc_Core_Opt**_tmpE5;enum Cyc_Absyn_AliasQual _tmpE7;struct Cyc_Core_Opt**_tmpE6;struct Cyc_Core_Opt**_tmpE8;switch(*((int*)_tmpE1)){case 1U: _LL1: _tmpE8=(struct Cyc_Core_Opt**)&((struct Cyc_Absyn_Unknown_kb_Absyn_KindBound_struct*)_tmpE1)->f1;_LL2: {struct Cyc_Core_Opt**f=_tmpE8;
# 537
({struct Cyc_Core_Opt*_tmp292=({struct Cyc_Core_Opt*_tmpE9=_cycalloc(sizeof(*_tmpE9));({void*_tmp291=Cyc_Tcutil_kind_to_bound(& Cyc_Tcutil_bk);_tmpE9->v=_tmp291;});_tmpE9;});*f=_tmp292;});continue;}case 2U: switch(((struct Cyc_Absyn_Kind*)((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmpE1)->f2)->kind){case Cyc_Absyn_MemKind: _LL3: _tmpE6=(struct Cyc_Core_Opt**)&((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmpE1)->f1;_tmpE7=(((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmpE1)->f2)->aliasqual;_LL4: {struct Cyc_Core_Opt**f=_tmpE6;enum Cyc_Absyn_AliasQual a=_tmpE7;
# 539
if(constrain_top_kind &&(int)a == (int)2U)
({struct Cyc_Core_Opt*_tmp294=({struct Cyc_Core_Opt*_tmpEB=_cycalloc(sizeof(*_tmpEB));({void*_tmp293=Cyc_Tcutil_kind_to_bound(({struct Cyc_Absyn_Kind*_tmpEA=_cycalloc(sizeof(*_tmpEA));_tmpEA->kind=Cyc_Absyn_BoxKind,_tmpEA->aliasqual=Cyc_Absyn_Aliasable;_tmpEA;}));_tmpEB->v=_tmp293;});_tmpEB;});*f=_tmp294;});else{
# 542
({struct Cyc_Core_Opt*_tmp296=({struct Cyc_Core_Opt*_tmpED=_cycalloc(sizeof(*_tmpED));({void*_tmp295=Cyc_Tcutil_kind_to_bound(({struct Cyc_Absyn_Kind*_tmpEC=_cycalloc(sizeof(*_tmpEC));_tmpEC->kind=Cyc_Absyn_BoxKind,_tmpEC->aliasqual=a;_tmpEC;}));_tmpED->v=_tmp295;});_tmpED;});*f=_tmp296;});}
continue;}case Cyc_Absyn_BoxKind: if(((struct Cyc_Absyn_Kind*)((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmpE1)->f2)->aliasqual == Cyc_Absyn_Top){_LL5: _tmpE5=(struct Cyc_Core_Opt**)&((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmpE1)->f1;if(constrain_top_kind){_LL6: {struct Cyc_Core_Opt**f=_tmpE5;
# 545
({struct Cyc_Core_Opt*_tmp298=({struct Cyc_Core_Opt*_tmpEF=_cycalloc(sizeof(*_tmpEF));({void*_tmp297=Cyc_Tcutil_kind_to_bound(({struct Cyc_Absyn_Kind*_tmpEE=_cycalloc(sizeof(*_tmpEE));_tmpEE->kind=Cyc_Absyn_BoxKind,_tmpEE->aliasqual=Cyc_Absyn_Aliasable;_tmpEE;}));_tmpEF->v=_tmp297;});_tmpEF;});*f=_tmp298;});
continue;}}else{goto _LL7;}}else{goto _LL7;}default: _LL7: _tmpE3=(struct Cyc_Core_Opt**)&((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmpE1)->f1;_tmpE4=((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmpE1)->f2;_LL8: {struct Cyc_Core_Opt**f=_tmpE3;struct Cyc_Absyn_Kind*k=_tmpE4;
# 548
({struct Cyc_Core_Opt*_tmp29A=({struct Cyc_Core_Opt*_tmpF0=_cycalloc(sizeof(*_tmpF0));({void*_tmp299=Cyc_Tcutil_kind_to_bound(k);_tmpF0->v=_tmp299;});_tmpF0;});*f=_tmp29A;});continue;}}default: if(((struct Cyc_Absyn_Kind*)((struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct*)_tmpE1)->f1)->kind == Cyc_Absyn_MemKind){_LL9: _tmpE2=(((struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct*)_tmpE1)->f1)->aliasqual;_LLA: {enum Cyc_Absyn_AliasQual a=_tmpE2;
# 550
({struct Cyc_String_pa_PrintArg_struct _tmpF3=({struct Cyc_String_pa_PrintArg_struct _tmp1BE;_tmp1BE.tag=0U,_tmp1BE.f1=(struct _fat_ptr)((struct _fat_ptr)*(*n).f2);_tmp1BE;});struct Cyc_String_pa_PrintArg_struct _tmpF4=({struct Cyc_String_pa_PrintArg_struct _tmp1BD;_tmp1BD.tag=0U,_tmp1BD.f1=(struct _fat_ptr)((struct _fat_ptr)*((struct Cyc_Absyn_Tvar*)tvs2->hd)->name);_tmp1BD;});struct Cyc_String_pa_PrintArg_struct _tmpF5=({struct Cyc_String_pa_PrintArg_struct _tmp1BC;_tmp1BC.tag=0U,({
struct _fat_ptr _tmp29B=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_kind2string(({struct Cyc_Absyn_Kind*_tmpF6=_cycalloc(sizeof(*_tmpF6));_tmpF6->kind=Cyc_Absyn_MemKind,_tmpF6->aliasqual=a;_tmpF6;})));_tmp1BC.f1=_tmp29B;});_tmp1BC;});void*_tmpF1[3U];_tmpF1[0]=& _tmpF3,_tmpF1[1]=& _tmpF4,_tmpF1[2]=& _tmpF5;({unsigned _tmp29D=loc;struct _fat_ptr _tmp29C=({const char*_tmpF2="type %s attempts to abstract type variable %s of kind %s";_tag_fat(_tmpF2,sizeof(char),57U);});Cyc_Tcutil_terr(_tmp29D,_tmp29C,_tag_fat(_tmpF1,sizeof(void*),3U));});});
continue;}}else{_LLB: _LLC:
 continue;}}_LL0:;}}struct _tuple16{struct Cyc_Absyn_AggrdeclImpl*f1;struct Cyc_Absyn_Aggrdecl***f2;};
# 47
void Cyc_Tc_tcAggrdecl(struct Cyc_Tcenv_Tenv*te,unsigned loc,struct Cyc_Absyn_Aggrdecl*ad){
# 559
struct _tuple0*q=ad->name;
# 565
{struct Cyc_List_List*atts=ad->attributes;for(0;atts != 0;atts=atts->tl){
void*_stmttmp13=(void*)atts->hd;void*_tmpF7=_stmttmp13;switch(*((int*)_tmpF7)){case 7U: _LL1: _LL2:
 goto _LL4;case 6U: _LL3: _LL4:
 continue;default: _LL5: _LL6:
# 570
({struct Cyc_String_pa_PrintArg_struct _tmpFA=({struct Cyc_String_pa_PrintArg_struct _tmp1C0;_tmp1C0.tag=0U,({
struct _fat_ptr _tmp29E=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absyn_attribute2string((void*)atts->hd));_tmp1C0.f1=_tmp29E;});_tmp1C0;});struct Cyc_String_pa_PrintArg_struct _tmpFB=({struct Cyc_String_pa_PrintArg_struct _tmp1BF;_tmp1BF.tag=0U,_tmp1BF.f1=(struct _fat_ptr)((struct _fat_ptr)*(*q).f2);_tmp1BF;});void*_tmpF8[2U];_tmpF8[0]=& _tmpFA,_tmpF8[1]=& _tmpFB;({unsigned _tmp2A0=loc;struct _fat_ptr _tmp29F=({const char*_tmpF9="bad attribute %s in %s definition";_tag_fat(_tmpF9,sizeof(char),34U);});Cyc_Tcutil_terr(_tmp2A0,_tmp29F,_tag_fat(_tmpF8,sizeof(void*),2U));});});
goto _LL0;}_LL0:;}}{
# 575
struct Cyc_List_List*tvs=ad->tvs;
# 578
Cyc_Tcutil_check_unique_tvars(loc,ad->tvs);
Cyc_Tcutil_add_tvar_identities(ad->tvs);{
# 583
struct _tuple16 _stmttmp14=({struct _tuple16 _tmp1C4;_tmp1C4.f1=ad->impl,({struct Cyc_Absyn_Aggrdecl***_tmp2A3=({struct Cyc_Absyn_Aggrdecl***(*_tmp2A2)(struct Cyc_Dict_Dict d,struct _tuple0*k)=({struct Cyc_Absyn_Aggrdecl***(*_tmp118)(struct Cyc_Dict_Dict d,struct _tuple0*k)=(struct Cyc_Absyn_Aggrdecl***(*)(struct Cyc_Dict_Dict d,struct _tuple0*k))Cyc_Dict_lookup_opt;_tmp118;});struct Cyc_Dict_Dict _tmp2A1=(te->ae)->aggrdecls;_tmp2A2(_tmp2A1,q);});_tmp1C4.f2=_tmp2A3;});_tmp1C4;});struct _tuple16 _tmpFC=_stmttmp14;struct Cyc_Absyn_Aggrdecl**_tmp101;int _tmp100;struct Cyc_List_List*_tmpFF;struct Cyc_List_List*_tmpFE;struct Cyc_List_List*_tmpFD;int _tmp105;struct Cyc_List_List*_tmp104;struct Cyc_List_List*_tmp103;struct Cyc_List_List*_tmp102;struct Cyc_Absyn_Aggrdecl**_tmp106;if(_tmpFC.f1 == 0){if(_tmpFC.f2 == 0){_LL8: _LL9:
# 586
 Cyc_Tc_rule_out_memkind(loc,q,tvs,0);
# 588
({struct Cyc_Dict_Dict _tmp2A7=({struct Cyc_Dict_Dict(*_tmp2A6)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Aggrdecl**v)=({struct Cyc_Dict_Dict(*_tmp107)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Aggrdecl**v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Aggrdecl**v))Cyc_Dict_insert;_tmp107;});struct Cyc_Dict_Dict _tmp2A5=(te->ae)->aggrdecls;struct _tuple0*_tmp2A4=q;_tmp2A6(_tmp2A5,_tmp2A4,({struct Cyc_Absyn_Aggrdecl**_tmp108=_cycalloc(sizeof(*_tmp108));*_tmp108=ad;_tmp108;}));});(te->ae)->aggrdecls=_tmp2A7;});
goto _LL7;}else{_LLE: _tmp106=*_tmpFC.f2;_LLF: {struct Cyc_Absyn_Aggrdecl**adp=_tmp106;
# 648
struct Cyc_Absyn_Aggrdecl*ad2=Cyc_Tcdecl_merge_aggrdecl(*adp,ad,loc,Cyc_Tc_tc_msg);
if(ad2 == 0)
return;else{
# 652
Cyc_Tc_rule_out_memkind(loc,q,tvs,0);
# 655
if(ad->impl != 0)
Cyc_Tc_rule_out_memkind(loc,q,((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->exist_vars,1);
# 655
*adp=ad2;
# 659
ad=ad2;
goto _LL7;}}}}else{if(_tmpFC.f2 == 0){_LLA: _tmp102=(_tmpFC.f1)->exist_vars;_tmp103=(_tmpFC.f1)->rgn_po;_tmp104=(_tmpFC.f1)->fields;_tmp105=(_tmpFC.f1)->tagged;_LLB: {struct Cyc_List_List*exist_vars=_tmp102;struct Cyc_List_List*rgn_po=_tmp103;struct Cyc_List_List*fs=_tmp104;int tagged=_tmp105;
# 593
struct Cyc_Absyn_Aggrdecl**adp=({struct Cyc_Absyn_Aggrdecl**_tmp112=_cycalloc(sizeof(*_tmp112));({struct Cyc_Absyn_Aggrdecl*_tmp2A8=({struct Cyc_Absyn_Aggrdecl*_tmp111=_cycalloc(sizeof(*_tmp111));_tmp111->kind=ad->kind,_tmp111->sc=Cyc_Absyn_Extern,_tmp111->name=ad->name,_tmp111->tvs=tvs,_tmp111->impl=0,_tmp111->attributes=ad->attributes,_tmp111->expected_mem_kind=0;_tmp111;});*_tmp112=_tmp2A8;});_tmp112;});
# 595
({struct Cyc_Dict_Dict _tmp2AC=({struct Cyc_Dict_Dict(*_tmp2AB)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Aggrdecl**v)=({struct Cyc_Dict_Dict(*_tmp109)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Aggrdecl**v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Aggrdecl**v))Cyc_Dict_insert;_tmp109;});struct Cyc_Dict_Dict _tmp2AA=(te->ae)->aggrdecls;struct _tuple0*_tmp2A9=q;_tmp2AB(_tmp2AA,_tmp2A9,adp);});(te->ae)->aggrdecls=_tmp2AC;});
# 600
Cyc_Tcutil_check_unique_tvars(loc,exist_vars);
Cyc_Tcutil_add_tvar_identities(exist_vars);
# 604
if(tagged &&(int)ad->kind == (int)Cyc_Absyn_StructA)
({void*_tmp10A=0U;({unsigned _tmp2AE=loc;struct _fat_ptr _tmp2AD=({const char*_tmp10B="@tagged qualifier is only allowed on union declarations";_tag_fat(_tmp10B,sizeof(char),56U);});Cyc_Tcutil_terr(_tmp2AE,_tmp2AD,_tag_fat(_tmp10A,sizeof(void*),0U));});});
# 604
({struct Cyc_Tcenv_Tenv*_tmp2B3=te;unsigned _tmp2B2=loc;enum Cyc_Absyn_AggrKind _tmp2B1=ad->kind;struct Cyc_List_List*_tmp2B0=
# 606
Cyc_List_append(tvs,exist_vars);
# 604
struct Cyc_List_List*_tmp2AF=rgn_po;Cyc_Tc_tcAggrImpl(_tmp2B3,_tmp2B2,_tmp2B1,_tmp2B0,_tmp2AF,fs);});
# 608
Cyc_Tc_rule_out_memkind(loc,q,tvs,0);
# 611
Cyc_Tc_rule_out_memkind(loc,q,exist_vars,1);
# 613
if((int)ad->kind == (int)Cyc_Absyn_UnionA && !tagged){
# 616
struct Cyc_List_List*f=fs;for(0;f != 0;f=f->tl){
if((Cyc_Tc_aggressive_warn && !
Cyc_Tcutil_is_bits_only_type(((struct Cyc_Absyn_Aggrfield*)f->hd)->type))&&((struct Cyc_Absyn_Aggrfield*)f->hd)->requires_clause == 0)
({struct Cyc_String_pa_PrintArg_struct _tmp10E=({struct Cyc_String_pa_PrintArg_struct _tmp1C3;_tmp1C3.tag=0U,_tmp1C3.f1=(struct _fat_ptr)((struct _fat_ptr)*((struct Cyc_Absyn_Aggrfield*)f->hd)->name);_tmp1C3;});struct Cyc_String_pa_PrintArg_struct _tmp10F=({struct Cyc_String_pa_PrintArg_struct _tmp1C2;_tmp1C2.tag=0U,_tmp1C2.f1=(struct _fat_ptr)((struct _fat_ptr)*(*q).f2);_tmp1C2;});struct Cyc_String_pa_PrintArg_struct _tmp110=({struct Cyc_String_pa_PrintArg_struct _tmp1C1;_tmp1C1.tag=0U,({
struct _fat_ptr _tmp2B4=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(((struct Cyc_Absyn_Aggrfield*)f->hd)->type));_tmp1C1.f1=_tmp2B4;});_tmp1C1;});void*_tmp10C[3U];_tmp10C[0]=& _tmp10E,_tmp10C[1]=& _tmp10F,_tmp10C[2]=& _tmp110;({unsigned _tmp2B6=loc;struct _fat_ptr _tmp2B5=({const char*_tmp10D="member %s of union %s has type %s which is not `bits-only' so it can only be written and not read";_tag_fat(_tmp10D,sizeof(char),98U);});Cyc_Tcutil_warn(_tmp2B6,_tmp2B5,_tag_fat(_tmp10C,sizeof(void*),3U));});});}}
# 613
*adp=ad;
# 623
goto _LL7;}}else{_LLC: _tmpFD=(_tmpFC.f1)->exist_vars;_tmpFE=(_tmpFC.f1)->rgn_po;_tmpFF=(_tmpFC.f1)->fields;_tmp100=(_tmpFC.f1)->tagged;_tmp101=*_tmpFC.f2;_LLD: {struct Cyc_List_List*exist_vars=_tmpFD;struct Cyc_List_List*rgn_po=_tmpFE;struct Cyc_List_List*fs=_tmpFF;int tagged=_tmp100;struct Cyc_Absyn_Aggrdecl**adp=_tmp101;
# 626
if((int)ad->kind != (int)(*adp)->kind)
({void*_tmp113=0U;({unsigned _tmp2B8=loc;struct _fat_ptr _tmp2B7=({const char*_tmp114="cannot reuse struct names for unions and vice-versa";_tag_fat(_tmp114,sizeof(char),52U);});Cyc_Tcutil_terr(_tmp2B8,_tmp2B7,_tag_fat(_tmp113,sizeof(void*),0U));});});{
# 626
struct Cyc_Absyn_Aggrdecl*ad0=*adp;
# 631
({struct Cyc_Absyn_Aggrdecl*_tmp2B9=({struct Cyc_Absyn_Aggrdecl*_tmp115=_cycalloc(sizeof(*_tmp115));_tmp115->kind=ad->kind,_tmp115->sc=Cyc_Absyn_Extern,_tmp115->name=ad->name,_tmp115->tvs=tvs,_tmp115->impl=0,_tmp115->attributes=ad->attributes,_tmp115->expected_mem_kind=0;_tmp115;});*adp=_tmp2B9;});
# 637
Cyc_Tcutil_check_unique_tvars(loc,exist_vars);
Cyc_Tcutil_add_tvar_identities(exist_vars);
# 640
if(tagged &&(int)ad->kind == (int)Cyc_Absyn_StructA)
({void*_tmp116=0U;({unsigned _tmp2BB=loc;struct _fat_ptr _tmp2BA=({const char*_tmp117="@tagged qualifier is only allowed on union declarations";_tag_fat(_tmp117,sizeof(char),56U);});Cyc_Tcutil_terr(_tmp2BB,_tmp2BA,_tag_fat(_tmp116,sizeof(void*),0U));});});
# 640
({struct Cyc_Tcenv_Tenv*_tmp2C0=te;unsigned _tmp2BF=loc;enum Cyc_Absyn_AggrKind _tmp2BE=ad->kind;struct Cyc_List_List*_tmp2BD=
# 642
Cyc_List_append(tvs,exist_vars);
# 640
struct Cyc_List_List*_tmp2BC=rgn_po;Cyc_Tc_tcAggrImpl(_tmp2C0,_tmp2BF,_tmp2BE,_tmp2BD,_tmp2BC,fs);});
# 644
*adp=ad0;
_tmp106=adp;goto _LLF;}}}}_LL7:;}}}
# 47
static void Cyc_Tc_rule_out_mem_and_unique(unsigned loc,struct _tuple0*q,struct Cyc_List_List*tvs){
# 666
struct Cyc_List_List*tvs2=tvs;for(0;tvs2 != 0;tvs2=tvs2->tl){
void*_stmttmp15=Cyc_Absyn_compress_kb(((struct Cyc_Absyn_Tvar*)tvs2->hd)->kind);void*_tmp119=_stmttmp15;enum Cyc_Absyn_AliasQual _tmp11A;enum Cyc_Absyn_KindQual _tmp11B;enum Cyc_Absyn_KindQual _tmp11D;struct Cyc_Core_Opt**_tmp11C;struct Cyc_Core_Opt**_tmp11E;struct Cyc_Core_Opt**_tmp11F;struct Cyc_Core_Opt**_tmp120;struct Cyc_Core_Opt**_tmp121;struct Cyc_Core_Opt**_tmp122;struct Cyc_Core_Opt**_tmp123;switch(*((int*)_tmp119)){case 1U: _LL1: _tmp123=(struct Cyc_Core_Opt**)&((struct Cyc_Absyn_Unknown_kb_Absyn_KindBound_struct*)_tmp119)->f1;_LL2: {struct Cyc_Core_Opt**f=_tmp123;
_tmp122=f;goto _LL4;}case 2U: switch(((struct Cyc_Absyn_Kind*)((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp119)->f2)->kind){case Cyc_Absyn_MemKind: switch(((struct Cyc_Absyn_Kind*)((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp119)->f2)->aliasqual){case Cyc_Absyn_Top: _LL3: _tmp122=(struct Cyc_Core_Opt**)&((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp119)->f1;_LL4: {struct Cyc_Core_Opt**f=_tmp122;
# 670
_tmp121=f;goto _LL6;}case Cyc_Absyn_Aliasable: _LL5: _tmp121=(struct Cyc_Core_Opt**)&((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp119)->f1;_LL6: {struct Cyc_Core_Opt**f=_tmp121;
# 672
({struct Cyc_Core_Opt*_tmp2C2=({struct Cyc_Core_Opt*_tmp124=_cycalloc(sizeof(*_tmp124));({void*_tmp2C1=Cyc_Tcutil_kind_to_bound(& Cyc_Tcutil_bk);_tmp124->v=_tmp2C1;});_tmp124;});*f=_tmp2C2;});goto _LL0;}case Cyc_Absyn_Unique: goto _LLF;default: goto _LL15;}case Cyc_Absyn_AnyKind: switch(((struct Cyc_Absyn_Kind*)((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp119)->f2)->aliasqual){case Cyc_Absyn_Top: _LL7: _tmp120=(struct Cyc_Core_Opt**)&((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp119)->f1;_LL8: {struct Cyc_Core_Opt**f=_tmp120;
_tmp11F=f;goto _LLA;}case Cyc_Absyn_Aliasable: _LL9: _tmp11F=(struct Cyc_Core_Opt**)&((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp119)->f1;_LLA: {struct Cyc_Core_Opt**f=_tmp11F;
# 675
({struct Cyc_Core_Opt*_tmp2C4=({struct Cyc_Core_Opt*_tmp125=_cycalloc(sizeof(*_tmp125));({void*_tmp2C3=Cyc_Tcutil_kind_to_bound(& Cyc_Tcutil_ak);_tmp125->v=_tmp2C3;});_tmp125;});*f=_tmp2C4;});goto _LL0;}case Cyc_Absyn_Unique: goto _LLF;default: goto _LL15;}case Cyc_Absyn_RgnKind: switch(((struct Cyc_Absyn_Kind*)((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp119)->f2)->aliasqual){case Cyc_Absyn_Top: _LLB: _tmp11E=(struct Cyc_Core_Opt**)&((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp119)->f1;_LLC: {struct Cyc_Core_Opt**f=_tmp11E;
# 677
({struct Cyc_Core_Opt*_tmp2C6=({struct Cyc_Core_Opt*_tmp126=_cycalloc(sizeof(*_tmp126));({void*_tmp2C5=Cyc_Tcutil_kind_to_bound(& Cyc_Tcutil_rk);_tmp126->v=_tmp2C5;});_tmp126;});*f=_tmp2C6;});goto _LL0;}case Cyc_Absyn_Unique: goto _LLF;default: goto _LL15;}default: if(((struct Cyc_Absyn_Kind*)((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp119)->f2)->aliasqual == Cyc_Absyn_Unique){_LLF: _tmp11C=(struct Cyc_Core_Opt**)&((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp119)->f1;_tmp11D=(((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp119)->f2)->kind;_LL10: {struct Cyc_Core_Opt**f=_tmp11C;enum Cyc_Absyn_KindQual k=_tmp11D;
# 681
_tmp11B=k;goto _LL12;}}else{goto _LL15;}}default: if(((struct Cyc_Absyn_Kind*)((struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct*)_tmp119)->f1)->kind == Cyc_Absyn_RgnKind)switch(((struct Cyc_Absyn_Kind*)((struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct*)_tmp119)->f1)->aliasqual){case Cyc_Absyn_Top: _LLD: _LLE:
# 679
({struct Cyc_String_pa_PrintArg_struct _tmp129=({struct Cyc_String_pa_PrintArg_struct _tmp1C6;_tmp1C6.tag=0U,_tmp1C6.f1=(struct _fat_ptr)((struct _fat_ptr)*(*q).f2);_tmp1C6;});struct Cyc_String_pa_PrintArg_struct _tmp12A=({struct Cyc_String_pa_PrintArg_struct _tmp1C5;_tmp1C5.tag=0U,_tmp1C5.f1=(struct _fat_ptr)((struct _fat_ptr)*((struct Cyc_Absyn_Tvar*)tvs2->hd)->name);_tmp1C5;});void*_tmp127[2U];_tmp127[0]=& _tmp129,_tmp127[1]=& _tmp12A;({unsigned _tmp2C8=loc;struct _fat_ptr _tmp2C7=({const char*_tmp128="type %s attempts to abstract type variable %s of kind TR";_tag_fat(_tmp128,sizeof(char),57U);});Cyc_Tcutil_terr(_tmp2C8,_tmp2C7,_tag_fat(_tmp127,sizeof(void*),2U));});});
goto _LL0;case Cyc_Absyn_Unique: goto _LL11;default: goto _LL15;}else{if(((struct Cyc_Absyn_Kind*)((struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct*)_tmp119)->f1)->aliasqual == Cyc_Absyn_Unique){_LL11: _tmp11B=(((struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct*)_tmp119)->f1)->kind;_LL12: {enum Cyc_Absyn_KindQual k=_tmp11B;
# 683
({struct Cyc_String_pa_PrintArg_struct _tmp12D=({struct Cyc_String_pa_PrintArg_struct _tmp1C9;_tmp1C9.tag=0U,_tmp1C9.f1=(struct _fat_ptr)((struct _fat_ptr)*(*q).f2);_tmp1C9;});struct Cyc_String_pa_PrintArg_struct _tmp12E=({struct Cyc_String_pa_PrintArg_struct _tmp1C8;_tmp1C8.tag=0U,_tmp1C8.f1=(struct _fat_ptr)((struct _fat_ptr)*((struct Cyc_Absyn_Tvar*)tvs2->hd)->name);_tmp1C8;});struct Cyc_String_pa_PrintArg_struct _tmp12F=({struct Cyc_String_pa_PrintArg_struct _tmp1C7;_tmp1C7.tag=0U,({
# 685
struct _fat_ptr _tmp2C9=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_kind2string(({struct Cyc_Absyn_Kind*_tmp130=_cycalloc(sizeof(*_tmp130));_tmp130->kind=k,_tmp130->aliasqual=Cyc_Absyn_Unique;_tmp130;})));_tmp1C7.f1=_tmp2C9;});_tmp1C7;});void*_tmp12B[3U];_tmp12B[0]=& _tmp12D,_tmp12B[1]=& _tmp12E,_tmp12B[2]=& _tmp12F;({unsigned _tmp2CB=loc;struct _fat_ptr _tmp2CA=({const char*_tmp12C="type %s attempts to abstract type variable %s of kind %s";_tag_fat(_tmp12C,sizeof(char),57U);});Cyc_Tcutil_terr(_tmp2CB,_tmp2CA,_tag_fat(_tmp12B,sizeof(void*),3U));});});goto _LL0;}}else{if(((struct Cyc_Absyn_Kind*)((struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct*)_tmp119)->f1)->kind == Cyc_Absyn_MemKind){_LL13: _tmp11A=(((struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct*)_tmp119)->f1)->aliasqual;_LL14: {enum Cyc_Absyn_AliasQual a=_tmp11A;
# 687
({struct Cyc_String_pa_PrintArg_struct _tmp133=({struct Cyc_String_pa_PrintArg_struct _tmp1CC;_tmp1CC.tag=0U,_tmp1CC.f1=(struct _fat_ptr)((struct _fat_ptr)*(*q).f2);_tmp1CC;});struct Cyc_String_pa_PrintArg_struct _tmp134=({struct Cyc_String_pa_PrintArg_struct _tmp1CB;_tmp1CB.tag=0U,_tmp1CB.f1=(struct _fat_ptr)((struct _fat_ptr)*((struct Cyc_Absyn_Tvar*)tvs2->hd)->name);_tmp1CB;});struct Cyc_String_pa_PrintArg_struct _tmp135=({struct Cyc_String_pa_PrintArg_struct _tmp1CA;_tmp1CA.tag=0U,({
# 689
struct _fat_ptr _tmp2CC=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_kind2string(({struct Cyc_Absyn_Kind*_tmp136=_cycalloc(sizeof(*_tmp136));_tmp136->kind=Cyc_Absyn_MemKind,_tmp136->aliasqual=a;_tmp136;})));_tmp1CA.f1=_tmp2CC;});_tmp1CA;});void*_tmp131[3U];_tmp131[0]=& _tmp133,_tmp131[1]=& _tmp134,_tmp131[2]=& _tmp135;({unsigned _tmp2CE=loc;struct _fat_ptr _tmp2CD=({const char*_tmp132="type %s attempts to abstract type variable %s of kind %s";_tag_fat(_tmp132,sizeof(char),57U);});Cyc_Tcutil_terr(_tmp2CE,_tmp2CD,_tag_fat(_tmp131,sizeof(void*),3U));});});goto _LL0;}}else{_LL15: _LL16:
 goto _LL0;}}}}_LL0:;}}struct _tuple17{struct Cyc_Absyn_Tqual f1;void*f2;};
# 47
static struct Cyc_List_List*Cyc_Tc_tcDatatypeFields(struct Cyc_Tcenv_Tenv*te,unsigned loc,struct _fat_ptr obj,int is_extensible,struct _tuple0*name,struct Cyc_List_List*fields,struct Cyc_List_List*tvs,struct Cyc_Absyn_Datatypedecl*tudres){
# 703
{struct Cyc_List_List*fs=fields;for(0;fs != 0;fs=fs->tl){
struct Cyc_Absyn_Datatypefield*f=(struct Cyc_Absyn_Datatypefield*)fs->hd;
struct Cyc_List_List*typs=f->typs;for(0;typs != 0;typs=typs->tl){
Cyc_Tctyp_check_type(f->loc,te,tvs,& Cyc_Tcutil_tmk,0,0,(*((struct _tuple17*)typs->hd)).f2);
# 708
if(Cyc_Tcutil_is_noalias_pointer_or_aggr((*((struct _tuple17*)typs->hd)).f2))
({struct Cyc_String_pa_PrintArg_struct _tmp139=({struct Cyc_String_pa_PrintArg_struct _tmp1CD;_tmp1CD.tag=0U,({
# 711
struct _fat_ptr _tmp2CF=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(f->name));_tmp1CD.f1=_tmp2CF;});_tmp1CD;});void*_tmp137[1U];_tmp137[0]=& _tmp139;({unsigned _tmp2D1=f->loc;struct _fat_ptr _tmp2D0=({const char*_tmp138="noalias pointers in datatypes are not allowed (%s)";_tag_fat(_tmp138,sizeof(char),51U);});Cyc_Tcutil_terr(_tmp2D1,_tmp2D0,_tag_fat(_tmp137,sizeof(void*),1U));});});
# 708
({int _tmp2D2=
# 713
Cyc_Tcutil_extract_const_from_typedef(f->loc,((*((struct _tuple17*)typs->hd)).f1).print_const,(*((struct _tuple17*)typs->hd)).f2);
# 708
((*((struct _tuple17*)typs->hd)).f1).real_const=_tmp2D2;});}}}{
# 718
struct Cyc_List_List*fields2;
if(is_extensible){
# 721
int res=1;
struct Cyc_List_List*fs=Cyc_Tcdecl_sort_xdatatype_fields(fields,& res,(*name).f2,loc,Cyc_Tc_tc_msg);
if(res)
fields2=fs;else{
# 726
fields2=0;}}else{
# 728
struct _RegionHandle _tmp13A=_new_region("uprev_rgn");struct _RegionHandle*uprev_rgn=& _tmp13A;_push_region(uprev_rgn);
# 730
{struct Cyc_List_List*prev_fields=0;
{struct Cyc_List_List*fs=fields;for(0;fs != 0;fs=fs->tl){
struct Cyc_Absyn_Datatypefield*f=(struct Cyc_Absyn_Datatypefield*)fs->hd;
if(({int(*_tmp2D4)(int(*compare)(struct _fat_ptr*,struct _fat_ptr*),struct Cyc_List_List*l,struct _fat_ptr*x)=({int(*_tmp13B)(int(*compare)(struct _fat_ptr*,struct _fat_ptr*),struct Cyc_List_List*l,struct _fat_ptr*x)=(int(*)(int(*compare)(struct _fat_ptr*,struct _fat_ptr*),struct Cyc_List_List*l,struct _fat_ptr*x))Cyc_List_mem;_tmp13B;});struct Cyc_List_List*_tmp2D3=prev_fields;_tmp2D4(Cyc_strptrcmp,_tmp2D3,(*f->name).f2);}))
({struct Cyc_String_pa_PrintArg_struct _tmp13E=({struct Cyc_String_pa_PrintArg_struct _tmp1CF;_tmp1CF.tag=0U,_tmp1CF.f1=(struct _fat_ptr)((struct _fat_ptr)*(*f->name).f2);_tmp1CF;});struct Cyc_String_pa_PrintArg_struct _tmp13F=({struct Cyc_String_pa_PrintArg_struct _tmp1CE;_tmp1CE.tag=0U,_tmp1CE.f1=(struct _fat_ptr)((struct _fat_ptr)obj);_tmp1CE;});void*_tmp13C[2U];_tmp13C[0]=& _tmp13E,_tmp13C[1]=& _tmp13F;({unsigned _tmp2D6=f->loc;struct _fat_ptr _tmp2D5=({const char*_tmp13D="duplicate field name %s in %s";_tag_fat(_tmp13D,sizeof(char),30U);});Cyc_Tcutil_terr(_tmp2D6,_tmp2D5,_tag_fat(_tmp13C,sizeof(void*),2U));});});else{
# 736
prev_fields=({struct Cyc_List_List*_tmp140=_region_malloc(uprev_rgn,sizeof(*_tmp140));_tmp140->hd=(*f->name).f2,_tmp140->tl=prev_fields;_tmp140;});}
# 738
if((int)f->sc != (int)Cyc_Absyn_Public){
({struct Cyc_String_pa_PrintArg_struct _tmp143=({struct Cyc_String_pa_PrintArg_struct _tmp1D0;_tmp1D0.tag=0U,_tmp1D0.f1=(struct _fat_ptr)((struct _fat_ptr)*(*f->name).f2);_tmp1D0;});void*_tmp141[1U];_tmp141[0]=& _tmp143;({unsigned _tmp2D8=loc;struct _fat_ptr _tmp2D7=({const char*_tmp142="ignoring scope of field %s";_tag_fat(_tmp142,sizeof(char),27U);});Cyc_Tcutil_warn(_tmp2D8,_tmp2D7,_tag_fat(_tmp141,sizeof(void*),1U));});});
f->sc=Cyc_Absyn_Public;}}}
# 743
fields2=fields;}
# 730
;_pop_region(uprev_rgn);}
# 745
return fields2;}}struct _tuple18{struct Cyc_Core_Opt*f1;struct Cyc_Absyn_Datatypedecl***f2;};
# 47
void Cyc_Tc_tcDatatypedecl(struct Cyc_Tcenv_Tenv*te,unsigned loc,struct Cyc_Absyn_Datatypedecl*tud){
# 749
struct _tuple0*q=tud->name;
struct _fat_ptr obj=tud->is_extensible?({const char*_tmp157="@extensible datatype";_tag_fat(_tmp157,sizeof(char),21U);}):({const char*_tmp158="datatype";_tag_fat(_tmp158,sizeof(char),9U);});
# 755
struct Cyc_List_List*tvs=tud->tvs;
# 758
Cyc_Tcutil_check_unique_tvars(loc,tvs);
Cyc_Tcutil_add_tvar_identities(tvs);{
# 764
struct Cyc_Absyn_Datatypedecl***tud_opt;
# 774 "tc.cyc"
{struct _handler_cons _tmp144;_push_handler(& _tmp144);{int _tmp146=0;if(setjmp(_tmp144.handler))_tmp146=1;if(!_tmp146){
tud_opt=Cyc_Tcenv_lookup_xdatatypedecl(Cyc_Core_heap_region,te,loc,tud->name);
if(tud_opt == 0 && !tud->is_extensible)(int)_throw((void*)({struct Cyc_Dict_Absent_exn_struct*_tmp147=_cycalloc(sizeof(*_tmp147));_tmp147->tag=Cyc_Dict_Absent;_tmp147;}));if(tud_opt != 0)
# 778
tud->name=(*(*tud_opt))->name;else{
# 780
({union Cyc_Absyn_Nmspace _tmp2D9=Cyc_Absyn_Abs_n(te->ns,0);(*tud->name).f1=_tmp2D9;});}
# 775
;_pop_handler();}else{void*_tmp145=(void*)Cyc_Core_get_exn_thrown();void*_tmp148=_tmp145;void*_tmp149;if(((struct Cyc_Dict_Absent_exn_struct*)_tmp148)->tag == Cyc_Dict_Absent){_LL1: _LL2: {
# 784
struct Cyc_Absyn_Datatypedecl***tdopt=({struct Cyc_Absyn_Datatypedecl***(*_tmp2DB)(struct Cyc_Dict_Dict d,struct _tuple0*k)=({struct Cyc_Absyn_Datatypedecl***(*_tmp14B)(struct Cyc_Dict_Dict d,struct _tuple0*k)=(struct Cyc_Absyn_Datatypedecl***(*)(struct Cyc_Dict_Dict d,struct _tuple0*k))Cyc_Dict_lookup_opt;_tmp14B;});struct Cyc_Dict_Dict _tmp2DA=(te->ae)->datatypedecls;_tmp2DB(_tmp2DA,q);});
tud_opt=(unsigned)tdopt?({struct Cyc_Absyn_Datatypedecl***_tmp14A=_cycalloc(sizeof(*_tmp14A));*_tmp14A=*tdopt;_tmp14A;}): 0;
goto _LL0;}}else{_LL3: _tmp149=_tmp148;_LL4: {void*exn=_tmp149;(int)_rethrow(exn);}}_LL0:;}}}{
# 791
struct _tuple18 _stmttmp16=({struct _tuple18 _tmp1D1;_tmp1D1.f1=tud->fields,_tmp1D1.f2=tud_opt;_tmp1D1;});struct _tuple18 _tmp14C=_stmttmp16;struct Cyc_Absyn_Datatypedecl**_tmp14E;struct Cyc_List_List**_tmp14D;struct Cyc_List_List**_tmp14F;struct Cyc_Absyn_Datatypedecl**_tmp150;if(_tmp14C.f1 == 0){if(_tmp14C.f2 == 0){_LL6: _LL7:
# 794
 Cyc_Tc_rule_out_mem_and_unique(loc,q,tvs);
({struct Cyc_Dict_Dict _tmp2DF=({struct Cyc_Dict_Dict(*_tmp2DE)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Datatypedecl**v)=({struct Cyc_Dict_Dict(*_tmp151)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Datatypedecl**v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Datatypedecl**v))Cyc_Dict_insert;_tmp151;});struct Cyc_Dict_Dict _tmp2DD=(te->ae)->datatypedecls;struct _tuple0*_tmp2DC=q;_tmp2DE(_tmp2DD,_tmp2DC,({struct Cyc_Absyn_Datatypedecl**_tmp152=_cycalloc(sizeof(*_tmp152));*_tmp152=tud;_tmp152;}));});(te->ae)->datatypedecls=_tmp2DF;});
goto _LL5;}else{_LLC: _tmp150=*_tmp14C.f2;_LLD: {struct Cyc_Absyn_Datatypedecl**tudp=_tmp150;
# 827
struct Cyc_Absyn_Datatypedecl*tud2=Cyc_Tcdecl_merge_datatypedecl(*tudp,tud,loc,Cyc_Tc_tc_msg);
Cyc_Tc_rule_out_mem_and_unique(loc,q,tvs);
if(tud2 == 0)
return;else{
# 832
*tudp=tud2;
goto _LL5;}}}}else{if(_tmp14C.f2 == 0){_LL8: _tmp14F=(struct Cyc_List_List**)&(_tmp14C.f1)->v;_LL9: {struct Cyc_List_List**fs=_tmp14F;
# 799
struct Cyc_Absyn_Datatypedecl**tudp=({struct Cyc_Absyn_Datatypedecl**_tmp155=_cycalloc(sizeof(*_tmp155));({struct Cyc_Absyn_Datatypedecl*_tmp2E0=({struct Cyc_Absyn_Datatypedecl*_tmp154=_cycalloc(sizeof(*_tmp154));_tmp154->sc=Cyc_Absyn_Extern,_tmp154->name=tud->name,_tmp154->tvs=tvs,_tmp154->fields=0,_tmp154->is_extensible=tud->is_extensible;_tmp154;});*_tmp155=_tmp2E0;});_tmp155;});
# 801
({struct Cyc_Dict_Dict _tmp2E4=({struct Cyc_Dict_Dict(*_tmp2E3)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Datatypedecl**v)=({struct Cyc_Dict_Dict(*_tmp153)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Datatypedecl**v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Datatypedecl**v))Cyc_Dict_insert;_tmp153;});struct Cyc_Dict_Dict _tmp2E2=(te->ae)->datatypedecls;struct _tuple0*_tmp2E1=q;_tmp2E3(_tmp2E2,_tmp2E1,tudp);});(te->ae)->datatypedecls=_tmp2E4;});
# 804
({struct Cyc_List_List*_tmp2E5=Cyc_Tc_tcDatatypeFields(te,loc,obj,tud->is_extensible,tud->name,*fs,tvs,tud);*fs=_tmp2E5;});
Cyc_Tc_rule_out_mem_and_unique(loc,q,tvs);
*tudp=tud;
goto _LL5;}}else{_LLA: _tmp14D=(struct Cyc_List_List**)&(_tmp14C.f1)->v;_tmp14E=*_tmp14C.f2;_LLB: {struct Cyc_List_List**fs=_tmp14D;struct Cyc_Absyn_Datatypedecl**tudp=_tmp14E;
# 809
struct Cyc_Absyn_Datatypedecl*tud0=*tudp;
# 812
if((!tud->is_extensible &&(unsigned)tud0)&& tud0->is_extensible)
tud->is_extensible=1;
# 812
({struct Cyc_Absyn_Datatypedecl*_tmp2E6=({struct Cyc_Absyn_Datatypedecl*_tmp156=_cycalloc(sizeof(*_tmp156));
# 815
_tmp156->sc=Cyc_Absyn_Extern,_tmp156->name=tud->name,_tmp156->tvs=tvs,_tmp156->fields=0,_tmp156->is_extensible=tud->is_extensible;_tmp156;});
# 812
*tudp=_tmp2E6;});
# 819
({struct Cyc_List_List*_tmp2E7=Cyc_Tc_tcDatatypeFields(te,loc,obj,tud->is_extensible,tud->name,*fs,tvs,tud);*fs=_tmp2E7;});
# 822
*tudp=tud0;
_tmp150=tudp;goto _LLD;}}}_LL5:;}}}
# 47 "tc.cyc"
void Cyc_Tc_tcEnumdecl(struct Cyc_Tcenv_Tenv*te,unsigned loc,struct Cyc_Absyn_Enumdecl*ed){
# 839 "tc.cyc"
struct _tuple0*q=ed->name;
# 845
if(ed->fields != 0){struct _RegionHandle _tmp159=_new_region("uprev_rgn");struct _RegionHandle*uprev_rgn=& _tmp159;_push_region(uprev_rgn);
{struct Cyc_List_List*prev_fields=0;
unsigned tag_count=0U;
struct Cyc_List_List*fs=(struct Cyc_List_List*)((struct Cyc_Core_Opt*)_check_null(ed->fields))->v;for(0;fs != 0;fs=fs->tl){
struct Cyc_Absyn_Enumfield*f=(struct Cyc_Absyn_Enumfield*)fs->hd;
# 851
if(({int(*_tmp2E9)(int(*compare)(struct _fat_ptr*,struct _fat_ptr*),struct Cyc_List_List*l,struct _fat_ptr*x)=({int(*_tmp15A)(int(*compare)(struct _fat_ptr*,struct _fat_ptr*),struct Cyc_List_List*l,struct _fat_ptr*x)=(int(*)(int(*compare)(struct _fat_ptr*,struct _fat_ptr*),struct Cyc_List_List*l,struct _fat_ptr*x))Cyc_List_mem;_tmp15A;});struct Cyc_List_List*_tmp2E8=prev_fields;_tmp2E9(Cyc_strptrcmp,_tmp2E8,(*f->name).f2);}))
({struct Cyc_String_pa_PrintArg_struct _tmp15D=({struct Cyc_String_pa_PrintArg_struct _tmp1D2;_tmp1D2.tag=0U,_tmp1D2.f1=(struct _fat_ptr)((struct _fat_ptr)*(*f->name).f2);_tmp1D2;});void*_tmp15B[1U];_tmp15B[0]=& _tmp15D;({unsigned _tmp2EB=f->loc;struct _fat_ptr _tmp2EA=({const char*_tmp15C="duplicate enum constructor %s";_tag_fat(_tmp15C,sizeof(char),30U);});Cyc_Tcutil_terr(_tmp2EB,_tmp2EA,_tag_fat(_tmp15B,sizeof(void*),1U));});});else{
# 854
prev_fields=({struct Cyc_List_List*_tmp15E=_region_malloc(uprev_rgn,sizeof(*_tmp15E));_tmp15E->hd=(*f->name).f2,_tmp15E->tl=prev_fields;_tmp15E;});}
# 856
if(({struct _tuple14**(*_tmp2ED)(struct Cyc_Dict_Dict d,struct _tuple0*k)=({struct _tuple14**(*_tmp15F)(struct Cyc_Dict_Dict d,struct _tuple0*k)=(struct _tuple14**(*)(struct Cyc_Dict_Dict d,struct _tuple0*k))Cyc_Dict_lookup_opt;_tmp15F;});struct Cyc_Dict_Dict _tmp2EC=(te->ae)->ordinaries;_tmp2ED(_tmp2EC,f->name);})!= 0)
({struct Cyc_String_pa_PrintArg_struct _tmp162=({struct Cyc_String_pa_PrintArg_struct _tmp1D3;_tmp1D3.tag=0U,_tmp1D3.f1=(struct _fat_ptr)((struct _fat_ptr)*(*f->name).f2);_tmp1D3;});void*_tmp160[1U];_tmp160[0]=& _tmp162;({unsigned _tmp2EF=f->loc;struct _fat_ptr _tmp2EE=({const char*_tmp161="enum field name %s shadows global name";_tag_fat(_tmp161,sizeof(char),39U);});Cyc_Tcutil_terr(_tmp2EF,_tmp2EE,_tag_fat(_tmp160,sizeof(void*),1U));});});
# 856
if(f->tag == 0){
# 860
({struct Cyc_Absyn_Exp*_tmp2F0=Cyc_Absyn_uint_exp(tag_count,f->loc);f->tag=_tmp2F0;});
++ tag_count;}else{
# 864
if(Cyc_Tcutil_is_const_exp((struct Cyc_Absyn_Exp*)_check_null(f->tag))){
struct _tuple11 _stmttmp17=Cyc_Evexp_eval_const_uint_exp((struct Cyc_Absyn_Exp*)_check_null(f->tag));int _tmp164;unsigned _tmp163;_LL1: _tmp163=_stmttmp17.f1;_tmp164=_stmttmp17.f2;_LL2: {unsigned t1=_tmp163;int known=_tmp164;
if(known)tag_count=t1 + (unsigned)1;}}}}}
# 846
;_pop_region(uprev_rgn);}
# 845
{struct _handler_cons _tmp165;_push_handler(& _tmp165);{int _tmp167=0;if(setjmp(_tmp165.handler))_tmp167=1;if(!_tmp167){
# 873
{struct Cyc_Absyn_Enumdecl**edp=({struct Cyc_Absyn_Enumdecl**(*_tmp2F2)(struct Cyc_Dict_Dict d,struct _tuple0*k)=({struct Cyc_Absyn_Enumdecl**(*_tmp168)(struct Cyc_Dict_Dict d,struct _tuple0*k)=(struct Cyc_Absyn_Enumdecl**(*)(struct Cyc_Dict_Dict d,struct _tuple0*k))Cyc_Dict_lookup;_tmp168;});struct Cyc_Dict_Dict _tmp2F1=(te->ae)->enumdecls;_tmp2F2(_tmp2F1,q);});
struct Cyc_Absyn_Enumdecl*ed2=Cyc_Tcdecl_merge_enumdecl(*edp,ed,loc,Cyc_Tc_tc_msg);
if(ed2 == 0){_npop_handler(0U);return;}*edp=ed2;}
# 873
;_pop_handler();}else{void*_tmp166=(void*)Cyc_Core_get_exn_thrown();void*_tmp169=_tmp166;void*_tmp16A;if(((struct Cyc_Dict_Absent_exn_struct*)_tmp169)->tag == Cyc_Dict_Absent){_LL4: _LL5: {
# 878
struct Cyc_Absyn_Enumdecl**edp=({struct Cyc_Absyn_Enumdecl**_tmp16C=_cycalloc(sizeof(*_tmp16C));*_tmp16C=ed;_tmp16C;});
({struct Cyc_Dict_Dict _tmp2F6=({struct Cyc_Dict_Dict(*_tmp2F5)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Enumdecl**v)=({struct Cyc_Dict_Dict(*_tmp16B)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Enumdecl**v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _tuple0*k,struct Cyc_Absyn_Enumdecl**v))Cyc_Dict_insert;_tmp16B;});struct Cyc_Dict_Dict _tmp2F4=(te->ae)->enumdecls;struct _tuple0*_tmp2F3=q;_tmp2F5(_tmp2F4,_tmp2F3,edp);});(te->ae)->enumdecls=_tmp2F6;});
goto _LL3;}}else{_LL6: _tmp16A=_tmp169;_LL7: {void*exn=_tmp16A;(int)_rethrow(exn);}}_LL3:;}}}
# 884
if(ed->fields != 0){
struct Cyc_List_List*fs=(struct Cyc_List_List*)((struct Cyc_Core_Opt*)_check_null(ed->fields))->v;for(0;fs != 0;fs=fs->tl){
struct Cyc_Absyn_Enumfield*f=(struct Cyc_Absyn_Enumfield*)fs->hd;
Cyc_Tcexp_tcExp(te,0,(struct Cyc_Absyn_Exp*)_check_null(f->tag));
if(!Cyc_Tcutil_is_const_exp((struct Cyc_Absyn_Exp*)_check_null(f->tag)))
({struct Cyc_String_pa_PrintArg_struct _tmp16F=({struct Cyc_String_pa_PrintArg_struct _tmp1D5;_tmp1D5.tag=0U,_tmp1D5.f1=(struct _fat_ptr)((struct _fat_ptr)*(*q).f2);_tmp1D5;});struct Cyc_String_pa_PrintArg_struct _tmp170=({struct Cyc_String_pa_PrintArg_struct _tmp1D4;_tmp1D4.tag=0U,_tmp1D4.f1=(struct _fat_ptr)((struct _fat_ptr)*(*f->name).f2);_tmp1D4;});void*_tmp16D[2U];_tmp16D[0]=& _tmp16F,_tmp16D[1]=& _tmp170;({unsigned _tmp2F8=loc;struct _fat_ptr _tmp2F7=({const char*_tmp16E="enum %s, field %s: expression is not constant";_tag_fat(_tmp16E,sizeof(char),46U);});Cyc_Tcutil_terr(_tmp2F8,_tmp2F7,_tag_fat(_tmp16D,sizeof(void*),2U));});});}}}
# 47 "tc.cyc"
static int Cyc_Tc_okay_externC(unsigned loc,enum Cyc_Absyn_Scope sc,int in_include){
# 895 "tc.cyc"
enum Cyc_Absyn_Scope _tmp171=sc;switch(_tmp171){case Cyc_Absyn_Static: _LL1: _LL2:
# 897
 if(!in_include)
({void*_tmp172=0U;({unsigned _tmp2FA=loc;struct _fat_ptr _tmp2F9=({const char*_tmp173="static declaration within extern \"C\"";_tag_fat(_tmp173,sizeof(char),37U);});Cyc_Tcutil_warn(_tmp2FA,_tmp2F9,_tag_fat(_tmp172,sizeof(void*),0U));});});
# 897
return 0;case Cyc_Absyn_Abstract: _LL3: _LL4:
# 901
({void*_tmp174=0U;({unsigned _tmp2FC=loc;struct _fat_ptr _tmp2FB=({const char*_tmp175="abstract declaration within extern \"C\"";_tag_fat(_tmp175,sizeof(char),39U);});Cyc_Tcutil_warn(_tmp2FC,_tmp2FB,_tag_fat(_tmp174,sizeof(void*),0U));});});
return 0;case Cyc_Absyn_Public: _LL5: _LL6:
 goto _LL8;case Cyc_Absyn_Register: _LL7: _LL8:
 goto _LLA;case Cyc_Absyn_Extern: _LL9: _LLA:
 return 1;case Cyc_Absyn_ExternC: _LLB: _LLC:
 goto _LLE;default: _LLD: _LLE:
# 908
({void*_tmp176=0U;({unsigned _tmp2FE=loc;struct _fat_ptr _tmp2FD=({const char*_tmp177="nested extern \"C\" declaration";_tag_fat(_tmp177,sizeof(char),30U);});Cyc_Tcutil_warn(_tmp2FE,_tmp2FD,_tag_fat(_tmp176,sizeof(void*),0U));});});
return 1;}_LL0:;}
# 47 "tc.cyc"
static void Cyc_Tc_tc_decls(struct Cyc_Tcenv_Tenv*te,struct Cyc_List_List*ds0,int in_externC,int check_var_init,struct Cyc_List_List**exports){
# 922 "tc.cyc"
struct Cyc_List_List*ds=ds0;for(0;ds != 0;ds=ds->tl){
struct Cyc_Absyn_Decl*d=(struct Cyc_Absyn_Decl*)ds->hd;
unsigned loc=d->loc;
void*_stmttmp18=d->r;void*_tmp178=_stmttmp18;struct _tuple10*_tmp17C;struct Cyc_List_List*_tmp17B;struct Cyc_List_List*_tmp17A;struct Cyc_List_List*_tmp179;struct Cyc_List_List*_tmp17D;struct Cyc_List_List*_tmp17E;struct Cyc_List_List*_tmp180;struct _fat_ptr*_tmp17F;struct Cyc_Absyn_Enumdecl*_tmp181;struct Cyc_Absyn_Datatypedecl*_tmp182;struct Cyc_Absyn_Aggrdecl*_tmp183;struct Cyc_Absyn_Typedefdecl*_tmp184;struct Cyc_Absyn_Fndecl*_tmp185;struct Cyc_Absyn_Vardecl*_tmp186;switch(*((int*)_tmp178)){case 2U: _LL1: _LL2:
 goto _LL4;case 3U: _LL3: _LL4:
# 928
({void*_tmp187=0U;({unsigned _tmp300=loc;struct _fat_ptr _tmp2FF=({const char*_tmp188="top level let-declarations are not implemented";_tag_fat(_tmp188,sizeof(char),47U);});Cyc_Tcutil_terr(_tmp300,_tmp2FF,_tag_fat(_tmp187,sizeof(void*),0U));});});goto _LL0;case 4U: _LL5: _LL6:
# 930
({void*_tmp189=0U;({unsigned _tmp302=loc;struct _fat_ptr _tmp301=({const char*_tmp18A="top level region declarations are not implemented";_tag_fat(_tmp18A,sizeof(char),50U);});Cyc_Tcutil_terr(_tmp302,_tmp301,_tag_fat(_tmp189,sizeof(void*),0U));});});goto _LL0;case 0U: _LL7: _tmp186=((struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct*)_tmp178)->f1;_LL8: {struct Cyc_Absyn_Vardecl*vd=_tmp186;
# 932
if(in_externC && Cyc_Tc_okay_externC(d->loc,vd->sc,te->in_extern_c_include))
vd->sc=Cyc_Absyn_ExternC;
# 932
Cyc_Tc_tcVardecl(te,loc,vd,check_var_init,te->in_extern_c_include,exports);
# 935
goto _LL0;}case 1U: _LL9: _tmp185=((struct Cyc_Absyn_Fn_d_Absyn_Raw_decl_struct*)_tmp178)->f1;_LLA: {struct Cyc_Absyn_Fndecl*fd=_tmp185;
# 937
if(in_externC && Cyc_Tc_okay_externC(d->loc,fd->sc,te->in_extern_c_include))
fd->sc=Cyc_Absyn_ExternC;
# 937
Cyc_Tc_tcFndecl(te,loc,fd,exports);
# 940
goto _LL0;}case 8U: _LLB: _tmp184=((struct Cyc_Absyn_Typedef_d_Absyn_Raw_decl_struct*)_tmp178)->f1;_LLC: {struct Cyc_Absyn_Typedefdecl*td=_tmp184;
# 942
td->extern_c=te->in_extern_c_include;
# 946
Cyc_Tc_tcTypedefdecl(te,loc,td);
goto _LL0;}case 5U: _LLD: _tmp183=((struct Cyc_Absyn_Aggr_d_Absyn_Raw_decl_struct*)_tmp178)->f1;_LLE: {struct Cyc_Absyn_Aggrdecl*ad=_tmp183;
# 949
if(in_externC && Cyc_Tc_okay_externC(d->loc,ad->sc,te->in_extern_c_include))
ad->sc=Cyc_Absyn_ExternC;
# 949
Cyc_Tc_tcAggrdecl(te,loc,ad);
# 952
goto _LL0;}case 6U: _LLF: _tmp182=((struct Cyc_Absyn_Datatype_d_Absyn_Raw_decl_struct*)_tmp178)->f1;_LL10: {struct Cyc_Absyn_Datatypedecl*tud=_tmp182;
# 954
if(in_externC && Cyc_Tc_okay_externC(d->loc,tud->sc,te->in_extern_c_include))
tud->sc=Cyc_Absyn_ExternC;
# 954
Cyc_Tc_tcDatatypedecl(te,loc,tud);
# 957
goto _LL0;}case 7U: _LL11: _tmp181=((struct Cyc_Absyn_Enum_d_Absyn_Raw_decl_struct*)_tmp178)->f1;_LL12: {struct Cyc_Absyn_Enumdecl*ed=_tmp181;
# 959
if(in_externC && Cyc_Tc_okay_externC(d->loc,ed->sc,te->in_extern_c_include))
ed->sc=Cyc_Absyn_ExternC;
# 959
Cyc_Tc_tcEnumdecl(te,loc,ed);
# 962
goto _LL0;}case 13U: _LL13: _LL14:
({void*_tmp18B=0U;({unsigned _tmp304=d->loc;struct _fat_ptr _tmp303=({const char*_tmp18C="spurious __cyclone_port_on__";_tag_fat(_tmp18C,sizeof(char),29U);});Cyc_Tcutil_warn(_tmp304,_tmp303,_tag_fat(_tmp18B,sizeof(void*),0U));});});goto _LL0;case 14U: _LL15: _LL16:
 goto _LL0;case 15U: _LL17: _LL18:
 te=Cyc_Tcenv_enter_tempest(te);goto _LL0;case 16U: _LL19: _LL1A:
 te=Cyc_Tcenv_clear_tempest(te);goto _LL0;case 9U: _LL1B: _tmp17F=((struct Cyc_Absyn_Namespace_d_Absyn_Raw_decl_struct*)_tmp178)->f1;_tmp180=((struct Cyc_Absyn_Namespace_d_Absyn_Raw_decl_struct*)_tmp178)->f2;_LL1C: {struct _fat_ptr*v=_tmp17F;struct Cyc_List_List*ds2=_tmp180;
# 969
struct Cyc_List_List*ns=te->ns;
({struct Cyc_List_List*_tmp306=({struct Cyc_List_List*_tmp305=ns;Cyc_List_append(_tmp305,({struct Cyc_List_List*_tmp18D=_cycalloc(sizeof(*_tmp18D));_tmp18D->hd=v,_tmp18D->tl=0;_tmp18D;}));});te->ns=_tmp306;});
Cyc_Tc_tc_decls(te,ds2,in_externC,check_var_init,exports);
te->ns=ns;
goto _LL0;}case 10U: _LL1D: _tmp17E=((struct Cyc_Absyn_Using_d_Absyn_Raw_decl_struct*)_tmp178)->f2;_LL1E: {struct Cyc_List_List*ds2=_tmp17E;
# 976
Cyc_Tc_tc_decls(te,ds2,in_externC,check_var_init,exports);goto _LL0;}case 11U: _LL1F: _tmp17D=((struct Cyc_Absyn_ExternC_d_Absyn_Raw_decl_struct*)_tmp178)->f1;_LL20: {struct Cyc_List_List*ds2=_tmp17D;
# 979
Cyc_Tc_tc_decls(te,ds2,1,check_var_init,exports);goto _LL0;}default: _LL21: _tmp179=((struct Cyc_Absyn_ExternCinclude_d_Absyn_Raw_decl_struct*)_tmp178)->f1;_tmp17A=((struct Cyc_Absyn_ExternCinclude_d_Absyn_Raw_decl_struct*)_tmp178)->f2;_tmp17B=((struct Cyc_Absyn_ExternCinclude_d_Absyn_Raw_decl_struct*)_tmp178)->f3;_tmp17C=((struct Cyc_Absyn_ExternCinclude_d_Absyn_Raw_decl_struct*)_tmp178)->f4;_LL22: {struct Cyc_List_List*ds2=_tmp179;struct Cyc_List_List*ovrs=_tmp17A;struct Cyc_List_List*exports2=_tmp17B;struct _tuple10*wc=_tmp17C;
# 984
struct Cyc_List_List*newexs=
Cyc_List_append(exports2,(unsigned)exports?*exports: 0);
struct Cyc_Tcenv_Tenv*te2=Cyc_Tcenv_enter_extern_c_include(te);
Cyc_Tc_tc_decls(te2,ds2,1,check_var_init,& newexs);
# 989
for(0;exports2 != 0;exports2=exports2->tl){
struct _tuple12*exp=(struct _tuple12*)exports2->hd;
if(!(*exp).f3)
({struct Cyc_String_pa_PrintArg_struct _tmp190=({struct Cyc_String_pa_PrintArg_struct _tmp1D6;_tmp1D6.tag=0U,({
struct _fat_ptr _tmp307=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string((*exp).f2));_tmp1D6.f1=_tmp307;});_tmp1D6;});void*_tmp18E[1U];_tmp18E[0]=& _tmp190;({unsigned _tmp309=(*exp).f1;struct _fat_ptr _tmp308=({const char*_tmp18F="%s is exported but not defined";_tag_fat(_tmp18F,sizeof(char),31U);});Cyc_Tcutil_warn(_tmp309,_tmp308,_tag_fat(_tmp18E,sizeof(void*),1U));});});}
# 995
goto _LL0;}}_LL0:;}}
# 47 "tc.cyc"
struct Cyc___cycFILE*Cyc_Tc_dotfile=0;
# 1002 "tc.cyc"
void Cyc_Tc_tc(struct Cyc_Tcenv_Tenv*te,int check_var_init,struct Cyc_List_List*ds){
Cyc_Absynpp_set_params(& Cyc_Absynpp_tc_params_r);{
struct Cyc_Dict_Dict cg=Cyc_Callgraph_compute_callgraph(ds);
# 1006
struct Cyc_Dict_Dict scc=Cyc_Graph_scc(cg);
if(Cyc_Tc_dotfile == 0)Cyc_Tc_dotfile=Cyc_fopen("cyclone_graph.dot","w+");if(Cyc_Tc_dotfile != 0)
Cyc_Callgraph_print_callgraph((struct Cyc___cycFILE*)_check_null(Cyc_Tc_dotfile),cg);
# 1007
Cyc_Tc_tc_decls(te,ds,0,check_var_init,0);}}struct Cyc_Tc_TreeshakeEnv{int in_cinclude;struct Cyc_Dict_Dict ordinaries;};
# 1029 "tc.cyc"
static int Cyc_Tc_vardecl_needed(struct Cyc_Tc_TreeshakeEnv*env,struct Cyc_Absyn_Decl*d);
# 1031
static struct Cyc_List_List*Cyc_Tc_treeshake_f(struct Cyc_Tc_TreeshakeEnv*env,struct Cyc_List_List*ds){
return({struct Cyc_List_List*(*_tmp30B)(int(*f)(struct Cyc_Tc_TreeshakeEnv*,struct Cyc_Absyn_Decl*),struct Cyc_Tc_TreeshakeEnv*env,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp191)(int(*f)(struct Cyc_Tc_TreeshakeEnv*,struct Cyc_Absyn_Decl*),struct Cyc_Tc_TreeshakeEnv*env,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(int(*f)(struct Cyc_Tc_TreeshakeEnv*,struct Cyc_Absyn_Decl*),struct Cyc_Tc_TreeshakeEnv*env,struct Cyc_List_List*x))Cyc_List_filter_c;_tmp191;});struct Cyc_Tc_TreeshakeEnv*_tmp30A=env;_tmp30B(Cyc_Tc_vardecl_needed,_tmp30A,ds);});}
# 1031
static int Cyc_Tc_is_extern(struct Cyc_Absyn_Vardecl*vd){
# 1036
if((int)vd->sc == (int)Cyc_Absyn_Extern ||(int)vd->sc == (int)Cyc_Absyn_ExternC)
return 1;{
# 1036
void*_stmttmp19=
# 1038
Cyc_Tcutil_compress(vd->type);
# 1036
void*_tmp192=_stmttmp19;if(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp192)->tag == 5U){_LL1: _LL2:
# 1039
 return 1;}else{_LL3: _LL4:
 return 0;}_LL0:;}}
# 1031
static int Cyc_Tc_vardecl_needed(struct Cyc_Tc_TreeshakeEnv*env,struct Cyc_Absyn_Decl*d){
# 1045
void*_stmttmp1A=d->r;void*_tmp193=_stmttmp1A;struct Cyc_List_List**_tmp194;struct Cyc_List_List**_tmp195;struct Cyc_List_List**_tmp196;struct Cyc_List_List**_tmp197;struct Cyc_Absyn_Vardecl*_tmp198;switch(*((int*)_tmp193)){case 0U: _LL1: _tmp198=((struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct*)_tmp193)->f1;_LL2: {struct Cyc_Absyn_Vardecl*vd=_tmp198;
# 1049
if((env->in_cinclude || !Cyc_Tc_is_extern(vd))|| !({
struct _tuple0*_tmp30C=vd->name;Cyc_Absyn_qvar_cmp(_tmp30C,Cyc_Absyn_uniquergn_qvar());}))
# 1052
return 1;
# 1049
return(*({
# 1054
struct _tuple14*(*_tmp30E)(struct Cyc_Dict_Dict d,struct _tuple0*k)=({struct _tuple14*(*_tmp199)(struct Cyc_Dict_Dict d,struct _tuple0*k)=(struct _tuple14*(*)(struct Cyc_Dict_Dict d,struct _tuple0*k))Cyc_Dict_lookup;_tmp199;});struct Cyc_Dict_Dict _tmp30D=env->ordinaries;_tmp30E(_tmp30D,vd->name);})).f2;}case 11U: _LL3: _tmp197=(struct Cyc_List_List**)&((struct Cyc_Absyn_ExternC_d_Absyn_Raw_decl_struct*)_tmp193)->f1;_LL4: {struct Cyc_List_List**ds2p=_tmp197;
_tmp196=ds2p;goto _LL6;}case 10U: _LL5: _tmp196=(struct Cyc_List_List**)&((struct Cyc_Absyn_Using_d_Absyn_Raw_decl_struct*)_tmp193)->f2;_LL6: {struct Cyc_List_List**ds2p=_tmp196;
_tmp195=ds2p;goto _LL8;}case 9U: _LL7: _tmp195=(struct Cyc_List_List**)&((struct Cyc_Absyn_Namespace_d_Absyn_Raw_decl_struct*)_tmp193)->f2;_LL8: {struct Cyc_List_List**ds2p=_tmp195;
({struct Cyc_List_List*_tmp30F=Cyc_Tc_treeshake_f(env,*ds2p);*ds2p=_tmp30F;});return 1;}case 12U: _LL9: _tmp194=(struct Cyc_List_List**)&((struct Cyc_Absyn_ExternCinclude_d_Absyn_Raw_decl_struct*)_tmp193)->f1;_LLA: {struct Cyc_List_List**ds2p=_tmp194;
# 1059
int in_cinclude=env->in_cinclude;
env->in_cinclude=1;
({struct Cyc_List_List*_tmp310=Cyc_Tc_treeshake_f(env,*ds2p);*ds2p=_tmp310;});
env->in_cinclude=in_cinclude;
return 1;}default: _LLB: _LLC:
 return 1;}_LL0:;}
# 1031
struct Cyc_List_List*Cyc_Tc_treeshake(struct Cyc_Tcenv_Tenv*te,struct Cyc_List_List*ds){
# 1069
struct Cyc_Tc_TreeshakeEnv env=({struct Cyc_Tc_TreeshakeEnv _tmp1D7;_tmp1D7.in_cinclude=0,_tmp1D7.ordinaries=(te->ae)->ordinaries;_tmp1D7;});
return Cyc_Tc_treeshake_f(& env,ds);}
