#include <setjmp.h>
/* This is a C header file to be used by the output of the Cyclone to
   C translator.  The corresponding definitions are in file lib/runtime_*.c */
#ifndef _CYC_INCLUDE_H_
#define _CYC_INCLUDE_H_

/* Need one of these per thread (see runtime_stack.c). The runtime maintains 
   a stack that contains either _handler_cons structs or _RegionHandle structs.
   The tag is 0 for a handler_cons and 1 for a region handle.  */
struct _RuntimeStack {
  int tag; 
  struct _RuntimeStack *next;
  void (*cleanup)(struct _RuntimeStack *frame);
};

#ifndef offsetof
/* should be size_t, but int is fine. */
#define offsetof(t,n) ((int)(&(((t *)0)->n)))
#endif

/* Fat pointers */
struct _fat_ptr {
  unsigned char *curr; 
  unsigned char *base; 
  unsigned char *last_plus_one; 
};  

/* Discriminated Unions */
struct _xtunion_struct { char *tag; };

/* Regions */
struct _RegionPage
#ifdef CYC_REGION_PROFILE
{ unsigned total_bytes;
  unsigned free_bytes;
  /* MWH: wish we didn't have to include the stuff below ... */
  struct _RegionPage *next;
  char data[1];
}
#endif
; // abstract -- defined in runtime_memory.c
struct _RegionHandle {
  struct _RuntimeStack s;
  struct _RegionPage *curr;
  char               *offset;
  char               *last_plus_one;
  struct _DynRegionHandle *sub_regions;
#ifdef CYC_REGION_PROFILE
  const char         *name;
#else
  unsigned used_bytes;
  unsigned wasted_bytes;
#endif
};
struct _DynRegionFrame {
  struct _RuntimeStack s;
  struct _DynRegionHandle *x;
};
// A dynamic region is just a region handle.  The wrapper struct is for type
// abstraction.
struct Cyc_Core_DynamicRegion {
  struct _RegionHandle h;
};

struct _RegionHandle _new_region(const char*);
void* _region_malloc(struct _RegionHandle*, unsigned);
void* _region_calloc(struct _RegionHandle*, unsigned t, unsigned n);
void   _free_region(struct _RegionHandle*);
struct _RegionHandle*_open_dynregion(struct _DynRegionFrame*,struct _DynRegionHandle*);
void   _pop_dynregion();

/* Exceptions */
struct _handler_cons {
  struct _RuntimeStack s;
  jmp_buf handler;
};
void _push_handler(struct _handler_cons *);
void _push_region(struct _RegionHandle *);
void _npop_handler(int);
void _pop_handler();
void _pop_region();

#ifndef _throw
void* _throw_null_fn(const char*,unsigned);
void* _throw_arraybounds_fn(const char*,unsigned);
void* _throw_badalloc_fn(const char*,unsigned);
void* _throw_match_fn(const char*,unsigned);
void* _throw_fn(void*,const char*,unsigned);
void* _rethrow(void*);
#define _throw_null() (_throw_null_fn(__FILE__,__LINE__))
#define _throw_arraybounds() (_throw_arraybounds_fn(__FILE__,__LINE__))
#define _throw_badalloc() (_throw_badalloc_fn(__FILE__,__LINE__))
#define _throw_match() (_throw_match_fn(__FILE__,__LINE__))
#define _throw(e) (_throw_fn((e),__FILE__,__LINE__))
#endif

struct _xtunion_struct* Cyc_Core_get_exn_thrown();
/* Built-in Exceptions */
struct Cyc_Null_Exception_exn_struct { char *tag; };
struct Cyc_Array_bounds_exn_struct { char *tag; };
struct Cyc_Match_Exception_exn_struct { char *tag; };
struct Cyc_Bad_alloc_exn_struct { char *tag; };
extern char Cyc_Null_Exception[];
extern char Cyc_Array_bounds[];
extern char Cyc_Match_Exception[];
extern char Cyc_Bad_alloc[];

/* Built-in Run-time Checks and company */
#ifdef CYC_ANSI_OUTPUT
#define _INLINE  
#else
#define _INLINE inline
#endif

#ifdef NO_CYC_NULL_CHECKS
#define _check_null(ptr) (ptr)
#else
#define _check_null(ptr) \
  ({ void*_cks_null = (void*)(ptr); \
     if (!_cks_null) _throw_null(); \
     _cks_null; })
#endif

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index)\
   (((char*)ptr) + (elt_sz)*(index))
#ifdef NO_CYC_NULL_CHECKS
#define _check_known_subscript_null _check_known_subscript_notnull
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr);\
  int _index = (index);\
  if (!_cks_ptr) _throw_null(); \
  _cks_ptr + (elt_sz)*_index; })
#endif
#define _zero_arr_plus_fn(orig_x,orig_sz,orig_i,f,l) ((orig_x)+(orig_i))
#define _zero_arr_plus_char_fn _zero_arr_plus_fn
#define _zero_arr_plus_short_fn _zero_arr_plus_fn
#define _zero_arr_plus_int_fn _zero_arr_plus_fn
#define _zero_arr_plus_float_fn _zero_arr_plus_fn
#define _zero_arr_plus_double_fn _zero_arr_plus_fn
#define _zero_arr_plus_longdouble_fn _zero_arr_plus_fn
#define _zero_arr_plus_voidstar_fn _zero_arr_plus_fn
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (!_cks_ptr) _throw_null(); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })

/* _zero_arr_plus_*_fn(x,sz,i,filename,lineno) adds i to zero-terminated ptr
   x that has at least sz elements */
char* _zero_arr_plus_char_fn(char*,unsigned,int,const char*,unsigned);
short* _zero_arr_plus_short_fn(short*,unsigned,int,const char*,unsigned);
int* _zero_arr_plus_int_fn(int*,unsigned,int,const char*,unsigned);
float* _zero_arr_plus_float_fn(float*,unsigned,int,const char*,unsigned);
double* _zero_arr_plus_double_fn(double*,unsigned,int,const char*,unsigned);
long double* _zero_arr_plus_longdouble_fn(long double*,unsigned,int,const char*, unsigned);
void** _zero_arr_plus_voidstar_fn(void**,unsigned,int,const char*,unsigned);
#endif

/* _get_zero_arr_size_*(x,sz) returns the number of elements in a
   zero-terminated array that is NULL or has at least sz elements */
int _get_zero_arr_size_char(const char*,unsigned);
int _get_zero_arr_size_short(const short*,unsigned);
int _get_zero_arr_size_int(const int*,unsigned);
int _get_zero_arr_size_float(const float*,unsigned);
int _get_zero_arr_size_double(const double*,unsigned);
int _get_zero_arr_size_longdouble(const long double*,unsigned);
int _get_zero_arr_size_voidstar(const void**,unsigned);

/* _zero_arr_inplace_plus_*_fn(x,i,filename,lineno) sets
   zero-terminated pointer *x to *x + i */
char* _zero_arr_inplace_plus_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_short_fn(short**,int,const char*,unsigned);
int* _zero_arr_inplace_plus_int(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_longdouble_fn(long double**,int,const char*,unsigned);
void** _zero_arr_inplace_plus_voidstar_fn(void***,int,const char*,unsigned);
/* like the previous functions, but does post-addition (as in e++) */
char* _zero_arr_inplace_plus_post_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_post_short_fn(short**x,int,const char*,unsigned);
int* _zero_arr_inplace_plus_post_int_fn(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_post_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_post_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_post_longdouble_fn(long double**,int,const char *,unsigned);
void** _zero_arr_inplace_plus_post_voidstar_fn(void***,int,const char*,unsigned);
#define _zero_arr_plus_char(x,s,i) \
  (_zero_arr_plus_char_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_short(x,s,i) \
  (_zero_arr_plus_short_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_int(x,s,i) \
  (_zero_arr_plus_int_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_float(x,s,i) \
  (_zero_arr_plus_float_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_double(x,s,i) \
  (_zero_arr_plus_double_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_longdouble(x,s,i) \
  (_zero_arr_plus_longdouble_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_voidstar(x,s,i) \
  (_zero_arr_plus_voidstar_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_inplace_plus_char(x,i) \
  _zero_arr_inplace_plus_char_fn((char **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_short(x,i) \
  _zero_arr_inplace_plus_short_fn((short **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_int(x,i) \
  _zero_arr_inplace_plus_int_fn((int **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_float(x,i) \
  _zero_arr_inplace_plus_float_fn((float **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_double(x,i) \
  _zero_arr_inplace_plus_double_fn((double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_longdouble(x,i) \
  _zero_arr_inplace_plus_longdouble_fn((long double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_voidstar(x,i) \
  _zero_arr_inplace_plus_voidstar_fn((void ***)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_char(x,i) \
  _zero_arr_inplace_plus_post_char_fn((char **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_short(x,i) \
  _zero_arr_inplace_plus_post_short_fn((short **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_int(x,i) \
  _zero_arr_inplace_plus_post_int_fn((int **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_float(x,i) \
  _zero_arr_inplace_plus_post_float_fn((float **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_double(x,i) \
  _zero_arr_inplace_plus_post_double_fn((double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_longdouble(x,i) \
  _zero_arr_inplace_plus_post_longdouble_fn((long double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_voidstar(x,i) \
  _zero_arr_inplace_plus_post_voidstar_fn((void***)(x),(i),__FILE__,__LINE__)

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_fat_subscript(arr,elt_sz,index) ((arr).curr + (elt_sz) * (index))
#define _untag_fat_ptr(arr,elt_sz,num_elts) ((arr).curr)
#else
#define _check_fat_subscript(arr,elt_sz,index) ({ \
  struct _fat_ptr _cus_arr = (arr); \
  unsigned char *_cus_ans = _cus_arr.curr + (elt_sz) * (index); \
  /* JGM: not needed! if (!_cus_arr.base) _throw_null();*/ \
  if (_cus_ans < _cus_arr.base || _cus_ans >= _cus_arr.last_plus_one) \
    _throw_arraybounds(); \
  _cus_ans; })
#define _untag_fat_ptr(arr,elt_sz,num_elts) ({ \
  struct _fat_ptr _arr = (arr); \
  unsigned char *_curr = _arr.curr; \
  if ((_curr < _arr.base || _curr + (elt_sz) * (num_elts) > _arr.last_plus_one) &&\
      _curr != (unsigned char *)0) \
    _throw_arraybounds(); \
  _curr; })
#endif

#define _tag_fat(tcurr,elt_sz,num_elts) ({ \
  struct _fat_ptr _ans; \
  unsigned _num_elts = (num_elts);\
  _ans.base = _ans.curr = (void*)(tcurr); \
  /* JGM: if we're tagging NULL, ignore num_elts */ \
  _ans.last_plus_one = _ans.base ? (_ans.base + (elt_sz) * _num_elts) : 0; \
  _ans; })

#define _get_fat_size(arr,elt_sz) \
  ({struct _fat_ptr _arr = (arr); \
    unsigned char *_arr_curr=_arr.curr; \
    unsigned char *_arr_last=_arr.last_plus_one; \
    (_arr_curr < _arr.base || _arr_curr >= _arr_last) ? 0 : \
    ((_arr_last - _arr_curr) / (elt_sz));})

#define _fat_ptr_plus(arr,elt_sz,change) ({ \
  struct _fat_ptr _ans = (arr); \
  int _change = (change);\
  _ans.curr += (elt_sz) * _change;\
  _ans; })
#define _fat_ptr_inplace_plus(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  _arr_ptr->curr += (elt_sz) * (change);\
  *_arr_ptr; })
#define _fat_ptr_inplace_plus_post(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  struct _fat_ptr _ans = *_arr_ptr; \
  _arr_ptr->curr += (elt_sz) * (change);\
  _ans; })

//Not a macro since initialization order matters. Defined in runtime_zeroterm.c.
struct _fat_ptr _fat_ptr_decrease_size(struct _fat_ptr,unsigned sz,unsigned numelts);

/* Allocation */
void* GC_malloc(int);
void* GC_malloc_atomic(int);
void* GC_calloc(unsigned,unsigned);
void* GC_calloc_atomic(unsigned,unsigned);
// bound the allocation size to be < MAX_ALLOC_SIZE. See macros below for usage.
#define MAX_MALLOC_SIZE (1 << 28)
void* _bounded_GC_malloc(int,const char*,int);
void* _bounded_GC_malloc_atomic(int,const char*,int);
void* _bounded_GC_calloc(unsigned,unsigned,const char*,int);
void* _bounded_GC_calloc_atomic(unsigned,unsigned,const char*,int);
/* these macros are overridden below ifdef CYC_REGION_PROFILE */
#ifndef CYC_REGION_PROFILE
#define _cycalloc(n) _bounded_GC_malloc(n,__FILE__,__LINE__)
#define _cycalloc_atomic(n) _bounded_GC_malloc_atomic(n,__FILE__,__LINE__)
#define _cyccalloc(n,s) _bounded_GC_calloc(n,s,__FILE__,__LINE__)
#define _cyccalloc_atomic(n,s) _bounded_GC_calloc_atomic(n,s,__FILE__,__LINE__)
#endif

static _INLINE unsigned int _check_times(unsigned x, unsigned y) {
  unsigned long long whole_ans = 
    ((unsigned long long) x)*((unsigned long long)y);
  unsigned word_ans = (unsigned)whole_ans;
  if(word_ans < whole_ans || word_ans > MAX_MALLOC_SIZE)
    _throw_badalloc();
  return word_ans;
}

#define _CYC_MAX_REGION_CONST 2
#define _CYC_MIN_ALIGNMENT (sizeof(double))

#ifdef CYC_REGION_PROFILE
extern int rgn_total_bytes;
#endif

static _INLINE void *_fast_region_malloc(struct _RegionHandle *r, unsigned orig_s) {  
  if (r > (struct _RegionHandle *)_CYC_MAX_REGION_CONST && r->curr != 0) { 
#ifdef CYC_NOALIGN
    unsigned s =  orig_s;
#else
    unsigned s =  (orig_s + _CYC_MIN_ALIGNMENT - 1) & (~(_CYC_MIN_ALIGNMENT -1)); 
#endif
    char *result; 
    result = r->offset; 
    if (s <= (r->last_plus_one - result)) {
      r->offset = result + s; 
#ifdef CYC_REGION_PROFILE
    r->curr->free_bytes = r->curr->free_bytes - s;
    rgn_total_bytes += s;
#endif
      return result;
    }
  } 
  return _region_malloc(r,orig_s); 
}

#ifdef CYC_REGION_PROFILE
/* see macros below for usage. defined in runtime_memory.c */
void* _profile_GC_malloc(int,const char*,const char*,int);
void* _profile_GC_malloc_atomic(int,const char*,const char*,int);
void* _profile_GC_calloc(unsigned,unsigned,const char*,const char*,int);
void* _profile_GC_calloc_atomic(unsigned,unsigned,const char*,const char*,int);
void* _profile_region_malloc(struct _RegionHandle*,unsigned,const char*,const char*,int);
void* _profile_region_calloc(struct _RegionHandle*,unsigned,unsigned,const char *,const char*,int);
struct _RegionHandle _profile_new_region(const char*,const char*,const char*,int);
void _profile_free_region(struct _RegionHandle*,const char*,const char*,int);
#ifndef RUNTIME_CYC
#define _new_region(n) _profile_new_region(n,__FILE__,__FUNCTION__,__LINE__)
#define _free_region(r) _profile_free_region(r,__FILE__,__FUNCTION__,__LINE__)
#define _region_malloc(rh,n) _profile_region_malloc(rh,n,__FILE__,__FUNCTION__,__LINE__)
#define _region_calloc(rh,n,t) _profile_region_calloc(rh,n,t,__FILE__,__FUNCTION__,__LINE__)
#  endif
#define _cycalloc(n) _profile_GC_malloc(n,__FILE__,__FUNCTION__,__LINE__)
#define _cycalloc_atomic(n) _profile_GC_malloc_atomic(n,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc(n,s) _profile_GC_calloc(n,s,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc_atomic(n,s) _profile_GC_calloc_atomic(n,s,__FILE__,__FUNCTION__,__LINE__)
#endif
#endif
 struct Cyc_Core_Opt{void*v;};struct _tuple0{void*f1;void*f2;};
# 111 "core.h"
void*Cyc_Core_snd(struct _tuple0*);extern char Cyc_Core_Invalid_argument[17U];struct Cyc_Core_Invalid_argument_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Failure[8U];struct Cyc_Core_Failure_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Impossible[11U];struct Cyc_Core_Impossible_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Not_found[10U];struct Cyc_Core_Not_found_exn_struct{char*tag;};extern char Cyc_Core_Unreachable[12U];struct Cyc_Core_Unreachable_exn_struct{char*tag;struct _fat_ptr f1;};
# 165
extern struct _RegionHandle*Cyc_Core_heap_region;
# 168
extern struct _RegionHandle*Cyc_Core_unique_region;struct Cyc_Core_DynamicRegion;struct Cyc_Core_NewDynamicRegion{struct Cyc_Core_DynamicRegion*key;};struct Cyc_Core_ThinRes{void*arr;unsigned nelts;};struct Cyc_List_List{void*hd;struct Cyc_List_List*tl;};
# 61 "list.h"
int Cyc_List_length(struct Cyc_List_List*x);
# 76
struct Cyc_List_List*Cyc_List_map(void*(*f)(void*),struct Cyc_List_List*x);
# 83
struct Cyc_List_List*Cyc_List_map_c(void*(*f)(void*,void*),void*env,struct Cyc_List_List*x);extern char Cyc_List_List_mismatch[14U];struct Cyc_List_List_mismatch_exn_struct{char*tag;};
# 178
struct Cyc_List_List*Cyc_List_imp_rev(struct Cyc_List_List*x);
# 184
struct Cyc_List_List*Cyc_List_append(struct Cyc_List_List*x,struct Cyc_List_List*y);extern char Cyc_List_Nth[4U];struct Cyc_List_Nth_exn_struct{char*tag;};
# 242
void*Cyc_List_nth(struct Cyc_List_List*x,int n);
# 246
struct Cyc_List_List*Cyc_List_nth_tail(struct Cyc_List_List*x,int i);
# 265
void*Cyc_List_find_c(void*(*pred)(void*,void*),void*env,struct Cyc_List_List*x);
# 270
struct Cyc_List_List*Cyc_List_zip(struct Cyc_List_List*x,struct Cyc_List_List*y);struct Cyc___cycFILE;struct Cyc_String_pa_PrintArg_struct{int tag;struct _fat_ptr f1;};struct Cyc_Int_pa_PrintArg_struct{int tag;unsigned long f1;};struct Cyc_Double_pa_PrintArg_struct{int tag;double f1;};struct Cyc_LongDouble_pa_PrintArg_struct{int tag;long double f1;};struct Cyc_ShortPtr_pa_PrintArg_struct{int tag;short*f1;};struct Cyc_IntPtr_pa_PrintArg_struct{int tag;unsigned long*f1;};
# 73 "cycboot.h"
struct _fat_ptr Cyc_aprintf(struct _fat_ptr,struct _fat_ptr);struct Cyc_ShortPtr_sa_ScanfArg_struct{int tag;short*f1;};struct Cyc_UShortPtr_sa_ScanfArg_struct{int tag;unsigned short*f1;};struct Cyc_IntPtr_sa_ScanfArg_struct{int tag;int*f1;};struct Cyc_UIntPtr_sa_ScanfArg_struct{int tag;unsigned*f1;};struct Cyc_StringPtr_sa_ScanfArg_struct{int tag;struct _fat_ptr f1;};struct Cyc_DoublePtr_sa_ScanfArg_struct{int tag;double*f1;};struct Cyc_FloatPtr_sa_ScanfArg_struct{int tag;float*f1;};struct Cyc_CharPtr_sa_ScanfArg_struct{int tag;struct _fat_ptr f1;};
# 232 "cycboot.h"
struct _fat_ptr Cyc_vrprintf(struct _RegionHandle*,struct _fat_ptr,struct _fat_ptr);extern char Cyc_FileCloseError[15U];struct Cyc_FileCloseError_exn_struct{char*tag;};extern char Cyc_FileOpenError[14U];struct Cyc_FileOpenError_exn_struct{char*tag;struct _fat_ptr f1;};
# 38 "string.h"
unsigned long Cyc_strlen(struct _fat_ptr s);
# 49 "string.h"
int Cyc_strcmp(struct _fat_ptr s1,struct _fat_ptr s2);struct Cyc_PP_Ppstate;struct Cyc_PP_Out;struct Cyc_PP_Doc;struct Cyc_Position_Error;struct Cyc_Relations_Reln;struct _union_Nmspace_Rel_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_Abs_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_C_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_Loc_n{int tag;int val;};union Cyc_Absyn_Nmspace{struct _union_Nmspace_Rel_n Rel_n;struct _union_Nmspace_Abs_n Abs_n;struct _union_Nmspace_C_n C_n;struct _union_Nmspace_Loc_n Loc_n;};struct _tuple1{union Cyc_Absyn_Nmspace f1;struct _fat_ptr*f2;};
# 161 "absyn.h"
enum Cyc_Absyn_Scope{Cyc_Absyn_Static =0U,Cyc_Absyn_Abstract =1U,Cyc_Absyn_Public =2U,Cyc_Absyn_Extern =3U,Cyc_Absyn_ExternC =4U,Cyc_Absyn_Register =5U};struct Cyc_Absyn_Tqual{int print_const: 1;int q_volatile: 1;int q_restrict: 1;int real_const: 1;unsigned loc;};
# 182
enum Cyc_Absyn_Size_of{Cyc_Absyn_Char_sz =0U,Cyc_Absyn_Short_sz =1U,Cyc_Absyn_Int_sz =2U,Cyc_Absyn_Long_sz =3U,Cyc_Absyn_LongLong_sz =4U};
enum Cyc_Absyn_Sign{Cyc_Absyn_Signed =0U,Cyc_Absyn_Unsigned =1U,Cyc_Absyn_None =2U};
enum Cyc_Absyn_AggrKind{Cyc_Absyn_StructA =0U,Cyc_Absyn_UnionA =1U};
# 187
enum Cyc_Absyn_AliasQual{Cyc_Absyn_Aliasable =0U,Cyc_Absyn_Unique =1U,Cyc_Absyn_Top =2U};
# 192
enum Cyc_Absyn_KindQual{Cyc_Absyn_AnyKind =0U,Cyc_Absyn_MemKind =1U,Cyc_Absyn_BoxKind =2U,Cyc_Absyn_RgnKind =3U,Cyc_Absyn_EffKind =4U,Cyc_Absyn_IntKind =5U,Cyc_Absyn_BoolKind =6U,Cyc_Absyn_PtrBndKind =7U};struct Cyc_Absyn_Kind{enum Cyc_Absyn_KindQual kind;enum Cyc_Absyn_AliasQual aliasqual;};struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct{int tag;struct Cyc_Absyn_Kind*f1;};struct Cyc_Absyn_Unknown_kb_Absyn_KindBound_struct{int tag;struct Cyc_Core_Opt*f1;};struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct{int tag;struct Cyc_Core_Opt*f1;struct Cyc_Absyn_Kind*f2;};struct Cyc_Absyn_Tvar{struct _fat_ptr*name;int identity;void*kind;};struct Cyc_Absyn_PtrLoc{unsigned ptr_loc;unsigned rgn_loc;unsigned zt_loc;};struct Cyc_Absyn_PtrAtts{void*rgn;void*nullable;void*bounds;void*zero_term;struct Cyc_Absyn_PtrLoc*ptrloc;};struct Cyc_Absyn_PtrInfo{void*elt_type;struct Cyc_Absyn_Tqual elt_tq;struct Cyc_Absyn_PtrAtts ptr_atts;};struct Cyc_Absyn_VarargInfo{struct _fat_ptr*name;struct Cyc_Absyn_Tqual tq;void*type;int inject;};struct Cyc_Absyn_Star_Absyn_Cap_struct{int tag;};struct Cyc_Absyn_Bot_Absyn_Cap_struct{int tag;};struct Cyc_Absyn_Nat_Absyn_Cap_struct{int tag;struct Cyc_Absyn_Exp*f1;int f2;};struct Cyc_Absyn_RgnEffect{void*name;void*rgn;void*lock;void*unadopt;void*parent;};struct Cyc_Absyn_FnInfo{struct Cyc_List_List*tvars;void*effect;struct Cyc_Absyn_Tqual ret_tqual;void*ret_type;struct Cyc_List_List*args;int c_varargs;struct Cyc_Absyn_VarargInfo*cyc_varargs;struct Cyc_List_List*rgn_po;struct Cyc_List_List*attributes;struct Cyc_Absyn_Exp*requires_clause;struct Cyc_List_List*requires_relns;struct Cyc_Absyn_Exp*ensures_clause;struct Cyc_List_List*ensures_relns;struct Cyc_List_List*ieffect;struct Cyc_List_List*oeffect;};struct Cyc_Absyn_UnknownDatatypeInfo{struct _tuple1*name;int is_extensible;};struct _union_DatatypeInfo_UnknownDatatype{int tag;struct Cyc_Absyn_UnknownDatatypeInfo val;};struct _union_DatatypeInfo_KnownDatatype{int tag;struct Cyc_Absyn_Datatypedecl**val;};union Cyc_Absyn_DatatypeInfo{struct _union_DatatypeInfo_UnknownDatatype UnknownDatatype;struct _union_DatatypeInfo_KnownDatatype KnownDatatype;};struct Cyc_Absyn_UnknownDatatypeFieldInfo{struct _tuple1*datatype_name;struct _tuple1*field_name;int is_extensible;};struct _union_DatatypeFieldInfo_UnknownDatatypefield{int tag;struct Cyc_Absyn_UnknownDatatypeFieldInfo val;};struct _tuple2{struct Cyc_Absyn_Datatypedecl*f1;struct Cyc_Absyn_Datatypefield*f2;};struct _union_DatatypeFieldInfo_KnownDatatypefield{int tag;struct _tuple2 val;};union Cyc_Absyn_DatatypeFieldInfo{struct _union_DatatypeFieldInfo_UnknownDatatypefield UnknownDatatypefield;struct _union_DatatypeFieldInfo_KnownDatatypefield KnownDatatypefield;};
# 325
union Cyc_Absyn_DatatypeFieldInfo Cyc_Absyn_KnownDatatypefield(struct Cyc_Absyn_Datatypedecl*,struct Cyc_Absyn_Datatypefield*);struct _tuple3{enum Cyc_Absyn_AggrKind f1;struct _tuple1*f2;struct Cyc_Core_Opt*f3;};struct _union_AggrInfo_UnknownAggr{int tag;struct _tuple3 val;};struct _union_AggrInfo_KnownAggr{int tag;struct Cyc_Absyn_Aggrdecl**val;};union Cyc_Absyn_AggrInfo{struct _union_AggrInfo_UnknownAggr UnknownAggr;struct _union_AggrInfo_KnownAggr KnownAggr;};
# 332
union Cyc_Absyn_AggrInfo Cyc_Absyn_KnownAggr(struct Cyc_Absyn_Aggrdecl**);struct Cyc_Absyn_ArrayInfo{void*elt_type;struct Cyc_Absyn_Tqual tq;struct Cyc_Absyn_Exp*num_elts;void*zero_term;unsigned zt_loc;};struct Cyc_Absyn_Aggr_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Aggrdecl*f1;};struct Cyc_Absyn_Enum_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;};struct Cyc_Absyn_Datatype_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;};struct Cyc_Absyn_TypeDecl{void*r;unsigned loc;};struct Cyc_Absyn_VoidCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_IntCon_Absyn_TyCon_struct{int tag;enum Cyc_Absyn_Sign f1;enum Cyc_Absyn_Size_of f2;};struct Cyc_Absyn_FloatCon_Absyn_TyCon_struct{int tag;int f1;};struct Cyc_Absyn_RgnHandleCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_TagCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_HeapCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_UniqueCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_RefCntCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_AccessCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_JoinCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_RgnsCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_TrueCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_FalseCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_ThinCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_FatCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_EnumCon_Absyn_TyCon_struct{int tag;struct _tuple1*f1;struct Cyc_Absyn_Enumdecl*f2;};struct Cyc_Absyn_AnonEnumCon_Absyn_TyCon_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_BuiltinCon_Absyn_TyCon_struct{int tag;struct _fat_ptr f1;struct Cyc_Absyn_Kind*f2;};struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_DatatypeInfo f1;};struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_DatatypeFieldInfo f1;};struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_AggrInfo f1;};struct Cyc_Absyn_AppType_Absyn_Type_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Evar_Absyn_Type_struct{int tag;struct Cyc_Core_Opt*f1;void*f2;int f3;struct Cyc_Core_Opt*f4;};struct Cyc_Absyn_VarType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Tvar*f1;};struct Cyc_Absyn_PointerType_Absyn_Type_struct{int tag;struct Cyc_Absyn_PtrInfo f1;};struct Cyc_Absyn_ArrayType_Absyn_Type_struct{int tag;struct Cyc_Absyn_ArrayInfo f1;};struct Cyc_Absyn_FnType_Absyn_Type_struct{int tag;struct Cyc_Absyn_FnInfo f1;};struct Cyc_Absyn_TupleType_Absyn_Type_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct{int tag;enum Cyc_Absyn_AggrKind f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_TypedefType_Absyn_Type_struct{int tag;struct _tuple1*f1;struct Cyc_List_List*f2;struct Cyc_Absyn_Typedefdecl*f3;void*f4;};struct Cyc_Absyn_ValueofType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct{int tag;struct Cyc_Absyn_TypeDecl*f1;void**f2;};struct Cyc_Absyn_TypeofType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_NoTypes_Absyn_Funcparams_struct{int tag;struct Cyc_List_List*f1;unsigned f2;};struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct{int tag;struct Cyc_List_List*f1;int f2;struct Cyc_Absyn_VarargInfo*f3;void*f4;struct Cyc_List_List*f5;struct Cyc_Absyn_Exp*f6;struct Cyc_Absyn_Exp*f7;struct Cyc_List_List*f8;struct Cyc_List_List*f9;};
# 437 "absyn.h"
enum Cyc_Absyn_Format_Type{Cyc_Absyn_Printf_ft =0U,Cyc_Absyn_Scanf_ft =1U};struct Cyc_Absyn_Regparm_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Stdcall_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Cdecl_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Fastcall_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Noreturn_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Const_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Aligned_att_Absyn_Attribute_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Packed_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Section_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Nocommon_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Shared_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Unused_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Weak_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Dllimport_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Dllexport_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_instrument_function_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Constructor_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Destructor_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_check_memory_usage_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Format_att_Absyn_Attribute_struct{int tag;enum Cyc_Absyn_Format_Type f1;int f2;int f3;};struct Cyc_Absyn_Initializes_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Noliveunique_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Consume_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Pure_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Mode_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Alias_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Always_inline_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_throw_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Non_null_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Deprecated_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Carray_mod_Absyn_Type_modifier_struct{int tag;void*f1;unsigned f2;};struct Cyc_Absyn_ConstArray_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_Absyn_Exp*f1;void*f2;unsigned f3;};struct Cyc_Absyn_Pointer_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_Absyn_PtrAtts f1;struct Cyc_Absyn_Tqual f2;};struct Cyc_Absyn_Function_mod_Absyn_Type_modifier_struct{int tag;void*f1;};struct Cyc_Absyn_TypeParams_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_List_List*f1;unsigned f2;int f3;};struct Cyc_Absyn_Attributes_mod_Absyn_Type_modifier_struct{int tag;unsigned f1;struct Cyc_List_List*f2;};struct _union_Cnst_Null_c{int tag;int val;};struct _tuple4{enum Cyc_Absyn_Sign f1;char f2;};struct _union_Cnst_Char_c{int tag;struct _tuple4 val;};struct _union_Cnst_Wchar_c{int tag;struct _fat_ptr val;};struct _tuple5{enum Cyc_Absyn_Sign f1;short f2;};struct _union_Cnst_Short_c{int tag;struct _tuple5 val;};struct _tuple6{enum Cyc_Absyn_Sign f1;int f2;};struct _union_Cnst_Int_c{int tag;struct _tuple6 val;};struct _tuple7{enum Cyc_Absyn_Sign f1;long long f2;};struct _union_Cnst_LongLong_c{int tag;struct _tuple7 val;};struct _tuple8{struct _fat_ptr f1;int f2;};struct _union_Cnst_Float_c{int tag;struct _tuple8 val;};struct _union_Cnst_String_c{int tag;struct _fat_ptr val;};struct _union_Cnst_Wstring_c{int tag;struct _fat_ptr val;};union Cyc_Absyn_Cnst{struct _union_Cnst_Null_c Null_c;struct _union_Cnst_Char_c Char_c;struct _union_Cnst_Wchar_c Wchar_c;struct _union_Cnst_Short_c Short_c;struct _union_Cnst_Int_c Int_c;struct _union_Cnst_LongLong_c LongLong_c;struct _union_Cnst_Float_c Float_c;struct _union_Cnst_String_c String_c;struct _union_Cnst_Wstring_c Wstring_c;};
# 522
union Cyc_Absyn_Cnst Cyc_Absyn_Char_c(enum Cyc_Absyn_Sign,char);
# 524
union Cyc_Absyn_Cnst Cyc_Absyn_Short_c(enum Cyc_Absyn_Sign,short);
union Cyc_Absyn_Cnst Cyc_Absyn_Int_c(enum Cyc_Absyn_Sign,int);
# 532
enum Cyc_Absyn_Primop{Cyc_Absyn_Plus =0U,Cyc_Absyn_Times =1U,Cyc_Absyn_Minus =2U,Cyc_Absyn_Div =3U,Cyc_Absyn_Mod =4U,Cyc_Absyn_Eq =5U,Cyc_Absyn_Neq =6U,Cyc_Absyn_Gt =7U,Cyc_Absyn_Lt =8U,Cyc_Absyn_Gte =9U,Cyc_Absyn_Lte =10U,Cyc_Absyn_Not =11U,Cyc_Absyn_Bitnot =12U,Cyc_Absyn_Bitand =13U,Cyc_Absyn_Bitor =14U,Cyc_Absyn_Bitxor =15U,Cyc_Absyn_Bitlshift =16U,Cyc_Absyn_Bitlrshift =17U,Cyc_Absyn_Numelts =18U};
# 539
enum Cyc_Absyn_Incrementor{Cyc_Absyn_PreInc =0U,Cyc_Absyn_PostInc =1U,Cyc_Absyn_PreDec =2U,Cyc_Absyn_PostDec =3U};struct Cyc_Absyn_VarargCallInfo{int num_varargs;struct Cyc_List_List*injectors;struct Cyc_Absyn_VarargInfo*vai;};struct Cyc_Absyn_StructField_Absyn_OffsetofField_struct{int tag;struct _fat_ptr*f1;};struct Cyc_Absyn_TupleIndex_Absyn_OffsetofField_struct{int tag;unsigned f1;};
# 557
enum Cyc_Absyn_Coercion{Cyc_Absyn_Unknown_coercion =0U,Cyc_Absyn_No_coercion =1U,Cyc_Absyn_Null_to_NonNull =2U,Cyc_Absyn_Other_coercion =3U};struct Cyc_Absyn_MallocInfo{int is_calloc;struct Cyc_Absyn_Exp*rgn;void**elt_type;struct Cyc_Absyn_Exp*num_elts;int fat_result;int inline_call;};struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct{int tag;union Cyc_Absyn_Cnst f1;};struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Pragma_e_Absyn_Raw_exp_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct{int tag;enum Cyc_Absyn_Primop f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_AssignOp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Core_Opt*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_Increment_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;enum Cyc_Absyn_Incrementor f2;};struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;struct Cyc_Absyn_VarargCallInfo*f3;int f4;};struct Cyc_Absyn_Throw_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;int f2;};struct Cyc_Absyn_NoInstantiate_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Instantiate_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_Absyn_Exp*f2;int f3;enum Cyc_Absyn_Coercion f4;};struct Cyc_Absyn_Address_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_New_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Sizeoftype_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Sizeofexp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Offsetof_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Deref_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;int f3;int f4;};struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;int f3;int f4;};struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Tuple_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;};struct _tuple9{struct _fat_ptr*f1;struct Cyc_Absyn_Tqual f2;void*f3;};struct Cyc_Absyn_CompoundLit_e_Absyn_Raw_exp_struct{int tag;struct _tuple9*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Array_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Exp*f2;struct Cyc_Absyn_Exp*f3;int f4;};struct Cyc_Absyn_ComprehensionNoinit_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;void*f2;int f3;};struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct{int tag;struct _tuple1*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;struct Cyc_Absyn_Aggrdecl*f4;};struct Cyc_Absyn_AnonStruct_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Datatype_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;struct Cyc_Absyn_Datatypedecl*f2;struct Cyc_Absyn_Datatypefield*f3;};struct Cyc_Absyn_Enum_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_AnonEnum_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_Spawn_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_MallocInfo f1;};struct Cyc_Absyn_Swap_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_UnresolvedMem_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Core_Opt*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_StmtExp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Stmt*f1;};struct Cyc_Absyn_Tagcheck_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;};struct Cyc_Absyn_Valueof_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Asm_e_Absyn_Raw_exp_struct{int tag;int f1;struct _fat_ptr f2;struct Cyc_List_List*f3;struct Cyc_List_List*f4;struct Cyc_List_List*f5;};struct Cyc_Absyn_Extension_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Exp{void*topt;void*r;unsigned loc;void*annot;};struct Cyc_Absyn_Skip_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Exp_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Seq_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Return_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_IfThenElse_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Stmt*f2;struct Cyc_Absyn_Stmt*f3;};struct _tuple10{struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_While_s_Absyn_Raw_stmt_struct{int tag;struct _tuple10 f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Break_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Continue_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Goto_s_Absyn_Raw_stmt_struct{int tag;struct _fat_ptr*f1;};struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _tuple10 f2;struct _tuple10 f3;struct Cyc_Absyn_Stmt*f4;};struct Cyc_Absyn_Switch_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;void*f3;};struct Cyc_Absyn_Fallthru_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_List_List*f1;struct Cyc_Absyn_Switch_clause**f2;};struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Decl*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Label_s_Absyn_Raw_stmt_struct{int tag;struct _fat_ptr*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Do_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct _tuple10 f2;};struct Cyc_Absyn_TryCatch_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct Cyc_List_List*f2;void*f3;};struct Cyc_Absyn_Stmt{void*r;unsigned loc;void*annot;};struct Cyc_Absyn_Wild_p_Absyn_Raw_pat_struct{int tag;};struct Cyc_Absyn_Var_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Pat*f2;};struct Cyc_Absyn_AliasVar_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;};struct Cyc_Absyn_Reference_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Pat*f2;};struct Cyc_Absyn_TagInt_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;};struct Cyc_Absyn_Tuple_p_Absyn_Raw_pat_struct{int tag;struct Cyc_List_List*f1;int f2;};struct Cyc_Absyn_Pointer_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Pat*f1;};struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct{int tag;union Cyc_Absyn_AggrInfo*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;int f4;};struct Cyc_Absyn_Datatype_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;struct Cyc_Absyn_Datatypefield*f2;struct Cyc_List_List*f3;int f4;};struct Cyc_Absyn_Null_p_Absyn_Raw_pat_struct{int tag;};struct Cyc_Absyn_Int_p_Absyn_Raw_pat_struct{int tag;enum Cyc_Absyn_Sign f1;int f2;};struct Cyc_Absyn_Char_p_Absyn_Raw_pat_struct{int tag;char f1;};struct Cyc_Absyn_Float_p_Absyn_Raw_pat_struct{int tag;struct _fat_ptr f1;int f2;};struct Cyc_Absyn_Enum_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_AnonEnum_p_Absyn_Raw_pat_struct{int tag;void*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_UnknownId_p_Absyn_Raw_pat_struct{int tag;struct _tuple1*f1;};struct Cyc_Absyn_UnknownCall_p_Absyn_Raw_pat_struct{int tag;struct _tuple1*f1;struct Cyc_List_List*f2;int f3;};struct Cyc_Absyn_Exp_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Pat{void*r;void*topt;unsigned loc;};struct Cyc_Absyn_Switch_clause{struct Cyc_Absyn_Pat*pattern;struct Cyc_Core_Opt*pat_vars;struct Cyc_Absyn_Exp*where_clause;struct Cyc_Absyn_Stmt*body;unsigned loc;};struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct{int tag;struct _tuple1*f1;};struct Cyc_Absyn_Global_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Funname_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Fndecl*f1;};struct Cyc_Absyn_Param_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Local_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Pat_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Vardecl{enum Cyc_Absyn_Scope sc;struct _tuple1*name;unsigned varloc;struct Cyc_Absyn_Tqual tq;void*type;struct Cyc_Absyn_Exp*initializer;void*rgn;struct Cyc_List_List*attributes;int escapes;};struct Cyc_Absyn_Fndecl{enum Cyc_Absyn_Scope sc;int is_inline;struct _tuple1*name;struct Cyc_Absyn_Stmt*body;struct Cyc_Absyn_FnInfo i;void*cached_type;struct Cyc_Core_Opt*param_vardecls;struct Cyc_Absyn_Vardecl*fn_vardecl;};struct Cyc_Absyn_Aggrfield{struct _fat_ptr*name;struct Cyc_Absyn_Tqual tq;void*type;struct Cyc_Absyn_Exp*width;struct Cyc_List_List*attributes;struct Cyc_Absyn_Exp*requires_clause;};struct Cyc_Absyn_AggrdeclImpl{struct Cyc_List_List*exist_vars;struct Cyc_List_List*rgn_po;struct Cyc_List_List*fields;int tagged;};struct Cyc_Absyn_Aggrdecl{enum Cyc_Absyn_AggrKind kind;enum Cyc_Absyn_Scope sc;struct _tuple1*name;struct Cyc_List_List*tvs;struct Cyc_Absyn_AggrdeclImpl*impl;struct Cyc_List_List*attributes;int expected_mem_kind;};struct Cyc_Absyn_Datatypefield{struct _tuple1*name;struct Cyc_List_List*typs;unsigned loc;enum Cyc_Absyn_Scope sc;};struct Cyc_Absyn_Datatypedecl{enum Cyc_Absyn_Scope sc;struct _tuple1*name;struct Cyc_List_List*tvs;struct Cyc_Core_Opt*fields;int is_extensible;};struct Cyc_Absyn_Enumfield{struct _tuple1*name;struct Cyc_Absyn_Exp*tag;unsigned loc;};struct Cyc_Absyn_Enumdecl{enum Cyc_Absyn_Scope sc;struct _tuple1*name;struct Cyc_Core_Opt*fields;};struct Cyc_Absyn_Typedefdecl{struct _tuple1*name;struct Cyc_Absyn_Tqual tq;struct Cyc_List_List*tvs;struct Cyc_Core_Opt*kind;void*defn;struct Cyc_List_List*atts;int extern_c;};struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Fn_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Fndecl*f1;};struct Cyc_Absyn_Let_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Pat*f1;struct Cyc_Core_Opt*f2;struct Cyc_Absyn_Exp*f3;void*f4;};struct Cyc_Absyn_Letv_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_Region_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_Aggr_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Aggrdecl*f1;};struct Cyc_Absyn_Datatype_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;};struct Cyc_Absyn_Enum_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;};struct Cyc_Absyn_Typedef_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Typedefdecl*f1;};struct Cyc_Absyn_Namespace_d_Absyn_Raw_decl_struct{int tag;struct _fat_ptr*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Using_d_Absyn_Raw_decl_struct{int tag;struct _tuple1*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_ExternC_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;};struct _tuple11{unsigned f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_ExternCinclude_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;struct _tuple11*f4;};struct Cyc_Absyn_Porton_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Portoff_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Tempeston_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Tempestoff_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Decl{void*r;unsigned loc;};struct Cyc_Absyn_ArrayElement_Absyn_Designator_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_FieldName_Absyn_Designator_struct{int tag;struct _fat_ptr*f1;};extern char Cyc_Absyn_EmptyAnnot[11U];struct Cyc_Absyn_EmptyAnnot_Absyn_AbsynAnnot_struct{char*tag;};
# 916 "absyn.h"
struct Cyc_Absyn_Tqual Cyc_Absyn_const_tqual(unsigned);
struct Cyc_Absyn_Tqual Cyc_Absyn_empty_tqual(unsigned);
# 933
void*Cyc_Absyn_new_evar(struct Cyc_Core_Opt*k,struct Cyc_Core_Opt*tenv);
# 935
void*Cyc_Absyn_wildtyp(struct Cyc_Core_Opt*);
# 938
extern void*Cyc_Absyn_char_type;extern void*Cyc_Absyn_uchar_type;extern void*Cyc_Absyn_ushort_type;extern void*Cyc_Absyn_uint_type;extern void*Cyc_Absyn_ulonglong_type;
# 940
extern void*Cyc_Absyn_schar_type;extern void*Cyc_Absyn_sshort_type;extern void*Cyc_Absyn_sint_type;extern void*Cyc_Absyn_slonglong_type;
# 942
extern void*Cyc_Absyn_double_type;void*Cyc_Absyn_wchar_type();
void*Cyc_Absyn_gen_float_type(unsigned i);
# 945
extern void*Cyc_Absyn_heap_rgn_type;extern void*Cyc_Absyn_unique_rgn_type;
# 949
extern void*Cyc_Absyn_true_type;extern void*Cyc_Absyn_false_type;
# 951
extern void*Cyc_Absyn_void_type;void*Cyc_Absyn_tag_type(void*);void*Cyc_Absyn_rgn_handle_type(void*);void*Cyc_Absyn_enum_type(struct _tuple1*n,struct Cyc_Absyn_Enumdecl*d);
# 968
void*Cyc_Absyn_exn_type();
# 976
extern void*Cyc_Absyn_fat_bound_type;
# 978
void*Cyc_Absyn_thin_bounds_exp(struct Cyc_Absyn_Exp*);
# 980
void*Cyc_Absyn_bounds_one();
# 987
void*Cyc_Absyn_atb_type(void*t,void*rgn,struct Cyc_Absyn_Tqual tq,void*b,void*zero_term);
# 990
void*Cyc_Absyn_star_type(void*t,void*rgn,struct Cyc_Absyn_Tqual,void*zero_term);
# 992
void*Cyc_Absyn_at_type(void*t,void*rgn,struct Cyc_Absyn_Tqual,void*zero_term);
# 996
void*Cyc_Absyn_fatptr_type(void*t,void*rgn,struct Cyc_Absyn_Tqual,void*zeroterm);
# 1004
void*Cyc_Absyn_array_type(void*elt_type,struct Cyc_Absyn_Tqual tq,struct Cyc_Absyn_Exp*num_elts,void*zero_term,unsigned ztloc);
# 1008
void*Cyc_Absyn_datatype_field_type(union Cyc_Absyn_DatatypeFieldInfo,struct Cyc_List_List*args);
void*Cyc_Absyn_aggr_type(union Cyc_Absyn_AggrInfo,struct Cyc_List_List*args);
# 1012
struct Cyc_Absyn_Exp*Cyc_Absyn_new_exp(void*,unsigned);
# 1014
struct Cyc_Absyn_Exp*Cyc_Absyn_copy_exp(struct Cyc_Absyn_Exp*);
struct Cyc_Absyn_Exp*Cyc_Absyn_const_exp(union Cyc_Absyn_Cnst,unsigned);
# 1022
struct Cyc_Absyn_Exp*Cyc_Absyn_uint_exp(unsigned,unsigned);
# 1036
struct Cyc_Absyn_Exp*Cyc_Absyn_add_exp(struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*,unsigned);
# 1053
struct Cyc_Absyn_Exp*Cyc_Absyn_fncall_exp(struct Cyc_Absyn_Exp*,struct Cyc_List_List*,unsigned);
# 1058
struct Cyc_Absyn_Exp*Cyc_Absyn_cast_exp(void*,struct Cyc_Absyn_Exp*,int user_cast,enum Cyc_Absyn_Coercion,unsigned);
# 1060
struct Cyc_Absyn_Exp*Cyc_Absyn_sizeoftype_exp(void*t,unsigned);
# 1068
struct Cyc_Absyn_Exp*Cyc_Absyn_stmt_exp(struct Cyc_Absyn_Stmt*,unsigned);
# 1071
struct Cyc_Absyn_Exp*Cyc_Absyn_valueof_exp(void*,unsigned);
# 1079
struct Cyc_Absyn_Exp*Cyc_Absyn_uniquergn_exp();
# 1083
struct Cyc_Absyn_Stmt*Cyc_Absyn_exp_stmt(struct Cyc_Absyn_Exp*,unsigned);
# 1094
struct Cyc_Absyn_Stmt*Cyc_Absyn_decl_stmt(struct Cyc_Absyn_Decl*,struct Cyc_Absyn_Stmt*,unsigned);
# 1149
void*Cyc_Absyn_pointer_expand(void*,int fresh_evar);
# 1151
int Cyc_Absyn_is_lvalue(struct Cyc_Absyn_Exp*);
# 1154
struct Cyc_Absyn_Aggrfield*Cyc_Absyn_lookup_field(struct Cyc_List_List*,struct _fat_ptr*);
# 1156
struct Cyc_Absyn_Aggrfield*Cyc_Absyn_lookup_decl_field(struct Cyc_Absyn_Aggrdecl*,struct _fat_ptr*);struct Cyc_Absynpp_Params{int expand_typedefs;int qvar_to_Cids;int add_cyc_prefix;int to_VC;int decls_first;int rewrite_temp_tvars;int print_all_tvars;int print_all_kinds;int print_all_effects;int print_using_stmts;int print_externC_stmts;int print_full_evars;int print_zeroterm;int generate_line_directives;int use_curr_namespace;struct Cyc_List_List*curr_namespace;};
# 62 "absynpp.h"
struct _fat_ptr Cyc_Absynpp_typ2string(void*);
# 67
struct _fat_ptr Cyc_Absynpp_exp2string(struct Cyc_Absyn_Exp*);
# 69
struct _fat_ptr Cyc_Absynpp_qvar2string(struct _tuple1*);
# 27 "unify.h"
void Cyc_Unify_explain_failure();
# 29
int Cyc_Unify_unify(void*,void*);struct _union_RelnOp_RConst{int tag;unsigned val;};struct _union_RelnOp_RVar{int tag;struct Cyc_Absyn_Vardecl*val;};struct _union_RelnOp_RNumelts{int tag;struct Cyc_Absyn_Vardecl*val;};struct _union_RelnOp_RType{int tag;void*val;};struct _union_RelnOp_RParam{int tag;unsigned val;};struct _union_RelnOp_RParamNumelts{int tag;unsigned val;};struct _union_RelnOp_RReturn{int tag;unsigned val;};union Cyc_Relations_RelnOp{struct _union_RelnOp_RConst RConst;struct _union_RelnOp_RVar RVar;struct _union_RelnOp_RNumelts RNumelts;struct _union_RelnOp_RType RType;struct _union_RelnOp_RParam RParam;struct _union_RelnOp_RParamNumelts RParamNumelts;struct _union_RelnOp_RReturn RReturn;};
# 50 "relations-ap.h"
enum Cyc_Relations_Relation{Cyc_Relations_Req =0U,Cyc_Relations_Rneq =1U,Cyc_Relations_Rlte =2U,Cyc_Relations_Rlt =3U};struct Cyc_Relations_Reln{union Cyc_Relations_RelnOp rop1;enum Cyc_Relations_Relation relation;union Cyc_Relations_RelnOp rop2;};struct Cyc_Iter_Iter{void*env;int(*next)(void*env,void*dest);};struct Cyc_Dict_T;struct Cyc_Dict_Dict{int(*rel)(void*,void*);struct _RegionHandle*r;const struct Cyc_Dict_T*t;};extern char Cyc_Dict_Present[8U];struct Cyc_Dict_Present_exn_struct{char*tag;};extern char Cyc_Dict_Absent[7U];struct Cyc_Dict_Absent_exn_struct{char*tag;};struct Cyc_RgnOrder_RgnPO;extern char Cyc_Tcenv_Env_error[10U];struct Cyc_Tcenv_Env_error_exn_struct{char*tag;};struct Cyc_Tcenv_Genv{struct Cyc_Dict_Dict aggrdecls;struct Cyc_Dict_Dict datatypedecls;struct Cyc_Dict_Dict enumdecls;struct Cyc_Dict_Dict typedefs;struct Cyc_Dict_Dict ordinaries;};struct Cyc_Tcenv_Fenv;struct Cyc_Tcenv_Tenv{struct Cyc_List_List*ns;struct Cyc_Tcenv_Genv*ae;struct Cyc_Tcenv_Fenv*le;int allow_valueof: 1;int in_extern_c_include: 1;int in_tempest: 1;int tempest_generalize: 1;};
# 75 "tcenv.h"
void*Cyc_Tcenv_lookup_ordinary_global(struct Cyc_Tcenv_Tenv*,unsigned,struct _tuple1*,int is_use);
struct Cyc_Absyn_Aggrdecl**Cyc_Tcenv_lookup_aggrdecl(struct Cyc_Tcenv_Tenv*,unsigned,struct _tuple1*);
struct Cyc_Absyn_Datatypedecl**Cyc_Tcenv_lookup_datatypedecl(struct Cyc_Tcenv_Tenv*,unsigned,struct _tuple1*);
# 82
struct Cyc_Tcenv_Tenv*Cyc_Tcenv_allow_valueof(struct Cyc_Tcenv_Tenv*);
# 88
enum Cyc_Tcenv_NewStatus{Cyc_Tcenv_NoneNew =0U,Cyc_Tcenv_InNew =1U,Cyc_Tcenv_InNewAggr =2U};
struct Cyc_Tcenv_Tenv*Cyc_Tcenv_set_new_status(enum Cyc_Tcenv_NewStatus,struct Cyc_Tcenv_Tenv*);
enum Cyc_Tcenv_NewStatus Cyc_Tcenv_new_status(struct Cyc_Tcenv_Tenv*);
# 92
int Cyc_Tcenv_abstract_val_ok(struct Cyc_Tcenv_Tenv*);
struct Cyc_Tcenv_Tenv*Cyc_Tcenv_enter_abstract_val_ok(struct Cyc_Tcenv_Tenv*);
struct Cyc_Tcenv_Tenv*Cyc_Tcenv_clear_abstract_val_ok(struct Cyc_Tcenv_Tenv*);
# 98
struct Cyc_List_List*Cyc_Tcenv_lookup_type_vars(struct Cyc_Tcenv_Tenv*);
struct Cyc_Core_Opt*Cyc_Tcenv_lookup_opt_type_vars(struct Cyc_Tcenv_Tenv*);
# 114
struct Cyc_Tcenv_Tenv*Cyc_Tcenv_enter_notreadctxt(struct Cyc_Tcenv_Tenv*);
struct Cyc_Tcenv_Tenv*Cyc_Tcenv_clear_notreadctxt(struct Cyc_Tcenv_Tenv*);
int Cyc_Tcenv_in_notreadctxt(struct Cyc_Tcenv_Tenv*);
# 119
struct Cyc_Tcenv_Tenv*Cyc_Tcenv_enter_lhs(struct Cyc_Tcenv_Tenv*);
struct Cyc_Tcenv_Tenv*Cyc_Tcenv_clear_lhs(struct Cyc_Tcenv_Tenv*);
# 123
struct Cyc_Tcenv_Tenv*Cyc_Tcenv_enter_stmt_exp(struct Cyc_Tcenv_Tenv*);
# 130
struct Cyc_Tcenv_Tenv*Cyc_Tcenv_new_block(unsigned,struct Cyc_Tcenv_Tenv*);
# 137
void*Cyc_Tcenv_curr_rgn(struct Cyc_Tcenv_Tenv*);
# 141
void Cyc_Tcenv_check_rgn_accessible(struct Cyc_Tcenv_Tenv*,unsigned,void*rgn);
# 143
void Cyc_Tcenv_check_effect_accessible(struct Cyc_Tcenv_Tenv*,unsigned,void*eff);
# 30 "tcutil.h"
void*Cyc_Tcutil_impos(struct _fat_ptr,struct _fat_ptr);
void Cyc_Tcutil_terr(unsigned,struct _fat_ptr,struct _fat_ptr);
void Cyc_Tcutil_warn(unsigned,struct _fat_ptr,struct _fat_ptr);
# 41
int Cyc_Tcutil_is_arithmetic_type(void*);
# 44
int Cyc_Tcutil_is_pointer_type(void*);
int Cyc_Tcutil_is_array_type(void*);
int Cyc_Tcutil_is_boxed(void*);
# 51
int Cyc_Tcutil_is_nullable_pointer_type(void*);
int Cyc_Tcutil_is_bound_one(void*);
# 54
int Cyc_Tcutil_is_fat_pointer_type(void*);
# 57
int Cyc_Tcutil_is_bits_only_type(void*);
# 64
void*Cyc_Tcutil_pointer_elt_type(void*);
# 66
void*Cyc_Tcutil_pointer_region(void*);
# 69
int Cyc_Tcutil_rgn_of_pointer(void*t,void**rgn);
# 72
struct Cyc_Absyn_Exp*Cyc_Tcutil_get_bounds_exp(void*def,void*b);
# 77
int Cyc_Tcutil_is_fat_pointer_type_elt(void*t,void**elt_dest);
# 79
int Cyc_Tcutil_is_zero_pointer_type_elt(void*t,void**elt_type_dest);
# 84
struct Cyc_Absyn_Exp*Cyc_Tcutil_get_bounds_exp(void*def,void*b);
# 87
int Cyc_Tcutil_is_integral(struct Cyc_Absyn_Exp*);
int Cyc_Tcutil_is_numeric(struct Cyc_Absyn_Exp*);
int Cyc_Tcutil_is_zero(struct Cyc_Absyn_Exp*);
# 94
void*Cyc_Tcutil_copy_type(void*);
# 97
struct Cyc_Absyn_Exp*Cyc_Tcutil_deep_copy_exp(int preserve_types,struct Cyc_Absyn_Exp*);
# 100
int Cyc_Tcutil_kind_leq(struct Cyc_Absyn_Kind*,struct Cyc_Absyn_Kind*);
# 105
struct Cyc_Absyn_Kind*Cyc_Tcutil_tvar_kind(struct Cyc_Absyn_Tvar*,struct Cyc_Absyn_Kind*def);
struct Cyc_Absyn_Kind*Cyc_Tcutil_type_kind(void*);
void*Cyc_Tcutil_compress(void*);
void Cyc_Tcutil_unchecked_cast(struct Cyc_Absyn_Exp*,void*,enum Cyc_Absyn_Coercion);
int Cyc_Tcutil_coerce_uint_type(struct Cyc_Absyn_Exp*);
int Cyc_Tcutil_coerce_sint_type(struct Cyc_Absyn_Exp*);
int Cyc_Tcutil_coerce_to_bool(struct Cyc_Absyn_Exp*);
# 113
int Cyc_Tcutil_coerce_arg(struct Cyc_Tcenv_Tenv*te,struct Cyc_Absyn_Exp*,void*,int*alias_coercion);
int Cyc_Tcutil_coerce_assign(struct Cyc_Tcenv_Tenv*te,struct Cyc_Absyn_Exp*,void*);
int Cyc_Tcutil_coerce_list(struct Cyc_Tcenv_Tenv*te,void*,struct Cyc_List_List*);
# 117
int Cyc_Tcutil_silent_castable(struct Cyc_Tcenv_Tenv*te,unsigned,void*,void*);
# 119
enum Cyc_Absyn_Coercion Cyc_Tcutil_castable(struct Cyc_Tcenv_Tenv*te,unsigned,void*,void*);
# 124
int Cyc_Tcutil_zero_to_null(void*,struct Cyc_Absyn_Exp*);struct _tuple12{struct Cyc_Absyn_Decl*f1;struct Cyc_Absyn_Exp*f2;};
# 127
struct _tuple12 Cyc_Tcutil_insert_alias(struct Cyc_Absyn_Exp*e,void*e_typ);
# 136
extern struct Cyc_Absyn_Kind Cyc_Tcutil_ak;
extern struct Cyc_Absyn_Kind Cyc_Tcutil_bk;
# 145
extern struct Cyc_Absyn_Kind Cyc_Tcutil_tak;
# 147
extern struct Cyc_Absyn_Kind Cyc_Tcutil_tmk;
# 154
extern struct Cyc_Core_Opt Cyc_Tcutil_rko;
extern struct Cyc_Core_Opt Cyc_Tcutil_ako;
extern struct Cyc_Core_Opt Cyc_Tcutil_bko;
# 163
extern struct Cyc_Core_Opt Cyc_Tcutil_trko;
extern struct Cyc_Core_Opt Cyc_Tcutil_tako;
# 166
extern struct Cyc_Core_Opt Cyc_Tcutil_tmko;
# 179
void*Cyc_Tcutil_max_arithmetic_type(void*,void*);
# 185
void*Cyc_Tcutil_substitute(struct Cyc_List_List*,void*);
# 202
void*Cyc_Tcutil_fndecl2type(struct Cyc_Absyn_Fndecl*);struct _tuple13{struct Cyc_List_List*f1;struct _RegionHandle*f2;};struct _tuple14{struct Cyc_Absyn_Tvar*f1;void*f2;};
# 207
struct _tuple14*Cyc_Tcutil_r_make_inst_var(struct _tuple13*,struct Cyc_Absyn_Tvar*);
# 218
void Cyc_Tcutil_check_nonzero_bound(unsigned,void*);
# 220
void Cyc_Tcutil_check_bound(unsigned,unsigned i,void*,int do_warn);
# 222
struct Cyc_List_List*Cyc_Tcutil_resolve_aggregate_designators(struct _RegionHandle*,unsigned,struct Cyc_List_List*,enum Cyc_Absyn_AggrKind,struct Cyc_List_List*);
# 233
int Cyc_Tcutil_is_noalias_region(void*r,int must_be_unique);
# 236
int Cyc_Tcutil_is_noalias_pointer(void*t,int must_be_unique);
# 241
int Cyc_Tcutil_is_noalias_path(struct Cyc_Absyn_Exp*);
# 246
int Cyc_Tcutil_is_noalias_pointer_or_aggr(void*);struct _tuple15{int f1;void*f2;};
# 250
struct _tuple15 Cyc_Tcutil_addressof_props(struct Cyc_Absyn_Exp*);
# 273
int Cyc_Tcutil_is_const_exp(struct Cyc_Absyn_Exp*);
# 287
void Cyc_Tcutil_check_no_qual(unsigned,void*);
# 298
void*Cyc_Tcutil_promote_array(void*t,void*rgn,int convert_tag);
# 301
int Cyc_Tcutil_zeroable_type(void*);
# 305
int Cyc_Tcutil_force_type2bool(int desired,void*);
# 308
void*Cyc_Tcutil_any_bool(struct Cyc_List_List*);
# 310
void*Cyc_Tcutil_any_bounds(struct Cyc_List_List*);struct _tuple16{unsigned f1;int f2;};
# 28 "evexp.h"
struct _tuple16 Cyc_Evexp_eval_const_uint_exp(struct Cyc_Absyn_Exp*e);
# 32
int Cyc_Evexp_c_can_eval(struct Cyc_Absyn_Exp*e);
# 41 "evexp.h"
int Cyc_Evexp_same_const_exp(struct Cyc_Absyn_Exp*e1,struct Cyc_Absyn_Exp*e2);
# 48
int Cyc_Evexp_okay_szofarg(void*t);
# 26 "tcstmt.h"
void Cyc_Tcstmt_tcStmt(struct Cyc_Tcenv_Tenv*,struct Cyc_Absyn_Stmt*,int new_block);
# 30 "formatstr.h"
struct Cyc_List_List*Cyc_Formatstr_get_format_types(struct Cyc_Tcenv_Tenv*,struct _fat_ptr,int isCproto,unsigned);
# 33
struct Cyc_List_List*Cyc_Formatstr_get_scanf_types(struct Cyc_Tcenv_Tenv*,struct _fat_ptr,int isCproto,unsigned);
# 31 "tc.h"
extern int Cyc_Tc_aggressive_warn;
# 44 "tctyp.h"
void Cyc_Tctyp_check_type(unsigned,struct Cyc_Tcenv_Tenv*,struct Cyc_List_List*bound_tvars,struct Cyc_Absyn_Kind*k,int allow_evars,int allow_abs_aggr,void*);
# 28 "tcexp.h"
void*Cyc_Tcexp_tcExp(struct Cyc_Tcenv_Tenv*,void**,struct Cyc_Absyn_Exp*);
void*Cyc_Tcexp_tcExpInitializer(struct Cyc_Tcenv_Tenv*,void**,struct Cyc_Absyn_Exp*);
void Cyc_Tcexp_tcTest(struct Cyc_Tcenv_Tenv*te,struct Cyc_Absyn_Exp*e,struct _fat_ptr msg_part);
# 49 "tcexp.cyc"
static void*Cyc_Tcexp_expr_err(struct Cyc_Tcenv_Tenv*te,unsigned loc,void**topt,struct _fat_ptr msg,struct _fat_ptr ap){
# 53
({void*_tmp0=0U;({unsigned _tmp4B9=loc;struct _fat_ptr _tmp4B8=(struct _fat_ptr)Cyc_vrprintf(Cyc_Core_heap_region,msg,ap);Cyc_Tcutil_terr(_tmp4B9,_tmp4B8,_tag_fat(_tmp0,sizeof(void*),0U));});});
if(topt == 0)
return Cyc_Absyn_wildtyp(Cyc_Tcenv_lookup_opt_type_vars(te));else{
# 57
return*topt;}}
# 49
static void Cyc_Tcexp_resolve_unresolved_mem(struct Cyc_Tcenv_Tenv*te,unsigned loc,void**topt,struct Cyc_Absyn_Exp*e,struct Cyc_List_List*des){
# 70
if(topt == 0){
# 72
({void*_tmp4BA=(void*)({struct Cyc_Absyn_Array_e_Absyn_Raw_exp_struct*_tmp1=_cycalloc(sizeof(*_tmp1));_tmp1->tag=26U,_tmp1->f1=des;_tmp1;});e->r=_tmp4BA;});
return;}{
# 70
void*t=*topt;
# 76
void*_stmttmp0=Cyc_Tcutil_compress(t);void*_tmp2=_stmttmp0;struct Cyc_Absyn_Tqual _tmp4;void*_tmp3;union Cyc_Absyn_AggrInfo _tmp5;switch(*((int*)_tmp2)){case 0U: if(((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2)->f1)->tag == 20U){_LL1: _tmp5=((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2)->f1)->f1;_LL2: {union Cyc_Absyn_AggrInfo info=_tmp5;
# 78
{union Cyc_Absyn_AggrInfo _tmp6=info;struct Cyc_Absyn_Aggrdecl*_tmp7;if((_tmp6.UnknownAggr).tag == 1){_LLA: _LLB:
({void*_tmp8=0U;({int(*_tmp4BC)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmpA)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmpA;});struct _fat_ptr _tmp4BB=({const char*_tmp9="struct type not properly set";_tag_fat(_tmp9,sizeof(char),29U);});_tmp4BC(_tmp4BB,_tag_fat(_tmp8,sizeof(void*),0U));});});}else{_LLC: _tmp7=*(_tmp6.KnownAggr).val;_LLD: {struct Cyc_Absyn_Aggrdecl*ad=_tmp7;
({void*_tmp4BD=(void*)({struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct*_tmpB=_cycalloc(sizeof(*_tmpB));_tmpB->tag=29U,_tmpB->f1=ad->name,_tmpB->f2=0,_tmpB->f3=des,_tmpB->f4=ad;_tmpB;});e->r=_tmp4BD;});}}_LL9:;}
# 82
goto _LL0;}}else{goto _LL7;}case 4U: _LL3: _tmp3=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp2)->f1).elt_type;_tmp4=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp2)->f1).tq;_LL4: {void*at=_tmp3;struct Cyc_Absyn_Tqual aq=_tmp4;
({void*_tmp4BE=(void*)({struct Cyc_Absyn_Array_e_Absyn_Raw_exp_struct*_tmpC=_cycalloc(sizeof(*_tmpC));_tmpC->tag=26U,_tmpC->f1=des;_tmpC;});e->r=_tmp4BE;});goto _LL0;}case 7U: _LL5: _LL6:
({void*_tmp4BF=(void*)({struct Cyc_Absyn_AnonStruct_e_Absyn_Raw_exp_struct*_tmpD=_cycalloc(sizeof(*_tmpD));_tmpD->tag=30U,_tmpD->f1=t,_tmpD->f2=des;_tmpD;});e->r=_tmp4BF;});goto _LL0;default: _LL7: _LL8:
({void*_tmp4C0=(void*)({struct Cyc_Absyn_Array_e_Absyn_Raw_exp_struct*_tmpE=_cycalloc(sizeof(*_tmpE));_tmpE->tag=26U,_tmpE->f1=des;_tmpE;});e->r=_tmp4C0;});goto _LL0;}_LL0:;}}
# 49
static void Cyc_Tcexp_tcExpNoInst(struct Cyc_Tcenv_Tenv*te,void**topt,struct Cyc_Absyn_Exp*e);
# 93
static void*Cyc_Tcexp_tcExpNoPromote(struct Cyc_Tcenv_Tenv*te,void**topt,struct Cyc_Absyn_Exp*e);
# 96
static void Cyc_Tcexp_tcExpList(struct Cyc_Tcenv_Tenv*te,struct Cyc_List_List*es){
for(0;es != 0;es=es->tl){
Cyc_Tcexp_tcExp(te,0,(struct Cyc_Absyn_Exp*)es->hd);}}
# 96
static void Cyc_Tcexp_check_contains_assign(struct Cyc_Absyn_Exp*e){
# 103
void*_stmttmp1=e->r;void*_tmpF=_stmttmp1;if(((struct Cyc_Absyn_AssignOp_e_Absyn_Raw_exp_struct*)_tmpF)->tag == 4U){if(((struct Cyc_Absyn_AssignOp_e_Absyn_Raw_exp_struct*)_tmpF)->f2 == 0){_LL1: _LL2:
# 105
 if(Cyc_Tc_aggressive_warn)({void*_tmp10=0U;({unsigned _tmp4C2=e->loc;struct _fat_ptr _tmp4C1=({const char*_tmp11="assignment in test";_tag_fat(_tmp11,sizeof(char),19U);});Cyc_Tcutil_warn(_tmp4C2,_tmp4C1,_tag_fat(_tmp10,sizeof(void*),0U));});});goto _LL0;}else{goto _LL3;}}else{_LL3: _LL4:
# 107
 goto _LL0;}_LL0:;}
# 96
void Cyc_Tcexp_tcTest(struct Cyc_Tcenv_Tenv*te,struct Cyc_Absyn_Exp*e,struct _fat_ptr msg_part){
# 113
Cyc_Tcexp_check_contains_assign(e);
Cyc_Tcexp_tcExp(te,& Cyc_Absyn_sint_type,e);
if(!Cyc_Tcutil_coerce_to_bool(e))
({struct Cyc_String_pa_PrintArg_struct _tmp14=({struct Cyc_String_pa_PrintArg_struct _tmp432;_tmp432.tag=0U,_tmp432.f1=(struct _fat_ptr)((struct _fat_ptr)msg_part);_tmp432;});struct Cyc_String_pa_PrintArg_struct _tmp15=({struct Cyc_String_pa_PrintArg_struct _tmp431;_tmp431.tag=0U,({
struct _fat_ptr _tmp4C3=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string((void*)_check_null(e->topt)));_tmp431.f1=_tmp4C3;});_tmp431;});void*_tmp12[2U];_tmp12[0]=& _tmp14,_tmp12[1]=& _tmp15;({unsigned _tmp4C5=e->loc;struct _fat_ptr _tmp4C4=({const char*_tmp13="test of %s has type %s instead of integral or pointer type";_tag_fat(_tmp13,sizeof(char),59U);});Cyc_Tcutil_terr(_tmp4C5,_tmp4C4,_tag_fat(_tmp12,sizeof(void*),2U));});});}
# 138 "tcexp.cyc"
static int Cyc_Tcexp_wchar_numelts(struct _fat_ptr s){
return 1;}
# 138
static void*Cyc_Tcexp_tcConst(struct Cyc_Tcenv_Tenv*te,unsigned loc,void**topt,union Cyc_Absyn_Cnst*c,struct Cyc_Absyn_Exp*e){
# 144
void*t;
void*string_elt_typ=Cyc_Absyn_char_type;
int string_numelts=0;
{union Cyc_Absyn_Cnst _stmttmp2=*((union Cyc_Absyn_Cnst*)_check_null(c));union Cyc_Absyn_Cnst _tmp16=_stmttmp2;struct _fat_ptr _tmp17;struct _fat_ptr _tmp18;int _tmp1A;enum Cyc_Absyn_Sign _tmp19;int _tmp1B;enum Cyc_Absyn_Sign _tmp1C;enum Cyc_Absyn_Sign _tmp1D;switch((_tmp16.Wstring_c).tag){case 2U: switch(((_tmp16.Char_c).val).f1){case Cyc_Absyn_Signed: _LL1: _LL2:
 t=Cyc_Absyn_schar_type;goto _LL0;case Cyc_Absyn_Unsigned: _LL3: _LL4:
 t=Cyc_Absyn_uchar_type;goto _LL0;default: _LL5: _LL6:
 t=Cyc_Absyn_char_type;goto _LL0;}case 3U: _LL7: _LL8:
 t=Cyc_Absyn_wchar_type();goto _LL0;case 4U: _LL9: _tmp1D=((_tmp16.Short_c).val).f1;_LLA: {enum Cyc_Absyn_Sign sn=_tmp1D;
# 153
t=(int)sn == (int)1U?Cyc_Absyn_ushort_type: Cyc_Absyn_sshort_type;goto _LL0;}case 6U: _LLB: _tmp1C=((_tmp16.LongLong_c).val).f1;_LLC: {enum Cyc_Absyn_Sign sn=_tmp1C;
# 155
t=(int)sn == (int)1U?Cyc_Absyn_ulonglong_type: Cyc_Absyn_slonglong_type;goto _LL0;}case 7U: _LLD: _tmp1B=((_tmp16.Float_c).val).f2;_LLE: {int i=_tmp1B;
# 157
if(topt == 0)t=Cyc_Absyn_gen_float_type((unsigned)i);else{
# 161
void*_stmttmp3=Cyc_Tcutil_compress(*topt);void*_tmp1E=_stmttmp3;int _tmp1F;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp1E)->tag == 0U){if(((struct Cyc_Absyn_FloatCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp1E)->f1)->tag == 2U){_LL18: _tmp1F=((struct Cyc_Absyn_FloatCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp1E)->f1)->f1;_LL19: {int i=_tmp1F;
t=Cyc_Absyn_gen_float_type((unsigned)i);goto _LL17;}}else{goto _LL1A;}}else{_LL1A: _LL1B:
# 165
 t=Cyc_Absyn_gen_float_type((unsigned)i);
goto _LL17;}_LL17:;}
# 169
goto _LL0;}case 5U: _LLF: _tmp19=((_tmp16.Int_c).val).f1;_tmp1A=((_tmp16.Int_c).val).f2;_LL10: {enum Cyc_Absyn_Sign csn=_tmp19;int i=_tmp1A;
# 171
if(topt == 0)
t=(int)csn == (int)1U?Cyc_Absyn_uint_type: Cyc_Absyn_sint_type;else{
# 178
void*_stmttmp4=Cyc_Tcutil_compress(*topt);void*_tmp20=_stmttmp4;void*_tmp26;void*_tmp25;void*_tmp24;void*_tmp23;struct Cyc_Absyn_Tqual _tmp22;void*_tmp21;void*_tmp27;enum Cyc_Absyn_Sign _tmp28;enum Cyc_Absyn_Sign _tmp29;enum Cyc_Absyn_Sign _tmp2A;enum Cyc_Absyn_Sign _tmp2B;switch(*((int*)_tmp20)){case 0U: switch(*((int*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp20)->f1)){case 1U: switch(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp20)->f1)->f2){case Cyc_Absyn_Char_sz: _LL1D: _tmp2B=((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp20)->f1)->f1;_LL1E: {enum Cyc_Absyn_Sign sn=_tmp2B;
# 180
{enum Cyc_Absyn_Sign _tmp2C=sn;switch(_tmp2C){case Cyc_Absyn_Unsigned: _LL2C: _LL2D:
 t=Cyc_Absyn_uchar_type;goto _LL2B;case Cyc_Absyn_Signed: _LL2E: _LL2F:
 t=Cyc_Absyn_schar_type;goto _LL2B;default: _LL30: _LL31:
 t=Cyc_Absyn_char_type;goto _LL2B;}_LL2B:;}
# 185
({union Cyc_Absyn_Cnst _tmp4C6=Cyc_Absyn_Char_c(sn,(char)i);*c=_tmp4C6;});
goto _LL1C;}case Cyc_Absyn_Short_sz: _LL1F: _tmp2A=((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp20)->f1)->f1;_LL20: {enum Cyc_Absyn_Sign sn=_tmp2A;
# 188
t=(int)sn == (int)1U?Cyc_Absyn_ushort_type: Cyc_Absyn_sshort_type;
({union Cyc_Absyn_Cnst _tmp4C7=Cyc_Absyn_Short_c(sn,(short)i);*c=_tmp4C7;});
goto _LL1C;}case Cyc_Absyn_Int_sz: _LL21: _tmp29=((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp20)->f1)->f1;_LL22: {enum Cyc_Absyn_Sign sn=_tmp29;
# 192
t=(int)sn == (int)1U?Cyc_Absyn_uint_type: Cyc_Absyn_sint_type;
# 195
({union Cyc_Absyn_Cnst _tmp4C8=Cyc_Absyn_Int_c(sn,i);*c=_tmp4C8;});
goto _LL1C;}case Cyc_Absyn_Long_sz: _LL23: _tmp28=((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp20)->f1)->f1;_LL24: {enum Cyc_Absyn_Sign sn=_tmp28;
# 198
t=(int)sn == (int)1U?Cyc_Absyn_uint_type: Cyc_Absyn_sint_type;
# 201
({union Cyc_Absyn_Cnst _tmp4C9=Cyc_Absyn_Int_c(sn,i);*c=_tmp4C9;});
goto _LL1C;}default: goto _LL29;}case 4U: if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp20)->f2 != 0){_LL27: _tmp27=(void*)(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp20)->f2)->hd;_LL28: {void*t1=_tmp27;
# 215
struct Cyc_Absyn_ValueofType_Absyn_Type_struct*t2=({struct Cyc_Absyn_ValueofType_Absyn_Type_struct*_tmp31=_cycalloc(sizeof(*_tmp31));_tmp31->tag=9U,({struct Cyc_Absyn_Exp*_tmp4CA=Cyc_Absyn_uint_exp((unsigned)i,0U);_tmp31->f1=_tmp4CA;});_tmp31;});
# 222
t=Cyc_Absyn_tag_type((void*)t2);
goto _LL1C;}}else{goto _LL29;}default: goto _LL29;}case 3U: _LL25: _tmp21=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp20)->f1).elt_type;_tmp22=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp20)->f1).elt_tq;_tmp23=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp20)->f1).ptr_atts).rgn;_tmp24=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp20)->f1).ptr_atts).nullable;_tmp25=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp20)->f1).ptr_atts).bounds;_tmp26=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp20)->f1).ptr_atts).zero_term;if(i == 0){_LL26: {void*et=_tmp21;struct Cyc_Absyn_Tqual etq=_tmp22;void*rgn=_tmp23;void*nbl=_tmp24;void*bnd=_tmp25;void*zt=_tmp26;
# 205
static struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct nullc={0U,{.Null_c={1,0}}};
e->r=(void*)& nullc;
if(Cyc_Tcutil_force_type2bool(1,nbl))return*topt;({struct Cyc_String_pa_PrintArg_struct _tmp2F=({struct Cyc_String_pa_PrintArg_struct _tmp433;_tmp433.tag=0U,({
# 209
struct _fat_ptr _tmp4CB=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(*topt));_tmp433.f1=_tmp4CB;});_tmp433;});void*_tmp2D[1U];_tmp2D[0]=& _tmp2F;({unsigned _tmp4CD=e->loc;struct _fat_ptr _tmp4CC=({const char*_tmp2E="Used NULL when expecting a non-NULL value of type %s";_tag_fat(_tmp2E,sizeof(char),53U);});Cyc_Tcutil_terr(_tmp4CD,_tmp4CC,_tag_fat(_tmp2D,sizeof(void*),1U));});});{
struct Cyc_List_List*tenv_tvs=Cyc_Tcenv_lookup_type_vars(te);
t=(void*)({struct Cyc_Absyn_PointerType_Absyn_Type_struct*_tmp30=_cycalloc(sizeof(*_tmp30));_tmp30->tag=3U,(_tmp30->f1).elt_type=et,(_tmp30->f1).elt_tq=etq,
((_tmp30->f1).ptr_atts).rgn=rgn,((_tmp30->f1).ptr_atts).nullable=Cyc_Absyn_true_type,((_tmp30->f1).ptr_atts).bounds=bnd,((_tmp30->f1).ptr_atts).zero_term=zt,((_tmp30->f1).ptr_atts).ptrloc=0;_tmp30;});
goto _LL1C;}}}else{goto _LL29;}default: _LL29: _LL2A:
# 225
 t=(int)csn == (int)1U?Cyc_Absyn_uint_type: Cyc_Absyn_sint_type;
goto _LL1C;}_LL1C:;}
# 228
goto _LL0;}case 8U: _LL11: _tmp18=(_tmp16.String_c).val;_LL12: {struct _fat_ptr s=_tmp18;
# 230
string_numelts=(int)_get_fat_size(s,sizeof(char));
_tmp17=s;goto _LL14;}case 9U: _LL13: _tmp17=(_tmp16.Wstring_c).val;_LL14: {struct _fat_ptr s=_tmp17;
# 233
if(string_numelts == 0){
string_numelts=Cyc_Tcexp_wchar_numelts(s);
string_elt_typ=Cyc_Absyn_wchar_type();}{
# 233
struct Cyc_Absyn_Exp*elen=({
# 237
union Cyc_Absyn_Cnst _tmp4CE=Cyc_Absyn_Int_c(Cyc_Absyn_Unsigned,string_numelts);Cyc_Absyn_const_exp(_tmp4CE,loc);});
elen->topt=Cyc_Absyn_uint_type;{
# 242
void*elen_bound=Cyc_Absyn_thin_bounds_exp(elen);
t=({void*_tmp4D2=string_elt_typ;void*_tmp4D1=Cyc_Absyn_heap_rgn_type;struct Cyc_Absyn_Tqual _tmp4D0=Cyc_Absyn_const_tqual(0U);void*_tmp4CF=elen_bound;Cyc_Absyn_atb_type(_tmp4D2,_tmp4D1,_tmp4D0,_tmp4CF,Cyc_Absyn_true_type);});
# 245
if(topt != 0){
void*_stmttmp5=Cyc_Tcutil_compress(*topt);void*_tmp32=_stmttmp5;struct Cyc_Absyn_Tqual _tmp33;switch(*((int*)_tmp32)){case 4U: _LL33: _tmp33=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp32)->f1).tq;_LL34: {struct Cyc_Absyn_Tqual tq=_tmp33;
# 251
return({void*_tmp4D5=string_elt_typ;struct Cyc_Absyn_Tqual _tmp4D4=tq;struct Cyc_Absyn_Exp*_tmp4D3=elen;Cyc_Absyn_array_type(_tmp4D5,_tmp4D4,_tmp4D3,
Cyc_Tcutil_any_bool(Cyc_Tcenv_lookup_type_vars(te)),0U);});}case 3U: _LL35: _LL36:
# 254
 if(!Cyc_Unify_unify(*topt,t)&&
 Cyc_Tcutil_silent_castable(te,loc,t,*topt)){
e->topt=t;
Cyc_Tcutil_unchecked_cast(e,*topt,Cyc_Absyn_Other_coercion);
t=*topt;}else{
# 261
t=({void*_tmp4D9=string_elt_typ;void*_tmp4D8=Cyc_Absyn_new_evar(& Cyc_Tcutil_rko,Cyc_Tcenv_lookup_opt_type_vars(te));struct Cyc_Absyn_Tqual _tmp4D7=
Cyc_Absyn_const_tqual(0U);
# 261
void*_tmp4D6=elen_bound;Cyc_Absyn_atb_type(_tmp4D9,_tmp4D8,_tmp4D7,_tmp4D6,Cyc_Absyn_true_type);});
# 263
if(!Cyc_Unify_unify(*topt,t)&&
 Cyc_Tcutil_silent_castable(te,loc,t,*topt)){
e->topt=t;
Cyc_Tcutil_unchecked_cast(e,*topt,Cyc_Absyn_Other_coercion);
t=*topt;}}
# 270
goto _LL32;default: _LL37: _LL38:
 goto _LL32;}_LL32:;}
# 245
return t;}}}default: _LL15: _LL16:
# 276
 if(topt != 0){
void*_stmttmp6=Cyc_Tcutil_compress(*topt);void*_tmp34=_stmttmp6;void*_tmp3A;void*_tmp39;void*_tmp38;void*_tmp37;struct Cyc_Absyn_Tqual _tmp36;void*_tmp35;if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp34)->tag == 3U){_LL3A: _tmp35=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp34)->f1).elt_type;_tmp36=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp34)->f1).elt_tq;_tmp37=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp34)->f1).ptr_atts).rgn;_tmp38=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp34)->f1).ptr_atts).nullable;_tmp39=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp34)->f1).ptr_atts).bounds;_tmp3A=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp34)->f1).ptr_atts).zero_term;_LL3B: {void*et=_tmp35;struct Cyc_Absyn_Tqual etq=_tmp36;void*rgn=_tmp37;void*nbl=_tmp38;void*bnd=_tmp39;void*zt=_tmp3A;
# 280
if(Cyc_Tcutil_force_type2bool(1,nbl))return*topt;({struct Cyc_String_pa_PrintArg_struct _tmp3D=({struct Cyc_String_pa_PrintArg_struct _tmp434;_tmp434.tag=0U,({
# 282
struct _fat_ptr _tmp4DA=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(*topt));_tmp434.f1=_tmp4DA;});_tmp434;});void*_tmp3B[1U];_tmp3B[0]=& _tmp3D;({unsigned _tmp4DC=e->loc;struct _fat_ptr _tmp4DB=({const char*_tmp3C="Used NULL when expecting a non-NULL value of type %s";_tag_fat(_tmp3C,sizeof(char),53U);});Cyc_Tcutil_terr(_tmp4DC,_tmp4DB,_tag_fat(_tmp3B,sizeof(void*),1U));});});
return(void*)({struct Cyc_Absyn_PointerType_Absyn_Type_struct*_tmp3E=_cycalloc(sizeof(*_tmp3E));_tmp3E->tag=3U,(_tmp3E->f1).elt_type=et,(_tmp3E->f1).elt_tq=etq,((_tmp3E->f1).ptr_atts).rgn=rgn,((_tmp3E->f1).ptr_atts).nullable=Cyc_Absyn_true_type,((_tmp3E->f1).ptr_atts).bounds=bnd,((_tmp3E->f1).ptr_atts).zero_term=zt,((_tmp3E->f1).ptr_atts).ptrloc=0;_tmp3E;});}}else{_LL3C: _LL3D:
# 285
 goto _LL39;}_LL39:;}{
# 276
struct Cyc_List_List*tenv_tvs=
# 288
Cyc_Tcenv_lookup_type_vars(te);
t=(void*)({struct Cyc_Absyn_PointerType_Absyn_Type_struct*_tmp41=_cycalloc(sizeof(*_tmp41));_tmp41->tag=3U,({void*_tmp4E1=Cyc_Absyn_new_evar(& Cyc_Tcutil_tako,({struct Cyc_Core_Opt*_tmp3F=_cycalloc(sizeof(*_tmp3F));_tmp3F->v=tenv_tvs;_tmp3F;}));(_tmp41->f1).elt_type=_tmp4E1;}),({
struct Cyc_Absyn_Tqual _tmp4E0=Cyc_Absyn_empty_tqual(0U);(_tmp41->f1).elt_tq=_tmp4E0;}),
({void*_tmp4DF=Cyc_Absyn_new_evar(& Cyc_Tcutil_trko,({struct Cyc_Core_Opt*_tmp40=_cycalloc(sizeof(*_tmp40));_tmp40->v=tenv_tvs;_tmp40;}));((_tmp41->f1).ptr_atts).rgn=_tmp4DF;}),((_tmp41->f1).ptr_atts).nullable=Cyc_Absyn_true_type,({
void*_tmp4DE=Cyc_Tcutil_any_bounds(tenv_tvs);((_tmp41->f1).ptr_atts).bounds=_tmp4DE;}),({
void*_tmp4DD=Cyc_Tcutil_any_bool(tenv_tvs);((_tmp41->f1).ptr_atts).zero_term=_tmp4DD;}),((_tmp41->f1).ptr_atts).ptrloc=0;_tmp41;});
# 295
goto _LL0;}}_LL0:;}
# 297
return t;}
# 138
static void*Cyc_Tcexp_tcDatatype(struct Cyc_Tcenv_Tenv*te,unsigned loc,void**topt,struct Cyc_Absyn_Exp*e,struct Cyc_List_List*es,struct Cyc_Absyn_Datatypedecl*tud,struct Cyc_Absyn_Datatypefield*tuf);
# 306
static void*Cyc_Tcexp_tcVar(struct Cyc_Tcenv_Tenv*te,unsigned loc,void**topt,struct Cyc_Absyn_Exp*e,void**b){
void*_stmttmp7=*((void**)_check_null(b));void*_tmp42=_stmttmp7;struct Cyc_Absyn_Vardecl*_tmp43;struct Cyc_Absyn_Vardecl*_tmp44;struct Cyc_Absyn_Vardecl*_tmp45;struct Cyc_Absyn_Fndecl*_tmp46;struct Cyc_Absyn_Vardecl*_tmp47;struct _tuple1*_tmp48;switch(*((int*)_tmp42)){case 0U: _LL1: _tmp48=((struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct*)_tmp42)->f1;_LL2: {struct _tuple1*q=_tmp48;
({void*_tmp49=0U;({int(*_tmp4E3)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp4B)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp4B;});struct _fat_ptr _tmp4E2=({const char*_tmp4A="unresolved binding in tcVar";_tag_fat(_tmp4A,sizeof(char),28U);});_tmp4E3(_tmp4E2,_tag_fat(_tmp49,sizeof(void*),0U));});});}case 1U: _LL3: _tmp47=((struct Cyc_Absyn_Global_b_Absyn_Binding_struct*)_tmp42)->f1;_LL4: {struct Cyc_Absyn_Vardecl*vd=_tmp47;
# 312
Cyc_Tcenv_lookup_ordinary_global(te,loc,vd->name,1);
return vd->type;}case 2U: _LL5: _tmp46=((struct Cyc_Absyn_Funname_b_Absyn_Binding_struct*)_tmp42)->f1;_LL6: {struct Cyc_Absyn_Fndecl*fd=_tmp46;
# 319
if(fd->fn_vardecl == 0)
Cyc_Tcenv_lookup_ordinary_global(te,loc,fd->name,1);
# 319
return
# 321
 Cyc_Tcutil_fndecl2type(fd);}case 5U: _LL7: _tmp45=((struct Cyc_Absyn_Pat_b_Absyn_Binding_struct*)_tmp42)->f1;_LL8: {struct Cyc_Absyn_Vardecl*vd=_tmp45;
_tmp44=vd;goto _LLA;}case 4U: _LL9: _tmp44=((struct Cyc_Absyn_Local_b_Absyn_Binding_struct*)_tmp42)->f1;_LLA: {struct Cyc_Absyn_Vardecl*vd=_tmp44;
_tmp43=vd;goto _LLC;}default: _LLB: _tmp43=((struct Cyc_Absyn_Param_b_Absyn_Binding_struct*)_tmp42)->f1;_LLC: {struct Cyc_Absyn_Vardecl*vd=_tmp43;
# 325
if(te->allow_valueof){
void*_stmttmp8=Cyc_Tcutil_compress(vd->type);void*_tmp4C=_stmttmp8;void*_tmp4D;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp4C)->tag == 0U){if(((struct Cyc_Absyn_TagCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp4C)->f1)->tag == 4U){if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp4C)->f2 != 0){_LLE: _tmp4D=(void*)(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp4C)->f2)->hd;_LLF: {void*i=_tmp4D;
# 328
({void*_tmp4E4=(void*)({struct Cyc_Absyn_Valueof_e_Absyn_Raw_exp_struct*_tmp4E=_cycalloc(sizeof(*_tmp4E));_tmp4E->tag=40U,_tmp4E->f1=i;_tmp4E;});e->r=_tmp4E4;});
goto _LLD;}}else{goto _LL10;}}else{goto _LL10;}}else{_LL10: _LL11:
 goto _LLD;}_LLD:;}
# 325
return vd->type;}}_LL0:;}
# 306
static void Cyc_Tcexp_check_format_args(struct Cyc_Tcenv_Tenv*te,struct Cyc_Absyn_Exp*fmt,struct Cyc_Core_Opt*opt_args,int arg_cnt,struct Cyc_List_List**alias_arg_exps,int isCproto,struct Cyc_List_List*(*type_getter)(struct Cyc_Tcenv_Tenv*,struct _fat_ptr,int,unsigned)){
# 345
struct Cyc_List_List*desc_types;
{void*_stmttmp9=fmt->r;void*_tmp4F=_stmttmp9;struct _fat_ptr _tmp50;struct _fat_ptr _tmp51;switch(*((int*)_tmp4F)){case 0U: if(((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmp4F)->f1).String_c).tag == 8){_LL1: _tmp51=((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmp4F)->f1).String_c).val;_LL2: {struct _fat_ptr s=_tmp51;
_tmp50=s;goto _LL4;}}else{goto _LL5;}case 14U: if(((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)((struct Cyc_Absyn_Exp*)((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp4F)->f2)->r)->tag == 0U){if(((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)((struct Cyc_Absyn_Exp*)((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp4F)->f2)->r)->f1).String_c).tag == 8){_LL3: _tmp50=((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)(((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp4F)->f2)->r)->f1).String_c).val;_LL4: {struct _fat_ptr s=_tmp50;
# 349
desc_types=type_getter(te,(struct _fat_ptr)s,isCproto,fmt->loc);goto _LL0;}}else{goto _LL5;}}else{goto _LL5;}default: _LL5: _LL6:
# 353
 if(opt_args != 0){
struct Cyc_List_List*args=(struct Cyc_List_List*)opt_args->v;
for(0;args != 0;args=args->tl){
Cyc_Tcexp_tcExp(te,0,(struct Cyc_Absyn_Exp*)args->hd);
if(Cyc_Tcutil_is_noalias_pointer_or_aggr((void*)_check_null(((struct Cyc_Absyn_Exp*)args->hd)->topt))&& !
Cyc_Tcutil_is_noalias_path((struct Cyc_Absyn_Exp*)args->hd))
({void*_tmp52=0U;({unsigned _tmp4E6=((struct Cyc_Absyn_Exp*)args->hd)->loc;struct _fat_ptr _tmp4E5=({const char*_tmp53="Cannot consume non-unique paths; do swap instead";_tag_fat(_tmp53,sizeof(char),49U);});Cyc_Tcutil_terr(_tmp4E6,_tmp4E5,_tag_fat(_tmp52,sizeof(void*),0U));});});}}
# 353
return;}_LL0:;}
# 364
if(opt_args != 0){
struct Cyc_List_List*args=(struct Cyc_List_List*)opt_args->v;
# 367
for(0;desc_types != 0 && args != 0;(
desc_types=desc_types->tl,args=args->tl,arg_cnt ++)){
int alias_coercion=0;
void*t=(void*)desc_types->hd;
struct Cyc_Absyn_Exp*e=(struct Cyc_Absyn_Exp*)args->hd;
Cyc_Tcexp_tcExp(te,& t,e);
if(!Cyc_Tcutil_coerce_arg(te,e,t,& alias_coercion)){
({struct Cyc_String_pa_PrintArg_struct _tmp56=({struct Cyc_String_pa_PrintArg_struct _tmp436;_tmp436.tag=0U,({
struct _fat_ptr _tmp4E7=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t));_tmp436.f1=_tmp4E7;});_tmp436;});struct Cyc_String_pa_PrintArg_struct _tmp57=({struct Cyc_String_pa_PrintArg_struct _tmp435;_tmp435.tag=0U,({struct _fat_ptr _tmp4E8=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string((void*)_check_null(e->topt)));_tmp435.f1=_tmp4E8;});_tmp435;});void*_tmp54[2U];_tmp54[0]=& _tmp56,_tmp54[1]=& _tmp57;({unsigned _tmp4EA=e->loc;struct _fat_ptr _tmp4E9=({const char*_tmp55="descriptor has type %s but argument has type %s";_tag_fat(_tmp55,sizeof(char),48U);});Cyc_Tcutil_terr(_tmp4EA,_tmp4E9,_tag_fat(_tmp54,sizeof(void*),2U));});});
Cyc_Unify_explain_failure();}
# 373
if(alias_coercion)
# 379
({struct Cyc_List_List*_tmp4EB=({struct Cyc_List_List*_tmp58=_cycalloc(sizeof(*_tmp58));_tmp58->hd=(void*)arg_cnt,_tmp58->tl=*alias_arg_exps;_tmp58;});*alias_arg_exps=_tmp4EB;});
# 373
if(
# 380
Cyc_Tcutil_is_noalias_pointer_or_aggr(t)&& !
Cyc_Tcutil_is_noalias_path(e))
({void*_tmp59=0U;({unsigned _tmp4ED=((struct Cyc_Absyn_Exp*)args->hd)->loc;struct _fat_ptr _tmp4EC=({const char*_tmp5A="Cannot consume non-unique paths; do swap instead";_tag_fat(_tmp5A,sizeof(char),49U);});Cyc_Tcutil_terr(_tmp4ED,_tmp4EC,_tag_fat(_tmp59,sizeof(void*),0U));});});}
# 385
if(desc_types != 0)
({void*_tmp5B=0U;({unsigned _tmp4EF=fmt->loc;struct _fat_ptr _tmp4EE=({const char*_tmp5C="too few arguments";_tag_fat(_tmp5C,sizeof(char),18U);});Cyc_Tcutil_terr(_tmp4EF,_tmp4EE,_tag_fat(_tmp5B,sizeof(void*),0U));});});
# 385
if(args != 0){
# 388
({void*_tmp5D=0U;({unsigned _tmp4F1=((struct Cyc_Absyn_Exp*)args->hd)->loc;struct _fat_ptr _tmp4F0=({const char*_tmp5E="too many arguments";_tag_fat(_tmp5E,sizeof(char),19U);});Cyc_Tcutil_terr(_tmp4F1,_tmp4F0,_tag_fat(_tmp5D,sizeof(void*),0U));});});
# 390
for(0;args != 0;args=args->tl){
Cyc_Tcexp_tcExp(te,0,(struct Cyc_Absyn_Exp*)args->hd);}}}}
# 306
static void*Cyc_Tcexp_tcUnPrimop(struct Cyc_Tcenv_Tenv*te,unsigned loc,void**topt,enum Cyc_Absyn_Primop p,struct Cyc_Absyn_Exp*e){
# 397
void*t=Cyc_Tcutil_compress((void*)_check_null(e->topt));
enum Cyc_Absyn_Primop _tmp5F=p;switch(_tmp5F){case Cyc_Absyn_Plus: _LL1: _LL2:
 goto _LL4;case Cyc_Absyn_Minus: _LL3: _LL4:
# 401
 if(!Cyc_Tcutil_is_numeric(e))
({struct Cyc_String_pa_PrintArg_struct _tmp62=({struct Cyc_String_pa_PrintArg_struct _tmp437;_tmp437.tag=0U,({struct _fat_ptr _tmp4F2=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t));_tmp437.f1=_tmp4F2;});_tmp437;});void*_tmp60[1U];_tmp60[0]=& _tmp62;({unsigned _tmp4F4=loc;struct _fat_ptr _tmp4F3=({const char*_tmp61="expecting numeric type but found %s";_tag_fat(_tmp61,sizeof(char),36U);});Cyc_Tcutil_terr(_tmp4F4,_tmp4F3,_tag_fat(_tmp60,sizeof(void*),1U));});});
# 401
return(void*)_check_null(e->topt);case Cyc_Absyn_Not: _LL5: _LL6:
# 405
 Cyc_Tcexp_check_contains_assign(e);
if(!Cyc_Tcutil_coerce_to_bool(e))
({struct Cyc_String_pa_PrintArg_struct _tmp65=({struct Cyc_String_pa_PrintArg_struct _tmp438;_tmp438.tag=0U,({struct _fat_ptr _tmp4F5=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t));_tmp438.f1=_tmp4F5;});_tmp438;});void*_tmp63[1U];_tmp63[0]=& _tmp65;({unsigned _tmp4F7=loc;struct _fat_ptr _tmp4F6=({const char*_tmp64="expecting integral or * type but found %s";_tag_fat(_tmp64,sizeof(char),42U);});Cyc_Tcutil_terr(_tmp4F7,_tmp4F6,_tag_fat(_tmp63,sizeof(void*),1U));});});
# 406
return Cyc_Absyn_sint_type;case Cyc_Absyn_Bitnot: _LL7: _LL8:
# 410
 if(!Cyc_Tcutil_is_integral(e))
({struct Cyc_String_pa_PrintArg_struct _tmp68=({struct Cyc_String_pa_PrintArg_struct _tmp439;_tmp439.tag=0U,({struct _fat_ptr _tmp4F8=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t));_tmp439.f1=_tmp4F8;});_tmp439;});void*_tmp66[1U];_tmp66[0]=& _tmp68;({unsigned _tmp4FA=loc;struct _fat_ptr _tmp4F9=({const char*_tmp67="expecting integral type but found %s";_tag_fat(_tmp67,sizeof(char),37U);});Cyc_Tcutil_terr(_tmp4FA,_tmp4F9,_tag_fat(_tmp66,sizeof(void*),1U));});});
# 410
return(void*)_check_null(e->topt);case Cyc_Absyn_Numelts: _LL9: _LLA:
# 414
{void*_tmp69=t;void*_tmp6A;if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp69)->tag == 3U){_LLE: _tmp6A=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp69)->f1).ptr_atts).bounds;_LLF: {void*b=_tmp6A;
# 416
struct Cyc_Absyn_Exp*eopt=Cyc_Tcutil_get_bounds_exp(Cyc_Absyn_fat_bound_type,b);
if(eopt != 0){
struct Cyc_Absyn_Exp*e=eopt;
if(!Cyc_Evexp_c_can_eval(e)&& !((unsigned)Cyc_Tcenv_allow_valueof))
({void*_tmp6B=0U;({unsigned _tmp4FC=loc;struct _fat_ptr _tmp4FB=({const char*_tmp6C="cannot apply numelts to a pointer with abstract bounds";_tag_fat(_tmp6C,sizeof(char),55U);});Cyc_Tcutil_terr(_tmp4FC,_tmp4FB,_tag_fat(_tmp6B,sizeof(void*),0U));});});}
# 417
goto _LLD;}}else{_LL10: _LL11:
# 424
({struct Cyc_String_pa_PrintArg_struct _tmp6F=({struct Cyc_String_pa_PrintArg_struct _tmp43A;_tmp43A.tag=0U,({struct _fat_ptr _tmp4FD=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t));_tmp43A.f1=_tmp4FD;});_tmp43A;});void*_tmp6D[1U];_tmp6D[0]=& _tmp6F;({unsigned _tmp4FF=loc;struct _fat_ptr _tmp4FE=({const char*_tmp6E="numelts requires pointer type, not %s";_tag_fat(_tmp6E,sizeof(char),38U);});Cyc_Tcutil_terr(_tmp4FF,_tmp4FE,_tag_fat(_tmp6D,sizeof(void*),1U));});});}_LLD:;}
# 426
return Cyc_Absyn_uint_type;default: _LLB: _LLC:
({void*_tmp70=0U;({int(*_tmp501)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp72)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp72;});struct _fat_ptr _tmp500=({const char*_tmp71="Non-unary primop";_tag_fat(_tmp71,sizeof(char),17U);});_tmp501(_tmp500,_tag_fat(_tmp70,sizeof(void*),0U));});});}_LL0:;}
# 306
static void*Cyc_Tcexp_arith_convert(struct Cyc_Tcenv_Tenv*te,struct Cyc_Absyn_Exp*e1,struct Cyc_Absyn_Exp*e2){
# 434
void*t1=(void*)_check_null(e1->topt);
void*t2=(void*)_check_null(e2->topt);
void*new_t=Cyc_Tcutil_max_arithmetic_type(t1,t2);
if(!Cyc_Unify_unify(t1,new_t))Cyc_Tcutil_unchecked_cast(e1,new_t,Cyc_Absyn_No_coercion);if(!
Cyc_Unify_unify(t2,new_t))Cyc_Tcutil_unchecked_cast(e2,new_t,Cyc_Absyn_No_coercion);
# 437
return new_t;}
# 306
static void*Cyc_Tcexp_tcArithBinop(struct Cyc_Tcenv_Tenv*te,struct Cyc_Absyn_Exp*e1,struct Cyc_Absyn_Exp*e2,int(*checker)(struct Cyc_Absyn_Exp*)){
# 446
if(!checker(e1)){
({struct Cyc_String_pa_PrintArg_struct _tmp75=({struct Cyc_String_pa_PrintArg_struct _tmp43B;_tmp43B.tag=0U,({struct _fat_ptr _tmp502=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string((void*)_check_null(e1->topt)));_tmp43B.f1=_tmp502;});_tmp43B;});void*_tmp73[1U];_tmp73[0]=& _tmp75;({unsigned _tmp504=e1->loc;struct _fat_ptr _tmp503=({const char*_tmp74="type %s cannot be used here";_tag_fat(_tmp74,sizeof(char),28U);});Cyc_Tcutil_terr(_tmp504,_tmp503,_tag_fat(_tmp73,sizeof(void*),1U));});});
return Cyc_Absyn_wildtyp(Cyc_Tcenv_lookup_opt_type_vars(te));}
# 446
if(!
# 450
checker(e2)){
({struct Cyc_String_pa_PrintArg_struct _tmp78=({struct Cyc_String_pa_PrintArg_struct _tmp43C;_tmp43C.tag=0U,({struct _fat_ptr _tmp505=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string((void*)_check_null(e2->topt)));_tmp43C.f1=_tmp505;});_tmp43C;});void*_tmp76[1U];_tmp76[0]=& _tmp78;({unsigned _tmp507=e2->loc;struct _fat_ptr _tmp506=({const char*_tmp77="type %s cannot be used here";_tag_fat(_tmp77,sizeof(char),28U);});Cyc_Tcutil_terr(_tmp507,_tmp506,_tag_fat(_tmp76,sizeof(void*),1U));});});
return Cyc_Absyn_wildtyp(Cyc_Tcenv_lookup_opt_type_vars(te));}
# 446
return
# 454
 Cyc_Tcexp_arith_convert(te,e1,e2);}
# 306
static void*Cyc_Tcexp_tcPlus(struct Cyc_Tcenv_Tenv*te,struct Cyc_Absyn_Exp*e1,struct Cyc_Absyn_Exp*e2){
# 458
void*t1=Cyc_Tcutil_compress((void*)_check_null(e1->topt));
void*t2=Cyc_Tcutil_compress((void*)_check_null(e2->topt));
void*_tmp79=t1;void*_tmp7F;void*_tmp7E;void*_tmp7D;void*_tmp7C;struct Cyc_Absyn_Tqual _tmp7B;void*_tmp7A;if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp79)->tag == 3U){_LL1: _tmp7A=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp79)->f1).elt_type;_tmp7B=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp79)->f1).elt_tq;_tmp7C=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp79)->f1).ptr_atts).rgn;_tmp7D=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp79)->f1).ptr_atts).nullable;_tmp7E=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp79)->f1).ptr_atts).bounds;_tmp7F=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp79)->f1).ptr_atts).zero_term;_LL2: {void*et=_tmp7A;struct Cyc_Absyn_Tqual tq=_tmp7B;void*r=_tmp7C;void*n=_tmp7D;void*b=_tmp7E;void*zt=_tmp7F;
# 462
if(!Cyc_Tcutil_kind_leq(Cyc_Tcutil_type_kind(et),& Cyc_Tcutil_tmk))
({void*_tmp80=0U;({unsigned _tmp509=e1->loc;struct _fat_ptr _tmp508=({const char*_tmp81="can't perform arithmetic on abstract pointer type";_tag_fat(_tmp81,sizeof(char),50U);});Cyc_Tcutil_terr(_tmp509,_tmp508,_tag_fat(_tmp80,sizeof(void*),0U));});});
# 462
if(
# 464
Cyc_Tcutil_is_noalias_pointer(t1,0))
({void*_tmp82=0U;({unsigned _tmp50B=e1->loc;struct _fat_ptr _tmp50A=({const char*_tmp83="can't perform arithmetic on non-aliasing pointer type";_tag_fat(_tmp83,sizeof(char),54U);});Cyc_Tcutil_terr(_tmp50B,_tmp50A,_tag_fat(_tmp82,sizeof(void*),0U));});});
# 462
if(!
# 466
Cyc_Tcutil_coerce_sint_type(e2))
({struct Cyc_String_pa_PrintArg_struct _tmp86=({struct Cyc_String_pa_PrintArg_struct _tmp43D;_tmp43D.tag=0U,({struct _fat_ptr _tmp50C=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t2));_tmp43D.f1=_tmp50C;});_tmp43D;});void*_tmp84[1U];_tmp84[0]=& _tmp86;({unsigned _tmp50E=e2->loc;struct _fat_ptr _tmp50D=({const char*_tmp85="expecting int but found %s";_tag_fat(_tmp85,sizeof(char),27U);});Cyc_Tcutil_terr(_tmp50E,_tmp50D,_tag_fat(_tmp84,sizeof(void*),1U));});});{
# 462
struct Cyc_Absyn_Exp*eopt=
# 468
Cyc_Tcutil_get_bounds_exp(Cyc_Absyn_fat_bound_type,b);
if(eopt == 0)return t1;{struct Cyc_Absyn_Exp*ub=eopt;
# 473
if(Cyc_Tcutil_force_type2bool(0,zt)){
struct _tuple16 _stmttmpA=Cyc_Evexp_eval_const_uint_exp(ub);int _tmp88;unsigned _tmp87;_LL6: _tmp87=_stmttmpA.f1;_tmp88=_stmttmpA.f2;_LL7: {unsigned i=_tmp87;int known=_tmp88;
if(known && i == (unsigned)1)
({void*_tmp89=0U;({unsigned _tmp510=e1->loc;struct _fat_ptr _tmp50F=({const char*_tmp8A="pointer arithmetic on thin, zero-terminated pointer may be expensive.";_tag_fat(_tmp8A,sizeof(char),70U);});Cyc_Tcutil_warn(_tmp510,_tmp50F,_tag_fat(_tmp89,sizeof(void*),0U));});});}}{
# 473
struct Cyc_Absyn_PointerType_Absyn_Type_struct*new_t1=({struct Cyc_Absyn_PointerType_Absyn_Type_struct*_tmp8B=_cycalloc(sizeof(*_tmp8B));
# 484
_tmp8B->tag=3U,(_tmp8B->f1).elt_type=et,(_tmp8B->f1).elt_tq=tq,
((_tmp8B->f1).ptr_atts).rgn=r,((_tmp8B->f1).ptr_atts).nullable=Cyc_Absyn_true_type,((_tmp8B->f1).ptr_atts).bounds=Cyc_Absyn_fat_bound_type,((_tmp8B->f1).ptr_atts).zero_term=zt,((_tmp8B->f1).ptr_atts).ptrloc=0;_tmp8B;});
# 488
Cyc_Tcutil_unchecked_cast(e1,(void*)new_t1,Cyc_Absyn_Other_coercion);
return(void*)new_t1;}}}}}else{_LL3: _LL4:
 return Cyc_Tcexp_tcArithBinop(te,e1,e2,Cyc_Tcutil_is_numeric);}_LL0:;}
# 306
static void*Cyc_Tcexp_tcMinus(struct Cyc_Tcenv_Tenv*te,struct Cyc_Absyn_Exp*e1,struct Cyc_Absyn_Exp*e2){
# 496
void*t1=(void*)_check_null(e1->topt);
void*t2=(void*)_check_null(e2->topt);
void*t1_elt=Cyc_Absyn_void_type;
void*t2_elt=Cyc_Absyn_void_type;
if(Cyc_Tcutil_is_fat_pointer_type_elt(t1,& t1_elt)){
if(Cyc_Tcutil_is_fat_pointer_type_elt(t2,& t2_elt)){
if(!Cyc_Unify_unify(t1_elt,t2_elt)){
({struct Cyc_String_pa_PrintArg_struct _tmp8E=({struct Cyc_String_pa_PrintArg_struct _tmp43F;_tmp43F.tag=0U,({
# 505
struct _fat_ptr _tmp511=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t1));_tmp43F.f1=_tmp511;});_tmp43F;});struct Cyc_String_pa_PrintArg_struct _tmp8F=({struct Cyc_String_pa_PrintArg_struct _tmp43E;_tmp43E.tag=0U,({struct _fat_ptr _tmp512=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t2));_tmp43E.f1=_tmp512;});_tmp43E;});void*_tmp8C[2U];_tmp8C[0]=& _tmp8E,_tmp8C[1]=& _tmp8F;({unsigned _tmp514=e1->loc;struct _fat_ptr _tmp513=({const char*_tmp8D="pointer arithmetic on values of different types (%s != %s)";_tag_fat(_tmp8D,sizeof(char),59U);});Cyc_Tcutil_terr(_tmp514,_tmp513,_tag_fat(_tmp8C,sizeof(void*),2U));});});
Cyc_Unify_explain_failure();}
# 502
return Cyc_Absyn_sint_type;}else{
# 509
if(Cyc_Tcutil_is_pointer_type(t2)){
if(!({void*_tmp515=t1_elt;Cyc_Unify_unify(_tmp515,Cyc_Tcutil_pointer_elt_type(t2));})){
({struct Cyc_String_pa_PrintArg_struct _tmp92=({struct Cyc_String_pa_PrintArg_struct _tmp441;_tmp441.tag=0U,({
struct _fat_ptr _tmp516=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t1));_tmp441.f1=_tmp516;});_tmp441;});struct Cyc_String_pa_PrintArg_struct _tmp93=({struct Cyc_String_pa_PrintArg_struct _tmp440;_tmp440.tag=0U,({struct _fat_ptr _tmp517=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t2));_tmp440.f1=_tmp517;});_tmp440;});void*_tmp90[2U];_tmp90[0]=& _tmp92,_tmp90[1]=& _tmp93;({unsigned _tmp519=e1->loc;struct _fat_ptr _tmp518=({const char*_tmp91="pointer arithmetic on values of different types (%s != %s)";_tag_fat(_tmp91,sizeof(char),59U);});Cyc_Tcutil_terr(_tmp519,_tmp518,_tag_fat(_tmp90,sizeof(void*),2U));});});
Cyc_Unify_explain_failure();}
# 510
({void*_tmp94=0U;({unsigned _tmp51B=e1->loc;struct _fat_ptr _tmp51A=({const char*_tmp95="coercing fat pointer to thin pointer to support subtraction";_tag_fat(_tmp95,sizeof(char),60U);});Cyc_Tcutil_warn(_tmp51B,_tmp51A,_tag_fat(_tmp94,sizeof(void*),0U));});});
# 517
({struct Cyc_Absyn_Exp*_tmp51F=e1;Cyc_Tcutil_unchecked_cast(_tmp51F,({void*_tmp51E=t1_elt;void*_tmp51D=Cyc_Tcutil_pointer_region(t1);struct Cyc_Absyn_Tqual _tmp51C=
Cyc_Absyn_empty_tqual(0U);
# 517
Cyc_Absyn_star_type(_tmp51E,_tmp51D,_tmp51C,Cyc_Absyn_false_type);}),Cyc_Absyn_Other_coercion);});
# 520
return Cyc_Absyn_sint_type;}else{
# 522
if(!Cyc_Tcutil_kind_leq(Cyc_Tcutil_type_kind(t1_elt),& Cyc_Tcutil_tmk))
({void*_tmp96=0U;({unsigned _tmp521=e1->loc;struct _fat_ptr _tmp520=({const char*_tmp97="can't perform arithmetic on abstract pointer type";_tag_fat(_tmp97,sizeof(char),50U);});Cyc_Tcutil_terr(_tmp521,_tmp520,_tag_fat(_tmp96,sizeof(void*),0U));});});
# 522
if(
# 524
Cyc_Tcutil_is_noalias_pointer(t1,0))
({void*_tmp98=0U;({unsigned _tmp523=e1->loc;struct _fat_ptr _tmp522=({const char*_tmp99="can't perform arithmetic on non-aliasing pointer type";_tag_fat(_tmp99,sizeof(char),54U);});Cyc_Tcutil_terr(_tmp523,_tmp522,_tag_fat(_tmp98,sizeof(void*),0U));});});
# 522
if(!
# 526
Cyc_Tcutil_coerce_sint_type(e2)){
({struct Cyc_String_pa_PrintArg_struct _tmp9C=({struct Cyc_String_pa_PrintArg_struct _tmp443;_tmp443.tag=0U,({
struct _fat_ptr _tmp524=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t1));_tmp443.f1=_tmp524;});_tmp443;});struct Cyc_String_pa_PrintArg_struct _tmp9D=({struct Cyc_String_pa_PrintArg_struct _tmp442;_tmp442.tag=0U,({struct _fat_ptr _tmp525=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t2));_tmp442.f1=_tmp525;});_tmp442;});void*_tmp9A[2U];_tmp9A[0]=& _tmp9C,_tmp9A[1]=& _tmp9D;({unsigned _tmp527=e2->loc;struct _fat_ptr _tmp526=({const char*_tmp9B="expecting either %s or int but found %s";_tag_fat(_tmp9B,sizeof(char),40U);});Cyc_Tcutil_terr(_tmp527,_tmp526,_tag_fat(_tmp9A,sizeof(void*),2U));});});
Cyc_Unify_explain_failure();}
# 522
return t1;}}}
# 500
if(
# 535
Cyc_Tcutil_is_pointer_type(t1)){
if(Cyc_Tcutil_is_pointer_type(t2)&&({void*_tmp528=Cyc_Tcutil_pointer_elt_type(t1);Cyc_Unify_unify(_tmp528,
Cyc_Tcutil_pointer_elt_type(t2));})){
if(Cyc_Tcutil_is_fat_pointer_type(t2)){
({void*_tmp9E=0U;({unsigned _tmp52A=e1->loc;struct _fat_ptr _tmp529=({const char*_tmp9F="coercing fat pointer to thin pointer to support subtraction";_tag_fat(_tmp9F,sizeof(char),60U);});Cyc_Tcutil_warn(_tmp52A,_tmp529,_tag_fat(_tmp9E,sizeof(void*),0U));});});
({struct Cyc_Absyn_Exp*_tmp52E=e2;Cyc_Tcutil_unchecked_cast(_tmp52E,({void*_tmp52D=Cyc_Tcutil_pointer_elt_type(t2);void*_tmp52C=
Cyc_Tcutil_pointer_region(t2);
# 540
struct Cyc_Absyn_Tqual _tmp52B=
# 542
Cyc_Absyn_empty_tqual(0U);
# 540
Cyc_Absyn_star_type(_tmp52D,_tmp52C,_tmp52B,Cyc_Absyn_false_type);}),Cyc_Absyn_Other_coercion);});}
# 538
({void*_tmpA0=0U;({unsigned _tmp530=e1->loc;struct _fat_ptr _tmp52F=({const char*_tmpA1="thin pointer subtraction!";_tag_fat(_tmpA1,sizeof(char),26U);});Cyc_Tcutil_warn(_tmp530,_tmp52F,_tag_fat(_tmpA0,sizeof(void*),0U));});});
# 546
return Cyc_Absyn_sint_type;}
# 536
({void*_tmpA2=0U;({unsigned _tmp532=e1->loc;struct _fat_ptr _tmp531=({const char*_tmpA3="coercing thin pointer to integer to support subtraction";_tag_fat(_tmpA3,sizeof(char),56U);});Cyc_Tcutil_warn(_tmp532,_tmp531,_tag_fat(_tmpA2,sizeof(void*),0U));});});
# 549
Cyc_Tcutil_unchecked_cast(e1,Cyc_Absyn_sint_type,Cyc_Absyn_Other_coercion);}
# 500
if(
# 551
Cyc_Tcutil_is_pointer_type(t2)){
({void*_tmpA4=0U;({unsigned _tmp534=e1->loc;struct _fat_ptr _tmp533=({const char*_tmpA5="coercing pointer to integer to support subtraction";_tag_fat(_tmpA5,sizeof(char),51U);});Cyc_Tcutil_warn(_tmp534,_tmp533,_tag_fat(_tmpA4,sizeof(void*),0U));});});
Cyc_Tcutil_unchecked_cast(e2,Cyc_Absyn_sint_type,Cyc_Absyn_Other_coercion);}
# 500
return
# 556
 Cyc_Tcexp_tcArithBinop(te,e1,e2,Cyc_Tcutil_is_numeric);}
# 306
static void*Cyc_Tcexp_tcAnyBinop(struct Cyc_Tcenv_Tenv*te,unsigned loc,struct Cyc_Absyn_Exp*e1,struct Cyc_Absyn_Exp*e2){
# 560
void*t1=Cyc_Tcutil_compress((void*)_check_null(e1->topt));
void*t2=Cyc_Tcutil_compress((void*)_check_null(e2->topt));
if(Cyc_Tcutil_is_numeric(e1)&& Cyc_Tcutil_is_numeric(e2)){
Cyc_Tcexp_arith_convert(te,e1,e2);
return Cyc_Absyn_sint_type;}
# 562
if(
# 567
(int)(Cyc_Tcutil_type_kind(t1))->kind == (int)Cyc_Absyn_BoxKind ||({
void*_tmp535=t1;Cyc_Unify_unify(_tmp535,Cyc_Absyn_new_evar(& Cyc_Tcutil_bko,Cyc_Tcenv_lookup_opt_type_vars(te)));})){
if(Cyc_Unify_unify(t1,t2))
return Cyc_Absyn_sint_type;
# 569
if(
# 572
Cyc_Tcutil_silent_castable(te,loc,t2,t1)){
Cyc_Tcutil_unchecked_cast(e2,t1,Cyc_Absyn_Other_coercion);
return Cyc_Absyn_sint_type;}
# 569
if(
# 576
Cyc_Tcutil_silent_castable(te,loc,t1,t2)){
Cyc_Tcutil_unchecked_cast(e1,t2,Cyc_Absyn_Other_coercion);
return Cyc_Absyn_sint_type;}
# 569
if(
# 580
Cyc_Tcutil_zero_to_null(t2,e1)|| Cyc_Tcutil_zero_to_null(t1,e2))
return Cyc_Absyn_sint_type;}
# 562
{struct _tuple0 _stmttmpB=({struct _tuple0 _tmp444;({
# 585
void*_tmp537=Cyc_Tcutil_compress(t1);_tmp444.f1=_tmp537;}),({void*_tmp536=Cyc_Tcutil_compress(t2);_tmp444.f2=_tmp536;});_tmp444;});
# 562
struct _tuple0 _tmpA6=_stmttmpB;void*_tmpA8;void*_tmpA7;switch(*((int*)_tmpA6.f1)){case 3U: if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmpA6.f2)->tag == 3U){_LL1: _tmpA7=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmpA6.f1)->f1).elt_type;_tmpA8=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmpA6.f2)->f1).elt_type;_LL2: {void*t1a=_tmpA7;void*t2a=_tmpA8;
# 587
if(Cyc_Unify_unify(t1a,t2a))
return Cyc_Absyn_sint_type;
# 587
goto _LL0;}}else{goto _LL5;}case 0U: if(((struct Cyc_Absyn_RgnHandleCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmpA6.f1)->f1)->tag == 3U){if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmpA6.f2)->tag == 0U){if(((struct Cyc_Absyn_RgnHandleCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmpA6.f2)->f1)->tag == 3U){_LL3: _LL4:
# 591
 return Cyc_Absyn_sint_type;}else{goto _LL5;}}else{goto _LL5;}}else{goto _LL5;}default: _LL5: _LL6:
 goto _LL0;}_LL0:;}
# 594
({struct Cyc_String_pa_PrintArg_struct _tmpAB=({struct Cyc_String_pa_PrintArg_struct _tmp446;_tmp446.tag=0U,({
struct _fat_ptr _tmp538=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t1));_tmp446.f1=_tmp538;});_tmp446;});struct Cyc_String_pa_PrintArg_struct _tmpAC=({struct Cyc_String_pa_PrintArg_struct _tmp445;_tmp445.tag=0U,({struct _fat_ptr _tmp539=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t2));_tmp445.f1=_tmp539;});_tmp445;});void*_tmpA9[2U];_tmpA9[0]=& _tmpAB,_tmpA9[1]=& _tmpAC;({unsigned _tmp53B=loc;struct _fat_ptr _tmp53A=({const char*_tmpAA="comparison not allowed between %s and %s";_tag_fat(_tmpAA,sizeof(char),41U);});Cyc_Tcutil_terr(_tmp53B,_tmp53A,_tag_fat(_tmpA9,sizeof(void*),2U));});});
Cyc_Unify_explain_failure();
return Cyc_Absyn_wildtyp(Cyc_Tcenv_lookup_opt_type_vars(te));}
# 306
static void*Cyc_Tcexp_tcEqPrimop(struct Cyc_Tcenv_Tenv*te,unsigned loc,struct Cyc_Absyn_Exp*e1,struct Cyc_Absyn_Exp*e2){
# 602
if(({void*_tmp53C=Cyc_Tcexp_tcAnyBinop(te,loc,e1,e2);_tmp53C == Cyc_Absyn_sint_type;}))
return Cyc_Absyn_sint_type;
# 602
({struct Cyc_String_pa_PrintArg_struct _tmpAF=({struct Cyc_String_pa_PrintArg_struct _tmp448;_tmp448.tag=0U,({
# 605
struct _fat_ptr _tmp53D=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string((void*)_check_null(e1->topt)));_tmp448.f1=_tmp53D;});_tmp448;});struct Cyc_String_pa_PrintArg_struct _tmpB0=({struct Cyc_String_pa_PrintArg_struct _tmp447;_tmp447.tag=0U,({struct _fat_ptr _tmp53E=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string((void*)_check_null(e2->topt)));_tmp447.f1=_tmp53E;});_tmp447;});void*_tmpAD[2U];_tmpAD[0]=& _tmpAF,_tmpAD[1]=& _tmpB0;({unsigned _tmp540=loc;struct _fat_ptr _tmp53F=({const char*_tmpAE="comparison not allowed between %s and %s";_tag_fat(_tmpAE,sizeof(char),41U);});Cyc_Tcutil_terr(_tmp540,_tmp53F,_tag_fat(_tmpAD,sizeof(void*),2U));});});
Cyc_Unify_explain_failure();
return Cyc_Absyn_wildtyp(Cyc_Tcenv_lookup_opt_type_vars(te));}
# 306
static void*Cyc_Tcexp_tcBinPrimop(struct Cyc_Tcenv_Tenv*te,unsigned loc,void**topt,enum Cyc_Absyn_Primop p,struct Cyc_Absyn_Exp*e1,struct Cyc_Absyn_Exp*e2){
# 614
enum Cyc_Absyn_Primop _tmpB1=p;switch(_tmpB1){case Cyc_Absyn_Plus: _LL1: _LL2:
 return Cyc_Tcexp_tcPlus(te,e1,e2);case Cyc_Absyn_Minus: _LL3: _LL4:
 return Cyc_Tcexp_tcMinus(te,e1,e2);case Cyc_Absyn_Times: _LL5: _LL6:
# 618
 goto _LL8;case Cyc_Absyn_Div: _LL7: _LL8:
 return Cyc_Tcexp_tcArithBinop(te,e1,e2,Cyc_Tcutil_is_numeric);case Cyc_Absyn_Mod: _LL9: _LLA:
# 621
 goto _LLC;case Cyc_Absyn_Bitand: _LLB: _LLC:
 goto _LLE;case Cyc_Absyn_Bitor: _LLD: _LLE:
 goto _LL10;case Cyc_Absyn_Bitxor: _LLF: _LL10:
 goto _LL12;case Cyc_Absyn_Bitlshift: _LL11: _LL12:
 goto _LL14;case Cyc_Absyn_Bitlrshift: _LL13: _LL14:
 return Cyc_Tcexp_tcArithBinop(te,e1,e2,Cyc_Tcutil_is_integral);case Cyc_Absyn_Eq: _LL15: _LL16:
# 630
 goto _LL18;case Cyc_Absyn_Neq: _LL17: _LL18:
 return Cyc_Tcexp_tcEqPrimop(te,loc,e1,e2);case Cyc_Absyn_Gt: _LL19: _LL1A:
# 633
 goto _LL1C;case Cyc_Absyn_Lt: _LL1B: _LL1C:
 goto _LL1E;case Cyc_Absyn_Gte: _LL1D: _LL1E:
 goto _LL20;case Cyc_Absyn_Lte: _LL1F: _LL20:
 return Cyc_Tcexp_tcAnyBinop(te,loc,e1,e2);default: _LL21: _LL22:
# 638
({void*_tmpB2=0U;({int(*_tmp542)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmpB4)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmpB4;});struct _fat_ptr _tmp541=({const char*_tmpB3="bad binary primop";_tag_fat(_tmpB3,sizeof(char),18U);});_tmp542(_tmp541,_tag_fat(_tmpB2,sizeof(void*),0U));});});}_LL0:;}
# 306
static void*Cyc_Tcexp_tcPrimop(struct Cyc_Tcenv_Tenv*te,unsigned loc,void**topt,enum Cyc_Absyn_Primop p,struct Cyc_List_List*es){
# 650
if((int)p == (int)2U && Cyc_List_length(es)== 1){
struct Cyc_Absyn_Exp*e=(struct Cyc_Absyn_Exp*)((struct Cyc_List_List*)_check_null(es))->hd;
void*t=Cyc_Tcexp_tcExp(te,topt,e);
if(!Cyc_Tcutil_is_numeric(e))
({struct Cyc_String_pa_PrintArg_struct _tmpB7=({struct Cyc_String_pa_PrintArg_struct _tmp449;_tmp449.tag=0U,({struct _fat_ptr _tmp543=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t));_tmp449.f1=_tmp543;});_tmp449;});void*_tmpB5[1U];_tmpB5[0]=& _tmpB7;({unsigned _tmp545=e->loc;struct _fat_ptr _tmp544=({const char*_tmpB6="expecting numeric type but found %s";_tag_fat(_tmpB6,sizeof(char),36U);});Cyc_Tcutil_terr(_tmp545,_tmp544,_tag_fat(_tmpB5,sizeof(void*),1U));});});
# 653
return t;}
# 650
Cyc_Tcexp_tcExpList(te,es);{
# 658
void*t;
{int _stmttmpC=Cyc_List_length(es);int _tmpB8=_stmttmpC;switch(_tmpB8){case 0U: _LL1: _LL2:
 return({void*_tmpB9=0U;({struct Cyc_Tcenv_Tenv*_tmp549=te;unsigned _tmp548=loc;void**_tmp547=topt;struct _fat_ptr _tmp546=({const char*_tmpBA="primitive operator has 0 arguments";_tag_fat(_tmpBA,sizeof(char),35U);});Cyc_Tcexp_expr_err(_tmp549,_tmp548,_tmp547,_tmp546,_tag_fat(_tmpB9,sizeof(void*),0U));});});case 1U: _LL3: _LL4:
 t=Cyc_Tcexp_tcUnPrimop(te,loc,topt,p,(struct Cyc_Absyn_Exp*)((struct Cyc_List_List*)_check_null(es))->hd);goto _LL0;case 2U: _LL5: _LL6:
 t=({struct Cyc_Tcenv_Tenv*_tmp54E=te;unsigned _tmp54D=loc;void**_tmp54C=topt;enum Cyc_Absyn_Primop _tmp54B=p;struct Cyc_Absyn_Exp*_tmp54A=(struct Cyc_Absyn_Exp*)((struct Cyc_List_List*)_check_null(es))->hd;Cyc_Tcexp_tcBinPrimop(_tmp54E,_tmp54D,_tmp54C,_tmp54B,_tmp54A,(struct Cyc_Absyn_Exp*)((struct Cyc_List_List*)_check_null(es->tl))->hd);});goto _LL0;default: _LL7: _LL8:
 return({void*_tmpBB=0U;({struct Cyc_Tcenv_Tenv*_tmp552=te;unsigned _tmp551=loc;void**_tmp550=topt;struct _fat_ptr _tmp54F=({const char*_tmpBC="primitive operator has > 2 arguments";_tag_fat(_tmpBC,sizeof(char),37U);});Cyc_Tcexp_expr_err(_tmp552,_tmp551,_tmp550,_tmp54F,_tag_fat(_tmpBB,sizeof(void*),0U));});});}_LL0:;}
# 665
return t;}}struct _tuple17{struct Cyc_Absyn_Tqual f1;void*f2;};
# 306
static int Cyc_Tcexp_check_writable_aggr(unsigned loc,void*t0){
# 669
void*t=Cyc_Tcutil_compress(t0);
void*_tmpBD=t;struct Cyc_List_List*_tmpBE;struct Cyc_Absyn_Tqual _tmpC0;void*_tmpBF;struct Cyc_List_List*_tmpC1;struct Cyc_Absyn_Datatypefield*_tmpC2;struct Cyc_Absyn_Aggrdecl*_tmpC3;switch(*((int*)_tmpBD)){case 0U: switch(*((int*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmpBD)->f1)){case 20U: if(((((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmpBD)->f1)->f1).KnownAggr).tag == 2){_LL1: _tmpC3=*((((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmpBD)->f1)->f1).KnownAggr).val;_LL2: {struct Cyc_Absyn_Aggrdecl*ad=_tmpC3;
# 672
if(ad->impl == 0){
({void*_tmpC4=0U;({unsigned _tmp554=loc;struct _fat_ptr _tmp553=({const char*_tmpC5="attempt to write an abstract aggregate";_tag_fat(_tmpC5,sizeof(char),39U);});Cyc_Tcutil_terr(_tmp554,_tmp553,_tag_fat(_tmpC4,sizeof(void*),0U));});});
return 0;}else{
# 676
_tmpC1=((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->fields;goto _LL4;}}}else{goto _LLB;}case 19U: if(((((struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmpBD)->f1)->f1).KnownDatatypefield).tag == 2){_LL5: _tmpC2=(((((struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmpBD)->f1)->f1).KnownDatatypefield).val).f2;_LL6: {struct Cyc_Absyn_Datatypefield*df=_tmpC2;
# 688
{struct Cyc_List_List*fs=df->typs;for(0;fs != 0;fs=fs->tl){
struct _tuple17*_stmttmpD=(struct _tuple17*)fs->hd;void*_tmpCA;struct Cyc_Absyn_Tqual _tmpC9;_LLE: _tmpC9=_stmttmpD->f1;_tmpCA=_stmttmpD->f2;_LLF: {struct Cyc_Absyn_Tqual tq=_tmpC9;void*t=_tmpCA;
if(tq.real_const){
({struct Cyc_String_pa_PrintArg_struct _tmpCD=({struct Cyc_String_pa_PrintArg_struct _tmp44A;_tmp44A.tag=0U,({struct _fat_ptr _tmp555=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(df->name));_tmp44A.f1=_tmp555;});_tmp44A;});void*_tmpCB[1U];_tmpCB[0]=& _tmpCD;({unsigned _tmp557=loc;struct _fat_ptr _tmp556=({const char*_tmpCC="attempt to over-write a datatype field (%s) with a const member";_tag_fat(_tmpCC,sizeof(char),64U);});Cyc_Tcutil_terr(_tmp557,_tmp556,_tag_fat(_tmpCB,sizeof(void*),1U));});});
return 0;}
# 690
if(!
# 694
Cyc_Tcexp_check_writable_aggr(loc,t))return 0;}}}
# 696
return 1;}}else{goto _LLB;}default: goto _LLB;}case 7U: _LL3: _tmpC1=((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmpBD)->f2;_LL4: {struct Cyc_List_List*fs=_tmpC1;
# 678
for(0;fs != 0;fs=fs->tl){
struct Cyc_Absyn_Aggrfield*f=(struct Cyc_Absyn_Aggrfield*)fs->hd;
if((f->tq).real_const){
({struct Cyc_String_pa_PrintArg_struct _tmpC8=({struct Cyc_String_pa_PrintArg_struct _tmp44B;_tmp44B.tag=0U,_tmp44B.f1=(struct _fat_ptr)((struct _fat_ptr)*f->name);_tmp44B;});void*_tmpC6[1U];_tmpC6[0]=& _tmpC8;({unsigned _tmp559=loc;struct _fat_ptr _tmp558=({const char*_tmpC7="attempt to over-write an aggregate with const member %s";_tag_fat(_tmpC7,sizeof(char),56U);});Cyc_Tcutil_terr(_tmp559,_tmp558,_tag_fat(_tmpC6,sizeof(void*),1U));});});
return 0;}
# 680
if(!
# 684
Cyc_Tcexp_check_writable_aggr(loc,f->type))return 0;}
# 686
return 1;}case 4U: _LL7: _tmpBF=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmpBD)->f1).elt_type;_tmpC0=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmpBD)->f1).tq;_LL8: {void*elt_type=_tmpBF;struct Cyc_Absyn_Tqual tq=_tmpC0;
# 698
if(tq.real_const){
({void*_tmpCE=0U;({unsigned _tmp55B=loc;struct _fat_ptr _tmp55A=({const char*_tmpCF="attempt to over-write a const array";_tag_fat(_tmpCF,sizeof(char),36U);});Cyc_Tcutil_terr(_tmp55B,_tmp55A,_tag_fat(_tmpCE,sizeof(void*),0U));});});
return 0;}
# 698
return
# 702
 Cyc_Tcexp_check_writable_aggr(loc,elt_type);}case 6U: _LL9: _tmpBE=((struct Cyc_Absyn_TupleType_Absyn_Type_struct*)_tmpBD)->f1;_LLA: {struct Cyc_List_List*fs=_tmpBE;
# 704
for(0;fs != 0;fs=fs->tl){
struct _tuple17*_stmttmpE=(struct _tuple17*)fs->hd;void*_tmpD1;struct Cyc_Absyn_Tqual _tmpD0;_LL11: _tmpD0=_stmttmpE->f1;_tmpD1=_stmttmpE->f2;_LL12: {struct Cyc_Absyn_Tqual tq=_tmpD0;void*t=_tmpD1;
if(tq.real_const){
({void*_tmpD2=0U;({unsigned _tmp55D=loc;struct _fat_ptr _tmp55C=({const char*_tmpD3="attempt to over-write a tuple field with a const member";_tag_fat(_tmpD3,sizeof(char),56U);});Cyc_Tcutil_terr(_tmp55D,_tmp55C,_tag_fat(_tmpD2,sizeof(void*),0U));});});
return 0;}
# 706
if(!
# 710
Cyc_Tcexp_check_writable_aggr(loc,t))return 0;}}
# 712
return 1;}default: _LLB: _LLC:
 return 1;}_LL0:;}
# 306
static void Cyc_Tcexp_check_writable(struct Cyc_Tcenv_Tenv*te,struct Cyc_Absyn_Exp*e){
# 723
if(!Cyc_Tcexp_check_writable_aggr(e->loc,(void*)_check_null(e->topt)))return;{void*_stmttmpF=e->r;void*_tmpD4=_stmttmpF;struct Cyc_Absyn_Exp*_tmpD5;struct Cyc_Absyn_Exp*_tmpD6;struct Cyc_Absyn_Exp*_tmpD7;struct _fat_ptr*_tmpD9;struct Cyc_Absyn_Exp*_tmpD8;struct _fat_ptr*_tmpDB;struct Cyc_Absyn_Exp*_tmpDA;struct Cyc_Absyn_Exp*_tmpDD;struct Cyc_Absyn_Exp*_tmpDC;struct Cyc_Absyn_Vardecl*_tmpDE;struct Cyc_Absyn_Vardecl*_tmpDF;struct Cyc_Absyn_Vardecl*_tmpE0;struct Cyc_Absyn_Vardecl*_tmpE1;switch(*((int*)_tmpD4)){case 1U: switch(*((int*)((struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct*)_tmpD4)->f1)){case 3U: _LL1: _tmpE1=((struct Cyc_Absyn_Param_b_Absyn_Binding_struct*)((struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct*)_tmpD4)->f1)->f1;_LL2: {struct Cyc_Absyn_Vardecl*vd=_tmpE1;
# 725
_tmpE0=vd;goto _LL4;}case 4U: _LL3: _tmpE0=((struct Cyc_Absyn_Local_b_Absyn_Binding_struct*)((struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct*)_tmpD4)->f1)->f1;_LL4: {struct Cyc_Absyn_Vardecl*vd=_tmpE0;
_tmpDF=vd;goto _LL6;}case 5U: _LL5: _tmpDF=((struct Cyc_Absyn_Pat_b_Absyn_Binding_struct*)((struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct*)_tmpD4)->f1)->f1;_LL6: {struct Cyc_Absyn_Vardecl*vd=_tmpDF;
_tmpDE=vd;goto _LL8;}case 1U: _LL7: _tmpDE=((struct Cyc_Absyn_Global_b_Absyn_Binding_struct*)((struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct*)_tmpD4)->f1)->f1;_LL8: {struct Cyc_Absyn_Vardecl*vd=_tmpDE;
if(!(vd->tq).real_const)return;goto _LL0;}default: goto _LL15;}case 23U: _LL9: _tmpDC=((struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct*)_tmpD4)->f1;_tmpDD=((struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct*)_tmpD4)->f2;_LLA: {struct Cyc_Absyn_Exp*e1=_tmpDC;struct Cyc_Absyn_Exp*e2=_tmpDD;
# 730
{void*_stmttmp10=Cyc_Tcutil_compress((void*)_check_null(e1->topt));void*_tmpE2=_stmttmp10;struct Cyc_List_List*_tmpE3;struct Cyc_Absyn_Tqual _tmpE4;struct Cyc_Absyn_Tqual _tmpE5;switch(*((int*)_tmpE2)){case 3U: _LL18: _tmpE5=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmpE2)->f1).elt_tq;_LL19: {struct Cyc_Absyn_Tqual tq=_tmpE5;
_tmpE4=tq;goto _LL1B;}case 4U: _LL1A: _tmpE4=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmpE2)->f1).tq;_LL1B: {struct Cyc_Absyn_Tqual tq=_tmpE4;
if(!tq.real_const)return;goto _LL17;}case 6U: _LL1C: _tmpE3=((struct Cyc_Absyn_TupleType_Absyn_Type_struct*)_tmpE2)->f1;_LL1D: {struct Cyc_List_List*ts=_tmpE3;
# 734
struct _tuple16 _stmttmp11=Cyc_Evexp_eval_const_uint_exp(e2);int _tmpE7;unsigned _tmpE6;_LL21: _tmpE6=_stmttmp11.f1;_tmpE7=_stmttmp11.f2;_LL22: {unsigned i=_tmpE6;int known=_tmpE7;
if(!known){
({void*_tmpE8=0U;({unsigned _tmp55F=e->loc;struct _fat_ptr _tmp55E=({const char*_tmpE9="tuple projection cannot use sizeof or offsetof";_tag_fat(_tmpE9,sizeof(char),47U);});Cyc_Tcutil_terr(_tmp55F,_tmp55E,_tag_fat(_tmpE8,sizeof(void*),0U));});});
return;}
# 735
{struct _handler_cons _tmpEA;_push_handler(& _tmpEA);{int _tmpEC=0;if(setjmp(_tmpEA.handler))_tmpEC=1;if(!_tmpEC){
# 740
{struct _tuple17*_stmttmp12=({struct _tuple17*(*_tmp561)(struct Cyc_List_List*x,int n)=({struct _tuple17*(*_tmpEE)(struct Cyc_List_List*x,int n)=(struct _tuple17*(*)(struct Cyc_List_List*x,int n))Cyc_List_nth;_tmpEE;});struct Cyc_List_List*_tmp560=ts;_tmp561(_tmp560,(int)i);});struct Cyc_Absyn_Tqual _tmpED;_LL24: _tmpED=_stmttmp12->f1;_LL25: {struct Cyc_Absyn_Tqual tq=_tmpED;
if(!tq.real_const){_npop_handler(0U);return;}}}
# 740
;_pop_handler();}else{void*_tmpEB=(void*)Cyc_Core_get_exn_thrown();void*_tmpEF=_tmpEB;void*_tmpF0;if(((struct Cyc_List_Nth_exn_struct*)_tmpEF)->tag == Cyc_List_Nth){_LL27: _LL28:
# 742
 return;}else{_LL29: _tmpF0=_tmpEF;_LL2A: {void*exn=_tmpF0;(int)_rethrow(exn);}}_LL26:;}}}
goto _LL17;}}default: _LL1E: _LL1F:
 goto _LL17;}_LL17:;}
# 746
goto _LL0;}case 21U: _LLB: _tmpDA=((struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct*)_tmpD4)->f1;_tmpDB=((struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct*)_tmpD4)->f2;_LLC: {struct Cyc_Absyn_Exp*e1=_tmpDA;struct _fat_ptr*f=_tmpDB;
# 748
{void*_stmttmp13=Cyc_Tcutil_compress((void*)_check_null(e1->topt));void*_tmpF1=_stmttmp13;struct Cyc_List_List*_tmpF2;struct Cyc_Absyn_Aggrdecl**_tmpF3;switch(*((int*)_tmpF1)){case 0U: if(((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmpF1)->f1)->tag == 20U){if(((((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmpF1)->f1)->f1).KnownAggr).tag == 2){_LL2C: _tmpF3=((((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmpF1)->f1)->f1).KnownAggr).val;_LL2D: {struct Cyc_Absyn_Aggrdecl**adp=_tmpF3;
# 750
struct Cyc_Absyn_Aggrfield*sf=
adp == 0?0: Cyc_Absyn_lookup_decl_field(*adp,f);
if(sf == 0 || !(sf->tq).real_const)return;goto _LL2B;}}else{goto _LL30;}}else{goto _LL30;}case 7U: _LL2E: _tmpF2=((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmpF1)->f2;_LL2F: {struct Cyc_List_List*fs=_tmpF2;
# 755
struct Cyc_Absyn_Aggrfield*sf=Cyc_Absyn_lookup_field(fs,f);
if(sf == 0 || !(sf->tq).real_const)return;goto _LL2B;}default: _LL30: _LL31:
# 758
 goto _LL2B;}_LL2B:;}
# 760
goto _LL0;}case 22U: _LLD: _tmpD8=((struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct*)_tmpD4)->f1;_tmpD9=((struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct*)_tmpD4)->f2;_LLE: {struct Cyc_Absyn_Exp*e1=_tmpD8;struct _fat_ptr*f=_tmpD9;
# 762
{void*_stmttmp14=Cyc_Tcutil_compress((void*)_check_null(e1->topt));void*_tmpF4=_stmttmp14;struct Cyc_Absyn_Tqual _tmpF6;void*_tmpF5;if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmpF4)->tag == 3U){_LL33: _tmpF5=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmpF4)->f1).elt_type;_tmpF6=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmpF4)->f1).elt_tq;_LL34: {void*elt_typ=_tmpF5;struct Cyc_Absyn_Tqual tq=_tmpF6;
# 764
if(!tq.real_const){
void*_stmttmp15=Cyc_Tcutil_compress(elt_typ);void*_tmpF7=_stmttmp15;struct Cyc_List_List*_tmpF8;struct Cyc_Absyn_Aggrdecl**_tmpF9;switch(*((int*)_tmpF7)){case 0U: if(((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmpF7)->f1)->tag == 20U){if(((((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmpF7)->f1)->f1).KnownAggr).tag == 2){_LL38: _tmpF9=((((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmpF7)->f1)->f1).KnownAggr).val;_LL39: {struct Cyc_Absyn_Aggrdecl**adp=_tmpF9;
# 767
struct Cyc_Absyn_Aggrfield*sf=
adp == 0?0: Cyc_Absyn_lookup_decl_field(*adp,f);
if(sf == 0 || !(sf->tq).real_const)return;goto _LL37;}}else{goto _LL3C;}}else{goto _LL3C;}case 7U: _LL3A: _tmpF8=((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmpF7)->f2;_LL3B: {struct Cyc_List_List*fs=_tmpF8;
# 772
struct Cyc_Absyn_Aggrfield*sf=Cyc_Absyn_lookup_field(fs,f);
if(sf == 0 || !(sf->tq).real_const)return;goto _LL37;}default: _LL3C: _LL3D:
# 775
 goto _LL37;}_LL37:;}
# 764
goto _LL32;}}else{_LL35: _LL36:
# 779
 goto _LL32;}_LL32:;}
# 781
goto _LL0;}case 20U: _LLF: _tmpD7=((struct Cyc_Absyn_Deref_e_Absyn_Raw_exp_struct*)_tmpD4)->f1;_LL10: {struct Cyc_Absyn_Exp*e1=_tmpD7;
# 783
{void*_stmttmp16=Cyc_Tcutil_compress((void*)_check_null(e1->topt));void*_tmpFA=_stmttmp16;struct Cyc_Absyn_Tqual _tmpFB;struct Cyc_Absyn_Tqual _tmpFC;switch(*((int*)_tmpFA)){case 3U: _LL3F: _tmpFC=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmpFA)->f1).elt_tq;_LL40: {struct Cyc_Absyn_Tqual tq=_tmpFC;
_tmpFB=tq;goto _LL42;}case 4U: _LL41: _tmpFB=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmpFA)->f1).tq;_LL42: {struct Cyc_Absyn_Tqual tq=_tmpFB;
if(!tq.real_const)return;goto _LL3E;}default: _LL43: _LL44:
 goto _LL3E;}_LL3E:;}
# 788
goto _LL0;}case 12U: _LL11: _tmpD6=((struct Cyc_Absyn_NoInstantiate_e_Absyn_Raw_exp_struct*)_tmpD4)->f1;_LL12: {struct Cyc_Absyn_Exp*e1=_tmpD6;
_tmpD5=e1;goto _LL14;}case 13U: _LL13: _tmpD5=((struct Cyc_Absyn_Instantiate_e_Absyn_Raw_exp_struct*)_tmpD4)->f1;_LL14: {struct Cyc_Absyn_Exp*e1=_tmpD5;
Cyc_Tcexp_check_writable(te,e1);return;}default: _LL15: _LL16:
 goto _LL0;}_LL0:;}
# 793
({struct Cyc_String_pa_PrintArg_struct _tmpFF=({struct Cyc_String_pa_PrintArg_struct _tmp44C;_tmp44C.tag=0U,({struct _fat_ptr _tmp562=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_exp2string(e));_tmp44C.f1=_tmp562;});_tmp44C;});void*_tmpFD[1U];_tmpFD[0]=& _tmpFF;({unsigned _tmp564=e->loc;struct _fat_ptr _tmp563=({const char*_tmpFE="attempt to write a const location: %s";_tag_fat(_tmpFE,sizeof(char),38U);});Cyc_Tcutil_terr(_tmp564,_tmp563,_tag_fat(_tmpFD,sizeof(void*),1U));});});}
# 306
static void*Cyc_Tcexp_tcIncrement(struct Cyc_Tcenv_Tenv*te,unsigned loc,void**topt,struct Cyc_Absyn_Exp*e,enum Cyc_Absyn_Incrementor i){
# 799
({struct Cyc_Tcenv_Tenv*_tmp565=Cyc_Tcenv_enter_lhs(te);Cyc_Tcexp_tcExpNoPromote(_tmp565,0,e);});
if(!Cyc_Absyn_is_lvalue(e))
return({void*_tmp100=0U;({struct Cyc_Tcenv_Tenv*_tmp569=te;unsigned _tmp568=loc;void**_tmp567=topt;struct _fat_ptr _tmp566=({const char*_tmp101="increment/decrement of non-lvalue";_tag_fat(_tmp101,sizeof(char),34U);});Cyc_Tcexp_expr_err(_tmp569,_tmp568,_tmp567,_tmp566,_tag_fat(_tmp100,sizeof(void*),0U));});});
# 800
Cyc_Tcexp_check_writable(te,e);{
# 803
void*t=(void*)_check_null(e->topt);
# 805
if(!Cyc_Tcutil_is_numeric(e)){
void*telt=Cyc_Absyn_void_type;
if(Cyc_Tcutil_is_fat_pointer_type_elt(t,& telt)||
 Cyc_Tcutil_is_zero_pointer_type_elt(t,& telt)&&((int)i == (int)0U ||(int)i == (int)1U)){
if(!Cyc_Tcutil_kind_leq(Cyc_Tcutil_type_kind(telt),& Cyc_Tcutil_tmk))
({void*_tmp102=0U;({unsigned _tmp56B=e->loc;struct _fat_ptr _tmp56A=({const char*_tmp103="can't perform arithmetic on abstract pointer type";_tag_fat(_tmp103,sizeof(char),50U);});Cyc_Tcutil_terr(_tmp56B,_tmp56A,_tag_fat(_tmp102,sizeof(void*),0U));});});
# 809
if(
# 811
Cyc_Tcutil_is_noalias_pointer(t,0))
({void*_tmp104=0U;({unsigned _tmp56D=e->loc;struct _fat_ptr _tmp56C=({const char*_tmp105="can't perform arithmetic on non-aliasing pointer type";_tag_fat(_tmp105,sizeof(char),54U);});Cyc_Tcutil_terr(_tmp56D,_tmp56C,_tag_fat(_tmp104,sizeof(void*),0U));});});}else{
# 814
({struct Cyc_String_pa_PrintArg_struct _tmp108=({struct Cyc_String_pa_PrintArg_struct _tmp44D;_tmp44D.tag=0U,({struct _fat_ptr _tmp56E=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t));_tmp44D.f1=_tmp56E;});_tmp44D;});void*_tmp106[1U];_tmp106[0]=& _tmp108;({unsigned _tmp570=e->loc;struct _fat_ptr _tmp56F=({const char*_tmp107="expecting arithmetic or ? type but found %s";_tag_fat(_tmp107,sizeof(char),44U);});Cyc_Tcutil_terr(_tmp570,_tmp56F,_tag_fat(_tmp106,sizeof(void*),1U));});});}}
# 805
return t;}}
# 306
static void*Cyc_Tcexp_tcConditional(struct Cyc_Tcenv_Tenv*te,unsigned loc,void**topt,struct Cyc_Absyn_Exp*e1,struct Cyc_Absyn_Exp*e2,struct Cyc_Absyn_Exp*e3){
# 822
({struct Cyc_Tcenv_Tenv*_tmp572=Cyc_Tcenv_clear_abstract_val_ok(te);struct Cyc_Absyn_Exp*_tmp571=e1;Cyc_Tcexp_tcTest(_tmp572,_tmp571,({const char*_tmp109="conditional expression";_tag_fat(_tmp109,sizeof(char),23U);}));});
Cyc_Tcexp_tcExp(te,topt,e2);
Cyc_Tcexp_tcExp(te,topt,e3);{
void*t;
if(Cyc_Tcenv_abstract_val_ok(te))
t=Cyc_Absyn_new_evar(& Cyc_Tcutil_tako,Cyc_Tcenv_lookup_opt_type_vars(te));else{
# 829
t=Cyc_Absyn_new_evar(& Cyc_Tcutil_tmko,Cyc_Tcenv_lookup_opt_type_vars(te));}{
struct Cyc_List_List l1=({struct Cyc_List_List _tmp451;_tmp451.hd=e3,_tmp451.tl=0;_tmp451;});
struct Cyc_List_List l2=({struct Cyc_List_List _tmp450;_tmp450.hd=e2,_tmp450.tl=& l1;_tmp450;});
if(!Cyc_Tcutil_coerce_list(te,t,& l2)){
({struct Cyc_String_pa_PrintArg_struct _tmp10C=({struct Cyc_String_pa_PrintArg_struct _tmp44F;_tmp44F.tag=0U,({
struct _fat_ptr _tmp573=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string((void*)_check_null(e2->topt)));_tmp44F.f1=_tmp573;});_tmp44F;});struct Cyc_String_pa_PrintArg_struct _tmp10D=({struct Cyc_String_pa_PrintArg_struct _tmp44E;_tmp44E.tag=0U,({struct _fat_ptr _tmp574=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string((void*)_check_null(e3->topt)));_tmp44E.f1=_tmp574;});_tmp44E;});void*_tmp10A[2U];_tmp10A[0]=& _tmp10C,_tmp10A[1]=& _tmp10D;({unsigned _tmp576=loc;struct _fat_ptr _tmp575=({const char*_tmp10B="conditional clause types do not match: %s != %s";_tag_fat(_tmp10B,sizeof(char),48U);});Cyc_Tcutil_terr(_tmp576,_tmp575,_tag_fat(_tmp10A,sizeof(void*),2U));});});
Cyc_Unify_explain_failure();}
# 832
return t;}}}
# 306
static void*Cyc_Tcexp_tcAnd(struct Cyc_Tcenv_Tenv*te,unsigned loc,struct Cyc_Absyn_Exp*e1,struct Cyc_Absyn_Exp*e2){
# 843
({struct Cyc_Tcenv_Tenv*_tmp578=te;struct Cyc_Absyn_Exp*_tmp577=e1;Cyc_Tcexp_tcTest(_tmp578,_tmp577,({const char*_tmp10E="logical-and expression";_tag_fat(_tmp10E,sizeof(char),23U);}));});
({struct Cyc_Tcenv_Tenv*_tmp57A=te;struct Cyc_Absyn_Exp*_tmp579=e2;Cyc_Tcexp_tcTest(_tmp57A,_tmp579,({const char*_tmp10F="logical-and expression";_tag_fat(_tmp10F,sizeof(char),23U);}));});
return Cyc_Absyn_sint_type;}
# 306
static void*Cyc_Tcexp_tcOr(struct Cyc_Tcenv_Tenv*te,unsigned loc,struct Cyc_Absyn_Exp*e1,struct Cyc_Absyn_Exp*e2){
# 851
({struct Cyc_Tcenv_Tenv*_tmp57C=te;struct Cyc_Absyn_Exp*_tmp57B=e1;Cyc_Tcexp_tcTest(_tmp57C,_tmp57B,({const char*_tmp110="logical-or expression";_tag_fat(_tmp110,sizeof(char),22U);}));});
({struct Cyc_Tcenv_Tenv*_tmp57E=te;struct Cyc_Absyn_Exp*_tmp57D=e2;Cyc_Tcexp_tcTest(_tmp57E,_tmp57D,({const char*_tmp111="logical-or expression";_tag_fat(_tmp111,sizeof(char),22U);}));});
return Cyc_Absyn_sint_type;}
# 306
static void*Cyc_Tcexp_tcAssignOp(struct Cyc_Tcenv_Tenv*te,unsigned loc,void**topt,struct Cyc_Absyn_Exp*e1,struct Cyc_Core_Opt*po,struct Cyc_Absyn_Exp*e2){
# 864
({struct Cyc_Tcenv_Tenv*_tmp57F=Cyc_Tcenv_enter_lhs(Cyc_Tcenv_enter_notreadctxt(te));Cyc_Tcexp_tcExpNoPromote(_tmp57F,0,e1);});{
void*t1=(void*)_check_null(e1->topt);
Cyc_Tcexp_tcExp(te,& t1,e2);{
void*t2=(void*)_check_null(e2->topt);
# 869
{void*_stmttmp17=Cyc_Tcutil_compress(t1);void*_tmp112=_stmttmp17;if(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp112)->tag == 4U){_LL1: _LL2:
({void*_tmp113=0U;({unsigned _tmp581=loc;struct _fat_ptr _tmp580=({const char*_tmp114="cannot assign to an array";_tag_fat(_tmp114,sizeof(char),26U);});Cyc_Tcutil_terr(_tmp581,_tmp580,_tag_fat(_tmp113,sizeof(void*),0U));});});goto _LL0;}else{_LL3: _LL4:
 goto _LL0;}_LL0:;}
# 874
if(!Cyc_Tcutil_kind_leq(Cyc_Tcutil_type_kind(t1),& Cyc_Tcutil_tmk))
({void*_tmp115=0U;({unsigned _tmp583=loc;struct _fat_ptr _tmp582=({const char*_tmp116="type is abstract (can't determine size).";_tag_fat(_tmp116,sizeof(char),41U);});Cyc_Tcutil_terr(_tmp583,_tmp582,_tag_fat(_tmp115,sizeof(void*),0U));});});
# 874
if(!
# 878
Cyc_Absyn_is_lvalue(e1))
return({void*_tmp117=0U;({struct Cyc_Tcenv_Tenv*_tmp587=te;unsigned _tmp586=loc;void**_tmp585=topt;struct _fat_ptr _tmp584=({const char*_tmp118="assignment to non-lvalue";_tag_fat(_tmp118,sizeof(char),25U);});Cyc_Tcexp_expr_err(_tmp587,_tmp586,_tmp585,_tmp584,_tag_fat(_tmp117,sizeof(void*),0U));});});
# 874
Cyc_Tcexp_check_writable(te,e1);
# 881
if(po == 0){
if(Cyc_Tcutil_is_noalias_pointer_or_aggr(t2)&& !Cyc_Tcutil_is_noalias_path(e2))
({void*_tmp119=0U;({unsigned _tmp589=e2->loc;struct _fat_ptr _tmp588=({const char*_tmp11A="Cannot consume non-unique paths; do swap instead";_tag_fat(_tmp11A,sizeof(char),49U);});Cyc_Tcutil_terr(_tmp589,_tmp588,_tag_fat(_tmp119,sizeof(void*),0U));});});
# 882
if(!
# 884
Cyc_Tcutil_coerce_assign(te,e2,t1)){
void*result=({struct Cyc_String_pa_PrintArg_struct _tmp11D=({struct Cyc_String_pa_PrintArg_struct _tmp453;_tmp453.tag=0U,({
struct _fat_ptr _tmp58A=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t1));_tmp453.f1=_tmp58A;});_tmp453;});struct Cyc_String_pa_PrintArg_struct _tmp11E=({struct Cyc_String_pa_PrintArg_struct _tmp452;_tmp452.tag=0U,({struct _fat_ptr _tmp58B=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t2));_tmp452.f1=_tmp58B;});_tmp452;});void*_tmp11B[2U];_tmp11B[0]=& _tmp11D,_tmp11B[1]=& _tmp11E;({struct Cyc_Tcenv_Tenv*_tmp58F=te;unsigned _tmp58E=loc;void**_tmp58D=topt;struct _fat_ptr _tmp58C=({const char*_tmp11C="type mismatch: %s != %s";_tag_fat(_tmp11C,sizeof(char),24U);});Cyc_Tcexp_expr_err(_tmp58F,_tmp58E,_tmp58D,_tmp58C,_tag_fat(_tmp11B,sizeof(void*),2U));});});
Cyc_Unify_unify(t1,t2);
Cyc_Unify_explain_failure();
return result;}}else{
# 892
enum Cyc_Absyn_Primop p=(enum Cyc_Absyn_Primop)po->v;
struct Cyc_Absyn_Exp*e1copy=Cyc_Absyn_copy_exp(e1);
void*t_result=Cyc_Tcexp_tcBinPrimop(te,loc,0,p,e1copy,e2);
if((!Cyc_Unify_unify(t_result,t1)&& Cyc_Tcutil_is_arithmetic_type(t_result))&& Cyc_Tcutil_is_arithmetic_type(t1))
t_result=t1;else{
if(!(Cyc_Unify_unify(t_result,t1)|| Cyc_Tcutil_is_arithmetic_type(t_result)&& Cyc_Tcutil_is_arithmetic_type(t1))){
void*result=({struct Cyc_String_pa_PrintArg_struct _tmp121=({struct Cyc_String_pa_PrintArg_struct _tmp455;_tmp455.tag=0U,({
# 900
struct _fat_ptr _tmp590=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t1));_tmp455.f1=_tmp590;});_tmp455;});struct Cyc_String_pa_PrintArg_struct _tmp122=({struct Cyc_String_pa_PrintArg_struct _tmp454;_tmp454.tag=0U,({
struct _fat_ptr _tmp591=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t2));_tmp454.f1=_tmp591;});_tmp454;});void*_tmp11F[2U];_tmp11F[0]=& _tmp121,_tmp11F[1]=& _tmp122;({struct Cyc_Tcenv_Tenv*_tmp595=te;unsigned _tmp594=loc;void**_tmp593=topt;struct _fat_ptr _tmp592=({const char*_tmp120="Cannot use this operator in an assignment when the arguments have types %s and %s";_tag_fat(_tmp120,sizeof(char),82U);});Cyc_Tcexp_expr_err(_tmp595,_tmp594,_tmp593,_tmp592,_tag_fat(_tmp11F,sizeof(void*),2U));});});
Cyc_Unify_unify(t_result,t1);
Cyc_Unify_explain_failure();
return result;}}
# 895
return t_result;}
# 908
return t1;}}}
# 306
static void*Cyc_Tcexp_tcSpawn(struct Cyc_Tcenv_Tenv*te,unsigned loc,void**topt,struct Cyc_Absyn_Exp*e1,struct Cyc_Absyn_Exp*e2){
# 913
void*t0=({struct Cyc_Tcenv_Tenv*_tmp596=Cyc_Tcenv_clear_abstract_val_ok(te);Cyc_Tcexp_tcExp(_tmp596,0,e1);});
void*t00=({struct Cyc_Tcenv_Tenv*_tmp598=Cyc_Tcenv_clear_abstract_val_ok(te);void**_tmp597=topt;Cyc_Tcexp_tcExp(_tmp598,_tmp597,e2);});
struct Cyc_Absyn_Exp*call=({struct Cyc_Absyn_Exp*_tmp59A=e1;struct Cyc_List_List*_tmp599=({struct Cyc_List_List*_tmp138=_cycalloc(sizeof(*_tmp138));_tmp138->hd=e2,_tmp138->tl=0;_tmp138;});Cyc_Absyn_fncall_exp(_tmp59A,_tmp599,e1->loc);});
void*t1=({struct Cyc_Tcenv_Tenv*_tmp59B=Cyc_Tcenv_clear_abstract_val_ok(te);Cyc_Tcexp_tcExp(_tmp59B,0,call);});
# 920
{void*_tmp123=t0;void*_tmp125;void*_tmp124;if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp123)->tag == 3U){_LL1: _tmp124=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp123)->f1).elt_type;_tmp125=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp123)->f1).ptr_atts).rgn;_LL2: {void*t=_tmp124;void*nullable=_tmp125;
# 923
if(Cyc_Tcutil_is_nullable_pointer_type(t0))
({struct Cyc_String_pa_PrintArg_struct _tmp128=({struct Cyc_String_pa_PrintArg_struct _tmp456;_tmp456.tag=0U,({
struct _fat_ptr _tmp59C=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t0));_tmp456.f1=_tmp59C;});_tmp456;});void*_tmp126[1U];_tmp126[0]=& _tmp128;({unsigned _tmp59E=loc;struct _fat_ptr _tmp59D=({const char*_tmp127="The first argument to spawn is expected to be a non-null function pointer that returns void but here it returns %s";_tag_fat(_tmp127,sizeof(char),115U);});Cyc_Tcutil_terr(_tmp59E,_tmp59D,_tag_fat(_tmp126,sizeof(void*),1U));});});
# 923
{void*_stmttmp18=
# 926
Cyc_Tcutil_compress(t);
# 923
void*_tmp129=_stmttmp18;struct Cyc_List_List*_tmp12B;void*_tmp12A;if(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp129)->tag == 5U){_LL6: _tmp12A=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp129)->f1).effect;_tmp12B=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp129)->f1).args;_LL7: {void*eff=_tmp12A;struct Cyc_List_List*args_info=_tmp12B;
# 929
if(!Cyc_Tcutil_is_boxed((*((struct _tuple9*)((struct Cyc_List_List*)_check_null(args_info))->hd)).f3))
# 931
({struct Cyc_String_pa_PrintArg_struct _tmp12E=({struct Cyc_String_pa_PrintArg_struct _tmp457;_tmp457.tag=0U,({
struct _fat_ptr _tmp59F=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string((*((struct _tuple9*)args_info->hd)).f3));_tmp457.f1=_tmp59F;});_tmp457;});void*_tmp12C[1U];_tmp12C[0]=& _tmp12E;({unsigned _tmp5A1=loc;struct _fat_ptr _tmp5A0=({const char*_tmp12D="The second argument of the function passed to spawn is expected to be a boxed type but here it is a non-boxed type : %s";_tag_fat(_tmp12D,sizeof(char),120U);});Cyc_Tcutil_terr(_tmp5A1,_tmp5A0,_tag_fat(_tmp12C,sizeof(void*),1U));});});}}else{_LL8: _LL9:
# 934
({void*_tmp12F=0U;({int(*_tmp5A3)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp131)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp131;});struct _fat_ptr _tmp5A2=({const char*_tmp130=" tcexp:tcSpawn : Not a pointer type (internal error)";_tag_fat(_tmp130,sizeof(char),53U);});_tmp5A3(_tmp5A2,_tag_fat(_tmp12F,sizeof(void*),0U));});});}_LL5:;}
# 937
goto _LL0;}}else{_LL3: _LL4:
({void*_tmp132=0U;({int(*_tmp5A5)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp134)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp134;});struct _fat_ptr _tmp5A4=({const char*_tmp133=" tcexp:tcSpawn : Not a pointer type (internal error)";_tag_fat(_tmp133,sizeof(char),53U);});_tmp5A5(_tmp5A4,_tag_fat(_tmp132,sizeof(void*),0U));});});}_LL0:;}
# 940
if(!Cyc_Unify_unify(Cyc_Absyn_void_type,t1)){
# 942
Cyc_Unify_explain_failure();
({struct Cyc_String_pa_PrintArg_struct _tmp137=({struct Cyc_String_pa_PrintArg_struct _tmp458;_tmp458.tag=0U,({
struct _fat_ptr _tmp5A6=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t1));_tmp458.f1=_tmp5A6;});_tmp458;});void*_tmp135[1U];_tmp135[0]=& _tmp137;({unsigned _tmp5A8=loc;struct _fat_ptr _tmp5A7=({const char*_tmp136="The first argument to spawn is expected to be a non-null function pointer that returns void but here it returns %s";_tag_fat(_tmp136,sizeof(char),115U);});Cyc_Tcutil_terr(_tmp5A8,_tmp5A7,_tag_fat(_tmp135,sizeof(void*),1U));});});}
# 940
return Cyc_Absyn_void_type;}
# 306
static void*Cyc_Tcexp_tcSeqExp(struct Cyc_Tcenv_Tenv*te,unsigned loc,void**topt,struct Cyc_Absyn_Exp*e1,struct Cyc_Absyn_Exp*e2){
# 951
({struct Cyc_Tcenv_Tenv*_tmp5A9=Cyc_Tcenv_clear_abstract_val_ok(te);Cyc_Tcexp_tcExp(_tmp5A9,0,e1);});
({struct Cyc_Tcenv_Tenv*_tmp5AB=Cyc_Tcenv_clear_abstract_val_ok(te);void**_tmp5AA=topt;Cyc_Tcexp_tcExp(_tmp5AB,_tmp5AA,e2);});
return(void*)_check_null(e2->topt);}
# 306
static struct Cyc_Absyn_Datatypefield*Cyc_Tcexp_tcInjection(struct Cyc_Tcenv_Tenv*te,struct Cyc_Absyn_Exp*e,void*tu,struct Cyc_List_List*inst,struct Cyc_List_List*fs){
# 960
struct Cyc_List_List*fields;
void*t1=(void*)_check_null(e->topt);
# 963
{void*_stmttmp19=Cyc_Tcutil_compress(t1);void*_tmp139=_stmttmp19;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp139)->tag == 0U)switch(*((int*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp139)->f1)){case 2U: if(((struct Cyc_Absyn_FloatCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp139)->f1)->f1 == 0){_LL1: _LL2:
 Cyc_Tcutil_unchecked_cast(e,Cyc_Absyn_double_type,Cyc_Absyn_No_coercion);t1=Cyc_Absyn_double_type;goto _LL0;}else{goto _LL7;}case 1U: switch(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp139)->f1)->f2){case Cyc_Absyn_Char_sz: _LL3: _LL4:
 goto _LL6;case Cyc_Absyn_Short_sz: _LL5: _LL6:
 Cyc_Tcutil_unchecked_cast(e,Cyc_Absyn_sint_type,Cyc_Absyn_No_coercion);t1=Cyc_Absyn_sint_type;goto _LL0;default: goto _LL7;}default: goto _LL7;}else{_LL7: _LL8:
 goto _LL0;}_LL0:;}
# 970
for(fields=fs;fields != 0;fields=fields->tl){
struct Cyc_Absyn_Datatypefield _stmttmp1A=*((struct Cyc_Absyn_Datatypefield*)fields->hd);enum Cyc_Absyn_Scope _tmp13D;unsigned _tmp13C;struct Cyc_List_List*_tmp13B;struct _tuple1*_tmp13A;_LLA: _tmp13A=_stmttmp1A.name;_tmp13B=_stmttmp1A.typs;_tmp13C=_stmttmp1A.loc;_tmp13D=_stmttmp1A.sc;_LLB: {struct _tuple1*n=_tmp13A;struct Cyc_List_List*typs=_tmp13B;unsigned loc=_tmp13C;enum Cyc_Absyn_Scope sc=_tmp13D;
# 973
if(typs == 0 || typs->tl != 0)continue;{void*t2=
Cyc_Tcutil_substitute(inst,(*((struct _tuple17*)typs->hd)).f2);
# 976
if(Cyc_Unify_unify(t1,t2))
return(struct Cyc_Absyn_Datatypefield*)fields->hd;}}}
# 980
for(fields=fs;fields != 0;fields=fields->tl){
struct Cyc_Absyn_Datatypefield _stmttmp1B=*((struct Cyc_Absyn_Datatypefield*)fields->hd);enum Cyc_Absyn_Scope _tmp141;unsigned _tmp140;struct Cyc_List_List*_tmp13F;struct _tuple1*_tmp13E;_LLD: _tmp13E=_stmttmp1B.name;_tmp13F=_stmttmp1B.typs;_tmp140=_stmttmp1B.loc;_tmp141=_stmttmp1B.sc;_LLE: {struct _tuple1*n=_tmp13E;struct Cyc_List_List*typs=_tmp13F;unsigned loc=_tmp140;enum Cyc_Absyn_Scope sc=_tmp141;
# 983
if(typs == 0 || typs->tl != 0)continue;{void*t2=
Cyc_Tcutil_substitute(inst,(*((struct _tuple17*)typs->hd)).f2);
# 987
int bogus=0;
if(Cyc_Tcutil_coerce_arg(te,e,t2,& bogus))
return(struct Cyc_Absyn_Datatypefield*)fields->hd;}}}
# 992
({struct Cyc_String_pa_PrintArg_struct _tmp144=({struct Cyc_String_pa_PrintArg_struct _tmp45A;_tmp45A.tag=0U,({
struct _fat_ptr _tmp5AC=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(tu));_tmp45A.f1=_tmp5AC;});_tmp45A;});struct Cyc_String_pa_PrintArg_struct _tmp145=({struct Cyc_String_pa_PrintArg_struct _tmp459;_tmp459.tag=0U,({struct _fat_ptr _tmp5AD=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t1));_tmp459.f1=_tmp5AD;});_tmp459;});void*_tmp142[2U];_tmp142[0]=& _tmp144,_tmp142[1]=& _tmp145;({unsigned _tmp5AF=e->loc;struct _fat_ptr _tmp5AE=({const char*_tmp143="can't find a field in %s to inject a value of type %s";_tag_fat(_tmp143,sizeof(char),54U);});Cyc_Tcutil_terr(_tmp5AF,_tmp5AE,_tag_fat(_tmp142,sizeof(void*),2U));});});
return 0;}
# 306
static void*Cyc_Tcexp_tcFnCall(struct Cyc_Tcenv_Tenv*te_orig,unsigned loc,void**topt,struct Cyc_Absyn_Exp*e,struct Cyc_List_List*args,struct Cyc_Absyn_VarargCallInfo**vararg_call_info,struct Cyc_List_List**alias_arg_exps){
# 1004
struct Cyc_List_List*es=args;
int arg_cnt=0;
struct Cyc_Tcenv_Tenv*te=Cyc_Tcenv_new_block(loc,te_orig);
struct Cyc_Tcenv_Tenv*_tmp146=Cyc_Tcenv_clear_abstract_val_ok(te);{struct Cyc_Tcenv_Tenv*te=_tmp146;
Cyc_Tcexp_tcExp(te,0,e);{
void*t=Cyc_Tcutil_compress((void*)_check_null(e->topt));
# 1013
void*_tmp147=t;void*_tmp14D;void*_tmp14C;void*_tmp14B;void*_tmp14A;struct Cyc_Absyn_Tqual _tmp149;void*_tmp148;if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp147)->tag == 3U){_LL1: _tmp148=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp147)->f1).elt_type;_tmp149=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp147)->f1).elt_tq;_tmp14A=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp147)->f1).ptr_atts).rgn;_tmp14B=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp147)->f1).ptr_atts).nullable;_tmp14C=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp147)->f1).ptr_atts).bounds;_tmp14D=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp147)->f1).ptr_atts).zero_term;_LL2: {void*t1=_tmp148;struct Cyc_Absyn_Tqual tq=_tmp149;void*rgn=_tmp14A;void*x=_tmp14B;void*b=_tmp14C;void*zt=_tmp14D;
# 1018
Cyc_Tcenv_check_rgn_accessible(te,loc,rgn);
# 1020
Cyc_Tcutil_check_nonzero_bound(loc,b);{
void*_stmttmp1C=Cyc_Tcutil_compress(t1);void*_tmp14E=_stmttmp1C;struct Cyc_List_List*_tmp15D;struct Cyc_List_List*_tmp15C;struct Cyc_List_List*_tmp15B;struct Cyc_Absyn_Exp*_tmp15A;struct Cyc_List_List*_tmp159;struct Cyc_Absyn_Exp*_tmp158;struct Cyc_List_List*_tmp157;struct Cyc_List_List*_tmp156;struct Cyc_Absyn_VarargInfo*_tmp155;int _tmp154;struct Cyc_List_List*_tmp153;void*_tmp152;struct Cyc_Absyn_Tqual _tmp151;void*_tmp150;struct Cyc_List_List*_tmp14F;if(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp14E)->tag == 5U){_LL6: _tmp14F=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp14E)->f1).tvars;_tmp150=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp14E)->f1).effect;_tmp151=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp14E)->f1).ret_tqual;_tmp152=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp14E)->f1).ret_type;_tmp153=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp14E)->f1).args;_tmp154=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp14E)->f1).c_varargs;_tmp155=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp14E)->f1).cyc_varargs;_tmp156=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp14E)->f1).rgn_po;_tmp157=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp14E)->f1).attributes;_tmp158=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp14E)->f1).requires_clause;_tmp159=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp14E)->f1).requires_relns;_tmp15A=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp14E)->f1).ensures_clause;_tmp15B=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp14E)->f1).ensures_relns;_tmp15C=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp14E)->f1).ieffect;_tmp15D=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp14E)->f1).oeffect;_LL7: {struct Cyc_List_List*tvars=_tmp14F;void*eff=_tmp150;struct Cyc_Absyn_Tqual res_tq=_tmp151;void*res_typ=_tmp152;struct Cyc_List_List*args_info=_tmp153;int c_vararg=_tmp154;struct Cyc_Absyn_VarargInfo*cyc_vararg=_tmp155;struct Cyc_List_List*rgn_po=_tmp156;struct Cyc_List_List*atts=_tmp157;struct Cyc_Absyn_Exp*req=_tmp158;struct Cyc_List_List*req_relns=_tmp159;struct Cyc_Absyn_Exp*ens=_tmp15A;struct Cyc_List_List*ens_relns=_tmp15B;struct Cyc_List_List*ieff=_tmp15C;struct Cyc_List_List*oeff=_tmp15D;
# 1025
if(tvars != 0 || rgn_po != 0)
({void*_tmp15E=0U;({unsigned _tmp5B1=e->loc;struct _fat_ptr _tmp5B0=({const char*_tmp15F="function should have been instantiated prior to use -- probably a compiler bug";_tag_fat(_tmp15F,sizeof(char),79U);});Cyc_Tcutil_terr(_tmp5B1,_tmp5B0,_tag_fat(_tmp15E,sizeof(void*),0U));});});
# 1025
if(topt != 0)
# 1029
Cyc_Unify_unify(res_typ,*topt);
# 1025
while(
# 1031
es != 0 && args_info != 0){
# 1033
int alias_coercion=0;
struct Cyc_Absyn_Exp*e1=(struct Cyc_Absyn_Exp*)es->hd;
void*t2=(*((struct _tuple9*)args_info->hd)).f3;
Cyc_Tcexp_tcExp(te,& t2,e1);
if(!Cyc_Tcutil_coerce_arg(te,e1,t2,& alias_coercion)){
struct _fat_ptr s0=({const char*_tmp16F="actual argument has type ";_tag_fat(_tmp16F,sizeof(char),26U);});
struct _fat_ptr s1=Cyc_Absynpp_typ2string((void*)_check_null(e1->topt));
struct _fat_ptr s2=({const char*_tmp16E=" but formal has type ";_tag_fat(_tmp16E,sizeof(char),22U);});
struct _fat_ptr s3=Cyc_Absynpp_typ2string(t2);
if(({unsigned long _tmp5B4=({unsigned long _tmp5B3=({unsigned long _tmp5B2=Cyc_strlen((struct _fat_ptr)s0);_tmp5B2 + Cyc_strlen((struct _fat_ptr)s1);});_tmp5B3 + Cyc_strlen((struct _fat_ptr)s2);});_tmp5B4 + Cyc_strlen((struct _fat_ptr)s3);})>= (unsigned long)70)
({void*_tmp160=0U;({unsigned _tmp5B7=e1->loc;struct _fat_ptr _tmp5B6=(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp163=({struct Cyc_String_pa_PrintArg_struct _tmp45E;_tmp45E.tag=0U,_tmp45E.f1=(struct _fat_ptr)((struct _fat_ptr)s0);_tmp45E;});struct Cyc_String_pa_PrintArg_struct _tmp164=({struct Cyc_String_pa_PrintArg_struct _tmp45D;_tmp45D.tag=0U,_tmp45D.f1=(struct _fat_ptr)((struct _fat_ptr)s1);_tmp45D;});struct Cyc_String_pa_PrintArg_struct _tmp165=({struct Cyc_String_pa_PrintArg_struct _tmp45C;_tmp45C.tag=0U,_tmp45C.f1=(struct _fat_ptr)((struct _fat_ptr)s2);_tmp45C;});struct Cyc_String_pa_PrintArg_struct _tmp166=({struct Cyc_String_pa_PrintArg_struct _tmp45B;_tmp45B.tag=0U,_tmp45B.f1=(struct _fat_ptr)((struct _fat_ptr)s3);_tmp45B;});void*_tmp161[4U];_tmp161[0]=& _tmp163,_tmp161[1]=& _tmp164,_tmp161[2]=& _tmp165,_tmp161[3]=& _tmp166;({struct _fat_ptr _tmp5B5=({const char*_tmp162="%s\n\t%s\n%s\n\t%s.";_tag_fat(_tmp162,sizeof(char),15U);});Cyc_aprintf(_tmp5B5,_tag_fat(_tmp161,sizeof(void*),4U));});});Cyc_Tcutil_terr(_tmp5B7,_tmp5B6,_tag_fat(_tmp160,sizeof(void*),0U));});});else{
# 1045
({void*_tmp167=0U;({unsigned _tmp5BA=e1->loc;struct _fat_ptr _tmp5B9=(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp16A=({struct Cyc_String_pa_PrintArg_struct _tmp462;_tmp462.tag=0U,_tmp462.f1=(struct _fat_ptr)((struct _fat_ptr)s0);_tmp462;});struct Cyc_String_pa_PrintArg_struct _tmp16B=({struct Cyc_String_pa_PrintArg_struct _tmp461;_tmp461.tag=0U,_tmp461.f1=(struct _fat_ptr)((struct _fat_ptr)s1);_tmp461;});struct Cyc_String_pa_PrintArg_struct _tmp16C=({struct Cyc_String_pa_PrintArg_struct _tmp460;_tmp460.tag=0U,_tmp460.f1=(struct _fat_ptr)((struct _fat_ptr)s2);_tmp460;});struct Cyc_String_pa_PrintArg_struct _tmp16D=({struct Cyc_String_pa_PrintArg_struct _tmp45F;_tmp45F.tag=0U,_tmp45F.f1=(struct _fat_ptr)((struct _fat_ptr)s3);_tmp45F;});void*_tmp168[4U];_tmp168[0]=& _tmp16A,_tmp168[1]=& _tmp16B,_tmp168[2]=& _tmp16C,_tmp168[3]=& _tmp16D;({struct _fat_ptr _tmp5B8=({const char*_tmp169="%s%s%s%s.";_tag_fat(_tmp169,sizeof(char),10U);});Cyc_aprintf(_tmp5B8,_tag_fat(_tmp168,sizeof(void*),4U));});});Cyc_Tcutil_terr(_tmp5BA,_tmp5B9,_tag_fat(_tmp167,sizeof(void*),0U));});});}
Cyc_Unify_unify((void*)_check_null(e1->topt),t2);
Cyc_Unify_explain_failure();}
# 1037
if(alias_coercion)
# 1051
({struct Cyc_List_List*_tmp5BB=({struct Cyc_List_List*_tmp170=_cycalloc(sizeof(*_tmp170));_tmp170->hd=(void*)arg_cnt,_tmp170->tl=*alias_arg_exps;_tmp170;});*alias_arg_exps=_tmp5BB;});
# 1037
if(
# 1052
Cyc_Tcutil_is_noalias_pointer_or_aggr(t2)&& !Cyc_Tcutil_is_noalias_path(e1))
({void*_tmp171=0U;({unsigned _tmp5BD=e1->loc;struct _fat_ptr _tmp5BC=({const char*_tmp172="Cannot consume non-unique paths; do swap instead";_tag_fat(_tmp172,sizeof(char),49U);});Cyc_Tcutil_terr(_tmp5BD,_tmp5BC,_tag_fat(_tmp171,sizeof(void*),0U));});});
# 1037
es=es->tl;
# 1055
args_info=args_info->tl;
++ arg_cnt;}{
# 1061
int args_already_checked=0;
{struct Cyc_List_List*a=atts;for(0;a != 0;a=a->tl){
void*_stmttmp1D=(void*)a->hd;void*_tmp173=_stmttmp1D;int _tmp176;int _tmp175;enum Cyc_Absyn_Format_Type _tmp174;if(((struct Cyc_Absyn_Format_att_Absyn_Attribute_struct*)_tmp173)->tag == 19U){_LLB: _tmp174=((struct Cyc_Absyn_Format_att_Absyn_Attribute_struct*)_tmp173)->f1;_tmp175=((struct Cyc_Absyn_Format_att_Absyn_Attribute_struct*)_tmp173)->f2;_tmp176=((struct Cyc_Absyn_Format_att_Absyn_Attribute_struct*)_tmp173)->f3;_LLC: {enum Cyc_Absyn_Format_Type ft=_tmp174;int fmt_arg_pos=_tmp175;int arg_start_pos=_tmp176;
# 1065
{struct _handler_cons _tmp177;_push_handler(& _tmp177);{int _tmp179=0;if(setjmp(_tmp177.handler))_tmp179=1;if(!_tmp179){
# 1067
{struct Cyc_Absyn_Exp*fmt_arg=({struct Cyc_Absyn_Exp*(*_tmp5BF)(struct Cyc_List_List*x,int n)=({struct Cyc_Absyn_Exp*(*_tmp17C)(struct Cyc_List_List*x,int n)=(struct Cyc_Absyn_Exp*(*)(struct Cyc_List_List*x,int n))Cyc_List_nth;_tmp17C;});struct Cyc_List_List*_tmp5BE=args;_tmp5BF(_tmp5BE,fmt_arg_pos - 1);});
# 1069
struct Cyc_Core_Opt*fmt_args;
if(arg_start_pos == 0)
fmt_args=0;else{
# 1073
fmt_args=({struct Cyc_Core_Opt*_tmp17A=_cycalloc(sizeof(*_tmp17A));({struct Cyc_List_List*_tmp5C0=Cyc_List_nth_tail(args,arg_start_pos - 1);_tmp17A->v=_tmp5C0;});_tmp17A;});}
args_already_checked=1;{
enum Cyc_Absyn_Format_Type _tmp17B=ft;switch(_tmp17B){case Cyc_Absyn_Printf_ft: _LL10: _LL11:
# 1077
 Cyc_Tcexp_check_format_args(te,fmt_arg,fmt_args,arg_start_pos - 1,alias_arg_exps,c_vararg,Cyc_Formatstr_get_format_types);
# 1080
goto _LLF;case Cyc_Absyn_Scanf_ft: _LL12: _LL13:
 goto _LL15;default: _LL14: _LL15:
# 1083
 Cyc_Tcexp_check_format_args(te,fmt_arg,fmt_args,arg_start_pos - 1,alias_arg_exps,c_vararg,Cyc_Formatstr_get_scanf_types);
# 1086
goto _LLF;}_LLF:;}}
# 1067
;_pop_handler();}else{void*_tmp178=(void*)Cyc_Core_get_exn_thrown();void*_tmp17D=_tmp178;void*_tmp17E;if(((struct Cyc_List_Nth_exn_struct*)_tmp17D)->tag == Cyc_List_Nth){_LL17: _LL18:
# 1088
({void*_tmp17F=0U;({unsigned _tmp5C2=loc;struct _fat_ptr _tmp5C1=({const char*_tmp180="bad format arguments";_tag_fat(_tmp180,sizeof(char),21U);});Cyc_Tcutil_terr(_tmp5C2,_tmp5C1,_tag_fat(_tmp17F,sizeof(void*),0U));});});goto _LL16;}else{_LL19: _tmp17E=_tmp17D;_LL1A: {void*exn=_tmp17E;(int)_rethrow(exn);}}_LL16:;}}}
goto _LLA;}}else{_LLD: _LLE:
 goto _LLA;}_LLA:;}}
# 1093
if(args_info != 0)({void*_tmp181=0U;({unsigned _tmp5C4=loc;struct _fat_ptr _tmp5C3=({const char*_tmp182="too few arguments for function";_tag_fat(_tmp182,sizeof(char),31U);});Cyc_Tcutil_terr(_tmp5C4,_tmp5C3,_tag_fat(_tmp181,sizeof(void*),0U));});});else{
# 1095
if((es != 0 || c_vararg)|| cyc_vararg != 0){
if(c_vararg)
for(0;es != 0;es=es->tl){
Cyc_Tcexp_tcExp(te,0,(struct Cyc_Absyn_Exp*)es->hd);}else{
if(cyc_vararg == 0)
({void*_tmp183=0U;({unsigned _tmp5C6=loc;struct _fat_ptr _tmp5C5=({const char*_tmp184="too many arguments for function";_tag_fat(_tmp184,sizeof(char),32U);});Cyc_Tcutil_terr(_tmp5C6,_tmp5C5,_tag_fat(_tmp183,sizeof(void*),0U));});});else{
# 1102
struct Cyc_Absyn_VarargInfo _stmttmp1E=*cyc_vararg;int _tmp186;void*_tmp185;_LL1C: _tmp185=_stmttmp1E.type;_tmp186=_stmttmp1E.inject;_LL1D: {void*vt=_tmp185;int inject=_tmp186;
struct Cyc_Absyn_VarargCallInfo*vci=({struct Cyc_Absyn_VarargCallInfo*_tmp19A=_cycalloc(sizeof(*_tmp19A));_tmp19A->num_varargs=0,_tmp19A->injectors=0,_tmp19A->vai=cyc_vararg;_tmp19A;});
# 1106
*vararg_call_info=vci;
# 1108
if(!inject)
# 1110
for(0;es != 0;(es=es->tl,arg_cnt ++)){
int alias_coercion=0;
struct Cyc_Absyn_Exp*e1=(struct Cyc_Absyn_Exp*)es->hd;
++ vci->num_varargs;
Cyc_Tcexp_tcExp(te,& vt,e1);
if(!Cyc_Tcutil_coerce_arg(te,e1,vt,& alias_coercion)){
({struct Cyc_String_pa_PrintArg_struct _tmp189=({struct Cyc_String_pa_PrintArg_struct _tmp464;_tmp464.tag=0U,({
struct _fat_ptr _tmp5C7=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(vt));_tmp464.f1=_tmp5C7;});_tmp464;});struct Cyc_String_pa_PrintArg_struct _tmp18A=({struct Cyc_String_pa_PrintArg_struct _tmp463;_tmp463.tag=0U,({struct _fat_ptr _tmp5C8=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string((void*)_check_null(e1->topt)));_tmp463.f1=_tmp5C8;});_tmp463;});void*_tmp187[2U];_tmp187[0]=& _tmp189,_tmp187[1]=& _tmp18A;({unsigned _tmp5CA=loc;struct _fat_ptr _tmp5C9=({const char*_tmp188="vararg requires type %s but argument has type %s";_tag_fat(_tmp188,sizeof(char),49U);});Cyc_Tcutil_terr(_tmp5CA,_tmp5C9,_tag_fat(_tmp187,sizeof(void*),2U));});});
Cyc_Unify_explain_failure();}
# 1115
if(alias_coercion)
# 1121
({struct Cyc_List_List*_tmp5CB=({struct Cyc_List_List*_tmp18B=_cycalloc(sizeof(*_tmp18B));_tmp18B->hd=(void*)arg_cnt,_tmp18B->tl=*alias_arg_exps;_tmp18B;});*alias_arg_exps=_tmp5CB;});
# 1115
if(
# 1122
Cyc_Tcutil_is_noalias_pointer_or_aggr(vt)&& !
Cyc_Tcutil_is_noalias_path(e1))
({void*_tmp18C=0U;({unsigned _tmp5CD=e1->loc;struct _fat_ptr _tmp5CC=({const char*_tmp18D="Cannot consume non-unique paths; do swap instead";_tag_fat(_tmp18D,sizeof(char),49U);});Cyc_Tcutil_terr(_tmp5CD,_tmp5CC,_tag_fat(_tmp18C,sizeof(void*),0U));});});}else{
# 1129
void*_stmttmp1F=Cyc_Tcutil_compress(Cyc_Tcutil_pointer_elt_type(vt));void*_tmp18E=_stmttmp1F;struct Cyc_List_List*_tmp190;struct Cyc_Absyn_Datatypedecl*_tmp18F;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp18E)->tag == 0U){if(((struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp18E)->f1)->tag == 18U){if(((((struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp18E)->f1)->f1).KnownDatatype).tag == 2){_LL1F: _tmp18F=*((((struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp18E)->f1)->f1).KnownDatatype).val;_tmp190=((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp18E)->f2;_LL20: {struct Cyc_Absyn_Datatypedecl*td=_tmp18F;struct Cyc_List_List*targs=_tmp190;
# 1133
struct Cyc_Absyn_Datatypedecl*_tmp191=*Cyc_Tcenv_lookup_datatypedecl(te,loc,td->name);{struct Cyc_Absyn_Datatypedecl*td=_tmp191;
struct Cyc_List_List*fields=0;
if(td->fields == 0)
({struct Cyc_String_pa_PrintArg_struct _tmp194=({struct Cyc_String_pa_PrintArg_struct _tmp465;_tmp465.tag=0U,({
struct _fat_ptr _tmp5CE=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(vt));_tmp465.f1=_tmp5CE;});_tmp465;});void*_tmp192[1U];_tmp192[0]=& _tmp194;({unsigned _tmp5D0=loc;struct _fat_ptr _tmp5CF=({const char*_tmp193="can't inject into abstract datatype %s";_tag_fat(_tmp193,sizeof(char),39U);});Cyc_Tcutil_terr(_tmp5D0,_tmp5CF,_tag_fat(_tmp192,sizeof(void*),1U));});});else{
fields=(struct Cyc_List_List*)((struct Cyc_Core_Opt*)_check_null(td->fields))->v;}
# 1144
({void*_tmp5D1=Cyc_Tcutil_pointer_region(vt);Cyc_Unify_unify(_tmp5D1,Cyc_Tcenv_curr_rgn(te));});{
# 1146
struct Cyc_List_List*inst=Cyc_List_zip(td->tvs,targs);
for(0;es != 0;es=es->tl){
++ vci->num_varargs;{
struct Cyc_Absyn_Exp*e1=(struct Cyc_Absyn_Exp*)es->hd;
# 1151
if(!args_already_checked){
Cyc_Tcexp_tcExp(te,0,e1);
if(Cyc_Tcutil_is_noalias_pointer_or_aggr((void*)_check_null(e1->topt))&& !
Cyc_Tcutil_is_noalias_path(e1))
({void*_tmp195=0U;({unsigned _tmp5D3=e1->loc;struct _fat_ptr _tmp5D2=({const char*_tmp196="Cannot consume non-unique paths; do swap instead";_tag_fat(_tmp196,sizeof(char),49U);});Cyc_Tcutil_terr(_tmp5D3,_tmp5D2,_tag_fat(_tmp195,sizeof(void*),0U));});});}{
# 1151
struct Cyc_Absyn_Datatypefield*f=({
# 1157
struct Cyc_Tcenv_Tenv*_tmp5D7=te;struct Cyc_Absyn_Exp*_tmp5D6=e1;void*_tmp5D5=Cyc_Tcutil_pointer_elt_type(vt);struct Cyc_List_List*_tmp5D4=inst;Cyc_Tcexp_tcInjection(_tmp5D7,_tmp5D6,_tmp5D5,_tmp5D4,fields);});
if(f != 0)
({struct Cyc_List_List*_tmp5D9=({
struct Cyc_List_List*_tmp5D8=vci->injectors;Cyc_List_append(_tmp5D8,({struct Cyc_List_List*_tmp197=_cycalloc(sizeof(*_tmp197));
_tmp197->hd=f,_tmp197->tl=0;_tmp197;}));});
# 1159
vci->injectors=_tmp5D9;});}}}
# 1163
goto _LL1E;}}}}else{goto _LL21;}}else{goto _LL21;}}else{_LL21: _LL22:
({void*_tmp198=0U;({unsigned _tmp5DB=loc;struct _fat_ptr _tmp5DA=({const char*_tmp199="bad inject vararg type";_tag_fat(_tmp199,sizeof(char),23U);});Cyc_Tcutil_terr(_tmp5DB,_tmp5DA,_tag_fat(_tmp198,sizeof(void*),0U));});});goto _LL1E;}_LL1E:;}}}}}}
# 1093
if(*alias_arg_exps == 0)
# 1178 "tcexp.cyc"
Cyc_Tcenv_check_effect_accessible(te,loc,(void*)_check_null(eff));
# 1093 "tcexp.cyc"
return res_typ;}}}else{_LL8: _LL9:
# 1181 "tcexp.cyc"
 return({struct Cyc_String_pa_PrintArg_struct _tmp19D=({struct Cyc_String_pa_PrintArg_struct _tmp466;_tmp466.tag=0U,({struct _fat_ptr _tmp5DC=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t));_tmp466.f1=_tmp5DC;});_tmp466;});void*_tmp19B[1U];_tmp19B[0]=& _tmp19D;({struct Cyc_Tcenv_Tenv*_tmp5E0=te;unsigned _tmp5DF=loc;void**_tmp5DE=topt;struct _fat_ptr _tmp5DD=({const char*_tmp19C="expected pointer to function but found %s";_tag_fat(_tmp19C,sizeof(char),42U);});Cyc_Tcexp_expr_err(_tmp5E0,_tmp5DF,_tmp5DE,_tmp5DD,_tag_fat(_tmp19B,sizeof(void*),1U));});});}_LL5:;}}}else{_LL3: _LL4:
# 1183
 return({struct Cyc_String_pa_PrintArg_struct _tmp1A0=({struct Cyc_String_pa_PrintArg_struct _tmp467;_tmp467.tag=0U,({struct _fat_ptr _tmp5E1=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t));_tmp467.f1=_tmp5E1;});_tmp467;});void*_tmp19E[1U];_tmp19E[0]=& _tmp1A0;({struct Cyc_Tcenv_Tenv*_tmp5E5=te;unsigned _tmp5E4=loc;void**_tmp5E3=topt;struct _fat_ptr _tmp5E2=({const char*_tmp19F="expected pointer to function but found %s";_tag_fat(_tmp19F,sizeof(char),42U);});Cyc_Tcexp_expr_err(_tmp5E5,_tmp5E4,_tmp5E3,_tmp5E2,_tag_fat(_tmp19E,sizeof(void*),1U));});});}_LL0:;}}}
# 306 "tcexp.cyc"
static void*Cyc_Tcexp_tcThrow(struct Cyc_Tcenv_Tenv*te,unsigned loc,void**topt,struct Cyc_Absyn_Exp*e){
# 1188 "tcexp.cyc"
int bogus=0;
void*exception_type=Cyc_Absyn_exn_type();
Cyc_Tcexp_tcExp(te,& exception_type,e);
if(!Cyc_Tcutil_coerce_arg(te,e,exception_type,& bogus))
({struct Cyc_String_pa_PrintArg_struct _tmp1A3=({struct Cyc_String_pa_PrintArg_struct _tmp469;_tmp469.tag=0U,({struct _fat_ptr _tmp5E6=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(exception_type));_tmp469.f1=_tmp5E6;});_tmp469;});struct Cyc_String_pa_PrintArg_struct _tmp1A4=({struct Cyc_String_pa_PrintArg_struct _tmp468;_tmp468.tag=0U,({
struct _fat_ptr _tmp5E7=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string((void*)_check_null(e->topt)));_tmp468.f1=_tmp5E7;});_tmp468;});void*_tmp1A1[2U];_tmp1A1[0]=& _tmp1A3,_tmp1A1[1]=& _tmp1A4;({unsigned _tmp5E9=loc;struct _fat_ptr _tmp5E8=({const char*_tmp1A2="expected %s but found %s";_tag_fat(_tmp1A2,sizeof(char),25U);});Cyc_Tcutil_terr(_tmp5E9,_tmp5E8,_tag_fat(_tmp1A1,sizeof(void*),2U));});});
# 1191
if(topt != 0)
# 1194
return*topt;
# 1191
return
# 1195
 Cyc_Absyn_wildtyp(Cyc_Tcenv_lookup_opt_type_vars(te));}
# 306 "tcexp.cyc"
static void*Cyc_Tcexp_tcInstantiate(struct Cyc_Tcenv_Tenv*te,unsigned loc,void**topt,struct Cyc_Absyn_Exp*e,struct Cyc_List_List*ts){
# 1201 "tcexp.cyc"
Cyc_Tcexp_tcExpNoInst(te,0,e);
# 1203
({void*_tmp5EA=Cyc_Absyn_pointer_expand((void*)_check_null(e->topt),0);e->topt=_tmp5EA;});{
void*t1=Cyc_Tcutil_compress((void*)_check_null(e->topt));
{void*_tmp1A5=t1;void*_tmp1AB;void*_tmp1AA;void*_tmp1A9;void*_tmp1A8;struct Cyc_Absyn_Tqual _tmp1A7;void*_tmp1A6;if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp1A5)->tag == 3U){_LL1: _tmp1A6=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp1A5)->f1).elt_type;_tmp1A7=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp1A5)->f1).elt_tq;_tmp1A8=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp1A5)->f1).ptr_atts).rgn;_tmp1A9=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp1A5)->f1).ptr_atts).nullable;_tmp1AA=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp1A5)->f1).ptr_atts).bounds;_tmp1AB=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp1A5)->f1).ptr_atts).zero_term;_LL2: {void*t0=_tmp1A6;struct Cyc_Absyn_Tqual tq=_tmp1A7;void*rt=_tmp1A8;void*x=_tmp1A9;void*b=_tmp1AA;void*zt=_tmp1AB;
# 1207
{void*_stmttmp20=Cyc_Tcutil_compress(t0);void*_tmp1AC=_stmttmp20;struct Cyc_List_List*_tmp1BB;struct Cyc_List_List*_tmp1BA;struct Cyc_List_List*_tmp1B9;struct Cyc_Absyn_Exp*_tmp1B8;struct Cyc_List_List*_tmp1B7;struct Cyc_Absyn_Exp*_tmp1B6;struct Cyc_List_List*_tmp1B5;struct Cyc_List_List*_tmp1B4;struct Cyc_Absyn_VarargInfo*_tmp1B3;int _tmp1B2;struct Cyc_List_List*_tmp1B1;void*_tmp1B0;struct Cyc_Absyn_Tqual _tmp1AF;void*_tmp1AE;struct Cyc_List_List*_tmp1AD;if(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp1AC)->tag == 5U){_LL6: _tmp1AD=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp1AC)->f1).tvars;_tmp1AE=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp1AC)->f1).effect;_tmp1AF=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp1AC)->f1).ret_tqual;_tmp1B0=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp1AC)->f1).ret_type;_tmp1B1=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp1AC)->f1).args;_tmp1B2=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp1AC)->f1).c_varargs;_tmp1B3=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp1AC)->f1).cyc_varargs;_tmp1B4=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp1AC)->f1).rgn_po;_tmp1B5=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp1AC)->f1).attributes;_tmp1B6=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp1AC)->f1).requires_clause;_tmp1B7=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp1AC)->f1).requires_relns;_tmp1B8=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp1AC)->f1).ensures_clause;_tmp1B9=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp1AC)->f1).ensures_relns;_tmp1BA=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp1AC)->f1).ieffect;_tmp1BB=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp1AC)->f1).oeffect;_LL7: {struct Cyc_List_List*tvars=_tmp1AD;void*eff=_tmp1AE;struct Cyc_Absyn_Tqual rtq=_tmp1AF;void*rtyp=_tmp1B0;struct Cyc_List_List*args=_tmp1B1;int c_varargs=_tmp1B2;struct Cyc_Absyn_VarargInfo*cyc_varargs=_tmp1B3;struct Cyc_List_List*rpo=_tmp1B4;struct Cyc_List_List*atts=_tmp1B5;struct Cyc_Absyn_Exp*req=_tmp1B6;struct Cyc_List_List*req_relns=_tmp1B7;struct Cyc_Absyn_Exp*ens=_tmp1B8;struct Cyc_List_List*ens_relns=_tmp1B9;struct Cyc_List_List*ieff=_tmp1BA;struct Cyc_List_List*oeff=_tmp1BB;
# 1209
struct Cyc_List_List*instantiation=0;
# 1211
for(0;ts != 0 && tvars != 0;(ts=ts->tl,tvars=tvars->tl)){
struct Cyc_Absyn_Kind*k=Cyc_Tcutil_tvar_kind((struct Cyc_Absyn_Tvar*)tvars->hd,& Cyc_Tcutil_bk);
({unsigned _tmp5EE=loc;struct Cyc_Tcenv_Tenv*_tmp5ED=te;struct Cyc_List_List*_tmp5EC=Cyc_Tcenv_lookup_type_vars(te);struct Cyc_Absyn_Kind*_tmp5EB=k;Cyc_Tctyp_check_type(_tmp5EE,_tmp5ED,_tmp5EC,_tmp5EB,1,1,(void*)ts->hd);});
Cyc_Tcutil_check_no_qual(loc,(void*)ts->hd);
instantiation=({struct Cyc_List_List*_tmp1BD=_cycalloc(sizeof(*_tmp1BD));({struct _tuple14*_tmp5EF=({struct _tuple14*_tmp1BC=_cycalloc(sizeof(*_tmp1BC));_tmp1BC->f1=(struct Cyc_Absyn_Tvar*)tvars->hd,_tmp1BC->f2=(void*)ts->hd;_tmp1BC;});_tmp1BD->hd=_tmp5EF;}),_tmp1BD->tl=instantiation;_tmp1BD;});}
# 1217
if(ts != 0)
return({void*_tmp1BE=0U;({struct Cyc_Tcenv_Tenv*_tmp5F3=te;unsigned _tmp5F2=loc;void**_tmp5F1=topt;struct _fat_ptr _tmp5F0=({const char*_tmp1BF="too many type variables in instantiation";_tag_fat(_tmp1BF,sizeof(char),41U);});Cyc_Tcexp_expr_err(_tmp5F3,_tmp5F2,_tmp5F1,_tmp5F0,_tag_fat(_tmp1BE,sizeof(void*),0U));});});
# 1217
if(tvars == 0)
# 1222
;{
# 1217
void*new_fn_typ=({
# 1229
struct Cyc_List_List*_tmp5F4=instantiation;Cyc_Tcutil_substitute(_tmp5F4,(void*)({struct Cyc_Absyn_FnType_Absyn_Type_struct*_tmp1C1=_cycalloc(sizeof(*_tmp1C1));
_tmp1C1->tag=5U,(_tmp1C1->f1).tvars=tvars,(_tmp1C1->f1).effect=eff,(_tmp1C1->f1).ret_tqual=rtq,(_tmp1C1->f1).ret_type=rtyp,(_tmp1C1->f1).args=args,(_tmp1C1->f1).c_varargs=c_varargs,(_tmp1C1->f1).cyc_varargs=cyc_varargs,(_tmp1C1->f1).rgn_po=rpo,(_tmp1C1->f1).attributes=atts,(_tmp1C1->f1).requires_clause=req,(_tmp1C1->f1).requires_relns=req_relns,(_tmp1C1->f1).ensures_clause=ens,(_tmp1C1->f1).ensures_relns=ens_relns,(_tmp1C1->f1).ieffect=ieff,(_tmp1C1->f1).oeffect=oeff;_tmp1C1;}));});
# 1233
return(void*)({struct Cyc_Absyn_PointerType_Absyn_Type_struct*_tmp1C0=_cycalloc(sizeof(*_tmp1C0));_tmp1C0->tag=3U,(_tmp1C0->f1).elt_type=new_fn_typ,(_tmp1C0->f1).elt_tq=tq,((_tmp1C0->f1).ptr_atts).rgn=rt,((_tmp1C0->f1).ptr_atts).nullable=x,((_tmp1C0->f1).ptr_atts).bounds=b,((_tmp1C0->f1).ptr_atts).zero_term=zt,((_tmp1C0->f1).ptr_atts).ptrloc=0;_tmp1C0;});}}}else{_LL8: _LL9:
 goto _LL5;}_LL5:;}
# 1236
goto _LL0;}}else{_LL3: _LL4:
 goto _LL0;}_LL0:;}
# 1239
return({struct Cyc_String_pa_PrintArg_struct _tmp1C4=({struct Cyc_String_pa_PrintArg_struct _tmp46A;_tmp46A.tag=0U,({
struct _fat_ptr _tmp5F5=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t1));_tmp46A.f1=_tmp5F5;});_tmp46A;});void*_tmp1C2[1U];_tmp1C2[0]=& _tmp1C4;({struct Cyc_Tcenv_Tenv*_tmp5F9=te;unsigned _tmp5F8=loc;void**_tmp5F7=topt;struct _fat_ptr _tmp5F6=({const char*_tmp1C3="expecting polymorphic type but found %s";_tag_fat(_tmp1C3,sizeof(char),40U);});Cyc_Tcexp_expr_err(_tmp5F9,_tmp5F8,_tmp5F7,_tmp5F6,_tag_fat(_tmp1C2,sizeof(void*),1U));});});}}
# 306 "tcexp.cyc"
static void*Cyc_Tcexp_tcCast(struct Cyc_Tcenv_Tenv*te,unsigned loc,void**topt,void*t,struct Cyc_Absyn_Exp*e,enum Cyc_Absyn_Coercion*c){
# 1246 "tcexp.cyc"
({unsigned _tmp5FD=loc;struct Cyc_Tcenv_Tenv*_tmp5FC=te;struct Cyc_List_List*_tmp5FB=Cyc_Tcenv_lookup_type_vars(te);struct Cyc_Absyn_Kind*_tmp5FA=
Cyc_Tcenv_abstract_val_ok(te)?& Cyc_Tcutil_tak:& Cyc_Tcutil_tmk;
# 1246
Cyc_Tctyp_check_type(_tmp5FD,_tmp5FC,_tmp5FB,_tmp5FA,1,0,t);});
# 1248
Cyc_Tcutil_check_no_qual(loc,t);
# 1250
Cyc_Tcexp_tcExp(te,& t,e);{
void*t2=(void*)_check_null(e->topt);
if(Cyc_Tcutil_silent_castable(te,loc,t2,t))
*((enum Cyc_Absyn_Coercion*)_check_null(c))=Cyc_Absyn_No_coercion;else{
# 1255
enum Cyc_Absyn_Coercion crc=Cyc_Tcutil_castable(te,loc,t2,t);
if((int)crc != (int)0U)
*((enum Cyc_Absyn_Coercion*)_check_null(c))=crc;else{
if(Cyc_Tcutil_zero_to_null(t,e))
*((enum Cyc_Absyn_Coercion*)_check_null(c))=Cyc_Absyn_No_coercion;else{
# 1262
Cyc_Unify_unify(t2,t);{
void*result=({struct Cyc_String_pa_PrintArg_struct _tmp1C7=({struct Cyc_String_pa_PrintArg_struct _tmp46C;_tmp46C.tag=0U,({
struct _fat_ptr _tmp5FE=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t2));_tmp46C.f1=_tmp5FE;});_tmp46C;});struct Cyc_String_pa_PrintArg_struct _tmp1C8=({struct Cyc_String_pa_PrintArg_struct _tmp46B;_tmp46B.tag=0U,({struct _fat_ptr _tmp5FF=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t));_tmp46B.f1=_tmp5FF;});_tmp46B;});void*_tmp1C5[2U];_tmp1C5[0]=& _tmp1C7,_tmp1C5[1]=& _tmp1C8;({struct Cyc_Tcenv_Tenv*_tmp602=te;unsigned _tmp601=loc;struct _fat_ptr _tmp600=({const char*_tmp1C6="cannot cast %s to %s";_tag_fat(_tmp1C6,sizeof(char),21U);});Cyc_Tcexp_expr_err(_tmp602,_tmp601,& t,_tmp600,_tag_fat(_tmp1C5,sizeof(void*),2U));});});
Cyc_Unify_explain_failure();
return result;}}}}
# 1272
{struct _tuple0 _stmttmp21=({struct _tuple0 _tmp46D;_tmp46D.f1=e->r,({void*_tmp603=Cyc_Tcutil_compress(t);_tmp46D.f2=_tmp603;});_tmp46D;});struct _tuple0 _tmp1C9=_stmttmp21;void*_tmp1CD;void*_tmp1CC;void*_tmp1CB;int _tmp1CA;if(((struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct*)_tmp1C9.f1)->tag == 35U){if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp1C9.f2)->tag == 3U){_LL1: _tmp1CA=(((struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct*)_tmp1C9.f1)->f1).fat_result;_tmp1CB=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp1C9.f2)->f1).ptr_atts).nullable;_tmp1CC=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp1C9.f2)->f1).ptr_atts).bounds;_tmp1CD=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp1C9.f2)->f1).ptr_atts).zero_term;_LL2: {int fat_result=_tmp1CA;void*nbl=_tmp1CB;void*bds=_tmp1CC;void*zt=_tmp1CD;
# 1276
if((fat_result && !Cyc_Tcutil_force_type2bool(0,zt))&& Cyc_Tcutil_force_type2bool(0,nbl)){
struct Cyc_Absyn_Exp*eopt=({void*_tmp604=Cyc_Absyn_bounds_one();Cyc_Tcutil_get_bounds_exp(_tmp604,bds);});
if(eopt != 0){
struct Cyc_Absyn_Exp*e2=e;
if((Cyc_Evexp_eval_const_uint_exp(e2)).f1 == (unsigned)1)
({void*_tmp1CE=0U;({unsigned _tmp606=loc;struct _fat_ptr _tmp605=({const char*_tmp1CF="cast from ? pointer to * pointer will lose size information";_tag_fat(_tmp1CF,sizeof(char),60U);});Cyc_Tcutil_warn(_tmp606,_tmp605,_tag_fat(_tmp1CE,sizeof(void*),0U));});});}}
# 1276
goto _LL0;}}else{goto _LL3;}}else{_LL3: _LL4:
# 1285
 goto _LL0;}_LL0:;}
# 1287
return t;}}
# 306 "tcexp.cyc"
static void*Cyc_Tcexp_tcAddress(struct Cyc_Tcenv_Tenv*te,unsigned loc,struct Cyc_Absyn_Exp*e0,void**topt,struct Cyc_Absyn_Exp*e){
# 1292 "tcexp.cyc"
void**topt2=0;
struct Cyc_Absyn_Tqual tq2=Cyc_Absyn_empty_tqual(0U);
int nullable=0;
if(topt != 0){
void*_stmttmp22=Cyc_Tcutil_compress(*topt);void*_tmp1D0=_stmttmp22;void*_tmp1D3;struct Cyc_Absyn_Tqual _tmp1D2;void**_tmp1D1;if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp1D0)->tag == 3U){_LL1: _tmp1D1=(void**)&(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp1D0)->f1).elt_type;_tmp1D2=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp1D0)->f1).elt_tq;_tmp1D3=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp1D0)->f1).ptr_atts).nullable;_LL2: {void**elttype=_tmp1D1;struct Cyc_Absyn_Tqual tq=_tmp1D2;void*n=_tmp1D3;
# 1298
topt2=elttype;
tq2=tq;
nullable=Cyc_Tcutil_force_type2bool(0,n);
goto _LL0;}}else{_LL3: _LL4:
 goto _LL0;}_LL0:;}
# 1295
({struct Cyc_Tcenv_Tenv*_tmp608=
# 1312
Cyc_Tcenv_enter_abstract_val_ok(Cyc_Tcenv_enter_lhs(Cyc_Tcenv_clear_notreadctxt(te)));
# 1295
void**_tmp607=topt2;Cyc_Tcexp_tcExpNoInst(_tmp608,_tmp607,e);});
# 1315
if(Cyc_Tcutil_is_noalias_pointer_or_aggr((void*)_check_null(e->topt)))
({void*_tmp1D4=0U;({unsigned _tmp60A=e->loc;struct _fat_ptr _tmp609=({const char*_tmp1D5="Cannot take the address of an alias-free path";_tag_fat(_tmp1D5,sizeof(char),46U);});Cyc_Tcutil_terr(_tmp60A,_tmp609,_tag_fat(_tmp1D4,sizeof(void*),0U));});});
# 1315
{void*_stmttmp23=e->r;void*_tmp1D6=_stmttmp23;struct Cyc_Absyn_Exp*_tmp1D8;struct Cyc_Absyn_Exp*_tmp1D7;if(((struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct*)_tmp1D6)->tag == 23U){_LL6: _tmp1D7=((struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct*)_tmp1D6)->f1;_tmp1D8=((struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct*)_tmp1D6)->f2;_LL7: {struct Cyc_Absyn_Exp*e1=_tmp1D7;struct Cyc_Absyn_Exp*e2=_tmp1D8;
# 1323
{void*_stmttmp24=Cyc_Tcutil_compress((void*)_check_null(e1->topt));void*_tmp1D9=_stmttmp24;if(((struct Cyc_Absyn_TupleType_Absyn_Type_struct*)_tmp1D9)->tag == 6U){_LLB: _LLC:
 goto _LLA;}else{_LLD: _LLE:
# 1328
({void*_tmp60B=(Cyc_Absyn_add_exp(e1,e2,0U))->r;e0->r=_tmp60B;});
return Cyc_Tcexp_tcPlus(te,e1,e2);}_LLA:;}
# 1331
goto _LL5;}}else{_LL8: _LL9:
 goto _LL5;}_LL5:;}
# 1336
{void*_stmttmp25=e->r;void*_tmp1DA=_stmttmp25;switch(*((int*)_tmp1DA)){case 21U: if(((struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct*)_tmp1DA)->f3 == 1){_LL10: _LL11:
 goto _LL13;}else{goto _LL14;}case 22U: if(((struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct*)_tmp1DA)->f3 == 1){_LL12: _LL13:
# 1339
({void*_tmp1DB=0U;({unsigned _tmp60D=e->loc;struct _fat_ptr _tmp60C=({const char*_tmp1DC="cannot take the address of a @tagged union member";_tag_fat(_tmp1DC,sizeof(char),50U);});Cyc_Tcutil_terr(_tmp60D,_tmp60C,_tag_fat(_tmp1DB,sizeof(void*),0U));});});goto _LLF;}else{goto _LL14;}default: _LL14: _LL15:
 goto _LLF;}_LLF:;}{
# 1344
struct _tuple15 _stmttmp26=Cyc_Tcutil_addressof_props(e);void*_tmp1DE;int _tmp1DD;_LL17: _tmp1DD=_stmttmp26.f1;_tmp1DE=_stmttmp26.f2;_LL18: {int is_const=_tmp1DD;void*rgn=_tmp1DE;
# 1346
if(Cyc_Tcutil_is_noalias_region(rgn,0))
({void*_tmp1DF=0U;({unsigned _tmp60F=e->loc;struct _fat_ptr _tmp60E=({const char*_tmp1E0="using & would manufacture an alias to an alias-free pointer";_tag_fat(_tmp1E0,sizeof(char),60U);});Cyc_Tcutil_terr(_tmp60F,_tmp60E,_tag_fat(_tmp1DF,sizeof(void*),0U));});});{
# 1346
struct Cyc_Absyn_Tqual tq=
# 1349
Cyc_Absyn_empty_tqual(0U);
if(is_const){
tq.print_const=1;
tq.real_const=1;}{
# 1350
void*t=
# 1355
nullable?
Cyc_Absyn_star_type((void*)_check_null(e->topt),rgn,tq,Cyc_Absyn_false_type):
 Cyc_Absyn_at_type((void*)_check_null(e->topt),rgn,tq,Cyc_Absyn_false_type);
return t;}}}}}
# 306 "tcexp.cyc"
static void*Cyc_Tcexp_tcSizeof(struct Cyc_Tcenv_Tenv*te,unsigned loc,void**topt,void*t){
# 1363 "tcexp.cyc"
if(te->allow_valueof)
# 1366
return Cyc_Absyn_uint_type;
# 1363
({unsigned _tmp612=loc;struct Cyc_Tcenv_Tenv*_tmp611=te;struct Cyc_List_List*_tmp610=
# 1368
Cyc_Tcenv_lookup_type_vars(te);
# 1363
Cyc_Tctyp_check_type(_tmp612,_tmp611,_tmp610,& Cyc_Tcutil_tmk,1,0,t);});
# 1369
Cyc_Tcutil_check_no_qual(loc,t);
if(!Cyc_Evexp_okay_szofarg(t))
({struct Cyc_String_pa_PrintArg_struct _tmp1E3=({struct Cyc_String_pa_PrintArg_struct _tmp46E;_tmp46E.tag=0U,({
struct _fat_ptr _tmp613=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t));_tmp46E.f1=_tmp613;});_tmp46E;});void*_tmp1E1[1U];_tmp1E1[0]=& _tmp1E3;({unsigned _tmp615=loc;struct _fat_ptr _tmp614=({const char*_tmp1E2="sizeof applied to type %s, which has unknown size here";_tag_fat(_tmp1E2,sizeof(char),55U);});Cyc_Tcutil_terr(_tmp615,_tmp614,_tag_fat(_tmp1E1,sizeof(void*),1U));});});
# 1370
if(topt != 0){
# 1374
void*_stmttmp27=Cyc_Tcutil_compress(*topt);void*_tmp1E4=_stmttmp27;void*_tmp1E6;void*_tmp1E5;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp1E4)->tag == 0U){if(((struct Cyc_Absyn_TagCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp1E4)->f1)->tag == 4U){if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp1E4)->f2 != 0){_LL1: _tmp1E5=_tmp1E4;_tmp1E6=(void*)(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp1E4)->f2)->hd;_LL2: {void*tagtyp=_tmp1E5;void*tt=_tmp1E6;
# 1377
struct Cyc_Absyn_Exp*e=Cyc_Absyn_sizeoftype_exp(t,0U);
struct Cyc_Absyn_ValueofType_Absyn_Type_struct*tt2=({struct Cyc_Absyn_ValueofType_Absyn_Type_struct*_tmp1E7=_cycalloc(sizeof(*_tmp1E7));_tmp1E7->tag=9U,_tmp1E7->f1=e;_tmp1E7;});
if(Cyc_Unify_unify(tt,(void*)tt2))return tagtyp;goto _LL0;}}else{goto _LL3;}}else{goto _LL3;}}else{_LL3: _LL4:
# 1381
 goto _LL0;}_LL0:;}
# 1370
return Cyc_Absyn_uint_type;}
# 306 "tcexp.cyc"
void*Cyc_Tcexp_structfield_type(struct _fat_ptr*n,struct Cyc_Absyn_Aggrfield*sf){
# 1387 "tcexp.cyc"
if(Cyc_strcmp((struct _fat_ptr)*n,(struct _fat_ptr)*sf->name)== 0)return sf->type;else{
return 0;}}
# 306 "tcexp.cyc"
static void*Cyc_Tcexp_tcOffsetof(struct Cyc_Tcenv_Tenv*te,unsigned loc,void**topt,void*t_orig,struct Cyc_List_List*fs){
# 1395 "tcexp.cyc"
({unsigned _tmp618=loc;struct Cyc_Tcenv_Tenv*_tmp617=te;struct Cyc_List_List*_tmp616=Cyc_Tcenv_lookup_type_vars(te);Cyc_Tctyp_check_type(_tmp618,_tmp617,_tmp616,& Cyc_Tcutil_tmk,1,0,t_orig);});
Cyc_Tcutil_check_no_qual(loc,t_orig);{
struct Cyc_List_List*l=fs;
void*t=t_orig;
for(0;l != 0;l=l->tl){
void*n=(void*)l->hd;
void*_tmp1E8=n;unsigned _tmp1E9;struct _fat_ptr*_tmp1EA;if(((struct Cyc_Absyn_StructField_Absyn_OffsetofField_struct*)_tmp1E8)->tag == 0U){_LL1: _tmp1EA=((struct Cyc_Absyn_StructField_Absyn_OffsetofField_struct*)_tmp1E8)->f1;_LL2: {struct _fat_ptr*n=_tmp1EA;
# 1403
int bad_type=1;
{void*_stmttmp28=Cyc_Tcutil_compress(t);void*_tmp1EB=_stmttmp28;struct Cyc_List_List*_tmp1EC;struct Cyc_Absyn_Aggrdecl**_tmp1ED;switch(*((int*)_tmp1EB)){case 0U: if(((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp1EB)->f1)->tag == 20U){if(((((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp1EB)->f1)->f1).KnownAggr).tag == 2){_LL6: _tmp1ED=((((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp1EB)->f1)->f1).KnownAggr).val;_LL7: {struct Cyc_Absyn_Aggrdecl**adp=_tmp1ED;
# 1406
if((*adp)->impl == 0)goto _LL5;{void*t2=({
void*(*_tmp61A)(void*(*pred)(struct _fat_ptr*,struct Cyc_Absyn_Aggrfield*),struct _fat_ptr*env,struct Cyc_List_List*x)=({void*(*_tmp1F1)(void*(*pred)(struct _fat_ptr*,struct Cyc_Absyn_Aggrfield*),struct _fat_ptr*env,struct Cyc_List_List*x)=(void*(*)(void*(*pred)(struct _fat_ptr*,struct Cyc_Absyn_Aggrfield*),struct _fat_ptr*env,struct Cyc_List_List*x))Cyc_List_find_c;_tmp1F1;});struct _fat_ptr*_tmp619=n;_tmp61A(Cyc_Tcexp_structfield_type,_tmp619,((struct Cyc_Absyn_AggrdeclImpl*)_check_null((*adp)->impl))->fields);});
if(!((unsigned)t2))
({struct Cyc_String_pa_PrintArg_struct _tmp1F0=({struct Cyc_String_pa_PrintArg_struct _tmp46F;_tmp46F.tag=0U,_tmp46F.f1=(struct _fat_ptr)((struct _fat_ptr)*n);_tmp46F;});void*_tmp1EE[1U];_tmp1EE[0]=& _tmp1F0;({unsigned _tmp61C=loc;struct _fat_ptr _tmp61B=({const char*_tmp1EF="no field of struct/union has name %s";_tag_fat(_tmp1EF,sizeof(char),37U);});Cyc_Tcutil_terr(_tmp61C,_tmp61B,_tag_fat(_tmp1EE,sizeof(void*),1U));});});else{
# 1411
t=t2;}
bad_type=0;
goto _LL5;}}}else{goto _LLA;}}else{goto _LLA;}case 7U: _LL8: _tmp1EC=((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmp1EB)->f2;_LL9: {struct Cyc_List_List*fields=_tmp1EC;
# 1415
void*t2=({void*(*_tmp61E)(void*(*pred)(struct _fat_ptr*,struct Cyc_Absyn_Aggrfield*),struct _fat_ptr*env,struct Cyc_List_List*x)=({void*(*_tmp1F5)(void*(*pred)(struct _fat_ptr*,struct Cyc_Absyn_Aggrfield*),struct _fat_ptr*env,struct Cyc_List_List*x)=(void*(*)(void*(*pred)(struct _fat_ptr*,struct Cyc_Absyn_Aggrfield*),struct _fat_ptr*env,struct Cyc_List_List*x))Cyc_List_find_c;_tmp1F5;});struct _fat_ptr*_tmp61D=n;_tmp61E(Cyc_Tcexp_structfield_type,_tmp61D,fields);});
if(!((unsigned)t2))
({struct Cyc_String_pa_PrintArg_struct _tmp1F4=({struct Cyc_String_pa_PrintArg_struct _tmp470;_tmp470.tag=0U,_tmp470.f1=(struct _fat_ptr)((struct _fat_ptr)*n);_tmp470;});void*_tmp1F2[1U];_tmp1F2[0]=& _tmp1F4;({unsigned _tmp620=loc;struct _fat_ptr _tmp61F=({const char*_tmp1F3="no field of struct/union has name %s";_tag_fat(_tmp1F3,sizeof(char),37U);});Cyc_Tcutil_terr(_tmp620,_tmp61F,_tag_fat(_tmp1F2,sizeof(void*),1U));});});else{
# 1419
t=t2;}
bad_type=0;
goto _LL5;}default: _LLA: _LLB:
 goto _LL5;}_LL5:;}
# 1424
if(bad_type){
if(l == fs)
({struct Cyc_String_pa_PrintArg_struct _tmp1F8=({struct Cyc_String_pa_PrintArg_struct _tmp471;_tmp471.tag=0U,({struct _fat_ptr _tmp621=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t));_tmp471.f1=_tmp621;});_tmp471;});void*_tmp1F6[1U];_tmp1F6[0]=& _tmp1F8;({unsigned _tmp623=loc;struct _fat_ptr _tmp622=({const char*_tmp1F7="%s is not a known struct/union type";_tag_fat(_tmp1F7,sizeof(char),36U);});Cyc_Tcutil_terr(_tmp623,_tmp622,_tag_fat(_tmp1F6,sizeof(void*),1U));});});else{
# 1428
struct _fat_ptr s=({struct Cyc_String_pa_PrintArg_struct _tmp20A=({struct Cyc_String_pa_PrintArg_struct _tmp478;_tmp478.tag=0U,({struct _fat_ptr _tmp624=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t_orig));_tmp478.f1=_tmp624;});_tmp478;});void*_tmp208[1U];_tmp208[0]=& _tmp20A;({struct _fat_ptr _tmp625=({const char*_tmp209="(%s)";_tag_fat(_tmp209,sizeof(char),5U);});Cyc_aprintf(_tmp625,_tag_fat(_tmp208,sizeof(void*),1U));});});
struct Cyc_List_List*x;
for(x=fs;x != l;x=x->tl){
void*_stmttmp29=(void*)((struct Cyc_List_List*)_check_null(x))->hd;void*_tmp1F9=_stmttmp29;unsigned _tmp1FA;struct _fat_ptr*_tmp1FB;if(((struct Cyc_Absyn_StructField_Absyn_OffsetofField_struct*)_tmp1F9)->tag == 0U){_LLD: _tmp1FB=((struct Cyc_Absyn_StructField_Absyn_OffsetofField_struct*)_tmp1F9)->f1;_LLE: {struct _fat_ptr*n=_tmp1FB;
# 1433
s=({struct Cyc_String_pa_PrintArg_struct _tmp1FE=({struct Cyc_String_pa_PrintArg_struct _tmp473;_tmp473.tag=0U,_tmp473.f1=(struct _fat_ptr)((struct _fat_ptr)s);_tmp473;});struct Cyc_String_pa_PrintArg_struct _tmp1FF=({struct Cyc_String_pa_PrintArg_struct _tmp472;_tmp472.tag=0U,_tmp472.f1=(struct _fat_ptr)((struct _fat_ptr)*n);_tmp472;});void*_tmp1FC[2U];_tmp1FC[0]=& _tmp1FE,_tmp1FC[1]=& _tmp1FF;({struct _fat_ptr _tmp626=({const char*_tmp1FD="%s.%s";_tag_fat(_tmp1FD,sizeof(char),6U);});Cyc_aprintf(_tmp626,_tag_fat(_tmp1FC,sizeof(void*),2U));});});goto _LLC;}}else{_LLF: _tmp1FA=((struct Cyc_Absyn_TupleIndex_Absyn_OffsetofField_struct*)_tmp1F9)->f1;_LL10: {unsigned n=_tmp1FA;
# 1435
s=({struct Cyc_String_pa_PrintArg_struct _tmp202=({struct Cyc_String_pa_PrintArg_struct _tmp475;_tmp475.tag=0U,_tmp475.f1=(struct _fat_ptr)((struct _fat_ptr)s);_tmp475;});struct Cyc_Int_pa_PrintArg_struct _tmp203=({struct Cyc_Int_pa_PrintArg_struct _tmp474;_tmp474.tag=1U,_tmp474.f1=(unsigned long)((int)n);_tmp474;});void*_tmp200[2U];_tmp200[0]=& _tmp202,_tmp200[1]=& _tmp203;({struct _fat_ptr _tmp627=({const char*_tmp201="%s.%d";_tag_fat(_tmp201,sizeof(char),6U);});Cyc_aprintf(_tmp627,_tag_fat(_tmp200,sizeof(void*),2U));});});goto _LLC;}}_LLC:;}
# 1437
({struct Cyc_String_pa_PrintArg_struct _tmp206=({struct Cyc_String_pa_PrintArg_struct _tmp477;_tmp477.tag=0U,_tmp477.f1=(struct _fat_ptr)((struct _fat_ptr)s);_tmp477;});struct Cyc_String_pa_PrintArg_struct _tmp207=({struct Cyc_String_pa_PrintArg_struct _tmp476;_tmp476.tag=0U,({struct _fat_ptr _tmp628=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t));_tmp476.f1=_tmp628;});_tmp476;});void*_tmp204[2U];_tmp204[0]=& _tmp206,_tmp204[1]=& _tmp207;({unsigned _tmp62A=loc;struct _fat_ptr _tmp629=({const char*_tmp205="%s == %s is not a struct/union type";_tag_fat(_tmp205,sizeof(char),36U);});Cyc_Tcutil_terr(_tmp62A,_tmp629,_tag_fat(_tmp204,sizeof(void*),2U));});});}}
# 1424
goto _LL0;}}else{_LL3: _tmp1E9=((struct Cyc_Absyn_TupleIndex_Absyn_OffsetofField_struct*)_tmp1E8)->f1;_LL4: {unsigned n=_tmp1E9;
# 1442
int bad_type=1;
{void*_stmttmp2A=Cyc_Tcutil_compress(t);void*_tmp20B=_stmttmp2A;struct Cyc_List_List*_tmp20C;struct Cyc_List_List*_tmp20D;struct Cyc_Absyn_Datatypefield*_tmp20E;struct Cyc_Absyn_Aggrdecl**_tmp20F;switch(*((int*)_tmp20B)){case 0U: switch(*((int*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp20B)->f1)){case 20U: if(((((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp20B)->f1)->f1).KnownAggr).tag == 2){_LL12: _tmp20F=((((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp20B)->f1)->f1).KnownAggr).val;_LL13: {struct Cyc_Absyn_Aggrdecl**adp=_tmp20F;
# 1445
if((*adp)->impl == 0)
goto _LL11;
# 1445
_tmp20D=((struct Cyc_Absyn_AggrdeclImpl*)_check_null((*adp)->impl))->fields;goto _LL15;}}else{goto _LL1A;}case 19U: if(((((struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp20B)->f1)->f1).KnownDatatypefield).tag == 2){_LL18: _tmp20E=(((((struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp20B)->f1)->f1).KnownDatatypefield).val).f2;_LL19: {struct Cyc_Absyn_Datatypefield*tuf=_tmp20E;
# 1465
if(({unsigned _tmp62B=(unsigned)Cyc_List_length(tuf->typs);_tmp62B < n;}))
({struct Cyc_Int_pa_PrintArg_struct _tmp21C=({struct Cyc_Int_pa_PrintArg_struct _tmp47A;_tmp47A.tag=1U,({
unsigned long _tmp62C=(unsigned long)Cyc_List_length(tuf->typs);_tmp47A.f1=_tmp62C;});_tmp47A;});struct Cyc_Int_pa_PrintArg_struct _tmp21D=({struct Cyc_Int_pa_PrintArg_struct _tmp479;_tmp479.tag=1U,_tmp479.f1=(unsigned long)((int)n);_tmp479;});void*_tmp21A[2U];_tmp21A[0]=& _tmp21C,_tmp21A[1]=& _tmp21D;({unsigned _tmp62E=loc;struct _fat_ptr _tmp62D=({const char*_tmp21B="datatype field has too few components: %d < %d";_tag_fat(_tmp21B,sizeof(char),47U);});Cyc_Tcutil_terr(_tmp62E,_tmp62D,_tag_fat(_tmp21A,sizeof(void*),2U));});});else{
# 1469
if(n != (unsigned)0)
t=(*({struct _tuple17*(*_tmp630)(struct Cyc_List_List*x,int n)=({struct _tuple17*(*_tmp21E)(struct Cyc_List_List*x,int n)=(struct _tuple17*(*)(struct Cyc_List_List*x,int n))Cyc_List_nth;_tmp21E;});struct Cyc_List_List*_tmp62F=tuf->typs;_tmp630(_tmp62F,(int)(n - (unsigned)1));})).f2;else{
if(l->tl != 0)
({void*_tmp21F=0U;({unsigned _tmp632=loc;struct _fat_ptr _tmp631=({const char*_tmp220="datatype field index 0 refers to the tag; cannot be indexed through";_tag_fat(_tmp220,sizeof(char),68U);});Cyc_Tcutil_terr(_tmp632,_tmp631,_tag_fat(_tmp21F,sizeof(void*),0U));});});}}
# 1474
bad_type=0;
goto _LL11;}}else{goto _LL1A;}default: goto _LL1A;}case 7U: _LL14: _tmp20D=((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmp20B)->f2;_LL15: {struct Cyc_List_List*fields=_tmp20D;
# 1449
if(({unsigned _tmp633=(unsigned)Cyc_List_length(fields);_tmp633 <= n;}))
({struct Cyc_Int_pa_PrintArg_struct _tmp212=({struct Cyc_Int_pa_PrintArg_struct _tmp47C;_tmp47C.tag=1U,({
unsigned long _tmp634=(unsigned long)Cyc_List_length(fields);_tmp47C.f1=_tmp634;});_tmp47C;});struct Cyc_Int_pa_PrintArg_struct _tmp213=({struct Cyc_Int_pa_PrintArg_struct _tmp47B;_tmp47B.tag=1U,_tmp47B.f1=(unsigned long)((int)n);_tmp47B;});void*_tmp210[2U];_tmp210[0]=& _tmp212,_tmp210[1]=& _tmp213;({unsigned _tmp636=loc;struct _fat_ptr _tmp635=({const char*_tmp211="struct/union has too few components: %d <= %d";_tag_fat(_tmp211,sizeof(char),46U);});Cyc_Tcutil_terr(_tmp636,_tmp635,_tag_fat(_tmp210,sizeof(void*),2U));});});else{
# 1453
t=({struct Cyc_Absyn_Aggrfield*(*_tmp638)(struct Cyc_List_List*x,int n)=({struct Cyc_Absyn_Aggrfield*(*_tmp214)(struct Cyc_List_List*x,int n)=(struct Cyc_Absyn_Aggrfield*(*)(struct Cyc_List_List*x,int n))Cyc_List_nth;_tmp214;});struct Cyc_List_List*_tmp637=fields;_tmp638(_tmp637,(int)n);})->type;}
bad_type=0;
goto _LL11;}case 6U: _LL16: _tmp20C=((struct Cyc_Absyn_TupleType_Absyn_Type_struct*)_tmp20B)->f1;_LL17: {struct Cyc_List_List*l=_tmp20C;
# 1457
if(({unsigned _tmp639=(unsigned)Cyc_List_length(l);_tmp639 <= n;}))
({struct Cyc_Int_pa_PrintArg_struct _tmp217=({struct Cyc_Int_pa_PrintArg_struct _tmp47E;_tmp47E.tag=1U,({
unsigned long _tmp63A=(unsigned long)Cyc_List_length(l);_tmp47E.f1=_tmp63A;});_tmp47E;});struct Cyc_Int_pa_PrintArg_struct _tmp218=({struct Cyc_Int_pa_PrintArg_struct _tmp47D;_tmp47D.tag=1U,_tmp47D.f1=(unsigned long)((int)n);_tmp47D;});void*_tmp215[2U];_tmp215[0]=& _tmp217,_tmp215[1]=& _tmp218;({unsigned _tmp63C=loc;struct _fat_ptr _tmp63B=({const char*_tmp216="tuple has too few components: %d <= %d";_tag_fat(_tmp216,sizeof(char),39U);});Cyc_Tcutil_terr(_tmp63C,_tmp63B,_tag_fat(_tmp215,sizeof(void*),2U));});});else{
# 1461
t=(*({struct _tuple17*(*_tmp63E)(struct Cyc_List_List*x,int n)=({struct _tuple17*(*_tmp219)(struct Cyc_List_List*x,int n)=(struct _tuple17*(*)(struct Cyc_List_List*x,int n))Cyc_List_nth;_tmp219;});struct Cyc_List_List*_tmp63D=l;_tmp63E(_tmp63D,(int)n);})).f2;}
bad_type=0;
goto _LL11;}default: _LL1A: _LL1B:
# 1476
 goto _LL11;}_LL11:;}
# 1478
if(bad_type)
({struct Cyc_String_pa_PrintArg_struct _tmp223=({struct Cyc_String_pa_PrintArg_struct _tmp47F;_tmp47F.tag=0U,({struct _fat_ptr _tmp63F=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t));_tmp47F.f1=_tmp63F;});_tmp47F;});void*_tmp221[1U];_tmp221[0]=& _tmp223;({unsigned _tmp641=loc;struct _fat_ptr _tmp640=({const char*_tmp222="%s is not a known type";_tag_fat(_tmp222,sizeof(char),23U);});Cyc_Tcutil_terr(_tmp641,_tmp640,_tag_fat(_tmp221,sizeof(void*),1U));});});
# 1478
goto _LL0;}}_LL0:;}
# 1483
return Cyc_Absyn_uint_type;}}
# 306 "tcexp.cyc"
static void*Cyc_Tcexp_tcDeref(struct Cyc_Tcenv_Tenv*te_orig,unsigned loc,void**topt,struct Cyc_Absyn_Exp*e){
# 1488 "tcexp.cyc"
struct Cyc_Tcenv_Tenv*te=Cyc_Tcenv_clear_lhs(Cyc_Tcenv_clear_notreadctxt(te_orig));
Cyc_Tcexp_tcExp(te,0,e);{
void*t=Cyc_Tcutil_compress((void*)_check_null(e->topt));
void*_tmp224=t;void*_tmp228;void*_tmp227;void*_tmp226;void*_tmp225;switch(*((int*)_tmp224)){case 1U: _LL1: _LL2: {
# 1493
struct Cyc_List_List*tenv_tvs=Cyc_Tcenv_lookup_type_vars(te);
void*t2=Cyc_Absyn_new_evar(& Cyc_Tcutil_ako,({struct Cyc_Core_Opt*_tmp22B=_cycalloc(sizeof(*_tmp22B));_tmp22B->v=tenv_tvs;_tmp22B;}));
void*rt=Cyc_Absyn_new_evar(& Cyc_Tcutil_trko,({struct Cyc_Core_Opt*_tmp22A=_cycalloc(sizeof(*_tmp22A));_tmp22A->v=tenv_tvs;_tmp22A;}));
void*b=Cyc_Tcutil_any_bounds(tenv_tvs);
void*zt=Cyc_Tcutil_any_bool(tenv_tvs);
struct Cyc_Absyn_PtrAtts atts=({struct Cyc_Absyn_PtrAtts _tmp480;_tmp480.rgn=rt,({void*_tmp642=Cyc_Tcutil_any_bool(tenv_tvs);_tmp480.nullable=_tmp642;}),_tmp480.bounds=b,_tmp480.zero_term=zt,_tmp480.ptrloc=0;_tmp480;});
struct Cyc_Absyn_PointerType_Absyn_Type_struct*new_typ=({struct Cyc_Absyn_PointerType_Absyn_Type_struct*_tmp229=_cycalloc(sizeof(*_tmp229));_tmp229->tag=3U,(_tmp229->f1).elt_type=t2,({struct Cyc_Absyn_Tqual _tmp643=Cyc_Absyn_empty_tqual(0U);(_tmp229->f1).elt_tq=_tmp643;}),(_tmp229->f1).ptr_atts=atts;_tmp229;});
Cyc_Unify_unify(t,(void*)new_typ);
_tmp225=t2;_tmp226=rt;_tmp227=b;_tmp228=zt;goto _LL4;}case 3U: _LL3: _tmp225=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp224)->f1).elt_type;_tmp226=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp224)->f1).ptr_atts).rgn;_tmp227=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp224)->f1).ptr_atts).bounds;_tmp228=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp224)->f1).ptr_atts).zero_term;_LL4: {void*t2=_tmp225;void*rt=_tmp226;void*b=_tmp227;void*zt=_tmp228;
# 1503
Cyc_Tcenv_check_rgn_accessible(te,loc,rt);
Cyc_Tcutil_check_nonzero_bound(loc,b);
if(!Cyc_Tcutil_kind_leq(Cyc_Tcutil_type_kind(t2),& Cyc_Tcutil_tmk)&& !Cyc_Tcenv_abstract_val_ok(te)){
void*_stmttmp2B=Cyc_Tcutil_compress(t2);void*_tmp22C=_stmttmp2B;if(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp22C)->tag == 5U){_LL8: _LL9:
# 1508
 if(Cyc_Tc_aggressive_warn)
({void*_tmp22D=0U;({unsigned _tmp645=loc;struct _fat_ptr _tmp644=({const char*_tmp22E="unnecessary dereference for function type";_tag_fat(_tmp22E,sizeof(char),42U);});Cyc_Tcutil_warn(_tmp645,_tmp644,_tag_fat(_tmp22D,sizeof(void*),0U));});});
# 1508
return t;}else{_LLA: _LLB:
# 1512
({void*_tmp22F=0U;({unsigned _tmp647=loc;struct _fat_ptr _tmp646=({const char*_tmp230="can't dereference abstract pointer type";_tag_fat(_tmp230,sizeof(char),40U);});Cyc_Tcutil_terr(_tmp647,_tmp646,_tag_fat(_tmp22F,sizeof(void*),0U));});});}_LL7:;}
# 1505
return t2;}default: _LL5: _LL6:
# 1517
 return({struct Cyc_String_pa_PrintArg_struct _tmp233=({struct Cyc_String_pa_PrintArg_struct _tmp481;_tmp481.tag=0U,({struct _fat_ptr _tmp648=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t));_tmp481.f1=_tmp648;});_tmp481;});void*_tmp231[1U];_tmp231[0]=& _tmp233;({struct Cyc_Tcenv_Tenv*_tmp64C=te;unsigned _tmp64B=loc;void**_tmp64A=topt;struct _fat_ptr _tmp649=({const char*_tmp232="expecting *, @, or ? type but found %s";_tag_fat(_tmp232,sizeof(char),39U);});Cyc_Tcexp_expr_err(_tmp64C,_tmp64B,_tmp64A,_tmp649,_tag_fat(_tmp231,sizeof(void*),1U));});});}_LL0:;}}
# 306 "tcexp.cyc"
static void*Cyc_Tcexp_tcAggrMember(struct Cyc_Tcenv_Tenv*te,unsigned loc,void**topt,struct Cyc_Absyn_Exp*outer_e,struct Cyc_Absyn_Exp*e,struct _fat_ptr*f,int*is_tagged,int*is_read){
# 1526 "tcexp.cyc"
({struct Cyc_Tcenv_Tenv*_tmp64D=Cyc_Tcenv_enter_abstract_val_ok(te);Cyc_Tcexp_tcExpNoPromote(_tmp64D,0,e);});
# 1528
({int _tmp64E=!Cyc_Tcenv_in_notreadctxt(te);*is_read=_tmp64E;});{
void*_stmttmp2C=Cyc_Tcutil_compress((void*)_check_null(e->topt));void*_tmp234=_stmttmp2C;struct Cyc_List_List*_tmp236;enum Cyc_Absyn_AggrKind _tmp235;struct Cyc_List_List*_tmp238;struct Cyc_Absyn_Aggrdecl*_tmp237;switch(*((int*)_tmp234)){case 0U: if(((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp234)->f1)->tag == 20U){if(((((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp234)->f1)->f1).KnownAggr).tag == 2){_LL1: _tmp237=*((((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp234)->f1)->f1).KnownAggr).val;_tmp238=((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp234)->f2;_LL2: {struct Cyc_Absyn_Aggrdecl*ad=_tmp237;struct Cyc_List_List*ts=_tmp238;
# 1531
struct Cyc_Absyn_Aggrfield*finfo=Cyc_Absyn_lookup_decl_field(ad,f);
if(finfo == 0)
return({struct Cyc_String_pa_PrintArg_struct _tmp23B=({struct Cyc_String_pa_PrintArg_struct _tmp483;_tmp483.tag=0U,({
struct _fat_ptr _tmp64F=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(ad->name));_tmp483.f1=_tmp64F;});_tmp483;});struct Cyc_String_pa_PrintArg_struct _tmp23C=({struct Cyc_String_pa_PrintArg_struct _tmp482;_tmp482.tag=0U,_tmp482.f1=(struct _fat_ptr)((struct _fat_ptr)*f);_tmp482;});void*_tmp239[2U];_tmp239[0]=& _tmp23B,_tmp239[1]=& _tmp23C;({struct Cyc_Tcenv_Tenv*_tmp653=te;unsigned _tmp652=loc;void**_tmp651=topt;struct _fat_ptr _tmp650=({const char*_tmp23A="%s has no %s member";_tag_fat(_tmp23A,sizeof(char),20U);});Cyc_Tcexp_expr_err(_tmp653,_tmp652,_tmp651,_tmp650,_tag_fat(_tmp239,sizeof(void*),2U));});});
# 1532
if(((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->tagged)
# 1536
*is_tagged=1;{
# 1532
void*t2=finfo->type;
# 1538
if(ts != 0){
struct Cyc_List_List*inst=Cyc_List_zip(ad->tvs,ts);
t2=Cyc_Tcutil_substitute(inst,finfo->type);}
# 1538
if(
# 1544
((((int)ad->kind == (int)Cyc_Absyn_UnionA && !((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->tagged)&& !Cyc_Tcutil_is_bits_only_type(t2))&& !Cyc_Tcenv_in_notreadctxt(te))&& finfo->requires_clause == 0)
return({struct Cyc_String_pa_PrintArg_struct _tmp23F=({struct Cyc_String_pa_PrintArg_struct _tmp484;_tmp484.tag=0U,_tmp484.f1=(struct _fat_ptr)((struct _fat_ptr)*f);_tmp484;});void*_tmp23D[1U];_tmp23D[0]=& _tmp23F;({struct Cyc_Tcenv_Tenv*_tmp657=te;unsigned _tmp656=loc;void**_tmp655=topt;struct _fat_ptr _tmp654=({const char*_tmp23E="cannot read union member %s since it is not `bits-only'";_tag_fat(_tmp23E,sizeof(char),56U);});Cyc_Tcexp_expr_err(_tmp657,_tmp656,_tmp655,_tmp654,_tag_fat(_tmp23D,sizeof(void*),1U));});});
# 1538
if(((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->exist_vars != 0){
# 1549
if(!({void*_tmp658=t2;Cyc_Unify_unify(_tmp658,Cyc_Absyn_wildtyp(Cyc_Tcenv_lookup_opt_type_vars(te)));}))
return({struct Cyc_String_pa_PrintArg_struct _tmp242=({struct Cyc_String_pa_PrintArg_struct _tmp485;_tmp485.tag=0U,_tmp485.f1=(struct _fat_ptr)((struct _fat_ptr)*f);_tmp485;});void*_tmp240[1U];_tmp240[0]=& _tmp242;({struct Cyc_Tcenv_Tenv*_tmp65C=te;unsigned _tmp65B=loc;void**_tmp65A=topt;struct _fat_ptr _tmp659=({const char*_tmp241="must use pattern-matching to access field %s\n\tdue to existential type variables.";_tag_fat(_tmp241,sizeof(char),81U);});Cyc_Tcexp_expr_err(_tmp65C,_tmp65B,_tmp65A,_tmp659,_tag_fat(_tmp240,sizeof(void*),1U));});});}
# 1538
return t2;}}}else{goto _LL5;}}else{goto _LL5;}case 7U: _LL3: _tmp235=((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmp234)->f1;_tmp236=((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmp234)->f2;_LL4: {enum Cyc_Absyn_AggrKind k=_tmp235;struct Cyc_List_List*fs=_tmp236;
# 1554
struct Cyc_Absyn_Aggrfield*finfo=Cyc_Absyn_lookup_field(fs,f);
if(finfo == 0)
return({struct Cyc_String_pa_PrintArg_struct _tmp245=({struct Cyc_String_pa_PrintArg_struct _tmp486;_tmp486.tag=0U,_tmp486.f1=(struct _fat_ptr)((struct _fat_ptr)*f);_tmp486;});void*_tmp243[1U];_tmp243[0]=& _tmp245;({struct Cyc_Tcenv_Tenv*_tmp660=te;unsigned _tmp65F=loc;void**_tmp65E=topt;struct _fat_ptr _tmp65D=({const char*_tmp244="type has no %s member";_tag_fat(_tmp244,sizeof(char),22U);});Cyc_Tcexp_expr_err(_tmp660,_tmp65F,_tmp65E,_tmp65D,_tag_fat(_tmp243,sizeof(void*),1U));});});
# 1555
if(
# 1559
(((int)k == (int)1U && !Cyc_Tcutil_is_bits_only_type(finfo->type))&& !Cyc_Tcenv_in_notreadctxt(te))&& finfo->requires_clause == 0)
# 1561
return({struct Cyc_String_pa_PrintArg_struct _tmp248=({struct Cyc_String_pa_PrintArg_struct _tmp487;_tmp487.tag=0U,_tmp487.f1=(struct _fat_ptr)((struct _fat_ptr)*f);_tmp487;});void*_tmp246[1U];_tmp246[0]=& _tmp248;({struct Cyc_Tcenv_Tenv*_tmp664=te;unsigned _tmp663=loc;void**_tmp662=topt;struct _fat_ptr _tmp661=({const char*_tmp247="cannot read union member %s since it is not `bits-only'";_tag_fat(_tmp247,sizeof(char),56U);});Cyc_Tcexp_expr_err(_tmp664,_tmp663,_tmp662,_tmp661,_tag_fat(_tmp246,sizeof(void*),1U));});});
# 1555
return finfo->type;}default: _LL5: _LL6:
# 1564
 return({struct Cyc_String_pa_PrintArg_struct _tmp24B=({struct Cyc_String_pa_PrintArg_struct _tmp488;_tmp488.tag=0U,({
struct _fat_ptr _tmp665=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string((void*)_check_null(e->topt)));_tmp488.f1=_tmp665;});_tmp488;});void*_tmp249[1U];_tmp249[0]=& _tmp24B;({struct Cyc_Tcenv_Tenv*_tmp669=te;unsigned _tmp668=loc;void**_tmp667=topt;struct _fat_ptr _tmp666=({const char*_tmp24A="expecting struct or union, found %s";_tag_fat(_tmp24A,sizeof(char),36U);});Cyc_Tcexp_expr_err(_tmp669,_tmp668,_tmp667,_tmp666,_tag_fat(_tmp249,sizeof(void*),1U));});});}_LL0:;}}
# 306 "tcexp.cyc"
static void*Cyc_Tcexp_tcAggrArrow(struct Cyc_Tcenv_Tenv*te,unsigned loc,void**topt,struct Cyc_Absyn_Exp*e,struct _fat_ptr*f,int*is_tagged,int*is_read){
# 1573 "tcexp.cyc"
({struct Cyc_Tcenv_Tenv*_tmp66A=Cyc_Tcenv_enter_abstract_val_ok(Cyc_Tcenv_clear_lhs(Cyc_Tcenv_clear_notreadctxt(te)));Cyc_Tcexp_tcExp(_tmp66A,0,e);});
# 1575
({int _tmp66B=!Cyc_Tcenv_in_notreadctxt(te);*is_read=_tmp66B;});
{void*_stmttmp2D=Cyc_Tcutil_compress((void*)_check_null(e->topt));void*_tmp24C=_stmttmp2D;void*_tmp250;void*_tmp24F;void*_tmp24E;void*_tmp24D;if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp24C)->tag == 3U){_LL1: _tmp24D=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp24C)->f1).elt_type;_tmp24E=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp24C)->f1).ptr_atts).rgn;_tmp24F=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp24C)->f1).ptr_atts).bounds;_tmp250=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp24C)->f1).ptr_atts).zero_term;_LL2: {void*t2=_tmp24D;void*rt=_tmp24E;void*b=_tmp24F;void*zt=_tmp250;
# 1578
Cyc_Tcutil_check_nonzero_bound(loc,b);
{void*_stmttmp2E=Cyc_Tcutil_compress(t2);void*_tmp251=_stmttmp2E;struct Cyc_List_List*_tmp253;enum Cyc_Absyn_AggrKind _tmp252;struct Cyc_List_List*_tmp255;struct Cyc_Absyn_Aggrdecl*_tmp254;switch(*((int*)_tmp251)){case 0U: if(((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp251)->f1)->tag == 20U){if(((((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp251)->f1)->f1).KnownAggr).tag == 2){_LL6: _tmp254=*((((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp251)->f1)->f1).KnownAggr).val;_tmp255=((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp251)->f2;_LL7: {struct Cyc_Absyn_Aggrdecl*ad=_tmp254;struct Cyc_List_List*ts=_tmp255;
# 1581
struct Cyc_Absyn_Aggrfield*finfo=Cyc_Absyn_lookup_decl_field(ad,f);
if(finfo == 0)
return({struct Cyc_String_pa_PrintArg_struct _tmp258=({struct Cyc_String_pa_PrintArg_struct _tmp48A;_tmp48A.tag=0U,({
struct _fat_ptr _tmp66C=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(ad->name));_tmp48A.f1=_tmp66C;});_tmp48A;});struct Cyc_String_pa_PrintArg_struct _tmp259=({struct Cyc_String_pa_PrintArg_struct _tmp489;_tmp489.tag=0U,_tmp489.f1=(struct _fat_ptr)((struct _fat_ptr)*f);_tmp489;});void*_tmp256[2U];_tmp256[0]=& _tmp258,_tmp256[1]=& _tmp259;({struct Cyc_Tcenv_Tenv*_tmp670=te;unsigned _tmp66F=loc;void**_tmp66E=topt;struct _fat_ptr _tmp66D=({const char*_tmp257="type %s has no %s member";_tag_fat(_tmp257,sizeof(char),25U);});Cyc_Tcexp_expr_err(_tmp670,_tmp66F,_tmp66E,_tmp66D,_tag_fat(_tmp256,sizeof(void*),2U));});});
# 1582
if(((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->tagged)
# 1586
*is_tagged=1;{
# 1582
void*t3=finfo->type;
# 1588
if(ts != 0){
struct Cyc_List_List*inst=Cyc_List_zip(ad->tvs,ts);
t3=Cyc_Tcutil_substitute(inst,finfo->type);}{
# 1588
struct Cyc_Absyn_Kind*t3_kind=
# 1592
Cyc_Tcutil_type_kind(t3);
# 1595
if(Cyc_Tcutil_kind_leq(& Cyc_Tcutil_ak,t3_kind)&& !Cyc_Tcenv_abstract_val_ok(te)){
void*_stmttmp2F=Cyc_Tcutil_compress(t3);void*_tmp25A=_stmttmp2F;if(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp25A)->tag == 4U){_LLD: _LLE:
 goto _LLC;}else{_LLF: _LL10:
# 1599
 return({struct Cyc_String_pa_PrintArg_struct _tmp25D=({struct Cyc_String_pa_PrintArg_struct _tmp48B;_tmp48B.tag=0U,_tmp48B.f1=(struct _fat_ptr)((struct _fat_ptr)*f);_tmp48B;});void*_tmp25B[1U];_tmp25B[0]=& _tmp25D;({struct Cyc_Tcenv_Tenv*_tmp674=te;unsigned _tmp673=loc;void**_tmp672=topt;struct _fat_ptr _tmp671=({const char*_tmp25C="cannot get member %s since its type is abstract";_tag_fat(_tmp25C,sizeof(char),48U);});Cyc_Tcexp_expr_err(_tmp674,_tmp673,_tmp672,_tmp671,_tag_fat(_tmp25B,sizeof(void*),1U));});});}_LLC:;}
# 1595
if(
# 1604
((((int)ad->kind == (int)Cyc_Absyn_UnionA && !((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->tagged)&& !
Cyc_Tcutil_is_bits_only_type(t3))&& !Cyc_Tcenv_in_notreadctxt(te))&& finfo->requires_clause == 0)
# 1607
return({struct Cyc_String_pa_PrintArg_struct _tmp260=({struct Cyc_String_pa_PrintArg_struct _tmp48C;_tmp48C.tag=0U,_tmp48C.f1=(struct _fat_ptr)((struct _fat_ptr)*f);_tmp48C;});void*_tmp25E[1U];_tmp25E[0]=& _tmp260;({struct Cyc_Tcenv_Tenv*_tmp678=te;unsigned _tmp677=loc;void**_tmp676=topt;struct _fat_ptr _tmp675=({const char*_tmp25F="cannot read union member %s since it is not `bits-only'";_tag_fat(_tmp25F,sizeof(char),56U);});Cyc_Tcexp_expr_err(_tmp678,_tmp677,_tmp676,_tmp675,_tag_fat(_tmp25E,sizeof(void*),1U));});});
# 1595
if(((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->exist_vars != 0){
# 1609
if(!({void*_tmp679=t3;Cyc_Unify_unify(_tmp679,Cyc_Absyn_wildtyp(Cyc_Tcenv_lookup_opt_type_vars(te)));}))
return({struct Cyc_String_pa_PrintArg_struct _tmp263=({struct Cyc_String_pa_PrintArg_struct _tmp48D;_tmp48D.tag=0U,_tmp48D.f1=(struct _fat_ptr)((struct _fat_ptr)*f);_tmp48D;});void*_tmp261[1U];_tmp261[0]=& _tmp263;({struct Cyc_Tcenv_Tenv*_tmp67D=te;unsigned _tmp67C=loc;void**_tmp67B=topt;struct _fat_ptr _tmp67A=({const char*_tmp262="must use pattern-matching to access field %s\n\tdue to extistential types";_tag_fat(_tmp262,sizeof(char),72U);});Cyc_Tcexp_expr_err(_tmp67D,_tmp67C,_tmp67B,_tmp67A,_tag_fat(_tmp261,sizeof(void*),1U));});});}
# 1595
return t3;}}}}else{goto _LLA;}}else{goto _LLA;}case 7U: _LL8: _tmp252=((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmp251)->f1;_tmp253=((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmp251)->f2;_LL9: {enum Cyc_Absyn_AggrKind k=_tmp252;struct Cyc_List_List*fs=_tmp253;
# 1616
struct Cyc_Absyn_Aggrfield*finfo=Cyc_Absyn_lookup_field(fs,f);
if(finfo == 0)
return({struct Cyc_String_pa_PrintArg_struct _tmp266=({struct Cyc_String_pa_PrintArg_struct _tmp48E;_tmp48E.tag=0U,_tmp48E.f1=(struct _fat_ptr)((struct _fat_ptr)*f);_tmp48E;});void*_tmp264[1U];_tmp264[0]=& _tmp266;({struct Cyc_Tcenv_Tenv*_tmp681=te;unsigned _tmp680=loc;void**_tmp67F=topt;struct _fat_ptr _tmp67E=({const char*_tmp265="type has no %s field";_tag_fat(_tmp265,sizeof(char),21U);});Cyc_Tcexp_expr_err(_tmp681,_tmp680,_tmp67F,_tmp67E,_tag_fat(_tmp264,sizeof(void*),1U));});});
# 1617
if(
# 1621
((int)k == (int)1U && !Cyc_Tcutil_is_bits_only_type(finfo->type))&& !Cyc_Tcenv_in_notreadctxt(te))
return({struct Cyc_String_pa_PrintArg_struct _tmp269=({struct Cyc_String_pa_PrintArg_struct _tmp48F;_tmp48F.tag=0U,_tmp48F.f1=(struct _fat_ptr)((struct _fat_ptr)*f);_tmp48F;});void*_tmp267[1U];_tmp267[0]=& _tmp269;({struct Cyc_Tcenv_Tenv*_tmp685=te;unsigned _tmp684=loc;void**_tmp683=topt;struct _fat_ptr _tmp682=({const char*_tmp268="cannot read union member %s since it is not `bits-only'";_tag_fat(_tmp268,sizeof(char),56U);});Cyc_Tcexp_expr_err(_tmp685,_tmp684,_tmp683,_tmp682,_tag_fat(_tmp267,sizeof(void*),1U));});});
# 1617
return finfo->type;}default: _LLA: _LLB:
# 1624
 goto _LL5;}_LL5:;}
# 1626
goto _LL0;}}else{_LL3: _LL4:
 goto _LL0;}_LL0:;}
# 1629
return({struct Cyc_String_pa_PrintArg_struct _tmp26C=({struct Cyc_String_pa_PrintArg_struct _tmp490;_tmp490.tag=0U,({
struct _fat_ptr _tmp686=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string((void*)_check_null(e->topt)));_tmp490.f1=_tmp686;});_tmp490;});void*_tmp26A[1U];_tmp26A[0]=& _tmp26C;({struct Cyc_Tcenv_Tenv*_tmp68A=te;unsigned _tmp689=loc;void**_tmp688=topt;struct _fat_ptr _tmp687=({const char*_tmp26B="expecting struct or union pointer, found %s";_tag_fat(_tmp26B,sizeof(char),44U);});Cyc_Tcexp_expr_err(_tmp68A,_tmp689,_tmp688,_tmp687,_tag_fat(_tmp26A,sizeof(void*),1U));});});}
# 306 "tcexp.cyc"
static void*Cyc_Tcexp_ithTupleType(struct Cyc_Tcenv_Tenv*te,unsigned loc,struct Cyc_List_List*ts,struct Cyc_Absyn_Exp*index){
# 1636 "tcexp.cyc"
struct _tuple16 _stmttmp30=Cyc_Evexp_eval_const_uint_exp(index);int _tmp26E;unsigned _tmp26D;_LL1: _tmp26D=_stmttmp30.f1;_tmp26E=_stmttmp30.f2;_LL2: {unsigned i=_tmp26D;int known=_tmp26E;
if(!known)
return({void*_tmp26F=0U;({struct Cyc_Tcenv_Tenv*_tmp68D=te;unsigned _tmp68C=loc;struct _fat_ptr _tmp68B=({const char*_tmp270="tuple projection cannot use sizeof or offsetof";_tag_fat(_tmp270,sizeof(char),47U);});Cyc_Tcexp_expr_err(_tmp68D,_tmp68C,0,_tmp68B,_tag_fat(_tmp26F,sizeof(void*),0U));});});{
# 1637
struct _handler_cons _tmp271;_push_handler(& _tmp271);{int _tmp273=0;if(setjmp(_tmp271.handler))_tmp273=1;if(!_tmp273){
# 1641
{void*_tmp275=(*({struct _tuple17*(*_tmp68F)(struct Cyc_List_List*x,int n)=({struct _tuple17*(*_tmp274)(struct Cyc_List_List*x,int n)=(struct _tuple17*(*)(struct Cyc_List_List*x,int n))Cyc_List_nth;_tmp274;});struct Cyc_List_List*_tmp68E=ts;_tmp68F(_tmp68E,(int)i);})).f2;_npop_handler(0U);return _tmp275;};_pop_handler();}else{void*_tmp272=(void*)Cyc_Core_get_exn_thrown();void*_tmp276=_tmp272;void*_tmp277;if(((struct Cyc_List_Nth_exn_struct*)_tmp276)->tag == Cyc_List_Nth){_LL4: _LL5:
# 1643
 return({struct Cyc_Int_pa_PrintArg_struct _tmp27A=({struct Cyc_Int_pa_PrintArg_struct _tmp492;_tmp492.tag=1U,_tmp492.f1=(unsigned long)((int)i);_tmp492;});struct Cyc_Int_pa_PrintArg_struct _tmp27B=({struct Cyc_Int_pa_PrintArg_struct _tmp491;_tmp491.tag=1U,({
unsigned long _tmp690=(unsigned long)Cyc_List_length(ts);_tmp491.f1=_tmp690;});_tmp491;});void*_tmp278[2U];_tmp278[0]=& _tmp27A,_tmp278[1]=& _tmp27B;({struct Cyc_Tcenv_Tenv*_tmp693=te;unsigned _tmp692=loc;struct _fat_ptr _tmp691=({const char*_tmp279="index is %d but tuple has only %d fields";_tag_fat(_tmp279,sizeof(char),41U);});Cyc_Tcexp_expr_err(_tmp693,_tmp692,0,_tmp691,_tag_fat(_tmp278,sizeof(void*),2U));});});}else{_LL6: _tmp277=_tmp276;_LL7: {void*exn=_tmp277;(int)_rethrow(exn);}}_LL3:;}}}}}
# 306 "tcexp.cyc"
static void*Cyc_Tcexp_tcSubscript(struct Cyc_Tcenv_Tenv*te_orig,unsigned loc,void**topt,struct Cyc_Absyn_Exp*e1,struct Cyc_Absyn_Exp*e2){
# 1650 "tcexp.cyc"
struct Cyc_Tcenv_Tenv*te=Cyc_Tcenv_clear_lhs(Cyc_Tcenv_clear_notreadctxt(te_orig));
({struct Cyc_Tcenv_Tenv*_tmp694=Cyc_Tcenv_clear_abstract_val_ok(te);Cyc_Tcexp_tcExp(_tmp694,0,e1);});
({struct Cyc_Tcenv_Tenv*_tmp695=Cyc_Tcenv_clear_abstract_val_ok(te);Cyc_Tcexp_tcExp(_tmp695,0,e2);});{
void*t1=Cyc_Tcutil_compress((void*)_check_null(e1->topt));
void*t2=Cyc_Tcutil_compress((void*)_check_null(e2->topt));
if(!Cyc_Tcutil_coerce_sint_type(e2))
return({struct Cyc_String_pa_PrintArg_struct _tmp27E=({struct Cyc_String_pa_PrintArg_struct _tmp493;_tmp493.tag=0U,({
struct _fat_ptr _tmp696=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t2));_tmp493.f1=_tmp696;});_tmp493;});void*_tmp27C[1U];_tmp27C[0]=& _tmp27E;({struct Cyc_Tcenv_Tenv*_tmp69A=te;unsigned _tmp699=e2->loc;void**_tmp698=topt;struct _fat_ptr _tmp697=({const char*_tmp27D="expecting int subscript, found %s";_tag_fat(_tmp27D,sizeof(char),34U);});Cyc_Tcexp_expr_err(_tmp69A,_tmp699,_tmp698,_tmp697,_tag_fat(_tmp27C,sizeof(void*),1U));});});{
# 1655
void*_tmp27F=t1;struct Cyc_List_List*_tmp280;void*_tmp285;void*_tmp284;void*_tmp283;struct Cyc_Absyn_Tqual _tmp282;void*_tmp281;switch(*((int*)_tmp27F)){case 3U: _LL1: _tmp281=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp27F)->f1).elt_type;_tmp282=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp27F)->f1).elt_tq;_tmp283=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp27F)->f1).ptr_atts).rgn;_tmp284=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp27F)->f1).ptr_atts).bounds;_tmp285=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp27F)->f1).ptr_atts).zero_term;_LL2: {void*t=_tmp281;struct Cyc_Absyn_Tqual tq=_tmp282;void*rt=_tmp283;void*b=_tmp284;void*zt=_tmp285;
# 1664
if(Cyc_Tcutil_force_type2bool(0,zt)){
int emit_warning=0;
struct Cyc_Absyn_Exp*eopt=Cyc_Tcutil_get_bounds_exp(Cyc_Absyn_fat_bound_type,b);
if(eopt != 0){
struct Cyc_Absyn_Exp*e3=eopt;
struct _tuple16 _stmttmp31=Cyc_Evexp_eval_const_uint_exp(e3);int _tmp287;unsigned _tmp286;_LL8: _tmp286=_stmttmp31.f1;_tmp287=_stmttmp31.f2;_LL9: {unsigned i=_tmp286;int known_i=_tmp287;
if(known_i && i == (unsigned)1)emit_warning=1;if(
Cyc_Tcutil_is_const_exp(e2)){
struct _tuple16 _stmttmp32=Cyc_Evexp_eval_const_uint_exp(e2);int _tmp289;unsigned _tmp288;_LLB: _tmp288=_stmttmp32.f1;_tmp289=_stmttmp32.f2;_LLC: {unsigned j=_tmp288;int known_j=_tmp289;
if(known_j){
struct _tuple16 _stmttmp33=Cyc_Evexp_eval_const_uint_exp(e3);int _tmp28B;unsigned _tmp28A;_LLE: _tmp28A=_stmttmp33.f1;_tmp28B=_stmttmp33.f2;_LLF: {unsigned j=_tmp28A;int knownj=_tmp28B;
if((known_i && j > i)&& i != (unsigned)1)
({void*_tmp28C=0U;({unsigned _tmp69C=loc;struct _fat_ptr _tmp69B=({const char*_tmp28D="subscript will fail at run-time";_tag_fat(_tmp28D,sizeof(char),32U);});Cyc_Tcutil_terr(_tmp69C,_tmp69B,_tag_fat(_tmp28C,sizeof(void*),0U));});});}}}}}}
# 1667
if(emit_warning)
# 1681
({void*_tmp28E=0U;({unsigned _tmp69E=e2->loc;struct _fat_ptr _tmp69D=({const char*_tmp28F="subscript on thin, zero-terminated pointer could be expensive.";_tag_fat(_tmp28F,sizeof(char),63U);});Cyc_Tcutil_warn(_tmp69E,_tmp69D,_tag_fat(_tmp28E,sizeof(void*),0U));});});}else{
# 1684
if(Cyc_Tcutil_is_const_exp(e2)){
struct _tuple16 _stmttmp34=Cyc_Evexp_eval_const_uint_exp(e2);int _tmp291;unsigned _tmp290;_LL11: _tmp290=_stmttmp34.f1;_tmp291=_stmttmp34.f2;_LL12: {unsigned i=_tmp290;int known=_tmp291;
if(known)
# 1689
({unsigned _tmp6A1=loc;unsigned _tmp6A0=i;void*_tmp69F=b;Cyc_Tcutil_check_bound(_tmp6A1,_tmp6A0,_tmp69F,Cyc_Tcenv_abstract_val_ok(te));});}}else{
# 1692
if(Cyc_Tcutil_is_bound_one(b)&& !Cyc_Tcutil_force_type2bool(0,zt))
({void*_tmp292=0U;({unsigned _tmp6A3=e1->loc;struct _fat_ptr _tmp6A2=({const char*_tmp293="subscript applied to pointer to one object";_tag_fat(_tmp293,sizeof(char),43U);});Cyc_Tcutil_warn(_tmp6A3,_tmp6A2,_tag_fat(_tmp292,sizeof(void*),0U));});});
# 1692
Cyc_Tcutil_check_nonzero_bound(loc,b);}}
# 1697
Cyc_Tcenv_check_rgn_accessible(te,loc,rt);
if(!Cyc_Tcutil_kind_leq(Cyc_Tcutil_type_kind(t),& Cyc_Tcutil_tmk)&& !Cyc_Tcenv_abstract_val_ok(te))
({void*_tmp294=0U;({unsigned _tmp6A5=e1->loc;struct _fat_ptr _tmp6A4=({const char*_tmp295="can't subscript an abstract pointer";_tag_fat(_tmp295,sizeof(char),36U);});Cyc_Tcutil_terr(_tmp6A5,_tmp6A4,_tag_fat(_tmp294,sizeof(void*),0U));});});
# 1698
return t;}case 6U: _LL3: _tmp280=((struct Cyc_Absyn_TupleType_Absyn_Type_struct*)_tmp27F)->f1;_LL4: {struct Cyc_List_List*ts=_tmp280;
# 1701
return Cyc_Tcexp_ithTupleType(te,loc,ts,e2);}default: _LL5: _LL6:
 return({struct Cyc_String_pa_PrintArg_struct _tmp298=({struct Cyc_String_pa_PrintArg_struct _tmp494;_tmp494.tag=0U,({struct _fat_ptr _tmp6A6=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t1));_tmp494.f1=_tmp6A6;});_tmp494;});void*_tmp296[1U];_tmp296[0]=& _tmp298;({struct Cyc_Tcenv_Tenv*_tmp6AA=te;unsigned _tmp6A9=loc;void**_tmp6A8=topt;struct _fat_ptr _tmp6A7=({const char*_tmp297="subscript applied to %s";_tag_fat(_tmp297,sizeof(char),24U);});Cyc_Tcexp_expr_err(_tmp6AA,_tmp6A9,_tmp6A8,_tmp6A7,_tag_fat(_tmp296,sizeof(void*),1U));});});}_LL0:;}}}
# 306 "tcexp.cyc"
static void*Cyc_Tcexp_tcTuple(struct Cyc_Tcenv_Tenv*te,unsigned loc,void**topt,struct Cyc_List_List*es){
# 1708 "tcexp.cyc"
int done=0;
struct Cyc_List_List*fields=0;
if(topt != 0){
void*_stmttmp35=Cyc_Tcutil_compress(*topt);void*_tmp299=_stmttmp35;struct Cyc_List_List*_tmp29A;if(((struct Cyc_Absyn_TupleType_Absyn_Type_struct*)_tmp299)->tag == 6U){_LL1: _tmp29A=((struct Cyc_Absyn_TupleType_Absyn_Type_struct*)_tmp299)->f1;_LL2: {struct Cyc_List_List*ts=_tmp29A;
# 1713
if(({int _tmp6AB=Cyc_List_length(ts);_tmp6AB != Cyc_List_length(es);}))
# 1716
goto _LL0;
# 1713
for(0;es != 0;(
# 1718
es=es->tl,ts=ts->tl)){
int bogus=0;
void*topt2=(*((struct _tuple17*)((struct Cyc_List_List*)_check_null(ts))->hd)).f2;
({struct Cyc_Tcenv_Tenv*_tmp6AC=Cyc_Tcenv_clear_abstract_val_ok(te);Cyc_Tcexp_tcExpInitializer(_tmp6AC,& topt2,(struct Cyc_Absyn_Exp*)es->hd);});
# 1723
Cyc_Tcutil_coerce_arg(te,(struct Cyc_Absyn_Exp*)es->hd,(*((struct _tuple17*)ts->hd)).f2,& bogus);
fields=({struct Cyc_List_List*_tmp29C=_cycalloc(sizeof(*_tmp29C));({struct _tuple17*_tmp6AD=({struct _tuple17*_tmp29B=_cycalloc(sizeof(*_tmp29B));_tmp29B->f1=(*((struct _tuple17*)ts->hd)).f1,_tmp29B->f2=(void*)_check_null(((struct Cyc_Absyn_Exp*)es->hd)->topt);_tmp29B;});_tmp29C->hd=_tmp6AD;}),_tmp29C->tl=fields;_tmp29C;});}
# 1726
done=1;
goto _LL0;}}else{_LL3: _LL4:
 goto _LL0;}_LL0:;}
# 1710
if(!done)
# 1731
for(0;es != 0;es=es->tl){
({struct Cyc_Tcenv_Tenv*_tmp6AE=Cyc_Tcenv_clear_abstract_val_ok(te);Cyc_Tcexp_tcExpInitializer(_tmp6AE,0,(struct Cyc_Absyn_Exp*)es->hd);});
fields=({struct Cyc_List_List*_tmp29E=_cycalloc(sizeof(*_tmp29E));({struct _tuple17*_tmp6B0=({struct _tuple17*_tmp29D=_cycalloc(sizeof(*_tmp29D));({struct Cyc_Absyn_Tqual _tmp6AF=Cyc_Absyn_empty_tqual(0U);_tmp29D->f1=_tmp6AF;}),_tmp29D->f2=(void*)_check_null(((struct Cyc_Absyn_Exp*)es->hd)->topt);_tmp29D;});_tmp29E->hd=_tmp6B0;}),_tmp29E->tl=fields;_tmp29E;});}
# 1710
return(void*)({struct Cyc_Absyn_TupleType_Absyn_Type_struct*_tmp29F=_cycalloc(sizeof(*_tmp29F));
# 1735
_tmp29F->tag=6U,({struct Cyc_List_List*_tmp6B1=Cyc_List_imp_rev(fields);_tmp29F->f1=_tmp6B1;});_tmp29F;});}
# 306 "tcexp.cyc"
static void*Cyc_Tcexp_tcCompoundLit(struct Cyc_Tcenv_Tenv*te,unsigned loc,struct Cyc_Absyn_Exp*orig_exp,void**topt,struct _tuple9*targ,struct Cyc_List_List*des){
# 1744 "tcexp.cyc"
void*_tmp2A0;_LL1: _tmp2A0=targ->f3;_LL2: {void*t=_tmp2A0;
({unsigned _tmp6B5=loc;struct Cyc_Tcenv_Tenv*_tmp6B4=te;struct Cyc_List_List*_tmp6B3=Cyc_Tcenv_lookup_type_vars(te);struct Cyc_Absyn_Kind*_tmp6B2=
Cyc_Tcenv_abstract_val_ok(te)?& Cyc_Tcutil_tak:& Cyc_Tcutil_tmk;
# 1745
Cyc_Tctyp_check_type(_tmp6B5,_tmp6B4,_tmp6B3,_tmp6B2,1,0,t);});
# 1747
({void*_tmp6B6=(void*)({struct Cyc_Absyn_UnresolvedMem_e_Absyn_Raw_exp_struct*_tmp2A1=_cycalloc(sizeof(*_tmp2A1));_tmp2A1->tag=37U,_tmp2A1->f1=0,_tmp2A1->f2=des;_tmp2A1;});orig_exp->r=_tmp6B6;});
Cyc_Tcexp_resolve_unresolved_mem(te,loc,& t,orig_exp,des);
Cyc_Tcexp_tcExpNoInst(te,topt,orig_exp);
return(void*)_check_null(orig_exp->topt);}}struct _tuple18{struct Cyc_List_List*f1;struct Cyc_Absyn_Exp*f2;};
# 306 "tcexp.cyc"
static void*Cyc_Tcexp_tcArray(struct Cyc_Tcenv_Tenv*te,unsigned loc,void**elt_topt,struct Cyc_Absyn_Tqual*elt_tqopt,int zero_term,struct Cyc_List_List*des){
# 1763 "tcexp.cyc"
void*res_t2;
int num_es=Cyc_List_length(des);
struct Cyc_List_List*es=({struct Cyc_List_List*(*_tmp6B8)(struct Cyc_Absyn_Exp*(*f)(struct _tuple18*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp2B5)(struct Cyc_Absyn_Exp*(*f)(struct _tuple18*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_Absyn_Exp*(*f)(struct _tuple18*),struct Cyc_List_List*x))Cyc_List_map;_tmp2B5;});struct Cyc_Absyn_Exp*(*_tmp6B7)(struct _tuple18*)=({struct Cyc_Absyn_Exp*(*_tmp2B6)(struct _tuple18*)=(struct Cyc_Absyn_Exp*(*)(struct _tuple18*))Cyc_Core_snd;_tmp2B6;});_tmp6B8(_tmp6B7,des);});
void*res=Cyc_Absyn_new_evar(& Cyc_Tcutil_tmko,Cyc_Tcenv_lookup_opt_type_vars(te));
struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*sz_rexp=({struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*_tmp2B4=_cycalloc(sizeof(*_tmp2B4));_tmp2B4->tag=0U,({union Cyc_Absyn_Cnst _tmp6B9=Cyc_Absyn_Int_c(Cyc_Absyn_Unsigned,num_es);_tmp2B4->f1=_tmp6B9;});_tmp2B4;});
struct Cyc_Absyn_Exp*sz_exp=Cyc_Absyn_new_exp((void*)sz_rexp,loc);
# 1771
if(zero_term){
struct Cyc_Absyn_Exp*e=({struct Cyc_Absyn_Exp*(*_tmp6BB)(struct Cyc_List_List*x,int n)=({struct Cyc_Absyn_Exp*(*_tmp2A4)(struct Cyc_List_List*x,int n)=(struct Cyc_Absyn_Exp*(*)(struct Cyc_List_List*x,int n))Cyc_List_nth;_tmp2A4;});struct Cyc_List_List*_tmp6BA=es;_tmp6BB(_tmp6BA,num_es - 1);});
if(!Cyc_Tcutil_is_zero(e))
({void*_tmp2A2=0U;({unsigned _tmp6BD=e->loc;struct _fat_ptr _tmp6BC=({const char*_tmp2A3="zero-terminated array doesn't end with zero.";_tag_fat(_tmp2A3,sizeof(char),45U);});Cyc_Tcutil_terr(_tmp6BD,_tmp6BC,_tag_fat(_tmp2A2,sizeof(void*),0U));});});}
# 1771
sz_exp->topt=Cyc_Absyn_uint_type;
# 1777
res_t2=({void*_tmp6C0=res;struct Cyc_Absyn_Tqual _tmp6BF=
(unsigned)elt_tqopt?*elt_tqopt: Cyc_Absyn_empty_tqual(0U);
# 1777
struct Cyc_Absyn_Exp*_tmp6BE=sz_exp;Cyc_Absyn_array_type(_tmp6C0,_tmp6BF,_tmp6BE,
# 1779
zero_term?Cyc_Absyn_true_type: Cyc_Absyn_false_type,0U);});
# 1781
{struct Cyc_List_List*es2=es;for(0;es2 != 0;es2=es2->tl){
Cyc_Tcexp_tcExpInitializer(te,elt_topt,(struct Cyc_Absyn_Exp*)es2->hd);}}
# 1784
if(!Cyc_Tcutil_coerce_list(te,res,es))
# 1786
({struct Cyc_String_pa_PrintArg_struct _tmp2A7=({struct Cyc_String_pa_PrintArg_struct _tmp495;_tmp495.tag=0U,({
struct _fat_ptr _tmp6C1=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(res));_tmp495.f1=_tmp6C1;});_tmp495;});void*_tmp2A5[1U];_tmp2A5[0]=& _tmp2A7;({unsigned _tmp6C3=((struct Cyc_Absyn_Exp*)((struct Cyc_List_List*)_check_null(es))->hd)->loc;struct _fat_ptr _tmp6C2=({const char*_tmp2A6="elements of array do not all have the same type (%s)";_tag_fat(_tmp2A6,sizeof(char),53U);});Cyc_Tcutil_terr(_tmp6C3,_tmp6C2,_tag_fat(_tmp2A5,sizeof(void*),1U));});});
# 1784
{
# 1789
int offset=0;
# 1784
for(0;des != 0;(
# 1789
offset ++,des=des->tl)){
struct Cyc_List_List*ds=(*((struct _tuple18*)des->hd)).f1;
if(ds != 0){
# 1794
void*_stmttmp36=(void*)ds->hd;void*_tmp2A8=_stmttmp36;struct Cyc_Absyn_Exp*_tmp2A9;if(((struct Cyc_Absyn_FieldName_Absyn_Designator_struct*)_tmp2A8)->tag == 1U){_LL1: _LL2:
# 1796
({void*_tmp2AA=0U;({unsigned _tmp6C5=loc;struct _fat_ptr _tmp6C4=({const char*_tmp2AB="only array index designators are supported";_tag_fat(_tmp2AB,sizeof(char),43U);});Cyc_Tcutil_terr(_tmp6C5,_tmp6C4,_tag_fat(_tmp2AA,sizeof(void*),0U));});});
goto _LL0;}else{_LL3: _tmp2A9=((struct Cyc_Absyn_ArrayElement_Absyn_Designator_struct*)_tmp2A8)->f1;_LL4: {struct Cyc_Absyn_Exp*e=_tmp2A9;
# 1799
Cyc_Tcexp_tcExpInitializer(te,0,e);{
struct _tuple16 _stmttmp37=Cyc_Evexp_eval_const_uint_exp(e);int _tmp2AD;unsigned _tmp2AC;_LL6: _tmp2AC=_stmttmp37.f1;_tmp2AD=_stmttmp37.f2;_LL7: {unsigned i=_tmp2AC;int known=_tmp2AD;
if(!known)
({void*_tmp2AE=0U;({unsigned _tmp6C7=e->loc;struct _fat_ptr _tmp6C6=({const char*_tmp2AF="index designator cannot use sizeof or offsetof";_tag_fat(_tmp2AF,sizeof(char),47U);});Cyc_Tcutil_terr(_tmp6C7,_tmp6C6,_tag_fat(_tmp2AE,sizeof(void*),0U));});});else{
if(i != (unsigned)offset)
({struct Cyc_Int_pa_PrintArg_struct _tmp2B2=({struct Cyc_Int_pa_PrintArg_struct _tmp497;_tmp497.tag=1U,_tmp497.f1=(unsigned long)offset;_tmp497;});struct Cyc_Int_pa_PrintArg_struct _tmp2B3=({struct Cyc_Int_pa_PrintArg_struct _tmp496;_tmp496.tag=1U,_tmp496.f1=(unsigned long)((int)i);_tmp496;});void*_tmp2B0[2U];_tmp2B0[0]=& _tmp2B2,_tmp2B0[1]=& _tmp2B3;({unsigned _tmp6C9=e->loc;struct _fat_ptr _tmp6C8=({const char*_tmp2B1="expecting index designator %d but found %d";_tag_fat(_tmp2B1,sizeof(char),43U);});Cyc_Tcutil_terr(_tmp6C9,_tmp6C8,_tag_fat(_tmp2B0,sizeof(void*),2U));});});}
# 1801
goto _LL0;}}}}_LL0:;}}}
# 1810
return res_t2;}
# 306 "tcexp.cyc"
static void*Cyc_Tcexp_tcComprehension(struct Cyc_Tcenv_Tenv*te,unsigned loc,void**topt,struct Cyc_Absyn_Vardecl*vd,struct Cyc_Absyn_Exp*bound,struct Cyc_Absyn_Exp*body,int*is_zero_term){
# 1817 "tcexp.cyc"
Cyc_Tcexp_tcExp(te,0,bound);
{void*_stmttmp38=Cyc_Tcutil_compress((void*)_check_null(bound->topt));void*_tmp2B7=_stmttmp38;void*_tmp2B8;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2B7)->tag == 0U){if(((struct Cyc_Absyn_TagCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2B7)->f1)->tag == 4U){if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2B7)->f2 != 0){_LL1: _tmp2B8=(void*)(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2B7)->f2)->hd;_LL2: {void*t=_tmp2B8;
# 1822
if((int)Cyc_Tcenv_new_status(te)== (int)Cyc_Tcenv_InNewAggr){
struct Cyc_Absyn_Exp*b=({void*_tmp6CA=Cyc_Absyn_uint_type;Cyc_Absyn_cast_exp(_tmp6CA,Cyc_Absyn_valueof_exp(t,0U),0,Cyc_Absyn_No_coercion,0U);});
b->topt=bound->topt;
# 1827
bound=b;}
# 1822
goto _LL0;}}else{goto _LL3;}}else{goto _LL3;}}else{_LL3: _LL4:
# 1831
 if(!Cyc_Tcutil_coerce_uint_type(bound))
({struct Cyc_String_pa_PrintArg_struct _tmp2BB=({struct Cyc_String_pa_PrintArg_struct _tmp498;_tmp498.tag=0U,({
struct _fat_ptr _tmp6CB=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string((void*)_check_null(bound->topt)));_tmp498.f1=_tmp6CB;});_tmp498;});void*_tmp2B9[1U];_tmp2B9[0]=& _tmp2BB;({unsigned _tmp6CD=bound->loc;struct _fat_ptr _tmp6CC=({const char*_tmp2BA="expecting unsigned int, found %s";_tag_fat(_tmp2BA,sizeof(char),33U);});Cyc_Tcutil_terr(_tmp6CD,_tmp6CC,_tag_fat(_tmp2B9,sizeof(void*),1U));});});}_LL0:;}
# 1836
if(!(vd->tq).real_const)
({void*_tmp2BC=0U;({int(*_tmp6CF)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp2BE)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp2BE;});struct _fat_ptr _tmp6CE=({const char*_tmp2BD="comprehension index variable is not declared const!";_tag_fat(_tmp2BD,sizeof(char),52U);});_tmp6CF(_tmp6CE,_tag_fat(_tmp2BC,sizeof(void*),0U));});});
# 1836
if(te->le != 0)
# 1840
te=Cyc_Tcenv_new_block(loc,te);{
# 1836
void**topt2=0;
# 1842
struct Cyc_Absyn_Tqual*tqopt=0;
void**ztopt=0;
# 1845
if(topt != 0){
void*_stmttmp39=Cyc_Tcutil_compress(*topt);void*_tmp2BF=_stmttmp39;void*_tmp2C3;struct Cyc_Absyn_Exp*_tmp2C2;struct Cyc_Absyn_Tqual _tmp2C1;void*_tmp2C0;struct Cyc_Absyn_PtrInfo _tmp2C4;switch(*((int*)_tmp2BF)){case 3U: _LL6: _tmp2C4=((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp2BF)->f1;_LL7: {struct Cyc_Absyn_PtrInfo x=_tmp2C4;
# 1848
topt2=({void**_tmp2C5=_cycalloc(sizeof(*_tmp2C5));*_tmp2C5=x.elt_type;_tmp2C5;});
tqopt=({struct Cyc_Absyn_Tqual*_tmp2C6=_cycalloc(sizeof(*_tmp2C6));*_tmp2C6=x.elt_tq;_tmp2C6;});
ztopt=({void**_tmp2C7=_cycalloc(sizeof(*_tmp2C7));*_tmp2C7=(x.ptr_atts).zero_term;_tmp2C7;});
goto _LL5;}case 4U: _LL8: _tmp2C0=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp2BF)->f1).elt_type;_tmp2C1=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp2BF)->f1).tq;_tmp2C2=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp2BF)->f1).num_elts;_tmp2C3=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp2BF)->f1).zero_term;_LL9: {void*t=_tmp2C0;struct Cyc_Absyn_Tqual tq=_tmp2C1;struct Cyc_Absyn_Exp*b=_tmp2C2;void*zt=_tmp2C3;
# 1853
topt2=({void**_tmp2C8=_cycalloc(sizeof(*_tmp2C8));*_tmp2C8=t;_tmp2C8;});
tqopt=({struct Cyc_Absyn_Tqual*_tmp2C9=_cycalloc(sizeof(*_tmp2C9));*_tmp2C9=tq;_tmp2C9;});
ztopt=({void**_tmp2CA=_cycalloc(sizeof(*_tmp2CA));*_tmp2CA=zt;_tmp2CA;});
goto _LL5;}default: _LLA: _LLB:
# 1858
 goto _LL5;}_LL5:;}{
# 1845
void*t=
# 1861
Cyc_Tcexp_tcExp(te,topt2,body);
# 1863
if(Cyc_Tcutil_is_noalias_pointer_or_aggr(t)&& !Cyc_Tcutil_is_noalias_path(body))
({void*_tmp2CB=0U;({unsigned _tmp6D1=body->loc;struct _fat_ptr _tmp6D0=({const char*_tmp2CC="Cannot consume non-unique paths; do swap instead";_tag_fat(_tmp2CC,sizeof(char),49U);});Cyc_Tcutil_terr(_tmp6D1,_tmp6D0,_tag_fat(_tmp2CB,sizeof(void*),0U));});});
# 1863
if(te->le == 0){
# 1867
if(!Cyc_Tcutil_is_const_exp(bound))
({void*_tmp2CD=0U;({unsigned _tmp6D3=bound->loc;struct _fat_ptr _tmp6D2=({const char*_tmp2CE="bound is not constant";_tag_fat(_tmp2CE,sizeof(char),22U);});Cyc_Tcutil_terr(_tmp6D3,_tmp6D2,_tag_fat(_tmp2CD,sizeof(void*),0U));});});
# 1867
if(!
# 1869
Cyc_Tcutil_is_const_exp(body))
({void*_tmp2CF=0U;({unsigned _tmp6D5=bound->loc;struct _fat_ptr _tmp6D4=({const char*_tmp2D0="body is not constant";_tag_fat(_tmp2D0,sizeof(char),21U);});Cyc_Tcutil_terr(_tmp6D5,_tmp6D4,_tag_fat(_tmp2CF,sizeof(void*),0U));});});}
# 1863
if(
# 1872
ztopt != 0 && Cyc_Tcutil_force_type2bool(0,*ztopt)){
# 1875
struct Cyc_Absyn_Exp*e1=Cyc_Absyn_uint_exp(1U,0U);e1->topt=Cyc_Absyn_uint_type;
bound=Cyc_Absyn_add_exp(bound,e1,0U);bound->topt=Cyc_Absyn_uint_type;
*is_zero_term=1;}
# 1863
if(
# 1879
Cyc_Tcutil_is_noalias_pointer_or_aggr((void*)_check_null(body->topt))&& !
Cyc_Tcutil_is_noalias_path(body))
({void*_tmp2D1=0U;({unsigned _tmp6D7=body->loc;struct _fat_ptr _tmp6D6=({const char*_tmp2D2="Cannot consume non-unique paths; do swap instead";_tag_fat(_tmp2D2,sizeof(char),49U);});Cyc_Tcutil_terr(_tmp6D7,_tmp6D6,_tag_fat(_tmp2D1,sizeof(void*),0U));});});{
# 1863
void*res=({
# 1883
void*_tmp6DA=t;struct Cyc_Absyn_Tqual _tmp6D9=tqopt == 0?Cyc_Absyn_empty_tqual(0U):*tqopt;struct Cyc_Absyn_Exp*_tmp6D8=bound;Cyc_Absyn_array_type(_tmp6DA,_tmp6D9,_tmp6D8,
ztopt == 0?Cyc_Absyn_false_type:*ztopt,0U);});
return res;}}}}
# 306 "tcexp.cyc"
static void*Cyc_Tcexp_tcComprehensionNoinit(struct Cyc_Tcenv_Tenv*te,unsigned loc,void**topt,struct Cyc_Absyn_Exp*bound,void*t,int*is_zero_term){
# 1892 "tcexp.cyc"
Cyc_Tcexp_tcExp(te,0,bound);
{void*_stmttmp3A=Cyc_Tcutil_compress((void*)_check_null(bound->topt));void*_tmp2D3=_stmttmp3A;void*_tmp2D4;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2D3)->tag == 0U){if(((struct Cyc_Absyn_TagCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2D3)->f1)->tag == 4U){if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2D3)->f2 != 0){_LL1: _tmp2D4=(void*)(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2D3)->f2)->hd;_LL2: {void*t=_tmp2D4;
# 1897
if((int)Cyc_Tcenv_new_status(te)== (int)Cyc_Tcenv_InNewAggr){
struct Cyc_Absyn_Exp*b=({void*_tmp6DB=Cyc_Absyn_uint_type;Cyc_Absyn_cast_exp(_tmp6DB,Cyc_Absyn_valueof_exp(t,0U),0,Cyc_Absyn_No_coercion,0U);});
b->topt=bound->topt;
# 1902
bound=b;}
# 1897
goto _LL0;}}else{goto _LL3;}}else{goto _LL3;}}else{_LL3: _LL4:
# 1906
 if(!Cyc_Tcutil_coerce_uint_type(bound))
({struct Cyc_String_pa_PrintArg_struct _tmp2D7=({struct Cyc_String_pa_PrintArg_struct _tmp499;_tmp499.tag=0U,({
struct _fat_ptr _tmp6DC=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string((void*)_check_null(bound->topt)));_tmp499.f1=_tmp6DC;});_tmp499;});void*_tmp2D5[1U];_tmp2D5[0]=& _tmp2D7;({unsigned _tmp6DE=bound->loc;struct _fat_ptr _tmp6DD=({const char*_tmp2D6="expecting unsigned int, found %s";_tag_fat(_tmp2D6,sizeof(char),33U);});Cyc_Tcutil_terr(_tmp6DE,_tmp6DD,_tag_fat(_tmp2D5,sizeof(void*),1U));});});}_LL0:;}{
# 1911
void**topt2=0;
struct Cyc_Absyn_Tqual*tqopt=0;
void**ztopt=0;
# 1915
if(topt != 0){
void*_stmttmp3B=Cyc_Tcutil_compress(*topt);void*_tmp2D8=_stmttmp3B;void*_tmp2DC;struct Cyc_Absyn_Exp*_tmp2DB;struct Cyc_Absyn_Tqual _tmp2DA;void*_tmp2D9;struct Cyc_Absyn_PtrInfo _tmp2DD;switch(*((int*)_tmp2D8)){case 3U: _LL6: _tmp2DD=((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp2D8)->f1;_LL7: {struct Cyc_Absyn_PtrInfo x=_tmp2DD;
# 1918
topt2=({void**_tmp2DE=_cycalloc(sizeof(*_tmp2DE));*_tmp2DE=x.elt_type;_tmp2DE;});
tqopt=({struct Cyc_Absyn_Tqual*_tmp2DF=_cycalloc(sizeof(*_tmp2DF));*_tmp2DF=x.elt_tq;_tmp2DF;});
ztopt=({void**_tmp2E0=_cycalloc(sizeof(*_tmp2E0));*_tmp2E0=(x.ptr_atts).zero_term;_tmp2E0;});
goto _LL5;}case 4U: _LL8: _tmp2D9=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp2D8)->f1).elt_type;_tmp2DA=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp2D8)->f1).tq;_tmp2DB=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp2D8)->f1).num_elts;_tmp2DC=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp2D8)->f1).zero_term;_LL9: {void*t=_tmp2D9;struct Cyc_Absyn_Tqual tq=_tmp2DA;struct Cyc_Absyn_Exp*b=_tmp2DB;void*zt=_tmp2DC;
# 1923
topt2=({void**_tmp2E1=_cycalloc(sizeof(*_tmp2E1));*_tmp2E1=t;_tmp2E1;});
tqopt=({struct Cyc_Absyn_Tqual*_tmp2E2=_cycalloc(sizeof(*_tmp2E2));*_tmp2E2=tq;_tmp2E2;});
ztopt=({void**_tmp2E3=_cycalloc(sizeof(*_tmp2E3));*_tmp2E3=zt;_tmp2E3;});
goto _LL5;}default: _LLA: _LLB:
# 1928
 goto _LL5;}_LL5:;}
# 1915
({unsigned _tmp6E1=loc;struct Cyc_Tcenv_Tenv*_tmp6E0=te;struct Cyc_List_List*_tmp6DF=
# 1931
Cyc_Tcenv_lookup_type_vars(te);
# 1915
Cyc_Tctyp_check_type(_tmp6E1,_tmp6E0,_tmp6DF,& Cyc_Tcutil_tmk,1,1,t);});
# 1932
if(topt2 != 0)Cyc_Unify_unify(*topt2,t);if(te->le == 0){
# 1935
if(!Cyc_Tcutil_is_const_exp(bound))
({void*_tmp2E4=0U;({unsigned _tmp6E3=bound->loc;struct _fat_ptr _tmp6E2=({const char*_tmp2E5="bound is not constant";_tag_fat(_tmp2E5,sizeof(char),22U);});Cyc_Tcutil_terr(_tmp6E3,_tmp6E2,_tag_fat(_tmp2E4,sizeof(void*),0U));});});}
# 1932
if(
# 1938
ztopt != 0 && Cyc_Tcutil_force_type2bool(0,*ztopt)){
# 1941
struct Cyc_Absyn_Exp*e1=Cyc_Absyn_uint_exp(1U,0U);e1->topt=Cyc_Absyn_uint_type;
bound=Cyc_Absyn_add_exp(bound,e1,0U);bound->topt=Cyc_Absyn_uint_type;
*is_zero_term=1;
# 1945
({void*_tmp2E6=0U;({unsigned _tmp6E5=loc;struct _fat_ptr _tmp6E4=({const char*_tmp2E7="non-initializing comprehensions do not currently support @zeroterm arrays";_tag_fat(_tmp2E7,sizeof(char),74U);});Cyc_Tcutil_terr(_tmp6E5,_tmp6E4,_tag_fat(_tmp2E6,sizeof(void*),0U));});});}{
# 1932
void*res=({
# 1948
void*_tmp6E8=t;struct Cyc_Absyn_Tqual _tmp6E7=tqopt == 0?Cyc_Absyn_empty_tqual(0U):*tqopt;struct Cyc_Absyn_Exp*_tmp6E6=bound;Cyc_Absyn_array_type(_tmp6E8,_tmp6E7,_tmp6E6,
ztopt == 0?Cyc_Absyn_false_type:*ztopt,0U);});
return res;}}}struct _tuple19{struct Cyc_Absyn_Aggrfield*f1;struct Cyc_Absyn_Exp*f2;};
# 306 "tcexp.cyc"
static void*Cyc_Tcexp_tcAggregate(struct Cyc_Tcenv_Tenv*te,unsigned loc,void**topt,struct _tuple1**tn,struct Cyc_List_List**ts,struct Cyc_List_List*args,struct Cyc_Absyn_Aggrdecl**ad_opt){
# 1967 "tcexp.cyc"
struct Cyc_Absyn_Aggrdecl**adptr;
struct Cyc_Absyn_Aggrdecl*ad;
if(*ad_opt != 0){
ad=(struct Cyc_Absyn_Aggrdecl*)_check_null(*ad_opt);
adptr=({struct Cyc_Absyn_Aggrdecl**_tmp2E8=_cycalloc(sizeof(*_tmp2E8));*_tmp2E8=ad;_tmp2E8;});}else{
# 1973
{struct _handler_cons _tmp2E9;_push_handler(& _tmp2E9);{int _tmp2EB=0;if(setjmp(_tmp2E9.handler))_tmp2EB=1;if(!_tmp2EB){adptr=Cyc_Tcenv_lookup_aggrdecl(te,loc,*tn);
ad=*adptr;
# 1973
;_pop_handler();}else{void*_tmp2EA=(void*)Cyc_Core_get_exn_thrown();void*_tmp2EC=_tmp2EA;void*_tmp2ED;if(((struct Cyc_Dict_Absent_exn_struct*)_tmp2EC)->tag == Cyc_Dict_Absent){_LL1: _LL2:
# 1976
({struct Cyc_String_pa_PrintArg_struct _tmp2F0=({struct Cyc_String_pa_PrintArg_struct _tmp49A;_tmp49A.tag=0U,({struct _fat_ptr _tmp6E9=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(*tn));_tmp49A.f1=_tmp6E9;});_tmp49A;});void*_tmp2EE[1U];_tmp2EE[0]=& _tmp2F0;({unsigned _tmp6EB=loc;struct _fat_ptr _tmp6EA=({const char*_tmp2EF="unbound struct/union name %s";_tag_fat(_tmp2EF,sizeof(char),29U);});Cyc_Tcutil_terr(_tmp6EB,_tmp6EA,_tag_fat(_tmp2EE,sizeof(void*),1U));});});
return topt != 0?*topt: Cyc_Absyn_void_type;}else{_LL3: _tmp2ED=_tmp2EC;_LL4: {void*exn=_tmp2ED;(int)_rethrow(exn);}}_LL0:;}}}
# 1979
*ad_opt=ad;
*tn=ad->name;}
# 1982
if(ad->impl == 0){
({struct Cyc_String_pa_PrintArg_struct _tmp2F3=({struct Cyc_String_pa_PrintArg_struct _tmp49B;_tmp49B.tag=0U,({struct _fat_ptr _tmp6EC=(struct _fat_ptr)((int)ad->kind == (int)Cyc_Absyn_StructA?({const char*_tmp2F4="struct";_tag_fat(_tmp2F4,sizeof(char),7U);}):({const char*_tmp2F5="union";_tag_fat(_tmp2F5,sizeof(char),6U);}));_tmp49B.f1=_tmp6EC;});_tmp49B;});void*_tmp2F1[1U];_tmp2F1[0]=& _tmp2F3;({unsigned _tmp6EE=loc;struct _fat_ptr _tmp6ED=({const char*_tmp2F2="can't construct abstract %s";_tag_fat(_tmp2F2,sizeof(char),28U);});Cyc_Tcutil_terr(_tmp6EE,_tmp6ED,_tag_fat(_tmp2F1,sizeof(void*),1U));});});
return Cyc_Absyn_wildtyp(Cyc_Tcenv_lookup_opt_type_vars(te));}{
# 1982
struct Cyc_Tcenv_Tenv*te2;
# 1991
enum Cyc_Tcenv_NewStatus status=Cyc_Tcenv_new_status(te);
if((int)status == (int)1U)
te2=Cyc_Tcenv_set_new_status(Cyc_Tcenv_InNewAggr,te);else{
# 1999
te2=Cyc_Tcenv_set_new_status(status,te);}{
# 2001
struct _tuple13 env=({struct _tuple13 _tmp4A1;({struct Cyc_List_List*_tmp6EF=Cyc_Tcenv_lookup_type_vars(te2);_tmp4A1.f1=_tmp6EF;}),_tmp4A1.f2=Cyc_Core_heap_region;_tmp4A1;});
struct Cyc_List_List*all_inst=({struct Cyc_List_List*(*_tmp6F0)(struct _tuple14*(*f)(struct _tuple13*,struct Cyc_Absyn_Tvar*),struct _tuple13*env,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp30B)(struct _tuple14*(*f)(struct _tuple13*,struct Cyc_Absyn_Tvar*),struct _tuple13*env,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct _tuple14*(*f)(struct _tuple13*,struct Cyc_Absyn_Tvar*),struct _tuple13*env,struct Cyc_List_List*x))Cyc_List_map_c;_tmp30B;});_tmp6F0(Cyc_Tcutil_r_make_inst_var,& env,ad->tvs);});
struct Cyc_List_List*exist_inst=({struct Cyc_List_List*(*_tmp6F1)(struct _tuple14*(*f)(struct _tuple13*,struct Cyc_Absyn_Tvar*),struct _tuple13*env,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp30A)(struct _tuple14*(*f)(struct _tuple13*,struct Cyc_Absyn_Tvar*),struct _tuple13*env,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct _tuple14*(*f)(struct _tuple13*,struct Cyc_Absyn_Tvar*),struct _tuple13*env,struct Cyc_List_List*x))Cyc_List_map_c;_tmp30A;});_tmp6F1(Cyc_Tcutil_r_make_inst_var,& env,((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->exist_vars);});
struct Cyc_List_List*all_typs=({struct Cyc_List_List*(*_tmp6F3)(void*(*f)(struct _tuple14*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp308)(void*(*f)(struct _tuple14*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(void*(*f)(struct _tuple14*),struct Cyc_List_List*x))Cyc_List_map;_tmp308;});void*(*_tmp6F2)(struct _tuple14*)=({void*(*_tmp309)(struct _tuple14*)=(void*(*)(struct _tuple14*))Cyc_Core_snd;_tmp309;});_tmp6F3(_tmp6F2,all_inst);});
struct Cyc_List_List*exist_typs=({struct Cyc_List_List*(*_tmp6F5)(void*(*f)(struct _tuple14*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp306)(void*(*f)(struct _tuple14*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(void*(*f)(struct _tuple14*),struct Cyc_List_List*x))Cyc_List_map;_tmp306;});void*(*_tmp6F4)(struct _tuple14*)=({void*(*_tmp307)(struct _tuple14*)=(void*(*)(struct _tuple14*))Cyc_Core_snd;_tmp307;});_tmp6F5(_tmp6F4,exist_inst);});
struct Cyc_List_List*inst=Cyc_List_append(all_inst,exist_inst);
void*res_typ;
# 2012
if(topt != 0){
void*_stmttmp3C=Cyc_Tcutil_compress(*topt);void*_tmp2F6=_stmttmp3C;struct Cyc_List_List*_tmp2F8;struct Cyc_Absyn_Aggrdecl**_tmp2F7;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2F6)->tag == 0U){if(((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2F6)->f1)->tag == 20U){if(((((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2F6)->f1)->f1).KnownAggr).tag == 2){_LL6: _tmp2F7=((((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2F6)->f1)->f1).KnownAggr).val;_tmp2F8=((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2F6)->f2;_LL7: {struct Cyc_Absyn_Aggrdecl**adptr2=_tmp2F7;struct Cyc_List_List*all_typs2=_tmp2F8;
# 2015
if(*adptr2 == *adptr){
{struct Cyc_List_List*ats=all_typs;for(0;ats != 0 && all_typs2 != 0;(
ats=ats->tl,all_typs2=all_typs2->tl)){
Cyc_Unify_unify((void*)ats->hd,(void*)all_typs2->hd);}}
# 2020
res_typ=*topt;
goto _LL5;}
# 2015
goto _LL9;}}else{goto _LL8;}}else{goto _LL8;}}else{_LL8: _LL9:
# 2025
 res_typ=({union Cyc_Absyn_AggrInfo _tmp6F6=Cyc_Absyn_KnownAggr(adptr);Cyc_Absyn_aggr_type(_tmp6F6,all_typs);});}_LL5:;}else{
# 2028
res_typ=({union Cyc_Absyn_AggrInfo _tmp6F7=Cyc_Absyn_KnownAggr(adptr);Cyc_Absyn_aggr_type(_tmp6F7,all_typs);});}{
# 2031
struct Cyc_List_List*user_ex_ts=*ts;
struct Cyc_List_List*ex_ts=exist_typs;
while(user_ex_ts != 0 && ex_ts != 0){
# 2035
({unsigned _tmp6FA=loc;struct Cyc_Tcenv_Tenv*_tmp6F9=te2;struct Cyc_List_List*_tmp6F8=Cyc_Tcenv_lookup_type_vars(te2);Cyc_Tctyp_check_type(_tmp6FA,_tmp6F9,_tmp6F8,& Cyc_Tcutil_ak,1,0,(void*)user_ex_ts->hd);});
Cyc_Tcutil_check_no_qual(loc,(void*)user_ex_ts->hd);
Cyc_Unify_unify((void*)user_ex_ts->hd,(void*)ex_ts->hd);
user_ex_ts=user_ex_ts->tl;
ex_ts=ex_ts->tl;}
# 2041
if(user_ex_ts != 0)
({void*_tmp2F9=0U;({unsigned _tmp6FC=loc;struct _fat_ptr _tmp6FB=({const char*_tmp2FA="too many explicit witness types";_tag_fat(_tmp2FA,sizeof(char),32U);});Cyc_Tcutil_terr(_tmp6FC,_tmp6FB,_tag_fat(_tmp2F9,sizeof(void*),0U));});});
# 2041
*ts=exist_typs;{
# 2047
struct Cyc_List_List*fields=
Cyc_Tcutil_resolve_aggregate_designators(Cyc_Core_heap_region,loc,args,ad->kind,((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->fields);
for(0;fields != 0;fields=fields->tl){
int bogus=0;
struct _tuple19*_stmttmp3D=(struct _tuple19*)fields->hd;struct Cyc_Absyn_Exp*_tmp2FC;struct Cyc_Absyn_Aggrfield*_tmp2FB;_LLB: _tmp2FB=_stmttmp3D->f1;_tmp2FC=_stmttmp3D->f2;_LLC: {struct Cyc_Absyn_Aggrfield*field=_tmp2FB;struct Cyc_Absyn_Exp*field_exp=_tmp2FC;
void*inst_fieldtyp=Cyc_Tcutil_substitute(inst,field->type);
({struct Cyc_Tcenv_Tenv*_tmp6FD=Cyc_Tcenv_clear_abstract_val_ok(te2);Cyc_Tcexp_tcExpInitializer(_tmp6FD,& inst_fieldtyp,field_exp);});
# 2058
if(!Cyc_Tcutil_coerce_arg(te2,field_exp,inst_fieldtyp,& bogus)){
({struct Cyc_String_pa_PrintArg_struct _tmp2FF=({struct Cyc_String_pa_PrintArg_struct _tmp4A0;_tmp4A0.tag=0U,_tmp4A0.f1=(struct _fat_ptr)((struct _fat_ptr)*field->name);_tmp4A0;});struct Cyc_String_pa_PrintArg_struct _tmp300=({struct Cyc_String_pa_PrintArg_struct _tmp49F;_tmp49F.tag=0U,({
struct _fat_ptr _tmp6FE=(struct _fat_ptr)((int)ad->kind == (int)Cyc_Absyn_StructA?({const char*_tmp304="struct";_tag_fat(_tmp304,sizeof(char),7U);}):({const char*_tmp305="union";_tag_fat(_tmp305,sizeof(char),6U);}));_tmp49F.f1=_tmp6FE;});_tmp49F;});struct Cyc_String_pa_PrintArg_struct _tmp301=({struct Cyc_String_pa_PrintArg_struct _tmp49E;_tmp49E.tag=0U,({
struct _fat_ptr _tmp6FF=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(*tn));_tmp49E.f1=_tmp6FF;});_tmp49E;});struct Cyc_String_pa_PrintArg_struct _tmp302=({struct Cyc_String_pa_PrintArg_struct _tmp49D;_tmp49D.tag=0U,({struct _fat_ptr _tmp700=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(inst_fieldtyp));_tmp49D.f1=_tmp700;});_tmp49D;});struct Cyc_String_pa_PrintArg_struct _tmp303=({struct Cyc_String_pa_PrintArg_struct _tmp49C;_tmp49C.tag=0U,({
struct _fat_ptr _tmp701=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string((void*)_check_null(field_exp->topt)));_tmp49C.f1=_tmp701;});_tmp49C;});void*_tmp2FD[5U];_tmp2FD[0]=& _tmp2FF,_tmp2FD[1]=& _tmp300,_tmp2FD[2]=& _tmp301,_tmp2FD[3]=& _tmp302,_tmp2FD[4]=& _tmp303;({unsigned _tmp703=field_exp->loc;struct _fat_ptr _tmp702=({const char*_tmp2FE="field %s of %s %s expects type %s != %s";_tag_fat(_tmp2FE,sizeof(char),40U);});Cyc_Tcutil_terr(_tmp703,_tmp702,_tag_fat(_tmp2FD,sizeof(void*),5U));});});
Cyc_Unify_explain_failure();}}}
# 2076 "tcexp.cyc"
return res_typ;}}}}}
# 306 "tcexp.cyc"
static void*Cyc_Tcexp_tcAnonStruct(struct Cyc_Tcenv_Tenv*te,unsigned loc,void*ts,struct Cyc_List_List*args){
# 2082 "tcexp.cyc"
{void*_stmttmp3E=Cyc_Tcutil_compress(ts);void*_tmp30C=_stmttmp3E;struct Cyc_List_List*_tmp30E;enum Cyc_Absyn_AggrKind _tmp30D;if(((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmp30C)->tag == 7U){_LL1: _tmp30D=((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmp30C)->f1;_tmp30E=((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmp30C)->f2;_LL2: {enum Cyc_Absyn_AggrKind k=_tmp30D;struct Cyc_List_List*fs=_tmp30E;
# 2084
if((int)k == (int)1U)
({void*_tmp30F=0U;({unsigned _tmp705=loc;struct _fat_ptr _tmp704=({const char*_tmp310="expecting struct but found union";_tag_fat(_tmp310,sizeof(char),33U);});Cyc_Tcutil_terr(_tmp705,_tmp704,_tag_fat(_tmp30F,sizeof(void*),0U));});});{
# 2084
struct Cyc_List_List*fields=
# 2087
Cyc_Tcutil_resolve_aggregate_designators(Cyc_Core_heap_region,loc,args,Cyc_Absyn_StructA,fs);
for(0;fields != 0;fields=fields->tl){
int bogus=0;
struct _tuple19*_stmttmp3F=(struct _tuple19*)fields->hd;struct Cyc_Absyn_Exp*_tmp312;struct Cyc_Absyn_Aggrfield*_tmp311;_LL6: _tmp311=_stmttmp3F->f1;_tmp312=_stmttmp3F->f2;_LL7: {struct Cyc_Absyn_Aggrfield*field=_tmp311;struct Cyc_Absyn_Exp*field_exp=_tmp312;
({struct Cyc_Tcenv_Tenv*_tmp707=Cyc_Tcenv_clear_abstract_val_ok(te);void**_tmp706=& field->type;Cyc_Tcexp_tcExpInitializer(_tmp707,_tmp706,field_exp);});
# 2093
if(!Cyc_Tcutil_coerce_arg(te,field_exp,field->type,& bogus)){
({struct Cyc_String_pa_PrintArg_struct _tmp315=({struct Cyc_String_pa_PrintArg_struct _tmp4A4;_tmp4A4.tag=0U,_tmp4A4.f1=(struct _fat_ptr)((struct _fat_ptr)*field->name);_tmp4A4;});struct Cyc_String_pa_PrintArg_struct _tmp316=({struct Cyc_String_pa_PrintArg_struct _tmp4A3;_tmp4A3.tag=0U,({
struct _fat_ptr _tmp708=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(field->type));_tmp4A3.f1=_tmp708;});_tmp4A3;});struct Cyc_String_pa_PrintArg_struct _tmp317=({struct Cyc_String_pa_PrintArg_struct _tmp4A2;_tmp4A2.tag=0U,({
struct _fat_ptr _tmp709=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string((void*)_check_null(field_exp->topt)));_tmp4A2.f1=_tmp709;});_tmp4A2;});void*_tmp313[3U];_tmp313[0]=& _tmp315,_tmp313[1]=& _tmp316,_tmp313[2]=& _tmp317;({unsigned _tmp70B=field_exp->loc;struct _fat_ptr _tmp70A=({const char*_tmp314="field %s of struct expects type %s != %s";_tag_fat(_tmp314,sizeof(char),41U);});Cyc_Tcutil_terr(_tmp70B,_tmp70A,_tag_fat(_tmp313,sizeof(void*),3U));});});
Cyc_Unify_explain_failure();}}}
# 2100
goto _LL0;}}}else{_LL3: _LL4:
({void*_tmp318=0U;({int(*_tmp70D)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp31A)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp31A;});struct _fat_ptr _tmp70C=({const char*_tmp319="tcAnonStruct: wrong type";_tag_fat(_tmp319,sizeof(char),25U);});_tmp70D(_tmp70C,_tag_fat(_tmp318,sizeof(void*),0U));});});}_LL0:;}
# 2103
return ts;}
# 306 "tcexp.cyc"
static void*Cyc_Tcexp_tcDatatype(struct Cyc_Tcenv_Tenv*te,unsigned loc,void**topt,struct Cyc_Absyn_Exp*e,struct Cyc_List_List*es,struct Cyc_Absyn_Datatypedecl*tud,struct Cyc_Absyn_Datatypefield*tuf){
# 2110 "tcexp.cyc"
struct _tuple13 env=({struct _tuple13 _tmp4AA;({struct Cyc_List_List*_tmp70E=Cyc_Tcenv_lookup_type_vars(te);_tmp4AA.f1=_tmp70E;}),_tmp4AA.f2=Cyc_Core_heap_region;_tmp4AA;});
struct Cyc_List_List*inst=({struct Cyc_List_List*(*_tmp70F)(struct _tuple14*(*f)(struct _tuple13*,struct Cyc_Absyn_Tvar*),struct _tuple13*env,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp32A)(struct _tuple14*(*f)(struct _tuple13*,struct Cyc_Absyn_Tvar*),struct _tuple13*env,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct _tuple14*(*f)(struct _tuple13*,struct Cyc_Absyn_Tvar*),struct _tuple13*env,struct Cyc_List_List*x))Cyc_List_map_c;_tmp32A;});_tmp70F(Cyc_Tcutil_r_make_inst_var,& env,tud->tvs);});
struct Cyc_List_List*all_typs=({struct Cyc_List_List*(*_tmp711)(void*(*f)(struct _tuple14*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp328)(void*(*f)(struct _tuple14*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(void*(*f)(struct _tuple14*),struct Cyc_List_List*x))Cyc_List_map;_tmp328;});void*(*_tmp710)(struct _tuple14*)=({void*(*_tmp329)(struct _tuple14*)=(void*(*)(struct _tuple14*))Cyc_Core_snd;_tmp329;});_tmp711(_tmp710,inst);});
void*res=({union Cyc_Absyn_DatatypeFieldInfo _tmp712=Cyc_Absyn_KnownDatatypefield(tud,tuf);Cyc_Absyn_datatype_field_type(_tmp712,all_typs);});
# 2115
if(topt != 0){
void*_stmttmp40=Cyc_Tcutil_compress(*topt);void*_tmp31B=_stmttmp40;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp31B)->tag == 0U){if(((struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp31B)->f1)->tag == 19U){_LL1: _LL2:
 Cyc_Unify_unify(*topt,res);goto _LL0;}else{goto _LL3;}}else{_LL3: _LL4:
 goto _LL0;}_LL0:;}{
# 2115
struct Cyc_List_List*ts=tuf->typs;
# 2122
for(0;es != 0 && ts != 0;(es=es->tl,ts=ts->tl)){
int bogus=0;
struct Cyc_Absyn_Exp*e=(struct Cyc_Absyn_Exp*)es->hd;
void*t=(*((struct _tuple17*)ts->hd)).f2;
if(inst != 0)t=Cyc_Tcutil_substitute(inst,t);Cyc_Tcexp_tcExpInitializer(te,& t,e);
# 2128
if(!Cyc_Tcutil_coerce_arg(te,e,t,& bogus)){
({struct Cyc_String_pa_PrintArg_struct _tmp31E=({struct Cyc_String_pa_PrintArg_struct _tmp4A7;_tmp4A7.tag=0U,({
# 2131
struct _fat_ptr _tmp713=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(tuf->name));_tmp4A7.f1=_tmp713;});_tmp4A7;});struct Cyc_String_pa_PrintArg_struct _tmp31F=({struct Cyc_String_pa_PrintArg_struct _tmp4A6;_tmp4A6.tag=0U,({struct _fat_ptr _tmp714=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t));_tmp4A6.f1=_tmp714;});_tmp4A6;});struct Cyc_String_pa_PrintArg_struct _tmp320=({struct Cyc_String_pa_PrintArg_struct _tmp4A5;_tmp4A5.tag=0U,({
struct _fat_ptr _tmp715=(struct _fat_ptr)((struct _fat_ptr)(e->topt == 0?(struct _fat_ptr)({const char*_tmp321="?";_tag_fat(_tmp321,sizeof(char),2U);}): Cyc_Absynpp_typ2string((void*)_check_null(e->topt))));_tmp4A5.f1=_tmp715;});_tmp4A5;});void*_tmp31C[3U];_tmp31C[0]=& _tmp31E,_tmp31C[1]=& _tmp31F,_tmp31C[2]=& _tmp320;({unsigned _tmp717=e->loc;struct _fat_ptr _tmp716=({const char*_tmp31D="datatype constructor %s expects argument of type %s but this argument has type %s";_tag_fat(_tmp31D,sizeof(char),82U);});Cyc_Tcutil_terr(_tmp717,_tmp716,_tag_fat(_tmp31C,sizeof(void*),3U));});});
Cyc_Unify_explain_failure();}}
# 2136
if(es != 0)
return({struct Cyc_String_pa_PrintArg_struct _tmp324=({struct Cyc_String_pa_PrintArg_struct _tmp4A8;_tmp4A8.tag=0U,({
# 2139
struct _fat_ptr _tmp718=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(tuf->name));_tmp4A8.f1=_tmp718;});_tmp4A8;});void*_tmp322[1U];_tmp322[0]=& _tmp324;({struct Cyc_Tcenv_Tenv*_tmp71C=te;unsigned _tmp71B=((struct Cyc_Absyn_Exp*)es->hd)->loc;void**_tmp71A=topt;struct _fat_ptr _tmp719=({const char*_tmp323="too many arguments for datatype constructor %s";_tag_fat(_tmp323,sizeof(char),47U);});Cyc_Tcexp_expr_err(_tmp71C,_tmp71B,_tmp71A,_tmp719,_tag_fat(_tmp322,sizeof(void*),1U));});});
# 2136
if(ts != 0)
# 2141
return({struct Cyc_String_pa_PrintArg_struct _tmp327=({struct Cyc_String_pa_PrintArg_struct _tmp4A9;_tmp4A9.tag=0U,({
struct _fat_ptr _tmp71D=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(tuf->name));_tmp4A9.f1=_tmp71D;});_tmp4A9;});void*_tmp325[1U];_tmp325[0]=& _tmp327;({struct Cyc_Tcenv_Tenv*_tmp721=te;unsigned _tmp720=loc;void**_tmp71F=topt;struct _fat_ptr _tmp71E=({const char*_tmp326="too few arguments for datatype constructor %s";_tag_fat(_tmp326,sizeof(char),46U);});Cyc_Tcexp_expr_err(_tmp721,_tmp720,_tmp71F,_tmp71E,_tag_fat(_tmp325,sizeof(void*),1U));});});
# 2136
return res;}}
# 306 "tcexp.cyc"
static int Cyc_Tcexp_check_malloc_type(int allow_zero,unsigned loc,void**topt,void*t){
# 2149 "tcexp.cyc"
if(Cyc_Tcutil_is_bits_only_type(t)|| allow_zero && Cyc_Tcutil_zeroable_type(t))return 1;if(topt != 0){
# 2152
void*_stmttmp41=Cyc_Tcutil_compress(*topt);void*_tmp32B=_stmttmp41;void*_tmp32C;if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp32B)->tag == 3U){_LL1: _tmp32C=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp32B)->f1).elt_type;_LL2: {void*elt_typ=_tmp32C;
# 2154
Cyc_Unify_unify(elt_typ,t);
if(Cyc_Tcutil_is_bits_only_type(t)|| allow_zero && Cyc_Tcutil_zeroable_type(t))return 1;goto _LL0;}}else{_LL3: _LL4:
# 2157
 goto _LL0;}_LL0:;}
# 2149
return 0;}
# 306 "tcexp.cyc"
static void*Cyc_Tcexp_mallocRgn(void*rgn){
# 2169 "tcexp.cyc"
enum Cyc_Absyn_AliasQual _stmttmp42=(Cyc_Tcutil_type_kind(Cyc_Tcutil_compress(rgn)))->aliasqual;enum Cyc_Absyn_AliasQual _tmp32D=_stmttmp42;if(_tmp32D == Cyc_Absyn_Unique){_LL1: _LL2:
 return Cyc_Absyn_unique_rgn_type;}else{_LL3: _LL4:
 return Cyc_Absyn_heap_rgn_type;}_LL0:;}
# 306 "tcexp.cyc"
static void*Cyc_Tcexp_tcMalloc(struct Cyc_Tcenv_Tenv*te,unsigned loc,void**topt,struct Cyc_Absyn_Exp**ropt,void***t,struct Cyc_Absyn_Exp**e,int*is_calloc,int*is_fat){
# 2179 "tcexp.cyc"
void*rgn=Cyc_Absyn_heap_rgn_type;
if(*ropt != 0){
# 2183
void*expected_type=
Cyc_Absyn_rgn_handle_type(Cyc_Absyn_new_evar(& Cyc_Tcutil_trko,Cyc_Tcenv_lookup_opt_type_vars(te)));
void*handle_type=Cyc_Tcexp_tcExp(te,& expected_type,(struct Cyc_Absyn_Exp*)_check_null(*ropt));
void*_stmttmp43=Cyc_Tcutil_compress(handle_type);void*_tmp32E=_stmttmp43;void*_tmp32F;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp32E)->tag == 0U){if(((struct Cyc_Absyn_RgnHandleCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp32E)->f1)->tag == 3U){if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp32E)->f2 != 0){_LL1: _tmp32F=(void*)(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp32E)->f2)->hd;_LL2: {void*r=_tmp32F;
# 2188
rgn=r;
Cyc_Tcenv_check_rgn_accessible(te,loc,rgn);
goto _LL0;}}else{goto _LL3;}}else{goto _LL3;}}else{_LL3: _LL4:
# 2192
({struct Cyc_String_pa_PrintArg_struct _tmp332=({struct Cyc_String_pa_PrintArg_struct _tmp4AB;_tmp4AB.tag=0U,({
struct _fat_ptr _tmp722=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(handle_type));_tmp4AB.f1=_tmp722;});_tmp4AB;});void*_tmp330[1U];_tmp330[0]=& _tmp332;({unsigned _tmp724=((struct Cyc_Absyn_Exp*)_check_null(*ropt))->loc;struct _fat_ptr _tmp723=({const char*_tmp331="expecting region_t type but found %s";_tag_fat(_tmp331,sizeof(char),37U);});Cyc_Tcutil_terr(_tmp724,_tmp723,_tag_fat(_tmp330,sizeof(void*),1U));});});
goto _LL0;}_LL0:;}else{
# 2199
if(topt != 0){
void*optrgn=Cyc_Absyn_void_type;
if(Cyc_Tcutil_rgn_of_pointer(*topt,& optrgn)){
rgn=Cyc_Tcexp_mallocRgn(optrgn);
if(rgn == Cyc_Absyn_unique_rgn_type)({struct Cyc_Absyn_Exp*_tmp725=Cyc_Absyn_uniquergn_exp();*ropt=_tmp725;});}}}
# 2180
({struct Cyc_Tcenv_Tenv*_tmp726=
# 2206
Cyc_Tcenv_clear_abstract_val_ok(te);
# 2180
Cyc_Tcexp_tcExp(_tmp726,& Cyc_Absyn_uint_type,*e);});{
# 2215 "tcexp.cyc"
void*elt_type;
struct Cyc_Absyn_Exp*num_elts;
int one_elt;
if(*is_calloc){
if(*t == 0)({void*_tmp333=0U;({int(*_tmp728)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp335)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp335;});struct _fat_ptr _tmp727=({const char*_tmp334="calloc with empty type";_tag_fat(_tmp334,sizeof(char),23U);});_tmp728(_tmp727,_tag_fat(_tmp333,sizeof(void*),0U));});});elt_type=*((void**)_check_null(*t));
# 2221
({unsigned _tmp72B=loc;struct Cyc_Tcenv_Tenv*_tmp72A=te;struct Cyc_List_List*_tmp729=Cyc_Tcenv_lookup_type_vars(te);Cyc_Tctyp_check_type(_tmp72B,_tmp72A,_tmp729,& Cyc_Tcutil_tmk,1,0,elt_type);});
Cyc_Tcutil_check_no_qual(loc,elt_type);
if(!Cyc_Tcexp_check_malloc_type(1,loc,topt,elt_type))
({struct Cyc_String_pa_PrintArg_struct _tmp338=({struct Cyc_String_pa_PrintArg_struct _tmp4AC;_tmp4AC.tag=0U,({struct _fat_ptr _tmp72C=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(elt_type));_tmp4AC.f1=_tmp72C;});_tmp4AC;});void*_tmp336[1U];_tmp336[0]=& _tmp338;({unsigned _tmp72E=loc;struct _fat_ptr _tmp72D=({const char*_tmp337="calloc cannot be used with type %s\n\t(type needs initialization)";_tag_fat(_tmp337,sizeof(char),64U);});Cyc_Tcutil_terr(_tmp72E,_tmp72D,_tag_fat(_tmp336,sizeof(void*),1U));});});
# 2223
num_elts=*e;
# 2226
one_elt=0;}else{
# 2228
void*er=(*e)->r;
retry_sizeof: {
void*_tmp339=er;struct Cyc_Absyn_Exp*_tmp33B;struct Cyc_Absyn_Exp*_tmp33A;void*_tmp33C;switch(*((int*)_tmp339)){case 17U: _LL6: _tmp33C=(void*)((struct Cyc_Absyn_Sizeoftype_e_Absyn_Raw_exp_struct*)_tmp339)->f1;_LL7: {void*t2=_tmp33C;
# 2232
elt_type=t2;
({void**_tmp72F=({void**_tmp33D=_cycalloc(sizeof(*_tmp33D));*_tmp33D=elt_type;_tmp33D;});*t=_tmp72F;});
num_elts=Cyc_Absyn_uint_exp(1U,0U);
Cyc_Tcexp_tcExp(te,& Cyc_Absyn_uint_type,num_elts);
one_elt=1;
goto _LL5;}case 3U: if(((struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct*)_tmp339)->f1 == Cyc_Absyn_Times){if(((struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct*)_tmp339)->f2 != 0){if(((struct Cyc_List_List*)((struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct*)_tmp339)->f2)->tl != 0){if(((struct Cyc_List_List*)((struct Cyc_List_List*)((struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct*)_tmp339)->f2)->tl)->tl == 0){_LL8: _tmp33A=(struct Cyc_Absyn_Exp*)(((struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct*)_tmp339)->f2)->hd;_tmp33B=(struct Cyc_Absyn_Exp*)((((struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct*)_tmp339)->f2)->tl)->hd;_LL9: {struct Cyc_Absyn_Exp*e1=_tmp33A;struct Cyc_Absyn_Exp*e2=_tmp33B;
# 2239
{struct _tuple0 _stmttmp44=({struct _tuple0 _tmp4AF;_tmp4AF.f1=e1->r,_tmp4AF.f2=e2->r;_tmp4AF;});struct _tuple0 _tmp33E=_stmttmp44;void*_tmp33F;void*_tmp340;if(((struct Cyc_Absyn_Sizeoftype_e_Absyn_Raw_exp_struct*)_tmp33E.f1)->tag == 17U){_LLD: _tmp340=(void*)((struct Cyc_Absyn_Sizeoftype_e_Absyn_Raw_exp_struct*)_tmp33E.f1)->f1;_LLE: {void*t1=_tmp340;
# 2242
if(!Cyc_Tcexp_check_malloc_type(0,loc,topt,t1)){
# 2245
if(!Cyc_Tcexp_check_malloc_type(1,loc,topt,t1))
({struct Cyc_String_pa_PrintArg_struct _tmp343=({struct Cyc_String_pa_PrintArg_struct _tmp4AD;_tmp4AD.tag=0U,({struct _fat_ptr _tmp730=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t1));_tmp4AD.f1=_tmp730;});_tmp4AD;});void*_tmp341[1U];_tmp341[0]=& _tmp343;({unsigned _tmp732=loc;struct _fat_ptr _tmp731=({const char*_tmp342="malloc cannot be used with type %s\n\t(type needs initialization)";_tag_fat(_tmp342,sizeof(char),64U);});Cyc_Tcutil_terr(_tmp732,_tmp731,_tag_fat(_tmp341,sizeof(void*),1U));});});else{
# 2248
*is_calloc=1;}}
# 2242
elt_type=t1;
# 2252
({void**_tmp733=({void**_tmp344=_cycalloc(sizeof(*_tmp344));*_tmp344=elt_type;_tmp344;});*t=_tmp733;});
num_elts=e2;
one_elt=0;
goto _LLC;}}else{if(((struct Cyc_Absyn_Sizeoftype_e_Absyn_Raw_exp_struct*)_tmp33E.f2)->tag == 17U){_LLF: _tmp33F=(void*)((struct Cyc_Absyn_Sizeoftype_e_Absyn_Raw_exp_struct*)_tmp33E.f2)->f1;_LL10: {void*t2=_tmp33F;
# 2258
if(!Cyc_Tcexp_check_malloc_type(0,loc,topt,t2)){
# 2261
if(!Cyc_Tcexp_check_malloc_type(1,loc,topt,t2))
({struct Cyc_String_pa_PrintArg_struct _tmp347=({struct Cyc_String_pa_PrintArg_struct _tmp4AE;_tmp4AE.tag=0U,({struct _fat_ptr _tmp734=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t2));_tmp4AE.f1=_tmp734;});_tmp4AE;});void*_tmp345[1U];_tmp345[0]=& _tmp347;({unsigned _tmp736=loc;struct _fat_ptr _tmp735=({const char*_tmp346="malloc cannot be used with type %s\n\t(type needs initialization)";_tag_fat(_tmp346,sizeof(char),64U);});Cyc_Tcutil_terr(_tmp736,_tmp735,_tag_fat(_tmp345,sizeof(void*),1U));});});else{
# 2264
*is_calloc=1;}}
# 2258
elt_type=t2;
# 2268
({void**_tmp737=({void**_tmp348=_cycalloc(sizeof(*_tmp348));*_tmp348=elt_type;_tmp348;});*t=_tmp737;});
num_elts=e1;
one_elt=0;
goto _LLC;}}else{_LL11: _LL12:
 goto No_sizeof;}}_LLC:;}
# 2274
goto _LL5;}}else{goto _LLA;}}else{goto _LLA;}}else{goto _LLA;}}else{goto _LLA;}default: _LLA: _LLB:
# 2276
 No_sizeof: {
# 2279
struct Cyc_Absyn_Exp*real_e=*e;
{void*_stmttmp45=real_e->r;void*_tmp349=_stmttmp45;struct Cyc_Absyn_Exp*_tmp34A;if(((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp349)->tag == 14U){_LL14: _tmp34A=((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp349)->f2;_LL15: {struct Cyc_Absyn_Exp*e=_tmp34A;
real_e=e;goto _LL13;}}else{_LL16: _LL17:
 goto _LL13;}_LL13:;}
# 2284
{void*_stmttmp46=Cyc_Tcutil_compress((void*)_check_null(real_e->topt));void*_tmp34B=_stmttmp46;void*_tmp34C;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp34B)->tag == 0U){if(((struct Cyc_Absyn_TagCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp34B)->f1)->tag == 4U){if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp34B)->f2 != 0){_LL19: _tmp34C=(void*)(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp34B)->f2)->hd;_LL1A: {void*tagt=_tmp34C;
# 2286
{void*_stmttmp47=Cyc_Tcutil_compress(tagt);void*_tmp34D=_stmttmp47;struct Cyc_Absyn_Exp*_tmp34E;if(((struct Cyc_Absyn_ValueofType_Absyn_Type_struct*)_tmp34D)->tag == 9U){_LL1E: _tmp34E=((struct Cyc_Absyn_ValueofType_Absyn_Type_struct*)_tmp34D)->f1;_LL1F: {struct Cyc_Absyn_Exp*vexp=_tmp34E;
# 2288
er=vexp->r;goto retry_sizeof;}}else{_LL20: _LL21:
 goto _LL1D;}_LL1D:;}
# 2291
goto _LL18;}}else{goto _LL1B;}}else{goto _LL1B;}}else{_LL1B: _LL1C:
 goto _LL18;}_LL18:;}
# 2294
elt_type=Cyc_Absyn_char_type;
({void**_tmp738=({void**_tmp34F=_cycalloc(sizeof(*_tmp34F));*_tmp34F=elt_type;_tmp34F;});*t=_tmp738;});
num_elts=*e;
one_elt=0;}
# 2299
goto _LL5;}_LL5:;}}
# 2303
*is_fat=!one_elt;
# 2306
{void*_tmp350=elt_type;struct Cyc_Absyn_Aggrdecl*_tmp351;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp350)->tag == 0U){if(((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp350)->f1)->tag == 20U){if(((((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp350)->f1)->f1).KnownAggr).tag == 2){_LL23: _tmp351=*((((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp350)->f1)->f1).KnownAggr).val;_LL24: {struct Cyc_Absyn_Aggrdecl*ad=_tmp351;
# 2308
if(ad->impl != 0 &&((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->exist_vars != 0)
({void*_tmp352=0U;({unsigned _tmp73A=loc;struct _fat_ptr _tmp739=({const char*_tmp353="malloc with existential types not yet implemented";_tag_fat(_tmp353,sizeof(char),50U);});Cyc_Tcutil_terr(_tmp73A,_tmp739,_tag_fat(_tmp352,sizeof(void*),0U));});});
# 2308
goto _LL22;}}else{goto _LL25;}}else{goto _LL25;}}else{_LL25: _LL26:
# 2311
 goto _LL22;}_LL22:;}{
# 2315
void*(*ptr_maker)(void*t,void*rgn,struct Cyc_Absyn_Tqual,void*zero_term)=Cyc_Absyn_at_type;
void*zero_term=Cyc_Absyn_false_type;
if(topt != 0){
void*_stmttmp48=Cyc_Tcutil_compress(*topt);void*_tmp354=_stmttmp48;void*_tmp357;void*_tmp356;void*_tmp355;if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp354)->tag == 3U){_LL28: _tmp355=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp354)->f1).ptr_atts).nullable;_tmp356=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp354)->f1).ptr_atts).bounds;_tmp357=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp354)->f1).ptr_atts).zero_term;_LL29: {void*n=_tmp355;void*b=_tmp356;void*zt=_tmp357;
# 2320
zero_term=zt;
if(Cyc_Tcutil_force_type2bool(0,n))
ptr_maker=Cyc_Absyn_star_type;
# 2321
if(
# 2325
Cyc_Tcutil_force_type2bool(0,zt)&& !(*is_calloc)){
({void*_tmp358=0U;({unsigned _tmp73C=loc;struct _fat_ptr _tmp73B=({const char*_tmp359="converting malloc to calloc to ensure zero-termination";_tag_fat(_tmp359,sizeof(char),55U);});Cyc_Tcutil_warn(_tmp73C,_tmp73B,_tag_fat(_tmp358,sizeof(void*),0U));});});
*is_calloc=1;}{
# 2321
struct Cyc_Absyn_Exp*eopt=({
# 2331
void*_tmp73D=Cyc_Absyn_bounds_one();Cyc_Tcutil_get_bounds_exp(_tmp73D,b);});
if(eopt != 0 && !one_elt){
struct Cyc_Absyn_Exp*upper_exp=eopt;
int is_constant=Cyc_Evexp_c_can_eval(num_elts);
if(is_constant && Cyc_Evexp_same_const_exp(upper_exp,num_elts)){
*is_fat=0;
return({void*_tmp741=elt_type;void*_tmp740=rgn;struct Cyc_Absyn_Tqual _tmp73F=Cyc_Absyn_empty_tqual(0U);void*_tmp73E=b;Cyc_Absyn_atb_type(_tmp741,_tmp740,_tmp73F,_tmp73E,zero_term);});}{
# 2335
void*_stmttmp49=
# 2339
Cyc_Tcutil_compress((void*)_check_null(num_elts->topt));
# 2335
void*_tmp35A=_stmttmp49;void*_tmp35B;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp35A)->tag == 0U){if(((struct Cyc_Absyn_TagCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp35A)->f1)->tag == 4U){if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp35A)->f2 != 0){_LL2D: _tmp35B=(void*)(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp35A)->f2)->hd;_LL2E: {void*tagtyp=_tmp35B;
# 2341
struct Cyc_Absyn_Exp*tagtyp_exp=({void*_tmp742=Cyc_Absyn_uint_type;Cyc_Absyn_cast_exp(_tmp742,Cyc_Absyn_valueof_exp(tagtyp,0U),0,Cyc_Absyn_No_coercion,0U);});
# 2343
if(Cyc_Evexp_same_const_exp(tagtyp_exp,upper_exp)){
*is_fat=0;
return({void*_tmp746=elt_type;void*_tmp745=rgn;struct Cyc_Absyn_Tqual _tmp744=Cyc_Absyn_empty_tqual(0U);void*_tmp743=b;Cyc_Absyn_atb_type(_tmp746,_tmp745,_tmp744,_tmp743,zero_term);});}
# 2343
goto _LL2C;}}else{goto _LL2F;}}else{goto _LL2F;}}else{_LL2F: _LL30:
# 2348
 goto _LL2C;}_LL2C:;}}
# 2332
goto _LL27;}}}else{_LL2A: _LL2B:
# 2352
 goto _LL27;}_LL27:;}
# 2317
if(!one_elt)
# 2354
ptr_maker=Cyc_Absyn_fatptr_type;
# 2317
return({
# 2355
void*(*_tmp74A)(void*t,void*rgn,struct Cyc_Absyn_Tqual,void*zero_term)=ptr_maker;void*_tmp749=elt_type;void*_tmp748=rgn;struct Cyc_Absyn_Tqual _tmp747=Cyc_Absyn_empty_tqual(0U);_tmp74A(_tmp749,_tmp748,_tmp747,zero_term);});}}}
# 306 "tcexp.cyc"
static void*Cyc_Tcexp_tcSwap(struct Cyc_Tcenv_Tenv*te,unsigned loc,void**topt,struct Cyc_Absyn_Exp*e1,struct Cyc_Absyn_Exp*e2){
# 2361 "tcexp.cyc"
struct Cyc_Tcenv_Tenv*te2=Cyc_Tcenv_enter_lhs(te);
Cyc_Tcexp_tcExpNoPromote(te2,0,e1);{
void*t1=(void*)_check_null(e1->topt);
Cyc_Tcexp_tcExpNoPromote(te2,& t1,e2);{
void*t1=(void*)_check_null(e1->topt);
void*t2=(void*)_check_null(e2->topt);
# 2368
{void*_stmttmp4A=Cyc_Tcutil_compress(t1);void*_tmp35C=_stmttmp4A;if(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp35C)->tag == 4U){_LL1: _LL2:
({void*_tmp35D=0U;({unsigned _tmp74C=loc;struct _fat_ptr _tmp74B=({const char*_tmp35E="cannot assign to an array";_tag_fat(_tmp35E,sizeof(char),26U);});Cyc_Tcutil_terr(_tmp74C,_tmp74B,_tag_fat(_tmp35D,sizeof(void*),0U));});});goto _LL0;}else{_LL3: _LL4:
 goto _LL0;}_LL0:;}
# 2373
if(!Cyc_Tcutil_is_boxed(t1)&& !Cyc_Tcutil_is_pointer_type(t1))
({void*_tmp35F=0U;({unsigned _tmp74E=loc;struct _fat_ptr _tmp74D=({const char*_tmp360="Swap not allowed for non-pointer or non-word-sized types.";_tag_fat(_tmp360,sizeof(char),58U);});Cyc_Tcutil_terr(_tmp74E,_tmp74D,_tag_fat(_tmp35F,sizeof(void*),0U));});});
# 2373
if(!
# 2377
Cyc_Absyn_is_lvalue(e1))
return({void*_tmp361=0U;({struct Cyc_Tcenv_Tenv*_tmp752=te;unsigned _tmp751=e1->loc;void**_tmp750=topt;struct _fat_ptr _tmp74F=({const char*_tmp362="swap non-lvalue";_tag_fat(_tmp362,sizeof(char),16U);});Cyc_Tcexp_expr_err(_tmp752,_tmp751,_tmp750,_tmp74F,_tag_fat(_tmp361,sizeof(void*),0U));});});
# 2373
if(!
# 2379
Cyc_Absyn_is_lvalue(e2))
return({void*_tmp363=0U;({struct Cyc_Tcenv_Tenv*_tmp756=te;unsigned _tmp755=e2->loc;void**_tmp754=topt;struct _fat_ptr _tmp753=({const char*_tmp364="swap non-lvalue";_tag_fat(_tmp364,sizeof(char),16U);});Cyc_Tcexp_expr_err(_tmp756,_tmp755,_tmp754,_tmp753,_tag_fat(_tmp363,sizeof(void*),0U));});});
# 2373
Cyc_Tcexp_check_writable(te,e1);
# 2383
Cyc_Tcexp_check_writable(te,e2);
if(!Cyc_Unify_unify(t1,t2)){
void*result=({struct Cyc_String_pa_PrintArg_struct _tmp367=({struct Cyc_String_pa_PrintArg_struct _tmp4B1;_tmp4B1.tag=0U,({
struct _fat_ptr _tmp757=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t1));_tmp4B1.f1=_tmp757;});_tmp4B1;});struct Cyc_String_pa_PrintArg_struct _tmp368=({struct Cyc_String_pa_PrintArg_struct _tmp4B0;_tmp4B0.tag=0U,({struct _fat_ptr _tmp758=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t2));_tmp4B0.f1=_tmp758;});_tmp4B0;});void*_tmp365[2U];_tmp365[0]=& _tmp367,_tmp365[1]=& _tmp368;({struct Cyc_Tcenv_Tenv*_tmp75C=te;unsigned _tmp75B=loc;void**_tmp75A=topt;struct _fat_ptr _tmp759=({const char*_tmp366="type mismatch: %s != %s";_tag_fat(_tmp366,sizeof(char),24U);});Cyc_Tcexp_expr_err(_tmp75C,_tmp75B,_tmp75A,_tmp759,_tag_fat(_tmp365,sizeof(void*),2U));});});
return result;}
# 2384
return Cyc_Absyn_void_type;}}}
# 306 "tcexp.cyc"
static void*Cyc_Tcexp_tcStmtExp(struct Cyc_Tcenv_Tenv*te,unsigned loc,void**topt,struct Cyc_Absyn_Stmt*s){
# 2395 "tcexp.cyc"
({struct Cyc_Tcenv_Tenv*_tmp75D=Cyc_Tcenv_enter_stmt_exp(Cyc_Tcenv_clear_abstract_val_ok(te));Cyc_Tcstmt_tcStmt(_tmp75D,s,1);});
# 2397
while(1){
void*_stmttmp4B=s->r;void*_tmp369=_stmttmp4B;struct Cyc_Absyn_Stmt*_tmp36B;struct Cyc_Absyn_Decl*_tmp36A;struct Cyc_Absyn_Stmt*_tmp36D;struct Cyc_Absyn_Stmt*_tmp36C;struct Cyc_Absyn_Exp*_tmp36E;switch(*((int*)_tmp369)){case 1U: _LL1: _tmp36E=((struct Cyc_Absyn_Exp_s_Absyn_Raw_stmt_struct*)_tmp369)->f1;_LL2: {struct Cyc_Absyn_Exp*e=_tmp36E;
# 2401
void*t=(void*)_check_null(e->topt);
if(!({void*_tmp75E=t;Cyc_Unify_unify(_tmp75E,Cyc_Absyn_wildtyp(Cyc_Tcenv_lookup_opt_type_vars(te)));})){
({struct Cyc_String_pa_PrintArg_struct _tmp371=({struct Cyc_String_pa_PrintArg_struct _tmp4B2;_tmp4B2.tag=0U,({
struct _fat_ptr _tmp75F=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t));_tmp4B2.f1=_tmp75F;});_tmp4B2;});void*_tmp36F[1U];_tmp36F[0]=& _tmp371;({unsigned _tmp761=loc;struct _fat_ptr _tmp760=({const char*_tmp370="statement expression returns type %s";_tag_fat(_tmp370,sizeof(char),37U);});Cyc_Tcutil_terr(_tmp761,_tmp760,_tag_fat(_tmp36F,sizeof(void*),1U));});});
Cyc_Unify_explain_failure();}
# 2402
return t;}case 2U: _LL3: _tmp36C=((struct Cyc_Absyn_Seq_s_Absyn_Raw_stmt_struct*)_tmp369)->f1;_tmp36D=((struct Cyc_Absyn_Seq_s_Absyn_Raw_stmt_struct*)_tmp369)->f2;_LL4: {struct Cyc_Absyn_Stmt*s1=_tmp36C;struct Cyc_Absyn_Stmt*s2=_tmp36D;
# 2408
s=s2;continue;}case 12U: _LL5: _tmp36A=((struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct*)_tmp369)->f1;_tmp36B=((struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct*)_tmp369)->f2;_LL6: {struct Cyc_Absyn_Decl*d=_tmp36A;struct Cyc_Absyn_Stmt*s1=_tmp36B;
s=s1;continue;}default: _LL7: _LL8:
# 2411
 return({void*_tmp372=0U;({struct Cyc_Tcenv_Tenv*_tmp765=te;unsigned _tmp764=loc;void**_tmp763=topt;struct _fat_ptr _tmp762=({const char*_tmp373="statement expression must end with expression";_tag_fat(_tmp373,sizeof(char),46U);});Cyc_Tcexp_expr_err(_tmp765,_tmp764,_tmp763,_tmp762,_tag_fat(_tmp372,sizeof(void*),0U));});});}_LL0:;}}
# 306 "tcexp.cyc"
static void*Cyc_Tcexp_tcTagcheck(struct Cyc_Tcenv_Tenv*te,unsigned loc,void**topt,struct Cyc_Absyn_Exp*e,struct _fat_ptr*f){
# 2417 "tcexp.cyc"
void*t=Cyc_Tcutil_compress(({struct Cyc_Tcenv_Tenv*_tmp766=Cyc_Tcenv_enter_abstract_val_ok(te);Cyc_Tcexp_tcExp(_tmp766,0,e);}));
{void*_tmp374=t;struct Cyc_Absyn_Aggrdecl*_tmp375;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp374)->tag == 0U){if(((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp374)->f1)->tag == 20U){if(((((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp374)->f1)->f1).KnownAggr).tag == 2){_LL1: _tmp375=*((((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp374)->f1)->f1).KnownAggr).val;_LL2: {struct Cyc_Absyn_Aggrdecl*ad=_tmp375;
# 2420
if(((int)ad->kind == (int)Cyc_Absyn_UnionA && ad->impl != 0)&&((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->tagged)goto _LL0;goto _LL4;}}else{goto _LL3;}}else{goto _LL3;}}else{_LL3: _LL4:
# 2423
({struct Cyc_String_pa_PrintArg_struct _tmp378=({struct Cyc_String_pa_PrintArg_struct _tmp4B3;_tmp4B3.tag=0U,({struct _fat_ptr _tmp767=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t));_tmp4B3.f1=_tmp767;});_tmp4B3;});void*_tmp376[1U];_tmp376[0]=& _tmp378;({unsigned _tmp769=loc;struct _fat_ptr _tmp768=({const char*_tmp377="expecting @tagged union but found %s";_tag_fat(_tmp377,sizeof(char),37U);});Cyc_Tcutil_terr(_tmp769,_tmp768,_tag_fat(_tmp376,sizeof(void*),1U));});});
goto _LL0;}_LL0:;}
# 2426
return Cyc_Absyn_uint_type;}
# 306 "tcexp.cyc"
static void*Cyc_Tcexp_tcNew(struct Cyc_Tcenv_Tenv*te0,unsigned loc,void**topt,struct Cyc_Absyn_Exp**rgn_handle,struct Cyc_Absyn_Exp*e,struct Cyc_Absyn_Exp*e1){
# 2434 "tcexp.cyc"
void*rgn=Cyc_Absyn_heap_rgn_type;
struct Cyc_Tcenv_Tenv*te=Cyc_Tcenv_clear_abstract_val_ok(Cyc_Tcenv_set_new_status(Cyc_Tcenv_InNew,te0));
if(*rgn_handle != 0){
# 2439
void*expected_type=
Cyc_Absyn_rgn_handle_type(Cyc_Absyn_new_evar(& Cyc_Tcutil_trko,Cyc_Tcenv_lookup_opt_type_vars(te)));
void*handle_type=Cyc_Tcexp_tcExp(te,& expected_type,(struct Cyc_Absyn_Exp*)_check_null(*rgn_handle));
void*_stmttmp4C=Cyc_Tcutil_compress(handle_type);void*_tmp379=_stmttmp4C;void*_tmp37A;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp379)->tag == 0U){if(((struct Cyc_Absyn_RgnHandleCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp379)->f1)->tag == 3U){if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp379)->f2 != 0){_LL1: _tmp37A=(void*)(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp379)->f2)->hd;_LL2: {void*r=_tmp37A;
# 2444
rgn=r;
Cyc_Tcenv_check_rgn_accessible(te,loc,rgn);
goto _LL0;}}else{goto _LL3;}}else{goto _LL3;}}else{_LL3: _LL4:
# 2448
({struct Cyc_String_pa_PrintArg_struct _tmp37D=({struct Cyc_String_pa_PrintArg_struct _tmp4B4;_tmp4B4.tag=0U,({
struct _fat_ptr _tmp76A=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(handle_type));_tmp4B4.f1=_tmp76A;});_tmp4B4;});void*_tmp37B[1U];_tmp37B[0]=& _tmp37D;({unsigned _tmp76C=((struct Cyc_Absyn_Exp*)_check_null(*rgn_handle))->loc;struct _fat_ptr _tmp76B=({const char*_tmp37C="expecting region_t type but found %s";_tag_fat(_tmp37C,sizeof(char),37U);});Cyc_Tcutil_terr(_tmp76C,_tmp76B,_tag_fat(_tmp37B,sizeof(void*),1U));});});
goto _LL0;}_LL0:;}else{
# 2455
if(topt != 0){
void*optrgn=Cyc_Absyn_void_type;
if(Cyc_Tcutil_rgn_of_pointer(*topt,& optrgn)){
rgn=Cyc_Tcexp_mallocRgn(optrgn);
if(rgn == Cyc_Absyn_unique_rgn_type)({struct Cyc_Absyn_Exp*_tmp76D=Cyc_Absyn_uniquergn_exp();*rgn_handle=_tmp76D;});}}}{
# 2436
void*_stmttmp4D=e1->r;void*_tmp37E=_stmttmp4D;struct Cyc_List_List*_tmp37F;struct Cyc_List_List*_tmp381;struct Cyc_Core_Opt*_tmp380;switch(*((int*)_tmp37E)){case 27U: _LL6: _LL7:
# 2464
 goto _LL9;case 28U: _LL8: _LL9: {
# 2468
void*res_typ=Cyc_Tcexp_tcExpNoPromote(te,topt,e1);
if(!Cyc_Tcutil_is_array_type(res_typ))
({void*_tmp382=0U;({int(*_tmp76F)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp384)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp384;});struct _fat_ptr _tmp76E=({const char*_tmp383="tcNew: comprehension returned non-array type";_tag_fat(_tmp383,sizeof(char),45U);});_tmp76F(_tmp76E,_tag_fat(_tmp382,sizeof(void*),0U));});});
# 2469
res_typ=
# 2471
Cyc_Tcutil_promote_array(res_typ,rgn,1);
if(topt != 0){
if(!Cyc_Unify_unify(*topt,res_typ)&&
 Cyc_Tcutil_silent_castable(te,loc,res_typ,*topt)){
e->topt=res_typ;
Cyc_Tcutil_unchecked_cast(e,*topt,Cyc_Absyn_Other_coercion);
res_typ=*topt;}}
# 2472
return res_typ;}case 37U: _LLA: _tmp380=((struct Cyc_Absyn_UnresolvedMem_e_Absyn_Raw_exp_struct*)_tmp37E)->f1;_tmp381=((struct Cyc_Absyn_UnresolvedMem_e_Absyn_Raw_exp_struct*)_tmp37E)->f2;_LLB: {struct Cyc_Core_Opt*nopt=_tmp380;struct Cyc_List_List*des=_tmp381;
# 2482
({void*_tmp770=(void*)({struct Cyc_Absyn_Array_e_Absyn_Raw_exp_struct*_tmp385=_cycalloc(sizeof(*_tmp385));_tmp385->tag=26U,_tmp385->f1=des;_tmp385;});e1->r=_tmp770;});
_tmp37F=des;goto _LLD;}case 26U: _LLC: _tmp37F=((struct Cyc_Absyn_Array_e_Absyn_Raw_exp_struct*)_tmp37E)->f1;_LLD: {struct Cyc_List_List*des=_tmp37F;
# 2485
void**elt_typ_opt=0;
int zero_term=0;
if(topt != 0){
void*_stmttmp4E=Cyc_Tcutil_compress(*topt);void*_tmp386=_stmttmp4E;void*_tmp389;struct Cyc_Absyn_Tqual _tmp388;void**_tmp387;if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp386)->tag == 3U){_LL15: _tmp387=(void**)&(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp386)->f1).elt_type;_tmp388=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp386)->f1).elt_tq;_tmp389=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp386)->f1).ptr_atts).zero_term;_LL16: {void**elt_typ=_tmp387;struct Cyc_Absyn_Tqual tq=_tmp388;void*zt=_tmp389;
# 2491
elt_typ_opt=elt_typ;
zero_term=Cyc_Tcutil_force_type2bool(0,zt);
goto _LL14;}}else{_LL17: _LL18:
 goto _LL14;}_LL14:;}{
# 2487
void*res_typ=
# 2497
Cyc_Tcexp_tcArray(te,e1->loc,elt_typ_opt,0,zero_term,des);
e1->topt=res_typ;
if(!Cyc_Tcutil_is_array_type(res_typ))
({void*_tmp38A=0U;({int(*_tmp772)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp38C)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp38C;});struct _fat_ptr _tmp771=({const char*_tmp38B="tcExpNoPromote on Array_e returned non-array type";_tag_fat(_tmp38B,sizeof(char),50U);});_tmp772(_tmp771,_tag_fat(_tmp38A,sizeof(void*),0U));});});
# 2499
res_typ=
# 2501
Cyc_Tcutil_promote_array(res_typ,rgn,0);
if(topt != 0){
# 2506
if(!Cyc_Unify_unify(*topt,res_typ)&&
 Cyc_Tcutil_silent_castable(te,loc,res_typ,*topt)){
e->topt=res_typ;
Cyc_Tcutil_unchecked_cast(e,*topt,Cyc_Absyn_Other_coercion);
res_typ=*topt;}}
# 2502
return res_typ;}}case 0U: switch(((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmp37E)->f1).Wstring_c).tag){case 8U: _LLE: _LLF: {
# 2518
void*topt2=({void*_tmp776=Cyc_Absyn_char_type;void*_tmp775=rgn;struct Cyc_Absyn_Tqual _tmp774=Cyc_Absyn_const_tqual(0U);void*_tmp773=Cyc_Absyn_fat_bound_type;Cyc_Absyn_atb_type(_tmp776,_tmp775,_tmp774,_tmp773,Cyc_Absyn_true_type);});
# 2520
void*t=Cyc_Tcexp_tcExp(te,& topt2,e1);
return({void*_tmp77A=t;void*_tmp779=rgn;struct Cyc_Absyn_Tqual _tmp778=Cyc_Absyn_empty_tqual(0U);void*_tmp777=Cyc_Absyn_thin_bounds_exp(Cyc_Absyn_uint_exp(1U,0U));Cyc_Absyn_atb_type(_tmp77A,_tmp779,_tmp778,_tmp777,Cyc_Absyn_false_type);});}case 9U: _LL10: _LL11: {
# 2525
void*topt2=({void*_tmp77E=Cyc_Absyn_wchar_type();void*_tmp77D=rgn;struct Cyc_Absyn_Tqual _tmp77C=Cyc_Absyn_const_tqual(0U);void*_tmp77B=Cyc_Absyn_fat_bound_type;Cyc_Absyn_atb_type(_tmp77E,_tmp77D,_tmp77C,_tmp77B,Cyc_Absyn_true_type);});
# 2527
void*t=Cyc_Tcexp_tcExp(te,& topt2,e1);
return({void*_tmp782=t;void*_tmp781=rgn;struct Cyc_Absyn_Tqual _tmp780=Cyc_Absyn_empty_tqual(0U);void*_tmp77F=Cyc_Absyn_thin_bounds_exp(Cyc_Absyn_uint_exp(1U,0U));Cyc_Absyn_atb_type(_tmp782,_tmp781,_tmp780,_tmp77F,Cyc_Absyn_false_type);});}default: goto _LL12;}default: _LL12: _LL13:
# 2534
 RG: {
void*bogus=Cyc_Absyn_void_type;
void**topt2=0;
if(topt != 0){
void*_stmttmp4F=Cyc_Tcutil_compress(*topt);void*_tmp38D=_stmttmp4F;struct Cyc_Absyn_Tqual _tmp38F;void**_tmp38E;switch(*((int*)_tmp38D)){case 3U: _LL1A: _tmp38E=(void**)&(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp38D)->f1).elt_type;_tmp38F=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp38D)->f1).elt_tq;_LL1B: {void**elttype=_tmp38E;struct Cyc_Absyn_Tqual tq=_tmp38F;
# 2540
topt2=elttype;goto _LL19;}case 0U: if(((struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp38D)->f1)->tag == 18U){_LL1C: _LL1D:
# 2544
 bogus=*topt;
topt2=& bogus;
goto _LL19;}else{goto _LL1E;}default: _LL1E: _LL1F:
 goto _LL19;}_LL19:;}{
# 2537
void*telt=
# 2550
Cyc_Tcexp_tcExp(te,topt2,e1);
# 2552
if(Cyc_Tcutil_is_noalias_pointer_or_aggr(telt)&& !Cyc_Tcutil_is_noalias_path(e1))
({void*_tmp390=0U;({unsigned _tmp784=e1->loc;struct _fat_ptr _tmp783=({const char*_tmp391="Cannot consume non-unique paths; do swap instead";_tag_fat(_tmp391,sizeof(char),49U);});Cyc_Tcutil_terr(_tmp784,_tmp783,_tag_fat(_tmp390,sizeof(void*),0U));});});{
# 2552
void*res_typ=(void*)({struct Cyc_Absyn_PointerType_Absyn_Type_struct*_tmp392=_cycalloc(sizeof(*_tmp392));
# 2555
_tmp392->tag=3U,(_tmp392->f1).elt_type=telt,({struct Cyc_Absyn_Tqual _tmp787=Cyc_Absyn_empty_tqual(0U);(_tmp392->f1).elt_tq=_tmp787;}),
((_tmp392->f1).ptr_atts).rgn=rgn,({void*_tmp786=Cyc_Tcutil_any_bool(Cyc_Tcenv_lookup_type_vars(te));((_tmp392->f1).ptr_atts).nullable=_tmp786;}),({
void*_tmp785=Cyc_Absyn_bounds_one();((_tmp392->f1).ptr_atts).bounds=_tmp785;}),((_tmp392->f1).ptr_atts).zero_term=Cyc_Absyn_false_type,((_tmp392->f1).ptr_atts).ptrloc=0;_tmp392;});
# 2559
if(topt != 0){
if(!Cyc_Unify_unify(*topt,res_typ)&&
 Cyc_Tcutil_silent_castable(te,loc,res_typ,*topt)){
e->topt=res_typ;
Cyc_Tcutil_unchecked_cast(e,*topt,Cyc_Absyn_Other_coercion);
res_typ=*topt;}}
# 2559
return res_typ;}}}}_LL5:;}}
# 306 "tcexp.cyc"
void*Cyc_Tcexp_tcExp(struct Cyc_Tcenv_Tenv*te,void**topt,struct Cyc_Absyn_Exp*e){
# 2574 "tcexp.cyc"
void*t=Cyc_Tcutil_compress(Cyc_Tcexp_tcExpNoPromote(te,topt,e));
if(Cyc_Tcutil_is_array_type(t))
({void*_tmp789=t=({void*_tmp788=t;Cyc_Tcutil_promote_array(_tmp788,(Cyc_Tcutil_addressof_props(e)).f2,0);});e->topt=_tmp789;});
# 2575
return t;}
# 306 "tcexp.cyc"
void*Cyc_Tcexp_tcExpInitializer(struct Cyc_Tcenv_Tenv*te,void**topt,struct Cyc_Absyn_Exp*e){
# 2584 "tcexp.cyc"
void*t=Cyc_Tcexp_tcExpNoPromote(te,topt,e);
# 2587
if(Cyc_Tcutil_is_noalias_pointer_or_aggr(t)&& !Cyc_Tcutil_is_noalias_path(e))
# 2592
({void*_tmp393=0U;({unsigned _tmp78B=e->loc;struct _fat_ptr _tmp78A=({const char*_tmp394="Cannot consume non-unique paths; do swap instead";_tag_fat(_tmp394,sizeof(char),49U);});Cyc_Tcutil_terr(_tmp78B,_tmp78A,_tag_fat(_tmp393,sizeof(void*),0U));});});{
# 2587
void*_stmttmp50=e->r;void*_tmp395=_stmttmp50;switch(*((int*)_tmp395)){case 26U: _LL1: _LL2:
# 2595
 goto _LL4;case 27U: _LL3: _LL4:
 goto _LL6;case 28U: _LL5: _LL6:
 goto _LL8;case 0U: switch(((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmp395)->f1).String_c).tag){case 9U: _LL7: _LL8:
 goto _LLA;case 8U: _LL9: _LLA:
 return t;default: goto _LLB;}default: _LLB: _LLC:
# 2601
 t=Cyc_Tcutil_compress(t);
if(Cyc_Tcutil_is_array_type(t)){
t=({void*_tmp78C=t;Cyc_Tcutil_promote_array(_tmp78C,(Cyc_Tcutil_addressof_props(e)).f2,0);});
Cyc_Tcutil_unchecked_cast(e,t,Cyc_Absyn_Other_coercion);}
# 2602
return t;}_LL0:;}}
# 306 "tcexp.cyc"
static void*Cyc_Tcexp_tcExpNoPromote(struct Cyc_Tcenv_Tenv*te,void**topt,struct Cyc_Absyn_Exp*e){
# 2618 "tcexp.cyc"
{void*_stmttmp51=e->r;void*_tmp396=_stmttmp51;struct Cyc_Absyn_Exp*_tmp397;if(((struct Cyc_Absyn_NoInstantiate_e_Absyn_Raw_exp_struct*)_tmp396)->tag == 12U){_LL1: _tmp397=((struct Cyc_Absyn_NoInstantiate_e_Absyn_Raw_exp_struct*)_tmp396)->f1;_LL2: {struct Cyc_Absyn_Exp*e2=_tmp397;
# 2621
Cyc_Tcexp_tcExpNoInst(te,topt,e2);
({void*_tmp78D=Cyc_Absyn_pointer_expand((void*)_check_null(e2->topt),0);e2->topt=_tmp78D;});
e->topt=e2->topt;
goto _LL0;}}else{_LL3: _LL4:
# 2627
 Cyc_Tcexp_tcExpNoInst(te,topt,e);
({void*_tmp78E=Cyc_Absyn_pointer_expand((void*)_check_null(e->topt),0);e->topt=_tmp78E;});
# 2630
{void*_stmttmp52=Cyc_Tcutil_compress((void*)_check_null(e->topt));void*_tmp398=_stmttmp52;void*_tmp39E;void*_tmp39D;void*_tmp39C;void*_tmp39B;struct Cyc_Absyn_Tqual _tmp39A;void*_tmp399;if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp398)->tag == 3U){_LL6: _tmp399=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp398)->f1).elt_type;_tmp39A=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp398)->f1).elt_tq;_tmp39B=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp398)->f1).ptr_atts).rgn;_tmp39C=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp398)->f1).ptr_atts).nullable;_tmp39D=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp398)->f1).ptr_atts).bounds;_tmp39E=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp398)->f1).ptr_atts).zero_term;_LL7: {void*t=_tmp399;struct Cyc_Absyn_Tqual tq=_tmp39A;void*rt=_tmp39B;void*x=_tmp39C;void*b=_tmp39D;void*zt=_tmp39E;
# 2632
{void*_stmttmp53=Cyc_Tcutil_compress(t);void*_tmp39F=_stmttmp53;struct Cyc_List_List*_tmp3AE;struct Cyc_List_List*_tmp3AD;struct Cyc_List_List*_tmp3AC;struct Cyc_Absyn_Exp*_tmp3AB;struct Cyc_List_List*_tmp3AA;struct Cyc_Absyn_Exp*_tmp3A9;struct Cyc_List_List*_tmp3A8;struct Cyc_List_List*_tmp3A7;struct Cyc_Absyn_VarargInfo*_tmp3A6;int _tmp3A5;struct Cyc_List_List*_tmp3A4;void*_tmp3A3;struct Cyc_Absyn_Tqual _tmp3A2;void*_tmp3A1;struct Cyc_List_List*_tmp3A0;if(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp39F)->tag == 5U){_LLB: _tmp3A0=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp39F)->f1).tvars;_tmp3A1=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp39F)->f1).effect;_tmp3A2=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp39F)->f1).ret_tqual;_tmp3A3=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp39F)->f1).ret_type;_tmp3A4=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp39F)->f1).args;_tmp3A5=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp39F)->f1).c_varargs;_tmp3A6=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp39F)->f1).cyc_varargs;_tmp3A7=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp39F)->f1).rgn_po;_tmp3A8=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp39F)->f1).attributes;_tmp3A9=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp39F)->f1).requires_clause;_tmp3AA=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp39F)->f1).requires_relns;_tmp3AB=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp39F)->f1).ensures_clause;_tmp3AC=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp39F)->f1).ensures_relns;_tmp3AD=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp39F)->f1).ieffect;_tmp3AE=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp39F)->f1).oeffect;_LLC: {struct Cyc_List_List*tvs=_tmp3A0;void*eff=_tmp3A1;struct Cyc_Absyn_Tqual rtq=_tmp3A2;void*rtyp=_tmp3A3;struct Cyc_List_List*args=_tmp3A4;int c_varargs=_tmp3A5;struct Cyc_Absyn_VarargInfo*cyc_varargs=_tmp3A6;struct Cyc_List_List*rpo=_tmp3A7;struct Cyc_List_List*atts=_tmp3A8;struct Cyc_Absyn_Exp*req=_tmp3A9;struct Cyc_List_List*req_relns=_tmp3AA;struct Cyc_Absyn_Exp*ens=_tmp3AB;struct Cyc_List_List*ens_relns=_tmp3AC;struct Cyc_List_List*ieff=_tmp3AD;struct Cyc_List_List*oeff=_tmp3AE;
# 2634
if(tvs != 0){
struct _tuple13 env=({struct _tuple13 _tmp4B5;({struct Cyc_List_List*_tmp78F=Cyc_Tcenv_lookup_type_vars(te);_tmp4B5.f1=_tmp78F;}),_tmp4B5.f2=Cyc_Core_heap_region;_tmp4B5;});
struct Cyc_List_List*inst=({
struct Cyc_List_List*(*_tmp790)(struct _tuple14*(*f)(struct _tuple13*,struct Cyc_Absyn_Tvar*),struct _tuple13*env,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp3B4)(struct _tuple14*(*f)(struct _tuple13*,struct Cyc_Absyn_Tvar*),struct _tuple13*env,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct _tuple14*(*f)(struct _tuple13*,struct Cyc_Absyn_Tvar*),struct _tuple13*env,struct Cyc_List_List*x))Cyc_List_map_c;_tmp3B4;});_tmp790(Cyc_Tcutil_r_make_inst_var,& env,tvs);});
struct Cyc_List_List*ts=({struct Cyc_List_List*(*_tmp792)(void*(*f)(struct _tuple14*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp3B2)(void*(*f)(struct _tuple14*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(void*(*f)(struct _tuple14*),struct Cyc_List_List*x))Cyc_List_map;_tmp3B2;});void*(*_tmp791)(struct _tuple14*)=({void*(*_tmp3B3)(struct _tuple14*)=(void*(*)(struct _tuple14*))Cyc_Core_snd;_tmp3B3;});_tmp792(_tmp791,inst);});
# 2645
void*ftyp=({struct Cyc_List_List*_tmp793=inst;Cyc_Tcutil_substitute(_tmp793,(void*)({struct Cyc_Absyn_FnType_Absyn_Type_struct*_tmp3B1=_cycalloc(sizeof(*_tmp3B1));
_tmp3B1->tag=5U,(_tmp3B1->f1).tvars=0,(_tmp3B1->f1).effect=eff,(_tmp3B1->f1).ret_tqual=rtq,(_tmp3B1->f1).ret_type=rtyp,(_tmp3B1->f1).args=args,(_tmp3B1->f1).c_varargs=c_varargs,(_tmp3B1->f1).cyc_varargs=cyc_varargs,(_tmp3B1->f1).rgn_po=0,(_tmp3B1->f1).attributes=atts,(_tmp3B1->f1).requires_clause=req,(_tmp3B1->f1).requires_relns=req_relns,(_tmp3B1->f1).ensures_clause=ens,(_tmp3B1->f1).ensures_relns=ens_relns,(_tmp3B1->f1).ieffect=ieff,(_tmp3B1->f1).oeffect=oeff;_tmp3B1;}));});
# 2650
struct Cyc_Absyn_PointerType_Absyn_Type_struct*new_typ=({struct Cyc_Absyn_PointerType_Absyn_Type_struct*_tmp3B0=_cycalloc(sizeof(*_tmp3B0));_tmp3B0->tag=3U,(_tmp3B0->f1).elt_type=ftyp,(_tmp3B0->f1).elt_tq=tq,((_tmp3B0->f1).ptr_atts).rgn=rt,((_tmp3B0->f1).ptr_atts).nullable=x,((_tmp3B0->f1).ptr_atts).bounds=b,((_tmp3B0->f1).ptr_atts).zero_term=zt,((_tmp3B0->f1).ptr_atts).ptrloc=0;_tmp3B0;});
# 2652
struct Cyc_Absyn_Exp*inner=Cyc_Absyn_copy_exp(e);
({void*_tmp794=(void*)({struct Cyc_Absyn_Instantiate_e_Absyn_Raw_exp_struct*_tmp3AF=_cycalloc(sizeof(*_tmp3AF));_tmp3AF->tag=13U,_tmp3AF->f1=inner,_tmp3AF->f2=ts;_tmp3AF;});e->r=_tmp794;});
e->topt=(void*)new_typ;}
# 2634
goto _LLA;}}else{_LLD: _LLE:
# 2657
 goto _LLA;}_LLA:;}
# 2659
goto _LL5;}}else{_LL8: _LL9:
 goto _LL5;}_LL5:;}
# 2662
goto _LL0;}_LL0:;}
# 2664
return(void*)_check_null(e->topt);}
# 306 "tcexp.cyc"
static void Cyc_Tcexp_insert_alias_stmts(struct Cyc_Tcenv_Tenv*te,struct Cyc_Absyn_Exp*e,struct Cyc_Absyn_Exp*fn_exp,struct Cyc_List_List*alias_arg_exps){
# 2675 "tcexp.cyc"
struct Cyc_List_List*decls=0;
# 2677
{void*_stmttmp54=fn_exp->r;void*_tmp3B5=_stmttmp54;struct Cyc_List_List*_tmp3B6;if(((struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct*)_tmp3B5)->tag == 10U){_LL1: _tmp3B6=((struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct*)_tmp3B5)->f2;_LL2: {struct Cyc_List_List*es=_tmp3B6;
# 2680
{void*_stmttmp55=e->r;void*_tmp3B7=_stmttmp55;struct Cyc_List_List*_tmp3B8;if(((struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct*)_tmp3B7)->tag == 10U){_LL6: _tmp3B8=((struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct*)_tmp3B7)->f2;_LL7: {struct Cyc_List_List*es2=_tmp3B8;
# 2683
struct Cyc_List_List*arg_exps=alias_arg_exps;
int arg_cnt=0;
while(arg_exps != 0){
# 2687
while(arg_cnt != (int)arg_exps->hd){
# 2689
if(es == 0)
({void*_tmp3B9=0U;({int(*_tmp797)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp3BE)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp3BE;});struct _fat_ptr _tmp796=(struct _fat_ptr)({struct Cyc_Int_pa_PrintArg_struct _tmp3BC=({struct Cyc_Int_pa_PrintArg_struct _tmp4B7;_tmp4B7.tag=1U,_tmp4B7.f1=(unsigned long)arg_cnt;_tmp4B7;});struct Cyc_Int_pa_PrintArg_struct _tmp3BD=({struct Cyc_Int_pa_PrintArg_struct _tmp4B6;_tmp4B6.tag=1U,_tmp4B6.f1=(unsigned long)((int)arg_exps->hd);_tmp4B6;});void*_tmp3BA[2U];_tmp3BA[0]=& _tmp3BC,_tmp3BA[1]=& _tmp3BD;({struct _fat_ptr _tmp795=({const char*_tmp3BB="bad count %d/%d for alias coercion!";_tag_fat(_tmp3BB,sizeof(char),36U);});Cyc_aprintf(_tmp795,_tag_fat(_tmp3BA,sizeof(void*),2U));});});_tmp797(_tmp796,_tag_fat(_tmp3B9,sizeof(void*),0U));});});
# 2689
++ arg_cnt;
# 2693
es=es->tl;
es2=((struct Cyc_List_List*)_check_null(es2))->tl;}{
# 2697
struct _tuple12 _stmttmp56=({struct Cyc_Absyn_Exp*_tmp798=(struct Cyc_Absyn_Exp*)((struct Cyc_List_List*)_check_null(es))->hd;Cyc_Tcutil_insert_alias(_tmp798,Cyc_Tcutil_copy_type((void*)_check_null(((struct Cyc_Absyn_Exp*)((struct Cyc_List_List*)_check_null(es2))->hd)->topt)));});struct Cyc_Absyn_Exp*_tmp3C0;struct Cyc_Absyn_Decl*_tmp3BF;_LLB: _tmp3BF=_stmttmp56.f1;_tmp3C0=_stmttmp56.f2;_LLC: {struct Cyc_Absyn_Decl*d=_tmp3BF;struct Cyc_Absyn_Exp*ve=_tmp3C0;
es->hd=(void*)ve;
decls=({struct Cyc_List_List*_tmp3C1=_cycalloc(sizeof(*_tmp3C1));_tmp3C1->hd=d,_tmp3C1->tl=decls;_tmp3C1;});
arg_exps=arg_exps->tl;}}}
# 2702
goto _LL5;}}else{_LL8: _LL9:
# 2704
({void*_tmp3C2=0U;({int(*_tmp79A)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp3C4)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp3C4;});struct _fat_ptr _tmp799=({const char*_tmp3C3="not a function call!";_tag_fat(_tmp3C3,sizeof(char),21U);});_tmp79A(_tmp799,_tag_fat(_tmp3C2,sizeof(void*),0U));});});}_LL5:;}
# 2706
goto _LL0;}}else{_LL3: _LL4:
# 2708
({void*_tmp3C5=0U;({int(*_tmp79C)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp3C7)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp3C7;});struct _fat_ptr _tmp79B=({const char*_tmp3C6="not a function call!";_tag_fat(_tmp3C6,sizeof(char),21U);});_tmp79C(_tmp79B,_tag_fat(_tmp3C5,sizeof(void*),0U));});});}_LL0:;}
# 2712
while(decls != 0){
# 2714
struct Cyc_Absyn_Decl*d=(struct Cyc_Absyn_Decl*)decls->hd;
fn_exp=({struct Cyc_Absyn_Stmt*_tmp79F=({struct Cyc_Absyn_Decl*_tmp79E=d;struct Cyc_Absyn_Stmt*_tmp79D=Cyc_Absyn_exp_stmt(fn_exp,e->loc);Cyc_Absyn_decl_stmt(_tmp79E,_tmp79D,e->loc);});Cyc_Absyn_stmt_exp(_tmp79F,e->loc);});
decls=decls->tl;}
# 2720
e->topt=0;
e->r=fn_exp->r;}
# 306 "tcexp.cyc"
static void Cyc_Tcexp_tcExpNoInst(struct Cyc_Tcenv_Tenv*te,void**topt,struct Cyc_Absyn_Exp*e){
# 2727 "tcexp.cyc"
unsigned loc=e->loc;
void*t;
# 2730
{void*_stmttmp57=e->r;void*_tmp3C8=_stmttmp57;void*_tmp3C9;struct Cyc_Absyn_Exp*_tmp3CA;struct _fat_ptr*_tmp3CC;struct Cyc_Absyn_Exp*_tmp3CB;struct Cyc_Absyn_Exp*_tmp3CE;struct Cyc_Absyn_Exp*_tmp3CD;int*_tmp3D3;struct Cyc_Absyn_Exp**_tmp3D2;void***_tmp3D1;struct Cyc_Absyn_Exp**_tmp3D0;int*_tmp3CF;struct Cyc_Absyn_Enumfield*_tmp3D5;void*_tmp3D4;struct Cyc_Absyn_Enumfield*_tmp3D7;struct Cyc_Absyn_Enumdecl*_tmp3D6;struct Cyc_Absyn_Datatypefield*_tmp3DA;struct Cyc_Absyn_Datatypedecl*_tmp3D9;struct Cyc_List_List*_tmp3D8;struct Cyc_List_List*_tmp3DC;void*_tmp3DB;struct Cyc_Absyn_Aggrdecl**_tmp3E0;struct Cyc_List_List*_tmp3DF;struct Cyc_List_List**_tmp3DE;struct _tuple1**_tmp3DD;int*_tmp3E3;void*_tmp3E2;struct Cyc_Absyn_Exp*_tmp3E1;int*_tmp3E7;struct Cyc_Absyn_Exp*_tmp3E6;struct Cyc_Absyn_Exp*_tmp3E5;struct Cyc_Absyn_Vardecl*_tmp3E4;struct Cyc_Absyn_Stmt*_tmp3E8;struct Cyc_List_List*_tmp3E9;struct Cyc_List_List*_tmp3EB;struct _tuple9*_tmp3EA;struct Cyc_List_List*_tmp3EC;struct Cyc_Absyn_Exp*_tmp3EE;struct Cyc_Absyn_Exp*_tmp3ED;int*_tmp3F2;int*_tmp3F1;struct _fat_ptr*_tmp3F0;struct Cyc_Absyn_Exp*_tmp3EF;int*_tmp3F6;int*_tmp3F5;struct _fat_ptr*_tmp3F4;struct Cyc_Absyn_Exp*_tmp3F3;struct Cyc_Absyn_Exp*_tmp3F7;struct Cyc_List_List*_tmp3F9;void*_tmp3F8;void*_tmp3FA;struct Cyc_Absyn_Exp*_tmp3FB;struct Cyc_Absyn_Exp*_tmp3FD;struct Cyc_Absyn_Exp**_tmp3FC;struct Cyc_Absyn_Exp*_tmp3FE;enum Cyc_Absyn_Coercion*_tmp401;struct Cyc_Absyn_Exp*_tmp400;void*_tmp3FF;struct Cyc_List_List*_tmp403;struct Cyc_Absyn_Exp*_tmp402;struct Cyc_Absyn_Exp*_tmp404;struct Cyc_Absyn_Exp*_tmp406;struct Cyc_Absyn_Exp*_tmp405;struct Cyc_Absyn_Exp*_tmp408;struct Cyc_Absyn_Exp*_tmp407;struct Cyc_Absyn_Exp*_tmp40A;struct Cyc_Absyn_Exp*_tmp409;struct Cyc_Absyn_Exp*_tmp40C;struct Cyc_Absyn_Exp*_tmp40B;struct Cyc_Absyn_Exp*_tmp40F;struct Cyc_Absyn_Exp*_tmp40E;struct Cyc_Absyn_Exp*_tmp40D;struct Cyc_Absyn_Exp*_tmp412;struct Cyc_Core_Opt*_tmp411;struct Cyc_Absyn_Exp*_tmp410;enum Cyc_Absyn_Incrementor _tmp414;struct Cyc_Absyn_Exp*_tmp413;struct Cyc_List_List*_tmp416;enum Cyc_Absyn_Primop _tmp415;void**_tmp417;union Cyc_Absyn_Cnst*_tmp418;struct Cyc_List_List*_tmp41A;struct Cyc_Core_Opt*_tmp419;struct Cyc_Absyn_VarargCallInfo**_tmp41D;struct Cyc_List_List*_tmp41C;struct Cyc_Absyn_Exp*_tmp41B;int*_tmp421;struct Cyc_Absyn_VarargCallInfo**_tmp420;struct Cyc_List_List*_tmp41F;struct Cyc_Absyn_Exp*_tmp41E;struct Cyc_Absyn_Exp*_tmp422;switch(*((int*)_tmp3C8)){case 12U: _LL1: _tmp422=((struct Cyc_Absyn_NoInstantiate_e_Absyn_Raw_exp_struct*)_tmp3C8)->f1;_LL2: {struct Cyc_Absyn_Exp*e2=_tmp422;
# 2735
Cyc_Tcexp_tcExpNoInst(te,0,e2);
return;}case 10U: _LL3: _tmp41E=((struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct*)_tmp3C8)->f1;_tmp41F=((struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct*)_tmp3C8)->f2;_tmp420=(struct Cyc_Absyn_VarargCallInfo**)&((struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct*)_tmp3C8)->f3;_tmp421=(int*)&((struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct*)_tmp3C8)->f4;if(!(*_tmp421)){_LL4: {struct Cyc_Absyn_Exp*e1=_tmp41E;struct Cyc_List_List*es=_tmp41F;struct Cyc_Absyn_VarargCallInfo**vci=_tmp420;int*resolved=_tmp421;
# 2739
({void*_tmp423=0U;({int(*_tmp7A1)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp425)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp425;});struct _fat_ptr _tmp7A0=({const char*_tmp424="unresolved function in tcExpNoInst";_tag_fat(_tmp424,sizeof(char),35U);});_tmp7A1(_tmp7A0,_tag_fat(_tmp423,sizeof(void*),0U));});});}}else{_LL1D: _tmp41B=((struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct*)_tmp3C8)->f1;_tmp41C=((struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct*)_tmp3C8)->f2;_tmp41D=(struct Cyc_Absyn_VarargCallInfo**)&((struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct*)_tmp3C8)->f3;_LL1E: {struct Cyc_Absyn_Exp*e1=_tmp41B;struct Cyc_List_List*es=_tmp41C;struct Cyc_Absyn_VarargCallInfo**vci=_tmp41D;
# 2776
struct Cyc_List_List*alias_arg_exps=0;
int ok=1;
struct Cyc_Absyn_Exp*fn_exp;
{struct _handler_cons _tmp426;_push_handler(& _tmp426);{int _tmp428=0;if(setjmp(_tmp426.handler))_tmp428=1;if(!_tmp428){
fn_exp=Cyc_Tcutil_deep_copy_exp(0,e);;_pop_handler();}else{void*_tmp427=(void*)Cyc_Core_get_exn_thrown();void*_tmp429=_tmp427;void*_tmp42A;if(((struct Cyc_Core_Failure_exn_struct*)_tmp429)->tag == Cyc_Core_Failure){_LL5A: _LL5B:
# 2783
 ok=0;
fn_exp=e;
goto _LL59;}else{_LL5C: _tmp42A=_tmp429;_LL5D: {void*exn=_tmp42A;(int)_rethrow(exn);}}_LL59:;}}}
# 2787
t=Cyc_Tcexp_tcFnCall(te,loc,topt,e1,es,vci,& alias_arg_exps);
if(alias_arg_exps != 0 && ok){
alias_arg_exps=Cyc_List_imp_rev(alias_arg_exps);
Cyc_Tcexp_insert_alias_stmts(te,e,fn_exp,alias_arg_exps);
Cyc_Tcexp_tcExpNoInst(te,topt,e);
return;}
# 2788
goto _LL0;}}case 37U: _LL5: _tmp419=((struct Cyc_Absyn_UnresolvedMem_e_Absyn_Raw_exp_struct*)_tmp3C8)->f1;_tmp41A=((struct Cyc_Absyn_UnresolvedMem_e_Absyn_Raw_exp_struct*)_tmp3C8)->f2;_LL6: {struct Cyc_Core_Opt*nopt=_tmp419;struct Cyc_List_List*des=_tmp41A;
# 2743
Cyc_Tcexp_resolve_unresolved_mem(te,loc,topt,e,des);
Cyc_Tcexp_tcExpNoInst(te,topt,e);
return;}case 0U: _LL7: _tmp418=(union Cyc_Absyn_Cnst*)&((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmp3C8)->f1;_LL8: {union Cyc_Absyn_Cnst*c=_tmp418;
# 2748
t=Cyc_Tcexp_tcConst(te,loc,topt,c,e);goto _LL0;}case 1U: _LL9: _tmp417=(void**)&((struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct*)_tmp3C8)->f1;_LLA: {void**b=_tmp417;
# 2750
t=Cyc_Tcexp_tcVar(te,loc,topt,e,b);goto _LL0;}case 2U: _LLB: _LLC:
# 2752
 t=Cyc_Absyn_sint_type;goto _LL0;case 3U: _LLD: _tmp415=((struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct*)_tmp3C8)->f1;_tmp416=((struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct*)_tmp3C8)->f2;_LLE: {enum Cyc_Absyn_Primop p=_tmp415;struct Cyc_List_List*es=_tmp416;
# 2754
t=Cyc_Tcexp_tcPrimop(te,loc,topt,p,es);goto _LL0;}case 5U: _LLF: _tmp413=((struct Cyc_Absyn_Increment_e_Absyn_Raw_exp_struct*)_tmp3C8)->f1;_tmp414=((struct Cyc_Absyn_Increment_e_Absyn_Raw_exp_struct*)_tmp3C8)->f2;_LL10: {struct Cyc_Absyn_Exp*e1=_tmp413;enum Cyc_Absyn_Incrementor i=_tmp414;
# 2756
t=Cyc_Tcexp_tcIncrement(te,loc,topt,e1,i);goto _LL0;}case 4U: _LL11: _tmp410=((struct Cyc_Absyn_AssignOp_e_Absyn_Raw_exp_struct*)_tmp3C8)->f1;_tmp411=((struct Cyc_Absyn_AssignOp_e_Absyn_Raw_exp_struct*)_tmp3C8)->f2;_tmp412=((struct Cyc_Absyn_AssignOp_e_Absyn_Raw_exp_struct*)_tmp3C8)->f3;_LL12: {struct Cyc_Absyn_Exp*e1=_tmp410;struct Cyc_Core_Opt*popt=_tmp411;struct Cyc_Absyn_Exp*e2=_tmp412;
# 2758
t=Cyc_Tcexp_tcAssignOp(te,loc,topt,e1,popt,e2);goto _LL0;}case 6U: _LL13: _tmp40D=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmp3C8)->f1;_tmp40E=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmp3C8)->f2;_tmp40F=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmp3C8)->f3;_LL14: {struct Cyc_Absyn_Exp*e1=_tmp40D;struct Cyc_Absyn_Exp*e2=_tmp40E;struct Cyc_Absyn_Exp*e3=_tmp40F;
# 2760
t=Cyc_Tcexp_tcConditional(te,loc,topt,e1,e2,e3);goto _LL0;}case 7U: _LL15: _tmp40B=((struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct*)_tmp3C8)->f1;_tmp40C=((struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct*)_tmp3C8)->f2;_LL16: {struct Cyc_Absyn_Exp*e1=_tmp40B;struct Cyc_Absyn_Exp*e2=_tmp40C;
# 2762
t=Cyc_Tcexp_tcAnd(te,loc,e1,e2);goto _LL0;}case 8U: _LL17: _tmp409=((struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct*)_tmp3C8)->f1;_tmp40A=((struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct*)_tmp3C8)->f2;_LL18: {struct Cyc_Absyn_Exp*e1=_tmp409;struct Cyc_Absyn_Exp*e2=_tmp40A;
# 2764
t=Cyc_Tcexp_tcOr(te,loc,e1,e2);goto _LL0;}case 34U: _LL19: _tmp407=((struct Cyc_Absyn_Spawn_e_Absyn_Raw_exp_struct*)_tmp3C8)->f1;_tmp408=((struct Cyc_Absyn_Spawn_e_Absyn_Raw_exp_struct*)_tmp3C8)->f2;_LL1A: {struct Cyc_Absyn_Exp*e1=_tmp407;struct Cyc_Absyn_Exp*e2=_tmp408;
# 2767
t=Cyc_Tcexp_tcSpawn(te,loc,topt,e1,e2);goto _LL0;}case 9U: _LL1B: _tmp405=((struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct*)_tmp3C8)->f1;_tmp406=((struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct*)_tmp3C8)->f2;_LL1C: {struct Cyc_Absyn_Exp*e1=_tmp405;struct Cyc_Absyn_Exp*e2=_tmp406;
# 2770
t=Cyc_Tcexp_tcSeqExp(te,loc,topt,e1,e2);goto _LL0;}case 11U: _LL1F: _tmp404=((struct Cyc_Absyn_Throw_e_Absyn_Raw_exp_struct*)_tmp3C8)->f1;_LL20: {struct Cyc_Absyn_Exp*e1=_tmp404;
# 2796
t=Cyc_Tcexp_tcThrow(te,loc,topt,e1);goto _LL0;}case 13U: _LL21: _tmp402=((struct Cyc_Absyn_Instantiate_e_Absyn_Raw_exp_struct*)_tmp3C8)->f1;_tmp403=((struct Cyc_Absyn_Instantiate_e_Absyn_Raw_exp_struct*)_tmp3C8)->f2;_LL22: {struct Cyc_Absyn_Exp*e2=_tmp402;struct Cyc_List_List*ts=_tmp403;
# 2798
t=Cyc_Tcexp_tcInstantiate(te,loc,topt,e2,ts);goto _LL0;}case 14U: _LL23: _tmp3FF=(void*)((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp3C8)->f1;_tmp400=((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp3C8)->f2;_tmp401=(enum Cyc_Absyn_Coercion*)&((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp3C8)->f4;_LL24: {void*t1=_tmp3FF;struct Cyc_Absyn_Exp*e1=_tmp400;enum Cyc_Absyn_Coercion*c=_tmp401;
# 2800
t=Cyc_Tcexp_tcCast(te,loc,topt,t1,e1,c);goto _LL0;}case 15U: _LL25: _tmp3FE=((struct Cyc_Absyn_Address_e_Absyn_Raw_exp_struct*)_tmp3C8)->f1;_LL26: {struct Cyc_Absyn_Exp*e1=_tmp3FE;
# 2802
t=Cyc_Tcexp_tcAddress(te,loc,e,topt,e1);goto _LL0;}case 16U: _LL27: _tmp3FC=(struct Cyc_Absyn_Exp**)&((struct Cyc_Absyn_New_e_Absyn_Raw_exp_struct*)_tmp3C8)->f1;_tmp3FD=((struct Cyc_Absyn_New_e_Absyn_Raw_exp_struct*)_tmp3C8)->f2;_LL28: {struct Cyc_Absyn_Exp**rgn_handle=_tmp3FC;struct Cyc_Absyn_Exp*e1=_tmp3FD;
# 2804
t=Cyc_Tcexp_tcNew(te,loc,topt,rgn_handle,e,e1);goto _LL0;}case 18U: _LL29: _tmp3FB=((struct Cyc_Absyn_Sizeofexp_e_Absyn_Raw_exp_struct*)_tmp3C8)->f1;_LL2A: {struct Cyc_Absyn_Exp*e1=_tmp3FB;
# 2806
void*t1=Cyc_Tcexp_tcExpNoPromote(te,0,e1);
t=Cyc_Tcexp_tcSizeof(te,loc,topt,t1);goto _LL0;}case 17U: _LL2B: _tmp3FA=(void*)((struct Cyc_Absyn_Sizeoftype_e_Absyn_Raw_exp_struct*)_tmp3C8)->f1;_LL2C: {void*t1=_tmp3FA;
# 2809
t=Cyc_Tcexp_tcSizeof(te,loc,topt,t1);goto _LL0;}case 19U: _LL2D: _tmp3F8=(void*)((struct Cyc_Absyn_Offsetof_e_Absyn_Raw_exp_struct*)_tmp3C8)->f1;_tmp3F9=((struct Cyc_Absyn_Offsetof_e_Absyn_Raw_exp_struct*)_tmp3C8)->f2;_LL2E: {void*t1=_tmp3F8;struct Cyc_List_List*l=_tmp3F9;
# 2811
t=Cyc_Tcexp_tcOffsetof(te,loc,topt,t1,l);goto _LL0;}case 20U: _LL2F: _tmp3F7=((struct Cyc_Absyn_Deref_e_Absyn_Raw_exp_struct*)_tmp3C8)->f1;_LL30: {struct Cyc_Absyn_Exp*e1=_tmp3F7;
# 2813
t=Cyc_Tcexp_tcDeref(te,loc,topt,e1);goto _LL0;}case 21U: _LL31: _tmp3F3=((struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct*)_tmp3C8)->f1;_tmp3F4=((struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct*)_tmp3C8)->f2;_tmp3F5=(int*)&((struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct*)_tmp3C8)->f3;_tmp3F6=(int*)&((struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct*)_tmp3C8)->f4;_LL32: {struct Cyc_Absyn_Exp*e1=_tmp3F3;struct _fat_ptr*f=_tmp3F4;int*is_tagged=_tmp3F5;int*is_read=_tmp3F6;
# 2815
t=Cyc_Tcexp_tcAggrMember(te,loc,topt,e,e1,f,is_tagged,is_read);goto _LL0;}case 22U: _LL33: _tmp3EF=((struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct*)_tmp3C8)->f1;_tmp3F0=((struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct*)_tmp3C8)->f2;_tmp3F1=(int*)&((struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct*)_tmp3C8)->f3;_tmp3F2=(int*)&((struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct*)_tmp3C8)->f4;_LL34: {struct Cyc_Absyn_Exp*e1=_tmp3EF;struct _fat_ptr*f=_tmp3F0;int*is_tagged=_tmp3F1;int*is_read=_tmp3F2;
# 2817
t=Cyc_Tcexp_tcAggrArrow(te,loc,topt,e1,f,is_tagged,is_read);goto _LL0;}case 23U: _LL35: _tmp3ED=((struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct*)_tmp3C8)->f1;_tmp3EE=((struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct*)_tmp3C8)->f2;_LL36: {struct Cyc_Absyn_Exp*e1=_tmp3ED;struct Cyc_Absyn_Exp*e2=_tmp3EE;
# 2819
t=Cyc_Tcexp_tcSubscript(te,loc,topt,e1,e2);goto _LL0;}case 24U: _LL37: _tmp3EC=((struct Cyc_Absyn_Tuple_e_Absyn_Raw_exp_struct*)_tmp3C8)->f1;_LL38: {struct Cyc_List_List*es=_tmp3EC;
# 2821
t=Cyc_Tcexp_tcTuple(te,loc,topt,es);goto _LL0;}case 25U: _LL39: _tmp3EA=((struct Cyc_Absyn_CompoundLit_e_Absyn_Raw_exp_struct*)_tmp3C8)->f1;_tmp3EB=((struct Cyc_Absyn_CompoundLit_e_Absyn_Raw_exp_struct*)_tmp3C8)->f2;_LL3A: {struct _tuple9*t1=_tmp3EA;struct Cyc_List_List*des=_tmp3EB;
# 2823
t=Cyc_Tcexp_tcCompoundLit(te,loc,e,topt,t1,des);goto _LL0;}case 26U: _LL3B: _tmp3E9=((struct Cyc_Absyn_Array_e_Absyn_Raw_exp_struct*)_tmp3C8)->f1;_LL3C: {struct Cyc_List_List*des=_tmp3E9;
# 2827
void**elt_topt=0;
struct Cyc_Absyn_Tqual*elt_tqopt=0;
int zero_term=0;
if(topt != 0){
void*_stmttmp58=Cyc_Tcutil_compress(*topt);void*_tmp42B=_stmttmp58;void*_tmp42E;struct Cyc_Absyn_Tqual*_tmp42D;void**_tmp42C;if(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp42B)->tag == 4U){_LL5F: _tmp42C=(void**)&(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp42B)->f1).elt_type;_tmp42D=(struct Cyc_Absyn_Tqual*)&(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp42B)->f1).tq;_tmp42E=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp42B)->f1).zero_term;_LL60: {void**et=_tmp42C;struct Cyc_Absyn_Tqual*etq=_tmp42D;void*zt=_tmp42E;
# 2833
elt_topt=et;
elt_tqopt=etq;
zero_term=Cyc_Tcutil_force_type2bool(0,zt);
goto _LL5E;}}else{_LL61: _LL62:
 goto _LL5E;}_LL5E:;}
# 2830
t=
# 2840
Cyc_Tcexp_tcArray(te,loc,elt_topt,elt_tqopt,zero_term,des);goto _LL0;}case 38U: _LL3D: _tmp3E8=((struct Cyc_Absyn_StmtExp_e_Absyn_Raw_exp_struct*)_tmp3C8)->f1;_LL3E: {struct Cyc_Absyn_Stmt*s=_tmp3E8;
# 2842
t=Cyc_Tcexp_tcStmtExp(te,loc,topt,s);goto _LL0;}case 27U: _LL3F: _tmp3E4=((struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct*)_tmp3C8)->f1;_tmp3E5=((struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct*)_tmp3C8)->f2;_tmp3E6=((struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct*)_tmp3C8)->f3;_tmp3E7=(int*)&((struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct*)_tmp3C8)->f4;_LL40: {struct Cyc_Absyn_Vardecl*vd=_tmp3E4;struct Cyc_Absyn_Exp*e1=_tmp3E5;struct Cyc_Absyn_Exp*e2=_tmp3E6;int*iszeroterm=_tmp3E7;
# 2844
t=Cyc_Tcexp_tcComprehension(te,loc,topt,vd,e1,e2,iszeroterm);goto _LL0;}case 28U: _LL41: _tmp3E1=((struct Cyc_Absyn_ComprehensionNoinit_e_Absyn_Raw_exp_struct*)_tmp3C8)->f1;_tmp3E2=(void*)((struct Cyc_Absyn_ComprehensionNoinit_e_Absyn_Raw_exp_struct*)_tmp3C8)->f2;_tmp3E3=(int*)&((struct Cyc_Absyn_ComprehensionNoinit_e_Absyn_Raw_exp_struct*)_tmp3C8)->f3;_LL42: {struct Cyc_Absyn_Exp*e1=_tmp3E1;void*t1=_tmp3E2;int*iszeroterm=_tmp3E3;
# 2846
t=Cyc_Tcexp_tcComprehensionNoinit(te,loc,topt,e1,t1,iszeroterm);goto _LL0;}case 29U: _LL43: _tmp3DD=(struct _tuple1**)&((struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct*)_tmp3C8)->f1;_tmp3DE=(struct Cyc_List_List**)&((struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct*)_tmp3C8)->f2;_tmp3DF=((struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct*)_tmp3C8)->f3;_tmp3E0=(struct Cyc_Absyn_Aggrdecl**)&((struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct*)_tmp3C8)->f4;_LL44: {struct _tuple1**tn=_tmp3DD;struct Cyc_List_List**ts=_tmp3DE;struct Cyc_List_List*args=_tmp3DF;struct Cyc_Absyn_Aggrdecl**sd_opt=_tmp3E0;
# 2848
t=Cyc_Tcexp_tcAggregate(te,loc,topt,tn,ts,args,sd_opt);goto _LL0;}case 30U: _LL45: _tmp3DB=(void*)((struct Cyc_Absyn_AnonStruct_e_Absyn_Raw_exp_struct*)_tmp3C8)->f1;_tmp3DC=((struct Cyc_Absyn_AnonStruct_e_Absyn_Raw_exp_struct*)_tmp3C8)->f2;_LL46: {void*ts=_tmp3DB;struct Cyc_List_List*args=_tmp3DC;
# 2850
t=Cyc_Tcexp_tcAnonStruct(te,loc,ts,args);goto _LL0;}case 31U: _LL47: _tmp3D8=((struct Cyc_Absyn_Datatype_e_Absyn_Raw_exp_struct*)_tmp3C8)->f1;_tmp3D9=((struct Cyc_Absyn_Datatype_e_Absyn_Raw_exp_struct*)_tmp3C8)->f2;_tmp3DA=((struct Cyc_Absyn_Datatype_e_Absyn_Raw_exp_struct*)_tmp3C8)->f3;_LL48: {struct Cyc_List_List*es=_tmp3D8;struct Cyc_Absyn_Datatypedecl*tud=_tmp3D9;struct Cyc_Absyn_Datatypefield*tuf=_tmp3DA;
# 2852
t=Cyc_Tcexp_tcDatatype(te,loc,topt,e,es,tud,tuf);goto _LL0;}case 32U: _LL49: _tmp3D6=((struct Cyc_Absyn_Enum_e_Absyn_Raw_exp_struct*)_tmp3C8)->f1;_tmp3D7=((struct Cyc_Absyn_Enum_e_Absyn_Raw_exp_struct*)_tmp3C8)->f2;_LL4A: {struct Cyc_Absyn_Enumdecl*ed=_tmp3D6;struct Cyc_Absyn_Enumfield*ef=_tmp3D7;
# 2854
t=Cyc_Absyn_enum_type(ed->name,ed);goto _LL0;}case 33U: _LL4B: _tmp3D4=(void*)((struct Cyc_Absyn_AnonEnum_e_Absyn_Raw_exp_struct*)_tmp3C8)->f1;_tmp3D5=((struct Cyc_Absyn_AnonEnum_e_Absyn_Raw_exp_struct*)_tmp3C8)->f2;_LL4C: {void*t2=_tmp3D4;struct Cyc_Absyn_Enumfield*ef=_tmp3D5;
# 2856
t=t2;goto _LL0;}case 35U: _LL4D: _tmp3CF=(int*)&(((struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct*)_tmp3C8)->f1).is_calloc;_tmp3D0=(struct Cyc_Absyn_Exp**)&(((struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct*)_tmp3C8)->f1).rgn;_tmp3D1=(void***)&(((struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct*)_tmp3C8)->f1).elt_type;_tmp3D2=(struct Cyc_Absyn_Exp**)&(((struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct*)_tmp3C8)->f1).num_elts;_tmp3D3=(int*)&(((struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct*)_tmp3C8)->f1).fat_result;_LL4E: {int*is_calloc=_tmp3CF;struct Cyc_Absyn_Exp**ropt=_tmp3D0;void***t2=_tmp3D1;struct Cyc_Absyn_Exp**e2=_tmp3D2;int*isfat=_tmp3D3;
# 2858
t=Cyc_Tcexp_tcMalloc(te,loc,topt,ropt,t2,e2,is_calloc,isfat);goto _LL0;}case 36U: _LL4F: _tmp3CD=((struct Cyc_Absyn_Swap_e_Absyn_Raw_exp_struct*)_tmp3C8)->f1;_tmp3CE=((struct Cyc_Absyn_Swap_e_Absyn_Raw_exp_struct*)_tmp3C8)->f2;_LL50: {struct Cyc_Absyn_Exp*e1=_tmp3CD;struct Cyc_Absyn_Exp*e2=_tmp3CE;
# 2860
t=Cyc_Tcexp_tcSwap(te,loc,topt,e1,e2);goto _LL0;}case 39U: _LL51: _tmp3CB=((struct Cyc_Absyn_Tagcheck_e_Absyn_Raw_exp_struct*)_tmp3C8)->f1;_tmp3CC=((struct Cyc_Absyn_Tagcheck_e_Absyn_Raw_exp_struct*)_tmp3C8)->f2;_LL52: {struct Cyc_Absyn_Exp*e=_tmp3CB;struct _fat_ptr*f=_tmp3CC;
# 2862
t=Cyc_Tcexp_tcTagcheck(te,loc,topt,e,f);goto _LL0;}case 42U: _LL53: _tmp3CA=((struct Cyc_Absyn_Extension_e_Absyn_Raw_exp_struct*)_tmp3C8)->f1;_LL54: {struct Cyc_Absyn_Exp*e1=_tmp3CA;
# 2864
t=Cyc_Tcexp_tcExp(te,topt,e1);goto _LL0;}case 40U: _LL55: _tmp3C9=(void*)((struct Cyc_Absyn_Valueof_e_Absyn_Raw_exp_struct*)_tmp3C8)->f1;_LL56: {void*t2=_tmp3C9;
# 2866
if(!te->allow_valueof)
({void*_tmp42F=0U;({unsigned _tmp7A3=e->loc;struct _fat_ptr _tmp7A2=({const char*_tmp430="valueof(-) can only occur within types";_tag_fat(_tmp430,sizeof(char),39U);});Cyc_Tcutil_terr(_tmp7A3,_tmp7A2,_tag_fat(_tmp42F,sizeof(void*),0U));});});
# 2866
t=Cyc_Absyn_sint_type;
# 2875
goto _LL0;}default: _LL57: _LL58:
# 2878
 t=Cyc_Absyn_void_type;
goto _LL0;}_LL0:;}
# 2881
e->topt=t;}
