#include <setjmp.h>
/* This is a C header file to be used by the output of the Cyclone to
   C translator.  The corresponding definitions are in file lib/runtime_*.c */
#ifndef _CYC_INCLUDE_H_
#define _CYC_INCLUDE_H_

/* Need one of these per thread (see runtime_stack.c). The runtime maintains 
   a stack that contains either _handler_cons structs or _RegionHandle structs.
   The tag is 0 for a handler_cons and 1 for a region handle.  */
struct _RuntimeStack {
  int tag; 
  struct _RuntimeStack *next;
  void (*cleanup)(struct _RuntimeStack *frame);
};

#ifndef offsetof
/* should be size_t, but int is fine. */
#define offsetof(t,n) ((int)(&(((t *)0)->n)))
#endif

/* Fat pointers */
struct _fat_ptr {
  unsigned char *curr; 
  unsigned char *base; 
  unsigned char *last_plus_one; 
};  

/* Discriminated Unions */
struct _xtunion_struct { char *tag; };

/* Regions */
struct _RegionPage
#ifdef CYC_REGION_PROFILE
{ unsigned total_bytes;
  unsigned free_bytes;
  /* MWH: wish we didn't have to include the stuff below ... */
  struct _RegionPage *next;
  char data[1];
}
#endif
; // abstract -- defined in runtime_memory.c
struct _RegionHandle {
  struct _RuntimeStack s;
  struct _RegionPage *curr;
  char               *offset;
  char               *last_plus_one;
  struct _DynRegionHandle *sub_regions;
#ifdef CYC_REGION_PROFILE
  const char         *name;
#else
  unsigned used_bytes;
  unsigned wasted_bytes;
#endif
};
struct _DynRegionFrame {
  struct _RuntimeStack s;
  struct _DynRegionHandle *x;
};
// A dynamic region is just a region handle.  The wrapper struct is for type
// abstraction.
struct Cyc_Core_DynamicRegion {
  struct _RegionHandle h;
};

struct _RegionHandle _new_region(const char*);
void* _region_malloc(struct _RegionHandle*, unsigned);
void* _region_calloc(struct _RegionHandle*, unsigned t, unsigned n);
void   _free_region(struct _RegionHandle*);
struct _RegionHandle*_open_dynregion(struct _DynRegionFrame*,struct _DynRegionHandle*);
void   _pop_dynregion();

/* Exceptions */
struct _handler_cons {
  struct _RuntimeStack s;
  jmp_buf handler;
};
void _push_handler(struct _handler_cons *);
void _push_region(struct _RegionHandle *);
void _npop_handler(int);
void _pop_handler();
void _pop_region();

#ifndef _throw
void* _throw_null_fn(const char*,unsigned);
void* _throw_arraybounds_fn(const char*,unsigned);
void* _throw_badalloc_fn(const char*,unsigned);
void* _throw_match_fn(const char*,unsigned);
void* _throw_fn(void*,const char*,unsigned);
void* _rethrow(void*);
#define _throw_null() (_throw_null_fn(__FILE__,__LINE__))
#define _throw_arraybounds() (_throw_arraybounds_fn(__FILE__,__LINE__))
#define _throw_badalloc() (_throw_badalloc_fn(__FILE__,__LINE__))
#define _throw_match() (_throw_match_fn(__FILE__,__LINE__))
#define _throw(e) (_throw_fn((e),__FILE__,__LINE__))
#endif

struct _xtunion_struct* Cyc_Core_get_exn_thrown();
/* Built-in Exceptions */
struct Cyc_Null_Exception_exn_struct { char *tag; };
struct Cyc_Array_bounds_exn_struct { char *tag; };
struct Cyc_Match_Exception_exn_struct { char *tag; };
struct Cyc_Bad_alloc_exn_struct { char *tag; };
extern char Cyc_Null_Exception[];
extern char Cyc_Array_bounds[];
extern char Cyc_Match_Exception[];
extern char Cyc_Bad_alloc[];

/* Built-in Run-time Checks and company */
#ifdef CYC_ANSI_OUTPUT
#define _INLINE  
#else
#define _INLINE inline
#endif

#ifdef NO_CYC_NULL_CHECKS
#define _check_null(ptr) (ptr)
#else
#define _check_null(ptr) \
  ({ void*_cks_null = (void*)(ptr); \
     if (!_cks_null) _throw_null(); \
     _cks_null; })
#endif

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index)\
   (((char*)ptr) + (elt_sz)*(index))
#ifdef NO_CYC_NULL_CHECKS
#define _check_known_subscript_null _check_known_subscript_notnull
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr);\
  int _index = (index);\
  if (!_cks_ptr) _throw_null(); \
  _cks_ptr + (elt_sz)*_index; })
#endif
#define _zero_arr_plus_fn(orig_x,orig_sz,orig_i,f,l) ((orig_x)+(orig_i))
#define _zero_arr_plus_char_fn _zero_arr_plus_fn
#define _zero_arr_plus_short_fn _zero_arr_plus_fn
#define _zero_arr_plus_int_fn _zero_arr_plus_fn
#define _zero_arr_plus_float_fn _zero_arr_plus_fn
#define _zero_arr_plus_double_fn _zero_arr_plus_fn
#define _zero_arr_plus_longdouble_fn _zero_arr_plus_fn
#define _zero_arr_plus_voidstar_fn _zero_arr_plus_fn
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (!_cks_ptr) _throw_null(); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })

/* _zero_arr_plus_*_fn(x,sz,i,filename,lineno) adds i to zero-terminated ptr
   x that has at least sz elements */
char* _zero_arr_plus_char_fn(char*,unsigned,int,const char*,unsigned);
short* _zero_arr_plus_short_fn(short*,unsigned,int,const char*,unsigned);
int* _zero_arr_plus_int_fn(int*,unsigned,int,const char*,unsigned);
float* _zero_arr_plus_float_fn(float*,unsigned,int,const char*,unsigned);
double* _zero_arr_plus_double_fn(double*,unsigned,int,const char*,unsigned);
long double* _zero_arr_plus_longdouble_fn(long double*,unsigned,int,const char*, unsigned);
void** _zero_arr_plus_voidstar_fn(void**,unsigned,int,const char*,unsigned);
#endif

/* _get_zero_arr_size_*(x,sz) returns the number of elements in a
   zero-terminated array that is NULL or has at least sz elements */
int _get_zero_arr_size_char(const char*,unsigned);
int _get_zero_arr_size_short(const short*,unsigned);
int _get_zero_arr_size_int(const int*,unsigned);
int _get_zero_arr_size_float(const float*,unsigned);
int _get_zero_arr_size_double(const double*,unsigned);
int _get_zero_arr_size_longdouble(const long double*,unsigned);
int _get_zero_arr_size_voidstar(const void**,unsigned);

/* _zero_arr_inplace_plus_*_fn(x,i,filename,lineno) sets
   zero-terminated pointer *x to *x + i */
char* _zero_arr_inplace_plus_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_short_fn(short**,int,const char*,unsigned);
int* _zero_arr_inplace_plus_int(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_longdouble_fn(long double**,int,const char*,unsigned);
void** _zero_arr_inplace_plus_voidstar_fn(void***,int,const char*,unsigned);
/* like the previous functions, but does post-addition (as in e++) */
char* _zero_arr_inplace_plus_post_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_post_short_fn(short**x,int,const char*,unsigned);
int* _zero_arr_inplace_plus_post_int_fn(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_post_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_post_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_post_longdouble_fn(long double**,int,const char *,unsigned);
void** _zero_arr_inplace_plus_post_voidstar_fn(void***,int,const char*,unsigned);
#define _zero_arr_plus_char(x,s,i) \
  (_zero_arr_plus_char_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_short(x,s,i) \
  (_zero_arr_plus_short_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_int(x,s,i) \
  (_zero_arr_plus_int_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_float(x,s,i) \
  (_zero_arr_plus_float_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_double(x,s,i) \
  (_zero_arr_plus_double_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_longdouble(x,s,i) \
  (_zero_arr_plus_longdouble_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_voidstar(x,s,i) \
  (_zero_arr_plus_voidstar_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_inplace_plus_char(x,i) \
  _zero_arr_inplace_plus_char_fn((char **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_short(x,i) \
  _zero_arr_inplace_plus_short_fn((short **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_int(x,i) \
  _zero_arr_inplace_plus_int_fn((int **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_float(x,i) \
  _zero_arr_inplace_plus_float_fn((float **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_double(x,i) \
  _zero_arr_inplace_plus_double_fn((double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_longdouble(x,i) \
  _zero_arr_inplace_plus_longdouble_fn((long double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_voidstar(x,i) \
  _zero_arr_inplace_plus_voidstar_fn((void ***)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_char(x,i) \
  _zero_arr_inplace_plus_post_char_fn((char **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_short(x,i) \
  _zero_arr_inplace_plus_post_short_fn((short **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_int(x,i) \
  _zero_arr_inplace_plus_post_int_fn((int **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_float(x,i) \
  _zero_arr_inplace_plus_post_float_fn((float **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_double(x,i) \
  _zero_arr_inplace_plus_post_double_fn((double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_longdouble(x,i) \
  _zero_arr_inplace_plus_post_longdouble_fn((long double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_voidstar(x,i) \
  _zero_arr_inplace_plus_post_voidstar_fn((void***)(x),(i),__FILE__,__LINE__)

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_fat_subscript(arr,elt_sz,index) ((arr).curr + (elt_sz) * (index))
#define _untag_fat_ptr(arr,elt_sz,num_elts) ((arr).curr)
#else
#define _check_fat_subscript(arr,elt_sz,index) ({ \
  struct _fat_ptr _cus_arr = (arr); \
  unsigned char *_cus_ans = _cus_arr.curr + (elt_sz) * (index); \
  /* JGM: not needed! if (!_cus_arr.base) _throw_null();*/ \
  if (_cus_ans < _cus_arr.base || _cus_ans >= _cus_arr.last_plus_one) \
    _throw_arraybounds(); \
  _cus_ans; })
#define _untag_fat_ptr(arr,elt_sz,num_elts) ({ \
  struct _fat_ptr _arr = (arr); \
  unsigned char *_curr = _arr.curr; \
  if ((_curr < _arr.base || _curr + (elt_sz) * (num_elts) > _arr.last_plus_one) &&\
      _curr != (unsigned char *)0) \
    _throw_arraybounds(); \
  _curr; })
#endif

#define _tag_fat(tcurr,elt_sz,num_elts) ({ \
  struct _fat_ptr _ans; \
  unsigned _num_elts = (num_elts);\
  _ans.base = _ans.curr = (void*)(tcurr); \
  /* JGM: if we're tagging NULL, ignore num_elts */ \
  _ans.last_plus_one = _ans.base ? (_ans.base + (elt_sz) * _num_elts) : 0; \
  _ans; })

#define _get_fat_size(arr,elt_sz) \
  ({struct _fat_ptr _arr = (arr); \
    unsigned char *_arr_curr=_arr.curr; \
    unsigned char *_arr_last=_arr.last_plus_one; \
    (_arr_curr < _arr.base || _arr_curr >= _arr_last) ? 0 : \
    ((_arr_last - _arr_curr) / (elt_sz));})

#define _fat_ptr_plus(arr,elt_sz,change) ({ \
  struct _fat_ptr _ans = (arr); \
  int _change = (change);\
  _ans.curr += (elt_sz) * _change;\
  _ans; })
#define _fat_ptr_inplace_plus(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  _arr_ptr->curr += (elt_sz) * (change);\
  *_arr_ptr; })
#define _fat_ptr_inplace_plus_post(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  struct _fat_ptr _ans = *_arr_ptr; \
  _arr_ptr->curr += (elt_sz) * (change);\
  _ans; })

//Not a macro since initialization order matters. Defined in runtime_zeroterm.c.
struct _fat_ptr _fat_ptr_decrease_size(struct _fat_ptr,unsigned sz,unsigned numelts);

/* Allocation */
void* GC_malloc(int);
void* GC_malloc_atomic(int);
void* GC_calloc(unsigned,unsigned);
void* GC_calloc_atomic(unsigned,unsigned);
// bound the allocation size to be < MAX_ALLOC_SIZE. See macros below for usage.
#define MAX_MALLOC_SIZE (1 << 28)
void* _bounded_GC_malloc(int,const char*,int);
void* _bounded_GC_malloc_atomic(int,const char*,int);
void* _bounded_GC_calloc(unsigned,unsigned,const char*,int);
void* _bounded_GC_calloc_atomic(unsigned,unsigned,const char*,int);
/* these macros are overridden below ifdef CYC_REGION_PROFILE */
#ifndef CYC_REGION_PROFILE
#define _cycalloc(n) _bounded_GC_malloc(n,__FILE__,__LINE__)
#define _cycalloc_atomic(n) _bounded_GC_malloc_atomic(n,__FILE__,__LINE__)
#define _cyccalloc(n,s) _bounded_GC_calloc(n,s,__FILE__,__LINE__)
#define _cyccalloc_atomic(n,s) _bounded_GC_calloc_atomic(n,s,__FILE__,__LINE__)
#endif

static _INLINE unsigned int _check_times(unsigned x, unsigned y) {
  unsigned long long whole_ans = 
    ((unsigned long long) x)*((unsigned long long)y);
  unsigned word_ans = (unsigned)whole_ans;
  if(word_ans < whole_ans || word_ans > MAX_MALLOC_SIZE)
    _throw_badalloc();
  return word_ans;
}

#define _CYC_MAX_REGION_CONST 2
#define _CYC_MIN_ALIGNMENT (sizeof(double))

#ifdef CYC_REGION_PROFILE
extern int rgn_total_bytes;
#endif

static _INLINE void *_fast_region_malloc(struct _RegionHandle *r, unsigned orig_s) {  
  if (r > (struct _RegionHandle *)_CYC_MAX_REGION_CONST && r->curr != 0) { 
#ifdef CYC_NOALIGN
    unsigned s =  orig_s;
#else
    unsigned s =  (orig_s + _CYC_MIN_ALIGNMENT - 1) & (~(_CYC_MIN_ALIGNMENT -1)); 
#endif
    char *result; 
    result = r->offset; 
    if (s <= (r->last_plus_one - result)) {
      r->offset = result + s; 
#ifdef CYC_REGION_PROFILE
    r->curr->free_bytes = r->curr->free_bytes - s;
    rgn_total_bytes += s;
#endif
      return result;
    }
  } 
  return _region_malloc(r,orig_s); 
}

#ifdef CYC_REGION_PROFILE
/* see macros below for usage. defined in runtime_memory.c */
void* _profile_GC_malloc(int,const char*,const char*,int);
void* _profile_GC_malloc_atomic(int,const char*,const char*,int);
void* _profile_GC_calloc(unsigned,unsigned,const char*,const char*,int);
void* _profile_GC_calloc_atomic(unsigned,unsigned,const char*,const char*,int);
void* _profile_region_malloc(struct _RegionHandle*,unsigned,const char*,const char*,int);
void* _profile_region_calloc(struct _RegionHandle*,unsigned,unsigned,const char *,const char*,int);
struct _RegionHandle _profile_new_region(const char*,const char*,const char*,int);
void _profile_free_region(struct _RegionHandle*,const char*,const char*,int);
#ifndef RUNTIME_CYC
#define _new_region(n) _profile_new_region(n,__FILE__,__FUNCTION__,__LINE__)
#define _free_region(r) _profile_free_region(r,__FILE__,__FUNCTION__,__LINE__)
#define _region_malloc(rh,n) _profile_region_malloc(rh,n,__FILE__,__FUNCTION__,__LINE__)
#define _region_calloc(rh,n,t) _profile_region_calloc(rh,n,t,__FILE__,__FUNCTION__,__LINE__)
#  endif
#define _cycalloc(n) _profile_GC_malloc(n,__FILE__,__FUNCTION__,__LINE__)
#define _cycalloc_atomic(n) _profile_GC_malloc_atomic(n,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc(n,s) _profile_GC_calloc(n,s,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc_atomic(n,s) _profile_GC_calloc_atomic(n,s,__FILE__,__FUNCTION__,__LINE__)
#endif
#endif
 struct Cyc___cycFILE;
# 53 "cycboot.h"
extern struct Cyc___cycFILE*Cyc_stderr;struct Cyc_String_pa_PrintArg_struct{int tag;struct _fat_ptr f1;};struct Cyc_Int_pa_PrintArg_struct{int tag;unsigned long f1;};struct Cyc_Double_pa_PrintArg_struct{int tag;double f1;};struct Cyc_LongDouble_pa_PrintArg_struct{int tag;long double f1;};struct Cyc_ShortPtr_pa_PrintArg_struct{int tag;short*f1;};struct Cyc_IntPtr_pa_PrintArg_struct{int tag;unsigned long*f1;};
# 73
struct _fat_ptr Cyc_aprintf(struct _fat_ptr,struct _fat_ptr);
# 100
int Cyc_fprintf(struct Cyc___cycFILE*,struct _fat_ptr,struct _fat_ptr);struct Cyc_ShortPtr_sa_ScanfArg_struct{int tag;short*f1;};struct Cyc_UShortPtr_sa_ScanfArg_struct{int tag;unsigned short*f1;};struct Cyc_IntPtr_sa_ScanfArg_struct{int tag;int*f1;};struct Cyc_UIntPtr_sa_ScanfArg_struct{int tag;unsigned*f1;};struct Cyc_StringPtr_sa_ScanfArg_struct{int tag;struct _fat_ptr f1;};struct Cyc_DoublePtr_sa_ScanfArg_struct{int tag;double*f1;};struct Cyc_FloatPtr_sa_ScanfArg_struct{int tag;float*f1;};struct Cyc_CharPtr_sa_ScanfArg_struct{int tag;struct _fat_ptr f1;};extern char Cyc_FileCloseError[15U];struct Cyc_FileCloseError_exn_struct{char*tag;};extern char Cyc_FileOpenError[14U];struct Cyc_FileOpenError_exn_struct{char*tag;struct _fat_ptr f1;};struct Cyc_Core_Opt{void*v;};struct _tuple0{void*f1;void*f2;};
# 111 "core.h"
void*Cyc_Core_snd(struct _tuple0*);extern char Cyc_Core_Invalid_argument[17U];struct Cyc_Core_Invalid_argument_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Failure[8U];struct Cyc_Core_Failure_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Impossible[11U];struct Cyc_Core_Impossible_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Not_found[10U];struct Cyc_Core_Not_found_exn_struct{char*tag;};extern char Cyc_Core_Unreachable[12U];struct Cyc_Core_Unreachable_exn_struct{char*tag;struct _fat_ptr f1;};
# 165
extern struct _RegionHandle*Cyc_Core_heap_region;
# 168
extern struct _RegionHandle*Cyc_Core_unique_region;struct Cyc_Core_DynamicRegion;struct Cyc_Core_NewDynamicRegion{struct Cyc_Core_DynamicRegion*key;};struct Cyc_Core_ThinRes{void*arr;unsigned nelts;};struct Cyc_List_List{void*hd;struct Cyc_List_List*tl;};
# 54 "list.h"
struct Cyc_List_List*Cyc_List_list(struct _fat_ptr);
# 61
int Cyc_List_length(struct Cyc_List_List*x);
# 76
struct Cyc_List_List*Cyc_List_map(void*(*f)(void*),struct Cyc_List_List*x);
# 83
struct Cyc_List_List*Cyc_List_map_c(void*(*f)(void*,void*),void*env,struct Cyc_List_List*x);
# 86
struct Cyc_List_List*Cyc_List_rmap_c(struct _RegionHandle*,void*(*f)(void*,void*),void*env,struct Cyc_List_List*x);extern char Cyc_List_List_mismatch[14U];struct Cyc_List_List_mismatch_exn_struct{char*tag;};
# 172
struct Cyc_List_List*Cyc_List_rev(struct Cyc_List_List*x);
# 178
struct Cyc_List_List*Cyc_List_imp_rev(struct Cyc_List_List*x);
# 184
struct Cyc_List_List*Cyc_List_append(struct Cyc_List_List*x,struct Cyc_List_List*y);
# 190
struct Cyc_List_List*Cyc_List_rappend(struct _RegionHandle*,struct Cyc_List_List*x,struct Cyc_List_List*y);
# 195
struct Cyc_List_List*Cyc_List_imp_append(struct Cyc_List_List*x,struct Cyc_List_List*y);extern char Cyc_List_Nth[4U];struct Cyc_List_Nth_exn_struct{char*tag;};
# 276
struct Cyc_List_List*Cyc_List_rzip(struct _RegionHandle*r1,struct _RegionHandle*r2,struct Cyc_List_List*x,struct Cyc_List_List*y);struct _tuple1{struct Cyc_List_List*f1;struct Cyc_List_List*f2;};
# 294
struct _tuple1 Cyc_List_split(struct Cyc_List_List*x);
# 379
struct Cyc_List_List*Cyc_List_tabulate_c(int n,void*(*f)(void*,int),void*env);struct Cyc_Iter_Iter{void*env;int(*next)(void*env,void*dest);};struct Cyc_Dict_T;struct Cyc_Dict_Dict{int(*rel)(void*,void*);struct _RegionHandle*r;const struct Cyc_Dict_T*t;};extern char Cyc_Dict_Present[8U];struct Cyc_Dict_Present_exn_struct{char*tag;};extern char Cyc_Dict_Absent[7U];struct Cyc_Dict_Absent_exn_struct{char*tag;};struct Cyc_Set_Set;
# 54 "set.h"
struct Cyc_Set_Set*Cyc_Set_rempty(struct _RegionHandle*r,int(*cmp)(void*,void*));
# 63
struct Cyc_Set_Set*Cyc_Set_insert(struct Cyc_Set_Set*s,void*elt);
# 94
int Cyc_Set_cardinality(struct Cyc_Set_Set*s);
# 97
int Cyc_Set_is_empty(struct Cyc_Set_Set*s);
# 100
int Cyc_Set_member(struct Cyc_Set_Set*s,void*elt);extern char Cyc_Set_Absent[7U];struct Cyc_Set_Absent_exn_struct{char*tag;};
# 137
void*Cyc_Set_choose(struct Cyc_Set_Set*s);
# 49 "string.h"
int Cyc_strcmp(struct _fat_ptr s1,struct _fat_ptr s2);
int Cyc_strptrcmp(struct _fat_ptr*s1,struct _fat_ptr*s2);
# 64
struct _fat_ptr Cyc_strconcat_l(struct Cyc_List_List*);struct Cyc_PP_Ppstate;struct Cyc_PP_Out;struct Cyc_PP_Doc;struct Cyc_Position_Error;struct Cyc_Relations_Reln;struct _union_Nmspace_Rel_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_Abs_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_C_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_Loc_n{int tag;int val;};union Cyc_Absyn_Nmspace{struct _union_Nmspace_Rel_n Rel_n;struct _union_Nmspace_Abs_n Abs_n;struct _union_Nmspace_C_n C_n;struct _union_Nmspace_Loc_n Loc_n;};struct _tuple2{union Cyc_Absyn_Nmspace f1;struct _fat_ptr*f2;};
# 161 "absyn.h"
enum Cyc_Absyn_Scope{Cyc_Absyn_Static =0U,Cyc_Absyn_Abstract =1U,Cyc_Absyn_Public =2U,Cyc_Absyn_Extern =3U,Cyc_Absyn_ExternC =4U,Cyc_Absyn_Register =5U};struct Cyc_Absyn_Tqual{int print_const: 1;int q_volatile: 1;int q_restrict: 1;int real_const: 1;unsigned loc;};
# 182
enum Cyc_Absyn_Size_of{Cyc_Absyn_Char_sz =0U,Cyc_Absyn_Short_sz =1U,Cyc_Absyn_Int_sz =2U,Cyc_Absyn_Long_sz =3U,Cyc_Absyn_LongLong_sz =4U};
enum Cyc_Absyn_Sign{Cyc_Absyn_Signed =0U,Cyc_Absyn_Unsigned =1U,Cyc_Absyn_None =2U};
enum Cyc_Absyn_AggrKind{Cyc_Absyn_StructA =0U,Cyc_Absyn_UnionA =1U};
# 187
enum Cyc_Absyn_AliasQual{Cyc_Absyn_Aliasable =0U,Cyc_Absyn_Unique =1U,Cyc_Absyn_Top =2U};
# 192
enum Cyc_Absyn_KindQual{Cyc_Absyn_AnyKind =0U,Cyc_Absyn_MemKind =1U,Cyc_Absyn_BoxKind =2U,Cyc_Absyn_RgnKind =3U,Cyc_Absyn_EffKind =4U,Cyc_Absyn_IntKind =5U,Cyc_Absyn_BoolKind =6U,Cyc_Absyn_PtrBndKind =7U};struct Cyc_Absyn_Kind{enum Cyc_Absyn_KindQual kind;enum Cyc_Absyn_AliasQual aliasqual;};struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct{int tag;struct Cyc_Absyn_Kind*f1;};struct Cyc_Absyn_Unknown_kb_Absyn_KindBound_struct{int tag;struct Cyc_Core_Opt*f1;};struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct{int tag;struct Cyc_Core_Opt*f1;struct Cyc_Absyn_Kind*f2;};struct Cyc_Absyn_Tvar{struct _fat_ptr*name;int identity;void*kind;};struct Cyc_Absyn_PtrLoc{unsigned ptr_loc;unsigned rgn_loc;unsigned zt_loc;};struct Cyc_Absyn_PtrAtts{void*rgn;void*nullable;void*bounds;void*zero_term;struct Cyc_Absyn_PtrLoc*ptrloc;};struct Cyc_Absyn_PtrInfo{void*elt_type;struct Cyc_Absyn_Tqual elt_tq;struct Cyc_Absyn_PtrAtts ptr_atts;};struct Cyc_Absyn_VarargInfo{struct _fat_ptr*name;struct Cyc_Absyn_Tqual tq;void*type;int inject;};struct Cyc_Absyn_Star_Absyn_Cap_struct{int tag;};struct Cyc_Absyn_Bot_Absyn_Cap_struct{int tag;};struct Cyc_Absyn_Nat_Absyn_Cap_struct{int tag;struct Cyc_Absyn_Exp*f1;int f2;};struct Cyc_Absyn_RgnEffect{void*name;void*rgn;void*lock;void*unadopt;void*parent;};struct Cyc_Absyn_FnInfo{struct Cyc_List_List*tvars;void*effect;struct Cyc_Absyn_Tqual ret_tqual;void*ret_type;struct Cyc_List_List*args;int c_varargs;struct Cyc_Absyn_VarargInfo*cyc_varargs;struct Cyc_List_List*rgn_po;struct Cyc_List_List*attributes;struct Cyc_Absyn_Exp*requires_clause;struct Cyc_List_List*requires_relns;struct Cyc_Absyn_Exp*ensures_clause;struct Cyc_List_List*ensures_relns;struct Cyc_List_List*ieffect;struct Cyc_List_List*oeffect;};struct Cyc_Absyn_UnknownDatatypeInfo{struct _tuple2*name;int is_extensible;};struct _union_DatatypeInfo_UnknownDatatype{int tag;struct Cyc_Absyn_UnknownDatatypeInfo val;};struct _union_DatatypeInfo_KnownDatatype{int tag;struct Cyc_Absyn_Datatypedecl**val;};union Cyc_Absyn_DatatypeInfo{struct _union_DatatypeInfo_UnknownDatatype UnknownDatatype;struct _union_DatatypeInfo_KnownDatatype KnownDatatype;};
# 312
union Cyc_Absyn_DatatypeInfo Cyc_Absyn_KnownDatatype(struct Cyc_Absyn_Datatypedecl**);struct Cyc_Absyn_UnknownDatatypeFieldInfo{struct _tuple2*datatype_name;struct _tuple2*field_name;int is_extensible;};struct _union_DatatypeFieldInfo_UnknownDatatypefield{int tag;struct Cyc_Absyn_UnknownDatatypeFieldInfo val;};struct _tuple3{struct Cyc_Absyn_Datatypedecl*f1;struct Cyc_Absyn_Datatypefield*f2;};struct _union_DatatypeFieldInfo_KnownDatatypefield{int tag;struct _tuple3 val;};union Cyc_Absyn_DatatypeFieldInfo{struct _union_DatatypeFieldInfo_UnknownDatatypefield UnknownDatatypefield;struct _union_DatatypeFieldInfo_KnownDatatypefield KnownDatatypefield;};
# 325
union Cyc_Absyn_DatatypeFieldInfo Cyc_Absyn_KnownDatatypefield(struct Cyc_Absyn_Datatypedecl*,struct Cyc_Absyn_Datatypefield*);struct _tuple4{enum Cyc_Absyn_AggrKind f1;struct _tuple2*f2;struct Cyc_Core_Opt*f3;};struct _union_AggrInfo_UnknownAggr{int tag;struct _tuple4 val;};struct _union_AggrInfo_KnownAggr{int tag;struct Cyc_Absyn_Aggrdecl**val;};union Cyc_Absyn_AggrInfo{struct _union_AggrInfo_UnknownAggr UnknownAggr;struct _union_AggrInfo_KnownAggr KnownAggr;};
# 332
union Cyc_Absyn_AggrInfo Cyc_Absyn_KnownAggr(struct Cyc_Absyn_Aggrdecl**);struct Cyc_Absyn_ArrayInfo{void*elt_type;struct Cyc_Absyn_Tqual tq;struct Cyc_Absyn_Exp*num_elts;void*zero_term;unsigned zt_loc;};struct Cyc_Absyn_Aggr_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Aggrdecl*f1;};struct Cyc_Absyn_Enum_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;};struct Cyc_Absyn_Datatype_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;};struct Cyc_Absyn_TypeDecl{void*r;unsigned loc;};struct Cyc_Absyn_VoidCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_IntCon_Absyn_TyCon_struct{int tag;enum Cyc_Absyn_Sign f1;enum Cyc_Absyn_Size_of f2;};struct Cyc_Absyn_FloatCon_Absyn_TyCon_struct{int tag;int f1;};struct Cyc_Absyn_RgnHandleCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_TagCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_HeapCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_UniqueCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_RefCntCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_AccessCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_JoinCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_RgnsCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_TrueCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_FalseCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_ThinCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_FatCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_EnumCon_Absyn_TyCon_struct{int tag;struct _tuple2*f1;struct Cyc_Absyn_Enumdecl*f2;};struct Cyc_Absyn_AnonEnumCon_Absyn_TyCon_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_BuiltinCon_Absyn_TyCon_struct{int tag;struct _fat_ptr f1;struct Cyc_Absyn_Kind*f2;};struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_DatatypeInfo f1;};struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_DatatypeFieldInfo f1;};struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_AggrInfo f1;};struct Cyc_Absyn_AppType_Absyn_Type_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Evar_Absyn_Type_struct{int tag;struct Cyc_Core_Opt*f1;void*f2;int f3;struct Cyc_Core_Opt*f4;};struct Cyc_Absyn_VarType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Tvar*f1;};struct Cyc_Absyn_PointerType_Absyn_Type_struct{int tag;struct Cyc_Absyn_PtrInfo f1;};struct Cyc_Absyn_ArrayType_Absyn_Type_struct{int tag;struct Cyc_Absyn_ArrayInfo f1;};struct Cyc_Absyn_FnType_Absyn_Type_struct{int tag;struct Cyc_Absyn_FnInfo f1;};struct Cyc_Absyn_TupleType_Absyn_Type_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct{int tag;enum Cyc_Absyn_AggrKind f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_TypedefType_Absyn_Type_struct{int tag;struct _tuple2*f1;struct Cyc_List_List*f2;struct Cyc_Absyn_Typedefdecl*f3;void*f4;};struct Cyc_Absyn_ValueofType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct{int tag;struct Cyc_Absyn_TypeDecl*f1;void**f2;};struct Cyc_Absyn_TypeofType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_NoTypes_Absyn_Funcparams_struct{int tag;struct Cyc_List_List*f1;unsigned f2;};struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct{int tag;struct Cyc_List_List*f1;int f2;struct Cyc_Absyn_VarargInfo*f3;void*f4;struct Cyc_List_List*f5;struct Cyc_Absyn_Exp*f6;struct Cyc_Absyn_Exp*f7;struct Cyc_List_List*f8;struct Cyc_List_List*f9;};
# 437 "absyn.h"
enum Cyc_Absyn_Format_Type{Cyc_Absyn_Printf_ft =0U,Cyc_Absyn_Scanf_ft =1U};struct Cyc_Absyn_Regparm_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Stdcall_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Cdecl_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Fastcall_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Noreturn_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Const_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Aligned_att_Absyn_Attribute_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Packed_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Section_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Nocommon_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Shared_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Unused_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Weak_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Dllimport_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Dllexport_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_instrument_function_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Constructor_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Destructor_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_check_memory_usage_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Format_att_Absyn_Attribute_struct{int tag;enum Cyc_Absyn_Format_Type f1;int f2;int f3;};struct Cyc_Absyn_Initializes_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Noliveunique_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Consume_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Pure_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Mode_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Alias_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Always_inline_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_throw_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Non_null_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Deprecated_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Carray_mod_Absyn_Type_modifier_struct{int tag;void*f1;unsigned f2;};struct Cyc_Absyn_ConstArray_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_Absyn_Exp*f1;void*f2;unsigned f3;};struct Cyc_Absyn_Pointer_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_Absyn_PtrAtts f1;struct Cyc_Absyn_Tqual f2;};struct Cyc_Absyn_Function_mod_Absyn_Type_modifier_struct{int tag;void*f1;};struct Cyc_Absyn_TypeParams_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_List_List*f1;unsigned f2;int f3;};struct Cyc_Absyn_Attributes_mod_Absyn_Type_modifier_struct{int tag;unsigned f1;struct Cyc_List_List*f2;};struct _union_Cnst_Null_c{int tag;int val;};struct _tuple5{enum Cyc_Absyn_Sign f1;char f2;};struct _union_Cnst_Char_c{int tag;struct _tuple5 val;};struct _union_Cnst_Wchar_c{int tag;struct _fat_ptr val;};struct _tuple6{enum Cyc_Absyn_Sign f1;short f2;};struct _union_Cnst_Short_c{int tag;struct _tuple6 val;};struct _tuple7{enum Cyc_Absyn_Sign f1;int f2;};struct _union_Cnst_Int_c{int tag;struct _tuple7 val;};struct _tuple8{enum Cyc_Absyn_Sign f1;long long f2;};struct _union_Cnst_LongLong_c{int tag;struct _tuple8 val;};struct _tuple9{struct _fat_ptr f1;int f2;};struct _union_Cnst_Float_c{int tag;struct _tuple9 val;};struct _union_Cnst_String_c{int tag;struct _fat_ptr val;};struct _union_Cnst_Wstring_c{int tag;struct _fat_ptr val;};union Cyc_Absyn_Cnst{struct _union_Cnst_Null_c Null_c;struct _union_Cnst_Char_c Char_c;struct _union_Cnst_Wchar_c Wchar_c;struct _union_Cnst_Short_c Short_c;struct _union_Cnst_Int_c Int_c;struct _union_Cnst_LongLong_c LongLong_c;struct _union_Cnst_Float_c Float_c;struct _union_Cnst_String_c String_c;struct _union_Cnst_Wstring_c Wstring_c;};
# 532
enum Cyc_Absyn_Primop{Cyc_Absyn_Plus =0U,Cyc_Absyn_Times =1U,Cyc_Absyn_Minus =2U,Cyc_Absyn_Div =3U,Cyc_Absyn_Mod =4U,Cyc_Absyn_Eq =5U,Cyc_Absyn_Neq =6U,Cyc_Absyn_Gt =7U,Cyc_Absyn_Lt =8U,Cyc_Absyn_Gte =9U,Cyc_Absyn_Lte =10U,Cyc_Absyn_Not =11U,Cyc_Absyn_Bitnot =12U,Cyc_Absyn_Bitand =13U,Cyc_Absyn_Bitor =14U,Cyc_Absyn_Bitxor =15U,Cyc_Absyn_Bitlshift =16U,Cyc_Absyn_Bitlrshift =17U,Cyc_Absyn_Numelts =18U};
# 539
enum Cyc_Absyn_Incrementor{Cyc_Absyn_PreInc =0U,Cyc_Absyn_PostInc =1U,Cyc_Absyn_PreDec =2U,Cyc_Absyn_PostDec =3U};struct Cyc_Absyn_VarargCallInfo{int num_varargs;struct Cyc_List_List*injectors;struct Cyc_Absyn_VarargInfo*vai;};struct Cyc_Absyn_StructField_Absyn_OffsetofField_struct{int tag;struct _fat_ptr*f1;};struct Cyc_Absyn_TupleIndex_Absyn_OffsetofField_struct{int tag;unsigned f1;};
# 557
enum Cyc_Absyn_Coercion{Cyc_Absyn_Unknown_coercion =0U,Cyc_Absyn_No_coercion =1U,Cyc_Absyn_Null_to_NonNull =2U,Cyc_Absyn_Other_coercion =3U};struct Cyc_Absyn_MallocInfo{int is_calloc;struct Cyc_Absyn_Exp*rgn;void**elt_type;struct Cyc_Absyn_Exp*num_elts;int fat_result;int inline_call;};struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct{int tag;union Cyc_Absyn_Cnst f1;};struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Pragma_e_Absyn_Raw_exp_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct{int tag;enum Cyc_Absyn_Primop f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_AssignOp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Core_Opt*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_Increment_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;enum Cyc_Absyn_Incrementor f2;};struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;struct Cyc_Absyn_VarargCallInfo*f3;int f4;};struct Cyc_Absyn_Throw_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;int f2;};struct Cyc_Absyn_NoInstantiate_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Instantiate_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_Absyn_Exp*f2;int f3;enum Cyc_Absyn_Coercion f4;};struct Cyc_Absyn_Address_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_New_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Sizeoftype_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Sizeofexp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Offsetof_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Deref_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;int f3;int f4;};struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;int f3;int f4;};struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Tuple_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;};struct _tuple10{struct _fat_ptr*f1;struct Cyc_Absyn_Tqual f2;void*f3;};struct Cyc_Absyn_CompoundLit_e_Absyn_Raw_exp_struct{int tag;struct _tuple10*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Array_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Exp*f2;struct Cyc_Absyn_Exp*f3;int f4;};struct Cyc_Absyn_ComprehensionNoinit_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;void*f2;int f3;};struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct{int tag;struct _tuple2*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;struct Cyc_Absyn_Aggrdecl*f4;};struct Cyc_Absyn_AnonStruct_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Datatype_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;struct Cyc_Absyn_Datatypedecl*f2;struct Cyc_Absyn_Datatypefield*f3;};struct Cyc_Absyn_Enum_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_AnonEnum_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_Spawn_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_MallocInfo f1;};struct Cyc_Absyn_Swap_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_UnresolvedMem_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Core_Opt*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_StmtExp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Stmt*f1;};struct Cyc_Absyn_Tagcheck_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;};struct Cyc_Absyn_Valueof_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Asm_e_Absyn_Raw_exp_struct{int tag;int f1;struct _fat_ptr f2;struct Cyc_List_List*f3;struct Cyc_List_List*f4;struct Cyc_List_List*f5;};struct Cyc_Absyn_Extension_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Exp{void*topt;void*r;unsigned loc;void*annot;};struct Cyc_Absyn_Skip_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Exp_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Seq_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Return_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_IfThenElse_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Stmt*f2;struct Cyc_Absyn_Stmt*f3;};struct _tuple11{struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_While_s_Absyn_Raw_stmt_struct{int tag;struct _tuple11 f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Break_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Continue_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Goto_s_Absyn_Raw_stmt_struct{int tag;struct _fat_ptr*f1;};struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _tuple11 f2;struct _tuple11 f3;struct Cyc_Absyn_Stmt*f4;};struct Cyc_Absyn_Switch_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;void*f3;};struct Cyc_Absyn_Fallthru_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_List_List*f1;struct Cyc_Absyn_Switch_clause**f2;};struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Decl*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Label_s_Absyn_Raw_stmt_struct{int tag;struct _fat_ptr*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Do_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct _tuple11 f2;};struct Cyc_Absyn_TryCatch_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct Cyc_List_List*f2;void*f3;};struct Cyc_Absyn_Stmt{void*r;unsigned loc;void*annot;};struct Cyc_Absyn_Wild_p_Absyn_Raw_pat_struct{int tag;};struct Cyc_Absyn_Var_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Pat*f2;};struct Cyc_Absyn_AliasVar_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;};struct Cyc_Absyn_Reference_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Pat*f2;};struct Cyc_Absyn_TagInt_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;};struct Cyc_Absyn_Tuple_p_Absyn_Raw_pat_struct{int tag;struct Cyc_List_List*f1;int f2;};struct Cyc_Absyn_Pointer_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Pat*f1;};struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct{int tag;union Cyc_Absyn_AggrInfo*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;int f4;};struct Cyc_Absyn_Datatype_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;struct Cyc_Absyn_Datatypefield*f2;struct Cyc_List_List*f3;int f4;};struct Cyc_Absyn_Null_p_Absyn_Raw_pat_struct{int tag;};struct Cyc_Absyn_Int_p_Absyn_Raw_pat_struct{int tag;enum Cyc_Absyn_Sign f1;int f2;};struct Cyc_Absyn_Char_p_Absyn_Raw_pat_struct{int tag;char f1;};struct Cyc_Absyn_Float_p_Absyn_Raw_pat_struct{int tag;struct _fat_ptr f1;int f2;};struct Cyc_Absyn_Enum_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_AnonEnum_p_Absyn_Raw_pat_struct{int tag;void*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_UnknownId_p_Absyn_Raw_pat_struct{int tag;struct _tuple2*f1;};struct Cyc_Absyn_UnknownCall_p_Absyn_Raw_pat_struct{int tag;struct _tuple2*f1;struct Cyc_List_List*f2;int f3;};struct Cyc_Absyn_Exp_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Exp*f1;};
# 725 "absyn.h"
extern struct Cyc_Absyn_Wild_p_Absyn_Raw_pat_struct Cyc_Absyn_Wild_p_val;struct Cyc_Absyn_Pat{void*r;void*topt;unsigned loc;};struct Cyc_Absyn_Switch_clause{struct Cyc_Absyn_Pat*pattern;struct Cyc_Core_Opt*pat_vars;struct Cyc_Absyn_Exp*where_clause;struct Cyc_Absyn_Stmt*body;unsigned loc;};struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct{int tag;struct _tuple2*f1;};struct Cyc_Absyn_Global_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Funname_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Fndecl*f1;};struct Cyc_Absyn_Param_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Local_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Pat_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Vardecl{enum Cyc_Absyn_Scope sc;struct _tuple2*name;unsigned varloc;struct Cyc_Absyn_Tqual tq;void*type;struct Cyc_Absyn_Exp*initializer;void*rgn;struct Cyc_List_List*attributes;int escapes;};struct Cyc_Absyn_Fndecl{enum Cyc_Absyn_Scope sc;int is_inline;struct _tuple2*name;struct Cyc_Absyn_Stmt*body;struct Cyc_Absyn_FnInfo i;void*cached_type;struct Cyc_Core_Opt*param_vardecls;struct Cyc_Absyn_Vardecl*fn_vardecl;};struct Cyc_Absyn_Aggrfield{struct _fat_ptr*name;struct Cyc_Absyn_Tqual tq;void*type;struct Cyc_Absyn_Exp*width;struct Cyc_List_List*attributes;struct Cyc_Absyn_Exp*requires_clause;};struct Cyc_Absyn_AggrdeclImpl{struct Cyc_List_List*exist_vars;struct Cyc_List_List*rgn_po;struct Cyc_List_List*fields;int tagged;};struct Cyc_Absyn_Aggrdecl{enum Cyc_Absyn_AggrKind kind;enum Cyc_Absyn_Scope sc;struct _tuple2*name;struct Cyc_List_List*tvs;struct Cyc_Absyn_AggrdeclImpl*impl;struct Cyc_List_List*attributes;int expected_mem_kind;};struct Cyc_Absyn_Datatypefield{struct _tuple2*name;struct Cyc_List_List*typs;unsigned loc;enum Cyc_Absyn_Scope sc;};struct Cyc_Absyn_Datatypedecl{enum Cyc_Absyn_Scope sc;struct _tuple2*name;struct Cyc_List_List*tvs;struct Cyc_Core_Opt*fields;int is_extensible;};struct Cyc_Absyn_Enumfield{struct _tuple2*name;struct Cyc_Absyn_Exp*tag;unsigned loc;};struct Cyc_Absyn_Enumdecl{enum Cyc_Absyn_Scope sc;struct _tuple2*name;struct Cyc_Core_Opt*fields;};struct Cyc_Absyn_Typedefdecl{struct _tuple2*name;struct Cyc_Absyn_Tqual tq;struct Cyc_List_List*tvs;struct Cyc_Core_Opt*kind;void*defn;struct Cyc_List_List*atts;int extern_c;};struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Fn_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Fndecl*f1;};struct Cyc_Absyn_Let_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Pat*f1;struct Cyc_Core_Opt*f2;struct Cyc_Absyn_Exp*f3;void*f4;};struct Cyc_Absyn_Letv_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_Region_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_Aggr_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Aggrdecl*f1;};struct Cyc_Absyn_Datatype_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;};struct Cyc_Absyn_Enum_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;};struct Cyc_Absyn_Typedef_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Typedefdecl*f1;};struct Cyc_Absyn_Namespace_d_Absyn_Raw_decl_struct{int tag;struct _fat_ptr*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Using_d_Absyn_Raw_decl_struct{int tag;struct _tuple2*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_ExternC_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;};struct _tuple12{unsigned f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_ExternCinclude_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;struct _tuple12*f4;};struct Cyc_Absyn_Porton_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Portoff_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Tempeston_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Tempestoff_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Decl{void*r;unsigned loc;};struct Cyc_Absyn_ArrayElement_Absyn_Designator_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_FieldName_Absyn_Designator_struct{int tag;struct _fat_ptr*f1;};extern char Cyc_Absyn_EmptyAnnot[11U];struct Cyc_Absyn_EmptyAnnot_Absyn_AbsynAnnot_struct{char*tag;};
# 908
int Cyc_Absyn_qvar_cmp(struct _tuple2*,struct _tuple2*);
# 916
struct Cyc_Absyn_Tqual Cyc_Absyn_const_tqual(unsigned);
struct Cyc_Absyn_Tqual Cyc_Absyn_empty_tqual(unsigned);
# 922
void*Cyc_Absyn_compress_kb(void*);
# 933
void*Cyc_Absyn_new_evar(struct Cyc_Core_Opt*k,struct Cyc_Core_Opt*tenv);
# 935
void*Cyc_Absyn_wildtyp(struct Cyc_Core_Opt*);
# 938
extern void*Cyc_Absyn_char_type;extern void*Cyc_Absyn_uint_type;
# 940
extern void*Cyc_Absyn_sint_type;
# 943
void*Cyc_Absyn_gen_float_type(unsigned i);
# 945
extern void*Cyc_Absyn_unique_rgn_type;
# 947
extern void*Cyc_Absyn_empty_effect;
# 949
extern void*Cyc_Absyn_true_type;extern void*Cyc_Absyn_false_type;
# 951
extern void*Cyc_Absyn_void_type;void*Cyc_Absyn_enum_type(struct _tuple2*n,struct Cyc_Absyn_Enumdecl*d);
# 980
void*Cyc_Absyn_bounds_one();
# 1007
void*Cyc_Absyn_datatype_type(union Cyc_Absyn_DatatypeInfo,struct Cyc_List_List*args);
void*Cyc_Absyn_datatype_field_type(union Cyc_Absyn_DatatypeFieldInfo,struct Cyc_List_List*args);
void*Cyc_Absyn_aggr_type(union Cyc_Absyn_AggrInfo,struct Cyc_List_List*args);
# 1015
struct Cyc_Absyn_Exp*Cyc_Absyn_const_exp(union Cyc_Absyn_Cnst,unsigned);
# 1059
struct Cyc_Absyn_Exp*Cyc_Absyn_address_exp(struct Cyc_Absyn_Exp*,unsigned);
# 1063
struct Cyc_Absyn_Exp*Cyc_Absyn_deref_exp(struct Cyc_Absyn_Exp*,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_aggrmember_exp(struct Cyc_Absyn_Exp*,struct _fat_ptr*,unsigned);
# 1066
struct Cyc_Absyn_Exp*Cyc_Absyn_subscript_exp(struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*,unsigned);
# 1082
struct Cyc_Absyn_Stmt*Cyc_Absyn_skip_stmt(unsigned);
# 1176
struct Cyc_Absyn_Aggrdecl*Cyc_Absyn_get_known_aggrdecl(union Cyc_Absyn_AggrInfo);struct Cyc_Absynpp_Params{int expand_typedefs;int qvar_to_Cids;int add_cyc_prefix;int to_VC;int decls_first;int rewrite_temp_tvars;int print_all_tvars;int print_all_kinds;int print_all_effects;int print_using_stmts;int print_externC_stmts;int print_full_evars;int print_zeroterm;int generate_line_directives;int use_curr_namespace;struct Cyc_List_List*curr_namespace;};
# 62 "absynpp.h"
struct _fat_ptr Cyc_Absynpp_typ2string(void*);
# 64
struct _fat_ptr Cyc_Absynpp_kind2string(struct Cyc_Absyn_Kind*);
struct _fat_ptr Cyc_Absynpp_kindbound2string(void*);
# 67
struct _fat_ptr Cyc_Absynpp_exp2string(struct Cyc_Absyn_Exp*);
struct _fat_ptr Cyc_Absynpp_stmt2string(struct Cyc_Absyn_Stmt*);
struct _fat_ptr Cyc_Absynpp_qvar2string(struct _tuple2*);
# 72
struct _fat_ptr Cyc_Absynpp_pat2string(struct Cyc_Absyn_Pat*p);
# 29 "unify.h"
int Cyc_Unify_unify(void*,void*);struct _union_RelnOp_RConst{int tag;unsigned val;};struct _union_RelnOp_RVar{int tag;struct Cyc_Absyn_Vardecl*val;};struct _union_RelnOp_RNumelts{int tag;struct Cyc_Absyn_Vardecl*val;};struct _union_RelnOp_RType{int tag;void*val;};struct _union_RelnOp_RParam{int tag;unsigned val;};struct _union_RelnOp_RParamNumelts{int tag;unsigned val;};struct _union_RelnOp_RReturn{int tag;unsigned val;};union Cyc_Relations_RelnOp{struct _union_RelnOp_RConst RConst;struct _union_RelnOp_RVar RVar;struct _union_RelnOp_RNumelts RNumelts;struct _union_RelnOp_RType RType;struct _union_RelnOp_RParam RParam;struct _union_RelnOp_RParamNumelts RParamNumelts;struct _union_RelnOp_RReturn RReturn;};
# 50 "relations-ap.h"
enum Cyc_Relations_Relation{Cyc_Relations_Req =0U,Cyc_Relations_Rneq =1U,Cyc_Relations_Rlte =2U,Cyc_Relations_Rlt =3U};struct Cyc_Relations_Reln{union Cyc_Relations_RelnOp rop1;enum Cyc_Relations_Relation relation;union Cyc_Relations_RelnOp rop2;};struct Cyc_RgnOrder_RgnPO;extern char Cyc_Tcenv_Env_error[10U];struct Cyc_Tcenv_Env_error_exn_struct{char*tag;};struct Cyc_Tcenv_Genv{struct Cyc_Dict_Dict aggrdecls;struct Cyc_Dict_Dict datatypedecls;struct Cyc_Dict_Dict enumdecls;struct Cyc_Dict_Dict typedefs;struct Cyc_Dict_Dict ordinaries;};struct Cyc_Tcenv_Fenv;struct Cyc_Tcenv_Tenv{struct Cyc_List_List*ns;struct Cyc_Tcenv_Genv*ae;struct Cyc_Tcenv_Fenv*le;int allow_valueof: 1;int in_extern_c_include: 1;int in_tempest: 1;int tempest_generalize: 1;};
# 88 "tcenv.h"
enum Cyc_Tcenv_NewStatus{Cyc_Tcenv_NoneNew =0U,Cyc_Tcenv_InNew =1U,Cyc_Tcenv_InNewAggr =2U};
# 98
struct Cyc_List_List*Cyc_Tcenv_lookup_type_vars(struct Cyc_Tcenv_Tenv*);
struct Cyc_Core_Opt*Cyc_Tcenv_lookup_opt_type_vars(struct Cyc_Tcenv_Tenv*);
struct Cyc_Tcenv_Tenv*Cyc_Tcenv_add_type_vars(unsigned,struct Cyc_Tcenv_Tenv*,struct Cyc_List_List*);
# 139
struct Cyc_Tcenv_Tenv*Cyc_Tcenv_add_region(struct Cyc_Tcenv_Tenv*,void*,int opened);
# 141
void Cyc_Tcenv_check_rgn_accessible(struct Cyc_Tcenv_Tenv*,unsigned,void*rgn);
# 30 "tcutil.h"
void*Cyc_Tcutil_impos(struct _fat_ptr,struct _fat_ptr);
void Cyc_Tcutil_terr(unsigned,struct _fat_ptr,struct _fat_ptr);
void Cyc_Tcutil_warn(unsigned,struct _fat_ptr,struct _fat_ptr);
# 41
int Cyc_Tcutil_is_arithmetic_type(void*);
# 45
int Cyc_Tcutil_is_array_type(void*);
# 97
struct Cyc_Absyn_Exp*Cyc_Tcutil_deep_copy_exp(int preserve_types,struct Cyc_Absyn_Exp*);
# 100
int Cyc_Tcutil_kind_leq(struct Cyc_Absyn_Kind*,struct Cyc_Absyn_Kind*);
# 106
struct Cyc_Absyn_Kind*Cyc_Tcutil_type_kind(void*);
void*Cyc_Tcutil_compress(void*);
# 114
int Cyc_Tcutil_coerce_assign(struct Cyc_Tcenv_Tenv*te,struct Cyc_Absyn_Exp*,void*);
# 121
int Cyc_Tcutil_subtype(struct Cyc_Tcenv_Tenv*te,struct Cyc_List_List*assume,void*t1,void*t2);
# 136
extern struct Cyc_Absyn_Kind Cyc_Tcutil_ak;
# 147
extern struct Cyc_Absyn_Kind Cyc_Tcutil_tmk;
# 154
extern struct Cyc_Core_Opt Cyc_Tcutil_rko;
extern struct Cyc_Core_Opt Cyc_Tcutil_ako;
# 157
extern struct Cyc_Core_Opt Cyc_Tcutil_mko;
# 163
extern struct Cyc_Core_Opt Cyc_Tcutil_trko;
# 185
void*Cyc_Tcutil_substitute(struct Cyc_List_List*,void*);
# 187
void*Cyc_Tcutil_rsubstitute(struct _RegionHandle*,struct Cyc_List_List*,void*);struct _tuple13{struct Cyc_List_List*f1;struct _RegionHandle*f2;};struct _tuple14{struct Cyc_Absyn_Tvar*f1;void*f2;};
# 207
struct _tuple14*Cyc_Tcutil_r_make_inst_var(struct _tuple13*,struct Cyc_Absyn_Tvar*);
# 214
void Cyc_Tcutil_check_unique_vars(struct Cyc_List_List*,unsigned,struct _fat_ptr err_msg);
# 222
struct Cyc_List_List*Cyc_Tcutil_resolve_aggregate_designators(struct _RegionHandle*,unsigned,struct Cyc_List_List*,enum Cyc_Absyn_AggrKind,struct Cyc_List_List*);
# 233
int Cyc_Tcutil_is_noalias_region(void*r,int must_be_unique);
# 241
int Cyc_Tcutil_is_noalias_path(struct Cyc_Absyn_Exp*);
# 246
int Cyc_Tcutil_is_noalias_pointer_or_aggr(void*);
# 256
struct Cyc_Absyn_Tvar*Cyc_Tcutil_new_tvar(void*);
# 273
int Cyc_Tcutil_is_const_exp(struct Cyc_Absyn_Exp*);
# 298
void*Cyc_Tcutil_promote_array(void*t,void*rgn,int convert_tag);
# 305
int Cyc_Tcutil_force_type2bool(int desired,void*);
# 308
void*Cyc_Tcutil_any_bool(struct Cyc_List_List*);
# 310
void*Cyc_Tcutil_any_bounds(struct Cyc_List_List*);
# 44 "tctyp.h"
void Cyc_Tctyp_check_type(unsigned,struct Cyc_Tcenv_Tenv*,struct Cyc_List_List*bound_tvars,struct Cyc_Absyn_Kind*k,int allow_evars,int allow_abs_aggr,void*);
# 28 "tcexp.h"
void*Cyc_Tcexp_tcExp(struct Cyc_Tcenv_Tenv*,void**,struct Cyc_Absyn_Exp*);struct Cyc_Tcpat_TcPatResult{struct _tuple1*tvars_and_bounds_opt;struct Cyc_List_List*patvars;};struct Cyc_Tcpat_WhereTest_Tcpat_PatTest_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Tcpat_EqNull_Tcpat_PatTest_struct{int tag;};struct Cyc_Tcpat_NeqNull_Tcpat_PatTest_struct{int tag;};struct Cyc_Tcpat_EqEnum_Tcpat_PatTest_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Tcpat_EqAnonEnum_Tcpat_PatTest_struct{int tag;void*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Tcpat_EqFloat_Tcpat_PatTest_struct{int tag;struct _fat_ptr f1;int f2;};struct Cyc_Tcpat_EqConst_Tcpat_PatTest_struct{int tag;unsigned f1;};struct Cyc_Tcpat_EqDatatypeTag_Tcpat_PatTest_struct{int tag;int f1;struct Cyc_Absyn_Datatypedecl*f2;struct Cyc_Absyn_Datatypefield*f3;};struct Cyc_Tcpat_EqTaggedUnion_Tcpat_PatTest_struct{int tag;struct _fat_ptr*f1;int f2;};struct Cyc_Tcpat_EqExtensibleDatatype_Tcpat_PatTest_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;struct Cyc_Absyn_Datatypefield*f2;};struct Cyc_Tcpat_Dummy_Tcpat_Access_struct{int tag;};struct Cyc_Tcpat_Deref_Tcpat_Access_struct{int tag;};struct Cyc_Tcpat_TupleField_Tcpat_Access_struct{int tag;unsigned f1;};struct Cyc_Tcpat_DatatypeField_Tcpat_Access_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;struct Cyc_Absyn_Datatypefield*f2;unsigned f3;};struct Cyc_Tcpat_AggrField_Tcpat_Access_struct{int tag;int f1;struct _fat_ptr*f2;};struct _union_PatOrWhere_pattern{int tag;struct Cyc_Absyn_Pat*val;};struct _union_PatOrWhere_where_clause{int tag;struct Cyc_Absyn_Exp*val;};union Cyc_Tcpat_PatOrWhere{struct _union_PatOrWhere_pattern pattern;struct _union_PatOrWhere_where_clause where_clause;};struct Cyc_Tcpat_PathNode{union Cyc_Tcpat_PatOrWhere orig_pat;void*access;};struct Cyc_Tcpat_Rhs{int used;unsigned pat_loc;struct Cyc_Absyn_Stmt*rhs;};struct Cyc_Tcpat_Failure_Tcpat_Decision_struct{int tag;void*f1;};struct Cyc_Tcpat_Success_Tcpat_Decision_struct{int tag;struct Cyc_Tcpat_Rhs*f1;};struct Cyc_Tcpat_SwitchDec_Tcpat_Decision_struct{int tag;struct Cyc_List_List*f1;struct Cyc_List_List*f2;void*f3;};struct _tuple15{unsigned f1;int f2;};
# 28 "evexp.h"
struct _tuple15 Cyc_Evexp_eval_const_uint_exp(struct Cyc_Absyn_Exp*e);
# 45 "tcpat.cyc"
static void Cyc_Tcpat_resolve_pat(struct Cyc_Tcenv_Tenv*te,void**topt,struct Cyc_Absyn_Pat*p){
void*_stmttmp0=p->r;void*_tmp0=_stmttmp0;struct Cyc_Absyn_Exp*_tmp1;int _tmp5;struct Cyc_List_List*_tmp4;struct Cyc_List_List**_tmp3;struct Cyc_Absyn_Aggrdecl**_tmp2;int _tmp8;struct Cyc_List_List*_tmp7;struct Cyc_List_List*_tmp6;switch(*((int*)_tmp0)){case 7U: if(((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmp0)->f1 == 0){_LL1: _tmp6=((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmp0)->f2;_tmp7=((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmp0)->f3;_tmp8=((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmp0)->f4;_LL2: {struct Cyc_List_List*exist_ts=_tmp6;struct Cyc_List_List*dps=_tmp7;int dots=_tmp8;
# 51
if(topt == 0)
({void*_tmp9=0U;({unsigned _tmp36E=p->loc;struct _fat_ptr _tmp36D=({const char*_tmpA="cannot determine pattern type";_tag_fat(_tmpA,sizeof(char),30U);});Cyc_Tcutil_terr(_tmp36E,_tmp36D,_tag_fat(_tmp9,sizeof(void*),0U));});});{
# 51
void*t=
# 53
Cyc_Tcutil_compress(*((void**)_check_null(topt)));
{void*_tmpB=t;union Cyc_Absyn_AggrInfo _tmpC;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmpB)->tag == 0U){if(((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmpB)->f1)->tag == 20U){_LL10: _tmpC=((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmpB)->f1)->f1;_LL11: {union Cyc_Absyn_AggrInfo ainfo=_tmpC;
# 56
({void*_tmp370=(void*)({struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*_tmpE=_cycalloc(sizeof(*_tmpE));_tmpE->tag=7U,({union Cyc_Absyn_AggrInfo*_tmp36F=({union Cyc_Absyn_AggrInfo*_tmpD=_cycalloc(sizeof(*_tmpD));*_tmpD=ainfo;_tmpD;});_tmpE->f1=_tmp36F;}),_tmpE->f2=exist_ts,_tmpE->f3=dps,_tmpE->f4=dots;_tmpE;});p->r=_tmp370;});
Cyc_Tcpat_resolve_pat(te,topt,p);
goto _LLF;}}else{goto _LL12;}}else{_LL12: _LL13:
# 60
({struct Cyc_String_pa_PrintArg_struct _tmp11=({struct Cyc_String_pa_PrintArg_struct _tmp31C;_tmp31C.tag=0U,({struct _fat_ptr _tmp371=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t));_tmp31C.f1=_tmp371;});_tmp31C;});void*_tmpF[1U];_tmpF[0]=& _tmp11;({unsigned _tmp373=p->loc;struct _fat_ptr _tmp372=({const char*_tmp10="pattern expects aggregate type instead of %s";_tag_fat(_tmp10,sizeof(char),45U);});Cyc_Tcutil_terr(_tmp373,_tmp372,_tag_fat(_tmpF,sizeof(void*),1U));});});
goto _LLF;}_LLF:;}
# 63
return;}}}else{if((((union Cyc_Absyn_AggrInfo*)((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmp0)->f1)->KnownAggr).tag == 2){_LL3: _tmp2=((((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmp0)->f1)->KnownAggr).val;_tmp3=(struct Cyc_List_List**)&((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmp0)->f2;_tmp4=((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmp0)->f3;_tmp5=((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmp0)->f4;_LL4: {struct Cyc_Absyn_Aggrdecl**adp=_tmp2;struct Cyc_List_List**exist_ts=_tmp3;struct Cyc_List_List*dps=_tmp4;int dots=_tmp5;
# 66
struct Cyc_Absyn_Aggrdecl*ad=*adp;
if(ad->impl == 0){
({struct Cyc_String_pa_PrintArg_struct _tmp14=({struct Cyc_String_pa_PrintArg_struct _tmp31D;_tmp31D.tag=0U,({
struct _fat_ptr _tmp374=(struct _fat_ptr)((int)ad->kind == (int)Cyc_Absyn_StructA?({const char*_tmp15="struct";_tag_fat(_tmp15,sizeof(char),7U);}):({const char*_tmp16="union";_tag_fat(_tmp16,sizeof(char),6U);}));_tmp31D.f1=_tmp374;});_tmp31D;});void*_tmp12[1U];_tmp12[0]=& _tmp14;({unsigned _tmp376=p->loc;struct _fat_ptr _tmp375=({const char*_tmp13="can't destructure an abstract %s";_tag_fat(_tmp13,sizeof(char),33U);});Cyc_Tcutil_terr(_tmp376,_tmp375,_tag_fat(_tmp12,sizeof(void*),1U));});});
p->r=(void*)& Cyc_Absyn_Wild_p_val;
return;}{
# 67
int more_exists=({
# 73
int _tmp377=Cyc_List_length(((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->exist_vars);_tmp377 - Cyc_List_length(*exist_ts);});
if(more_exists < 0){
({void*_tmp17=0U;({unsigned _tmp379=p->loc;struct _fat_ptr _tmp378=({const char*_tmp18="too many existentially bound type variables in pattern";_tag_fat(_tmp18,sizeof(char),55U);});Cyc_Tcutil_terr(_tmp379,_tmp378,_tag_fat(_tmp17,sizeof(void*),0U));});});{
struct Cyc_List_List**ts=exist_ts;
{int n=Cyc_List_length(((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->exist_vars);for(0;n != 0;-- n){
ts=&((struct Cyc_List_List*)_check_null(*ts))->tl;}}
*ts=0;}}else{
if(more_exists > 0){
# 82
struct Cyc_List_List*new_ts=0;
for(0;more_exists != 0;-- more_exists){
new_ts=({struct Cyc_List_List*_tmp1A=_cycalloc(sizeof(*_tmp1A));({struct Cyc_Absyn_Tvar*_tmp37A=Cyc_Tcutil_new_tvar((void*)({struct Cyc_Absyn_Unknown_kb_Absyn_KindBound_struct*_tmp19=_cycalloc(sizeof(*_tmp19));_tmp19->tag=1U,_tmp19->f1=0;_tmp19;}));_tmp1A->hd=_tmp37A;}),_tmp1A->tl=new_ts;_tmp1A;});}
({struct Cyc_List_List*_tmp37B=Cyc_List_imp_append(*exist_ts,new_ts);*exist_ts=_tmp37B;});}}
# 74
return;}}}else{_LLB: _LLC:
# 100
({void*_tmp26=0U;({int(*_tmp37D)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp28)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp28;});struct _fat_ptr _tmp37C=({const char*_tmp27="resolve_pat unknownAggr";_tag_fat(_tmp27,sizeof(char),24U);});_tmp37D(_tmp37C,_tag_fat(_tmp26,sizeof(void*),0U));});});}}case 17U: _LL5: _tmp1=((struct Cyc_Absyn_Exp_p_Absyn_Raw_pat_struct*)_tmp0)->f1;_LL6: {struct Cyc_Absyn_Exp*e=_tmp1;
# 89
Cyc_Tcexp_tcExp(te,0,e);
if(!Cyc_Tcutil_is_const_exp(e)){
({void*_tmp1B=0U;({unsigned _tmp37F=p->loc;struct _fat_ptr _tmp37E=({const char*_tmp1C="non-constant expression in case pattern";_tag_fat(_tmp1C,sizeof(char),40U);});Cyc_Tcutil_terr(_tmp37F,_tmp37E,_tag_fat(_tmp1B,sizeof(void*),0U));});});
p->r=(void*)& Cyc_Absyn_Wild_p_val;}{
# 90
struct _tuple15 _stmttmp1=
# 94
Cyc_Evexp_eval_const_uint_exp(e);
# 90
int _tmp1E;unsigned _tmp1D;_LL15: _tmp1D=_stmttmp1.f1;_tmp1E=_stmttmp1.f2;_LL16: {unsigned i=_tmp1D;int known=_tmp1E;
# 95
({void*_tmp380=(void*)({struct Cyc_Absyn_Int_p_Absyn_Raw_pat_struct*_tmp1F=_cycalloc(sizeof(*_tmp1F));_tmp1F->tag=10U,_tmp1F->f1=Cyc_Absyn_None,_tmp1F->f2=(int)i;_tmp1F;});p->r=_tmp380;});
return;}}}case 15U: _LL7: _LL8:
# 98
({void*_tmp20=0U;({int(*_tmp382)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp22)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp22;});struct _fat_ptr _tmp381=({const char*_tmp21="resolve_pat UnknownId_p";_tag_fat(_tmp21,sizeof(char),24U);});_tmp382(_tmp381,_tag_fat(_tmp20,sizeof(void*),0U));});});case 16U: _LL9: _LLA:
({void*_tmp23=0U;({int(*_tmp384)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp25)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp25;});struct _fat_ptr _tmp383=({const char*_tmp24="resolve_pat UnknownCall_p";_tag_fat(_tmp24,sizeof(char),26U);});_tmp384(_tmp383,_tag_fat(_tmp23,sizeof(void*),0U));});});default: _LLD: _LLE:
# 102
 return;}_LL0:;}
# 45
static struct _fat_ptr*Cyc_Tcpat_get_name(struct Cyc_Absyn_Vardecl*vd){
# 107
return(*vd->name).f2;}
# 45
static void*Cyc_Tcpat_any_type(struct Cyc_List_List*s,void**topt){
# 110
if(topt != 0)
return*topt;
# 110
return
# 112
 Cyc_Absyn_new_evar(& Cyc_Tcutil_mko,({struct Cyc_Core_Opt*_tmp29=_cycalloc(sizeof(*_tmp29));_tmp29->v=s;_tmp29;}));}
# 45
static void*Cyc_Tcpat_num_type(void**topt,void*numt){
# 118
if(topt != 0 && Cyc_Tcutil_is_arithmetic_type(*topt))
return*topt;
# 118
{void*_stmttmp2=
# 121
Cyc_Tcutil_compress(numt);
# 118
void*_tmp2A=_stmttmp2;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2A)->tag == 0U)switch(*((int*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2A)->f1)){case 15U: _LL1: _LL2:
# 122
 goto _LL4;case 16U: _LL3: _LL4:
 if(topt != 0)return*topt;goto _LL0;default: goto _LL5;}else{_LL5: _LL6:
 goto _LL0;}_LL0:;}
# 126
return numt;}struct _tuple16{struct Cyc_Absyn_Vardecl**f1;struct Cyc_Absyn_Exp*f2;};
# 45
static void Cyc_Tcpat_set_vd(struct Cyc_Absyn_Vardecl**vd,struct Cyc_Absyn_Exp*e,struct Cyc_List_List**v_result_ptr,void*t){
# 131
if(vd != 0){
(*vd)->type=t;
({struct Cyc_Absyn_Tqual _tmp385=Cyc_Absyn_empty_tqual(0U);(*vd)->tq=_tmp385;});}
# 131
({struct Cyc_List_List*_tmp387=({struct Cyc_List_List*_tmp2C=_cycalloc(sizeof(*_tmp2C));
# 135
({struct _tuple16*_tmp386=({struct _tuple16*_tmp2B=_cycalloc(sizeof(*_tmp2B));_tmp2B->f1=vd,_tmp2B->f2=e;_tmp2B;});_tmp2C->hd=_tmp386;}),_tmp2C->tl=*v_result_ptr;_tmp2C;});
# 131
*v_result_ptr=_tmp387;});}
# 45
static struct Cyc_Tcpat_TcPatResult Cyc_Tcpat_combine_results(struct Cyc_Tcpat_TcPatResult res1,struct Cyc_Tcpat_TcPatResult res2){
# 139
struct Cyc_List_List*_tmp2E;struct _tuple1*_tmp2D;_LL1: _tmp2D=res1.tvars_and_bounds_opt;_tmp2E=res1.patvars;_LL2: {struct _tuple1*p1=_tmp2D;struct Cyc_List_List*vs1=_tmp2E;
struct Cyc_List_List*_tmp30;struct _tuple1*_tmp2F;_LL4: _tmp2F=res2.tvars_and_bounds_opt;_tmp30=res2.patvars;_LL5: {struct _tuple1*p2=_tmp2F;struct Cyc_List_List*vs2=_tmp30;
if(p1 != 0 || p2 != 0){
if(p1 == 0)
p1=({struct _tuple1*_tmp31=_cycalloc(sizeof(*_tmp31));_tmp31->f1=0,_tmp31->f2=0;_tmp31;});
# 142
if(p2 == 0)
# 145
p2=({struct _tuple1*_tmp32=_cycalloc(sizeof(*_tmp32));_tmp32->f1=0,_tmp32->f2=0;_tmp32;});
# 142
return({struct Cyc_Tcpat_TcPatResult _tmp31E;({
# 146
struct _tuple1*_tmp38B=({struct _tuple1*_tmp33=_cycalloc(sizeof(*_tmp33));({struct Cyc_List_List*_tmp38A=Cyc_List_append((*p1).f1,(*p2).f1);_tmp33->f1=_tmp38A;}),({
struct Cyc_List_List*_tmp389=Cyc_List_append((*p1).f2,(*p2).f2);_tmp33->f2=_tmp389;});_tmp33;});
# 146
_tmp31E.tvars_and_bounds_opt=_tmp38B;}),({
# 148
struct Cyc_List_List*_tmp388=Cyc_List_append(vs1,vs2);_tmp31E.patvars=_tmp388;});_tmp31E;});}
# 141
return({struct Cyc_Tcpat_TcPatResult _tmp31F;_tmp31F.tvars_and_bounds_opt=0,({
# 150
struct Cyc_List_List*_tmp38C=Cyc_List_append(vs1,vs2);_tmp31F.patvars=_tmp38C;});_tmp31F;});}}}
# 45
static struct Cyc_Absyn_Pat*Cyc_Tcpat_wild_pat(unsigned loc){
# 154
return({struct Cyc_Absyn_Pat*_tmp34=_cycalloc(sizeof(*_tmp34));_tmp34->loc=loc,_tmp34->topt=0,_tmp34->r=(void*)& Cyc_Absyn_Wild_p_val;_tmp34;});}
# 45
static void*Cyc_Tcpat_pat_promote_array(struct Cyc_Tcenv_Tenv*te,void*t,void*rgn_opt){
# 159
return Cyc_Tcutil_is_array_type(t)?({
void*_tmp38E=t;Cyc_Tcutil_promote_array(_tmp38E,rgn_opt == 0?Cyc_Absyn_new_evar(& Cyc_Tcutil_rko,({struct Cyc_Core_Opt*_tmp35=_cycalloc(sizeof(*_tmp35));({struct Cyc_List_List*_tmp38D=Cyc_Tcenv_lookup_type_vars(te);_tmp35->v=_tmp38D;});_tmp35;})): rgn_opt,0);}): t;}struct _tuple17{struct Cyc_Absyn_Tvar*f1;int f2;};
# 45
static struct _tuple17*Cyc_Tcpat_add_false(struct Cyc_Absyn_Tvar*tv){
# 166
return({struct _tuple17*_tmp36=_cycalloc(sizeof(*_tmp36));_tmp36->f1=tv,_tmp36->f2=0;_tmp36;});}struct _tuple18{struct Cyc_Absyn_Tqual f1;void*f2;};struct _tuple19{struct Cyc_List_List*f1;struct Cyc_Absyn_Pat*f2;};struct _tuple20{struct Cyc_Absyn_Aggrfield*f1;struct Cyc_Absyn_Pat*f2;};
# 45
static struct Cyc_Tcpat_TcPatResult Cyc_Tcpat_tcPatRec(struct Cyc_Tcenv_Tenv*te,struct Cyc_Absyn_Pat*p,void**topt,void*rgn_pat,int allow_ref_pat,struct Cyc_Absyn_Exp*access_exp){
# 172
Cyc_Tcpat_resolve_pat(te,topt,p);{
void*t;
struct Cyc_Tcpat_TcPatResult res=({struct Cyc_Tcpat_TcPatResult _tmp332;_tmp332.tvars_and_bounds_opt=0,_tmp332.patvars=0;_tmp332;});
# 177
{void*_stmttmp3=p->r;void*_tmp37=_stmttmp3;int _tmp3B;struct Cyc_List_List**_tmp3A;struct Cyc_Absyn_Datatypefield*_tmp39;struct Cyc_Absyn_Datatypedecl*_tmp38;int _tmp3F;struct Cyc_List_List**_tmp3E;struct Cyc_List_List*_tmp3D;struct Cyc_Absyn_Aggrdecl*_tmp3C;int _tmp41;struct Cyc_List_List**_tmp40;struct Cyc_Absyn_Pat*_tmp42;void*_tmp43;struct Cyc_Absyn_Enumdecl*_tmp44;int _tmp45;struct Cyc_Absyn_Vardecl*_tmp47;struct Cyc_Absyn_Tvar*_tmp46;struct Cyc_Absyn_Pat*_tmp49;struct Cyc_Absyn_Vardecl*_tmp48;struct Cyc_Absyn_Vardecl*_tmp4B;struct Cyc_Absyn_Tvar*_tmp4A;struct Cyc_Absyn_Pat*_tmp4D;struct Cyc_Absyn_Vardecl*_tmp4C;switch(*((int*)_tmp37)){case 0U: _LL1: _LL2:
# 180
 if(topt != 0)
t=*topt;else{
# 183
t=({struct Cyc_List_List*_tmp38F=Cyc_Tcenv_lookup_type_vars(te);Cyc_Tcpat_any_type(_tmp38F,topt);});}
goto _LL0;case 1U: _LL3: _tmp4C=((struct Cyc_Absyn_Var_p_Absyn_Raw_pat_struct*)_tmp37)->f1;_tmp4D=((struct Cyc_Absyn_Var_p_Absyn_Raw_pat_struct*)_tmp37)->f2;_LL4: {struct Cyc_Absyn_Vardecl*vd=_tmp4C;struct Cyc_Absyn_Pat*p2=_tmp4D;
# 187
struct _tuple2*_stmttmp4=vd->name;struct _fat_ptr _tmp4E;_LL2E: _tmp4E=*_stmttmp4->f2;_LL2F: {struct _fat_ptr v=_tmp4E;
if(({struct _fat_ptr _tmp391=(struct _fat_ptr)v;Cyc_strcmp(_tmp391,({const char*_tmp4F="true";_tag_fat(_tmp4F,sizeof(char),5U);}));})== 0 ||({struct _fat_ptr _tmp390=(struct _fat_ptr)v;Cyc_strcmp(_tmp390,({const char*_tmp50="false";_tag_fat(_tmp50,sizeof(char),6U);}));})== 0)
({struct Cyc_String_pa_PrintArg_struct _tmp53=({struct Cyc_String_pa_PrintArg_struct _tmp320;_tmp320.tag=0U,_tmp320.f1=(struct _fat_ptr)((struct _fat_ptr)v);_tmp320;});void*_tmp51[1U];_tmp51[0]=& _tmp53;({unsigned _tmp393=p->loc;struct _fat_ptr _tmp392=({const char*_tmp52="you probably do not want to use %s as a local variable";_tag_fat(_tmp52,sizeof(char),55U);});Cyc_Tcutil_warn(_tmp393,_tmp392,_tag_fat(_tmp51,sizeof(void*),1U));});});
# 188
res=
# 190
Cyc_Tcpat_tcPatRec(te,p2,topt,rgn_pat,allow_ref_pat,access_exp);
t=(void*)_check_null(p2->topt);
# 194
{void*_stmttmp5=Cyc_Tcutil_compress(t);void*_tmp54=_stmttmp5;if(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp54)->tag == 4U){_LL31: _LL32:
# 196
 if(rgn_pat == 0 || !allow_ref_pat)
({void*_tmp55=0U;({unsigned _tmp395=p->loc;struct _fat_ptr _tmp394=({const char*_tmp56="array reference would point into unknown/unallowed region";_tag_fat(_tmp56,sizeof(char),58U);});Cyc_Tcutil_terr(_tmp395,_tmp394,_tag_fat(_tmp55,sizeof(void*),0U));});});
# 196
goto _LL30;}else{_LL33: _LL34:
# 200
 if(!Cyc_Tcutil_kind_leq(Cyc_Tcutil_type_kind(t),& Cyc_Tcutil_tmk))
({void*_tmp57=0U;({unsigned _tmp397=p->loc;struct _fat_ptr _tmp396=({const char*_tmp58="pattern would point to an abstract member";_tag_fat(_tmp58,sizeof(char),42U);});Cyc_Tcutil_terr(_tmp397,_tmp396,_tag_fat(_tmp57,sizeof(void*),0U));});});
# 200
goto _LL30;}_LL30:;}
# 204
({struct Cyc_Absyn_Vardecl**_tmp39A=({struct Cyc_Absyn_Vardecl**_tmp59=_cycalloc(sizeof(*_tmp59));*_tmp59=vd;_tmp59;});struct Cyc_Absyn_Exp*_tmp399=access_exp;struct Cyc_List_List**_tmp398=& res.patvars;Cyc_Tcpat_set_vd(_tmp39A,_tmp399,_tmp398,Cyc_Tcpat_pat_promote_array(te,t,rgn_pat));});
goto _LL0;}}case 2U: _LL5: _tmp4A=((struct Cyc_Absyn_AliasVar_p_Absyn_Raw_pat_struct*)_tmp37)->f1;_tmp4B=((struct Cyc_Absyn_AliasVar_p_Absyn_Raw_pat_struct*)_tmp37)->f2;_LL6: {struct Cyc_Absyn_Tvar*tv=_tmp4A;struct Cyc_Absyn_Vardecl*vd=_tmp4B;
# 207
struct Cyc_Tcenv_Tenv*te2=({unsigned _tmp39C=p->loc;struct Cyc_Tcenv_Tenv*_tmp39B=te;Cyc_Tcenv_add_type_vars(_tmp39C,_tmp39B,({struct Cyc_Absyn_Tvar*_tmp63[1U];_tmp63[0]=tv;Cyc_List_list(_tag_fat(_tmp63,sizeof(struct Cyc_Absyn_Tvar*),1U));}));});
if(res.tvars_and_bounds_opt == 0)
({struct _tuple1*_tmp39D=({struct _tuple1*_tmp5A=_cycalloc(sizeof(*_tmp5A));_tmp5A->f1=0,_tmp5A->f2=0;_tmp5A;});res.tvars_and_bounds_opt=_tmp39D;});
# 208
({struct Cyc_List_List*_tmp3A0=({
# 211
struct Cyc_List_List*_tmp39F=(*res.tvars_and_bounds_opt).f1;Cyc_List_append(_tmp39F,({struct Cyc_List_List*_tmp5C=_cycalloc(sizeof(*_tmp5C));({struct _tuple17*_tmp39E=({struct _tuple17*_tmp5B=_cycalloc(sizeof(*_tmp5B));_tmp5B->f1=tv,_tmp5B->f2=1;_tmp5B;});_tmp5C->hd=_tmp39E;}),_tmp5C->tl=0;_tmp5C;}));});
# 208
(*res.tvars_and_bounds_opt).f1=_tmp3A0;});
# 212
({unsigned _tmp3A3=p->loc;struct Cyc_Tcenv_Tenv*_tmp3A2=te2;struct Cyc_List_List*_tmp3A1=Cyc_Tcenv_lookup_type_vars(te2);Cyc_Tctyp_check_type(_tmp3A3,_tmp3A2,_tmp3A1,& Cyc_Tcutil_tmk,1,0,vd->type);});
# 215
if(topt != 0)t=*topt;else{
t=({struct Cyc_List_List*_tmp3A4=Cyc_Tcenv_lookup_type_vars(te);Cyc_Tcpat_any_type(_tmp3A4,topt);});}
{void*_stmttmp6=Cyc_Tcutil_compress(t);void*_tmp5D=_stmttmp6;if(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp5D)->tag == 4U){_LL36: _LL37:
# 219
 if(rgn_pat == 0 || !allow_ref_pat)
({void*_tmp5E=0U;({unsigned _tmp3A6=p->loc;struct _fat_ptr _tmp3A5=({const char*_tmp5F="array reference would point into unknown/unallowed region";_tag_fat(_tmp5F,sizeof(char),58U);});Cyc_Tcutil_terr(_tmp3A6,_tmp3A5,_tag_fat(_tmp5E,sizeof(void*),0U));});});
# 219
goto _LL35;}else{_LL38: _LL39:
# 223
 if(!Cyc_Tcutil_kind_leq(Cyc_Tcutil_type_kind(t),& Cyc_Tcutil_tmk))
({void*_tmp60=0U;({unsigned _tmp3A8=p->loc;struct _fat_ptr _tmp3A7=({const char*_tmp61="pattern would point to an abstract member";_tag_fat(_tmp61,sizeof(char),42U);});Cyc_Tcutil_terr(_tmp3A8,_tmp3A7,_tag_fat(_tmp60,sizeof(void*),0U));});});
# 223
goto _LL35;}_LL35:;}
# 227
({struct Cyc_Absyn_Vardecl**_tmp3AB=({struct Cyc_Absyn_Vardecl**_tmp62=_cycalloc(sizeof(*_tmp62));*_tmp62=vd;_tmp62;});struct Cyc_Absyn_Exp*_tmp3AA=access_exp;struct Cyc_List_List**_tmp3A9=& res.patvars;Cyc_Tcpat_set_vd(_tmp3AB,_tmp3AA,_tmp3A9,vd->type);});
goto _LL0;}case 3U: _LL7: _tmp48=((struct Cyc_Absyn_Reference_p_Absyn_Raw_pat_struct*)_tmp37)->f1;_tmp49=((struct Cyc_Absyn_Reference_p_Absyn_Raw_pat_struct*)_tmp37)->f2;_LL8: {struct Cyc_Absyn_Vardecl*vd=_tmp48;struct Cyc_Absyn_Pat*p2=_tmp49;
# 231
res=Cyc_Tcpat_tcPatRec(te,p2,topt,rgn_pat,allow_ref_pat,access_exp);
t=(void*)_check_null(p2->topt);
if(!allow_ref_pat || rgn_pat == 0){
({void*_tmp64=0U;({unsigned _tmp3AD=p->loc;struct _fat_ptr _tmp3AC=({const char*_tmp65="* pattern would point into an unknown/unallowed region";_tag_fat(_tmp65,sizeof(char),55U);});Cyc_Tcutil_terr(_tmp3AD,_tmp3AC,_tag_fat(_tmp64,sizeof(void*),0U));});});
goto _LL0;}else{
# 238
if(Cyc_Tcutil_is_noalias_pointer_or_aggr(t))
({void*_tmp66=0U;({unsigned _tmp3AF=p->loc;struct _fat_ptr _tmp3AE=({const char*_tmp67="* pattern cannot take the address of an alias-free path";_tag_fat(_tmp67,sizeof(char),56U);});Cyc_Tcutil_terr(_tmp3AF,_tmp3AE,_tag_fat(_tmp66,sizeof(void*),0U));});});}{
# 241
struct Cyc_Absyn_Exp*new_access_exp=0;
struct Cyc_List_List*tvs=Cyc_Tcenv_lookup_type_vars(te);
void*t2=(void*)({struct Cyc_Absyn_PointerType_Absyn_Type_struct*_tmp69=_cycalloc(sizeof(*_tmp69));_tmp69->tag=3U,(_tmp69->f1).elt_type=t,({struct Cyc_Absyn_Tqual _tmp3B2=Cyc_Absyn_empty_tqual(0U);(_tmp69->f1).elt_tq=_tmp3B2;}),
((_tmp69->f1).ptr_atts).rgn=rgn_pat,((_tmp69->f1).ptr_atts).nullable=Cyc_Absyn_false_type,({
void*_tmp3B1=Cyc_Tcutil_any_bounds(tvs);((_tmp69->f1).ptr_atts).bounds=_tmp3B1;}),({void*_tmp3B0=Cyc_Tcutil_any_bool(tvs);((_tmp69->f1).ptr_atts).zero_term=_tmp3B0;}),((_tmp69->f1).ptr_atts).ptrloc=0;_tmp69;});
# 247
if((unsigned)access_exp){
new_access_exp=Cyc_Absyn_address_exp(access_exp,0U);
new_access_exp->topt=t2;}
# 247
({struct Cyc_Absyn_Vardecl**_tmp3B5=({struct Cyc_Absyn_Vardecl**_tmp68=_cycalloc(sizeof(*_tmp68));*_tmp68=vd;_tmp68;});struct Cyc_Absyn_Exp*_tmp3B4=new_access_exp;struct Cyc_List_List**_tmp3B3=& res.patvars;Cyc_Tcpat_set_vd(_tmp3B5,_tmp3B4,_tmp3B3,t2);});
# 252
goto _LL0;}}case 4U: _LL9: _tmp46=((struct Cyc_Absyn_TagInt_p_Absyn_Raw_pat_struct*)_tmp37)->f1;_tmp47=((struct Cyc_Absyn_TagInt_p_Absyn_Raw_pat_struct*)_tmp37)->f2;_LLA: {struct Cyc_Absyn_Tvar*tv=_tmp46;struct Cyc_Absyn_Vardecl*vd=_tmp47;
# 257
({struct Cyc_Absyn_Vardecl**_tmp3B8=({struct Cyc_Absyn_Vardecl**_tmp6A=_cycalloc(sizeof(*_tmp6A));*_tmp6A=vd;_tmp6A;});struct Cyc_Absyn_Exp*_tmp3B7=access_exp;struct Cyc_List_List**_tmp3B6=& res.patvars;Cyc_Tcpat_set_vd(_tmp3B8,_tmp3B7,_tmp3B6,vd->type);});
# 261
({unsigned _tmp3BA=p->loc;struct Cyc_Tcenv_Tenv*_tmp3B9=te;Cyc_Tcenv_add_type_vars(_tmp3BA,_tmp3B9,({struct Cyc_Absyn_Tvar*_tmp6B[1U];_tmp6B[0]=tv;Cyc_List_list(_tag_fat(_tmp6B,sizeof(struct Cyc_Absyn_Tvar*),1U));}));});
if(res.tvars_and_bounds_opt == 0)
({struct _tuple1*_tmp3BB=({struct _tuple1*_tmp6C=_cycalloc(sizeof(*_tmp6C));_tmp6C->f1=0,_tmp6C->f2=0;_tmp6C;});res.tvars_and_bounds_opt=_tmp3BB;});
# 262
({struct Cyc_List_List*_tmp3BD=({struct Cyc_List_List*_tmp6E=_cycalloc(sizeof(*_tmp6E));
# 265
({struct _tuple17*_tmp3BC=({struct _tuple17*_tmp6D=_cycalloc(sizeof(*_tmp6D));_tmp6D->f1=tv,_tmp6D->f2=0;_tmp6D;});_tmp6E->hd=_tmp3BC;}),_tmp6E->tl=(*res.tvars_and_bounds_opt).f1;_tmp6E;});
# 262
(*res.tvars_and_bounds_opt).f1=_tmp3BD;});
# 266
t=Cyc_Absyn_uint_type;
goto _LL0;}case 10U: switch(((struct Cyc_Absyn_Int_p_Absyn_Raw_pat_struct*)_tmp37)->f1){case Cyc_Absyn_Unsigned: _LLB: _LLC:
# 269
 t=Cyc_Tcpat_num_type(topt,Cyc_Absyn_uint_type);goto _LL0;case Cyc_Absyn_None: _LLD: _LLE:
 goto _LL10;default: _LLF: _LL10:
 t=Cyc_Tcpat_num_type(topt,Cyc_Absyn_sint_type);goto _LL0;}case 11U: _LL11: _LL12:
 t=Cyc_Tcpat_num_type(topt,Cyc_Absyn_char_type);goto _LL0;case 12U: _LL13: _tmp45=((struct Cyc_Absyn_Float_p_Absyn_Raw_pat_struct*)_tmp37)->f2;_LL14: {int i=_tmp45;
t=({void**_tmp3BE=topt;Cyc_Tcpat_num_type(_tmp3BE,Cyc_Absyn_gen_float_type((unsigned)i));});goto _LL0;}case 13U: _LL15: _tmp44=((struct Cyc_Absyn_Enum_p_Absyn_Raw_pat_struct*)_tmp37)->f1;_LL16: {struct Cyc_Absyn_Enumdecl*ed=_tmp44;
# 275
t=({void**_tmp3BF=topt;Cyc_Tcpat_num_type(_tmp3BF,Cyc_Absyn_enum_type(ed->name,ed));});
goto _LL0;}case 14U: _LL17: _tmp43=(void*)((struct Cyc_Absyn_AnonEnum_p_Absyn_Raw_pat_struct*)_tmp37)->f1;_LL18: {void*tenum=_tmp43;
t=Cyc_Tcpat_num_type(topt,tenum);goto _LL0;}case 9U: _LL19: _LL1A:
# 279
 if(topt != 0){
void*_stmttmp7=Cyc_Tcutil_compress(*topt);void*_tmp6F=_stmttmp7;if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp6F)->tag == 3U){_LL3B: _LL3C:
 t=*topt;goto tcpat_end;}else{_LL3D: _LL3E:
 goto _LL3A;}_LL3A:;}{
# 279
struct Cyc_List_List*tvs=
# 284
Cyc_Tcenv_lookup_type_vars(te);
t=(void*)({struct Cyc_Absyn_PointerType_Absyn_Type_struct*_tmp72=_cycalloc(sizeof(*_tmp72));_tmp72->tag=3U,({void*_tmp3C4=Cyc_Absyn_new_evar(& Cyc_Tcutil_ako,({struct Cyc_Core_Opt*_tmp70=_cycalloc(sizeof(*_tmp70));_tmp70->v=tvs;_tmp70;}));(_tmp72->f1).elt_type=_tmp3C4;}),({
struct Cyc_Absyn_Tqual _tmp3C3=Cyc_Absyn_empty_tqual(0U);(_tmp72->f1).elt_tq=_tmp3C3;}),
({void*_tmp3C2=Cyc_Absyn_new_evar(& Cyc_Tcutil_trko,({struct Cyc_Core_Opt*_tmp71=_cycalloc(sizeof(*_tmp71));_tmp71->v=tvs;_tmp71;}));((_tmp72->f1).ptr_atts).rgn=_tmp3C2;}),((_tmp72->f1).ptr_atts).nullable=Cyc_Absyn_true_type,({
# 289
void*_tmp3C1=Cyc_Tcutil_any_bounds(tvs);((_tmp72->f1).ptr_atts).bounds=_tmp3C1;}),({void*_tmp3C0=Cyc_Tcutil_any_bool(tvs);((_tmp72->f1).ptr_atts).zero_term=_tmp3C0;}),((_tmp72->f1).ptr_atts).ptrloc=0;_tmp72;});
goto _LL0;}case 6U: _LL1B: _tmp42=((struct Cyc_Absyn_Pointer_p_Absyn_Raw_pat_struct*)_tmp37)->f1;_LL1C: {struct Cyc_Absyn_Pat*p2=_tmp42;
# 295
void*inner_typ=Cyc_Absyn_void_type;
void**inner_topt=0;
int elt_const=0;
if(topt != 0){
void*_stmttmp8=Cyc_Tcutil_compress(*topt);void*_tmp73=_stmttmp8;struct Cyc_Absyn_Tqual _tmp75;void*_tmp74;if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp73)->tag == 3U){_LL40: _tmp74=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp73)->f1).elt_type;_tmp75=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp73)->f1).elt_tq;_LL41: {void*elt_typ=_tmp74;struct Cyc_Absyn_Tqual elt_tq=_tmp75;
# 301
inner_typ=elt_typ;
inner_topt=& inner_typ;
elt_const=elt_tq.real_const;
goto _LL3F;}}else{_LL42: _LL43:
 goto _LL3F;}_LL3F:;}{
# 298
void*ptr_rgn=
# 310
Cyc_Absyn_new_evar(& Cyc_Tcutil_trko,Cyc_Tcenv_lookup_opt_type_vars(te));
struct Cyc_Absyn_Exp*new_access_exp=0;
if((unsigned)access_exp)new_access_exp=Cyc_Absyn_deref_exp(access_exp,0U);res=({
struct Cyc_Tcpat_TcPatResult _tmp3C5=res;Cyc_Tcpat_combine_results(_tmp3C5,Cyc_Tcpat_tcPatRec(te,p2,inner_topt,ptr_rgn,1,new_access_exp));});
# 319
{void*_stmttmp9=Cyc_Tcutil_compress((void*)_check_null(p2->topt));void*_tmp76=_stmttmp9;struct Cyc_List_List*_tmp79;struct Cyc_Absyn_Datatypefield*_tmp78;struct Cyc_Absyn_Datatypedecl*_tmp77;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp76)->tag == 0U){if(((struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp76)->f1)->tag == 19U){if(((((struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp76)->f1)->f1).KnownDatatypefield).tag == 2){_LL45: _tmp77=(((((struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp76)->f1)->f1).KnownDatatypefield).val).f1;_tmp78=(((((struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp76)->f1)->f1).KnownDatatypefield).val).f2;_tmp79=((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp76)->f2;_LL46: {struct Cyc_Absyn_Datatypedecl*tud=_tmp77;struct Cyc_Absyn_Datatypefield*tuf=_tmp78;struct Cyc_List_List*targs=_tmp79;
# 323
{void*_stmttmpA=Cyc_Tcutil_compress(inner_typ);void*_tmp7A=_stmttmpA;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp7A)->tag == 0U){if(((struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp7A)->f1)->tag == 19U){_LL4A: _LL4B:
# 325
 goto DONT_PROMOTE;}else{goto _LL4C;}}else{_LL4C: _LL4D:
 goto _LL49;}_LL49:;}{
# 329
void*new_type=({union Cyc_Absyn_DatatypeInfo _tmp3C6=Cyc_Absyn_KnownDatatype(({struct Cyc_Absyn_Datatypedecl**_tmp7C=_cycalloc(sizeof(*_tmp7C));*_tmp7C=tud;_tmp7C;}));Cyc_Absyn_datatype_type(_tmp3C6,targs);});
# 331
p2->topt=new_type;
t=(void*)({struct Cyc_Absyn_PointerType_Absyn_Type_struct*_tmp7B=_cycalloc(sizeof(*_tmp7B));_tmp7B->tag=3U,(_tmp7B->f1).elt_type=new_type,
elt_const?({struct Cyc_Absyn_Tqual _tmp3CA=Cyc_Absyn_const_tqual(0U);(_tmp7B->f1).elt_tq=_tmp3CA;}):({
struct Cyc_Absyn_Tqual _tmp3C9=Cyc_Absyn_empty_tqual(0U);(_tmp7B->f1).elt_tq=_tmp3C9;}),
((_tmp7B->f1).ptr_atts).rgn=ptr_rgn,({void*_tmp3C8=Cyc_Tcutil_any_bool(Cyc_Tcenv_lookup_type_vars(te));((_tmp7B->f1).ptr_atts).nullable=_tmp3C8;}),({
void*_tmp3C7=Cyc_Absyn_bounds_one();((_tmp7B->f1).ptr_atts).bounds=_tmp3C7;}),((_tmp7B->f1).ptr_atts).zero_term=Cyc_Absyn_false_type,((_tmp7B->f1).ptr_atts).ptrloc=0;_tmp7B;});
# 338
goto _LL44;}}}else{goto _LL47;}}else{goto _LL47;}}else{_LL47: _LL48:
# 340
 DONT_PROMOTE: {
struct Cyc_List_List*tvs=Cyc_Tcenv_lookup_type_vars(te);
t=(void*)({struct Cyc_Absyn_PointerType_Absyn_Type_struct*_tmp7D=_cycalloc(sizeof(*_tmp7D));_tmp7D->tag=3U,(_tmp7D->f1).elt_type=(void*)_check_null(p2->topt),
elt_const?({struct Cyc_Absyn_Tqual _tmp3CF=Cyc_Absyn_const_tqual(0U);(_tmp7D->f1).elt_tq=_tmp3CF;}):({
struct Cyc_Absyn_Tqual _tmp3CE=Cyc_Absyn_empty_tqual(0U);(_tmp7D->f1).elt_tq=_tmp3CE;}),
((_tmp7D->f1).ptr_atts).rgn=ptr_rgn,({void*_tmp3CD=Cyc_Tcutil_any_bool(tvs);((_tmp7D->f1).ptr_atts).nullable=_tmp3CD;}),({
void*_tmp3CC=Cyc_Tcutil_any_bounds(tvs);((_tmp7D->f1).ptr_atts).bounds=_tmp3CC;}),({void*_tmp3CB=Cyc_Tcutil_any_bool(tvs);((_tmp7D->f1).ptr_atts).zero_term=_tmp3CB;}),((_tmp7D->f1).ptr_atts).ptrloc=0;_tmp7D;});}}_LL44:;}
# 348
if((unsigned)new_access_exp)new_access_exp->topt=p2->topt;goto _LL0;}}case 5U: _LL1D: _tmp40=(struct Cyc_List_List**)&((struct Cyc_Absyn_Tuple_p_Absyn_Raw_pat_struct*)_tmp37)->f1;_tmp41=((struct Cyc_Absyn_Tuple_p_Absyn_Raw_pat_struct*)_tmp37)->f2;_LL1E: {struct Cyc_List_List**ps_p=_tmp40;int dots=_tmp41;
# 352
struct Cyc_List_List*ps=*ps_p;
struct Cyc_List_List*pat_ts=0;
struct Cyc_List_List*topt_ts=0;
if(topt != 0){
void*_stmttmpB=Cyc_Tcutil_compress(*topt);void*_tmp7E=_stmttmpB;struct Cyc_List_List*_tmp7F;if(((struct Cyc_Absyn_TupleType_Absyn_Type_struct*)_tmp7E)->tag == 6U){_LL4F: _tmp7F=((struct Cyc_Absyn_TupleType_Absyn_Type_struct*)_tmp7E)->f1;_LL50: {struct Cyc_List_List*tqts=_tmp7F;
# 358
topt_ts=tqts;
if(dots){
# 361
int lps=Cyc_List_length(ps);
int lts=Cyc_List_length(tqts);
if(lps < lts){
struct Cyc_List_List*wild_ps=0;
{int i=0;for(0;i < lts - lps;++ i){
wild_ps=({struct Cyc_List_List*_tmp80=_cycalloc(sizeof(*_tmp80));({struct Cyc_Absyn_Pat*_tmp3D0=Cyc_Tcpat_wild_pat(p->loc);_tmp80->hd=_tmp3D0;}),_tmp80->tl=wild_ps;_tmp80;});}}
({struct Cyc_List_List*_tmp3D1=Cyc_List_imp_append(ps,wild_ps);*ps_p=_tmp3D1;});
ps=*ps_p;}else{
if(({int _tmp3D2=Cyc_List_length(ps);_tmp3D2 == Cyc_List_length(tqts);}))
({void*_tmp81=0U;({unsigned _tmp3D4=p->loc;struct _fat_ptr _tmp3D3=({const char*_tmp82="unnecessary ... in tuple pattern";_tag_fat(_tmp82,sizeof(char),33U);});Cyc_Tcutil_warn(_tmp3D4,_tmp3D3,_tag_fat(_tmp81,sizeof(void*),0U));});});}}
# 359
goto _LL4E;}}else{_LL51: _LL52:
# 375
 goto _LL4E;}_LL4E:;}else{
# 377
if(dots)
({void*_tmp83=0U;({unsigned _tmp3D6=p->loc;struct _fat_ptr _tmp3D5=({const char*_tmp84="cannot determine missing fields for ... in tuple pattern";_tag_fat(_tmp84,sizeof(char),57U);});Cyc_Tcutil_terr(_tmp3D6,_tmp3D5,_tag_fat(_tmp83,sizeof(void*),0U));});});}
# 355
{
# 379
int i=0;
# 355
for(0;ps != 0;(
# 379
ps=ps->tl,i ++)){
void**inner_topt=0;
if(topt_ts != 0){
inner_topt=&(*((struct _tuple18*)topt_ts->hd)).f2;
topt_ts=topt_ts->tl;}{
# 381
struct Cyc_Absyn_Exp*new_access_exp=0;
# 386
if((unsigned)access_exp)
new_access_exp=({struct Cyc_Absyn_Exp*_tmp3D7=access_exp;Cyc_Absyn_subscript_exp(_tmp3D7,
Cyc_Absyn_const_exp(({union Cyc_Absyn_Cnst _tmp321;(_tmp321.Int_c).tag=5U,((_tmp321.Int_c).val).f1=Cyc_Absyn_Unsigned,((_tmp321.Int_c).val).f2=i;_tmp321;}),0U),0U);});
# 386
res=({
# 390
struct Cyc_Tcpat_TcPatResult _tmp3D8=res;Cyc_Tcpat_combine_results(_tmp3D8,Cyc_Tcpat_tcPatRec(te,(struct Cyc_Absyn_Pat*)ps->hd,inner_topt,rgn_pat,allow_ref_pat,new_access_exp));});
# 393
if((unsigned)new_access_exp)new_access_exp->topt=((struct Cyc_Absyn_Pat*)ps->hd)->topt;pat_ts=({struct Cyc_List_List*_tmp86=_cycalloc(sizeof(*_tmp86));
({struct _tuple18*_tmp3DA=({struct _tuple18*_tmp85=_cycalloc(sizeof(*_tmp85));({struct Cyc_Absyn_Tqual _tmp3D9=Cyc_Absyn_empty_tqual(0U);_tmp85->f1=_tmp3D9;}),_tmp85->f2=(void*)_check_null(((struct Cyc_Absyn_Pat*)ps->hd)->topt);_tmp85;});_tmp86->hd=_tmp3DA;}),_tmp86->tl=pat_ts;_tmp86;});}}}
# 396
t=(void*)({struct Cyc_Absyn_TupleType_Absyn_Type_struct*_tmp87=_cycalloc(sizeof(*_tmp87));_tmp87->tag=6U,({struct Cyc_List_List*_tmp3DB=Cyc_List_imp_rev(pat_ts);_tmp87->f1=_tmp3DB;});_tmp87;});
goto _LL0;}case 7U: if(((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmp37)->f1 != 0){if((((union Cyc_Absyn_AggrInfo*)((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmp37)->f1)->KnownAggr).tag == 2){_LL1F: _tmp3C=*((((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmp37)->f1)->KnownAggr).val;_tmp3D=((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmp37)->f2;_tmp3E=(struct Cyc_List_List**)&((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmp37)->f3;_tmp3F=((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmp37)->f4;_LL20: {struct Cyc_Absyn_Aggrdecl*ad=_tmp3C;struct Cyc_List_List*exist_ts=_tmp3D;struct Cyc_List_List**dps_p=_tmp3E;int dots=_tmp3F;
# 400
struct Cyc_List_List*dps=*dps_p;
struct _fat_ptr aggr_str=(int)ad->kind == (int)Cyc_Absyn_StructA?({const char*_tmpBC="struct";_tag_fat(_tmpBC,sizeof(char),7U);}):({const char*_tmpBD="union";_tag_fat(_tmpBD,sizeof(char),6U);});
if(ad->impl == 0){
({struct Cyc_String_pa_PrintArg_struct _tmp8A=({struct Cyc_String_pa_PrintArg_struct _tmp322;_tmp322.tag=0U,_tmp322.f1=(struct _fat_ptr)((struct _fat_ptr)aggr_str);_tmp322;});void*_tmp88[1U];_tmp88[0]=& _tmp8A;({unsigned _tmp3DD=p->loc;struct _fat_ptr _tmp3DC=({const char*_tmp89="can't destructure an abstract %s";_tag_fat(_tmp89,sizeof(char),33U);});Cyc_Tcutil_terr(_tmp3DD,_tmp3DC,_tag_fat(_tmp88,sizeof(void*),1U));});});
t=Cyc_Absyn_wildtyp(Cyc_Tcenv_lookup_opt_type_vars(te));
goto _LL0;}
# 402
if(
# 409
(int)ad->kind == (int)Cyc_Absyn_UnionA &&((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->tagged)
allow_ref_pat=0;
# 402
if(exist_ts != 0){
# 415
if(topt == 0 || Cyc_Tcutil_type_kind(*topt)!= & Cyc_Tcutil_ak)
allow_ref_pat=0;}
# 402
{
# 418
struct _RegionHandle _tmp8B=_new_region("rgn");struct _RegionHandle*rgn=& _tmp8B;_push_region(rgn);
# 420
{struct Cyc_List_List*var_tvs=0;
struct Cyc_List_List*outlives_constraints=0;
struct Cyc_List_List*u=((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->exist_vars;
{struct Cyc_List_List*t=exist_ts;for(0;t != 0;t=t->tl){
struct Cyc_Absyn_Tvar*tv=(struct Cyc_Absyn_Tvar*)t->hd;
struct Cyc_Absyn_Tvar*uv=(struct Cyc_Absyn_Tvar*)((struct Cyc_List_List*)_check_null(u))->hd;
u=u->tl;{
void*kb1=Cyc_Absyn_compress_kb(tv->kind);
void*kb2=Cyc_Absyn_compress_kb(uv->kind);
int error=0;
struct Cyc_Absyn_Kind*k2;
{void*_tmp8C=kb2;struct Cyc_Absyn_Kind*_tmp8D;struct Cyc_Absyn_Kind*_tmp8E;switch(*((int*)_tmp8C)){case 2U: _LL54: _tmp8E=((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp8C)->f2;_LL55: {struct Cyc_Absyn_Kind*k=_tmp8E;
_tmp8D=k;goto _LL57;}case 0U: _LL56: _tmp8D=((struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct*)_tmp8C)->f1;_LL57: {struct Cyc_Absyn_Kind*k=_tmp8D;
k2=k;goto _LL53;}default: _LL58: _LL59:
({void*_tmp8F=0U;({int(*_tmp3DF)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp91)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp91;});struct _fat_ptr _tmp3DE=({const char*_tmp90="unconstrained existential type variable in aggregate";_tag_fat(_tmp90,sizeof(char),53U);});_tmp3DF(_tmp3DE,_tag_fat(_tmp8F,sizeof(void*),0U));});});}_LL53:;}
# 436
{void*_tmp92=kb1;struct Cyc_Core_Opt**_tmp93;struct Cyc_Absyn_Kind*_tmp95;struct Cyc_Core_Opt**_tmp94;struct Cyc_Absyn_Kind*_tmp96;switch(*((int*)_tmp92)){case 0U: _LL5B: _tmp96=((struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct*)_tmp92)->f1;_LL5C: {struct Cyc_Absyn_Kind*k1=_tmp96;
# 439
if(!Cyc_Tcutil_kind_leq(k2,k1))
error=1;
# 439
goto _LL5A;}case 2U: _LL5D: _tmp94=(struct Cyc_Core_Opt**)&((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp92)->f1;_tmp95=((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp92)->f2;_LL5E: {struct Cyc_Core_Opt**f=_tmp94;struct Cyc_Absyn_Kind*k=_tmp95;
# 442
_tmp93=f;goto _LL60;}default: _LL5F: _tmp93=(struct Cyc_Core_Opt**)&((struct Cyc_Absyn_Unknown_kb_Absyn_KindBound_struct*)_tmp92)->f1;_LL60: {struct Cyc_Core_Opt**f=_tmp93;
({struct Cyc_Core_Opt*_tmp3E0=({struct Cyc_Core_Opt*_tmp97=_cycalloc(sizeof(*_tmp97));_tmp97->v=kb2;_tmp97;});*f=_tmp3E0;});goto _LL5A;}}_LL5A:;}
# 445
if(error)
({struct Cyc_String_pa_PrintArg_struct _tmp9A=({struct Cyc_String_pa_PrintArg_struct _tmp325;_tmp325.tag=0U,_tmp325.f1=(struct _fat_ptr)((struct _fat_ptr)*tv->name);_tmp325;});struct Cyc_String_pa_PrintArg_struct _tmp9B=({struct Cyc_String_pa_PrintArg_struct _tmp324;_tmp324.tag=0U,({
# 449
struct _fat_ptr _tmp3E1=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_kindbound2string(kb1));_tmp324.f1=_tmp3E1;});_tmp324;});struct Cyc_String_pa_PrintArg_struct _tmp9C=({struct Cyc_String_pa_PrintArg_struct _tmp323;_tmp323.tag=0U,({struct _fat_ptr _tmp3E2=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_kind2string(k2));_tmp323.f1=_tmp3E2;});_tmp323;});void*_tmp98[3U];_tmp98[0]=& _tmp9A,_tmp98[1]=& _tmp9B,_tmp98[2]=& _tmp9C;({unsigned _tmp3E4=p->loc;struct _fat_ptr _tmp3E3=({const char*_tmp99="type variable %s has kind %s but must have at least kind %s";_tag_fat(_tmp99,sizeof(char),60U);});Cyc_Tcutil_terr(_tmp3E4,_tmp3E3,_tag_fat(_tmp98,sizeof(void*),3U));});});{
# 445
void*vartype=(void*)({struct Cyc_Absyn_VarType_Absyn_Type_struct*_tmpA3=_cycalloc(sizeof(*_tmpA3));
# 451
_tmpA3->tag=2U,_tmpA3->f1=tv;_tmpA3;});
var_tvs=({struct Cyc_List_List*_tmp9D=_region_malloc(rgn,sizeof(*_tmp9D));_tmp9D->hd=vartype,_tmp9D->tl=var_tvs;_tmp9D;});
# 455
if((int)k2->kind == (int)Cyc_Absyn_RgnKind){
if((int)k2->aliasqual == (int)Cyc_Absyn_Aliasable)
outlives_constraints=({struct Cyc_List_List*_tmp9F=_cycalloc(sizeof(*_tmp9F));
({struct _tuple0*_tmp3E5=({struct _tuple0*_tmp9E=_cycalloc(sizeof(*_tmp9E));_tmp9E->f1=Cyc_Absyn_empty_effect,_tmp9E->f2=vartype;_tmp9E;});_tmp9F->hd=_tmp3E5;}),_tmp9F->tl=outlives_constraints;_tmp9F;});else{
# 460
({void*_tmpA0=0U;({int(*_tmp3E7)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmpA2)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmpA2;});struct _fat_ptr _tmp3E6=({const char*_tmpA1="opened existential had unique or top region kind";_tag_fat(_tmpA1,sizeof(char),49U);});_tmp3E7(_tmp3E6,_tag_fat(_tmpA0,sizeof(void*),0U));});});}}}}}}
# 464
var_tvs=Cyc_List_imp_rev(var_tvs);{
# 466
struct Cyc_Tcenv_Tenv*te2=Cyc_Tcenv_add_type_vars(p->loc,te,exist_ts);
# 468
struct Cyc_List_List*tenv_tvs=Cyc_Tcenv_lookup_type_vars(te2);
struct _tuple13 env=({struct _tuple13 _tmp32A;_tmp32A.f1=tenv_tvs,_tmp32A.f2=rgn;_tmp32A;});
struct Cyc_List_List*all_inst=({struct Cyc_List_List*(*_tmp3E9)(struct _RegionHandle*,struct _tuple14*(*f)(struct _tuple13*,struct Cyc_Absyn_Tvar*),struct _tuple13*env,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmpBB)(struct _RegionHandle*,struct _tuple14*(*f)(struct _tuple13*,struct Cyc_Absyn_Tvar*),struct _tuple13*env,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct _RegionHandle*,struct _tuple14*(*f)(struct _tuple13*,struct Cyc_Absyn_Tvar*),struct _tuple13*env,struct Cyc_List_List*x))Cyc_List_rmap_c;_tmpBB;});struct _RegionHandle*_tmp3E8=rgn;_tmp3E9(_tmp3E8,Cyc_Tcutil_r_make_inst_var,& env,ad->tvs);});
struct Cyc_List_List*exist_inst=Cyc_List_rzip(rgn,rgn,((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->exist_vars,var_tvs);
struct Cyc_List_List*all_typs=({struct Cyc_List_List*(*_tmp3EB)(void*(*f)(struct _tuple14*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmpB9)(void*(*f)(struct _tuple14*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(void*(*f)(struct _tuple14*),struct Cyc_List_List*x))Cyc_List_map;_tmpB9;});void*(*_tmp3EA)(struct _tuple14*)=({void*(*_tmpBA)(struct _tuple14*)=(void*(*)(struct _tuple14*))Cyc_Core_snd;_tmpBA;});_tmp3EB(_tmp3EA,all_inst);});
struct Cyc_List_List*exist_typs=({struct Cyc_List_List*(*_tmp3ED)(void*(*f)(struct _tuple14*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmpB7)(void*(*f)(struct _tuple14*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(void*(*f)(struct _tuple14*),struct Cyc_List_List*x))Cyc_List_map;_tmpB7;});void*(*_tmp3EC)(struct _tuple14*)=({void*(*_tmpB8)(struct _tuple14*)=(void*(*)(struct _tuple14*))Cyc_Core_snd;_tmpB8;});_tmp3ED(_tmp3EC,exist_inst);});
struct Cyc_List_List*inst=Cyc_List_rappend(rgn,all_inst,exist_inst);
# 476
if(exist_ts != 0 ||((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->rgn_po != 0){
if(res.tvars_and_bounds_opt == 0)
({struct _tuple1*_tmp3EE=({struct _tuple1*_tmpA4=_cycalloc(sizeof(*_tmpA4));_tmpA4->f1=0,_tmpA4->f2=0;_tmpA4;});res.tvars_and_bounds_opt=_tmp3EE;});
# 477
({struct Cyc_List_List*_tmp3F1=({
# 480
struct Cyc_List_List*_tmp3F0=(*res.tvars_and_bounds_opt).f1;Cyc_List_append(_tmp3F0,({struct Cyc_List_List*(*_tmp3EF)(struct _tuple17*(*f)(struct Cyc_Absyn_Tvar*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmpA5)(struct _tuple17*(*f)(struct Cyc_Absyn_Tvar*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct _tuple17*(*f)(struct Cyc_Absyn_Tvar*),struct Cyc_List_List*x))Cyc_List_map;_tmpA5;});_tmp3EF(Cyc_Tcpat_add_false,exist_ts);}));});
# 477
(*res.tvars_and_bounds_opt).f1=_tmp3F1;});
# 481
({struct Cyc_List_List*_tmp3F2=
Cyc_List_append((*res.tvars_and_bounds_opt).f2,outlives_constraints);
# 481
(*res.tvars_and_bounds_opt).f2=_tmp3F2;});{
# 483
struct Cyc_List_List*rpo_inst=0;
{struct Cyc_List_List*rpo=((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->rgn_po;for(0;rpo != 0;rpo=rpo->tl){
rpo_inst=({struct Cyc_List_List*_tmpA7=_cycalloc(sizeof(*_tmpA7));({struct _tuple0*_tmp3F5=({struct _tuple0*_tmpA6=_cycalloc(sizeof(*_tmpA6));({void*_tmp3F4=Cyc_Tcutil_rsubstitute(rgn,inst,(*((struct _tuple0*)rpo->hd)).f1);_tmpA6->f1=_tmp3F4;}),({
void*_tmp3F3=Cyc_Tcutil_rsubstitute(rgn,inst,(*((struct _tuple0*)rpo->hd)).f2);_tmpA6->f2=_tmp3F3;});_tmpA6;});
# 485
_tmpA7->hd=_tmp3F5;}),_tmpA7->tl=rpo_inst;_tmpA7;});}}
# 488
rpo_inst=Cyc_List_imp_rev(rpo_inst);
({struct Cyc_List_List*_tmp3F6=
Cyc_List_append((*res.tvars_and_bounds_opt).f2,rpo_inst);
# 489
(*res.tvars_and_bounds_opt).f2=_tmp3F6;});}}
# 476
t=({
# 494
union Cyc_Absyn_AggrInfo _tmp3F7=Cyc_Absyn_KnownAggr(({struct Cyc_Absyn_Aggrdecl**_tmpA8=_cycalloc(sizeof(*_tmpA8));*_tmpA8=ad;_tmpA8;}));Cyc_Absyn_aggr_type(_tmp3F7,all_typs);});
if(dots &&(int)ad->kind == (int)Cyc_Absyn_UnionA)
({void*_tmpA9=0U;({unsigned _tmp3F9=p->loc;struct _fat_ptr _tmp3F8=({const char*_tmpAA="`...' pattern not allowed in union pattern";_tag_fat(_tmpAA,sizeof(char),43U);});Cyc_Tcutil_warn(_tmp3F9,_tmp3F8,_tag_fat(_tmpA9,sizeof(void*),0U));});});else{
if(dots){
# 499
int ldps=Cyc_List_length(dps);
int lfields=Cyc_List_length(((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->fields);
if(ldps < lfields){
struct Cyc_List_List*wild_dps=0;
{int i=0;for(0;i < lfields - ldps;++ i){
wild_dps=({struct Cyc_List_List*_tmpAC=_cycalloc(sizeof(*_tmpAC));({struct _tuple19*_tmp3FB=({struct _tuple19*_tmpAB=_cycalloc(sizeof(*_tmpAB));_tmpAB->f1=0,({struct Cyc_Absyn_Pat*_tmp3FA=Cyc_Tcpat_wild_pat(p->loc);_tmpAB->f2=_tmp3FA;});_tmpAB;});_tmpAC->hd=_tmp3FB;}),_tmpAC->tl=wild_dps;_tmpAC;});}}
({struct Cyc_List_List*_tmp3FC=Cyc_List_imp_append(dps,wild_dps);*dps_p=_tmp3FC;});
dps=*dps_p;}else{
if(ldps == lfields)
({void*_tmpAD=0U;({unsigned _tmp3FE=p->loc;struct _fat_ptr _tmp3FD=({const char*_tmpAE="unnecessary ... in struct pattern";_tag_fat(_tmpAE,sizeof(char),34U);});Cyc_Tcutil_warn(_tmp3FE,_tmp3FD,_tag_fat(_tmpAD,sizeof(void*),0U));});});}}}{
# 495
struct Cyc_List_List*fields=
# 511
Cyc_Tcutil_resolve_aggregate_designators(rgn,p->loc,dps,ad->kind,((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->fields);
for(0;fields != 0;fields=fields->tl){
struct _tuple20*_stmttmpC=(struct _tuple20*)fields->hd;struct Cyc_Absyn_Pat*_tmpB0;struct Cyc_Absyn_Aggrfield*_tmpAF;_LL62: _tmpAF=_stmttmpC->f1;_tmpB0=_stmttmpC->f2;_LL63: {struct Cyc_Absyn_Aggrfield*field=_tmpAF;struct Cyc_Absyn_Pat*pat=_tmpB0;
void*inst_fieldtyp=Cyc_Tcutil_rsubstitute(rgn,inst,field->type);
# 516
struct Cyc_Absyn_Exp*new_access_exp=0;
if((unsigned)access_exp)
new_access_exp=Cyc_Absyn_aggrmember_exp(access_exp,field->name,0U);
# 517
res=({
# 519
struct Cyc_Tcpat_TcPatResult _tmp3FF=res;Cyc_Tcpat_combine_results(_tmp3FF,Cyc_Tcpat_tcPatRec(te2,pat,& inst_fieldtyp,rgn_pat,allow_ref_pat,new_access_exp));});
# 524
if(!Cyc_Unify_unify((void*)_check_null(pat->topt),inst_fieldtyp))
({struct Cyc_String_pa_PrintArg_struct _tmpB3=({struct Cyc_String_pa_PrintArg_struct _tmp329;_tmp329.tag=0U,_tmp329.f1=(struct _fat_ptr)((struct _fat_ptr)*field->name);_tmp329;});struct Cyc_String_pa_PrintArg_struct _tmpB4=({struct Cyc_String_pa_PrintArg_struct _tmp328;_tmp328.tag=0U,_tmp328.f1=(struct _fat_ptr)((struct _fat_ptr)aggr_str);_tmp328;});struct Cyc_String_pa_PrintArg_struct _tmpB5=({struct Cyc_String_pa_PrintArg_struct _tmp327;_tmp327.tag=0U,({
struct _fat_ptr _tmp400=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(inst_fieldtyp));_tmp327.f1=_tmp400;});_tmp327;});struct Cyc_String_pa_PrintArg_struct _tmpB6=({struct Cyc_String_pa_PrintArg_struct _tmp326;_tmp326.tag=0U,({
struct _fat_ptr _tmp401=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string((void*)_check_null(pat->topt)));_tmp326.f1=_tmp401;});_tmp326;});void*_tmpB1[4U];_tmpB1[0]=& _tmpB3,_tmpB1[1]=& _tmpB4,_tmpB1[2]=& _tmpB5,_tmpB1[3]=& _tmpB6;({unsigned _tmp403=p->loc;struct _fat_ptr _tmp402=({const char*_tmpB2="field %s of %s pattern expects type %s != %s";_tag_fat(_tmpB2,sizeof(char),45U);});Cyc_Tcutil_terr(_tmp403,_tmp402,_tag_fat(_tmpB1,sizeof(void*),4U));});});
# 524
if((unsigned)new_access_exp)
# 528
new_access_exp->topt=pat->topt;}}}}}
# 420
;_pop_region(rgn);}
# 531
goto _LL0;}}else{_LL25: _LL26:
# 584
 goto _LL28;}}else{_LL23: _LL24:
# 583
 goto _LL26;}case 8U: _LL21: _tmp38=((struct Cyc_Absyn_Datatype_p_Absyn_Raw_pat_struct*)_tmp37)->f1;_tmp39=((struct Cyc_Absyn_Datatype_p_Absyn_Raw_pat_struct*)_tmp37)->f2;_tmp3A=(struct Cyc_List_List**)&((struct Cyc_Absyn_Datatype_p_Absyn_Raw_pat_struct*)_tmp37)->f3;_tmp3B=((struct Cyc_Absyn_Datatype_p_Absyn_Raw_pat_struct*)_tmp37)->f4;_LL22: {struct Cyc_Absyn_Datatypedecl*tud=_tmp38;struct Cyc_Absyn_Datatypefield*tuf=_tmp39;struct Cyc_List_List**ps_p=_tmp3A;int dots=_tmp3B;
# 534
struct Cyc_List_List*ps=*ps_p;
struct Cyc_List_List*tqts=tuf->typs;
# 537
struct Cyc_List_List*tenv_tvs=Cyc_Tcenv_lookup_type_vars(te);
struct _tuple13 env=({struct _tuple13 _tmp331;_tmp331.f1=tenv_tvs,_tmp331.f2=Cyc_Core_heap_region;_tmp331;});
struct Cyc_List_List*inst=({struct Cyc_List_List*(*_tmp404)(struct _tuple14*(*f)(struct _tuple13*,struct Cyc_Absyn_Tvar*),struct _tuple13*env,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmpCF)(struct _tuple14*(*f)(struct _tuple13*,struct Cyc_Absyn_Tvar*),struct _tuple13*env,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct _tuple14*(*f)(struct _tuple13*,struct Cyc_Absyn_Tvar*),struct _tuple13*env,struct Cyc_List_List*x))Cyc_List_map_c;_tmpCF;});_tmp404(Cyc_Tcutil_r_make_inst_var,& env,tud->tvs);});
struct Cyc_List_List*all_typs=({struct Cyc_List_List*(*_tmp406)(void*(*f)(struct _tuple14*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmpCD)(void*(*f)(struct _tuple14*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(void*(*f)(struct _tuple14*),struct Cyc_List_List*x))Cyc_List_map;_tmpCD;});void*(*_tmp405)(struct _tuple14*)=({void*(*_tmpCE)(struct _tuple14*)=(void*(*)(struct _tuple14*))Cyc_Core_snd;_tmpCE;});_tmp406(_tmp405,inst);});
t=({union Cyc_Absyn_DatatypeFieldInfo _tmp407=Cyc_Absyn_KnownDatatypefield(tud,tuf);Cyc_Absyn_datatype_field_type(_tmp407,all_typs);});
if(dots){
# 544
int lps=Cyc_List_length(ps);
int ltqts=Cyc_List_length(tqts);
if(lps < ltqts){
struct Cyc_List_List*wild_ps=0;
{int i=0;for(0;i < ltqts - lps;++ i){
wild_ps=({struct Cyc_List_List*_tmpBE=_cycalloc(sizeof(*_tmpBE));({struct Cyc_Absyn_Pat*_tmp408=Cyc_Tcpat_wild_pat(p->loc);_tmpBE->hd=_tmp408;}),_tmpBE->tl=wild_ps;_tmpBE;});}}
({struct Cyc_List_List*_tmp409=Cyc_List_imp_append(ps,wild_ps);*ps_p=_tmp409;});
ps=*ps_p;}else{
if(lps == ltqts)
({struct Cyc_String_pa_PrintArg_struct _tmpC1=({struct Cyc_String_pa_PrintArg_struct _tmp32B;_tmp32B.tag=0U,({
struct _fat_ptr _tmp40A=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(tud->name));_tmp32B.f1=_tmp40A;});_tmp32B;});void*_tmpBF[1U];_tmpBF[0]=& _tmpC1;({unsigned _tmp40C=p->loc;struct _fat_ptr _tmp40B=({const char*_tmpC0="unnecessary ... in datatype field %s";_tag_fat(_tmpC0,sizeof(char),37U);});Cyc_Tcutil_warn(_tmp40C,_tmp40B,_tag_fat(_tmpBF,sizeof(void*),1U));});});}}
# 542
for(0;
# 556
ps != 0 && tqts != 0;(ps=ps->tl,tqts=tqts->tl)){
struct Cyc_Absyn_Pat*p2=(struct Cyc_Absyn_Pat*)ps->hd;
# 560
void*field_typ=Cyc_Tcutil_substitute(inst,(*((struct _tuple18*)tqts->hd)).f2);
# 563
if((unsigned)access_exp)
Cyc_Tcpat_set_vd(0,access_exp,& res.patvars,Cyc_Absyn_char_type);
# 563
res=({
# 565
struct Cyc_Tcpat_TcPatResult _tmp40D=res;Cyc_Tcpat_combine_results(_tmp40D,Cyc_Tcpat_tcPatRec(te,p2,& field_typ,rgn_pat,allow_ref_pat,0));});
# 570
if(!Cyc_Unify_unify((void*)_check_null(p2->topt),field_typ))
({struct Cyc_String_pa_PrintArg_struct _tmpC4=({struct Cyc_String_pa_PrintArg_struct _tmp32E;_tmp32E.tag=0U,({
struct _fat_ptr _tmp40E=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(tuf->name));_tmp32E.f1=_tmp40E;});_tmp32E;});struct Cyc_String_pa_PrintArg_struct _tmpC5=({struct Cyc_String_pa_PrintArg_struct _tmp32D;_tmp32D.tag=0U,({struct _fat_ptr _tmp40F=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(field_typ));_tmp32D.f1=_tmp40F;});_tmp32D;});struct Cyc_String_pa_PrintArg_struct _tmpC6=({struct Cyc_String_pa_PrintArg_struct _tmp32C;_tmp32C.tag=0U,({
struct _fat_ptr _tmp410=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string((void*)_check_null(p2->topt)));_tmp32C.f1=_tmp410;});_tmp32C;});void*_tmpC2[3U];_tmpC2[0]=& _tmpC4,_tmpC2[1]=& _tmpC5,_tmpC2[2]=& _tmpC6;({unsigned _tmp412=p2->loc;struct _fat_ptr _tmp411=({const char*_tmpC3="%s expects argument type %s, not %s";_tag_fat(_tmpC3,sizeof(char),36U);});Cyc_Tcutil_terr(_tmp412,_tmp411,_tag_fat(_tmpC2,sizeof(void*),3U));});});}
# 575
if(ps != 0)
({struct Cyc_String_pa_PrintArg_struct _tmpC9=({struct Cyc_String_pa_PrintArg_struct _tmp32F;_tmp32F.tag=0U,({
struct _fat_ptr _tmp413=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(tuf->name));_tmp32F.f1=_tmp413;});_tmp32F;});void*_tmpC7[1U];_tmpC7[0]=& _tmpC9;({unsigned _tmp415=p->loc;struct _fat_ptr _tmp414=({const char*_tmpC8="too many arguments for datatype constructor %s";_tag_fat(_tmpC8,sizeof(char),47U);});Cyc_Tcutil_terr(_tmp415,_tmp414,_tag_fat(_tmpC7,sizeof(void*),1U));});});
# 575
if(tqts != 0)
# 579
({struct Cyc_String_pa_PrintArg_struct _tmpCC=({struct Cyc_String_pa_PrintArg_struct _tmp330;_tmp330.tag=0U,({
struct _fat_ptr _tmp416=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(tuf->name));_tmp330.f1=_tmp416;});_tmp330;});void*_tmpCA[1U];_tmpCA[0]=& _tmpCC;({unsigned _tmp418=p->loc;struct _fat_ptr _tmp417=({const char*_tmpCB="too few arguments for datatype constructor %s";_tag_fat(_tmpCB,sizeof(char),46U);});Cyc_Tcutil_terr(_tmp418,_tmp417,_tag_fat(_tmpCA,sizeof(void*),1U));});});
# 575
goto _LL0;}case 15U: _LL27: _LL28:
# 585
 goto _LL2A;case 17U: _LL29: _LL2A:
 goto _LL2C;default: _LL2B: _LL2C:
# 588
 t=Cyc_Absyn_wildtyp(Cyc_Tcenv_lookup_opt_type_vars(te));goto _LL0;}_LL0:;}
# 590
tcpat_end:
 p->topt=t;
return res;}}
# 45
struct Cyc_Tcpat_TcPatResult Cyc_Tcpat_tcPat(struct Cyc_Tcenv_Tenv*te,struct Cyc_Absyn_Pat*p,void**topt,struct Cyc_Absyn_Exp*pat_var_exp){
# 597
struct Cyc_Tcpat_TcPatResult ans=Cyc_Tcpat_tcPatRec(te,p,topt,0,0,pat_var_exp);
# 599
struct _tuple1 _stmttmpD=Cyc_List_split(ans.patvars);struct Cyc_List_List*_tmpD0;_LL1: _tmpD0=_stmttmpD.f1;_LL2: {struct Cyc_List_List*vs1=_tmpD0;
struct Cyc_List_List*vs=0;
{struct Cyc_List_List*x=vs1;for(0;x != 0;x=x->tl){
if((struct Cyc_Absyn_Vardecl**)x->hd != 0)vs=({struct Cyc_List_List*_tmpD1=_cycalloc(sizeof(*_tmpD1));_tmpD1->hd=*((struct Cyc_Absyn_Vardecl**)_check_null((struct Cyc_Absyn_Vardecl**)x->hd)),_tmpD1->tl=vs;_tmpD1;});}}
# 601
({struct Cyc_List_List*_tmp41B=({
# 603
struct Cyc_List_List*(*_tmp419)(struct _fat_ptr*(*f)(struct Cyc_Absyn_Vardecl*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmpD2)(struct _fat_ptr*(*f)(struct Cyc_Absyn_Vardecl*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct _fat_ptr*(*f)(struct Cyc_Absyn_Vardecl*),struct Cyc_List_List*x))Cyc_List_map;_tmpD2;});_tmp419(Cyc_Tcpat_get_name,vs);});
# 601
unsigned _tmp41A=p->loc;Cyc_Tcutil_check_unique_vars(_tmp41B,_tmp41A,({const char*_tmpD3="pattern contains a repeated variable";_tag_fat(_tmpD3,sizeof(char),37U);}));});
# 608
{struct Cyc_List_List*x=ans.patvars;for(0;x != 0;x=x->tl){
struct _tuple16*_stmttmpE=(struct _tuple16*)x->hd;struct Cyc_Absyn_Exp**_tmpD5;struct Cyc_Absyn_Vardecl**_tmpD4;_LL4: _tmpD4=_stmttmpE->f1;_tmpD5=(struct Cyc_Absyn_Exp**)& _stmttmpE->f2;_LL5: {struct Cyc_Absyn_Vardecl**vdopt=_tmpD4;struct Cyc_Absyn_Exp**expopt=_tmpD5;
if(*expopt != 0 &&*expopt != pat_var_exp)
({struct Cyc_Absyn_Exp*_tmp41C=Cyc_Tcutil_deep_copy_exp(1,(struct Cyc_Absyn_Exp*)_check_null(*expopt));*expopt=_tmp41C;});}}}
# 613
return ans;}}
# 45
static int Cyc_Tcpat_try_alias_coerce(struct Cyc_Tcenv_Tenv*tenv,void*old_type,void*new_type,struct Cyc_Absyn_Exp*initializer,struct Cyc_List_List*assump){
# 622
struct _tuple0 _stmttmpF=({struct _tuple0 _tmp333;({void*_tmp41E=Cyc_Tcutil_compress(old_type);_tmp333.f1=_tmp41E;}),({void*_tmp41D=Cyc_Tcutil_compress(new_type);_tmp333.f2=_tmp41D;});_tmp333;});struct _tuple0 _tmpD6=_stmttmpF;struct Cyc_Absyn_PtrInfo _tmpD8;struct Cyc_Absyn_PtrInfo _tmpD7;if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmpD6.f1)->tag == 3U){if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmpD6.f2)->tag == 3U){_LL1: _tmpD7=((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmpD6.f1)->f1;_tmpD8=((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmpD6.f2)->f1;_LL2: {struct Cyc_Absyn_PtrInfo pold=_tmpD7;struct Cyc_Absyn_PtrInfo pnew=_tmpD8;
# 624
struct Cyc_Absyn_PointerType_Absyn_Type_struct*ptry=({struct Cyc_Absyn_PointerType_Absyn_Type_struct*_tmpD9=_cycalloc(sizeof(*_tmpD9));_tmpD9->tag=3U,(_tmpD9->f1).elt_type=pold.elt_type,(_tmpD9->f1).elt_tq=pnew.elt_tq,
((_tmpD9->f1).ptr_atts).rgn=(pold.ptr_atts).rgn,((_tmpD9->f1).ptr_atts).nullable=(pnew.ptr_atts).nullable,((_tmpD9->f1).ptr_atts).bounds=(pnew.ptr_atts).bounds,((_tmpD9->f1).ptr_atts).zero_term=(pnew.ptr_atts).zero_term,((_tmpD9->f1).ptr_atts).ptrloc=(pold.ptr_atts).ptrloc;_tmpD9;});
# 630
return Cyc_Tcutil_subtype(tenv,assump,(void*)ptry,new_type)&&
 Cyc_Tcutil_coerce_assign(tenv,initializer,(void*)ptry);}}else{goto _LL3;}}else{_LL3: _LL4:
 return 0;}_LL0:;}
# 45
static void Cyc_Tcpat_check_alias_coercion(struct Cyc_Tcenv_Tenv*tenv,unsigned loc,void*old_type,struct Cyc_Absyn_Tvar*tv,void*new_type,struct Cyc_Absyn_Exp*initializer){
# 642
struct Cyc_List_List*assump=({struct Cyc_List_List*_tmpE5=_cycalloc(sizeof(*_tmpE5));({struct _tuple0*_tmp420=({struct _tuple0*_tmpE4=_cycalloc(sizeof(*_tmpE4));_tmpE4->f1=Cyc_Absyn_unique_rgn_type,({void*_tmp41F=(void*)({struct Cyc_Absyn_VarType_Absyn_Type_struct*_tmpE3=_cycalloc(sizeof(*_tmpE3));_tmpE3->tag=2U,_tmpE3->f1=tv;_tmpE3;});_tmpE4->f2=_tmp41F;});_tmpE4;});_tmpE5->hd=_tmp420;}),_tmpE5->tl=0;_tmpE5;});
if(Cyc_Tcutil_subtype(tenv,assump,old_type,new_type)){
# 660 "tcpat.cyc"
struct _tuple0 _stmttmp10=({struct _tuple0 _tmp334;({void*_tmp422=Cyc_Tcutil_compress(old_type);_tmp334.f1=_tmp422;}),({void*_tmp421=Cyc_Tcutil_compress(new_type);_tmp334.f2=_tmp421;});_tmp334;});struct _tuple0 _tmpDA=_stmttmp10;struct Cyc_Absyn_PtrInfo _tmpDC;struct Cyc_Absyn_PtrInfo _tmpDB;if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmpDA.f1)->tag == 3U){if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmpDA.f2)->tag == 3U){_LL1: _tmpDB=((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmpDA.f1)->f1;_tmpDC=((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmpDA.f2)->f1;_LL2: {struct Cyc_Absyn_PtrInfo pold=_tmpDB;struct Cyc_Absyn_PtrInfo pnew=_tmpDC;
goto _LL0;}}else{goto _LL3;}}else{_LL3: _LL4:
({void*_tmpDD=0U;({unsigned _tmp424=loc;struct _fat_ptr _tmp423=({const char*_tmpDE="alias requires pointer type";_tag_fat(_tmpDE,sizeof(char),28U);});Cyc_Tcutil_terr(_tmp424,_tmp423,_tag_fat(_tmpDD,sizeof(void*),0U));});});goto _LL0;}_LL0:;}else{
# 665
({struct Cyc_String_pa_PrintArg_struct _tmpE1=({struct Cyc_String_pa_PrintArg_struct _tmp336;_tmp336.tag=0U,({
struct _fat_ptr _tmp425=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(old_type));_tmp336.f1=_tmp425;});_tmp336;});struct Cyc_String_pa_PrintArg_struct _tmpE2=({struct Cyc_String_pa_PrintArg_struct _tmp335;_tmp335.tag=0U,({struct _fat_ptr _tmp426=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(new_type));_tmp335.f1=_tmp426;});_tmp335;});void*_tmpDF[2U];_tmpDF[0]=& _tmpE1,_tmpDF[1]=& _tmpE2;({unsigned _tmp428=loc;struct _fat_ptr _tmp427=({const char*_tmpE0="cannot alias value of type %s to type %s";_tag_fat(_tmpE0,sizeof(char),41U);});Cyc_Tcutil_terr(_tmp428,_tmp427,_tag_fat(_tmpDF,sizeof(void*),2U));});});}}
# 45 "tcpat.cyc"
void Cyc_Tcpat_check_pat_regions_rec(struct Cyc_Tcenv_Tenv*te,struct Cyc_Absyn_Pat*p,int did_noalias_deref,struct Cyc_List_List*patvars){
# 676 "tcpat.cyc"
void*_stmttmp11=p->r;void*_tmpE6=_stmttmp11;struct Cyc_Absyn_Vardecl*_tmpE8;struct Cyc_Absyn_Tvar*_tmpE7;struct Cyc_Absyn_Pat*_tmpEA;struct Cyc_Absyn_Vardecl*_tmpE9;struct Cyc_Absyn_Pat*_tmpEC;struct Cyc_Absyn_Vardecl*_tmpEB;struct Cyc_List_List*_tmpED;struct Cyc_List_List*_tmpEE;struct Cyc_List_List*_tmpF1;struct Cyc_List_List*_tmpF0;union Cyc_Absyn_AggrInfo*_tmpEF;struct Cyc_Absyn_Pat*_tmpF2;switch(*((int*)_tmpE6)){case 6U: _LL1: _tmpF2=((struct Cyc_Absyn_Pointer_p_Absyn_Raw_pat_struct*)_tmpE6)->f1;_LL2: {struct Cyc_Absyn_Pat*p2=_tmpF2;
# 678
void*_stmttmp12=(void*)_check_null(p->topt);void*_tmpF3=_stmttmp12;void*_tmpF4;if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmpF3)->tag == 3U){_LL12: _tmpF4=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmpF3)->f1).ptr_atts).rgn;_LL13: {void*rt=_tmpF4;
# 680
Cyc_Tcenv_check_rgn_accessible(te,p->loc,rt);
({struct Cyc_Tcenv_Tenv*_tmp42B=te;struct Cyc_Absyn_Pat*_tmp42A=p2;int _tmp429=Cyc_Tcutil_is_noalias_region(rt,0);Cyc_Tcpat_check_pat_regions_rec(_tmp42B,_tmp42A,_tmp429,patvars);});
return;}}else{_LL14: _LL15:
({void*_tmpF5=0U;({int(*_tmp42D)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmpF7)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmpF7;});struct _fat_ptr _tmp42C=({const char*_tmpF6="check_pat_regions: bad pointer type";_tag_fat(_tmpF6,sizeof(char),36U);});_tmp42D(_tmp42C,_tag_fat(_tmpF5,sizeof(void*),0U));});});}_LL11:;}case 7U: _LL3: _tmpEF=((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmpE6)->f1;_tmpF0=((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmpE6)->f2;_tmpF1=((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmpE6)->f3;_LL4: {union Cyc_Absyn_AggrInfo*ai=_tmpEF;struct Cyc_List_List*exist_ts=_tmpF0;struct Cyc_List_List*dps=_tmpF1;
# 686
for(0;dps != 0;dps=dps->tl){
Cyc_Tcpat_check_pat_regions_rec(te,(*((struct _tuple19*)dps->hd)).f2,did_noalias_deref,patvars);}
return;}case 8U: _LL5: _tmpEE=((struct Cyc_Absyn_Datatype_p_Absyn_Raw_pat_struct*)_tmpE6)->f3;_LL6: {struct Cyc_List_List*ps=_tmpEE;
# 690
did_noalias_deref=0;_tmpED=ps;goto _LL8;}case 5U: _LL7: _tmpED=((struct Cyc_Absyn_Tuple_p_Absyn_Raw_pat_struct*)_tmpE6)->f1;_LL8: {struct Cyc_List_List*ps=_tmpED;
# 692
for(0;ps != 0;ps=ps->tl){
Cyc_Tcpat_check_pat_regions_rec(te,(struct Cyc_Absyn_Pat*)ps->hd,did_noalias_deref,patvars);}
return;}case 3U: _LL9: _tmpEB=((struct Cyc_Absyn_Reference_p_Absyn_Raw_pat_struct*)_tmpE6)->f1;_tmpEC=((struct Cyc_Absyn_Reference_p_Absyn_Raw_pat_struct*)_tmpE6)->f2;_LLA: {struct Cyc_Absyn_Vardecl*vd=_tmpEB;struct Cyc_Absyn_Pat*p2=_tmpEC;
# 696
{struct Cyc_List_List*x=patvars;for(0;x != 0;x=x->tl){
struct _tuple16*_stmttmp13=(struct _tuple16*)x->hd;struct Cyc_Absyn_Exp*_tmpF9;struct Cyc_Absyn_Vardecl**_tmpF8;_LL17: _tmpF8=_stmttmp13->f1;_tmpF9=_stmttmp13->f2;_LL18: {struct Cyc_Absyn_Vardecl**vdopt=_tmpF8;struct Cyc_Absyn_Exp*eopt=_tmpF9;
# 702
if((vdopt != 0 &&*vdopt == vd)&& eopt != 0){
{void*_stmttmp14=eopt->r;void*_tmpFA=_stmttmp14;struct Cyc_Absyn_Exp*_tmpFB;if(((struct Cyc_Absyn_Address_e_Absyn_Raw_exp_struct*)_tmpFA)->tag == 15U){_LL1A: _tmpFB=((struct Cyc_Absyn_Address_e_Absyn_Raw_exp_struct*)_tmpFA)->f1;_LL1B: {struct Cyc_Absyn_Exp*e=_tmpFB;
# 705
if(Cyc_Tcutil_is_noalias_pointer_or_aggr((void*)_check_null(e->topt)))
({void*_tmpFC=0U;({unsigned _tmp42F=p->loc;struct _fat_ptr _tmp42E=({const char*_tmpFD="reference pattern not allowed on alias-free pointers";_tag_fat(_tmpFD,sizeof(char),53U);});Cyc_Tcutil_terr(_tmp42F,_tmp42E,_tag_fat(_tmpFC,sizeof(void*),0U));});});
# 705
goto _LL19;}}else{_LL1C: _LL1D:
# 709
({void*_tmpFE=0U;({int(*_tmp431)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp100)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp100;});struct _fat_ptr _tmp430=({const char*_tmpFF="check_pat_regions: bad reference access expression";_tag_fat(_tmpFF,sizeof(char),51U);});_tmp431(_tmp430,_tag_fat(_tmpFE,sizeof(void*),0U));});});}_LL19:;}
# 711
break;}}}}
# 714
Cyc_Tcpat_check_pat_regions_rec(te,p2,did_noalias_deref,patvars);
return;}case 1U: _LLB: _tmpE9=((struct Cyc_Absyn_Var_p_Absyn_Raw_pat_struct*)_tmpE6)->f1;_tmpEA=((struct Cyc_Absyn_Var_p_Absyn_Raw_pat_struct*)_tmpE6)->f2;_LLC: {struct Cyc_Absyn_Vardecl*vd=_tmpE9;struct Cyc_Absyn_Pat*p2=_tmpEA;
# 717
{void*_stmttmp15=p->topt;void*_tmp101=_stmttmp15;if(_tmp101 != 0){if(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp101)->tag == 4U){_LL1F: _LL20:
# 719
 if(did_noalias_deref){
({void*_tmp102=0U;({unsigned _tmp433=p->loc;struct _fat_ptr _tmp432=({const char*_tmp103="pattern to array would create alias of no-alias pointer";_tag_fat(_tmp103,sizeof(char),56U);});Cyc_Tcutil_terr(_tmp433,_tmp432,_tag_fat(_tmp102,sizeof(void*),0U));});});
return;}
# 719
goto _LL1E;}else{goto _LL21;}}else{_LL21: _LL22:
# 724
 goto _LL1E;}_LL1E:;}
# 726
Cyc_Tcpat_check_pat_regions_rec(te,p2,did_noalias_deref,patvars);
return;}case 2U: _LLD: _tmpE7=((struct Cyc_Absyn_AliasVar_p_Absyn_Raw_pat_struct*)_tmpE6)->f1;_tmpE8=((struct Cyc_Absyn_AliasVar_p_Absyn_Raw_pat_struct*)_tmpE6)->f2;_LLE: {struct Cyc_Absyn_Tvar*tv=_tmpE7;struct Cyc_Absyn_Vardecl*vd=_tmpE8;
# 729
{struct Cyc_List_List*x=patvars;for(0;x != 0;x=x->tl){
struct _tuple16*_stmttmp16=(struct _tuple16*)x->hd;struct Cyc_Absyn_Exp*_tmp105;struct Cyc_Absyn_Vardecl**_tmp104;_LL24: _tmp104=_stmttmp16->f1;_tmp105=_stmttmp16->f2;_LL25: {struct Cyc_Absyn_Vardecl**vdopt=_tmp104;struct Cyc_Absyn_Exp*eopt=_tmp105;
# 733
if(vdopt != 0 &&*vdopt == vd){
if(eopt == 0)
({void*_tmp106=0U;({unsigned _tmp435=p->loc;struct _fat_ptr _tmp434=({const char*_tmp107="cannot alias pattern expression in datatype";_tag_fat(_tmp107,sizeof(char),44U);});Cyc_Tcutil_terr(_tmp435,_tmp434,_tag_fat(_tmp106,sizeof(void*),0U));});});else{
# 737
struct Cyc_Tcenv_Tenv*te2=({unsigned _tmp437=p->loc;struct Cyc_Tcenv_Tenv*_tmp436=te;Cyc_Tcenv_add_type_vars(_tmp437,_tmp436,({struct Cyc_Absyn_Tvar*_tmp109[1U];_tmp109[0]=tv;Cyc_List_list(_tag_fat(_tmp109,sizeof(struct Cyc_Absyn_Tvar*),1U));}));});
te2=({struct Cyc_Tcenv_Tenv*_tmp438=te2;Cyc_Tcenv_add_region(_tmp438,(void*)({struct Cyc_Absyn_VarType_Absyn_Type_struct*_tmp108=_cycalloc(sizeof(*_tmp108));_tmp108->tag=2U,_tmp108->f1=tv;_tmp108;}),0);});
# 740
Cyc_Tcpat_check_alias_coercion(te2,p->loc,(void*)_check_null(eopt->topt),tv,vd->type,eopt);}
# 743
break;}}}}
# 746
goto _LL0;}default: _LLF: _LL10:
 return;}_LL0:;}
# 45 "tcpat.cyc"
void Cyc_Tcpat_check_pat_regions(struct Cyc_Tcenv_Tenv*te,struct Cyc_Absyn_Pat*p,struct Cyc_List_List*patvars){
# 764 "tcpat.cyc"
Cyc_Tcpat_check_pat_regions_rec(te,p,0,patvars);{
struct Cyc_List_List*x=patvars;for(0;x != 0;x=x->tl){
struct _tuple16*_stmttmp17=(struct _tuple16*)x->hd;struct Cyc_Absyn_Exp*_tmp10B;struct Cyc_Absyn_Vardecl**_tmp10A;_LL1: _tmp10A=_stmttmp17->f1;_tmp10B=_stmttmp17->f2;_LL2: {struct Cyc_Absyn_Vardecl**vdopt=_tmp10A;struct Cyc_Absyn_Exp*eopt=_tmp10B;
if(eopt != 0){
struct Cyc_Absyn_Exp*e=eopt;
# 771
if(Cyc_Tcutil_is_noalias_pointer_or_aggr((void*)_check_null(e->topt))&& !Cyc_Tcutil_is_noalias_path(e))
# 773
({struct Cyc_String_pa_PrintArg_struct _tmp10E=({struct Cyc_String_pa_PrintArg_struct _tmp337;_tmp337.tag=0U,({
# 775
struct _fat_ptr _tmp43B=(struct _fat_ptr)(vdopt != 0?(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp111=({struct Cyc_String_pa_PrintArg_struct _tmp338;_tmp338.tag=0U,({
# 777
struct _fat_ptr _tmp439=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string((*vdopt)->name));_tmp338.f1=_tmp439;});_tmp338;});void*_tmp10F[1U];_tmp10F[0]=& _tmp111;({struct _fat_ptr _tmp43A=({const char*_tmp110="for variable %s";_tag_fat(_tmp110,sizeof(char),16U);});Cyc_aprintf(_tmp43A,_tag_fat(_tmp10F,sizeof(void*),1U));});}):({const char*_tmp112="";_tag_fat(_tmp112,sizeof(char),1U);}));
# 775
_tmp337.f1=_tmp43B;});_tmp337;});void*_tmp10C[1U];_tmp10C[0]=& _tmp10E;({unsigned _tmp43D=p->loc;struct _fat_ptr _tmp43C=({const char*_tmp10D="pattern %s dereferences a alias-free pointer from a non-unique path";_tag_fat(_tmp10D,sizeof(char),68U);});Cyc_Tcutil_terr(_tmp43D,_tmp43C,_tag_fat(_tmp10C,sizeof(void*),1U));});});}}}}}
# 831 "tcpat.cyc"
struct Cyc_Tcpat_EqNull_Tcpat_PatTest_struct Cyc_Tcpat_EqNull_val={1U};
struct Cyc_Tcpat_NeqNull_Tcpat_PatTest_struct Cyc_Tcpat_NeqNull_val={2U};
# 841
struct Cyc_Tcpat_Dummy_Tcpat_Access_struct Cyc_Tcpat_Dummy_val={0U};
struct Cyc_Tcpat_Deref_Tcpat_Access_struct Cyc_Tcpat_Deref_val={1U};struct _union_Name_value_Name_v{int tag;struct _fat_ptr val;};struct _union_Name_value_Int_v{int tag;int val;};union Cyc_Tcpat_Name_value{struct _union_Name_value_Name_v Name_v;struct _union_Name_value_Int_v Int_v;};
# 855
union Cyc_Tcpat_Name_value Cyc_Tcpat_Name_v(struct _fat_ptr s){return({union Cyc_Tcpat_Name_value _tmp339;(_tmp339.Name_v).tag=1U,(_tmp339.Name_v).val=s;_tmp339;});}union Cyc_Tcpat_Name_value Cyc_Tcpat_Int_v(int i){
return({union Cyc_Tcpat_Name_value _tmp33A;(_tmp33A.Int_v).tag=2U,(_tmp33A.Int_v).val=i;_tmp33A;});}struct Cyc_Tcpat_Con_s{union Cyc_Tcpat_Name_value name;int arity;int*span;union Cyc_Tcpat_PatOrWhere orig_pat;};struct Cyc_Tcpat_Any_Tcpat_Simple_pat_struct{int tag;};struct Cyc_Tcpat_Con_Tcpat_Simple_pat_struct{int tag;struct Cyc_Tcpat_Con_s*f1;struct Cyc_List_List*f2;};
# 872
static int Cyc_Tcpat_compare_con(struct Cyc_Tcpat_Con_s*c1,struct Cyc_Tcpat_Con_s*c2){
union Cyc_Tcpat_Name_value _stmttmp18=c1->name;union Cyc_Tcpat_Name_value _tmp113=_stmttmp18;int _tmp114;struct _fat_ptr _tmp115;if((_tmp113.Name_v).tag == 1){_LL1: _tmp115=(_tmp113.Name_v).val;_LL2: {struct _fat_ptr n1=_tmp115;
# 875
union Cyc_Tcpat_Name_value _stmttmp19=c2->name;union Cyc_Tcpat_Name_value _tmp116=_stmttmp19;struct _fat_ptr _tmp117;if((_tmp116.Name_v).tag == 1){_LL6: _tmp117=(_tmp116.Name_v).val;_LL7: {struct _fat_ptr n2=_tmp117;
return Cyc_strcmp((struct _fat_ptr)n1,(struct _fat_ptr)n2);}}else{_LL8: _LL9:
 return - 1;}_LL5:;}}else{_LL3: _tmp114=(_tmp113.Int_v).val;_LL4: {int i1=_tmp114;
# 880
union Cyc_Tcpat_Name_value _stmttmp1A=c2->name;union Cyc_Tcpat_Name_value _tmp118=_stmttmp1A;int _tmp119;if((_tmp118.Name_v).tag == 1){_LLB: _LLC:
 return 1;}else{_LLD: _tmp119=(_tmp118.Int_v).val;_LLE: {int i2=_tmp119;
return i1 - i2;}}_LLA:;}}_LL0:;}
# 872
static struct Cyc_Set_Set*Cyc_Tcpat_empty_con_set(){
# 889
return({struct Cyc_Set_Set*(*_tmp43E)(struct _RegionHandle*r,int(*cmp)(struct Cyc_Tcpat_Con_s*,struct Cyc_Tcpat_Con_s*))=({struct Cyc_Set_Set*(*_tmp11A)(struct _RegionHandle*r,int(*cmp)(struct Cyc_Tcpat_Con_s*,struct Cyc_Tcpat_Con_s*))=(struct Cyc_Set_Set*(*)(struct _RegionHandle*r,int(*cmp)(struct Cyc_Tcpat_Con_s*,struct Cyc_Tcpat_Con_s*)))Cyc_Set_rempty;_tmp11A;});_tmp43E(Cyc_Core_heap_region,Cyc_Tcpat_compare_con);});}
# 872
static int Cyc_Tcpat_one_opt=1;
# 893
static int Cyc_Tcpat_two_opt=2;
static int Cyc_Tcpat_twofiftysix_opt=256;
# 896
static unsigned Cyc_Tcpat_datatype_tag_number(struct Cyc_Absyn_Datatypedecl*td,struct _tuple2*name){
unsigned ans=0U;
struct Cyc_List_List*fs=(struct Cyc_List_List*)((struct Cyc_Core_Opt*)_check_null(td->fields))->v;
while(Cyc_Absyn_qvar_cmp(name,((struct Cyc_Absyn_Datatypefield*)((struct Cyc_List_List*)_check_null(fs))->hd)->name)!= 0){
++ ans;
fs=fs->tl;}
# 903
return ans;}
# 896
static int Cyc_Tcpat_get_member_offset(struct Cyc_Absyn_Aggrdecl*ad,struct _fat_ptr*f){
# 907
int i=1;
{struct Cyc_List_List*fields=((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->fields;for(0;fields != 0;fields=fields->tl){
struct Cyc_Absyn_Aggrfield*field=(struct Cyc_Absyn_Aggrfield*)fields->hd;
if(Cyc_strcmp((struct _fat_ptr)*field->name,(struct _fat_ptr)*f)== 0)return i;++ i;}}
# 913
({void*_tmp11B=0U;({int(*_tmp441)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp11F)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp11F;});struct _fat_ptr _tmp440=(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp11E=({struct Cyc_String_pa_PrintArg_struct _tmp33B;_tmp33B.tag=0U,_tmp33B.f1=(struct _fat_ptr)((struct _fat_ptr)*f);_tmp33B;});void*_tmp11C[1U];_tmp11C[0]=& _tmp11E;({struct _fat_ptr _tmp43F=({const char*_tmp11D="get_member_offset %s failed";_tag_fat(_tmp11D,sizeof(char),28U);});Cyc_aprintf(_tmp43F,_tag_fat(_tmp11C,sizeof(void*),1U));});});_tmp441(_tmp440,_tag_fat(_tmp11B,sizeof(void*),0U));});});}
# 896
static void*Cyc_Tcpat_get_pat_test(union Cyc_Tcpat_PatOrWhere pw){
# 917
union Cyc_Tcpat_PatOrWhere _tmp120=pw;struct Cyc_Absyn_Pat*_tmp121;struct Cyc_Absyn_Exp*_tmp122;if((_tmp120.where_clause).tag == 2){_LL1: _tmp122=(_tmp120.where_clause).val;_LL2: {struct Cyc_Absyn_Exp*e=_tmp122;
return(void*)({struct Cyc_Tcpat_WhereTest_Tcpat_PatTest_struct*_tmp123=_cycalloc(sizeof(*_tmp123));_tmp123->tag=0U,_tmp123->f1=e;_tmp123;});}}else{_LL3: _tmp121=(_tmp120.pattern).val;_LL4: {struct Cyc_Absyn_Pat*p=_tmp121;
# 920
void*_stmttmp1B=p->r;void*_tmp124=_stmttmp1B;struct Cyc_List_List*_tmp126;union Cyc_Absyn_AggrInfo _tmp125;struct Cyc_Absyn_Datatypefield*_tmp128;struct Cyc_Absyn_Datatypedecl*_tmp127;struct Cyc_Absyn_Enumfield*_tmp12A;void*_tmp129;struct Cyc_Absyn_Enumfield*_tmp12C;struct Cyc_Absyn_Enumdecl*_tmp12B;int _tmp12E;struct _fat_ptr _tmp12D;char _tmp12F;int _tmp131;enum Cyc_Absyn_Sign _tmp130;struct Cyc_Absyn_Pat*_tmp132;struct Cyc_Absyn_Pat*_tmp133;switch(*((int*)_tmp124)){case 1U: _LL6: _tmp133=((struct Cyc_Absyn_Var_p_Absyn_Raw_pat_struct*)_tmp124)->f2;_LL7: {struct Cyc_Absyn_Pat*p1=_tmp133;
_tmp132=p1;goto _LL9;}case 3U: _LL8: _tmp132=((struct Cyc_Absyn_Reference_p_Absyn_Raw_pat_struct*)_tmp124)->f2;_LL9: {struct Cyc_Absyn_Pat*p1=_tmp132;
return Cyc_Tcpat_get_pat_test(({union Cyc_Tcpat_PatOrWhere _tmp33C;(_tmp33C.pattern).tag=1U,(_tmp33C.pattern).val=p1;_tmp33C;}));}case 9U: _LLA: _LLB:
 return(void*)& Cyc_Tcpat_EqNull_val;case 10U: _LLC: _tmp130=((struct Cyc_Absyn_Int_p_Absyn_Raw_pat_struct*)_tmp124)->f1;_tmp131=((struct Cyc_Absyn_Int_p_Absyn_Raw_pat_struct*)_tmp124)->f2;_LLD: {enum Cyc_Absyn_Sign s=_tmp130;int i=_tmp131;
return(void*)({struct Cyc_Tcpat_EqConst_Tcpat_PatTest_struct*_tmp134=_cycalloc(sizeof(*_tmp134));_tmp134->tag=6U,_tmp134->f1=(unsigned)i;_tmp134;});}case 11U: _LLE: _tmp12F=((struct Cyc_Absyn_Char_p_Absyn_Raw_pat_struct*)_tmp124)->f1;_LLF: {char c=_tmp12F;
return(void*)({struct Cyc_Tcpat_EqConst_Tcpat_PatTest_struct*_tmp135=_cycalloc(sizeof(*_tmp135));_tmp135->tag=6U,_tmp135->f1=(unsigned)c;_tmp135;});}case 12U: _LL10: _tmp12D=((struct Cyc_Absyn_Float_p_Absyn_Raw_pat_struct*)_tmp124)->f1;_tmp12E=((struct Cyc_Absyn_Float_p_Absyn_Raw_pat_struct*)_tmp124)->f2;_LL11: {struct _fat_ptr f=_tmp12D;int i=_tmp12E;
return(void*)({struct Cyc_Tcpat_EqFloat_Tcpat_PatTest_struct*_tmp136=_cycalloc(sizeof(*_tmp136));_tmp136->tag=5U,_tmp136->f1=f,_tmp136->f2=i;_tmp136;});}case 13U: _LL12: _tmp12B=((struct Cyc_Absyn_Enum_p_Absyn_Raw_pat_struct*)_tmp124)->f1;_tmp12C=((struct Cyc_Absyn_Enum_p_Absyn_Raw_pat_struct*)_tmp124)->f2;_LL13: {struct Cyc_Absyn_Enumdecl*ed=_tmp12B;struct Cyc_Absyn_Enumfield*ef=_tmp12C;
return(void*)({struct Cyc_Tcpat_EqEnum_Tcpat_PatTest_struct*_tmp137=_cycalloc(sizeof(*_tmp137));_tmp137->tag=3U,_tmp137->f1=ed,_tmp137->f2=ef;_tmp137;});}case 14U: _LL14: _tmp129=(void*)((struct Cyc_Absyn_AnonEnum_p_Absyn_Raw_pat_struct*)_tmp124)->f1;_tmp12A=((struct Cyc_Absyn_AnonEnum_p_Absyn_Raw_pat_struct*)_tmp124)->f2;_LL15: {void*t=_tmp129;struct Cyc_Absyn_Enumfield*ef=_tmp12A;
return(void*)({struct Cyc_Tcpat_EqAnonEnum_Tcpat_PatTest_struct*_tmp138=_cycalloc(sizeof(*_tmp138));_tmp138->tag=4U,_tmp138->f1=t,_tmp138->f2=ef;_tmp138;});}case 6U: _LL16: _LL17:
# 930
{void*_stmttmp1C=Cyc_Tcutil_compress((void*)_check_null(p->topt));void*_tmp139=_stmttmp1C;void*_tmp13A;if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp139)->tag == 3U){_LL1F: _tmp13A=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp139)->f1).ptr_atts).nullable;_LL20: {void*n=_tmp13A;
# 932
if(Cyc_Tcutil_force_type2bool(0,n))
return(void*)& Cyc_Tcpat_NeqNull_val;
# 932
goto _LL1E;}}else{_LL21: _LL22:
# 935
 goto _LL1E;}_LL1E:;}
# 937
({void*_tmp13B=0U;({int(*_tmp443)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp13D)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp13D;});struct _fat_ptr _tmp442=({const char*_tmp13C="non-null pointer type or non-pointer type in pointer pattern";_tag_fat(_tmp13C,sizeof(char),61U);});_tmp443(_tmp442,_tag_fat(_tmp13B,sizeof(void*),0U));});});case 8U: _LL18: _tmp127=((struct Cyc_Absyn_Datatype_p_Absyn_Raw_pat_struct*)_tmp124)->f1;_tmp128=((struct Cyc_Absyn_Datatype_p_Absyn_Raw_pat_struct*)_tmp124)->f2;_LL19: {struct Cyc_Absyn_Datatypedecl*ddecl=_tmp127;struct Cyc_Absyn_Datatypefield*df=_tmp128;
# 939
if(ddecl->is_extensible)
return(void*)({struct Cyc_Tcpat_EqExtensibleDatatype_Tcpat_PatTest_struct*_tmp13E=_cycalloc(sizeof(*_tmp13E));_tmp13E->tag=9U,_tmp13E->f1=ddecl,_tmp13E->f2=df;_tmp13E;});else{
# 942
return(void*)({struct Cyc_Tcpat_EqDatatypeTag_Tcpat_PatTest_struct*_tmp13F=_cycalloc(sizeof(*_tmp13F));_tmp13F->tag=7U,({int _tmp444=(int)Cyc_Tcpat_datatype_tag_number(ddecl,df->name);_tmp13F->f1=_tmp444;}),_tmp13F->f2=ddecl,_tmp13F->f3=df;_tmp13F;});}}case 7U: if(((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmp124)->f1 != 0){_LL1A: _tmp125=*((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmp124)->f1;_tmp126=((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmp124)->f3;_LL1B: {union Cyc_Absyn_AggrInfo info=_tmp125;struct Cyc_List_List*dlps=_tmp126;
# 944
struct Cyc_Absyn_Aggrdecl*ad=Cyc_Absyn_get_known_aggrdecl(info);
if(((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->tagged){
struct _tuple19*_stmttmp1D=(struct _tuple19*)((struct Cyc_List_List*)_check_null(dlps))->hd;struct Cyc_Absyn_Pat*_tmp141;struct Cyc_List_List*_tmp140;_LL24: _tmp140=_stmttmp1D->f1;_tmp141=_stmttmp1D->f2;_LL25: {struct Cyc_List_List*designators=_tmp140;struct Cyc_Absyn_Pat*pat=_tmp141;
struct _fat_ptr*f;
{void*_stmttmp1E=(void*)((struct Cyc_List_List*)_check_null(designators))->hd;void*_tmp142=_stmttmp1E;struct _fat_ptr*_tmp143;if(((struct Cyc_Absyn_FieldName_Absyn_Designator_struct*)_tmp142)->tag == 1U){_LL27: _tmp143=((struct Cyc_Absyn_FieldName_Absyn_Designator_struct*)_tmp142)->f1;_LL28: {struct _fat_ptr*fn=_tmp143;
f=fn;goto _LL26;}}else{_LL29: _LL2A:
({void*_tmp144=0U;({int(*_tmp446)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp146)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp146;});struct _fat_ptr _tmp445=({const char*_tmp145="no field name in tagged union pattern";_tag_fat(_tmp145,sizeof(char),38U);});_tmp446(_tmp445,_tag_fat(_tmp144,sizeof(void*),0U));});});}_LL26:;}
# 952
return(void*)({struct Cyc_Tcpat_EqTaggedUnion_Tcpat_PatTest_struct*_tmp147=_cycalloc(sizeof(*_tmp147));_tmp147->tag=8U,_tmp147->f1=f,({int _tmp447=Cyc_Tcpat_get_member_offset(ad,f);_tmp147->f2=_tmp447;});_tmp147;});}}else{
# 954
({void*_tmp148=0U;({int(*_tmp449)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp14A)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp14A;});struct _fat_ptr _tmp448=({const char*_tmp149="non-tagged aggregate in pattern test";_tag_fat(_tmp149,sizeof(char),37U);});_tmp449(_tmp448,_tag_fat(_tmp148,sizeof(void*),0U));});});}}}else{goto _LL1C;}default: _LL1C: _LL1D:
({void*_tmp14B=0U;({int(*_tmp44B)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp14D)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp14D;});struct _fat_ptr _tmp44A=({const char*_tmp14C="non-test pattern in pattern test";_tag_fat(_tmp14C,sizeof(char),33U);});_tmp44B(_tmp44A,_tag_fat(_tmp14B,sizeof(void*),0U));});});}_LL5:;}}_LL0:;}
# 896
static union Cyc_Tcpat_PatOrWhere Cyc_Tcpat_pw(struct Cyc_Absyn_Pat*p){
# 961
return({union Cyc_Tcpat_PatOrWhere _tmp33D;(_tmp33D.pattern).tag=1U,(_tmp33D.pattern).val=p;_tmp33D;});}
# 896
static struct Cyc_Tcpat_Con_s*Cyc_Tcpat_null_con(struct Cyc_Absyn_Pat*p){
# 965
return({struct Cyc_Tcpat_Con_s*_tmp14F=_cycalloc(sizeof(*_tmp14F));({union Cyc_Tcpat_Name_value _tmp44D=Cyc_Tcpat_Name_v(({const char*_tmp14E="NULL";_tag_fat(_tmp14E,sizeof(char),5U);}));_tmp14F->name=_tmp44D;}),_tmp14F->arity=0,_tmp14F->span=& Cyc_Tcpat_two_opt,({union Cyc_Tcpat_PatOrWhere _tmp44C=Cyc_Tcpat_pw(p);_tmp14F->orig_pat=_tmp44C;});_tmp14F;});}
# 896
static struct Cyc_Tcpat_Con_s*Cyc_Tcpat_null_ptr_con(struct Cyc_Absyn_Pat*p){
# 968
return({struct Cyc_Tcpat_Con_s*_tmp151=_cycalloc(sizeof(*_tmp151));({union Cyc_Tcpat_Name_value _tmp44F=Cyc_Tcpat_Name_v(({const char*_tmp150="&";_tag_fat(_tmp150,sizeof(char),2U);}));_tmp151->name=_tmp44F;}),_tmp151->arity=1,_tmp151->span=& Cyc_Tcpat_two_opt,({union Cyc_Tcpat_PatOrWhere _tmp44E=Cyc_Tcpat_pw(p);_tmp151->orig_pat=_tmp44E;});_tmp151;});}
# 896
static struct Cyc_Tcpat_Con_s*Cyc_Tcpat_ptr_con(struct Cyc_Absyn_Pat*p){
# 971
return({struct Cyc_Tcpat_Con_s*_tmp153=_cycalloc(sizeof(*_tmp153));({union Cyc_Tcpat_Name_value _tmp451=Cyc_Tcpat_Name_v(({const char*_tmp152="&";_tag_fat(_tmp152,sizeof(char),2U);}));_tmp153->name=_tmp451;}),_tmp153->arity=1,_tmp153->span=& Cyc_Tcpat_one_opt,({union Cyc_Tcpat_PatOrWhere _tmp450=Cyc_Tcpat_pw(p);_tmp153->orig_pat=_tmp450;});_tmp153;});}
# 896
static struct Cyc_Tcpat_Con_s*Cyc_Tcpat_int_con(int i,union Cyc_Tcpat_PatOrWhere p){
# 974
return({struct Cyc_Tcpat_Con_s*_tmp154=_cycalloc(sizeof(*_tmp154));({union Cyc_Tcpat_Name_value _tmp452=Cyc_Tcpat_Int_v(i);_tmp154->name=_tmp452;}),_tmp154->arity=0,_tmp154->span=0,_tmp154->orig_pat=p;_tmp154;});}
# 896
static struct Cyc_Tcpat_Con_s*Cyc_Tcpat_float_con(struct _fat_ptr f,struct Cyc_Absyn_Pat*p){
# 977
return({struct Cyc_Tcpat_Con_s*_tmp155=_cycalloc(sizeof(*_tmp155));({union Cyc_Tcpat_Name_value _tmp454=Cyc_Tcpat_Name_v(f);_tmp155->name=_tmp454;}),_tmp155->arity=0,_tmp155->span=0,({union Cyc_Tcpat_PatOrWhere _tmp453=Cyc_Tcpat_pw(p);_tmp155->orig_pat=_tmp453;});_tmp155;});}
# 896
static struct Cyc_Tcpat_Con_s*Cyc_Tcpat_char_con(char c,struct Cyc_Absyn_Pat*p){
# 980
return({struct Cyc_Tcpat_Con_s*_tmp156=_cycalloc(sizeof(*_tmp156));({union Cyc_Tcpat_Name_value _tmp456=Cyc_Tcpat_Int_v((int)c);_tmp156->name=_tmp456;}),_tmp156->arity=0,_tmp156->span=& Cyc_Tcpat_twofiftysix_opt,({union Cyc_Tcpat_PatOrWhere _tmp455=Cyc_Tcpat_pw(p);_tmp156->orig_pat=_tmp455;});_tmp156;});}
# 896
static struct Cyc_Tcpat_Con_s*Cyc_Tcpat_tuple_con(int i,union Cyc_Tcpat_PatOrWhere p){
# 983
return({struct Cyc_Tcpat_Con_s*_tmp158=_cycalloc(sizeof(*_tmp158));({union Cyc_Tcpat_Name_value _tmp457=Cyc_Tcpat_Name_v(({const char*_tmp157="$";_tag_fat(_tmp157,sizeof(char),2U);}));_tmp158->name=_tmp457;}),_tmp158->arity=i,_tmp158->span=& Cyc_Tcpat_one_opt,_tmp158->orig_pat=p;_tmp158;});}
# 896
static void*Cyc_Tcpat_null_pat(struct Cyc_Absyn_Pat*p){
# 988
return(void*)({struct Cyc_Tcpat_Con_Tcpat_Simple_pat_struct*_tmp159=_cycalloc(sizeof(*_tmp159));_tmp159->tag=1U,({struct Cyc_Tcpat_Con_s*_tmp458=Cyc_Tcpat_null_con(p);_tmp159->f1=_tmp458;}),_tmp159->f2=0;_tmp159;});}
# 896
static void*Cyc_Tcpat_int_pat(int i,union Cyc_Tcpat_PatOrWhere p){
# 991
return(void*)({struct Cyc_Tcpat_Con_Tcpat_Simple_pat_struct*_tmp15A=_cycalloc(sizeof(*_tmp15A));_tmp15A->tag=1U,({struct Cyc_Tcpat_Con_s*_tmp459=Cyc_Tcpat_int_con(i,p);_tmp15A->f1=_tmp459;}),_tmp15A->f2=0;_tmp15A;});}
# 896
static void*Cyc_Tcpat_char_pat(char c,struct Cyc_Absyn_Pat*p){
# 994
return(void*)({struct Cyc_Tcpat_Con_Tcpat_Simple_pat_struct*_tmp15B=_cycalloc(sizeof(*_tmp15B));_tmp15B->tag=1U,({struct Cyc_Tcpat_Con_s*_tmp45A=Cyc_Tcpat_char_con(c,p);_tmp15B->f1=_tmp45A;}),_tmp15B->f2=0;_tmp15B;});}
# 896
static void*Cyc_Tcpat_float_pat(struct _fat_ptr f,struct Cyc_Absyn_Pat*p){
# 997
return(void*)({struct Cyc_Tcpat_Con_Tcpat_Simple_pat_struct*_tmp15C=_cycalloc(sizeof(*_tmp15C));_tmp15C->tag=1U,({struct Cyc_Tcpat_Con_s*_tmp45B=Cyc_Tcpat_float_con(f,p);_tmp15C->f1=_tmp45B;}),_tmp15C->f2=0;_tmp15C;});}
# 896
static void*Cyc_Tcpat_null_ptr_pat(void*p,struct Cyc_Absyn_Pat*p0){
# 1000
return(void*)({struct Cyc_Tcpat_Con_Tcpat_Simple_pat_struct*_tmp15E=_cycalloc(sizeof(*_tmp15E));_tmp15E->tag=1U,({struct Cyc_Tcpat_Con_s*_tmp45D=Cyc_Tcpat_null_ptr_con(p0);_tmp15E->f1=_tmp45D;}),({struct Cyc_List_List*_tmp45C=({struct Cyc_List_List*_tmp15D=_cycalloc(sizeof(*_tmp15D));_tmp15D->hd=p,_tmp15D->tl=0;_tmp15D;});_tmp15E->f2=_tmp45C;});_tmp15E;});}
# 896
static void*Cyc_Tcpat_ptr_pat(void*p,struct Cyc_Absyn_Pat*p0){
# 1003
return(void*)({struct Cyc_Tcpat_Con_Tcpat_Simple_pat_struct*_tmp160=_cycalloc(sizeof(*_tmp160));_tmp160->tag=1U,({struct Cyc_Tcpat_Con_s*_tmp45F=Cyc_Tcpat_ptr_con(p0);_tmp160->f1=_tmp45F;}),({struct Cyc_List_List*_tmp45E=({struct Cyc_List_List*_tmp15F=_cycalloc(sizeof(*_tmp15F));_tmp15F->hd=p,_tmp15F->tl=0;_tmp15F;});_tmp160->f2=_tmp45E;});_tmp160;});}
# 896
static void*Cyc_Tcpat_tuple_pat(struct Cyc_List_List*ss,union Cyc_Tcpat_PatOrWhere p){
# 1006
return(void*)({struct Cyc_Tcpat_Con_Tcpat_Simple_pat_struct*_tmp161=_cycalloc(sizeof(*_tmp161));_tmp161->tag=1U,({struct Cyc_Tcpat_Con_s*_tmp461=({int _tmp460=Cyc_List_length(ss);Cyc_Tcpat_tuple_con(_tmp460,p);});_tmp161->f1=_tmp461;}),_tmp161->f2=ss;_tmp161;});}
# 896
static void*Cyc_Tcpat_con_pat(struct _fat_ptr con_name,int*span,struct Cyc_List_List*ps,struct Cyc_Absyn_Pat*p){
# 1010
struct Cyc_Tcpat_Con_s*c=({struct Cyc_Tcpat_Con_s*_tmp163=_cycalloc(sizeof(*_tmp163));({union Cyc_Tcpat_Name_value _tmp464=Cyc_Tcpat_Name_v(con_name);_tmp163->name=_tmp464;}),({int _tmp463=Cyc_List_length(ps);_tmp163->arity=_tmp463;}),_tmp163->span=span,({union Cyc_Tcpat_PatOrWhere _tmp462=Cyc_Tcpat_pw(p);_tmp163->orig_pat=_tmp462;});_tmp163;});
return(void*)({struct Cyc_Tcpat_Con_Tcpat_Simple_pat_struct*_tmp162=_cycalloc(sizeof(*_tmp162));_tmp162->tag=1U,_tmp162->f1=c,_tmp162->f2=ps;_tmp162;});}
# 896
static void*Cyc_Tcpat_compile_pat(struct Cyc_Absyn_Pat*p){
# 1016
void*s;
{void*_stmttmp1F=p->r;void*_tmp164=_stmttmp1F;struct Cyc_Absyn_Enumfield*_tmp166;void*_tmp165;struct Cyc_Absyn_Enumfield*_tmp168;struct Cyc_Absyn_Enumdecl*_tmp167;struct Cyc_List_List*_tmp16A;struct Cyc_Absyn_Aggrdecl*_tmp169;struct Cyc_List_List*_tmp16B;struct Cyc_List_List*_tmp16E;struct Cyc_Absyn_Datatypefield*_tmp16D;struct Cyc_Absyn_Datatypedecl*_tmp16C;struct Cyc_Absyn_Pat*_tmp16F;struct Cyc_Absyn_Pat*_tmp170;struct Cyc_Absyn_Pat*_tmp171;struct _fat_ptr _tmp172;char _tmp173;int _tmp175;enum Cyc_Absyn_Sign _tmp174;switch(*((int*)_tmp164)){case 0U: _LL1: _LL2:
 goto _LL4;case 2U: _LL3: _LL4:
 goto _LL6;case 4U: _LL5: _LL6:
 s=(void*)({struct Cyc_Tcpat_Any_Tcpat_Simple_pat_struct*_tmp176=_cycalloc(sizeof(*_tmp176));_tmp176->tag=0U;_tmp176;});goto _LL0;case 9U: _LL7: _LL8:
 s=Cyc_Tcpat_null_pat(p);goto _LL0;case 10U: _LL9: _tmp174=((struct Cyc_Absyn_Int_p_Absyn_Raw_pat_struct*)_tmp164)->f1;_tmp175=((struct Cyc_Absyn_Int_p_Absyn_Raw_pat_struct*)_tmp164)->f2;_LLA: {enum Cyc_Absyn_Sign sn=_tmp174;int i=_tmp175;
s=({int _tmp465=i;Cyc_Tcpat_int_pat(_tmp465,Cyc_Tcpat_pw(p));});goto _LL0;}case 11U: _LLB: _tmp173=((struct Cyc_Absyn_Char_p_Absyn_Raw_pat_struct*)_tmp164)->f1;_LLC: {char c=_tmp173;
s=Cyc_Tcpat_char_pat(c,p);goto _LL0;}case 12U: _LLD: _tmp172=((struct Cyc_Absyn_Float_p_Absyn_Raw_pat_struct*)_tmp164)->f1;_LLE: {struct _fat_ptr f=_tmp172;
s=Cyc_Tcpat_float_pat(f,p);goto _LL0;}case 1U: _LLF: _tmp171=((struct Cyc_Absyn_Var_p_Absyn_Raw_pat_struct*)_tmp164)->f2;_LL10: {struct Cyc_Absyn_Pat*p2=_tmp171;
s=Cyc_Tcpat_compile_pat(p2);goto _LL0;}case 3U: _LL11: _tmp170=((struct Cyc_Absyn_Reference_p_Absyn_Raw_pat_struct*)_tmp164)->f2;_LL12: {struct Cyc_Absyn_Pat*p2=_tmp170;
s=Cyc_Tcpat_compile_pat(p2);goto _LL0;}case 6U: _LL13: _tmp16F=((struct Cyc_Absyn_Pointer_p_Absyn_Raw_pat_struct*)_tmp164)->f1;_LL14: {struct Cyc_Absyn_Pat*pp=_tmp16F;
# 1028
{void*_stmttmp20=Cyc_Tcutil_compress((void*)_check_null(p->topt));void*_tmp177=_stmttmp20;void*_tmp178;if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp177)->tag == 3U){_LL28: _tmp178=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp177)->f1).ptr_atts).nullable;_LL29: {void*n=_tmp178;
# 1030
int is_nullable=Cyc_Tcutil_force_type2bool(0,n);
void*ss=Cyc_Tcpat_compile_pat(pp);
if(is_nullable)
s=Cyc_Tcpat_null_ptr_pat(ss,p);else{
# 1035
s=Cyc_Tcpat_ptr_pat(ss,p);}
goto _LL27;}}else{_LL2A: _LL2B:
({void*_tmp179=0U;({int(*_tmp467)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp17B)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp17B;});struct _fat_ptr _tmp466=({const char*_tmp17A="expecting pointertype for pattern!";_tag_fat(_tmp17A,sizeof(char),35U);});_tmp467(_tmp466,_tag_fat(_tmp179,sizeof(void*),0U));});});}_LL27:;}
# 1039
goto _LL0;}case 8U: _LL15: _tmp16C=((struct Cyc_Absyn_Datatype_p_Absyn_Raw_pat_struct*)_tmp164)->f1;_tmp16D=((struct Cyc_Absyn_Datatype_p_Absyn_Raw_pat_struct*)_tmp164)->f2;_tmp16E=((struct Cyc_Absyn_Datatype_p_Absyn_Raw_pat_struct*)_tmp164)->f3;_LL16: {struct Cyc_Absyn_Datatypedecl*tud=_tmp16C;struct Cyc_Absyn_Datatypefield*tuf=_tmp16D;struct Cyc_List_List*ps=_tmp16E;
# 1041
int*span;
{void*_stmttmp21=Cyc_Tcutil_compress((void*)_check_null(p->topt));void*_tmp17C=_stmttmp21;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp17C)->tag == 0U)switch(*((int*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp17C)->f1)){case 18U: _LL2D: _LL2E:
# 1044
 if(tud->is_extensible)
span=0;else{
# 1047
span=({int*_tmp17D=_cycalloc_atomic(sizeof(*_tmp17D));({int _tmp468=Cyc_List_length((struct Cyc_List_List*)((struct Cyc_Core_Opt*)_check_null(tud->fields))->v);*_tmp17D=_tmp468;});_tmp17D;});}
goto _LL2C;case 19U: _LL2F: _LL30:
 span=& Cyc_Tcpat_one_opt;goto _LL2C;default: goto _LL31;}else{_LL31: _LL32:
 span=({void*_tmp17E=0U;({int*(*_tmp46A)(struct _fat_ptr,struct _fat_ptr)=({int*(*_tmp180)(struct _fat_ptr,struct _fat_ptr)=(int*(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp180;});struct _fat_ptr _tmp469=({const char*_tmp17F="void datatype pattern has bad type";_tag_fat(_tmp17F,sizeof(char),35U);});_tmp46A(_tmp469,_tag_fat(_tmp17E,sizeof(void*),0U));});});goto _LL2C;}_LL2C:;}
# 1052
s=({struct _fat_ptr _tmp46E=*(*tuf->name).f2;int*_tmp46D=span;struct Cyc_List_List*_tmp46C=({struct Cyc_List_List*(*_tmp46B)(void*(*f)(struct Cyc_Absyn_Pat*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp181)(void*(*f)(struct Cyc_Absyn_Pat*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(void*(*f)(struct Cyc_Absyn_Pat*),struct Cyc_List_List*x))Cyc_List_map;_tmp181;});_tmp46B(Cyc_Tcpat_compile_pat,ps);});Cyc_Tcpat_con_pat(_tmp46E,_tmp46D,_tmp46C,p);});
goto _LL0;}case 5U: _LL17: _tmp16B=((struct Cyc_Absyn_Tuple_p_Absyn_Raw_pat_struct*)_tmp164)->f1;_LL18: {struct Cyc_List_List*ps=_tmp16B;
# 1056
s=({struct Cyc_List_List*_tmp470=({struct Cyc_List_List*(*_tmp46F)(void*(*f)(struct Cyc_Absyn_Pat*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp182)(void*(*f)(struct Cyc_Absyn_Pat*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(void*(*f)(struct Cyc_Absyn_Pat*),struct Cyc_List_List*x))Cyc_List_map;_tmp182;});_tmp46F(Cyc_Tcpat_compile_pat,ps);});Cyc_Tcpat_tuple_pat(_tmp470,Cyc_Tcpat_pw(p));});goto _LL0;}case 7U: if(((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmp164)->f1 != 0){if((((union Cyc_Absyn_AggrInfo*)((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmp164)->f1)->KnownAggr).tag == 2){_LL19: _tmp169=*((((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmp164)->f1)->KnownAggr).val;_tmp16A=((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmp164)->f3;_LL1A: {struct Cyc_Absyn_Aggrdecl*ad=_tmp169;struct Cyc_List_List*dlps=_tmp16A;
# 1061
if((int)ad->kind == (int)Cyc_Absyn_StructA){
struct Cyc_List_List*ps=0;
{struct Cyc_List_List*fields=((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->fields;for(0;fields != 0;fields=fields->tl){
# 1065
int found=({struct _fat_ptr _tmp471=(struct _fat_ptr)*((struct Cyc_Absyn_Aggrfield*)fields->hd)->name;Cyc_strcmp(_tmp471,({const char*_tmp18E="";_tag_fat(_tmp18E,sizeof(char),1U);}));})== 0;
{struct Cyc_List_List*dlps0=dlps;for(0;!found && dlps0 != 0;dlps0=dlps0->tl){
struct _tuple19*_stmttmp22=(struct _tuple19*)dlps0->hd;struct Cyc_Absyn_Pat*_tmp184;struct Cyc_List_List*_tmp183;_LL34: _tmp183=_stmttmp22->f1;_tmp184=_stmttmp22->f2;_LL35: {struct Cyc_List_List*dl=_tmp183;struct Cyc_Absyn_Pat*p=_tmp184;
struct Cyc_List_List*_tmp185=dl;struct _fat_ptr*_tmp186;if(_tmp185 != 0){if(((struct Cyc_Absyn_FieldName_Absyn_Designator_struct*)((struct Cyc_List_List*)_tmp185)->hd)->tag == 1U){if(((struct Cyc_List_List*)_tmp185)->tl == 0){_LL37: _tmp186=((struct Cyc_Absyn_FieldName_Absyn_Designator_struct*)_tmp185->hd)->f1;_LL38: {struct _fat_ptr*f=_tmp186;
# 1070
if(Cyc_strptrcmp(f,((struct Cyc_Absyn_Aggrfield*)fields->hd)->name)== 0){
ps=({struct Cyc_List_List*_tmp187=_cycalloc(sizeof(*_tmp187));({void*_tmp472=Cyc_Tcpat_compile_pat(p);_tmp187->hd=_tmp472;}),_tmp187->tl=ps;_tmp187;});
found=1;}
# 1070
goto _LL36;}}else{goto _LL39;}}else{goto _LL39;}}else{_LL39: _LL3A:
# 1075
({void*_tmp188=0U;({int(*_tmp474)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp18A)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp18A;});struct _fat_ptr _tmp473=({const char*_tmp189="bad designator(s)";_tag_fat(_tmp189,sizeof(char),18U);});_tmp474(_tmp473,_tag_fat(_tmp188,sizeof(void*),0U));});});}_LL36:;}}}
# 1078
if(!found)
({void*_tmp18B=0U;({int(*_tmp476)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp18D)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp18D;});struct _fat_ptr _tmp475=({const char*_tmp18C="bad designator";_tag_fat(_tmp18C,sizeof(char),15U);});_tmp476(_tmp475,_tag_fat(_tmp18B,sizeof(void*),0U));});});}}
# 1081
s=({struct Cyc_List_List*_tmp477=Cyc_List_imp_rev(ps);Cyc_Tcpat_tuple_pat(_tmp477,Cyc_Tcpat_pw(p));});}else{
# 1084
if(!((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->tagged)
({void*_tmp18F=0U;({unsigned _tmp479=p->loc;struct _fat_ptr _tmp478=({const char*_tmp190="patterns on untagged unions not yet supported.";_tag_fat(_tmp190,sizeof(char),47U);});Cyc_Tcutil_terr(_tmp479,_tmp478,_tag_fat(_tmp18F,sizeof(void*),0U));});});{
# 1084
int*span=({int*_tmp198=_cycalloc_atomic(sizeof(*_tmp198));({
# 1086
int _tmp47A=Cyc_List_length(((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->fields);*_tmp198=_tmp47A;});_tmp198;});
struct Cyc_List_List*_tmp191=dlps;struct Cyc_Absyn_Pat*_tmp193;struct _fat_ptr*_tmp192;if(_tmp191 != 0){if(((struct _tuple19*)((struct Cyc_List_List*)_tmp191)->hd)->f1 != 0){if(((struct Cyc_Absyn_FieldName_Absyn_Designator_struct*)((struct Cyc_List_List*)((struct _tuple19*)((struct Cyc_List_List*)_tmp191)->hd)->f1)->hd)->tag == 1U){if(((struct Cyc_List_List*)((struct _tuple19*)((struct Cyc_List_List*)_tmp191)->hd)->f1)->tl == 0){if(((struct Cyc_List_List*)_tmp191)->tl == 0){_LL3C: _tmp192=((struct Cyc_Absyn_FieldName_Absyn_Designator_struct*)(((struct _tuple19*)_tmp191->hd)->f1)->hd)->f1;_tmp193=((struct _tuple19*)_tmp191->hd)->f2;_LL3D: {struct _fat_ptr*f=_tmp192;struct Cyc_Absyn_Pat*p2=_tmp193;
# 1089
s=({struct _fat_ptr _tmp47E=*f;int*_tmp47D=span;struct Cyc_List_List*_tmp47C=({struct Cyc_List_List*_tmp194=_cycalloc(sizeof(*_tmp194));({void*_tmp47B=Cyc_Tcpat_compile_pat(p2);_tmp194->hd=_tmp47B;}),_tmp194->tl=0;_tmp194;});Cyc_Tcpat_con_pat(_tmp47E,_tmp47D,_tmp47C,p);});
goto _LL3B;}}else{goto _LL3E;}}else{goto _LL3E;}}else{goto _LL3E;}}else{goto _LL3E;}}else{_LL3E: _LL3F:
({void*_tmp195=0U;({int(*_tmp480)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp197)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp197;});struct _fat_ptr _tmp47F=({const char*_tmp196="bad union pattern";_tag_fat(_tmp196,sizeof(char),18U);});_tmp480(_tmp47F,_tag_fat(_tmp195,sizeof(void*),0U));});});}_LL3B:;}}
# 1094
goto _LL0;}}else{goto _LL23;}}else{_LL23: _LL24:
# 1122
 goto _LL26;}case 13U: _LL1B: _tmp167=((struct Cyc_Absyn_Enum_p_Absyn_Raw_pat_struct*)_tmp164)->f1;_tmp168=((struct Cyc_Absyn_Enum_p_Absyn_Raw_pat_struct*)_tmp164)->f2;_LL1C: {struct Cyc_Absyn_Enumdecl*ed=_tmp167;struct Cyc_Absyn_Enumfield*ef=_tmp168;
# 1102
s=Cyc_Tcpat_con_pat(*(*ef->name).f2,0,0,p);
goto _LL0;}case 14U: _LL1D: _tmp165=(void*)((struct Cyc_Absyn_AnonEnum_p_Absyn_Raw_pat_struct*)_tmp164)->f1;_tmp166=((struct Cyc_Absyn_AnonEnum_p_Absyn_Raw_pat_struct*)_tmp164)->f2;_LL1E: {void*tenum=_tmp165;struct Cyc_Absyn_Enumfield*ef=_tmp166;
# 1108
struct Cyc_List_List*fields;
{void*_stmttmp23=Cyc_Tcutil_compress(tenum);void*_tmp199=_stmttmp23;struct Cyc_List_List*_tmp19A;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp199)->tag == 0U){if(((struct Cyc_Absyn_AnonEnumCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp199)->f1)->tag == 16U){_LL41: _tmp19A=((struct Cyc_Absyn_AnonEnumCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp199)->f1)->f1;_LL42: {struct Cyc_List_List*fs=_tmp19A;
fields=fs;goto _LL40;}}else{goto _LL43;}}else{_LL43: _LL44:
({void*_tmp19B=0U;({int(*_tmp482)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp19D)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp19D;});struct _fat_ptr _tmp481=({const char*_tmp19C="bad type in AnonEnum_p";_tag_fat(_tmp19C,sizeof(char),23U);});_tmp482(_tmp481,_tag_fat(_tmp19B,sizeof(void*),0U));});});}_LL40:;}
# 1118
s=Cyc_Tcpat_con_pat(*(*ef->name).f2,0,0,p);
goto _LL0;}case 15U: _LL1F: _LL20:
 goto _LL22;case 16U: _LL21: _LL22:
 goto _LL24;default: _LL25: _LL26:
# 1123
 s=(void*)({struct Cyc_Tcpat_Any_Tcpat_Simple_pat_struct*_tmp19E=_cycalloc(sizeof(*_tmp19E));_tmp19E->tag=0U;_tmp19E;});}_LL0:;}
# 1125
return s;}struct Cyc_Tcpat_Pos_Tcpat_Term_desc_struct{int tag;struct Cyc_Tcpat_Con_s*f1;struct Cyc_List_List*f2;};struct Cyc_Tcpat_Neg_Tcpat_Term_desc_struct{int tag;struct Cyc_Set_Set*f1;};
# 1154
static int Cyc_Tcpat_same_access(void*a1,void*a2){
struct _tuple0 _stmttmp24=({struct _tuple0 _tmp33E;_tmp33E.f1=a1,_tmp33E.f2=a2;_tmp33E;});struct _tuple0 _tmp19F=_stmttmp24;struct _fat_ptr*_tmp1A3;int _tmp1A2;struct _fat_ptr*_tmp1A1;int _tmp1A0;unsigned _tmp1A7;struct Cyc_Absyn_Datatypefield*_tmp1A6;unsigned _tmp1A5;struct Cyc_Absyn_Datatypefield*_tmp1A4;unsigned _tmp1A9;unsigned _tmp1A8;switch(*((int*)_tmp19F.f1)){case 0U: if(((struct Cyc_Tcpat_Dummy_Tcpat_Access_struct*)_tmp19F.f2)->tag == 0U){_LL1: _LL2:
 return 1;}else{goto _LLB;}case 1U: if(((struct Cyc_Tcpat_Deref_Tcpat_Access_struct*)_tmp19F.f2)->tag == 1U){_LL3: _LL4:
 return 1;}else{goto _LLB;}case 2U: if(((struct Cyc_Tcpat_TupleField_Tcpat_Access_struct*)_tmp19F.f2)->tag == 2U){_LL5: _tmp1A8=((struct Cyc_Tcpat_TupleField_Tcpat_Access_struct*)_tmp19F.f1)->f1;_tmp1A9=((struct Cyc_Tcpat_TupleField_Tcpat_Access_struct*)_tmp19F.f2)->f1;_LL6: {unsigned i1=_tmp1A8;unsigned i2=_tmp1A9;
return i1 == i2;}}else{goto _LLB;}case 3U: if(((struct Cyc_Tcpat_DatatypeField_Tcpat_Access_struct*)_tmp19F.f2)->tag == 3U){_LL7: _tmp1A4=((struct Cyc_Tcpat_DatatypeField_Tcpat_Access_struct*)_tmp19F.f1)->f2;_tmp1A5=((struct Cyc_Tcpat_DatatypeField_Tcpat_Access_struct*)_tmp19F.f1)->f3;_tmp1A6=((struct Cyc_Tcpat_DatatypeField_Tcpat_Access_struct*)_tmp19F.f2)->f2;_tmp1A7=((struct Cyc_Tcpat_DatatypeField_Tcpat_Access_struct*)_tmp19F.f2)->f3;_LL8: {struct Cyc_Absyn_Datatypefield*df1=_tmp1A4;unsigned i1=_tmp1A5;struct Cyc_Absyn_Datatypefield*df2=_tmp1A6;unsigned i2=_tmp1A7;
# 1160
return df1 == df2 && i1 == i2;}}else{goto _LLB;}default: if(((struct Cyc_Tcpat_AggrField_Tcpat_Access_struct*)_tmp19F.f2)->tag == 4U){_LL9: _tmp1A0=((struct Cyc_Tcpat_AggrField_Tcpat_Access_struct*)_tmp19F.f1)->f1;_tmp1A1=((struct Cyc_Tcpat_AggrField_Tcpat_Access_struct*)_tmp19F.f1)->f2;_tmp1A2=((struct Cyc_Tcpat_AggrField_Tcpat_Access_struct*)_tmp19F.f2)->f1;_tmp1A3=((struct Cyc_Tcpat_AggrField_Tcpat_Access_struct*)_tmp19F.f2)->f2;_LLA: {int b1=_tmp1A0;struct _fat_ptr*f1=_tmp1A1;int b2=_tmp1A2;struct _fat_ptr*f2=_tmp1A3;
# 1162
return b1 == b2 && Cyc_strptrcmp(f1,f2)== 0;}}else{_LLB: _LLC:
 return 0;}}_LL0:;}
# 1154
static int Cyc_Tcpat_same_path(struct Cyc_List_List*p1,struct Cyc_List_List*p2){
# 1168
while(p1 != 0 && p2 != 0){
if(!Cyc_Tcpat_same_access(((struct Cyc_Tcpat_PathNode*)p1->hd)->access,((struct Cyc_Tcpat_PathNode*)p2->hd)->access))return 0;p1=p1->tl;
# 1171
p2=p2->tl;}
# 1173
if(p1 != p2)return 0;return 1;}
# 1154
static void*Cyc_Tcpat_ifeq(struct Cyc_List_List*access,struct Cyc_Tcpat_Con_s*con,void*d1,void*d2){
# 1178
void*test=Cyc_Tcpat_get_pat_test(con->orig_pat);
{void*_tmp1AA=d2;void*_tmp1AD;struct Cyc_List_List*_tmp1AC;struct Cyc_List_List*_tmp1AB;if(((struct Cyc_Tcpat_SwitchDec_Tcpat_Decision_struct*)_tmp1AA)->tag == 2U){_LL1: _tmp1AB=((struct Cyc_Tcpat_SwitchDec_Tcpat_Decision_struct*)_tmp1AA)->f1;_tmp1AC=((struct Cyc_Tcpat_SwitchDec_Tcpat_Decision_struct*)_tmp1AA)->f2;_tmp1AD=(void*)((struct Cyc_Tcpat_SwitchDec_Tcpat_Decision_struct*)_tmp1AA)->f3;_LL2: {struct Cyc_List_List*access2=_tmp1AB;struct Cyc_List_List*switch_clauses=_tmp1AC;void*default_decision=_tmp1AD;
# 1181
if(Cyc_Tcpat_same_path(access,access2)&&(int)(((con->orig_pat).pattern).tag == 1))
return(void*)({struct Cyc_Tcpat_SwitchDec_Tcpat_Decision_struct*_tmp1B0=_cycalloc(sizeof(*_tmp1B0));_tmp1B0->tag=2U,_tmp1B0->f1=access2,({
struct Cyc_List_List*_tmp484=({struct Cyc_List_List*_tmp1AF=_cycalloc(sizeof(*_tmp1AF));({struct _tuple0*_tmp483=({struct _tuple0*_tmp1AE=_cycalloc(sizeof(*_tmp1AE));_tmp1AE->f1=test,_tmp1AE->f2=d1;_tmp1AE;});_tmp1AF->hd=_tmp483;}),_tmp1AF->tl=switch_clauses;_tmp1AF;});_tmp1B0->f2=_tmp484;}),_tmp1B0->f3=default_decision;_tmp1B0;});else{
# 1185
goto _LL0;}}}else{_LL3: _LL4:
 goto _LL0;}_LL0:;}
# 1188
return(void*)({struct Cyc_Tcpat_SwitchDec_Tcpat_Decision_struct*_tmp1B3=_cycalloc(sizeof(*_tmp1B3));_tmp1B3->tag=2U,_tmp1B3->f1=access,({struct Cyc_List_List*_tmp486=({struct Cyc_List_List*_tmp1B2=_cycalloc(sizeof(*_tmp1B2));({struct _tuple0*_tmp485=({struct _tuple0*_tmp1B1=_cycalloc(sizeof(*_tmp1B1));_tmp1B1->f1=test,_tmp1B1->f2=d1;_tmp1B1;});_tmp1B2->hd=_tmp485;}),_tmp1B2->tl=0;_tmp1B2;});_tmp1B3->f2=_tmp486;}),_tmp1B3->f3=d2;_tmp1B3;});}struct _tuple21{struct Cyc_List_List*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;};
# 1199
enum Cyc_Tcpat_Answer{Cyc_Tcpat_Yes =0U,Cyc_Tcpat_No =1U,Cyc_Tcpat_Maybe =2U};
# 1202
static void Cyc_Tcpat_print_tab(int i){
for(0;i != 0;-- i){
({void*_tmp1B4=0U;({struct Cyc___cycFILE*_tmp488=Cyc_stderr;struct _fat_ptr _tmp487=({const char*_tmp1B5=" ";_tag_fat(_tmp1B5,sizeof(char),2U);});Cyc_fprintf(_tmp488,_tmp487,_tag_fat(_tmp1B4,sizeof(void*),0U));});});}}
# 1202
static void Cyc_Tcpat_print_con(struct Cyc_Tcpat_Con_s*c){
# 1209
union Cyc_Tcpat_Name_value n=c->name;
union Cyc_Tcpat_Name_value _tmp1B6=n;int _tmp1B7;struct _fat_ptr _tmp1B8;if((_tmp1B6.Name_v).tag == 1){_LL1: _tmp1B8=(_tmp1B6.Name_v).val;_LL2: {struct _fat_ptr s=_tmp1B8;
({struct Cyc_String_pa_PrintArg_struct _tmp1BB=({struct Cyc_String_pa_PrintArg_struct _tmp33F;_tmp33F.tag=0U,_tmp33F.f1=(struct _fat_ptr)((struct _fat_ptr)s);_tmp33F;});void*_tmp1B9[1U];_tmp1B9[0]=& _tmp1BB;({struct Cyc___cycFILE*_tmp48A=Cyc_stderr;struct _fat_ptr _tmp489=({const char*_tmp1BA="%s";_tag_fat(_tmp1BA,sizeof(char),3U);});Cyc_fprintf(_tmp48A,_tmp489,_tag_fat(_tmp1B9,sizeof(void*),1U));});});goto _LL0;}}else{_LL3: _tmp1B7=(_tmp1B6.Int_v).val;_LL4: {int i=_tmp1B7;
({struct Cyc_Int_pa_PrintArg_struct _tmp1BE=({struct Cyc_Int_pa_PrintArg_struct _tmp340;_tmp340.tag=1U,_tmp340.f1=(unsigned long)i;_tmp340;});void*_tmp1BC[1U];_tmp1BC[0]=& _tmp1BE;({struct Cyc___cycFILE*_tmp48C=Cyc_stderr;struct _fat_ptr _tmp48B=({const char*_tmp1BD="%d";_tag_fat(_tmp1BD,sizeof(char),3U);});Cyc_fprintf(_tmp48C,_tmp48B,_tag_fat(_tmp1BC,sizeof(void*),1U));});});goto _LL0;}}_LL0:;}
# 1202
static void Cyc_Tcpat_print_access(void*a){
# 1217
void*_tmp1BF=a;struct _fat_ptr*_tmp1C1;int _tmp1C0;unsigned _tmp1C3;struct Cyc_Absyn_Datatypefield*_tmp1C2;unsigned _tmp1C4;switch(*((int*)_tmp1BF)){case 0U: _LL1: _LL2:
({void*_tmp1C5=0U;({struct Cyc___cycFILE*_tmp48E=Cyc_stderr;struct _fat_ptr _tmp48D=({const char*_tmp1C6="DUMMY";_tag_fat(_tmp1C6,sizeof(char),6U);});Cyc_fprintf(_tmp48E,_tmp48D,_tag_fat(_tmp1C5,sizeof(void*),0U));});});goto _LL0;case 1U: _LL3: _LL4:
({void*_tmp1C7=0U;({struct Cyc___cycFILE*_tmp490=Cyc_stderr;struct _fat_ptr _tmp48F=({const char*_tmp1C8="*";_tag_fat(_tmp1C8,sizeof(char),2U);});Cyc_fprintf(_tmp490,_tmp48F,_tag_fat(_tmp1C7,sizeof(void*),0U));});});goto _LL0;case 2U: _LL5: _tmp1C4=((struct Cyc_Tcpat_TupleField_Tcpat_Access_struct*)_tmp1BF)->f1;_LL6: {unsigned i=_tmp1C4;
({struct Cyc_Int_pa_PrintArg_struct _tmp1CB=({struct Cyc_Int_pa_PrintArg_struct _tmp341;_tmp341.tag=1U,_tmp341.f1=(unsigned long)((int)i);_tmp341;});void*_tmp1C9[1U];_tmp1C9[0]=& _tmp1CB;({struct Cyc___cycFILE*_tmp492=Cyc_stderr;struct _fat_ptr _tmp491=({const char*_tmp1CA="[%d]";_tag_fat(_tmp1CA,sizeof(char),5U);});Cyc_fprintf(_tmp492,_tmp491,_tag_fat(_tmp1C9,sizeof(void*),1U));});});goto _LL0;}case 3U: _LL7: _tmp1C2=((struct Cyc_Tcpat_DatatypeField_Tcpat_Access_struct*)_tmp1BF)->f2;_tmp1C3=((struct Cyc_Tcpat_DatatypeField_Tcpat_Access_struct*)_tmp1BF)->f3;_LL8: {struct Cyc_Absyn_Datatypefield*f=_tmp1C2;unsigned i=_tmp1C3;
# 1222
({struct Cyc_String_pa_PrintArg_struct _tmp1CE=({struct Cyc_String_pa_PrintArg_struct _tmp343;_tmp343.tag=0U,({struct _fat_ptr _tmp493=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(f->name));_tmp343.f1=_tmp493;});_tmp343;});struct Cyc_Int_pa_PrintArg_struct _tmp1CF=({struct Cyc_Int_pa_PrintArg_struct _tmp342;_tmp342.tag=1U,_tmp342.f1=(unsigned long)((int)i);_tmp342;});void*_tmp1CC[2U];_tmp1CC[0]=& _tmp1CE,_tmp1CC[1]=& _tmp1CF;({struct Cyc___cycFILE*_tmp495=Cyc_stderr;struct _fat_ptr _tmp494=({const char*_tmp1CD="%s[%d]";_tag_fat(_tmp1CD,sizeof(char),7U);});Cyc_fprintf(_tmp495,_tmp494,_tag_fat(_tmp1CC,sizeof(void*),2U));});});goto _LL0;}default: _LL9: _tmp1C0=((struct Cyc_Tcpat_AggrField_Tcpat_Access_struct*)_tmp1BF)->f1;_tmp1C1=((struct Cyc_Tcpat_AggrField_Tcpat_Access_struct*)_tmp1BF)->f2;_LLA: {int tagged=_tmp1C0;struct _fat_ptr*f=_tmp1C1;
# 1224
if(tagged)
({struct Cyc_String_pa_PrintArg_struct _tmp1D2=({struct Cyc_String_pa_PrintArg_struct _tmp344;_tmp344.tag=0U,_tmp344.f1=(struct _fat_ptr)((struct _fat_ptr)*f);_tmp344;});void*_tmp1D0[1U];_tmp1D0[0]=& _tmp1D2;({struct Cyc___cycFILE*_tmp497=Cyc_stderr;struct _fat_ptr _tmp496=({const char*_tmp1D1=".tagunion.%s";_tag_fat(_tmp1D1,sizeof(char),13U);});Cyc_fprintf(_tmp497,_tmp496,_tag_fat(_tmp1D0,sizeof(void*),1U));});});else{
# 1227
({struct Cyc_String_pa_PrintArg_struct _tmp1D5=({struct Cyc_String_pa_PrintArg_struct _tmp345;_tmp345.tag=0U,_tmp345.f1=(struct _fat_ptr)((struct _fat_ptr)*f);_tmp345;});void*_tmp1D3[1U];_tmp1D3[0]=& _tmp1D5;({struct Cyc___cycFILE*_tmp499=Cyc_stderr;struct _fat_ptr _tmp498=({const char*_tmp1D4=".%s";_tag_fat(_tmp1D4,sizeof(char),4U);});Cyc_fprintf(_tmp499,_tmp498,_tag_fat(_tmp1D3,sizeof(void*),1U));});});}
goto _LL0;}}_LL0:;}
# 1202
static void Cyc_Tcpat_print_pat_test(void*p){
# 1233
void*_tmp1D6=p;struct Cyc_Absyn_Datatypefield*_tmp1D7;int _tmp1D9;struct _fat_ptr*_tmp1D8;int _tmp1DA;unsigned _tmp1DB;struct _fat_ptr _tmp1DC;struct Cyc_Absyn_Enumfield*_tmp1DD;struct Cyc_Absyn_Enumfield*_tmp1DE;struct Cyc_Absyn_Exp*_tmp1DF;switch(*((int*)_tmp1D6)){case 0U: if(((struct Cyc_Tcpat_WhereTest_Tcpat_PatTest_struct*)_tmp1D6)->f1 == 0){_LL1: _LL2:
({void*_tmp1E0=0U;({struct Cyc___cycFILE*_tmp49B=Cyc_stderr;struct _fat_ptr _tmp49A=({const char*_tmp1E1="where(NULL)";_tag_fat(_tmp1E1,sizeof(char),12U);});Cyc_fprintf(_tmp49B,_tmp49A,_tag_fat(_tmp1E0,sizeof(void*),0U));});});goto _LL0;}else{_LL3: _tmp1DF=((struct Cyc_Tcpat_WhereTest_Tcpat_PatTest_struct*)_tmp1D6)->f1;_LL4: {struct Cyc_Absyn_Exp*e=_tmp1DF;
({struct Cyc_String_pa_PrintArg_struct _tmp1E4=({struct Cyc_String_pa_PrintArg_struct _tmp346;_tmp346.tag=0U,({struct _fat_ptr _tmp49C=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_exp2string((struct Cyc_Absyn_Exp*)_check_null(e)));_tmp346.f1=_tmp49C;});_tmp346;});void*_tmp1E2[1U];_tmp1E2[0]=& _tmp1E4;({struct Cyc___cycFILE*_tmp49E=Cyc_stderr;struct _fat_ptr _tmp49D=({const char*_tmp1E3="where(%s)";_tag_fat(_tmp1E3,sizeof(char),10U);});Cyc_fprintf(_tmp49E,_tmp49D,_tag_fat(_tmp1E2,sizeof(void*),1U));});});goto _LL0;}}case 1U: _LL5: _LL6:
({void*_tmp1E5=0U;({struct Cyc___cycFILE*_tmp4A0=Cyc_stderr;struct _fat_ptr _tmp49F=({const char*_tmp1E6="NULL";_tag_fat(_tmp1E6,sizeof(char),5U);});Cyc_fprintf(_tmp4A0,_tmp49F,_tag_fat(_tmp1E5,sizeof(void*),0U));});});goto _LL0;case 2U: _LL7: _LL8:
({void*_tmp1E7=0U;({struct Cyc___cycFILE*_tmp4A2=Cyc_stderr;struct _fat_ptr _tmp4A1=({const char*_tmp1E8="NOT-NULL:";_tag_fat(_tmp1E8,sizeof(char),10U);});Cyc_fprintf(_tmp4A2,_tmp4A1,_tag_fat(_tmp1E7,sizeof(void*),0U));});});goto _LL0;case 4U: _LL9: _tmp1DE=((struct Cyc_Tcpat_EqAnonEnum_Tcpat_PatTest_struct*)_tmp1D6)->f2;_LLA: {struct Cyc_Absyn_Enumfield*ef=_tmp1DE;
_tmp1DD=ef;goto _LLC;}case 3U: _LLB: _tmp1DD=((struct Cyc_Tcpat_EqEnum_Tcpat_PatTest_struct*)_tmp1D6)->f2;_LLC: {struct Cyc_Absyn_Enumfield*ef=_tmp1DD;
({struct Cyc_String_pa_PrintArg_struct _tmp1EB=({struct Cyc_String_pa_PrintArg_struct _tmp347;_tmp347.tag=0U,({struct _fat_ptr _tmp4A3=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(ef->name));_tmp347.f1=_tmp4A3;});_tmp347;});void*_tmp1E9[1U];_tmp1E9[0]=& _tmp1EB;({struct Cyc___cycFILE*_tmp4A5=Cyc_stderr;struct _fat_ptr _tmp4A4=({const char*_tmp1EA="%s";_tag_fat(_tmp1EA,sizeof(char),3U);});Cyc_fprintf(_tmp4A5,_tmp4A4,_tag_fat(_tmp1E9,sizeof(void*),1U));});});goto _LL0;}case 5U: _LLD: _tmp1DC=((struct Cyc_Tcpat_EqFloat_Tcpat_PatTest_struct*)_tmp1D6)->f1;_LLE: {struct _fat_ptr s=_tmp1DC;
({struct Cyc_String_pa_PrintArg_struct _tmp1EE=({struct Cyc_String_pa_PrintArg_struct _tmp348;_tmp348.tag=0U,_tmp348.f1=(struct _fat_ptr)((struct _fat_ptr)s);_tmp348;});void*_tmp1EC[1U];_tmp1EC[0]=& _tmp1EE;({struct Cyc___cycFILE*_tmp4A7=Cyc_stderr;struct _fat_ptr _tmp4A6=({const char*_tmp1ED="%s";_tag_fat(_tmp1ED,sizeof(char),3U);});Cyc_fprintf(_tmp4A7,_tmp4A6,_tag_fat(_tmp1EC,sizeof(void*),1U));});});goto _LL0;}case 6U: _LLF: _tmp1DB=((struct Cyc_Tcpat_EqConst_Tcpat_PatTest_struct*)_tmp1D6)->f1;_LL10: {unsigned i=_tmp1DB;
({struct Cyc_Int_pa_PrintArg_struct _tmp1F1=({struct Cyc_Int_pa_PrintArg_struct _tmp349;_tmp349.tag=1U,_tmp349.f1=(unsigned long)((int)i);_tmp349;});void*_tmp1EF[1U];_tmp1EF[0]=& _tmp1F1;({struct Cyc___cycFILE*_tmp4A9=Cyc_stderr;struct _fat_ptr _tmp4A8=({const char*_tmp1F0="%d";_tag_fat(_tmp1F0,sizeof(char),3U);});Cyc_fprintf(_tmp4A9,_tmp4A8,_tag_fat(_tmp1EF,sizeof(void*),1U));});});goto _LL0;}case 7U: _LL11: _tmp1DA=((struct Cyc_Tcpat_EqDatatypeTag_Tcpat_PatTest_struct*)_tmp1D6)->f1;_LL12: {int i=_tmp1DA;
({struct Cyc_Int_pa_PrintArg_struct _tmp1F4=({struct Cyc_Int_pa_PrintArg_struct _tmp34A;_tmp34A.tag=1U,_tmp34A.f1=(unsigned long)i;_tmp34A;});void*_tmp1F2[1U];_tmp1F2[0]=& _tmp1F4;({struct Cyc___cycFILE*_tmp4AB=Cyc_stderr;struct _fat_ptr _tmp4AA=({const char*_tmp1F3="datatypetag(%d)";_tag_fat(_tmp1F3,sizeof(char),16U);});Cyc_fprintf(_tmp4AB,_tmp4AA,_tag_fat(_tmp1F2,sizeof(void*),1U));});});goto _LL0;}case 8U: _LL13: _tmp1D8=((struct Cyc_Tcpat_EqTaggedUnion_Tcpat_PatTest_struct*)_tmp1D6)->f1;_tmp1D9=((struct Cyc_Tcpat_EqTaggedUnion_Tcpat_PatTest_struct*)_tmp1D6)->f2;_LL14: {struct _fat_ptr*f=_tmp1D8;int i=_tmp1D9;
({struct Cyc_String_pa_PrintArg_struct _tmp1F7=({struct Cyc_String_pa_PrintArg_struct _tmp34C;_tmp34C.tag=0U,_tmp34C.f1=(struct _fat_ptr)((struct _fat_ptr)*f);_tmp34C;});struct Cyc_Int_pa_PrintArg_struct _tmp1F8=({struct Cyc_Int_pa_PrintArg_struct _tmp34B;_tmp34B.tag=1U,_tmp34B.f1=(unsigned long)i;_tmp34B;});void*_tmp1F5[2U];_tmp1F5[0]=& _tmp1F7,_tmp1F5[1]=& _tmp1F8;({struct Cyc___cycFILE*_tmp4AD=Cyc_stderr;struct _fat_ptr _tmp4AC=({const char*_tmp1F6="uniontag[%s](%d)";_tag_fat(_tmp1F6,sizeof(char),17U);});Cyc_fprintf(_tmp4AD,_tmp4AC,_tag_fat(_tmp1F5,sizeof(void*),2U));});});goto _LL0;}default: _LL15: _tmp1D7=((struct Cyc_Tcpat_EqExtensibleDatatype_Tcpat_PatTest_struct*)_tmp1D6)->f2;_LL16: {struct Cyc_Absyn_Datatypefield*f=_tmp1D7;
# 1245
({struct Cyc_String_pa_PrintArg_struct _tmp1FB=({struct Cyc_String_pa_PrintArg_struct _tmp34D;_tmp34D.tag=0U,({struct _fat_ptr _tmp4AE=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(f->name));_tmp34D.f1=_tmp4AE;});_tmp34D;});void*_tmp1F9[1U];_tmp1F9[0]=& _tmp1FB;({struct Cyc___cycFILE*_tmp4B0=Cyc_stderr;struct _fat_ptr _tmp4AF=({const char*_tmp1FA="datatypefield(%s)";_tag_fat(_tmp1FA,sizeof(char),18U);});Cyc_fprintf(_tmp4B0,_tmp4AF,_tag_fat(_tmp1F9,sizeof(void*),1U));});});}}_LL0:;}
# 1202
static void Cyc_Tcpat_print_rhs(struct Cyc_Tcpat_Rhs*r){
# 1250
({struct Cyc_String_pa_PrintArg_struct _tmp1FE=({struct Cyc_String_pa_PrintArg_struct _tmp34E;_tmp34E.tag=0U,({struct _fat_ptr _tmp4B1=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_stmt2string(r->rhs));_tmp34E.f1=_tmp4B1;});_tmp34E;});void*_tmp1FC[1U];_tmp1FC[0]=& _tmp1FE;({struct Cyc___cycFILE*_tmp4B3=Cyc_stderr;struct _fat_ptr _tmp4B2=({const char*_tmp1FD="%s";_tag_fat(_tmp1FD,sizeof(char),3U);});Cyc_fprintf(_tmp4B3,_tmp4B2,_tag_fat(_tmp1FC,sizeof(void*),1U));});});}
# 1202
static void Cyc_Tcpat_print_dec_tree(void*d,int tab){
# 1254
void*_tmp1FF=d;void*_tmp202;struct Cyc_List_List*_tmp201;struct Cyc_List_List*_tmp200;struct Cyc_Tcpat_Rhs*_tmp203;switch(*((int*)_tmp1FF)){case 1U: _LL1: _tmp203=((struct Cyc_Tcpat_Success_Tcpat_Decision_struct*)_tmp1FF)->f1;_LL2: {struct Cyc_Tcpat_Rhs*rhs=_tmp203;
# 1256
Cyc_Tcpat_print_tab(tab);
({void*_tmp204=0U;({struct Cyc___cycFILE*_tmp4B5=Cyc_stderr;struct _fat_ptr _tmp4B4=({const char*_tmp205="Success(";_tag_fat(_tmp205,sizeof(char),9U);});Cyc_fprintf(_tmp4B5,_tmp4B4,_tag_fat(_tmp204,sizeof(void*),0U));});});Cyc_Tcpat_print_rhs(rhs);({void*_tmp206=0U;({struct Cyc___cycFILE*_tmp4B7=Cyc_stderr;struct _fat_ptr _tmp4B6=({const char*_tmp207=")\n";_tag_fat(_tmp207,sizeof(char),3U);});Cyc_fprintf(_tmp4B7,_tmp4B6,_tag_fat(_tmp206,sizeof(void*),0U));});});
goto _LL0;}case 0U: _LL3: _LL4:
({void*_tmp208=0U;({struct Cyc___cycFILE*_tmp4B9=Cyc_stderr;struct _fat_ptr _tmp4B8=({const char*_tmp209="Failure\n";_tag_fat(_tmp209,sizeof(char),9U);});Cyc_fprintf(_tmp4B9,_tmp4B8,_tag_fat(_tmp208,sizeof(void*),0U));});});goto _LL0;default: _LL5: _tmp200=((struct Cyc_Tcpat_SwitchDec_Tcpat_Decision_struct*)_tmp1FF)->f1;_tmp201=((struct Cyc_Tcpat_SwitchDec_Tcpat_Decision_struct*)_tmp1FF)->f2;_tmp202=(void*)((struct Cyc_Tcpat_SwitchDec_Tcpat_Decision_struct*)_tmp1FF)->f3;_LL6: {struct Cyc_List_List*a=_tmp200;struct Cyc_List_List*cases=_tmp201;void*def=_tmp202;
# 1261
Cyc_Tcpat_print_tab(tab);
({void*_tmp20A=0U;({struct Cyc___cycFILE*_tmp4BB=Cyc_stderr;struct _fat_ptr _tmp4BA=({const char*_tmp20B="Switch[";_tag_fat(_tmp20B,sizeof(char),8U);});Cyc_fprintf(_tmp4BB,_tmp4BA,_tag_fat(_tmp20A,sizeof(void*),0U));});});
for(0;a != 0;a=a->tl){
Cyc_Tcpat_print_access(((struct Cyc_Tcpat_PathNode*)a->hd)->access);
if(a->tl != 0)({void*_tmp20C=0U;({struct Cyc___cycFILE*_tmp4BD=Cyc_stderr;struct _fat_ptr _tmp4BC=({const char*_tmp20D=",";_tag_fat(_tmp20D,sizeof(char),2U);});Cyc_fprintf(_tmp4BD,_tmp4BC,_tag_fat(_tmp20C,sizeof(void*),0U));});});}
# 1267
({void*_tmp20E=0U;({struct Cyc___cycFILE*_tmp4BF=Cyc_stderr;struct _fat_ptr _tmp4BE=({const char*_tmp20F="] {\n";_tag_fat(_tmp20F,sizeof(char),5U);});Cyc_fprintf(_tmp4BF,_tmp4BE,_tag_fat(_tmp20E,sizeof(void*),0U));});});
for(0;cases != 0;cases=cases->tl){
struct _tuple0 _stmttmp25=*((struct _tuple0*)cases->hd);void*_tmp211;void*_tmp210;_LL8: _tmp210=_stmttmp25.f1;_tmp211=_stmttmp25.f2;_LL9: {void*pt=_tmp210;void*d=_tmp211;
Cyc_Tcpat_print_tab(tab);
({void*_tmp212=0U;({struct Cyc___cycFILE*_tmp4C1=Cyc_stderr;struct _fat_ptr _tmp4C0=({const char*_tmp213="case ";_tag_fat(_tmp213,sizeof(char),6U);});Cyc_fprintf(_tmp4C1,_tmp4C0,_tag_fat(_tmp212,sizeof(void*),0U));});});
Cyc_Tcpat_print_pat_test(pt);
({void*_tmp214=0U;({struct Cyc___cycFILE*_tmp4C3=Cyc_stderr;struct _fat_ptr _tmp4C2=({const char*_tmp215=":\n";_tag_fat(_tmp215,sizeof(char),3U);});Cyc_fprintf(_tmp4C3,_tmp4C2,_tag_fat(_tmp214,sizeof(void*),0U));});});
Cyc_Tcpat_print_dec_tree(d,tab + 7);}}
# 1276
Cyc_Tcpat_print_tab(tab);
({void*_tmp216=0U;({struct Cyc___cycFILE*_tmp4C5=Cyc_stderr;struct _fat_ptr _tmp4C4=({const char*_tmp217="default:\n";_tag_fat(_tmp217,sizeof(char),10U);});Cyc_fprintf(_tmp4C5,_tmp4C4,_tag_fat(_tmp216,sizeof(void*),0U));});});
Cyc_Tcpat_print_dec_tree(def,tab + 7);
Cyc_Tcpat_print_tab(tab);
({void*_tmp218=0U;({struct Cyc___cycFILE*_tmp4C7=Cyc_stderr;struct _fat_ptr _tmp4C6=({const char*_tmp219="}\n";_tag_fat(_tmp219,sizeof(char),3U);});Cyc_fprintf(_tmp4C7,_tmp4C6,_tag_fat(_tmp218,sizeof(void*),0U));});});}}_LL0:;}
# 1202
void Cyc_Tcpat_print_decision_tree(void*d){
# 1285
Cyc_Tcpat_print_dec_tree(d,0);}
# 1202
static void*Cyc_Tcpat_add_neg(void*td,struct Cyc_Tcpat_Con_s*c){
# 1293
void*_tmp21A=td;struct Cyc_Set_Set*_tmp21B;if(((struct Cyc_Tcpat_Neg_Tcpat_Term_desc_struct*)_tmp21A)->tag == 1U){_LL1: _tmp21B=((struct Cyc_Tcpat_Neg_Tcpat_Term_desc_struct*)_tmp21A)->f1;_LL2: {struct Cyc_Set_Set*cs=_tmp21B;
# 1302
return(void*)({struct Cyc_Tcpat_Neg_Tcpat_Term_desc_struct*_tmp21D=_cycalloc(sizeof(*_tmp21D));_tmp21D->tag=1U,({struct Cyc_Set_Set*_tmp4CA=({struct Cyc_Set_Set*(*_tmp4C9)(struct Cyc_Set_Set*s,struct Cyc_Tcpat_Con_s*elt)=({struct Cyc_Set_Set*(*_tmp21C)(struct Cyc_Set_Set*s,struct Cyc_Tcpat_Con_s*elt)=(struct Cyc_Set_Set*(*)(struct Cyc_Set_Set*s,struct Cyc_Tcpat_Con_s*elt))Cyc_Set_insert;_tmp21C;});struct Cyc_Set_Set*_tmp4C8=cs;_tmp4C9(_tmp4C8,c);});_tmp21D->f1=_tmp4CA;});_tmp21D;});}}else{_LL3: _LL4:
({void*_tmp21E=0U;({int(*_tmp4CC)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp220)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp220;});struct _fat_ptr _tmp4CB=({const char*_tmp21F="add_neg called when td is Positive";_tag_fat(_tmp21F,sizeof(char),35U);});_tmp4CC(_tmp4CB,_tag_fat(_tmp21E,sizeof(void*),0U));});});}_LL0:;}
# 1202
static enum Cyc_Tcpat_Answer Cyc_Tcpat_static_match(struct Cyc_Tcpat_Con_s*c,void*td){
# 1310
void*_tmp221=td;struct Cyc_Set_Set*_tmp222;struct Cyc_Tcpat_Con_s*_tmp223;if(((struct Cyc_Tcpat_Pos_Tcpat_Term_desc_struct*)_tmp221)->tag == 0U){_LL1: _tmp223=((struct Cyc_Tcpat_Pos_Tcpat_Term_desc_struct*)_tmp221)->f1;_LL2: {struct Cyc_Tcpat_Con_s*c2=_tmp223;
# 1313
if(Cyc_Tcpat_compare_con(c,c2)== 0)return Cyc_Tcpat_Yes;else{
return Cyc_Tcpat_No;}}}else{_LL3: _tmp222=((struct Cyc_Tcpat_Neg_Tcpat_Term_desc_struct*)_tmp221)->f1;_LL4: {struct Cyc_Set_Set*cs=_tmp222;
# 1317
if(({int(*_tmp4CE)(struct Cyc_Set_Set*s,struct Cyc_Tcpat_Con_s*elt)=({int(*_tmp224)(struct Cyc_Set_Set*s,struct Cyc_Tcpat_Con_s*elt)=(int(*)(struct Cyc_Set_Set*s,struct Cyc_Tcpat_Con_s*elt))Cyc_Set_member;_tmp224;});struct Cyc_Set_Set*_tmp4CD=cs;_tmp4CE(_tmp4CD,c);}))return Cyc_Tcpat_No;else{
# 1320
if(c->span != 0 &&({int _tmp4CF=*((int*)_check_null(c->span));_tmp4CF == Cyc_Set_cardinality(cs)+ 1;}))
return Cyc_Tcpat_Yes;else{
# 1323
return Cyc_Tcpat_Maybe;}}}}_LL0:;}struct _tuple22{struct Cyc_Tcpat_Con_s*f1;struct Cyc_List_List*f2;};
# 1202
static struct Cyc_List_List*Cyc_Tcpat_augment(struct Cyc_List_List*ctxt,void*dsc){
# 1332
struct Cyc_List_List*_tmp225=ctxt;struct Cyc_List_List*_tmp228;struct Cyc_List_List*_tmp227;struct Cyc_Tcpat_Con_s*_tmp226;if(_tmp225 == 0){_LL1: _LL2:
 return 0;}else{_LL3: _tmp226=((struct _tuple22*)_tmp225->hd)->f1;_tmp227=((struct _tuple22*)_tmp225->hd)->f2;_tmp228=_tmp225->tl;_LL4: {struct Cyc_Tcpat_Con_s*c=_tmp226;struct Cyc_List_List*args=_tmp227;struct Cyc_List_List*rest=_tmp228;
# 1335
return({struct Cyc_List_List*_tmp22B=_cycalloc(sizeof(*_tmp22B));
({struct _tuple22*_tmp4D1=({struct _tuple22*_tmp22A=_cycalloc(sizeof(*_tmp22A));_tmp22A->f1=c,({struct Cyc_List_List*_tmp4D0=({struct Cyc_List_List*_tmp229=_cycalloc(sizeof(*_tmp229));_tmp229->hd=dsc,_tmp229->tl=args;_tmp229;});_tmp22A->f2=_tmp4D0;});_tmp22A;});_tmp22B->hd=_tmp4D1;}),_tmp22B->tl=rest;_tmp22B;});}}_LL0:;}
# 1202
static struct Cyc_List_List*Cyc_Tcpat_norm_context(struct Cyc_List_List*ctxt){
# 1344
struct Cyc_List_List*_tmp22C=ctxt;struct Cyc_List_List*_tmp22F;struct Cyc_List_List*_tmp22E;struct Cyc_Tcpat_Con_s*_tmp22D;if(_tmp22C == 0){_LL1: _LL2:
({void*_tmp230=0U;({int(*_tmp4D3)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp232)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp232;});struct _fat_ptr _tmp4D2=({const char*_tmp231="norm_context: empty context";_tag_fat(_tmp231,sizeof(char),28U);});_tmp4D3(_tmp4D2,_tag_fat(_tmp230,sizeof(void*),0U));});});}else{_LL3: _tmp22D=((struct _tuple22*)_tmp22C->hd)->f1;_tmp22E=((struct _tuple22*)_tmp22C->hd)->f2;_tmp22F=_tmp22C->tl;_LL4: {struct Cyc_Tcpat_Con_s*c=_tmp22D;struct Cyc_List_List*args=_tmp22E;struct Cyc_List_List*rest=_tmp22F;
# 1347
return({struct Cyc_List_List*_tmp4D5=rest;Cyc_Tcpat_augment(_tmp4D5,(void*)({struct Cyc_Tcpat_Pos_Tcpat_Term_desc_struct*_tmp233=_cycalloc(sizeof(*_tmp233));_tmp233->tag=0U,_tmp233->f1=c,({struct Cyc_List_List*_tmp4D4=Cyc_List_rev(args);_tmp233->f2=_tmp4D4;});_tmp233;}));});}}_LL0:;}
# 1202
static void*Cyc_Tcpat_build_desc(struct Cyc_List_List*ctxt,void*dsc,struct Cyc_List_List*work){
# 1358
struct _tuple1 _stmttmp26=({struct _tuple1 _tmp34F;_tmp34F.f1=ctxt,_tmp34F.f2=work;_tmp34F;});struct _tuple1 _tmp234=_stmttmp26;struct Cyc_List_List*_tmp239;struct Cyc_List_List*_tmp238;struct Cyc_List_List*_tmp237;struct Cyc_List_List*_tmp236;struct Cyc_Tcpat_Con_s*_tmp235;if(_tmp234.f1 == 0){if(_tmp234.f2 == 0){_LL1: _LL2:
 return dsc;}else{_LL3: _LL4:
 goto _LL6;}}else{if(_tmp234.f2 == 0){_LL5: _LL6:
({void*_tmp23A=0U;({int(*_tmp4D7)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp23C)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp23C;});struct _fat_ptr _tmp4D6=({const char*_tmp23B="build_desc: ctxt and work don't match";_tag_fat(_tmp23B,sizeof(char),38U);});_tmp4D7(_tmp4D6,_tag_fat(_tmp23A,sizeof(void*),0U));});});}else{_LL7: _tmp235=((struct _tuple22*)(_tmp234.f1)->hd)->f1;_tmp236=((struct _tuple22*)(_tmp234.f1)->hd)->f2;_tmp237=(_tmp234.f1)->tl;_tmp238=((struct _tuple21*)(_tmp234.f2)->hd)->f3;_tmp239=(_tmp234.f2)->tl;_LL8: {struct Cyc_Tcpat_Con_s*c=_tmp235;struct Cyc_List_List*args=_tmp236;struct Cyc_List_List*rest=_tmp237;struct Cyc_List_List*dargs=_tmp238;struct Cyc_List_List*work2=_tmp239;
# 1363
struct Cyc_Tcpat_Pos_Tcpat_Term_desc_struct*td=({struct Cyc_Tcpat_Pos_Tcpat_Term_desc_struct*_tmp23E=_cycalloc(sizeof(*_tmp23E));_tmp23E->tag=0U,_tmp23E->f1=c,({struct Cyc_List_List*_tmp4D9=({struct Cyc_List_List*_tmp4D8=Cyc_List_rev(args);Cyc_List_append(_tmp4D8,({struct Cyc_List_List*_tmp23D=_cycalloc(sizeof(*_tmp23D));_tmp23D->hd=dsc,_tmp23D->tl=dargs;_tmp23D;}));});_tmp23E->f2=_tmp4D9;});_tmp23E;});
return Cyc_Tcpat_build_desc(rest,(void*)td,work2);}}}_LL0:;}
# 1202
static void*Cyc_Tcpat_match(void*p,struct Cyc_List_List*obj,void*dsc,struct Cyc_List_List*ctx,struct Cyc_List_List*work,struct Cyc_Tcpat_Rhs*right_hand_side,struct Cyc_List_List*rules);struct _tuple23{void*f1;struct Cyc_Tcpat_Rhs*f2;};
# 1375
static void*Cyc_Tcpat_or_match(void*dsc,struct Cyc_List_List*allmrules){
struct Cyc_List_List*_tmp23F=allmrules;struct Cyc_List_List*_tmp242;struct Cyc_Tcpat_Rhs*_tmp241;void*_tmp240;if(_tmp23F == 0){_LL1: _LL2:
 return(void*)({struct Cyc_Tcpat_Failure_Tcpat_Decision_struct*_tmp243=_cycalloc(sizeof(*_tmp243));_tmp243->tag=0U,_tmp243->f1=dsc;_tmp243;});}else{_LL3: _tmp240=((struct _tuple23*)_tmp23F->hd)->f1;_tmp241=((struct _tuple23*)_tmp23F->hd)->f2;_tmp242=_tmp23F->tl;_LL4: {void*pat1=_tmp240;struct Cyc_Tcpat_Rhs*rhs1=_tmp241;struct Cyc_List_List*rulerest=_tmp242;
# 1379
return Cyc_Tcpat_match(pat1,0,dsc,0,0,rhs1,rulerest);}}_LL0:;}
# 1375
static void*Cyc_Tcpat_match_compile(struct Cyc_List_List*allmrules){
# 1385
return({void*_tmp4DB=(void*)({struct Cyc_Tcpat_Neg_Tcpat_Term_desc_struct*_tmp244=_cycalloc(sizeof(*_tmp244));_tmp244->tag=1U,({struct Cyc_Set_Set*_tmp4DA=Cyc_Tcpat_empty_con_set();_tmp244->f1=_tmp4DA;});_tmp244;});Cyc_Tcpat_or_match(_tmp4DB,allmrules);});}
# 1375
static void*Cyc_Tcpat_and_match(struct Cyc_List_List*ctx,struct Cyc_List_List*work,struct Cyc_Tcpat_Rhs*right_hand_side,struct Cyc_List_List*rules){
# 1394
struct Cyc_List_List*_tmp245=work;struct Cyc_List_List*_tmp249;struct Cyc_List_List*_tmp248;struct Cyc_List_List*_tmp247;struct Cyc_List_List*_tmp246;struct Cyc_List_List*_tmp24A;if(_tmp245 == 0){_LL1: _LL2:
 return(void*)({struct Cyc_Tcpat_Success_Tcpat_Decision_struct*_tmp24B=_cycalloc(sizeof(*_tmp24B));_tmp24B->tag=1U,_tmp24B->f1=right_hand_side;_tmp24B;});}else{if(((struct _tuple21*)((struct Cyc_List_List*)_tmp245)->hd)->f1 == 0){if(((struct _tuple21*)((struct Cyc_List_List*)_tmp245)->hd)->f2 == 0){if(((struct _tuple21*)((struct Cyc_List_List*)_tmp245)->hd)->f3 == 0){_LL3: _tmp24A=_tmp245->tl;_LL4: {struct Cyc_List_List*workr=_tmp24A;
# 1397
return({struct Cyc_List_List*_tmp4DE=Cyc_Tcpat_norm_context(ctx);struct Cyc_List_List*_tmp4DD=workr;struct Cyc_Tcpat_Rhs*_tmp4DC=right_hand_side;Cyc_Tcpat_and_match(_tmp4DE,_tmp4DD,_tmp4DC,rules);});}}else{goto _LL5;}}else{goto _LL5;}}else{_LL5: _tmp246=((struct _tuple21*)_tmp245->hd)->f1;_tmp247=((struct _tuple21*)_tmp245->hd)->f2;_tmp248=((struct _tuple21*)_tmp245->hd)->f3;_tmp249=_tmp245->tl;_LL6: {struct Cyc_List_List*pats=_tmp246;struct Cyc_List_List*objs=_tmp247;struct Cyc_List_List*dscs=_tmp248;struct Cyc_List_List*workr=_tmp249;
# 1399
if((pats == 0 || objs == 0)|| dscs == 0)
({void*_tmp24C=0U;({int(*_tmp4E0)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp24E)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp24E;});struct _fat_ptr _tmp4DF=({const char*_tmp24D="tcpat:and_match: malformed work frame";_tag_fat(_tmp24D,sizeof(char),38U);});_tmp4E0(_tmp4DF,_tag_fat(_tmp24C,sizeof(void*),0U));});});{
# 1399
struct Cyc_List_List*_stmttmp27=pats;struct Cyc_List_List*_tmp250;void*_tmp24F;_LL8: _tmp24F=(void*)_stmttmp27->hd;_tmp250=_stmttmp27->tl;_LL9: {void*pat1=_tmp24F;struct Cyc_List_List*patr=_tmp250;
# 1402
struct Cyc_List_List*_stmttmp28=objs;struct Cyc_List_List*_tmp252;struct Cyc_List_List*_tmp251;_LLB: _tmp251=(struct Cyc_List_List*)_stmttmp28->hd;_tmp252=_stmttmp28->tl;_LLC: {struct Cyc_List_List*obj1=_tmp251;struct Cyc_List_List*objr=_tmp252;
struct Cyc_List_List*_stmttmp29=dscs;struct Cyc_List_List*_tmp254;void*_tmp253;_LLE: _tmp253=(void*)_stmttmp29->hd;_tmp254=_stmttmp29->tl;_LLF: {void*dsc1=_tmp253;struct Cyc_List_List*dscr=_tmp254;
struct _tuple21*wf=({struct _tuple21*_tmp256=_cycalloc(sizeof(*_tmp256));_tmp256->f1=patr,_tmp256->f2=objr,_tmp256->f3=dscr;_tmp256;});
return({void*_tmp4E6=pat1;struct Cyc_List_List*_tmp4E5=obj1;void*_tmp4E4=dsc1;struct Cyc_List_List*_tmp4E3=ctx;struct Cyc_List_List*_tmp4E2=({struct Cyc_List_List*_tmp255=_cycalloc(sizeof(*_tmp255));_tmp255->hd=wf,_tmp255->tl=workr;_tmp255;});struct Cyc_Tcpat_Rhs*_tmp4E1=right_hand_side;Cyc_Tcpat_match(_tmp4E6,_tmp4E5,_tmp4E4,_tmp4E3,_tmp4E2,_tmp4E1,rules);});}}}}}}}_LL0:;}
# 1375
static struct Cyc_List_List*Cyc_Tcpat_getdargs(struct Cyc_Tcpat_Con_s*pcon,void*dsc){
# 1411
void*_tmp257=dsc;struct Cyc_List_List*_tmp258;struct Cyc_Set_Set*_tmp259;if(((struct Cyc_Tcpat_Neg_Tcpat_Term_desc_struct*)_tmp257)->tag == 1U){_LL1: _tmp259=((struct Cyc_Tcpat_Neg_Tcpat_Term_desc_struct*)_tmp257)->f1;_LL2: {struct Cyc_Set_Set*ncs=_tmp259;
# 1416
void*any=(void*)({struct Cyc_Tcpat_Neg_Tcpat_Term_desc_struct*_tmp25B=_cycalloc(sizeof(*_tmp25B));_tmp25B->tag=1U,({struct Cyc_Set_Set*_tmp4E7=Cyc_Tcpat_empty_con_set();_tmp25B->f1=_tmp4E7;});_tmp25B;});
struct Cyc_List_List*res=0;
{int i=0;for(0;i < pcon->arity;++ i){
res=({struct Cyc_List_List*_tmp25A=_cycalloc(sizeof(*_tmp25A));_tmp25A->hd=any,_tmp25A->tl=res;_tmp25A;});}}
return res;}}else{_LL3: _tmp258=((struct Cyc_Tcpat_Pos_Tcpat_Term_desc_struct*)_tmp257)->f2;_LL4: {struct Cyc_List_List*dargs=_tmp258;
return dargs;}}_LL0:;}
# 1375
static void*Cyc_Tcpat_get_access(union Cyc_Tcpat_PatOrWhere pw,int i){
# 1426
union Cyc_Tcpat_PatOrWhere _tmp25C=pw;struct Cyc_Absyn_Pat*_tmp25D;if((_tmp25C.where_clause).tag == 2){_LL1: _LL2:
 return(void*)& Cyc_Tcpat_Dummy_val;}else{_LL3: _tmp25D=(_tmp25C.pattern).val;_LL4: {struct Cyc_Absyn_Pat*p=_tmp25D;
# 1429
void*_stmttmp2A=p->r;void*_tmp25E=_stmttmp2A;struct Cyc_List_List*_tmp260;union Cyc_Absyn_AggrInfo _tmp25F;struct Cyc_Absyn_Datatypefield*_tmp262;struct Cyc_Absyn_Datatypedecl*_tmp261;switch(*((int*)_tmp25E)){case 6U: _LL6: _LL7:
# 1431
 if(i != 0)
({void*_tmp263=0U;({int(*_tmp4EA)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp267)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp267;});struct _fat_ptr _tmp4E9=(struct _fat_ptr)({struct Cyc_Int_pa_PrintArg_struct _tmp266=({struct Cyc_Int_pa_PrintArg_struct _tmp350;_tmp350.tag=1U,_tmp350.f1=(unsigned long)i;_tmp350;});void*_tmp264[1U];_tmp264[0]=& _tmp266;({struct _fat_ptr _tmp4E8=({const char*_tmp265="get_access on pointer pattern with offset %d\n";_tag_fat(_tmp265,sizeof(char),46U);});Cyc_aprintf(_tmp4E8,_tag_fat(_tmp264,sizeof(void*),1U));});});_tmp4EA(_tmp4E9,_tag_fat(_tmp263,sizeof(void*),0U));});});
# 1431
return(void*)& Cyc_Tcpat_Deref_val;case 5U: _LL8: _LL9:
# 1434
 return(void*)({struct Cyc_Tcpat_TupleField_Tcpat_Access_struct*_tmp268=_cycalloc(sizeof(*_tmp268));_tmp268->tag=2U,_tmp268->f1=(unsigned)i;_tmp268;});case 8U: _LLA: _tmp261=((struct Cyc_Absyn_Datatype_p_Absyn_Raw_pat_struct*)_tmp25E)->f1;_tmp262=((struct Cyc_Absyn_Datatype_p_Absyn_Raw_pat_struct*)_tmp25E)->f2;_LLB: {struct Cyc_Absyn_Datatypedecl*tud=_tmp261;struct Cyc_Absyn_Datatypefield*tuf=_tmp262;
return(void*)({struct Cyc_Tcpat_DatatypeField_Tcpat_Access_struct*_tmp269=_cycalloc(sizeof(*_tmp269));_tmp269->tag=3U,_tmp269->f1=tud,_tmp269->f2=tuf,_tmp269->f3=(unsigned)i;_tmp269;});}case 7U: if(((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmp25E)->f1 != 0){_LLC: _tmp25F=*((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmp25E)->f1;_tmp260=((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmp25E)->f3;_LLD: {union Cyc_Absyn_AggrInfo info=_tmp25F;struct Cyc_List_List*dlps=_tmp260;
# 1437
struct Cyc_Absyn_Aggrdecl*ad=Cyc_Absyn_get_known_aggrdecl(info);
if(((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->tagged){
struct Cyc_List_List*_stmttmp2B=(*((struct _tuple19*)((struct Cyc_List_List*)_check_null(dlps))->hd)).f1;struct Cyc_List_List*_tmp26A=_stmttmp2B;struct _fat_ptr*_tmp26B;if(_tmp26A != 0){if(((struct Cyc_Absyn_FieldName_Absyn_Designator_struct*)((struct Cyc_List_List*)_tmp26A)->hd)->tag == 1U){if(((struct Cyc_List_List*)_tmp26A)->tl == 0){_LL11: _tmp26B=((struct Cyc_Absyn_FieldName_Absyn_Designator_struct*)_tmp26A->hd)->f1;_LL12: {struct _fat_ptr*f=_tmp26B;
# 1441
return(void*)({struct Cyc_Tcpat_AggrField_Tcpat_Access_struct*_tmp26C=_cycalloc(sizeof(*_tmp26C));_tmp26C->tag=4U,_tmp26C->f1=((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->tagged,_tmp26C->f2=f;_tmp26C;});}}else{goto _LL13;}}else{goto _LL13;}}else{_LL13: _LL14:
({void*_tmp26D=0U;({int(*_tmp4EE)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp271)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp271;});struct _fat_ptr _tmp4ED=(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp270=({struct Cyc_String_pa_PrintArg_struct _tmp351;_tmp351.tag=0U,({struct _fat_ptr _tmp4EB=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_pat2string(p));_tmp351.f1=_tmp4EB;});_tmp351;});void*_tmp26E[1U];_tmp26E[0]=& _tmp270;({struct _fat_ptr _tmp4EC=({const char*_tmp26F="get_access on bad aggr pattern: %s";_tag_fat(_tmp26F,sizeof(char),35U);});Cyc_aprintf(_tmp4EC,_tag_fat(_tmp26E,sizeof(void*),1U));});});_tmp4EE(_tmp4ED,_tag_fat(_tmp26D,sizeof(void*),0U));});});}_LL10:;}{
# 1438
struct Cyc_List_List*fields=((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->fields;
# 1446
int orig_i=i;
for(0;i != 0;-- i){fields=((struct Cyc_List_List*)_check_null(fields))->tl;}
return(void*)({struct Cyc_Tcpat_AggrField_Tcpat_Access_struct*_tmp272=_cycalloc(sizeof(*_tmp272));_tmp272->tag=4U,_tmp272->f1=((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->tagged,_tmp272->f2=((struct Cyc_Absyn_Aggrfield*)((struct Cyc_List_List*)_check_null(fields))->hd)->name;_tmp272;});}}}else{goto _LLE;}default: _LLE: _LLF:
({void*_tmp273=0U;({int(*_tmp4F2)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp277)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp277;});struct _fat_ptr _tmp4F1=(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp276=({struct Cyc_String_pa_PrintArg_struct _tmp352;_tmp352.tag=0U,({struct _fat_ptr _tmp4EF=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_pat2string(p));_tmp352.f1=_tmp4EF;});_tmp352;});void*_tmp274[1U];_tmp274[0]=& _tmp276;({struct _fat_ptr _tmp4F0=({const char*_tmp275="get_access on bad pattern: %s";_tag_fat(_tmp275,sizeof(char),30U);});Cyc_aprintf(_tmp4F0,_tag_fat(_tmp274,sizeof(void*),1U));});});_tmp4F2(_tmp4F1,_tag_fat(_tmp273,sizeof(void*),0U));});});}_LL5:;}}_LL0:;}struct _tuple24{struct Cyc_List_List*f1;struct Cyc_Tcpat_Con_s*f2;};
# 1375
static struct Cyc_List_List*Cyc_Tcpat_getoarg(struct _tuple24*env,int i){
# 1456
struct Cyc_Tcpat_Con_s*_tmp279;struct Cyc_List_List*_tmp278;_LL1: _tmp278=env->f1;_tmp279=env->f2;_LL2: {struct Cyc_List_List*obj=_tmp278;struct Cyc_Tcpat_Con_s*pcon=_tmp279;
void*acc=Cyc_Tcpat_get_access(pcon->orig_pat,i);
return({struct Cyc_List_List*_tmp27B=_cycalloc(sizeof(*_tmp27B));({struct Cyc_Tcpat_PathNode*_tmp4F3=({struct Cyc_Tcpat_PathNode*_tmp27A=_cycalloc(sizeof(*_tmp27A));_tmp27A->orig_pat=pcon->orig_pat,_tmp27A->access=acc;_tmp27A;});_tmp27B->hd=_tmp4F3;}),_tmp27B->tl=obj;_tmp27B;});}}
# 1375
static struct Cyc_List_List*Cyc_Tcpat_getoargs(struct Cyc_Tcpat_Con_s*pcon,struct Cyc_List_List*obj){
# 1461
struct _tuple24 env=({struct _tuple24 _tmp353;_tmp353.f1=obj,_tmp353.f2=pcon;_tmp353;});
return({struct Cyc_List_List*(*_tmp4F4)(int n,struct Cyc_List_List*(*f)(struct _tuple24*,int),struct _tuple24*env)=({struct Cyc_List_List*(*_tmp27C)(int n,struct Cyc_List_List*(*f)(struct _tuple24*,int),struct _tuple24*env)=(struct Cyc_List_List*(*)(int n,struct Cyc_List_List*(*f)(struct _tuple24*,int),struct _tuple24*env))Cyc_List_tabulate_c;_tmp27C;});_tmp4F4(pcon->arity,Cyc_Tcpat_getoarg,& env);});}
# 1375
static void*Cyc_Tcpat_match(void*p,struct Cyc_List_List*obj,void*dsc,struct Cyc_List_List*ctx,struct Cyc_List_List*work,struct Cyc_Tcpat_Rhs*right_hand_side,struct Cyc_List_List*rules){
# 1472
void*_tmp27D=p;struct Cyc_List_List*_tmp27F;struct Cyc_Tcpat_Con_s*_tmp27E;if(((struct Cyc_Tcpat_Any_Tcpat_Simple_pat_struct*)_tmp27D)->tag == 0U){_LL1: _LL2:
# 1474
 return({struct Cyc_List_List*_tmp4F7=Cyc_Tcpat_augment(ctx,dsc);struct Cyc_List_List*_tmp4F6=work;struct Cyc_Tcpat_Rhs*_tmp4F5=right_hand_side;Cyc_Tcpat_and_match(_tmp4F7,_tmp4F6,_tmp4F5,rules);});}else{_LL3: _tmp27E=((struct Cyc_Tcpat_Con_Tcpat_Simple_pat_struct*)_tmp27D)->f1;_tmp27F=((struct Cyc_Tcpat_Con_Tcpat_Simple_pat_struct*)_tmp27D)->f2;_LL4: {struct Cyc_Tcpat_Con_s*pcon=_tmp27E;struct Cyc_List_List*pargs=_tmp27F;
# 1476
enum Cyc_Tcpat_Answer _stmttmp2C=Cyc_Tcpat_static_match(pcon,dsc);enum Cyc_Tcpat_Answer _tmp280=_stmttmp2C;switch(_tmp280){case Cyc_Tcpat_Yes: _LL6: _LL7: {
# 1478
struct Cyc_List_List*ctx2=({struct Cyc_List_List*_tmp284=_cycalloc(sizeof(*_tmp284));({struct _tuple22*_tmp4F8=({struct _tuple22*_tmp283=_cycalloc(sizeof(*_tmp283));_tmp283->f1=pcon,_tmp283->f2=0;_tmp283;});_tmp284->hd=_tmp4F8;}),_tmp284->tl=ctx;_tmp284;});
struct _tuple21*work_frame=({struct _tuple21*_tmp282=_cycalloc(sizeof(*_tmp282));_tmp282->f1=pargs,({struct Cyc_List_List*_tmp4FA=Cyc_Tcpat_getoargs(pcon,obj);_tmp282->f2=_tmp4FA;}),({
struct Cyc_List_List*_tmp4F9=Cyc_Tcpat_getdargs(pcon,dsc);_tmp282->f3=_tmp4F9;});_tmp282;});
struct Cyc_List_List*work2=({struct Cyc_List_List*_tmp281=_cycalloc(sizeof(*_tmp281));_tmp281->hd=work_frame,_tmp281->tl=work;_tmp281;});
return Cyc_Tcpat_and_match(ctx2,work2,right_hand_side,rules);}case Cyc_Tcpat_No: _LL8: _LL9:
# 1484
 return({void*_tmp4FB=Cyc_Tcpat_build_desc(ctx,dsc,work);Cyc_Tcpat_or_match(_tmp4FB,rules);});default: _LLA: _LLB: {
# 1486
struct Cyc_List_List*ctx2=({struct Cyc_List_List*_tmp288=_cycalloc(sizeof(*_tmp288));({struct _tuple22*_tmp4FC=({struct _tuple22*_tmp287=_cycalloc(sizeof(*_tmp287));_tmp287->f1=pcon,_tmp287->f2=0;_tmp287;});_tmp288->hd=_tmp4FC;}),_tmp288->tl=ctx;_tmp288;});
struct _tuple21*work_frame=({struct _tuple21*_tmp286=_cycalloc(sizeof(*_tmp286));_tmp286->f1=pargs,({struct Cyc_List_List*_tmp4FE=Cyc_Tcpat_getoargs(pcon,obj);_tmp286->f2=_tmp4FE;}),({
struct Cyc_List_List*_tmp4FD=Cyc_Tcpat_getdargs(pcon,dsc);_tmp286->f3=_tmp4FD;});_tmp286;});
struct Cyc_List_List*work2=({struct Cyc_List_List*_tmp285=_cycalloc(sizeof(*_tmp285));_tmp285->hd=work_frame,_tmp285->tl=work;_tmp285;});
void*s=Cyc_Tcpat_and_match(ctx2,work2,right_hand_side,rules);
void*f=({void*_tmp501=({struct Cyc_List_List*_tmp500=ctx;void*_tmp4FF=Cyc_Tcpat_add_neg(dsc,pcon);Cyc_Tcpat_build_desc(_tmp500,_tmp4FF,work);});Cyc_Tcpat_or_match(_tmp501,rules);});
# 1493
return Cyc_Tcpat_ifeq(obj,pcon,s,f);}}_LL5:;}}_LL0:;}
# 1375
static void Cyc_Tcpat_check_exhaust_overlap(void*d,void(*not_exhaust)(void*,void*),void*env1,void(*rhs_appears)(void*,struct Cyc_Tcpat_Rhs*),void*env2,int*exhaust_done){
# 1508
void*_tmp289=d;void*_tmp28B;struct Cyc_List_List*_tmp28A;struct Cyc_Tcpat_Rhs*_tmp28C;void*_tmp28D;switch(*((int*)_tmp289)){case 0U: _LL1: _tmp28D=(void*)((struct Cyc_Tcpat_Failure_Tcpat_Decision_struct*)_tmp289)->f1;_LL2: {void*td=_tmp28D;
# 1510
if(!(*exhaust_done)){not_exhaust(env1,td);*exhaust_done=1;}goto _LL0;}case 1U: _LL3: _tmp28C=((struct Cyc_Tcpat_Success_Tcpat_Decision_struct*)_tmp289)->f1;_LL4: {struct Cyc_Tcpat_Rhs*r=_tmp28C;
# 1512
rhs_appears(env2,r);goto _LL0;}default: _LL5: _tmp28A=((struct Cyc_Tcpat_SwitchDec_Tcpat_Decision_struct*)_tmp289)->f2;_tmp28B=(void*)((struct Cyc_Tcpat_SwitchDec_Tcpat_Decision_struct*)_tmp289)->f3;_LL6: {struct Cyc_List_List*cases=_tmp28A;void*def=_tmp28B;
# 1514
for(0;cases != 0;cases=cases->tl){
struct _tuple0 _stmttmp2D=*((struct _tuple0*)cases->hd);void*_tmp28E;_LL8: _tmp28E=_stmttmp2D.f2;_LL9: {void*d=_tmp28E;
Cyc_Tcpat_check_exhaust_overlap(d,not_exhaust,env1,rhs_appears,env2,exhaust_done);}}
# 1519
Cyc_Tcpat_check_exhaust_overlap(def,not_exhaust,env1,rhs_appears,env2,exhaust_done);
# 1521
goto _LL0;}}_LL0:;}
# 1375
static struct _tuple23*Cyc_Tcpat_get_match(int*ctr,struct Cyc_Absyn_Switch_clause*swc){
# 1531
void*sp0=Cyc_Tcpat_compile_pat(swc->pattern);
struct Cyc_Tcpat_Rhs*rhs=({struct Cyc_Tcpat_Rhs*_tmp293=_cycalloc(sizeof(*_tmp293));_tmp293->used=0,_tmp293->pat_loc=(swc->pattern)->loc,_tmp293->rhs=swc->body;_tmp293;});
void*sp;
if(swc->where_clause != 0){
union Cyc_Tcpat_PatOrWhere w=({union Cyc_Tcpat_PatOrWhere _tmp354;(_tmp354.where_clause).tag=2U,(_tmp354.where_clause).val=swc->where_clause;_tmp354;});
sp=({struct Cyc_List_List*_tmp503=({void*_tmp28F[2U];_tmp28F[0]=sp0,({void*_tmp502=Cyc_Tcpat_int_pat(*ctr,w);_tmp28F[1]=_tmp502;});Cyc_List_list(_tag_fat(_tmp28F,sizeof(void*),2U));});Cyc_Tcpat_tuple_pat(_tmp503,w);});
*ctr=*ctr + 1;}else{
# 1539
sp=({struct Cyc_List_List*_tmp505=({void*_tmp290[2U];_tmp290[0]=sp0,({void*_tmp504=(void*)({struct Cyc_Tcpat_Any_Tcpat_Simple_pat_struct*_tmp291=_cycalloc(sizeof(*_tmp291));_tmp291->tag=0U;_tmp291;});_tmp290[1]=_tmp504;});Cyc_List_list(_tag_fat(_tmp290,sizeof(void*),2U));});Cyc_Tcpat_tuple_pat(_tmp505,({union Cyc_Tcpat_PatOrWhere _tmp355;(_tmp355.where_clause).tag=2U,(_tmp355.where_clause).val=0;_tmp355;}));});}
return({struct _tuple23*_tmp292=_cycalloc(sizeof(*_tmp292));_tmp292->f1=sp,_tmp292->f2=rhs;_tmp292;});}char Cyc_Tcpat_Desc2string[12U]="Desc2string";struct Cyc_Tcpat_Desc2string_exn_struct{char*tag;};
# 1545
struct Cyc_Tcpat_Desc2string_exn_struct Cyc_Tcpat_Desc2string_val={Cyc_Tcpat_Desc2string};
# 1547
static struct _fat_ptr Cyc_Tcpat_descs2string(struct Cyc_List_List*);
static struct _fat_ptr Cyc_Tcpat_desc2string(void*d){
void*_tmp294=d;struct Cyc_Set_Set*_tmp295;struct Cyc_List_List*_tmp297;struct Cyc_Tcpat_Con_s*_tmp296;if(((struct Cyc_Tcpat_Pos_Tcpat_Term_desc_struct*)_tmp294)->tag == 0U){_LL1: _tmp296=((struct Cyc_Tcpat_Pos_Tcpat_Term_desc_struct*)_tmp294)->f1;_tmp297=((struct Cyc_Tcpat_Pos_Tcpat_Term_desc_struct*)_tmp294)->f2;_LL2: {struct Cyc_Tcpat_Con_s*c=_tmp296;struct Cyc_List_List*ds=_tmp297;
# 1551
union Cyc_Tcpat_Name_value n=c->name;
struct Cyc_Absyn_Pat*p;
{union Cyc_Tcpat_PatOrWhere _stmttmp2E=c->orig_pat;union Cyc_Tcpat_PatOrWhere _tmp298=_stmttmp2E;struct Cyc_Absyn_Pat*_tmp299;if((_tmp298.where_clause).tag == 2){_LL6: _LL7:
 return Cyc_Tcpat_desc2string((void*)((struct Cyc_List_List*)_check_null(ds))->hd);}else{_LL8: _tmp299=(_tmp298.pattern).val;_LL9: {struct Cyc_Absyn_Pat*p2=_tmp299;
p=p2;goto _LL5;}}_LL5:;}{
# 1557
void*_stmttmp2F=p->r;void*_tmp29A=_stmttmp2F;struct Cyc_Absyn_Exp*_tmp29B;struct Cyc_Absyn_Enumfield*_tmp29C;struct Cyc_Absyn_Enumfield*_tmp29D;int _tmp29F;struct _fat_ptr _tmp29E;char _tmp2A0;int _tmp2A1;struct Cyc_Absyn_Datatypefield*_tmp2A2;struct Cyc_List_List*_tmp2A4;struct Cyc_Absyn_Aggrdecl*_tmp2A3;struct Cyc_Absyn_Vardecl*_tmp2A6;struct Cyc_Absyn_Tvar*_tmp2A5;struct Cyc_Absyn_Vardecl*_tmp2A7;struct Cyc_Absyn_Vardecl*_tmp2A8;switch(*((int*)_tmp29A)){case 0U: _LLB: _LLC:
 return({const char*_tmp2A9="_";_tag_fat(_tmp2A9,sizeof(char),2U);});case 1U: _LLD: _tmp2A8=((struct Cyc_Absyn_Var_p_Absyn_Raw_pat_struct*)_tmp29A)->f1;_LLE: {struct Cyc_Absyn_Vardecl*vd=_tmp2A8;
return Cyc_Absynpp_qvar2string(vd->name);}case 3U: _LLF: _tmp2A7=((struct Cyc_Absyn_Reference_p_Absyn_Raw_pat_struct*)_tmp29A)->f1;_LL10: {struct Cyc_Absyn_Vardecl*vd=_tmp2A7;
return(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp2AC=({struct Cyc_String_pa_PrintArg_struct _tmp356;_tmp356.tag=0U,({struct _fat_ptr _tmp506=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(vd->name));_tmp356.f1=_tmp506;});_tmp356;});void*_tmp2AA[1U];_tmp2AA[0]=& _tmp2AC;({struct _fat_ptr _tmp507=({const char*_tmp2AB="*%s";_tag_fat(_tmp2AB,sizeof(char),4U);});Cyc_aprintf(_tmp507,_tag_fat(_tmp2AA,sizeof(void*),1U));});});}case 4U: _LL11: _tmp2A5=((struct Cyc_Absyn_TagInt_p_Absyn_Raw_pat_struct*)_tmp29A)->f1;_tmp2A6=((struct Cyc_Absyn_TagInt_p_Absyn_Raw_pat_struct*)_tmp29A)->f2;_LL12: {struct Cyc_Absyn_Tvar*tv=_tmp2A5;struct Cyc_Absyn_Vardecl*vd=_tmp2A6;
return(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp2AF=({struct Cyc_String_pa_PrintArg_struct _tmp358;_tmp358.tag=0U,({struct _fat_ptr _tmp508=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(vd->name));_tmp358.f1=_tmp508;});_tmp358;});struct Cyc_String_pa_PrintArg_struct _tmp2B0=({struct Cyc_String_pa_PrintArg_struct _tmp357;_tmp357.tag=0U,_tmp357.f1=(struct _fat_ptr)((struct _fat_ptr)*tv->name);_tmp357;});void*_tmp2AD[2U];_tmp2AD[0]=& _tmp2AF,_tmp2AD[1]=& _tmp2B0;({struct _fat_ptr _tmp509=({const char*_tmp2AE="%s<`%s>";_tag_fat(_tmp2AE,sizeof(char),8U);});Cyc_aprintf(_tmp509,_tag_fat(_tmp2AD,sizeof(void*),2U));});});}case 5U: _LL13: _LL14:
# 1563
 return(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp2B3=({struct Cyc_String_pa_PrintArg_struct _tmp359;_tmp359.tag=0U,({struct _fat_ptr _tmp50A=(struct _fat_ptr)((struct _fat_ptr)Cyc_Tcpat_descs2string(ds));_tmp359.f1=_tmp50A;});_tmp359;});void*_tmp2B1[1U];_tmp2B1[0]=& _tmp2B3;({struct _fat_ptr _tmp50B=({const char*_tmp2B2="$(%s)";_tag_fat(_tmp2B2,sizeof(char),6U);});Cyc_aprintf(_tmp50B,_tag_fat(_tmp2B1,sizeof(void*),1U));});});case 6U: _LL15: _LL16:
# 1565
 return(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp2B6=({struct Cyc_String_pa_PrintArg_struct _tmp35A;_tmp35A.tag=0U,({struct _fat_ptr _tmp50C=(struct _fat_ptr)((struct _fat_ptr)Cyc_Tcpat_descs2string(ds));_tmp35A.f1=_tmp50C;});_tmp35A;});void*_tmp2B4[1U];_tmp2B4[0]=& _tmp2B6;({struct _fat_ptr _tmp50D=({const char*_tmp2B5="&%s";_tag_fat(_tmp2B5,sizeof(char),4U);});Cyc_aprintf(_tmp50D,_tag_fat(_tmp2B4,sizeof(void*),1U));});});case 7U: if(((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmp29A)->f1 != 0){if((((union Cyc_Absyn_AggrInfo*)((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmp29A)->f1)->KnownAggr).tag == 2){_LL17: _tmp2A3=*((((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmp29A)->f1)->KnownAggr).val;_tmp2A4=((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmp29A)->f3;_LL18: {struct Cyc_Absyn_Aggrdecl*ad=_tmp2A3;struct Cyc_List_List*dlps=_tmp2A4;
# 1567
if((int)ad->kind == (int)Cyc_Absyn_UnionA){
struct Cyc_List_List*_tmp2B7=dlps;struct _fat_ptr*_tmp2B8;if(_tmp2B7 != 0){if(((struct _tuple19*)((struct Cyc_List_List*)_tmp2B7)->hd)->f1 != 0){if(((struct Cyc_Absyn_FieldName_Absyn_Designator_struct*)((struct Cyc_List_List*)((struct _tuple19*)((struct Cyc_List_List*)_tmp2B7)->hd)->f1)->hd)->tag == 1U){_LL2C: _tmp2B8=((struct Cyc_Absyn_FieldName_Absyn_Designator_struct*)(((struct _tuple19*)_tmp2B7->hd)->f1)->hd)->f1;_LL2D: {struct _fat_ptr*f=_tmp2B8;
# 1570
return(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp2BB=({struct Cyc_String_pa_PrintArg_struct _tmp35D;_tmp35D.tag=0U,({struct _fat_ptr _tmp50E=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(ad->name));_tmp35D.f1=_tmp50E;});_tmp35D;});struct Cyc_String_pa_PrintArg_struct _tmp2BC=({struct Cyc_String_pa_PrintArg_struct _tmp35C;_tmp35C.tag=0U,_tmp35C.f1=(struct _fat_ptr)((struct _fat_ptr)*f);_tmp35C;});struct Cyc_String_pa_PrintArg_struct _tmp2BD=({struct Cyc_String_pa_PrintArg_struct _tmp35B;_tmp35B.tag=0U,({
struct _fat_ptr _tmp50F=(struct _fat_ptr)((struct _fat_ptr)Cyc_Tcpat_descs2string(ds));_tmp35B.f1=_tmp50F;});_tmp35B;});void*_tmp2B9[3U];_tmp2B9[0]=& _tmp2BB,_tmp2B9[1]=& _tmp2BC,_tmp2B9[2]=& _tmp2BD;({struct _fat_ptr _tmp510=({const char*_tmp2BA="%s{.%s = %s}";_tag_fat(_tmp2BA,sizeof(char),13U);});Cyc_aprintf(_tmp510,_tag_fat(_tmp2B9,sizeof(void*),3U));});});}}else{goto _LL2E;}}else{goto _LL2E;}}else{_LL2E: _LL2F:
 goto _LL2B;}_LL2B:;}
# 1567
return(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp2C0=({struct Cyc_String_pa_PrintArg_struct _tmp35F;_tmp35F.tag=0U,({
# 1575
struct _fat_ptr _tmp511=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(ad->name));_tmp35F.f1=_tmp511;});_tmp35F;});struct Cyc_String_pa_PrintArg_struct _tmp2C1=({struct Cyc_String_pa_PrintArg_struct _tmp35E;_tmp35E.tag=0U,({struct _fat_ptr _tmp512=(struct _fat_ptr)((struct _fat_ptr)Cyc_Tcpat_descs2string(ds));_tmp35E.f1=_tmp512;});_tmp35E;});void*_tmp2BE[2U];_tmp2BE[0]=& _tmp2C0,_tmp2BE[1]=& _tmp2C1;({struct _fat_ptr _tmp513=({const char*_tmp2BF="%s{%s}";_tag_fat(_tmp2BF,sizeof(char),7U);});Cyc_aprintf(_tmp513,_tag_fat(_tmp2BE,sizeof(void*),2U));});});}}else{goto _LL29;}}else{goto _LL29;}case 8U: _LL19: _tmp2A2=((struct Cyc_Absyn_Datatype_p_Absyn_Raw_pat_struct*)_tmp29A)->f2;_LL1A: {struct Cyc_Absyn_Datatypefield*tuf=_tmp2A2;
# 1577
if(ds == 0)
return(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp2C4=({struct Cyc_String_pa_PrintArg_struct _tmp360;_tmp360.tag=0U,({struct _fat_ptr _tmp514=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(tuf->name));_tmp360.f1=_tmp514;});_tmp360;});void*_tmp2C2[1U];_tmp2C2[0]=& _tmp2C4;({struct _fat_ptr _tmp515=({const char*_tmp2C3="%s";_tag_fat(_tmp2C3,sizeof(char),3U);});Cyc_aprintf(_tmp515,_tag_fat(_tmp2C2,sizeof(void*),1U));});});else{
# 1580
return(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp2C7=({struct Cyc_String_pa_PrintArg_struct _tmp362;_tmp362.tag=0U,({struct _fat_ptr _tmp516=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(tuf->name));_tmp362.f1=_tmp516;});_tmp362;});struct Cyc_String_pa_PrintArg_struct _tmp2C8=({struct Cyc_String_pa_PrintArg_struct _tmp361;_tmp361.tag=0U,({struct _fat_ptr _tmp517=(struct _fat_ptr)((struct _fat_ptr)Cyc_Tcpat_descs2string(ds));_tmp361.f1=_tmp517;});_tmp361;});void*_tmp2C5[2U];_tmp2C5[0]=& _tmp2C7,_tmp2C5[1]=& _tmp2C8;({struct _fat_ptr _tmp518=({const char*_tmp2C6="%s(%s)";_tag_fat(_tmp2C6,sizeof(char),7U);});Cyc_aprintf(_tmp518,_tag_fat(_tmp2C5,sizeof(void*),2U));});});}}case 9U: _LL1B: _LL1C:
 return({const char*_tmp2C9="NULL";_tag_fat(_tmp2C9,sizeof(char),5U);});case 10U: _LL1D: _tmp2A1=((struct Cyc_Absyn_Int_p_Absyn_Raw_pat_struct*)_tmp29A)->f2;_LL1E: {int i=_tmp2A1;
return(struct _fat_ptr)({struct Cyc_Int_pa_PrintArg_struct _tmp2CC=({struct Cyc_Int_pa_PrintArg_struct _tmp363;_tmp363.tag=1U,_tmp363.f1=(unsigned long)i;_tmp363;});void*_tmp2CA[1U];_tmp2CA[0]=& _tmp2CC;({struct _fat_ptr _tmp519=({const char*_tmp2CB="%d";_tag_fat(_tmp2CB,sizeof(char),3U);});Cyc_aprintf(_tmp519,_tag_fat(_tmp2CA,sizeof(void*),1U));});});}case 11U: _LL1F: _tmp2A0=((struct Cyc_Absyn_Char_p_Absyn_Raw_pat_struct*)_tmp29A)->f1;_LL20: {char c=_tmp2A0;
return(struct _fat_ptr)({struct Cyc_Int_pa_PrintArg_struct _tmp2CF=({struct Cyc_Int_pa_PrintArg_struct _tmp364;_tmp364.tag=1U,_tmp364.f1=(unsigned long)((int)c);_tmp364;});void*_tmp2CD[1U];_tmp2CD[0]=& _tmp2CF;({struct _fat_ptr _tmp51A=({const char*_tmp2CE="%d";_tag_fat(_tmp2CE,sizeof(char),3U);});Cyc_aprintf(_tmp51A,_tag_fat(_tmp2CD,sizeof(void*),1U));});});}case 12U: _LL21: _tmp29E=((struct Cyc_Absyn_Float_p_Absyn_Raw_pat_struct*)_tmp29A)->f1;_tmp29F=((struct Cyc_Absyn_Float_p_Absyn_Raw_pat_struct*)_tmp29A)->f2;_LL22: {struct _fat_ptr f=_tmp29E;int i=_tmp29F;
return f;}case 13U: _LL23: _tmp29D=((struct Cyc_Absyn_Enum_p_Absyn_Raw_pat_struct*)_tmp29A)->f2;_LL24: {struct Cyc_Absyn_Enumfield*ef=_tmp29D;
_tmp29C=ef;goto _LL26;}case 14U: _LL25: _tmp29C=((struct Cyc_Absyn_AnonEnum_p_Absyn_Raw_pat_struct*)_tmp29A)->f2;_LL26: {struct Cyc_Absyn_Enumfield*ef=_tmp29C;
return Cyc_Absynpp_qvar2string(ef->name);}case 17U: _LL27: _tmp29B=((struct Cyc_Absyn_Exp_p_Absyn_Raw_pat_struct*)_tmp29A)->f1;_LL28: {struct Cyc_Absyn_Exp*e=_tmp29B;
return Cyc_Absynpp_exp2string(e);}default: _LL29: _LL2A:
(int)_throw((void*)& Cyc_Tcpat_Desc2string_val);}_LLA:;}}}else{_LL3: _tmp295=((struct Cyc_Tcpat_Neg_Tcpat_Term_desc_struct*)_tmp294)->f1;_LL4: {struct Cyc_Set_Set*s=_tmp295;
# 1591
if(Cyc_Set_is_empty(s))return({const char*_tmp2D0="_";_tag_fat(_tmp2D0,sizeof(char),2U);});{struct Cyc_Tcpat_Con_s*c=({
struct Cyc_Tcpat_Con_s*(*_tmp51B)(struct Cyc_Set_Set*s)=({struct Cyc_Tcpat_Con_s*(*_tmp2EE)(struct Cyc_Set_Set*s)=(struct Cyc_Tcpat_Con_s*(*)(struct Cyc_Set_Set*s))Cyc_Set_choose;_tmp2EE;});_tmp51B(s);});
# 1595
if((int)(((c->orig_pat).where_clause).tag == 2))(int)_throw((void*)& Cyc_Tcpat_Desc2string_val);{struct Cyc_Absyn_Pat*orig_pat=({union Cyc_Tcpat_PatOrWhere _tmp2ED=c->orig_pat;if((_tmp2ED.pattern).tag != 1)_throw_match();(_tmp2ED.pattern).val;});
# 1597
void*_stmttmp30=Cyc_Tcutil_compress((void*)_check_null(orig_pat->topt));void*_tmp2D1=_stmttmp30;struct Cyc_Absyn_PtrAtts _tmp2D2;struct Cyc_Absyn_Aggrdecl*_tmp2D3;struct Cyc_Absyn_Datatypedecl*_tmp2D4;switch(*((int*)_tmp2D1)){case 0U: switch(*((int*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2D1)->f1)){case 1U: if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2D1)->f1)->f2 == Cyc_Absyn_Char_sz){_LL31: _LL32:
# 1600
{int i=0;for(0;i < 256;++ i){
struct Cyc_Tcpat_Con_s*c=Cyc_Tcpat_char_con((char)i,orig_pat);
if(!({int(*_tmp51D)(struct Cyc_Set_Set*s,struct Cyc_Tcpat_Con_s*elt)=({int(*_tmp2D5)(struct Cyc_Set_Set*s,struct Cyc_Tcpat_Con_s*elt)=(int(*)(struct Cyc_Set_Set*s,struct Cyc_Tcpat_Con_s*elt))Cyc_Set_member;_tmp2D5;});struct Cyc_Set_Set*_tmp51C=s;_tmp51D(_tmp51C,c);}))
return(struct _fat_ptr)({struct Cyc_Int_pa_PrintArg_struct _tmp2D8=({struct Cyc_Int_pa_PrintArg_struct _tmp365;_tmp365.tag=1U,_tmp365.f1=(unsigned long)i;_tmp365;});void*_tmp2D6[1U];_tmp2D6[0]=& _tmp2D8;({struct _fat_ptr _tmp51E=({const char*_tmp2D7="%d";_tag_fat(_tmp2D7,sizeof(char),3U);});Cyc_aprintf(_tmp51E,_tag_fat(_tmp2D6,sizeof(void*),1U));});});}}
# 1605
(int)_throw((void*)& Cyc_Tcpat_Desc2string_val);}else{_LL33: _LL34:
# 1608
{unsigned i=0U;for(0;i < (unsigned)-1;++ i){
struct Cyc_Tcpat_Con_s*_tmp2D9=Cyc_Tcpat_int_con((int)i,c->orig_pat);struct Cyc_Tcpat_Con_s*c=_tmp2D9;
if(!({int(*_tmp520)(struct Cyc_Set_Set*s,struct Cyc_Tcpat_Con_s*elt)=({int(*_tmp2DA)(struct Cyc_Set_Set*s,struct Cyc_Tcpat_Con_s*elt)=(int(*)(struct Cyc_Set_Set*s,struct Cyc_Tcpat_Con_s*elt))Cyc_Set_member;_tmp2DA;});struct Cyc_Set_Set*_tmp51F=s;_tmp520(_tmp51F,c);}))
return(struct _fat_ptr)({struct Cyc_Int_pa_PrintArg_struct _tmp2DD=({struct Cyc_Int_pa_PrintArg_struct _tmp366;_tmp366.tag=1U,_tmp366.f1=(unsigned long)((int)i);_tmp366;});void*_tmp2DB[1U];_tmp2DB[0]=& _tmp2DD;({struct _fat_ptr _tmp521=({const char*_tmp2DC="%d";_tag_fat(_tmp2DC,sizeof(char),3U);});Cyc_aprintf(_tmp521,_tag_fat(_tmp2DB,sizeof(void*),1U));});});}}
# 1613
(int)_throw((void*)& Cyc_Tcpat_Desc2string_val);}case 18U: if(((((struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2D1)->f1)->f1).KnownDatatype).tag == 2){_LL37: _tmp2D4=*((((struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2D1)->f1)->f1).KnownDatatype).val;_LL38: {struct Cyc_Absyn_Datatypedecl*tud=_tmp2D4;
# 1623
if(tud->is_extensible)(int)_throw((void*)& Cyc_Tcpat_Desc2string_val);{struct Cyc_List_List*fields=(struct Cyc_List_List*)((struct Cyc_Core_Opt*)_check_null(tud->fields))->v;
# 1625
int span=Cyc_List_length(fields);
for(0;(unsigned)fields;fields=fields->tl){
struct _fat_ptr n=*(*((struct Cyc_Absyn_Datatypefield*)fields->hd)->name).f2;
struct Cyc_List_List*args=((struct Cyc_Absyn_Datatypefield*)fields->hd)->typs;
if(!({int(*_tmp524)(struct Cyc_Set_Set*s,struct Cyc_Tcpat_Con_s*elt)=({int(*_tmp2E1)(struct Cyc_Set_Set*s,struct Cyc_Tcpat_Con_s*elt)=(int(*)(struct Cyc_Set_Set*s,struct Cyc_Tcpat_Con_s*elt))Cyc_Set_member;_tmp2E1;});struct Cyc_Set_Set*_tmp523=s;_tmp524(_tmp523,({struct Cyc_Tcpat_Con_s*_tmp2E2=_cycalloc(sizeof(*_tmp2E2));({union Cyc_Tcpat_Name_value _tmp522=Cyc_Tcpat_Name_v(n);_tmp2E2->name=_tmp522;}),_tmp2E2->arity=0,_tmp2E2->span=0,_tmp2E2->orig_pat=c->orig_pat;_tmp2E2;}));})){
if(Cyc_List_length(args)== 0)
return n;else{
# 1633
return(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp2E5=({struct Cyc_String_pa_PrintArg_struct _tmp367;_tmp367.tag=0U,_tmp367.f1=(struct _fat_ptr)((struct _fat_ptr)n);_tmp367;});void*_tmp2E3[1U];_tmp2E3[0]=& _tmp2E5;({struct _fat_ptr _tmp525=({const char*_tmp2E4="%s(...)";_tag_fat(_tmp2E4,sizeof(char),8U);});Cyc_aprintf(_tmp525,_tag_fat(_tmp2E3,sizeof(void*),1U));});});}}}
# 1636
(int)_throw((void*)& Cyc_Tcpat_Desc2string_val);}}}else{goto _LL3B;}case 20U: if(((((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2D1)->f1)->f1).KnownAggr).tag == 2){_LL39: _tmp2D3=*((((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2D1)->f1)->f1).KnownAggr).val;if((int)_tmp2D3->kind == (int)Cyc_Absyn_UnionA){_LL3A: {struct Cyc_Absyn_Aggrdecl*ad=_tmp2D3;
# 1638
struct Cyc_List_List*fields=((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->fields;
int span=Cyc_List_length(fields);
struct _tuple2*_stmttmp31=ad->name;struct _fat_ptr _tmp2E6;_LL3E: _tmp2E6=*_stmttmp31->f2;_LL3F: {struct _fat_ptr union_name=_tmp2E6;
for(0;(unsigned)fields;fields=fields->tl){
struct _fat_ptr n=*((struct Cyc_Absyn_Aggrfield*)fields->hd)->name;
if(!({int(*_tmp528)(struct Cyc_Set_Set*s,struct Cyc_Tcpat_Con_s*elt)=({int(*_tmp2E7)(struct Cyc_Set_Set*s,struct Cyc_Tcpat_Con_s*elt)=(int(*)(struct Cyc_Set_Set*s,struct Cyc_Tcpat_Con_s*elt))Cyc_Set_member;_tmp2E7;});struct Cyc_Set_Set*_tmp527=s;_tmp528(_tmp527,({struct Cyc_Tcpat_Con_s*_tmp2E8=_cycalloc(sizeof(*_tmp2E8));({union Cyc_Tcpat_Name_value _tmp526=Cyc_Tcpat_Name_v(n);_tmp2E8->name=_tmp526;}),_tmp2E8->arity=0,_tmp2E8->span=0,_tmp2E8->orig_pat=c->orig_pat;_tmp2E8;}));}))
return(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp2EB=({struct Cyc_String_pa_PrintArg_struct _tmp369;_tmp369.tag=0U,_tmp369.f1=(struct _fat_ptr)((struct _fat_ptr)union_name);_tmp369;});struct Cyc_String_pa_PrintArg_struct _tmp2EC=({struct Cyc_String_pa_PrintArg_struct _tmp368;_tmp368.tag=0U,_tmp368.f1=(struct _fat_ptr)((struct _fat_ptr)n);_tmp368;});void*_tmp2E9[2U];_tmp2E9[0]=& _tmp2EB,_tmp2E9[1]=& _tmp2EC;({struct _fat_ptr _tmp529=({const char*_tmp2EA="%s{.%s = _}";_tag_fat(_tmp2EA,sizeof(char),12U);});Cyc_aprintf(_tmp529,_tag_fat(_tmp2E9,sizeof(void*),2U));});});}
# 1646
(int)_throw((void*)& Cyc_Tcpat_Desc2string_val);}}}else{goto _LL3B;}}else{goto _LL3B;}default: goto _LL3B;}case 3U: _LL35: _tmp2D2=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp2D1)->f1).ptr_atts;_LL36: {struct Cyc_Absyn_PtrAtts patts=_tmp2D2;
# 1615
void*n=patts.nullable;
int is_nullable=Cyc_Tcutil_force_type2bool(0,n);
if(is_nullable){
if(!({int(*_tmp52B)(struct Cyc_Set_Set*s,struct Cyc_Tcpat_Con_s*elt)=({int(*_tmp2DE)(struct Cyc_Set_Set*s,struct Cyc_Tcpat_Con_s*elt)=(int(*)(struct Cyc_Set_Set*s,struct Cyc_Tcpat_Con_s*elt))Cyc_Set_member;_tmp2DE;});struct Cyc_Set_Set*_tmp52A=s;_tmp52B(_tmp52A,Cyc_Tcpat_null_con(orig_pat));}))
return({const char*_tmp2DF="NULL";_tag_fat(_tmp2DF,sizeof(char),5U);});}
# 1617
return({const char*_tmp2E0="&_";_tag_fat(_tmp2E0,sizeof(char),3U);});}default: _LL3B: _LL3C:
# 1647
(int)_throw((void*)& Cyc_Tcpat_Desc2string_val);}_LL30:;}}}}_LL0:;}
# 1548
static struct _fat_ptr*Cyc_Tcpat_desc2stringptr(void*d){
# 1652
return({struct _fat_ptr*_tmp2EF=_cycalloc(sizeof(*_tmp2EF));({struct _fat_ptr _tmp52C=Cyc_Tcpat_desc2string(d);*_tmp2EF=_tmp52C;});_tmp2EF;});}
# 1548
static struct _fat_ptr Cyc_Tcpat_descs2string(struct Cyc_List_List*ds){
# 1655
struct Cyc_List_List*ss=({struct Cyc_List_List*(*_tmp52D)(struct _fat_ptr*(*f)(void*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp2F3)(struct _fat_ptr*(*f)(void*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct _fat_ptr*(*f)(void*),struct Cyc_List_List*x))Cyc_List_map;_tmp2F3;});_tmp52D(Cyc_Tcpat_desc2stringptr,ds);});
struct _fat_ptr*comma=({struct _fat_ptr*_tmp2F2=_cycalloc(sizeof(*_tmp2F2));({struct _fat_ptr _tmp52E=({const char*_tmp2F1=",";_tag_fat(_tmp2F1,sizeof(char),2U);});*_tmp2F2=_tmp52E;});_tmp2F2;});
{struct Cyc_List_List*x=ss;for(0;x != 0;x=((struct Cyc_List_List*)_check_null(x))->tl){
if(x->tl != 0){
({struct Cyc_List_List*_tmp52F=({struct Cyc_List_List*_tmp2F0=_cycalloc(sizeof(*_tmp2F0));_tmp2F0->hd=comma,_tmp2F0->tl=x->tl;_tmp2F0;});x->tl=_tmp52F;});
x=x->tl;}}}
# 1663
return(struct _fat_ptr)Cyc_strconcat_l(ss);}
# 1548
static void Cyc_Tcpat_not_exhaust_err(unsigned loc,void*desc){
# 1667
struct _handler_cons _tmp2F4;_push_handler(& _tmp2F4);{int _tmp2F6=0;if(setjmp(_tmp2F4.handler))_tmp2F6=1;if(!_tmp2F6){
{struct _fat_ptr s=Cyc_Tcpat_desc2string(desc);
({struct Cyc_String_pa_PrintArg_struct _tmp2F9=({struct Cyc_String_pa_PrintArg_struct _tmp36A;_tmp36A.tag=0U,_tmp36A.f1=(struct _fat_ptr)((struct _fat_ptr)s);_tmp36A;});void*_tmp2F7[1U];_tmp2F7[0]=& _tmp2F9;({unsigned _tmp531=loc;struct _fat_ptr _tmp530=({const char*_tmp2F8="patterns may not be exhaustive.\n\tmissing case for %s";_tag_fat(_tmp2F8,sizeof(char),53U);});Cyc_Tcutil_terr(_tmp531,_tmp530,_tag_fat(_tmp2F7,sizeof(void*),1U));});});}
# 1668
;_pop_handler();}else{void*_tmp2F5=(void*)Cyc_Core_get_exn_thrown();void*_tmp2FA=_tmp2F5;void*_tmp2FB;if(((struct Cyc_Tcpat_Desc2string_exn_struct*)_tmp2FA)->tag == Cyc_Tcpat_Desc2string){_LL1: _LL2:
# 1672
({void*_tmp2FC=0U;({unsigned _tmp533=loc;struct _fat_ptr _tmp532=({const char*_tmp2FD="patterns may not be exhaustive.";_tag_fat(_tmp2FD,sizeof(char),32U);});Cyc_Tcutil_terr(_tmp533,_tmp532,_tag_fat(_tmp2FC,sizeof(void*),0U));});});
goto _LL0;}else{_LL3: _tmp2FB=_tmp2FA;_LL4: {void*exn=_tmp2FB;(int)_rethrow(exn);}}_LL0:;}}}
# 1548
static void Cyc_Tcpat_rule_occurs(int dummy,struct Cyc_Tcpat_Rhs*rhs){
# 1677
rhs->used=1;}
# 1548
void Cyc_Tcpat_check_switch_exhaustive(unsigned loc,struct Cyc_Tcenv_Tenv*te,struct Cyc_List_List*swcs,void**dopt){
# 1686
int where_ctr=0;
int*env=& where_ctr;
struct Cyc_List_List*match_rules=({struct Cyc_List_List*(*_tmp535)(struct _tuple23*(*f)(int*,struct Cyc_Absyn_Switch_clause*),int*env,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp303)(struct _tuple23*(*f)(int*,struct Cyc_Absyn_Switch_clause*),int*env,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct _tuple23*(*f)(int*,struct Cyc_Absyn_Switch_clause*),int*env,struct Cyc_List_List*x))Cyc_List_map_c;_tmp303;});int*_tmp534=env;_tmp535(Cyc_Tcpat_get_match,_tmp534,swcs);});
void*dec_tree=Cyc_Tcpat_match_compile(match_rules);
*dopt=dec_tree;{
# 1692
int ex_done=0;
({void(*_tmp537)(void*d,void(*not_exhaust)(unsigned,void*),unsigned env1,void(*rhs_appears)(int,struct Cyc_Tcpat_Rhs*),int env2,int*exhaust_done)=({void(*_tmp2FE)(void*d,void(*not_exhaust)(unsigned,void*),unsigned env1,void(*rhs_appears)(int,struct Cyc_Tcpat_Rhs*),int env2,int*exhaust_done)=(void(*)(void*d,void(*not_exhaust)(unsigned,void*),unsigned env1,void(*rhs_appears)(int,struct Cyc_Tcpat_Rhs*),int env2,int*exhaust_done))Cyc_Tcpat_check_exhaust_overlap;_tmp2FE;});void*_tmp536=dec_tree;_tmp537(_tmp536,Cyc_Tcpat_not_exhaust_err,loc,Cyc_Tcpat_rule_occurs,0,& ex_done);});
# 1695
for(0;match_rules != 0;match_rules=match_rules->tl){
struct _tuple23*_stmttmp32=(struct _tuple23*)match_rules->hd;unsigned _tmp300;int _tmp2FF;_LL1: _tmp2FF=(_stmttmp32->f2)->used;_tmp300=(_stmttmp32->f2)->pat_loc;_LL2: {int b=_tmp2FF;unsigned loc2=_tmp300;
if(!b){
({void*_tmp301=0U;({unsigned _tmp539=loc2;struct _fat_ptr _tmp538=({const char*_tmp302="redundant pattern (check for misspelled constructors in earlier patterns)";_tag_fat(_tmp302,sizeof(char),74U);});Cyc_Tcutil_terr(_tmp539,_tmp538,_tag_fat(_tmp301,sizeof(void*),0U));});});
# 1700
break;}}}}}
# 1548
static void Cyc_Tcpat_not_exhaust_warn(struct _tuple15*pr,void*desc){
# 1710
(*pr).f2=0;{
struct _handler_cons _tmp304;_push_handler(& _tmp304);{int _tmp306=0;if(setjmp(_tmp304.handler))_tmp306=1;if(!_tmp306){
{struct _fat_ptr s=Cyc_Tcpat_desc2string(desc);
({struct Cyc_String_pa_PrintArg_struct _tmp309=({struct Cyc_String_pa_PrintArg_struct _tmp36B;_tmp36B.tag=0U,_tmp36B.f1=(struct _fat_ptr)((struct _fat_ptr)s);_tmp36B;});void*_tmp307[1U];_tmp307[0]=& _tmp309;({unsigned _tmp53B=(*pr).f1;struct _fat_ptr _tmp53A=({const char*_tmp308="pattern not exhaustive.\n\tmissing case for %s";_tag_fat(_tmp308,sizeof(char),45U);});Cyc_Tcutil_warn(_tmp53B,_tmp53A,_tag_fat(_tmp307,sizeof(void*),1U));});});}
# 1712
;_pop_handler();}else{void*_tmp305=(void*)Cyc_Core_get_exn_thrown();void*_tmp30A=_tmp305;void*_tmp30B;if(((struct Cyc_Tcpat_Desc2string_exn_struct*)_tmp30A)->tag == Cyc_Tcpat_Desc2string){_LL1: _LL2:
# 1716
({void*_tmp30C=0U;({unsigned _tmp53D=(*pr).f1;struct _fat_ptr _tmp53C=({const char*_tmp30D="pattern not exhaustive.";_tag_fat(_tmp30D,sizeof(char),24U);});Cyc_Tcutil_warn(_tmp53D,_tmp53C,_tag_fat(_tmp30C,sizeof(void*),0U));});});
goto _LL0;}else{_LL3: _tmp30B=_tmp30A;_LL4: {void*exn=_tmp30B;(int)_rethrow(exn);}}_LL0:;}}}}
# 1548
static void Cyc_Tcpat_dummy_fn(int i,struct Cyc_Tcpat_Rhs*rhs){
# 1721
return;}
# 1548
int Cyc_Tcpat_check_let_pat_exhaustive(unsigned loc,struct Cyc_Tcenv_Tenv*te,struct Cyc_Absyn_Pat*p,void**dopt){
# 1725
struct Cyc_Tcpat_Rhs*rhs=({struct Cyc_Tcpat_Rhs*_tmp311=_cycalloc(sizeof(*_tmp311));_tmp311->used=0,_tmp311->pat_loc=p->loc,({struct Cyc_Absyn_Stmt*_tmp53E=Cyc_Absyn_skip_stmt(0U);_tmp311->rhs=_tmp53E;});_tmp311;});
struct Cyc_List_List*match_rules=({struct Cyc_List_List*_tmp310=_cycalloc(sizeof(*_tmp310));({struct _tuple23*_tmp540=({struct _tuple23*_tmp30F=_cycalloc(sizeof(*_tmp30F));({void*_tmp53F=Cyc_Tcpat_compile_pat(p);_tmp30F->f1=_tmp53F;}),_tmp30F->f2=rhs;_tmp30F;});_tmp310->hd=_tmp540;}),_tmp310->tl=0;_tmp310;});
void*dec_tree=Cyc_Tcpat_match_compile(match_rules);
struct _tuple15 exhaust_env=({struct _tuple15 _tmp36C;_tmp36C.f1=loc,_tmp36C.f2=1;_tmp36C;});
int ex_done=0;
({void(*_tmp541)(void*d,void(*not_exhaust)(struct _tuple15*,void*),struct _tuple15*env1,void(*rhs_appears)(int,struct Cyc_Tcpat_Rhs*),int env2,int*exhaust_done)=({void(*_tmp30E)(void*d,void(*not_exhaust)(struct _tuple15*,void*),struct _tuple15*env1,void(*rhs_appears)(int,struct Cyc_Tcpat_Rhs*),int env2,int*exhaust_done)=(void(*)(void*d,void(*not_exhaust)(struct _tuple15*,void*),struct _tuple15*env1,void(*rhs_appears)(int,struct Cyc_Tcpat_Rhs*),int env2,int*exhaust_done))Cyc_Tcpat_check_exhaust_overlap;_tmp30E;});_tmp541(dec_tree,Cyc_Tcpat_not_exhaust_warn,& exhaust_env,Cyc_Tcpat_dummy_fn,0,& ex_done);});
# 1732
*dopt=dec_tree;
return exhaust_env.f2;}
# 1548
static struct _tuple23*Cyc_Tcpat_get_match2(struct Cyc_Absyn_Switch_clause*swc){
# 1741
void*sp0=Cyc_Tcpat_compile_pat(swc->pattern);
# 1744
if(swc->where_clause != 0)
({void*_tmp312=0U;({unsigned _tmp543=((struct Cyc_Absyn_Exp*)_check_null(swc->where_clause))->loc;struct _fat_ptr _tmp542=({const char*_tmp313="&&-clauses not supported in exception handlers.";_tag_fat(_tmp313,sizeof(char),48U);});Cyc_Tcutil_terr(_tmp543,_tmp542,_tag_fat(_tmp312,sizeof(void*),0U));});});{
# 1744
struct Cyc_Tcpat_Rhs*rhs=({struct Cyc_Tcpat_Rhs*_tmp315=_cycalloc(sizeof(*_tmp315));
# 1747
_tmp315->used=0,_tmp315->pat_loc=(swc->pattern)->loc,_tmp315->rhs=swc->body;_tmp315;});
return({struct _tuple23*_tmp314=_cycalloc(sizeof(*_tmp314));_tmp314->f1=sp0,_tmp314->f2=rhs;_tmp314;});}}
# 1548
static void Cyc_Tcpat_not_exhaust_err2(unsigned loc,void*d){
# 1750
;}
# 1548
void Cyc_Tcpat_check_catch_overlap(unsigned loc,struct Cyc_Tcenv_Tenv*te,struct Cyc_List_List*swcs,void**dopt){
# 1757
struct Cyc_List_List*match_rules=({struct Cyc_List_List*(*_tmp544)(struct _tuple23*(*f)(struct Cyc_Absyn_Switch_clause*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp31B)(struct _tuple23*(*f)(struct Cyc_Absyn_Switch_clause*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct _tuple23*(*f)(struct Cyc_Absyn_Switch_clause*),struct Cyc_List_List*x))Cyc_List_map;_tmp31B;});_tmp544(Cyc_Tcpat_get_match2,swcs);});
void*dec_tree=Cyc_Tcpat_match_compile(match_rules);
*dopt=dec_tree;{
int ex_done=0;
({void(*_tmp546)(void*d,void(*not_exhaust)(unsigned,void*),unsigned env1,void(*rhs_appears)(int,struct Cyc_Tcpat_Rhs*),int env2,int*exhaust_done)=({void(*_tmp316)(void*d,void(*not_exhaust)(unsigned,void*),unsigned env1,void(*rhs_appears)(int,struct Cyc_Tcpat_Rhs*),int env2,int*exhaust_done)=(void(*)(void*d,void(*not_exhaust)(unsigned,void*),unsigned env1,void(*rhs_appears)(int,struct Cyc_Tcpat_Rhs*),int env2,int*exhaust_done))Cyc_Tcpat_check_exhaust_overlap;_tmp316;});void*_tmp545=dec_tree;_tmp546(_tmp545,Cyc_Tcpat_not_exhaust_err2,loc,Cyc_Tcpat_rule_occurs,0,& ex_done);});
# 1763
for(0;match_rules != 0;match_rules=match_rules->tl){
# 1765
if(match_rules->tl == 0)break;{struct _tuple23*_stmttmp33=(struct _tuple23*)match_rules->hd;unsigned _tmp318;int _tmp317;_LL1: _tmp317=(_stmttmp33->f2)->used;_tmp318=(_stmttmp33->f2)->pat_loc;_LL2: {int b=_tmp317;unsigned loc2=_tmp318;
# 1767
if(!b){
({void*_tmp319=0U;({unsigned _tmp548=loc2;struct _fat_ptr _tmp547=({const char*_tmp31A="redundant pattern (check for misspelled constructors in earlier patterns)";_tag_fat(_tmp31A,sizeof(char),74U);});Cyc_Tcutil_terr(_tmp548,_tmp547,_tag_fat(_tmp319,sizeof(void*),0U));});});
break;}}}}}}
# 1548
int Cyc_Tcpat_has_vars(struct Cyc_Core_Opt*pat_vars){
# 1775
{struct Cyc_List_List*l=(struct Cyc_List_List*)((struct Cyc_Core_Opt*)_check_null(pat_vars))->v;for(0;l != 0;l=l->tl){
if((*((struct _tuple16*)l->hd)).f1 != 0)
return 1;}}
# 1775
return 0;}
