#include <setjmp.h>
/* This is a C header file to be used by the output of the Cyclone to
   C translator.  The corresponding definitions are in file lib/runtime_*.c */
#ifndef _CYC_INCLUDE_H_
#define _CYC_INCLUDE_H_

/* Need one of these per thread (see runtime_stack.c). The runtime maintains 
   a stack that contains either _handler_cons structs or _RegionHandle structs.
   The tag is 0 for a handler_cons and 1 for a region handle.  */
struct _RuntimeStack {
  int tag; 
  struct _RuntimeStack *next;
  void (*cleanup)(struct _RuntimeStack *frame);
};

#ifndef offsetof
/* should be size_t, but int is fine. */
#define offsetof(t,n) ((int)(&(((t *)0)->n)))
#endif

/* Fat pointers */
struct _fat_ptr {
  unsigned char *curr; 
  unsigned char *base; 
  unsigned char *last_plus_one; 
};  

/* Discriminated Unions */
struct _xtunion_struct { char *tag; };

/* Regions */
struct _RegionPage
#ifdef CYC_REGION_PROFILE
{ unsigned total_bytes;
  unsigned free_bytes;
  /* MWH: wish we didn't have to include the stuff below ... */
  struct _RegionPage *next;
  char data[1];
}
#endif
; // abstract -- defined in runtime_memory.c
struct _RegionHandle {
  struct _RuntimeStack s;
  struct _RegionPage *curr;
  char               *offset;
  char               *last_plus_one;
  struct _DynRegionHandle *sub_regions;
#ifdef CYC_REGION_PROFILE
  const char         *name;
#else
  unsigned used_bytes;
  unsigned wasted_bytes;
#endif
};
struct _DynRegionFrame {
  struct _RuntimeStack s;
  struct _DynRegionHandle *x;
};
// A dynamic region is just a region handle.  The wrapper struct is for type
// abstraction.
struct Cyc_Core_DynamicRegion {
  struct _RegionHandle h;
};

struct _RegionHandle _new_region(const char*);
void* _region_malloc(struct _RegionHandle*, unsigned);
void* _region_calloc(struct _RegionHandle*, unsigned t, unsigned n);
void   _free_region(struct _RegionHandle*);
struct _RegionHandle*_open_dynregion(struct _DynRegionFrame*,struct _DynRegionHandle*);
void   _pop_dynregion();

/* Exceptions */
struct _handler_cons {
  struct _RuntimeStack s;
  jmp_buf handler;
};
void _push_handler(struct _handler_cons *);
void _push_region(struct _RegionHandle *);
void _npop_handler(int);
void _pop_handler();
void _pop_region();

#ifndef _throw
void* _throw_null_fn(const char*,unsigned);
void* _throw_arraybounds_fn(const char*,unsigned);
void* _throw_badalloc_fn(const char*,unsigned);
void* _throw_match_fn(const char*,unsigned);
void* _throw_fn(void*,const char*,unsigned);
void* _rethrow(void*);
#define _throw_null() (_throw_null_fn(__FILE__,__LINE__))
#define _throw_arraybounds() (_throw_arraybounds_fn(__FILE__,__LINE__))
#define _throw_badalloc() (_throw_badalloc_fn(__FILE__,__LINE__))
#define _throw_match() (_throw_match_fn(__FILE__,__LINE__))
#define _throw(e) (_throw_fn((e),__FILE__,__LINE__))
#endif

struct _xtunion_struct* Cyc_Core_get_exn_thrown();
/* Built-in Exceptions */
struct Cyc_Null_Exception_exn_struct { char *tag; };
struct Cyc_Array_bounds_exn_struct { char *tag; };
struct Cyc_Match_Exception_exn_struct { char *tag; };
struct Cyc_Bad_alloc_exn_struct { char *tag; };
extern char Cyc_Null_Exception[];
extern char Cyc_Array_bounds[];
extern char Cyc_Match_Exception[];
extern char Cyc_Bad_alloc[];

/* Built-in Run-time Checks and company */
#ifdef CYC_ANSI_OUTPUT
#define _INLINE  
#else
#define _INLINE inline
#endif

#ifdef NO_CYC_NULL_CHECKS
#define _check_null(ptr) (ptr)
#else
#define _check_null(ptr) \
  ({ void*_cks_null = (void*)(ptr); \
     if (!_cks_null) _throw_null(); \
     _cks_null; })
#endif

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index)\
   (((char*)ptr) + (elt_sz)*(index))
#ifdef NO_CYC_NULL_CHECKS
#define _check_known_subscript_null _check_known_subscript_notnull
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr);\
  int _index = (index);\
  if (!_cks_ptr) _throw_null(); \
  _cks_ptr + (elt_sz)*_index; })
#endif
#define _zero_arr_plus_fn(orig_x,orig_sz,orig_i,f,l) ((orig_x)+(orig_i))
#define _zero_arr_plus_char_fn _zero_arr_plus_fn
#define _zero_arr_plus_short_fn _zero_arr_plus_fn
#define _zero_arr_plus_int_fn _zero_arr_plus_fn
#define _zero_arr_plus_float_fn _zero_arr_plus_fn
#define _zero_arr_plus_double_fn _zero_arr_plus_fn
#define _zero_arr_plus_longdouble_fn _zero_arr_plus_fn
#define _zero_arr_plus_voidstar_fn _zero_arr_plus_fn
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (!_cks_ptr) _throw_null(); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })

/* _zero_arr_plus_*_fn(x,sz,i,filename,lineno) adds i to zero-terminated ptr
   x that has at least sz elements */
char* _zero_arr_plus_char_fn(char*,unsigned,int,const char*,unsigned);
short* _zero_arr_plus_short_fn(short*,unsigned,int,const char*,unsigned);
int* _zero_arr_plus_int_fn(int*,unsigned,int,const char*,unsigned);
float* _zero_arr_plus_float_fn(float*,unsigned,int,const char*,unsigned);
double* _zero_arr_plus_double_fn(double*,unsigned,int,const char*,unsigned);
long double* _zero_arr_plus_longdouble_fn(long double*,unsigned,int,const char*, unsigned);
void** _zero_arr_plus_voidstar_fn(void**,unsigned,int,const char*,unsigned);
#endif

/* _get_zero_arr_size_*(x,sz) returns the number of elements in a
   zero-terminated array that is NULL or has at least sz elements */
int _get_zero_arr_size_char(const char*,unsigned);
int _get_zero_arr_size_short(const short*,unsigned);
int _get_zero_arr_size_int(const int*,unsigned);
int _get_zero_arr_size_float(const float*,unsigned);
int _get_zero_arr_size_double(const double*,unsigned);
int _get_zero_arr_size_longdouble(const long double*,unsigned);
int _get_zero_arr_size_voidstar(const void**,unsigned);

/* _zero_arr_inplace_plus_*_fn(x,i,filename,lineno) sets
   zero-terminated pointer *x to *x + i */
char* _zero_arr_inplace_plus_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_short_fn(short**,int,const char*,unsigned);
int* _zero_arr_inplace_plus_int(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_longdouble_fn(long double**,int,const char*,unsigned);
void** _zero_arr_inplace_plus_voidstar_fn(void***,int,const char*,unsigned);
/* like the previous functions, but does post-addition (as in e++) */
char* _zero_arr_inplace_plus_post_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_post_short_fn(short**x,int,const char*,unsigned);
int* _zero_arr_inplace_plus_post_int_fn(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_post_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_post_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_post_longdouble_fn(long double**,int,const char *,unsigned);
void** _zero_arr_inplace_plus_post_voidstar_fn(void***,int,const char*,unsigned);
#define _zero_arr_plus_char(x,s,i) \
  (_zero_arr_plus_char_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_short(x,s,i) \
  (_zero_arr_plus_short_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_int(x,s,i) \
  (_zero_arr_plus_int_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_float(x,s,i) \
  (_zero_arr_plus_float_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_double(x,s,i) \
  (_zero_arr_plus_double_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_longdouble(x,s,i) \
  (_zero_arr_plus_longdouble_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_voidstar(x,s,i) \
  (_zero_arr_plus_voidstar_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_inplace_plus_char(x,i) \
  _zero_arr_inplace_plus_char_fn((char **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_short(x,i) \
  _zero_arr_inplace_plus_short_fn((short **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_int(x,i) \
  _zero_arr_inplace_plus_int_fn((int **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_float(x,i) \
  _zero_arr_inplace_plus_float_fn((float **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_double(x,i) \
  _zero_arr_inplace_plus_double_fn((double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_longdouble(x,i) \
  _zero_arr_inplace_plus_longdouble_fn((long double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_voidstar(x,i) \
  _zero_arr_inplace_plus_voidstar_fn((void ***)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_char(x,i) \
  _zero_arr_inplace_plus_post_char_fn((char **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_short(x,i) \
  _zero_arr_inplace_plus_post_short_fn((short **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_int(x,i) \
  _zero_arr_inplace_plus_post_int_fn((int **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_float(x,i) \
  _zero_arr_inplace_plus_post_float_fn((float **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_double(x,i) \
  _zero_arr_inplace_plus_post_double_fn((double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_longdouble(x,i) \
  _zero_arr_inplace_plus_post_longdouble_fn((long double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_voidstar(x,i) \
  _zero_arr_inplace_plus_post_voidstar_fn((void***)(x),(i),__FILE__,__LINE__)

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_fat_subscript(arr,elt_sz,index) ((arr).curr + (elt_sz) * (index))
#define _untag_fat_ptr(arr,elt_sz,num_elts) ((arr).curr)
#else
#define _check_fat_subscript(arr,elt_sz,index) ({ \
  struct _fat_ptr _cus_arr = (arr); \
  unsigned char *_cus_ans = _cus_arr.curr + (elt_sz) * (index); \
  /* JGM: not needed! if (!_cus_arr.base) _throw_null();*/ \
  if (_cus_ans < _cus_arr.base || _cus_ans >= _cus_arr.last_plus_one) \
    _throw_arraybounds(); \
  _cus_ans; })
#define _untag_fat_ptr(arr,elt_sz,num_elts) ({ \
  struct _fat_ptr _arr = (arr); \
  unsigned char *_curr = _arr.curr; \
  if ((_curr < _arr.base || _curr + (elt_sz) * (num_elts) > _arr.last_plus_one) &&\
      _curr != (unsigned char *)0) \
    _throw_arraybounds(); \
  _curr; })
#endif

#define _tag_fat(tcurr,elt_sz,num_elts) ({ \
  struct _fat_ptr _ans; \
  unsigned _num_elts = (num_elts);\
  _ans.base = _ans.curr = (void*)(tcurr); \
  /* JGM: if we're tagging NULL, ignore num_elts */ \
  _ans.last_plus_one = _ans.base ? (_ans.base + (elt_sz) * _num_elts) : 0; \
  _ans; })

#define _get_fat_size(arr,elt_sz) \
  ({struct _fat_ptr _arr = (arr); \
    unsigned char *_arr_curr=_arr.curr; \
    unsigned char *_arr_last=_arr.last_plus_one; \
    (_arr_curr < _arr.base || _arr_curr >= _arr_last) ? 0 : \
    ((_arr_last - _arr_curr) / (elt_sz));})

#define _fat_ptr_plus(arr,elt_sz,change) ({ \
  struct _fat_ptr _ans = (arr); \
  int _change = (change);\
  _ans.curr += (elt_sz) * _change;\
  _ans; })
#define _fat_ptr_inplace_plus(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  _arr_ptr->curr += (elt_sz) * (change);\
  *_arr_ptr; })
#define _fat_ptr_inplace_plus_post(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  struct _fat_ptr _ans = *_arr_ptr; \
  _arr_ptr->curr += (elt_sz) * (change);\
  _ans; })

//Not a macro since initialization order matters. Defined in runtime_zeroterm.c.
struct _fat_ptr _fat_ptr_decrease_size(struct _fat_ptr,unsigned sz,unsigned numelts);

/* Allocation */
void* GC_malloc(int);
void* GC_malloc_atomic(int);
void* GC_calloc(unsigned,unsigned);
void* GC_calloc_atomic(unsigned,unsigned);
// bound the allocation size to be < MAX_ALLOC_SIZE. See macros below for usage.
#define MAX_MALLOC_SIZE (1 << 28)
void* _bounded_GC_malloc(int,const char*,int);
void* _bounded_GC_malloc_atomic(int,const char*,int);
void* _bounded_GC_calloc(unsigned,unsigned,const char*,int);
void* _bounded_GC_calloc_atomic(unsigned,unsigned,const char*,int);
/* these macros are overridden below ifdef CYC_REGION_PROFILE */
#ifndef CYC_REGION_PROFILE
#define _cycalloc(n) _bounded_GC_malloc(n,__FILE__,__LINE__)
#define _cycalloc_atomic(n) _bounded_GC_malloc_atomic(n,__FILE__,__LINE__)
#define _cyccalloc(n,s) _bounded_GC_calloc(n,s,__FILE__,__LINE__)
#define _cyccalloc_atomic(n,s) _bounded_GC_calloc_atomic(n,s,__FILE__,__LINE__)
#endif

static _INLINE unsigned int _check_times(unsigned x, unsigned y) {
  unsigned long long whole_ans = 
    ((unsigned long long) x)*((unsigned long long)y);
  unsigned word_ans = (unsigned)whole_ans;
  if(word_ans < whole_ans || word_ans > MAX_MALLOC_SIZE)
    _throw_badalloc();
  return word_ans;
}

#define _CYC_MAX_REGION_CONST 2
#define _CYC_MIN_ALIGNMENT (sizeof(double))

#ifdef CYC_REGION_PROFILE
extern int rgn_total_bytes;
#endif

static _INLINE void *_fast_region_malloc(struct _RegionHandle *r, unsigned orig_s) {  
  if (r > (struct _RegionHandle *)_CYC_MAX_REGION_CONST && r->curr != 0) { 
#ifdef CYC_NOALIGN
    unsigned s =  orig_s;
#else
    unsigned s =  (orig_s + _CYC_MIN_ALIGNMENT - 1) & (~(_CYC_MIN_ALIGNMENT -1)); 
#endif
    char *result; 
    result = r->offset; 
    if (s <= (r->last_plus_one - result)) {
      r->offset = result + s; 
#ifdef CYC_REGION_PROFILE
    r->curr->free_bytes = r->curr->free_bytes - s;
    rgn_total_bytes += s;
#endif
      return result;
    }
  } 
  return _region_malloc(r,orig_s); 
}

#ifdef CYC_REGION_PROFILE
/* see macros below for usage. defined in runtime_memory.c */
void* _profile_GC_malloc(int,const char*,const char*,int);
void* _profile_GC_malloc_atomic(int,const char*,const char*,int);
void* _profile_GC_calloc(unsigned,unsigned,const char*,const char*,int);
void* _profile_GC_calloc_atomic(unsigned,unsigned,const char*,const char*,int);
void* _profile_region_malloc(struct _RegionHandle*,unsigned,const char*,const char*,int);
void* _profile_region_calloc(struct _RegionHandle*,unsigned,unsigned,const char *,const char*,int);
struct _RegionHandle _profile_new_region(const char*,const char*,const char*,int);
void _profile_free_region(struct _RegionHandle*,const char*,const char*,int);
#ifndef RUNTIME_CYC
#define _new_region(n) _profile_new_region(n,__FILE__,__FUNCTION__,__LINE__)
#define _free_region(r) _profile_free_region(r,__FILE__,__FUNCTION__,__LINE__)
#define _region_malloc(rh,n) _profile_region_malloc(rh,n,__FILE__,__FUNCTION__,__LINE__)
#define _region_calloc(rh,n,t) _profile_region_calloc(rh,n,t,__FILE__,__FUNCTION__,__LINE__)
#  endif
#define _cycalloc(n) _profile_GC_malloc(n,__FILE__,__FUNCTION__,__LINE__)
#define _cycalloc_atomic(n) _profile_GC_malloc_atomic(n,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc(n,s) _profile_GC_calloc(n,s,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc_atomic(n,s) _profile_GC_calloc_atomic(n,s,__FILE__,__FUNCTION__,__LINE__)
#endif
#endif
 struct Cyc_Core_Opt{void*v;};struct _tuple0{void*f1;void*f2;};
# 108 "core.h"
void*Cyc_Core_fst(struct _tuple0*);extern char Cyc_Core_Invalid_argument[17U];struct Cyc_Core_Invalid_argument_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Failure[8U];struct Cyc_Core_Failure_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Impossible[11U];struct Cyc_Core_Impossible_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Not_found[10U];struct Cyc_Core_Not_found_exn_struct{char*tag;};extern char Cyc_Core_Unreachable[12U];struct Cyc_Core_Unreachable_exn_struct{char*tag;struct _fat_ptr f1;};
# 165
extern struct _RegionHandle*Cyc_Core_heap_region;
# 168
extern struct _RegionHandle*Cyc_Core_unique_region;struct Cyc_Core_DynamicRegion;struct Cyc_Core_NewDynamicRegion{struct Cyc_Core_DynamicRegion*key;};struct Cyc_Core_ThinRes{void*arr;unsigned nelts;};struct Cyc_List_List{void*hd;struct Cyc_List_List*tl;};
# 61 "list.h"
int Cyc_List_length(struct Cyc_List_List*x);
# 70
struct Cyc_List_List*Cyc_List_copy(struct Cyc_List_List*x);
# 76
struct Cyc_List_List*Cyc_List_map(void*(*f)(void*),struct Cyc_List_List*x);extern char Cyc_List_List_mismatch[14U];struct Cyc_List_List_mismatch_exn_struct{char*tag;};
# 135
void Cyc_List_iter_c(void(*f)(void*,void*),void*env,struct Cyc_List_List*x);
# 178
struct Cyc_List_List*Cyc_List_imp_rev(struct Cyc_List_List*x);
# 195
struct Cyc_List_List*Cyc_List_imp_append(struct Cyc_List_List*x,struct Cyc_List_List*y);extern char Cyc_List_Nth[4U];struct Cyc_List_Nth_exn_struct{char*tag;};
# 242
void*Cyc_List_nth(struct Cyc_List_List*x,int n);
# 322
int Cyc_List_mem(int(*compare)(void*,void*),struct Cyc_List_List*l,void*x);struct Cyc___cycFILE;struct Cyc_String_pa_PrintArg_struct{int tag;struct _fat_ptr f1;};struct Cyc_Int_pa_PrintArg_struct{int tag;unsigned long f1;};struct Cyc_Double_pa_PrintArg_struct{int tag;double f1;};struct Cyc_LongDouble_pa_PrintArg_struct{int tag;long double f1;};struct Cyc_ShortPtr_pa_PrintArg_struct{int tag;short*f1;};struct Cyc_IntPtr_pa_PrintArg_struct{int tag;unsigned long*f1;};struct Cyc_ShortPtr_sa_ScanfArg_struct{int tag;short*f1;};struct Cyc_UShortPtr_sa_ScanfArg_struct{int tag;unsigned short*f1;};struct Cyc_IntPtr_sa_ScanfArg_struct{int tag;int*f1;};struct Cyc_UIntPtr_sa_ScanfArg_struct{int tag;unsigned*f1;};struct Cyc_StringPtr_sa_ScanfArg_struct{int tag;struct _fat_ptr f1;};struct Cyc_DoublePtr_sa_ScanfArg_struct{int tag;double*f1;};struct Cyc_FloatPtr_sa_ScanfArg_struct{int tag;float*f1;};struct Cyc_CharPtr_sa_ScanfArg_struct{int tag;struct _fat_ptr f1;};extern char Cyc_FileCloseError[15U];struct Cyc_FileCloseError_exn_struct{char*tag;};extern char Cyc_FileOpenError[14U];struct Cyc_FileOpenError_exn_struct{char*tag;struct _fat_ptr f1;};
# 49 "string.h"
int Cyc_strcmp(struct _fat_ptr s1,struct _fat_ptr s2);
int Cyc_strptrcmp(struct _fat_ptr*s1,struct _fat_ptr*s2);struct Cyc_PP_Ppstate;struct Cyc_PP_Out;struct Cyc_PP_Doc;struct Cyc_Position_Error;struct Cyc_Relations_Reln;struct _union_Nmspace_Rel_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_Abs_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_C_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_Loc_n{int tag;int val;};union Cyc_Absyn_Nmspace{struct _union_Nmspace_Rel_n Rel_n;struct _union_Nmspace_Abs_n Abs_n;struct _union_Nmspace_C_n C_n;struct _union_Nmspace_Loc_n Loc_n;};struct _tuple1{union Cyc_Absyn_Nmspace f1;struct _fat_ptr*f2;};
# 161 "absyn.h"
enum Cyc_Absyn_Scope{Cyc_Absyn_Static =0U,Cyc_Absyn_Abstract =1U,Cyc_Absyn_Public =2U,Cyc_Absyn_Extern =3U,Cyc_Absyn_ExternC =4U,Cyc_Absyn_Register =5U};struct Cyc_Absyn_Tqual{int print_const: 1;int q_volatile: 1;int q_restrict: 1;int real_const: 1;unsigned loc;};
# 182
enum Cyc_Absyn_Size_of{Cyc_Absyn_Char_sz =0U,Cyc_Absyn_Short_sz =1U,Cyc_Absyn_Int_sz =2U,Cyc_Absyn_Long_sz =3U,Cyc_Absyn_LongLong_sz =4U};
enum Cyc_Absyn_Sign{Cyc_Absyn_Signed =0U,Cyc_Absyn_Unsigned =1U,Cyc_Absyn_None =2U};
enum Cyc_Absyn_AggrKind{Cyc_Absyn_StructA =0U,Cyc_Absyn_UnionA =1U};
# 187
enum Cyc_Absyn_AliasQual{Cyc_Absyn_Aliasable =0U,Cyc_Absyn_Unique =1U,Cyc_Absyn_Top =2U};
# 192
enum Cyc_Absyn_KindQual{Cyc_Absyn_AnyKind =0U,Cyc_Absyn_MemKind =1U,Cyc_Absyn_BoxKind =2U,Cyc_Absyn_RgnKind =3U,Cyc_Absyn_EffKind =4U,Cyc_Absyn_IntKind =5U,Cyc_Absyn_BoolKind =6U,Cyc_Absyn_PtrBndKind =7U};struct Cyc_Absyn_Kind{enum Cyc_Absyn_KindQual kind;enum Cyc_Absyn_AliasQual aliasqual;};struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct{int tag;struct Cyc_Absyn_Kind*f1;};struct Cyc_Absyn_Unknown_kb_Absyn_KindBound_struct{int tag;struct Cyc_Core_Opt*f1;};struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct{int tag;struct Cyc_Core_Opt*f1;struct Cyc_Absyn_Kind*f2;};struct Cyc_Absyn_Tvar{struct _fat_ptr*name;int identity;void*kind;};struct Cyc_Absyn_PtrLoc{unsigned ptr_loc;unsigned rgn_loc;unsigned zt_loc;};struct Cyc_Absyn_PtrAtts{void*rgn;void*nullable;void*bounds;void*zero_term;struct Cyc_Absyn_PtrLoc*ptrloc;};struct Cyc_Absyn_PtrInfo{void*elt_type;struct Cyc_Absyn_Tqual elt_tq;struct Cyc_Absyn_PtrAtts ptr_atts;};struct Cyc_Absyn_VarargInfo{struct _fat_ptr*name;struct Cyc_Absyn_Tqual tq;void*type;int inject;};struct Cyc_Absyn_Star_Absyn_Cap_struct{int tag;};struct Cyc_Absyn_Bot_Absyn_Cap_struct{int tag;};struct Cyc_Absyn_Nat_Absyn_Cap_struct{int tag;struct Cyc_Absyn_Exp*f1;int f2;};struct Cyc_Absyn_RgnEffect{void*name;void*rgn;void*lock;void*unadopt;void*parent;};struct Cyc_Absyn_FnInfo{struct Cyc_List_List*tvars;void*effect;struct Cyc_Absyn_Tqual ret_tqual;void*ret_type;struct Cyc_List_List*args;int c_varargs;struct Cyc_Absyn_VarargInfo*cyc_varargs;struct Cyc_List_List*rgn_po;struct Cyc_List_List*attributes;struct Cyc_Absyn_Exp*requires_clause;struct Cyc_List_List*requires_relns;struct Cyc_Absyn_Exp*ensures_clause;struct Cyc_List_List*ensures_relns;struct Cyc_List_List*ieffect;struct Cyc_List_List*oeffect;};struct Cyc_Absyn_UnknownDatatypeInfo{struct _tuple1*name;int is_extensible;};struct _union_DatatypeInfo_UnknownDatatype{int tag;struct Cyc_Absyn_UnknownDatatypeInfo val;};struct _union_DatatypeInfo_KnownDatatype{int tag;struct Cyc_Absyn_Datatypedecl**val;};union Cyc_Absyn_DatatypeInfo{struct _union_DatatypeInfo_UnknownDatatype UnknownDatatype;struct _union_DatatypeInfo_KnownDatatype KnownDatatype;};
# 312
union Cyc_Absyn_DatatypeInfo Cyc_Absyn_KnownDatatype(struct Cyc_Absyn_Datatypedecl**);struct Cyc_Absyn_UnknownDatatypeFieldInfo{struct _tuple1*datatype_name;struct _tuple1*field_name;int is_extensible;};struct _union_DatatypeFieldInfo_UnknownDatatypefield{int tag;struct Cyc_Absyn_UnknownDatatypeFieldInfo val;};struct _tuple2{struct Cyc_Absyn_Datatypedecl*f1;struct Cyc_Absyn_Datatypefield*f2;};struct _union_DatatypeFieldInfo_KnownDatatypefield{int tag;struct _tuple2 val;};union Cyc_Absyn_DatatypeFieldInfo{struct _union_DatatypeFieldInfo_UnknownDatatypefield UnknownDatatypefield;struct _union_DatatypeFieldInfo_KnownDatatypefield KnownDatatypefield;};
# 325
union Cyc_Absyn_DatatypeFieldInfo Cyc_Absyn_KnownDatatypefield(struct Cyc_Absyn_Datatypedecl*,struct Cyc_Absyn_Datatypefield*);struct _tuple3{enum Cyc_Absyn_AggrKind f1;struct _tuple1*f2;struct Cyc_Core_Opt*f3;};struct _union_AggrInfo_UnknownAggr{int tag;struct _tuple3 val;};struct _union_AggrInfo_KnownAggr{int tag;struct Cyc_Absyn_Aggrdecl**val;};union Cyc_Absyn_AggrInfo{struct _union_AggrInfo_UnknownAggr UnknownAggr;struct _union_AggrInfo_KnownAggr KnownAggr;};
# 332
union Cyc_Absyn_AggrInfo Cyc_Absyn_KnownAggr(struct Cyc_Absyn_Aggrdecl**);struct Cyc_Absyn_ArrayInfo{void*elt_type;struct Cyc_Absyn_Tqual tq;struct Cyc_Absyn_Exp*num_elts;void*zero_term;unsigned zt_loc;};struct Cyc_Absyn_Aggr_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Aggrdecl*f1;};struct Cyc_Absyn_Enum_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;};struct Cyc_Absyn_Datatype_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;};struct Cyc_Absyn_TypeDecl{void*r;unsigned loc;};struct Cyc_Absyn_VoidCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_IntCon_Absyn_TyCon_struct{int tag;enum Cyc_Absyn_Sign f1;enum Cyc_Absyn_Size_of f2;};struct Cyc_Absyn_FloatCon_Absyn_TyCon_struct{int tag;int f1;};struct Cyc_Absyn_RgnHandleCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_TagCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_HeapCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_UniqueCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_RefCntCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_AccessCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_JoinCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_RgnsCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_TrueCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_FalseCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_ThinCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_FatCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_EnumCon_Absyn_TyCon_struct{int tag;struct _tuple1*f1;struct Cyc_Absyn_Enumdecl*f2;};struct Cyc_Absyn_AnonEnumCon_Absyn_TyCon_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_BuiltinCon_Absyn_TyCon_struct{int tag;struct _fat_ptr f1;struct Cyc_Absyn_Kind*f2;};struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_DatatypeInfo f1;};struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_DatatypeFieldInfo f1;};struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_AggrInfo f1;};struct Cyc_Absyn_AppType_Absyn_Type_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Evar_Absyn_Type_struct{int tag;struct Cyc_Core_Opt*f1;void*f2;int f3;struct Cyc_Core_Opt*f4;};struct Cyc_Absyn_VarType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Tvar*f1;};struct Cyc_Absyn_PointerType_Absyn_Type_struct{int tag;struct Cyc_Absyn_PtrInfo f1;};struct Cyc_Absyn_ArrayType_Absyn_Type_struct{int tag;struct Cyc_Absyn_ArrayInfo f1;};struct Cyc_Absyn_FnType_Absyn_Type_struct{int tag;struct Cyc_Absyn_FnInfo f1;};struct Cyc_Absyn_TupleType_Absyn_Type_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct{int tag;enum Cyc_Absyn_AggrKind f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_TypedefType_Absyn_Type_struct{int tag;struct _tuple1*f1;struct Cyc_List_List*f2;struct Cyc_Absyn_Typedefdecl*f3;void*f4;};struct Cyc_Absyn_ValueofType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct{int tag;struct Cyc_Absyn_TypeDecl*f1;void**f2;};struct Cyc_Absyn_TypeofType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_NoTypes_Absyn_Funcparams_struct{int tag;struct Cyc_List_List*f1;unsigned f2;};struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct{int tag;struct Cyc_List_List*f1;int f2;struct Cyc_Absyn_VarargInfo*f3;void*f4;struct Cyc_List_List*f5;struct Cyc_Absyn_Exp*f6;struct Cyc_Absyn_Exp*f7;struct Cyc_List_List*f8;struct Cyc_List_List*f9;};
# 437 "absyn.h"
enum Cyc_Absyn_Format_Type{Cyc_Absyn_Printf_ft =0U,Cyc_Absyn_Scanf_ft =1U};struct Cyc_Absyn_Regparm_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Stdcall_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Cdecl_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Fastcall_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Noreturn_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Const_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Aligned_att_Absyn_Attribute_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Packed_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Section_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Nocommon_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Shared_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Unused_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Weak_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Dllimport_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Dllexport_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_instrument_function_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Constructor_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Destructor_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_check_memory_usage_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Format_att_Absyn_Attribute_struct{int tag;enum Cyc_Absyn_Format_Type f1;int f2;int f3;};struct Cyc_Absyn_Initializes_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Noliveunique_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Consume_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Pure_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Mode_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Alias_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Always_inline_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_throw_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Non_null_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Deprecated_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Carray_mod_Absyn_Type_modifier_struct{int tag;void*f1;unsigned f2;};struct Cyc_Absyn_ConstArray_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_Absyn_Exp*f1;void*f2;unsigned f3;};struct Cyc_Absyn_Pointer_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_Absyn_PtrAtts f1;struct Cyc_Absyn_Tqual f2;};struct Cyc_Absyn_Function_mod_Absyn_Type_modifier_struct{int tag;void*f1;};struct Cyc_Absyn_TypeParams_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_List_List*f1;unsigned f2;int f3;};struct Cyc_Absyn_Attributes_mod_Absyn_Type_modifier_struct{int tag;unsigned f1;struct Cyc_List_List*f2;};struct _union_Cnst_Null_c{int tag;int val;};struct _tuple4{enum Cyc_Absyn_Sign f1;char f2;};struct _union_Cnst_Char_c{int tag;struct _tuple4 val;};struct _union_Cnst_Wchar_c{int tag;struct _fat_ptr val;};struct _tuple5{enum Cyc_Absyn_Sign f1;short f2;};struct _union_Cnst_Short_c{int tag;struct _tuple5 val;};struct _tuple6{enum Cyc_Absyn_Sign f1;int f2;};struct _union_Cnst_Int_c{int tag;struct _tuple6 val;};struct _tuple7{enum Cyc_Absyn_Sign f1;long long f2;};struct _union_Cnst_LongLong_c{int tag;struct _tuple7 val;};struct _tuple8{struct _fat_ptr f1;int f2;};struct _union_Cnst_Float_c{int tag;struct _tuple8 val;};struct _union_Cnst_String_c{int tag;struct _fat_ptr val;};struct _union_Cnst_Wstring_c{int tag;struct _fat_ptr val;};union Cyc_Absyn_Cnst{struct _union_Cnst_Null_c Null_c;struct _union_Cnst_Char_c Char_c;struct _union_Cnst_Wchar_c Wchar_c;struct _union_Cnst_Short_c Short_c;struct _union_Cnst_Int_c Int_c;struct _union_Cnst_LongLong_c LongLong_c;struct _union_Cnst_Float_c Float_c;struct _union_Cnst_String_c String_c;struct _union_Cnst_Wstring_c Wstring_c;};
# 532
enum Cyc_Absyn_Primop{Cyc_Absyn_Plus =0U,Cyc_Absyn_Times =1U,Cyc_Absyn_Minus =2U,Cyc_Absyn_Div =3U,Cyc_Absyn_Mod =4U,Cyc_Absyn_Eq =5U,Cyc_Absyn_Neq =6U,Cyc_Absyn_Gt =7U,Cyc_Absyn_Lt =8U,Cyc_Absyn_Gte =9U,Cyc_Absyn_Lte =10U,Cyc_Absyn_Not =11U,Cyc_Absyn_Bitnot =12U,Cyc_Absyn_Bitand =13U,Cyc_Absyn_Bitor =14U,Cyc_Absyn_Bitxor =15U,Cyc_Absyn_Bitlshift =16U,Cyc_Absyn_Bitlrshift =17U,Cyc_Absyn_Numelts =18U};
# 539
enum Cyc_Absyn_Incrementor{Cyc_Absyn_PreInc =0U,Cyc_Absyn_PostInc =1U,Cyc_Absyn_PreDec =2U,Cyc_Absyn_PostDec =3U};struct Cyc_Absyn_VarargCallInfo{int num_varargs;struct Cyc_List_List*injectors;struct Cyc_Absyn_VarargInfo*vai;};struct Cyc_Absyn_StructField_Absyn_OffsetofField_struct{int tag;struct _fat_ptr*f1;};struct Cyc_Absyn_TupleIndex_Absyn_OffsetofField_struct{int tag;unsigned f1;};
# 557
enum Cyc_Absyn_Coercion{Cyc_Absyn_Unknown_coercion =0U,Cyc_Absyn_No_coercion =1U,Cyc_Absyn_Null_to_NonNull =2U,Cyc_Absyn_Other_coercion =3U};struct Cyc_Absyn_MallocInfo{int is_calloc;struct Cyc_Absyn_Exp*rgn;void**elt_type;struct Cyc_Absyn_Exp*num_elts;int fat_result;int inline_call;};struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct{int tag;union Cyc_Absyn_Cnst f1;};struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Pragma_e_Absyn_Raw_exp_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct{int tag;enum Cyc_Absyn_Primop f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_AssignOp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Core_Opt*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_Increment_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;enum Cyc_Absyn_Incrementor f2;};struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;struct Cyc_Absyn_VarargCallInfo*f3;int f4;};struct Cyc_Absyn_Throw_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;int f2;};struct Cyc_Absyn_NoInstantiate_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Instantiate_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_Absyn_Exp*f2;int f3;enum Cyc_Absyn_Coercion f4;};struct Cyc_Absyn_Address_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_New_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Sizeoftype_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Sizeofexp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Offsetof_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Deref_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;int f3;int f4;};struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;int f3;int f4;};struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Tuple_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;};struct _tuple9{struct _fat_ptr*f1;struct Cyc_Absyn_Tqual f2;void*f3;};struct Cyc_Absyn_CompoundLit_e_Absyn_Raw_exp_struct{int tag;struct _tuple9*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Array_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Exp*f2;struct Cyc_Absyn_Exp*f3;int f4;};struct Cyc_Absyn_ComprehensionNoinit_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;void*f2;int f3;};struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct{int tag;struct _tuple1*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;struct Cyc_Absyn_Aggrdecl*f4;};struct Cyc_Absyn_AnonStruct_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Datatype_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;struct Cyc_Absyn_Datatypedecl*f2;struct Cyc_Absyn_Datatypefield*f3;};struct Cyc_Absyn_Enum_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_AnonEnum_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_Spawn_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_MallocInfo f1;};struct Cyc_Absyn_Swap_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_UnresolvedMem_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Core_Opt*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_StmtExp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Stmt*f1;};struct Cyc_Absyn_Tagcheck_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;};struct Cyc_Absyn_Valueof_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Asm_e_Absyn_Raw_exp_struct{int tag;int f1;struct _fat_ptr f2;struct Cyc_List_List*f3;struct Cyc_List_List*f4;struct Cyc_List_List*f5;};struct Cyc_Absyn_Extension_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Exp{void*topt;void*r;unsigned loc;void*annot;};struct Cyc_Absyn_Skip_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Exp_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Seq_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Return_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_IfThenElse_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Stmt*f2;struct Cyc_Absyn_Stmt*f3;};struct _tuple10{struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_While_s_Absyn_Raw_stmt_struct{int tag;struct _tuple10 f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Break_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Continue_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Goto_s_Absyn_Raw_stmt_struct{int tag;struct _fat_ptr*f1;};struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _tuple10 f2;struct _tuple10 f3;struct Cyc_Absyn_Stmt*f4;};struct Cyc_Absyn_Switch_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;void*f3;};struct Cyc_Absyn_Fallthru_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_List_List*f1;struct Cyc_Absyn_Switch_clause**f2;};struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Decl*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Label_s_Absyn_Raw_stmt_struct{int tag;struct _fat_ptr*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Do_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct _tuple10 f2;};struct Cyc_Absyn_TryCatch_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct Cyc_List_List*f2;void*f3;};struct Cyc_Absyn_Stmt{void*r;unsigned loc;void*annot;};struct Cyc_Absyn_Wild_p_Absyn_Raw_pat_struct{int tag;};struct Cyc_Absyn_Var_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Pat*f2;};struct Cyc_Absyn_AliasVar_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;};struct Cyc_Absyn_Reference_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Pat*f2;};struct Cyc_Absyn_TagInt_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;};struct Cyc_Absyn_Tuple_p_Absyn_Raw_pat_struct{int tag;struct Cyc_List_List*f1;int f2;};struct Cyc_Absyn_Pointer_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Pat*f1;};struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct{int tag;union Cyc_Absyn_AggrInfo*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;int f4;};struct Cyc_Absyn_Datatype_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;struct Cyc_Absyn_Datatypefield*f2;struct Cyc_List_List*f3;int f4;};struct Cyc_Absyn_Null_p_Absyn_Raw_pat_struct{int tag;};struct Cyc_Absyn_Int_p_Absyn_Raw_pat_struct{int tag;enum Cyc_Absyn_Sign f1;int f2;};struct Cyc_Absyn_Char_p_Absyn_Raw_pat_struct{int tag;char f1;};struct Cyc_Absyn_Float_p_Absyn_Raw_pat_struct{int tag;struct _fat_ptr f1;int f2;};struct Cyc_Absyn_Enum_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_AnonEnum_p_Absyn_Raw_pat_struct{int tag;void*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_UnknownId_p_Absyn_Raw_pat_struct{int tag;struct _tuple1*f1;};struct Cyc_Absyn_UnknownCall_p_Absyn_Raw_pat_struct{int tag;struct _tuple1*f1;struct Cyc_List_List*f2;int f3;};struct Cyc_Absyn_Exp_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Pat{void*r;void*topt;unsigned loc;};struct Cyc_Absyn_Switch_clause{struct Cyc_Absyn_Pat*pattern;struct Cyc_Core_Opt*pat_vars;struct Cyc_Absyn_Exp*where_clause;struct Cyc_Absyn_Stmt*body;unsigned loc;};struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct{int tag;struct _tuple1*f1;};struct Cyc_Absyn_Global_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Funname_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Fndecl*f1;};struct Cyc_Absyn_Param_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Local_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Pat_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Vardecl{enum Cyc_Absyn_Scope sc;struct _tuple1*name;unsigned varloc;struct Cyc_Absyn_Tqual tq;void*type;struct Cyc_Absyn_Exp*initializer;void*rgn;struct Cyc_List_List*attributes;int escapes;};struct Cyc_Absyn_Fndecl{enum Cyc_Absyn_Scope sc;int is_inline;struct _tuple1*name;struct Cyc_Absyn_Stmt*body;struct Cyc_Absyn_FnInfo i;void*cached_type;struct Cyc_Core_Opt*param_vardecls;struct Cyc_Absyn_Vardecl*fn_vardecl;};struct Cyc_Absyn_Aggrfield{struct _fat_ptr*name;struct Cyc_Absyn_Tqual tq;void*type;struct Cyc_Absyn_Exp*width;struct Cyc_List_List*attributes;struct Cyc_Absyn_Exp*requires_clause;};struct Cyc_Absyn_AggrdeclImpl{struct Cyc_List_List*exist_vars;struct Cyc_List_List*rgn_po;struct Cyc_List_List*fields;int tagged;};struct Cyc_Absyn_Aggrdecl{enum Cyc_Absyn_AggrKind kind;enum Cyc_Absyn_Scope sc;struct _tuple1*name;struct Cyc_List_List*tvs;struct Cyc_Absyn_AggrdeclImpl*impl;struct Cyc_List_List*attributes;int expected_mem_kind;};struct Cyc_Absyn_Datatypefield{struct _tuple1*name;struct Cyc_List_List*typs;unsigned loc;enum Cyc_Absyn_Scope sc;};struct Cyc_Absyn_Datatypedecl{enum Cyc_Absyn_Scope sc;struct _tuple1*name;struct Cyc_List_List*tvs;struct Cyc_Core_Opt*fields;int is_extensible;};struct Cyc_Absyn_Enumfield{struct _tuple1*name;struct Cyc_Absyn_Exp*tag;unsigned loc;};struct Cyc_Absyn_Enumdecl{enum Cyc_Absyn_Scope sc;struct _tuple1*name;struct Cyc_Core_Opt*fields;};struct Cyc_Absyn_Typedefdecl{struct _tuple1*name;struct Cyc_Absyn_Tqual tq;struct Cyc_List_List*tvs;struct Cyc_Core_Opt*kind;void*defn;struct Cyc_List_List*atts;int extern_c;};struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Fn_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Fndecl*f1;};struct Cyc_Absyn_Let_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Pat*f1;struct Cyc_Core_Opt*f2;struct Cyc_Absyn_Exp*f3;void*f4;};struct Cyc_Absyn_Letv_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_Region_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_Aggr_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Aggrdecl*f1;};struct Cyc_Absyn_Datatype_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;};struct Cyc_Absyn_Enum_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;};struct Cyc_Absyn_Typedef_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Typedefdecl*f1;};struct Cyc_Absyn_Namespace_d_Absyn_Raw_decl_struct{int tag;struct _fat_ptr*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Using_d_Absyn_Raw_decl_struct{int tag;struct _tuple1*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_ExternC_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;};struct _tuple11{unsigned f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_ExternCinclude_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;struct _tuple11*f4;};struct Cyc_Absyn_Porton_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Portoff_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Tempeston_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Tempestoff_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Decl{void*r;unsigned loc;};struct Cyc_Absyn_ArrayElement_Absyn_Designator_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_FieldName_Absyn_Designator_struct{int tag;struct _fat_ptr*f1;};extern char Cyc_Absyn_EmptyAnnot[11U];struct Cyc_Absyn_EmptyAnnot_Absyn_AbsynAnnot_struct{char*tag;};
# 908 "absyn.h"
int Cyc_Absyn_qvar_cmp(struct _tuple1*,struct _tuple1*);
# 910
int Cyc_Absyn_tvar_cmp(struct Cyc_Absyn_Tvar*,struct Cyc_Absyn_Tvar*);
# 922
void*Cyc_Absyn_compress_kb(void*);
# 928
int Cyc_Absyn_type2bool(int def,void*);
# 933
void*Cyc_Absyn_new_evar(struct Cyc_Core_Opt*k,struct Cyc_Core_Opt*tenv);
# 945
extern void*Cyc_Absyn_heap_rgn_type;extern void*Cyc_Absyn_unique_rgn_type;
# 947
extern void*Cyc_Absyn_empty_effect;
# 949
extern void*Cyc_Absyn_true_type;extern void*Cyc_Absyn_false_type;
# 951
void*Cyc_Absyn_var_type(struct Cyc_Absyn_Tvar*);void*Cyc_Absyn_access_eff(void*);void*Cyc_Absyn_join_eff(struct Cyc_List_List*);void*Cyc_Absyn_regionsof_eff(void*);
# 970
struct _tuple1*Cyc_Absyn_datatype_print_arg_qvar();
struct _tuple1*Cyc_Absyn_datatype_scanf_arg_qvar();
# 980
void*Cyc_Absyn_bounds_one();
# 1022
struct Cyc_Absyn_Exp*Cyc_Absyn_uint_exp(unsigned,unsigned);
# 1164
struct _fat_ptr Cyc_Absyn_attribute2string(void*);
# 1166
int Cyc_Absyn_fntype_att(void*);
# 1210
struct Cyc_List_List*Cyc_Absyn_rgneffect_rnames(struct Cyc_List_List*l);struct Cyc_Absynpp_Params{int expand_typedefs;int qvar_to_Cids;int add_cyc_prefix;int to_VC;int decls_first;int rewrite_temp_tvars;int print_all_tvars;int print_all_kinds;int print_all_effects;int print_using_stmts;int print_externC_stmts;int print_full_evars;int print_zeroterm;int generate_line_directives;int use_curr_namespace;struct Cyc_List_List*curr_namespace;};
# 62 "absynpp.h"
struct _fat_ptr Cyc_Absynpp_typ2string(void*);
# 64
struct _fat_ptr Cyc_Absynpp_kind2string(struct Cyc_Absyn_Kind*);
struct _fat_ptr Cyc_Absynpp_kindbound2string(void*);
# 67
struct _fat_ptr Cyc_Absynpp_exp2string(struct Cyc_Absyn_Exp*);
# 69
struct _fat_ptr Cyc_Absynpp_qvar2string(struct _tuple1*);
# 74
struct _fat_ptr Cyc_Absynpp_tvar2string(struct Cyc_Absyn_Tvar*);
# 29 "unify.h"
int Cyc_Unify_unify(void*,void*);struct _union_RelnOp_RConst{int tag;unsigned val;};struct _union_RelnOp_RVar{int tag;struct Cyc_Absyn_Vardecl*val;};struct _union_RelnOp_RNumelts{int tag;struct Cyc_Absyn_Vardecl*val;};struct _union_RelnOp_RType{int tag;void*val;};struct _union_RelnOp_RParam{int tag;unsigned val;};struct _union_RelnOp_RParamNumelts{int tag;unsigned val;};struct _union_RelnOp_RReturn{int tag;unsigned val;};union Cyc_Relations_RelnOp{struct _union_RelnOp_RConst RConst;struct _union_RelnOp_RVar RVar;struct _union_RelnOp_RNumelts RNumelts;struct _union_RelnOp_RType RType;struct _union_RelnOp_RParam RParam;struct _union_RelnOp_RParamNumelts RParamNumelts;struct _union_RelnOp_RReturn RReturn;};
# 50 "relations-ap.h"
enum Cyc_Relations_Relation{Cyc_Relations_Req =0U,Cyc_Relations_Rneq =1U,Cyc_Relations_Rlte =2U,Cyc_Relations_Rlt =3U};struct Cyc_Relations_Reln{union Cyc_Relations_RelnOp rop1;enum Cyc_Relations_Relation relation;union Cyc_Relations_RelnOp rop2;};
# 84
struct Cyc_List_List*Cyc_Relations_exp2relns(struct _RegionHandle*r,struct Cyc_Absyn_Exp*e);
# 129
int Cyc_Relations_consistent_relations(struct Cyc_List_List*rlns);struct Cyc_Iter_Iter{void*env;int(*next)(void*env,void*dest);};struct Cyc_Dict_T;struct Cyc_Dict_Dict{int(*rel)(void*,void*);struct _RegionHandle*r;const struct Cyc_Dict_T*t;};extern char Cyc_Dict_Present[8U];struct Cyc_Dict_Present_exn_struct{char*tag;};extern char Cyc_Dict_Absent[7U];struct Cyc_Dict_Absent_exn_struct{char*tag;};struct Cyc_RgnOrder_RgnPO;extern char Cyc_Tcenv_Env_error[10U];struct Cyc_Tcenv_Env_error_exn_struct{char*tag;};struct Cyc_Tcenv_Genv{struct Cyc_Dict_Dict aggrdecls;struct Cyc_Dict_Dict datatypedecls;struct Cyc_Dict_Dict enumdecls;struct Cyc_Dict_Dict typedefs;struct Cyc_Dict_Dict ordinaries;};struct Cyc_Tcenv_Fenv;struct Cyc_Tcenv_Tenv{struct Cyc_List_List*ns;struct Cyc_Tcenv_Genv*ae;struct Cyc_Tcenv_Fenv*le;int allow_valueof: 1;int in_extern_c_include: 1;int in_tempest: 1;int tempest_generalize: 1;};
# 72 "tcenv.h"
struct Cyc_Tcenv_Fenv*Cyc_Tcenv_bogus_fenv(void*ret_type,struct Cyc_List_List*args);
# 76
struct Cyc_Absyn_Aggrdecl**Cyc_Tcenv_lookup_aggrdecl(struct Cyc_Tcenv_Tenv*,unsigned,struct _tuple1*);
struct Cyc_Absyn_Datatypedecl**Cyc_Tcenv_lookup_datatypedecl(struct Cyc_Tcenv_Tenv*,unsigned,struct _tuple1*);
# 79
struct Cyc_Absyn_Enumdecl**Cyc_Tcenv_lookup_enumdecl(struct Cyc_Tcenv_Tenv*,unsigned,struct _tuple1*);
struct Cyc_Absyn_Typedefdecl*Cyc_Tcenv_lookup_typedefdecl(struct Cyc_Tcenv_Tenv*,unsigned,struct _tuple1*);
# 82
struct Cyc_Tcenv_Tenv*Cyc_Tcenv_allow_valueof(struct Cyc_Tcenv_Tenv*);
# 88
enum Cyc_Tcenv_NewStatus{Cyc_Tcenv_NoneNew =0U,Cyc_Tcenv_InNew =1U,Cyc_Tcenv_InNewAggr =2U};
# 98
struct Cyc_List_List*Cyc_Tcenv_lookup_type_vars(struct Cyc_Tcenv_Tenv*);
# 30 "tcutil.h"
void*Cyc_Tcutil_impos(struct _fat_ptr,struct _fat_ptr);
void Cyc_Tcutil_terr(unsigned,struct _fat_ptr,struct _fat_ptr);
void Cyc_Tcutil_warn(unsigned,struct _fat_ptr,struct _fat_ptr);
# 37
int Cyc_Tcutil_is_char_type(void*);
# 43
int Cyc_Tcutil_is_function_type(void*);
# 45
int Cyc_Tcutil_is_array_type(void*);
# 64
void*Cyc_Tcutil_pointer_elt_type(void*);
# 72
struct Cyc_Absyn_Exp*Cyc_Tcutil_get_bounds_exp(void*def,void*b);
# 84
struct Cyc_Absyn_Exp*Cyc_Tcutil_get_bounds_exp(void*def,void*b);
# 87
int Cyc_Tcutil_is_integral(struct Cyc_Absyn_Exp*);
# 94
void*Cyc_Tcutil_copy_type(void*);
# 100
int Cyc_Tcutil_kind_leq(struct Cyc_Absyn_Kind*,struct Cyc_Absyn_Kind*);
# 105
struct Cyc_Absyn_Kind*Cyc_Tcutil_tvar_kind(struct Cyc_Absyn_Tvar*,struct Cyc_Absyn_Kind*def);
struct Cyc_Absyn_Kind*Cyc_Tcutil_type_kind(void*);
void*Cyc_Tcutil_compress(void*);
# 109
int Cyc_Tcutil_coerce_uint_type(struct Cyc_Absyn_Exp*);
# 135
extern struct Cyc_Absyn_Kind Cyc_Tcutil_rk;
extern struct Cyc_Absyn_Kind Cyc_Tcutil_ak;
extern struct Cyc_Absyn_Kind Cyc_Tcutil_bk;
# 139
extern struct Cyc_Absyn_Kind Cyc_Tcutil_ek;
extern struct Cyc_Absyn_Kind Cyc_Tcutil_ik;
extern struct Cyc_Absyn_Kind Cyc_Tcutil_boolk;
extern struct Cyc_Absyn_Kind Cyc_Tcutil_ptrbk;
# 144
extern struct Cyc_Absyn_Kind Cyc_Tcutil_trk;
extern struct Cyc_Absyn_Kind Cyc_Tcutil_tak;
extern struct Cyc_Absyn_Kind Cyc_Tcutil_tbk;
extern struct Cyc_Absyn_Kind Cyc_Tcutil_tmk;
# 149
extern struct Cyc_Absyn_Kind Cyc_Tcutil_urk;
# 151
extern struct Cyc_Absyn_Kind Cyc_Tcutil_ubk;
# 173
struct Cyc_Core_Opt*Cyc_Tcutil_kind_to_opt(struct Cyc_Absyn_Kind*k);
# 185
void*Cyc_Tcutil_substitute(struct Cyc_List_List*,void*);
# 202
void*Cyc_Tcutil_fndecl2type(struct Cyc_Absyn_Fndecl*);
# 212
void Cyc_Tcutil_check_bitfield(unsigned,void*field_typ,struct Cyc_Absyn_Exp*width,struct _fat_ptr*fn);
# 214
void Cyc_Tcutil_check_unique_vars(struct Cyc_List_List*,unsigned,struct _fat_ptr err_msg);
void Cyc_Tcutil_check_unique_tvars(unsigned,struct Cyc_List_List*);
# 256
struct Cyc_Absyn_Tvar*Cyc_Tcutil_new_tvar(void*);
# 258
int Cyc_Tcutil_new_tvar_id();
# 273
int Cyc_Tcutil_is_const_exp(struct Cyc_Absyn_Exp*);
# 280
int Cyc_Tcutil_extract_const_from_typedef(unsigned,int declared_const,void*);
# 287
void Cyc_Tcutil_check_no_qual(unsigned,void*);
# 298
void*Cyc_Tcutil_promote_array(void*t,void*rgn,int convert_tag);
# 305
int Cyc_Tcutil_force_type2bool(int desired,void*);
# 313
int Cyc_Tcutil_admits_zero(void*);
void Cyc_Tcutil_replace_rops(struct Cyc_List_List*,struct Cyc_Relations_Reln*);
# 316
struct Cyc_Core_Opt*Cyc_Tcutil_kind_to_bound_opt(struct Cyc_Absyn_Kind*k);
int Cyc_Tcutil_fast_tvar_cmp(struct Cyc_Absyn_Tvar*,struct Cyc_Absyn_Tvar*);struct _tuple12{unsigned f1;int f2;};
# 28 "evexp.h"
struct _tuple12 Cyc_Evexp_eval_const_uint_exp(struct Cyc_Absyn_Exp*e);
# 28 "tcexp.h"
void*Cyc_Tcexp_tcExp(struct Cyc_Tcenv_Tenv*,void**,struct Cyc_Absyn_Exp*);
# 40 "tc.h"
void Cyc_Tc_tcAggrdecl(struct Cyc_Tcenv_Tenv*,unsigned,struct Cyc_Absyn_Aggrdecl*);
void Cyc_Tc_tcDatatypedecl(struct Cyc_Tcenv_Tenv*,unsigned,struct Cyc_Absyn_Datatypedecl*);
void Cyc_Tc_tcEnumdecl(struct Cyc_Tcenv_Tenv*,unsigned,struct Cyc_Absyn_Enumdecl*);
# 24 "cyclone.h"
extern int Cyc_Cyclone_tovc_r;
# 26
enum Cyc_Cyclone_C_Compilers{Cyc_Cyclone_Gcc_c =0U,Cyc_Cyclone_Vc_c =1U};
# 34 "tctyp.h"
void Cyc_Tctyp_check_valid_toplevel_type(unsigned,struct Cyc_Tcenv_Tenv*,void*);struct Cyc_Tctyp_CVTEnv{struct _RegionHandle*r;struct Cyc_List_List*kind_env;int fn_result;int generalize_evars;struct Cyc_List_List*free_vars;struct Cyc_List_List*free_evars;};
# 56 "tctyp.cyc"
static struct _fat_ptr*Cyc_Tctyp_fst_fdarg(struct _tuple9*t){return(struct _fat_ptr*)_check_null((*t).f1);}struct _tuple13{void*f1;int f2;};static struct Cyc_List_List*Cyc_Tctyp_add_free_evar(struct _RegionHandle*r,struct Cyc_List_List*es,void*e,int b){
# 64
void*_stmttmp0=Cyc_Tcutil_compress(e);void*_tmp0=_stmttmp0;int _tmp1;if(((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp0)->tag == 1U){_LL1: _tmp1=((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp0)->f3;_LL2: {int i=_tmp1;
# 66
{struct Cyc_List_List*es2=es;for(0;es2 != 0;es2=es2->tl){
struct _tuple13*_stmttmp1=(struct _tuple13*)es2->hd;int*_tmp3;void*_tmp2;_LL6: _tmp2=_stmttmp1->f1;_tmp3=(int*)& _stmttmp1->f2;_LL7: {void*t=_tmp2;int*b2=_tmp3;
void*_stmttmp2=Cyc_Tcutil_compress(t);void*_tmp4=_stmttmp2;int _tmp5;if(((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp4)->tag == 1U){_LL9: _tmp5=((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp4)->f3;_LLA: {int j=_tmp5;
# 70
if(i == j){
if(b != *b2)*b2=1;return es;}
# 70
goto _LL8;}}else{_LLB: _LLC:
# 75
 goto _LL8;}_LL8:;}}}
# 78
return({struct Cyc_List_List*_tmp7=_region_malloc(r,sizeof(*_tmp7));({struct _tuple13*_tmp23B=({struct _tuple13*_tmp6=_region_malloc(r,sizeof(*_tmp6));_tmp6->f1=e,_tmp6->f2=b;_tmp6;});_tmp7->hd=_tmp23B;}),_tmp7->tl=es;_tmp7;});}}else{_LL3: _LL4:
 return es;}_LL0:;}
# 56
static struct Cyc_List_List*Cyc_Tctyp_add_bound_tvar(struct Cyc_List_List*tvs,struct Cyc_Absyn_Tvar*tv){
# 85
if(tv->identity == - 1)
({struct Cyc_String_pa_PrintArg_struct _tmpB=({struct Cyc_String_pa_PrintArg_struct _tmp203;_tmp203.tag=0U,({struct _fat_ptr _tmp23C=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_tvar2string(tv));_tmp203.f1=_tmp23C;});_tmp203;});void*_tmp8[1U];_tmp8[0]=& _tmpB;({int(*_tmp23E)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmpA)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmpA;});struct _fat_ptr _tmp23D=({const char*_tmp9="bound tvar id for %s is NULL";_tag_fat(_tmp9,sizeof(char),29U);});_tmp23E(_tmp23D,_tag_fat(_tmp8,sizeof(void*),1U));});});
# 85
return({struct Cyc_List_List*_tmpC=_cycalloc(sizeof(*_tmpC));
# 87
_tmpC->hd=tv,_tmpC->tl=tvs;_tmpC;});}
# 56
static struct Cyc_List_List*Cyc_Tctyp_remove_bound_tvars(struct _RegionHandle*rgn,struct Cyc_List_List*tvs,struct Cyc_List_List*btvs){
# 92
struct Cyc_List_List*r=0;
for(0;tvs != 0;tvs=tvs->tl){
int present=0;
{struct Cyc_List_List*b=btvs;for(0;b != 0;b=b->tl){
if(((struct Cyc_Absyn_Tvar*)tvs->hd)->identity == ((struct Cyc_Absyn_Tvar*)b->hd)->identity){
present=1;
break;}}}
# 95
if(!present)
# 101
r=({struct Cyc_List_List*_tmpD=_region_malloc(rgn,sizeof(*_tmpD));_tmpD->hd=(struct Cyc_Absyn_Tvar*)tvs->hd,_tmpD->tl=r;_tmpD;});}
# 103
r=Cyc_List_imp_rev(r);
return r;}struct _tuple14{struct Cyc_Absyn_Tvar*f1;int f2;};
# 56
static struct Cyc_List_List*Cyc_Tctyp_remove_bound_tvars_bool(struct _RegionHandle*r,struct Cyc_List_List*tvs,struct Cyc_List_List*btvs){
# 109
struct Cyc_List_List*res=0;
for(0;tvs != 0;tvs=tvs->tl){
struct _tuple14 _stmttmp3=*((struct _tuple14*)tvs->hd);int _tmpF;struct Cyc_Absyn_Tvar*_tmpE;_LL1: _tmpE=_stmttmp3.f1;_tmpF=_stmttmp3.f2;_LL2: {struct Cyc_Absyn_Tvar*tv=_tmpE;int b=_tmpF;
int present=0;
{struct Cyc_List_List*b=btvs;for(0;b != 0;b=b->tl){
if(tv->identity == ((struct Cyc_Absyn_Tvar*)b->hd)->identity){
present=1;
break;}}}
# 113
if(!present)
# 119
res=({struct Cyc_List_List*_tmp10=_region_malloc(r,sizeof(*_tmp10));_tmp10->hd=(struct _tuple14*)tvs->hd,_tmp10->tl=res;_tmp10;});}}
# 121
res=Cyc_List_imp_rev(res);
return res;}
# 56
static void Cyc_Tctyp_check_free_evars(struct Cyc_List_List*free_evars,void*in_typ,unsigned loc){
# 131
for(0;free_evars != 0;free_evars=free_evars->tl){
void*e=(void*)free_evars->hd;
{void*_stmttmp4=Cyc_Tcutil_compress(e);void*_tmp11=_stmttmp4;if(((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp11)->tag == 1U){_LL1: _LL2:
 goto _LL0;}else{_LL3: _LL4:
 continue;}_LL0:;}{
# 137
struct Cyc_Absyn_Kind*_stmttmp5=Cyc_Tcutil_type_kind(e);struct Cyc_Absyn_Kind*_tmp12=_stmttmp5;switch(((struct Cyc_Absyn_Kind*)_tmp12)->kind){case Cyc_Absyn_RgnKind: switch(((struct Cyc_Absyn_Kind*)_tmp12)->aliasqual){case Cyc_Absyn_Unique: _LL6: _LL7:
# 139
 if(!Cyc_Unify_unify(e,Cyc_Absyn_unique_rgn_type))
({void*_tmp13=0U;({int(*_tmp240)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp15)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp15;});struct _fat_ptr _tmp23F=({const char*_tmp14="can't unify evar with unique region!";_tag_fat(_tmp14,sizeof(char),37U);});_tmp240(_tmp23F,_tag_fat(_tmp13,sizeof(void*),0U));});});
# 139
goto _LL5;case Cyc_Absyn_Aliasable: _LL8: _LL9:
# 142
 goto _LLB;case Cyc_Absyn_Top: _LLA: _LLB:
# 144
 if(!Cyc_Unify_unify(e,Cyc_Absyn_heap_rgn_type))({void*_tmp16=0U;({int(*_tmp242)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp18)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp18;});struct _fat_ptr _tmp241=({const char*_tmp17="can't unify evar with heap!";_tag_fat(_tmp17,sizeof(char),28U);});_tmp242(_tmp241,_tag_fat(_tmp16,sizeof(void*),0U));});});goto _LL5;default: goto _LL12;}case Cyc_Absyn_EffKind: _LLC: _LLD:
# 147
 if(!Cyc_Unify_unify(e,Cyc_Absyn_empty_effect))({void*_tmp19=0U;({int(*_tmp244)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp1B)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp1B;});struct _fat_ptr _tmp243=({const char*_tmp1A="can't unify evar with {}!";_tag_fat(_tmp1A,sizeof(char),26U);});_tmp244(_tmp243,_tag_fat(_tmp19,sizeof(void*),0U));});});goto _LL5;case Cyc_Absyn_BoolKind: _LLE: _LLF:
# 150
 if(!Cyc_Unify_unify(e,Cyc_Absyn_false_type))({struct Cyc_String_pa_PrintArg_struct _tmp1F=({struct Cyc_String_pa_PrintArg_struct _tmp204;_tmp204.tag=0U,({
struct _fat_ptr _tmp245=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(e));_tmp204.f1=_tmp245;});_tmp204;});void*_tmp1C[1U];_tmp1C[0]=& _tmp1F;({int(*_tmp247)(struct _fat_ptr,struct _fat_ptr)=({
# 150
int(*_tmp1E)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp1E;});struct _fat_ptr _tmp246=({const char*_tmp1D="can't unify evar %s with @false!";_tag_fat(_tmp1D,sizeof(char),33U);});_tmp247(_tmp246,_tag_fat(_tmp1C,sizeof(void*),1U));});});goto _LL5;case Cyc_Absyn_PtrBndKind: _LL10: _LL11:
# 154
 if(!({void*_tmp248=e;Cyc_Unify_unify(_tmp248,Cyc_Absyn_bounds_one());}))
({void*_tmp20=0U;({int(*_tmp24A)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp22)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp22;});struct _fat_ptr _tmp249=({const char*_tmp21="can't unify evar with bounds_one!";_tag_fat(_tmp21,sizeof(char),34U);});_tmp24A(_tmp249,_tag_fat(_tmp20,sizeof(void*),0U));});});
# 154
goto _LL5;default: _LL12: _LL13:
# 158
({struct Cyc_String_pa_PrintArg_struct _tmp25=({struct Cyc_String_pa_PrintArg_struct _tmp206;_tmp206.tag=0U,({
struct _fat_ptr _tmp24B=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(e));_tmp206.f1=_tmp24B;});_tmp206;});struct Cyc_String_pa_PrintArg_struct _tmp26=({struct Cyc_String_pa_PrintArg_struct _tmp205;_tmp205.tag=0U,({struct _fat_ptr _tmp24C=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(in_typ));_tmp205.f1=_tmp24C;});_tmp205;});void*_tmp23[2U];_tmp23[0]=& _tmp25,_tmp23[1]=& _tmp26;({unsigned _tmp24E=loc;struct _fat_ptr _tmp24D=({const char*_tmp24="hidden type variable %s isn't abstracted in type %s";_tag_fat(_tmp24,sizeof(char),52U);});Cyc_Tcutil_terr(_tmp24E,_tmp24D,_tag_fat(_tmp23,sizeof(void*),2U));});});
goto _LL5;}_LL5:;}}}
# 56
static int Cyc_Tctyp_constrain_kinds(void*c1,void*c2){
# 170
c1=Cyc_Absyn_compress_kb(c1);
c2=Cyc_Absyn_compress_kb(c2);
if(c1 == c2)return 1;{struct _tuple0 _stmttmp6=({struct _tuple0 _tmp207;_tmp207.f1=c1,_tmp207.f2=c2;_tmp207;});struct _tuple0 _tmp27=_stmttmp6;struct Cyc_Absyn_Kind*_tmp2B;struct Cyc_Core_Opt**_tmp2A;struct Cyc_Absyn_Kind*_tmp29;struct Cyc_Core_Opt**_tmp28;struct Cyc_Absyn_Kind*_tmp2E;struct Cyc_Absyn_Kind*_tmp2D;struct Cyc_Core_Opt**_tmp2C;struct Cyc_Core_Opt**_tmp2F;struct Cyc_Core_Opt**_tmp30;struct Cyc_Absyn_Kind*_tmp33;struct Cyc_Core_Opt**_tmp32;struct Cyc_Absyn_Kind*_tmp31;struct Cyc_Absyn_Kind*_tmp35;struct Cyc_Absyn_Kind*_tmp34;if(((struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct*)_tmp27.f1)->tag == 0U)switch(*((int*)_tmp27.f2)){case 0U: _LL1: _tmp34=((struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct*)_tmp27.f1)->f1;_tmp35=((struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct*)_tmp27.f2)->f1;_LL2: {struct Cyc_Absyn_Kind*k1=_tmp34;struct Cyc_Absyn_Kind*k2=_tmp35;
# 174
return k1 == k2;}case 1U: goto _LL3;default: _LL9: _tmp31=((struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct*)_tmp27.f1)->f1;_tmp32=(struct Cyc_Core_Opt**)&((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp27.f2)->f1;_tmp33=((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp27.f2)->f2;_LLA: {struct Cyc_Absyn_Kind*k1=_tmp31;struct Cyc_Core_Opt**f=_tmp32;struct Cyc_Absyn_Kind*k2=_tmp33;
# 182
if(Cyc_Tcutil_kind_leq(k1,k2)){
({struct Cyc_Core_Opt*_tmp24F=({struct Cyc_Core_Opt*_tmp39=_cycalloc(sizeof(*_tmp39));_tmp39->v=c1;_tmp39;});*f=_tmp24F;});return 1;}else{
return 0;}}}else{if(((struct Cyc_Absyn_Unknown_kb_Absyn_KindBound_struct*)_tmp27.f2)->tag == 1U){_LL3: _tmp30=(struct Cyc_Core_Opt**)&((struct Cyc_Absyn_Unknown_kb_Absyn_KindBound_struct*)_tmp27.f2)->f1;_LL4: {struct Cyc_Core_Opt**f=_tmp30;
# 175
({struct Cyc_Core_Opt*_tmp250=({struct Cyc_Core_Opt*_tmp36=_cycalloc(sizeof(*_tmp36));_tmp36->v=c1;_tmp36;});*f=_tmp250;});return 1;}}else{if(((struct Cyc_Absyn_Unknown_kb_Absyn_KindBound_struct*)_tmp27.f1)->tag == 1U){_LL5: _tmp2F=(struct Cyc_Core_Opt**)&((struct Cyc_Absyn_Unknown_kb_Absyn_KindBound_struct*)_tmp27.f1)->f1;_LL6: {struct Cyc_Core_Opt**f=_tmp2F;
({struct Cyc_Core_Opt*_tmp251=({struct Cyc_Core_Opt*_tmp37=_cycalloc(sizeof(*_tmp37));_tmp37->v=c2;_tmp37;});*f=_tmp251;});return 1;}}else{if(((struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct*)_tmp27.f2)->tag == 0U){_LL7: _tmp2C=(struct Cyc_Core_Opt**)&((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp27.f1)->f1;_tmp2D=((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp27.f1)->f2;_tmp2E=((struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct*)_tmp27.f2)->f1;_LL8: {struct Cyc_Core_Opt**f=_tmp2C;struct Cyc_Absyn_Kind*k1=_tmp2D;struct Cyc_Absyn_Kind*k2=_tmp2E;
# 178
if(Cyc_Tcutil_kind_leq(k2,k1)){
({struct Cyc_Core_Opt*_tmp252=({struct Cyc_Core_Opt*_tmp38=_cycalloc(sizeof(*_tmp38));_tmp38->v=c2;_tmp38;});*f=_tmp252;});return 1;}else{
return 0;}}}else{_LLB: _tmp28=(struct Cyc_Core_Opt**)&((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp27.f1)->f1;_tmp29=((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp27.f1)->f2;_tmp2A=(struct Cyc_Core_Opt**)&((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp27.f2)->f1;_tmp2B=((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp27.f2)->f2;_LLC: {struct Cyc_Core_Opt**f1=_tmp28;struct Cyc_Absyn_Kind*k1=_tmp29;struct Cyc_Core_Opt**f2=_tmp2A;struct Cyc_Absyn_Kind*k2=_tmp2B;
# 186
if(Cyc_Tcutil_kind_leq(k1,k2)){
({struct Cyc_Core_Opt*_tmp253=({struct Cyc_Core_Opt*_tmp3A=_cycalloc(sizeof(*_tmp3A));_tmp3A->v=c1;_tmp3A;});*f2=_tmp253;});return 1;}else{
if(Cyc_Tcutil_kind_leq(k2,k1)){
({struct Cyc_Core_Opt*_tmp254=({struct Cyc_Core_Opt*_tmp3B=_cycalloc(sizeof(*_tmp3B));_tmp3B->v=c2;_tmp3B;});*f1=_tmp254;});return 1;}else{
return 0;}}}}}}}_LL0:;}}
# 56
static struct Cyc_List_List*Cyc_Tctyp_add_free_tvar(unsigned loc,struct Cyc_List_List*tvs,struct Cyc_Absyn_Tvar*tv){
# 202
{struct Cyc_List_List*tvs2=tvs;for(0;tvs2 != 0;tvs2=tvs2->tl){
if(Cyc_strptrcmp(((struct Cyc_Absyn_Tvar*)tvs2->hd)->name,tv->name)== 0){
void*k1=((struct Cyc_Absyn_Tvar*)tvs2->hd)->kind;
void*k2=tv->kind;
if(!Cyc_Tctyp_constrain_kinds(k1,k2))
({struct Cyc_String_pa_PrintArg_struct _tmp3E=({struct Cyc_String_pa_PrintArg_struct _tmp20A;_tmp20A.tag=0U,_tmp20A.f1=(struct _fat_ptr)((struct _fat_ptr)*tv->name);_tmp20A;});struct Cyc_String_pa_PrintArg_struct _tmp3F=({struct Cyc_String_pa_PrintArg_struct _tmp209;_tmp209.tag=0U,({
struct _fat_ptr _tmp255=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_kindbound2string(k1));_tmp209.f1=_tmp255;});_tmp209;});struct Cyc_String_pa_PrintArg_struct _tmp40=({struct Cyc_String_pa_PrintArg_struct _tmp208;_tmp208.tag=0U,({struct _fat_ptr _tmp256=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_kindbound2string(k2));_tmp208.f1=_tmp256;});_tmp208;});void*_tmp3C[3U];_tmp3C[0]=& _tmp3E,_tmp3C[1]=& _tmp3F,_tmp3C[2]=& _tmp40;({unsigned _tmp258=loc;struct _fat_ptr _tmp257=({const char*_tmp3D="type variable %s is used with inconsistent kinds %s and %s";_tag_fat(_tmp3D,sizeof(char),59U);});Cyc_Tcutil_terr(_tmp258,_tmp257,_tag_fat(_tmp3C,sizeof(void*),3U));});});
# 206
if(tv->identity == - 1)
# 210
tv->identity=((struct Cyc_Absyn_Tvar*)tvs2->hd)->identity;else{
if(tv->identity != ((struct Cyc_Absyn_Tvar*)tvs2->hd)->identity)
({void*_tmp41=0U;({int(*_tmp25A)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp43)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp43;});struct _fat_ptr _tmp259=({const char*_tmp42="same type variable has different identity!";_tag_fat(_tmp42,sizeof(char),43U);});_tmp25A(_tmp259,_tag_fat(_tmp41,sizeof(void*),0U));});});}
# 206
return tvs;}}}
# 216
({int _tmp25B=Cyc_Tcutil_new_tvar_id();tv->identity=_tmp25B;});
return({struct Cyc_List_List*_tmp44=_cycalloc(sizeof(*_tmp44));_tmp44->hd=tv,_tmp44->tl=tvs;_tmp44;});}
# 56
static struct Cyc_List_List*Cyc_Tctyp_fast_add_free_tvar(struct Cyc_List_List*tvs,struct Cyc_Absyn_Tvar*tv){
# 223
if(tv->identity == - 1)
({void*_tmp45=0U;({int(*_tmp25D)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp47)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp47;});struct _fat_ptr _tmp25C=({const char*_tmp46="fast_add_free_tvar: bad identity in tv";_tag_fat(_tmp46,sizeof(char),39U);});_tmp25D(_tmp25C,_tag_fat(_tmp45,sizeof(void*),0U));});});
# 223
{
# 225
struct Cyc_List_List*tvs2=tvs;
# 223
for(0;tvs2 != 0;tvs2=tvs2->tl){
# 226
struct Cyc_Absyn_Tvar*tv2=(struct Cyc_Absyn_Tvar*)tvs2->hd;
if(tv2->identity == - 1)
({void*_tmp48=0U;({int(*_tmp25F)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp4A)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp4A;});struct _fat_ptr _tmp25E=({const char*_tmp49="fast_add_free_tvar: bad identity in tvs2";_tag_fat(_tmp49,sizeof(char),41U);});_tmp25F(_tmp25E,_tag_fat(_tmp48,sizeof(void*),0U));});});
# 227
if(tv2->identity == tv->identity)
# 230
return tvs;}}
# 232
return({struct Cyc_List_List*_tmp4B=_cycalloc(sizeof(*_tmp4B));_tmp4B->hd=tv,_tmp4B->tl=tvs;_tmp4B;});}
# 56
static struct Cyc_List_List*Cyc_Tctyp_fast_add_free_tvar_bool(struct _RegionHandle*r,struct Cyc_List_List*tvs,struct Cyc_Absyn_Tvar*tv,int b){
# 241
if(tv->identity == - 1)
({void*_tmp4C=0U;({int(*_tmp261)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp4E)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp4E;});struct _fat_ptr _tmp260=({const char*_tmp4D="fast_add_free_tvar_bool: bad identity in tv";_tag_fat(_tmp4D,sizeof(char),44U);});_tmp261(_tmp260,_tag_fat(_tmp4C,sizeof(void*),0U));});});
# 241
{
# 243
struct Cyc_List_List*tvs2=tvs;
# 241
for(0;tvs2 != 0;tvs2=tvs2->tl){
# 244
struct _tuple14*_stmttmp7=(struct _tuple14*)tvs2->hd;int*_tmp50;struct Cyc_Absyn_Tvar*_tmp4F;_LL1: _tmp4F=_stmttmp7->f1;_tmp50=(int*)& _stmttmp7->f2;_LL2: {struct Cyc_Absyn_Tvar*tv2=_tmp4F;int*b2=_tmp50;
if(tv2->identity == - 1)
({void*_tmp51=0U;({int(*_tmp263)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp53)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp53;});struct _fat_ptr _tmp262=({const char*_tmp52="fast_add_free_tvar_bool: bad identity in tvs2";_tag_fat(_tmp52,sizeof(char),46U);});_tmp263(_tmp262,_tag_fat(_tmp51,sizeof(void*),0U));});});
# 245
if(tv2->identity == tv->identity){
# 248
*b2=*b2 || b;
return tvs;}}}}
# 252
return({struct Cyc_List_List*_tmp55=_region_malloc(r,sizeof(*_tmp55));({struct _tuple14*_tmp264=({struct _tuple14*_tmp54=_region_malloc(r,sizeof(*_tmp54));_tmp54->f1=tv,_tmp54->f2=b;_tmp54;});_tmp55->hd=_tmp264;}),_tmp55->tl=tvs;_tmp55;});}
# 56
static int Cyc_Tctyp_typedef_tvar_is_ptr_rgn(struct Cyc_Absyn_Tvar*tvar,struct Cyc_Absyn_Typedefdecl*td){
# 256
if(td != 0){
if(td->defn != 0){
void*_stmttmp8=Cyc_Tcutil_compress((void*)_check_null(td->defn));void*_tmp56=_stmttmp8;void*_tmp57;if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp56)->tag == 3U){_LL1: _tmp57=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp56)->f1).ptr_atts).rgn;_LL2: {void*r=_tmp57;
# 260
{void*_stmttmp9=Cyc_Tcutil_compress(r);void*_tmp58=_stmttmp9;struct Cyc_Absyn_Tvar*_tmp59;if(((struct Cyc_Absyn_VarType_Absyn_Type_struct*)_tmp58)->tag == 2U){_LL6: _tmp59=((struct Cyc_Absyn_VarType_Absyn_Type_struct*)_tmp58)->f1;_LL7: {struct Cyc_Absyn_Tvar*tv=_tmp59;
return Cyc_Absyn_tvar_cmp(tvar,tv)== 0;}}else{_LL8: _LL9:
 goto _LL5;}_LL5:;}
# 264
goto _LL0;}}else{_LL3: _LL4:
 goto _LL0;}_LL0:;}}else{
# 270
return 1;}
return 0;}
# 56
static struct Cyc_Absyn_Kind*Cyc_Tctyp_tvar_inst_kind(struct Cyc_Absyn_Tvar*tvar,struct Cyc_Absyn_Kind*def_kind,struct Cyc_Absyn_Kind*expected_kind,struct Cyc_Absyn_Typedefdecl*td){
# 277
void*_stmttmpA=Cyc_Absyn_compress_kb(tvar->kind);void*_tmp5A=_stmttmpA;switch(*((int*)_tmp5A)){case 2U: if(((struct Cyc_Absyn_Kind*)((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp5A)->f2)->kind == Cyc_Absyn_RgnKind){if(((struct Cyc_Absyn_Kind*)((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp5A)->f2)->aliasqual == Cyc_Absyn_Top){_LL1: _LL2:
 goto _LL4;}else{goto _LL5;}}else{goto _LL5;}case 0U: if(((struct Cyc_Absyn_Kind*)((struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct*)_tmp5A)->f1)->kind == Cyc_Absyn_RgnKind){if(((struct Cyc_Absyn_Kind*)((struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct*)_tmp5A)->f1)->aliasqual == Cyc_Absyn_Top){_LL3: _LL4:
# 286
 if((((int)expected_kind->kind == (int)Cyc_Absyn_BoxKind ||(int)expected_kind->kind == (int)Cyc_Absyn_MemKind)||(int)expected_kind->kind == (int)Cyc_Absyn_AnyKind)&&
# 289
 Cyc_Tctyp_typedef_tvar_is_ptr_rgn(tvar,td)){
if((int)expected_kind->aliasqual == (int)Cyc_Absyn_Aliasable)
return& Cyc_Tcutil_rk;else{
if((int)expected_kind->aliasqual == (int)Cyc_Absyn_Unique)
return& Cyc_Tcutil_urk;}}
# 286
return& Cyc_Tcutil_trk;}else{goto _LL5;}}else{goto _LL5;}default: _LL5: _LL6:
# 296
 return Cyc_Tcutil_tvar_kind(tvar,def_kind);}_LL0:;}
# 56
static void Cyc_Tctyp_check_field_atts(unsigned loc,struct _fat_ptr*fn,struct Cyc_List_List*atts){
# 301
for(0;atts != 0;atts=atts->tl){
void*_stmttmpB=(void*)atts->hd;void*_tmp5B=_stmttmpB;switch(*((int*)_tmp5B)){case 7U: _LL1: _LL2:
 goto _LL4;case 6U: _LL3: _LL4:
 continue;default: _LL5: _LL6:
({struct Cyc_String_pa_PrintArg_struct _tmp5E=({struct Cyc_String_pa_PrintArg_struct _tmp20C;_tmp20C.tag=0U,({
struct _fat_ptr _tmp265=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absyn_attribute2string((void*)atts->hd));_tmp20C.f1=_tmp265;});_tmp20C;});struct Cyc_String_pa_PrintArg_struct _tmp5F=({struct Cyc_String_pa_PrintArg_struct _tmp20B;_tmp20B.tag=0U,_tmp20B.f1=(struct _fat_ptr)((struct _fat_ptr)*fn);_tmp20B;});void*_tmp5C[2U];_tmp5C[0]=& _tmp5E,_tmp5C[1]=& _tmp5F;({unsigned _tmp267=loc;struct _fat_ptr _tmp266=({const char*_tmp5D="bad attribute %s on member %s";_tag_fat(_tmp5D,sizeof(char),30U);});Cyc_Tcutil_terr(_tmp267,_tmp266,_tag_fat(_tmp5C,sizeof(void*),2U));});});}_LL0:;}}
# 56
static struct Cyc_Tctyp_CVTEnv Cyc_Tctyp_i_check_valid_type_level_exp(struct Cyc_Absyn_Exp*,struct Cyc_Tcenv_Tenv*,struct Cyc_Tctyp_CVTEnv);struct _tuple15{struct Cyc_Tctyp_CVTEnv f1;struct Cyc_List_List*f2;};
# 314
static struct _tuple15 Cyc_Tctyp_check_clause(unsigned loc,struct Cyc_Tcenv_Tenv*te,struct Cyc_Tctyp_CVTEnv cvtenv,struct _fat_ptr clause_name,struct Cyc_Absyn_Exp*clause){
# 317
struct Cyc_List_List*relns=0;
if(clause != 0){
Cyc_Tcexp_tcExp(te,0,clause);
if(!Cyc_Tcutil_is_integral(clause))
({struct Cyc_String_pa_PrintArg_struct _tmp62=({struct Cyc_String_pa_PrintArg_struct _tmp20E;_tmp20E.tag=0U,_tmp20E.f1=(struct _fat_ptr)((struct _fat_ptr)clause_name);_tmp20E;});struct Cyc_String_pa_PrintArg_struct _tmp63=({struct Cyc_String_pa_PrintArg_struct _tmp20D;_tmp20D.tag=0U,({
struct _fat_ptr _tmp268=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string((void*)_check_null(clause->topt)));_tmp20D.f1=_tmp268;});_tmp20D;});void*_tmp60[2U];_tmp60[0]=& _tmp62,_tmp60[1]=& _tmp63;({unsigned _tmp26A=loc;struct _fat_ptr _tmp269=({const char*_tmp61="%s clause has type %s instead of integral type";_tag_fat(_tmp61,sizeof(char),47U);});Cyc_Tcutil_terr(_tmp26A,_tmp269,_tag_fat(_tmp60,sizeof(void*),2U));});});
# 320
cvtenv=
# 323
Cyc_Tctyp_i_check_valid_type_level_exp(clause,te,cvtenv);
relns=Cyc_Relations_exp2relns(Cyc_Core_heap_region,clause);
if(!Cyc_Relations_consistent_relations(relns))
({struct Cyc_String_pa_PrintArg_struct _tmp66=({struct Cyc_String_pa_PrintArg_struct _tmp210;_tmp210.tag=0U,_tmp210.f1=(struct _fat_ptr)((struct _fat_ptr)clause_name);_tmp210;});struct Cyc_String_pa_PrintArg_struct _tmp67=({struct Cyc_String_pa_PrintArg_struct _tmp20F;_tmp20F.tag=0U,({
struct _fat_ptr _tmp26B=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_exp2string(clause));_tmp20F.f1=_tmp26B;});_tmp20F;});void*_tmp64[2U];_tmp64[0]=& _tmp66,_tmp64[1]=& _tmp67;({unsigned _tmp26D=clause->loc;struct _fat_ptr _tmp26C=({const char*_tmp65="%s clause '%s' may be unsatisfiable";_tag_fat(_tmp65,sizeof(char),36U);});Cyc_Tcutil_terr(_tmp26D,_tmp26C,_tag_fat(_tmp64,sizeof(void*),2U));});});}
# 318
return({struct _tuple15 _tmp211;_tmp211.f1=cvtenv,_tmp211.f2=relns;_tmp211;});}
# 314
static struct Cyc_Tctyp_CVTEnv Cyc_Tctyp_i_check_valid_type(unsigned,struct Cyc_Tcenv_Tenv*,struct Cyc_Tctyp_CVTEnv,struct Cyc_Absyn_Kind*expected_kind,void*t,int put_in_effect,int allow_abs_aggr);
# 364 "tctyp.cyc"
static struct Cyc_Tctyp_CVTEnv Cyc_Tctyp_i_check_valid_aggr(unsigned loc,struct Cyc_Tcenv_Tenv*te,struct Cyc_Tctyp_CVTEnv cvtenv,struct Cyc_Absyn_Kind*expected_kind,union Cyc_Absyn_AggrInfo*info,struct Cyc_List_List**targs,int allow_abs_aggr){
# 370
{union Cyc_Absyn_AggrInfo _stmttmpC=*info;union Cyc_Absyn_AggrInfo _tmp68=_stmttmpC;struct Cyc_Absyn_Aggrdecl*_tmp69;struct Cyc_Core_Opt*_tmp6C;struct _tuple1*_tmp6B;enum Cyc_Absyn_AggrKind _tmp6A;if((_tmp68.UnknownAggr).tag == 1){_LL1: _tmp6A=((_tmp68.UnknownAggr).val).f1;_tmp6B=((_tmp68.UnknownAggr).val).f2;_tmp6C=((_tmp68.UnknownAggr).val).f3;_LL2: {enum Cyc_Absyn_AggrKind k=_tmp6A;struct _tuple1*n=_tmp6B;struct Cyc_Core_Opt*tgd=_tmp6C;
# 372
struct Cyc_Absyn_Aggrdecl**adp;
{struct _handler_cons _tmp6D;_push_handler(& _tmp6D);{int _tmp6F=0;if(setjmp(_tmp6D.handler))_tmp6F=1;if(!_tmp6F){
adp=Cyc_Tcenv_lookup_aggrdecl(te,loc,n);{
struct Cyc_Absyn_Aggrdecl*ad=*adp;
if((int)ad->kind != (int)k){
if((int)ad->kind == (int)Cyc_Absyn_StructA)
({struct Cyc_String_pa_PrintArg_struct _tmp72=({struct Cyc_String_pa_PrintArg_struct _tmp213;_tmp213.tag=0U,({
struct _fat_ptr _tmp26E=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(n));_tmp213.f1=_tmp26E;});_tmp213;});struct Cyc_String_pa_PrintArg_struct _tmp73=({struct Cyc_String_pa_PrintArg_struct _tmp212;_tmp212.tag=0U,({struct _fat_ptr _tmp26F=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(n));_tmp212.f1=_tmp26F;});_tmp212;});void*_tmp70[2U];_tmp70[0]=& _tmp72,_tmp70[1]=& _tmp73;({unsigned _tmp271=loc;struct _fat_ptr _tmp270=({const char*_tmp71="expecting struct %s instead of union %s";_tag_fat(_tmp71,sizeof(char),40U);});Cyc_Tcutil_terr(_tmp271,_tmp270,_tag_fat(_tmp70,sizeof(void*),2U));});});else{
# 381
({struct Cyc_String_pa_PrintArg_struct _tmp76=({struct Cyc_String_pa_PrintArg_struct _tmp215;_tmp215.tag=0U,({
struct _fat_ptr _tmp272=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(n));_tmp215.f1=_tmp272;});_tmp215;});struct Cyc_String_pa_PrintArg_struct _tmp77=({struct Cyc_String_pa_PrintArg_struct _tmp214;_tmp214.tag=0U,({struct _fat_ptr _tmp273=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(n));_tmp214.f1=_tmp273;});_tmp214;});void*_tmp74[2U];_tmp74[0]=& _tmp76,_tmp74[1]=& _tmp77;({unsigned _tmp275=loc;struct _fat_ptr _tmp274=({const char*_tmp75="expecting union %s instead of struct %s";_tag_fat(_tmp75,sizeof(char),40U);});Cyc_Tcutil_terr(_tmp275,_tmp274,_tag_fat(_tmp74,sizeof(void*),2U));});});}}
# 376
if(
# 384
(unsigned)tgd &&(int)tgd->v){
if(!((unsigned)ad->impl)|| !((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->tagged)
({struct Cyc_String_pa_PrintArg_struct _tmp7A=({struct Cyc_String_pa_PrintArg_struct _tmp216;_tmp216.tag=0U,({
struct _fat_ptr _tmp276=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(n));_tmp216.f1=_tmp276;});_tmp216;});void*_tmp78[1U];_tmp78[0]=& _tmp7A;({unsigned _tmp278=loc;struct _fat_ptr _tmp277=({const char*_tmp79="@tagged qualfiers don't agree on union %s";_tag_fat(_tmp79,sizeof(char),42U);});Cyc_Tcutil_terr(_tmp278,_tmp277,_tag_fat(_tmp78,sizeof(void*),1U));});});}
# 376
({union Cyc_Absyn_AggrInfo _tmp279=
# 390
Cyc_Absyn_KnownAggr(adp);
# 376
*info=_tmp279;});}
# 374
;_pop_handler();}else{void*_tmp6E=(void*)Cyc_Core_get_exn_thrown();void*_tmp7B=_tmp6E;void*_tmp7C;if(((struct Cyc_Dict_Absent_exn_struct*)_tmp7B)->tag == Cyc_Dict_Absent){_LL6: _LL7: {
# 394
struct Cyc_Absyn_Aggrdecl*ad=({struct Cyc_Absyn_Aggrdecl*_tmp80=_cycalloc(sizeof(*_tmp80));_tmp80->kind=k,_tmp80->sc=Cyc_Absyn_Extern,_tmp80->name=n,_tmp80->tvs=0,_tmp80->impl=0,_tmp80->attributes=0,_tmp80->expected_mem_kind=0;_tmp80;});
Cyc_Tc_tcAggrdecl(te,loc,ad);
adp=Cyc_Tcenv_lookup_aggrdecl(te,loc,n);
({union Cyc_Absyn_AggrInfo _tmp27A=Cyc_Absyn_KnownAggr(adp);*info=_tmp27A;});
# 399
if(*targs != 0){
({struct Cyc_String_pa_PrintArg_struct _tmp7F=({struct Cyc_String_pa_PrintArg_struct _tmp217;_tmp217.tag=0U,({struct _fat_ptr _tmp27B=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(n));_tmp217.f1=_tmp27B;});_tmp217;});void*_tmp7D[1U];_tmp7D[0]=& _tmp7F;({unsigned _tmp27D=loc;struct _fat_ptr _tmp27C=({const char*_tmp7E="declare parameterized type %s before using";_tag_fat(_tmp7E,sizeof(char),43U);});Cyc_Tcutil_terr(_tmp27D,_tmp27C,_tag_fat(_tmp7D,sizeof(void*),1U));});});
return cvtenv;}
# 399
goto _LL5;}}else{_LL8: _tmp7C=_tmp7B;_LL9: {void*exn=_tmp7C;(int)_rethrow(exn);}}_LL5:;}}}
# 405
_tmp69=*adp;goto _LL4;}}else{_LL3: _tmp69=*(_tmp68.KnownAggr).val;_LL4: {struct Cyc_Absyn_Aggrdecl*ad=_tmp69;
# 407
struct Cyc_List_List*tvs=ad->tvs;
struct Cyc_List_List*ts=*targs;
for(0;ts != 0 && tvs != 0;(ts=ts->tl,tvs=tvs->tl)){
struct Cyc_Absyn_Tvar*tv=(struct Cyc_Absyn_Tvar*)tvs->hd;
void*t=(void*)ts->hd;
# 415
{struct _tuple0 _stmttmpD=({struct _tuple0 _tmp218;({void*_tmp27E=Cyc_Absyn_compress_kb(tv->kind);_tmp218.f1=_tmp27E;}),_tmp218.f2=t;_tmp218;});struct _tuple0 _tmp81=_stmttmpD;struct Cyc_Absyn_Tvar*_tmp82;if(((struct Cyc_Absyn_Unknown_kb_Absyn_KindBound_struct*)_tmp81.f1)->tag == 1U){if(((struct Cyc_Absyn_VarType_Absyn_Type_struct*)_tmp81.f2)->tag == 2U){_LLB: _tmp82=((struct Cyc_Absyn_VarType_Absyn_Type_struct*)_tmp81.f2)->f1;_LLC: {struct Cyc_Absyn_Tvar*tv2=_tmp82;
# 417
({struct Cyc_List_List*_tmp27F=Cyc_Tctyp_add_free_tvar(loc,cvtenv.kind_env,tv2);cvtenv.kind_env=_tmp27F;});
({struct Cyc_List_List*_tmp280=Cyc_Tctyp_fast_add_free_tvar_bool(cvtenv.r,cvtenv.free_vars,tv2,1);cvtenv.free_vars=_tmp280;});
continue;}}else{goto _LLD;}}else{_LLD: _LLE:
 goto _LLA;}_LLA:;}{
# 422
struct Cyc_Absyn_Kind*k=Cyc_Tcutil_tvar_kind((struct Cyc_Absyn_Tvar*)tvs->hd,& Cyc_Tcutil_bk);
cvtenv=Cyc_Tctyp_i_check_valid_type(loc,te,cvtenv,k,(void*)ts->hd,1,allow_abs_aggr);
Cyc_Tcutil_check_no_qual(loc,(void*)ts->hd);}}
# 426
if(ts != 0)
({struct Cyc_String_pa_PrintArg_struct _tmp85=({struct Cyc_String_pa_PrintArg_struct _tmp219;_tmp219.tag=0U,({struct _fat_ptr _tmp281=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(ad->name));_tmp219.f1=_tmp281;});_tmp219;});void*_tmp83[1U];_tmp83[0]=& _tmp85;({unsigned _tmp283=loc;struct _fat_ptr _tmp282=({const char*_tmp84="too many parameters for type %s";_tag_fat(_tmp84,sizeof(char),32U);});Cyc_Tcutil_terr(_tmp283,_tmp282,_tag_fat(_tmp83,sizeof(void*),1U));});});
# 426
if(tvs != 0){
# 430
struct Cyc_List_List*hidden_ts=0;
for(0;tvs != 0;tvs=tvs->tl){
struct Cyc_Absyn_Kind*k=Cyc_Tctyp_tvar_inst_kind((struct Cyc_Absyn_Tvar*)tvs->hd,& Cyc_Tcutil_bk,expected_kind,0);
void*e=Cyc_Absyn_new_evar(0,0);
hidden_ts=({struct Cyc_List_List*_tmp86=_cycalloc(sizeof(*_tmp86));_tmp86->hd=e,_tmp86->tl=hidden_ts;_tmp86;});
cvtenv=Cyc_Tctyp_i_check_valid_type(loc,te,cvtenv,k,e,1,allow_abs_aggr);}
# 437
({struct Cyc_List_List*_tmp285=({struct Cyc_List_List*_tmp284=*targs;Cyc_List_imp_append(_tmp284,Cyc_List_imp_rev(hidden_ts));});*targs=_tmp285;});}
# 426
if(
# 439
(allow_abs_aggr && ad->impl == 0)&& !
Cyc_Tcutil_kind_leq(& Cyc_Tcutil_ak,expected_kind))
# 444
ad->expected_mem_kind=1;}}_LL0:;}
# 447
return cvtenv;}
# 364
static struct Cyc_Tctyp_CVTEnv Cyc_Tctyp_i_check_valid_datatype(unsigned loc,struct Cyc_Tcenv_Tenv*te,struct Cyc_Tctyp_CVTEnv cvtenv,struct Cyc_Absyn_Kind*expected_kind,union Cyc_Absyn_DatatypeInfo*info,struct Cyc_List_List**targsp,int allow_abs_aggr){
# 457
struct Cyc_List_List*targs=*targsp;
{union Cyc_Absyn_DatatypeInfo _stmttmpE=*info;union Cyc_Absyn_DatatypeInfo _tmp87=_stmttmpE;struct Cyc_Absyn_Datatypedecl*_tmp88;int _tmp8A;struct _tuple1*_tmp89;if((_tmp87.UnknownDatatype).tag == 1){_LL1: _tmp89=((_tmp87.UnknownDatatype).val).name;_tmp8A=((_tmp87.UnknownDatatype).val).is_extensible;_LL2: {struct _tuple1*n=_tmp89;int is_x=_tmp8A;
# 460
struct Cyc_Absyn_Datatypedecl**tudp;
{struct _handler_cons _tmp8B;_push_handler(& _tmp8B);{int _tmp8D=0;if(setjmp(_tmp8B.handler))_tmp8D=1;if(!_tmp8D){tudp=Cyc_Tcenv_lookup_datatypedecl(te,loc,n);;_pop_handler();}else{void*_tmp8C=(void*)Cyc_Core_get_exn_thrown();void*_tmp8E=_tmp8C;void*_tmp8F;if(((struct Cyc_Dict_Absent_exn_struct*)_tmp8E)->tag == Cyc_Dict_Absent){_LL6: _LL7: {
# 464
struct Cyc_Absyn_Datatypedecl*tud=({struct Cyc_Absyn_Datatypedecl*_tmp93=_cycalloc(sizeof(*_tmp93));_tmp93->sc=Cyc_Absyn_Extern,_tmp93->name=n,_tmp93->tvs=0,_tmp93->fields=0,_tmp93->is_extensible=is_x;_tmp93;});
Cyc_Tc_tcDatatypedecl(te,loc,tud);
tudp=Cyc_Tcenv_lookup_datatypedecl(te,loc,n);
# 468
if(targs != 0){
({struct Cyc_String_pa_PrintArg_struct _tmp92=({struct Cyc_String_pa_PrintArg_struct _tmp21A;_tmp21A.tag=0U,({
struct _fat_ptr _tmp286=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(n));_tmp21A.f1=_tmp286;});_tmp21A;});void*_tmp90[1U];_tmp90[0]=& _tmp92;({unsigned _tmp288=loc;struct _fat_ptr _tmp287=({const char*_tmp91="declare parameterized datatype %s before using";_tag_fat(_tmp91,sizeof(char),47U);});Cyc_Tcutil_terr(_tmp288,_tmp287,_tag_fat(_tmp90,sizeof(void*),1U));});});
return cvtenv;}
# 468
goto _LL5;}}else{_LL8: _tmp8F=_tmp8E;_LL9: {void*exn=_tmp8F;(int)_rethrow(exn);}}_LL5:;}}}
# 477
if(is_x && !(*tudp)->is_extensible)
({struct Cyc_String_pa_PrintArg_struct _tmp96=({struct Cyc_String_pa_PrintArg_struct _tmp21B;_tmp21B.tag=0U,({struct _fat_ptr _tmp289=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(n));_tmp21B.f1=_tmp289;});_tmp21B;});void*_tmp94[1U];_tmp94[0]=& _tmp96;({unsigned _tmp28B=loc;struct _fat_ptr _tmp28A=({const char*_tmp95="datatype %s was not declared @extensible";_tag_fat(_tmp95,sizeof(char),41U);});Cyc_Tcutil_terr(_tmp28B,_tmp28A,_tag_fat(_tmp94,sizeof(void*),1U));});});
# 477
({union Cyc_Absyn_DatatypeInfo _tmp28C=
# 479
Cyc_Absyn_KnownDatatype(tudp);
# 477
*info=_tmp28C;});
# 480
_tmp88=*tudp;goto _LL4;}}else{_LL3: _tmp88=*(_tmp87.KnownDatatype).val;_LL4: {struct Cyc_Absyn_Datatypedecl*tud=_tmp88;
# 483
struct Cyc_List_List*tvs=tud->tvs;
for(0;targs != 0 && tvs != 0;(targs=targs->tl,tvs=tvs->tl)){
void*t=(void*)targs->hd;
struct Cyc_Absyn_Tvar*tv=(struct Cyc_Absyn_Tvar*)tvs->hd;
# 489
{struct _tuple0 _stmttmpF=({struct _tuple0 _tmp21C;({void*_tmp28D=Cyc_Absyn_compress_kb(tv->kind);_tmp21C.f1=_tmp28D;}),_tmp21C.f2=t;_tmp21C;});struct _tuple0 _tmp97=_stmttmpF;struct Cyc_Absyn_Tvar*_tmp98;if(((struct Cyc_Absyn_Unknown_kb_Absyn_KindBound_struct*)_tmp97.f1)->tag == 1U){if(((struct Cyc_Absyn_VarType_Absyn_Type_struct*)_tmp97.f2)->tag == 2U){_LLB: _tmp98=((struct Cyc_Absyn_VarType_Absyn_Type_struct*)_tmp97.f2)->f1;_LLC: {struct Cyc_Absyn_Tvar*tv2=_tmp98;
# 491
({struct Cyc_List_List*_tmp28E=Cyc_Tctyp_add_free_tvar(loc,cvtenv.kind_env,tv2);cvtenv.kind_env=_tmp28E;});
({struct Cyc_List_List*_tmp28F=Cyc_Tctyp_fast_add_free_tvar_bool(cvtenv.r,cvtenv.free_vars,tv2,1);cvtenv.free_vars=_tmp28F;});
continue;}}else{goto _LLD;}}else{_LLD: _LLE:
 goto _LLA;}_LLA:;}{
# 496
struct Cyc_Absyn_Kind*k=Cyc_Tcutil_tvar_kind(tv,& Cyc_Tcutil_bk);
cvtenv=Cyc_Tctyp_i_check_valid_type(loc,te,cvtenv,k,t,1,allow_abs_aggr);
Cyc_Tcutil_check_no_qual(loc,t);}}
# 500
if(targs != 0)
({struct Cyc_String_pa_PrintArg_struct _tmp9B=({struct Cyc_String_pa_PrintArg_struct _tmp21D;_tmp21D.tag=0U,({
struct _fat_ptr _tmp290=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(tud->name));_tmp21D.f1=_tmp290;});_tmp21D;});void*_tmp99[1U];_tmp99[0]=& _tmp9B;({unsigned _tmp292=loc;struct _fat_ptr _tmp291=({const char*_tmp9A="too many type arguments for datatype %s";_tag_fat(_tmp9A,sizeof(char),40U);});Cyc_Tcutil_terr(_tmp292,_tmp291,_tag_fat(_tmp99,sizeof(void*),1U));});});
# 500
if(tvs != 0){
# 505
struct Cyc_List_List*hidden_ts=0;
for(0;tvs != 0;tvs=tvs->tl){
struct Cyc_Absyn_Kind*k1=Cyc_Tctyp_tvar_inst_kind((struct Cyc_Absyn_Tvar*)tvs->hd,& Cyc_Tcutil_bk,expected_kind,0);
void*e=Cyc_Absyn_new_evar(0,0);
hidden_ts=({struct Cyc_List_List*_tmp9C=_cycalloc(sizeof(*_tmp9C));_tmp9C->hd=e,_tmp9C->tl=hidden_ts;_tmp9C;});
cvtenv=Cyc_Tctyp_i_check_valid_type(loc,te,cvtenv,k1,e,1,allow_abs_aggr);}
# 512
({struct Cyc_List_List*_tmp294=({struct Cyc_List_List*_tmp293=*targsp;Cyc_List_imp_append(_tmp293,Cyc_List_imp_rev(hidden_ts));});*targsp=_tmp294;});}
# 500
goto _LL0;}}_LL0:;}
# 516
return cvtenv;}
# 364
static struct Cyc_Tctyp_CVTEnv Cyc_Tctyp_i_check_valid_datatype_field(unsigned loc,struct Cyc_Tcenv_Tenv*te,struct Cyc_Tctyp_CVTEnv cvtenv,struct Cyc_Absyn_Kind*expected_kind,union Cyc_Absyn_DatatypeFieldInfo*info,struct Cyc_List_List*targs,int allow_abs_aggr){
# 526
{union Cyc_Absyn_DatatypeFieldInfo _stmttmp10=*info;union Cyc_Absyn_DatatypeFieldInfo _tmp9D=_stmttmp10;struct Cyc_Absyn_Datatypefield*_tmp9F;struct Cyc_Absyn_Datatypedecl*_tmp9E;int _tmpA2;struct _tuple1*_tmpA1;struct _tuple1*_tmpA0;if((_tmp9D.UnknownDatatypefield).tag == 1){_LL1: _tmpA0=((_tmp9D.UnknownDatatypefield).val).datatype_name;_tmpA1=((_tmp9D.UnknownDatatypefield).val).field_name;_tmpA2=((_tmp9D.UnknownDatatypefield).val).is_extensible;_LL2: {struct _tuple1*tname=_tmpA0;struct _tuple1*fname=_tmpA1;int is_x=_tmpA2;
# 529
struct Cyc_Absyn_Datatypedecl*tud=*Cyc_Tcenv_lookup_datatypedecl(te,loc,tname);
struct Cyc_Absyn_Datatypefield*tuf;
# 534
{struct Cyc_List_List*fs=(struct Cyc_List_List*)((struct Cyc_Core_Opt*)_check_null(tud->fields))->v;for(0;1;fs=fs->tl){
if(fs == 0)({void*_tmpA3=0U;({int(*_tmp296)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmpA5)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmpA5;});struct _fat_ptr _tmp295=({const char*_tmpA4="Tcutil found a bad datatypefield";_tag_fat(_tmpA4,sizeof(char),33U);});_tmp296(_tmp295,_tag_fat(_tmpA3,sizeof(void*),0U));});});if(
Cyc_Absyn_qvar_cmp(((struct Cyc_Absyn_Datatypefield*)fs->hd)->name,fname)== 0){
tuf=(struct Cyc_Absyn_Datatypefield*)fs->hd;
break;}}}
# 541
({union Cyc_Absyn_DatatypeFieldInfo _tmp297=Cyc_Absyn_KnownDatatypefield(tud,tuf);*info=_tmp297;});
_tmp9E=tud;_tmp9F=tuf;goto _LL4;}}else{_LL3: _tmp9E=((_tmp9D.KnownDatatypefield).val).f1;_tmp9F=((_tmp9D.KnownDatatypefield).val).f2;_LL4: {struct Cyc_Absyn_Datatypedecl*tud=_tmp9E;struct Cyc_Absyn_Datatypefield*tuf=_tmp9F;
# 545
struct Cyc_List_List*tvs=tud->tvs;
for(0;targs != 0 && tvs != 0;(targs=targs->tl,tvs=tvs->tl)){
void*t=(void*)targs->hd;
struct Cyc_Absyn_Tvar*tv=(struct Cyc_Absyn_Tvar*)tvs->hd;
# 551
{struct _tuple0 _stmttmp11=({struct _tuple0 _tmp21E;({void*_tmp298=Cyc_Absyn_compress_kb(tv->kind);_tmp21E.f1=_tmp298;}),_tmp21E.f2=t;_tmp21E;});struct _tuple0 _tmpA6=_stmttmp11;struct Cyc_Absyn_Tvar*_tmpA7;if(((struct Cyc_Absyn_Unknown_kb_Absyn_KindBound_struct*)_tmpA6.f1)->tag == 1U){if(((struct Cyc_Absyn_VarType_Absyn_Type_struct*)_tmpA6.f2)->tag == 2U){_LL6: _tmpA7=((struct Cyc_Absyn_VarType_Absyn_Type_struct*)_tmpA6.f2)->f1;_LL7: {struct Cyc_Absyn_Tvar*tv2=_tmpA7;
# 553
({struct Cyc_List_List*_tmp299=Cyc_Tctyp_add_free_tvar(loc,cvtenv.kind_env,tv2);cvtenv.kind_env=_tmp299;});
({struct Cyc_List_List*_tmp29A=Cyc_Tctyp_fast_add_free_tvar_bool(cvtenv.r,cvtenv.free_vars,tv2,1);cvtenv.free_vars=_tmp29A;});
continue;}}else{goto _LL8;}}else{_LL8: _LL9:
 goto _LL5;}_LL5:;}{
# 558
struct Cyc_Absyn_Kind*k=Cyc_Tcutil_tvar_kind(tv,& Cyc_Tcutil_bk);
cvtenv=Cyc_Tctyp_i_check_valid_type(loc,te,cvtenv,k,t,1,allow_abs_aggr);
Cyc_Tcutil_check_no_qual(loc,t);}}
# 562
if(targs != 0)
({struct Cyc_String_pa_PrintArg_struct _tmpAA=({struct Cyc_String_pa_PrintArg_struct _tmp220;_tmp220.tag=0U,({
struct _fat_ptr _tmp29B=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(tud->name));_tmp220.f1=_tmp29B;});_tmp220;});struct Cyc_String_pa_PrintArg_struct _tmpAB=({struct Cyc_String_pa_PrintArg_struct _tmp21F;_tmp21F.tag=0U,({struct _fat_ptr _tmp29C=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(tuf->name));_tmp21F.f1=_tmp29C;});_tmp21F;});void*_tmpA8[2U];_tmpA8[0]=& _tmpAA,_tmpA8[1]=& _tmpAB;({unsigned _tmp29E=loc;struct _fat_ptr _tmp29D=({const char*_tmpA9="too many type arguments for datatype %s.%s";_tag_fat(_tmpA9,sizeof(char),43U);});Cyc_Tcutil_terr(_tmp29E,_tmp29D,_tag_fat(_tmpA8,sizeof(void*),2U));});});
# 562
if(tvs != 0)
# 566
({struct Cyc_String_pa_PrintArg_struct _tmpAE=({struct Cyc_String_pa_PrintArg_struct _tmp222;_tmp222.tag=0U,({
struct _fat_ptr _tmp29F=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(tud->name));_tmp222.f1=_tmp29F;});_tmp222;});struct Cyc_String_pa_PrintArg_struct _tmpAF=({struct Cyc_String_pa_PrintArg_struct _tmp221;_tmp221.tag=0U,({struct _fat_ptr _tmp2A0=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(tuf->name));_tmp221.f1=_tmp2A0;});_tmp221;});void*_tmpAC[2U];_tmpAC[0]=& _tmpAE,_tmpAC[1]=& _tmpAF;({unsigned _tmp2A2=loc;struct _fat_ptr _tmp2A1=({const char*_tmpAD="too few type arguments for datatype %s.%s";_tag_fat(_tmpAD,sizeof(char),42U);});Cyc_Tcutil_terr(_tmp2A2,_tmp2A1,_tag_fat(_tmpAC,sizeof(void*),2U));});});
# 562
goto _LL0;}}_LL0:;}
# 570
return cvtenv;}
# 364
static struct Cyc_Tctyp_CVTEnv Cyc_Tctyp_i_check_valid_type_app(unsigned loc,struct Cyc_Tcenv_Tenv*te,struct Cyc_Tctyp_CVTEnv cvtenv,struct Cyc_Absyn_Kind*expected_kind,void*c,struct Cyc_List_List**targsp,int put_in_effect,int allow_abs_aggr){
# 578
struct Cyc_List_List*ts=*targsp;
{void*_tmpB0=c;union Cyc_Absyn_DatatypeFieldInfo*_tmpB1;union Cyc_Absyn_DatatypeInfo*_tmpB2;union Cyc_Absyn_AggrInfo*_tmpB3;struct Cyc_List_List*_tmpB4;struct Cyc_Absyn_Enumdecl**_tmpB6;struct _tuple1*_tmpB5;switch(*((int*)_tmpB0)){case 1U: _LL1: _LL2:
# 581
 goto _LL4;case 2U: _LL3: _LL4: goto _LL6;case 0U: _LL5: _LL6: goto _LL8;case 7U: _LL7: _LL8:
 goto _LLA;case 6U: _LL9: _LLA: goto _LLC;case 5U: _LLB: _LLC: goto _LLE;case 12U: _LLD: _LLE:
 goto _LL10;case 11U: _LLF: _LL10: goto _LL12;case 14U: _LL11: _LL12: goto _LL14;case 17U: _LL13: _LL14:
# 585
 if(ts != 0)
({struct Cyc_String_pa_PrintArg_struct _tmpBA=({struct Cyc_String_pa_PrintArg_struct _tmp223;_tmp223.tag=0U,({struct _fat_ptr _tmp2A3=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string((void*)({struct Cyc_Absyn_AppType_Absyn_Type_struct*_tmpBB=_cycalloc(sizeof(*_tmpBB));_tmpBB->tag=0U,_tmpBB->f1=c,_tmpBB->f2=ts;_tmpBB;})));_tmp223.f1=_tmp2A3;});_tmp223;});void*_tmpB7[1U];_tmpB7[0]=& _tmpBA;({int(*_tmp2A5)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmpB9)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmpB9;});struct _fat_ptr _tmp2A4=({const char*_tmpB8="%s applied to argument(s)";_tag_fat(_tmpB8,sizeof(char),26U);});_tmp2A5(_tmp2A4,_tag_fat(_tmpB7,sizeof(void*),1U));});});
# 585
goto _LL0;case 9U: _LL15: _LL16:
# 589
 for(0;ts != 0;ts=ts->tl){
cvtenv=Cyc_Tctyp_i_check_valid_type(loc,te,cvtenv,& Cyc_Tcutil_ek,(void*)ts->hd,1,1);}
goto _LL0;case 4U: _LL17: _LL18:
# 593
 if(Cyc_List_length(ts)!= 1)
({void*_tmpBC=0U;({int(*_tmp2A7)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmpBE)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmpBE;});struct _fat_ptr _tmp2A6=({const char*_tmpBD="tag_t applied to wrong number of arguments";_tag_fat(_tmpBD,sizeof(char),43U);});_tmp2A7(_tmp2A6,_tag_fat(_tmpBC,sizeof(void*),0U));});});
# 593
cvtenv=
# 595
Cyc_Tctyp_i_check_valid_type(loc,te,cvtenv,& Cyc_Tcutil_ik,(void*)((struct Cyc_List_List*)_check_null(ts))->hd,0,1);
goto _LL0;case 15U: _LL19: _tmpB5=((struct Cyc_Absyn_EnumCon_Absyn_TyCon_struct*)_tmpB0)->f1;_tmpB6=(struct Cyc_Absyn_Enumdecl**)&((struct Cyc_Absyn_EnumCon_Absyn_TyCon_struct*)_tmpB0)->f2;_LL1A: {struct _tuple1*n=_tmpB5;struct Cyc_Absyn_Enumdecl**edp=_tmpB6;
# 598
if(ts != 0)({void*_tmpBF=0U;({int(*_tmp2A9)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmpC1)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmpC1;});struct _fat_ptr _tmp2A8=({const char*_tmpC0="enum applied to argument(s)";_tag_fat(_tmpC0,sizeof(char),28U);});_tmp2A9(_tmp2A8,_tag_fat(_tmpBF,sizeof(void*),0U));});});if(
*edp == 0 ||((struct Cyc_Absyn_Enumdecl*)_check_null(*edp))->fields == 0){
struct _handler_cons _tmpC2;_push_handler(& _tmpC2);{int _tmpC4=0;if(setjmp(_tmpC2.handler))_tmpC4=1;if(!_tmpC4){
{struct Cyc_Absyn_Enumdecl**ed=Cyc_Tcenv_lookup_enumdecl(te,loc,n);
*edp=*ed;}
# 601
;_pop_handler();}else{void*_tmpC3=(void*)Cyc_Core_get_exn_thrown();void*_tmpC5=_tmpC3;void*_tmpC6;if(((struct Cyc_Dict_Absent_exn_struct*)_tmpC5)->tag == Cyc_Dict_Absent){_LL2C: _LL2D: {
# 605
struct Cyc_Absyn_Enumdecl*ed_orig=({struct Cyc_Absyn_Enumdecl*_tmpC7=_cycalloc(sizeof(*_tmpC7));_tmpC7->sc=Cyc_Absyn_Extern,_tmpC7->name=n,_tmpC7->fields=0;_tmpC7;});
Cyc_Tc_tcEnumdecl(te,loc,ed_orig);{
struct Cyc_Absyn_Enumdecl**ed=Cyc_Tcenv_lookup_enumdecl(te,loc,n);
*edp=*ed;
goto _LL2B;}}}else{_LL2E: _tmpC6=_tmpC5;_LL2F: {void*exn=_tmpC6;(int)_rethrow(exn);}}_LL2B:;}}}
# 598
goto _LL0;}case 16U: _LL1B: _tmpB4=((struct Cyc_Absyn_AnonEnumCon_Absyn_TyCon_struct*)_tmpB0)->f1;_LL1C: {struct Cyc_List_List*fs=_tmpB4;
# 613
if(ts != 0)({void*_tmpC8=0U;({int(*_tmp2AB)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmpCA)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmpCA;});struct _fat_ptr _tmp2AA=({const char*_tmpC9="enum applied to argument(s)";_tag_fat(_tmpC9,sizeof(char),28U);});_tmp2AB(_tmp2AA,_tag_fat(_tmpC8,sizeof(void*),0U));});});{struct Cyc_List_List*prev_fields=0;
# 616
unsigned tag_count=0U;
for(0;fs != 0;fs=fs->tl){
struct Cyc_Absyn_Enumfield*f=(struct Cyc_Absyn_Enumfield*)fs->hd;
if(({int(*_tmp2AD)(int(*compare)(struct _fat_ptr*,struct _fat_ptr*),struct Cyc_List_List*l,struct _fat_ptr*x)=({int(*_tmpCB)(int(*compare)(struct _fat_ptr*,struct _fat_ptr*),struct Cyc_List_List*l,struct _fat_ptr*x)=(int(*)(int(*compare)(struct _fat_ptr*,struct _fat_ptr*),struct Cyc_List_List*l,struct _fat_ptr*x))Cyc_List_mem;_tmpCB;});struct Cyc_List_List*_tmp2AC=prev_fields;_tmp2AD(Cyc_strptrcmp,_tmp2AC,(*f->name).f2);}))
({struct Cyc_String_pa_PrintArg_struct _tmpCE=({struct Cyc_String_pa_PrintArg_struct _tmp224;_tmp224.tag=0U,_tmp224.f1=(struct _fat_ptr)((struct _fat_ptr)*(*f->name).f2);_tmp224;});void*_tmpCC[1U];_tmpCC[0]=& _tmpCE;({unsigned _tmp2AF=f->loc;struct _fat_ptr _tmp2AE=({const char*_tmpCD="duplicate enum field name %s";_tag_fat(_tmpCD,sizeof(char),29U);});Cyc_Tcutil_terr(_tmp2AF,_tmp2AE,_tag_fat(_tmpCC,sizeof(void*),1U));});});else{
# 622
prev_fields=({struct Cyc_List_List*_tmpCF=_cycalloc(sizeof(*_tmpCF));_tmpCF->hd=(*f->name).f2,_tmpCF->tl=prev_fields;_tmpCF;});}
if(f->tag == 0)
({struct Cyc_Absyn_Exp*_tmp2B0=Cyc_Absyn_uint_exp(tag_count,f->loc);f->tag=_tmp2B0;});else{
if(!Cyc_Tcutil_is_const_exp((struct Cyc_Absyn_Exp*)_check_null(f->tag)))
({struct Cyc_String_pa_PrintArg_struct _tmpD2=({struct Cyc_String_pa_PrintArg_struct _tmp225;_tmp225.tag=0U,_tmp225.f1=(struct _fat_ptr)((struct _fat_ptr)*(*f->name).f2);_tmp225;});void*_tmpD0[1U];_tmpD0[0]=& _tmpD2;({unsigned _tmp2B2=loc;struct _fat_ptr _tmp2B1=({const char*_tmpD1="enum field %s: expression is not constant";_tag_fat(_tmpD1,sizeof(char),42U);});Cyc_Tcutil_terr(_tmp2B2,_tmp2B1,_tag_fat(_tmpD0,sizeof(void*),1U));});});}{
# 623
unsigned t1=(
# 627
Cyc_Evexp_eval_const_uint_exp((struct Cyc_Absyn_Exp*)_check_null(f->tag))).f1;
tag_count=t1 + (unsigned)1;}}
# 630
goto _LL0;}}case 10U: _LL1D: _LL1E:
# 632
 if(Cyc_List_length(ts)!= 1)({void*_tmpD3=0U;({int(*_tmp2B4)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmpD5)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmpD5;});struct _fat_ptr _tmp2B3=({const char*_tmpD4="regions has wrong number of arguments";_tag_fat(_tmpD4,sizeof(char),38U);});_tmp2B4(_tmp2B3,_tag_fat(_tmpD3,sizeof(void*),0U));});});cvtenv=
Cyc_Tctyp_i_check_valid_type(loc,te,cvtenv,& Cyc_Tcutil_tak,(void*)((struct Cyc_List_List*)_check_null(ts))->hd,1,1);goto _LL0;case 3U: _LL1F: _LL20:
# 635
 if(Cyc_List_length(ts)!= 1)({void*_tmpD6=0U;({int(*_tmp2B6)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmpD8)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmpD8;});struct _fat_ptr _tmp2B5=({const char*_tmpD7="region_t has wrong number of arguments";_tag_fat(_tmpD7,sizeof(char),39U);});_tmp2B6(_tmp2B5,_tag_fat(_tmpD6,sizeof(void*),0U));});});cvtenv=
Cyc_Tctyp_i_check_valid_type(loc,te,cvtenv,& Cyc_Tcutil_trk,(void*)((struct Cyc_List_List*)_check_null(ts))->hd,1,1);
goto _LL0;case 13U: _LL21: _LL22:
# 639
 if(Cyc_List_length(ts)!= 1)({void*_tmpD9=0U;({int(*_tmp2B8)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmpDB)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmpDB;});struct _fat_ptr _tmp2B7=({const char*_tmpDA="@thin has wrong number of arguments";_tag_fat(_tmpDA,sizeof(char),36U);});_tmp2B8(_tmp2B7,_tag_fat(_tmpD9,sizeof(void*),0U));});});cvtenv=
Cyc_Tctyp_i_check_valid_type(loc,te,cvtenv,& Cyc_Tcutil_ik,(void*)((struct Cyc_List_List*)_check_null(ts))->hd,0,1);
goto _LL0;case 8U: _LL23: _LL24:
# 643
 if(Cyc_List_length(ts)!= 1)({void*_tmpDC=0U;({int(*_tmp2BA)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmpDE)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmpDE;});struct _fat_ptr _tmp2B9=({const char*_tmpDD="access has wrong number of arguments";_tag_fat(_tmpDD,sizeof(char),37U);});_tmp2BA(_tmp2B9,_tag_fat(_tmpDC,sizeof(void*),0U));});});cvtenv=
Cyc_Tctyp_i_check_valid_type(loc,te,cvtenv,& Cyc_Tcutil_trk,(void*)((struct Cyc_List_List*)_check_null(ts))->hd,1,1);goto _LL0;case 20U: _LL25: _tmpB3=(union Cyc_Absyn_AggrInfo*)&((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)_tmpB0)->f1;_LL26: {union Cyc_Absyn_AggrInfo*info=_tmpB3;
# 646
cvtenv=Cyc_Tctyp_i_check_valid_aggr(loc,te,cvtenv,expected_kind,info,targsp,allow_abs_aggr);
# 648
goto _LL0;}case 18U: _LL27: _tmpB2=(union Cyc_Absyn_DatatypeInfo*)&((struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct*)_tmpB0)->f1;_LL28: {union Cyc_Absyn_DatatypeInfo*info=_tmpB2;
# 650
cvtenv=Cyc_Tctyp_i_check_valid_datatype(loc,te,cvtenv,expected_kind,info,targsp,allow_abs_aggr);
# 652
goto _LL0;}default: _LL29: _tmpB1=(union Cyc_Absyn_DatatypeFieldInfo*)&((struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct*)_tmpB0)->f1;_LL2A: {union Cyc_Absyn_DatatypeFieldInfo*info=_tmpB1;
# 654
cvtenv=Cyc_Tctyp_i_check_valid_datatype_field(loc,te,cvtenv,expected_kind,info,ts,allow_abs_aggr);
# 656
goto _LL0;}}_LL0:;}
# 658
return cvtenv;}struct _tuple16{enum Cyc_Absyn_Format_Type f1;void*f2;};struct _tuple17{struct Cyc_Absyn_Tqual f1;void*f2;};struct _tuple18{struct Cyc_Absyn_Tvar*f1;void*f2;};
# 364
static struct Cyc_Tctyp_CVTEnv Cyc_Tctyp_i_check_valid_type(unsigned loc,struct Cyc_Tcenv_Tenv*te,struct Cyc_Tctyp_CVTEnv cvtenv,struct Cyc_Absyn_Kind*expected_kind,void*t,int put_in_effect,int allow_abs_aggr){
# 670
{void*_stmttmp12=Cyc_Tcutil_compress(t);void*_tmpDF=_stmttmp12;void**_tmpE3;struct Cyc_Absyn_Typedefdecl**_tmpE2;struct Cyc_List_List**_tmpE1;struct _tuple1*_tmpE0;struct Cyc_List_List*_tmpE5;enum Cyc_Absyn_AggrKind _tmpE4;struct Cyc_List_List*_tmpE6;struct Cyc_List_List*_tmpF5;struct Cyc_List_List*_tmpF4;struct Cyc_List_List**_tmpF3;struct Cyc_Absyn_Exp*_tmpF2;struct Cyc_List_List**_tmpF1;struct Cyc_Absyn_Exp*_tmpF0;struct Cyc_List_List*_tmpEF;struct Cyc_List_List*_tmpEE;struct Cyc_Absyn_VarargInfo*_tmpED;int _tmpEC;struct Cyc_List_List*_tmpEB;void*_tmpEA;struct Cyc_Absyn_Tqual*_tmpE9;void**_tmpE8;struct Cyc_List_List**_tmpE7;unsigned _tmpFA;void*_tmpF9;struct Cyc_Absyn_Exp**_tmpF8;struct Cyc_Absyn_Tqual*_tmpF7;void*_tmpF6;struct Cyc_Absyn_Exp*_tmpFB;struct Cyc_Absyn_Exp*_tmpFC;void*_tmp102;void*_tmp101;void*_tmp100;void*_tmpFF;struct Cyc_Absyn_Tqual*_tmpFE;void*_tmpFD;void***_tmp104;void*_tmp103;struct Cyc_Absyn_Tvar*_tmp105;void**_tmp107;struct Cyc_Core_Opt**_tmp106;struct Cyc_List_List**_tmp109;void*_tmp108;switch(*((int*)_tmpDF)){case 0U: _LL1: _tmp108=(void*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmpDF)->f1;_tmp109=(struct Cyc_List_List**)&((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmpDF)->f2;_LL2: {void*c=_tmp108;struct Cyc_List_List**targsp=_tmp109;
# 673
cvtenv=Cyc_Tctyp_i_check_valid_type_app(loc,te,cvtenv,expected_kind,c,targsp,put_in_effect,allow_abs_aggr);
# 675
goto _LL0;}case 1U: _LL3: _tmp106=(struct Cyc_Core_Opt**)&((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmpDF)->f1;_tmp107=(void**)&((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmpDF)->f2;_LL4: {struct Cyc_Core_Opt**k=_tmp106;void**ref=_tmp107;
# 678
if(*k == 0 ||
 Cyc_Tcutil_kind_leq(expected_kind,(struct Cyc_Absyn_Kind*)((struct Cyc_Core_Opt*)_check_null(*k))->v)&& !Cyc_Tcutil_kind_leq((struct Cyc_Absyn_Kind*)((struct Cyc_Core_Opt*)_check_null(*k))->v,expected_kind))
({struct Cyc_Core_Opt*_tmp2BB=Cyc_Tcutil_kind_to_opt(expected_kind);*k=_tmp2BB;});
# 678
if(
# 681
((cvtenv.fn_result && cvtenv.generalize_evars)&&(int)expected_kind->kind == (int)Cyc_Absyn_RgnKind)&& !te->tempest_generalize){
# 684
if((int)expected_kind->aliasqual == (int)Cyc_Absyn_Unique)
*ref=Cyc_Absyn_unique_rgn_type;else{
# 687
*ref=Cyc_Absyn_heap_rgn_type;}}else{
# 689
if((cvtenv.generalize_evars &&(int)expected_kind->kind != (int)Cyc_Absyn_BoolKind)&&(int)expected_kind->kind != (int)Cyc_Absyn_PtrBndKind){
# 693
struct Cyc_Absyn_Tvar*v=Cyc_Tcutil_new_tvar((void*)({struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*_tmp10A=_cycalloc(sizeof(*_tmp10A));_tmp10A->tag=2U,_tmp10A->f1=0,_tmp10A->f2=expected_kind;_tmp10A;}));
({void*_tmp2BC=Cyc_Absyn_var_type(v);*ref=_tmp2BC;});
_tmp105=v;goto _LL6;}else{
# 697
({struct Cyc_List_List*_tmp2BD=Cyc_Tctyp_add_free_evar(cvtenv.r,cvtenv.free_evars,t,put_in_effect);cvtenv.free_evars=_tmp2BD;});}}
goto _LL0;}case 2U: _LL5: _tmp105=((struct Cyc_Absyn_VarType_Absyn_Type_struct*)_tmpDF)->f1;_LL6: {struct Cyc_Absyn_Tvar*v=_tmp105;
# 701
{void*_stmttmp13=Cyc_Absyn_compress_kb(v->kind);void*_tmp10B=_stmttmp13;struct Cyc_Core_Opt**_tmp10C;if(((struct Cyc_Absyn_Unknown_kb_Absyn_KindBound_struct*)_tmp10B)->tag == 1U){_LL1A: _tmp10C=(struct Cyc_Core_Opt**)&((struct Cyc_Absyn_Unknown_kb_Absyn_KindBound_struct*)_tmp10B)->f1;_LL1B: {struct Cyc_Core_Opt**f=_tmp10C;
# 703
({struct Cyc_Core_Opt*_tmp2BF=({struct Cyc_Core_Opt*_tmp10E=_cycalloc(sizeof(*_tmp10E));({void*_tmp2BE=(void*)({struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*_tmp10D=_cycalloc(sizeof(*_tmp10D));_tmp10D->tag=2U,_tmp10D->f1=0,_tmp10D->f2=expected_kind;_tmp10D;});_tmp10E->v=_tmp2BE;});_tmp10E;});*f=_tmp2BF;});goto _LL19;}}else{_LL1C: _LL1D:
 goto _LL19;}_LL19:;}
# 708
({struct Cyc_List_List*_tmp2C0=Cyc_Tctyp_add_free_tvar(loc,cvtenv.kind_env,v);cvtenv.kind_env=_tmp2C0;});
# 711
({struct Cyc_List_List*_tmp2C1=Cyc_Tctyp_fast_add_free_tvar_bool(cvtenv.r,cvtenv.free_vars,v,put_in_effect);cvtenv.free_vars=_tmp2C1;});
# 713
{void*_stmttmp14=Cyc_Absyn_compress_kb(v->kind);void*_tmp10F=_stmttmp14;struct Cyc_Absyn_Kind*_tmp111;struct Cyc_Core_Opt**_tmp110;if(((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp10F)->tag == 2U){_LL1F: _tmp110=(struct Cyc_Core_Opt**)&((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp10F)->f1;_tmp111=((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp10F)->f2;_LL20: {struct Cyc_Core_Opt**f=_tmp110;struct Cyc_Absyn_Kind*k=_tmp111;
# 716
if(Cyc_Tcutil_kind_leq(expected_kind,k))
({struct Cyc_Core_Opt*_tmp2C3=({struct Cyc_Core_Opt*_tmp113=_cycalloc(sizeof(*_tmp113));({void*_tmp2C2=(void*)({struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*_tmp112=_cycalloc(sizeof(*_tmp112));_tmp112->tag=2U,_tmp112->f1=0,_tmp112->f2=expected_kind;_tmp112;});_tmp113->v=_tmp2C2;});_tmp113;});*f=_tmp2C3;});
# 716
goto _LL1E;}}else{_LL21: _LL22:
# 719
 goto _LL1E;}_LL1E:;}
# 721
goto _LL0;}case 10U: _LL7: _tmp103=(((struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct*)_tmpDF)->f1)->r;_tmp104=(void***)&((struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct*)_tmpDF)->f2;_LL8: {void*td=_tmp103;void***topt=_tmp104;
# 727
void*new_typ=Cyc_Tcutil_copy_type(Cyc_Tcutil_compress(t));
{void*_tmp114=td;struct Cyc_Absyn_Datatypedecl*_tmp115;struct Cyc_Absyn_Enumdecl*_tmp116;struct Cyc_Absyn_Aggrdecl*_tmp117;switch(*((int*)_tmp114)){case 0U: _LL24: _tmp117=((struct Cyc_Absyn_Aggr_td_Absyn_Raw_typedecl_struct*)_tmp114)->f1;_LL25: {struct Cyc_Absyn_Aggrdecl*ad=_tmp117;
# 730
if(te->in_extern_c_include)
ad->sc=Cyc_Absyn_ExternC;
# 730
Cyc_Tc_tcAggrdecl(te,loc,ad);
# 732
goto _LL23;}case 1U: _LL26: _tmp116=((struct Cyc_Absyn_Enum_td_Absyn_Raw_typedecl_struct*)_tmp114)->f1;_LL27: {struct Cyc_Absyn_Enumdecl*ed=_tmp116;
# 734
if(te->in_extern_c_include)
ed->sc=Cyc_Absyn_ExternC;
# 734
Cyc_Tc_tcEnumdecl(te,loc,ed);
# 736
goto _LL23;}default: _LL28: _tmp115=((struct Cyc_Absyn_Datatype_td_Absyn_Raw_typedecl_struct*)_tmp114)->f1;_LL29: {struct Cyc_Absyn_Datatypedecl*dd=_tmp115;
# 738
Cyc_Tc_tcDatatypedecl(te,loc,dd);goto _LL23;}}_LL23:;}
# 740
({void**_tmp2C4=({void**_tmp118=_cycalloc(sizeof(*_tmp118));*_tmp118=new_typ;_tmp118;});*topt=_tmp2C4;});
return Cyc_Tctyp_i_check_valid_type(loc,te,cvtenv,expected_kind,new_typ,put_in_effect,allow_abs_aggr);}case 3U: _LL9: _tmpFD=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmpDF)->f1).elt_type;_tmpFE=(struct Cyc_Absyn_Tqual*)&(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmpDF)->f1).elt_tq;_tmpFF=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmpDF)->f1).ptr_atts).rgn;_tmp100=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmpDF)->f1).ptr_atts).nullable;_tmp101=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmpDF)->f1).ptr_atts).bounds;_tmp102=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmpDF)->f1).ptr_atts).zero_term;_LLA: {void*t1=_tmpFD;struct Cyc_Absyn_Tqual*tqp=_tmpFE;void*rgn_type=_tmpFF;void*nullable=_tmp100;void*b=_tmp101;void*zt=_tmp102;
# 746
int is_zero_terminated;
# 748
cvtenv=Cyc_Tctyp_i_check_valid_type(loc,te,cvtenv,& Cyc_Tcutil_tak,t1,1,1);
({int _tmp2C5=Cyc_Tcutil_extract_const_from_typedef(loc,tqp->print_const,t1);tqp->real_const=_tmp2C5;});{
struct Cyc_Absyn_Kind*k;
{enum Cyc_Absyn_AliasQual _stmttmp15=expected_kind->aliasqual;enum Cyc_Absyn_AliasQual _tmp119=_stmttmp15;switch(_tmp119){case Cyc_Absyn_Aliasable: _LL2B: _LL2C:
 k=& Cyc_Tcutil_rk;goto _LL2A;case Cyc_Absyn_Unique: _LL2D: _LL2E:
 k=& Cyc_Tcutil_urk;goto _LL2A;case Cyc_Absyn_Top: _LL2F: _LL30:
 goto _LL32;default: _LL31: _LL32:
 k=& Cyc_Tcutil_trk;goto _LL2A;}_LL2A:;}
# 757
cvtenv=Cyc_Tctyp_i_check_valid_type(loc,te,cvtenv,k,rgn_type,1,1);
# 760
cvtenv=Cyc_Tctyp_i_check_valid_type(loc,te,cvtenv,& Cyc_Tcutil_boolk,zt,0,1);
cvtenv=Cyc_Tctyp_i_check_valid_type(loc,te,cvtenv,& Cyc_Tcutil_boolk,nullable,0,1);
({void*_tmp2C6=zt;Cyc_Unify_unify(_tmp2C6,Cyc_Tcutil_is_char_type(t1)?Cyc_Absyn_true_type: Cyc_Absyn_false_type);});
is_zero_terminated=Cyc_Absyn_type2bool(0,zt);
if(is_zero_terminated){
# 766
if(!Cyc_Tcutil_admits_zero(t1))
({struct Cyc_String_pa_PrintArg_struct _tmp11C=({struct Cyc_String_pa_PrintArg_struct _tmp226;_tmp226.tag=0U,({
struct _fat_ptr _tmp2C7=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t1));_tmp226.f1=_tmp2C7;});_tmp226;});void*_tmp11A[1U];_tmp11A[0]=& _tmp11C;({unsigned _tmp2C9=loc;struct _fat_ptr _tmp2C8=({const char*_tmp11B="cannot have a pointer to zero-terminated %s elements";_tag_fat(_tmp11B,sizeof(char),53U);});Cyc_Tcutil_terr(_tmp2C9,_tmp2C8,_tag_fat(_tmp11A,sizeof(void*),1U));});});}
# 764
cvtenv=
# 771
Cyc_Tctyp_i_check_valid_type(loc,te,cvtenv,& Cyc_Tcutil_ptrbk,b,0,allow_abs_aggr);{
struct Cyc_Absyn_Exp*e=({void*_tmp2CA=Cyc_Absyn_bounds_one();Cyc_Tcutil_get_bounds_exp(_tmp2CA,b);});
if(e != 0){
struct _tuple12 _stmttmp16=Cyc_Evexp_eval_const_uint_exp(e);int _tmp11E;unsigned _tmp11D;_LL34: _tmp11D=_stmttmp16.f1;_tmp11E=_stmttmp16.f2;_LL35: {unsigned sz=_tmp11D;int known=_tmp11E;
if(is_zero_terminated &&(!known || sz < (unsigned)1))
({void*_tmp11F=0U;({unsigned _tmp2CC=loc;struct _fat_ptr _tmp2CB=({const char*_tmp120="zero-terminated pointer cannot point to empty sequence";_tag_fat(_tmp120,sizeof(char),55U);});Cyc_Tcutil_terr(_tmp2CC,_tmp2CB,_tag_fat(_tmp11F,sizeof(void*),0U));});});}}
# 773
goto _LL0;}}}case 9U: _LLB: _tmpFC=((struct Cyc_Absyn_ValueofType_Absyn_Type_struct*)_tmpDF)->f1;_LLC: {struct Cyc_Absyn_Exp*e=_tmpFC;
# 783
({struct Cyc_Tcenv_Tenv*_tmp2CD=Cyc_Tcenv_allow_valueof(te);Cyc_Tcexp_tcExp(_tmp2CD,0,e);});
if(!Cyc_Tcutil_coerce_uint_type(e))
({void*_tmp121=0U;({unsigned _tmp2CF=loc;struct _fat_ptr _tmp2CE=({const char*_tmp122="valueof_t requires an int expression";_tag_fat(_tmp122,sizeof(char),37U);});Cyc_Tcutil_terr(_tmp2CF,_tmp2CE,_tag_fat(_tmp121,sizeof(void*),0U));});});
# 784
cvtenv=
# 786
Cyc_Tctyp_i_check_valid_type_level_exp(e,te,cvtenv);
goto _LL0;}case 11U: _LLD: _tmpFB=((struct Cyc_Absyn_TypeofType_Absyn_Type_struct*)_tmpDF)->f1;_LLE: {struct Cyc_Absyn_Exp*e=_tmpFB;
# 792
({struct Cyc_Tcenv_Tenv*_tmp2D0=Cyc_Tcenv_allow_valueof(te);Cyc_Tcexp_tcExp(_tmp2D0,0,e);});
goto _LL0;}case 4U: _LLF: _tmpF6=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmpDF)->f1).elt_type;_tmpF7=(struct Cyc_Absyn_Tqual*)&(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmpDF)->f1).tq;_tmpF8=(struct Cyc_Absyn_Exp**)&(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmpDF)->f1).num_elts;_tmpF9=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmpDF)->f1).zero_term;_tmpFA=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmpDF)->f1).zt_loc;_LL10: {void*t1=_tmpF6;struct Cyc_Absyn_Tqual*tqp=_tmpF7;struct Cyc_Absyn_Exp**eptr=_tmpF8;void*zt=_tmpF9;unsigned ztl=_tmpFA;
# 797
struct Cyc_Absyn_Exp*e=*eptr;
cvtenv=Cyc_Tctyp_i_check_valid_type(loc,te,cvtenv,& Cyc_Tcutil_tmk,t1,1,allow_abs_aggr);
({int _tmp2D1=Cyc_Tcutil_extract_const_from_typedef(loc,tqp->print_const,t1);tqp->real_const=_tmp2D1;});{
# 801
int is_zero_terminated=Cyc_Tcutil_force_type2bool(0,zt);
if(is_zero_terminated){
# 804
if(!Cyc_Tcutil_admits_zero(t1))
({struct Cyc_String_pa_PrintArg_struct _tmp125=({struct Cyc_String_pa_PrintArg_struct _tmp227;_tmp227.tag=0U,({
struct _fat_ptr _tmp2D2=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t1));_tmp227.f1=_tmp2D2;});_tmp227;});void*_tmp123[1U];_tmp123[0]=& _tmp125;({unsigned _tmp2D4=loc;struct _fat_ptr _tmp2D3=({const char*_tmp124="cannot have a zero-terminated array of %s elements";_tag_fat(_tmp124,sizeof(char),51U);});Cyc_Tcutil_terr(_tmp2D4,_tmp2D3,_tag_fat(_tmp123,sizeof(void*),1U));});});}
# 802
if(e == 0){
# 812
if(is_zero_terminated)
# 814
({struct Cyc_Absyn_Exp*_tmp2D5=e=Cyc_Absyn_uint_exp(1U,0U);*eptr=_tmp2D5;});else{
# 817
({void*_tmp126=0U;({unsigned _tmp2D7=loc;struct _fat_ptr _tmp2D6=({const char*_tmp127="array bound defaults to 1 here";_tag_fat(_tmp127,sizeof(char),31U);});Cyc_Tcutil_warn(_tmp2D7,_tmp2D6,_tag_fat(_tmp126,sizeof(void*),0U));});});
({struct Cyc_Absyn_Exp*_tmp2D8=e=Cyc_Absyn_uint_exp(1U,0U);*eptr=_tmp2D8;});}}
# 802
({struct Cyc_Tcenv_Tenv*_tmp2D9=
# 821
Cyc_Tcenv_allow_valueof(te);
# 802
Cyc_Tcexp_tcExp(_tmp2D9,0,e);});
# 822
if(!Cyc_Tcutil_coerce_uint_type(e))
({void*_tmp128=0U;({unsigned _tmp2DB=loc;struct _fat_ptr _tmp2DA=({const char*_tmp129="array bounds expression is not an unsigned int";_tag_fat(_tmp129,sizeof(char),47U);});Cyc_Tcutil_terr(_tmp2DB,_tmp2DA,_tag_fat(_tmp128,sizeof(void*),0U));});});
# 822
cvtenv=
# 824
Cyc_Tctyp_i_check_valid_type_level_exp(e,te,cvtenv);{
# 829
struct _tuple12 _stmttmp17=Cyc_Evexp_eval_const_uint_exp(e);int _tmp12B;unsigned _tmp12A;_LL37: _tmp12A=_stmttmp17.f1;_tmp12B=_stmttmp17.f2;_LL38: {unsigned sz=_tmp12A;int known=_tmp12B;
# 831
if((is_zero_terminated && known)&& sz < (unsigned)1)
({void*_tmp12C=0U;({unsigned _tmp2DD=loc;struct _fat_ptr _tmp2DC=({const char*_tmp12D="zero terminated array cannot have zero elements";_tag_fat(_tmp12D,sizeof(char),48U);});Cyc_Tcutil_warn(_tmp2DD,_tmp2DC,_tag_fat(_tmp12C,sizeof(void*),0U));});});
# 831
if(
# 834
(known && sz < (unsigned)1)&& Cyc_Cyclone_tovc_r){
({void*_tmp12E=0U;({unsigned _tmp2DF=loc;struct _fat_ptr _tmp2DE=({const char*_tmp12F="arrays with 0 elements are not supported except with gcc -- changing to 1.";_tag_fat(_tmp12F,sizeof(char),75U);});Cyc_Tcutil_warn(_tmp2DF,_tmp2DE,_tag_fat(_tmp12E,sizeof(void*),0U));});});
({struct Cyc_Absyn_Exp*_tmp2E0=Cyc_Absyn_uint_exp(1U,0U);*eptr=_tmp2E0;});}
# 831
goto _LL0;}}}}case 5U: _LL11: _tmpE7=(struct Cyc_List_List**)&(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmpDF)->f1).tvars;_tmpE8=(void**)&(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmpDF)->f1).effect;_tmpE9=(struct Cyc_Absyn_Tqual*)&(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmpDF)->f1).ret_tqual;_tmpEA=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmpDF)->f1).ret_type;_tmpEB=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmpDF)->f1).args;_tmpEC=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmpDF)->f1).c_varargs;_tmpED=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmpDF)->f1).cyc_varargs;_tmpEE=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmpDF)->f1).rgn_po;_tmpEF=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmpDF)->f1).attributes;_tmpF0=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmpDF)->f1).requires_clause;_tmpF1=(struct Cyc_List_List**)&(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmpDF)->f1).requires_relns;_tmpF2=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmpDF)->f1).ensures_clause;_tmpF3=(struct Cyc_List_List**)&(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmpDF)->f1).ensures_relns;_tmpF4=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmpDF)->f1).ieffect;_tmpF5=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmpDF)->f1).oeffect;_LL12: {struct Cyc_List_List**btvs=_tmpE7;void**eff=_tmpE8;struct Cyc_Absyn_Tqual*rtq=_tmpE9;void*tr=_tmpEA;struct Cyc_List_List*args=_tmpEB;int c_vararg=_tmpEC;struct Cyc_Absyn_VarargInfo*cyc_vararg=_tmpED;struct Cyc_List_List*rgn_po=_tmpEE;struct Cyc_List_List*atts=_tmpEF;struct Cyc_Absyn_Exp*req=_tmpF0;struct Cyc_List_List**req_relns=_tmpF1;struct Cyc_Absyn_Exp*ens=_tmpF2;struct Cyc_List_List**ens_relns=_tmpF3;struct Cyc_List_List*ieff=_tmpF4;struct Cyc_List_List*oeff=_tmpF5;
# 845
int num_convs=0;
int seen_cdecl=0;
int seen_stdcall=0;
int seen_fastcall=0;
int seen_format=0;
enum Cyc_Absyn_Format_Type ft=0U;
int fmt_desc_arg=-1;
int fmt_arg_start=-1;
for(0;atts != 0;atts=atts->tl){
# 855
if(!Cyc_Absyn_fntype_att((void*)atts->hd))
({struct Cyc_String_pa_PrintArg_struct _tmp132=({struct Cyc_String_pa_PrintArg_struct _tmp228;_tmp228.tag=0U,({struct _fat_ptr _tmp2E1=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absyn_attribute2string((void*)atts->hd));_tmp228.f1=_tmp2E1;});_tmp228;});void*_tmp130[1U];_tmp130[0]=& _tmp132;({unsigned _tmp2E3=loc;struct _fat_ptr _tmp2E2=({const char*_tmp131="bad function type attribute %s";_tag_fat(_tmp131,sizeof(char),31U);});Cyc_Tcutil_terr(_tmp2E3,_tmp2E2,_tag_fat(_tmp130,sizeof(void*),1U));});});{
# 855
void*_stmttmp18=(void*)atts->hd;void*_tmp133=_stmttmp18;int _tmp136;int _tmp135;enum Cyc_Absyn_Format_Type _tmp134;switch(*((int*)_tmp133)){case 1U: _LL3A: _LL3B:
# 860
 if(!seen_stdcall){seen_stdcall=1;++ num_convs;}goto _LL39;case 2U: _LL3C: _LL3D:
# 862
 if(!seen_cdecl){seen_cdecl=1;++ num_convs;}goto _LL39;case 3U: _LL3E: _LL3F:
# 864
 if(!seen_fastcall){seen_fastcall=1;++ num_convs;}goto _LL39;case 19U: _LL40: _tmp134=((struct Cyc_Absyn_Format_att_Absyn_Attribute_struct*)_tmp133)->f1;_tmp135=((struct Cyc_Absyn_Format_att_Absyn_Attribute_struct*)_tmp133)->f2;_tmp136=((struct Cyc_Absyn_Format_att_Absyn_Attribute_struct*)_tmp133)->f3;_LL41: {enum Cyc_Absyn_Format_Type fmttype=_tmp134;int i=_tmp135;int j=_tmp136;
# 866
if(!seen_format){
# 868
seen_format=1;
ft=fmttype;
fmt_desc_arg=i;
fmt_arg_start=j;}else{
({void*_tmp137=0U;({unsigned _tmp2E5=loc;struct _fat_ptr _tmp2E4=({const char*_tmp138="function can't have multiple format attributes";_tag_fat(_tmp138,sizeof(char),47U);});Cyc_Tcutil_terr(_tmp2E5,_tmp2E4,_tag_fat(_tmp137,sizeof(void*),0U));});});}
goto _LL39;}default: _LL42: _LL43:
 goto _LL39;}_LL39:;}}
# 877
if(num_convs > 1)({void*_tmp139=0U;({unsigned _tmp2E7=loc;struct _fat_ptr _tmp2E6=({const char*_tmp13A="function can't have multiple calling conventions";_tag_fat(_tmp13A,sizeof(char),49U);});Cyc_Tcutil_terr(_tmp2E7,_tmp2E6,_tag_fat(_tmp139,sizeof(void*),0U));});});Cyc_Tcutil_check_unique_tvars(loc,*btvs);
# 882
{struct Cyc_List_List*b=*btvs;for(0;b != 0;b=b->tl){
# 884
({int _tmp2E8=Cyc_Tcutil_new_tvar_id();((struct Cyc_Absyn_Tvar*)b->hd)->identity=_tmp2E8;});
({struct Cyc_List_List*_tmp2E9=Cyc_Tctyp_add_bound_tvar(cvtenv.kind_env,(struct Cyc_Absyn_Tvar*)b->hd);cvtenv.kind_env=_tmp2E9;});{
void*_stmttmp19=Cyc_Absyn_compress_kb(((struct Cyc_Absyn_Tvar*)b->hd)->kind);void*_tmp13B=_stmttmp19;if(((struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct*)_tmp13B)->tag == 0U){if(((struct Cyc_Absyn_Kind*)((struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct*)_tmp13B)->f1)->kind == Cyc_Absyn_MemKind){_LL45: _LL46:
# 889
({struct Cyc_String_pa_PrintArg_struct _tmp13E=({struct Cyc_String_pa_PrintArg_struct _tmp229;_tmp229.tag=0U,_tmp229.f1=(struct _fat_ptr)((struct _fat_ptr)*((struct Cyc_Absyn_Tvar*)b->hd)->name);_tmp229;});void*_tmp13C[1U];_tmp13C[0]=& _tmp13E;({unsigned _tmp2EB=loc;struct _fat_ptr _tmp2EA=({const char*_tmp13D="function attempts to abstract Mem type variable %s";_tag_fat(_tmp13D,sizeof(char),51U);});Cyc_Tcutil_terr(_tmp2EB,_tmp2EA,_tag_fat(_tmp13C,sizeof(void*),1U));});});goto _LL44;}else{goto _LL47;}}else{_LL47: _LL48:
 goto _LL44;}_LL44:;}}}{
# 896
struct Cyc_Tctyp_CVTEnv new_cvtenv=({struct Cyc_Tctyp_CVTEnv _tmp22C;_tmp22C.r=Cyc_Core_heap_region,_tmp22C.kind_env=cvtenv.kind_env,_tmp22C.fn_result=1,_tmp22C.generalize_evars=cvtenv.generalize_evars,_tmp22C.free_vars=0,_tmp22C.free_evars=0;_tmp22C;});
# 899
struct Cyc_List_List*itf=Cyc_Absyn_rgneffect_rnames(ieff);
struct Cyc_List_List*otf=Cyc_Absyn_rgneffect_rnames(oeff);
for(0;itf != 0;itf=itf->tl){new_cvtenv=Cyc_Tctyp_i_check_valid_type(loc,te,new_cvtenv,& Cyc_Tcutil_trk,(void*)itf->hd,1,1);}
for(0;otf != 0;otf=otf->tl){new_cvtenv=Cyc_Tctyp_i_check_valid_type(loc,te,new_cvtenv,& Cyc_Tcutil_trk,(void*)otf->hd,1,1);}
# 905
new_cvtenv=Cyc_Tctyp_i_check_valid_type(loc,te,new_cvtenv,& Cyc_Tcutil_tmk,tr,1,1);
# 907
({int _tmp2EC=Cyc_Tcutil_extract_const_from_typedef(loc,rtq->print_const,tr);rtq->real_const=_tmp2EC;});
new_cvtenv.fn_result=0;
# 911
{struct Cyc_List_List*a=args;for(0;a != 0;a=a->tl){
struct _tuple9*trip=(struct _tuple9*)a->hd;
void*t=(*trip).f3;
new_cvtenv=Cyc_Tctyp_i_check_valid_type(loc,te,new_cvtenv,& Cyc_Tcutil_tmk,t,1,1);{
int ec=Cyc_Tcutil_extract_const_from_typedef(loc,((*trip).f2).print_const,t);
((*trip).f2).real_const=ec;
# 919
if(Cyc_Tcutil_is_array_type(t)){
# 921
void*ptr_rgn=Cyc_Absyn_new_evar(0,0);
new_cvtenv=Cyc_Tctyp_i_check_valid_type(loc,te,new_cvtenv,& Cyc_Tcutil_rk,ptr_rgn,1,1);
({void*_tmp2ED=Cyc_Tcutil_promote_array(t,ptr_rgn,0);(*trip).f3=_tmp2ED;});}}}}
# 928
if(cyc_vararg != 0){
if(c_vararg)({void*_tmp13F=0U;({int(*_tmp2EF)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp141)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp141;});struct _fat_ptr _tmp2EE=({const char*_tmp140="both c_vararg and cyc_vararg";_tag_fat(_tmp140,sizeof(char),29U);});_tmp2EF(_tmp2EE,_tag_fat(_tmp13F,sizeof(void*),0U));});});{struct Cyc_Absyn_VarargInfo _stmttmp1A=*cyc_vararg;int _tmp143;void*_tmp142;_LL4A: _tmp142=_stmttmp1A.type;_tmp143=_stmttmp1A.inject;_LL4B: {void*vt=_tmp142;int vi=_tmp143;
# 931
new_cvtenv=Cyc_Tctyp_i_check_valid_type(loc,te,new_cvtenv,& Cyc_Tcutil_tmk,vt,1,1);
({int _tmp2F0=Cyc_Tcutil_extract_const_from_typedef(loc,(cyc_vararg->tq).print_const,vt);(cyc_vararg->tq).real_const=_tmp2F0;});
# 934
if(vi){
void*_stmttmp1B=Cyc_Tcutil_compress(vt);void*_tmp144=_stmttmp1B;void*_tmp147;void*_tmp146;void*_tmp145;if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp144)->tag == 3U){_LL4D: _tmp145=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp144)->f1).elt_type;_tmp146=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp144)->f1).ptr_atts).bounds;_tmp147=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp144)->f1).ptr_atts).zero_term;_LL4E: {void*et=_tmp145;void*bs=_tmp146;void*zt=_tmp147;
# 937
{void*_stmttmp1C=Cyc_Tcutil_compress(et);void*_tmp148=_stmttmp1C;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp148)->tag == 0U){if(((struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp148)->f1)->tag == 18U){_LL52: _LL53:
# 939
 if(Cyc_Tcutil_force_type2bool(0,zt))
({void*_tmp149=0U;({unsigned _tmp2F2=loc;struct _fat_ptr _tmp2F1=({const char*_tmp14A="can't inject into a zeroterm pointer";_tag_fat(_tmp14A,sizeof(char),37U);});Cyc_Tcutil_terr(_tmp2F2,_tmp2F1,_tag_fat(_tmp149,sizeof(void*),0U));});});
# 939
if(!({
# 941
void*_tmp2F3=Cyc_Absyn_bounds_one();Cyc_Unify_unify(_tmp2F3,bs);}))
({void*_tmp14B=0U;({unsigned _tmp2F5=loc;struct _fat_ptr _tmp2F4=({const char*_tmp14C="can't inject into a fat pointer to datatype";_tag_fat(_tmp14C,sizeof(char),44U);});Cyc_Tcutil_terr(_tmp2F5,_tmp2F4,_tag_fat(_tmp14B,sizeof(void*),0U));});});
# 939
goto _LL51;}else{goto _LL54;}}else{_LL54: _LL55:
# 944
({void*_tmp14D=0U;({unsigned _tmp2F7=loc;struct _fat_ptr _tmp2F6=({const char*_tmp14E="can't inject a non-datatype type";_tag_fat(_tmp14E,sizeof(char),33U);});Cyc_Tcutil_terr(_tmp2F7,_tmp2F6,_tag_fat(_tmp14D,sizeof(void*),0U));});});goto _LL51;}_LL51:;}
# 946
goto _LL4C;}}else{_LL4F: _LL50:
({void*_tmp14F=0U;({unsigned _tmp2F9=loc;struct _fat_ptr _tmp2F8=({const char*_tmp150="expecting a datatype pointer type";_tag_fat(_tmp150,sizeof(char),34U);});Cyc_Tcutil_terr(_tmp2F9,_tmp2F8,_tag_fat(_tmp14F,sizeof(void*),0U));});});goto _LL4C;}_LL4C:;}}}}
# 928
if(seen_format){
# 953
int num_args=Cyc_List_length(args);
if((((fmt_desc_arg < 0 || fmt_desc_arg > num_args)|| fmt_arg_start < 0)||
# 956
(cyc_vararg == 0 && !c_vararg)&& fmt_arg_start != 0)||
(cyc_vararg != 0 || c_vararg)&& fmt_arg_start != num_args + 1)
# 959
({void*_tmp151=0U;({unsigned _tmp2FB=loc;struct _fat_ptr _tmp2FA=({const char*_tmp152="bad format descriptor";_tag_fat(_tmp152,sizeof(char),22U);});Cyc_Tcutil_terr(_tmp2FB,_tmp2FA,_tag_fat(_tmp151,sizeof(void*),0U));});});else{
# 962
struct _tuple9 _stmttmp1D=*({struct _tuple9*(*_tmp2FD)(struct Cyc_List_List*x,int n)=({struct _tuple9*(*_tmp165)(struct Cyc_List_List*x,int n)=(struct _tuple9*(*)(struct Cyc_List_List*x,int n))Cyc_List_nth;_tmp165;});struct Cyc_List_List*_tmp2FC=args;_tmp2FD(_tmp2FC,fmt_desc_arg - 1);});void*_tmp153;_LL57: _tmp153=_stmttmp1D.f3;_LL58: {void*t=_tmp153;
# 964
{void*_stmttmp1E=Cyc_Tcutil_compress(t);void*_tmp154=_stmttmp1E;void*_tmp157;void*_tmp156;void*_tmp155;if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp154)->tag == 3U){_LL5A: _tmp155=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp154)->f1).elt_type;_tmp156=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp154)->f1).ptr_atts).bounds;_tmp157=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp154)->f1).ptr_atts).zero_term;_LL5B: {void*et=_tmp155;void*b=_tmp156;void*zt=_tmp157;
# 967
if(!Cyc_Tcutil_is_char_type(et))
({void*_tmp158=0U;({unsigned _tmp2FF=loc;struct _fat_ptr _tmp2FE=({const char*_tmp159="format descriptor is not a string";_tag_fat(_tmp159,sizeof(char),34U);});Cyc_Tcutil_terr(_tmp2FF,_tmp2FE,_tag_fat(_tmp158,sizeof(void*),0U));});});else{
# 970
struct Cyc_Absyn_Exp*e=({void*_tmp300=Cyc_Absyn_bounds_one();Cyc_Tcutil_get_bounds_exp(_tmp300,b);});
if(e == 0 && c_vararg)
({void*_tmp15A=0U;({unsigned _tmp302=loc;struct _fat_ptr _tmp301=({const char*_tmp15B="format descriptor is not a char * type";_tag_fat(_tmp15B,sizeof(char),39U);});Cyc_Tcutil_terr(_tmp302,_tmp301,_tag_fat(_tmp15A,sizeof(void*),0U));});});else{
if(e != 0 && !c_vararg)
({void*_tmp15C=0U;({unsigned _tmp304=loc;struct _fat_ptr _tmp303=({const char*_tmp15D="format descriptor is not a char ? type";_tag_fat(_tmp15D,sizeof(char),39U);});Cyc_Tcutil_terr(_tmp304,_tmp303,_tag_fat(_tmp15C,sizeof(void*),0U));});});}}
# 976
goto _LL59;}}else{_LL5C: _LL5D:
({void*_tmp15E=0U;({unsigned _tmp306=loc;struct _fat_ptr _tmp305=({const char*_tmp15F="format descriptor is not a string type";_tag_fat(_tmp15F,sizeof(char),39U);});Cyc_Tcutil_terr(_tmp306,_tmp305,_tag_fat(_tmp15E,sizeof(void*),0U));});});goto _LL59;}_LL59:;}
# 979
if(fmt_arg_start != 0 && !c_vararg){
# 983
int problem;
{struct _tuple16 _stmttmp1F=({struct _tuple16 _tmp22A;_tmp22A.f1=ft,({void*_tmp307=Cyc_Tcutil_compress(Cyc_Tcutil_pointer_elt_type(((struct Cyc_Absyn_VarargInfo*)_check_null(cyc_vararg))->type));_tmp22A.f2=_tmp307;});_tmp22A;});struct _tuple16 _tmp160=_stmttmp1F;struct Cyc_Absyn_Datatypedecl*_tmp161;struct Cyc_Absyn_Datatypedecl*_tmp162;switch(_tmp160.f1){case Cyc_Absyn_Printf_ft: if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp160.f2)->tag == 0U){if(((struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp160.f2)->f1)->tag == 18U){if(((((struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp160.f2)->f1)->f1).KnownDatatype).tag == 2){_LL5F: _tmp162=*((((struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp160.f2)->f1)->f1).KnownDatatype).val;_LL60: {struct Cyc_Absyn_Datatypedecl*tud=_tmp162;
# 986
problem=({struct _tuple1*_tmp308=tud->name;Cyc_Absyn_qvar_cmp(_tmp308,Cyc_Absyn_datatype_print_arg_qvar());})!= 0;
goto _LL5E;}}else{goto _LL63;}}else{goto _LL63;}}else{goto _LL63;}case Cyc_Absyn_Scanf_ft: if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp160.f2)->tag == 0U){if(((struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp160.f2)->f1)->tag == 18U){if(((((struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp160.f2)->f1)->f1).KnownDatatype).tag == 2){_LL61: _tmp161=*((((struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp160.f2)->f1)->f1).KnownDatatype).val;_LL62: {struct Cyc_Absyn_Datatypedecl*tud=_tmp161;
# 989
problem=({struct _tuple1*_tmp309=tud->name;Cyc_Absyn_qvar_cmp(_tmp309,Cyc_Absyn_datatype_scanf_arg_qvar());})!= 0;
goto _LL5E;}}else{goto _LL63;}}else{goto _LL63;}}else{goto _LL63;}default: _LL63: _LL64:
# 992
 problem=1;
goto _LL5E;}_LL5E:;}
# 995
if(problem)
({void*_tmp163=0U;({unsigned _tmp30B=loc;struct _fat_ptr _tmp30A=({const char*_tmp164="format attribute and vararg types don't match";_tag_fat(_tmp164,sizeof(char),46U);});Cyc_Tcutil_terr(_tmp30B,_tmp30A,_tag_fat(_tmp163,sizeof(void*),0U));});});}}}}
# 928
{
# 1003
struct Cyc_List_List*rpo=rgn_po;
# 928
for(0;rpo != 0;rpo=rpo->tl){
# 1005
struct _tuple0*_stmttmp20=(struct _tuple0*)rpo->hd;void*_tmp167;void*_tmp166;_LL66: _tmp166=_stmttmp20->f1;_tmp167=_stmttmp20->f2;_LL67: {void*r1=_tmp166;void*r2=_tmp167;
new_cvtenv=Cyc_Tctyp_i_check_valid_type(loc,te,new_cvtenv,& Cyc_Tcutil_ek,r1,1,1);
new_cvtenv=Cyc_Tctyp_i_check_valid_type(loc,te,new_cvtenv,& Cyc_Tcutil_trk,r2,1,1);}}}{
# 1012
struct Cyc_Tcenv_Fenv*fenv=Cyc_Tcenv_bogus_fenv(tr,args);
struct Cyc_Tcenv_Tenv*ftenv=({struct Cyc_Tcenv_Tenv*_tmp198=_cycalloc(sizeof(*_tmp198));_tmp198->ns=te->ns,_tmp198->ae=te->ae,_tmp198->le=fenv,_tmp198->allow_valueof=1,_tmp198->in_extern_c_include=te->in_extern_c_include,_tmp198->in_tempest=te->in_tempest,_tmp198->tempest_generalize=te->tempest_generalize;_tmp198;});
struct _tuple15 _stmttmp21=({unsigned _tmp30F=loc;struct Cyc_Tcenv_Tenv*_tmp30E=ftenv;struct Cyc_Tctyp_CVTEnv _tmp30D=new_cvtenv;struct _fat_ptr _tmp30C=({const char*_tmp197="@requires";_tag_fat(_tmp197,sizeof(char),10U);});Cyc_Tctyp_check_clause(_tmp30F,_tmp30E,_tmp30D,_tmp30C,req);});struct Cyc_List_List*_tmp169;struct Cyc_Tctyp_CVTEnv _tmp168;_LL69: _tmp168=_stmttmp21.f1;_tmp169=_stmttmp21.f2;_LL6A: {struct Cyc_Tctyp_CVTEnv nenv=_tmp168;struct Cyc_List_List*req_rs=_tmp169;
new_cvtenv=nenv;
*req_relns=req_rs;
({void(*_tmp311)(void(*f)(struct Cyc_List_List*,struct Cyc_Relations_Reln*),struct Cyc_List_List*env,struct Cyc_List_List*x)=({void(*_tmp16A)(void(*f)(struct Cyc_List_List*,struct Cyc_Relations_Reln*),struct Cyc_List_List*env,struct Cyc_List_List*x)=(void(*)(void(*f)(struct Cyc_List_List*,struct Cyc_Relations_Reln*),struct Cyc_List_List*env,struct Cyc_List_List*x))Cyc_List_iter_c;_tmp16A;});struct Cyc_List_List*_tmp310=args;_tmp311(Cyc_Tcutil_replace_rops,_tmp310,req_rs);});{
struct _tuple15 _stmttmp22=({unsigned _tmp315=loc;struct Cyc_Tcenv_Tenv*_tmp314=ftenv;struct Cyc_Tctyp_CVTEnv _tmp313=new_cvtenv;struct _fat_ptr _tmp312=({const char*_tmp196="@ensures";_tag_fat(_tmp196,sizeof(char),9U);});Cyc_Tctyp_check_clause(_tmp315,_tmp314,_tmp313,_tmp312,ens);});struct Cyc_List_List*_tmp16C;struct Cyc_Tctyp_CVTEnv _tmp16B;_LL6C: _tmp16B=_stmttmp22.f1;_tmp16C=_stmttmp22.f2;_LL6D: {struct Cyc_Tctyp_CVTEnv nenv=_tmp16B;struct Cyc_List_List*ens_rs=_tmp16C;
new_cvtenv=nenv;
*ens_relns=ens_rs;
({void(*_tmp317)(void(*f)(struct Cyc_List_List*,struct Cyc_Relations_Reln*),struct Cyc_List_List*env,struct Cyc_List_List*x)=({void(*_tmp16D)(void(*f)(struct Cyc_List_List*,struct Cyc_Relations_Reln*),struct Cyc_List_List*env,struct Cyc_List_List*x)=(void(*)(void(*f)(struct Cyc_List_List*,struct Cyc_Relations_Reln*),struct Cyc_List_List*env,struct Cyc_List_List*x))Cyc_List_iter_c;_tmp16D;});struct Cyc_List_List*_tmp316=args;_tmp317(Cyc_Tcutil_replace_rops,_tmp316,ens_rs);});
# 1024
if(*eff != 0)
new_cvtenv=Cyc_Tctyp_i_check_valid_type(loc,te,new_cvtenv,& Cyc_Tcutil_ek,(void*)_check_null(*eff),1,1);else{
# 1028
struct Cyc_List_List*effect=0;
# 1033
{struct Cyc_List_List*tvs=new_cvtenv.free_vars;for(0;tvs != 0;tvs=tvs->tl){
# 1035
struct _tuple14 _stmttmp23=*((struct _tuple14*)tvs->hd);int _tmp16F;struct Cyc_Absyn_Tvar*_tmp16E;_LL6F: _tmp16E=_stmttmp23.f1;_tmp16F=_stmttmp23.f2;_LL70: {struct Cyc_Absyn_Tvar*tv=_tmp16E;int put_in_eff=_tmp16F;
if(!put_in_eff)continue;{void*_stmttmp24=
Cyc_Absyn_compress_kb(tv->kind);
# 1036
void*_tmp170=_stmttmp24;struct Cyc_Core_Opt**_tmp171;struct Cyc_Absyn_Kind*_tmp172;struct Cyc_Core_Opt**_tmp173;struct Cyc_Absyn_Kind*_tmp175;struct Cyc_Core_Opt**_tmp174;switch(*((int*)_tmp170)){case 2U: _LL72: _tmp174=(struct Cyc_Core_Opt**)&((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp170)->f1;_tmp175=((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp170)->f2;if((int)_tmp175->kind == (int)Cyc_Absyn_RgnKind){_LL73: {struct Cyc_Core_Opt**f=_tmp174;struct Cyc_Absyn_Kind*r=_tmp175;
# 1041
if((int)r->aliasqual == (int)Cyc_Absyn_Top){
# 1043
({struct Cyc_Core_Opt*_tmp318=Cyc_Tcutil_kind_to_bound_opt(& Cyc_Tcutil_rk);*f=_tmp318;});
_tmp172=r;goto _LL75;}
# 1041
({struct Cyc_Core_Opt*_tmp319=
# 1046
Cyc_Tcutil_kind_to_bound_opt(r);
# 1041
*f=_tmp319;});
# 1047
_tmp172=r;goto _LL75;}}else{switch(((struct Cyc_Absyn_Kind*)((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp170)->f2)->kind){case Cyc_Absyn_BoolKind: _LL76: _LL77:
# 1050
 goto _LL79;case Cyc_Absyn_PtrBndKind: _LL78: _LL79:
 goto _LL7B;case Cyc_Absyn_IntKind: _LL7A: _LL7B:
 goto _LL7D;case Cyc_Absyn_EffKind: _LL82: _tmp173=(struct Cyc_Core_Opt**)&((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp170)->f1;_LL83: {struct Cyc_Core_Opt**f=_tmp173;
# 1057
({struct Cyc_Core_Opt*_tmp31A=Cyc_Tcutil_kind_to_bound_opt(& Cyc_Tcutil_ek);*f=_tmp31A;});goto _LL85;}default: goto _LL88;}}case 0U: _LL74: _tmp172=((struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct*)_tmp170)->f1;if((int)_tmp172->kind == (int)Cyc_Absyn_RgnKind){_LL75: {struct Cyc_Absyn_Kind*r=_tmp172;
# 1049
effect=({struct Cyc_List_List*_tmp176=_cycalloc(sizeof(*_tmp176));({void*_tmp31B=Cyc_Absyn_access_eff(Cyc_Absyn_var_type(tv));_tmp176->hd=_tmp31B;}),_tmp176->tl=effect;_tmp176;});goto _LL71;}}else{switch(((struct Cyc_Absyn_Kind*)((struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct*)_tmp170)->f1)->kind){case Cyc_Absyn_BoolKind: _LL7C: _LL7D:
# 1053
 goto _LL7F;case Cyc_Absyn_PtrBndKind: _LL7E: _LL7F:
 goto _LL81;case Cyc_Absyn_IntKind: _LL80: _LL81:
 goto _LL71;case Cyc_Absyn_EffKind: _LL84: _LL85:
# 1059
 effect=({struct Cyc_List_List*_tmp177=_cycalloc(sizeof(*_tmp177));({void*_tmp31C=Cyc_Absyn_var_type(tv);_tmp177->hd=_tmp31C;}),_tmp177->tl=effect;_tmp177;});goto _LL71;default: _LL88: _LL89:
# 1064
 effect=({struct Cyc_List_List*_tmp17A=_cycalloc(sizeof(*_tmp17A));({void*_tmp31D=Cyc_Absyn_regionsof_eff(Cyc_Absyn_var_type(tv));_tmp17A->hd=_tmp31D;}),_tmp17A->tl=effect;_tmp17A;});goto _LL71;}}default: _LL86: _tmp171=(struct Cyc_Core_Opt**)&((struct Cyc_Absyn_Unknown_kb_Absyn_KindBound_struct*)_tmp170)->f1;_LL87: {struct Cyc_Core_Opt**f=_tmp171;
# 1061
({struct Cyc_Core_Opt*_tmp31F=({struct Cyc_Core_Opt*_tmp179=_cycalloc(sizeof(*_tmp179));({void*_tmp31E=(void*)({struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*_tmp178=_cycalloc(sizeof(*_tmp178));_tmp178->tag=2U,_tmp178->f1=0,_tmp178->f2=& Cyc_Tcutil_ak;_tmp178;});_tmp179->v=_tmp31E;});_tmp179;});*f=_tmp31F;});goto _LL89;}}_LL71:;}}}}
# 1068
{struct Cyc_List_List*ts=new_cvtenv.free_evars;for(0;ts != 0;ts=ts->tl){
# 1070
struct _tuple13 _stmttmp25=*((struct _tuple13*)ts->hd);int _tmp17C;void*_tmp17B;_LL8B: _tmp17B=_stmttmp25.f1;_tmp17C=_stmttmp25.f2;_LL8C: {void*tv=_tmp17B;int put_in_eff=_tmp17C;
if(!put_in_eff)continue;{struct Cyc_Absyn_Kind*_stmttmp26=
Cyc_Tcutil_type_kind(tv);
# 1071
struct Cyc_Absyn_Kind*_tmp17D=_stmttmp26;switch(((struct Cyc_Absyn_Kind*)_tmp17D)->kind){case Cyc_Absyn_RgnKind: _LL8E: _LL8F:
# 1074
 effect=({struct Cyc_List_List*_tmp17E=_cycalloc(sizeof(*_tmp17E));({void*_tmp320=Cyc_Absyn_access_eff(tv);_tmp17E->hd=_tmp320;}),_tmp17E->tl=effect;_tmp17E;});goto _LL8D;case Cyc_Absyn_EffKind: _LL90: _LL91:
 effect=({struct Cyc_List_List*_tmp17F=_cycalloc(sizeof(*_tmp17F));_tmp17F->hd=tv,_tmp17F->tl=effect;_tmp17F;});goto _LL8D;case Cyc_Absyn_IntKind: _LL92: _LL93:
 goto _LL8D;default: _LL94: _LL95:
 effect=({struct Cyc_List_List*_tmp180=_cycalloc(sizeof(*_tmp180));({void*_tmp321=Cyc_Absyn_regionsof_eff(tv);_tmp180->hd=_tmp321;}),_tmp180->tl=effect;_tmp180;});goto _LL8D;}_LL8D:;}}}}
# 1080
({void*_tmp322=Cyc_Absyn_join_eff(effect);*eff=_tmp322;});}
# 1086
if(*btvs != 0){
struct Cyc_List_List*bs=*btvs;for(0;bs != 0;bs=bs->tl){
# 1089
void*_stmttmp27=Cyc_Absyn_compress_kb(((struct Cyc_Absyn_Tvar*)bs->hd)->kind);void*_tmp181=_stmttmp27;struct Cyc_Absyn_Kind*_tmp183;struct Cyc_Core_Opt**_tmp182;struct Cyc_Core_Opt**_tmp184;struct Cyc_Core_Opt**_tmp185;struct Cyc_Core_Opt**_tmp186;struct Cyc_Core_Opt**_tmp187;struct Cyc_Core_Opt**_tmp188;struct Cyc_Core_Opt**_tmp189;struct Cyc_Core_Opt**_tmp18A;struct Cyc_Core_Opt**_tmp18B;struct Cyc_Core_Opt**_tmp18C;switch(*((int*)_tmp181)){case 1U: _LL97: _tmp18C=(struct Cyc_Core_Opt**)&((struct Cyc_Absyn_Unknown_kb_Absyn_KindBound_struct*)_tmp181)->f1;_LL98: {struct Cyc_Core_Opt**f=_tmp18C;
# 1092
({struct Cyc_String_pa_PrintArg_struct _tmp18F=({struct Cyc_String_pa_PrintArg_struct _tmp22B;_tmp22B.tag=0U,_tmp22B.f1=(struct _fat_ptr)((struct _fat_ptr)*((struct Cyc_Absyn_Tvar*)bs->hd)->name);_tmp22B;});void*_tmp18D[1U];_tmp18D[0]=& _tmp18F;({unsigned _tmp324=loc;struct _fat_ptr _tmp323=({const char*_tmp18E="Type variable %s unconstrained, assuming boxed";_tag_fat(_tmp18E,sizeof(char),47U);});Cyc_Tcutil_warn(_tmp324,_tmp323,_tag_fat(_tmp18D,sizeof(void*),1U));});});
# 1094
_tmp18B=f;goto _LL9A;}case 2U: switch(((struct Cyc_Absyn_Kind*)((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp181)->f2)->kind){case Cyc_Absyn_BoxKind: if(((struct Cyc_Absyn_Kind*)((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp181)->f2)->aliasqual == Cyc_Absyn_Top){_LL99: _tmp18B=(struct Cyc_Core_Opt**)&((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp181)->f1;_LL9A: {struct Cyc_Core_Opt**f=_tmp18B;
_tmp18A=f;goto _LL9C;}}else{goto _LLA9;}case Cyc_Absyn_MemKind: switch(((struct Cyc_Absyn_Kind*)((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp181)->f2)->aliasqual){case Cyc_Absyn_Top: _LL9B: _tmp18A=(struct Cyc_Core_Opt**)&((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp181)->f1;_LL9C: {struct Cyc_Core_Opt**f=_tmp18A;
_tmp189=f;goto _LL9E;}case Cyc_Absyn_Aliasable: _LL9D: _tmp189=(struct Cyc_Core_Opt**)&((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp181)->f1;_LL9E: {struct Cyc_Core_Opt**f=_tmp189;
_tmp187=f;goto _LLA0;}case Cyc_Absyn_Unique: _LLA3: _tmp188=(struct Cyc_Core_Opt**)&((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp181)->f1;_LLA4: {struct Cyc_Core_Opt**f=_tmp188;
# 1100
_tmp185=f;goto _LLA6;}default: goto _LLA9;}case Cyc_Absyn_AnyKind: switch(((struct Cyc_Absyn_Kind*)((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp181)->f2)->aliasqual){case Cyc_Absyn_Top: _LL9F: _tmp187=(struct Cyc_Core_Opt**)&((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp181)->f1;_LLA0: {struct Cyc_Core_Opt**f=_tmp187;
# 1098
_tmp186=f;goto _LLA2;}case Cyc_Absyn_Aliasable: _LLA1: _tmp186=(struct Cyc_Core_Opt**)&((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp181)->f1;_LLA2: {struct Cyc_Core_Opt**f=_tmp186;
({struct Cyc_Core_Opt*_tmp325=Cyc_Tcutil_kind_to_bound_opt(& Cyc_Tcutil_bk);*f=_tmp325;});goto _LL96;}case Cyc_Absyn_Unique: _LLA5: _tmp185=(struct Cyc_Core_Opt**)&((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp181)->f1;_LLA6: {struct Cyc_Core_Opt**f=_tmp185;
# 1101
({struct Cyc_Core_Opt*_tmp326=Cyc_Tcutil_kind_to_bound_opt(& Cyc_Tcutil_ubk);*f=_tmp326;});goto _LL96;}default: goto _LLA9;}case Cyc_Absyn_RgnKind: if(((struct Cyc_Absyn_Kind*)((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp181)->f2)->aliasqual == Cyc_Absyn_Top){_LLA7: _tmp184=(struct Cyc_Core_Opt**)&((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp181)->f1;_LLA8: {struct Cyc_Core_Opt**f=_tmp184;
({struct Cyc_Core_Opt*_tmp327=Cyc_Tcutil_kind_to_bound_opt(& Cyc_Tcutil_rk);*f=_tmp327;});goto _LL96;}}else{goto _LLA9;}default: _LLA9: _tmp182=(struct Cyc_Core_Opt**)&((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp181)->f1;_tmp183=((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp181)->f2;_LLAA: {struct Cyc_Core_Opt**f=_tmp182;struct Cyc_Absyn_Kind*k=_tmp183;
({struct Cyc_Core_Opt*_tmp328=Cyc_Tcutil_kind_to_bound_opt(k);*f=_tmp328;});goto _LL96;}}default: if(((struct Cyc_Absyn_Kind*)((struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct*)_tmp181)->f1)->kind == Cyc_Absyn_MemKind){_LLAB: _LLAC:
({void*_tmp190=0U;({unsigned _tmp32A=loc;struct _fat_ptr _tmp329=({const char*_tmp191="functions can't abstract M types";_tag_fat(_tmp191,sizeof(char),33U);});Cyc_Tcutil_terr(_tmp32A,_tmp329,_tag_fat(_tmp190,sizeof(void*),0U));});});goto _LL96;}else{_LLAD: _LLAE:
 goto _LL96;}}_LL96:;}}
# 1086
({struct Cyc_List_List*_tmp32B=
# 1109
Cyc_Tctyp_remove_bound_tvars(Cyc_Core_heap_region,new_cvtenv.kind_env,*btvs);
# 1086
cvtenv.kind_env=_tmp32B;});
# 1110
({struct Cyc_List_List*_tmp32C=Cyc_Tctyp_remove_bound_tvars_bool(new_cvtenv.r,new_cvtenv.free_vars,*btvs);new_cvtenv.free_vars=_tmp32C;});
# 1112
{struct Cyc_List_List*tvs=new_cvtenv.free_vars;for(0;tvs != 0;tvs=tvs->tl){
# 1114
struct _tuple14 _stmttmp28=*((struct _tuple14*)tvs->hd);int _tmp193;struct Cyc_Absyn_Tvar*_tmp192;_LLB0: _tmp192=_stmttmp28.f1;_tmp193=_stmttmp28.f2;_LLB1: {struct Cyc_Absyn_Tvar*t=_tmp192;int b=_tmp193;
({struct Cyc_List_List*_tmp32D=Cyc_Tctyp_fast_add_free_tvar_bool(cvtenv.r,cvtenv.free_vars,t,b);cvtenv.free_vars=_tmp32D;});}}}
# 1118
{struct Cyc_List_List*evs=new_cvtenv.free_evars;for(0;evs != 0;evs=evs->tl){
# 1120
struct _tuple13 _stmttmp29=*((struct _tuple13*)evs->hd);int _tmp195;void*_tmp194;_LLB3: _tmp194=_stmttmp29.f1;_tmp195=_stmttmp29.f2;_LLB4: {void*e=_tmp194;int b=_tmp195;
({struct Cyc_List_List*_tmp32E=Cyc_Tctyp_add_free_evar(cvtenv.r,cvtenv.free_evars,e,b);cvtenv.free_evars=_tmp32E;});}}}
# 1123
goto _LL0;}}}}}}case 6U: _LL13: _tmpE6=((struct Cyc_Absyn_TupleType_Absyn_Type_struct*)_tmpDF)->f1;_LL14: {struct Cyc_List_List*tq_ts=_tmpE6;
# 1126
for(0;tq_ts != 0;tq_ts=tq_ts->tl){
struct _tuple17*p=(struct _tuple17*)tq_ts->hd;
cvtenv=Cyc_Tctyp_i_check_valid_type(loc,te,cvtenv,& Cyc_Tcutil_tmk,(*p).f2,1,0);
({int _tmp32F=
Cyc_Tcutil_extract_const_from_typedef(loc,((*p).f1).print_const,(*p).f2);
# 1129
((*p).f1).real_const=_tmp32F;});}
# 1132
goto _LL0;}case 7U: _LL15: _tmpE4=((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmpDF)->f1;_tmpE5=((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmpDF)->f2;_LL16: {enum Cyc_Absyn_AggrKind k=_tmpE4;struct Cyc_List_List*fs=_tmpE5;
# 1136
struct Cyc_List_List*prev_fields=0;
for(0;fs != 0;fs=fs->tl){
struct Cyc_Absyn_Aggrfield*_stmttmp2A=(struct Cyc_Absyn_Aggrfield*)fs->hd;struct Cyc_Absyn_Exp*_tmp19E;struct Cyc_List_List*_tmp19D;struct Cyc_Absyn_Exp*_tmp19C;void*_tmp19B;struct Cyc_Absyn_Tqual*_tmp19A;struct _fat_ptr*_tmp199;_LLB6: _tmp199=_stmttmp2A->name;_tmp19A=(struct Cyc_Absyn_Tqual*)& _stmttmp2A->tq;_tmp19B=_stmttmp2A->type;_tmp19C=_stmttmp2A->width;_tmp19D=_stmttmp2A->attributes;_tmp19E=_stmttmp2A->requires_clause;_LLB7: {struct _fat_ptr*fn=_tmp199;struct Cyc_Absyn_Tqual*tqp=_tmp19A;void*t=_tmp19B;struct Cyc_Absyn_Exp*width=_tmp19C;struct Cyc_List_List*atts=_tmp19D;struct Cyc_Absyn_Exp*requires_clause=_tmp19E;
if(({int(*_tmp331)(int(*compare)(struct _fat_ptr*,struct _fat_ptr*),struct Cyc_List_List*l,struct _fat_ptr*x)=({int(*_tmp19F)(int(*compare)(struct _fat_ptr*,struct _fat_ptr*),struct Cyc_List_List*l,struct _fat_ptr*x)=(int(*)(int(*compare)(struct _fat_ptr*,struct _fat_ptr*),struct Cyc_List_List*l,struct _fat_ptr*x))Cyc_List_mem;_tmp19F;});struct Cyc_List_List*_tmp330=prev_fields;_tmp331(Cyc_strptrcmp,_tmp330,fn);}))
({struct Cyc_String_pa_PrintArg_struct _tmp1A2=({struct Cyc_String_pa_PrintArg_struct _tmp22D;_tmp22D.tag=0U,_tmp22D.f1=(struct _fat_ptr)((struct _fat_ptr)*fn);_tmp22D;});void*_tmp1A0[1U];_tmp1A0[0]=& _tmp1A2;({unsigned _tmp333=loc;struct _fat_ptr _tmp332=({const char*_tmp1A1="duplicate field %s";_tag_fat(_tmp1A1,sizeof(char),19U);});Cyc_Tcutil_terr(_tmp333,_tmp332,_tag_fat(_tmp1A0,sizeof(void*),1U));});});
# 1139
if(({
# 1141
struct _fat_ptr _tmp334=(struct _fat_ptr)*fn;Cyc_strcmp(_tmp334,({const char*_tmp1A3="";_tag_fat(_tmp1A3,sizeof(char),1U);}));})!= 0)
prev_fields=({struct Cyc_List_List*_tmp1A4=_cycalloc(sizeof(*_tmp1A4));_tmp1A4->hd=fn,_tmp1A4->tl=prev_fields;_tmp1A4;});
# 1139
cvtenv=
# 1143
Cyc_Tctyp_i_check_valid_type(loc,te,cvtenv,& Cyc_Tcutil_tmk,t,1,0);
({int _tmp335=Cyc_Tcutil_extract_const_from_typedef(loc,tqp->print_const,t);tqp->real_const=_tmp335;});
Cyc_Tcutil_check_bitfield(loc,t,width,fn);
Cyc_Tctyp_check_field_atts(loc,fn,atts);
if(requires_clause != 0){
# 1149
if((int)k != (int)1U)
({void*_tmp1A5=0U;({unsigned _tmp337=loc;struct _fat_ptr _tmp336=({const char*_tmp1A6="@requires clause is only allowed on union members";_tag_fat(_tmp1A6,sizeof(char),50U);});Cyc_Tcutil_terr(_tmp337,_tmp336,_tag_fat(_tmp1A5,sizeof(void*),0U));});});
# 1149
({struct Cyc_Tcenv_Tenv*_tmp338=
# 1151
Cyc_Tcenv_allow_valueof(te);
# 1149
Cyc_Tcexp_tcExp(_tmp338,0,requires_clause);});
# 1152
if(!Cyc_Tcutil_is_integral(requires_clause))
({struct Cyc_String_pa_PrintArg_struct _tmp1A9=({struct Cyc_String_pa_PrintArg_struct _tmp22E;_tmp22E.tag=0U,({
struct _fat_ptr _tmp339=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string((void*)_check_null(requires_clause->topt)));_tmp22E.f1=_tmp339;});_tmp22E;});void*_tmp1A7[1U];_tmp1A7[0]=& _tmp1A9;({unsigned _tmp33B=loc;struct _fat_ptr _tmp33A=({const char*_tmp1A8="@requires clause has type %s instead of integral type";_tag_fat(_tmp1A8,sizeof(char),54U);});Cyc_Tcutil_terr(_tmp33B,_tmp33A,_tag_fat(_tmp1A7,sizeof(void*),1U));});});
# 1152
cvtenv=
# 1155
Cyc_Tctyp_i_check_valid_type_level_exp(requires_clause,te,cvtenv);}}}
# 1158
goto _LL0;}default: _LL17: _tmpE0=((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmpDF)->f1;_tmpE1=(struct Cyc_List_List**)&((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmpDF)->f2;_tmpE2=(struct Cyc_Absyn_Typedefdecl**)&((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmpDF)->f3;_tmpE3=(void**)&((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmpDF)->f4;_LL18: {struct _tuple1*tdn=_tmpE0;struct Cyc_List_List**targs_ref=_tmpE1;struct Cyc_Absyn_Typedefdecl**tdopt=_tmpE2;void**toptp=_tmpE3;
# 1161
struct Cyc_List_List*targs=*targs_ref;
# 1163
struct Cyc_Absyn_Typedefdecl*td;
{struct _handler_cons _tmp1AA;_push_handler(& _tmp1AA);{int _tmp1AC=0;if(setjmp(_tmp1AA.handler))_tmp1AC=1;if(!_tmp1AC){td=Cyc_Tcenv_lookup_typedefdecl(te,loc,tdn);;_pop_handler();}else{void*_tmp1AB=(void*)Cyc_Core_get_exn_thrown();void*_tmp1AD=_tmp1AB;void*_tmp1AE;if(((struct Cyc_Dict_Absent_exn_struct*)_tmp1AD)->tag == Cyc_Dict_Absent){_LLB9: _LLBA:
# 1166
({struct Cyc_String_pa_PrintArg_struct _tmp1B1=({struct Cyc_String_pa_PrintArg_struct _tmp22F;_tmp22F.tag=0U,({struct _fat_ptr _tmp33C=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(tdn));_tmp22F.f1=_tmp33C;});_tmp22F;});void*_tmp1AF[1U];_tmp1AF[0]=& _tmp1B1;({unsigned _tmp33E=loc;struct _fat_ptr _tmp33D=({const char*_tmp1B0="unbound typedef name %s";_tag_fat(_tmp1B0,sizeof(char),24U);});Cyc_Tcutil_terr(_tmp33E,_tmp33D,_tag_fat(_tmp1AF,sizeof(void*),1U));});});
return cvtenv;}else{_LLBB: _tmp1AE=_tmp1AD;_LLBC: {void*exn=_tmp1AE;(int)_rethrow(exn);}}_LLB8:;}}}
# 1169
*tdopt=td;{
struct Cyc_List_List*tvs=td->tvs;
struct Cyc_List_List*ts=targs;
struct Cyc_List_List*inst=0;
# 1174
for(0;ts != 0 && tvs != 0;(ts=ts->tl,tvs=tvs->tl)){
struct Cyc_Absyn_Kind*k=Cyc_Tctyp_tvar_inst_kind((struct Cyc_Absyn_Tvar*)tvs->hd,& Cyc_Tcutil_tbk,expected_kind,td);
# 1179
cvtenv=Cyc_Tctyp_i_check_valid_type(loc,te,cvtenv,k,(void*)ts->hd,1,allow_abs_aggr);
Cyc_Tcutil_check_no_qual(loc,(void*)ts->hd);
inst=({struct Cyc_List_List*_tmp1B3=_cycalloc(sizeof(*_tmp1B3));({struct _tuple18*_tmp33F=({struct _tuple18*_tmp1B2=_cycalloc(sizeof(*_tmp1B2));_tmp1B2->f1=(struct Cyc_Absyn_Tvar*)tvs->hd,_tmp1B2->f2=(void*)ts->hd;_tmp1B2;});_tmp1B3->hd=_tmp33F;}),_tmp1B3->tl=inst;_tmp1B3;});}
# 1183
if(ts != 0)
({struct Cyc_String_pa_PrintArg_struct _tmp1B6=({struct Cyc_String_pa_PrintArg_struct _tmp230;_tmp230.tag=0U,({struct _fat_ptr _tmp340=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(tdn));_tmp230.f1=_tmp340;});_tmp230;});void*_tmp1B4[1U];_tmp1B4[0]=& _tmp1B6;({unsigned _tmp342=loc;struct _fat_ptr _tmp341=({const char*_tmp1B5="too many parameters for typedef %s";_tag_fat(_tmp1B5,sizeof(char),35U);});Cyc_Tcutil_terr(_tmp342,_tmp341,_tag_fat(_tmp1B4,sizeof(void*),1U));});});
# 1183
if(tvs != 0){
# 1186
struct Cyc_List_List*hidden_ts=0;
# 1188
for(0;tvs != 0;tvs=tvs->tl){
struct Cyc_Absyn_Kind*k=Cyc_Tctyp_tvar_inst_kind((struct Cyc_Absyn_Tvar*)tvs->hd,& Cyc_Tcutil_bk,expected_kind,td);
void*e=Cyc_Absyn_new_evar(0,0);
hidden_ts=({struct Cyc_List_List*_tmp1B7=_cycalloc(sizeof(*_tmp1B7));_tmp1B7->hd=e,_tmp1B7->tl=hidden_ts;_tmp1B7;});
cvtenv=Cyc_Tctyp_i_check_valid_type(loc,te,cvtenv,k,e,1,allow_abs_aggr);
inst=({struct Cyc_List_List*_tmp1B9=_cycalloc(sizeof(*_tmp1B9));({struct _tuple18*_tmp343=({struct _tuple18*_tmp1B8=_cycalloc(sizeof(*_tmp1B8));_tmp1B8->f1=(struct Cyc_Absyn_Tvar*)tvs->hd,_tmp1B8->f2=e;_tmp1B8;});_tmp1B9->hd=_tmp343;}),_tmp1B9->tl=inst;_tmp1B9;});}
# 1196
({struct Cyc_List_List*_tmp345=({struct Cyc_List_List*_tmp344=targs;Cyc_List_imp_append(_tmp344,Cyc_List_imp_rev(hidden_ts));});*targs_ref=_tmp345;});}
# 1183
if(td->defn != 0){
# 1199
void*new_typ=
inst == 0?(void*)_check_null(td->defn):
 Cyc_Tcutil_substitute(inst,(void*)_check_null(td->defn));
*toptp=new_typ;}
# 1183
goto _LL0;}}}_LL0:;}
# 1206
if(!({struct Cyc_Absyn_Kind*_tmp346=Cyc_Tcutil_type_kind(t);Cyc_Tcutil_kind_leq(_tmp346,expected_kind);}))
({struct Cyc_String_pa_PrintArg_struct _tmp1BC=({struct Cyc_String_pa_PrintArg_struct _tmp233;_tmp233.tag=0U,({
struct _fat_ptr _tmp347=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t));_tmp233.f1=_tmp347;});_tmp233;});struct Cyc_String_pa_PrintArg_struct _tmp1BD=({struct Cyc_String_pa_PrintArg_struct _tmp232;_tmp232.tag=0U,({struct _fat_ptr _tmp348=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_kind2string(Cyc_Tcutil_type_kind(t)));_tmp232.f1=_tmp348;});_tmp232;});struct Cyc_String_pa_PrintArg_struct _tmp1BE=({struct Cyc_String_pa_PrintArg_struct _tmp231;_tmp231.tag=0U,({struct _fat_ptr _tmp349=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_kind2string(expected_kind));_tmp231.f1=_tmp349;});_tmp231;});void*_tmp1BA[3U];_tmp1BA[0]=& _tmp1BC,_tmp1BA[1]=& _tmp1BD,_tmp1BA[2]=& _tmp1BE;({unsigned _tmp34B=loc;struct _fat_ptr _tmp34A=({const char*_tmp1BB="type %s has kind %s but as used here needs kind %s";_tag_fat(_tmp1BB,sizeof(char),51U);});Cyc_Tcutil_terr(_tmp34B,_tmp34A,_tag_fat(_tmp1BA,sizeof(void*),3U));});});
# 1206
return cvtenv;}
# 364
static struct Cyc_Tctyp_CVTEnv Cyc_Tctyp_i_check_valid_type_level_exp(struct Cyc_Absyn_Exp*e,struct Cyc_Tcenv_Tenv*te,struct Cyc_Tctyp_CVTEnv cvtenv){
# 1220
{void*_stmttmp2B=e->r;void*_tmp1BF=_stmttmp2B;struct Cyc_Absyn_Exp*_tmp1C0;struct Cyc_Absyn_Exp*_tmp1C1;void*_tmp1C2;void*_tmp1C3;void*_tmp1C4;struct Cyc_Absyn_Exp*_tmp1C6;void*_tmp1C5;struct Cyc_Absyn_Exp*_tmp1C8;struct Cyc_Absyn_Exp*_tmp1C7;struct Cyc_Absyn_Exp*_tmp1CA;struct Cyc_Absyn_Exp*_tmp1C9;struct Cyc_Absyn_Exp*_tmp1CC;struct Cyc_Absyn_Exp*_tmp1CB;struct Cyc_Absyn_Exp*_tmp1CF;struct Cyc_Absyn_Exp*_tmp1CE;struct Cyc_Absyn_Exp*_tmp1CD;struct Cyc_List_List*_tmp1D0;switch(*((int*)_tmp1BF)){case 0U: _LL1: _LL2:
 goto _LL4;case 32U: _LL3: _LL4:
 goto _LL6;case 33U: _LL5: _LL6:
 goto _LL8;case 2U: _LL7: _LL8:
 goto _LLA;case 1U: _LL9: _LLA:
 goto _LL0;case 3U: _LLB: _tmp1D0=((struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct*)_tmp1BF)->f2;_LLC: {struct Cyc_List_List*es=_tmp1D0;
# 1227
for(0;es != 0;es=es->tl){
cvtenv=Cyc_Tctyp_i_check_valid_type_level_exp((struct Cyc_Absyn_Exp*)es->hd,te,cvtenv);}
goto _LL0;}case 6U: _LLD: _tmp1CD=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmp1BF)->f1;_tmp1CE=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmp1BF)->f2;_tmp1CF=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmp1BF)->f3;_LLE: {struct Cyc_Absyn_Exp*e1=_tmp1CD;struct Cyc_Absyn_Exp*e2=_tmp1CE;struct Cyc_Absyn_Exp*e3=_tmp1CF;
# 1231
cvtenv=Cyc_Tctyp_i_check_valid_type_level_exp(e1,te,cvtenv);
cvtenv=Cyc_Tctyp_i_check_valid_type_level_exp(e2,te,cvtenv);
cvtenv=Cyc_Tctyp_i_check_valid_type_level_exp(e3,te,cvtenv);
goto _LL0;}case 7U: _LLF: _tmp1CB=((struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct*)_tmp1BF)->f1;_tmp1CC=((struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct*)_tmp1BF)->f2;_LL10: {struct Cyc_Absyn_Exp*e1=_tmp1CB;struct Cyc_Absyn_Exp*e2=_tmp1CC;
_tmp1C9=e1;_tmp1CA=e2;goto _LL12;}case 8U: _LL11: _tmp1C9=((struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct*)_tmp1BF)->f1;_tmp1CA=((struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct*)_tmp1BF)->f2;_LL12: {struct Cyc_Absyn_Exp*e1=_tmp1C9;struct Cyc_Absyn_Exp*e2=_tmp1CA;
_tmp1C7=e1;_tmp1C8=e2;goto _LL14;}case 9U: _LL13: _tmp1C7=((struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct*)_tmp1BF)->f1;_tmp1C8=((struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct*)_tmp1BF)->f2;_LL14: {struct Cyc_Absyn_Exp*e1=_tmp1C7;struct Cyc_Absyn_Exp*e2=_tmp1C8;
# 1238
cvtenv=Cyc_Tctyp_i_check_valid_type_level_exp(e1,te,cvtenv);
cvtenv=Cyc_Tctyp_i_check_valid_type_level_exp(e2,te,cvtenv);
goto _LL0;}case 14U: _LL15: _tmp1C5=(void*)((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp1BF)->f1;_tmp1C6=((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp1BF)->f2;_LL16: {void*t=_tmp1C5;struct Cyc_Absyn_Exp*e1=_tmp1C6;
# 1242
cvtenv=Cyc_Tctyp_i_check_valid_type_level_exp(e1,te,cvtenv);
cvtenv=Cyc_Tctyp_i_check_valid_type(e->loc,te,cvtenv,& Cyc_Tcutil_tak,t,1,0);
goto _LL0;}case 19U: _LL17: _tmp1C4=(void*)((struct Cyc_Absyn_Offsetof_e_Absyn_Raw_exp_struct*)_tmp1BF)->f1;_LL18: {void*t=_tmp1C4;
_tmp1C3=t;goto _LL1A;}case 17U: _LL19: _tmp1C3=(void*)((struct Cyc_Absyn_Sizeoftype_e_Absyn_Raw_exp_struct*)_tmp1BF)->f1;_LL1A: {void*t=_tmp1C3;
# 1247
cvtenv=Cyc_Tctyp_i_check_valid_type(e->loc,te,cvtenv,& Cyc_Tcutil_tak,t,1,0);
goto _LL0;}case 40U: _LL1B: _tmp1C2=(void*)((struct Cyc_Absyn_Valueof_e_Absyn_Raw_exp_struct*)_tmp1BF)->f1;_LL1C: {void*t=_tmp1C2;
# 1250
cvtenv=Cyc_Tctyp_i_check_valid_type(e->loc,te,cvtenv,& Cyc_Tcutil_ik,t,1,0);
goto _LL0;}case 18U: _LL1D: _tmp1C1=((struct Cyc_Absyn_Sizeofexp_e_Absyn_Raw_exp_struct*)_tmp1BF)->f1;_LL1E: {struct Cyc_Absyn_Exp*e=_tmp1C1;
# 1253
cvtenv=Cyc_Tctyp_i_check_valid_type_level_exp(e,te,cvtenv);
goto _LL0;}case 42U: _LL1F: _tmp1C0=((struct Cyc_Absyn_Extension_e_Absyn_Raw_exp_struct*)_tmp1BF)->f1;_LL20: {struct Cyc_Absyn_Exp*e=_tmp1C0;
# 1256
cvtenv=Cyc_Tctyp_i_check_valid_type_level_exp(e,te,cvtenv);
goto _LL0;}default: _LL21: _LL22:
# 1259
({void*_tmp1D1=0U;({int(*_tmp34D)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp1D3)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp1D3;});struct _fat_ptr _tmp34C=({const char*_tmp1D2="non-type-level-expression in Tcutil::i_check_valid_type_level_exp";_tag_fat(_tmp1D2,sizeof(char),66U);});_tmp34D(_tmp34C,_tag_fat(_tmp1D1,sizeof(void*),0U));});});}_LL0:;}
# 1261
return cvtenv;}
# 364
static struct Cyc_Tctyp_CVTEnv Cyc_Tctyp_check_valid_type(unsigned loc,struct Cyc_Tcenv_Tenv*te,struct Cyc_Tctyp_CVTEnv cvt,struct Cyc_Absyn_Kind*expected_kind,int allow_abs_aggr,void*t){
# 1269
struct Cyc_List_List*kind_env=cvt.kind_env;
cvt=Cyc_Tctyp_i_check_valid_type(loc,te,cvt,expected_kind,t,1,allow_abs_aggr);
# 1272
{struct Cyc_List_List*vs=cvt.free_vars;for(0;vs != 0;vs=vs->tl){
({struct Cyc_List_List*_tmp34E=Cyc_Tctyp_fast_add_free_tvar(kind_env,(*((struct _tuple14*)vs->hd)).f1);cvt.kind_env=_tmp34E;});}}
# 1278
{struct Cyc_List_List*evs=cvt.free_evars;for(0;evs != 0;evs=evs->tl){
struct _tuple13 _stmttmp2C=*((struct _tuple13*)evs->hd);void*_tmp1D4;_LL1: _tmp1D4=_stmttmp2C.f1;_LL2: {void*e=_tmp1D4;
void*_stmttmp2D=Cyc_Tcutil_compress(e);void*_tmp1D5=_stmttmp2D;struct Cyc_Core_Opt**_tmp1D6;if(((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp1D5)->tag == 1U){_LL4: _tmp1D6=(struct Cyc_Core_Opt**)&((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp1D5)->f4;_LL5: {struct Cyc_Core_Opt**s=_tmp1D6;
# 1282
if(*s == 0)
({struct Cyc_Core_Opt*_tmp34F=({struct Cyc_Core_Opt*_tmp1D7=_cycalloc(sizeof(*_tmp1D7));_tmp1D7->v=kind_env;_tmp1D7;});*s=_tmp34F;});else{
# 1286
struct Cyc_List_List*tvs=(struct Cyc_List_List*)((struct Cyc_Core_Opt*)_check_null(*s))->v;
struct Cyc_List_List*result=0;
for(0;tvs != 0;tvs=tvs->tl){
if(({int(*_tmp351)(int(*compare)(struct Cyc_Absyn_Tvar*,struct Cyc_Absyn_Tvar*),struct Cyc_List_List*l,struct Cyc_Absyn_Tvar*x)=({int(*_tmp1D8)(int(*compare)(struct Cyc_Absyn_Tvar*,struct Cyc_Absyn_Tvar*),struct Cyc_List_List*l,struct Cyc_Absyn_Tvar*x)=(int(*)(int(*compare)(struct Cyc_Absyn_Tvar*,struct Cyc_Absyn_Tvar*),struct Cyc_List_List*l,struct Cyc_Absyn_Tvar*x))Cyc_List_mem;_tmp1D8;});struct Cyc_List_List*_tmp350=kind_env;_tmp351(Cyc_Tcutil_fast_tvar_cmp,_tmp350,(struct Cyc_Absyn_Tvar*)tvs->hd);}))
result=({struct Cyc_List_List*_tmp1D9=_cycalloc(sizeof(*_tmp1D9));_tmp1D9->hd=(struct Cyc_Absyn_Tvar*)tvs->hd,_tmp1D9->tl=result;_tmp1D9;});}
# 1288
({struct Cyc_Core_Opt*_tmp352=({struct Cyc_Core_Opt*_tmp1DA=_cycalloc(sizeof(*_tmp1DA));_tmp1DA->v=result;_tmp1DA;});*s=_tmp352;});}
# 1293
goto _LL3;}}else{_LL6: _LL7:
 goto _LL3;}_LL3:;}}}
# 1297
return cvt;}
# 364
void Cyc_Tctyp_check_valid_toplevel_type(unsigned loc,struct Cyc_Tcenv_Tenv*te,void*t){
# 1306
int generalize_evars=Cyc_Tcutil_is_function_type(t);
if(te->in_tempest && !te->tempest_generalize)generalize_evars=0;{struct Cyc_List_List*te_vars=
# 1309
Cyc_Tcenv_lookup_type_vars(te);
struct Cyc_Absyn_Kind*expected_kind=Cyc_Tcutil_is_function_type(t)?& Cyc_Tcutil_tak:& Cyc_Tcutil_tmk;
struct Cyc_Tctyp_CVTEnv cvt=({unsigned _tmp356=loc;struct Cyc_Tcenv_Tenv*_tmp355=te;struct Cyc_Tctyp_CVTEnv _tmp354=({struct Cyc_Tctyp_CVTEnv _tmp237;_tmp237.r=Cyc_Core_heap_region,_tmp237.kind_env=te_vars,_tmp237.fn_result=0,_tmp237.generalize_evars=generalize_evars,_tmp237.free_vars=0,_tmp237.free_evars=0;_tmp237;});struct Cyc_Absyn_Kind*_tmp353=expected_kind;Cyc_Tctyp_check_valid_type(_tmp356,_tmp355,_tmp354,_tmp353,1,t);});
# 1314
struct Cyc_List_List*free_tvars=({struct Cyc_List_List*(*_tmp358)(struct Cyc_Absyn_Tvar*(*f)(struct _tuple14*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp1F2)(struct Cyc_Absyn_Tvar*(*f)(struct _tuple14*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_Absyn_Tvar*(*f)(struct _tuple14*),struct Cyc_List_List*x))Cyc_List_map;_tmp1F2;});struct Cyc_Absyn_Tvar*(*_tmp357)(struct _tuple14*)=({struct Cyc_Absyn_Tvar*(*_tmp1F3)(struct _tuple14*)=(struct Cyc_Absyn_Tvar*(*)(struct _tuple14*))Cyc_Core_fst;_tmp1F3;});_tmp358(_tmp357,cvt.free_vars);});
struct Cyc_List_List*free_evars=({struct Cyc_List_List*(*_tmp35A)(void*(*f)(struct _tuple13*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp1F0)(void*(*f)(struct _tuple13*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(void*(*f)(struct _tuple13*),struct Cyc_List_List*x))Cyc_List_map;_tmp1F0;});void*(*_tmp359)(struct _tuple13*)=({void*(*_tmp1F1)(struct _tuple13*)=(void*(*)(struct _tuple13*))Cyc_Core_fst;_tmp1F1;});_tmp35A(_tmp359,cvt.free_evars);});
# 1318
if(te_vars != 0){
# 1320
struct Cyc_List_List*res=0;
# 1322
{struct Cyc_List_List*fs=free_tvars;for(0;(unsigned)fs;fs=fs->tl){
# 1324
struct Cyc_Absyn_Tvar*f=(struct Cyc_Absyn_Tvar*)fs->hd;
int found=0;
{struct Cyc_List_List*ts=te_vars;for(0;(unsigned)ts;ts=ts->tl){
if(Cyc_Absyn_tvar_cmp(f,(struct Cyc_Absyn_Tvar*)ts->hd)== 0){found=1;break;}}}
# 1326
if(!found)
# 1328
res=({struct Cyc_List_List*_tmp1DB=_cycalloc(sizeof(*_tmp1DB));_tmp1DB->hd=(struct Cyc_Absyn_Tvar*)fs->hd,_tmp1DB->tl=res;_tmp1DB;});}}
# 1330
free_tvars=Cyc_List_imp_rev(res);}
# 1318
{
# 1336
struct Cyc_List_List*x=free_tvars;
# 1318
for(0;x != 0;x=x->tl){
# 1337
void*_stmttmp2E=Cyc_Absyn_compress_kb(((struct Cyc_Absyn_Tvar*)x->hd)->kind);void*_tmp1DC=_stmttmp2E;enum Cyc_Absyn_AliasQual _tmp1DD;struct Cyc_Absyn_Kind*_tmp1DF;struct Cyc_Core_Opt**_tmp1DE;struct Cyc_Core_Opt**_tmp1E0;struct Cyc_Core_Opt**_tmp1E1;struct Cyc_Core_Opt**_tmp1E2;struct Cyc_Core_Opt**_tmp1E3;struct Cyc_Core_Opt**_tmp1E4;struct Cyc_Core_Opt**_tmp1E5;switch(*((int*)_tmp1DC)){case 1U: _LL1: _tmp1E5=(struct Cyc_Core_Opt**)&((struct Cyc_Absyn_Unknown_kb_Absyn_KindBound_struct*)_tmp1DC)->f1;_LL2: {struct Cyc_Core_Opt**f=_tmp1E5;
_tmp1E4=f;goto _LL4;}case 2U: switch(((struct Cyc_Absyn_Kind*)((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp1DC)->f2)->kind){case Cyc_Absyn_BoxKind: if(((struct Cyc_Absyn_Kind*)((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp1DC)->f2)->aliasqual == Cyc_Absyn_Top){_LL3: _tmp1E4=(struct Cyc_Core_Opt**)&((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp1DC)->f1;_LL4: {struct Cyc_Core_Opt**f=_tmp1E4;
_tmp1E3=f;goto _LL6;}}else{goto _LLD;}case Cyc_Absyn_MemKind: switch(((struct Cyc_Absyn_Kind*)((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp1DC)->f2)->aliasqual){case Cyc_Absyn_Top: _LL5: _tmp1E3=(struct Cyc_Core_Opt**)&((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp1DC)->f1;_LL6: {struct Cyc_Core_Opt**f=_tmp1E3;
_tmp1E2=f;goto _LL8;}case Cyc_Absyn_Aliasable: _LL7: _tmp1E2=(struct Cyc_Core_Opt**)&((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp1DC)->f1;_LL8: {struct Cyc_Core_Opt**f=_tmp1E2;
# 1342
({struct Cyc_Core_Opt*_tmp35B=Cyc_Tcutil_kind_to_bound_opt(& Cyc_Tcutil_bk);*f=_tmp35B;});goto _LL0;}case Cyc_Absyn_Unique: _LL9: _tmp1E1=(struct Cyc_Core_Opt**)&((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp1DC)->f1;_LLA: {struct Cyc_Core_Opt**f=_tmp1E1;
# 1344
({struct Cyc_Core_Opt*_tmp35C=Cyc_Tcutil_kind_to_bound_opt(& Cyc_Tcutil_ubk);*f=_tmp35C;});goto _LL0;}default: goto _LLD;}case Cyc_Absyn_RgnKind: if(((struct Cyc_Absyn_Kind*)((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp1DC)->f2)->aliasqual == Cyc_Absyn_Top){_LLB: _tmp1E0=(struct Cyc_Core_Opt**)&((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp1DC)->f1;_LLC: {struct Cyc_Core_Opt**f=_tmp1E0;
# 1346
({struct Cyc_Core_Opt*_tmp35D=Cyc_Tcutil_kind_to_bound_opt(& Cyc_Tcutil_rk);*f=_tmp35D;});goto _LL0;}}else{goto _LLD;}default: _LLD: _tmp1DE=(struct Cyc_Core_Opt**)&((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp1DC)->f1;_tmp1DF=((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp1DC)->f2;_LLE: {struct Cyc_Core_Opt**f=_tmp1DE;struct Cyc_Absyn_Kind*k=_tmp1DF;
# 1348
({struct Cyc_Core_Opt*_tmp35E=Cyc_Tcutil_kind_to_bound_opt(k);*f=_tmp35E;});goto _LL0;}}default: if(((struct Cyc_Absyn_Kind*)((struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct*)_tmp1DC)->f1)->kind == Cyc_Absyn_MemKind){_LLF: _tmp1DD=(((struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct*)_tmp1DC)->f1)->aliasqual;_LL10: {enum Cyc_Absyn_AliasQual a=_tmp1DD;
# 1350
({struct Cyc_String_pa_PrintArg_struct _tmp1E8=({struct Cyc_String_pa_PrintArg_struct _tmp235;_tmp235.tag=0U,({
struct _fat_ptr _tmp35F=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_tvar2string((struct Cyc_Absyn_Tvar*)x->hd));_tmp235.f1=_tmp35F;});_tmp235;});struct Cyc_String_pa_PrintArg_struct _tmp1E9=({struct Cyc_String_pa_PrintArg_struct _tmp234;_tmp234.tag=0U,({struct _fat_ptr _tmp360=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_kind2string(({struct Cyc_Absyn_Kind*_tmp1EA=_cycalloc(sizeof(*_tmp1EA));_tmp1EA->kind=Cyc_Absyn_MemKind,_tmp1EA->aliasqual=a;_tmp1EA;})));_tmp234.f1=_tmp360;});_tmp234;});void*_tmp1E6[2U];_tmp1E6[0]=& _tmp1E8,_tmp1E6[1]=& _tmp1E9;({unsigned _tmp362=loc;struct _fat_ptr _tmp361=({const char*_tmp1E7="type variable %s cannot have kind %s";_tag_fat(_tmp1E7,sizeof(char),37U);});Cyc_Tcutil_terr(_tmp362,_tmp361,_tag_fat(_tmp1E6,sizeof(void*),2U));});});
goto _LL0;}}else{_LL11: _LL12:
 goto _LL0;}}_LL0:;}}
# 1357
if(free_tvars != 0 || free_evars != 0){
{void*_stmttmp2F=Cyc_Tcutil_compress(t);void*_tmp1EB=_stmttmp2F;struct Cyc_List_List**_tmp1EC;if(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp1EB)->tag == 5U){_LL14: _tmp1EC=(struct Cyc_List_List**)&(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp1EB)->f1).tvars;_LL15: {struct Cyc_List_List**btvs=_tmp1EC;
# 1360
if(*btvs == 0){
# 1362
({struct Cyc_List_List*_tmp363=Cyc_List_copy(free_tvars);*btvs=_tmp363;});
free_tvars=0;}
# 1360
goto _LL13;}}else{_LL16: _LL17:
# 1366
 goto _LL13;}_LL13:;}
# 1368
if(free_tvars != 0)
({struct Cyc_String_pa_PrintArg_struct _tmp1EF=({struct Cyc_String_pa_PrintArg_struct _tmp236;_tmp236.tag=0U,_tmp236.f1=(struct _fat_ptr)((struct _fat_ptr)*((struct Cyc_Absyn_Tvar*)free_tvars->hd)->name);_tmp236;});void*_tmp1ED[1U];_tmp1ED[0]=& _tmp1EF;({unsigned _tmp365=loc;struct _fat_ptr _tmp364=({const char*_tmp1EE="unbound type variable %s";_tag_fat(_tmp1EE,sizeof(char),25U);});Cyc_Tcutil_terr(_tmp365,_tmp364,_tag_fat(_tmp1ED,sizeof(void*),1U));});});
# 1368
if(
# 1370
!Cyc_Tcutil_is_function_type(t)|| !te->in_tempest)
Cyc_Tctyp_check_free_evars(free_evars,t,loc);}}}
# 364
void Cyc_Tctyp_check_fndecl_valid_type(unsigned loc,struct Cyc_Tcenv_Tenv*te,struct Cyc_Absyn_Fndecl*fd){
# 1380
void*t=Cyc_Tcutil_fndecl2type(fd);
# 1382
Cyc_Tctyp_check_valid_toplevel_type(loc,te,t);
{void*_stmttmp30=Cyc_Tcutil_compress(t);void*_tmp1F4=_stmttmp30;struct Cyc_Absyn_FnInfo _tmp1F5;if(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp1F4)->tag == 5U){_LL1: _tmp1F5=((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp1F4)->f1;_LL2: {struct Cyc_Absyn_FnInfo i=_tmp1F5;
# 1385
struct Cyc_List_List*atts=(fd->i).attributes;
fd->i=i;
(fd->i).attributes=atts;
({int _tmp366=
Cyc_Tcutil_extract_const_from_typedef(loc,(i.ret_tqual).print_const,i.ret_type);
# 1388
((fd->i).ret_tqual).real_const=_tmp366;});
# 1390
goto _LL0;}}else{_LL3: _LL4:
({void*_tmp1F6=0U;({int(*_tmp368)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp1F8)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp1F8;});struct _fat_ptr _tmp367=({const char*_tmp1F7="check_fndecl_valid_type: not a FnType";_tag_fat(_tmp1F7,sizeof(char),38U);});_tmp368(_tmp367,_tag_fat(_tmp1F6,sizeof(void*),0U));});});}_LL0:;}
# 1393
({struct Cyc_List_List*_tmp36B=({struct Cyc_List_List*(*_tmp369)(struct _fat_ptr*(*f)(struct _tuple9*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp1F9)(struct _fat_ptr*(*f)(struct _tuple9*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct _fat_ptr*(*f)(struct _tuple9*),struct Cyc_List_List*x))Cyc_List_map;_tmp1F9;});_tmp369(Cyc_Tctyp_fst_fdarg,(fd->i).args);});unsigned _tmp36A=loc;Cyc_Tcutil_check_unique_vars(_tmp36B,_tmp36A,({const char*_tmp1FA="function declaration has repeated parameter";_tag_fat(_tmp1FA,sizeof(char),44U);}));});
# 1396
fd->cached_type=t;}
# 364
void Cyc_Tctyp_check_type(unsigned loc,struct Cyc_Tcenv_Tenv*te,struct Cyc_List_List*bound_tvars,struct Cyc_Absyn_Kind*expected_kind,int allow_evars,int allow_abs_aggr,void*t){
# 1405
struct Cyc_Tctyp_CVTEnv cvt=({unsigned _tmp370=loc;struct Cyc_Tcenv_Tenv*_tmp36F=te;struct Cyc_Tctyp_CVTEnv _tmp36E=({struct Cyc_Tctyp_CVTEnv _tmp23A;_tmp23A.r=Cyc_Core_heap_region,_tmp23A.kind_env=bound_tvars,_tmp23A.fn_result=0,_tmp23A.generalize_evars=0,_tmp23A.free_vars=0,_tmp23A.free_evars=0;_tmp23A;});struct Cyc_Absyn_Kind*_tmp36D=expected_kind;int _tmp36C=allow_abs_aggr;Cyc_Tctyp_check_valid_type(_tmp370,_tmp36F,_tmp36E,_tmp36D,_tmp36C,t);});
# 1409
struct Cyc_List_List*free_tvars=({struct _RegionHandle*_tmp374=Cyc_Core_heap_region;struct Cyc_List_List*_tmp373=({struct Cyc_List_List*(*_tmp372)(struct Cyc_Absyn_Tvar*(*f)(struct _tuple14*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp201)(struct Cyc_Absyn_Tvar*(*f)(struct _tuple14*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_Absyn_Tvar*(*f)(struct _tuple14*),struct Cyc_List_List*x))Cyc_List_map;_tmp201;});struct Cyc_Absyn_Tvar*(*_tmp371)(struct _tuple14*)=({struct Cyc_Absyn_Tvar*(*_tmp202)(struct _tuple14*)=(struct Cyc_Absyn_Tvar*(*)(struct _tuple14*))Cyc_Core_fst;_tmp202;});_tmp372(_tmp371,cvt.free_vars);});Cyc_Tctyp_remove_bound_tvars(_tmp374,_tmp373,bound_tvars);});
# 1411
struct Cyc_List_List*free_evars=({struct Cyc_List_List*(*_tmp376)(void*(*f)(struct _tuple13*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp1FF)(void*(*f)(struct _tuple13*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(void*(*f)(struct _tuple13*),struct Cyc_List_List*x))Cyc_List_map;_tmp1FF;});void*(*_tmp375)(struct _tuple13*)=({void*(*_tmp200)(struct _tuple13*)=(void*(*)(struct _tuple13*))Cyc_Core_fst;_tmp200;});_tmp376(_tmp375,cvt.free_evars);});
{struct Cyc_List_List*fs=free_tvars;for(0;fs != 0;fs=fs->tl){
({struct Cyc_String_pa_PrintArg_struct _tmp1FD=({struct Cyc_String_pa_PrintArg_struct _tmp239;_tmp239.tag=0U,_tmp239.f1=(struct _fat_ptr)((struct _fat_ptr)*((struct Cyc_Absyn_Tvar*)fs->hd)->name);_tmp239;});struct Cyc_String_pa_PrintArg_struct _tmp1FE=({struct Cyc_String_pa_PrintArg_struct _tmp238;_tmp238.tag=0U,({struct _fat_ptr _tmp377=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t));_tmp238.f1=_tmp377;});_tmp238;});void*_tmp1FB[2U];_tmp1FB[0]=& _tmp1FD,_tmp1FB[1]=& _tmp1FE;({unsigned _tmp379=loc;struct _fat_ptr _tmp378=({const char*_tmp1FC="unbound type variable %s in type %s";_tag_fat(_tmp1FC,sizeof(char),36U);});Cyc_Tcutil_terr(_tmp379,_tmp378,_tag_fat(_tmp1FB,sizeof(void*),2U));});});}}
if(!allow_evars)
Cyc_Tctyp_check_free_evars(free_evars,t,loc);}
