#include <setjmp.h>
/* This is a C header file to be used by the output of the Cyclone to
   C translator.  The corresponding definitions are in file lib/runtime_*.c */
#ifndef _CYC_INCLUDE_H_
#define _CYC_INCLUDE_H_

/* Need one of these per thread (see runtime_stack.c). The runtime maintains 
   a stack that contains either _handler_cons structs or _RegionHandle structs.
   The tag is 0 for a handler_cons and 1 for a region handle.  */
struct _RuntimeStack {
  int tag; 
  struct _RuntimeStack *next;
  void (*cleanup)(struct _RuntimeStack *frame);
};

#ifndef offsetof
/* should be size_t, but int is fine. */
#define offsetof(t,n) ((int)(&(((t *)0)->n)))
#endif

/* Fat pointers */
struct _fat_ptr {
  unsigned char *curr; 
  unsigned char *base; 
  unsigned char *last_plus_one; 
};  

/* Discriminated Unions */
struct _xtunion_struct { char *tag; };

/* Regions */
struct _RegionPage
#ifdef CYC_REGION_PROFILE
{ unsigned total_bytes;
  unsigned free_bytes;
  /* MWH: wish we didn't have to include the stuff below ... */
  struct _RegionPage *next;
  char data[1];
}
#endif
; // abstract -- defined in runtime_memory.c
struct _RegionHandle {
  struct _RuntimeStack s;
  struct _RegionPage *curr;
  char               *offset;
  char               *last_plus_one;
  struct _DynRegionHandle *sub_regions;
#ifdef CYC_REGION_PROFILE
  const char         *name;
#else
  unsigned used_bytes;
  unsigned wasted_bytes;
#endif
};
struct _DynRegionFrame {
  struct _RuntimeStack s;
  struct _DynRegionHandle *x;
};
// A dynamic region is just a region handle.  The wrapper struct is for type
// abstraction.
struct Cyc_Core_DynamicRegion {
  struct _RegionHandle h;
};

struct _RegionHandle _new_region(const char*);
void* _region_malloc(struct _RegionHandle*, unsigned);
void* _region_calloc(struct _RegionHandle*, unsigned t, unsigned n);
void   _free_region(struct _RegionHandle*);
struct _RegionHandle*_open_dynregion(struct _DynRegionFrame*,struct _DynRegionHandle*);
void   _pop_dynregion();

/* Exceptions */
struct _handler_cons {
  struct _RuntimeStack s;
  jmp_buf handler;
};
void _push_handler(struct _handler_cons *);
void _push_region(struct _RegionHandle *);
void _npop_handler(int);
void _pop_handler();
void _pop_region();

#ifndef _throw
void* _throw_null_fn(const char*,unsigned);
void* _throw_arraybounds_fn(const char*,unsigned);
void* _throw_badalloc_fn(const char*,unsigned);
void* _throw_match_fn(const char*,unsigned);
void* _throw_fn(void*,const char*,unsigned);
void* _rethrow(void*);
#define _throw_null() (_throw_null_fn(__FILE__,__LINE__))
#define _throw_arraybounds() (_throw_arraybounds_fn(__FILE__,__LINE__))
#define _throw_badalloc() (_throw_badalloc_fn(__FILE__,__LINE__))
#define _throw_match() (_throw_match_fn(__FILE__,__LINE__))
#define _throw(e) (_throw_fn((e),__FILE__,__LINE__))
#endif

struct _xtunion_struct* Cyc_Core_get_exn_thrown();
/* Built-in Exceptions */
struct Cyc_Null_Exception_exn_struct { char *tag; };
struct Cyc_Array_bounds_exn_struct { char *tag; };
struct Cyc_Match_Exception_exn_struct { char *tag; };
struct Cyc_Bad_alloc_exn_struct { char *tag; };
extern char Cyc_Null_Exception[];
extern char Cyc_Array_bounds[];
extern char Cyc_Match_Exception[];
extern char Cyc_Bad_alloc[];

/* Built-in Run-time Checks and company */
#ifdef CYC_ANSI_OUTPUT
#define _INLINE  
#else
#define _INLINE inline
#endif

#ifdef NO_CYC_NULL_CHECKS
#define _check_null(ptr) (ptr)
#else
#define _check_null(ptr) \
  ({ void*_cks_null = (void*)(ptr); \
     if (!_cks_null) _throw_null(); \
     _cks_null; })
#endif

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index)\
   (((char*)ptr) + (elt_sz)*(index))
#ifdef NO_CYC_NULL_CHECKS
#define _check_known_subscript_null _check_known_subscript_notnull
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr);\
  int _index = (index);\
  if (!_cks_ptr) _throw_null(); \
  _cks_ptr + (elt_sz)*_index; })
#endif
#define _zero_arr_plus_fn(orig_x,orig_sz,orig_i,f,l) ((orig_x)+(orig_i))
#define _zero_arr_plus_char_fn _zero_arr_plus_fn
#define _zero_arr_plus_short_fn _zero_arr_plus_fn
#define _zero_arr_plus_int_fn _zero_arr_plus_fn
#define _zero_arr_plus_float_fn _zero_arr_plus_fn
#define _zero_arr_plus_double_fn _zero_arr_plus_fn
#define _zero_arr_plus_longdouble_fn _zero_arr_plus_fn
#define _zero_arr_plus_voidstar_fn _zero_arr_plus_fn
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (!_cks_ptr) _throw_null(); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })

/* _zero_arr_plus_*_fn(x,sz,i,filename,lineno) adds i to zero-terminated ptr
   x that has at least sz elements */
char* _zero_arr_plus_char_fn(char*,unsigned,int,const char*,unsigned);
short* _zero_arr_plus_short_fn(short*,unsigned,int,const char*,unsigned);
int* _zero_arr_plus_int_fn(int*,unsigned,int,const char*,unsigned);
float* _zero_arr_plus_float_fn(float*,unsigned,int,const char*,unsigned);
double* _zero_arr_plus_double_fn(double*,unsigned,int,const char*,unsigned);
long double* _zero_arr_plus_longdouble_fn(long double*,unsigned,int,const char*, unsigned);
void** _zero_arr_plus_voidstar_fn(void**,unsigned,int,const char*,unsigned);
#endif

/* _get_zero_arr_size_*(x,sz) returns the number of elements in a
   zero-terminated array that is NULL or has at least sz elements */
int _get_zero_arr_size_char(const char*,unsigned);
int _get_zero_arr_size_short(const short*,unsigned);
int _get_zero_arr_size_int(const int*,unsigned);
int _get_zero_arr_size_float(const float*,unsigned);
int _get_zero_arr_size_double(const double*,unsigned);
int _get_zero_arr_size_longdouble(const long double*,unsigned);
int _get_zero_arr_size_voidstar(const void**,unsigned);

/* _zero_arr_inplace_plus_*_fn(x,i,filename,lineno) sets
   zero-terminated pointer *x to *x + i */
char* _zero_arr_inplace_plus_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_short_fn(short**,int,const char*,unsigned);
int* _zero_arr_inplace_plus_int(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_longdouble_fn(long double**,int,const char*,unsigned);
void** _zero_arr_inplace_plus_voidstar_fn(void***,int,const char*,unsigned);
/* like the previous functions, but does post-addition (as in e++) */
char* _zero_arr_inplace_plus_post_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_post_short_fn(short**x,int,const char*,unsigned);
int* _zero_arr_inplace_plus_post_int_fn(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_post_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_post_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_post_longdouble_fn(long double**,int,const char *,unsigned);
void** _zero_arr_inplace_plus_post_voidstar_fn(void***,int,const char*,unsigned);
#define _zero_arr_plus_char(x,s,i) \
  (_zero_arr_plus_char_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_short(x,s,i) \
  (_zero_arr_plus_short_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_int(x,s,i) \
  (_zero_arr_plus_int_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_float(x,s,i) \
  (_zero_arr_plus_float_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_double(x,s,i) \
  (_zero_arr_plus_double_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_longdouble(x,s,i) \
  (_zero_arr_plus_longdouble_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_voidstar(x,s,i) \
  (_zero_arr_plus_voidstar_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_inplace_plus_char(x,i) \
  _zero_arr_inplace_plus_char_fn((char **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_short(x,i) \
  _zero_arr_inplace_plus_short_fn((short **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_int(x,i) \
  _zero_arr_inplace_plus_int_fn((int **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_float(x,i) \
  _zero_arr_inplace_plus_float_fn((float **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_double(x,i) \
  _zero_arr_inplace_plus_double_fn((double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_longdouble(x,i) \
  _zero_arr_inplace_plus_longdouble_fn((long double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_voidstar(x,i) \
  _zero_arr_inplace_plus_voidstar_fn((void ***)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_char(x,i) \
  _zero_arr_inplace_plus_post_char_fn((char **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_short(x,i) \
  _zero_arr_inplace_plus_post_short_fn((short **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_int(x,i) \
  _zero_arr_inplace_plus_post_int_fn((int **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_float(x,i) \
  _zero_arr_inplace_plus_post_float_fn((float **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_double(x,i) \
  _zero_arr_inplace_plus_post_double_fn((double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_longdouble(x,i) \
  _zero_arr_inplace_plus_post_longdouble_fn((long double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_voidstar(x,i) \
  _zero_arr_inplace_plus_post_voidstar_fn((void***)(x),(i),__FILE__,__LINE__)

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_fat_subscript(arr,elt_sz,index) ((arr).curr + (elt_sz) * (index))
#define _untag_fat_ptr(arr,elt_sz,num_elts) ((arr).curr)
#else
#define _check_fat_subscript(arr,elt_sz,index) ({ \
  struct _fat_ptr _cus_arr = (arr); \
  unsigned char *_cus_ans = _cus_arr.curr + (elt_sz) * (index); \
  /* JGM: not needed! if (!_cus_arr.base) _throw_null();*/ \
  if (_cus_ans < _cus_arr.base || _cus_ans >= _cus_arr.last_plus_one) \
    _throw_arraybounds(); \
  _cus_ans; })
#define _untag_fat_ptr(arr,elt_sz,num_elts) ({ \
  struct _fat_ptr _arr = (arr); \
  unsigned char *_curr = _arr.curr; \
  if ((_curr < _arr.base || _curr + (elt_sz) * (num_elts) > _arr.last_plus_one) &&\
      _curr != (unsigned char *)0) \
    _throw_arraybounds(); \
  _curr; })
#endif

#define _tag_fat(tcurr,elt_sz,num_elts) ({ \
  struct _fat_ptr _ans; \
  unsigned _num_elts = (num_elts);\
  _ans.base = _ans.curr = (void*)(tcurr); \
  /* JGM: if we're tagging NULL, ignore num_elts */ \
  _ans.last_plus_one = _ans.base ? (_ans.base + (elt_sz) * _num_elts) : 0; \
  _ans; })

#define _get_fat_size(arr,elt_sz) \
  ({struct _fat_ptr _arr = (arr); \
    unsigned char *_arr_curr=_arr.curr; \
    unsigned char *_arr_last=_arr.last_plus_one; \
    (_arr_curr < _arr.base || _arr_curr >= _arr_last) ? 0 : \
    ((_arr_last - _arr_curr) / (elt_sz));})

#define _fat_ptr_plus(arr,elt_sz,change) ({ \
  struct _fat_ptr _ans = (arr); \
  int _change = (change);\
  _ans.curr += (elt_sz) * _change;\
  _ans; })
#define _fat_ptr_inplace_plus(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  _arr_ptr->curr += (elt_sz) * (change);\
  *_arr_ptr; })
#define _fat_ptr_inplace_plus_post(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  struct _fat_ptr _ans = *_arr_ptr; \
  _arr_ptr->curr += (elt_sz) * (change);\
  _ans; })

//Not a macro since initialization order matters. Defined in runtime_zeroterm.c.
struct _fat_ptr _fat_ptr_decrease_size(struct _fat_ptr,unsigned sz,unsigned numelts);

/* Allocation */
void* GC_malloc(int);
void* GC_malloc_atomic(int);
void* GC_calloc(unsigned,unsigned);
void* GC_calloc_atomic(unsigned,unsigned);
// bound the allocation size to be < MAX_ALLOC_SIZE. See macros below for usage.
#define MAX_MALLOC_SIZE (1 << 28)
void* _bounded_GC_malloc(int,const char*,int);
void* _bounded_GC_malloc_atomic(int,const char*,int);
void* _bounded_GC_calloc(unsigned,unsigned,const char*,int);
void* _bounded_GC_calloc_atomic(unsigned,unsigned,const char*,int);
/* these macros are overridden below ifdef CYC_REGION_PROFILE */
#ifndef CYC_REGION_PROFILE
#define _cycalloc(n) _bounded_GC_malloc(n,__FILE__,__LINE__)
#define _cycalloc_atomic(n) _bounded_GC_malloc_atomic(n,__FILE__,__LINE__)
#define _cyccalloc(n,s) _bounded_GC_calloc(n,s,__FILE__,__LINE__)
#define _cyccalloc_atomic(n,s) _bounded_GC_calloc_atomic(n,s,__FILE__,__LINE__)
#endif

static _INLINE unsigned int _check_times(unsigned x, unsigned y) {
  unsigned long long whole_ans = 
    ((unsigned long long) x)*((unsigned long long)y);
  unsigned word_ans = (unsigned)whole_ans;
  if(word_ans < whole_ans || word_ans > MAX_MALLOC_SIZE)
    _throw_badalloc();
  return word_ans;
}

#define _CYC_MAX_REGION_CONST 2
#define _CYC_MIN_ALIGNMENT (sizeof(double))

#ifdef CYC_REGION_PROFILE
extern int rgn_total_bytes;
#endif

static _INLINE void *_fast_region_malloc(struct _RegionHandle *r, unsigned orig_s) {  
  if (r > (struct _RegionHandle *)_CYC_MAX_REGION_CONST && r->curr != 0) { 
#ifdef CYC_NOALIGN
    unsigned s =  orig_s;
#else
    unsigned s =  (orig_s + _CYC_MIN_ALIGNMENT - 1) & (~(_CYC_MIN_ALIGNMENT -1)); 
#endif
    char *result; 
    result = r->offset; 
    if (s <= (r->last_plus_one - result)) {
      r->offset = result + s; 
#ifdef CYC_REGION_PROFILE
    r->curr->free_bytes = r->curr->free_bytes - s;
    rgn_total_bytes += s;
#endif
      return result;
    }
  } 
  return _region_malloc(r,orig_s); 
}

#ifdef CYC_REGION_PROFILE
/* see macros below for usage. defined in runtime_memory.c */
void* _profile_GC_malloc(int,const char*,const char*,int);
void* _profile_GC_malloc_atomic(int,const char*,const char*,int);
void* _profile_GC_calloc(unsigned,unsigned,const char*,const char*,int);
void* _profile_GC_calloc_atomic(unsigned,unsigned,const char*,const char*,int);
void* _profile_region_malloc(struct _RegionHandle*,unsigned,const char*,const char*,int);
void* _profile_region_calloc(struct _RegionHandle*,unsigned,unsigned,const char *,const char*,int);
struct _RegionHandle _profile_new_region(const char*,const char*,const char*,int);
void _profile_free_region(struct _RegionHandle*,const char*,const char*,int);
#ifndef RUNTIME_CYC
#define _new_region(n) _profile_new_region(n,__FILE__,__FUNCTION__,__LINE__)
#define _free_region(r) _profile_free_region(r,__FILE__,__FUNCTION__,__LINE__)
#define _region_malloc(rh,n) _profile_region_malloc(rh,n,__FILE__,__FUNCTION__,__LINE__)
#define _region_calloc(rh,n,t) _profile_region_calloc(rh,n,t,__FILE__,__FUNCTION__,__LINE__)
#  endif
#define _cycalloc(n) _profile_GC_malloc(n,__FILE__,__FUNCTION__,__LINE__)
#define _cycalloc_atomic(n) _profile_GC_malloc_atomic(n,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc(n,s) _profile_GC_calloc(n,s,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc_atomic(n,s) _profile_GC_calloc_atomic(n,s,__FILE__,__FUNCTION__,__LINE__)
#endif
#endif
 struct Cyc_Core_Opt{void*v;};
# 119 "core.h"
int Cyc_Core_intcmp(int,int);extern char Cyc_Core_Invalid_argument[17U];struct Cyc_Core_Invalid_argument_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Failure[8U];struct Cyc_Core_Failure_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Impossible[11U];struct Cyc_Core_Impossible_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Not_found[10U];struct Cyc_Core_Not_found_exn_struct{char*tag;};extern char Cyc_Core_Unreachable[12U];struct Cyc_Core_Unreachable_exn_struct{char*tag;struct _fat_ptr f1;};
# 165
extern struct _RegionHandle*Cyc_Core_heap_region;
# 168
extern struct _RegionHandle*Cyc_Core_unique_region;struct Cyc_Core_DynamicRegion;struct Cyc_Core_NewDynamicRegion{struct Cyc_Core_DynamicRegion*key;};struct Cyc_Core_ThinRes{void*arr;unsigned nelts;};struct Cyc_List_List{void*hd;struct Cyc_List_List*tl;};
# 54 "list.h"
struct Cyc_List_List*Cyc_List_list(struct _fat_ptr);
# 61
int Cyc_List_length(struct Cyc_List_List*x);
# 76
struct Cyc_List_List*Cyc_List_map(void*(*f)(void*),struct Cyc_List_List*x);
# 83
struct Cyc_List_List*Cyc_List_map_c(void*(*f)(void*,void*),void*env,struct Cyc_List_List*x);
# 86
struct Cyc_List_List*Cyc_List_rmap_c(struct _RegionHandle*,void*(*f)(void*,void*),void*env,struct Cyc_List_List*x);extern char Cyc_List_List_mismatch[14U];struct Cyc_List_List_mismatch_exn_struct{char*tag;};
# 94
struct Cyc_List_List*Cyc_List_map2(void*(*f)(void*,void*),struct Cyc_List_List*x,struct Cyc_List_List*y);
# 133
void Cyc_List_iter(void(*f)(void*),struct Cyc_List_List*x);
# 135
void Cyc_List_iter_c(void(*f)(void*,void*),void*env,struct Cyc_List_List*x);
# 161
struct Cyc_List_List*Cyc_List_revappend(struct Cyc_List_List*x,struct Cyc_List_List*y);
# 178
struct Cyc_List_List*Cyc_List_imp_rev(struct Cyc_List_List*x);
# 205
struct Cyc_List_List*Cyc_List_rflatten(struct _RegionHandle*,struct Cyc_List_List*x);extern char Cyc_List_Nth[4U];struct Cyc_List_Nth_exn_struct{char*tag;};
# 261
int Cyc_List_exists_c(int(*pred)(void*,void*),void*env,struct Cyc_List_List*x);
# 270
struct Cyc_List_List*Cyc_List_zip(struct Cyc_List_List*x,struct Cyc_List_List*y);
# 276
struct Cyc_List_List*Cyc_List_rzip(struct _RegionHandle*r1,struct _RegionHandle*r2,struct Cyc_List_List*x,struct Cyc_List_List*y);struct _tuple0{struct Cyc_List_List*f1;struct Cyc_List_List*f2;};
# 303
struct _tuple0 Cyc_List_rsplit(struct _RegionHandle*r1,struct _RegionHandle*r2,struct Cyc_List_List*x);
# 322
int Cyc_List_mem(int(*compare)(void*,void*),struct Cyc_List_List*l,void*x);
# 336
void*Cyc_List_assoc_cmp(int(*cmp)(void*,void*),struct Cyc_List_List*l,void*x);
# 383
int Cyc_List_list_cmp(int(*cmp)(void*,void*),struct Cyc_List_List*l1,struct Cyc_List_List*l2);struct Cyc___cycFILE;struct Cyc_String_pa_PrintArg_struct{int tag;struct _fat_ptr f1;};struct Cyc_Int_pa_PrintArg_struct{int tag;unsigned long f1;};struct Cyc_Double_pa_PrintArg_struct{int tag;double f1;};struct Cyc_LongDouble_pa_PrintArg_struct{int tag;long double f1;};struct Cyc_ShortPtr_pa_PrintArg_struct{int tag;short*f1;};struct Cyc_IntPtr_pa_PrintArg_struct{int tag;unsigned long*f1;};
# 73 "cycboot.h"
struct _fat_ptr Cyc_aprintf(struct _fat_ptr,struct _fat_ptr);struct Cyc_ShortPtr_sa_ScanfArg_struct{int tag;short*f1;};struct Cyc_UShortPtr_sa_ScanfArg_struct{int tag;unsigned short*f1;};struct Cyc_IntPtr_sa_ScanfArg_struct{int tag;int*f1;};struct Cyc_UIntPtr_sa_ScanfArg_struct{int tag;unsigned*f1;};struct Cyc_StringPtr_sa_ScanfArg_struct{int tag;struct _fat_ptr f1;};struct Cyc_DoublePtr_sa_ScanfArg_struct{int tag;double*f1;};struct Cyc_FloatPtr_sa_ScanfArg_struct{int tag;float*f1;};struct Cyc_CharPtr_sa_ScanfArg_struct{int tag;struct _fat_ptr f1;};extern char Cyc_FileCloseError[15U];struct Cyc_FileCloseError_exn_struct{char*tag;};extern char Cyc_FileOpenError[14U];struct Cyc_FileOpenError_exn_struct{char*tag;struct _fat_ptr f1;};
# 38 "string.h"
unsigned long Cyc_strlen(struct _fat_ptr s);
# 49 "string.h"
int Cyc_strcmp(struct _fat_ptr s1,struct _fat_ptr s2);
int Cyc_strptrcmp(struct _fat_ptr*s1,struct _fat_ptr*s2);struct Cyc_PP_Ppstate;struct Cyc_PP_Out;struct Cyc_PP_Doc;struct Cyc_Position_Error;struct Cyc_Relations_Reln;struct _union_Nmspace_Rel_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_Abs_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_C_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_Loc_n{int tag;int val;};union Cyc_Absyn_Nmspace{struct _union_Nmspace_Rel_n Rel_n;struct _union_Nmspace_Abs_n Abs_n;struct _union_Nmspace_C_n C_n;struct _union_Nmspace_Loc_n Loc_n;};
# 95 "absyn.h"
extern union Cyc_Absyn_Nmspace Cyc_Absyn_Loc_n;struct _tuple1{union Cyc_Absyn_Nmspace f1;struct _fat_ptr*f2;};
# 161
enum Cyc_Absyn_Scope{Cyc_Absyn_Static =0U,Cyc_Absyn_Abstract =1U,Cyc_Absyn_Public =2U,Cyc_Absyn_Extern =3U,Cyc_Absyn_ExternC =4U,Cyc_Absyn_Register =5U};struct Cyc_Absyn_Tqual{int print_const: 1;int q_volatile: 1;int q_restrict: 1;int real_const: 1;unsigned loc;};
# 182
enum Cyc_Absyn_Size_of{Cyc_Absyn_Char_sz =0U,Cyc_Absyn_Short_sz =1U,Cyc_Absyn_Int_sz =2U,Cyc_Absyn_Long_sz =3U,Cyc_Absyn_LongLong_sz =4U};
enum Cyc_Absyn_Sign{Cyc_Absyn_Signed =0U,Cyc_Absyn_Unsigned =1U,Cyc_Absyn_None =2U};
enum Cyc_Absyn_AggrKind{Cyc_Absyn_StructA =0U,Cyc_Absyn_UnionA =1U};
# 187
enum Cyc_Absyn_AliasQual{Cyc_Absyn_Aliasable =0U,Cyc_Absyn_Unique =1U,Cyc_Absyn_Top =2U};
# 192
enum Cyc_Absyn_KindQual{Cyc_Absyn_AnyKind =0U,Cyc_Absyn_MemKind =1U,Cyc_Absyn_BoxKind =2U,Cyc_Absyn_RgnKind =3U,Cyc_Absyn_EffKind =4U,Cyc_Absyn_IntKind =5U,Cyc_Absyn_BoolKind =6U,Cyc_Absyn_PtrBndKind =7U};struct Cyc_Absyn_Kind{enum Cyc_Absyn_KindQual kind;enum Cyc_Absyn_AliasQual aliasqual;};struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct{int tag;struct Cyc_Absyn_Kind*f1;};struct Cyc_Absyn_Unknown_kb_Absyn_KindBound_struct{int tag;struct Cyc_Core_Opt*f1;};struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct{int tag;struct Cyc_Core_Opt*f1;struct Cyc_Absyn_Kind*f2;};struct Cyc_Absyn_Tvar{struct _fat_ptr*name;int identity;void*kind;};struct Cyc_Absyn_PtrLoc{unsigned ptr_loc;unsigned rgn_loc;unsigned zt_loc;};struct Cyc_Absyn_PtrAtts{void*rgn;void*nullable;void*bounds;void*zero_term;struct Cyc_Absyn_PtrLoc*ptrloc;};struct Cyc_Absyn_PtrInfo{void*elt_type;struct Cyc_Absyn_Tqual elt_tq;struct Cyc_Absyn_PtrAtts ptr_atts;};struct Cyc_Absyn_VarargInfo{struct _fat_ptr*name;struct Cyc_Absyn_Tqual tq;void*type;int inject;};struct Cyc_Absyn_Star_Absyn_Cap_struct{int tag;};struct Cyc_Absyn_Bot_Absyn_Cap_struct{int tag;};struct Cyc_Absyn_Nat_Absyn_Cap_struct{int tag;struct Cyc_Absyn_Exp*f1;int f2;};struct Cyc_Absyn_RgnEffect{void*name;void*rgn;void*lock;void*unadopt;void*parent;};struct Cyc_Absyn_FnInfo{struct Cyc_List_List*tvars;void*effect;struct Cyc_Absyn_Tqual ret_tqual;void*ret_type;struct Cyc_List_List*args;int c_varargs;struct Cyc_Absyn_VarargInfo*cyc_varargs;struct Cyc_List_List*rgn_po;struct Cyc_List_List*attributes;struct Cyc_Absyn_Exp*requires_clause;struct Cyc_List_List*requires_relns;struct Cyc_Absyn_Exp*ensures_clause;struct Cyc_List_List*ensures_relns;struct Cyc_List_List*ieffect;struct Cyc_List_List*oeffect;};struct Cyc_Absyn_UnknownDatatypeInfo{struct _tuple1*name;int is_extensible;};struct _union_DatatypeInfo_UnknownDatatype{int tag;struct Cyc_Absyn_UnknownDatatypeInfo val;};struct _union_DatatypeInfo_KnownDatatype{int tag;struct Cyc_Absyn_Datatypedecl**val;};union Cyc_Absyn_DatatypeInfo{struct _union_DatatypeInfo_UnknownDatatype UnknownDatatype;struct _union_DatatypeInfo_KnownDatatype KnownDatatype;};
# 311
union Cyc_Absyn_DatatypeInfo Cyc_Absyn_UnknownDatatype(struct Cyc_Absyn_UnknownDatatypeInfo);struct Cyc_Absyn_UnknownDatatypeFieldInfo{struct _tuple1*datatype_name;struct _tuple1*field_name;int is_extensible;};struct _union_DatatypeFieldInfo_UnknownDatatypefield{int tag;struct Cyc_Absyn_UnknownDatatypeFieldInfo val;};struct _tuple2{struct Cyc_Absyn_Datatypedecl*f1;struct Cyc_Absyn_Datatypefield*f2;};struct _union_DatatypeFieldInfo_KnownDatatypefield{int tag;struct _tuple2 val;};union Cyc_Absyn_DatatypeFieldInfo{struct _union_DatatypeFieldInfo_UnknownDatatypefield UnknownDatatypefield;struct _union_DatatypeFieldInfo_KnownDatatypefield KnownDatatypefield;};struct _tuple3{enum Cyc_Absyn_AggrKind f1;struct _tuple1*f2;struct Cyc_Core_Opt*f3;};struct _union_AggrInfo_UnknownAggr{int tag;struct _tuple3 val;};struct _union_AggrInfo_KnownAggr{int tag;struct Cyc_Absyn_Aggrdecl**val;};union Cyc_Absyn_AggrInfo{struct _union_AggrInfo_UnknownAggr UnknownAggr;struct _union_AggrInfo_KnownAggr KnownAggr;};
# 331
union Cyc_Absyn_AggrInfo Cyc_Absyn_UnknownAggr(enum Cyc_Absyn_AggrKind,struct _tuple1*,struct Cyc_Core_Opt*);struct Cyc_Absyn_ArrayInfo{void*elt_type;struct Cyc_Absyn_Tqual tq;struct Cyc_Absyn_Exp*num_elts;void*zero_term;unsigned zt_loc;};struct Cyc_Absyn_Aggr_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Aggrdecl*f1;};struct Cyc_Absyn_Enum_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;};struct Cyc_Absyn_Datatype_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;};struct Cyc_Absyn_TypeDecl{void*r;unsigned loc;};struct Cyc_Absyn_VoidCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_IntCon_Absyn_TyCon_struct{int tag;enum Cyc_Absyn_Sign f1;enum Cyc_Absyn_Size_of f2;};struct Cyc_Absyn_FloatCon_Absyn_TyCon_struct{int tag;int f1;};struct Cyc_Absyn_RgnHandleCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_TagCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_HeapCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_UniqueCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_RefCntCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_AccessCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_JoinCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_RgnsCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_TrueCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_FalseCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_ThinCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_FatCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_EnumCon_Absyn_TyCon_struct{int tag;struct _tuple1*f1;struct Cyc_Absyn_Enumdecl*f2;};struct Cyc_Absyn_AnonEnumCon_Absyn_TyCon_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_BuiltinCon_Absyn_TyCon_struct{int tag;struct _fat_ptr f1;struct Cyc_Absyn_Kind*f2;};struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_DatatypeInfo f1;};struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_DatatypeFieldInfo f1;};struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_AggrInfo f1;};struct Cyc_Absyn_AppType_Absyn_Type_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Evar_Absyn_Type_struct{int tag;struct Cyc_Core_Opt*f1;void*f2;int f3;struct Cyc_Core_Opt*f4;};struct Cyc_Absyn_VarType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Tvar*f1;};struct Cyc_Absyn_PointerType_Absyn_Type_struct{int tag;struct Cyc_Absyn_PtrInfo f1;};struct Cyc_Absyn_ArrayType_Absyn_Type_struct{int tag;struct Cyc_Absyn_ArrayInfo f1;};struct Cyc_Absyn_FnType_Absyn_Type_struct{int tag;struct Cyc_Absyn_FnInfo f1;};struct Cyc_Absyn_TupleType_Absyn_Type_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct{int tag;enum Cyc_Absyn_AggrKind f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_TypedefType_Absyn_Type_struct{int tag;struct _tuple1*f1;struct Cyc_List_List*f2;struct Cyc_Absyn_Typedefdecl*f3;void*f4;};struct Cyc_Absyn_ValueofType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct{int tag;struct Cyc_Absyn_TypeDecl*f1;void**f2;};struct Cyc_Absyn_TypeofType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_NoTypes_Absyn_Funcparams_struct{int tag;struct Cyc_List_List*f1;unsigned f2;};struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct{int tag;struct Cyc_List_List*f1;int f2;struct Cyc_Absyn_VarargInfo*f3;void*f4;struct Cyc_List_List*f5;struct Cyc_Absyn_Exp*f6;struct Cyc_Absyn_Exp*f7;struct Cyc_List_List*f8;struct Cyc_List_List*f9;};
# 437 "absyn.h"
enum Cyc_Absyn_Format_Type{Cyc_Absyn_Printf_ft =0U,Cyc_Absyn_Scanf_ft =1U};struct Cyc_Absyn_Regparm_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Stdcall_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Cdecl_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Fastcall_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Noreturn_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Const_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Aligned_att_Absyn_Attribute_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Packed_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Section_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Nocommon_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Shared_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Unused_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Weak_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Dllimport_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Dllexport_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_instrument_function_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Constructor_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Destructor_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_check_memory_usage_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Format_att_Absyn_Attribute_struct{int tag;enum Cyc_Absyn_Format_Type f1;int f2;int f3;};struct Cyc_Absyn_Initializes_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Noliveunique_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Consume_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Pure_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Mode_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Alias_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Always_inline_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_throw_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Non_null_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Deprecated_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Carray_mod_Absyn_Type_modifier_struct{int tag;void*f1;unsigned f2;};struct Cyc_Absyn_ConstArray_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_Absyn_Exp*f1;void*f2;unsigned f3;};struct Cyc_Absyn_Pointer_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_Absyn_PtrAtts f1;struct Cyc_Absyn_Tqual f2;};struct Cyc_Absyn_Function_mod_Absyn_Type_modifier_struct{int tag;void*f1;};struct Cyc_Absyn_TypeParams_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_List_List*f1;unsigned f2;int f3;};struct Cyc_Absyn_Attributes_mod_Absyn_Type_modifier_struct{int tag;unsigned f1;struct Cyc_List_List*f2;};struct _union_Cnst_Null_c{int tag;int val;};struct _tuple4{enum Cyc_Absyn_Sign f1;char f2;};struct _union_Cnst_Char_c{int tag;struct _tuple4 val;};struct _union_Cnst_Wchar_c{int tag;struct _fat_ptr val;};struct _tuple5{enum Cyc_Absyn_Sign f1;short f2;};struct _union_Cnst_Short_c{int tag;struct _tuple5 val;};struct _tuple6{enum Cyc_Absyn_Sign f1;int f2;};struct _union_Cnst_Int_c{int tag;struct _tuple6 val;};struct _tuple7{enum Cyc_Absyn_Sign f1;long long f2;};struct _union_Cnst_LongLong_c{int tag;struct _tuple7 val;};struct _tuple8{struct _fat_ptr f1;int f2;};struct _union_Cnst_Float_c{int tag;struct _tuple8 val;};struct _union_Cnst_String_c{int tag;struct _fat_ptr val;};struct _union_Cnst_Wstring_c{int tag;struct _fat_ptr val;};union Cyc_Absyn_Cnst{struct _union_Cnst_Null_c Null_c;struct _union_Cnst_Char_c Char_c;struct _union_Cnst_Wchar_c Wchar_c;struct _union_Cnst_Short_c Short_c;struct _union_Cnst_Int_c Int_c;struct _union_Cnst_LongLong_c LongLong_c;struct _union_Cnst_Float_c Float_c;struct _union_Cnst_String_c String_c;struct _union_Cnst_Wstring_c Wstring_c;};
# 532
enum Cyc_Absyn_Primop{Cyc_Absyn_Plus =0U,Cyc_Absyn_Times =1U,Cyc_Absyn_Minus =2U,Cyc_Absyn_Div =3U,Cyc_Absyn_Mod =4U,Cyc_Absyn_Eq =5U,Cyc_Absyn_Neq =6U,Cyc_Absyn_Gt =7U,Cyc_Absyn_Lt =8U,Cyc_Absyn_Gte =9U,Cyc_Absyn_Lte =10U,Cyc_Absyn_Not =11U,Cyc_Absyn_Bitnot =12U,Cyc_Absyn_Bitand =13U,Cyc_Absyn_Bitor =14U,Cyc_Absyn_Bitxor =15U,Cyc_Absyn_Bitlshift =16U,Cyc_Absyn_Bitlrshift =17U,Cyc_Absyn_Numelts =18U};
# 539
enum Cyc_Absyn_Incrementor{Cyc_Absyn_PreInc =0U,Cyc_Absyn_PostInc =1U,Cyc_Absyn_PreDec =2U,Cyc_Absyn_PostDec =3U};struct Cyc_Absyn_VarargCallInfo{int num_varargs;struct Cyc_List_List*injectors;struct Cyc_Absyn_VarargInfo*vai;};struct Cyc_Absyn_StructField_Absyn_OffsetofField_struct{int tag;struct _fat_ptr*f1;};struct Cyc_Absyn_TupleIndex_Absyn_OffsetofField_struct{int tag;unsigned f1;};
# 557
enum Cyc_Absyn_Coercion{Cyc_Absyn_Unknown_coercion =0U,Cyc_Absyn_No_coercion =1U,Cyc_Absyn_Null_to_NonNull =2U,Cyc_Absyn_Other_coercion =3U};struct Cyc_Absyn_MallocInfo{int is_calloc;struct Cyc_Absyn_Exp*rgn;void**elt_type;struct Cyc_Absyn_Exp*num_elts;int fat_result;int inline_call;};struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct{int tag;union Cyc_Absyn_Cnst f1;};struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Pragma_e_Absyn_Raw_exp_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct{int tag;enum Cyc_Absyn_Primop f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_AssignOp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Core_Opt*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_Increment_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;enum Cyc_Absyn_Incrementor f2;};struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;struct Cyc_Absyn_VarargCallInfo*f3;int f4;};struct Cyc_Absyn_Throw_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;int f2;};struct Cyc_Absyn_NoInstantiate_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Instantiate_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_Absyn_Exp*f2;int f3;enum Cyc_Absyn_Coercion f4;};struct Cyc_Absyn_Address_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_New_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Sizeoftype_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Sizeofexp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Offsetof_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Deref_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;int f3;int f4;};struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;int f3;int f4;};struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Tuple_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;};struct _tuple9{struct _fat_ptr*f1;struct Cyc_Absyn_Tqual f2;void*f3;};struct Cyc_Absyn_CompoundLit_e_Absyn_Raw_exp_struct{int tag;struct _tuple9*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Array_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Exp*f2;struct Cyc_Absyn_Exp*f3;int f4;};struct Cyc_Absyn_ComprehensionNoinit_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;void*f2;int f3;};struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct{int tag;struct _tuple1*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;struct Cyc_Absyn_Aggrdecl*f4;};struct Cyc_Absyn_AnonStruct_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Datatype_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;struct Cyc_Absyn_Datatypedecl*f2;struct Cyc_Absyn_Datatypefield*f3;};struct Cyc_Absyn_Enum_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_AnonEnum_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_Spawn_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_MallocInfo f1;};struct Cyc_Absyn_Swap_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_UnresolvedMem_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Core_Opt*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_StmtExp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Stmt*f1;};struct Cyc_Absyn_Tagcheck_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;};struct Cyc_Absyn_Valueof_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Asm_e_Absyn_Raw_exp_struct{int tag;int f1;struct _fat_ptr f2;struct Cyc_List_List*f3;struct Cyc_List_List*f4;struct Cyc_List_List*f5;};struct Cyc_Absyn_Extension_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Exp{void*topt;void*r;unsigned loc;void*annot;};struct Cyc_Absyn_Skip_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Exp_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Seq_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Return_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_IfThenElse_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Stmt*f2;struct Cyc_Absyn_Stmt*f3;};struct _tuple10{struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_While_s_Absyn_Raw_stmt_struct{int tag;struct _tuple10 f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Break_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Continue_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Goto_s_Absyn_Raw_stmt_struct{int tag;struct _fat_ptr*f1;};struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _tuple10 f2;struct _tuple10 f3;struct Cyc_Absyn_Stmt*f4;};struct Cyc_Absyn_Switch_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;void*f3;};struct Cyc_Absyn_Fallthru_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_List_List*f1;struct Cyc_Absyn_Switch_clause**f2;};struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Decl*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Label_s_Absyn_Raw_stmt_struct{int tag;struct _fat_ptr*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Do_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct _tuple10 f2;};struct Cyc_Absyn_TryCatch_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct Cyc_List_List*f2;void*f3;};struct Cyc_Absyn_Stmt{void*r;unsigned loc;void*annot;};struct Cyc_Absyn_Wild_p_Absyn_Raw_pat_struct{int tag;};struct Cyc_Absyn_Var_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Pat*f2;};struct Cyc_Absyn_AliasVar_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;};struct Cyc_Absyn_Reference_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Pat*f2;};struct Cyc_Absyn_TagInt_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;};struct Cyc_Absyn_Tuple_p_Absyn_Raw_pat_struct{int tag;struct Cyc_List_List*f1;int f2;};struct Cyc_Absyn_Pointer_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Pat*f1;};struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct{int tag;union Cyc_Absyn_AggrInfo*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;int f4;};struct Cyc_Absyn_Datatype_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;struct Cyc_Absyn_Datatypefield*f2;struct Cyc_List_List*f3;int f4;};struct Cyc_Absyn_Null_p_Absyn_Raw_pat_struct{int tag;};struct Cyc_Absyn_Int_p_Absyn_Raw_pat_struct{int tag;enum Cyc_Absyn_Sign f1;int f2;};struct Cyc_Absyn_Char_p_Absyn_Raw_pat_struct{int tag;char f1;};struct Cyc_Absyn_Float_p_Absyn_Raw_pat_struct{int tag;struct _fat_ptr f1;int f2;};struct Cyc_Absyn_Enum_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_AnonEnum_p_Absyn_Raw_pat_struct{int tag;void*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_UnknownId_p_Absyn_Raw_pat_struct{int tag;struct _tuple1*f1;};struct Cyc_Absyn_UnknownCall_p_Absyn_Raw_pat_struct{int tag;struct _tuple1*f1;struct Cyc_List_List*f2;int f3;};struct Cyc_Absyn_Exp_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Pat{void*r;void*topt;unsigned loc;};struct Cyc_Absyn_Switch_clause{struct Cyc_Absyn_Pat*pattern;struct Cyc_Core_Opt*pat_vars;struct Cyc_Absyn_Exp*where_clause;struct Cyc_Absyn_Stmt*body;unsigned loc;};struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct{int tag;struct _tuple1*f1;};struct Cyc_Absyn_Global_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Funname_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Fndecl*f1;};struct Cyc_Absyn_Param_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Local_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Pat_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Vardecl{enum Cyc_Absyn_Scope sc;struct _tuple1*name;unsigned varloc;struct Cyc_Absyn_Tqual tq;void*type;struct Cyc_Absyn_Exp*initializer;void*rgn;struct Cyc_List_List*attributes;int escapes;};struct Cyc_Absyn_Fndecl{enum Cyc_Absyn_Scope sc;int is_inline;struct _tuple1*name;struct Cyc_Absyn_Stmt*body;struct Cyc_Absyn_FnInfo i;void*cached_type;struct Cyc_Core_Opt*param_vardecls;struct Cyc_Absyn_Vardecl*fn_vardecl;};struct Cyc_Absyn_Aggrfield{struct _fat_ptr*name;struct Cyc_Absyn_Tqual tq;void*type;struct Cyc_Absyn_Exp*width;struct Cyc_List_List*attributes;struct Cyc_Absyn_Exp*requires_clause;};struct Cyc_Absyn_AggrdeclImpl{struct Cyc_List_List*exist_vars;struct Cyc_List_List*rgn_po;struct Cyc_List_List*fields;int tagged;};struct Cyc_Absyn_Aggrdecl{enum Cyc_Absyn_AggrKind kind;enum Cyc_Absyn_Scope sc;struct _tuple1*name;struct Cyc_List_List*tvs;struct Cyc_Absyn_AggrdeclImpl*impl;struct Cyc_List_List*attributes;int expected_mem_kind;};struct Cyc_Absyn_Datatypefield{struct _tuple1*name;struct Cyc_List_List*typs;unsigned loc;enum Cyc_Absyn_Scope sc;};struct Cyc_Absyn_Datatypedecl{enum Cyc_Absyn_Scope sc;struct _tuple1*name;struct Cyc_List_List*tvs;struct Cyc_Core_Opt*fields;int is_extensible;};struct Cyc_Absyn_Enumfield{struct _tuple1*name;struct Cyc_Absyn_Exp*tag;unsigned loc;};struct Cyc_Absyn_Enumdecl{enum Cyc_Absyn_Scope sc;struct _tuple1*name;struct Cyc_Core_Opt*fields;};struct Cyc_Absyn_Typedefdecl{struct _tuple1*name;struct Cyc_Absyn_Tqual tq;struct Cyc_List_List*tvs;struct Cyc_Core_Opt*kind;void*defn;struct Cyc_List_List*atts;int extern_c;};struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Fn_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Fndecl*f1;};struct Cyc_Absyn_Let_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Pat*f1;struct Cyc_Core_Opt*f2;struct Cyc_Absyn_Exp*f3;void*f4;};struct Cyc_Absyn_Letv_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_Region_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_Aggr_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Aggrdecl*f1;};struct Cyc_Absyn_Datatype_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;};struct Cyc_Absyn_Enum_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;};struct Cyc_Absyn_Typedef_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Typedefdecl*f1;};struct Cyc_Absyn_Namespace_d_Absyn_Raw_decl_struct{int tag;struct _fat_ptr*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Using_d_Absyn_Raw_decl_struct{int tag;struct _tuple1*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_ExternC_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;};struct _tuple11{unsigned f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_ExternCinclude_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;struct _tuple11*f4;};struct Cyc_Absyn_Porton_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Portoff_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Tempeston_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Tempestoff_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Decl{void*r;unsigned loc;};struct Cyc_Absyn_ArrayElement_Absyn_Designator_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_FieldName_Absyn_Designator_struct{int tag;struct _fat_ptr*f1;};extern char Cyc_Absyn_EmptyAnnot[11U];struct Cyc_Absyn_EmptyAnnot_Absyn_AbsynAnnot_struct{char*tag;};
# 908 "absyn.h"
int Cyc_Absyn_qvar_cmp(struct _tuple1*,struct _tuple1*);
# 910
int Cyc_Absyn_tvar_cmp(struct Cyc_Absyn_Tvar*,struct Cyc_Absyn_Tvar*);
# 917
struct Cyc_Absyn_Tqual Cyc_Absyn_empty_tqual(unsigned);
# 922
void*Cyc_Absyn_compress_kb(void*);
# 928
int Cyc_Absyn_type2bool(int def,void*);
# 933
void*Cyc_Absyn_new_evar(struct Cyc_Core_Opt*k,struct Cyc_Core_Opt*tenv);
# 938
extern void*Cyc_Absyn_uint_type;extern void*Cyc_Absyn_ulong_type;extern void*Cyc_Absyn_ulonglong_type;
# 940
extern void*Cyc_Absyn_sint_type;extern void*Cyc_Absyn_slong_type;extern void*Cyc_Absyn_slonglong_type;
# 945
extern void*Cyc_Absyn_heap_rgn_type;extern void*Cyc_Absyn_unique_rgn_type;extern void*Cyc_Absyn_refcnt_rgn_type;
# 947
extern void*Cyc_Absyn_empty_effect;
# 949
extern void*Cyc_Absyn_true_type;extern void*Cyc_Absyn_false_type;
# 951
extern void*Cyc_Absyn_void_type;void*Cyc_Absyn_var_type(struct Cyc_Absyn_Tvar*);void*Cyc_Absyn_access_eff(void*);void*Cyc_Absyn_join_eff(struct Cyc_List_List*);void*Cyc_Absyn_regionsof_eff(void*);void*Cyc_Absyn_enum_type(struct _tuple1*n,struct Cyc_Absyn_Enumdecl*d);
# 976
extern void*Cyc_Absyn_fat_bound_type;
# 978
void*Cyc_Absyn_thin_bounds_exp(struct Cyc_Absyn_Exp*);
# 980
void*Cyc_Absyn_bounds_one();
# 982
void*Cyc_Absyn_pointer_type(struct Cyc_Absyn_PtrInfo);
# 987
void*Cyc_Absyn_atb_type(void*t,void*rgn,struct Cyc_Absyn_Tqual tq,void*b,void*zero_term);
# 1007
void*Cyc_Absyn_datatype_type(union Cyc_Absyn_DatatypeInfo,struct Cyc_List_List*args);
# 1009
void*Cyc_Absyn_aggr_type(union Cyc_Absyn_AggrInfo,struct Cyc_List_List*args);
# 1012
struct Cyc_Absyn_Exp*Cyc_Absyn_new_exp(void*,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_New_exp(struct Cyc_Absyn_Exp*rgn_handle,struct Cyc_Absyn_Exp*,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_copy_exp(struct Cyc_Absyn_Exp*);
struct Cyc_Absyn_Exp*Cyc_Absyn_const_exp(union Cyc_Absyn_Cnst,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_null_exp(unsigned);
# 1022
struct Cyc_Absyn_Exp*Cyc_Absyn_uint_exp(unsigned,unsigned);
# 1029
struct Cyc_Absyn_Exp*Cyc_Absyn_varb_exp(void*,unsigned);
# 1031
struct Cyc_Absyn_Exp*Cyc_Absyn_pragma_exp(struct _fat_ptr,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_primop_exp(enum Cyc_Absyn_Primop,struct Cyc_List_List*,unsigned);
# 1035
struct Cyc_Absyn_Exp*Cyc_Absyn_swap_exp(struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*,unsigned);
# 1045
struct Cyc_Absyn_Exp*Cyc_Absyn_assignop_exp(struct Cyc_Absyn_Exp*,struct Cyc_Core_Opt*,struct Cyc_Absyn_Exp*,unsigned);
# 1047
struct Cyc_Absyn_Exp*Cyc_Absyn_increment_exp(struct Cyc_Absyn_Exp*,enum Cyc_Absyn_Incrementor,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_conditional_exp(struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_and_exp(struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_or_exp(struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_seq_exp(struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*,unsigned);
# 1054
struct Cyc_Absyn_Exp*Cyc_Absyn_throw_exp(struct Cyc_Absyn_Exp*,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_rethrow_exp(struct Cyc_Absyn_Exp*,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_noinstantiate_exp(struct Cyc_Absyn_Exp*,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_instantiate_exp(struct Cyc_Absyn_Exp*,struct Cyc_List_List*,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_cast_exp(void*,struct Cyc_Absyn_Exp*,int user_cast,enum Cyc_Absyn_Coercion,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_address_exp(struct Cyc_Absyn_Exp*,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_sizeoftype_exp(void*t,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_sizeofexp_exp(struct Cyc_Absyn_Exp*e,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_offsetof_exp(void*,struct Cyc_List_List*,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_deref_exp(struct Cyc_Absyn_Exp*,unsigned);
# 1066
struct Cyc_Absyn_Exp*Cyc_Absyn_subscript_exp(struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_tuple_exp(struct Cyc_List_List*,unsigned);
# 1071
struct Cyc_Absyn_Exp*Cyc_Absyn_valueof_exp(void*,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_asm_exp(int,struct _fat_ptr,struct Cyc_List_List*,struct Cyc_List_List*,struct Cyc_List_List*,unsigned);
# 1074
struct Cyc_Absyn_Exp*Cyc_Absyn_extension_exp(struct Cyc_Absyn_Exp*,unsigned);
# 1111
struct Cyc_Absyn_Decl*Cyc_Absyn_alias_decl(struct Cyc_Absyn_Tvar*,struct Cyc_Absyn_Vardecl*,struct Cyc_Absyn_Exp*,unsigned);
struct Cyc_Absyn_Vardecl*Cyc_Absyn_new_vardecl(unsigned varloc,struct _tuple1*,void*,struct Cyc_Absyn_Exp*init);
# 1154
struct Cyc_Absyn_Aggrfield*Cyc_Absyn_lookup_field(struct Cyc_List_List*,struct _fat_ptr*);
# 1156
struct Cyc_Absyn_Aggrfield*Cyc_Absyn_lookup_decl_field(struct Cyc_Absyn_Aggrdecl*,struct _fat_ptr*);struct _tuple12{struct Cyc_Absyn_Tqual f1;void*f2;};
# 1158
struct _tuple12*Cyc_Absyn_lookup_tuple_field(struct Cyc_List_List*,int);
# 1166
int Cyc_Absyn_fntype_att(void*);
# 1168
int Cyc_Absyn_equal_att(void*,void*);
# 1170
int Cyc_Absyn_attribute_cmp(void*,void*);
# 1176
struct Cyc_Absyn_Aggrdecl*Cyc_Absyn_get_known_aggrdecl(union Cyc_Absyn_AggrInfo);
# 1202
struct Cyc_Absyn_Exp*Cyc_Absyn_spawn_exp(struct Cyc_Absyn_Exp*e1,struct Cyc_Absyn_Exp*e2,unsigned loc);struct Cyc_Absynpp_Params{int expand_typedefs;int qvar_to_Cids;int add_cyc_prefix;int to_VC;int decls_first;int rewrite_temp_tvars;int print_all_tvars;int print_all_kinds;int print_all_effects;int print_using_stmts;int print_externC_stmts;int print_full_evars;int print_zeroterm;int generate_line_directives;int use_curr_namespace;struct Cyc_List_List*curr_namespace;};
# 62 "absynpp.h"
struct _fat_ptr Cyc_Absynpp_typ2string(void*);
# 64
struct _fat_ptr Cyc_Absynpp_kind2string(struct Cyc_Absyn_Kind*);
# 67
struct _fat_ptr Cyc_Absynpp_exp2string(struct Cyc_Absyn_Exp*);
# 74
struct _fat_ptr Cyc_Absynpp_tvar2string(struct Cyc_Absyn_Tvar*);
# 27 "warn.h"
void Cyc_Warn_vwarn(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap);
# 33
void Cyc_Warn_verr(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap);
# 38
void*Cyc_Warn_vimpos(struct _fat_ptr fmt,struct _fat_ptr ap);struct Cyc_Warn_String_Warn_Warg_struct{int tag;struct _fat_ptr f1;};struct Cyc_Warn_Exp_Warn_Warg_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Warn_Typ_Warn_Warg_struct{int tag;void*f1;};struct Cyc_Warn_Qvar_Warn_Warg_struct{int tag;struct _tuple1*f1;};struct _tuple13{unsigned f1;int f2;};
# 28 "evexp.h"
struct _tuple13 Cyc_Evexp_eval_const_uint_exp(struct Cyc_Absyn_Exp*e);
# 41 "evexp.h"
int Cyc_Evexp_same_const_exp(struct Cyc_Absyn_Exp*e1,struct Cyc_Absyn_Exp*e2);
int Cyc_Evexp_lte_const_exp(struct Cyc_Absyn_Exp*e1,struct Cyc_Absyn_Exp*e2);
# 45
int Cyc_Evexp_const_exp_cmp(struct Cyc_Absyn_Exp*e1,struct Cyc_Absyn_Exp*e2);
# 28 "unify.h"
int Cyc_Unify_unify_kindbound(void*,void*);
int Cyc_Unify_unify(void*,void*);
# 32
void Cyc_Unify_occurs(void*evar,struct _RegionHandle*r,struct Cyc_List_List*env,void*t);struct _union_RelnOp_RConst{int tag;unsigned val;};struct _union_RelnOp_RVar{int tag;struct Cyc_Absyn_Vardecl*val;};struct _union_RelnOp_RNumelts{int tag;struct Cyc_Absyn_Vardecl*val;};struct _union_RelnOp_RType{int tag;void*val;};struct _union_RelnOp_RParam{int tag;unsigned val;};struct _union_RelnOp_RParamNumelts{int tag;unsigned val;};struct _union_RelnOp_RReturn{int tag;unsigned val;};union Cyc_Relations_RelnOp{struct _union_RelnOp_RConst RConst;struct _union_RelnOp_RVar RVar;struct _union_RelnOp_RNumelts RNumelts;struct _union_RelnOp_RType RType;struct _union_RelnOp_RParam RParam;struct _union_RelnOp_RParamNumelts RParamNumelts;struct _union_RelnOp_RReturn RReturn;};
# 41 "relations-ap.h"
union Cyc_Relations_RelnOp Cyc_Relations_RParam(unsigned);union Cyc_Relations_RelnOp Cyc_Relations_RParamNumelts(unsigned);union Cyc_Relations_RelnOp Cyc_Relations_RReturn();
# 50
enum Cyc_Relations_Relation{Cyc_Relations_Req =0U,Cyc_Relations_Rneq =1U,Cyc_Relations_Rlte =2U,Cyc_Relations_Rlt =3U};struct Cyc_Relations_Reln{union Cyc_Relations_RelnOp rop1;enum Cyc_Relations_Relation relation;union Cyc_Relations_RelnOp rop2;};
# 84
struct Cyc_List_List*Cyc_Relations_exp2relns(struct _RegionHandle*r,struct Cyc_Absyn_Exp*e);
# 110
struct Cyc_List_List*Cyc_Relations_copy_relns(struct _RegionHandle*,struct Cyc_List_List*);
# 131
int Cyc_Relations_check_logical_implication(struct Cyc_List_List*r1,struct Cyc_List_List*r2);struct Cyc_Iter_Iter{void*env;int(*next)(void*env,void*dest);};struct Cyc_Dict_T;struct Cyc_Dict_Dict{int(*rel)(void*,void*);struct _RegionHandle*r;const struct Cyc_Dict_T*t;};extern char Cyc_Dict_Present[8U];struct Cyc_Dict_Present_exn_struct{char*tag;};extern char Cyc_Dict_Absent[7U];struct Cyc_Dict_Absent_exn_struct{char*tag;};struct Cyc_RgnOrder_RgnPO;extern char Cyc_Tcenv_Env_error[10U];struct Cyc_Tcenv_Env_error_exn_struct{char*tag;};struct Cyc_Tcenv_Genv{struct Cyc_Dict_Dict aggrdecls;struct Cyc_Dict_Dict datatypedecls;struct Cyc_Dict_Dict enumdecls;struct Cyc_Dict_Dict typedefs;struct Cyc_Dict_Dict ordinaries;};struct Cyc_Tcenv_Fenv;struct Cyc_Tcenv_Tenv{struct Cyc_List_List*ns;struct Cyc_Tcenv_Genv*ae;struct Cyc_Tcenv_Fenv*le;int allow_valueof: 1;int in_extern_c_include: 1;int in_tempest: 1;int tempest_generalize: 1;};
# 88 "tcenv.h"
enum Cyc_Tcenv_NewStatus{Cyc_Tcenv_NoneNew =0U,Cyc_Tcenv_InNew =1U,Cyc_Tcenv_InNewAggr =2U};
# 151
int Cyc_Tcenv_rgn_outlives_rgn(struct Cyc_Tcenv_Tenv*te,void*rgn1,void*rgn2);
# 30 "tcutil.h"
void*Cyc_Tcutil_impos(struct _fat_ptr,struct _fat_ptr);
void Cyc_Tcutil_terr(unsigned,struct _fat_ptr,struct _fat_ptr);
void Cyc_Tcutil_warn(unsigned,struct _fat_ptr,struct _fat_ptr);
# 37
int Cyc_Tcutil_is_char_type(void*);
int Cyc_Tcutil_is_any_int_type(void*);
int Cyc_Tcutil_is_any_float_type(void*);
int Cyc_Tcutil_is_integral_type(void*);
int Cyc_Tcutil_is_arithmetic_type(void*);
# 44
int Cyc_Tcutil_is_pointer_type(void*);
# 57
int Cyc_Tcutil_is_bits_only_type(void*);
# 59
int Cyc_Tcutil_is_noreturn_fn_type(void*);
# 72
struct Cyc_Absyn_Exp*Cyc_Tcutil_get_bounds_exp(void*def,void*b);
# 77
int Cyc_Tcutil_is_fat_pointer_type_elt(void*t,void**elt_dest);
# 81
int Cyc_Tcutil_is_zero_ptr_type(void*t,void**ptr_type,int*is_fat,void**elt_type);
# 84
struct Cyc_Absyn_Exp*Cyc_Tcutil_get_bounds_exp(void*def,void*b);
# 87
int Cyc_Tcutil_is_integral(struct Cyc_Absyn_Exp*);
# 89
int Cyc_Tcutil_is_zero(struct Cyc_Absyn_Exp*);
# 94
void*Cyc_Tcutil_copy_type(void*);
# 97
struct Cyc_Absyn_Exp*Cyc_Tcutil_deep_copy_exp(int preserve_types,struct Cyc_Absyn_Exp*);
# 100
int Cyc_Tcutil_kind_leq(struct Cyc_Absyn_Kind*,struct Cyc_Absyn_Kind*);
# 105
struct Cyc_Absyn_Kind*Cyc_Tcutil_tvar_kind(struct Cyc_Absyn_Tvar*,struct Cyc_Absyn_Kind*def);
struct Cyc_Absyn_Kind*Cyc_Tcutil_type_kind(void*);
void*Cyc_Tcutil_compress(void*);
void Cyc_Tcutil_unchecked_cast(struct Cyc_Absyn_Exp*,void*,enum Cyc_Absyn_Coercion);
# 110
int Cyc_Tcutil_coerce_sint_type(struct Cyc_Absyn_Exp*);
# 113
int Cyc_Tcutil_coerce_arg(struct Cyc_Tcenv_Tenv*te,struct Cyc_Absyn_Exp*,void*,int*alias_coercion);
int Cyc_Tcutil_coerce_assign(struct Cyc_Tcenv_Tenv*te,struct Cyc_Absyn_Exp*,void*);
# 117
int Cyc_Tcutil_silent_castable(struct Cyc_Tcenv_Tenv*te,unsigned,void*,void*);
# 119
enum Cyc_Absyn_Coercion Cyc_Tcutil_castable(struct Cyc_Tcenv_Tenv*te,unsigned,void*,void*);
# 121
int Cyc_Tcutil_subtype(struct Cyc_Tcenv_Tenv*te,struct Cyc_List_List*assume,void*t1,void*t2);
# 124
int Cyc_Tcutil_zero_to_null(void*,struct Cyc_Absyn_Exp*);
# 129
extern int Cyc_Tcutil_warn_alias_coerce;
# 132
extern int Cyc_Tcutil_warn_region_coerce;
# 135
extern struct Cyc_Absyn_Kind Cyc_Tcutil_rk;
extern struct Cyc_Absyn_Kind Cyc_Tcutil_ak;
extern struct Cyc_Absyn_Kind Cyc_Tcutil_bk;
extern struct Cyc_Absyn_Kind Cyc_Tcutil_mk;
extern struct Cyc_Absyn_Kind Cyc_Tcutil_ek;
extern struct Cyc_Absyn_Kind Cyc_Tcutil_ik;
extern struct Cyc_Absyn_Kind Cyc_Tcutil_boolk;
extern struct Cyc_Absyn_Kind Cyc_Tcutil_ptrbk;
# 144
extern struct Cyc_Absyn_Kind Cyc_Tcutil_trk;
extern struct Cyc_Absyn_Kind Cyc_Tcutil_tak;
extern struct Cyc_Absyn_Kind Cyc_Tcutil_tbk;
extern struct Cyc_Absyn_Kind Cyc_Tcutil_tmk;
# 149
extern struct Cyc_Absyn_Kind Cyc_Tcutil_urk;
extern struct Cyc_Absyn_Kind Cyc_Tcutil_uak;
extern struct Cyc_Absyn_Kind Cyc_Tcutil_ubk;
extern struct Cyc_Absyn_Kind Cyc_Tcutil_umk;
# 154
extern struct Cyc_Core_Opt Cyc_Tcutil_rko;
extern struct Cyc_Core_Opt Cyc_Tcutil_ako;
extern struct Cyc_Core_Opt Cyc_Tcutil_bko;
extern struct Cyc_Core_Opt Cyc_Tcutil_mko;
extern struct Cyc_Core_Opt Cyc_Tcutil_iko;
extern struct Cyc_Core_Opt Cyc_Tcutil_eko;
extern struct Cyc_Core_Opt Cyc_Tcutil_boolko;
extern struct Cyc_Core_Opt Cyc_Tcutil_ptrbko;
# 163
extern struct Cyc_Core_Opt Cyc_Tcutil_trko;
extern struct Cyc_Core_Opt Cyc_Tcutil_tako;
extern struct Cyc_Core_Opt Cyc_Tcutil_tbko;
extern struct Cyc_Core_Opt Cyc_Tcutil_tmko;
# 168
extern struct Cyc_Core_Opt Cyc_Tcutil_urko;
extern struct Cyc_Core_Opt Cyc_Tcutil_uako;
extern struct Cyc_Core_Opt Cyc_Tcutil_ubko;
extern struct Cyc_Core_Opt Cyc_Tcutil_umko;
# 173
struct Cyc_Core_Opt*Cyc_Tcutil_kind_to_opt(struct Cyc_Absyn_Kind*k);
void*Cyc_Tcutil_kind_to_bound(struct Cyc_Absyn_Kind*k);
# 182
int Cyc_Tcutil_typecmp(void*,void*);
int Cyc_Tcutil_aggrfield_cmp(struct Cyc_Absyn_Aggrfield*,struct Cyc_Absyn_Aggrfield*);
# 185
void*Cyc_Tcutil_substitute(struct Cyc_List_List*,void*);
# 187
void*Cyc_Tcutil_rsubstitute(struct _RegionHandle*,struct Cyc_List_List*,void*);
struct Cyc_List_List*Cyc_Tcutil_rsubst_rgnpo(struct _RegionHandle*,struct Cyc_List_List*,struct Cyc_List_List*);
# 193
struct Cyc_Absyn_Exp*Cyc_Tcutil_rsubsexp(struct _RegionHandle*,struct Cyc_List_List*,struct Cyc_Absyn_Exp*);
# 196
int Cyc_Tcutil_subset_effect(int may_constrain_evars,void*e1,void*e2);
# 200
int Cyc_Tcutil_region_in_effect(int constrain,void*r,void*e);
# 220
void Cyc_Tcutil_check_bound(unsigned,unsigned i,void*,int do_warn);
# 233
int Cyc_Tcutil_is_noalias_region(void*r,int must_be_unique);
# 236
int Cyc_Tcutil_is_noalias_pointer(void*t,int must_be_unique);
# 241
int Cyc_Tcutil_is_noalias_path(struct Cyc_Absyn_Exp*);
# 246
int Cyc_Tcutil_is_noalias_pointer_or_aggr(void*);struct _tuple14{int f1;void*f2;};
# 250
struct _tuple14 Cyc_Tcutil_addressof_props(struct Cyc_Absyn_Exp*);
# 253
void*Cyc_Tcutil_normalize_effect(void*e);
# 256
struct Cyc_Absyn_Tvar*Cyc_Tcutil_new_tvar(void*);
# 258
int Cyc_Tcutil_new_tvar_id();
# 260
void Cyc_Tcutil_add_tvar_identity(struct Cyc_Absyn_Tvar*);
# 273
int Cyc_Tcutil_is_const_exp(struct Cyc_Absyn_Exp*);
# 280
int Cyc_Tcutil_extract_const_from_typedef(unsigned,int declared_const,void*);
# 298
void*Cyc_Tcutil_promote_array(void*t,void*rgn,int convert_tag);
# 301
int Cyc_Tcutil_zeroable_type(void*);
# 305
int Cyc_Tcutil_force_type2bool(int desired,void*);
# 313
int Cyc_Tcutil_admits_zero(void*);
void Cyc_Tcutil_replace_rops(struct Cyc_List_List*,struct Cyc_Relations_Reln*);
# 317
int Cyc_Tcutil_fast_tvar_cmp(struct Cyc_Absyn_Tvar*,struct Cyc_Absyn_Tvar*);
# 322
int Cyc_Tcutil_tycon_cmp(void*,void*);
int Cyc_Tcutil_star_cmp(int(*cmp)(void*,void*),void*,void*);
# 38 "tcutil.cyc"
int Cyc_Tcutil_is_void_type(void*t){
void*_stmttmp0=Cyc_Tcutil_compress(t);void*_tmp0=_stmttmp0;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp0)->tag == 0U){if(((struct Cyc_Absyn_VoidCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp0)->f1)->tag == 0U){_LL1: _LL2:
 return 1;}else{goto _LL3;}}else{_LL3: _LL4:
 return 0;}_LL0:;}
# 38
int Cyc_Tcutil_is_array_type(void*t){
# 45
void*_stmttmp1=Cyc_Tcutil_compress(t);void*_tmp1=_stmttmp1;if(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp1)->tag == 4U){_LL1: _LL2:
 return 1;}else{_LL3: _LL4:
 return 0;}_LL0:;}
# 38
int Cyc_Tcutil_is_heap_rgn_type(void*t){
# 51
void*_stmttmp2=Cyc_Tcutil_compress(t);void*_tmp2=_stmttmp2;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2)->tag == 0U){if(((struct Cyc_Absyn_HeapCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2)->f1)->tag == 5U){_LL1: _LL2:
 return 1;}else{goto _LL3;}}else{_LL3: _LL4:
 return 0;}_LL0:;}
# 38
int Cyc_Tcutil_is_pointer_type(void*t){
# 57
void*_stmttmp3=Cyc_Tcutil_compress(t);void*_tmp3=_stmttmp3;if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp3)->tag == 3U){_LL1: _LL2:
 return 1;}else{_LL3: _LL4:
 return 0;}_LL0:;}
# 38
int Cyc_Tcutil_is_char_type(void*t){
# 64
void*_stmttmp4=Cyc_Tcutil_compress(t);void*_tmp4=_stmttmp4;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp4)->tag == 0U){if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp4)->f1)->tag == 1U){if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp4)->f1)->f2 == Cyc_Absyn_Char_sz){_LL1: _LL2:
 return 1;}else{goto _LL3;}}else{goto _LL3;}}else{_LL3: _LL4:
 return 0;}_LL0:;}
# 38
int Cyc_Tcutil_is_any_int_type(void*t){
# 71
void*_stmttmp5=Cyc_Tcutil_compress(t);void*_tmp5=_stmttmp5;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp5)->tag == 0U){if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp5)->f1)->tag == 1U){_LL1: _LL2:
 return 1;}else{goto _LL3;}}else{_LL3: _LL4:
 return 0;}_LL0:;}
# 38
int Cyc_Tcutil_is_any_float_type(void*t){
# 78
void*_stmttmp6=Cyc_Tcutil_compress(t);void*_tmp6=_stmttmp6;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp6)->tag == 0U){if(((struct Cyc_Absyn_FloatCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp6)->f1)->tag == 2U){_LL1: _LL2:
 return 1;}else{goto _LL3;}}else{_LL3: _LL4:
 return 0;}_LL0:;}
# 38
int Cyc_Tcutil_is_integral_type(void*t){
# 85
void*_stmttmp7=Cyc_Tcutil_compress(t);void*_tmp7=_stmttmp7;void*_tmp8;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp7)->tag == 0U){_LL1: _tmp8=(void*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp7)->f1;_LL2: {void*c=_tmp8;
# 87
void*_tmp9=c;switch(*((int*)_tmp9)){case 1U: _LL6: _LL7:
 goto _LL9;case 4U: _LL8: _LL9:
 goto _LLB;case 15U: _LLA: _LLB:
 goto _LLD;case 16U: _LLC: _LLD:
 return 1;default: _LLE: _LLF:
 return 0;}_LL5:;}}else{_LL3: _LL4:
# 94
 return 0;}_LL0:;}
# 38
int Cyc_Tcutil_is_signed_type(void*t){
# 98
void*_stmttmp8=Cyc_Tcutil_compress(t);void*_tmpA=_stmttmp8;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmpA)->tag == 0U)switch(*((int*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmpA)->f1)){case 1U: if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmpA)->f1)->f1 == Cyc_Absyn_Signed){_LL1: _LL2:
 return 1;}else{goto _LL5;}case 2U: _LL3: _LL4:
 return 1;default: goto _LL5;}else{_LL5: _LL6:
 return 0;}_LL0:;}
# 38
int Cyc_Tcutil_is_arithmetic_type(void*t){
# 105
return Cyc_Tcutil_is_integral_type(t)|| Cyc_Tcutil_is_any_float_type(t);}
# 38
int Cyc_Tcutil_is_strict_arithmetic_type(void*t){
# 108
return Cyc_Tcutil_is_any_int_type(t)|| Cyc_Tcutil_is_any_float_type(t);}
# 38
int Cyc_Tcutil_is_function_type(void*t){
# 111
void*_stmttmp9=Cyc_Tcutil_compress(t);void*_tmpB=_stmttmp9;if(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmpB)->tag == 5U){_LL1: _LL2:
 return 1;}else{_LL3: _LL4:
 return 0;}_LL0:;}
# 38
int Cyc_Tcutil_is_boxed(void*t){
# 117
return(int)(Cyc_Tcutil_type_kind(t))->kind == (int)Cyc_Absyn_BoxKind;}
# 38
int Cyc_Tcutil_is_integral(struct Cyc_Absyn_Exp*e){
# 125
void*_stmttmpA=Cyc_Tcutil_compress((void*)_check_null(e->topt));void*_tmpC=_stmttmpA;void*_tmpD;if(((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmpC)->tag == 1U){_LL1: _LL2:
 return Cyc_Unify_unify((void*)_check_null(e->topt),Cyc_Absyn_sint_type);}else{_LL3: _tmpD=_tmpC;_LL4: {void*t=_tmpD;
return Cyc_Tcutil_is_integral_type(t);}}_LL0:;}
# 38
int Cyc_Tcutil_is_numeric(struct Cyc_Absyn_Exp*e){
# 133
if(Cyc_Tcutil_is_integral(e))
return 1;{
# 133
void*_stmttmpB=
# 135
Cyc_Tcutil_compress((void*)_check_null(e->topt));
# 133
void*_tmpE=_stmttmpB;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmpE)->tag == 0U){if(((struct Cyc_Absyn_FloatCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmpE)->f1)->tag == 2U){_LL1: _LL2:
# 136
 return 1;}else{goto _LL3;}}else{_LL3: _LL4:
 return 0;}_LL0:;}}
# 38
int Cyc_Tcutil_is_zeroterm_pointer_type(void*t){
# 143
void*_stmttmpC=Cyc_Tcutil_compress(t);void*_tmpF=_stmttmpC;void*_tmp10;if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmpF)->tag == 3U){_LL1: _tmp10=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmpF)->f1).ptr_atts).zero_term;_LL2: {void*ztl=_tmp10;
# 145
return Cyc_Tcutil_force_type2bool(0,ztl);}}else{_LL3: _LL4:
 return 0;}_LL0:;}
# 38
int Cyc_Tcutil_is_nullable_pointer_type(void*t){
# 152
void*_stmttmpD=Cyc_Tcutil_compress(t);void*_tmp11=_stmttmpD;void*_tmp12;if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp11)->tag == 3U){_LL1: _tmp12=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp11)->f1).ptr_atts).nullable;_LL2: {void*nbl=_tmp12;
# 154
return Cyc_Tcutil_force_type2bool(0,nbl);}}else{_LL3: _LL4:
 return 0;}_LL0:;}
# 38
int Cyc_Tcutil_is_fat_ptr(void*t){
# 161
void*_stmttmpE=Cyc_Tcutil_compress(t);void*_tmp13=_stmttmpE;void*_tmp14;if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp13)->tag == 3U){_LL1: _tmp14=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp13)->f1).ptr_atts).bounds;_LL2: {void*b=_tmp14;
# 163
return Cyc_Unify_unify(Cyc_Absyn_fat_bound_type,b);}}else{_LL3: _LL4:
 return 0;}_LL0:;}
# 38
int Cyc_Tcutil_is_fat_pointer_type_elt(void*t,void**elt_type_dest){
# 171
void*_stmttmpF=Cyc_Tcutil_compress(t);void*_tmp15=_stmttmpF;void*_tmp17;void*_tmp16;if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp15)->tag == 3U){_LL1: _tmp16=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp15)->f1).elt_type;_tmp17=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp15)->f1).ptr_atts).bounds;_LL2: {void*elt_type=_tmp16;void*b=_tmp17;
# 173
if(Cyc_Unify_unify(b,Cyc_Absyn_fat_bound_type)){
*elt_type_dest=elt_type;
return 1;}else{
# 177
return 0;}}}else{_LL3: _LL4:
 return 0;}_LL0:;}
# 38
int Cyc_Tcutil_is_zero_pointer_type_elt(void*t,void**elt_type_dest){
# 185
void*_stmttmp10=Cyc_Tcutil_compress(t);void*_tmp18=_stmttmp10;void*_tmp1A;void*_tmp19;if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp18)->tag == 3U){_LL1: _tmp19=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp18)->f1).elt_type;_tmp1A=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp18)->f1).ptr_atts).zero_term;_LL2: {void*elt_type=_tmp19;void*zt=_tmp1A;
# 187
*elt_type_dest=elt_type;
return Cyc_Absyn_type2bool(0,zt);}}else{_LL3: _LL4:
 return 0;}_LL0:;}
# 38
int Cyc_Tcutil_is_zero_ptr_type(void*t,void**ptr_type,int*is_fat,void**elt_type){
# 198
void*_stmttmp11=Cyc_Tcutil_compress(t);void*_tmp1B=_stmttmp11;void*_tmp1F;struct Cyc_Absyn_Exp*_tmp1E;struct Cyc_Absyn_Tqual _tmp1D;void*_tmp1C;void*_tmp22;void*_tmp21;void*_tmp20;switch(*((int*)_tmp1B)){case 3U: _LL1: _tmp20=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp1B)->f1).elt_type;_tmp21=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp1B)->f1).ptr_atts).bounds;_tmp22=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp1B)->f1).ptr_atts).zero_term;_LL2: {void*elt=_tmp20;void*bnds=_tmp21;void*zt=_tmp22;
# 200
if(Cyc_Absyn_type2bool(0,zt)){
*ptr_type=t;
*elt_type=elt;
{void*_stmttmp12=Cyc_Tcutil_compress(bnds);void*_tmp23=_stmttmp12;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp23)->tag == 0U){if(((struct Cyc_Absyn_FatCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp23)->f1)->tag == 14U){_LL8: _LL9:
*is_fat=1;goto _LL7;}else{goto _LLA;}}else{_LLA: _LLB:
*is_fat=0;goto _LL7;}_LL7:;}
# 207
return 1;}else{
return 0;}}case 4U: _LL3: _tmp1C=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp1B)->f1).elt_type;_tmp1D=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp1B)->f1).tq;_tmp1E=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp1B)->f1).num_elts;_tmp1F=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp1B)->f1).zero_term;_LL4: {void*elt=_tmp1C;struct Cyc_Absyn_Tqual tq=_tmp1D;struct Cyc_Absyn_Exp*n=_tmp1E;void*zt=_tmp1F;
# 210
if(Cyc_Absyn_type2bool(0,zt)){
*elt_type=elt;
*is_fat=0;
({void*_tmp529=Cyc_Tcutil_promote_array(t,Cyc_Absyn_heap_rgn_type,0);*ptr_type=_tmp529;});
return 1;}else{
return 0;}}default: _LL5: _LL6:
 return 0;}_LL0:;}
# 38
int Cyc_Tcutil_is_fat_pointer_type(void*t){
# 224
void*ignore=Cyc_Absyn_void_type;
return Cyc_Tcutil_is_fat_pointer_type_elt(t,& ignore);}
# 38
int Cyc_Tcutil_is_bound_one(void*b){
# 230
struct Cyc_Absyn_Exp*eopt=({void*_tmp52A=Cyc_Absyn_bounds_one();Cyc_Tcutil_get_bounds_exp(_tmp52A,b);});
if(eopt == 0)return 0;{struct Cyc_Absyn_Exp*e=eopt;
# 233
struct _tuple13 _stmttmp13=Cyc_Evexp_eval_const_uint_exp(e);int _tmp25;unsigned _tmp24;_LL1: _tmp24=_stmttmp13.f1;_tmp25=_stmttmp13.f2;_LL2: {unsigned i=_tmp24;int known=_tmp25;
return known && i == (unsigned)1;}}}
# 38
int Cyc_Tcutil_is_bits_only_type(void*t){
# 239
void*_stmttmp14=Cyc_Tcutil_compress(t);void*_tmp26=_stmttmp14;struct Cyc_List_List*_tmp27;struct Cyc_List_List*_tmp28;void*_tmp2A;void*_tmp29;struct Cyc_List_List*_tmp2C;void*_tmp2B;switch(*((int*)_tmp26)){case 0U: _LL1: _tmp2B=(void*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp26)->f1;_tmp2C=((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp26)->f2;_LL2: {void*c=_tmp2B;struct Cyc_List_List*ts=_tmp2C;
# 241
void*_tmp2D=c;struct Cyc_Absyn_Aggrdecl*_tmp2E;switch(*((int*)_tmp2D)){case 0U: _LLC: _LLD:
 goto _LLF;case 1U: _LLE: _LLF:
 goto _LL11;case 2U: _LL10: _LL11:
 return 1;case 15U: _LL12: _LL13:
 goto _LL15;case 16U: _LL14: _LL15:
 return 0;case 20U: if(((((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)_tmp2D)->f1).UnknownAggr).tag == 1){_LL16: _LL17:
 return 0;}else{_LL18: _tmp2E=*((((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)_tmp2D)->f1).KnownAggr).val;_LL19: {struct Cyc_Absyn_Aggrdecl*ad=_tmp2E;
# 249
if(ad->impl == 0)
return 0;{
# 249
int okay=1;
# 252
{struct Cyc_List_List*fs=((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->fields;for(0;fs != 0;fs=fs->tl){
if(!Cyc_Tcutil_is_bits_only_type(((struct Cyc_Absyn_Aggrfield*)fs->hd)->type)){okay=0;break;}}}
# 252
if(okay)
# 254
return 1;{
# 252
struct _RegionHandle _tmp2F=_new_region("rgn");struct _RegionHandle*rgn=& _tmp2F;_push_region(rgn);
# 256
{struct Cyc_List_List*inst=Cyc_List_rzip(rgn,rgn,ad->tvs,ts);
{struct Cyc_List_List*fs=((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->fields;for(0;fs != 0;fs=fs->tl){
if(!Cyc_Tcutil_is_bits_only_type(Cyc_Tcutil_rsubstitute(rgn,inst,((struct Cyc_Absyn_Aggrfield*)fs->hd)->type))){int _tmp30=0;_npop_handler(0U);return _tmp30;}}}{
# 257
int _tmp31=1;_npop_handler(0U);return _tmp31;}}
# 256
;_pop_region(rgn);}}}}default: _LL1A: _LL1B:
# 260
 return 0;}_LLB:;}case 4U: _LL3: _tmp29=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp26)->f1).elt_type;_tmp2A=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp26)->f1).zero_term;_LL4: {void*t=_tmp29;void*zero_term=_tmp2A;
# 265
return !Cyc_Absyn_type2bool(0,zero_term)&& Cyc_Tcutil_is_bits_only_type(t);}case 6U: _LL5: _tmp28=((struct Cyc_Absyn_TupleType_Absyn_Type_struct*)_tmp26)->f1;_LL6: {struct Cyc_List_List*tqs=_tmp28;
# 267
for(0;tqs != 0;tqs=tqs->tl){
if(!Cyc_Tcutil_is_bits_only_type((*((struct _tuple12*)tqs->hd)).f2))return 0;}
# 267
return 1;}case 7U: _LL7: _tmp27=((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmp26)->f2;_LL8: {struct Cyc_List_List*fs=_tmp27;
# 271
for(0;fs != 0;fs=fs->tl){
if(!Cyc_Tcutil_is_bits_only_type(((struct Cyc_Absyn_Aggrfield*)fs->hd)->type))return 0;}
# 271
return 1;}default: _LL9: _LLA:
# 274
 return 0;}_LL0:;}
# 38
int Cyc_Tcutil_is_noreturn_fn_type(void*t){
# 281
void*_stmttmp15=Cyc_Tcutil_compress(t);void*_tmp32=_stmttmp15;struct Cyc_List_List*_tmp33;void*_tmp34;switch(*((int*)_tmp32)){case 3U: _LL1: _tmp34=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp32)->f1).elt_type;_LL2: {void*elt=_tmp34;
return Cyc_Tcutil_is_noreturn_fn_type(elt);}case 5U: _LL3: _tmp33=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp32)->f1).attributes;_LL4: {struct Cyc_List_List*atts=_tmp33;
# 284
for(0;atts != 0;atts=atts->tl){
void*_stmttmp16=(void*)atts->hd;void*_tmp35=_stmttmp16;if(((struct Cyc_Absyn_Noreturn_att_Absyn_Attribute_struct*)_tmp35)->tag == 4U){_LL8: _LL9:
 return 1;}else{_LLA: _LLB:
 continue;}_LL7:;}
# 289
return 0;}default: _LL5: _LL6:
 return 0;}_LL0:;}
# 38
int Cyc_Tcutil_warn_region_coerce=0;
# 298
void Cyc_Tcutil_terr(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap){
# 300
Cyc_Warn_verr(loc,fmt,ap);}
# 298
void*Cyc_Tcutil_impos(struct _fat_ptr fmt,struct _fat_ptr ap){
# 304
Cyc_Warn_vimpos(fmt,ap);}
# 298
void Cyc_Tcutil_warn(unsigned sg,struct _fat_ptr fmt,struct _fat_ptr ap){
# 308
Cyc_Warn_vwarn(sg,fmt,ap);}
# 298
int Cyc_Tcutil_fast_tvar_cmp(struct Cyc_Absyn_Tvar*tv1,struct Cyc_Absyn_Tvar*tv2){
# 313
return tv1->identity - tv2->identity;}
# 298
void*Cyc_Tcutil_compress(void*t){
# 318
void*_tmp36=t;void*_tmp37;struct Cyc_Absyn_Exp*_tmp38;struct Cyc_Absyn_Exp*_tmp39;void**_tmp3A;void**_tmp3B;switch(*((int*)_tmp36)){case 1U: if(((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp36)->f2 == 0){_LL1: _LL2:
 goto _LL4;}else{_LL7: _tmp3B=(void**)&((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp36)->f2;_LL8: {void**t2opt_ref=_tmp3B;
# 328
void*ta=(void*)_check_null(*t2opt_ref);
void*t2=Cyc_Tcutil_compress(ta);
if(t2 != ta)
*t2opt_ref=t2;
# 330
return t2;}}case 8U: if(((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmp36)->f4 == 0){_LL3: _LL4:
# 320
 return t;}else{_LL5: _tmp3A=(void**)&((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmp36)->f4;_LL6: {void**topt_ref=_tmp3A;
# 322
void*ta=(void*)_check_null(*topt_ref);
void*t2=Cyc_Tcutil_compress(ta);
if(t2 != ta)
*topt_ref=t2;
# 324
return t2;}}case 9U: _LL9: _tmp39=((struct Cyc_Absyn_ValueofType_Absyn_Type_struct*)_tmp36)->f1;_LLA: {struct Cyc_Absyn_Exp*e=_tmp39;
# 334
Cyc_Evexp_eval_const_uint_exp(e);{
void*_stmttmp17=e->r;void*_tmp3C=_stmttmp17;void*_tmp3D;if(((struct Cyc_Absyn_Valueof_e_Absyn_Raw_exp_struct*)_tmp3C)->tag == 40U){_LL12: _tmp3D=(void*)((struct Cyc_Absyn_Valueof_e_Absyn_Raw_exp_struct*)_tmp3C)->f1;_LL13: {void*t2=_tmp3D;
return Cyc_Tcutil_compress(t2);}}else{_LL14: _LL15:
 return t;}_LL11:;}}case 11U: _LLB: _tmp38=((struct Cyc_Absyn_TypeofType_Absyn_Type_struct*)_tmp36)->f1;_LLC: {struct Cyc_Absyn_Exp*e=_tmp38;
# 340
void*t2=e->topt;
if(t2 != 0)return t2;else{
return t;}}case 10U: if(((struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct*)_tmp36)->f2 != 0){_LLD: _tmp37=*((struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct*)_tmp36)->f2;_LLE: {void*t=_tmp37;
return Cyc_Tcutil_compress(t);}}else{goto _LLF;}default: _LLF: _LL10:
 return t;}_LL0:;}
# 298
void*Cyc_Tcutil_copy_type(void*t);
# 353
static struct Cyc_List_List*Cyc_Tcutil_copy_types(struct Cyc_List_List*ts){
return Cyc_List_map(Cyc_Tcutil_copy_type,ts);}
# 353
static void*Cyc_Tcutil_copy_kindbound(void*kb){
# 357
void*_stmttmp18=Cyc_Absyn_compress_kb(kb);void*_tmp3E=_stmttmp18;struct Cyc_Absyn_Kind*_tmp3F;switch(*((int*)_tmp3E)){case 1U: _LL1: _LL2:
 return(void*)({struct Cyc_Absyn_Unknown_kb_Absyn_KindBound_struct*_tmp40=_cycalloc(sizeof(*_tmp40));_tmp40->tag=1U,_tmp40->f1=0;_tmp40;});case 2U: _LL3: _tmp3F=((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp3E)->f2;_LL4: {struct Cyc_Absyn_Kind*k=_tmp3F;
return(void*)({struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*_tmp41=_cycalloc(sizeof(*_tmp41));_tmp41->tag=2U,_tmp41->f1=0,_tmp41->f2=k;_tmp41;});}default: _LL5: _LL6:
 return kb;}_LL0:;}
# 353
static struct Cyc_Absyn_Tvar*Cyc_Tcutil_copy_tvar(struct Cyc_Absyn_Tvar*tv){
# 365
return({struct Cyc_Absyn_Tvar*_tmp42=_cycalloc(sizeof(*_tmp42));_tmp42->name=tv->name,_tmp42->identity=- 1,({void*_tmp52B=Cyc_Tcutil_copy_kindbound(tv->kind);_tmp42->kind=_tmp52B;});_tmp42;});}
# 353
static struct _tuple9*Cyc_Tcutil_copy_arg(struct _tuple9*arg){
# 369
void*_tmp45;struct Cyc_Absyn_Tqual _tmp44;struct _fat_ptr*_tmp43;_LL1: _tmp43=arg->f1;_tmp44=arg->f2;_tmp45=arg->f3;_LL2: {struct _fat_ptr*x=_tmp43;struct Cyc_Absyn_Tqual y=_tmp44;void*t=_tmp45;
return({struct _tuple9*_tmp46=_cycalloc(sizeof(*_tmp46));_tmp46->f1=x,_tmp46->f2=y,({void*_tmp52C=Cyc_Tcutil_copy_type(t);_tmp46->f3=_tmp52C;});_tmp46;});}}
# 353
static struct _tuple12*Cyc_Tcutil_copy_tqt(struct _tuple12*arg){
# 373
return({struct _tuple12*_tmp47=_cycalloc(sizeof(*_tmp47));_tmp47->f1=(*arg).f1,({void*_tmp52D=Cyc_Tcutil_copy_type((*arg).f2);_tmp47->f2=_tmp52D;});_tmp47;});}
# 353
static struct Cyc_Absyn_Exp*Cyc_Tcutil_deep_copy_exp_opt(int preserve_types,struct Cyc_Absyn_Exp*e){
# 376
if(e == 0)return 0;else{
return Cyc_Tcutil_deep_copy_exp(preserve_types,e);}}
# 353
static struct Cyc_Absyn_Aggrfield*Cyc_Tcutil_copy_field(struct Cyc_Absyn_Aggrfield*f){
# 380
return({struct Cyc_Absyn_Aggrfield*_tmp48=_cycalloc(sizeof(*_tmp48));_tmp48->name=f->name,_tmp48->tq=f->tq,({void*_tmp52F=Cyc_Tcutil_copy_type(f->type);_tmp48->type=_tmp52F;}),_tmp48->width=f->width,_tmp48->attributes=f->attributes,({
struct Cyc_Absyn_Exp*_tmp52E=Cyc_Tcutil_deep_copy_exp_opt(1,f->requires_clause);_tmp48->requires_clause=_tmp52E;});_tmp48;});}struct _tuple15{void*f1;void*f2;};
# 353
static struct _tuple15*Cyc_Tcutil_copy_rgncmp(struct _tuple15*x){
# 384
void*_tmp4A;void*_tmp49;_LL1: _tmp49=x->f1;_tmp4A=x->f2;_LL2: {void*r1=_tmp49;void*r2=_tmp4A;
return({struct _tuple15*_tmp4B=_cycalloc(sizeof(*_tmp4B));({void*_tmp531=Cyc_Tcutil_copy_type(r1);_tmp4B->f1=_tmp531;}),({void*_tmp530=Cyc_Tcutil_copy_type(r2);_tmp4B->f2=_tmp530;});_tmp4B;});}}
# 353
static void*Cyc_Tcutil_tvar2type(struct Cyc_Absyn_Tvar*t){
# 388
return Cyc_Absyn_var_type(Cyc_Tcutil_copy_tvar(t));}
# 353
void*Cyc_Tcutil_copy_cap(void*c){
# 394
void*_tmp4C=c;int _tmp4E;struct Cyc_Absyn_Exp*_tmp4D;switch(*((int*)_tmp4C)){case 0U: _LL1: _LL2:
# 396
 return(void*)({struct Cyc_Absyn_Star_Absyn_Cap_struct*_tmp4F=_cycalloc(sizeof(*_tmp4F));_tmp4F->tag=0U;_tmp4F;});case 1U: _LL3: _LL4:
 return(void*)({struct Cyc_Absyn_Bot_Absyn_Cap_struct*_tmp50=_cycalloc(sizeof(*_tmp50));_tmp50->tag=1U;_tmp50;});default: _LL5: _tmp4D=((struct Cyc_Absyn_Nat_Absyn_Cap_struct*)_tmp4C)->f1;_tmp4E=((struct Cyc_Absyn_Nat_Absyn_Cap_struct*)_tmp4C)->f2;_LL6: {struct Cyc_Absyn_Exp*e=_tmp4D;int b=_tmp4E;
return(void*)({struct Cyc_Absyn_Nat_Absyn_Cap_struct*_tmp51=_cycalloc(sizeof(*_tmp51));_tmp51->tag=2U,({struct Cyc_Absyn_Exp*_tmp532=Cyc_Absyn_copy_exp(e);_tmp51->f1=_tmp532;}),_tmp51->f2=(int)e;_tmp51;});}}_LL0:;}
# 353
struct Cyc_Absyn_RgnEffect*Cyc_Tcutil_copy_eff(struct Cyc_Absyn_RgnEffect*eff){
# 404
return({struct Cyc_Absyn_RgnEffect*_tmp52=_cycalloc(sizeof(*_tmp52));({void*_tmp537=Cyc_Tcutil_copy_type(eff->name);_tmp52->name=_tmp537;}),({
void*_tmp536=Cyc_Tcutil_copy_cap(eff->rgn);_tmp52->rgn=_tmp536;}),({
void*_tmp535=Cyc_Tcutil_copy_cap(eff->lock);_tmp52->lock=_tmp535;}),({
void*_tmp534=Cyc_Tcutil_copy_cap(eff->unadopt);_tmp52->unadopt=_tmp534;}),
eff->parent == 0?_tmp52->parent=0:({void*_tmp533=Cyc_Tcutil_copy_type((void*)_check_null(eff->parent));_tmp52->parent=_tmp533;});_tmp52;});}
# 353
void*Cyc_Tcutil_copy_type(void*t){
# 414
void*_stmttmp19=Cyc_Tcutil_compress(t);void*_tmp53=_stmttmp19;struct Cyc_Absyn_Datatypedecl*_tmp54;struct Cyc_Absyn_Enumdecl*_tmp55;struct Cyc_Absyn_Aggrdecl*_tmp56;struct Cyc_Absyn_Typedefdecl*_tmp59;struct Cyc_List_List*_tmp58;struct _tuple1*_tmp57;struct Cyc_Absyn_Exp*_tmp5A;struct Cyc_Absyn_Exp*_tmp5B;struct Cyc_List_List*_tmp5D;enum Cyc_Absyn_AggrKind _tmp5C;struct Cyc_List_List*_tmp5E;struct Cyc_List_List*_tmp6D;struct Cyc_List_List*_tmp6C;struct Cyc_List_List*_tmp6B;struct Cyc_Absyn_Exp*_tmp6A;struct Cyc_List_List*_tmp69;struct Cyc_Absyn_Exp*_tmp68;struct Cyc_List_List*_tmp67;struct Cyc_List_List*_tmp66;struct Cyc_Absyn_VarargInfo*_tmp65;int _tmp64;struct Cyc_List_List*_tmp63;void*_tmp62;struct Cyc_Absyn_Tqual _tmp61;void*_tmp60;struct Cyc_List_List*_tmp5F;unsigned _tmp72;void*_tmp71;struct Cyc_Absyn_Exp*_tmp70;struct Cyc_Absyn_Tqual _tmp6F;void*_tmp6E;struct Cyc_Absyn_PtrLoc*_tmp79;void*_tmp78;void*_tmp77;void*_tmp76;void*_tmp75;struct Cyc_Absyn_Tqual _tmp74;void*_tmp73;struct Cyc_Absyn_Tvar*_tmp7A;struct Cyc_List_List*_tmp7C;void*_tmp7B;void*_tmp7D;switch(*((int*)_tmp53)){case 0U: if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp53)->f2 == 0){_LL1: _tmp7D=(void*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp53)->f1;_LL2: {void*c=_tmp7D;
return t;}}else{_LL3: _tmp7B=(void*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp53)->f1;_tmp7C=((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp53)->f2;_LL4: {void*c=_tmp7B;struct Cyc_List_List*ts=_tmp7C;
return(void*)({struct Cyc_Absyn_AppType_Absyn_Type_struct*_tmp7E=_cycalloc(sizeof(*_tmp7E));_tmp7E->tag=0U,_tmp7E->f1=c,({struct Cyc_List_List*_tmp538=Cyc_Tcutil_copy_types(ts);_tmp7E->f2=_tmp538;});_tmp7E;});}}case 1U: _LL5: _LL6:
 return t;case 2U: _LL7: _tmp7A=((struct Cyc_Absyn_VarType_Absyn_Type_struct*)_tmp53)->f1;_LL8: {struct Cyc_Absyn_Tvar*tv=_tmp7A;
return Cyc_Absyn_var_type(Cyc_Tcutil_copy_tvar(tv));}case 3U: _LL9: _tmp73=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp53)->f1).elt_type;_tmp74=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp53)->f1).elt_tq;_tmp75=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp53)->f1).ptr_atts).rgn;_tmp76=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp53)->f1).ptr_atts).nullable;_tmp77=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp53)->f1).ptr_atts).bounds;_tmp78=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp53)->f1).ptr_atts).zero_term;_tmp79=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp53)->f1).ptr_atts).ptrloc;_LLA: {void*elt=_tmp73;struct Cyc_Absyn_Tqual tq=_tmp74;void*rgn=_tmp75;void*nbl=_tmp76;void*bs=_tmp77;void*zt=_tmp78;struct Cyc_Absyn_PtrLoc*loc=_tmp79;
# 420
void*elt2=Cyc_Tcutil_copy_type(elt);
void*rgn2=Cyc_Tcutil_copy_type(rgn);
void*nbl2=Cyc_Tcutil_copy_type(nbl);
struct Cyc_Absyn_Tqual tq2=tq;
# 425
void*bs2=Cyc_Tcutil_copy_type(bs);
void*zt2=Cyc_Tcutil_copy_type(zt);
return(void*)({struct Cyc_Absyn_PointerType_Absyn_Type_struct*_tmp7F=_cycalloc(sizeof(*_tmp7F));_tmp7F->tag=3U,(_tmp7F->f1).elt_type=elt2,(_tmp7F->f1).elt_tq=tq2,((_tmp7F->f1).ptr_atts).rgn=rgn2,((_tmp7F->f1).ptr_atts).nullable=nbl2,((_tmp7F->f1).ptr_atts).bounds=bs2,((_tmp7F->f1).ptr_atts).zero_term=zt2,((_tmp7F->f1).ptr_atts).ptrloc=loc;_tmp7F;});}case 4U: _LLB: _tmp6E=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp53)->f1).elt_type;_tmp6F=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp53)->f1).tq;_tmp70=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp53)->f1).num_elts;_tmp71=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp53)->f1).zero_term;_tmp72=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp53)->f1).zt_loc;_LLC: {void*et=_tmp6E;struct Cyc_Absyn_Tqual tq=_tmp6F;struct Cyc_Absyn_Exp*eo=_tmp70;void*zt=_tmp71;unsigned ztl=_tmp72;
# 429
return(void*)({struct Cyc_Absyn_ArrayType_Absyn_Type_struct*_tmp80=_cycalloc(sizeof(*_tmp80));_tmp80->tag=4U,({void*_tmp53B=Cyc_Tcutil_copy_type(et);(_tmp80->f1).elt_type=_tmp53B;}),(_tmp80->f1).tq=tq,({struct Cyc_Absyn_Exp*_tmp53A=Cyc_Tcutil_deep_copy_exp_opt(1,eo);(_tmp80->f1).num_elts=_tmp53A;}),({
void*_tmp539=Cyc_Tcutil_copy_type(zt);(_tmp80->f1).zero_term=_tmp539;}),(_tmp80->f1).zt_loc=ztl;_tmp80;});}case 5U: _LLD: _tmp5F=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp53)->f1).tvars;_tmp60=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp53)->f1).effect;_tmp61=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp53)->f1).ret_tqual;_tmp62=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp53)->f1).ret_type;_tmp63=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp53)->f1).args;_tmp64=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp53)->f1).c_varargs;_tmp65=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp53)->f1).cyc_varargs;_tmp66=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp53)->f1).rgn_po;_tmp67=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp53)->f1).attributes;_tmp68=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp53)->f1).requires_clause;_tmp69=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp53)->f1).requires_relns;_tmp6A=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp53)->f1).ensures_clause;_tmp6B=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp53)->f1).ensures_relns;_tmp6C=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp53)->f1).ieffect;_tmp6D=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp53)->f1).oeffect;_LLE: {struct Cyc_List_List*tvs=_tmp5F;void*effopt=_tmp60;struct Cyc_Absyn_Tqual rt_tq=_tmp61;void*rt=_tmp62;struct Cyc_List_List*args=_tmp63;int c_varargs=_tmp64;struct Cyc_Absyn_VarargInfo*cyc_varargs=_tmp65;struct Cyc_List_List*rgn_po=_tmp66;struct Cyc_List_List*atts=_tmp67;struct Cyc_Absyn_Exp*req=_tmp68;struct Cyc_List_List*req_rlns=_tmp69;struct Cyc_Absyn_Exp*ens=_tmp6A;struct Cyc_List_List*ens_rlns=_tmp6B;struct Cyc_List_List*ieff=_tmp6C;struct Cyc_List_List*oeff=_tmp6D;
# 432
struct Cyc_List_List*tvs2=({struct Cyc_List_List*(*_tmp53C)(struct Cyc_Absyn_Tvar*(*f)(struct Cyc_Absyn_Tvar*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp87)(struct Cyc_Absyn_Tvar*(*f)(struct Cyc_Absyn_Tvar*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_Absyn_Tvar*(*f)(struct Cyc_Absyn_Tvar*),struct Cyc_List_List*x))Cyc_List_map;_tmp87;});_tmp53C(Cyc_Tcutil_copy_tvar,tvs);});
void*effopt2=effopt == 0?0: Cyc_Tcutil_copy_type(effopt);
void*rt2=Cyc_Tcutil_copy_type(rt);
struct Cyc_List_List*args2=({struct Cyc_List_List*(*_tmp53D)(struct _tuple9*(*f)(struct _tuple9*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp86)(struct _tuple9*(*f)(struct _tuple9*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct _tuple9*(*f)(struct _tuple9*),struct Cyc_List_List*x))Cyc_List_map;_tmp86;});_tmp53D(Cyc_Tcutil_copy_arg,args);});
int c_varargs2=c_varargs;
struct Cyc_Absyn_VarargInfo*cyc_varargs2=0;
if(cyc_varargs != 0){
struct Cyc_Absyn_VarargInfo*cv=cyc_varargs;
cyc_varargs2=({struct Cyc_Absyn_VarargInfo*_tmp81=_cycalloc(sizeof(*_tmp81));_tmp81->name=cv->name,_tmp81->tq=cv->tq,({void*_tmp53E=Cyc_Tcutil_copy_type(cv->type);_tmp81->type=_tmp53E;}),_tmp81->inject=cv->inject;_tmp81;});}{
# 438
struct Cyc_List_List*rgn_po2=({
# 443
struct Cyc_List_List*(*_tmp53F)(struct _tuple15*(*f)(struct _tuple15*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp85)(struct _tuple15*(*f)(struct _tuple15*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct _tuple15*(*f)(struct _tuple15*),struct Cyc_List_List*x))Cyc_List_map;_tmp85;});_tmp53F(Cyc_Tcutil_copy_rgncmp,rgn_po);});
struct Cyc_List_List*atts2=atts;
struct Cyc_Absyn_Exp*req2=Cyc_Tcutil_deep_copy_exp_opt(1,req);
struct Cyc_List_List*req_rlns2=Cyc_Relations_copy_relns(Cyc_Core_heap_region,req_rlns);
struct Cyc_Absyn_Exp*ens2=Cyc_Tcutil_deep_copy_exp_opt(1,ens);
struct Cyc_List_List*ens_rlns2=Cyc_Relations_copy_relns(Cyc_Core_heap_region,ens_rlns);
struct Cyc_List_List*ieff2=({struct Cyc_List_List*(*_tmp540)(struct Cyc_Absyn_RgnEffect*(*f)(struct Cyc_Absyn_RgnEffect*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp84)(struct Cyc_Absyn_RgnEffect*(*f)(struct Cyc_Absyn_RgnEffect*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_Absyn_RgnEffect*(*f)(struct Cyc_Absyn_RgnEffect*),struct Cyc_List_List*x))Cyc_List_map;_tmp84;});_tmp540(Cyc_Tcutil_copy_eff,ieff);});
struct Cyc_List_List*oeff2=({struct Cyc_List_List*(*_tmp541)(struct Cyc_Absyn_RgnEffect*(*f)(struct Cyc_Absyn_RgnEffect*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp83)(struct Cyc_Absyn_RgnEffect*(*f)(struct Cyc_Absyn_RgnEffect*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_Absyn_RgnEffect*(*f)(struct Cyc_Absyn_RgnEffect*),struct Cyc_List_List*x))Cyc_List_map;_tmp83;});_tmp541(Cyc_Tcutil_copy_eff,oeff);});
return(void*)({struct Cyc_Absyn_FnType_Absyn_Type_struct*_tmp82=_cycalloc(sizeof(*_tmp82));_tmp82->tag=5U,(_tmp82->f1).tvars=tvs2,(_tmp82->f1).effect=effopt2,(_tmp82->f1).ret_tqual=rt_tq,(_tmp82->f1).ret_type=rt2,(_tmp82->f1).args=args2,(_tmp82->f1).c_varargs=c_varargs2,(_tmp82->f1).cyc_varargs=cyc_varargs2,(_tmp82->f1).rgn_po=rgn_po2,(_tmp82->f1).attributes=atts2,(_tmp82->f1).requires_clause=req2,(_tmp82->f1).requires_relns=req_rlns2,(_tmp82->f1).ensures_clause=ens2,(_tmp82->f1).ensures_relns=ens_rlns2,(_tmp82->f1).ieffect=ieff2,(_tmp82->f1).oeffect=oeff2;_tmp82;});}}case 6U: _LLF: _tmp5E=((struct Cyc_Absyn_TupleType_Absyn_Type_struct*)_tmp53)->f1;_LL10: {struct Cyc_List_List*tqts=_tmp5E;
# 454
return(void*)({struct Cyc_Absyn_TupleType_Absyn_Type_struct*_tmp89=_cycalloc(sizeof(*_tmp89));_tmp89->tag=6U,({struct Cyc_List_List*_tmp543=({struct Cyc_List_List*(*_tmp542)(struct _tuple12*(*f)(struct _tuple12*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp88)(struct _tuple12*(*f)(struct _tuple12*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct _tuple12*(*f)(struct _tuple12*),struct Cyc_List_List*x))Cyc_List_map;_tmp88;});_tmp542(Cyc_Tcutil_copy_tqt,tqts);});_tmp89->f1=_tmp543;});_tmp89;});}case 7U: _LL11: _tmp5C=((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmp53)->f1;_tmp5D=((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmp53)->f2;_LL12: {enum Cyc_Absyn_AggrKind k=_tmp5C;struct Cyc_List_List*fs=_tmp5D;
return(void*)({struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*_tmp8B=_cycalloc(sizeof(*_tmp8B));_tmp8B->tag=7U,_tmp8B->f1=k,({struct Cyc_List_List*_tmp545=({struct Cyc_List_List*(*_tmp544)(struct Cyc_Absyn_Aggrfield*(*f)(struct Cyc_Absyn_Aggrfield*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp8A)(struct Cyc_Absyn_Aggrfield*(*f)(struct Cyc_Absyn_Aggrfield*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_Absyn_Aggrfield*(*f)(struct Cyc_Absyn_Aggrfield*),struct Cyc_List_List*x))Cyc_List_map;_tmp8A;});_tmp544(Cyc_Tcutil_copy_field,fs);});_tmp8B->f2=_tmp545;});_tmp8B;});}case 9U: _LL13: _tmp5B=((struct Cyc_Absyn_ValueofType_Absyn_Type_struct*)_tmp53)->f1;_LL14: {struct Cyc_Absyn_Exp*e=_tmp5B;
return(void*)({struct Cyc_Absyn_ValueofType_Absyn_Type_struct*_tmp8C=_cycalloc(sizeof(*_tmp8C));_tmp8C->tag=9U,_tmp8C->f1=e;_tmp8C;});}case 11U: _LL15: _tmp5A=((struct Cyc_Absyn_TypeofType_Absyn_Type_struct*)_tmp53)->f1;_LL16: {struct Cyc_Absyn_Exp*e=_tmp5A;
return(void*)({struct Cyc_Absyn_TypeofType_Absyn_Type_struct*_tmp8D=_cycalloc(sizeof(*_tmp8D));_tmp8D->tag=11U,_tmp8D->f1=e;_tmp8D;});}case 8U: _LL17: _tmp57=((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmp53)->f1;_tmp58=((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmp53)->f2;_tmp59=((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmp53)->f3;_LL18: {struct _tuple1*tdn=_tmp57;struct Cyc_List_List*ts=_tmp58;struct Cyc_Absyn_Typedefdecl*td=_tmp59;
# 459
return(void*)({struct Cyc_Absyn_TypedefType_Absyn_Type_struct*_tmp8E=_cycalloc(sizeof(*_tmp8E));_tmp8E->tag=8U,_tmp8E->f1=tdn,({struct Cyc_List_List*_tmp546=Cyc_Tcutil_copy_types(ts);_tmp8E->f2=_tmp546;}),_tmp8E->f3=td,_tmp8E->f4=0;_tmp8E;});}default: switch(*((int*)((struct Cyc_Absyn_TypeDecl*)((struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct*)_tmp53)->f1)->r)){case 0U: _LL19: _tmp56=((struct Cyc_Absyn_Aggr_td_Absyn_Raw_typedecl_struct*)(((struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct*)_tmp53)->f1)->r)->f1;_LL1A: {struct Cyc_Absyn_Aggrdecl*ad=_tmp56;
# 462
struct Cyc_List_List*targs=({struct Cyc_List_List*(*_tmp547)(void*(*f)(struct Cyc_Absyn_Tvar*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp8F)(void*(*f)(struct Cyc_Absyn_Tvar*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(void*(*f)(struct Cyc_Absyn_Tvar*),struct Cyc_List_List*x))Cyc_List_map;_tmp8F;});_tmp547(Cyc_Tcutil_tvar2type,ad->tvs);});
return({union Cyc_Absyn_AggrInfo _tmp548=Cyc_Absyn_UnknownAggr(ad->kind,ad->name,0);Cyc_Absyn_aggr_type(_tmp548,targs);});}case 1U: _LL1B: _tmp55=((struct Cyc_Absyn_Enum_td_Absyn_Raw_typedecl_struct*)(((struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct*)_tmp53)->f1)->r)->f1;_LL1C: {struct Cyc_Absyn_Enumdecl*ed=_tmp55;
# 465
return Cyc_Absyn_enum_type(ed->name,0);}default: _LL1D: _tmp54=((struct Cyc_Absyn_Datatype_td_Absyn_Raw_typedecl_struct*)(((struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct*)_tmp53)->f1)->r)->f1;_LL1E: {struct Cyc_Absyn_Datatypedecl*dd=_tmp54;
# 467
struct Cyc_List_List*targs=({struct Cyc_List_List*(*_tmp549)(void*(*f)(struct Cyc_Absyn_Tvar*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp90)(void*(*f)(struct Cyc_Absyn_Tvar*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(void*(*f)(struct Cyc_Absyn_Tvar*),struct Cyc_List_List*x))Cyc_List_map;_tmp90;});_tmp549(Cyc_Tcutil_tvar2type,dd->tvs);});
return({union Cyc_Absyn_DatatypeInfo _tmp54A=Cyc_Absyn_UnknownDatatype(({struct Cyc_Absyn_UnknownDatatypeInfo _tmp4DE;_tmp4DE.name=dd->name,_tmp4DE.is_extensible=0;_tmp4DE;}));Cyc_Absyn_datatype_type(_tmp54A,targs);});}}}_LL0:;}
# 353
static void*Cyc_Tcutil_copy_designator(int preserve_types,void*d){
# 475
void*_tmp91=d;struct _fat_ptr*_tmp92;struct Cyc_Absyn_Exp*_tmp93;if(((struct Cyc_Absyn_ArrayElement_Absyn_Designator_struct*)_tmp91)->tag == 0U){_LL1: _tmp93=((struct Cyc_Absyn_ArrayElement_Absyn_Designator_struct*)_tmp91)->f1;_LL2: {struct Cyc_Absyn_Exp*e=_tmp93;
return(void*)({struct Cyc_Absyn_ArrayElement_Absyn_Designator_struct*_tmp94=_cycalloc(sizeof(*_tmp94));_tmp94->tag=0U,({struct Cyc_Absyn_Exp*_tmp54B=Cyc_Tcutil_deep_copy_exp(preserve_types,e);_tmp94->f1=_tmp54B;});_tmp94;});}}else{_LL3: _tmp92=((struct Cyc_Absyn_FieldName_Absyn_Designator_struct*)_tmp91)->f1;_LL4: {struct _fat_ptr*v=_tmp92;
return d;}}_LL0:;}struct _tuple16{struct Cyc_List_List*f1;struct Cyc_Absyn_Exp*f2;};
# 353
static struct _tuple16*Cyc_Tcutil_copy_eds(int preserve_types,struct _tuple16*e){
# 482
return({struct _tuple16*_tmp96=_cycalloc(sizeof(*_tmp96));({struct Cyc_List_List*_tmp54F=({struct Cyc_List_List*(*_tmp54E)(void*(*f)(int,void*),int env,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp95)(void*(*f)(int,void*),int env,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(void*(*f)(int,void*),int env,struct Cyc_List_List*x))Cyc_List_map_c;_tmp95;});int _tmp54D=preserve_types;_tmp54E(Cyc_Tcutil_copy_designator,_tmp54D,(e[0]).f1);});_tmp96->f1=_tmp54F;}),({struct Cyc_Absyn_Exp*_tmp54C=Cyc_Tcutil_deep_copy_exp(preserve_types,(e[0]).f2);_tmp96->f2=_tmp54C;});_tmp96;});}
# 353
struct Cyc_Absyn_Exp*Cyc_Tcutil_deep_copy_exp(int preserve_types,struct Cyc_Absyn_Exp*e){
# 486
struct Cyc_Absyn_Exp*new_e;
int pt=preserve_types;
{void*_stmttmp1A=e->r;void*_tmp97=_stmttmp1A;struct Cyc_List_List*_tmp9C;struct Cyc_List_List*_tmp9B;struct Cyc_List_List*_tmp9A;struct _fat_ptr _tmp99;int _tmp98;void*_tmp9D;struct _fat_ptr*_tmp9F;struct Cyc_Absyn_Exp*_tmp9E;struct Cyc_List_List*_tmpA1;struct Cyc_Core_Opt*_tmpA0;struct Cyc_Absyn_Exp*_tmpA3;struct Cyc_Absyn_Exp*_tmpA2;int _tmpA9;int _tmpA8;struct Cyc_Absyn_Exp*_tmpA7;void**_tmpA6;struct Cyc_Absyn_Exp*_tmpA5;int _tmpA4;struct Cyc_Absyn_Enumfield*_tmpAB;void*_tmpAA;struct Cyc_Absyn_Enumfield*_tmpAD;struct Cyc_Absyn_Enumdecl*_tmpAC;struct Cyc_List_List*_tmpB1;void*_tmpB0;struct Cyc_Absyn_Tqual _tmpAF;struct _fat_ptr*_tmpAE;struct Cyc_List_List*_tmpB3;void*_tmpB2;struct Cyc_Absyn_Aggrdecl*_tmpB7;struct Cyc_List_List*_tmpB6;struct Cyc_List_List*_tmpB5;struct _tuple1*_tmpB4;int _tmpBA;void*_tmpB9;struct Cyc_Absyn_Exp*_tmpB8;int _tmpBE;struct Cyc_Absyn_Exp*_tmpBD;struct Cyc_Absyn_Exp*_tmpBC;struct Cyc_Absyn_Vardecl*_tmpBB;struct Cyc_Absyn_Datatypefield*_tmpC1;struct Cyc_Absyn_Datatypedecl*_tmpC0;struct Cyc_List_List*_tmpBF;struct Cyc_List_List*_tmpC2;struct Cyc_Absyn_Exp*_tmpC4;struct Cyc_Absyn_Exp*_tmpC3;int _tmpC8;int _tmpC7;struct _fat_ptr*_tmpC6;struct Cyc_Absyn_Exp*_tmpC5;int _tmpCC;int _tmpCB;struct _fat_ptr*_tmpCA;struct Cyc_Absyn_Exp*_tmpC9;struct Cyc_List_List*_tmpCD;struct Cyc_Absyn_Exp*_tmpCE;struct Cyc_Absyn_Exp*_tmpCF;struct Cyc_List_List*_tmpD1;void*_tmpD0;struct Cyc_Absyn_Exp*_tmpD2;void*_tmpD3;struct Cyc_Absyn_Exp*_tmpD4;struct Cyc_Absyn_Exp*_tmpD6;struct Cyc_Absyn_Exp*_tmpD5;enum Cyc_Absyn_Coercion _tmpDA;int _tmpD9;struct Cyc_Absyn_Exp*_tmpD8;void*_tmpD7;struct Cyc_List_List*_tmpDC;struct Cyc_Absyn_Exp*_tmpDB;struct Cyc_Absyn_Exp*_tmpDD;int _tmpDF;struct Cyc_Absyn_Exp*_tmpDE;int _tmpE3;struct Cyc_Absyn_VarargCallInfo*_tmpE2;struct Cyc_List_List*_tmpE1;struct Cyc_Absyn_Exp*_tmpE0;int _tmpEC;int _tmpEB;void*_tmpEA;struct Cyc_Absyn_Tqual _tmpE9;struct _fat_ptr*_tmpE8;struct Cyc_List_List*_tmpE7;int _tmpE6;struct Cyc_List_List*_tmpE5;struct Cyc_Absyn_Exp*_tmpE4;struct Cyc_Absyn_Exp*_tmpEF;struct Cyc_Core_Opt*_tmpEE;struct Cyc_Absyn_Exp*_tmpED;struct Cyc_Absyn_Exp*_tmpF2;struct Cyc_Absyn_Exp*_tmpF1;struct Cyc_Absyn_Exp*_tmpF0;struct Cyc_Absyn_Exp*_tmpF4;struct Cyc_Absyn_Exp*_tmpF3;struct Cyc_Absyn_Exp*_tmpF6;struct Cyc_Absyn_Exp*_tmpF5;struct Cyc_Absyn_Exp*_tmpF8;struct Cyc_Absyn_Exp*_tmpF7;struct Cyc_Absyn_Exp*_tmpFA;struct Cyc_Absyn_Exp*_tmpF9;enum Cyc_Absyn_Incrementor _tmpFC;struct Cyc_Absyn_Exp*_tmpFB;struct Cyc_List_List*_tmpFE;enum Cyc_Absyn_Primop _tmpFD;struct _fat_ptr _tmpFF;void*_tmp100;union Cyc_Absyn_Cnst _tmp101;switch(*((int*)_tmp97)){case 0U: _LL1: _tmp101=((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmp97)->f1;_LL2: {union Cyc_Absyn_Cnst c=_tmp101;
new_e=Cyc_Absyn_const_exp(c,e->loc);goto _LL0;}case 1U: _LL3: _tmp100=(void*)((struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct*)_tmp97)->f1;_LL4: {void*b=_tmp100;
new_e=Cyc_Absyn_varb_exp(b,e->loc);goto _LL0;}case 2U: _LL5: _tmpFF=((struct Cyc_Absyn_Pragma_e_Absyn_Raw_exp_struct*)_tmp97)->f1;_LL6: {struct _fat_ptr p=_tmpFF;
new_e=Cyc_Absyn_pragma_exp(p,e->loc);goto _LL0;}case 3U: _LL7: _tmpFD=((struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct*)_tmp97)->f1;_tmpFE=((struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct*)_tmp97)->f2;_LL8: {enum Cyc_Absyn_Primop p=_tmpFD;struct Cyc_List_List*es=_tmpFE;
new_e=({enum Cyc_Absyn_Primop _tmp553=p;struct Cyc_List_List*_tmp552=({struct Cyc_List_List*(*_tmp551)(struct Cyc_Absyn_Exp*(*f)(int,struct Cyc_Absyn_Exp*),int env,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp102)(struct Cyc_Absyn_Exp*(*f)(int,struct Cyc_Absyn_Exp*),int env,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_Absyn_Exp*(*f)(int,struct Cyc_Absyn_Exp*),int env,struct Cyc_List_List*x))Cyc_List_map_c;_tmp102;});int _tmp550=pt;_tmp551(Cyc_Tcutil_deep_copy_exp,_tmp550,es);});Cyc_Absyn_primop_exp(_tmp553,_tmp552,e->loc);});goto _LL0;}case 5U: _LL9: _tmpFB=((struct Cyc_Absyn_Increment_e_Absyn_Raw_exp_struct*)_tmp97)->f1;_tmpFC=((struct Cyc_Absyn_Increment_e_Absyn_Raw_exp_struct*)_tmp97)->f2;_LLA: {struct Cyc_Absyn_Exp*e1=_tmpFB;enum Cyc_Absyn_Incrementor i=_tmpFC;
new_e=({struct Cyc_Absyn_Exp*_tmp555=Cyc_Tcutil_deep_copy_exp(pt,e1);enum Cyc_Absyn_Incrementor _tmp554=i;Cyc_Absyn_increment_exp(_tmp555,_tmp554,e->loc);});goto _LL0;}case 7U: _LLB: _tmpF9=((struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct*)_tmp97)->f1;_tmpFA=((struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct*)_tmp97)->f2;_LLC: {struct Cyc_Absyn_Exp*e1=_tmpF9;struct Cyc_Absyn_Exp*e2=_tmpFA;
new_e=({struct Cyc_Absyn_Exp*_tmp557=Cyc_Tcutil_deep_copy_exp(pt,e1);struct Cyc_Absyn_Exp*_tmp556=Cyc_Tcutil_deep_copy_exp(pt,e2);Cyc_Absyn_and_exp(_tmp557,_tmp556,e->loc);});goto _LL0;}case 8U: _LLD: _tmpF7=((struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct*)_tmp97)->f1;_tmpF8=((struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct*)_tmp97)->f2;_LLE: {struct Cyc_Absyn_Exp*e1=_tmpF7;struct Cyc_Absyn_Exp*e2=_tmpF8;
new_e=({struct Cyc_Absyn_Exp*_tmp559=Cyc_Tcutil_deep_copy_exp(pt,e1);struct Cyc_Absyn_Exp*_tmp558=Cyc_Tcutil_deep_copy_exp(pt,e2);Cyc_Absyn_or_exp(_tmp559,_tmp558,e->loc);});goto _LL0;}case 9U: _LLF: _tmpF5=((struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct*)_tmp97)->f1;_tmpF6=((struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct*)_tmp97)->f2;_LL10: {struct Cyc_Absyn_Exp*e1=_tmpF5;struct Cyc_Absyn_Exp*e2=_tmpF6;
new_e=({struct Cyc_Absyn_Exp*_tmp55B=Cyc_Tcutil_deep_copy_exp(pt,e1);struct Cyc_Absyn_Exp*_tmp55A=Cyc_Tcutil_deep_copy_exp(pt,e2);Cyc_Absyn_seq_exp(_tmp55B,_tmp55A,e->loc);});goto _LL0;}case 34U: _LL11: _tmpF3=((struct Cyc_Absyn_Spawn_e_Absyn_Raw_exp_struct*)_tmp97)->f1;_tmpF4=((struct Cyc_Absyn_Spawn_e_Absyn_Raw_exp_struct*)_tmp97)->f2;_LL12: {struct Cyc_Absyn_Exp*e1=_tmpF3;struct Cyc_Absyn_Exp*e2=_tmpF4;
new_e=({struct Cyc_Absyn_Exp*_tmp55D=Cyc_Tcutil_deep_copy_exp(pt,e1);struct Cyc_Absyn_Exp*_tmp55C=Cyc_Tcutil_deep_copy_exp(pt,e2);Cyc_Absyn_spawn_exp(_tmp55D,_tmp55C,e->loc);});goto _LL0;}case 6U: _LL13: _tmpF0=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmp97)->f1;_tmpF1=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmp97)->f2;_tmpF2=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmp97)->f3;_LL14: {struct Cyc_Absyn_Exp*e1=_tmpF0;struct Cyc_Absyn_Exp*e2=_tmpF1;struct Cyc_Absyn_Exp*e3=_tmpF2;
# 499
new_e=({struct Cyc_Absyn_Exp*_tmp560=Cyc_Tcutil_deep_copy_exp(pt,e1);struct Cyc_Absyn_Exp*_tmp55F=Cyc_Tcutil_deep_copy_exp(pt,e2);struct Cyc_Absyn_Exp*_tmp55E=Cyc_Tcutil_deep_copy_exp(pt,e3);Cyc_Absyn_conditional_exp(_tmp560,_tmp55F,_tmp55E,e->loc);});goto _LL0;}case 4U: _LL15: _tmpED=((struct Cyc_Absyn_AssignOp_e_Absyn_Raw_exp_struct*)_tmp97)->f1;_tmpEE=((struct Cyc_Absyn_AssignOp_e_Absyn_Raw_exp_struct*)_tmp97)->f2;_tmpEF=((struct Cyc_Absyn_AssignOp_e_Absyn_Raw_exp_struct*)_tmp97)->f3;_LL16: {struct Cyc_Absyn_Exp*e1=_tmpED;struct Cyc_Core_Opt*po=_tmpEE;struct Cyc_Absyn_Exp*e2=_tmpEF;
# 501
new_e=({struct Cyc_Absyn_Exp*_tmp563=Cyc_Tcutil_deep_copy_exp(pt,e1);struct Cyc_Core_Opt*_tmp562=(unsigned)po?({struct Cyc_Core_Opt*_tmp103=_cycalloc(sizeof(*_tmp103));_tmp103->v=(void*)po->v;_tmp103;}): 0;struct Cyc_Absyn_Exp*_tmp561=Cyc_Tcutil_deep_copy_exp(pt,e2);Cyc_Absyn_assignop_exp(_tmp563,_tmp562,_tmp561,e->loc);});
goto _LL0;}case 10U: if(((struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct*)_tmp97)->f3 != 0){_LL17: _tmpE4=((struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct*)_tmp97)->f1;_tmpE5=((struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct*)_tmp97)->f2;_tmpE6=(((struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct*)_tmp97)->f3)->num_varargs;_tmpE7=(((struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct*)_tmp97)->f3)->injectors;_tmpE8=((((struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct*)_tmp97)->f3)->vai)->name;_tmpE9=((((struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct*)_tmp97)->f3)->vai)->tq;_tmpEA=((((struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct*)_tmp97)->f3)->vai)->type;_tmpEB=((((struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct*)_tmp97)->f3)->vai)->inject;_tmpEC=((struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct*)_tmp97)->f4;_LL18: {struct Cyc_Absyn_Exp*e1=_tmpE4;struct Cyc_List_List*es=_tmpE5;int n=_tmpE6;struct Cyc_List_List*is=_tmpE7;struct _fat_ptr*nm=_tmpE8;struct Cyc_Absyn_Tqual tq=_tmpE9;void*t=_tmpEA;int i=_tmpEB;int resolved=_tmpEC;
# 504
new_e=({void*_tmp56B=(void*)({struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct*_tmp107=_cycalloc(sizeof(*_tmp107));_tmp107->tag=10U,({
struct Cyc_Absyn_Exp*_tmp56A=Cyc_Tcutil_deep_copy_exp(pt,e1);_tmp107->f1=_tmp56A;}),({struct Cyc_List_List*_tmp569=({struct Cyc_List_List*(*_tmp568)(struct Cyc_Absyn_Exp*(*f)(int,struct Cyc_Absyn_Exp*),int env,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp104)(struct Cyc_Absyn_Exp*(*f)(int,struct Cyc_Absyn_Exp*),int env,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_Absyn_Exp*(*f)(int,struct Cyc_Absyn_Exp*),int env,struct Cyc_List_List*x))Cyc_List_map_c;_tmp104;});int _tmp567=pt;_tmp568(Cyc_Tcutil_deep_copy_exp,_tmp567,es);});_tmp107->f2=_tmp569;}),({
struct Cyc_Absyn_VarargCallInfo*_tmp566=({struct Cyc_Absyn_VarargCallInfo*_tmp106=_cycalloc(sizeof(*_tmp106));_tmp106->num_varargs=n,_tmp106->injectors=is,({
struct Cyc_Absyn_VarargInfo*_tmp565=({struct Cyc_Absyn_VarargInfo*_tmp105=_cycalloc(sizeof(*_tmp105));_tmp105->name=nm,_tmp105->tq=tq,({void*_tmp564=Cyc_Tcutil_copy_type(t);_tmp105->type=_tmp564;}),_tmp105->inject=i;_tmp105;});_tmp106->vai=_tmp565;});_tmp106;});
# 506
_tmp107->f3=_tmp566;}),_tmp107->f4=resolved;_tmp107;});
# 504
Cyc_Absyn_new_exp(_tmp56B,e->loc);});
# 509
goto _LL0;}}else{_LL19: _tmpE0=((struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct*)_tmp97)->f1;_tmpE1=((struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct*)_tmp97)->f2;_tmpE2=((struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct*)_tmp97)->f3;_tmpE3=((struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct*)_tmp97)->f4;_LL1A: {struct Cyc_Absyn_Exp*e1=_tmpE0;struct Cyc_List_List*es=_tmpE1;struct Cyc_Absyn_VarargCallInfo*vci=_tmpE2;int resolved=_tmpE3;
# 511
new_e=({void*_tmp570=(void*)({struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct*_tmp109=_cycalloc(sizeof(*_tmp109));_tmp109->tag=10U,({struct Cyc_Absyn_Exp*_tmp56F=Cyc_Tcutil_deep_copy_exp(pt,e1);_tmp109->f1=_tmp56F;}),({struct Cyc_List_List*_tmp56E=({struct Cyc_List_List*(*_tmp56D)(struct Cyc_Absyn_Exp*(*f)(int,struct Cyc_Absyn_Exp*),int env,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp108)(struct Cyc_Absyn_Exp*(*f)(int,struct Cyc_Absyn_Exp*),int env,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_Absyn_Exp*(*f)(int,struct Cyc_Absyn_Exp*),int env,struct Cyc_List_List*x))Cyc_List_map_c;_tmp108;});int _tmp56C=pt;_tmp56D(Cyc_Tcutil_deep_copy_exp,_tmp56C,es);});_tmp109->f2=_tmp56E;}),_tmp109->f3=vci,_tmp109->f4=resolved;_tmp109;});Cyc_Absyn_new_exp(_tmp570,e->loc);});
goto _LL0;}}case 11U: _LL1B: _tmpDE=((struct Cyc_Absyn_Throw_e_Absyn_Raw_exp_struct*)_tmp97)->f1;_tmpDF=((struct Cyc_Absyn_Throw_e_Absyn_Raw_exp_struct*)_tmp97)->f2;_LL1C: {struct Cyc_Absyn_Exp*e1=_tmpDE;int b=_tmpDF;
# 514
new_e=b?({struct Cyc_Absyn_Exp*_tmp572=Cyc_Tcutil_deep_copy_exp(pt,e1);Cyc_Absyn_rethrow_exp(_tmp572,e->loc);}):({struct Cyc_Absyn_Exp*_tmp571=Cyc_Tcutil_deep_copy_exp(pt,e1);Cyc_Absyn_throw_exp(_tmp571,e->loc);});
goto _LL0;}case 12U: _LL1D: _tmpDD=((struct Cyc_Absyn_NoInstantiate_e_Absyn_Raw_exp_struct*)_tmp97)->f1;_LL1E: {struct Cyc_Absyn_Exp*e1=_tmpDD;
# 517
new_e=({struct Cyc_Absyn_Exp*_tmp573=Cyc_Tcutil_deep_copy_exp(pt,e1);Cyc_Absyn_noinstantiate_exp(_tmp573,e->loc);});goto _LL0;}case 13U: _LL1F: _tmpDB=((struct Cyc_Absyn_Instantiate_e_Absyn_Raw_exp_struct*)_tmp97)->f1;_tmpDC=((struct Cyc_Absyn_Instantiate_e_Absyn_Raw_exp_struct*)_tmp97)->f2;_LL20: {struct Cyc_Absyn_Exp*e1=_tmpDB;struct Cyc_List_List*ts=_tmpDC;
# 519
new_e=({struct Cyc_Absyn_Exp*_tmp575=Cyc_Tcutil_deep_copy_exp(pt,e1);struct Cyc_List_List*_tmp574=Cyc_List_map(Cyc_Tcutil_copy_type,ts);Cyc_Absyn_instantiate_exp(_tmp575,_tmp574,e->loc);});goto _LL0;}case 14U: _LL21: _tmpD7=(void*)((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp97)->f1;_tmpD8=((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp97)->f2;_tmpD9=((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp97)->f3;_tmpDA=((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp97)->f4;_LL22: {void*t=_tmpD7;struct Cyc_Absyn_Exp*e1=_tmpD8;int b=_tmpD9;enum Cyc_Absyn_Coercion c=_tmpDA;
# 521
new_e=({void*_tmp579=Cyc_Tcutil_copy_type(t);struct Cyc_Absyn_Exp*_tmp578=Cyc_Tcutil_deep_copy_exp(pt,e1);int _tmp577=b;enum Cyc_Absyn_Coercion _tmp576=c;Cyc_Absyn_cast_exp(_tmp579,_tmp578,_tmp577,_tmp576,e->loc);});goto _LL0;}case 16U: _LL23: _tmpD5=((struct Cyc_Absyn_New_e_Absyn_Raw_exp_struct*)_tmp97)->f1;_tmpD6=((struct Cyc_Absyn_New_e_Absyn_Raw_exp_struct*)_tmp97)->f2;_LL24: {struct Cyc_Absyn_Exp*eo=_tmpD5;struct Cyc_Absyn_Exp*e1=_tmpD6;
# 523
new_e=({struct Cyc_Absyn_Exp*_tmp57B=(unsigned)eo?Cyc_Tcutil_deep_copy_exp(pt,eo): 0;struct Cyc_Absyn_Exp*_tmp57A=Cyc_Tcutil_deep_copy_exp(pt,e1);Cyc_Absyn_New_exp(_tmp57B,_tmp57A,e->loc);});goto _LL0;}case 15U: _LL25: _tmpD4=((struct Cyc_Absyn_Address_e_Absyn_Raw_exp_struct*)_tmp97)->f1;_LL26: {struct Cyc_Absyn_Exp*e1=_tmpD4;
new_e=({struct Cyc_Absyn_Exp*_tmp57C=Cyc_Tcutil_deep_copy_exp(pt,e1);Cyc_Absyn_address_exp(_tmp57C,e->loc);});goto _LL0;}case 17U: _LL27: _tmpD3=(void*)((struct Cyc_Absyn_Sizeoftype_e_Absyn_Raw_exp_struct*)_tmp97)->f1;_LL28: {void*t=_tmpD3;
new_e=({void*_tmp57D=Cyc_Tcutil_copy_type(t);Cyc_Absyn_sizeoftype_exp(_tmp57D,e->loc);});goto _LL0;}case 18U: _LL29: _tmpD2=((struct Cyc_Absyn_Sizeofexp_e_Absyn_Raw_exp_struct*)_tmp97)->f1;_LL2A: {struct Cyc_Absyn_Exp*e1=_tmpD2;
new_e=({struct Cyc_Absyn_Exp*_tmp57E=Cyc_Tcutil_deep_copy_exp(pt,e1);Cyc_Absyn_sizeofexp_exp(_tmp57E,e->loc);});goto _LL0;}case 19U: _LL2B: _tmpD0=(void*)((struct Cyc_Absyn_Offsetof_e_Absyn_Raw_exp_struct*)_tmp97)->f1;_tmpD1=((struct Cyc_Absyn_Offsetof_e_Absyn_Raw_exp_struct*)_tmp97)->f2;_LL2C: {void*t=_tmpD0;struct Cyc_List_List*ofs=_tmpD1;
new_e=({void*_tmp580=Cyc_Tcutil_copy_type(t);struct Cyc_List_List*_tmp57F=ofs;Cyc_Absyn_offsetof_exp(_tmp580,_tmp57F,e->loc);});goto _LL0;}case 20U: _LL2D: _tmpCF=((struct Cyc_Absyn_Deref_e_Absyn_Raw_exp_struct*)_tmp97)->f1;_LL2E: {struct Cyc_Absyn_Exp*e1=_tmpCF;
new_e=({struct Cyc_Absyn_Exp*_tmp581=Cyc_Tcutil_deep_copy_exp(pt,e1);Cyc_Absyn_deref_exp(_tmp581,e->loc);});goto _LL0;}case 42U: _LL2F: _tmpCE=((struct Cyc_Absyn_Extension_e_Absyn_Raw_exp_struct*)_tmp97)->f1;_LL30: {struct Cyc_Absyn_Exp*e1=_tmpCE;
new_e=({struct Cyc_Absyn_Exp*_tmp582=Cyc_Tcutil_deep_copy_exp(pt,e1);Cyc_Absyn_extension_exp(_tmp582,e->loc);});goto _LL0;}case 24U: _LL31: _tmpCD=((struct Cyc_Absyn_Tuple_e_Absyn_Raw_exp_struct*)_tmp97)->f1;_LL32: {struct Cyc_List_List*es=_tmpCD;
new_e=({struct Cyc_List_List*_tmp585=({struct Cyc_List_List*(*_tmp584)(struct Cyc_Absyn_Exp*(*f)(int,struct Cyc_Absyn_Exp*),int env,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp10A)(struct Cyc_Absyn_Exp*(*f)(int,struct Cyc_Absyn_Exp*),int env,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_Absyn_Exp*(*f)(int,struct Cyc_Absyn_Exp*),int env,struct Cyc_List_List*x))Cyc_List_map_c;_tmp10A;});int _tmp583=pt;_tmp584(Cyc_Tcutil_deep_copy_exp,_tmp583,es);});Cyc_Absyn_tuple_exp(_tmp585,e->loc);});goto _LL0;}case 21U: _LL33: _tmpC9=((struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct*)_tmp97)->f1;_tmpCA=((struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct*)_tmp97)->f2;_tmpCB=((struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct*)_tmp97)->f3;_tmpCC=((struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct*)_tmp97)->f4;_LL34: {struct Cyc_Absyn_Exp*e1=_tmpC9;struct _fat_ptr*n=_tmpCA;int f1=_tmpCB;int f2=_tmpCC;
# 532
new_e=({void*_tmp587=(void*)({struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct*_tmp10B=_cycalloc(sizeof(*_tmp10B));_tmp10B->tag=21U,({struct Cyc_Absyn_Exp*_tmp586=Cyc_Tcutil_deep_copy_exp(pt,e1);_tmp10B->f1=_tmp586;}),_tmp10B->f2=n,_tmp10B->f3=f1,_tmp10B->f4=f2;_tmp10B;});Cyc_Absyn_new_exp(_tmp587,e->loc);});goto _LL0;}case 22U: _LL35: _tmpC5=((struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct*)_tmp97)->f1;_tmpC6=((struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct*)_tmp97)->f2;_tmpC7=((struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct*)_tmp97)->f3;_tmpC8=((struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct*)_tmp97)->f4;_LL36: {struct Cyc_Absyn_Exp*e1=_tmpC5;struct _fat_ptr*n=_tmpC6;int f1=_tmpC7;int f2=_tmpC8;
# 534
new_e=({void*_tmp589=(void*)({struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct*_tmp10C=_cycalloc(sizeof(*_tmp10C));_tmp10C->tag=22U,({struct Cyc_Absyn_Exp*_tmp588=Cyc_Tcutil_deep_copy_exp(pt,e1);_tmp10C->f1=_tmp588;}),_tmp10C->f2=n,_tmp10C->f3=f1,_tmp10C->f4=f2;_tmp10C;});Cyc_Absyn_new_exp(_tmp589,e->loc);});goto _LL0;}case 23U: _LL37: _tmpC3=((struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct*)_tmp97)->f1;_tmpC4=((struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct*)_tmp97)->f2;_LL38: {struct Cyc_Absyn_Exp*e1=_tmpC3;struct Cyc_Absyn_Exp*e2=_tmpC4;
# 536
new_e=({struct Cyc_Absyn_Exp*_tmp58B=Cyc_Tcutil_deep_copy_exp(pt,e1);struct Cyc_Absyn_Exp*_tmp58A=Cyc_Tcutil_deep_copy_exp(pt,e2);Cyc_Absyn_subscript_exp(_tmp58B,_tmp58A,e->loc);});goto _LL0;}case 26U: _LL39: _tmpC2=((struct Cyc_Absyn_Array_e_Absyn_Raw_exp_struct*)_tmp97)->f1;_LL3A: {struct Cyc_List_List*eds=_tmpC2;
# 538
new_e=({void*_tmp58F=(void*)({struct Cyc_Absyn_Array_e_Absyn_Raw_exp_struct*_tmp10E=_cycalloc(sizeof(*_tmp10E));_tmp10E->tag=26U,({struct Cyc_List_List*_tmp58E=({struct Cyc_List_List*(*_tmp58D)(struct _tuple16*(*f)(int,struct _tuple16*),int env,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp10D)(struct _tuple16*(*f)(int,struct _tuple16*),int env,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct _tuple16*(*f)(int,struct _tuple16*),int env,struct Cyc_List_List*x))Cyc_List_map_c;_tmp10D;});int _tmp58C=pt;_tmp58D(Cyc_Tcutil_copy_eds,_tmp58C,eds);});_tmp10E->f1=_tmp58E;});_tmp10E;});Cyc_Absyn_new_exp(_tmp58F,e->loc);});goto _LL0;}case 31U: _LL3B: _tmpBF=((struct Cyc_Absyn_Datatype_e_Absyn_Raw_exp_struct*)_tmp97)->f1;_tmpC0=((struct Cyc_Absyn_Datatype_e_Absyn_Raw_exp_struct*)_tmp97)->f2;_tmpC1=((struct Cyc_Absyn_Datatype_e_Absyn_Raw_exp_struct*)_tmp97)->f3;_LL3C: {struct Cyc_List_List*es=_tmpBF;struct Cyc_Absyn_Datatypedecl*dtd=_tmpC0;struct Cyc_Absyn_Datatypefield*dtf=_tmpC1;
# 540
new_e=({void*_tmp593=(void*)({struct Cyc_Absyn_Datatype_e_Absyn_Raw_exp_struct*_tmp110=_cycalloc(sizeof(*_tmp110));_tmp110->tag=31U,({struct Cyc_List_List*_tmp592=({struct Cyc_List_List*(*_tmp591)(struct Cyc_Absyn_Exp*(*f)(int,struct Cyc_Absyn_Exp*),int env,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp10F)(struct Cyc_Absyn_Exp*(*f)(int,struct Cyc_Absyn_Exp*),int env,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_Absyn_Exp*(*f)(int,struct Cyc_Absyn_Exp*),int env,struct Cyc_List_List*x))Cyc_List_map_c;_tmp10F;});int _tmp590=pt;_tmp591(Cyc_Tcutil_deep_copy_exp,_tmp590,es);});_tmp110->f1=_tmp592;}),_tmp110->f2=dtd,_tmp110->f3=dtf;_tmp110;});Cyc_Absyn_new_exp(_tmp593,e->loc);});goto _LL0;}case 27U: _LL3D: _tmpBB=((struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct*)_tmp97)->f1;_tmpBC=((struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct*)_tmp97)->f2;_tmpBD=((struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct*)_tmp97)->f3;_tmpBE=((struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct*)_tmp97)->f4;_LL3E: {struct Cyc_Absyn_Vardecl*vd=_tmpBB;struct Cyc_Absyn_Exp*e1=_tmpBC;struct Cyc_Absyn_Exp*e2=_tmpBD;int b=_tmpBE;
# 542
new_e=({void*_tmp596=(void*)({struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct*_tmp111=_cycalloc(sizeof(*_tmp111));_tmp111->tag=27U,_tmp111->f1=vd,({struct Cyc_Absyn_Exp*_tmp595=Cyc_Tcutil_deep_copy_exp(pt,e1);_tmp111->f2=_tmp595;}),({struct Cyc_Absyn_Exp*_tmp594=Cyc_Tcutil_deep_copy_exp(pt,e2);_tmp111->f3=_tmp594;}),_tmp111->f4=b;_tmp111;});Cyc_Absyn_new_exp(_tmp596,e->loc);});goto _LL0;}case 28U: _LL3F: _tmpB8=((struct Cyc_Absyn_ComprehensionNoinit_e_Absyn_Raw_exp_struct*)_tmp97)->f1;_tmpB9=(void*)((struct Cyc_Absyn_ComprehensionNoinit_e_Absyn_Raw_exp_struct*)_tmp97)->f2;_tmpBA=((struct Cyc_Absyn_ComprehensionNoinit_e_Absyn_Raw_exp_struct*)_tmp97)->f3;_LL40: {struct Cyc_Absyn_Exp*e=_tmpB8;void*t=_tmpB9;int b=_tmpBA;
# 544
new_e=({void*_tmp599=(void*)({struct Cyc_Absyn_ComprehensionNoinit_e_Absyn_Raw_exp_struct*_tmp112=_cycalloc(sizeof(*_tmp112));_tmp112->tag=28U,({struct Cyc_Absyn_Exp*_tmp598=Cyc_Tcutil_deep_copy_exp(pt,e);_tmp112->f1=_tmp598;}),({void*_tmp597=Cyc_Tcutil_copy_type(t);_tmp112->f2=_tmp597;}),_tmp112->f3=b;_tmp112;});Cyc_Absyn_new_exp(_tmp599,e->loc);});
goto _LL0;}case 29U: _LL41: _tmpB4=((struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct*)_tmp97)->f1;_tmpB5=((struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct*)_tmp97)->f2;_tmpB6=((struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct*)_tmp97)->f3;_tmpB7=((struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct*)_tmp97)->f4;_LL42: {struct _tuple1*n=_tmpB4;struct Cyc_List_List*ts=_tmpB5;struct Cyc_List_List*eds=_tmpB6;struct Cyc_Absyn_Aggrdecl*agr=_tmpB7;
# 547
new_e=({void*_tmp59E=(void*)({struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct*_tmp114=_cycalloc(sizeof(*_tmp114));_tmp114->tag=29U,_tmp114->f1=n,({struct Cyc_List_List*_tmp59D=Cyc_List_map(Cyc_Tcutil_copy_type,ts);_tmp114->f2=_tmp59D;}),({struct Cyc_List_List*_tmp59C=({struct Cyc_List_List*(*_tmp59B)(struct _tuple16*(*f)(int,struct _tuple16*),int env,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp113)(struct _tuple16*(*f)(int,struct _tuple16*),int env,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct _tuple16*(*f)(int,struct _tuple16*),int env,struct Cyc_List_List*x))Cyc_List_map_c;_tmp113;});int _tmp59A=pt;_tmp59B(Cyc_Tcutil_copy_eds,_tmp59A,eds);});_tmp114->f3=_tmp59C;}),_tmp114->f4=agr;_tmp114;});Cyc_Absyn_new_exp(_tmp59E,e->loc);});
# 549
goto _LL0;}case 30U: _LL43: _tmpB2=(void*)((struct Cyc_Absyn_AnonStruct_e_Absyn_Raw_exp_struct*)_tmp97)->f1;_tmpB3=((struct Cyc_Absyn_AnonStruct_e_Absyn_Raw_exp_struct*)_tmp97)->f2;_LL44: {void*t=_tmpB2;struct Cyc_List_List*eds=_tmpB3;
# 551
new_e=({void*_tmp5A3=(void*)({struct Cyc_Absyn_AnonStruct_e_Absyn_Raw_exp_struct*_tmp116=_cycalloc(sizeof(*_tmp116));_tmp116->tag=30U,({void*_tmp5A2=Cyc_Tcutil_copy_type(t);_tmp116->f1=_tmp5A2;}),({struct Cyc_List_List*_tmp5A1=({struct Cyc_List_List*(*_tmp5A0)(struct _tuple16*(*f)(int,struct _tuple16*),int env,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp115)(struct _tuple16*(*f)(int,struct _tuple16*),int env,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct _tuple16*(*f)(int,struct _tuple16*),int env,struct Cyc_List_List*x))Cyc_List_map_c;_tmp115;});int _tmp59F=pt;_tmp5A0(Cyc_Tcutil_copy_eds,_tmp59F,eds);});_tmp116->f2=_tmp5A1;});_tmp116;});Cyc_Absyn_new_exp(_tmp5A3,e->loc);});
goto _LL0;}case 25U: _LL45: _tmpAE=(((struct Cyc_Absyn_CompoundLit_e_Absyn_Raw_exp_struct*)_tmp97)->f1)->f1;_tmpAF=(((struct Cyc_Absyn_CompoundLit_e_Absyn_Raw_exp_struct*)_tmp97)->f1)->f2;_tmpB0=(((struct Cyc_Absyn_CompoundLit_e_Absyn_Raw_exp_struct*)_tmp97)->f1)->f3;_tmpB1=((struct Cyc_Absyn_CompoundLit_e_Absyn_Raw_exp_struct*)_tmp97)->f2;_LL46: {struct _fat_ptr*vopt=_tmpAE;struct Cyc_Absyn_Tqual tq=_tmpAF;void*t=_tmpB0;struct Cyc_List_List*eds=_tmpB1;
# 554
new_e=({void*_tmp5A9=(void*)({struct Cyc_Absyn_CompoundLit_e_Absyn_Raw_exp_struct*_tmp119=_cycalloc(sizeof(*_tmp119));_tmp119->tag=25U,({struct _tuple9*_tmp5A8=({struct _tuple9*_tmp117=_cycalloc(sizeof(*_tmp117));_tmp117->f1=vopt,_tmp117->f2=tq,({void*_tmp5A7=Cyc_Tcutil_copy_type(t);_tmp117->f3=_tmp5A7;});_tmp117;});_tmp119->f1=_tmp5A8;}),({
struct Cyc_List_List*_tmp5A6=({struct Cyc_List_List*(*_tmp5A5)(struct _tuple16*(*f)(int,struct _tuple16*),int env,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp118)(struct _tuple16*(*f)(int,struct _tuple16*),int env,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct _tuple16*(*f)(int,struct _tuple16*),int env,struct Cyc_List_List*x))Cyc_List_map_c;_tmp118;});int _tmp5A4=pt;_tmp5A5(Cyc_Tcutil_copy_eds,_tmp5A4,eds);});_tmp119->f2=_tmp5A6;});_tmp119;});
# 554
Cyc_Absyn_new_exp(_tmp5A9,e->loc);});
# 556
goto _LL0;}case 32U: _LL47: _tmpAC=((struct Cyc_Absyn_Enum_e_Absyn_Raw_exp_struct*)_tmp97)->f1;_tmpAD=((struct Cyc_Absyn_Enum_e_Absyn_Raw_exp_struct*)_tmp97)->f2;_LL48: {struct Cyc_Absyn_Enumdecl*ed=_tmpAC;struct Cyc_Absyn_Enumfield*ef=_tmpAD;
new_e=e;goto _LL0;}case 33U: _LL49: _tmpAA=(void*)((struct Cyc_Absyn_AnonEnum_e_Absyn_Raw_exp_struct*)_tmp97)->f1;_tmpAB=((struct Cyc_Absyn_AnonEnum_e_Absyn_Raw_exp_struct*)_tmp97)->f2;_LL4A: {void*t=_tmpAA;struct Cyc_Absyn_Enumfield*ef=_tmpAB;
# 559
new_e=({void*_tmp5AB=(void*)({struct Cyc_Absyn_AnonEnum_e_Absyn_Raw_exp_struct*_tmp11A=_cycalloc(sizeof(*_tmp11A));_tmp11A->tag=33U,({void*_tmp5AA=Cyc_Tcutil_copy_type(t);_tmp11A->f1=_tmp5AA;}),_tmp11A->f2=ef;_tmp11A;});Cyc_Absyn_new_exp(_tmp5AB,e->loc);});goto _LL0;}case 35U: _LL4B: _tmpA4=(((struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct*)_tmp97)->f1).is_calloc;_tmpA5=(((struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct*)_tmp97)->f1).rgn;_tmpA6=(((struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct*)_tmp97)->f1).elt_type;_tmpA7=(((struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct*)_tmp97)->f1).num_elts;_tmpA8=(((struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct*)_tmp97)->f1).fat_result;_tmpA9=(((struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct*)_tmp97)->f1).inline_call;_LL4C: {int ic=_tmpA4;struct Cyc_Absyn_Exp*r=_tmpA5;void**t=_tmpA6;struct Cyc_Absyn_Exp*n=_tmpA7;int res=_tmpA8;int inlc=_tmpA9;
# 561
struct Cyc_Absyn_Exp*e2=Cyc_Absyn_copy_exp(e);
struct Cyc_Absyn_Exp*r1=r;if(r != 0)r1=Cyc_Tcutil_deep_copy_exp(pt,r);{void**t1=t;
if(t != 0)t1=({void**_tmp11B=_cycalloc(sizeof(*_tmp11B));({void*_tmp5AC=Cyc_Tcutil_copy_type(*t);*_tmp11B=_tmp5AC;});_tmp11B;});({void*_tmp5AD=(void*)({struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct*_tmp11C=_cycalloc(sizeof(*_tmp11C));
_tmp11C->tag=35U,(_tmp11C->f1).is_calloc=ic,(_tmp11C->f1).rgn=r1,(_tmp11C->f1).elt_type=t1,(_tmp11C->f1).num_elts=n,(_tmp11C->f1).fat_result=res,(_tmp11C->f1).inline_call=inlc;_tmp11C;});
# 563
e2->r=_tmp5AD;});
# 565
new_e=e2;
goto _LL0;}}case 36U: _LL4D: _tmpA2=((struct Cyc_Absyn_Swap_e_Absyn_Raw_exp_struct*)_tmp97)->f1;_tmpA3=((struct Cyc_Absyn_Swap_e_Absyn_Raw_exp_struct*)_tmp97)->f2;_LL4E: {struct Cyc_Absyn_Exp*e1=_tmpA2;struct Cyc_Absyn_Exp*e2=_tmpA3;
new_e=({struct Cyc_Absyn_Exp*_tmp5AF=Cyc_Tcutil_deep_copy_exp(pt,e1);struct Cyc_Absyn_Exp*_tmp5AE=Cyc_Tcutil_deep_copy_exp(pt,e2);Cyc_Absyn_swap_exp(_tmp5AF,_tmp5AE,e->loc);});goto _LL0;}case 37U: _LL4F: _tmpA0=((struct Cyc_Absyn_UnresolvedMem_e_Absyn_Raw_exp_struct*)_tmp97)->f1;_tmpA1=((struct Cyc_Absyn_UnresolvedMem_e_Absyn_Raw_exp_struct*)_tmp97)->f2;_LL50: {struct Cyc_Core_Opt*nopt=_tmpA0;struct Cyc_List_List*eds=_tmpA1;
# 569
struct Cyc_Core_Opt*nopt1=nopt;
if(nopt != 0)nopt1=({struct Cyc_Core_Opt*_tmp11D=_cycalloc(sizeof(*_tmp11D));_tmp11D->v=(struct _tuple1*)nopt->v;_tmp11D;});new_e=({
void*_tmp5B3=(void*)({struct Cyc_Absyn_UnresolvedMem_e_Absyn_Raw_exp_struct*_tmp11F=_cycalloc(sizeof(*_tmp11F));_tmp11F->tag=37U,_tmp11F->f1=nopt1,({struct Cyc_List_List*_tmp5B2=({struct Cyc_List_List*(*_tmp5B1)(struct _tuple16*(*f)(int,struct _tuple16*),int env,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp11E)(struct _tuple16*(*f)(int,struct _tuple16*),int env,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct _tuple16*(*f)(int,struct _tuple16*),int env,struct Cyc_List_List*x))Cyc_List_map_c;_tmp11E;});int _tmp5B0=pt;_tmp5B1(Cyc_Tcutil_copy_eds,_tmp5B0,eds);});_tmp11F->f2=_tmp5B2;});_tmp11F;});Cyc_Absyn_new_exp(_tmp5B3,e->loc);});
goto _LL0;}case 38U: _LL51: _LL52:
# 574
(int)_throw((void*)({struct Cyc_Core_Failure_exn_struct*_tmp121=_cycalloc(sizeof(*_tmp121));_tmp121->tag=Cyc_Core_Failure,({struct _fat_ptr _tmp5B4=({const char*_tmp120="deep_copy: statement expressions unsupported";_tag_fat(_tmp120,sizeof(char),45U);});_tmp121->f1=_tmp5B4;});_tmp121;}));case 39U: _LL53: _tmp9E=((struct Cyc_Absyn_Tagcheck_e_Absyn_Raw_exp_struct*)_tmp97)->f1;_tmp9F=((struct Cyc_Absyn_Tagcheck_e_Absyn_Raw_exp_struct*)_tmp97)->f2;_LL54: {struct Cyc_Absyn_Exp*e1=_tmp9E;struct _fat_ptr*fn=_tmp9F;
new_e=({void*_tmp5B6=(void*)({struct Cyc_Absyn_Tagcheck_e_Absyn_Raw_exp_struct*_tmp122=_cycalloc(sizeof(*_tmp122));_tmp122->tag=39U,({struct Cyc_Absyn_Exp*_tmp5B5=Cyc_Tcutil_deep_copy_exp(pt,e1);_tmp122->f1=_tmp5B5;}),_tmp122->f2=fn;_tmp122;});Cyc_Absyn_new_exp(_tmp5B6,e->loc);});
goto _LL0;}case 40U: _LL55: _tmp9D=(void*)((struct Cyc_Absyn_Valueof_e_Absyn_Raw_exp_struct*)_tmp97)->f1;_LL56: {void*t=_tmp9D;
new_e=({void*_tmp5B7=Cyc_Tcutil_copy_type(t);Cyc_Absyn_valueof_exp(_tmp5B7,e->loc);});
goto _LL0;}default: _LL57: _tmp98=((struct Cyc_Absyn_Asm_e_Absyn_Raw_exp_struct*)_tmp97)->f1;_tmp99=((struct Cyc_Absyn_Asm_e_Absyn_Raw_exp_struct*)_tmp97)->f2;_tmp9A=((struct Cyc_Absyn_Asm_e_Absyn_Raw_exp_struct*)_tmp97)->f3;_tmp9B=((struct Cyc_Absyn_Asm_e_Absyn_Raw_exp_struct*)_tmp97)->f4;_tmp9C=((struct Cyc_Absyn_Asm_e_Absyn_Raw_exp_struct*)_tmp97)->f5;_LL58: {int v=_tmp98;struct _fat_ptr t=_tmp99;struct Cyc_List_List*o=_tmp9A;struct Cyc_List_List*i=_tmp9B;struct Cyc_List_List*c=_tmp9C;
new_e=Cyc_Absyn_asm_exp(v,t,o,i,c,e->loc);goto _LL0;}}_LL0:;}
# 582
if(preserve_types){
new_e->topt=e->topt;
new_e->annot=e->annot;}
# 582
return new_e;}struct _tuple17{enum Cyc_Absyn_KindQual f1;enum Cyc_Absyn_KindQual f2;};struct _tuple18{enum Cyc_Absyn_AliasQual f1;enum Cyc_Absyn_AliasQual f2;};
# 353
int Cyc_Tcutil_kind_leq(struct Cyc_Absyn_Kind*ka1,struct Cyc_Absyn_Kind*ka2){
# 598 "tcutil.cyc"
enum Cyc_Absyn_AliasQual _tmp124;enum Cyc_Absyn_KindQual _tmp123;_LL1: _tmp123=ka1->kind;_tmp124=ka1->aliasqual;_LL2: {enum Cyc_Absyn_KindQual k1=_tmp123;enum Cyc_Absyn_AliasQual a1=_tmp124;
enum Cyc_Absyn_AliasQual _tmp126;enum Cyc_Absyn_KindQual _tmp125;_LL4: _tmp125=ka2->kind;_tmp126=ka2->aliasqual;_LL5: {enum Cyc_Absyn_KindQual k2=_tmp125;enum Cyc_Absyn_AliasQual a2=_tmp126;
# 601
if((int)k1 != (int)k2){
struct _tuple17 _stmttmp1B=({struct _tuple17 _tmp4DF;_tmp4DF.f1=k1,_tmp4DF.f2=k2;_tmp4DF;});struct _tuple17 _tmp127=_stmttmp1B;switch(_tmp127.f1){case Cyc_Absyn_BoxKind: switch(_tmp127.f2){case Cyc_Absyn_MemKind: _LL7: _LL8:
 goto _LLA;case Cyc_Absyn_AnyKind: _LL9: _LLA:
 goto _LLC;default: goto _LLD;}case Cyc_Absyn_MemKind: if(_tmp127.f2 == Cyc_Absyn_AnyKind){_LLB: _LLC:
 goto _LL6;}else{goto _LLD;}default: _LLD: _LLE:
 return 0;}_LL6:;}
# 601
if((int)a1 != (int)a2){
# 610
struct _tuple18 _stmttmp1C=({struct _tuple18 _tmp4E0;_tmp4E0.f1=a1,_tmp4E0.f2=a2;_tmp4E0;});struct _tuple18 _tmp128=_stmttmp1C;switch(_tmp128.f1){case Cyc_Absyn_Aliasable: if(_tmp128.f2 == Cyc_Absyn_Top){_LL10: _LL11:
 goto _LL13;}else{goto _LL14;}case Cyc_Absyn_Unique: if(_tmp128.f2 == Cyc_Absyn_Top){_LL12: _LL13:
 return 1;}else{goto _LL14;}default: _LL14: _LL15:
 return 0;}_LLF:;}
# 601
return 1;}}}
# 353 "tcutil.cyc"
struct Cyc_Absyn_Kind*Cyc_Tcutil_tvar_kind(struct Cyc_Absyn_Tvar*tv,struct Cyc_Absyn_Kind*def){
# 619 "tcutil.cyc"
void*_stmttmp1D=Cyc_Absyn_compress_kb(tv->kind);void*_tmp129=_stmttmp1D;struct Cyc_Absyn_Kind*_tmp12A;struct Cyc_Absyn_Kind*_tmp12B;switch(*((int*)_tmp129)){case 0U: _LL1: _tmp12B=((struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct*)_tmp129)->f1;_LL2: {struct Cyc_Absyn_Kind*k=_tmp12B;
return k;}case 2U: _LL3: _tmp12A=((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp129)->f2;_LL4: {struct Cyc_Absyn_Kind*k=_tmp12A;
return k;}default: _LL5: _LL6:
({void*_tmp5B8=(void*)({struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*_tmp12C=_cycalloc(sizeof(*_tmp12C));_tmp12C->tag=2U,_tmp12C->f1=0,_tmp12C->f2=def;_tmp12C;});tv->kind=_tmp5B8;});return def;}_LL0:;}struct _tuple19{struct Cyc_Absyn_Tvar*f1;void*f2;};
# 353 "tcutil.cyc"
struct _tuple19 Cyc_Tcutil_swap_kind(void*t,void*kb){
# 627 "tcutil.cyc"
void*_stmttmp1E=Cyc_Tcutil_compress(t);void*_tmp12D=_stmttmp1E;struct Cyc_Absyn_Tvar*_tmp12E;if(((struct Cyc_Absyn_VarType_Absyn_Type_struct*)_tmp12D)->tag == 2U){_LL1: _tmp12E=((struct Cyc_Absyn_VarType_Absyn_Type_struct*)_tmp12D)->f1;_LL2: {struct Cyc_Absyn_Tvar*tv=_tmp12E;
# 629
void*oldkb=tv->kind;
tv->kind=kb;
return({struct _tuple19 _tmp4E1;_tmp4E1.f1=tv,_tmp4E1.f2=oldkb;_tmp4E1;});}}else{_LL3: _LL4:
({struct Cyc_String_pa_PrintArg_struct _tmp132=({struct Cyc_String_pa_PrintArg_struct _tmp4E2;_tmp4E2.tag=0U,({struct _fat_ptr _tmp5B9=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t));_tmp4E2.f1=_tmp5B9;});_tmp4E2;});void*_tmp12F[1U];_tmp12F[0]=& _tmp132;({int(*_tmp5BB)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp131)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Tcutil_impos;_tmp131;});struct _fat_ptr _tmp5BA=({const char*_tmp130="swap_kind: cannot update the kind of %s";_tag_fat(_tmp130,sizeof(char),40U);});_tmp5BB(_tmp5BA,_tag_fat(_tmp12F,sizeof(void*),1U));});});}_LL0:;}
# 353 "tcutil.cyc"
static struct Cyc_Absyn_Kind*Cyc_Tcutil_field_kind(void*field_type,struct Cyc_List_List*ts,struct Cyc_List_List*tvs){
# 640 "tcutil.cyc"
struct Cyc_Absyn_Kind*k=Cyc_Tcutil_type_kind(field_type);
if(ts != 0 &&(k == & Cyc_Tcutil_ak || k == & Cyc_Tcutil_tak)){
# 644
struct _RegionHandle _tmp133=_new_region("temp");struct _RegionHandle*temp=& _tmp133;_push_region(temp);
{struct Cyc_List_List*inst=0;
# 647
for(0;tvs != 0;(tvs=tvs->tl,ts=ts->tl)){
struct Cyc_Absyn_Tvar*tv=(struct Cyc_Absyn_Tvar*)tvs->hd;
void*t=(void*)((struct Cyc_List_List*)_check_null(ts))->hd;
struct Cyc_Absyn_Kind*_stmttmp1F=Cyc_Tcutil_tvar_kind(tv,& Cyc_Tcutil_bk);struct Cyc_Absyn_Kind*_tmp134=_stmttmp1F;switch(((struct Cyc_Absyn_Kind*)_tmp134)->kind){case Cyc_Absyn_IntKind: _LL1: _LL2:
 goto _LL4;case Cyc_Absyn_AnyKind: _LL3: _LL4:
# 653
 inst=({struct Cyc_List_List*_tmp136=_region_malloc(temp,sizeof(*_tmp136));({struct _tuple19*_tmp5BC=({struct _tuple19*_tmp135=_region_malloc(temp,sizeof(*_tmp135));_tmp135->f1=tv,_tmp135->f2=t;_tmp135;});_tmp136->hd=_tmp5BC;}),_tmp136->tl=inst;_tmp136;});goto _LL0;default: _LL5: _LL6:
 goto _LL0;}_LL0:;}
# 657
if(inst != 0){
field_type=({struct _RegionHandle*_tmp5BE=temp;struct Cyc_List_List*_tmp5BD=Cyc_List_imp_rev(inst);Cyc_Tcutil_rsubstitute(_tmp5BE,_tmp5BD,field_type);});
k=Cyc_Tcutil_type_kind(field_type);}}
# 645
;_pop_region(temp);}
# 641
return k;}
# 353 "tcutil.cyc"
struct Cyc_Absyn_Kind*Cyc_Tcutil_type_kind(void*t){
# 671 "tcutil.cyc"
void*_stmttmp20=Cyc_Tcutil_compress(t);void*_tmp137=_stmttmp20;struct Cyc_Absyn_Typedefdecl*_tmp138;struct Cyc_Absyn_Exp*_tmp139;struct Cyc_Absyn_PtrInfo _tmp13A;struct Cyc_List_List*_tmp13C;void*_tmp13B;struct Cyc_Absyn_Tvar*_tmp13D;struct Cyc_Core_Opt*_tmp13E;switch(*((int*)_tmp137)){case 1U: _LL1: _tmp13E=((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp137)->f1;_LL2: {struct Cyc_Core_Opt*k=_tmp13E;
return(struct Cyc_Absyn_Kind*)((struct Cyc_Core_Opt*)_check_null(k))->v;}case 2U: _LL3: _tmp13D=((struct Cyc_Absyn_VarType_Absyn_Type_struct*)_tmp137)->f1;_LL4: {struct Cyc_Absyn_Tvar*tv=_tmp13D;
return Cyc_Tcutil_tvar_kind(tv,& Cyc_Tcutil_bk);}case 0U: _LL5: _tmp13B=(void*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp137)->f1;_tmp13C=((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp137)->f2;_LL6: {void*c=_tmp13B;struct Cyc_List_List*ts=_tmp13C;
# 675
void*_tmp13F=c;int _tmp143;struct Cyc_Absyn_AggrdeclImpl*_tmp142;struct Cyc_List_List*_tmp141;enum Cyc_Absyn_AggrKind _tmp140;struct Cyc_Absyn_Kind*_tmp144;enum Cyc_Absyn_Size_of _tmp145;switch(*((int*)_tmp13F)){case 0U: _LL1E: _LL1F:
 return& Cyc_Tcutil_mk;case 1U: _LL20: _tmp145=((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)_tmp13F)->f2;_LL21: {enum Cyc_Absyn_Size_of sz=_tmp145;
return((int)sz == (int)2U ||(int)sz == (int)3U)?& Cyc_Tcutil_bk:& Cyc_Tcutil_mk;}case 2U: _LL22: _LL23:
 return& Cyc_Tcutil_mk;case 15U: _LL24: _LL25:
 goto _LL27;case 16U: _LL26: _LL27:
 goto _LL29;case 3U: _LL28: _LL29:
 return& Cyc_Tcutil_bk;case 6U: _LL2A: _LL2B:
 return& Cyc_Tcutil_urk;case 5U: _LL2C: _LL2D:
 return& Cyc_Tcutil_rk;case 7U: _LL2E: _LL2F:
 return& Cyc_Tcutil_trk;case 17U: _LL30: _tmp144=((struct Cyc_Absyn_BuiltinCon_Absyn_TyCon_struct*)_tmp13F)->f2;_LL31: {struct Cyc_Absyn_Kind*k=_tmp144;
return k;}case 4U: _LL32: _LL33:
 return& Cyc_Tcutil_bk;case 8U: _LL34: _LL35:
 goto _LL37;case 9U: _LL36: _LL37:
 goto _LL39;case 10U: _LL38: _LL39:
 return& Cyc_Tcutil_ek;case 12U: _LL3A: _LL3B:
 goto _LL3D;case 11U: _LL3C: _LL3D:
 return& Cyc_Tcutil_boolk;case 13U: _LL3E: _LL3F:
 goto _LL41;case 14U: _LL40: _LL41:
 return& Cyc_Tcutil_ptrbk;case 18U: _LL42: _LL43:
 return& Cyc_Tcutil_ak;case 19U: if(((((struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct*)_tmp13F)->f1).KnownDatatypefield).tag == 2){_LL44: _LL45:
# 696
 return& Cyc_Tcutil_mk;}else{_LL46: _LL47:
# 698
({void*_tmp146=0U;({int(*_tmp5C0)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp148)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Tcutil_impos;_tmp148;});struct _fat_ptr _tmp5BF=({const char*_tmp147="type_kind: Unresolved DatatypeFieldType";_tag_fat(_tmp147,sizeof(char),40U);});_tmp5C0(_tmp5BF,_tag_fat(_tmp146,sizeof(void*),0U));});});}default: if(((((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)_tmp13F)->f1).UnknownAggr).tag == 1){_LL48: _LL49:
# 702
 return& Cyc_Tcutil_ak;}else{_LL4A: _tmp140=(*((((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)_tmp13F)->f1).KnownAggr).val)->kind;_tmp141=(*((((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)_tmp13F)->f1).KnownAggr).val)->tvs;_tmp142=(*((((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)_tmp13F)->f1).KnownAggr).val)->impl;_tmp143=(*((((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)_tmp13F)->f1).KnownAggr).val)->expected_mem_kind;_LL4B: {enum Cyc_Absyn_AggrKind strOrU=_tmp140;struct Cyc_List_List*tvs=_tmp141;struct Cyc_Absyn_AggrdeclImpl*i=_tmp142;int expected_mem_kind=_tmp143;
# 704
if(i == 0){
if(expected_mem_kind)
return& Cyc_Tcutil_mk;else{
# 708
return& Cyc_Tcutil_ak;}}{
# 704
struct Cyc_List_List*fields=i->fields;
# 711
if(fields == 0)return& Cyc_Tcutil_mk;if((int)strOrU == (int)0U){
# 714
for(0;((struct Cyc_List_List*)_check_null(fields))->tl != 0;fields=fields->tl){;}{
void*last_type=((struct Cyc_Absyn_Aggrfield*)fields->hd)->type;
struct Cyc_Absyn_Kind*k=Cyc_Tcutil_field_kind(last_type,ts,tvs);
if(k == & Cyc_Tcutil_ak || k == & Cyc_Tcutil_tak)return k;}}else{
# 721
for(0;fields != 0;fields=fields->tl){
void*type=((struct Cyc_Absyn_Aggrfield*)fields->hd)->type;
struct Cyc_Absyn_Kind*k=Cyc_Tcutil_field_kind(type,ts,tvs);
if(k == & Cyc_Tcutil_ak || k == & Cyc_Tcutil_tak)return k;}}
# 727
return& Cyc_Tcutil_mk;}}}}_LL1D:;}case 5U: _LL7: _LL8:
# 729
 return& Cyc_Tcutil_ak;case 7U: _LL9: _LLA:
 return& Cyc_Tcutil_mk;case 3U: _LLB: _tmp13A=((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp137)->f1;_LLC: {struct Cyc_Absyn_PtrInfo pinfo=_tmp13A;
# 732
void*_stmttmp21=Cyc_Tcutil_compress((pinfo.ptr_atts).bounds);void*_tmp149=_stmttmp21;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp149)->tag == 0U)switch(*((int*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp149)->f1)){case 13U: _LL4D: _LL4E: {
# 734
enum Cyc_Absyn_AliasQual _stmttmp22=(Cyc_Tcutil_type_kind((pinfo.ptr_atts).rgn))->aliasqual;enum Cyc_Absyn_AliasQual _tmp14A=_stmttmp22;switch(_tmp14A){case Cyc_Absyn_Aliasable: _LL54: _LL55:
 return& Cyc_Tcutil_bk;case Cyc_Absyn_Unique: _LL56: _LL57:
 return& Cyc_Tcutil_ubk;case Cyc_Absyn_Top: _LL58: _LL59:
 goto _LL5B;default: _LL5A: _LL5B: return& Cyc_Tcutil_tbk;}_LL53:;}case 14U: _LL4F: _LL50:
# 740
 goto _LL52;default: goto _LL51;}else{_LL51: _LL52: {
# 742
enum Cyc_Absyn_AliasQual _stmttmp23=(Cyc_Tcutil_type_kind((pinfo.ptr_atts).rgn))->aliasqual;enum Cyc_Absyn_AliasQual _tmp14B=_stmttmp23;switch(_tmp14B){case Cyc_Absyn_Aliasable: _LL5D: _LL5E:
 return& Cyc_Tcutil_mk;case Cyc_Absyn_Unique: _LL5F: _LL60:
 return& Cyc_Tcutil_umk;case Cyc_Absyn_Top: _LL61: _LL62:
 goto _LL64;default: _LL63: _LL64: return& Cyc_Tcutil_tmk;}_LL5C:;}}_LL4C:;}case 9U: _LLD: _LLE:
# 748
 return& Cyc_Tcutil_ik;case 11U: _LLF: _LL10:
# 752
 return& Cyc_Tcutil_ak;case 4U: _LL11: _tmp139=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp137)->f1).num_elts;_LL12: {struct Cyc_Absyn_Exp*num_elts=_tmp139;
# 754
if(num_elts == 0 || Cyc_Tcutil_is_const_exp(num_elts))return& Cyc_Tcutil_mk;return& Cyc_Tcutil_ak;}case 6U: _LL13: _LL14:
# 756
 return& Cyc_Tcutil_mk;case 8U: _LL15: _tmp138=((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmp137)->f3;_LL16: {struct Cyc_Absyn_Typedefdecl*td=_tmp138;
# 758
if(td == 0 || td->kind == 0)
({struct Cyc_String_pa_PrintArg_struct _tmp14F=({struct Cyc_String_pa_PrintArg_struct _tmp4E3;_tmp4E3.tag=0U,({struct _fat_ptr _tmp5C1=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t));_tmp4E3.f1=_tmp5C1;});_tmp4E3;});void*_tmp14C[1U];_tmp14C[0]=& _tmp14F;({int(*_tmp5C3)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp14E)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Tcutil_impos;_tmp14E;});struct _fat_ptr _tmp5C2=({const char*_tmp14D="type_kind: typedef found: %s";_tag_fat(_tmp14D,sizeof(char),29U);});_tmp5C3(_tmp5C2,_tag_fat(_tmp14C,sizeof(void*),1U));});});
# 758
return(struct Cyc_Absyn_Kind*)((struct Cyc_Core_Opt*)_check_null(td->kind))->v;}default: switch(*((int*)((struct Cyc_Absyn_TypeDecl*)((struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct*)_tmp137)->f1)->r)){case 0U: _LL17: _LL18:
# 761
 return& Cyc_Tcutil_ak;case 1U: _LL19: _LL1A:
 return& Cyc_Tcutil_bk;default: _LL1B: _LL1C:
 return& Cyc_Tcutil_ak;}}_LL0:;}
# 353 "tcutil.cyc"
int Cyc_Tcutil_kind_eq(struct Cyc_Absyn_Kind*k1,struct Cyc_Absyn_Kind*k2){
# 768 "tcutil.cyc"
return k1 == k2 ||(int)k1->kind == (int)k2->kind &&(int)k1->aliasqual == (int)k2->aliasqual;}
# 353 "tcutil.cyc"
int Cyc_Tcutil_same_atts(struct Cyc_List_List*a1,struct Cyc_List_List*a2){
# 772 "tcutil.cyc"
{struct Cyc_List_List*a=a1;for(0;a != 0;a=a->tl){
if(!Cyc_List_exists_c(Cyc_Absyn_equal_att,(void*)a->hd,a2))return 0;}}
# 772
{
# 774
struct Cyc_List_List*a=a2;
# 772
for(0;a != 0;a=a->tl){
# 775
if(!Cyc_List_exists_c(Cyc_Absyn_equal_att,(void*)a->hd,a1))return 0;}}
# 772
return 1;}
# 353 "tcutil.cyc"
int Cyc_Tcutil_is_regparm0_att(void*a){
# 780 "tcutil.cyc"
void*_tmp150=a;if(((struct Cyc_Absyn_Regparm_att_Absyn_Attribute_struct*)_tmp150)->tag == 0U){if(((struct Cyc_Absyn_Regparm_att_Absyn_Attribute_struct*)_tmp150)->f1 == 0){_LL1: _LL2:
 return 1;}else{goto _LL3;}}else{_LL3: _LL4:
 return 0;}_LL0:;}
# 353 "tcutil.cyc"
int Cyc_Tcutil_equiv_fn_atts(struct Cyc_List_List*a1,struct Cyc_List_List*a2){
# 787 "tcutil.cyc"
{struct Cyc_List_List*a=a1;for(0;a != 0;a=a->tl){
if(!Cyc_List_exists_c(Cyc_Absyn_equal_att,(void*)a->hd,a2)&& !Cyc_Tcutil_is_regparm0_att((void*)a->hd))return 0;}}
# 787
{
# 789
struct Cyc_List_List*a=a2;
# 787
for(0;a != 0;a=a->tl){
# 790
if(!Cyc_List_exists_c(Cyc_Absyn_equal_att,(void*)a->hd,a1)&& !Cyc_Tcutil_is_regparm0_att((void*)a->hd))return 0;}}
# 787
return 1;}
# 353 "tcutil.cyc"
static void*Cyc_Tcutil_rgns_of(void*t);
# 797 "tcutil.cyc"
static void*Cyc_Tcutil_rgns_of_field(struct Cyc_Absyn_Aggrfield*af){
return Cyc_Tcutil_rgns_of(af->type);}
# 797
static struct _tuple19*Cyc_Tcutil_region_free_subst(struct Cyc_Absyn_Tvar*tv){
# 802
void*t;
{struct Cyc_Absyn_Kind*_stmttmp24=Cyc_Tcutil_tvar_kind(tv,& Cyc_Tcutil_bk);struct Cyc_Absyn_Kind*_tmp151=_stmttmp24;switch(((struct Cyc_Absyn_Kind*)_tmp151)->kind){case Cyc_Absyn_RgnKind: switch(((struct Cyc_Absyn_Kind*)_tmp151)->aliasqual){case Cyc_Absyn_Unique: _LL1: _LL2:
 t=Cyc_Absyn_unique_rgn_type;goto _LL0;case Cyc_Absyn_Aliasable: _LL3: _LL4:
 t=Cyc_Absyn_heap_rgn_type;goto _LL0;default: goto _LLD;}case Cyc_Absyn_EffKind: _LL5: _LL6:
 t=Cyc_Absyn_empty_effect;goto _LL0;case Cyc_Absyn_IntKind: _LL7: _LL8:
 t=(void*)({struct Cyc_Absyn_ValueofType_Absyn_Type_struct*_tmp152=_cycalloc(sizeof(*_tmp152));_tmp152->tag=9U,({struct Cyc_Absyn_Exp*_tmp5C4=Cyc_Absyn_uint_exp(0U,0U);_tmp152->f1=_tmp5C4;});_tmp152;});goto _LL0;case Cyc_Absyn_BoolKind: _LL9: _LLA:
 t=Cyc_Absyn_true_type;goto _LL0;case Cyc_Absyn_PtrBndKind: _LLB: _LLC:
 t=Cyc_Absyn_fat_bound_type;goto _LL0;default: _LLD: _LLE:
 t=Cyc_Absyn_sint_type;goto _LL0;}_LL0:;}
# 812
return({struct _tuple19*_tmp153=_cycalloc(sizeof(*_tmp153));_tmp153->f1=tv,_tmp153->f2=t;_tmp153;});}
# 797
static void*Cyc_Tcutil_rgns_of(void*t){
# 820
void*_stmttmp25=Cyc_Tcutil_compress(t);void*_tmp154=_stmttmp25;struct Cyc_List_List*_tmp155;struct Cyc_List_List*_tmp156;struct Cyc_List_List*_tmp15D;struct Cyc_Absyn_VarargInfo*_tmp15C;struct Cyc_List_List*_tmp15B;void*_tmp15A;struct Cyc_Absyn_Tqual _tmp159;void*_tmp158;struct Cyc_List_List*_tmp157;struct Cyc_List_List*_tmp15E;void*_tmp15F;void*_tmp161;void*_tmp160;struct Cyc_List_List*_tmp162;switch(*((int*)_tmp154)){case 0U: if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp154)->f2 == 0){_LL1: _LL2:
 return Cyc_Absyn_empty_effect;}else{if(((struct Cyc_Absyn_JoinCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp154)->f1)->tag == 9U){_LL3: _LL4:
 return t;}else{_LL5: _tmp162=((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp154)->f2;_LL6: {struct Cyc_List_List*ts=_tmp162;
# 824
struct Cyc_List_List*new_ts=Cyc_List_map(Cyc_Tcutil_rgns_of,ts);
return Cyc_Tcutil_normalize_effect(Cyc_Absyn_join_eff(new_ts));}}}case 1U: _LL7: _LL8:
 goto _LLA;case 2U: _LL9: _LLA: {
# 828
struct Cyc_Absyn_Kind*_stmttmp26=Cyc_Tcutil_type_kind(t);struct Cyc_Absyn_Kind*_tmp163=_stmttmp26;switch(((struct Cyc_Absyn_Kind*)_tmp163)->kind){case Cyc_Absyn_RgnKind: _LL1E: _LL1F:
 return Cyc_Absyn_access_eff(t);case Cyc_Absyn_EffKind: _LL20: _LL21:
 return t;case Cyc_Absyn_IntKind: _LL22: _LL23:
 return Cyc_Absyn_empty_effect;default: _LL24: _LL25:
 return Cyc_Absyn_regionsof_eff(t);}_LL1D:;}case 3U: _LLB: _tmp160=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp154)->f1).elt_type;_tmp161=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp154)->f1).ptr_atts).rgn;_LLC: {void*et=_tmp160;void*r=_tmp161;
# 836
return Cyc_Tcutil_normalize_effect(Cyc_Absyn_join_eff(({void*_tmp164[2U];({void*_tmp5C6=Cyc_Absyn_access_eff(r);_tmp164[0]=_tmp5C6;}),({void*_tmp5C5=Cyc_Tcutil_rgns_of(et);_tmp164[1]=_tmp5C5;});Cyc_List_list(_tag_fat(_tmp164,sizeof(void*),2U));})));}case 4U: _LLD: _tmp15F=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp154)->f1).elt_type;_LLE: {void*et=_tmp15F;
# 838
return Cyc_Tcutil_normalize_effect(Cyc_Tcutil_rgns_of(et));}case 7U: _LLF: _tmp15E=((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmp154)->f2;_LL10: {struct Cyc_List_List*fs=_tmp15E;
# 840
return Cyc_Tcutil_normalize_effect(Cyc_Absyn_join_eff(({struct Cyc_List_List*(*_tmp5C7)(void*(*f)(struct Cyc_Absyn_Aggrfield*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp165)(void*(*f)(struct Cyc_Absyn_Aggrfield*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(void*(*f)(struct Cyc_Absyn_Aggrfield*),struct Cyc_List_List*x))Cyc_List_map;_tmp165;});_tmp5C7(Cyc_Tcutil_rgns_of_field,fs);})));}case 5U: _LL11: _tmp157=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp154)->f1).tvars;_tmp158=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp154)->f1).effect;_tmp159=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp154)->f1).ret_tqual;_tmp15A=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp154)->f1).ret_type;_tmp15B=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp154)->f1).args;_tmp15C=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp154)->f1).cyc_varargs;_tmp15D=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp154)->f1).rgn_po;_LL12: {struct Cyc_List_List*tvs=_tmp157;void*eff=_tmp158;struct Cyc_Absyn_Tqual rt_tq=_tmp159;void*rt=_tmp15A;struct Cyc_List_List*args=_tmp15B;struct Cyc_Absyn_VarargInfo*cyc_varargs=_tmp15C;struct Cyc_List_List*rpo=_tmp15D;
# 849
void*e=({struct Cyc_List_List*_tmp5C9=({struct Cyc_List_List*(*_tmp5C8)(struct _tuple19*(*f)(struct Cyc_Absyn_Tvar*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp166)(struct _tuple19*(*f)(struct Cyc_Absyn_Tvar*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct _tuple19*(*f)(struct Cyc_Absyn_Tvar*),struct Cyc_List_List*x))Cyc_List_map;_tmp166;});_tmp5C8(Cyc_Tcutil_region_free_subst,tvs);});Cyc_Tcutil_substitute(_tmp5C9,(void*)_check_null(eff));});
return Cyc_Tcutil_normalize_effect(e);}case 6U: _LL13: _tmp156=((struct Cyc_Absyn_TupleType_Absyn_Type_struct*)_tmp154)->f1;_LL14: {struct Cyc_List_List*tqts=_tmp156;
# 852
struct Cyc_List_List*ts=0;
for(0;tqts != 0;tqts=tqts->tl){
ts=({struct Cyc_List_List*_tmp167=_cycalloc(sizeof(*_tmp167));_tmp167->hd=(*((struct _tuple12*)tqts->hd)).f2,_tmp167->tl=ts;_tmp167;});}
_tmp155=ts;goto _LL16;}case 8U: _LL15: _tmp155=((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmp154)->f2;_LL16: {struct Cyc_List_List*ts=_tmp155;
# 857
return Cyc_Tcutil_normalize_effect(Cyc_Absyn_join_eff(Cyc_List_map(Cyc_Tcutil_rgns_of,ts)));}case 10U: _LL17: _LL18:
({void*_tmp168=0U;({int(*_tmp5CB)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp16A)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Tcutil_impos;_tmp16A;});struct _fat_ptr _tmp5CA=({const char*_tmp169="typedecl in rgns_of";_tag_fat(_tmp169,sizeof(char),20U);});_tmp5CB(_tmp5CA,_tag_fat(_tmp168,sizeof(void*),0U));});});case 9U: _LL19: _LL1A:
 goto _LL1C;default: _LL1B: _LL1C:
 return Cyc_Absyn_empty_effect;}_LL0:;}
# 797
void*Cyc_Tcutil_normalize_effect(void*e){
# 868
e=Cyc_Tcutil_compress(e);{
void*_tmp16B=e;void*_tmp16C;struct Cyc_List_List**_tmp16D;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp16B)->tag == 0U)switch(*((int*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp16B)->f1)){case 9U: _LL1: _tmp16D=(struct Cyc_List_List**)&((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp16B)->f2;_LL2: {struct Cyc_List_List**es=_tmp16D;
# 871
int redo_join=0;
{struct Cyc_List_List*effs=*es;for(0;effs != 0;effs=effs->tl){
void*eff=(void*)effs->hd;
({void*_tmp5CC=(void*)Cyc_Tcutil_compress(Cyc_Tcutil_normalize_effect(eff));effs->hd=_tmp5CC;});{
void*_stmttmp27=(void*)effs->hd;void*_tmp16E=_stmttmp27;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp16E)->tag == 0U)switch(*((int*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp16E)->f1)){case 9U: _LL8: _LL9:
 goto _LLB;case 8U: if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp16E)->f2 != 0){if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)((struct Cyc_List_List*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp16E)->f2)->hd)->tag == 0U)switch(*((int*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)((struct Cyc_List_List*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp16E)->f2)->hd)->f1)){case 5U: if(((struct Cyc_List_List*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp16E)->f2)->tl == 0){_LLA: _LLB:
 goto _LLD;}else{goto _LL10;}case 7U: if(((struct Cyc_List_List*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp16E)->f2)->tl == 0){_LLC: _LLD:
 goto _LLF;}else{goto _LL10;}case 6U: if(((struct Cyc_List_List*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp16E)->f2)->tl == 0){_LLE: _LLF:
# 880
 redo_join=1;goto _LL7;}else{goto _LL10;}default: goto _LL10;}else{goto _LL10;}}else{goto _LL10;}default: goto _LL10;}else{_LL10: _LL11:
 goto _LL7;}_LL7:;}}}
# 884
if(!redo_join)return e;{struct Cyc_List_List*effects=0;
# 886
{struct Cyc_List_List*effs=*es;for(0;effs != 0;effs=effs->tl){
void*_stmttmp28=Cyc_Tcutil_compress((void*)effs->hd);void*_tmp16F=_stmttmp28;void*_tmp170;struct Cyc_List_List*_tmp171;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp16F)->tag == 0U)switch(*((int*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp16F)->f1)){case 9U: _LL13: _tmp171=((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp16F)->f2;_LL14: {struct Cyc_List_List*nested_effs=_tmp171;
# 889
effects=Cyc_List_revappend(nested_effs,effects);
goto _LL12;}case 8U: if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp16F)->f2 != 0){if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)((struct Cyc_List_List*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp16F)->f2)->hd)->tag == 0U)switch(*((int*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)((struct Cyc_List_List*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp16F)->f2)->hd)->f1)){case 5U: if(((struct Cyc_List_List*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp16F)->f2)->tl == 0){_LL15: _LL16:
 goto _LL18;}else{goto _LL1B;}case 7U: if(((struct Cyc_List_List*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp16F)->f2)->tl == 0){_LL17: _LL18:
 goto _LL1A;}else{goto _LL1B;}case 6U: if(((struct Cyc_List_List*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp16F)->f2)->tl == 0){_LL19: _LL1A:
 goto _LL12;}else{goto _LL1B;}default: goto _LL1B;}else{goto _LL1B;}}else{goto _LL1B;}default: goto _LL1B;}else{_LL1B: _tmp170=_tmp16F;_LL1C: {void*e=_tmp170;
effects=({struct Cyc_List_List*_tmp172=_cycalloc(sizeof(*_tmp172));_tmp172->hd=e,_tmp172->tl=effects;_tmp172;});goto _LL12;}}_LL12:;}}
# 897
({struct Cyc_List_List*_tmp5CD=Cyc_List_imp_rev(effects);*es=_tmp5CD;});
return e;}}case 10U: if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp16B)->f2 != 0){_LL3: _tmp16C=(void*)(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp16B)->f2)->hd;_LL4: {void*t=_tmp16C;
# 900
void*_stmttmp29=Cyc_Tcutil_compress(t);void*_tmp173=_stmttmp29;switch(*((int*)_tmp173)){case 1U: _LL1E: _LL1F:
 goto _LL21;case 2U: _LL20: _LL21:
 return e;default: _LL22: _LL23:
 return Cyc_Tcutil_rgns_of(t);}_LL1D:;}}else{goto _LL5;}default: goto _LL5;}else{_LL5: _LL6:
# 905
 return e;}_LL0:;}}
# 797
static void*Cyc_Tcutil_dummy_fntype(void*eff){
# 911
struct Cyc_Absyn_FnType_Absyn_Type_struct*fntype=({struct Cyc_Absyn_FnType_Absyn_Type_struct*_tmp174=_cycalloc(sizeof(*_tmp174));_tmp174->tag=5U,(_tmp174->f1).tvars=0,(_tmp174->f1).effect=eff,({
struct Cyc_Absyn_Tqual _tmp5CE=Cyc_Absyn_empty_tqual(0U);(_tmp174->f1).ret_tqual=_tmp5CE;}),(_tmp174->f1).ret_type=Cyc_Absyn_void_type,(_tmp174->f1).args=0,(_tmp174->f1).c_varargs=0,(_tmp174->f1).cyc_varargs=0,(_tmp174->f1).rgn_po=0,(_tmp174->f1).attributes=0,(_tmp174->f1).requires_clause=0,(_tmp174->f1).requires_relns=0,(_tmp174->f1).ensures_clause=0,(_tmp174->f1).ensures_relns=0,(_tmp174->f1).ieffect=0,(_tmp174->f1).oeffect=0;_tmp174;});
# 922
return({void*_tmp5D2=(void*)fntype;void*_tmp5D1=Cyc_Absyn_heap_rgn_type;struct Cyc_Absyn_Tqual _tmp5D0=Cyc_Absyn_empty_tqual(0U);void*_tmp5CF=Cyc_Absyn_bounds_one();Cyc_Absyn_atb_type(_tmp5D2,_tmp5D1,_tmp5D0,_tmp5CF,Cyc_Absyn_false_type);});}
# 797
int Cyc_Tcutil_region_in_effect(int constrain,void*r,void*e){
# 930
r=Cyc_Tcutil_compress(r);
if((r == Cyc_Absyn_heap_rgn_type || r == Cyc_Absyn_unique_rgn_type)|| r == Cyc_Absyn_refcnt_rgn_type)
return 1;{
# 931
void*_stmttmp2A=
# 933
Cyc_Tcutil_compress(e);
# 931
void*_tmp175=_stmttmp2A;struct Cyc_Core_Opt*_tmp178;void**_tmp177;struct Cyc_Core_Opt*_tmp176;void*_tmp179;struct Cyc_List_List*_tmp17A;void*_tmp17B;switch(*((int*)_tmp175)){case 0U: switch(*((int*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp175)->f1)){case 8U: if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp175)->f2 != 0){_LL1: _tmp17B=(void*)(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp175)->f2)->hd;_LL2: {void*r2=_tmp17B;
# 936
if(constrain)return Cyc_Unify_unify(r,r2);r2=
Cyc_Tcutil_compress(r2);
if(r == r2)return 1;{struct _tuple15 _stmttmp2B=({struct _tuple15 _tmp4E4;_tmp4E4.f1=r,_tmp4E4.f2=r2;_tmp4E4;});struct _tuple15 _tmp17C=_stmttmp2B;struct Cyc_Absyn_Tvar*_tmp17E;struct Cyc_Absyn_Tvar*_tmp17D;if(((struct Cyc_Absyn_VarType_Absyn_Type_struct*)_tmp17C.f1)->tag == 2U){if(((struct Cyc_Absyn_VarType_Absyn_Type_struct*)_tmp17C.f2)->tag == 2U){_LLC: _tmp17D=((struct Cyc_Absyn_VarType_Absyn_Type_struct*)_tmp17C.f1)->f1;_tmp17E=((struct Cyc_Absyn_VarType_Absyn_Type_struct*)_tmp17C.f2)->f1;_LLD: {struct Cyc_Absyn_Tvar*tv1=_tmp17D;struct Cyc_Absyn_Tvar*tv2=_tmp17E;
# 940
return Cyc_Absyn_tvar_cmp(tv1,tv2)== 0;}}else{goto _LLE;}}else{_LLE: _LLF:
 return 0;}_LLB:;}}}else{goto _LL9;}case 9U: _LL3: _tmp17A=((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp175)->f2;_LL4: {struct Cyc_List_List*es=_tmp17A;
# 944
for(0;es != 0;es=es->tl){
if(Cyc_Tcutil_region_in_effect(constrain,r,(void*)es->hd))return 1;}
# 944
return 0;}case 10U: if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp175)->f2 != 0){_LL5: _tmp179=(void*)(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp175)->f2)->hd;_LL6: {void*t=_tmp179;
# 948
void*_stmttmp2C=Cyc_Tcutil_rgns_of(t);void*_tmp17F=_stmttmp2C;void*_tmp180;void*_tmp181;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp17F)->tag == 0U){if(((struct Cyc_Absyn_RgnsCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp17F)->f1)->tag == 10U){if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp17F)->f2 != 0){_LL11: _tmp181=(void*)(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp17F)->f2)->hd;_LL12: {void*t=_tmp181;
# 950
if(!constrain)return 0;{void*_stmttmp2D=
Cyc_Tcutil_compress(t);
# 950
void*_tmp182=_stmttmp2D;struct Cyc_Core_Opt*_tmp185;void**_tmp184;struct Cyc_Core_Opt*_tmp183;if(((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp182)->tag == 1U){_LL16: _tmp183=((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp182)->f1;_tmp184=(void**)&((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp182)->f2;_tmp185=((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp182)->f4;_LL17: {struct Cyc_Core_Opt*k=_tmp183;void**p=_tmp184;struct Cyc_Core_Opt*s=_tmp185;
# 955
void*ev=Cyc_Absyn_new_evar(& Cyc_Tcutil_eko,s);
# 958
Cyc_Unify_occurs(ev,Cyc_Core_heap_region,(struct Cyc_List_List*)((struct Cyc_Core_Opt*)_check_null(s))->v,r);
({void*_tmp5D4=Cyc_Tcutil_dummy_fntype(Cyc_Absyn_join_eff(({void*_tmp186[2U];_tmp186[0]=ev,({void*_tmp5D3=Cyc_Absyn_access_eff(r);_tmp186[1]=_tmp5D3;});Cyc_List_list(_tag_fat(_tmp186,sizeof(void*),2U));})));*p=_tmp5D4;});
return 1;}}else{_LL18: _LL19:
 return 0;}_LL15:;}}}else{goto _LL13;}}else{goto _LL13;}}else{_LL13: _tmp180=_tmp17F;_LL14: {void*e2=_tmp180;
# 963
return Cyc_Tcutil_region_in_effect(constrain,r,e2);}}_LL10:;}}else{goto _LL9;}default: goto _LL9;}case 1U: _LL7: _tmp176=((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp175)->f1;_tmp177=(void**)&((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp175)->f2;_tmp178=((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp175)->f4;_LL8: {struct Cyc_Core_Opt*k=_tmp176;void**p=_tmp177;struct Cyc_Core_Opt*s=_tmp178;
# 966
if(k == 0 ||(int)((struct Cyc_Absyn_Kind*)k->v)->kind != (int)Cyc_Absyn_EffKind)
({void*_tmp187=0U;({int(*_tmp5D6)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp189)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Tcutil_impos;_tmp189;});struct _fat_ptr _tmp5D5=({const char*_tmp188="effect evar has wrong kind";_tag_fat(_tmp188,sizeof(char),27U);});_tmp5D6(_tmp5D5,_tag_fat(_tmp187,sizeof(void*),0U));});});
# 966
if(!constrain)
# 968
return 0;{
# 966
void*ev=
# 971
Cyc_Absyn_new_evar(& Cyc_Tcutil_eko,s);
# 974
Cyc_Unify_occurs(ev,Cyc_Core_heap_region,(struct Cyc_List_List*)((struct Cyc_Core_Opt*)_check_null(s))->v,r);{
void*new_typ=Cyc_Absyn_join_eff(({void*_tmp18A[2U];_tmp18A[0]=ev,({void*_tmp5D7=Cyc_Absyn_access_eff(r);_tmp18A[1]=_tmp5D7;});Cyc_List_list(_tag_fat(_tmp18A,sizeof(void*),2U));}));
*p=new_typ;
return 1;}}}default: _LL9: _LLA:
 return 0;}_LL0:;}}
# 797
static int Cyc_Tcutil_type_in_effect(int may_constrain_evars,void*t,void*e){
# 986
t=Cyc_Tcutil_compress(t);{
void*_stmttmp2E=Cyc_Tcutil_normalize_effect(Cyc_Tcutil_compress(e));void*_tmp18B=_stmttmp2E;struct Cyc_Core_Opt*_tmp18E;void**_tmp18D;struct Cyc_Core_Opt*_tmp18C;void*_tmp18F;struct Cyc_List_List*_tmp190;switch(*((int*)_tmp18B)){case 0U: switch(*((int*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp18B)->f1)){case 8U: _LL1: _LL2:
 return 0;case 9U: _LL3: _tmp190=((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp18B)->f2;_LL4: {struct Cyc_List_List*es=_tmp190;
# 990
for(0;es != 0;es=es->tl){
if(Cyc_Tcutil_type_in_effect(may_constrain_evars,t,(void*)es->hd))
return 1;}
# 990
return 0;}case 10U: if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp18B)->f2 != 0){_LL5: _tmp18F=(void*)(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp18B)->f2)->hd;_LL6: {void*t2=_tmp18F;
# 995
t2=Cyc_Tcutil_compress(t2);
if(t == t2)return 1;if(may_constrain_evars)
return Cyc_Unify_unify(t,t2);{
# 996
void*_stmttmp2F=
# 998
Cyc_Tcutil_rgns_of(t);
# 996
void*_tmp191=_stmttmp2F;void*_tmp192;void*_tmp193;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp191)->tag == 0U){if(((struct Cyc_Absyn_RgnsCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp191)->f1)->tag == 10U){if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp191)->f2 != 0){_LLC: _tmp193=(void*)(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp191)->f2)->hd;_LLD: {void*t3=_tmp193;
# 1000
struct _tuple15 _stmttmp30=({struct _tuple15 _tmp4E5;({void*_tmp5D8=Cyc_Tcutil_compress(t3);_tmp4E5.f1=_tmp5D8;}),_tmp4E5.f2=t2;_tmp4E5;});struct _tuple15 _tmp194=_stmttmp30;struct Cyc_Absyn_Tvar*_tmp196;struct Cyc_Absyn_Tvar*_tmp195;if(((struct Cyc_Absyn_VarType_Absyn_Type_struct*)_tmp194.f1)->tag == 2U){if(((struct Cyc_Absyn_VarType_Absyn_Type_struct*)_tmp194.f2)->tag == 2U){_LL11: _tmp195=((struct Cyc_Absyn_VarType_Absyn_Type_struct*)_tmp194.f1)->f1;_tmp196=((struct Cyc_Absyn_VarType_Absyn_Type_struct*)_tmp194.f2)->f1;_LL12: {struct Cyc_Absyn_Tvar*tv1=_tmp195;struct Cyc_Absyn_Tvar*tv2=_tmp196;
return Cyc_Unify_unify(t,t2);}}else{goto _LL13;}}else{_LL13: _LL14:
 return t3 == t2;}_LL10:;}}else{goto _LLE;}}else{goto _LLE;}}else{_LLE: _tmp192=_tmp191;_LLF: {void*e2=_tmp192;
# 1004
return Cyc_Tcutil_type_in_effect(may_constrain_evars,t,e2);}}_LLB:;}}}else{goto _LL9;}default: goto _LL9;}case 1U: _LL7: _tmp18C=((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp18B)->f1;_tmp18D=(void**)&((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp18B)->f2;_tmp18E=((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp18B)->f4;_LL8: {struct Cyc_Core_Opt*k=_tmp18C;void**p=_tmp18D;struct Cyc_Core_Opt*s=_tmp18E;
# 1007
if(k == 0 ||(int)((struct Cyc_Absyn_Kind*)k->v)->kind != (int)Cyc_Absyn_EffKind)
({void*_tmp197=0U;({int(*_tmp5DA)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp199)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Tcutil_impos;_tmp199;});struct _fat_ptr _tmp5D9=({const char*_tmp198="effect evar has wrong kind";_tag_fat(_tmp198,sizeof(char),27U);});_tmp5DA(_tmp5D9,_tag_fat(_tmp197,sizeof(void*),0U));});});
# 1007
if(!may_constrain_evars)
# 1009
return 0;{
# 1007
void*ev=
# 1012
Cyc_Absyn_new_evar(& Cyc_Tcutil_eko,s);
# 1015
Cyc_Unify_occurs(ev,Cyc_Core_heap_region,(struct Cyc_List_List*)((struct Cyc_Core_Opt*)_check_null(s))->v,t);{
void*new_typ=Cyc_Absyn_join_eff(({void*_tmp19A[2U];_tmp19A[0]=ev,({void*_tmp5DB=Cyc_Absyn_regionsof_eff(t);_tmp19A[1]=_tmp5DB;});Cyc_List_list(_tag_fat(_tmp19A,sizeof(void*),2U));}));
*p=new_typ;
return 1;}}}default: _LL9: _LLA:
 return 0;}_LL0:;}}
# 797
static int Cyc_Tcutil_variable_in_effect(int may_constrain_evars,struct Cyc_Absyn_Tvar*v,void*e){
# 1027
e=Cyc_Tcutil_compress(e);{
void*_tmp19B=e;struct Cyc_Core_Opt*_tmp19E;void**_tmp19D;struct Cyc_Core_Opt*_tmp19C;void*_tmp19F;struct Cyc_List_List*_tmp1A0;struct Cyc_Absyn_Tvar*_tmp1A1;switch(*((int*)_tmp19B)){case 2U: _LL1: _tmp1A1=((struct Cyc_Absyn_VarType_Absyn_Type_struct*)_tmp19B)->f1;_LL2: {struct Cyc_Absyn_Tvar*v2=_tmp1A1;
return Cyc_Absyn_tvar_cmp(v,v2)== 0;}case 0U: switch(*((int*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp19B)->f1)){case 9U: _LL3: _tmp1A0=((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp19B)->f2;_LL4: {struct Cyc_List_List*es=_tmp1A0;
# 1031
for(0;es != 0;es=es->tl){
if(Cyc_Tcutil_variable_in_effect(may_constrain_evars,v,(void*)es->hd))
return 1;}
# 1031
return 0;}case 10U: if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp19B)->f2 != 0){_LL5: _tmp19F=(void*)(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp19B)->f2)->hd;_LL6: {void*t=_tmp19F;
# 1036
void*_stmttmp31=Cyc_Tcutil_rgns_of(t);void*_tmp1A2=_stmttmp31;void*_tmp1A3;void*_tmp1A4;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp1A2)->tag == 0U){if(((struct Cyc_Absyn_RgnsCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp1A2)->f1)->tag == 10U){if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp1A2)->f2 != 0){_LLC: _tmp1A4=(void*)(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp1A2)->f2)->hd;_LLD: {void*t2=_tmp1A4;
# 1038
if(!may_constrain_evars)return 0;{void*_stmttmp32=
Cyc_Tcutil_compress(t2);
# 1038
void*_tmp1A5=_stmttmp32;struct Cyc_Core_Opt*_tmp1A8;void**_tmp1A7;struct Cyc_Core_Opt*_tmp1A6;if(((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp1A5)->tag == 1U){_LL11: _tmp1A6=((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp1A5)->f1;_tmp1A7=(void**)&((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp1A5)->f2;_tmp1A8=((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp1A5)->f4;_LL12: {struct Cyc_Core_Opt*k=_tmp1A6;void**p=_tmp1A7;struct Cyc_Core_Opt*s=_tmp1A8;
# 1044
void*ev=Cyc_Absyn_new_evar(& Cyc_Tcutil_eko,s);
# 1046
if(!({int(*_tmp5DD)(int(*compare)(struct Cyc_Absyn_Tvar*,struct Cyc_Absyn_Tvar*),struct Cyc_List_List*l,struct Cyc_Absyn_Tvar*x)=({int(*_tmp1A9)(int(*compare)(struct Cyc_Absyn_Tvar*,struct Cyc_Absyn_Tvar*),struct Cyc_List_List*l,struct Cyc_Absyn_Tvar*x)=(int(*)(int(*compare)(struct Cyc_Absyn_Tvar*,struct Cyc_Absyn_Tvar*),struct Cyc_List_List*l,struct Cyc_Absyn_Tvar*x))Cyc_List_mem;_tmp1A9;});struct Cyc_List_List*_tmp5DC=(struct Cyc_List_List*)((struct Cyc_Core_Opt*)_check_null(s))->v;_tmp5DD(Cyc_Tcutil_fast_tvar_cmp,_tmp5DC,v);}))return 0;({void*_tmp5DF=
Cyc_Tcutil_dummy_fntype(Cyc_Absyn_join_eff(({void*_tmp1AA[2U];_tmp1AA[0]=ev,({void*_tmp5DE=Cyc_Absyn_var_type(v);_tmp1AA[1]=_tmp5DE;});Cyc_List_list(_tag_fat(_tmp1AA,sizeof(void*),2U));})));
# 1046
*p=_tmp5DF;});
# 1048
return 1;}}else{_LL13: _LL14:
 return 0;}_LL10:;}}}else{goto _LLE;}}else{goto _LLE;}}else{_LLE: _tmp1A3=_tmp1A2;_LLF: {void*e2=_tmp1A3;
# 1051
return Cyc_Tcutil_variable_in_effect(may_constrain_evars,v,e2);}}_LLB:;}}else{goto _LL9;}default: goto _LL9;}case 1U: _LL7: _tmp19C=((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp19B)->f1;_tmp19D=(void**)&((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp19B)->f2;_tmp19E=((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp19B)->f4;_LL8: {struct Cyc_Core_Opt*k=_tmp19C;void**p=_tmp19D;struct Cyc_Core_Opt*s=_tmp19E;
# 1054
if(k == 0 ||(int)((struct Cyc_Absyn_Kind*)k->v)->kind != (int)Cyc_Absyn_EffKind)
({void*_tmp1AB=0U;({int(*_tmp5E1)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp1AD)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Tcutil_impos;_tmp1AD;});struct _fat_ptr _tmp5E0=({const char*_tmp1AC="effect evar has wrong kind";_tag_fat(_tmp1AC,sizeof(char),27U);});_tmp5E1(_tmp5E0,_tag_fat(_tmp1AB,sizeof(void*),0U));});});{
# 1054
void*ev=
# 1058
Cyc_Absyn_new_evar(& Cyc_Tcutil_eko,s);
# 1060
if(!({int(*_tmp5E3)(int(*compare)(struct Cyc_Absyn_Tvar*,struct Cyc_Absyn_Tvar*),struct Cyc_List_List*l,struct Cyc_Absyn_Tvar*x)=({int(*_tmp1AE)(int(*compare)(struct Cyc_Absyn_Tvar*,struct Cyc_Absyn_Tvar*),struct Cyc_List_List*l,struct Cyc_Absyn_Tvar*x)=(int(*)(int(*compare)(struct Cyc_Absyn_Tvar*,struct Cyc_Absyn_Tvar*),struct Cyc_List_List*l,struct Cyc_Absyn_Tvar*x))Cyc_List_mem;_tmp1AE;});struct Cyc_List_List*_tmp5E2=(struct Cyc_List_List*)((struct Cyc_Core_Opt*)_check_null(s))->v;_tmp5E3(Cyc_Tcutil_fast_tvar_cmp,_tmp5E2,v);}))
return 0;{
# 1060
void*new_typ=
# 1062
Cyc_Absyn_join_eff(({void*_tmp1AF[2U];_tmp1AF[0]=ev,({void*_tmp5E4=Cyc_Absyn_var_type(v);_tmp1AF[1]=_tmp5E4;});Cyc_List_list(_tag_fat(_tmp1AF,sizeof(void*),2U));}));
*p=new_typ;
return 1;}}}default: _LL9: _LLA:
 return 0;}_LL0:;}}
# 797
static int Cyc_Tcutil_evar_in_effect(void*evar,void*e){
# 1071
e=Cyc_Tcutil_compress(e);{
void*_tmp1B0=e;void*_tmp1B1;struct Cyc_List_List*_tmp1B2;switch(*((int*)_tmp1B0)){case 0U: switch(*((int*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp1B0)->f1)){case 9U: _LL1: _tmp1B2=((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp1B0)->f2;_LL2: {struct Cyc_List_List*es=_tmp1B2;
# 1074
for(0;es != 0;es=es->tl){
if(Cyc_Tcutil_evar_in_effect(evar,(void*)es->hd))
return 1;}
# 1074
return 0;}case 10U: if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp1B0)->f2 != 0){_LL3: _tmp1B1=(void*)(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp1B0)->f2)->hd;_LL4: {void*t=_tmp1B1;
# 1079
void*_stmttmp33=Cyc_Tcutil_rgns_of(t);void*_tmp1B3=_stmttmp33;void*_tmp1B4;void*_tmp1B5;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp1B3)->tag == 0U){if(((struct Cyc_Absyn_RgnsCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp1B3)->f1)->tag == 10U){if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp1B3)->f2 != 0){_LLA: _tmp1B5=(void*)(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp1B3)->f2)->hd;_LLB: {void*t2=_tmp1B5;
return 0;}}else{goto _LLC;}}else{goto _LLC;}}else{_LLC: _tmp1B4=_tmp1B3;_LLD: {void*e2=_tmp1B4;
return Cyc_Tcutil_evar_in_effect(evar,e2);}}_LL9:;}}else{goto _LL7;}default: goto _LL7;}case 1U: _LL5: _LL6:
# 1083
 return evar == e;default: _LL7: _LL8:
 return 0;}_LL0:;}}
# 797
int Cyc_Tcutil_subset_effect(int may_constrain_evars,void*e1,void*e2){
# 1102 "tcutil.cyc"
void*_stmttmp34=Cyc_Tcutil_compress(e1);void*_tmp1B6=_stmttmp34;struct Cyc_Core_Opt*_tmp1B8;void**_tmp1B7;struct Cyc_Absyn_Tvar*_tmp1B9;void*_tmp1BA;void*_tmp1BB;struct Cyc_List_List*_tmp1BC;switch(*((int*)_tmp1B6)){case 0U: switch(*((int*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp1B6)->f1)){case 9U: _LL1: _tmp1BC=((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp1B6)->f2;_LL2: {struct Cyc_List_List*es=_tmp1BC;
# 1104
for(0;es != 0;es=es->tl){
if(!Cyc_Tcutil_subset_effect(may_constrain_evars,(void*)es->hd,e2))
return 0;}
# 1104
return 1;}case 8U: if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp1B6)->f2 != 0){_LL3: _tmp1BB=(void*)(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp1B6)->f2)->hd;_LL4: {void*r=_tmp1BB;
# 1115
return Cyc_Tcutil_region_in_effect(may_constrain_evars,r,e2)||
 may_constrain_evars && Cyc_Unify_unify(r,Cyc_Absyn_heap_rgn_type);}}else{goto _LLB;}case 10U: if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp1B6)->f2 != 0){_LL7: _tmp1BA=(void*)(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp1B6)->f2)->hd;_LL8: {void*t=_tmp1BA;
# 1119
void*_stmttmp35=Cyc_Tcutil_rgns_of(t);void*_tmp1BD=_stmttmp35;void*_tmp1BE;void*_tmp1BF;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp1BD)->tag == 0U){if(((struct Cyc_Absyn_RgnsCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp1BD)->f1)->tag == 10U){if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp1BD)->f2 != 0){_LLE: _tmp1BF=(void*)(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp1BD)->f2)->hd;_LLF: {void*t2=_tmp1BF;
# 1124
return Cyc_Tcutil_type_in_effect(may_constrain_evars,t2,e2)||
 may_constrain_evars && Cyc_Unify_unify(t2,Cyc_Absyn_sint_type);}}else{goto _LL10;}}else{goto _LL10;}}else{_LL10: _tmp1BE=_tmp1BD;_LL11: {void*e=_tmp1BE;
return Cyc_Tcutil_subset_effect(may_constrain_evars,e,e2);}}_LLD:;}}else{goto _LLB;}default: goto _LLB;}case 2U: _LL5: _tmp1B9=((struct Cyc_Absyn_VarType_Absyn_Type_struct*)_tmp1B6)->f1;_LL6: {struct Cyc_Absyn_Tvar*v=_tmp1B9;
# 1117
return Cyc_Tcutil_variable_in_effect(may_constrain_evars,v,e2);}case 1U: _LL9: _tmp1B7=(void**)&((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp1B6)->f2;_tmp1B8=((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp1B6)->f4;_LLA: {void**p=_tmp1B7;struct Cyc_Core_Opt*s=_tmp1B8;
# 1129
if(!Cyc_Tcutil_evar_in_effect(e1,e2)){
# 1133
*p=Cyc_Absyn_empty_effect;
# 1136
return 1;}else{
# 1138
return 0;}}default: _LLB: _LLC:
({struct Cyc_String_pa_PrintArg_struct _tmp1C3=({struct Cyc_String_pa_PrintArg_struct _tmp4E6;_tmp4E6.tag=0U,({struct _fat_ptr _tmp5E5=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(e1));_tmp4E6.f1=_tmp5E5;});_tmp4E6;});void*_tmp1C0[1U];_tmp1C0[0]=& _tmp1C3;({int(*_tmp5E7)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp1C2)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Tcutil_impos;_tmp1C2;});struct _fat_ptr _tmp5E6=({const char*_tmp1C1="subset_effect: bad effect: %s";_tag_fat(_tmp1C1,sizeof(char),30U);});_tmp5E7(_tmp5E6,_tag_fat(_tmp1C0,sizeof(void*),1U));});});}_LL0:;}
# 797 "tcutil.cyc"
static int Cyc_Tcutil_sub_rgnpo(struct Cyc_List_List*rpo1,struct Cyc_List_List*rpo2){
# 1151 "tcutil.cyc"
{struct Cyc_List_List*r1=rpo1;for(0;r1 != 0;r1=r1->tl){
struct _tuple15*_stmttmp36=(struct _tuple15*)r1->hd;void*_tmp1C5;void*_tmp1C4;_LL1: _tmp1C4=_stmttmp36->f1;_tmp1C5=_stmttmp36->f2;_LL2: {void*t1a=_tmp1C4;void*t1b=_tmp1C5;
int found=t1a == Cyc_Absyn_heap_rgn_type;
{struct Cyc_List_List*r2=rpo2;for(0;r2 != 0 && !found;r2=r2->tl){
struct _tuple15*_stmttmp37=(struct _tuple15*)r2->hd;void*_tmp1C7;void*_tmp1C6;_LL4: _tmp1C6=_stmttmp37->f1;_tmp1C7=_stmttmp37->f2;_LL5: {void*t2a=_tmp1C6;void*t2b=_tmp1C7;
if(Cyc_Unify_unify(t1a,t2a)&& Cyc_Unify_unify(t1b,t2b)){
found=1;
break;}}}}
# 1161
if(!found)return 0;}}}
# 1163
return 1;}
# 1169
int Cyc_Tcutil_same_rgn_po(struct Cyc_List_List*rpo1,struct Cyc_List_List*rpo2){
# 1171
return Cyc_Tcutil_sub_rgnpo(rpo1,rpo2)&& Cyc_Tcutil_sub_rgnpo(rpo2,rpo1);}
# 1169
static int Cyc_Tcutil_tycon2int(void*t){
# 1175
void*_tmp1C8=t;switch(*((int*)_tmp1C8)){case 0U: _LL1: _LL2:
 return 0;case 1U: switch(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)_tmp1C8)->f1){case Cyc_Absyn_Unsigned: switch(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)_tmp1C8)->f2){case Cyc_Absyn_Char_sz: _LL3: _LL4:
 return 1;case Cyc_Absyn_Short_sz: _LL9: _LLA:
# 1180
 return 4;case Cyc_Absyn_Int_sz: _LLF: _LL10:
# 1183
 return 7;case Cyc_Absyn_Long_sz: _LL15: _LL16:
# 1186
 return 7;case Cyc_Absyn_LongLong_sz: _LL1B: _LL1C:
# 1189
 return 13;default: goto _LL4B;}case Cyc_Absyn_Signed: switch(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)_tmp1C8)->f2){case Cyc_Absyn_Char_sz: _LL5: _LL6:
# 1178
 return 2;case Cyc_Absyn_Short_sz: _LLB: _LLC:
# 1181
 return 5;case Cyc_Absyn_Int_sz: _LL11: _LL12:
# 1184
 return 8;case Cyc_Absyn_Long_sz: _LL17: _LL18:
# 1187
 return 8;case Cyc_Absyn_LongLong_sz: _LL1D: _LL1E:
# 1190
 return 14;default: goto _LL4B;}case Cyc_Absyn_None: switch(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)_tmp1C8)->f2){case Cyc_Absyn_Char_sz: _LL7: _LL8:
# 1179
 return 3;case Cyc_Absyn_Short_sz: _LLD: _LLE:
# 1182
 return 6;case Cyc_Absyn_Int_sz: _LL13: _LL14:
# 1185
 return 9;case Cyc_Absyn_Long_sz: _LL19: _LL1A:
# 1188
 return 9;case Cyc_Absyn_LongLong_sz: _LL1F: _LL20:
# 1191
 return 15;default: goto _LL4B;}default: _LL4B: _LL4C:
# 1213
({void*_tmp1C9=0U;({int(*_tmp5E9)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp1CB)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Tcutil_impos;_tmp1CB;});struct _fat_ptr _tmp5E8=({const char*_tmp1CA="bad con";_tag_fat(_tmp1CA,sizeof(char),8U);});_tmp5E9(_tmp5E8,_tag_fat(_tmp1C9,sizeof(void*),0U));});});}case 2U: switch(((struct Cyc_Absyn_FloatCon_Absyn_TyCon_struct*)_tmp1C8)->f1){case 0U: _LL21: _LL22:
# 1192
 return 16;case 1U: _LL23: _LL24:
 return 17;default: _LL25: _LL26:
 return 18;}case 3U: _LL27: _LL28:
 return 19;case 4U: _LL29: _LL2A:
 return 20;case 5U: _LL2B: _LL2C:
 return 21;case 6U: _LL2D: _LL2E:
 return 22;case 7U: _LL2F: _LL30:
 return 23;case 8U: _LL31: _LL32:
 return 24;case 9U: _LL33: _LL34:
 return 25;case 10U: _LL35: _LL36:
 return 26;case 11U: _LL37: _LL38:
 return 27;case 12U: _LL39: _LL3A:
 return 28;case 14U: _LL3B: _LL3C:
 return 29;case 13U: _LL3D: _LL3E:
 return 30;case 15U: _LL3F: _LL40:
 return 31;case 16U: _LL41: _LL42:
 return 32;case 17U: _LL43: _LL44:
 return 33;case 18U: _LL45: _LL46:
 return 34;case 19U: _LL47: _LL48:
 return 35;default: _LL49: _LL4A:
 return 36;}_LL0:;}
# 1169
static int Cyc_Tcutil_type_case_number(void*t){
# 1217
void*_tmp1CC=t;void*_tmp1CD;switch(*((int*)_tmp1CC)){case 1U: _LL1: _LL2:
 return 1;case 2U: _LL3: _LL4:
 return 2;case 3U: _LL5: _LL6:
 return 3;case 4U: _LL7: _LL8:
 return 4;case 5U: _LL9: _LLA:
 return 5;case 6U: _LLB: _LLC:
 return 6;case 7U: _LLD: _LLE:
 return 7;case 8U: _LLF: _LL10:
 return 8;case 9U: _LL11: _LL12:
 return 9;case 10U: _LL13: _LL14:
 return 10;case 11U: _LL15: _LL16:
 return 11;default: _LL17: _tmp1CD=(void*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp1CC)->f1;_LL18: {void*c=_tmp1CD;
return 12 + Cyc_Tcutil_tycon2int(c);}}_LL0:;}
# 1169
static int Cyc_Tcutil_enumfield_cmp(struct Cyc_Absyn_Enumfield*e1,struct Cyc_Absyn_Enumfield*e2){
# 1233
int qc=Cyc_Absyn_qvar_cmp(e1->name,e2->name);
if(qc != 0)return qc;return({
int(*_tmp5EB)(int(*cmp)(struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*),struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*)=({int(*_tmp1CE)(int(*cmp)(struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*),struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*)=(int(*)(int(*cmp)(struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*),struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*))Cyc_Tcutil_star_cmp;_tmp1CE;});struct Cyc_Absyn_Exp*_tmp5EA=e1->tag;_tmp5EB(Cyc_Evexp_const_exp_cmp,_tmp5EA,e2->tag);});}
# 1169
static struct _tuple1*Cyc_Tcutil_get_datatype_qvar(union Cyc_Absyn_DatatypeInfo i){
# 1238
union Cyc_Absyn_DatatypeInfo _tmp1CF=i;struct _tuple1*_tmp1D0;struct Cyc_Absyn_Datatypedecl*_tmp1D1;if((_tmp1CF.KnownDatatype).tag == 2){_LL1: _tmp1D1=*(_tmp1CF.KnownDatatype).val;_LL2: {struct Cyc_Absyn_Datatypedecl*dd=_tmp1D1;
return dd->name;}}else{_LL3: _tmp1D0=((_tmp1CF.UnknownDatatype).val).name;_LL4: {struct _tuple1*n=_tmp1D0;
return n;}}_LL0:;}struct _tuple20{struct _tuple1*f1;struct _tuple1*f2;};
# 1169
static struct _tuple20 Cyc_Tcutil_get_datatype_field_qvars(union Cyc_Absyn_DatatypeFieldInfo i){
# 1244
union Cyc_Absyn_DatatypeFieldInfo _tmp1D2=i;struct _tuple1*_tmp1D4;struct _tuple1*_tmp1D3;struct Cyc_Absyn_Datatypefield*_tmp1D6;struct Cyc_Absyn_Datatypedecl*_tmp1D5;if((_tmp1D2.KnownDatatypefield).tag == 2){_LL1: _tmp1D5=((_tmp1D2.KnownDatatypefield).val).f1;_tmp1D6=((_tmp1D2.KnownDatatypefield).val).f2;_LL2: {struct Cyc_Absyn_Datatypedecl*dd=_tmp1D5;struct Cyc_Absyn_Datatypefield*df=_tmp1D6;
# 1246
return({struct _tuple20 _tmp4E7;_tmp4E7.f1=dd->name,_tmp4E7.f2=df->name;_tmp4E7;});}}else{_LL3: _tmp1D3=((_tmp1D2.UnknownDatatypefield).val).datatype_name;_tmp1D4=((_tmp1D2.UnknownDatatypefield).val).field_name;_LL4: {struct _tuple1*d=_tmp1D3;struct _tuple1*f=_tmp1D4;
# 1248
return({struct _tuple20 _tmp4E8;_tmp4E8.f1=d,_tmp4E8.f2=f;_tmp4E8;});}}_LL0:;}struct _tuple21{enum Cyc_Absyn_AggrKind f1;struct _tuple1*f2;};
# 1169
static struct _tuple21 Cyc_Tcutil_get_aggr_kind_and_qvar(union Cyc_Absyn_AggrInfo i){
# 1252
union Cyc_Absyn_AggrInfo _tmp1D7=i;struct Cyc_Absyn_Aggrdecl*_tmp1D8;struct _tuple1*_tmp1DA;enum Cyc_Absyn_AggrKind _tmp1D9;if((_tmp1D7.UnknownAggr).tag == 1){_LL1: _tmp1D9=((_tmp1D7.UnknownAggr).val).f1;_tmp1DA=((_tmp1D7.UnknownAggr).val).f2;_LL2: {enum Cyc_Absyn_AggrKind k=_tmp1D9;struct _tuple1*n=_tmp1DA;
return({struct _tuple21 _tmp4E9;_tmp4E9.f1=k,_tmp4E9.f2=n;_tmp4E9;});}}else{_LL3: _tmp1D8=*(_tmp1D7.KnownAggr).val;_LL4: {struct Cyc_Absyn_Aggrdecl*ad=_tmp1D8;
return({struct _tuple21 _tmp4EA;_tmp4EA.f1=ad->kind,_tmp4EA.f2=ad->name;_tmp4EA;});}}_LL0:;}
# 1169
int Cyc_Tcutil_tycon_cmp(void*t1,void*t2){
# 1258
if(t1 == t2)return 0;{int i1=
Cyc_Tcutil_tycon2int(t1);
int i2=Cyc_Tcutil_tycon2int(t2);
if(i1 != i2)return i1 - i2;{struct _tuple15 _stmttmp38=({struct _tuple15 _tmp4EB;_tmp4EB.f1=t1,_tmp4EB.f2=t2;_tmp4EB;});struct _tuple15 _tmp1DB=_stmttmp38;union Cyc_Absyn_AggrInfo _tmp1DD;union Cyc_Absyn_AggrInfo _tmp1DC;union Cyc_Absyn_DatatypeFieldInfo _tmp1DF;union Cyc_Absyn_DatatypeFieldInfo _tmp1DE;union Cyc_Absyn_DatatypeInfo _tmp1E1;union Cyc_Absyn_DatatypeInfo _tmp1E0;struct Cyc_List_List*_tmp1E3;struct Cyc_List_List*_tmp1E2;struct _fat_ptr _tmp1E5;struct _fat_ptr _tmp1E4;struct _tuple1*_tmp1E7;struct _tuple1*_tmp1E6;switch(*((int*)_tmp1DB.f1)){case 15U: if(((struct Cyc_Absyn_EnumCon_Absyn_TyCon_struct*)_tmp1DB.f2)->tag == 15U){_LL1: _tmp1E6=((struct Cyc_Absyn_EnumCon_Absyn_TyCon_struct*)_tmp1DB.f1)->f1;_tmp1E7=((struct Cyc_Absyn_EnumCon_Absyn_TyCon_struct*)_tmp1DB.f2)->f1;_LL2: {struct _tuple1*n1=_tmp1E6;struct _tuple1*n2=_tmp1E7;
# 1264
return Cyc_Absyn_qvar_cmp(n1,n2);}}else{goto _LLD;}case 17U: if(((struct Cyc_Absyn_BuiltinCon_Absyn_TyCon_struct*)_tmp1DB.f2)->tag == 17U){_LL3: _tmp1E4=((struct Cyc_Absyn_BuiltinCon_Absyn_TyCon_struct*)_tmp1DB.f1)->f1;_tmp1E5=((struct Cyc_Absyn_BuiltinCon_Absyn_TyCon_struct*)_tmp1DB.f2)->f1;_LL4: {struct _fat_ptr s1=_tmp1E4;struct _fat_ptr s2=_tmp1E5;
return Cyc_strcmp((struct _fat_ptr)s1,(struct _fat_ptr)s2);}}else{goto _LLD;}case 16U: if(((struct Cyc_Absyn_AnonEnumCon_Absyn_TyCon_struct*)_tmp1DB.f2)->tag == 16U){_LL5: _tmp1E2=((struct Cyc_Absyn_AnonEnumCon_Absyn_TyCon_struct*)_tmp1DB.f1)->f1;_tmp1E3=((struct Cyc_Absyn_AnonEnumCon_Absyn_TyCon_struct*)_tmp1DB.f2)->f1;_LL6: {struct Cyc_List_List*fs1=_tmp1E2;struct Cyc_List_List*fs2=_tmp1E3;
# 1267
return({int(*_tmp5ED)(int(*cmp)(struct Cyc_Absyn_Enumfield*,struct Cyc_Absyn_Enumfield*),struct Cyc_List_List*l1,struct Cyc_List_List*l2)=({int(*_tmp1E8)(int(*cmp)(struct Cyc_Absyn_Enumfield*,struct Cyc_Absyn_Enumfield*),struct Cyc_List_List*l1,struct Cyc_List_List*l2)=(int(*)(int(*cmp)(struct Cyc_Absyn_Enumfield*,struct Cyc_Absyn_Enumfield*),struct Cyc_List_List*l1,struct Cyc_List_List*l2))Cyc_List_list_cmp;_tmp1E8;});struct Cyc_List_List*_tmp5EC=fs1;_tmp5ED(Cyc_Tcutil_enumfield_cmp,_tmp5EC,fs2);});}}else{goto _LLD;}case 18U: if(((struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct*)_tmp1DB.f2)->tag == 18U){_LL7: _tmp1E0=((struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct*)_tmp1DB.f1)->f1;_tmp1E1=((struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct*)_tmp1DB.f2)->f1;_LL8: {union Cyc_Absyn_DatatypeInfo info1=_tmp1E0;union Cyc_Absyn_DatatypeInfo info2=_tmp1E1;
# 1269
struct _tuple1*q1=Cyc_Tcutil_get_datatype_qvar(info1);
struct _tuple1*q2=Cyc_Tcutil_get_datatype_qvar(info2);
return Cyc_Absyn_qvar_cmp(q1,q2);}}else{goto _LLD;}case 19U: if(((struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct*)_tmp1DB.f2)->tag == 19U){_LL9: _tmp1DE=((struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct*)_tmp1DB.f1)->f1;_tmp1DF=((struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct*)_tmp1DB.f2)->f1;_LLA: {union Cyc_Absyn_DatatypeFieldInfo info1=_tmp1DE;union Cyc_Absyn_DatatypeFieldInfo info2=_tmp1DF;
# 1273
struct _tuple20 _stmttmp39=Cyc_Tcutil_get_datatype_field_qvars(info1);struct _tuple1*_tmp1EA;struct _tuple1*_tmp1E9;_LL10: _tmp1E9=_stmttmp39.f1;_tmp1EA=_stmttmp39.f2;_LL11: {struct _tuple1*d1=_tmp1E9;struct _tuple1*f1=_tmp1EA;
struct _tuple20 _stmttmp3A=Cyc_Tcutil_get_datatype_field_qvars(info2);struct _tuple1*_tmp1EC;struct _tuple1*_tmp1EB;_LL13: _tmp1EB=_stmttmp3A.f1;_tmp1EC=_stmttmp3A.f2;_LL14: {struct _tuple1*d2=_tmp1EB;struct _tuple1*f2=_tmp1EC;
int c=Cyc_Absyn_qvar_cmp(d1,d2);
if(c != 0)return c;return
 Cyc_Absyn_qvar_cmp(f1,f2);}}}}else{goto _LLD;}case 20U: if(((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)_tmp1DB.f2)->tag == 20U){_LLB: _tmp1DC=((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)_tmp1DB.f1)->f1;_tmp1DD=((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)_tmp1DB.f2)->f1;_LLC: {union Cyc_Absyn_AggrInfo info1=_tmp1DC;union Cyc_Absyn_AggrInfo info2=_tmp1DD;
# 1279
struct _tuple21 _stmttmp3B=Cyc_Tcutil_get_aggr_kind_and_qvar(info1);struct _tuple1*_tmp1EE;enum Cyc_Absyn_AggrKind _tmp1ED;_LL16: _tmp1ED=_stmttmp3B.f1;_tmp1EE=_stmttmp3B.f2;_LL17: {enum Cyc_Absyn_AggrKind k1=_tmp1ED;struct _tuple1*q1=_tmp1EE;
struct _tuple21 _stmttmp3C=Cyc_Tcutil_get_aggr_kind_and_qvar(info2);struct _tuple1*_tmp1F0;enum Cyc_Absyn_AggrKind _tmp1EF;_LL19: _tmp1EF=_stmttmp3C.f1;_tmp1F0=_stmttmp3C.f2;_LL1A: {enum Cyc_Absyn_AggrKind k2=_tmp1EF;struct _tuple1*q2=_tmp1F0;
int c=Cyc_Absyn_qvar_cmp(q1,q2);
if(c != 0)return c;return(int)k1 - (int)k2;}}}}else{goto _LLD;}default: _LLD: _LLE:
# 1284
 return 0;}_LL0:;}}}
# 1169
int Cyc_Tcutil_star_cmp(int(*cmp)(void*,void*),void*a1,void*a2){
# 1289
if(a1 == a2)return 0;if(
a1 == 0 && a2 != 0)return - 1;
# 1289
if(
# 1291
a1 != 0 && a2 == 0)return 1;
# 1289
return({
# 1292
int(*_tmp5EF)(void*,void*)=cmp;void*_tmp5EE=(void*)_check_null(a1);_tmp5EF(_tmp5EE,(void*)_check_null(a2));});}
# 1169
static int Cyc_Tcutil_tqual_cmp(struct Cyc_Absyn_Tqual tq1,struct Cyc_Absyn_Tqual tq2){
# 1295
int i1=(tq1.real_const + (tq1.q_volatile << 1))+ (tq1.q_restrict << 2);
int i2=(tq2.real_const + (tq2.q_volatile << 1))+ (tq2.q_restrict << 2);
return Cyc_Core_intcmp(i1,i2);}
# 1169
static int Cyc_Tcutil_tqual_type_cmp(struct _tuple12*tqt1,struct _tuple12*tqt2){
# 1300
void*_tmp1F2;struct Cyc_Absyn_Tqual _tmp1F1;_LL1: _tmp1F1=tqt1->f1;_tmp1F2=tqt1->f2;_LL2: {struct Cyc_Absyn_Tqual tq1=_tmp1F1;void*t1=_tmp1F2;
void*_tmp1F4;struct Cyc_Absyn_Tqual _tmp1F3;_LL4: _tmp1F3=tqt2->f1;_tmp1F4=tqt2->f2;_LL5: {struct Cyc_Absyn_Tqual tq2=_tmp1F3;void*t2=_tmp1F4;
int tqc=Cyc_Tcutil_tqual_cmp(tq1,tq2);
if(tqc != 0)return tqc;return
 Cyc_Tcutil_typecmp(t1,t2);}}}
# 1169
int Cyc_Tcutil_aggrfield_cmp(struct Cyc_Absyn_Aggrfield*f1,struct Cyc_Absyn_Aggrfield*f2){
# 1308
int zsc=Cyc_strptrcmp(f1->name,f2->name);
if(zsc != 0)return zsc;{int tqc=
Cyc_Tcutil_tqual_cmp(f1->tq,f2->tq);
if(tqc != 0)return tqc;{int tc=
Cyc_Tcutil_typecmp(f1->type,f2->type);
if(tc != 0)return tc;{int ac=
Cyc_List_list_cmp(Cyc_Absyn_attribute_cmp,f1->attributes,f2->attributes);
if(ac != 0)return ac;ac=({
int(*_tmp5F1)(int(*cmp)(struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*),struct Cyc_Absyn_Exp*a1,struct Cyc_Absyn_Exp*a2)=({int(*_tmp1F5)(int(*cmp)(struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*),struct Cyc_Absyn_Exp*a1,struct Cyc_Absyn_Exp*a2)=(int(*)(int(*cmp)(struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*),struct Cyc_Absyn_Exp*a1,struct Cyc_Absyn_Exp*a2))Cyc_Tcutil_star_cmp;_tmp1F5;});struct Cyc_Absyn_Exp*_tmp5F0=f1->width;_tmp5F1(Cyc_Evexp_const_exp_cmp,_tmp5F0,f2->width);});
if(ac != 0)return ac;return({
int(*_tmp5F3)(int(*cmp)(struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*),struct Cyc_Absyn_Exp*a1,struct Cyc_Absyn_Exp*a2)=({int(*_tmp1F6)(int(*cmp)(struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*),struct Cyc_Absyn_Exp*a1,struct Cyc_Absyn_Exp*a2)=(int(*)(int(*cmp)(struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*),struct Cyc_Absyn_Exp*a1,struct Cyc_Absyn_Exp*a2))Cyc_Tcutil_star_cmp;_tmp1F6;});struct Cyc_Absyn_Exp*_tmp5F2=f1->requires_clause;_tmp5F3(Cyc_Evexp_const_exp_cmp,_tmp5F2,f2->requires_clause);});}}}}
# 1169
int Cyc_Tcutil_typecmp(void*t1,void*t2){
# 1325
t1=Cyc_Tcutil_compress(t1);
t2=Cyc_Tcutil_compress(t2);
if(t1 == t2)return 0;{int shallowcmp=({
int _tmp5F4=Cyc_Tcutil_type_case_number(t1);Cyc_Core_intcmp(_tmp5F4,Cyc_Tcutil_type_case_number(t2));});
if(shallowcmp != 0)
return shallowcmp;{
# 1329
struct _tuple15 _stmttmp3D=({struct _tuple15 _tmp4EC;_tmp4EC.f1=t2,_tmp4EC.f2=t1;_tmp4EC;});struct _tuple15 _tmp1F7=_stmttmp3D;struct Cyc_Absyn_Exp*_tmp1F9;struct Cyc_Absyn_Exp*_tmp1F8;struct Cyc_Absyn_Exp*_tmp1FB;struct Cyc_Absyn_Exp*_tmp1FA;struct Cyc_List_List*_tmp1FF;enum Cyc_Absyn_AggrKind _tmp1FE;struct Cyc_List_List*_tmp1FD;enum Cyc_Absyn_AggrKind _tmp1FC;struct Cyc_List_List*_tmp201;struct Cyc_List_List*_tmp200;struct Cyc_Absyn_FnInfo _tmp203;struct Cyc_Absyn_FnInfo _tmp202;void*_tmp20B;struct Cyc_Absyn_Exp*_tmp20A;struct Cyc_Absyn_Tqual _tmp209;void*_tmp208;void*_tmp207;struct Cyc_Absyn_Exp*_tmp206;struct Cyc_Absyn_Tqual _tmp205;void*_tmp204;void*_tmp217;void*_tmp216;void*_tmp215;void*_tmp214;struct Cyc_Absyn_Tqual _tmp213;void*_tmp212;void*_tmp211;void*_tmp210;void*_tmp20F;void*_tmp20E;struct Cyc_Absyn_Tqual _tmp20D;void*_tmp20C;struct Cyc_Absyn_Tvar*_tmp219;struct Cyc_Absyn_Tvar*_tmp218;struct Cyc_List_List*_tmp21D;void*_tmp21C;struct Cyc_List_List*_tmp21B;void*_tmp21A;switch(*((int*)_tmp1F7.f1)){case 0U: if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp1F7.f2)->tag == 0U){_LL1: _tmp21A=(void*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp1F7.f1)->f1;_tmp21B=((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp1F7.f1)->f2;_tmp21C=(void*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp1F7.f2)->f1;_tmp21D=((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp1F7.f2)->f2;_LL2: {void*c1=_tmp21A;struct Cyc_List_List*ts1=_tmp21B;void*c2=_tmp21C;struct Cyc_List_List*ts2=_tmp21D;
# 1335
int c=Cyc_Tcutil_tycon_cmp(c1,c2);
if(c != 0)return c;return
 Cyc_List_list_cmp(Cyc_Tcutil_typecmp,ts1,ts2);}}else{goto _LL15;}case 1U: if(((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp1F7.f2)->tag == 1U){_LL3: _LL4:
# 1339
({void*_tmp21E=0U;({int(*_tmp5F6)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp220)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Tcutil_impos;_tmp220;});struct _fat_ptr _tmp5F5=({const char*_tmp21F="typecmp: can only compare closed types";_tag_fat(_tmp21F,sizeof(char),39U);});_tmp5F6(_tmp5F5,_tag_fat(_tmp21E,sizeof(void*),0U));});});}else{goto _LL15;}case 2U: if(((struct Cyc_Absyn_VarType_Absyn_Type_struct*)_tmp1F7.f2)->tag == 2U){_LL5: _tmp218=((struct Cyc_Absyn_VarType_Absyn_Type_struct*)_tmp1F7.f1)->f1;_tmp219=((struct Cyc_Absyn_VarType_Absyn_Type_struct*)_tmp1F7.f2)->f1;_LL6: {struct Cyc_Absyn_Tvar*tv2=_tmp218;struct Cyc_Absyn_Tvar*tv1=_tmp219;
# 1343
return Cyc_Core_intcmp(tv1->identity,tv2->identity);}}else{goto _LL15;}case 3U: if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp1F7.f2)->tag == 3U){_LL7: _tmp20C=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp1F7.f1)->f1).elt_type;_tmp20D=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp1F7.f1)->f1).elt_tq;_tmp20E=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp1F7.f1)->f1).ptr_atts).rgn;_tmp20F=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp1F7.f1)->f1).ptr_atts).nullable;_tmp210=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp1F7.f1)->f1).ptr_atts).bounds;_tmp211=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp1F7.f1)->f1).ptr_atts).zero_term;_tmp212=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp1F7.f2)->f1).elt_type;_tmp213=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp1F7.f2)->f1).elt_tq;_tmp214=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp1F7.f2)->f1).ptr_atts).rgn;_tmp215=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp1F7.f2)->f1).ptr_atts).nullable;_tmp216=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp1F7.f2)->f1).ptr_atts).bounds;_tmp217=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp1F7.f2)->f1).ptr_atts).zero_term;_LL8: {void*t2a=_tmp20C;struct Cyc_Absyn_Tqual tqual2a=_tmp20D;void*rgn2=_tmp20E;void*null2a=_tmp20F;void*b2=_tmp210;void*zt2=_tmp211;void*t1a=_tmp212;struct Cyc_Absyn_Tqual tqual1a=_tmp213;void*rgn1=_tmp214;void*null1a=_tmp215;void*b1=_tmp216;void*zt1=_tmp217;
# 1347
int etc=Cyc_Tcutil_typecmp(t1a,t2a);
if(etc != 0)return etc;{int rc=
Cyc_Tcutil_typecmp(rgn1,rgn2);
if(rc != 0)return rc;{int tqc=
Cyc_Tcutil_tqual_cmp(tqual1a,tqual2a);
if(tqc != 0)return tqc;{int cc=
Cyc_Tcutil_typecmp(b1,b2);
if(cc != 0)return cc;{int zc=
Cyc_Tcutil_typecmp(zt1,zt2);
if(zc != 0)return zc;{int bc=
Cyc_Tcutil_typecmp(b1,b2);
if(bc != 0)return bc;return
 Cyc_Tcutil_typecmp(null1a,null2a);}}}}}}}else{goto _LL15;}case 4U: if(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp1F7.f2)->tag == 4U){_LL9: _tmp204=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp1F7.f1)->f1).elt_type;_tmp205=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp1F7.f1)->f1).tq;_tmp206=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp1F7.f1)->f1).num_elts;_tmp207=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp1F7.f1)->f1).zero_term;_tmp208=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp1F7.f2)->f1).elt_type;_tmp209=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp1F7.f2)->f1).tq;_tmp20A=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp1F7.f2)->f1).num_elts;_tmp20B=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp1F7.f2)->f1).zero_term;_LLA: {void*t2a=_tmp204;struct Cyc_Absyn_Tqual tq2a=_tmp205;struct Cyc_Absyn_Exp*e1=_tmp206;void*zt1=_tmp207;void*t1a=_tmp208;struct Cyc_Absyn_Tqual tq1a=_tmp209;struct Cyc_Absyn_Exp*e2=_tmp20A;void*zt2=_tmp20B;
# 1363
int tqc=Cyc_Tcutil_tqual_cmp(tq1a,tq2a);
if(tqc != 0)return tqc;{int tc=
Cyc_Tcutil_typecmp(t1a,t2a);
if(tc != 0)return tc;{int ztc=
Cyc_Tcutil_typecmp(zt1,zt2);
if(ztc != 0)return ztc;if(e1 == e2)
return 0;
# 1368
if(
# 1370
e1 == 0 || e2 == 0)
({void*_tmp221=0U;({int(*_tmp5F8)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp223)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Tcutil_impos;_tmp223;});struct _fat_ptr _tmp5F7=({const char*_tmp222="missing expression in array index";_tag_fat(_tmp222,sizeof(char),34U);});_tmp5F8(_tmp5F7,_tag_fat(_tmp221,sizeof(void*),0U));});});
# 1368
return({
# 1373
int(*_tmp5FA)(int(*cmp)(struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*),struct Cyc_Absyn_Exp*a1,struct Cyc_Absyn_Exp*a2)=({int(*_tmp224)(int(*cmp)(struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*),struct Cyc_Absyn_Exp*a1,struct Cyc_Absyn_Exp*a2)=(int(*)(int(*cmp)(struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*),struct Cyc_Absyn_Exp*a1,struct Cyc_Absyn_Exp*a2))Cyc_Tcutil_star_cmp;_tmp224;});struct Cyc_Absyn_Exp*_tmp5F9=e1;_tmp5FA(Cyc_Evexp_const_exp_cmp,_tmp5F9,e2);});}}}}else{goto _LL15;}case 5U: if(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp1F7.f2)->tag == 5U){_LLB: _tmp202=((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp1F7.f1)->f1;_tmp203=((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp1F7.f2)->f1;_LLC: {struct Cyc_Absyn_FnInfo f1=_tmp202;struct Cyc_Absyn_FnInfo f2=_tmp203;
# 1376
if(Cyc_Unify_unify(t1,t2))return 0;{int r=
Cyc_Tcutil_typecmp(f1.ret_type,f2.ret_type);
if(r != 0)return r;r=
Cyc_Tcutil_tqual_cmp(f1.ret_tqual,f2.ret_tqual);
if(r != 0)return r;{struct Cyc_List_List*args1=f1.args;
# 1382
struct Cyc_List_List*args2=f2.args;
for(0;args1 != 0 && args2 != 0;(args1=args1->tl,args2=args2->tl)){
struct _tuple9 _stmttmp3E=*((struct _tuple9*)args1->hd);void*_tmp226;struct Cyc_Absyn_Tqual _tmp225;_LL18: _tmp225=_stmttmp3E.f2;_tmp226=_stmttmp3E.f3;_LL19: {struct Cyc_Absyn_Tqual tq1=_tmp225;void*t1=_tmp226;
struct _tuple9 _stmttmp3F=*((struct _tuple9*)args2->hd);void*_tmp228;struct Cyc_Absyn_Tqual _tmp227;_LL1B: _tmp227=_stmttmp3F.f2;_tmp228=_stmttmp3F.f3;_LL1C: {struct Cyc_Absyn_Tqual tq2=_tmp227;void*t2=_tmp228;
r=Cyc_Tcutil_tqual_cmp(tq1,tq2);
if(r != 0)return r;r=
Cyc_Tcutil_typecmp(t1,t2);
if(r != 0)return r;}}}
# 1391
if(args1 != 0)return 1;if(args2 != 0)
return - 1;
# 1391
if(
# 1393
f1.c_varargs && !f2.c_varargs)return 1;
# 1391
if(
# 1394
!f1.c_varargs && f2.c_varargs)return - 1;
# 1391
if(f1.cyc_varargs != 0 & f2.cyc_varargs == 0)
# 1395
return 1;
# 1391
if(f1.cyc_varargs == 0 & f2.cyc_varargs != 0)
# 1396
return - 1;
# 1391
if(f1.cyc_varargs != 0 & f2.cyc_varargs != 0){
# 1398
r=({struct Cyc_Absyn_Tqual _tmp5FB=((struct Cyc_Absyn_VarargInfo*)_check_null(f1.cyc_varargs))->tq;Cyc_Tcutil_tqual_cmp(_tmp5FB,((struct Cyc_Absyn_VarargInfo*)_check_null(f2.cyc_varargs))->tq);});
if(r != 0)return r;r=
Cyc_Tcutil_typecmp((f1.cyc_varargs)->type,(f2.cyc_varargs)->type);
if(r != 0)return r;if(
(f1.cyc_varargs)->inject && !(f2.cyc_varargs)->inject)return 1;
# 1401
if(
# 1403
!(f1.cyc_varargs)->inject &&(f2.cyc_varargs)->inject)return - 1;}
# 1391
r=
# 1405
Cyc_Tcutil_star_cmp(Cyc_Tcutil_typecmp,f1.effect,f2.effect);
if(r != 0)return r;{struct Cyc_List_List*rpo1=f1.rgn_po;
# 1408
struct Cyc_List_List*rpo2=f2.rgn_po;
for(0;rpo1 != 0 && rpo2 != 0;(rpo1=rpo1->tl,rpo2=rpo2->tl)){
struct _tuple15 _stmttmp40=*((struct _tuple15*)rpo1->hd);void*_tmp22A;void*_tmp229;_LL1E: _tmp229=_stmttmp40.f1;_tmp22A=_stmttmp40.f2;_LL1F: {void*t1a=_tmp229;void*t1b=_tmp22A;
struct _tuple15 _stmttmp41=*((struct _tuple15*)rpo2->hd);void*_tmp22C;void*_tmp22B;_LL21: _tmp22B=_stmttmp41.f1;_tmp22C=_stmttmp41.f2;_LL22: {void*t2a=_tmp22B;void*t2b=_tmp22C;
r=Cyc_Tcutil_typecmp(t1a,t2a);if(r != 0)return r;r=
Cyc_Tcutil_typecmp(t1b,t2b);if(r != 0)return r;}}}
# 1415
if(rpo1 != 0)return 1;if(rpo2 != 0)
return - 1;
# 1415
r=({
# 1417
int(*_tmp5FD)(int(*cmp)(struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*),struct Cyc_Absyn_Exp*a1,struct Cyc_Absyn_Exp*a2)=({int(*_tmp22D)(int(*cmp)(struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*),struct Cyc_Absyn_Exp*a1,struct Cyc_Absyn_Exp*a2)=(int(*)(int(*cmp)(struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*),struct Cyc_Absyn_Exp*a1,struct Cyc_Absyn_Exp*a2))Cyc_Tcutil_star_cmp;_tmp22D;});struct Cyc_Absyn_Exp*_tmp5FC=f1.requires_clause;_tmp5FD(Cyc_Evexp_const_exp_cmp,_tmp5FC,f2.requires_clause);});
if(r != 0)return r;r=({
int(*_tmp5FF)(int(*cmp)(struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*),struct Cyc_Absyn_Exp*a1,struct Cyc_Absyn_Exp*a2)=({int(*_tmp22E)(int(*cmp)(struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*),struct Cyc_Absyn_Exp*a1,struct Cyc_Absyn_Exp*a2)=(int(*)(int(*cmp)(struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*),struct Cyc_Absyn_Exp*a1,struct Cyc_Absyn_Exp*a2))Cyc_Tcutil_star_cmp;_tmp22E;});struct Cyc_Absyn_Exp*_tmp5FE=f1.ensures_clause;_tmp5FF(Cyc_Evexp_const_exp_cmp,_tmp5FE,f2.ensures_clause);});
if(r != 0)return r;({void*_tmp22F=0U;({int(*_tmp601)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp231)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Tcutil_impos;_tmp231;});struct _fat_ptr _tmp600=({const char*_tmp230="typecmp: function type comparison should never get here!";_tag_fat(_tmp230,sizeof(char),57U);});_tmp601(_tmp600,_tag_fat(_tmp22F,sizeof(void*),0U));});});}}}}}else{goto _LL15;}case 6U: if(((struct Cyc_Absyn_TupleType_Absyn_Type_struct*)_tmp1F7.f2)->tag == 6U){_LLD: _tmp200=((struct Cyc_Absyn_TupleType_Absyn_Type_struct*)_tmp1F7.f1)->f1;_tmp201=((struct Cyc_Absyn_TupleType_Absyn_Type_struct*)_tmp1F7.f2)->f1;_LLE: {struct Cyc_List_List*ts2=_tmp200;struct Cyc_List_List*ts1=_tmp201;
# 1426
return({int(*_tmp603)(int(*cmp)(struct _tuple12*,struct _tuple12*),struct Cyc_List_List*l1,struct Cyc_List_List*l2)=({int(*_tmp232)(int(*cmp)(struct _tuple12*,struct _tuple12*),struct Cyc_List_List*l1,struct Cyc_List_List*l2)=(int(*)(int(*cmp)(struct _tuple12*,struct _tuple12*),struct Cyc_List_List*l1,struct Cyc_List_List*l2))Cyc_List_list_cmp;_tmp232;});struct Cyc_List_List*_tmp602=ts1;_tmp603(Cyc_Tcutil_tqual_type_cmp,_tmp602,ts2);});}}else{goto _LL15;}case 7U: if(((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmp1F7.f2)->tag == 7U){_LLF: _tmp1FC=((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmp1F7.f1)->f1;_tmp1FD=((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmp1F7.f1)->f2;_tmp1FE=((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmp1F7.f2)->f1;_tmp1FF=((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmp1F7.f2)->f2;_LL10: {enum Cyc_Absyn_AggrKind k2=_tmp1FC;struct Cyc_List_List*fs2=_tmp1FD;enum Cyc_Absyn_AggrKind k1=_tmp1FE;struct Cyc_List_List*fs1=_tmp1FF;
# 1429
if((int)k1 != (int)k2){
if((int)k1 == (int)0U)return - 1;else{
return 1;}}
# 1429
return({
# 1432
int(*_tmp605)(int(*cmp)(struct Cyc_Absyn_Aggrfield*,struct Cyc_Absyn_Aggrfield*),struct Cyc_List_List*l1,struct Cyc_List_List*l2)=({int(*_tmp233)(int(*cmp)(struct Cyc_Absyn_Aggrfield*,struct Cyc_Absyn_Aggrfield*),struct Cyc_List_List*l1,struct Cyc_List_List*l2)=(int(*)(int(*cmp)(struct Cyc_Absyn_Aggrfield*,struct Cyc_Absyn_Aggrfield*),struct Cyc_List_List*l1,struct Cyc_List_List*l2))Cyc_List_list_cmp;_tmp233;});struct Cyc_List_List*_tmp604=fs1;_tmp605(Cyc_Tcutil_aggrfield_cmp,_tmp604,fs2);});}}else{goto _LL15;}case 9U: if(((struct Cyc_Absyn_ValueofType_Absyn_Type_struct*)_tmp1F7.f2)->tag == 9U){_LL11: _tmp1FA=((struct Cyc_Absyn_ValueofType_Absyn_Type_struct*)_tmp1F7.f1)->f1;_tmp1FB=((struct Cyc_Absyn_ValueofType_Absyn_Type_struct*)_tmp1F7.f2)->f1;_LL12: {struct Cyc_Absyn_Exp*e1=_tmp1FA;struct Cyc_Absyn_Exp*e2=_tmp1FB;
# 1434
_tmp1F8=e1;_tmp1F9=e2;goto _LL14;}}else{goto _LL15;}case 11U: if(((struct Cyc_Absyn_TypeofType_Absyn_Type_struct*)_tmp1F7.f2)->tag == 11U){_LL13: _tmp1F8=((struct Cyc_Absyn_TypeofType_Absyn_Type_struct*)_tmp1F7.f1)->f1;_tmp1F9=((struct Cyc_Absyn_TypeofType_Absyn_Type_struct*)_tmp1F7.f2)->f1;_LL14: {struct Cyc_Absyn_Exp*e1=_tmp1F8;struct Cyc_Absyn_Exp*e2=_tmp1F9;
# 1436
return Cyc_Evexp_const_exp_cmp(e1,e2);}}else{goto _LL15;}default: _LL15: _LL16:
({void*_tmp234=0U;({int(*_tmp607)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp236)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Tcutil_impos;_tmp236;});struct _fat_ptr _tmp606=({const char*_tmp235="Unmatched case in typecmp";_tag_fat(_tmp235,sizeof(char),26U);});_tmp607(_tmp606,_tag_fat(_tmp234,sizeof(void*),0U));});});}_LL0:;}}}
# 1169
static int Cyc_Tcutil_will_lose_precision(void*t1,void*t2){
# 1444
struct _tuple15 _stmttmp42=({struct _tuple15 _tmp4EE;({void*_tmp609=Cyc_Tcutil_compress(t1);_tmp4EE.f1=_tmp609;}),({void*_tmp608=Cyc_Tcutil_compress(t2);_tmp4EE.f2=_tmp608;});_tmp4EE;});struct _tuple15 _tmp237=_stmttmp42;void*_tmp239;void*_tmp238;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp237.f1)->tag == 0U){if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp237.f2)->tag == 0U){_LL1: _tmp238=(void*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp237.f1)->f1;_tmp239=(void*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp237.f2)->f1;_LL2: {void*c1=_tmp238;void*c2=_tmp239;
# 1446
struct _tuple15 _stmttmp43=({struct _tuple15 _tmp4ED;_tmp4ED.f1=c1,_tmp4ED.f2=c2;_tmp4ED;});struct _tuple15 _tmp23A=_stmttmp43;int _tmp23C;int _tmp23B;switch(*((int*)_tmp23A.f1)){case 2U: switch(*((int*)_tmp23A.f2)){case 2U: _LL6: _tmp23B=((struct Cyc_Absyn_FloatCon_Absyn_TyCon_struct*)_tmp23A.f1)->f1;_tmp23C=((struct Cyc_Absyn_FloatCon_Absyn_TyCon_struct*)_tmp23A.f2)->f1;_LL7: {int i1=_tmp23B;int i2=_tmp23C;
return i2 < i1;}case 1U: _LL8: _LL9:
 goto _LLB;case 4U: _LLA: _LLB:
 return 1;default: goto _LL26;}case 1U: switch(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)_tmp23A.f1)->f2){case Cyc_Absyn_LongLong_sz: if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)_tmp23A.f2)->tag == 1U){if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)_tmp23A.f2)->f2 == Cyc_Absyn_LongLong_sz){_LLC: _LLD:
 return 0;}else{goto _LLE;}}else{_LLE: _LLF:
 return 1;}case Cyc_Absyn_Long_sz: switch(*((int*)_tmp23A.f2)){case 1U: switch(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)_tmp23A.f2)->f2){case Cyc_Absyn_Int_sz: _LL10: _LL11:
# 1454
 goto _LL13;case Cyc_Absyn_Short_sz: _LL18: _LL19:
# 1459
 goto _LL1B;case Cyc_Absyn_Char_sz: _LL1E: _LL1F:
# 1462
 goto _LL21;default: goto _LL26;}case 2U: if(((struct Cyc_Absyn_FloatCon_Absyn_TyCon_struct*)_tmp23A.f2)->f1 == 0){_LL14: _LL15:
# 1457
 goto _LL17;}else{goto _LL26;}default: goto _LL26;}case Cyc_Absyn_Int_sz: switch(*((int*)_tmp23A.f2)){case 1U: switch(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)_tmp23A.f2)->f2){case Cyc_Absyn_Long_sz: _LL12: _LL13:
# 1455
 return 0;case Cyc_Absyn_Short_sz: _LL1A: _LL1B:
# 1460
 goto _LL1D;case Cyc_Absyn_Char_sz: _LL20: _LL21:
# 1463
 goto _LL23;default: goto _LL26;}case 2U: if(((struct Cyc_Absyn_FloatCon_Absyn_TyCon_struct*)_tmp23A.f2)->f1 == 0){_LL16: _LL17:
# 1458
 goto _LL19;}else{goto _LL26;}default: goto _LL26;}case Cyc_Absyn_Short_sz: if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)_tmp23A.f2)->tag == 1U){if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)_tmp23A.f2)->f2 == Cyc_Absyn_Char_sz){_LL22: _LL23:
# 1464
 goto _LL25;}else{goto _LL26;}}else{goto _LL26;}default: goto _LL26;}case 4U: if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)_tmp23A.f2)->tag == 1U)switch(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)_tmp23A.f2)->f2){case Cyc_Absyn_Short_sz: _LL1C: _LL1D:
# 1461
 goto _LL1F;case Cyc_Absyn_Char_sz: _LL24: _LL25:
# 1465
 return 1;default: goto _LL26;}else{goto _LL26;}default: _LL26: _LL27:
# 1467
 return 0;}_LL5:;}}else{goto _LL3;}}else{_LL3: _LL4:
# 1469
 return 0;}_LL0:;}
# 1169
void*Cyc_Tcutil_max_arithmetic_type(void*t1,void*t2){
# 1474
{struct _tuple15 _stmttmp44=({struct _tuple15 _tmp4F0;({void*_tmp60B=Cyc_Tcutil_compress(t1);_tmp4F0.f1=_tmp60B;}),({void*_tmp60A=Cyc_Tcutil_compress(t2);_tmp4F0.f2=_tmp60A;});_tmp4F0;});struct _tuple15 _tmp23D=_stmttmp44;void*_tmp23F;void*_tmp23E;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp23D.f1)->tag == 0U){if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp23D.f2)->tag == 0U){_LL1: _tmp23E=(void*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp23D.f1)->f1;_tmp23F=(void*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp23D.f2)->f1;_LL2: {void*c1=_tmp23E;void*c2=_tmp23F;
# 1476
{struct _tuple15 _stmttmp45=({struct _tuple15 _tmp4EF;_tmp4EF.f1=c1,_tmp4EF.f2=c2;_tmp4EF;});struct _tuple15 _tmp240=_stmttmp45;int _tmp242;int _tmp241;if(((struct Cyc_Absyn_FloatCon_Absyn_TyCon_struct*)_tmp240.f1)->tag == 2U){if(((struct Cyc_Absyn_FloatCon_Absyn_TyCon_struct*)_tmp240.f2)->tag == 2U){_LL6: _tmp241=((struct Cyc_Absyn_FloatCon_Absyn_TyCon_struct*)_tmp240.f1)->f1;_tmp242=((struct Cyc_Absyn_FloatCon_Absyn_TyCon_struct*)_tmp240.f2)->f1;_LL7: {int i1=_tmp241;int i2=_tmp242;
# 1478
if(i1 != 0 && i1 != 1)return t1;else{
if(i2 != 0 && i2 != 1)return t2;else{
if(i1 >= i2)return t1;else{
return t2;}}}}}else{_LL8: _LL9:
 return t1;}}else{if(((struct Cyc_Absyn_FloatCon_Absyn_TyCon_struct*)_tmp240.f2)->tag == 2U){_LLA: _LLB:
 return t2;}else{if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)_tmp240.f1)->tag == 1U){if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)_tmp240.f1)->f1 == Cyc_Absyn_Unsigned){if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)_tmp240.f1)->f2 == Cyc_Absyn_LongLong_sz){_LLC: _LLD:
 goto _LLF;}else{if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)_tmp240.f2)->tag == 1U){if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)_tmp240.f2)->f1 == Cyc_Absyn_Unsigned){if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)_tmp240.f2)->f2 == Cyc_Absyn_LongLong_sz)goto _LLE;else{if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)_tmp240.f1)->f2 == Cyc_Absyn_Long_sz)goto _LL14;else{if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)_tmp240.f2)->f2 == Cyc_Absyn_Long_sz)goto _LL16;else{if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)_tmp240.f1)->f2 == Cyc_Absyn_Int_sz)goto _LL1C;else{if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)_tmp240.f2)->f2 == Cyc_Absyn_Int_sz)goto _LL1E;else{goto _LL24;}}}}}}else{if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)_tmp240.f2)->f2 == Cyc_Absyn_LongLong_sz)goto _LL12;else{switch(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)_tmp240.f1)->f2){case Cyc_Absyn_Long_sz: goto _LL14;case Cyc_Absyn_Int_sz: goto _LL1C;default: if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)_tmp240.f2)->f2 == Cyc_Absyn_Long_sz)goto _LL22;else{goto _LL24;}}}}}else{if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)_tmp240.f1)->f2 == Cyc_Absyn_Long_sz){_LL14: _LL15:
# 1488
 goto _LL17;}else{if(((struct Cyc_Absyn_TagCon_Absyn_TyCon_struct*)_tmp240.f2)->tag == 4U)goto _LL1A;else{if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)_tmp240.f1)->f2 == Cyc_Absyn_Int_sz){_LL1C: _LL1D:
# 1493
 goto _LL1F;}else{goto _LL24;}}}}}}else{if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)_tmp240.f2)->tag == 1U){if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)_tmp240.f2)->f1 == Cyc_Absyn_Unsigned){if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)_tmp240.f2)->f2 == Cyc_Absyn_LongLong_sz)goto _LLE;else{if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)_tmp240.f1)->f2 == Cyc_Absyn_LongLong_sz)goto _LL10;else{switch(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)_tmp240.f2)->f2){case Cyc_Absyn_Long_sz: goto _LL16;case Cyc_Absyn_Int_sz: goto _LL1E;default: if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)_tmp240.f1)->f2 == Cyc_Absyn_Long_sz)goto _LL20;else{goto _LL24;}}}}}else{if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)_tmp240.f1)->f2 == Cyc_Absyn_LongLong_sz)goto _LL10;else{if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)_tmp240.f2)->f2 == Cyc_Absyn_LongLong_sz)goto _LL12;else{if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)_tmp240.f1)->f2 == Cyc_Absyn_Long_sz)goto _LL20;else{if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)_tmp240.f2)->f2 == Cyc_Absyn_Long_sz)goto _LL22;else{goto _LL24;}}}}}}else{if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)_tmp240.f1)->f2 == Cyc_Absyn_LongLong_sz){_LL10: _LL11:
# 1486
 goto _LL13;}else{if(((struct Cyc_Absyn_TagCon_Absyn_TyCon_struct*)_tmp240.f2)->tag == 4U)goto _LL1A;else{if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)_tmp240.f1)->f2 == Cyc_Absyn_Long_sz){_LL20: _LL21:
# 1495
 goto _LL23;}else{goto _LL24;}}}}}}else{if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)_tmp240.f2)->tag == 1U){if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)_tmp240.f2)->f1 == Cyc_Absyn_Unsigned)switch(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)_tmp240.f2)->f2){case Cyc_Absyn_LongLong_sz: _LLE: _LLF:
# 1485
 return Cyc_Absyn_ulonglong_type;case Cyc_Absyn_Long_sz: _LL16: _LL17:
# 1489
 return Cyc_Absyn_ulong_type;default: if(((struct Cyc_Absyn_TagCon_Absyn_TyCon_struct*)_tmp240.f1)->tag == 4U)goto _LL18;else{if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)_tmp240.f2)->f2 == Cyc_Absyn_Int_sz){_LL1E: _LL1F:
# 1494
 return Cyc_Absyn_uint_type;}else{goto _LL24;}}}else{if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)_tmp240.f2)->f2 == Cyc_Absyn_LongLong_sz){_LL12: _LL13:
# 1487
 return Cyc_Absyn_slonglong_type;}else{if(((struct Cyc_Absyn_TagCon_Absyn_TyCon_struct*)_tmp240.f1)->tag == 4U)goto _LL18;else{if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)_tmp240.f2)->f2 == Cyc_Absyn_Long_sz){_LL22: _LL23:
# 1496
 return Cyc_Absyn_slong_type;}else{goto _LL24;}}}}}else{if(((struct Cyc_Absyn_TagCon_Absyn_TyCon_struct*)_tmp240.f1)->tag == 4U){_LL18: _LL19:
# 1491
 goto _LL1B;}else{if(((struct Cyc_Absyn_TagCon_Absyn_TyCon_struct*)_tmp240.f2)->tag == 4U){_LL1A: _LL1B:
 goto _LL1D;}else{_LL24: _LL25:
# 1497
 goto _LL5;}}}}}}_LL5:;}
# 1499
goto _LL0;}}else{goto _LL3;}}else{_LL3: _LL4:
 goto _LL0;}_LL0:;}
# 1502
return Cyc_Absyn_sint_type;}
# 1169
int Cyc_Tcutil_coerce_list(struct Cyc_Tcenv_Tenv*te,void*t,struct Cyc_List_List*es){
# 1510
struct Cyc_Core_Opt*max_arith_type=0;
{struct Cyc_List_List*el=es;for(0;el != 0;el=el->tl){
void*t1=Cyc_Tcutil_compress((void*)_check_null(((struct Cyc_Absyn_Exp*)el->hd)->topt));
if(Cyc_Tcutil_is_arithmetic_type(t1)){
if(max_arith_type == 0 ||
 Cyc_Tcutil_will_lose_precision(t1,(void*)max_arith_type->v))
max_arith_type=({struct Cyc_Core_Opt*_tmp243=_cycalloc(sizeof(*_tmp243));_tmp243->v=t1;_tmp243;});}}}
# 1519
if(max_arith_type != 0){
if(!Cyc_Unify_unify(t,(void*)max_arith_type->v))
return 0;}
# 1519
{
# 1523
struct Cyc_List_List*el=es;
# 1519
for(0;el != 0;el=el->tl){
# 1524
if(!Cyc_Tcutil_coerce_assign(te,(struct Cyc_Absyn_Exp*)el->hd,t)){
({struct Cyc_String_pa_PrintArg_struct _tmp246=({struct Cyc_String_pa_PrintArg_struct _tmp4F2;_tmp4F2.tag=0U,({
struct _fat_ptr _tmp60C=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t));_tmp4F2.f1=_tmp60C;});_tmp4F2;});struct Cyc_String_pa_PrintArg_struct _tmp247=({struct Cyc_String_pa_PrintArg_struct _tmp4F1;_tmp4F1.tag=0U,({struct _fat_ptr _tmp60D=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string((void*)_check_null(((struct Cyc_Absyn_Exp*)el->hd)->topt)));_tmp4F1.f1=_tmp60D;});_tmp4F1;});void*_tmp244[2U];_tmp244[0]=& _tmp246,_tmp244[1]=& _tmp247;({unsigned _tmp60F=((struct Cyc_Absyn_Exp*)el->hd)->loc;struct _fat_ptr _tmp60E=({const char*_tmp245="type mismatch: expecting %s but found %s";_tag_fat(_tmp245,sizeof(char),41U);});Cyc_Tcutil_terr(_tmp60F,_tmp60E,_tag_fat(_tmp244,sizeof(void*),2U));});});
return 0;}}}
# 1519
return 1;}
# 1169
int Cyc_Tcutil_coerce_to_bool(struct Cyc_Absyn_Exp*e){
# 1535
if(!Cyc_Tcutil_coerce_sint_type(e)){
void*_stmttmp46=Cyc_Tcutil_compress((void*)_check_null(e->topt));void*_tmp248=_stmttmp46;if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp248)->tag == 3U){_LL1: _LL2:
 Cyc_Tcutil_unchecked_cast(e,Cyc_Absyn_uint_type,Cyc_Absyn_Other_coercion);goto _LL0;}else{_LL3: _LL4:
 return 0;}_LL0:;}
# 1535
return 1;}
# 1169
int Cyc_Tcutil_coerce_uint_type(struct Cyc_Absyn_Exp*e){
# 1545
if(Cyc_Unify_unify((void*)_check_null(e->topt),Cyc_Absyn_uint_type))
return 1;
# 1545
if(
# 1548
Cyc_Tcutil_is_integral_type((void*)_check_null(e->topt))){
if(Cyc_Tcutil_will_lose_precision((void*)_check_null(e->topt),Cyc_Absyn_uint_type))
({void*_tmp249=0U;({unsigned _tmp611=e->loc;struct _fat_ptr _tmp610=({const char*_tmp24A="integral size mismatch; conversion supplied";_tag_fat(_tmp24A,sizeof(char),44U);});Cyc_Tcutil_warn(_tmp611,_tmp610,_tag_fat(_tmp249,sizeof(void*),0U));});});
# 1549
Cyc_Tcutil_unchecked_cast(e,Cyc_Absyn_uint_type,Cyc_Absyn_No_coercion);
# 1552
return 1;}
# 1545
return 0;}
# 1169
int Cyc_Tcutil_coerce_sint_type(struct Cyc_Absyn_Exp*e){
# 1559
if(Cyc_Unify_unify((void*)_check_null(e->topt),Cyc_Absyn_sint_type))
return 1;
# 1559
if(
# 1562
Cyc_Tcutil_is_integral_type((void*)_check_null(e->topt))){
if(Cyc_Tcutil_will_lose_precision((void*)_check_null(e->topt),Cyc_Absyn_sint_type))
({void*_tmp24B=0U;({unsigned _tmp613=e->loc;struct _fat_ptr _tmp612=({const char*_tmp24C="integral size mismatch; conversion supplied";_tag_fat(_tmp24C,sizeof(char),44U);});Cyc_Tcutil_warn(_tmp613,_tmp612,_tag_fat(_tmp24B,sizeof(void*),0U));});});
# 1563
Cyc_Tcutil_unchecked_cast(e,Cyc_Absyn_sint_type,Cyc_Absyn_No_coercion);
# 1566
return 1;}
# 1559
return 0;}
# 1169
int Cyc_Tcutil_force_type2bool(int desired,void*t){
# 1574
Cyc_Unify_unify(desired?Cyc_Absyn_true_type: Cyc_Absyn_false_type,t);
return Cyc_Absyn_type2bool(desired,t);}
# 1169
void*Cyc_Tcutil_force_bounds_one(void*t){
# 1580
({void*_tmp614=t;Cyc_Unify_unify(_tmp614,Cyc_Absyn_bounds_one());});
return Cyc_Tcutil_compress(t);}
# 1169
struct Cyc_Absyn_Exp*Cyc_Tcutil_get_thin_bound(struct Cyc_List_List*ts){
# 1585
void*t=Cyc_Tcutil_compress((void*)((struct Cyc_List_List*)_check_null(ts))->hd);
void*_tmp24D=t;struct Cyc_Absyn_Exp*_tmp24E;if(((struct Cyc_Absyn_ValueofType_Absyn_Type_struct*)_tmp24D)->tag == 9U){_LL1: _tmp24E=((struct Cyc_Absyn_ValueofType_Absyn_Type_struct*)_tmp24D)->f1;_LL2: {struct Cyc_Absyn_Exp*e=_tmp24E;
return e;}}else{_LL3: _LL4: {
# 1589
struct Cyc_Absyn_Exp*v=Cyc_Absyn_valueof_exp(t,0U);
v->topt=Cyc_Absyn_uint_type;
return v;}}_LL0:;}
# 1169
struct Cyc_Absyn_Exp*Cyc_Tcutil_get_bounds_exp(void*def,void*b){
# 1599
Cyc_Unify_unify(def,b);{
void*_stmttmp47=Cyc_Tcutil_compress(b);void*_tmp24F=_stmttmp47;struct Cyc_List_List*_tmp250;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp24F)->tag == 0U)switch(*((int*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp24F)->f1)){case 14U: _LL1: _LL2:
 return 0;case 13U: _LL3: _tmp250=((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp24F)->f2;_LL4: {struct Cyc_List_List*ts=_tmp250;
return Cyc_Tcutil_get_thin_bound(ts);}default: goto _LL5;}else{_LL5: _LL6:
({struct Cyc_String_pa_PrintArg_struct _tmp254=({struct Cyc_String_pa_PrintArg_struct _tmp4F3;_tmp4F3.tag=0U,({struct _fat_ptr _tmp615=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(b));_tmp4F3.f1=_tmp615;});_tmp4F3;});void*_tmp251[1U];_tmp251[0]=& _tmp254;({int(*_tmp617)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp253)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Tcutil_impos;_tmp253;});struct _fat_ptr _tmp616=({const char*_tmp252="get_bounds_exp: %s";_tag_fat(_tmp252,sizeof(char),19U);});_tmp617(_tmp616,_tag_fat(_tmp251,sizeof(void*),1U));});});}_LL0:;}}
# 1169
struct Cyc_Absyn_Exp*Cyc_Tcutil_get_ptr_bounds_exp(void*def,void*t){
# 1608
void*_stmttmp48=Cyc_Tcutil_compress(t);void*_tmp255=_stmttmp48;void*_tmp256;if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp255)->tag == 3U){_LL1: _tmp256=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp255)->f1).ptr_atts).bounds;_LL2: {void*b=_tmp256;
# 1610
return Cyc_Tcutil_get_bounds_exp(def,b);}}else{_LL3: _LL4:
({struct Cyc_String_pa_PrintArg_struct _tmp25A=({struct Cyc_String_pa_PrintArg_struct _tmp4F4;_tmp4F4.tag=0U,({struct _fat_ptr _tmp618=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t));_tmp4F4.f1=_tmp618;});_tmp4F4;});void*_tmp257[1U];_tmp257[0]=& _tmp25A;({int(*_tmp61A)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp259)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Tcutil_impos;_tmp259;});struct _fat_ptr _tmp619=({const char*_tmp258="get_ptr_bounds_exp not pointer: %s";_tag_fat(_tmp258,sizeof(char),35U);});_tmp61A(_tmp619,_tag_fat(_tmp257,sizeof(void*),1U));});});}_LL0:;}
# 1169
void*Cyc_Tcutil_any_bool(struct Cyc_List_List*tvs){
# 1617
return Cyc_Absyn_new_evar(& Cyc_Tcutil_boolko,({struct Cyc_Core_Opt*_tmp25B=_cycalloc(sizeof(*_tmp25B));_tmp25B->v=tvs;_tmp25B;}));}
# 1169
void*Cyc_Tcutil_any_bounds(struct Cyc_List_List*tvs){
# 1621
return Cyc_Absyn_new_evar(& Cyc_Tcutil_ptrbko,({struct Cyc_Core_Opt*_tmp25C=_cycalloc(sizeof(*_tmp25C));_tmp25C->v=tvs;_tmp25C;}));}
# 1169
static int Cyc_Tcutil_ptrsubtype(struct Cyc_Tcenv_Tenv*te,struct Cyc_List_List*assume,void*t1,void*t2);struct _tuple22{struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};
# 1633
int Cyc_Tcutil_silent_castable(struct Cyc_Tcenv_Tenv*te,unsigned loc,void*t1,void*t2){
t1=Cyc_Tcutil_compress(t1);
t2=Cyc_Tcutil_compress(t2);{
struct _tuple15 _stmttmp49=({struct _tuple15 _tmp4F8;_tmp4F8.f1=t1,_tmp4F8.f2=t2;_tmp4F8;});struct _tuple15 _tmp25D=_stmttmp49;void*_tmp265;struct Cyc_Absyn_Exp*_tmp264;struct Cyc_Absyn_Tqual _tmp263;void*_tmp262;void*_tmp261;struct Cyc_Absyn_Exp*_tmp260;struct Cyc_Absyn_Tqual _tmp25F;void*_tmp25E;struct Cyc_Absyn_PtrInfo _tmp267;struct Cyc_Absyn_PtrInfo _tmp266;switch(*((int*)_tmp25D.f1)){case 3U: if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp25D.f2)->tag == 3U){_LL1: _tmp266=((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp25D.f1)->f1;_tmp267=((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp25D.f2)->f1;_LL2: {struct Cyc_Absyn_PtrInfo pinfo_a=_tmp266;struct Cyc_Absyn_PtrInfo pinfo_b=_tmp267;
# 1638
int okay=1;
# 1640
if(!Cyc_Unify_unify((pinfo_a.ptr_atts).nullable,(pinfo_b.ptr_atts).nullable))
# 1642
okay=!Cyc_Tcutil_force_type2bool(0,(pinfo_a.ptr_atts).nullable);
# 1640
if(!
# 1644
Cyc_Unify_unify((pinfo_a.ptr_atts).bounds,(pinfo_b.ptr_atts).bounds)){
struct _tuple22 _stmttmp4A=({struct _tuple22 _tmp4F5;({struct Cyc_Absyn_Exp*_tmp61E=({void*_tmp61D=Cyc_Absyn_bounds_one();Cyc_Tcutil_get_bounds_exp(_tmp61D,(pinfo_a.ptr_atts).bounds);});_tmp4F5.f1=_tmp61E;}),({
struct Cyc_Absyn_Exp*_tmp61C=({void*_tmp61B=Cyc_Absyn_bounds_one();Cyc_Tcutil_get_bounds_exp(_tmp61B,(pinfo_b.ptr_atts).bounds);});_tmp4F5.f2=_tmp61C;});_tmp4F5;});
# 1645
struct _tuple22 _tmp268=_stmttmp4A;struct Cyc_Absyn_Exp*_tmp26A;struct Cyc_Absyn_Exp*_tmp269;if(_tmp268.f2 == 0){_LLA: _LLB:
# 1648
 okay=1;goto _LL9;}else{if(_tmp268.f1 == 0){_LLC: _LLD:
# 1651
 if(Cyc_Tcutil_force_type2bool(0,(pinfo_a.ptr_atts).zero_term)&&({
void*_tmp61F=Cyc_Absyn_bounds_one();Cyc_Unify_unify(_tmp61F,(pinfo_b.ptr_atts).bounds);}))
goto _LL9;
# 1651
okay=0;
# 1655
goto _LL9;}else{_LLE: _tmp269=_tmp268.f1;_tmp26A=_tmp268.f2;_LLF: {struct Cyc_Absyn_Exp*e1=_tmp269;struct Cyc_Absyn_Exp*e2=_tmp26A;
# 1657
okay=okay &&({struct Cyc_Absyn_Exp*_tmp620=(struct Cyc_Absyn_Exp*)_check_null(e2);Cyc_Evexp_lte_const_exp(_tmp620,(struct Cyc_Absyn_Exp*)_check_null(e1));});
# 1661
if(!Cyc_Tcutil_force_type2bool(0,(pinfo_b.ptr_atts).zero_term))
({void*_tmp26B=0U;({unsigned _tmp622=loc;struct _fat_ptr _tmp621=({const char*_tmp26C="implicit cast to shorter array";_tag_fat(_tmp26C,sizeof(char),31U);});Cyc_Tcutil_warn(_tmp622,_tmp621,_tag_fat(_tmp26B,sizeof(void*),0U));});});
# 1661
goto _LL9;}}}_LL9:;}
# 1640
okay=
# 1668
okay &&(!(pinfo_a.elt_tq).real_const ||(pinfo_b.elt_tq).real_const);
# 1671
if(!Cyc_Unify_unify((pinfo_a.ptr_atts).rgn,(pinfo_b.ptr_atts).rgn)){
# 1673
if(Cyc_Tcenv_rgn_outlives_rgn(te,(pinfo_a.ptr_atts).rgn,(pinfo_b.ptr_atts).rgn)){
# 1678
if(Cyc_Tcutil_warn_region_coerce)
({struct Cyc_String_pa_PrintArg_struct _tmp26F=({struct Cyc_String_pa_PrintArg_struct _tmp4F7;_tmp4F7.tag=0U,({
struct _fat_ptr _tmp623=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string((pinfo_a.ptr_atts).rgn));_tmp4F7.f1=_tmp623;});_tmp4F7;});struct Cyc_String_pa_PrintArg_struct _tmp270=({struct Cyc_String_pa_PrintArg_struct _tmp4F6;_tmp4F6.tag=0U,({
struct _fat_ptr _tmp624=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string((pinfo_b.ptr_atts).rgn));_tmp4F6.f1=_tmp624;});_tmp4F6;});void*_tmp26D[2U];_tmp26D[0]=& _tmp26F,_tmp26D[1]=& _tmp270;({unsigned _tmp626=loc;struct _fat_ptr _tmp625=({const char*_tmp26E="implicit cast from region %s to region %s";_tag_fat(_tmp26E,sizeof(char),42U);});Cyc_Tcutil_warn(_tmp626,_tmp625,_tag_fat(_tmp26D,sizeof(void*),2U));});});}else{
okay=0;}}
# 1671
okay=
# 1686
okay &&(Cyc_Unify_unify((pinfo_a.ptr_atts).zero_term,(pinfo_b.ptr_atts).zero_term)||
# 1688
 Cyc_Tcutil_force_type2bool(1,(pinfo_a.ptr_atts).zero_term)&&(pinfo_b.elt_tq).real_const);{
# 1696
int deep_subtype=
({void*_tmp627=Cyc_Absyn_bounds_one();Cyc_Unify_unify(_tmp627,(pinfo_b.ptr_atts).bounds);})&& !
Cyc_Tcutil_force_type2bool(0,(pinfo_b.ptr_atts).zero_term);
# 1702
okay=okay &&(Cyc_Unify_unify(pinfo_a.elt_type,pinfo_b.elt_type)||
(deep_subtype &&((pinfo_b.elt_tq).real_const || Cyc_Tcutil_kind_leq(& Cyc_Tcutil_ak,Cyc_Tcutil_type_kind(pinfo_b.elt_type))))&&
 Cyc_Tcutil_ptrsubtype(te,0,pinfo_a.elt_type,pinfo_b.elt_type));
# 1706
return okay;}}}else{goto _LL7;}case 4U: if(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp25D.f2)->tag == 4U){_LL3: _tmp25E=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp25D.f1)->f1).elt_type;_tmp25F=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp25D.f1)->f1).tq;_tmp260=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp25D.f1)->f1).num_elts;_tmp261=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp25D.f1)->f1).zero_term;_tmp262=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp25D.f2)->f1).elt_type;_tmp263=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp25D.f2)->f1).tq;_tmp264=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp25D.f2)->f1).num_elts;_tmp265=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp25D.f2)->f1).zero_term;_LL4: {void*t1a=_tmp25E;struct Cyc_Absyn_Tqual tq1a=_tmp25F;struct Cyc_Absyn_Exp*e1=_tmp260;void*zt1=_tmp261;void*t2a=_tmp262;struct Cyc_Absyn_Tqual tq2a=_tmp263;struct Cyc_Absyn_Exp*e2=_tmp264;void*zt2=_tmp265;
# 1710
int okay;
# 1713
okay=Cyc_Unify_unify(zt1,zt2)&&(
(e1 != 0 && e2 != 0)&& Cyc_Evexp_same_const_exp(e1,e2));
# 1716
return(okay && Cyc_Unify_unify(t1a,t2a))&&(!tq1a.real_const || tq2a.real_const);}}else{goto _LL7;}case 0U: if(((struct Cyc_Absyn_TagCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp25D.f1)->f1)->tag == 4U){if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp25D.f2)->tag == 0U){if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp25D.f2)->f1)->tag == 1U){_LL5: _LL6:
# 1718
 return 0;}else{goto _LL7;}}else{goto _LL7;}}else{goto _LL7;}default: _LL7: _LL8:
# 1720
 return Cyc_Unify_unify(t1,t2);}_LL0:;}}
# 1633
void*Cyc_Tcutil_pointer_elt_type(void*t){
# 1725
void*_stmttmp4B=Cyc_Tcutil_compress(t);void*_tmp271=_stmttmp4B;void*_tmp272;if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp271)->tag == 3U){_LL1: _tmp272=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp271)->f1).elt_type;_LL2: {void*e=_tmp272;
return e;}}else{_LL3: _LL4:
({void*_tmp273=0U;({int(*_tmp629)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp275)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Tcutil_impos;_tmp275;});struct _fat_ptr _tmp628=({const char*_tmp274="pointer_elt_type";_tag_fat(_tmp274,sizeof(char),17U);});_tmp629(_tmp628,_tag_fat(_tmp273,sizeof(void*),0U));});});}_LL0:;}
# 1633
void*Cyc_Tcutil_pointer_region(void*t){
# 1731
void*_stmttmp4C=Cyc_Tcutil_compress(t);void*_tmp276=_stmttmp4C;struct Cyc_Absyn_PtrAtts*_tmp277;if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp276)->tag == 3U){_LL1: _tmp277=(struct Cyc_Absyn_PtrAtts*)&(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp276)->f1).ptr_atts;_LL2: {struct Cyc_Absyn_PtrAtts*p=_tmp277;
return p->rgn;}}else{_LL3: _LL4:
({void*_tmp278=0U;({int(*_tmp62B)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp27A)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Tcutil_impos;_tmp27A;});struct _fat_ptr _tmp62A=({const char*_tmp279="pointer_elt_type";_tag_fat(_tmp279,sizeof(char),17U);});_tmp62B(_tmp62A,_tag_fat(_tmp278,sizeof(void*),0U));});});}_LL0:;}
# 1633
int Cyc_Tcutil_rgn_of_pointer(void*t,void**rgn){
# 1738
void*_stmttmp4D=Cyc_Tcutil_compress(t);void*_tmp27B=_stmttmp4D;void*_tmp27C;if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp27B)->tag == 3U){_LL1: _tmp27C=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp27B)->f1).ptr_atts).rgn;_LL2: {void*r=_tmp27C;
# 1740
*rgn=r;
return 1;}}else{_LL3: _LL4:
 return 0;}_LL0:;}
# 1633
int Cyc_Tcutil_admits_zero(void*t){
# 1750
void*_stmttmp4E=Cyc_Tcutil_compress(t);void*_tmp27D=_stmttmp4E;void*_tmp27F;void*_tmp27E;switch(*((int*)_tmp27D)){case 0U: switch(*((int*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp27D)->f1)){case 1U: _LL1: _LL2:
 goto _LL4;case 2U: _LL3: _LL4:
 return 1;default: goto _LL7;}case 3U: _LL5: _tmp27E=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp27D)->f1).ptr_atts).nullable;_tmp27F=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp27D)->f1).ptr_atts).bounds;_LL6: {void*nullable=_tmp27E;void*bounds=_tmp27F;
# 1757
return !Cyc_Unify_unify(Cyc_Absyn_fat_bound_type,bounds)&& Cyc_Tcutil_force_type2bool(0,nullable);}default: _LL7: _LL8:
 return 0;}_LL0:;}
# 1633
int Cyc_Tcutil_is_zero(struct Cyc_Absyn_Exp*e){
# 1764
void*_stmttmp4F=e->r;void*_tmp280=_stmttmp4F;struct Cyc_Absyn_Exp*_tmp282;void*_tmp281;struct _fat_ptr _tmp283;switch(*((int*)_tmp280)){case 0U: switch(((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmp280)->f1).Wchar_c).tag){case 5U: if((((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmp280)->f1).Int_c).val).f2 == 0){_LL1: _LL2:
 goto _LL4;}else{goto _LLF;}case 2U: if((((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmp280)->f1).Char_c).val).f2 == 0){_LL3: _LL4:
 goto _LL6;}else{goto _LLF;}case 4U: if((((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmp280)->f1).Short_c).val).f2 == 0){_LL5: _LL6:
 goto _LL8;}else{goto _LLF;}case 6U: if((((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmp280)->f1).LongLong_c).val).f2 == 0){_LL7: _LL8:
 goto _LLA;}else{goto _LLF;}case 3U: _LLB: _tmp283=((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmp280)->f1).Wchar_c).val;_LLC: {struct _fat_ptr s=_tmp283;
# 1772
unsigned long l=Cyc_strlen((struct _fat_ptr)s);
int i=0;
if(l >= (unsigned long)2 &&(int)*((const char*)_check_fat_subscript(s,sizeof(char),0))== (int)'\\'){
if((int)*((const char*)_check_fat_subscript(s,sizeof(char),1))== (int)'0')i=2;else{
if(((int)((const char*)s.curr)[1]== (int)'x' && l >= (unsigned long)3)&&(int)*((const char*)_check_fat_subscript(s,sizeof(char),2))== (int)'0')i=3;else{
return 0;}}
for(0;(unsigned long)i < l;++ i){
if((int)*((const char*)_check_fat_subscript(s,sizeof(char),i))!= (int)'0')return 0;}
# 1778
return 1;}else{
# 1782
return 0;}}default: goto _LLF;}case 2U: _LL9: _LLA:
# 1770
 return 1;case 14U: _LLD: _tmp281=(void*)((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp280)->f1;_tmp282=((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp280)->f2;_LLE: {void*t=_tmp281;struct Cyc_Absyn_Exp*e2=_tmp282;
# 1783
return Cyc_Tcutil_is_zero(e2)&& Cyc_Tcutil_admits_zero(t);}default: _LLF: _LL10:
 return 0;}_LL0:;}
# 1633
struct Cyc_Absyn_Kind Cyc_Tcutil_rk={Cyc_Absyn_RgnKind,Cyc_Absyn_Aliasable};
# 1789
struct Cyc_Absyn_Kind Cyc_Tcutil_ak={Cyc_Absyn_AnyKind,Cyc_Absyn_Aliasable};
struct Cyc_Absyn_Kind Cyc_Tcutil_bk={Cyc_Absyn_BoxKind,Cyc_Absyn_Aliasable};
struct Cyc_Absyn_Kind Cyc_Tcutil_mk={Cyc_Absyn_MemKind,Cyc_Absyn_Aliasable};
struct Cyc_Absyn_Kind Cyc_Tcutil_ik={Cyc_Absyn_IntKind,Cyc_Absyn_Aliasable};
struct Cyc_Absyn_Kind Cyc_Tcutil_ek={Cyc_Absyn_EffKind,Cyc_Absyn_Aliasable};
struct Cyc_Absyn_Kind Cyc_Tcutil_boolk={Cyc_Absyn_BoolKind,Cyc_Absyn_Aliasable};
struct Cyc_Absyn_Kind Cyc_Tcutil_ptrbk={Cyc_Absyn_PtrBndKind,Cyc_Absyn_Aliasable};
# 1797
struct Cyc_Absyn_Kind Cyc_Tcutil_trk={Cyc_Absyn_RgnKind,Cyc_Absyn_Top};
struct Cyc_Absyn_Kind Cyc_Tcutil_tak={Cyc_Absyn_AnyKind,Cyc_Absyn_Top};
struct Cyc_Absyn_Kind Cyc_Tcutil_tbk={Cyc_Absyn_BoxKind,Cyc_Absyn_Top};
struct Cyc_Absyn_Kind Cyc_Tcutil_tmk={Cyc_Absyn_MemKind,Cyc_Absyn_Top};
# 1802
struct Cyc_Absyn_Kind Cyc_Tcutil_urk={Cyc_Absyn_RgnKind,Cyc_Absyn_Unique};
struct Cyc_Absyn_Kind Cyc_Tcutil_uak={Cyc_Absyn_AnyKind,Cyc_Absyn_Unique};
struct Cyc_Absyn_Kind Cyc_Tcutil_ubk={Cyc_Absyn_BoxKind,Cyc_Absyn_Unique};
struct Cyc_Absyn_Kind Cyc_Tcutil_umk={Cyc_Absyn_MemKind,Cyc_Absyn_Unique};
# 1807
struct Cyc_Core_Opt Cyc_Tcutil_rko={(void*)& Cyc_Tcutil_rk};
struct Cyc_Core_Opt Cyc_Tcutil_ako={(void*)& Cyc_Tcutil_ak};
struct Cyc_Core_Opt Cyc_Tcutil_bko={(void*)& Cyc_Tcutil_bk};
struct Cyc_Core_Opt Cyc_Tcutil_mko={(void*)& Cyc_Tcutil_mk};
struct Cyc_Core_Opt Cyc_Tcutil_iko={(void*)& Cyc_Tcutil_ik};
struct Cyc_Core_Opt Cyc_Tcutil_eko={(void*)& Cyc_Tcutil_ek};
struct Cyc_Core_Opt Cyc_Tcutil_boolko={(void*)& Cyc_Tcutil_boolk};
struct Cyc_Core_Opt Cyc_Tcutil_ptrbko={(void*)& Cyc_Tcutil_ptrbk};
# 1816
struct Cyc_Core_Opt Cyc_Tcutil_trko={(void*)& Cyc_Tcutil_trk};
struct Cyc_Core_Opt Cyc_Tcutil_tako={(void*)& Cyc_Tcutil_tak};
struct Cyc_Core_Opt Cyc_Tcutil_tbko={(void*)& Cyc_Tcutil_tbk};
struct Cyc_Core_Opt Cyc_Tcutil_tmko={(void*)& Cyc_Tcutil_tmk};
# 1821
struct Cyc_Core_Opt Cyc_Tcutil_urko={(void*)& Cyc_Tcutil_urk};
struct Cyc_Core_Opt Cyc_Tcutil_uako={(void*)& Cyc_Tcutil_uak};
struct Cyc_Core_Opt Cyc_Tcutil_ubko={(void*)& Cyc_Tcutil_ubk};
struct Cyc_Core_Opt Cyc_Tcutil_umko={(void*)& Cyc_Tcutil_umk};
# 1826
struct Cyc_Core_Opt*Cyc_Tcutil_kind_to_opt(struct Cyc_Absyn_Kind*ka){
enum Cyc_Absyn_AliasQual _tmp285;enum Cyc_Absyn_KindQual _tmp284;_LL1: _tmp284=ka->kind;_tmp285=ka->aliasqual;_LL2: {enum Cyc_Absyn_KindQual k=_tmp284;enum Cyc_Absyn_AliasQual a=_tmp285;
{enum Cyc_Absyn_AliasQual _tmp286=a;switch(_tmp286){case Cyc_Absyn_Aliasable: _LL4: _LL5: {
# 1830
enum Cyc_Absyn_KindQual _tmp287=k;switch(_tmp287){case Cyc_Absyn_AnyKind: _LLD: _LLE:
 return& Cyc_Tcutil_ako;case Cyc_Absyn_MemKind: _LLF: _LL10:
 return& Cyc_Tcutil_mko;case Cyc_Absyn_BoxKind: _LL11: _LL12:
 return& Cyc_Tcutil_bko;case Cyc_Absyn_RgnKind: _LL13: _LL14:
 return& Cyc_Tcutil_rko;case Cyc_Absyn_EffKind: _LL15: _LL16:
 return& Cyc_Tcutil_eko;case Cyc_Absyn_IntKind: _LL17: _LL18:
 return& Cyc_Tcutil_iko;case Cyc_Absyn_BoolKind: _LL19: _LL1A:
 return& Cyc_Tcutil_bko;case Cyc_Absyn_PtrBndKind: _LL1B: _LL1C:
 goto _LL1E;default: _LL1D: _LL1E: return& Cyc_Tcutil_ptrbko;}_LLC:;}case Cyc_Absyn_Unique: _LL6: _LL7:
# 1841
{enum Cyc_Absyn_KindQual _tmp288=k;switch(_tmp288){case Cyc_Absyn_AnyKind: _LL20: _LL21:
 return& Cyc_Tcutil_uako;case Cyc_Absyn_MemKind: _LL22: _LL23:
 return& Cyc_Tcutil_umko;case Cyc_Absyn_BoxKind: _LL24: _LL25:
 return& Cyc_Tcutil_ubko;case Cyc_Absyn_RgnKind: _LL26: _LL27:
 return& Cyc_Tcutil_urko;default: _LL28: _LL29:
 goto _LL1F;}_LL1F:;}
# 1848
goto _LL3;case Cyc_Absyn_Top: _LL8: _LL9:
# 1850
{enum Cyc_Absyn_KindQual _tmp289=k;switch(_tmp289){case Cyc_Absyn_AnyKind: _LL2B: _LL2C:
 return& Cyc_Tcutil_tako;case Cyc_Absyn_MemKind: _LL2D: _LL2E:
 return& Cyc_Tcutil_tmko;case Cyc_Absyn_BoxKind: _LL2F: _LL30:
 return& Cyc_Tcutil_tbko;case Cyc_Absyn_RgnKind: _LL31: _LL32:
 return& Cyc_Tcutil_trko;default: _LL33: _LL34:
 goto _LL2A;}_LL2A:;}
# 1857
goto _LL3;default: _LLA: _LLB:
 goto _LL3;}_LL3:;}
# 1860
({struct Cyc_String_pa_PrintArg_struct _tmp28D=({struct Cyc_String_pa_PrintArg_struct _tmp4F9;_tmp4F9.tag=0U,({struct _fat_ptr _tmp62C=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_kind2string(ka));_tmp4F9.f1=_tmp62C;});_tmp4F9;});void*_tmp28A[1U];_tmp28A[0]=& _tmp28D;({int(*_tmp62E)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp28C)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Tcutil_impos;_tmp28C;});struct _fat_ptr _tmp62D=({const char*_tmp28B="kind_to_opt: bad kind %s\n";_tag_fat(_tmp28B,sizeof(char),26U);});_tmp62E(_tmp62D,_tag_fat(_tmp28A,sizeof(void*),1U));});});}}
# 1826
void*Cyc_Tcutil_kind_to_bound(struct Cyc_Absyn_Kind*k){
# 1864
return(void*)({struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct*_tmp28E=_cycalloc(sizeof(*_tmp28E));_tmp28E->tag=0U,_tmp28E->f1=k;_tmp28E;});}
# 1826
struct Cyc_Core_Opt*Cyc_Tcutil_kind_to_bound_opt(struct Cyc_Absyn_Kind*k){
# 1867
return({struct Cyc_Core_Opt*_tmp28F=_cycalloc(sizeof(*_tmp28F));({void*_tmp62F=Cyc_Tcutil_kind_to_bound(k);_tmp28F->v=_tmp62F;});_tmp28F;});}
# 1826
int Cyc_Tcutil_zero_to_null(void*t2,struct Cyc_Absyn_Exp*e1){
# 1873
if(!Cyc_Tcutil_is_zero(e1))
return 0;{
# 1873
void*_stmttmp50=
# 1875
Cyc_Tcutil_compress(t2);
# 1873
void*_tmp290=_stmttmp50;void*_tmp291;if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp290)->tag == 3U){_LL1: _tmp291=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp290)->f1).ptr_atts).nullable;_LL2: {void*nbl=_tmp291;
# 1877
if(!Cyc_Tcutil_force_type2bool(1,nbl))
return 0;
# 1877
({void*_tmp630=(
# 1879
Cyc_Absyn_null_exp(0U))->r;
# 1877
e1->r=_tmp630;});
# 1880
e1->topt=t2;
return 1;}}else{_LL3: _LL4:
 return 0;}_LL0:;}}
# 1826
int Cyc_Tcutil_warn_alias_coerce=0;struct _tuple23{struct Cyc_Absyn_Decl*f1;struct Cyc_Absyn_Exp*f2;};
# 1892
struct _tuple23 Cyc_Tcutil_insert_alias(struct Cyc_Absyn_Exp*e,void*e_type){
static struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct rgn_kb={0U,& Cyc_Tcutil_rk};
# 1896
static int counter=0;
struct _tuple1*v=({struct _tuple1*_tmp2A2=_cycalloc(sizeof(*_tmp2A2));_tmp2A2->f1=Cyc_Absyn_Loc_n,({struct _fat_ptr*_tmp633=({struct _fat_ptr*_tmp2A1=_cycalloc(sizeof(*_tmp2A1));({struct _fat_ptr _tmp632=(struct _fat_ptr)({struct Cyc_Int_pa_PrintArg_struct _tmp2A0=({struct Cyc_Int_pa_PrintArg_struct _tmp4FB;_tmp4FB.tag=1U,_tmp4FB.f1=(unsigned long)counter ++;_tmp4FB;});void*_tmp29E[1U];_tmp29E[0]=& _tmp2A0;({struct _fat_ptr _tmp631=({const char*_tmp29F="__aliasvar%d";_tag_fat(_tmp29F,sizeof(char),13U);});Cyc_aprintf(_tmp631,_tag_fat(_tmp29E,sizeof(void*),1U));});});*_tmp2A1=_tmp632;});_tmp2A1;});_tmp2A2->f2=_tmp633;});_tmp2A2;});
struct Cyc_Absyn_Vardecl*vd=Cyc_Absyn_new_vardecl(0U,v,e_type,e);
struct Cyc_Absyn_Exp*ve=({void*_tmp634=(void*)({struct Cyc_Absyn_Local_b_Absyn_Binding_struct*_tmp29D=_cycalloc(sizeof(*_tmp29D));_tmp29D->tag=4U,_tmp29D->f1=vd;_tmp29D;});Cyc_Absyn_varb_exp(_tmp634,e->loc);});
# 1905
struct Cyc_Absyn_Tvar*tv=Cyc_Tcutil_new_tvar((void*)& rgn_kb);
# 1907
{void*_stmttmp51=Cyc_Tcutil_compress(e_type);void*_tmp292=_stmttmp51;struct Cyc_Absyn_PtrLoc*_tmp299;void*_tmp298;void*_tmp297;void*_tmp296;void*_tmp295;struct Cyc_Absyn_Tqual _tmp294;void*_tmp293;if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp292)->tag == 3U){_LL1: _tmp293=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp292)->f1).elt_type;_tmp294=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp292)->f1).elt_tq;_tmp295=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp292)->f1).ptr_atts).rgn;_tmp296=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp292)->f1).ptr_atts).nullable;_tmp297=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp292)->f1).ptr_atts).bounds;_tmp298=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp292)->f1).ptr_atts).zero_term;_tmp299=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp292)->f1).ptr_atts).ptrloc;_LL2: {void*et=_tmp293;struct Cyc_Absyn_Tqual tq=_tmp294;void*old_r=_tmp295;void*nb=_tmp296;void*b=_tmp297;void*zt=_tmp298;struct Cyc_Absyn_PtrLoc*pl=_tmp299;
# 1909
{void*_stmttmp52=Cyc_Tcutil_compress(old_r);void*_tmp29A=_stmttmp52;struct Cyc_Core_Opt*_tmp29C;void**_tmp29B;if(((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp29A)->tag == 1U){_LL6: _tmp29B=(void**)&((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp29A)->f2;_tmp29C=((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp29A)->f4;_LL7: {void**topt=_tmp29B;struct Cyc_Core_Opt*ts=_tmp29C;
# 1911
void*new_r=Cyc_Absyn_var_type(tv);
*topt=new_r;
goto _LL5;}}else{_LL8: _LL9:
 goto _LL5;}_LL5:;}
# 1916
goto _LL0;}}else{_LL3: _LL4:
 goto _LL0;}_LL0:;}
# 1920
e->topt=0;
vd->initializer=0;{
# 1924
struct Cyc_Absyn_Decl*d=Cyc_Absyn_alias_decl(tv,vd,e,e->loc);
# 1926
return({struct _tuple23 _tmp4FA;_tmp4FA.f1=d,_tmp4FA.f2=ve;_tmp4FA;});}}
# 1892
static int Cyc_Tcutil_can_insert_alias(struct Cyc_Absyn_Exp*e,void*e_type,void*wants_type,unsigned loc){
# 1934
if((Cyc_Tcutil_is_noalias_path(e)&&
 Cyc_Tcutil_is_noalias_pointer(e_type,0))&&
 Cyc_Tcutil_is_pointer_type(e_type)){
# 1939
void*_stmttmp53=Cyc_Tcutil_compress(wants_type);void*_tmp2A3=_stmttmp53;void*_tmp2A4;if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp2A3)->tag == 3U){_LL1: _tmp2A4=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp2A3)->f1).ptr_atts).rgn;_LL2: {void*r2=_tmp2A4;
# 1941
if(Cyc_Tcutil_is_heap_rgn_type(r2))return 0;{struct Cyc_Absyn_Kind*k=
Cyc_Tcutil_type_kind(r2);
return(int)k->kind == (int)Cyc_Absyn_RgnKind &&(int)k->aliasqual == (int)Cyc_Absyn_Aliasable;}}}else{_LL3: _LL4:
 return 0;}_LL0:;}
# 1934
return 0;}
# 1892
int Cyc_Tcutil_coerce_arg(struct Cyc_Tcenv_Tenv*te,struct Cyc_Absyn_Exp*e,void*t2,int*alias_coercion){
# 1954
void*t1=Cyc_Tcutil_compress((void*)_check_null(e->topt));
enum Cyc_Absyn_Coercion c;
int do_alias_coercion=0;
# 1958
if(Cyc_Unify_unify(t1,t2))
return 1;
# 1958
if(
# 1961
Cyc_Tcutil_is_arithmetic_type(t2)&& Cyc_Tcutil_is_arithmetic_type(t1)){
# 1963
if(Cyc_Tcutil_will_lose_precision(t1,t2))
({struct Cyc_String_pa_PrintArg_struct _tmp2A7=({struct Cyc_String_pa_PrintArg_struct _tmp4FD;_tmp4FD.tag=0U,({
struct _fat_ptr _tmp635=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t1));_tmp4FD.f1=_tmp635;});_tmp4FD;});struct Cyc_String_pa_PrintArg_struct _tmp2A8=({struct Cyc_String_pa_PrintArg_struct _tmp4FC;_tmp4FC.tag=0U,({struct _fat_ptr _tmp636=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t2));_tmp4FC.f1=_tmp636;});_tmp4FC;});void*_tmp2A5[2U];_tmp2A5[0]=& _tmp2A7,_tmp2A5[1]=& _tmp2A8;({unsigned _tmp638=e->loc;struct _fat_ptr _tmp637=({const char*_tmp2A6="integral size mismatch; %s -> %s conversion supplied";_tag_fat(_tmp2A6,sizeof(char),53U);});Cyc_Tcutil_warn(_tmp638,_tmp637,_tag_fat(_tmp2A5,sizeof(void*),2U));});});
# 1963
Cyc_Tcutil_unchecked_cast(e,t2,Cyc_Absyn_No_coercion);
# 1967
return 1;}
# 1958
if(
# 1971
Cyc_Tcutil_can_insert_alias(e,t1,t2,e->loc)){
if(Cyc_Tcutil_warn_alias_coerce)
({struct Cyc_String_pa_PrintArg_struct _tmp2AB=({struct Cyc_String_pa_PrintArg_struct _tmp500;_tmp500.tag=0U,({
struct _fat_ptr _tmp639=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_exp2string(e));_tmp500.f1=_tmp639;});_tmp500;});struct Cyc_String_pa_PrintArg_struct _tmp2AC=({struct Cyc_String_pa_PrintArg_struct _tmp4FF;_tmp4FF.tag=0U,({struct _fat_ptr _tmp63A=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t1));_tmp4FF.f1=_tmp63A;});_tmp4FF;});struct Cyc_String_pa_PrintArg_struct _tmp2AD=({struct Cyc_String_pa_PrintArg_struct _tmp4FE;_tmp4FE.tag=0U,({struct _fat_ptr _tmp63B=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t2));_tmp4FE.f1=_tmp63B;});_tmp4FE;});void*_tmp2A9[3U];_tmp2A9[0]=& _tmp2AB,_tmp2A9[1]=& _tmp2AC,_tmp2A9[2]=& _tmp2AD;({unsigned _tmp63D=e->loc;struct _fat_ptr _tmp63C=({const char*_tmp2AA="implicit alias coercion for %s:%s to %s";_tag_fat(_tmp2AA,sizeof(char),40U);});Cyc_Tcutil_warn(_tmp63D,_tmp63C,_tag_fat(_tmp2A9,sizeof(void*),3U));});});
# 1972
*alias_coercion=1;}
# 1958
if(
# 1978
Cyc_Tcutil_silent_castable(te,e->loc,t1,t2)){
Cyc_Tcutil_unchecked_cast(e,t2,Cyc_Absyn_Other_coercion);
return 1;}
# 1958
if(
# 1982
Cyc_Tcutil_zero_to_null(t2,e))
return 1;
# 1958
if((int)(c=
# 1984
Cyc_Tcutil_castable(te,e->loc,t1,t2))!= (int)Cyc_Absyn_Unknown_coercion){
# 1986
if((int)c != (int)1U)
Cyc_Tcutil_unchecked_cast(e,t2,c);
# 1986
if((int)c != (int)2U)
# 1989
({struct Cyc_String_pa_PrintArg_struct _tmp2B0=({struct Cyc_String_pa_PrintArg_struct _tmp502;_tmp502.tag=0U,({struct _fat_ptr _tmp63E=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t1));_tmp502.f1=_tmp63E;});_tmp502;});struct Cyc_String_pa_PrintArg_struct _tmp2B1=({struct Cyc_String_pa_PrintArg_struct _tmp501;_tmp501.tag=0U,({struct _fat_ptr _tmp63F=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t2));_tmp501.f1=_tmp63F;});_tmp501;});void*_tmp2AE[2U];_tmp2AE[0]=& _tmp2B0,_tmp2AE[1]=& _tmp2B1;({unsigned _tmp641=e->loc;struct _fat_ptr _tmp640=({const char*_tmp2AF="implicit cast from %s to %s";_tag_fat(_tmp2AF,sizeof(char),28U);});Cyc_Tcutil_warn(_tmp641,_tmp640,_tag_fat(_tmp2AE,sizeof(void*),2U));});});
# 1986
return 1;}
# 1958
return 0;}
# 1892
int Cyc_Tcutil_coerce_assign(struct Cyc_Tcenv_Tenv*te,struct Cyc_Absyn_Exp*e,void*t){
# 2001
int bogus=0;
return Cyc_Tcutil_coerce_arg(te,e,t,& bogus);}
# 1892
static struct Cyc_List_List*Cyc_Tcutil_flatten_type(struct _RegionHandle*r,int flatten,void*t1);struct _tuple24{struct Cyc_List_List*f1;struct _RegionHandle*f2;int f3;};
# 2018 "tcutil.cyc"
static struct Cyc_List_List*Cyc_Tcutil_flatten_type_f(struct _tuple24*env,struct Cyc_Absyn_Aggrfield*x){
# 2021
struct _tuple24 _stmttmp54=*env;int _tmp2B4;struct _RegionHandle*_tmp2B3;struct Cyc_List_List*_tmp2B2;_LL1: _tmp2B2=_stmttmp54.f1;_tmp2B3=_stmttmp54.f2;_tmp2B4=_stmttmp54.f3;_LL2: {struct Cyc_List_List*inst=_tmp2B2;struct _RegionHandle*r=_tmp2B3;int flatten=_tmp2B4;
void*t=inst == 0?x->type: Cyc_Tcutil_rsubstitute(r,inst,x->type);
struct Cyc_List_List*ts=Cyc_Tcutil_flatten_type(r,flatten,t);
if(Cyc_List_length(ts)== 1)
return({struct Cyc_List_List*_tmp2B6=_region_malloc(r,sizeof(*_tmp2B6));({struct _tuple12*_tmp642=({struct _tuple12*_tmp2B5=_region_malloc(r,sizeof(*_tmp2B5));_tmp2B5->f1=x->tq,_tmp2B5->f2=t;_tmp2B5;});_tmp2B6->hd=_tmp642;}),_tmp2B6->tl=0;_tmp2B6;});
# 2024
return ts;}}struct _tuple25{struct _RegionHandle*f1;int f2;};
# 2018
static struct Cyc_List_List*Cyc_Tcutil_rcopy_tqt(struct _tuple25*env,struct _tuple12*x){
# 2030
struct _tuple25 _stmttmp55=*env;int _tmp2B8;struct _RegionHandle*_tmp2B7;_LL1: _tmp2B7=_stmttmp55.f1;_tmp2B8=_stmttmp55.f2;_LL2: {struct _RegionHandle*r=_tmp2B7;int flatten=_tmp2B8;
struct _tuple12 _stmttmp56=*x;void*_tmp2BA;struct Cyc_Absyn_Tqual _tmp2B9;_LL4: _tmp2B9=_stmttmp56.f1;_tmp2BA=_stmttmp56.f2;_LL5: {struct Cyc_Absyn_Tqual tq=_tmp2B9;void*t=_tmp2BA;
struct Cyc_List_List*ts=Cyc_Tcutil_flatten_type(r,flatten,t);
if(Cyc_List_length(ts)== 1)
return({struct Cyc_List_List*_tmp2BC=_region_malloc(r,sizeof(*_tmp2BC));({struct _tuple12*_tmp643=({struct _tuple12*_tmp2BB=_region_malloc(r,sizeof(*_tmp2BB));_tmp2BB->f1=tq,_tmp2BB->f2=t;_tmp2BB;});_tmp2BC->hd=_tmp643;}),_tmp2BC->tl=0;_tmp2BC;});
# 2033
return ts;}}}
# 2018
static struct Cyc_List_List*Cyc_Tcutil_flatten_type(struct _RegionHandle*r,int flatten,void*t1){
# 2040
if(flatten){
t1=Cyc_Tcutil_compress(t1);{
void*_tmp2BD=t1;struct Cyc_List_List*_tmp2BE;struct Cyc_List_List*_tmp2BF;struct Cyc_List_List*_tmp2C1;struct Cyc_Absyn_Aggrdecl*_tmp2C0;switch(*((int*)_tmp2BD)){case 0U: switch(*((int*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2BD)->f1)){case 0U: _LL1: _LL2:
 return 0;case 20U: if(((((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2BD)->f1)->f1).KnownAggr).tag == 2){_LL5: _tmp2C0=*((((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2BD)->f1)->f1).KnownAggr).val;_tmp2C1=((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2BD)->f2;_LL6: {struct Cyc_Absyn_Aggrdecl*ad=_tmp2C0;struct Cyc_List_List*ts=_tmp2C1;
# 2058
if((((int)ad->kind == (int)Cyc_Absyn_UnionA || ad->impl == 0)||((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->exist_vars != 0)||((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->rgn_po != 0)
# 2060
return({struct Cyc_List_List*_tmp2C9=_region_malloc(r,sizeof(*_tmp2C9));({struct _tuple12*_tmp645=({struct _tuple12*_tmp2C8=_region_malloc(r,sizeof(*_tmp2C8));({struct Cyc_Absyn_Tqual _tmp644=Cyc_Absyn_empty_tqual(0U);_tmp2C8->f1=_tmp644;}),_tmp2C8->f2=t1;_tmp2C8;});_tmp2C9->hd=_tmp645;}),_tmp2C9->tl=0;_tmp2C9;});{
# 2058
struct Cyc_List_List*inst=
# 2061
Cyc_List_rzip(r,r,ad->tvs,ts);
struct _tuple24 env=({struct _tuple24 _tmp503;_tmp503.f1=inst,_tmp503.f2=r,_tmp503.f3=flatten;_tmp503;});
struct Cyc_List_List*_stmttmp57=((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->fields;struct Cyc_List_List*_tmp2CA=_stmttmp57;struct Cyc_List_List*_tmp2CC;struct Cyc_Absyn_Aggrfield*_tmp2CB;if(_tmp2CA == 0){_LL11: _LL12:
 return 0;}else{_LL13: _tmp2CB=(struct Cyc_Absyn_Aggrfield*)_tmp2CA->hd;_tmp2CC=_tmp2CA->tl;_LL14: {struct Cyc_Absyn_Aggrfield*hd=_tmp2CB;struct Cyc_List_List*tl=_tmp2CC;
# 2066
struct Cyc_List_List*hd2=Cyc_Tcutil_flatten_type_f(& env,hd);
env.f3=0;{
struct Cyc_List_List*tl2=({struct Cyc_List_List*(*_tmp647)(struct _RegionHandle*,struct Cyc_List_List*(*f)(struct _tuple24*,struct Cyc_Absyn_Aggrfield*),struct _tuple24*env,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp2CE)(struct _RegionHandle*,struct Cyc_List_List*(*f)(struct _tuple24*,struct Cyc_Absyn_Aggrfield*),struct _tuple24*env,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct _RegionHandle*,struct Cyc_List_List*(*f)(struct _tuple24*,struct Cyc_Absyn_Aggrfield*),struct _tuple24*env,struct Cyc_List_List*x))Cyc_List_rmap_c;_tmp2CE;});struct _RegionHandle*_tmp646=r;_tmp647(_tmp646,Cyc_Tcutil_flatten_type_f,& env,tl);});
struct Cyc_List_List*tts=({struct Cyc_List_List*_tmp2CD=_region_malloc(r,sizeof(*_tmp2CD));_tmp2CD->hd=hd2,_tmp2CD->tl=tl2;_tmp2CD;});
return Cyc_List_rflatten(r,tts);}}}_LL10:;}}}else{goto _LL9;}default: goto _LL9;}case 6U: _LL3: _tmp2BF=((struct Cyc_Absyn_TupleType_Absyn_Type_struct*)_tmp2BD)->f1;_LL4: {struct Cyc_List_List*tqs=_tmp2BF;
# 2045
struct _tuple25 env=({struct _tuple25 _tmp504;_tmp504.f1=r,_tmp504.f2=flatten;_tmp504;});
# 2047
struct Cyc_List_List*_tmp2C2=tqs;struct Cyc_List_List*_tmp2C4;struct _tuple12*_tmp2C3;if(_tmp2C2 == 0){_LLC: _LLD:
 return 0;}else{_LLE: _tmp2C3=(struct _tuple12*)_tmp2C2->hd;_tmp2C4=_tmp2C2->tl;_LLF: {struct _tuple12*hd=_tmp2C3;struct Cyc_List_List*tl=_tmp2C4;
# 2050
struct Cyc_List_List*hd2=Cyc_Tcutil_rcopy_tqt(& env,hd);
env.f2=0;{
struct Cyc_List_List*tl2=({struct Cyc_List_List*(*_tmp649)(struct _RegionHandle*,struct Cyc_List_List*(*f)(struct _tuple25*,struct _tuple12*),struct _tuple25*env,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp2C7)(struct _RegionHandle*,struct Cyc_List_List*(*f)(struct _tuple25*,struct _tuple12*),struct _tuple25*env,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct _RegionHandle*,struct Cyc_List_List*(*f)(struct _tuple25*,struct _tuple12*),struct _tuple25*env,struct Cyc_List_List*x))Cyc_List_rmap_c;_tmp2C7;});struct _RegionHandle*_tmp648=r;_tmp649(_tmp648,Cyc_Tcutil_rcopy_tqt,& env,tqs);});
struct Cyc_List_List*temp=({struct Cyc_List_List*_tmp2C6=_region_malloc(r,sizeof(*_tmp2C6));_tmp2C6->hd=hd2,_tmp2C6->tl=tl2;_tmp2C6;});
return({struct _RegionHandle*_tmp64A=r;Cyc_List_rflatten(_tmp64A,({struct Cyc_List_List*_tmp2C5=_region_malloc(r,sizeof(*_tmp2C5));_tmp2C5->hd=hd2,_tmp2C5->tl=tl2;_tmp2C5;}));});}}}_LLB:;}case 7U: if(((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmp2BD)->f1 == Cyc_Absyn_StructA){_LL7: _tmp2BE=((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmp2BD)->f2;_LL8: {struct Cyc_List_List*fs=_tmp2BE;
# 2073
struct _tuple24 env=({struct _tuple24 _tmp505;_tmp505.f1=0,_tmp505.f2=r,_tmp505.f3=flatten;_tmp505;});
struct Cyc_List_List*_tmp2CF=fs;struct Cyc_List_List*_tmp2D1;struct Cyc_Absyn_Aggrfield*_tmp2D0;if(_tmp2CF == 0){_LL16: _LL17:
 return 0;}else{_LL18: _tmp2D0=(struct Cyc_Absyn_Aggrfield*)_tmp2CF->hd;_tmp2D1=_tmp2CF->tl;_LL19: {struct Cyc_Absyn_Aggrfield*hd=_tmp2D0;struct Cyc_List_List*tl=_tmp2D1;
# 2077
struct Cyc_List_List*hd2=Cyc_Tcutil_flatten_type_f(& env,hd);
env.f3=0;{
struct Cyc_List_List*tl2=({struct Cyc_List_List*(*_tmp64C)(struct _RegionHandle*,struct Cyc_List_List*(*f)(struct _tuple24*,struct Cyc_Absyn_Aggrfield*),struct _tuple24*env,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp2D3)(struct _RegionHandle*,struct Cyc_List_List*(*f)(struct _tuple24*,struct Cyc_Absyn_Aggrfield*),struct _tuple24*env,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct _RegionHandle*,struct Cyc_List_List*(*f)(struct _tuple24*,struct Cyc_Absyn_Aggrfield*),struct _tuple24*env,struct Cyc_List_List*x))Cyc_List_rmap_c;_tmp2D3;});struct _RegionHandle*_tmp64B=r;_tmp64C(_tmp64B,Cyc_Tcutil_flatten_type_f,& env,tl);});
struct Cyc_List_List*tts=({struct Cyc_List_List*_tmp2D2=_region_malloc(r,sizeof(*_tmp2D2));_tmp2D2->hd=hd2,_tmp2D2->tl=tl2;_tmp2D2;});
return Cyc_List_rflatten(r,tts);}}}_LL15:;}}else{goto _LL9;}default: _LL9: _LLA:
# 2083
 goto _LL0;}_LL0:;}}
# 2040
return({struct Cyc_List_List*_tmp2D5=_region_malloc(r,sizeof(*_tmp2D5));
# 2086
({struct _tuple12*_tmp64E=({struct _tuple12*_tmp2D4=_region_malloc(r,sizeof(*_tmp2D4));({struct Cyc_Absyn_Tqual _tmp64D=Cyc_Absyn_empty_tqual(0U);_tmp2D4->f1=_tmp64D;}),_tmp2D4->f2=t1;_tmp2D4;});_tmp2D5->hd=_tmp64E;}),_tmp2D5->tl=0;_tmp2D5;});}
# 2018
static int Cyc_Tcutil_sub_attributes(struct Cyc_List_List*a1,struct Cyc_List_List*a2){
# 2091
{struct Cyc_List_List*t=a1;for(0;t != 0;t=t->tl){
void*_stmttmp58=(void*)t->hd;void*_tmp2D6=_stmttmp58;switch(*((int*)_tmp2D6)){case 23U: _LL1: _LL2:
 goto _LL4;case 4U: _LL3: _LL4:
 goto _LL6;case 20U: _LL5: _LL6:
 continue;default: _LL7: _LL8:
# 2097
 if(!Cyc_List_exists_c(Cyc_Absyn_equal_att,(void*)t->hd,a2))return 0;}_LL0:;}}
# 2100
for(0;a2 != 0;a2=a2->tl){
if(!Cyc_List_exists_c(Cyc_Absyn_equal_att,(void*)a2->hd,a1))return 0;}
# 2103
return 1;}
# 2018
static int Cyc_Tcutil_isomorphic(void*t1,void*t2){
# 2107
struct _tuple15 _stmttmp59=({struct _tuple15 _tmp506;({void*_tmp650=Cyc_Tcutil_compress(t1);_tmp506.f1=_tmp650;}),({void*_tmp64F=Cyc_Tcutil_compress(t2);_tmp506.f2=_tmp64F;});_tmp506;});struct _tuple15 _tmp2D7=_stmttmp59;enum Cyc_Absyn_Size_of _tmp2D9;enum Cyc_Absyn_Size_of _tmp2D8;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2D7.f1)->tag == 0U){if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2D7.f1)->f1)->tag == 1U){if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2D7.f2)->tag == 0U){if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2D7.f2)->f1)->tag == 1U){_LL1: _tmp2D8=((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2D7.f1)->f1)->f2;_tmp2D9=((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2D7.f2)->f1)->f2;_LL2: {enum Cyc_Absyn_Size_of b1=_tmp2D8;enum Cyc_Absyn_Size_of b2=_tmp2D9;
# 2109
return((int)b1 == (int)b2 ||(int)b1 == (int)2U &&(int)b2 == (int)3U)||
(int)b1 == (int)3U &&(int)b2 == (int)Cyc_Absyn_Int_sz;}}else{goto _LL3;}}else{goto _LL3;}}else{goto _LL3;}}else{_LL3: _LL4:
 return 0;}_LL0:;}
# 2018
int Cyc_Tcutil_subtype(struct Cyc_Tcenv_Tenv*te,struct Cyc_List_List*assume,void*t1,void*t2){
# 2119
if(Cyc_Unify_unify(t1,t2))return 1;{
struct Cyc_List_List*a=assume;
# 2119
for(0;a != 0;a=a->tl){
# 2121
if(Cyc_Unify_unify(t1,(*((struct _tuple15*)a->hd)).f1)&& Cyc_Unify_unify(t2,(*((struct _tuple15*)a->hd)).f2))
return 1;}}
# 2119
t1=
# 2123
Cyc_Tcutil_compress(t1);
t2=Cyc_Tcutil_compress(t2);{
struct _tuple15 _stmttmp5A=({struct _tuple15 _tmp507;_tmp507.f1=t1,_tmp507.f2=t2;_tmp507;});struct _tuple15 _tmp2DA=_stmttmp5A;struct Cyc_Absyn_FnInfo _tmp2DC;struct Cyc_Absyn_FnInfo _tmp2DB;struct Cyc_List_List*_tmp2E1;struct Cyc_Absyn_Datatypedecl*_tmp2E0;struct Cyc_List_List*_tmp2DF;struct Cyc_Absyn_Datatypefield*_tmp2DE;struct Cyc_Absyn_Datatypedecl*_tmp2DD;void*_tmp2ED;void*_tmp2EC;void*_tmp2EB;void*_tmp2EA;struct Cyc_Absyn_Tqual _tmp2E9;void*_tmp2E8;void*_tmp2E7;void*_tmp2E6;void*_tmp2E5;void*_tmp2E4;struct Cyc_Absyn_Tqual _tmp2E3;void*_tmp2E2;switch(*((int*)_tmp2DA.f1)){case 3U: if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp2DA.f2)->tag == 3U){_LL1: _tmp2E2=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp2DA.f1)->f1).elt_type;_tmp2E3=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp2DA.f1)->f1).elt_tq;_tmp2E4=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp2DA.f1)->f1).ptr_atts).rgn;_tmp2E5=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp2DA.f1)->f1).ptr_atts).nullable;_tmp2E6=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp2DA.f1)->f1).ptr_atts).bounds;_tmp2E7=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp2DA.f1)->f1).ptr_atts).zero_term;_tmp2E8=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp2DA.f2)->f1).elt_type;_tmp2E9=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp2DA.f2)->f1).elt_tq;_tmp2EA=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp2DA.f2)->f1).ptr_atts).rgn;_tmp2EB=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp2DA.f2)->f1).ptr_atts).nullable;_tmp2EC=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp2DA.f2)->f1).ptr_atts).bounds;_tmp2ED=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp2DA.f2)->f1).ptr_atts).zero_term;_LL2: {void*t_a=_tmp2E2;struct Cyc_Absyn_Tqual q_a=_tmp2E3;void*rt_a=_tmp2E4;void*null_a=_tmp2E5;void*b_a=_tmp2E6;void*zt_a=_tmp2E7;void*t_b=_tmp2E8;struct Cyc_Absyn_Tqual q_b=_tmp2E9;void*rt_b=_tmp2EA;void*null_b=_tmp2EB;void*b_b=_tmp2EC;void*zt_b=_tmp2ED;
# 2131
if(q_a.real_const && !q_b.real_const)
return 0;
# 2131
if(
# 2134
(!Cyc_Unify_unify(null_a,null_b)&&
 Cyc_Absyn_type2bool(0,null_a))&& !Cyc_Absyn_type2bool(0,null_b))
return 0;
# 2131
if(
# 2138
(Cyc_Unify_unify(zt_a,zt_b)&& !
Cyc_Absyn_type2bool(0,zt_a))&& Cyc_Absyn_type2bool(0,zt_b))
return 0;
# 2131
if(
# 2142
(!Cyc_Unify_unify(rt_a,rt_b)&& !Cyc_Tcenv_rgn_outlives_rgn(te,rt_a,rt_b))&& !
Cyc_Tcutil_subtype(te,assume,rt_a,rt_b))
return 0;
# 2131
if(!
# 2146
Cyc_Unify_unify(b_a,b_b)){
struct Cyc_Absyn_Exp*e1=({void*_tmp651=Cyc_Absyn_bounds_one();Cyc_Tcutil_get_bounds_exp(_tmp651,b_a);});
struct Cyc_Absyn_Exp*e2=({void*_tmp652=Cyc_Absyn_bounds_one();Cyc_Tcutil_get_bounds_exp(_tmp652,b_b);});
if(e1 != e2){
if((e1 == 0 || e2 == 0)|| !Cyc_Evexp_lte_const_exp(e2,e2))
return 0;}}
# 2131
if(
# 2156
!q_b.real_const && q_a.real_const){
if(!Cyc_Tcutil_kind_leq(& Cyc_Tcutil_ak,Cyc_Tcutil_type_kind(t_b)))
return 0;}{
# 2131
int deep_subtype=
# 2163
({void*_tmp653=b_b;Cyc_Unify_unify(_tmp653,Cyc_Absyn_bounds_one());})&& !Cyc_Tcutil_force_type2bool(0,zt_b);
# 2167
return(deep_subtype &&({struct Cyc_Tcenv_Tenv*_tmp657=te;struct Cyc_List_List*_tmp656=({struct Cyc_List_List*_tmp2EF=_cycalloc(sizeof(*_tmp2EF));({struct _tuple15*_tmp654=({struct _tuple15*_tmp2EE=_cycalloc(sizeof(*_tmp2EE));_tmp2EE->f1=t1,_tmp2EE->f2=t2;_tmp2EE;});_tmp2EF->hd=_tmp654;}),_tmp2EF->tl=assume;_tmp2EF;});void*_tmp655=t_a;Cyc_Tcutil_ptrsubtype(_tmp657,_tmp656,_tmp655,t_b);})|| Cyc_Unify_unify(t_a,t_b))|| Cyc_Tcutil_isomorphic(t_a,t_b);}}}else{goto _LL7;}case 0U: if(((struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2DA.f1)->f1)->tag == 19U){if(((((struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2DA.f1)->f1)->f1).KnownDatatypefield).tag == 2){if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2DA.f2)->tag == 0U){if(((struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2DA.f2)->f1)->tag == 18U){if(((((struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2DA.f2)->f1)->f1).KnownDatatype).tag == 2){_LL3: _tmp2DD=(((((struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2DA.f1)->f1)->f1).KnownDatatypefield).val).f1;_tmp2DE=(((((struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2DA.f1)->f1)->f1).KnownDatatypefield).val).f2;_tmp2DF=((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2DA.f1)->f2;_tmp2E0=*((((struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2DA.f2)->f1)->f1).KnownDatatype).val;_tmp2E1=((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2DA.f2)->f2;_LL4: {struct Cyc_Absyn_Datatypedecl*dd1=_tmp2DD;struct Cyc_Absyn_Datatypefield*df=_tmp2DE;struct Cyc_List_List*ts1=_tmp2DF;struct Cyc_Absyn_Datatypedecl*dd2=_tmp2E0;struct Cyc_List_List*ts2=_tmp2E1;
# 2174
if(dd1 != dd2 && Cyc_Absyn_qvar_cmp(dd1->name,dd2->name)!= 0)return 0;if(({
# 2176
int _tmp658=Cyc_List_length(ts1);_tmp658 != Cyc_List_length(ts2);}))return 0;
# 2174
for(0;ts1 != 0;(
# 2177
ts1=ts1->tl,ts2=ts2->tl)){
if(!Cyc_Unify_unify((void*)ts1->hd,(void*)((struct Cyc_List_List*)_check_null(ts2))->hd))return 0;}
# 2174
return 1;}}else{goto _LL7;}}else{goto _LL7;}}else{goto _LL7;}}else{goto _LL7;}}else{goto _LL7;}case 5U: if(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp2DA.f2)->tag == 5U){_LL5: _tmp2DB=((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp2DA.f1)->f1;_tmp2DC=((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp2DA.f2)->f1;_LL6: {struct Cyc_Absyn_FnInfo f1=_tmp2DB;struct Cyc_Absyn_FnInfo f2=_tmp2DC;
# 2183
if(f1.tvars != 0 || f2.tvars != 0){
struct Cyc_List_List*tvs1=f1.tvars;
struct Cyc_List_List*tvs2=f2.tvars;
if(({int _tmp659=Cyc_List_length(tvs1);_tmp659 != Cyc_List_length(tvs2);}))return 0;{struct Cyc_List_List*inst=0;
# 2188
while(tvs1 != 0){
if(!Cyc_Unify_unify_kindbound(((struct Cyc_Absyn_Tvar*)tvs1->hd)->kind,((struct Cyc_Absyn_Tvar*)((struct Cyc_List_List*)_check_null(tvs2))->hd)->kind))return 0;inst=({struct Cyc_List_List*_tmp2F1=_cycalloc(sizeof(*_tmp2F1));
({struct _tuple19*_tmp65B=({struct _tuple19*_tmp2F0=_cycalloc(sizeof(*_tmp2F0));_tmp2F0->f1=(struct Cyc_Absyn_Tvar*)tvs2->hd,({void*_tmp65A=Cyc_Absyn_var_type((struct Cyc_Absyn_Tvar*)tvs1->hd);_tmp2F0->f2=_tmp65A;});_tmp2F0;});_tmp2F1->hd=_tmp65B;}),_tmp2F1->tl=inst;_tmp2F1;});
tvs1=tvs1->tl;
tvs2=tvs2->tl;}
# 2194
if(inst != 0){
f1.tvars=0;
f2.tvars=0;
return({struct Cyc_Tcenv_Tenv*_tmp65E=te;struct Cyc_List_List*_tmp65D=assume;void*_tmp65C=(void*)({struct Cyc_Absyn_FnType_Absyn_Type_struct*_tmp2F2=_cycalloc(sizeof(*_tmp2F2));_tmp2F2->tag=5U,_tmp2F2->f1=f1;_tmp2F2;});Cyc_Tcutil_subtype(_tmp65E,_tmp65D,_tmp65C,(void*)({struct Cyc_Absyn_FnType_Absyn_Type_struct*_tmp2F3=_cycalloc(sizeof(*_tmp2F3));_tmp2F3->tag=5U,_tmp2F3->f1=f2;_tmp2F3;}));});}}}
# 2183
if(!
# 2201
Cyc_Tcutil_subtype(te,assume,f1.ret_type,f2.ret_type))return 0;{
# 2183
struct Cyc_List_List*args1=f1.args;
# 2203
struct Cyc_List_List*args2=f2.args;
# 2206
if(({int _tmp65F=Cyc_List_length(args1);_tmp65F != Cyc_List_length(args2);}))return 0;for(0;args1 != 0;(
# 2208
args1=args1->tl,args2=args2->tl)){
struct _tuple9 _stmttmp5B=*((struct _tuple9*)args1->hd);void*_tmp2F5;struct Cyc_Absyn_Tqual _tmp2F4;_LLA: _tmp2F4=_stmttmp5B.f2;_tmp2F5=_stmttmp5B.f3;_LLB: {struct Cyc_Absyn_Tqual tq1=_tmp2F4;void*t1=_tmp2F5;
struct _tuple9 _stmttmp5C=*((struct _tuple9*)((struct Cyc_List_List*)_check_null(args2))->hd);void*_tmp2F7;struct Cyc_Absyn_Tqual _tmp2F6;_LLD: _tmp2F6=_stmttmp5C.f2;_tmp2F7=_stmttmp5C.f3;_LLE: {struct Cyc_Absyn_Tqual tq2=_tmp2F6;void*t2=_tmp2F7;
# 2212
if(tq2.real_const && !tq1.real_const || !Cyc_Tcutil_subtype(te,assume,t2,t1))
return 0;}}}
# 2216
if(f1.c_varargs != f2.c_varargs)return 0;if(
f1.cyc_varargs != 0 && f2.cyc_varargs != 0){
struct Cyc_Absyn_VarargInfo v1=*f1.cyc_varargs;
struct Cyc_Absyn_VarargInfo v2=*f2.cyc_varargs;
# 2221
if((v2.tq).real_const && !(v1.tq).real_const || !
Cyc_Tcutil_subtype(te,assume,v2.type,v1.type))
return 0;}else{
if(f1.cyc_varargs != 0 || f2.cyc_varargs != 0)return 0;}
# 2216
if(!({
# 2226
void*_tmp660=(void*)_check_null(f1.effect);Cyc_Tcutil_subset_effect(1,_tmp660,(void*)_check_null(f2.effect));}))return 0;
# 2216
if(!
# 2228
Cyc_Tcutil_sub_rgnpo(f1.rgn_po,f2.rgn_po))return 0;
# 2216
if(!
# 2230
Cyc_Tcutil_sub_attributes(f1.attributes,f2.attributes))return 0;
# 2216
if(!
# 2232
Cyc_Relations_check_logical_implication(f2.requires_relns,f1.requires_relns))
# 2234
return 0;
# 2216
if(!
# 2236
Cyc_Relations_check_logical_implication(f1.ensures_relns,f2.ensures_relns))
# 2238
return 0;
# 2216
return 1;}}}else{goto _LL7;}default: _LL7: _LL8:
# 2241
 return 0;}_LL0:;}}
# 2018
static int Cyc_Tcutil_ptrsubtype(struct Cyc_Tcenv_Tenv*te,struct Cyc_List_List*assume,void*t1,void*t2){
# 2255
struct Cyc_List_List*tqs1=Cyc_Tcutil_flatten_type(Cyc_Core_heap_region,1,t1);
struct Cyc_List_List*tqs2=Cyc_Tcutil_flatten_type(Cyc_Core_heap_region,1,t2);
for(0;tqs2 != 0;(tqs2=tqs2->tl,tqs1=tqs1->tl)){
if(tqs1 == 0)return 0;{struct _tuple12*_stmttmp5D=(struct _tuple12*)tqs1->hd;void*_tmp2F9;struct Cyc_Absyn_Tqual _tmp2F8;_LL1: _tmp2F8=_stmttmp5D->f1;_tmp2F9=_stmttmp5D->f2;_LL2: {struct Cyc_Absyn_Tqual tq1=_tmp2F8;void*t1a=_tmp2F9;
# 2260
struct _tuple12*_stmttmp5E=(struct _tuple12*)tqs2->hd;void*_tmp2FB;struct Cyc_Absyn_Tqual _tmp2FA;_LL4: _tmp2FA=_stmttmp5E->f1;_tmp2FB=_stmttmp5E->f2;_LL5: {struct Cyc_Absyn_Tqual tq2=_tmp2FA;void*t2a=_tmp2FB;
if(tq1.real_const && !tq2.real_const)return 0;if(
(tq2.real_const || Cyc_Tcutil_kind_leq(& Cyc_Tcutil_ak,Cyc_Tcutil_type_kind(t2a)))&&
 Cyc_Tcutil_subtype(te,assume,t1a,t2a))
continue;
# 2261
if(
# 2265
Cyc_Unify_unify(t1a,t2a))
continue;
# 2261
if(
# 2267
Cyc_Tcutil_isomorphic(t1a,t2a))
continue;
# 2261
return 0;}}}}
# 2271
return 1;}
# 2018
enum Cyc_Absyn_Coercion Cyc_Tcutil_castable(struct Cyc_Tcenv_Tenv*te,unsigned loc,void*t1,void*t2){
# 2277
if(Cyc_Unify_unify(t1,t2))
return Cyc_Absyn_No_coercion;
# 2277
t1=
# 2279
Cyc_Tcutil_compress(t1);
t2=Cyc_Tcutil_compress(t2);
# 2282
{void*_tmp2FC=t2;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2FC)->tag == 0U)switch(*((int*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2FC)->f1)){case 0U: _LL1: _LL2:
 return Cyc_Absyn_No_coercion;case 1U: switch(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2FC)->f1)->f2){case Cyc_Absyn_Int_sz: _LL3: _LL4:
# 2285
 goto _LL6;case Cyc_Absyn_Long_sz: _LL5: _LL6:
# 2287
 if((int)(Cyc_Tcutil_type_kind(t1))->kind == (int)Cyc_Absyn_BoxKind)return Cyc_Absyn_Other_coercion;goto _LL0;default: goto _LL7;}default: goto _LL7;}else{_LL7: _LL8:
# 2289
 goto _LL0;}_LL0:;}{
# 2291
void*_tmp2FD=t1;void*_tmp2FE;struct Cyc_Absyn_Enumdecl*_tmp2FF;void*_tmp303;struct Cyc_Absyn_Exp*_tmp302;struct Cyc_Absyn_Tqual _tmp301;void*_tmp300;void*_tmp309;void*_tmp308;void*_tmp307;void*_tmp306;struct Cyc_Absyn_Tqual _tmp305;void*_tmp304;switch(*((int*)_tmp2FD)){case 3U: _LLA: _tmp304=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp2FD)->f1).elt_type;_tmp305=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp2FD)->f1).elt_tq;_tmp306=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp2FD)->f1).ptr_atts).rgn;_tmp307=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp2FD)->f1).ptr_atts).nullable;_tmp308=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp2FD)->f1).ptr_atts).bounds;_tmp309=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp2FD)->f1).ptr_atts).zero_term;_LLB: {void*t_a=_tmp304;struct Cyc_Absyn_Tqual q_a=_tmp305;void*rt_a=_tmp306;void*null_a=_tmp307;void*b_a=_tmp308;void*zt_a=_tmp309;
# 2300
{void*_tmp30A=t2;void*_tmp310;void*_tmp30F;void*_tmp30E;void*_tmp30D;struct Cyc_Absyn_Tqual _tmp30C;void*_tmp30B;if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp30A)->tag == 3U){_LL19: _tmp30B=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp30A)->f1).elt_type;_tmp30C=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp30A)->f1).elt_tq;_tmp30D=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp30A)->f1).ptr_atts).rgn;_tmp30E=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp30A)->f1).ptr_atts).nullable;_tmp30F=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp30A)->f1).ptr_atts).bounds;_tmp310=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp30A)->f1).ptr_atts).zero_term;_LL1A: {void*t_b=_tmp30B;struct Cyc_Absyn_Tqual q_b=_tmp30C;void*rt_b=_tmp30D;void*null_b=_tmp30E;void*b_b=_tmp30F;void*zt_b=_tmp310;
# 2302
enum Cyc_Absyn_Coercion coercion=3U;
struct Cyc_List_List*assump=({struct Cyc_List_List*_tmp312=_cycalloc(sizeof(*_tmp312));({struct _tuple15*_tmp661=({struct _tuple15*_tmp311=_cycalloc(sizeof(*_tmp311));_tmp311->f1=t1,_tmp311->f2=t2;_tmp311;});_tmp312->hd=_tmp661;}),_tmp312->tl=0;_tmp312;});
int quals_okay=q_b.real_const || !q_a.real_const;
# 2315 "tcutil.cyc"
int deep_castable=
({void*_tmp662=b_b;Cyc_Unify_unify(_tmp662,Cyc_Absyn_bounds_one());})&& !Cyc_Tcutil_force_type2bool(0,zt_b);
# 2318
int ptrsub=quals_okay &&(
((deep_castable && Cyc_Tcutil_ptrsubtype(te,assump,t_a,t_b)||
 Cyc_Unify_unify(t_a,t_b))|| Cyc_Tcutil_isomorphic(t_a,t_b))|| Cyc_Unify_unify(t_b,Cyc_Absyn_void_type));
int zeroterm_ok=Cyc_Unify_unify(zt_a,zt_b)|| !Cyc_Absyn_type2bool(0,zt_b);
# 2323
int bitcase=ptrsub?0:((Cyc_Tcutil_is_bits_only_type(t_a)&& Cyc_Tcutil_is_char_type(t_b))&& !
Cyc_Tcutil_force_type2bool(0,zt_b))&&(
q_b.real_const || !q_a.real_const);
int bounds_ok=Cyc_Unify_unify(b_a,b_b);
if(!bounds_ok && !bitcase){
struct Cyc_Absyn_Exp*e_a=({void*_tmp663=Cyc_Absyn_bounds_one();Cyc_Tcutil_get_bounds_exp(_tmp663,b_a);});
struct Cyc_Absyn_Exp*e_b=({void*_tmp664=Cyc_Absyn_bounds_one();Cyc_Tcutil_get_bounds_exp(_tmp664,b_b);});
if((e_a != 0 && e_b != 0)&& Cyc_Evexp_lte_const_exp(e_b,e_a))
bounds_ok=1;else{
if(e_a == 0 || e_b == 0)
bounds_ok=1;}}{
# 2327
int t1_nullable=
# 2335
Cyc_Tcutil_force_type2bool(0,null_a);
int t2_nullable=Cyc_Tcutil_force_type2bool(0,null_b);
if(t1_nullable && !t2_nullable)
coercion=2U;
# 2337
if(
# 2340
((bounds_ok && zeroterm_ok)&&(ptrsub || bitcase))&&(
Cyc_Unify_unify(rt_a,rt_b)|| Cyc_Tcenv_rgn_outlives_rgn(te,rt_a,rt_b)))
return coercion;else{
return Cyc_Absyn_Unknown_coercion;}}}}else{_LL1B: _LL1C:
 goto _LL18;}_LL18:;}
# 2346
return Cyc_Absyn_Unknown_coercion;}case 4U: _LLC: _tmp300=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp2FD)->f1).elt_type;_tmp301=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp2FD)->f1).tq;_tmp302=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp2FD)->f1).num_elts;_tmp303=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp2FD)->f1).zero_term;_LLD: {void*t1a=_tmp300;struct Cyc_Absyn_Tqual tq1a=_tmp301;struct Cyc_Absyn_Exp*e1=_tmp302;void*zt1=_tmp303;
# 2348
{void*_tmp313=t2;void*_tmp317;struct Cyc_Absyn_Exp*_tmp316;struct Cyc_Absyn_Tqual _tmp315;void*_tmp314;if(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp313)->tag == 4U){_LL1E: _tmp314=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp313)->f1).elt_type;_tmp315=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp313)->f1).tq;_tmp316=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp313)->f1).num_elts;_tmp317=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp313)->f1).zero_term;_LL1F: {void*t2a=_tmp314;struct Cyc_Absyn_Tqual tq2a=_tmp315;struct Cyc_Absyn_Exp*e2=_tmp316;void*zt2=_tmp317;
# 2350
int okay=
(((e1 != 0 && e2 != 0)&& Cyc_Unify_unify(zt1,zt2))&&
 Cyc_Evexp_lte_const_exp(e2,e1))&& Cyc_Evexp_lte_const_exp(e1,e2);
return
# 2355
(okay && Cyc_Unify_unify(t1a,t2a))&&(!tq1a.real_const || tq2a.real_const)?Cyc_Absyn_No_coercion: Cyc_Absyn_Unknown_coercion;}}else{_LL20: _LL21:
# 2357
 return Cyc_Absyn_Unknown_coercion;}_LL1D:;}
# 2359
return Cyc_Absyn_Unknown_coercion;}case 0U: switch(*((int*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2FD)->f1)){case 15U: _LLE: _tmp2FF=((struct Cyc_Absyn_EnumCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2FD)->f1)->f2;_LLF: {struct Cyc_Absyn_Enumdecl*ed1=_tmp2FF;
# 2363
{void*_tmp318=t2;struct Cyc_Absyn_Enumdecl*_tmp319;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp318)->tag == 0U){if(((struct Cyc_Absyn_EnumCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp318)->f1)->tag == 15U){_LL23: _tmp319=((struct Cyc_Absyn_EnumCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp318)->f1)->f2;_LL24: {struct Cyc_Absyn_Enumdecl*ed2=_tmp319;
# 2365
if((((struct Cyc_Absyn_Enumdecl*)_check_null(ed1))->fields != 0 &&((struct Cyc_Absyn_Enumdecl*)_check_null(ed2))->fields != 0)&&({
int _tmp665=Cyc_List_length((struct Cyc_List_List*)((struct Cyc_Core_Opt*)_check_null(ed1->fields))->v);_tmp665 >= Cyc_List_length((struct Cyc_List_List*)((struct Cyc_Core_Opt*)_check_null(ed2->fields))->v);}))
return Cyc_Absyn_Other_coercion;
# 2365
goto _LL22;}}else{goto _LL25;}}else{_LL25: _LL26:
# 2369
 goto _LL22;}_LL22:;}
# 2371
goto _LL11;}case 1U: _LL10: _LL11:
 goto _LL13;case 2U: _LL12: _LL13:
# 2374
 return Cyc_Tcutil_is_strict_arithmetic_type(t2)?Cyc_Absyn_Other_coercion: Cyc_Absyn_Unknown_coercion;case 3U: if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2FD)->f2 != 0){_LL14: _tmp2FE=(void*)(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2FD)->f2)->hd;_LL15: {void*r1=_tmp2FE;
# 2377
{void*_tmp31A=t2;void*_tmp31B;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp31A)->tag == 0U){if(((struct Cyc_Absyn_RgnHandleCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp31A)->f1)->tag == 3U){if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp31A)->f2 != 0){_LL28: _tmp31B=(void*)(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp31A)->f2)->hd;_LL29: {void*r2=_tmp31B;
# 2379
if(Cyc_Tcenv_rgn_outlives_rgn(te,r1,r2))return Cyc_Absyn_No_coercion;goto _LL27;}}else{goto _LL2A;}}else{goto _LL2A;}}else{_LL2A: _LL2B:
# 2381
 goto _LL27;}_LL27:;}
# 2383
return Cyc_Absyn_Unknown_coercion;}}else{goto _LL16;}default: goto _LL16;}default: _LL16: _LL17:
 return Cyc_Absyn_Unknown_coercion;}_LL9:;}}
# 2018 "tcutil.cyc"
void Cyc_Tcutil_unchecked_cast(struct Cyc_Absyn_Exp*e,void*t,enum Cyc_Absyn_Coercion c){
# 2390 "tcutil.cyc"
if(Cyc_Unify_unify((void*)_check_null(e->topt),t))
return;{
# 2390
struct Cyc_Absyn_Exp*inner=
# 2392
Cyc_Absyn_copy_exp(e);
({void*_tmp666=(void*)({struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*_tmp31C=_cycalloc(sizeof(*_tmp31C));_tmp31C->tag=14U,_tmp31C->f1=t,_tmp31C->f2=inner,_tmp31C->f3=0,_tmp31C->f4=c;_tmp31C;});e->r=_tmp666;});
e->topt=t;}}
# 2018 "tcutil.cyc"
static int Cyc_Tcutil_tvar_id_counter=0;
# 2399 "tcutil.cyc"
int Cyc_Tcutil_new_tvar_id(){
return Cyc_Tcutil_tvar_id_counter ++;}
# 2399
static int Cyc_Tcutil_tvar_counter=0;
# 2404
struct Cyc_Absyn_Tvar*Cyc_Tcutil_new_tvar(void*k){
int i=Cyc_Tcutil_tvar_counter ++;
struct _fat_ptr s=(struct _fat_ptr)({struct Cyc_Int_pa_PrintArg_struct _tmp322=({struct Cyc_Int_pa_PrintArg_struct _tmp508;_tmp508.tag=1U,_tmp508.f1=(unsigned long)i;_tmp508;});void*_tmp320[1U];_tmp320[0]=& _tmp322;({struct _fat_ptr _tmp667=({const char*_tmp321="#%d";_tag_fat(_tmp321,sizeof(char),4U);});Cyc_aprintf(_tmp667,_tag_fat(_tmp320,sizeof(void*),1U));});});
return({struct Cyc_Absyn_Tvar*_tmp31F=_cycalloc(sizeof(*_tmp31F));({struct _fat_ptr*_tmp668=({unsigned _tmp31E=1;struct _fat_ptr*_tmp31D=_cycalloc(_check_times(_tmp31E,sizeof(struct _fat_ptr)));_tmp31D[0]=s;_tmp31D;});_tmp31F->name=_tmp668;}),_tmp31F->identity=- 1,_tmp31F->kind=k;_tmp31F;});}
# 2404
int Cyc_Tcutil_is_temp_tvar(struct Cyc_Absyn_Tvar*t){
# 2411
struct _fat_ptr s=*t->name;
return(int)*((const char*)_check_fat_subscript(s,sizeof(char),0))== (int)'#';}
# 2404
void*Cyc_Tcutil_fndecl2type(struct Cyc_Absyn_Fndecl*fd){
# 2417
if(fd->cached_type == 0){
# 2423
struct Cyc_List_List*fn_type_atts=0;
{struct Cyc_List_List*atts=(fd->i).attributes;for(0;atts != 0;atts=atts->tl){
if(Cyc_Absyn_fntype_att((void*)atts->hd))
fn_type_atts=({struct Cyc_List_List*_tmp323=_cycalloc(sizeof(*_tmp323));_tmp323->hd=(void*)atts->hd,_tmp323->tl=fn_type_atts;_tmp323;});}}{
# 2424
struct Cyc_Absyn_FnInfo type_info=fd->i;
# 2428
type_info.attributes=fn_type_atts;
return(void*)({struct Cyc_Absyn_FnType_Absyn_Type_struct*_tmp324=_cycalloc(sizeof(*_tmp324));_tmp324->tag=5U,_tmp324->f1=type_info;_tmp324;});}}
# 2417
return(void*)_check_null(fd->cached_type);}
# 2404
static void Cyc_Tcutil_replace_rop(struct Cyc_List_List*args,union Cyc_Relations_RelnOp*rop){
# 2439
union Cyc_Relations_RelnOp _stmttmp5F=*rop;union Cyc_Relations_RelnOp _tmp325=_stmttmp5F;struct Cyc_Absyn_Vardecl*_tmp326;struct Cyc_Absyn_Vardecl*_tmp327;switch((_tmp325.RNumelts).tag){case 2U: _LL1: _tmp327=(_tmp325.RVar).val;_LL2: {struct Cyc_Absyn_Vardecl*vd=_tmp327;
# 2441
struct _tuple1 _stmttmp60=*vd->name;struct _fat_ptr*_tmp329;union Cyc_Absyn_Nmspace _tmp328;_LL8: _tmp328=_stmttmp60.f1;_tmp329=_stmttmp60.f2;_LL9: {union Cyc_Absyn_Nmspace nmspace=_tmp328;struct _fat_ptr*var=_tmp329;
if(!((int)((nmspace.Loc_n).tag == 4)))goto _LL0;if(({
struct _fat_ptr _tmp669=(struct _fat_ptr)*var;Cyc_strcmp(_tmp669,({const char*_tmp32A="return_value";_tag_fat(_tmp32A,sizeof(char),13U);}));})== 0){
({union Cyc_Relations_RelnOp _tmp66A=Cyc_Relations_RReturn();*rop=_tmp66A;});
goto _LL0;}{
# 2442
unsigned c=0U;
# 2448
{struct Cyc_List_List*a=args;for(0;a != 0;(a=a->tl,c ++)){
struct _tuple9*_stmttmp61=(struct _tuple9*)a->hd;struct _fat_ptr*_tmp32B;_LLB: _tmp32B=_stmttmp61->f1;_LLC: {struct _fat_ptr*vopt=_tmp32B;
if(vopt != 0){
if(Cyc_strcmp((struct _fat_ptr)*var,(struct _fat_ptr)*vopt)== 0){
({union Cyc_Relations_RelnOp _tmp66B=Cyc_Relations_RParam(c);*rop=_tmp66B;});
break;}}}}}
# 2457
goto _LL0;}}}case 3U: _LL3: _tmp326=(_tmp325.RNumelts).val;_LL4: {struct Cyc_Absyn_Vardecl*vd=_tmp326;
# 2459
struct _tuple1 _stmttmp62=*vd->name;struct _fat_ptr*_tmp32D;union Cyc_Absyn_Nmspace _tmp32C;_LLE: _tmp32C=_stmttmp62.f1;_tmp32D=_stmttmp62.f2;_LLF: {union Cyc_Absyn_Nmspace nmspace=_tmp32C;struct _fat_ptr*var=_tmp32D;
if(!((int)((nmspace.Loc_n).tag == 4)))goto _LL0;{unsigned c=0U;
# 2462
{struct Cyc_List_List*a=args;for(0;a != 0;(a=a->tl,c ++)){
struct _tuple9*_stmttmp63=(struct _tuple9*)a->hd;struct _fat_ptr*_tmp32E;_LL11: _tmp32E=_stmttmp63->f1;_LL12: {struct _fat_ptr*vopt=_tmp32E;
if(vopt != 0){
if(Cyc_strcmp((struct _fat_ptr)*var,(struct _fat_ptr)*vopt)== 0){
({union Cyc_Relations_RelnOp _tmp66C=Cyc_Relations_RParamNumelts(c);*rop=_tmp66C;});
break;}}}}}
# 2471
goto _LL0;}}}default: _LL5: _LL6:
 goto _LL0;}_LL0:;}
# 2404
void Cyc_Tcutil_replace_rops(struct Cyc_List_List*args,struct Cyc_Relations_Reln*r){
# 2478
Cyc_Tcutil_replace_rop(args,& r->rop1);
Cyc_Tcutil_replace_rop(args,& r->rop2);}
# 2404
static struct Cyc_List_List*Cyc_Tcutil_extract_relns(struct Cyc_List_List*args,struct Cyc_Absyn_Exp*e){
# 2484
if(e == 0)return 0;{struct Cyc_List_List*relns=
Cyc_Relations_exp2relns(Cyc_Core_heap_region,e);
({void(*_tmp66E)(void(*f)(struct Cyc_List_List*,struct Cyc_Relations_Reln*),struct Cyc_List_List*env,struct Cyc_List_List*x)=({void(*_tmp32F)(void(*f)(struct Cyc_List_List*,struct Cyc_Relations_Reln*),struct Cyc_List_List*env,struct Cyc_List_List*x)=(void(*)(void(*f)(struct Cyc_List_List*,struct Cyc_Relations_Reln*),struct Cyc_List_List*env,struct Cyc_List_List*x))Cyc_List_iter_c;_tmp32F;});struct Cyc_List_List*_tmp66D=args;_tmp66E(Cyc_Tcutil_replace_rops,_tmp66D,relns);});
return relns;}}
# 2404
void*Cyc_Tcutil_snd_tqt(struct _tuple12*t){
# 2491
return(*t).f2;}
# 2404
static struct _tuple12*Cyc_Tcutil_map2_tq(struct _tuple12*pr,void*t){
# 2493
void*_tmp331;struct Cyc_Absyn_Tqual _tmp330;_LL1: _tmp330=pr->f1;_tmp331=pr->f2;_LL2: {struct Cyc_Absyn_Tqual tq=_tmp330;void*t2=_tmp331;
if(t2 == t)return pr;else{
return({struct _tuple12*_tmp332=_cycalloc(sizeof(*_tmp332));_tmp332->f1=tq,_tmp332->f2=t;_tmp332;});}}}struct _tuple26{struct _fat_ptr*f1;struct Cyc_Absyn_Tqual f2;};struct _tuple27{struct _tuple26*f1;void*f2;};
# 2404
static struct _tuple27*Cyc_Tcutil_substitute_f1(struct _RegionHandle*rgn,struct _tuple9*y){
# 2499
return({struct _tuple27*_tmp334=_region_malloc(rgn,sizeof(*_tmp334));({struct _tuple26*_tmp66F=({struct _tuple26*_tmp333=_region_malloc(rgn,sizeof(*_tmp333));_tmp333->f1=(*y).f1,_tmp333->f2=(*y).f2;_tmp333;});_tmp334->f1=_tmp66F;}),_tmp334->f2=(*y).f3;_tmp334;});}
# 2404
static struct _tuple9*Cyc_Tcutil_substitute_f2(struct _tuple9*orig_arg,void*t){
# 2503
struct _tuple9 _stmttmp64=*orig_arg;void*_tmp337;struct Cyc_Absyn_Tqual _tmp336;struct _fat_ptr*_tmp335;_LL1: _tmp335=_stmttmp64.f1;_tmp336=_stmttmp64.f2;_tmp337=_stmttmp64.f3;_LL2: {struct _fat_ptr*vopt_orig=_tmp335;struct Cyc_Absyn_Tqual tq_orig=_tmp336;void*t_orig=_tmp337;
if(t == t_orig)return orig_arg;return({struct _tuple9*_tmp338=_cycalloc(sizeof(*_tmp338));
_tmp338->f1=vopt_orig,_tmp338->f2=tq_orig,_tmp338->f3=t;_tmp338;});}}
# 2404
static struct Cyc_List_List*Cyc_Tcutil_substs(struct _RegionHandle*rgn,struct Cyc_List_List*inst,struct Cyc_List_List*ts);
# 2512
static struct Cyc_Absyn_Exp*Cyc_Tcutil_copye(struct Cyc_Absyn_Exp*old,void*r){
# 2514
return({struct Cyc_Absyn_Exp*_tmp339=_cycalloc(sizeof(*_tmp339));_tmp339->topt=old->topt,_tmp339->r=r,_tmp339->loc=old->loc,_tmp339->annot=old->annot;_tmp339;});}
# 2512
struct Cyc_Absyn_Exp*Cyc_Tcutil_rsubsexp(struct _RegionHandle*r,struct Cyc_List_List*inst,struct Cyc_Absyn_Exp*e){
# 2520
void*_stmttmp65=e->r;void*_tmp33A=_stmttmp65;void*_tmp33B;struct Cyc_List_List*_tmp33D;void*_tmp33C;struct Cyc_Absyn_Exp*_tmp33E;struct Cyc_Absyn_Exp*_tmp33F;void*_tmp340;enum Cyc_Absyn_Coercion _tmp344;int _tmp343;struct Cyc_Absyn_Exp*_tmp342;void*_tmp341;struct Cyc_Absyn_Exp*_tmp346;struct Cyc_Absyn_Exp*_tmp345;struct Cyc_Absyn_Exp*_tmp348;struct Cyc_Absyn_Exp*_tmp347;struct Cyc_Absyn_Exp*_tmp34A;struct Cyc_Absyn_Exp*_tmp349;struct Cyc_Absyn_Exp*_tmp34C;struct Cyc_Absyn_Exp*_tmp34B;struct Cyc_Absyn_Exp*_tmp34F;struct Cyc_Absyn_Exp*_tmp34E;struct Cyc_Absyn_Exp*_tmp34D;struct Cyc_List_List*_tmp351;enum Cyc_Absyn_Primop _tmp350;switch(*((int*)_tmp33A)){case 0U: _LL1: _LL2:
 goto _LL4;case 32U: _LL3: _LL4:
 goto _LL6;case 33U: _LL5: _LL6:
 goto _LL8;case 2U: _LL7: _LL8:
 goto _LLA;case 1U: _LL9: _LLA:
 return e;case 3U: _LLB: _tmp350=((struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct*)_tmp33A)->f1;_tmp351=((struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct*)_tmp33A)->f2;_LLC: {enum Cyc_Absyn_Primop p=_tmp350;struct Cyc_List_List*es=_tmp351;
# 2528
if(Cyc_List_length(es)== 1){
struct Cyc_Absyn_Exp*e1=(struct Cyc_Absyn_Exp*)((struct Cyc_List_List*)_check_null(es))->hd;
struct Cyc_Absyn_Exp*new_e1=Cyc_Tcutil_rsubsexp(r,inst,e1);
if(new_e1 == e1)return e;return({
struct Cyc_Absyn_Exp*_tmp671=e;Cyc_Tcutil_copye(_tmp671,(void*)({struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct*_tmp353=_cycalloc(sizeof(*_tmp353));_tmp353->tag=3U,_tmp353->f1=p,({struct Cyc_List_List*_tmp670=({struct Cyc_Absyn_Exp*_tmp352[1U];_tmp352[0]=new_e1;Cyc_List_list(_tag_fat(_tmp352,sizeof(struct Cyc_Absyn_Exp*),1U));});_tmp353->f2=_tmp670;});_tmp353;}));});}else{
if(Cyc_List_length(es)== 2){
struct Cyc_Absyn_Exp*e1=(struct Cyc_Absyn_Exp*)((struct Cyc_List_List*)_check_null(es))->hd;
struct Cyc_Absyn_Exp*e2=(struct Cyc_Absyn_Exp*)((struct Cyc_List_List*)_check_null(es->tl))->hd;
struct Cyc_Absyn_Exp*new_e1=Cyc_Tcutil_rsubsexp(r,inst,e1);
struct Cyc_Absyn_Exp*new_e2=Cyc_Tcutil_rsubsexp(r,inst,e2);
if(new_e1 == e1 && new_e2 == e2)return e;return({
struct Cyc_Absyn_Exp*_tmp673=e;Cyc_Tcutil_copye(_tmp673,(void*)({struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct*_tmp355=_cycalloc(sizeof(*_tmp355));_tmp355->tag=3U,_tmp355->f1=p,({struct Cyc_List_List*_tmp672=({struct Cyc_Absyn_Exp*_tmp354[2U];_tmp354[0]=new_e1,_tmp354[1]=new_e2;Cyc_List_list(_tag_fat(_tmp354,sizeof(struct Cyc_Absyn_Exp*),2U));});_tmp355->f2=_tmp672;});_tmp355;}));});}else{
return({void*_tmp356=0U;({struct Cyc_Absyn_Exp*(*_tmp675)(struct _fat_ptr fmt,struct _fat_ptr ap)=({struct Cyc_Absyn_Exp*(*_tmp358)(struct _fat_ptr fmt,struct _fat_ptr ap)=(struct Cyc_Absyn_Exp*(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Tcutil_impos;_tmp358;});struct _fat_ptr _tmp674=({const char*_tmp357="primop does not have 1 or 2 args!";_tag_fat(_tmp357,sizeof(char),34U);});_tmp675(_tmp674,_tag_fat(_tmp356,sizeof(void*),0U));});});}}}case 6U: _LLD: _tmp34D=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmp33A)->f1;_tmp34E=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmp33A)->f2;_tmp34F=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmp33A)->f3;_LLE: {struct Cyc_Absyn_Exp*e1=_tmp34D;struct Cyc_Absyn_Exp*e2=_tmp34E;struct Cyc_Absyn_Exp*e3=_tmp34F;
# 2542
struct Cyc_Absyn_Exp*new_e1=Cyc_Tcutil_rsubsexp(r,inst,e1);
struct Cyc_Absyn_Exp*new_e2=Cyc_Tcutil_rsubsexp(r,inst,e2);
struct Cyc_Absyn_Exp*new_e3=Cyc_Tcutil_rsubsexp(r,inst,e3);
if((new_e1 == e1 && new_e2 == e2)&& new_e3 == e3)return e;return({
struct Cyc_Absyn_Exp*_tmp676=e;Cyc_Tcutil_copye(_tmp676,(void*)({struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*_tmp359=_cycalloc(sizeof(*_tmp359));_tmp359->tag=6U,_tmp359->f1=new_e1,_tmp359->f2=new_e2,_tmp359->f3=new_e3;_tmp359;}));});}case 7U: _LLF: _tmp34B=((struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct*)_tmp33A)->f1;_tmp34C=((struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct*)_tmp33A)->f2;_LL10: {struct Cyc_Absyn_Exp*e1=_tmp34B;struct Cyc_Absyn_Exp*e2=_tmp34C;
# 2548
struct Cyc_Absyn_Exp*new_e1=Cyc_Tcutil_rsubsexp(r,inst,e1);
struct Cyc_Absyn_Exp*new_e2=Cyc_Tcutil_rsubsexp(r,inst,e2);
if(new_e1 == e1 && new_e2 == e2)return e;return({
struct Cyc_Absyn_Exp*_tmp677=e;Cyc_Tcutil_copye(_tmp677,(void*)({struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct*_tmp35A=_cycalloc(sizeof(*_tmp35A));_tmp35A->tag=7U,_tmp35A->f1=new_e1,_tmp35A->f2=new_e2;_tmp35A;}));});}case 8U: _LL11: _tmp349=((struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct*)_tmp33A)->f1;_tmp34A=((struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct*)_tmp33A)->f2;_LL12: {struct Cyc_Absyn_Exp*e1=_tmp349;struct Cyc_Absyn_Exp*e2=_tmp34A;
# 2553
struct Cyc_Absyn_Exp*new_e1=Cyc_Tcutil_rsubsexp(r,inst,e1);
struct Cyc_Absyn_Exp*new_e2=Cyc_Tcutil_rsubsexp(r,inst,e2);
if(new_e1 == e1 && new_e2 == e2)return e;return({
struct Cyc_Absyn_Exp*_tmp678=e;Cyc_Tcutil_copye(_tmp678,(void*)({struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct*_tmp35B=_cycalloc(sizeof(*_tmp35B));_tmp35B->tag=8U,_tmp35B->f1=new_e1,_tmp35B->f2=new_e2;_tmp35B;}));});}case 34U: _LL13: _tmp347=((struct Cyc_Absyn_Spawn_e_Absyn_Raw_exp_struct*)_tmp33A)->f1;_tmp348=((struct Cyc_Absyn_Spawn_e_Absyn_Raw_exp_struct*)_tmp33A)->f2;_LL14: {struct Cyc_Absyn_Exp*e1=_tmp347;struct Cyc_Absyn_Exp*e2=_tmp348;
# 2559
struct Cyc_Absyn_Exp*new_e1=Cyc_Tcutil_rsubsexp(r,inst,e1);
struct Cyc_Absyn_Exp*new_e2=Cyc_Tcutil_rsubsexp(r,inst,e2);
if(new_e1 == e1 && new_e2 == e2)return e;return({
struct Cyc_Absyn_Exp*_tmp679=e;Cyc_Tcutil_copye(_tmp679,(void*)({struct Cyc_Absyn_Spawn_e_Absyn_Raw_exp_struct*_tmp35C=_cycalloc(sizeof(*_tmp35C));_tmp35C->tag=34U,_tmp35C->f1=new_e1,_tmp35C->f2=new_e2;_tmp35C;}));});}case 9U: _LL15: _tmp345=((struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct*)_tmp33A)->f1;_tmp346=((struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct*)_tmp33A)->f2;_LL16: {struct Cyc_Absyn_Exp*e1=_tmp345;struct Cyc_Absyn_Exp*e2=_tmp346;
# 2566
struct Cyc_Absyn_Exp*new_e1=Cyc_Tcutil_rsubsexp(r,inst,e1);
struct Cyc_Absyn_Exp*new_e2=Cyc_Tcutil_rsubsexp(r,inst,e2);
if(new_e1 == e1 && new_e2 == e2)return e;return({
struct Cyc_Absyn_Exp*_tmp67A=e;Cyc_Tcutil_copye(_tmp67A,(void*)({struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct*_tmp35D=_cycalloc(sizeof(*_tmp35D));_tmp35D->tag=9U,_tmp35D->f1=new_e1,_tmp35D->f2=new_e2;_tmp35D;}));});}case 14U: _LL17: _tmp341=(void*)((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp33A)->f1;_tmp342=((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp33A)->f2;_tmp343=((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp33A)->f3;_tmp344=((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp33A)->f4;_LL18: {void*t=_tmp341;struct Cyc_Absyn_Exp*e1=_tmp342;int b=_tmp343;enum Cyc_Absyn_Coercion c=_tmp344;
# 2572
struct Cyc_Absyn_Exp*new_e1=Cyc_Tcutil_rsubsexp(r,inst,e1);
void*new_typ=Cyc_Tcutil_rsubstitute(r,inst,t);
if(new_e1 == e1 && new_typ == t)return e;return({
struct Cyc_Absyn_Exp*_tmp67B=e;Cyc_Tcutil_copye(_tmp67B,(void*)({struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*_tmp35E=_cycalloc(sizeof(*_tmp35E));_tmp35E->tag=14U,_tmp35E->f1=new_typ,_tmp35E->f2=new_e1,_tmp35E->f3=b,_tmp35E->f4=c;_tmp35E;}));});}case 17U: _LL19: _tmp340=(void*)((struct Cyc_Absyn_Sizeoftype_e_Absyn_Raw_exp_struct*)_tmp33A)->f1;_LL1A: {void*t=_tmp340;
# 2577
void*new_typ=Cyc_Tcutil_rsubstitute(r,inst,t);
if(new_typ == t)return e;return({
struct Cyc_Absyn_Exp*_tmp67C=e;Cyc_Tcutil_copye(_tmp67C,(void*)({struct Cyc_Absyn_Sizeoftype_e_Absyn_Raw_exp_struct*_tmp35F=_cycalloc(sizeof(*_tmp35F));_tmp35F->tag=17U,_tmp35F->f1=new_typ;_tmp35F;}));});}case 18U: _LL1B: _tmp33F=((struct Cyc_Absyn_Sizeofexp_e_Absyn_Raw_exp_struct*)_tmp33A)->f1;_LL1C: {struct Cyc_Absyn_Exp*e1=_tmp33F;
# 2581
struct Cyc_Absyn_Exp*new_e1=Cyc_Tcutil_rsubsexp(r,inst,e1);
if(new_e1 == e1)return e;return({
struct Cyc_Absyn_Exp*_tmp67D=e;Cyc_Tcutil_copye(_tmp67D,(void*)({struct Cyc_Absyn_Sizeofexp_e_Absyn_Raw_exp_struct*_tmp360=_cycalloc(sizeof(*_tmp360));_tmp360->tag=18U,_tmp360->f1=new_e1;_tmp360;}));});}case 42U: _LL1D: _tmp33E=((struct Cyc_Absyn_Extension_e_Absyn_Raw_exp_struct*)_tmp33A)->f1;_LL1E: {struct Cyc_Absyn_Exp*e1=_tmp33E;
# 2585
struct Cyc_Absyn_Exp*new_e1=Cyc_Tcutil_rsubsexp(r,inst,e1);
if(new_e1 == e1)return e;return({
struct Cyc_Absyn_Exp*_tmp67E=e;Cyc_Tcutil_copye(_tmp67E,(void*)({struct Cyc_Absyn_Extension_e_Absyn_Raw_exp_struct*_tmp361=_cycalloc(sizeof(*_tmp361));_tmp361->tag=42U,_tmp361->f1=new_e1;_tmp361;}));});}case 19U: _LL1F: _tmp33C=(void*)((struct Cyc_Absyn_Offsetof_e_Absyn_Raw_exp_struct*)_tmp33A)->f1;_tmp33D=((struct Cyc_Absyn_Offsetof_e_Absyn_Raw_exp_struct*)_tmp33A)->f2;_LL20: {void*t=_tmp33C;struct Cyc_List_List*f=_tmp33D;
# 2589
void*new_typ=Cyc_Tcutil_rsubstitute(r,inst,t);
if(new_typ == t)return e;return({
struct Cyc_Absyn_Exp*_tmp67F=e;Cyc_Tcutil_copye(_tmp67F,(void*)({struct Cyc_Absyn_Offsetof_e_Absyn_Raw_exp_struct*_tmp362=_cycalloc(sizeof(*_tmp362));_tmp362->tag=19U,_tmp362->f1=new_typ,_tmp362->f2=f;_tmp362;}));});}case 40U: _LL21: _tmp33B=(void*)((struct Cyc_Absyn_Valueof_e_Absyn_Raw_exp_struct*)_tmp33A)->f1;_LL22: {void*t=_tmp33B;
# 2593
void*new_typ=Cyc_Tcutil_rsubstitute(r,inst,t);
if(new_typ == t)return e;{void*_stmttmp66=
# 2596
Cyc_Tcutil_compress(new_typ);
# 2594
void*_tmp363=_stmttmp66;struct Cyc_Absyn_Exp*_tmp364;if(((struct Cyc_Absyn_ValueofType_Absyn_Type_struct*)_tmp363)->tag == 9U){_LL26: _tmp364=((struct Cyc_Absyn_ValueofType_Absyn_Type_struct*)_tmp363)->f1;_LL27: {struct Cyc_Absyn_Exp*e=_tmp364;
# 2597
return e;}}else{_LL28: _LL29:
 return({struct Cyc_Absyn_Exp*_tmp680=e;Cyc_Tcutil_copye(_tmp680,(void*)({struct Cyc_Absyn_Valueof_e_Absyn_Raw_exp_struct*_tmp365=_cycalloc(sizeof(*_tmp365));_tmp365->tag=40U,_tmp365->f1=new_typ;_tmp365;}));});}_LL25:;}}default: _LL23: _LL24:
# 2600
 return({void*_tmp366=0U;({struct Cyc_Absyn_Exp*(*_tmp682)(struct _fat_ptr fmt,struct _fat_ptr ap)=({struct Cyc_Absyn_Exp*(*_tmp368)(struct _fat_ptr fmt,struct _fat_ptr ap)=(struct Cyc_Absyn_Exp*(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Tcutil_impos;_tmp368;});struct _fat_ptr _tmp681=({const char*_tmp367="non-type-level-expression in Tcutil::rsubsexp";_tag_fat(_tmp367,sizeof(char),46U);});_tmp682(_tmp681,_tag_fat(_tmp366,sizeof(void*),0U));});});}_LL0:;}
# 2512
static struct Cyc_Absyn_Exp*Cyc_Tcutil_rsubs_exp_opt(struct _RegionHandle*r,struct Cyc_List_List*inst,struct Cyc_Absyn_Exp*e){
# 2607
if(e == 0)return 0;else{
return Cyc_Tcutil_rsubsexp(r,inst,e);}}
# 2512
static struct Cyc_Absyn_Aggrfield*Cyc_Tcutil_subst_aggrfield(struct _RegionHandle*r,struct Cyc_List_List*inst,struct Cyc_Absyn_Aggrfield*f){
# 2614
void*t=f->type;
struct Cyc_Absyn_Exp*req=f->requires_clause;
void*new_t=Cyc_Tcutil_rsubstitute(r,inst,t);
struct Cyc_Absyn_Exp*new_req=Cyc_Tcutil_rsubs_exp_opt(r,inst,req);
if(t == new_t && req == new_req)return f;else{
return({struct Cyc_Absyn_Aggrfield*_tmp369=_cycalloc(sizeof(*_tmp369));_tmp369->name=f->name,_tmp369->tq=f->tq,_tmp369->type=new_t,_tmp369->width=f->width,_tmp369->attributes=f->attributes,_tmp369->requires_clause=new_req;_tmp369;});}}
# 2512
static struct Cyc_List_List*Cyc_Tcutil_subst_aggrfields(struct _RegionHandle*r,struct Cyc_List_List*inst,struct Cyc_List_List*fs){
# 2627
if(fs == 0)return 0;{struct Cyc_Absyn_Aggrfield*f=(struct Cyc_Absyn_Aggrfield*)fs->hd;
# 2629
struct Cyc_List_List*t=fs->tl;
struct Cyc_Absyn_Aggrfield*new_f=Cyc_Tcutil_subst_aggrfield(r,inst,f);
struct Cyc_List_List*new_t=Cyc_Tcutil_subst_aggrfields(r,inst,t);
if(new_f == f && new_t == t)return fs;return({struct Cyc_List_List*_tmp36A=_cycalloc(sizeof(*_tmp36A));
_tmp36A->hd=new_f,_tmp36A->tl=new_t;_tmp36A;});}}
# 2512
struct Cyc_List_List*Cyc_Tcutil_rsubst_rgnpo(struct _RegionHandle*rgn,struct Cyc_List_List*inst,struct Cyc_List_List*rgn_po){
# 2639
struct _tuple0 _stmttmp67=Cyc_List_rsplit(rgn,rgn,rgn_po);struct Cyc_List_List*_tmp36C;struct Cyc_List_List*_tmp36B;_LL1: _tmp36B=_stmttmp67.f1;_tmp36C=_stmttmp67.f2;_LL2: {struct Cyc_List_List*rpo1a=_tmp36B;struct Cyc_List_List*rpo1b=_tmp36C;
struct Cyc_List_List*rpo2a=Cyc_Tcutil_substs(rgn,inst,rpo1a);
struct Cyc_List_List*rpo2b=Cyc_Tcutil_substs(rgn,inst,rpo1b);
if(rpo2a == rpo1a && rpo2b == rpo1b)
return rgn_po;else{
# 2645
return Cyc_List_zip(rpo2a,rpo2b);}}}
# 2512
struct Cyc_Absyn_RgnEffect*Cyc_Tcutil_rsubstitute_eff(struct _RegionHandle*rgn,struct Cyc_List_List*inst,struct Cyc_Absyn_RgnEffect*eff){
# 2651
return({struct Cyc_Absyn_RgnEffect*_tmp36D=_cycalloc(sizeof(*_tmp36D));({void*_tmp687=Cyc_Tcutil_rsubstitute(rgn,inst,eff->name);_tmp36D->name=_tmp687;}),({
void*_tmp686=Cyc_Tcutil_copy_cap(eff->rgn);_tmp36D->rgn=_tmp686;}),({
void*_tmp685=Cyc_Tcutil_copy_cap(eff->lock);_tmp36D->lock=_tmp685;}),({
void*_tmp684=Cyc_Tcutil_copy_cap(eff->unadopt);_tmp36D->unadopt=_tmp684;}),({
void*_tmp683=Cyc_Tcutil_rsubstitute(rgn,inst,(void*)_check_null(eff->parent));_tmp36D->parent=_tmp683;});_tmp36D;});}
# 2512
struct Cyc_List_List*Cyc_Tcutil_rsubstitute_eff_list(struct _RegionHandle*rgn,struct Cyc_List_List*inst,struct Cyc_List_List*l){
# 2661
struct Cyc_List_List*hd;struct Cyc_List_List*tl;
if(l == 0)return 0;hd=(tl=({struct Cyc_List_List*_tmp36E=_cycalloc(sizeof(*_tmp36E));
({struct Cyc_Absyn_RgnEffect*_tmp688=Cyc_Tcutil_rsubstitute_eff(rgn,inst,(struct Cyc_Absyn_RgnEffect*)l->hd);_tmp36E->hd=_tmp688;}),_tmp36E->tl=0;_tmp36E;}));
for(l=l->tl;l != 0;(({struct Cyc_List_List*_tmp68A=({struct Cyc_List_List*_tmp36F=_cycalloc(sizeof(*_tmp36F));({struct Cyc_Absyn_RgnEffect*_tmp689=Cyc_Tcutil_rsubstitute_eff(rgn,inst,(struct Cyc_Absyn_RgnEffect*)l->hd);_tmp36F->hd=_tmp689;}),_tmp36F->tl=0;_tmp36F;});((struct Cyc_List_List*)_check_null(tl))->tl=_tmp68A;}),tl=tl->tl,l=l->tl)){;}
return hd;}
# 2512
void*Cyc_Tcutil_rsubstitute(struct _RegionHandle*rgn,struct Cyc_List_List*inst,void*t){
# 2674
void*_stmttmp68=Cyc_Tcutil_compress(t);void*_tmp370=_stmttmp68;struct Cyc_Absyn_Exp*_tmp371;struct Cyc_Absyn_Exp*_tmp372;struct Cyc_List_List*_tmp374;void*_tmp373;void*_tmp375;void*_tmp376;struct Cyc_List_List*_tmp378;enum Cyc_Absyn_AggrKind _tmp377;struct Cyc_List_List*_tmp379;struct Cyc_List_List*_tmp386;struct Cyc_List_List*_tmp385;struct Cyc_Absyn_Exp*_tmp384;struct Cyc_Absyn_Exp*_tmp383;struct Cyc_List_List*_tmp382;struct Cyc_List_List*_tmp381;struct Cyc_Absyn_VarargInfo*_tmp380;int _tmp37F;struct Cyc_List_List*_tmp37E;void*_tmp37D;struct Cyc_Absyn_Tqual _tmp37C;void*_tmp37B;struct Cyc_List_List*_tmp37A;void*_tmp38C;void*_tmp38B;void*_tmp38A;void*_tmp389;struct Cyc_Absyn_Tqual _tmp388;void*_tmp387;unsigned _tmp391;void*_tmp390;struct Cyc_Absyn_Exp*_tmp38F;struct Cyc_Absyn_Tqual _tmp38E;void*_tmp38D;void*_tmp395;struct Cyc_Absyn_Typedefdecl*_tmp394;struct Cyc_List_List*_tmp393;struct _tuple1*_tmp392;struct Cyc_Absyn_Tvar*_tmp396;switch(*((int*)_tmp370)){case 2U: _LL1: _tmp396=((struct Cyc_Absyn_VarType_Absyn_Type_struct*)_tmp370)->f1;_LL2: {struct Cyc_Absyn_Tvar*v=_tmp396;
# 2678
struct _handler_cons _tmp397;_push_handler(& _tmp397);{int _tmp399=0;if(setjmp(_tmp397.handler))_tmp399=1;if(!_tmp399){{void*_tmp39B=({void*(*_tmp68C)(int(*cmp)(struct Cyc_Absyn_Tvar*,struct Cyc_Absyn_Tvar*),struct Cyc_List_List*l,struct Cyc_Absyn_Tvar*x)=({void*(*_tmp39A)(int(*cmp)(struct Cyc_Absyn_Tvar*,struct Cyc_Absyn_Tvar*),struct Cyc_List_List*l,struct Cyc_Absyn_Tvar*x)=(void*(*)(int(*cmp)(struct Cyc_Absyn_Tvar*,struct Cyc_Absyn_Tvar*),struct Cyc_List_List*l,struct Cyc_Absyn_Tvar*x))Cyc_List_assoc_cmp;_tmp39A;});struct Cyc_List_List*_tmp68B=inst;_tmp68C(Cyc_Absyn_tvar_cmp,_tmp68B,v);});_npop_handler(0U);return _tmp39B;};_pop_handler();}else{void*_tmp398=(void*)Cyc_Core_get_exn_thrown();void*_tmp39C=_tmp398;void*_tmp39D;if(((struct Cyc_Core_Not_found_exn_struct*)_tmp39C)->tag == Cyc_Core_Not_found){_LL1C: _LL1D:
 return t;}else{_LL1E: _tmp39D=_tmp39C;_LL1F: {void*exn=_tmp39D;(int)_rethrow(exn);}}_LL1B:;}}}case 8U: _LL3: _tmp392=((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmp370)->f1;_tmp393=((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmp370)->f2;_tmp394=((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmp370)->f3;_tmp395=(void*)((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmp370)->f4;_LL4: {struct _tuple1*n=_tmp392;struct Cyc_List_List*ts=_tmp393;struct Cyc_Absyn_Typedefdecl*td=_tmp394;void*topt=_tmp395;
# 2681
struct Cyc_List_List*new_ts=Cyc_Tcutil_substs(rgn,inst,ts);
return new_ts == ts?t:(void*)({struct Cyc_Absyn_TypedefType_Absyn_Type_struct*_tmp39E=_cycalloc(sizeof(*_tmp39E));_tmp39E->tag=8U,_tmp39E->f1=n,_tmp39E->f2=new_ts,_tmp39E->f3=td,_tmp39E->f4=topt;_tmp39E;});}case 4U: _LL5: _tmp38D=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp370)->f1).elt_type;_tmp38E=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp370)->f1).tq;_tmp38F=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp370)->f1).num_elts;_tmp390=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp370)->f1).zero_term;_tmp391=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp370)->f1).zt_loc;_LL6: {void*t1=_tmp38D;struct Cyc_Absyn_Tqual tq=_tmp38E;struct Cyc_Absyn_Exp*e=_tmp38F;void*zt=_tmp390;unsigned ztl=_tmp391;
# 2684
void*new_t1=Cyc_Tcutil_rsubstitute(rgn,inst,t1);
struct Cyc_Absyn_Exp*new_e=e == 0?0: Cyc_Tcutil_rsubsexp(rgn,inst,e);
void*new_zt=Cyc_Tcutil_rsubstitute(rgn,inst,zt);
return(new_t1 == t1 && new_e == e)&& new_zt == zt?t:(void*)({struct Cyc_Absyn_ArrayType_Absyn_Type_struct*_tmp39F=_cycalloc(sizeof(*_tmp39F));
_tmp39F->tag=4U,(_tmp39F->f1).elt_type=new_t1,(_tmp39F->f1).tq=tq,(_tmp39F->f1).num_elts=new_e,(_tmp39F->f1).zero_term=new_zt,(_tmp39F->f1).zt_loc=ztl;_tmp39F;});}case 3U: _LL7: _tmp387=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp370)->f1).elt_type;_tmp388=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp370)->f1).elt_tq;_tmp389=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp370)->f1).ptr_atts).rgn;_tmp38A=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp370)->f1).ptr_atts).nullable;_tmp38B=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp370)->f1).ptr_atts).bounds;_tmp38C=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp370)->f1).ptr_atts).zero_term;_LL8: {void*t1=_tmp387;struct Cyc_Absyn_Tqual tq=_tmp388;void*r=_tmp389;void*n=_tmp38A;void*b=_tmp38B;void*zt=_tmp38C;
# 2690
void*new_t1=Cyc_Tcutil_rsubstitute(rgn,inst,t1);
void*new_r=Cyc_Tcutil_rsubstitute(rgn,inst,r);
void*new_b=Cyc_Tcutil_rsubstitute(rgn,inst,b);
void*new_zt=Cyc_Tcutil_rsubstitute(rgn,inst,zt);
if(((new_t1 == t1 && new_r == r)&& new_b == b)&& new_zt == zt)
return t;
# 2694
return
# 2696
 Cyc_Absyn_pointer_type(({struct Cyc_Absyn_PtrInfo _tmp509;_tmp509.elt_type=new_t1,_tmp509.elt_tq=tq,(_tmp509.ptr_atts).rgn=new_r,(_tmp509.ptr_atts).nullable=n,(_tmp509.ptr_atts).bounds=new_b,(_tmp509.ptr_atts).zero_term=new_zt,(_tmp509.ptr_atts).ptrloc=0;_tmp509;}));}case 5U: _LL9: _tmp37A=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp370)->f1).tvars;_tmp37B=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp370)->f1).effect;_tmp37C=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp370)->f1).ret_tqual;_tmp37D=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp370)->f1).ret_type;_tmp37E=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp370)->f1).args;_tmp37F=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp370)->f1).c_varargs;_tmp380=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp370)->f1).cyc_varargs;_tmp381=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp370)->f1).rgn_po;_tmp382=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp370)->f1).attributes;_tmp383=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp370)->f1).requires_clause;_tmp384=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp370)->f1).ensures_clause;_tmp385=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp370)->f1).ieffect;_tmp386=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp370)->f1).oeffect;_LLA: {struct Cyc_List_List*vs=_tmp37A;void*eff=_tmp37B;struct Cyc_Absyn_Tqual rtq=_tmp37C;void*rtyp=_tmp37D;struct Cyc_List_List*args=_tmp37E;int c_varargs=_tmp37F;struct Cyc_Absyn_VarargInfo*cyc_varargs=_tmp380;struct Cyc_List_List*rgn_po=_tmp381;struct Cyc_List_List*atts=_tmp382;struct Cyc_Absyn_Exp*req=_tmp383;struct Cyc_Absyn_Exp*ens=_tmp384;struct Cyc_List_List*ieff=_tmp385;struct Cyc_List_List*oeff=_tmp386;
# 2700
{struct Cyc_List_List*p=vs;for(0;p != 0;p=p->tl){
inst=({struct Cyc_List_List*_tmp3A1=_region_malloc(rgn,sizeof(*_tmp3A1));({struct _tuple19*_tmp68E=({struct _tuple19*_tmp3A0=_region_malloc(rgn,sizeof(*_tmp3A0));_tmp3A0->f1=(struct Cyc_Absyn_Tvar*)p->hd,({void*_tmp68D=Cyc_Absyn_var_type((struct Cyc_Absyn_Tvar*)p->hd);_tmp3A0->f2=_tmp68D;});_tmp3A0;});_tmp3A1->hd=_tmp68E;}),_tmp3A1->tl=inst;_tmp3A1;});}}{
struct _tuple0 _stmttmp69=({struct _RegionHandle*_tmp693=rgn;struct _RegionHandle*_tmp692=rgn;Cyc_List_rsplit(_tmp693,_tmp692,({
struct Cyc_List_List*(*_tmp691)(struct _RegionHandle*,struct _tuple27*(*f)(struct _RegionHandle*,struct _tuple9*),struct _RegionHandle*env,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp3AB)(struct _RegionHandle*,struct _tuple27*(*f)(struct _RegionHandle*,struct _tuple9*),struct _RegionHandle*env,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct _RegionHandle*,struct _tuple27*(*f)(struct _RegionHandle*,struct _tuple9*),struct _RegionHandle*env,struct Cyc_List_List*x))Cyc_List_rmap_c;_tmp3AB;});struct _RegionHandle*_tmp690=rgn;struct _RegionHandle*_tmp68F=rgn;_tmp691(_tmp690,Cyc_Tcutil_substitute_f1,_tmp68F,args);}));});
# 2702
struct Cyc_List_List*_tmp3A3;struct Cyc_List_List*_tmp3A2;_LL21: _tmp3A2=_stmttmp69.f1;_tmp3A3=_stmttmp69.f2;_LL22: {struct Cyc_List_List*qs=_tmp3A2;struct Cyc_List_List*ts=_tmp3A3;
# 2704
struct Cyc_List_List*args2=args;
struct Cyc_List_List*ts2=Cyc_Tcutil_substs(rgn,inst,ts);
if(ts2 != ts)
args2=({struct Cyc_List_List*(*_tmp695)(struct _tuple9*(*f)(struct _tuple9*,void*),struct Cyc_List_List*x,struct Cyc_List_List*y)=({struct Cyc_List_List*(*_tmp3A4)(struct _tuple9*(*f)(struct _tuple9*,void*),struct Cyc_List_List*x,struct Cyc_List_List*y)=(struct Cyc_List_List*(*)(struct _tuple9*(*f)(struct _tuple9*,void*),struct Cyc_List_List*x,struct Cyc_List_List*y))Cyc_List_map2;_tmp3A4;});struct Cyc_List_List*_tmp694=args;_tmp695(Cyc_Tcutil_substitute_f2,_tmp694,ts2);});{
# 2706
void*eff2;
# 2709
if(eff == 0)
eff2=0;else{
# 2712
void*new_eff=Cyc_Tcutil_rsubstitute(rgn,inst,eff);
if(new_eff == eff)
eff2=eff;else{
# 2716
eff2=new_eff;}}{
# 2718
struct Cyc_Absyn_VarargInfo*cyc_varargs2;
if(cyc_varargs == 0)
cyc_varargs2=0;else{
# 2722
struct Cyc_Absyn_VarargInfo _stmttmp6A=*cyc_varargs;int _tmp3A8;void*_tmp3A7;struct Cyc_Absyn_Tqual _tmp3A6;struct _fat_ptr*_tmp3A5;_LL24: _tmp3A5=_stmttmp6A.name;_tmp3A6=_stmttmp6A.tq;_tmp3A7=_stmttmp6A.type;_tmp3A8=_stmttmp6A.inject;_LL25: {struct _fat_ptr*n=_tmp3A5;struct Cyc_Absyn_Tqual tq=_tmp3A6;void*t=_tmp3A7;int i=_tmp3A8;
void*t2=Cyc_Tcutil_rsubstitute(rgn,inst,t);
if(t2 == t)cyc_varargs2=cyc_varargs;else{
cyc_varargs2=({struct Cyc_Absyn_VarargInfo*_tmp3A9=_cycalloc(sizeof(*_tmp3A9));_tmp3A9->name=n,_tmp3A9->tq=tq,_tmp3A9->type=t2,_tmp3A9->inject=i;_tmp3A9;});}}}{
# 2727
struct Cyc_List_List*rgn_po2=Cyc_Tcutil_rsubst_rgnpo(rgn,inst,rgn_po);
struct Cyc_Absyn_Exp*req2=Cyc_Tcutil_rsubs_exp_opt(rgn,inst,req);
struct Cyc_Absyn_Exp*ens2=Cyc_Tcutil_rsubs_exp_opt(rgn,inst,ens);
struct Cyc_List_List*req_relns2=Cyc_Tcutil_extract_relns(args2,req2);
struct Cyc_List_List*ens_relns2=Cyc_Tcutil_extract_relns(args2,ens2);
return(void*)({struct Cyc_Absyn_FnType_Absyn_Type_struct*_tmp3AA=_cycalloc(sizeof(*_tmp3AA));_tmp3AA->tag=5U,(_tmp3AA->f1).tvars=vs,(_tmp3AA->f1).effect=eff2,(_tmp3AA->f1).ret_tqual=rtq,({void*_tmp698=Cyc_Tcutil_rsubstitute(rgn,inst,rtyp);(_tmp3AA->f1).ret_type=_tmp698;}),(_tmp3AA->f1).args=args2,(_tmp3AA->f1).c_varargs=c_varargs,(_tmp3AA->f1).cyc_varargs=cyc_varargs2,(_tmp3AA->f1).rgn_po=rgn_po2,(_tmp3AA->f1).attributes=atts,(_tmp3AA->f1).requires_clause=req2,(_tmp3AA->f1).requires_relns=req_relns2,(_tmp3AA->f1).ensures_clause=ens2,(_tmp3AA->f1).ensures_relns=ens_relns2,({
# 2735
struct Cyc_List_List*_tmp697=Cyc_Tcutil_rsubstitute_eff_list(rgn,inst,ieff);(_tmp3AA->f1).ieffect=_tmp697;}),({
struct Cyc_List_List*_tmp696=Cyc_Tcutil_rsubstitute_eff_list(rgn,inst,oeff);(_tmp3AA->f1).oeffect=_tmp696;});_tmp3AA;});}}}}}}case 6U: _LLB: _tmp379=((struct Cyc_Absyn_TupleType_Absyn_Type_struct*)_tmp370)->f1;_LLC: {struct Cyc_List_List*tqts=_tmp379;
# 2738
struct Cyc_List_List*ts2=0;
int change=0;
{struct Cyc_List_List*ts1=tqts;for(0;ts1 != 0;ts1=ts1->tl){
void*t1=(*((struct _tuple12*)ts1->hd)).f2;
void*t2=Cyc_Tcutil_rsubstitute(rgn,inst,t1);
if(t1 != t2)
change=1;
# 2743
ts2=({struct Cyc_List_List*_tmp3AC=_region_malloc(rgn,sizeof(*_tmp3AC));
# 2746
_tmp3AC->hd=t2,_tmp3AC->tl=ts2;_tmp3AC;});}}
# 2748
if(!change)
return t;
# 2748
return(void*)({struct Cyc_Absyn_TupleType_Absyn_Type_struct*_tmp3AE=_cycalloc(sizeof(*_tmp3AE));
# 2750
_tmp3AE->tag=6U,({struct Cyc_List_List*_tmp69B=({struct Cyc_List_List*(*_tmp69A)(struct _tuple12*(*f)(struct _tuple12*,void*),struct Cyc_List_List*x,struct Cyc_List_List*y)=({struct Cyc_List_List*(*_tmp3AD)(struct _tuple12*(*f)(struct _tuple12*,void*),struct Cyc_List_List*x,struct Cyc_List_List*y)=(struct Cyc_List_List*(*)(struct _tuple12*(*f)(struct _tuple12*,void*),struct Cyc_List_List*x,struct Cyc_List_List*y))Cyc_List_map2;_tmp3AD;});struct Cyc_List_List*_tmp699=tqts;_tmp69A(Cyc_Tcutil_map2_tq,_tmp699,Cyc_List_imp_rev(ts2));});_tmp3AE->f1=_tmp69B;});_tmp3AE;});}case 7U: _LLD: _tmp377=((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmp370)->f1;_tmp378=((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmp370)->f2;_LLE: {enum Cyc_Absyn_AggrKind k=_tmp377;struct Cyc_List_List*fs=_tmp378;
# 2752
struct Cyc_List_List*new_fs=Cyc_Tcutil_subst_aggrfields(rgn,inst,fs);
if(fs == new_fs)return t;return(void*)({struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*_tmp3AF=_cycalloc(sizeof(*_tmp3AF));
_tmp3AF->tag=7U,_tmp3AF->f1=k,_tmp3AF->f2=new_fs;_tmp3AF;});}case 1U: _LLF: _tmp376=(void*)((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp370)->f2;_LL10: {void*r=_tmp376;
# 2756
if(r != 0)return Cyc_Tcutil_rsubstitute(rgn,inst,r);else{
return t;}}case 0U: if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp370)->f2 == 0){_LL11: _tmp375=(void*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp370)->f1;_LL12: {void*c=_tmp375;
return t;}}else{_LL13: _tmp373=(void*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp370)->f1;_tmp374=((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp370)->f2;_LL14: {void*c=_tmp373;struct Cyc_List_List*ts=_tmp374;
# 2760
struct Cyc_List_List*new_ts=Cyc_Tcutil_substs(rgn,inst,ts);
if(ts == new_ts)return t;else{
return(void*)({struct Cyc_Absyn_AppType_Absyn_Type_struct*_tmp3B0=_cycalloc(sizeof(*_tmp3B0));_tmp3B0->tag=0U,_tmp3B0->f1=c,_tmp3B0->f2=new_ts;_tmp3B0;});}}}case 9U: _LL15: _tmp372=((struct Cyc_Absyn_ValueofType_Absyn_Type_struct*)_tmp370)->f1;_LL16: {struct Cyc_Absyn_Exp*e=_tmp372;
# 2764
struct Cyc_Absyn_Exp*new_e=Cyc_Tcutil_rsubsexp(rgn,inst,e);
return new_e == e?t:(void*)({struct Cyc_Absyn_ValueofType_Absyn_Type_struct*_tmp3B1=_cycalloc(sizeof(*_tmp3B1));_tmp3B1->tag=9U,_tmp3B1->f1=new_e;_tmp3B1;});}case 11U: _LL17: _tmp371=((struct Cyc_Absyn_TypeofType_Absyn_Type_struct*)_tmp370)->f1;_LL18: {struct Cyc_Absyn_Exp*e=_tmp371;
# 2767
struct Cyc_Absyn_Exp*new_e=Cyc_Tcutil_rsubsexp(rgn,inst,e);
return new_e == e?t:(void*)({struct Cyc_Absyn_TypeofType_Absyn_Type_struct*_tmp3B2=_cycalloc(sizeof(*_tmp3B2));_tmp3B2->tag=11U,_tmp3B2->f1=new_e;_tmp3B2;});}default: _LL19: _LL1A:
({void*_tmp3B3=0U;({int(*_tmp69D)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp3B5)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Tcutil_impos;_tmp3B5;});struct _fat_ptr _tmp69C=({const char*_tmp3B4="found typedecltype in rsubs";_tag_fat(_tmp3B4,sizeof(char),28U);});_tmp69D(_tmp69C,_tag_fat(_tmp3B3,sizeof(void*),0U));});});}_LL0:;}
# 2512
static struct Cyc_List_List*Cyc_Tcutil_substs(struct _RegionHandle*rgn,struct Cyc_List_List*inst,struct Cyc_List_List*ts){
# 2776
if(ts == 0)
return 0;{
# 2776
void*old_hd=(void*)ts->hd;
# 2779
struct Cyc_List_List*old_tl=ts->tl;
void*new_hd=Cyc_Tcutil_rsubstitute(rgn,inst,old_hd);
struct Cyc_List_List*new_tl=Cyc_Tcutil_substs(rgn,inst,old_tl);
if(old_hd == new_hd && old_tl == new_tl)
return ts;
# 2782
return({struct Cyc_List_List*_tmp3B6=_cycalloc(sizeof(*_tmp3B6));
# 2784
_tmp3B6->hd=new_hd,_tmp3B6->tl=new_tl;_tmp3B6;});}}
# 2512
extern void*Cyc_Tcutil_substitute(struct Cyc_List_List*inst,void*t){
# 2788
if(inst != 0)
return Cyc_Tcutil_rsubstitute(Cyc_Core_heap_region,inst,t);else{
return t;}}
# 2512
struct _tuple19*Cyc_Tcutil_make_inst_var(struct Cyc_List_List*s,struct Cyc_Absyn_Tvar*tv){
# 2795
struct Cyc_Core_Opt*k=Cyc_Tcutil_kind_to_opt(Cyc_Tcutil_tvar_kind(tv,& Cyc_Tcutil_bk));
return({struct _tuple19*_tmp3B8=_cycalloc(sizeof(*_tmp3B8));_tmp3B8->f1=tv,({void*_tmp69F=({struct Cyc_Core_Opt*_tmp69E=k;Cyc_Absyn_new_evar(_tmp69E,({struct Cyc_Core_Opt*_tmp3B7=_cycalloc(sizeof(*_tmp3B7));_tmp3B7->v=s;_tmp3B7;}));});_tmp3B8->f2=_tmp69F;});_tmp3B8;});}struct _tuple28{struct Cyc_List_List*f1;struct _RegionHandle*f2;};
# 2512
struct _tuple19*Cyc_Tcutil_r_make_inst_var(struct _tuple28*env,struct Cyc_Absyn_Tvar*tv){
# 2801
struct _RegionHandle*_tmp3BA;struct Cyc_List_List*_tmp3B9;_LL1: _tmp3B9=env->f1;_tmp3BA=env->f2;_LL2: {struct Cyc_List_List*s=_tmp3B9;struct _RegionHandle*rgn=_tmp3BA;
struct Cyc_Core_Opt*k=Cyc_Tcutil_kind_to_opt(Cyc_Tcutil_tvar_kind(tv,& Cyc_Tcutil_bk));
return({struct _tuple19*_tmp3BC=_region_malloc(rgn,sizeof(*_tmp3BC));_tmp3BC->f1=tv,({void*_tmp6A1=({struct Cyc_Core_Opt*_tmp6A0=k;Cyc_Absyn_new_evar(_tmp6A0,({struct Cyc_Core_Opt*_tmp3BB=_cycalloc(sizeof(*_tmp3BB));_tmp3BB->v=s;_tmp3BB;}));});_tmp3BC->f2=_tmp6A1;});_tmp3BC;});}}
# 2512
void Cyc_Tcutil_check_bitfield(unsigned loc,void*field_type,struct Cyc_Absyn_Exp*width,struct _fat_ptr*fn){
# 2808
if(width != 0){
unsigned w=0U;
if(!Cyc_Tcutil_is_const_exp(width))
({struct Cyc_String_pa_PrintArg_struct _tmp3BF=({struct Cyc_String_pa_PrintArg_struct _tmp50A;_tmp50A.tag=0U,_tmp50A.f1=(struct _fat_ptr)((struct _fat_ptr)*fn);_tmp50A;});void*_tmp3BD[1U];_tmp3BD[0]=& _tmp3BF;({unsigned _tmp6A3=loc;struct _fat_ptr _tmp6A2=({const char*_tmp3BE="bitfield %s does not have constant width";_tag_fat(_tmp3BE,sizeof(char),41U);});Cyc_Tcutil_terr(_tmp6A3,_tmp6A2,_tag_fat(_tmp3BD,sizeof(void*),1U));});});else{
# 2813
struct _tuple13 _stmttmp6B=Cyc_Evexp_eval_const_uint_exp(width);int _tmp3C1;unsigned _tmp3C0;_LL1: _tmp3C0=_stmttmp6B.f1;_tmp3C1=_stmttmp6B.f2;_LL2: {unsigned i=_tmp3C0;int known=_tmp3C1;
if(!known)
({void*_tmp3C2=0U;({unsigned _tmp6A5=loc;struct _fat_ptr _tmp6A4=({const char*_tmp3C3="cannot evaluate bitfield width at compile time";_tag_fat(_tmp3C3,sizeof(char),47U);});Cyc_Tcutil_warn(_tmp6A5,_tmp6A4,_tag_fat(_tmp3C2,sizeof(void*),0U));});});
# 2814
if((int)i < 0)
# 2817
({void*_tmp3C4=0U;({unsigned _tmp6A7=loc;struct _fat_ptr _tmp6A6=({const char*_tmp3C5="bitfield has negative width";_tag_fat(_tmp3C5,sizeof(char),28U);});Cyc_Tcutil_terr(_tmp6A7,_tmp6A6,_tag_fat(_tmp3C4,sizeof(void*),0U));});});
# 2814
w=i;}}{
# 2820
void*_stmttmp6C=Cyc_Tcutil_compress(field_type);void*_tmp3C6=_stmttmp6C;enum Cyc_Absyn_Size_of _tmp3C7;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp3C6)->tag == 0U){if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp3C6)->f1)->tag == 1U){_LL4: _tmp3C7=((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp3C6)->f1)->f2;_LL5: {enum Cyc_Absyn_Size_of b=_tmp3C7;
# 2823
{enum Cyc_Absyn_Size_of _tmp3C8=b;switch(_tmp3C8){case Cyc_Absyn_Char_sz: _LL9: _LLA:
 if(w > (unsigned)8)({void*_tmp3C9=0U;({unsigned _tmp6A9=loc;struct _fat_ptr _tmp6A8=({const char*_tmp3CA="bitfield larger than type";_tag_fat(_tmp3CA,sizeof(char),26U);});Cyc_Tcutil_warn(_tmp6A9,_tmp6A8,_tag_fat(_tmp3C9,sizeof(void*),0U));});});goto _LL8;case Cyc_Absyn_Short_sz: _LLB: _LLC:
 if(w > (unsigned)16)({void*_tmp3CB=0U;({unsigned _tmp6AB=loc;struct _fat_ptr _tmp6AA=({const char*_tmp3CC="bitfield larger than type";_tag_fat(_tmp3CC,sizeof(char),26U);});Cyc_Tcutil_warn(_tmp6AB,_tmp6AA,_tag_fat(_tmp3CB,sizeof(void*),0U));});});goto _LL8;case Cyc_Absyn_Long_sz: _LLD: _LLE:
 goto _LL10;case Cyc_Absyn_Int_sz: _LLF: _LL10:
 if(w > (unsigned)32)({void*_tmp3CD=0U;({unsigned _tmp6AD=loc;struct _fat_ptr _tmp6AC=({const char*_tmp3CE="bitfield larger than type";_tag_fat(_tmp3CE,sizeof(char),26U);});Cyc_Tcutil_warn(_tmp6AD,_tmp6AC,_tag_fat(_tmp3CD,sizeof(void*),0U));});});goto _LL8;case Cyc_Absyn_LongLong_sz: _LL11: _LL12:
 goto _LL14;default: _LL13: _LL14:
 if(w > (unsigned)64)({void*_tmp3CF=0U;({unsigned _tmp6AF=loc;struct _fat_ptr _tmp6AE=({const char*_tmp3D0="bitfield larger than type";_tag_fat(_tmp3D0,sizeof(char),26U);});Cyc_Tcutil_warn(_tmp6AF,_tmp6AE,_tag_fat(_tmp3CF,sizeof(void*),0U));});});goto _LL8;}_LL8:;}
# 2831
goto _LL3;}}else{goto _LL6;}}else{_LL6: _LL7:
# 2833
({struct Cyc_String_pa_PrintArg_struct _tmp3D3=({struct Cyc_String_pa_PrintArg_struct _tmp50C;_tmp50C.tag=0U,_tmp50C.f1=(struct _fat_ptr)((struct _fat_ptr)*fn);_tmp50C;});struct Cyc_String_pa_PrintArg_struct _tmp3D4=({struct Cyc_String_pa_PrintArg_struct _tmp50B;_tmp50B.tag=0U,({
struct _fat_ptr _tmp6B0=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(field_type));_tmp50B.f1=_tmp6B0;});_tmp50B;});void*_tmp3D1[2U];_tmp3D1[0]=& _tmp3D3,_tmp3D1[1]=& _tmp3D4;({unsigned _tmp6B2=loc;struct _fat_ptr _tmp6B1=({const char*_tmp3D2="bitfield %s must have integral type but has type %s";_tag_fat(_tmp3D2,sizeof(char),52U);});Cyc_Tcutil_terr(_tmp6B2,_tmp6B1,_tag_fat(_tmp3D1,sizeof(void*),2U));});});
goto _LL3;}_LL3:;}}}
# 2512
int Cyc_Tcutil_extract_const_from_typedef(unsigned loc,int declared_const,void*t){
# 2843
void*_tmp3D5=t;void*_tmp3D7;struct Cyc_Absyn_Typedefdecl*_tmp3D6;if(((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmp3D5)->tag == 8U){_LL1: _tmp3D6=((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmp3D5)->f3;_tmp3D7=(void*)((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmp3D5)->f4;_LL2: {struct Cyc_Absyn_Typedefdecl*td=_tmp3D6;void*tdopt=_tmp3D7;
# 2845
if((((struct Cyc_Absyn_Typedefdecl*)_check_null(td))->tq).real_const ||(td->tq).print_const){
if(declared_const)({void*_tmp3D8=0U;({unsigned _tmp6B4=loc;struct _fat_ptr _tmp6B3=({const char*_tmp3D9="extra const";_tag_fat(_tmp3D9,sizeof(char),12U);});Cyc_Tcutil_warn(_tmp6B4,_tmp6B3,_tag_fat(_tmp3D8,sizeof(void*),0U));});});return 1;}
# 2845
if((unsigned)tdopt)
# 2851
return Cyc_Tcutil_extract_const_from_typedef(loc,declared_const,tdopt);else{
return declared_const;}}}else{_LL3: _LL4:
 return declared_const;}_LL0:;}
# 2512
void Cyc_Tcutil_add_tvar_identity(struct Cyc_Absyn_Tvar*tv){
# 2858
if(tv->identity == - 1)
({int _tmp6B5=Cyc_Tcutil_new_tvar_id();tv->identity=_tmp6B5;});}
# 2512
void Cyc_Tcutil_add_tvar_identities(struct Cyc_List_List*tvs){
# 2862
({void(*_tmp6B6)(void(*f)(struct Cyc_Absyn_Tvar*),struct Cyc_List_List*x)=({void(*_tmp3DA)(void(*f)(struct Cyc_Absyn_Tvar*),struct Cyc_List_List*x)=(void(*)(void(*f)(struct Cyc_Absyn_Tvar*),struct Cyc_List_List*x))Cyc_List_iter;_tmp3DA;});_tmp6B6(Cyc_Tcutil_add_tvar_identity,tvs);});}
# 2512
static void Cyc_Tcutil_check_unique_unsorted(int(*cmp)(void*,void*),struct Cyc_List_List*vs,unsigned loc,struct _fat_ptr(*a2string)(void*),struct _fat_ptr msg){
# 2870
for(0;vs != 0;vs=vs->tl){
struct Cyc_List_List*vs2=vs->tl;for(0;vs2 != 0;vs2=vs2->tl){
if(cmp(vs->hd,vs2->hd)== 0)
({struct Cyc_String_pa_PrintArg_struct _tmp3DD=({struct Cyc_String_pa_PrintArg_struct _tmp50E;_tmp50E.tag=0U,_tmp50E.f1=(struct _fat_ptr)((struct _fat_ptr)msg);_tmp50E;});struct Cyc_String_pa_PrintArg_struct _tmp3DE=({struct Cyc_String_pa_PrintArg_struct _tmp50D;_tmp50D.tag=0U,({struct _fat_ptr _tmp6B7=(struct _fat_ptr)((struct _fat_ptr)a2string(vs->hd));_tmp50D.f1=_tmp6B7;});_tmp50D;});void*_tmp3DB[2U];_tmp3DB[0]=& _tmp3DD,_tmp3DB[1]=& _tmp3DE;({unsigned _tmp6B9=loc;struct _fat_ptr _tmp6B8=({const char*_tmp3DC="%s: %s";_tag_fat(_tmp3DC,sizeof(char),7U);});Cyc_Tcutil_terr(_tmp6B9,_tmp6B8,_tag_fat(_tmp3DB,sizeof(void*),2U));});});}}}
# 2512
static struct _fat_ptr Cyc_Tcutil_strptr2string(struct _fat_ptr*s){
# 2877
return*s;}
# 2512
void Cyc_Tcutil_check_unique_vars(struct Cyc_List_List*vs,unsigned loc,struct _fat_ptr msg){
# 2881
({void(*_tmp6BC)(int(*cmp)(struct _fat_ptr*,struct _fat_ptr*),struct Cyc_List_List*vs,unsigned loc,struct _fat_ptr(*a2string)(struct _fat_ptr*),struct _fat_ptr msg)=({void(*_tmp3DF)(int(*cmp)(struct _fat_ptr*,struct _fat_ptr*),struct Cyc_List_List*vs,unsigned loc,struct _fat_ptr(*a2string)(struct _fat_ptr*),struct _fat_ptr msg)=(void(*)(int(*cmp)(struct _fat_ptr*,struct _fat_ptr*),struct Cyc_List_List*vs,unsigned loc,struct _fat_ptr(*a2string)(struct _fat_ptr*),struct _fat_ptr msg))Cyc_Tcutil_check_unique_unsorted;_tmp3DF;});struct Cyc_List_List*_tmp6BB=vs;unsigned _tmp6BA=loc;_tmp6BC(Cyc_strptrcmp,_tmp6BB,_tmp6BA,Cyc_Tcutil_strptr2string,msg);});}
# 2512
void Cyc_Tcutil_check_unique_tvars(unsigned loc,struct Cyc_List_List*tvs){
# 2885
({void(*_tmp6BF)(int(*cmp)(struct Cyc_Absyn_Tvar*,struct Cyc_Absyn_Tvar*),struct Cyc_List_List*vs,unsigned loc,struct _fat_ptr(*a2string)(struct Cyc_Absyn_Tvar*),struct _fat_ptr msg)=({void(*_tmp3E0)(int(*cmp)(struct Cyc_Absyn_Tvar*,struct Cyc_Absyn_Tvar*),struct Cyc_List_List*vs,unsigned loc,struct _fat_ptr(*a2string)(struct Cyc_Absyn_Tvar*),struct _fat_ptr msg)=(void(*)(int(*cmp)(struct Cyc_Absyn_Tvar*,struct Cyc_Absyn_Tvar*),struct Cyc_List_List*vs,unsigned loc,struct _fat_ptr(*a2string)(struct Cyc_Absyn_Tvar*),struct _fat_ptr msg))Cyc_Tcutil_check_unique_unsorted;_tmp3E0;});struct Cyc_List_List*_tmp6BE=tvs;unsigned _tmp6BD=loc;_tmp6BF(Cyc_Absyn_tvar_cmp,_tmp6BE,_tmp6BD,Cyc_Absynpp_tvar2string,({const char*_tmp3E1="duplicate type variable";_tag_fat(_tmp3E1,sizeof(char),24U);}));});}struct _tuple29{struct Cyc_Absyn_Aggrfield*f1;int f2;};struct _tuple30{struct Cyc_List_List*f1;void*f2;};struct _tuple31{struct Cyc_Absyn_Aggrfield*f1;void*f2;};
# 2512
struct Cyc_List_List*Cyc_Tcutil_resolve_aggregate_designators(struct _RegionHandle*rgn,unsigned loc,struct Cyc_List_List*des,enum Cyc_Absyn_AggrKind aggr_kind,struct Cyc_List_List*sdfields){
# 2903 "tcutil.cyc"
struct _RegionHandle _tmp3E2=_new_region("temp");struct _RegionHandle*temp=& _tmp3E2;_push_region(temp);
# 2907
{struct Cyc_List_List*fields=0;
{struct Cyc_List_List*sd_fields=sdfields;for(0;sd_fields != 0;sd_fields=sd_fields->tl){
if(({struct _fat_ptr _tmp6C0=(struct _fat_ptr)*((struct Cyc_Absyn_Aggrfield*)sd_fields->hd)->name;Cyc_strcmp(_tmp6C0,({const char*_tmp3E3="";_tag_fat(_tmp3E3,sizeof(char),1U);}));})!= 0)
fields=({struct Cyc_List_List*_tmp3E5=_region_malloc(temp,sizeof(*_tmp3E5));({struct _tuple29*_tmp6C1=({struct _tuple29*_tmp3E4=_region_malloc(temp,sizeof(*_tmp3E4));_tmp3E4->f1=(struct Cyc_Absyn_Aggrfield*)sd_fields->hd,_tmp3E4->f2=0;_tmp3E4;});_tmp3E5->hd=_tmp6C1;}),_tmp3E5->tl=fields;_tmp3E5;});}}
# 2912
fields=Cyc_List_imp_rev(fields);{
# 2914
struct _fat_ptr aggr_str=(int)aggr_kind == (int)0U?({const char*_tmp405="struct";_tag_fat(_tmp405,sizeof(char),7U);}):({const char*_tmp406="union";_tag_fat(_tmp406,sizeof(char),6U);});
# 2917
struct Cyc_List_List*ans=0;
for(0;des != 0;des=des->tl){
struct _tuple30*_stmttmp6D=(struct _tuple30*)des->hd;void*_tmp3E7;struct Cyc_List_List*_tmp3E6;_LL1: _tmp3E6=_stmttmp6D->f1;_tmp3E7=_stmttmp6D->f2;_LL2: {struct Cyc_List_List*dl=_tmp3E6;void*a=_tmp3E7;
if(dl == 0){
# 2922
struct Cyc_List_List*fields2=fields;
for(0;fields2 != 0;fields2=fields2->tl){
if(!(*((struct _tuple29*)fields2->hd)).f2){
(*((struct _tuple29*)fields2->hd)).f2=1;
({struct Cyc_List_List*_tmp6C3=({struct Cyc_List_List*_tmp3E9=_cycalloc(sizeof(*_tmp3E9));({void*_tmp6C2=(void*)({struct Cyc_Absyn_FieldName_Absyn_Designator_struct*_tmp3E8=_cycalloc(sizeof(*_tmp3E8));_tmp3E8->tag=1U,_tmp3E8->f1=((*((struct _tuple29*)fields2->hd)).f1)->name;_tmp3E8;});_tmp3E9->hd=_tmp6C2;}),_tmp3E9->tl=0;_tmp3E9;});(*((struct _tuple30*)des->hd)).f1=_tmp6C3;});
ans=({struct Cyc_List_List*_tmp3EB=_region_malloc(rgn,sizeof(*_tmp3EB));({struct _tuple31*_tmp6C4=({struct _tuple31*_tmp3EA=_region_malloc(rgn,sizeof(*_tmp3EA));_tmp3EA->f1=(*((struct _tuple29*)fields2->hd)).f1,_tmp3EA->f2=a;_tmp3EA;});_tmp3EB->hd=_tmp6C4;}),_tmp3EB->tl=ans;_tmp3EB;});
break;}}
# 2923
if(fields2 == 0)
# 2931
({struct Cyc_String_pa_PrintArg_struct _tmp3EE=({struct Cyc_String_pa_PrintArg_struct _tmp50F;_tmp50F.tag=0U,_tmp50F.f1=(struct _fat_ptr)((struct _fat_ptr)aggr_str);_tmp50F;});void*_tmp3EC[1U];_tmp3EC[0]=& _tmp3EE;({unsigned _tmp6C6=loc;struct _fat_ptr _tmp6C5=({const char*_tmp3ED="too many arguments to %s";_tag_fat(_tmp3ED,sizeof(char),25U);});Cyc_Tcutil_terr(_tmp6C6,_tmp6C5,_tag_fat(_tmp3EC,sizeof(void*),1U));});});}else{
if(dl->tl != 0)
# 2934
({void*_tmp3EF=0U;({unsigned _tmp6C8=loc;struct _fat_ptr _tmp6C7=({const char*_tmp3F0="multiple designators are not yet supported";_tag_fat(_tmp3F0,sizeof(char),43U);});Cyc_Tcutil_terr(_tmp6C8,_tmp6C7,_tag_fat(_tmp3EF,sizeof(void*),0U));});});else{
# 2937
void*_stmttmp6E=(void*)dl->hd;void*_tmp3F1=_stmttmp6E;struct _fat_ptr*_tmp3F2;if(((struct Cyc_Absyn_ArrayElement_Absyn_Designator_struct*)_tmp3F1)->tag == 0U){_LL4: _LL5:
# 2939
({struct Cyc_String_pa_PrintArg_struct _tmp3F5=({struct Cyc_String_pa_PrintArg_struct _tmp510;_tmp510.tag=0U,_tmp510.f1=(struct _fat_ptr)((struct _fat_ptr)aggr_str);_tmp510;});void*_tmp3F3[1U];_tmp3F3[0]=& _tmp3F5;({unsigned _tmp6CA=loc;struct _fat_ptr _tmp6C9=({const char*_tmp3F4="array designator used in argument to %s";_tag_fat(_tmp3F4,sizeof(char),40U);});Cyc_Tcutil_terr(_tmp6CA,_tmp6C9,_tag_fat(_tmp3F3,sizeof(void*),1U));});});
goto _LL3;}else{_LL6: _tmp3F2=((struct Cyc_Absyn_FieldName_Absyn_Designator_struct*)_tmp3F1)->f1;_LL7: {struct _fat_ptr*v=_tmp3F2;
# 2942
struct Cyc_List_List*fields2=fields;
for(0;fields2 != 0;fields2=fields2->tl){
if(Cyc_strptrcmp(v,((*((struct _tuple29*)fields2->hd)).f1)->name)== 0){
if((*((struct _tuple29*)fields2->hd)).f2)
({struct Cyc_String_pa_PrintArg_struct _tmp3F8=({struct Cyc_String_pa_PrintArg_struct _tmp511;_tmp511.tag=0U,_tmp511.f1=(struct _fat_ptr)((struct _fat_ptr)*v);_tmp511;});void*_tmp3F6[1U];_tmp3F6[0]=& _tmp3F8;({unsigned _tmp6CC=loc;struct _fat_ptr _tmp6CB=({const char*_tmp3F7="member %s has already been used as an argument";_tag_fat(_tmp3F7,sizeof(char),47U);});Cyc_Tcutil_terr(_tmp6CC,_tmp6CB,_tag_fat(_tmp3F6,sizeof(void*),1U));});});
# 2945
(*((struct _tuple29*)fields2->hd)).f2=1;
# 2948
ans=({struct Cyc_List_List*_tmp3FA=_region_malloc(rgn,sizeof(*_tmp3FA));({struct _tuple31*_tmp6CD=({struct _tuple31*_tmp3F9=_region_malloc(rgn,sizeof(*_tmp3F9));_tmp3F9->f1=(*((struct _tuple29*)fields2->hd)).f1,_tmp3F9->f2=a;_tmp3F9;});_tmp3FA->hd=_tmp6CD;}),_tmp3FA->tl=ans;_tmp3FA;});
break;}}
# 2943
if(fields2 == 0)
# 2952
({struct Cyc_String_pa_PrintArg_struct _tmp3FD=({struct Cyc_String_pa_PrintArg_struct _tmp512;_tmp512.tag=0U,_tmp512.f1=(struct _fat_ptr)((struct _fat_ptr)*v);_tmp512;});void*_tmp3FB[1U];_tmp3FB[0]=& _tmp3FD;({unsigned _tmp6CF=loc;struct _fat_ptr _tmp6CE=({const char*_tmp3FC="bad field designator %s";_tag_fat(_tmp3FC,sizeof(char),24U);});Cyc_Tcutil_terr(_tmp6CF,_tmp6CE,_tag_fat(_tmp3FB,sizeof(void*),1U));});});
# 2943
goto _LL3;}}_LL3:;}}}}
# 2956
if((int)aggr_kind == (int)0U)
# 2958
for(0;fields != 0;fields=fields->tl){
if(!(*((struct _tuple29*)fields->hd)).f2){
({void*_tmp3FE=0U;({unsigned _tmp6D1=loc;struct _fat_ptr _tmp6D0=({const char*_tmp3FF="too few arguments to struct";_tag_fat(_tmp3FF,sizeof(char),28U);});Cyc_Tcutil_terr(_tmp6D1,_tmp6D0,_tag_fat(_tmp3FE,sizeof(void*),0U));});});
break;}}else{
# 2965
int found=0;
for(0;fields != 0;fields=fields->tl){
if((*((struct _tuple29*)fields->hd)).f2){
if(found)({void*_tmp400=0U;({unsigned _tmp6D3=loc;struct _fat_ptr _tmp6D2=({const char*_tmp401="only one member of a union is allowed";_tag_fat(_tmp401,sizeof(char),38U);});Cyc_Tcutil_terr(_tmp6D3,_tmp6D2,_tag_fat(_tmp400,sizeof(void*),0U));});});found=1;}}
# 2966
if(!found)
# 2971
({void*_tmp402=0U;({unsigned _tmp6D5=loc;struct _fat_ptr _tmp6D4=({const char*_tmp403="missing member for union";_tag_fat(_tmp403,sizeof(char),25U);});Cyc_Tcutil_terr(_tmp6D5,_tmp6D4,_tag_fat(_tmp402,sizeof(void*),0U));});});}{
# 2974
struct Cyc_List_List*_tmp404=Cyc_List_imp_rev(ans);_npop_handler(0U);return _tmp404;}}}
# 2907
;_pop_region(temp);}
# 2512 "tcutil.cyc"
int Cyc_Tcutil_is_zero_ptr_deref(struct Cyc_Absyn_Exp*e1,void**ptr_type,int*is_fat,void**elt_type){
# 2982 "tcutil.cyc"
void*_stmttmp6F=e1->r;void*_tmp407=_stmttmp6F;struct Cyc_Absyn_Exp*_tmp408;struct Cyc_Absyn_Exp*_tmp409;struct Cyc_Absyn_Exp*_tmp40A;struct Cyc_Absyn_Exp*_tmp40B;struct Cyc_Absyn_Exp*_tmp40C;struct Cyc_Absyn_Exp*_tmp40D;switch(*((int*)_tmp407)){case 14U: _LL1: _LL2:
({struct Cyc_String_pa_PrintArg_struct _tmp411=({struct Cyc_String_pa_PrintArg_struct _tmp513;_tmp513.tag=0U,({struct _fat_ptr _tmp6D6=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_exp2string(e1));_tmp513.f1=_tmp6D6;});_tmp513;});void*_tmp40E[1U];_tmp40E[0]=& _tmp411;({int(*_tmp6D8)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp410)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Tcutil_impos;_tmp410;});struct _fat_ptr _tmp6D7=({const char*_tmp40F="we have a cast in a lhs:  %s";_tag_fat(_tmp40F,sizeof(char),29U);});_tmp6D8(_tmp6D7,_tag_fat(_tmp40E,sizeof(void*),1U));});});case 20U: _LL3: _tmp40D=((struct Cyc_Absyn_Deref_e_Absyn_Raw_exp_struct*)_tmp407)->f1;_LL4: {struct Cyc_Absyn_Exp*e1a=_tmp40D;
_tmp40C=e1a;goto _LL6;}case 23U: _LL5: _tmp40C=((struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct*)_tmp407)->f1;_LL6: {struct Cyc_Absyn_Exp*e1a=_tmp40C;
# 2986
return Cyc_Tcutil_is_zero_ptr_type((void*)_check_null(e1a->topt),ptr_type,is_fat,elt_type);}case 22U: _LL7: _tmp40B=((struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct*)_tmp407)->f1;_LL8: {struct Cyc_Absyn_Exp*e1a=_tmp40B;
_tmp40A=e1a;goto _LLA;}case 21U: _LL9: _tmp40A=((struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct*)_tmp407)->f1;_LLA: {struct Cyc_Absyn_Exp*e1a=_tmp40A;
# 2990
if(Cyc_Tcutil_is_zero_ptr_type((void*)_check_null(e1a->topt),ptr_type,is_fat,elt_type))
({struct Cyc_String_pa_PrintArg_struct _tmp415=({struct Cyc_String_pa_PrintArg_struct _tmp514;_tmp514.tag=0U,({
struct _fat_ptr _tmp6D9=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_exp2string(e1));_tmp514.f1=_tmp6D9;});_tmp514;});void*_tmp412[1U];_tmp412[0]=& _tmp415;({int(*_tmp6DB)(struct _fat_ptr fmt,struct _fat_ptr ap)=({
# 2991
int(*_tmp414)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Tcutil_impos;_tmp414;});struct _fat_ptr _tmp6DA=({const char*_tmp413="found zero pointer aggregate member assignment: %s";_tag_fat(_tmp413,sizeof(char),51U);});_tmp6DB(_tmp6DA,_tag_fat(_tmp412,sizeof(void*),1U));});});
# 2990
return 0;}case 13U: _LLB: _tmp409=((struct Cyc_Absyn_Instantiate_e_Absyn_Raw_exp_struct*)_tmp407)->f1;_LLC: {struct Cyc_Absyn_Exp*e1a=_tmp409;
# 2994
_tmp408=e1a;goto _LLE;}case 12U: _LLD: _tmp408=((struct Cyc_Absyn_NoInstantiate_e_Absyn_Raw_exp_struct*)_tmp407)->f1;_LLE: {struct Cyc_Absyn_Exp*e1a=_tmp408;
# 2996
if(Cyc_Tcutil_is_zero_ptr_type((void*)_check_null(e1a->topt),ptr_type,is_fat,elt_type))
({struct Cyc_String_pa_PrintArg_struct _tmp419=({struct Cyc_String_pa_PrintArg_struct _tmp515;_tmp515.tag=0U,({
struct _fat_ptr _tmp6DC=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_exp2string(e1));_tmp515.f1=_tmp6DC;});_tmp515;});void*_tmp416[1U];_tmp416[0]=& _tmp419;({int(*_tmp6DE)(struct _fat_ptr fmt,struct _fat_ptr ap)=({
# 2997
int(*_tmp418)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Tcutil_impos;_tmp418;});struct _fat_ptr _tmp6DD=({const char*_tmp417="found zero pointer instantiate/noinstantiate: %s";_tag_fat(_tmp417,sizeof(char),49U);});_tmp6DE(_tmp6DD,_tag_fat(_tmp416,sizeof(void*),1U));});});
# 2996
return 0;}case 1U: _LLF: _LL10:
# 3000
 return 0;default: _LL11: _LL12:
({struct Cyc_String_pa_PrintArg_struct _tmp41D=({struct Cyc_String_pa_PrintArg_struct _tmp516;_tmp516.tag=0U,({struct _fat_ptr _tmp6DF=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_exp2string(e1));_tmp516.f1=_tmp6DF;});_tmp516;});void*_tmp41A[1U];_tmp41A[0]=& _tmp41D;({int(*_tmp6E1)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp41C)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Tcutil_impos;_tmp41C;});struct _fat_ptr _tmp6E0=({const char*_tmp41B="found bad lhs in is_zero_ptr_deref: %s";_tag_fat(_tmp41B,sizeof(char),39U);});_tmp6E1(_tmp6E0,_tag_fat(_tmp41A,sizeof(void*),1U));});});}_LL0:;}
# 2512 "tcutil.cyc"
int Cyc_Tcutil_is_noalias_region(void*r,int must_be_unique){
# 3014 "tcutil.cyc"
void*_stmttmp70=Cyc_Tcutil_compress(r);void*_tmp41E=_stmttmp70;struct Cyc_Absyn_Tvar*_tmp41F;enum Cyc_Absyn_AliasQual _tmp421;enum Cyc_Absyn_KindQual _tmp420;switch(*((int*)_tmp41E)){case 0U: switch(*((int*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp41E)->f1)){case 7U: _LL1: _LL2:
 return !must_be_unique;case 6U: _LL3: _LL4:
 return 1;default: goto _LL9;}case 8U: if(((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmp41E)->f3 != 0){if(((struct Cyc_Absyn_Typedefdecl*)((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmp41E)->f3)->kind != 0){if(((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmp41E)->f4 == 0){_LL5: _tmp420=((struct Cyc_Absyn_Kind*)((((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmp41E)->f3)->kind)->v)->kind;_tmp421=((struct Cyc_Absyn_Kind*)((((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmp41E)->f3)->kind)->v)->aliasqual;_LL6: {enum Cyc_Absyn_KindQual k=_tmp420;enum Cyc_Absyn_AliasQual a=_tmp421;
# 3018
return(int)k == (int)3U &&((int)a == (int)1U ||(int)a == (int)2U && !must_be_unique);}}else{goto _LL9;}}else{goto _LL9;}}else{goto _LL9;}case 2U: _LL7: _tmp41F=((struct Cyc_Absyn_VarType_Absyn_Type_struct*)_tmp41E)->f1;_LL8: {struct Cyc_Absyn_Tvar*tv=_tmp41F;
# 3022
struct Cyc_Absyn_Kind*_stmttmp71=Cyc_Tcutil_tvar_kind(tv,& Cyc_Tcutil_rk);enum Cyc_Absyn_AliasQual _tmp423;enum Cyc_Absyn_KindQual _tmp422;_LLC: _tmp422=_stmttmp71->kind;_tmp423=_stmttmp71->aliasqual;_LLD: {enum Cyc_Absyn_KindQual k=_tmp422;enum Cyc_Absyn_AliasQual a=_tmp423;
if((int)k == (int)3U &&((int)a == (int)1U ||(int)a == (int)2U && !must_be_unique)){
void*_stmttmp72=Cyc_Absyn_compress_kb(tv->kind);void*_tmp424=_stmttmp72;struct Cyc_Core_Opt**_tmp425;if(((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp424)->tag == 2U){if(((struct Cyc_Absyn_Kind*)((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp424)->f2)->kind == Cyc_Absyn_RgnKind){if(((struct Cyc_Absyn_Kind*)((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp424)->f2)->aliasqual == Cyc_Absyn_Top){_LLF: _tmp425=(struct Cyc_Core_Opt**)&((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp424)->f1;_LL10: {struct Cyc_Core_Opt**x=_tmp425;
# 3026
({struct Cyc_Core_Opt*_tmp6E3=({struct Cyc_Core_Opt*_tmp427=_cycalloc(sizeof(*_tmp427));({void*_tmp6E2=(void*)({struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*_tmp426=_cycalloc(sizeof(*_tmp426));_tmp426->tag=2U,_tmp426->f1=0,_tmp426->f2=& Cyc_Tcutil_rk;_tmp426;});_tmp427->v=_tmp6E2;});_tmp427;});*x=_tmp6E3;});
return 0;}}else{goto _LL11;}}else{goto _LL11;}}else{_LL11: _LL12:
 return 1;}_LLE:;}
# 3023
return 0;}}default: _LL9: _LLA:
# 3032
 return 0;}_LL0:;}
# 2512 "tcutil.cyc"
int Cyc_Tcutil_is_noalias_pointer(void*t,int must_be_unique){
# 3039 "tcutil.cyc"
void*_stmttmp73=Cyc_Tcutil_compress(t);void*_tmp428=_stmttmp73;struct Cyc_Absyn_Tvar*_tmp429;void*_tmp42A;switch(*((int*)_tmp428)){case 3U: _LL1: _tmp42A=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp428)->f1).ptr_atts).rgn;_LL2: {void*r=_tmp42A;
# 3041
return Cyc_Tcutil_is_noalias_region(r,must_be_unique);}case 2U: _LL3: _tmp429=((struct Cyc_Absyn_VarType_Absyn_Type_struct*)_tmp428)->f1;_LL4: {struct Cyc_Absyn_Tvar*tv=_tmp429;
# 3043
struct Cyc_Absyn_Kind*_stmttmp74=Cyc_Tcutil_tvar_kind(tv,& Cyc_Tcutil_bk);enum Cyc_Absyn_AliasQual _tmp42C;enum Cyc_Absyn_KindQual _tmp42B;_LL8: _tmp42B=_stmttmp74->kind;_tmp42C=_stmttmp74->aliasqual;_LL9: {enum Cyc_Absyn_KindQual k=_tmp42B;enum Cyc_Absyn_AliasQual a=_tmp42C;
enum Cyc_Absyn_KindQual _tmp42D=k;switch(_tmp42D){case Cyc_Absyn_BoxKind: _LLB: _LLC:
 goto _LLE;case Cyc_Absyn_AnyKind: _LLD: _LLE: goto _LL10;case Cyc_Absyn_MemKind: _LLF: _LL10:
 if((int)a == (int)1U ||(int)a == (int)2U && !must_be_unique){
void*_stmttmp75=Cyc_Absyn_compress_kb(tv->kind);void*_tmp42E=_stmttmp75;enum Cyc_Absyn_KindQual _tmp430;struct Cyc_Core_Opt**_tmp42F;if(((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp42E)->tag == 2U){if(((struct Cyc_Absyn_Kind*)((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp42E)->f2)->aliasqual == Cyc_Absyn_Top){_LL14: _tmp42F=(struct Cyc_Core_Opt**)&((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp42E)->f1;_tmp430=(((struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*)_tmp42E)->f2)->kind;_LL15: {struct Cyc_Core_Opt**x=_tmp42F;enum Cyc_Absyn_KindQual k=_tmp430;
# 3049
({struct Cyc_Core_Opt*_tmp6E6=({struct Cyc_Core_Opt*_tmp433=_cycalloc(sizeof(*_tmp433));({void*_tmp6E5=(void*)({struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct*_tmp432=_cycalloc(sizeof(*_tmp432));_tmp432->tag=2U,_tmp432->f1=0,({struct Cyc_Absyn_Kind*_tmp6E4=({struct Cyc_Absyn_Kind*_tmp431=_cycalloc(sizeof(*_tmp431));_tmp431->kind=k,_tmp431->aliasqual=Cyc_Absyn_Aliasable;_tmp431;});_tmp432->f2=_tmp6E4;});_tmp432;});_tmp433->v=_tmp6E5;});_tmp433;});*x=_tmp6E6;});
return 0;}}else{goto _LL16;}}else{_LL16: _LL17:
# 3053
 return 1;}_LL13:;}
# 3046
return 0;default: _LL11: _LL12:
# 3057
 return 0;}_LLA:;}}default: _LL5: _LL6:
# 3059
 return 0;}_LL0:;}
# 2512 "tcutil.cyc"
int Cyc_Tcutil_is_noalias_pointer_or_aggr(void*t0){
# 3063 "tcutil.cyc"
void*t=Cyc_Tcutil_compress(t0);
if(Cyc_Tcutil_is_noalias_pointer(t,0))return 1;{void*_tmp434=t;struct Cyc_List_List*_tmp435;struct Cyc_List_List*_tmp437;union Cyc_Absyn_DatatypeFieldInfo _tmp436;struct Cyc_List_List*_tmp439;union Cyc_Absyn_DatatypeInfo _tmp438;struct Cyc_List_List*_tmp43B;struct Cyc_Absyn_Aggrdecl**_tmp43A;struct Cyc_List_List*_tmp43C;switch(*((int*)_tmp434)){case 6U: _LL1: _tmp43C=((struct Cyc_Absyn_TupleType_Absyn_Type_struct*)_tmp434)->f1;_LL2: {struct Cyc_List_List*qts=_tmp43C;
# 3067
while(qts != 0){
if(Cyc_Tcutil_is_noalias_pointer_or_aggr((*((struct _tuple12*)qts->hd)).f2))return 1;qts=qts->tl;}
# 3071
return 0;}case 0U: switch(*((int*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp434)->f1)){case 20U: if(((((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp434)->f1)->f1).KnownAggr).tag == 2){_LL3: _tmp43A=((((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp434)->f1)->f1).KnownAggr).val;_tmp43B=((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp434)->f2;_LL4: {struct Cyc_Absyn_Aggrdecl**adp=_tmp43A;struct Cyc_List_List*ts=_tmp43B;
# 3073
if((*adp)->impl == 0)return 0;else{
# 3075
struct Cyc_List_List*inst=Cyc_List_zip((*adp)->tvs,ts);
struct Cyc_List_List*x=((struct Cyc_Absyn_AggrdeclImpl*)_check_null((*adp)->impl))->fields;
void*t;
while(x != 0){
t=inst == 0?((struct Cyc_Absyn_Aggrfield*)x->hd)->type: Cyc_Tcutil_substitute(inst,((struct Cyc_Absyn_Aggrfield*)x->hd)->type);
if(Cyc_Tcutil_is_noalias_pointer_or_aggr(t))return 1;x=x->tl;}
# 3083
return 0;}}}else{_LL7: _LL8:
# 3093
 return 0;}case 18U: _LL9: _tmp438=((struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp434)->f1)->f1;_tmp439=((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp434)->f2;_LLA: {union Cyc_Absyn_DatatypeInfo tinfo=_tmp438;struct Cyc_List_List*ts=_tmp439;
# 3095
union Cyc_Absyn_DatatypeInfo _tmp43D=tinfo;struct Cyc_Core_Opt*_tmp43F;struct Cyc_List_List*_tmp43E;int _tmp441;struct _tuple1*_tmp440;if((_tmp43D.UnknownDatatype).tag == 1){_LL10: _tmp440=((_tmp43D.UnknownDatatype).val).name;_tmp441=((_tmp43D.UnknownDatatype).val).is_extensible;_LL11: {struct _tuple1*nm=_tmp440;int isxt=_tmp441;
# 3098
return 0;}}else{_LL12: _tmp43E=(*(_tmp43D.KnownDatatype).val)->tvs;_tmp43F=(*(_tmp43D.KnownDatatype).val)->fields;_LL13: {struct Cyc_List_List*tvs=_tmp43E;struct Cyc_Core_Opt*flds=_tmp43F;
# 3101
return 0;}}_LLF:;}case 19U: _LLB: _tmp436=((struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp434)->f1)->f1;_tmp437=((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp434)->f2;_LLC: {union Cyc_Absyn_DatatypeFieldInfo tinfo=_tmp436;struct Cyc_List_List*ts=_tmp437;
# 3104
union Cyc_Absyn_DatatypeFieldInfo _tmp442=tinfo;struct Cyc_Absyn_Datatypefield*_tmp444;struct Cyc_Absyn_Datatypedecl*_tmp443;if((_tmp442.UnknownDatatypefield).tag == 1){_LL15: _LL16:
# 3107
 return 0;}else{_LL17: _tmp443=((_tmp442.KnownDatatypefield).val).f1;_tmp444=((_tmp442.KnownDatatypefield).val).f2;_LL18: {struct Cyc_Absyn_Datatypedecl*td=_tmp443;struct Cyc_Absyn_Datatypefield*fld=_tmp444;
# 3109
struct Cyc_List_List*inst=Cyc_List_zip(td->tvs,ts);
struct Cyc_List_List*typs=fld->typs;
while(typs != 0){
t=inst == 0?(*((struct _tuple12*)typs->hd)).f2: Cyc_Tcutil_substitute(inst,(*((struct _tuple12*)typs->hd)).f2);
if(Cyc_Tcutil_is_noalias_pointer_or_aggr(t))return 1;typs=typs->tl;}
# 3116
return 0;}}_LL14:;}default: goto _LLD;}case 7U: _LL5: _tmp435=((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmp434)->f2;_LL6: {struct Cyc_List_List*x=_tmp435;
# 3086
while(x != 0){
if(Cyc_Tcutil_is_noalias_pointer_or_aggr(((struct Cyc_Absyn_Aggrfield*)x->hd)->type))return 1;x=x->tl;}
# 3090
return 0;}default: _LLD: _LLE:
# 3118
 return 0;}_LL0:;}}
# 2512 "tcutil.cyc"
int Cyc_Tcutil_is_noalias_path(struct Cyc_Absyn_Exp*e){
# 3126 "tcutil.cyc"
void*_stmttmp76=e->r;void*_tmp445=_stmttmp76;struct Cyc_Absyn_Stmt*_tmp446;struct Cyc_Absyn_Exp*_tmp447;struct Cyc_Absyn_Exp*_tmp448;struct Cyc_Absyn_Exp*_tmp44A;struct Cyc_Absyn_Exp*_tmp449;struct Cyc_Absyn_Exp*_tmp44C;struct Cyc_Absyn_Exp*_tmp44B;struct _fat_ptr*_tmp44E;struct Cyc_Absyn_Exp*_tmp44D;struct Cyc_Absyn_Exp*_tmp44F;struct Cyc_Absyn_Exp*_tmp450;switch(*((int*)_tmp445)){case 1U: if(((struct Cyc_Absyn_Global_b_Absyn_Binding_struct*)((struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct*)_tmp445)->f1)->tag == 1U){_LL1: _LL2:
 return 0;}else{goto _LL13;}case 22U: _LL3: _tmp450=((struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct*)_tmp445)->f1;_LL4: {struct Cyc_Absyn_Exp*e1=_tmp450;
_tmp44F=e1;goto _LL6;}case 20U: _LL5: _tmp44F=((struct Cyc_Absyn_Deref_e_Absyn_Raw_exp_struct*)_tmp445)->f1;_LL6: {struct Cyc_Absyn_Exp*e1=_tmp44F;
# 3130
return Cyc_Tcutil_is_noalias_pointer((void*)_check_null(e1->topt),1)&& Cyc_Tcutil_is_noalias_path(e1);}case 21U: _LL7: _tmp44D=((struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct*)_tmp445)->f1;_tmp44E=((struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct*)_tmp445)->f2;_LL8: {struct Cyc_Absyn_Exp*e1=_tmp44D;struct _fat_ptr*f=_tmp44E;
return Cyc_Tcutil_is_noalias_path(e1);}case 23U: _LL9: _tmp44B=((struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct*)_tmp445)->f1;_tmp44C=((struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct*)_tmp445)->f2;_LLA: {struct Cyc_Absyn_Exp*e1=_tmp44B;struct Cyc_Absyn_Exp*e2=_tmp44C;
# 3133
void*_stmttmp77=Cyc_Tcutil_compress((void*)_check_null(e1->topt));void*_tmp451=_stmttmp77;if(((struct Cyc_Absyn_TupleType_Absyn_Type_struct*)_tmp451)->tag == 6U){_LL16: _LL17:
 return Cyc_Tcutil_is_noalias_path(e1);}else{_LL18: _LL19:
 return 0;}_LL15:;}case 6U: _LLB: _tmp449=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmp445)->f2;_tmp44A=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmp445)->f3;_LLC: {struct Cyc_Absyn_Exp*e1=_tmp449;struct Cyc_Absyn_Exp*e2=_tmp44A;
# 3138
return Cyc_Tcutil_is_noalias_path(e1)&& Cyc_Tcutil_is_noalias_path(e2);}case 9U: _LLD: _tmp448=((struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct*)_tmp445)->f2;_LLE: {struct Cyc_Absyn_Exp*e2=_tmp448;
_tmp447=e2;goto _LL10;}case 14U: _LLF: _tmp447=((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp445)->f2;_LL10: {struct Cyc_Absyn_Exp*e2=_tmp447;
return Cyc_Tcutil_is_noalias_path(e2);}case 38U: _LL11: _tmp446=((struct Cyc_Absyn_StmtExp_e_Absyn_Raw_exp_struct*)_tmp445)->f1;_LL12: {struct Cyc_Absyn_Stmt*s=_tmp446;
# 3142
while(1){
void*_stmttmp78=s->r;void*_tmp452=_stmttmp78;struct Cyc_Absyn_Exp*_tmp453;struct Cyc_Absyn_Stmt*_tmp455;struct Cyc_Absyn_Decl*_tmp454;struct Cyc_Absyn_Stmt*_tmp457;struct Cyc_Absyn_Stmt*_tmp456;switch(*((int*)_tmp452)){case 2U: _LL1B: _tmp456=((struct Cyc_Absyn_Seq_s_Absyn_Raw_stmt_struct*)_tmp452)->f1;_tmp457=((struct Cyc_Absyn_Seq_s_Absyn_Raw_stmt_struct*)_tmp452)->f2;_LL1C: {struct Cyc_Absyn_Stmt*s1=_tmp456;struct Cyc_Absyn_Stmt*s2=_tmp457;
s=s2;goto _LL1A;}case 12U: _LL1D: _tmp454=((struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct*)_tmp452)->f1;_tmp455=((struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct*)_tmp452)->f2;_LL1E: {struct Cyc_Absyn_Decl*d=_tmp454;struct Cyc_Absyn_Stmt*s1=_tmp455;
s=s1;goto _LL1A;}case 1U: _LL1F: _tmp453=((struct Cyc_Absyn_Exp_s_Absyn_Raw_stmt_struct*)_tmp452)->f1;_LL20: {struct Cyc_Absyn_Exp*e=_tmp453;
return Cyc_Tcutil_is_noalias_path(e);}default: _LL21: _LL22:
({void*_tmp458=0U;({int(*_tmp6E8)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp45A)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Tcutil_impos;_tmp45A;});struct _fat_ptr _tmp6E7=({const char*_tmp459="is_noalias_stmt_exp: ill-formed StmtExp";_tag_fat(_tmp459,sizeof(char),40U);});_tmp6E8(_tmp6E7,_tag_fat(_tmp458,sizeof(void*),0U));});});}_LL1A:;}}default: _LL13: _LL14:
# 3150
 return 1;}_LL0:;}
# 2512 "tcutil.cyc"
struct _tuple14 Cyc_Tcutil_addressof_props(struct Cyc_Absyn_Exp*e){
# 3169 "tcutil.cyc"
struct _tuple14 bogus_ans=({struct _tuple14 _tmp523;_tmp523.f1=0,_tmp523.f2=Cyc_Absyn_heap_rgn_type;_tmp523;});
void*_stmttmp79=e->r;void*_tmp45B=_stmttmp79;struct Cyc_Absyn_Exp*_tmp45D;struct Cyc_Absyn_Exp*_tmp45C;struct Cyc_Absyn_Exp*_tmp45E;int _tmp461;struct _fat_ptr*_tmp460;struct Cyc_Absyn_Exp*_tmp45F;int _tmp464;struct _fat_ptr*_tmp463;struct Cyc_Absyn_Exp*_tmp462;void*_tmp465;switch(*((int*)_tmp45B)){case 1U: _LL1: _tmp465=(void*)((struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct*)_tmp45B)->f1;_LL2: {void*x=_tmp465;
# 3173
void*_tmp466=x;struct Cyc_Absyn_Vardecl*_tmp467;struct Cyc_Absyn_Vardecl*_tmp468;struct Cyc_Absyn_Vardecl*_tmp469;struct Cyc_Absyn_Vardecl*_tmp46A;switch(*((int*)_tmp466)){case 0U: _LLE: _LLF:
 goto _LL11;case 2U: _LL10: _LL11:
 return bogus_ans;case 1U: _LL12: _tmp46A=((struct Cyc_Absyn_Global_b_Absyn_Binding_struct*)_tmp466)->f1;_LL13: {struct Cyc_Absyn_Vardecl*vd=_tmp46A;
# 3177
void*_stmttmp7A=Cyc_Tcutil_compress((void*)_check_null(e->topt));void*_tmp46B=_stmttmp7A;if(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp46B)->tag == 4U){_LL1B: _LL1C:
# 3179
 return({struct _tuple14 _tmp517;_tmp517.f1=1,_tmp517.f2=Cyc_Absyn_heap_rgn_type;_tmp517;});}else{_LL1D: _LL1E:
 return({struct _tuple14 _tmp518;_tmp518.f1=(vd->tq).real_const,_tmp518.f2=Cyc_Absyn_heap_rgn_type;_tmp518;});}_LL1A:;}case 4U: _LL14: _tmp469=((struct Cyc_Absyn_Local_b_Absyn_Binding_struct*)_tmp466)->f1;_LL15: {struct Cyc_Absyn_Vardecl*vd=_tmp469;
# 3183
void*_stmttmp7B=Cyc_Tcutil_compress((void*)_check_null(e->topt));void*_tmp46C=_stmttmp7B;if(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp46C)->tag == 4U){_LL20: _LL21:
 return({struct _tuple14 _tmp519;_tmp519.f1=1,_tmp519.f2=(void*)_check_null(vd->rgn);_tmp519;});}else{_LL22: _LL23:
# 3186
 vd->escapes=1;
return({struct _tuple14 _tmp51A;_tmp51A.f1=(vd->tq).real_const,_tmp51A.f2=(void*)_check_null(vd->rgn);_tmp51A;});}_LL1F:;}case 5U: _LL16: _tmp468=((struct Cyc_Absyn_Pat_b_Absyn_Binding_struct*)_tmp466)->f1;_LL17: {struct Cyc_Absyn_Vardecl*vd=_tmp468;
# 3189
_tmp467=vd;goto _LL19;}default: _LL18: _tmp467=((struct Cyc_Absyn_Param_b_Absyn_Binding_struct*)_tmp466)->f1;_LL19: {struct Cyc_Absyn_Vardecl*vd=_tmp467;
# 3191
vd->escapes=1;
return({struct _tuple14 _tmp51B;_tmp51B.f1=(vd->tq).real_const,_tmp51B.f2=(void*)_check_null(vd->rgn);_tmp51B;});}}_LLD:;}case 21U: _LL3: _tmp462=((struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct*)_tmp45B)->f1;_tmp463=((struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct*)_tmp45B)->f2;_tmp464=((struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct*)_tmp45B)->f3;_LL4: {struct Cyc_Absyn_Exp*e1=_tmp462;struct _fat_ptr*f=_tmp463;int is_tagged=_tmp464;
# 3196
if(is_tagged)return bogus_ans;{void*_stmttmp7C=
# 3199
Cyc_Tcutil_compress((void*)_check_null(e1->topt));
# 3196
void*_tmp46D=_stmttmp7C;struct Cyc_Absyn_Aggrdecl*_tmp46E;struct Cyc_List_List*_tmp46F;switch(*((int*)_tmp46D)){case 7U: _LL25: _tmp46F=((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmp46D)->f2;_LL26: {struct Cyc_List_List*fs=_tmp46F;
# 3201
struct Cyc_Absyn_Aggrfield*finfo=Cyc_Absyn_lookup_field(fs,f);
if(finfo != 0 && finfo->width == 0){
struct _tuple14 _stmttmp7D=Cyc_Tcutil_addressof_props(e1);void*_tmp471;int _tmp470;_LL2C: _tmp470=_stmttmp7D.f1;_tmp471=_stmttmp7D.f2;_LL2D: {int c=_tmp470;void*t=_tmp471;
return({struct _tuple14 _tmp51C;_tmp51C.f1=(finfo->tq).real_const || c,_tmp51C.f2=t;_tmp51C;});}}
# 3202
return bogus_ans;}case 0U: if(((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp46D)->f1)->tag == 20U){if(((((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp46D)->f1)->f1).KnownAggr).tag == 2){_LL27: _tmp46E=*((((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp46D)->f1)->f1).KnownAggr).val;_LL28: {struct Cyc_Absyn_Aggrdecl*ad=_tmp46E;
# 3208
struct Cyc_Absyn_Aggrfield*finfo=Cyc_Absyn_lookup_decl_field(ad,f);
if(finfo != 0 && finfo->width == 0){
struct _tuple14 _stmttmp7E=Cyc_Tcutil_addressof_props(e1);void*_tmp473;int _tmp472;_LL2F: _tmp472=_stmttmp7E.f1;_tmp473=_stmttmp7E.f2;_LL30: {int c=_tmp472;void*t=_tmp473;
return({struct _tuple14 _tmp51D;_tmp51D.f1=(finfo->tq).real_const || c,_tmp51D.f2=t;_tmp51D;});}}
# 3209
return bogus_ans;}}else{goto _LL29;}}else{goto _LL29;}default: _LL29: _LL2A:
# 3214
 return bogus_ans;}_LL24:;}}case 22U: _LL5: _tmp45F=((struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct*)_tmp45B)->f1;_tmp460=((struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct*)_tmp45B)->f2;_tmp461=((struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct*)_tmp45B)->f3;_LL6: {struct Cyc_Absyn_Exp*e1=_tmp45F;struct _fat_ptr*f=_tmp460;int is_tagged=_tmp461;
# 3218
if(is_tagged)return bogus_ans;{void*_stmttmp7F=
# 3221
Cyc_Tcutil_compress((void*)_check_null(e1->topt));
# 3218
void*_tmp474=_stmttmp7F;void*_tmp476;void*_tmp475;if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp474)->tag == 3U){_LL32: _tmp475=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp474)->f1).elt_type;_tmp476=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp474)->f1).ptr_atts).rgn;_LL33: {void*t1=_tmp475;void*r=_tmp476;
# 3223
struct Cyc_Absyn_Aggrfield*finfo;
{void*_stmttmp80=Cyc_Tcutil_compress(t1);void*_tmp477=_stmttmp80;struct Cyc_Absyn_Aggrdecl*_tmp478;struct Cyc_List_List*_tmp479;switch(*((int*)_tmp477)){case 7U: _LL37: _tmp479=((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmp477)->f2;_LL38: {struct Cyc_List_List*fs=_tmp479;
# 3226
finfo=Cyc_Absyn_lookup_field(fs,f);goto _LL36;}case 0U: if(((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp477)->f1)->tag == 20U){if(((((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp477)->f1)->f1).KnownAggr).tag == 2){_LL39: _tmp478=*((((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp477)->f1)->f1).KnownAggr).val;_LL3A: {struct Cyc_Absyn_Aggrdecl*ad=_tmp478;
# 3228
finfo=Cyc_Absyn_lookup_decl_field(ad,f);goto _LL36;}}else{goto _LL3B;}}else{goto _LL3B;}default: _LL3B: _LL3C:
 return bogus_ans;}_LL36:;}
# 3231
if(finfo != 0 && finfo->width == 0)
return({struct _tuple14 _tmp51E;_tmp51E.f1=(finfo->tq).real_const,_tmp51E.f2=r;_tmp51E;});
# 3231
return bogus_ans;}}else{_LL34: _LL35:
# 3234
 return bogus_ans;}_LL31:;}}case 20U: _LL7: _tmp45E=((struct Cyc_Absyn_Deref_e_Absyn_Raw_exp_struct*)_tmp45B)->f1;_LL8: {struct Cyc_Absyn_Exp*e1=_tmp45E;
# 3238
void*_stmttmp81=Cyc_Tcutil_compress((void*)_check_null(e1->topt));void*_tmp47A=_stmttmp81;void*_tmp47C;struct Cyc_Absyn_Tqual _tmp47B;if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp47A)->tag == 3U){_LL3E: _tmp47B=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp47A)->f1).elt_tq;_tmp47C=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp47A)->f1).ptr_atts).rgn;_LL3F: {struct Cyc_Absyn_Tqual tq=_tmp47B;void*r=_tmp47C;
# 3240
return({struct _tuple14 _tmp51F;_tmp51F.f1=tq.real_const,_tmp51F.f2=r;_tmp51F;});}}else{_LL40: _LL41:
 return bogus_ans;}_LL3D:;}case 23U: _LL9: _tmp45C=((struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct*)_tmp45B)->f1;_tmp45D=((struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct*)_tmp45B)->f2;_LLA: {struct Cyc_Absyn_Exp*e1=_tmp45C;struct Cyc_Absyn_Exp*e2=_tmp45D;
# 3246
void*t=Cyc_Tcutil_compress((void*)_check_null(e1->topt));
void*_tmp47D=t;struct Cyc_Absyn_Tqual _tmp47E;void*_tmp480;struct Cyc_Absyn_Tqual _tmp47F;struct Cyc_List_List*_tmp481;switch(*((int*)_tmp47D)){case 6U: _LL43: _tmp481=((struct Cyc_Absyn_TupleType_Absyn_Type_struct*)_tmp47D)->f1;_LL44: {struct Cyc_List_List*ts=_tmp481;
# 3250
struct _tuple13 _stmttmp82=Cyc_Evexp_eval_const_uint_exp(e2);int _tmp483;unsigned _tmp482;_LL4C: _tmp482=_stmttmp82.f1;_tmp483=_stmttmp82.f2;_LL4D: {unsigned i=_tmp482;int known=_tmp483;
if(!known)
return bogus_ans;{
# 3251
struct _tuple12*finfo=
# 3253
Cyc_Absyn_lookup_tuple_field(ts,(int)i);
if(finfo != 0)
return({struct _tuple14 _tmp520;_tmp520.f1=((*finfo).f1).real_const,({void*_tmp6E9=(Cyc_Tcutil_addressof_props(e1)).f2;_tmp520.f2=_tmp6E9;});_tmp520;});
# 3254
return bogus_ans;}}}case 3U: _LL45: _tmp47F=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp47D)->f1).elt_tq;_tmp480=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp47D)->f1).ptr_atts).rgn;_LL46: {struct Cyc_Absyn_Tqual tq=_tmp47F;void*r=_tmp480;
# 3258
return({struct _tuple14 _tmp521;_tmp521.f1=tq.real_const,_tmp521.f2=r;_tmp521;});}case 4U: _LL47: _tmp47E=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp47D)->f1).tq;_LL48: {struct Cyc_Absyn_Tqual tq=_tmp47E;
# 3264
return({struct _tuple14 _tmp522;_tmp522.f1=tq.real_const,({void*_tmp6EA=(Cyc_Tcutil_addressof_props(e1)).f2;_tmp522.f2=_tmp6EA;});_tmp522;});}default: _LL49: _LL4A:
 return bogus_ans;}_LL42:;}default: _LLB: _LLC:
# 3268
({void*_tmp484=0U;({unsigned _tmp6EC=e->loc;struct _fat_ptr _tmp6EB=({const char*_tmp485="unary & applied to non-lvalue";_tag_fat(_tmp485,sizeof(char),30U);});Cyc_Tcutil_terr(_tmp6EC,_tmp6EB,_tag_fat(_tmp484,sizeof(void*),0U));});});
return bogus_ans;}_LL0:;}
# 2512 "tcutil.cyc"
void Cyc_Tcutil_check_bound(unsigned loc,unsigned i,void*b,int do_warn){
# 3276 "tcutil.cyc"
struct Cyc_Absyn_Exp*eopt=({void*_tmp6ED=Cyc_Absyn_bounds_one();Cyc_Tcutil_get_bounds_exp(_tmp6ED,b);});
if(eopt == 0)return;{struct Cyc_Absyn_Exp*e=eopt;
# 3279
struct _tuple13 _stmttmp83=Cyc_Evexp_eval_const_uint_exp(e);int _tmp487;unsigned _tmp486;_LL1: _tmp486=_stmttmp83.f1;_tmp487=_stmttmp83.f2;_LL2: {unsigned j=_tmp486;int known=_tmp487;
if(known && j <= i){
if(do_warn)
({struct Cyc_Int_pa_PrintArg_struct _tmp48A=({struct Cyc_Int_pa_PrintArg_struct _tmp525;_tmp525.tag=1U,_tmp525.f1=(unsigned long)((int)j);_tmp525;});struct Cyc_Int_pa_PrintArg_struct _tmp48B=({struct Cyc_Int_pa_PrintArg_struct _tmp524;_tmp524.tag=1U,_tmp524.f1=(unsigned long)((int)i);_tmp524;});void*_tmp488[2U];_tmp488[0]=& _tmp48A,_tmp488[1]=& _tmp48B;({unsigned _tmp6EF=loc;struct _fat_ptr _tmp6EE=({const char*_tmp489="a dereference will be out of bounds: %d <= %d";_tag_fat(_tmp489,sizeof(char),46U);});Cyc_Tcutil_warn(_tmp6EF,_tmp6EE,_tag_fat(_tmp488,sizeof(void*),2U));});});else{
# 3284
({struct Cyc_Int_pa_PrintArg_struct _tmp48E=({struct Cyc_Int_pa_PrintArg_struct _tmp527;_tmp527.tag=1U,_tmp527.f1=(unsigned long)((int)j);_tmp527;});struct Cyc_Int_pa_PrintArg_struct _tmp48F=({struct Cyc_Int_pa_PrintArg_struct _tmp526;_tmp526.tag=1U,_tmp526.f1=(unsigned long)((int)i);_tmp526;});void*_tmp48C[2U];_tmp48C[0]=& _tmp48E,_tmp48C[1]=& _tmp48F;({unsigned _tmp6F1=loc;struct _fat_ptr _tmp6F0=({const char*_tmp48D="dereference is out of bounds: %d <= %d";_tag_fat(_tmp48D,sizeof(char),39U);});Cyc_Tcutil_terr(_tmp6F1,_tmp6F0,_tag_fat(_tmp48C,sizeof(void*),2U));});});}}
# 3280
return;}}}
# 2512 "tcutil.cyc"
void Cyc_Tcutil_check_nonzero_bound(unsigned loc,void*b){
# 3289 "tcutil.cyc"
Cyc_Tcutil_check_bound(loc,0U,b,0);}
# 2512 "tcutil.cyc"
static int Cyc_Tcutil_cnst_exp(int var_okay,struct Cyc_Absyn_Exp*e){
# 3298 "tcutil.cyc"
void*_stmttmp84=e->r;void*_tmp490=_stmttmp84;struct Cyc_List_List*_tmp491;struct Cyc_List_List*_tmp492;struct Cyc_List_List*_tmp494;enum Cyc_Absyn_Primop _tmp493;struct Cyc_List_List*_tmp495;struct Cyc_List_List*_tmp496;struct Cyc_List_List*_tmp497;struct Cyc_Absyn_Exp*_tmp498;struct Cyc_Absyn_Exp*_tmp49A;struct Cyc_Absyn_Exp*_tmp499;struct Cyc_Absyn_Exp*_tmp49B;struct Cyc_Absyn_Exp*_tmp49D;void*_tmp49C;struct Cyc_Absyn_Exp*_tmp49F;void*_tmp49E;struct Cyc_Absyn_Exp*_tmp4A0;struct Cyc_Absyn_Exp*_tmp4A1;struct Cyc_Absyn_Exp*_tmp4A2;struct Cyc_Absyn_Exp*_tmp4A4;struct Cyc_Absyn_Exp*_tmp4A3;struct Cyc_Absyn_Exp*_tmp4A7;struct Cyc_Absyn_Exp*_tmp4A6;struct Cyc_Absyn_Exp*_tmp4A5;void*_tmp4A8;switch(*((int*)_tmp490)){case 0U: _LL1: _LL2:
 goto _LL4;case 2U: _LL3: _LL4:
 goto _LL6;case 17U: _LL5: _LL6:
 goto _LL8;case 18U: _LL7: _LL8:
 goto _LLA;case 19U: _LL9: _LLA:
 goto _LLC;case 32U: _LLB: _LLC:
 goto _LLE;case 33U: _LLD: _LLE:
 return 1;case 1U: _LLF: _tmp4A8=(void*)((struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct*)_tmp490)->f1;_LL10: {void*b=_tmp4A8;
# 3309
void*_tmp4A9=b;struct Cyc_Absyn_Vardecl*_tmp4AA;struct Cyc_Absyn_Vardecl*_tmp4AB;switch(*((int*)_tmp4A9)){case 2U: _LL34: _LL35:
 return 1;case 1U: _LL36: _tmp4AB=((struct Cyc_Absyn_Global_b_Absyn_Binding_struct*)_tmp4A9)->f1;_LL37: {struct Cyc_Absyn_Vardecl*vd=_tmp4AB;
# 3312
void*_stmttmp85=Cyc_Tcutil_compress(vd->type);void*_tmp4AC=_stmttmp85;switch(*((int*)_tmp4AC)){case 4U: _LL3F: _LL40:
 goto _LL42;case 5U: _LL41: _LL42:
 return 1;default: _LL43: _LL44:
 return var_okay;}_LL3E:;}case 4U: _LL38: _tmp4AA=((struct Cyc_Absyn_Local_b_Absyn_Binding_struct*)_tmp4A9)->f1;_LL39: {struct Cyc_Absyn_Vardecl*vd=_tmp4AA;
# 3319
if((int)vd->sc == (int)Cyc_Absyn_Static){
void*_stmttmp86=Cyc_Tcutil_compress(vd->type);void*_tmp4AD=_stmttmp86;switch(*((int*)_tmp4AD)){case 4U: _LL46: _LL47:
 goto _LL49;case 5U: _LL48: _LL49:
 return 1;default: _LL4A: _LL4B:
 return var_okay;}_LL45:;}else{
# 3326
return var_okay;}}case 0U: _LL3A: _LL3B:
 return 0;default: _LL3C: _LL3D:
 return var_okay;}_LL33:;}case 6U: _LL11: _tmp4A5=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmp490)->f1;_tmp4A6=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmp490)->f2;_tmp4A7=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmp490)->f3;_LL12: {struct Cyc_Absyn_Exp*e1=_tmp4A5;struct Cyc_Absyn_Exp*e2=_tmp4A6;struct Cyc_Absyn_Exp*e3=_tmp4A7;
# 3332
return(Cyc_Tcutil_cnst_exp(0,e1)&&
 Cyc_Tcutil_cnst_exp(0,e2))&&
 Cyc_Tcutil_cnst_exp(0,e3);}case 9U: _LL13: _tmp4A3=((struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct*)_tmp490)->f1;_tmp4A4=((struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct*)_tmp490)->f2;_LL14: {struct Cyc_Absyn_Exp*e1=_tmp4A3;struct Cyc_Absyn_Exp*e2=_tmp4A4;
# 3336
return Cyc_Tcutil_cnst_exp(0,e1)&& Cyc_Tcutil_cnst_exp(0,e2);}case 42U: _LL15: _tmp4A2=((struct Cyc_Absyn_Extension_e_Absyn_Raw_exp_struct*)_tmp490)->f1;_LL16: {struct Cyc_Absyn_Exp*e2=_tmp4A2;
_tmp4A1=e2;goto _LL18;}case 12U: _LL17: _tmp4A1=((struct Cyc_Absyn_NoInstantiate_e_Absyn_Raw_exp_struct*)_tmp490)->f1;_LL18: {struct Cyc_Absyn_Exp*e2=_tmp4A1;
_tmp4A0=e2;goto _LL1A;}case 13U: _LL19: _tmp4A0=((struct Cyc_Absyn_Instantiate_e_Absyn_Raw_exp_struct*)_tmp490)->f1;_LL1A: {struct Cyc_Absyn_Exp*e2=_tmp4A0;
# 3340
return Cyc_Tcutil_cnst_exp(var_okay,e2);}case 14U: if(((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp490)->f4 == Cyc_Absyn_No_coercion){_LL1B: _tmp49E=(void*)((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp490)->f1;_tmp49F=((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp490)->f2;_LL1C: {void*t=_tmp49E;struct Cyc_Absyn_Exp*e2=_tmp49F;
# 3342
return Cyc_Tcutil_cnst_exp(var_okay,e2);}}else{_LL1D: _tmp49C=(void*)((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp490)->f1;_tmp49D=((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp490)->f2;_LL1E: {void*t=_tmp49C;struct Cyc_Absyn_Exp*e2=_tmp49D;
# 3345
return Cyc_Tcutil_cnst_exp(var_okay,e2);}}case 15U: _LL1F: _tmp49B=((struct Cyc_Absyn_Address_e_Absyn_Raw_exp_struct*)_tmp490)->f1;_LL20: {struct Cyc_Absyn_Exp*e2=_tmp49B;
# 3347
return Cyc_Tcutil_cnst_exp(1,e2);}case 27U: _LL21: _tmp499=((struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct*)_tmp490)->f2;_tmp49A=((struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct*)_tmp490)->f3;_LL22: {struct Cyc_Absyn_Exp*e1=_tmp499;struct Cyc_Absyn_Exp*e2=_tmp49A;
# 3349
return Cyc_Tcutil_cnst_exp(0,e1)&& Cyc_Tcutil_cnst_exp(0,e2);}case 28U: _LL23: _tmp498=((struct Cyc_Absyn_ComprehensionNoinit_e_Absyn_Raw_exp_struct*)_tmp490)->f1;_LL24: {struct Cyc_Absyn_Exp*e=_tmp498;
# 3351
return Cyc_Tcutil_cnst_exp(0,e);}case 26U: _LL25: _tmp497=((struct Cyc_Absyn_Array_e_Absyn_Raw_exp_struct*)_tmp490)->f1;_LL26: {struct Cyc_List_List*des=_tmp497;
_tmp496=des;goto _LL28;}case 30U: _LL27: _tmp496=((struct Cyc_Absyn_AnonStruct_e_Absyn_Raw_exp_struct*)_tmp490)->f2;_LL28: {struct Cyc_List_List*des=_tmp496;
_tmp495=des;goto _LL2A;}case 29U: _LL29: _tmp495=((struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct*)_tmp490)->f3;_LL2A: {struct Cyc_List_List*des=_tmp495;
# 3355
for(0;des != 0;des=des->tl){
if(!Cyc_Tcutil_cnst_exp(0,(*((struct _tuple16*)des->hd)).f2))
return 0;}
# 3355
return 1;}case 3U: _LL2B: _tmp493=((struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct*)_tmp490)->f1;_tmp494=((struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct*)_tmp490)->f2;_LL2C: {enum Cyc_Absyn_Primop p=_tmp493;struct Cyc_List_List*es=_tmp494;
# 3360
_tmp492=es;goto _LL2E;}case 24U: _LL2D: _tmp492=((struct Cyc_Absyn_Tuple_e_Absyn_Raw_exp_struct*)_tmp490)->f1;_LL2E: {struct Cyc_List_List*es=_tmp492;
_tmp491=es;goto _LL30;}case 31U: _LL2F: _tmp491=((struct Cyc_Absyn_Datatype_e_Absyn_Raw_exp_struct*)_tmp490)->f1;_LL30: {struct Cyc_List_List*es=_tmp491;
# 3363
for(0;es != 0;es=es->tl){
if(!Cyc_Tcutil_cnst_exp(0,(struct Cyc_Absyn_Exp*)es->hd))
return 0;}
# 3363
return 1;}default: _LL31: _LL32:
# 3367
 return 0;}_LL0:;}
# 2512 "tcutil.cyc"
int Cyc_Tcutil_is_const_exp(struct Cyc_Absyn_Exp*e){
# 3371 "tcutil.cyc"
return Cyc_Tcutil_cnst_exp(0,e);}
# 2512 "tcutil.cyc"
static int Cyc_Tcutil_fields_zeroable(struct Cyc_List_List*,struct Cyc_List_List*,struct Cyc_List_List*);
# 3375 "tcutil.cyc"
int Cyc_Tcutil_zeroable_type(void*t){
void*_stmttmp87=Cyc_Tcutil_compress(t);void*_tmp4AE=_stmttmp87;struct Cyc_List_List*_tmp4AF;struct Cyc_List_List*_tmp4B0;void*_tmp4B1;void*_tmp4B2;struct Cyc_List_List*_tmp4B4;void*_tmp4B3;switch(*((int*)_tmp4AE)){case 0U: _LL1: _tmp4B3=(void*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp4AE)->f1;_tmp4B4=((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp4AE)->f2;_LL2: {void*c=_tmp4B3;struct Cyc_List_List*ts=_tmp4B4;
# 3378
void*_tmp4B5=c;union Cyc_Absyn_AggrInfo _tmp4B6;struct Cyc_List_List*_tmp4B7;struct Cyc_Absyn_Enumdecl*_tmp4B8;switch(*((int*)_tmp4B5)){case 0U: _LLE: _LLF:
 goto _LL11;case 1U: _LL10: _LL11:
 goto _LL13;case 2U: _LL12: _LL13:
 return 1;case 15U: _LL14: _tmp4B8=((struct Cyc_Absyn_EnumCon_Absyn_TyCon_struct*)_tmp4B5)->f2;_LL15: {struct Cyc_Absyn_Enumdecl*edo=_tmp4B8;
# 3383
if(edo == 0 || edo->fields == 0)
return 0;
# 3383
_tmp4B7=(struct Cyc_List_List*)((struct Cyc_Core_Opt*)_check_null(edo->fields))->v;goto _LL17;}case 16U: _LL16: _tmp4B7=((struct Cyc_Absyn_AnonEnumCon_Absyn_TyCon_struct*)_tmp4B5)->f1;_LL17: {struct Cyc_List_List*fs=_tmp4B7;
# 3387
{struct Cyc_List_List*fs2=fs;for(0;fs2 != 0;fs2=fs2->tl){
if(((struct Cyc_Absyn_Enumfield*)fs2->hd)->tag == 0)
return fs2 == fs;{
# 3388
struct _tuple13 _stmttmp88=
# 3390
Cyc_Evexp_eval_const_uint_exp((struct Cyc_Absyn_Exp*)_check_null(((struct Cyc_Absyn_Enumfield*)fs2->hd)->tag));
# 3388
int _tmp4BA;unsigned _tmp4B9;_LL1D: _tmp4B9=_stmttmp88.f1;_tmp4BA=_stmttmp88.f2;_LL1E: {unsigned i=_tmp4B9;int known=_tmp4BA;
# 3391
if(known && i == (unsigned)0)
return 1;}}}}
# 3394
return 0;}case 20U: _LL18: _tmp4B6=((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)_tmp4B5)->f1;_LL19: {union Cyc_Absyn_AggrInfo info=_tmp4B6;
# 3397
struct Cyc_Absyn_Aggrdecl*ad=Cyc_Absyn_get_known_aggrdecl(info);
if(ad->impl == 0)return 0;if(((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->exist_vars != 0)
return 0;
# 3398
if(
# 3400
(int)ad->kind == (int)Cyc_Absyn_UnionA &&((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->tagged)return 0;
# 3398
return
# 3401
 Cyc_Tcutil_fields_zeroable(ad->tvs,ts,((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->fields);}default: _LL1A: _LL1B:
 return 0;}_LLD:;}case 3U: _LL3: _tmp4B2=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp4AE)->f1).ptr_atts).nullable;_LL4: {void*n=_tmp4B2;
# 3405
return Cyc_Tcutil_force_type2bool(1,n);}case 4U: _LL5: _tmp4B1=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp4AE)->f1).elt_type;_LL6: {void*t=_tmp4B1;
return Cyc_Tcutil_zeroable_type(t);}case 6U: _LL7: _tmp4B0=((struct Cyc_Absyn_TupleType_Absyn_Type_struct*)_tmp4AE)->f1;_LL8: {struct Cyc_List_List*tqs=_tmp4B0;
# 3408
for(0;tqs != 0;tqs=tqs->tl){
if(!Cyc_Tcutil_zeroable_type((*((struct _tuple12*)tqs->hd)).f2))return 0;}
# 3408
return 1;}case 7U: _LL9: _tmp4AF=((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmp4AE)->f2;_LLA: {struct Cyc_List_List*fs=_tmp4AF;
# 3411
return Cyc_Tcutil_fields_zeroable(0,0,fs);}default: _LLB: _LLC:
 return 0;}_LL0:;}
# 3375
static int Cyc_Tcutil_fields_zeroable(struct Cyc_List_List*tvs,struct Cyc_List_List*ts,struct Cyc_List_List*fs){
# 3417
struct _RegionHandle _tmp4BB=_new_region("rgn");struct _RegionHandle*rgn=& _tmp4BB;_push_region(rgn);
{struct Cyc_List_List*inst=Cyc_List_rzip(rgn,rgn,tvs,ts);
for(0;fs != 0;fs=fs->tl){
void*t=((struct Cyc_Absyn_Aggrfield*)fs->hd)->type;
if(Cyc_Tcutil_zeroable_type(t))continue;t=
Cyc_Tcutil_rsubstitute(rgn,inst,((struct Cyc_Absyn_Aggrfield*)fs->hd)->type);
if(!Cyc_Tcutil_zeroable_type(t)){int _tmp4BC=0;_npop_handler(0U);return _tmp4BC;}}{
# 3425
int _tmp4BD=1;_npop_handler(0U);return _tmp4BD;}}
# 3418
;_pop_region(rgn);}
# 3375
void Cyc_Tcutil_check_no_qual(unsigned loc,void*t){
# 3430
void*_tmp4BE=t;struct Cyc_Absyn_Typedefdecl*_tmp4BF;if(((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmp4BE)->tag == 8U){_LL1: _tmp4BF=((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmp4BE)->f3;_LL2: {struct Cyc_Absyn_Typedefdecl*tdopt=_tmp4BF;
# 3432
if(tdopt != 0){
struct Cyc_Absyn_Tqual tq=tdopt->tq;
if(((tq.print_const || tq.q_volatile)|| tq.q_restrict)|| tq.real_const)
({struct Cyc_String_pa_PrintArg_struct _tmp4C2=({struct Cyc_String_pa_PrintArg_struct _tmp528;_tmp528.tag=0U,({struct _fat_ptr _tmp6F2=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t));_tmp528.f1=_tmp6F2;});_tmp528;});void*_tmp4C0[1U];_tmp4C0[0]=& _tmp4C2;({unsigned _tmp6F4=loc;struct _fat_ptr _tmp6F3=({const char*_tmp4C1="qualifier within typedef type %s is ignored";_tag_fat(_tmp4C1,sizeof(char),44U);});Cyc_Tcutil_warn(_tmp6F4,_tmp6F3,_tag_fat(_tmp4C0,sizeof(void*),1U));});});}
# 3432
goto _LL0;}}else{_LL3: _LL4:
# 3438
 goto _LL0;}_LL0:;}
# 3375
struct Cyc_List_List*Cyc_Tcutil_transfer_fn_type_atts(void*t,struct Cyc_List_List*atts){
# 3445
void*_stmttmp89=Cyc_Tcutil_compress(t);void*_tmp4C3=_stmttmp89;struct Cyc_List_List**_tmp4C4;if(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp4C3)->tag == 5U){_LL1: _tmp4C4=(struct Cyc_List_List**)&(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp4C3)->f1).attributes;_LL2: {struct Cyc_List_List**fnatts=_tmp4C4;
# 3447
struct Cyc_List_List*res_atts=0;
for(0;atts != 0;atts=atts->tl){
if(Cyc_Absyn_fntype_att((void*)atts->hd)){
if(!Cyc_List_mem(Cyc_Absyn_attribute_cmp,*fnatts,(void*)atts->hd))
({struct Cyc_List_List*_tmp6F5=({struct Cyc_List_List*_tmp4C5=_cycalloc(sizeof(*_tmp4C5));_tmp4C5->hd=(void*)atts->hd,_tmp4C5->tl=*fnatts;_tmp4C5;});*fnatts=_tmp6F5;});}else{
# 3454
res_atts=({struct Cyc_List_List*_tmp4C6=_cycalloc(sizeof(*_tmp4C6));_tmp4C6->hd=(void*)atts->hd,_tmp4C6->tl=res_atts;_tmp4C6;});}}
return res_atts;}}else{_LL3: _LL4:
({void*_tmp4C7=0U;({int(*_tmp6F7)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp4C9)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Tcutil_impos;_tmp4C9;});struct _fat_ptr _tmp6F6=({const char*_tmp4C8="transfer_fn_type_atts";_tag_fat(_tmp4C8,sizeof(char),22U);});_tmp6F7(_tmp6F6,_tag_fat(_tmp4C7,sizeof(void*),0U));});});}_LL0:;}
# 3375
struct Cyc_Absyn_Exp*Cyc_Tcutil_get_type_bound(void*t){
# 3462
void*_stmttmp8A=Cyc_Tcutil_compress(t);void*_tmp4CA=_stmttmp8A;struct Cyc_Absyn_Exp*_tmp4CB;struct Cyc_Absyn_PtrInfo _tmp4CC;switch(*((int*)_tmp4CA)){case 3U: _LL1: _tmp4CC=((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp4CA)->f1;_LL2: {struct Cyc_Absyn_PtrInfo pi=_tmp4CC;
return({void*_tmp6F8=Cyc_Absyn_bounds_one();Cyc_Tcutil_get_bounds_exp(_tmp6F8,(pi.ptr_atts).bounds);});}case 4U: _LL3: _tmp4CB=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp4CA)->f1).num_elts;_LL4: {struct Cyc_Absyn_Exp*e=_tmp4CB;
return e;}default: _LL5: _LL6:
 return 0;}_LL0:;}
# 3375
struct Cyc_Absyn_Vardecl*Cyc_Tcutil_nonesc_vardecl(void*b){
# 3472
void*_tmp4CD=b;struct Cyc_Absyn_Vardecl*_tmp4CE;struct Cyc_Absyn_Vardecl*_tmp4CF;struct Cyc_Absyn_Vardecl*_tmp4D0;struct Cyc_Absyn_Vardecl*_tmp4D1;switch(*((int*)_tmp4CD)){case 5U: _LL1: _tmp4D1=((struct Cyc_Absyn_Pat_b_Absyn_Binding_struct*)_tmp4CD)->f1;_LL2: {struct Cyc_Absyn_Vardecl*x=_tmp4D1;
_tmp4D0=x;goto _LL4;}case 4U: _LL3: _tmp4D0=((struct Cyc_Absyn_Local_b_Absyn_Binding_struct*)_tmp4CD)->f1;_LL4: {struct Cyc_Absyn_Vardecl*x=_tmp4D0;
_tmp4CF=x;goto _LL6;}case 3U: _LL5: _tmp4CF=((struct Cyc_Absyn_Param_b_Absyn_Binding_struct*)_tmp4CD)->f1;_LL6: {struct Cyc_Absyn_Vardecl*x=_tmp4CF;
_tmp4CE=x;goto _LL8;}case 1U: _LL7: _tmp4CE=((struct Cyc_Absyn_Global_b_Absyn_Binding_struct*)_tmp4CD)->f1;_LL8: {struct Cyc_Absyn_Vardecl*x=_tmp4CE;
# 3477
if(!x->escapes)
return x;
# 3477
return 0;}default: _LL9: _LLA:
# 3480
 return 0;}_LL0:;}
# 3375
struct Cyc_List_List*Cyc_Tcutil_filter_nulls(struct Cyc_List_List*l){
# 3486
struct Cyc_List_List*res=0;
{struct Cyc_List_List*x=l;for(0;x != 0;x=x->tl){
if((void**)x->hd != 0)res=({struct Cyc_List_List*_tmp4D2=_cycalloc(sizeof(*_tmp4D2));_tmp4D2->hd=*((void**)_check_null((void**)x->hd)),_tmp4D2->tl=res;_tmp4D2;});}}
# 3487
return res;}
# 3375
void*Cyc_Tcutil_promote_array(void*t,void*rgn,int convert_tag){
# 3493
void*_stmttmp8B=Cyc_Tcutil_compress(t);void*_tmp4D3=_stmttmp8B;unsigned _tmp4D8;void*_tmp4D7;struct Cyc_Absyn_Exp*_tmp4D6;struct Cyc_Absyn_Tqual _tmp4D5;void*_tmp4D4;if(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp4D3)->tag == 4U){_LL1: _tmp4D4=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp4D3)->f1).elt_type;_tmp4D5=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp4D3)->f1).tq;_tmp4D6=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp4D3)->f1).num_elts;_tmp4D7=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp4D3)->f1).zero_term;_tmp4D8=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp4D3)->f1).zt_loc;_LL2: {void*et=_tmp4D4;struct Cyc_Absyn_Tqual tq=_tmp4D5;struct Cyc_Absyn_Exp*eopt=_tmp4D6;void*zt=_tmp4D7;unsigned ztl=_tmp4D8;
# 3495
void*b;
if(eopt == 0)
b=Cyc_Absyn_fat_bound_type;else{
# 3499
if(convert_tag){
if(eopt->topt == 0)
({void*_tmp4D9=0U;({int(*_tmp6FA)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp4DB)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Tcutil_impos;_tmp4DB;});struct _fat_ptr _tmp6F9=({const char*_tmp4DA="cannot convert tag without type!";_tag_fat(_tmp4DA,sizeof(char),33U);});_tmp6FA(_tmp6F9,_tag_fat(_tmp4D9,sizeof(void*),0U));});});{
# 3500
void*_stmttmp8C=
# 3502
Cyc_Tcutil_compress((void*)_check_null(eopt->topt));
# 3500
void*_tmp4DC=_stmttmp8C;void*_tmp4DD;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp4DC)->tag == 0U){if(((struct Cyc_Absyn_TagCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp4DC)->f1)->tag == 4U){if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp4DC)->f2 != 0){_LL6: _tmp4DD=(void*)(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp4DC)->f2)->hd;_LL7: {void*t=_tmp4DD;
# 3505
b=Cyc_Absyn_thin_bounds_exp(Cyc_Absyn_valueof_exp(t,0U));
goto _LL5;}}else{goto _LL8;}}else{goto _LL8;}}else{_LL8: _LL9:
# 3508
 b=Cyc_Tcutil_is_const_exp(eopt)?Cyc_Absyn_thin_bounds_exp(eopt): Cyc_Absyn_fat_bound_type;}_LL5:;}}else{
# 3512
b=Cyc_Absyn_thin_bounds_exp(eopt);}}
# 3514
return Cyc_Absyn_atb_type(et,rgn,tq,b,zt);}}else{_LL3: _LL4:
 return t;}_LL0:;}
