#include <setjmp.h>
/* This is a C header file to be used by the output of the Cyclone to
   C translator.  The corresponding definitions are in file lib/runtime_*.c */
#ifndef _CYC_INCLUDE_H_
#define _CYC_INCLUDE_H_

/* Need one of these per thread (see runtime_stack.c). The runtime maintains 
   a stack that contains either _handler_cons structs or _RegionHandle structs.
   The tag is 0 for a handler_cons and 1 for a region handle.  */
struct _RuntimeStack {
  int tag; 
  struct _RuntimeStack *next;
  void (*cleanup)(struct _RuntimeStack *frame);
};

#ifndef offsetof
/* should be size_t, but int is fine. */
#define offsetof(t,n) ((int)(&(((t *)0)->n)))
#endif

/* Fat pointers */
struct _fat_ptr {
  unsigned char *curr; 
  unsigned char *base; 
  unsigned char *last_plus_one; 
};  

/* Discriminated Unions */
struct _xtunion_struct { char *tag; };

/* Regions */
struct _RegionPage
#ifdef CYC_REGION_PROFILE
{ unsigned total_bytes;
  unsigned free_bytes;
  /* MWH: wish we didn't have to include the stuff below ... */
  struct _RegionPage *next;
  char data[1];
}
#endif
; // abstract -- defined in runtime_memory.c
struct _RegionHandle {
  struct _RuntimeStack s;
  struct _RegionPage *curr;
  char               *offset;
  char               *last_plus_one;
  struct _DynRegionHandle *sub_regions;
#ifdef CYC_REGION_PROFILE
  const char         *name;
#else
  unsigned used_bytes;
  unsigned wasted_bytes;
#endif
};
struct _DynRegionFrame {
  struct _RuntimeStack s;
  struct _DynRegionHandle *x;
};
// A dynamic region is just a region handle.  The wrapper struct is for type
// abstraction.
struct Cyc_Core_DynamicRegion {
  struct _RegionHandle h;
};

struct _RegionHandle _new_region(const char*);
void* _region_malloc(struct _RegionHandle*, unsigned);
void* _region_calloc(struct _RegionHandle*, unsigned t, unsigned n);
void   _free_region(struct _RegionHandle*);
struct _RegionHandle*_open_dynregion(struct _DynRegionFrame*,struct _DynRegionHandle*);
void   _pop_dynregion();

/* Exceptions */
struct _handler_cons {
  struct _RuntimeStack s;
  jmp_buf handler;
};
void _push_handler(struct _handler_cons *);
void _push_region(struct _RegionHandle *);
void _npop_handler(int);
void _pop_handler();
void _pop_region();

#ifndef _throw
void* _throw_null_fn(const char*,unsigned);
void* _throw_arraybounds_fn(const char*,unsigned);
void* _throw_badalloc_fn(const char*,unsigned);
void* _throw_match_fn(const char*,unsigned);
void* _throw_fn(void*,const char*,unsigned);
void* _rethrow(void*);
#define _throw_null() (_throw_null_fn(__FILE__,__LINE__))
#define _throw_arraybounds() (_throw_arraybounds_fn(__FILE__,__LINE__))
#define _throw_badalloc() (_throw_badalloc_fn(__FILE__,__LINE__))
#define _throw_match() (_throw_match_fn(__FILE__,__LINE__))
#define _throw(e) (_throw_fn((e),__FILE__,__LINE__))
#endif

struct _xtunion_struct* Cyc_Core_get_exn_thrown();
/* Built-in Exceptions */
struct Cyc_Null_Exception_exn_struct { char *tag; };
struct Cyc_Array_bounds_exn_struct { char *tag; };
struct Cyc_Match_Exception_exn_struct { char *tag; };
struct Cyc_Bad_alloc_exn_struct { char *tag; };
extern char Cyc_Null_Exception[];
extern char Cyc_Array_bounds[];
extern char Cyc_Match_Exception[];
extern char Cyc_Bad_alloc[];

/* Built-in Run-time Checks and company */
#ifdef CYC_ANSI_OUTPUT
#define _INLINE  
#else
#define _INLINE inline
#endif

#ifdef NO_CYC_NULL_CHECKS
#define _check_null(ptr) (ptr)
#else
#define _check_null(ptr) \
  ({ void*_cks_null = (void*)(ptr); \
     if (!_cks_null) _throw_null(); \
     _cks_null; })
#endif

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index)\
   (((char*)ptr) + (elt_sz)*(index))
#ifdef NO_CYC_NULL_CHECKS
#define _check_known_subscript_null _check_known_subscript_notnull
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr);\
  int _index = (index);\
  if (!_cks_ptr) _throw_null(); \
  _cks_ptr + (elt_sz)*_index; })
#endif
#define _zero_arr_plus_fn(orig_x,orig_sz,orig_i,f,l) ((orig_x)+(orig_i))
#define _zero_arr_plus_char_fn _zero_arr_plus_fn
#define _zero_arr_plus_short_fn _zero_arr_plus_fn
#define _zero_arr_plus_int_fn _zero_arr_plus_fn
#define _zero_arr_plus_float_fn _zero_arr_plus_fn
#define _zero_arr_plus_double_fn _zero_arr_plus_fn
#define _zero_arr_plus_longdouble_fn _zero_arr_plus_fn
#define _zero_arr_plus_voidstar_fn _zero_arr_plus_fn
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (!_cks_ptr) _throw_null(); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })
#define _check_known_subscript_notnull(ptr,bound,elt_sz,index) ({ \
  char*_cks_ptr = (char*)(ptr); \
  unsigned _cks_index = (index); \
  if (_cks_index >= (bound)) _throw_arraybounds(); \
  _cks_ptr + (elt_sz)*_cks_index; })

/* _zero_arr_plus_*_fn(x,sz,i,filename,lineno) adds i to zero-terminated ptr
   x that has at least sz elements */
char* _zero_arr_plus_char_fn(char*,unsigned,int,const char*,unsigned);
short* _zero_arr_plus_short_fn(short*,unsigned,int,const char*,unsigned);
int* _zero_arr_plus_int_fn(int*,unsigned,int,const char*,unsigned);
float* _zero_arr_plus_float_fn(float*,unsigned,int,const char*,unsigned);
double* _zero_arr_plus_double_fn(double*,unsigned,int,const char*,unsigned);
long double* _zero_arr_plus_longdouble_fn(long double*,unsigned,int,const char*, unsigned);
void** _zero_arr_plus_voidstar_fn(void**,unsigned,int,const char*,unsigned);
#endif

/* _get_zero_arr_size_*(x,sz) returns the number of elements in a
   zero-terminated array that is NULL or has at least sz elements */
int _get_zero_arr_size_char(const char*,unsigned);
int _get_zero_arr_size_short(const short*,unsigned);
int _get_zero_arr_size_int(const int*,unsigned);
int _get_zero_arr_size_float(const float*,unsigned);
int _get_zero_arr_size_double(const double*,unsigned);
int _get_zero_arr_size_longdouble(const long double*,unsigned);
int _get_zero_arr_size_voidstar(const void**,unsigned);

/* _zero_arr_inplace_plus_*_fn(x,i,filename,lineno) sets
   zero-terminated pointer *x to *x + i */
char* _zero_arr_inplace_plus_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_short_fn(short**,int,const char*,unsigned);
int* _zero_arr_inplace_plus_int(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_longdouble_fn(long double**,int,const char*,unsigned);
void** _zero_arr_inplace_plus_voidstar_fn(void***,int,const char*,unsigned);
/* like the previous functions, but does post-addition (as in e++) */
char* _zero_arr_inplace_plus_post_char_fn(char**,int,const char*,unsigned);
short* _zero_arr_inplace_plus_post_short_fn(short**x,int,const char*,unsigned);
int* _zero_arr_inplace_plus_post_int_fn(int**,int,const char*,unsigned);
float* _zero_arr_inplace_plus_post_float_fn(float**,int,const char*,unsigned);
double* _zero_arr_inplace_plus_post_double_fn(double**,int,const char*,unsigned);
long double* _zero_arr_inplace_plus_post_longdouble_fn(long double**,int,const char *,unsigned);
void** _zero_arr_inplace_plus_post_voidstar_fn(void***,int,const char*,unsigned);
#define _zero_arr_plus_char(x,s,i) \
  (_zero_arr_plus_char_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_short(x,s,i) \
  (_zero_arr_plus_short_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_int(x,s,i) \
  (_zero_arr_plus_int_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_float(x,s,i) \
  (_zero_arr_plus_float_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_double(x,s,i) \
  (_zero_arr_plus_double_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_longdouble(x,s,i) \
  (_zero_arr_plus_longdouble_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_plus_voidstar(x,s,i) \
  (_zero_arr_plus_voidstar_fn(x,s,i,__FILE__,__LINE__))
#define _zero_arr_inplace_plus_char(x,i) \
  _zero_arr_inplace_plus_char_fn((char **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_short(x,i) \
  _zero_arr_inplace_plus_short_fn((short **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_int(x,i) \
  _zero_arr_inplace_plus_int_fn((int **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_float(x,i) \
  _zero_arr_inplace_plus_float_fn((float **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_double(x,i) \
  _zero_arr_inplace_plus_double_fn((double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_longdouble(x,i) \
  _zero_arr_inplace_plus_longdouble_fn((long double **)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_voidstar(x,i) \
  _zero_arr_inplace_plus_voidstar_fn((void ***)(x),i,__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_char(x,i) \
  _zero_arr_inplace_plus_post_char_fn((char **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_short(x,i) \
  _zero_arr_inplace_plus_post_short_fn((short **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_int(x,i) \
  _zero_arr_inplace_plus_post_int_fn((int **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_float(x,i) \
  _zero_arr_inplace_plus_post_float_fn((float **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_double(x,i) \
  _zero_arr_inplace_plus_post_double_fn((double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_longdouble(x,i) \
  _zero_arr_inplace_plus_post_longdouble_fn((long double **)(x),(i),__FILE__,__LINE__)
#define _zero_arr_inplace_plus_post_voidstar(x,i) \
  _zero_arr_inplace_plus_post_voidstar_fn((void***)(x),(i),__FILE__,__LINE__)

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_fat_subscript(arr,elt_sz,index) ((arr).curr + (elt_sz) * (index))
#define _untag_fat_ptr(arr,elt_sz,num_elts) ((arr).curr)
#else
#define _check_fat_subscript(arr,elt_sz,index) ({ \
  struct _fat_ptr _cus_arr = (arr); \
  unsigned char *_cus_ans = _cus_arr.curr + (elt_sz) * (index); \
  /* JGM: not needed! if (!_cus_arr.base) _throw_null();*/ \
  if (_cus_ans < _cus_arr.base || _cus_ans >= _cus_arr.last_plus_one) \
    _throw_arraybounds(); \
  _cus_ans; })
#define _untag_fat_ptr(arr,elt_sz,num_elts) ({ \
  struct _fat_ptr _arr = (arr); \
  unsigned char *_curr = _arr.curr; \
  if ((_curr < _arr.base || _curr + (elt_sz) * (num_elts) > _arr.last_plus_one) &&\
      _curr != (unsigned char *)0) \
    _throw_arraybounds(); \
  _curr; })
#endif

#define _tag_fat(tcurr,elt_sz,num_elts) ({ \
  struct _fat_ptr _ans; \
  unsigned _num_elts = (num_elts);\
  _ans.base = _ans.curr = (void*)(tcurr); \
  /* JGM: if we're tagging NULL, ignore num_elts */ \
  _ans.last_plus_one = _ans.base ? (_ans.base + (elt_sz) * _num_elts) : 0; \
  _ans; })

#define _get_fat_size(arr,elt_sz) \
  ({struct _fat_ptr _arr = (arr); \
    unsigned char *_arr_curr=_arr.curr; \
    unsigned char *_arr_last=_arr.last_plus_one; \
    (_arr_curr < _arr.base || _arr_curr >= _arr_last) ? 0 : \
    ((_arr_last - _arr_curr) / (elt_sz));})

#define _fat_ptr_plus(arr,elt_sz,change) ({ \
  struct _fat_ptr _ans = (arr); \
  int _change = (change);\
  _ans.curr += (elt_sz) * _change;\
  _ans; })
#define _fat_ptr_inplace_plus(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  _arr_ptr->curr += (elt_sz) * (change);\
  *_arr_ptr; })
#define _fat_ptr_inplace_plus_post(arr_ptr,elt_sz,change) ({ \
  struct _fat_ptr * _arr_ptr = (arr_ptr); \
  struct _fat_ptr _ans = *_arr_ptr; \
  _arr_ptr->curr += (elt_sz) * (change);\
  _ans; })

//Not a macro since initialization order matters. Defined in runtime_zeroterm.c.
struct _fat_ptr _fat_ptr_decrease_size(struct _fat_ptr,unsigned sz,unsigned numelts);

/* Allocation */
void* GC_malloc(int);
void* GC_malloc_atomic(int);
void* GC_calloc(unsigned,unsigned);
void* GC_calloc_atomic(unsigned,unsigned);
// bound the allocation size to be < MAX_ALLOC_SIZE. See macros below for usage.
#define MAX_MALLOC_SIZE (1 << 28)
void* _bounded_GC_malloc(int,const char*,int);
void* _bounded_GC_malloc_atomic(int,const char*,int);
void* _bounded_GC_calloc(unsigned,unsigned,const char*,int);
void* _bounded_GC_calloc_atomic(unsigned,unsigned,const char*,int);
/* these macros are overridden below ifdef CYC_REGION_PROFILE */
#ifndef CYC_REGION_PROFILE
#define _cycalloc(n) _bounded_GC_malloc(n,__FILE__,__LINE__)
#define _cycalloc_atomic(n) _bounded_GC_malloc_atomic(n,__FILE__,__LINE__)
#define _cyccalloc(n,s) _bounded_GC_calloc(n,s,__FILE__,__LINE__)
#define _cyccalloc_atomic(n,s) _bounded_GC_calloc_atomic(n,s,__FILE__,__LINE__)
#endif

static _INLINE unsigned int _check_times(unsigned x, unsigned y) {
  unsigned long long whole_ans = 
    ((unsigned long long) x)*((unsigned long long)y);
  unsigned word_ans = (unsigned)whole_ans;
  if(word_ans < whole_ans || word_ans > MAX_MALLOC_SIZE)
    _throw_badalloc();
  return word_ans;
}

#define _CYC_MAX_REGION_CONST 2
#define _CYC_MIN_ALIGNMENT (sizeof(double))

#ifdef CYC_REGION_PROFILE
extern int rgn_total_bytes;
#endif

static _INLINE void *_fast_region_malloc(struct _RegionHandle *r, unsigned orig_s) {  
  if (r > (struct _RegionHandle *)_CYC_MAX_REGION_CONST && r->curr != 0) { 
#ifdef CYC_NOALIGN
    unsigned s =  orig_s;
#else
    unsigned s =  (orig_s + _CYC_MIN_ALIGNMENT - 1) & (~(_CYC_MIN_ALIGNMENT -1)); 
#endif
    char *result; 
    result = r->offset; 
    if (s <= (r->last_plus_one - result)) {
      r->offset = result + s; 
#ifdef CYC_REGION_PROFILE
    r->curr->free_bytes = r->curr->free_bytes - s;
    rgn_total_bytes += s;
#endif
      return result;
    }
  } 
  return _region_malloc(r,orig_s); 
}

#ifdef CYC_REGION_PROFILE
/* see macros below for usage. defined in runtime_memory.c */
void* _profile_GC_malloc(int,const char*,const char*,int);
void* _profile_GC_malloc_atomic(int,const char*,const char*,int);
void* _profile_GC_calloc(unsigned,unsigned,const char*,const char*,int);
void* _profile_GC_calloc_atomic(unsigned,unsigned,const char*,const char*,int);
void* _profile_region_malloc(struct _RegionHandle*,unsigned,const char*,const char*,int);
void* _profile_region_calloc(struct _RegionHandle*,unsigned,unsigned,const char *,const char*,int);
struct _RegionHandle _profile_new_region(const char*,const char*,const char*,int);
void _profile_free_region(struct _RegionHandle*,const char*,const char*,int);
#ifndef RUNTIME_CYC
#define _new_region(n) _profile_new_region(n,__FILE__,__FUNCTION__,__LINE__)
#define _free_region(r) _profile_free_region(r,__FILE__,__FUNCTION__,__LINE__)
#define _region_malloc(rh,n) _profile_region_malloc(rh,n,__FILE__,__FUNCTION__,__LINE__)
#define _region_calloc(rh,n,t) _profile_region_calloc(rh,n,t,__FILE__,__FUNCTION__,__LINE__)
#  endif
#define _cycalloc(n) _profile_GC_malloc(n,__FILE__,__FUNCTION__,__LINE__)
#define _cycalloc_atomic(n) _profile_GC_malloc_atomic(n,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc(n,s) _profile_GC_calloc(n,s,__FILE__,__FUNCTION__,__LINE__)
#define _cyccalloc_atomic(n,s) _profile_GC_calloc_atomic(n,s,__FILE__,__FUNCTION__,__LINE__)
#endif
#endif
 struct Cyc_Core_Opt{void*v;};extern char Cyc_Core_Invalid_argument[17U];struct Cyc_Core_Invalid_argument_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Failure[8U];struct Cyc_Core_Failure_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Impossible[11U];struct Cyc_Core_Impossible_exn_struct{char*tag;struct _fat_ptr f1;};extern char Cyc_Core_Not_found[10U];struct Cyc_Core_Not_found_exn_struct{char*tag;};extern char Cyc_Core_Unreachable[12U];struct Cyc_Core_Unreachable_exn_struct{char*tag;struct _fat_ptr f1;};
# 165 "core.h"
extern struct _RegionHandle*Cyc_Core_heap_region;
# 168
extern struct _RegionHandle*Cyc_Core_unique_region;
# 175
void Cyc_Core_ufree(void*ptr);
# 188
struct _fat_ptr Cyc_Core_alias_refptr(struct _fat_ptr ptr);struct Cyc_Core_DynamicRegion;struct Cyc_Core_NewDynamicRegion{struct Cyc_Core_DynamicRegion*key;};
# 237
struct Cyc_Core_NewDynamicRegion Cyc_Core__new_rckey(const char*file,const char*func,int lineno);
# 255 "core.h"
void Cyc_Core_free_rckey(struct Cyc_Core_DynamicRegion*k);struct Cyc_Core_ThinRes{void*arr;unsigned nelts;};struct Cyc_List_List{void*hd;struct Cyc_List_List*tl;};
# 54 "list.h"
struct Cyc_List_List*Cyc_List_list(struct _fat_ptr);
# 57
struct Cyc_List_List*Cyc_List_rlist(struct _RegionHandle*,struct _fat_ptr);
# 61
int Cyc_List_length(struct Cyc_List_List*x);
# 76
struct Cyc_List_List*Cyc_List_map(void*(*f)(void*),struct Cyc_List_List*x);
# 79
struct Cyc_List_List*Cyc_List_rmap(struct _RegionHandle*,void*(*f)(void*),struct Cyc_List_List*x);
# 83
struct Cyc_List_List*Cyc_List_map_c(void*(*f)(void*,void*),void*env,struct Cyc_List_List*x);
# 86
struct Cyc_List_List*Cyc_List_rmap_c(struct _RegionHandle*,void*(*f)(void*,void*),void*env,struct Cyc_List_List*x);extern char Cyc_List_List_mismatch[14U];struct Cyc_List_List_mismatch_exn_struct{char*tag;};
# 133
void Cyc_List_iter(void(*f)(void*),struct Cyc_List_List*x);
# 135
void Cyc_List_iter_c(void(*f)(void*,void*),void*env,struct Cyc_List_List*x);
# 172
struct Cyc_List_List*Cyc_List_rev(struct Cyc_List_List*x);
# 178
struct Cyc_List_List*Cyc_List_imp_rev(struct Cyc_List_List*x);
# 184
struct Cyc_List_List*Cyc_List_append(struct Cyc_List_List*x,struct Cyc_List_List*y);
# 195
struct Cyc_List_List*Cyc_List_imp_append(struct Cyc_List_List*x,struct Cyc_List_List*y);extern char Cyc_List_Nth[4U];struct Cyc_List_Nth_exn_struct{char*tag;};
# 242
void*Cyc_List_nth(struct Cyc_List_List*x,int n);
# 251
int Cyc_List_forall(int(*pred)(void*),struct Cyc_List_List*x);
# 270
struct Cyc_List_List*Cyc_List_zip(struct Cyc_List_List*x,struct Cyc_List_List*y);
# 276
struct Cyc_List_List*Cyc_List_rzip(struct _RegionHandle*r1,struct _RegionHandle*r2,struct Cyc_List_List*x,struct Cyc_List_List*y);struct _tuple0{struct Cyc_List_List*f1;struct Cyc_List_List*f2;};
# 294
struct _tuple0 Cyc_List_split(struct Cyc_List_List*x);
# 328
void*Cyc_List_assoc(struct Cyc_List_List*l,void*k);
# 371
struct Cyc_List_List*Cyc_List_from_array(struct _fat_ptr arr);
# 383
int Cyc_List_list_cmp(int(*cmp)(void*,void*),struct Cyc_List_List*l1,struct Cyc_List_List*l2);struct Cyc___cycFILE;struct Cyc_String_pa_PrintArg_struct{int tag;struct _fat_ptr f1;};struct Cyc_Int_pa_PrintArg_struct{int tag;unsigned long f1;};struct Cyc_Double_pa_PrintArg_struct{int tag;double f1;};struct Cyc_LongDouble_pa_PrintArg_struct{int tag;long double f1;};struct Cyc_ShortPtr_pa_PrintArg_struct{int tag;short*f1;};struct Cyc_IntPtr_pa_PrintArg_struct{int tag;unsigned long*f1;};
# 73 "cycboot.h"
struct _fat_ptr Cyc_aprintf(struct _fat_ptr,struct _fat_ptr);struct Cyc_ShortPtr_sa_ScanfArg_struct{int tag;short*f1;};struct Cyc_UShortPtr_sa_ScanfArg_struct{int tag;unsigned short*f1;};struct Cyc_IntPtr_sa_ScanfArg_struct{int tag;int*f1;};struct Cyc_UIntPtr_sa_ScanfArg_struct{int tag;unsigned*f1;};struct Cyc_StringPtr_sa_ScanfArg_struct{int tag;struct _fat_ptr f1;};struct Cyc_DoublePtr_sa_ScanfArg_struct{int tag;double*f1;};struct Cyc_FloatPtr_sa_ScanfArg_struct{int tag;float*f1;};struct Cyc_CharPtr_sa_ScanfArg_struct{int tag;struct _fat_ptr f1;};extern char Cyc_FileCloseError[15U];struct Cyc_FileCloseError_exn_struct{char*tag;};extern char Cyc_FileOpenError[14U];struct Cyc_FileOpenError_exn_struct{char*tag;struct _fat_ptr f1;};
# 38 "string.h"
unsigned long Cyc_strlen(struct _fat_ptr s);
# 49 "string.h"
int Cyc_strcmp(struct _fat_ptr s1,struct _fat_ptr s2);
int Cyc_strptrcmp(struct _fat_ptr*s1,struct _fat_ptr*s2);
# 62
struct _fat_ptr Cyc_strconcat(struct _fat_ptr,struct _fat_ptr);
# 67
struct _fat_ptr Cyc_rstr_sepstr(struct _RegionHandle*,struct Cyc_List_List*,struct _fat_ptr);struct Cyc_Iter_Iter{void*env;int(*next)(void*env,void*dest);};struct Cyc_Set_Set;
# 54 "set.h"
struct Cyc_Set_Set*Cyc_Set_rempty(struct _RegionHandle*r,int(*cmp)(void*,void*));
# 69
struct Cyc_Set_Set*Cyc_Set_rinsert(struct _RegionHandle*r,struct Cyc_Set_Set*s,void*elt);
# 100
int Cyc_Set_member(struct Cyc_Set_Set*s,void*elt);extern char Cyc_Set_Absent[7U];struct Cyc_Set_Absent_exn_struct{char*tag;};struct Cyc_Dict_T;struct Cyc_Dict_Dict{int(*rel)(void*,void*);struct _RegionHandle*r;const struct Cyc_Dict_T*t;};extern char Cyc_Dict_Present[8U];struct Cyc_Dict_Present_exn_struct{char*tag;};extern char Cyc_Dict_Absent[7U];struct Cyc_Dict_Absent_exn_struct{char*tag;};
# 68 "dict.h"
struct Cyc_Dict_Dict Cyc_Dict_rempty(struct _RegionHandle*,int(*cmp)(void*,void*));
# 87
struct Cyc_Dict_Dict Cyc_Dict_insert(struct Cyc_Dict_Dict d,void*k,void*v);
# 113
void*Cyc_Dict_lookup_other(struct Cyc_Dict_Dict d,int(*cmp)(void*,void*),void*k);
# 122 "dict.h"
void**Cyc_Dict_lookup_opt(struct Cyc_Dict_Dict d,void*k);struct Cyc_Xarray_Xarray{struct _fat_ptr elmts;int num_elmts;};
# 40 "xarray.h"
int Cyc_Xarray_length(struct Cyc_Xarray_Xarray*);
# 42
void*Cyc_Xarray_get(struct Cyc_Xarray_Xarray*,int);
# 57
struct Cyc_Xarray_Xarray*Cyc_Xarray_rcreate_empty(struct _RegionHandle*);
# 69
int Cyc_Xarray_add_ind(struct Cyc_Xarray_Xarray*,void*);
# 121
void Cyc_Xarray_reuse(struct Cyc_Xarray_Xarray*xarr);struct Cyc_Position_Error;struct Cyc_Relations_Reln;struct _union_Nmspace_Rel_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_Abs_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_C_n{int tag;struct Cyc_List_List*val;};struct _union_Nmspace_Loc_n{int tag;int val;};union Cyc_Absyn_Nmspace{struct _union_Nmspace_Rel_n Rel_n;struct _union_Nmspace_Abs_n Abs_n;struct _union_Nmspace_C_n C_n;struct _union_Nmspace_Loc_n Loc_n;};
# 95 "absyn.h"
extern union Cyc_Absyn_Nmspace Cyc_Absyn_Loc_n;
union Cyc_Absyn_Nmspace Cyc_Absyn_Rel_n(struct Cyc_List_List*);
# 98
union Cyc_Absyn_Nmspace Cyc_Absyn_Abs_n(struct Cyc_List_List*ns,int C_scope);struct _tuple1{union Cyc_Absyn_Nmspace f1;struct _fat_ptr*f2;};
# 161
enum Cyc_Absyn_Scope{Cyc_Absyn_Static =0U,Cyc_Absyn_Abstract =1U,Cyc_Absyn_Public =2U,Cyc_Absyn_Extern =3U,Cyc_Absyn_ExternC =4U,Cyc_Absyn_Register =5U};struct Cyc_Absyn_Tqual{int print_const: 1;int q_volatile: 1;int q_restrict: 1;int real_const: 1;unsigned loc;};
# 182
enum Cyc_Absyn_Size_of{Cyc_Absyn_Char_sz =0U,Cyc_Absyn_Short_sz =1U,Cyc_Absyn_Int_sz =2U,Cyc_Absyn_Long_sz =3U,Cyc_Absyn_LongLong_sz =4U};
enum Cyc_Absyn_Sign{Cyc_Absyn_Signed =0U,Cyc_Absyn_Unsigned =1U,Cyc_Absyn_None =2U};
enum Cyc_Absyn_AggrKind{Cyc_Absyn_StructA =0U,Cyc_Absyn_UnionA =1U};
# 187
enum Cyc_Absyn_AliasQual{Cyc_Absyn_Aliasable =0U,Cyc_Absyn_Unique =1U,Cyc_Absyn_Top =2U};
# 192
enum Cyc_Absyn_KindQual{Cyc_Absyn_AnyKind =0U,Cyc_Absyn_MemKind =1U,Cyc_Absyn_BoxKind =2U,Cyc_Absyn_RgnKind =3U,Cyc_Absyn_EffKind =4U,Cyc_Absyn_IntKind =5U,Cyc_Absyn_BoolKind =6U,Cyc_Absyn_PtrBndKind =7U};struct Cyc_Absyn_Kind{enum Cyc_Absyn_KindQual kind;enum Cyc_Absyn_AliasQual aliasqual;};struct Cyc_Absyn_Eq_kb_Absyn_KindBound_struct{int tag;struct Cyc_Absyn_Kind*f1;};struct Cyc_Absyn_Unknown_kb_Absyn_KindBound_struct{int tag;struct Cyc_Core_Opt*f1;};struct Cyc_Absyn_Less_kb_Absyn_KindBound_struct{int tag;struct Cyc_Core_Opt*f1;struct Cyc_Absyn_Kind*f2;};struct Cyc_Absyn_Tvar{struct _fat_ptr*name;int identity;void*kind;};struct Cyc_Absyn_PtrLoc{unsigned ptr_loc;unsigned rgn_loc;unsigned zt_loc;};struct Cyc_Absyn_PtrAtts{void*rgn;void*nullable;void*bounds;void*zero_term;struct Cyc_Absyn_PtrLoc*ptrloc;};struct Cyc_Absyn_PtrInfo{void*elt_type;struct Cyc_Absyn_Tqual elt_tq;struct Cyc_Absyn_PtrAtts ptr_atts;};struct Cyc_Absyn_VarargInfo{struct _fat_ptr*name;struct Cyc_Absyn_Tqual tq;void*type;int inject;};struct Cyc_Absyn_Star_Absyn_Cap_struct{int tag;};struct Cyc_Absyn_Bot_Absyn_Cap_struct{int tag;};struct Cyc_Absyn_Nat_Absyn_Cap_struct{int tag;struct Cyc_Absyn_Exp*f1;int f2;};struct Cyc_Absyn_RgnEffect{void*name;void*rgn;void*lock;void*unadopt;void*parent;};struct Cyc_Absyn_FnInfo{struct Cyc_List_List*tvars;void*effect;struct Cyc_Absyn_Tqual ret_tqual;void*ret_type;struct Cyc_List_List*args;int c_varargs;struct Cyc_Absyn_VarargInfo*cyc_varargs;struct Cyc_List_List*rgn_po;struct Cyc_List_List*attributes;struct Cyc_Absyn_Exp*requires_clause;struct Cyc_List_List*requires_relns;struct Cyc_Absyn_Exp*ensures_clause;struct Cyc_List_List*ensures_relns;struct Cyc_List_List*ieffect;struct Cyc_List_List*oeffect;};struct Cyc_Absyn_UnknownDatatypeInfo{struct _tuple1*name;int is_extensible;};struct _union_DatatypeInfo_UnknownDatatype{int tag;struct Cyc_Absyn_UnknownDatatypeInfo val;};struct _union_DatatypeInfo_KnownDatatype{int tag;struct Cyc_Absyn_Datatypedecl**val;};union Cyc_Absyn_DatatypeInfo{struct _union_DatatypeInfo_UnknownDatatype UnknownDatatype;struct _union_DatatypeInfo_KnownDatatype KnownDatatype;};struct Cyc_Absyn_UnknownDatatypeFieldInfo{struct _tuple1*datatype_name;struct _tuple1*field_name;int is_extensible;};struct _union_DatatypeFieldInfo_UnknownDatatypefield{int tag;struct Cyc_Absyn_UnknownDatatypeFieldInfo val;};struct _tuple2{struct Cyc_Absyn_Datatypedecl*f1;struct Cyc_Absyn_Datatypefield*f2;};struct _union_DatatypeFieldInfo_KnownDatatypefield{int tag;struct _tuple2 val;};union Cyc_Absyn_DatatypeFieldInfo{struct _union_DatatypeFieldInfo_UnknownDatatypefield UnknownDatatypefield;struct _union_DatatypeFieldInfo_KnownDatatypefield KnownDatatypefield;};struct _tuple3{enum Cyc_Absyn_AggrKind f1;struct _tuple1*f2;struct Cyc_Core_Opt*f3;};struct _union_AggrInfo_UnknownAggr{int tag;struct _tuple3 val;};struct _union_AggrInfo_KnownAggr{int tag;struct Cyc_Absyn_Aggrdecl**val;};union Cyc_Absyn_AggrInfo{struct _union_AggrInfo_UnknownAggr UnknownAggr;struct _union_AggrInfo_KnownAggr KnownAggr;};
# 331
union Cyc_Absyn_AggrInfo Cyc_Absyn_UnknownAggr(enum Cyc_Absyn_AggrKind,struct _tuple1*,struct Cyc_Core_Opt*);
union Cyc_Absyn_AggrInfo Cyc_Absyn_KnownAggr(struct Cyc_Absyn_Aggrdecl**);struct Cyc_Absyn_ArrayInfo{void*elt_type;struct Cyc_Absyn_Tqual tq;struct Cyc_Absyn_Exp*num_elts;void*zero_term;unsigned zt_loc;};struct Cyc_Absyn_Aggr_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Aggrdecl*f1;};struct Cyc_Absyn_Enum_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;};struct Cyc_Absyn_Datatype_td_Absyn_Raw_typedecl_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;};struct Cyc_Absyn_TypeDecl{void*r;unsigned loc;};struct Cyc_Absyn_VoidCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_IntCon_Absyn_TyCon_struct{int tag;enum Cyc_Absyn_Sign f1;enum Cyc_Absyn_Size_of f2;};struct Cyc_Absyn_FloatCon_Absyn_TyCon_struct{int tag;int f1;};struct Cyc_Absyn_RgnHandleCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_TagCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_HeapCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_UniqueCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_RefCntCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_AccessCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_JoinCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_RgnsCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_TrueCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_FalseCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_ThinCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_FatCon_Absyn_TyCon_struct{int tag;};struct Cyc_Absyn_EnumCon_Absyn_TyCon_struct{int tag;struct _tuple1*f1;struct Cyc_Absyn_Enumdecl*f2;};struct Cyc_Absyn_AnonEnumCon_Absyn_TyCon_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_BuiltinCon_Absyn_TyCon_struct{int tag;struct _fat_ptr f1;struct Cyc_Absyn_Kind*f2;};struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_DatatypeInfo f1;};struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_DatatypeFieldInfo f1;};struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct{int tag;union Cyc_Absyn_AggrInfo f1;};struct Cyc_Absyn_AppType_Absyn_Type_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Evar_Absyn_Type_struct{int tag;struct Cyc_Core_Opt*f1;void*f2;int f3;struct Cyc_Core_Opt*f4;};struct Cyc_Absyn_VarType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Tvar*f1;};struct Cyc_Absyn_PointerType_Absyn_Type_struct{int tag;struct Cyc_Absyn_PtrInfo f1;};struct Cyc_Absyn_ArrayType_Absyn_Type_struct{int tag;struct Cyc_Absyn_ArrayInfo f1;};struct Cyc_Absyn_FnType_Absyn_Type_struct{int tag;struct Cyc_Absyn_FnInfo f1;};struct Cyc_Absyn_TupleType_Absyn_Type_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct{int tag;enum Cyc_Absyn_AggrKind f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_TypedefType_Absyn_Type_struct{int tag;struct _tuple1*f1;struct Cyc_List_List*f2;struct Cyc_Absyn_Typedefdecl*f3;void*f4;};struct Cyc_Absyn_ValueofType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct{int tag;struct Cyc_Absyn_TypeDecl*f1;void**f2;};struct Cyc_Absyn_TypeofType_Absyn_Type_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_NoTypes_Absyn_Funcparams_struct{int tag;struct Cyc_List_List*f1;unsigned f2;};struct Cyc_Absyn_WithTypes_Absyn_Funcparams_struct{int tag;struct Cyc_List_List*f1;int f2;struct Cyc_Absyn_VarargInfo*f3;void*f4;struct Cyc_List_List*f5;struct Cyc_Absyn_Exp*f6;struct Cyc_Absyn_Exp*f7;struct Cyc_List_List*f8;struct Cyc_List_List*f9;};
# 437 "absyn.h"
enum Cyc_Absyn_Format_Type{Cyc_Absyn_Printf_ft =0U,Cyc_Absyn_Scanf_ft =1U};struct Cyc_Absyn_Regparm_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Stdcall_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Cdecl_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Fastcall_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Noreturn_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Const_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Aligned_att_Absyn_Attribute_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Packed_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Section_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Nocommon_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Shared_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Unused_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Weak_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Dllimport_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Dllexport_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_instrument_function_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Constructor_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Destructor_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_check_memory_usage_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Format_att_Absyn_Attribute_struct{int tag;enum Cyc_Absyn_Format_Type f1;int f2;int f3;};struct Cyc_Absyn_Initializes_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Noliveunique_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Consume_att_Absyn_Attribute_struct{int tag;int f1;};struct Cyc_Absyn_Pure_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Mode_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Alias_att_Absyn_Attribute_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Always_inline_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_No_throw_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Non_null_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Deprecated_att_Absyn_Attribute_struct{int tag;};struct Cyc_Absyn_Carray_mod_Absyn_Type_modifier_struct{int tag;void*f1;unsigned f2;};struct Cyc_Absyn_ConstArray_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_Absyn_Exp*f1;void*f2;unsigned f3;};struct Cyc_Absyn_Pointer_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_Absyn_PtrAtts f1;struct Cyc_Absyn_Tqual f2;};struct Cyc_Absyn_Function_mod_Absyn_Type_modifier_struct{int tag;void*f1;};struct Cyc_Absyn_TypeParams_mod_Absyn_Type_modifier_struct{int tag;struct Cyc_List_List*f1;unsigned f2;int f3;};struct Cyc_Absyn_Attributes_mod_Absyn_Type_modifier_struct{int tag;unsigned f1;struct Cyc_List_List*f2;};struct _union_Cnst_Null_c{int tag;int val;};struct _tuple4{enum Cyc_Absyn_Sign f1;char f2;};struct _union_Cnst_Char_c{int tag;struct _tuple4 val;};struct _union_Cnst_Wchar_c{int tag;struct _fat_ptr val;};struct _tuple5{enum Cyc_Absyn_Sign f1;short f2;};struct _union_Cnst_Short_c{int tag;struct _tuple5 val;};struct _tuple6{enum Cyc_Absyn_Sign f1;int f2;};struct _union_Cnst_Int_c{int tag;struct _tuple6 val;};struct _tuple7{enum Cyc_Absyn_Sign f1;long long f2;};struct _union_Cnst_LongLong_c{int tag;struct _tuple7 val;};struct _tuple8{struct _fat_ptr f1;int f2;};struct _union_Cnst_Float_c{int tag;struct _tuple8 val;};struct _union_Cnst_String_c{int tag;struct _fat_ptr val;};struct _union_Cnst_Wstring_c{int tag;struct _fat_ptr val;};union Cyc_Absyn_Cnst{struct _union_Cnst_Null_c Null_c;struct _union_Cnst_Char_c Char_c;struct _union_Cnst_Wchar_c Wchar_c;struct _union_Cnst_Short_c Short_c;struct _union_Cnst_Int_c Int_c;struct _union_Cnst_LongLong_c LongLong_c;struct _union_Cnst_Float_c Float_c;struct _union_Cnst_String_c String_c;struct _union_Cnst_Wstring_c Wstring_c;};
# 532
enum Cyc_Absyn_Primop{Cyc_Absyn_Plus =0U,Cyc_Absyn_Times =1U,Cyc_Absyn_Minus =2U,Cyc_Absyn_Div =3U,Cyc_Absyn_Mod =4U,Cyc_Absyn_Eq =5U,Cyc_Absyn_Neq =6U,Cyc_Absyn_Gt =7U,Cyc_Absyn_Lt =8U,Cyc_Absyn_Gte =9U,Cyc_Absyn_Lte =10U,Cyc_Absyn_Not =11U,Cyc_Absyn_Bitnot =12U,Cyc_Absyn_Bitand =13U,Cyc_Absyn_Bitor =14U,Cyc_Absyn_Bitxor =15U,Cyc_Absyn_Bitlshift =16U,Cyc_Absyn_Bitlrshift =17U,Cyc_Absyn_Numelts =18U};
# 539
enum Cyc_Absyn_Incrementor{Cyc_Absyn_PreInc =0U,Cyc_Absyn_PostInc =1U,Cyc_Absyn_PreDec =2U,Cyc_Absyn_PostDec =3U};struct Cyc_Absyn_VarargCallInfo{int num_varargs;struct Cyc_List_List*injectors;struct Cyc_Absyn_VarargInfo*vai;};struct Cyc_Absyn_StructField_Absyn_OffsetofField_struct{int tag;struct _fat_ptr*f1;};struct Cyc_Absyn_TupleIndex_Absyn_OffsetofField_struct{int tag;unsigned f1;};
# 557
enum Cyc_Absyn_Coercion{Cyc_Absyn_Unknown_coercion =0U,Cyc_Absyn_No_coercion =1U,Cyc_Absyn_Null_to_NonNull =2U,Cyc_Absyn_Other_coercion =3U};struct Cyc_Absyn_MallocInfo{int is_calloc;struct Cyc_Absyn_Exp*rgn;void**elt_type;struct Cyc_Absyn_Exp*num_elts;int fat_result;int inline_call;};struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct{int tag;union Cyc_Absyn_Cnst f1;};struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Pragma_e_Absyn_Raw_exp_struct{int tag;struct _fat_ptr f1;};struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct{int tag;enum Cyc_Absyn_Primop f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_AssignOp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Core_Opt*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_Increment_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;enum Cyc_Absyn_Incrementor f2;};struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;struct Cyc_Absyn_VarargCallInfo*f3;int f4;};struct Cyc_Absyn_Throw_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;int f2;};struct Cyc_Absyn_NoInstantiate_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Instantiate_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_Absyn_Exp*f2;int f3;enum Cyc_Absyn_Coercion f4;};struct Cyc_Absyn_Address_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_New_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Sizeoftype_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Sizeofexp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Offsetof_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Deref_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;int f3;int f4;};struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;int f3;int f4;};struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Tuple_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;};struct _tuple9{struct _fat_ptr*f1;struct Cyc_Absyn_Tqual f2;void*f3;};struct Cyc_Absyn_CompoundLit_e_Absyn_Raw_exp_struct{int tag;struct _tuple9*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Array_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Exp*f2;struct Cyc_Absyn_Exp*f3;int f4;};struct Cyc_Absyn_ComprehensionNoinit_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;void*f2;int f3;};struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct{int tag;struct _tuple1*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;struct Cyc_Absyn_Aggrdecl*f4;};struct Cyc_Absyn_AnonStruct_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Datatype_e_Absyn_Raw_exp_struct{int tag;struct Cyc_List_List*f1;struct Cyc_Absyn_Datatypedecl*f2;struct Cyc_Absyn_Datatypefield*f3;};struct Cyc_Absyn_Enum_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_AnonEnum_e_Absyn_Raw_exp_struct{int tag;void*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_Spawn_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_MallocInfo f1;};struct Cyc_Absyn_Swap_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct Cyc_Absyn_UnresolvedMem_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Core_Opt*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_StmtExp_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Stmt*f1;};struct Cyc_Absyn_Tagcheck_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _fat_ptr*f2;};struct Cyc_Absyn_Valueof_e_Absyn_Raw_exp_struct{int tag;void*f1;};struct Cyc_Absyn_Asm_e_Absyn_Raw_exp_struct{int tag;int f1;struct _fat_ptr f2;struct Cyc_List_List*f3;struct Cyc_List_List*f4;struct Cyc_List_List*f5;};struct Cyc_Absyn_Extension_e_Absyn_Raw_exp_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Exp{void*topt;void*r;unsigned loc;void*annot;};struct Cyc_Absyn_Skip_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Exp_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_Seq_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Return_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_IfThenElse_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Stmt*f2;struct Cyc_Absyn_Stmt*f3;};struct _tuple10{struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_While_s_Absyn_Raw_stmt_struct{int tag;struct _tuple10 f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Break_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Continue_s_Absyn_Raw_stmt_struct{int tag;};struct Cyc_Absyn_Goto_s_Absyn_Raw_stmt_struct{int tag;struct _fat_ptr*f1;};struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct _tuple10 f2;struct _tuple10 f3;struct Cyc_Absyn_Stmt*f4;};struct Cyc_Absyn_Switch_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Exp*f1;struct Cyc_List_List*f2;void*f3;};struct Cyc_Absyn_Fallthru_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_List_List*f1;struct Cyc_Absyn_Switch_clause**f2;};struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Decl*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Label_s_Absyn_Raw_stmt_struct{int tag;struct _fat_ptr*f1;struct Cyc_Absyn_Stmt*f2;};struct Cyc_Absyn_Do_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct _tuple10 f2;};struct Cyc_Absyn_TryCatch_s_Absyn_Raw_stmt_struct{int tag;struct Cyc_Absyn_Stmt*f1;struct Cyc_List_List*f2;void*f3;};struct Cyc_Absyn_Stmt{void*r;unsigned loc;void*annot;};struct Cyc_Absyn_Wild_p_Absyn_Raw_pat_struct{int tag;};struct Cyc_Absyn_Var_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Pat*f2;};struct Cyc_Absyn_AliasVar_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;};struct Cyc_Absyn_Reference_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Pat*f2;};struct Cyc_Absyn_TagInt_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;};struct Cyc_Absyn_Tuple_p_Absyn_Raw_pat_struct{int tag;struct Cyc_List_List*f1;int f2;};struct Cyc_Absyn_Pointer_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Pat*f1;};struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct{int tag;union Cyc_Absyn_AggrInfo*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;int f4;};struct Cyc_Absyn_Datatype_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;struct Cyc_Absyn_Datatypefield*f2;struct Cyc_List_List*f3;int f4;};struct Cyc_Absyn_Null_p_Absyn_Raw_pat_struct{int tag;};struct Cyc_Absyn_Int_p_Absyn_Raw_pat_struct{int tag;enum Cyc_Absyn_Sign f1;int f2;};struct Cyc_Absyn_Char_p_Absyn_Raw_pat_struct{int tag;char f1;};struct Cyc_Absyn_Float_p_Absyn_Raw_pat_struct{int tag;struct _fat_ptr f1;int f2;};struct Cyc_Absyn_Enum_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_AnonEnum_p_Absyn_Raw_pat_struct{int tag;void*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Absyn_UnknownId_p_Absyn_Raw_pat_struct{int tag;struct _tuple1*f1;};struct Cyc_Absyn_UnknownCall_p_Absyn_Raw_pat_struct{int tag;struct _tuple1*f1;struct Cyc_List_List*f2;int f3;};struct Cyc_Absyn_Exp_p_Absyn_Raw_pat_struct{int tag;struct Cyc_Absyn_Exp*f1;};
# 725 "absyn.h"
extern struct Cyc_Absyn_Wild_p_Absyn_Raw_pat_struct Cyc_Absyn_Wild_p_val;struct Cyc_Absyn_Pat{void*r;void*topt;unsigned loc;};struct Cyc_Absyn_Switch_clause{struct Cyc_Absyn_Pat*pattern;struct Cyc_Core_Opt*pat_vars;struct Cyc_Absyn_Exp*where_clause;struct Cyc_Absyn_Stmt*body;unsigned loc;};struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct{int tag;struct _tuple1*f1;};struct Cyc_Absyn_Global_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Funname_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Fndecl*f1;};struct Cyc_Absyn_Param_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Local_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Pat_b_Absyn_Binding_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Vardecl{enum Cyc_Absyn_Scope sc;struct _tuple1*name;unsigned varloc;struct Cyc_Absyn_Tqual tq;void*type;struct Cyc_Absyn_Exp*initializer;void*rgn;struct Cyc_List_List*attributes;int escapes;};struct Cyc_Absyn_Fndecl{enum Cyc_Absyn_Scope sc;int is_inline;struct _tuple1*name;struct Cyc_Absyn_Stmt*body;struct Cyc_Absyn_FnInfo i;void*cached_type;struct Cyc_Core_Opt*param_vardecls;struct Cyc_Absyn_Vardecl*fn_vardecl;};struct Cyc_Absyn_Aggrfield{struct _fat_ptr*name;struct Cyc_Absyn_Tqual tq;void*type;struct Cyc_Absyn_Exp*width;struct Cyc_List_List*attributes;struct Cyc_Absyn_Exp*requires_clause;};struct Cyc_Absyn_AggrdeclImpl{struct Cyc_List_List*exist_vars;struct Cyc_List_List*rgn_po;struct Cyc_List_List*fields;int tagged;};struct Cyc_Absyn_Aggrdecl{enum Cyc_Absyn_AggrKind kind;enum Cyc_Absyn_Scope sc;struct _tuple1*name;struct Cyc_List_List*tvs;struct Cyc_Absyn_AggrdeclImpl*impl;struct Cyc_List_List*attributes;int expected_mem_kind;};struct Cyc_Absyn_Datatypefield{struct _tuple1*name;struct Cyc_List_List*typs;unsigned loc;enum Cyc_Absyn_Scope sc;};struct Cyc_Absyn_Datatypedecl{enum Cyc_Absyn_Scope sc;struct _tuple1*name;struct Cyc_List_List*tvs;struct Cyc_Core_Opt*fields;int is_extensible;};struct Cyc_Absyn_Enumfield{struct _tuple1*name;struct Cyc_Absyn_Exp*tag;unsigned loc;};struct Cyc_Absyn_Enumdecl{enum Cyc_Absyn_Scope sc;struct _tuple1*name;struct Cyc_Core_Opt*fields;};struct Cyc_Absyn_Typedefdecl{struct _tuple1*name;struct Cyc_Absyn_Tqual tq;struct Cyc_List_List*tvs;struct Cyc_Core_Opt*kind;void*defn;struct Cyc_List_List*atts;int extern_c;};struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Vardecl*f1;};struct Cyc_Absyn_Fn_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Fndecl*f1;};struct Cyc_Absyn_Let_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Pat*f1;struct Cyc_Core_Opt*f2;struct Cyc_Absyn_Exp*f3;void*f4;};struct Cyc_Absyn_Letv_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;};struct Cyc_Absyn_Region_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Tvar*f1;struct Cyc_Absyn_Vardecl*f2;struct Cyc_Absyn_Exp*f3;};struct Cyc_Absyn_Aggr_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Aggrdecl*f1;};struct Cyc_Absyn_Datatype_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;};struct Cyc_Absyn_Enum_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;};struct Cyc_Absyn_Typedef_d_Absyn_Raw_decl_struct{int tag;struct Cyc_Absyn_Typedefdecl*f1;};struct Cyc_Absyn_Namespace_d_Absyn_Raw_decl_struct{int tag;struct _fat_ptr*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_Using_d_Absyn_Raw_decl_struct{int tag;struct _tuple1*f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_ExternC_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;};struct _tuple11{unsigned f1;struct Cyc_List_List*f2;};struct Cyc_Absyn_ExternCinclude_d_Absyn_Raw_decl_struct{int tag;struct Cyc_List_List*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;struct _tuple11*f4;};struct Cyc_Absyn_Porton_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Portoff_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Tempeston_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Tempestoff_d_Absyn_Raw_decl_struct{int tag;};struct Cyc_Absyn_Decl{void*r;unsigned loc;};struct Cyc_Absyn_ArrayElement_Absyn_Designator_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Absyn_FieldName_Absyn_Designator_struct{int tag;struct _fat_ptr*f1;};extern char Cyc_Absyn_EmptyAnnot[11U];struct Cyc_Absyn_EmptyAnnot_Absyn_AbsynAnnot_struct{char*tag;};
# 901
extern struct Cyc_Absyn_EmptyAnnot_Absyn_AbsynAnnot_struct Cyc_Absyn_EmptyAnnot_val;
# 908
int Cyc_Absyn_qvar_cmp(struct _tuple1*,struct _tuple1*);
# 917
struct Cyc_Absyn_Tqual Cyc_Absyn_empty_tqual(unsigned);
# 938
extern void*Cyc_Absyn_char_type;extern void*Cyc_Absyn_uchar_type;extern void*Cyc_Absyn_uint_type;
# 940
extern void*Cyc_Absyn_sint_type;
# 945
extern void*Cyc_Absyn_heap_rgn_type;
# 949
extern void*Cyc_Absyn_true_type;extern void*Cyc_Absyn_false_type;
# 951
extern void*Cyc_Absyn_void_type;
# 968
void*Cyc_Absyn_exn_type();
# 976
extern void*Cyc_Absyn_fat_bound_type;
# 980
void*Cyc_Absyn_bounds_one();
# 990
void*Cyc_Absyn_star_type(void*t,void*rgn,struct Cyc_Absyn_Tqual,void*zero_term);
# 994
void*Cyc_Absyn_cstar_type(void*,struct Cyc_Absyn_Tqual);
# 996
void*Cyc_Absyn_fatptr_type(void*t,void*rgn,struct Cyc_Absyn_Tqual,void*zeroterm);
# 998
void*Cyc_Absyn_strct(struct _fat_ptr*name);
void*Cyc_Absyn_strctq(struct _tuple1*name);
void*Cyc_Absyn_unionq_type(struct _tuple1*name);
# 1004
void*Cyc_Absyn_array_type(void*elt_type,struct Cyc_Absyn_Tqual tq,struct Cyc_Absyn_Exp*num_elts,void*zero_term,unsigned ztloc);
# 1009
void*Cyc_Absyn_aggr_type(union Cyc_Absyn_AggrInfo,struct Cyc_List_List*args);
# 1012
struct Cyc_Absyn_Exp*Cyc_Absyn_new_exp(void*,unsigned);
# 1014
struct Cyc_Absyn_Exp*Cyc_Absyn_copy_exp(struct Cyc_Absyn_Exp*);
# 1020
struct Cyc_Absyn_Exp*Cyc_Absyn_int_exp(enum Cyc_Absyn_Sign,int,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_signed_int_exp(int,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_uint_exp(unsigned,unsigned);
# 1025
struct Cyc_Absyn_Exp*Cyc_Absyn_float_exp(struct _fat_ptr,int,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_string_exp(struct _fat_ptr,unsigned);
# 1028
struct Cyc_Absyn_Exp*Cyc_Absyn_var_exp(struct _tuple1*,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_varb_exp(void*,unsigned);
# 1033
struct Cyc_Absyn_Exp*Cyc_Absyn_prim1_exp(enum Cyc_Absyn_Primop,struct Cyc_Absyn_Exp*,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_prim2_exp(enum Cyc_Absyn_Primop,struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*,unsigned);
# 1036
struct Cyc_Absyn_Exp*Cyc_Absyn_add_exp(struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*,unsigned);
# 1038
struct Cyc_Absyn_Exp*Cyc_Absyn_divide_exp(struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_eq_exp(struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_neq_exp(struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*,unsigned);
# 1045
struct Cyc_Absyn_Exp*Cyc_Absyn_assignop_exp(struct Cyc_Absyn_Exp*,struct Cyc_Core_Opt*,struct Cyc_Absyn_Exp*,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_assign_exp(struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_increment_exp(struct Cyc_Absyn_Exp*,enum Cyc_Absyn_Incrementor,unsigned);
# 1049
struct Cyc_Absyn_Exp*Cyc_Absyn_and_exp(struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*,unsigned);
# 1053
struct Cyc_Absyn_Exp*Cyc_Absyn_fncall_exp(struct Cyc_Absyn_Exp*,struct Cyc_List_List*,unsigned);
# 1058
struct Cyc_Absyn_Exp*Cyc_Absyn_cast_exp(void*,struct Cyc_Absyn_Exp*,int user_cast,enum Cyc_Absyn_Coercion,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_address_exp(struct Cyc_Absyn_Exp*,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_sizeoftype_exp(void*t,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_sizeofexp_exp(struct Cyc_Absyn_Exp*e,unsigned);
# 1063
struct Cyc_Absyn_Exp*Cyc_Absyn_deref_exp(struct Cyc_Absyn_Exp*,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_aggrmember_exp(struct Cyc_Absyn_Exp*,struct _fat_ptr*,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_aggrarrow_exp(struct Cyc_Absyn_Exp*,struct _fat_ptr*,unsigned);
struct Cyc_Absyn_Exp*Cyc_Absyn_subscript_exp(struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*,unsigned);
# 1068
struct Cyc_Absyn_Exp*Cyc_Absyn_stmt_exp(struct Cyc_Absyn_Stmt*,unsigned);
# 1070
struct Cyc_Absyn_Exp*Cyc_Absyn_array_exp(struct Cyc_List_List*,unsigned);
# 1075
struct Cyc_Absyn_Exp*Cyc_Absyn_unresolvedmem_exp(struct Cyc_Core_Opt*,struct Cyc_List_List*,unsigned);
# 1081
struct Cyc_Absyn_Stmt*Cyc_Absyn_new_stmt(void*,unsigned);
struct Cyc_Absyn_Stmt*Cyc_Absyn_skip_stmt(unsigned);
struct Cyc_Absyn_Stmt*Cyc_Absyn_exp_stmt(struct Cyc_Absyn_Exp*,unsigned);
struct Cyc_Absyn_Stmt*Cyc_Absyn_seq_stmt(struct Cyc_Absyn_Stmt*,struct Cyc_Absyn_Stmt*,unsigned);
# 1086
struct Cyc_Absyn_Stmt*Cyc_Absyn_return_stmt(struct Cyc_Absyn_Exp*,unsigned);
struct Cyc_Absyn_Stmt*Cyc_Absyn_ifthenelse_stmt(struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Stmt*,struct Cyc_Absyn_Stmt*,unsigned);
# 1094
struct Cyc_Absyn_Stmt*Cyc_Absyn_decl_stmt(struct Cyc_Absyn_Decl*,struct Cyc_Absyn_Stmt*,unsigned);
struct Cyc_Absyn_Stmt*Cyc_Absyn_declare_stmt(struct _tuple1*,void*,struct Cyc_Absyn_Exp*init,struct Cyc_Absyn_Stmt*,unsigned);
struct Cyc_Absyn_Stmt*Cyc_Absyn_label_stmt(struct _fat_ptr*,struct Cyc_Absyn_Stmt*,unsigned);
# 1098
struct Cyc_Absyn_Stmt*Cyc_Absyn_goto_stmt(struct _fat_ptr*,unsigned);
struct Cyc_Absyn_Stmt*Cyc_Absyn_assign_stmt(struct Cyc_Absyn_Exp*,struct Cyc_Absyn_Exp*,unsigned);
# 1103
struct Cyc_Absyn_Pat*Cyc_Absyn_new_pat(void*,unsigned);
# 1107
struct Cyc_Absyn_Decl*Cyc_Absyn_new_decl(void*,unsigned);
# 1112
struct Cyc_Absyn_Vardecl*Cyc_Absyn_new_vardecl(unsigned varloc,struct _tuple1*,void*,struct Cyc_Absyn_Exp*init);
struct Cyc_Absyn_Vardecl*Cyc_Absyn_static_vardecl(struct _tuple1*x,void*t,struct Cyc_Absyn_Exp*init);
# 1151
int Cyc_Absyn_is_lvalue(struct Cyc_Absyn_Exp*);
# 1154
struct Cyc_Absyn_Aggrfield*Cyc_Absyn_lookup_field(struct Cyc_List_List*,struct _fat_ptr*);
# 1172
struct _fat_ptr*Cyc_Absyn_fieldname(int);struct _tuple12{enum Cyc_Absyn_AggrKind f1;struct _tuple1*f2;};
# 1174
struct _tuple12 Cyc_Absyn_aggr_kinded_name(union Cyc_Absyn_AggrInfo);
# 1176
struct Cyc_Absyn_Aggrdecl*Cyc_Absyn_get_known_aggrdecl(union Cyc_Absyn_AggrInfo);
# 1182
struct _tuple1*Cyc_Absyn_binding2qvar(void*);
# 1185
struct _fat_ptr*Cyc_Absyn_designatorlist_to_fieldname(struct Cyc_List_List*);
# 1190
extern int Cyc_Absyn_no_regions;
# 1200
int Cyc_Absyn_var_may_appear_exp(struct _tuple1*,struct Cyc_Absyn_Exp*);
# 38 "warn.h"
void*Cyc_Warn_vimpos(struct _fat_ptr fmt,struct _fat_ptr ap);
# 46
void*Cyc_Warn_impos_loc(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap);struct Cyc_Warn_String_Warn_Warg_struct{int tag;struct _fat_ptr f1;};struct Cyc_Warn_Exp_Warn_Warg_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Warn_Typ_Warn_Warg_struct{int tag;void*f1;};struct Cyc_Warn_Qvar_Warn_Warg_struct{int tag;struct _tuple1*f1;};struct Cyc_PP_Ppstate;struct Cyc_PP_Out;struct Cyc_PP_Doc;struct Cyc_Absynpp_Params{int expand_typedefs;int qvar_to_Cids;int add_cyc_prefix;int to_VC;int decls_first;int rewrite_temp_tvars;int print_all_tvars;int print_all_kinds;int print_all_effects;int print_using_stmts;int print_externC_stmts;int print_full_evars;int print_zeroterm;int generate_line_directives;int use_curr_namespace;struct Cyc_List_List*curr_namespace;};
# 62 "absynpp.h"
struct _fat_ptr Cyc_Absynpp_typ2string(void*);
# 67
struct _fat_ptr Cyc_Absynpp_exp2string(struct Cyc_Absyn_Exp*);
struct _fat_ptr Cyc_Absynpp_stmt2string(struct Cyc_Absyn_Stmt*);
struct _fat_ptr Cyc_Absynpp_qvar2string(struct _tuple1*);
# 29 "unify.h"
int Cyc_Unify_unify(void*,void*);struct _union_RelnOp_RConst{int tag;unsigned val;};struct _union_RelnOp_RVar{int tag;struct Cyc_Absyn_Vardecl*val;};struct _union_RelnOp_RNumelts{int tag;struct Cyc_Absyn_Vardecl*val;};struct _union_RelnOp_RType{int tag;void*val;};struct _union_RelnOp_RParam{int tag;unsigned val;};struct _union_RelnOp_RParamNumelts{int tag;unsigned val;};struct _union_RelnOp_RReturn{int tag;unsigned val;};union Cyc_Relations_RelnOp{struct _union_RelnOp_RConst RConst;struct _union_RelnOp_RVar RVar;struct _union_RelnOp_RNumelts RNumelts;struct _union_RelnOp_RType RType;struct _union_RelnOp_RParam RParam;struct _union_RelnOp_RParamNumelts RParamNumelts;struct _union_RelnOp_RReturn RReturn;};
# 50 "relations-ap.h"
enum Cyc_Relations_Relation{Cyc_Relations_Req =0U,Cyc_Relations_Rneq =1U,Cyc_Relations_Rlte =2U,Cyc_Relations_Rlt =3U};struct Cyc_Relations_Reln{union Cyc_Relations_RelnOp rop1;enum Cyc_Relations_Relation relation;union Cyc_Relations_RelnOp rop2;};struct Cyc_RgnOrder_RgnPO;extern char Cyc_Tcenv_Env_error[10U];struct Cyc_Tcenv_Env_error_exn_struct{char*tag;};struct Cyc_Tcenv_Genv{struct Cyc_Dict_Dict aggrdecls;struct Cyc_Dict_Dict datatypedecls;struct Cyc_Dict_Dict enumdecls;struct Cyc_Dict_Dict typedefs;struct Cyc_Dict_Dict ordinaries;};struct Cyc_Tcenv_Fenv;struct Cyc_Tcenv_Tenv{struct Cyc_List_List*ns;struct Cyc_Tcenv_Genv*ae;struct Cyc_Tcenv_Fenv*le;int allow_valueof: 1;int in_extern_c_include: 1;int in_tempest: 1;int tempest_generalize: 1;};
# 88 "tcenv.h"
enum Cyc_Tcenv_NewStatus{Cyc_Tcenv_NoneNew =0U,Cyc_Tcenv_InNew =1U,Cyc_Tcenv_InNewAggr =2U};
# 30 "tcutil.h"
void*Cyc_Tcutil_impos(struct _fat_ptr,struct _fat_ptr);
void Cyc_Tcutil_terr(unsigned,struct _fat_ptr,struct _fat_ptr);
void Cyc_Tcutil_warn(unsigned,struct _fat_ptr,struct _fat_ptr);
# 36
int Cyc_Tcutil_is_void_type(void*);
# 41
int Cyc_Tcutil_is_arithmetic_type(void*);
# 44
int Cyc_Tcutil_is_pointer_type(void*);
int Cyc_Tcutil_is_array_type(void*);
int Cyc_Tcutil_is_boxed(void*);
# 49
int Cyc_Tcutil_is_fat_ptr(void*);
int Cyc_Tcutil_is_zeroterm_pointer_type(void*);
# 54
int Cyc_Tcutil_is_fat_pointer_type(void*);
# 64
void*Cyc_Tcutil_pointer_elt_type(void*);
# 72
struct Cyc_Absyn_Exp*Cyc_Tcutil_get_bounds_exp(void*def,void*b);
# 77
int Cyc_Tcutil_is_fat_pointer_type_elt(void*t,void**elt_dest);
# 79
int Cyc_Tcutil_is_zero_pointer_type_elt(void*t,void**elt_type_dest);
# 84
struct Cyc_Absyn_Exp*Cyc_Tcutil_get_bounds_exp(void*def,void*b);
# 97
struct Cyc_Absyn_Exp*Cyc_Tcutil_deep_copy_exp(int preserve_types,struct Cyc_Absyn_Exp*);
# 100
int Cyc_Tcutil_kind_leq(struct Cyc_Absyn_Kind*,struct Cyc_Absyn_Kind*);
# 105
struct Cyc_Absyn_Kind*Cyc_Tcutil_tvar_kind(struct Cyc_Absyn_Tvar*,struct Cyc_Absyn_Kind*def);
struct Cyc_Absyn_Kind*Cyc_Tcutil_type_kind(void*);
void*Cyc_Tcutil_compress(void*);
# 136
extern struct Cyc_Absyn_Kind Cyc_Tcutil_ak;
extern struct Cyc_Absyn_Kind Cyc_Tcutil_bk;
# 146
extern struct Cyc_Absyn_Kind Cyc_Tcutil_tbk;
# 182
int Cyc_Tcutil_typecmp(void*,void*);
int Cyc_Tcutil_aggrfield_cmp(struct Cyc_Absyn_Aggrfield*,struct Cyc_Absyn_Aggrfield*);
# 185
void*Cyc_Tcutil_substitute(struct Cyc_List_List*,void*);
# 187
void*Cyc_Tcutil_rsubstitute(struct _RegionHandle*,struct Cyc_List_List*,void*);
# 222
struct Cyc_List_List*Cyc_Tcutil_resolve_aggregate_designators(struct _RegionHandle*,unsigned,struct Cyc_List_List*,enum Cyc_Absyn_AggrKind,struct Cyc_List_List*);
# 229
int Cyc_Tcutil_is_zero_ptr_deref(struct Cyc_Absyn_Exp*,void**ptr_type,int*is_fat,void**elt_type);struct _tuple13{struct Cyc_Absyn_Tqual f1;void*f2;};
# 276
void*Cyc_Tcutil_snd_tqt(struct _tuple13*);
# 294
struct Cyc_List_List*Cyc_Tcutil_filter_nulls(struct Cyc_List_List*);
# 305
int Cyc_Tcutil_force_type2bool(int desired,void*);struct _tuple14{unsigned f1;int f2;};
# 28 "evexp.h"
struct _tuple14 Cyc_Evexp_eval_const_uint_exp(struct Cyc_Absyn_Exp*e);
# 32
int Cyc_Evexp_c_can_eval(struct Cyc_Absyn_Exp*e);extern char Cyc_InsertChecks_FatBound[9U];struct Cyc_InsertChecks_FatBound_Absyn_AbsynAnnot_struct{char*tag;};extern char Cyc_InsertChecks_NoCheck[8U];struct Cyc_InsertChecks_NoCheck_Absyn_AbsynAnnot_struct{char*tag;};extern char Cyc_InsertChecks_NullAndFatBound[16U];struct Cyc_InsertChecks_NullAndFatBound_Absyn_AbsynAnnot_struct{char*tag;};extern char Cyc_InsertChecks_NullAndThinBound[17U];struct Cyc_InsertChecks_NullAndThinBound_Absyn_AbsynAnnot_struct{char*tag;struct Cyc_Absyn_Exp*f1;};extern char Cyc_InsertChecks_NullOnly[9U];struct Cyc_InsertChecks_NullOnly_Absyn_AbsynAnnot_struct{char*tag;};extern char Cyc_InsertChecks_ThinBound[10U];struct Cyc_InsertChecks_ThinBound_Absyn_AbsynAnnot_struct{char*tag;struct Cyc_Absyn_Exp*f1;};
# 34 "insert_checks.h"
extern struct Cyc_InsertChecks_NoCheck_Absyn_AbsynAnnot_struct Cyc_InsertChecks_NoCheck_val;
# 36
extern struct Cyc_InsertChecks_NullAndFatBound_Absyn_AbsynAnnot_struct Cyc_InsertChecks_NullAndFatBound_val;struct Cyc_Hashtable_Table;
# 52 "hashtable.h"
void*Cyc_Hashtable_lookup(struct Cyc_Hashtable_Table*t,void*key);
# 38 "toc.h"
void*Cyc_Toc_typ_to_c(void*);
# 40
struct _tuple1*Cyc_Toc_temp_var();
extern struct _fat_ptr Cyc_Toc_globals;extern char Cyc_Toc_Dest[5U];struct Cyc_Toc_Dest_Absyn_AbsynAnnot_struct{char*tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Tcpat_TcPatResult{struct _tuple0*tvars_and_bounds_opt;struct Cyc_List_List*patvars;};struct Cyc_Tcpat_WhereTest_Tcpat_PatTest_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Tcpat_EqNull_Tcpat_PatTest_struct{int tag;};struct Cyc_Tcpat_NeqNull_Tcpat_PatTest_struct{int tag;};struct Cyc_Tcpat_EqEnum_Tcpat_PatTest_struct{int tag;struct Cyc_Absyn_Enumdecl*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Tcpat_EqAnonEnum_Tcpat_PatTest_struct{int tag;void*f1;struct Cyc_Absyn_Enumfield*f2;};struct Cyc_Tcpat_EqFloat_Tcpat_PatTest_struct{int tag;struct _fat_ptr f1;int f2;};struct Cyc_Tcpat_EqConst_Tcpat_PatTest_struct{int tag;unsigned f1;};struct Cyc_Tcpat_EqDatatypeTag_Tcpat_PatTest_struct{int tag;int f1;struct Cyc_Absyn_Datatypedecl*f2;struct Cyc_Absyn_Datatypefield*f3;};struct Cyc_Tcpat_EqTaggedUnion_Tcpat_PatTest_struct{int tag;struct _fat_ptr*f1;int f2;};struct Cyc_Tcpat_EqExtensibleDatatype_Tcpat_PatTest_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;struct Cyc_Absyn_Datatypefield*f2;};struct Cyc_Tcpat_Dummy_Tcpat_Access_struct{int tag;};struct Cyc_Tcpat_Deref_Tcpat_Access_struct{int tag;};struct Cyc_Tcpat_TupleField_Tcpat_Access_struct{int tag;unsigned f1;};struct Cyc_Tcpat_DatatypeField_Tcpat_Access_struct{int tag;struct Cyc_Absyn_Datatypedecl*f1;struct Cyc_Absyn_Datatypefield*f2;unsigned f3;};struct Cyc_Tcpat_AggrField_Tcpat_Access_struct{int tag;int f1;struct _fat_ptr*f2;};struct _union_PatOrWhere_pattern{int tag;struct Cyc_Absyn_Pat*val;};struct _union_PatOrWhere_where_clause{int tag;struct Cyc_Absyn_Exp*val;};union Cyc_Tcpat_PatOrWhere{struct _union_PatOrWhere_pattern pattern;struct _union_PatOrWhere_where_clause where_clause;};struct Cyc_Tcpat_PathNode{union Cyc_Tcpat_PatOrWhere orig_pat;void*access;};struct Cyc_Tcpat_Rhs{int used;unsigned pat_loc;struct Cyc_Absyn_Stmt*rhs;};struct Cyc_Tcpat_Failure_Tcpat_Decision_struct{int tag;void*f1;};struct Cyc_Tcpat_Success_Tcpat_Decision_struct{int tag;struct Cyc_Tcpat_Rhs*f1;};struct Cyc_Tcpat_SwitchDec_Tcpat_Decision_struct{int tag;struct Cyc_List_List*f1;struct Cyc_List_List*f2;void*f3;};
# 58 "toc.cyc"
extern int Cyc_noexpand_r;char Cyc_Toc_Dest[5U]="Dest";
# 73 "toc.cyc"
static void*Cyc_Toc_unimp(struct _fat_ptr fmt,struct _fat_ptr ap){
# 75
({struct _fat_ptr _tmp607=(struct _fat_ptr)({struct _fat_ptr _tmp606=({const char*_tmp0="Toc (unimplemented): ";_tag_fat(_tmp0,sizeof(char),22U);});Cyc_strconcat(_tmp606,(struct _fat_ptr)fmt);});Cyc_Warn_vimpos(_tmp607,ap);});}
# 73
static void*Cyc_Toc_toc_impos(struct _fat_ptr fmt,struct _fat_ptr ap){
# 79
({struct _fat_ptr _tmp609=(struct _fat_ptr)({struct _fat_ptr _tmp608=({const char*_tmp1="Toc: ";_tag_fat(_tmp1,sizeof(char),6U);});Cyc_strconcat(_tmp608,(struct _fat_ptr)fmt);});Cyc_Warn_vimpos(_tmp609,ap);});}
# 73
struct _fat_ptr Cyc_Toc_globals={(void*)0,(void*)0,(void*)(0 + 0)};
# 85
static struct Cyc_Hashtable_Table**Cyc_Toc_gpop_tables=0;
static struct Cyc_Hashtable_Table**Cyc_Toc_fn_pop_table=0;
static int Cyc_Toc_tuple_type_counter=0;
static int Cyc_Toc_temp_var_counter=0;
static int Cyc_Toc_fresh_label_counter=0;
# 95
static struct Cyc_List_List*Cyc_Toc_result_decls=0;
# 97
static int Cyc_Toc_get_npop(struct Cyc_Absyn_Stmt*s){
return({int(*_tmp60B)(struct Cyc_Hashtable_Table*t,struct Cyc_Absyn_Stmt*key)=({int(*_tmp2)(struct Cyc_Hashtable_Table*t,struct Cyc_Absyn_Stmt*key)=(int(*)(struct Cyc_Hashtable_Table*t,struct Cyc_Absyn_Stmt*key))Cyc_Hashtable_lookup;_tmp2;});struct Cyc_Hashtable_Table*_tmp60A=*((struct Cyc_Hashtable_Table**)_check_null(Cyc_Toc_fn_pop_table));_tmp60B(_tmp60A,s);});}struct Cyc_Toc_TocState{struct Cyc_List_List**tuple_types;struct Cyc_List_List**anon_aggr_types;struct Cyc_Dict_Dict*aggrs_so_far;struct Cyc_List_List**abs_struct_types;struct Cyc_Set_Set**datatypes_so_far;struct Cyc_Dict_Dict*xdatatypes_so_far;struct Cyc_Dict_Dict*qvar_tags;struct Cyc_Xarray_Xarray*temp_labels;};struct _tuple15{struct _tuple1*f1;struct _tuple1*f2;};
# 123
static int Cyc_Toc_qvar_tag_cmp(struct _tuple15*x,struct _tuple15*y){
int i=Cyc_Absyn_qvar_cmp((*x).f1,(*y).f1);
if(i != 0)return i;return
 Cyc_Absyn_qvar_cmp((*x).f2,(*y).f2);}
# 123
static struct Cyc_Toc_TocState*Cyc_Toc_empty_toc_state(struct _RegionHandle*d){
# 131
return({struct Cyc_Toc_TocState*_tmpE=_region_malloc(d,sizeof(*_tmpE));
({struct Cyc_List_List**_tmp61B=({struct Cyc_List_List**_tmp3=_region_malloc(d,sizeof(*_tmp3));*_tmp3=0;_tmp3;});_tmpE->tuple_types=_tmp61B;}),({
struct Cyc_List_List**_tmp61A=({struct Cyc_List_List**_tmp4=_region_malloc(d,sizeof(*_tmp4));*_tmp4=0;_tmp4;});_tmpE->anon_aggr_types=_tmp61A;}),({
struct Cyc_Dict_Dict*_tmp619=({struct Cyc_Dict_Dict*_tmp6=_region_malloc(d,sizeof(*_tmp6));({struct Cyc_Dict_Dict _tmp618=({struct Cyc_Dict_Dict(*_tmp617)(struct _RegionHandle*,int(*cmp)(struct _tuple1*,struct _tuple1*))=({struct Cyc_Dict_Dict(*_tmp5)(struct _RegionHandle*,int(*cmp)(struct _tuple1*,struct _tuple1*))=(struct Cyc_Dict_Dict(*)(struct _RegionHandle*,int(*cmp)(struct _tuple1*,struct _tuple1*)))Cyc_Dict_rempty;_tmp5;});_tmp617(d,Cyc_Absyn_qvar_cmp);});*_tmp6=_tmp618;});_tmp6;});_tmpE->aggrs_so_far=_tmp619;}),({
struct Cyc_List_List**_tmp616=({struct Cyc_List_List**_tmp7=_region_malloc(d,sizeof(*_tmp7));*_tmp7=0;_tmp7;});_tmpE->abs_struct_types=_tmp616;}),({
struct Cyc_Set_Set**_tmp615=({struct Cyc_Set_Set**_tmp9=_region_malloc(d,sizeof(*_tmp9));({struct Cyc_Set_Set*_tmp614=({struct Cyc_Set_Set*(*_tmp613)(struct _RegionHandle*r,int(*cmp)(struct _tuple1*,struct _tuple1*))=({struct Cyc_Set_Set*(*_tmp8)(struct _RegionHandle*r,int(*cmp)(struct _tuple1*,struct _tuple1*))=(struct Cyc_Set_Set*(*)(struct _RegionHandle*r,int(*cmp)(struct _tuple1*,struct _tuple1*)))Cyc_Set_rempty;_tmp8;});_tmp613(d,Cyc_Absyn_qvar_cmp);});*_tmp9=_tmp614;});_tmp9;});_tmpE->datatypes_so_far=_tmp615;}),({
struct Cyc_Dict_Dict*_tmp612=({struct Cyc_Dict_Dict*_tmpB=_region_malloc(d,sizeof(*_tmpB));({struct Cyc_Dict_Dict _tmp611=({struct Cyc_Dict_Dict(*_tmp610)(struct _RegionHandle*,int(*cmp)(struct _tuple1*,struct _tuple1*))=({struct Cyc_Dict_Dict(*_tmpA)(struct _RegionHandle*,int(*cmp)(struct _tuple1*,struct _tuple1*))=(struct Cyc_Dict_Dict(*)(struct _RegionHandle*,int(*cmp)(struct _tuple1*,struct _tuple1*)))Cyc_Dict_rempty;_tmpA;});_tmp610(d,Cyc_Absyn_qvar_cmp);});*_tmpB=_tmp611;});_tmpB;});_tmpE->xdatatypes_so_far=_tmp612;}),({
struct Cyc_Dict_Dict*_tmp60F=({struct Cyc_Dict_Dict*_tmpD=_region_malloc(d,sizeof(*_tmpD));({struct Cyc_Dict_Dict _tmp60E=({struct Cyc_Dict_Dict(*_tmp60D)(struct _RegionHandle*,int(*cmp)(struct _tuple15*,struct _tuple15*))=({struct Cyc_Dict_Dict(*_tmpC)(struct _RegionHandle*,int(*cmp)(struct _tuple15*,struct _tuple15*))=(struct Cyc_Dict_Dict(*)(struct _RegionHandle*,int(*cmp)(struct _tuple15*,struct _tuple15*)))Cyc_Dict_rempty;_tmpC;});_tmp60D(d,Cyc_Toc_qvar_tag_cmp);});*_tmpD=_tmp60E;});_tmpD;});_tmpE->qvar_tags=_tmp60F;}),({
struct Cyc_Xarray_Xarray*_tmp60C=Cyc_Xarray_rcreate_empty(d);_tmpE->temp_labels=_tmp60C;});_tmpE;});}struct Cyc_Toc_TocStateWrap{struct Cyc_Core_DynamicRegion*dyn;struct Cyc_Toc_TocState*state;};
# 149
static struct Cyc_Toc_TocStateWrap*Cyc_Toc_toc_state=0;struct _tuple16{struct Cyc_Toc_TocState*f1;void*f2;};
# 155
static void*Cyc_Toc_use_toc_state(void*arg,void*(*f)(struct _RegionHandle*,struct _tuple16*)){
# 158
struct Cyc_Toc_TocStateWrap*ts=0;
({struct Cyc_Toc_TocStateWrap*_tmpF=ts;struct Cyc_Toc_TocStateWrap*_tmp10=Cyc_Toc_toc_state;ts=_tmp10;Cyc_Toc_toc_state=_tmpF;});{
struct Cyc_Toc_TocStateWrap _stmttmp0=*((struct Cyc_Toc_TocStateWrap*)_check_null(ts));struct Cyc_Toc_TocState*_tmp12;struct Cyc_Core_DynamicRegion*_tmp11;_LL1: _tmp11=_stmttmp0.dyn;_tmp12=_stmttmp0.state;_LL2: {struct Cyc_Core_DynamicRegion*dyn=_tmp11;struct Cyc_Toc_TocState*s=_tmp12;
struct _fat_ptr dyn2=Cyc_Core_alias_refptr(_tag_fat(dyn,sizeof(struct Cyc_Core_DynamicRegion),1U));
({struct Cyc_Toc_TocStateWrap _tmp61C=({struct Cyc_Toc_TocStateWrap _tmp5D5;_tmp5D5.dyn=dyn,_tmp5D5.state=s;_tmp5D5;});*ts=_tmp61C;});
({struct Cyc_Toc_TocStateWrap*_tmp13=ts;struct Cyc_Toc_TocStateWrap*_tmp14=Cyc_Toc_toc_state;ts=_tmp14;Cyc_Toc_toc_state=_tmp13;});{
void*res;
{struct _RegionHandle*h=&((struct Cyc_Core_DynamicRegion*)_check_null(_untag_fat_ptr(dyn2,sizeof(struct Cyc_Core_DynamicRegion),1U)))->h;
{struct _tuple16 env=({struct _tuple16 _tmp5D6;_tmp5D6.f1=s,_tmp5D6.f2=arg;_tmp5D6;});
res=f(h,& env);}
# 166
;}
# 168
Cyc_Core_free_rckey((struct Cyc_Core_DynamicRegion*)_untag_fat_ptr(dyn2,sizeof(struct Cyc_Core_DynamicRegion),1U));
return res;}}}}struct _tuple17{struct _tuple1*f1;void*(*f2)(struct _tuple1*);};struct _tuple18{struct Cyc_Toc_TocState*f1;struct _tuple17*f2;};struct _tuple19{struct Cyc_Absyn_Aggrdecl*f1;void*f2;};
# 155
static void*Cyc_Toc_aggrdecl_type_body(struct _RegionHandle*d,struct _tuple18*env){
# 175
struct _tuple18 _stmttmp1=*env;void*(*_tmp18)(struct _tuple1*);struct _tuple1*_tmp17;struct Cyc_Toc_TocState*_tmp16;_LL1: _tmp16=_stmttmp1.f1;_tmp17=(_stmttmp1.f2)->f1;_tmp18=(_stmttmp1.f2)->f2;_LL2: {struct Cyc_Toc_TocState*s=_tmp16;struct _tuple1*q=_tmp17;void*(*type_maker)(struct _tuple1*)=_tmp18;
struct _tuple19**v=({struct _tuple19**(*_tmp61E)(struct Cyc_Dict_Dict d,struct _tuple1*k)=({struct _tuple19**(*_tmp19)(struct Cyc_Dict_Dict d,struct _tuple1*k)=(struct _tuple19**(*)(struct Cyc_Dict_Dict d,struct _tuple1*k))Cyc_Dict_lookup_opt;_tmp19;});struct Cyc_Dict_Dict _tmp61D=*s->aggrs_so_far;_tmp61E(_tmp61D,q);});
return v == 0?type_maker(q):(*(*v)).f2;}}
# 155
static void*Cyc_Toc_aggrdecl_type(struct _tuple1*q,void*(*type_maker)(struct _tuple1*)){
# 181
struct _tuple17 env=({struct _tuple17 _tmp5D7;_tmp5D7.f1=q,_tmp5D7.f2=type_maker;_tmp5D7;});
return({void*(*_tmp1A)(struct _tuple17*arg,void*(*f)(struct _RegionHandle*,struct _tuple18*))=(void*(*)(struct _tuple17*arg,void*(*f)(struct _RegionHandle*,struct _tuple18*)))Cyc_Toc_use_toc_state;_tmp1A;})(& env,Cyc_Toc_aggrdecl_type_body);}static char _tmp1B[5U]="curr";
# 155
static struct _fat_ptr Cyc_Toc_curr_string={_tmp1B,_tmp1B,_tmp1B + 5U};
# 193 "toc.cyc"
static struct _fat_ptr*Cyc_Toc_curr_sp=& Cyc_Toc_curr_string;static char _tmp1C[4U]="tag";
static struct _fat_ptr Cyc_Toc_tag_string={_tmp1C,_tmp1C,_tmp1C + 4U};static struct _fat_ptr*Cyc_Toc_tag_sp=& Cyc_Toc_tag_string;static char _tmp1D[4U]="val";
static struct _fat_ptr Cyc_Toc_val_string={_tmp1D,_tmp1D,_tmp1D + 4U};static struct _fat_ptr*Cyc_Toc_val_sp=& Cyc_Toc_val_string;static char _tmp1E[14U]="_handler_cons";
static struct _fat_ptr Cyc_Toc__handler_cons_string={_tmp1E,_tmp1E,_tmp1E + 14U};static struct _fat_ptr*Cyc_Toc__handler_cons_sp=& Cyc_Toc__handler_cons_string;static char _tmp1F[8U]="handler";
static struct _fat_ptr Cyc_Toc_handler_string={_tmp1F,_tmp1F,_tmp1F + 8U};static struct _fat_ptr*Cyc_Toc_handler_sp=& Cyc_Toc_handler_string;static char _tmp20[14U]="_RegionHandle";
static struct _fat_ptr Cyc_Toc__RegionHandle_string={_tmp20,_tmp20,_tmp20 + 14U};static struct _fat_ptr*Cyc_Toc__RegionHandle_sp=& Cyc_Toc__RegionHandle_string;static char _tmp21[7U]="_throw";
# 212 "toc.cyc"
static struct _fat_ptr Cyc_Toc__throw_str={_tmp21,_tmp21,_tmp21 + 7U};static struct _tuple1 Cyc_Toc__throw_pr={{.Loc_n={4,0}},& Cyc_Toc__throw_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__throw_bnd={0U,& Cyc_Toc__throw_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__throw_re={1U,(void*)& Cyc_Toc__throw_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__throw_ev={0,(void*)& Cyc_Toc__throw_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__throw_e=& Cyc_Toc__throw_ev;static char _tmp22[7U]="setjmp";
static struct _fat_ptr Cyc_Toc_setjmp_str={_tmp22,_tmp22,_tmp22 + 7U};static struct _tuple1 Cyc_Toc_setjmp_pr={{.Loc_n={4,0}},& Cyc_Toc_setjmp_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc_setjmp_bnd={0U,& Cyc_Toc_setjmp_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc_setjmp_re={1U,(void*)& Cyc_Toc_setjmp_bnd};static struct Cyc_Absyn_Exp Cyc_Toc_setjmp_ev={0,(void*)& Cyc_Toc_setjmp_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc_setjmp_e=& Cyc_Toc_setjmp_ev;static char _tmp23[14U]="_push_handler";
static struct _fat_ptr Cyc_Toc__push_handler_str={_tmp23,_tmp23,_tmp23 + 14U};static struct _tuple1 Cyc_Toc__push_handler_pr={{.Loc_n={4,0}},& Cyc_Toc__push_handler_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__push_handler_bnd={0U,& Cyc_Toc__push_handler_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__push_handler_re={1U,(void*)& Cyc_Toc__push_handler_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__push_handler_ev={0,(void*)& Cyc_Toc__push_handler_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__push_handler_e=& Cyc_Toc__push_handler_ev;static char _tmp24[13U]="_pop_handler";
static struct _fat_ptr Cyc_Toc__pop_handler_str={_tmp24,_tmp24,_tmp24 + 13U};static struct _tuple1 Cyc_Toc__pop_handler_pr={{.Loc_n={4,0}},& Cyc_Toc__pop_handler_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__pop_handler_bnd={0U,& Cyc_Toc__pop_handler_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__pop_handler_re={1U,(void*)& Cyc_Toc__pop_handler_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__pop_handler_ev={0,(void*)& Cyc_Toc__pop_handler_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__pop_handler_e=& Cyc_Toc__pop_handler_ev;static char _tmp25[12U]="_exn_thrown";
static struct _fat_ptr Cyc_Toc__exn_thrown_str={_tmp25,_tmp25,_tmp25 + 12U};static struct _tuple1 Cyc_Toc__exn_thrown_pr={{.Loc_n={4,0}},& Cyc_Toc__exn_thrown_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__exn_thrown_bnd={0U,& Cyc_Toc__exn_thrown_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__exn_thrown_re={1U,(void*)& Cyc_Toc__exn_thrown_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__exn_thrown_ev={0,(void*)& Cyc_Toc__exn_thrown_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__exn_thrown_e=& Cyc_Toc__exn_thrown_ev;static char _tmp26[14U]="_npop_handler";
static struct _fat_ptr Cyc_Toc__npop_handler_str={_tmp26,_tmp26,_tmp26 + 14U};static struct _tuple1 Cyc_Toc__npop_handler_pr={{.Loc_n={4,0}},& Cyc_Toc__npop_handler_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__npop_handler_bnd={0U,& Cyc_Toc__npop_handler_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__npop_handler_re={1U,(void*)& Cyc_Toc__npop_handler_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__npop_handler_ev={0,(void*)& Cyc_Toc__npop_handler_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__npop_handler_e=& Cyc_Toc__npop_handler_ev;static char _tmp27[12U]="_check_null";
static struct _fat_ptr Cyc_Toc__check_null_str={_tmp27,_tmp27,_tmp27 + 12U};static struct _tuple1 Cyc_Toc__check_null_pr={{.Loc_n={4,0}},& Cyc_Toc__check_null_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__check_null_bnd={0U,& Cyc_Toc__check_null_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__check_null_re={1U,(void*)& Cyc_Toc__check_null_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__check_null_ev={0,(void*)& Cyc_Toc__check_null_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__check_null_e=& Cyc_Toc__check_null_ev;static char _tmp28[28U]="_check_known_subscript_null";
static struct _fat_ptr Cyc_Toc__check_known_subscript_null_str={_tmp28,_tmp28,_tmp28 + 28U};static struct _tuple1 Cyc_Toc__check_known_subscript_null_pr={{.Loc_n={4,0}},& Cyc_Toc__check_known_subscript_null_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__check_known_subscript_null_bnd={0U,& Cyc_Toc__check_known_subscript_null_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__check_known_subscript_null_re={1U,(void*)& Cyc_Toc__check_known_subscript_null_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__check_known_subscript_null_ev={0,(void*)& Cyc_Toc__check_known_subscript_null_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__check_known_subscript_null_e=& Cyc_Toc__check_known_subscript_null_ev;static char _tmp29[31U]="_check_known_subscript_notnull";
static struct _fat_ptr Cyc_Toc__check_known_subscript_notnull_str={_tmp29,_tmp29,_tmp29 + 31U};static struct _tuple1 Cyc_Toc__check_known_subscript_notnull_pr={{.Loc_n={4,0}},& Cyc_Toc__check_known_subscript_notnull_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__check_known_subscript_notnull_bnd={0U,& Cyc_Toc__check_known_subscript_notnull_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__check_known_subscript_notnull_re={1U,(void*)& Cyc_Toc__check_known_subscript_notnull_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__check_known_subscript_notnull_ev={0,(void*)& Cyc_Toc__check_known_subscript_notnull_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__check_known_subscript_notnull_e=& Cyc_Toc__check_known_subscript_notnull_ev;static char _tmp2A[21U]="_check_fat_subscript";
static struct _fat_ptr Cyc_Toc__check_fat_subscript_str={_tmp2A,_tmp2A,_tmp2A + 21U};static struct _tuple1 Cyc_Toc__check_fat_subscript_pr={{.Loc_n={4,0}},& Cyc_Toc__check_fat_subscript_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__check_fat_subscript_bnd={0U,& Cyc_Toc__check_fat_subscript_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__check_fat_subscript_re={1U,(void*)& Cyc_Toc__check_fat_subscript_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__check_fat_subscript_ev={0,(void*)& Cyc_Toc__check_fat_subscript_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__check_fat_subscript_e=& Cyc_Toc__check_fat_subscript_ev;static char _tmp2B[9U]="_fat_ptr";
static struct _fat_ptr Cyc_Toc__fat_ptr_str={_tmp2B,_tmp2B,_tmp2B + 9U};static struct _tuple1 Cyc_Toc__fat_ptr_pr={{.Loc_n={4,0}},& Cyc_Toc__fat_ptr_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__fat_ptr_bnd={0U,& Cyc_Toc__fat_ptr_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__fat_ptr_re={1U,(void*)& Cyc_Toc__fat_ptr_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__fat_ptr_ev={0,(void*)& Cyc_Toc__fat_ptr_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__fat_ptr_e=& Cyc_Toc__fat_ptr_ev;static char _tmp2C[9U]="_tag_fat";
static struct _fat_ptr Cyc_Toc__tag_fat_str={_tmp2C,_tmp2C,_tmp2C + 9U};static struct _tuple1 Cyc_Toc__tag_fat_pr={{.Loc_n={4,0}},& Cyc_Toc__tag_fat_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__tag_fat_bnd={0U,& Cyc_Toc__tag_fat_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__tag_fat_re={1U,(void*)& Cyc_Toc__tag_fat_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__tag_fat_ev={0,(void*)& Cyc_Toc__tag_fat_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__tag_fat_e=& Cyc_Toc__tag_fat_ev;static char _tmp2D[15U]="_untag_fat_ptr";
static struct _fat_ptr Cyc_Toc__untag_fat_ptr_str={_tmp2D,_tmp2D,_tmp2D + 15U};static struct _tuple1 Cyc_Toc__untag_fat_ptr_pr={{.Loc_n={4,0}},& Cyc_Toc__untag_fat_ptr_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__untag_fat_ptr_bnd={0U,& Cyc_Toc__untag_fat_ptr_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__untag_fat_ptr_re={1U,(void*)& Cyc_Toc__untag_fat_ptr_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__untag_fat_ptr_ev={0,(void*)& Cyc_Toc__untag_fat_ptr_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__untag_fat_ptr_e=& Cyc_Toc__untag_fat_ptr_ev;static char _tmp2E[14U]="_get_fat_size";
static struct _fat_ptr Cyc_Toc__get_fat_size_str={_tmp2E,_tmp2E,_tmp2E + 14U};static struct _tuple1 Cyc_Toc__get_fat_size_pr={{.Loc_n={4,0}},& Cyc_Toc__get_fat_size_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__get_fat_size_bnd={0U,& Cyc_Toc__get_fat_size_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__get_fat_size_re={1U,(void*)& Cyc_Toc__get_fat_size_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__get_fat_size_ev={0,(void*)& Cyc_Toc__get_fat_size_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__get_fat_size_e=& Cyc_Toc__get_fat_size_ev;static char _tmp2F[19U]="_get_zero_arr_size";
static struct _fat_ptr Cyc_Toc__get_zero_arr_size_str={_tmp2F,_tmp2F,_tmp2F + 19U};static struct _tuple1 Cyc_Toc__get_zero_arr_size_pr={{.Loc_n={4,0}},& Cyc_Toc__get_zero_arr_size_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__get_zero_arr_size_bnd={0U,& Cyc_Toc__get_zero_arr_size_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__get_zero_arr_size_re={1U,(void*)& Cyc_Toc__get_zero_arr_size_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__get_zero_arr_size_ev={0,(void*)& Cyc_Toc__get_zero_arr_size_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__get_zero_arr_size_e=& Cyc_Toc__get_zero_arr_size_ev;static char _tmp30[24U]="_get_zero_arr_size_char";
static struct _fat_ptr Cyc_Toc__get_zero_arr_size_char_str={_tmp30,_tmp30,_tmp30 + 24U};static struct _tuple1 Cyc_Toc__get_zero_arr_size_char_pr={{.Loc_n={4,0}},& Cyc_Toc__get_zero_arr_size_char_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__get_zero_arr_size_char_bnd={0U,& Cyc_Toc__get_zero_arr_size_char_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__get_zero_arr_size_char_re={1U,(void*)& Cyc_Toc__get_zero_arr_size_char_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__get_zero_arr_size_char_ev={0,(void*)& Cyc_Toc__get_zero_arr_size_char_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__get_zero_arr_size_char_e=& Cyc_Toc__get_zero_arr_size_char_ev;static char _tmp31[25U]="_get_zero_arr_size_short";
static struct _fat_ptr Cyc_Toc__get_zero_arr_size_short_str={_tmp31,_tmp31,_tmp31 + 25U};static struct _tuple1 Cyc_Toc__get_zero_arr_size_short_pr={{.Loc_n={4,0}},& Cyc_Toc__get_zero_arr_size_short_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__get_zero_arr_size_short_bnd={0U,& Cyc_Toc__get_zero_arr_size_short_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__get_zero_arr_size_short_re={1U,(void*)& Cyc_Toc__get_zero_arr_size_short_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__get_zero_arr_size_short_ev={0,(void*)& Cyc_Toc__get_zero_arr_size_short_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__get_zero_arr_size_short_e=& Cyc_Toc__get_zero_arr_size_short_ev;static char _tmp32[23U]="_get_zero_arr_size_int";
static struct _fat_ptr Cyc_Toc__get_zero_arr_size_int_str={_tmp32,_tmp32,_tmp32 + 23U};static struct _tuple1 Cyc_Toc__get_zero_arr_size_int_pr={{.Loc_n={4,0}},& Cyc_Toc__get_zero_arr_size_int_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__get_zero_arr_size_int_bnd={0U,& Cyc_Toc__get_zero_arr_size_int_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__get_zero_arr_size_int_re={1U,(void*)& Cyc_Toc__get_zero_arr_size_int_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__get_zero_arr_size_int_ev={0,(void*)& Cyc_Toc__get_zero_arr_size_int_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__get_zero_arr_size_int_e=& Cyc_Toc__get_zero_arr_size_int_ev;static char _tmp33[25U]="_get_zero_arr_size_float";
static struct _fat_ptr Cyc_Toc__get_zero_arr_size_float_str={_tmp33,_tmp33,_tmp33 + 25U};static struct _tuple1 Cyc_Toc__get_zero_arr_size_float_pr={{.Loc_n={4,0}},& Cyc_Toc__get_zero_arr_size_float_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__get_zero_arr_size_float_bnd={0U,& Cyc_Toc__get_zero_arr_size_float_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__get_zero_arr_size_float_re={1U,(void*)& Cyc_Toc__get_zero_arr_size_float_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__get_zero_arr_size_float_ev={0,(void*)& Cyc_Toc__get_zero_arr_size_float_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__get_zero_arr_size_float_e=& Cyc_Toc__get_zero_arr_size_float_ev;static char _tmp34[26U]="_get_zero_arr_size_double";
static struct _fat_ptr Cyc_Toc__get_zero_arr_size_double_str={_tmp34,_tmp34,_tmp34 + 26U};static struct _tuple1 Cyc_Toc__get_zero_arr_size_double_pr={{.Loc_n={4,0}},& Cyc_Toc__get_zero_arr_size_double_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__get_zero_arr_size_double_bnd={0U,& Cyc_Toc__get_zero_arr_size_double_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__get_zero_arr_size_double_re={1U,(void*)& Cyc_Toc__get_zero_arr_size_double_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__get_zero_arr_size_double_ev={0,(void*)& Cyc_Toc__get_zero_arr_size_double_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__get_zero_arr_size_double_e=& Cyc_Toc__get_zero_arr_size_double_ev;static char _tmp35[30U]="_get_zero_arr_size_longdouble";
static struct _fat_ptr Cyc_Toc__get_zero_arr_size_longdouble_str={_tmp35,_tmp35,_tmp35 + 30U};static struct _tuple1 Cyc_Toc__get_zero_arr_size_longdouble_pr={{.Loc_n={4,0}},& Cyc_Toc__get_zero_arr_size_longdouble_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__get_zero_arr_size_longdouble_bnd={0U,& Cyc_Toc__get_zero_arr_size_longdouble_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__get_zero_arr_size_longdouble_re={1U,(void*)& Cyc_Toc__get_zero_arr_size_longdouble_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__get_zero_arr_size_longdouble_ev={0,(void*)& Cyc_Toc__get_zero_arr_size_longdouble_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__get_zero_arr_size_longdouble_e=& Cyc_Toc__get_zero_arr_size_longdouble_ev;static char _tmp36[28U]="_get_zero_arr_size_voidstar";
static struct _fat_ptr Cyc_Toc__get_zero_arr_size_voidstar_str={_tmp36,_tmp36,_tmp36 + 28U};static struct _tuple1 Cyc_Toc__get_zero_arr_size_voidstar_pr={{.Loc_n={4,0}},& Cyc_Toc__get_zero_arr_size_voidstar_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__get_zero_arr_size_voidstar_bnd={0U,& Cyc_Toc__get_zero_arr_size_voidstar_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__get_zero_arr_size_voidstar_re={1U,(void*)& Cyc_Toc__get_zero_arr_size_voidstar_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__get_zero_arr_size_voidstar_ev={0,(void*)& Cyc_Toc__get_zero_arr_size_voidstar_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__get_zero_arr_size_voidstar_e=& Cyc_Toc__get_zero_arr_size_voidstar_ev;static char _tmp37[14U]="_fat_ptr_plus";
static struct _fat_ptr Cyc_Toc__fat_ptr_plus_str={_tmp37,_tmp37,_tmp37 + 14U};static struct _tuple1 Cyc_Toc__fat_ptr_plus_pr={{.Loc_n={4,0}},& Cyc_Toc__fat_ptr_plus_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__fat_ptr_plus_bnd={0U,& Cyc_Toc__fat_ptr_plus_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__fat_ptr_plus_re={1U,(void*)& Cyc_Toc__fat_ptr_plus_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__fat_ptr_plus_ev={0,(void*)& Cyc_Toc__fat_ptr_plus_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__fat_ptr_plus_e=& Cyc_Toc__fat_ptr_plus_ev;static char _tmp38[15U]="_zero_arr_plus";
static struct _fat_ptr Cyc_Toc__zero_arr_plus_str={_tmp38,_tmp38,_tmp38 + 15U};static struct _tuple1 Cyc_Toc__zero_arr_plus_pr={{.Loc_n={4,0}},& Cyc_Toc__zero_arr_plus_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__zero_arr_plus_bnd={0U,& Cyc_Toc__zero_arr_plus_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__zero_arr_plus_re={1U,(void*)& Cyc_Toc__zero_arr_plus_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__zero_arr_plus_ev={0,(void*)& Cyc_Toc__zero_arr_plus_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__zero_arr_plus_e=& Cyc_Toc__zero_arr_plus_ev;static char _tmp39[20U]="_zero_arr_plus_char";
static struct _fat_ptr Cyc_Toc__zero_arr_plus_char_str={_tmp39,_tmp39,_tmp39 + 20U};static struct _tuple1 Cyc_Toc__zero_arr_plus_char_pr={{.Loc_n={4,0}},& Cyc_Toc__zero_arr_plus_char_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__zero_arr_plus_char_bnd={0U,& Cyc_Toc__zero_arr_plus_char_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__zero_arr_plus_char_re={1U,(void*)& Cyc_Toc__zero_arr_plus_char_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__zero_arr_plus_char_ev={0,(void*)& Cyc_Toc__zero_arr_plus_char_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__zero_arr_plus_char_e=& Cyc_Toc__zero_arr_plus_char_ev;static char _tmp3A[21U]="_zero_arr_plus_short";
static struct _fat_ptr Cyc_Toc__zero_arr_plus_short_str={_tmp3A,_tmp3A,_tmp3A + 21U};static struct _tuple1 Cyc_Toc__zero_arr_plus_short_pr={{.Loc_n={4,0}},& Cyc_Toc__zero_arr_plus_short_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__zero_arr_plus_short_bnd={0U,& Cyc_Toc__zero_arr_plus_short_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__zero_arr_plus_short_re={1U,(void*)& Cyc_Toc__zero_arr_plus_short_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__zero_arr_plus_short_ev={0,(void*)& Cyc_Toc__zero_arr_plus_short_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__zero_arr_plus_short_e=& Cyc_Toc__zero_arr_plus_short_ev;static char _tmp3B[19U]="_zero_arr_plus_int";
static struct _fat_ptr Cyc_Toc__zero_arr_plus_int_str={_tmp3B,_tmp3B,_tmp3B + 19U};static struct _tuple1 Cyc_Toc__zero_arr_plus_int_pr={{.Loc_n={4,0}},& Cyc_Toc__zero_arr_plus_int_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__zero_arr_plus_int_bnd={0U,& Cyc_Toc__zero_arr_plus_int_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__zero_arr_plus_int_re={1U,(void*)& Cyc_Toc__zero_arr_plus_int_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__zero_arr_plus_int_ev={0,(void*)& Cyc_Toc__zero_arr_plus_int_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__zero_arr_plus_int_e=& Cyc_Toc__zero_arr_plus_int_ev;static char _tmp3C[21U]="_zero_arr_plus_float";
static struct _fat_ptr Cyc_Toc__zero_arr_plus_float_str={_tmp3C,_tmp3C,_tmp3C + 21U};static struct _tuple1 Cyc_Toc__zero_arr_plus_float_pr={{.Loc_n={4,0}},& Cyc_Toc__zero_arr_plus_float_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__zero_arr_plus_float_bnd={0U,& Cyc_Toc__zero_arr_plus_float_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__zero_arr_plus_float_re={1U,(void*)& Cyc_Toc__zero_arr_plus_float_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__zero_arr_plus_float_ev={0,(void*)& Cyc_Toc__zero_arr_plus_float_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__zero_arr_plus_float_e=& Cyc_Toc__zero_arr_plus_float_ev;static char _tmp3D[22U]="_zero_arr_plus_double";
static struct _fat_ptr Cyc_Toc__zero_arr_plus_double_str={_tmp3D,_tmp3D,_tmp3D + 22U};static struct _tuple1 Cyc_Toc__zero_arr_plus_double_pr={{.Loc_n={4,0}},& Cyc_Toc__zero_arr_plus_double_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__zero_arr_plus_double_bnd={0U,& Cyc_Toc__zero_arr_plus_double_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__zero_arr_plus_double_re={1U,(void*)& Cyc_Toc__zero_arr_plus_double_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__zero_arr_plus_double_ev={0,(void*)& Cyc_Toc__zero_arr_plus_double_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__zero_arr_plus_double_e=& Cyc_Toc__zero_arr_plus_double_ev;static char _tmp3E[26U]="_zero_arr_plus_longdouble";
static struct _fat_ptr Cyc_Toc__zero_arr_plus_longdouble_str={_tmp3E,_tmp3E,_tmp3E + 26U};static struct _tuple1 Cyc_Toc__zero_arr_plus_longdouble_pr={{.Loc_n={4,0}},& Cyc_Toc__zero_arr_plus_longdouble_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__zero_arr_plus_longdouble_bnd={0U,& Cyc_Toc__zero_arr_plus_longdouble_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__zero_arr_plus_longdouble_re={1U,(void*)& Cyc_Toc__zero_arr_plus_longdouble_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__zero_arr_plus_longdouble_ev={0,(void*)& Cyc_Toc__zero_arr_plus_longdouble_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__zero_arr_plus_longdouble_e=& Cyc_Toc__zero_arr_plus_longdouble_ev;static char _tmp3F[24U]="_zero_arr_plus_voidstar";
static struct _fat_ptr Cyc_Toc__zero_arr_plus_voidstar_str={_tmp3F,_tmp3F,_tmp3F + 24U};static struct _tuple1 Cyc_Toc__zero_arr_plus_voidstar_pr={{.Loc_n={4,0}},& Cyc_Toc__zero_arr_plus_voidstar_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__zero_arr_plus_voidstar_bnd={0U,& Cyc_Toc__zero_arr_plus_voidstar_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__zero_arr_plus_voidstar_re={1U,(void*)& Cyc_Toc__zero_arr_plus_voidstar_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__zero_arr_plus_voidstar_ev={0,(void*)& Cyc_Toc__zero_arr_plus_voidstar_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__zero_arr_plus_voidstar_e=& Cyc_Toc__zero_arr_plus_voidstar_ev;static char _tmp40[22U]="_fat_ptr_inplace_plus";
static struct _fat_ptr Cyc_Toc__fat_ptr_inplace_plus_str={_tmp40,_tmp40,_tmp40 + 22U};static struct _tuple1 Cyc_Toc__fat_ptr_inplace_plus_pr={{.Loc_n={4,0}},& Cyc_Toc__fat_ptr_inplace_plus_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__fat_ptr_inplace_plus_bnd={0U,& Cyc_Toc__fat_ptr_inplace_plus_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__fat_ptr_inplace_plus_re={1U,(void*)& Cyc_Toc__fat_ptr_inplace_plus_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__fat_ptr_inplace_plus_ev={0,(void*)& Cyc_Toc__fat_ptr_inplace_plus_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__fat_ptr_inplace_plus_e=& Cyc_Toc__fat_ptr_inplace_plus_ev;static char _tmp41[23U]="_zero_arr_inplace_plus";
static struct _fat_ptr Cyc_Toc__zero_arr_inplace_plus_str={_tmp41,_tmp41,_tmp41 + 23U};static struct _tuple1 Cyc_Toc__zero_arr_inplace_plus_pr={{.Loc_n={4,0}},& Cyc_Toc__zero_arr_inplace_plus_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__zero_arr_inplace_plus_bnd={0U,& Cyc_Toc__zero_arr_inplace_plus_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__zero_arr_inplace_plus_re={1U,(void*)& Cyc_Toc__zero_arr_inplace_plus_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__zero_arr_inplace_plus_ev={0,(void*)& Cyc_Toc__zero_arr_inplace_plus_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__zero_arr_inplace_plus_e=& Cyc_Toc__zero_arr_inplace_plus_ev;static char _tmp42[28U]="_zero_arr_inplace_plus_char";
static struct _fat_ptr Cyc_Toc__zero_arr_inplace_plus_char_str={_tmp42,_tmp42,_tmp42 + 28U};static struct _tuple1 Cyc_Toc__zero_arr_inplace_plus_char_pr={{.Loc_n={4,0}},& Cyc_Toc__zero_arr_inplace_plus_char_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__zero_arr_inplace_plus_char_bnd={0U,& Cyc_Toc__zero_arr_inplace_plus_char_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__zero_arr_inplace_plus_char_re={1U,(void*)& Cyc_Toc__zero_arr_inplace_plus_char_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__zero_arr_inplace_plus_char_ev={0,(void*)& Cyc_Toc__zero_arr_inplace_plus_char_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__zero_arr_inplace_plus_char_e=& Cyc_Toc__zero_arr_inplace_plus_char_ev;static char _tmp43[29U]="_zero_arr_inplace_plus_short";
static struct _fat_ptr Cyc_Toc__zero_arr_inplace_plus_short_str={_tmp43,_tmp43,_tmp43 + 29U};static struct _tuple1 Cyc_Toc__zero_arr_inplace_plus_short_pr={{.Loc_n={4,0}},& Cyc_Toc__zero_arr_inplace_plus_short_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__zero_arr_inplace_plus_short_bnd={0U,& Cyc_Toc__zero_arr_inplace_plus_short_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__zero_arr_inplace_plus_short_re={1U,(void*)& Cyc_Toc__zero_arr_inplace_plus_short_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__zero_arr_inplace_plus_short_ev={0,(void*)& Cyc_Toc__zero_arr_inplace_plus_short_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__zero_arr_inplace_plus_short_e=& Cyc_Toc__zero_arr_inplace_plus_short_ev;static char _tmp44[27U]="_zero_arr_inplace_plus_int";
static struct _fat_ptr Cyc_Toc__zero_arr_inplace_plus_int_str={_tmp44,_tmp44,_tmp44 + 27U};static struct _tuple1 Cyc_Toc__zero_arr_inplace_plus_int_pr={{.Loc_n={4,0}},& Cyc_Toc__zero_arr_inplace_plus_int_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__zero_arr_inplace_plus_int_bnd={0U,& Cyc_Toc__zero_arr_inplace_plus_int_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__zero_arr_inplace_plus_int_re={1U,(void*)& Cyc_Toc__zero_arr_inplace_plus_int_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__zero_arr_inplace_plus_int_ev={0,(void*)& Cyc_Toc__zero_arr_inplace_plus_int_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__zero_arr_inplace_plus_int_e=& Cyc_Toc__zero_arr_inplace_plus_int_ev;static char _tmp45[29U]="_zero_arr_inplace_plus_float";
static struct _fat_ptr Cyc_Toc__zero_arr_inplace_plus_float_str={_tmp45,_tmp45,_tmp45 + 29U};static struct _tuple1 Cyc_Toc__zero_arr_inplace_plus_float_pr={{.Loc_n={4,0}},& Cyc_Toc__zero_arr_inplace_plus_float_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__zero_arr_inplace_plus_float_bnd={0U,& Cyc_Toc__zero_arr_inplace_plus_float_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__zero_arr_inplace_plus_float_re={1U,(void*)& Cyc_Toc__zero_arr_inplace_plus_float_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__zero_arr_inplace_plus_float_ev={0,(void*)& Cyc_Toc__zero_arr_inplace_plus_float_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__zero_arr_inplace_plus_float_e=& Cyc_Toc__zero_arr_inplace_plus_float_ev;static char _tmp46[30U]="_zero_arr_inplace_plus_double";
static struct _fat_ptr Cyc_Toc__zero_arr_inplace_plus_double_str={_tmp46,_tmp46,_tmp46 + 30U};static struct _tuple1 Cyc_Toc__zero_arr_inplace_plus_double_pr={{.Loc_n={4,0}},& Cyc_Toc__zero_arr_inplace_plus_double_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__zero_arr_inplace_plus_double_bnd={0U,& Cyc_Toc__zero_arr_inplace_plus_double_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__zero_arr_inplace_plus_double_re={1U,(void*)& Cyc_Toc__zero_arr_inplace_plus_double_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__zero_arr_inplace_plus_double_ev={0,(void*)& Cyc_Toc__zero_arr_inplace_plus_double_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__zero_arr_inplace_plus_double_e=& Cyc_Toc__zero_arr_inplace_plus_double_ev;static char _tmp47[34U]="_zero_arr_inplace_plus_longdouble";
static struct _fat_ptr Cyc_Toc__zero_arr_inplace_plus_longdouble_str={_tmp47,_tmp47,_tmp47 + 34U};static struct _tuple1 Cyc_Toc__zero_arr_inplace_plus_longdouble_pr={{.Loc_n={4,0}},& Cyc_Toc__zero_arr_inplace_plus_longdouble_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__zero_arr_inplace_plus_longdouble_bnd={0U,& Cyc_Toc__zero_arr_inplace_plus_longdouble_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__zero_arr_inplace_plus_longdouble_re={1U,(void*)& Cyc_Toc__zero_arr_inplace_plus_longdouble_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__zero_arr_inplace_plus_longdouble_ev={0,(void*)& Cyc_Toc__zero_arr_inplace_plus_longdouble_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__zero_arr_inplace_plus_longdouble_e=& Cyc_Toc__zero_arr_inplace_plus_longdouble_ev;static char _tmp48[32U]="_zero_arr_inplace_plus_voidstar";
static struct _fat_ptr Cyc_Toc__zero_arr_inplace_plus_voidstar_str={_tmp48,_tmp48,_tmp48 + 32U};static struct _tuple1 Cyc_Toc__zero_arr_inplace_plus_voidstar_pr={{.Loc_n={4,0}},& Cyc_Toc__zero_arr_inplace_plus_voidstar_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__zero_arr_inplace_plus_voidstar_bnd={0U,& Cyc_Toc__zero_arr_inplace_plus_voidstar_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__zero_arr_inplace_plus_voidstar_re={1U,(void*)& Cyc_Toc__zero_arr_inplace_plus_voidstar_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__zero_arr_inplace_plus_voidstar_ev={0,(void*)& Cyc_Toc__zero_arr_inplace_plus_voidstar_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__zero_arr_inplace_plus_voidstar_e=& Cyc_Toc__zero_arr_inplace_plus_voidstar_ev;static char _tmp49[27U]="_fat_ptr_inplace_plus_post";
static struct _fat_ptr Cyc_Toc__fat_ptr_inplace_plus_post_str={_tmp49,_tmp49,_tmp49 + 27U};static struct _tuple1 Cyc_Toc__fat_ptr_inplace_plus_post_pr={{.Loc_n={4,0}},& Cyc_Toc__fat_ptr_inplace_plus_post_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__fat_ptr_inplace_plus_post_bnd={0U,& Cyc_Toc__fat_ptr_inplace_plus_post_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__fat_ptr_inplace_plus_post_re={1U,(void*)& Cyc_Toc__fat_ptr_inplace_plus_post_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__fat_ptr_inplace_plus_post_ev={0,(void*)& Cyc_Toc__fat_ptr_inplace_plus_post_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__fat_ptr_inplace_plus_post_e=& Cyc_Toc__fat_ptr_inplace_plus_post_ev;static char _tmp4A[28U]="_zero_arr_inplace_plus_post";
static struct _fat_ptr Cyc_Toc__zero_arr_inplace_plus_post_str={_tmp4A,_tmp4A,_tmp4A + 28U};static struct _tuple1 Cyc_Toc__zero_arr_inplace_plus_post_pr={{.Loc_n={4,0}},& Cyc_Toc__zero_arr_inplace_plus_post_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__zero_arr_inplace_plus_post_bnd={0U,& Cyc_Toc__zero_arr_inplace_plus_post_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__zero_arr_inplace_plus_post_re={1U,(void*)& Cyc_Toc__zero_arr_inplace_plus_post_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__zero_arr_inplace_plus_post_ev={0,(void*)& Cyc_Toc__zero_arr_inplace_plus_post_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__zero_arr_inplace_plus_post_e=& Cyc_Toc__zero_arr_inplace_plus_post_ev;static char _tmp4B[33U]="_zero_arr_inplace_plus_post_char";
static struct _fat_ptr Cyc_Toc__zero_arr_inplace_plus_post_char_str={_tmp4B,_tmp4B,_tmp4B + 33U};static struct _tuple1 Cyc_Toc__zero_arr_inplace_plus_post_char_pr={{.Loc_n={4,0}},& Cyc_Toc__zero_arr_inplace_plus_post_char_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__zero_arr_inplace_plus_post_char_bnd={0U,& Cyc_Toc__zero_arr_inplace_plus_post_char_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__zero_arr_inplace_plus_post_char_re={1U,(void*)& Cyc_Toc__zero_arr_inplace_plus_post_char_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__zero_arr_inplace_plus_post_char_ev={0,(void*)& Cyc_Toc__zero_arr_inplace_plus_post_char_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__zero_arr_inplace_plus_post_char_e=& Cyc_Toc__zero_arr_inplace_plus_post_char_ev;static char _tmp4C[34U]="_zero_arr_inplace_plus_post_short";
static struct _fat_ptr Cyc_Toc__zero_arr_inplace_plus_post_short_str={_tmp4C,_tmp4C,_tmp4C + 34U};static struct _tuple1 Cyc_Toc__zero_arr_inplace_plus_post_short_pr={{.Loc_n={4,0}},& Cyc_Toc__zero_arr_inplace_plus_post_short_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__zero_arr_inplace_plus_post_short_bnd={0U,& Cyc_Toc__zero_arr_inplace_plus_post_short_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__zero_arr_inplace_plus_post_short_re={1U,(void*)& Cyc_Toc__zero_arr_inplace_plus_post_short_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__zero_arr_inplace_plus_post_short_ev={0,(void*)& Cyc_Toc__zero_arr_inplace_plus_post_short_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__zero_arr_inplace_plus_post_short_e=& Cyc_Toc__zero_arr_inplace_plus_post_short_ev;static char _tmp4D[32U]="_zero_arr_inplace_plus_post_int";
static struct _fat_ptr Cyc_Toc__zero_arr_inplace_plus_post_int_str={_tmp4D,_tmp4D,_tmp4D + 32U};static struct _tuple1 Cyc_Toc__zero_arr_inplace_plus_post_int_pr={{.Loc_n={4,0}},& Cyc_Toc__zero_arr_inplace_plus_post_int_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__zero_arr_inplace_plus_post_int_bnd={0U,& Cyc_Toc__zero_arr_inplace_plus_post_int_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__zero_arr_inplace_plus_post_int_re={1U,(void*)& Cyc_Toc__zero_arr_inplace_plus_post_int_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__zero_arr_inplace_plus_post_int_ev={0,(void*)& Cyc_Toc__zero_arr_inplace_plus_post_int_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__zero_arr_inplace_plus_post_int_e=& Cyc_Toc__zero_arr_inplace_plus_post_int_ev;static char _tmp4E[34U]="_zero_arr_inplace_plus_post_float";
static struct _fat_ptr Cyc_Toc__zero_arr_inplace_plus_post_float_str={_tmp4E,_tmp4E,_tmp4E + 34U};static struct _tuple1 Cyc_Toc__zero_arr_inplace_plus_post_float_pr={{.Loc_n={4,0}},& Cyc_Toc__zero_arr_inplace_plus_post_float_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__zero_arr_inplace_plus_post_float_bnd={0U,& Cyc_Toc__zero_arr_inplace_plus_post_float_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__zero_arr_inplace_plus_post_float_re={1U,(void*)& Cyc_Toc__zero_arr_inplace_plus_post_float_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__zero_arr_inplace_plus_post_float_ev={0,(void*)& Cyc_Toc__zero_arr_inplace_plus_post_float_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__zero_arr_inplace_plus_post_float_e=& Cyc_Toc__zero_arr_inplace_plus_post_float_ev;static char _tmp4F[35U]="_zero_arr_inplace_plus_post_double";
static struct _fat_ptr Cyc_Toc__zero_arr_inplace_plus_post_double_str={_tmp4F,_tmp4F,_tmp4F + 35U};static struct _tuple1 Cyc_Toc__zero_arr_inplace_plus_post_double_pr={{.Loc_n={4,0}},& Cyc_Toc__zero_arr_inplace_plus_post_double_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__zero_arr_inplace_plus_post_double_bnd={0U,& Cyc_Toc__zero_arr_inplace_plus_post_double_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__zero_arr_inplace_plus_post_double_re={1U,(void*)& Cyc_Toc__zero_arr_inplace_plus_post_double_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__zero_arr_inplace_plus_post_double_ev={0,(void*)& Cyc_Toc__zero_arr_inplace_plus_post_double_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__zero_arr_inplace_plus_post_double_e=& Cyc_Toc__zero_arr_inplace_plus_post_double_ev;static char _tmp50[39U]="_zero_arr_inplace_plus_post_longdouble";
static struct _fat_ptr Cyc_Toc__zero_arr_inplace_plus_post_longdouble_str={_tmp50,_tmp50,_tmp50 + 39U};static struct _tuple1 Cyc_Toc__zero_arr_inplace_plus_post_longdouble_pr={{.Loc_n={4,0}},& Cyc_Toc__zero_arr_inplace_plus_post_longdouble_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__zero_arr_inplace_plus_post_longdouble_bnd={0U,& Cyc_Toc__zero_arr_inplace_plus_post_longdouble_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__zero_arr_inplace_plus_post_longdouble_re={1U,(void*)& Cyc_Toc__zero_arr_inplace_plus_post_longdouble_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__zero_arr_inplace_plus_post_longdouble_ev={0,(void*)& Cyc_Toc__zero_arr_inplace_plus_post_longdouble_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__zero_arr_inplace_plus_post_longdouble_e=& Cyc_Toc__zero_arr_inplace_plus_post_longdouble_ev;static char _tmp51[37U]="_zero_arr_inplace_plus_post_voidstar";
static struct _fat_ptr Cyc_Toc__zero_arr_inplace_plus_post_voidstar_str={_tmp51,_tmp51,_tmp51 + 37U};static struct _tuple1 Cyc_Toc__zero_arr_inplace_plus_post_voidstar_pr={{.Loc_n={4,0}},& Cyc_Toc__zero_arr_inplace_plus_post_voidstar_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__zero_arr_inplace_plus_post_voidstar_bnd={0U,& Cyc_Toc__zero_arr_inplace_plus_post_voidstar_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__zero_arr_inplace_plus_post_voidstar_re={1U,(void*)& Cyc_Toc__zero_arr_inplace_plus_post_voidstar_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__zero_arr_inplace_plus_post_voidstar_ev={0,(void*)& Cyc_Toc__zero_arr_inplace_plus_post_voidstar_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__zero_arr_inplace_plus_post_voidstar_e=& Cyc_Toc__zero_arr_inplace_plus_post_voidstar_ev;static char _tmp52[10U]="_cycalloc";
static struct _fat_ptr Cyc_Toc__cycalloc_str={_tmp52,_tmp52,_tmp52 + 10U};static struct _tuple1 Cyc_Toc__cycalloc_pr={{.Loc_n={4,0}},& Cyc_Toc__cycalloc_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__cycalloc_bnd={0U,& Cyc_Toc__cycalloc_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__cycalloc_re={1U,(void*)& Cyc_Toc__cycalloc_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__cycalloc_ev={0,(void*)& Cyc_Toc__cycalloc_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__cycalloc_e=& Cyc_Toc__cycalloc_ev;static char _tmp53[11U]="_cyccalloc";
static struct _fat_ptr Cyc_Toc__cyccalloc_str={_tmp53,_tmp53,_tmp53 + 11U};static struct _tuple1 Cyc_Toc__cyccalloc_pr={{.Loc_n={4,0}},& Cyc_Toc__cyccalloc_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__cyccalloc_bnd={0U,& Cyc_Toc__cyccalloc_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__cyccalloc_re={1U,(void*)& Cyc_Toc__cyccalloc_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__cyccalloc_ev={0,(void*)& Cyc_Toc__cyccalloc_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__cyccalloc_e=& Cyc_Toc__cyccalloc_ev;static char _tmp54[17U]="_cycalloc_atomic";
static struct _fat_ptr Cyc_Toc__cycalloc_atomic_str={_tmp54,_tmp54,_tmp54 + 17U};static struct _tuple1 Cyc_Toc__cycalloc_atomic_pr={{.Loc_n={4,0}},& Cyc_Toc__cycalloc_atomic_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__cycalloc_atomic_bnd={0U,& Cyc_Toc__cycalloc_atomic_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__cycalloc_atomic_re={1U,(void*)& Cyc_Toc__cycalloc_atomic_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__cycalloc_atomic_ev={0,(void*)& Cyc_Toc__cycalloc_atomic_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__cycalloc_atomic_e=& Cyc_Toc__cycalloc_atomic_ev;static char _tmp55[18U]="_cyccalloc_atomic";
static struct _fat_ptr Cyc_Toc__cyccalloc_atomic_str={_tmp55,_tmp55,_tmp55 + 18U};static struct _tuple1 Cyc_Toc__cyccalloc_atomic_pr={{.Loc_n={4,0}},& Cyc_Toc__cyccalloc_atomic_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__cyccalloc_atomic_bnd={0U,& Cyc_Toc__cyccalloc_atomic_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__cyccalloc_atomic_re={1U,(void*)& Cyc_Toc__cyccalloc_atomic_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__cyccalloc_atomic_ev={0,(void*)& Cyc_Toc__cyccalloc_atomic_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__cyccalloc_atomic_e=& Cyc_Toc__cyccalloc_atomic_ev;static char _tmp56[15U]="_region_malloc";
static struct _fat_ptr Cyc_Toc__region_malloc_str={_tmp56,_tmp56,_tmp56 + 15U};static struct _tuple1 Cyc_Toc__region_malloc_pr={{.Loc_n={4,0}},& Cyc_Toc__region_malloc_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__region_malloc_bnd={0U,& Cyc_Toc__region_malloc_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__region_malloc_re={1U,(void*)& Cyc_Toc__region_malloc_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__region_malloc_ev={0,(void*)& Cyc_Toc__region_malloc_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__region_malloc_e=& Cyc_Toc__region_malloc_ev;static char _tmp57[15U]="_region_calloc";
static struct _fat_ptr Cyc_Toc__region_calloc_str={_tmp57,_tmp57,_tmp57 + 15U};static struct _tuple1 Cyc_Toc__region_calloc_pr={{.Loc_n={4,0}},& Cyc_Toc__region_calloc_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__region_calloc_bnd={0U,& Cyc_Toc__region_calloc_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__region_calloc_re={1U,(void*)& Cyc_Toc__region_calloc_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__region_calloc_ev={0,(void*)& Cyc_Toc__region_calloc_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__region_calloc_e=& Cyc_Toc__region_calloc_ev;static char _tmp58[13U]="_check_times";
static struct _fat_ptr Cyc_Toc__check_times_str={_tmp58,_tmp58,_tmp58 + 13U};static struct _tuple1 Cyc_Toc__check_times_pr={{.Loc_n={4,0}},& Cyc_Toc__check_times_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__check_times_bnd={0U,& Cyc_Toc__check_times_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__check_times_re={1U,(void*)& Cyc_Toc__check_times_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__check_times_ev={0,(void*)& Cyc_Toc__check_times_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__check_times_e=& Cyc_Toc__check_times_ev;static char _tmp59[12U]="_new_region";
static struct _fat_ptr Cyc_Toc__new_region_str={_tmp59,_tmp59,_tmp59 + 12U};static struct _tuple1 Cyc_Toc__new_region_pr={{.Loc_n={4,0}},& Cyc_Toc__new_region_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__new_region_bnd={0U,& Cyc_Toc__new_region_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__new_region_re={1U,(void*)& Cyc_Toc__new_region_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__new_region_ev={0,(void*)& Cyc_Toc__new_region_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__new_region_e=& Cyc_Toc__new_region_ev;static char _tmp5A[13U]="_push_region";
static struct _fat_ptr Cyc_Toc__push_region_str={_tmp5A,_tmp5A,_tmp5A + 13U};static struct _tuple1 Cyc_Toc__push_region_pr={{.Loc_n={4,0}},& Cyc_Toc__push_region_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__push_region_bnd={0U,& Cyc_Toc__push_region_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__push_region_re={1U,(void*)& Cyc_Toc__push_region_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__push_region_ev={0,(void*)& Cyc_Toc__push_region_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__push_region_e=& Cyc_Toc__push_region_ev;static char _tmp5B[12U]="_pop_region";
static struct _fat_ptr Cyc_Toc__pop_region_str={_tmp5B,_tmp5B,_tmp5B + 12U};static struct _tuple1 Cyc_Toc__pop_region_pr={{.Loc_n={4,0}},& Cyc_Toc__pop_region_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__pop_region_bnd={0U,& Cyc_Toc__pop_region_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__pop_region_re={1U,(void*)& Cyc_Toc__pop_region_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__pop_region_ev={0,(void*)& Cyc_Toc__pop_region_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__pop_region_e=& Cyc_Toc__pop_region_ev;static char _tmp5C[19U]="_throw_arraybounds";
static struct _fat_ptr Cyc_Toc__throw_arraybounds_str={_tmp5C,_tmp5C,_tmp5C + 19U};static struct _tuple1 Cyc_Toc__throw_arraybounds_pr={{.Loc_n={4,0}},& Cyc_Toc__throw_arraybounds_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__throw_arraybounds_bnd={0U,& Cyc_Toc__throw_arraybounds_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__throw_arraybounds_re={1U,(void*)& Cyc_Toc__throw_arraybounds_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__throw_arraybounds_ev={0,(void*)& Cyc_Toc__throw_arraybounds_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__throw_arraybounds_e=& Cyc_Toc__throw_arraybounds_ev;static char _tmp5D[23U]="_fat_ptr_decrease_size";
static struct _fat_ptr Cyc_Toc__fat_ptr_decrease_size_str={_tmp5D,_tmp5D,_tmp5D + 23U};static struct _tuple1 Cyc_Toc__fat_ptr_decrease_size_pr={{.Loc_n={4,0}},& Cyc_Toc__fat_ptr_decrease_size_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__fat_ptr_decrease_size_bnd={0U,& Cyc_Toc__fat_ptr_decrease_size_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__fat_ptr_decrease_size_re={1U,(void*)& Cyc_Toc__fat_ptr_decrease_size_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__fat_ptr_decrease_size_ev={0,(void*)& Cyc_Toc__fat_ptr_decrease_size_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__fat_ptr_decrease_size_e=& Cyc_Toc__fat_ptr_decrease_size_ev;static char _tmp5E[13U]="_throw_match";
static struct _fat_ptr Cyc_Toc__throw_match_str={_tmp5E,_tmp5E,_tmp5E + 13U};static struct _tuple1 Cyc_Toc__throw_match_pr={{.Loc_n={4,0}},& Cyc_Toc__throw_match_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__throw_match_bnd={0U,& Cyc_Toc__throw_match_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__throw_match_re={1U,(void*)& Cyc_Toc__throw_match_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__throw_match_ev={0,(void*)& Cyc_Toc__throw_match_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__throw_match_e=& Cyc_Toc__throw_match_ev;static char _tmp5F[9U]="_rethrow";
static struct _fat_ptr Cyc_Toc__rethrow_str={_tmp5F,_tmp5F,_tmp5F + 9U};static struct _tuple1 Cyc_Toc__rethrow_pr={{.Loc_n={4,0}},& Cyc_Toc__rethrow_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__rethrow_bnd={0U,& Cyc_Toc__rethrow_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__rethrow_re={1U,(void*)& Cyc_Toc__rethrow_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__rethrow_ev={0,(void*)& Cyc_Toc__rethrow_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__rethrow_e=& Cyc_Toc__rethrow_ev;static char _tmp60[20U]="_fast_region_malloc";
static struct _fat_ptr Cyc_Toc__fast_region_malloc_str={_tmp60,_tmp60,_tmp60 + 20U};static struct _tuple1 Cyc_Toc__fast_region_malloc_pr={{.Loc_n={4,0}},& Cyc_Toc__fast_region_malloc_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__fast_region_malloc_bnd={0U,& Cyc_Toc__fast_region_malloc_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__fast_region_malloc_re={1U,(void*)& Cyc_Toc__fast_region_malloc_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__fast_region_malloc_ev={0,(void*)& Cyc_Toc__fast_region_malloc_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__fast_region_malloc_e=& Cyc_Toc__fast_region_malloc_ev;static char _tmp61[7U]="_spawn";
static struct _fat_ptr Cyc_Toc__spawn_str={_tmp61,_tmp61,_tmp61 + 7U};static struct _tuple1 Cyc_Toc__spawn_pr={{.Loc_n={4,0}},& Cyc_Toc__spawn_str};static struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct Cyc_Toc__spawn_bnd={0U,& Cyc_Toc__spawn_pr};static struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct Cyc_Toc__spawn_re={1U,(void*)& Cyc_Toc__spawn_bnd};static struct Cyc_Absyn_Exp Cyc_Toc__spawn_ev={0,(void*)& Cyc_Toc__spawn_re,0U,(void*)& Cyc_Absyn_EmptyAnnot_val};static struct Cyc_Absyn_Exp*Cyc_Toc__spawn_e=& Cyc_Toc__spawn_ev;
# 281
struct Cyc_Absyn_Exp*Cyc_Toc_get_exn_thrown_expression(){
static struct Cyc_Absyn_Exp*_get_exn_thrown_e=0;
# 284
if((unsigned)_get_exn_thrown_e)
return(struct Cyc_Absyn_Exp*)_check_null(_get_exn_thrown_e);{
# 284
struct _tuple1*qv=({struct _tuple1*_tmp6C=_cycalloc(sizeof(*_tmp6C));
# 286
((_tmp6C->f1).Abs_n).tag=2U,({struct Cyc_List_List*_tmp623=({struct _fat_ptr*_tmp67[1U];({struct _fat_ptr*_tmp622=({struct _fat_ptr*_tmp69=_cycalloc(sizeof(*_tmp69));({struct _fat_ptr _tmp621=({const char*_tmp68="Core";_tag_fat(_tmp68,sizeof(char),5U);});*_tmp69=_tmp621;});_tmp69;});_tmp67[0]=_tmp622;});Cyc_List_list(_tag_fat(_tmp67,sizeof(struct _fat_ptr*),1U));});((_tmp6C->f1).Abs_n).val=_tmp623;}),({struct _fat_ptr*_tmp620=({struct _fat_ptr*_tmp6B=_cycalloc(sizeof(*_tmp6B));({struct _fat_ptr _tmp61F=({const char*_tmp6A="get_exn_thrown";_tag_fat(_tmp6A,sizeof(char),15U);});*_tmp6B=_tmp61F;});_tmp6B;});_tmp6C->f2=_tmp620;});_tmp6C;});
void*bnd=(void*)({struct Cyc_Absyn_Unresolved_b_Absyn_Binding_struct*_tmp66=_cycalloc(sizeof(*_tmp66));_tmp66->tag=0U,_tmp66->f1=qv;_tmp66;});
struct Cyc_Absyn_Exp*fnname=({struct Cyc_Absyn_Exp*_tmp65=_cycalloc(sizeof(*_tmp65));_tmp65->topt=0,({void*_tmp624=(void*)({struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct*_tmp64=_cycalloc(sizeof(*_tmp64));_tmp64->tag=1U,_tmp64->f1=bnd;_tmp64;});_tmp65->r=_tmp624;}),_tmp65->loc=0U,_tmp65->annot=(void*)& Cyc_Absyn_EmptyAnnot_val;_tmp65;});
void*fncall_re=(void*)({struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct*_tmp63=_cycalloc(sizeof(*_tmp63));_tmp63->tag=10U,_tmp63->f1=fnname,_tmp63->f2=0,_tmp63->f3=0,_tmp63->f4=0;_tmp63;});
_get_exn_thrown_e=({struct Cyc_Absyn_Exp*_tmp62=_cycalloc(sizeof(*_tmp62));_tmp62->topt=0,_tmp62->r=fncall_re,_tmp62->loc=0U,_tmp62->annot=(void*)& Cyc_Absyn_EmptyAnnot_val;_tmp62;});
return(struct Cyc_Absyn_Exp*)_check_null(_get_exn_thrown_e);}}
# 281
static struct Cyc_Absyn_Tqual Cyc_Toc_mt_tq={0,0,0,0,0U};
# 296
void*Cyc_Toc_void_star_type(){
static void*t=0;
if(t == 0)
t=({void*_tmp627=Cyc_Absyn_void_type;void*_tmp626=Cyc_Absyn_heap_rgn_type;struct Cyc_Absyn_Tqual _tmp625=Cyc_Absyn_empty_tqual(0U);Cyc_Absyn_star_type(_tmp627,_tmp626,_tmp625,Cyc_Absyn_false_type);});
# 298
return(void*)_check_null(t);}
# 296
static void*Cyc_Toc_fat_ptr_type(){
# 303
static void*t=0;
if(t == 0)
t=Cyc_Absyn_aggr_type(Cyc_Absyn_UnknownAggr(Cyc_Absyn_StructA,& Cyc_Toc__fat_ptr_pr,0),0);
# 304
return(void*)_check_null(t);}
# 296
static void*Cyc_Toc_rgn_type(){
# 309
static void*r=0;
if(r == 0)
r=({void*_tmp628=Cyc_Absyn_strct(Cyc_Toc__RegionHandle_sp);Cyc_Absyn_cstar_type(_tmp628,Cyc_Toc_mt_tq);});
# 310
return(void*)_check_null(r);}
# 296
static struct Cyc_Absyn_Stmt*Cyc_Toc_skip_stmt_dl(){
# 316
return Cyc_Absyn_skip_stmt(0U);}struct _tuple20{struct Cyc_List_List*f1;struct Cyc_Absyn_Exp*f2;};
# 296
static struct _tuple20*Cyc_Toc_make_field(struct _fat_ptr*name,struct Cyc_Absyn_Exp*e){
# 320
return({struct _tuple20*_tmp6F=_cycalloc(sizeof(*_tmp6F));({struct Cyc_List_List*_tmp62A=({struct Cyc_List_List*_tmp6E=_cycalloc(sizeof(*_tmp6E));({void*_tmp629=(void*)({struct Cyc_Absyn_FieldName_Absyn_Designator_struct*_tmp6D=_cycalloc(sizeof(*_tmp6D));_tmp6D->tag=1U,_tmp6D->f1=name;_tmp6D;});_tmp6E->hd=_tmp629;}),_tmp6E->tl=0;_tmp6E;});_tmp6F->f1=_tmp62A;}),_tmp6F->f2=e;_tmp6F;});}
# 296
static struct Cyc_Absyn_Exp*Cyc_Toc_fncall_exp_dl(struct Cyc_Absyn_Exp*f,struct _fat_ptr args){
# 324
return({struct Cyc_Absyn_Exp*_tmp62B=f;Cyc_Absyn_fncall_exp(_tmp62B,Cyc_List_from_array(args),0U);});}
# 296
static struct Cyc_Absyn_Exp*Cyc_Toc_cast_it(void*t,struct Cyc_Absyn_Exp*e){
# 327
void*_stmttmp2=e->r;void*_tmp70=_stmttmp2;struct Cyc_Absyn_Exp*_tmp71;if(((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp70)->tag == 14U){if(((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp70)->f4 == Cyc_Absyn_No_coercion){_LL1: _tmp71=((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp70)->f2;_LL2: {struct Cyc_Absyn_Exp*e=_tmp71;
return Cyc_Toc_cast_it(t,e);}}else{goto _LL3;}}else{_LL3: _LL4:
 return Cyc_Absyn_cast_exp(t,e,0,Cyc_Absyn_No_coercion,0U);}_LL0:;}
# 296
static void*Cyc_Toc_cast_it_r(void*t,struct Cyc_Absyn_Exp*e){
# 335
return(void*)({struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*_tmp72=_cycalloc(sizeof(*_tmp72));_tmp72->tag=14U,_tmp72->f1=t,_tmp72->f2=e,_tmp72->f3=0,_tmp72->f4=Cyc_Absyn_No_coercion;_tmp72;});}
# 296
static void*Cyc_Toc_deref_exp_r(struct Cyc_Absyn_Exp*e){
# 338
return(void*)({struct Cyc_Absyn_Deref_e_Absyn_Raw_exp_struct*_tmp73=_cycalloc(sizeof(*_tmp73));_tmp73->tag=20U,_tmp73->f1=e;_tmp73;});}
# 296
static void*Cyc_Toc_subscript_exp_r(struct Cyc_Absyn_Exp*e1,struct Cyc_Absyn_Exp*e2){
# 341
return(void*)({struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct*_tmp74=_cycalloc(sizeof(*_tmp74));_tmp74->tag=23U,_tmp74->f1=e1,_tmp74->f2=e2;_tmp74;});}
# 296
static void*Cyc_Toc_stmt_exp_r(struct Cyc_Absyn_Stmt*s){
# 344
return(void*)({struct Cyc_Absyn_StmtExp_e_Absyn_Raw_exp_struct*_tmp75=_cycalloc(sizeof(*_tmp75));_tmp75->tag=38U,_tmp75->f1=s;_tmp75;});}
# 296
static void*Cyc_Toc_sizeoftype_exp_r(void*t){
# 347
return(void*)({struct Cyc_Absyn_Sizeoftype_e_Absyn_Raw_exp_struct*_tmp76=_cycalloc(sizeof(*_tmp76));_tmp76->tag=17U,_tmp76->f1=t;_tmp76;});}
# 296
static void*Cyc_Toc_fncall_exp_r(struct Cyc_Absyn_Exp*e,struct _fat_ptr es){
# 350
return(void*)({struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct*_tmp77=_cycalloc(sizeof(*_tmp77));_tmp77->tag=10U,_tmp77->f1=e,({struct Cyc_List_List*_tmp62C=Cyc_List_from_array(es);_tmp77->f2=_tmp62C;}),_tmp77->f3=0,_tmp77->f4=1;_tmp77;});}
# 296
static void*Cyc_Toc_seq_stmt_r(struct Cyc_Absyn_Stmt*s1,struct Cyc_Absyn_Stmt*s2){
# 353
return(void*)({struct Cyc_Absyn_Seq_s_Absyn_Raw_stmt_struct*_tmp78=_cycalloc(sizeof(*_tmp78));_tmp78->tag=2U,_tmp78->f1=s1,_tmp78->f2=s2;_tmp78;});}
# 296
static void*Cyc_Toc_conditional_exp_r(struct Cyc_Absyn_Exp*e1,struct Cyc_Absyn_Exp*e2,struct Cyc_Absyn_Exp*e3){
# 356
return(void*)({struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*_tmp79=_cycalloc(sizeof(*_tmp79));_tmp79->tag=6U,_tmp79->f1=e1,_tmp79->f2=e2,_tmp79->f3=e3;_tmp79;});}
# 296
static void*Cyc_Toc_aggrmember_exp_r(struct Cyc_Absyn_Exp*e,struct _fat_ptr*n){
# 359
return(void*)({struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct*_tmp7A=_cycalloc(sizeof(*_tmp7A));_tmp7A->tag=21U,_tmp7A->f1=e,_tmp7A->f2=n,_tmp7A->f3=0,_tmp7A->f4=0;_tmp7A;});}
# 296
static void*Cyc_Toc_unresolvedmem_exp_r(struct Cyc_Core_Opt*tdopt,struct Cyc_List_List*ds){
# 364
return(void*)({struct Cyc_Absyn_UnresolvedMem_e_Absyn_Raw_exp_struct*_tmp7B=_cycalloc(sizeof(*_tmp7B));_tmp7B->tag=37U,_tmp7B->f1=tdopt,_tmp7B->f2=ds;_tmp7B;});}
# 296
static void*Cyc_Toc_goto_stmt_r(struct _fat_ptr*v){
# 367
return(void*)({struct Cyc_Absyn_Goto_s_Absyn_Raw_stmt_struct*_tmp7C=_cycalloc(sizeof(*_tmp7C));_tmp7C->tag=8U,_tmp7C->f1=v;_tmp7C;});}
# 296
static struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct Cyc_Toc_zero_exp={0U,{.Int_c={5,{Cyc_Absyn_Signed,0}}}};
# 373
static struct Cyc_Absyn_Exp*Cyc_Toc_member_exp(struct Cyc_Absyn_Exp*e,struct _fat_ptr*f,unsigned loc){
void*_stmttmp3=e->r;void*_tmp7D=_stmttmp3;struct Cyc_Absyn_Exp*_tmp7E;if(((struct Cyc_Absyn_Deref_e_Absyn_Raw_exp_struct*)_tmp7D)->tag == 20U){_LL1: _tmp7E=((struct Cyc_Absyn_Deref_e_Absyn_Raw_exp_struct*)_tmp7D)->f1;_LL2: {struct Cyc_Absyn_Exp*e1=_tmp7E;
return Cyc_Absyn_aggrarrow_exp(e1,f,loc);}}else{_LL3: _LL4:
 return Cyc_Absyn_aggrmember_exp(e,f,loc);}_LL0:;}struct Cyc_Toc_functionSet{struct Cyc_Absyn_Exp*fchar;struct Cyc_Absyn_Exp*fshort;struct Cyc_Absyn_Exp*fint;struct Cyc_Absyn_Exp*ffloat;struct Cyc_Absyn_Exp*fdouble;struct Cyc_Absyn_Exp*flongdouble;struct Cyc_Absyn_Exp*fvoidstar;};
# 401 "toc.cyc"
struct Cyc_Toc_functionSet Cyc_Toc__zero_arr_plus_functionSet={& Cyc_Toc__zero_arr_plus_char_ev,& Cyc_Toc__zero_arr_plus_short_ev,& Cyc_Toc__zero_arr_plus_int_ev,& Cyc_Toc__zero_arr_plus_float_ev,& Cyc_Toc__zero_arr_plus_double_ev,& Cyc_Toc__zero_arr_plus_longdouble_ev,& Cyc_Toc__zero_arr_plus_voidstar_ev};
struct Cyc_Toc_functionSet Cyc_Toc__zero_arr_inplace_plus_functionSet={& Cyc_Toc__zero_arr_inplace_plus_char_ev,& Cyc_Toc__zero_arr_inplace_plus_short_ev,& Cyc_Toc__zero_arr_inplace_plus_int_ev,& Cyc_Toc__zero_arr_inplace_plus_float_ev,& Cyc_Toc__zero_arr_inplace_plus_double_ev,& Cyc_Toc__zero_arr_inplace_plus_longdouble_ev,& Cyc_Toc__zero_arr_inplace_plus_voidstar_ev};
struct Cyc_Toc_functionSet Cyc_Toc__zero_arr_inplace_plus_post_functionSet={& Cyc_Toc__zero_arr_inplace_plus_post_char_ev,& Cyc_Toc__zero_arr_inplace_plus_post_short_ev,& Cyc_Toc__zero_arr_inplace_plus_post_int_ev,& Cyc_Toc__zero_arr_inplace_plus_post_float_ev,& Cyc_Toc__zero_arr_inplace_plus_post_double_ev,& Cyc_Toc__zero_arr_inplace_plus_post_longdouble_ev,& Cyc_Toc__zero_arr_inplace_plus_post_voidstar_ev};
struct Cyc_Toc_functionSet Cyc_Toc__get_zero_arr_size_functionSet={& Cyc_Toc__get_zero_arr_size_char_ev,& Cyc_Toc__get_zero_arr_size_short_ev,& Cyc_Toc__get_zero_arr_size_int_ev,& Cyc_Toc__get_zero_arr_size_float_ev,& Cyc_Toc__get_zero_arr_size_double_ev,& Cyc_Toc__get_zero_arr_size_longdouble_ev,& Cyc_Toc__get_zero_arr_size_voidstar_ev};
# 407
static struct Cyc_Absyn_Exp*Cyc_Toc_getFunctionType(struct Cyc_Toc_functionSet*fS,void*t){
void*_stmttmp4=Cyc_Tcutil_compress(t);void*_tmp7F=_stmttmp4;switch(*((int*)_tmp7F)){case 0U: switch(*((int*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp7F)->f1)){case 1U: switch(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp7F)->f1)->f2){case Cyc_Absyn_Char_sz: _LL1: _LL2:
 return fS->fchar;case Cyc_Absyn_Short_sz: _LL3: _LL4:
 return fS->fshort;case Cyc_Absyn_Int_sz: _LL5: _LL6:
 return fS->fint;default: goto _LLF;}case 2U: switch(((struct Cyc_Absyn_FloatCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp7F)->f1)->f1){case 0U: _LL7: _LL8:
 return fS->ffloat;case 1U: _LL9: _LLA:
 return fS->fdouble;default: _LLB: _LLC:
 return fS->flongdouble;}default: goto _LLF;}case 3U: _LLD: _LLE:
 return fS->fvoidstar;default: _LLF: _LL10:
({struct Cyc_String_pa_PrintArg_struct _tmp83=({struct Cyc_String_pa_PrintArg_struct _tmp5D8;_tmp5D8.tag=0U,({struct _fat_ptr _tmp62D=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t));_tmp5D8.f1=_tmp62D;});_tmp5D8;});void*_tmp80[1U];_tmp80[0]=& _tmp83;({int(*_tmp62F)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp82)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Toc_toc_impos;_tmp82;});struct _fat_ptr _tmp62E=({const char*_tmp81="expression type %s (not int, float, double, or pointer)";_tag_fat(_tmp81,sizeof(char),56U);});_tmp62F(_tmp62E,_tag_fat(_tmp80,sizeof(void*),1U));});});}_LL0:;}
# 407
static struct Cyc_Absyn_Exp*Cyc_Toc_getFunctionRemovePointer(struct Cyc_Toc_functionSet*fS,struct Cyc_Absyn_Exp*arr){
# 420
void*_stmttmp5=Cyc_Tcutil_compress((void*)_check_null(arr->topt));void*_tmp84=_stmttmp5;void*_tmp85;if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp84)->tag == 3U){_LL1: _tmp85=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp84)->f1).elt_type;_LL2: {void*et=_tmp85;
return Cyc_Toc_getFunctionType(fS,et);}}else{_LL3: _LL4:
({void*_tmp86=0U;({int(*_tmp631)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp88)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Toc_toc_impos;_tmp88;});struct _fat_ptr _tmp630=({const char*_tmp87="impossible type (not pointer)";_tag_fat(_tmp87,sizeof(char),30U);});_tmp631(_tmp630,_tag_fat(_tmp86,sizeof(void*),0U));});});}_LL0:;}
# 407
static int Cyc_Toc_is_zero(struct Cyc_Absyn_Exp*e){
# 430
void*_stmttmp6=e->r;void*_tmp89=_stmttmp6;struct Cyc_List_List*_tmp8A;struct Cyc_List_List*_tmp8B;struct Cyc_List_List*_tmp8C;struct Cyc_List_List*_tmp8D;struct Cyc_List_List*_tmp8E;struct Cyc_Absyn_Exp*_tmp8F;long long _tmp90;int _tmp91;short _tmp92;struct _fat_ptr _tmp93;char _tmp94;switch(*((int*)_tmp89)){case 0U: switch(((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmp89)->f1).Null_c).tag){case 2U: _LL1: _tmp94=(((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmp89)->f1).Char_c).val).f2;_LL2: {char c=_tmp94;
return(int)c == (int)'\000';}case 3U: _LL3: _tmp93=((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmp89)->f1).Wchar_c).val;_LL4: {struct _fat_ptr s=_tmp93;
# 433
unsigned long l=Cyc_strlen((struct _fat_ptr)s);
int i=0;
if(l >= (unsigned long)2 &&(int)*((const char*)_check_fat_subscript(s,sizeof(char),0))== (int)'\\'){
if((int)*((const char*)_check_fat_subscript(s,sizeof(char),1))== (int)'0')i=2;else{
if(((int)((const char*)s.curr)[1]== (int)'x' && l >= (unsigned long)3)&&(int)*((const char*)_check_fat_subscript(s,sizeof(char),2))== (int)'0')i=3;else{
return 0;}}
for(0;(unsigned long)i < l;++ i){
if((int)*((const char*)_check_fat_subscript(s,sizeof(char),i))!= (int)'0')return 0;}
# 439
return 1;}else{
# 443
return 0;}}case 4U: _LL5: _tmp92=(((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmp89)->f1).Short_c).val).f2;_LL6: {short i=_tmp92;
return(int)i == 0;}case 5U: _LL7: _tmp91=(((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmp89)->f1).Int_c).val).f2;_LL8: {int i=_tmp91;
return i == 0;}case 6U: _LL9: _tmp90=(((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmp89)->f1).LongLong_c).val).f2;_LLA: {long long i=_tmp90;
return i == (long long)0;}case 1U: _LLD: _LLE:
# 448
 return 1;default: goto _LL1B;}case 2U: _LLB: _LLC:
# 447
 goto _LLE;case 14U: _LLF: _tmp8F=((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp89)->f2;_LL10: {struct Cyc_Absyn_Exp*e1=_tmp8F;
# 449
return Cyc_Toc_is_zero(e1);}case 24U: _LL11: _tmp8E=((struct Cyc_Absyn_Tuple_e_Absyn_Raw_exp_struct*)_tmp89)->f1;_LL12: {struct Cyc_List_List*es=_tmp8E;
return({int(*_tmp632)(int(*pred)(struct Cyc_Absyn_Exp*),struct Cyc_List_List*x)=({int(*_tmp95)(int(*pred)(struct Cyc_Absyn_Exp*),struct Cyc_List_List*x)=(int(*)(int(*pred)(struct Cyc_Absyn_Exp*),struct Cyc_List_List*x))Cyc_List_forall;_tmp95;});_tmp632(Cyc_Toc_is_zero,es);});}case 26U: _LL13: _tmp8D=((struct Cyc_Absyn_Array_e_Absyn_Raw_exp_struct*)_tmp89)->f1;_LL14: {struct Cyc_List_List*dles=_tmp8D;
_tmp8C=dles;goto _LL16;}case 29U: _LL15: _tmp8C=((struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct*)_tmp89)->f3;_LL16: {struct Cyc_List_List*dles=_tmp8C;
_tmp8B=dles;goto _LL18;}case 25U: _LL17: _tmp8B=((struct Cyc_Absyn_CompoundLit_e_Absyn_Raw_exp_struct*)_tmp89)->f2;_LL18: {struct Cyc_List_List*dles=_tmp8B;
_tmp8A=dles;goto _LL1A;}case 37U: _LL19: _tmp8A=((struct Cyc_Absyn_UnresolvedMem_e_Absyn_Raw_exp_struct*)_tmp89)->f2;_LL1A: {struct Cyc_List_List*dles=_tmp8A;
# 455
for(0;dles != 0;dles=dles->tl){
if(!Cyc_Toc_is_zero((*((struct _tuple20*)dles->hd)).f2))return 0;}
# 455
return 1;}default: _LL1B: _LL1C:
# 458
 return 0;}_LL0:;}
# 407
static struct _fat_ptr Cyc_Toc_collapse_qvar(struct _fat_ptr*s,struct _tuple1*x){
# 464
struct _fat_ptr*_tmp97;union Cyc_Absyn_Nmspace _tmp96;_LL1: _tmp96=x->f1;_tmp97=x->f2;_LL2: {union Cyc_Absyn_Nmspace ns=_tmp96;struct _fat_ptr*v=_tmp97;
union Cyc_Absyn_Nmspace _tmp98=ns;struct Cyc_List_List*_tmp99;struct Cyc_List_List*_tmp9A;struct Cyc_List_List*_tmp9B;switch((_tmp98.Abs_n).tag){case 4U: _LL4: _LL5:
 _tmp9B=0;goto _LL7;case 1U: _LL6: _tmp9B=(_tmp98.Rel_n).val;_LL7: {struct Cyc_List_List*vs=_tmp9B;
_tmp9A=vs;goto _LL9;}case 2U: _LL8: _tmp9A=(_tmp98.Abs_n).val;_LL9: {struct Cyc_List_List*vs=_tmp9A;
_tmp99=vs;goto _LLB;}default: _LLA: _tmp99=(_tmp98.C_n).val;_LLB: {struct Cyc_List_List*vs=_tmp99;
# 472
if(vs == 0)
return(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp9E=({struct Cyc_String_pa_PrintArg_struct _tmp5DA;_tmp5DA.tag=0U,_tmp5DA.f1=(struct _fat_ptr)((struct _fat_ptr)*s);_tmp5DA;});struct Cyc_String_pa_PrintArg_struct _tmp9F=({struct Cyc_String_pa_PrintArg_struct _tmp5D9;_tmp5D9.tag=0U,_tmp5D9.f1=(struct _fat_ptr)((struct _fat_ptr)*v);_tmp5D9;});void*_tmp9C[2U];_tmp9C[0]=& _tmp9E,_tmp9C[1]=& _tmp9F;({struct _fat_ptr _tmp633=({const char*_tmp9D="%s_%s_struct";_tag_fat(_tmp9D,sizeof(char),13U);});Cyc_aprintf(_tmp633,_tag_fat(_tmp9C,sizeof(void*),2U));});});{
# 472
struct _RegionHandle _tmpA0=_new_region("r");struct _RegionHandle*r=& _tmpA0;_push_region(r);
# 475
{struct _fat_ptr _tmpA7=(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmpA3=({struct Cyc_String_pa_PrintArg_struct _tmp5DD;_tmp5DD.tag=0U,_tmp5DD.f1=(struct _fat_ptr)((struct _fat_ptr)*s);_tmp5DD;});struct Cyc_String_pa_PrintArg_struct _tmpA4=({struct Cyc_String_pa_PrintArg_struct _tmp5DC;_tmp5DC.tag=0U,({struct _fat_ptr _tmp636=(struct _fat_ptr)((struct _fat_ptr)({struct _RegionHandle*_tmp635=r;struct Cyc_List_List*_tmp634=vs;Cyc_rstr_sepstr(_tmp635,_tmp634,({const char*_tmpA6="_";_tag_fat(_tmpA6,sizeof(char),2U);}));}));_tmp5DC.f1=_tmp636;});_tmp5DC;});struct Cyc_String_pa_PrintArg_struct _tmpA5=({struct Cyc_String_pa_PrintArg_struct _tmp5DB;_tmp5DB.tag=0U,_tmp5DB.f1=(struct _fat_ptr)((struct _fat_ptr)*v);_tmp5DB;});void*_tmpA1[3U];_tmpA1[0]=& _tmpA3,_tmpA1[1]=& _tmpA4,_tmpA1[2]=& _tmpA5;({struct _fat_ptr _tmp637=({const char*_tmpA2="%s_%s_%s_struct";_tag_fat(_tmpA2,sizeof(char),16U);});Cyc_aprintf(_tmp637,_tag_fat(_tmpA1,sizeof(void*),3U));});});_npop_handler(0U);return _tmpA7;};_pop_region(r);}}}_LL3:;}}struct _tuple21{struct Cyc_Toc_TocState*f1;struct _tuple15*f2;};
# 407
static struct _tuple1*Cyc_Toc_collapse_qvars_body(struct _RegionHandle*d,struct _tuple21*env){
# 488
struct _tuple21 _stmttmp7=*env;struct _tuple15*_tmpA9;struct Cyc_Dict_Dict*_tmpA8;_LL1: _tmpA8=(_stmttmp7.f1)->qvar_tags;_tmpA9=_stmttmp7.f2;_LL2: {struct Cyc_Dict_Dict*qvs=_tmpA8;struct _tuple15*pair=_tmpA9;
struct _tuple15 _stmttmp8=*pair;struct _tuple1*_tmpAB;struct _tuple1*_tmpAA;_LL4: _tmpAA=_stmttmp8.f1;_tmpAB=_stmttmp8.f2;_LL5: {struct _tuple1*fieldname=_tmpAA;struct _tuple1*dtname=_tmpAB;
struct _handler_cons _tmpAC;_push_handler(& _tmpAC);{int _tmpAE=0;if(setjmp(_tmpAC.handler))_tmpAE=1;if(!_tmpAE){{struct _tuple1*_tmpB0=({struct _tuple1*(*_tmp639)(struct Cyc_Dict_Dict d,int(*cmp)(struct _tuple15*,struct _tuple15*),struct _tuple15*k)=({struct _tuple1*(*_tmpAF)(struct Cyc_Dict_Dict d,int(*cmp)(struct _tuple15*,struct _tuple15*),struct _tuple15*k)=(struct _tuple1*(*)(struct Cyc_Dict_Dict d,int(*cmp)(struct _tuple15*,struct _tuple15*),struct _tuple15*k))Cyc_Dict_lookup_other;_tmpAF;});struct Cyc_Dict_Dict _tmp638=*qvs;_tmp639(_tmp638,Cyc_Toc_qvar_tag_cmp,pair);});_npop_handler(0U);return _tmpB0;};_pop_handler();}else{void*_tmpAD=(void*)Cyc_Core_get_exn_thrown();void*_tmpB1=_tmpAD;void*_tmpB2;if(((struct Cyc_Dict_Absent_exn_struct*)_tmpB1)->tag == Cyc_Dict_Absent){_LL7: _LL8: {
# 492
struct _tuple15*new_pair=({struct _tuple15*_tmpB8=_cycalloc(sizeof(*_tmpB8));_tmpB8->f1=fieldname,_tmpB8->f2=dtname;_tmpB8;});
struct _fat_ptr*_tmpB4;union Cyc_Absyn_Nmspace _tmpB3;_LLC: _tmpB3=fieldname->f1;_tmpB4=fieldname->f2;_LLD: {union Cyc_Absyn_Nmspace nmspace=_tmpB3;struct _fat_ptr*fieldvar=_tmpB4;
struct _fat_ptr newvar=Cyc_Toc_collapse_qvar(fieldvar,dtname);
struct _tuple1*res=({struct _tuple1*_tmpB7=_cycalloc(sizeof(*_tmpB7));_tmpB7->f1=nmspace,({struct _fat_ptr*_tmp63A=({struct _fat_ptr*_tmpB6=_cycalloc(sizeof(*_tmpB6));*_tmpB6=newvar;_tmpB6;});_tmpB7->f2=_tmp63A;});_tmpB7;});
({struct Cyc_Dict_Dict _tmp63E=({struct Cyc_Dict_Dict(*_tmp63D)(struct Cyc_Dict_Dict d,struct _tuple15*k,struct _tuple1*v)=({struct Cyc_Dict_Dict(*_tmpB5)(struct Cyc_Dict_Dict d,struct _tuple15*k,struct _tuple1*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _tuple15*k,struct _tuple1*v))Cyc_Dict_insert;_tmpB5;});struct Cyc_Dict_Dict _tmp63C=*qvs;struct _tuple15*_tmp63B=new_pair;_tmp63D(_tmp63C,_tmp63B,res);});*qvs=_tmp63E;});
return res;}}}else{_LL9: _tmpB2=_tmpB1;_LLA: {void*exn=_tmpB2;(int)_rethrow(exn);}}_LL6:;}}}}}
# 407
static struct _tuple1*Cyc_Toc_collapse_qvars(struct _tuple1*fieldname,struct _tuple1*dtname){
# 501
struct _tuple15 env=({struct _tuple15 _tmp5DE;_tmp5DE.f1=fieldname,_tmp5DE.f2=dtname;_tmp5DE;});
return({struct _tuple1*(*_tmpB9)(struct _tuple15*arg,struct _tuple1*(*f)(struct _RegionHandle*,struct _tuple21*))=(struct _tuple1*(*)(struct _tuple15*arg,struct _tuple1*(*f)(struct _RegionHandle*,struct _tuple21*)))Cyc_Toc_use_toc_state;_tmpB9;})(& env,Cyc_Toc_collapse_qvars_body);}
# 407
static struct Cyc_Absyn_Aggrdecl*Cyc_Toc_make_c_struct_defn(struct _fat_ptr*name,struct Cyc_List_List*fs){
# 508
return({struct Cyc_Absyn_Aggrdecl*_tmpBC=_cycalloc(sizeof(*_tmpBC));_tmpBC->kind=Cyc_Absyn_StructA,_tmpBC->sc=Cyc_Absyn_Public,_tmpBC->tvs=0,_tmpBC->attributes=0,_tmpBC->expected_mem_kind=0,({
# 510
struct _tuple1*_tmp641=({struct _tuple1*_tmpBA=_cycalloc(sizeof(*_tmpBA));({union Cyc_Absyn_Nmspace _tmp640=Cyc_Absyn_Rel_n(0);_tmpBA->f1=_tmp640;}),_tmpBA->f2=name;_tmpBA;});_tmpBC->name=_tmp641;}),({
struct Cyc_Absyn_AggrdeclImpl*_tmp63F=({struct Cyc_Absyn_AggrdeclImpl*_tmpBB=_cycalloc(sizeof(*_tmpBB));_tmpBB->exist_vars=0,_tmpBB->rgn_po=0,_tmpBB->fields=fs,_tmpBB->tagged=0;_tmpBB;});_tmpBC->impl=_tmp63F;});_tmpBC;});}struct _tuple22{struct Cyc_Toc_TocState*f1;struct Cyc_List_List*f2;};struct _tuple23{void*f1;struct Cyc_List_List*f2;};
# 407
static void*Cyc_Toc_add_tuple_type_body(struct _RegionHandle*d,struct _tuple22*env){
# 519
struct _tuple22 _stmttmp9=*env;struct Cyc_List_List*_tmpBE;struct Cyc_List_List**_tmpBD;_LL1: _tmpBD=(_stmttmp9.f1)->tuple_types;_tmpBE=_stmttmp9.f2;_LL2: {struct Cyc_List_List**tuple_types=_tmpBD;struct Cyc_List_List*tqs0=_tmpBE;
# 521
{struct Cyc_List_List*tts=*tuple_types;for(0;tts != 0;tts=tts->tl){
struct _tuple23*_stmttmpA=(struct _tuple23*)tts->hd;struct Cyc_List_List*_tmpC0;void*_tmpBF;_LL4: _tmpBF=_stmttmpA->f1;_tmpC0=_stmttmpA->f2;_LL5: {void*x=_tmpBF;struct Cyc_List_List*ts=_tmpC0;
struct Cyc_List_List*tqs=tqs0;
for(0;tqs != 0 && ts != 0;(tqs=tqs->tl,ts=ts->tl)){
if(!Cyc_Unify_unify((*((struct _tuple13*)tqs->hd)).f2,(void*)ts->hd))
break;}
# 524
if(
# 527
tqs == 0 && ts == 0)
return x;}}}{
# 532
struct _fat_ptr*xname=({struct _fat_ptr*_tmpCC=_cycalloc(sizeof(*_tmpCC));({struct _fat_ptr _tmp643=(struct _fat_ptr)({struct Cyc_Int_pa_PrintArg_struct _tmpCB=({struct Cyc_Int_pa_PrintArg_struct _tmp5DF;_tmp5DF.tag=1U,_tmp5DF.f1=(unsigned long)Cyc_Toc_tuple_type_counter ++;_tmp5DF;});void*_tmpC9[1U];_tmpC9[0]=& _tmpCB;({struct _fat_ptr _tmp642=({const char*_tmpCA="_tuple%d";_tag_fat(_tmpCA,sizeof(char),9U);});Cyc_aprintf(_tmp642,_tag_fat(_tmpC9,sizeof(void*),1U));});});*_tmpCC=_tmp643;});_tmpCC;});
struct Cyc_List_List*ts=({struct Cyc_List_List*(*_tmp645)(struct _RegionHandle*,void*(*f)(struct _tuple13*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmpC8)(struct _RegionHandle*,void*(*f)(struct _tuple13*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct _RegionHandle*,void*(*f)(struct _tuple13*),struct Cyc_List_List*x))Cyc_List_rmap;_tmpC8;});struct _RegionHandle*_tmp644=d;_tmp645(_tmp644,Cyc_Tcutil_snd_tqt,tqs0);});
struct Cyc_List_List*fs=0;
struct Cyc_List_List*ts2=ts;
{int i=1;for(0;ts2 != 0;(ts2=ts2->tl,i ++)){
fs=({struct Cyc_List_List*_tmpC2=_cycalloc(sizeof(*_tmpC2));({struct Cyc_Absyn_Aggrfield*_tmp647=({struct Cyc_Absyn_Aggrfield*_tmpC1=_cycalloc(sizeof(*_tmpC1));({struct _fat_ptr*_tmp646=Cyc_Absyn_fieldname(i);_tmpC1->name=_tmp646;}),_tmpC1->tq=Cyc_Toc_mt_tq,_tmpC1->type=(void*)ts2->hd,_tmpC1->width=0,_tmpC1->attributes=0,_tmpC1->requires_clause=0;_tmpC1;});_tmpC2->hd=_tmp647;}),_tmpC2->tl=fs;_tmpC2;});}}
fs=Cyc_List_imp_rev(fs);{
struct Cyc_Absyn_Aggrdecl*sd=Cyc_Toc_make_c_struct_defn(xname,fs);
void*ans=Cyc_Absyn_aggr_type(Cyc_Absyn_KnownAggr(({struct Cyc_Absyn_Aggrdecl**_tmpC7=_cycalloc(sizeof(*_tmpC7));*_tmpC7=sd;_tmpC7;})),0);
Cyc_Toc_result_decls=({struct Cyc_List_List*_tmpC4=_cycalloc(sizeof(*_tmpC4));({struct Cyc_Absyn_Decl*_tmp648=Cyc_Absyn_new_decl((void*)({struct Cyc_Absyn_Aggr_d_Absyn_Raw_decl_struct*_tmpC3=_cycalloc(sizeof(*_tmpC3));_tmpC3->tag=5U,_tmpC3->f1=sd;_tmpC3;}),0U);_tmpC4->hd=_tmp648;}),_tmpC4->tl=Cyc_Toc_result_decls;_tmpC4;});
({struct Cyc_List_List*_tmp64A=({struct Cyc_List_List*_tmpC6=_region_malloc(d,sizeof(*_tmpC6));({struct _tuple23*_tmp649=({struct _tuple23*_tmpC5=_region_malloc(d,sizeof(*_tmpC5));_tmpC5->f1=ans,_tmpC5->f2=ts;_tmpC5;});_tmpC6->hd=_tmp649;}),_tmpC6->tl=*tuple_types;_tmpC6;});*tuple_types=_tmp64A;});
return ans;}}}}
# 407
static void*Cyc_Toc_add_tuple_type(struct Cyc_List_List*tqs0){
# 546
return({void*(*_tmp64B)(struct Cyc_List_List*arg,void*(*f)(struct _RegionHandle*,struct _tuple22*))=({void*(*_tmpCD)(struct Cyc_List_List*arg,void*(*f)(struct _RegionHandle*,struct _tuple22*))=(void*(*)(struct Cyc_List_List*arg,void*(*f)(struct _RegionHandle*,struct _tuple22*)))Cyc_Toc_use_toc_state;_tmpCD;});_tmp64B(tqs0,Cyc_Toc_add_tuple_type_body);});}struct _tuple24{enum Cyc_Absyn_AggrKind f1;struct Cyc_List_List*f2;};struct _tuple25{struct Cyc_Toc_TocState*f1;struct _tuple24*f2;};struct _tuple26{void*f1;enum Cyc_Absyn_AggrKind f2;struct Cyc_List_List*f3;};
# 407
static void*Cyc_Toc_add_anon_aggr_type_body(struct _RegionHandle*d,struct _tuple25*env){
# 553
struct Cyc_List_List*_tmpD0;enum Cyc_Absyn_AggrKind _tmpCF;struct Cyc_List_List**_tmpCE;_LL1: _tmpCE=(env->f1)->anon_aggr_types;_tmpCF=(env->f2)->f1;_tmpD0=(env->f2)->f2;_LL2: {struct Cyc_List_List**anon_aggr_types=_tmpCE;enum Cyc_Absyn_AggrKind ak=_tmpCF;struct Cyc_List_List*fs=_tmpD0;
# 555
{struct Cyc_List_List*ts=*anon_aggr_types;for(0;ts != 0;ts=ts->tl){
struct _tuple26*_stmttmpB=(struct _tuple26*)ts->hd;struct Cyc_List_List*_tmpD3;enum Cyc_Absyn_AggrKind _tmpD2;void*_tmpD1;_LL4: _tmpD1=_stmttmpB->f1;_tmpD2=_stmttmpB->f2;_tmpD3=_stmttmpB->f3;_LL5: {void*x=_tmpD1;enum Cyc_Absyn_AggrKind ak2=_tmpD2;struct Cyc_List_List*fs2=_tmpD3;
if((int)ak != (int)ak2)
continue;
# 557
if(!({
# 559
int(*_tmp64D)(int(*cmp)(struct Cyc_Absyn_Aggrfield*,struct Cyc_Absyn_Aggrfield*),struct Cyc_List_List*l1,struct Cyc_List_List*l2)=({int(*_tmpD4)(int(*cmp)(struct Cyc_Absyn_Aggrfield*,struct Cyc_Absyn_Aggrfield*),struct Cyc_List_List*l1,struct Cyc_List_List*l2)=(int(*)(int(*cmp)(struct Cyc_Absyn_Aggrfield*,struct Cyc_Absyn_Aggrfield*),struct Cyc_List_List*l1,struct Cyc_List_List*l2))Cyc_List_list_cmp;_tmpD4;});struct Cyc_List_List*_tmp64C=fs2;_tmp64D(Cyc_Tcutil_aggrfield_cmp,_tmp64C,fs);}))
return x;}}}{
# 564
struct _fat_ptr*xname=({struct _fat_ptr*_tmpDD=_cycalloc(sizeof(*_tmpDD));({struct _fat_ptr _tmp64F=(struct _fat_ptr)({struct Cyc_Int_pa_PrintArg_struct _tmpDC=({struct Cyc_Int_pa_PrintArg_struct _tmp5E0;_tmp5E0.tag=1U,_tmp5E0.f1=(unsigned long)Cyc_Toc_tuple_type_counter ++;_tmp5E0;});void*_tmpDA[1U];_tmpDA[0]=& _tmpDC;({struct _fat_ptr _tmp64E=({const char*_tmpDB="_tuple%d";_tag_fat(_tmpDB,sizeof(char),9U);});Cyc_aprintf(_tmp64E,_tag_fat(_tmpDA,sizeof(void*),1U));});});*_tmpDD=_tmp64F;});_tmpDD;});
struct Cyc_Absyn_Aggrdecl*sd=Cyc_Toc_make_c_struct_defn(xname,fs);
sd->kind=ak;{
void*ans=Cyc_Absyn_aggr_type(Cyc_Absyn_KnownAggr(({struct Cyc_Absyn_Aggrdecl**_tmpD9=_cycalloc(sizeof(*_tmpD9));*_tmpD9=sd;_tmpD9;})),0);
Cyc_Toc_result_decls=({struct Cyc_List_List*_tmpD6=_cycalloc(sizeof(*_tmpD6));({struct Cyc_Absyn_Decl*_tmp650=Cyc_Absyn_new_decl((void*)({struct Cyc_Absyn_Aggr_d_Absyn_Raw_decl_struct*_tmpD5=_cycalloc(sizeof(*_tmpD5));_tmpD5->tag=5U,_tmpD5->f1=sd;_tmpD5;}),0U);_tmpD6->hd=_tmp650;}),_tmpD6->tl=Cyc_Toc_result_decls;_tmpD6;});
({struct Cyc_List_List*_tmp652=({struct Cyc_List_List*_tmpD8=_region_malloc(d,sizeof(*_tmpD8));({struct _tuple26*_tmp651=({struct _tuple26*_tmpD7=_region_malloc(d,sizeof(*_tmpD7));_tmpD7->f1=ans,_tmpD7->f2=ak,_tmpD7->f3=fs;_tmpD7;});_tmpD8->hd=_tmp651;}),_tmpD8->tl=*anon_aggr_types;_tmpD8;});*anon_aggr_types=_tmp652;});
return ans;}}}}
# 407
static void*Cyc_Toc_add_anon_aggr_type(enum Cyc_Absyn_AggrKind ak,struct Cyc_List_List*fs){
# 573
return({void*(*_tmp653)(struct _tuple24*arg,void*(*f)(struct _RegionHandle*,struct _tuple25*))=({void*(*_tmpDE)(struct _tuple24*arg,void*(*f)(struct _RegionHandle*,struct _tuple25*))=(void*(*)(struct _tuple24*arg,void*(*f)(struct _RegionHandle*,struct _tuple25*)))Cyc_Toc_use_toc_state;_tmpDE;});_tmp653(({struct _tuple24*_tmpDF=_cycalloc(sizeof(*_tmpDF));_tmpDF->f1=ak,_tmpDF->f2=fs;_tmpDF;}),Cyc_Toc_add_anon_aggr_type_body);});}struct _tuple27{struct _tuple1*f1;struct Cyc_List_List*f2;struct Cyc_List_List*f3;struct Cyc_List_List*f4;};struct _tuple28{struct Cyc_Toc_TocState*f1;struct _tuple27*f2;};struct _tuple29{struct _tuple1*f1;struct Cyc_List_List*f2;void*f3;};
# 407
static void*Cyc_Toc_add_struct_type_body(struct _RegionHandle*d,struct _tuple28*env){
# 590
struct _tuple28 _stmttmpC=*env;struct Cyc_List_List*_tmpE4;struct Cyc_List_List*_tmpE3;struct Cyc_List_List*_tmpE2;struct _tuple1*_tmpE1;struct Cyc_List_List**_tmpE0;_LL1: _tmpE0=(_stmttmpC.f1)->abs_struct_types;_tmpE1=(_stmttmpC.f2)->f1;_tmpE2=(_stmttmpC.f2)->f2;_tmpE3=(_stmttmpC.f2)->f3;_tmpE4=(_stmttmpC.f2)->f4;_LL2: {struct Cyc_List_List**abs_struct_types=_tmpE0;struct _tuple1*struct_name=_tmpE1;struct Cyc_List_List*type_vars=_tmpE2;struct Cyc_List_List*type_args=_tmpE3;struct Cyc_List_List*fields=_tmpE4;
# 594
{struct Cyc_List_List*tts=*abs_struct_types;for(0;tts != 0;tts=tts->tl){
struct _tuple29*_stmttmpD=(struct _tuple29*)tts->hd;void*_tmpE7;struct Cyc_List_List*_tmpE6;struct _tuple1*_tmpE5;_LL4: _tmpE5=_stmttmpD->f1;_tmpE6=_stmttmpD->f2;_tmpE7=_stmttmpD->f3;_LL5: {struct _tuple1*x=_tmpE5;struct Cyc_List_List*ts2=_tmpE6;void*t=_tmpE7;
if(Cyc_Absyn_qvar_cmp(x,struct_name)== 0 &&({
int _tmp654=Cyc_List_length(type_args);_tmp654 == Cyc_List_length(ts2);})){
int okay=1;
{struct Cyc_List_List*ts=type_args;for(0;ts != 0;(ts=ts->tl,ts2=ts2->tl)){
void*t=(void*)ts->hd;
void*t2=(void*)((struct Cyc_List_List*)_check_null(ts2))->hd;
{struct Cyc_Absyn_Kind*_stmttmpE=Cyc_Tcutil_type_kind(t);struct Cyc_Absyn_Kind*_tmpE8=_stmttmpE;switch(((struct Cyc_Absyn_Kind*)_tmpE8)->kind){case Cyc_Absyn_EffKind: _LL7: _LL8:
 goto _LLA;case Cyc_Absyn_RgnKind: _LL9: _LLA:
 continue;default: _LLB: _LLC:
# 607
 if(Cyc_Unify_unify(t,t2)||({void*_tmp655=Cyc_Toc_typ_to_c(t);Cyc_Unify_unify(_tmp655,Cyc_Toc_typ_to_c(t2));}))
continue;
# 607
okay=0;
# 610
goto _LL6;}_LL6:;}
# 612
break;}}
# 614
if(okay)
return t;}}}}{
# 621
struct _fat_ptr*xname=({struct _fat_ptr*_tmpF7=_cycalloc(sizeof(*_tmpF7));({struct _fat_ptr _tmp657=(struct _fat_ptr)({struct Cyc_Int_pa_PrintArg_struct _tmpF6=({struct Cyc_Int_pa_PrintArg_struct _tmp5E1;_tmp5E1.tag=1U,_tmp5E1.f1=(unsigned long)Cyc_Toc_tuple_type_counter ++;_tmp5E1;});void*_tmpF4[1U];_tmpF4[0]=& _tmpF6;({struct _fat_ptr _tmp656=({const char*_tmpF5="_tuple%d";_tag_fat(_tmpF5,sizeof(char),9U);});Cyc_aprintf(_tmp656,_tag_fat(_tmpF4,sizeof(void*),1U));});});*_tmpF7=_tmp657;});_tmpF7;});
void*x=Cyc_Absyn_strct(xname);
struct Cyc_List_List*fs=0;
# 625
({struct Cyc_List_List*_tmp659=({struct Cyc_List_List*_tmpEA=_region_malloc(d,sizeof(*_tmpEA));({struct _tuple29*_tmp658=({struct _tuple29*_tmpE9=_region_malloc(d,sizeof(*_tmpE9));_tmpE9->f1=struct_name,_tmpE9->f2=type_args,_tmpE9->f3=x;_tmpE9;});_tmpEA->hd=_tmp658;}),_tmpEA->tl=*abs_struct_types;_tmpEA;});*abs_struct_types=_tmp659;});{
# 628
struct _RegionHandle _tmpEB=_new_region("r");struct _RegionHandle*r=& _tmpEB;_push_region(r);
{struct Cyc_List_List*inst=Cyc_List_rzip(r,r,type_vars,type_args);
for(0;fields != 0;fields=fields->tl){
struct Cyc_Absyn_Aggrfield*f=(struct Cyc_Absyn_Aggrfield*)fields->hd;
void*t=f->type;
struct Cyc_List_List*atts=f->attributes;
# 638
if((fields->tl == 0 &&
 Cyc_Tcutil_kind_leq(& Cyc_Tcutil_ak,Cyc_Tcutil_type_kind(t)))&& !
Cyc_Tcutil_is_array_type(t))
atts=({struct Cyc_List_List*_tmpED=_cycalloc(sizeof(*_tmpED));({void*_tmp65A=(void*)({struct Cyc_Absyn_Aligned_att_Absyn_Attribute_struct*_tmpEC=_cycalloc(sizeof(*_tmpEC));_tmpEC->tag=6U,_tmpEC->f1=0;_tmpEC;});_tmpED->hd=_tmp65A;}),_tmpED->tl=atts;_tmpED;});
# 638
t=
# 642
Cyc_Toc_typ_to_c(Cyc_Tcutil_rsubstitute(r,inst,t));
# 645
if(Cyc_Unify_unify(t,Cyc_Absyn_void_type))
t=(void*)({struct Cyc_Absyn_ArrayType_Absyn_Type_struct*_tmpEE=_cycalloc(sizeof(*_tmpEE));_tmpEE->tag=4U,({void*_tmp65D=Cyc_Toc_void_star_type();(_tmpEE->f1).elt_type=_tmp65D;}),({struct Cyc_Absyn_Tqual _tmp65C=Cyc_Absyn_empty_tqual(0U);(_tmpEE->f1).tq=_tmp65C;}),({
struct Cyc_Absyn_Exp*_tmp65B=Cyc_Absyn_uint_exp(0U,0U);(_tmpEE->f1).num_elts=_tmp65B;}),(_tmpEE->f1).zero_term=Cyc_Absyn_false_type,(_tmpEE->f1).zt_loc=0U;_tmpEE;});
# 645
fs=({struct Cyc_List_List*_tmpF0=_cycalloc(sizeof(*_tmpF0));
# 649
({struct Cyc_Absyn_Aggrfield*_tmp65E=({struct Cyc_Absyn_Aggrfield*_tmpEF=_cycalloc(sizeof(*_tmpEF));_tmpEF->name=f->name,_tmpEF->tq=Cyc_Toc_mt_tq,_tmpEF->type=t,_tmpEF->width=f->width,_tmpEF->attributes=atts,_tmpEF->requires_clause=0;_tmpEF;});_tmpF0->hd=_tmp65E;}),_tmpF0->tl=fs;_tmpF0;});}
# 651
fs=Cyc_List_imp_rev(fs);{
struct Cyc_Absyn_Aggrdecl*sd=Cyc_Toc_make_c_struct_defn(xname,fs);
Cyc_Toc_result_decls=({struct Cyc_List_List*_tmpF2=_cycalloc(sizeof(*_tmpF2));({struct Cyc_Absyn_Decl*_tmp65F=Cyc_Absyn_new_decl((void*)({struct Cyc_Absyn_Aggr_d_Absyn_Raw_decl_struct*_tmpF1=_cycalloc(sizeof(*_tmpF1));_tmpF1->tag=5U,_tmpF1->f1=sd;_tmpF1;}),0U);_tmpF2->hd=_tmp65F;}),_tmpF2->tl=Cyc_Toc_result_decls;_tmpF2;});{
void*_tmpF3=x;_npop_handler(0U);return _tmpF3;}}}
# 629
;_pop_region(r);}}}}
# 407
static void*Cyc_Toc_add_struct_type(struct _tuple1*struct_name,struct Cyc_List_List*type_vars,struct Cyc_List_List*type_args,struct Cyc_List_List*fields){
# 661
struct _tuple27 env=({struct _tuple27 _tmp5E2;_tmp5E2.f1=struct_name,_tmp5E2.f2=type_vars,_tmp5E2.f3=type_args,_tmp5E2.f4=fields;_tmp5E2;});
return({void*(*_tmpF8)(struct _tuple27*arg,void*(*f)(struct _RegionHandle*,struct _tuple28*))=(void*(*)(struct _tuple27*arg,void*(*f)(struct _RegionHandle*,struct _tuple28*)))Cyc_Toc_use_toc_state;_tmpF8;})(& env,Cyc_Toc_add_struct_type_body);}
# 407
struct _tuple1*Cyc_Toc_temp_var(){
# 670
return({struct _tuple1*_tmpFD=_cycalloc(sizeof(*_tmpFD));_tmpFD->f1=Cyc_Absyn_Loc_n,({struct _fat_ptr*_tmp662=({struct _fat_ptr*_tmpFC=_cycalloc(sizeof(*_tmpFC));({struct _fat_ptr _tmp661=(struct _fat_ptr)({struct Cyc_Int_pa_PrintArg_struct _tmpFB=({struct Cyc_Int_pa_PrintArg_struct _tmp5E3;_tmp5E3.tag=1U,_tmp5E3.f1=(unsigned)Cyc_Toc_temp_var_counter ++;_tmp5E3;});void*_tmpF9[1U];_tmpF9[0]=& _tmpFB;({struct _fat_ptr _tmp660=({const char*_tmpFA="_tmp%X";_tag_fat(_tmpFA,sizeof(char),7U);});Cyc_aprintf(_tmp660,_tag_fat(_tmpF9,sizeof(void*),1U));});});*_tmpFC=_tmp661;});_tmpFC;});_tmpFD->f2=_tmp662;});_tmpFD;});}struct _tuple30{struct _tuple1*f1;struct Cyc_Absyn_Exp*f2;};
# 407
struct _tuple30 Cyc_Toc_temp_var_and_exp(){
# 673
struct _tuple1*v=Cyc_Toc_temp_var();
return({struct _tuple30 _tmp5E4;_tmp5E4.f1=v,({struct Cyc_Absyn_Exp*_tmp663=Cyc_Absyn_var_exp(v,0U);_tmp5E4.f2=_tmp663;});_tmp5E4;});}struct _tuple31{struct Cyc_Toc_TocState*f1;int f2;};
# 407
static struct _fat_ptr*Cyc_Toc_fresh_label_body(struct _RegionHandle*d,struct _tuple31*env){
# 680
struct _tuple31 _stmttmpF=*env;struct Cyc_Xarray_Xarray*_tmpFE;_LL1: _tmpFE=(_stmttmpF.f1)->temp_labels;_LL2: {struct Cyc_Xarray_Xarray*temp_labels=_tmpFE;
int i=Cyc_Toc_fresh_label_counter ++;
if(({int _tmp664=i;_tmp664 < Cyc_Xarray_length(temp_labels);}))
return({struct _fat_ptr*(*_tmp666)(struct Cyc_Xarray_Xarray*,int)=({struct _fat_ptr*(*_tmpFF)(struct Cyc_Xarray_Xarray*,int)=(struct _fat_ptr*(*)(struct Cyc_Xarray_Xarray*,int))Cyc_Xarray_get;_tmpFF;});struct Cyc_Xarray_Xarray*_tmp665=temp_labels;_tmp666(_tmp665,i);});{
# 682
struct _fat_ptr*res=({struct _fat_ptr*_tmp107=_cycalloc(sizeof(*_tmp107));({
# 684
struct _fat_ptr _tmp668=(struct _fat_ptr)({struct Cyc_Int_pa_PrintArg_struct _tmp106=({struct Cyc_Int_pa_PrintArg_struct _tmp5E5;_tmp5E5.tag=1U,_tmp5E5.f1=(unsigned)i;_tmp5E5;});void*_tmp104[1U];_tmp104[0]=& _tmp106;({struct _fat_ptr _tmp667=({const char*_tmp105="_LL%X";_tag_fat(_tmp105,sizeof(char),6U);});Cyc_aprintf(_tmp667,_tag_fat(_tmp104,sizeof(void*),1U));});});*_tmp107=_tmp668;});_tmp107;});
if(({int _tmp66B=({int(*_tmp66A)(struct Cyc_Xarray_Xarray*,struct _fat_ptr*)=({int(*_tmp100)(struct Cyc_Xarray_Xarray*,struct _fat_ptr*)=(int(*)(struct Cyc_Xarray_Xarray*,struct _fat_ptr*))Cyc_Xarray_add_ind;_tmp100;});struct Cyc_Xarray_Xarray*_tmp669=temp_labels;_tmp66A(_tmp669,res);});_tmp66B != i;}))
({void*_tmp101=0U;({int(*_tmp66D)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp103)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Toc_toc_impos;_tmp103;});struct _fat_ptr _tmp66C=({const char*_tmp102="fresh_label: add_ind returned bad index...";_tag_fat(_tmp102,sizeof(char),43U);});_tmp66D(_tmp66C,_tag_fat(_tmp101,sizeof(void*),0U));});});
# 685
return res;}}}
# 407
static struct _fat_ptr*Cyc_Toc_fresh_label(){
# 690
return({struct _fat_ptr*(*_tmp108)(int arg,struct _fat_ptr*(*f)(struct _RegionHandle*,struct _tuple31*))=(struct _fat_ptr*(*)(int arg,struct _fat_ptr*(*f)(struct _RegionHandle*,struct _tuple31*)))Cyc_Toc_use_toc_state;_tmp108;})(0,Cyc_Toc_fresh_label_body);}
# 407
static struct Cyc_Absyn_Exp*Cyc_Toc_datatype_tag(struct Cyc_Absyn_Datatypedecl*td,struct _tuple1*name){
# 696
int ans=0;
struct Cyc_List_List*fs=(struct Cyc_List_List*)((struct Cyc_Core_Opt*)_check_null(td->fields))->v;
while(Cyc_Absyn_qvar_cmp(name,((struct Cyc_Absyn_Datatypefield*)((struct Cyc_List_List*)_check_null(fs))->hd)->name)!= 0){
++ ans;
fs=fs->tl;}
# 702
return Cyc_Absyn_uint_exp((unsigned)ans,0U);}
# 407
static void Cyc_Toc_enumdecl_to_c(struct Cyc_Absyn_Enumdecl*ed);
# 709
static void Cyc_Toc_aggrdecl_to_c(struct Cyc_Absyn_Aggrdecl*ad);
static void Cyc_Toc_datatypedecl_to_c(struct Cyc_Absyn_Datatypedecl*tud);
static struct _tuple9*Cyc_Toc_arg_to_c(struct _tuple9*a){
return({struct _tuple9*_tmp109=_cycalloc(sizeof(*_tmp109));_tmp109->f1=(*a).f1,_tmp109->f2=(*a).f2,({void*_tmp66E=Cyc_Toc_typ_to_c((*a).f3);_tmp109->f3=_tmp66E;});_tmp109;});}
# 711
static void*Cyc_Toc_typ_to_c_array(void*t){
# 731 "toc.cyc"
void*_stmttmp10=Cyc_Tcutil_compress(t);void*_tmp10A=_stmttmp10;struct Cyc_Absyn_ArrayInfo _tmp10B;if(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp10A)->tag == 4U){_LL1: _tmp10B=((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp10A)->f1;_LL2: {struct Cyc_Absyn_ArrayInfo ai=_tmp10B;
return({void*_tmp66F=Cyc_Toc_typ_to_c_array(ai.elt_type);Cyc_Absyn_cstar_type(_tmp66F,ai.tq);});}}else{_LL3: _LL4:
 return Cyc_Toc_typ_to_c(t);}_LL0:;}
# 711 "toc.cyc"
static struct Cyc_Absyn_Aggrfield*Cyc_Toc_aggrfield_to_c(struct Cyc_Absyn_Aggrfield*f,void*new_type){
# 739 "toc.cyc"
struct Cyc_Absyn_Aggrfield*ans=({struct Cyc_Absyn_Aggrfield*_tmp10C=_cycalloc(sizeof(*_tmp10C));*_tmp10C=*f;_tmp10C;});
ans->type=new_type;
ans->requires_clause=0;
ans->tq=Cyc_Toc_mt_tq;
return ans;}
# 711 "toc.cyc"
static void Cyc_Toc_enumfields_to_c(struct Cyc_List_List*fs){
# 748 "toc.cyc"
return;}
# 711 "toc.cyc"
static int Cyc_Toc_is_boxed_tvar(void*t){
# 752 "toc.cyc"
void*_stmttmp11=Cyc_Tcutil_compress(t);void*_tmp10D=_stmttmp11;struct Cyc_Absyn_Tvar*_tmp10E;if(((struct Cyc_Absyn_VarType_Absyn_Type_struct*)_tmp10D)->tag == 2U){_LL1: _tmp10E=((struct Cyc_Absyn_VarType_Absyn_Type_struct*)_tmp10D)->f1;_LL2: {struct Cyc_Absyn_Tvar*tv=_tmp10E;
return Cyc_Tcutil_kind_leq(Cyc_Tcutil_type_kind(t),& Cyc_Tcutil_tbk);}}else{_LL3: _LL4:
 return 0;}_LL0:;}
# 711 "toc.cyc"
static int Cyc_Toc_is_abstract_type(void*t){
# 758 "toc.cyc"
struct Cyc_Absyn_Kind*_stmttmp12=Cyc_Tcutil_type_kind(t);struct Cyc_Absyn_Kind*_tmp10F=_stmttmp12;if(((struct Cyc_Absyn_Kind*)_tmp10F)->kind == Cyc_Absyn_AnyKind){_LL1: _LL2:
 return 1;}else{_LL3: _LL4:
 return 0;}_LL0:;}
# 711 "toc.cyc"
static int Cyc_Toc_is_void_star(void*t){
# 764 "toc.cyc"
void*_stmttmp13=Cyc_Tcutil_compress(t);void*_tmp110=_stmttmp13;void*_tmp111;if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp110)->tag == 3U){_LL1: _tmp111=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp110)->f1).elt_type;_LL2: {void*t2=_tmp111;
return Cyc_Tcutil_is_void_type(t2);}}else{_LL3: _LL4:
 return 0;}_LL0:;}
# 711 "toc.cyc"
static int Cyc_Toc_is_void_star_or_boxed_tvar(void*t){
# 770 "toc.cyc"
return Cyc_Toc_is_void_star(t)|| Cyc_Toc_is_boxed_tvar(t);}
# 711 "toc.cyc"
static int Cyc_Toc_is_pointer_or_boxed_tvar(void*t){
# 773 "toc.cyc"
return Cyc_Tcutil_is_pointer_type(t)|| Cyc_Toc_is_boxed_tvar(t);}
# 711 "toc.cyc"
void*Cyc_Toc_typ_to_c(void*t){
# 777 "toc.cyc"
void*_tmp112=t;void**_tmp114;struct Cyc_Absyn_Datatypedecl*_tmp113;struct Cyc_Absyn_Enumdecl*_tmp115;struct Cyc_Absyn_Aggrdecl*_tmp116;struct Cyc_Absyn_Exp*_tmp117;struct Cyc_Absyn_Exp*_tmp118;void*_tmp11C;struct Cyc_Absyn_Typedefdecl*_tmp11B;struct Cyc_List_List*_tmp11A;struct _tuple1*_tmp119;struct Cyc_List_List*_tmp11E;enum Cyc_Absyn_AggrKind _tmp11D;struct Cyc_List_List*_tmp11F;struct Cyc_List_List*_tmp125;struct Cyc_Absyn_VarargInfo*_tmp124;int _tmp123;struct Cyc_List_List*_tmp122;void*_tmp121;struct Cyc_Absyn_Tqual _tmp120;unsigned _tmp129;struct Cyc_Absyn_Exp*_tmp128;struct Cyc_Absyn_Tqual _tmp127;void*_tmp126;void*_tmp12C;struct Cyc_Absyn_Tqual _tmp12B;void*_tmp12A;struct Cyc_Absyn_Tvar*_tmp12D;void**_tmp12E;struct Cyc_List_List*_tmp12F;struct _tuple1*_tmp130;struct Cyc_List_List*_tmp133;union Cyc_Absyn_AggrInfo _tmp132;void*_tmp131;struct Cyc_Absyn_Datatypefield*_tmp135;struct Cyc_Absyn_Datatypedecl*_tmp134;switch(*((int*)_tmp112)){case 0U: switch(*((int*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp112)->f1)){case 0U: _LL1: _LL2:
 return t;case 18U: _LL7: _LL8:
# 789
 return Cyc_Absyn_void_type;case 19U: if(((((struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp112)->f1)->f1).KnownDatatypefield).tag == 2){_LL9: _tmp134=(((((struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp112)->f1)->f1).KnownDatatypefield).val).f1;_tmp135=(((((struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp112)->f1)->f1).KnownDatatypefield).val).f2;_LLA: {struct Cyc_Absyn_Datatypedecl*tud=_tmp134;struct Cyc_Absyn_Datatypefield*tuf=_tmp135;
# 791
return Cyc_Absyn_strctq(Cyc_Toc_collapse_qvars(tuf->name,tud->name));}}else{_LLB: _LLC:
# 793
({void*_tmp136=0U;({int(*_tmp671)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp138)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Toc_toc_impos;_tmp138;});struct _fat_ptr _tmp670=({const char*_tmp137="unresolved DatatypeFieldType";_tag_fat(_tmp137,sizeof(char),29U);});_tmp671(_tmp670,_tag_fat(_tmp136,sizeof(void*),0U));});});}case 1U: _LLF: _LL10:
# 802
 goto _LL12;case 2U: _LL11: _LL12:
 return t;case 20U: _LL1B: _tmp131=(void*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp112)->f1;_tmp132=((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp112)->f1)->f1;_tmp133=((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp112)->f2;_LL1C: {void*c=_tmp131;union Cyc_Absyn_AggrInfo info=_tmp132;struct Cyc_List_List*ts=_tmp133;
# 849
{union Cyc_Absyn_AggrInfo _tmp142=info;if((_tmp142.UnknownAggr).tag == 1){_LL45: _LL46:
# 851
 if(ts == 0)return t;else{
return(void*)({struct Cyc_Absyn_AppType_Absyn_Type_struct*_tmp143=_cycalloc(sizeof(*_tmp143));_tmp143->tag=0U,_tmp143->f1=c,_tmp143->f2=0;_tmp143;});}}else{_LL47: _LL48:
 goto _LL44;}_LL44:;}{
# 855
struct Cyc_Absyn_Aggrdecl*ad=Cyc_Absyn_get_known_aggrdecl(info);
if(ad->expected_mem_kind){
# 858
if(ad->impl == 0)
({struct Cyc_String_pa_PrintArg_struct _tmp146=({struct Cyc_String_pa_PrintArg_struct _tmp5E7;_tmp5E7.tag=0U,({
struct _fat_ptr _tmp672=(struct _fat_ptr)((int)ad->kind == (int)Cyc_Absyn_UnionA?({const char*_tmp148="union";_tag_fat(_tmp148,sizeof(char),6U);}):({const char*_tmp149="struct";_tag_fat(_tmp149,sizeof(char),7U);}));_tmp5E7.f1=_tmp672;});_tmp5E7;});struct Cyc_String_pa_PrintArg_struct _tmp147=({struct Cyc_String_pa_PrintArg_struct _tmp5E6;_tmp5E6.tag=0U,({
struct _fat_ptr _tmp673=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(ad->name));_tmp5E6.f1=_tmp673;});_tmp5E6;});void*_tmp144[2U];_tmp144[0]=& _tmp146,_tmp144[1]=& _tmp147;({struct _fat_ptr _tmp674=({const char*_tmp145="%s %s was never defined.";_tag_fat(_tmp145,sizeof(char),25U);});Cyc_Tcutil_warn(0U,_tmp674,_tag_fat(_tmp144,sizeof(void*),2U));});});}
# 856
if((int)ad->kind == (int)Cyc_Absyn_UnionA)
# 865
return Cyc_Toc_aggrdecl_type(ad->name,Cyc_Absyn_unionq_type);{
# 856
struct Cyc_List_List*fs=
# 866
ad->impl == 0?0:((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->fields;
if(fs == 0)return Cyc_Toc_aggrdecl_type(ad->name,Cyc_Absyn_strctq);for(0;((struct Cyc_List_List*)_check_null(fs))->tl != 0;fs=fs->tl){
;}{
void*last_type=((struct Cyc_Absyn_Aggrfield*)fs->hd)->type;
if(Cyc_Tcutil_kind_leq(& Cyc_Tcutil_ak,Cyc_Tcutil_type_kind(last_type))){
if(ad->expected_mem_kind)
({struct Cyc_String_pa_PrintArg_struct _tmp14C=({struct Cyc_String_pa_PrintArg_struct _tmp5E8;_tmp5E8.tag=0U,({struct _fat_ptr _tmp675=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_qvar2string(ad->name));_tmp5E8.f1=_tmp675;});_tmp5E8;});void*_tmp14A[1U];_tmp14A[0]=& _tmp14C;({struct _fat_ptr _tmp676=({const char*_tmp14B="struct %s ended up being abstract.";_tag_fat(_tmp14B,sizeof(char),35U);});Cyc_Tcutil_warn(0U,_tmp676,_tag_fat(_tmp14A,sizeof(void*),1U));});});{
# 871
struct _RegionHandle _tmp14D=_new_region("r");struct _RegionHandle*r=& _tmp14D;_push_region(r);
# 878
{struct Cyc_List_List*inst=Cyc_List_rzip(r,r,ad->tvs,ts);
void*t=Cyc_Tcutil_rsubstitute(r,inst,last_type);
if(Cyc_Toc_is_abstract_type(t)){void*_tmp14E=Cyc_Toc_aggrdecl_type(ad->name,Cyc_Absyn_strctq);_npop_handler(0U);return _tmp14E;}{void*_tmp14F=
Cyc_Toc_add_struct_type(ad->name,ad->tvs,ts,((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->fields);_npop_handler(0U);return _tmp14F;}}
# 878
;_pop_region(r);}}
# 870
return
# 883
 Cyc_Toc_aggrdecl_type(ad->name,Cyc_Absyn_strctq);}}}}case 15U: _LL1D: _tmp130=((struct Cyc_Absyn_EnumCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp112)->f1)->f1;_LL1E: {struct _tuple1*tdn=_tmp130;
return t;}case 16U: _LL1F: _tmp12F=((struct Cyc_Absyn_AnonEnumCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp112)->f1)->f1;_LL20: {struct Cyc_List_List*fs=_tmp12F;
Cyc_Toc_enumfields_to_c(fs);return t;}case 4U: _LL23: _LL24:
# 895
 return Cyc_Absyn_uint_type;case 3U: _LL25: _LL26:
 return Cyc_Toc_rgn_type();case 17U: _LL27: _LL28:
 return t;default: _LL29: _LL2A:
# 900
 return Cyc_Toc_void_star_type();}case 1U: _LL3: _tmp12E=(void**)&((struct Cyc_Absyn_Evar_Absyn_Type_struct*)_tmp112)->f2;_LL4: {void**t2=_tmp12E;
# 780
if(*t2 == 0){
*t2=Cyc_Absyn_sint_type;
return Cyc_Absyn_sint_type;}
# 780
return
# 784
 Cyc_Toc_typ_to_c((void*)_check_null(*t2));}case 2U: _LL5: _tmp12D=((struct Cyc_Absyn_VarType_Absyn_Type_struct*)_tmp112)->f1;_LL6: {struct Cyc_Absyn_Tvar*tv=_tmp12D;
# 786
if((int)(Cyc_Tcutil_tvar_kind(tv,& Cyc_Tcutil_bk))->kind == (int)Cyc_Absyn_AnyKind)
return Cyc_Absyn_void_type;else{
return Cyc_Toc_void_star_type();}}case 3U: _LLD: _tmp12A=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp112)->f1).elt_type;_tmp12B=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp112)->f1).elt_tq;_tmp12C=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp112)->f1).ptr_atts).bounds;_LLE: {void*t2=_tmp12A;struct Cyc_Absyn_Tqual tq=_tmp12B;void*bnds=_tmp12C;
# 797
t2=Cyc_Toc_typ_to_c(t2);
if(Cyc_Tcutil_get_bounds_exp(Cyc_Absyn_fat_bound_type,bnds)== 0)
return Cyc_Toc_fat_ptr_type();else{
# 801
return Cyc_Absyn_star_type(t2,Cyc_Absyn_heap_rgn_type,tq,Cyc_Absyn_false_type);}}case 4U: _LL13: _tmp126=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp112)->f1).elt_type;_tmp127=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp112)->f1).tq;_tmp128=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp112)->f1).num_elts;_tmp129=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp112)->f1).zt_loc;_LL14: {void*t2=_tmp126;struct Cyc_Absyn_Tqual tq=_tmp127;struct Cyc_Absyn_Exp*e=_tmp128;unsigned ztl=_tmp129;
# 805
return({void*_tmp67A=Cyc_Toc_typ_to_c(t2);struct Cyc_Absyn_Tqual _tmp679=tq;struct Cyc_Absyn_Exp*_tmp678=e;void*_tmp677=Cyc_Absyn_false_type;Cyc_Absyn_array_type(_tmp67A,_tmp679,_tmp678,_tmp677,ztl);});}case 5U: _LL15: _tmp120=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp112)->f1).ret_tqual;_tmp121=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp112)->f1).ret_type;_tmp122=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp112)->f1).args;_tmp123=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp112)->f1).c_varargs;_tmp124=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp112)->f1).cyc_varargs;_tmp125=(((struct Cyc_Absyn_FnType_Absyn_Type_struct*)_tmp112)->f1).attributes;_LL16: {struct Cyc_Absyn_Tqual tq2=_tmp120;void*t2=_tmp121;struct Cyc_List_List*args=_tmp122;int c_vararg=_tmp123;struct Cyc_Absyn_VarargInfo*cyc_vararg=_tmp124;struct Cyc_List_List*atts=_tmp125;
# 811
struct Cyc_List_List*new_atts=0;
for(0;atts != 0;atts=atts->tl){
void*_stmttmp14=(void*)atts->hd;void*_tmp139=_stmttmp14;switch(*((int*)_tmp139)){case 4U: _LL36: _LL37:
 goto _LL39;case 5U: _LL38: _LL39:
 goto _LL3B;case 19U: _LL3A: _LL3B:
 continue;case 22U: _LL3C: _LL3D:
 continue;case 21U: _LL3E: _LL3F:
 continue;case 20U: _LL40: _LL41:
 continue;default: _LL42: _LL43:
 new_atts=({struct Cyc_List_List*_tmp13A=_cycalloc(sizeof(*_tmp13A));_tmp13A->hd=(void*)atts->hd,_tmp13A->tl=new_atts;_tmp13A;});goto _LL35;}_LL35:;}{
# 822
struct Cyc_List_List*new_args=({struct Cyc_List_List*(*_tmp67B)(struct _tuple9*(*f)(struct _tuple9*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp13E)(struct _tuple9*(*f)(struct _tuple9*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct _tuple9*(*f)(struct _tuple9*),struct Cyc_List_List*x))Cyc_List_map;_tmp13E;});_tmp67B(Cyc_Toc_arg_to_c,args);});
if(cyc_vararg != 0){
# 825
void*t=Cyc_Toc_typ_to_c(Cyc_Absyn_fatptr_type(cyc_vararg->type,Cyc_Absyn_heap_rgn_type,Cyc_Toc_mt_tq,Cyc_Absyn_false_type));
struct _tuple9*vararg=({struct _tuple9*_tmp13C=_cycalloc(sizeof(*_tmp13C));_tmp13C->f1=cyc_vararg->name,_tmp13C->f2=cyc_vararg->tq,_tmp13C->f3=t;_tmp13C;});
new_args=({struct Cyc_List_List*_tmp67C=new_args;Cyc_List_imp_append(_tmp67C,({struct Cyc_List_List*_tmp13B=_cycalloc(sizeof(*_tmp13B));_tmp13B->hd=vararg,_tmp13B->tl=0;_tmp13B;}));});}
# 823
return(void*)({struct Cyc_Absyn_FnType_Absyn_Type_struct*_tmp13D=_cycalloc(sizeof(*_tmp13D));
# 829
_tmp13D->tag=5U,(_tmp13D->f1).tvars=0,(_tmp13D->f1).effect=0,(_tmp13D->f1).ret_tqual=tq2,({void*_tmp67D=Cyc_Toc_typ_to_c(t2);(_tmp13D->f1).ret_type=_tmp67D;}),(_tmp13D->f1).args=new_args,(_tmp13D->f1).c_varargs=c_vararg,(_tmp13D->f1).cyc_varargs=0,(_tmp13D->f1).rgn_po=0,(_tmp13D->f1).attributes=new_atts,(_tmp13D->f1).requires_clause=0,(_tmp13D->f1).requires_relns=0,(_tmp13D->f1).ensures_clause=0,(_tmp13D->f1).ensures_relns=0,(_tmp13D->f1).ieffect=0,(_tmp13D->f1).oeffect=0;_tmp13D;});}}case 6U: _LL17: _tmp11F=((struct Cyc_Absyn_TupleType_Absyn_Type_struct*)_tmp112)->f1;_LL18: {struct Cyc_List_List*tqs=_tmp11F;
# 834
struct Cyc_List_List*tqs2=0;
for(0;tqs != 0;tqs=tqs->tl){
tqs2=({struct Cyc_List_List*_tmp140=_cycalloc(sizeof(*_tmp140));({struct _tuple13*_tmp67F=({struct _tuple13*_tmp13F=_cycalloc(sizeof(*_tmp13F));_tmp13F->f1=(*((struct _tuple13*)tqs->hd)).f1,({void*_tmp67E=Cyc_Toc_typ_to_c((*((struct _tuple13*)tqs->hd)).f2);_tmp13F->f2=_tmp67E;});_tmp13F;});_tmp140->hd=_tmp67F;}),_tmp140->tl=tqs2;_tmp140;});}
return Cyc_Toc_add_tuple_type(Cyc_List_imp_rev(tqs2));}case 7U: _LL19: _tmp11D=((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmp112)->f1;_tmp11E=((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmp112)->f2;_LL1A: {enum Cyc_Absyn_AggrKind k=_tmp11D;struct Cyc_List_List*fs=_tmp11E;
# 843
struct Cyc_List_List*fs2=0;
for(0;fs != 0;fs=fs->tl){
fs2=({struct Cyc_List_List*_tmp141=_cycalloc(sizeof(*_tmp141));({struct Cyc_Absyn_Aggrfield*_tmp681=({struct Cyc_Absyn_Aggrfield*_tmp680=(struct Cyc_Absyn_Aggrfield*)fs->hd;Cyc_Toc_aggrfield_to_c(_tmp680,Cyc_Toc_typ_to_c(((struct Cyc_Absyn_Aggrfield*)fs->hd)->type));});_tmp141->hd=_tmp681;}),_tmp141->tl=fs2;_tmp141;});}
return({enum Cyc_Absyn_AggrKind _tmp682=k;Cyc_Toc_add_anon_aggr_type(_tmp682,Cyc_List_imp_rev(fs2));});}case 8U: _LL21: _tmp119=((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmp112)->f1;_tmp11A=((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmp112)->f2;_tmp11B=((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmp112)->f3;_tmp11C=(void*)((struct Cyc_Absyn_TypedefType_Absyn_Type_struct*)_tmp112)->f4;_LL22: {struct _tuple1*tdn=_tmp119;struct Cyc_List_List*ts=_tmp11A;struct Cyc_Absyn_Typedefdecl*td=_tmp11B;void*topt=_tmp11C;
# 889
if(topt == 0){
if(ts != 0)
return(void*)({struct Cyc_Absyn_TypedefType_Absyn_Type_struct*_tmp150=_cycalloc(sizeof(*_tmp150));_tmp150->tag=8U,_tmp150->f1=tdn,_tmp150->f2=0,_tmp150->f3=td,_tmp150->f4=0;_tmp150;});else{
return t;}}else{
# 894
return(void*)({struct Cyc_Absyn_TypedefType_Absyn_Type_struct*_tmp151=_cycalloc(sizeof(*_tmp151));_tmp151->tag=8U,_tmp151->f1=tdn,_tmp151->f2=0,_tmp151->f3=td,({void*_tmp683=Cyc_Toc_typ_to_c(topt);_tmp151->f4=_tmp683;});_tmp151;});}}case 9U: _LL2B: _tmp118=((struct Cyc_Absyn_ValueofType_Absyn_Type_struct*)_tmp112)->f1;_LL2C: {struct Cyc_Absyn_Exp*e=_tmp118;
# 905
return t;}case 11U: _LL2D: _tmp117=((struct Cyc_Absyn_TypeofType_Absyn_Type_struct*)_tmp112)->f1;_LL2E: {struct Cyc_Absyn_Exp*e=_tmp117;
# 910
if(e->topt != 0)
return Cyc_Toc_typ_to_c((void*)_check_null(e->topt));
# 910
return t;}default: switch(*((int*)((struct Cyc_Absyn_TypeDecl*)((struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct*)_tmp112)->f1)->r)){case 0U: _LL2F: _tmp116=((struct Cyc_Absyn_Aggr_td_Absyn_Raw_typedecl_struct*)(((struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct*)_tmp112)->f1)->r)->f1;_LL30: {struct Cyc_Absyn_Aggrdecl*ad=_tmp116;
# 914
Cyc_Toc_aggrdecl_to_c(ad);
if((int)ad->kind == (int)Cyc_Absyn_UnionA)
return Cyc_Toc_aggrdecl_type(ad->name,Cyc_Absyn_unionq_type);
# 915
return
# 917
 Cyc_Toc_aggrdecl_type(ad->name,Cyc_Absyn_strctq);}case 1U: _LL31: _tmp115=((struct Cyc_Absyn_Enum_td_Absyn_Raw_typedecl_struct*)(((struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct*)_tmp112)->f1)->r)->f1;_LL32: {struct Cyc_Absyn_Enumdecl*ed=_tmp115;
# 919
Cyc_Toc_enumdecl_to_c(ed);
return t;}default: _LL33: _tmp113=((struct Cyc_Absyn_Datatype_td_Absyn_Raw_typedecl_struct*)(((struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct*)_tmp112)->f1)->r)->f1;_tmp114=((struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct*)_tmp112)->f2;_LL34: {struct Cyc_Absyn_Datatypedecl*dd=_tmp113;void**t=_tmp114;
# 922
Cyc_Toc_datatypedecl_to_c(dd);
return Cyc_Toc_typ_to_c(*((void**)_check_null(t)));}}}_LL0:;}
# 711 "toc.cyc"
static struct Cyc_Absyn_Exp*Cyc_Toc_array_to_ptr_cast(void*t,struct Cyc_Absyn_Exp*e,unsigned l){
# 928 "toc.cyc"
void*_tmp152=t;struct Cyc_Absyn_Tqual _tmp154;void*_tmp153;if(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp152)->tag == 4U){_LL1: _tmp153=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp152)->f1).elt_type;_tmp154=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp152)->f1).tq;_LL2: {void*t2=_tmp153;struct Cyc_Absyn_Tqual tq=_tmp154;
# 930
return({void*_tmp684=Cyc_Absyn_star_type(t2,Cyc_Absyn_heap_rgn_type,tq,Cyc_Absyn_false_type);Cyc_Toc_cast_it(_tmp684,e);});}}else{_LL3: _LL4:
 return Cyc_Toc_cast_it(t,e);}_LL0:;}
# 711 "toc.cyc"
static int Cyc_Toc_atomic_type(void*t){
# 938 "toc.cyc"
void*_stmttmp15=Cyc_Tcutil_compress(t);void*_tmp155=_stmttmp15;struct Cyc_List_List*_tmp156;struct Cyc_List_List*_tmp157;void*_tmp158;struct Cyc_List_List*_tmp15A;void*_tmp159;switch(*((int*)_tmp155)){case 2U: _LL1: _LL2:
 return 0;case 0U: _LL3: _tmp159=(void*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp155)->f1;_tmp15A=((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp155)->f2;_LL4: {void*c=_tmp159;struct Cyc_List_List*ts=_tmp15A;
# 941
void*_tmp15B=c;struct Cyc_Absyn_Datatypefield*_tmp15D;struct Cyc_Absyn_Datatypedecl*_tmp15C;union Cyc_Absyn_AggrInfo _tmp15E;switch(*((int*)_tmp15B)){case 0U: _LL12: _LL13:
 goto _LL15;case 1U: _LL14: _LL15: goto _LL17;case 2U: _LL16: _LL17: goto _LL19;case 4U: _LL18: _LL19:
 goto _LL1B;case 15U: _LL1A: _LL1B: goto _LL1D;case 16U: _LL1C: _LL1D: return 1;case 18U: _LL1E: _LL1F:
 goto _LL21;case 3U: _LL20: _LL21:
 goto _LL23;case 17U: _LL22: _LL23:
 return 0;case 20U: _LL24: _tmp15E=((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)_tmp15B)->f1;_LL25: {union Cyc_Absyn_AggrInfo info=_tmp15E;
# 948
{union Cyc_Absyn_AggrInfo _tmp15F=info;if((_tmp15F.UnknownAggr).tag == 1){_LL2B: _LL2C:
 return 0;}else{_LL2D: _LL2E:
 goto _LL2A;}_LL2A:;}{
# 952
struct Cyc_Absyn_Aggrdecl*ad=Cyc_Absyn_get_known_aggrdecl(info);
if(ad->impl == 0)
return 0;
# 953
{
# 955
struct Cyc_List_List*fs=((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->fields;
# 953
for(0;fs != 0;fs=fs->tl){
# 956
if(!Cyc_Toc_atomic_type(((struct Cyc_Absyn_Aggrfield*)fs->hd)->type))return 0;}}
# 953
return 1;}}case 19U: if(((((struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct*)_tmp15B)->f1).KnownDatatypefield).tag == 2){_LL26: _tmp15C=(((((struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct*)_tmp15B)->f1).KnownDatatypefield).val).f1;_tmp15D=(((((struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct*)_tmp15B)->f1).KnownDatatypefield).val).f2;_LL27: {struct Cyc_Absyn_Datatypedecl*tud=_tmp15C;struct Cyc_Absyn_Datatypefield*tuf=_tmp15D;
# 959
{struct Cyc_List_List*tqs=tuf->typs;for(0;tqs != 0;tqs=tqs->tl){
if(!Cyc_Toc_atomic_type((*((struct _tuple13*)tqs->hd)).f2))return 0;}}
# 959
return 1;}}else{goto _LL28;}default: _LL28: _LL29:
# 962
({struct Cyc_String_pa_PrintArg_struct _tmp163=({struct Cyc_String_pa_PrintArg_struct _tmp5E9;_tmp5E9.tag=0U,({struct _fat_ptr _tmp685=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t));_tmp5E9.f1=_tmp685;});_tmp5E9;});void*_tmp160[1U];_tmp160[0]=& _tmp163;({int(*_tmp687)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp162)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Toc_toc_impos;_tmp162;});struct _fat_ptr _tmp686=({const char*_tmp161="atomic_typ:  bad type %s";_tag_fat(_tmp161,sizeof(char),25U);});_tmp687(_tmp686,_tag_fat(_tmp160,sizeof(void*),1U));});});}_LL11:;}case 5U: _LL5: _LL6:
# 964
 return 1;case 4U: _LL7: _tmp158=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp155)->f1).elt_type;_LL8: {void*t=_tmp158;
return Cyc_Toc_atomic_type(t);}case 7U: _LL9: _tmp157=((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmp155)->f2;_LLA: {struct Cyc_List_List*fs=_tmp157;
# 972
for(0;fs != 0;fs=fs->tl){
if(!Cyc_Toc_atomic_type(((struct Cyc_Absyn_Aggrfield*)fs->hd)->type))return 0;}
# 972
return 1;}case 6U: _LLB: _tmp156=((struct Cyc_Absyn_TupleType_Absyn_Type_struct*)_tmp155)->f1;_LLC: {struct Cyc_List_List*tqs=_tmp156;
# 976
for(0;tqs != 0;tqs=tqs->tl){
if(!Cyc_Toc_atomic_type((*((struct _tuple13*)tqs->hd)).f2))return 0;}
# 976
return 1;}case 3U: _LLD: _LLE:
# 981
 return 0;default: _LLF: _LL10:
({struct Cyc_String_pa_PrintArg_struct _tmp167=({struct Cyc_String_pa_PrintArg_struct _tmp5EA;_tmp5EA.tag=0U,({struct _fat_ptr _tmp688=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t));_tmp5EA.f1=_tmp688;});_tmp5EA;});void*_tmp164[1U];_tmp164[0]=& _tmp167;({int(*_tmp68A)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp166)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Toc_toc_impos;_tmp166;});struct _fat_ptr _tmp689=({const char*_tmp165="atomic_typ:  bad type %s";_tag_fat(_tmp165,sizeof(char),25U);});_tmp68A(_tmp689,_tag_fat(_tmp164,sizeof(void*),1U));});});}_LL0:;}
# 711 "toc.cyc"
static int Cyc_Toc_is_poly_field(void*t,struct _fat_ptr*f){
# 988 "toc.cyc"
void*_stmttmp16=Cyc_Tcutil_compress(t);void*_tmp168=_stmttmp16;struct Cyc_List_List*_tmp169;union Cyc_Absyn_AggrInfo _tmp16A;switch(*((int*)_tmp168)){case 0U: if(((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp168)->f1)->tag == 20U){_LL1: _tmp16A=((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp168)->f1)->f1;_LL2: {union Cyc_Absyn_AggrInfo info=_tmp16A;
# 990
struct Cyc_Absyn_Aggrdecl*ad=Cyc_Absyn_get_known_aggrdecl(info);
if(ad->impl == 0)
({void*_tmp16B=0U;({int(*_tmp68C)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp16D)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Toc_toc_impos;_tmp16D;});struct _fat_ptr _tmp68B=({const char*_tmp16C="is_poly_field: type missing fields";_tag_fat(_tmp16C,sizeof(char),35U);});_tmp68C(_tmp68B,_tag_fat(_tmp16B,sizeof(void*),0U));});});
# 991
_tmp169=((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->fields;goto _LL4;}}else{goto _LL5;}case 7U: _LL3: _tmp169=((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmp168)->f2;_LL4: {struct Cyc_List_List*fs=_tmp169;
# 995
struct Cyc_Absyn_Aggrfield*field=Cyc_Absyn_lookup_field(fs,f);
if(field == 0)
({struct Cyc_String_pa_PrintArg_struct _tmp171=({struct Cyc_String_pa_PrintArg_struct _tmp5EB;_tmp5EB.tag=0U,_tmp5EB.f1=(struct _fat_ptr)((struct _fat_ptr)*f);_tmp5EB;});void*_tmp16E[1U];_tmp16E[0]=& _tmp171;({int(*_tmp68E)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp170)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Toc_toc_impos;_tmp170;});struct _fat_ptr _tmp68D=({const char*_tmp16F="is_poly_field: bad field %s";_tag_fat(_tmp16F,sizeof(char),28U);});_tmp68E(_tmp68D,_tag_fat(_tmp16E,sizeof(void*),1U));});});
# 996
return
# 998
 Cyc_Toc_is_void_star_or_boxed_tvar(field->type);}default: _LL5: _LL6:
({struct Cyc_String_pa_PrintArg_struct _tmp175=({struct Cyc_String_pa_PrintArg_struct _tmp5EC;_tmp5EC.tag=0U,({struct _fat_ptr _tmp68F=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(t));_tmp5EC.f1=_tmp68F;});_tmp5EC;});void*_tmp172[1U];_tmp172[0]=& _tmp175;({int(*_tmp691)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp174)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Toc_toc_impos;_tmp174;});struct _fat_ptr _tmp690=({const char*_tmp173="is_poly_field: bad type %s";_tag_fat(_tmp173,sizeof(char),27U);});_tmp691(_tmp690,_tag_fat(_tmp172,sizeof(void*),1U));});});}_LL0:;}
# 711 "toc.cyc"
static int Cyc_Toc_is_poly_project(struct Cyc_Absyn_Exp*e){
# 1007 "toc.cyc"
void*_stmttmp17=e->r;void*_tmp176=_stmttmp17;struct _fat_ptr*_tmp178;struct Cyc_Absyn_Exp*_tmp177;struct _fat_ptr*_tmp17A;struct Cyc_Absyn_Exp*_tmp179;switch(*((int*)_tmp176)){case 21U: _LL1: _tmp179=((struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct*)_tmp176)->f1;_tmp17A=((struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct*)_tmp176)->f2;_LL2: {struct Cyc_Absyn_Exp*e1=_tmp179;struct _fat_ptr*f=_tmp17A;
# 1009
return Cyc_Toc_is_poly_field((void*)_check_null(e1->topt),f)&& !
Cyc_Toc_is_void_star_or_boxed_tvar((void*)_check_null(e->topt));}case 22U: _LL3: _tmp177=((struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct*)_tmp176)->f1;_tmp178=((struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct*)_tmp176)->f2;_LL4: {struct Cyc_Absyn_Exp*e1=_tmp177;struct _fat_ptr*f=_tmp178;
# 1012
void*_stmttmp18=Cyc_Tcutil_compress((void*)_check_null(e1->topt));void*_tmp17B=_stmttmp18;void*_tmp17C;if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp17B)->tag == 3U){_LL8: _tmp17C=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp17B)->f1).elt_type;_LL9: {void*t=_tmp17C;
# 1014
return Cyc_Toc_is_poly_field(t,f)&& !Cyc_Toc_is_void_star_or_boxed_tvar((void*)_check_null(e->topt));}}else{_LLA: _LLB:
({struct Cyc_String_pa_PrintArg_struct _tmp180=({struct Cyc_String_pa_PrintArg_struct _tmp5ED;_tmp5ED.tag=0U,({struct _fat_ptr _tmp692=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string((void*)_check_null(e1->topt)));_tmp5ED.f1=_tmp692;});_tmp5ED;});void*_tmp17D[1U];_tmp17D[0]=& _tmp180;({int(*_tmp694)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp17F)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Toc_toc_impos;_tmp17F;});struct _fat_ptr _tmp693=({const char*_tmp17E="is_poly_project: bad type %s";_tag_fat(_tmp17E,sizeof(char),29U);});_tmp694(_tmp693,_tag_fat(_tmp17D,sizeof(void*),1U));});});}_LL7:;}default: _LL5: _LL6:
# 1017
 return 0;}_LL0:;}
# 711 "toc.cyc"
static struct Cyc_Absyn_Exp*Cyc_Toc_spawn_exp(struct Cyc_Absyn_Exp*e1,struct Cyc_Absyn_Exp*e2){
# 1023 "toc.cyc"
return({struct Cyc_Absyn_Exp*_tmp181[2U];_tmp181[0]=e1,_tmp181[1]=e2;({struct Cyc_Absyn_Exp*_tmp695=Cyc_Toc__spawn_e;Cyc_Toc_fncall_exp_dl(_tmp695,_tag_fat(_tmp181,sizeof(struct Cyc_Absyn_Exp*),2U));});});}
# 711 "toc.cyc"
static struct Cyc_Absyn_Exp*Cyc_Toc_malloc_exp(void*t,struct Cyc_Absyn_Exp*s){
# 1028 "toc.cyc"
struct Cyc_Absyn_Exp*fn_e=Cyc_Toc_atomic_type(t)?Cyc_Toc__cycalloc_atomic_e: Cyc_Toc__cycalloc_e;
return({struct Cyc_Absyn_Exp*_tmp182[1U];_tmp182[0]=s;({struct Cyc_Absyn_Exp*_tmp696=fn_e;Cyc_Toc_fncall_exp_dl(_tmp696,_tag_fat(_tmp182,sizeof(struct Cyc_Absyn_Exp*),1U));});});}
# 711 "toc.cyc"
static struct Cyc_Absyn_Exp*Cyc_Toc_calloc_exp(void*elt_type,struct Cyc_Absyn_Exp*s,struct Cyc_Absyn_Exp*n){
# 1033 "toc.cyc"
struct Cyc_Absyn_Exp*fn_e=Cyc_Toc_atomic_type(elt_type)?Cyc_Toc__cyccalloc_atomic_e: Cyc_Toc__cyccalloc_e;
return({struct Cyc_Absyn_Exp*_tmp183[2U];_tmp183[0]=s,_tmp183[1]=n;({struct Cyc_Absyn_Exp*_tmp697=fn_e;Cyc_Toc_fncall_exp_dl(_tmp697,_tag_fat(_tmp183,sizeof(struct Cyc_Absyn_Exp*),2U));});});}
# 711 "toc.cyc"
static struct Cyc_Absyn_Exp*Cyc_Toc_rmalloc_exp(struct Cyc_Absyn_Exp*rgn,struct Cyc_Absyn_Exp*s){
# 1038 "toc.cyc"
return({struct Cyc_Absyn_Exp*_tmp184[2U];_tmp184[0]=rgn,_tmp184[1]=s;({struct Cyc_Absyn_Exp*_tmp698=Cyc_Toc__region_malloc_e;Cyc_Toc_fncall_exp_dl(_tmp698,_tag_fat(_tmp184,sizeof(struct Cyc_Absyn_Exp*),2U));});});}
# 711 "toc.cyc"
static struct Cyc_Absyn_Exp*Cyc_Toc_rmalloc_inline_exp(struct Cyc_Absyn_Exp*rgn,struct Cyc_Absyn_Exp*s){
# 1042 "toc.cyc"
return({struct Cyc_Absyn_Exp*_tmp185[2U];_tmp185[0]=rgn,_tmp185[1]=s;({struct Cyc_Absyn_Exp*_tmp699=Cyc_Toc__fast_region_malloc_e;Cyc_Toc_fncall_exp_dl(_tmp699,_tag_fat(_tmp185,sizeof(struct Cyc_Absyn_Exp*),2U));});});}
# 711 "toc.cyc"
static struct Cyc_Absyn_Exp*Cyc_Toc_rcalloc_exp(struct Cyc_Absyn_Exp*rgn,struct Cyc_Absyn_Exp*s,struct Cyc_Absyn_Exp*n){
# 1046 "toc.cyc"
return({struct Cyc_Absyn_Exp*_tmp186[3U];_tmp186[0]=rgn,_tmp186[1]=s,_tmp186[2]=n;({struct Cyc_Absyn_Exp*_tmp69A=Cyc_Toc__region_calloc_e;Cyc_Toc_fncall_exp_dl(_tmp69A,_tag_fat(_tmp186,sizeof(struct Cyc_Absyn_Exp*),3U));});});}
# 711 "toc.cyc"
static struct Cyc_Absyn_Stmt*Cyc_Toc_throw_match_stmt(){
# 1050 "toc.cyc"
return Cyc_Absyn_exp_stmt(({void*_tmp187=0U;({struct Cyc_Absyn_Exp*_tmp69B=Cyc_Toc__throw_match_e;Cyc_Toc_fncall_exp_dl(_tmp69B,_tag_fat(_tmp187,sizeof(struct Cyc_Absyn_Exp*),0U));});}),0U);}
# 711 "toc.cyc"
static struct Cyc_Absyn_Exp*Cyc_Toc_make_toplevel_dyn_arr(void*t,struct Cyc_Absyn_Exp*sz,struct Cyc_Absyn_Exp*e){
# 1062 "toc.cyc"
struct Cyc_Absyn_Exp*xexp;
struct Cyc_Absyn_Exp*xplussz;
{void*_stmttmp19=e->r;void*_tmp188=_stmttmp19;if(((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmp188)->tag == 0U)switch(((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmp188)->f1).Wstring_c).tag){case 8U: _LL1: _LL2:
 goto _LL4;case 9U: _LL3: _LL4: {
# 1067
struct _tuple1*x=Cyc_Toc_temp_var();
void*vd_typ=Cyc_Absyn_array_type(Cyc_Absyn_char_type,Cyc_Toc_mt_tq,sz,Cyc_Absyn_false_type,0U);
struct Cyc_Absyn_Vardecl*vd=Cyc_Absyn_static_vardecl(x,vd_typ,e);
Cyc_Toc_result_decls=({struct Cyc_List_List*_tmp18A=_cycalloc(sizeof(*_tmp18A));({struct Cyc_Absyn_Decl*_tmp69C=Cyc_Absyn_new_decl((void*)({struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct*_tmp189=_cycalloc(sizeof(*_tmp189));_tmp189->tag=0U,_tmp189->f1=vd;_tmp189;}),0U);_tmp18A->hd=_tmp69C;}),_tmp18A->tl=Cyc_Toc_result_decls;_tmp18A;});
xexp=Cyc_Absyn_var_exp(x,0U);
xplussz=Cyc_Absyn_add_exp(xexp,sz,0U);
goto _LL0;}default: goto _LL5;}else{_LL5: _LL6:
# 1075
 xexp=({void*_tmp69D=Cyc_Toc_void_star_type();Cyc_Toc_cast_it(_tmp69D,e);});
# 1077
xplussz=({void*_tmp69E=Cyc_Toc_void_star_type();Cyc_Toc_cast_it(_tmp69E,Cyc_Absyn_add_exp(e,sz,0U));});
goto _LL0;}_LL0:;}
# 1080
return Cyc_Absyn_unresolvedmem_exp(0,({struct _tuple20*_tmp18B[3U];({struct _tuple20*_tmp6A1=({struct _tuple20*_tmp18C=_cycalloc(sizeof(*_tmp18C));_tmp18C->f1=0,_tmp18C->f2=xexp;_tmp18C;});_tmp18B[0]=_tmp6A1;}),({
struct _tuple20*_tmp6A0=({struct _tuple20*_tmp18D=_cycalloc(sizeof(*_tmp18D));_tmp18D->f1=0,_tmp18D->f2=xexp;_tmp18D;});_tmp18B[1]=_tmp6A0;}),({
struct _tuple20*_tmp69F=({struct _tuple20*_tmp18E=_cycalloc(sizeof(*_tmp18E));_tmp18E->f1=0,_tmp18E->f2=xplussz;_tmp18E;});_tmp18B[2]=_tmp69F;});Cyc_List_list(_tag_fat(_tmp18B,sizeof(struct _tuple20*),3U));}),0U);}struct Cyc_Toc_FallthruInfo{struct _fat_ptr*label;struct Cyc_List_List*binders;};struct Cyc_Toc_Env{struct _fat_ptr**break_lab;struct _fat_ptr**continue_lab;struct Cyc_Toc_FallthruInfo*fallthru_info;int toplevel;int*in_lhs;struct _RegionHandle*rgn;};
# 1115 "toc.cyc"
static struct Cyc_Toc_Env*Cyc_Toc_empty_env(struct _RegionHandle*r){
return({struct Cyc_Toc_Env*_tmp190=_region_malloc(r,sizeof(*_tmp190));_tmp190->break_lab=0,_tmp190->continue_lab=0,_tmp190->fallthru_info=0,_tmp190->toplevel=1,({int*_tmp6A2=({int*_tmp18F=_region_malloc(r,sizeof(*_tmp18F));*_tmp18F=0;_tmp18F;});_tmp190->in_lhs=_tmp6A2;}),_tmp190->rgn=r;_tmp190;});}
# 1115
static int Cyc_Toc_is_toplevel(struct Cyc_Toc_Env*nv){
# 1120
int _tmp191;_LL1: _tmp191=nv->toplevel;_LL2: {int t=_tmp191;
return t;}}
# 1115
static struct Cyc_Toc_Env*Cyc_Toc_clear_toplevel(struct _RegionHandle*r,struct Cyc_Toc_Env*e){
# 1124
int*_tmp196;int _tmp195;struct Cyc_Toc_FallthruInfo*_tmp194;struct _fat_ptr**_tmp193;struct _fat_ptr**_tmp192;_LL1: _tmp192=e->break_lab;_tmp193=e->continue_lab;_tmp194=e->fallthru_info;_tmp195=e->toplevel;_tmp196=e->in_lhs;_LL2: {struct _fat_ptr**b=_tmp192;struct _fat_ptr**c=_tmp193;struct Cyc_Toc_FallthruInfo*f=_tmp194;int t=_tmp195;int*lhs=_tmp196;
return({struct Cyc_Toc_Env*_tmp197=_region_malloc(r,sizeof(*_tmp197));_tmp197->break_lab=b,_tmp197->continue_lab=c,_tmp197->fallthru_info=f,_tmp197->toplevel=0,_tmp197->in_lhs=lhs,_tmp197->rgn=r;_tmp197;});}}
# 1115
static struct Cyc_Toc_Env*Cyc_Toc_set_toplevel(struct _RegionHandle*r,struct Cyc_Toc_Env*e){
# 1128
int*_tmp19C;int _tmp19B;struct Cyc_Toc_FallthruInfo*_tmp19A;struct _fat_ptr**_tmp199;struct _fat_ptr**_tmp198;_LL1: _tmp198=e->break_lab;_tmp199=e->continue_lab;_tmp19A=e->fallthru_info;_tmp19B=e->toplevel;_tmp19C=e->in_lhs;_LL2: {struct _fat_ptr**b=_tmp198;struct _fat_ptr**c=_tmp199;struct Cyc_Toc_FallthruInfo*f=_tmp19A;int t=_tmp19B;int*lhs=_tmp19C;
return({struct Cyc_Toc_Env*_tmp19D=_region_malloc(r,sizeof(*_tmp19D));_tmp19D->break_lab=b,_tmp19D->continue_lab=c,_tmp19D->fallthru_info=f,_tmp19D->toplevel=1,_tmp19D->in_lhs=lhs,_tmp19D->rgn=r;_tmp19D;});}}
# 1115
static int Cyc_Toc_in_lhs(struct Cyc_Toc_Env*nv){
# 1132
int*_tmp19E;_LL1: _tmp19E=nv->in_lhs;_LL2: {int*b=_tmp19E;
return*b;}}
# 1115
static void Cyc_Toc_set_lhs(struct Cyc_Toc_Env*e,int b){
# 1136
int*_tmp19F;_LL1: _tmp19F=e->in_lhs;_LL2: {int*lhs=_tmp19F;
*lhs=b;}}
# 1115
static struct Cyc_Toc_Env*Cyc_Toc_share_env(struct _RegionHandle*r,struct Cyc_Toc_Env*e){
# 1141
int*_tmp1A4;int _tmp1A3;struct Cyc_Toc_FallthruInfo*_tmp1A2;struct _fat_ptr**_tmp1A1;struct _fat_ptr**_tmp1A0;_LL1: _tmp1A0=e->break_lab;_tmp1A1=e->continue_lab;_tmp1A2=e->fallthru_info;_tmp1A3=e->toplevel;_tmp1A4=e->in_lhs;_LL2: {struct _fat_ptr**b=_tmp1A0;struct _fat_ptr**c=_tmp1A1;struct Cyc_Toc_FallthruInfo*f=_tmp1A2;int t=_tmp1A3;int*lhs=_tmp1A4;
return({struct Cyc_Toc_Env*_tmp1A5=_region_malloc(r,sizeof(*_tmp1A5));_tmp1A5->break_lab=b,_tmp1A5->continue_lab=c,_tmp1A5->fallthru_info=f,_tmp1A5->toplevel=t,_tmp1A5->in_lhs=lhs,_tmp1A5->rgn=r;_tmp1A5;});}}
# 1115
static struct Cyc_Toc_Env*Cyc_Toc_loop_env(struct _RegionHandle*r,struct Cyc_Toc_Env*e){
# 1148
int*_tmp1AA;int _tmp1A9;struct Cyc_Toc_FallthruInfo*_tmp1A8;struct _fat_ptr**_tmp1A7;struct _fat_ptr**_tmp1A6;_LL1: _tmp1A6=e->break_lab;_tmp1A7=e->continue_lab;_tmp1A8=e->fallthru_info;_tmp1A9=e->toplevel;_tmp1AA=e->in_lhs;_LL2: {struct _fat_ptr**b=_tmp1A6;struct _fat_ptr**c=_tmp1A7;struct Cyc_Toc_FallthruInfo*f=_tmp1A8;int t=_tmp1A9;int*lhs=_tmp1AA;
return({struct Cyc_Toc_Env*_tmp1AB=_region_malloc(r,sizeof(*_tmp1AB));_tmp1AB->break_lab=0,_tmp1AB->continue_lab=0,_tmp1AB->fallthru_info=f,_tmp1AB->toplevel=t,_tmp1AB->in_lhs=lhs,_tmp1AB->rgn=r;_tmp1AB;});}}
# 1115
static struct Cyc_Toc_Env*Cyc_Toc_non_last_switchclause_env(struct _RegionHandle*r,struct Cyc_Toc_Env*e,struct _fat_ptr*break_l,struct _fat_ptr*fallthru_l,struct Cyc_List_List*fallthru_binders){
# 1157
int*_tmp1B0;int _tmp1AF;struct Cyc_Toc_FallthruInfo*_tmp1AE;struct _fat_ptr**_tmp1AD;struct _fat_ptr**_tmp1AC;_LL1: _tmp1AC=e->break_lab;_tmp1AD=e->continue_lab;_tmp1AE=e->fallthru_info;_tmp1AF=e->toplevel;_tmp1B0=e->in_lhs;_LL2: {struct _fat_ptr**b=_tmp1AC;struct _fat_ptr**c=_tmp1AD;struct Cyc_Toc_FallthruInfo*f=_tmp1AE;int t=_tmp1AF;int*lhs=_tmp1B0;
struct Cyc_Toc_FallthruInfo*fi=({struct Cyc_Toc_FallthruInfo*_tmp1B3=_region_malloc(r,sizeof(*_tmp1B3));
_tmp1B3->label=fallthru_l,_tmp1B3->binders=fallthru_binders;_tmp1B3;});
return({struct Cyc_Toc_Env*_tmp1B2=_region_malloc(r,sizeof(*_tmp1B2));({struct _fat_ptr**_tmp6A3=({struct _fat_ptr**_tmp1B1=_region_malloc(r,sizeof(*_tmp1B1));*_tmp1B1=break_l;_tmp1B1;});_tmp1B2->break_lab=_tmp6A3;}),_tmp1B2->continue_lab=c,_tmp1B2->fallthru_info=fi,_tmp1B2->toplevel=t,_tmp1B2->in_lhs=lhs,_tmp1B2->rgn=r;_tmp1B2;});}}
# 1115
static struct Cyc_Toc_Env*Cyc_Toc_last_switchclause_env(struct _RegionHandle*r,struct Cyc_Toc_Env*e,struct _fat_ptr*break_l){
# 1164
int*_tmp1B8;int _tmp1B7;struct Cyc_Toc_FallthruInfo*_tmp1B6;struct _fat_ptr**_tmp1B5;struct _fat_ptr**_tmp1B4;_LL1: _tmp1B4=e->break_lab;_tmp1B5=e->continue_lab;_tmp1B6=e->fallthru_info;_tmp1B7=e->toplevel;_tmp1B8=e->in_lhs;_LL2: {struct _fat_ptr**b=_tmp1B4;struct _fat_ptr**c=_tmp1B5;struct Cyc_Toc_FallthruInfo*f=_tmp1B6;int t=_tmp1B7;int*lhs=_tmp1B8;
return({struct Cyc_Toc_Env*_tmp1BA=_region_malloc(r,sizeof(*_tmp1BA));({struct _fat_ptr**_tmp6A4=({struct _fat_ptr**_tmp1B9=_region_malloc(r,sizeof(*_tmp1B9));*_tmp1B9=break_l;_tmp1B9;});_tmp1BA->break_lab=_tmp6A4;}),_tmp1BA->continue_lab=c,_tmp1BA->fallthru_info=0,_tmp1BA->toplevel=t,_tmp1BA->in_lhs=lhs,_tmp1BA->rgn=r;_tmp1BA;});}}
# 1115
static struct Cyc_Toc_Env*Cyc_Toc_switch_as_switch_env(struct _RegionHandle*r,struct Cyc_Toc_Env*e,struct _fat_ptr*next_l){
# 1173
int*_tmp1BF;int _tmp1BE;struct Cyc_Toc_FallthruInfo*_tmp1BD;struct _fat_ptr**_tmp1BC;struct _fat_ptr**_tmp1BB;_LL1: _tmp1BB=e->break_lab;_tmp1BC=e->continue_lab;_tmp1BD=e->fallthru_info;_tmp1BE=e->toplevel;_tmp1BF=e->in_lhs;_LL2: {struct _fat_ptr**b=_tmp1BB;struct _fat_ptr**c=_tmp1BC;struct Cyc_Toc_FallthruInfo*f=_tmp1BD;int t=_tmp1BE;int*lhs=_tmp1BF;
return({struct Cyc_Toc_Env*_tmp1C1=_region_malloc(r,sizeof(*_tmp1C1));_tmp1C1->break_lab=0,_tmp1C1->continue_lab=c,({struct Cyc_Toc_FallthruInfo*_tmp6A5=({struct Cyc_Toc_FallthruInfo*_tmp1C0=_region_malloc(r,sizeof(*_tmp1C0));_tmp1C0->label=next_l,_tmp1C0->binders=0;_tmp1C0;});_tmp1C1->fallthru_info=_tmp6A5;}),_tmp1C1->toplevel=t,_tmp1C1->in_lhs=lhs,_tmp1C1->rgn=r;_tmp1C1;});}}
# 1115
static void Cyc_Toc_exp_to_c(struct Cyc_Toc_Env*nv,struct Cyc_Absyn_Exp*e);
# 1187 "toc.cyc"
static void Cyc_Toc_stmt_to_c(struct Cyc_Toc_Env*nv,struct Cyc_Absyn_Stmt*s);
# 1189
static int Cyc_Toc_do_null_check(struct Cyc_Absyn_Exp*e){
void*_stmttmp1A=e->annot;void*_tmp1C2=_stmttmp1A;if(((struct Cyc_InsertChecks_NoCheck_Absyn_AbsynAnnot_struct*)_tmp1C2)->tag == Cyc_InsertChecks_NoCheck){_LL1: _LL2:
 return 0;}else{if(((struct Cyc_InsertChecks_NullOnly_Absyn_AbsynAnnot_struct*)_tmp1C2)->tag == Cyc_InsertChecks_NullOnly){_LL3: _LL4:
 goto _LL6;}else{if(((struct Cyc_InsertChecks_NullAndFatBound_Absyn_AbsynAnnot_struct*)_tmp1C2)->tag == Cyc_InsertChecks_NullAndFatBound){_LL5: _LL6:
 goto _LL8;}else{if(((struct Cyc_InsertChecks_NullAndThinBound_Absyn_AbsynAnnot_struct*)_tmp1C2)->tag == Cyc_InsertChecks_NullAndThinBound){_LL7: _LL8:
 return 1;}else{if(((struct Cyc_InsertChecks_FatBound_Absyn_AbsynAnnot_struct*)_tmp1C2)->tag == Cyc_InsertChecks_FatBound){_LL9: _LLA:
 goto _LLC;}else{if(((struct Cyc_InsertChecks_ThinBound_Absyn_AbsynAnnot_struct*)_tmp1C2)->tag == Cyc_InsertChecks_ThinBound){_LLB: _LLC:
 return 0;}else{_LLD: _LLE:
({void*_tmp1C3=0U;({int(*_tmp6A8)(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp1C5)(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos_loc;_tmp1C5;});unsigned _tmp6A7=e->loc;struct _fat_ptr _tmp6A6=({const char*_tmp1C4="Toc: do_null_check";_tag_fat(_tmp1C4,sizeof(char),19U);});_tmp6A8(_tmp6A7,_tmp6A6,_tag_fat(_tmp1C3,sizeof(void*),0U));});});}}}}}}_LL0:;}
# 1189
static int Cyc_Toc_ptr_use_to_c(struct Cyc_Toc_Env*nv,struct Cyc_Absyn_Exp*ptr,void*annot,struct Cyc_Absyn_Exp*index){
# 1207
int ans;
int old_lhs=Cyc_Toc_in_lhs(nv);
void*old_typ=Cyc_Tcutil_compress((void*)_check_null(ptr->topt));
void*new_typ=Cyc_Toc_typ_to_c(old_typ);
struct Cyc_Absyn_Exp*fn_e=Cyc_Toc__check_known_subscript_notnull_e;
Cyc_Toc_set_lhs(nv,0);
Cyc_Toc_exp_to_c(nv,ptr);
if(index != 0)
Cyc_Toc_exp_to_c(nv,index);{
# 1214
void*_tmp1C6=old_typ;void*_tmp1CA;void*_tmp1C9;struct Cyc_Absyn_Tqual _tmp1C8;void*_tmp1C7;if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp1C6)->tag == 3U){_LL1: _tmp1C7=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp1C6)->f1).elt_type;_tmp1C8=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp1C6)->f1).elt_tq;_tmp1C9=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp1C6)->f1).ptr_atts).bounds;_tmp1CA=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp1C6)->f1).ptr_atts).zero_term;_LL2: {void*ta=_tmp1C7;struct Cyc_Absyn_Tqual tq=_tmp1C8;void*b=_tmp1C9;void*zt=_tmp1CA;
# 1218
{void*_tmp1CB=annot;struct Cyc_Absyn_Exp*_tmp1CC;struct Cyc_Absyn_Exp*_tmp1CD;if(((struct Cyc_InsertChecks_NoCheck_Absyn_AbsynAnnot_struct*)_tmp1CB)->tag == Cyc_InsertChecks_NoCheck){_LL6: _LL7:
# 1220
 if(!((unsigned)({void*_tmp6A9=Cyc_Absyn_bounds_one();Cyc_Tcutil_get_bounds_exp(_tmp6A9,b);}))){
# 1223
void*newt=({void*_tmp6AA=Cyc_Toc_typ_to_c(ta);Cyc_Absyn_cstar_type(_tmp6AA,tq);});
({void*_tmp6AD=({void*_tmp6AC=newt;Cyc_Toc_cast_it_r(_tmp6AC,({struct Cyc_Absyn_Exp*_tmp6AB=Cyc_Absyn_new_exp(ptr->r,0U);Cyc_Toc_member_exp(_tmp6AB,Cyc_Toc_curr_sp,0U);}));});ptr->r=_tmp6AD;});
ptr->topt=newt;}
# 1220
ans=0;
# 1228
goto _LL5;}else{if(((struct Cyc_InsertChecks_NullOnly_Absyn_AbsynAnnot_struct*)_tmp1CB)->tag == Cyc_InsertChecks_NullOnly){_LL8: _LL9:
# 1230
 if(!((unsigned)({void*_tmp6AE=Cyc_Absyn_bounds_one();Cyc_Tcutil_get_bounds_exp(_tmp6AE,b);}))){
# 1232
void*newt=({void*_tmp6AF=Cyc_Toc_typ_to_c(ta);Cyc_Absyn_cstar_type(_tmp6AF,tq);});
({void*_tmp6B2=({void*_tmp6B1=newt;Cyc_Toc_cast_it_r(_tmp6B1,({struct Cyc_Absyn_Exp*_tmp6B0=Cyc_Absyn_new_exp(ptr->r,0U);Cyc_Toc_member_exp(_tmp6B0,Cyc_Toc_curr_sp,0U);}));});ptr->r=_tmp6B2;});
ptr->topt=newt;
# 1239
if(index != 0)
({void*_tmp1CE=0U;({int(*_tmp6B4)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp1D0)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Toc_toc_impos;_tmp1D0;});struct _fat_ptr _tmp6B3=({const char*_tmp1CF="subscript of ? with no bounds check but need null check";_tag_fat(_tmp1CF,sizeof(char),56U);});_tmp6B4(_tmp6B3,_tag_fat(_tmp1CE,sizeof(void*),0U));});});}
# 1230
({void*_tmp6B8=({
# 1243
void*_tmp6B7=new_typ;Cyc_Toc_cast_it_r(_tmp6B7,({struct Cyc_Absyn_Exp*_tmp1D1[1U];({struct Cyc_Absyn_Exp*_tmp6B5=Cyc_Absyn_new_exp(ptr->r,0U);_tmp1D1[0]=_tmp6B5;});({struct Cyc_Absyn_Exp*_tmp6B6=Cyc_Toc__check_null_e;Cyc_Toc_fncall_exp_dl(_tmp6B6,_tag_fat(_tmp1D1,sizeof(struct Cyc_Absyn_Exp*),1U));});}));});
# 1230
ptr->r=_tmp6B8;});
# 1244
ans=0;
goto _LL5;}else{if(((struct Cyc_InsertChecks_NullAndFatBound_Absyn_AbsynAnnot_struct*)_tmp1CB)->tag == Cyc_InsertChecks_NullAndFatBound){_LLA: _LLB:
 goto _LLD;}else{if(((struct Cyc_InsertChecks_FatBound_Absyn_AbsynAnnot_struct*)_tmp1CB)->tag == Cyc_InsertChecks_FatBound){_LLC: _LLD: {
# 1249
void*ta1=Cyc_Toc_typ_to_c(ta);
void*newt=Cyc_Absyn_cstar_type(ta1,tq);
struct Cyc_Absyn_Exp*ind=(unsigned)index?index: Cyc_Absyn_uint_exp(0U,0U);
({void*_tmp6BD=({void*_tmp6BC=newt;Cyc_Toc_cast_it_r(_tmp6BC,({struct Cyc_Absyn_Exp*_tmp1D2[3U];({
struct Cyc_Absyn_Exp*_tmp6BA=Cyc_Absyn_new_exp(ptr->r,0U);_tmp1D2[0]=_tmp6BA;}),({
struct Cyc_Absyn_Exp*_tmp6B9=Cyc_Absyn_sizeoftype_exp(ta1,0U);_tmp1D2[1]=_tmp6B9;}),_tmp1D2[2]=ind;({struct Cyc_Absyn_Exp*_tmp6BB=Cyc_Toc__check_fat_subscript_e;Cyc_Toc_fncall_exp_dl(_tmp6BB,_tag_fat(_tmp1D2,sizeof(struct Cyc_Absyn_Exp*),3U));});}));});
# 1252
ptr->r=_tmp6BD;});
# 1256
ptr->topt=newt;
ans=1;
goto _LL5;}}else{if(((struct Cyc_InsertChecks_NullAndThinBound_Absyn_AbsynAnnot_struct*)_tmp1CB)->tag == Cyc_InsertChecks_NullAndThinBound){_LLE: _tmp1CD=((struct Cyc_InsertChecks_NullAndThinBound_Absyn_AbsynAnnot_struct*)_tmp1CB)->f1;_LLF: {struct Cyc_Absyn_Exp*bd=_tmp1CD;
# 1260
fn_e=Cyc_Toc__check_known_subscript_null_e;
_tmp1CC=bd;goto _LL11;}}else{if(((struct Cyc_InsertChecks_ThinBound_Absyn_AbsynAnnot_struct*)_tmp1CB)->tag == Cyc_InsertChecks_ThinBound){_LL10: _tmp1CC=((struct Cyc_InsertChecks_ThinBound_Absyn_AbsynAnnot_struct*)_tmp1CB)->f1;_LL11: {struct Cyc_Absyn_Exp*bd=_tmp1CC;
# 1263
void*ta1=Cyc_Toc_typ_to_c(ta);
struct Cyc_Absyn_Exp*ind=(unsigned)index?index: Cyc_Absyn_uint_exp(0U,0U);
# 1268
struct _tuple14 _stmttmp1B=Cyc_Evexp_eval_const_uint_exp(bd);int _tmp1D4;unsigned _tmp1D3;_LL15: _tmp1D3=_stmttmp1B.f1;_tmp1D4=_stmttmp1B.f2;_LL16: {unsigned i=_tmp1D3;int valid=_tmp1D4;
if((!valid || i != (unsigned)1)|| !Cyc_Tcutil_is_zeroterm_pointer_type((void*)_check_null(ptr->topt))){
# 1271
({void*_tmp6C2=({void*_tmp6C1=Cyc_Absyn_cstar_type(ta1,tq);Cyc_Toc_cast_it_r(_tmp6C1,({struct Cyc_Absyn_Exp*_tmp1D5[4U];({
struct Cyc_Absyn_Exp*_tmp6BF=Cyc_Absyn_new_exp(ptr->r,0U);_tmp1D5[0]=_tmp6BF;}),_tmp1D5[1]=bd,({
struct Cyc_Absyn_Exp*_tmp6BE=Cyc_Absyn_sizeoftype_exp(ta1,0U);_tmp1D5[2]=_tmp6BE;}),_tmp1D5[3]=ind;({struct Cyc_Absyn_Exp*_tmp6C0=fn_e;Cyc_Toc_fncall_exp_dl(_tmp6C0,_tag_fat(_tmp1D5,sizeof(struct Cyc_Absyn_Exp*),4U));});}));});
# 1271
ptr->r=_tmp6C2;});
# 1274
ans=1;}else{
# 1277
if(Cyc_Toc_is_zero(bd)){
if(fn_e == Cyc_Toc__check_known_subscript_null_e)
# 1280
({void*_tmp6C6=({void*_tmp6C5=new_typ;Cyc_Toc_cast_it_r(_tmp6C5,({struct Cyc_Absyn_Exp*_tmp1D6[1U];({struct Cyc_Absyn_Exp*_tmp6C3=
Cyc_Absyn_new_exp(ptr->r,0U);_tmp1D6[0]=_tmp6C3;});({struct Cyc_Absyn_Exp*_tmp6C4=Cyc_Toc__check_null_e;Cyc_Toc_fncall_exp_dl(_tmp6C4,_tag_fat(_tmp1D6,sizeof(struct Cyc_Absyn_Exp*),1U));});}));});
# 1280
ptr->r=_tmp6C6;});
# 1278
ans=0;}else{
# 1286
fn_e=Cyc_Toc_getFunctionRemovePointer(& Cyc_Toc__zero_arr_plus_functionSet,ptr);
({void*_tmp6CA=({void*_tmp6C9=Cyc_Absyn_cstar_type(ta1,tq);Cyc_Toc_cast_it_r(_tmp6C9,({struct Cyc_Absyn_Exp*_tmp1D7[3U];({
struct Cyc_Absyn_Exp*_tmp6C7=Cyc_Absyn_new_exp(ptr->r,0U);_tmp1D7[0]=_tmp6C7;}),_tmp1D7[1]=bd,_tmp1D7[2]=ind;({struct Cyc_Absyn_Exp*_tmp6C8=fn_e;Cyc_Toc_fncall_exp_dl(_tmp6C8,_tag_fat(_tmp1D7,sizeof(struct Cyc_Absyn_Exp*),3U));});}));});
# 1287
ptr->r=_tmp6CA;});
# 1289
ans=1;}}
# 1292
goto _LL5;}}}else{_LL12: _LL13:
({void*_tmp1D8=0U;({int(*_tmp6CC)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp1DA)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp1DA;});struct _fat_ptr _tmp6CB=({const char*_tmp1D9="FIX: ptr_use_to_c, bad annotation";_tag_fat(_tmp1D9,sizeof(char),34U);});_tmp6CC(_tmp6CB,_tag_fat(_tmp1D8,sizeof(void*),0U));});});}}}}}}_LL5:;}
# 1295
Cyc_Toc_set_lhs(nv,old_lhs);
return ans;}}else{_LL3: _LL4:
({void*_tmp1DB=0U;({int(*_tmp6CE)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp1DD)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp1DD;});struct _fat_ptr _tmp6CD=({const char*_tmp1DC="ptr_use_to_c: non-pointer-type";_tag_fat(_tmp1DC,sizeof(char),31U);});_tmp6CE(_tmp6CD,_tag_fat(_tmp1DB,sizeof(void*),0U));});});}_LL0:;}}
# 1189
static void*Cyc_Toc_get_cyc_type(struct Cyc_Absyn_Exp*e){
# 1302
if(e->topt == 0)({void*_tmp1DE=0U;({int(*_tmp6D0)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp1E0)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Toc_toc_impos;_tmp1E0;});struct _fat_ptr _tmp6CF=({const char*_tmp1DF="Missing type in primop ";_tag_fat(_tmp1DF,sizeof(char),24U);});_tmp6D0(_tmp6CF,_tag_fat(_tmp1DE,sizeof(void*),0U));});});return(void*)_check_null(e->topt);}
# 1189
static struct _tuple13*Cyc_Toc_tup_to_c(struct Cyc_Absyn_Exp*e){
# 1306
return({struct _tuple13*_tmp1E1=_cycalloc(sizeof(*_tmp1E1));_tmp1E1->f1=Cyc_Toc_mt_tq,({void*_tmp6D1=Cyc_Toc_typ_to_c((void*)_check_null(e->topt));_tmp1E1->f2=_tmp6D1;});_tmp1E1;});}
# 1189
static struct Cyc_Absyn_Exp*Cyc_Toc_array_length_exp(struct Cyc_Absyn_Exp*e){
# 1311
void*_stmttmp1C=e->r;void*_tmp1E2=_stmttmp1C;int _tmp1E4;struct Cyc_Absyn_Exp*_tmp1E3;int _tmp1E6;struct Cyc_Absyn_Exp*_tmp1E5;struct Cyc_List_List*_tmp1E7;switch(*((int*)_tmp1E2)){case 26U: _LL1: _tmp1E7=((struct Cyc_Absyn_Array_e_Absyn_Raw_exp_struct*)_tmp1E2)->f1;_LL2: {struct Cyc_List_List*dles=_tmp1E7;
# 1313
{struct Cyc_List_List*dles2=dles;for(0;dles2 != 0;dles2=dles2->tl){
if((*((struct _tuple20*)dles2->hd)).f1 != 0)
({void*_tmp1E8=0U;({int(*_tmp6D3)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp1EA)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Toc_unimp;_tmp1EA;});struct _fat_ptr _tmp6D2=({const char*_tmp1E9="array designators for abstract-field initialization";_tag_fat(_tmp1E9,sizeof(char),52U);});_tmp6D3(_tmp6D2,_tag_fat(_tmp1E8,sizeof(void*),0U));});});}}
# 1313
_tmp1E5=
# 1316
Cyc_Absyn_signed_int_exp(Cyc_List_length(dles),0U);_tmp1E6=0;goto _LL4;}case 27U: _LL3: _tmp1E5=((struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct*)_tmp1E2)->f2;_tmp1E6=((struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct*)_tmp1E2)->f4;_LL4: {struct Cyc_Absyn_Exp*bd=_tmp1E5;int zt=_tmp1E6;
_tmp1E3=bd;_tmp1E4=zt;goto _LL6;}case 28U: _LL5: _tmp1E3=((struct Cyc_Absyn_ComprehensionNoinit_e_Absyn_Raw_exp_struct*)_tmp1E2)->f1;_tmp1E4=((struct Cyc_Absyn_ComprehensionNoinit_e_Absyn_Raw_exp_struct*)_tmp1E2)->f3;_LL6: {struct Cyc_Absyn_Exp*bd=_tmp1E3;int zt=_tmp1E4;
# 1319
bd=Cyc_Absyn_copy_exp(bd);
return zt?({struct Cyc_Absyn_Exp*_tmp6D4=bd;Cyc_Absyn_add_exp(_tmp6D4,Cyc_Absyn_uint_exp(1U,0U),0U);}): bd;}default: _LL7: _LL8:
 return 0;}_LL0:;}
# 1189
static struct Cyc_Absyn_Exp*Cyc_Toc_get_varsizeexp(struct Cyc_Absyn_Exp*e){
# 1336
struct Cyc_List_List*dles;
struct Cyc_List_List*field_types;
{void*_stmttmp1D=e->r;void*_tmp1EB=_stmttmp1D;struct Cyc_List_List*_tmp1EC;if(((struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct*)_tmp1EB)->tag == 29U){_LL1: _tmp1EC=((struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct*)_tmp1EB)->f3;_LL2: {struct Cyc_List_List*dles2=_tmp1EC;
dles=dles2;goto _LL0;}}else{_LL3: _LL4:
 return 0;}_LL0:;}
# 1342
{void*_stmttmp1E=Cyc_Tcutil_compress((void*)_check_null(e->topt));void*_tmp1ED=_stmttmp1E;struct Cyc_Absyn_Aggrdecl*_tmp1EE;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp1ED)->tag == 0U){if(((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp1ED)->f1)->tag == 20U){if(((((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp1ED)->f1)->f1).KnownAggr).tag == 2){_LL6: _tmp1EE=*((((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp1ED)->f1)->f1).KnownAggr).val;_LL7: {struct Cyc_Absyn_Aggrdecl*ad=_tmp1EE;
# 1344
if(ad->impl == 0)
return 0;
# 1344
if((int)ad->kind == (int)Cyc_Absyn_UnionA)
# 1347
return 0;
# 1344
field_types=((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->fields;
# 1349
goto _LL5;}}else{goto _LL8;}}else{goto _LL8;}}else{_LL8: _LL9:
# 1352
 return 0;}_LL5:;}
# 1354
if(field_types == 0)
return 0;
# 1354
for(0;((struct Cyc_List_List*)_check_null(field_types))->tl != 0;field_types=field_types->tl){
# 1357
;}{
struct Cyc_Absyn_Aggrfield*last_type_field=(struct Cyc_Absyn_Aggrfield*)field_types->hd;
for(0;dles != 0;dles=dles->tl){
struct _tuple20*_stmttmp1F=(struct _tuple20*)dles->hd;struct Cyc_Absyn_Exp*_tmp1F0;struct Cyc_List_List*_tmp1EF;_LLB: _tmp1EF=_stmttmp1F->f1;_tmp1F0=_stmttmp1F->f2;_LLC: {struct Cyc_List_List*ds=_tmp1EF;struct Cyc_Absyn_Exp*e2=_tmp1F0;
struct _fat_ptr*f=Cyc_Absyn_designatorlist_to_fieldname(ds);
if(!Cyc_strptrcmp(f,last_type_field->name)){
struct Cyc_Absyn_Exp*nested_ans=Cyc_Toc_get_varsizeexp(e2);
if(nested_ans != 0)
return nested_ans;{
# 1364
void*_stmttmp20=
# 1366
Cyc_Tcutil_compress(last_type_field->type);
# 1364
void*_tmp1F1=_stmttmp20;struct Cyc_Absyn_Exp*_tmp1F3;void*_tmp1F2;if(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp1F1)->tag == 4U){_LLE: _tmp1F2=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp1F1)->f1).elt_type;_tmp1F3=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp1F1)->f1).num_elts;_LLF: {void*elt_type=_tmp1F2;struct Cyc_Absyn_Exp*type_bd=_tmp1F3;
# 1369
if(type_bd == 0 || !Cyc_Toc_is_zero(type_bd))
return 0;
# 1369
return({
# 1375
struct Cyc_Absyn_Exp*_tmp6D7=({struct Cyc_Absyn_Exp*_tmp1F4[2U];_tmp1F4[0]=(struct Cyc_Absyn_Exp*)_check_null(Cyc_Toc_array_length_exp(e2)),({
# 1377
struct Cyc_Absyn_Exp*_tmp6D5=Cyc_Absyn_sizeoftype_exp(elt_type,0U);_tmp1F4[1]=_tmp6D5;});({struct Cyc_Absyn_Exp*_tmp6D6=Cyc_Toc__check_times_e;Cyc_Toc_fncall_exp_dl(_tmp6D6,_tag_fat(_tmp1F4,sizeof(struct Cyc_Absyn_Exp*),2U));});});
# 1375
Cyc_Absyn_add_exp(_tmp6D7,
# 1378
Cyc_Absyn_signed_int_exp((int)sizeof(double),0U),0U);});}}else{_LL10: _LL11:
 return 0;}_LLD:;}}}}
# 1383
({void*_tmp1F5=0U;({int(*_tmp6D9)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp1F7)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Toc_toc_impos;_tmp1F7;});struct _fat_ptr _tmp6D8=({const char*_tmp1F6="get_varsizeexp: did not find last struct field";_tag_fat(_tmp1F6,sizeof(char),47U);});_tmp6D9(_tmp6D8,_tag_fat(_tmp1F5,sizeof(void*),0U));});});}}
# 1189
static int Cyc_Toc_get_member_offset(struct Cyc_Absyn_Aggrdecl*ad,struct _fat_ptr*f){
# 1387
int i=1;
{struct Cyc_List_List*fields=((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->fields;for(0;fields != 0;fields=fields->tl){
struct Cyc_Absyn_Aggrfield*field=(struct Cyc_Absyn_Aggrfield*)fields->hd;
if(Cyc_strcmp((struct _fat_ptr)*field->name,(struct _fat_ptr)*f)== 0)return i;++ i;}}
# 1393
({void*_tmp1F8=0U;({int(*_tmp6DC)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp1FC)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Toc_toc_impos;_tmp1FC;});struct _fat_ptr _tmp6DB=(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp1FB=({struct Cyc_String_pa_PrintArg_struct _tmp5EE;_tmp5EE.tag=0U,_tmp5EE.f1=(struct _fat_ptr)((struct _fat_ptr)*f);_tmp5EE;});void*_tmp1F9[1U];_tmp1F9[0]=& _tmp1FB;({struct _fat_ptr _tmp6DA=({const char*_tmp1FA="get_member_offset %s failed";_tag_fat(_tmp1FA,sizeof(char),28U);});Cyc_aprintf(_tmp6DA,_tag_fat(_tmp1F9,sizeof(void*),1U));});});_tmp6DC(_tmp6DB,_tag_fat(_tmp1F8,sizeof(void*),0U));});});}struct _tuple32{struct Cyc_Core_Opt*f1;struct Cyc_Absyn_Exp*f2;};
# 1189
static struct Cyc_Absyn_Exp*Cyc_Toc_assignop_lvalue(struct Cyc_Absyn_Exp*el,struct _tuple32*pr){
# 1398
return Cyc_Absyn_assignop_exp(el,(*pr).f1,(*pr).f2,0U);}
# 1189
static struct Cyc_Absyn_Exp*Cyc_Toc_address_lvalue(struct Cyc_Absyn_Exp*e1,int ignore){
# 1401
return Cyc_Absyn_address_exp(e1,0U);}
# 1189
static struct Cyc_Absyn_Exp*Cyc_Toc_incr_lvalue(struct Cyc_Absyn_Exp*e1,enum Cyc_Absyn_Incrementor incr){
# 1404
return Cyc_Absyn_new_exp((void*)({struct Cyc_Absyn_Increment_e_Absyn_Raw_exp_struct*_tmp1FD=_cycalloc(sizeof(*_tmp1FD));_tmp1FD->tag=5U,_tmp1FD->f1=e1,_tmp1FD->f2=incr;_tmp1FD;}),0U);}
# 1189
static void Cyc_Toc_lvalue_assign_stmt(struct Cyc_Absyn_Stmt*s,struct Cyc_List_List*fs,struct Cyc_Absyn_Exp*(*f)(struct Cyc_Absyn_Exp*,void*),void*f_env);
# 1416
static void Cyc_Toc_lvalue_assign(struct Cyc_Absyn_Exp*e1,struct Cyc_List_List*fs,struct Cyc_Absyn_Exp*(*f)(struct Cyc_Absyn_Exp*,void*),void*f_env){
# 1418
void*_stmttmp21=e1->r;void*_tmp1FE=_stmttmp21;int _tmp202;int _tmp201;struct _fat_ptr*_tmp200;struct Cyc_Absyn_Exp*_tmp1FF;struct Cyc_Absyn_Exp*_tmp204;void*_tmp203;struct Cyc_Absyn_Stmt*_tmp205;switch(*((int*)_tmp1FE)){case 38U: _LL1: _tmp205=((struct Cyc_Absyn_StmtExp_e_Absyn_Raw_exp_struct*)_tmp1FE)->f1;_LL2: {struct Cyc_Absyn_Stmt*s=_tmp205;
Cyc_Toc_lvalue_assign_stmt(s,fs,f,f_env);goto _LL0;}case 14U: _LL3: _tmp203=(void*)((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp1FE)->f1;_tmp204=((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp1FE)->f2;_LL4: {void*t=_tmp203;struct Cyc_Absyn_Exp*e=_tmp204;
Cyc_Toc_lvalue_assign(e,fs,f,f_env);goto _LL0;}case 21U: _LL5: _tmp1FF=((struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct*)_tmp1FE)->f1;_tmp200=((struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct*)_tmp1FE)->f2;_tmp201=((struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct*)_tmp1FE)->f3;_tmp202=((struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct*)_tmp1FE)->f4;_LL6: {struct Cyc_Absyn_Exp*e=_tmp1FF;struct _fat_ptr*fld=_tmp200;int is_tagged=_tmp201;int is_read=_tmp202;
# 1423
e1->r=e->r;
({struct Cyc_Absyn_Exp*_tmp6DF=e1;struct Cyc_List_List*_tmp6DE=({struct Cyc_List_List*_tmp206=_cycalloc(sizeof(*_tmp206));_tmp206->hd=fld,_tmp206->tl=fs;_tmp206;});struct Cyc_Absyn_Exp*(*_tmp6DD)(struct Cyc_Absyn_Exp*,void*)=f;Cyc_Toc_lvalue_assign(_tmp6DF,_tmp6DE,_tmp6DD,f_env);});
goto _LL0;}default: _LL7: _LL8: {
# 1431
struct Cyc_Absyn_Exp*e1_copy=Cyc_Absyn_copy_exp(e1);
# 1433
for(0;fs != 0;fs=fs->tl){
e1_copy=Cyc_Toc_member_exp(e1_copy,(struct _fat_ptr*)fs->hd,e1_copy->loc);}
({void*_tmp6E0=(f(e1_copy,f_env))->r;e1->r=_tmp6E0;});
goto _LL0;}}_LL0:;}
# 1416
static void Cyc_Toc_lvalue_assign_stmt(struct Cyc_Absyn_Stmt*s,struct Cyc_List_List*fs,struct Cyc_Absyn_Exp*(*f)(struct Cyc_Absyn_Exp*,void*),void*f_env){
# 1441
void*_stmttmp22=s->r;void*_tmp207=_stmttmp22;struct Cyc_Absyn_Stmt*_tmp208;struct Cyc_Absyn_Stmt*_tmp20A;struct Cyc_Absyn_Decl*_tmp209;struct Cyc_Absyn_Exp*_tmp20B;switch(*((int*)_tmp207)){case 1U: _LL1: _tmp20B=((struct Cyc_Absyn_Exp_s_Absyn_Raw_stmt_struct*)_tmp207)->f1;_LL2: {struct Cyc_Absyn_Exp*e1=_tmp20B;
Cyc_Toc_lvalue_assign(e1,fs,f,f_env);goto _LL0;}case 12U: _LL3: _tmp209=((struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct*)_tmp207)->f1;_tmp20A=((struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct*)_tmp207)->f2;_LL4: {struct Cyc_Absyn_Decl*d=_tmp209;struct Cyc_Absyn_Stmt*s2=_tmp20A;
# 1444
Cyc_Toc_lvalue_assign_stmt(s2,fs,f,f_env);goto _LL0;}case 2U: _LL5: _tmp208=((struct Cyc_Absyn_Seq_s_Absyn_Raw_stmt_struct*)_tmp207)->f2;_LL6: {struct Cyc_Absyn_Stmt*s2=_tmp208;
Cyc_Toc_lvalue_assign_stmt(s2,fs,f,f_env);goto _LL0;}default: _LL7: _LL8:
({struct Cyc_String_pa_PrintArg_struct _tmp20E=({struct Cyc_String_pa_PrintArg_struct _tmp5EF;_tmp5EF.tag=0U,({struct _fat_ptr _tmp6E1=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_stmt2string(s));_tmp5EF.f1=_tmp6E1;});_tmp5EF;});void*_tmp20C[1U];_tmp20C[0]=& _tmp20E;({struct _fat_ptr _tmp6E2=({const char*_tmp20D="lvalue_assign_stmt: %s";_tag_fat(_tmp20D,sizeof(char),23U);});Cyc_Toc_toc_impos(_tmp6E2,_tag_fat(_tmp20C,sizeof(void*),1U));});});}_LL0:;}
# 1416
static void Cyc_Toc_push_address_stmt(struct Cyc_Absyn_Stmt*s);
# 1454
static struct Cyc_Absyn_Exp*Cyc_Toc_push_address_exp(struct Cyc_Absyn_Exp*e){
void*_stmttmp23=e->r;void*_tmp20F=_stmttmp23;struct Cyc_Absyn_Stmt*_tmp210;struct Cyc_Absyn_Exp*_tmp211;struct Cyc_Absyn_Exp**_tmp213;void**_tmp212;switch(*((int*)_tmp20F)){case 14U: _LL1: _tmp212=(void**)&((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp20F)->f1;_tmp213=(struct Cyc_Absyn_Exp**)&((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp20F)->f2;_LL2: {void**t=_tmp212;struct Cyc_Absyn_Exp**e1=_tmp213;
# 1457
({struct Cyc_Absyn_Exp*_tmp6E3=Cyc_Toc_push_address_exp(*e1);*e1=_tmp6E3;});
({void*_tmp6E4=Cyc_Absyn_cstar_type(*t,Cyc_Toc_mt_tq);*t=_tmp6E4;});
return e;}case 20U: _LL3: _tmp211=((struct Cyc_Absyn_Deref_e_Absyn_Raw_exp_struct*)_tmp20F)->f1;_LL4: {struct Cyc_Absyn_Exp*e1=_tmp211;
# 1461
return e1;}case 38U: _LL5: _tmp210=((struct Cyc_Absyn_StmtExp_e_Absyn_Raw_exp_struct*)_tmp20F)->f1;_LL6: {struct Cyc_Absyn_Stmt*s=_tmp210;
# 1465
Cyc_Toc_push_address_stmt(s);
return e;}default: _LL7: _LL8:
# 1468
 if(Cyc_Absyn_is_lvalue(e))return Cyc_Absyn_address_exp(e,0U);({struct Cyc_String_pa_PrintArg_struct _tmp217=({struct Cyc_String_pa_PrintArg_struct _tmp5F0;_tmp5F0.tag=0U,({
struct _fat_ptr _tmp6E5=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_exp2string(e));_tmp5F0.f1=_tmp6E5;});_tmp5F0;});void*_tmp214[1U];_tmp214[0]=& _tmp217;({int(*_tmp6E7)(struct _fat_ptr fmt,struct _fat_ptr ap)=({
# 1468
int(*_tmp216)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Toc_toc_impos;_tmp216;});struct _fat_ptr _tmp6E6=({const char*_tmp215="can't take & of exp %s";_tag_fat(_tmp215,sizeof(char),23U);});_tmp6E7(_tmp6E6,_tag_fat(_tmp214,sizeof(void*),1U));});});}_LL0:;}
# 1454
static void Cyc_Toc_push_address_stmt(struct Cyc_Absyn_Stmt*s){
# 1473
void*_stmttmp24=s->r;void*_tmp218=_stmttmp24;struct Cyc_Absyn_Exp**_tmp219;struct Cyc_Absyn_Stmt*_tmp21A;struct Cyc_Absyn_Stmt*_tmp21B;switch(*((int*)_tmp218)){case 2U: _LL1: _tmp21B=((struct Cyc_Absyn_Seq_s_Absyn_Raw_stmt_struct*)_tmp218)->f2;_LL2: {struct Cyc_Absyn_Stmt*s2=_tmp21B;
_tmp21A=s2;goto _LL4;}case 12U: _LL3: _tmp21A=((struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct*)_tmp218)->f2;_LL4: {struct Cyc_Absyn_Stmt*s2=_tmp21A;
Cyc_Toc_push_address_stmt(s2);goto _LL0;}case 1U: _LL5: _tmp219=(struct Cyc_Absyn_Exp**)&((struct Cyc_Absyn_Exp_s_Absyn_Raw_stmt_struct*)_tmp218)->f1;_LL6: {struct Cyc_Absyn_Exp**e=_tmp219;
({struct Cyc_Absyn_Exp*_tmp6E8=Cyc_Toc_push_address_exp(*e);*e=_tmp6E8;});goto _LL0;}default: _LL7: _LL8:
({struct Cyc_String_pa_PrintArg_struct _tmp21F=({struct Cyc_String_pa_PrintArg_struct _tmp5F1;_tmp5F1.tag=0U,({struct _fat_ptr _tmp6E9=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_stmt2string(s));_tmp5F1.f1=_tmp6E9;});_tmp5F1;});void*_tmp21C[1U];_tmp21C[0]=& _tmp21F;({int(*_tmp6EB)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp21E)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Toc_toc_impos;_tmp21E;});struct _fat_ptr _tmp6EA=({const char*_tmp21D="can't take & of stmt %s";_tag_fat(_tmp21D,sizeof(char),24U);});_tmp6EB(_tmp6EA,_tag_fat(_tmp21C,sizeof(void*),1U));});});}_LL0:;}
# 1454
static void Cyc_Toc_zero_ptr_assign_to_c(struct Cyc_Toc_Env*nv,struct Cyc_Absyn_Exp*e,struct Cyc_Absyn_Exp*e1,struct Cyc_Core_Opt*popt,struct Cyc_Absyn_Exp*e2,void*ptr_type,int is_fat,void*elt_type){
# 1496
void*fat_ptr_type=Cyc_Absyn_fatptr_type(elt_type,Cyc_Absyn_heap_rgn_type,Cyc_Toc_mt_tq,Cyc_Absyn_true_type);
void*c_elt_type=Cyc_Toc_typ_to_c(elt_type);
void*c_fat_ptr_type=Cyc_Toc_typ_to_c(fat_ptr_type);
void*c_ptr_type=Cyc_Absyn_cstar_type(c_elt_type,Cyc_Toc_mt_tq);
struct Cyc_Core_Opt*c_ptr_type_opt=({struct Cyc_Core_Opt*_tmp237=_cycalloc(sizeof(*_tmp237));_tmp237->v=c_ptr_type;_tmp237;});
struct Cyc_Absyn_Exp*xinit;
{void*_stmttmp25=e1->r;void*_tmp220=_stmttmp25;struct Cyc_Absyn_Exp*_tmp222;struct Cyc_Absyn_Exp*_tmp221;struct Cyc_Absyn_Exp*_tmp223;switch(*((int*)_tmp220)){case 20U: _LL1: _tmp223=((struct Cyc_Absyn_Deref_e_Absyn_Raw_exp_struct*)_tmp220)->f1;_LL2: {struct Cyc_Absyn_Exp*ea=_tmp223;
# 1504
if(!is_fat){
ea=Cyc_Absyn_cast_exp(fat_ptr_type,ea,0,Cyc_Absyn_Other_coercion,0U);
ea->topt=fat_ptr_type;
ea->annot=(void*)& Cyc_InsertChecks_NoCheck_val;}
# 1504
Cyc_Toc_exp_to_c(nv,ea);
# 1509
xinit=ea;goto _LL0;}case 23U: _LL3: _tmp221=((struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct*)_tmp220)->f1;_tmp222=((struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct*)_tmp220)->f2;_LL4: {struct Cyc_Absyn_Exp*ea=_tmp221;struct Cyc_Absyn_Exp*eb=_tmp222;
# 1511
if(!is_fat){
ea=Cyc_Absyn_cast_exp(fat_ptr_type,ea,0,Cyc_Absyn_Other_coercion,0U);
ea->topt=fat_ptr_type;
ea->annot=(void*)& Cyc_InsertChecks_NoCheck_val;}
# 1511
Cyc_Toc_exp_to_c(nv,ea);
# 1516
Cyc_Toc_exp_to_c(nv,eb);
xinit=({struct Cyc_Absyn_Exp*_tmp224[3U];_tmp224[0]=ea,({
struct Cyc_Absyn_Exp*_tmp6EC=Cyc_Absyn_sizeoftype_exp(Cyc_Toc_typ_to_c(elt_type),0U);_tmp224[1]=_tmp6EC;}),_tmp224[2]=eb;({struct Cyc_Absyn_Exp*_tmp6ED=Cyc_Toc__fat_ptr_plus_e;Cyc_Toc_fncall_exp_dl(_tmp6ED,_tag_fat(_tmp224,sizeof(struct Cyc_Absyn_Exp*),3U));});});
goto _LL0;}default: _LL5: _LL6:
({void*_tmp225=0U;({int(*_tmp6EF)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp227)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Toc_toc_impos;_tmp227;});struct _fat_ptr _tmp6EE=({const char*_tmp226="found bad lhs for zero-terminated pointer assignment";_tag_fat(_tmp226,sizeof(char),53U);});_tmp6EF(_tmp6EE,_tag_fat(_tmp225,sizeof(void*),0U));});});}_LL0:;}{
# 1522
struct _tuple1*x=Cyc_Toc_temp_var();
struct _RegionHandle _tmp228=_new_region("rgn2");struct _RegionHandle*rgn2=& _tmp228;_push_region(rgn2);
{struct Cyc_Absyn_Vardecl*x_vd=({struct Cyc_Absyn_Vardecl*_tmp236=_cycalloc(sizeof(*_tmp236));_tmp236->sc=Cyc_Absyn_Public,_tmp236->name=x,_tmp236->varloc=0U,_tmp236->tq=Cyc_Toc_mt_tq,_tmp236->type=c_fat_ptr_type,_tmp236->initializer=xinit,_tmp236->rgn=0,_tmp236->attributes=0,_tmp236->escapes=0;_tmp236;});
struct Cyc_Absyn_Local_b_Absyn_Binding_struct*x_bnd=({struct Cyc_Absyn_Local_b_Absyn_Binding_struct*_tmp235=_cycalloc(sizeof(*_tmp235));_tmp235->tag=4U,_tmp235->f1=x_vd;_tmp235;});
struct Cyc_Absyn_Exp*x_exp=Cyc_Absyn_varb_exp((void*)x_bnd,0U);
x_exp->topt=fat_ptr_type;{
struct Cyc_Absyn_Exp*deref_x=Cyc_Absyn_deref_exp(x_exp,0U);
deref_x->topt=elt_type;
deref_x->annot=(void*)& Cyc_InsertChecks_NullAndFatBound_val;
Cyc_Toc_exp_to_c(nv,deref_x);{
struct _tuple1*y=Cyc_Toc_temp_var();
struct Cyc_Absyn_Vardecl*y_vd=({struct Cyc_Absyn_Vardecl*_tmp234=_cycalloc(sizeof(*_tmp234));_tmp234->sc=Cyc_Absyn_Public,_tmp234->name=y,_tmp234->varloc=0U,_tmp234->tq=Cyc_Toc_mt_tq,_tmp234->type=c_elt_type,_tmp234->initializer=deref_x,_tmp234->rgn=0,_tmp234->attributes=0,_tmp234->escapes=0;_tmp234;});
struct Cyc_Absyn_Local_b_Absyn_Binding_struct*y_bnd=({struct Cyc_Absyn_Local_b_Absyn_Binding_struct*_tmp233=_cycalloc(sizeof(*_tmp233));_tmp233->tag=4U,_tmp233->f1=y_vd;_tmp233;});
struct Cyc_Absyn_Exp*z_init=e2;
if(popt != 0){
struct Cyc_Absyn_Exp*y_exp=Cyc_Absyn_varb_exp((void*)y_bnd,0U);
y_exp->topt=deref_x->topt;
z_init=({enum Cyc_Absyn_Primop _tmp6F1=(enum Cyc_Absyn_Primop)popt->v;struct Cyc_Absyn_Exp*_tmp6F0=y_exp;Cyc_Absyn_prim2_exp(_tmp6F1,_tmp6F0,Cyc_Absyn_copy_exp(e2),0U);});
z_init->topt=y_exp->topt;
z_init->annot=(void*)& Cyc_InsertChecks_NoCheck_val;}
# 1536
Cyc_Toc_exp_to_c(nv,z_init);{
# 1544
struct _tuple1*z=Cyc_Toc_temp_var();
struct Cyc_Absyn_Vardecl*z_vd=({struct Cyc_Absyn_Vardecl*_tmp232=_cycalloc(sizeof(*_tmp232));_tmp232->sc=Cyc_Absyn_Public,_tmp232->name=z,_tmp232->varloc=0U,_tmp232->tq=Cyc_Toc_mt_tq,_tmp232->type=c_elt_type,_tmp232->initializer=z_init,_tmp232->rgn=0,_tmp232->attributes=0,_tmp232->escapes=0;_tmp232;});
struct Cyc_Absyn_Local_b_Absyn_Binding_struct*z_bnd=({struct Cyc_Absyn_Local_b_Absyn_Binding_struct*_tmp231=_cycalloc(sizeof(*_tmp231));_tmp231->tag=4U,_tmp231->f1=z_vd;_tmp231;});
# 1549
struct Cyc_Absyn_Exp*y2_exp=Cyc_Absyn_varb_exp((void*)y_bnd,0U);y2_exp->topt=deref_x->topt;{
struct Cyc_Absyn_Exp*zero1_exp=Cyc_Absyn_signed_int_exp(0,0U);
struct Cyc_Absyn_Exp*comp1_exp=Cyc_Absyn_prim2_exp(Cyc_Absyn_Eq,y2_exp,zero1_exp,0U);
zero1_exp->topt=Cyc_Absyn_sint_type;
comp1_exp->topt=Cyc_Absyn_sint_type;
Cyc_Toc_exp_to_c(nv,comp1_exp);{
# 1556
struct Cyc_Absyn_Exp*z_exp=Cyc_Absyn_varb_exp((void*)z_bnd,0U);z_exp->topt=deref_x->topt;{
struct Cyc_Absyn_Exp*zero2_exp=Cyc_Absyn_signed_int_exp(0,0U);
struct Cyc_Absyn_Exp*comp2_exp=Cyc_Absyn_prim2_exp(Cyc_Absyn_Neq,z_exp,zero2_exp,0U);
zero2_exp->topt=Cyc_Absyn_sint_type;
comp2_exp->topt=Cyc_Absyn_sint_type;
Cyc_Toc_exp_to_c(nv,comp2_exp);{
# 1563
struct Cyc_List_List*xsizeargs=({struct Cyc_Absyn_Exp*_tmp230[2U];({struct Cyc_Absyn_Exp*_tmp6F3=Cyc_Absyn_varb_exp((void*)x_bnd,0U);_tmp230[0]=_tmp6F3;}),({
struct Cyc_Absyn_Exp*_tmp6F2=Cyc_Absyn_sizeoftype_exp(Cyc_Toc_typ_to_c(elt_type),0U);_tmp230[1]=_tmp6F2;});Cyc_List_list(_tag_fat(_tmp230,sizeof(struct Cyc_Absyn_Exp*),2U));});
struct Cyc_Absyn_Exp*oneexp=Cyc_Absyn_uint_exp(1U,0U);
struct Cyc_Absyn_Exp*xsize;
xsize=({struct Cyc_Absyn_Exp*_tmp6F4=Cyc_Absyn_fncall_exp(Cyc_Toc__get_fat_size_e,xsizeargs,0U);Cyc_Absyn_prim2_exp(Cyc_Absyn_Eq,_tmp6F4,oneexp,0U);});{
# 1570
struct Cyc_Absyn_Exp*comp_exp=({struct Cyc_Absyn_Exp*_tmp6F5=xsize;Cyc_Absyn_and_exp(_tmp6F5,Cyc_Absyn_and_exp(comp1_exp,comp2_exp,0U),0U);});
struct Cyc_Absyn_Stmt*thr_stmt=Cyc_Absyn_exp_stmt(({void*_tmp22F=0U;({struct Cyc_Absyn_Exp*_tmp6F6=Cyc_Toc__throw_arraybounds_e;Cyc_Toc_fncall_exp_dl(_tmp6F6,_tag_fat(_tmp22F,sizeof(struct Cyc_Absyn_Exp*),0U));});}),0U);
struct Cyc_Absyn_Exp*xcurr=({struct Cyc_Absyn_Exp*_tmp6F7=Cyc_Absyn_varb_exp((void*)x_bnd,0U);Cyc_Toc_member_exp(_tmp6F7,Cyc_Toc_curr_sp,0U);});
xcurr=Cyc_Toc_cast_it(c_ptr_type,xcurr);{
struct Cyc_Absyn_Exp*deref_xcurr=Cyc_Absyn_deref_exp(xcurr,0U);
struct Cyc_Absyn_Exp*asn_exp=({struct Cyc_Absyn_Exp*_tmp6F8=deref_xcurr;Cyc_Absyn_assign_exp(_tmp6F8,Cyc_Absyn_var_exp(z,0U),0U);});
struct Cyc_Absyn_Stmt*s=Cyc_Absyn_exp_stmt(asn_exp,0U);
s=({struct Cyc_Absyn_Stmt*_tmp6FB=({struct Cyc_Absyn_Exp*_tmp6FA=comp_exp;struct Cyc_Absyn_Stmt*_tmp6F9=thr_stmt;Cyc_Absyn_ifthenelse_stmt(_tmp6FA,_tmp6F9,Cyc_Absyn_skip_stmt(0U),0U);});Cyc_Absyn_seq_stmt(_tmp6FB,s,0U);});
s=({struct Cyc_Absyn_Decl*_tmp6FD=({struct Cyc_Absyn_Decl*_tmp22A=_cycalloc(sizeof(*_tmp22A));({void*_tmp6FC=(void*)({struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct*_tmp229=_cycalloc(sizeof(*_tmp229));_tmp229->tag=0U,_tmp229->f1=z_vd;_tmp229;});_tmp22A->r=_tmp6FC;}),_tmp22A->loc=0U;_tmp22A;});Cyc_Absyn_decl_stmt(_tmp6FD,s,0U);});
s=({struct Cyc_Absyn_Decl*_tmp6FF=({struct Cyc_Absyn_Decl*_tmp22C=_cycalloc(sizeof(*_tmp22C));({void*_tmp6FE=(void*)({struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct*_tmp22B=_cycalloc(sizeof(*_tmp22B));_tmp22B->tag=0U,_tmp22B->f1=y_vd;_tmp22B;});_tmp22C->r=_tmp6FE;}),_tmp22C->loc=0U;_tmp22C;});Cyc_Absyn_decl_stmt(_tmp6FF,s,0U);});
s=({struct Cyc_Absyn_Decl*_tmp701=({struct Cyc_Absyn_Decl*_tmp22E=_cycalloc(sizeof(*_tmp22E));({void*_tmp700=(void*)({struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct*_tmp22D=_cycalloc(sizeof(*_tmp22D));_tmp22D->tag=0U,_tmp22D->f1=x_vd;_tmp22D;});_tmp22E->r=_tmp700;}),_tmp22E->loc=0U;_tmp22E;});Cyc_Absyn_decl_stmt(_tmp701,s,0U);});
({void*_tmp702=Cyc_Toc_stmt_exp_r(s);e->r=_tmp702;});}}}}}}}}}}
# 1524
;_pop_region(rgn2);}}
# 1454
static void*Cyc_Toc_check_tagged_union(struct Cyc_Absyn_Exp*e1,void*e1_c_type,void*aggrtype,struct _fat_ptr*f,int in_lhs,struct Cyc_Absyn_Exp*(*aggrproj)(struct Cyc_Absyn_Exp*,struct _fat_ptr*,unsigned)){
# 1600 "toc.cyc"
struct Cyc_Absyn_Aggrdecl*ad;
{void*_stmttmp26=Cyc_Tcutil_compress(aggrtype);void*_tmp238=_stmttmp26;union Cyc_Absyn_AggrInfo _tmp239;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp238)->tag == 0U){if(((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp238)->f1)->tag == 20U){_LL1: _tmp239=((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp238)->f1)->f1;_LL2: {union Cyc_Absyn_AggrInfo info=_tmp239;
ad=Cyc_Absyn_get_known_aggrdecl(info);goto _LL0;}}else{goto _LL3;}}else{_LL3: _LL4:
({void*_tmp23A=0U;({int(*_tmp706)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp23E)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp23E;});struct _fat_ptr _tmp705=(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp23D=({struct Cyc_String_pa_PrintArg_struct _tmp5F2;_tmp5F2.tag=0U,({
struct _fat_ptr _tmp703=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string(aggrtype));_tmp5F2.f1=_tmp703;});_tmp5F2;});void*_tmp23B[1U];_tmp23B[0]=& _tmp23D;({struct _fat_ptr _tmp704=({const char*_tmp23C="expecting union but found %s in check_tagged_union";_tag_fat(_tmp23C,sizeof(char),51U);});Cyc_aprintf(_tmp704,_tag_fat(_tmp23B,sizeof(void*),1U));});});_tmp706(_tmp705,_tag_fat(_tmp23A,sizeof(void*),0U));});});}_LL0:;}{
# 1606
struct _tuple30 _stmttmp27=Cyc_Toc_temp_var_and_exp();struct Cyc_Absyn_Exp*_tmp240;struct _tuple1*_tmp23F;_LL6: _tmp23F=_stmttmp27.f1;_tmp240=_stmttmp27.f2;_LL7: {struct _tuple1*temp=_tmp23F;struct Cyc_Absyn_Exp*temp_exp=_tmp240;
struct Cyc_Absyn_Exp*f_tag=Cyc_Absyn_signed_int_exp(Cyc_Toc_get_member_offset(ad,f),0U);
if(in_lhs){
struct Cyc_Absyn_Exp*temp_f_tag=Cyc_Absyn_aggrarrow_exp(temp_exp,Cyc_Toc_tag_sp,0U);
struct Cyc_Absyn_Exp*test_exp=Cyc_Absyn_neq_exp(temp_f_tag,f_tag,0U);
struct Cyc_Absyn_Exp*temp_f_val=Cyc_Absyn_aggrarrow_exp(temp_exp,Cyc_Toc_val_sp,0U);
struct Cyc_Absyn_Stmt*sres=Cyc_Absyn_exp_stmt(Cyc_Absyn_address_exp(temp_f_val,0U),0U);
struct Cyc_Absyn_Stmt*ifs=({struct Cyc_Absyn_Exp*_tmp708=test_exp;struct Cyc_Absyn_Stmt*_tmp707=Cyc_Toc_throw_match_stmt();Cyc_Absyn_ifthenelse_stmt(_tmp708,_tmp707,Cyc_Toc_skip_stmt_dl(),0U);});
void*e1_p_type=Cyc_Absyn_cstar_type(e1_c_type,Cyc_Toc_mt_tq);
struct Cyc_Absyn_Exp*e1_f=Cyc_Absyn_address_exp(aggrproj(e1,f,0U),0U);
struct Cyc_Absyn_Stmt*s=({struct _tuple1*_tmp70B=temp;void*_tmp70A=e1_p_type;struct Cyc_Absyn_Exp*_tmp709=e1_f;Cyc_Absyn_declare_stmt(_tmp70B,_tmp70A,_tmp709,Cyc_Absyn_seq_stmt(ifs,sres,0U),0U);});
return Cyc_Toc_stmt_exp_r(s);}else{
# 1619
struct Cyc_Absyn_Exp*temp_f_tag=({struct Cyc_Absyn_Exp*_tmp70C=aggrproj(temp_exp,f,0U);Cyc_Toc_member_exp(_tmp70C,Cyc_Toc_tag_sp,0U);});
struct Cyc_Absyn_Exp*test_exp=Cyc_Absyn_neq_exp(temp_f_tag,f_tag,0U);
struct Cyc_Absyn_Exp*temp_f_val=({struct Cyc_Absyn_Exp*_tmp70D=aggrproj(temp_exp,f,0U);Cyc_Toc_member_exp(_tmp70D,Cyc_Toc_val_sp,0U);});
struct Cyc_Absyn_Stmt*sres=Cyc_Absyn_exp_stmt(temp_f_val,0U);
struct Cyc_Absyn_Stmt*ifs=({struct Cyc_Absyn_Exp*_tmp70F=test_exp;struct Cyc_Absyn_Stmt*_tmp70E=Cyc_Toc_throw_match_stmt();Cyc_Absyn_ifthenelse_stmt(_tmp70F,_tmp70E,Cyc_Toc_skip_stmt_dl(),0U);});
struct Cyc_Absyn_Stmt*s=({struct _tuple1*_tmp712=temp;void*_tmp711=e1_c_type;struct Cyc_Absyn_Exp*_tmp710=e1;Cyc_Absyn_declare_stmt(_tmp712,_tmp711,_tmp710,Cyc_Absyn_seq_stmt(ifs,sres,0U),0U);});
return Cyc_Toc_stmt_exp_r(s);}}}}
# 1454 "toc.cyc"
static int Cyc_Toc_is_tagged_union_project_impl(void*t,struct _fat_ptr*f,int*f_tag,void**tagged_member_type,int clear_read,int*is_read){
# 1632 "toc.cyc"
void*_stmttmp28=Cyc_Tcutil_compress(t);void*_tmp241=_stmttmp28;union Cyc_Absyn_AggrInfo _tmp242;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp241)->tag == 0U){if(((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp241)->f1)->tag == 20U){_LL1: _tmp242=((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp241)->f1)->f1;_LL2: {union Cyc_Absyn_AggrInfo info=_tmp242;
# 1634
struct Cyc_Absyn_Aggrdecl*ad=Cyc_Absyn_get_known_aggrdecl(info);
({int _tmp713=Cyc_Toc_get_member_offset(ad,f);*f_tag=_tmp713;});{
# 1637
struct _fat_ptr str=(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp246=({struct Cyc_String_pa_PrintArg_struct _tmp5F4;_tmp5F4.tag=0U,_tmp5F4.f1=(struct _fat_ptr)((struct _fat_ptr)*(*ad->name).f2);_tmp5F4;});struct Cyc_String_pa_PrintArg_struct _tmp247=({struct Cyc_String_pa_PrintArg_struct _tmp5F3;_tmp5F3.tag=0U,_tmp5F3.f1=(struct _fat_ptr)((struct _fat_ptr)*f);_tmp5F3;});void*_tmp244[2U];_tmp244[0]=& _tmp246,_tmp244[1]=& _tmp247;({struct _fat_ptr _tmp714=({const char*_tmp245="_union_%s_%s";_tag_fat(_tmp245,sizeof(char),13U);});Cyc_aprintf(_tmp714,_tag_fat(_tmp244,sizeof(void*),2U));});});
({void*_tmp715=Cyc_Absyn_strct(({struct _fat_ptr*_tmp243=_cycalloc(sizeof(*_tmp243));*_tmp243=str;_tmp243;}));*tagged_member_type=_tmp715;});
if(clear_read)*((int*)_check_null(is_read))=0;return((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->tagged;}}}else{goto _LL3;}}else{_LL3: _LL4:
# 1641
 return 0;}_LL0:;}
# 1454 "toc.cyc"
static int Cyc_Toc_is_tagged_union_project(struct Cyc_Absyn_Exp*e,int*f_tag,void**tagged_member_type,int clear_read){
# 1651 "toc.cyc"
void*_stmttmp29=e->r;void*_tmp248=_stmttmp29;int*_tmp24B;struct _fat_ptr*_tmp24A;struct Cyc_Absyn_Exp*_tmp249;int*_tmp24E;struct _fat_ptr*_tmp24D;struct Cyc_Absyn_Exp*_tmp24C;struct Cyc_Absyn_Exp*_tmp24F;switch(*((int*)_tmp248)){case 14U: _LL1: _tmp24F=((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp248)->f2;_LL2: {struct Cyc_Absyn_Exp*e1=_tmp24F;
({void*_tmp250=0U;({int(*_tmp717)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp252)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Toc_toc_impos;_tmp252;});struct _fat_ptr _tmp716=({const char*_tmp251="cast on lhs!";_tag_fat(_tmp251,sizeof(char),13U);});_tmp717(_tmp716,_tag_fat(_tmp250,sizeof(void*),0U));});});}case 21U: _LL3: _tmp24C=((struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct*)_tmp248)->f1;_tmp24D=((struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct*)_tmp248)->f2;_tmp24E=(int*)&((struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct*)_tmp248)->f4;_LL4: {struct Cyc_Absyn_Exp*e1=_tmp24C;struct _fat_ptr*f=_tmp24D;int*is_read=_tmp24E;
# 1654
return Cyc_Toc_is_tagged_union_project_impl((void*)_check_null(e1->topt),f,f_tag,tagged_member_type,clear_read,is_read);}case 22U: _LL5: _tmp249=((struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct*)_tmp248)->f1;_tmp24A=((struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct*)_tmp248)->f2;_tmp24B=(int*)&((struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct*)_tmp248)->f4;_LL6: {struct Cyc_Absyn_Exp*e1=_tmp249;struct _fat_ptr*f=_tmp24A;int*is_read=_tmp24B;
# 1657
void*_stmttmp2A=Cyc_Tcutil_compress((void*)_check_null(e1->topt));void*_tmp253=_stmttmp2A;void*_tmp254;if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp253)->tag == 3U){_LLA: _tmp254=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp253)->f1).elt_type;_LLB: {void*et=_tmp254;
# 1659
return Cyc_Toc_is_tagged_union_project_impl(et,f,f_tag,tagged_member_type,clear_read,is_read);}}else{_LLC: _LLD:
# 1661
 return 0;}_LL9:;}default: _LL7: _LL8:
# 1663
 return 0;}_LL0:;}
# 1454 "toc.cyc"
static void*Cyc_Toc_tagged_union_assignop(struct Cyc_Absyn_Exp*e1,void*e1_cyc_type,struct Cyc_Core_Opt*popt,struct Cyc_Absyn_Exp*e2,void*e2_cyc_type,int tag_num,void*member_type){
# 1680 "toc.cyc"
struct _tuple30 _stmttmp2B=Cyc_Toc_temp_var_and_exp();struct Cyc_Absyn_Exp*_tmp256;struct _tuple1*_tmp255;_LL1: _tmp255=_stmttmp2B.f1;_tmp256=_stmttmp2B.f2;_LL2: {struct _tuple1*temp=_tmp255;struct Cyc_Absyn_Exp*temp_exp=_tmp256;
struct Cyc_Absyn_Exp*temp_val=Cyc_Absyn_aggrarrow_exp(temp_exp,Cyc_Toc_val_sp,0U);
struct Cyc_Absyn_Exp*temp_tag=Cyc_Absyn_aggrarrow_exp(temp_exp,Cyc_Toc_tag_sp,0U);
struct Cyc_Absyn_Exp*f_tag=Cyc_Absyn_signed_int_exp(tag_num,0U);
struct Cyc_Absyn_Stmt*s3=Cyc_Absyn_exp_stmt(Cyc_Absyn_assignop_exp(temp_val,popt,e2,0U),0U);
struct Cyc_Absyn_Stmt*s2;
if(popt == 0)
s2=Cyc_Absyn_exp_stmt(Cyc_Absyn_assign_exp(temp_tag,f_tag,0U),0U);else{
# 1689
struct Cyc_Absyn_Exp*test_exp=Cyc_Absyn_neq_exp(temp_tag,f_tag,0U);
s2=({struct Cyc_Absyn_Exp*_tmp719=test_exp;struct Cyc_Absyn_Stmt*_tmp718=Cyc_Toc_throw_match_stmt();Cyc_Absyn_ifthenelse_stmt(_tmp719,_tmp718,Cyc_Toc_skip_stmt_dl(),0U);});}{
# 1692
struct Cyc_Absyn_Stmt*s1=({struct _tuple1*_tmp71C=temp;void*_tmp71B=Cyc_Absyn_cstar_type(member_type,Cyc_Toc_mt_tq);struct Cyc_Absyn_Exp*_tmp71A=
Cyc_Toc_push_address_exp(e1);
# 1692
Cyc_Absyn_declare_stmt(_tmp71C,_tmp71B,_tmp71A,
# 1694
Cyc_Absyn_seq_stmt(s2,s3,0U),0U);});
return Cyc_Toc_stmt_exp_r(s1);}}}struct _tuple33{void*f1;void*f2;};struct _tuple34{struct Cyc_Absyn_Exp*f1;struct Cyc_Absyn_Exp*f2;};struct _tuple35{struct Cyc_Absyn_Aggrfield*f1;struct Cyc_Absyn_Exp*f2;};
# 1454 "toc.cyc"
static void Cyc_Toc_exp_to_c(struct Cyc_Toc_Env*nv,struct Cyc_Absyn_Exp*e){
# 1699 "toc.cyc"
if(e->topt == 0)
({struct Cyc_String_pa_PrintArg_struct _tmp25A=({struct Cyc_String_pa_PrintArg_struct _tmp5F5;_tmp5F5.tag=0U,({struct _fat_ptr _tmp71D=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_exp2string(e));_tmp5F5.f1=_tmp71D;});_tmp5F5;});void*_tmp257[1U];_tmp257[0]=& _tmp25A;({int(*_tmp720)(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp259)(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos_loc;_tmp259;});unsigned _tmp71F=e->loc;struct _fat_ptr _tmp71E=({const char*_tmp258="exp_to_c: no type for %s";_tag_fat(_tmp258,sizeof(char),25U);});_tmp720(_tmp71F,_tmp71E,_tag_fat(_tmp257,sizeof(void*),1U));});});{
# 1699
void*old_typ=(void*)_check_null(e->topt);
# 1703
int did_inserted_checks=0;
{void*_stmttmp2C=e->annot;void*_tmp25B=_stmttmp2C;if(((struct Cyc_Absyn_EmptyAnnot_Absyn_AbsynAnnot_struct*)_tmp25B)->tag == Cyc_Absyn_EmptyAnnot){_LL1: _LL2:
 goto _LL4;}else{if(((struct Cyc_InsertChecks_NoCheck_Absyn_AbsynAnnot_struct*)_tmp25B)->tag == Cyc_InsertChecks_NoCheck){_LL3: _LL4:
 did_inserted_checks=1;goto _LL0;}else{_LL5: _LL6:
 goto _LL0;}}_LL0:;}
# 1709
{void*_stmttmp2D=e->r;void*_tmp25C=_stmttmp2D;struct Cyc_Absyn_Stmt*_tmp25D;struct _fat_ptr*_tmp25F;struct Cyc_Absyn_Exp*_tmp25E;struct Cyc_Absyn_Exp*_tmp261;struct Cyc_Absyn_Exp*_tmp260;int _tmp267;int _tmp266;struct Cyc_Absyn_Exp*_tmp265;void**_tmp264;struct Cyc_Absyn_Exp*_tmp263;int _tmp262;struct Cyc_Absyn_Datatypefield*_tmp26A;struct Cyc_Absyn_Datatypedecl*_tmp269;struct Cyc_List_List*_tmp268;struct Cyc_Absyn_Aggrdecl*_tmp26E;struct Cyc_List_List*_tmp26D;struct Cyc_List_List*_tmp26C;struct _tuple1**_tmp26B;struct Cyc_List_List*_tmp270;void*_tmp26F;int _tmp273;void*_tmp272;struct Cyc_Absyn_Exp*_tmp271;int _tmp277;struct Cyc_Absyn_Exp*_tmp276;struct Cyc_Absyn_Exp*_tmp275;struct Cyc_Absyn_Vardecl*_tmp274;struct Cyc_List_List*_tmp278;struct Cyc_List_List*_tmp279;struct Cyc_Absyn_Exp*_tmp27B;struct Cyc_Absyn_Exp*_tmp27A;struct Cyc_Absyn_Exp*_tmp27C;int _tmp280;int _tmp27F;struct _fat_ptr*_tmp27E;struct Cyc_Absyn_Exp*_tmp27D;int _tmp284;int _tmp283;struct _fat_ptr*_tmp282;struct Cyc_Absyn_Exp*_tmp281;struct Cyc_List_List*_tmp286;void*_tmp285;void*_tmp287;struct Cyc_Absyn_Exp*_tmp288;struct Cyc_Absyn_Exp*_tmp28A;struct Cyc_Absyn_Exp*_tmp289;struct Cyc_Absyn_Exp*_tmp28B;enum Cyc_Absyn_Coercion _tmp28F;int _tmp28E;struct Cyc_Absyn_Exp*_tmp28D;void**_tmp28C;struct Cyc_List_List*_tmp291;struct Cyc_Absyn_Exp*_tmp290;struct Cyc_Absyn_Exp*_tmp292;int _tmp294;struct Cyc_Absyn_Exp*_tmp293;struct Cyc_Absyn_Exp*_tmp296;struct Cyc_Absyn_Exp*_tmp295;struct Cyc_Absyn_VarargInfo*_tmp29B;struct Cyc_List_List*_tmp29A;int _tmp299;struct Cyc_List_List*_tmp298;struct Cyc_Absyn_Exp*_tmp297;struct Cyc_List_List*_tmp29D;struct Cyc_Absyn_Exp*_tmp29C;struct Cyc_Absyn_Exp*_tmp29F;struct Cyc_Absyn_Exp*_tmp29E;struct Cyc_Absyn_Exp*_tmp2A1;struct Cyc_Absyn_Exp*_tmp2A0;struct Cyc_Absyn_Exp*_tmp2A3;struct Cyc_Absyn_Exp*_tmp2A2;struct Cyc_Absyn_Exp*_tmp2A6;struct Cyc_Absyn_Exp*_tmp2A5;struct Cyc_Absyn_Exp*_tmp2A4;struct Cyc_Absyn_Exp*_tmp2A9;struct Cyc_Core_Opt*_tmp2A8;struct Cyc_Absyn_Exp*_tmp2A7;enum Cyc_Absyn_Incrementor _tmp2AB;struct Cyc_Absyn_Exp*_tmp2AA;struct Cyc_List_List*_tmp2AD;enum Cyc_Absyn_Primop _tmp2AC;struct Cyc_Absyn_Exp*_tmp2AE;switch(*((int*)_tmp25C)){case 2U: _LL8: _LL9:
# 1711
 e->r=(void*)& Cyc_Toc_zero_exp;
goto _LL7;case 0U: if(((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmp25C)->f1).Null_c).tag == 1){_LLA: _LLB: {
# 1718
struct Cyc_Absyn_Exp*zero=Cyc_Absyn_signed_int_exp(0,0U);
if(Cyc_Tcutil_is_fat_pointer_type(old_typ)){
if(Cyc_Toc_is_toplevel(nv))
({void*_tmp721=(Cyc_Toc_make_toplevel_dyn_arr(old_typ,zero,zero))->r;e->r=_tmp721;});else{
# 1723
({void*_tmp723=({struct Cyc_Absyn_Exp*_tmp2AF[3U];_tmp2AF[0]=zero,_tmp2AF[1]=zero,_tmp2AF[2]=zero;({struct Cyc_Absyn_Exp*_tmp722=Cyc_Toc__tag_fat_e;Cyc_Toc_fncall_exp_r(_tmp722,_tag_fat(_tmp2AF,sizeof(struct Cyc_Absyn_Exp*),3U));});});e->r=_tmp723;});}}else{
# 1725
e->r=(void*)& Cyc_Toc_zero_exp;}
# 1727
goto _LL7;}}else{_LLC: _LLD:
 goto _LL7;}case 1U: _LLE: _LLF:
 goto _LL7;case 42U: _LL10: _tmp2AE=((struct Cyc_Absyn_Extension_e_Absyn_Raw_exp_struct*)_tmp25C)->f1;_LL11: {struct Cyc_Absyn_Exp*e1=_tmp2AE;
Cyc_Toc_exp_to_c(nv,e1);goto _LL7;}case 3U: _LL12: _tmp2AC=((struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct*)_tmp25C)->f1;_tmp2AD=((struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct*)_tmp25C)->f2;_LL13: {enum Cyc_Absyn_Primop p=_tmp2AC;struct Cyc_List_List*es=_tmp2AD;
# 1733
struct Cyc_List_List*old_types=({struct Cyc_List_List*(*_tmp724)(void*(*f)(struct Cyc_Absyn_Exp*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp2C9)(void*(*f)(struct Cyc_Absyn_Exp*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(void*(*f)(struct Cyc_Absyn_Exp*),struct Cyc_List_List*x))Cyc_List_map;_tmp2C9;});_tmp724(Cyc_Toc_get_cyc_type,es);});
# 1735
({void(*_tmp726)(void(*f)(struct Cyc_Toc_Env*,struct Cyc_Absyn_Exp*),struct Cyc_Toc_Env*env,struct Cyc_List_List*x)=({void(*_tmp2B0)(void(*f)(struct Cyc_Toc_Env*,struct Cyc_Absyn_Exp*),struct Cyc_Toc_Env*env,struct Cyc_List_List*x)=(void(*)(void(*f)(struct Cyc_Toc_Env*,struct Cyc_Absyn_Exp*),struct Cyc_Toc_Env*env,struct Cyc_List_List*x))Cyc_List_iter_c;_tmp2B0;});struct Cyc_Toc_Env*_tmp725=nv;_tmp726(Cyc_Toc_exp_to_c,_tmp725,es);});
{enum Cyc_Absyn_Primop _tmp2B1=p;switch(_tmp2B1){case Cyc_Absyn_Numelts: _LL63: _LL64: {
# 1738
struct Cyc_Absyn_Exp*arg=(struct Cyc_Absyn_Exp*)((struct Cyc_List_List*)_check_null(es))->hd;
{void*_stmttmp2E=Cyc_Tcutil_compress((void*)_check_null(arg->topt));void*_tmp2B2=_stmttmp2E;void*_tmp2B6;void*_tmp2B5;void*_tmp2B4;void*_tmp2B3;if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp2B2)->tag == 3U){_LL78: _tmp2B3=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp2B2)->f1).elt_type;_tmp2B4=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp2B2)->f1).ptr_atts).nullable;_tmp2B5=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp2B2)->f1).ptr_atts).bounds;_tmp2B6=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp2B2)->f1).ptr_atts).zero_term;_LL79: {void*elt_type=_tmp2B3;void*nbl=_tmp2B4;void*bound=_tmp2B5;void*zt=_tmp2B6;
# 1741
struct Cyc_Absyn_Exp*eopt=({void*_tmp727=Cyc_Absyn_bounds_one();Cyc_Tcutil_get_bounds_exp(_tmp727,bound);});
if(eopt == 0)
# 1744
({void*_tmp72A=({struct Cyc_Absyn_Exp*_tmp2B7[2U];_tmp2B7[0]=(struct Cyc_Absyn_Exp*)es->hd,({
struct Cyc_Absyn_Exp*_tmp728=Cyc_Absyn_sizeoftype_exp(Cyc_Toc_typ_to_c(elt_type),0U);_tmp2B7[1]=_tmp728;});({struct Cyc_Absyn_Exp*_tmp729=Cyc_Toc__get_fat_size_e;Cyc_Toc_fncall_exp_r(_tmp729,_tag_fat(_tmp2B7,sizeof(struct Cyc_Absyn_Exp*),2U));});});
# 1744
e->r=_tmp72A;});else{
# 1747
struct Cyc_Absyn_Exp*e2=eopt;
# 1749
if(Cyc_Tcutil_force_type2bool(0,zt)){
struct Cyc_Absyn_Exp*function_e=
Cyc_Toc_getFunctionRemovePointer(& Cyc_Toc__get_zero_arr_size_functionSet,(struct Cyc_Absyn_Exp*)es->hd);
# 1753
({void*_tmp72C=({struct Cyc_Absyn_Exp*_tmp2B8[2U];_tmp2B8[0]=(struct Cyc_Absyn_Exp*)es->hd,_tmp2B8[1]=e2;({struct Cyc_Absyn_Exp*_tmp72B=function_e;Cyc_Toc_fncall_exp_r(_tmp72B,_tag_fat(_tmp2B8,sizeof(struct Cyc_Absyn_Exp*),2U));});});e->r=_tmp72C;});}else{
if(Cyc_Tcutil_force_type2bool(0,nbl)){
if(!Cyc_Evexp_c_can_eval(e2))
({void*_tmp2B9=0U;({unsigned _tmp72E=e->loc;struct _fat_ptr _tmp72D=({const char*_tmp2BA="can't calculate numelts";_tag_fat(_tmp2BA,sizeof(char),24U);});Cyc_Tcutil_terr(_tmp72E,_tmp72D,_tag_fat(_tmp2B9,sizeof(void*),0U));});});
# 1755
({void*_tmp731=({
# 1758
struct Cyc_Absyn_Exp*_tmp730=arg;struct Cyc_Absyn_Exp*_tmp72F=e2;Cyc_Toc_conditional_exp_r(_tmp730,_tmp72F,Cyc_Absyn_uint_exp(0U,0U));});
# 1755
e->r=_tmp731;});}else{
# 1760
e->r=e2->r;goto _LL77;}}}
# 1763
goto _LL77;}}else{_LL7A: _LL7B:
({struct Cyc_String_pa_PrintArg_struct _tmp2BE=({struct Cyc_String_pa_PrintArg_struct _tmp5F6;_tmp5F6.tag=0U,({
struct _fat_ptr _tmp732=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_typ2string((void*)_check_null(arg->topt)));_tmp5F6.f1=_tmp732;});_tmp5F6;});void*_tmp2BB[1U];_tmp2BB[0]=& _tmp2BE;({int(*_tmp734)(struct _fat_ptr fmt,struct _fat_ptr ap)=({
# 1764
int(*_tmp2BD)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Toc_toc_impos;_tmp2BD;});struct _fat_ptr _tmp733=({const char*_tmp2BC="numelts primop applied to non-pointer %s";_tag_fat(_tmp2BC,sizeof(char),41U);});_tmp734(_tmp733,_tag_fat(_tmp2BB,sizeof(void*),1U));});});}_LL77:;}
# 1767
goto _LL62;}case Cyc_Absyn_Plus: _LL65: _LL66:
# 1772
 if(Cyc_Toc_is_toplevel(nv))
({void*_tmp2BF=0U;({int(*_tmp736)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp2C1)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Toc_unimp;_tmp2C1;});struct _fat_ptr _tmp735=({const char*_tmp2C0="can't do pointer arithmetic at top-level";_tag_fat(_tmp2C0,sizeof(char),41U);});_tmp736(_tmp735,_tag_fat(_tmp2BF,sizeof(void*),0U));});});
# 1772
{void*_stmttmp2F=
# 1774
Cyc_Tcutil_compress((void*)((struct Cyc_List_List*)_check_null(old_types))->hd);
# 1772
void*_tmp2C2=_stmttmp2F;void*_tmp2C5;void*_tmp2C4;void*_tmp2C3;if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp2C2)->tag == 3U){_LL7D: _tmp2C3=(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp2C2)->f1).elt_type;_tmp2C4=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp2C2)->f1).ptr_atts).bounds;_tmp2C5=((((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp2C2)->f1).ptr_atts).zero_term;_LL7E: {void*elt_type=_tmp2C3;void*b=_tmp2C4;void*zt=_tmp2C5;
# 1776
struct Cyc_Absyn_Exp*eopt=({void*_tmp737=Cyc_Absyn_bounds_one();Cyc_Tcutil_get_bounds_exp(_tmp737,b);});
struct Cyc_Absyn_Exp*e1=(struct Cyc_Absyn_Exp*)((struct Cyc_List_List*)_check_null(es))->hd;
struct Cyc_Absyn_Exp*e2=(struct Cyc_Absyn_Exp*)((struct Cyc_List_List*)_check_null(es->tl))->hd;
if(eopt == 0)
({void*_tmp73A=({struct Cyc_Absyn_Exp*_tmp2C6[3U];_tmp2C6[0]=e1,({
struct Cyc_Absyn_Exp*_tmp738=Cyc_Absyn_sizeoftype_exp(Cyc_Toc_typ_to_c(elt_type),0U);_tmp2C6[1]=_tmp738;}),_tmp2C6[2]=e2;({struct Cyc_Absyn_Exp*_tmp739=Cyc_Toc__fat_ptr_plus_e;Cyc_Toc_fncall_exp_r(_tmp739,_tag_fat(_tmp2C6,sizeof(struct Cyc_Absyn_Exp*),3U));});});
# 1780
e->r=_tmp73A;});else{
# 1782
if(Cyc_Tcutil_force_type2bool(0,zt))
({void*_tmp73C=({struct Cyc_Absyn_Exp*_tmp2C7[3U];_tmp2C7[0]=e1,_tmp2C7[1]=eopt,_tmp2C7[2]=e2;({struct Cyc_Absyn_Exp*_tmp73B=Cyc_Toc_getFunctionRemovePointer(& Cyc_Toc__zero_arr_plus_functionSet,e1);Cyc_Toc_fncall_exp_r(_tmp73B,_tag_fat(_tmp2C7,sizeof(struct Cyc_Absyn_Exp*),3U));});});e->r=_tmp73C;});}
# 1779
goto _LL7C;}}else{_LL7F: _LL80:
# 1785
 goto _LL7C;}_LL7C:;}
# 1787
goto _LL62;case Cyc_Absyn_Minus: _LL67: _LL68: {
# 1792
void*elt_type=Cyc_Absyn_void_type;
if(Cyc_Tcutil_is_fat_pointer_type_elt((void*)((struct Cyc_List_List*)_check_null(old_types))->hd,& elt_type)){
struct Cyc_Absyn_Exp*e1=(struct Cyc_Absyn_Exp*)((struct Cyc_List_List*)_check_null(es))->hd;
struct Cyc_Absyn_Exp*e2=(struct Cyc_Absyn_Exp*)((struct Cyc_List_List*)_check_null(es->tl))->hd;
if(Cyc_Tcutil_is_fat_pointer_type((void*)((struct Cyc_List_List*)_check_null(old_types->tl))->hd)){
({void*_tmp73E=({struct Cyc_Absyn_Exp*_tmp73D=Cyc_Absyn_new_exp(e1->r,0U);Cyc_Toc_aggrmember_exp_r(_tmp73D,Cyc_Toc_curr_sp);});e1->r=_tmp73E;});
({void*_tmp740=({struct Cyc_Absyn_Exp*_tmp73F=Cyc_Absyn_new_exp(e2->r,0U);Cyc_Toc_aggrmember_exp_r(_tmp73F,Cyc_Toc_curr_sp);});e2->r=_tmp740;});
({void*_tmp742=({void*_tmp741=Cyc_Absyn_cstar_type(Cyc_Absyn_uchar_type,Cyc_Toc_mt_tq);e2->topt=_tmp741;});e1->topt=_tmp742;});
({void*_tmp744=({struct Cyc_Absyn_Exp*_tmp743=Cyc_Absyn_copy_exp(e);Cyc_Absyn_divide_exp(_tmp743,
Cyc_Absyn_sizeoftype_exp(Cyc_Toc_typ_to_c(elt_type),0U),0U);})->r;
# 1800
e->r=_tmp744;});}else{
# 1803
({void*_tmp748=({struct Cyc_Absyn_Exp*_tmp2C8[3U];_tmp2C8[0]=e1,({
struct Cyc_Absyn_Exp*_tmp746=Cyc_Absyn_sizeoftype_exp(Cyc_Toc_typ_to_c(elt_type),0U);_tmp2C8[1]=_tmp746;}),({
struct Cyc_Absyn_Exp*_tmp745=Cyc_Absyn_prim1_exp(Cyc_Absyn_Minus,e2,0U);_tmp2C8[2]=_tmp745;});({struct Cyc_Absyn_Exp*_tmp747=Cyc_Toc__fat_ptr_plus_e;Cyc_Toc_fncall_exp_r(_tmp747,_tag_fat(_tmp2C8,sizeof(struct Cyc_Absyn_Exp*),3U));});});
# 1803
e->r=_tmp748;});}}
# 1793
goto _LL62;}case Cyc_Absyn_Eq: _LL69: _LL6A:
# 1808
 goto _LL6C;case Cyc_Absyn_Neq: _LL6B: _LL6C: goto _LL6E;case Cyc_Absyn_Gt: _LL6D: _LL6E: goto _LL70;case Cyc_Absyn_Gte: _LL6F: _LL70: goto _LL72;case Cyc_Absyn_Lt: _LL71: _LL72: goto _LL74;case Cyc_Absyn_Lte: _LL73: _LL74: {
# 1810
struct Cyc_Absyn_Exp*e1=(struct Cyc_Absyn_Exp*)((struct Cyc_List_List*)_check_null(es))->hd;
struct Cyc_Absyn_Exp*e2=(struct Cyc_Absyn_Exp*)((struct Cyc_List_List*)_check_null(es->tl))->hd;
void*t1=(void*)((struct Cyc_List_List*)_check_null(old_types))->hd;
void*t2=(void*)((struct Cyc_List_List*)_check_null(old_types->tl))->hd;
void*elt_type=Cyc_Absyn_void_type;
if(Cyc_Tcutil_is_fat_pointer_type_elt(t1,& elt_type)){
void*t=({void*_tmp749=Cyc_Toc_typ_to_c(elt_type);Cyc_Absyn_cstar_type(_tmp749,Cyc_Toc_mt_tq);});
({void*_tmp74C=({void*_tmp74B=t;Cyc_Toc_cast_it_r(_tmp74B,({struct Cyc_Absyn_Exp*_tmp74A=Cyc_Absyn_new_exp(e1->r,0U);Cyc_Toc_member_exp(_tmp74A,Cyc_Toc_curr_sp,0U);}));});e1->r=_tmp74C;});
e1->topt=t;}
# 1815
if(
# 1820
Cyc_Tcutil_is_fat_pointer_type(t2)){
void*t=({void*_tmp74D=Cyc_Toc_typ_to_c(elt_type);Cyc_Absyn_cstar_type(_tmp74D,Cyc_Toc_mt_tq);});
({void*_tmp750=({void*_tmp74F=t;Cyc_Toc_cast_it_r(_tmp74F,({struct Cyc_Absyn_Exp*_tmp74E=Cyc_Absyn_new_exp(e2->r,0U);Cyc_Toc_member_exp(_tmp74E,Cyc_Toc_curr_sp,0U);}));});e2->r=_tmp750;});
e2->topt=t;}
# 1815
goto _LL62;}default: _LL75: _LL76:
# 1826
 goto _LL62;}_LL62:;}
# 1828
goto _LL7;}case 5U: _LL14: _tmp2AA=((struct Cyc_Absyn_Increment_e_Absyn_Raw_exp_struct*)_tmp25C)->f1;_tmp2AB=((struct Cyc_Absyn_Increment_e_Absyn_Raw_exp_struct*)_tmp25C)->f2;_LL15: {struct Cyc_Absyn_Exp*e2=_tmp2AA;enum Cyc_Absyn_Incrementor incr=_tmp2AB;
# 1830
void*e2_cyc_typ=(void*)_check_null(e2->topt);
# 1839 "toc.cyc"
void*ignore_typ=Cyc_Absyn_void_type;
int ignore_bool=0;
int ignore_int=0;
struct _fat_ptr incr_str=({const char*_tmp2DA="increment";_tag_fat(_tmp2DA,sizeof(char),10U);});
if((int)incr == (int)2U ||(int)incr == (int)3U)incr_str=({const char*_tmp2CA="decrement";_tag_fat(_tmp2CA,sizeof(char),10U);});if(
Cyc_Tcutil_is_zero_ptr_deref(e2,& ignore_typ,& ignore_bool,& ignore_typ))
({struct Cyc_String_pa_PrintArg_struct _tmp2CE=({struct Cyc_String_pa_PrintArg_struct _tmp5F7;_tmp5F7.tag=0U,_tmp5F7.f1=(struct _fat_ptr)((struct _fat_ptr)incr_str);_tmp5F7;});void*_tmp2CB[1U];_tmp2CB[0]=& _tmp2CE;({int(*_tmp753)(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp2CD)(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos_loc;_tmp2CD;});unsigned _tmp752=e->loc;struct _fat_ptr _tmp751=({const char*_tmp2CC="in-place %s is not supported when dereferencing a zero-terminated pointer";_tag_fat(_tmp2CC,sizeof(char),74U);});_tmp753(_tmp752,_tmp751,_tag_fat(_tmp2CB,sizeof(void*),1U));});});
# 1843
if(
# 1847
Cyc_Toc_is_tagged_union_project(e2,& ignore_int,& ignore_typ,1)){
struct Cyc_Absyn_Exp*one=Cyc_Absyn_signed_int_exp(1,0U);
enum Cyc_Absyn_Primop op;
one->topt=Cyc_Absyn_sint_type;
{enum Cyc_Absyn_Incrementor _tmp2CF=incr;switch(_tmp2CF){case Cyc_Absyn_PreInc: _LL82: _LL83:
 op=0U;goto _LL81;case Cyc_Absyn_PreDec: _LL84: _LL85:
 op=2U;goto _LL81;default: _LL86: _LL87:
({struct Cyc_String_pa_PrintArg_struct _tmp2D3=({struct Cyc_String_pa_PrintArg_struct _tmp5F8;_tmp5F8.tag=0U,_tmp5F8.f1=(struct _fat_ptr)((struct _fat_ptr)incr_str);_tmp5F8;});void*_tmp2D0[1U];_tmp2D0[0]=& _tmp2D3;({int(*_tmp756)(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp2D2)(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos_loc;_tmp2D2;});unsigned _tmp755=e->loc;struct _fat_ptr _tmp754=({const char*_tmp2D1="in-place post%s is not supported on @tagged union members";_tag_fat(_tmp2D1,sizeof(char),58U);});_tmp756(_tmp755,_tmp754,_tag_fat(_tmp2D0,sizeof(void*),1U));});});}_LL81:;}
# 1857
({void*_tmp758=(void*)({struct Cyc_Absyn_AssignOp_e_Absyn_Raw_exp_struct*_tmp2D5=_cycalloc(sizeof(*_tmp2D5));_tmp2D5->tag=4U,_tmp2D5->f1=e2,({struct Cyc_Core_Opt*_tmp757=({struct Cyc_Core_Opt*_tmp2D4=_cycalloc(sizeof(*_tmp2D4));_tmp2D4->v=(void*)op;_tmp2D4;});_tmp2D5->f2=_tmp757;}),_tmp2D5->f3=one;_tmp2D5;});e->r=_tmp758;});
Cyc_Toc_exp_to_c(nv,e);
return;}
# 1843
Cyc_Toc_set_lhs(nv,1);
# 1862
Cyc_Toc_exp_to_c(nv,e2);
Cyc_Toc_set_lhs(nv,0);{
# 1866
void*elt_typ=Cyc_Absyn_void_type;
if(Cyc_Tcutil_is_fat_pointer_type_elt(old_typ,& elt_typ)){
struct Cyc_Absyn_Exp*fn_e;
int change=1;
fn_e=((int)incr == (int)1U ||(int)incr == (int)3U)?Cyc_Toc__fat_ptr_inplace_plus_post_e: Cyc_Toc__fat_ptr_inplace_plus_e;
# 1872
if((int)incr == (int)2U ||(int)incr == (int)3U)
change=-1;
# 1872
({void*_tmp75D=({struct Cyc_Absyn_Exp*_tmp2D6[3U];({
# 1874
struct Cyc_Absyn_Exp*_tmp75B=Cyc_Toc_push_address_exp(e2);_tmp2D6[0]=_tmp75B;}),({
struct Cyc_Absyn_Exp*_tmp75A=Cyc_Absyn_sizeoftype_exp(Cyc_Toc_typ_to_c(elt_typ),0U);_tmp2D6[1]=_tmp75A;}),({
struct Cyc_Absyn_Exp*_tmp759=Cyc_Absyn_signed_int_exp(change,0U);_tmp2D6[2]=_tmp759;});({struct Cyc_Absyn_Exp*_tmp75C=fn_e;Cyc_Toc_fncall_exp_r(_tmp75C,_tag_fat(_tmp2D6,sizeof(struct Cyc_Absyn_Exp*),3U));});});
# 1872
e->r=_tmp75D;});}else{
# 1877
if(Cyc_Tcutil_is_zero_pointer_type_elt(old_typ,& elt_typ)){
# 1883
did_inserted_checks=1;
if((int)incr != (int)1U){
struct _tuple1*x=Cyc_Toc_temp_var();
void*t=({void*_tmp75E=Cyc_Toc_typ_to_c(old_typ);Cyc_Absyn_cstar_type(_tmp75E,Cyc_Toc_mt_tq);});
struct Cyc_Absyn_Exp*xexp=Cyc_Toc_push_address_exp(e2);
struct Cyc_Absyn_Exp*testexp=({struct Cyc_Absyn_Exp*_tmp75F=Cyc_Absyn_deref_exp(Cyc_Absyn_deref_exp(Cyc_Absyn_var_exp(x,0U),0U),0U);Cyc_Absyn_neq_exp(_tmp75F,
Cyc_Absyn_int_exp(Cyc_Absyn_None,0,0U),0U);});
if(Cyc_Toc_do_null_check(e))
testexp=({struct Cyc_Absyn_Exp*_tmp761=({struct Cyc_Absyn_Exp*_tmp760=Cyc_Absyn_deref_exp(Cyc_Absyn_var_exp(x,0U),0U);Cyc_Absyn_neq_exp(_tmp760,
Cyc_Absyn_int_exp(Cyc_Absyn_None,0,0U),0U);});
# 1891
Cyc_Absyn_and_exp(_tmp761,testexp,0U);});{
# 1890
struct Cyc_Absyn_Stmt*s=({
# 1893
struct Cyc_Absyn_Exp*_tmp764=testexp;struct Cyc_Absyn_Stmt*_tmp763=
Cyc_Absyn_exp_stmt(Cyc_Absyn_increment_exp(Cyc_Absyn_deref_exp(Cyc_Absyn_var_exp(x,0U),0U),Cyc_Absyn_PreInc,0U),0U);
# 1893
Cyc_Absyn_ifthenelse_stmt(_tmp764,_tmp763,
# 1895
Cyc_Absyn_exp_stmt(({void*_tmp2D7=0U;({struct Cyc_Absyn_Exp*_tmp762=Cyc_Toc__throw_arraybounds_e;Cyc_Toc_fncall_exp_dl(_tmp762,_tag_fat(_tmp2D7,sizeof(struct Cyc_Absyn_Exp*),0U));});}),0U),0U);});
s=({struct Cyc_Absyn_Stmt*_tmp765=s;Cyc_Absyn_seq_stmt(_tmp765,Cyc_Absyn_exp_stmt(Cyc_Absyn_deref_exp(Cyc_Absyn_var_exp(x,0U),0U),0U),0U);});
({void*_tmp766=(Cyc_Absyn_stmt_exp(Cyc_Absyn_declare_stmt(x,t,xexp,s,0U),0U))->r;e->r=_tmp766;});}}else{
# 1900
struct Cyc_Toc_functionSet*fnSet=& Cyc_Toc__zero_arr_inplace_plus_post_functionSet;
struct Cyc_Absyn_Exp*fn_e=Cyc_Toc_getFunctionRemovePointer(fnSet,e2);
({void*_tmp76A=({struct Cyc_Absyn_Exp*_tmp2D8[2U];({struct Cyc_Absyn_Exp*_tmp768=Cyc_Toc_push_address_exp(e2);_tmp2D8[0]=_tmp768;}),({struct Cyc_Absyn_Exp*_tmp767=Cyc_Absyn_signed_int_exp(1,0U);_tmp2D8[1]=_tmp767;});({struct Cyc_Absyn_Exp*_tmp769=fn_e;Cyc_Toc_fncall_exp_r(_tmp769,_tag_fat(_tmp2D8,sizeof(struct Cyc_Absyn_Exp*),2U));});});e->r=_tmp76A;});}}else{
# 1904
if(elt_typ == Cyc_Absyn_void_type && !Cyc_Absyn_is_lvalue(e2)){
({void(*_tmp76C)(struct Cyc_Absyn_Exp*e1,struct Cyc_List_List*fs,struct Cyc_Absyn_Exp*(*f)(struct Cyc_Absyn_Exp*,enum Cyc_Absyn_Incrementor),enum Cyc_Absyn_Incrementor f_env)=({void(*_tmp2D9)(struct Cyc_Absyn_Exp*e1,struct Cyc_List_List*fs,struct Cyc_Absyn_Exp*(*f)(struct Cyc_Absyn_Exp*,enum Cyc_Absyn_Incrementor),enum Cyc_Absyn_Incrementor f_env)=(void(*)(struct Cyc_Absyn_Exp*e1,struct Cyc_List_List*fs,struct Cyc_Absyn_Exp*(*f)(struct Cyc_Absyn_Exp*,enum Cyc_Absyn_Incrementor),enum Cyc_Absyn_Incrementor f_env))Cyc_Toc_lvalue_assign;_tmp2D9;});struct Cyc_Absyn_Exp*_tmp76B=e2;_tmp76C(_tmp76B,0,Cyc_Toc_incr_lvalue,incr);});
e->r=e2->r;}}}
# 1867
goto _LL7;}}case 4U: _LL16: _tmp2A7=((struct Cyc_Absyn_AssignOp_e_Absyn_Raw_exp_struct*)_tmp25C)->f1;_tmp2A8=((struct Cyc_Absyn_AssignOp_e_Absyn_Raw_exp_struct*)_tmp25C)->f2;_tmp2A9=((struct Cyc_Absyn_AssignOp_e_Absyn_Raw_exp_struct*)_tmp25C)->f3;_LL17: {struct Cyc_Absyn_Exp*e1=_tmp2A7;struct Cyc_Core_Opt*popt=_tmp2A8;struct Cyc_Absyn_Exp*e2=_tmp2A9;
# 1927 "toc.cyc"
void*e1_old_typ=(void*)_check_null(e1->topt);
void*e2_old_typ=(void*)_check_null(e2->topt);
int f_tag=0;
void*tagged_member_struct_type=Cyc_Absyn_void_type;
if(Cyc_Toc_is_tagged_union_project(e1,& f_tag,& tagged_member_struct_type,1)){
Cyc_Toc_set_lhs(nv,1);
Cyc_Toc_exp_to_c(nv,e1);
Cyc_Toc_set_lhs(nv,0);
Cyc_Toc_exp_to_c(nv,e2);
({void*_tmp76D=Cyc_Toc_tagged_union_assignop(e1,e1_old_typ,popt,e2,e2_old_typ,f_tag,tagged_member_struct_type);e->r=_tmp76D;});
# 1938
goto _LL7;}{
# 1931
void*ptr_type=Cyc_Absyn_void_type;
# 1941
void*elt_type=Cyc_Absyn_void_type;
int is_fat=0;
if(Cyc_Tcutil_is_zero_ptr_deref(e1,& ptr_type,& is_fat,& elt_type)){
Cyc_Toc_zero_ptr_assign_to_c(nv,e,e1,popt,e2,ptr_type,is_fat,elt_type);
# 1946
return;}{
# 1943
int e1_poly=
# 1950
Cyc_Toc_is_poly_project(e1);
Cyc_Toc_set_lhs(nv,1);
Cyc_Toc_exp_to_c(nv,e1);
Cyc_Toc_set_lhs(nv,0);
Cyc_Toc_exp_to_c(nv,e2);{
# 1956
int done=0;
if(popt != 0){
void*elt_typ=Cyc_Absyn_void_type;
if(Cyc_Tcutil_is_fat_pointer_type_elt(old_typ,& elt_typ)){
struct Cyc_Absyn_Exp*change;
{enum Cyc_Absyn_Primop _stmttmp30=(enum Cyc_Absyn_Primop)popt->v;enum Cyc_Absyn_Primop _tmp2DB=_stmttmp30;switch(_tmp2DB){case Cyc_Absyn_Plus: _LL89: _LL8A:
 change=e2;goto _LL88;case Cyc_Absyn_Minus: _LL8B: _LL8C:
 change=Cyc_Absyn_prim1_exp(Cyc_Absyn_Minus,e2,0U);goto _LL88;default: _LL8D: _LL8E:
({void*_tmp2DC=0U;({int(*_tmp76F)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp2DE)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Toc_toc_impos;_tmp2DE;});struct _fat_ptr _tmp76E=({const char*_tmp2DD="bad t ? pointer arithmetic";_tag_fat(_tmp2DD,sizeof(char),27U);});_tmp76F(_tmp76E,_tag_fat(_tmp2DC,sizeof(void*),0U));});});}_LL88:;}
# 1966
done=1;{
# 1968
struct Cyc_Absyn_Exp*fn_e=Cyc_Toc__fat_ptr_inplace_plus_e;
({void*_tmp773=({struct Cyc_Absyn_Exp*_tmp2DF[3U];({struct Cyc_Absyn_Exp*_tmp771=Cyc_Toc_push_address_exp(e1);_tmp2DF[0]=_tmp771;}),({
struct Cyc_Absyn_Exp*_tmp770=Cyc_Absyn_sizeoftype_exp(Cyc_Toc_typ_to_c(elt_typ),0U);_tmp2DF[1]=_tmp770;}),_tmp2DF[2]=change;({struct Cyc_Absyn_Exp*_tmp772=fn_e;Cyc_Toc_fncall_exp_r(_tmp772,_tag_fat(_tmp2DF,sizeof(struct Cyc_Absyn_Exp*),3U));});});
# 1969
e->r=_tmp773;});}}else{
# 1972
if(Cyc_Tcutil_is_zero_pointer_type_elt(old_typ,& elt_typ)){
# 1975
enum Cyc_Absyn_Primop _stmttmp31=(enum Cyc_Absyn_Primop)popt->v;enum Cyc_Absyn_Primop _tmp2E0=_stmttmp31;if(_tmp2E0 == Cyc_Absyn_Plus){_LL90: _LL91:
# 1977
 done=1;
({void*_tmp775=({struct Cyc_Absyn_Exp*_tmp2E1[2U];_tmp2E1[0]=e1,_tmp2E1[1]=e2;({struct Cyc_Absyn_Exp*_tmp774=Cyc_Toc_getFunctionRemovePointer(& Cyc_Toc__zero_arr_inplace_plus_functionSet,e1);Cyc_Toc_fncall_exp_r(_tmp774,_tag_fat(_tmp2E1,sizeof(struct Cyc_Absyn_Exp*),2U));});});e->r=_tmp775;});
goto _LL8F;}else{_LL92: _LL93:
({void*_tmp2E2=0U;({int(*_tmp777)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp2E4)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Toc_toc_impos;_tmp2E4;});struct _fat_ptr _tmp776=({const char*_tmp2E3="bad zero-terminated pointer arithmetic";_tag_fat(_tmp2E3,sizeof(char),39U);});_tmp777(_tmp776,_tag_fat(_tmp2E2,sizeof(void*),0U));});});}_LL8F:;}}}
# 1957
if(!done){
# 1986
if(e1_poly)
({void*_tmp779=({void*_tmp778=Cyc_Toc_void_star_type();Cyc_Toc_cast_it_r(_tmp778,Cyc_Absyn_new_exp(e2->r,0U));});e2->r=_tmp779;});
# 1986
if(!
# 1992
Cyc_Absyn_is_lvalue(e1)){
({void(*_tmp77B)(struct Cyc_Absyn_Exp*e1,struct Cyc_List_List*fs,struct Cyc_Absyn_Exp*(*f)(struct Cyc_Absyn_Exp*,struct _tuple32*),struct _tuple32*f_env)=({void(*_tmp2E5)(struct Cyc_Absyn_Exp*e1,struct Cyc_List_List*fs,struct Cyc_Absyn_Exp*(*f)(struct Cyc_Absyn_Exp*,struct _tuple32*),struct _tuple32*f_env)=(void(*)(struct Cyc_Absyn_Exp*e1,struct Cyc_List_List*fs,struct Cyc_Absyn_Exp*(*f)(struct Cyc_Absyn_Exp*,struct _tuple32*),struct _tuple32*f_env))Cyc_Toc_lvalue_assign;_tmp2E5;});struct Cyc_Absyn_Exp*_tmp77A=e1;_tmp77B(_tmp77A,0,Cyc_Toc_assignop_lvalue,({struct _tuple32*_tmp2E6=_cycalloc(sizeof(*_tmp2E6));_tmp2E6->f1=popt,_tmp2E6->f2=e2;_tmp2E6;}));});
e->r=e1->r;}}
# 1957
goto _LL7;}}}}case 6U: _LL18: _tmp2A4=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmp25C)->f1;_tmp2A5=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmp25C)->f2;_tmp2A6=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmp25C)->f3;_LL19: {struct Cyc_Absyn_Exp*e1=_tmp2A4;struct Cyc_Absyn_Exp*e2=_tmp2A5;struct Cyc_Absyn_Exp*e3=_tmp2A6;
# 1999
Cyc_Toc_exp_to_c(nv,e1);
Cyc_Toc_exp_to_c(nv,e2);
Cyc_Toc_exp_to_c(nv,e3);
goto _LL7;}case 7U: _LL1A: _tmp2A2=((struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct*)_tmp25C)->f1;_tmp2A3=((struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct*)_tmp25C)->f2;_LL1B: {struct Cyc_Absyn_Exp*e1=_tmp2A2;struct Cyc_Absyn_Exp*e2=_tmp2A3;
_tmp2A0=e1;_tmp2A1=e2;goto _LL1D;}case 8U: _LL1C: _tmp2A0=((struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct*)_tmp25C)->f1;_tmp2A1=((struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct*)_tmp25C)->f2;_LL1D: {struct Cyc_Absyn_Exp*e1=_tmp2A0;struct Cyc_Absyn_Exp*e2=_tmp2A1;
_tmp29E=e1;_tmp29F=e2;goto _LL1F;}case 9U: _LL1E: _tmp29E=((struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct*)_tmp25C)->f1;_tmp29F=((struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct*)_tmp25C)->f2;_LL1F: {struct Cyc_Absyn_Exp*e1=_tmp29E;struct Cyc_Absyn_Exp*e2=_tmp29F;
# 2006
Cyc_Toc_exp_to_c(nv,e1);
Cyc_Toc_exp_to_c(nv,e2);
goto _LL7;}case 10U: if(((struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct*)_tmp25C)->f3 == 0){_LL20: _tmp29C=((struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct*)_tmp25C)->f1;_tmp29D=((struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct*)_tmp25C)->f2;_LL21: {struct Cyc_Absyn_Exp*e1=_tmp29C;struct Cyc_List_List*es=_tmp29D;
# 2010
void*e1_typ=(void*)_check_null(e1->topt);
Cyc_Toc_exp_to_c(nv,e1);
did_inserted_checks=1;
if(Cyc_Toc_do_null_check(e))
# 2015
({void*_tmp77F=({void*_tmp77E=Cyc_Toc_typ_to_c(e1_typ);Cyc_Toc_cast_it_r(_tmp77E,({struct Cyc_Absyn_Exp*_tmp2E7[1U];({struct Cyc_Absyn_Exp*_tmp77C=
Cyc_Absyn_copy_exp(e1);_tmp2E7[0]=_tmp77C;});({struct Cyc_Absyn_Exp*_tmp77D=Cyc_Toc__check_null_e;Cyc_Toc_fncall_exp_dl(_tmp77D,_tag_fat(_tmp2E7,sizeof(struct Cyc_Absyn_Exp*),1U));});}));});
# 2015
e1->r=_tmp77F;});
# 2013
({void(*_tmp781)(void(*f)(struct Cyc_Toc_Env*,struct Cyc_Absyn_Exp*),struct Cyc_Toc_Env*env,struct Cyc_List_List*x)=({void(*_tmp2E8)(void(*f)(struct Cyc_Toc_Env*,struct Cyc_Absyn_Exp*),struct Cyc_Toc_Env*env,struct Cyc_List_List*x)=(void(*)(void(*f)(struct Cyc_Toc_Env*,struct Cyc_Absyn_Exp*),struct Cyc_Toc_Env*env,struct Cyc_List_List*x))Cyc_List_iter_c;_tmp2E8;});struct Cyc_Toc_Env*_tmp780=nv;_tmp781(Cyc_Toc_exp_to_c,_tmp780,es);});
# 2018
goto _LL7;}}else{_LL24: _tmp297=((struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct*)_tmp25C)->f1;_tmp298=((struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct*)_tmp25C)->f2;_tmp299=(((struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct*)_tmp25C)->f3)->num_varargs;_tmp29A=(((struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct*)_tmp25C)->f3)->injectors;_tmp29B=(((struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct*)_tmp25C)->f3)->vai;_LL25: {struct Cyc_Absyn_Exp*e1=_tmp297;struct Cyc_List_List*es=_tmp298;int num_varargs=_tmp299;struct Cyc_List_List*injectors=_tmp29A;struct Cyc_Absyn_VarargInfo*vai=_tmp29B;
# 2034 "toc.cyc"
struct _RegionHandle _tmp2E9=_new_region("r");struct _RegionHandle*r=& _tmp2E9;_push_region(r);{
struct _tuple30 _stmttmp32=Cyc_Toc_temp_var_and_exp();struct Cyc_Absyn_Exp*_tmp2EB;struct _tuple1*_tmp2EA;_LL95: _tmp2EA=_stmttmp32.f1;_tmp2EB=_stmttmp32.f2;_LL96: {struct _tuple1*argv=_tmp2EA;struct Cyc_Absyn_Exp*argvexp=_tmp2EB;
struct Cyc_Absyn_Exp*num_varargs_exp=Cyc_Absyn_uint_exp((unsigned)num_varargs,0U);
void*cva_type=Cyc_Toc_typ_to_c(vai->type);
void*arr_type=Cyc_Absyn_array_type(cva_type,Cyc_Toc_mt_tq,num_varargs_exp,Cyc_Absyn_false_type,0U);
# 2041
int num_args=Cyc_List_length(es);
int num_normargs=num_args - num_varargs;
# 2045
struct Cyc_List_List*new_args=0;
{int i=0;for(0;i < num_normargs;(++ i,es=es->tl)){
Cyc_Toc_exp_to_c(nv,(struct Cyc_Absyn_Exp*)((struct Cyc_List_List*)_check_null(es))->hd);
new_args=({struct Cyc_List_List*_tmp2EC=_cycalloc(sizeof(*_tmp2EC));_tmp2EC->hd=(struct Cyc_Absyn_Exp*)es->hd,_tmp2EC->tl=new_args;_tmp2EC;});}}
# 2050
new_args=({struct Cyc_List_List*_tmp2EE=_cycalloc(sizeof(*_tmp2EE));({struct Cyc_Absyn_Exp*_tmp784=({struct Cyc_Absyn_Exp*_tmp2ED[3U];_tmp2ED[0]=argvexp,({
# 2052
struct Cyc_Absyn_Exp*_tmp782=Cyc_Absyn_sizeoftype_exp(cva_type,0U);_tmp2ED[1]=_tmp782;}),_tmp2ED[2]=num_varargs_exp;({struct Cyc_Absyn_Exp*_tmp783=Cyc_Toc__tag_fat_e;Cyc_Toc_fncall_exp_dl(_tmp783,_tag_fat(_tmp2ED,sizeof(struct Cyc_Absyn_Exp*),3U));});});
# 2050
_tmp2EE->hd=_tmp784;}),_tmp2EE->tl=new_args;_tmp2EE;});
# 2055
new_args=Cyc_List_imp_rev(new_args);{
# 2057
void*e1_typ=(void*)_check_null(e1->topt);
Cyc_Toc_exp_to_c(nv,e1);
did_inserted_checks=1;
if(Cyc_Toc_do_null_check(e))
# 2062
({void*_tmp788=({void*_tmp787=Cyc_Toc_typ_to_c(e1_typ);Cyc_Toc_cast_it_r(_tmp787,({struct Cyc_Absyn_Exp*_tmp2EF[1U];({struct Cyc_Absyn_Exp*_tmp785=
Cyc_Absyn_copy_exp(e1);_tmp2EF[0]=_tmp785;});({struct Cyc_Absyn_Exp*_tmp786=Cyc_Toc__check_null_e;Cyc_Toc_fncall_exp_dl(_tmp786,_tag_fat(_tmp2EF,sizeof(struct Cyc_Absyn_Exp*),1U));});}));});
# 2062
e1->r=_tmp788;});{
# 2060
struct Cyc_Absyn_Stmt*s=
# 2064
Cyc_Absyn_exp_stmt(Cyc_Absyn_fncall_exp(e1,new_args,0U),0U);
# 2067
if(vai->inject){
struct Cyc_Absyn_Datatypedecl*tud;
{void*_stmttmp33=Cyc_Tcutil_compress(Cyc_Tcutil_pointer_elt_type(vai->type));void*_tmp2F0=_stmttmp33;struct Cyc_Absyn_Datatypedecl*_tmp2F1;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2F0)->tag == 0U){if(((struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2F0)->f1)->tag == 18U){if(((((struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2F0)->f1)->f1).KnownDatatype).tag == 2){_LL98: _tmp2F1=*((((struct Cyc_Absyn_DatatypeCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp2F0)->f1)->f1).KnownDatatype).val;_LL99: {struct Cyc_Absyn_Datatypedecl*x=_tmp2F1;
tud=x;goto _LL97;}}else{goto _LL9A;}}else{goto _LL9A;}}else{_LL9A: _LL9B:
({void*_tmp2F2=0U;({int(*_tmp78A)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp2F4)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Toc_toc_impos;_tmp2F4;});struct _fat_ptr _tmp789=({const char*_tmp2F3="toc: unknown datatype in vararg with inject";_tag_fat(_tmp2F3,sizeof(char),44U);});_tmp78A(_tmp789,_tag_fat(_tmp2F2,sizeof(void*),0U));});});}_LL97:;}{
# 2073
struct _fat_ptr vs=({unsigned _tmp2FB=(unsigned)num_varargs;struct _tuple1**_tmp2FA=({struct _RegionHandle*_tmp78B=r;_region_malloc(_tmp78B,_check_times(_tmp2FB,sizeof(struct _tuple1*)));});({{unsigned _tmp5F9=(unsigned)num_varargs;unsigned i;for(i=0;i < _tmp5F9;++ i){({struct _tuple1*_tmp78C=Cyc_Toc_temp_var();_tmp2FA[i]=_tmp78C;});}}0;});_tag_fat(_tmp2FA,sizeof(struct _tuple1*),_tmp2FB);});
# 2075
if(num_varargs != 0){
# 2079
struct Cyc_List_List*array_args=0;
{int i=num_varargs - 1;for(0;i >= 0;-- i){
array_args=({struct Cyc_List_List*_tmp2F5=_cycalloc(sizeof(*_tmp2F5));({struct Cyc_Absyn_Exp*_tmp78D=Cyc_Absyn_address_exp(Cyc_Absyn_var_exp(*((struct _tuple1**)_check_fat_subscript(vs,sizeof(struct _tuple1*),i)),0U),0U);_tmp2F5->hd=_tmp78D;}),_tmp2F5->tl=array_args;_tmp2F5;});}}
s=({struct _tuple1*_tmp790=argv;void*_tmp78F=arr_type;struct Cyc_Absyn_Exp*_tmp78E=Cyc_Absyn_array_exp(array_args,0U);Cyc_Absyn_declare_stmt(_tmp790,_tmp78F,_tmp78E,s,0U);});
# 2084
es=Cyc_List_imp_rev(es);
injectors=Cyc_List_imp_rev(injectors);{
int i=Cyc_List_length(es)- 1;
for(0;es != 0;(es=es->tl,injectors=injectors->tl,-- i)){
struct Cyc_Absyn_Exp*arg=(struct Cyc_Absyn_Exp*)es->hd;
void*arg_type=(void*)_check_null(arg->topt);
struct _tuple1*var=*((struct _tuple1**)_check_fat_subscript(vs,sizeof(struct _tuple1*),i));
struct Cyc_Absyn_Exp*varexp=Cyc_Absyn_var_exp(var,0U);
struct Cyc_Absyn_Datatypefield*_stmttmp34=(struct Cyc_Absyn_Datatypefield*)((struct Cyc_List_List*)_check_null(injectors))->hd;struct Cyc_List_List*_tmp2F7;struct _tuple1*_tmp2F6;_LL9D: _tmp2F6=_stmttmp34->name;_tmp2F7=_stmttmp34->typs;_LL9E: {struct _tuple1*qv=_tmp2F6;struct Cyc_List_List*tqts=_tmp2F7;
void*field_typ=Cyc_Toc_typ_to_c((*((struct _tuple13*)((struct Cyc_List_List*)_check_null(tqts))->hd)).f2);
Cyc_Toc_exp_to_c(nv,arg);
if(Cyc_Toc_is_void_star_or_boxed_tvar(field_typ))
arg=Cyc_Toc_cast_it(field_typ,arg);{
# 2095
struct _tuple1*tdn=
# 2098
Cyc_Toc_collapse_qvars(qv,tud->name);
struct Cyc_List_List*dles=({struct _tuple20*_tmp2F9[2U];({struct _tuple20*_tmp794=({struct _fat_ptr*_tmp793=Cyc_Toc_tag_sp;Cyc_Toc_make_field(_tmp793,Cyc_Toc_datatype_tag(tud,qv));});_tmp2F9[0]=_tmp794;}),({
struct _tuple20*_tmp792=({struct _fat_ptr*_tmp791=Cyc_Absyn_fieldname(1);Cyc_Toc_make_field(_tmp791,arg);});_tmp2F9[1]=_tmp792;});Cyc_List_list(_tag_fat(_tmp2F9,sizeof(struct _tuple20*),2U));});
s=({struct _tuple1*_tmp797=var;void*_tmp796=Cyc_Absyn_strctq(tdn);struct Cyc_Absyn_Exp*_tmp795=
Cyc_Absyn_new_exp((void*)({struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct*_tmp2F8=_cycalloc(sizeof(*_tmp2F8));_tmp2F8->tag=29U,_tmp2F8->f1=tdn,_tmp2F8->f2=0,_tmp2F8->f3=dles,_tmp2F8->f4=0;_tmp2F8;}),0U);
# 2101
Cyc_Absyn_declare_stmt(_tmp797,_tmp796,_tmp795,s,0U);});}}}}}else{
# 2109
s=({struct _tuple1*_tmp79A=argv;void*_tmp799=Cyc_Toc_void_star_type();struct Cyc_Absyn_Exp*_tmp798=Cyc_Absyn_uint_exp(0U,0U);Cyc_Absyn_declare_stmt(_tmp79A,_tmp799,_tmp798,s,0U);});}}}else{
# 2114
if(num_varargs != 0){
struct Cyc_List_List*array_args=0;
for(0;es != 0;es=es->tl){
Cyc_Toc_exp_to_c(nv,(struct Cyc_Absyn_Exp*)es->hd);
array_args=({struct Cyc_List_List*_tmp2FC=_cycalloc(sizeof(*_tmp2FC));_tmp2FC->hd=(struct Cyc_Absyn_Exp*)es->hd,_tmp2FC->tl=array_args;_tmp2FC;});}{
# 2120
struct Cyc_Absyn_Exp*init=Cyc_Absyn_array_exp(Cyc_List_imp_rev(array_args),0U);
s=Cyc_Absyn_declare_stmt(argv,arr_type,init,s,0U);}}else{
# 2123
s=({struct _tuple1*_tmp79D=argv;void*_tmp79C=Cyc_Toc_void_star_type();struct Cyc_Absyn_Exp*_tmp79B=Cyc_Absyn_uint_exp(0U,0U);Cyc_Absyn_declare_stmt(_tmp79D,_tmp79C,_tmp79B,s,0U);});}}
# 2126
({void*_tmp79E=Cyc_Toc_stmt_exp_r(s);e->r=_tmp79E;});}}}}
# 2128
_npop_handler(0U);goto _LL7;
# 2034
;_pop_region(r);}}case 34U: _LL22: _tmp295=((struct Cyc_Absyn_Spawn_e_Absyn_Raw_exp_struct*)_tmp25C)->f1;_tmp296=((struct Cyc_Absyn_Spawn_e_Absyn_Raw_exp_struct*)_tmp25C)->f2;_LL23: {struct Cyc_Absyn_Exp*e1=_tmp295;struct Cyc_Absyn_Exp*e2=_tmp296;
# 2021 "toc.cyc"
Cyc_Toc_exp_to_c(nv,e1);
Cyc_Toc_exp_to_c(nv,e2);
({void*_tmp79F=(Cyc_Toc_spawn_exp(e1,e2))->r;e->r=_tmp79F;});
goto _LL7;}case 11U: _LL26: _tmp293=((struct Cyc_Absyn_Throw_e_Absyn_Raw_exp_struct*)_tmp25C)->f1;_tmp294=((struct Cyc_Absyn_Throw_e_Absyn_Raw_exp_struct*)_tmp25C)->f2;_LL27: {struct Cyc_Absyn_Exp*e1=_tmp293;int b=_tmp294;
# 2131 "toc.cyc"
Cyc_Toc_exp_to_c(nv,e1);{
struct Cyc_Absyn_Exp*fn_e=b?Cyc_Toc__rethrow_e: Cyc_Toc__throw_e;
({void*_tmp7A2=({void*_tmp7A1=Cyc_Toc_typ_to_c(old_typ);Cyc_Toc_array_to_ptr_cast(_tmp7A1,({struct Cyc_Absyn_Exp*_tmp2FD[1U];_tmp2FD[0]=e1;({struct Cyc_Absyn_Exp*_tmp7A0=fn_e;Cyc_Toc_fncall_exp_dl(_tmp7A0,_tag_fat(_tmp2FD,sizeof(struct Cyc_Absyn_Exp*),1U));});}),0U);})->r;e->r=_tmp7A2;});
goto _LL7;}}case 12U: _LL28: _tmp292=((struct Cyc_Absyn_NoInstantiate_e_Absyn_Raw_exp_struct*)_tmp25C)->f1;_LL29: {struct Cyc_Absyn_Exp*e1=_tmp292;
Cyc_Toc_exp_to_c(nv,e1);goto _LL7;}case 13U: _LL2A: _tmp290=((struct Cyc_Absyn_Instantiate_e_Absyn_Raw_exp_struct*)_tmp25C)->f1;_tmp291=((struct Cyc_Absyn_Instantiate_e_Absyn_Raw_exp_struct*)_tmp25C)->f2;_LL2B: {struct Cyc_Absyn_Exp*e1=_tmp290;struct Cyc_List_List*ts=_tmp291;
# 2137
Cyc_Toc_exp_to_c(nv,e1);{
# 2146 "toc.cyc"
void*tc=Cyc_Toc_typ_to_c((void*)_check_null(e->topt));
void*tc1=Cyc_Toc_typ_to_c((void*)_check_null(e1->topt));
if(!Cyc_Tcutil_typecmp(tc,tc1)== 0){
# 2158 "toc.cyc"
struct _tuple30 _stmttmp35=Cyc_Toc_temp_var_and_exp();struct Cyc_Absyn_Exp*_tmp2FF;struct _tuple1*_tmp2FE;_LLA0: _tmp2FE=_stmttmp35.f1;_tmp2FF=_stmttmp35.f2;_LLA1: {struct _tuple1*temp=_tmp2FE;struct Cyc_Absyn_Exp*temp_exp=_tmp2FF;
({void*_tmp7A7=Cyc_Toc_stmt_exp_r(({struct _tuple1*_tmp7A6=temp;void*_tmp7A5=tc;struct Cyc_Absyn_Exp*_tmp7A4=Cyc_Toc_array_to_ptr_cast(tc,e1,0U);struct Cyc_Absyn_Stmt*_tmp7A3=
Cyc_Absyn_exp_stmt(temp_exp,e->loc);
# 2159
Cyc_Absyn_declare_stmt(_tmp7A6,_tmp7A5,_tmp7A4,_tmp7A3,e->loc);}));e->r=_tmp7A7;});}}
# 2148 "toc.cyc"
goto _LL7;}}case 14U: _LL2C: _tmp28C=(void**)&((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp25C)->f1;_tmp28D=((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp25C)->f2;_tmp28E=((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp25C)->f3;_tmp28F=((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp25C)->f4;_LL2D: {void**t=_tmp28C;struct Cyc_Absyn_Exp*e1=_tmp28D;int user_inserted=_tmp28E;enum Cyc_Absyn_Coercion coercion=_tmp28F;
# 2190 "toc.cyc"
void*old_t2=(void*)_check_null(e1->topt);
void*old_t2_c=Cyc_Toc_typ_to_c(old_t2);
void*new_typ=*t;
void*new_typ_c=Cyc_Toc_typ_to_c(new_typ);
*t=new_typ_c;
Cyc_Toc_exp_to_c(nv,e1);
# 2197
{struct _tuple33 _stmttmp36=({struct _tuple33 _tmp5FB;({void*_tmp7A9=Cyc_Tcutil_compress(old_t2);_tmp5FB.f1=_tmp7A9;}),({void*_tmp7A8=Cyc_Tcutil_compress(new_typ);_tmp5FB.f2=_tmp7A8;});_tmp5FB;});struct _tuple33 _tmp300=_stmttmp36;struct Cyc_Absyn_PtrInfo _tmp301;struct Cyc_Absyn_PtrInfo _tmp303;struct Cyc_Absyn_PtrInfo _tmp302;if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp300.f1)->tag == 3U)switch(*((int*)_tmp300.f2)){case 3U: _LLA3: _tmp302=((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp300.f1)->f1;_tmp303=((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp300.f2)->f1;_LLA4: {struct Cyc_Absyn_PtrInfo p1=_tmp302;struct Cyc_Absyn_PtrInfo p2=_tmp303;
# 2199
struct Cyc_Absyn_Exp*b1=({void*_tmp7AA=Cyc_Absyn_bounds_one();Cyc_Tcutil_get_bounds_exp(_tmp7AA,(p1.ptr_atts).bounds);});
struct Cyc_Absyn_Exp*b2=({void*_tmp7AB=Cyc_Absyn_bounds_one();Cyc_Tcutil_get_bounds_exp(_tmp7AB,(p2.ptr_atts).bounds);});
int zt1=Cyc_Tcutil_force_type2bool(0,(p1.ptr_atts).zero_term);
int zt2=Cyc_Tcutil_force_type2bool(0,(p2.ptr_atts).zero_term);
{struct _tuple34 _stmttmp37=({struct _tuple34 _tmp5FA;_tmp5FA.f1=b1,_tmp5FA.f2=b2;_tmp5FA;});struct _tuple34 _tmp304=_stmttmp37;if(_tmp304.f1 != 0){if(_tmp304.f2 != 0){_LLAA: _LLAB:
# 2206
 did_inserted_checks=1;
if(Cyc_Toc_do_null_check(e))
({void*_tmp7AE=({void*_tmp7AD=*t;Cyc_Toc_cast_it_r(_tmp7AD,({struct Cyc_Absyn_Exp*_tmp305[1U];_tmp305[0]=e1;({struct Cyc_Absyn_Exp*_tmp7AC=Cyc_Toc__check_null_e;Cyc_Toc_fncall_exp_dl(_tmp7AC,_tag_fat(_tmp305,sizeof(struct Cyc_Absyn_Exp*),1U));});}));});e->r=_tmp7AE;});else{
if(Cyc_Unify_unify(old_t2_c,new_typ_c))
e->r=e1->r;}
# 2207
goto _LLA9;}else{_LLAC: _LLAD: {
# 2217
struct Cyc_Absyn_Exp*e2=(struct Cyc_Absyn_Exp*)_check_null(b1);
struct _tuple14 _stmttmp38=Cyc_Evexp_eval_const_uint_exp(e2);int _tmp307;unsigned _tmp306;_LLB3: _tmp306=_stmttmp38.f1;_tmp307=_stmttmp38.f2;_LLB4: {unsigned i=_tmp306;int valid=_tmp307;
if(Cyc_Toc_is_toplevel(nv)){
# 2223
if((zt1 && !(p2.elt_tq).real_const)&& !zt2)
e2=({struct Cyc_Absyn_Exp*_tmp7AF=e2;Cyc_Absyn_prim2_exp(Cyc_Absyn_Minus,_tmp7AF,Cyc_Absyn_uint_exp(1U,0U),0U);});
# 2223
({void*_tmp7B0=(
# 2225
Cyc_Toc_make_toplevel_dyn_arr(old_t2,e2,e1))->r;
# 2223
e->r=_tmp7B0;});}else{
# 2228
if(zt1){
# 2233
struct _tuple30 _stmttmp39=Cyc_Toc_temp_var_and_exp();struct Cyc_Absyn_Exp*_tmp309;struct _tuple1*_tmp308;_LLB6: _tmp308=_stmttmp39.f1;_tmp309=_stmttmp39.f2;_LLB7: {struct _tuple1*x=_tmp308;struct Cyc_Absyn_Exp*x_exp=_tmp309;
struct Cyc_Absyn_Exp*arg3;
# 2237
{void*_stmttmp3A=e1->r;void*_tmp30A=_stmttmp3A;struct Cyc_Absyn_Vardecl*_tmp30B;struct Cyc_Absyn_Vardecl*_tmp30C;switch(*((int*)_tmp30A)){case 0U: switch(((((struct Cyc_Absyn_Const_e_Absyn_Raw_exp_struct*)_tmp30A)->f1).Wstring_c).tag){case 8U: _LLB9: _LLBA:
 arg3=e2;goto _LLB8;case 9U: _LLBB: _LLBC:
 arg3=e2;goto _LLB8;default: goto _LLC1;}case 1U: switch(*((int*)((struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct*)_tmp30A)->f1)){case 1U: _LLBD: _tmp30C=((struct Cyc_Absyn_Global_b_Absyn_Binding_struct*)((struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct*)_tmp30A)->f1)->f1;_LLBE: {struct Cyc_Absyn_Vardecl*vd=_tmp30C;
_tmp30B=vd;goto _LLC0;}case 4U: _LLBF: _tmp30B=((struct Cyc_Absyn_Local_b_Absyn_Binding_struct*)((struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct*)_tmp30A)->f1)->f1;_LLC0: {struct Cyc_Absyn_Vardecl*vd=_tmp30B;
# 2242
if(!Cyc_Tcutil_is_array_type(vd->type))
goto _LLC2;
# 2242
arg3=e2;
# 2245
goto _LLB8;}default: goto _LLC1;}default: _LLC1: _LLC2:
# 2249
 if(valid && i != (unsigned)1)
arg3=e2;else{
# 2252
arg3=({struct Cyc_Absyn_Exp*_tmp30D[2U];({
struct Cyc_Absyn_Exp*_tmp7B2=({void*_tmp7B1=Cyc_Toc_void_star_type();Cyc_Toc_cast_it(_tmp7B1,x_exp);});_tmp30D[0]=_tmp7B2;}),_tmp30D[1]=e2;({struct Cyc_Absyn_Exp*_tmp7B3=
# 2252
Cyc_Toc_getFunctionRemovePointer(& Cyc_Toc__get_zero_arr_size_functionSet,e1);Cyc_Toc_fncall_exp_dl(_tmp7B3,_tag_fat(_tmp30D,sizeof(struct Cyc_Absyn_Exp*),2U));});});}
# 2254
goto _LLB8;}_LLB8:;}
# 2256
if(!zt2 && !(p2.elt_tq).real_const)
# 2259
arg3=({struct Cyc_Absyn_Exp*_tmp7B4=arg3;Cyc_Absyn_prim2_exp(Cyc_Absyn_Minus,_tmp7B4,Cyc_Absyn_uint_exp(1U,0U),0U);});{
# 2256
struct Cyc_Absyn_Exp*arg2=
# 2261
Cyc_Absyn_sizeoftype_exp(Cyc_Toc_typ_to_c(p2.elt_type),0U);
struct Cyc_Absyn_Exp*tg_exp=({struct Cyc_Absyn_Exp*_tmp30E[3U];_tmp30E[0]=x_exp,_tmp30E[1]=arg2,_tmp30E[2]=arg3;({struct Cyc_Absyn_Exp*_tmp7B5=Cyc_Toc__tag_fat_e;Cyc_Toc_fncall_exp_dl(_tmp7B5,_tag_fat(_tmp30E,sizeof(struct Cyc_Absyn_Exp*),3U));});});
struct Cyc_Absyn_Stmt*s=Cyc_Absyn_exp_stmt(tg_exp,0U);
s=({struct _tuple1*_tmp7B8=x;void*_tmp7B7=Cyc_Toc_typ_to_c(old_t2);struct Cyc_Absyn_Exp*_tmp7B6=e1;Cyc_Absyn_declare_stmt(_tmp7B8,_tmp7B7,_tmp7B6,s,0U);});
({void*_tmp7B9=Cyc_Toc_stmt_exp_r(s);e->r=_tmp7B9;});}}}else{
# 2268
({void*_tmp7BC=({struct Cyc_Absyn_Exp*_tmp30F[3U];_tmp30F[0]=e1,({
# 2270
struct Cyc_Absyn_Exp*_tmp7BA=Cyc_Absyn_sizeoftype_exp(Cyc_Toc_typ_to_c(p2.elt_type),0U);_tmp30F[1]=_tmp7BA;}),_tmp30F[2]=e2;({struct Cyc_Absyn_Exp*_tmp7BB=Cyc_Toc__tag_fat_e;Cyc_Toc_fncall_exp_r(_tmp7BB,_tag_fat(_tmp30F,sizeof(struct Cyc_Absyn_Exp*),3U));});});
# 2268
e->r=_tmp7BC;});}}
# 2273
goto _LLA9;}}}}else{if(_tmp304.f2 != 0){_LLAE: _LLAF: {
# 2283 "toc.cyc"
struct Cyc_Absyn_Exp*new_e2=(struct Cyc_Absyn_Exp*)_check_null(b2);
if(zt1 && !zt2)
new_e2=({struct Cyc_Absyn_Exp*_tmp7BD=b2;Cyc_Absyn_add_exp(_tmp7BD,Cyc_Absyn_uint_exp(1U,0U),0U);});{
# 2284
struct Cyc_Absyn_Exp*ptr_exp=({struct Cyc_Absyn_Exp*_tmp311[3U];_tmp311[0]=e1,({
# 2292
struct Cyc_Absyn_Exp*_tmp7BE=Cyc_Absyn_sizeoftype_exp(Cyc_Toc_typ_to_c(p1.elt_type),0U);_tmp311[1]=_tmp7BE;}),_tmp311[2]=new_e2;({struct Cyc_Absyn_Exp*_tmp7BF=Cyc_Toc__untag_fat_ptr_e;Cyc_Toc_fncall_exp_dl(_tmp7BF,_tag_fat(_tmp311,sizeof(struct Cyc_Absyn_Exp*),3U));});});
# 2294
did_inserted_checks=1;
if(Cyc_Toc_do_null_check(e))
({void*_tmp7C2=({struct Cyc_Absyn_Exp*_tmp310[1U];({struct Cyc_Absyn_Exp*_tmp7C0=Cyc_Absyn_copy_exp(ptr_exp);_tmp310[0]=_tmp7C0;});({struct Cyc_Absyn_Exp*_tmp7C1=Cyc_Toc__check_null_e;Cyc_Toc_fncall_exp_r(_tmp7C1,_tag_fat(_tmp310,sizeof(struct Cyc_Absyn_Exp*),1U));});});ptr_exp->r=_tmp7C2;});
# 2295
({void*_tmp7C3=
# 2297
Cyc_Toc_cast_it_r(*t,ptr_exp);
# 2295
e->r=_tmp7C3;});
# 2298
goto _LLA9;}}}else{_LLB0: _LLB1:
# 2302
 if((zt1 && !zt2)&& !(p2.elt_tq).real_const){
if(Cyc_Toc_is_toplevel(nv))
({void*_tmp312=0U;({int(*_tmp7C5)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp314)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Toc_unimp;_tmp314;});struct _fat_ptr _tmp7C4=({const char*_tmp313="can't coerce a ZEROTERM to a non-const NOZEROTERM pointer at toplevel";_tag_fat(_tmp313,sizeof(char),70U);});_tmp7C5(_tmp7C4,_tag_fat(_tmp312,sizeof(void*),0U));});});
# 2303
({void*_tmp7C9=({struct Cyc_Absyn_Exp*_tmp315[3U];_tmp315[0]=e1,({
# 2306
struct Cyc_Absyn_Exp*_tmp7C7=Cyc_Absyn_sizeoftype_exp(Cyc_Toc_typ_to_c(p1.elt_type),0U);_tmp315[1]=_tmp7C7;}),({
struct Cyc_Absyn_Exp*_tmp7C6=Cyc_Absyn_uint_exp(1U,0U);_tmp315[2]=_tmp7C6;});({struct Cyc_Absyn_Exp*_tmp7C8=Cyc_Toc__fat_ptr_decrease_size_e;Cyc_Toc_fncall_exp_r(_tmp7C8,_tag_fat(_tmp315,sizeof(struct Cyc_Absyn_Exp*),3U));});});
# 2303
e->r=_tmp7C9;});}
# 2302
goto _LLA9;}}_LLA9:;}
# 2311
goto _LLA2;}case 0U: if(((struct Cyc_Absyn_IntCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp300.f2)->f1)->tag == 1U){_LLA5: _tmp301=((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp300.f1)->f1;_LLA6: {struct Cyc_Absyn_PtrInfo p1=_tmp301;
# 2313
{struct Cyc_Absyn_Exp*_stmttmp3B=({void*_tmp7CA=Cyc_Absyn_bounds_one();Cyc_Tcutil_get_bounds_exp(_tmp7CA,(p1.ptr_atts).bounds);});struct Cyc_Absyn_Exp*_tmp316=_stmttmp3B;if(_tmp316 == 0){_LLC4: _LLC5:
# 2315
({void*_tmp7CC=({struct Cyc_Absyn_Exp*_tmp7CB=Cyc_Absyn_new_exp(e1->r,e1->loc);Cyc_Toc_aggrmember_exp_r(_tmp7CB,Cyc_Toc_curr_sp);});e1->r=_tmp7CC;});
e1->topt=new_typ_c;
goto _LLC3;}else{_LLC6: _LLC7:
 goto _LLC3;}_LLC3:;}
# 2320
goto _LLA2;}}else{goto _LLA7;}default: goto _LLA7;}else{_LLA7: _LLA8:
# 2322
 if(Cyc_Unify_unify(old_t2_c,new_typ_c))
e->r=e1->r;
# 2322
goto _LLA2;}_LLA2:;}
# 2326
goto _LL7;}case 15U: _LL2E: _tmp28B=((struct Cyc_Absyn_Address_e_Absyn_Raw_exp_struct*)_tmp25C)->f1;_LL2F: {struct Cyc_Absyn_Exp*e1=_tmp28B;
# 2329
Cyc_Toc_set_lhs(nv,1);
Cyc_Toc_exp_to_c(nv,e1);
Cyc_Toc_set_lhs(nv,0);
if(!Cyc_Absyn_is_lvalue(e1)){
({void(*_tmp7CD)(struct Cyc_Absyn_Exp*e1,struct Cyc_List_List*fs,struct Cyc_Absyn_Exp*(*f)(struct Cyc_Absyn_Exp*,int),int f_env)=({void(*_tmp317)(struct Cyc_Absyn_Exp*e1,struct Cyc_List_List*fs,struct Cyc_Absyn_Exp*(*f)(struct Cyc_Absyn_Exp*,int),int f_env)=(void(*)(struct Cyc_Absyn_Exp*e1,struct Cyc_List_List*fs,struct Cyc_Absyn_Exp*(*f)(struct Cyc_Absyn_Exp*,int),int f_env))Cyc_Toc_lvalue_assign;_tmp317;});_tmp7CD(e1,0,Cyc_Toc_address_lvalue,1);});
# 2335
({void*_tmp7CF=({void*_tmp7CE=Cyc_Toc_typ_to_c((void*)_check_null(e->topt));Cyc_Toc_cast_it_r(_tmp7CE,e1);});e->r=_tmp7CF;});}else{
if(Cyc_Tcutil_kind_leq(& Cyc_Tcutil_ak,Cyc_Tcutil_type_kind((void*)_check_null(e1->topt))))
# 2339
({void*_tmp7D1=({void*_tmp7D0=Cyc_Toc_typ_to_c((void*)_check_null(e->topt));Cyc_Toc_cast_it_r(_tmp7D0,e1);});e->r=_tmp7D1;});}
# 2332
goto _LL7;}case 16U: _LL30: _tmp289=((struct Cyc_Absyn_New_e_Absyn_Raw_exp_struct*)_tmp25C)->f1;_tmp28A=((struct Cyc_Absyn_New_e_Absyn_Raw_exp_struct*)_tmp25C)->f2;_LL31: {struct Cyc_Absyn_Exp*rgnopt=_tmp289;struct Cyc_Absyn_Exp*e1=_tmp28A;
# 2348
Cyc_Toc_exp_to_c(nv,e1);{
# 2350
void*elt_typ=Cyc_Toc_typ_to_c((void*)_check_null(e1->topt));
struct _tuple30 _stmttmp3C=Cyc_Toc_temp_var_and_exp();struct Cyc_Absyn_Exp*_tmp319;struct _tuple1*_tmp318;_LLC9: _tmp318=_stmttmp3C.f1;_tmp319=_stmttmp3C.f2;_LLCA: {struct _tuple1*x=_tmp318;struct Cyc_Absyn_Exp*xexp=_tmp319;
struct Cyc_Absyn_Exp*lhs;
{void*_stmttmp3D=Cyc_Tcutil_compress(elt_typ);void*_tmp31A=_stmttmp3D;void*_tmp31B;if(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp31A)->tag == 4U){_LLCC: _tmp31B=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp31A)->f1).elt_type;_LLCD: {void*t2=_tmp31B;
# 2355
elt_typ=Cyc_Toc_typ_to_c(t2);
lhs=Cyc_Absyn_copy_exp(xexp);
goto _LLCB;}}else{_LLCE: _LLCF:
# 2359
 lhs=Cyc_Absyn_deref_exp(Cyc_Absyn_copy_exp(xexp),0U);
goto _LLCB;}_LLCB:;}{
# 2362
struct Cyc_Absyn_Exp*array_len=Cyc_Toc_array_length_exp(e1);
struct _tuple1*lenvar=(unsigned)array_len?Cyc_Toc_temp_var(): 0;
struct Cyc_Absyn_Exp*lenexp=(unsigned)array_len?Cyc_Absyn_var_exp((struct _tuple1*)_check_null(lenvar),0U): 0;
struct Cyc_Absyn_Exp*mexp;
if((unsigned)lenexp)
mexp=({struct Cyc_Absyn_Exp*_tmp31C[2U];_tmp31C[0]=lenexp,({struct Cyc_Absyn_Exp*_tmp7D2=Cyc_Absyn_sizeoftype_exp(elt_typ,0U);_tmp31C[1]=_tmp7D2;});({struct Cyc_Absyn_Exp*_tmp7D3=Cyc_Toc__check_times_e;Cyc_Toc_fncall_exp_dl(_tmp7D3,_tag_fat(_tmp31C,sizeof(struct Cyc_Absyn_Exp*),2U));});});else{
# 2369
mexp=Cyc_Absyn_sizeofexp_exp(Cyc_Absyn_deref_exp(xexp,0U),0U);}{
# 2372
struct Cyc_Absyn_Exp*vse=Cyc_Toc_get_varsizeexp(e1);
if(vse != 0)
mexp=Cyc_Absyn_add_exp(mexp,vse,0U);
# 2373
if(
# 2376
rgnopt == 0 || Cyc_Absyn_no_regions)
mexp=Cyc_Toc_malloc_exp(elt_typ,mexp);else{
# 2379
Cyc_Toc_exp_to_c(nv,rgnopt);
mexp=Cyc_Toc_rmalloc_exp(rgnopt,mexp);}{
# 2382
struct Cyc_Absyn_Exp*answer;
if(Cyc_Tcutil_is_fat_ptr(old_typ))
answer=({struct Cyc_Absyn_Exp*_tmp31D[3U];({
struct Cyc_Absyn_Exp*_tmp7D6=Cyc_Absyn_copy_exp(xexp);_tmp31D[0]=_tmp7D6;}),({
struct Cyc_Absyn_Exp*_tmp7D5=Cyc_Absyn_sizeoftype_exp(elt_typ,0U);_tmp31D[1]=_tmp7D5;}),
(unsigned)lenexp?_tmp31D[2]=lenexp:({struct Cyc_Absyn_Exp*_tmp7D4=Cyc_Absyn_uint_exp(1U,0U);_tmp31D[2]=_tmp7D4;});({struct Cyc_Absyn_Exp*_tmp7D7=Cyc_Toc__tag_fat_e;Cyc_Toc_fncall_exp_dl(_tmp7D7,_tag_fat(_tmp31D,sizeof(struct Cyc_Absyn_Exp*),3U));});});else{
# 2389
answer=Cyc_Absyn_copy_exp(xexp);}
({void*_tmp7D8=(void*)({struct Cyc_Toc_Dest_Absyn_AbsynAnnot_struct*_tmp31E=_cycalloc(sizeof(*_tmp31E));_tmp31E->tag=Cyc_Toc_Dest,_tmp31E->f1=xexp;_tmp31E;});e->annot=_tmp7D8;});{
struct Cyc_Absyn_Stmt*s=({struct _tuple1*_tmp7DC=x;void*_tmp7DB=Cyc_Absyn_cstar_type(elt_typ,Cyc_Toc_mt_tq);struct Cyc_Absyn_Exp*_tmp7DA=mexp;Cyc_Absyn_declare_stmt(_tmp7DC,_tmp7DB,_tmp7DA,({
struct Cyc_Absyn_Stmt*_tmp7D9=Cyc_Absyn_exp_stmt(Cyc_Absyn_copy_exp(e),0U);Cyc_Absyn_seq_stmt(_tmp7D9,
Cyc_Absyn_exp_stmt(answer,0U),0U);}),0U);});
# 2395
if((unsigned)array_len)
s=Cyc_Absyn_declare_stmt((struct _tuple1*)_check_null(lenvar),Cyc_Absyn_uint_type,array_len,s,0U);
# 2395
({void*_tmp7DD=
# 2397
Cyc_Toc_stmt_exp_r(s);
# 2395
e->r=_tmp7DD;});
# 2398
if(vse != 0)
({void*_tmp7DF=({void*_tmp7DE=Cyc_Toc_typ_to_c(old_typ);Cyc_Toc_cast_it_r(_tmp7DE,Cyc_Absyn_copy_exp(e));});e->r=_tmp7DF;});
# 2398
goto _LL7;}}}}}}}case 18U: _LL32: _tmp288=((struct Cyc_Absyn_Sizeofexp_e_Absyn_Raw_exp_struct*)_tmp25C)->f1;_LL33: {struct Cyc_Absyn_Exp*e1=_tmp288;
# 2403
Cyc_Toc_exp_to_c(nv,e1);goto _LL7;}case 17U: _LL34: _tmp287=(void*)((struct Cyc_Absyn_Sizeoftype_e_Absyn_Raw_exp_struct*)_tmp25C)->f1;_LL35: {void*t=_tmp287;
({void*_tmp7E1=(void*)({struct Cyc_Absyn_Sizeoftype_e_Absyn_Raw_exp_struct*_tmp31F=_cycalloc(sizeof(*_tmp31F));_tmp31F->tag=17U,({void*_tmp7E0=Cyc_Toc_typ_to_c(t);_tmp31F->f1=_tmp7E0;});_tmp31F;});e->r=_tmp7E1;});goto _LL7;}case 19U: _LL36: _tmp285=(void*)((struct Cyc_Absyn_Offsetof_e_Absyn_Raw_exp_struct*)_tmp25C)->f1;_tmp286=((struct Cyc_Absyn_Offsetof_e_Absyn_Raw_exp_struct*)_tmp25C)->f2;_LL37: {void*t=_tmp285;struct Cyc_List_List*fs=_tmp286;
# 2406
void*t2=t;
struct Cyc_List_List*l=fs;
for(0;l != 0;l=l->tl){
void*_stmttmp3E=(void*)l->hd;void*_tmp320=_stmttmp3E;unsigned _tmp321;struct _fat_ptr*_tmp322;if(((struct Cyc_Absyn_StructField_Absyn_OffsetofField_struct*)_tmp320)->tag == 0U){_LLD1: _tmp322=((struct Cyc_Absyn_StructField_Absyn_OffsetofField_struct*)_tmp320)->f1;_LLD2: {struct _fat_ptr*n=_tmp322;
goto _LLD0;}}else{_LLD3: _tmp321=((struct Cyc_Absyn_TupleIndex_Absyn_OffsetofField_struct*)_tmp320)->f1;_LLD4: {unsigned n=_tmp321;
# 2412
{void*_stmttmp3F=Cyc_Tcutil_compress(t2);void*_tmp323=_stmttmp3F;struct Cyc_List_List*_tmp324;struct Cyc_List_List*_tmp325;struct Cyc_Absyn_Datatypefield*_tmp326;union Cyc_Absyn_AggrInfo _tmp327;switch(*((int*)_tmp323)){case 0U: switch(*((int*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp323)->f1)){case 20U: _LLD6: _tmp327=((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp323)->f1)->f1;_LLD7: {union Cyc_Absyn_AggrInfo info=_tmp327;
# 2414
struct Cyc_Absyn_Aggrdecl*ad=Cyc_Absyn_get_known_aggrdecl(info);
if(ad->impl == 0)
({void*_tmp328=0U;({int(*_tmp7E3)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp32A)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Toc_toc_impos;_tmp32A;});struct _fat_ptr _tmp7E2=({const char*_tmp329="struct fields must be known";_tag_fat(_tmp329,sizeof(char),28U);});_tmp7E3(_tmp7E2,_tag_fat(_tmp328,sizeof(void*),0U));});});
# 2415
_tmp325=((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->fields;goto _LLD9;}case 19U: if(((((struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp323)->f1)->f1).KnownDatatypefield).tag == 2){_LLDC: _tmp326=(((((struct Cyc_Absyn_DatatypeFieldCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp323)->f1)->f1).KnownDatatypefield).val).f2;_LLDD: {struct Cyc_Absyn_Datatypefield*tuf=_tmp326;
# 2428
if(n == (unsigned)0)
({void*_tmp7E4=(void*)((void*)({struct Cyc_Absyn_StructField_Absyn_OffsetofField_struct*_tmp32F=_cycalloc(sizeof(*_tmp32F));_tmp32F->tag=0U,_tmp32F->f1=Cyc_Toc_tag_sp;_tmp32F;}));l->hd=_tmp7E4;});else{
# 2431
t2=(*({struct _tuple13*(*_tmp7E6)(struct Cyc_List_List*x,int n)=({struct _tuple13*(*_tmp330)(struct Cyc_List_List*x,int n)=(struct _tuple13*(*)(struct Cyc_List_List*x,int n))Cyc_List_nth;_tmp330;});struct Cyc_List_List*_tmp7E5=tuf->typs;_tmp7E6(_tmp7E5,(int)(n - (unsigned)1));})).f2;
({void*_tmp7E8=(void*)((void*)({struct Cyc_Absyn_StructField_Absyn_OffsetofField_struct*_tmp331=_cycalloc(sizeof(*_tmp331));_tmp331->tag=0U,({struct _fat_ptr*_tmp7E7=Cyc_Absyn_fieldname((int)n);_tmp331->f1=_tmp7E7;});_tmp331;}));l->hd=_tmp7E8;});}
# 2434
goto _LLD5;}}else{goto _LLDE;}default: goto _LLDE;}case 7U: _LLD8: _tmp325=((struct Cyc_Absyn_AnonAggrType_Absyn_Type_struct*)_tmp323)->f2;_LLD9: {struct Cyc_List_List*fields=_tmp325;
# 2419
struct Cyc_Absyn_Aggrfield*nth_field=({struct Cyc_Absyn_Aggrfield*(*_tmp7EA)(struct Cyc_List_List*x,int n)=({struct Cyc_Absyn_Aggrfield*(*_tmp32C)(struct Cyc_List_List*x,int n)=(struct Cyc_Absyn_Aggrfield*(*)(struct Cyc_List_List*x,int n))Cyc_List_nth;_tmp32C;});struct Cyc_List_List*_tmp7E9=fields;_tmp7EA(_tmp7E9,(int)n);});
({void*_tmp7EB=(void*)((void*)({struct Cyc_Absyn_StructField_Absyn_OffsetofField_struct*_tmp32B=_cycalloc(sizeof(*_tmp32B));_tmp32B->tag=0U,_tmp32B->f1=nth_field->name;_tmp32B;}));l->hd=_tmp7EB;});
t2=nth_field->type;
goto _LLD5;}case 6U: _LLDA: _tmp324=((struct Cyc_Absyn_TupleType_Absyn_Type_struct*)_tmp323)->f1;_LLDB: {struct Cyc_List_List*ts=_tmp324;
# 2424
({void*_tmp7ED=(void*)((void*)({struct Cyc_Absyn_StructField_Absyn_OffsetofField_struct*_tmp32D=_cycalloc(sizeof(*_tmp32D));_tmp32D->tag=0U,({struct _fat_ptr*_tmp7EC=Cyc_Absyn_fieldname((int)(n + (unsigned)1));_tmp32D->f1=_tmp7EC;});_tmp32D;}));l->hd=_tmp7ED;});
t2=(*({struct _tuple13*(*_tmp7EF)(struct Cyc_List_List*x,int n)=({struct _tuple13*(*_tmp32E)(struct Cyc_List_List*x,int n)=(struct _tuple13*(*)(struct Cyc_List_List*x,int n))Cyc_List_nth;_tmp32E;});struct Cyc_List_List*_tmp7EE=ts;_tmp7EF(_tmp7EE,(int)n);})).f2;
goto _LLD5;}default: _LLDE: _LLDF:
# 2435
({void*_tmp332=0U;({int(*_tmp7F1)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp334)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Toc_toc_impos;_tmp334;});struct _fat_ptr _tmp7F0=({const char*_tmp333="impossible type for offsetof tuple index";_tag_fat(_tmp333,sizeof(char),41U);});_tmp7F1(_tmp7F0,_tag_fat(_tmp332,sizeof(void*),0U));});});}_LLD5:;}
# 2437
goto _LLD0;}}_LLD0:;}
# 2440
({void*_tmp7F3=(void*)({struct Cyc_Absyn_Offsetof_e_Absyn_Raw_exp_struct*_tmp335=_cycalloc(sizeof(*_tmp335));_tmp335->tag=19U,({void*_tmp7F2=Cyc_Toc_typ_to_c(t);_tmp335->f1=_tmp7F2;}),_tmp335->f2=fs;_tmp335;});e->r=_tmp7F3;});
goto _LL7;}case 21U: _LL38: _tmp281=((struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct*)_tmp25C)->f1;_tmp282=((struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct*)_tmp25C)->f2;_tmp283=((struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct*)_tmp25C)->f3;_tmp284=((struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct*)_tmp25C)->f4;_LL39: {struct Cyc_Absyn_Exp*e1=_tmp281;struct _fat_ptr*f=_tmp282;int is_tagged=_tmp283;int is_read=_tmp284;
# 2443
int is_poly=Cyc_Toc_is_poly_project(e);
void*e1_cyc_type=(void*)_check_null(e1->topt);
Cyc_Toc_exp_to_c(nv,e1);
if(is_tagged && is_read)
({void*_tmp7F8=({struct Cyc_Absyn_Exp*_tmp7F7=e1;void*_tmp7F6=Cyc_Toc_typ_to_c(e1_cyc_type);void*_tmp7F5=e1_cyc_type;struct _fat_ptr*_tmp7F4=f;Cyc_Toc_check_tagged_union(_tmp7F7,_tmp7F6,_tmp7F5,_tmp7F4,
Cyc_Toc_in_lhs(nv),Cyc_Toc_member_exp);});
# 2447
e->r=_tmp7F8;});
# 2446
if(is_poly)
# 2450
({void*_tmp7FA=({void*_tmp7F9=Cyc_Toc_typ_to_c((void*)_check_null(e->topt));Cyc_Toc_array_to_ptr_cast(_tmp7F9,Cyc_Absyn_new_exp(e->r,0U),0U);})->r;e->r=_tmp7FA;});
# 2446
goto _LL7;}case 22U: _LL3A: _tmp27D=((struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct*)_tmp25C)->f1;_tmp27E=((struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct*)_tmp25C)->f2;_tmp27F=((struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct*)_tmp25C)->f3;_tmp280=((struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct*)_tmp25C)->f4;_LL3B: {struct Cyc_Absyn_Exp*e1=_tmp27D;struct _fat_ptr*f=_tmp27E;int is_tagged=_tmp27F;int is_read=_tmp280;
# 2453
int is_poly=Cyc_Toc_is_poly_project(e);
void*e1typ=(void*)_check_null(e1->topt);
void*ta;
{void*_stmttmp40=Cyc_Tcutil_compress(e1typ);void*_tmp336=_stmttmp40;struct Cyc_Absyn_PtrInfo _tmp337;if(((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp336)->tag == 3U){_LLE1: _tmp337=((struct Cyc_Absyn_PointerType_Absyn_Type_struct*)_tmp336)->f1;_LLE2: {struct Cyc_Absyn_PtrInfo p=_tmp337;
ta=p.elt_type;goto _LLE0;}}else{_LLE3: _LLE4:
({void*_tmp338=0U;({int(*_tmp7FC)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp33A)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Toc_toc_impos;_tmp33A;});struct _fat_ptr _tmp7FB=({const char*_tmp339="get_ptr_typ: not a pointer!";_tag_fat(_tmp339,sizeof(char),28U);});_tmp7FC(_tmp7FB,_tag_fat(_tmp338,sizeof(void*),0U));});});}_LLE0:;}
# 2460
did_inserted_checks=1;
Cyc_Toc_ptr_use_to_c(nv,e1,e->annot,0);
if(is_tagged && is_read)
({void*_tmp801=({struct Cyc_Absyn_Exp*_tmp800=e1;void*_tmp7FF=Cyc_Toc_typ_to_c(e1typ);void*_tmp7FE=ta;struct _fat_ptr*_tmp7FD=f;Cyc_Toc_check_tagged_union(_tmp800,_tmp7FF,_tmp7FE,_tmp7FD,Cyc_Toc_in_lhs(nv),Cyc_Absyn_aggrarrow_exp);});e->r=_tmp801;});
# 2462
if(
# 2464
is_poly && is_read)
({void*_tmp803=({void*_tmp802=Cyc_Toc_typ_to_c((void*)_check_null(e->topt));Cyc_Toc_array_to_ptr_cast(_tmp802,Cyc_Absyn_new_exp(e->r,0U),0U);})->r;e->r=_tmp803;});
# 2462
goto _LL7;}case 20U: _LL3C: _tmp27C=((struct Cyc_Absyn_Deref_e_Absyn_Raw_exp_struct*)_tmp25C)->f1;_LL3D: {struct Cyc_Absyn_Exp*e1=_tmp27C;
# 2468
did_inserted_checks=1;
Cyc_Toc_ptr_use_to_c(nv,e1,e->annot,0);
goto _LL7;}case 23U: _LL3E: _tmp27A=((struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct*)_tmp25C)->f1;_tmp27B=((struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct*)_tmp25C)->f2;_LL3F: {struct Cyc_Absyn_Exp*e1=_tmp27A;struct Cyc_Absyn_Exp*e2=_tmp27B;
# 2472
{void*_stmttmp41=Cyc_Tcutil_compress((void*)_check_null(e1->topt));void*_tmp33B=_stmttmp41;struct Cyc_List_List*_tmp33C;if(((struct Cyc_Absyn_TupleType_Absyn_Type_struct*)_tmp33B)->tag == 6U){_LLE6: _tmp33C=((struct Cyc_Absyn_TupleType_Absyn_Type_struct*)_tmp33B)->f1;_LLE7: {struct Cyc_List_List*ts=_tmp33C;
# 2474
Cyc_Toc_exp_to_c(nv,e1);{
int old_lhs=Cyc_Toc_in_lhs(nv);
Cyc_Toc_set_lhs(nv,0);
Cyc_Toc_exp_to_c(nv,e2);{
struct _tuple14 _stmttmp42=Cyc_Evexp_eval_const_uint_exp(e2);int _tmp33E;unsigned _tmp33D;_LLEB: _tmp33D=_stmttmp42.f1;_tmp33E=_stmttmp42.f2;_LLEC: {unsigned i=_tmp33D;int known=_tmp33E;
if(!known)
({void*_tmp33F=0U;({int(*_tmp805)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp341)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp341;});struct _fat_ptr _tmp804=({const char*_tmp340="unknown tuple subscript in translation to C";_tag_fat(_tmp340,sizeof(char),44U);});_tmp805(_tmp804,_tag_fat(_tmp33F,sizeof(void*),0U));});});
# 2479
({void*_tmp807=({
# 2481
struct Cyc_Absyn_Exp*_tmp806=e1;Cyc_Toc_aggrmember_exp_r(_tmp806,Cyc_Absyn_fieldname((int)(i + (unsigned)1)));});
# 2479
e->r=_tmp807;});
# 2482
goto _LLE5;}}}}}else{_LLE8: _LLE9:
# 2484
 did_inserted_checks=1;{
int index_used=Cyc_Toc_ptr_use_to_c(nv,e1,e->annot,e2);
if(index_used)
({void*_tmp808=Cyc_Toc_deref_exp_r(e1);e->r=_tmp808;});
# 2486
goto _LLE5;}}_LLE5:;}
# 2490
goto _LL7;}case 24U: _LL40: _tmp279=((struct Cyc_Absyn_Tuple_e_Absyn_Raw_exp_struct*)_tmp25C)->f1;_LL41: {struct Cyc_List_List*es=_tmp279;
# 2492
if(!Cyc_Toc_is_toplevel(nv)){
# 2494
void*strct_typ=Cyc_Toc_add_tuple_type(({struct Cyc_List_List*(*_tmp809)(struct _tuple13*(*f)(struct Cyc_Absyn_Exp*),struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp349)(struct _tuple13*(*f)(struct Cyc_Absyn_Exp*),struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct _tuple13*(*f)(struct Cyc_Absyn_Exp*),struct Cyc_List_List*x))Cyc_List_map;_tmp349;});_tmp809(Cyc_Toc_tup_to_c,es);}));
{void*_tmp342=strct_typ;union Cyc_Absyn_AggrInfo _tmp343;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp342)->tag == 0U){if(((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp342)->f1)->tag == 20U){if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp342)->f2 == 0){_LLEE: _tmp343=((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp342)->f1)->f1;_LLEF: {union Cyc_Absyn_AggrInfo aggrinfo=_tmp343;
# 2497
struct Cyc_List_List*dles=0;
struct Cyc_Absyn_Aggrdecl*sd=Cyc_Absyn_get_known_aggrdecl(aggrinfo);
{int i=1;for(0;es != 0;(es=es->tl,++ i)){
Cyc_Toc_exp_to_c(nv,(struct Cyc_Absyn_Exp*)es->hd);
dles=({struct Cyc_List_List*_tmp344=_cycalloc(sizeof(*_tmp344));({struct _tuple20*_tmp80B=({struct _fat_ptr*_tmp80A=Cyc_Absyn_fieldname(i);Cyc_Toc_make_field(_tmp80A,(struct Cyc_Absyn_Exp*)es->hd);});_tmp344->hd=_tmp80B;}),_tmp344->tl=dles;_tmp344;});}}
# 2503
({void*_tmp80D=(void*)({struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct*_tmp345=_cycalloc(sizeof(*_tmp345));_tmp345->tag=29U,_tmp345->f1=sd->name,_tmp345->f2=0,({struct Cyc_List_List*_tmp80C=Cyc_List_imp_rev(dles);_tmp345->f3=_tmp80C;}),_tmp345->f4=sd;_tmp345;});e->r=_tmp80D;});
e->topt=strct_typ;
goto _LLED;}}else{goto _LLF0;}}else{goto _LLF0;}}else{_LLF0: _LLF1:
({void*_tmp346=0U;({int(*_tmp80F)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp348)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Toc_toc_impos;_tmp348;});struct _fat_ptr _tmp80E=({const char*_tmp347="tuple type not an aggregate";_tag_fat(_tmp347,sizeof(char),28U);});_tmp80F(_tmp80E,_tag_fat(_tmp346,sizeof(void*),0U));});});}_LLED:;}
# 2508
goto _LL7;}else{
# 2512
struct Cyc_List_List*dles=0;
for(0;es != 0;es=es->tl){
Cyc_Toc_exp_to_c(nv,(struct Cyc_Absyn_Exp*)es->hd);
dles=({struct Cyc_List_List*_tmp34B=_cycalloc(sizeof(*_tmp34B));({struct _tuple20*_tmp810=({struct _tuple20*_tmp34A=_cycalloc(sizeof(*_tmp34A));_tmp34A->f1=0,_tmp34A->f2=(struct Cyc_Absyn_Exp*)es->hd;_tmp34A;});_tmp34B->hd=_tmp810;}),_tmp34B->tl=dles;_tmp34B;});}
# 2517
({void*_tmp811=Cyc_Toc_unresolvedmem_exp_r(0,Cyc_List_imp_rev(dles));e->r=_tmp811;});}
# 2519
goto _LL7;}case 26U: _LL42: _tmp278=((struct Cyc_Absyn_Array_e_Absyn_Raw_exp_struct*)_tmp25C)->f1;_LL43: {struct Cyc_List_List*dles0=_tmp278;
# 2525
{struct Cyc_List_List*dles=dles0;for(0;dles != 0;dles=dles->tl){
Cyc_Toc_exp_to_c(nv,(*((struct _tuple20*)dles->hd)).f2);}}
if(Cyc_Toc_is_toplevel(nv))
({void*_tmp812=Cyc_Toc_unresolvedmem_exp_r(0,dles0);e->r=_tmp812;});
# 2527
goto _LL7;}case 27U: _LL44: _tmp274=((struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct*)_tmp25C)->f1;_tmp275=((struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct*)_tmp25C)->f2;_tmp276=((struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct*)_tmp25C)->f3;_tmp277=((struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct*)_tmp25C)->f4;_LL45: {struct Cyc_Absyn_Vardecl*vd=_tmp274;struct Cyc_Absyn_Exp*e1=_tmp275;struct Cyc_Absyn_Exp*e2=_tmp276;int iszeroterm=_tmp277;
# 2533
Cyc_Toc_exp_to_c(nv,e1);
Cyc_Toc_exp_to_c(nv,e2);
if(Cyc_Toc_is_toplevel(nv)){
struct _tuple14 _stmttmp43=Cyc_Evexp_eval_const_uint_exp(e1);int _tmp34D;unsigned _tmp34C;_LLF3: _tmp34C=_stmttmp43.f1;_tmp34D=_stmttmp43.f2;_LLF4: {unsigned sz=_tmp34C;int known=_tmp34D;
void*elt_typ=Cyc_Toc_typ_to_c((void*)_check_null(e2->topt));
struct Cyc_List_List*es=0;
# 2540
if(!Cyc_Toc_is_zero(e2)){
if(!known)
({void*_tmp34E=0U;({unsigned _tmp814=e1->loc;struct _fat_ptr _tmp813=({const char*_tmp34F="cannot determine value of constant";_tag_fat(_tmp34F,sizeof(char),35U);});Cyc_Tcutil_terr(_tmp814,_tmp813,_tag_fat(_tmp34E,sizeof(void*),0U));});});
# 2541
{
# 2543
unsigned i=0U;
# 2541
for(0;i < sz;++ i){
# 2544
es=({struct Cyc_List_List*_tmp351=_cycalloc(sizeof(*_tmp351));({struct _tuple20*_tmp815=({struct _tuple20*_tmp350=_cycalloc(sizeof(*_tmp350));_tmp350->f1=0,_tmp350->f2=e2;_tmp350;});_tmp351->hd=_tmp815;}),_tmp351->tl=es;_tmp351;});}}
# 2546
if(iszeroterm){
struct Cyc_Absyn_Exp*ezero=({void*_tmp816=elt_typ;Cyc_Toc_cast_it(_tmp816,Cyc_Absyn_uint_exp(0U,0U));});
es=({struct Cyc_List_List*_tmp818=es;Cyc_List_imp_append(_tmp818,({struct Cyc_List_List*_tmp353=_cycalloc(sizeof(*_tmp353));({struct _tuple20*_tmp817=({struct _tuple20*_tmp352=_cycalloc(sizeof(*_tmp352));_tmp352->f1=0,_tmp352->f2=ezero;_tmp352;});_tmp353->hd=_tmp817;}),_tmp353->tl=0;_tmp353;}));});}}
# 2540
({void*_tmp819=
# 2551
Cyc_Toc_unresolvedmem_exp_r(0,es);
# 2540
e->r=_tmp819;});}}
# 2535
goto _LL7;}case 28U: _LL46: _tmp271=((struct Cyc_Absyn_ComprehensionNoinit_e_Absyn_Raw_exp_struct*)_tmp25C)->f1;_tmp272=(void*)((struct Cyc_Absyn_ComprehensionNoinit_e_Absyn_Raw_exp_struct*)_tmp25C)->f2;_tmp273=((struct Cyc_Absyn_ComprehensionNoinit_e_Absyn_Raw_exp_struct*)_tmp25C)->f3;_LL47: {struct Cyc_Absyn_Exp*e1=_tmp271;void*t1=_tmp272;int iszeroterm=_tmp273;
# 2560
if(Cyc_Toc_is_toplevel(nv))
({void*_tmp81A=Cyc_Toc_unresolvedmem_exp_r(0,0);e->r=_tmp81A;});else{
# 2563
Cyc_Toc_exp_to_c(nv,e1);}
goto _LL7;}case 30U: _LL48: _tmp26F=(void*)((struct Cyc_Absyn_AnonStruct_e_Absyn_Raw_exp_struct*)_tmp25C)->f1;_tmp270=((struct Cyc_Absyn_AnonStruct_e_Absyn_Raw_exp_struct*)_tmp25C)->f2;_LL49: {void*st=_tmp26F;struct Cyc_List_List*dles=_tmp270;
# 2567
{struct Cyc_List_List*dles2=dles;for(0;dles2 != 0;dles2=dles2->tl){
Cyc_Toc_exp_to_c(nv,(*((struct _tuple20*)dles2->hd)).f2);}}{
void*strct_typ=Cyc_Toc_typ_to_c((void*)_check_null(e->topt));
e->topt=strct_typ;
if(!Cyc_Toc_is_toplevel(nv)){
void*_stmttmp44=Cyc_Tcutil_compress(strct_typ);void*_tmp354=_stmttmp44;union Cyc_Absyn_AggrInfo _tmp355;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp354)->tag == 0U){if(((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp354)->f1)->tag == 20U){if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp354)->f2 == 0){_LLF6: _tmp355=((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp354)->f1)->f1;_LLF7: {union Cyc_Absyn_AggrInfo aggrinfo=_tmp355;
# 2574
struct Cyc_Absyn_Aggrdecl*sd=Cyc_Absyn_get_known_aggrdecl(aggrinfo);
({void*_tmp81B=(void*)({struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct*_tmp356=_cycalloc(sizeof(*_tmp356));_tmp356->tag=29U,_tmp356->f1=sd->name,_tmp356->f2=0,_tmp356->f3=dles,_tmp356->f4=sd;_tmp356;});e->r=_tmp81B;});
e->topt=strct_typ;
goto _LLF5;}}else{goto _LLF8;}}else{goto _LLF8;}}else{_LLF8: _LLF9:
({void*_tmp357=0U;({int(*_tmp81D)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp359)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Toc_toc_impos;_tmp359;});struct _fat_ptr _tmp81C=({const char*_tmp358="anonStruct type not an aggregate";_tag_fat(_tmp358,sizeof(char),33U);});_tmp81D(_tmp81C,_tag_fat(_tmp357,sizeof(void*),0U));});});}_LLF5:;}else{
# 2581
({void*_tmp81E=Cyc_Toc_unresolvedmem_exp_r(0,dles);e->r=_tmp81E;});}
goto _LL7;}}case 29U: _LL4A: _tmp26B=(struct _tuple1**)&((struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct*)_tmp25C)->f1;_tmp26C=((struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct*)_tmp25C)->f2;_tmp26D=((struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct*)_tmp25C)->f3;_tmp26E=((struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct*)_tmp25C)->f4;_LL4B: {struct _tuple1**tdn=_tmp26B;struct Cyc_List_List*exist_ts=_tmp26C;struct Cyc_List_List*dles=_tmp26D;struct Cyc_Absyn_Aggrdecl*sd=_tmp26E;
# 2587
if(sd == 0 || sd->impl == 0)
({void*_tmp35A=0U;({int(*_tmp820)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp35C)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Toc_toc_impos;_tmp35C;});struct _fat_ptr _tmp81F=({const char*_tmp35B="exp_to_c, Aggregate_e: missing aggrdecl pointer or fields";_tag_fat(_tmp35B,sizeof(char),58U);});_tmp820(_tmp81F,_tag_fat(_tmp35A,sizeof(void*),0U));});});{
# 2587
void*new_typ=
# 2589
Cyc_Toc_typ_to_c(old_typ);
{void*_stmttmp45=Cyc_Tcutil_compress(new_typ);void*_tmp35D=_stmttmp45;union Cyc_Absyn_AggrInfo _tmp35E;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp35D)->tag == 0U){if(((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp35D)->f1)->tag == 20U){_LLFB: _tmp35E=((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp35D)->f1)->f1;_LLFC: {union Cyc_Absyn_AggrInfo info=_tmp35E;
({struct _tuple1*_tmp821=(Cyc_Absyn_aggr_kinded_name(info)).f2;*tdn=_tmp821;});goto _LLFA;}}else{goto _LLFD;}}else{_LLFD: _LLFE:
({void*_tmp35F=0U;({int(*_tmp823)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp361)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Toc_toc_impos;_tmp361;});struct _fat_ptr _tmp822=({const char*_tmp360="exp_to_c, Aggregate_e: bad type translation";_tag_fat(_tmp360,sizeof(char),44U);});_tmp823(_tmp822,_tag_fat(_tmp35F,sizeof(void*),0U));});});}_LLFA:;}{
# 2598
struct Cyc_List_List*typ_fields=((struct Cyc_Absyn_AggrdeclImpl*)_check_null(sd->impl))->fields;
if(typ_fields == 0)goto _LL7;for(0;((struct Cyc_List_List*)_check_null(typ_fields))->tl != 0;typ_fields=typ_fields->tl){
# 2601
;}{
struct Cyc_Absyn_Aggrfield*last_typ_field=(struct Cyc_Absyn_Aggrfield*)typ_fields->hd;
struct Cyc_List_List*fields=Cyc_Tcutil_resolve_aggregate_designators(Cyc_Core_heap_region,e->loc,dles,sd->kind,((struct Cyc_Absyn_AggrdeclImpl*)_check_null(sd->impl))->fields);
# 2605
if(!Cyc_Toc_is_toplevel(nv)){
void*_stmttmp46=Cyc_Tcutil_compress(old_typ);void*_tmp362=_stmttmp46;struct Cyc_List_List*_tmp363;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp362)->tag == 0U){if(((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp362)->f1)->tag == 20U){_LL100: _tmp363=((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp362)->f2;_LL101: {struct Cyc_List_List*param_ts=_tmp363;
# 2609
{struct Cyc_List_List*fields2=fields;for(0;fields2 != 0;fields2=fields2->tl){
struct _tuple35*_stmttmp47=(struct _tuple35*)fields2->hd;struct Cyc_Absyn_Exp*_tmp365;struct Cyc_Absyn_Aggrfield*_tmp364;_LL105: _tmp364=_stmttmp47->f1;_tmp365=_stmttmp47->f2;_LL106: {struct Cyc_Absyn_Aggrfield*aggrfield=_tmp364;struct Cyc_Absyn_Exp*fieldexp=_tmp365;
void*old_field_typ=fieldexp->topt;
Cyc_Toc_exp_to_c(nv,fieldexp);
# 2614
if(Cyc_Toc_is_void_star_or_boxed_tvar(aggrfield->type)&& !
Cyc_Toc_is_pointer_or_boxed_tvar((void*)_check_null(fieldexp->topt)))
({void*_tmp825=({void*_tmp824=Cyc_Toc_typ_to_c(aggrfield->type);Cyc_Toc_cast_it(_tmp824,
Cyc_Absyn_copy_exp(fieldexp));})->r;
# 2616
fieldexp->r=_tmp825;});
# 2614
if(((struct Cyc_Absyn_AggrdeclImpl*)_check_null(sd->impl))->tagged){
# 2620
struct _fat_ptr*n=aggrfield->name;
struct Cyc_Absyn_Exp*tag_exp=Cyc_Absyn_uint_exp((unsigned)Cyc_Toc_get_member_offset(sd,n),0U);
struct _tuple20*tag_dle=Cyc_Toc_make_field(Cyc_Toc_tag_sp,tag_exp);
struct _tuple20*val_dle=Cyc_Toc_make_field(Cyc_Toc_val_sp,fieldexp);
struct _tuple1*s=({struct _tuple1*_tmp36F=_cycalloc(sizeof(*_tmp36F));({union Cyc_Absyn_Nmspace _tmp829=Cyc_Absyn_Abs_n(0,1);_tmp36F->f1=_tmp829;}),({
struct _fat_ptr*_tmp828=({struct _fat_ptr*_tmp36E=_cycalloc(sizeof(*_tmp36E));({struct _fat_ptr _tmp827=(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp36C=({struct Cyc_String_pa_PrintArg_struct _tmp5FD;_tmp5FD.tag=0U,_tmp5FD.f1=(struct _fat_ptr)((struct _fat_ptr)*(*sd->name).f2);_tmp5FD;});struct Cyc_String_pa_PrintArg_struct _tmp36D=({struct Cyc_String_pa_PrintArg_struct _tmp5FC;_tmp5FC.tag=0U,_tmp5FC.f1=(struct _fat_ptr)((struct _fat_ptr)*n);_tmp5FC;});void*_tmp36A[2U];_tmp36A[0]=& _tmp36C,_tmp36A[1]=& _tmp36D;({struct _fat_ptr _tmp826=({const char*_tmp36B="_union_%s_%s";_tag_fat(_tmp36B,sizeof(char),13U);});Cyc_aprintf(_tmp826,_tag_fat(_tmp36A,sizeof(void*),2U));});});*_tmp36E=_tmp827;});_tmp36E;});_tmp36F->f2=_tmp828;});_tmp36F;});
# 2627
struct _tuple20*u_dle=({
struct _fat_ptr*_tmp82B=n;Cyc_Toc_make_field(_tmp82B,Cyc_Absyn_new_exp((void*)({struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct*_tmp369=_cycalloc(sizeof(*_tmp369));_tmp369->tag=29U,_tmp369->f1=s,_tmp369->f2=0,({struct Cyc_List_List*_tmp82A=({struct _tuple20*_tmp368[2U];_tmp368[0]=tag_dle,_tmp368[1]=val_dle;Cyc_List_list(_tag_fat(_tmp368,sizeof(struct _tuple20*),2U));});_tmp369->f3=_tmp82A;}),_tmp369->f4=0;_tmp369;}),0U));});
# 2630
({void*_tmp82D=(void*)({struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct*_tmp367=_cycalloc(sizeof(*_tmp367));_tmp367->tag=29U,_tmp367->f1=*tdn,_tmp367->f2=0,({struct Cyc_List_List*_tmp82C=({struct _tuple20*_tmp366[1U];_tmp366[0]=u_dle;Cyc_List_list(_tag_fat(_tmp366,sizeof(struct _tuple20*),1U));});_tmp367->f3=_tmp82C;}),_tmp367->f4=sd;_tmp367;});e->r=_tmp82D;});}
# 2614
if(
# 2636
Cyc_Toc_is_abstract_type(old_typ)&& last_typ_field == aggrfield){
struct Cyc_List_List*inst=({struct Cyc_List_List*_tmp82E=Cyc_List_zip(((struct Cyc_Absyn_AggrdeclImpl*)_check_null(sd->impl))->exist_vars,exist_ts);Cyc_List_append(_tmp82E,
Cyc_List_zip(sd->tvs,param_ts));});
struct Cyc_List_List*new_fields=0;
{struct Cyc_List_List*fs=((struct Cyc_Absyn_AggrdeclImpl*)_check_null(sd->impl))->fields;for(0;fs != 0;fs=fs->tl){
struct Cyc_Absyn_Aggrfield*old_f=(struct Cyc_Absyn_Aggrfield*)fs->hd;
void*old_ftyp=Cyc_Tcutil_substitute(inst,old_f->type);
struct Cyc_Absyn_Aggrfield*new_f=({struct Cyc_Absyn_Aggrfield*_tmp82F=old_f;Cyc_Toc_aggrfield_to_c(_tmp82F,
Cyc_Toc_typ_to_c(Cyc_Tcutil_substitute(inst,old_ftyp)));});
new_fields=({struct Cyc_List_List*_tmp370=_cycalloc(sizeof(*_tmp370));_tmp370->hd=new_f,_tmp370->tl=new_fields;_tmp370;});
# 2651
if(fs->tl == 0){
{void*_stmttmp48=Cyc_Tcutil_compress(new_f->type);void*_tmp371=_stmttmp48;struct Cyc_Absyn_ArrayInfo _tmp372;if(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp371)->tag == 4U){_LL108: _tmp372=((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp371)->f1;_LL109: {struct Cyc_Absyn_ArrayInfo ai=_tmp372;
# 2654
if(!Cyc_Evexp_c_can_eval((struct Cyc_Absyn_Exp*)_check_null(ai.num_elts))){
struct Cyc_Absyn_ArrayInfo ai2=ai;
({struct Cyc_Absyn_Exp*_tmp830=Cyc_Absyn_uint_exp(0U,0U);ai2.num_elts=_tmp830;});
({void*_tmp831=(void*)({struct Cyc_Absyn_ArrayType_Absyn_Type_struct*_tmp373=_cycalloc(sizeof(*_tmp373));_tmp373->tag=4U,_tmp373->f1=ai2;_tmp373;});new_f->type=_tmp831;});}
# 2654
goto _LL107;}}else{_LL10A: _LL10B:
# 2663
 if(fieldexp->topt == 0)
goto _LL107;
# 2663
{void*_stmttmp49=
# 2665
Cyc_Tcutil_compress((void*)_check_null(fieldexp->topt));
# 2663
void*_tmp374=_stmttmp49;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp374)->tag == 0U){if(((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp374)->f1)->tag == 20U){_LL10D: _LL10E:
# 2667
 new_f->type=(void*)_check_null(fieldexp->topt);goto _LL10C;}else{goto _LL10F;}}else{_LL10F: _LL110:
 goto _LL10C;}_LL10C:;}
# 2670
goto _LL107;}_LL107:;}
# 2674
if(!Cyc_Tcutil_is_array_type(old_f->type)&&
 Cyc_Tcutil_kind_leq(& Cyc_Tcutil_ak,Cyc_Tcutil_type_kind(old_f->type)))
({struct Cyc_List_List*_tmp833=({struct Cyc_List_List*_tmp376=_cycalloc(sizeof(*_tmp376));({void*_tmp832=(void*)({struct Cyc_Absyn_Aligned_att_Absyn_Attribute_struct*_tmp375=_cycalloc(sizeof(*_tmp375));_tmp375->tag=6U,_tmp375->f1=0;_tmp375;});_tmp376->hd=_tmp832;}),_tmp376->tl=new_f->attributes;_tmp376;});new_f->attributes=_tmp833;});}}}
# 2680
sd=({struct _fat_ptr*_tmp836=({struct _fat_ptr*_tmp37A=_cycalloc(sizeof(*_tmp37A));({struct _fat_ptr _tmp835=(struct _fat_ptr)({struct Cyc_Int_pa_PrintArg_struct _tmp379=({struct Cyc_Int_pa_PrintArg_struct _tmp5FE;_tmp5FE.tag=1U,_tmp5FE.f1=(unsigned long)Cyc_Toc_tuple_type_counter ++;_tmp5FE;});void*_tmp377[1U];_tmp377[0]=& _tmp379;({struct _fat_ptr _tmp834=({const char*_tmp378="_genStruct%d";_tag_fat(_tmp378,sizeof(char),13U);});Cyc_aprintf(_tmp834,_tag_fat(_tmp377,sizeof(void*),1U));});});*_tmp37A=_tmp835;});_tmp37A;});Cyc_Toc_make_c_struct_defn(_tmp836,
# 2682
Cyc_List_imp_rev(new_fields));});
*tdn=sd->name;
Cyc_Toc_aggrdecl_to_c(sd);
# 2686
({void*_tmp837=Cyc_Absyn_aggr_type(Cyc_Absyn_KnownAggr(({struct Cyc_Absyn_Aggrdecl**_tmp37B=_cycalloc(sizeof(*_tmp37B));*_tmp37B=sd;_tmp37B;})),0);e->topt=_tmp837;});}}}}
# 2689
goto _LLFF;}}else{goto _LL102;}}else{_LL102: _LL103:
({void*_tmp37C=0U;({int(*_tmp839)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp37E)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Toc_toc_impos;_tmp37E;});struct _fat_ptr _tmp838=({const char*_tmp37D="exp_to_c, Aggregate_e: bad struct type";_tag_fat(_tmp37D,sizeof(char),39U);});_tmp839(_tmp838,_tag_fat(_tmp37C,sizeof(void*),0U));});});}_LLFF:;}else{
# 2698
if(((struct Cyc_Absyn_AggrdeclImpl*)_check_null(sd->impl))->tagged){
# 2700
struct _tuple35*_stmttmp4A=(struct _tuple35*)((struct Cyc_List_List*)_check_null(fields))->hd;struct Cyc_Absyn_Exp*_tmp380;struct Cyc_Absyn_Aggrfield*_tmp37F;_LL112: _tmp37F=_stmttmp4A->f1;_tmp380=_stmttmp4A->f2;_LL113: {struct Cyc_Absyn_Aggrfield*field=_tmp37F;struct Cyc_Absyn_Exp*fieldexp=_tmp380;
void*fieldexp_type=(void*)_check_null(fieldexp->topt);
void*fieldtyp=field->type;
Cyc_Toc_exp_to_c(nv,fieldexp);
if(Cyc_Toc_is_void_star_or_boxed_tvar(fieldtyp)&& !
Cyc_Toc_is_void_star_or_boxed_tvar(fieldexp_type))
({void*_tmp83B=({
void*_tmp83A=Cyc_Toc_void_star_type();Cyc_Toc_cast_it_r(_tmp83A,Cyc_Absyn_new_exp(fieldexp->r,0U));});
# 2706
fieldexp->r=_tmp83B;});{
# 2704
int i=
# 2709
Cyc_Toc_get_member_offset(sd,field->name);
struct Cyc_Absyn_Exp*field_tag_exp=Cyc_Absyn_signed_int_exp(i,0U);
struct Cyc_List_List*newdles=({struct _tuple20*_tmp382[2U];({struct _tuple20*_tmp83D=({struct _tuple20*_tmp383=_cycalloc(sizeof(*_tmp383));_tmp383->f1=0,_tmp383->f2=field_tag_exp;_tmp383;});_tmp382[0]=_tmp83D;}),({struct _tuple20*_tmp83C=({struct _tuple20*_tmp384=_cycalloc(sizeof(*_tmp384));_tmp384->f1=0,_tmp384->f2=fieldexp;_tmp384;});_tmp382[1]=_tmp83C;});Cyc_List_list(_tag_fat(_tmp382,sizeof(struct _tuple20*),2U));});
struct Cyc_Absyn_Exp*umem=Cyc_Absyn_unresolvedmem_exp(0,newdles,0U);
({void*_tmp83F=Cyc_Toc_unresolvedmem_exp_r(0,({struct _tuple20*_tmp381[1U];({struct _tuple20*_tmp83E=Cyc_Toc_make_field(field->name,umem);_tmp381[0]=_tmp83E;});Cyc_List_list(_tag_fat(_tmp381,sizeof(struct _tuple20*),1U));}));e->r=_tmp83F;});}}}else{
# 2716
struct Cyc_List_List*newdles=0;
struct Cyc_List_List*sdfields=((struct Cyc_Absyn_AggrdeclImpl*)_check_null(sd->impl))->fields;
for(0;sdfields != 0;sdfields=sdfields->tl){
struct Cyc_List_List*fields2=fields;for(0;fields2 != 0;fields2=fields2->tl){
if((*((struct _tuple35*)fields2->hd)).f1 == (struct Cyc_Absyn_Aggrfield*)sdfields->hd){
struct _tuple35*_stmttmp4B=(struct _tuple35*)fields2->hd;struct Cyc_Absyn_Exp*_tmp386;struct Cyc_Absyn_Aggrfield*_tmp385;_LL115: _tmp385=_stmttmp4B->f1;_tmp386=_stmttmp4B->f2;_LL116: {struct Cyc_Absyn_Aggrfield*field=_tmp385;struct Cyc_Absyn_Exp*fieldexp=_tmp386;
void*fieldtyp=Cyc_Toc_typ_to_c(field->type);
void*fieldexp_typ=Cyc_Toc_typ_to_c((void*)_check_null(fieldexp->topt));
Cyc_Toc_exp_to_c(nv,fieldexp);
# 2726
if(!Cyc_Unify_unify(fieldtyp,fieldexp_typ)){
# 2728
if(!Cyc_Tcutil_is_arithmetic_type(fieldtyp)|| !
Cyc_Tcutil_is_arithmetic_type(fieldexp_typ))
fieldexp=({void*_tmp840=fieldtyp;Cyc_Toc_cast_it(_tmp840,Cyc_Absyn_copy_exp(fieldexp));});}
# 2726
newdles=({struct Cyc_List_List*_tmp388=_cycalloc(sizeof(*_tmp388));
# 2731
({struct _tuple20*_tmp841=({struct _tuple20*_tmp387=_cycalloc(sizeof(*_tmp387));_tmp387->f1=0,_tmp387->f2=fieldexp;_tmp387;});_tmp388->hd=_tmp841;}),_tmp388->tl=newdles;_tmp388;});
break;}}}}
# 2735
({void*_tmp842=Cyc_Toc_unresolvedmem_exp_r(0,Cyc_List_imp_rev(newdles));e->r=_tmp842;});}}
# 2738
goto _LL7;}}}}case 31U: _LL4C: _tmp268=((struct Cyc_Absyn_Datatype_e_Absyn_Raw_exp_struct*)_tmp25C)->f1;_tmp269=((struct Cyc_Absyn_Datatype_e_Absyn_Raw_exp_struct*)_tmp25C)->f2;_tmp26A=((struct Cyc_Absyn_Datatype_e_Absyn_Raw_exp_struct*)_tmp25C)->f3;_LL4D: {struct Cyc_List_List*es=_tmp268;struct Cyc_Absyn_Datatypedecl*tud=_tmp269;struct Cyc_Absyn_Datatypefield*tuf=_tmp26A;
# 2742
struct Cyc_List_List*tqts=tuf->typs;
{struct Cyc_List_List*es2=es;for(0;es2 != 0;(es2=es2->tl,tqts=tqts->tl)){
struct Cyc_Absyn_Exp*cur_e=(struct Cyc_Absyn_Exp*)es2->hd;
void*cur_e_typ=(void*)_check_null(cur_e->topt);
void*field_typ=Cyc_Toc_typ_to_c((*((struct _tuple13*)((struct Cyc_List_List*)_check_null(tqts))->hd)).f2);
Cyc_Toc_exp_to_c(nv,cur_e);
if(Cyc_Toc_is_void_star_or_boxed_tvar(field_typ)&& !
Cyc_Toc_is_pointer_or_boxed_tvar(cur_e_typ))
({void*_tmp843=(Cyc_Toc_cast_it(field_typ,cur_e))->r;cur_e->r=_tmp843;});}}{
# 2753
struct Cyc_Absyn_Exp*tag_exp=
tud->is_extensible?Cyc_Absyn_var_exp(tuf->name,0U): Cyc_Toc_datatype_tag(tud,tuf->name);
# 2756
if(!Cyc_Toc_is_toplevel(nv)){
# 2758
struct Cyc_List_List*dles=0;
{int i=1;for(0;es != 0;(es=es->tl,++ i)){
dles=({struct Cyc_List_List*_tmp389=_cycalloc(sizeof(*_tmp389));({struct _tuple20*_tmp845=({struct _fat_ptr*_tmp844=Cyc_Absyn_fieldname(i);Cyc_Toc_make_field(_tmp844,(struct Cyc_Absyn_Exp*)es->hd);});_tmp389->hd=_tmp845;}),_tmp389->tl=dles;_tmp389;});}}{
# 2764
struct _tuple20*tag_dle=Cyc_Toc_make_field(Cyc_Toc_tag_sp,tag_exp);
({void*_tmp849=(void*)({struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct*_tmp38B=_cycalloc(sizeof(*_tmp38B));_tmp38B->tag=29U,({struct _tuple1*_tmp848=Cyc_Toc_collapse_qvars(tuf->name,tud->name);_tmp38B->f1=_tmp848;}),_tmp38B->f2=0,({
struct Cyc_List_List*_tmp847=({struct Cyc_List_List*_tmp38A=_cycalloc(sizeof(*_tmp38A));_tmp38A->hd=tag_dle,({struct Cyc_List_List*_tmp846=Cyc_List_imp_rev(dles);_tmp38A->tl=_tmp846;});_tmp38A;});_tmp38B->f3=_tmp847;}),_tmp38B->f4=0;_tmp38B;});
# 2765
e->r=_tmp849;});}}else{
# 2770
struct Cyc_List_List*dles=0;
for(0;es != 0;es=es->tl){
dles=({struct Cyc_List_List*_tmp38D=_cycalloc(sizeof(*_tmp38D));({struct _tuple20*_tmp84A=({struct _tuple20*_tmp38C=_cycalloc(sizeof(*_tmp38C));_tmp38C->f1=0,_tmp38C->f2=(struct Cyc_Absyn_Exp*)es->hd;_tmp38C;});_tmp38D->hd=_tmp84A;}),_tmp38D->tl=dles;_tmp38D;});}
({void*_tmp84D=Cyc_Toc_unresolvedmem_exp_r(0,({struct Cyc_List_List*_tmp38F=_cycalloc(sizeof(*_tmp38F));({struct _tuple20*_tmp84C=({struct _tuple20*_tmp38E=_cycalloc(sizeof(*_tmp38E));_tmp38E->f1=0,_tmp38E->f2=tag_exp;_tmp38E;});_tmp38F->hd=_tmp84C;}),({
struct Cyc_List_List*_tmp84B=Cyc_List_imp_rev(dles);_tmp38F->tl=_tmp84B;});_tmp38F;}));
# 2773
e->r=_tmp84D;});}
# 2776
goto _LL7;}}case 32U: _LL4E: _LL4F:
# 2778
 goto _LL51;case 33U: _LL50: _LL51:
 goto _LL7;case 35U: _LL52: _tmp262=(((struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct*)_tmp25C)->f1).is_calloc;_tmp263=(((struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct*)_tmp25C)->f1).rgn;_tmp264=(((struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct*)_tmp25C)->f1).elt_type;_tmp265=(((struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct*)_tmp25C)->f1).num_elts;_tmp266=(((struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct*)_tmp25C)->f1).fat_result;_tmp267=(((struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct*)_tmp25C)->f1).inline_call;_LL53: {int is_calloc=_tmp262;struct Cyc_Absyn_Exp*rgnopt=_tmp263;void**topt=_tmp264;struct Cyc_Absyn_Exp*num_elts=_tmp265;int is_fat=_tmp266;int inline_call=_tmp267;
# 2782
void*t_c=Cyc_Toc_typ_to_c(*((void**)_check_null(topt)));
Cyc_Toc_exp_to_c(nv,num_elts);{
struct _tuple1*x=0;
struct Cyc_Absyn_Exp*pexp;
struct Cyc_Absyn_Exp*num_elts2=num_elts;
if(is_fat){
x=Cyc_Toc_temp_var();
num_elts2=Cyc_Absyn_var_exp(x,0U);}
# 2787
if(is_calloc){
# 2792
if(rgnopt != 0 && !Cyc_Absyn_no_regions){
Cyc_Toc_exp_to_c(nv,rgnopt);
pexp=({struct Cyc_Absyn_Exp*_tmp390[3U];_tmp390[0]=rgnopt,({
struct Cyc_Absyn_Exp*_tmp84E=Cyc_Absyn_sizeoftype_exp(t_c,0U);_tmp390[1]=_tmp84E;}),_tmp390[2]=num_elts2;({struct Cyc_Absyn_Exp*_tmp84F=Cyc_Toc__region_calloc_e;Cyc_Toc_fncall_exp_dl(_tmp84F,_tag_fat(_tmp390,sizeof(struct Cyc_Absyn_Exp*),3U));});});}else{
# 2797
pexp=({void*_tmp851=*topt;struct Cyc_Absyn_Exp*_tmp850=Cyc_Absyn_sizeoftype_exp(t_c,0U);Cyc_Toc_calloc_exp(_tmp851,_tmp850,num_elts2);});}}else{
# 2799
if(rgnopt != 0 && !Cyc_Absyn_no_regions){
Cyc_Toc_exp_to_c(nv,rgnopt);
if(inline_call)
pexp=Cyc_Toc_rmalloc_inline_exp(rgnopt,num_elts2);else{
# 2804
pexp=Cyc_Toc_rmalloc_exp(rgnopt,num_elts2);}}else{
# 2806
pexp=Cyc_Toc_malloc_exp(*topt,num_elts2);}}
# 2808
if(is_fat){
struct Cyc_Absyn_Exp*elt_sz=is_calloc?Cyc_Absyn_sizeoftype_exp(t_c,0U): Cyc_Absyn_uint_exp(1U,0U);
struct Cyc_Absyn_Exp*rexp=({struct Cyc_Absyn_Exp*_tmp391[3U];_tmp391[0]=pexp,_tmp391[1]=elt_sz,_tmp391[2]=num_elts2;({struct Cyc_Absyn_Exp*_tmp852=Cyc_Toc__tag_fat_e;Cyc_Toc_fncall_exp_dl(_tmp852,_tag_fat(_tmp391,sizeof(struct Cyc_Absyn_Exp*),3U));});});
struct Cyc_Absyn_Stmt*s=({struct _tuple1*_tmp855=(struct _tuple1*)_check_null(x);void*_tmp854=Cyc_Absyn_uint_type;struct Cyc_Absyn_Exp*_tmp853=num_elts;Cyc_Absyn_declare_stmt(_tmp855,_tmp854,_tmp853,
Cyc_Absyn_exp_stmt(rexp,0U),0U);});
({void*_tmp856=Cyc_Toc_stmt_exp_r(s);e->r=_tmp856;});}else{
# 2815
e->r=pexp->r;}
goto _LL7;}}case 36U: _LL54: _tmp260=((struct Cyc_Absyn_Swap_e_Absyn_Raw_exp_struct*)_tmp25C)->f1;_tmp261=((struct Cyc_Absyn_Swap_e_Absyn_Raw_exp_struct*)_tmp25C)->f2;_LL55: {struct Cyc_Absyn_Exp*e1=_tmp260;struct Cyc_Absyn_Exp*e2=_tmp261;
# 2825
void*e1_old_typ=(void*)_check_null(e1->topt);
void*e2_old_typ=(void*)_check_null(e2->topt);
if(!Cyc_Tcutil_is_boxed(e1_old_typ)&& !Cyc_Tcutil_is_pointer_type(e1_old_typ))
({void*_tmp392=0U;({int(*_tmp858)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp394)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Toc_toc_impos;_tmp394;});struct _fat_ptr _tmp857=({const char*_tmp393="Swap_e: is_pointer_or_boxed: not a pointer or boxed type";_tag_fat(_tmp393,sizeof(char),57U);});_tmp858(_tmp857,_tag_fat(_tmp392,sizeof(void*),0U));});});{
# 2827
unsigned loc=e->loc;
# 2834
struct _tuple1*v1=Cyc_Toc_temp_var();
struct Cyc_Absyn_Exp*v1e=Cyc_Absyn_var_exp(v1,loc);v1e->topt=e1_old_typ;{
struct _tuple1*v2=Cyc_Toc_temp_var();
struct Cyc_Absyn_Exp*v2e=Cyc_Absyn_var_exp(v2,loc);v2e->topt=e2_old_typ;{
# 2839
struct Cyc_Absyn_Exp*s1e=({struct Cyc_Absyn_Exp*_tmp85A=Cyc_Tcutil_deep_copy_exp(1,e1);struct Cyc_Absyn_Exp*_tmp859=v2e;Cyc_Absyn_assign_exp(_tmp85A,_tmp859,loc);});s1e->topt=e1_old_typ;{
struct Cyc_Absyn_Stmt*s1=Cyc_Absyn_exp_stmt(s1e,loc);
struct Cyc_Absyn_Exp*s2e=({struct Cyc_Absyn_Exp*_tmp85C=Cyc_Tcutil_deep_copy_exp(1,e2);struct Cyc_Absyn_Exp*_tmp85B=v1e;Cyc_Absyn_assign_exp(_tmp85C,_tmp85B,loc);});s2e->topt=e2_old_typ;{
struct Cyc_Absyn_Stmt*s2=Cyc_Absyn_exp_stmt(s2e,loc);
# 2844
struct Cyc_Absyn_Stmt*s=({struct _tuple1*_tmp864=v1;void*_tmp863=e1_old_typ;struct Cyc_Absyn_Exp*_tmp862=e1;struct Cyc_Absyn_Stmt*_tmp861=({
struct _tuple1*_tmp860=v2;void*_tmp85F=e2_old_typ;struct Cyc_Absyn_Exp*_tmp85E=e2;struct Cyc_Absyn_Stmt*_tmp85D=
Cyc_Absyn_seq_stmt(s1,s2,loc);
# 2845
Cyc_Absyn_declare_stmt(_tmp860,_tmp85F,_tmp85E,_tmp85D,loc);});
# 2844
Cyc_Absyn_declare_stmt(_tmp864,_tmp863,_tmp862,_tmp861,loc);});
# 2847
Cyc_Toc_stmt_to_c(nv,s);
({void*_tmp865=Cyc_Toc_stmt_exp_r(s);e->r=_tmp865;});
goto _LL7;}}}}}}case 39U: _LL56: _tmp25E=((struct Cyc_Absyn_Tagcheck_e_Absyn_Raw_exp_struct*)_tmp25C)->f1;_tmp25F=((struct Cyc_Absyn_Tagcheck_e_Absyn_Raw_exp_struct*)_tmp25C)->f2;_LL57: {struct Cyc_Absyn_Exp*e1=_tmp25E;struct _fat_ptr*f=_tmp25F;
# 2852
void*e1_typ=Cyc_Tcutil_compress((void*)_check_null(e1->topt));
Cyc_Toc_exp_to_c(nv,e1);
{void*_tmp395=e1_typ;struct Cyc_Absyn_Aggrdecl*_tmp396;if(((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp395)->tag == 0U){if(((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp395)->f1)->tag == 20U){if(((((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp395)->f1)->f1).KnownAggr).tag == 2){_LL118: _tmp396=*((((struct Cyc_Absyn_AggrCon_Absyn_TyCon_struct*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp395)->f1)->f1).KnownAggr).val;_LL119: {struct Cyc_Absyn_Aggrdecl*ad=_tmp396;
# 2856
struct Cyc_Absyn_Exp*f_tag=Cyc_Absyn_signed_int_exp(Cyc_Toc_get_member_offset(ad,f),0U);
struct Cyc_Absyn_Exp*e1_f=Cyc_Toc_member_exp(e1,f,0U);
struct Cyc_Absyn_Exp*e1_f_tag=Cyc_Toc_member_exp(e1_f,Cyc_Toc_tag_sp,0U);
({void*_tmp866=(Cyc_Absyn_eq_exp(e1_f_tag,f_tag,0U))->r;e->r=_tmp866;});
goto _LL117;}}else{goto _LL11A;}}else{goto _LL11A;}}else{_LL11A: _LL11B:
({void*_tmp397=0U;({int(*_tmp868)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp399)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Toc_toc_impos;_tmp399;});struct _fat_ptr _tmp867=({const char*_tmp398="non-aggregate type in tagcheck";_tag_fat(_tmp398,sizeof(char),31U);});_tmp868(_tmp867,_tag_fat(_tmp397,sizeof(void*),0U));});});}_LL117:;}
# 2863
goto _LL7;}case 38U: _LL58: _tmp25D=((struct Cyc_Absyn_StmtExp_e_Absyn_Raw_exp_struct*)_tmp25C)->f1;_LL59: {struct Cyc_Absyn_Stmt*s=_tmp25D;
Cyc_Toc_stmt_to_c(nv,s);goto _LL7;}case 41U: _LL5A: _LL5B:
 goto _LL7;case 37U: _LL5C: _LL5D:
({void*_tmp39A=0U;({int(*_tmp86A)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp39C)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Toc_toc_impos;_tmp39C;});struct _fat_ptr _tmp869=({const char*_tmp39B="UnresolvedMem";_tag_fat(_tmp39B,sizeof(char),14U);});_tmp86A(_tmp869,_tag_fat(_tmp39A,sizeof(void*),0U));});});case 25U: _LL5E: _LL5F:
({void*_tmp39D=0U;({int(*_tmp86C)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp39F)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Toc_toc_impos;_tmp39F;});struct _fat_ptr _tmp86B=({const char*_tmp39E="compoundlit";_tag_fat(_tmp39E,sizeof(char),12U);});_tmp86C(_tmp86B,_tag_fat(_tmp39D,sizeof(void*),0U));});});default: _LL60: _LL61:
({void*_tmp3A0=0U;({int(*_tmp86E)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp3A2)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Toc_toc_impos;_tmp3A2;});struct _fat_ptr _tmp86D=({const char*_tmp3A1="valueof(-)";_tag_fat(_tmp3A1,sizeof(char),11U);});_tmp86E(_tmp86D,_tag_fat(_tmp3A0,sizeof(void*),0U));});});}_LL7:;}
# 2870
if(!did_inserted_checks)
({void*_tmp3A3=0U;({int(*_tmp873)(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp3A7)(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos_loc;_tmp3A7;});unsigned _tmp872=e->loc;struct _fat_ptr _tmp871=(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp3A6=({struct Cyc_String_pa_PrintArg_struct _tmp5FF;_tmp5FF.tag=0U,({
struct _fat_ptr _tmp86F=(struct _fat_ptr)((struct _fat_ptr)Cyc_Absynpp_exp2string(e));_tmp5FF.f1=_tmp86F;});_tmp5FF;});void*_tmp3A4[1U];_tmp3A4[0]=& _tmp3A6;({struct _fat_ptr _tmp870=({const char*_tmp3A5="Toc did not examine an inserted check: %s";_tag_fat(_tmp3A5,sizeof(char),42U);});Cyc_aprintf(_tmp870,_tag_fat(_tmp3A4,sizeof(void*),1U));});});_tmp873(_tmp872,_tmp871,_tag_fat(_tmp3A3,sizeof(void*),0U));});});}}struct _tuple36{int f1;struct _fat_ptr*f2;struct _fat_ptr*f3;struct Cyc_Absyn_Switch_clause*f4;};
# 1454 "toc.cyc"
static struct _tuple36*Cyc_Toc_gen_labels(struct _RegionHandle*r,struct Cyc_Absyn_Switch_clause*sc){
# 2901 "toc.cyc"
return({struct _tuple36*_tmp3A8=_region_malloc(r,sizeof(*_tmp3A8));_tmp3A8->f1=0,({struct _fat_ptr*_tmp875=Cyc_Toc_fresh_label();_tmp3A8->f2=_tmp875;}),({struct _fat_ptr*_tmp874=Cyc_Toc_fresh_label();_tmp3A8->f3=_tmp874;}),_tmp3A8->f4=sc;_tmp3A8;});}
# 2907
static struct Cyc_Absyn_Exp*Cyc_Toc_compile_path(struct Cyc_List_List*ps,struct Cyc_Absyn_Exp*v){
for(0;ps != 0;ps=((struct Cyc_List_List*)_check_null(ps))->tl){
struct Cyc_Tcpat_PathNode*p=(struct Cyc_Tcpat_PathNode*)ps->hd;
# 2913
if((int)(((p->orig_pat).pattern).tag == 1)){
void*t=(void*)_check_null(({union Cyc_Tcpat_PatOrWhere _tmp3AA=p->orig_pat;if((_tmp3AA.pattern).tag != 1)_throw_match();(_tmp3AA.pattern).val;})->topt);
void*t2=Cyc_Tcutil_compress(Cyc_Toc_typ_to_c_array(t));
void*_tmp3A9=t2;switch(*((int*)_tmp3A9)){case 0U: switch(*((int*)((struct Cyc_Absyn_AppType_Absyn_Type_struct*)_tmp3A9)->f1)){case 0U: _LL1: _LL2:
# 2918
 goto _LL4;case 20U: _LL3: _LL4:
 goto _LL6;default: goto _LL7;}case 7U: _LL5: _LL6:
 goto _LL0;default: _LL7: _LL8:
 v=({void*_tmp876=Cyc_Toc_typ_to_c_array(t);Cyc_Toc_cast_it(_tmp876,v);});goto _LL0;}_LL0:;}{
# 2913
void*_stmttmp4C=p->access;void*_tmp3AB=_stmttmp4C;struct _fat_ptr*_tmp3AD;int _tmp3AC;unsigned _tmp3AE;unsigned _tmp3AF;switch(*((int*)_tmp3AB)){case 0U: _LLA: _LLB:
# 2928
 goto _LL9;case 1U: _LLC: _LLD:
# 2933
 if(ps->tl != 0){
void*_stmttmp4D=((struct Cyc_Tcpat_PathNode*)((struct Cyc_List_List*)_check_null(ps->tl))->hd)->access;void*_tmp3B0=_stmttmp4D;unsigned _tmp3B3;struct Cyc_Absyn_Datatypefield*_tmp3B2;struct Cyc_Absyn_Datatypedecl*_tmp3B1;if(((struct Cyc_Tcpat_DatatypeField_Tcpat_Access_struct*)_tmp3B0)->tag == 3U){_LL15: _tmp3B1=((struct Cyc_Tcpat_DatatypeField_Tcpat_Access_struct*)_tmp3B0)->f1;_tmp3B2=((struct Cyc_Tcpat_DatatypeField_Tcpat_Access_struct*)_tmp3B0)->f2;_tmp3B3=((struct Cyc_Tcpat_DatatypeField_Tcpat_Access_struct*)_tmp3B0)->f3;_LL16: {struct Cyc_Absyn_Datatypedecl*tud=_tmp3B1;struct Cyc_Absyn_Datatypefield*tuf=_tmp3B2;unsigned i=_tmp3B3;
# 2936
ps=ps->tl;
v=({void*_tmp878=({void*_tmp877=Cyc_Absyn_strctq(Cyc_Toc_collapse_qvars(tuf->name,tud->name));Cyc_Absyn_cstar_type(_tmp877,Cyc_Toc_mt_tq);});Cyc_Toc_cast_it(_tmp878,v);});
v=({struct Cyc_Absyn_Exp*_tmp879=v;Cyc_Absyn_aggrarrow_exp(_tmp879,Cyc_Absyn_fieldname((int)(i + (unsigned)1)),0U);});
continue;}}else{_LL17: _LL18:
 goto _LL14;}_LL14:;}
# 2933
v=
# 2943
Cyc_Absyn_deref_exp(v,0U);
goto _LL9;case 3U: _LLE: _tmp3AF=((struct Cyc_Tcpat_DatatypeField_Tcpat_Access_struct*)_tmp3AB)->f3;_LLF: {unsigned i=_tmp3AF;
_tmp3AE=i;goto _LL11;}case 2U: _LL10: _tmp3AE=((struct Cyc_Tcpat_TupleField_Tcpat_Access_struct*)_tmp3AB)->f1;_LL11: {unsigned i=_tmp3AE;
v=({struct Cyc_Absyn_Exp*_tmp87A=v;Cyc_Toc_member_exp(_tmp87A,Cyc_Absyn_fieldname((int)(i + (unsigned)1)),0U);});goto _LL9;}default: _LL12: _tmp3AC=((struct Cyc_Tcpat_AggrField_Tcpat_Access_struct*)_tmp3AB)->f1;_tmp3AD=((struct Cyc_Tcpat_AggrField_Tcpat_Access_struct*)_tmp3AB)->f2;_LL13: {int tagged=_tmp3AC;struct _fat_ptr*f=_tmp3AD;
# 2948
v=Cyc_Toc_member_exp(v,f,0U);
if(tagged)
v=Cyc_Toc_member_exp(v,Cyc_Toc_val_sp,0U);
# 2949
goto _LL9;}}_LL9:;}}
# 2954
return v;}
# 2907
static struct Cyc_Absyn_Exp*Cyc_Toc_compile_pat_test(struct Cyc_Absyn_Exp*v,void*t){
# 2960
void*_tmp3B4=t;struct Cyc_Absyn_Datatypefield*_tmp3B6;struct Cyc_Absyn_Datatypedecl*_tmp3B5;int _tmp3B8;struct _fat_ptr*_tmp3B7;struct Cyc_Absyn_Datatypefield*_tmp3BB;struct Cyc_Absyn_Datatypedecl*_tmp3BA;int _tmp3B9;unsigned _tmp3BC;int _tmp3BE;struct _fat_ptr _tmp3BD;struct Cyc_Absyn_Enumfield*_tmp3C0;void*_tmp3BF;struct Cyc_Absyn_Enumfield*_tmp3C2;struct Cyc_Absyn_Enumdecl*_tmp3C1;struct Cyc_Absyn_Exp*_tmp3C3;switch(*((int*)_tmp3B4)){case 0U: _LL1: _tmp3C3=((struct Cyc_Tcpat_WhereTest_Tcpat_PatTest_struct*)_tmp3B4)->f1;_LL2: {struct Cyc_Absyn_Exp*e=_tmp3C3;
return e == 0?v: e;}case 1U: _LL3: _LL4:
 return({struct Cyc_Absyn_Exp*_tmp87B=v;Cyc_Absyn_eq_exp(_tmp87B,Cyc_Absyn_signed_int_exp(0,0U),0U);});case 2U: _LL5: _LL6:
 return({struct Cyc_Absyn_Exp*_tmp87C=v;Cyc_Absyn_neq_exp(_tmp87C,Cyc_Absyn_signed_int_exp(0,0U),0U);});case 3U: _LL7: _tmp3C1=((struct Cyc_Tcpat_EqEnum_Tcpat_PatTest_struct*)_tmp3B4)->f1;_tmp3C2=((struct Cyc_Tcpat_EqEnum_Tcpat_PatTest_struct*)_tmp3B4)->f2;_LL8: {struct Cyc_Absyn_Enumdecl*ed=_tmp3C1;struct Cyc_Absyn_Enumfield*ef=_tmp3C2;
return({struct Cyc_Absyn_Exp*_tmp87D=v;Cyc_Absyn_eq_exp(_tmp87D,Cyc_Absyn_new_exp((void*)({struct Cyc_Absyn_Enum_e_Absyn_Raw_exp_struct*_tmp3C4=_cycalloc(sizeof(*_tmp3C4));_tmp3C4->tag=32U,_tmp3C4->f1=ed,_tmp3C4->f2=ef;_tmp3C4;}),0U),0U);});}case 4U: _LL9: _tmp3BF=(void*)((struct Cyc_Tcpat_EqAnonEnum_Tcpat_PatTest_struct*)_tmp3B4)->f1;_tmp3C0=((struct Cyc_Tcpat_EqAnonEnum_Tcpat_PatTest_struct*)_tmp3B4)->f2;_LLA: {void*t=_tmp3BF;struct Cyc_Absyn_Enumfield*ef=_tmp3C0;
return({struct Cyc_Absyn_Exp*_tmp87E=v;Cyc_Absyn_eq_exp(_tmp87E,Cyc_Absyn_new_exp((void*)({struct Cyc_Absyn_AnonEnum_e_Absyn_Raw_exp_struct*_tmp3C5=_cycalloc(sizeof(*_tmp3C5));_tmp3C5->tag=33U,_tmp3C5->f1=t,_tmp3C5->f2=ef;_tmp3C5;}),0U),0U);});}case 5U: _LLB: _tmp3BD=((struct Cyc_Tcpat_EqFloat_Tcpat_PatTest_struct*)_tmp3B4)->f1;_tmp3BE=((struct Cyc_Tcpat_EqFloat_Tcpat_PatTest_struct*)_tmp3B4)->f2;_LLC: {struct _fat_ptr s=_tmp3BD;int i=_tmp3BE;
return({struct Cyc_Absyn_Exp*_tmp87F=v;Cyc_Absyn_eq_exp(_tmp87F,Cyc_Absyn_float_exp(s,i,0U),0U);});}case 6U: _LLD: _tmp3BC=((struct Cyc_Tcpat_EqConst_Tcpat_PatTest_struct*)_tmp3B4)->f1;_LLE: {unsigned i=_tmp3BC;
return({struct Cyc_Absyn_Exp*_tmp880=v;Cyc_Absyn_eq_exp(_tmp880,Cyc_Absyn_signed_int_exp((int)i,0U),0U);});}case 7U: _LLF: _tmp3B9=((struct Cyc_Tcpat_EqDatatypeTag_Tcpat_PatTest_struct*)_tmp3B4)->f1;_tmp3BA=((struct Cyc_Tcpat_EqDatatypeTag_Tcpat_PatTest_struct*)_tmp3B4)->f2;_tmp3BB=((struct Cyc_Tcpat_EqDatatypeTag_Tcpat_PatTest_struct*)_tmp3B4)->f3;_LL10: {int i=_tmp3B9;struct Cyc_Absyn_Datatypedecl*tud=_tmp3BA;struct Cyc_Absyn_Datatypefield*tuf=_tmp3BB;
# 2971
LOOP1: {
void*_stmttmp4E=v->r;void*_tmp3C6=_stmttmp4E;struct Cyc_Absyn_Exp*_tmp3C7;struct Cyc_Absyn_Exp*_tmp3C8;switch(*((int*)_tmp3C6)){case 14U: _LL16: _tmp3C8=((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp3C6)->f2;_LL17: {struct Cyc_Absyn_Exp*e=_tmp3C8;
v=e;goto LOOP1;}case 20U: _LL18: _tmp3C7=((struct Cyc_Absyn_Deref_e_Absyn_Raw_exp_struct*)_tmp3C6)->f1;_LL19: {struct Cyc_Absyn_Exp*e=_tmp3C7;
v=e;goto _LL15;}default: _LL1A: _LL1B:
 goto _LL15;}_LL15:;}
# 2978
v=({void*_tmp882=({void*_tmp881=Cyc_Absyn_strctq(Cyc_Toc_collapse_qvars(tuf->name,tud->name));Cyc_Absyn_cstar_type(_tmp881,Cyc_Toc_mt_tq);});Cyc_Toc_cast_it(_tmp882,v);});return({struct Cyc_Absyn_Exp*_tmp883=Cyc_Absyn_aggrarrow_exp(v,Cyc_Toc_tag_sp,0U);Cyc_Absyn_eq_exp(_tmp883,Cyc_Absyn_uint_exp((unsigned)i,0U),0U);});}case 8U: _LL11: _tmp3B7=((struct Cyc_Tcpat_EqTaggedUnion_Tcpat_PatTest_struct*)_tmp3B4)->f1;_tmp3B8=((struct Cyc_Tcpat_EqTaggedUnion_Tcpat_PatTest_struct*)_tmp3B4)->f2;_LL12: {struct _fat_ptr*f=_tmp3B7;int i=_tmp3B8;
# 2982
return({struct Cyc_Absyn_Exp*_tmp885=({struct Cyc_Absyn_Exp*_tmp884=Cyc_Toc_member_exp(v,f,0U);Cyc_Toc_member_exp(_tmp884,Cyc_Toc_tag_sp,0U);});Cyc_Absyn_eq_exp(_tmp885,
Cyc_Absyn_signed_int_exp(i,0U),0U);});}default: _LL13: _tmp3B5=((struct Cyc_Tcpat_EqExtensibleDatatype_Tcpat_PatTest_struct*)_tmp3B4)->f1;_tmp3B6=((struct Cyc_Tcpat_EqExtensibleDatatype_Tcpat_PatTest_struct*)_tmp3B4)->f2;_LL14: {struct Cyc_Absyn_Datatypedecl*tud=_tmp3B5;struct Cyc_Absyn_Datatypefield*tuf=_tmp3B6;
# 2986
LOOP2: {
void*_stmttmp4F=v->r;void*_tmp3C9=_stmttmp4F;struct Cyc_Absyn_Exp*_tmp3CA;struct Cyc_Absyn_Exp*_tmp3CB;switch(*((int*)_tmp3C9)){case 14U: _LL1D: _tmp3CB=((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp3C9)->f2;_LL1E: {struct Cyc_Absyn_Exp*e=_tmp3CB;
v=e;goto LOOP2;}case 20U: _LL1F: _tmp3CA=((struct Cyc_Absyn_Deref_e_Absyn_Raw_exp_struct*)_tmp3C9)->f1;_LL20: {struct Cyc_Absyn_Exp*e=_tmp3CA;
v=e;goto _LL1C;}default: _LL21: _LL22:
 goto _LL1C;}_LL1C:;}
# 2993
v=({void*_tmp887=({void*_tmp886=Cyc_Absyn_strctq(Cyc_Toc_collapse_qvars(tuf->name,tud->name));Cyc_Absyn_cstar_type(_tmp886,Cyc_Toc_mt_tq);});Cyc_Toc_cast_it(_tmp887,v);});
return({struct Cyc_Absyn_Exp*_tmp888=Cyc_Absyn_aggrarrow_exp(v,Cyc_Toc_tag_sp,0U);Cyc_Absyn_eq_exp(_tmp888,Cyc_Absyn_var_exp(tuf->name,0U),0U);});}}_LL0:;}struct Cyc_Toc_OtherTest_Toc_TestKind_struct{int tag;};struct Cyc_Toc_DatatypeTagTest_Toc_TestKind_struct{int tag;};struct Cyc_Toc_WhereTest_Toc_TestKind_struct{int tag;struct Cyc_Absyn_Exp*f1;};struct Cyc_Toc_TaggedUnionTest_Toc_TestKind_struct{int tag;struct _fat_ptr*f1;};
# 3005
struct Cyc_Toc_OtherTest_Toc_TestKind_struct Cyc_Toc_OtherTest_val={0U};
struct Cyc_Toc_DatatypeTagTest_Toc_TestKind_struct Cyc_Toc_DatatypeTagTest_val={1U};struct _tuple37{int f1;void*f2;};
# 3012
static struct _tuple37 Cyc_Toc_admits_switch(struct Cyc_List_List*ss){
# 3014
int c=0;
void*k=(void*)& Cyc_Toc_OtherTest_val;
for(0;ss != 0;(ss=ss->tl,c=c + 1)){
struct _tuple33 _stmttmp50=*((struct _tuple33*)ss->hd);void*_tmp3CC;_LL1: _tmp3CC=_stmttmp50.f1;_LL2: {void*ptest=_tmp3CC;
void*_tmp3CD=ptest;struct Cyc_Absyn_Exp*_tmp3CE;struct _fat_ptr*_tmp3CF;switch(*((int*)_tmp3CD)){case 3U: _LL4: _LL5:
# 3020
 goto _LL7;case 4U: _LL6: _LL7:
 goto _LL9;case 6U: _LL8: _LL9:
 continue;case 8U: _LLA: _tmp3CF=((struct Cyc_Tcpat_EqTaggedUnion_Tcpat_PatTest_struct*)_tmp3CD)->f1;_LLB: {struct _fat_ptr*f=_tmp3CF;
# 3024
if(k == (void*)& Cyc_Toc_OtherTest_val)
k=(void*)({struct Cyc_Toc_TaggedUnionTest_Toc_TestKind_struct*_tmp3D0=_cycalloc(sizeof(*_tmp3D0));_tmp3D0->tag=3U,_tmp3D0->f1=f;_tmp3D0;});
# 3024
continue;}case 7U: _LLC: _LLD:
# 3027
 k=(void*)& Cyc_Toc_DatatypeTagTest_val;continue;case 0U: _LLE: _tmp3CE=((struct Cyc_Tcpat_WhereTest_Tcpat_PatTest_struct*)_tmp3CD)->f1;if(_tmp3CE != 0){_LLF: {struct Cyc_Absyn_Exp*e=_tmp3CE;
# 3029
k=(void*)({struct Cyc_Toc_WhereTest_Toc_TestKind_struct*_tmp3D1=_cycalloc(sizeof(*_tmp3D1));_tmp3D1->tag=2U,_tmp3D1->f1=e;_tmp3D1;});return({struct _tuple37 _tmp600;_tmp600.f1=0,_tmp600.f2=k;_tmp600;});}}else{_LL10: _LL11:
 goto _LL13;}case 1U: _LL12: _LL13:
 goto _LL15;case 2U: _LL14: _LL15:
 goto _LL17;case 5U: _LL16: _LL17:
 goto _LL19;default: _LL18: _LL19:
 return({struct _tuple37 _tmp601;_tmp601.f1=0,_tmp601.f2=k;_tmp601;});}_LL3:;}}
# 3037
return({struct _tuple37 _tmp602;_tmp602.f1=c,_tmp602.f2=k;_tmp602;});}
# 3012
static struct Cyc_Absyn_Pat*Cyc_Toc_compile_pat_test_as_case(void*p){
# 3043
struct Cyc_Absyn_Exp*e;
{void*_tmp3D2=p;int _tmp3D3;int _tmp3D4;unsigned _tmp3D5;struct Cyc_Absyn_Enumfield*_tmp3D7;void*_tmp3D6;struct Cyc_Absyn_Enumfield*_tmp3D9;struct Cyc_Absyn_Enumdecl*_tmp3D8;switch(*((int*)_tmp3D2)){case 3U: _LL1: _tmp3D8=((struct Cyc_Tcpat_EqEnum_Tcpat_PatTest_struct*)_tmp3D2)->f1;_tmp3D9=((struct Cyc_Tcpat_EqEnum_Tcpat_PatTest_struct*)_tmp3D2)->f2;_LL2: {struct Cyc_Absyn_Enumdecl*ed=_tmp3D8;struct Cyc_Absyn_Enumfield*ef=_tmp3D9;
e=Cyc_Absyn_new_exp((void*)({struct Cyc_Absyn_Enum_e_Absyn_Raw_exp_struct*_tmp3DA=_cycalloc(sizeof(*_tmp3DA));_tmp3DA->tag=32U,_tmp3DA->f1=ed,_tmp3DA->f2=ef;_tmp3DA;}),0U);goto _LL0;}case 4U: _LL3: _tmp3D6=(void*)((struct Cyc_Tcpat_EqAnonEnum_Tcpat_PatTest_struct*)_tmp3D2)->f1;_tmp3D7=((struct Cyc_Tcpat_EqAnonEnum_Tcpat_PatTest_struct*)_tmp3D2)->f2;_LL4: {void*t=_tmp3D6;struct Cyc_Absyn_Enumfield*ef=_tmp3D7;
e=Cyc_Absyn_new_exp((void*)({struct Cyc_Absyn_AnonEnum_e_Absyn_Raw_exp_struct*_tmp3DB=_cycalloc(sizeof(*_tmp3DB));_tmp3DB->tag=33U,_tmp3DB->f1=t,_tmp3DB->f2=ef;_tmp3DB;}),0U);goto _LL0;}case 6U: _LL5: _tmp3D5=((struct Cyc_Tcpat_EqConst_Tcpat_PatTest_struct*)_tmp3D2)->f1;_LL6: {unsigned i=_tmp3D5;
_tmp3D4=(int)i;goto _LL8;}case 7U: _LL7: _tmp3D4=((struct Cyc_Tcpat_EqDatatypeTag_Tcpat_PatTest_struct*)_tmp3D2)->f1;_LL8: {int i=_tmp3D4;
_tmp3D3=i;goto _LLA;}case 8U: _LL9: _tmp3D3=((struct Cyc_Tcpat_EqTaggedUnion_Tcpat_PatTest_struct*)_tmp3D2)->f2;_LLA: {int i=_tmp3D3;
e=Cyc_Absyn_uint_exp((unsigned)i,0U);goto _LL0;}default: _LLB: _LLC:
({void*_tmp3DC=0U;({int(*_tmp88A)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp3DE)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Toc_toc_impos;_tmp3DE;});struct _fat_ptr _tmp889=({const char*_tmp3DD="compile_pat_test_as_case!";_tag_fat(_tmp3DD,sizeof(char),26U);});_tmp88A(_tmp889,_tag_fat(_tmp3DC,sizeof(void*),0U));});});}_LL0:;}
# 3052
return Cyc_Absyn_new_pat((void*)({struct Cyc_Absyn_Exp_p_Absyn_Raw_pat_struct*_tmp3DF=_cycalloc(sizeof(*_tmp3DF));_tmp3DF->tag=17U,_tmp3DF->f1=e;_tmp3DF;}),0U);}
# 3056
static struct Cyc_Absyn_Stmt*Cyc_Toc_seq_stmt_opt(struct Cyc_Absyn_Stmt*s1,struct Cyc_Absyn_Stmt*s2){
if(s1 == 0)return s2;if(s2 == 0)
return s1;
# 3057
return
# 3059
 Cyc_Absyn_seq_stmt(s1,s2,0U);}struct _tuple38{struct Cyc_Absyn_Vardecl*f1;struct Cyc_Absyn_Vardecl*f2;};struct _tuple39{struct Cyc_List_List*f1;struct Cyc_Absyn_Pat*f2;};
# 3067
static struct Cyc_Absyn_Stmt*Cyc_Toc_extract_pattern_vars(struct _RegionHandle*rgn,struct Cyc_Toc_Env**nv,struct Cyc_List_List**decls,struct Cyc_Absyn_Exp*path,struct Cyc_Absyn_Pat*p){
# 3071
void*t=(void*)_check_null(p->topt);
void*_stmttmp51=p->r;void*_tmp3E0=_stmttmp51;struct Cyc_List_List*_tmp3E2;union Cyc_Absyn_AggrInfo _tmp3E1;struct Cyc_List_List*_tmp3E3;struct Cyc_List_List*_tmp3E4;struct Cyc_Absyn_Pat*_tmp3E5;struct Cyc_List_List*_tmp3E8;struct Cyc_Absyn_Datatypefield*_tmp3E7;struct Cyc_Absyn_Datatypedecl*_tmp3E6;struct Cyc_Absyn_Pat*_tmp3EA;struct Cyc_Absyn_Vardecl*_tmp3E9;struct Cyc_Absyn_Vardecl*_tmp3EB;struct Cyc_Absyn_Pat*_tmp3ED;struct Cyc_Absyn_Vardecl*_tmp3EC;struct Cyc_Absyn_Vardecl*_tmp3EE;switch(*((int*)_tmp3E0)){case 2U: _LL1: _tmp3EE=((struct Cyc_Absyn_AliasVar_p_Absyn_Raw_pat_struct*)_tmp3E0)->f2;_LL2: {struct Cyc_Absyn_Vardecl*vd=_tmp3EE;
# 3074
struct Cyc_Absyn_Pat*p2=Cyc_Absyn_new_pat((void*)& Cyc_Absyn_Wild_p_val,0U);
p2->topt=p->topt;
_tmp3EC=vd;_tmp3ED=p2;goto _LL4;}case 1U: _LL3: _tmp3EC=((struct Cyc_Absyn_Var_p_Absyn_Raw_pat_struct*)_tmp3E0)->f1;_tmp3ED=((struct Cyc_Absyn_Var_p_Absyn_Raw_pat_struct*)_tmp3E0)->f2;_LL4: {struct Cyc_Absyn_Vardecl*vd=_tmp3EC;struct Cyc_Absyn_Pat*p2=_tmp3ED;
# 3079
struct Cyc_Absyn_Vardecl*new_vd=({struct _tuple1*_tmp88B=Cyc_Toc_temp_var();Cyc_Absyn_new_vardecl(0U,_tmp88B,Cyc_Toc_typ_to_c_array(vd->type),0);});
({struct Cyc_List_List*_tmp88D=({struct Cyc_List_List*_tmp3F0=_region_malloc(rgn,sizeof(*_tmp3F0));({struct _tuple38*_tmp88C=({struct _tuple38*_tmp3EF=_region_malloc(rgn,sizeof(*_tmp3EF));_tmp3EF->f1=vd,_tmp3EF->f2=new_vd;_tmp3EF;});_tmp3F0->hd=_tmp88C;}),_tmp3F0->tl=*decls;_tmp3F0;});*decls=_tmp88D;});{
struct Cyc_Absyn_Stmt*s=({struct Cyc_Absyn_Exp*_tmp88E=Cyc_Absyn_varb_exp((void*)({struct Cyc_Absyn_Local_b_Absyn_Binding_struct*_tmp3F1=_cycalloc(sizeof(*_tmp3F1));_tmp3F1->tag=4U,_tmp3F1->f1=new_vd;_tmp3F1;}),0U);Cyc_Absyn_assign_stmt(_tmp88E,Cyc_Absyn_copy_exp(path),0U);});
return({struct Cyc_Absyn_Stmt*_tmp88F=s;Cyc_Toc_seq_stmt_opt(_tmp88F,Cyc_Toc_extract_pattern_vars(rgn,nv,decls,path,p2));});}}case 4U: _LL5: _tmp3EB=((struct Cyc_Absyn_TagInt_p_Absyn_Raw_pat_struct*)_tmp3E0)->f2;_LL6: {struct Cyc_Absyn_Vardecl*vd=_tmp3EB;
# 3085
struct Cyc_Absyn_Vardecl*new_vd=({struct _tuple1*_tmp890=Cyc_Toc_temp_var();Cyc_Absyn_new_vardecl(0U,_tmp890,Cyc_Toc_typ_to_c_array(vd->type),0);});
({struct Cyc_List_List*_tmp892=({struct Cyc_List_List*_tmp3F3=_region_malloc(rgn,sizeof(*_tmp3F3));({struct _tuple38*_tmp891=({struct _tuple38*_tmp3F2=_region_malloc(rgn,sizeof(*_tmp3F2));_tmp3F2->f1=vd,_tmp3F2->f2=new_vd;_tmp3F2;});_tmp3F3->hd=_tmp891;}),_tmp3F3->tl=*decls;_tmp3F3;});*decls=_tmp892;});
return({struct Cyc_Absyn_Exp*_tmp893=Cyc_Absyn_varb_exp((void*)({struct Cyc_Absyn_Local_b_Absyn_Binding_struct*_tmp3F4=_cycalloc(sizeof(*_tmp3F4));_tmp3F4->tag=4U,_tmp3F4->f1=new_vd;_tmp3F4;}),0U);Cyc_Absyn_assign_stmt(_tmp893,Cyc_Absyn_copy_exp(path),0U);});}case 0U: _LL7: _LL8:
 return 0;case 3U: _LL9: _tmp3E9=((struct Cyc_Absyn_Reference_p_Absyn_Raw_pat_struct*)_tmp3E0)->f1;_tmp3EA=((struct Cyc_Absyn_Reference_p_Absyn_Raw_pat_struct*)_tmp3E0)->f2;_LLA: {struct Cyc_Absyn_Vardecl*vd=_tmp3E9;struct Cyc_Absyn_Pat*p2=_tmp3EA;
# 3091
({void*_tmp894=Cyc_Absyn_cstar_type(t,Cyc_Toc_mt_tq);vd->type=_tmp894;});{
struct Cyc_Absyn_Vardecl*new_vd=({struct _tuple1*_tmp895=Cyc_Toc_temp_var();Cyc_Absyn_new_vardecl(0U,_tmp895,Cyc_Toc_typ_to_c_array(vd->type),0);});
({struct Cyc_List_List*_tmp897=({struct Cyc_List_List*_tmp3F6=_region_malloc(rgn,sizeof(*_tmp3F6));({struct _tuple38*_tmp896=({struct _tuple38*_tmp3F5=_region_malloc(rgn,sizeof(*_tmp3F5));_tmp3F5->f1=vd,_tmp3F5->f2=new_vd;_tmp3F5;});_tmp3F6->hd=_tmp896;}),_tmp3F6->tl=*decls;_tmp3F6;});*decls=_tmp897;});{
# 3095
struct Cyc_Absyn_Stmt*s=({struct Cyc_Absyn_Exp*_tmp89A=Cyc_Absyn_varb_exp((void*)({struct Cyc_Absyn_Local_b_Absyn_Binding_struct*_tmp3F7=_cycalloc(sizeof(*_tmp3F7));_tmp3F7->tag=4U,_tmp3F7->f1=new_vd;_tmp3F7;}),0U);Cyc_Absyn_assign_stmt(_tmp89A,({
void*_tmp899=({void*_tmp898=Cyc_Toc_typ_to_c_array(t);Cyc_Absyn_cstar_type(_tmp898,Cyc_Toc_mt_tq);});Cyc_Toc_cast_it(_tmp899,
Cyc_Toc_push_address_exp(Cyc_Absyn_copy_exp(path)));}),0U);});
return({struct Cyc_Absyn_Stmt*_tmp89B=s;Cyc_Toc_seq_stmt_opt(_tmp89B,Cyc_Toc_extract_pattern_vars(rgn,nv,decls,path,p2));});}}}case 9U: _LLB: _LLC:
# 3100
 goto _LLE;case 10U: _LLD: _LLE:
 goto _LL10;case 11U: _LLF: _LL10:
 goto _LL12;case 12U: _LL11: _LL12:
 goto _LL14;case 13U: _LL13: _LL14:
 goto _LL16;case 14U: _LL15: _LL16:
 return 0;case 6U: if(((struct Cyc_Absyn_Datatype_p_Absyn_Raw_pat_struct*)((struct Cyc_Absyn_Pat*)((struct Cyc_Absyn_Pointer_p_Absyn_Raw_pat_struct*)_tmp3E0)->f1)->r)->tag == 8U){_LL17: _tmp3E6=((struct Cyc_Absyn_Datatype_p_Absyn_Raw_pat_struct*)(((struct Cyc_Absyn_Pointer_p_Absyn_Raw_pat_struct*)_tmp3E0)->f1)->r)->f1;_tmp3E7=((struct Cyc_Absyn_Datatype_p_Absyn_Raw_pat_struct*)(((struct Cyc_Absyn_Pointer_p_Absyn_Raw_pat_struct*)_tmp3E0)->f1)->r)->f2;_tmp3E8=((struct Cyc_Absyn_Datatype_p_Absyn_Raw_pat_struct*)(((struct Cyc_Absyn_Pointer_p_Absyn_Raw_pat_struct*)_tmp3E0)->f1)->r)->f3;_LL18: {struct Cyc_Absyn_Datatypedecl*tud=_tmp3E6;struct Cyc_Absyn_Datatypefield*tuf=_tmp3E7;struct Cyc_List_List*ps=_tmp3E8;
# 3109
if(ps == 0)return 0;{struct _tuple1*tufstrct=
Cyc_Toc_collapse_qvars(tuf->name,tud->name);
void*field_ptr_typ=({void*_tmp89C=Cyc_Absyn_strctq(tufstrct);Cyc_Absyn_cstar_type(_tmp89C,Cyc_Toc_mt_tq);});
path=Cyc_Toc_cast_it(field_ptr_typ,path);{
int cnt=1;
struct Cyc_List_List*tuf_tqts=tuf->typs;
struct Cyc_Absyn_Stmt*s=0;
for(0;ps != 0;(ps=ps->tl,tuf_tqts=((struct Cyc_List_List*)_check_null(tuf_tqts))->tl,++ cnt)){
struct Cyc_Absyn_Pat*p2=(struct Cyc_Absyn_Pat*)ps->hd;
if(p2->r == (void*)& Cyc_Absyn_Wild_p_val)continue;{void*tuf_typ=(*((struct _tuple13*)((struct Cyc_List_List*)_check_null(tuf_tqts))->hd)).f2;
# 3120
void*t2=(void*)_check_null(p2->topt);
void*t2c=Cyc_Toc_typ_to_c_array(t2);
struct Cyc_Absyn_Exp*arrow_exp=({struct Cyc_Absyn_Exp*_tmp89D=path;Cyc_Absyn_aggrarrow_exp(_tmp89D,Cyc_Absyn_fieldname(cnt),0U);});
if(Cyc_Toc_is_void_star_or_boxed_tvar(Cyc_Toc_typ_to_c(tuf_typ)))
arrow_exp=Cyc_Toc_cast_it(t2c,arrow_exp);
# 3123
s=({
# 3125
struct Cyc_Absyn_Stmt*_tmp89E=s;Cyc_Toc_seq_stmt_opt(_tmp89E,Cyc_Toc_extract_pattern_vars(rgn,nv,decls,arrow_exp,p2));});}}
# 3127
return s;}}}}else{_LL21: _tmp3E5=((struct Cyc_Absyn_Pointer_p_Absyn_Raw_pat_struct*)_tmp3E0)->f1;_LL22: {struct Cyc_Absyn_Pat*p2=_tmp3E5;
# 3172
return({struct _RegionHandle*_tmp8A2=rgn;struct Cyc_Toc_Env**_tmp8A1=nv;struct Cyc_List_List**_tmp8A0=decls;struct Cyc_Absyn_Exp*_tmp89F=Cyc_Absyn_deref_exp(path,0U);Cyc_Toc_extract_pattern_vars(_tmp8A2,_tmp8A1,_tmp8A0,_tmp89F,p2);});}}case 8U: _LL19: _tmp3E4=((struct Cyc_Absyn_Datatype_p_Absyn_Raw_pat_struct*)_tmp3E0)->f3;_LL1A: {struct Cyc_List_List*ps=_tmp3E4;
# 3129
_tmp3E3=ps;goto _LL1C;}case 5U: _LL1B: _tmp3E3=((struct Cyc_Absyn_Tuple_p_Absyn_Raw_pat_struct*)_tmp3E0)->f1;_LL1C: {struct Cyc_List_List*ps=_tmp3E3;
# 3131
struct Cyc_Absyn_Stmt*s=0;
int cnt=1;
for(0;ps != 0;(ps=ps->tl,++ cnt)){
struct Cyc_Absyn_Pat*p2=(struct Cyc_Absyn_Pat*)ps->hd;
if(p2->r == (void*)& Cyc_Absyn_Wild_p_val)
continue;{
# 3135
void*t2=(void*)_check_null(p2->topt);
# 3138
struct _fat_ptr*f=Cyc_Absyn_fieldname(cnt);
s=({struct Cyc_Absyn_Stmt*_tmp8A7=s;Cyc_Toc_seq_stmt_opt(_tmp8A7,({struct _RegionHandle*_tmp8A6=rgn;struct Cyc_Toc_Env**_tmp8A5=nv;struct Cyc_List_List**_tmp8A4=decls;struct Cyc_Absyn_Exp*_tmp8A3=Cyc_Toc_member_exp(path,f,0U);Cyc_Toc_extract_pattern_vars(_tmp8A6,_tmp8A5,_tmp8A4,_tmp8A3,p2);}));});}}
# 3141
return s;}case 7U: if(((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmp3E0)->f1 == 0){_LL1D: _LL1E:
({void*_tmp3F8=0U;({int(*_tmp8A9)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp3FA)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Toc_toc_impos;_tmp3FA;});struct _fat_ptr _tmp8A8=({const char*_tmp3F9="unresolved aggregate pattern!";_tag_fat(_tmp3F9,sizeof(char),30U);});_tmp8A9(_tmp8A8,_tag_fat(_tmp3F8,sizeof(void*),0U));});});}else{_LL1F: _tmp3E1=*((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmp3E0)->f1;_tmp3E2=((struct Cyc_Absyn_Aggr_p_Absyn_Raw_pat_struct*)_tmp3E0)->f3;_LL20: {union Cyc_Absyn_AggrInfo info=_tmp3E1;struct Cyc_List_List*dlps=_tmp3E2;
# 3145
struct Cyc_Absyn_Aggrdecl*ad=Cyc_Absyn_get_known_aggrdecl(info);
struct Cyc_Absyn_Stmt*s=0;
for(0;dlps != 0;dlps=dlps->tl){
struct _tuple39*tup=(struct _tuple39*)dlps->hd;
struct Cyc_Absyn_Pat*p2=(*tup).f2;
if(p2->r == (void*)& Cyc_Absyn_Wild_p_val)
continue;{
# 3150
struct _fat_ptr*f=
# 3152
Cyc_Absyn_designatorlist_to_fieldname((*tup).f1);
void*t2=(void*)_check_null(p2->topt);
void*t2c=Cyc_Toc_typ_to_c_array(t2);
struct Cyc_Absyn_Exp*memexp=Cyc_Toc_member_exp(path,f,0U);
# 3157
if(((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->tagged)memexp=Cyc_Toc_member_exp(memexp,Cyc_Toc_val_sp,0U);{void*ftype=((struct Cyc_Absyn_Aggrfield*)_check_null(Cyc_Absyn_lookup_field(((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->fields,f)))->type;
# 3159
if(Cyc_Toc_is_void_star_or_boxed_tvar(ftype))
memexp=Cyc_Toc_cast_it(t2c,memexp);else{
if(!Cyc_Tcutil_is_array_type(ftype)&& Cyc_Tcutil_kind_leq(& Cyc_Tcutil_ak,Cyc_Tcutil_type_kind(ftype)))
# 3164
memexp=Cyc_Absyn_deref_exp(({void*_tmp8AA=Cyc_Absyn_cstar_type(t2c,Cyc_Toc_mt_tq);Cyc_Toc_cast_it(_tmp8AA,
Cyc_Absyn_address_exp(memexp,0U));}),0U);}
# 3159
s=({
# 3167
struct Cyc_Absyn_Stmt*_tmp8AB=s;Cyc_Toc_seq_stmt_opt(_tmp8AB,Cyc_Toc_extract_pattern_vars(rgn,nv,decls,memexp,p2));});}}}
# 3169
return s;}}case 15U: _LL23: _LL24:
# 3174
({void*_tmp3FB=0U;({int(*_tmp8AD)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp3FD)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Toc_toc_impos;_tmp3FD;});struct _fat_ptr _tmp8AC=({const char*_tmp3FC="unknownid pat";_tag_fat(_tmp3FC,sizeof(char),14U);});_tmp8AD(_tmp8AC,_tag_fat(_tmp3FB,sizeof(void*),0U));});});case 16U: _LL25: _LL26:
({void*_tmp3FE=0U;({int(*_tmp8AF)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp400)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Toc_toc_impos;_tmp400;});struct _fat_ptr _tmp8AE=({const char*_tmp3FF="unknowncall pat";_tag_fat(_tmp3FF,sizeof(char),16U);});_tmp8AF(_tmp8AE,_tag_fat(_tmp3FE,sizeof(void*),0U));});});default: _LL27: _LL28:
({void*_tmp401=0U;({int(*_tmp8B1)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp403)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Toc_toc_impos;_tmp403;});struct _fat_ptr _tmp8B0=({const char*_tmp402="exp pat";_tag_fat(_tmp402,sizeof(char),8U);});_tmp8B1(_tmp8B0,_tag_fat(_tmp401,sizeof(void*),0U));});});}_LL0:;}
# 3067
static struct Cyc_Absyn_Vardecl*Cyc_Toc_find_new_var(struct Cyc_List_List*vs,struct Cyc_Absyn_Vardecl*v){
# 3181
for(0;vs != 0;vs=vs->tl){
struct _tuple38 _stmttmp52=*((struct _tuple38*)vs->hd);struct Cyc_Absyn_Vardecl*_tmp405;struct Cyc_Absyn_Vardecl*_tmp404;_LL1: _tmp404=_stmttmp52.f1;_tmp405=_stmttmp52.f2;_LL2: {struct Cyc_Absyn_Vardecl*oldv=_tmp404;struct Cyc_Absyn_Vardecl*newv=_tmp405;
if(oldv == newv)return newv;}}
# 3185
({void*_tmp406=0U;({int(*_tmp8B3)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp408)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Toc_toc_impos;_tmp408;});struct _fat_ptr _tmp8B2=({const char*_tmp407="find_new_var";_tag_fat(_tmp407,sizeof(char),13U);});_tmp8B3(_tmp8B2,_tag_fat(_tmp406,sizeof(void*),0U));});});}
# 3067
static void Cyc_Toc_subst_pattern_vars(struct Cyc_List_List*env,struct Cyc_Absyn_Exp*e){
# 3192
void*_stmttmp53=e->r;void*_tmp409=_stmttmp53;struct Cyc_List_List*_tmp40A;struct Cyc_List_List*_tmp40B;struct Cyc_List_List*_tmp40C;struct Cyc_List_List*_tmp40D;struct Cyc_List_List*_tmp40E;struct Cyc_List_List*_tmp40F;struct Cyc_List_List*_tmp410;struct Cyc_List_List*_tmp411;struct Cyc_Absyn_Exp*_tmp413;struct Cyc_Absyn_Exp*_tmp412;struct Cyc_Absyn_Exp*_tmp415;struct Cyc_Absyn_Exp*_tmp414;struct Cyc_Absyn_Exp*_tmp416;struct Cyc_Absyn_Exp*_tmp417;struct Cyc_Absyn_Exp*_tmp418;struct Cyc_Absyn_Exp*_tmp419;struct Cyc_Absyn_Exp*_tmp41A;struct Cyc_Absyn_Exp*_tmp41B;struct Cyc_Absyn_Exp*_tmp41C;struct Cyc_Absyn_Exp*_tmp41D;struct Cyc_Absyn_Exp*_tmp41E;struct Cyc_Absyn_Exp*_tmp41F;struct Cyc_Absyn_Exp*_tmp420;struct Cyc_Absyn_Exp*_tmp421;struct Cyc_Absyn_Exp*_tmp423;struct Cyc_Absyn_Exp*_tmp422;struct Cyc_Absyn_Exp*_tmp425;struct Cyc_Absyn_Exp*_tmp424;struct Cyc_Absyn_Exp*_tmp427;struct Cyc_Absyn_Exp*_tmp426;struct Cyc_Absyn_Exp*_tmp429;struct Cyc_Absyn_Exp*_tmp428;struct Cyc_Absyn_Exp*_tmp42B;struct Cyc_Absyn_Exp*_tmp42A;struct Cyc_Absyn_Exp*_tmp42D;struct Cyc_Absyn_Exp*_tmp42C;struct Cyc_Absyn_Exp*_tmp430;struct Cyc_Absyn_Exp*_tmp42F;struct Cyc_Absyn_Exp*_tmp42E;struct Cyc_Absyn_Vardecl*_tmp431;struct Cyc_Absyn_Vardecl*_tmp432;switch(*((int*)_tmp409)){case 1U: switch(*((int*)((struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct*)_tmp409)->f1)){case 5U: _LL1: _tmp432=((struct Cyc_Absyn_Pat_b_Absyn_Binding_struct*)((struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct*)_tmp409)->f1)->f1;_LL2: {struct Cyc_Absyn_Vardecl*vd=_tmp432;
_tmp431=vd;goto _LL4;}case 4U: _LL3: _tmp431=((struct Cyc_Absyn_Local_b_Absyn_Binding_struct*)((struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct*)_tmp409)->f1)->f1;_LL4: {struct Cyc_Absyn_Vardecl*vd=_tmp431;
# 3195
{struct _handler_cons _tmp433;_push_handler(& _tmp433);{int _tmp435=0;if(setjmp(_tmp433.handler))_tmp435=1;if(!_tmp435){
({void*_tmp8B8=(void*)({struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct*_tmp438=_cycalloc(sizeof(*_tmp438));_tmp438->tag=1U,({void*_tmp8B7=(void*)({struct Cyc_Absyn_Local_b_Absyn_Binding_struct*_tmp437=_cycalloc(sizeof(*_tmp437));_tmp437->tag=4U,({struct Cyc_Absyn_Vardecl*_tmp8B6=({struct Cyc_Absyn_Vardecl*(*_tmp8B5)(struct Cyc_List_List*l,struct Cyc_Absyn_Vardecl*k)=({struct Cyc_Absyn_Vardecl*(*_tmp436)(struct Cyc_List_List*l,struct Cyc_Absyn_Vardecl*k)=(struct Cyc_Absyn_Vardecl*(*)(struct Cyc_List_List*l,struct Cyc_Absyn_Vardecl*k))Cyc_List_assoc;_tmp436;});struct Cyc_List_List*_tmp8B4=env;_tmp8B5(_tmp8B4,vd);});_tmp437->f1=_tmp8B6;});_tmp437;});_tmp438->f1=_tmp8B7;});_tmp438;});e->r=_tmp8B8;});;_pop_handler();}else{void*_tmp434=(void*)Cyc_Core_get_exn_thrown();void*_tmp439=_tmp434;void*_tmp43A;if(((struct Cyc_Core_Not_found_exn_struct*)_tmp439)->tag == Cyc_Core_Not_found){_LL42: _LL43:
# 3198
 goto _LL41;}else{_LL44: _tmp43A=_tmp439;_LL45: {void*exn=_tmp43A;(int)_rethrow(exn);}}_LL41:;}}}
# 3200
goto _LL0;}default: goto _LL3F;}case 6U: _LL5: _tmp42E=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmp409)->f1;_tmp42F=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmp409)->f2;_tmp430=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmp409)->f3;_LL6: {struct Cyc_Absyn_Exp*e1=_tmp42E;struct Cyc_Absyn_Exp*e2=_tmp42F;struct Cyc_Absyn_Exp*e3=_tmp430;
# 3202
Cyc_Toc_subst_pattern_vars(env,e1);_tmp42C=e2;_tmp42D=e3;goto _LL8;}case 27U: _LL7: _tmp42C=((struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct*)_tmp409)->f2;_tmp42D=((struct Cyc_Absyn_Comprehension_e_Absyn_Raw_exp_struct*)_tmp409)->f3;_LL8: {struct Cyc_Absyn_Exp*e1=_tmp42C;struct Cyc_Absyn_Exp*e2=_tmp42D;
_tmp42A=e1;_tmp42B=e2;goto _LLA;}case 7U: _LL9: _tmp42A=((struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct*)_tmp409)->f1;_tmp42B=((struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct*)_tmp409)->f2;_LLA: {struct Cyc_Absyn_Exp*e1=_tmp42A;struct Cyc_Absyn_Exp*e2=_tmp42B;
_tmp428=e1;_tmp429=e2;goto _LLC;}case 8U: _LLB: _tmp428=((struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct*)_tmp409)->f1;_tmp429=((struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct*)_tmp409)->f2;_LLC: {struct Cyc_Absyn_Exp*e1=_tmp428;struct Cyc_Absyn_Exp*e2=_tmp429;
_tmp426=e1;_tmp427=e2;goto _LLE;}case 23U: _LLD: _tmp426=((struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct*)_tmp409)->f1;_tmp427=((struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct*)_tmp409)->f2;_LLE: {struct Cyc_Absyn_Exp*e1=_tmp426;struct Cyc_Absyn_Exp*e2=_tmp427;
_tmp424=e1;_tmp425=e2;goto _LL10;}case 34U: _LLF: _tmp424=((struct Cyc_Absyn_Spawn_e_Absyn_Raw_exp_struct*)_tmp409)->f1;_tmp425=((struct Cyc_Absyn_Spawn_e_Absyn_Raw_exp_struct*)_tmp409)->f2;_LL10: {struct Cyc_Absyn_Exp*e1=_tmp424;struct Cyc_Absyn_Exp*e2=_tmp425;
_tmp422=e1;_tmp423=e2;goto _LL12;}case 9U: _LL11: _tmp422=((struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct*)_tmp409)->f1;_tmp423=((struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct*)_tmp409)->f2;_LL12: {struct Cyc_Absyn_Exp*e1=_tmp422;struct Cyc_Absyn_Exp*e2=_tmp423;
# 3209
Cyc_Toc_subst_pattern_vars(env,e1);_tmp421=e2;goto _LL14;}case 42U: _LL13: _tmp421=((struct Cyc_Absyn_Extension_e_Absyn_Raw_exp_struct*)_tmp409)->f1;_LL14: {struct Cyc_Absyn_Exp*e=_tmp421;
_tmp420=e;goto _LL16;}case 39U: _LL15: _tmp420=((struct Cyc_Absyn_Tagcheck_e_Absyn_Raw_exp_struct*)_tmp409)->f1;_LL16: {struct Cyc_Absyn_Exp*e=_tmp420;
_tmp41F=e;goto _LL18;}case 11U: _LL17: _tmp41F=((struct Cyc_Absyn_Throw_e_Absyn_Raw_exp_struct*)_tmp409)->f1;_LL18: {struct Cyc_Absyn_Exp*e=_tmp41F;
_tmp41E=e;goto _LL1A;}case 12U: _LL19: _tmp41E=((struct Cyc_Absyn_NoInstantiate_e_Absyn_Raw_exp_struct*)_tmp409)->f1;_LL1A: {struct Cyc_Absyn_Exp*e=_tmp41E;
_tmp41D=e;goto _LL1C;}case 13U: _LL1B: _tmp41D=((struct Cyc_Absyn_Instantiate_e_Absyn_Raw_exp_struct*)_tmp409)->f1;_LL1C: {struct Cyc_Absyn_Exp*e=_tmp41D;
_tmp41C=e;goto _LL1E;}case 14U: _LL1D: _tmp41C=((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp409)->f2;_LL1E: {struct Cyc_Absyn_Exp*e=_tmp41C;
_tmp41B=e;goto _LL20;}case 18U: _LL1F: _tmp41B=((struct Cyc_Absyn_Sizeofexp_e_Absyn_Raw_exp_struct*)_tmp409)->f1;_LL20: {struct Cyc_Absyn_Exp*e=_tmp41B;
_tmp41A=e;goto _LL22;}case 20U: _LL21: _tmp41A=((struct Cyc_Absyn_Deref_e_Absyn_Raw_exp_struct*)_tmp409)->f1;_LL22: {struct Cyc_Absyn_Exp*e=_tmp41A;
_tmp419=e;goto _LL24;}case 21U: _LL23: _tmp419=((struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct*)_tmp409)->f1;_LL24: {struct Cyc_Absyn_Exp*e=_tmp419;
_tmp418=e;goto _LL26;}case 22U: _LL25: _tmp418=((struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct*)_tmp409)->f1;_LL26: {struct Cyc_Absyn_Exp*e=_tmp418;
_tmp417=e;goto _LL28;}case 28U: _LL27: _tmp417=((struct Cyc_Absyn_ComprehensionNoinit_e_Absyn_Raw_exp_struct*)_tmp409)->f1;_LL28: {struct Cyc_Absyn_Exp*e=_tmp417;
_tmp416=e;goto _LL2A;}case 15U: _LL29: _tmp416=((struct Cyc_Absyn_Address_e_Absyn_Raw_exp_struct*)_tmp409)->f1;_LL2A: {struct Cyc_Absyn_Exp*e=_tmp416;
Cyc_Toc_subst_pattern_vars(env,e);goto _LL0;}case 35U: _LL2B: _tmp414=(((struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct*)_tmp409)->f1).rgn;_tmp415=(((struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct*)_tmp409)->f1).num_elts;_LL2C: {struct Cyc_Absyn_Exp*eopt=_tmp414;struct Cyc_Absyn_Exp*e=_tmp415;
_tmp412=eopt;_tmp413=e;goto _LL2E;}case 16U: _LL2D: _tmp412=((struct Cyc_Absyn_New_e_Absyn_Raw_exp_struct*)_tmp409)->f1;_tmp413=((struct Cyc_Absyn_New_e_Absyn_Raw_exp_struct*)_tmp409)->f2;_LL2E: {struct Cyc_Absyn_Exp*eopt=_tmp412;struct Cyc_Absyn_Exp*e=_tmp413;
# 3224
if(eopt != 0)Cyc_Toc_subst_pattern_vars(env,eopt);Cyc_Toc_subst_pattern_vars(env,e);
goto _LL0;}case 3U: _LL2F: _tmp411=((struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct*)_tmp409)->f2;_LL30: {struct Cyc_List_List*es=_tmp411;
_tmp410=es;goto _LL32;}case 31U: _LL31: _tmp410=((struct Cyc_Absyn_Datatype_e_Absyn_Raw_exp_struct*)_tmp409)->f1;_LL32: {struct Cyc_List_List*es=_tmp410;
_tmp40F=es;goto _LL34;}case 24U: _LL33: _tmp40F=((struct Cyc_Absyn_Tuple_e_Absyn_Raw_exp_struct*)_tmp409)->f1;_LL34: {struct Cyc_List_List*es=_tmp40F;
({void(*_tmp8BA)(void(*f)(struct Cyc_List_List*,struct Cyc_Absyn_Exp*),struct Cyc_List_List*env,struct Cyc_List_List*x)=({void(*_tmp43B)(void(*f)(struct Cyc_List_List*,struct Cyc_Absyn_Exp*),struct Cyc_List_List*env,struct Cyc_List_List*x)=(void(*)(void(*f)(struct Cyc_List_List*,struct Cyc_Absyn_Exp*),struct Cyc_List_List*env,struct Cyc_List_List*x))Cyc_List_iter_c;_tmp43B;});struct Cyc_List_List*_tmp8B9=env;_tmp8BA(Cyc_Toc_subst_pattern_vars,_tmp8B9,es);});goto _LL0;}case 37U: _LL35: _tmp40E=((struct Cyc_Absyn_UnresolvedMem_e_Absyn_Raw_exp_struct*)_tmp409)->f2;_LL36: {struct Cyc_List_List*dles=_tmp40E;
_tmp40D=dles;goto _LL38;}case 29U: _LL37: _tmp40D=((struct Cyc_Absyn_Aggregate_e_Absyn_Raw_exp_struct*)_tmp409)->f3;_LL38: {struct Cyc_List_List*dles=_tmp40D;
_tmp40C=dles;goto _LL3A;}case 30U: _LL39: _tmp40C=((struct Cyc_Absyn_AnonStruct_e_Absyn_Raw_exp_struct*)_tmp409)->f2;_LL3A: {struct Cyc_List_List*dles=_tmp40C;
_tmp40B=dles;goto _LL3C;}case 26U: _LL3B: _tmp40B=((struct Cyc_Absyn_Array_e_Absyn_Raw_exp_struct*)_tmp409)->f1;_LL3C: {struct Cyc_List_List*dles=_tmp40B;
_tmp40A=dles;goto _LL3E;}case 25U: _LL3D: _tmp40A=((struct Cyc_Absyn_CompoundLit_e_Absyn_Raw_exp_struct*)_tmp409)->f2;_LL3E: {struct Cyc_List_List*dles=_tmp40A;
# 3234
for(0;dles != 0;dles=dles->tl){
struct _tuple20*_stmttmp54=(struct _tuple20*)dles->hd;struct Cyc_Absyn_Exp*_tmp43C;_LL47: _tmp43C=_stmttmp54->f2;_LL48: {struct Cyc_Absyn_Exp*e=_tmp43C;
Cyc_Toc_subst_pattern_vars(env,e);}}
# 3238
goto _LL0;}default: _LL3F: _LL40:
 goto _LL0;}_LL0:;}struct _tuple40{struct Cyc_Toc_Env*f1;struct _fat_ptr*f2;struct Cyc_Absyn_Stmt*f3;};
# 3067
static struct Cyc_Absyn_Stmt*Cyc_Toc_compile_decision_tree(struct _RegionHandle*rgn,struct Cyc_Toc_Env*nv,struct Cyc_List_List**decls,struct Cyc_List_List**bodies,void*dopt,struct Cyc_List_List*lscs,struct _tuple1*v){
# 3256
void*_tmp43D=dopt;void*_tmp440;struct Cyc_List_List*_tmp43F;struct Cyc_List_List*_tmp43E;struct Cyc_Tcpat_Rhs*_tmp441;if(_tmp43D == 0){_LL1: _LL2:
# 3258
 return Cyc_Absyn_skip_stmt(0U);}else{switch(*((int*)_tmp43D)){case 0U: _LL3: _LL4:
 return Cyc_Toc_throw_match_stmt();case 1U: _LL5: _tmp441=((struct Cyc_Tcpat_Success_Tcpat_Decision_struct*)_tmp43D)->f1;_LL6: {struct Cyc_Tcpat_Rhs*rhs=_tmp441;
# 3262
for(0;lscs != 0;lscs=lscs->tl){
struct _tuple36*_stmttmp55=(struct _tuple36*)lscs->hd;struct Cyc_Absyn_Switch_clause*_tmp445;struct _fat_ptr*_tmp444;struct _fat_ptr*_tmp443;int*_tmp442;_LLA: _tmp442=(int*)& _stmttmp55->f1;_tmp443=_stmttmp55->f2;_tmp444=_stmttmp55->f3;_tmp445=_stmttmp55->f4;_LLB: {int*already_emitted=_tmp442;struct _fat_ptr*init_lab=_tmp443;struct _fat_ptr*code_lab=_tmp444;struct Cyc_Absyn_Switch_clause*sc=_tmp445;
struct Cyc_Absyn_Stmt*body=sc->body;
if(body == rhs->rhs){
# 3267
if(*already_emitted)
return Cyc_Absyn_goto_stmt(init_lab,0U);
# 3267
*already_emitted=1;{
# 3270
struct Cyc_List_List*newdecls=0;
# 3272
struct Cyc_Absyn_Stmt*init_opt=({struct _RegionHandle*_tmp8BC=rgn;struct Cyc_Absyn_Exp*_tmp8BB=
Cyc_Absyn_var_exp(v,0U);
# 3272
Cyc_Toc_extract_pattern_vars(_tmp8BC,& nv,& newdecls,_tmp8BB,sc->pattern);});
# 3276
struct Cyc_Absyn_Stmt*res=sc->body;
{struct Cyc_List_List*ds=newdecls;for(0;ds != 0;ds=ds->tl){
struct _tuple38 _stmttmp56=*((struct _tuple38*)ds->hd);struct Cyc_Absyn_Vardecl*_tmp447;struct Cyc_Absyn_Vardecl*_tmp446;_LLD: _tmp446=_stmttmp56.f1;_tmp447=_stmttmp56.f2;_LLE: {struct Cyc_Absyn_Vardecl*oldv=_tmp446;struct Cyc_Absyn_Vardecl*newv=_tmp447;
({struct Cyc_List_List*_tmp8BE=({struct Cyc_List_List*_tmp449=_region_malloc(rgn,sizeof(*_tmp449));({struct _tuple38*_tmp8BD=({struct _tuple38*_tmp448=_region_malloc(rgn,sizeof(*_tmp448));_tmp448->f1=oldv,_tmp448->f2=newv;_tmp448;});_tmp449->hd=_tmp8BD;}),_tmp449->tl=*decls;_tmp449;});*decls=_tmp8BE;});
({struct Cyc_Absyn_Exp*_tmp8BF=Cyc_Absyn_varb_exp((void*)({struct Cyc_Absyn_Local_b_Absyn_Binding_struct*_tmp44A=_cycalloc(sizeof(*_tmp44A));_tmp44A->tag=4U,_tmp44A->f1=newv;_tmp44A;}),0U);oldv->initializer=_tmp8BF;});
((struct Cyc_Absyn_Exp*)_check_null(oldv->initializer))->topt=newv->type;
oldv->type=newv->type;
res=({struct Cyc_Absyn_Decl*_tmp8C1=({struct Cyc_Absyn_Decl*_tmp44C=_cycalloc(sizeof(*_tmp44C));({void*_tmp8C0=(void*)({struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct*_tmp44B=_cycalloc(sizeof(*_tmp44B));_tmp44B->tag=0U,_tmp44B->f1=oldv;_tmp44B;});_tmp44C->r=_tmp8C0;}),_tmp44C->loc=0U;_tmp44C;});Cyc_Absyn_decl_stmt(_tmp8C1,res,0U);});}}}
# 3285
res=Cyc_Absyn_label_stmt(code_lab,res,0U);
if(init_opt != 0)
res=Cyc_Absyn_seq_stmt(init_opt,res,0U);
# 3286
res=
# 3288
Cyc_Absyn_label_stmt(init_lab,res,0U);
({struct Cyc_List_List*_tmp8C3=({struct Cyc_List_List*_tmp44E=_region_malloc(rgn,sizeof(*_tmp44E));({struct _tuple40*_tmp8C2=({struct _tuple40*_tmp44D=_region_malloc(rgn,sizeof(*_tmp44D));_tmp44D->f1=nv,_tmp44D->f2=code_lab,_tmp44D->f3=body;_tmp44D;});_tmp44E->hd=_tmp8C2;}),_tmp44E->tl=*bodies;_tmp44E;});*bodies=_tmp8C3;});
return res;}}}}
# 3293
({void*_tmp44F=0U;({int(*_tmp8C5)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp451)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Toc_toc_impos;_tmp451;});struct _fat_ptr _tmp8C4=({const char*_tmp450="couldn't find rhs!";_tag_fat(_tmp450,sizeof(char),19U);});_tmp8C5(_tmp8C4,_tag_fat(_tmp44F,sizeof(void*),0U));});});}default: _LL7: _tmp43E=((struct Cyc_Tcpat_SwitchDec_Tcpat_Decision_struct*)_tmp43D)->f1;_tmp43F=((struct Cyc_Tcpat_SwitchDec_Tcpat_Decision_struct*)_tmp43D)->f2;_tmp440=(void*)((struct Cyc_Tcpat_SwitchDec_Tcpat_Decision_struct*)_tmp43D)->f3;_LL8: {struct Cyc_List_List*symbolic_path=_tmp43E;struct Cyc_List_List*switches=_tmp43F;void*other_decision=_tmp440;
# 3296
struct Cyc_Absyn_Stmt*res=Cyc_Toc_compile_decision_tree(rgn,nv,decls,bodies,other_decision,lscs,v);
# 3298
struct Cyc_Absyn_Exp*p=({struct Cyc_List_List*_tmp8C6=Cyc_List_rev(symbolic_path);Cyc_Toc_compile_path(_tmp8C6,Cyc_Absyn_var_exp(v,0U));});
struct Cyc_List_List*ss=Cyc_List_rev(switches);
# 3301
struct _tuple37 _stmttmp57=Cyc_Toc_admits_switch(ss);void*_tmp453;int _tmp452;_LL10: _tmp452=_stmttmp57.f1;_tmp453=_stmttmp57.f2;_LL11: {int allows_switch=_tmp452;void*test_kind=_tmp453;
if(allows_switch > 1){
# 3305
struct Cyc_List_List*new_lscs=({struct Cyc_List_List*_tmp45F=_cycalloc(sizeof(*_tmp45F));
({struct Cyc_Absyn_Switch_clause*_tmp8C8=({struct Cyc_Absyn_Switch_clause*_tmp45E=_cycalloc(sizeof(*_tmp45E));({struct Cyc_Absyn_Pat*_tmp8C7=Cyc_Absyn_new_pat((void*)& Cyc_Absyn_Wild_p_val,0U);_tmp45E->pattern=_tmp8C7;}),_tmp45E->pat_vars=0,_tmp45E->where_clause=0,_tmp45E->body=res,_tmp45E->loc=0U;_tmp45E;});_tmp45F->hd=_tmp8C8;}),_tmp45F->tl=0;_tmp45F;});
# 3308
for(0;ss != 0;ss=ss->tl){
struct _tuple33 _stmttmp58=*((struct _tuple33*)ss->hd);void*_tmp455;void*_tmp454;_LL13: _tmp454=_stmttmp58.f1;_tmp455=_stmttmp58.f2;_LL14: {void*pat_test=_tmp454;void*dec_tree=_tmp455;
# 3311
struct Cyc_Absyn_Pat*case_exp=Cyc_Toc_compile_pat_test_as_case(pat_test);
# 3313
struct Cyc_Absyn_Stmt*s=Cyc_Toc_compile_decision_tree(rgn,nv,decls,bodies,dec_tree,lscs,v);
# 3315
new_lscs=({struct Cyc_List_List*_tmp457=_cycalloc(sizeof(*_tmp457));({struct Cyc_Absyn_Switch_clause*_tmp8C9=({struct Cyc_Absyn_Switch_clause*_tmp456=_cycalloc(sizeof(*_tmp456));_tmp456->pattern=case_exp,_tmp456->pat_vars=0,_tmp456->where_clause=0,_tmp456->body=s,_tmp456->loc=0U;_tmp456;});_tmp457->hd=_tmp8C9;}),_tmp457->tl=new_lscs;_tmp457;});}}
# 3317
{void*_tmp458=test_kind;struct _fat_ptr*_tmp459;switch(*((int*)_tmp458)){case 1U: _LL16: _LL17:
# 3319
 LOOP1: {
void*_stmttmp59=p->r;void*_tmp45A=_stmttmp59;struct Cyc_Absyn_Exp*_tmp45B;struct Cyc_Absyn_Exp*_tmp45C;switch(*((int*)_tmp45A)){case 14U: _LL1F: _tmp45C=((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp45A)->f2;_LL20: {struct Cyc_Absyn_Exp*e=_tmp45C;
p=e;goto LOOP1;}case 20U: _LL21: _tmp45B=((struct Cyc_Absyn_Deref_e_Absyn_Raw_exp_struct*)_tmp45A)->f1;_LL22: {struct Cyc_Absyn_Exp*e=_tmp45B;
p=e;goto _LL1E;}default: _LL23: _LL24:
 goto _LL1E;}_LL1E:;}
# 3325
p=Cyc_Absyn_deref_exp(({void*_tmp8CA=Cyc_Absyn_cstar_type(Cyc_Absyn_sint_type,Cyc_Toc_mt_tq);Cyc_Toc_cast_it(_tmp8CA,p);}),0U);goto _LL15;case 3U: _LL18: _tmp459=((struct Cyc_Toc_TaggedUnionTest_Toc_TestKind_struct*)_tmp458)->f1;_LL19: {struct _fat_ptr*f=_tmp459;
# 3327
p=({struct Cyc_Absyn_Exp*_tmp8CB=Cyc_Toc_member_exp(p,f,0U);Cyc_Toc_member_exp(_tmp8CB,Cyc_Toc_tag_sp,0U);});goto _LL15;}case 2U: _LL1A: _LL1B:
 goto _LL1D;default: _LL1C: _LL1D:
 goto _LL15;}_LL15:;}
# 3331
res=Cyc_Absyn_new_stmt((void*)({struct Cyc_Absyn_Switch_s_Absyn_Raw_stmt_struct*_tmp45D=_cycalloc(sizeof(*_tmp45D));_tmp45D->tag=10U,_tmp45D->f1=p,_tmp45D->f2=new_lscs,_tmp45D->f3=0;_tmp45D;}),0U);}else{
# 3335
void*_tmp460=test_kind;struct Cyc_Absyn_Exp*_tmp461;if(((struct Cyc_Toc_WhereTest_Toc_TestKind_struct*)_tmp460)->tag == 2U){_LL26: _tmp461=((struct Cyc_Toc_WhereTest_Toc_TestKind_struct*)_tmp460)->f1;_LL27: {struct Cyc_Absyn_Exp*e=_tmp461;
# 3337
struct Cyc_List_List*_tmp462=ss;struct Cyc_Tcpat_Rhs*_tmp464;void*_tmp463;if(_tmp462 != 0){if(((struct Cyc_Tcpat_Success_Tcpat_Decision_struct*)((struct _tuple33*)((struct Cyc_List_List*)_tmp462)->hd)->f2)->tag == 1U){if(((struct Cyc_List_List*)_tmp462)->tl == 0){_LL2B: _tmp463=((struct _tuple33*)_tmp462->hd)->f1;_tmp464=((struct Cyc_Tcpat_Success_Tcpat_Decision_struct*)((struct _tuple33*)_tmp462->hd)->f2)->f1;_LL2C: {void*pat_test=_tmp463;struct Cyc_Tcpat_Rhs*rhs=_tmp464;
# 3343
for(0;lscs != 0;lscs=lscs->tl){
struct _tuple36*_stmttmp5A=(struct _tuple36*)lscs->hd;struct Cyc_Absyn_Switch_clause*_tmp468;struct _fat_ptr*_tmp467;struct _fat_ptr*_tmp466;int*_tmp465;_LL30: _tmp465=(int*)& _stmttmp5A->f1;_tmp466=_stmttmp5A->f2;_tmp467=_stmttmp5A->f3;_tmp468=_stmttmp5A->f4;_LL31: {int*already_emitted=_tmp465;struct _fat_ptr*init_lab=_tmp466;struct _fat_ptr*code_lab=_tmp467;struct Cyc_Absyn_Switch_clause*sc=_tmp468;
struct Cyc_Absyn_Stmt*body=sc->body;
if(body == rhs->rhs){
# 3348
if(*already_emitted)
return Cyc_Absyn_goto_stmt(init_lab,0U);
# 3348
*already_emitted=1;{
# 3351
struct Cyc_List_List*newdecls=0;
# 3353
struct Cyc_Absyn_Stmt*init_opt=({struct _RegionHandle*_tmp8CD=rgn;struct Cyc_Absyn_Exp*_tmp8CC=Cyc_Absyn_var_exp(v,0U);Cyc_Toc_extract_pattern_vars(_tmp8CD,& nv,& newdecls,_tmp8CC,sc->pattern);});
# 3357
struct Cyc_Absyn_Stmt*r=sc->body;
{struct Cyc_List_List*ds=newdecls;for(0;ds != 0;ds=ds->tl){
struct _tuple38 _stmttmp5B=*((struct _tuple38*)ds->hd);struct Cyc_Absyn_Vardecl*_tmp46A;struct Cyc_Absyn_Vardecl*_tmp469;_LL33: _tmp469=_stmttmp5B.f1;_tmp46A=_stmttmp5B.f2;_LL34: {struct Cyc_Absyn_Vardecl*oldv=_tmp469;struct Cyc_Absyn_Vardecl*newv=_tmp46A;
({struct Cyc_List_List*_tmp8CF=({struct Cyc_List_List*_tmp46C=_region_malloc(rgn,sizeof(*_tmp46C));({struct _tuple38*_tmp8CE=({struct _tuple38*_tmp46B=_region_malloc(rgn,sizeof(*_tmp46B));_tmp46B->f1=oldv,_tmp46B->f2=newv;_tmp46B;});_tmp46C->hd=_tmp8CE;}),_tmp46C->tl=*decls;_tmp46C;});*decls=_tmp8CF;});
({struct Cyc_Absyn_Exp*_tmp8D0=Cyc_Absyn_varb_exp((void*)({struct Cyc_Absyn_Local_b_Absyn_Binding_struct*_tmp46D=_cycalloc(sizeof(*_tmp46D));_tmp46D->tag=4U,_tmp46D->f1=newv;_tmp46D;}),0U);oldv->initializer=_tmp8D0;});
((struct Cyc_Absyn_Exp*)_check_null(oldv->initializer))->topt=newv->type;
oldv->type=newv->type;
r=({struct Cyc_Absyn_Decl*_tmp8D2=({struct Cyc_Absyn_Decl*_tmp46F=_cycalloc(sizeof(*_tmp46F));({void*_tmp8D1=(void*)({struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct*_tmp46E=_cycalloc(sizeof(*_tmp46E));_tmp46E->tag=0U,_tmp46E->f1=oldv;_tmp46E;});_tmp46F->r=_tmp8D1;}),_tmp46F->loc=0U;_tmp46F;});Cyc_Absyn_decl_stmt(_tmp8D2,r,0U);});}}}
# 3366
r=Cyc_Absyn_label_stmt(code_lab,r,0U);
# 3370
Cyc_Toc_subst_pattern_vars(*decls,e);
Cyc_Toc_exp_to_c(nv,e);
r=Cyc_Absyn_ifthenelse_stmt(e,r,res,0U);
if(init_opt != 0)
r=Cyc_Absyn_seq_stmt(init_opt,r,0U);
# 3373
r=
# 3375
Cyc_Absyn_label_stmt(init_lab,r,0U);
({struct Cyc_List_List*_tmp8D4=({struct Cyc_List_List*_tmp471=_region_malloc(rgn,sizeof(*_tmp471));({struct _tuple40*_tmp8D3=({struct _tuple40*_tmp470=_region_malloc(rgn,sizeof(*_tmp470));_tmp470->f1=nv,_tmp470->f2=code_lab,_tmp470->f3=body;_tmp470;});_tmp471->hd=_tmp8D3;}),_tmp471->tl=*bodies;_tmp471;});*bodies=_tmp8D4;});
return r;}}}}
# 3380
({void*_tmp472=0U;({int(*_tmp8D6)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp474)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Toc_toc_impos;_tmp474;});struct _fat_ptr _tmp8D5=({const char*_tmp473="couldn't find rhs!";_tag_fat(_tmp473,sizeof(char),19U);});_tmp8D6(_tmp8D5,_tag_fat(_tmp472,sizeof(void*),0U));});});}}else{goto _LL2D;}}else{goto _LL2D;}}else{_LL2D: _LL2E:
({void*_tmp475=0U;({int(*_tmp8D8)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp477)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Toc_toc_impos;_tmp477;});struct _fat_ptr _tmp8D7=({const char*_tmp476="bad where clause in match compiler";_tag_fat(_tmp476,sizeof(char),35U);});_tmp8D8(_tmp8D7,_tag_fat(_tmp475,sizeof(void*),0U));});});}_LL2A:;}}else{_LL28: _LL29:
# 3385
 for(0;ss != 0;ss=ss->tl){
struct _tuple33 _stmttmp5C=*((struct _tuple33*)ss->hd);void*_tmp479;void*_tmp478;_LL36: _tmp478=_stmttmp5C.f1;_tmp479=_stmttmp5C.f2;_LL37: {void*pat_test=_tmp478;void*dec_tree=_tmp479;
struct Cyc_Absyn_Exp*test_exp=Cyc_Toc_compile_pat_test(p,pat_test);
struct Cyc_Absyn_Stmt*s=Cyc_Toc_compile_decision_tree(rgn,nv,decls,bodies,dec_tree,lscs,v);
res=Cyc_Absyn_ifthenelse_stmt(test_exp,s,res,0U);}}}_LL25:;}
# 3393
return res;}}}}_LL0:;}
# 3067
static struct Cyc_Toc_Env**Cyc_Toc_find_case_env(struct Cyc_List_List*bodies,struct Cyc_Absyn_Stmt*s){
# 3405
for(0;bodies != 0;bodies=bodies->tl){
struct _tuple40*_stmttmp5D=(struct _tuple40*)bodies->hd;struct Cyc_Absyn_Stmt*_tmp47B;struct Cyc_Toc_Env**_tmp47A;_LL1: _tmp47A=(struct Cyc_Toc_Env**)& _stmttmp5D->f1;_tmp47B=_stmttmp5D->f3;_LL2: {struct Cyc_Toc_Env**nv=_tmp47A;struct Cyc_Absyn_Stmt*s2=_tmp47B;
if(s2 == s)return nv;}}
# 3411
return 0;}
# 3067
static void Cyc_Toc_xlate_switch(struct Cyc_Toc_Env*nv,struct Cyc_Absyn_Stmt*whole_s,struct Cyc_Absyn_Exp*e,struct Cyc_List_List*scs,void*dopt){
# 3418
void*t=(void*)_check_null(e->topt);
Cyc_Toc_exp_to_c(nv,e);{
# 3421
struct _tuple30 _stmttmp5E=Cyc_Toc_temp_var_and_exp();struct Cyc_Absyn_Exp*_tmp47D;struct _tuple1*_tmp47C;_LL1: _tmp47C=_stmttmp5E.f1;_tmp47D=_stmttmp5E.f2;_LL2: {struct _tuple1*v=_tmp47C;struct Cyc_Absyn_Exp*path=_tmp47D;
struct _fat_ptr*end_l=Cyc_Toc_fresh_label();
struct _RegionHandle _tmp47E=_new_region("rgn");struct _RegionHandle*rgn=& _tmp47E;_push_region(rgn);
{struct Cyc_Toc_Env*_tmp47F=Cyc_Toc_share_env(rgn,nv);struct Cyc_Toc_Env*nv=_tmp47F;
# 3427
struct Cyc_List_List*lscs=({
struct Cyc_List_List*(*_tmp8DB)(struct _RegionHandle*,struct _tuple36*(*f)(struct _RegionHandle*,struct Cyc_Absyn_Switch_clause*),struct _RegionHandle*env,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp488)(struct _RegionHandle*,struct _tuple36*(*f)(struct _RegionHandle*,struct Cyc_Absyn_Switch_clause*),struct _RegionHandle*env,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct _RegionHandle*,struct _tuple36*(*f)(struct _RegionHandle*,struct Cyc_Absyn_Switch_clause*),struct _RegionHandle*env,struct Cyc_List_List*x))Cyc_List_rmap_c;_tmp488;});struct _RegionHandle*_tmp8DA=rgn;struct _RegionHandle*_tmp8D9=rgn;_tmp8DB(_tmp8DA,Cyc_Toc_gen_labels,_tmp8D9,scs);});
# 3432
struct Cyc_List_List*mydecls=0;
struct Cyc_List_List*mybodies=0;
struct Cyc_Absyn_Stmt*test_tree=Cyc_Toc_compile_decision_tree(rgn,nv,& mydecls,& mybodies,dopt,lscs,v);
# 3440
{struct Cyc_List_List*lscs2=lscs;for(0;lscs2 != 0;lscs2=lscs2->tl){
struct _tuple36*_stmttmp5F=(struct _tuple36*)lscs2->hd;struct Cyc_Absyn_Switch_clause*_tmp481;struct _fat_ptr*_tmp480;_LL4: _tmp480=_stmttmp5F->f3;_tmp481=_stmttmp5F->f4;_LL5: {struct _fat_ptr*body_lab=_tmp480;struct Cyc_Absyn_Switch_clause*body_sc=_tmp481;
struct Cyc_Absyn_Stmt*s=body_sc->body;
# 3444
struct Cyc_Toc_Env**envp=Cyc_Toc_find_case_env(mybodies,s);
# 3447
if(envp == 0)continue;{struct Cyc_Toc_Env*env=*envp;
# 3450
if(lscs2->tl != 0){
struct _tuple36*_stmttmp60=(struct _tuple36*)((struct Cyc_List_List*)_check_null(lscs2->tl))->hd;struct Cyc_Absyn_Switch_clause*_tmp483;struct _fat_ptr*_tmp482;_LL7: _tmp482=_stmttmp60->f3;_tmp483=_stmttmp60->f4;_LL8: {struct _fat_ptr*fallthru_lab=_tmp482;struct Cyc_Absyn_Switch_clause*next_sc=_tmp483;
# 3454
struct Cyc_Toc_Env**next_case_env=Cyc_Toc_find_case_env(mybodies,next_sc->body);
# 3459
if(next_case_env == 0)
({struct Cyc_Toc_Env*_tmp8DC=Cyc_Toc_last_switchclause_env(rgn,env,end_l);Cyc_Toc_stmt_to_c(_tmp8DC,s);});else{
# 3463
struct Cyc_List_List*vs=0;
if(next_sc->pat_vars != 0){
vs=Cyc_Tcutil_filter_nulls((Cyc_List_split((struct Cyc_List_List*)((struct Cyc_Core_Opt*)_check_null(next_sc->pat_vars))->v)).f1);
vs=({struct Cyc_List_List*(*_tmp8DF)(struct Cyc_Absyn_Vardecl*(*f)(struct Cyc_List_List*,struct Cyc_Absyn_Vardecl*),struct Cyc_List_List*env,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp484)(struct Cyc_Absyn_Vardecl*(*f)(struct Cyc_List_List*,struct Cyc_Absyn_Vardecl*),struct Cyc_List_List*env,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct Cyc_Absyn_Vardecl*(*f)(struct Cyc_List_List*,struct Cyc_Absyn_Vardecl*),struct Cyc_List_List*env,struct Cyc_List_List*x))Cyc_List_map_c;_tmp484;});struct Cyc_Absyn_Vardecl*(*_tmp8DE)(struct Cyc_List_List*l,struct Cyc_Absyn_Vardecl*k)=({struct Cyc_Absyn_Vardecl*(*_tmp485)(struct Cyc_List_List*l,struct Cyc_Absyn_Vardecl*k)=(struct Cyc_Absyn_Vardecl*(*)(struct Cyc_List_List*l,struct Cyc_Absyn_Vardecl*k))Cyc_List_assoc;_tmp485;});struct Cyc_List_List*_tmp8DD=mydecls;_tmp8DF(_tmp8DE,_tmp8DD,Cyc_List_imp_rev(vs));});}
# 3464
({struct Cyc_Toc_Env*_tmp8E0=
# 3468
Cyc_Toc_non_last_switchclause_env(rgn,env,end_l,fallthru_lab,vs);
# 3464
Cyc_Toc_stmt_to_c(_tmp8E0,s);});}}}else{
# 3472
({struct Cyc_Toc_Env*_tmp8E1=Cyc_Toc_last_switchclause_env(rgn,env,end_l);Cyc_Toc_stmt_to_c(_tmp8E1,s);});}}}}}{
# 3476
struct Cyc_Absyn_Stmt*res=({struct Cyc_Absyn_Stmt*_tmp8E3=test_tree;Cyc_Absyn_seq_stmt(_tmp8E3,({struct _fat_ptr*_tmp8E2=end_l;Cyc_Absyn_label_stmt(_tmp8E2,Cyc_Toc_skip_stmt_dl(),0U);}),0U);});
# 3478
for(0;mydecls != 0;mydecls=((struct Cyc_List_List*)_check_null(mydecls))->tl){
struct _tuple38 _stmttmp61=*((struct _tuple38*)((struct Cyc_List_List*)_check_null(mydecls))->hd);struct Cyc_Absyn_Vardecl*_tmp486;_LLA: _tmp486=_stmttmp61.f2;_LLB: {struct Cyc_Absyn_Vardecl*vd=_tmp486;
res=({struct Cyc_Absyn_Decl*_tmp8E4=Cyc_Absyn_new_decl((void*)({struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct*_tmp487=_cycalloc(sizeof(*_tmp487));_tmp487->tag=0U,_tmp487->f1=vd;_tmp487;}),0U);Cyc_Absyn_decl_stmt(_tmp8E4,res,0U);});}}
# 3483
({void*_tmp8E8=({struct _tuple1*_tmp8E7=v;void*_tmp8E6=Cyc_Toc_typ_to_c((void*)_check_null(e->topt));struct Cyc_Absyn_Exp*_tmp8E5=e;Cyc_Absyn_declare_stmt(_tmp8E7,_tmp8E6,_tmp8E5,res,0U);})->r;whole_s->r=_tmp8E8;});
_npop_handler(0U);return;}}
# 3424
;_pop_region(rgn);}}}
# 3067
static struct Cyc_Absyn_Stmt*Cyc_Toc_letdecl_to_c(struct Cyc_Toc_Env*nv,struct Cyc_Absyn_Pat*p,void*,void*t,struct Cyc_Absyn_Exp*e,struct Cyc_Absyn_Stmt*s);
# 3491
static void Cyc_Toc_local_decl_to_c(struct Cyc_Toc_Env*nv,struct Cyc_Absyn_Vardecl*vd,struct Cyc_Absyn_Stmt*s);
# 3496
static void Cyc_Toc_fndecl_to_c(struct Cyc_Toc_Env*nv,struct Cyc_Absyn_Fndecl*f,int cinclude);
# 3498
struct Cyc_Absyn_Stmt*Cyc_Toc_make_npop_handler(int n){
return Cyc_Absyn_exp_stmt(({struct Cyc_Absyn_Exp*_tmp489[1U];({struct Cyc_Absyn_Exp*_tmp8E9=Cyc_Absyn_uint_exp((unsigned)(n - 1),0U);_tmp489[0]=_tmp8E9;});({struct Cyc_Absyn_Exp*_tmp8EA=Cyc_Toc__npop_handler_e;Cyc_Toc_fncall_exp_dl(_tmp8EA,_tag_fat(_tmp489,sizeof(struct Cyc_Absyn_Exp*),1U));});}),0U);}
# 3498
void Cyc_Toc_do_npop_before(int n,struct Cyc_Absyn_Stmt*s){
# 3502
if(n > 0)
({void*_tmp8EC=({struct Cyc_Absyn_Stmt*_tmp8EB=Cyc_Toc_make_npop_handler(n);Cyc_Toc_seq_stmt_r(_tmp8EB,Cyc_Absyn_new_stmt(s->r,0U));});s->r=_tmp8EC;});}
# 3498
static void Cyc_Toc_stmt_to_c(struct Cyc_Toc_Env*nv,struct Cyc_Absyn_Stmt*s){
# 3508
while(1){
void*_stmttmp62=s->r;void*_tmp48A=_stmttmp62;void*_tmp48D;struct Cyc_List_List*_tmp48C;struct Cyc_Absyn_Stmt*_tmp48B;struct Cyc_Absyn_Stmt*_tmp48F;struct Cyc_Absyn_Decl*_tmp48E;struct Cyc_Absyn_Switch_clause**_tmp491;struct Cyc_List_List*_tmp490;void*_tmp494;struct Cyc_List_List*_tmp493;struct Cyc_Absyn_Exp*_tmp492;struct Cyc_Absyn_Stmt*_tmp496;struct _fat_ptr*_tmp495;struct Cyc_Absyn_Stmt*_tmp498;struct Cyc_Absyn_Exp*_tmp497;struct Cyc_Absyn_Exp*_tmp49A;struct Cyc_Absyn_Stmt*_tmp499;struct Cyc_Absyn_Stmt*_tmp49E;struct Cyc_Absyn_Exp*_tmp49D;struct Cyc_Absyn_Exp*_tmp49C;struct Cyc_Absyn_Exp*_tmp49B;struct Cyc_Absyn_Stmt*_tmp4A1;struct Cyc_Absyn_Stmt*_tmp4A0;struct Cyc_Absyn_Exp*_tmp49F;struct Cyc_Absyn_Stmt*_tmp4A3;struct Cyc_Absyn_Stmt*_tmp4A2;struct Cyc_Absyn_Exp*_tmp4A4;struct Cyc_Absyn_Exp*_tmp4A5;switch(*((int*)_tmp48A)){case 0U: _LL1: _LL2:
 return;case 1U: _LL3: _tmp4A5=((struct Cyc_Absyn_Exp_s_Absyn_Raw_stmt_struct*)_tmp48A)->f1;_LL4: {struct Cyc_Absyn_Exp*e=_tmp4A5;
Cyc_Toc_exp_to_c(nv,e);return;}case 3U: if(((struct Cyc_Absyn_Return_s_Absyn_Raw_stmt_struct*)_tmp48A)->f1 == 0){_LL5: _LL6:
({int _tmp8ED=Cyc_Toc_get_npop(s);Cyc_Toc_do_npop_before(_tmp8ED,s);});return;}else{_LL7: _tmp4A4=((struct Cyc_Absyn_Return_s_Absyn_Raw_stmt_struct*)_tmp48A)->f1;_LL8: {struct Cyc_Absyn_Exp*e=_tmp4A4;
# 3515
void*t=Cyc_Toc_typ_to_c((void*)_check_null(((struct Cyc_Absyn_Exp*)_check_null(e))->topt));
Cyc_Toc_exp_to_c(nv,e);{
int npop=Cyc_Toc_get_npop(s);
if(npop > 0){
struct _tuple1*x=Cyc_Toc_temp_var();
struct Cyc_Absyn_Stmt*retn_stmt=Cyc_Absyn_return_stmt(Cyc_Absyn_var_exp(x,0U),0U);
Cyc_Toc_do_npop_before(npop,retn_stmt);
({void*_tmp8EE=(Cyc_Absyn_declare_stmt(x,t,e,retn_stmt,0U))->r;s->r=_tmp8EE;});}
# 3518
return;}}}case 2U: _LL9: _tmp4A2=((struct Cyc_Absyn_Seq_s_Absyn_Raw_stmt_struct*)_tmp48A)->f1;_tmp4A3=((struct Cyc_Absyn_Seq_s_Absyn_Raw_stmt_struct*)_tmp48A)->f2;_LLA: {struct Cyc_Absyn_Stmt*s1=_tmp4A2;struct Cyc_Absyn_Stmt*s2=_tmp4A3;
# 3526
Cyc_Toc_stmt_to_c(nv,s1);
s=s2;
continue;}case 4U: _LLB: _tmp49F=((struct Cyc_Absyn_IfThenElse_s_Absyn_Raw_stmt_struct*)_tmp48A)->f1;_tmp4A0=((struct Cyc_Absyn_IfThenElse_s_Absyn_Raw_stmt_struct*)_tmp48A)->f2;_tmp4A1=((struct Cyc_Absyn_IfThenElse_s_Absyn_Raw_stmt_struct*)_tmp48A)->f3;_LLC: {struct Cyc_Absyn_Exp*e=_tmp49F;struct Cyc_Absyn_Stmt*s1=_tmp4A0;struct Cyc_Absyn_Stmt*s2=_tmp4A1;
# 3530
Cyc_Toc_exp_to_c(nv,e);
Cyc_Toc_stmt_to_c(nv,s1);
s=s2;
continue;}case 9U: _LLD: _tmp49B=((struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct*)_tmp48A)->f1;_tmp49C=(((struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct*)_tmp48A)->f2).f1;_tmp49D=(((struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct*)_tmp48A)->f3).f1;_tmp49E=((struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct*)_tmp48A)->f4;_LLE: {struct Cyc_Absyn_Exp*e1=_tmp49B;struct Cyc_Absyn_Exp*e2=_tmp49C;struct Cyc_Absyn_Exp*e3=_tmp49D;struct Cyc_Absyn_Stmt*s2=_tmp49E;
# 3535
Cyc_Toc_exp_to_c(nv,e1);
Cyc_Toc_exp_to_c(nv,e2);
_tmp499=s2;_tmp49A=e3;goto _LL10;}case 14U: _LLF: _tmp499=((struct Cyc_Absyn_Do_s_Absyn_Raw_stmt_struct*)_tmp48A)->f1;_tmp49A=(((struct Cyc_Absyn_Do_s_Absyn_Raw_stmt_struct*)_tmp48A)->f2).f1;_LL10: {struct Cyc_Absyn_Stmt*s2=_tmp499;struct Cyc_Absyn_Exp*e=_tmp49A;
_tmp497=e;_tmp498=s2;goto _LL12;}case 5U: _LL11: _tmp497=(((struct Cyc_Absyn_While_s_Absyn_Raw_stmt_struct*)_tmp48A)->f1).f1;_tmp498=((struct Cyc_Absyn_While_s_Absyn_Raw_stmt_struct*)_tmp48A)->f2;_LL12: {struct Cyc_Absyn_Exp*e=_tmp497;struct Cyc_Absyn_Stmt*s2=_tmp498;
# 3540
Cyc_Toc_exp_to_c(nv,e);{
struct _RegionHandle _tmp4A6=_new_region("temp");struct _RegionHandle*temp=& _tmp4A6;_push_region(temp);({struct Cyc_Toc_Env*_tmp8EF=Cyc_Toc_loop_env(temp,nv);Cyc_Toc_stmt_to_c(_tmp8EF,s2);});
_npop_handler(0U);return;
# 3541
;_pop_region(temp);}}case 6U: _LL13: _LL14: {
# 3544
struct _fat_ptr**_tmp4A7;_LL24: _tmp4A7=nv->break_lab;_LL25: {struct _fat_ptr**b=_tmp4A7;
if(b != 0)
({void*_tmp8F0=Cyc_Toc_goto_stmt_r(*b);s->r=_tmp8F0;});
# 3545
({int _tmp8F1=
# 3548
Cyc_Toc_get_npop(s);
# 3545
Cyc_Toc_do_npop_before(_tmp8F1,s);});
# 3549
return;}}case 7U: _LL15: _LL16: {
# 3551
struct _fat_ptr**_tmp4A8;_LL27: _tmp4A8=nv->continue_lab;_LL28: {struct _fat_ptr**c=_tmp4A8;
if(c != 0)
({void*_tmp8F2=Cyc_Toc_goto_stmt_r(*c);s->r=_tmp8F2;});
# 3552
goto _LL18;}}case 8U: _LL17: _LL18:
# 3556
({int _tmp8F3=Cyc_Toc_get_npop(s);Cyc_Toc_do_npop_before(_tmp8F3,s);});
return;case 13U: _LL19: _tmp495=((struct Cyc_Absyn_Label_s_Absyn_Raw_stmt_struct*)_tmp48A)->f1;_tmp496=((struct Cyc_Absyn_Label_s_Absyn_Raw_stmt_struct*)_tmp48A)->f2;_LL1A: {struct _fat_ptr*lab=_tmp495;struct Cyc_Absyn_Stmt*s1=_tmp496;
s=s1;continue;}case 10U: _LL1B: _tmp492=((struct Cyc_Absyn_Switch_s_Absyn_Raw_stmt_struct*)_tmp48A)->f1;_tmp493=((struct Cyc_Absyn_Switch_s_Absyn_Raw_stmt_struct*)_tmp48A)->f2;_tmp494=(void*)((struct Cyc_Absyn_Switch_s_Absyn_Raw_stmt_struct*)_tmp48A)->f3;_LL1C: {struct Cyc_Absyn_Exp*e=_tmp492;struct Cyc_List_List*scs=_tmp493;void*dec_tree_opt=_tmp494;
# 3560
Cyc_Toc_xlate_switch(nv,s,e,scs,dec_tree_opt);
return;}case 11U: _LL1D: _tmp490=((struct Cyc_Absyn_Fallthru_s_Absyn_Raw_stmt_struct*)_tmp48A)->f1;_tmp491=((struct Cyc_Absyn_Fallthru_s_Absyn_Raw_stmt_struct*)_tmp48A)->f2;_LL1E: {struct Cyc_List_List*es=_tmp490;struct Cyc_Absyn_Switch_clause**dest_clause=_tmp491;
# 3563
struct Cyc_Toc_FallthruInfo*_tmp4A9;_LL2A: _tmp4A9=nv->fallthru_info;_LL2B: {struct Cyc_Toc_FallthruInfo*fi=_tmp4A9;
if(fi == 0)
({void*_tmp4AA=0U;({int(*_tmp8F5)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp4AC)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Toc_toc_impos;_tmp4AC;});struct _fat_ptr _tmp8F4=({const char*_tmp4AB="fallthru in unexpected place";_tag_fat(_tmp4AB,sizeof(char),29U);});_tmp8F5(_tmp8F4,_tag_fat(_tmp4AA,sizeof(void*),0U));});});{
# 3564
struct Cyc_Toc_FallthruInfo _stmttmp63=*fi;struct Cyc_List_List*_tmp4AE;struct _fat_ptr*_tmp4AD;_LL2D: _tmp4AD=_stmttmp63.label;_tmp4AE=_stmttmp63.binders;_LL2E: {struct _fat_ptr*l=_tmp4AD;struct Cyc_List_List*vs=_tmp4AE;
# 3567
struct Cyc_Absyn_Stmt*s2=Cyc_Absyn_goto_stmt(l,0U);
# 3569
({int _tmp8F6=Cyc_Toc_get_npop(s);Cyc_Toc_do_npop_before(_tmp8F6,s2);});{
struct Cyc_List_List*vs2=Cyc_List_rev(vs);
struct Cyc_List_List*es2=Cyc_List_rev(es);
for(0;vs2 != 0;(vs2=vs2->tl,es2=es2->tl)){
Cyc_Toc_exp_to_c(nv,(struct Cyc_Absyn_Exp*)((struct Cyc_List_List*)_check_null(es2))->hd);
s2=({struct Cyc_Absyn_Stmt*_tmp8F8=({struct Cyc_Absyn_Exp*_tmp8F7=Cyc_Absyn_varb_exp((void*)({struct Cyc_Absyn_Pat_b_Absyn_Binding_struct*_tmp4AF=_cycalloc(sizeof(*_tmp4AF));_tmp4AF->tag=5U,_tmp4AF->f1=(struct Cyc_Absyn_Vardecl*)vs2->hd;_tmp4AF;}),0U);Cyc_Absyn_assign_stmt(_tmp8F7,(struct Cyc_Absyn_Exp*)es2->hd,0U);});Cyc_Absyn_seq_stmt(_tmp8F8,s2,0U);});}
# 3577
s->r=s2->r;
return;}}}}}case 12U: _LL1F: _tmp48E=((struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct*)_tmp48A)->f1;_tmp48F=((struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct*)_tmp48A)->f2;_LL20: {struct Cyc_Absyn_Decl*d=_tmp48E;struct Cyc_Absyn_Stmt*s1=_tmp48F;
# 3583
{void*_stmttmp64=d->r;void*_tmp4B0=_stmttmp64;struct Cyc_Absyn_Exp*_tmp4B3;struct Cyc_Absyn_Vardecl*_tmp4B2;struct Cyc_Absyn_Tvar*_tmp4B1;struct Cyc_Absyn_Fndecl*_tmp4B4;struct Cyc_List_List*_tmp4B5;void*_tmp4B8;struct Cyc_Absyn_Exp*_tmp4B7;struct Cyc_Absyn_Pat*_tmp4B6;struct Cyc_Absyn_Vardecl*_tmp4B9;switch(*((int*)_tmp4B0)){case 0U: _LL30: _tmp4B9=((struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct*)_tmp4B0)->f1;_LL31: {struct Cyc_Absyn_Vardecl*vd=_tmp4B9;
Cyc_Toc_local_decl_to_c(nv,vd,s1);goto _LL2F;}case 2U: _LL32: _tmp4B6=((struct Cyc_Absyn_Let_d_Absyn_Raw_decl_struct*)_tmp4B0)->f1;_tmp4B7=((struct Cyc_Absyn_Let_d_Absyn_Raw_decl_struct*)_tmp4B0)->f3;_tmp4B8=(void*)((struct Cyc_Absyn_Let_d_Absyn_Raw_decl_struct*)_tmp4B0)->f4;_LL33: {struct Cyc_Absyn_Pat*p=_tmp4B6;struct Cyc_Absyn_Exp*e=_tmp4B7;void*dec_tree=_tmp4B8;
# 3593
{void*_stmttmp65=p->r;void*_tmp4BA=_stmttmp65;struct Cyc_Absyn_Vardecl*_tmp4BB;if(((struct Cyc_Absyn_Var_p_Absyn_Raw_pat_struct*)_tmp4BA)->tag == 1U){if(((struct Cyc_Absyn_Wild_p_Absyn_Raw_pat_struct*)((struct Cyc_Absyn_Pat*)((struct Cyc_Absyn_Var_p_Absyn_Raw_pat_struct*)_tmp4BA)->f2)->r)->tag == 0U){_LL3D: _tmp4BB=((struct Cyc_Absyn_Var_p_Absyn_Raw_pat_struct*)_tmp4BA)->f1;_LL3E: {struct Cyc_Absyn_Vardecl*vd=_tmp4BB;
# 3595
if(Cyc_Absyn_var_may_appear_exp(vd->name,e)){
struct Cyc_Absyn_Vardecl*new_vd=({struct _tuple1*_tmp8FA=Cyc_Toc_temp_var();void*_tmp8F9=vd->type;Cyc_Absyn_new_vardecl(0U,_tmp8FA,_tmp8F9,e);});
({struct Cyc_Absyn_Exp*_tmp8FB=Cyc_Absyn_varb_exp((void*)({struct Cyc_Absyn_Local_b_Absyn_Binding_struct*_tmp4BC=_cycalloc(sizeof(*_tmp4BC));_tmp4BC->tag=4U,_tmp4BC->f1=new_vd;_tmp4BC;}),0U);vd->initializer=_tmp8FB;});
((struct Cyc_Absyn_Exp*)_check_null(vd->initializer))->topt=new_vd->type;
({void*_tmp902=({struct Cyc_Absyn_Decl*_tmp901=({void*_tmp8FC=(void*)({struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct*_tmp4BD=_cycalloc(sizeof(*_tmp4BD));_tmp4BD->tag=0U,_tmp4BD->f1=new_vd;_tmp4BD;});Cyc_Absyn_new_decl(_tmp8FC,s->loc);});struct Cyc_Absyn_Stmt*_tmp900=({
struct Cyc_Absyn_Decl*_tmp8FF=({void*_tmp8FD=(void*)({struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct*_tmp4BE=_cycalloc(sizeof(*_tmp4BE));_tmp4BE->tag=0U,_tmp4BE->f1=vd;_tmp4BE;});Cyc_Absyn_new_decl(_tmp8FD,s->loc);});struct Cyc_Absyn_Stmt*_tmp8FE=s1;Cyc_Absyn_decl_stmt(_tmp8FF,_tmp8FE,s->loc);});
# 3599
Cyc_Absyn_decl_stmt(_tmp901,_tmp900,s->loc);})->r;s->r=_tmp902;});}else{
# 3603
vd->initializer=e;
({void*_tmp906=({struct Cyc_Absyn_Decl*_tmp905=({void*_tmp903=(void*)({struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct*_tmp4BF=_cycalloc(sizeof(*_tmp4BF));_tmp4BF->tag=0U,_tmp4BF->f1=vd;_tmp4BF;});Cyc_Absyn_new_decl(_tmp903,s->loc);});struct Cyc_Absyn_Stmt*_tmp904=s1;Cyc_Absyn_decl_stmt(_tmp905,_tmp904,s->loc);})->r;s->r=_tmp906;});}
# 3606
Cyc_Toc_stmt_to_c(nv,s);
goto _LL3C;}}else{goto _LL3F;}}else{_LL3F: _LL40:
# 3612
({void*_tmp907=(Cyc_Toc_letdecl_to_c(nv,p,dec_tree,(void*)_check_null(e->topt),e,s1))->r;s->r=_tmp907;});
goto _LL3C;}_LL3C:;}
# 3635 "toc.cyc"
goto _LL2F;}case 3U: _LL34: _tmp4B5=((struct Cyc_Absyn_Letv_d_Absyn_Raw_decl_struct*)_tmp4B0)->f1;_LL35: {struct Cyc_List_List*vds=_tmp4B5;
# 3639
struct Cyc_List_List*rvds=Cyc_List_rev(vds);
if(rvds == 0)
({void*_tmp4C0=0U;({int(*_tmp909)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp4C2)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp4C2;});struct _fat_ptr _tmp908=({const char*_tmp4C1="empty Letv_d";_tag_fat(_tmp4C1,sizeof(char),13U);});_tmp909(_tmp908,_tag_fat(_tmp4C0,sizeof(void*),0U));});});
# 3640
({void*_tmp90A=(void*)({struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct*_tmp4C3=_cycalloc(sizeof(*_tmp4C3));
# 3642
_tmp4C3->tag=0U,_tmp4C3->f1=(struct Cyc_Absyn_Vardecl*)rvds->hd;_tmp4C3;});
# 3640
d->r=_tmp90A;});
# 3643
rvds=rvds->tl;
for(0;rvds != 0;rvds=rvds->tl){
struct Cyc_Absyn_Decl*d2=Cyc_Absyn_new_decl((void*)({struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct*_tmp4C4=_cycalloc(sizeof(*_tmp4C4));_tmp4C4->tag=0U,_tmp4C4->f1=(struct Cyc_Absyn_Vardecl*)rvds->hd;_tmp4C4;}),0U);
({void*_tmp90C=({struct Cyc_Absyn_Decl*_tmp90B=d2;Cyc_Absyn_decl_stmt(_tmp90B,Cyc_Absyn_new_stmt(s->r,0U),0U);})->r;s->r=_tmp90C;});}
# 3648
Cyc_Toc_stmt_to_c(nv,s);
goto _LL2F;}case 1U: _LL36: _tmp4B4=((struct Cyc_Absyn_Fn_d_Absyn_Raw_decl_struct*)_tmp4B0)->f1;_LL37: {struct Cyc_Absyn_Fndecl*fd=_tmp4B4;
# 3651
Cyc_Toc_fndecl_to_c(nv,fd,0);
Cyc_Toc_stmt_to_c(nv,s1);
goto _LL2F;}case 4U: _LL38: _tmp4B1=((struct Cyc_Absyn_Region_d_Absyn_Raw_decl_struct*)_tmp4B0)->f1;_tmp4B2=((struct Cyc_Absyn_Region_d_Absyn_Raw_decl_struct*)_tmp4B0)->f2;_tmp4B3=((struct Cyc_Absyn_Region_d_Absyn_Raw_decl_struct*)_tmp4B0)->f3;_LL39: {struct Cyc_Absyn_Tvar*tv=_tmp4B1;struct Cyc_Absyn_Vardecl*vd=_tmp4B2;struct Cyc_Absyn_Exp*open_exp_opt=_tmp4B3;
# 3655
struct Cyc_Absyn_Stmt*body=s1;
# 3657
void*rh_struct_typ=Cyc_Absyn_strct(Cyc_Toc__RegionHandle_sp);
void*rh_struct_ptr_typ=Cyc_Absyn_cstar_type(rh_struct_typ,Cyc_Toc_mt_tq);
struct _tuple30 _stmttmp66=Cyc_Toc_temp_var_and_exp();struct Cyc_Absyn_Exp*_tmp4C6;struct _tuple1*_tmp4C5;_LL42: _tmp4C5=_stmttmp66.f1;_tmp4C6=_stmttmp66.f2;_LL43: {struct _tuple1*rh_var=_tmp4C5;struct Cyc_Absyn_Exp*rh_exp=_tmp4C6;
struct _tuple1*x_var=vd->name;
struct Cyc_Absyn_Exp*x_exp=Cyc_Absyn_var_exp(x_var,0U);
# 3663
Cyc_Toc_stmt_to_c(nv,body);
if(Cyc_Absyn_no_regions)
({void*_tmp910=({struct _tuple1*_tmp90F=x_var;void*_tmp90E=rh_struct_ptr_typ;struct Cyc_Absyn_Exp*_tmp90D=
Cyc_Absyn_uint_exp(0U,0U);
# 3665
Cyc_Absyn_declare_stmt(_tmp90F,_tmp90E,_tmp90D,body,0U);})->r;s->r=_tmp910;});else{
# 3667
if((unsigned)open_exp_opt){
# 3669
Cyc_Toc_exp_to_c(nv,open_exp_opt);{
struct Cyc_Absyn_Exp*arg=Cyc_Absyn_address_exp(({struct Cyc_Absyn_Exp*_tmp912=open_exp_opt;Cyc_Absyn_aggrarrow_exp(_tmp912,({struct _fat_ptr*_tmp4C8=_cycalloc(sizeof(*_tmp4C8));({struct _fat_ptr _tmp911=({const char*_tmp4C7="h";_tag_fat(_tmp4C7,sizeof(char),2U);});*_tmp4C8=_tmp911;});_tmp4C8;}),0U);}),0U);
({void*_tmp913=(Cyc_Absyn_declare_stmt(x_var,rh_struct_ptr_typ,arg,body,0U))->r;s->r=_tmp913;});}}else{
# 3678
({void*_tmp920=({struct _tuple1*_tmp91F=rh_var;void*_tmp91E=rh_struct_typ;struct Cyc_Absyn_Exp*_tmp91D=({struct Cyc_Absyn_Exp*_tmp4C9[1U];({struct Cyc_Absyn_Exp*_tmp914=
# 3680
Cyc_Absyn_string_exp(Cyc_Absynpp_qvar2string(x_var),0U);_tmp4C9[0]=_tmp914;});({struct Cyc_Absyn_Exp*_tmp915=Cyc_Toc__new_region_e;Cyc_Toc_fncall_exp_dl(_tmp915,_tag_fat(_tmp4C9,sizeof(struct Cyc_Absyn_Exp*),1U));});});
# 3678
Cyc_Absyn_declare_stmt(_tmp91F,_tmp91E,_tmp91D,({
# 3681
struct _tuple1*_tmp91C=x_var;void*_tmp91B=rh_struct_ptr_typ;struct Cyc_Absyn_Exp*_tmp91A=Cyc_Absyn_address_exp(rh_exp,0U);Cyc_Absyn_declare_stmt(_tmp91C,_tmp91B,_tmp91A,({
struct Cyc_Absyn_Stmt*_tmp919=Cyc_Absyn_exp_stmt(({struct Cyc_Absyn_Exp*_tmp4CA[1U];_tmp4CA[0]=x_exp;({struct Cyc_Absyn_Exp*_tmp916=Cyc_Toc__push_region_e;Cyc_Toc_fncall_exp_dl(_tmp916,_tag_fat(_tmp4CA,sizeof(struct Cyc_Absyn_Exp*),1U));});}),0U);Cyc_Absyn_seq_stmt(_tmp919,({
struct Cyc_Absyn_Stmt*_tmp918=body;Cyc_Absyn_seq_stmt(_tmp918,
Cyc_Absyn_exp_stmt(({struct Cyc_Absyn_Exp*_tmp4CB[1U];_tmp4CB[0]=x_exp;({struct Cyc_Absyn_Exp*_tmp917=Cyc_Toc__pop_region_e;Cyc_Toc_fncall_exp_dl(_tmp917,_tag_fat(_tmp4CB,sizeof(struct Cyc_Absyn_Exp*),1U));});}),0U),0U);}),0U);}),0U);}),0U);})->r;
# 3678
s->r=_tmp920;});}}
# 3687
return;}}default: _LL3A: _LL3B:
({void*_tmp4CC=0U;({int(*_tmp922)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp4CE)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp4CE;});struct _fat_ptr _tmp921=({const char*_tmp4CD="bad nested declaration within function";_tag_fat(_tmp4CD,sizeof(char),39U);});_tmp922(_tmp921,_tag_fat(_tmp4CC,sizeof(void*),0U));});});}_LL2F:;}
# 3690
return;}default: _LL21: _tmp48B=((struct Cyc_Absyn_TryCatch_s_Absyn_Raw_stmt_struct*)_tmp48A)->f1;_tmp48C=((struct Cyc_Absyn_TryCatch_s_Absyn_Raw_stmt_struct*)_tmp48A)->f2;_tmp48D=(void*)((struct Cyc_Absyn_TryCatch_s_Absyn_Raw_stmt_struct*)_tmp48A)->f3;_LL22: {struct Cyc_Absyn_Stmt*body=_tmp48B;struct Cyc_List_List*scs=_tmp48C;void*dec_tree=_tmp48D;
# 3704 "toc.cyc"
struct _tuple30 _stmttmp67=Cyc_Toc_temp_var_and_exp();struct Cyc_Absyn_Exp*_tmp4D0;struct _tuple1*_tmp4CF;_LL45: _tmp4CF=_stmttmp67.f1;_tmp4D0=_stmttmp67.f2;_LL46: {struct _tuple1*h_var=_tmp4CF;struct Cyc_Absyn_Exp*h_exp=_tmp4D0;
struct _tuple30 _stmttmp68=Cyc_Toc_temp_var_and_exp();struct Cyc_Absyn_Exp*_tmp4D2;struct _tuple1*_tmp4D1;_LL48: _tmp4D1=_stmttmp68.f1;_tmp4D2=_stmttmp68.f2;_LL49: {struct _tuple1*e_var=_tmp4D1;struct Cyc_Absyn_Exp*e_exp=_tmp4D2;
struct _tuple30 _stmttmp69=Cyc_Toc_temp_var_and_exp();struct Cyc_Absyn_Exp*_tmp4D4;struct _tuple1*_tmp4D3;_LL4B: _tmp4D3=_stmttmp69.f1;_tmp4D4=_stmttmp69.f2;_LL4C: {struct _tuple1*was_thrown_var=_tmp4D3;struct Cyc_Absyn_Exp*was_thrown_exp=_tmp4D4;
void*h_typ=Cyc_Absyn_strct(Cyc_Toc__handler_cons_sp);
void*e_typ=Cyc_Toc_typ_to_c(Cyc_Absyn_exn_type());
void*was_thrown_typ=Cyc_Toc_typ_to_c(Cyc_Absyn_sint_type);
# 3711
e_exp->topt=e_typ;{
struct _RegionHandle _tmp4D5=_new_region("temp");struct _RegionHandle*temp=& _tmp4D5;_push_region(temp);
# 3714
Cyc_Toc_stmt_to_c(nv,body);{
struct Cyc_Absyn_Stmt*tryandpop_stmt=({struct Cyc_Absyn_Stmt*_tmp924=body;Cyc_Absyn_seq_stmt(_tmp924,
Cyc_Absyn_exp_stmt(({void*_tmp4D9=0U;({struct Cyc_Absyn_Exp*_tmp923=Cyc_Toc__pop_handler_e;Cyc_Toc_fncall_exp_dl(_tmp923,_tag_fat(_tmp4D9,sizeof(struct Cyc_Absyn_Exp*),0U));});}),0U),0U);});
# 3718
struct Cyc_Absyn_Stmt*handler_stmt=Cyc_Absyn_new_stmt((void*)({struct Cyc_Absyn_Switch_s_Absyn_Raw_stmt_struct*_tmp4D8=_cycalloc(sizeof(*_tmp4D8));_tmp4D8->tag=10U,_tmp4D8->f1=e_exp,_tmp4D8->f2=scs,_tmp4D8->f3=dec_tree;_tmp4D8;}),0U);
# 3720
Cyc_Toc_stmt_to_c(nv,handler_stmt);{
# 3723
struct Cyc_Absyn_Exp*setjmp_call=({struct Cyc_Absyn_Exp*_tmp4D7[1U];({struct Cyc_Absyn_Exp*_tmp925=
Cyc_Toc_member_exp(h_exp,Cyc_Toc_handler_sp,0U);_tmp4D7[0]=_tmp925;});({struct Cyc_Absyn_Exp*_tmp926=Cyc_Toc_setjmp_e;Cyc_Toc_fncall_exp_dl(_tmp926,_tag_fat(_tmp4D7,sizeof(struct Cyc_Absyn_Exp*),1U));});});
struct Cyc_Absyn_Stmt*pushhandler_call=
Cyc_Absyn_exp_stmt(({struct Cyc_Absyn_Exp*_tmp4D6[1U];({struct Cyc_Absyn_Exp*_tmp927=Cyc_Absyn_address_exp(h_exp,0U);_tmp4D6[0]=_tmp927;});({struct Cyc_Absyn_Exp*_tmp928=Cyc_Toc__push_handler_e;Cyc_Toc_fncall_exp_dl(_tmp928,_tag_fat(_tmp4D6,sizeof(struct Cyc_Absyn_Exp*),1U));});}),0U);
struct Cyc_Absyn_Exp*zero_exp=Cyc_Absyn_int_exp(Cyc_Absyn_Signed,0,0U);
struct Cyc_Absyn_Exp*one_exp=Cyc_Absyn_int_exp(Cyc_Absyn_Signed,1,0U);
({void*_tmp938=({
struct _tuple1*_tmp937=h_var;void*_tmp936=h_typ;Cyc_Absyn_declare_stmt(_tmp937,_tmp936,0,({
struct Cyc_Absyn_Stmt*_tmp935=pushhandler_call;Cyc_Absyn_seq_stmt(_tmp935,({
struct _tuple1*_tmp934=was_thrown_var;void*_tmp933=was_thrown_typ;struct Cyc_Absyn_Exp*_tmp932=zero_exp;Cyc_Absyn_declare_stmt(_tmp934,_tmp933,_tmp932,({
struct Cyc_Absyn_Stmt*_tmp931=({struct Cyc_Absyn_Exp*_tmp92A=setjmp_call;struct Cyc_Absyn_Stmt*_tmp929=
Cyc_Absyn_assign_stmt(was_thrown_exp,one_exp,0U);
# 3733
Cyc_Absyn_ifthenelse_stmt(_tmp92A,_tmp929,
# 3735
Cyc_Toc_skip_stmt_dl(),0U);});
# 3733
Cyc_Absyn_seq_stmt(_tmp931,({
# 3736
struct Cyc_Absyn_Exp*_tmp930=Cyc_Absyn_prim1_exp(Cyc_Absyn_Not,was_thrown_exp,0U);struct Cyc_Absyn_Stmt*_tmp92F=tryandpop_stmt;Cyc_Absyn_ifthenelse_stmt(_tmp930,_tmp92F,({
# 3738
struct _tuple1*_tmp92E=e_var;void*_tmp92D=e_typ;struct Cyc_Absyn_Exp*_tmp92C=({
void*_tmp92B=e_typ;Cyc_Toc_cast_it(_tmp92B,Cyc_Toc_get_exn_thrown_expression());});
# 3738
Cyc_Absyn_declare_stmt(_tmp92E,_tmp92D,_tmp92C,handler_stmt,0U);}),0U);}),0U);}),0U);}),0U);}),0U);})->r;
# 3729
s->r=_tmp938;});}}
# 3742
_npop_handler(0U);return;
# 3712
;_pop_region(temp);}}}}}}_LL0:;}}
# 3498 "toc.cyc"
static void Cyc_Toc_stmttypes_to_c(struct Cyc_Absyn_Stmt*s);
# 3752 "toc.cyc"
static void Cyc_Toc_fndecl_to_c(struct Cyc_Toc_Env*nv,struct Cyc_Absyn_Fndecl*f,int cinclude){
(f->i).tvars=0;
(f->i).effect=0;
(f->i).rgn_po=0;
(f->i).requires_clause=0;
(f->i).ensures_clause=0;
({void*_tmp939=Cyc_Toc_typ_to_c((f->i).ret_type);(f->i).ret_type=_tmp939;});
({void*_tmp93A=Cyc_Toc_typ_to_c((void*)_check_null(f->cached_type));f->cached_type=_tmp93A;});{
struct _RegionHandle _tmp4DA=_new_region("frgn");struct _RegionHandle*frgn=& _tmp4DA;_push_region(frgn);
{struct Cyc_Toc_Env*_tmp4DB=Cyc_Toc_share_env(frgn,nv);struct Cyc_Toc_Env*nv=_tmp4DB;
{struct Cyc_List_List*args=(f->i).args;for(0;args != 0;args=args->tl){
struct _tuple1*x=({struct _tuple1*_tmp4DC=_cycalloc(sizeof(*_tmp4DC));_tmp4DC->f1=Cyc_Absyn_Loc_n,_tmp4DC->f2=(*((struct _tuple9*)args->hd)).f1;_tmp4DC;});
({void*_tmp93B=Cyc_Toc_typ_to_c((*((struct _tuple9*)args->hd)).f3);(*((struct _tuple9*)args->hd)).f3=_tmp93B;});}}
# 3768
if(cinclude){
Cyc_Toc_stmttypes_to_c(f->body);
_npop_handler(0U);return;}
# 3768
Cyc_Toc_fn_pop_table=({struct Cyc_Hashtable_Table**_tmp4DE=_cycalloc(sizeof(*_tmp4DE));({
# 3772
struct Cyc_Hashtable_Table*_tmp93E=({struct Cyc_Hashtable_Table*(*_tmp93D)(struct Cyc_Hashtable_Table*t,struct Cyc_Absyn_Fndecl*key)=({struct Cyc_Hashtable_Table*(*_tmp4DD)(struct Cyc_Hashtable_Table*t,struct Cyc_Absyn_Fndecl*key)=(struct Cyc_Hashtable_Table*(*)(struct Cyc_Hashtable_Table*t,struct Cyc_Absyn_Fndecl*key))Cyc_Hashtable_lookup;_tmp4DD;});struct Cyc_Hashtable_Table*_tmp93C=*((struct Cyc_Hashtable_Table**)_check_null(Cyc_Toc_gpop_tables));_tmp93D(_tmp93C,f);});*_tmp4DE=_tmp93E;});_tmp4DE;});
if((unsigned)(f->i).cyc_varargs &&((struct Cyc_Absyn_VarargInfo*)_check_null((f->i).cyc_varargs))->name != 0){
struct Cyc_Absyn_VarargInfo _stmttmp6A=*((struct Cyc_Absyn_VarargInfo*)_check_null((f->i).cyc_varargs));int _tmp4E2;void*_tmp4E1;struct Cyc_Absyn_Tqual _tmp4E0;struct _fat_ptr*_tmp4DF;_LL1: _tmp4DF=_stmttmp6A.name;_tmp4E0=_stmttmp6A.tq;_tmp4E1=_stmttmp6A.type;_tmp4E2=_stmttmp6A.inject;_LL2: {struct _fat_ptr*n=_tmp4DF;struct Cyc_Absyn_Tqual tq=_tmp4E0;void*t=_tmp4E1;int i=_tmp4E2;
void*t2=Cyc_Toc_typ_to_c(Cyc_Absyn_fatptr_type(t,Cyc_Absyn_heap_rgn_type,tq,Cyc_Absyn_false_type));
struct _tuple1*x2=({struct _tuple1*_tmp4E5=_cycalloc(sizeof(*_tmp4E5));_tmp4E5->f1=Cyc_Absyn_Loc_n,_tmp4E5->f2=(struct _fat_ptr*)_check_null(n);_tmp4E5;});
({struct Cyc_List_List*_tmp941=({struct Cyc_List_List*_tmp940=(f->i).args;Cyc_List_append(_tmp940,({struct Cyc_List_List*_tmp4E4=_cycalloc(sizeof(*_tmp4E4));({struct _tuple9*_tmp93F=({struct _tuple9*_tmp4E3=_cycalloc(sizeof(*_tmp4E3));_tmp4E3->f1=n,_tmp4E3->f2=tq,_tmp4E3->f3=t2;_tmp4E3;});_tmp4E4->hd=_tmp93F;}),_tmp4E4->tl=0;_tmp4E4;}));});(f->i).args=_tmp941;});
(f->i).cyc_varargs=0;}}
# 3773
{
# 3781
struct Cyc_List_List*arg_vds=(struct Cyc_List_List*)((struct Cyc_Core_Opt*)_check_null(f->param_vardecls))->v;
# 3773
for(0;arg_vds != 0;arg_vds=arg_vds->tl){
# 3782
({void*_tmp942=Cyc_Toc_typ_to_c(((struct Cyc_Absyn_Vardecl*)arg_vds->hd)->type);((struct Cyc_Absyn_Vardecl*)arg_vds->hd)->type=_tmp942;});}}
({struct Cyc_Toc_Env*_tmp943=Cyc_Toc_clear_toplevel(frgn,nv);Cyc_Toc_stmt_to_c(_tmp943,f->body);});}
# 3761
;_pop_region(frgn);}}
# 3752
static enum Cyc_Absyn_Scope Cyc_Toc_scope_to_c(enum Cyc_Absyn_Scope s){
# 3787
enum Cyc_Absyn_Scope _tmp4E6=s;switch(_tmp4E6){case Cyc_Absyn_Abstract: _LL1: _LL2:
 return Cyc_Absyn_Public;case Cyc_Absyn_ExternC: _LL3: _LL4:
 return Cyc_Absyn_Extern;default: _LL5: _LL6:
 return s;}_LL0:;}struct _tuple41{struct Cyc_Toc_TocState*f1;struct Cyc_Absyn_Aggrdecl**f2;};
# 3752
static int Cyc_Toc_aggrdecl_to_c_body(struct _RegionHandle*d,struct _tuple41*env){
# 3804 "toc.cyc"
struct _tuple41 _stmttmp6B=*env;struct Cyc_Absyn_Aggrdecl*_tmp4E8;struct Cyc_Toc_TocState*_tmp4E7;_LL1: _tmp4E7=_stmttmp6B.f1;_tmp4E8=*_stmttmp6B.f2;_LL2: {struct Cyc_Toc_TocState*s=_tmp4E7;struct Cyc_Absyn_Aggrdecl*ad=_tmp4E8;
struct _tuple1*n=ad->name;
struct Cyc_Toc_TocState _stmttmp6C=*s;struct Cyc_Dict_Dict*_tmp4E9;_LL4: _tmp4E9=_stmttmp6C.aggrs_so_far;_LL5: {struct Cyc_Dict_Dict*aggrs_so_far=_tmp4E9;
int seen_defn_before;
struct _tuple19**dopt=({struct _tuple19**(*_tmp945)(struct Cyc_Dict_Dict d,struct _tuple1*k)=({struct _tuple19**(*_tmp50B)(struct Cyc_Dict_Dict d,struct _tuple1*k)=(struct _tuple19**(*)(struct Cyc_Dict_Dict d,struct _tuple1*k))Cyc_Dict_lookup_opt;_tmp50B;});struct Cyc_Dict_Dict _tmp944=*aggrs_so_far;_tmp945(_tmp944,n);});
if(dopt == 0){
seen_defn_before=0;{
struct _tuple19*v;
if((int)ad->kind == (int)Cyc_Absyn_StructA)
v=({struct _tuple19*_tmp4EA=_region_malloc(d,sizeof(*_tmp4EA));_tmp4EA->f1=ad,({void*_tmp946=Cyc_Absyn_strctq(n);_tmp4EA->f2=_tmp946;});_tmp4EA;});else{
# 3815
v=({struct _tuple19*_tmp4EB=_region_malloc(d,sizeof(*_tmp4EB));_tmp4EB->f1=ad,({void*_tmp947=Cyc_Absyn_unionq_type(n);_tmp4EB->f2=_tmp947;});_tmp4EB;});}
({struct Cyc_Dict_Dict _tmp94B=({struct Cyc_Dict_Dict(*_tmp94A)(struct Cyc_Dict_Dict d,struct _tuple1*k,struct _tuple19*v)=({struct Cyc_Dict_Dict(*_tmp4EC)(struct Cyc_Dict_Dict d,struct _tuple1*k,struct _tuple19*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _tuple1*k,struct _tuple19*v))Cyc_Dict_insert;_tmp4EC;});struct Cyc_Dict_Dict _tmp949=*aggrs_so_far;struct _tuple1*_tmp948=n;_tmp94A(_tmp949,_tmp948,v);});*aggrs_so_far=_tmp94B;});}}else{
# 3818
struct _tuple19*_stmttmp6D=*dopt;void*_tmp4EE;struct Cyc_Absyn_Aggrdecl*_tmp4ED;_LL7: _tmp4ED=_stmttmp6D->f1;_tmp4EE=_stmttmp6D->f2;_LL8: {struct Cyc_Absyn_Aggrdecl*ad2=_tmp4ED;void*t=_tmp4EE;
if(ad2->impl == 0){
({struct Cyc_Dict_Dict _tmp94F=({struct Cyc_Dict_Dict(*_tmp94E)(struct Cyc_Dict_Dict d,struct _tuple1*k,struct _tuple19*v)=({struct Cyc_Dict_Dict(*_tmp4EF)(struct Cyc_Dict_Dict d,struct _tuple1*k,struct _tuple19*v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _tuple1*k,struct _tuple19*v))Cyc_Dict_insert;_tmp4EF;});struct Cyc_Dict_Dict _tmp94D=*aggrs_so_far;struct _tuple1*_tmp94C=n;_tmp94E(_tmp94D,_tmp94C,({struct _tuple19*_tmp4F0=_region_malloc(d,sizeof(*_tmp4F0));_tmp4F0->f1=ad,_tmp4F0->f2=t;_tmp4F0;}));});*aggrs_so_far=_tmp94F;});
seen_defn_before=0;}else{
# 3823
seen_defn_before=1;}}}{
# 3825
struct Cyc_Absyn_Aggrdecl*new_ad=({struct Cyc_Absyn_Aggrdecl*_tmp50A=_cycalloc(sizeof(*_tmp50A));_tmp50A->kind=ad->kind,_tmp50A->sc=Cyc_Absyn_Public,_tmp50A->name=ad->name,_tmp50A->tvs=0,_tmp50A->impl=0,_tmp50A->expected_mem_kind=0,_tmp50A->attributes=ad->attributes;_tmp50A;});
# 3832
if(ad->impl != 0 && !seen_defn_before){
({struct Cyc_Absyn_AggrdeclImpl*_tmp950=({struct Cyc_Absyn_AggrdeclImpl*_tmp4F1=_cycalloc(sizeof(*_tmp4F1));_tmp4F1->exist_vars=0,_tmp4F1->rgn_po=0,_tmp4F1->fields=0,_tmp4F1->tagged=0;_tmp4F1;});new_ad->impl=_tmp950;});{
# 3837
struct Cyc_List_List*new_fields=0;
{struct Cyc_List_List*fields=((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->fields;for(0;fields != 0;fields=fields->tl){
# 3841
struct Cyc_Absyn_Aggrfield*old_field=(struct Cyc_Absyn_Aggrfield*)fields->hd;
void*old_type=old_field->type;
struct Cyc_List_List*old_atts=old_field->attributes;
if(Cyc_Tcutil_kind_leq(& Cyc_Tcutil_ak,Cyc_Tcutil_type_kind(old_type))&&(
(int)ad->kind == (int)Cyc_Absyn_StructA && fields->tl == 0 ||(int)ad->kind == (int)Cyc_Absyn_UnionA)){
# 3855 "toc.cyc"
void*_stmttmp6E=Cyc_Tcutil_compress(old_type);void*_tmp4F2=_stmttmp6E;unsigned _tmp4F6;void*_tmp4F5;struct Cyc_Absyn_Tqual _tmp4F4;void*_tmp4F3;if(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp4F2)->tag == 4U){_LLA: _tmp4F3=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp4F2)->f1).elt_type;_tmp4F4=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp4F2)->f1).tq;_tmp4F5=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp4F2)->f1).zero_term;_tmp4F6=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp4F2)->f1).zt_loc;_LLB: {void*et=_tmp4F3;struct Cyc_Absyn_Tqual tq=_tmp4F4;void*zt=_tmp4F5;unsigned ztl=_tmp4F6;
# 3858
old_type=(void*)({struct Cyc_Absyn_ArrayType_Absyn_Type_struct*_tmp4F7=_cycalloc(sizeof(*_tmp4F7));_tmp4F7->tag=4U,(_tmp4F7->f1).elt_type=et,(_tmp4F7->f1).tq=tq,({struct Cyc_Absyn_Exp*_tmp951=Cyc_Absyn_uint_exp(0U,0U);(_tmp4F7->f1).num_elts=_tmp951;}),(_tmp4F7->f1).zero_term=zt,(_tmp4F7->f1).zt_loc=ztl;_tmp4F7;});
goto _LL9;}}else{_LLC: _LLD:
# 3861
 old_atts=({struct Cyc_List_List*_tmp4F9=_cycalloc(sizeof(*_tmp4F9));({void*_tmp952=(void*)({struct Cyc_Absyn_Aligned_att_Absyn_Attribute_struct*_tmp4F8=_cycalloc(sizeof(*_tmp4F8));_tmp4F8->tag=6U,_tmp4F8->f1=0;_tmp4F8;});_tmp4F9->hd=_tmp952;}),_tmp4F9->tl=old_atts;_tmp4F9;});
old_type=(void*)({struct Cyc_Absyn_ArrayType_Absyn_Type_struct*_tmp4FA=_cycalloc(sizeof(*_tmp4FA));_tmp4FA->tag=4U,({void*_tmp955=Cyc_Toc_void_star_type();(_tmp4FA->f1).elt_type=_tmp955;}),({
struct Cyc_Absyn_Tqual _tmp954=Cyc_Absyn_empty_tqual(0U);(_tmp4FA->f1).tq=_tmp954;}),({
struct Cyc_Absyn_Exp*_tmp953=Cyc_Absyn_uint_exp(0U,0U);(_tmp4FA->f1).num_elts=_tmp953;}),(_tmp4FA->f1).zero_term=Cyc_Absyn_false_type,(_tmp4FA->f1).zt_loc=0U;_tmp4FA;});}_LL9:;}{
# 3844 "toc.cyc"
struct Cyc_Absyn_Aggrfield*new_field=({struct Cyc_Absyn_Aggrfield*_tmp506=_cycalloc(sizeof(*_tmp506));
# 3868 "toc.cyc"
_tmp506->name=old_field->name,_tmp506->tq=Cyc_Toc_mt_tq,({
# 3870
void*_tmp956=Cyc_Toc_typ_to_c(old_type);_tmp506->type=_tmp956;}),_tmp506->width=old_field->width,_tmp506->attributes=old_atts,_tmp506->requires_clause=0;_tmp506;});
# 3878
if(((struct Cyc_Absyn_AggrdeclImpl*)_check_null(ad->impl))->tagged){
void*T=new_field->type;
struct _fat_ptr*f=new_field->name;
struct _fat_ptr s=(struct _fat_ptr)({struct Cyc_String_pa_PrintArg_struct _tmp503=({struct Cyc_String_pa_PrintArg_struct _tmp604;_tmp604.tag=0U,_tmp604.f1=(struct _fat_ptr)((struct _fat_ptr)*(*ad->name).f2);_tmp604;});struct Cyc_String_pa_PrintArg_struct _tmp504=({struct Cyc_String_pa_PrintArg_struct _tmp603;_tmp603.tag=0U,_tmp603.f1=(struct _fat_ptr)((struct _fat_ptr)*f);_tmp603;});void*_tmp501[2U];_tmp501[0]=& _tmp503,_tmp501[1]=& _tmp504;({struct _fat_ptr _tmp957=({const char*_tmp502="_union_%s_%s";_tag_fat(_tmp502,sizeof(char),13U);});Cyc_aprintf(_tmp957,_tag_fat(_tmp501,sizeof(void*),2U));});});
struct _fat_ptr*str=({struct _fat_ptr*_tmp500=_cycalloc(sizeof(*_tmp500));*_tmp500=s;_tmp500;});
struct Cyc_Absyn_Aggrfield*value_field=({struct Cyc_Absyn_Aggrfield*_tmp4FF=_cycalloc(sizeof(*_tmp4FF));_tmp4FF->name=Cyc_Toc_val_sp,_tmp4FF->tq=Cyc_Toc_mt_tq,_tmp4FF->type=T,_tmp4FF->width=0,_tmp4FF->attributes=0,_tmp4FF->requires_clause=0;_tmp4FF;});
struct Cyc_Absyn_Aggrfield*tag_field=({struct Cyc_Absyn_Aggrfield*_tmp4FE=_cycalloc(sizeof(*_tmp4FE));_tmp4FE->name=Cyc_Toc_tag_sp,_tmp4FE->tq=Cyc_Toc_mt_tq,_tmp4FE->type=Cyc_Absyn_sint_type,_tmp4FE->width=0,_tmp4FE->attributes=0,_tmp4FE->requires_clause=0;_tmp4FE;});
struct Cyc_Absyn_Aggrdecl*ad2=({struct _fat_ptr*_tmp958=str;Cyc_Toc_make_c_struct_defn(_tmp958,({struct Cyc_Absyn_Aggrfield*_tmp4FD[2U];_tmp4FD[0]=tag_field,_tmp4FD[1]=value_field;Cyc_List_list(_tag_fat(_tmp4FD,sizeof(struct Cyc_Absyn_Aggrfield*),2U));}));});
Cyc_Toc_result_decls=({struct Cyc_List_List*_tmp4FC=_cycalloc(sizeof(*_tmp4FC));({struct Cyc_Absyn_Decl*_tmp959=Cyc_Absyn_new_decl((void*)({struct Cyc_Absyn_Aggr_d_Absyn_Raw_decl_struct*_tmp4FB=_cycalloc(sizeof(*_tmp4FB));_tmp4FB->tag=5U,_tmp4FB->f1=ad2;_tmp4FB;}),0U);_tmp4FC->hd=_tmp959;}),_tmp4FC->tl=Cyc_Toc_result_decls;_tmp4FC;});
({void*_tmp95A=Cyc_Absyn_strct(str);new_field->type=_tmp95A;});}
# 3878
new_fields=({struct Cyc_List_List*_tmp505=_cycalloc(sizeof(*_tmp505));
# 3889
_tmp505->hd=new_field,_tmp505->tl=new_fields;_tmp505;});}}}
# 3891
({struct Cyc_List_List*_tmp95B=Cyc_List_imp_rev(new_fields);(new_ad->impl)->fields=_tmp95B;});}}
# 3832 "toc.cyc"
if(!seen_defn_before)
# 3895 "toc.cyc"
Cyc_Toc_result_decls=({struct Cyc_List_List*_tmp509=_cycalloc(sizeof(*_tmp509));({struct Cyc_Absyn_Decl*_tmp95D=({struct Cyc_Absyn_Decl*_tmp508=_cycalloc(sizeof(*_tmp508));({void*_tmp95C=(void*)({struct Cyc_Absyn_Aggr_d_Absyn_Raw_decl_struct*_tmp507=_cycalloc(sizeof(*_tmp507));_tmp507->tag=5U,_tmp507->f1=new_ad;_tmp507;});_tmp508->r=_tmp95C;}),_tmp508->loc=0U;_tmp508;});_tmp509->hd=_tmp95D;}),_tmp509->tl=Cyc_Toc_result_decls;_tmp509;});
# 3832 "toc.cyc"
return 0;}}}}
# 3752 "toc.cyc"
static void Cyc_Toc_aggrdecl_to_c(struct Cyc_Absyn_Aggrdecl*ad){
# 3899 "toc.cyc"
({int(*_tmp50C)(struct Cyc_Absyn_Aggrdecl**arg,int(*f)(struct _RegionHandle*,struct _tuple41*))=(int(*)(struct Cyc_Absyn_Aggrdecl**arg,int(*f)(struct _RegionHandle*,struct _tuple41*)))Cyc_Toc_use_toc_state;_tmp50C;})(& ad,Cyc_Toc_aggrdecl_to_c_body);}struct _tuple42{struct Cyc_Toc_TocState*f1;struct Cyc_Absyn_Datatypedecl*f2;};
# 3752 "toc.cyc"
static int Cyc_Toc_datatypedecl_to_c_body(struct _RegionHandle*d,struct _tuple42*env){
# 3929 "toc.cyc"
struct _tuple42 _stmttmp6F=*env;struct Cyc_Absyn_Datatypedecl*_tmp50E;struct Cyc_Set_Set**_tmp50D;_LL1: _tmp50D=(_stmttmp6F.f1)->datatypes_so_far;_tmp50E=_stmttmp6F.f2;_LL2: {struct Cyc_Set_Set**datatypes_so_far=_tmp50D;struct Cyc_Absyn_Datatypedecl*tud=_tmp50E;
struct _tuple1*n=tud->name;
if(tud->fields == 0 ||({int(*_tmp95F)(struct Cyc_Set_Set*s,struct _tuple1*elt)=({int(*_tmp50F)(struct Cyc_Set_Set*s,struct _tuple1*elt)=(int(*)(struct Cyc_Set_Set*s,struct _tuple1*elt))Cyc_Set_member;_tmp50F;});struct Cyc_Set_Set*_tmp95E=*datatypes_so_far;_tmp95F(_tmp95E,n);}))
return 0;
# 3931
({struct Cyc_Set_Set*_tmp963=({
# 3933
struct Cyc_Set_Set*(*_tmp962)(struct _RegionHandle*r,struct Cyc_Set_Set*s,struct _tuple1*elt)=({struct Cyc_Set_Set*(*_tmp510)(struct _RegionHandle*r,struct Cyc_Set_Set*s,struct _tuple1*elt)=(struct Cyc_Set_Set*(*)(struct _RegionHandle*r,struct Cyc_Set_Set*s,struct _tuple1*elt))Cyc_Set_rinsert;_tmp510;});struct _RegionHandle*_tmp961=d;struct Cyc_Set_Set*_tmp960=*datatypes_so_far;_tmp962(_tmp961,_tmp960,n);});
# 3931
*datatypes_so_far=_tmp963;});
# 3934
{struct Cyc_List_List*fields=(struct Cyc_List_List*)((struct Cyc_Core_Opt*)_check_null(tud->fields))->v;for(0;fields != 0;fields=fields->tl){
struct Cyc_Absyn_Datatypefield*f=(struct Cyc_Absyn_Datatypefield*)fields->hd;
# 3937
struct Cyc_List_List*fs=0;
int i=1;
{struct Cyc_List_List*ts=f->typs;for(0;ts != 0;(ts=ts->tl,i ++)){
struct _fat_ptr*fname=Cyc_Absyn_fieldname(i);
struct Cyc_Absyn_Aggrfield*f=({struct Cyc_Absyn_Aggrfield*_tmp512=_cycalloc(sizeof(*_tmp512));_tmp512->name=fname,_tmp512->tq=(*((struct _tuple13*)ts->hd)).f1,({
void*_tmp964=Cyc_Toc_typ_to_c((*((struct _tuple13*)ts->hd)).f2);_tmp512->type=_tmp964;}),_tmp512->width=0,_tmp512->attributes=0,_tmp512->requires_clause=0;_tmp512;});
fs=({struct Cyc_List_List*_tmp511=_cycalloc(sizeof(*_tmp511));_tmp511->hd=f,_tmp511->tl=fs;_tmp511;});}}
# 3945
fs=({struct Cyc_List_List*_tmp514=_cycalloc(sizeof(*_tmp514));({struct Cyc_Absyn_Aggrfield*_tmp966=({struct Cyc_Absyn_Aggrfield*_tmp513=_cycalloc(sizeof(*_tmp513));_tmp513->name=Cyc_Toc_tag_sp,_tmp513->tq=Cyc_Toc_mt_tq,_tmp513->type=Cyc_Absyn_sint_type,_tmp513->width=0,_tmp513->attributes=0,_tmp513->requires_clause=0;_tmp513;});_tmp514->hd=_tmp966;}),({
struct Cyc_List_List*_tmp965=Cyc_List_imp_rev(fs);_tmp514->tl=_tmp965;});_tmp514;});{
struct Cyc_Absyn_Aggrdecl*ad=({struct _fat_ptr*_tmp968=({struct _fat_ptr*_tmp518=_cycalloc(sizeof(*_tmp518));({struct _fat_ptr _tmp967=({const char*_tmp517="";_tag_fat(_tmp517,sizeof(char),1U);});*_tmp518=_tmp967;});_tmp518;});Cyc_Toc_make_c_struct_defn(_tmp968,fs);});
({struct _tuple1*_tmp969=Cyc_Toc_collapse_qvars(f->name,tud->name);ad->name=_tmp969;});
Cyc_Toc_result_decls=({struct Cyc_List_List*_tmp516=_cycalloc(sizeof(*_tmp516));({struct Cyc_Absyn_Decl*_tmp96A=Cyc_Absyn_new_decl((void*)({struct Cyc_Absyn_Aggr_d_Absyn_Raw_decl_struct*_tmp515=_cycalloc(sizeof(*_tmp515));_tmp515->tag=5U,_tmp515->f1=ad;_tmp515;}),0U);_tmp516->hd=_tmp96A;}),_tmp516->tl=Cyc_Toc_result_decls;_tmp516;});}}}
# 3951
return 0;}}
# 3752 "toc.cyc"
static void Cyc_Toc_datatypedecl_to_c(struct Cyc_Absyn_Datatypedecl*tud){
# 3955 "toc.cyc"
({int(*_tmp96B)(struct Cyc_Absyn_Datatypedecl*arg,int(*f)(struct _RegionHandle*,struct _tuple42*))=({int(*_tmp519)(struct Cyc_Absyn_Datatypedecl*arg,int(*f)(struct _RegionHandle*,struct _tuple42*))=(int(*)(struct Cyc_Absyn_Datatypedecl*arg,int(*f)(struct _RegionHandle*,struct _tuple42*)))Cyc_Toc_use_toc_state;_tmp519;});_tmp96B(tud,Cyc_Toc_datatypedecl_to_c_body);});}
# 3752 "toc.cyc"
static int Cyc_Toc_xdatatypedecl_to_c_body(struct _RegionHandle*d,struct _tuple42*env){
# 3976 "toc.cyc"
struct _tuple42 _stmttmp70=*env;struct Cyc_Absyn_Datatypedecl*_tmp51B;struct Cyc_Toc_TocState*_tmp51A;_LL1: _tmp51A=_stmttmp70.f1;_tmp51B=_stmttmp70.f2;_LL2: {struct Cyc_Toc_TocState*s=_tmp51A;struct Cyc_Absyn_Datatypedecl*xd=_tmp51B;
if(xd->fields == 0)
return 0;{
# 3977
struct Cyc_Toc_TocState _stmttmp71=*s;struct Cyc_Dict_Dict*_tmp51C;_LL4: _tmp51C=_stmttmp71.xdatatypes_so_far;_LL5: {struct Cyc_Dict_Dict*xdatatypes_so_far=_tmp51C;
# 3980
struct _tuple1*n=xd->name;
{struct Cyc_List_List*fs=(struct Cyc_List_List*)((struct Cyc_Core_Opt*)_check_null(xd->fields))->v;for(0;fs != 0;fs=fs->tl){
struct Cyc_Absyn_Datatypefield*f=(struct Cyc_Absyn_Datatypefield*)fs->hd;
struct _fat_ptr*fn=(*f->name).f2;
struct Cyc_Absyn_Exp*sz_exp=Cyc_Absyn_uint_exp(_get_fat_size(*fn,sizeof(char)),0U);
void*tag_typ=Cyc_Absyn_array_type(Cyc_Absyn_char_type,Cyc_Toc_mt_tq,sz_exp,Cyc_Absyn_false_type,0U);
# 3987
int*_stmttmp72=({int*(*_tmp96D)(struct Cyc_Dict_Dict d,struct _tuple1*k)=({int*(*_tmp530)(struct Cyc_Dict_Dict d,struct _tuple1*k)=(int*(*)(struct Cyc_Dict_Dict d,struct _tuple1*k))Cyc_Dict_lookup_opt;_tmp530;});struct Cyc_Dict_Dict _tmp96C=*xdatatypes_so_far;_tmp96D(_tmp96C,f->name);});int*_tmp51D=_stmttmp72;if(_tmp51D == 0){_LL7: _LL8: {
# 3989
struct Cyc_Absyn_Exp*initopt=0;
if((int)f->sc != (int)Cyc_Absyn_Extern)
initopt=Cyc_Absyn_string_exp(*fn,0U);{
# 3990
struct Cyc_Absyn_Vardecl*tag_decl=
# 3993
Cyc_Absyn_new_vardecl(0U,f->name,tag_typ,initopt);
tag_decl->sc=f->sc;
Cyc_Toc_result_decls=({struct Cyc_List_List*_tmp51F=_cycalloc(sizeof(*_tmp51F));({struct Cyc_Absyn_Decl*_tmp96E=Cyc_Absyn_new_decl((void*)({struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct*_tmp51E=_cycalloc(sizeof(*_tmp51E));_tmp51E->tag=0U,_tmp51E->f1=tag_decl;_tmp51E;}),0U);_tmp51F->hd=_tmp96E;}),_tmp51F->tl=Cyc_Toc_result_decls;_tmp51F;});
({struct Cyc_Dict_Dict _tmp972=({
struct Cyc_Dict_Dict(*_tmp971)(struct Cyc_Dict_Dict d,struct _tuple1*k,int v)=({struct Cyc_Dict_Dict(*_tmp520)(struct Cyc_Dict_Dict d,struct _tuple1*k,int v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _tuple1*k,int v))Cyc_Dict_insert;_tmp520;});struct Cyc_Dict_Dict _tmp970=*xdatatypes_so_far;struct _tuple1*_tmp96F=f->name;_tmp971(_tmp970,_tmp96F,(int)f->sc != (int)Cyc_Absyn_Extern);});
# 3996
*xdatatypes_so_far=_tmp972;});{
# 3998
struct Cyc_List_List*fields=0;
int i=1;
{struct Cyc_List_List*tqts=f->typs;for(0;tqts != 0;(tqts=tqts->tl,i ++)){
struct _fat_ptr*field_n=({struct _fat_ptr*_tmp526=_cycalloc(sizeof(*_tmp526));({struct _fat_ptr _tmp974=(struct _fat_ptr)({struct Cyc_Int_pa_PrintArg_struct _tmp525=({struct Cyc_Int_pa_PrintArg_struct _tmp605;_tmp605.tag=1U,_tmp605.f1=(unsigned long)i;_tmp605;});void*_tmp523[1U];_tmp523[0]=& _tmp525;({struct _fat_ptr _tmp973=({const char*_tmp524="f%d";_tag_fat(_tmp524,sizeof(char),4U);});Cyc_aprintf(_tmp973,_tag_fat(_tmp523,sizeof(void*),1U));});});*_tmp526=_tmp974;});_tmp526;});
struct Cyc_Absyn_Aggrfield*newf=({struct Cyc_Absyn_Aggrfield*_tmp522=_cycalloc(sizeof(*_tmp522));_tmp522->name=field_n,_tmp522->tq=(*((struct _tuple13*)tqts->hd)).f1,({
void*_tmp975=Cyc_Toc_typ_to_c((*((struct _tuple13*)tqts->hd)).f2);_tmp522->type=_tmp975;}),_tmp522->width=0,_tmp522->attributes=0,_tmp522->requires_clause=0;_tmp522;});
fields=({struct Cyc_List_List*_tmp521=_cycalloc(sizeof(*_tmp521));_tmp521->hd=newf,_tmp521->tl=fields;_tmp521;});}}
# 4006
fields=({struct Cyc_List_List*_tmp528=_cycalloc(sizeof(*_tmp528));({struct Cyc_Absyn_Aggrfield*_tmp978=({struct Cyc_Absyn_Aggrfield*_tmp527=_cycalloc(sizeof(*_tmp527));_tmp527->name=Cyc_Toc_tag_sp,_tmp527->tq=Cyc_Toc_mt_tq,({
void*_tmp977=Cyc_Absyn_cstar_type(Cyc_Absyn_char_type,Cyc_Toc_mt_tq);_tmp527->type=_tmp977;}),_tmp527->width=0,_tmp527->attributes=0,_tmp527->requires_clause=0;_tmp527;});
# 4006
_tmp528->hd=_tmp978;}),({
# 4008
struct Cyc_List_List*_tmp976=Cyc_List_imp_rev(fields);_tmp528->tl=_tmp976;});_tmp528;});{
struct Cyc_Absyn_Aggrdecl*strct_decl=({struct _fat_ptr*_tmp97A=({struct _fat_ptr*_tmp52C=_cycalloc(sizeof(*_tmp52C));({struct _fat_ptr _tmp979=({const char*_tmp52B="";_tag_fat(_tmp52B,sizeof(char),1U);});*_tmp52C=_tmp979;});_tmp52C;});Cyc_Toc_make_c_struct_defn(_tmp97A,fields);});
({struct _tuple1*_tmp97B=Cyc_Toc_collapse_qvars(f->name,xd->name);strct_decl->name=_tmp97B;});
Cyc_Toc_result_decls=({struct Cyc_List_List*_tmp52A=_cycalloc(sizeof(*_tmp52A));({struct Cyc_Absyn_Decl*_tmp97C=Cyc_Absyn_new_decl((void*)({struct Cyc_Absyn_Aggr_d_Absyn_Raw_decl_struct*_tmp529=_cycalloc(sizeof(*_tmp529));_tmp529->tag=5U,_tmp529->f1=strct_decl;_tmp529;}),0U);_tmp52A->hd=_tmp97C;}),_tmp52A->tl=Cyc_Toc_result_decls;_tmp52A;});
goto _LL6;}}}}}else{if(*((int*)_tmp51D)== 0){_LL9: _LLA:
# 4014
 if((int)f->sc != (int)Cyc_Absyn_Extern){
struct Cyc_Absyn_Exp*initopt=Cyc_Absyn_string_exp(*fn,0U);
struct Cyc_Absyn_Vardecl*tag_decl=Cyc_Absyn_new_vardecl(0U,f->name,tag_typ,initopt);
tag_decl->sc=f->sc;
Cyc_Toc_result_decls=({struct Cyc_List_List*_tmp52E=_cycalloc(sizeof(*_tmp52E));({struct Cyc_Absyn_Decl*_tmp97D=Cyc_Absyn_new_decl((void*)({struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct*_tmp52D=_cycalloc(sizeof(*_tmp52D));_tmp52D->tag=0U,_tmp52D->f1=tag_decl;_tmp52D;}),0U);_tmp52E->hd=_tmp97D;}),_tmp52E->tl=Cyc_Toc_result_decls;_tmp52E;});
({struct Cyc_Dict_Dict _tmp980=({struct Cyc_Dict_Dict(*_tmp97F)(struct Cyc_Dict_Dict d,struct _tuple1*k,int v)=({struct Cyc_Dict_Dict(*_tmp52F)(struct Cyc_Dict_Dict d,struct _tuple1*k,int v)=(struct Cyc_Dict_Dict(*)(struct Cyc_Dict_Dict d,struct _tuple1*k,int v))Cyc_Dict_insert;_tmp52F;});struct Cyc_Dict_Dict _tmp97E=*xdatatypes_so_far;_tmp97F(_tmp97E,f->name,1);});*xdatatypes_so_far=_tmp980;});}
# 4014
goto _LL6;}else{_LLB: _LLC:
# 4022
 goto _LL6;}}_LL6:;}}
# 4025
return 0;}}}}
# 3752 "toc.cyc"
static void Cyc_Toc_xdatatypedecl_to_c(struct Cyc_Absyn_Datatypedecl*xd){
# 4029 "toc.cyc"
({int(*_tmp981)(struct Cyc_Absyn_Datatypedecl*arg,int(*f)(struct _RegionHandle*,struct _tuple42*))=({int(*_tmp531)(struct Cyc_Absyn_Datatypedecl*arg,int(*f)(struct _RegionHandle*,struct _tuple42*))=(int(*)(struct Cyc_Absyn_Datatypedecl*arg,int(*f)(struct _RegionHandle*,struct _tuple42*)))Cyc_Toc_use_toc_state;_tmp531;});_tmp981(xd,Cyc_Toc_xdatatypedecl_to_c_body);});}
# 3752 "toc.cyc"
static void Cyc_Toc_enumdecl_to_c(struct Cyc_Absyn_Enumdecl*ed){
# 4033 "toc.cyc"
ed->sc=Cyc_Absyn_Public;
if(ed->fields != 0)
Cyc_Toc_enumfields_to_c((struct Cyc_List_List*)((struct Cyc_Core_Opt*)_check_null(ed->fields))->v);}
# 3752 "toc.cyc"
static void Cyc_Toc_local_decl_to_c(struct Cyc_Toc_Env*nv,struct Cyc_Absyn_Vardecl*vd,struct Cyc_Absyn_Stmt*s){
# 4039 "toc.cyc"
void*old_typ=vd->type;
({void*_tmp982=Cyc_Toc_typ_to_c(old_typ);vd->type=_tmp982;});
# 4042
if((int)vd->sc == (int)Cyc_Absyn_Register && Cyc_Tcutil_is_fat_pointer_type(old_typ))
vd->sc=Cyc_Absyn_Public;
# 4042
Cyc_Toc_stmt_to_c(nv,s);
# 4045
if(vd->initializer != 0){
struct Cyc_Absyn_Exp*init=(struct Cyc_Absyn_Exp*)_check_null(vd->initializer);
if((int)vd->sc == (int)Cyc_Absyn_Static){
# 4051
struct _RegionHandle _tmp532=_new_region("temp");struct _RegionHandle*temp=& _tmp532;_push_region(temp);
{struct Cyc_Toc_Env*nv2=Cyc_Toc_set_toplevel(temp,nv);
Cyc_Toc_exp_to_c(nv2,init);}
# 4052
;_pop_region(temp);}else{
# 4056
Cyc_Toc_exp_to_c(nv,init);}}else{
# 4059
void*_stmttmp73=Cyc_Tcutil_compress(old_typ);void*_tmp533=_stmttmp73;void*_tmp536;struct Cyc_Absyn_Exp*_tmp535;void*_tmp534;if(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp533)->tag == 4U){_LL1: _tmp534=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp533)->f1).elt_type;_tmp535=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp533)->f1).num_elts;_tmp536=(((struct Cyc_Absyn_ArrayType_Absyn_Type_struct*)_tmp533)->f1).zero_term;_LL2: {void*et=_tmp534;struct Cyc_Absyn_Exp*num_elts_opt=_tmp535;void*zt=_tmp536;
# 4061
if(Cyc_Tcutil_force_type2bool(0,zt)){
if(num_elts_opt == 0)
({void*_tmp537=0U;({int(*_tmp984)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp539)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Toc_toc_impos;_tmp539;});struct _fat_ptr _tmp983=({const char*_tmp538="can't initialize zero-terminated array -- size unknown";_tag_fat(_tmp538,sizeof(char),55U);});_tmp984(_tmp983,_tag_fat(_tmp537,sizeof(void*),0U));});});{
# 4062
struct Cyc_Absyn_Exp*num_elts=num_elts_opt;
# 4065
struct Cyc_Absyn_Exp*lhs=({struct Cyc_Absyn_Exp*_tmp986=Cyc_Absyn_var_exp(vd->name,0U);Cyc_Absyn_subscript_exp(_tmp986,({
struct Cyc_Absyn_Exp*_tmp985=num_elts;Cyc_Absyn_add_exp(_tmp985,Cyc_Absyn_signed_int_exp(- 1,0U),0U);}),0U);});
# 4068
struct Cyc_Absyn_Exp*rhs=Cyc_Absyn_signed_int_exp(0,0U);
({void*_tmp988=({struct Cyc_Absyn_Stmt*_tmp987=Cyc_Absyn_exp_stmt(Cyc_Absyn_assign_exp(lhs,rhs,0U),0U);Cyc_Toc_seq_stmt_r(_tmp987,
Cyc_Absyn_new_stmt(s->r,0U));});
# 4069
s->r=_tmp988;});}}
# 4061
goto _LL0;}}else{_LL3: _LL4:
# 4073
 goto _LL0;}_LL0:;}}
# 3752 "toc.cyc"
static void*Cyc_Toc_rewrite_decision(void*d,struct Cyc_Absyn_Stmt*success){
# 4081 "toc.cyc"
void*_tmp53A=d;void**_tmp53D;struct Cyc_List_List*_tmp53C;struct Cyc_List_List*_tmp53B;struct Cyc_Tcpat_Rhs*_tmp53E;switch(*((int*)_tmp53A)){case 0U: _LL1: _LL2:
 return d;case 1U: _LL3: _tmp53E=((struct Cyc_Tcpat_Success_Tcpat_Decision_struct*)_tmp53A)->f1;_LL4: {struct Cyc_Tcpat_Rhs*rhs=_tmp53E;
rhs->rhs=success;return d;}default: _LL5: _tmp53B=((struct Cyc_Tcpat_SwitchDec_Tcpat_Decision_struct*)_tmp53A)->f1;_tmp53C=((struct Cyc_Tcpat_SwitchDec_Tcpat_Decision_struct*)_tmp53A)->f2;_tmp53D=(void**)&((struct Cyc_Tcpat_SwitchDec_Tcpat_Decision_struct*)_tmp53A)->f3;_LL6: {struct Cyc_List_List*path=_tmp53B;struct Cyc_List_List*sws=_tmp53C;void**d2=_tmp53D;
# 4085
({void*_tmp989=Cyc_Toc_rewrite_decision(*d2,success);*d2=_tmp989;});
for(0;sws != 0;sws=sws->tl){
struct _tuple33*_stmttmp74=(struct _tuple33*)sws->hd;void**_tmp53F;_LL8: _tmp53F=(void**)& _stmttmp74->f2;_LL9: {void**d2=_tmp53F;
({void*_tmp98A=Cyc_Toc_rewrite_decision(*d2,success);*d2=_tmp98A;});}}
# 4090
return d;}}_LL0:;}
# 3752 "toc.cyc"
static struct Cyc_Absyn_Stmt*Cyc_Toc_letdecl_to_c(struct Cyc_Toc_Env*nv,struct Cyc_Absyn_Pat*p,void*dopt,void*t,struct Cyc_Absyn_Exp*e,struct Cyc_Absyn_Stmt*s){
# 4103 "toc.cyc"
struct _RegionHandle _tmp540=_new_region("rgn");struct _RegionHandle*rgn=& _tmp540;_push_region(rgn);
{struct Cyc_Toc_Env*_tmp541=Cyc_Toc_share_env(rgn,nv);struct Cyc_Toc_Env*nv=_tmp541;
void*t=(void*)_check_null(e->topt);
Cyc_Toc_exp_to_c(nv,e);{
struct _tuple1*v;
int already_var;
{void*_stmttmp75=e->r;void*_tmp542=_stmttmp75;void*_tmp543;if(((struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct*)_tmp542)->tag == 1U){_LL1: _tmp543=(void*)((struct Cyc_Absyn_Var_e_Absyn_Raw_exp_struct*)_tmp542)->f1;_LL2: {void*b=_tmp543;
already_var=1;v=Cyc_Absyn_binding2qvar(b);goto _LL0;}}else{_LL3: _LL4:
 already_var=0;v=Cyc_Toc_temp_var();goto _LL0;}_LL0:;}{
# 4114
struct Cyc_Absyn_Exp*path=Cyc_Absyn_var_exp(v,0U);
struct _fat_ptr*end_l=Cyc_Toc_fresh_label();
# 4119
struct Cyc_Absyn_Stmt*succ_stmt=s;
if(dopt != 0)
# 4123
dopt=Cyc_Toc_rewrite_decision(dopt,succ_stmt);{
# 4120
struct Cyc_Absyn_Switch_clause*c1=({struct Cyc_Absyn_Switch_clause*_tmp54E=_cycalloc(sizeof(*_tmp54E));
# 4125
_tmp54E->pattern=p,_tmp54E->pat_vars=0,_tmp54E->where_clause=0,_tmp54E->body=succ_stmt,_tmp54E->loc=0U;_tmp54E;});
struct Cyc_List_List*lscs=({struct Cyc_List_List*(*_tmp98E)(struct _RegionHandle*,struct _tuple36*(*f)(struct _RegionHandle*,struct Cyc_Absyn_Switch_clause*),struct _RegionHandle*env,struct Cyc_List_List*x)=({struct Cyc_List_List*(*_tmp54C)(struct _RegionHandle*,struct _tuple36*(*f)(struct _RegionHandle*,struct Cyc_Absyn_Switch_clause*),struct _RegionHandle*env,struct Cyc_List_List*x)=(struct Cyc_List_List*(*)(struct _RegionHandle*,struct _tuple36*(*f)(struct _RegionHandle*,struct Cyc_Absyn_Switch_clause*),struct _RegionHandle*env,struct Cyc_List_List*x))Cyc_List_rmap_c;_tmp54C;});struct _RegionHandle*_tmp98D=rgn;struct _RegionHandle*_tmp98C=rgn;_tmp98E(_tmp98D,Cyc_Toc_gen_labels,_tmp98C,({struct Cyc_Absyn_Switch_clause*_tmp54D[1U];_tmp54D[0]=c1;({struct _RegionHandle*_tmp98B=rgn;Cyc_List_rlist(_tmp98B,_tag_fat(_tmp54D,sizeof(struct Cyc_Absyn_Switch_clause*),1U));});}));});
# 4130
struct Cyc_List_List*mydecls=0;
struct Cyc_List_List*mybodies=0;
struct Cyc_Absyn_Stmt*test_tree=Cyc_Toc_compile_decision_tree(rgn,nv,& mydecls,& mybodies,dopt,lscs,v);
# 4136
for(0;1;mybodies=((struct Cyc_List_List*)_check_null(mybodies))->tl){
if(mybodies == 0)
({void*_tmp544=0U;({int(*_tmp990)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp546)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Toc_toc_impos;_tmp546;});struct _fat_ptr _tmp98F=({const char*_tmp545="letdecl_to_c: couldn't find env!";_tag_fat(_tmp545,sizeof(char),33U);});_tmp990(_tmp98F,_tag_fat(_tmp544,sizeof(void*),0U));});});{
# 4137
struct _tuple40*_stmttmp76=(struct _tuple40*)((struct Cyc_List_List*)_check_null(mybodies))->hd;struct Cyc_Absyn_Stmt*_tmp548;struct Cyc_Toc_Env*_tmp547;_LL6: _tmp547=_stmttmp76->f1;_tmp548=_stmttmp76->f3;_LL7: {struct Cyc_Toc_Env*env=_tmp547;struct Cyc_Absyn_Stmt*st=_tmp548;
# 4140
if(st == succ_stmt){Cyc_Toc_stmt_to_c(env,s);break;}}}}{
# 4143
struct Cyc_Absyn_Stmt*res=test_tree;
# 4145
for(0;mydecls != 0;mydecls=((struct Cyc_List_List*)_check_null(mydecls))->tl){
struct _tuple38*_stmttmp77=(struct _tuple38*)((struct Cyc_List_List*)_check_null(mydecls))->hd;struct Cyc_Absyn_Vardecl*_tmp549;_LL9: _tmp549=_stmttmp77->f2;_LLA: {struct Cyc_Absyn_Vardecl*vd=_tmp549;
res=({struct Cyc_Absyn_Decl*_tmp991=Cyc_Absyn_new_decl((void*)({struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct*_tmp54A=_cycalloc(sizeof(*_tmp54A));_tmp54A->tag=0U,_tmp54A->f1=vd;_tmp54A;}),0U);Cyc_Absyn_decl_stmt(_tmp991,res,0U);});}}
# 4149
if(!already_var)
res=({struct _tuple1*_tmp994=v;void*_tmp993=Cyc_Toc_typ_to_c(t);struct Cyc_Absyn_Exp*_tmp992=e;Cyc_Absyn_declare_stmt(_tmp994,_tmp993,_tmp992,res,0U);});{
# 4149
struct Cyc_Absyn_Stmt*_tmp54B=res;_npop_handler(0U);return _tmp54B;}}}}}}
# 4104
;_pop_region(rgn);}
# 3752 "toc.cyc"
static void Cyc_Toc_exptypes_to_c(struct Cyc_Absyn_Exp*e){
# 4187 "toc.cyc"
void*_stmttmp78=e->r;void*_tmp54F=_stmttmp78;struct Cyc_Absyn_MallocInfo*_tmp550;struct Cyc_Absyn_Stmt*_tmp551;void**_tmp552;void**_tmp553;struct Cyc_List_List*_tmp554;struct Cyc_List_List*_tmp555;struct Cyc_List_List*_tmp557;void**_tmp556;struct Cyc_Absyn_Exp*_tmp559;void**_tmp558;struct Cyc_List_List*_tmp55B;struct Cyc_Absyn_Exp*_tmp55A;struct Cyc_Absyn_Exp*_tmp55E;struct Cyc_Absyn_Exp*_tmp55D;struct Cyc_Absyn_Exp*_tmp55C;struct Cyc_Absyn_Exp*_tmp560;struct Cyc_Absyn_Exp*_tmp55F;struct Cyc_Absyn_Exp*_tmp562;struct Cyc_Absyn_Exp*_tmp561;struct Cyc_Absyn_Exp*_tmp564;struct Cyc_Absyn_Exp*_tmp563;struct Cyc_Absyn_Exp*_tmp566;struct Cyc_Absyn_Exp*_tmp565;struct Cyc_Absyn_Exp*_tmp568;struct Cyc_Absyn_Exp*_tmp567;struct Cyc_Absyn_Exp*_tmp56A;struct Cyc_Absyn_Exp*_tmp569;struct Cyc_List_List*_tmp56B;struct Cyc_Absyn_Exp*_tmp56C;struct Cyc_Absyn_Exp*_tmp56D;struct Cyc_Absyn_Exp*_tmp56E;struct Cyc_Absyn_Exp*_tmp56F;struct Cyc_Absyn_Exp*_tmp570;struct Cyc_Absyn_Exp*_tmp571;struct Cyc_Absyn_Exp*_tmp572;struct Cyc_Absyn_Exp*_tmp573;struct Cyc_Absyn_Exp*_tmp574;switch(*((int*)_tmp54F)){case 42U: _LL1: _tmp574=((struct Cyc_Absyn_Extension_e_Absyn_Raw_exp_struct*)_tmp54F)->f1;_LL2: {struct Cyc_Absyn_Exp*e=_tmp574;
_tmp573=e;goto _LL4;}case 20U: _LL3: _tmp573=((struct Cyc_Absyn_Deref_e_Absyn_Raw_exp_struct*)_tmp54F)->f1;_LL4: {struct Cyc_Absyn_Exp*e=_tmp573;
_tmp572=e;goto _LL6;}case 21U: _LL5: _tmp572=((struct Cyc_Absyn_AggrMember_e_Absyn_Raw_exp_struct*)_tmp54F)->f1;_LL6: {struct Cyc_Absyn_Exp*e=_tmp572;
_tmp571=e;goto _LL8;}case 22U: _LL7: _tmp571=((struct Cyc_Absyn_AggrArrow_e_Absyn_Raw_exp_struct*)_tmp54F)->f1;_LL8: {struct Cyc_Absyn_Exp*e=_tmp571;
_tmp570=e;goto _LLA;}case 15U: _LL9: _tmp570=((struct Cyc_Absyn_Address_e_Absyn_Raw_exp_struct*)_tmp54F)->f1;_LLA: {struct Cyc_Absyn_Exp*e=_tmp570;
_tmp56F=e;goto _LLC;}case 11U: _LLB: _tmp56F=((struct Cyc_Absyn_Throw_e_Absyn_Raw_exp_struct*)_tmp54F)->f1;_LLC: {struct Cyc_Absyn_Exp*e=_tmp56F;
_tmp56E=e;goto _LLE;}case 12U: _LLD: _tmp56E=((struct Cyc_Absyn_NoInstantiate_e_Absyn_Raw_exp_struct*)_tmp54F)->f1;_LLE: {struct Cyc_Absyn_Exp*e=_tmp56E;
_tmp56D=e;goto _LL10;}case 18U: _LLF: _tmp56D=((struct Cyc_Absyn_Sizeofexp_e_Absyn_Raw_exp_struct*)_tmp54F)->f1;_LL10: {struct Cyc_Absyn_Exp*e=_tmp56D;
_tmp56C=e;goto _LL12;}case 5U: _LL11: _tmp56C=((struct Cyc_Absyn_Increment_e_Absyn_Raw_exp_struct*)_tmp54F)->f1;_LL12: {struct Cyc_Absyn_Exp*e1=_tmp56C;
Cyc_Toc_exptypes_to_c(e1);goto _LL0;}case 3U: _LL13: _tmp56B=((struct Cyc_Absyn_Primop_e_Absyn_Raw_exp_struct*)_tmp54F)->f2;_LL14: {struct Cyc_List_List*es=_tmp56B;
({void(*_tmp995)(void(*f)(struct Cyc_Absyn_Exp*),struct Cyc_List_List*x)=({void(*_tmp575)(void(*f)(struct Cyc_Absyn_Exp*),struct Cyc_List_List*x)=(void(*)(void(*f)(struct Cyc_Absyn_Exp*),struct Cyc_List_List*x))Cyc_List_iter;_tmp575;});_tmp995(Cyc_Toc_exptypes_to_c,es);});goto _LL0;}case 7U: _LL15: _tmp569=((struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct*)_tmp54F)->f1;_tmp56A=((struct Cyc_Absyn_And_e_Absyn_Raw_exp_struct*)_tmp54F)->f2;_LL16: {struct Cyc_Absyn_Exp*e1=_tmp569;struct Cyc_Absyn_Exp*e2=_tmp56A;
_tmp567=e1;_tmp568=e2;goto _LL18;}case 8U: _LL17: _tmp567=((struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct*)_tmp54F)->f1;_tmp568=((struct Cyc_Absyn_Or_e_Absyn_Raw_exp_struct*)_tmp54F)->f2;_LL18: {struct Cyc_Absyn_Exp*e1=_tmp567;struct Cyc_Absyn_Exp*e2=_tmp568;
_tmp565=e1;_tmp566=e2;goto _LL1A;}case 9U: _LL19: _tmp565=((struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct*)_tmp54F)->f1;_tmp566=((struct Cyc_Absyn_SeqExp_e_Absyn_Raw_exp_struct*)_tmp54F)->f2;_LL1A: {struct Cyc_Absyn_Exp*e1=_tmp565;struct Cyc_Absyn_Exp*e2=_tmp566;
_tmp563=e1;_tmp564=e2;goto _LL1C;}case 23U: _LL1B: _tmp563=((struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct*)_tmp54F)->f1;_tmp564=((struct Cyc_Absyn_Subscript_e_Absyn_Raw_exp_struct*)_tmp54F)->f2;_LL1C: {struct Cyc_Absyn_Exp*e1=_tmp563;struct Cyc_Absyn_Exp*e2=_tmp564;
_tmp561=e1;_tmp562=e2;goto _LL1E;}case 36U: _LL1D: _tmp561=((struct Cyc_Absyn_Swap_e_Absyn_Raw_exp_struct*)_tmp54F)->f1;_tmp562=((struct Cyc_Absyn_Swap_e_Absyn_Raw_exp_struct*)_tmp54F)->f2;_LL1E: {struct Cyc_Absyn_Exp*e1=_tmp561;struct Cyc_Absyn_Exp*e2=_tmp562;
_tmp55F=e1;_tmp560=e2;goto _LL20;}case 4U: _LL1F: _tmp55F=((struct Cyc_Absyn_AssignOp_e_Absyn_Raw_exp_struct*)_tmp54F)->f1;_tmp560=((struct Cyc_Absyn_AssignOp_e_Absyn_Raw_exp_struct*)_tmp54F)->f3;_LL20: {struct Cyc_Absyn_Exp*e1=_tmp55F;struct Cyc_Absyn_Exp*e2=_tmp560;
Cyc_Toc_exptypes_to_c(e1);Cyc_Toc_exptypes_to_c(e2);goto _LL0;}case 6U: _LL21: _tmp55C=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmp54F)->f1;_tmp55D=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmp54F)->f2;_tmp55E=((struct Cyc_Absyn_Conditional_e_Absyn_Raw_exp_struct*)_tmp54F)->f3;_LL22: {struct Cyc_Absyn_Exp*e1=_tmp55C;struct Cyc_Absyn_Exp*e2=_tmp55D;struct Cyc_Absyn_Exp*e3=_tmp55E;
# 4205
Cyc_Toc_exptypes_to_c(e1);Cyc_Toc_exptypes_to_c(e2);Cyc_Toc_exptypes_to_c(e3);goto _LL0;}case 10U: _LL23: _tmp55A=((struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct*)_tmp54F)->f1;_tmp55B=((struct Cyc_Absyn_FnCall_e_Absyn_Raw_exp_struct*)_tmp54F)->f2;_LL24: {struct Cyc_Absyn_Exp*e=_tmp55A;struct Cyc_List_List*es=_tmp55B;
# 4207
Cyc_Toc_exptypes_to_c(e);({void(*_tmp996)(void(*f)(struct Cyc_Absyn_Exp*),struct Cyc_List_List*x)=({void(*_tmp576)(void(*f)(struct Cyc_Absyn_Exp*),struct Cyc_List_List*x)=(void(*)(void(*f)(struct Cyc_Absyn_Exp*),struct Cyc_List_List*x))Cyc_List_iter;_tmp576;});_tmp996(Cyc_Toc_exptypes_to_c,es);});goto _LL0;}case 14U: _LL25: _tmp558=(void**)&((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp54F)->f1;_tmp559=((struct Cyc_Absyn_Cast_e_Absyn_Raw_exp_struct*)_tmp54F)->f2;_LL26: {void**t=_tmp558;struct Cyc_Absyn_Exp*e=_tmp559;
({void*_tmp997=Cyc_Toc_typ_to_c(*t);*t=_tmp997;});Cyc_Toc_exptypes_to_c(e);goto _LL0;}case 25U: _LL27: _tmp556=(void**)&(((struct Cyc_Absyn_CompoundLit_e_Absyn_Raw_exp_struct*)_tmp54F)->f1)->f3;_tmp557=((struct Cyc_Absyn_CompoundLit_e_Absyn_Raw_exp_struct*)_tmp54F)->f2;_LL28: {void**t=_tmp556;struct Cyc_List_List*dles=_tmp557;
# 4210
({void*_tmp998=Cyc_Toc_typ_to_c(*t);*t=_tmp998;});
_tmp555=dles;goto _LL2A;}case 37U: _LL29: _tmp555=((struct Cyc_Absyn_UnresolvedMem_e_Absyn_Raw_exp_struct*)_tmp54F)->f2;_LL2A: {struct Cyc_List_List*dles=_tmp555;
_tmp554=dles;goto _LL2C;}case 26U: _LL2B: _tmp554=((struct Cyc_Absyn_Array_e_Absyn_Raw_exp_struct*)_tmp54F)->f1;_LL2C: {struct Cyc_List_List*dles=_tmp554;
# 4214
for(0;dles != 0;dles=dles->tl){
Cyc_Toc_exptypes_to_c((*((struct _tuple20*)dles->hd)).f2);}
goto _LL0;}case 19U: _LL2D: _tmp553=(void**)&((struct Cyc_Absyn_Offsetof_e_Absyn_Raw_exp_struct*)_tmp54F)->f1;_LL2E: {void**t=_tmp553;
_tmp552=t;goto _LL30;}case 17U: _LL2F: _tmp552=(void**)&((struct Cyc_Absyn_Sizeoftype_e_Absyn_Raw_exp_struct*)_tmp54F)->f1;_LL30: {void**t=_tmp552;
({void*_tmp999=Cyc_Toc_typ_to_c(*t);*t=_tmp999;});goto _LL0;}case 38U: _LL31: _tmp551=((struct Cyc_Absyn_StmtExp_e_Absyn_Raw_exp_struct*)_tmp54F)->f1;_LL32: {struct Cyc_Absyn_Stmt*s=_tmp551;
Cyc_Toc_stmttypes_to_c(s);goto _LL0;}case 35U: _LL33: _tmp550=(struct Cyc_Absyn_MallocInfo*)&((struct Cyc_Absyn_Malloc_e_Absyn_Raw_exp_struct*)_tmp54F)->f1;_LL34: {struct Cyc_Absyn_MallocInfo*m=_tmp550;
# 4221
if(m->elt_type != 0)
({void**_tmp99B=({void**_tmp577=_cycalloc(sizeof(*_tmp577));({void*_tmp99A=Cyc_Toc_typ_to_c(*((void**)_check_null(m->elt_type)));*_tmp577=_tmp99A;});_tmp577;});m->elt_type=_tmp99B;});
# 4221
Cyc_Toc_exptypes_to_c(m->num_elts);
# 4224
goto _LL0;}case 0U: _LL35: _LL36:
 goto _LL38;case 1U: _LL37: _LL38:
 goto _LL3A;case 32U: _LL39: _LL3A:
 goto _LL3C;case 41U: _LL3B: _LL3C:
 goto _LL3E;case 33U: _LL3D: _LL3E:
 goto _LL0;case 2U: _LL3F: _LL40:
# 4231
 goto _LL42;case 30U: _LL41: _LL42:
 goto _LL44;case 31U: _LL43: _LL44:
 goto _LL46;case 29U: _LL45: _LL46:
 goto _LL48;case 27U: _LL47: _LL48:
 goto _LL4A;case 28U: _LL49: _LL4A:
 goto _LL4C;case 34U: _LL4B: _LL4C:
 goto _LL4E;case 24U: _LL4D: _LL4E:
 goto _LL50;case 13U: _LL4F: _LL50:
 goto _LL52;case 16U: _LL51: _LL52:
 goto _LL54;case 40U: _LL53: _LL54:
 goto _LL56;default: _LL55: _LL56:
({void*_tmp578=0U;({int(*_tmp99E)(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp57A)(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos_loc;_tmp57A;});unsigned _tmp99D=e->loc;struct _fat_ptr _tmp99C=({const char*_tmp579="Cyclone expression in C code";_tag_fat(_tmp579,sizeof(char),29U);});_tmp99E(_tmp99D,_tmp99C,_tag_fat(_tmp578,sizeof(void*),0U));});});}_LL0:;}
# 3752 "toc.cyc"
static void Cyc_Toc_decltypes_to_c(struct Cyc_Absyn_Decl*d){
# 4247 "toc.cyc"
void*_stmttmp79=d->r;void*_tmp57B=_stmttmp79;struct Cyc_Absyn_Typedefdecl*_tmp57C;struct Cyc_Absyn_Enumdecl*_tmp57D;struct Cyc_Absyn_Aggrdecl*_tmp57E;struct Cyc_Absyn_Fndecl*_tmp57F;struct Cyc_Absyn_Vardecl*_tmp580;switch(*((int*)_tmp57B)){case 0U: _LL1: _tmp580=((struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct*)_tmp57B)->f1;_LL2: {struct Cyc_Absyn_Vardecl*vd=_tmp580;
# 4249
({void*_tmp99F=Cyc_Toc_typ_to_c(vd->type);vd->type=_tmp99F;});
if(vd->initializer != 0)Cyc_Toc_exptypes_to_c((struct Cyc_Absyn_Exp*)_check_null(vd->initializer));goto _LL0;}case 1U: _LL3: _tmp57F=((struct Cyc_Absyn_Fn_d_Absyn_Raw_decl_struct*)_tmp57B)->f1;_LL4: {struct Cyc_Absyn_Fndecl*fd=_tmp57F;
# 4253
({void*_tmp9A0=Cyc_Toc_typ_to_c((fd->i).ret_type);(fd->i).ret_type=_tmp9A0;});
{struct Cyc_List_List*args=(fd->i).args;for(0;args != 0;args=args->tl){
({void*_tmp9A1=Cyc_Toc_typ_to_c((*((struct _tuple9*)args->hd)).f3);(*((struct _tuple9*)args->hd)).f3=_tmp9A1;});}}
goto _LL0;}case 5U: _LL5: _tmp57E=((struct Cyc_Absyn_Aggr_d_Absyn_Raw_decl_struct*)_tmp57B)->f1;_LL6: {struct Cyc_Absyn_Aggrdecl*ad=_tmp57E;
Cyc_Toc_aggrdecl_to_c(ad);goto _LL0;}case 7U: _LL7: _tmp57D=((struct Cyc_Absyn_Enum_d_Absyn_Raw_decl_struct*)_tmp57B)->f1;_LL8: {struct Cyc_Absyn_Enumdecl*ed=_tmp57D;
# 4259
if(ed->fields != 0){
struct Cyc_List_List*fs=(struct Cyc_List_List*)((struct Cyc_Core_Opt*)_check_null(ed->fields))->v;for(0;fs != 0;fs=fs->tl){
struct Cyc_Absyn_Enumfield*f=(struct Cyc_Absyn_Enumfield*)fs->hd;
if(f->tag != 0)Cyc_Toc_exptypes_to_c((struct Cyc_Absyn_Exp*)_check_null(f->tag));}}
# 4259
goto _LL0;}case 8U: _LL9: _tmp57C=((struct Cyc_Absyn_Typedef_d_Absyn_Raw_decl_struct*)_tmp57B)->f1;_LLA: {struct Cyc_Absyn_Typedefdecl*td=_tmp57C;
# 4265
({void*_tmp9A2=Cyc_Toc_typ_to_c((void*)_check_null(td->defn));td->defn=_tmp9A2;});goto _LL0;}case 2U: _LLB: _LLC:
 goto _LLE;case 3U: _LLD: _LLE:
 goto _LL10;case 6U: _LLF: _LL10:
 goto _LL12;case 9U: _LL11: _LL12:
 goto _LL14;case 10U: _LL13: _LL14:
 goto _LL16;case 11U: _LL15: _LL16:
 goto _LL18;case 12U: _LL17: _LL18:
 goto _LL1A;case 4U: _LL19: _LL1A:
# 4274
({void*_tmp581=0U;({int(*_tmp9A5)(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp583)(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos_loc;_tmp583;});unsigned _tmp9A4=d->loc;struct _fat_ptr _tmp9A3=({const char*_tmp582="Cyclone declaration in C code";_tag_fat(_tmp582,sizeof(char),30U);});_tmp9A5(_tmp9A4,_tmp9A3,_tag_fat(_tmp581,sizeof(void*),0U));});});case 13U: _LL1B: _LL1C:
 goto _LL1E;case 14U: _LL1D: _LL1E:
 goto _LL20;case 15U: _LL1F: _LL20:
 goto _LL22;default: _LL21: _LL22:
 goto _LL0;}_LL0:;}
# 3752 "toc.cyc"
static void Cyc_Toc_stmttypes_to_c(struct Cyc_Absyn_Stmt*s){
# 4283 "toc.cyc"
void*_stmttmp7A=s->r;void*_tmp584=_stmttmp7A;struct Cyc_Absyn_Stmt*_tmp585;struct Cyc_Absyn_Exp*_tmp587;struct Cyc_Absyn_Stmt*_tmp586;struct Cyc_Absyn_Stmt*_tmp589;struct Cyc_Absyn_Decl*_tmp588;void*_tmp58C;struct Cyc_List_List*_tmp58B;struct Cyc_Absyn_Exp*_tmp58A;struct Cyc_Absyn_Stmt*_tmp590;struct Cyc_Absyn_Exp*_tmp58F;struct Cyc_Absyn_Exp*_tmp58E;struct Cyc_Absyn_Exp*_tmp58D;struct Cyc_Absyn_Stmt*_tmp592;struct Cyc_Absyn_Exp*_tmp591;struct Cyc_Absyn_Stmt*_tmp595;struct Cyc_Absyn_Stmt*_tmp594;struct Cyc_Absyn_Exp*_tmp593;struct Cyc_Absyn_Exp*_tmp596;struct Cyc_Absyn_Stmt*_tmp598;struct Cyc_Absyn_Stmt*_tmp597;struct Cyc_Absyn_Exp*_tmp599;switch(*((int*)_tmp584)){case 1U: _LL1: _tmp599=((struct Cyc_Absyn_Exp_s_Absyn_Raw_stmt_struct*)_tmp584)->f1;_LL2: {struct Cyc_Absyn_Exp*e=_tmp599;
Cyc_Toc_exptypes_to_c(e);goto _LL0;}case 2U: _LL3: _tmp597=((struct Cyc_Absyn_Seq_s_Absyn_Raw_stmt_struct*)_tmp584)->f1;_tmp598=((struct Cyc_Absyn_Seq_s_Absyn_Raw_stmt_struct*)_tmp584)->f2;_LL4: {struct Cyc_Absyn_Stmt*s1=_tmp597;struct Cyc_Absyn_Stmt*s2=_tmp598;
Cyc_Toc_stmttypes_to_c(s1);Cyc_Toc_stmttypes_to_c(s2);goto _LL0;}case 3U: _LL5: _tmp596=((struct Cyc_Absyn_Return_s_Absyn_Raw_stmt_struct*)_tmp584)->f1;_LL6: {struct Cyc_Absyn_Exp*eopt=_tmp596;
if(eopt != 0)Cyc_Toc_exptypes_to_c(eopt);goto _LL0;}case 4U: _LL7: _tmp593=((struct Cyc_Absyn_IfThenElse_s_Absyn_Raw_stmt_struct*)_tmp584)->f1;_tmp594=((struct Cyc_Absyn_IfThenElse_s_Absyn_Raw_stmt_struct*)_tmp584)->f2;_tmp595=((struct Cyc_Absyn_IfThenElse_s_Absyn_Raw_stmt_struct*)_tmp584)->f3;_LL8: {struct Cyc_Absyn_Exp*e=_tmp593;struct Cyc_Absyn_Stmt*s1=_tmp594;struct Cyc_Absyn_Stmt*s2=_tmp595;
# 4288
Cyc_Toc_exptypes_to_c(e);Cyc_Toc_stmttypes_to_c(s1);Cyc_Toc_stmttypes_to_c(s2);goto _LL0;}case 5U: _LL9: _tmp591=(((struct Cyc_Absyn_While_s_Absyn_Raw_stmt_struct*)_tmp584)->f1).f1;_tmp592=((struct Cyc_Absyn_While_s_Absyn_Raw_stmt_struct*)_tmp584)->f2;_LLA: {struct Cyc_Absyn_Exp*e=_tmp591;struct Cyc_Absyn_Stmt*s=_tmp592;
# 4290
Cyc_Toc_exptypes_to_c(e);Cyc_Toc_stmttypes_to_c(s);goto _LL0;}case 9U: _LLB: _tmp58D=((struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct*)_tmp584)->f1;_tmp58E=(((struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct*)_tmp584)->f2).f1;_tmp58F=(((struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct*)_tmp584)->f3).f1;_tmp590=((struct Cyc_Absyn_For_s_Absyn_Raw_stmt_struct*)_tmp584)->f4;_LLC: {struct Cyc_Absyn_Exp*e1=_tmp58D;struct Cyc_Absyn_Exp*e2=_tmp58E;struct Cyc_Absyn_Exp*e3=_tmp58F;struct Cyc_Absyn_Stmt*s=_tmp590;
# 4292
Cyc_Toc_exptypes_to_c(e1);Cyc_Toc_exptypes_to_c(e2);Cyc_Toc_exptypes_to_c(e3);
Cyc_Toc_stmttypes_to_c(s);goto _LL0;}case 10U: _LLD: _tmp58A=((struct Cyc_Absyn_Switch_s_Absyn_Raw_stmt_struct*)_tmp584)->f1;_tmp58B=((struct Cyc_Absyn_Switch_s_Absyn_Raw_stmt_struct*)_tmp584)->f2;_tmp58C=(void*)((struct Cyc_Absyn_Switch_s_Absyn_Raw_stmt_struct*)_tmp584)->f3;_LLE: {struct Cyc_Absyn_Exp*e=_tmp58A;struct Cyc_List_List*scs=_tmp58B;void*dec_tree=_tmp58C;
# 4295
Cyc_Toc_exptypes_to_c(e);
for(0;scs != 0;scs=scs->tl){Cyc_Toc_stmttypes_to_c(((struct Cyc_Absyn_Switch_clause*)scs->hd)->body);}
goto _LL0;}case 12U: _LLF: _tmp588=((struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct*)_tmp584)->f1;_tmp589=((struct Cyc_Absyn_Decl_s_Absyn_Raw_stmt_struct*)_tmp584)->f2;_LL10: {struct Cyc_Absyn_Decl*d=_tmp588;struct Cyc_Absyn_Stmt*s=_tmp589;
Cyc_Toc_decltypes_to_c(d);Cyc_Toc_stmttypes_to_c(s);goto _LL0;}case 14U: _LL11: _tmp586=((struct Cyc_Absyn_Do_s_Absyn_Raw_stmt_struct*)_tmp584)->f1;_tmp587=(((struct Cyc_Absyn_Do_s_Absyn_Raw_stmt_struct*)_tmp584)->f2).f1;_LL12: {struct Cyc_Absyn_Stmt*s=_tmp586;struct Cyc_Absyn_Exp*e=_tmp587;
Cyc_Toc_stmttypes_to_c(s);Cyc_Toc_exptypes_to_c(e);goto _LL0;}case 13U: _LL13: _tmp585=((struct Cyc_Absyn_Label_s_Absyn_Raw_stmt_struct*)_tmp584)->f2;_LL14: {struct Cyc_Absyn_Stmt*s=_tmp585;
Cyc_Toc_stmttypes_to_c(s);goto _LL0;}case 0U: _LL15: _LL16:
 goto _LL18;case 6U: _LL17: _LL18:
 goto _LL1A;case 7U: _LL19: _LL1A:
 goto _LL1C;case 8U: _LL1B: _LL1C:
 goto _LL0;case 11U: _LL1D: _LL1E:
# 4307
({void*_tmp9A6=(void*)({struct Cyc_Absyn_Skip_s_Absyn_Raw_stmt_struct*_tmp59A=_cycalloc(sizeof(*_tmp59A));_tmp59A->tag=0U;_tmp59A;});s->r=_tmp9A6;});goto _LL0;default: _LL1F: _LL20:
({void*_tmp59B=0U;({int(*_tmp9A9)(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp59D)(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(unsigned loc,struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Warn_impos_loc;_tmp59D;});unsigned _tmp9A8=s->loc;struct _fat_ptr _tmp9A7=({const char*_tmp59C="Cyclone statement in C code";_tag_fat(_tmp59C,sizeof(char),28U);});_tmp9A9(_tmp9A8,_tmp9A7,_tag_fat(_tmp59B,sizeof(void*),0U));});});}_LL0:;}
# 3752 "toc.cyc"
static struct Cyc_Toc_Env*Cyc_Toc_decls_to_c(struct _RegionHandle*r,struct Cyc_Toc_Env*nv,struct Cyc_List_List*ds,int cinclude){
# 4317 "toc.cyc"
for(0;ds != 0;ds=ds->tl){
if(!Cyc_Toc_is_toplevel(nv))
({void*_tmp59E=0U;({int(*_tmp9AB)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp5A0)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp5A0;});struct _fat_ptr _tmp9AA=({const char*_tmp59F="decls_to_c: not at toplevel!";_tag_fat(_tmp59F,sizeof(char),29U);});_tmp9AB(_tmp9AA,_tag_fat(_tmp59E,sizeof(void*),0U));});});
# 4318
Cyc_Toc_fresh_label_counter=0;{
# 4321
struct Cyc_Absyn_Decl*d=(struct Cyc_Absyn_Decl*)ds->hd;
void*_stmttmp7B=d->r;void*_tmp5A1=_stmttmp7B;struct Cyc_List_List*_tmp5A2;struct Cyc_List_List*_tmp5A3;struct Cyc_List_List*_tmp5A4;struct Cyc_List_List*_tmp5A5;struct Cyc_Absyn_Typedefdecl*_tmp5A6;struct Cyc_Absyn_Enumdecl*_tmp5A7;struct Cyc_Absyn_Datatypedecl*_tmp5A8;struct Cyc_Absyn_Aggrdecl*_tmp5A9;struct Cyc_Absyn_Fndecl*_tmp5AA;struct Cyc_Absyn_Vardecl*_tmp5AB;switch(*((int*)_tmp5A1)){case 0U: _LL1: _tmp5AB=((struct Cyc_Absyn_Var_d_Absyn_Raw_decl_struct*)_tmp5A1)->f1;_LL2: {struct Cyc_Absyn_Vardecl*vd=_tmp5AB;
# 4324
struct _tuple1*c_name=vd->name;
# 4326
if((int)vd->sc == (int)Cyc_Absyn_ExternC)
c_name=({struct _tuple1*_tmp5AC=_cycalloc(sizeof(*_tmp5AC));({union Cyc_Absyn_Nmspace _tmp9AC=Cyc_Absyn_Abs_n(0,1);_tmp5AC->f1=_tmp9AC;}),_tmp5AC->f2=(*c_name).f2;_tmp5AC;});
# 4326
if(vd->initializer != 0){
# 4329
if((int)vd->sc == (int)Cyc_Absyn_ExternC)vd->sc=Cyc_Absyn_Public;if(cinclude)
# 4331
Cyc_Toc_exptypes_to_c((struct Cyc_Absyn_Exp*)_check_null(vd->initializer));else{
# 4333
Cyc_Toc_exp_to_c(nv,(struct Cyc_Absyn_Exp*)_check_null(vd->initializer));}}
# 4326
vd->name=c_name;
# 4336
({enum Cyc_Absyn_Scope _tmp9AD=Cyc_Toc_scope_to_c(vd->sc);vd->sc=_tmp9AD;});
({void*_tmp9AE=Cyc_Toc_typ_to_c(vd->type);vd->type=_tmp9AE;});
Cyc_Toc_result_decls=({struct Cyc_List_List*_tmp5AD=_cycalloc(sizeof(*_tmp5AD));_tmp5AD->hd=d,_tmp5AD->tl=Cyc_Toc_result_decls;_tmp5AD;});
goto _LL0;}case 1U: _LL3: _tmp5AA=((struct Cyc_Absyn_Fn_d_Absyn_Raw_decl_struct*)_tmp5A1)->f1;_LL4: {struct Cyc_Absyn_Fndecl*fd=_tmp5AA;
# 4342
if((int)fd->sc == (int)Cyc_Absyn_ExternC){
({struct _tuple1*_tmp9B0=({struct _tuple1*_tmp5AE=_cycalloc(sizeof(*_tmp5AE));({union Cyc_Absyn_Nmspace _tmp9AF=Cyc_Absyn_Abs_n(0,1);_tmp5AE->f1=_tmp9AF;}),_tmp5AE->f2=(*fd->name).f2;_tmp5AE;});fd->name=_tmp9B0;});
fd->sc=Cyc_Absyn_Public;}
# 4342
Cyc_Toc_fndecl_to_c(nv,fd,cinclude);
# 4347
Cyc_Toc_result_decls=({struct Cyc_List_List*_tmp5AF=_cycalloc(sizeof(*_tmp5AF));_tmp5AF->hd=d,_tmp5AF->tl=Cyc_Toc_result_decls;_tmp5AF;});
goto _LL0;}case 2U: _LL5: _LL6:
 goto _LL8;case 3U: _LL7: _LL8:
({void*_tmp5B0=0U;({int(*_tmp9B2)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp5B2)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Toc_toc_impos;_tmp5B2;});struct _fat_ptr _tmp9B1=({const char*_tmp5B1="letdecl at toplevel";_tag_fat(_tmp5B1,sizeof(char),20U);});_tmp9B2(_tmp9B1,_tag_fat(_tmp5B0,sizeof(void*),0U));});});case 4U: _LL9: _LLA:
({void*_tmp5B3=0U;({int(*_tmp9B4)(struct _fat_ptr fmt,struct _fat_ptr ap)=({int(*_tmp5B5)(struct _fat_ptr fmt,struct _fat_ptr ap)=(int(*)(struct _fat_ptr fmt,struct _fat_ptr ap))Cyc_Toc_toc_impos;_tmp5B5;});struct _fat_ptr _tmp9B3=({const char*_tmp5B4="region decl at toplevel";_tag_fat(_tmp5B4,sizeof(char),24U);});_tmp9B4(_tmp9B3,_tag_fat(_tmp5B3,sizeof(void*),0U));});});case 5U: _LLB: _tmp5A9=((struct Cyc_Absyn_Aggr_d_Absyn_Raw_decl_struct*)_tmp5A1)->f1;_LLC: {struct Cyc_Absyn_Aggrdecl*sd=_tmp5A9;
Cyc_Toc_aggrdecl_to_c(sd);goto _LL0;}case 6U: _LLD: _tmp5A8=((struct Cyc_Absyn_Datatype_d_Absyn_Raw_decl_struct*)_tmp5A1)->f1;_LLE: {struct Cyc_Absyn_Datatypedecl*tud=_tmp5A8;
# 4354
tud->is_extensible?Cyc_Toc_xdatatypedecl_to_c(tud): Cyc_Toc_datatypedecl_to_c(tud);
goto _LL0;}case 7U: _LLF: _tmp5A7=((struct Cyc_Absyn_Enum_d_Absyn_Raw_decl_struct*)_tmp5A1)->f1;_LL10: {struct Cyc_Absyn_Enumdecl*ed=_tmp5A7;
# 4357
Cyc_Toc_enumdecl_to_c(ed);
Cyc_Toc_result_decls=({struct Cyc_List_List*_tmp5B6=_cycalloc(sizeof(*_tmp5B6));_tmp5B6->hd=d,_tmp5B6->tl=Cyc_Toc_result_decls;_tmp5B6;});
goto _LL0;}case 8U: _LL11: _tmp5A6=((struct Cyc_Absyn_Typedef_d_Absyn_Raw_decl_struct*)_tmp5A1)->f1;_LL12: {struct Cyc_Absyn_Typedefdecl*td=_tmp5A6;
# 4361
td->tvs=0;
if(td->defn != 0){
({void*_tmp9B5=Cyc_Toc_typ_to_c((void*)_check_null(td->defn));td->defn=_tmp9B5;});{
# 4366
struct Cyc_Absyn_Decl*ed;
{void*_stmttmp7C=td->defn;void*_tmp5B7=_stmttmp7C;unsigned _tmp5B9;struct Cyc_Absyn_Enumdecl*_tmp5B8;if(_tmp5B7 != 0){if(((struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct*)_tmp5B7)->tag == 10U){if(((struct Cyc_Absyn_Enum_td_Absyn_Raw_typedecl_struct*)((struct Cyc_Absyn_TypeDecl*)((struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct*)_tmp5B7)->f1)->r)->tag == 1U){_LL24: _tmp5B8=((struct Cyc_Absyn_Enum_td_Absyn_Raw_typedecl_struct*)(((struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct*)_tmp5B7)->f1)->r)->f1;_tmp5B9=(((struct Cyc_Absyn_TypeDeclType_Absyn_Type_struct*)_tmp5B7)->f1)->loc;_LL25: {struct Cyc_Absyn_Enumdecl*ed2=_tmp5B8;unsigned loc=_tmp5B9;
# 4369
ed=({struct Cyc_Absyn_Decl*_tmp5BB=_cycalloc(sizeof(*_tmp5BB));({void*_tmp9B6=(void*)({struct Cyc_Absyn_Enum_d_Absyn_Raw_decl_struct*_tmp5BA=_cycalloc(sizeof(*_tmp5BA));_tmp5BA->tag=7U,_tmp5BA->f1=ed2;_tmp5BA;});_tmp5BB->r=_tmp9B6;}),_tmp5BB->loc=loc;_tmp5BB;});goto _LL23;}}else{goto _LL26;}}else{goto _LL26;}}else{_LL26: _LL27:
 ed=0;}_LL23:;}
# 4372
if(ed != 0){
Cyc_Toc_result_decls=({struct Cyc_List_List*_tmp5BC=_cycalloc(sizeof(*_tmp5BC));_tmp5BC->hd=ed,_tmp5BC->tl=Cyc_Toc_result_decls;_tmp5BC;});{
void*_stmttmp7D=ed->r;void*_tmp5BD=_stmttmp7D;struct Cyc_Absyn_Enumdecl*_tmp5BE;if(((struct Cyc_Absyn_Enum_d_Absyn_Raw_decl_struct*)_tmp5BD)->tag == 7U){_LL29: _tmp5BE=((struct Cyc_Absyn_Enum_d_Absyn_Raw_decl_struct*)_tmp5BD)->f1;_LL2A: {struct Cyc_Absyn_Enumdecl*ed=_tmp5BE;
# 4376
({void*_tmp9B8=(void*)({struct Cyc_Absyn_AppType_Absyn_Type_struct*_tmp5C0=_cycalloc(sizeof(*_tmp5C0));_tmp5C0->tag=0U,({void*_tmp9B7=(void*)({struct Cyc_Absyn_EnumCon_Absyn_TyCon_struct*_tmp5BF=_cycalloc(sizeof(*_tmp5BF));_tmp5BF->tag=15U,_tmp5BF->f1=ed->name,_tmp5BF->f2=ed;_tmp5BF;});_tmp5C0->f1=_tmp9B7;}),_tmp5C0->f2=0;_tmp5C0;});td->defn=_tmp9B8;});goto _LL28;}}else{_LL2B: _LL2C:
({void*_tmp5C1=0U;({int(*_tmp9BA)(struct _fat_ptr,struct _fat_ptr)=({int(*_tmp5C3)(struct _fat_ptr,struct _fat_ptr)=(int(*)(struct _fat_ptr,struct _fat_ptr))Cyc_Tcutil_impos;_tmp5C3;});struct _fat_ptr _tmp9B9=({const char*_tmp5C2="Toc: enumdecl to name";_tag_fat(_tmp5C2,sizeof(char),22U);});_tmp9BA(_tmp9B9,_tag_fat(_tmp5C1,sizeof(void*),0U));});});}_LL28:;}}}}else{
# 4381
enum Cyc_Absyn_KindQual _stmttmp7E=((struct Cyc_Absyn_Kind*)((struct Cyc_Core_Opt*)_check_null(td->kind))->v)->kind;enum Cyc_Absyn_KindQual _tmp5C4=_stmttmp7E;if(_tmp5C4 == Cyc_Absyn_BoxKind){_LL2E: _LL2F:
({void*_tmp9BB=Cyc_Toc_void_star_type();td->defn=_tmp9BB;});goto _LL2D;}else{_LL30: _LL31:
 td->defn=Cyc_Absyn_void_type;goto _LL2D;}_LL2D:;}
# 4389
if(Cyc_noexpand_r)
Cyc_Toc_result_decls=({struct Cyc_List_List*_tmp5C5=_cycalloc(sizeof(*_tmp5C5));_tmp5C5->hd=d,_tmp5C5->tl=Cyc_Toc_result_decls;_tmp5C5;});
# 4389
goto _LL0;}case 13U: _LL13: _LL14:
# 4392
 goto _LL16;case 14U: _LL15: _LL16:
 goto _LL18;case 15U: _LL17: _LL18:
 goto _LL1A;case 16U: _LL19: _LL1A:
 goto _LL0;case 9U: _LL1B: _tmp5A5=((struct Cyc_Absyn_Namespace_d_Absyn_Raw_decl_struct*)_tmp5A1)->f2;_LL1C: {struct Cyc_List_List*ds2=_tmp5A5;
_tmp5A4=ds2;goto _LL1E;}case 10U: _LL1D: _tmp5A4=((struct Cyc_Absyn_Using_d_Absyn_Raw_decl_struct*)_tmp5A1)->f2;_LL1E: {struct Cyc_List_List*ds2=_tmp5A4;
_tmp5A3=ds2;goto _LL20;}case 11U: _LL1F: _tmp5A3=((struct Cyc_Absyn_ExternC_d_Absyn_Raw_decl_struct*)_tmp5A1)->f1;_LL20: {struct Cyc_List_List*ds2=_tmp5A3;
nv=Cyc_Toc_decls_to_c(r,nv,ds2,cinclude);goto _LL0;}default: _LL21: _tmp5A2=((struct Cyc_Absyn_ExternCinclude_d_Absyn_Raw_decl_struct*)_tmp5A1)->f1;_LL22: {struct Cyc_List_List*ds2=_tmp5A2;
nv=Cyc_Toc_decls_to_c(r,nv,ds2,1);goto _LL0;}}_LL0:;}}
# 4402
return nv;}
# 3752 "toc.cyc"
static void Cyc_Toc_init(){
# 4407 "toc.cyc"
struct Cyc_Core_NewDynamicRegion _stmttmp7F=Cyc_Core__new_rckey("internal-error","internal-error",0);struct Cyc_Core_DynamicRegion*_tmp5C6;_LL1: _tmp5C6=_stmttmp7F.key;_LL2: {struct Cyc_Core_DynamicRegion*dyn=_tmp5C6;
struct Cyc_Toc_TocState*ts;
{struct _RegionHandle*h=& dyn->h;
ts=Cyc_Toc_empty_toc_state(h);;}
Cyc_Toc_toc_state=({struct Cyc_Toc_TocStateWrap*_tmp5C8=_region_malloc(Cyc_Core_unique_region,sizeof(*_tmp5C8));_tmp5C8->dyn=dyn,_tmp5C8->state=ts;_tmp5C8;});
Cyc_Toc_result_decls=0;
Cyc_Toc_tuple_type_counter=0;
Cyc_Toc_temp_var_counter=0;
Cyc_Toc_fresh_label_counter=0;
Cyc_Toc_globals=_tag_fat(({unsigned _tmp5CA=35;struct _fat_ptr**_tmp5C9=_cycalloc(_check_times(_tmp5CA,sizeof(struct _fat_ptr*)));_tmp5C9[0]=& Cyc_Toc__throw_str,_tmp5C9[1]=& Cyc_Toc_setjmp_str,_tmp5C9[2]=& Cyc_Toc__push_handler_str,_tmp5C9[3]=& Cyc_Toc__pop_handler_str,_tmp5C9[4]=& Cyc_Toc__exn_thrown_str,_tmp5C9[5]=& Cyc_Toc__npop_handler_str,_tmp5C9[6]=& Cyc_Toc__check_null_str,_tmp5C9[7]=& Cyc_Toc__check_known_subscript_null_str,_tmp5C9[8]=& Cyc_Toc__check_known_subscript_notnull_str,_tmp5C9[9]=& Cyc_Toc__check_fat_subscript_str,_tmp5C9[10]=& Cyc_Toc__tag_fat_str,_tmp5C9[11]=& Cyc_Toc__untag_fat_ptr_str,_tmp5C9[12]=& Cyc_Toc__get_fat_size_str,_tmp5C9[13]=& Cyc_Toc__get_zero_arr_size_str,_tmp5C9[14]=& Cyc_Toc__fat_ptr_plus_str,_tmp5C9[15]=& Cyc_Toc__zero_arr_plus_str,_tmp5C9[16]=& Cyc_Toc__fat_ptr_inplace_plus_str,_tmp5C9[17]=& Cyc_Toc__zero_arr_inplace_plus_str,_tmp5C9[18]=& Cyc_Toc__fat_ptr_inplace_plus_post_str,_tmp5C9[19]=& Cyc_Toc__zero_arr_inplace_plus_post_str,_tmp5C9[20]=& Cyc_Toc__cycalloc_str,_tmp5C9[21]=& Cyc_Toc__cyccalloc_str,_tmp5C9[22]=& Cyc_Toc__cycalloc_atomic_str,_tmp5C9[23]=& Cyc_Toc__cyccalloc_atomic_str,_tmp5C9[24]=& Cyc_Toc__region_malloc_str,_tmp5C9[25]=& Cyc_Toc__region_calloc_str,_tmp5C9[26]=& Cyc_Toc__check_times_str,_tmp5C9[27]=& Cyc_Toc__new_region_str,_tmp5C9[28]=& Cyc_Toc__push_region_str,_tmp5C9[29]=& Cyc_Toc__pop_region_str,_tmp5C9[30]=& Cyc_Toc__throw_arraybounds_str,_tmp5C9[31]=& Cyc_Toc__fat_ptr_decrease_size_str,_tmp5C9[32]=& Cyc_Toc__throw_match_str,_tmp5C9[33]=& Cyc_Toc__fast_region_malloc_str,_tmp5C9[34]=& Cyc_Toc__spawn_str;_tmp5C9;}),sizeof(struct _fat_ptr*),35U);}}
# 3752 "toc.cyc"
void Cyc_Toc_finish(){
# 4456 "toc.cyc"
struct Cyc_Toc_TocStateWrap*ts=0;
({struct Cyc_Toc_TocStateWrap*_tmp5CB=ts;struct Cyc_Toc_TocStateWrap*_tmp5CC=Cyc_Toc_toc_state;ts=_tmp5CC;Cyc_Toc_toc_state=_tmp5CB;});{
struct Cyc_Toc_TocStateWrap _stmttmp80=*((struct Cyc_Toc_TocStateWrap*)_check_null(ts));struct Cyc_Toc_TocState*_tmp5CE;struct Cyc_Core_DynamicRegion*_tmp5CD;_LL1: _tmp5CD=_stmttmp80.dyn;_tmp5CE=_stmttmp80.state;_LL2: {struct Cyc_Core_DynamicRegion*dyn=_tmp5CD;struct Cyc_Toc_TocState*s=_tmp5CE;
# 4460
{struct _RegionHandle*h=& dyn->h;
{struct Cyc_Toc_TocState _stmttmp81=*s;struct Cyc_Xarray_Xarray*_tmp5D0;_LL4: _tmp5D0=_stmttmp81.temp_labels;_LL5: {struct Cyc_Xarray_Xarray*tls=_tmp5D0;
Cyc_Xarray_reuse(tls);}}
# 4461
;}
# 4464
Cyc_Core_free_rckey(dyn);
({void(*_tmp9BC)(struct Cyc_Toc_TocStateWrap*ptr)=({void(*_tmp5D1)(struct Cyc_Toc_TocStateWrap*ptr)=(void(*)(struct Cyc_Toc_TocStateWrap*ptr))Cyc_Core_ufree;_tmp5D1;});_tmp9BC(ts);});
# 4467
Cyc_Toc_gpop_tables=0;
Cyc_Toc_fn_pop_table=0;}}}
# 3752 "toc.cyc"
struct Cyc_List_List*Cyc_Toc_toc(struct Cyc_Hashtable_Table*pop_tables,struct Cyc_List_List*ds){
# 4475 "toc.cyc"
Cyc_Toc_gpop_tables=({struct Cyc_Hashtable_Table**_tmp5D2=_cycalloc(sizeof(*_tmp5D2));*_tmp5D2=pop_tables;_tmp5D2;});
Cyc_Toc_init();{
struct _RegionHandle _tmp5D3=_new_region("start");struct _RegionHandle*start=& _tmp5D3;_push_region(start);
({struct _RegionHandle*_tmp9BE=start;struct Cyc_Toc_Env*_tmp9BD=Cyc_Toc_empty_env(start);Cyc_Toc_decls_to_c(_tmp9BE,_tmp9BD,ds,0);});{
struct Cyc_List_List*_tmp5D4=Cyc_List_imp_rev(Cyc_Toc_result_decls);_npop_handler(0U);return _tmp5D4;}
# 4478
;_pop_region(start);}}
